# -*- coding: utf-8 -*-
# Generated by the protocol buffer compiler.  DO NOT EDIT!
# source: protos/cfg.proto
"""Generated protocol buffer code."""
from google.protobuf import descriptor as _descriptor
from google.protobuf import descriptor_pool as _descriptor_pool
from google.protobuf import message as _message
from google.protobuf import reflection as _reflection
from google.protobuf import symbol_database as _symbol_database
# @@protoc_insertion_point(imports)

_sym_db = _symbol_database.Default()


from . import primitives_pb2 as protos_dot_primitives__pb2


DESCRIPTOR = _descriptor_pool.Default().AddSerializedFile(b'\n\x10protos/cfg.proto\x12\x0b\x61ngr.protos\x1a\x17protos/primitives.proto\"H\n\x07\x43\x46GNode\x12\n\n\x02\x65\x61\x18\x01 \x01(\x04\x12\x0c\n\x04size\x18\x02 \x01(\r\x12\x10\n\x08\x62lock_id\x18\x03 \x03(\x03\x12\x11\n\treturning\x18\x04 \x01(\x08\"\x89\x01\n\x03\x43\x46G\x12\r\n\x05ident\x18\x01 \x01(\t\x12#\n\x05nodes\x18\x02 \x03(\x0b\x32\x14.angr.protos.CFGNode\x12 \n\x05\x65\x64ges\x18\x03 \x03(\x0b\x32\x11.angr.protos.Edge\x12,\n\x0bmemory_data\x18\x04 \x03(\x0b\x32\x17.angr.protos.MemoryData\"\xae\x02\n\nMemoryData\x12\n\n\x02\x65\x61\x18\x01 \x01(\x04\x12\x0c\n\x04size\x18\x02 \x01(\r\x12\x34\n\x04type\x18\x03 \x01(\x0e\x32&.angr.protos.MemoryData.MemoryDataType\"\xcf\x01\n\x0eMemoryDataType\x12\x13\n\x0fUnknownDataType\x10\x00\x12\x0f\n\x0bUnspecified\x10\x01\x12\x0b\n\x07Integer\x10\x02\x12\x10\n\x0cPointerArray\x10\x03\x12\n\n\x06String\x10\x04\x12\x11\n\rUnicodeString\x10\x05\x12\x13\n\x0fSegmentBoundary\x10\x06\x12\x11\n\rCodeReference\x10\x07\x12\x0f\n\x0bGOTPLTEntry\x10\x08\x12\r\n\tELFHeader\x10\t\x12\x11\n\rFloatingPoint\x10\nb\x06proto3')



_CFGNODE = DESCRIPTOR.message_types_by_name['CFGNode']
_CFG = DESCRIPTOR.message_types_by_name['CFG']
_MEMORYDATA = DESCRIPTOR.message_types_by_name['MemoryData']
_MEMORYDATA_MEMORYDATATYPE = _MEMORYDATA.enum_types_by_name['MemoryDataType']
CFGNode = _reflection.GeneratedProtocolMessageType('CFGNode', (_message.Message,), {
  'DESCRIPTOR' : _CFGNODE,
  '__module__' : 'protos.cfg_pb2'
  # @@protoc_insertion_point(class_scope:angr.protos.CFGNode)
  })
_sym_db.RegisterMessage(CFGNode)

CFG = _reflection.GeneratedProtocolMessageType('CFG', (_message.Message,), {
  'DESCRIPTOR' : _CFG,
  '__module__' : 'protos.cfg_pb2'
  # @@protoc_insertion_point(class_scope:angr.protos.CFG)
  })
_sym_db.RegisterMessage(CFG)

MemoryData = _reflection.GeneratedProtocolMessageType('MemoryData', (_message.Message,), {
  'DESCRIPTOR' : _MEMORYDATA,
  '__module__' : 'protos.cfg_pb2'
  # @@protoc_insertion_point(class_scope:angr.protos.MemoryData)
  })
_sym_db.RegisterMessage(MemoryData)

if _descriptor._USE_C_DESCRIPTORS == False:

  DESCRIPTOR._options = None
  _CFGNODE._serialized_start=58
  _CFGNODE._serialized_end=130
  _CFG._serialized_start=133
  _CFG._serialized_end=270
  _MEMORYDATA._serialized_start=273
  _MEMORYDATA._serialized_end=575
  _MEMORYDATA_MEMORYDATATYPE._serialized_start=368
  _MEMORYDATA_MEMORYDATATYPE._serialized_end=575
# @@protoc_insertion_point(module_scope)
