# pylint:disable=line-too-long
import logging

from ...sim_type import SimTypeFunction,     SimTypeShort, SimTypeInt, SimTypeLong, SimTypeLongLong, SimTypeDouble, SimTypeFloat,     SimTypePointer,     SimTypeChar,     SimStruct,     SimTypeFixedSizeArray,     SimTypeBottom,     SimUnion,     SimTypeBool
from ...calling_conventions import SimCCStdcall, SimCCMicrosoftAMD64
from .. import SIM_PROCEDURES as P
from . import SimLibrary


_l = logging.getLogger(name=__name__)


lib = SimLibrary()
lib.set_default_cc('X86', SimCCStdcall)
lib.set_default_cc('AMD64', SimCCMicrosoftAMD64)
lib.set_library_names("fwpuclnt.dll")
prototypes = \
    {
        # 
        'WSASetSocketSecurity': SimTypeFunction([SimTypePointer(SimTypeInt(signed=False, label="UInt"), label="UIntPtr", offset=0), SimTypePointer(SimStruct({"SecurityProtocol": SimTypeInt(signed=False, label="SOCKET_SECURITY_PROTOCOL"), "SecurityFlags": SimTypeInt(signed=False, label="UInt32")}, name="SOCKET_SECURITY_SETTINGS", pack=False, align=None), offset=0), SimTypeInt(signed=False, label="UInt32"), SimTypePointer(SimStruct({"Internal": SimTypePointer(SimTypeInt(signed=False, label="UInt"), label="UIntPtr", offset=0), "InternalHigh": SimTypePointer(SimTypeInt(signed=False, label="UInt"), label="UIntPtr", offset=0), "Anonymous": SimUnion({"Anonymous": SimStruct({"Offset": SimTypeInt(signed=False, label="UInt32"), "OffsetHigh": SimTypeInt(signed=False, label="UInt32")}, name="_Anonymous_e__Struct", pack=False, align=None), "Pointer": SimTypePointer(SimTypeBottom(label="Void"), offset=0)}, name="<anon>", label="None"), "hEvent": SimTypePointer(SimTypeInt(signed=True, label="Int"), label="IntPtr", offset=0)}, name="OVERLAPPED", pack=False, align=None), offset=0), SimTypePointer(SimTypeFunction([SimTypeInt(signed=False, label="UInt32"), SimTypeInt(signed=False, label="UInt32"), SimTypePointer(SimTypeBottom(label="OVERLAPPED"), offset=0), SimTypeInt(signed=False, label="UInt32")], SimTypeBottom(label="Void"), arg_names=["dwError", "cbTransferred", "lpOverlapped", "dwFlags"]), offset=0)], SimTypeInt(signed=True, label="Int32"), arg_names=["Socket", "SecuritySettings", "SecuritySettingsLen", "Overlapped", "CompletionRoutine"]),
        # 
        'WSAQuerySocketSecurity': SimTypeFunction([SimTypePointer(SimTypeInt(signed=False, label="UInt"), label="UIntPtr", offset=0), SimTypePointer(SimStruct({"SecurityProtocol": SimTypeInt(signed=False, label="SOCKET_SECURITY_PROTOCOL"), "PeerAddress": SimStruct({"ss_family": SimTypeShort(signed=False, label="UInt16"), "__ss_pad1": SimTypeFixedSizeArray(SimTypeBottom(label="CHAR"), 6), "__ss_align": SimTypeLongLong(signed=True, label="Int64"), "__ss_pad2": SimTypeFixedSizeArray(SimTypeBottom(label="CHAR"), 112)}, name="SOCKADDR_STORAGE", pack=False, align=None), "PeerTokenAccessMask": SimTypeInt(signed=False, label="UInt32")}, name="SOCKET_SECURITY_QUERY_TEMPLATE", pack=False, align=None), offset=0), SimTypeInt(signed=False, label="UInt32"), SimTypePointer(SimStruct({"SecurityProtocol": SimTypeInt(signed=False, label="SOCKET_SECURITY_PROTOCOL"), "Flags": SimTypeInt(signed=False, label="UInt32"), "PeerApplicationAccessTokenHandle": SimTypeLongLong(signed=False, label="UInt64"), "PeerMachineAccessTokenHandle": SimTypeLongLong(signed=False, label="UInt64")}, name="SOCKET_SECURITY_QUERY_INFO", pack=False, align=None), offset=0), SimTypePointer(SimTypeInt(signed=False, label="UInt32"), offset=0), SimTypePointer(SimStruct({"Internal": SimTypePointer(SimTypeInt(signed=False, label="UInt"), label="UIntPtr", offset=0), "InternalHigh": SimTypePointer(SimTypeInt(signed=False, label="UInt"), label="UIntPtr", offset=0), "Anonymous": SimUnion({"Anonymous": SimStruct({"Offset": SimTypeInt(signed=False, label="UInt32"), "OffsetHigh": SimTypeInt(signed=False, label="UInt32")}, name="_Anonymous_e__Struct", pack=False, align=None), "Pointer": SimTypePointer(SimTypeBottom(label="Void"), offset=0)}, name="<anon>", label="None"), "hEvent": SimTypePointer(SimTypeInt(signed=True, label="Int"), label="IntPtr", offset=0)}, name="OVERLAPPED", pack=False, align=None), offset=0), SimTypePointer(SimTypeFunction([SimTypeInt(signed=False, label="UInt32"), SimTypeInt(signed=False, label="UInt32"), SimTypePointer(SimTypeBottom(label="OVERLAPPED"), offset=0), SimTypeInt(signed=False, label="UInt32")], SimTypeBottom(label="Void"), arg_names=["dwError", "cbTransferred", "lpOverlapped", "dwFlags"]), offset=0)], SimTypeInt(signed=True, label="Int32"), arg_names=["Socket", "SecurityQueryTemplate", "SecurityQueryTemplateLen", "SecurityQueryInfo", "SecurityQueryInfoLen", "Overlapped", "CompletionRoutine"]),
        # 
        'WSASetSocketPeerTargetName': SimTypeFunction([SimTypePointer(SimTypeInt(signed=False, label="UInt"), label="UIntPtr", offset=0), SimTypePointer(SimStruct({"SecurityProtocol": SimTypeInt(signed=False, label="SOCKET_SECURITY_PROTOCOL"), "PeerAddress": SimStruct({"ss_family": SimTypeShort(signed=False, label="UInt16"), "__ss_pad1": SimTypeFixedSizeArray(SimTypeBottom(label="CHAR"), 6), "__ss_align": SimTypeLongLong(signed=True, label="Int64"), "__ss_pad2": SimTypeFixedSizeArray(SimTypeBottom(label="CHAR"), 112)}, name="SOCKADDR_STORAGE", pack=False, align=None), "PeerTargetNameStringLen": SimTypeInt(signed=False, label="UInt32"), "AllStrings": SimTypePointer(SimTypeChar(label="Char"), offset=0)}, name="SOCKET_PEER_TARGET_NAME", pack=False, align=None), offset=0), SimTypeInt(signed=False, label="UInt32"), SimTypePointer(SimStruct({"Internal": SimTypePointer(SimTypeInt(signed=False, label="UInt"), label="UIntPtr", offset=0), "InternalHigh": SimTypePointer(SimTypeInt(signed=False, label="UInt"), label="UIntPtr", offset=0), "Anonymous": SimUnion({"Anonymous": SimStruct({"Offset": SimTypeInt(signed=False, label="UInt32"), "OffsetHigh": SimTypeInt(signed=False, label="UInt32")}, name="_Anonymous_e__Struct", pack=False, align=None), "Pointer": SimTypePointer(SimTypeBottom(label="Void"), offset=0)}, name="<anon>", label="None"), "hEvent": SimTypePointer(SimTypeInt(signed=True, label="Int"), label="IntPtr", offset=0)}, name="OVERLAPPED", pack=False, align=None), offset=0), SimTypePointer(SimTypeFunction([SimTypeInt(signed=False, label="UInt32"), SimTypeInt(signed=False, label="UInt32"), SimTypePointer(SimTypeBottom(label="OVERLAPPED"), offset=0), SimTypeInt(signed=False, label="UInt32")], SimTypeBottom(label="Void"), arg_names=["dwError", "cbTransferred", "lpOverlapped", "dwFlags"]), offset=0)], SimTypeInt(signed=True, label="Int32"), arg_names=["Socket", "PeerTargetName", "PeerTargetNameLen", "Overlapped", "CompletionRoutine"]),
        # 
        'WSADeleteSocketPeerTargetName': SimTypeFunction([SimTypePointer(SimTypeInt(signed=False, label="UInt"), label="UIntPtr", offset=0), SimTypePointer(SimStruct({"sa_family": SimTypeShort(signed=False, label="UInt16"), "sa_data": SimTypeFixedSizeArray(SimTypeBottom(label="CHAR"), 14)}, name="SOCKADDR", pack=False, align=None), offset=0), SimTypeInt(signed=False, label="UInt32"), SimTypePointer(SimStruct({"Internal": SimTypePointer(SimTypeInt(signed=False, label="UInt"), label="UIntPtr", offset=0), "InternalHigh": SimTypePointer(SimTypeInt(signed=False, label="UInt"), label="UIntPtr", offset=0), "Anonymous": SimUnion({"Anonymous": SimStruct({"Offset": SimTypeInt(signed=False, label="UInt32"), "OffsetHigh": SimTypeInt(signed=False, label="UInt32")}, name="_Anonymous_e__Struct", pack=False, align=None), "Pointer": SimTypePointer(SimTypeBottom(label="Void"), offset=0)}, name="<anon>", label="None"), "hEvent": SimTypePointer(SimTypeInt(signed=True, label="Int"), label="IntPtr", offset=0)}, name="OVERLAPPED", pack=False, align=None), offset=0), SimTypePointer(SimTypeFunction([SimTypeInt(signed=False, label="UInt32"), SimTypeInt(signed=False, label="UInt32"), SimTypePointer(SimTypeBottom(label="OVERLAPPED"), offset=0), SimTypeInt(signed=False, label="UInt32")], SimTypeBottom(label="Void"), arg_names=["dwError", "cbTransferred", "lpOverlapped", "dwFlags"]), offset=0)], SimTypeInt(signed=True, label="Int32"), arg_names=["Socket", "PeerAddr", "PeerAddrLen", "Overlapped", "CompletionRoutine"]),
        # 
        'WSAImpersonateSocketPeer': SimTypeFunction([SimTypePointer(SimTypeInt(signed=False, label="UInt"), label="UIntPtr", offset=0), SimTypePointer(SimStruct({"sa_family": SimTypeShort(signed=False, label="UInt16"), "sa_data": SimTypeFixedSizeArray(SimTypeBottom(label="CHAR"), 14)}, name="SOCKADDR", pack=False, align=None), offset=0), SimTypeInt(signed=False, label="UInt32")], SimTypeInt(signed=True, label="Int32"), arg_names=["Socket", "PeerAddr", "PeerAddrLen"]),
        # 
        'WSARevertImpersonation': SimTypeFunction([], SimTypeInt(signed=True, label="Int32")),
        # 
        'FwpmFreeMemory0': SimTypeFunction([SimTypePointer(SimTypePointer(SimTypeBottom(label="Void"), offset=0), offset=0)], SimTypeBottom(label="Void"), arg_names=["p"]),
        # 
        'FwpmEngineOpen0': SimTypeFunction([SimTypePointer(SimTypeChar(label="Char"), offset=0), SimTypeInt(signed=False, label="UInt32"), SimTypePointer(SimStruct({"User": SimTypePointer(SimTypeShort(signed=False, label="UInt16"), offset=0), "UserLength": SimTypeInt(signed=False, label="UInt32"), "Domain": SimTypePointer(SimTypeShort(signed=False, label="UInt16"), offset=0), "DomainLength": SimTypeInt(signed=False, label="UInt32"), "Password": SimTypePointer(SimTypeShort(signed=False, label="UInt16"), offset=0), "PasswordLength": SimTypeInt(signed=False, label="UInt32"), "Flags": SimTypeInt(signed=False, label="SEC_WINNT_AUTH_IDENTITY")}, name="SEC_WINNT_AUTH_IDENTITY_W", pack=False, align=None), offset=0), SimTypePointer(SimStruct({"sessionKey": SimTypeBottom(label="Guid"), "displayData": SimStruct({"name": SimTypePointer(SimTypeChar(label="Char"), offset=0), "description": SimTypePointer(SimTypeChar(label="Char"), offset=0)}, name="FWPM_DISPLAY_DATA0", pack=False, align=None), "flags": SimTypeInt(signed=False, label="UInt32"), "txnWaitTimeoutInMSec": SimTypeInt(signed=False, label="UInt32"), "processId": SimTypeInt(signed=False, label="UInt32"), "sid": SimTypePointer(SimTypeBottom(label="SID"), offset=0), "username": SimTypePointer(SimTypeChar(label="Char"), offset=0), "kernelMode": SimTypeInt(signed=True, label="Int32")}, name="FWPM_SESSION0", pack=False, align=None), offset=0), SimTypePointer(SimTypePointer(SimTypeInt(signed=True, label="Int"), label="IntPtr", offset=0), offset=0)], SimTypeInt(signed=False, label="UInt32"), arg_names=["serverName", "authnService", "authIdentity", "session", "engineHandle"]),
        # 
        'FwpmEngineClose0': SimTypeFunction([SimTypePointer(SimTypeInt(signed=True, label="Int"), label="IntPtr", offset=0)], SimTypeInt(signed=False, label="UInt32"), arg_names=["engineHandle"]),
        # 
        'FwpmEngineGetOption0': SimTypeFunction([SimTypePointer(SimTypeInt(signed=True, label="Int"), label="IntPtr", offset=0), SimTypeInt(signed=False, label="FWPM_ENGINE_OPTION"), SimTypePointer(SimTypePointer(SimStruct({"type": SimTypeInt(signed=False, label="FWP_DATA_TYPE"), "Anonymous": SimUnion({"uint8": SimTypeChar(label="Byte"), "uint16": SimTypeShort(signed=False, label="UInt16"), "uint32": SimTypeInt(signed=False, label="UInt32"), "uint64": SimTypePointer(SimTypeLongLong(signed=False, label="UInt64"), offset=0), "int8": SimTypeChar(label="SByte"), "int16": SimTypeShort(signed=True, label="Int16"), "int32": SimTypeInt(signed=True, label="Int32"), "int64": SimTypePointer(SimTypeLongLong(signed=True, label="Int64"), offset=0), "float32": SimTypeFloat(size=32), "double64": SimTypePointer(SimTypeFloat(size=64), offset=0), "byteArray16": SimTypePointer(SimStruct({"byteArray16": SimTypeFixedSizeArray(SimTypeChar(label="Byte"), 16)}, name="FWP_BYTE_ARRAY16", pack=False, align=None), offset=0), "byteBlob": SimTypePointer(SimStruct({"size": SimTypeInt(signed=False, label="UInt32"), "data": SimTypePointer(SimTypeChar(label="Byte"), offset=0)}, name="FWP_BYTE_BLOB", pack=False, align=None), offset=0), "sid": SimTypePointer(SimTypeBottom(label="SID"), offset=0), "sd": SimTypePointer(SimStruct({"size": SimTypeInt(signed=False, label="UInt32"), "data": SimTypePointer(SimTypeChar(label="Byte"), offset=0)}, name="FWP_BYTE_BLOB", pack=False, align=None), offset=0), "tokenInformation": SimTypePointer(SimStruct({"sidCount": SimTypeInt(signed=False, label="UInt32"), "sids": SimTypePointer(SimTypeBottom(label="SID_AND_ATTRIBUTES"), offset=0), "restrictedSidCount": SimTypeInt(signed=False, label="UInt32"), "restrictedSids": SimTypePointer(SimTypeBottom(label="SID_AND_ATTRIBUTES"), offset=0)}, name="FWP_TOKEN_INFORMATION", pack=False, align=None), offset=0), "tokenAccessInformation": SimTypePointer(SimStruct({"size": SimTypeInt(signed=False, label="UInt32"), "data": SimTypePointer(SimTypeChar(label="Byte"), offset=0)}, name="FWP_BYTE_BLOB", pack=False, align=None), offset=0), "unicodeString": SimTypePointer(SimTypeChar(label="Char"), offset=0), "byteArray6": SimTypePointer(SimStruct({"byteArray6": SimTypeFixedSizeArray(SimTypeChar(label="Byte"), 6)}, name="FWP_BYTE_ARRAY6", pack=False, align=None), offset=0), "bitmapArray64": SimTypePointer(SimStruct({"bitmapArray64": SimTypeFixedSizeArray(SimTypeChar(label="Byte"), 8)}, name="FWP_BITMAP_ARRAY64_", pack=False, align=None), offset=0)}, name="<anon>", label="None")}, name="FWP_VALUE0", pack=False, align=None), offset=0), offset=0)], SimTypeInt(signed=False, label="UInt32"), arg_names=["engineHandle", "option", "value"]),
        # 
        'FwpmEngineSetOption0': SimTypeFunction([SimTypePointer(SimTypeInt(signed=True, label="Int"), label="IntPtr", offset=0), SimTypeInt(signed=False, label="FWPM_ENGINE_OPTION"), SimTypePointer(SimStruct({"type": SimTypeInt(signed=False, label="FWP_DATA_TYPE"), "Anonymous": SimUnion({"uint8": SimTypeChar(label="Byte"), "uint16": SimTypeShort(signed=False, label="UInt16"), "uint32": SimTypeInt(signed=False, label="UInt32"), "uint64": SimTypePointer(SimTypeLongLong(signed=False, label="UInt64"), offset=0), "int8": SimTypeChar(label="SByte"), "int16": SimTypeShort(signed=True, label="Int16"), "int32": SimTypeInt(signed=True, label="Int32"), "int64": SimTypePointer(SimTypeLongLong(signed=True, label="Int64"), offset=0), "float32": SimTypeFloat(size=32), "double64": SimTypePointer(SimTypeFloat(size=64), offset=0), "byteArray16": SimTypePointer(SimStruct({"byteArray16": SimTypeFixedSizeArray(SimTypeChar(label="Byte"), 16)}, name="FWP_BYTE_ARRAY16", pack=False, align=None), offset=0), "byteBlob": SimTypePointer(SimStruct({"size": SimTypeInt(signed=False, label="UInt32"), "data": SimTypePointer(SimTypeChar(label="Byte"), offset=0)}, name="FWP_BYTE_BLOB", pack=False, align=None), offset=0), "sid": SimTypePointer(SimTypeBottom(label="SID"), offset=0), "sd": SimTypePointer(SimStruct({"size": SimTypeInt(signed=False, label="UInt32"), "data": SimTypePointer(SimTypeChar(label="Byte"), offset=0)}, name="FWP_BYTE_BLOB", pack=False, align=None), offset=0), "tokenInformation": SimTypePointer(SimStruct({"sidCount": SimTypeInt(signed=False, label="UInt32"), "sids": SimTypePointer(SimTypeBottom(label="SID_AND_ATTRIBUTES"), offset=0), "restrictedSidCount": SimTypeInt(signed=False, label="UInt32"), "restrictedSids": SimTypePointer(SimTypeBottom(label="SID_AND_ATTRIBUTES"), offset=0)}, name="FWP_TOKEN_INFORMATION", pack=False, align=None), offset=0), "tokenAccessInformation": SimTypePointer(SimStruct({"size": SimTypeInt(signed=False, label="UInt32"), "data": SimTypePointer(SimTypeChar(label="Byte"), offset=0)}, name="FWP_BYTE_BLOB", pack=False, align=None), offset=0), "unicodeString": SimTypePointer(SimTypeChar(label="Char"), offset=0), "byteArray6": SimTypePointer(SimStruct({"byteArray6": SimTypeFixedSizeArray(SimTypeChar(label="Byte"), 6)}, name="FWP_BYTE_ARRAY6", pack=False, align=None), offset=0), "bitmapArray64": SimTypePointer(SimStruct({"bitmapArray64": SimTypeFixedSizeArray(SimTypeChar(label="Byte"), 8)}, name="FWP_BITMAP_ARRAY64_", pack=False, align=None), offset=0)}, name="<anon>", label="None")}, name="FWP_VALUE0", pack=False, align=None), offset=0)], SimTypeInt(signed=False, label="UInt32"), arg_names=["engineHandle", "option", "newValue"]),
        # 
        'FwpmEngineGetSecurityInfo0': SimTypeFunction([SimTypePointer(SimTypeInt(signed=True, label="Int"), label="IntPtr", offset=0), SimTypeInt(signed=False, label="UInt32"), SimTypePointer(SimTypePointer(SimTypeInt(signed=True, label="Int"), label="IntPtr", offset=0), offset=0), SimTypePointer(SimTypePointer(SimTypeInt(signed=True, label="Int"), label="IntPtr", offset=0), offset=0), SimTypePointer(SimTypePointer(SimStruct({"AclRevision": SimTypeChar(label="Byte"), "Sbz1": SimTypeChar(label="Byte"), "AclSize": SimTypeShort(signed=False, label="UInt16"), "AceCount": SimTypeShort(signed=False, label="UInt16"), "Sbz2": SimTypeShort(signed=False, label="UInt16")}, name="ACL", pack=False, align=None), offset=0), offset=0), SimTypePointer(SimTypePointer(SimStruct({"AclRevision": SimTypeChar(label="Byte"), "Sbz1": SimTypeChar(label="Byte"), "AclSize": SimTypeShort(signed=False, label="UInt16"), "AceCount": SimTypeShort(signed=False, label="UInt16"), "Sbz2": SimTypeShort(signed=False, label="UInt16")}, name="ACL", pack=False, align=None), offset=0), offset=0), SimTypePointer(SimTypePointer(SimStruct({"Revision": SimTypeChar(label="Byte"), "Sbz1": SimTypeChar(label="Byte"), "Control": SimTypeShort(signed=False, label="UInt16"), "Owner": SimTypePointer(SimTypeInt(signed=True, label="Int"), label="IntPtr", offset=0), "Group": SimTypePointer(SimTypeInt(signed=True, label="Int"), label="IntPtr", offset=0), "Sacl": SimTypePointer(SimStruct({"AclRevision": SimTypeChar(label="Byte"), "Sbz1": SimTypeChar(label="Byte"), "AclSize": SimTypeShort(signed=False, label="UInt16"), "AceCount": SimTypeShort(signed=False, label="UInt16"), "Sbz2": SimTypeShort(signed=False, label="UInt16")}, name="ACL", pack=False, align=None), offset=0), "Dacl": SimTypePointer(SimStruct({"AclRevision": SimTypeChar(label="Byte"), "Sbz1": SimTypeChar(label="Byte"), "AclSize": SimTypeShort(signed=False, label="UInt16"), "AceCount": SimTypeShort(signed=False, label="UInt16"), "Sbz2": SimTypeShort(signed=False, label="UInt16")}, name="ACL", pack=False, align=None), offset=0)}, name="SECURITY_DESCRIPTOR", pack=False, align=None), offset=0), offset=0)], SimTypeInt(signed=False, label="UInt32"), arg_names=["engineHandle", "securityInfo", "sidOwner", "sidGroup", "dacl", "sacl", "securityDescriptor"]),
        # 
        'FwpmEngineSetSecurityInfo0': SimTypeFunction([SimTypePointer(SimTypeInt(signed=True, label="Int"), label="IntPtr", offset=0), SimTypeInt(signed=False, label="UInt32"), SimTypePointer(SimStruct({"Revision": SimTypeChar(label="Byte"), "SubAuthorityCount": SimTypeChar(label="Byte"), "IdentifierAuthority": SimStruct({"Value": SimTypeFixedSizeArray(SimTypeChar(label="Byte"), 6)}, name="SID_IDENTIFIER_AUTHORITY", pack=False, align=None), "SubAuthority": SimTypePointer(SimTypeInt(signed=False, label="UInt32"), offset=0)}, name="SID", pack=False, align=None), offset=0), SimTypePointer(SimStruct({"Revision": SimTypeChar(label="Byte"), "SubAuthorityCount": SimTypeChar(label="Byte"), "IdentifierAuthority": SimStruct({"Value": SimTypeFixedSizeArray(SimTypeChar(label="Byte"), 6)}, name="SID_IDENTIFIER_AUTHORITY", pack=False, align=None), "SubAuthority": SimTypePointer(SimTypeInt(signed=False, label="UInt32"), offset=0)}, name="SID", pack=False, align=None), offset=0), SimTypePointer(SimStruct({"AclRevision": SimTypeChar(label="Byte"), "Sbz1": SimTypeChar(label="Byte"), "AclSize": SimTypeShort(signed=False, label="UInt16"), "AceCount": SimTypeShort(signed=False, label="UInt16"), "Sbz2": SimTypeShort(signed=False, label="UInt16")}, name="ACL", pack=False, align=None), offset=0), SimTypePointer(SimStruct({"AclRevision": SimTypeChar(label="Byte"), "Sbz1": SimTypeChar(label="Byte"), "AclSize": SimTypeShort(signed=False, label="UInt16"), "AceCount": SimTypeShort(signed=False, label="UInt16"), "Sbz2": SimTypeShort(signed=False, label="UInt16")}, name="ACL", pack=False, align=None), offset=0)], SimTypeInt(signed=False, label="UInt32"), arg_names=["engineHandle", "securityInfo", "sidOwner", "sidGroup", "dacl", "sacl"]),
        # 
        'FwpmSessionCreateEnumHandle0': SimTypeFunction([SimTypePointer(SimTypeInt(signed=True, label="Int"), label="IntPtr", offset=0), SimTypePointer(SimStruct({"reserved": SimTypeLongLong(signed=False, label="UInt64")}, name="FWPM_SESSION_ENUM_TEMPLATE0", pack=False, align=None), offset=0), SimTypePointer(SimTypePointer(SimTypeInt(signed=True, label="Int"), label="IntPtr", offset=0), offset=0)], SimTypeInt(signed=False, label="UInt32"), arg_names=["engineHandle", "enumTemplate", "enumHandle"]),
        # 
        'FwpmSessionEnum0': SimTypeFunction([SimTypePointer(SimTypeInt(signed=True, label="Int"), label="IntPtr", offset=0), SimTypePointer(SimTypeInt(signed=True, label="Int"), label="IntPtr", offset=0), SimTypeInt(signed=False, label="UInt32"), SimTypePointer(SimTypePointer(SimTypePointer(SimStruct({"sessionKey": SimTypeBottom(label="Guid"), "displayData": SimStruct({"name": SimTypePointer(SimTypeChar(label="Char"), offset=0), "description": SimTypePointer(SimTypeChar(label="Char"), offset=0)}, name="FWPM_DISPLAY_DATA0", pack=False, align=None), "flags": SimTypeInt(signed=False, label="UInt32"), "txnWaitTimeoutInMSec": SimTypeInt(signed=False, label="UInt32"), "processId": SimTypeInt(signed=False, label="UInt32"), "sid": SimTypePointer(SimTypeBottom(label="SID"), offset=0), "username": SimTypePointer(SimTypeChar(label="Char"), offset=0), "kernelMode": SimTypeInt(signed=True, label="Int32")}, name="FWPM_SESSION0", pack=False, align=None), offset=0), offset=0), offset=0), SimTypePointer(SimTypeInt(signed=False, label="UInt32"), offset=0)], SimTypeInt(signed=False, label="UInt32"), arg_names=["engineHandle", "enumHandle", "numEntriesRequested", "entries", "numEntriesReturned"]),
        # 
        'FwpmSessionDestroyEnumHandle0': SimTypeFunction([SimTypePointer(SimTypeInt(signed=True, label="Int"), label="IntPtr", offset=0), SimTypePointer(SimTypeInt(signed=True, label="Int"), label="IntPtr", offset=0)], SimTypeInt(signed=False, label="UInt32"), arg_names=["engineHandle", "enumHandle"]),
        # 
        'FwpmTransactionBegin0': SimTypeFunction([SimTypePointer(SimTypeInt(signed=True, label="Int"), label="IntPtr", offset=0), SimTypeInt(signed=False, label="UInt32")], SimTypeInt(signed=False, label="UInt32"), arg_names=["engineHandle", "flags"]),
        # 
        'FwpmTransactionCommit0': SimTypeFunction([SimTypePointer(SimTypeInt(signed=True, label="Int"), label="IntPtr", offset=0)], SimTypeInt(signed=False, label="UInt32"), arg_names=["engineHandle"]),
        # 
        'FwpmTransactionAbort0': SimTypeFunction([SimTypePointer(SimTypeInt(signed=True, label="Int"), label="IntPtr", offset=0)], SimTypeInt(signed=False, label="UInt32"), arg_names=["engineHandle"]),
        # 
        'FwpmProviderAdd0': SimTypeFunction([SimTypePointer(SimTypeInt(signed=True, label="Int"), label="IntPtr", offset=0), SimTypePointer(SimStruct({"providerKey": SimTypeBottom(label="Guid"), "displayData": SimStruct({"name": SimTypePointer(SimTypeChar(label="Char"), offset=0), "description": SimTypePointer(SimTypeChar(label="Char"), offset=0)}, name="FWPM_DISPLAY_DATA0", pack=False, align=None), "flags": SimTypeInt(signed=False, label="UInt32"), "providerData": SimStruct({"size": SimTypeInt(signed=False, label="UInt32"), "data": SimTypePointer(SimTypeChar(label="Byte"), offset=0)}, name="FWP_BYTE_BLOB", pack=False, align=None), "serviceName": SimTypePointer(SimTypeChar(label="Char"), offset=0)}, name="FWPM_PROVIDER0", pack=False, align=None), offset=0), SimTypePointer(SimStruct({"Revision": SimTypeChar(label="Byte"), "Sbz1": SimTypeChar(label="Byte"), "Control": SimTypeShort(signed=False, label="UInt16"), "Owner": SimTypePointer(SimTypeInt(signed=True, label="Int"), label="IntPtr", offset=0), "Group": SimTypePointer(SimTypeInt(signed=True, label="Int"), label="IntPtr", offset=0), "Sacl": SimTypePointer(SimStruct({"AclRevision": SimTypeChar(label="Byte"), "Sbz1": SimTypeChar(label="Byte"), "AclSize": SimTypeShort(signed=False, label="UInt16"), "AceCount": SimTypeShort(signed=False, label="UInt16"), "Sbz2": SimTypeShort(signed=False, label="UInt16")}, name="ACL", pack=False, align=None), offset=0), "Dacl": SimTypePointer(SimStruct({"AclRevision": SimTypeChar(label="Byte"), "Sbz1": SimTypeChar(label="Byte"), "AclSize": SimTypeShort(signed=False, label="UInt16"), "AceCount": SimTypeShort(signed=False, label="UInt16"), "Sbz2": SimTypeShort(signed=False, label="UInt16")}, name="ACL", pack=False, align=None), offset=0)}, name="SECURITY_DESCRIPTOR", pack=False, align=None), offset=0)], SimTypeInt(signed=False, label="UInt32"), arg_names=["engineHandle", "provider", "sd"]),
        # 
        'FwpmProviderDeleteByKey0': SimTypeFunction([SimTypePointer(SimTypeInt(signed=True, label="Int"), label="IntPtr", offset=0), SimTypePointer(SimTypeBottom(label="Guid"), offset=0)], SimTypeInt(signed=False, label="UInt32"), arg_names=["engineHandle", "key"]),
        # 
        'FwpmProviderGetByKey0': SimTypeFunction([SimTypePointer(SimTypeInt(signed=True, label="Int"), label="IntPtr", offset=0), SimTypePointer(SimTypeBottom(label="Guid"), offset=0), SimTypePointer(SimTypePointer(SimStruct({"providerKey": SimTypeBottom(label="Guid"), "displayData": SimStruct({"name": SimTypePointer(SimTypeChar(label="Char"), offset=0), "description": SimTypePointer(SimTypeChar(label="Char"), offset=0)}, name="FWPM_DISPLAY_DATA0", pack=False, align=None), "flags": SimTypeInt(signed=False, label="UInt32"), "providerData": SimStruct({"size": SimTypeInt(signed=False, label="UInt32"), "data": SimTypePointer(SimTypeChar(label="Byte"), offset=0)}, name="FWP_BYTE_BLOB", pack=False, align=None), "serviceName": SimTypePointer(SimTypeChar(label="Char"), offset=0)}, name="FWPM_PROVIDER0", pack=False, align=None), offset=0), offset=0)], SimTypeInt(signed=False, label="UInt32"), arg_names=["engineHandle", "key", "provider"]),
        # 
        'FwpmProviderCreateEnumHandle0': SimTypeFunction([SimTypePointer(SimTypeInt(signed=True, label="Int"), label="IntPtr", offset=0), SimTypePointer(SimStruct({"reserved": SimTypeLongLong(signed=False, label="UInt64")}, name="FWPM_PROVIDER_ENUM_TEMPLATE0", pack=False, align=None), offset=0), SimTypePointer(SimTypePointer(SimTypeInt(signed=True, label="Int"), label="IntPtr", offset=0), offset=0)], SimTypeInt(signed=False, label="UInt32"), arg_names=["engineHandle", "enumTemplate", "enumHandle"]),
        # 
        'FwpmProviderEnum0': SimTypeFunction([SimTypePointer(SimTypeInt(signed=True, label="Int"), label="IntPtr", offset=0), SimTypePointer(SimTypeInt(signed=True, label="Int"), label="IntPtr", offset=0), SimTypeInt(signed=False, label="UInt32"), SimTypePointer(SimTypePointer(SimTypePointer(SimStruct({"providerKey": SimTypeBottom(label="Guid"), "displayData": SimStruct({"name": SimTypePointer(SimTypeChar(label="Char"), offset=0), "description": SimTypePointer(SimTypeChar(label="Char"), offset=0)}, name="FWPM_DISPLAY_DATA0", pack=False, align=None), "flags": SimTypeInt(signed=False, label="UInt32"), "providerData": SimStruct({"size": SimTypeInt(signed=False, label="UInt32"), "data": SimTypePointer(SimTypeChar(label="Byte"), offset=0)}, name="FWP_BYTE_BLOB", pack=False, align=None), "serviceName": SimTypePointer(SimTypeChar(label="Char"), offset=0)}, name="FWPM_PROVIDER0", pack=False, align=None), offset=0), offset=0), offset=0), SimTypePointer(SimTypeInt(signed=False, label="UInt32"), offset=0)], SimTypeInt(signed=False, label="UInt32"), arg_names=["engineHandle", "enumHandle", "numEntriesRequested", "entries", "numEntriesReturned"]),
        # 
        'FwpmProviderDestroyEnumHandle0': SimTypeFunction([SimTypePointer(SimTypeInt(signed=True, label="Int"), label="IntPtr", offset=0), SimTypePointer(SimTypeInt(signed=True, label="Int"), label="IntPtr", offset=0)], SimTypeInt(signed=False, label="UInt32"), arg_names=["engineHandle", "enumHandle"]),
        # 
        'FwpmProviderGetSecurityInfoByKey0': SimTypeFunction([SimTypePointer(SimTypeInt(signed=True, label="Int"), label="IntPtr", offset=0), SimTypePointer(SimTypeBottom(label="Guid"), offset=0), SimTypeInt(signed=False, label="UInt32"), SimTypePointer(SimTypePointer(SimTypeInt(signed=True, label="Int"), label="IntPtr", offset=0), offset=0), SimTypePointer(SimTypePointer(SimTypeInt(signed=True, label="Int"), label="IntPtr", offset=0), offset=0), SimTypePointer(SimTypePointer(SimStruct({"AclRevision": SimTypeChar(label="Byte"), "Sbz1": SimTypeChar(label="Byte"), "AclSize": SimTypeShort(signed=False, label="UInt16"), "AceCount": SimTypeShort(signed=False, label="UInt16"), "Sbz2": SimTypeShort(signed=False, label="UInt16")}, name="ACL", pack=False, align=None), offset=0), offset=0), SimTypePointer(SimTypePointer(SimStruct({"AclRevision": SimTypeChar(label="Byte"), "Sbz1": SimTypeChar(label="Byte"), "AclSize": SimTypeShort(signed=False, label="UInt16"), "AceCount": SimTypeShort(signed=False, label="UInt16"), "Sbz2": SimTypeShort(signed=False, label="UInt16")}, name="ACL", pack=False, align=None), offset=0), offset=0), SimTypePointer(SimTypePointer(SimStruct({"Revision": SimTypeChar(label="Byte"), "Sbz1": SimTypeChar(label="Byte"), "Control": SimTypeShort(signed=False, label="UInt16"), "Owner": SimTypePointer(SimTypeInt(signed=True, label="Int"), label="IntPtr", offset=0), "Group": SimTypePointer(SimTypeInt(signed=True, label="Int"), label="IntPtr", offset=0), "Sacl": SimTypePointer(SimStruct({"AclRevision": SimTypeChar(label="Byte"), "Sbz1": SimTypeChar(label="Byte"), "AclSize": SimTypeShort(signed=False, label="UInt16"), "AceCount": SimTypeShort(signed=False, label="UInt16"), "Sbz2": SimTypeShort(signed=False, label="UInt16")}, name="ACL", pack=False, align=None), offset=0), "Dacl": SimTypePointer(SimStruct({"AclRevision": SimTypeChar(label="Byte"), "Sbz1": SimTypeChar(label="Byte"), "AclSize": SimTypeShort(signed=False, label="UInt16"), "AceCount": SimTypeShort(signed=False, label="UInt16"), "Sbz2": SimTypeShort(signed=False, label="UInt16")}, name="ACL", pack=False, align=None), offset=0)}, name="SECURITY_DESCRIPTOR", pack=False, align=None), offset=0), offset=0)], SimTypeInt(signed=False, label="UInt32"), arg_names=["engineHandle", "key", "securityInfo", "sidOwner", "sidGroup", "dacl", "sacl", "securityDescriptor"]),
        # 
        'FwpmProviderSetSecurityInfoByKey0': SimTypeFunction([SimTypePointer(SimTypeInt(signed=True, label="Int"), label="IntPtr", offset=0), SimTypePointer(SimTypeBottom(label="Guid"), offset=0), SimTypeInt(signed=False, label="UInt32"), SimTypePointer(SimStruct({"Revision": SimTypeChar(label="Byte"), "SubAuthorityCount": SimTypeChar(label="Byte"), "IdentifierAuthority": SimStruct({"Value": SimTypeFixedSizeArray(SimTypeChar(label="Byte"), 6)}, name="SID_IDENTIFIER_AUTHORITY", pack=False, align=None), "SubAuthority": SimTypePointer(SimTypeInt(signed=False, label="UInt32"), offset=0)}, name="SID", pack=False, align=None), offset=0), SimTypePointer(SimStruct({"Revision": SimTypeChar(label="Byte"), "SubAuthorityCount": SimTypeChar(label="Byte"), "IdentifierAuthority": SimStruct({"Value": SimTypeFixedSizeArray(SimTypeChar(label="Byte"), 6)}, name="SID_IDENTIFIER_AUTHORITY", pack=False, align=None), "SubAuthority": SimTypePointer(SimTypeInt(signed=False, label="UInt32"), offset=0)}, name="SID", pack=False, align=None), offset=0), SimTypePointer(SimStruct({"AclRevision": SimTypeChar(label="Byte"), "Sbz1": SimTypeChar(label="Byte"), "AclSize": SimTypeShort(signed=False, label="UInt16"), "AceCount": SimTypeShort(signed=False, label="UInt16"), "Sbz2": SimTypeShort(signed=False, label="UInt16")}, name="ACL", pack=False, align=None), offset=0), SimTypePointer(SimStruct({"AclRevision": SimTypeChar(label="Byte"), "Sbz1": SimTypeChar(label="Byte"), "AclSize": SimTypeShort(signed=False, label="UInt16"), "AceCount": SimTypeShort(signed=False, label="UInt16"), "Sbz2": SimTypeShort(signed=False, label="UInt16")}, name="ACL", pack=False, align=None), offset=0)], SimTypeInt(signed=False, label="UInt32"), arg_names=["engineHandle", "key", "securityInfo", "sidOwner", "sidGroup", "dacl", "sacl"]),
        # 
        'FwpmProviderSubscribeChanges0': SimTypeFunction([SimTypePointer(SimTypeInt(signed=True, label="Int"), label="IntPtr", offset=0), SimTypePointer(SimStruct({"enumTemplate": SimTypePointer(SimStruct({"reserved": SimTypeLongLong(signed=False, label="UInt64")}, name="FWPM_PROVIDER_ENUM_TEMPLATE0", pack=False, align=None), offset=0), "flags": SimTypeInt(signed=False, label="UInt32"), "sessionKey": SimTypeBottom(label="Guid")}, name="FWPM_PROVIDER_SUBSCRIPTION0", pack=False, align=None), offset=0), SimTypePointer(SimTypeFunction([SimTypePointer(SimTypeBottom(label="Void"), offset=0), SimTypePointer(SimStruct({"changeType": SimTypeInt(signed=False, label="FWPM_CHANGE_TYPE"), "providerKey": SimTypeBottom(label="Guid")}, name="FWPM_PROVIDER_CHANGE0", pack=False, align=None), offset=0)], SimTypeBottom(label="Void"), arg_names=["context", "change"]), offset=0), SimTypePointer(SimTypeBottom(label="Void"), offset=0), SimTypePointer(SimTypePointer(SimTypeInt(signed=True, label="Int"), label="IntPtr", offset=0), offset=0)], SimTypeInt(signed=False, label="UInt32"), arg_names=["engineHandle", "subscription", "callback", "context", "changeHandle"]),
        # 
        'FwpmProviderUnsubscribeChanges0': SimTypeFunction([SimTypePointer(SimTypeInt(signed=True, label="Int"), label="IntPtr", offset=0), SimTypePointer(SimTypeInt(signed=True, label="Int"), label="IntPtr", offset=0)], SimTypeInt(signed=False, label="UInt32"), arg_names=["engineHandle", "changeHandle"]),
        # 
        'FwpmProviderSubscriptionsGet0': SimTypeFunction([SimTypePointer(SimTypeInt(signed=True, label="Int"), label="IntPtr", offset=0), SimTypePointer(SimTypePointer(SimTypePointer(SimStruct({"enumTemplate": SimTypePointer(SimStruct({"reserved": SimTypeLongLong(signed=False, label="UInt64")}, name="FWPM_PROVIDER_ENUM_TEMPLATE0", pack=False, align=None), offset=0), "flags": SimTypeInt(signed=False, label="UInt32"), "sessionKey": SimTypeBottom(label="Guid")}, name="FWPM_PROVIDER_SUBSCRIPTION0", pack=False, align=None), offset=0), offset=0), offset=0), SimTypePointer(SimTypeInt(signed=False, label="UInt32"), offset=0)], SimTypeInt(signed=False, label="UInt32"), arg_names=["engineHandle", "entries", "numEntries"]),
        # 
        'FwpmProviderContextAdd0': SimTypeFunction([SimTypePointer(SimTypeInt(signed=True, label="Int"), label="IntPtr", offset=0), SimTypePointer(SimStruct({"providerContextKey": SimTypeBottom(label="Guid"), "displayData": SimStruct({"name": SimTypePointer(SimTypeChar(label="Char"), offset=0), "description": SimTypePointer(SimTypeChar(label="Char"), offset=0)}, name="FWPM_DISPLAY_DATA0", pack=False, align=None), "flags": SimTypeInt(signed=False, label="UInt32"), "providerKey": SimTypePointer(SimTypeBottom(label="Guid"), offset=0), "providerData": SimStruct({"size": SimTypeInt(signed=False, label="UInt32"), "data": SimTypePointer(SimTypeChar(label="Byte"), offset=0)}, name="FWP_BYTE_BLOB", pack=False, align=None), "type": SimTypeInt(signed=False, label="FWPM_PROVIDER_CONTEXT_TYPE"), "Anonymous": SimUnion({"keyingPolicy": SimTypePointer(SimStruct({"numKeyMods": SimTypeInt(signed=False, label="UInt32"), "keyModKeys": SimTypePointer(SimTypeBottom(label="Guid"), offset=0)}, name="IPSEC_KEYING_POLICY0", pack=False, align=None), offset=0), "ikeQmTransportPolicy": SimTypePointer(SimStruct({"numIpsecProposals": SimTypeInt(signed=False, label="UInt32"), "ipsecProposals": SimTypePointer(SimStruct({"lifetime": SimStruct({"lifetimeSeconds": SimTypeInt(signed=False, label="UInt32"), "lifetimeKilobytes": SimTypeInt(signed=False, label="UInt32"), "lifetimePackets": SimTypeInt(signed=False, label="UInt32")}, name="IPSEC_SA_LIFETIME0", pack=False, align=None), "numSaTransforms": SimTypeInt(signed=False, label="UInt32"), "saTransforms": SimTypePointer(SimStruct({"ipsecTransformType": SimTypeInt(signed=False, label="IPSEC_TRANSFORM_TYPE"), "Anonymous": SimUnion({"ahTransform": SimTypePointer(SimStruct({"authTransformId": SimStruct({"authType": SimTypeInt(signed=False, label="IPSEC_AUTH_TYPE"), "authConfig": SimTypeChar(label="Byte")}, name="IPSEC_AUTH_TRANSFORM_ID0", pack=False, align=None), "cryptoModuleId": SimTypePointer(SimTypeBottom(label="Guid"), offset=0)}, name="IPSEC_AUTH_TRANSFORM0", pack=False, align=None), offset=0), "espAuthTransform": SimTypePointer(SimStruct({"authTransformId": SimStruct({"authType": SimTypeInt(signed=False, label="IPSEC_AUTH_TYPE"), "authConfig": SimTypeChar(label="Byte")}, name="IPSEC_AUTH_TRANSFORM_ID0", pack=False, align=None), "cryptoModuleId": SimTypePointer(SimTypeBottom(label="Guid"), offset=0)}, name="IPSEC_AUTH_TRANSFORM0", pack=False, align=None), offset=0), "espCipherTransform": SimTypePointer(SimStruct({"cipherTransformId": SimStruct({"cipherType": SimTypeInt(signed=False, label="IPSEC_CIPHER_TYPE"), "cipherConfig": SimTypeChar(label="Byte")}, name="IPSEC_CIPHER_TRANSFORM_ID0", pack=False, align=None), "cryptoModuleId": SimTypePointer(SimTypeBottom(label="Guid"), offset=0)}, name="IPSEC_CIPHER_TRANSFORM0", pack=False, align=None), offset=0), "espAuthAndCipherTransform": SimTypePointer(SimStruct({"authTransform": SimStruct({"authTransformId": SimStruct({"authType": SimTypeInt(signed=False, label="IPSEC_AUTH_TYPE"), "authConfig": SimTypeChar(label="Byte")}, name="IPSEC_AUTH_TRANSFORM_ID0", pack=False, align=None), "cryptoModuleId": SimTypePointer(SimTypeBottom(label="Guid"), offset=0)}, name="IPSEC_AUTH_TRANSFORM0", pack=False, align=None), "cipherTransform": SimStruct({"cipherTransformId": SimStruct({"cipherType": SimTypeInt(signed=False, label="IPSEC_CIPHER_TYPE"), "cipherConfig": SimTypeChar(label="Byte")}, name="IPSEC_CIPHER_TRANSFORM_ID0", pack=False, align=None), "cryptoModuleId": SimTypePointer(SimTypeBottom(label="Guid"), offset=0)}, name="IPSEC_CIPHER_TRANSFORM0", pack=False, align=None)}, name="IPSEC_AUTH_AND_CIPHER_TRANSFORM0", pack=False, align=None), offset=0), "espAuthFwTransform": SimTypePointer(SimStruct({"authTransformId": SimStruct({"authType": SimTypeInt(signed=False, label="IPSEC_AUTH_TYPE"), "authConfig": SimTypeChar(label="Byte")}, name="IPSEC_AUTH_TRANSFORM_ID0", pack=False, align=None), "cryptoModuleId": SimTypePointer(SimTypeBottom(label="Guid"), offset=0)}, name="IPSEC_AUTH_TRANSFORM0", pack=False, align=None), offset=0)}, name="<anon>", label="None")}, name="IPSEC_SA_TRANSFORM0", pack=False, align=None), offset=0), "pfsGroup": SimTypeInt(signed=False, label="IPSEC_PFS_GROUP")}, name="IPSEC_PROPOSAL0", pack=False, align=None), offset=0), "flags": SimTypeInt(signed=False, label="IPSEC_POLICY_FLAG"), "ndAllowClearTimeoutSeconds": SimTypeInt(signed=False, label="UInt32"), "saIdleTimeout": SimStruct({"idleTimeoutSeconds": SimTypeInt(signed=False, label="UInt32"), "idleTimeoutSecondsFailOver": SimTypeInt(signed=False, label="UInt32")}, name="IPSEC_SA_IDLE_TIMEOUT0", pack=False, align=None), "emPolicy": SimTypePointer(SimStruct({"numAuthenticationMethods": SimTypeInt(signed=False, label="UInt32"), "authenticationMethods": SimTypePointer(SimStruct({"authenticationMethodType": SimTypeInt(signed=False, label="IKEEXT_AUTHENTICATION_METHOD_TYPE"), "Anonymous": SimUnion({"presharedKeyAuthentication": SimStruct({"presharedKey": SimStruct({"size": SimTypeInt(signed=False, label="UInt32"), "data": SimTypePointer(SimTypeChar(label="Byte"), offset=0)}, name="FWP_BYTE_BLOB", pack=False, align=None)}, name="IKEEXT_PRESHARED_KEY_AUTHENTICATION0", pack=False, align=None), "certificateAuthentication": SimStruct({"inboundConfigType": SimTypeInt(signed=False, label="IKEEXT_CERT_CONFIG_TYPE"), "Anonymous1": SimUnion({"Anonymous": SimStruct({"inboundRootArraySize": SimTypeInt(signed=False, label="UInt32"), "inboundRootArray": SimTypePointer(SimStruct({"certData": SimStruct({"size": SimTypeInt(signed=False, label="UInt32"), "data": SimTypePointer(SimTypeChar(label="Byte"), offset=0)}, name="FWP_BYTE_BLOB", pack=False, align=None), "flags": SimTypeInt(signed=False, label="IKEEXT_CERT_FLAGS")}, name="IKEEXT_CERT_ROOT_CONFIG0", pack=False, align=None), offset=0)}, name="_Anonymous_e__Struct", pack=False, align=None), "inboundEnterpriseStoreConfig": SimTypePointer(SimStruct({"certData": SimStruct({"size": SimTypeInt(signed=False, label="UInt32"), "data": SimTypePointer(SimTypeChar(label="Byte"), offset=0)}, name="FWP_BYTE_BLOB", pack=False, align=None), "flags": SimTypeInt(signed=False, label="IKEEXT_CERT_FLAGS")}, name="IKEEXT_CERT_ROOT_CONFIG0", pack=False, align=None), offset=0), "inboundTrustedRootStoreConfig": SimTypePointer(SimStruct({"certData": SimStruct({"size": SimTypeInt(signed=False, label="UInt32"), "data": SimTypePointer(SimTypeChar(label="Byte"), offset=0)}, name="FWP_BYTE_BLOB", pack=False, align=None), "flags": SimTypeInt(signed=False, label="IKEEXT_CERT_FLAGS")}, name="IKEEXT_CERT_ROOT_CONFIG0", pack=False, align=None), offset=0)}, name="<anon>", label="None"), "outboundConfigType": SimTypeInt(signed=False, label="IKEEXT_CERT_CONFIG_TYPE"), "Anonymous2": SimUnion({"Anonymous": SimStruct({"outboundRootArraySize": SimTypeInt(signed=False, label="UInt32"), "outboundRootArray": SimTypePointer(SimStruct({"certData": SimStruct({"size": SimTypeInt(signed=False, label="UInt32"), "data": SimTypePointer(SimTypeChar(label="Byte"), offset=0)}, name="FWP_BYTE_BLOB", pack=False, align=None), "flags": SimTypeInt(signed=False, label="IKEEXT_CERT_FLAGS")}, name="IKEEXT_CERT_ROOT_CONFIG0", pack=False, align=None), offset=0)}, name="_Anonymous_e__Struct", pack=False, align=None), "outboundEnterpriseStoreConfig": SimTypePointer(SimStruct({"certData": SimStruct({"size": SimTypeInt(signed=False, label="UInt32"), "data": SimTypePointer(SimTypeChar(label="Byte"), offset=0)}, name="FWP_BYTE_BLOB", pack=False, align=None), "flags": SimTypeInt(signed=False, label="IKEEXT_CERT_FLAGS")}, name="IKEEXT_CERT_ROOT_CONFIG0", pack=False, align=None), offset=0), "outboundTrustedRootStoreConfig": SimTypePointer(SimStruct({"certData": SimStruct({"size": SimTypeInt(signed=False, label="UInt32"), "data": SimTypePointer(SimTypeChar(label="Byte"), offset=0)}, name="FWP_BYTE_BLOB", pack=False, align=None), "flags": SimTypeInt(signed=False, label="IKEEXT_CERT_FLAGS")}, name="IKEEXT_CERT_ROOT_CONFIG0", pack=False, align=None), offset=0)}, name="<anon>", label="None"), "flags": SimTypeInt(signed=False, label="IKEEXT_CERT_AUTH")}, name="IKEEXT_CERTIFICATE_AUTHENTICATION0", pack=False, align=None), "kerberosAuthentication": SimStruct({"flags": SimTypeInt(signed=False, label="IKEEXT_KERBEROS_AUTHENTICATION_FLAGS")}, name="IKEEXT_KERBEROS_AUTHENTICATION0", pack=False, align=None), "ntlmV2Authentication": SimStruct({"flags": SimTypeInt(signed=False, label="UInt32")}, name="IKEEXT_NTLM_V2_AUTHENTICATION0", pack=False, align=None), "sslAuthentication": SimStruct({"inboundConfigType": SimTypeInt(signed=False, label="IKEEXT_CERT_CONFIG_TYPE"), "Anonymous1": SimUnion({"Anonymous": SimStruct({"inboundRootArraySize": SimTypeInt(signed=False, label="UInt32"), "inboundRootArray": SimTypePointer(SimStruct({"certData": SimStruct({"size": SimTypeInt(signed=False, label="UInt32"), "data": SimTypePointer(SimTypeChar(label="Byte"), offset=0)}, name="FWP_BYTE_BLOB", pack=False, align=None), "flags": SimTypeInt(signed=False, label="IKEEXT_CERT_FLAGS")}, name="IKEEXT_CERT_ROOT_CONFIG0", pack=False, align=None), offset=0)}, name="_Anonymous_e__Struct", pack=False, align=None), "inboundEnterpriseStoreConfig": SimTypePointer(SimStruct({"certData": SimStruct({"size": SimTypeInt(signed=False, label="UInt32"), "data": SimTypePointer(SimTypeChar(label="Byte"), offset=0)}, name="FWP_BYTE_BLOB", pack=False, align=None), "flags": SimTypeInt(signed=False, label="IKEEXT_CERT_FLAGS")}, name="IKEEXT_CERT_ROOT_CONFIG0", pack=False, align=None), offset=0), "inboundTrustedRootStoreConfig": SimTypePointer(SimStruct({"certData": SimStruct({"size": SimTypeInt(signed=False, label="UInt32"), "data": SimTypePointer(SimTypeChar(label="Byte"), offset=0)}, name="FWP_BYTE_BLOB", pack=False, align=None), "flags": SimTypeInt(signed=False, label="IKEEXT_CERT_FLAGS")}, name="IKEEXT_CERT_ROOT_CONFIG0", pack=False, align=None), offset=0)}, name="<anon>", label="None"), "outboundConfigType": SimTypeInt(signed=False, label="IKEEXT_CERT_CONFIG_TYPE"), "Anonymous2": SimUnion({"Anonymous": SimStruct({"outboundRootArraySize": SimTypeInt(signed=False, label="UInt32"), "outboundRootArray": SimTypePointer(SimStruct({"certData": SimStruct({"size": SimTypeInt(signed=False, label="UInt32"), "data": SimTypePointer(SimTypeChar(label="Byte"), offset=0)}, name="FWP_BYTE_BLOB", pack=False, align=None), "flags": SimTypeInt(signed=False, label="IKEEXT_CERT_FLAGS")}, name="IKEEXT_CERT_ROOT_CONFIG0", pack=False, align=None), offset=0)}, name="_Anonymous_e__Struct", pack=False, align=None), "outboundEnterpriseStoreConfig": SimTypePointer(SimStruct({"certData": SimStruct({"size": SimTypeInt(signed=False, label="UInt32"), "data": SimTypePointer(SimTypeChar(label="Byte"), offset=0)}, name="FWP_BYTE_BLOB", pack=False, align=None), "flags": SimTypeInt(signed=False, label="IKEEXT_CERT_FLAGS")}, name="IKEEXT_CERT_ROOT_CONFIG0", pack=False, align=None), offset=0), "outboundTrustedRootStoreConfig": SimTypePointer(SimStruct({"certData": SimStruct({"size": SimTypeInt(signed=False, label="UInt32"), "data": SimTypePointer(SimTypeChar(label="Byte"), offset=0)}, name="FWP_BYTE_BLOB", pack=False, align=None), "flags": SimTypeInt(signed=False, label="IKEEXT_CERT_FLAGS")}, name="IKEEXT_CERT_ROOT_CONFIG0", pack=False, align=None), offset=0)}, name="<anon>", label="None"), "flags": SimTypeInt(signed=False, label="IKEEXT_CERT_AUTH")}, name="IKEEXT_CERTIFICATE_AUTHENTICATION0", pack=False, align=None), "cgaAuthentication": SimStruct({"keyContainerName": SimTypePointer(SimTypeChar(label="Char"), offset=0), "cspName": SimTypePointer(SimTypeChar(label="Char"), offset=0), "cspType": SimTypeInt(signed=False, label="UInt32"), "cgaModifier": SimStruct({"byteArray16": SimTypeFixedSizeArray(SimTypeChar(label="Byte"), 16)}, name="FWP_BYTE_ARRAY16", pack=False, align=None), "cgaCollisionCount": SimTypeChar(label="Byte")}, name="IKEEXT_IPV6_CGA_AUTHENTICATION0", pack=False, align=None)}, name="<anon>", label="None")}, name="IKEEXT_AUTHENTICATION_METHOD0", pack=False, align=None), offset=0), "initiatorImpersonationType": SimTypeInt(signed=False, label="IKEEXT_AUTHENTICATION_IMPERSONATION_TYPE")}, name="IKEEXT_EM_POLICY0", pack=False, align=None), offset=0)}, name="IPSEC_TRANSPORT_POLICY0", pack=False, align=None), offset=0), "ikeQmTunnelPolicy": SimTypePointer(SimStruct({"flags": SimTypeInt(signed=False, label="IPSEC_POLICY_FLAG"), "numIpsecProposals": SimTypeInt(signed=False, label="UInt32"), "ipsecProposals": SimTypePointer(SimStruct({"lifetime": SimStruct({"lifetimeSeconds": SimTypeInt(signed=False, label="UInt32"), "lifetimeKilobytes": SimTypeInt(signed=False, label="UInt32"), "lifetimePackets": SimTypeInt(signed=False, label="UInt32")}, name="IPSEC_SA_LIFETIME0", pack=False, align=None), "numSaTransforms": SimTypeInt(signed=False, label="UInt32"), "saTransforms": SimTypePointer(SimStruct({"ipsecTransformType": SimTypeInt(signed=False, label="IPSEC_TRANSFORM_TYPE"), "Anonymous": SimUnion({"ahTransform": SimTypePointer(SimStruct({"authTransformId": SimStruct({"authType": SimTypeInt(signed=False, label="IPSEC_AUTH_TYPE"), "authConfig": SimTypeChar(label="Byte")}, name="IPSEC_AUTH_TRANSFORM_ID0", pack=False, align=None), "cryptoModuleId": SimTypePointer(SimTypeBottom(label="Guid"), offset=0)}, name="IPSEC_AUTH_TRANSFORM0", pack=False, align=None), offset=0), "espAuthTransform": SimTypePointer(SimStruct({"authTransformId": SimStruct({"authType": SimTypeInt(signed=False, label="IPSEC_AUTH_TYPE"), "authConfig": SimTypeChar(label="Byte")}, name="IPSEC_AUTH_TRANSFORM_ID0", pack=False, align=None), "cryptoModuleId": SimTypePointer(SimTypeBottom(label="Guid"), offset=0)}, name="IPSEC_AUTH_TRANSFORM0", pack=False, align=None), offset=0), "espCipherTransform": SimTypePointer(SimStruct({"cipherTransformId": SimStruct({"cipherType": SimTypeInt(signed=False, label="IPSEC_CIPHER_TYPE"), "cipherConfig": SimTypeChar(label="Byte")}, name="IPSEC_CIPHER_TRANSFORM_ID0", pack=False, align=None), "cryptoModuleId": SimTypePointer(SimTypeBottom(label="Guid"), offset=0)}, name="IPSEC_CIPHER_TRANSFORM0", pack=False, align=None), offset=0), "espAuthAndCipherTransform": SimTypePointer(SimStruct({"authTransform": SimStruct({"authTransformId": SimStruct({"authType": SimTypeInt(signed=False, label="IPSEC_AUTH_TYPE"), "authConfig": SimTypeChar(label="Byte")}, name="IPSEC_AUTH_TRANSFORM_ID0", pack=False, align=None), "cryptoModuleId": SimTypePointer(SimTypeBottom(label="Guid"), offset=0)}, name="IPSEC_AUTH_TRANSFORM0", pack=False, align=None), "cipherTransform": SimStruct({"cipherTransformId": SimStruct({"cipherType": SimTypeInt(signed=False, label="IPSEC_CIPHER_TYPE"), "cipherConfig": SimTypeChar(label="Byte")}, name="IPSEC_CIPHER_TRANSFORM_ID0", pack=False, align=None), "cryptoModuleId": SimTypePointer(SimTypeBottom(label="Guid"), offset=0)}, name="IPSEC_CIPHER_TRANSFORM0", pack=False, align=None)}, name="IPSEC_AUTH_AND_CIPHER_TRANSFORM0", pack=False, align=None), offset=0), "espAuthFwTransform": SimTypePointer(SimStruct({"authTransformId": SimStruct({"authType": SimTypeInt(signed=False, label="IPSEC_AUTH_TYPE"), "authConfig": SimTypeChar(label="Byte")}, name="IPSEC_AUTH_TRANSFORM_ID0", pack=False, align=None), "cryptoModuleId": SimTypePointer(SimTypeBottom(label="Guid"), offset=0)}, name="IPSEC_AUTH_TRANSFORM0", pack=False, align=None), offset=0)}, name="<anon>", label="None")}, name="IPSEC_SA_TRANSFORM0", pack=False, align=None), offset=0), "pfsGroup": SimTypeInt(signed=False, label="IPSEC_PFS_GROUP")}, name="IPSEC_PROPOSAL0", pack=False, align=None), offset=0), "tunnelEndpoints": SimStruct({"ipVersion": SimTypeInt(signed=False, label="FWP_IP_VERSION"), "Anonymous1": SimUnion({"localV4Address": SimTypeInt(signed=False, label="UInt32"), "localV6Address": SimTypeFixedSizeArray(SimTypeChar(label="Byte"), 16)}, name="<anon>", label="None"), "Anonymous2": SimUnion({"remoteV4Address": SimTypeInt(signed=False, label="UInt32"), "remoteV6Address": SimTypeFixedSizeArray(SimTypeChar(label="Byte"), 16)}, name="<anon>", label="None")}, name="IPSEC_TUNNEL_ENDPOINTS0", pack=False, align=None), "saIdleTimeout": SimStruct({"idleTimeoutSeconds": SimTypeInt(signed=False, label="UInt32"), "idleTimeoutSecondsFailOver": SimTypeInt(signed=False, label="UInt32")}, name="IPSEC_SA_IDLE_TIMEOUT0", pack=False, align=None), "emPolicy": SimTypePointer(SimStruct({"numAuthenticationMethods": SimTypeInt(signed=False, label="UInt32"), "authenticationMethods": SimTypePointer(SimStruct({"authenticationMethodType": SimTypeInt(signed=False, label="IKEEXT_AUTHENTICATION_METHOD_TYPE"), "Anonymous": SimUnion({"presharedKeyAuthentication": SimStruct({"presharedKey": SimStruct({"size": SimTypeInt(signed=False, label="UInt32"), "data": SimTypePointer(SimTypeChar(label="Byte"), offset=0)}, name="FWP_BYTE_BLOB", pack=False, align=None)}, name="IKEEXT_PRESHARED_KEY_AUTHENTICATION0", pack=False, align=None), "certificateAuthentication": SimStruct({"inboundConfigType": SimTypeInt(signed=False, label="IKEEXT_CERT_CONFIG_TYPE"), "Anonymous1": SimUnion({"Anonymous": SimStruct({"inboundRootArraySize": SimTypeInt(signed=False, label="UInt32"), "inboundRootArray": SimTypePointer(SimStruct({"certData": SimStruct({"size": SimTypeInt(signed=False, label="UInt32"), "data": SimTypePointer(SimTypeChar(label="Byte"), offset=0)}, name="FWP_BYTE_BLOB", pack=False, align=None), "flags": SimTypeInt(signed=False, label="IKEEXT_CERT_FLAGS")}, name="IKEEXT_CERT_ROOT_CONFIG0", pack=False, align=None), offset=0)}, name="_Anonymous_e__Struct", pack=False, align=None), "inboundEnterpriseStoreConfig": SimTypePointer(SimStruct({"certData": SimStruct({"size": SimTypeInt(signed=False, label="UInt32"), "data": SimTypePointer(SimTypeChar(label="Byte"), offset=0)}, name="FWP_BYTE_BLOB", pack=False, align=None), "flags": SimTypeInt(signed=False, label="IKEEXT_CERT_FLAGS")}, name="IKEEXT_CERT_ROOT_CONFIG0", pack=False, align=None), offset=0), "inboundTrustedRootStoreConfig": SimTypePointer(SimStruct({"certData": SimStruct({"size": SimTypeInt(signed=False, label="UInt32"), "data": SimTypePointer(SimTypeChar(label="Byte"), offset=0)}, name="FWP_BYTE_BLOB", pack=False, align=None), "flags": SimTypeInt(signed=False, label="IKEEXT_CERT_FLAGS")}, name="IKEEXT_CERT_ROOT_CONFIG0", pack=False, align=None), offset=0)}, name="<anon>", label="None"), "outboundConfigType": SimTypeInt(signed=False, label="IKEEXT_CERT_CONFIG_TYPE"), "Anonymous2": SimUnion({"Anonymous": SimStruct({"outboundRootArraySize": SimTypeInt(signed=False, label="UInt32"), "outboundRootArray": SimTypePointer(SimStruct({"certData": SimStruct({"size": SimTypeInt(signed=False, label="UInt32"), "data": SimTypePointer(SimTypeChar(label="Byte"), offset=0)}, name="FWP_BYTE_BLOB", pack=False, align=None), "flags": SimTypeInt(signed=False, label="IKEEXT_CERT_FLAGS")}, name="IKEEXT_CERT_ROOT_CONFIG0", pack=False, align=None), offset=0)}, name="_Anonymous_e__Struct", pack=False, align=None), "outboundEnterpriseStoreConfig": SimTypePointer(SimStruct({"certData": SimStruct({"size": SimTypeInt(signed=False, label="UInt32"), "data": SimTypePointer(SimTypeChar(label="Byte"), offset=0)}, name="FWP_BYTE_BLOB", pack=False, align=None), "flags": SimTypeInt(signed=False, label="IKEEXT_CERT_FLAGS")}, name="IKEEXT_CERT_ROOT_CONFIG0", pack=False, align=None), offset=0), "outboundTrustedRootStoreConfig": SimTypePointer(SimStruct({"certData": SimStruct({"size": SimTypeInt(signed=False, label="UInt32"), "data": SimTypePointer(SimTypeChar(label="Byte"), offset=0)}, name="FWP_BYTE_BLOB", pack=False, align=None), "flags": SimTypeInt(signed=False, label="IKEEXT_CERT_FLAGS")}, name="IKEEXT_CERT_ROOT_CONFIG0", pack=False, align=None), offset=0)}, name="<anon>", label="None"), "flags": SimTypeInt(signed=False, label="IKEEXT_CERT_AUTH")}, name="IKEEXT_CERTIFICATE_AUTHENTICATION0", pack=False, align=None), "kerberosAuthentication": SimStruct({"flags": SimTypeInt(signed=False, label="IKEEXT_KERBEROS_AUTHENTICATION_FLAGS")}, name="IKEEXT_KERBEROS_AUTHENTICATION0", pack=False, align=None), "ntlmV2Authentication": SimStruct({"flags": SimTypeInt(signed=False, label="UInt32")}, name="IKEEXT_NTLM_V2_AUTHENTICATION0", pack=False, align=None), "sslAuthentication": SimStruct({"inboundConfigType": SimTypeInt(signed=False, label="IKEEXT_CERT_CONFIG_TYPE"), "Anonymous1": SimUnion({"Anonymous": SimStruct({"inboundRootArraySize": SimTypeInt(signed=False, label="UInt32"), "inboundRootArray": SimTypePointer(SimStruct({"certData": SimStruct({"size": SimTypeInt(signed=False, label="UInt32"), "data": SimTypePointer(SimTypeChar(label="Byte"), offset=0)}, name="FWP_BYTE_BLOB", pack=False, align=None), "flags": SimTypeInt(signed=False, label="IKEEXT_CERT_FLAGS")}, name="IKEEXT_CERT_ROOT_CONFIG0", pack=False, align=None), offset=0)}, name="_Anonymous_e__Struct", pack=False, align=None), "inboundEnterpriseStoreConfig": SimTypePointer(SimStruct({"certData": SimStruct({"size": SimTypeInt(signed=False, label="UInt32"), "data": SimTypePointer(SimTypeChar(label="Byte"), offset=0)}, name="FWP_BYTE_BLOB", pack=False, align=None), "flags": SimTypeInt(signed=False, label="IKEEXT_CERT_FLAGS")}, name="IKEEXT_CERT_ROOT_CONFIG0", pack=False, align=None), offset=0), "inboundTrustedRootStoreConfig": SimTypePointer(SimStruct({"certData": SimStruct({"size": SimTypeInt(signed=False, label="UInt32"), "data": SimTypePointer(SimTypeChar(label="Byte"), offset=0)}, name="FWP_BYTE_BLOB", pack=False, align=None), "flags": SimTypeInt(signed=False, label="IKEEXT_CERT_FLAGS")}, name="IKEEXT_CERT_ROOT_CONFIG0", pack=False, align=None), offset=0)}, name="<anon>", label="None"), "outboundConfigType": SimTypeInt(signed=False, label="IKEEXT_CERT_CONFIG_TYPE"), "Anonymous2": SimUnion({"Anonymous": SimStruct({"outboundRootArraySize": SimTypeInt(signed=False, label="UInt32"), "outboundRootArray": SimTypePointer(SimStruct({"certData": SimStruct({"size": SimTypeInt(signed=False, label="UInt32"), "data": SimTypePointer(SimTypeChar(label="Byte"), offset=0)}, name="FWP_BYTE_BLOB", pack=False, align=None), "flags": SimTypeInt(signed=False, label="IKEEXT_CERT_FLAGS")}, name="IKEEXT_CERT_ROOT_CONFIG0", pack=False, align=None), offset=0)}, name="_Anonymous_e__Struct", pack=False, align=None), "outboundEnterpriseStoreConfig": SimTypePointer(SimStruct({"certData": SimStruct({"size": SimTypeInt(signed=False, label="UInt32"), "data": SimTypePointer(SimTypeChar(label="Byte"), offset=0)}, name="FWP_BYTE_BLOB", pack=False, align=None), "flags": SimTypeInt(signed=False, label="IKEEXT_CERT_FLAGS")}, name="IKEEXT_CERT_ROOT_CONFIG0", pack=False, align=None), offset=0), "outboundTrustedRootStoreConfig": SimTypePointer(SimStruct({"certData": SimStruct({"size": SimTypeInt(signed=False, label="UInt32"), "data": SimTypePointer(SimTypeChar(label="Byte"), offset=0)}, name="FWP_BYTE_BLOB", pack=False, align=None), "flags": SimTypeInt(signed=False, label="IKEEXT_CERT_FLAGS")}, name="IKEEXT_CERT_ROOT_CONFIG0", pack=False, align=None), offset=0)}, name="<anon>", label="None"), "flags": SimTypeInt(signed=False, label="IKEEXT_CERT_AUTH")}, name="IKEEXT_CERTIFICATE_AUTHENTICATION0", pack=False, align=None), "cgaAuthentication": SimStruct({"keyContainerName": SimTypePointer(SimTypeChar(label="Char"), offset=0), "cspName": SimTypePointer(SimTypeChar(label="Char"), offset=0), "cspType": SimTypeInt(signed=False, label="UInt32"), "cgaModifier": SimStruct({"byteArray16": SimTypeFixedSizeArray(SimTypeChar(label="Byte"), 16)}, name="FWP_BYTE_ARRAY16", pack=False, align=None), "cgaCollisionCount": SimTypeChar(label="Byte")}, name="IKEEXT_IPV6_CGA_AUTHENTICATION0", pack=False, align=None)}, name="<anon>", label="None")}, name="IKEEXT_AUTHENTICATION_METHOD0", pack=False, align=None), offset=0), "initiatorImpersonationType": SimTypeInt(signed=False, label="IKEEXT_AUTHENTICATION_IMPERSONATION_TYPE")}, name="IKEEXT_EM_POLICY0", pack=False, align=None), offset=0)}, name="IPSEC_TUNNEL_POLICY0", pack=False, align=None), offset=0), "authipQmTransportPolicy": SimTypePointer(SimStruct({"numIpsecProposals": SimTypeInt(signed=False, label="UInt32"), "ipsecProposals": SimTypePointer(SimStruct({"lifetime": SimStruct({"lifetimeSeconds": SimTypeInt(signed=False, label="UInt32"), "lifetimeKilobytes": SimTypeInt(signed=False, label="UInt32"), "lifetimePackets": SimTypeInt(signed=False, label="UInt32")}, name="IPSEC_SA_LIFETIME0", pack=False, align=None), "numSaTransforms": SimTypeInt(signed=False, label="UInt32"), "saTransforms": SimTypePointer(SimStruct({"ipsecTransformType": SimTypeInt(signed=False, label="IPSEC_TRANSFORM_TYPE"), "Anonymous": SimUnion({"ahTransform": SimTypePointer(SimStruct({"authTransformId": SimStruct({"authType": SimTypeInt(signed=False, label="IPSEC_AUTH_TYPE"), "authConfig": SimTypeChar(label="Byte")}, name="IPSEC_AUTH_TRANSFORM_ID0", pack=False, align=None), "cryptoModuleId": SimTypePointer(SimTypeBottom(label="Guid"), offset=0)}, name="IPSEC_AUTH_TRANSFORM0", pack=False, align=None), offset=0), "espAuthTransform": SimTypePointer(SimStruct({"authTransformId": SimStruct({"authType": SimTypeInt(signed=False, label="IPSEC_AUTH_TYPE"), "authConfig": SimTypeChar(label="Byte")}, name="IPSEC_AUTH_TRANSFORM_ID0", pack=False, align=None), "cryptoModuleId": SimTypePointer(SimTypeBottom(label="Guid"), offset=0)}, name="IPSEC_AUTH_TRANSFORM0", pack=False, align=None), offset=0), "espCipherTransform": SimTypePointer(SimStruct({"cipherTransformId": SimStruct({"cipherType": SimTypeInt(signed=False, label="IPSEC_CIPHER_TYPE"), "cipherConfig": SimTypeChar(label="Byte")}, name="IPSEC_CIPHER_TRANSFORM_ID0", pack=False, align=None), "cryptoModuleId": SimTypePointer(SimTypeBottom(label="Guid"), offset=0)}, name="IPSEC_CIPHER_TRANSFORM0", pack=False, align=None), offset=0), "espAuthAndCipherTransform": SimTypePointer(SimStruct({"authTransform": SimStruct({"authTransformId": SimStruct({"authType": SimTypeInt(signed=False, label="IPSEC_AUTH_TYPE"), "authConfig": SimTypeChar(label="Byte")}, name="IPSEC_AUTH_TRANSFORM_ID0", pack=False, align=None), "cryptoModuleId": SimTypePointer(SimTypeBottom(label="Guid"), offset=0)}, name="IPSEC_AUTH_TRANSFORM0", pack=False, align=None), "cipherTransform": SimStruct({"cipherTransformId": SimStruct({"cipherType": SimTypeInt(signed=False, label="IPSEC_CIPHER_TYPE"), "cipherConfig": SimTypeChar(label="Byte")}, name="IPSEC_CIPHER_TRANSFORM_ID0", pack=False, align=None), "cryptoModuleId": SimTypePointer(SimTypeBottom(label="Guid"), offset=0)}, name="IPSEC_CIPHER_TRANSFORM0", pack=False, align=None)}, name="IPSEC_AUTH_AND_CIPHER_TRANSFORM0", pack=False, align=None), offset=0), "espAuthFwTransform": SimTypePointer(SimStruct({"authTransformId": SimStruct({"authType": SimTypeInt(signed=False, label="IPSEC_AUTH_TYPE"), "authConfig": SimTypeChar(label="Byte")}, name="IPSEC_AUTH_TRANSFORM_ID0", pack=False, align=None), "cryptoModuleId": SimTypePointer(SimTypeBottom(label="Guid"), offset=0)}, name="IPSEC_AUTH_TRANSFORM0", pack=False, align=None), offset=0)}, name="<anon>", label="None")}, name="IPSEC_SA_TRANSFORM0", pack=False, align=None), offset=0), "pfsGroup": SimTypeInt(signed=False, label="IPSEC_PFS_GROUP")}, name="IPSEC_PROPOSAL0", pack=False, align=None), offset=0), "flags": SimTypeInt(signed=False, label="IPSEC_POLICY_FLAG"), "ndAllowClearTimeoutSeconds": SimTypeInt(signed=False, label="UInt32"), "saIdleTimeout": SimStruct({"idleTimeoutSeconds": SimTypeInt(signed=False, label="UInt32"), "idleTimeoutSecondsFailOver": SimTypeInt(signed=False, label="UInt32")}, name="IPSEC_SA_IDLE_TIMEOUT0", pack=False, align=None), "emPolicy": SimTypePointer(SimStruct({"numAuthenticationMethods": SimTypeInt(signed=False, label="UInt32"), "authenticationMethods": SimTypePointer(SimStruct({"authenticationMethodType": SimTypeInt(signed=False, label="IKEEXT_AUTHENTICATION_METHOD_TYPE"), "Anonymous": SimUnion({"presharedKeyAuthentication": SimStruct({"presharedKey": SimStruct({"size": SimTypeInt(signed=False, label="UInt32"), "data": SimTypePointer(SimTypeChar(label="Byte"), offset=0)}, name="FWP_BYTE_BLOB", pack=False, align=None)}, name="IKEEXT_PRESHARED_KEY_AUTHENTICATION0", pack=False, align=None), "certificateAuthentication": SimStruct({"inboundConfigType": SimTypeInt(signed=False, label="IKEEXT_CERT_CONFIG_TYPE"), "Anonymous1": SimUnion({"Anonymous": SimStruct({"inboundRootArraySize": SimTypeInt(signed=False, label="UInt32"), "inboundRootArray": SimTypePointer(SimStruct({"certData": SimStruct({"size": SimTypeInt(signed=False, label="UInt32"), "data": SimTypePointer(SimTypeChar(label="Byte"), offset=0)}, name="FWP_BYTE_BLOB", pack=False, align=None), "flags": SimTypeInt(signed=False, label="IKEEXT_CERT_FLAGS")}, name="IKEEXT_CERT_ROOT_CONFIG0", pack=False, align=None), offset=0)}, name="_Anonymous_e__Struct", pack=False, align=None), "inboundEnterpriseStoreConfig": SimTypePointer(SimStruct({"certData": SimStruct({"size": SimTypeInt(signed=False, label="UInt32"), "data": SimTypePointer(SimTypeChar(label="Byte"), offset=0)}, name="FWP_BYTE_BLOB", pack=False, align=None), "flags": SimTypeInt(signed=False, label="IKEEXT_CERT_FLAGS")}, name="IKEEXT_CERT_ROOT_CONFIG0", pack=False, align=None), offset=0), "inboundTrustedRootStoreConfig": SimTypePointer(SimStruct({"certData": SimStruct({"size": SimTypeInt(signed=False, label="UInt32"), "data": SimTypePointer(SimTypeChar(label="Byte"), offset=0)}, name="FWP_BYTE_BLOB", pack=False, align=None), "flags": SimTypeInt(signed=False, label="IKEEXT_CERT_FLAGS")}, name="IKEEXT_CERT_ROOT_CONFIG0", pack=False, align=None), offset=0)}, name="<anon>", label="None"), "outboundConfigType": SimTypeInt(signed=False, label="IKEEXT_CERT_CONFIG_TYPE"), "Anonymous2": SimUnion({"Anonymous": SimStruct({"outboundRootArraySize": SimTypeInt(signed=False, label="UInt32"), "outboundRootArray": SimTypePointer(SimStruct({"certData": SimStruct({"size": SimTypeInt(signed=False, label="UInt32"), "data": SimTypePointer(SimTypeChar(label="Byte"), offset=0)}, name="FWP_BYTE_BLOB", pack=False, align=None), "flags": SimTypeInt(signed=False, label="IKEEXT_CERT_FLAGS")}, name="IKEEXT_CERT_ROOT_CONFIG0", pack=False, align=None), offset=0)}, name="_Anonymous_e__Struct", pack=False, align=None), "outboundEnterpriseStoreConfig": SimTypePointer(SimStruct({"certData": SimStruct({"size": SimTypeInt(signed=False, label="UInt32"), "data": SimTypePointer(SimTypeChar(label="Byte"), offset=0)}, name="FWP_BYTE_BLOB", pack=False, align=None), "flags": SimTypeInt(signed=False, label="IKEEXT_CERT_FLAGS")}, name="IKEEXT_CERT_ROOT_CONFIG0", pack=False, align=None), offset=0), "outboundTrustedRootStoreConfig": SimTypePointer(SimStruct({"certData": SimStruct({"size": SimTypeInt(signed=False, label="UInt32"), "data": SimTypePointer(SimTypeChar(label="Byte"), offset=0)}, name="FWP_BYTE_BLOB", pack=False, align=None), "flags": SimTypeInt(signed=False, label="IKEEXT_CERT_FLAGS")}, name="IKEEXT_CERT_ROOT_CONFIG0", pack=False, align=None), offset=0)}, name="<anon>", label="None"), "flags": SimTypeInt(signed=False, label="IKEEXT_CERT_AUTH")}, name="IKEEXT_CERTIFICATE_AUTHENTICATION0", pack=False, align=None), "kerberosAuthentication": SimStruct({"flags": SimTypeInt(signed=False, label="IKEEXT_KERBEROS_AUTHENTICATION_FLAGS")}, name="IKEEXT_KERBEROS_AUTHENTICATION0", pack=False, align=None), "ntlmV2Authentication": SimStruct({"flags": SimTypeInt(signed=False, label="UInt32")}, name="IKEEXT_NTLM_V2_AUTHENTICATION0", pack=False, align=None), "sslAuthentication": SimStruct({"inboundConfigType": SimTypeInt(signed=False, label="IKEEXT_CERT_CONFIG_TYPE"), "Anonymous1": SimUnion({"Anonymous": SimStruct({"inboundRootArraySize": SimTypeInt(signed=False, label="UInt32"), "inboundRootArray": SimTypePointer(SimStruct({"certData": SimStruct({"size": SimTypeInt(signed=False, label="UInt32"), "data": SimTypePointer(SimTypeChar(label="Byte"), offset=0)}, name="FWP_BYTE_BLOB", pack=False, align=None), "flags": SimTypeInt(signed=False, label="IKEEXT_CERT_FLAGS")}, name="IKEEXT_CERT_ROOT_CONFIG0", pack=False, align=None), offset=0)}, name="_Anonymous_e__Struct", pack=False, align=None), "inboundEnterpriseStoreConfig": SimTypePointer(SimStruct({"certData": SimStruct({"size": SimTypeInt(signed=False, label="UInt32"), "data": SimTypePointer(SimTypeChar(label="Byte"), offset=0)}, name="FWP_BYTE_BLOB", pack=False, align=None), "flags": SimTypeInt(signed=False, label="IKEEXT_CERT_FLAGS")}, name="IKEEXT_CERT_ROOT_CONFIG0", pack=False, align=None), offset=0), "inboundTrustedRootStoreConfig": SimTypePointer(SimStruct({"certData": SimStruct({"size": SimTypeInt(signed=False, label="UInt32"), "data": SimTypePointer(SimTypeChar(label="Byte"), offset=0)}, name="FWP_BYTE_BLOB", pack=False, align=None), "flags": SimTypeInt(signed=False, label="IKEEXT_CERT_FLAGS")}, name="IKEEXT_CERT_ROOT_CONFIG0", pack=False, align=None), offset=0)}, name="<anon>", label="None"), "outboundConfigType": SimTypeInt(signed=False, label="IKEEXT_CERT_CONFIG_TYPE"), "Anonymous2": SimUnion({"Anonymous": SimStruct({"outboundRootArraySize": SimTypeInt(signed=False, label="UInt32"), "outboundRootArray": SimTypePointer(SimStruct({"certData": SimStruct({"size": SimTypeInt(signed=False, label="UInt32"), "data": SimTypePointer(SimTypeChar(label="Byte"), offset=0)}, name="FWP_BYTE_BLOB", pack=False, align=None), "flags": SimTypeInt(signed=False, label="IKEEXT_CERT_FLAGS")}, name="IKEEXT_CERT_ROOT_CONFIG0", pack=False, align=None), offset=0)}, name="_Anonymous_e__Struct", pack=False, align=None), "outboundEnterpriseStoreConfig": SimTypePointer(SimStruct({"certData": SimStruct({"size": SimTypeInt(signed=False, label="UInt32"), "data": SimTypePointer(SimTypeChar(label="Byte"), offset=0)}, name="FWP_BYTE_BLOB", pack=False, align=None), "flags": SimTypeInt(signed=False, label="IKEEXT_CERT_FLAGS")}, name="IKEEXT_CERT_ROOT_CONFIG0", pack=False, align=None), offset=0), "outboundTrustedRootStoreConfig": SimTypePointer(SimStruct({"certData": SimStruct({"size": SimTypeInt(signed=False, label="UInt32"), "data": SimTypePointer(SimTypeChar(label="Byte"), offset=0)}, name="FWP_BYTE_BLOB", pack=False, align=None), "flags": SimTypeInt(signed=False, label="IKEEXT_CERT_FLAGS")}, name="IKEEXT_CERT_ROOT_CONFIG0", pack=False, align=None), offset=0)}, name="<anon>", label="None"), "flags": SimTypeInt(signed=False, label="IKEEXT_CERT_AUTH")}, name="IKEEXT_CERTIFICATE_AUTHENTICATION0", pack=False, align=None), "cgaAuthentication": SimStruct({"keyContainerName": SimTypePointer(SimTypeChar(label="Char"), offset=0), "cspName": SimTypePointer(SimTypeChar(label="Char"), offset=0), "cspType": SimTypeInt(signed=False, label="UInt32"), "cgaModifier": SimStruct({"byteArray16": SimTypeFixedSizeArray(SimTypeChar(label="Byte"), 16)}, name="FWP_BYTE_ARRAY16", pack=False, align=None), "cgaCollisionCount": SimTypeChar(label="Byte")}, name="IKEEXT_IPV6_CGA_AUTHENTICATION0", pack=False, align=None)}, name="<anon>", label="None")}, name="IKEEXT_AUTHENTICATION_METHOD0", pack=False, align=None), offset=0), "initiatorImpersonationType": SimTypeInt(signed=False, label="IKEEXT_AUTHENTICATION_IMPERSONATION_TYPE")}, name="IKEEXT_EM_POLICY0", pack=False, align=None), offset=0)}, name="IPSEC_TRANSPORT_POLICY0", pack=False, align=None), offset=0), "authipQmTunnelPolicy": SimTypePointer(SimStruct({"flags": SimTypeInt(signed=False, label="IPSEC_POLICY_FLAG"), "numIpsecProposals": SimTypeInt(signed=False, label="UInt32"), "ipsecProposals": SimTypePointer(SimStruct({"lifetime": SimStruct({"lifetimeSeconds": SimTypeInt(signed=False, label="UInt32"), "lifetimeKilobytes": SimTypeInt(signed=False, label="UInt32"), "lifetimePackets": SimTypeInt(signed=False, label="UInt32")}, name="IPSEC_SA_LIFETIME0", pack=False, align=None), "numSaTransforms": SimTypeInt(signed=False, label="UInt32"), "saTransforms": SimTypePointer(SimStruct({"ipsecTransformType": SimTypeInt(signed=False, label="IPSEC_TRANSFORM_TYPE"), "Anonymous": SimUnion({"ahTransform": SimTypePointer(SimStruct({"authTransformId": SimStruct({"authType": SimTypeInt(signed=False, label="IPSEC_AUTH_TYPE"), "authConfig": SimTypeChar(label="Byte")}, name="IPSEC_AUTH_TRANSFORM_ID0", pack=False, align=None), "cryptoModuleId": SimTypePointer(SimTypeBottom(label="Guid"), offset=0)}, name="IPSEC_AUTH_TRANSFORM0", pack=False, align=None), offset=0), "espAuthTransform": SimTypePointer(SimStruct({"authTransformId": SimStruct({"authType": SimTypeInt(signed=False, label="IPSEC_AUTH_TYPE"), "authConfig": SimTypeChar(label="Byte")}, name="IPSEC_AUTH_TRANSFORM_ID0", pack=False, align=None), "cryptoModuleId": SimTypePointer(SimTypeBottom(label="Guid"), offset=0)}, name="IPSEC_AUTH_TRANSFORM0", pack=False, align=None), offset=0), "espCipherTransform": SimTypePointer(SimStruct({"cipherTransformId": SimStruct({"cipherType": SimTypeInt(signed=False, label="IPSEC_CIPHER_TYPE"), "cipherConfig": SimTypeChar(label="Byte")}, name="IPSEC_CIPHER_TRANSFORM_ID0", pack=False, align=None), "cryptoModuleId": SimTypePointer(SimTypeBottom(label="Guid"), offset=0)}, name="IPSEC_CIPHER_TRANSFORM0", pack=False, align=None), offset=0), "espAuthAndCipherTransform": SimTypePointer(SimStruct({"authTransform": SimStruct({"authTransformId": SimStruct({"authType": SimTypeInt(signed=False, label="IPSEC_AUTH_TYPE"), "authConfig": SimTypeChar(label="Byte")}, name="IPSEC_AUTH_TRANSFORM_ID0", pack=False, align=None), "cryptoModuleId": SimTypePointer(SimTypeBottom(label="Guid"), offset=0)}, name="IPSEC_AUTH_TRANSFORM0", pack=False, align=None), "cipherTransform": SimStruct({"cipherTransformId": SimStruct({"cipherType": SimTypeInt(signed=False, label="IPSEC_CIPHER_TYPE"), "cipherConfig": SimTypeChar(label="Byte")}, name="IPSEC_CIPHER_TRANSFORM_ID0", pack=False, align=None), "cryptoModuleId": SimTypePointer(SimTypeBottom(label="Guid"), offset=0)}, name="IPSEC_CIPHER_TRANSFORM0", pack=False, align=None)}, name="IPSEC_AUTH_AND_CIPHER_TRANSFORM0", pack=False, align=None), offset=0), "espAuthFwTransform": SimTypePointer(SimStruct({"authTransformId": SimStruct({"authType": SimTypeInt(signed=False, label="IPSEC_AUTH_TYPE"), "authConfig": SimTypeChar(label="Byte")}, name="IPSEC_AUTH_TRANSFORM_ID0", pack=False, align=None), "cryptoModuleId": SimTypePointer(SimTypeBottom(label="Guid"), offset=0)}, name="IPSEC_AUTH_TRANSFORM0", pack=False, align=None), offset=0)}, name="<anon>", label="None")}, name="IPSEC_SA_TRANSFORM0", pack=False, align=None), offset=0), "pfsGroup": SimTypeInt(signed=False, label="IPSEC_PFS_GROUP")}, name="IPSEC_PROPOSAL0", pack=False, align=None), offset=0), "tunnelEndpoints": SimStruct({"ipVersion": SimTypeInt(signed=False, label="FWP_IP_VERSION"), "Anonymous1": SimUnion({"localV4Address": SimTypeInt(signed=False, label="UInt32"), "localV6Address": SimTypeFixedSizeArray(SimTypeChar(label="Byte"), 16)}, name="<anon>", label="None"), "Anonymous2": SimUnion({"remoteV4Address": SimTypeInt(signed=False, label="UInt32"), "remoteV6Address": SimTypeFixedSizeArray(SimTypeChar(label="Byte"), 16)}, name="<anon>", label="None")}, name="IPSEC_TUNNEL_ENDPOINTS0", pack=False, align=None), "saIdleTimeout": SimStruct({"idleTimeoutSeconds": SimTypeInt(signed=False, label="UInt32"), "idleTimeoutSecondsFailOver": SimTypeInt(signed=False, label="UInt32")}, name="IPSEC_SA_IDLE_TIMEOUT0", pack=False, align=None), "emPolicy": SimTypePointer(SimStruct({"numAuthenticationMethods": SimTypeInt(signed=False, label="UInt32"), "authenticationMethods": SimTypePointer(SimStruct({"authenticationMethodType": SimTypeInt(signed=False, label="IKEEXT_AUTHENTICATION_METHOD_TYPE"), "Anonymous": SimUnion({"presharedKeyAuthentication": SimStruct({"presharedKey": SimStruct({"size": SimTypeInt(signed=False, label="UInt32"), "data": SimTypePointer(SimTypeChar(label="Byte"), offset=0)}, name="FWP_BYTE_BLOB", pack=False, align=None)}, name="IKEEXT_PRESHARED_KEY_AUTHENTICATION0", pack=False, align=None), "certificateAuthentication": SimStruct({"inboundConfigType": SimTypeInt(signed=False, label="IKEEXT_CERT_CONFIG_TYPE"), "Anonymous1": SimUnion({"Anonymous": SimStruct({"inboundRootArraySize": SimTypeInt(signed=False, label="UInt32"), "inboundRootArray": SimTypePointer(SimStruct({"certData": SimStruct({"size": SimTypeInt(signed=False, label="UInt32"), "data": SimTypePointer(SimTypeChar(label="Byte"), offset=0)}, name="FWP_BYTE_BLOB", pack=False, align=None), "flags": SimTypeInt(signed=False, label="IKEEXT_CERT_FLAGS")}, name="IKEEXT_CERT_ROOT_CONFIG0", pack=False, align=None), offset=0)}, name="_Anonymous_e__Struct", pack=False, align=None), "inboundEnterpriseStoreConfig": SimTypePointer(SimStruct({"certData": SimStruct({"size": SimTypeInt(signed=False, label="UInt32"), "data": SimTypePointer(SimTypeChar(label="Byte"), offset=0)}, name="FWP_BYTE_BLOB", pack=False, align=None), "flags": SimTypeInt(signed=False, label="IKEEXT_CERT_FLAGS")}, name="IKEEXT_CERT_ROOT_CONFIG0", pack=False, align=None), offset=0), "inboundTrustedRootStoreConfig": SimTypePointer(SimStruct({"certData": SimStruct({"size": SimTypeInt(signed=False, label="UInt32"), "data": SimTypePointer(SimTypeChar(label="Byte"), offset=0)}, name="FWP_BYTE_BLOB", pack=False, align=None), "flags": SimTypeInt(signed=False, label="IKEEXT_CERT_FLAGS")}, name="IKEEXT_CERT_ROOT_CONFIG0", pack=False, align=None), offset=0)}, name="<anon>", label="None"), "outboundConfigType": SimTypeInt(signed=False, label="IKEEXT_CERT_CONFIG_TYPE"), "Anonymous2": SimUnion({"Anonymous": SimStruct({"outboundRootArraySize": SimTypeInt(signed=False, label="UInt32"), "outboundRootArray": SimTypePointer(SimStruct({"certData": SimStruct({"size": SimTypeInt(signed=False, label="UInt32"), "data": SimTypePointer(SimTypeChar(label="Byte"), offset=0)}, name="FWP_BYTE_BLOB", pack=False, align=None), "flags": SimTypeInt(signed=False, label="IKEEXT_CERT_FLAGS")}, name="IKEEXT_CERT_ROOT_CONFIG0", pack=False, align=None), offset=0)}, name="_Anonymous_e__Struct", pack=False, align=None), "outboundEnterpriseStoreConfig": SimTypePointer(SimStruct({"certData": SimStruct({"size": SimTypeInt(signed=False, label="UInt32"), "data": SimTypePointer(SimTypeChar(label="Byte"), offset=0)}, name="FWP_BYTE_BLOB", pack=False, align=None), "flags": SimTypeInt(signed=False, label="IKEEXT_CERT_FLAGS")}, name="IKEEXT_CERT_ROOT_CONFIG0", pack=False, align=None), offset=0), "outboundTrustedRootStoreConfig": SimTypePointer(SimStruct({"certData": SimStruct({"size": SimTypeInt(signed=False, label="UInt32"), "data": SimTypePointer(SimTypeChar(label="Byte"), offset=0)}, name="FWP_BYTE_BLOB", pack=False, align=None), "flags": SimTypeInt(signed=False, label="IKEEXT_CERT_FLAGS")}, name="IKEEXT_CERT_ROOT_CONFIG0", pack=False, align=None), offset=0)}, name="<anon>", label="None"), "flags": SimTypeInt(signed=False, label="IKEEXT_CERT_AUTH")}, name="IKEEXT_CERTIFICATE_AUTHENTICATION0", pack=False, align=None), "kerberosAuthentication": SimStruct({"flags": SimTypeInt(signed=False, label="IKEEXT_KERBEROS_AUTHENTICATION_FLAGS")}, name="IKEEXT_KERBEROS_AUTHENTICATION0", pack=False, align=None), "ntlmV2Authentication": SimStruct({"flags": SimTypeInt(signed=False, label="UInt32")}, name="IKEEXT_NTLM_V2_AUTHENTICATION0", pack=False, align=None), "sslAuthentication": SimStruct({"inboundConfigType": SimTypeInt(signed=False, label="IKEEXT_CERT_CONFIG_TYPE"), "Anonymous1": SimUnion({"Anonymous": SimStruct({"inboundRootArraySize": SimTypeInt(signed=False, label="UInt32"), "inboundRootArray": SimTypePointer(SimStruct({"certData": SimStruct({"size": SimTypeInt(signed=False, label="UInt32"), "data": SimTypePointer(SimTypeChar(label="Byte"), offset=0)}, name="FWP_BYTE_BLOB", pack=False, align=None), "flags": SimTypeInt(signed=False, label="IKEEXT_CERT_FLAGS")}, name="IKEEXT_CERT_ROOT_CONFIG0", pack=False, align=None), offset=0)}, name="_Anonymous_e__Struct", pack=False, align=None), "inboundEnterpriseStoreConfig": SimTypePointer(SimStruct({"certData": SimStruct({"size": SimTypeInt(signed=False, label="UInt32"), "data": SimTypePointer(SimTypeChar(label="Byte"), offset=0)}, name="FWP_BYTE_BLOB", pack=False, align=None), "flags": SimTypeInt(signed=False, label="IKEEXT_CERT_FLAGS")}, name="IKEEXT_CERT_ROOT_CONFIG0", pack=False, align=None), offset=0), "inboundTrustedRootStoreConfig": SimTypePointer(SimStruct({"certData": SimStruct({"size": SimTypeInt(signed=False, label="UInt32"), "data": SimTypePointer(SimTypeChar(label="Byte"), offset=0)}, name="FWP_BYTE_BLOB", pack=False, align=None), "flags": SimTypeInt(signed=False, label="IKEEXT_CERT_FLAGS")}, name="IKEEXT_CERT_ROOT_CONFIG0", pack=False, align=None), offset=0)}, name="<anon>", label="None"), "outboundConfigType": SimTypeInt(signed=False, label="IKEEXT_CERT_CONFIG_TYPE"), "Anonymous2": SimUnion({"Anonymous": SimStruct({"outboundRootArraySize": SimTypeInt(signed=False, label="UInt32"), "outboundRootArray": SimTypePointer(SimStruct({"certData": SimStruct({"size": SimTypeInt(signed=False, label="UInt32"), "data": SimTypePointer(SimTypeChar(label="Byte"), offset=0)}, name="FWP_BYTE_BLOB", pack=False, align=None), "flags": SimTypeInt(signed=False, label="IKEEXT_CERT_FLAGS")}, name="IKEEXT_CERT_ROOT_CONFIG0", pack=False, align=None), offset=0)}, name="_Anonymous_e__Struct", pack=False, align=None), "outboundEnterpriseStoreConfig": SimTypePointer(SimStruct({"certData": SimStruct({"size": SimTypeInt(signed=False, label="UInt32"), "data": SimTypePointer(SimTypeChar(label="Byte"), offset=0)}, name="FWP_BYTE_BLOB", pack=False, align=None), "flags": SimTypeInt(signed=False, label="IKEEXT_CERT_FLAGS")}, name="IKEEXT_CERT_ROOT_CONFIG0", pack=False, align=None), offset=0), "outboundTrustedRootStoreConfig": SimTypePointer(SimStruct({"certData": SimStruct({"size": SimTypeInt(signed=False, label="UInt32"), "data": SimTypePointer(SimTypeChar(label="Byte"), offset=0)}, name="FWP_BYTE_BLOB", pack=False, align=None), "flags": SimTypeInt(signed=False, label="IKEEXT_CERT_FLAGS")}, name="IKEEXT_CERT_ROOT_CONFIG0", pack=False, align=None), offset=0)}, name="<anon>", label="None"), "flags": SimTypeInt(signed=False, label="IKEEXT_CERT_AUTH")}, name="IKEEXT_CERTIFICATE_AUTHENTICATION0", pack=False, align=None), "cgaAuthentication": SimStruct({"keyContainerName": SimTypePointer(SimTypeChar(label="Char"), offset=0), "cspName": SimTypePointer(SimTypeChar(label="Char"), offset=0), "cspType": SimTypeInt(signed=False, label="UInt32"), "cgaModifier": SimStruct({"byteArray16": SimTypeFixedSizeArray(SimTypeChar(label="Byte"), 16)}, name="FWP_BYTE_ARRAY16", pack=False, align=None), "cgaCollisionCount": SimTypeChar(label="Byte")}, name="IKEEXT_IPV6_CGA_AUTHENTICATION0", pack=False, align=None)}, name="<anon>", label="None")}, name="IKEEXT_AUTHENTICATION_METHOD0", pack=False, align=None), offset=0), "initiatorImpersonationType": SimTypeInt(signed=False, label="IKEEXT_AUTHENTICATION_IMPERSONATION_TYPE")}, name="IKEEXT_EM_POLICY0", pack=False, align=None), offset=0)}, name="IPSEC_TUNNEL_POLICY0", pack=False, align=None), offset=0), "ikeMmPolicy": SimTypePointer(SimStruct({"softExpirationTime": SimTypeInt(signed=False, label="UInt32"), "numAuthenticationMethods": SimTypeInt(signed=False, label="UInt32"), "authenticationMethods": SimTypePointer(SimStruct({"authenticationMethodType": SimTypeInt(signed=False, label="IKEEXT_AUTHENTICATION_METHOD_TYPE"), "Anonymous": SimUnion({"presharedKeyAuthentication": SimStruct({"presharedKey": SimStruct({"size": SimTypeInt(signed=False, label="UInt32"), "data": SimTypePointer(SimTypeChar(label="Byte"), offset=0)}, name="FWP_BYTE_BLOB", pack=False, align=None)}, name="IKEEXT_PRESHARED_KEY_AUTHENTICATION0", pack=False, align=None), "certificateAuthentication": SimStruct({"inboundConfigType": SimTypeInt(signed=False, label="IKEEXT_CERT_CONFIG_TYPE"), "Anonymous1": SimUnion({"Anonymous": SimStruct({"inboundRootArraySize": SimTypeInt(signed=False, label="UInt32"), "inboundRootArray": SimTypePointer(SimStruct({"certData": SimStruct({"size": SimTypeInt(signed=False, label="UInt32"), "data": SimTypePointer(SimTypeChar(label="Byte"), offset=0)}, name="FWP_BYTE_BLOB", pack=False, align=None), "flags": SimTypeInt(signed=False, label="IKEEXT_CERT_FLAGS")}, name="IKEEXT_CERT_ROOT_CONFIG0", pack=False, align=None), offset=0)}, name="_Anonymous_e__Struct", pack=False, align=None), "inboundEnterpriseStoreConfig": SimTypePointer(SimStruct({"certData": SimStruct({"size": SimTypeInt(signed=False, label="UInt32"), "data": SimTypePointer(SimTypeChar(label="Byte"), offset=0)}, name="FWP_BYTE_BLOB", pack=False, align=None), "flags": SimTypeInt(signed=False, label="IKEEXT_CERT_FLAGS")}, name="IKEEXT_CERT_ROOT_CONFIG0", pack=False, align=None), offset=0), "inboundTrustedRootStoreConfig": SimTypePointer(SimStruct({"certData": SimStruct({"size": SimTypeInt(signed=False, label="UInt32"), "data": SimTypePointer(SimTypeChar(label="Byte"), offset=0)}, name="FWP_BYTE_BLOB", pack=False, align=None), "flags": SimTypeInt(signed=False, label="IKEEXT_CERT_FLAGS")}, name="IKEEXT_CERT_ROOT_CONFIG0", pack=False, align=None), offset=0)}, name="<anon>", label="None"), "outboundConfigType": SimTypeInt(signed=False, label="IKEEXT_CERT_CONFIG_TYPE"), "Anonymous2": SimUnion({"Anonymous": SimStruct({"outboundRootArraySize": SimTypeInt(signed=False, label="UInt32"), "outboundRootArray": SimTypePointer(SimStruct({"certData": SimStruct({"size": SimTypeInt(signed=False, label="UInt32"), "data": SimTypePointer(SimTypeChar(label="Byte"), offset=0)}, name="FWP_BYTE_BLOB", pack=False, align=None), "flags": SimTypeInt(signed=False, label="IKEEXT_CERT_FLAGS")}, name="IKEEXT_CERT_ROOT_CONFIG0", pack=False, align=None), offset=0)}, name="_Anonymous_e__Struct", pack=False, align=None), "outboundEnterpriseStoreConfig": SimTypePointer(SimStruct({"certData": SimStruct({"size": SimTypeInt(signed=False, label="UInt32"), "data": SimTypePointer(SimTypeChar(label="Byte"), offset=0)}, name="FWP_BYTE_BLOB", pack=False, align=None), "flags": SimTypeInt(signed=False, label="IKEEXT_CERT_FLAGS")}, name="IKEEXT_CERT_ROOT_CONFIG0", pack=False, align=None), offset=0), "outboundTrustedRootStoreConfig": SimTypePointer(SimStruct({"certData": SimStruct({"size": SimTypeInt(signed=False, label="UInt32"), "data": SimTypePointer(SimTypeChar(label="Byte"), offset=0)}, name="FWP_BYTE_BLOB", pack=False, align=None), "flags": SimTypeInt(signed=False, label="IKEEXT_CERT_FLAGS")}, name="IKEEXT_CERT_ROOT_CONFIG0", pack=False, align=None), offset=0)}, name="<anon>", label="None"), "flags": SimTypeInt(signed=False, label="IKEEXT_CERT_AUTH")}, name="IKEEXT_CERTIFICATE_AUTHENTICATION0", pack=False, align=None), "kerberosAuthentication": SimStruct({"flags": SimTypeInt(signed=False, label="IKEEXT_KERBEROS_AUTHENTICATION_FLAGS")}, name="IKEEXT_KERBEROS_AUTHENTICATION0", pack=False, align=None), "ntlmV2Authentication": SimStruct({"flags": SimTypeInt(signed=False, label="UInt32")}, name="IKEEXT_NTLM_V2_AUTHENTICATION0", pack=False, align=None), "sslAuthentication": SimStruct({"inboundConfigType": SimTypeInt(signed=False, label="IKEEXT_CERT_CONFIG_TYPE"), "Anonymous1": SimUnion({"Anonymous": SimStruct({"inboundRootArraySize": SimTypeInt(signed=False, label="UInt32"), "inboundRootArray": SimTypePointer(SimStruct({"certData": SimStruct({"size": SimTypeInt(signed=False, label="UInt32"), "data": SimTypePointer(SimTypeChar(label="Byte"), offset=0)}, name="FWP_BYTE_BLOB", pack=False, align=None), "flags": SimTypeInt(signed=False, label="IKEEXT_CERT_FLAGS")}, name="IKEEXT_CERT_ROOT_CONFIG0", pack=False, align=None), offset=0)}, name="_Anonymous_e__Struct", pack=False, align=None), "inboundEnterpriseStoreConfig": SimTypePointer(SimStruct({"certData": SimStruct({"size": SimTypeInt(signed=False, label="UInt32"), "data": SimTypePointer(SimTypeChar(label="Byte"), offset=0)}, name="FWP_BYTE_BLOB", pack=False, align=None), "flags": SimTypeInt(signed=False, label="IKEEXT_CERT_FLAGS")}, name="IKEEXT_CERT_ROOT_CONFIG0", pack=False, align=None), offset=0), "inboundTrustedRootStoreConfig": SimTypePointer(SimStruct({"certData": SimStruct({"size": SimTypeInt(signed=False, label="UInt32"), "data": SimTypePointer(SimTypeChar(label="Byte"), offset=0)}, name="FWP_BYTE_BLOB", pack=False, align=None), "flags": SimTypeInt(signed=False, label="IKEEXT_CERT_FLAGS")}, name="IKEEXT_CERT_ROOT_CONFIG0", pack=False, align=None), offset=0)}, name="<anon>", label="None"), "outboundConfigType": SimTypeInt(signed=False, label="IKEEXT_CERT_CONFIG_TYPE"), "Anonymous2": SimUnion({"Anonymous": SimStruct({"outboundRootArraySize": SimTypeInt(signed=False, label="UInt32"), "outboundRootArray": SimTypePointer(SimStruct({"certData": SimStruct({"size": SimTypeInt(signed=False, label="UInt32"), "data": SimTypePointer(SimTypeChar(label="Byte"), offset=0)}, name="FWP_BYTE_BLOB", pack=False, align=None), "flags": SimTypeInt(signed=False, label="IKEEXT_CERT_FLAGS")}, name="IKEEXT_CERT_ROOT_CONFIG0", pack=False, align=None), offset=0)}, name="_Anonymous_e__Struct", pack=False, align=None), "outboundEnterpriseStoreConfig": SimTypePointer(SimStruct({"certData": SimStruct({"size": SimTypeInt(signed=False, label="UInt32"), "data": SimTypePointer(SimTypeChar(label="Byte"), offset=0)}, name="FWP_BYTE_BLOB", pack=False, align=None), "flags": SimTypeInt(signed=False, label="IKEEXT_CERT_FLAGS")}, name="IKEEXT_CERT_ROOT_CONFIG0", pack=False, align=None), offset=0), "outboundTrustedRootStoreConfig": SimTypePointer(SimStruct({"certData": SimStruct({"size": SimTypeInt(signed=False, label="UInt32"), "data": SimTypePointer(SimTypeChar(label="Byte"), offset=0)}, name="FWP_BYTE_BLOB", pack=False, align=None), "flags": SimTypeInt(signed=False, label="IKEEXT_CERT_FLAGS")}, name="IKEEXT_CERT_ROOT_CONFIG0", pack=False, align=None), offset=0)}, name="<anon>", label="None"), "flags": SimTypeInt(signed=False, label="IKEEXT_CERT_AUTH")}, name="IKEEXT_CERTIFICATE_AUTHENTICATION0", pack=False, align=None), "cgaAuthentication": SimStruct({"keyContainerName": SimTypePointer(SimTypeChar(label="Char"), offset=0), "cspName": SimTypePointer(SimTypeChar(label="Char"), offset=0), "cspType": SimTypeInt(signed=False, label="UInt32"), "cgaModifier": SimStruct({"byteArray16": SimTypeFixedSizeArray(SimTypeChar(label="Byte"), 16)}, name="FWP_BYTE_ARRAY16", pack=False, align=None), "cgaCollisionCount": SimTypeChar(label="Byte")}, name="IKEEXT_IPV6_CGA_AUTHENTICATION0", pack=False, align=None)}, name="<anon>", label="None")}, name="IKEEXT_AUTHENTICATION_METHOD0", pack=False, align=None), offset=0), "initiatorImpersonationType": SimTypeInt(signed=False, label="IKEEXT_AUTHENTICATION_IMPERSONATION_TYPE"), "numIkeProposals": SimTypeInt(signed=False, label="UInt32"), "ikeProposals": SimTypePointer(SimStruct({"cipherAlgorithm": SimStruct({"algoIdentifier": SimTypeInt(signed=False, label="IKEEXT_CIPHER_TYPE"), "keyLen": SimTypeInt(signed=False, label="UInt32"), "rounds": SimTypeInt(signed=False, label="UInt32")}, name="IKEEXT_CIPHER_ALGORITHM0", pack=False, align=None), "integrityAlgorithm": SimStruct({"algoIdentifier": SimTypeInt(signed=False, label="IKEEXT_INTEGRITY_TYPE")}, name="IKEEXT_INTEGRITY_ALGORITHM0", pack=False, align=None), "maxLifetimeSeconds": SimTypeInt(signed=False, label="UInt32"), "dhGroup": SimTypeInt(signed=False, label="IKEEXT_DH_GROUP"), "quickModeLimit": SimTypeInt(signed=False, label="UInt32")}, name="IKEEXT_PROPOSAL0", pack=False, align=None), offset=0), "flags": SimTypeInt(signed=False, label="IKEEXT_POLICY_FLAG"), "maxDynamicFilters": SimTypeInt(signed=False, label="UInt32")}, name="IKEEXT_POLICY0", pack=False, align=None), offset=0), "authIpMmPolicy": SimTypePointer(SimStruct({"softExpirationTime": SimTypeInt(signed=False, label="UInt32"), "numAuthenticationMethods": SimTypeInt(signed=False, label="UInt32"), "authenticationMethods": SimTypePointer(SimStruct({"authenticationMethodType": SimTypeInt(signed=False, label="IKEEXT_AUTHENTICATION_METHOD_TYPE"), "Anonymous": SimUnion({"presharedKeyAuthentication": SimStruct({"presharedKey": SimStruct({"size": SimTypeInt(signed=False, label="UInt32"), "data": SimTypePointer(SimTypeChar(label="Byte"), offset=0)}, name="FWP_BYTE_BLOB", pack=False, align=None)}, name="IKEEXT_PRESHARED_KEY_AUTHENTICATION0", pack=False, align=None), "certificateAuthentication": SimStruct({"inboundConfigType": SimTypeInt(signed=False, label="IKEEXT_CERT_CONFIG_TYPE"), "Anonymous1": SimUnion({"Anonymous": SimStruct({"inboundRootArraySize": SimTypeInt(signed=False, label="UInt32"), "inboundRootArray": SimTypePointer(SimStruct({"certData": SimStruct({"size": SimTypeInt(signed=False, label="UInt32"), "data": SimTypePointer(SimTypeChar(label="Byte"), offset=0)}, name="FWP_BYTE_BLOB", pack=False, align=None), "flags": SimTypeInt(signed=False, label="IKEEXT_CERT_FLAGS")}, name="IKEEXT_CERT_ROOT_CONFIG0", pack=False, align=None), offset=0)}, name="_Anonymous_e__Struct", pack=False, align=None), "inboundEnterpriseStoreConfig": SimTypePointer(SimStruct({"certData": SimStruct({"size": SimTypeInt(signed=False, label="UInt32"), "data": SimTypePointer(SimTypeChar(label="Byte"), offset=0)}, name="FWP_BYTE_BLOB", pack=False, align=None), "flags": SimTypeInt(signed=False, label="IKEEXT_CERT_FLAGS")}, name="IKEEXT_CERT_ROOT_CONFIG0", pack=False, align=None), offset=0), "inboundTrustedRootStoreConfig": SimTypePointer(SimStruct({"certData": SimStruct({"size": SimTypeInt(signed=False, label="UInt32"), "data": SimTypePointer(SimTypeChar(label="Byte"), offset=0)}, name="FWP_BYTE_BLOB", pack=False, align=None), "flags": SimTypeInt(signed=False, label="IKEEXT_CERT_FLAGS")}, name="IKEEXT_CERT_ROOT_CONFIG0", pack=False, align=None), offset=0)}, name="<anon>", label="None"), "outboundConfigType": SimTypeInt(signed=False, label="IKEEXT_CERT_CONFIG_TYPE"), "Anonymous2": SimUnion({"Anonymous": SimStruct({"outboundRootArraySize": SimTypeInt(signed=False, label="UInt32"), "outboundRootArray": SimTypePointer(SimStruct({"certData": SimStruct({"size": SimTypeInt(signed=False, label="UInt32"), "data": SimTypePointer(SimTypeChar(label="Byte"), offset=0)}, name="FWP_BYTE_BLOB", pack=False, align=None), "flags": SimTypeInt(signed=False, label="IKEEXT_CERT_FLAGS")}, name="IKEEXT_CERT_ROOT_CONFIG0", pack=False, align=None), offset=0)}, name="_Anonymous_e__Struct", pack=False, align=None), "outboundEnterpriseStoreConfig": SimTypePointer(SimStruct({"certData": SimStruct({"size": SimTypeInt(signed=False, label="UInt32"), "data": SimTypePointer(SimTypeChar(label="Byte"), offset=0)}, name="FWP_BYTE_BLOB", pack=False, align=None), "flags": SimTypeInt(signed=False, label="IKEEXT_CERT_FLAGS")}, name="IKEEXT_CERT_ROOT_CONFIG0", pack=False, align=None), offset=0), "outboundTrustedRootStoreConfig": SimTypePointer(SimStruct({"certData": SimStruct({"size": SimTypeInt(signed=False, label="UInt32"), "data": SimTypePointer(SimTypeChar(label="Byte"), offset=0)}, name="FWP_BYTE_BLOB", pack=False, align=None), "flags": SimTypeInt(signed=False, label="IKEEXT_CERT_FLAGS")}, name="IKEEXT_CERT_ROOT_CONFIG0", pack=False, align=None), offset=0)}, name="<anon>", label="None"), "flags": SimTypeInt(signed=False, label="IKEEXT_CERT_AUTH")}, name="IKEEXT_CERTIFICATE_AUTHENTICATION0", pack=False, align=None), "kerberosAuthentication": SimStruct({"flags": SimTypeInt(signed=False, label="IKEEXT_KERBEROS_AUTHENTICATION_FLAGS")}, name="IKEEXT_KERBEROS_AUTHENTICATION0", pack=False, align=None), "ntlmV2Authentication": SimStruct({"flags": SimTypeInt(signed=False, label="UInt32")}, name="IKEEXT_NTLM_V2_AUTHENTICATION0", pack=False, align=None), "sslAuthentication": SimStruct({"inboundConfigType": SimTypeInt(signed=False, label="IKEEXT_CERT_CONFIG_TYPE"), "Anonymous1": SimUnion({"Anonymous": SimStruct({"inboundRootArraySize": SimTypeInt(signed=False, label="UInt32"), "inboundRootArray": SimTypePointer(SimStruct({"certData": SimStruct({"size": SimTypeInt(signed=False, label="UInt32"), "data": SimTypePointer(SimTypeChar(label="Byte"), offset=0)}, name="FWP_BYTE_BLOB", pack=False, align=None), "flags": SimTypeInt(signed=False, label="IKEEXT_CERT_FLAGS")}, name="IKEEXT_CERT_ROOT_CONFIG0", pack=False, align=None), offset=0)}, name="_Anonymous_e__Struct", pack=False, align=None), "inboundEnterpriseStoreConfig": SimTypePointer(SimStruct({"certData": SimStruct({"size": SimTypeInt(signed=False, label="UInt32"), "data": SimTypePointer(SimTypeChar(label="Byte"), offset=0)}, name="FWP_BYTE_BLOB", pack=False, align=None), "flags": SimTypeInt(signed=False, label="IKEEXT_CERT_FLAGS")}, name="IKEEXT_CERT_ROOT_CONFIG0", pack=False, align=None), offset=0), "inboundTrustedRootStoreConfig": SimTypePointer(SimStruct({"certData": SimStruct({"size": SimTypeInt(signed=False, label="UInt32"), "data": SimTypePointer(SimTypeChar(label="Byte"), offset=0)}, name="FWP_BYTE_BLOB", pack=False, align=None), "flags": SimTypeInt(signed=False, label="IKEEXT_CERT_FLAGS")}, name="IKEEXT_CERT_ROOT_CONFIG0", pack=False, align=None), offset=0)}, name="<anon>", label="None"), "outboundConfigType": SimTypeInt(signed=False, label="IKEEXT_CERT_CONFIG_TYPE"), "Anonymous2": SimUnion({"Anonymous": SimStruct({"outboundRootArraySize": SimTypeInt(signed=False, label="UInt32"), "outboundRootArray": SimTypePointer(SimStruct({"certData": SimStruct({"size": SimTypeInt(signed=False, label="UInt32"), "data": SimTypePointer(SimTypeChar(label="Byte"), offset=0)}, name="FWP_BYTE_BLOB", pack=False, align=None), "flags": SimTypeInt(signed=False, label="IKEEXT_CERT_FLAGS")}, name="IKEEXT_CERT_ROOT_CONFIG0", pack=False, align=None), offset=0)}, name="_Anonymous_e__Struct", pack=False, align=None), "outboundEnterpriseStoreConfig": SimTypePointer(SimStruct({"certData": SimStruct({"size": SimTypeInt(signed=False, label="UInt32"), "data": SimTypePointer(SimTypeChar(label="Byte"), offset=0)}, name="FWP_BYTE_BLOB", pack=False, align=None), "flags": SimTypeInt(signed=False, label="IKEEXT_CERT_FLAGS")}, name="IKEEXT_CERT_ROOT_CONFIG0", pack=False, align=None), offset=0), "outboundTrustedRootStoreConfig": SimTypePointer(SimStruct({"certData": SimStruct({"size": SimTypeInt(signed=False, label="UInt32"), "data": SimTypePointer(SimTypeChar(label="Byte"), offset=0)}, name="FWP_BYTE_BLOB", pack=False, align=None), "flags": SimTypeInt(signed=False, label="IKEEXT_CERT_FLAGS")}, name="IKEEXT_CERT_ROOT_CONFIG0", pack=False, align=None), offset=0)}, name="<anon>", label="None"), "flags": SimTypeInt(signed=False, label="IKEEXT_CERT_AUTH")}, name="IKEEXT_CERTIFICATE_AUTHENTICATION0", pack=False, align=None), "cgaAuthentication": SimStruct({"keyContainerName": SimTypePointer(SimTypeChar(label="Char"), offset=0), "cspName": SimTypePointer(SimTypeChar(label="Char"), offset=0), "cspType": SimTypeInt(signed=False, label="UInt32"), "cgaModifier": SimStruct({"byteArray16": SimTypeFixedSizeArray(SimTypeChar(label="Byte"), 16)}, name="FWP_BYTE_ARRAY16", pack=False, align=None), "cgaCollisionCount": SimTypeChar(label="Byte")}, name="IKEEXT_IPV6_CGA_AUTHENTICATION0", pack=False, align=None)}, name="<anon>", label="None")}, name="IKEEXT_AUTHENTICATION_METHOD0", pack=False, align=None), offset=0), "initiatorImpersonationType": SimTypeInt(signed=False, label="IKEEXT_AUTHENTICATION_IMPERSONATION_TYPE"), "numIkeProposals": SimTypeInt(signed=False, label="UInt32"), "ikeProposals": SimTypePointer(SimStruct({"cipherAlgorithm": SimStruct({"algoIdentifier": SimTypeInt(signed=False, label="IKEEXT_CIPHER_TYPE"), "keyLen": SimTypeInt(signed=False, label="UInt32"), "rounds": SimTypeInt(signed=False, label="UInt32")}, name="IKEEXT_CIPHER_ALGORITHM0", pack=False, align=None), "integrityAlgorithm": SimStruct({"algoIdentifier": SimTypeInt(signed=False, label="IKEEXT_INTEGRITY_TYPE")}, name="IKEEXT_INTEGRITY_ALGORITHM0", pack=False, align=None), "maxLifetimeSeconds": SimTypeInt(signed=False, label="UInt32"), "dhGroup": SimTypeInt(signed=False, label="IKEEXT_DH_GROUP"), "quickModeLimit": SimTypeInt(signed=False, label="UInt32")}, name="IKEEXT_PROPOSAL0", pack=False, align=None), offset=0), "flags": SimTypeInt(signed=False, label="IKEEXT_POLICY_FLAG"), "maxDynamicFilters": SimTypeInt(signed=False, label="UInt32")}, name="IKEEXT_POLICY0", pack=False, align=None), offset=0), "dataBuffer": SimTypePointer(SimStruct({"size": SimTypeInt(signed=False, label="UInt32"), "data": SimTypePointer(SimTypeChar(label="Byte"), offset=0)}, name="FWP_BYTE_BLOB", pack=False, align=None), offset=0), "classifyOptions": SimTypePointer(SimStruct({"numOptions": SimTypeInt(signed=False, label="UInt32"), "options": SimTypePointer(SimStruct({"type": SimTypeInt(signed=False, label="FWP_CLASSIFY_OPTION_TYPE"), "value": SimStruct({"type": SimTypeInt(signed=False, label="FWP_DATA_TYPE"), "Anonymous": SimUnion({"uint8": SimTypeChar(label="Byte"), "uint16": SimTypeShort(signed=False, label="UInt16"), "uint32": SimTypeInt(signed=False, label="UInt32"), "uint64": SimTypePointer(SimTypeLongLong(signed=False, label="UInt64"), offset=0), "int8": SimTypeChar(label="SByte"), "int16": SimTypeShort(signed=True, label="Int16"), "int32": SimTypeInt(signed=True, label="Int32"), "int64": SimTypePointer(SimTypeLongLong(signed=True, label="Int64"), offset=0), "float32": SimTypeFloat(size=32), "double64": SimTypePointer(SimTypeFloat(size=64), offset=0), "byteArray16": SimTypePointer(SimStruct({"byteArray16": SimTypeFixedSizeArray(SimTypeChar(label="Byte"), 16)}, name="FWP_BYTE_ARRAY16", pack=False, align=None), offset=0), "byteBlob": SimTypePointer(SimStruct({"size": SimTypeInt(signed=False, label="UInt32"), "data": SimTypePointer(SimTypeChar(label="Byte"), offset=0)}, name="FWP_BYTE_BLOB", pack=False, align=None), offset=0), "sid": SimTypePointer(SimTypeBottom(label="SID"), offset=0), "sd": SimTypePointer(SimStruct({"size": SimTypeInt(signed=False, label="UInt32"), "data": SimTypePointer(SimTypeChar(label="Byte"), offset=0)}, name="FWP_BYTE_BLOB", pack=False, align=None), offset=0), "tokenInformation": SimTypePointer(SimStruct({"sidCount": SimTypeInt(signed=False, label="UInt32"), "sids": SimTypePointer(SimTypeBottom(label="SID_AND_ATTRIBUTES"), offset=0), "restrictedSidCount": SimTypeInt(signed=False, label="UInt32"), "restrictedSids": SimTypePointer(SimTypeBottom(label="SID_AND_ATTRIBUTES"), offset=0)}, name="FWP_TOKEN_INFORMATION", pack=False, align=None), offset=0), "tokenAccessInformation": SimTypePointer(SimStruct({"size": SimTypeInt(signed=False, label="UInt32"), "data": SimTypePointer(SimTypeChar(label="Byte"), offset=0)}, name="FWP_BYTE_BLOB", pack=False, align=None), offset=0), "unicodeString": SimTypePointer(SimTypeChar(label="Char"), offset=0), "byteArray6": SimTypePointer(SimStruct({"byteArray6": SimTypeFixedSizeArray(SimTypeChar(label="Byte"), 6)}, name="FWP_BYTE_ARRAY6", pack=False, align=None), offset=0), "bitmapArray64": SimTypePointer(SimStruct({"bitmapArray64": SimTypeFixedSizeArray(SimTypeChar(label="Byte"), 8)}, name="FWP_BITMAP_ARRAY64_", pack=False, align=None), offset=0)}, name="<anon>", label="None")}, name="FWP_VALUE0", pack=False, align=None)}, name="FWPM_CLASSIFY_OPTION0", pack=False, align=None), offset=0)}, name="FWPM_CLASSIFY_OPTIONS0", pack=False, align=None), offset=0)}, name="<anon>", label="None"), "providerContextId": SimTypeLongLong(signed=False, label="UInt64")}, name="FWPM_PROVIDER_CONTEXT0", pack=False, align=None), offset=0), SimTypePointer(SimStruct({"Revision": SimTypeChar(label="Byte"), "Sbz1": SimTypeChar(label="Byte"), "Control": SimTypeShort(signed=False, label="UInt16"), "Owner": SimTypePointer(SimTypeInt(signed=True, label="Int"), label="IntPtr", offset=0), "Group": SimTypePointer(SimTypeInt(signed=True, label="Int"), label="IntPtr", offset=0), "Sacl": SimTypePointer(SimStruct({"AclRevision": SimTypeChar(label="Byte"), "Sbz1": SimTypeChar(label="Byte"), "AclSize": SimTypeShort(signed=False, label="UInt16"), "AceCount": SimTypeShort(signed=False, label="UInt16"), "Sbz2": SimTypeShort(signed=False, label="UInt16")}, name="ACL", pack=False, align=None), offset=0), "Dacl": SimTypePointer(SimStruct({"AclRevision": SimTypeChar(label="Byte"), "Sbz1": SimTypeChar(label="Byte"), "AclSize": SimTypeShort(signed=False, label="UInt16"), "AceCount": SimTypeShort(signed=False, label="UInt16"), "Sbz2": SimTypeShort(signed=False, label="UInt16")}, name="ACL", pack=False, align=None), offset=0)}, name="SECURITY_DESCRIPTOR", pack=False, align=None), offset=0), SimTypePointer(SimTypeLongLong(signed=False, label="UInt64"), offset=0)], SimTypeInt(signed=False, label="UInt32"), arg_names=["engineHandle", "providerContext", "sd", "id"]),
        # 
        'FwpmProviderContextAdd1': SimTypeFunction([SimTypePointer(SimTypeInt(signed=True, label="Int"), label="IntPtr", offset=0), SimTypePointer(SimStruct({"providerContextKey": SimTypeBottom(label="Guid"), "displayData": SimStruct({"name": SimTypePointer(SimTypeChar(label="Char"), offset=0), "description": SimTypePointer(SimTypeChar(label="Char"), offset=0)}, name="FWPM_DISPLAY_DATA0", pack=False, align=None), "flags": SimTypeInt(signed=False, label="UInt32"), "providerKey": SimTypePointer(SimTypeBottom(label="Guid"), offset=0), "providerData": SimStruct({"size": SimTypeInt(signed=False, label="UInt32"), "data": SimTypePointer(SimTypeChar(label="Byte"), offset=0)}, name="FWP_BYTE_BLOB", pack=False, align=None), "type": SimTypeInt(signed=False, label="FWPM_PROVIDER_CONTEXT_TYPE"), "Anonymous": SimUnion({"keyingPolicy": SimTypePointer(SimStruct({"numKeyMods": SimTypeInt(signed=False, label="UInt32"), "keyModKeys": SimTypePointer(SimTypeBottom(label="Guid"), offset=0)}, name="IPSEC_KEYING_POLICY0", pack=False, align=None), offset=0), "ikeQmTransportPolicy": SimTypePointer(SimStruct({"numIpsecProposals": SimTypeInt(signed=False, label="UInt32"), "ipsecProposals": SimTypePointer(SimStruct({"lifetime": SimStruct({"lifetimeSeconds": SimTypeInt(signed=False, label="UInt32"), "lifetimeKilobytes": SimTypeInt(signed=False, label="UInt32"), "lifetimePackets": SimTypeInt(signed=False, label="UInt32")}, name="IPSEC_SA_LIFETIME0", pack=False, align=None), "numSaTransforms": SimTypeInt(signed=False, label="UInt32"), "saTransforms": SimTypePointer(SimStruct({"ipsecTransformType": SimTypeInt(signed=False, label="IPSEC_TRANSFORM_TYPE"), "Anonymous": SimUnion({"ahTransform": SimTypePointer(SimStruct({"authTransformId": SimStruct({"authType": SimTypeInt(signed=False, label="IPSEC_AUTH_TYPE"), "authConfig": SimTypeChar(label="Byte")}, name="IPSEC_AUTH_TRANSFORM_ID0", pack=False, align=None), "cryptoModuleId": SimTypePointer(SimTypeBottom(label="Guid"), offset=0)}, name="IPSEC_AUTH_TRANSFORM0", pack=False, align=None), offset=0), "espAuthTransform": SimTypePointer(SimStruct({"authTransformId": SimStruct({"authType": SimTypeInt(signed=False, label="IPSEC_AUTH_TYPE"), "authConfig": SimTypeChar(label="Byte")}, name="IPSEC_AUTH_TRANSFORM_ID0", pack=False, align=None), "cryptoModuleId": SimTypePointer(SimTypeBottom(label="Guid"), offset=0)}, name="IPSEC_AUTH_TRANSFORM0", pack=False, align=None), offset=0), "espCipherTransform": SimTypePointer(SimStruct({"cipherTransformId": SimStruct({"cipherType": SimTypeInt(signed=False, label="IPSEC_CIPHER_TYPE"), "cipherConfig": SimTypeChar(label="Byte")}, name="IPSEC_CIPHER_TRANSFORM_ID0", pack=False, align=None), "cryptoModuleId": SimTypePointer(SimTypeBottom(label="Guid"), offset=0)}, name="IPSEC_CIPHER_TRANSFORM0", pack=False, align=None), offset=0), "espAuthAndCipherTransform": SimTypePointer(SimStruct({"authTransform": SimStruct({"authTransformId": SimStruct({"authType": SimTypeInt(signed=False, label="IPSEC_AUTH_TYPE"), "authConfig": SimTypeChar(label="Byte")}, name="IPSEC_AUTH_TRANSFORM_ID0", pack=False, align=None), "cryptoModuleId": SimTypePointer(SimTypeBottom(label="Guid"), offset=0)}, name="IPSEC_AUTH_TRANSFORM0", pack=False, align=None), "cipherTransform": SimStruct({"cipherTransformId": SimStruct({"cipherType": SimTypeInt(signed=False, label="IPSEC_CIPHER_TYPE"), "cipherConfig": SimTypeChar(label="Byte")}, name="IPSEC_CIPHER_TRANSFORM_ID0", pack=False, align=None), "cryptoModuleId": SimTypePointer(SimTypeBottom(label="Guid"), offset=0)}, name="IPSEC_CIPHER_TRANSFORM0", pack=False, align=None)}, name="IPSEC_AUTH_AND_CIPHER_TRANSFORM0", pack=False, align=None), offset=0), "espAuthFwTransform": SimTypePointer(SimStruct({"authTransformId": SimStruct({"authType": SimTypeInt(signed=False, label="IPSEC_AUTH_TYPE"), "authConfig": SimTypeChar(label="Byte")}, name="IPSEC_AUTH_TRANSFORM_ID0", pack=False, align=None), "cryptoModuleId": SimTypePointer(SimTypeBottom(label="Guid"), offset=0)}, name="IPSEC_AUTH_TRANSFORM0", pack=False, align=None), offset=0)}, name="<anon>", label="None")}, name="IPSEC_SA_TRANSFORM0", pack=False, align=None), offset=0), "pfsGroup": SimTypeInt(signed=False, label="IPSEC_PFS_GROUP")}, name="IPSEC_PROPOSAL0", pack=False, align=None), offset=0), "flags": SimTypeInt(signed=False, label="IPSEC_POLICY_FLAG"), "ndAllowClearTimeoutSeconds": SimTypeInt(signed=False, label="UInt32"), "saIdleTimeout": SimStruct({"idleTimeoutSeconds": SimTypeInt(signed=False, label="UInt32"), "idleTimeoutSecondsFailOver": SimTypeInt(signed=False, label="UInt32")}, name="IPSEC_SA_IDLE_TIMEOUT0", pack=False, align=None), "emPolicy": SimTypePointer(SimStruct({"numAuthenticationMethods": SimTypeInt(signed=False, label="UInt32"), "authenticationMethods": SimTypePointer(SimStruct({"authenticationMethodType": SimTypeInt(signed=False, label="IKEEXT_AUTHENTICATION_METHOD_TYPE"), "Anonymous": SimUnion({"presharedKeyAuthentication": SimStruct({"presharedKey": SimStruct({"size": SimTypeInt(signed=False, label="UInt32"), "data": SimTypePointer(SimTypeChar(label="Byte"), offset=0)}, name="FWP_BYTE_BLOB", pack=False, align=None), "flags": SimTypeInt(signed=False, label="IKEEXT_PRESHARED_KEY_AUTHENTICATION_FLAGS")}, name="IKEEXT_PRESHARED_KEY_AUTHENTICATION1", pack=False, align=None), "certificateAuthentication": SimStruct({"inboundConfigType": SimTypeInt(signed=False, label="IKEEXT_CERT_CONFIG_TYPE"), "Anonymous1": SimUnion({"Anonymous": SimStruct({"inboundRootArraySize": SimTypeInt(signed=False, label="UInt32"), "inboundRootArray": SimTypePointer(SimStruct({"certData": SimStruct({"size": SimTypeInt(signed=False, label="UInt32"), "data": SimTypePointer(SimTypeChar(label="Byte"), offset=0)}, name="FWP_BYTE_BLOB", pack=False, align=None), "flags": SimTypeInt(signed=False, label="IKEEXT_CERT_FLAGS")}, name="IKEEXT_CERT_ROOT_CONFIG0", pack=False, align=None), offset=0)}, name="_Anonymous_e__Struct", pack=False, align=None), "inboundEnterpriseStoreConfig": SimTypePointer(SimStruct({"certData": SimStruct({"size": SimTypeInt(signed=False, label="UInt32"), "data": SimTypePointer(SimTypeChar(label="Byte"), offset=0)}, name="FWP_BYTE_BLOB", pack=False, align=None), "flags": SimTypeInt(signed=False, label="IKEEXT_CERT_FLAGS")}, name="IKEEXT_CERT_ROOT_CONFIG0", pack=False, align=None), offset=0), "inboundTrustedRootStoreConfig": SimTypePointer(SimStruct({"certData": SimStruct({"size": SimTypeInt(signed=False, label="UInt32"), "data": SimTypePointer(SimTypeChar(label="Byte"), offset=0)}, name="FWP_BYTE_BLOB", pack=False, align=None), "flags": SimTypeInt(signed=False, label="IKEEXT_CERT_FLAGS")}, name="IKEEXT_CERT_ROOT_CONFIG0", pack=False, align=None), offset=0)}, name="<anon>", label="None"), "outboundConfigType": SimTypeInt(signed=False, label="IKEEXT_CERT_CONFIG_TYPE"), "Anonymous2": SimUnion({"Anonymous": SimStruct({"outboundRootArraySize": SimTypeInt(signed=False, label="UInt32"), "outboundRootArray": SimTypePointer(SimStruct({"certData": SimStruct({"size": SimTypeInt(signed=False, label="UInt32"), "data": SimTypePointer(SimTypeChar(label="Byte"), offset=0)}, name="FWP_BYTE_BLOB", pack=False, align=None), "flags": SimTypeInt(signed=False, label="IKEEXT_CERT_FLAGS")}, name="IKEEXT_CERT_ROOT_CONFIG0", pack=False, align=None), offset=0)}, name="_Anonymous_e__Struct", pack=False, align=None), "outboundEnterpriseStoreConfig": SimTypePointer(SimStruct({"certData": SimStruct({"size": SimTypeInt(signed=False, label="UInt32"), "data": SimTypePointer(SimTypeChar(label="Byte"), offset=0)}, name="FWP_BYTE_BLOB", pack=False, align=None), "flags": SimTypeInt(signed=False, label="IKEEXT_CERT_FLAGS")}, name="IKEEXT_CERT_ROOT_CONFIG0", pack=False, align=None), offset=0), "outboundTrustedRootStoreConfig": SimTypePointer(SimStruct({"certData": SimStruct({"size": SimTypeInt(signed=False, label="UInt32"), "data": SimTypePointer(SimTypeChar(label="Byte"), offset=0)}, name="FWP_BYTE_BLOB", pack=False, align=None), "flags": SimTypeInt(signed=False, label="IKEEXT_CERT_FLAGS")}, name="IKEEXT_CERT_ROOT_CONFIG0", pack=False, align=None), offset=0)}, name="<anon>", label="None"), "flags": SimTypeInt(signed=False, label="IKEEXT_CERT_AUTH"), "localCertLocationUrl": SimStruct({"size": SimTypeInt(signed=False, label="UInt32"), "data": SimTypePointer(SimTypeChar(label="Byte"), offset=0)}, name="FWP_BYTE_BLOB", pack=False, align=None)}, name="IKEEXT_CERTIFICATE_AUTHENTICATION1", pack=False, align=None), "kerberosAuthentication": SimStruct({"flags": SimTypeInt(signed=False, label="IKEEXT_KERBEROS_AUTHENTICATION_FLAGS")}, name="IKEEXT_KERBEROS_AUTHENTICATION0", pack=False, align=None), "ntlmV2Authentication": SimStruct({"flags": SimTypeInt(signed=False, label="UInt32")}, name="IKEEXT_NTLM_V2_AUTHENTICATION0", pack=False, align=None), "sslAuthentication": SimStruct({"inboundConfigType": SimTypeInt(signed=False, label="IKEEXT_CERT_CONFIG_TYPE"), "Anonymous1": SimUnion({"Anonymous": SimStruct({"inboundRootArraySize": SimTypeInt(signed=False, label="UInt32"), "inboundRootArray": SimTypePointer(SimStruct({"certData": SimStruct({"size": SimTypeInt(signed=False, label="UInt32"), "data": SimTypePointer(SimTypeChar(label="Byte"), offset=0)}, name="FWP_BYTE_BLOB", pack=False, align=None), "flags": SimTypeInt(signed=False, label="IKEEXT_CERT_FLAGS")}, name="IKEEXT_CERT_ROOT_CONFIG0", pack=False, align=None), offset=0)}, name="_Anonymous_e__Struct", pack=False, align=None), "inboundEnterpriseStoreConfig": SimTypePointer(SimStruct({"certData": SimStruct({"size": SimTypeInt(signed=False, label="UInt32"), "data": SimTypePointer(SimTypeChar(label="Byte"), offset=0)}, name="FWP_BYTE_BLOB", pack=False, align=None), "flags": SimTypeInt(signed=False, label="IKEEXT_CERT_FLAGS")}, name="IKEEXT_CERT_ROOT_CONFIG0", pack=False, align=None), offset=0), "inboundTrustedRootStoreConfig": SimTypePointer(SimStruct({"certData": SimStruct({"size": SimTypeInt(signed=False, label="UInt32"), "data": SimTypePointer(SimTypeChar(label="Byte"), offset=0)}, name="FWP_BYTE_BLOB", pack=False, align=None), "flags": SimTypeInt(signed=False, label="IKEEXT_CERT_FLAGS")}, name="IKEEXT_CERT_ROOT_CONFIG0", pack=False, align=None), offset=0)}, name="<anon>", label="None"), "outboundConfigType": SimTypeInt(signed=False, label="IKEEXT_CERT_CONFIG_TYPE"), "Anonymous2": SimUnion({"Anonymous": SimStruct({"outboundRootArraySize": SimTypeInt(signed=False, label="UInt32"), "outboundRootArray": SimTypePointer(SimStruct({"certData": SimStruct({"size": SimTypeInt(signed=False, label="UInt32"), "data": SimTypePointer(SimTypeChar(label="Byte"), offset=0)}, name="FWP_BYTE_BLOB", pack=False, align=None), "flags": SimTypeInt(signed=False, label="IKEEXT_CERT_FLAGS")}, name="IKEEXT_CERT_ROOT_CONFIG0", pack=False, align=None), offset=0)}, name="_Anonymous_e__Struct", pack=False, align=None), "outboundEnterpriseStoreConfig": SimTypePointer(SimStruct({"certData": SimStruct({"size": SimTypeInt(signed=False, label="UInt32"), "data": SimTypePointer(SimTypeChar(label="Byte"), offset=0)}, name="FWP_BYTE_BLOB", pack=False, align=None), "flags": SimTypeInt(signed=False, label="IKEEXT_CERT_FLAGS")}, name="IKEEXT_CERT_ROOT_CONFIG0", pack=False, align=None), offset=0), "outboundTrustedRootStoreConfig": SimTypePointer(SimStruct({"certData": SimStruct({"size": SimTypeInt(signed=False, label="UInt32"), "data": SimTypePointer(SimTypeChar(label="Byte"), offset=0)}, name="FWP_BYTE_BLOB", pack=False, align=None), "flags": SimTypeInt(signed=False, label="IKEEXT_CERT_FLAGS")}, name="IKEEXT_CERT_ROOT_CONFIG0", pack=False, align=None), offset=0)}, name="<anon>", label="None"), "flags": SimTypeInt(signed=False, label="IKEEXT_CERT_AUTH"), "localCertLocationUrl": SimStruct({"size": SimTypeInt(signed=False, label="UInt32"), "data": SimTypePointer(SimTypeChar(label="Byte"), offset=0)}, name="FWP_BYTE_BLOB", pack=False, align=None)}, name="IKEEXT_CERTIFICATE_AUTHENTICATION1", pack=False, align=None), "cgaAuthentication": SimStruct({"keyContainerName": SimTypePointer(SimTypeChar(label="Char"), offset=0), "cspName": SimTypePointer(SimTypeChar(label="Char"), offset=0), "cspType": SimTypeInt(signed=False, label="UInt32"), "cgaModifier": SimStruct({"byteArray16": SimTypeFixedSizeArray(SimTypeChar(label="Byte"), 16)}, name="FWP_BYTE_ARRAY16", pack=False, align=None), "cgaCollisionCount": SimTypeChar(label="Byte")}, name="IKEEXT_IPV6_CGA_AUTHENTICATION0", pack=False, align=None), "eapAuthentication": SimStruct({"flags": SimTypeInt(signed=False, label="IKEEXT_EAP_AUTHENTICATION_FLAGS")}, name="IKEEXT_EAP_AUTHENTICATION0", pack=False, align=None)}, name="<anon>", label="None")}, name="IKEEXT_AUTHENTICATION_METHOD1", pack=False, align=None), offset=0), "initiatorImpersonationType": SimTypeInt(signed=False, label="IKEEXT_AUTHENTICATION_IMPERSONATION_TYPE")}, name="IKEEXT_EM_POLICY1", pack=False, align=None), offset=0)}, name="IPSEC_TRANSPORT_POLICY1", pack=False, align=None), offset=0), "ikeQmTunnelPolicy": SimTypePointer(SimStruct({"flags": SimTypeInt(signed=False, label="IPSEC_POLICY_FLAG"), "numIpsecProposals": SimTypeInt(signed=False, label="UInt32"), "ipsecProposals": SimTypePointer(SimStruct({"lifetime": SimStruct({"lifetimeSeconds": SimTypeInt(signed=False, label="UInt32"), "lifetimeKilobytes": SimTypeInt(signed=False, label="UInt32"), "lifetimePackets": SimTypeInt(signed=False, label="UInt32")}, name="IPSEC_SA_LIFETIME0", pack=False, align=None), "numSaTransforms": SimTypeInt(signed=False, label="UInt32"), "saTransforms": SimTypePointer(SimStruct({"ipsecTransformType": SimTypeInt(signed=False, label="IPSEC_TRANSFORM_TYPE"), "Anonymous": SimUnion({"ahTransform": SimTypePointer(SimStruct({"authTransformId": SimStruct({"authType": SimTypeInt(signed=False, label="IPSEC_AUTH_TYPE"), "authConfig": SimTypeChar(label="Byte")}, name="IPSEC_AUTH_TRANSFORM_ID0", pack=False, align=None), "cryptoModuleId": SimTypePointer(SimTypeBottom(label="Guid"), offset=0)}, name="IPSEC_AUTH_TRANSFORM0", pack=False, align=None), offset=0), "espAuthTransform": SimTypePointer(SimStruct({"authTransformId": SimStruct({"authType": SimTypeInt(signed=False, label="IPSEC_AUTH_TYPE"), "authConfig": SimTypeChar(label="Byte")}, name="IPSEC_AUTH_TRANSFORM_ID0", pack=False, align=None), "cryptoModuleId": SimTypePointer(SimTypeBottom(label="Guid"), offset=0)}, name="IPSEC_AUTH_TRANSFORM0", pack=False, align=None), offset=0), "espCipherTransform": SimTypePointer(SimStruct({"cipherTransformId": SimStruct({"cipherType": SimTypeInt(signed=False, label="IPSEC_CIPHER_TYPE"), "cipherConfig": SimTypeChar(label="Byte")}, name="IPSEC_CIPHER_TRANSFORM_ID0", pack=False, align=None), "cryptoModuleId": SimTypePointer(SimTypeBottom(label="Guid"), offset=0)}, name="IPSEC_CIPHER_TRANSFORM0", pack=False, align=None), offset=0), "espAuthAndCipherTransform": SimTypePointer(SimStruct({"authTransform": SimStruct({"authTransformId": SimStruct({"authType": SimTypeInt(signed=False, label="IPSEC_AUTH_TYPE"), "authConfig": SimTypeChar(label="Byte")}, name="IPSEC_AUTH_TRANSFORM_ID0", pack=False, align=None), "cryptoModuleId": SimTypePointer(SimTypeBottom(label="Guid"), offset=0)}, name="IPSEC_AUTH_TRANSFORM0", pack=False, align=None), "cipherTransform": SimStruct({"cipherTransformId": SimStruct({"cipherType": SimTypeInt(signed=False, label="IPSEC_CIPHER_TYPE"), "cipherConfig": SimTypeChar(label="Byte")}, name="IPSEC_CIPHER_TRANSFORM_ID0", pack=False, align=None), "cryptoModuleId": SimTypePointer(SimTypeBottom(label="Guid"), offset=0)}, name="IPSEC_CIPHER_TRANSFORM0", pack=False, align=None)}, name="IPSEC_AUTH_AND_CIPHER_TRANSFORM0", pack=False, align=None), offset=0), "espAuthFwTransform": SimTypePointer(SimStruct({"authTransformId": SimStruct({"authType": SimTypeInt(signed=False, label="IPSEC_AUTH_TYPE"), "authConfig": SimTypeChar(label="Byte")}, name="IPSEC_AUTH_TRANSFORM_ID0", pack=False, align=None), "cryptoModuleId": SimTypePointer(SimTypeBottom(label="Guid"), offset=0)}, name="IPSEC_AUTH_TRANSFORM0", pack=False, align=None), offset=0)}, name="<anon>", label="None")}, name="IPSEC_SA_TRANSFORM0", pack=False, align=None), offset=0), "pfsGroup": SimTypeInt(signed=False, label="IPSEC_PFS_GROUP")}, name="IPSEC_PROPOSAL0", pack=False, align=None), offset=0), "tunnelEndpoints": SimStruct({"ipVersion": SimTypeInt(signed=False, label="FWP_IP_VERSION"), "Anonymous1": SimUnion({"localV4Address": SimTypeInt(signed=False, label="UInt32"), "localV6Address": SimTypeFixedSizeArray(SimTypeChar(label="Byte"), 16)}, name="<anon>", label="None"), "Anonymous2": SimUnion({"remoteV4Address": SimTypeInt(signed=False, label="UInt32"), "remoteV6Address": SimTypeFixedSizeArray(SimTypeChar(label="Byte"), 16)}, name="<anon>", label="None"), "localIfLuid": SimTypeLongLong(signed=False, label="UInt64")}, name="IPSEC_TUNNEL_ENDPOINTS1", pack=False, align=None), "saIdleTimeout": SimStruct({"idleTimeoutSeconds": SimTypeInt(signed=False, label="UInt32"), "idleTimeoutSecondsFailOver": SimTypeInt(signed=False, label="UInt32")}, name="IPSEC_SA_IDLE_TIMEOUT0", pack=False, align=None), "emPolicy": SimTypePointer(SimStruct({"numAuthenticationMethods": SimTypeInt(signed=False, label="UInt32"), "authenticationMethods": SimTypePointer(SimStruct({"authenticationMethodType": SimTypeInt(signed=False, label="IKEEXT_AUTHENTICATION_METHOD_TYPE"), "Anonymous": SimUnion({"presharedKeyAuthentication": SimStruct({"presharedKey": SimStruct({"size": SimTypeInt(signed=False, label="UInt32"), "data": SimTypePointer(SimTypeChar(label="Byte"), offset=0)}, name="FWP_BYTE_BLOB", pack=False, align=None), "flags": SimTypeInt(signed=False, label="IKEEXT_PRESHARED_KEY_AUTHENTICATION_FLAGS")}, name="IKEEXT_PRESHARED_KEY_AUTHENTICATION1", pack=False, align=None), "certificateAuthentication": SimStruct({"inboundConfigType": SimTypeInt(signed=False, label="IKEEXT_CERT_CONFIG_TYPE"), "Anonymous1": SimUnion({"Anonymous": SimStruct({"inboundRootArraySize": SimTypeInt(signed=False, label="UInt32"), "inboundRootArray": SimTypePointer(SimStruct({"certData": SimStruct({"size": SimTypeInt(signed=False, label="UInt32"), "data": SimTypePointer(SimTypeChar(label="Byte"), offset=0)}, name="FWP_BYTE_BLOB", pack=False, align=None), "flags": SimTypeInt(signed=False, label="IKEEXT_CERT_FLAGS")}, name="IKEEXT_CERT_ROOT_CONFIG0", pack=False, align=None), offset=0)}, name="_Anonymous_e__Struct", pack=False, align=None), "inboundEnterpriseStoreConfig": SimTypePointer(SimStruct({"certData": SimStruct({"size": SimTypeInt(signed=False, label="UInt32"), "data": SimTypePointer(SimTypeChar(label="Byte"), offset=0)}, name="FWP_BYTE_BLOB", pack=False, align=None), "flags": SimTypeInt(signed=False, label="IKEEXT_CERT_FLAGS")}, name="IKEEXT_CERT_ROOT_CONFIG0", pack=False, align=None), offset=0), "inboundTrustedRootStoreConfig": SimTypePointer(SimStruct({"certData": SimStruct({"size": SimTypeInt(signed=False, label="UInt32"), "data": SimTypePointer(SimTypeChar(label="Byte"), offset=0)}, name="FWP_BYTE_BLOB", pack=False, align=None), "flags": SimTypeInt(signed=False, label="IKEEXT_CERT_FLAGS")}, name="IKEEXT_CERT_ROOT_CONFIG0", pack=False, align=None), offset=0)}, name="<anon>", label="None"), "outboundConfigType": SimTypeInt(signed=False, label="IKEEXT_CERT_CONFIG_TYPE"), "Anonymous2": SimUnion({"Anonymous": SimStruct({"outboundRootArraySize": SimTypeInt(signed=False, label="UInt32"), "outboundRootArray": SimTypePointer(SimStruct({"certData": SimStruct({"size": SimTypeInt(signed=False, label="UInt32"), "data": SimTypePointer(SimTypeChar(label="Byte"), offset=0)}, name="FWP_BYTE_BLOB", pack=False, align=None), "flags": SimTypeInt(signed=False, label="IKEEXT_CERT_FLAGS")}, name="IKEEXT_CERT_ROOT_CONFIG0", pack=False, align=None), offset=0)}, name="_Anonymous_e__Struct", pack=False, align=None), "outboundEnterpriseStoreConfig": SimTypePointer(SimStruct({"certData": SimStruct({"size": SimTypeInt(signed=False, label="UInt32"), "data": SimTypePointer(SimTypeChar(label="Byte"), offset=0)}, name="FWP_BYTE_BLOB", pack=False, align=None), "flags": SimTypeInt(signed=False, label="IKEEXT_CERT_FLAGS")}, name="IKEEXT_CERT_ROOT_CONFIG0", pack=False, align=None), offset=0), "outboundTrustedRootStoreConfig": SimTypePointer(SimStruct({"certData": SimStruct({"size": SimTypeInt(signed=False, label="UInt32"), "data": SimTypePointer(SimTypeChar(label="Byte"), offset=0)}, name="FWP_BYTE_BLOB", pack=False, align=None), "flags": SimTypeInt(signed=False, label="IKEEXT_CERT_FLAGS")}, name="IKEEXT_CERT_ROOT_CONFIG0", pack=False, align=None), offset=0)}, name="<anon>", label="None"), "flags": SimTypeInt(signed=False, label="IKEEXT_CERT_AUTH"), "localCertLocationUrl": SimStruct({"size": SimTypeInt(signed=False, label="UInt32"), "data": SimTypePointer(SimTypeChar(label="Byte"), offset=0)}, name="FWP_BYTE_BLOB", pack=False, align=None)}, name="IKEEXT_CERTIFICATE_AUTHENTICATION1", pack=False, align=None), "kerberosAuthentication": SimStruct({"flags": SimTypeInt(signed=False, label="IKEEXT_KERBEROS_AUTHENTICATION_FLAGS")}, name="IKEEXT_KERBEROS_AUTHENTICATION0", pack=False, align=None), "ntlmV2Authentication": SimStruct({"flags": SimTypeInt(signed=False, label="UInt32")}, name="IKEEXT_NTLM_V2_AUTHENTICATION0", pack=False, align=None), "sslAuthentication": SimStruct({"inboundConfigType": SimTypeInt(signed=False, label="IKEEXT_CERT_CONFIG_TYPE"), "Anonymous1": SimUnion({"Anonymous": SimStruct({"inboundRootArraySize": SimTypeInt(signed=False, label="UInt32"), "inboundRootArray": SimTypePointer(SimStruct({"certData": SimStruct({"size": SimTypeInt(signed=False, label="UInt32"), "data": SimTypePointer(SimTypeChar(label="Byte"), offset=0)}, name="FWP_BYTE_BLOB", pack=False, align=None), "flags": SimTypeInt(signed=False, label="IKEEXT_CERT_FLAGS")}, name="IKEEXT_CERT_ROOT_CONFIG0", pack=False, align=None), offset=0)}, name="_Anonymous_e__Struct", pack=False, align=None), "inboundEnterpriseStoreConfig": SimTypePointer(SimStruct({"certData": SimStruct({"size": SimTypeInt(signed=False, label="UInt32"), "data": SimTypePointer(SimTypeChar(label="Byte"), offset=0)}, name="FWP_BYTE_BLOB", pack=False, align=None), "flags": SimTypeInt(signed=False, label="IKEEXT_CERT_FLAGS")}, name="IKEEXT_CERT_ROOT_CONFIG0", pack=False, align=None), offset=0), "inboundTrustedRootStoreConfig": SimTypePointer(SimStruct({"certData": SimStruct({"size": SimTypeInt(signed=False, label="UInt32"), "data": SimTypePointer(SimTypeChar(label="Byte"), offset=0)}, name="FWP_BYTE_BLOB", pack=False, align=None), "flags": SimTypeInt(signed=False, label="IKEEXT_CERT_FLAGS")}, name="IKEEXT_CERT_ROOT_CONFIG0", pack=False, align=None), offset=0)}, name="<anon>", label="None"), "outboundConfigType": SimTypeInt(signed=False, label="IKEEXT_CERT_CONFIG_TYPE"), "Anonymous2": SimUnion({"Anonymous": SimStruct({"outboundRootArraySize": SimTypeInt(signed=False, label="UInt32"), "outboundRootArray": SimTypePointer(SimStruct({"certData": SimStruct({"size": SimTypeInt(signed=False, label="UInt32"), "data": SimTypePointer(SimTypeChar(label="Byte"), offset=0)}, name="FWP_BYTE_BLOB", pack=False, align=None), "flags": SimTypeInt(signed=False, label="IKEEXT_CERT_FLAGS")}, name="IKEEXT_CERT_ROOT_CONFIG0", pack=False, align=None), offset=0)}, name="_Anonymous_e__Struct", pack=False, align=None), "outboundEnterpriseStoreConfig": SimTypePointer(SimStruct({"certData": SimStruct({"size": SimTypeInt(signed=False, label="UInt32"), "data": SimTypePointer(SimTypeChar(label="Byte"), offset=0)}, name="FWP_BYTE_BLOB", pack=False, align=None), "flags": SimTypeInt(signed=False, label="IKEEXT_CERT_FLAGS")}, name="IKEEXT_CERT_ROOT_CONFIG0", pack=False, align=None), offset=0), "outboundTrustedRootStoreConfig": SimTypePointer(SimStruct({"certData": SimStruct({"size": SimTypeInt(signed=False, label="UInt32"), "data": SimTypePointer(SimTypeChar(label="Byte"), offset=0)}, name="FWP_BYTE_BLOB", pack=False, align=None), "flags": SimTypeInt(signed=False, label="IKEEXT_CERT_FLAGS")}, name="IKEEXT_CERT_ROOT_CONFIG0", pack=False, align=None), offset=0)}, name="<anon>", label="None"), "flags": SimTypeInt(signed=False, label="IKEEXT_CERT_AUTH"), "localCertLocationUrl": SimStruct({"size": SimTypeInt(signed=False, label="UInt32"), "data": SimTypePointer(SimTypeChar(label="Byte"), offset=0)}, name="FWP_BYTE_BLOB", pack=False, align=None)}, name="IKEEXT_CERTIFICATE_AUTHENTICATION1", pack=False, align=None), "cgaAuthentication": SimStruct({"keyContainerName": SimTypePointer(SimTypeChar(label="Char"), offset=0), "cspName": SimTypePointer(SimTypeChar(label="Char"), offset=0), "cspType": SimTypeInt(signed=False, label="UInt32"), "cgaModifier": SimStruct({"byteArray16": SimTypeFixedSizeArray(SimTypeChar(label="Byte"), 16)}, name="FWP_BYTE_ARRAY16", pack=False, align=None), "cgaCollisionCount": SimTypeChar(label="Byte")}, name="IKEEXT_IPV6_CGA_AUTHENTICATION0", pack=False, align=None), "eapAuthentication": SimStruct({"flags": SimTypeInt(signed=False, label="IKEEXT_EAP_AUTHENTICATION_FLAGS")}, name="IKEEXT_EAP_AUTHENTICATION0", pack=False, align=None)}, name="<anon>", label="None")}, name="IKEEXT_AUTHENTICATION_METHOD1", pack=False, align=None), offset=0), "initiatorImpersonationType": SimTypeInt(signed=False, label="IKEEXT_AUTHENTICATION_IMPERSONATION_TYPE")}, name="IKEEXT_EM_POLICY1", pack=False, align=None), offset=0)}, name="IPSEC_TUNNEL_POLICY1", pack=False, align=None), offset=0), "authipQmTransportPolicy": SimTypePointer(SimStruct({"numIpsecProposals": SimTypeInt(signed=False, label="UInt32"), "ipsecProposals": SimTypePointer(SimStruct({"lifetime": SimStruct({"lifetimeSeconds": SimTypeInt(signed=False, label="UInt32"), "lifetimeKilobytes": SimTypeInt(signed=False, label="UInt32"), "lifetimePackets": SimTypeInt(signed=False, label="UInt32")}, name="IPSEC_SA_LIFETIME0", pack=False, align=None), "numSaTransforms": SimTypeInt(signed=False, label="UInt32"), "saTransforms": SimTypePointer(SimStruct({"ipsecTransformType": SimTypeInt(signed=False, label="IPSEC_TRANSFORM_TYPE"), "Anonymous": SimUnion({"ahTransform": SimTypePointer(SimStruct({"authTransformId": SimStruct({"authType": SimTypeInt(signed=False, label="IPSEC_AUTH_TYPE"), "authConfig": SimTypeChar(label="Byte")}, name="IPSEC_AUTH_TRANSFORM_ID0", pack=False, align=None), "cryptoModuleId": SimTypePointer(SimTypeBottom(label="Guid"), offset=0)}, name="IPSEC_AUTH_TRANSFORM0", pack=False, align=None), offset=0), "espAuthTransform": SimTypePointer(SimStruct({"authTransformId": SimStruct({"authType": SimTypeInt(signed=False, label="IPSEC_AUTH_TYPE"), "authConfig": SimTypeChar(label="Byte")}, name="IPSEC_AUTH_TRANSFORM_ID0", pack=False, align=None), "cryptoModuleId": SimTypePointer(SimTypeBottom(label="Guid"), offset=0)}, name="IPSEC_AUTH_TRANSFORM0", pack=False, align=None), offset=0), "espCipherTransform": SimTypePointer(SimStruct({"cipherTransformId": SimStruct({"cipherType": SimTypeInt(signed=False, label="IPSEC_CIPHER_TYPE"), "cipherConfig": SimTypeChar(label="Byte")}, name="IPSEC_CIPHER_TRANSFORM_ID0", pack=False, align=None), "cryptoModuleId": SimTypePointer(SimTypeBottom(label="Guid"), offset=0)}, name="IPSEC_CIPHER_TRANSFORM0", pack=False, align=None), offset=0), "espAuthAndCipherTransform": SimTypePointer(SimStruct({"authTransform": SimStruct({"authTransformId": SimStruct({"authType": SimTypeInt(signed=False, label="IPSEC_AUTH_TYPE"), "authConfig": SimTypeChar(label="Byte")}, name="IPSEC_AUTH_TRANSFORM_ID0", pack=False, align=None), "cryptoModuleId": SimTypePointer(SimTypeBottom(label="Guid"), offset=0)}, name="IPSEC_AUTH_TRANSFORM0", pack=False, align=None), "cipherTransform": SimStruct({"cipherTransformId": SimStruct({"cipherType": SimTypeInt(signed=False, label="IPSEC_CIPHER_TYPE"), "cipherConfig": SimTypeChar(label="Byte")}, name="IPSEC_CIPHER_TRANSFORM_ID0", pack=False, align=None), "cryptoModuleId": SimTypePointer(SimTypeBottom(label="Guid"), offset=0)}, name="IPSEC_CIPHER_TRANSFORM0", pack=False, align=None)}, name="IPSEC_AUTH_AND_CIPHER_TRANSFORM0", pack=False, align=None), offset=0), "espAuthFwTransform": SimTypePointer(SimStruct({"authTransformId": SimStruct({"authType": SimTypeInt(signed=False, label="IPSEC_AUTH_TYPE"), "authConfig": SimTypeChar(label="Byte")}, name="IPSEC_AUTH_TRANSFORM_ID0", pack=False, align=None), "cryptoModuleId": SimTypePointer(SimTypeBottom(label="Guid"), offset=0)}, name="IPSEC_AUTH_TRANSFORM0", pack=False, align=None), offset=0)}, name="<anon>", label="None")}, name="IPSEC_SA_TRANSFORM0", pack=False, align=None), offset=0), "pfsGroup": SimTypeInt(signed=False, label="IPSEC_PFS_GROUP")}, name="IPSEC_PROPOSAL0", pack=False, align=None), offset=0), "flags": SimTypeInt(signed=False, label="IPSEC_POLICY_FLAG"), "ndAllowClearTimeoutSeconds": SimTypeInt(signed=False, label="UInt32"), "saIdleTimeout": SimStruct({"idleTimeoutSeconds": SimTypeInt(signed=False, label="UInt32"), "idleTimeoutSecondsFailOver": SimTypeInt(signed=False, label="UInt32")}, name="IPSEC_SA_IDLE_TIMEOUT0", pack=False, align=None), "emPolicy": SimTypePointer(SimStruct({"numAuthenticationMethods": SimTypeInt(signed=False, label="UInt32"), "authenticationMethods": SimTypePointer(SimStruct({"authenticationMethodType": SimTypeInt(signed=False, label="IKEEXT_AUTHENTICATION_METHOD_TYPE"), "Anonymous": SimUnion({"presharedKeyAuthentication": SimStruct({"presharedKey": SimStruct({"size": SimTypeInt(signed=False, label="UInt32"), "data": SimTypePointer(SimTypeChar(label="Byte"), offset=0)}, name="FWP_BYTE_BLOB", pack=False, align=None), "flags": SimTypeInt(signed=False, label="IKEEXT_PRESHARED_KEY_AUTHENTICATION_FLAGS")}, name="IKEEXT_PRESHARED_KEY_AUTHENTICATION1", pack=False, align=None), "certificateAuthentication": SimStruct({"inboundConfigType": SimTypeInt(signed=False, label="IKEEXT_CERT_CONFIG_TYPE"), "Anonymous1": SimUnion({"Anonymous": SimStruct({"inboundRootArraySize": SimTypeInt(signed=False, label="UInt32"), "inboundRootArray": SimTypePointer(SimStruct({"certData": SimStruct({"size": SimTypeInt(signed=False, label="UInt32"), "data": SimTypePointer(SimTypeChar(label="Byte"), offset=0)}, name="FWP_BYTE_BLOB", pack=False, align=None), "flags": SimTypeInt(signed=False, label="IKEEXT_CERT_FLAGS")}, name="IKEEXT_CERT_ROOT_CONFIG0", pack=False, align=None), offset=0)}, name="_Anonymous_e__Struct", pack=False, align=None), "inboundEnterpriseStoreConfig": SimTypePointer(SimStruct({"certData": SimStruct({"size": SimTypeInt(signed=False, label="UInt32"), "data": SimTypePointer(SimTypeChar(label="Byte"), offset=0)}, name="FWP_BYTE_BLOB", pack=False, align=None), "flags": SimTypeInt(signed=False, label="IKEEXT_CERT_FLAGS")}, name="IKEEXT_CERT_ROOT_CONFIG0", pack=False, align=None), offset=0), "inboundTrustedRootStoreConfig": SimTypePointer(SimStruct({"certData": SimStruct({"size": SimTypeInt(signed=False, label="UInt32"), "data": SimTypePointer(SimTypeChar(label="Byte"), offset=0)}, name="FWP_BYTE_BLOB", pack=False, align=None), "flags": SimTypeInt(signed=False, label="IKEEXT_CERT_FLAGS")}, name="IKEEXT_CERT_ROOT_CONFIG0", pack=False, align=None), offset=0)}, name="<anon>", label="None"), "outboundConfigType": SimTypeInt(signed=False, label="IKEEXT_CERT_CONFIG_TYPE"), "Anonymous2": SimUnion({"Anonymous": SimStruct({"outboundRootArraySize": SimTypeInt(signed=False, label="UInt32"), "outboundRootArray": SimTypePointer(SimStruct({"certData": SimStruct({"size": SimTypeInt(signed=False, label="UInt32"), "data": SimTypePointer(SimTypeChar(label="Byte"), offset=0)}, name="FWP_BYTE_BLOB", pack=False, align=None), "flags": SimTypeInt(signed=False, label="IKEEXT_CERT_FLAGS")}, name="IKEEXT_CERT_ROOT_CONFIG0", pack=False, align=None), offset=0)}, name="_Anonymous_e__Struct", pack=False, align=None), "outboundEnterpriseStoreConfig": SimTypePointer(SimStruct({"certData": SimStruct({"size": SimTypeInt(signed=False, label="UInt32"), "data": SimTypePointer(SimTypeChar(label="Byte"), offset=0)}, name="FWP_BYTE_BLOB", pack=False, align=None), "flags": SimTypeInt(signed=False, label="IKEEXT_CERT_FLAGS")}, name="IKEEXT_CERT_ROOT_CONFIG0", pack=False, align=None), offset=0), "outboundTrustedRootStoreConfig": SimTypePointer(SimStruct({"certData": SimStruct({"size": SimTypeInt(signed=False, label="UInt32"), "data": SimTypePointer(SimTypeChar(label="Byte"), offset=0)}, name="FWP_BYTE_BLOB", pack=False, align=None), "flags": SimTypeInt(signed=False, label="IKEEXT_CERT_FLAGS")}, name="IKEEXT_CERT_ROOT_CONFIG0", pack=False, align=None), offset=0)}, name="<anon>", label="None"), "flags": SimTypeInt(signed=False, label="IKEEXT_CERT_AUTH"), "localCertLocationUrl": SimStruct({"size": SimTypeInt(signed=False, label="UInt32"), "data": SimTypePointer(SimTypeChar(label="Byte"), offset=0)}, name="FWP_BYTE_BLOB", pack=False, align=None)}, name="IKEEXT_CERTIFICATE_AUTHENTICATION1", pack=False, align=None), "kerberosAuthentication": SimStruct({"flags": SimTypeInt(signed=False, label="IKEEXT_KERBEROS_AUTHENTICATION_FLAGS")}, name="IKEEXT_KERBEROS_AUTHENTICATION0", pack=False, align=None), "ntlmV2Authentication": SimStruct({"flags": SimTypeInt(signed=False, label="UInt32")}, name="IKEEXT_NTLM_V2_AUTHENTICATION0", pack=False, align=None), "sslAuthentication": SimStruct({"inboundConfigType": SimTypeInt(signed=False, label="IKEEXT_CERT_CONFIG_TYPE"), "Anonymous1": SimUnion({"Anonymous": SimStruct({"inboundRootArraySize": SimTypeInt(signed=False, label="UInt32"), "inboundRootArray": SimTypePointer(SimStruct({"certData": SimStruct({"size": SimTypeInt(signed=False, label="UInt32"), "data": SimTypePointer(SimTypeChar(label="Byte"), offset=0)}, name="FWP_BYTE_BLOB", pack=False, align=None), "flags": SimTypeInt(signed=False, label="IKEEXT_CERT_FLAGS")}, name="IKEEXT_CERT_ROOT_CONFIG0", pack=False, align=None), offset=0)}, name="_Anonymous_e__Struct", pack=False, align=None), "inboundEnterpriseStoreConfig": SimTypePointer(SimStruct({"certData": SimStruct({"size": SimTypeInt(signed=False, label="UInt32"), "data": SimTypePointer(SimTypeChar(label="Byte"), offset=0)}, name="FWP_BYTE_BLOB", pack=False, align=None), "flags": SimTypeInt(signed=False, label="IKEEXT_CERT_FLAGS")}, name="IKEEXT_CERT_ROOT_CONFIG0", pack=False, align=None), offset=0), "inboundTrustedRootStoreConfig": SimTypePointer(SimStruct({"certData": SimStruct({"size": SimTypeInt(signed=False, label="UInt32"), "data": SimTypePointer(SimTypeChar(label="Byte"), offset=0)}, name="FWP_BYTE_BLOB", pack=False, align=None), "flags": SimTypeInt(signed=False, label="IKEEXT_CERT_FLAGS")}, name="IKEEXT_CERT_ROOT_CONFIG0", pack=False, align=None), offset=0)}, name="<anon>", label="None"), "outboundConfigType": SimTypeInt(signed=False, label="IKEEXT_CERT_CONFIG_TYPE"), "Anonymous2": SimUnion({"Anonymous": SimStruct({"outboundRootArraySize": SimTypeInt(signed=False, label="UInt32"), "outboundRootArray": SimTypePointer(SimStruct({"certData": SimStruct({"size": SimTypeInt(signed=False, label="UInt32"), "data": SimTypePointer(SimTypeChar(label="Byte"), offset=0)}, name="FWP_BYTE_BLOB", pack=False, align=None), "flags": SimTypeInt(signed=False, label="IKEEXT_CERT_FLAGS")}, name="IKEEXT_CERT_ROOT_CONFIG0", pack=False, align=None), offset=0)}, name="_Anonymous_e__Struct", pack=False, align=None), "outboundEnterpriseStoreConfig": SimTypePointer(SimStruct({"certData": SimStruct({"size": SimTypeInt(signed=False, label="UInt32"), "data": SimTypePointer(SimTypeChar(label="Byte"), offset=0)}, name="FWP_BYTE_BLOB", pack=False, align=None), "flags": SimTypeInt(signed=False, label="IKEEXT_CERT_FLAGS")}, name="IKEEXT_CERT_ROOT_CONFIG0", pack=False, align=None), offset=0), "outboundTrustedRootStoreConfig": SimTypePointer(SimStruct({"certData": SimStruct({"size": SimTypeInt(signed=False, label="UInt32"), "data": SimTypePointer(SimTypeChar(label="Byte"), offset=0)}, name="FWP_BYTE_BLOB", pack=False, align=None), "flags": SimTypeInt(signed=False, label="IKEEXT_CERT_FLAGS")}, name="IKEEXT_CERT_ROOT_CONFIG0", pack=False, align=None), offset=0)}, name="<anon>", label="None"), "flags": SimTypeInt(signed=False, label="IKEEXT_CERT_AUTH"), "localCertLocationUrl": SimStruct({"size": SimTypeInt(signed=False, label="UInt32"), "data": SimTypePointer(SimTypeChar(label="Byte"), offset=0)}, name="FWP_BYTE_BLOB", pack=False, align=None)}, name="IKEEXT_CERTIFICATE_AUTHENTICATION1", pack=False, align=None), "cgaAuthentication": SimStruct({"keyContainerName": SimTypePointer(SimTypeChar(label="Char"), offset=0), "cspName": SimTypePointer(SimTypeChar(label="Char"), offset=0), "cspType": SimTypeInt(signed=False, label="UInt32"), "cgaModifier": SimStruct({"byteArray16": SimTypeFixedSizeArray(SimTypeChar(label="Byte"), 16)}, name="FWP_BYTE_ARRAY16", pack=False, align=None), "cgaCollisionCount": SimTypeChar(label="Byte")}, name="IKEEXT_IPV6_CGA_AUTHENTICATION0", pack=False, align=None), "eapAuthentication": SimStruct({"flags": SimTypeInt(signed=False, label="IKEEXT_EAP_AUTHENTICATION_FLAGS")}, name="IKEEXT_EAP_AUTHENTICATION0", pack=False, align=None)}, name="<anon>", label="None")}, name="IKEEXT_AUTHENTICATION_METHOD1", pack=False, align=None), offset=0), "initiatorImpersonationType": SimTypeInt(signed=False, label="IKEEXT_AUTHENTICATION_IMPERSONATION_TYPE")}, name="IKEEXT_EM_POLICY1", pack=False, align=None), offset=0)}, name="IPSEC_TRANSPORT_POLICY1", pack=False, align=None), offset=0), "authipQmTunnelPolicy": SimTypePointer(SimStruct({"flags": SimTypeInt(signed=False, label="IPSEC_POLICY_FLAG"), "numIpsecProposals": SimTypeInt(signed=False, label="UInt32"), "ipsecProposals": SimTypePointer(SimStruct({"lifetime": SimStruct({"lifetimeSeconds": SimTypeInt(signed=False, label="UInt32"), "lifetimeKilobytes": SimTypeInt(signed=False, label="UInt32"), "lifetimePackets": SimTypeInt(signed=False, label="UInt32")}, name="IPSEC_SA_LIFETIME0", pack=False, align=None), "numSaTransforms": SimTypeInt(signed=False, label="UInt32"), "saTransforms": SimTypePointer(SimStruct({"ipsecTransformType": SimTypeInt(signed=False, label="IPSEC_TRANSFORM_TYPE"), "Anonymous": SimUnion({"ahTransform": SimTypePointer(SimStruct({"authTransformId": SimStruct({"authType": SimTypeInt(signed=False, label="IPSEC_AUTH_TYPE"), "authConfig": SimTypeChar(label="Byte")}, name="IPSEC_AUTH_TRANSFORM_ID0", pack=False, align=None), "cryptoModuleId": SimTypePointer(SimTypeBottom(label="Guid"), offset=0)}, name="IPSEC_AUTH_TRANSFORM0", pack=False, align=None), offset=0), "espAuthTransform": SimTypePointer(SimStruct({"authTransformId": SimStruct({"authType": SimTypeInt(signed=False, label="IPSEC_AUTH_TYPE"), "authConfig": SimTypeChar(label="Byte")}, name="IPSEC_AUTH_TRANSFORM_ID0", pack=False, align=None), "cryptoModuleId": SimTypePointer(SimTypeBottom(label="Guid"), offset=0)}, name="IPSEC_AUTH_TRANSFORM0", pack=False, align=None), offset=0), "espCipherTransform": SimTypePointer(SimStruct({"cipherTransformId": SimStruct({"cipherType": SimTypeInt(signed=False, label="IPSEC_CIPHER_TYPE"), "cipherConfig": SimTypeChar(label="Byte")}, name="IPSEC_CIPHER_TRANSFORM_ID0", pack=False, align=None), "cryptoModuleId": SimTypePointer(SimTypeBottom(label="Guid"), offset=0)}, name="IPSEC_CIPHER_TRANSFORM0", pack=False, align=None), offset=0), "espAuthAndCipherTransform": SimTypePointer(SimStruct({"authTransform": SimStruct({"authTransformId": SimStruct({"authType": SimTypeInt(signed=False, label="IPSEC_AUTH_TYPE"), "authConfig": SimTypeChar(label="Byte")}, name="IPSEC_AUTH_TRANSFORM_ID0", pack=False, align=None), "cryptoModuleId": SimTypePointer(SimTypeBottom(label="Guid"), offset=0)}, name="IPSEC_AUTH_TRANSFORM0", pack=False, align=None), "cipherTransform": SimStruct({"cipherTransformId": SimStruct({"cipherType": SimTypeInt(signed=False, label="IPSEC_CIPHER_TYPE"), "cipherConfig": SimTypeChar(label="Byte")}, name="IPSEC_CIPHER_TRANSFORM_ID0", pack=False, align=None), "cryptoModuleId": SimTypePointer(SimTypeBottom(label="Guid"), offset=0)}, name="IPSEC_CIPHER_TRANSFORM0", pack=False, align=None)}, name="IPSEC_AUTH_AND_CIPHER_TRANSFORM0", pack=False, align=None), offset=0), "espAuthFwTransform": SimTypePointer(SimStruct({"authTransformId": SimStruct({"authType": SimTypeInt(signed=False, label="IPSEC_AUTH_TYPE"), "authConfig": SimTypeChar(label="Byte")}, name="IPSEC_AUTH_TRANSFORM_ID0", pack=False, align=None), "cryptoModuleId": SimTypePointer(SimTypeBottom(label="Guid"), offset=0)}, name="IPSEC_AUTH_TRANSFORM0", pack=False, align=None), offset=0)}, name="<anon>", label="None")}, name="IPSEC_SA_TRANSFORM0", pack=False, align=None), offset=0), "pfsGroup": SimTypeInt(signed=False, label="IPSEC_PFS_GROUP")}, name="IPSEC_PROPOSAL0", pack=False, align=None), offset=0), "tunnelEndpoints": SimStruct({"ipVersion": SimTypeInt(signed=False, label="FWP_IP_VERSION"), "Anonymous1": SimUnion({"localV4Address": SimTypeInt(signed=False, label="UInt32"), "localV6Address": SimTypeFixedSizeArray(SimTypeChar(label="Byte"), 16)}, name="<anon>", label="None"), "Anonymous2": SimUnion({"remoteV4Address": SimTypeInt(signed=False, label="UInt32"), "remoteV6Address": SimTypeFixedSizeArray(SimTypeChar(label="Byte"), 16)}, name="<anon>", label="None"), "localIfLuid": SimTypeLongLong(signed=False, label="UInt64")}, name="IPSEC_TUNNEL_ENDPOINTS1", pack=False, align=None), "saIdleTimeout": SimStruct({"idleTimeoutSeconds": SimTypeInt(signed=False, label="UInt32"), "idleTimeoutSecondsFailOver": SimTypeInt(signed=False, label="UInt32")}, name="IPSEC_SA_IDLE_TIMEOUT0", pack=False, align=None), "emPolicy": SimTypePointer(SimStruct({"numAuthenticationMethods": SimTypeInt(signed=False, label="UInt32"), "authenticationMethods": SimTypePointer(SimStruct({"authenticationMethodType": SimTypeInt(signed=False, label="IKEEXT_AUTHENTICATION_METHOD_TYPE"), "Anonymous": SimUnion({"presharedKeyAuthentication": SimStruct({"presharedKey": SimStruct({"size": SimTypeInt(signed=False, label="UInt32"), "data": SimTypePointer(SimTypeChar(label="Byte"), offset=0)}, name="FWP_BYTE_BLOB", pack=False, align=None), "flags": SimTypeInt(signed=False, label="IKEEXT_PRESHARED_KEY_AUTHENTICATION_FLAGS")}, name="IKEEXT_PRESHARED_KEY_AUTHENTICATION1", pack=False, align=None), "certificateAuthentication": SimStruct({"inboundConfigType": SimTypeInt(signed=False, label="IKEEXT_CERT_CONFIG_TYPE"), "Anonymous1": SimUnion({"Anonymous": SimStruct({"inboundRootArraySize": SimTypeInt(signed=False, label="UInt32"), "inboundRootArray": SimTypePointer(SimStruct({"certData": SimStruct({"size": SimTypeInt(signed=False, label="UInt32"), "data": SimTypePointer(SimTypeChar(label="Byte"), offset=0)}, name="FWP_BYTE_BLOB", pack=False, align=None), "flags": SimTypeInt(signed=False, label="IKEEXT_CERT_FLAGS")}, name="IKEEXT_CERT_ROOT_CONFIG0", pack=False, align=None), offset=0)}, name="_Anonymous_e__Struct", pack=False, align=None), "inboundEnterpriseStoreConfig": SimTypePointer(SimStruct({"certData": SimStruct({"size": SimTypeInt(signed=False, label="UInt32"), "data": SimTypePointer(SimTypeChar(label="Byte"), offset=0)}, name="FWP_BYTE_BLOB", pack=False, align=None), "flags": SimTypeInt(signed=False, label="IKEEXT_CERT_FLAGS")}, name="IKEEXT_CERT_ROOT_CONFIG0", pack=False, align=None), offset=0), "inboundTrustedRootStoreConfig": SimTypePointer(SimStruct({"certData": SimStruct({"size": SimTypeInt(signed=False, label="UInt32"), "data": SimTypePointer(SimTypeChar(label="Byte"), offset=0)}, name="FWP_BYTE_BLOB", pack=False, align=None), "flags": SimTypeInt(signed=False, label="IKEEXT_CERT_FLAGS")}, name="IKEEXT_CERT_ROOT_CONFIG0", pack=False, align=None), offset=0)}, name="<anon>", label="None"), "outboundConfigType": SimTypeInt(signed=False, label="IKEEXT_CERT_CONFIG_TYPE"), "Anonymous2": SimUnion({"Anonymous": SimStruct({"outboundRootArraySize": SimTypeInt(signed=False, label="UInt32"), "outboundRootArray": SimTypePointer(SimStruct({"certData": SimStruct({"size": SimTypeInt(signed=False, label="UInt32"), "data": SimTypePointer(SimTypeChar(label="Byte"), offset=0)}, name="FWP_BYTE_BLOB", pack=False, align=None), "flags": SimTypeInt(signed=False, label="IKEEXT_CERT_FLAGS")}, name="IKEEXT_CERT_ROOT_CONFIG0", pack=False, align=None), offset=0)}, name="_Anonymous_e__Struct", pack=False, align=None), "outboundEnterpriseStoreConfig": SimTypePointer(SimStruct({"certData": SimStruct({"size": SimTypeInt(signed=False, label="UInt32"), "data": SimTypePointer(SimTypeChar(label="Byte"), offset=0)}, name="FWP_BYTE_BLOB", pack=False, align=None), "flags": SimTypeInt(signed=False, label="IKEEXT_CERT_FLAGS")}, name="IKEEXT_CERT_ROOT_CONFIG0", pack=False, align=None), offset=0), "outboundTrustedRootStoreConfig": SimTypePointer(SimStruct({"certData": SimStruct({"size": SimTypeInt(signed=False, label="UInt32"), "data": SimTypePointer(SimTypeChar(label="Byte"), offset=0)}, name="FWP_BYTE_BLOB", pack=False, align=None), "flags": SimTypeInt(signed=False, label="IKEEXT_CERT_FLAGS")}, name="IKEEXT_CERT_ROOT_CONFIG0", pack=False, align=None), offset=0)}, name="<anon>", label="None"), "flags": SimTypeInt(signed=False, label="IKEEXT_CERT_AUTH"), "localCertLocationUrl": SimStruct({"size": SimTypeInt(signed=False, label="UInt32"), "data": SimTypePointer(SimTypeChar(label="Byte"), offset=0)}, name="FWP_BYTE_BLOB", pack=False, align=None)}, name="IKEEXT_CERTIFICATE_AUTHENTICATION1", pack=False, align=None), "kerberosAuthentication": SimStruct({"flags": SimTypeInt(signed=False, label="IKEEXT_KERBEROS_AUTHENTICATION_FLAGS")}, name="IKEEXT_KERBEROS_AUTHENTICATION0", pack=False, align=None), "ntlmV2Authentication": SimStruct({"flags": SimTypeInt(signed=False, label="UInt32")}, name="IKEEXT_NTLM_V2_AUTHENTICATION0", pack=False, align=None), "sslAuthentication": SimStruct({"inboundConfigType": SimTypeInt(signed=False, label="IKEEXT_CERT_CONFIG_TYPE"), "Anonymous1": SimUnion({"Anonymous": SimStruct({"inboundRootArraySize": SimTypeInt(signed=False, label="UInt32"), "inboundRootArray": SimTypePointer(SimStruct({"certData": SimStruct({"size": SimTypeInt(signed=False, label="UInt32"), "data": SimTypePointer(SimTypeChar(label="Byte"), offset=0)}, name="FWP_BYTE_BLOB", pack=False, align=None), "flags": SimTypeInt(signed=False, label="IKEEXT_CERT_FLAGS")}, name="IKEEXT_CERT_ROOT_CONFIG0", pack=False, align=None), offset=0)}, name="_Anonymous_e__Struct", pack=False, align=None), "inboundEnterpriseStoreConfig": SimTypePointer(SimStruct({"certData": SimStruct({"size": SimTypeInt(signed=False, label="UInt32"), "data": SimTypePointer(SimTypeChar(label="Byte"), offset=0)}, name="FWP_BYTE_BLOB", pack=False, align=None), "flags": SimTypeInt(signed=False, label="IKEEXT_CERT_FLAGS")}, name="IKEEXT_CERT_ROOT_CONFIG0", pack=False, align=None), offset=0), "inboundTrustedRootStoreConfig": SimTypePointer(SimStruct({"certData": SimStruct({"size": SimTypeInt(signed=False, label="UInt32"), "data": SimTypePointer(SimTypeChar(label="Byte"), offset=0)}, name="FWP_BYTE_BLOB", pack=False, align=None), "flags": SimTypeInt(signed=False, label="IKEEXT_CERT_FLAGS")}, name="IKEEXT_CERT_ROOT_CONFIG0", pack=False, align=None), offset=0)}, name="<anon>", label="None"), "outboundConfigType": SimTypeInt(signed=False, label="IKEEXT_CERT_CONFIG_TYPE"), "Anonymous2": SimUnion({"Anonymous": SimStruct({"outboundRootArraySize": SimTypeInt(signed=False, label="UInt32"), "outboundRootArray": SimTypePointer(SimStruct({"certData": SimStruct({"size": SimTypeInt(signed=False, label="UInt32"), "data": SimTypePointer(SimTypeChar(label="Byte"), offset=0)}, name="FWP_BYTE_BLOB", pack=False, align=None), "flags": SimTypeInt(signed=False, label="IKEEXT_CERT_FLAGS")}, name="IKEEXT_CERT_ROOT_CONFIG0", pack=False, align=None), offset=0)}, name="_Anonymous_e__Struct", pack=False, align=None), "outboundEnterpriseStoreConfig": SimTypePointer(SimStruct({"certData": SimStruct({"size": SimTypeInt(signed=False, label="UInt32"), "data": SimTypePointer(SimTypeChar(label="Byte"), offset=0)}, name="FWP_BYTE_BLOB", pack=False, align=None), "flags": SimTypeInt(signed=False, label="IKEEXT_CERT_FLAGS")}, name="IKEEXT_CERT_ROOT_CONFIG0", pack=False, align=None), offset=0), "outboundTrustedRootStoreConfig": SimTypePointer(SimStruct({"certData": SimStruct({"size": SimTypeInt(signed=False, label="UInt32"), "data": SimTypePointer(SimTypeChar(label="Byte"), offset=0)}, name="FWP_BYTE_BLOB", pack=False, align=None), "flags": SimTypeInt(signed=False, label="IKEEXT_CERT_FLAGS")}, name="IKEEXT_CERT_ROOT_CONFIG0", pack=False, align=None), offset=0)}, name="<anon>", label="None"), "flags": SimTypeInt(signed=False, label="IKEEXT_CERT_AUTH"), "localCertLocationUrl": SimStruct({"size": SimTypeInt(signed=False, label="UInt32"), "data": SimTypePointer(SimTypeChar(label="Byte"), offset=0)}, name="FWP_BYTE_BLOB", pack=False, align=None)}, name="IKEEXT_CERTIFICATE_AUTHENTICATION1", pack=False, align=None), "cgaAuthentication": SimStruct({"keyContainerName": SimTypePointer(SimTypeChar(label="Char"), offset=0), "cspName": SimTypePointer(SimTypeChar(label="Char"), offset=0), "cspType": SimTypeInt(signed=False, label="UInt32"), "cgaModifier": SimStruct({"byteArray16": SimTypeFixedSizeArray(SimTypeChar(label="Byte"), 16)}, name="FWP_BYTE_ARRAY16", pack=False, align=None), "cgaCollisionCount": SimTypeChar(label="Byte")}, name="IKEEXT_IPV6_CGA_AUTHENTICATION0", pack=False, align=None), "eapAuthentication": SimStruct({"flags": SimTypeInt(signed=False, label="IKEEXT_EAP_AUTHENTICATION_FLAGS")}, name="IKEEXT_EAP_AUTHENTICATION0", pack=False, align=None)}, name="<anon>", label="None")}, name="IKEEXT_AUTHENTICATION_METHOD1", pack=False, align=None), offset=0), "initiatorImpersonationType": SimTypeInt(signed=False, label="IKEEXT_AUTHENTICATION_IMPERSONATION_TYPE")}, name="IKEEXT_EM_POLICY1", pack=False, align=None), offset=0)}, name="IPSEC_TUNNEL_POLICY1", pack=False, align=None), offset=0), "ikeMmPolicy": SimTypePointer(SimStruct({"softExpirationTime": SimTypeInt(signed=False, label="UInt32"), "numAuthenticationMethods": SimTypeInt(signed=False, label="UInt32"), "authenticationMethods": SimTypePointer(SimStruct({"authenticationMethodType": SimTypeInt(signed=False, label="IKEEXT_AUTHENTICATION_METHOD_TYPE"), "Anonymous": SimUnion({"presharedKeyAuthentication": SimStruct({"presharedKey": SimStruct({"size": SimTypeInt(signed=False, label="UInt32"), "data": SimTypePointer(SimTypeChar(label="Byte"), offset=0)}, name="FWP_BYTE_BLOB", pack=False, align=None), "flags": SimTypeInt(signed=False, label="IKEEXT_PRESHARED_KEY_AUTHENTICATION_FLAGS")}, name="IKEEXT_PRESHARED_KEY_AUTHENTICATION1", pack=False, align=None), "certificateAuthentication": SimStruct({"inboundConfigType": SimTypeInt(signed=False, label="IKEEXT_CERT_CONFIG_TYPE"), "Anonymous1": SimUnion({"Anonymous": SimStruct({"inboundRootArraySize": SimTypeInt(signed=False, label="UInt32"), "inboundRootArray": SimTypePointer(SimStruct({"certData": SimStruct({"size": SimTypeInt(signed=False, label="UInt32"), "data": SimTypePointer(SimTypeChar(label="Byte"), offset=0)}, name="FWP_BYTE_BLOB", pack=False, align=None), "flags": SimTypeInt(signed=False, label="IKEEXT_CERT_FLAGS")}, name="IKEEXT_CERT_ROOT_CONFIG0", pack=False, align=None), offset=0)}, name="_Anonymous_e__Struct", pack=False, align=None), "inboundEnterpriseStoreConfig": SimTypePointer(SimStruct({"certData": SimStruct({"size": SimTypeInt(signed=False, label="UInt32"), "data": SimTypePointer(SimTypeChar(label="Byte"), offset=0)}, name="FWP_BYTE_BLOB", pack=False, align=None), "flags": SimTypeInt(signed=False, label="IKEEXT_CERT_FLAGS")}, name="IKEEXT_CERT_ROOT_CONFIG0", pack=False, align=None), offset=0), "inboundTrustedRootStoreConfig": SimTypePointer(SimStruct({"certData": SimStruct({"size": SimTypeInt(signed=False, label="UInt32"), "data": SimTypePointer(SimTypeChar(label="Byte"), offset=0)}, name="FWP_BYTE_BLOB", pack=False, align=None), "flags": SimTypeInt(signed=False, label="IKEEXT_CERT_FLAGS")}, name="IKEEXT_CERT_ROOT_CONFIG0", pack=False, align=None), offset=0)}, name="<anon>", label="None"), "outboundConfigType": SimTypeInt(signed=False, label="IKEEXT_CERT_CONFIG_TYPE"), "Anonymous2": SimUnion({"Anonymous": SimStruct({"outboundRootArraySize": SimTypeInt(signed=False, label="UInt32"), "outboundRootArray": SimTypePointer(SimStruct({"certData": SimStruct({"size": SimTypeInt(signed=False, label="UInt32"), "data": SimTypePointer(SimTypeChar(label="Byte"), offset=0)}, name="FWP_BYTE_BLOB", pack=False, align=None), "flags": SimTypeInt(signed=False, label="IKEEXT_CERT_FLAGS")}, name="IKEEXT_CERT_ROOT_CONFIG0", pack=False, align=None), offset=0)}, name="_Anonymous_e__Struct", pack=False, align=None), "outboundEnterpriseStoreConfig": SimTypePointer(SimStruct({"certData": SimStruct({"size": SimTypeInt(signed=False, label="UInt32"), "data": SimTypePointer(SimTypeChar(label="Byte"), offset=0)}, name="FWP_BYTE_BLOB", pack=False, align=None), "flags": SimTypeInt(signed=False, label="IKEEXT_CERT_FLAGS")}, name="IKEEXT_CERT_ROOT_CONFIG0", pack=False, align=None), offset=0), "outboundTrustedRootStoreConfig": SimTypePointer(SimStruct({"certData": SimStruct({"size": SimTypeInt(signed=False, label="UInt32"), "data": SimTypePointer(SimTypeChar(label="Byte"), offset=0)}, name="FWP_BYTE_BLOB", pack=False, align=None), "flags": SimTypeInt(signed=False, label="IKEEXT_CERT_FLAGS")}, name="IKEEXT_CERT_ROOT_CONFIG0", pack=False, align=None), offset=0)}, name="<anon>", label="None"), "flags": SimTypeInt(signed=False, label="IKEEXT_CERT_AUTH"), "localCertLocationUrl": SimStruct({"size": SimTypeInt(signed=False, label="UInt32"), "data": SimTypePointer(SimTypeChar(label="Byte"), offset=0)}, name="FWP_BYTE_BLOB", pack=False, align=None)}, name="IKEEXT_CERTIFICATE_AUTHENTICATION1", pack=False, align=None), "kerberosAuthentication": SimStruct({"flags": SimTypeInt(signed=False, label="IKEEXT_KERBEROS_AUTHENTICATION_FLAGS")}, name="IKEEXT_KERBEROS_AUTHENTICATION0", pack=False, align=None), "ntlmV2Authentication": SimStruct({"flags": SimTypeInt(signed=False, label="UInt32")}, name="IKEEXT_NTLM_V2_AUTHENTICATION0", pack=False, align=None), "sslAuthentication": SimStruct({"inboundConfigType": SimTypeInt(signed=False, label="IKEEXT_CERT_CONFIG_TYPE"), "Anonymous1": SimUnion({"Anonymous": SimStruct({"inboundRootArraySize": SimTypeInt(signed=False, label="UInt32"), "inboundRootArray": SimTypePointer(SimStruct({"certData": SimStruct({"size": SimTypeInt(signed=False, label="UInt32"), "data": SimTypePointer(SimTypeChar(label="Byte"), offset=0)}, name="FWP_BYTE_BLOB", pack=False, align=None), "flags": SimTypeInt(signed=False, label="IKEEXT_CERT_FLAGS")}, name="IKEEXT_CERT_ROOT_CONFIG0", pack=False, align=None), offset=0)}, name="_Anonymous_e__Struct", pack=False, align=None), "inboundEnterpriseStoreConfig": SimTypePointer(SimStruct({"certData": SimStruct({"size": SimTypeInt(signed=False, label="UInt32"), "data": SimTypePointer(SimTypeChar(label="Byte"), offset=0)}, name="FWP_BYTE_BLOB", pack=False, align=None), "flags": SimTypeInt(signed=False, label="IKEEXT_CERT_FLAGS")}, name="IKEEXT_CERT_ROOT_CONFIG0", pack=False, align=None), offset=0), "inboundTrustedRootStoreConfig": SimTypePointer(SimStruct({"certData": SimStruct({"size": SimTypeInt(signed=False, label="UInt32"), "data": SimTypePointer(SimTypeChar(label="Byte"), offset=0)}, name="FWP_BYTE_BLOB", pack=False, align=None), "flags": SimTypeInt(signed=False, label="IKEEXT_CERT_FLAGS")}, name="IKEEXT_CERT_ROOT_CONFIG0", pack=False, align=None), offset=0)}, name="<anon>", label="None"), "outboundConfigType": SimTypeInt(signed=False, label="IKEEXT_CERT_CONFIG_TYPE"), "Anonymous2": SimUnion({"Anonymous": SimStruct({"outboundRootArraySize": SimTypeInt(signed=False, label="UInt32"), "outboundRootArray": SimTypePointer(SimStruct({"certData": SimStruct({"size": SimTypeInt(signed=False, label="UInt32"), "data": SimTypePointer(SimTypeChar(label="Byte"), offset=0)}, name="FWP_BYTE_BLOB", pack=False, align=None), "flags": SimTypeInt(signed=False, label="IKEEXT_CERT_FLAGS")}, name="IKEEXT_CERT_ROOT_CONFIG0", pack=False, align=None), offset=0)}, name="_Anonymous_e__Struct", pack=False, align=None), "outboundEnterpriseStoreConfig": SimTypePointer(SimStruct({"certData": SimStruct({"size": SimTypeInt(signed=False, label="UInt32"), "data": SimTypePointer(SimTypeChar(label="Byte"), offset=0)}, name="FWP_BYTE_BLOB", pack=False, align=None), "flags": SimTypeInt(signed=False, label="IKEEXT_CERT_FLAGS")}, name="IKEEXT_CERT_ROOT_CONFIG0", pack=False, align=None), offset=0), "outboundTrustedRootStoreConfig": SimTypePointer(SimStruct({"certData": SimStruct({"size": SimTypeInt(signed=False, label="UInt32"), "data": SimTypePointer(SimTypeChar(label="Byte"), offset=0)}, name="FWP_BYTE_BLOB", pack=False, align=None), "flags": SimTypeInt(signed=False, label="IKEEXT_CERT_FLAGS")}, name="IKEEXT_CERT_ROOT_CONFIG0", pack=False, align=None), offset=0)}, name="<anon>", label="None"), "flags": SimTypeInt(signed=False, label="IKEEXT_CERT_AUTH"), "localCertLocationUrl": SimStruct({"size": SimTypeInt(signed=False, label="UInt32"), "data": SimTypePointer(SimTypeChar(label="Byte"), offset=0)}, name="FWP_BYTE_BLOB", pack=False, align=None)}, name="IKEEXT_CERTIFICATE_AUTHENTICATION1", pack=False, align=None), "cgaAuthentication": SimStruct({"keyContainerName": SimTypePointer(SimTypeChar(label="Char"), offset=0), "cspName": SimTypePointer(SimTypeChar(label="Char"), offset=0), "cspType": SimTypeInt(signed=False, label="UInt32"), "cgaModifier": SimStruct({"byteArray16": SimTypeFixedSizeArray(SimTypeChar(label="Byte"), 16)}, name="FWP_BYTE_ARRAY16", pack=False, align=None), "cgaCollisionCount": SimTypeChar(label="Byte")}, name="IKEEXT_IPV6_CGA_AUTHENTICATION0", pack=False, align=None), "eapAuthentication": SimStruct({"flags": SimTypeInt(signed=False, label="IKEEXT_EAP_AUTHENTICATION_FLAGS")}, name="IKEEXT_EAP_AUTHENTICATION0", pack=False, align=None)}, name="<anon>", label="None")}, name="IKEEXT_AUTHENTICATION_METHOD1", pack=False, align=None), offset=0), "initiatorImpersonationType": SimTypeInt(signed=False, label="IKEEXT_AUTHENTICATION_IMPERSONATION_TYPE"), "numIkeProposals": SimTypeInt(signed=False, label="UInt32"), "ikeProposals": SimTypePointer(SimStruct({"cipherAlgorithm": SimStruct({"algoIdentifier": SimTypeInt(signed=False, label="IKEEXT_CIPHER_TYPE"), "keyLen": SimTypeInt(signed=False, label="UInt32"), "rounds": SimTypeInt(signed=False, label="UInt32")}, name="IKEEXT_CIPHER_ALGORITHM0", pack=False, align=None), "integrityAlgorithm": SimStruct({"algoIdentifier": SimTypeInt(signed=False, label="IKEEXT_INTEGRITY_TYPE")}, name="IKEEXT_INTEGRITY_ALGORITHM0", pack=False, align=None), "maxLifetimeSeconds": SimTypeInt(signed=False, label="UInt32"), "dhGroup": SimTypeInt(signed=False, label="IKEEXT_DH_GROUP"), "quickModeLimit": SimTypeInt(signed=False, label="UInt32")}, name="IKEEXT_PROPOSAL0", pack=False, align=None), offset=0), "flags": SimTypeInt(signed=False, label="IKEEXT_POLICY_FLAG"), "maxDynamicFilters": SimTypeInt(signed=False, label="UInt32"), "retransmitDurationSecs": SimTypeInt(signed=False, label="UInt32")}, name="IKEEXT_POLICY1", pack=False, align=None), offset=0), "authIpMmPolicy": SimTypePointer(SimStruct({"softExpirationTime": SimTypeInt(signed=False, label="UInt32"), "numAuthenticationMethods": SimTypeInt(signed=False, label="UInt32"), "authenticationMethods": SimTypePointer(SimStruct({"authenticationMethodType": SimTypeInt(signed=False, label="IKEEXT_AUTHENTICATION_METHOD_TYPE"), "Anonymous": SimUnion({"presharedKeyAuthentication": SimStruct({"presharedKey": SimStruct({"size": SimTypeInt(signed=False, label="UInt32"), "data": SimTypePointer(SimTypeChar(label="Byte"), offset=0)}, name="FWP_BYTE_BLOB", pack=False, align=None), "flags": SimTypeInt(signed=False, label="IKEEXT_PRESHARED_KEY_AUTHENTICATION_FLAGS")}, name="IKEEXT_PRESHARED_KEY_AUTHENTICATION1", pack=False, align=None), "certificateAuthentication": SimStruct({"inboundConfigType": SimTypeInt(signed=False, label="IKEEXT_CERT_CONFIG_TYPE"), "Anonymous1": SimUnion({"Anonymous": SimStruct({"inboundRootArraySize": SimTypeInt(signed=False, label="UInt32"), "inboundRootArray": SimTypePointer(SimStruct({"certData": SimStruct({"size": SimTypeInt(signed=False, label="UInt32"), "data": SimTypePointer(SimTypeChar(label="Byte"), offset=0)}, name="FWP_BYTE_BLOB", pack=False, align=None), "flags": SimTypeInt(signed=False, label="IKEEXT_CERT_FLAGS")}, name="IKEEXT_CERT_ROOT_CONFIG0", pack=False, align=None), offset=0)}, name="_Anonymous_e__Struct", pack=False, align=None), "inboundEnterpriseStoreConfig": SimTypePointer(SimStruct({"certData": SimStruct({"size": SimTypeInt(signed=False, label="UInt32"), "data": SimTypePointer(SimTypeChar(label="Byte"), offset=0)}, name="FWP_BYTE_BLOB", pack=False, align=None), "flags": SimTypeInt(signed=False, label="IKEEXT_CERT_FLAGS")}, name="IKEEXT_CERT_ROOT_CONFIG0", pack=False, align=None), offset=0), "inboundTrustedRootStoreConfig": SimTypePointer(SimStruct({"certData": SimStruct({"size": SimTypeInt(signed=False, label="UInt32"), "data": SimTypePointer(SimTypeChar(label="Byte"), offset=0)}, name="FWP_BYTE_BLOB", pack=False, align=None), "flags": SimTypeInt(signed=False, label="IKEEXT_CERT_FLAGS")}, name="IKEEXT_CERT_ROOT_CONFIG0", pack=False, align=None), offset=0)}, name="<anon>", label="None"), "outboundConfigType": SimTypeInt(signed=False, label="IKEEXT_CERT_CONFIG_TYPE"), "Anonymous2": SimUnion({"Anonymous": SimStruct({"outboundRootArraySize": SimTypeInt(signed=False, label="UInt32"), "outboundRootArray": SimTypePointer(SimStruct({"certData": SimStruct({"size": SimTypeInt(signed=False, label="UInt32"), "data": SimTypePointer(SimTypeChar(label="Byte"), offset=0)}, name="FWP_BYTE_BLOB", pack=False, align=None), "flags": SimTypeInt(signed=False, label="IKEEXT_CERT_FLAGS")}, name="IKEEXT_CERT_ROOT_CONFIG0", pack=False, align=None), offset=0)}, name="_Anonymous_e__Struct", pack=False, align=None), "outboundEnterpriseStoreConfig": SimTypePointer(SimStruct({"certData": SimStruct({"size": SimTypeInt(signed=False, label="UInt32"), "data": SimTypePointer(SimTypeChar(label="Byte"), offset=0)}, name="FWP_BYTE_BLOB", pack=False, align=None), "flags": SimTypeInt(signed=False, label="IKEEXT_CERT_FLAGS")}, name="IKEEXT_CERT_ROOT_CONFIG0", pack=False, align=None), offset=0), "outboundTrustedRootStoreConfig": SimTypePointer(SimStruct({"certData": SimStruct({"size": SimTypeInt(signed=False, label="UInt32"), "data": SimTypePointer(SimTypeChar(label="Byte"), offset=0)}, name="FWP_BYTE_BLOB", pack=False, align=None), "flags": SimTypeInt(signed=False, label="IKEEXT_CERT_FLAGS")}, name="IKEEXT_CERT_ROOT_CONFIG0", pack=False, align=None), offset=0)}, name="<anon>", label="None"), "flags": SimTypeInt(signed=False, label="IKEEXT_CERT_AUTH"), "localCertLocationUrl": SimStruct({"size": SimTypeInt(signed=False, label="UInt32"), "data": SimTypePointer(SimTypeChar(label="Byte"), offset=0)}, name="FWP_BYTE_BLOB", pack=False, align=None)}, name="IKEEXT_CERTIFICATE_AUTHENTICATION1", pack=False, align=None), "kerberosAuthentication": SimStruct({"flags": SimTypeInt(signed=False, label="IKEEXT_KERBEROS_AUTHENTICATION_FLAGS")}, name="IKEEXT_KERBEROS_AUTHENTICATION0", pack=False, align=None), "ntlmV2Authentication": SimStruct({"flags": SimTypeInt(signed=False, label="UInt32")}, name="IKEEXT_NTLM_V2_AUTHENTICATION0", pack=False, align=None), "sslAuthentication": SimStruct({"inboundConfigType": SimTypeInt(signed=False, label="IKEEXT_CERT_CONFIG_TYPE"), "Anonymous1": SimUnion({"Anonymous": SimStruct({"inboundRootArraySize": SimTypeInt(signed=False, label="UInt32"), "inboundRootArray": SimTypePointer(SimStruct({"certData": SimStruct({"size": SimTypeInt(signed=False, label="UInt32"), "data": SimTypePointer(SimTypeChar(label="Byte"), offset=0)}, name="FWP_BYTE_BLOB", pack=False, align=None), "flags": SimTypeInt(signed=False, label="IKEEXT_CERT_FLAGS")}, name="IKEEXT_CERT_ROOT_CONFIG0", pack=False, align=None), offset=0)}, name="_Anonymous_e__Struct", pack=False, align=None), "inboundEnterpriseStoreConfig": SimTypePointer(SimStruct({"certData": SimStruct({"size": SimTypeInt(signed=False, label="UInt32"), "data": SimTypePointer(SimTypeChar(label="Byte"), offset=0)}, name="FWP_BYTE_BLOB", pack=False, align=None), "flags": SimTypeInt(signed=False, label="IKEEXT_CERT_FLAGS")}, name="IKEEXT_CERT_ROOT_CONFIG0", pack=False, align=None), offset=0), "inboundTrustedRootStoreConfig": SimTypePointer(SimStruct({"certData": SimStruct({"size": SimTypeInt(signed=False, label="UInt32"), "data": SimTypePointer(SimTypeChar(label="Byte"), offset=0)}, name="FWP_BYTE_BLOB", pack=False, align=None), "flags": SimTypeInt(signed=False, label="IKEEXT_CERT_FLAGS")}, name="IKEEXT_CERT_ROOT_CONFIG0", pack=False, align=None), offset=0)}, name="<anon>", label="None"), "outboundConfigType": SimTypeInt(signed=False, label="IKEEXT_CERT_CONFIG_TYPE"), "Anonymous2": SimUnion({"Anonymous": SimStruct({"outboundRootArraySize": SimTypeInt(signed=False, label="UInt32"), "outboundRootArray": SimTypePointer(SimStruct({"certData": SimStruct({"size": SimTypeInt(signed=False, label="UInt32"), "data": SimTypePointer(SimTypeChar(label="Byte"), offset=0)}, name="FWP_BYTE_BLOB", pack=False, align=None), "flags": SimTypeInt(signed=False, label="IKEEXT_CERT_FLAGS")}, name="IKEEXT_CERT_ROOT_CONFIG0", pack=False, align=None), offset=0)}, name="_Anonymous_e__Struct", pack=False, align=None), "outboundEnterpriseStoreConfig": SimTypePointer(SimStruct({"certData": SimStruct({"size": SimTypeInt(signed=False, label="UInt32"), "data": SimTypePointer(SimTypeChar(label="Byte"), offset=0)}, name="FWP_BYTE_BLOB", pack=False, align=None), "flags": SimTypeInt(signed=False, label="IKEEXT_CERT_FLAGS")}, name="IKEEXT_CERT_ROOT_CONFIG0", pack=False, align=None), offset=0), "outboundTrustedRootStoreConfig": SimTypePointer(SimStruct({"certData": SimStruct({"size": SimTypeInt(signed=False, label="UInt32"), "data": SimTypePointer(SimTypeChar(label="Byte"), offset=0)}, name="FWP_BYTE_BLOB", pack=False, align=None), "flags": SimTypeInt(signed=False, label="IKEEXT_CERT_FLAGS")}, name="IKEEXT_CERT_ROOT_CONFIG0", pack=False, align=None), offset=0)}, name="<anon>", label="None"), "flags": SimTypeInt(signed=False, label="IKEEXT_CERT_AUTH"), "localCertLocationUrl": SimStruct({"size": SimTypeInt(signed=False, label="UInt32"), "data": SimTypePointer(SimTypeChar(label="Byte"), offset=0)}, name="FWP_BYTE_BLOB", pack=False, align=None)}, name="IKEEXT_CERTIFICATE_AUTHENTICATION1", pack=False, align=None), "cgaAuthentication": SimStruct({"keyContainerName": SimTypePointer(SimTypeChar(label="Char"), offset=0), "cspName": SimTypePointer(SimTypeChar(label="Char"), offset=0), "cspType": SimTypeInt(signed=False, label="UInt32"), "cgaModifier": SimStruct({"byteArray16": SimTypeFixedSizeArray(SimTypeChar(label="Byte"), 16)}, name="FWP_BYTE_ARRAY16", pack=False, align=None), "cgaCollisionCount": SimTypeChar(label="Byte")}, name="IKEEXT_IPV6_CGA_AUTHENTICATION0", pack=False, align=None), "eapAuthentication": SimStruct({"flags": SimTypeInt(signed=False, label="IKEEXT_EAP_AUTHENTICATION_FLAGS")}, name="IKEEXT_EAP_AUTHENTICATION0", pack=False, align=None)}, name="<anon>", label="None")}, name="IKEEXT_AUTHENTICATION_METHOD1", pack=False, align=None), offset=0), "initiatorImpersonationType": SimTypeInt(signed=False, label="IKEEXT_AUTHENTICATION_IMPERSONATION_TYPE"), "numIkeProposals": SimTypeInt(signed=False, label="UInt32"), "ikeProposals": SimTypePointer(SimStruct({"cipherAlgorithm": SimStruct({"algoIdentifier": SimTypeInt(signed=False, label="IKEEXT_CIPHER_TYPE"), "keyLen": SimTypeInt(signed=False, label="UInt32"), "rounds": SimTypeInt(signed=False, label="UInt32")}, name="IKEEXT_CIPHER_ALGORITHM0", pack=False, align=None), "integrityAlgorithm": SimStruct({"algoIdentifier": SimTypeInt(signed=False, label="IKEEXT_INTEGRITY_TYPE")}, name="IKEEXT_INTEGRITY_ALGORITHM0", pack=False, align=None), "maxLifetimeSeconds": SimTypeInt(signed=False, label="UInt32"), "dhGroup": SimTypeInt(signed=False, label="IKEEXT_DH_GROUP"), "quickModeLimit": SimTypeInt(signed=False, label="UInt32")}, name="IKEEXT_PROPOSAL0", pack=False, align=None), offset=0), "flags": SimTypeInt(signed=False, label="IKEEXT_POLICY_FLAG"), "maxDynamicFilters": SimTypeInt(signed=False, label="UInt32"), "retransmitDurationSecs": SimTypeInt(signed=False, label="UInt32")}, name="IKEEXT_POLICY1", pack=False, align=None), offset=0), "dataBuffer": SimTypePointer(SimStruct({"size": SimTypeInt(signed=False, label="UInt32"), "data": SimTypePointer(SimTypeChar(label="Byte"), offset=0)}, name="FWP_BYTE_BLOB", pack=False, align=None), offset=0), "classifyOptions": SimTypePointer(SimStruct({"numOptions": SimTypeInt(signed=False, label="UInt32"), "options": SimTypePointer(SimStruct({"type": SimTypeInt(signed=False, label="FWP_CLASSIFY_OPTION_TYPE"), "value": SimStruct({"type": SimTypeInt(signed=False, label="FWP_DATA_TYPE"), "Anonymous": SimUnion({"uint8": SimTypeChar(label="Byte"), "uint16": SimTypeShort(signed=False, label="UInt16"), "uint32": SimTypeInt(signed=False, label="UInt32"), "uint64": SimTypePointer(SimTypeLongLong(signed=False, label="UInt64"), offset=0), "int8": SimTypeChar(label="SByte"), "int16": SimTypeShort(signed=True, label="Int16"), "int32": SimTypeInt(signed=True, label="Int32"), "int64": SimTypePointer(SimTypeLongLong(signed=True, label="Int64"), offset=0), "float32": SimTypeFloat(size=32), "double64": SimTypePointer(SimTypeFloat(size=64), offset=0), "byteArray16": SimTypePointer(SimStruct({"byteArray16": SimTypeFixedSizeArray(SimTypeChar(label="Byte"), 16)}, name="FWP_BYTE_ARRAY16", pack=False, align=None), offset=0), "byteBlob": SimTypePointer(SimStruct({"size": SimTypeInt(signed=False, label="UInt32"), "data": SimTypePointer(SimTypeChar(label="Byte"), offset=0)}, name="FWP_BYTE_BLOB", pack=False, align=None), offset=0), "sid": SimTypePointer(SimTypeBottom(label="SID"), offset=0), "sd": SimTypePointer(SimStruct({"size": SimTypeInt(signed=False, label="UInt32"), "data": SimTypePointer(SimTypeChar(label="Byte"), offset=0)}, name="FWP_BYTE_BLOB", pack=False, align=None), offset=0), "tokenInformation": SimTypePointer(SimStruct({"sidCount": SimTypeInt(signed=False, label="UInt32"), "sids": SimTypePointer(SimTypeBottom(label="SID_AND_ATTRIBUTES"), offset=0), "restrictedSidCount": SimTypeInt(signed=False, label="UInt32"), "restrictedSids": SimTypePointer(SimTypeBottom(label="SID_AND_ATTRIBUTES"), offset=0)}, name="FWP_TOKEN_INFORMATION", pack=False, align=None), offset=0), "tokenAccessInformation": SimTypePointer(SimStruct({"size": SimTypeInt(signed=False, label="UInt32"), "data": SimTypePointer(SimTypeChar(label="Byte"), offset=0)}, name="FWP_BYTE_BLOB", pack=False, align=None), offset=0), "unicodeString": SimTypePointer(SimTypeChar(label="Char"), offset=0), "byteArray6": SimTypePointer(SimStruct({"byteArray6": SimTypeFixedSizeArray(SimTypeChar(label="Byte"), 6)}, name="FWP_BYTE_ARRAY6", pack=False, align=None), offset=0), "bitmapArray64": SimTypePointer(SimStruct({"bitmapArray64": SimTypeFixedSizeArray(SimTypeChar(label="Byte"), 8)}, name="FWP_BITMAP_ARRAY64_", pack=False, align=None), offset=0)}, name="<anon>", label="None")}, name="FWP_VALUE0", pack=False, align=None)}, name="FWPM_CLASSIFY_OPTION0", pack=False, align=None), offset=0)}, name="FWPM_CLASSIFY_OPTIONS0", pack=False, align=None), offset=0), "ikeV2QmTunnelPolicy": SimTypePointer(SimStruct({"flags": SimTypeInt(signed=False, label="IPSEC_POLICY_FLAG"), "numIpsecProposals": SimTypeInt(signed=False, label="UInt32"), "ipsecProposals": SimTypePointer(SimStruct({"lifetime": SimStruct({"lifetimeSeconds": SimTypeInt(signed=False, label="UInt32"), "lifetimeKilobytes": SimTypeInt(signed=False, label="UInt32"), "lifetimePackets": SimTypeInt(signed=False, label="UInt32")}, name="IPSEC_SA_LIFETIME0", pack=False, align=None), "numSaTransforms": SimTypeInt(signed=False, label="UInt32"), "saTransforms": SimTypePointer(SimStruct({"ipsecTransformType": SimTypeInt(signed=False, label="IPSEC_TRANSFORM_TYPE"), "Anonymous": SimUnion({"ahTransform": SimTypePointer(SimStruct({"authTransformId": SimStruct({"authType": SimTypeInt(signed=False, label="IPSEC_AUTH_TYPE"), "authConfig": SimTypeChar(label="Byte")}, name="IPSEC_AUTH_TRANSFORM_ID0", pack=False, align=None), "cryptoModuleId": SimTypePointer(SimTypeBottom(label="Guid"), offset=0)}, name="IPSEC_AUTH_TRANSFORM0", pack=False, align=None), offset=0), "espAuthTransform": SimTypePointer(SimStruct({"authTransformId": SimStruct({"authType": SimTypeInt(signed=False, label="IPSEC_AUTH_TYPE"), "authConfig": SimTypeChar(label="Byte")}, name="IPSEC_AUTH_TRANSFORM_ID0", pack=False, align=None), "cryptoModuleId": SimTypePointer(SimTypeBottom(label="Guid"), offset=0)}, name="IPSEC_AUTH_TRANSFORM0", pack=False, align=None), offset=0), "espCipherTransform": SimTypePointer(SimStruct({"cipherTransformId": SimStruct({"cipherType": SimTypeInt(signed=False, label="IPSEC_CIPHER_TYPE"), "cipherConfig": SimTypeChar(label="Byte")}, name="IPSEC_CIPHER_TRANSFORM_ID0", pack=False, align=None), "cryptoModuleId": SimTypePointer(SimTypeBottom(label="Guid"), offset=0)}, name="IPSEC_CIPHER_TRANSFORM0", pack=False, align=None), offset=0), "espAuthAndCipherTransform": SimTypePointer(SimStruct({"authTransform": SimStruct({"authTransformId": SimStruct({"authType": SimTypeInt(signed=False, label="IPSEC_AUTH_TYPE"), "authConfig": SimTypeChar(label="Byte")}, name="IPSEC_AUTH_TRANSFORM_ID0", pack=False, align=None), "cryptoModuleId": SimTypePointer(SimTypeBottom(label="Guid"), offset=0)}, name="IPSEC_AUTH_TRANSFORM0", pack=False, align=None), "cipherTransform": SimStruct({"cipherTransformId": SimStruct({"cipherType": SimTypeInt(signed=False, label="IPSEC_CIPHER_TYPE"), "cipherConfig": SimTypeChar(label="Byte")}, name="IPSEC_CIPHER_TRANSFORM_ID0", pack=False, align=None), "cryptoModuleId": SimTypePointer(SimTypeBottom(label="Guid"), offset=0)}, name="IPSEC_CIPHER_TRANSFORM0", pack=False, align=None)}, name="IPSEC_AUTH_AND_CIPHER_TRANSFORM0", pack=False, align=None), offset=0), "espAuthFwTransform": SimTypePointer(SimStruct({"authTransformId": SimStruct({"authType": SimTypeInt(signed=False, label="IPSEC_AUTH_TYPE"), "authConfig": SimTypeChar(label="Byte")}, name="IPSEC_AUTH_TRANSFORM_ID0", pack=False, align=None), "cryptoModuleId": SimTypePointer(SimTypeBottom(label="Guid"), offset=0)}, name="IPSEC_AUTH_TRANSFORM0", pack=False, align=None), offset=0)}, name="<anon>", label="None")}, name="IPSEC_SA_TRANSFORM0", pack=False, align=None), offset=0), "pfsGroup": SimTypeInt(signed=False, label="IPSEC_PFS_GROUP")}, name="IPSEC_PROPOSAL0", pack=False, align=None), offset=0), "tunnelEndpoints": SimStruct({"ipVersion": SimTypeInt(signed=False, label="FWP_IP_VERSION"), "Anonymous1": SimUnion({"localV4Address": SimTypeInt(signed=False, label="UInt32"), "localV6Address": SimTypeFixedSizeArray(SimTypeChar(label="Byte"), 16)}, name="<anon>", label="None"), "Anonymous2": SimUnion({"remoteV4Address": SimTypeInt(signed=False, label="UInt32"), "remoteV6Address": SimTypeFixedSizeArray(SimTypeChar(label="Byte"), 16)}, name="<anon>", label="None"), "localIfLuid": SimTypeLongLong(signed=False, label="UInt64")}, name="IPSEC_TUNNEL_ENDPOINTS1", pack=False, align=None), "saIdleTimeout": SimStruct({"idleTimeoutSeconds": SimTypeInt(signed=False, label="UInt32"), "idleTimeoutSecondsFailOver": SimTypeInt(signed=False, label="UInt32")}, name="IPSEC_SA_IDLE_TIMEOUT0", pack=False, align=None), "emPolicy": SimTypePointer(SimStruct({"numAuthenticationMethods": SimTypeInt(signed=False, label="UInt32"), "authenticationMethods": SimTypePointer(SimStruct({"authenticationMethodType": SimTypeInt(signed=False, label="IKEEXT_AUTHENTICATION_METHOD_TYPE"), "Anonymous": SimUnion({"presharedKeyAuthentication": SimStruct({"presharedKey": SimStruct({"size": SimTypeInt(signed=False, label="UInt32"), "data": SimTypePointer(SimTypeChar(label="Byte"), offset=0)}, name="FWP_BYTE_BLOB", pack=False, align=None), "flags": SimTypeInt(signed=False, label="IKEEXT_PRESHARED_KEY_AUTHENTICATION_FLAGS")}, name="IKEEXT_PRESHARED_KEY_AUTHENTICATION1", pack=False, align=None), "certificateAuthentication": SimStruct({"inboundConfigType": SimTypeInt(signed=False, label="IKEEXT_CERT_CONFIG_TYPE"), "Anonymous1": SimUnion({"Anonymous": SimStruct({"inboundRootArraySize": SimTypeInt(signed=False, label="UInt32"), "inboundRootArray": SimTypePointer(SimStruct({"certData": SimStruct({"size": SimTypeInt(signed=False, label="UInt32"), "data": SimTypePointer(SimTypeChar(label="Byte"), offset=0)}, name="FWP_BYTE_BLOB", pack=False, align=None), "flags": SimTypeInt(signed=False, label="IKEEXT_CERT_FLAGS")}, name="IKEEXT_CERT_ROOT_CONFIG0", pack=False, align=None), offset=0)}, name="_Anonymous_e__Struct", pack=False, align=None), "inboundEnterpriseStoreConfig": SimTypePointer(SimStruct({"certData": SimStruct({"size": SimTypeInt(signed=False, label="UInt32"), "data": SimTypePointer(SimTypeChar(label="Byte"), offset=0)}, name="FWP_BYTE_BLOB", pack=False, align=None), "flags": SimTypeInt(signed=False, label="IKEEXT_CERT_FLAGS")}, name="IKEEXT_CERT_ROOT_CONFIG0", pack=False, align=None), offset=0), "inboundTrustedRootStoreConfig": SimTypePointer(SimStruct({"certData": SimStruct({"size": SimTypeInt(signed=False, label="UInt32"), "data": SimTypePointer(SimTypeChar(label="Byte"), offset=0)}, name="FWP_BYTE_BLOB", pack=False, align=None), "flags": SimTypeInt(signed=False, label="IKEEXT_CERT_FLAGS")}, name="IKEEXT_CERT_ROOT_CONFIG0", pack=False, align=None), offset=0)}, name="<anon>", label="None"), "outboundConfigType": SimTypeInt(signed=False, label="IKEEXT_CERT_CONFIG_TYPE"), "Anonymous2": SimUnion({"Anonymous": SimStruct({"outboundRootArraySize": SimTypeInt(signed=False, label="UInt32"), "outboundRootArray": SimTypePointer(SimStruct({"certData": SimStruct({"size": SimTypeInt(signed=False, label="UInt32"), "data": SimTypePointer(SimTypeChar(label="Byte"), offset=0)}, name="FWP_BYTE_BLOB", pack=False, align=None), "flags": SimTypeInt(signed=False, label="IKEEXT_CERT_FLAGS")}, name="IKEEXT_CERT_ROOT_CONFIG0", pack=False, align=None), offset=0)}, name="_Anonymous_e__Struct", pack=False, align=None), "outboundEnterpriseStoreConfig": SimTypePointer(SimStruct({"certData": SimStruct({"size": SimTypeInt(signed=False, label="UInt32"), "data": SimTypePointer(SimTypeChar(label="Byte"), offset=0)}, name="FWP_BYTE_BLOB", pack=False, align=None), "flags": SimTypeInt(signed=False, label="IKEEXT_CERT_FLAGS")}, name="IKEEXT_CERT_ROOT_CONFIG0", pack=False, align=None), offset=0), "outboundTrustedRootStoreConfig": SimTypePointer(SimStruct({"certData": SimStruct({"size": SimTypeInt(signed=False, label="UInt32"), "data": SimTypePointer(SimTypeChar(label="Byte"), offset=0)}, name="FWP_BYTE_BLOB", pack=False, align=None), "flags": SimTypeInt(signed=False, label="IKEEXT_CERT_FLAGS")}, name="IKEEXT_CERT_ROOT_CONFIG0", pack=False, align=None), offset=0)}, name="<anon>", label="None"), "flags": SimTypeInt(signed=False, label="IKEEXT_CERT_AUTH"), "localCertLocationUrl": SimStruct({"size": SimTypeInt(signed=False, label="UInt32"), "data": SimTypePointer(SimTypeChar(label="Byte"), offset=0)}, name="FWP_BYTE_BLOB", pack=False, align=None)}, name="IKEEXT_CERTIFICATE_AUTHENTICATION1", pack=False, align=None), "kerberosAuthentication": SimStruct({"flags": SimTypeInt(signed=False, label="IKEEXT_KERBEROS_AUTHENTICATION_FLAGS")}, name="IKEEXT_KERBEROS_AUTHENTICATION0", pack=False, align=None), "ntlmV2Authentication": SimStruct({"flags": SimTypeInt(signed=False, label="UInt32")}, name="IKEEXT_NTLM_V2_AUTHENTICATION0", pack=False, align=None), "sslAuthentication": SimStruct({"inboundConfigType": SimTypeInt(signed=False, label="IKEEXT_CERT_CONFIG_TYPE"), "Anonymous1": SimUnion({"Anonymous": SimStruct({"inboundRootArraySize": SimTypeInt(signed=False, label="UInt32"), "inboundRootArray": SimTypePointer(SimStruct({"certData": SimStruct({"size": SimTypeInt(signed=False, label="UInt32"), "data": SimTypePointer(SimTypeChar(label="Byte"), offset=0)}, name="FWP_BYTE_BLOB", pack=False, align=None), "flags": SimTypeInt(signed=False, label="IKEEXT_CERT_FLAGS")}, name="IKEEXT_CERT_ROOT_CONFIG0", pack=False, align=None), offset=0)}, name="_Anonymous_e__Struct", pack=False, align=None), "inboundEnterpriseStoreConfig": SimTypePointer(SimStruct({"certData": SimStruct({"size": SimTypeInt(signed=False, label="UInt32"), "data": SimTypePointer(SimTypeChar(label="Byte"), offset=0)}, name="FWP_BYTE_BLOB", pack=False, align=None), "flags": SimTypeInt(signed=False, label="IKEEXT_CERT_FLAGS")}, name="IKEEXT_CERT_ROOT_CONFIG0", pack=False, align=None), offset=0), "inboundTrustedRootStoreConfig": SimTypePointer(SimStruct({"certData": SimStruct({"size": SimTypeInt(signed=False, label="UInt32"), "data": SimTypePointer(SimTypeChar(label="Byte"), offset=0)}, name="FWP_BYTE_BLOB", pack=False, align=None), "flags": SimTypeInt(signed=False, label="IKEEXT_CERT_FLAGS")}, name="IKEEXT_CERT_ROOT_CONFIG0", pack=False, align=None), offset=0)}, name="<anon>", label="None"), "outboundConfigType": SimTypeInt(signed=False, label="IKEEXT_CERT_CONFIG_TYPE"), "Anonymous2": SimUnion({"Anonymous": SimStruct({"outboundRootArraySize": SimTypeInt(signed=False, label="UInt32"), "outboundRootArray": SimTypePointer(SimStruct({"certData": SimStruct({"size": SimTypeInt(signed=False, label="UInt32"), "data": SimTypePointer(SimTypeChar(label="Byte"), offset=0)}, name="FWP_BYTE_BLOB", pack=False, align=None), "flags": SimTypeInt(signed=False, label="IKEEXT_CERT_FLAGS")}, name="IKEEXT_CERT_ROOT_CONFIG0", pack=False, align=None), offset=0)}, name="_Anonymous_e__Struct", pack=False, align=None), "outboundEnterpriseStoreConfig": SimTypePointer(SimStruct({"certData": SimStruct({"size": SimTypeInt(signed=False, label="UInt32"), "data": SimTypePointer(SimTypeChar(label="Byte"), offset=0)}, name="FWP_BYTE_BLOB", pack=False, align=None), "flags": SimTypeInt(signed=False, label="IKEEXT_CERT_FLAGS")}, name="IKEEXT_CERT_ROOT_CONFIG0", pack=False, align=None), offset=0), "outboundTrustedRootStoreConfig": SimTypePointer(SimStruct({"certData": SimStruct({"size": SimTypeInt(signed=False, label="UInt32"), "data": SimTypePointer(SimTypeChar(label="Byte"), offset=0)}, name="FWP_BYTE_BLOB", pack=False, align=None), "flags": SimTypeInt(signed=False, label="IKEEXT_CERT_FLAGS")}, name="IKEEXT_CERT_ROOT_CONFIG0", pack=False, align=None), offset=0)}, name="<anon>", label="None"), "flags": SimTypeInt(signed=False, label="IKEEXT_CERT_AUTH"), "localCertLocationUrl": SimStruct({"size": SimTypeInt(signed=False, label="UInt32"), "data": SimTypePointer(SimTypeChar(label="Byte"), offset=0)}, name="FWP_BYTE_BLOB", pack=False, align=None)}, name="IKEEXT_CERTIFICATE_AUTHENTICATION1", pack=False, align=None), "cgaAuthentication": SimStruct({"keyContainerName": SimTypePointer(SimTypeChar(label="Char"), offset=0), "cspName": SimTypePointer(SimTypeChar(label="Char"), offset=0), "cspType": SimTypeInt(signed=False, label="UInt32"), "cgaModifier": SimStruct({"byteArray16": SimTypeFixedSizeArray(SimTypeChar(label="Byte"), 16)}, name="FWP_BYTE_ARRAY16", pack=False, align=None), "cgaCollisionCount": SimTypeChar(label="Byte")}, name="IKEEXT_IPV6_CGA_AUTHENTICATION0", pack=False, align=None), "eapAuthentication": SimStruct({"flags": SimTypeInt(signed=False, label="IKEEXT_EAP_AUTHENTICATION_FLAGS")}, name="IKEEXT_EAP_AUTHENTICATION0", pack=False, align=None)}, name="<anon>", label="None")}, name="IKEEXT_AUTHENTICATION_METHOD1", pack=False, align=None), offset=0), "initiatorImpersonationType": SimTypeInt(signed=False, label="IKEEXT_AUTHENTICATION_IMPERSONATION_TYPE")}, name="IKEEXT_EM_POLICY1", pack=False, align=None), offset=0)}, name="IPSEC_TUNNEL_POLICY1", pack=False, align=None), offset=0), "ikeV2MmPolicy": SimTypePointer(SimStruct({"softExpirationTime": SimTypeInt(signed=False, label="UInt32"), "numAuthenticationMethods": SimTypeInt(signed=False, label="UInt32"), "authenticationMethods": SimTypePointer(SimStruct({"authenticationMethodType": SimTypeInt(signed=False, label="IKEEXT_AUTHENTICATION_METHOD_TYPE"), "Anonymous": SimUnion({"presharedKeyAuthentication": SimStruct({"presharedKey": SimStruct({"size": SimTypeInt(signed=False, label="UInt32"), "data": SimTypePointer(SimTypeChar(label="Byte"), offset=0)}, name="FWP_BYTE_BLOB", pack=False, align=None), "flags": SimTypeInt(signed=False, label="IKEEXT_PRESHARED_KEY_AUTHENTICATION_FLAGS")}, name="IKEEXT_PRESHARED_KEY_AUTHENTICATION1", pack=False, align=None), "certificateAuthentication": SimStruct({"inboundConfigType": SimTypeInt(signed=False, label="IKEEXT_CERT_CONFIG_TYPE"), "Anonymous1": SimUnion({"Anonymous": SimStruct({"inboundRootArraySize": SimTypeInt(signed=False, label="UInt32"), "inboundRootArray": SimTypePointer(SimStruct({"certData": SimStruct({"size": SimTypeInt(signed=False, label="UInt32"), "data": SimTypePointer(SimTypeChar(label="Byte"), offset=0)}, name="FWP_BYTE_BLOB", pack=False, align=None), "flags": SimTypeInt(signed=False, label="IKEEXT_CERT_FLAGS")}, name="IKEEXT_CERT_ROOT_CONFIG0", pack=False, align=None), offset=0)}, name="_Anonymous_e__Struct", pack=False, align=None), "inboundEnterpriseStoreConfig": SimTypePointer(SimStruct({"certData": SimStruct({"size": SimTypeInt(signed=False, label="UInt32"), "data": SimTypePointer(SimTypeChar(label="Byte"), offset=0)}, name="FWP_BYTE_BLOB", pack=False, align=None), "flags": SimTypeInt(signed=False, label="IKEEXT_CERT_FLAGS")}, name="IKEEXT_CERT_ROOT_CONFIG0", pack=False, align=None), offset=0), "inboundTrustedRootStoreConfig": SimTypePointer(SimStruct({"certData": SimStruct({"size": SimTypeInt(signed=False, label="UInt32"), "data": SimTypePointer(SimTypeChar(label="Byte"), offset=0)}, name="FWP_BYTE_BLOB", pack=False, align=None), "flags": SimTypeInt(signed=False, label="IKEEXT_CERT_FLAGS")}, name="IKEEXT_CERT_ROOT_CONFIG0", pack=False, align=None), offset=0)}, name="<anon>", label="None"), "outboundConfigType": SimTypeInt(signed=False, label="IKEEXT_CERT_CONFIG_TYPE"), "Anonymous2": SimUnion({"Anonymous": SimStruct({"outboundRootArraySize": SimTypeInt(signed=False, label="UInt32"), "outboundRootArray": SimTypePointer(SimStruct({"certData": SimStruct({"size": SimTypeInt(signed=False, label="UInt32"), "data": SimTypePointer(SimTypeChar(label="Byte"), offset=0)}, name="FWP_BYTE_BLOB", pack=False, align=None), "flags": SimTypeInt(signed=False, label="IKEEXT_CERT_FLAGS")}, name="IKEEXT_CERT_ROOT_CONFIG0", pack=False, align=None), offset=0)}, name="_Anonymous_e__Struct", pack=False, align=None), "outboundEnterpriseStoreConfig": SimTypePointer(SimStruct({"certData": SimStruct({"size": SimTypeInt(signed=False, label="UInt32"), "data": SimTypePointer(SimTypeChar(label="Byte"), offset=0)}, name="FWP_BYTE_BLOB", pack=False, align=None), "flags": SimTypeInt(signed=False, label="IKEEXT_CERT_FLAGS")}, name="IKEEXT_CERT_ROOT_CONFIG0", pack=False, align=None), offset=0), "outboundTrustedRootStoreConfig": SimTypePointer(SimStruct({"certData": SimStruct({"size": SimTypeInt(signed=False, label="UInt32"), "data": SimTypePointer(SimTypeChar(label="Byte"), offset=0)}, name="FWP_BYTE_BLOB", pack=False, align=None), "flags": SimTypeInt(signed=False, label="IKEEXT_CERT_FLAGS")}, name="IKEEXT_CERT_ROOT_CONFIG0", pack=False, align=None), offset=0)}, name="<anon>", label="None"), "flags": SimTypeInt(signed=False, label="IKEEXT_CERT_AUTH"), "localCertLocationUrl": SimStruct({"size": SimTypeInt(signed=False, label="UInt32"), "data": SimTypePointer(SimTypeChar(label="Byte"), offset=0)}, name="FWP_BYTE_BLOB", pack=False, align=None)}, name="IKEEXT_CERTIFICATE_AUTHENTICATION1", pack=False, align=None), "kerberosAuthentication": SimStruct({"flags": SimTypeInt(signed=False, label="IKEEXT_KERBEROS_AUTHENTICATION_FLAGS")}, name="IKEEXT_KERBEROS_AUTHENTICATION0", pack=False, align=None), "ntlmV2Authentication": SimStruct({"flags": SimTypeInt(signed=False, label="UInt32")}, name="IKEEXT_NTLM_V2_AUTHENTICATION0", pack=False, align=None), "sslAuthentication": SimStruct({"inboundConfigType": SimTypeInt(signed=False, label="IKEEXT_CERT_CONFIG_TYPE"), "Anonymous1": SimUnion({"Anonymous": SimStruct({"inboundRootArraySize": SimTypeInt(signed=False, label="UInt32"), "inboundRootArray": SimTypePointer(SimStruct({"certData": SimStruct({"size": SimTypeInt(signed=False, label="UInt32"), "data": SimTypePointer(SimTypeChar(label="Byte"), offset=0)}, name="FWP_BYTE_BLOB", pack=False, align=None), "flags": SimTypeInt(signed=False, label="IKEEXT_CERT_FLAGS")}, name="IKEEXT_CERT_ROOT_CONFIG0", pack=False, align=None), offset=0)}, name="_Anonymous_e__Struct", pack=False, align=None), "inboundEnterpriseStoreConfig": SimTypePointer(SimStruct({"certData": SimStruct({"size": SimTypeInt(signed=False, label="UInt32"), "data": SimTypePointer(SimTypeChar(label="Byte"), offset=0)}, name="FWP_BYTE_BLOB", pack=False, align=None), "flags": SimTypeInt(signed=False, label="IKEEXT_CERT_FLAGS")}, name="IKEEXT_CERT_ROOT_CONFIG0", pack=False, align=None), offset=0), "inboundTrustedRootStoreConfig": SimTypePointer(SimStruct({"certData": SimStruct({"size": SimTypeInt(signed=False, label="UInt32"), "data": SimTypePointer(SimTypeChar(label="Byte"), offset=0)}, name="FWP_BYTE_BLOB", pack=False, align=None), "flags": SimTypeInt(signed=False, label="IKEEXT_CERT_FLAGS")}, name="IKEEXT_CERT_ROOT_CONFIG0", pack=False, align=None), offset=0)}, name="<anon>", label="None"), "outboundConfigType": SimTypeInt(signed=False, label="IKEEXT_CERT_CONFIG_TYPE"), "Anonymous2": SimUnion({"Anonymous": SimStruct({"outboundRootArraySize": SimTypeInt(signed=False, label="UInt32"), "outboundRootArray": SimTypePointer(SimStruct({"certData": SimStruct({"size": SimTypeInt(signed=False, label="UInt32"), "data": SimTypePointer(SimTypeChar(label="Byte"), offset=0)}, name="FWP_BYTE_BLOB", pack=False, align=None), "flags": SimTypeInt(signed=False, label="IKEEXT_CERT_FLAGS")}, name="IKEEXT_CERT_ROOT_CONFIG0", pack=False, align=None), offset=0)}, name="_Anonymous_e__Struct", pack=False, align=None), "outboundEnterpriseStoreConfig": SimTypePointer(SimStruct({"certData": SimStruct({"size": SimTypeInt(signed=False, label="UInt32"), "data": SimTypePointer(SimTypeChar(label="Byte"), offset=0)}, name="FWP_BYTE_BLOB", pack=False, align=None), "flags": SimTypeInt(signed=False, label="IKEEXT_CERT_FLAGS")}, name="IKEEXT_CERT_ROOT_CONFIG0", pack=False, align=None), offset=0), "outboundTrustedRootStoreConfig": SimTypePointer(SimStruct({"certData": SimStruct({"size": SimTypeInt(signed=False, label="UInt32"), "data": SimTypePointer(SimTypeChar(label="Byte"), offset=0)}, name="FWP_BYTE_BLOB", pack=False, align=None), "flags": SimTypeInt(signed=False, label="IKEEXT_CERT_FLAGS")}, name="IKEEXT_CERT_ROOT_CONFIG0", pack=False, align=None), offset=0)}, name="<anon>", label="None"), "flags": SimTypeInt(signed=False, label="IKEEXT_CERT_AUTH"), "localCertLocationUrl": SimStruct({"size": SimTypeInt(signed=False, label="UInt32"), "data": SimTypePointer(SimTypeChar(label="Byte"), offset=0)}, name="FWP_BYTE_BLOB", pack=False, align=None)}, name="IKEEXT_CERTIFICATE_AUTHENTICATION1", pack=False, align=None), "cgaAuthentication": SimStruct({"keyContainerName": SimTypePointer(SimTypeChar(label="Char"), offset=0), "cspName": SimTypePointer(SimTypeChar(label="Char"), offset=0), "cspType": SimTypeInt(signed=False, label="UInt32"), "cgaModifier": SimStruct({"byteArray16": SimTypeFixedSizeArray(SimTypeChar(label="Byte"), 16)}, name="FWP_BYTE_ARRAY16", pack=False, align=None), "cgaCollisionCount": SimTypeChar(label="Byte")}, name="IKEEXT_IPV6_CGA_AUTHENTICATION0", pack=False, align=None), "eapAuthentication": SimStruct({"flags": SimTypeInt(signed=False, label="IKEEXT_EAP_AUTHENTICATION_FLAGS")}, name="IKEEXT_EAP_AUTHENTICATION0", pack=False, align=None)}, name="<anon>", label="None")}, name="IKEEXT_AUTHENTICATION_METHOD1", pack=False, align=None), offset=0), "initiatorImpersonationType": SimTypeInt(signed=False, label="IKEEXT_AUTHENTICATION_IMPERSONATION_TYPE"), "numIkeProposals": SimTypeInt(signed=False, label="UInt32"), "ikeProposals": SimTypePointer(SimStruct({"cipherAlgorithm": SimStruct({"algoIdentifier": SimTypeInt(signed=False, label="IKEEXT_CIPHER_TYPE"), "keyLen": SimTypeInt(signed=False, label="UInt32"), "rounds": SimTypeInt(signed=False, label="UInt32")}, name="IKEEXT_CIPHER_ALGORITHM0", pack=False, align=None), "integrityAlgorithm": SimStruct({"algoIdentifier": SimTypeInt(signed=False, label="IKEEXT_INTEGRITY_TYPE")}, name="IKEEXT_INTEGRITY_ALGORITHM0", pack=False, align=None), "maxLifetimeSeconds": SimTypeInt(signed=False, label="UInt32"), "dhGroup": SimTypeInt(signed=False, label="IKEEXT_DH_GROUP"), "quickModeLimit": SimTypeInt(signed=False, label="UInt32")}, name="IKEEXT_PROPOSAL0", pack=False, align=None), offset=0), "flags": SimTypeInt(signed=False, label="IKEEXT_POLICY_FLAG"), "maxDynamicFilters": SimTypeInt(signed=False, label="UInt32"), "retransmitDurationSecs": SimTypeInt(signed=False, label="UInt32")}, name="IKEEXT_POLICY1", pack=False, align=None), offset=0), "idpOptions": SimTypePointer(SimStruct({"stateIdleTimeoutSeconds": SimTypeInt(signed=False, label="UInt32"), "perIPRateLimitQueueIdleTimeoutSeconds": SimTypeInt(signed=False, label="UInt32"), "ipV6IPsecUnauthDscp": SimTypeChar(label="Byte"), "ipV6IPsecUnauthRateLimitBytesPerSec": SimTypeInt(signed=False, label="UInt32"), "ipV6IPsecUnauthPerIPRateLimitBytesPerSec": SimTypeInt(signed=False, label="UInt32"), "ipV6IPsecAuthDscp": SimTypeChar(label="Byte"), "ipV6IPsecAuthRateLimitBytesPerSec": SimTypeInt(signed=False, label="UInt32"), "icmpV6Dscp": SimTypeChar(label="Byte"), "icmpV6RateLimitBytesPerSec": SimTypeInt(signed=False, label="UInt32"), "ipV6FilterExemptDscp": SimTypeChar(label="Byte"), "ipV6FilterExemptRateLimitBytesPerSec": SimTypeInt(signed=False, label="UInt32"), "defBlockExemptDscp": SimTypeChar(label="Byte"), "defBlockExemptRateLimitBytesPerSec": SimTypeInt(signed=False, label="UInt32"), "maxStateEntries": SimTypeInt(signed=False, label="UInt32"), "maxPerIPRateLimitQueues": SimTypeInt(signed=False, label="UInt32"), "flags": SimTypeInt(signed=False, label="IPSEC_DOSP_FLAGS"), "numPublicIFLuids": SimTypeInt(signed=False, label="UInt32"), "publicIFLuids": SimTypePointer(SimTypeLongLong(signed=False, label="UInt64"), offset=0), "numInternalIFLuids": SimTypeInt(signed=False, label="UInt32"), "internalIFLuids": SimTypePointer(SimTypeLongLong(signed=False, label="UInt64"), offset=0), "publicV6AddrMask": SimStruct({"addr": SimTypeFixedSizeArray(SimTypeChar(label="Byte"), 16), "prefixLength": SimTypeChar(label="Byte")}, name="FWP_V6_ADDR_AND_MASK", pack=False, align=None), "internalV6AddrMask": SimStruct({"addr": SimTypeFixedSizeArray(SimTypeChar(label="Byte"), 16), "prefixLength": SimTypeChar(label="Byte")}, name="FWP_V6_ADDR_AND_MASK", pack=False, align=None)}, name="IPSEC_DOSP_OPTIONS0", pack=False, align=None), offset=0)}, name="<anon>", label="None"), "providerContextId": SimTypeLongLong(signed=False, label="UInt64")}, name="FWPM_PROVIDER_CONTEXT1", pack=False, align=None), offset=0), SimTypePointer(SimStruct({"Revision": SimTypeChar(label="Byte"), "Sbz1": SimTypeChar(label="Byte"), "Control": SimTypeShort(signed=False, label="UInt16"), "Owner": SimTypePointer(SimTypeInt(signed=True, label="Int"), label="IntPtr", offset=0), "Group": SimTypePointer(SimTypeInt(signed=True, label="Int"), label="IntPtr", offset=0), "Sacl": SimTypePointer(SimStruct({"AclRevision": SimTypeChar(label="Byte"), "Sbz1": SimTypeChar(label="Byte"), "AclSize": SimTypeShort(signed=False, label="UInt16"), "AceCount": SimTypeShort(signed=False, label="UInt16"), "Sbz2": SimTypeShort(signed=False, label="UInt16")}, name="ACL", pack=False, align=None), offset=0), "Dacl": SimTypePointer(SimStruct({"AclRevision": SimTypeChar(label="Byte"), "Sbz1": SimTypeChar(label="Byte"), "AclSize": SimTypeShort(signed=False, label="UInt16"), "AceCount": SimTypeShort(signed=False, label="UInt16"), "Sbz2": SimTypeShort(signed=False, label="UInt16")}, name="ACL", pack=False, align=None), offset=0)}, name="SECURITY_DESCRIPTOR", pack=False, align=None), offset=0), SimTypePointer(SimTypeLongLong(signed=False, label="UInt64"), offset=0)], SimTypeInt(signed=False, label="UInt32"), arg_names=["engineHandle", "providerContext", "sd", "id"]),
        # 
        'FwpmProviderContextAdd2': SimTypeFunction([SimTypePointer(SimTypeInt(signed=True, label="Int"), label="IntPtr", offset=0), SimTypePointer(SimStruct({"providerContextKey": SimTypeBottom(label="Guid"), "displayData": SimStruct({"name": SimTypePointer(SimTypeChar(label="Char"), offset=0), "description": SimTypePointer(SimTypeChar(label="Char"), offset=0)}, name="FWPM_DISPLAY_DATA0", pack=False, align=None), "flags": SimTypeInt(signed=False, label="UInt32"), "providerKey": SimTypePointer(SimTypeBottom(label="Guid"), offset=0), "providerData": SimStruct({"size": SimTypeInt(signed=False, label="UInt32"), "data": SimTypePointer(SimTypeChar(label="Byte"), offset=0)}, name="FWP_BYTE_BLOB", pack=False, align=None), "type": SimTypeInt(signed=False, label="FWPM_PROVIDER_CONTEXT_TYPE"), "Anonymous": SimUnion({"keyingPolicy": SimTypePointer(SimStruct({"numKeyMods": SimTypeInt(signed=False, label="UInt32"), "keyModKeys": SimTypePointer(SimTypeBottom(label="Guid"), offset=0), "flags": SimTypeInt(signed=False, label="UInt32")}, name="IPSEC_KEYING_POLICY1", pack=False, align=None), offset=0), "ikeQmTransportPolicy": SimTypePointer(SimStruct({"numIpsecProposals": SimTypeInt(signed=False, label="UInt32"), "ipsecProposals": SimTypePointer(SimStruct({"lifetime": SimStruct({"lifetimeSeconds": SimTypeInt(signed=False, label="UInt32"), "lifetimeKilobytes": SimTypeInt(signed=False, label="UInt32"), "lifetimePackets": SimTypeInt(signed=False, label="UInt32")}, name="IPSEC_SA_LIFETIME0", pack=False, align=None), "numSaTransforms": SimTypeInt(signed=False, label="UInt32"), "saTransforms": SimTypePointer(SimStruct({"ipsecTransformType": SimTypeInt(signed=False, label="IPSEC_TRANSFORM_TYPE"), "Anonymous": SimUnion({"ahTransform": SimTypePointer(SimStruct({"authTransformId": SimStruct({"authType": SimTypeInt(signed=False, label="IPSEC_AUTH_TYPE"), "authConfig": SimTypeChar(label="Byte")}, name="IPSEC_AUTH_TRANSFORM_ID0", pack=False, align=None), "cryptoModuleId": SimTypePointer(SimTypeBottom(label="Guid"), offset=0)}, name="IPSEC_AUTH_TRANSFORM0", pack=False, align=None), offset=0), "espAuthTransform": SimTypePointer(SimStruct({"authTransformId": SimStruct({"authType": SimTypeInt(signed=False, label="IPSEC_AUTH_TYPE"), "authConfig": SimTypeChar(label="Byte")}, name="IPSEC_AUTH_TRANSFORM_ID0", pack=False, align=None), "cryptoModuleId": SimTypePointer(SimTypeBottom(label="Guid"), offset=0)}, name="IPSEC_AUTH_TRANSFORM0", pack=False, align=None), offset=0), "espCipherTransform": SimTypePointer(SimStruct({"cipherTransformId": SimStruct({"cipherType": SimTypeInt(signed=False, label="IPSEC_CIPHER_TYPE"), "cipherConfig": SimTypeChar(label="Byte")}, name="IPSEC_CIPHER_TRANSFORM_ID0", pack=False, align=None), "cryptoModuleId": SimTypePointer(SimTypeBottom(label="Guid"), offset=0)}, name="IPSEC_CIPHER_TRANSFORM0", pack=False, align=None), offset=0), "espAuthAndCipherTransform": SimTypePointer(SimStruct({"authTransform": SimStruct({"authTransformId": SimStruct({"authType": SimTypeInt(signed=False, label="IPSEC_AUTH_TYPE"), "authConfig": SimTypeChar(label="Byte")}, name="IPSEC_AUTH_TRANSFORM_ID0", pack=False, align=None), "cryptoModuleId": SimTypePointer(SimTypeBottom(label="Guid"), offset=0)}, name="IPSEC_AUTH_TRANSFORM0", pack=False, align=None), "cipherTransform": SimStruct({"cipherTransformId": SimStruct({"cipherType": SimTypeInt(signed=False, label="IPSEC_CIPHER_TYPE"), "cipherConfig": SimTypeChar(label="Byte")}, name="IPSEC_CIPHER_TRANSFORM_ID0", pack=False, align=None), "cryptoModuleId": SimTypePointer(SimTypeBottom(label="Guid"), offset=0)}, name="IPSEC_CIPHER_TRANSFORM0", pack=False, align=None)}, name="IPSEC_AUTH_AND_CIPHER_TRANSFORM0", pack=False, align=None), offset=0), "espAuthFwTransform": SimTypePointer(SimStruct({"authTransformId": SimStruct({"authType": SimTypeInt(signed=False, label="IPSEC_AUTH_TYPE"), "authConfig": SimTypeChar(label="Byte")}, name="IPSEC_AUTH_TRANSFORM_ID0", pack=False, align=None), "cryptoModuleId": SimTypePointer(SimTypeBottom(label="Guid"), offset=0)}, name="IPSEC_AUTH_TRANSFORM0", pack=False, align=None), offset=0)}, name="<anon>", label="None")}, name="IPSEC_SA_TRANSFORM0", pack=False, align=None), offset=0), "pfsGroup": SimTypeInt(signed=False, label="IPSEC_PFS_GROUP")}, name="IPSEC_PROPOSAL0", pack=False, align=None), offset=0), "flags": SimTypeInt(signed=False, label="IPSEC_POLICY_FLAG"), "ndAllowClearTimeoutSeconds": SimTypeInt(signed=False, label="UInt32"), "saIdleTimeout": SimStruct({"idleTimeoutSeconds": SimTypeInt(signed=False, label="UInt32"), "idleTimeoutSecondsFailOver": SimTypeInt(signed=False, label="UInt32")}, name="IPSEC_SA_IDLE_TIMEOUT0", pack=False, align=None), "emPolicy": SimTypePointer(SimStruct({"numAuthenticationMethods": SimTypeInt(signed=False, label="UInt32"), "authenticationMethods": SimTypePointer(SimStruct({"authenticationMethodType": SimTypeInt(signed=False, label="IKEEXT_AUTHENTICATION_METHOD_TYPE"), "Anonymous": SimUnion({"presharedKeyAuthentication": SimStruct({"presharedKey": SimStruct({"size": SimTypeInt(signed=False, label="UInt32"), "data": SimTypePointer(SimTypeChar(label="Byte"), offset=0)}, name="FWP_BYTE_BLOB", pack=False, align=None), "flags": SimTypeInt(signed=False, label="IKEEXT_PRESHARED_KEY_AUTHENTICATION_FLAGS")}, name="IKEEXT_PRESHARED_KEY_AUTHENTICATION1", pack=False, align=None), "certificateAuthentication": SimStruct({"inboundConfigType": SimTypeInt(signed=False, label="IKEEXT_CERT_CONFIG_TYPE"), "Anonymous1": SimUnion({"Anonymous1": SimStruct({"inboundRootArraySize": SimTypeInt(signed=False, label="UInt32"), "inboundRootCriteria": SimTypePointer(SimStruct({"certData": SimStruct({"size": SimTypeInt(signed=False, label="UInt32"), "data": SimTypePointer(SimTypeChar(label="Byte"), offset=0)}, name="FWP_BYTE_BLOB", pack=False, align=None), "certHash": SimStruct({"size": SimTypeInt(signed=False, label="UInt32"), "data": SimTypePointer(SimTypeChar(label="Byte"), offset=0)}, name="FWP_BYTE_BLOB", pack=False, align=None), "eku": SimTypePointer(SimStruct({"numEku": SimTypeInt(signed=False, label="UInt32"), "eku": SimTypePointer(SimTypePointer(SimTypeChar(label="Byte"), offset=0), offset=0)}, name="IKEEXT_CERT_EKUS0", pack=False, align=None), offset=0), "name": SimTypePointer(SimStruct({"nameType": SimTypeInt(signed=False, label="IKEEXT_CERT_CRITERIA_NAME_TYPE"), "certName": SimTypePointer(SimTypeChar(label="Char"), offset=0)}, name="IKEEXT_CERT_NAME0", pack=False, align=None), offset=0), "flags": SimTypeInt(signed=False, label="UInt32")}, name="IKEEXT_CERTIFICATE_CRITERIA0", pack=False, align=None), offset=0)}, name="_Anonymous1_e__Struct", pack=False, align=None), "Anonymous2": SimStruct({"inboundEnterpriseStoreArraySize": SimTypeInt(signed=False, label="UInt32"), "inboundEnterpriseStoreCriteria": SimTypePointer(SimStruct({"certData": SimStruct({"size": SimTypeInt(signed=False, label="UInt32"), "data": SimTypePointer(SimTypeChar(label="Byte"), offset=0)}, name="FWP_BYTE_BLOB", pack=False, align=None), "certHash": SimStruct({"size": SimTypeInt(signed=False, label="UInt32"), "data": SimTypePointer(SimTypeChar(label="Byte"), offset=0)}, name="FWP_BYTE_BLOB", pack=False, align=None), "eku": SimTypePointer(SimStruct({"numEku": SimTypeInt(signed=False, label="UInt32"), "eku": SimTypePointer(SimTypePointer(SimTypeChar(label="Byte"), offset=0), offset=0)}, name="IKEEXT_CERT_EKUS0", pack=False, align=None), offset=0), "name": SimTypePointer(SimStruct({"nameType": SimTypeInt(signed=False, label="IKEEXT_CERT_CRITERIA_NAME_TYPE"), "certName": SimTypePointer(SimTypeChar(label="Char"), offset=0)}, name="IKEEXT_CERT_NAME0", pack=False, align=None), offset=0), "flags": SimTypeInt(signed=False, label="UInt32")}, name="IKEEXT_CERTIFICATE_CRITERIA0", pack=False, align=None), offset=0)}, name="_Anonymous2_e__Struct", pack=False, align=None), "Anonymous3": SimStruct({"inboundRootStoreArraySize": SimTypeInt(signed=False, label="UInt32"), "inboundTrustedRootStoreCriteria": SimTypePointer(SimStruct({"certData": SimStruct({"size": SimTypeInt(signed=False, label="UInt32"), "data": SimTypePointer(SimTypeChar(label="Byte"), offset=0)}, name="FWP_BYTE_BLOB", pack=False, align=None), "certHash": SimStruct({"size": SimTypeInt(signed=False, label="UInt32"), "data": SimTypePointer(SimTypeChar(label="Byte"), offset=0)}, name="FWP_BYTE_BLOB", pack=False, align=None), "eku": SimTypePointer(SimStruct({"numEku": SimTypeInt(signed=False, label="UInt32"), "eku": SimTypePointer(SimTypePointer(SimTypeChar(label="Byte"), offset=0), offset=0)}, name="IKEEXT_CERT_EKUS0", pack=False, align=None), offset=0), "name": SimTypePointer(SimStruct({"nameType": SimTypeInt(signed=False, label="IKEEXT_CERT_CRITERIA_NAME_TYPE"), "certName": SimTypePointer(SimTypeChar(label="Char"), offset=0)}, name="IKEEXT_CERT_NAME0", pack=False, align=None), offset=0), "flags": SimTypeInt(signed=False, label="UInt32")}, name="IKEEXT_CERTIFICATE_CRITERIA0", pack=False, align=None), offset=0)}, name="_Anonymous3_e__Struct", pack=False, align=None)}, name="<anon>", label="None"), "outboundConfigType": SimTypeInt(signed=False, label="IKEEXT_CERT_CONFIG_TYPE"), "Anonymous2": SimUnion({"Anonymous1": SimStruct({"outboundRootArraySize": SimTypeInt(signed=False, label="UInt32"), "outboundRootCriteria": SimTypePointer(SimStruct({"certData": SimStruct({"size": SimTypeInt(signed=False, label="UInt32"), "data": SimTypePointer(SimTypeChar(label="Byte"), offset=0)}, name="FWP_BYTE_BLOB", pack=False, align=None), "certHash": SimStruct({"size": SimTypeInt(signed=False, label="UInt32"), "data": SimTypePointer(SimTypeChar(label="Byte"), offset=0)}, name="FWP_BYTE_BLOB", pack=False, align=None), "eku": SimTypePointer(SimStruct({"numEku": SimTypeInt(signed=False, label="UInt32"), "eku": SimTypePointer(SimTypePointer(SimTypeChar(label="Byte"), offset=0), offset=0)}, name="IKEEXT_CERT_EKUS0", pack=False, align=None), offset=0), "name": SimTypePointer(SimStruct({"nameType": SimTypeInt(signed=False, label="IKEEXT_CERT_CRITERIA_NAME_TYPE"), "certName": SimTypePointer(SimTypeChar(label="Char"), offset=0)}, name="IKEEXT_CERT_NAME0", pack=False, align=None), offset=0), "flags": SimTypeInt(signed=False, label="UInt32")}, name="IKEEXT_CERTIFICATE_CRITERIA0", pack=False, align=None), offset=0)}, name="_Anonymous1_e__Struct", pack=False, align=None), "Anonymous2": SimStruct({"outboundEnterpriseStoreArraySize": SimTypeInt(signed=False, label="UInt32"), "outboundEnterpriseStoreCriteria": SimTypePointer(SimStruct({"certData": SimStruct({"size": SimTypeInt(signed=False, label="UInt32"), "data": SimTypePointer(SimTypeChar(label="Byte"), offset=0)}, name="FWP_BYTE_BLOB", pack=False, align=None), "certHash": SimStruct({"size": SimTypeInt(signed=False, label="UInt32"), "data": SimTypePointer(SimTypeChar(label="Byte"), offset=0)}, name="FWP_BYTE_BLOB", pack=False, align=None), "eku": SimTypePointer(SimStruct({"numEku": SimTypeInt(signed=False, label="UInt32"), "eku": SimTypePointer(SimTypePointer(SimTypeChar(label="Byte"), offset=0), offset=0)}, name="IKEEXT_CERT_EKUS0", pack=False, align=None), offset=0), "name": SimTypePointer(SimStruct({"nameType": SimTypeInt(signed=False, label="IKEEXT_CERT_CRITERIA_NAME_TYPE"), "certName": SimTypePointer(SimTypeChar(label="Char"), offset=0)}, name="IKEEXT_CERT_NAME0", pack=False, align=None), offset=0), "flags": SimTypeInt(signed=False, label="UInt32")}, name="IKEEXT_CERTIFICATE_CRITERIA0", pack=False, align=None), offset=0)}, name="_Anonymous2_e__Struct", pack=False, align=None), "Anonymous3": SimStruct({"outboundRootStoreArraySize": SimTypeInt(signed=False, label="UInt32"), "outboundTrustedRootStoreCriteria": SimTypePointer(SimStruct({"certData": SimStruct({"size": SimTypeInt(signed=False, label="UInt32"), "data": SimTypePointer(SimTypeChar(label="Byte"), offset=0)}, name="FWP_BYTE_BLOB", pack=False, align=None), "certHash": SimStruct({"size": SimTypeInt(signed=False, label="UInt32"), "data": SimTypePointer(SimTypeChar(label="Byte"), offset=0)}, name="FWP_BYTE_BLOB", pack=False, align=None), "eku": SimTypePointer(SimStruct({"numEku": SimTypeInt(signed=False, label="UInt32"), "eku": SimTypePointer(SimTypePointer(SimTypeChar(label="Byte"), offset=0), offset=0)}, name="IKEEXT_CERT_EKUS0", pack=False, align=None), offset=0), "name": SimTypePointer(SimStruct({"nameType": SimTypeInt(signed=False, label="IKEEXT_CERT_CRITERIA_NAME_TYPE"), "certName": SimTypePointer(SimTypeChar(label="Char"), offset=0)}, name="IKEEXT_CERT_NAME0", pack=False, align=None), offset=0), "flags": SimTypeInt(signed=False, label="UInt32")}, name="IKEEXT_CERTIFICATE_CRITERIA0", pack=False, align=None), offset=0)}, name="_Anonymous3_e__Struct", pack=False, align=None)}, name="<anon>", label="None"), "flags": SimTypeInt(signed=False, label="IKEEXT_CERT_AUTH"), "localCertLocationUrl": SimStruct({"size": SimTypeInt(signed=False, label="UInt32"), "data": SimTypePointer(SimTypeChar(label="Byte"), offset=0)}, name="FWP_BYTE_BLOB", pack=False, align=None)}, name="IKEEXT_CERTIFICATE_AUTHENTICATION2", pack=False, align=None), "kerberosAuthentication": SimStruct({"flags": SimTypeInt(signed=False, label="IKEEXT_KERBEROS_AUTHENTICATION_FLAGS"), "proxyServer": SimTypePointer(SimTypeChar(label="Char"), offset=0)}, name="IKEEXT_KERBEROS_AUTHENTICATION1", pack=False, align=None), "reservedAuthentication": SimStruct({"flags": SimTypeInt(signed=False, label="IKEEXT_RESERVED_AUTHENTICATION_FLAGS")}, name="IKEEXT_RESERVED_AUTHENTICATION0", pack=False, align=None), "ntlmV2Authentication": SimStruct({"flags": SimTypeInt(signed=False, label="UInt32")}, name="IKEEXT_NTLM_V2_AUTHENTICATION0", pack=False, align=None), "sslAuthentication": SimStruct({"inboundConfigType": SimTypeInt(signed=False, label="IKEEXT_CERT_CONFIG_TYPE"), "Anonymous1": SimUnion({"Anonymous1": SimStruct({"inboundRootArraySize": SimTypeInt(signed=False, label="UInt32"), "inboundRootCriteria": SimTypePointer(SimStruct({"certData": SimStruct({"size": SimTypeInt(signed=False, label="UInt32"), "data": SimTypePointer(SimTypeChar(label="Byte"), offset=0)}, name="FWP_BYTE_BLOB", pack=False, align=None), "certHash": SimStruct({"size": SimTypeInt(signed=False, label="UInt32"), "data": SimTypePointer(SimTypeChar(label="Byte"), offset=0)}, name="FWP_BYTE_BLOB", pack=False, align=None), "eku": SimTypePointer(SimStruct({"numEku": SimTypeInt(signed=False, label="UInt32"), "eku": SimTypePointer(SimTypePointer(SimTypeChar(label="Byte"), offset=0), offset=0)}, name="IKEEXT_CERT_EKUS0", pack=False, align=None), offset=0), "name": SimTypePointer(SimStruct({"nameType": SimTypeInt(signed=False, label="IKEEXT_CERT_CRITERIA_NAME_TYPE"), "certName": SimTypePointer(SimTypeChar(label="Char"), offset=0)}, name="IKEEXT_CERT_NAME0", pack=False, align=None), offset=0), "flags": SimTypeInt(signed=False, label="UInt32")}, name="IKEEXT_CERTIFICATE_CRITERIA0", pack=False, align=None), offset=0)}, name="_Anonymous1_e__Struct", pack=False, align=None), "Anonymous2": SimStruct({"inboundEnterpriseStoreArraySize": SimTypeInt(signed=False, label="UInt32"), "inboundEnterpriseStoreCriteria": SimTypePointer(SimStruct({"certData": SimStruct({"size": SimTypeInt(signed=False, label="UInt32"), "data": SimTypePointer(SimTypeChar(label="Byte"), offset=0)}, name="FWP_BYTE_BLOB", pack=False, align=None), "certHash": SimStruct({"size": SimTypeInt(signed=False, label="UInt32"), "data": SimTypePointer(SimTypeChar(label="Byte"), offset=0)}, name="FWP_BYTE_BLOB", pack=False, align=None), "eku": SimTypePointer(SimStruct({"numEku": SimTypeInt(signed=False, label="UInt32"), "eku": SimTypePointer(SimTypePointer(SimTypeChar(label="Byte"), offset=0), offset=0)}, name="IKEEXT_CERT_EKUS0", pack=False, align=None), offset=0), "name": SimTypePointer(SimStruct({"nameType": SimTypeInt(signed=False, label="IKEEXT_CERT_CRITERIA_NAME_TYPE"), "certName": SimTypePointer(SimTypeChar(label="Char"), offset=0)}, name="IKEEXT_CERT_NAME0", pack=False, align=None), offset=0), "flags": SimTypeInt(signed=False, label="UInt32")}, name="IKEEXT_CERTIFICATE_CRITERIA0", pack=False, align=None), offset=0)}, name="_Anonymous2_e__Struct", pack=False, align=None), "Anonymous3": SimStruct({"inboundRootStoreArraySize": SimTypeInt(signed=False, label="UInt32"), "inboundTrustedRootStoreCriteria": SimTypePointer(SimStruct({"certData": SimStruct({"size": SimTypeInt(signed=False, label="UInt32"), "data": SimTypePointer(SimTypeChar(label="Byte"), offset=0)}, name="FWP_BYTE_BLOB", pack=False, align=None), "certHash": SimStruct({"size": SimTypeInt(signed=False, label="UInt32"), "data": SimTypePointer(SimTypeChar(label="Byte"), offset=0)}, name="FWP_BYTE_BLOB", pack=False, align=None), "eku": SimTypePointer(SimStruct({"numEku": SimTypeInt(signed=False, label="UInt32"), "eku": SimTypePointer(SimTypePointer(SimTypeChar(label="Byte"), offset=0), offset=0)}, name="IKEEXT_CERT_EKUS0", pack=False, align=None), offset=0), "name": SimTypePointer(SimStruct({"nameType": SimTypeInt(signed=False, label="IKEEXT_CERT_CRITERIA_NAME_TYPE"), "certName": SimTypePointer(SimTypeChar(label="Char"), offset=0)}, name="IKEEXT_CERT_NAME0", pack=False, align=None), offset=0), "flags": SimTypeInt(signed=False, label="UInt32")}, name="IKEEXT_CERTIFICATE_CRITERIA0", pack=False, align=None), offset=0)}, name="_Anonymous3_e__Struct", pack=False, align=None)}, name="<anon>", label="None"), "outboundConfigType": SimTypeInt(signed=False, label="IKEEXT_CERT_CONFIG_TYPE"), "Anonymous2": SimUnion({"Anonymous1": SimStruct({"outboundRootArraySize": SimTypeInt(signed=False, label="UInt32"), "outboundRootCriteria": SimTypePointer(SimStruct({"certData": SimStruct({"size": SimTypeInt(signed=False, label="UInt32"), "data": SimTypePointer(SimTypeChar(label="Byte"), offset=0)}, name="FWP_BYTE_BLOB", pack=False, align=None), "certHash": SimStruct({"size": SimTypeInt(signed=False, label="UInt32"), "data": SimTypePointer(SimTypeChar(label="Byte"), offset=0)}, name="FWP_BYTE_BLOB", pack=False, align=None), "eku": SimTypePointer(SimStruct({"numEku": SimTypeInt(signed=False, label="UInt32"), "eku": SimTypePointer(SimTypePointer(SimTypeChar(label="Byte"), offset=0), offset=0)}, name="IKEEXT_CERT_EKUS0", pack=False, align=None), offset=0), "name": SimTypePointer(SimStruct({"nameType": SimTypeInt(signed=False, label="IKEEXT_CERT_CRITERIA_NAME_TYPE"), "certName": SimTypePointer(SimTypeChar(label="Char"), offset=0)}, name="IKEEXT_CERT_NAME0", pack=False, align=None), offset=0), "flags": SimTypeInt(signed=False, label="UInt32")}, name="IKEEXT_CERTIFICATE_CRITERIA0", pack=False, align=None), offset=0)}, name="_Anonymous1_e__Struct", pack=False, align=None), "Anonymous2": SimStruct({"outboundEnterpriseStoreArraySize": SimTypeInt(signed=False, label="UInt32"), "outboundEnterpriseStoreCriteria": SimTypePointer(SimStruct({"certData": SimStruct({"size": SimTypeInt(signed=False, label="UInt32"), "data": SimTypePointer(SimTypeChar(label="Byte"), offset=0)}, name="FWP_BYTE_BLOB", pack=False, align=None), "certHash": SimStruct({"size": SimTypeInt(signed=False, label="UInt32"), "data": SimTypePointer(SimTypeChar(label="Byte"), offset=0)}, name="FWP_BYTE_BLOB", pack=False, align=None), "eku": SimTypePointer(SimStruct({"numEku": SimTypeInt(signed=False, label="UInt32"), "eku": SimTypePointer(SimTypePointer(SimTypeChar(label="Byte"), offset=0), offset=0)}, name="IKEEXT_CERT_EKUS0", pack=False, align=None), offset=0), "name": SimTypePointer(SimStruct({"nameType": SimTypeInt(signed=False, label="IKEEXT_CERT_CRITERIA_NAME_TYPE"), "certName": SimTypePointer(SimTypeChar(label="Char"), offset=0)}, name="IKEEXT_CERT_NAME0", pack=False, align=None), offset=0), "flags": SimTypeInt(signed=False, label="UInt32")}, name="IKEEXT_CERTIFICATE_CRITERIA0", pack=False, align=None), offset=0)}, name="_Anonymous2_e__Struct", pack=False, align=None), "Anonymous3": SimStruct({"outboundRootStoreArraySize": SimTypeInt(signed=False, label="UInt32"), "outboundTrustedRootStoreCriteria": SimTypePointer(SimStruct({"certData": SimStruct({"size": SimTypeInt(signed=False, label="UInt32"), "data": SimTypePointer(SimTypeChar(label="Byte"), offset=0)}, name="FWP_BYTE_BLOB", pack=False, align=None), "certHash": SimStruct({"size": SimTypeInt(signed=False, label="UInt32"), "data": SimTypePointer(SimTypeChar(label="Byte"), offset=0)}, name="FWP_BYTE_BLOB", pack=False, align=None), "eku": SimTypePointer(SimStruct({"numEku": SimTypeInt(signed=False, label="UInt32"), "eku": SimTypePointer(SimTypePointer(SimTypeChar(label="Byte"), offset=0), offset=0)}, name="IKEEXT_CERT_EKUS0", pack=False, align=None), offset=0), "name": SimTypePointer(SimStruct({"nameType": SimTypeInt(signed=False, label="IKEEXT_CERT_CRITERIA_NAME_TYPE"), "certName": SimTypePointer(SimTypeChar(label="Char"), offset=0)}, name="IKEEXT_CERT_NAME0", pack=False, align=None), offset=0), "flags": SimTypeInt(signed=False, label="UInt32")}, name="IKEEXT_CERTIFICATE_CRITERIA0", pack=False, align=None), offset=0)}, name="_Anonymous3_e__Struct", pack=False, align=None)}, name="<anon>", label="None"), "flags": SimTypeInt(signed=False, label="IKEEXT_CERT_AUTH"), "localCertLocationUrl": SimStruct({"size": SimTypeInt(signed=False, label="UInt32"), "data": SimTypePointer(SimTypeChar(label="Byte"), offset=0)}, name="FWP_BYTE_BLOB", pack=False, align=None)}, name="IKEEXT_CERTIFICATE_AUTHENTICATION2", pack=False, align=None), "cgaAuthentication": SimStruct({"keyContainerName": SimTypePointer(SimTypeChar(label="Char"), offset=0), "cspName": SimTypePointer(SimTypeChar(label="Char"), offset=0), "cspType": SimTypeInt(signed=False, label="UInt32"), "cgaModifier": SimStruct({"byteArray16": SimTypeFixedSizeArray(SimTypeChar(label="Byte"), 16)}, name="FWP_BYTE_ARRAY16", pack=False, align=None), "cgaCollisionCount": SimTypeChar(label="Byte")}, name="IKEEXT_IPV6_CGA_AUTHENTICATION0", pack=False, align=None), "eapAuthentication": SimStruct({"flags": SimTypeInt(signed=False, label="IKEEXT_EAP_AUTHENTICATION_FLAGS")}, name="IKEEXT_EAP_AUTHENTICATION0", pack=False, align=None)}, name="<anon>", label="None")}, name="IKEEXT_AUTHENTICATION_METHOD2", pack=False, align=None), offset=0), "initiatorImpersonationType": SimTypeInt(signed=False, label="IKEEXT_AUTHENTICATION_IMPERSONATION_TYPE")}, name="IKEEXT_EM_POLICY2", pack=False, align=None), offset=0)}, name="IPSEC_TRANSPORT_POLICY2", pack=False, align=None), offset=0), "ikeQmTunnelPolicy": SimTypePointer(SimStruct({"flags": SimTypeInt(signed=False, label="IPSEC_POLICY_FLAG"), "numIpsecProposals": SimTypeInt(signed=False, label="UInt32"), "ipsecProposals": SimTypePointer(SimStruct({"lifetime": SimStruct({"lifetimeSeconds": SimTypeInt(signed=False, label="UInt32"), "lifetimeKilobytes": SimTypeInt(signed=False, label="UInt32"), "lifetimePackets": SimTypeInt(signed=False, label="UInt32")}, name="IPSEC_SA_LIFETIME0", pack=False, align=None), "numSaTransforms": SimTypeInt(signed=False, label="UInt32"), "saTransforms": SimTypePointer(SimStruct({"ipsecTransformType": SimTypeInt(signed=False, label="IPSEC_TRANSFORM_TYPE"), "Anonymous": SimUnion({"ahTransform": SimTypePointer(SimStruct({"authTransformId": SimStruct({"authType": SimTypeInt(signed=False, label="IPSEC_AUTH_TYPE"), "authConfig": SimTypeChar(label="Byte")}, name="IPSEC_AUTH_TRANSFORM_ID0", pack=False, align=None), "cryptoModuleId": SimTypePointer(SimTypeBottom(label="Guid"), offset=0)}, name="IPSEC_AUTH_TRANSFORM0", pack=False, align=None), offset=0), "espAuthTransform": SimTypePointer(SimStruct({"authTransformId": SimStruct({"authType": SimTypeInt(signed=False, label="IPSEC_AUTH_TYPE"), "authConfig": SimTypeChar(label="Byte")}, name="IPSEC_AUTH_TRANSFORM_ID0", pack=False, align=None), "cryptoModuleId": SimTypePointer(SimTypeBottom(label="Guid"), offset=0)}, name="IPSEC_AUTH_TRANSFORM0", pack=False, align=None), offset=0), "espCipherTransform": SimTypePointer(SimStruct({"cipherTransformId": SimStruct({"cipherType": SimTypeInt(signed=False, label="IPSEC_CIPHER_TYPE"), "cipherConfig": SimTypeChar(label="Byte")}, name="IPSEC_CIPHER_TRANSFORM_ID0", pack=False, align=None), "cryptoModuleId": SimTypePointer(SimTypeBottom(label="Guid"), offset=0)}, name="IPSEC_CIPHER_TRANSFORM0", pack=False, align=None), offset=0), "espAuthAndCipherTransform": SimTypePointer(SimStruct({"authTransform": SimStruct({"authTransformId": SimStruct({"authType": SimTypeInt(signed=False, label="IPSEC_AUTH_TYPE"), "authConfig": SimTypeChar(label="Byte")}, name="IPSEC_AUTH_TRANSFORM_ID0", pack=False, align=None), "cryptoModuleId": SimTypePointer(SimTypeBottom(label="Guid"), offset=0)}, name="IPSEC_AUTH_TRANSFORM0", pack=False, align=None), "cipherTransform": SimStruct({"cipherTransformId": SimStruct({"cipherType": SimTypeInt(signed=False, label="IPSEC_CIPHER_TYPE"), "cipherConfig": SimTypeChar(label="Byte")}, name="IPSEC_CIPHER_TRANSFORM_ID0", pack=False, align=None), "cryptoModuleId": SimTypePointer(SimTypeBottom(label="Guid"), offset=0)}, name="IPSEC_CIPHER_TRANSFORM0", pack=False, align=None)}, name="IPSEC_AUTH_AND_CIPHER_TRANSFORM0", pack=False, align=None), offset=0), "espAuthFwTransform": SimTypePointer(SimStruct({"authTransformId": SimStruct({"authType": SimTypeInt(signed=False, label="IPSEC_AUTH_TYPE"), "authConfig": SimTypeChar(label="Byte")}, name="IPSEC_AUTH_TRANSFORM_ID0", pack=False, align=None), "cryptoModuleId": SimTypePointer(SimTypeBottom(label="Guid"), offset=0)}, name="IPSEC_AUTH_TRANSFORM0", pack=False, align=None), offset=0)}, name="<anon>", label="None")}, name="IPSEC_SA_TRANSFORM0", pack=False, align=None), offset=0), "pfsGroup": SimTypeInt(signed=False, label="IPSEC_PFS_GROUP")}, name="IPSEC_PROPOSAL0", pack=False, align=None), offset=0), "tunnelEndpoints": SimStruct({"ipVersion": SimTypeInt(signed=False, label="FWP_IP_VERSION"), "Anonymous1": SimUnion({"localV4Address": SimTypeInt(signed=False, label="UInt32"), "localV6Address": SimTypeFixedSizeArray(SimTypeChar(label="Byte"), 16)}, name="<anon>", label="None"), "Anonymous2": SimUnion({"remoteV4Address": SimTypeInt(signed=False, label="UInt32"), "remoteV6Address": SimTypeFixedSizeArray(SimTypeChar(label="Byte"), 16)}, name="<anon>", label="None"), "localIfLuid": SimTypeLongLong(signed=False, label="UInt64"), "remoteFqdn": SimTypePointer(SimTypeChar(label="Char"), offset=0), "numAddresses": SimTypeInt(signed=False, label="UInt32"), "remoteAddresses": SimTypePointer(SimStruct({"ipVersion": SimTypeInt(signed=False, label="FWP_IP_VERSION"), "Anonymous": SimUnion({"v4Address": SimTypeInt(signed=False, label="UInt32"), "v6Address": SimTypeFixedSizeArray(SimTypeChar(label="Byte"), 16)}, name="<anon>", label="None")}, name="IPSEC_TUNNEL_ENDPOINT0", pack=False, align=None), offset=0)}, name="IPSEC_TUNNEL_ENDPOINTS2", pack=False, align=None), "saIdleTimeout": SimStruct({"idleTimeoutSeconds": SimTypeInt(signed=False, label="UInt32"), "idleTimeoutSecondsFailOver": SimTypeInt(signed=False, label="UInt32")}, name="IPSEC_SA_IDLE_TIMEOUT0", pack=False, align=None), "emPolicy": SimTypePointer(SimStruct({"numAuthenticationMethods": SimTypeInt(signed=False, label="UInt32"), "authenticationMethods": SimTypePointer(SimStruct({"authenticationMethodType": SimTypeInt(signed=False, label="IKEEXT_AUTHENTICATION_METHOD_TYPE"), "Anonymous": SimUnion({"presharedKeyAuthentication": SimStruct({"presharedKey": SimStruct({"size": SimTypeInt(signed=False, label="UInt32"), "data": SimTypePointer(SimTypeChar(label="Byte"), offset=0)}, name="FWP_BYTE_BLOB", pack=False, align=None), "flags": SimTypeInt(signed=False, label="IKEEXT_PRESHARED_KEY_AUTHENTICATION_FLAGS")}, name="IKEEXT_PRESHARED_KEY_AUTHENTICATION1", pack=False, align=None), "certificateAuthentication": SimStruct({"inboundConfigType": SimTypeInt(signed=False, label="IKEEXT_CERT_CONFIG_TYPE"), "Anonymous1": SimUnion({"Anonymous1": SimStruct({"inboundRootArraySize": SimTypeInt(signed=False, label="UInt32"), "inboundRootCriteria": SimTypePointer(SimStruct({"certData": SimStruct({"size": SimTypeInt(signed=False, label="UInt32"), "data": SimTypePointer(SimTypeChar(label="Byte"), offset=0)}, name="FWP_BYTE_BLOB", pack=False, align=None), "certHash": SimStruct({"size": SimTypeInt(signed=False, label="UInt32"), "data": SimTypePointer(SimTypeChar(label="Byte"), offset=0)}, name="FWP_BYTE_BLOB", pack=False, align=None), "eku": SimTypePointer(SimStruct({"numEku": SimTypeInt(signed=False, label="UInt32"), "eku": SimTypePointer(SimTypePointer(SimTypeChar(label="Byte"), offset=0), offset=0)}, name="IKEEXT_CERT_EKUS0", pack=False, align=None), offset=0), "name": SimTypePointer(SimStruct({"nameType": SimTypeInt(signed=False, label="IKEEXT_CERT_CRITERIA_NAME_TYPE"), "certName": SimTypePointer(SimTypeChar(label="Char"), offset=0)}, name="IKEEXT_CERT_NAME0", pack=False, align=None), offset=0), "flags": SimTypeInt(signed=False, label="UInt32")}, name="IKEEXT_CERTIFICATE_CRITERIA0", pack=False, align=None), offset=0)}, name="_Anonymous1_e__Struct", pack=False, align=None), "Anonymous2": SimStruct({"inboundEnterpriseStoreArraySize": SimTypeInt(signed=False, label="UInt32"), "inboundEnterpriseStoreCriteria": SimTypePointer(SimStruct({"certData": SimStruct({"size": SimTypeInt(signed=False, label="UInt32"), "data": SimTypePointer(SimTypeChar(label="Byte"), offset=0)}, name="FWP_BYTE_BLOB", pack=False, align=None), "certHash": SimStruct({"size": SimTypeInt(signed=False, label="UInt32"), "data": SimTypePointer(SimTypeChar(label="Byte"), offset=0)}, name="FWP_BYTE_BLOB", pack=False, align=None), "eku": SimTypePointer(SimStruct({"numEku": SimTypeInt(signed=False, label="UInt32"), "eku": SimTypePointer(SimTypePointer(SimTypeChar(label="Byte"), offset=0), offset=0)}, name="IKEEXT_CERT_EKUS0", pack=False, align=None), offset=0), "name": SimTypePointer(SimStruct({"nameType": SimTypeInt(signed=False, label="IKEEXT_CERT_CRITERIA_NAME_TYPE"), "certName": SimTypePointer(SimTypeChar(label="Char"), offset=0)}, name="IKEEXT_CERT_NAME0", pack=False, align=None), offset=0), "flags": SimTypeInt(signed=False, label="UInt32")}, name="IKEEXT_CERTIFICATE_CRITERIA0", pack=False, align=None), offset=0)}, name="_Anonymous2_e__Struct", pack=False, align=None), "Anonymous3": SimStruct({"inboundRootStoreArraySize": SimTypeInt(signed=False, label="UInt32"), "inboundTrustedRootStoreCriteria": SimTypePointer(SimStruct({"certData": SimStruct({"size": SimTypeInt(signed=False, label="UInt32"), "data": SimTypePointer(SimTypeChar(label="Byte"), offset=0)}, name="FWP_BYTE_BLOB", pack=False, align=None), "certHash": SimStruct({"size": SimTypeInt(signed=False, label="UInt32"), "data": SimTypePointer(SimTypeChar(label="Byte"), offset=0)}, name="FWP_BYTE_BLOB", pack=False, align=None), "eku": SimTypePointer(SimStruct({"numEku": SimTypeInt(signed=False, label="UInt32"), "eku": SimTypePointer(SimTypePointer(SimTypeChar(label="Byte"), offset=0), offset=0)}, name="IKEEXT_CERT_EKUS0", pack=False, align=None), offset=0), "name": SimTypePointer(SimStruct({"nameType": SimTypeInt(signed=False, label="IKEEXT_CERT_CRITERIA_NAME_TYPE"), "certName": SimTypePointer(SimTypeChar(label="Char"), offset=0)}, name="IKEEXT_CERT_NAME0", pack=False, align=None), offset=0), "flags": SimTypeInt(signed=False, label="UInt32")}, name="IKEEXT_CERTIFICATE_CRITERIA0", pack=False, align=None), offset=0)}, name="_Anonymous3_e__Struct", pack=False, align=None)}, name="<anon>", label="None"), "outboundConfigType": SimTypeInt(signed=False, label="IKEEXT_CERT_CONFIG_TYPE"), "Anonymous2": SimUnion({"Anonymous1": SimStruct({"outboundRootArraySize": SimTypeInt(signed=False, label="UInt32"), "outboundRootCriteria": SimTypePointer(SimStruct({"certData": SimStruct({"size": SimTypeInt(signed=False, label="UInt32"), "data": SimTypePointer(SimTypeChar(label="Byte"), offset=0)}, name="FWP_BYTE_BLOB", pack=False, align=None), "certHash": SimStruct({"size": SimTypeInt(signed=False, label="UInt32"), "data": SimTypePointer(SimTypeChar(label="Byte"), offset=0)}, name="FWP_BYTE_BLOB", pack=False, align=None), "eku": SimTypePointer(SimStruct({"numEku": SimTypeInt(signed=False, label="UInt32"), "eku": SimTypePointer(SimTypePointer(SimTypeChar(label="Byte"), offset=0), offset=0)}, name="IKEEXT_CERT_EKUS0", pack=False, align=None), offset=0), "name": SimTypePointer(SimStruct({"nameType": SimTypeInt(signed=False, label="IKEEXT_CERT_CRITERIA_NAME_TYPE"), "certName": SimTypePointer(SimTypeChar(label="Char"), offset=0)}, name="IKEEXT_CERT_NAME0", pack=False, align=None), offset=0), "flags": SimTypeInt(signed=False, label="UInt32")}, name="IKEEXT_CERTIFICATE_CRITERIA0", pack=False, align=None), offset=0)}, name="_Anonymous1_e__Struct", pack=False, align=None), "Anonymous2": SimStruct({"outboundEnterpriseStoreArraySize": SimTypeInt(signed=False, label="UInt32"), "outboundEnterpriseStoreCriteria": SimTypePointer(SimStruct({"certData": SimStruct({"size": SimTypeInt(signed=False, label="UInt32"), "data": SimTypePointer(SimTypeChar(label="Byte"), offset=0)}, name="FWP_BYTE_BLOB", pack=False, align=None), "certHash": SimStruct({"size": SimTypeInt(signed=False, label="UInt32"), "data": SimTypePointer(SimTypeChar(label="Byte"), offset=0)}, name="FWP_BYTE_BLOB", pack=False, align=None), "eku": SimTypePointer(SimStruct({"numEku": SimTypeInt(signed=False, label="UInt32"), "eku": SimTypePointer(SimTypePointer(SimTypeChar(label="Byte"), offset=0), offset=0)}, name="IKEEXT_CERT_EKUS0", pack=False, align=None), offset=0), "name": SimTypePointer(SimStruct({"nameType": SimTypeInt(signed=False, label="IKEEXT_CERT_CRITERIA_NAME_TYPE"), "certName": SimTypePointer(SimTypeChar(label="Char"), offset=0)}, name="IKEEXT_CERT_NAME0", pack=False, align=None), offset=0), "flags": SimTypeInt(signed=False, label="UInt32")}, name="IKEEXT_CERTIFICATE_CRITERIA0", pack=False, align=None), offset=0)}, name="_Anonymous2_e__Struct", pack=False, align=None), "Anonymous3": SimStruct({"outboundRootStoreArraySize": SimTypeInt(signed=False, label="UInt32"), "outboundTrustedRootStoreCriteria": SimTypePointer(SimStruct({"certData": SimStruct({"size": SimTypeInt(signed=False, label="UInt32"), "data": SimTypePointer(SimTypeChar(label="Byte"), offset=0)}, name="FWP_BYTE_BLOB", pack=False, align=None), "certHash": SimStruct({"size": SimTypeInt(signed=False, label="UInt32"), "data": SimTypePointer(SimTypeChar(label="Byte"), offset=0)}, name="FWP_BYTE_BLOB", pack=False, align=None), "eku": SimTypePointer(SimStruct({"numEku": SimTypeInt(signed=False, label="UInt32"), "eku": SimTypePointer(SimTypePointer(SimTypeChar(label="Byte"), offset=0), offset=0)}, name="IKEEXT_CERT_EKUS0", pack=False, align=None), offset=0), "name": SimTypePointer(SimStruct({"nameType": SimTypeInt(signed=False, label="IKEEXT_CERT_CRITERIA_NAME_TYPE"), "certName": SimTypePointer(SimTypeChar(label="Char"), offset=0)}, name="IKEEXT_CERT_NAME0", pack=False, align=None), offset=0), "flags": SimTypeInt(signed=False, label="UInt32")}, name="IKEEXT_CERTIFICATE_CRITERIA0", pack=False, align=None), offset=0)}, name="_Anonymous3_e__Struct", pack=False, align=None)}, name="<anon>", label="None"), "flags": SimTypeInt(signed=False, label="IKEEXT_CERT_AUTH"), "localCertLocationUrl": SimStruct({"size": SimTypeInt(signed=False, label="UInt32"), "data": SimTypePointer(SimTypeChar(label="Byte"), offset=0)}, name="FWP_BYTE_BLOB", pack=False, align=None)}, name="IKEEXT_CERTIFICATE_AUTHENTICATION2", pack=False, align=None), "kerberosAuthentication": SimStruct({"flags": SimTypeInt(signed=False, label="IKEEXT_KERBEROS_AUTHENTICATION_FLAGS"), "proxyServer": SimTypePointer(SimTypeChar(label="Char"), offset=0)}, name="IKEEXT_KERBEROS_AUTHENTICATION1", pack=False, align=None), "reservedAuthentication": SimStruct({"flags": SimTypeInt(signed=False, label="IKEEXT_RESERVED_AUTHENTICATION_FLAGS")}, name="IKEEXT_RESERVED_AUTHENTICATION0", pack=False, align=None), "ntlmV2Authentication": SimStruct({"flags": SimTypeInt(signed=False, label="UInt32")}, name="IKEEXT_NTLM_V2_AUTHENTICATION0", pack=False, align=None), "sslAuthentication": SimStruct({"inboundConfigType": SimTypeInt(signed=False, label="IKEEXT_CERT_CONFIG_TYPE"), "Anonymous1": SimUnion({"Anonymous1": SimStruct({"inboundRootArraySize": SimTypeInt(signed=False, label="UInt32"), "inboundRootCriteria": SimTypePointer(SimStruct({"certData": SimStruct({"size": SimTypeInt(signed=False, label="UInt32"), "data": SimTypePointer(SimTypeChar(label="Byte"), offset=0)}, name="FWP_BYTE_BLOB", pack=False, align=None), "certHash": SimStruct({"size": SimTypeInt(signed=False, label="UInt32"), "data": SimTypePointer(SimTypeChar(label="Byte"), offset=0)}, name="FWP_BYTE_BLOB", pack=False, align=None), "eku": SimTypePointer(SimStruct({"numEku": SimTypeInt(signed=False, label="UInt32"), "eku": SimTypePointer(SimTypePointer(SimTypeChar(label="Byte"), offset=0), offset=0)}, name="IKEEXT_CERT_EKUS0", pack=False, align=None), offset=0), "name": SimTypePointer(SimStruct({"nameType": SimTypeInt(signed=False, label="IKEEXT_CERT_CRITERIA_NAME_TYPE"), "certName": SimTypePointer(SimTypeChar(label="Char"), offset=0)}, name="IKEEXT_CERT_NAME0", pack=False, align=None), offset=0), "flags": SimTypeInt(signed=False, label="UInt32")}, name="IKEEXT_CERTIFICATE_CRITERIA0", pack=False, align=None), offset=0)}, name="_Anonymous1_e__Struct", pack=False, align=None), "Anonymous2": SimStruct({"inboundEnterpriseStoreArraySize": SimTypeInt(signed=False, label="UInt32"), "inboundEnterpriseStoreCriteria": SimTypePointer(SimStruct({"certData": SimStruct({"size": SimTypeInt(signed=False, label="UInt32"), "data": SimTypePointer(SimTypeChar(label="Byte"), offset=0)}, name="FWP_BYTE_BLOB", pack=False, align=None), "certHash": SimStruct({"size": SimTypeInt(signed=False, label="UInt32"), "data": SimTypePointer(SimTypeChar(label="Byte"), offset=0)}, name="FWP_BYTE_BLOB", pack=False, align=None), "eku": SimTypePointer(SimStruct({"numEku": SimTypeInt(signed=False, label="UInt32"), "eku": SimTypePointer(SimTypePointer(SimTypeChar(label="Byte"), offset=0), offset=0)}, name="IKEEXT_CERT_EKUS0", pack=False, align=None), offset=0), "name": SimTypePointer(SimStruct({"nameType": SimTypeInt(signed=False, label="IKEEXT_CERT_CRITERIA_NAME_TYPE"), "certName": SimTypePointer(SimTypeChar(label="Char"), offset=0)}, name="IKEEXT_CERT_NAME0", pack=False, align=None), offset=0), "flags": SimTypeInt(signed=False, label="UInt32")}, name="IKEEXT_CERTIFICATE_CRITERIA0", pack=False, align=None), offset=0)}, name="_Anonymous2_e__Struct", pack=False, align=None), "Anonymous3": SimStruct({"inboundRootStoreArraySize": SimTypeInt(signed=False, label="UInt32"), "inboundTrustedRootStoreCriteria": SimTypePointer(SimStruct({"certData": SimStruct({"size": SimTypeInt(signed=False, label="UInt32"), "data": SimTypePointer(SimTypeChar(label="Byte"), offset=0)}, name="FWP_BYTE_BLOB", pack=False, align=None), "certHash": SimStruct({"size": SimTypeInt(signed=False, label="UInt32"), "data": SimTypePointer(SimTypeChar(label="Byte"), offset=0)}, name="FWP_BYTE_BLOB", pack=False, align=None), "eku": SimTypePointer(SimStruct({"numEku": SimTypeInt(signed=False, label="UInt32"), "eku": SimTypePointer(SimTypePointer(SimTypeChar(label="Byte"), offset=0), offset=0)}, name="IKEEXT_CERT_EKUS0", pack=False, align=None), offset=0), "name": SimTypePointer(SimStruct({"nameType": SimTypeInt(signed=False, label="IKEEXT_CERT_CRITERIA_NAME_TYPE"), "certName": SimTypePointer(SimTypeChar(label="Char"), offset=0)}, name="IKEEXT_CERT_NAME0", pack=False, align=None), offset=0), "flags": SimTypeInt(signed=False, label="UInt32")}, name="IKEEXT_CERTIFICATE_CRITERIA0", pack=False, align=None), offset=0)}, name="_Anonymous3_e__Struct", pack=False, align=None)}, name="<anon>", label="None"), "outboundConfigType": SimTypeInt(signed=False, label="IKEEXT_CERT_CONFIG_TYPE"), "Anonymous2": SimUnion({"Anonymous1": SimStruct({"outboundRootArraySize": SimTypeInt(signed=False, label="UInt32"), "outboundRootCriteria": SimTypePointer(SimStruct({"certData": SimStruct({"size": SimTypeInt(signed=False, label="UInt32"), "data": SimTypePointer(SimTypeChar(label="Byte"), offset=0)}, name="FWP_BYTE_BLOB", pack=False, align=None), "certHash": SimStruct({"size": SimTypeInt(signed=False, label="UInt32"), "data": SimTypePointer(SimTypeChar(label="Byte"), offset=0)}, name="FWP_BYTE_BLOB", pack=False, align=None), "eku": SimTypePointer(SimStruct({"numEku": SimTypeInt(signed=False, label="UInt32"), "eku": SimTypePointer(SimTypePointer(SimTypeChar(label="Byte"), offset=0), offset=0)}, name="IKEEXT_CERT_EKUS0", pack=False, align=None), offset=0), "name": SimTypePointer(SimStruct({"nameType": SimTypeInt(signed=False, label="IKEEXT_CERT_CRITERIA_NAME_TYPE"), "certName": SimTypePointer(SimTypeChar(label="Char"), offset=0)}, name="IKEEXT_CERT_NAME0", pack=False, align=None), offset=0), "flags": SimTypeInt(signed=False, label="UInt32")}, name="IKEEXT_CERTIFICATE_CRITERIA0", pack=False, align=None), offset=0)}, name="_Anonymous1_e__Struct", pack=False, align=None), "Anonymous2": SimStruct({"outboundEnterpriseStoreArraySize": SimTypeInt(signed=False, label="UInt32"), "outboundEnterpriseStoreCriteria": SimTypePointer(SimStruct({"certData": SimStruct({"size": SimTypeInt(signed=False, label="UInt32"), "data": SimTypePointer(SimTypeChar(label="Byte"), offset=0)}, name="FWP_BYTE_BLOB", pack=False, align=None), "certHash": SimStruct({"size": SimTypeInt(signed=False, label="UInt32"), "data": SimTypePointer(SimTypeChar(label="Byte"), offset=0)}, name="FWP_BYTE_BLOB", pack=False, align=None), "eku": SimTypePointer(SimStruct({"numEku": SimTypeInt(signed=False, label="UInt32"), "eku": SimTypePointer(SimTypePointer(SimTypeChar(label="Byte"), offset=0), offset=0)}, name="IKEEXT_CERT_EKUS0", pack=False, align=None), offset=0), "name": SimTypePointer(SimStruct({"nameType": SimTypeInt(signed=False, label="IKEEXT_CERT_CRITERIA_NAME_TYPE"), "certName": SimTypePointer(SimTypeChar(label="Char"), offset=0)}, name="IKEEXT_CERT_NAME0", pack=False, align=None), offset=0), "flags": SimTypeInt(signed=False, label="UInt32")}, name="IKEEXT_CERTIFICATE_CRITERIA0", pack=False, align=None), offset=0)}, name="_Anonymous2_e__Struct", pack=False, align=None), "Anonymous3": SimStruct({"outboundRootStoreArraySize": SimTypeInt(signed=False, label="UInt32"), "outboundTrustedRootStoreCriteria": SimTypePointer(SimStruct({"certData": SimStruct({"size": SimTypeInt(signed=False, label="UInt32"), "data": SimTypePointer(SimTypeChar(label="Byte"), offset=0)}, name="FWP_BYTE_BLOB", pack=False, align=None), "certHash": SimStruct({"size": SimTypeInt(signed=False, label="UInt32"), "data": SimTypePointer(SimTypeChar(label="Byte"), offset=0)}, name="FWP_BYTE_BLOB", pack=False, align=None), "eku": SimTypePointer(SimStruct({"numEku": SimTypeInt(signed=False, label="UInt32"), "eku": SimTypePointer(SimTypePointer(SimTypeChar(label="Byte"), offset=0), offset=0)}, name="IKEEXT_CERT_EKUS0", pack=False, align=None), offset=0), "name": SimTypePointer(SimStruct({"nameType": SimTypeInt(signed=False, label="IKEEXT_CERT_CRITERIA_NAME_TYPE"), "certName": SimTypePointer(SimTypeChar(label="Char"), offset=0)}, name="IKEEXT_CERT_NAME0", pack=False, align=None), offset=0), "flags": SimTypeInt(signed=False, label="UInt32")}, name="IKEEXT_CERTIFICATE_CRITERIA0", pack=False, align=None), offset=0)}, name="_Anonymous3_e__Struct", pack=False, align=None)}, name="<anon>", label="None"), "flags": SimTypeInt(signed=False, label="IKEEXT_CERT_AUTH"), "localCertLocationUrl": SimStruct({"size": SimTypeInt(signed=False, label="UInt32"), "data": SimTypePointer(SimTypeChar(label="Byte"), offset=0)}, name="FWP_BYTE_BLOB", pack=False, align=None)}, name="IKEEXT_CERTIFICATE_AUTHENTICATION2", pack=False, align=None), "cgaAuthentication": SimStruct({"keyContainerName": SimTypePointer(SimTypeChar(label="Char"), offset=0), "cspName": SimTypePointer(SimTypeChar(label="Char"), offset=0), "cspType": SimTypeInt(signed=False, label="UInt32"), "cgaModifier": SimStruct({"byteArray16": SimTypeFixedSizeArray(SimTypeChar(label="Byte"), 16)}, name="FWP_BYTE_ARRAY16", pack=False, align=None), "cgaCollisionCount": SimTypeChar(label="Byte")}, name="IKEEXT_IPV6_CGA_AUTHENTICATION0", pack=False, align=None), "eapAuthentication": SimStruct({"flags": SimTypeInt(signed=False, label="IKEEXT_EAP_AUTHENTICATION_FLAGS")}, name="IKEEXT_EAP_AUTHENTICATION0", pack=False, align=None)}, name="<anon>", label="None")}, name="IKEEXT_AUTHENTICATION_METHOD2", pack=False, align=None), offset=0), "initiatorImpersonationType": SimTypeInt(signed=False, label="IKEEXT_AUTHENTICATION_IMPERSONATION_TYPE")}, name="IKEEXT_EM_POLICY2", pack=False, align=None), offset=0), "fwdPathSaLifetime": SimTypeInt(signed=False, label="UInt32")}, name="IPSEC_TUNNEL_POLICY2", pack=False, align=None), offset=0), "authipQmTransportPolicy": SimTypePointer(SimStruct({"numIpsecProposals": SimTypeInt(signed=False, label="UInt32"), "ipsecProposals": SimTypePointer(SimStruct({"lifetime": SimStruct({"lifetimeSeconds": SimTypeInt(signed=False, label="UInt32"), "lifetimeKilobytes": SimTypeInt(signed=False, label="UInt32"), "lifetimePackets": SimTypeInt(signed=False, label="UInt32")}, name="IPSEC_SA_LIFETIME0", pack=False, align=None), "numSaTransforms": SimTypeInt(signed=False, label="UInt32"), "saTransforms": SimTypePointer(SimStruct({"ipsecTransformType": SimTypeInt(signed=False, label="IPSEC_TRANSFORM_TYPE"), "Anonymous": SimUnion({"ahTransform": SimTypePointer(SimStruct({"authTransformId": SimStruct({"authType": SimTypeInt(signed=False, label="IPSEC_AUTH_TYPE"), "authConfig": SimTypeChar(label="Byte")}, name="IPSEC_AUTH_TRANSFORM_ID0", pack=False, align=None), "cryptoModuleId": SimTypePointer(SimTypeBottom(label="Guid"), offset=0)}, name="IPSEC_AUTH_TRANSFORM0", pack=False, align=None), offset=0), "espAuthTransform": SimTypePointer(SimStruct({"authTransformId": SimStruct({"authType": SimTypeInt(signed=False, label="IPSEC_AUTH_TYPE"), "authConfig": SimTypeChar(label="Byte")}, name="IPSEC_AUTH_TRANSFORM_ID0", pack=False, align=None), "cryptoModuleId": SimTypePointer(SimTypeBottom(label="Guid"), offset=0)}, name="IPSEC_AUTH_TRANSFORM0", pack=False, align=None), offset=0), "espCipherTransform": SimTypePointer(SimStruct({"cipherTransformId": SimStruct({"cipherType": SimTypeInt(signed=False, label="IPSEC_CIPHER_TYPE"), "cipherConfig": SimTypeChar(label="Byte")}, name="IPSEC_CIPHER_TRANSFORM_ID0", pack=False, align=None), "cryptoModuleId": SimTypePointer(SimTypeBottom(label="Guid"), offset=0)}, name="IPSEC_CIPHER_TRANSFORM0", pack=False, align=None), offset=0), "espAuthAndCipherTransform": SimTypePointer(SimStruct({"authTransform": SimStruct({"authTransformId": SimStruct({"authType": SimTypeInt(signed=False, label="IPSEC_AUTH_TYPE"), "authConfig": SimTypeChar(label="Byte")}, name="IPSEC_AUTH_TRANSFORM_ID0", pack=False, align=None), "cryptoModuleId": SimTypePointer(SimTypeBottom(label="Guid"), offset=0)}, name="IPSEC_AUTH_TRANSFORM0", pack=False, align=None), "cipherTransform": SimStruct({"cipherTransformId": SimStruct({"cipherType": SimTypeInt(signed=False, label="IPSEC_CIPHER_TYPE"), "cipherConfig": SimTypeChar(label="Byte")}, name="IPSEC_CIPHER_TRANSFORM_ID0", pack=False, align=None), "cryptoModuleId": SimTypePointer(SimTypeBottom(label="Guid"), offset=0)}, name="IPSEC_CIPHER_TRANSFORM0", pack=False, align=None)}, name="IPSEC_AUTH_AND_CIPHER_TRANSFORM0", pack=False, align=None), offset=0), "espAuthFwTransform": SimTypePointer(SimStruct({"authTransformId": SimStruct({"authType": SimTypeInt(signed=False, label="IPSEC_AUTH_TYPE"), "authConfig": SimTypeChar(label="Byte")}, name="IPSEC_AUTH_TRANSFORM_ID0", pack=False, align=None), "cryptoModuleId": SimTypePointer(SimTypeBottom(label="Guid"), offset=0)}, name="IPSEC_AUTH_TRANSFORM0", pack=False, align=None), offset=0)}, name="<anon>", label="None")}, name="IPSEC_SA_TRANSFORM0", pack=False, align=None), offset=0), "pfsGroup": SimTypeInt(signed=False, label="IPSEC_PFS_GROUP")}, name="IPSEC_PROPOSAL0", pack=False, align=None), offset=0), "flags": SimTypeInt(signed=False, label="IPSEC_POLICY_FLAG"), "ndAllowClearTimeoutSeconds": SimTypeInt(signed=False, label="UInt32"), "saIdleTimeout": SimStruct({"idleTimeoutSeconds": SimTypeInt(signed=False, label="UInt32"), "idleTimeoutSecondsFailOver": SimTypeInt(signed=False, label="UInt32")}, name="IPSEC_SA_IDLE_TIMEOUT0", pack=False, align=None), "emPolicy": SimTypePointer(SimStruct({"numAuthenticationMethods": SimTypeInt(signed=False, label="UInt32"), "authenticationMethods": SimTypePointer(SimStruct({"authenticationMethodType": SimTypeInt(signed=False, label="IKEEXT_AUTHENTICATION_METHOD_TYPE"), "Anonymous": SimUnion({"presharedKeyAuthentication": SimStruct({"presharedKey": SimStruct({"size": SimTypeInt(signed=False, label="UInt32"), "data": SimTypePointer(SimTypeChar(label="Byte"), offset=0)}, name="FWP_BYTE_BLOB", pack=False, align=None), "flags": SimTypeInt(signed=False, label="IKEEXT_PRESHARED_KEY_AUTHENTICATION_FLAGS")}, name="IKEEXT_PRESHARED_KEY_AUTHENTICATION1", pack=False, align=None), "certificateAuthentication": SimStruct({"inboundConfigType": SimTypeInt(signed=False, label="IKEEXT_CERT_CONFIG_TYPE"), "Anonymous1": SimUnion({"Anonymous1": SimStruct({"inboundRootArraySize": SimTypeInt(signed=False, label="UInt32"), "inboundRootCriteria": SimTypePointer(SimStruct({"certData": SimStruct({"size": SimTypeInt(signed=False, label="UInt32"), "data": SimTypePointer(SimTypeChar(label="Byte"), offset=0)}, name="FWP_BYTE_BLOB", pack=False, align=None), "certHash": SimStruct({"size": SimTypeInt(signed=False, label="UInt32"), "data": SimTypePointer(SimTypeChar(label="Byte"), offset=0)}, name="FWP_BYTE_BLOB", pack=False, align=None), "eku": SimTypePointer(SimStruct({"numEku": SimTypeInt(signed=False, label="UInt32"), "eku": SimTypePointer(SimTypePointer(SimTypeChar(label="Byte"), offset=0), offset=0)}, name="IKEEXT_CERT_EKUS0", pack=False, align=None), offset=0), "name": SimTypePointer(SimStruct({"nameType": SimTypeInt(signed=False, label="IKEEXT_CERT_CRITERIA_NAME_TYPE"), "certName": SimTypePointer(SimTypeChar(label="Char"), offset=0)}, name="IKEEXT_CERT_NAME0", pack=False, align=None), offset=0), "flags": SimTypeInt(signed=False, label="UInt32")}, name="IKEEXT_CERTIFICATE_CRITERIA0", pack=False, align=None), offset=0)}, name="_Anonymous1_e__Struct", pack=False, align=None), "Anonymous2": SimStruct({"inboundEnterpriseStoreArraySize": SimTypeInt(signed=False, label="UInt32"), "inboundEnterpriseStoreCriteria": SimTypePointer(SimStruct({"certData": SimStruct({"size": SimTypeInt(signed=False, label="UInt32"), "data": SimTypePointer(SimTypeChar(label="Byte"), offset=0)}, name="FWP_BYTE_BLOB", pack=False, align=None), "certHash": SimStruct({"size": SimTypeInt(signed=False, label="UInt32"), "data": SimTypePointer(SimTypeChar(label="Byte"), offset=0)}, name="FWP_BYTE_BLOB", pack=False, align=None), "eku": SimTypePointer(SimStruct({"numEku": SimTypeInt(signed=False, label="UInt32"), "eku": SimTypePointer(SimTypePointer(SimTypeChar(label="Byte"), offset=0), offset=0)}, name="IKEEXT_CERT_EKUS0", pack=False, align=None), offset=0), "name": SimTypePointer(SimStruct({"nameType": SimTypeInt(signed=False, label="IKEEXT_CERT_CRITERIA_NAME_TYPE"), "certName": SimTypePointer(SimTypeChar(label="Char"), offset=0)}, name="IKEEXT_CERT_NAME0", pack=False, align=None), offset=0), "flags": SimTypeInt(signed=False, label="UInt32")}, name="IKEEXT_CERTIFICATE_CRITERIA0", pack=False, align=None), offset=0)}, name="_Anonymous2_e__Struct", pack=False, align=None), "Anonymous3": SimStruct({"inboundRootStoreArraySize": SimTypeInt(signed=False, label="UInt32"), "inboundTrustedRootStoreCriteria": SimTypePointer(SimStruct({"certData": SimStruct({"size": SimTypeInt(signed=False, label="UInt32"), "data": SimTypePointer(SimTypeChar(label="Byte"), offset=0)}, name="FWP_BYTE_BLOB", pack=False, align=None), "certHash": SimStruct({"size": SimTypeInt(signed=False, label="UInt32"), "data": SimTypePointer(SimTypeChar(label="Byte"), offset=0)}, name="FWP_BYTE_BLOB", pack=False, align=None), "eku": SimTypePointer(SimStruct({"numEku": SimTypeInt(signed=False, label="UInt32"), "eku": SimTypePointer(SimTypePointer(SimTypeChar(label="Byte"), offset=0), offset=0)}, name="IKEEXT_CERT_EKUS0", pack=False, align=None), offset=0), "name": SimTypePointer(SimStruct({"nameType": SimTypeInt(signed=False, label="IKEEXT_CERT_CRITERIA_NAME_TYPE"), "certName": SimTypePointer(SimTypeChar(label="Char"), offset=0)}, name="IKEEXT_CERT_NAME0", pack=False, align=None), offset=0), "flags": SimTypeInt(signed=False, label="UInt32")}, name="IKEEXT_CERTIFICATE_CRITERIA0", pack=False, align=None), offset=0)}, name="_Anonymous3_e__Struct", pack=False, align=None)}, name="<anon>", label="None"), "outboundConfigType": SimTypeInt(signed=False, label="IKEEXT_CERT_CONFIG_TYPE"), "Anonymous2": SimUnion({"Anonymous1": SimStruct({"outboundRootArraySize": SimTypeInt(signed=False, label="UInt32"), "outboundRootCriteria": SimTypePointer(SimStruct({"certData": SimStruct({"size": SimTypeInt(signed=False, label="UInt32"), "data": SimTypePointer(SimTypeChar(label="Byte"), offset=0)}, name="FWP_BYTE_BLOB", pack=False, align=None), "certHash": SimStruct({"size": SimTypeInt(signed=False, label="UInt32"), "data": SimTypePointer(SimTypeChar(label="Byte"), offset=0)}, name="FWP_BYTE_BLOB", pack=False, align=None), "eku": SimTypePointer(SimStruct({"numEku": SimTypeInt(signed=False, label="UInt32"), "eku": SimTypePointer(SimTypePointer(SimTypeChar(label="Byte"), offset=0), offset=0)}, name="IKEEXT_CERT_EKUS0", pack=False, align=None), offset=0), "name": SimTypePointer(SimStruct({"nameType": SimTypeInt(signed=False, label="IKEEXT_CERT_CRITERIA_NAME_TYPE"), "certName": SimTypePointer(SimTypeChar(label="Char"), offset=0)}, name="IKEEXT_CERT_NAME0", pack=False, align=None), offset=0), "flags": SimTypeInt(signed=False, label="UInt32")}, name="IKEEXT_CERTIFICATE_CRITERIA0", pack=False, align=None), offset=0)}, name="_Anonymous1_e__Struct", pack=False, align=None), "Anonymous2": SimStruct({"outboundEnterpriseStoreArraySize": SimTypeInt(signed=False, label="UInt32"), "outboundEnterpriseStoreCriteria": SimTypePointer(SimStruct({"certData": SimStruct({"size": SimTypeInt(signed=False, label="UInt32"), "data": SimTypePointer(SimTypeChar(label="Byte"), offset=0)}, name="FWP_BYTE_BLOB", pack=False, align=None), "certHash": SimStruct({"size": SimTypeInt(signed=False, label="UInt32"), "data": SimTypePointer(SimTypeChar(label="Byte"), offset=0)}, name="FWP_BYTE_BLOB", pack=False, align=None), "eku": SimTypePointer(SimStruct({"numEku": SimTypeInt(signed=False, label="UInt32"), "eku": SimTypePointer(SimTypePointer(SimTypeChar(label="Byte"), offset=0), offset=0)}, name="IKEEXT_CERT_EKUS0", pack=False, align=None), offset=0), "name": SimTypePointer(SimStruct({"nameType": SimTypeInt(signed=False, label="IKEEXT_CERT_CRITERIA_NAME_TYPE"), "certName": SimTypePointer(SimTypeChar(label="Char"), offset=0)}, name="IKEEXT_CERT_NAME0", pack=False, align=None), offset=0), "flags": SimTypeInt(signed=False, label="UInt32")}, name="IKEEXT_CERTIFICATE_CRITERIA0", pack=False, align=None), offset=0)}, name="_Anonymous2_e__Struct", pack=False, align=None), "Anonymous3": SimStruct({"outboundRootStoreArraySize": SimTypeInt(signed=False, label="UInt32"), "outboundTrustedRootStoreCriteria": SimTypePointer(SimStruct({"certData": SimStruct({"size": SimTypeInt(signed=False, label="UInt32"), "data": SimTypePointer(SimTypeChar(label="Byte"), offset=0)}, name="FWP_BYTE_BLOB", pack=False, align=None), "certHash": SimStruct({"size": SimTypeInt(signed=False, label="UInt32"), "data": SimTypePointer(SimTypeChar(label="Byte"), offset=0)}, name="FWP_BYTE_BLOB", pack=False, align=None), "eku": SimTypePointer(SimStruct({"numEku": SimTypeInt(signed=False, label="UInt32"), "eku": SimTypePointer(SimTypePointer(SimTypeChar(label="Byte"), offset=0), offset=0)}, name="IKEEXT_CERT_EKUS0", pack=False, align=None), offset=0), "name": SimTypePointer(SimStruct({"nameType": SimTypeInt(signed=False, label="IKEEXT_CERT_CRITERIA_NAME_TYPE"), "certName": SimTypePointer(SimTypeChar(label="Char"), offset=0)}, name="IKEEXT_CERT_NAME0", pack=False, align=None), offset=0), "flags": SimTypeInt(signed=False, label="UInt32")}, name="IKEEXT_CERTIFICATE_CRITERIA0", pack=False, align=None), offset=0)}, name="_Anonymous3_e__Struct", pack=False, align=None)}, name="<anon>", label="None"), "flags": SimTypeInt(signed=False, label="IKEEXT_CERT_AUTH"), "localCertLocationUrl": SimStruct({"size": SimTypeInt(signed=False, label="UInt32"), "data": SimTypePointer(SimTypeChar(label="Byte"), offset=0)}, name="FWP_BYTE_BLOB", pack=False, align=None)}, name="IKEEXT_CERTIFICATE_AUTHENTICATION2", pack=False, align=None), "kerberosAuthentication": SimStruct({"flags": SimTypeInt(signed=False, label="IKEEXT_KERBEROS_AUTHENTICATION_FLAGS"), "proxyServer": SimTypePointer(SimTypeChar(label="Char"), offset=0)}, name="IKEEXT_KERBEROS_AUTHENTICATION1", pack=False, align=None), "reservedAuthentication": SimStruct({"flags": SimTypeInt(signed=False, label="IKEEXT_RESERVED_AUTHENTICATION_FLAGS")}, name="IKEEXT_RESERVED_AUTHENTICATION0", pack=False, align=None), "ntlmV2Authentication": SimStruct({"flags": SimTypeInt(signed=False, label="UInt32")}, name="IKEEXT_NTLM_V2_AUTHENTICATION0", pack=False, align=None), "sslAuthentication": SimStruct({"inboundConfigType": SimTypeInt(signed=False, label="IKEEXT_CERT_CONFIG_TYPE"), "Anonymous1": SimUnion({"Anonymous1": SimStruct({"inboundRootArraySize": SimTypeInt(signed=False, label="UInt32"), "inboundRootCriteria": SimTypePointer(SimStruct({"certData": SimStruct({"size": SimTypeInt(signed=False, label="UInt32"), "data": SimTypePointer(SimTypeChar(label="Byte"), offset=0)}, name="FWP_BYTE_BLOB", pack=False, align=None), "certHash": SimStruct({"size": SimTypeInt(signed=False, label="UInt32"), "data": SimTypePointer(SimTypeChar(label="Byte"), offset=0)}, name="FWP_BYTE_BLOB", pack=False, align=None), "eku": SimTypePointer(SimStruct({"numEku": SimTypeInt(signed=False, label="UInt32"), "eku": SimTypePointer(SimTypePointer(SimTypeChar(label="Byte"), offset=0), offset=0)}, name="IKEEXT_CERT_EKUS0", pack=False, align=None), offset=0), "name": SimTypePointer(SimStruct({"nameType": SimTypeInt(signed=False, label="IKEEXT_CERT_CRITERIA_NAME_TYPE"), "certName": SimTypePointer(SimTypeChar(label="Char"), offset=0)}, name="IKEEXT_CERT_NAME0", pack=False, align=None), offset=0), "flags": SimTypeInt(signed=False, label="UInt32")}, name="IKEEXT_CERTIFICATE_CRITERIA0", pack=False, align=None), offset=0)}, name="_Anonymous1_e__Struct", pack=False, align=None), "Anonymous2": SimStruct({"inboundEnterpriseStoreArraySize": SimTypeInt(signed=False, label="UInt32"), "inboundEnterpriseStoreCriteria": SimTypePointer(SimStruct({"certData": SimStruct({"size": SimTypeInt(signed=False, label="UInt32"), "data": SimTypePointer(SimTypeChar(label="Byte"), offset=0)}, name="FWP_BYTE_BLOB", pack=False, align=None), "certHash": SimStruct({"size": SimTypeInt(signed=False, label="UInt32"), "data": SimTypePointer(SimTypeChar(label="Byte"), offset=0)}, name="FWP_BYTE_BLOB", pack=False, align=None), "eku": SimTypePointer(SimStruct({"numEku": SimTypeInt(signed=False, label="UInt32"), "eku": SimTypePointer(SimTypePointer(SimTypeChar(label="Byte"), offset=0), offset=0)}, name="IKEEXT_CERT_EKUS0", pack=False, align=None), offset=0), "name": SimTypePointer(SimStruct({"nameType": SimTypeInt(signed=False, label="IKEEXT_CERT_CRITERIA_NAME_TYPE"), "certName": SimTypePointer(SimTypeChar(label="Char"), offset=0)}, name="IKEEXT_CERT_NAME0", pack=False, align=None), offset=0), "flags": SimTypeInt(signed=False, label="UInt32")}, name="IKEEXT_CERTIFICATE_CRITERIA0", pack=False, align=None), offset=0)}, name="_Anonymous2_e__Struct", pack=False, align=None), "Anonymous3": SimStruct({"inboundRootStoreArraySize": SimTypeInt(signed=False, label="UInt32"), "inboundTrustedRootStoreCriteria": SimTypePointer(SimStruct({"certData": SimStruct({"size": SimTypeInt(signed=False, label="UInt32"), "data": SimTypePointer(SimTypeChar(label="Byte"), offset=0)}, name="FWP_BYTE_BLOB", pack=False, align=None), "certHash": SimStruct({"size": SimTypeInt(signed=False, label="UInt32"), "data": SimTypePointer(SimTypeChar(label="Byte"), offset=0)}, name="FWP_BYTE_BLOB", pack=False, align=None), "eku": SimTypePointer(SimStruct({"numEku": SimTypeInt(signed=False, label="UInt32"), "eku": SimTypePointer(SimTypePointer(SimTypeChar(label="Byte"), offset=0), offset=0)}, name="IKEEXT_CERT_EKUS0", pack=False, align=None), offset=0), "name": SimTypePointer(SimStruct({"nameType": SimTypeInt(signed=False, label="IKEEXT_CERT_CRITERIA_NAME_TYPE"), "certName": SimTypePointer(SimTypeChar(label="Char"), offset=0)}, name="IKEEXT_CERT_NAME0", pack=False, align=None), offset=0), "flags": SimTypeInt(signed=False, label="UInt32")}, name="IKEEXT_CERTIFICATE_CRITERIA0", pack=False, align=None), offset=0)}, name="_Anonymous3_e__Struct", pack=False, align=None)}, name="<anon>", label="None"), "outboundConfigType": SimTypeInt(signed=False, label="IKEEXT_CERT_CONFIG_TYPE"), "Anonymous2": SimUnion({"Anonymous1": SimStruct({"outboundRootArraySize": SimTypeInt(signed=False, label="UInt32"), "outboundRootCriteria": SimTypePointer(SimStruct({"certData": SimStruct({"size": SimTypeInt(signed=False, label="UInt32"), "data": SimTypePointer(SimTypeChar(label="Byte"), offset=0)}, name="FWP_BYTE_BLOB", pack=False, align=None), "certHash": SimStruct({"size": SimTypeInt(signed=False, label="UInt32"), "data": SimTypePointer(SimTypeChar(label="Byte"), offset=0)}, name="FWP_BYTE_BLOB", pack=False, align=None), "eku": SimTypePointer(SimStruct({"numEku": SimTypeInt(signed=False, label="UInt32"), "eku": SimTypePointer(SimTypePointer(SimTypeChar(label="Byte"), offset=0), offset=0)}, name="IKEEXT_CERT_EKUS0", pack=False, align=None), offset=0), "name": SimTypePointer(SimStruct({"nameType": SimTypeInt(signed=False, label="IKEEXT_CERT_CRITERIA_NAME_TYPE"), "certName": SimTypePointer(SimTypeChar(label="Char"), offset=0)}, name="IKEEXT_CERT_NAME0", pack=False, align=None), offset=0), "flags": SimTypeInt(signed=False, label="UInt32")}, name="IKEEXT_CERTIFICATE_CRITERIA0", pack=False, align=None), offset=0)}, name="_Anonymous1_e__Struct", pack=False, align=None), "Anonymous2": SimStruct({"outboundEnterpriseStoreArraySize": SimTypeInt(signed=False, label="UInt32"), "outboundEnterpriseStoreCriteria": SimTypePointer(SimStruct({"certData": SimStruct({"size": SimTypeInt(signed=False, label="UInt32"), "data": SimTypePointer(SimTypeChar(label="Byte"), offset=0)}, name="FWP_BYTE_BLOB", pack=False, align=None), "certHash": SimStruct({"size": SimTypeInt(signed=False, label="UInt32"), "data": SimTypePointer(SimTypeChar(label="Byte"), offset=0)}, name="FWP_BYTE_BLOB", pack=False, align=None), "eku": SimTypePointer(SimStruct({"numEku": SimTypeInt(signed=False, label="UInt32"), "eku": SimTypePointer(SimTypePointer(SimTypeChar(label="Byte"), offset=0), offset=0)}, name="IKEEXT_CERT_EKUS0", pack=False, align=None), offset=0), "name": SimTypePointer(SimStruct({"nameType": SimTypeInt(signed=False, label="IKEEXT_CERT_CRITERIA_NAME_TYPE"), "certName": SimTypePointer(SimTypeChar(label="Char"), offset=0)}, name="IKEEXT_CERT_NAME0", pack=False, align=None), offset=0), "flags": SimTypeInt(signed=False, label="UInt32")}, name="IKEEXT_CERTIFICATE_CRITERIA0", pack=False, align=None), offset=0)}, name="_Anonymous2_e__Struct", pack=False, align=None), "Anonymous3": SimStruct({"outboundRootStoreArraySize": SimTypeInt(signed=False, label="UInt32"), "outboundTrustedRootStoreCriteria": SimTypePointer(SimStruct({"certData": SimStruct({"size": SimTypeInt(signed=False, label="UInt32"), "data": SimTypePointer(SimTypeChar(label="Byte"), offset=0)}, name="FWP_BYTE_BLOB", pack=False, align=None), "certHash": SimStruct({"size": SimTypeInt(signed=False, label="UInt32"), "data": SimTypePointer(SimTypeChar(label="Byte"), offset=0)}, name="FWP_BYTE_BLOB", pack=False, align=None), "eku": SimTypePointer(SimStruct({"numEku": SimTypeInt(signed=False, label="UInt32"), "eku": SimTypePointer(SimTypePointer(SimTypeChar(label="Byte"), offset=0), offset=0)}, name="IKEEXT_CERT_EKUS0", pack=False, align=None), offset=0), "name": SimTypePointer(SimStruct({"nameType": SimTypeInt(signed=False, label="IKEEXT_CERT_CRITERIA_NAME_TYPE"), "certName": SimTypePointer(SimTypeChar(label="Char"), offset=0)}, name="IKEEXT_CERT_NAME0", pack=False, align=None), offset=0), "flags": SimTypeInt(signed=False, label="UInt32")}, name="IKEEXT_CERTIFICATE_CRITERIA0", pack=False, align=None), offset=0)}, name="_Anonymous3_e__Struct", pack=False, align=None)}, name="<anon>", label="None"), "flags": SimTypeInt(signed=False, label="IKEEXT_CERT_AUTH"), "localCertLocationUrl": SimStruct({"size": SimTypeInt(signed=False, label="UInt32"), "data": SimTypePointer(SimTypeChar(label="Byte"), offset=0)}, name="FWP_BYTE_BLOB", pack=False, align=None)}, name="IKEEXT_CERTIFICATE_AUTHENTICATION2", pack=False, align=None), "cgaAuthentication": SimStruct({"keyContainerName": SimTypePointer(SimTypeChar(label="Char"), offset=0), "cspName": SimTypePointer(SimTypeChar(label="Char"), offset=0), "cspType": SimTypeInt(signed=False, label="UInt32"), "cgaModifier": SimStruct({"byteArray16": SimTypeFixedSizeArray(SimTypeChar(label="Byte"), 16)}, name="FWP_BYTE_ARRAY16", pack=False, align=None), "cgaCollisionCount": SimTypeChar(label="Byte")}, name="IKEEXT_IPV6_CGA_AUTHENTICATION0", pack=False, align=None), "eapAuthentication": SimStruct({"flags": SimTypeInt(signed=False, label="IKEEXT_EAP_AUTHENTICATION_FLAGS")}, name="IKEEXT_EAP_AUTHENTICATION0", pack=False, align=None)}, name="<anon>", label="None")}, name="IKEEXT_AUTHENTICATION_METHOD2", pack=False, align=None), offset=0), "initiatorImpersonationType": SimTypeInt(signed=False, label="IKEEXT_AUTHENTICATION_IMPERSONATION_TYPE")}, name="IKEEXT_EM_POLICY2", pack=False, align=None), offset=0)}, name="IPSEC_TRANSPORT_POLICY2", pack=False, align=None), offset=0), "authipQmTunnelPolicy": SimTypePointer(SimStruct({"flags": SimTypeInt(signed=False, label="IPSEC_POLICY_FLAG"), "numIpsecProposals": SimTypeInt(signed=False, label="UInt32"), "ipsecProposals": SimTypePointer(SimStruct({"lifetime": SimStruct({"lifetimeSeconds": SimTypeInt(signed=False, label="UInt32"), "lifetimeKilobytes": SimTypeInt(signed=False, label="UInt32"), "lifetimePackets": SimTypeInt(signed=False, label="UInt32")}, name="IPSEC_SA_LIFETIME0", pack=False, align=None), "numSaTransforms": SimTypeInt(signed=False, label="UInt32"), "saTransforms": SimTypePointer(SimStruct({"ipsecTransformType": SimTypeInt(signed=False, label="IPSEC_TRANSFORM_TYPE"), "Anonymous": SimUnion({"ahTransform": SimTypePointer(SimStruct({"authTransformId": SimStruct({"authType": SimTypeInt(signed=False, label="IPSEC_AUTH_TYPE"), "authConfig": SimTypeChar(label="Byte")}, name="IPSEC_AUTH_TRANSFORM_ID0", pack=False, align=None), "cryptoModuleId": SimTypePointer(SimTypeBottom(label="Guid"), offset=0)}, name="IPSEC_AUTH_TRANSFORM0", pack=False, align=None), offset=0), "espAuthTransform": SimTypePointer(SimStruct({"authTransformId": SimStruct({"authType": SimTypeInt(signed=False, label="IPSEC_AUTH_TYPE"), "authConfig": SimTypeChar(label="Byte")}, name="IPSEC_AUTH_TRANSFORM_ID0", pack=False, align=None), "cryptoModuleId": SimTypePointer(SimTypeBottom(label="Guid"), offset=0)}, name="IPSEC_AUTH_TRANSFORM0", pack=False, align=None), offset=0), "espCipherTransform": SimTypePointer(SimStruct({"cipherTransformId": SimStruct({"cipherType": SimTypeInt(signed=False, label="IPSEC_CIPHER_TYPE"), "cipherConfig": SimTypeChar(label="Byte")}, name="IPSEC_CIPHER_TRANSFORM_ID0", pack=False, align=None), "cryptoModuleId": SimTypePointer(SimTypeBottom(label="Guid"), offset=0)}, name="IPSEC_CIPHER_TRANSFORM0", pack=False, align=None), offset=0), "espAuthAndCipherTransform": SimTypePointer(SimStruct({"authTransform": SimStruct({"authTransformId": SimStruct({"authType": SimTypeInt(signed=False, label="IPSEC_AUTH_TYPE"), "authConfig": SimTypeChar(label="Byte")}, name="IPSEC_AUTH_TRANSFORM_ID0", pack=False, align=None), "cryptoModuleId": SimTypePointer(SimTypeBottom(label="Guid"), offset=0)}, name="IPSEC_AUTH_TRANSFORM0", pack=False, align=None), "cipherTransform": SimStruct({"cipherTransformId": SimStruct({"cipherType": SimTypeInt(signed=False, label="IPSEC_CIPHER_TYPE"), "cipherConfig": SimTypeChar(label="Byte")}, name="IPSEC_CIPHER_TRANSFORM_ID0", pack=False, align=None), "cryptoModuleId": SimTypePointer(SimTypeBottom(label="Guid"), offset=0)}, name="IPSEC_CIPHER_TRANSFORM0", pack=False, align=None)}, name="IPSEC_AUTH_AND_CIPHER_TRANSFORM0", pack=False, align=None), offset=0), "espAuthFwTransform": SimTypePointer(SimStruct({"authTransformId": SimStruct({"authType": SimTypeInt(signed=False, label="IPSEC_AUTH_TYPE"), "authConfig": SimTypeChar(label="Byte")}, name="IPSEC_AUTH_TRANSFORM_ID0", pack=False, align=None), "cryptoModuleId": SimTypePointer(SimTypeBottom(label="Guid"), offset=0)}, name="IPSEC_AUTH_TRANSFORM0", pack=False, align=None), offset=0)}, name="<anon>", label="None")}, name="IPSEC_SA_TRANSFORM0", pack=False, align=None), offset=0), "pfsGroup": SimTypeInt(signed=False, label="IPSEC_PFS_GROUP")}, name="IPSEC_PROPOSAL0", pack=False, align=None), offset=0), "tunnelEndpoints": SimStruct({"ipVersion": SimTypeInt(signed=False, label="FWP_IP_VERSION"), "Anonymous1": SimUnion({"localV4Address": SimTypeInt(signed=False, label="UInt32"), "localV6Address": SimTypeFixedSizeArray(SimTypeChar(label="Byte"), 16)}, name="<anon>", label="None"), "Anonymous2": SimUnion({"remoteV4Address": SimTypeInt(signed=False, label="UInt32"), "remoteV6Address": SimTypeFixedSizeArray(SimTypeChar(label="Byte"), 16)}, name="<anon>", label="None"), "localIfLuid": SimTypeLongLong(signed=False, label="UInt64"), "remoteFqdn": SimTypePointer(SimTypeChar(label="Char"), offset=0), "numAddresses": SimTypeInt(signed=False, label="UInt32"), "remoteAddresses": SimTypePointer(SimStruct({"ipVersion": SimTypeInt(signed=False, label="FWP_IP_VERSION"), "Anonymous": SimUnion({"v4Address": SimTypeInt(signed=False, label="UInt32"), "v6Address": SimTypeFixedSizeArray(SimTypeChar(label="Byte"), 16)}, name="<anon>", label="None")}, name="IPSEC_TUNNEL_ENDPOINT0", pack=False, align=None), offset=0)}, name="IPSEC_TUNNEL_ENDPOINTS2", pack=False, align=None), "saIdleTimeout": SimStruct({"idleTimeoutSeconds": SimTypeInt(signed=False, label="UInt32"), "idleTimeoutSecondsFailOver": SimTypeInt(signed=False, label="UInt32")}, name="IPSEC_SA_IDLE_TIMEOUT0", pack=False, align=None), "emPolicy": SimTypePointer(SimStruct({"numAuthenticationMethods": SimTypeInt(signed=False, label="UInt32"), "authenticationMethods": SimTypePointer(SimStruct({"authenticationMethodType": SimTypeInt(signed=False, label="IKEEXT_AUTHENTICATION_METHOD_TYPE"), "Anonymous": SimUnion({"presharedKeyAuthentication": SimStruct({"presharedKey": SimStruct({"size": SimTypeInt(signed=False, label="UInt32"), "data": SimTypePointer(SimTypeChar(label="Byte"), offset=0)}, name="FWP_BYTE_BLOB", pack=False, align=None), "flags": SimTypeInt(signed=False, label="IKEEXT_PRESHARED_KEY_AUTHENTICATION_FLAGS")}, name="IKEEXT_PRESHARED_KEY_AUTHENTICATION1", pack=False, align=None), "certificateAuthentication": SimStruct({"inboundConfigType": SimTypeInt(signed=False, label="IKEEXT_CERT_CONFIG_TYPE"), "Anonymous1": SimUnion({"Anonymous1": SimStruct({"inboundRootArraySize": SimTypeInt(signed=False, label="UInt32"), "inboundRootCriteria": SimTypePointer(SimStruct({"certData": SimStruct({"size": SimTypeInt(signed=False, label="UInt32"), "data": SimTypePointer(SimTypeChar(label="Byte"), offset=0)}, name="FWP_BYTE_BLOB", pack=False, align=None), "certHash": SimStruct({"size": SimTypeInt(signed=False, label="UInt32"), "data": SimTypePointer(SimTypeChar(label="Byte"), offset=0)}, name="FWP_BYTE_BLOB", pack=False, align=None), "eku": SimTypePointer(SimStruct({"numEku": SimTypeInt(signed=False, label="UInt32"), "eku": SimTypePointer(SimTypePointer(SimTypeChar(label="Byte"), offset=0), offset=0)}, name="IKEEXT_CERT_EKUS0", pack=False, align=None), offset=0), "name": SimTypePointer(SimStruct({"nameType": SimTypeInt(signed=False, label="IKEEXT_CERT_CRITERIA_NAME_TYPE"), "certName": SimTypePointer(SimTypeChar(label="Char"), offset=0)}, name="IKEEXT_CERT_NAME0", pack=False, align=None), offset=0), "flags": SimTypeInt(signed=False, label="UInt32")}, name="IKEEXT_CERTIFICATE_CRITERIA0", pack=False, align=None), offset=0)}, name="_Anonymous1_e__Struct", pack=False, align=None), "Anonymous2": SimStruct({"inboundEnterpriseStoreArraySize": SimTypeInt(signed=False, label="UInt32"), "inboundEnterpriseStoreCriteria": SimTypePointer(SimStruct({"certData": SimStruct({"size": SimTypeInt(signed=False, label="UInt32"), "data": SimTypePointer(SimTypeChar(label="Byte"), offset=0)}, name="FWP_BYTE_BLOB", pack=False, align=None), "certHash": SimStruct({"size": SimTypeInt(signed=False, label="UInt32"), "data": SimTypePointer(SimTypeChar(label="Byte"), offset=0)}, name="FWP_BYTE_BLOB", pack=False, align=None), "eku": SimTypePointer(SimStruct({"numEku": SimTypeInt(signed=False, label="UInt32"), "eku": SimTypePointer(SimTypePointer(SimTypeChar(label="Byte"), offset=0), offset=0)}, name="IKEEXT_CERT_EKUS0", pack=False, align=None), offset=0), "name": SimTypePointer(SimStruct({"nameType": SimTypeInt(signed=False, label="IKEEXT_CERT_CRITERIA_NAME_TYPE"), "certName": SimTypePointer(SimTypeChar(label="Char"), offset=0)}, name="IKEEXT_CERT_NAME0", pack=False, align=None), offset=0), "flags": SimTypeInt(signed=False, label="UInt32")}, name="IKEEXT_CERTIFICATE_CRITERIA0", pack=False, align=None), offset=0)}, name="_Anonymous2_e__Struct", pack=False, align=None), "Anonymous3": SimStruct({"inboundRootStoreArraySize": SimTypeInt(signed=False, label="UInt32"), "inboundTrustedRootStoreCriteria": SimTypePointer(SimStruct({"certData": SimStruct({"size": SimTypeInt(signed=False, label="UInt32"), "data": SimTypePointer(SimTypeChar(label="Byte"), offset=0)}, name="FWP_BYTE_BLOB", pack=False, align=None), "certHash": SimStruct({"size": SimTypeInt(signed=False, label="UInt32"), "data": SimTypePointer(SimTypeChar(label="Byte"), offset=0)}, name="FWP_BYTE_BLOB", pack=False, align=None), "eku": SimTypePointer(SimStruct({"numEku": SimTypeInt(signed=False, label="UInt32"), "eku": SimTypePointer(SimTypePointer(SimTypeChar(label="Byte"), offset=0), offset=0)}, name="IKEEXT_CERT_EKUS0", pack=False, align=None), offset=0), "name": SimTypePointer(SimStruct({"nameType": SimTypeInt(signed=False, label="IKEEXT_CERT_CRITERIA_NAME_TYPE"), "certName": SimTypePointer(SimTypeChar(label="Char"), offset=0)}, name="IKEEXT_CERT_NAME0", pack=False, align=None), offset=0), "flags": SimTypeInt(signed=False, label="UInt32")}, name="IKEEXT_CERTIFICATE_CRITERIA0", pack=False, align=None), offset=0)}, name="_Anonymous3_e__Struct", pack=False, align=None)}, name="<anon>", label="None"), "outboundConfigType": SimTypeInt(signed=False, label="IKEEXT_CERT_CONFIG_TYPE"), "Anonymous2": SimUnion({"Anonymous1": SimStruct({"outboundRootArraySize": SimTypeInt(signed=False, label="UInt32"), "outboundRootCriteria": SimTypePointer(SimStruct({"certData": SimStruct({"size": SimTypeInt(signed=False, label="UInt32"), "data": SimTypePointer(SimTypeChar(label="Byte"), offset=0)}, name="FWP_BYTE_BLOB", pack=False, align=None), "certHash": SimStruct({"size": SimTypeInt(signed=False, label="UInt32"), "data": SimTypePointer(SimTypeChar(label="Byte"), offset=0)}, name="FWP_BYTE_BLOB", pack=False, align=None), "eku": SimTypePointer(SimStruct({"numEku": SimTypeInt(signed=False, label="UInt32"), "eku": SimTypePointer(SimTypePointer(SimTypeChar(label="Byte"), offset=0), offset=0)}, name="IKEEXT_CERT_EKUS0", pack=False, align=None), offset=0), "name": SimTypePointer(SimStruct({"nameType": SimTypeInt(signed=False, label="IKEEXT_CERT_CRITERIA_NAME_TYPE"), "certName": SimTypePointer(SimTypeChar(label="Char"), offset=0)}, name="IKEEXT_CERT_NAME0", pack=False, align=None), offset=0), "flags": SimTypeInt(signed=False, label="UInt32")}, name="IKEEXT_CERTIFICATE_CRITERIA0", pack=False, align=None), offset=0)}, name="_Anonymous1_e__Struct", pack=False, align=None), "Anonymous2": SimStruct({"outboundEnterpriseStoreArraySize": SimTypeInt(signed=False, label="UInt32"), "outboundEnterpriseStoreCriteria": SimTypePointer(SimStruct({"certData": SimStruct({"size": SimTypeInt(signed=False, label="UInt32"), "data": SimTypePointer(SimTypeChar(label="Byte"), offset=0)}, name="FWP_BYTE_BLOB", pack=False, align=None), "certHash": SimStruct({"size": SimTypeInt(signed=False, label="UInt32"), "data": SimTypePointer(SimTypeChar(label="Byte"), offset=0)}, name="FWP_BYTE_BLOB", pack=False, align=None), "eku": SimTypePointer(SimStruct({"numEku": SimTypeInt(signed=False, label="UInt32"), "eku": SimTypePointer(SimTypePointer(SimTypeChar(label="Byte"), offset=0), offset=0)}, name="IKEEXT_CERT_EKUS0", pack=False, align=None), offset=0), "name": SimTypePointer(SimStruct({"nameType": SimTypeInt(signed=False, label="IKEEXT_CERT_CRITERIA_NAME_TYPE"), "certName": SimTypePointer(SimTypeChar(label="Char"), offset=0)}, name="IKEEXT_CERT_NAME0", pack=False, align=None), offset=0), "flags": SimTypeInt(signed=False, label="UInt32")}, name="IKEEXT_CERTIFICATE_CRITERIA0", pack=False, align=None), offset=0)}, name="_Anonymous2_e__Struct", pack=False, align=None), "Anonymous3": SimStruct({"outboundRootStoreArraySize": SimTypeInt(signed=False, label="UInt32"), "outboundTrustedRootStoreCriteria": SimTypePointer(SimStruct({"certData": SimStruct({"size": SimTypeInt(signed=False, label="UInt32"), "data": SimTypePointer(SimTypeChar(label="Byte"), offset=0)}, name="FWP_BYTE_BLOB", pack=False, align=None), "certHash": SimStruct({"size": SimTypeInt(signed=False, label="UInt32"), "data": SimTypePointer(SimTypeChar(label="Byte"), offset=0)}, name="FWP_BYTE_BLOB", pack=False, align=None), "eku": SimTypePointer(SimStruct({"numEku": SimTypeInt(signed=False, label="UInt32"), "eku": SimTypePointer(SimTypePointer(SimTypeChar(label="Byte"), offset=0), offset=0)}, name="IKEEXT_CERT_EKUS0", pack=False, align=None), offset=0), "name": SimTypePointer(SimStruct({"nameType": SimTypeInt(signed=False, label="IKEEXT_CERT_CRITERIA_NAME_TYPE"), "certName": SimTypePointer(SimTypeChar(label="Char"), offset=0)}, name="IKEEXT_CERT_NAME0", pack=False, align=None), offset=0), "flags": SimTypeInt(signed=False, label="UInt32")}, name="IKEEXT_CERTIFICATE_CRITERIA0", pack=False, align=None), offset=0)}, name="_Anonymous3_e__Struct", pack=False, align=None)}, name="<anon>", label="None"), "flags": SimTypeInt(signed=False, label="IKEEXT_CERT_AUTH"), "localCertLocationUrl": SimStruct({"size": SimTypeInt(signed=False, label="UInt32"), "data": SimTypePointer(SimTypeChar(label="Byte"), offset=0)}, name="FWP_BYTE_BLOB", pack=False, align=None)}, name="IKEEXT_CERTIFICATE_AUTHENTICATION2", pack=False, align=None), "kerberosAuthentication": SimStruct({"flags": SimTypeInt(signed=False, label="IKEEXT_KERBEROS_AUTHENTICATION_FLAGS"), "proxyServer": SimTypePointer(SimTypeChar(label="Char"), offset=0)}, name="IKEEXT_KERBEROS_AUTHENTICATION1", pack=False, align=None), "reservedAuthentication": SimStruct({"flags": SimTypeInt(signed=False, label="IKEEXT_RESERVED_AUTHENTICATION_FLAGS")}, name="IKEEXT_RESERVED_AUTHENTICATION0", pack=False, align=None), "ntlmV2Authentication": SimStruct({"flags": SimTypeInt(signed=False, label="UInt32")}, name="IKEEXT_NTLM_V2_AUTHENTICATION0", pack=False, align=None), "sslAuthentication": SimStruct({"inboundConfigType": SimTypeInt(signed=False, label="IKEEXT_CERT_CONFIG_TYPE"), "Anonymous1": SimUnion({"Anonymous1": SimStruct({"inboundRootArraySize": SimTypeInt(signed=False, label="UInt32"), "inboundRootCriteria": SimTypePointer(SimStruct({"certData": SimStruct({"size": SimTypeInt(signed=False, label="UInt32"), "data": SimTypePointer(SimTypeChar(label="Byte"), offset=0)}, name="FWP_BYTE_BLOB", pack=False, align=None), "certHash": SimStruct({"size": SimTypeInt(signed=False, label="UInt32"), "data": SimTypePointer(SimTypeChar(label="Byte"), offset=0)}, name="FWP_BYTE_BLOB", pack=False, align=None), "eku": SimTypePointer(SimStruct({"numEku": SimTypeInt(signed=False, label="UInt32"), "eku": SimTypePointer(SimTypePointer(SimTypeChar(label="Byte"), offset=0), offset=0)}, name="IKEEXT_CERT_EKUS0", pack=False, align=None), offset=0), "name": SimTypePointer(SimStruct({"nameType": SimTypeInt(signed=False, label="IKEEXT_CERT_CRITERIA_NAME_TYPE"), "certName": SimTypePointer(SimTypeChar(label="Char"), offset=0)}, name="IKEEXT_CERT_NAME0", pack=False, align=None), offset=0), "flags": SimTypeInt(signed=False, label="UInt32")}, name="IKEEXT_CERTIFICATE_CRITERIA0", pack=False, align=None), offset=0)}, name="_Anonymous1_e__Struct", pack=False, align=None), "Anonymous2": SimStruct({"inboundEnterpriseStoreArraySize": SimTypeInt(signed=False, label="UInt32"), "inboundEnterpriseStoreCriteria": SimTypePointer(SimStruct({"certData": SimStruct({"size": SimTypeInt(signed=False, label="UInt32"), "data": SimTypePointer(SimTypeChar(label="Byte"), offset=0)}, name="FWP_BYTE_BLOB", pack=False, align=None), "certHash": SimStruct({"size": SimTypeInt(signed=False, label="UInt32"), "data": SimTypePointer(SimTypeChar(label="Byte"), offset=0)}, name="FWP_BYTE_BLOB", pack=False, align=None), "eku": SimTypePointer(SimStruct({"numEku": SimTypeInt(signed=False, label="UInt32"), "eku": SimTypePointer(SimTypePointer(SimTypeChar(label="Byte"), offset=0), offset=0)}, name="IKEEXT_CERT_EKUS0", pack=False, align=None), offset=0), "name": SimTypePointer(SimStruct({"nameType": SimTypeInt(signed=False, label="IKEEXT_CERT_CRITERIA_NAME_TYPE"), "certName": SimTypePointer(SimTypeChar(label="Char"), offset=0)}, name="IKEEXT_CERT_NAME0", pack=False, align=None), offset=0), "flags": SimTypeInt(signed=False, label="UInt32")}, name="IKEEXT_CERTIFICATE_CRITERIA0", pack=False, align=None), offset=0)}, name="_Anonymous2_e__Struct", pack=False, align=None), "Anonymous3": SimStruct({"inboundRootStoreArraySize": SimTypeInt(signed=False, label="UInt32"), "inboundTrustedRootStoreCriteria": SimTypePointer(SimStruct({"certData": SimStruct({"size": SimTypeInt(signed=False, label="UInt32"), "data": SimTypePointer(SimTypeChar(label="Byte"), offset=0)}, name="FWP_BYTE_BLOB", pack=False, align=None), "certHash": SimStruct({"size": SimTypeInt(signed=False, label="UInt32"), "data": SimTypePointer(SimTypeChar(label="Byte"), offset=0)}, name="FWP_BYTE_BLOB", pack=False, align=None), "eku": SimTypePointer(SimStruct({"numEku": SimTypeInt(signed=False, label="UInt32"), "eku": SimTypePointer(SimTypePointer(SimTypeChar(label="Byte"), offset=0), offset=0)}, name="IKEEXT_CERT_EKUS0", pack=False, align=None), offset=0), "name": SimTypePointer(SimStruct({"nameType": SimTypeInt(signed=False, label="IKEEXT_CERT_CRITERIA_NAME_TYPE"), "certName": SimTypePointer(SimTypeChar(label="Char"), offset=0)}, name="IKEEXT_CERT_NAME0", pack=False, align=None), offset=0), "flags": SimTypeInt(signed=False, label="UInt32")}, name="IKEEXT_CERTIFICATE_CRITERIA0", pack=False, align=None), offset=0)}, name="_Anonymous3_e__Struct", pack=False, align=None)}, name="<anon>", label="None"), "outboundConfigType": SimTypeInt(signed=False, label="IKEEXT_CERT_CONFIG_TYPE"), "Anonymous2": SimUnion({"Anonymous1": SimStruct({"outboundRootArraySize": SimTypeInt(signed=False, label="UInt32"), "outboundRootCriteria": SimTypePointer(SimStruct({"certData": SimStruct({"size": SimTypeInt(signed=False, label="UInt32"), "data": SimTypePointer(SimTypeChar(label="Byte"), offset=0)}, name="FWP_BYTE_BLOB", pack=False, align=None), "certHash": SimStruct({"size": SimTypeInt(signed=False, label="UInt32"), "data": SimTypePointer(SimTypeChar(label="Byte"), offset=0)}, name="FWP_BYTE_BLOB", pack=False, align=None), "eku": SimTypePointer(SimStruct({"numEku": SimTypeInt(signed=False, label="UInt32"), "eku": SimTypePointer(SimTypePointer(SimTypeChar(label="Byte"), offset=0), offset=0)}, name="IKEEXT_CERT_EKUS0", pack=False, align=None), offset=0), "name": SimTypePointer(SimStruct({"nameType": SimTypeInt(signed=False, label="IKEEXT_CERT_CRITERIA_NAME_TYPE"), "certName": SimTypePointer(SimTypeChar(label="Char"), offset=0)}, name="IKEEXT_CERT_NAME0", pack=False, align=None), offset=0), "flags": SimTypeInt(signed=False, label="UInt32")}, name="IKEEXT_CERTIFICATE_CRITERIA0", pack=False, align=None), offset=0)}, name="_Anonymous1_e__Struct", pack=False, align=None), "Anonymous2": SimStruct({"outboundEnterpriseStoreArraySize": SimTypeInt(signed=False, label="UInt32"), "outboundEnterpriseStoreCriteria": SimTypePointer(SimStruct({"certData": SimStruct({"size": SimTypeInt(signed=False, label="UInt32"), "data": SimTypePointer(SimTypeChar(label="Byte"), offset=0)}, name="FWP_BYTE_BLOB", pack=False, align=None), "certHash": SimStruct({"size": SimTypeInt(signed=False, label="UInt32"), "data": SimTypePointer(SimTypeChar(label="Byte"), offset=0)}, name="FWP_BYTE_BLOB", pack=False, align=None), "eku": SimTypePointer(SimStruct({"numEku": SimTypeInt(signed=False, label="UInt32"), "eku": SimTypePointer(SimTypePointer(SimTypeChar(label="Byte"), offset=0), offset=0)}, name="IKEEXT_CERT_EKUS0", pack=False, align=None), offset=0), "name": SimTypePointer(SimStruct({"nameType": SimTypeInt(signed=False, label="IKEEXT_CERT_CRITERIA_NAME_TYPE"), "certName": SimTypePointer(SimTypeChar(label="Char"), offset=0)}, name="IKEEXT_CERT_NAME0", pack=False, align=None), offset=0), "flags": SimTypeInt(signed=False, label="UInt32")}, name="IKEEXT_CERTIFICATE_CRITERIA0", pack=False, align=None), offset=0)}, name="_Anonymous2_e__Struct", pack=False, align=None), "Anonymous3": SimStruct({"outboundRootStoreArraySize": SimTypeInt(signed=False, label="UInt32"), "outboundTrustedRootStoreCriteria": SimTypePointer(SimStruct({"certData": SimStruct({"size": SimTypeInt(signed=False, label="UInt32"), "data": SimTypePointer(SimTypeChar(label="Byte"), offset=0)}, name="FWP_BYTE_BLOB", pack=False, align=None), "certHash": SimStruct({"size": SimTypeInt(signed=False, label="UInt32"), "data": SimTypePointer(SimTypeChar(label="Byte"), offset=0)}, name="FWP_BYTE_BLOB", pack=False, align=None), "eku": SimTypePointer(SimStruct({"numEku": SimTypeInt(signed=False, label="UInt32"), "eku": SimTypePointer(SimTypePointer(SimTypeChar(label="Byte"), offset=0), offset=0)}, name="IKEEXT_CERT_EKUS0", pack=False, align=None), offset=0), "name": SimTypePointer(SimStruct({"nameType": SimTypeInt(signed=False, label="IKEEXT_CERT_CRITERIA_NAME_TYPE"), "certName": SimTypePointer(SimTypeChar(label="Char"), offset=0)}, name="IKEEXT_CERT_NAME0", pack=False, align=None), offset=0), "flags": SimTypeInt(signed=False, label="UInt32")}, name="IKEEXT_CERTIFICATE_CRITERIA0", pack=False, align=None), offset=0)}, name="_Anonymous3_e__Struct", pack=False, align=None)}, name="<anon>", label="None"), "flags": SimTypeInt(signed=False, label="IKEEXT_CERT_AUTH"), "localCertLocationUrl": SimStruct({"size": SimTypeInt(signed=False, label="UInt32"), "data": SimTypePointer(SimTypeChar(label="Byte"), offset=0)}, name="FWP_BYTE_BLOB", pack=False, align=None)}, name="IKEEXT_CERTIFICATE_AUTHENTICATION2", pack=False, align=None), "cgaAuthentication": SimStruct({"keyContainerName": SimTypePointer(SimTypeChar(label="Char"), offset=0), "cspName": SimTypePointer(SimTypeChar(label="Char"), offset=0), "cspType": SimTypeInt(signed=False, label="UInt32"), "cgaModifier": SimStruct({"byteArray16": SimTypeFixedSizeArray(SimTypeChar(label="Byte"), 16)}, name="FWP_BYTE_ARRAY16", pack=False, align=None), "cgaCollisionCount": SimTypeChar(label="Byte")}, name="IKEEXT_IPV6_CGA_AUTHENTICATION0", pack=False, align=None), "eapAuthentication": SimStruct({"flags": SimTypeInt(signed=False, label="IKEEXT_EAP_AUTHENTICATION_FLAGS")}, name="IKEEXT_EAP_AUTHENTICATION0", pack=False, align=None)}, name="<anon>", label="None")}, name="IKEEXT_AUTHENTICATION_METHOD2", pack=False, align=None), offset=0), "initiatorImpersonationType": SimTypeInt(signed=False, label="IKEEXT_AUTHENTICATION_IMPERSONATION_TYPE")}, name="IKEEXT_EM_POLICY2", pack=False, align=None), offset=0), "fwdPathSaLifetime": SimTypeInt(signed=False, label="UInt32")}, name="IPSEC_TUNNEL_POLICY2", pack=False, align=None), offset=0), "ikeMmPolicy": SimTypePointer(SimStruct({"softExpirationTime": SimTypeInt(signed=False, label="UInt32"), "numAuthenticationMethods": SimTypeInt(signed=False, label="UInt32"), "authenticationMethods": SimTypePointer(SimStruct({"authenticationMethodType": SimTypeInt(signed=False, label="IKEEXT_AUTHENTICATION_METHOD_TYPE"), "Anonymous": SimUnion({"presharedKeyAuthentication": SimStruct({"presharedKey": SimStruct({"size": SimTypeInt(signed=False, label="UInt32"), "data": SimTypePointer(SimTypeChar(label="Byte"), offset=0)}, name="FWP_BYTE_BLOB", pack=False, align=None), "flags": SimTypeInt(signed=False, label="IKEEXT_PRESHARED_KEY_AUTHENTICATION_FLAGS")}, name="IKEEXT_PRESHARED_KEY_AUTHENTICATION1", pack=False, align=None), "certificateAuthentication": SimStruct({"inboundConfigType": SimTypeInt(signed=False, label="IKEEXT_CERT_CONFIG_TYPE"), "Anonymous1": SimUnion({"Anonymous1": SimStruct({"inboundRootArraySize": SimTypeInt(signed=False, label="UInt32"), "inboundRootCriteria": SimTypePointer(SimStruct({"certData": SimStruct({"size": SimTypeInt(signed=False, label="UInt32"), "data": SimTypePointer(SimTypeChar(label="Byte"), offset=0)}, name="FWP_BYTE_BLOB", pack=False, align=None), "certHash": SimStruct({"size": SimTypeInt(signed=False, label="UInt32"), "data": SimTypePointer(SimTypeChar(label="Byte"), offset=0)}, name="FWP_BYTE_BLOB", pack=False, align=None), "eku": SimTypePointer(SimStruct({"numEku": SimTypeInt(signed=False, label="UInt32"), "eku": SimTypePointer(SimTypePointer(SimTypeChar(label="Byte"), offset=0), offset=0)}, name="IKEEXT_CERT_EKUS0", pack=False, align=None), offset=0), "name": SimTypePointer(SimStruct({"nameType": SimTypeInt(signed=False, label="IKEEXT_CERT_CRITERIA_NAME_TYPE"), "certName": SimTypePointer(SimTypeChar(label="Char"), offset=0)}, name="IKEEXT_CERT_NAME0", pack=False, align=None), offset=0), "flags": SimTypeInt(signed=False, label="UInt32")}, name="IKEEXT_CERTIFICATE_CRITERIA0", pack=False, align=None), offset=0)}, name="_Anonymous1_e__Struct", pack=False, align=None), "Anonymous2": SimStruct({"inboundEnterpriseStoreArraySize": SimTypeInt(signed=False, label="UInt32"), "inboundEnterpriseStoreCriteria": SimTypePointer(SimStruct({"certData": SimStruct({"size": SimTypeInt(signed=False, label="UInt32"), "data": SimTypePointer(SimTypeChar(label="Byte"), offset=0)}, name="FWP_BYTE_BLOB", pack=False, align=None), "certHash": SimStruct({"size": SimTypeInt(signed=False, label="UInt32"), "data": SimTypePointer(SimTypeChar(label="Byte"), offset=0)}, name="FWP_BYTE_BLOB", pack=False, align=None), "eku": SimTypePointer(SimStruct({"numEku": SimTypeInt(signed=False, label="UInt32"), "eku": SimTypePointer(SimTypePointer(SimTypeChar(label="Byte"), offset=0), offset=0)}, name="IKEEXT_CERT_EKUS0", pack=False, align=None), offset=0), "name": SimTypePointer(SimStruct({"nameType": SimTypeInt(signed=False, label="IKEEXT_CERT_CRITERIA_NAME_TYPE"), "certName": SimTypePointer(SimTypeChar(label="Char"), offset=0)}, name="IKEEXT_CERT_NAME0", pack=False, align=None), offset=0), "flags": SimTypeInt(signed=False, label="UInt32")}, name="IKEEXT_CERTIFICATE_CRITERIA0", pack=False, align=None), offset=0)}, name="_Anonymous2_e__Struct", pack=False, align=None), "Anonymous3": SimStruct({"inboundRootStoreArraySize": SimTypeInt(signed=False, label="UInt32"), "inboundTrustedRootStoreCriteria": SimTypePointer(SimStruct({"certData": SimStruct({"size": SimTypeInt(signed=False, label="UInt32"), "data": SimTypePointer(SimTypeChar(label="Byte"), offset=0)}, name="FWP_BYTE_BLOB", pack=False, align=None), "certHash": SimStruct({"size": SimTypeInt(signed=False, label="UInt32"), "data": SimTypePointer(SimTypeChar(label="Byte"), offset=0)}, name="FWP_BYTE_BLOB", pack=False, align=None), "eku": SimTypePointer(SimStruct({"numEku": SimTypeInt(signed=False, label="UInt32"), "eku": SimTypePointer(SimTypePointer(SimTypeChar(label="Byte"), offset=0), offset=0)}, name="IKEEXT_CERT_EKUS0", pack=False, align=None), offset=0), "name": SimTypePointer(SimStruct({"nameType": SimTypeInt(signed=False, label="IKEEXT_CERT_CRITERIA_NAME_TYPE"), "certName": SimTypePointer(SimTypeChar(label="Char"), offset=0)}, name="IKEEXT_CERT_NAME0", pack=False, align=None), offset=0), "flags": SimTypeInt(signed=False, label="UInt32")}, name="IKEEXT_CERTIFICATE_CRITERIA0", pack=False, align=None), offset=0)}, name="_Anonymous3_e__Struct", pack=False, align=None)}, name="<anon>", label="None"), "outboundConfigType": SimTypeInt(signed=False, label="IKEEXT_CERT_CONFIG_TYPE"), "Anonymous2": SimUnion({"Anonymous1": SimStruct({"outboundRootArraySize": SimTypeInt(signed=False, label="UInt32"), "outboundRootCriteria": SimTypePointer(SimStruct({"certData": SimStruct({"size": SimTypeInt(signed=False, label="UInt32"), "data": SimTypePointer(SimTypeChar(label="Byte"), offset=0)}, name="FWP_BYTE_BLOB", pack=False, align=None), "certHash": SimStruct({"size": SimTypeInt(signed=False, label="UInt32"), "data": SimTypePointer(SimTypeChar(label="Byte"), offset=0)}, name="FWP_BYTE_BLOB", pack=False, align=None), "eku": SimTypePointer(SimStruct({"numEku": SimTypeInt(signed=False, label="UInt32"), "eku": SimTypePointer(SimTypePointer(SimTypeChar(label="Byte"), offset=0), offset=0)}, name="IKEEXT_CERT_EKUS0", pack=False, align=None), offset=0), "name": SimTypePointer(SimStruct({"nameType": SimTypeInt(signed=False, label="IKEEXT_CERT_CRITERIA_NAME_TYPE"), "certName": SimTypePointer(SimTypeChar(label="Char"), offset=0)}, name="IKEEXT_CERT_NAME0", pack=False, align=None), offset=0), "flags": SimTypeInt(signed=False, label="UInt32")}, name="IKEEXT_CERTIFICATE_CRITERIA0", pack=False, align=None), offset=0)}, name="_Anonymous1_e__Struct", pack=False, align=None), "Anonymous2": SimStruct({"outboundEnterpriseStoreArraySize": SimTypeInt(signed=False, label="UInt32"), "outboundEnterpriseStoreCriteria": SimTypePointer(SimStruct({"certData": SimStruct({"size": SimTypeInt(signed=False, label="UInt32"), "data": SimTypePointer(SimTypeChar(label="Byte"), offset=0)}, name="FWP_BYTE_BLOB", pack=False, align=None), "certHash": SimStruct({"size": SimTypeInt(signed=False, label="UInt32"), "data": SimTypePointer(SimTypeChar(label="Byte"), offset=0)}, name="FWP_BYTE_BLOB", pack=False, align=None), "eku": SimTypePointer(SimStruct({"numEku": SimTypeInt(signed=False, label="UInt32"), "eku": SimTypePointer(SimTypePointer(SimTypeChar(label="Byte"), offset=0), offset=0)}, name="IKEEXT_CERT_EKUS0", pack=False, align=None), offset=0), "name": SimTypePointer(SimStruct({"nameType": SimTypeInt(signed=False, label="IKEEXT_CERT_CRITERIA_NAME_TYPE"), "certName": SimTypePointer(SimTypeChar(label="Char"), offset=0)}, name="IKEEXT_CERT_NAME0", pack=False, align=None), offset=0), "flags": SimTypeInt(signed=False, label="UInt32")}, name="IKEEXT_CERTIFICATE_CRITERIA0", pack=False, align=None), offset=0)}, name="_Anonymous2_e__Struct", pack=False, align=None), "Anonymous3": SimStruct({"outboundRootStoreArraySize": SimTypeInt(signed=False, label="UInt32"), "outboundTrustedRootStoreCriteria": SimTypePointer(SimStruct({"certData": SimStruct({"size": SimTypeInt(signed=False, label="UInt32"), "data": SimTypePointer(SimTypeChar(label="Byte"), offset=0)}, name="FWP_BYTE_BLOB", pack=False, align=None), "certHash": SimStruct({"size": SimTypeInt(signed=False, label="UInt32"), "data": SimTypePointer(SimTypeChar(label="Byte"), offset=0)}, name="FWP_BYTE_BLOB", pack=False, align=None), "eku": SimTypePointer(SimStruct({"numEku": SimTypeInt(signed=False, label="UInt32"), "eku": SimTypePointer(SimTypePointer(SimTypeChar(label="Byte"), offset=0), offset=0)}, name="IKEEXT_CERT_EKUS0", pack=False, align=None), offset=0), "name": SimTypePointer(SimStruct({"nameType": SimTypeInt(signed=False, label="IKEEXT_CERT_CRITERIA_NAME_TYPE"), "certName": SimTypePointer(SimTypeChar(label="Char"), offset=0)}, name="IKEEXT_CERT_NAME0", pack=False, align=None), offset=0), "flags": SimTypeInt(signed=False, label="UInt32")}, name="IKEEXT_CERTIFICATE_CRITERIA0", pack=False, align=None), offset=0)}, name="_Anonymous3_e__Struct", pack=False, align=None)}, name="<anon>", label="None"), "flags": SimTypeInt(signed=False, label="IKEEXT_CERT_AUTH"), "localCertLocationUrl": SimStruct({"size": SimTypeInt(signed=False, label="UInt32"), "data": SimTypePointer(SimTypeChar(label="Byte"), offset=0)}, name="FWP_BYTE_BLOB", pack=False, align=None)}, name="IKEEXT_CERTIFICATE_AUTHENTICATION2", pack=False, align=None), "kerberosAuthentication": SimStruct({"flags": SimTypeInt(signed=False, label="IKEEXT_KERBEROS_AUTHENTICATION_FLAGS"), "proxyServer": SimTypePointer(SimTypeChar(label="Char"), offset=0)}, name="IKEEXT_KERBEROS_AUTHENTICATION1", pack=False, align=None), "reservedAuthentication": SimStruct({"flags": SimTypeInt(signed=False, label="IKEEXT_RESERVED_AUTHENTICATION_FLAGS")}, name="IKEEXT_RESERVED_AUTHENTICATION0", pack=False, align=None), "ntlmV2Authentication": SimStruct({"flags": SimTypeInt(signed=False, label="UInt32")}, name="IKEEXT_NTLM_V2_AUTHENTICATION0", pack=False, align=None), "sslAuthentication": SimStruct({"inboundConfigType": SimTypeInt(signed=False, label="IKEEXT_CERT_CONFIG_TYPE"), "Anonymous1": SimUnion({"Anonymous1": SimStruct({"inboundRootArraySize": SimTypeInt(signed=False, label="UInt32"), "inboundRootCriteria": SimTypePointer(SimStruct({"certData": SimStruct({"size": SimTypeInt(signed=False, label="UInt32"), "data": SimTypePointer(SimTypeChar(label="Byte"), offset=0)}, name="FWP_BYTE_BLOB", pack=False, align=None), "certHash": SimStruct({"size": SimTypeInt(signed=False, label="UInt32"), "data": SimTypePointer(SimTypeChar(label="Byte"), offset=0)}, name="FWP_BYTE_BLOB", pack=False, align=None), "eku": SimTypePointer(SimStruct({"numEku": SimTypeInt(signed=False, label="UInt32"), "eku": SimTypePointer(SimTypePointer(SimTypeChar(label="Byte"), offset=0), offset=0)}, name="IKEEXT_CERT_EKUS0", pack=False, align=None), offset=0), "name": SimTypePointer(SimStruct({"nameType": SimTypeInt(signed=False, label="IKEEXT_CERT_CRITERIA_NAME_TYPE"), "certName": SimTypePointer(SimTypeChar(label="Char"), offset=0)}, name="IKEEXT_CERT_NAME0", pack=False, align=None), offset=0), "flags": SimTypeInt(signed=False, label="UInt32")}, name="IKEEXT_CERTIFICATE_CRITERIA0", pack=False, align=None), offset=0)}, name="_Anonymous1_e__Struct", pack=False, align=None), "Anonymous2": SimStruct({"inboundEnterpriseStoreArraySize": SimTypeInt(signed=False, label="UInt32"), "inboundEnterpriseStoreCriteria": SimTypePointer(SimStruct({"certData": SimStruct({"size": SimTypeInt(signed=False, label="UInt32"), "data": SimTypePointer(SimTypeChar(label="Byte"), offset=0)}, name="FWP_BYTE_BLOB", pack=False, align=None), "certHash": SimStruct({"size": SimTypeInt(signed=False, label="UInt32"), "data": SimTypePointer(SimTypeChar(label="Byte"), offset=0)}, name="FWP_BYTE_BLOB", pack=False, align=None), "eku": SimTypePointer(SimStruct({"numEku": SimTypeInt(signed=False, label="UInt32"), "eku": SimTypePointer(SimTypePointer(SimTypeChar(label="Byte"), offset=0), offset=0)}, name="IKEEXT_CERT_EKUS0", pack=False, align=None), offset=0), "name": SimTypePointer(SimStruct({"nameType": SimTypeInt(signed=False, label="IKEEXT_CERT_CRITERIA_NAME_TYPE"), "certName": SimTypePointer(SimTypeChar(label="Char"), offset=0)}, name="IKEEXT_CERT_NAME0", pack=False, align=None), offset=0), "flags": SimTypeInt(signed=False, label="UInt32")}, name="IKEEXT_CERTIFICATE_CRITERIA0", pack=False, align=None), offset=0)}, name="_Anonymous2_e__Struct", pack=False, align=None), "Anonymous3": SimStruct({"inboundRootStoreArraySize": SimTypeInt(signed=False, label="UInt32"), "inboundTrustedRootStoreCriteria": SimTypePointer(SimStruct({"certData": SimStruct({"size": SimTypeInt(signed=False, label="UInt32"), "data": SimTypePointer(SimTypeChar(label="Byte"), offset=0)}, name="FWP_BYTE_BLOB", pack=False, align=None), "certHash": SimStruct({"size": SimTypeInt(signed=False, label="UInt32"), "data": SimTypePointer(SimTypeChar(label="Byte"), offset=0)}, name="FWP_BYTE_BLOB", pack=False, align=None), "eku": SimTypePointer(SimStruct({"numEku": SimTypeInt(signed=False, label="UInt32"), "eku": SimTypePointer(SimTypePointer(SimTypeChar(label="Byte"), offset=0), offset=0)}, name="IKEEXT_CERT_EKUS0", pack=False, align=None), offset=0), "name": SimTypePointer(SimStruct({"nameType": SimTypeInt(signed=False, label="IKEEXT_CERT_CRITERIA_NAME_TYPE"), "certName": SimTypePointer(SimTypeChar(label="Char"), offset=0)}, name="IKEEXT_CERT_NAME0", pack=False, align=None), offset=0), "flags": SimTypeInt(signed=False, label="UInt32")}, name="IKEEXT_CERTIFICATE_CRITERIA0", pack=False, align=None), offset=0)}, name="_Anonymous3_e__Struct", pack=False, align=None)}, name="<anon>", label="None"), "outboundConfigType": SimTypeInt(signed=False, label="IKEEXT_CERT_CONFIG_TYPE"), "Anonymous2": SimUnion({"Anonymous1": SimStruct({"outboundRootArraySize": SimTypeInt(signed=False, label="UInt32"), "outboundRootCriteria": SimTypePointer(SimStruct({"certData": SimStruct({"size": SimTypeInt(signed=False, label="UInt32"), "data": SimTypePointer(SimTypeChar(label="Byte"), offset=0)}, name="FWP_BYTE_BLOB", pack=False, align=None), "certHash": SimStruct({"size": SimTypeInt(signed=False, label="UInt32"), "data": SimTypePointer(SimTypeChar(label="Byte"), offset=0)}, name="FWP_BYTE_BLOB", pack=False, align=None), "eku": SimTypePointer(SimStruct({"numEku": SimTypeInt(signed=False, label="UInt32"), "eku": SimTypePointer(SimTypePointer(SimTypeChar(label="Byte"), offset=0), offset=0)}, name="IKEEXT_CERT_EKUS0", pack=False, align=None), offset=0), "name": SimTypePointer(SimStruct({"nameType": SimTypeInt(signed=False, label="IKEEXT_CERT_CRITERIA_NAME_TYPE"), "certName": SimTypePointer(SimTypeChar(label="Char"), offset=0)}, name="IKEEXT_CERT_NAME0", pack=False, align=None), offset=0), "flags": SimTypeInt(signed=False, label="UInt32")}, name="IKEEXT_CERTIFICATE_CRITERIA0", pack=False, align=None), offset=0)}, name="_Anonymous1_e__Struct", pack=False, align=None), "Anonymous2": SimStruct({"outboundEnterpriseStoreArraySize": SimTypeInt(signed=False, label="UInt32"), "outboundEnterpriseStoreCriteria": SimTypePointer(SimStruct({"certData": SimStruct({"size": SimTypeInt(signed=False, label="UInt32"), "data": SimTypePointer(SimTypeChar(label="Byte"), offset=0)}, name="FWP_BYTE_BLOB", pack=False, align=None), "certHash": SimStruct({"size": SimTypeInt(signed=False, label="UInt32"), "data": SimTypePointer(SimTypeChar(label="Byte"), offset=0)}, name="FWP_BYTE_BLOB", pack=False, align=None), "eku": SimTypePointer(SimStruct({"numEku": SimTypeInt(signed=False, label="UInt32"), "eku": SimTypePointer(SimTypePointer(SimTypeChar(label="Byte"), offset=0), offset=0)}, name="IKEEXT_CERT_EKUS0", pack=False, align=None), offset=0), "name": SimTypePointer(SimStruct({"nameType": SimTypeInt(signed=False, label="IKEEXT_CERT_CRITERIA_NAME_TYPE"), "certName": SimTypePointer(SimTypeChar(label="Char"), offset=0)}, name="IKEEXT_CERT_NAME0", pack=False, align=None), offset=0), "flags": SimTypeInt(signed=False, label="UInt32")}, name="IKEEXT_CERTIFICATE_CRITERIA0", pack=False, align=None), offset=0)}, name="_Anonymous2_e__Struct", pack=False, align=None), "Anonymous3": SimStruct({"outboundRootStoreArraySize": SimTypeInt(signed=False, label="UInt32"), "outboundTrustedRootStoreCriteria": SimTypePointer(SimStruct({"certData": SimStruct({"size": SimTypeInt(signed=False, label="UInt32"), "data": SimTypePointer(SimTypeChar(label="Byte"), offset=0)}, name="FWP_BYTE_BLOB", pack=False, align=None), "certHash": SimStruct({"size": SimTypeInt(signed=False, label="UInt32"), "data": SimTypePointer(SimTypeChar(label="Byte"), offset=0)}, name="FWP_BYTE_BLOB", pack=False, align=None), "eku": SimTypePointer(SimStruct({"numEku": SimTypeInt(signed=False, label="UInt32"), "eku": SimTypePointer(SimTypePointer(SimTypeChar(label="Byte"), offset=0), offset=0)}, name="IKEEXT_CERT_EKUS0", pack=False, align=None), offset=0), "name": SimTypePointer(SimStruct({"nameType": SimTypeInt(signed=False, label="IKEEXT_CERT_CRITERIA_NAME_TYPE"), "certName": SimTypePointer(SimTypeChar(label="Char"), offset=0)}, name="IKEEXT_CERT_NAME0", pack=False, align=None), offset=0), "flags": SimTypeInt(signed=False, label="UInt32")}, name="IKEEXT_CERTIFICATE_CRITERIA0", pack=False, align=None), offset=0)}, name="_Anonymous3_e__Struct", pack=False, align=None)}, name="<anon>", label="None"), "flags": SimTypeInt(signed=False, label="IKEEXT_CERT_AUTH"), "localCertLocationUrl": SimStruct({"size": SimTypeInt(signed=False, label="UInt32"), "data": SimTypePointer(SimTypeChar(label="Byte"), offset=0)}, name="FWP_BYTE_BLOB", pack=False, align=None)}, name="IKEEXT_CERTIFICATE_AUTHENTICATION2", pack=False, align=None), "cgaAuthentication": SimStruct({"keyContainerName": SimTypePointer(SimTypeChar(label="Char"), offset=0), "cspName": SimTypePointer(SimTypeChar(label="Char"), offset=0), "cspType": SimTypeInt(signed=False, label="UInt32"), "cgaModifier": SimStruct({"byteArray16": SimTypeFixedSizeArray(SimTypeChar(label="Byte"), 16)}, name="FWP_BYTE_ARRAY16", pack=False, align=None), "cgaCollisionCount": SimTypeChar(label="Byte")}, name="IKEEXT_IPV6_CGA_AUTHENTICATION0", pack=False, align=None), "eapAuthentication": SimStruct({"flags": SimTypeInt(signed=False, label="IKEEXT_EAP_AUTHENTICATION_FLAGS")}, name="IKEEXT_EAP_AUTHENTICATION0", pack=False, align=None)}, name="<anon>", label="None")}, name="IKEEXT_AUTHENTICATION_METHOD2", pack=False, align=None), offset=0), "initiatorImpersonationType": SimTypeInt(signed=False, label="IKEEXT_AUTHENTICATION_IMPERSONATION_TYPE"), "numIkeProposals": SimTypeInt(signed=False, label="UInt32"), "ikeProposals": SimTypePointer(SimStruct({"cipherAlgorithm": SimStruct({"algoIdentifier": SimTypeInt(signed=False, label="IKEEXT_CIPHER_TYPE"), "keyLen": SimTypeInt(signed=False, label="UInt32"), "rounds": SimTypeInt(signed=False, label="UInt32")}, name="IKEEXT_CIPHER_ALGORITHM0", pack=False, align=None), "integrityAlgorithm": SimStruct({"algoIdentifier": SimTypeInt(signed=False, label="IKEEXT_INTEGRITY_TYPE")}, name="IKEEXT_INTEGRITY_ALGORITHM0", pack=False, align=None), "maxLifetimeSeconds": SimTypeInt(signed=False, label="UInt32"), "dhGroup": SimTypeInt(signed=False, label="IKEEXT_DH_GROUP"), "quickModeLimit": SimTypeInt(signed=False, label="UInt32")}, name="IKEEXT_PROPOSAL0", pack=False, align=None), offset=0), "flags": SimTypeInt(signed=False, label="IKEEXT_POLICY_FLAG"), "maxDynamicFilters": SimTypeInt(signed=False, label="UInt32"), "retransmitDurationSecs": SimTypeInt(signed=False, label="UInt32")}, name="IKEEXT_POLICY2", pack=False, align=None), offset=0), "authIpMmPolicy": SimTypePointer(SimStruct({"softExpirationTime": SimTypeInt(signed=False, label="UInt32"), "numAuthenticationMethods": SimTypeInt(signed=False, label="UInt32"), "authenticationMethods": SimTypePointer(SimStruct({"authenticationMethodType": SimTypeInt(signed=False, label="IKEEXT_AUTHENTICATION_METHOD_TYPE"), "Anonymous": SimUnion({"presharedKeyAuthentication": SimStruct({"presharedKey": SimStruct({"size": SimTypeInt(signed=False, label="UInt32"), "data": SimTypePointer(SimTypeChar(label="Byte"), offset=0)}, name="FWP_BYTE_BLOB", pack=False, align=None), "flags": SimTypeInt(signed=False, label="IKEEXT_PRESHARED_KEY_AUTHENTICATION_FLAGS")}, name="IKEEXT_PRESHARED_KEY_AUTHENTICATION1", pack=False, align=None), "certificateAuthentication": SimStruct({"inboundConfigType": SimTypeInt(signed=False, label="IKEEXT_CERT_CONFIG_TYPE"), "Anonymous1": SimUnion({"Anonymous1": SimStruct({"inboundRootArraySize": SimTypeInt(signed=False, label="UInt32"), "inboundRootCriteria": SimTypePointer(SimStruct({"certData": SimStruct({"size": SimTypeInt(signed=False, label="UInt32"), "data": SimTypePointer(SimTypeChar(label="Byte"), offset=0)}, name="FWP_BYTE_BLOB", pack=False, align=None), "certHash": SimStruct({"size": SimTypeInt(signed=False, label="UInt32"), "data": SimTypePointer(SimTypeChar(label="Byte"), offset=0)}, name="FWP_BYTE_BLOB", pack=False, align=None), "eku": SimTypePointer(SimStruct({"numEku": SimTypeInt(signed=False, label="UInt32"), "eku": SimTypePointer(SimTypePointer(SimTypeChar(label="Byte"), offset=0), offset=0)}, name="IKEEXT_CERT_EKUS0", pack=False, align=None), offset=0), "name": SimTypePointer(SimStruct({"nameType": SimTypeInt(signed=False, label="IKEEXT_CERT_CRITERIA_NAME_TYPE"), "certName": SimTypePointer(SimTypeChar(label="Char"), offset=0)}, name="IKEEXT_CERT_NAME0", pack=False, align=None), offset=0), "flags": SimTypeInt(signed=False, label="UInt32")}, name="IKEEXT_CERTIFICATE_CRITERIA0", pack=False, align=None), offset=0)}, name="_Anonymous1_e__Struct", pack=False, align=None), "Anonymous2": SimStruct({"inboundEnterpriseStoreArraySize": SimTypeInt(signed=False, label="UInt32"), "inboundEnterpriseStoreCriteria": SimTypePointer(SimStruct({"certData": SimStruct({"size": SimTypeInt(signed=False, label="UInt32"), "data": SimTypePointer(SimTypeChar(label="Byte"), offset=0)}, name="FWP_BYTE_BLOB", pack=False, align=None), "certHash": SimStruct({"size": SimTypeInt(signed=False, label="UInt32"), "data": SimTypePointer(SimTypeChar(label="Byte"), offset=0)}, name="FWP_BYTE_BLOB", pack=False, align=None), "eku": SimTypePointer(SimStruct({"numEku": SimTypeInt(signed=False, label="UInt32"), "eku": SimTypePointer(SimTypePointer(SimTypeChar(label="Byte"), offset=0), offset=0)}, name="IKEEXT_CERT_EKUS0", pack=False, align=None), offset=0), "name": SimTypePointer(SimStruct({"nameType": SimTypeInt(signed=False, label="IKEEXT_CERT_CRITERIA_NAME_TYPE"), "certName": SimTypePointer(SimTypeChar(label="Char"), offset=0)}, name="IKEEXT_CERT_NAME0", pack=False, align=None), offset=0), "flags": SimTypeInt(signed=False, label="UInt32")}, name="IKEEXT_CERTIFICATE_CRITERIA0", pack=False, align=None), offset=0)}, name="_Anonymous2_e__Struct", pack=False, align=None), "Anonymous3": SimStruct({"inboundRootStoreArraySize": SimTypeInt(signed=False, label="UInt32"), "inboundTrustedRootStoreCriteria": SimTypePointer(SimStruct({"certData": SimStruct({"size": SimTypeInt(signed=False, label="UInt32"), "data": SimTypePointer(SimTypeChar(label="Byte"), offset=0)}, name="FWP_BYTE_BLOB", pack=False, align=None), "certHash": SimStruct({"size": SimTypeInt(signed=False, label="UInt32"), "data": SimTypePointer(SimTypeChar(label="Byte"), offset=0)}, name="FWP_BYTE_BLOB", pack=False, align=None), "eku": SimTypePointer(SimStruct({"numEku": SimTypeInt(signed=False, label="UInt32"), "eku": SimTypePointer(SimTypePointer(SimTypeChar(label="Byte"), offset=0), offset=0)}, name="IKEEXT_CERT_EKUS0", pack=False, align=None), offset=0), "name": SimTypePointer(SimStruct({"nameType": SimTypeInt(signed=False, label="IKEEXT_CERT_CRITERIA_NAME_TYPE"), "certName": SimTypePointer(SimTypeChar(label="Char"), offset=0)}, name="IKEEXT_CERT_NAME0", pack=False, align=None), offset=0), "flags": SimTypeInt(signed=False, label="UInt32")}, name="IKEEXT_CERTIFICATE_CRITERIA0", pack=False, align=None), offset=0)}, name="_Anonymous3_e__Struct", pack=False, align=None)}, name="<anon>", label="None"), "outboundConfigType": SimTypeInt(signed=False, label="IKEEXT_CERT_CONFIG_TYPE"), "Anonymous2": SimUnion({"Anonymous1": SimStruct({"outboundRootArraySize": SimTypeInt(signed=False, label="UInt32"), "outboundRootCriteria": SimTypePointer(SimStruct({"certData": SimStruct({"size": SimTypeInt(signed=False, label="UInt32"), "data": SimTypePointer(SimTypeChar(label="Byte"), offset=0)}, name="FWP_BYTE_BLOB", pack=False, align=None), "certHash": SimStruct({"size": SimTypeInt(signed=False, label="UInt32"), "data": SimTypePointer(SimTypeChar(label="Byte"), offset=0)}, name="FWP_BYTE_BLOB", pack=False, align=None), "eku": SimTypePointer(SimStruct({"numEku": SimTypeInt(signed=False, label="UInt32"), "eku": SimTypePointer(SimTypePointer(SimTypeChar(label="Byte"), offset=0), offset=0)}, name="IKEEXT_CERT_EKUS0", pack=False, align=None), offset=0), "name": SimTypePointer(SimStruct({"nameType": SimTypeInt(signed=False, label="IKEEXT_CERT_CRITERIA_NAME_TYPE"), "certName": SimTypePointer(SimTypeChar(label="Char"), offset=0)}, name="IKEEXT_CERT_NAME0", pack=False, align=None), offset=0), "flags": SimTypeInt(signed=False, label="UInt32")}, name="IKEEXT_CERTIFICATE_CRITERIA0", pack=False, align=None), offset=0)}, name="_Anonymous1_e__Struct", pack=False, align=None), "Anonymous2": SimStruct({"outboundEnterpriseStoreArraySize": SimTypeInt(signed=False, label="UInt32"), "outboundEnterpriseStoreCriteria": SimTypePointer(SimStruct({"certData": SimStruct({"size": SimTypeInt(signed=False, label="UInt32"), "data": SimTypePointer(SimTypeChar(label="Byte"), offset=0)}, name="FWP_BYTE_BLOB", pack=False, align=None), "certHash": SimStruct({"size": SimTypeInt(signed=False, label="UInt32"), "data": SimTypePointer(SimTypeChar(label="Byte"), offset=0)}, name="FWP_BYTE_BLOB", pack=False, align=None), "eku": SimTypePointer(SimStruct({"numEku": SimTypeInt(signed=False, label="UInt32"), "eku": SimTypePointer(SimTypePointer(SimTypeChar(label="Byte"), offset=0), offset=0)}, name="IKEEXT_CERT_EKUS0", pack=False, align=None), offset=0), "name": SimTypePointer(SimStruct({"nameType": SimTypeInt(signed=False, label="IKEEXT_CERT_CRITERIA_NAME_TYPE"), "certName": SimTypePointer(SimTypeChar(label="Char"), offset=0)}, name="IKEEXT_CERT_NAME0", pack=False, align=None), offset=0), "flags": SimTypeInt(signed=False, label="UInt32")}, name="IKEEXT_CERTIFICATE_CRITERIA0", pack=False, align=None), offset=0)}, name="_Anonymous2_e__Struct", pack=False, align=None), "Anonymous3": SimStruct({"outboundRootStoreArraySize": SimTypeInt(signed=False, label="UInt32"), "outboundTrustedRootStoreCriteria": SimTypePointer(SimStruct({"certData": SimStruct({"size": SimTypeInt(signed=False, label="UInt32"), "data": SimTypePointer(SimTypeChar(label="Byte"), offset=0)}, name="FWP_BYTE_BLOB", pack=False, align=None), "certHash": SimStruct({"size": SimTypeInt(signed=False, label="UInt32"), "data": SimTypePointer(SimTypeChar(label="Byte"), offset=0)}, name="FWP_BYTE_BLOB", pack=False, align=None), "eku": SimTypePointer(SimStruct({"numEku": SimTypeInt(signed=False, label="UInt32"), "eku": SimTypePointer(SimTypePointer(SimTypeChar(label="Byte"), offset=0), offset=0)}, name="IKEEXT_CERT_EKUS0", pack=False, align=None), offset=0), "name": SimTypePointer(SimStruct({"nameType": SimTypeInt(signed=False, label="IKEEXT_CERT_CRITERIA_NAME_TYPE"), "certName": SimTypePointer(SimTypeChar(label="Char"), offset=0)}, name="IKEEXT_CERT_NAME0", pack=False, align=None), offset=0), "flags": SimTypeInt(signed=False, label="UInt32")}, name="IKEEXT_CERTIFICATE_CRITERIA0", pack=False, align=None), offset=0)}, name="_Anonymous3_e__Struct", pack=False, align=None)}, name="<anon>", label="None"), "flags": SimTypeInt(signed=False, label="IKEEXT_CERT_AUTH"), "localCertLocationUrl": SimStruct({"size": SimTypeInt(signed=False, label="UInt32"), "data": SimTypePointer(SimTypeChar(label="Byte"), offset=0)}, name="FWP_BYTE_BLOB", pack=False, align=None)}, name="IKEEXT_CERTIFICATE_AUTHENTICATION2", pack=False, align=None), "kerberosAuthentication": SimStruct({"flags": SimTypeInt(signed=False, label="IKEEXT_KERBEROS_AUTHENTICATION_FLAGS"), "proxyServer": SimTypePointer(SimTypeChar(label="Char"), offset=0)}, name="IKEEXT_KERBEROS_AUTHENTICATION1", pack=False, align=None), "reservedAuthentication": SimStruct({"flags": SimTypeInt(signed=False, label="IKEEXT_RESERVED_AUTHENTICATION_FLAGS")}, name="IKEEXT_RESERVED_AUTHENTICATION0", pack=False, align=None), "ntlmV2Authentication": SimStruct({"flags": SimTypeInt(signed=False, label="UInt32")}, name="IKEEXT_NTLM_V2_AUTHENTICATION0", pack=False, align=None), "sslAuthentication": SimStruct({"inboundConfigType": SimTypeInt(signed=False, label="IKEEXT_CERT_CONFIG_TYPE"), "Anonymous1": SimUnion({"Anonymous1": SimStruct({"inboundRootArraySize": SimTypeInt(signed=False, label="UInt32"), "inboundRootCriteria": SimTypePointer(SimStruct({"certData": SimStruct({"size": SimTypeInt(signed=False, label="UInt32"), "data": SimTypePointer(SimTypeChar(label="Byte"), offset=0)}, name="FWP_BYTE_BLOB", pack=False, align=None), "certHash": SimStruct({"size": SimTypeInt(signed=False, label="UInt32"), "data": SimTypePointer(SimTypeChar(label="Byte"), offset=0)}, name="FWP_BYTE_BLOB", pack=False, align=None), "eku": SimTypePointer(SimStruct({"numEku": SimTypeInt(signed=False, label="UInt32"), "eku": SimTypePointer(SimTypePointer(SimTypeChar(label="Byte"), offset=0), offset=0)}, name="IKEEXT_CERT_EKUS0", pack=False, align=None), offset=0), "name": SimTypePointer(SimStruct({"nameType": SimTypeInt(signed=False, label="IKEEXT_CERT_CRITERIA_NAME_TYPE"), "certName": SimTypePointer(SimTypeChar(label="Char"), offset=0)}, name="IKEEXT_CERT_NAME0", pack=False, align=None), offset=0), "flags": SimTypeInt(signed=False, label="UInt32")}, name="IKEEXT_CERTIFICATE_CRITERIA0", pack=False, align=None), offset=0)}, name="_Anonymous1_e__Struct", pack=False, align=None), "Anonymous2": SimStruct({"inboundEnterpriseStoreArraySize": SimTypeInt(signed=False, label="UInt32"), "inboundEnterpriseStoreCriteria": SimTypePointer(SimStruct({"certData": SimStruct({"size": SimTypeInt(signed=False, label="UInt32"), "data": SimTypePointer(SimTypeChar(label="Byte"), offset=0)}, name="FWP_BYTE_BLOB", pack=False, align=None), "certHash": SimStruct({"size": SimTypeInt(signed=False, label="UInt32"), "data": SimTypePointer(SimTypeChar(label="Byte"), offset=0)}, name="FWP_BYTE_BLOB", pack=False, align=None), "eku": SimTypePointer(SimStruct({"numEku": SimTypeInt(signed=False, label="UInt32"), "eku": SimTypePointer(SimTypePointer(SimTypeChar(label="Byte"), offset=0), offset=0)}, name="IKEEXT_CERT_EKUS0", pack=False, align=None), offset=0), "name": SimTypePointer(SimStruct({"nameType": SimTypeInt(signed=False, label="IKEEXT_CERT_CRITERIA_NAME_TYPE"), "certName": SimTypePointer(SimTypeChar(label="Char"), offset=0)}, name="IKEEXT_CERT_NAME0", pack=False, align=None), offset=0), "flags": SimTypeInt(signed=False, label="UInt32")}, name="IKEEXT_CERTIFICATE_CRITERIA0", pack=False, align=None), offset=0)}, name="_Anonymous2_e__Struct", pack=False, align=None), "Anonymous3": SimStruct({"inboundRootStoreArraySize": SimTypeInt(signed=False, label="UInt32"), "inboundTrustedRootStoreCriteria": SimTypePointer(SimStruct({"certData": SimStruct({"size": SimTypeInt(signed=False, label="UInt32"), "data": SimTypePointer(SimTypeChar(label="Byte"), offset=0)}, name="FWP_BYTE_BLOB", pack=False, align=None), "certHash": SimStruct({"size": SimTypeInt(signed=False, label="UInt32"), "data": SimTypePointer(SimTypeChar(label="Byte"), offset=0)}, name="FWP_BYTE_BLOB", pack=False, align=None), "eku": SimTypePointer(SimStruct({"numEku": SimTypeInt(signed=False, label="UInt32"), "eku": SimTypePointer(SimTypePointer(SimTypeChar(label="Byte"), offset=0), offset=0)}, name="IKEEXT_CERT_EKUS0", pack=False, align=None), offset=0), "name": SimTypePointer(SimStruct({"nameType": SimTypeInt(signed=False, label="IKEEXT_CERT_CRITERIA_NAME_TYPE"), "certName": SimTypePointer(SimTypeChar(label="Char"), offset=0)}, name="IKEEXT_CERT_NAME0", pack=False, align=None), offset=0), "flags": SimTypeInt(signed=False, label="UInt32")}, name="IKEEXT_CERTIFICATE_CRITERIA0", pack=False, align=None), offset=0)}, name="_Anonymous3_e__Struct", pack=False, align=None)}, name="<anon>", label="None"), "outboundConfigType": SimTypeInt(signed=False, label="IKEEXT_CERT_CONFIG_TYPE"), "Anonymous2": SimUnion({"Anonymous1": SimStruct({"outboundRootArraySize": SimTypeInt(signed=False, label="UInt32"), "outboundRootCriteria": SimTypePointer(SimStruct({"certData": SimStruct({"size": SimTypeInt(signed=False, label="UInt32"), "data": SimTypePointer(SimTypeChar(label="Byte"), offset=0)}, name="FWP_BYTE_BLOB", pack=False, align=None), "certHash": SimStruct({"size": SimTypeInt(signed=False, label="UInt32"), "data": SimTypePointer(SimTypeChar(label="Byte"), offset=0)}, name="FWP_BYTE_BLOB", pack=False, align=None), "eku": SimTypePointer(SimStruct({"numEku": SimTypeInt(signed=False, label="UInt32"), "eku": SimTypePointer(SimTypePointer(SimTypeChar(label="Byte"), offset=0), offset=0)}, name="IKEEXT_CERT_EKUS0", pack=False, align=None), offset=0), "name": SimTypePointer(SimStruct({"nameType": SimTypeInt(signed=False, label="IKEEXT_CERT_CRITERIA_NAME_TYPE"), "certName": SimTypePointer(SimTypeChar(label="Char"), offset=0)}, name="IKEEXT_CERT_NAME0", pack=False, align=None), offset=0), "flags": SimTypeInt(signed=False, label="UInt32")}, name="IKEEXT_CERTIFICATE_CRITERIA0", pack=False, align=None), offset=0)}, name="_Anonymous1_e__Struct", pack=False, align=None), "Anonymous2": SimStruct({"outboundEnterpriseStoreArraySize": SimTypeInt(signed=False, label="UInt32"), "outboundEnterpriseStoreCriteria": SimTypePointer(SimStruct({"certData": SimStruct({"size": SimTypeInt(signed=False, label="UInt32"), "data": SimTypePointer(SimTypeChar(label="Byte"), offset=0)}, name="FWP_BYTE_BLOB", pack=False, align=None), "certHash": SimStruct({"size": SimTypeInt(signed=False, label="UInt32"), "data": SimTypePointer(SimTypeChar(label="Byte"), offset=0)}, name="FWP_BYTE_BLOB", pack=False, align=None), "eku": SimTypePointer(SimStruct({"numEku": SimTypeInt(signed=False, label="UInt32"), "eku": SimTypePointer(SimTypePointer(SimTypeChar(label="Byte"), offset=0), offset=0)}, name="IKEEXT_CERT_EKUS0", pack=False, align=None), offset=0), "name": SimTypePointer(SimStruct({"nameType": SimTypeInt(signed=False, label="IKEEXT_CERT_CRITERIA_NAME_TYPE"), "certName": SimTypePointer(SimTypeChar(label="Char"), offset=0)}, name="IKEEXT_CERT_NAME0", pack=False, align=None), offset=0), "flags": SimTypeInt(signed=False, label="UInt32")}, name="IKEEXT_CERTIFICATE_CRITERIA0", pack=False, align=None), offset=0)}, name="_Anonymous2_e__Struct", pack=False, align=None), "Anonymous3": SimStruct({"outboundRootStoreArraySize": SimTypeInt(signed=False, label="UInt32"), "outboundTrustedRootStoreCriteria": SimTypePointer(SimStruct({"certData": SimStruct({"size": SimTypeInt(signed=False, label="UInt32"), "data": SimTypePointer(SimTypeChar(label="Byte"), offset=0)}, name="FWP_BYTE_BLOB", pack=False, align=None), "certHash": SimStruct({"size": SimTypeInt(signed=False, label="UInt32"), "data": SimTypePointer(SimTypeChar(label="Byte"), offset=0)}, name="FWP_BYTE_BLOB", pack=False, align=None), "eku": SimTypePointer(SimStruct({"numEku": SimTypeInt(signed=False, label="UInt32"), "eku": SimTypePointer(SimTypePointer(SimTypeChar(label="Byte"), offset=0), offset=0)}, name="IKEEXT_CERT_EKUS0", pack=False, align=None), offset=0), "name": SimTypePointer(SimStruct({"nameType": SimTypeInt(signed=False, label="IKEEXT_CERT_CRITERIA_NAME_TYPE"), "certName": SimTypePointer(SimTypeChar(label="Char"), offset=0)}, name="IKEEXT_CERT_NAME0", pack=False, align=None), offset=0), "flags": SimTypeInt(signed=False, label="UInt32")}, name="IKEEXT_CERTIFICATE_CRITERIA0", pack=False, align=None), offset=0)}, name="_Anonymous3_e__Struct", pack=False, align=None)}, name="<anon>", label="None"), "flags": SimTypeInt(signed=False, label="IKEEXT_CERT_AUTH"), "localCertLocationUrl": SimStruct({"size": SimTypeInt(signed=False, label="UInt32"), "data": SimTypePointer(SimTypeChar(label="Byte"), offset=0)}, name="FWP_BYTE_BLOB", pack=False, align=None)}, name="IKEEXT_CERTIFICATE_AUTHENTICATION2", pack=False, align=None), "cgaAuthentication": SimStruct({"keyContainerName": SimTypePointer(SimTypeChar(label="Char"), offset=0), "cspName": SimTypePointer(SimTypeChar(label="Char"), offset=0), "cspType": SimTypeInt(signed=False, label="UInt32"), "cgaModifier": SimStruct({"byteArray16": SimTypeFixedSizeArray(SimTypeChar(label="Byte"), 16)}, name="FWP_BYTE_ARRAY16", pack=False, align=None), "cgaCollisionCount": SimTypeChar(label="Byte")}, name="IKEEXT_IPV6_CGA_AUTHENTICATION0", pack=False, align=None), "eapAuthentication": SimStruct({"flags": SimTypeInt(signed=False, label="IKEEXT_EAP_AUTHENTICATION_FLAGS")}, name="IKEEXT_EAP_AUTHENTICATION0", pack=False, align=None)}, name="<anon>", label="None")}, name="IKEEXT_AUTHENTICATION_METHOD2", pack=False, align=None), offset=0), "initiatorImpersonationType": SimTypeInt(signed=False, label="IKEEXT_AUTHENTICATION_IMPERSONATION_TYPE"), "numIkeProposals": SimTypeInt(signed=False, label="UInt32"), "ikeProposals": SimTypePointer(SimStruct({"cipherAlgorithm": SimStruct({"algoIdentifier": SimTypeInt(signed=False, label="IKEEXT_CIPHER_TYPE"), "keyLen": SimTypeInt(signed=False, label="UInt32"), "rounds": SimTypeInt(signed=False, label="UInt32")}, name="IKEEXT_CIPHER_ALGORITHM0", pack=False, align=None), "integrityAlgorithm": SimStruct({"algoIdentifier": SimTypeInt(signed=False, label="IKEEXT_INTEGRITY_TYPE")}, name="IKEEXT_INTEGRITY_ALGORITHM0", pack=False, align=None), "maxLifetimeSeconds": SimTypeInt(signed=False, label="UInt32"), "dhGroup": SimTypeInt(signed=False, label="IKEEXT_DH_GROUP"), "quickModeLimit": SimTypeInt(signed=False, label="UInt32")}, name="IKEEXT_PROPOSAL0", pack=False, align=None), offset=0), "flags": SimTypeInt(signed=False, label="IKEEXT_POLICY_FLAG"), "maxDynamicFilters": SimTypeInt(signed=False, label="UInt32"), "retransmitDurationSecs": SimTypeInt(signed=False, label="UInt32")}, name="IKEEXT_POLICY2", pack=False, align=None), offset=0), "dataBuffer": SimTypePointer(SimStruct({"size": SimTypeInt(signed=False, label="UInt32"), "data": SimTypePointer(SimTypeChar(label="Byte"), offset=0)}, name="FWP_BYTE_BLOB", pack=False, align=None), offset=0), "classifyOptions": SimTypePointer(SimStruct({"numOptions": SimTypeInt(signed=False, label="UInt32"), "options": SimTypePointer(SimStruct({"type": SimTypeInt(signed=False, label="FWP_CLASSIFY_OPTION_TYPE"), "value": SimStruct({"type": SimTypeInt(signed=False, label="FWP_DATA_TYPE"), "Anonymous": SimUnion({"uint8": SimTypeChar(label="Byte"), "uint16": SimTypeShort(signed=False, label="UInt16"), "uint32": SimTypeInt(signed=False, label="UInt32"), "uint64": SimTypePointer(SimTypeLongLong(signed=False, label="UInt64"), offset=0), "int8": SimTypeChar(label="SByte"), "int16": SimTypeShort(signed=True, label="Int16"), "int32": SimTypeInt(signed=True, label="Int32"), "int64": SimTypePointer(SimTypeLongLong(signed=True, label="Int64"), offset=0), "float32": SimTypeFloat(size=32), "double64": SimTypePointer(SimTypeFloat(size=64), offset=0), "byteArray16": SimTypePointer(SimStruct({"byteArray16": SimTypeFixedSizeArray(SimTypeChar(label="Byte"), 16)}, name="FWP_BYTE_ARRAY16", pack=False, align=None), offset=0), "byteBlob": SimTypePointer(SimStruct({"size": SimTypeInt(signed=False, label="UInt32"), "data": SimTypePointer(SimTypeChar(label="Byte"), offset=0)}, name="FWP_BYTE_BLOB", pack=False, align=None), offset=0), "sid": SimTypePointer(SimTypeBottom(label="SID"), offset=0), "sd": SimTypePointer(SimStruct({"size": SimTypeInt(signed=False, label="UInt32"), "data": SimTypePointer(SimTypeChar(label="Byte"), offset=0)}, name="FWP_BYTE_BLOB", pack=False, align=None), offset=0), "tokenInformation": SimTypePointer(SimStruct({"sidCount": SimTypeInt(signed=False, label="UInt32"), "sids": SimTypePointer(SimTypeBottom(label="SID_AND_ATTRIBUTES"), offset=0), "restrictedSidCount": SimTypeInt(signed=False, label="UInt32"), "restrictedSids": SimTypePointer(SimTypeBottom(label="SID_AND_ATTRIBUTES"), offset=0)}, name="FWP_TOKEN_INFORMATION", pack=False, align=None), offset=0), "tokenAccessInformation": SimTypePointer(SimStruct({"size": SimTypeInt(signed=False, label="UInt32"), "data": SimTypePointer(SimTypeChar(label="Byte"), offset=0)}, name="FWP_BYTE_BLOB", pack=False, align=None), offset=0), "unicodeString": SimTypePointer(SimTypeChar(label="Char"), offset=0), "byteArray6": SimTypePointer(SimStruct({"byteArray6": SimTypeFixedSizeArray(SimTypeChar(label="Byte"), 6)}, name="FWP_BYTE_ARRAY6", pack=False, align=None), offset=0), "bitmapArray64": SimTypePointer(SimStruct({"bitmapArray64": SimTypeFixedSizeArray(SimTypeChar(label="Byte"), 8)}, name="FWP_BITMAP_ARRAY64_", pack=False, align=None), offset=0)}, name="<anon>", label="None")}, name="FWP_VALUE0", pack=False, align=None)}, name="FWPM_CLASSIFY_OPTION0", pack=False, align=None), offset=0)}, name="FWPM_CLASSIFY_OPTIONS0", pack=False, align=None), offset=0), "ikeV2QmTunnelPolicy": SimTypePointer(SimStruct({"flags": SimTypeInt(signed=False, label="IPSEC_POLICY_FLAG"), "numIpsecProposals": SimTypeInt(signed=False, label="UInt32"), "ipsecProposals": SimTypePointer(SimStruct({"lifetime": SimStruct({"lifetimeSeconds": SimTypeInt(signed=False, label="UInt32"), "lifetimeKilobytes": SimTypeInt(signed=False, label="UInt32"), "lifetimePackets": SimTypeInt(signed=False, label="UInt32")}, name="IPSEC_SA_LIFETIME0", pack=False, align=None), "numSaTransforms": SimTypeInt(signed=False, label="UInt32"), "saTransforms": SimTypePointer(SimStruct({"ipsecTransformType": SimTypeInt(signed=False, label="IPSEC_TRANSFORM_TYPE"), "Anonymous": SimUnion({"ahTransform": SimTypePointer(SimStruct({"authTransformId": SimStruct({"authType": SimTypeInt(signed=False, label="IPSEC_AUTH_TYPE"), "authConfig": SimTypeChar(label="Byte")}, name="IPSEC_AUTH_TRANSFORM_ID0", pack=False, align=None), "cryptoModuleId": SimTypePointer(SimTypeBottom(label="Guid"), offset=0)}, name="IPSEC_AUTH_TRANSFORM0", pack=False, align=None), offset=0), "espAuthTransform": SimTypePointer(SimStruct({"authTransformId": SimStruct({"authType": SimTypeInt(signed=False, label="IPSEC_AUTH_TYPE"), "authConfig": SimTypeChar(label="Byte")}, name="IPSEC_AUTH_TRANSFORM_ID0", pack=False, align=None), "cryptoModuleId": SimTypePointer(SimTypeBottom(label="Guid"), offset=0)}, name="IPSEC_AUTH_TRANSFORM0", pack=False, align=None), offset=0), "espCipherTransform": SimTypePointer(SimStruct({"cipherTransformId": SimStruct({"cipherType": SimTypeInt(signed=False, label="IPSEC_CIPHER_TYPE"), "cipherConfig": SimTypeChar(label="Byte")}, name="IPSEC_CIPHER_TRANSFORM_ID0", pack=False, align=None), "cryptoModuleId": SimTypePointer(SimTypeBottom(label="Guid"), offset=0)}, name="IPSEC_CIPHER_TRANSFORM0", pack=False, align=None), offset=0), "espAuthAndCipherTransform": SimTypePointer(SimStruct({"authTransform": SimStruct({"authTransformId": SimStruct({"authType": SimTypeInt(signed=False, label="IPSEC_AUTH_TYPE"), "authConfig": SimTypeChar(label="Byte")}, name="IPSEC_AUTH_TRANSFORM_ID0", pack=False, align=None), "cryptoModuleId": SimTypePointer(SimTypeBottom(label="Guid"), offset=0)}, name="IPSEC_AUTH_TRANSFORM0", pack=False, align=None), "cipherTransform": SimStruct({"cipherTransformId": SimStruct({"cipherType": SimTypeInt(signed=False, label="IPSEC_CIPHER_TYPE"), "cipherConfig": SimTypeChar(label="Byte")}, name="IPSEC_CIPHER_TRANSFORM_ID0", pack=False, align=None), "cryptoModuleId": SimTypePointer(SimTypeBottom(label="Guid"), offset=0)}, name="IPSEC_CIPHER_TRANSFORM0", pack=False, align=None)}, name="IPSEC_AUTH_AND_CIPHER_TRANSFORM0", pack=False, align=None), offset=0), "espAuthFwTransform": SimTypePointer(SimStruct({"authTransformId": SimStruct({"authType": SimTypeInt(signed=False, label="IPSEC_AUTH_TYPE"), "authConfig": SimTypeChar(label="Byte")}, name="IPSEC_AUTH_TRANSFORM_ID0", pack=False, align=None), "cryptoModuleId": SimTypePointer(SimTypeBottom(label="Guid"), offset=0)}, name="IPSEC_AUTH_TRANSFORM0", pack=False, align=None), offset=0)}, name="<anon>", label="None")}, name="IPSEC_SA_TRANSFORM0", pack=False, align=None), offset=0), "pfsGroup": SimTypeInt(signed=False, label="IPSEC_PFS_GROUP")}, name="IPSEC_PROPOSAL0", pack=False, align=None), offset=0), "tunnelEndpoints": SimStruct({"ipVersion": SimTypeInt(signed=False, label="FWP_IP_VERSION"), "Anonymous1": SimUnion({"localV4Address": SimTypeInt(signed=False, label="UInt32"), "localV6Address": SimTypeFixedSizeArray(SimTypeChar(label="Byte"), 16)}, name="<anon>", label="None"), "Anonymous2": SimUnion({"remoteV4Address": SimTypeInt(signed=False, label="UInt32"), "remoteV6Address": SimTypeFixedSizeArray(SimTypeChar(label="Byte"), 16)}, name="<anon>", label="None"), "localIfLuid": SimTypeLongLong(signed=False, label="UInt64"), "remoteFqdn": SimTypePointer(SimTypeChar(label="Char"), offset=0), "numAddresses": SimTypeInt(signed=False, label="UInt32"), "remoteAddresses": SimTypePointer(SimStruct({"ipVersion": SimTypeInt(signed=False, label="FWP_IP_VERSION"), "Anonymous": SimUnion({"v4Address": SimTypeInt(signed=False, label="UInt32"), "v6Address": SimTypeFixedSizeArray(SimTypeChar(label="Byte"), 16)}, name="<anon>", label="None")}, name="IPSEC_TUNNEL_ENDPOINT0", pack=False, align=None), offset=0)}, name="IPSEC_TUNNEL_ENDPOINTS2", pack=False, align=None), "saIdleTimeout": SimStruct({"idleTimeoutSeconds": SimTypeInt(signed=False, label="UInt32"), "idleTimeoutSecondsFailOver": SimTypeInt(signed=False, label="UInt32")}, name="IPSEC_SA_IDLE_TIMEOUT0", pack=False, align=None), "emPolicy": SimTypePointer(SimStruct({"numAuthenticationMethods": SimTypeInt(signed=False, label="UInt32"), "authenticationMethods": SimTypePointer(SimStruct({"authenticationMethodType": SimTypeInt(signed=False, label="IKEEXT_AUTHENTICATION_METHOD_TYPE"), "Anonymous": SimUnion({"presharedKeyAuthentication": SimStruct({"presharedKey": SimStruct({"size": SimTypeInt(signed=False, label="UInt32"), "data": SimTypePointer(SimTypeChar(label="Byte"), offset=0)}, name="FWP_BYTE_BLOB", pack=False, align=None), "flags": SimTypeInt(signed=False, label="IKEEXT_PRESHARED_KEY_AUTHENTICATION_FLAGS")}, name="IKEEXT_PRESHARED_KEY_AUTHENTICATION1", pack=False, align=None), "certificateAuthentication": SimStruct({"inboundConfigType": SimTypeInt(signed=False, label="IKEEXT_CERT_CONFIG_TYPE"), "Anonymous1": SimUnion({"Anonymous1": SimStruct({"inboundRootArraySize": SimTypeInt(signed=False, label="UInt32"), "inboundRootCriteria": SimTypePointer(SimStruct({"certData": SimStruct({"size": SimTypeInt(signed=False, label="UInt32"), "data": SimTypePointer(SimTypeChar(label="Byte"), offset=0)}, name="FWP_BYTE_BLOB", pack=False, align=None), "certHash": SimStruct({"size": SimTypeInt(signed=False, label="UInt32"), "data": SimTypePointer(SimTypeChar(label="Byte"), offset=0)}, name="FWP_BYTE_BLOB", pack=False, align=None), "eku": SimTypePointer(SimStruct({"numEku": SimTypeInt(signed=False, label="UInt32"), "eku": SimTypePointer(SimTypePointer(SimTypeChar(label="Byte"), offset=0), offset=0)}, name="IKEEXT_CERT_EKUS0", pack=False, align=None), offset=0), "name": SimTypePointer(SimStruct({"nameType": SimTypeInt(signed=False, label="IKEEXT_CERT_CRITERIA_NAME_TYPE"), "certName": SimTypePointer(SimTypeChar(label="Char"), offset=0)}, name="IKEEXT_CERT_NAME0", pack=False, align=None), offset=0), "flags": SimTypeInt(signed=False, label="UInt32")}, name="IKEEXT_CERTIFICATE_CRITERIA0", pack=False, align=None), offset=0)}, name="_Anonymous1_e__Struct", pack=False, align=None), "Anonymous2": SimStruct({"inboundEnterpriseStoreArraySize": SimTypeInt(signed=False, label="UInt32"), "inboundEnterpriseStoreCriteria": SimTypePointer(SimStruct({"certData": SimStruct({"size": SimTypeInt(signed=False, label="UInt32"), "data": SimTypePointer(SimTypeChar(label="Byte"), offset=0)}, name="FWP_BYTE_BLOB", pack=False, align=None), "certHash": SimStruct({"size": SimTypeInt(signed=False, label="UInt32"), "data": SimTypePointer(SimTypeChar(label="Byte"), offset=0)}, name="FWP_BYTE_BLOB", pack=False, align=None), "eku": SimTypePointer(SimStruct({"numEku": SimTypeInt(signed=False, label="UInt32"), "eku": SimTypePointer(SimTypePointer(SimTypeChar(label="Byte"), offset=0), offset=0)}, name="IKEEXT_CERT_EKUS0", pack=False, align=None), offset=0), "name": SimTypePointer(SimStruct({"nameType": SimTypeInt(signed=False, label="IKEEXT_CERT_CRITERIA_NAME_TYPE"), "certName": SimTypePointer(SimTypeChar(label="Char"), offset=0)}, name="IKEEXT_CERT_NAME0", pack=False, align=None), offset=0), "flags": SimTypeInt(signed=False, label="UInt32")}, name="IKEEXT_CERTIFICATE_CRITERIA0", pack=False, align=None), offset=0)}, name="_Anonymous2_e__Struct", pack=False, align=None), "Anonymous3": SimStruct({"inboundRootStoreArraySize": SimTypeInt(signed=False, label="UInt32"), "inboundTrustedRootStoreCriteria": SimTypePointer(SimStruct({"certData": SimStruct({"size": SimTypeInt(signed=False, label="UInt32"), "data": SimTypePointer(SimTypeChar(label="Byte"), offset=0)}, name="FWP_BYTE_BLOB", pack=False, align=None), "certHash": SimStruct({"size": SimTypeInt(signed=False, label="UInt32"), "data": SimTypePointer(SimTypeChar(label="Byte"), offset=0)}, name="FWP_BYTE_BLOB", pack=False, align=None), "eku": SimTypePointer(SimStruct({"numEku": SimTypeInt(signed=False, label="UInt32"), "eku": SimTypePointer(SimTypePointer(SimTypeChar(label="Byte"), offset=0), offset=0)}, name="IKEEXT_CERT_EKUS0", pack=False, align=None), offset=0), "name": SimTypePointer(SimStruct({"nameType": SimTypeInt(signed=False, label="IKEEXT_CERT_CRITERIA_NAME_TYPE"), "certName": SimTypePointer(SimTypeChar(label="Char"), offset=0)}, name="IKEEXT_CERT_NAME0", pack=False, align=None), offset=0), "flags": SimTypeInt(signed=False, label="UInt32")}, name="IKEEXT_CERTIFICATE_CRITERIA0", pack=False, align=None), offset=0)}, name="_Anonymous3_e__Struct", pack=False, align=None)}, name="<anon>", label="None"), "outboundConfigType": SimTypeInt(signed=False, label="IKEEXT_CERT_CONFIG_TYPE"), "Anonymous2": SimUnion({"Anonymous1": SimStruct({"outboundRootArraySize": SimTypeInt(signed=False, label="UInt32"), "outboundRootCriteria": SimTypePointer(SimStruct({"certData": SimStruct({"size": SimTypeInt(signed=False, label="UInt32"), "data": SimTypePointer(SimTypeChar(label="Byte"), offset=0)}, name="FWP_BYTE_BLOB", pack=False, align=None), "certHash": SimStruct({"size": SimTypeInt(signed=False, label="UInt32"), "data": SimTypePointer(SimTypeChar(label="Byte"), offset=0)}, name="FWP_BYTE_BLOB", pack=False, align=None), "eku": SimTypePointer(SimStruct({"numEku": SimTypeInt(signed=False, label="UInt32"), "eku": SimTypePointer(SimTypePointer(SimTypeChar(label="Byte"), offset=0), offset=0)}, name="IKEEXT_CERT_EKUS0", pack=False, align=None), offset=0), "name": SimTypePointer(SimStruct({"nameType": SimTypeInt(signed=False, label="IKEEXT_CERT_CRITERIA_NAME_TYPE"), "certName": SimTypePointer(SimTypeChar(label="Char"), offset=0)}, name="IKEEXT_CERT_NAME0", pack=False, align=None), offset=0), "flags": SimTypeInt(signed=False, label="UInt32")}, name="IKEEXT_CERTIFICATE_CRITERIA0", pack=False, align=None), offset=0)}, name="_Anonymous1_e__Struct", pack=False, align=None), "Anonymous2": SimStruct({"outboundEnterpriseStoreArraySize": SimTypeInt(signed=False, label="UInt32"), "outboundEnterpriseStoreCriteria": SimTypePointer(SimStruct({"certData": SimStruct({"size": SimTypeInt(signed=False, label="UInt32"), "data": SimTypePointer(SimTypeChar(label="Byte"), offset=0)}, name="FWP_BYTE_BLOB", pack=False, align=None), "certHash": SimStruct({"size": SimTypeInt(signed=False, label="UInt32"), "data": SimTypePointer(SimTypeChar(label="Byte"), offset=0)}, name="FWP_BYTE_BLOB", pack=False, align=None), "eku": SimTypePointer(SimStruct({"numEku": SimTypeInt(signed=False, label="UInt32"), "eku": SimTypePointer(SimTypePointer(SimTypeChar(label="Byte"), offset=0), offset=0)}, name="IKEEXT_CERT_EKUS0", pack=False, align=None), offset=0), "name": SimTypePointer(SimStruct({"nameType": SimTypeInt(signed=False, label="IKEEXT_CERT_CRITERIA_NAME_TYPE"), "certName": SimTypePointer(SimTypeChar(label="Char"), offset=0)}, name="IKEEXT_CERT_NAME0", pack=False, align=None), offset=0), "flags": SimTypeInt(signed=False, label="UInt32")}, name="IKEEXT_CERTIFICATE_CRITERIA0", pack=False, align=None), offset=0)}, name="_Anonymous2_e__Struct", pack=False, align=None), "Anonymous3": SimStruct({"outboundRootStoreArraySize": SimTypeInt(signed=False, label="UInt32"), "outboundTrustedRootStoreCriteria": SimTypePointer(SimStruct({"certData": SimStruct({"size": SimTypeInt(signed=False, label="UInt32"), "data": SimTypePointer(SimTypeChar(label="Byte"), offset=0)}, name="FWP_BYTE_BLOB", pack=False, align=None), "certHash": SimStruct({"size": SimTypeInt(signed=False, label="UInt32"), "data": SimTypePointer(SimTypeChar(label="Byte"), offset=0)}, name="FWP_BYTE_BLOB", pack=False, align=None), "eku": SimTypePointer(SimStruct({"numEku": SimTypeInt(signed=False, label="UInt32"), "eku": SimTypePointer(SimTypePointer(SimTypeChar(label="Byte"), offset=0), offset=0)}, name="IKEEXT_CERT_EKUS0", pack=False, align=None), offset=0), "name": SimTypePointer(SimStruct({"nameType": SimTypeInt(signed=False, label="IKEEXT_CERT_CRITERIA_NAME_TYPE"), "certName": SimTypePointer(SimTypeChar(label="Char"), offset=0)}, name="IKEEXT_CERT_NAME0", pack=False, align=None), offset=0), "flags": SimTypeInt(signed=False, label="UInt32")}, name="IKEEXT_CERTIFICATE_CRITERIA0", pack=False, align=None), offset=0)}, name="_Anonymous3_e__Struct", pack=False, align=None)}, name="<anon>", label="None"), "flags": SimTypeInt(signed=False, label="IKEEXT_CERT_AUTH"), "localCertLocationUrl": SimStruct({"size": SimTypeInt(signed=False, label="UInt32"), "data": SimTypePointer(SimTypeChar(label="Byte"), offset=0)}, name="FWP_BYTE_BLOB", pack=False, align=None)}, name="IKEEXT_CERTIFICATE_AUTHENTICATION2", pack=False, align=None), "kerberosAuthentication": SimStruct({"flags": SimTypeInt(signed=False, label="IKEEXT_KERBEROS_AUTHENTICATION_FLAGS"), "proxyServer": SimTypePointer(SimTypeChar(label="Char"), offset=0)}, name="IKEEXT_KERBEROS_AUTHENTICATION1", pack=False, align=None), "reservedAuthentication": SimStruct({"flags": SimTypeInt(signed=False, label="IKEEXT_RESERVED_AUTHENTICATION_FLAGS")}, name="IKEEXT_RESERVED_AUTHENTICATION0", pack=False, align=None), "ntlmV2Authentication": SimStruct({"flags": SimTypeInt(signed=False, label="UInt32")}, name="IKEEXT_NTLM_V2_AUTHENTICATION0", pack=False, align=None), "sslAuthentication": SimStruct({"inboundConfigType": SimTypeInt(signed=False, label="IKEEXT_CERT_CONFIG_TYPE"), "Anonymous1": SimUnion({"Anonymous1": SimStruct({"inboundRootArraySize": SimTypeInt(signed=False, label="UInt32"), "inboundRootCriteria": SimTypePointer(SimStruct({"certData": SimStruct({"size": SimTypeInt(signed=False, label="UInt32"), "data": SimTypePointer(SimTypeChar(label="Byte"), offset=0)}, name="FWP_BYTE_BLOB", pack=False, align=None), "certHash": SimStruct({"size": SimTypeInt(signed=False, label="UInt32"), "data": SimTypePointer(SimTypeChar(label="Byte"), offset=0)}, name="FWP_BYTE_BLOB", pack=False, align=None), "eku": SimTypePointer(SimStruct({"numEku": SimTypeInt(signed=False, label="UInt32"), "eku": SimTypePointer(SimTypePointer(SimTypeChar(label="Byte"), offset=0), offset=0)}, name="IKEEXT_CERT_EKUS0", pack=False, align=None), offset=0), "name": SimTypePointer(SimStruct({"nameType": SimTypeInt(signed=False, label="IKEEXT_CERT_CRITERIA_NAME_TYPE"), "certName": SimTypePointer(SimTypeChar(label="Char"), offset=0)}, name="IKEEXT_CERT_NAME0", pack=False, align=None), offset=0), "flags": SimTypeInt(signed=False, label="UInt32")}, name="IKEEXT_CERTIFICATE_CRITERIA0", pack=False, align=None), offset=0)}, name="_Anonymous1_e__Struct", pack=False, align=None), "Anonymous2": SimStruct({"inboundEnterpriseStoreArraySize": SimTypeInt(signed=False, label="UInt32"), "inboundEnterpriseStoreCriteria": SimTypePointer(SimStruct({"certData": SimStruct({"size": SimTypeInt(signed=False, label="UInt32"), "data": SimTypePointer(SimTypeChar(label="Byte"), offset=0)}, name="FWP_BYTE_BLOB", pack=False, align=None), "certHash": SimStruct({"size": SimTypeInt(signed=False, label="UInt32"), "data": SimTypePointer(SimTypeChar(label="Byte"), offset=0)}, name="FWP_BYTE_BLOB", pack=False, align=None), "eku": SimTypePointer(SimStruct({"numEku": SimTypeInt(signed=False, label="UInt32"), "eku": SimTypePointer(SimTypePointer(SimTypeChar(label="Byte"), offset=0), offset=0)}, name="IKEEXT_CERT_EKUS0", pack=False, align=None), offset=0), "name": SimTypePointer(SimStruct({"nameType": SimTypeInt(signed=False, label="IKEEXT_CERT_CRITERIA_NAME_TYPE"), "certName": SimTypePointer(SimTypeChar(label="Char"), offset=0)}, name="IKEEXT_CERT_NAME0", pack=False, align=None), offset=0), "flags": SimTypeInt(signed=False, label="UInt32")}, name="IKEEXT_CERTIFICATE_CRITERIA0", pack=False, align=None), offset=0)}, name="_Anonymous2_e__Struct", pack=False, align=None), "Anonymous3": SimStruct({"inboundRootStoreArraySize": SimTypeInt(signed=False, label="UInt32"), "inboundTrustedRootStoreCriteria": SimTypePointer(SimStruct({"certData": SimStruct({"size": SimTypeInt(signed=False, label="UInt32"), "data": SimTypePointer(SimTypeChar(label="Byte"), offset=0)}, name="FWP_BYTE_BLOB", pack=False, align=None), "certHash": SimStruct({"size": SimTypeInt(signed=False, label="UInt32"), "data": SimTypePointer(SimTypeChar(label="Byte"), offset=0)}, name="FWP_BYTE_BLOB", pack=False, align=None), "eku": SimTypePointer(SimStruct({"numEku": SimTypeInt(signed=False, label="UInt32"), "eku": SimTypePointer(SimTypePointer(SimTypeChar(label="Byte"), offset=0), offset=0)}, name="IKEEXT_CERT_EKUS0", pack=False, align=None), offset=0), "name": SimTypePointer(SimStruct({"nameType": SimTypeInt(signed=False, label="IKEEXT_CERT_CRITERIA_NAME_TYPE"), "certName": SimTypePointer(SimTypeChar(label="Char"), offset=0)}, name="IKEEXT_CERT_NAME0", pack=False, align=None), offset=0), "flags": SimTypeInt(signed=False, label="UInt32")}, name="IKEEXT_CERTIFICATE_CRITERIA0", pack=False, align=None), offset=0)}, name="_Anonymous3_e__Struct", pack=False, align=None)}, name="<anon>", label="None"), "outboundConfigType": SimTypeInt(signed=False, label="IKEEXT_CERT_CONFIG_TYPE"), "Anonymous2": SimUnion({"Anonymous1": SimStruct({"outboundRootArraySize": SimTypeInt(signed=False, label="UInt32"), "outboundRootCriteria": SimTypePointer(SimStruct({"certData": SimStruct({"size": SimTypeInt(signed=False, label="UInt32"), "data": SimTypePointer(SimTypeChar(label="Byte"), offset=0)}, name="FWP_BYTE_BLOB", pack=False, align=None), "certHash": SimStruct({"size": SimTypeInt(signed=False, label="UInt32"), "data": SimTypePointer(SimTypeChar(label="Byte"), offset=0)}, name="FWP_BYTE_BLOB", pack=False, align=None), "eku": SimTypePointer(SimStruct({"numEku": SimTypeInt(signed=False, label="UInt32"), "eku": SimTypePointer(SimTypePointer(SimTypeChar(label="Byte"), offset=0), offset=0)}, name="IKEEXT_CERT_EKUS0", pack=False, align=None), offset=0), "name": SimTypePointer(SimStruct({"nameType": SimTypeInt(signed=False, label="IKEEXT_CERT_CRITERIA_NAME_TYPE"), "certName": SimTypePointer(SimTypeChar(label="Char"), offset=0)}, name="IKEEXT_CERT_NAME0", pack=False, align=None), offset=0), "flags": SimTypeInt(signed=False, label="UInt32")}, name="IKEEXT_CERTIFICATE_CRITERIA0", pack=False, align=None), offset=0)}, name="_Anonymous1_e__Struct", pack=False, align=None), "Anonymous2": SimStruct({"outboundEnterpriseStoreArraySize": SimTypeInt(signed=False, label="UInt32"), "outboundEnterpriseStoreCriteria": SimTypePointer(SimStruct({"certData": SimStruct({"size": SimTypeInt(signed=False, label="UInt32"), "data": SimTypePointer(SimTypeChar(label="Byte"), offset=0)}, name="FWP_BYTE_BLOB", pack=False, align=None), "certHash": SimStruct({"size": SimTypeInt(signed=False, label="UInt32"), "data": SimTypePointer(SimTypeChar(label="Byte"), offset=0)}, name="FWP_BYTE_BLOB", pack=False, align=None), "eku": SimTypePointer(SimStruct({"numEku": SimTypeInt(signed=False, label="UInt32"), "eku": SimTypePointer(SimTypePointer(SimTypeChar(label="Byte"), offset=0), offset=0)}, name="IKEEXT_CERT_EKUS0", pack=False, align=None), offset=0), "name": SimTypePointer(SimStruct({"nameType": SimTypeInt(signed=False, label="IKEEXT_CERT_CRITERIA_NAME_TYPE"), "certName": SimTypePointer(SimTypeChar(label="Char"), offset=0)}, name="IKEEXT_CERT_NAME0", pack=False, align=None), offset=0), "flags": SimTypeInt(signed=False, label="UInt32")}, name="IKEEXT_CERTIFICATE_CRITERIA0", pack=False, align=None), offset=0)}, name="_Anonymous2_e__Struct", pack=False, align=None), "Anonymous3": SimStruct({"outboundRootStoreArraySize": SimTypeInt(signed=False, label="UInt32"), "outboundTrustedRootStoreCriteria": SimTypePointer(SimStruct({"certData": SimStruct({"size": SimTypeInt(signed=False, label="UInt32"), "data": SimTypePointer(SimTypeChar(label="Byte"), offset=0)}, name="FWP_BYTE_BLOB", pack=False, align=None), "certHash": SimStruct({"size": SimTypeInt(signed=False, label="UInt32"), "data": SimTypePointer(SimTypeChar(label="Byte"), offset=0)}, name="FWP_BYTE_BLOB", pack=False, align=None), "eku": SimTypePointer(SimStruct({"numEku": SimTypeInt(signed=False, label="UInt32"), "eku": SimTypePointer(SimTypePointer(SimTypeChar(label="Byte"), offset=0), offset=0)}, name="IKEEXT_CERT_EKUS0", pack=False, align=None), offset=0), "name": SimTypePointer(SimStruct({"nameType": SimTypeInt(signed=False, label="IKEEXT_CERT_CRITERIA_NAME_TYPE"), "certName": SimTypePointer(SimTypeChar(label="Char"), offset=0)}, name="IKEEXT_CERT_NAME0", pack=False, align=None), offset=0), "flags": SimTypeInt(signed=False, label="UInt32")}, name="IKEEXT_CERTIFICATE_CRITERIA0", pack=False, align=None), offset=0)}, name="_Anonymous3_e__Struct", pack=False, align=None)}, name="<anon>", label="None"), "flags": SimTypeInt(signed=False, label="IKEEXT_CERT_AUTH"), "localCertLocationUrl": SimStruct({"size": SimTypeInt(signed=False, label="UInt32"), "data": SimTypePointer(SimTypeChar(label="Byte"), offset=0)}, name="FWP_BYTE_BLOB", pack=False, align=None)}, name="IKEEXT_CERTIFICATE_AUTHENTICATION2", pack=False, align=None), "cgaAuthentication": SimStruct({"keyContainerName": SimTypePointer(SimTypeChar(label="Char"), offset=0), "cspName": SimTypePointer(SimTypeChar(label="Char"), offset=0), "cspType": SimTypeInt(signed=False, label="UInt32"), "cgaModifier": SimStruct({"byteArray16": SimTypeFixedSizeArray(SimTypeChar(label="Byte"), 16)}, name="FWP_BYTE_ARRAY16", pack=False, align=None), "cgaCollisionCount": SimTypeChar(label="Byte")}, name="IKEEXT_IPV6_CGA_AUTHENTICATION0", pack=False, align=None), "eapAuthentication": SimStruct({"flags": SimTypeInt(signed=False, label="IKEEXT_EAP_AUTHENTICATION_FLAGS")}, name="IKEEXT_EAP_AUTHENTICATION0", pack=False, align=None)}, name="<anon>", label="None")}, name="IKEEXT_AUTHENTICATION_METHOD2", pack=False, align=None), offset=0), "initiatorImpersonationType": SimTypeInt(signed=False, label="IKEEXT_AUTHENTICATION_IMPERSONATION_TYPE")}, name="IKEEXT_EM_POLICY2", pack=False, align=None), offset=0), "fwdPathSaLifetime": SimTypeInt(signed=False, label="UInt32")}, name="IPSEC_TUNNEL_POLICY2", pack=False, align=None), offset=0), "ikeV2QmTransportPolicy": SimTypePointer(SimStruct({"numIpsecProposals": SimTypeInt(signed=False, label="UInt32"), "ipsecProposals": SimTypePointer(SimStruct({"lifetime": SimStruct({"lifetimeSeconds": SimTypeInt(signed=False, label="UInt32"), "lifetimeKilobytes": SimTypeInt(signed=False, label="UInt32"), "lifetimePackets": SimTypeInt(signed=False, label="UInt32")}, name="IPSEC_SA_LIFETIME0", pack=False, align=None), "numSaTransforms": SimTypeInt(signed=False, label="UInt32"), "saTransforms": SimTypePointer(SimStruct({"ipsecTransformType": SimTypeInt(signed=False, label="IPSEC_TRANSFORM_TYPE"), "Anonymous": SimUnion({"ahTransform": SimTypePointer(SimStruct({"authTransformId": SimStruct({"authType": SimTypeInt(signed=False, label="IPSEC_AUTH_TYPE"), "authConfig": SimTypeChar(label="Byte")}, name="IPSEC_AUTH_TRANSFORM_ID0", pack=False, align=None), "cryptoModuleId": SimTypePointer(SimTypeBottom(label="Guid"), offset=0)}, name="IPSEC_AUTH_TRANSFORM0", pack=False, align=None), offset=0), "espAuthTransform": SimTypePointer(SimStruct({"authTransformId": SimStruct({"authType": SimTypeInt(signed=False, label="IPSEC_AUTH_TYPE"), "authConfig": SimTypeChar(label="Byte")}, name="IPSEC_AUTH_TRANSFORM_ID0", pack=False, align=None), "cryptoModuleId": SimTypePointer(SimTypeBottom(label="Guid"), offset=0)}, name="IPSEC_AUTH_TRANSFORM0", pack=False, align=None), offset=0), "espCipherTransform": SimTypePointer(SimStruct({"cipherTransformId": SimStruct({"cipherType": SimTypeInt(signed=False, label="IPSEC_CIPHER_TYPE"), "cipherConfig": SimTypeChar(label="Byte")}, name="IPSEC_CIPHER_TRANSFORM_ID0", pack=False, align=None), "cryptoModuleId": SimTypePointer(SimTypeBottom(label="Guid"), offset=0)}, name="IPSEC_CIPHER_TRANSFORM0", pack=False, align=None), offset=0), "espAuthAndCipherTransform": SimTypePointer(SimStruct({"authTransform": SimStruct({"authTransformId": SimStruct({"authType": SimTypeInt(signed=False, label="IPSEC_AUTH_TYPE"), "authConfig": SimTypeChar(label="Byte")}, name="IPSEC_AUTH_TRANSFORM_ID0", pack=False, align=None), "cryptoModuleId": SimTypePointer(SimTypeBottom(label="Guid"), offset=0)}, name="IPSEC_AUTH_TRANSFORM0", pack=False, align=None), "cipherTransform": SimStruct({"cipherTransformId": SimStruct({"cipherType": SimTypeInt(signed=False, label="IPSEC_CIPHER_TYPE"), "cipherConfig": SimTypeChar(label="Byte")}, name="IPSEC_CIPHER_TRANSFORM_ID0", pack=False, align=None), "cryptoModuleId": SimTypePointer(SimTypeBottom(label="Guid"), offset=0)}, name="IPSEC_CIPHER_TRANSFORM0", pack=False, align=None)}, name="IPSEC_AUTH_AND_CIPHER_TRANSFORM0", pack=False, align=None), offset=0), "espAuthFwTransform": SimTypePointer(SimStruct({"authTransformId": SimStruct({"authType": SimTypeInt(signed=False, label="IPSEC_AUTH_TYPE"), "authConfig": SimTypeChar(label="Byte")}, name="IPSEC_AUTH_TRANSFORM_ID0", pack=False, align=None), "cryptoModuleId": SimTypePointer(SimTypeBottom(label="Guid"), offset=0)}, name="IPSEC_AUTH_TRANSFORM0", pack=False, align=None), offset=0)}, name="<anon>", label="None")}, name="IPSEC_SA_TRANSFORM0", pack=False, align=None), offset=0), "pfsGroup": SimTypeInt(signed=False, label="IPSEC_PFS_GROUP")}, name="IPSEC_PROPOSAL0", pack=False, align=None), offset=0), "flags": SimTypeInt(signed=False, label="IPSEC_POLICY_FLAG"), "ndAllowClearTimeoutSeconds": SimTypeInt(signed=False, label="UInt32"), "saIdleTimeout": SimStruct({"idleTimeoutSeconds": SimTypeInt(signed=False, label="UInt32"), "idleTimeoutSecondsFailOver": SimTypeInt(signed=False, label="UInt32")}, name="IPSEC_SA_IDLE_TIMEOUT0", pack=False, align=None), "emPolicy": SimTypePointer(SimStruct({"numAuthenticationMethods": SimTypeInt(signed=False, label="UInt32"), "authenticationMethods": SimTypePointer(SimStruct({"authenticationMethodType": SimTypeInt(signed=False, label="IKEEXT_AUTHENTICATION_METHOD_TYPE"), "Anonymous": SimUnion({"presharedKeyAuthentication": SimStruct({"presharedKey": SimStruct({"size": SimTypeInt(signed=False, label="UInt32"), "data": SimTypePointer(SimTypeChar(label="Byte"), offset=0)}, name="FWP_BYTE_BLOB", pack=False, align=None), "flags": SimTypeInt(signed=False, label="IKEEXT_PRESHARED_KEY_AUTHENTICATION_FLAGS")}, name="IKEEXT_PRESHARED_KEY_AUTHENTICATION1", pack=False, align=None), "certificateAuthentication": SimStruct({"inboundConfigType": SimTypeInt(signed=False, label="IKEEXT_CERT_CONFIG_TYPE"), "Anonymous1": SimUnion({"Anonymous1": SimStruct({"inboundRootArraySize": SimTypeInt(signed=False, label="UInt32"), "inboundRootCriteria": SimTypePointer(SimStruct({"certData": SimStruct({"size": SimTypeInt(signed=False, label="UInt32"), "data": SimTypePointer(SimTypeChar(label="Byte"), offset=0)}, name="FWP_BYTE_BLOB", pack=False, align=None), "certHash": SimStruct({"size": SimTypeInt(signed=False, label="UInt32"), "data": SimTypePointer(SimTypeChar(label="Byte"), offset=0)}, name="FWP_BYTE_BLOB", pack=False, align=None), "eku": SimTypePointer(SimStruct({"numEku": SimTypeInt(signed=False, label="UInt32"), "eku": SimTypePointer(SimTypePointer(SimTypeChar(label="Byte"), offset=0), offset=0)}, name="IKEEXT_CERT_EKUS0", pack=False, align=None), offset=0), "name": SimTypePointer(SimStruct({"nameType": SimTypeInt(signed=False, label="IKEEXT_CERT_CRITERIA_NAME_TYPE"), "certName": SimTypePointer(SimTypeChar(label="Char"), offset=0)}, name="IKEEXT_CERT_NAME0", pack=False, align=None), offset=0), "flags": SimTypeInt(signed=False, label="UInt32")}, name="IKEEXT_CERTIFICATE_CRITERIA0", pack=False, align=None), offset=0)}, name="_Anonymous1_e__Struct", pack=False, align=None), "Anonymous2": SimStruct({"inboundEnterpriseStoreArraySize": SimTypeInt(signed=False, label="UInt32"), "inboundEnterpriseStoreCriteria": SimTypePointer(SimStruct({"certData": SimStruct({"size": SimTypeInt(signed=False, label="UInt32"), "data": SimTypePointer(SimTypeChar(label="Byte"), offset=0)}, name="FWP_BYTE_BLOB", pack=False, align=None), "certHash": SimStruct({"size": SimTypeInt(signed=False, label="UInt32"), "data": SimTypePointer(SimTypeChar(label="Byte"), offset=0)}, name="FWP_BYTE_BLOB", pack=False, align=None), "eku": SimTypePointer(SimStruct({"numEku": SimTypeInt(signed=False, label="UInt32"), "eku": SimTypePointer(SimTypePointer(SimTypeChar(label="Byte"), offset=0), offset=0)}, name="IKEEXT_CERT_EKUS0", pack=False, align=None), offset=0), "name": SimTypePointer(SimStruct({"nameType": SimTypeInt(signed=False, label="IKEEXT_CERT_CRITERIA_NAME_TYPE"), "certName": SimTypePointer(SimTypeChar(label="Char"), offset=0)}, name="IKEEXT_CERT_NAME0", pack=False, align=None), offset=0), "flags": SimTypeInt(signed=False, label="UInt32")}, name="IKEEXT_CERTIFICATE_CRITERIA0", pack=False, align=None), offset=0)}, name="_Anonymous2_e__Struct", pack=False, align=None), "Anonymous3": SimStruct({"inboundRootStoreArraySize": SimTypeInt(signed=False, label="UInt32"), "inboundTrustedRootStoreCriteria": SimTypePointer(SimStruct({"certData": SimStruct({"size": SimTypeInt(signed=False, label="UInt32"), "data": SimTypePointer(SimTypeChar(label="Byte"), offset=0)}, name="FWP_BYTE_BLOB", pack=False, align=None), "certHash": SimStruct({"size": SimTypeInt(signed=False, label="UInt32"), "data": SimTypePointer(SimTypeChar(label="Byte"), offset=0)}, name="FWP_BYTE_BLOB", pack=False, align=None), "eku": SimTypePointer(SimStruct({"numEku": SimTypeInt(signed=False, label="UInt32"), "eku": SimTypePointer(SimTypePointer(SimTypeChar(label="Byte"), offset=0), offset=0)}, name="IKEEXT_CERT_EKUS0", pack=False, align=None), offset=0), "name": SimTypePointer(SimStruct({"nameType": SimTypeInt(signed=False, label="IKEEXT_CERT_CRITERIA_NAME_TYPE"), "certName": SimTypePointer(SimTypeChar(label="Char"), offset=0)}, name="IKEEXT_CERT_NAME0", pack=False, align=None), offset=0), "flags": SimTypeInt(signed=False, label="UInt32")}, name="IKEEXT_CERTIFICATE_CRITERIA0", pack=False, align=None), offset=0)}, name="_Anonymous3_e__Struct", pack=False, align=None)}, name="<anon>", label="None"), "outboundConfigType": SimTypeInt(signed=False, label="IKEEXT_CERT_CONFIG_TYPE"), "Anonymous2": SimUnion({"Anonymous1": SimStruct({"outboundRootArraySize": SimTypeInt(signed=False, label="UInt32"), "outboundRootCriteria": SimTypePointer(SimStruct({"certData": SimStruct({"size": SimTypeInt(signed=False, label="UInt32"), "data": SimTypePointer(SimTypeChar(label="Byte"), offset=0)}, name="FWP_BYTE_BLOB", pack=False, align=None), "certHash": SimStruct({"size": SimTypeInt(signed=False, label="UInt32"), "data": SimTypePointer(SimTypeChar(label="Byte"), offset=0)}, name="FWP_BYTE_BLOB", pack=False, align=None), "eku": SimTypePointer(SimStruct({"numEku": SimTypeInt(signed=False, label="UInt32"), "eku": SimTypePointer(SimTypePointer(SimTypeChar(label="Byte"), offset=0), offset=0)}, name="IKEEXT_CERT_EKUS0", pack=False, align=None), offset=0), "name": SimTypePointer(SimStruct({"nameType": SimTypeInt(signed=False, label="IKEEXT_CERT_CRITERIA_NAME_TYPE"), "certName": SimTypePointer(SimTypeChar(label="Char"), offset=0)}, name="IKEEXT_CERT_NAME0", pack=False, align=None), offset=0), "flags": SimTypeInt(signed=False, label="UInt32")}, name="IKEEXT_CERTIFICATE_CRITERIA0", pack=False, align=None), offset=0)}, name="_Anonymous1_e__Struct", pack=False, align=None), "Anonymous2": SimStruct({"outboundEnterpriseStoreArraySize": SimTypeInt(signed=False, label="UInt32"), "outboundEnterpriseStoreCriteria": SimTypePointer(SimStruct({"certData": SimStruct({"size": SimTypeInt(signed=False, label="UInt32"), "data": SimTypePointer(SimTypeChar(label="Byte"), offset=0)}, name="FWP_BYTE_BLOB", pack=False, align=None), "certHash": SimStruct({"size": SimTypeInt(signed=False, label="UInt32"), "data": SimTypePointer(SimTypeChar(label="Byte"), offset=0)}, name="FWP_BYTE_BLOB", pack=False, align=None), "eku": SimTypePointer(SimStruct({"numEku": SimTypeInt(signed=False, label="UInt32"), "eku": SimTypePointer(SimTypePointer(SimTypeChar(label="Byte"), offset=0), offset=0)}, name="IKEEXT_CERT_EKUS0", pack=False, align=None), offset=0), "name": SimTypePointer(SimStruct({"nameType": SimTypeInt(signed=False, label="IKEEXT_CERT_CRITERIA_NAME_TYPE"), "certName": SimTypePointer(SimTypeChar(label="Char"), offset=0)}, name="IKEEXT_CERT_NAME0", pack=False, align=None), offset=0), "flags": SimTypeInt(signed=False, label="UInt32")}, name="IKEEXT_CERTIFICATE_CRITERIA0", pack=False, align=None), offset=0)}, name="_Anonymous2_e__Struct", pack=False, align=None), "Anonymous3": SimStruct({"outboundRootStoreArraySize": SimTypeInt(signed=False, label="UInt32"), "outboundTrustedRootStoreCriteria": SimTypePointer(SimStruct({"certData": SimStruct({"size": SimTypeInt(signed=False, label="UInt32"), "data": SimTypePointer(SimTypeChar(label="Byte"), offset=0)}, name="FWP_BYTE_BLOB", pack=False, align=None), "certHash": SimStruct({"size": SimTypeInt(signed=False, label="UInt32"), "data": SimTypePointer(SimTypeChar(label="Byte"), offset=0)}, name="FWP_BYTE_BLOB", pack=False, align=None), "eku": SimTypePointer(SimStruct({"numEku": SimTypeInt(signed=False, label="UInt32"), "eku": SimTypePointer(SimTypePointer(SimTypeChar(label="Byte"), offset=0), offset=0)}, name="IKEEXT_CERT_EKUS0", pack=False, align=None), offset=0), "name": SimTypePointer(SimStruct({"nameType": SimTypeInt(signed=False, label="IKEEXT_CERT_CRITERIA_NAME_TYPE"), "certName": SimTypePointer(SimTypeChar(label="Char"), offset=0)}, name="IKEEXT_CERT_NAME0", pack=False, align=None), offset=0), "flags": SimTypeInt(signed=False, label="UInt32")}, name="IKEEXT_CERTIFICATE_CRITERIA0", pack=False, align=None), offset=0)}, name="_Anonymous3_e__Struct", pack=False, align=None)}, name="<anon>", label="None"), "flags": SimTypeInt(signed=False, label="IKEEXT_CERT_AUTH"), "localCertLocationUrl": SimStruct({"size": SimTypeInt(signed=False, label="UInt32"), "data": SimTypePointer(SimTypeChar(label="Byte"), offset=0)}, name="FWP_BYTE_BLOB", pack=False, align=None)}, name="IKEEXT_CERTIFICATE_AUTHENTICATION2", pack=False, align=None), "kerberosAuthentication": SimStruct({"flags": SimTypeInt(signed=False, label="IKEEXT_KERBEROS_AUTHENTICATION_FLAGS"), "proxyServer": SimTypePointer(SimTypeChar(label="Char"), offset=0)}, name="IKEEXT_KERBEROS_AUTHENTICATION1", pack=False, align=None), "reservedAuthentication": SimStruct({"flags": SimTypeInt(signed=False, label="IKEEXT_RESERVED_AUTHENTICATION_FLAGS")}, name="IKEEXT_RESERVED_AUTHENTICATION0", pack=False, align=None), "ntlmV2Authentication": SimStruct({"flags": SimTypeInt(signed=False, label="UInt32")}, name="IKEEXT_NTLM_V2_AUTHENTICATION0", pack=False, align=None), "sslAuthentication": SimStruct({"inboundConfigType": SimTypeInt(signed=False, label="IKEEXT_CERT_CONFIG_TYPE"), "Anonymous1": SimUnion({"Anonymous1": SimStruct({"inboundRootArraySize": SimTypeInt(signed=False, label="UInt32"), "inboundRootCriteria": SimTypePointer(SimStruct({"certData": SimStruct({"size": SimTypeInt(signed=False, label="UInt32"), "data": SimTypePointer(SimTypeChar(label="Byte"), offset=0)}, name="FWP_BYTE_BLOB", pack=False, align=None), "certHash": SimStruct({"size": SimTypeInt(signed=False, label="UInt32"), "data": SimTypePointer(SimTypeChar(label="Byte"), offset=0)}, name="FWP_BYTE_BLOB", pack=False, align=None), "eku": SimTypePointer(SimStruct({"numEku": SimTypeInt(signed=False, label="UInt32"), "eku": SimTypePointer(SimTypePointer(SimTypeChar(label="Byte"), offset=0), offset=0)}, name="IKEEXT_CERT_EKUS0", pack=False, align=None), offset=0), "name": SimTypePointer(SimStruct({"nameType": SimTypeInt(signed=False, label="IKEEXT_CERT_CRITERIA_NAME_TYPE"), "certName": SimTypePointer(SimTypeChar(label="Char"), offset=0)}, name="IKEEXT_CERT_NAME0", pack=False, align=None), offset=0), "flags": SimTypeInt(signed=False, label="UInt32")}, name="IKEEXT_CERTIFICATE_CRITERIA0", pack=False, align=None), offset=0)}, name="_Anonymous1_e__Struct", pack=False, align=None), "Anonymous2": SimStruct({"inboundEnterpriseStoreArraySize": SimTypeInt(signed=False, label="UInt32"), "inboundEnterpriseStoreCriteria": SimTypePointer(SimStruct({"certData": SimStruct({"size": SimTypeInt(signed=False, label="UInt32"), "data": SimTypePointer(SimTypeChar(label="Byte"), offset=0)}, name="FWP_BYTE_BLOB", pack=False, align=None), "certHash": SimStruct({"size": SimTypeInt(signed=False, label="UInt32"), "data": SimTypePointer(SimTypeChar(label="Byte"), offset=0)}, name="FWP_BYTE_BLOB", pack=False, align=None), "eku": SimTypePointer(SimStruct({"numEku": SimTypeInt(signed=False, label="UInt32"), "eku": SimTypePointer(SimTypePointer(SimTypeChar(label="Byte"), offset=0), offset=0)}, name="IKEEXT_CERT_EKUS0", pack=False, align=None), offset=0), "name": SimTypePointer(SimStruct({"nameType": SimTypeInt(signed=False, label="IKEEXT_CERT_CRITERIA_NAME_TYPE"), "certName": SimTypePointer(SimTypeChar(label="Char"), offset=0)}, name="IKEEXT_CERT_NAME0", pack=False, align=None), offset=0), "flags": SimTypeInt(signed=False, label="UInt32")}, name="IKEEXT_CERTIFICATE_CRITERIA0", pack=False, align=None), offset=0)}, name="_Anonymous2_e__Struct", pack=False, align=None), "Anonymous3": SimStruct({"inboundRootStoreArraySize": SimTypeInt(signed=False, label="UInt32"), "inboundTrustedRootStoreCriteria": SimTypePointer(SimStruct({"certData": SimStruct({"size": SimTypeInt(signed=False, label="UInt32"), "data": SimTypePointer(SimTypeChar(label="Byte"), offset=0)}, name="FWP_BYTE_BLOB", pack=False, align=None), "certHash": SimStruct({"size": SimTypeInt(signed=False, label="UInt32"), "data": SimTypePointer(SimTypeChar(label="Byte"), offset=0)}, name="FWP_BYTE_BLOB", pack=False, align=None), "eku": SimTypePointer(SimStruct({"numEku": SimTypeInt(signed=False, label="UInt32"), "eku": SimTypePointer(SimTypePointer(SimTypeChar(label="Byte"), offset=0), offset=0)}, name="IKEEXT_CERT_EKUS0", pack=False, align=None), offset=0), "name": SimTypePointer(SimStruct({"nameType": SimTypeInt(signed=False, label="IKEEXT_CERT_CRITERIA_NAME_TYPE"), "certName": SimTypePointer(SimTypeChar(label="Char"), offset=0)}, name="IKEEXT_CERT_NAME0", pack=False, align=None), offset=0), "flags": SimTypeInt(signed=False, label="UInt32")}, name="IKEEXT_CERTIFICATE_CRITERIA0", pack=False, align=None), offset=0)}, name="_Anonymous3_e__Struct", pack=False, align=None)}, name="<anon>", label="None"), "outboundConfigType": SimTypeInt(signed=False, label="IKEEXT_CERT_CONFIG_TYPE"), "Anonymous2": SimUnion({"Anonymous1": SimStruct({"outboundRootArraySize": SimTypeInt(signed=False, label="UInt32"), "outboundRootCriteria": SimTypePointer(SimStruct({"certData": SimStruct({"size": SimTypeInt(signed=False, label="UInt32"), "data": SimTypePointer(SimTypeChar(label="Byte"), offset=0)}, name="FWP_BYTE_BLOB", pack=False, align=None), "certHash": SimStruct({"size": SimTypeInt(signed=False, label="UInt32"), "data": SimTypePointer(SimTypeChar(label="Byte"), offset=0)}, name="FWP_BYTE_BLOB", pack=False, align=None), "eku": SimTypePointer(SimStruct({"numEku": SimTypeInt(signed=False, label="UInt32"), "eku": SimTypePointer(SimTypePointer(SimTypeChar(label="Byte"), offset=0), offset=0)}, name="IKEEXT_CERT_EKUS0", pack=False, align=None), offset=0), "name": SimTypePointer(SimStruct({"nameType": SimTypeInt(signed=False, label="IKEEXT_CERT_CRITERIA_NAME_TYPE"), "certName": SimTypePointer(SimTypeChar(label="Char"), offset=0)}, name="IKEEXT_CERT_NAME0", pack=False, align=None), offset=0), "flags": SimTypeInt(signed=False, label="UInt32")}, name="IKEEXT_CERTIFICATE_CRITERIA0", pack=False, align=None), offset=0)}, name="_Anonymous1_e__Struct", pack=False, align=None), "Anonymous2": SimStruct({"outboundEnterpriseStoreArraySize": SimTypeInt(signed=False, label="UInt32"), "outboundEnterpriseStoreCriteria": SimTypePointer(SimStruct({"certData": SimStruct({"size": SimTypeInt(signed=False, label="UInt32"), "data": SimTypePointer(SimTypeChar(label="Byte"), offset=0)}, name="FWP_BYTE_BLOB", pack=False, align=None), "certHash": SimStruct({"size": SimTypeInt(signed=False, label="UInt32"), "data": SimTypePointer(SimTypeChar(label="Byte"), offset=0)}, name="FWP_BYTE_BLOB", pack=False, align=None), "eku": SimTypePointer(SimStruct({"numEku": SimTypeInt(signed=False, label="UInt32"), "eku": SimTypePointer(SimTypePointer(SimTypeChar(label="Byte"), offset=0), offset=0)}, name="IKEEXT_CERT_EKUS0", pack=False, align=None), offset=0), "name": SimTypePointer(SimStruct({"nameType": SimTypeInt(signed=False, label="IKEEXT_CERT_CRITERIA_NAME_TYPE"), "certName": SimTypePointer(SimTypeChar(label="Char"), offset=0)}, name="IKEEXT_CERT_NAME0", pack=False, align=None), offset=0), "flags": SimTypeInt(signed=False, label="UInt32")}, name="IKEEXT_CERTIFICATE_CRITERIA0", pack=False, align=None), offset=0)}, name="_Anonymous2_e__Struct", pack=False, align=None), "Anonymous3": SimStruct({"outboundRootStoreArraySize": SimTypeInt(signed=False, label="UInt32"), "outboundTrustedRootStoreCriteria": SimTypePointer(SimStruct({"certData": SimStruct({"size": SimTypeInt(signed=False, label="UInt32"), "data": SimTypePointer(SimTypeChar(label="Byte"), offset=0)}, name="FWP_BYTE_BLOB", pack=False, align=None), "certHash": SimStruct({"size": SimTypeInt(signed=False, label="UInt32"), "data": SimTypePointer(SimTypeChar(label="Byte"), offset=0)}, name="FWP_BYTE_BLOB", pack=False, align=None), "eku": SimTypePointer(SimStruct({"numEku": SimTypeInt(signed=False, label="UInt32"), "eku": SimTypePointer(SimTypePointer(SimTypeChar(label="Byte"), offset=0), offset=0)}, name="IKEEXT_CERT_EKUS0", pack=False, align=None), offset=0), "name": SimTypePointer(SimStruct({"nameType": SimTypeInt(signed=False, label="IKEEXT_CERT_CRITERIA_NAME_TYPE"), "certName": SimTypePointer(SimTypeChar(label="Char"), offset=0)}, name="IKEEXT_CERT_NAME0", pack=False, align=None), offset=0), "flags": SimTypeInt(signed=False, label="UInt32")}, name="IKEEXT_CERTIFICATE_CRITERIA0", pack=False, align=None), offset=0)}, name="_Anonymous3_e__Struct", pack=False, align=None)}, name="<anon>", label="None"), "flags": SimTypeInt(signed=False, label="IKEEXT_CERT_AUTH"), "localCertLocationUrl": SimStruct({"size": SimTypeInt(signed=False, label="UInt32"), "data": SimTypePointer(SimTypeChar(label="Byte"), offset=0)}, name="FWP_BYTE_BLOB", pack=False, align=None)}, name="IKEEXT_CERTIFICATE_AUTHENTICATION2", pack=False, align=None), "cgaAuthentication": SimStruct({"keyContainerName": SimTypePointer(SimTypeChar(label="Char"), offset=0), "cspName": SimTypePointer(SimTypeChar(label="Char"), offset=0), "cspType": SimTypeInt(signed=False, label="UInt32"), "cgaModifier": SimStruct({"byteArray16": SimTypeFixedSizeArray(SimTypeChar(label="Byte"), 16)}, name="FWP_BYTE_ARRAY16", pack=False, align=None), "cgaCollisionCount": SimTypeChar(label="Byte")}, name="IKEEXT_IPV6_CGA_AUTHENTICATION0", pack=False, align=None), "eapAuthentication": SimStruct({"flags": SimTypeInt(signed=False, label="IKEEXT_EAP_AUTHENTICATION_FLAGS")}, name="IKEEXT_EAP_AUTHENTICATION0", pack=False, align=None)}, name="<anon>", label="None")}, name="IKEEXT_AUTHENTICATION_METHOD2", pack=False, align=None), offset=0), "initiatorImpersonationType": SimTypeInt(signed=False, label="IKEEXT_AUTHENTICATION_IMPERSONATION_TYPE")}, name="IKEEXT_EM_POLICY2", pack=False, align=None), offset=0)}, name="IPSEC_TRANSPORT_POLICY2", pack=False, align=None), offset=0), "ikeV2MmPolicy": SimTypePointer(SimStruct({"softExpirationTime": SimTypeInt(signed=False, label="UInt32"), "numAuthenticationMethods": SimTypeInt(signed=False, label="UInt32"), "authenticationMethods": SimTypePointer(SimStruct({"authenticationMethodType": SimTypeInt(signed=False, label="IKEEXT_AUTHENTICATION_METHOD_TYPE"), "Anonymous": SimUnion({"presharedKeyAuthentication": SimStruct({"presharedKey": SimStruct({"size": SimTypeInt(signed=False, label="UInt32"), "data": SimTypePointer(SimTypeChar(label="Byte"), offset=0)}, name="FWP_BYTE_BLOB", pack=False, align=None), "flags": SimTypeInt(signed=False, label="IKEEXT_PRESHARED_KEY_AUTHENTICATION_FLAGS")}, name="IKEEXT_PRESHARED_KEY_AUTHENTICATION1", pack=False, align=None), "certificateAuthentication": SimStruct({"inboundConfigType": SimTypeInt(signed=False, label="IKEEXT_CERT_CONFIG_TYPE"), "Anonymous1": SimUnion({"Anonymous1": SimStruct({"inboundRootArraySize": SimTypeInt(signed=False, label="UInt32"), "inboundRootCriteria": SimTypePointer(SimStruct({"certData": SimStruct({"size": SimTypeInt(signed=False, label="UInt32"), "data": SimTypePointer(SimTypeChar(label="Byte"), offset=0)}, name="FWP_BYTE_BLOB", pack=False, align=None), "certHash": SimStruct({"size": SimTypeInt(signed=False, label="UInt32"), "data": SimTypePointer(SimTypeChar(label="Byte"), offset=0)}, name="FWP_BYTE_BLOB", pack=False, align=None), "eku": SimTypePointer(SimStruct({"numEku": SimTypeInt(signed=False, label="UInt32"), "eku": SimTypePointer(SimTypePointer(SimTypeChar(label="Byte"), offset=0), offset=0)}, name="IKEEXT_CERT_EKUS0", pack=False, align=None), offset=0), "name": SimTypePointer(SimStruct({"nameType": SimTypeInt(signed=False, label="IKEEXT_CERT_CRITERIA_NAME_TYPE"), "certName": SimTypePointer(SimTypeChar(label="Char"), offset=0)}, name="IKEEXT_CERT_NAME0", pack=False, align=None), offset=0), "flags": SimTypeInt(signed=False, label="UInt32")}, name="IKEEXT_CERTIFICATE_CRITERIA0", pack=False, align=None), offset=0)}, name="_Anonymous1_e__Struct", pack=False, align=None), "Anonymous2": SimStruct({"inboundEnterpriseStoreArraySize": SimTypeInt(signed=False, label="UInt32"), "inboundEnterpriseStoreCriteria": SimTypePointer(SimStruct({"certData": SimStruct({"size": SimTypeInt(signed=False, label="UInt32"), "data": SimTypePointer(SimTypeChar(label="Byte"), offset=0)}, name="FWP_BYTE_BLOB", pack=False, align=None), "certHash": SimStruct({"size": SimTypeInt(signed=False, label="UInt32"), "data": SimTypePointer(SimTypeChar(label="Byte"), offset=0)}, name="FWP_BYTE_BLOB", pack=False, align=None), "eku": SimTypePointer(SimStruct({"numEku": SimTypeInt(signed=False, label="UInt32"), "eku": SimTypePointer(SimTypePointer(SimTypeChar(label="Byte"), offset=0), offset=0)}, name="IKEEXT_CERT_EKUS0", pack=False, align=None), offset=0), "name": SimTypePointer(SimStruct({"nameType": SimTypeInt(signed=False, label="IKEEXT_CERT_CRITERIA_NAME_TYPE"), "certName": SimTypePointer(SimTypeChar(label="Char"), offset=0)}, name="IKEEXT_CERT_NAME0", pack=False, align=None), offset=0), "flags": SimTypeInt(signed=False, label="UInt32")}, name="IKEEXT_CERTIFICATE_CRITERIA0", pack=False, align=None), offset=0)}, name="_Anonymous2_e__Struct", pack=False, align=None), "Anonymous3": SimStruct({"inboundRootStoreArraySize": SimTypeInt(signed=False, label="UInt32"), "inboundTrustedRootStoreCriteria": SimTypePointer(SimStruct({"certData": SimStruct({"size": SimTypeInt(signed=False, label="UInt32"), "data": SimTypePointer(SimTypeChar(label="Byte"), offset=0)}, name="FWP_BYTE_BLOB", pack=False, align=None), "certHash": SimStruct({"size": SimTypeInt(signed=False, label="UInt32"), "data": SimTypePointer(SimTypeChar(label="Byte"), offset=0)}, name="FWP_BYTE_BLOB", pack=False, align=None), "eku": SimTypePointer(SimStruct({"numEku": SimTypeInt(signed=False, label="UInt32"), "eku": SimTypePointer(SimTypePointer(SimTypeChar(label="Byte"), offset=0), offset=0)}, name="IKEEXT_CERT_EKUS0", pack=False, align=None), offset=0), "name": SimTypePointer(SimStruct({"nameType": SimTypeInt(signed=False, label="IKEEXT_CERT_CRITERIA_NAME_TYPE"), "certName": SimTypePointer(SimTypeChar(label="Char"), offset=0)}, name="IKEEXT_CERT_NAME0", pack=False, align=None), offset=0), "flags": SimTypeInt(signed=False, label="UInt32")}, name="IKEEXT_CERTIFICATE_CRITERIA0", pack=False, align=None), offset=0)}, name="_Anonymous3_e__Struct", pack=False, align=None)}, name="<anon>", label="None"), "outboundConfigType": SimTypeInt(signed=False, label="IKEEXT_CERT_CONFIG_TYPE"), "Anonymous2": SimUnion({"Anonymous1": SimStruct({"outboundRootArraySize": SimTypeInt(signed=False, label="UInt32"), "outboundRootCriteria": SimTypePointer(SimStruct({"certData": SimStruct({"size": SimTypeInt(signed=False, label="UInt32"), "data": SimTypePointer(SimTypeChar(label="Byte"), offset=0)}, name="FWP_BYTE_BLOB", pack=False, align=None), "certHash": SimStruct({"size": SimTypeInt(signed=False, label="UInt32"), "data": SimTypePointer(SimTypeChar(label="Byte"), offset=0)}, name="FWP_BYTE_BLOB", pack=False, align=None), "eku": SimTypePointer(SimStruct({"numEku": SimTypeInt(signed=False, label="UInt32"), "eku": SimTypePointer(SimTypePointer(SimTypeChar(label="Byte"), offset=0), offset=0)}, name="IKEEXT_CERT_EKUS0", pack=False, align=None), offset=0), "name": SimTypePointer(SimStruct({"nameType": SimTypeInt(signed=False, label="IKEEXT_CERT_CRITERIA_NAME_TYPE"), "certName": SimTypePointer(SimTypeChar(label="Char"), offset=0)}, name="IKEEXT_CERT_NAME0", pack=False, align=None), offset=0), "flags": SimTypeInt(signed=False, label="UInt32")}, name="IKEEXT_CERTIFICATE_CRITERIA0", pack=False, align=None), offset=0)}, name="_Anonymous1_e__Struct", pack=False, align=None), "Anonymous2": SimStruct({"outboundEnterpriseStoreArraySize": SimTypeInt(signed=False, label="UInt32"), "outboundEnterpriseStoreCriteria": SimTypePointer(SimStruct({"certData": SimStruct({"size": SimTypeInt(signed=False, label="UInt32"), "data": SimTypePointer(SimTypeChar(label="Byte"), offset=0)}, name="FWP_BYTE_BLOB", pack=False, align=None), "certHash": SimStruct({"size": SimTypeInt(signed=False, label="UInt32"), "data": SimTypePointer(SimTypeChar(label="Byte"), offset=0)}, name="FWP_BYTE_BLOB", pack=False, align=None), "eku": SimTypePointer(SimStruct({"numEku": SimTypeInt(signed=False, label="UInt32"), "eku": SimTypePointer(SimTypePointer(SimTypeChar(label="Byte"), offset=0), offset=0)}, name="IKEEXT_CERT_EKUS0", pack=False, align=None), offset=0), "name": SimTypePointer(SimStruct({"nameType": SimTypeInt(signed=False, label="IKEEXT_CERT_CRITERIA_NAME_TYPE"), "certName": SimTypePointer(SimTypeChar(label="Char"), offset=0)}, name="IKEEXT_CERT_NAME0", pack=False, align=None), offset=0), "flags": SimTypeInt(signed=False, label="UInt32")}, name="IKEEXT_CERTIFICATE_CRITERIA0", pack=False, align=None), offset=0)}, name="_Anonymous2_e__Struct", pack=False, align=None), "Anonymous3": SimStruct({"outboundRootStoreArraySize": SimTypeInt(signed=False, label="UInt32"), "outboundTrustedRootStoreCriteria": SimTypePointer(SimStruct({"certData": SimStruct({"size": SimTypeInt(signed=False, label="UInt32"), "data": SimTypePointer(SimTypeChar(label="Byte"), offset=0)}, name="FWP_BYTE_BLOB", pack=False, align=None), "certHash": SimStruct({"size": SimTypeInt(signed=False, label="UInt32"), "data": SimTypePointer(SimTypeChar(label="Byte"), offset=0)}, name="FWP_BYTE_BLOB", pack=False, align=None), "eku": SimTypePointer(SimStruct({"numEku": SimTypeInt(signed=False, label="UInt32"), "eku": SimTypePointer(SimTypePointer(SimTypeChar(label="Byte"), offset=0), offset=0)}, name="IKEEXT_CERT_EKUS0", pack=False, align=None), offset=0), "name": SimTypePointer(SimStruct({"nameType": SimTypeInt(signed=False, label="IKEEXT_CERT_CRITERIA_NAME_TYPE"), "certName": SimTypePointer(SimTypeChar(label="Char"), offset=0)}, name="IKEEXT_CERT_NAME0", pack=False, align=None), offset=0), "flags": SimTypeInt(signed=False, label="UInt32")}, name="IKEEXT_CERTIFICATE_CRITERIA0", pack=False, align=None), offset=0)}, name="_Anonymous3_e__Struct", pack=False, align=None)}, name="<anon>", label="None"), "flags": SimTypeInt(signed=False, label="IKEEXT_CERT_AUTH"), "localCertLocationUrl": SimStruct({"size": SimTypeInt(signed=False, label="UInt32"), "data": SimTypePointer(SimTypeChar(label="Byte"), offset=0)}, name="FWP_BYTE_BLOB", pack=False, align=None)}, name="IKEEXT_CERTIFICATE_AUTHENTICATION2", pack=False, align=None), "kerberosAuthentication": SimStruct({"flags": SimTypeInt(signed=False, label="IKEEXT_KERBEROS_AUTHENTICATION_FLAGS"), "proxyServer": SimTypePointer(SimTypeChar(label="Char"), offset=0)}, name="IKEEXT_KERBEROS_AUTHENTICATION1", pack=False, align=None), "reservedAuthentication": SimStruct({"flags": SimTypeInt(signed=False, label="IKEEXT_RESERVED_AUTHENTICATION_FLAGS")}, name="IKEEXT_RESERVED_AUTHENTICATION0", pack=False, align=None), "ntlmV2Authentication": SimStruct({"flags": SimTypeInt(signed=False, label="UInt32")}, name="IKEEXT_NTLM_V2_AUTHENTICATION0", pack=False, align=None), "sslAuthentication": SimStruct({"inboundConfigType": SimTypeInt(signed=False, label="IKEEXT_CERT_CONFIG_TYPE"), "Anonymous1": SimUnion({"Anonymous1": SimStruct({"inboundRootArraySize": SimTypeInt(signed=False, label="UInt32"), "inboundRootCriteria": SimTypePointer(SimStruct({"certData": SimStruct({"size": SimTypeInt(signed=False, label="UInt32"), "data": SimTypePointer(SimTypeChar(label="Byte"), offset=0)}, name="FWP_BYTE_BLOB", pack=False, align=None), "certHash": SimStruct({"size": SimTypeInt(signed=False, label="UInt32"), "data": SimTypePointer(SimTypeChar(label="Byte"), offset=0)}, name="FWP_BYTE_BLOB", pack=False, align=None), "eku": SimTypePointer(SimStruct({"numEku": SimTypeInt(signed=False, label="UInt32"), "eku": SimTypePointer(SimTypePointer(SimTypeChar(label="Byte"), offset=0), offset=0)}, name="IKEEXT_CERT_EKUS0", pack=False, align=None), offset=0), "name": SimTypePointer(SimStruct({"nameType": SimTypeInt(signed=False, label="IKEEXT_CERT_CRITERIA_NAME_TYPE"), "certName": SimTypePointer(SimTypeChar(label="Char"), offset=0)}, name="IKEEXT_CERT_NAME0", pack=False, align=None), offset=0), "flags": SimTypeInt(signed=False, label="UInt32")}, name="IKEEXT_CERTIFICATE_CRITERIA0", pack=False, align=None), offset=0)}, name="_Anonymous1_e__Struct", pack=False, align=None), "Anonymous2": SimStruct({"inboundEnterpriseStoreArraySize": SimTypeInt(signed=False, label="UInt32"), "inboundEnterpriseStoreCriteria": SimTypePointer(SimStruct({"certData": SimStruct({"size": SimTypeInt(signed=False, label="UInt32"), "data": SimTypePointer(SimTypeChar(label="Byte"), offset=0)}, name="FWP_BYTE_BLOB", pack=False, align=None), "certHash": SimStruct({"size": SimTypeInt(signed=False, label="UInt32"), "data": SimTypePointer(SimTypeChar(label="Byte"), offset=0)}, name="FWP_BYTE_BLOB", pack=False, align=None), "eku": SimTypePointer(SimStruct({"numEku": SimTypeInt(signed=False, label="UInt32"), "eku": SimTypePointer(SimTypePointer(SimTypeChar(label="Byte"), offset=0), offset=0)}, name="IKEEXT_CERT_EKUS0", pack=False, align=None), offset=0), "name": SimTypePointer(SimStruct({"nameType": SimTypeInt(signed=False, label="IKEEXT_CERT_CRITERIA_NAME_TYPE"), "certName": SimTypePointer(SimTypeChar(label="Char"), offset=0)}, name="IKEEXT_CERT_NAME0", pack=False, align=None), offset=0), "flags": SimTypeInt(signed=False, label="UInt32")}, name="IKEEXT_CERTIFICATE_CRITERIA0", pack=False, align=None), offset=0)}, name="_Anonymous2_e__Struct", pack=False, align=None), "Anonymous3": SimStruct({"inboundRootStoreArraySize": SimTypeInt(signed=False, label="UInt32"), "inboundTrustedRootStoreCriteria": SimTypePointer(SimStruct({"certData": SimStruct({"size": SimTypeInt(signed=False, label="UInt32"), "data": SimTypePointer(SimTypeChar(label="Byte"), offset=0)}, name="FWP_BYTE_BLOB", pack=False, align=None), "certHash": SimStruct({"size": SimTypeInt(signed=False, label="UInt32"), "data": SimTypePointer(SimTypeChar(label="Byte"), offset=0)}, name="FWP_BYTE_BLOB", pack=False, align=None), "eku": SimTypePointer(SimStruct({"numEku": SimTypeInt(signed=False, label="UInt32"), "eku": SimTypePointer(SimTypePointer(SimTypeChar(label="Byte"), offset=0), offset=0)}, name="IKEEXT_CERT_EKUS0", pack=False, align=None), offset=0), "name": SimTypePointer(SimStruct({"nameType": SimTypeInt(signed=False, label="IKEEXT_CERT_CRITERIA_NAME_TYPE"), "certName": SimTypePointer(SimTypeChar(label="Char"), offset=0)}, name="IKEEXT_CERT_NAME0", pack=False, align=None), offset=0), "flags": SimTypeInt(signed=False, label="UInt32")}, name="IKEEXT_CERTIFICATE_CRITERIA0", pack=False, align=None), offset=0)}, name="_Anonymous3_e__Struct", pack=False, align=None)}, name="<anon>", label="None"), "outboundConfigType": SimTypeInt(signed=False, label="IKEEXT_CERT_CONFIG_TYPE"), "Anonymous2": SimUnion({"Anonymous1": SimStruct({"outboundRootArraySize": SimTypeInt(signed=False, label="UInt32"), "outboundRootCriteria": SimTypePointer(SimStruct({"certData": SimStruct({"size": SimTypeInt(signed=False, label="UInt32"), "data": SimTypePointer(SimTypeChar(label="Byte"), offset=0)}, name="FWP_BYTE_BLOB", pack=False, align=None), "certHash": SimStruct({"size": SimTypeInt(signed=False, label="UInt32"), "data": SimTypePointer(SimTypeChar(label="Byte"), offset=0)}, name="FWP_BYTE_BLOB", pack=False, align=None), "eku": SimTypePointer(SimStruct({"numEku": SimTypeInt(signed=False, label="UInt32"), "eku": SimTypePointer(SimTypePointer(SimTypeChar(label="Byte"), offset=0), offset=0)}, name="IKEEXT_CERT_EKUS0", pack=False, align=None), offset=0), "name": SimTypePointer(SimStruct({"nameType": SimTypeInt(signed=False, label="IKEEXT_CERT_CRITERIA_NAME_TYPE"), "certName": SimTypePointer(SimTypeChar(label="Char"), offset=0)}, name="IKEEXT_CERT_NAME0", pack=False, align=None), offset=0), "flags": SimTypeInt(signed=False, label="UInt32")}, name="IKEEXT_CERTIFICATE_CRITERIA0", pack=False, align=None), offset=0)}, name="_Anonymous1_e__Struct", pack=False, align=None), "Anonymous2": SimStruct({"outboundEnterpriseStoreArraySize": SimTypeInt(signed=False, label="UInt32"), "outboundEnterpriseStoreCriteria": SimTypePointer(SimStruct({"certData": SimStruct({"size": SimTypeInt(signed=False, label="UInt32"), "data": SimTypePointer(SimTypeChar(label="Byte"), offset=0)}, name="FWP_BYTE_BLOB", pack=False, align=None), "certHash": SimStruct({"size": SimTypeInt(signed=False, label="UInt32"), "data": SimTypePointer(SimTypeChar(label="Byte"), offset=0)}, name="FWP_BYTE_BLOB", pack=False, align=None), "eku": SimTypePointer(SimStruct({"numEku": SimTypeInt(signed=False, label="UInt32"), "eku": SimTypePointer(SimTypePointer(SimTypeChar(label="Byte"), offset=0), offset=0)}, name="IKEEXT_CERT_EKUS0", pack=False, align=None), offset=0), "name": SimTypePointer(SimStruct({"nameType": SimTypeInt(signed=False, label="IKEEXT_CERT_CRITERIA_NAME_TYPE"), "certName": SimTypePointer(SimTypeChar(label="Char"), offset=0)}, name="IKEEXT_CERT_NAME0", pack=False, align=None), offset=0), "flags": SimTypeInt(signed=False, label="UInt32")}, name="IKEEXT_CERTIFICATE_CRITERIA0", pack=False, align=None), offset=0)}, name="_Anonymous2_e__Struct", pack=False, align=None), "Anonymous3": SimStruct({"outboundRootStoreArraySize": SimTypeInt(signed=False, label="UInt32"), "outboundTrustedRootStoreCriteria": SimTypePointer(SimStruct({"certData": SimStruct({"size": SimTypeInt(signed=False, label="UInt32"), "data": SimTypePointer(SimTypeChar(label="Byte"), offset=0)}, name="FWP_BYTE_BLOB", pack=False, align=None), "certHash": SimStruct({"size": SimTypeInt(signed=False, label="UInt32"), "data": SimTypePointer(SimTypeChar(label="Byte"), offset=0)}, name="FWP_BYTE_BLOB", pack=False, align=None), "eku": SimTypePointer(SimStruct({"numEku": SimTypeInt(signed=False, label="UInt32"), "eku": SimTypePointer(SimTypePointer(SimTypeChar(label="Byte"), offset=0), offset=0)}, name="IKEEXT_CERT_EKUS0", pack=False, align=None), offset=0), "name": SimTypePointer(SimStruct({"nameType": SimTypeInt(signed=False, label="IKEEXT_CERT_CRITERIA_NAME_TYPE"), "certName": SimTypePointer(SimTypeChar(label="Char"), offset=0)}, name="IKEEXT_CERT_NAME0", pack=False, align=None), offset=0), "flags": SimTypeInt(signed=False, label="UInt32")}, name="IKEEXT_CERTIFICATE_CRITERIA0", pack=False, align=None), offset=0)}, name="_Anonymous3_e__Struct", pack=False, align=None)}, name="<anon>", label="None"), "flags": SimTypeInt(signed=False, label="IKEEXT_CERT_AUTH"), "localCertLocationUrl": SimStruct({"size": SimTypeInt(signed=False, label="UInt32"), "data": SimTypePointer(SimTypeChar(label="Byte"), offset=0)}, name="FWP_BYTE_BLOB", pack=False, align=None)}, name="IKEEXT_CERTIFICATE_AUTHENTICATION2", pack=False, align=None), "cgaAuthentication": SimStruct({"keyContainerName": SimTypePointer(SimTypeChar(label="Char"), offset=0), "cspName": SimTypePointer(SimTypeChar(label="Char"), offset=0), "cspType": SimTypeInt(signed=False, label="UInt32"), "cgaModifier": SimStruct({"byteArray16": SimTypeFixedSizeArray(SimTypeChar(label="Byte"), 16)}, name="FWP_BYTE_ARRAY16", pack=False, align=None), "cgaCollisionCount": SimTypeChar(label="Byte")}, name="IKEEXT_IPV6_CGA_AUTHENTICATION0", pack=False, align=None), "eapAuthentication": SimStruct({"flags": SimTypeInt(signed=False, label="IKEEXT_EAP_AUTHENTICATION_FLAGS")}, name="IKEEXT_EAP_AUTHENTICATION0", pack=False, align=None)}, name="<anon>", label="None")}, name="IKEEXT_AUTHENTICATION_METHOD2", pack=False, align=None), offset=0), "initiatorImpersonationType": SimTypeInt(signed=False, label="IKEEXT_AUTHENTICATION_IMPERSONATION_TYPE"), "numIkeProposals": SimTypeInt(signed=False, label="UInt32"), "ikeProposals": SimTypePointer(SimStruct({"cipherAlgorithm": SimStruct({"algoIdentifier": SimTypeInt(signed=False, label="IKEEXT_CIPHER_TYPE"), "keyLen": SimTypeInt(signed=False, label="UInt32"), "rounds": SimTypeInt(signed=False, label="UInt32")}, name="IKEEXT_CIPHER_ALGORITHM0", pack=False, align=None), "integrityAlgorithm": SimStruct({"algoIdentifier": SimTypeInt(signed=False, label="IKEEXT_INTEGRITY_TYPE")}, name="IKEEXT_INTEGRITY_ALGORITHM0", pack=False, align=None), "maxLifetimeSeconds": SimTypeInt(signed=False, label="UInt32"), "dhGroup": SimTypeInt(signed=False, label="IKEEXT_DH_GROUP"), "quickModeLimit": SimTypeInt(signed=False, label="UInt32")}, name="IKEEXT_PROPOSAL0", pack=False, align=None), offset=0), "flags": SimTypeInt(signed=False, label="IKEEXT_POLICY_FLAG"), "maxDynamicFilters": SimTypeInt(signed=False, label="UInt32"), "retransmitDurationSecs": SimTypeInt(signed=False, label="UInt32")}, name="IKEEXT_POLICY2", pack=False, align=None), offset=0), "idpOptions": SimTypePointer(SimStruct({"stateIdleTimeoutSeconds": SimTypeInt(signed=False, label="UInt32"), "perIPRateLimitQueueIdleTimeoutSeconds": SimTypeInt(signed=False, label="UInt32"), "ipV6IPsecUnauthDscp": SimTypeChar(label="Byte"), "ipV6IPsecUnauthRateLimitBytesPerSec": SimTypeInt(signed=False, label="UInt32"), "ipV6IPsecUnauthPerIPRateLimitBytesPerSec": SimTypeInt(signed=False, label="UInt32"), "ipV6IPsecAuthDscp": SimTypeChar(label="Byte"), "ipV6IPsecAuthRateLimitBytesPerSec": SimTypeInt(signed=False, label="UInt32"), "icmpV6Dscp": SimTypeChar(label="Byte"), "icmpV6RateLimitBytesPerSec": SimTypeInt(signed=False, label="UInt32"), "ipV6FilterExemptDscp": SimTypeChar(label="Byte"), "ipV6FilterExemptRateLimitBytesPerSec": SimTypeInt(signed=False, label="UInt32"), "defBlockExemptDscp": SimTypeChar(label="Byte"), "defBlockExemptRateLimitBytesPerSec": SimTypeInt(signed=False, label="UInt32"), "maxStateEntries": SimTypeInt(signed=False, label="UInt32"), "maxPerIPRateLimitQueues": SimTypeInt(signed=False, label="UInt32"), "flags": SimTypeInt(signed=False, label="IPSEC_DOSP_FLAGS"), "numPublicIFLuids": SimTypeInt(signed=False, label="UInt32"), "publicIFLuids": SimTypePointer(SimTypeLongLong(signed=False, label="UInt64"), offset=0), "numInternalIFLuids": SimTypeInt(signed=False, label="UInt32"), "internalIFLuids": SimTypePointer(SimTypeLongLong(signed=False, label="UInt64"), offset=0), "publicV6AddrMask": SimStruct({"addr": SimTypeFixedSizeArray(SimTypeChar(label="Byte"), 16), "prefixLength": SimTypeChar(label="Byte")}, name="FWP_V6_ADDR_AND_MASK", pack=False, align=None), "internalV6AddrMask": SimStruct({"addr": SimTypeFixedSizeArray(SimTypeChar(label="Byte"), 16), "prefixLength": SimTypeChar(label="Byte")}, name="FWP_V6_ADDR_AND_MASK", pack=False, align=None)}, name="IPSEC_DOSP_OPTIONS0", pack=False, align=None), offset=0)}, name="<anon>", label="None"), "providerContextId": SimTypeLongLong(signed=False, label="UInt64")}, name="FWPM_PROVIDER_CONTEXT2", pack=False, align=None), offset=0), SimTypePointer(SimStruct({"Revision": SimTypeChar(label="Byte"), "Sbz1": SimTypeChar(label="Byte"), "Control": SimTypeShort(signed=False, label="UInt16"), "Owner": SimTypePointer(SimTypeInt(signed=True, label="Int"), label="IntPtr", offset=0), "Group": SimTypePointer(SimTypeInt(signed=True, label="Int"), label="IntPtr", offset=0), "Sacl": SimTypePointer(SimStruct({"AclRevision": SimTypeChar(label="Byte"), "Sbz1": SimTypeChar(label="Byte"), "AclSize": SimTypeShort(signed=False, label="UInt16"), "AceCount": SimTypeShort(signed=False, label="UInt16"), "Sbz2": SimTypeShort(signed=False, label="UInt16")}, name="ACL", pack=False, align=None), offset=0), "Dacl": SimTypePointer(SimStruct({"AclRevision": SimTypeChar(label="Byte"), "Sbz1": SimTypeChar(label="Byte"), "AclSize": SimTypeShort(signed=False, label="UInt16"), "AceCount": SimTypeShort(signed=False, label="UInt16"), "Sbz2": SimTypeShort(signed=False, label="UInt16")}, name="ACL", pack=False, align=None), offset=0)}, name="SECURITY_DESCRIPTOR", pack=False, align=None), offset=0), SimTypePointer(SimTypeLongLong(signed=False, label="UInt64"), offset=0)], SimTypeInt(signed=False, label="UInt32"), arg_names=["engineHandle", "providerContext", "sd", "id"]),
        # 
        'FwpmProviderContextAdd3': SimTypeFunction([SimTypePointer(SimTypeInt(signed=True, label="Int"), label="IntPtr", offset=0), SimTypePointer(SimStruct({"providerContextKey": SimTypeBottom(label="Guid"), "displayData": SimStruct({"name": SimTypePointer(SimTypeChar(label="Char"), offset=0), "description": SimTypePointer(SimTypeChar(label="Char"), offset=0)}, name="FWPM_DISPLAY_DATA0", pack=False, align=None), "flags": SimTypeInt(signed=False, label="UInt32"), "providerKey": SimTypePointer(SimTypeBottom(label="Guid"), offset=0), "providerData": SimStruct({"size": SimTypeInt(signed=False, label="UInt32"), "data": SimTypePointer(SimTypeChar(label="Byte"), offset=0)}, name="FWP_BYTE_BLOB", pack=False, align=None), "type": SimTypeInt(signed=False, label="FWPM_PROVIDER_CONTEXT_TYPE"), "Anonymous": SimUnion({"keyingPolicy": SimTypePointer(SimStruct({"numKeyMods": SimTypeInt(signed=False, label="UInt32"), "keyModKeys": SimTypePointer(SimTypeBottom(label="Guid"), offset=0), "flags": SimTypeInt(signed=False, label="UInt32")}, name="IPSEC_KEYING_POLICY1", pack=False, align=None), offset=0), "ikeQmTransportPolicy": SimTypePointer(SimStruct({"numIpsecProposals": SimTypeInt(signed=False, label="UInt32"), "ipsecProposals": SimTypePointer(SimStruct({"lifetime": SimStruct({"lifetimeSeconds": SimTypeInt(signed=False, label="UInt32"), "lifetimeKilobytes": SimTypeInt(signed=False, label="UInt32"), "lifetimePackets": SimTypeInt(signed=False, label="UInt32")}, name="IPSEC_SA_LIFETIME0", pack=False, align=None), "numSaTransforms": SimTypeInt(signed=False, label="UInt32"), "saTransforms": SimTypePointer(SimStruct({"ipsecTransformType": SimTypeInt(signed=False, label="IPSEC_TRANSFORM_TYPE"), "Anonymous": SimUnion({"ahTransform": SimTypePointer(SimStruct({"authTransformId": SimStruct({"authType": SimTypeInt(signed=False, label="IPSEC_AUTH_TYPE"), "authConfig": SimTypeChar(label="Byte")}, name="IPSEC_AUTH_TRANSFORM_ID0", pack=False, align=None), "cryptoModuleId": SimTypePointer(SimTypeBottom(label="Guid"), offset=0)}, name="IPSEC_AUTH_TRANSFORM0", pack=False, align=None), offset=0), "espAuthTransform": SimTypePointer(SimStruct({"authTransformId": SimStruct({"authType": SimTypeInt(signed=False, label="IPSEC_AUTH_TYPE"), "authConfig": SimTypeChar(label="Byte")}, name="IPSEC_AUTH_TRANSFORM_ID0", pack=False, align=None), "cryptoModuleId": SimTypePointer(SimTypeBottom(label="Guid"), offset=0)}, name="IPSEC_AUTH_TRANSFORM0", pack=False, align=None), offset=0), "espCipherTransform": SimTypePointer(SimStruct({"cipherTransformId": SimStruct({"cipherType": SimTypeInt(signed=False, label="IPSEC_CIPHER_TYPE"), "cipherConfig": SimTypeChar(label="Byte")}, name="IPSEC_CIPHER_TRANSFORM_ID0", pack=False, align=None), "cryptoModuleId": SimTypePointer(SimTypeBottom(label="Guid"), offset=0)}, name="IPSEC_CIPHER_TRANSFORM0", pack=False, align=None), offset=0), "espAuthAndCipherTransform": SimTypePointer(SimStruct({"authTransform": SimStruct({"authTransformId": SimStruct({"authType": SimTypeInt(signed=False, label="IPSEC_AUTH_TYPE"), "authConfig": SimTypeChar(label="Byte")}, name="IPSEC_AUTH_TRANSFORM_ID0", pack=False, align=None), "cryptoModuleId": SimTypePointer(SimTypeBottom(label="Guid"), offset=0)}, name="IPSEC_AUTH_TRANSFORM0", pack=False, align=None), "cipherTransform": SimStruct({"cipherTransformId": SimStruct({"cipherType": SimTypeInt(signed=False, label="IPSEC_CIPHER_TYPE"), "cipherConfig": SimTypeChar(label="Byte")}, name="IPSEC_CIPHER_TRANSFORM_ID0", pack=False, align=None), "cryptoModuleId": SimTypePointer(SimTypeBottom(label="Guid"), offset=0)}, name="IPSEC_CIPHER_TRANSFORM0", pack=False, align=None)}, name="IPSEC_AUTH_AND_CIPHER_TRANSFORM0", pack=False, align=None), offset=0), "espAuthFwTransform": SimTypePointer(SimStruct({"authTransformId": SimStruct({"authType": SimTypeInt(signed=False, label="IPSEC_AUTH_TYPE"), "authConfig": SimTypeChar(label="Byte")}, name="IPSEC_AUTH_TRANSFORM_ID0", pack=False, align=None), "cryptoModuleId": SimTypePointer(SimTypeBottom(label="Guid"), offset=0)}, name="IPSEC_AUTH_TRANSFORM0", pack=False, align=None), offset=0)}, name="<anon>", label="None")}, name="IPSEC_SA_TRANSFORM0", pack=False, align=None), offset=0), "pfsGroup": SimTypeInt(signed=False, label="IPSEC_PFS_GROUP")}, name="IPSEC_PROPOSAL0", pack=False, align=None), offset=0), "flags": SimTypeInt(signed=False, label="IPSEC_POLICY_FLAG"), "ndAllowClearTimeoutSeconds": SimTypeInt(signed=False, label="UInt32"), "saIdleTimeout": SimStruct({"idleTimeoutSeconds": SimTypeInt(signed=False, label="UInt32"), "idleTimeoutSecondsFailOver": SimTypeInt(signed=False, label="UInt32")}, name="IPSEC_SA_IDLE_TIMEOUT0", pack=False, align=None), "emPolicy": SimTypePointer(SimStruct({"numAuthenticationMethods": SimTypeInt(signed=False, label="UInt32"), "authenticationMethods": SimTypePointer(SimStruct({"authenticationMethodType": SimTypeInt(signed=False, label="IKEEXT_AUTHENTICATION_METHOD_TYPE"), "Anonymous": SimUnion({"presharedKeyAuthentication": SimStruct({"presharedKey": SimStruct({"size": SimTypeInt(signed=False, label="UInt32"), "data": SimTypePointer(SimTypeChar(label="Byte"), offset=0)}, name="FWP_BYTE_BLOB", pack=False, align=None), "flags": SimTypeInt(signed=False, label="IKEEXT_PRESHARED_KEY_AUTHENTICATION_FLAGS")}, name="IKEEXT_PRESHARED_KEY_AUTHENTICATION1", pack=False, align=None), "certificateAuthentication": SimStruct({"inboundConfigType": SimTypeInt(signed=False, label="IKEEXT_CERT_CONFIG_TYPE"), "Anonymous1": SimUnion({"Anonymous1": SimStruct({"inboundRootArraySize": SimTypeInt(signed=False, label="UInt32"), "inboundRootCriteria": SimTypePointer(SimStruct({"certData": SimStruct({"size": SimTypeInt(signed=False, label="UInt32"), "data": SimTypePointer(SimTypeChar(label="Byte"), offset=0)}, name="FWP_BYTE_BLOB", pack=False, align=None), "certHash": SimStruct({"size": SimTypeInt(signed=False, label="UInt32"), "data": SimTypePointer(SimTypeChar(label="Byte"), offset=0)}, name="FWP_BYTE_BLOB", pack=False, align=None), "eku": SimTypePointer(SimStruct({"numEku": SimTypeInt(signed=False, label="UInt32"), "eku": SimTypePointer(SimTypePointer(SimTypeChar(label="Byte"), offset=0), offset=0)}, name="IKEEXT_CERT_EKUS0", pack=False, align=None), offset=0), "name": SimTypePointer(SimStruct({"nameType": SimTypeInt(signed=False, label="IKEEXT_CERT_CRITERIA_NAME_TYPE"), "certName": SimTypePointer(SimTypeChar(label="Char"), offset=0)}, name="IKEEXT_CERT_NAME0", pack=False, align=None), offset=0), "flags": SimTypeInt(signed=False, label="UInt32")}, name="IKEEXT_CERTIFICATE_CRITERIA0", pack=False, align=None), offset=0)}, name="_Anonymous1_e__Struct", pack=False, align=None), "Anonymous2": SimStruct({"inboundEnterpriseStoreArraySize": SimTypeInt(signed=False, label="UInt32"), "inboundEnterpriseStoreCriteria": SimTypePointer(SimStruct({"certData": SimStruct({"size": SimTypeInt(signed=False, label="UInt32"), "data": SimTypePointer(SimTypeChar(label="Byte"), offset=0)}, name="FWP_BYTE_BLOB", pack=False, align=None), "certHash": SimStruct({"size": SimTypeInt(signed=False, label="UInt32"), "data": SimTypePointer(SimTypeChar(label="Byte"), offset=0)}, name="FWP_BYTE_BLOB", pack=False, align=None), "eku": SimTypePointer(SimStruct({"numEku": SimTypeInt(signed=False, label="UInt32"), "eku": SimTypePointer(SimTypePointer(SimTypeChar(label="Byte"), offset=0), offset=0)}, name="IKEEXT_CERT_EKUS0", pack=False, align=None), offset=0), "name": SimTypePointer(SimStruct({"nameType": SimTypeInt(signed=False, label="IKEEXT_CERT_CRITERIA_NAME_TYPE"), "certName": SimTypePointer(SimTypeChar(label="Char"), offset=0)}, name="IKEEXT_CERT_NAME0", pack=False, align=None), offset=0), "flags": SimTypeInt(signed=False, label="UInt32")}, name="IKEEXT_CERTIFICATE_CRITERIA0", pack=False, align=None), offset=0)}, name="_Anonymous2_e__Struct", pack=False, align=None), "Anonymous3": SimStruct({"inboundRootStoreArraySize": SimTypeInt(signed=False, label="UInt32"), "inboundTrustedRootStoreCriteria": SimTypePointer(SimStruct({"certData": SimStruct({"size": SimTypeInt(signed=False, label="UInt32"), "data": SimTypePointer(SimTypeChar(label="Byte"), offset=0)}, name="FWP_BYTE_BLOB", pack=False, align=None), "certHash": SimStruct({"size": SimTypeInt(signed=False, label="UInt32"), "data": SimTypePointer(SimTypeChar(label="Byte"), offset=0)}, name="FWP_BYTE_BLOB", pack=False, align=None), "eku": SimTypePointer(SimStruct({"numEku": SimTypeInt(signed=False, label="UInt32"), "eku": SimTypePointer(SimTypePointer(SimTypeChar(label="Byte"), offset=0), offset=0)}, name="IKEEXT_CERT_EKUS0", pack=False, align=None), offset=0), "name": SimTypePointer(SimStruct({"nameType": SimTypeInt(signed=False, label="IKEEXT_CERT_CRITERIA_NAME_TYPE"), "certName": SimTypePointer(SimTypeChar(label="Char"), offset=0)}, name="IKEEXT_CERT_NAME0", pack=False, align=None), offset=0), "flags": SimTypeInt(signed=False, label="UInt32")}, name="IKEEXT_CERTIFICATE_CRITERIA0", pack=False, align=None), offset=0)}, name="_Anonymous3_e__Struct", pack=False, align=None)}, name="<anon>", label="None"), "outboundConfigType": SimTypeInt(signed=False, label="IKEEXT_CERT_CONFIG_TYPE"), "Anonymous2": SimUnion({"Anonymous1": SimStruct({"outboundRootArraySize": SimTypeInt(signed=False, label="UInt32"), "outboundRootCriteria": SimTypePointer(SimStruct({"certData": SimStruct({"size": SimTypeInt(signed=False, label="UInt32"), "data": SimTypePointer(SimTypeChar(label="Byte"), offset=0)}, name="FWP_BYTE_BLOB", pack=False, align=None), "certHash": SimStruct({"size": SimTypeInt(signed=False, label="UInt32"), "data": SimTypePointer(SimTypeChar(label="Byte"), offset=0)}, name="FWP_BYTE_BLOB", pack=False, align=None), "eku": SimTypePointer(SimStruct({"numEku": SimTypeInt(signed=False, label="UInt32"), "eku": SimTypePointer(SimTypePointer(SimTypeChar(label="Byte"), offset=0), offset=0)}, name="IKEEXT_CERT_EKUS0", pack=False, align=None), offset=0), "name": SimTypePointer(SimStruct({"nameType": SimTypeInt(signed=False, label="IKEEXT_CERT_CRITERIA_NAME_TYPE"), "certName": SimTypePointer(SimTypeChar(label="Char"), offset=0)}, name="IKEEXT_CERT_NAME0", pack=False, align=None), offset=0), "flags": SimTypeInt(signed=False, label="UInt32")}, name="IKEEXT_CERTIFICATE_CRITERIA0", pack=False, align=None), offset=0)}, name="_Anonymous1_e__Struct", pack=False, align=None), "Anonymous2": SimStruct({"outboundEnterpriseStoreArraySize": SimTypeInt(signed=False, label="UInt32"), "outboundEnterpriseStoreCriteria": SimTypePointer(SimStruct({"certData": SimStruct({"size": SimTypeInt(signed=False, label="UInt32"), "data": SimTypePointer(SimTypeChar(label="Byte"), offset=0)}, name="FWP_BYTE_BLOB", pack=False, align=None), "certHash": SimStruct({"size": SimTypeInt(signed=False, label="UInt32"), "data": SimTypePointer(SimTypeChar(label="Byte"), offset=0)}, name="FWP_BYTE_BLOB", pack=False, align=None), "eku": SimTypePointer(SimStruct({"numEku": SimTypeInt(signed=False, label="UInt32"), "eku": SimTypePointer(SimTypePointer(SimTypeChar(label="Byte"), offset=0), offset=0)}, name="IKEEXT_CERT_EKUS0", pack=False, align=None), offset=0), "name": SimTypePointer(SimStruct({"nameType": SimTypeInt(signed=False, label="IKEEXT_CERT_CRITERIA_NAME_TYPE"), "certName": SimTypePointer(SimTypeChar(label="Char"), offset=0)}, name="IKEEXT_CERT_NAME0", pack=False, align=None), offset=0), "flags": SimTypeInt(signed=False, label="UInt32")}, name="IKEEXT_CERTIFICATE_CRITERIA0", pack=False, align=None), offset=0)}, name="_Anonymous2_e__Struct", pack=False, align=None), "Anonymous3": SimStruct({"outboundRootStoreArraySize": SimTypeInt(signed=False, label="UInt32"), "outboundTrustedRootStoreCriteria": SimTypePointer(SimStruct({"certData": SimStruct({"size": SimTypeInt(signed=False, label="UInt32"), "data": SimTypePointer(SimTypeChar(label="Byte"), offset=0)}, name="FWP_BYTE_BLOB", pack=False, align=None), "certHash": SimStruct({"size": SimTypeInt(signed=False, label="UInt32"), "data": SimTypePointer(SimTypeChar(label="Byte"), offset=0)}, name="FWP_BYTE_BLOB", pack=False, align=None), "eku": SimTypePointer(SimStruct({"numEku": SimTypeInt(signed=False, label="UInt32"), "eku": SimTypePointer(SimTypePointer(SimTypeChar(label="Byte"), offset=0), offset=0)}, name="IKEEXT_CERT_EKUS0", pack=False, align=None), offset=0), "name": SimTypePointer(SimStruct({"nameType": SimTypeInt(signed=False, label="IKEEXT_CERT_CRITERIA_NAME_TYPE"), "certName": SimTypePointer(SimTypeChar(label="Char"), offset=0)}, name="IKEEXT_CERT_NAME0", pack=False, align=None), offset=0), "flags": SimTypeInt(signed=False, label="UInt32")}, name="IKEEXT_CERTIFICATE_CRITERIA0", pack=False, align=None), offset=0)}, name="_Anonymous3_e__Struct", pack=False, align=None)}, name="<anon>", label="None"), "flags": SimTypeInt(signed=False, label="IKEEXT_CERT_AUTH"), "localCertLocationUrl": SimStruct({"size": SimTypeInt(signed=False, label="UInt32"), "data": SimTypePointer(SimTypeChar(label="Byte"), offset=0)}, name="FWP_BYTE_BLOB", pack=False, align=None)}, name="IKEEXT_CERTIFICATE_AUTHENTICATION2", pack=False, align=None), "kerberosAuthentication": SimStruct({"flags": SimTypeInt(signed=False, label="IKEEXT_KERBEROS_AUTHENTICATION_FLAGS"), "proxyServer": SimTypePointer(SimTypeChar(label="Char"), offset=0)}, name="IKEEXT_KERBEROS_AUTHENTICATION1", pack=False, align=None), "reservedAuthentication": SimStruct({"flags": SimTypeInt(signed=False, label="IKEEXT_RESERVED_AUTHENTICATION_FLAGS")}, name="IKEEXT_RESERVED_AUTHENTICATION0", pack=False, align=None), "ntlmV2Authentication": SimStruct({"flags": SimTypeInt(signed=False, label="UInt32")}, name="IKEEXT_NTLM_V2_AUTHENTICATION0", pack=False, align=None), "sslAuthentication": SimStruct({"inboundConfigType": SimTypeInt(signed=False, label="IKEEXT_CERT_CONFIG_TYPE"), "Anonymous1": SimUnion({"Anonymous1": SimStruct({"inboundRootArraySize": SimTypeInt(signed=False, label="UInt32"), "inboundRootCriteria": SimTypePointer(SimStruct({"certData": SimStruct({"size": SimTypeInt(signed=False, label="UInt32"), "data": SimTypePointer(SimTypeChar(label="Byte"), offset=0)}, name="FWP_BYTE_BLOB", pack=False, align=None), "certHash": SimStruct({"size": SimTypeInt(signed=False, label="UInt32"), "data": SimTypePointer(SimTypeChar(label="Byte"), offset=0)}, name="FWP_BYTE_BLOB", pack=False, align=None), "eku": SimTypePointer(SimStruct({"numEku": SimTypeInt(signed=False, label="UInt32"), "eku": SimTypePointer(SimTypePointer(SimTypeChar(label="Byte"), offset=0), offset=0)}, name="IKEEXT_CERT_EKUS0", pack=False, align=None), offset=0), "name": SimTypePointer(SimStruct({"nameType": SimTypeInt(signed=False, label="IKEEXT_CERT_CRITERIA_NAME_TYPE"), "certName": SimTypePointer(SimTypeChar(label="Char"), offset=0)}, name="IKEEXT_CERT_NAME0", pack=False, align=None), offset=0), "flags": SimTypeInt(signed=False, label="UInt32")}, name="IKEEXT_CERTIFICATE_CRITERIA0", pack=False, align=None), offset=0)}, name="_Anonymous1_e__Struct", pack=False, align=None), "Anonymous2": SimStruct({"inboundEnterpriseStoreArraySize": SimTypeInt(signed=False, label="UInt32"), "inboundEnterpriseStoreCriteria": SimTypePointer(SimStruct({"certData": SimStruct({"size": SimTypeInt(signed=False, label="UInt32"), "data": SimTypePointer(SimTypeChar(label="Byte"), offset=0)}, name="FWP_BYTE_BLOB", pack=False, align=None), "certHash": SimStruct({"size": SimTypeInt(signed=False, label="UInt32"), "data": SimTypePointer(SimTypeChar(label="Byte"), offset=0)}, name="FWP_BYTE_BLOB", pack=False, align=None), "eku": SimTypePointer(SimStruct({"numEku": SimTypeInt(signed=False, label="UInt32"), "eku": SimTypePointer(SimTypePointer(SimTypeChar(label="Byte"), offset=0), offset=0)}, name="IKEEXT_CERT_EKUS0", pack=False, align=None), offset=0), "name": SimTypePointer(SimStruct({"nameType": SimTypeInt(signed=False, label="IKEEXT_CERT_CRITERIA_NAME_TYPE"), "certName": SimTypePointer(SimTypeChar(label="Char"), offset=0)}, name="IKEEXT_CERT_NAME0", pack=False, align=None), offset=0), "flags": SimTypeInt(signed=False, label="UInt32")}, name="IKEEXT_CERTIFICATE_CRITERIA0", pack=False, align=None), offset=0)}, name="_Anonymous2_e__Struct", pack=False, align=None), "Anonymous3": SimStruct({"inboundRootStoreArraySize": SimTypeInt(signed=False, label="UInt32"), "inboundTrustedRootStoreCriteria": SimTypePointer(SimStruct({"certData": SimStruct({"size": SimTypeInt(signed=False, label="UInt32"), "data": SimTypePointer(SimTypeChar(label="Byte"), offset=0)}, name="FWP_BYTE_BLOB", pack=False, align=None), "certHash": SimStruct({"size": SimTypeInt(signed=False, label="UInt32"), "data": SimTypePointer(SimTypeChar(label="Byte"), offset=0)}, name="FWP_BYTE_BLOB", pack=False, align=None), "eku": SimTypePointer(SimStruct({"numEku": SimTypeInt(signed=False, label="UInt32"), "eku": SimTypePointer(SimTypePointer(SimTypeChar(label="Byte"), offset=0), offset=0)}, name="IKEEXT_CERT_EKUS0", pack=False, align=None), offset=0), "name": SimTypePointer(SimStruct({"nameType": SimTypeInt(signed=False, label="IKEEXT_CERT_CRITERIA_NAME_TYPE"), "certName": SimTypePointer(SimTypeChar(label="Char"), offset=0)}, name="IKEEXT_CERT_NAME0", pack=False, align=None), offset=0), "flags": SimTypeInt(signed=False, label="UInt32")}, name="IKEEXT_CERTIFICATE_CRITERIA0", pack=False, align=None), offset=0)}, name="_Anonymous3_e__Struct", pack=False, align=None)}, name="<anon>", label="None"), "outboundConfigType": SimTypeInt(signed=False, label="IKEEXT_CERT_CONFIG_TYPE"), "Anonymous2": SimUnion({"Anonymous1": SimStruct({"outboundRootArraySize": SimTypeInt(signed=False, label="UInt32"), "outboundRootCriteria": SimTypePointer(SimStruct({"certData": SimStruct({"size": SimTypeInt(signed=False, label="UInt32"), "data": SimTypePointer(SimTypeChar(label="Byte"), offset=0)}, name="FWP_BYTE_BLOB", pack=False, align=None), "certHash": SimStruct({"size": SimTypeInt(signed=False, label="UInt32"), "data": SimTypePointer(SimTypeChar(label="Byte"), offset=0)}, name="FWP_BYTE_BLOB", pack=False, align=None), "eku": SimTypePointer(SimStruct({"numEku": SimTypeInt(signed=False, label="UInt32"), "eku": SimTypePointer(SimTypePointer(SimTypeChar(label="Byte"), offset=0), offset=0)}, name="IKEEXT_CERT_EKUS0", pack=False, align=None), offset=0), "name": SimTypePointer(SimStruct({"nameType": SimTypeInt(signed=False, label="IKEEXT_CERT_CRITERIA_NAME_TYPE"), "certName": SimTypePointer(SimTypeChar(label="Char"), offset=0)}, name="IKEEXT_CERT_NAME0", pack=False, align=None), offset=0), "flags": SimTypeInt(signed=False, label="UInt32")}, name="IKEEXT_CERTIFICATE_CRITERIA0", pack=False, align=None), offset=0)}, name="_Anonymous1_e__Struct", pack=False, align=None), "Anonymous2": SimStruct({"outboundEnterpriseStoreArraySize": SimTypeInt(signed=False, label="UInt32"), "outboundEnterpriseStoreCriteria": SimTypePointer(SimStruct({"certData": SimStruct({"size": SimTypeInt(signed=False, label="UInt32"), "data": SimTypePointer(SimTypeChar(label="Byte"), offset=0)}, name="FWP_BYTE_BLOB", pack=False, align=None), "certHash": SimStruct({"size": SimTypeInt(signed=False, label="UInt32"), "data": SimTypePointer(SimTypeChar(label="Byte"), offset=0)}, name="FWP_BYTE_BLOB", pack=False, align=None), "eku": SimTypePointer(SimStruct({"numEku": SimTypeInt(signed=False, label="UInt32"), "eku": SimTypePointer(SimTypePointer(SimTypeChar(label="Byte"), offset=0), offset=0)}, name="IKEEXT_CERT_EKUS0", pack=False, align=None), offset=0), "name": SimTypePointer(SimStruct({"nameType": SimTypeInt(signed=False, label="IKEEXT_CERT_CRITERIA_NAME_TYPE"), "certName": SimTypePointer(SimTypeChar(label="Char"), offset=0)}, name="IKEEXT_CERT_NAME0", pack=False, align=None), offset=0), "flags": SimTypeInt(signed=False, label="UInt32")}, name="IKEEXT_CERTIFICATE_CRITERIA0", pack=False, align=None), offset=0)}, name="_Anonymous2_e__Struct", pack=False, align=None), "Anonymous3": SimStruct({"outboundRootStoreArraySize": SimTypeInt(signed=False, label="UInt32"), "outboundTrustedRootStoreCriteria": SimTypePointer(SimStruct({"certData": SimStruct({"size": SimTypeInt(signed=False, label="UInt32"), "data": SimTypePointer(SimTypeChar(label="Byte"), offset=0)}, name="FWP_BYTE_BLOB", pack=False, align=None), "certHash": SimStruct({"size": SimTypeInt(signed=False, label="UInt32"), "data": SimTypePointer(SimTypeChar(label="Byte"), offset=0)}, name="FWP_BYTE_BLOB", pack=False, align=None), "eku": SimTypePointer(SimStruct({"numEku": SimTypeInt(signed=False, label="UInt32"), "eku": SimTypePointer(SimTypePointer(SimTypeChar(label="Byte"), offset=0), offset=0)}, name="IKEEXT_CERT_EKUS0", pack=False, align=None), offset=0), "name": SimTypePointer(SimStruct({"nameType": SimTypeInt(signed=False, label="IKEEXT_CERT_CRITERIA_NAME_TYPE"), "certName": SimTypePointer(SimTypeChar(label="Char"), offset=0)}, name="IKEEXT_CERT_NAME0", pack=False, align=None), offset=0), "flags": SimTypeInt(signed=False, label="UInt32")}, name="IKEEXT_CERTIFICATE_CRITERIA0", pack=False, align=None), offset=0)}, name="_Anonymous3_e__Struct", pack=False, align=None)}, name="<anon>", label="None"), "flags": SimTypeInt(signed=False, label="IKEEXT_CERT_AUTH"), "localCertLocationUrl": SimStruct({"size": SimTypeInt(signed=False, label="UInt32"), "data": SimTypePointer(SimTypeChar(label="Byte"), offset=0)}, name="FWP_BYTE_BLOB", pack=False, align=None)}, name="IKEEXT_CERTIFICATE_AUTHENTICATION2", pack=False, align=None), "cgaAuthentication": SimStruct({"keyContainerName": SimTypePointer(SimTypeChar(label="Char"), offset=0), "cspName": SimTypePointer(SimTypeChar(label="Char"), offset=0), "cspType": SimTypeInt(signed=False, label="UInt32"), "cgaModifier": SimStruct({"byteArray16": SimTypeFixedSizeArray(SimTypeChar(label="Byte"), 16)}, name="FWP_BYTE_ARRAY16", pack=False, align=None), "cgaCollisionCount": SimTypeChar(label="Byte")}, name="IKEEXT_IPV6_CGA_AUTHENTICATION0", pack=False, align=None), "eapAuthentication": SimStruct({"flags": SimTypeInt(signed=False, label="IKEEXT_EAP_AUTHENTICATION_FLAGS")}, name="IKEEXT_EAP_AUTHENTICATION0", pack=False, align=None)}, name="<anon>", label="None")}, name="IKEEXT_AUTHENTICATION_METHOD2", pack=False, align=None), offset=0), "initiatorImpersonationType": SimTypeInt(signed=False, label="IKEEXT_AUTHENTICATION_IMPERSONATION_TYPE")}, name="IKEEXT_EM_POLICY2", pack=False, align=None), offset=0)}, name="IPSEC_TRANSPORT_POLICY2", pack=False, align=None), offset=0), "ikeQmTunnelPolicy": SimTypePointer(SimStruct({"flags": SimTypeInt(signed=False, label="UInt32"), "numIpsecProposals": SimTypeInt(signed=False, label="UInt32"), "ipsecProposals": SimTypePointer(SimStruct({"lifetime": SimStruct({"lifetimeSeconds": SimTypeInt(signed=False, label="UInt32"), "lifetimeKilobytes": SimTypeInt(signed=False, label="UInt32"), "lifetimePackets": SimTypeInt(signed=False, label="UInt32")}, name="IPSEC_SA_LIFETIME0", pack=False, align=None), "numSaTransforms": SimTypeInt(signed=False, label="UInt32"), "saTransforms": SimTypePointer(SimStruct({"ipsecTransformType": SimTypeInt(signed=False, label="IPSEC_TRANSFORM_TYPE"), "Anonymous": SimUnion({"ahTransform": SimTypePointer(SimStruct({"authTransformId": SimStruct({"authType": SimTypeInt(signed=False, label="IPSEC_AUTH_TYPE"), "authConfig": SimTypeChar(label="Byte")}, name="IPSEC_AUTH_TRANSFORM_ID0", pack=False, align=None), "cryptoModuleId": SimTypePointer(SimTypeBottom(label="Guid"), offset=0)}, name="IPSEC_AUTH_TRANSFORM0", pack=False, align=None), offset=0), "espAuthTransform": SimTypePointer(SimStruct({"authTransformId": SimStruct({"authType": SimTypeInt(signed=False, label="IPSEC_AUTH_TYPE"), "authConfig": SimTypeChar(label="Byte")}, name="IPSEC_AUTH_TRANSFORM_ID0", pack=False, align=None), "cryptoModuleId": SimTypePointer(SimTypeBottom(label="Guid"), offset=0)}, name="IPSEC_AUTH_TRANSFORM0", pack=False, align=None), offset=0), "espCipherTransform": SimTypePointer(SimStruct({"cipherTransformId": SimStruct({"cipherType": SimTypeInt(signed=False, label="IPSEC_CIPHER_TYPE"), "cipherConfig": SimTypeChar(label="Byte")}, name="IPSEC_CIPHER_TRANSFORM_ID0", pack=False, align=None), "cryptoModuleId": SimTypePointer(SimTypeBottom(label="Guid"), offset=0)}, name="IPSEC_CIPHER_TRANSFORM0", pack=False, align=None), offset=0), "espAuthAndCipherTransform": SimTypePointer(SimStruct({"authTransform": SimStruct({"authTransformId": SimStruct({"authType": SimTypeInt(signed=False, label="IPSEC_AUTH_TYPE"), "authConfig": SimTypeChar(label="Byte")}, name="IPSEC_AUTH_TRANSFORM_ID0", pack=False, align=None), "cryptoModuleId": SimTypePointer(SimTypeBottom(label="Guid"), offset=0)}, name="IPSEC_AUTH_TRANSFORM0", pack=False, align=None), "cipherTransform": SimStruct({"cipherTransformId": SimStruct({"cipherType": SimTypeInt(signed=False, label="IPSEC_CIPHER_TYPE"), "cipherConfig": SimTypeChar(label="Byte")}, name="IPSEC_CIPHER_TRANSFORM_ID0", pack=False, align=None), "cryptoModuleId": SimTypePointer(SimTypeBottom(label="Guid"), offset=0)}, name="IPSEC_CIPHER_TRANSFORM0", pack=False, align=None)}, name="IPSEC_AUTH_AND_CIPHER_TRANSFORM0", pack=False, align=None), offset=0), "espAuthFwTransform": SimTypePointer(SimStruct({"authTransformId": SimStruct({"authType": SimTypeInt(signed=False, label="IPSEC_AUTH_TYPE"), "authConfig": SimTypeChar(label="Byte")}, name="IPSEC_AUTH_TRANSFORM_ID0", pack=False, align=None), "cryptoModuleId": SimTypePointer(SimTypeBottom(label="Guid"), offset=0)}, name="IPSEC_AUTH_TRANSFORM0", pack=False, align=None), offset=0)}, name="<anon>", label="None")}, name="IPSEC_SA_TRANSFORM0", pack=False, align=None), offset=0), "pfsGroup": SimTypeInt(signed=False, label="IPSEC_PFS_GROUP")}, name="IPSEC_PROPOSAL0", pack=False, align=None), offset=0), "tunnelEndpoints": SimStruct({"ipVersion": SimTypeInt(signed=False, label="FWP_IP_VERSION"), "Anonymous1": SimUnion({"localV4Address": SimTypeInt(signed=False, label="UInt32"), "localV6Address": SimTypeFixedSizeArray(SimTypeChar(label="Byte"), 16)}, name="<anon>", label="None"), "Anonymous2": SimUnion({"remoteV4Address": SimTypeInt(signed=False, label="UInt32"), "remoteV6Address": SimTypeFixedSizeArray(SimTypeChar(label="Byte"), 16)}, name="<anon>", label="None"), "localIfLuid": SimTypeLongLong(signed=False, label="UInt64"), "remoteFqdn": SimTypePointer(SimTypeChar(label="Char"), offset=0), "numAddresses": SimTypeInt(signed=False, label="UInt32"), "remoteAddresses": SimTypePointer(SimStruct({"ipVersion": SimTypeInt(signed=False, label="FWP_IP_VERSION"), "Anonymous": SimUnion({"v4Address": SimTypeInt(signed=False, label="UInt32"), "v6Address": SimTypeFixedSizeArray(SimTypeChar(label="Byte"), 16)}, name="<anon>", label="None")}, name="IPSEC_TUNNEL_ENDPOINT0", pack=False, align=None), offset=0)}, name="IPSEC_TUNNEL_ENDPOINTS2", pack=False, align=None), "saIdleTimeout": SimStruct({"idleTimeoutSeconds": SimTypeInt(signed=False, label="UInt32"), "idleTimeoutSecondsFailOver": SimTypeInt(signed=False, label="UInt32")}, name="IPSEC_SA_IDLE_TIMEOUT0", pack=False, align=None), "emPolicy": SimTypePointer(SimStruct({"numAuthenticationMethods": SimTypeInt(signed=False, label="UInt32"), "authenticationMethods": SimTypePointer(SimStruct({"authenticationMethodType": SimTypeInt(signed=False, label="IKEEXT_AUTHENTICATION_METHOD_TYPE"), "Anonymous": SimUnion({"presharedKeyAuthentication": SimStruct({"presharedKey": SimStruct({"size": SimTypeInt(signed=False, label="UInt32"), "data": SimTypePointer(SimTypeChar(label="Byte"), offset=0)}, name="FWP_BYTE_BLOB", pack=False, align=None), "flags": SimTypeInt(signed=False, label="IKEEXT_PRESHARED_KEY_AUTHENTICATION_FLAGS")}, name="IKEEXT_PRESHARED_KEY_AUTHENTICATION1", pack=False, align=None), "certificateAuthentication": SimStruct({"inboundConfigType": SimTypeInt(signed=False, label="IKEEXT_CERT_CONFIG_TYPE"), "Anonymous1": SimUnion({"Anonymous1": SimStruct({"inboundRootArraySize": SimTypeInt(signed=False, label="UInt32"), "inboundRootCriteria": SimTypePointer(SimStruct({"certData": SimStruct({"size": SimTypeInt(signed=False, label="UInt32"), "data": SimTypePointer(SimTypeChar(label="Byte"), offset=0)}, name="FWP_BYTE_BLOB", pack=False, align=None), "certHash": SimStruct({"size": SimTypeInt(signed=False, label="UInt32"), "data": SimTypePointer(SimTypeChar(label="Byte"), offset=0)}, name="FWP_BYTE_BLOB", pack=False, align=None), "eku": SimTypePointer(SimStruct({"numEku": SimTypeInt(signed=False, label="UInt32"), "eku": SimTypePointer(SimTypePointer(SimTypeChar(label="Byte"), offset=0), offset=0)}, name="IKEEXT_CERT_EKUS0", pack=False, align=None), offset=0), "name": SimTypePointer(SimStruct({"nameType": SimTypeInt(signed=False, label="IKEEXT_CERT_CRITERIA_NAME_TYPE"), "certName": SimTypePointer(SimTypeChar(label="Char"), offset=0)}, name="IKEEXT_CERT_NAME0", pack=False, align=None), offset=0), "flags": SimTypeInt(signed=False, label="UInt32")}, name="IKEEXT_CERTIFICATE_CRITERIA0", pack=False, align=None), offset=0)}, name="_Anonymous1_e__Struct", pack=False, align=None), "Anonymous2": SimStruct({"inboundEnterpriseStoreArraySize": SimTypeInt(signed=False, label="UInt32"), "inboundEnterpriseStoreCriteria": SimTypePointer(SimStruct({"certData": SimStruct({"size": SimTypeInt(signed=False, label="UInt32"), "data": SimTypePointer(SimTypeChar(label="Byte"), offset=0)}, name="FWP_BYTE_BLOB", pack=False, align=None), "certHash": SimStruct({"size": SimTypeInt(signed=False, label="UInt32"), "data": SimTypePointer(SimTypeChar(label="Byte"), offset=0)}, name="FWP_BYTE_BLOB", pack=False, align=None), "eku": SimTypePointer(SimStruct({"numEku": SimTypeInt(signed=False, label="UInt32"), "eku": SimTypePointer(SimTypePointer(SimTypeChar(label="Byte"), offset=0), offset=0)}, name="IKEEXT_CERT_EKUS0", pack=False, align=None), offset=0), "name": SimTypePointer(SimStruct({"nameType": SimTypeInt(signed=False, label="IKEEXT_CERT_CRITERIA_NAME_TYPE"), "certName": SimTypePointer(SimTypeChar(label="Char"), offset=0)}, name="IKEEXT_CERT_NAME0", pack=False, align=None), offset=0), "flags": SimTypeInt(signed=False, label="UInt32")}, name="IKEEXT_CERTIFICATE_CRITERIA0", pack=False, align=None), offset=0)}, name="_Anonymous2_e__Struct", pack=False, align=None), "Anonymous3": SimStruct({"inboundRootStoreArraySize": SimTypeInt(signed=False, label="UInt32"), "inboundTrustedRootStoreCriteria": SimTypePointer(SimStruct({"certData": SimStruct({"size": SimTypeInt(signed=False, label="UInt32"), "data": SimTypePointer(SimTypeChar(label="Byte"), offset=0)}, name="FWP_BYTE_BLOB", pack=False, align=None), "certHash": SimStruct({"size": SimTypeInt(signed=False, label="UInt32"), "data": SimTypePointer(SimTypeChar(label="Byte"), offset=0)}, name="FWP_BYTE_BLOB", pack=False, align=None), "eku": SimTypePointer(SimStruct({"numEku": SimTypeInt(signed=False, label="UInt32"), "eku": SimTypePointer(SimTypePointer(SimTypeChar(label="Byte"), offset=0), offset=0)}, name="IKEEXT_CERT_EKUS0", pack=False, align=None), offset=0), "name": SimTypePointer(SimStruct({"nameType": SimTypeInt(signed=False, label="IKEEXT_CERT_CRITERIA_NAME_TYPE"), "certName": SimTypePointer(SimTypeChar(label="Char"), offset=0)}, name="IKEEXT_CERT_NAME0", pack=False, align=None), offset=0), "flags": SimTypeInt(signed=False, label="UInt32")}, name="IKEEXT_CERTIFICATE_CRITERIA0", pack=False, align=None), offset=0)}, name="_Anonymous3_e__Struct", pack=False, align=None)}, name="<anon>", label="None"), "outboundConfigType": SimTypeInt(signed=False, label="IKEEXT_CERT_CONFIG_TYPE"), "Anonymous2": SimUnion({"Anonymous1": SimStruct({"outboundRootArraySize": SimTypeInt(signed=False, label="UInt32"), "outboundRootCriteria": SimTypePointer(SimStruct({"certData": SimStruct({"size": SimTypeInt(signed=False, label="UInt32"), "data": SimTypePointer(SimTypeChar(label="Byte"), offset=0)}, name="FWP_BYTE_BLOB", pack=False, align=None), "certHash": SimStruct({"size": SimTypeInt(signed=False, label="UInt32"), "data": SimTypePointer(SimTypeChar(label="Byte"), offset=0)}, name="FWP_BYTE_BLOB", pack=False, align=None), "eku": SimTypePointer(SimStruct({"numEku": SimTypeInt(signed=False, label="UInt32"), "eku": SimTypePointer(SimTypePointer(SimTypeChar(label="Byte"), offset=0), offset=0)}, name="IKEEXT_CERT_EKUS0", pack=False, align=None), offset=0), "name": SimTypePointer(SimStruct({"nameType": SimTypeInt(signed=False, label="IKEEXT_CERT_CRITERIA_NAME_TYPE"), "certName": SimTypePointer(SimTypeChar(label="Char"), offset=0)}, name="IKEEXT_CERT_NAME0", pack=False, align=None), offset=0), "flags": SimTypeInt(signed=False, label="UInt32")}, name="IKEEXT_CERTIFICATE_CRITERIA0", pack=False, align=None), offset=0)}, name="_Anonymous1_e__Struct", pack=False, align=None), "Anonymous2": SimStruct({"outboundEnterpriseStoreArraySize": SimTypeInt(signed=False, label="UInt32"), "outboundEnterpriseStoreCriteria": SimTypePointer(SimStruct({"certData": SimStruct({"size": SimTypeInt(signed=False, label="UInt32"), "data": SimTypePointer(SimTypeChar(label="Byte"), offset=0)}, name="FWP_BYTE_BLOB", pack=False, align=None), "certHash": SimStruct({"size": SimTypeInt(signed=False, label="UInt32"), "data": SimTypePointer(SimTypeChar(label="Byte"), offset=0)}, name="FWP_BYTE_BLOB", pack=False, align=None), "eku": SimTypePointer(SimStruct({"numEku": SimTypeInt(signed=False, label="UInt32"), "eku": SimTypePointer(SimTypePointer(SimTypeChar(label="Byte"), offset=0), offset=0)}, name="IKEEXT_CERT_EKUS0", pack=False, align=None), offset=0), "name": SimTypePointer(SimStruct({"nameType": SimTypeInt(signed=False, label="IKEEXT_CERT_CRITERIA_NAME_TYPE"), "certName": SimTypePointer(SimTypeChar(label="Char"), offset=0)}, name="IKEEXT_CERT_NAME0", pack=False, align=None), offset=0), "flags": SimTypeInt(signed=False, label="UInt32")}, name="IKEEXT_CERTIFICATE_CRITERIA0", pack=False, align=None), offset=0)}, name="_Anonymous2_e__Struct", pack=False, align=None), "Anonymous3": SimStruct({"outboundRootStoreArraySize": SimTypeInt(signed=False, label="UInt32"), "outboundTrustedRootStoreCriteria": SimTypePointer(SimStruct({"certData": SimStruct({"size": SimTypeInt(signed=False, label="UInt32"), "data": SimTypePointer(SimTypeChar(label="Byte"), offset=0)}, name="FWP_BYTE_BLOB", pack=False, align=None), "certHash": SimStruct({"size": SimTypeInt(signed=False, label="UInt32"), "data": SimTypePointer(SimTypeChar(label="Byte"), offset=0)}, name="FWP_BYTE_BLOB", pack=False, align=None), "eku": SimTypePointer(SimStruct({"numEku": SimTypeInt(signed=False, label="UInt32"), "eku": SimTypePointer(SimTypePointer(SimTypeChar(label="Byte"), offset=0), offset=0)}, name="IKEEXT_CERT_EKUS0", pack=False, align=None), offset=0), "name": SimTypePointer(SimStruct({"nameType": SimTypeInt(signed=False, label="IKEEXT_CERT_CRITERIA_NAME_TYPE"), "certName": SimTypePointer(SimTypeChar(label="Char"), offset=0)}, name="IKEEXT_CERT_NAME0", pack=False, align=None), offset=0), "flags": SimTypeInt(signed=False, label="UInt32")}, name="IKEEXT_CERTIFICATE_CRITERIA0", pack=False, align=None), offset=0)}, name="_Anonymous3_e__Struct", pack=False, align=None)}, name="<anon>", label="None"), "flags": SimTypeInt(signed=False, label="IKEEXT_CERT_AUTH"), "localCertLocationUrl": SimStruct({"size": SimTypeInt(signed=False, label="UInt32"), "data": SimTypePointer(SimTypeChar(label="Byte"), offset=0)}, name="FWP_BYTE_BLOB", pack=False, align=None)}, name="IKEEXT_CERTIFICATE_AUTHENTICATION2", pack=False, align=None), "kerberosAuthentication": SimStruct({"flags": SimTypeInt(signed=False, label="IKEEXT_KERBEROS_AUTHENTICATION_FLAGS"), "proxyServer": SimTypePointer(SimTypeChar(label="Char"), offset=0)}, name="IKEEXT_KERBEROS_AUTHENTICATION1", pack=False, align=None), "reservedAuthentication": SimStruct({"flags": SimTypeInt(signed=False, label="IKEEXT_RESERVED_AUTHENTICATION_FLAGS")}, name="IKEEXT_RESERVED_AUTHENTICATION0", pack=False, align=None), "ntlmV2Authentication": SimStruct({"flags": SimTypeInt(signed=False, label="UInt32")}, name="IKEEXT_NTLM_V2_AUTHENTICATION0", pack=False, align=None), "sslAuthentication": SimStruct({"inboundConfigType": SimTypeInt(signed=False, label="IKEEXT_CERT_CONFIG_TYPE"), "Anonymous1": SimUnion({"Anonymous1": SimStruct({"inboundRootArraySize": SimTypeInt(signed=False, label="UInt32"), "inboundRootCriteria": SimTypePointer(SimStruct({"certData": SimStruct({"size": SimTypeInt(signed=False, label="UInt32"), "data": SimTypePointer(SimTypeChar(label="Byte"), offset=0)}, name="FWP_BYTE_BLOB", pack=False, align=None), "certHash": SimStruct({"size": SimTypeInt(signed=False, label="UInt32"), "data": SimTypePointer(SimTypeChar(label="Byte"), offset=0)}, name="FWP_BYTE_BLOB", pack=False, align=None), "eku": SimTypePointer(SimStruct({"numEku": SimTypeInt(signed=False, label="UInt32"), "eku": SimTypePointer(SimTypePointer(SimTypeChar(label="Byte"), offset=0), offset=0)}, name="IKEEXT_CERT_EKUS0", pack=False, align=None), offset=0), "name": SimTypePointer(SimStruct({"nameType": SimTypeInt(signed=False, label="IKEEXT_CERT_CRITERIA_NAME_TYPE"), "certName": SimTypePointer(SimTypeChar(label="Char"), offset=0)}, name="IKEEXT_CERT_NAME0", pack=False, align=None), offset=0), "flags": SimTypeInt(signed=False, label="UInt32")}, name="IKEEXT_CERTIFICATE_CRITERIA0", pack=False, align=None), offset=0)}, name="_Anonymous1_e__Struct", pack=False, align=None), "Anonymous2": SimStruct({"inboundEnterpriseStoreArraySize": SimTypeInt(signed=False, label="UInt32"), "inboundEnterpriseStoreCriteria": SimTypePointer(SimStruct({"certData": SimStruct({"size": SimTypeInt(signed=False, label="UInt32"), "data": SimTypePointer(SimTypeChar(label="Byte"), offset=0)}, name="FWP_BYTE_BLOB", pack=False, align=None), "certHash": SimStruct({"size": SimTypeInt(signed=False, label="UInt32"), "data": SimTypePointer(SimTypeChar(label="Byte"), offset=0)}, name="FWP_BYTE_BLOB", pack=False, align=None), "eku": SimTypePointer(SimStruct({"numEku": SimTypeInt(signed=False, label="UInt32"), "eku": SimTypePointer(SimTypePointer(SimTypeChar(label="Byte"), offset=0), offset=0)}, name="IKEEXT_CERT_EKUS0", pack=False, align=None), offset=0), "name": SimTypePointer(SimStruct({"nameType": SimTypeInt(signed=False, label="IKEEXT_CERT_CRITERIA_NAME_TYPE"), "certName": SimTypePointer(SimTypeChar(label="Char"), offset=0)}, name="IKEEXT_CERT_NAME0", pack=False, align=None), offset=0), "flags": SimTypeInt(signed=False, label="UInt32")}, name="IKEEXT_CERTIFICATE_CRITERIA0", pack=False, align=None), offset=0)}, name="_Anonymous2_e__Struct", pack=False, align=None), "Anonymous3": SimStruct({"inboundRootStoreArraySize": SimTypeInt(signed=False, label="UInt32"), "inboundTrustedRootStoreCriteria": SimTypePointer(SimStruct({"certData": SimStruct({"size": SimTypeInt(signed=False, label="UInt32"), "data": SimTypePointer(SimTypeChar(label="Byte"), offset=0)}, name="FWP_BYTE_BLOB", pack=False, align=None), "certHash": SimStruct({"size": SimTypeInt(signed=False, label="UInt32"), "data": SimTypePointer(SimTypeChar(label="Byte"), offset=0)}, name="FWP_BYTE_BLOB", pack=False, align=None), "eku": SimTypePointer(SimStruct({"numEku": SimTypeInt(signed=False, label="UInt32"), "eku": SimTypePointer(SimTypePointer(SimTypeChar(label="Byte"), offset=0), offset=0)}, name="IKEEXT_CERT_EKUS0", pack=False, align=None), offset=0), "name": SimTypePointer(SimStruct({"nameType": SimTypeInt(signed=False, label="IKEEXT_CERT_CRITERIA_NAME_TYPE"), "certName": SimTypePointer(SimTypeChar(label="Char"), offset=0)}, name="IKEEXT_CERT_NAME0", pack=False, align=None), offset=0), "flags": SimTypeInt(signed=False, label="UInt32")}, name="IKEEXT_CERTIFICATE_CRITERIA0", pack=False, align=None), offset=0)}, name="_Anonymous3_e__Struct", pack=False, align=None)}, name="<anon>", label="None"), "outboundConfigType": SimTypeInt(signed=False, label="IKEEXT_CERT_CONFIG_TYPE"), "Anonymous2": SimUnion({"Anonymous1": SimStruct({"outboundRootArraySize": SimTypeInt(signed=False, label="UInt32"), "outboundRootCriteria": SimTypePointer(SimStruct({"certData": SimStruct({"size": SimTypeInt(signed=False, label="UInt32"), "data": SimTypePointer(SimTypeChar(label="Byte"), offset=0)}, name="FWP_BYTE_BLOB", pack=False, align=None), "certHash": SimStruct({"size": SimTypeInt(signed=False, label="UInt32"), "data": SimTypePointer(SimTypeChar(label="Byte"), offset=0)}, name="FWP_BYTE_BLOB", pack=False, align=None), "eku": SimTypePointer(SimStruct({"numEku": SimTypeInt(signed=False, label="UInt32"), "eku": SimTypePointer(SimTypePointer(SimTypeChar(label="Byte"), offset=0), offset=0)}, name="IKEEXT_CERT_EKUS0", pack=False, align=None), offset=0), "name": SimTypePointer(SimStruct({"nameType": SimTypeInt(signed=False, label="IKEEXT_CERT_CRITERIA_NAME_TYPE"), "certName": SimTypePointer(SimTypeChar(label="Char"), offset=0)}, name="IKEEXT_CERT_NAME0", pack=False, align=None), offset=0), "flags": SimTypeInt(signed=False, label="UInt32")}, name="IKEEXT_CERTIFICATE_CRITERIA0", pack=False, align=None), offset=0)}, name="_Anonymous1_e__Struct", pack=False, align=None), "Anonymous2": SimStruct({"outboundEnterpriseStoreArraySize": SimTypeInt(signed=False, label="UInt32"), "outboundEnterpriseStoreCriteria": SimTypePointer(SimStruct({"certData": SimStruct({"size": SimTypeInt(signed=False, label="UInt32"), "data": SimTypePointer(SimTypeChar(label="Byte"), offset=0)}, name="FWP_BYTE_BLOB", pack=False, align=None), "certHash": SimStruct({"size": SimTypeInt(signed=False, label="UInt32"), "data": SimTypePointer(SimTypeChar(label="Byte"), offset=0)}, name="FWP_BYTE_BLOB", pack=False, align=None), "eku": SimTypePointer(SimStruct({"numEku": SimTypeInt(signed=False, label="UInt32"), "eku": SimTypePointer(SimTypePointer(SimTypeChar(label="Byte"), offset=0), offset=0)}, name="IKEEXT_CERT_EKUS0", pack=False, align=None), offset=0), "name": SimTypePointer(SimStruct({"nameType": SimTypeInt(signed=False, label="IKEEXT_CERT_CRITERIA_NAME_TYPE"), "certName": SimTypePointer(SimTypeChar(label="Char"), offset=0)}, name="IKEEXT_CERT_NAME0", pack=False, align=None), offset=0), "flags": SimTypeInt(signed=False, label="UInt32")}, name="IKEEXT_CERTIFICATE_CRITERIA0", pack=False, align=None), offset=0)}, name="_Anonymous2_e__Struct", pack=False, align=None), "Anonymous3": SimStruct({"outboundRootStoreArraySize": SimTypeInt(signed=False, label="UInt32"), "outboundTrustedRootStoreCriteria": SimTypePointer(SimStruct({"certData": SimStruct({"size": SimTypeInt(signed=False, label="UInt32"), "data": SimTypePointer(SimTypeChar(label="Byte"), offset=0)}, name="FWP_BYTE_BLOB", pack=False, align=None), "certHash": SimStruct({"size": SimTypeInt(signed=False, label="UInt32"), "data": SimTypePointer(SimTypeChar(label="Byte"), offset=0)}, name="FWP_BYTE_BLOB", pack=False, align=None), "eku": SimTypePointer(SimStruct({"numEku": SimTypeInt(signed=False, label="UInt32"), "eku": SimTypePointer(SimTypePointer(SimTypeChar(label="Byte"), offset=0), offset=0)}, name="IKEEXT_CERT_EKUS0", pack=False, align=None), offset=0), "name": SimTypePointer(SimStruct({"nameType": SimTypeInt(signed=False, label="IKEEXT_CERT_CRITERIA_NAME_TYPE"), "certName": SimTypePointer(SimTypeChar(label="Char"), offset=0)}, name="IKEEXT_CERT_NAME0", pack=False, align=None), offset=0), "flags": SimTypeInt(signed=False, label="UInt32")}, name="IKEEXT_CERTIFICATE_CRITERIA0", pack=False, align=None), offset=0)}, name="_Anonymous3_e__Struct", pack=False, align=None)}, name="<anon>", label="None"), "flags": SimTypeInt(signed=False, label="IKEEXT_CERT_AUTH"), "localCertLocationUrl": SimStruct({"size": SimTypeInt(signed=False, label="UInt32"), "data": SimTypePointer(SimTypeChar(label="Byte"), offset=0)}, name="FWP_BYTE_BLOB", pack=False, align=None)}, name="IKEEXT_CERTIFICATE_AUTHENTICATION2", pack=False, align=None), "cgaAuthentication": SimStruct({"keyContainerName": SimTypePointer(SimTypeChar(label="Char"), offset=0), "cspName": SimTypePointer(SimTypeChar(label="Char"), offset=0), "cspType": SimTypeInt(signed=False, label="UInt32"), "cgaModifier": SimStruct({"byteArray16": SimTypeFixedSizeArray(SimTypeChar(label="Byte"), 16)}, name="FWP_BYTE_ARRAY16", pack=False, align=None), "cgaCollisionCount": SimTypeChar(label="Byte")}, name="IKEEXT_IPV6_CGA_AUTHENTICATION0", pack=False, align=None), "eapAuthentication": SimStruct({"flags": SimTypeInt(signed=False, label="IKEEXT_EAP_AUTHENTICATION_FLAGS")}, name="IKEEXT_EAP_AUTHENTICATION0", pack=False, align=None)}, name="<anon>", label="None")}, name="IKEEXT_AUTHENTICATION_METHOD2", pack=False, align=None), offset=0), "initiatorImpersonationType": SimTypeInt(signed=False, label="IKEEXT_AUTHENTICATION_IMPERSONATION_TYPE")}, name="IKEEXT_EM_POLICY2", pack=False, align=None), offset=0), "fwdPathSaLifetime": SimTypeInt(signed=False, label="UInt32"), "compartmentId": SimTypeInt(signed=False, label="UInt32"), "numTrafficSelectorPolicy": SimTypeInt(signed=False, label="UInt32"), "trafficSelectorPolicies": SimTypePointer(SimStruct({"flags": SimTypeInt(signed=False, label="UInt32"), "numLocalTrafficSelectors": SimTypeInt(signed=False, label="UInt32"), "localTrafficSelectors": SimTypePointer(SimStruct({"protocolId": SimTypeChar(label="Byte"), "portStart": SimTypeShort(signed=False, label="UInt16"), "portEnd": SimTypeShort(signed=False, label="UInt16"), "ipVersion": SimTypeInt(signed=False, label="FWP_IP_VERSION"), "Anonymous1": SimUnion({"startV4Address": SimTypeInt(signed=False, label="UInt32"), "startV6Address": SimTypeFixedSizeArray(SimTypeChar(label="Byte"), 16)}, name="<anon>", label="None"), "Anonymous2": SimUnion({"endV4Address": SimTypeInt(signed=False, label="UInt32"), "endV6Address": SimTypeFixedSizeArray(SimTypeChar(label="Byte"), 16)}, name="<anon>", label="None")}, name="IPSEC_TRAFFIC_SELECTOR0_", pack=False, align=None), offset=0), "numRemoteTrafficSelectors": SimTypeInt(signed=False, label="UInt32"), "remoteTrafficSelectors": SimTypePointer(SimStruct({"protocolId": SimTypeChar(label="Byte"), "portStart": SimTypeShort(signed=False, label="UInt16"), "portEnd": SimTypeShort(signed=False, label="UInt16"), "ipVersion": SimTypeInt(signed=False, label="FWP_IP_VERSION"), "Anonymous1": SimUnion({"startV4Address": SimTypeInt(signed=False, label="UInt32"), "startV6Address": SimTypeFixedSizeArray(SimTypeChar(label="Byte"), 16)}, name="<anon>", label="None"), "Anonymous2": SimUnion({"endV4Address": SimTypeInt(signed=False, label="UInt32"), "endV6Address": SimTypeFixedSizeArray(SimTypeChar(label="Byte"), 16)}, name="<anon>", label="None")}, name="IPSEC_TRAFFIC_SELECTOR0_", pack=False, align=None), offset=0)}, name="IPSEC_TRAFFIC_SELECTOR_POLICY0_", pack=False, align=None), offset=0)}, name="IPSEC_TUNNEL_POLICY3_", pack=False, align=None), offset=0), "authipQmTransportPolicy": SimTypePointer(SimStruct({"numIpsecProposals": SimTypeInt(signed=False, label="UInt32"), "ipsecProposals": SimTypePointer(SimStruct({"lifetime": SimStruct({"lifetimeSeconds": SimTypeInt(signed=False, label="UInt32"), "lifetimeKilobytes": SimTypeInt(signed=False, label="UInt32"), "lifetimePackets": SimTypeInt(signed=False, label="UInt32")}, name="IPSEC_SA_LIFETIME0", pack=False, align=None), "numSaTransforms": SimTypeInt(signed=False, label="UInt32"), "saTransforms": SimTypePointer(SimStruct({"ipsecTransformType": SimTypeInt(signed=False, label="IPSEC_TRANSFORM_TYPE"), "Anonymous": SimUnion({"ahTransform": SimTypePointer(SimStruct({"authTransformId": SimStruct({"authType": SimTypeInt(signed=False, label="IPSEC_AUTH_TYPE"), "authConfig": SimTypeChar(label="Byte")}, name="IPSEC_AUTH_TRANSFORM_ID0", pack=False, align=None), "cryptoModuleId": SimTypePointer(SimTypeBottom(label="Guid"), offset=0)}, name="IPSEC_AUTH_TRANSFORM0", pack=False, align=None), offset=0), "espAuthTransform": SimTypePointer(SimStruct({"authTransformId": SimStruct({"authType": SimTypeInt(signed=False, label="IPSEC_AUTH_TYPE"), "authConfig": SimTypeChar(label="Byte")}, name="IPSEC_AUTH_TRANSFORM_ID0", pack=False, align=None), "cryptoModuleId": SimTypePointer(SimTypeBottom(label="Guid"), offset=0)}, name="IPSEC_AUTH_TRANSFORM0", pack=False, align=None), offset=0), "espCipherTransform": SimTypePointer(SimStruct({"cipherTransformId": SimStruct({"cipherType": SimTypeInt(signed=False, label="IPSEC_CIPHER_TYPE"), "cipherConfig": SimTypeChar(label="Byte")}, name="IPSEC_CIPHER_TRANSFORM_ID0", pack=False, align=None), "cryptoModuleId": SimTypePointer(SimTypeBottom(label="Guid"), offset=0)}, name="IPSEC_CIPHER_TRANSFORM0", pack=False, align=None), offset=0), "espAuthAndCipherTransform": SimTypePointer(SimStruct({"authTransform": SimStruct({"authTransformId": SimStruct({"authType": SimTypeInt(signed=False, label="IPSEC_AUTH_TYPE"), "authConfig": SimTypeChar(label="Byte")}, name="IPSEC_AUTH_TRANSFORM_ID0", pack=False, align=None), "cryptoModuleId": SimTypePointer(SimTypeBottom(label="Guid"), offset=0)}, name="IPSEC_AUTH_TRANSFORM0", pack=False, align=None), "cipherTransform": SimStruct({"cipherTransformId": SimStruct({"cipherType": SimTypeInt(signed=False, label="IPSEC_CIPHER_TYPE"), "cipherConfig": SimTypeChar(label="Byte")}, name="IPSEC_CIPHER_TRANSFORM_ID0", pack=False, align=None), "cryptoModuleId": SimTypePointer(SimTypeBottom(label="Guid"), offset=0)}, name="IPSEC_CIPHER_TRANSFORM0", pack=False, align=None)}, name="IPSEC_AUTH_AND_CIPHER_TRANSFORM0", pack=False, align=None), offset=0), "espAuthFwTransform": SimTypePointer(SimStruct({"authTransformId": SimStruct({"authType": SimTypeInt(signed=False, label="IPSEC_AUTH_TYPE"), "authConfig": SimTypeChar(label="Byte")}, name="IPSEC_AUTH_TRANSFORM_ID0", pack=False, align=None), "cryptoModuleId": SimTypePointer(SimTypeBottom(label="Guid"), offset=0)}, name="IPSEC_AUTH_TRANSFORM0", pack=False, align=None), offset=0)}, name="<anon>", label="None")}, name="IPSEC_SA_TRANSFORM0", pack=False, align=None), offset=0), "pfsGroup": SimTypeInt(signed=False, label="IPSEC_PFS_GROUP")}, name="IPSEC_PROPOSAL0", pack=False, align=None), offset=0), "flags": SimTypeInt(signed=False, label="IPSEC_POLICY_FLAG"), "ndAllowClearTimeoutSeconds": SimTypeInt(signed=False, label="UInt32"), "saIdleTimeout": SimStruct({"idleTimeoutSeconds": SimTypeInt(signed=False, label="UInt32"), "idleTimeoutSecondsFailOver": SimTypeInt(signed=False, label="UInt32")}, name="IPSEC_SA_IDLE_TIMEOUT0", pack=False, align=None), "emPolicy": SimTypePointer(SimStruct({"numAuthenticationMethods": SimTypeInt(signed=False, label="UInt32"), "authenticationMethods": SimTypePointer(SimStruct({"authenticationMethodType": SimTypeInt(signed=False, label="IKEEXT_AUTHENTICATION_METHOD_TYPE"), "Anonymous": SimUnion({"presharedKeyAuthentication": SimStruct({"presharedKey": SimStruct({"size": SimTypeInt(signed=False, label="UInt32"), "data": SimTypePointer(SimTypeChar(label="Byte"), offset=0)}, name="FWP_BYTE_BLOB", pack=False, align=None), "flags": SimTypeInt(signed=False, label="IKEEXT_PRESHARED_KEY_AUTHENTICATION_FLAGS")}, name="IKEEXT_PRESHARED_KEY_AUTHENTICATION1", pack=False, align=None), "certificateAuthentication": SimStruct({"inboundConfigType": SimTypeInt(signed=False, label="IKEEXT_CERT_CONFIG_TYPE"), "Anonymous1": SimUnion({"Anonymous1": SimStruct({"inboundRootArraySize": SimTypeInt(signed=False, label="UInt32"), "inboundRootCriteria": SimTypePointer(SimStruct({"certData": SimStruct({"size": SimTypeInt(signed=False, label="UInt32"), "data": SimTypePointer(SimTypeChar(label="Byte"), offset=0)}, name="FWP_BYTE_BLOB", pack=False, align=None), "certHash": SimStruct({"size": SimTypeInt(signed=False, label="UInt32"), "data": SimTypePointer(SimTypeChar(label="Byte"), offset=0)}, name="FWP_BYTE_BLOB", pack=False, align=None), "eku": SimTypePointer(SimStruct({"numEku": SimTypeInt(signed=False, label="UInt32"), "eku": SimTypePointer(SimTypePointer(SimTypeChar(label="Byte"), offset=0), offset=0)}, name="IKEEXT_CERT_EKUS0", pack=False, align=None), offset=0), "name": SimTypePointer(SimStruct({"nameType": SimTypeInt(signed=False, label="IKEEXT_CERT_CRITERIA_NAME_TYPE"), "certName": SimTypePointer(SimTypeChar(label="Char"), offset=0)}, name="IKEEXT_CERT_NAME0", pack=False, align=None), offset=0), "flags": SimTypeInt(signed=False, label="UInt32")}, name="IKEEXT_CERTIFICATE_CRITERIA0", pack=False, align=None), offset=0)}, name="_Anonymous1_e__Struct", pack=False, align=None), "Anonymous2": SimStruct({"inboundEnterpriseStoreArraySize": SimTypeInt(signed=False, label="UInt32"), "inboundEnterpriseStoreCriteria": SimTypePointer(SimStruct({"certData": SimStruct({"size": SimTypeInt(signed=False, label="UInt32"), "data": SimTypePointer(SimTypeChar(label="Byte"), offset=0)}, name="FWP_BYTE_BLOB", pack=False, align=None), "certHash": SimStruct({"size": SimTypeInt(signed=False, label="UInt32"), "data": SimTypePointer(SimTypeChar(label="Byte"), offset=0)}, name="FWP_BYTE_BLOB", pack=False, align=None), "eku": SimTypePointer(SimStruct({"numEku": SimTypeInt(signed=False, label="UInt32"), "eku": SimTypePointer(SimTypePointer(SimTypeChar(label="Byte"), offset=0), offset=0)}, name="IKEEXT_CERT_EKUS0", pack=False, align=None), offset=0), "name": SimTypePointer(SimStruct({"nameType": SimTypeInt(signed=False, label="IKEEXT_CERT_CRITERIA_NAME_TYPE"), "certName": SimTypePointer(SimTypeChar(label="Char"), offset=0)}, name="IKEEXT_CERT_NAME0", pack=False, align=None), offset=0), "flags": SimTypeInt(signed=False, label="UInt32")}, name="IKEEXT_CERTIFICATE_CRITERIA0", pack=False, align=None), offset=0)}, name="_Anonymous2_e__Struct", pack=False, align=None), "Anonymous3": SimStruct({"inboundRootStoreArraySize": SimTypeInt(signed=False, label="UInt32"), "inboundTrustedRootStoreCriteria": SimTypePointer(SimStruct({"certData": SimStruct({"size": SimTypeInt(signed=False, label="UInt32"), "data": SimTypePointer(SimTypeChar(label="Byte"), offset=0)}, name="FWP_BYTE_BLOB", pack=False, align=None), "certHash": SimStruct({"size": SimTypeInt(signed=False, label="UInt32"), "data": SimTypePointer(SimTypeChar(label="Byte"), offset=0)}, name="FWP_BYTE_BLOB", pack=False, align=None), "eku": SimTypePointer(SimStruct({"numEku": SimTypeInt(signed=False, label="UInt32"), "eku": SimTypePointer(SimTypePointer(SimTypeChar(label="Byte"), offset=0), offset=0)}, name="IKEEXT_CERT_EKUS0", pack=False, align=None), offset=0), "name": SimTypePointer(SimStruct({"nameType": SimTypeInt(signed=False, label="IKEEXT_CERT_CRITERIA_NAME_TYPE"), "certName": SimTypePointer(SimTypeChar(label="Char"), offset=0)}, name="IKEEXT_CERT_NAME0", pack=False, align=None), offset=0), "flags": SimTypeInt(signed=False, label="UInt32")}, name="IKEEXT_CERTIFICATE_CRITERIA0", pack=False, align=None), offset=0)}, name="_Anonymous3_e__Struct", pack=False, align=None)}, name="<anon>", label="None"), "outboundConfigType": SimTypeInt(signed=False, label="IKEEXT_CERT_CONFIG_TYPE"), "Anonymous2": SimUnion({"Anonymous1": SimStruct({"outboundRootArraySize": SimTypeInt(signed=False, label="UInt32"), "outboundRootCriteria": SimTypePointer(SimStruct({"certData": SimStruct({"size": SimTypeInt(signed=False, label="UInt32"), "data": SimTypePointer(SimTypeChar(label="Byte"), offset=0)}, name="FWP_BYTE_BLOB", pack=False, align=None), "certHash": SimStruct({"size": SimTypeInt(signed=False, label="UInt32"), "data": SimTypePointer(SimTypeChar(label="Byte"), offset=0)}, name="FWP_BYTE_BLOB", pack=False, align=None), "eku": SimTypePointer(SimStruct({"numEku": SimTypeInt(signed=False, label="UInt32"), "eku": SimTypePointer(SimTypePointer(SimTypeChar(label="Byte"), offset=0), offset=0)}, name="IKEEXT_CERT_EKUS0", pack=False, align=None), offset=0), "name": SimTypePointer(SimStruct({"nameType": SimTypeInt(signed=False, label="IKEEXT_CERT_CRITERIA_NAME_TYPE"), "certName": SimTypePointer(SimTypeChar(label="Char"), offset=0)}, name="IKEEXT_CERT_NAME0", pack=False, align=None), offset=0), "flags": SimTypeInt(signed=False, label="UInt32")}, name="IKEEXT_CERTIFICATE_CRITERIA0", pack=False, align=None), offset=0)}, name="_Anonymous1_e__Struct", pack=False, align=None), "Anonymous2": SimStruct({"outboundEnterpriseStoreArraySize": SimTypeInt(signed=False, label="UInt32"), "outboundEnterpriseStoreCriteria": SimTypePointer(SimStruct({"certData": SimStruct({"size": SimTypeInt(signed=False, label="UInt32"), "data": SimTypePointer(SimTypeChar(label="Byte"), offset=0)}, name="FWP_BYTE_BLOB", pack=False, align=None), "certHash": SimStruct({"size": SimTypeInt(signed=False, label="UInt32"), "data": SimTypePointer(SimTypeChar(label="Byte"), offset=0)}, name="FWP_BYTE_BLOB", pack=False, align=None), "eku": SimTypePointer(SimStruct({"numEku": SimTypeInt(signed=False, label="UInt32"), "eku": SimTypePointer(SimTypePointer(SimTypeChar(label="Byte"), offset=0), offset=0)}, name="IKEEXT_CERT_EKUS0", pack=False, align=None), offset=0), "name": SimTypePointer(SimStruct({"nameType": SimTypeInt(signed=False, label="IKEEXT_CERT_CRITERIA_NAME_TYPE"), "certName": SimTypePointer(SimTypeChar(label="Char"), offset=0)}, name="IKEEXT_CERT_NAME0", pack=False, align=None), offset=0), "flags": SimTypeInt(signed=False, label="UInt32")}, name="IKEEXT_CERTIFICATE_CRITERIA0", pack=False, align=None), offset=0)}, name="_Anonymous2_e__Struct", pack=False, align=None), "Anonymous3": SimStruct({"outboundRootStoreArraySize": SimTypeInt(signed=False, label="UInt32"), "outboundTrustedRootStoreCriteria": SimTypePointer(SimStruct({"certData": SimStruct({"size": SimTypeInt(signed=False, label="UInt32"), "data": SimTypePointer(SimTypeChar(label="Byte"), offset=0)}, name="FWP_BYTE_BLOB", pack=False, align=None), "certHash": SimStruct({"size": SimTypeInt(signed=False, label="UInt32"), "data": SimTypePointer(SimTypeChar(label="Byte"), offset=0)}, name="FWP_BYTE_BLOB", pack=False, align=None), "eku": SimTypePointer(SimStruct({"numEku": SimTypeInt(signed=False, label="UInt32"), "eku": SimTypePointer(SimTypePointer(SimTypeChar(label="Byte"), offset=0), offset=0)}, name="IKEEXT_CERT_EKUS0", pack=False, align=None), offset=0), "name": SimTypePointer(SimStruct({"nameType": SimTypeInt(signed=False, label="IKEEXT_CERT_CRITERIA_NAME_TYPE"), "certName": SimTypePointer(SimTypeChar(label="Char"), offset=0)}, name="IKEEXT_CERT_NAME0", pack=False, align=None), offset=0), "flags": SimTypeInt(signed=False, label="UInt32")}, name="IKEEXT_CERTIFICATE_CRITERIA0", pack=False, align=None), offset=0)}, name="_Anonymous3_e__Struct", pack=False, align=None)}, name="<anon>", label="None"), "flags": SimTypeInt(signed=False, label="IKEEXT_CERT_AUTH"), "localCertLocationUrl": SimStruct({"size": SimTypeInt(signed=False, label="UInt32"), "data": SimTypePointer(SimTypeChar(label="Byte"), offset=0)}, name="FWP_BYTE_BLOB", pack=False, align=None)}, name="IKEEXT_CERTIFICATE_AUTHENTICATION2", pack=False, align=None), "kerberosAuthentication": SimStruct({"flags": SimTypeInt(signed=False, label="IKEEXT_KERBEROS_AUTHENTICATION_FLAGS"), "proxyServer": SimTypePointer(SimTypeChar(label="Char"), offset=0)}, name="IKEEXT_KERBEROS_AUTHENTICATION1", pack=False, align=None), "reservedAuthentication": SimStruct({"flags": SimTypeInt(signed=False, label="IKEEXT_RESERVED_AUTHENTICATION_FLAGS")}, name="IKEEXT_RESERVED_AUTHENTICATION0", pack=False, align=None), "ntlmV2Authentication": SimStruct({"flags": SimTypeInt(signed=False, label="UInt32")}, name="IKEEXT_NTLM_V2_AUTHENTICATION0", pack=False, align=None), "sslAuthentication": SimStruct({"inboundConfigType": SimTypeInt(signed=False, label="IKEEXT_CERT_CONFIG_TYPE"), "Anonymous1": SimUnion({"Anonymous1": SimStruct({"inboundRootArraySize": SimTypeInt(signed=False, label="UInt32"), "inboundRootCriteria": SimTypePointer(SimStruct({"certData": SimStruct({"size": SimTypeInt(signed=False, label="UInt32"), "data": SimTypePointer(SimTypeChar(label="Byte"), offset=0)}, name="FWP_BYTE_BLOB", pack=False, align=None), "certHash": SimStruct({"size": SimTypeInt(signed=False, label="UInt32"), "data": SimTypePointer(SimTypeChar(label="Byte"), offset=0)}, name="FWP_BYTE_BLOB", pack=False, align=None), "eku": SimTypePointer(SimStruct({"numEku": SimTypeInt(signed=False, label="UInt32"), "eku": SimTypePointer(SimTypePointer(SimTypeChar(label="Byte"), offset=0), offset=0)}, name="IKEEXT_CERT_EKUS0", pack=False, align=None), offset=0), "name": SimTypePointer(SimStruct({"nameType": SimTypeInt(signed=False, label="IKEEXT_CERT_CRITERIA_NAME_TYPE"), "certName": SimTypePointer(SimTypeChar(label="Char"), offset=0)}, name="IKEEXT_CERT_NAME0", pack=False, align=None), offset=0), "flags": SimTypeInt(signed=False, label="UInt32")}, name="IKEEXT_CERTIFICATE_CRITERIA0", pack=False, align=None), offset=0)}, name="_Anonymous1_e__Struct", pack=False, align=None), "Anonymous2": SimStruct({"inboundEnterpriseStoreArraySize": SimTypeInt(signed=False, label="UInt32"), "inboundEnterpriseStoreCriteria": SimTypePointer(SimStruct({"certData": SimStruct({"size": SimTypeInt(signed=False, label="UInt32"), "data": SimTypePointer(SimTypeChar(label="Byte"), offset=0)}, name="FWP_BYTE_BLOB", pack=False, align=None), "certHash": SimStruct({"size": SimTypeInt(signed=False, label="UInt32"), "data": SimTypePointer(SimTypeChar(label="Byte"), offset=0)}, name="FWP_BYTE_BLOB", pack=False, align=None), "eku": SimTypePointer(SimStruct({"numEku": SimTypeInt(signed=False, label="UInt32"), "eku": SimTypePointer(SimTypePointer(SimTypeChar(label="Byte"), offset=0), offset=0)}, name="IKEEXT_CERT_EKUS0", pack=False, align=None), offset=0), "name": SimTypePointer(SimStruct({"nameType": SimTypeInt(signed=False, label="IKEEXT_CERT_CRITERIA_NAME_TYPE"), "certName": SimTypePointer(SimTypeChar(label="Char"), offset=0)}, name="IKEEXT_CERT_NAME0", pack=False, align=None), offset=0), "flags": SimTypeInt(signed=False, label="UInt32")}, name="IKEEXT_CERTIFICATE_CRITERIA0", pack=False, align=None), offset=0)}, name="_Anonymous2_e__Struct", pack=False, align=None), "Anonymous3": SimStruct({"inboundRootStoreArraySize": SimTypeInt(signed=False, label="UInt32"), "inboundTrustedRootStoreCriteria": SimTypePointer(SimStruct({"certData": SimStruct({"size": SimTypeInt(signed=False, label="UInt32"), "data": SimTypePointer(SimTypeChar(label="Byte"), offset=0)}, name="FWP_BYTE_BLOB", pack=False, align=None), "certHash": SimStruct({"size": SimTypeInt(signed=False, label="UInt32"), "data": SimTypePointer(SimTypeChar(label="Byte"), offset=0)}, name="FWP_BYTE_BLOB", pack=False, align=None), "eku": SimTypePointer(SimStruct({"numEku": SimTypeInt(signed=False, label="UInt32"), "eku": SimTypePointer(SimTypePointer(SimTypeChar(label="Byte"), offset=0), offset=0)}, name="IKEEXT_CERT_EKUS0", pack=False, align=None), offset=0), "name": SimTypePointer(SimStruct({"nameType": SimTypeInt(signed=False, label="IKEEXT_CERT_CRITERIA_NAME_TYPE"), "certName": SimTypePointer(SimTypeChar(label="Char"), offset=0)}, name="IKEEXT_CERT_NAME0", pack=False, align=None), offset=0), "flags": SimTypeInt(signed=False, label="UInt32")}, name="IKEEXT_CERTIFICATE_CRITERIA0", pack=False, align=None), offset=0)}, name="_Anonymous3_e__Struct", pack=False, align=None)}, name="<anon>", label="None"), "outboundConfigType": SimTypeInt(signed=False, label="IKEEXT_CERT_CONFIG_TYPE"), "Anonymous2": SimUnion({"Anonymous1": SimStruct({"outboundRootArraySize": SimTypeInt(signed=False, label="UInt32"), "outboundRootCriteria": SimTypePointer(SimStruct({"certData": SimStruct({"size": SimTypeInt(signed=False, label="UInt32"), "data": SimTypePointer(SimTypeChar(label="Byte"), offset=0)}, name="FWP_BYTE_BLOB", pack=False, align=None), "certHash": SimStruct({"size": SimTypeInt(signed=False, label="UInt32"), "data": SimTypePointer(SimTypeChar(label="Byte"), offset=0)}, name="FWP_BYTE_BLOB", pack=False, align=None), "eku": SimTypePointer(SimStruct({"numEku": SimTypeInt(signed=False, label="UInt32"), "eku": SimTypePointer(SimTypePointer(SimTypeChar(label="Byte"), offset=0), offset=0)}, name="IKEEXT_CERT_EKUS0", pack=False, align=None), offset=0), "name": SimTypePointer(SimStruct({"nameType": SimTypeInt(signed=False, label="IKEEXT_CERT_CRITERIA_NAME_TYPE"), "certName": SimTypePointer(SimTypeChar(label="Char"), offset=0)}, name="IKEEXT_CERT_NAME0", pack=False, align=None), offset=0), "flags": SimTypeInt(signed=False, label="UInt32")}, name="IKEEXT_CERTIFICATE_CRITERIA0", pack=False, align=None), offset=0)}, name="_Anonymous1_e__Struct", pack=False, align=None), "Anonymous2": SimStruct({"outboundEnterpriseStoreArraySize": SimTypeInt(signed=False, label="UInt32"), "outboundEnterpriseStoreCriteria": SimTypePointer(SimStruct({"certData": SimStruct({"size": SimTypeInt(signed=False, label="UInt32"), "data": SimTypePointer(SimTypeChar(label="Byte"), offset=0)}, name="FWP_BYTE_BLOB", pack=False, align=None), "certHash": SimStruct({"size": SimTypeInt(signed=False, label="UInt32"), "data": SimTypePointer(SimTypeChar(label="Byte"), offset=0)}, name="FWP_BYTE_BLOB", pack=False, align=None), "eku": SimTypePointer(SimStruct({"numEku": SimTypeInt(signed=False, label="UInt32"), "eku": SimTypePointer(SimTypePointer(SimTypeChar(label="Byte"), offset=0), offset=0)}, name="IKEEXT_CERT_EKUS0", pack=False, align=None), offset=0), "name": SimTypePointer(SimStruct({"nameType": SimTypeInt(signed=False, label="IKEEXT_CERT_CRITERIA_NAME_TYPE"), "certName": SimTypePointer(SimTypeChar(label="Char"), offset=0)}, name="IKEEXT_CERT_NAME0", pack=False, align=None), offset=0), "flags": SimTypeInt(signed=False, label="UInt32")}, name="IKEEXT_CERTIFICATE_CRITERIA0", pack=False, align=None), offset=0)}, name="_Anonymous2_e__Struct", pack=False, align=None), "Anonymous3": SimStruct({"outboundRootStoreArraySize": SimTypeInt(signed=False, label="UInt32"), "outboundTrustedRootStoreCriteria": SimTypePointer(SimStruct({"certData": SimStruct({"size": SimTypeInt(signed=False, label="UInt32"), "data": SimTypePointer(SimTypeChar(label="Byte"), offset=0)}, name="FWP_BYTE_BLOB", pack=False, align=None), "certHash": SimStruct({"size": SimTypeInt(signed=False, label="UInt32"), "data": SimTypePointer(SimTypeChar(label="Byte"), offset=0)}, name="FWP_BYTE_BLOB", pack=False, align=None), "eku": SimTypePointer(SimStruct({"numEku": SimTypeInt(signed=False, label="UInt32"), "eku": SimTypePointer(SimTypePointer(SimTypeChar(label="Byte"), offset=0), offset=0)}, name="IKEEXT_CERT_EKUS0", pack=False, align=None), offset=0), "name": SimTypePointer(SimStruct({"nameType": SimTypeInt(signed=False, label="IKEEXT_CERT_CRITERIA_NAME_TYPE"), "certName": SimTypePointer(SimTypeChar(label="Char"), offset=0)}, name="IKEEXT_CERT_NAME0", pack=False, align=None), offset=0), "flags": SimTypeInt(signed=False, label="UInt32")}, name="IKEEXT_CERTIFICATE_CRITERIA0", pack=False, align=None), offset=0)}, name="_Anonymous3_e__Struct", pack=False, align=None)}, name="<anon>", label="None"), "flags": SimTypeInt(signed=False, label="IKEEXT_CERT_AUTH"), "localCertLocationUrl": SimStruct({"size": SimTypeInt(signed=False, label="UInt32"), "data": SimTypePointer(SimTypeChar(label="Byte"), offset=0)}, name="FWP_BYTE_BLOB", pack=False, align=None)}, name="IKEEXT_CERTIFICATE_AUTHENTICATION2", pack=False, align=None), "cgaAuthentication": SimStruct({"keyContainerName": SimTypePointer(SimTypeChar(label="Char"), offset=0), "cspName": SimTypePointer(SimTypeChar(label="Char"), offset=0), "cspType": SimTypeInt(signed=False, label="UInt32"), "cgaModifier": SimStruct({"byteArray16": SimTypeFixedSizeArray(SimTypeChar(label="Byte"), 16)}, name="FWP_BYTE_ARRAY16", pack=False, align=None), "cgaCollisionCount": SimTypeChar(label="Byte")}, name="IKEEXT_IPV6_CGA_AUTHENTICATION0", pack=False, align=None), "eapAuthentication": SimStruct({"flags": SimTypeInt(signed=False, label="IKEEXT_EAP_AUTHENTICATION_FLAGS")}, name="IKEEXT_EAP_AUTHENTICATION0", pack=False, align=None)}, name="<anon>", label="None")}, name="IKEEXT_AUTHENTICATION_METHOD2", pack=False, align=None), offset=0), "initiatorImpersonationType": SimTypeInt(signed=False, label="IKEEXT_AUTHENTICATION_IMPERSONATION_TYPE")}, name="IKEEXT_EM_POLICY2", pack=False, align=None), offset=0)}, name="IPSEC_TRANSPORT_POLICY2", pack=False, align=None), offset=0), "authipQmTunnelPolicy": SimTypePointer(SimStruct({"flags": SimTypeInt(signed=False, label="UInt32"), "numIpsecProposals": SimTypeInt(signed=False, label="UInt32"), "ipsecProposals": SimTypePointer(SimStruct({"lifetime": SimStruct({"lifetimeSeconds": SimTypeInt(signed=False, label="UInt32"), "lifetimeKilobytes": SimTypeInt(signed=False, label="UInt32"), "lifetimePackets": SimTypeInt(signed=False, label="UInt32")}, name="IPSEC_SA_LIFETIME0", pack=False, align=None), "numSaTransforms": SimTypeInt(signed=False, label="UInt32"), "saTransforms": SimTypePointer(SimStruct({"ipsecTransformType": SimTypeInt(signed=False, label="IPSEC_TRANSFORM_TYPE"), "Anonymous": SimUnion({"ahTransform": SimTypePointer(SimStruct({"authTransformId": SimStruct({"authType": SimTypeInt(signed=False, label="IPSEC_AUTH_TYPE"), "authConfig": SimTypeChar(label="Byte")}, name="IPSEC_AUTH_TRANSFORM_ID0", pack=False, align=None), "cryptoModuleId": SimTypePointer(SimTypeBottom(label="Guid"), offset=0)}, name="IPSEC_AUTH_TRANSFORM0", pack=False, align=None), offset=0), "espAuthTransform": SimTypePointer(SimStruct({"authTransformId": SimStruct({"authType": SimTypeInt(signed=False, label="IPSEC_AUTH_TYPE"), "authConfig": SimTypeChar(label="Byte")}, name="IPSEC_AUTH_TRANSFORM_ID0", pack=False, align=None), "cryptoModuleId": SimTypePointer(SimTypeBottom(label="Guid"), offset=0)}, name="IPSEC_AUTH_TRANSFORM0", pack=False, align=None), offset=0), "espCipherTransform": SimTypePointer(SimStruct({"cipherTransformId": SimStruct({"cipherType": SimTypeInt(signed=False, label="IPSEC_CIPHER_TYPE"), "cipherConfig": SimTypeChar(label="Byte")}, name="IPSEC_CIPHER_TRANSFORM_ID0", pack=False, align=None), "cryptoModuleId": SimTypePointer(SimTypeBottom(label="Guid"), offset=0)}, name="IPSEC_CIPHER_TRANSFORM0", pack=False, align=None), offset=0), "espAuthAndCipherTransform": SimTypePointer(SimStruct({"authTransform": SimStruct({"authTransformId": SimStruct({"authType": SimTypeInt(signed=False, label="IPSEC_AUTH_TYPE"), "authConfig": SimTypeChar(label="Byte")}, name="IPSEC_AUTH_TRANSFORM_ID0", pack=False, align=None), "cryptoModuleId": SimTypePointer(SimTypeBottom(label="Guid"), offset=0)}, name="IPSEC_AUTH_TRANSFORM0", pack=False, align=None), "cipherTransform": SimStruct({"cipherTransformId": SimStruct({"cipherType": SimTypeInt(signed=False, label="IPSEC_CIPHER_TYPE"), "cipherConfig": SimTypeChar(label="Byte")}, name="IPSEC_CIPHER_TRANSFORM_ID0", pack=False, align=None), "cryptoModuleId": SimTypePointer(SimTypeBottom(label="Guid"), offset=0)}, name="IPSEC_CIPHER_TRANSFORM0", pack=False, align=None)}, name="IPSEC_AUTH_AND_CIPHER_TRANSFORM0", pack=False, align=None), offset=0), "espAuthFwTransform": SimTypePointer(SimStruct({"authTransformId": SimStruct({"authType": SimTypeInt(signed=False, label="IPSEC_AUTH_TYPE"), "authConfig": SimTypeChar(label="Byte")}, name="IPSEC_AUTH_TRANSFORM_ID0", pack=False, align=None), "cryptoModuleId": SimTypePointer(SimTypeBottom(label="Guid"), offset=0)}, name="IPSEC_AUTH_TRANSFORM0", pack=False, align=None), offset=0)}, name="<anon>", label="None")}, name="IPSEC_SA_TRANSFORM0", pack=False, align=None), offset=0), "pfsGroup": SimTypeInt(signed=False, label="IPSEC_PFS_GROUP")}, name="IPSEC_PROPOSAL0", pack=False, align=None), offset=0), "tunnelEndpoints": SimStruct({"ipVersion": SimTypeInt(signed=False, label="FWP_IP_VERSION"), "Anonymous1": SimUnion({"localV4Address": SimTypeInt(signed=False, label="UInt32"), "localV6Address": SimTypeFixedSizeArray(SimTypeChar(label="Byte"), 16)}, name="<anon>", label="None"), "Anonymous2": SimUnion({"remoteV4Address": SimTypeInt(signed=False, label="UInt32"), "remoteV6Address": SimTypeFixedSizeArray(SimTypeChar(label="Byte"), 16)}, name="<anon>", label="None"), "localIfLuid": SimTypeLongLong(signed=False, label="UInt64"), "remoteFqdn": SimTypePointer(SimTypeChar(label="Char"), offset=0), "numAddresses": SimTypeInt(signed=False, label="UInt32"), "remoteAddresses": SimTypePointer(SimStruct({"ipVersion": SimTypeInt(signed=False, label="FWP_IP_VERSION"), "Anonymous": SimUnion({"v4Address": SimTypeInt(signed=False, label="UInt32"), "v6Address": SimTypeFixedSizeArray(SimTypeChar(label="Byte"), 16)}, name="<anon>", label="None")}, name="IPSEC_TUNNEL_ENDPOINT0", pack=False, align=None), offset=0)}, name="IPSEC_TUNNEL_ENDPOINTS2", pack=False, align=None), "saIdleTimeout": SimStruct({"idleTimeoutSeconds": SimTypeInt(signed=False, label="UInt32"), "idleTimeoutSecondsFailOver": SimTypeInt(signed=False, label="UInt32")}, name="IPSEC_SA_IDLE_TIMEOUT0", pack=False, align=None), "emPolicy": SimTypePointer(SimStruct({"numAuthenticationMethods": SimTypeInt(signed=False, label="UInt32"), "authenticationMethods": SimTypePointer(SimStruct({"authenticationMethodType": SimTypeInt(signed=False, label="IKEEXT_AUTHENTICATION_METHOD_TYPE"), "Anonymous": SimUnion({"presharedKeyAuthentication": SimStruct({"presharedKey": SimStruct({"size": SimTypeInt(signed=False, label="UInt32"), "data": SimTypePointer(SimTypeChar(label="Byte"), offset=0)}, name="FWP_BYTE_BLOB", pack=False, align=None), "flags": SimTypeInt(signed=False, label="IKEEXT_PRESHARED_KEY_AUTHENTICATION_FLAGS")}, name="IKEEXT_PRESHARED_KEY_AUTHENTICATION1", pack=False, align=None), "certificateAuthentication": SimStruct({"inboundConfigType": SimTypeInt(signed=False, label="IKEEXT_CERT_CONFIG_TYPE"), "Anonymous1": SimUnion({"Anonymous1": SimStruct({"inboundRootArraySize": SimTypeInt(signed=False, label="UInt32"), "inboundRootCriteria": SimTypePointer(SimStruct({"certData": SimStruct({"size": SimTypeInt(signed=False, label="UInt32"), "data": SimTypePointer(SimTypeChar(label="Byte"), offset=0)}, name="FWP_BYTE_BLOB", pack=False, align=None), "certHash": SimStruct({"size": SimTypeInt(signed=False, label="UInt32"), "data": SimTypePointer(SimTypeChar(label="Byte"), offset=0)}, name="FWP_BYTE_BLOB", pack=False, align=None), "eku": SimTypePointer(SimStruct({"numEku": SimTypeInt(signed=False, label="UInt32"), "eku": SimTypePointer(SimTypePointer(SimTypeChar(label="Byte"), offset=0), offset=0)}, name="IKEEXT_CERT_EKUS0", pack=False, align=None), offset=0), "name": SimTypePointer(SimStruct({"nameType": SimTypeInt(signed=False, label="IKEEXT_CERT_CRITERIA_NAME_TYPE"), "certName": SimTypePointer(SimTypeChar(label="Char"), offset=0)}, name="IKEEXT_CERT_NAME0", pack=False, align=None), offset=0), "flags": SimTypeInt(signed=False, label="UInt32")}, name="IKEEXT_CERTIFICATE_CRITERIA0", pack=False, align=None), offset=0)}, name="_Anonymous1_e__Struct", pack=False, align=None), "Anonymous2": SimStruct({"inboundEnterpriseStoreArraySize": SimTypeInt(signed=False, label="UInt32"), "inboundEnterpriseStoreCriteria": SimTypePointer(SimStruct({"certData": SimStruct({"size": SimTypeInt(signed=False, label="UInt32"), "data": SimTypePointer(SimTypeChar(label="Byte"), offset=0)}, name="FWP_BYTE_BLOB", pack=False, align=None), "certHash": SimStruct({"size": SimTypeInt(signed=False, label="UInt32"), "data": SimTypePointer(SimTypeChar(label="Byte"), offset=0)}, name="FWP_BYTE_BLOB", pack=False, align=None), "eku": SimTypePointer(SimStruct({"numEku": SimTypeInt(signed=False, label="UInt32"), "eku": SimTypePointer(SimTypePointer(SimTypeChar(label="Byte"), offset=0), offset=0)}, name="IKEEXT_CERT_EKUS0", pack=False, align=None), offset=0), "name": SimTypePointer(SimStruct({"nameType": SimTypeInt(signed=False, label="IKEEXT_CERT_CRITERIA_NAME_TYPE"), "certName": SimTypePointer(SimTypeChar(label="Char"), offset=0)}, name="IKEEXT_CERT_NAME0", pack=False, align=None), offset=0), "flags": SimTypeInt(signed=False, label="UInt32")}, name="IKEEXT_CERTIFICATE_CRITERIA0", pack=False, align=None), offset=0)}, name="_Anonymous2_e__Struct", pack=False, align=None), "Anonymous3": SimStruct({"inboundRootStoreArraySize": SimTypeInt(signed=False, label="UInt32"), "inboundTrustedRootStoreCriteria": SimTypePointer(SimStruct({"certData": SimStruct({"size": SimTypeInt(signed=False, label="UInt32"), "data": SimTypePointer(SimTypeChar(label="Byte"), offset=0)}, name="FWP_BYTE_BLOB", pack=False, align=None), "certHash": SimStruct({"size": SimTypeInt(signed=False, label="UInt32"), "data": SimTypePointer(SimTypeChar(label="Byte"), offset=0)}, name="FWP_BYTE_BLOB", pack=False, align=None), "eku": SimTypePointer(SimStruct({"numEku": SimTypeInt(signed=False, label="UInt32"), "eku": SimTypePointer(SimTypePointer(SimTypeChar(label="Byte"), offset=0), offset=0)}, name="IKEEXT_CERT_EKUS0", pack=False, align=None), offset=0), "name": SimTypePointer(SimStruct({"nameType": SimTypeInt(signed=False, label="IKEEXT_CERT_CRITERIA_NAME_TYPE"), "certName": SimTypePointer(SimTypeChar(label="Char"), offset=0)}, name="IKEEXT_CERT_NAME0", pack=False, align=None), offset=0), "flags": SimTypeInt(signed=False, label="UInt32")}, name="IKEEXT_CERTIFICATE_CRITERIA0", pack=False, align=None), offset=0)}, name="_Anonymous3_e__Struct", pack=False, align=None)}, name="<anon>", label="None"), "outboundConfigType": SimTypeInt(signed=False, label="IKEEXT_CERT_CONFIG_TYPE"), "Anonymous2": SimUnion({"Anonymous1": SimStruct({"outboundRootArraySize": SimTypeInt(signed=False, label="UInt32"), "outboundRootCriteria": SimTypePointer(SimStruct({"certData": SimStruct({"size": SimTypeInt(signed=False, label="UInt32"), "data": SimTypePointer(SimTypeChar(label="Byte"), offset=0)}, name="FWP_BYTE_BLOB", pack=False, align=None), "certHash": SimStruct({"size": SimTypeInt(signed=False, label="UInt32"), "data": SimTypePointer(SimTypeChar(label="Byte"), offset=0)}, name="FWP_BYTE_BLOB", pack=False, align=None), "eku": SimTypePointer(SimStruct({"numEku": SimTypeInt(signed=False, label="UInt32"), "eku": SimTypePointer(SimTypePointer(SimTypeChar(label="Byte"), offset=0), offset=0)}, name="IKEEXT_CERT_EKUS0", pack=False, align=None), offset=0), "name": SimTypePointer(SimStruct({"nameType": SimTypeInt(signed=False, label="IKEEXT_CERT_CRITERIA_NAME_TYPE"), "certName": SimTypePointer(SimTypeChar(label="Char"), offset=0)}, name="IKEEXT_CERT_NAME0", pack=False, align=None), offset=0), "flags": SimTypeInt(signed=False, label="UInt32")}, name="IKEEXT_CERTIFICATE_CRITERIA0", pack=False, align=None), offset=0)}, name="_Anonymous1_e__Struct", pack=False, align=None), "Anonymous2": SimStruct({"outboundEnterpriseStoreArraySize": SimTypeInt(signed=False, label="UInt32"), "outboundEnterpriseStoreCriteria": SimTypePointer(SimStruct({"certData": SimStruct({"size": SimTypeInt(signed=False, label="UInt32"), "data": SimTypePointer(SimTypeChar(label="Byte"), offset=0)}, name="FWP_BYTE_BLOB", pack=False, align=None), "certHash": SimStruct({"size": SimTypeInt(signed=False, label="UInt32"), "data": SimTypePointer(SimTypeChar(label="Byte"), offset=0)}, name="FWP_BYTE_BLOB", pack=False, align=None), "eku": SimTypePointer(SimStruct({"numEku": SimTypeInt(signed=False, label="UInt32"), "eku": SimTypePointer(SimTypePointer(SimTypeChar(label="Byte"), offset=0), offset=0)}, name="IKEEXT_CERT_EKUS0", pack=False, align=None), offset=0), "name": SimTypePointer(SimStruct({"nameType": SimTypeInt(signed=False, label="IKEEXT_CERT_CRITERIA_NAME_TYPE"), "certName": SimTypePointer(SimTypeChar(label="Char"), offset=0)}, name="IKEEXT_CERT_NAME0", pack=False, align=None), offset=0), "flags": SimTypeInt(signed=False, label="UInt32")}, name="IKEEXT_CERTIFICATE_CRITERIA0", pack=False, align=None), offset=0)}, name="_Anonymous2_e__Struct", pack=False, align=None), "Anonymous3": SimStruct({"outboundRootStoreArraySize": SimTypeInt(signed=False, label="UInt32"), "outboundTrustedRootStoreCriteria": SimTypePointer(SimStruct({"certData": SimStruct({"size": SimTypeInt(signed=False, label="UInt32"), "data": SimTypePointer(SimTypeChar(label="Byte"), offset=0)}, name="FWP_BYTE_BLOB", pack=False, align=None), "certHash": SimStruct({"size": SimTypeInt(signed=False, label="UInt32"), "data": SimTypePointer(SimTypeChar(label="Byte"), offset=0)}, name="FWP_BYTE_BLOB", pack=False, align=None), "eku": SimTypePointer(SimStruct({"numEku": SimTypeInt(signed=False, label="UInt32"), "eku": SimTypePointer(SimTypePointer(SimTypeChar(label="Byte"), offset=0), offset=0)}, name="IKEEXT_CERT_EKUS0", pack=False, align=None), offset=0), "name": SimTypePointer(SimStruct({"nameType": SimTypeInt(signed=False, label="IKEEXT_CERT_CRITERIA_NAME_TYPE"), "certName": SimTypePointer(SimTypeChar(label="Char"), offset=0)}, name="IKEEXT_CERT_NAME0", pack=False, align=None), offset=0), "flags": SimTypeInt(signed=False, label="UInt32")}, name="IKEEXT_CERTIFICATE_CRITERIA0", pack=False, align=None), offset=0)}, name="_Anonymous3_e__Struct", pack=False, align=None)}, name="<anon>", label="None"), "flags": SimTypeInt(signed=False, label="IKEEXT_CERT_AUTH"), "localCertLocationUrl": SimStruct({"size": SimTypeInt(signed=False, label="UInt32"), "data": SimTypePointer(SimTypeChar(label="Byte"), offset=0)}, name="FWP_BYTE_BLOB", pack=False, align=None)}, name="IKEEXT_CERTIFICATE_AUTHENTICATION2", pack=False, align=None), "kerberosAuthentication": SimStruct({"flags": SimTypeInt(signed=False, label="IKEEXT_KERBEROS_AUTHENTICATION_FLAGS"), "proxyServer": SimTypePointer(SimTypeChar(label="Char"), offset=0)}, name="IKEEXT_KERBEROS_AUTHENTICATION1", pack=False, align=None), "reservedAuthentication": SimStruct({"flags": SimTypeInt(signed=False, label="IKEEXT_RESERVED_AUTHENTICATION_FLAGS")}, name="IKEEXT_RESERVED_AUTHENTICATION0", pack=False, align=None), "ntlmV2Authentication": SimStruct({"flags": SimTypeInt(signed=False, label="UInt32")}, name="IKEEXT_NTLM_V2_AUTHENTICATION0", pack=False, align=None), "sslAuthentication": SimStruct({"inboundConfigType": SimTypeInt(signed=False, label="IKEEXT_CERT_CONFIG_TYPE"), "Anonymous1": SimUnion({"Anonymous1": SimStruct({"inboundRootArraySize": SimTypeInt(signed=False, label="UInt32"), "inboundRootCriteria": SimTypePointer(SimStruct({"certData": SimStruct({"size": SimTypeInt(signed=False, label="UInt32"), "data": SimTypePointer(SimTypeChar(label="Byte"), offset=0)}, name="FWP_BYTE_BLOB", pack=False, align=None), "certHash": SimStruct({"size": SimTypeInt(signed=False, label="UInt32"), "data": SimTypePointer(SimTypeChar(label="Byte"), offset=0)}, name="FWP_BYTE_BLOB", pack=False, align=None), "eku": SimTypePointer(SimStruct({"numEku": SimTypeInt(signed=False, label="UInt32"), "eku": SimTypePointer(SimTypePointer(SimTypeChar(label="Byte"), offset=0), offset=0)}, name="IKEEXT_CERT_EKUS0", pack=False, align=None), offset=0), "name": SimTypePointer(SimStruct({"nameType": SimTypeInt(signed=False, label="IKEEXT_CERT_CRITERIA_NAME_TYPE"), "certName": SimTypePointer(SimTypeChar(label="Char"), offset=0)}, name="IKEEXT_CERT_NAME0", pack=False, align=None), offset=0), "flags": SimTypeInt(signed=False, label="UInt32")}, name="IKEEXT_CERTIFICATE_CRITERIA0", pack=False, align=None), offset=0)}, name="_Anonymous1_e__Struct", pack=False, align=None), "Anonymous2": SimStruct({"inboundEnterpriseStoreArraySize": SimTypeInt(signed=False, label="UInt32"), "inboundEnterpriseStoreCriteria": SimTypePointer(SimStruct({"certData": SimStruct({"size": SimTypeInt(signed=False, label="UInt32"), "data": SimTypePointer(SimTypeChar(label="Byte"), offset=0)}, name="FWP_BYTE_BLOB", pack=False, align=None), "certHash": SimStruct({"size": SimTypeInt(signed=False, label="UInt32"), "data": SimTypePointer(SimTypeChar(label="Byte"), offset=0)}, name="FWP_BYTE_BLOB", pack=False, align=None), "eku": SimTypePointer(SimStruct({"numEku": SimTypeInt(signed=False, label="UInt32"), "eku": SimTypePointer(SimTypePointer(SimTypeChar(label="Byte"), offset=0), offset=0)}, name="IKEEXT_CERT_EKUS0", pack=False, align=None), offset=0), "name": SimTypePointer(SimStruct({"nameType": SimTypeInt(signed=False, label="IKEEXT_CERT_CRITERIA_NAME_TYPE"), "certName": SimTypePointer(SimTypeChar(label="Char"), offset=0)}, name="IKEEXT_CERT_NAME0", pack=False, align=None), offset=0), "flags": SimTypeInt(signed=False, label="UInt32")}, name="IKEEXT_CERTIFICATE_CRITERIA0", pack=False, align=None), offset=0)}, name="_Anonymous2_e__Struct", pack=False, align=None), "Anonymous3": SimStruct({"inboundRootStoreArraySize": SimTypeInt(signed=False, label="UInt32"), "inboundTrustedRootStoreCriteria": SimTypePointer(SimStruct({"certData": SimStruct({"size": SimTypeInt(signed=False, label="UInt32"), "data": SimTypePointer(SimTypeChar(label="Byte"), offset=0)}, name="FWP_BYTE_BLOB", pack=False, align=None), "certHash": SimStruct({"size": SimTypeInt(signed=False, label="UInt32"), "data": SimTypePointer(SimTypeChar(label="Byte"), offset=0)}, name="FWP_BYTE_BLOB", pack=False, align=None), "eku": SimTypePointer(SimStruct({"numEku": SimTypeInt(signed=False, label="UInt32"), "eku": SimTypePointer(SimTypePointer(SimTypeChar(label="Byte"), offset=0), offset=0)}, name="IKEEXT_CERT_EKUS0", pack=False, align=None), offset=0), "name": SimTypePointer(SimStruct({"nameType": SimTypeInt(signed=False, label="IKEEXT_CERT_CRITERIA_NAME_TYPE"), "certName": SimTypePointer(SimTypeChar(label="Char"), offset=0)}, name="IKEEXT_CERT_NAME0", pack=False, align=None), offset=0), "flags": SimTypeInt(signed=False, label="UInt32")}, name="IKEEXT_CERTIFICATE_CRITERIA0", pack=False, align=None), offset=0)}, name="_Anonymous3_e__Struct", pack=False, align=None)}, name="<anon>", label="None"), "outboundConfigType": SimTypeInt(signed=False, label="IKEEXT_CERT_CONFIG_TYPE"), "Anonymous2": SimUnion({"Anonymous1": SimStruct({"outboundRootArraySize": SimTypeInt(signed=False, label="UInt32"), "outboundRootCriteria": SimTypePointer(SimStruct({"certData": SimStruct({"size": SimTypeInt(signed=False, label="UInt32"), "data": SimTypePointer(SimTypeChar(label="Byte"), offset=0)}, name="FWP_BYTE_BLOB", pack=False, align=None), "certHash": SimStruct({"size": SimTypeInt(signed=False, label="UInt32"), "data": SimTypePointer(SimTypeChar(label="Byte"), offset=0)}, name="FWP_BYTE_BLOB", pack=False, align=None), "eku": SimTypePointer(SimStruct({"numEku": SimTypeInt(signed=False, label="UInt32"), "eku": SimTypePointer(SimTypePointer(SimTypeChar(label="Byte"), offset=0), offset=0)}, name="IKEEXT_CERT_EKUS0", pack=False, align=None), offset=0), "name": SimTypePointer(SimStruct({"nameType": SimTypeInt(signed=False, label="IKEEXT_CERT_CRITERIA_NAME_TYPE"), "certName": SimTypePointer(SimTypeChar(label="Char"), offset=0)}, name="IKEEXT_CERT_NAME0", pack=False, align=None), offset=0), "flags": SimTypeInt(signed=False, label="UInt32")}, name="IKEEXT_CERTIFICATE_CRITERIA0", pack=False, align=None), offset=0)}, name="_Anonymous1_e__Struct", pack=False, align=None), "Anonymous2": SimStruct({"outboundEnterpriseStoreArraySize": SimTypeInt(signed=False, label="UInt32"), "outboundEnterpriseStoreCriteria": SimTypePointer(SimStruct({"certData": SimStruct({"size": SimTypeInt(signed=False, label="UInt32"), "data": SimTypePointer(SimTypeChar(label="Byte"), offset=0)}, name="FWP_BYTE_BLOB", pack=False, align=None), "certHash": SimStruct({"size": SimTypeInt(signed=False, label="UInt32"), "data": SimTypePointer(SimTypeChar(label="Byte"), offset=0)}, name="FWP_BYTE_BLOB", pack=False, align=None), "eku": SimTypePointer(SimStruct({"numEku": SimTypeInt(signed=False, label="UInt32"), "eku": SimTypePointer(SimTypePointer(SimTypeChar(label="Byte"), offset=0), offset=0)}, name="IKEEXT_CERT_EKUS0", pack=False, align=None), offset=0), "name": SimTypePointer(SimStruct({"nameType": SimTypeInt(signed=False, label="IKEEXT_CERT_CRITERIA_NAME_TYPE"), "certName": SimTypePointer(SimTypeChar(label="Char"), offset=0)}, name="IKEEXT_CERT_NAME0", pack=False, align=None), offset=0), "flags": SimTypeInt(signed=False, label="UInt32")}, name="IKEEXT_CERTIFICATE_CRITERIA0", pack=False, align=None), offset=0)}, name="_Anonymous2_e__Struct", pack=False, align=None), "Anonymous3": SimStruct({"outboundRootStoreArraySize": SimTypeInt(signed=False, label="UInt32"), "outboundTrustedRootStoreCriteria": SimTypePointer(SimStruct({"certData": SimStruct({"size": SimTypeInt(signed=False, label="UInt32"), "data": SimTypePointer(SimTypeChar(label="Byte"), offset=0)}, name="FWP_BYTE_BLOB", pack=False, align=None), "certHash": SimStruct({"size": SimTypeInt(signed=False, label="UInt32"), "data": SimTypePointer(SimTypeChar(label="Byte"), offset=0)}, name="FWP_BYTE_BLOB", pack=False, align=None), "eku": SimTypePointer(SimStruct({"numEku": SimTypeInt(signed=False, label="UInt32"), "eku": SimTypePointer(SimTypePointer(SimTypeChar(label="Byte"), offset=0), offset=0)}, name="IKEEXT_CERT_EKUS0", pack=False, align=None), offset=0), "name": SimTypePointer(SimStruct({"nameType": SimTypeInt(signed=False, label="IKEEXT_CERT_CRITERIA_NAME_TYPE"), "certName": SimTypePointer(SimTypeChar(label="Char"), offset=0)}, name="IKEEXT_CERT_NAME0", pack=False, align=None), offset=0), "flags": SimTypeInt(signed=False, label="UInt32")}, name="IKEEXT_CERTIFICATE_CRITERIA0", pack=False, align=None), offset=0)}, name="_Anonymous3_e__Struct", pack=False, align=None)}, name="<anon>", label="None"), "flags": SimTypeInt(signed=False, label="IKEEXT_CERT_AUTH"), "localCertLocationUrl": SimStruct({"size": SimTypeInt(signed=False, label="UInt32"), "data": SimTypePointer(SimTypeChar(label="Byte"), offset=0)}, name="FWP_BYTE_BLOB", pack=False, align=None)}, name="IKEEXT_CERTIFICATE_AUTHENTICATION2", pack=False, align=None), "cgaAuthentication": SimStruct({"keyContainerName": SimTypePointer(SimTypeChar(label="Char"), offset=0), "cspName": SimTypePointer(SimTypeChar(label="Char"), offset=0), "cspType": SimTypeInt(signed=False, label="UInt32"), "cgaModifier": SimStruct({"byteArray16": SimTypeFixedSizeArray(SimTypeChar(label="Byte"), 16)}, name="FWP_BYTE_ARRAY16", pack=False, align=None), "cgaCollisionCount": SimTypeChar(label="Byte")}, name="IKEEXT_IPV6_CGA_AUTHENTICATION0", pack=False, align=None), "eapAuthentication": SimStruct({"flags": SimTypeInt(signed=False, label="IKEEXT_EAP_AUTHENTICATION_FLAGS")}, name="IKEEXT_EAP_AUTHENTICATION0", pack=False, align=None)}, name="<anon>", label="None")}, name="IKEEXT_AUTHENTICATION_METHOD2", pack=False, align=None), offset=0), "initiatorImpersonationType": SimTypeInt(signed=False, label="IKEEXT_AUTHENTICATION_IMPERSONATION_TYPE")}, name="IKEEXT_EM_POLICY2", pack=False, align=None), offset=0), "fwdPathSaLifetime": SimTypeInt(signed=False, label="UInt32"), "compartmentId": SimTypeInt(signed=False, label="UInt32"), "numTrafficSelectorPolicy": SimTypeInt(signed=False, label="UInt32"), "trafficSelectorPolicies": SimTypePointer(SimStruct({"flags": SimTypeInt(signed=False, label="UInt32"), "numLocalTrafficSelectors": SimTypeInt(signed=False, label="UInt32"), "localTrafficSelectors": SimTypePointer(SimStruct({"protocolId": SimTypeChar(label="Byte"), "portStart": SimTypeShort(signed=False, label="UInt16"), "portEnd": SimTypeShort(signed=False, label="UInt16"), "ipVersion": SimTypeInt(signed=False, label="FWP_IP_VERSION"), "Anonymous1": SimUnion({"startV4Address": SimTypeInt(signed=False, label="UInt32"), "startV6Address": SimTypeFixedSizeArray(SimTypeChar(label="Byte"), 16)}, name="<anon>", label="None"), "Anonymous2": SimUnion({"endV4Address": SimTypeInt(signed=False, label="UInt32"), "endV6Address": SimTypeFixedSizeArray(SimTypeChar(label="Byte"), 16)}, name="<anon>", label="None")}, name="IPSEC_TRAFFIC_SELECTOR0_", pack=False, align=None), offset=0), "numRemoteTrafficSelectors": SimTypeInt(signed=False, label="UInt32"), "remoteTrafficSelectors": SimTypePointer(SimStruct({"protocolId": SimTypeChar(label="Byte"), "portStart": SimTypeShort(signed=False, label="UInt16"), "portEnd": SimTypeShort(signed=False, label="UInt16"), "ipVersion": SimTypeInt(signed=False, label="FWP_IP_VERSION"), "Anonymous1": SimUnion({"startV4Address": SimTypeInt(signed=False, label="UInt32"), "startV6Address": SimTypeFixedSizeArray(SimTypeChar(label="Byte"), 16)}, name="<anon>", label="None"), "Anonymous2": SimUnion({"endV4Address": SimTypeInt(signed=False, label="UInt32"), "endV6Address": SimTypeFixedSizeArray(SimTypeChar(label="Byte"), 16)}, name="<anon>", label="None")}, name="IPSEC_TRAFFIC_SELECTOR0_", pack=False, align=None), offset=0)}, name="IPSEC_TRAFFIC_SELECTOR_POLICY0_", pack=False, align=None), offset=0)}, name="IPSEC_TUNNEL_POLICY3_", pack=False, align=None), offset=0), "ikeMmPolicy": SimTypePointer(SimStruct({"softExpirationTime": SimTypeInt(signed=False, label="UInt32"), "numAuthenticationMethods": SimTypeInt(signed=False, label="UInt32"), "authenticationMethods": SimTypePointer(SimStruct({"authenticationMethodType": SimTypeInt(signed=False, label="IKEEXT_AUTHENTICATION_METHOD_TYPE"), "Anonymous": SimUnion({"presharedKeyAuthentication": SimStruct({"presharedKey": SimStruct({"size": SimTypeInt(signed=False, label="UInt32"), "data": SimTypePointer(SimTypeChar(label="Byte"), offset=0)}, name="FWP_BYTE_BLOB", pack=False, align=None), "flags": SimTypeInt(signed=False, label="IKEEXT_PRESHARED_KEY_AUTHENTICATION_FLAGS")}, name="IKEEXT_PRESHARED_KEY_AUTHENTICATION1", pack=False, align=None), "certificateAuthentication": SimStruct({"inboundConfigType": SimTypeInt(signed=False, label="IKEEXT_CERT_CONFIG_TYPE"), "Anonymous1": SimUnion({"Anonymous1": SimStruct({"inboundRootArraySize": SimTypeInt(signed=False, label="UInt32"), "inboundRootCriteria": SimTypePointer(SimStruct({"certData": SimStruct({"size": SimTypeInt(signed=False, label="UInt32"), "data": SimTypePointer(SimTypeChar(label="Byte"), offset=0)}, name="FWP_BYTE_BLOB", pack=False, align=None), "certHash": SimStruct({"size": SimTypeInt(signed=False, label="UInt32"), "data": SimTypePointer(SimTypeChar(label="Byte"), offset=0)}, name="FWP_BYTE_BLOB", pack=False, align=None), "eku": SimTypePointer(SimStruct({"numEku": SimTypeInt(signed=False, label="UInt32"), "eku": SimTypePointer(SimTypePointer(SimTypeChar(label="Byte"), offset=0), offset=0)}, name="IKEEXT_CERT_EKUS0", pack=False, align=None), offset=0), "name": SimTypePointer(SimStruct({"nameType": SimTypeInt(signed=False, label="IKEEXT_CERT_CRITERIA_NAME_TYPE"), "certName": SimTypePointer(SimTypeChar(label="Char"), offset=0)}, name="IKEEXT_CERT_NAME0", pack=False, align=None), offset=0), "flags": SimTypeInt(signed=False, label="UInt32")}, name="IKEEXT_CERTIFICATE_CRITERIA0", pack=False, align=None), offset=0)}, name="_Anonymous1_e__Struct", pack=False, align=None), "Anonymous2": SimStruct({"inboundEnterpriseStoreArraySize": SimTypeInt(signed=False, label="UInt32"), "inboundEnterpriseStoreCriteria": SimTypePointer(SimStruct({"certData": SimStruct({"size": SimTypeInt(signed=False, label="UInt32"), "data": SimTypePointer(SimTypeChar(label="Byte"), offset=0)}, name="FWP_BYTE_BLOB", pack=False, align=None), "certHash": SimStruct({"size": SimTypeInt(signed=False, label="UInt32"), "data": SimTypePointer(SimTypeChar(label="Byte"), offset=0)}, name="FWP_BYTE_BLOB", pack=False, align=None), "eku": SimTypePointer(SimStruct({"numEku": SimTypeInt(signed=False, label="UInt32"), "eku": SimTypePointer(SimTypePointer(SimTypeChar(label="Byte"), offset=0), offset=0)}, name="IKEEXT_CERT_EKUS0", pack=False, align=None), offset=0), "name": SimTypePointer(SimStruct({"nameType": SimTypeInt(signed=False, label="IKEEXT_CERT_CRITERIA_NAME_TYPE"), "certName": SimTypePointer(SimTypeChar(label="Char"), offset=0)}, name="IKEEXT_CERT_NAME0", pack=False, align=None), offset=0), "flags": SimTypeInt(signed=False, label="UInt32")}, name="IKEEXT_CERTIFICATE_CRITERIA0", pack=False, align=None), offset=0)}, name="_Anonymous2_e__Struct", pack=False, align=None), "Anonymous3": SimStruct({"inboundRootStoreArraySize": SimTypeInt(signed=False, label="UInt32"), "inboundTrustedRootStoreCriteria": SimTypePointer(SimStruct({"certData": SimStruct({"size": SimTypeInt(signed=False, label="UInt32"), "data": SimTypePointer(SimTypeChar(label="Byte"), offset=0)}, name="FWP_BYTE_BLOB", pack=False, align=None), "certHash": SimStruct({"size": SimTypeInt(signed=False, label="UInt32"), "data": SimTypePointer(SimTypeChar(label="Byte"), offset=0)}, name="FWP_BYTE_BLOB", pack=False, align=None), "eku": SimTypePointer(SimStruct({"numEku": SimTypeInt(signed=False, label="UInt32"), "eku": SimTypePointer(SimTypePointer(SimTypeChar(label="Byte"), offset=0), offset=0)}, name="IKEEXT_CERT_EKUS0", pack=False, align=None), offset=0), "name": SimTypePointer(SimStruct({"nameType": SimTypeInt(signed=False, label="IKEEXT_CERT_CRITERIA_NAME_TYPE"), "certName": SimTypePointer(SimTypeChar(label="Char"), offset=0)}, name="IKEEXT_CERT_NAME0", pack=False, align=None), offset=0), "flags": SimTypeInt(signed=False, label="UInt32")}, name="IKEEXT_CERTIFICATE_CRITERIA0", pack=False, align=None), offset=0)}, name="_Anonymous3_e__Struct", pack=False, align=None)}, name="<anon>", label="None"), "outboundConfigType": SimTypeInt(signed=False, label="IKEEXT_CERT_CONFIG_TYPE"), "Anonymous2": SimUnion({"Anonymous1": SimStruct({"outboundRootArraySize": SimTypeInt(signed=False, label="UInt32"), "outboundRootCriteria": SimTypePointer(SimStruct({"certData": SimStruct({"size": SimTypeInt(signed=False, label="UInt32"), "data": SimTypePointer(SimTypeChar(label="Byte"), offset=0)}, name="FWP_BYTE_BLOB", pack=False, align=None), "certHash": SimStruct({"size": SimTypeInt(signed=False, label="UInt32"), "data": SimTypePointer(SimTypeChar(label="Byte"), offset=0)}, name="FWP_BYTE_BLOB", pack=False, align=None), "eku": SimTypePointer(SimStruct({"numEku": SimTypeInt(signed=False, label="UInt32"), "eku": SimTypePointer(SimTypePointer(SimTypeChar(label="Byte"), offset=0), offset=0)}, name="IKEEXT_CERT_EKUS0", pack=False, align=None), offset=0), "name": SimTypePointer(SimStruct({"nameType": SimTypeInt(signed=False, label="IKEEXT_CERT_CRITERIA_NAME_TYPE"), "certName": SimTypePointer(SimTypeChar(label="Char"), offset=0)}, name="IKEEXT_CERT_NAME0", pack=False, align=None), offset=0), "flags": SimTypeInt(signed=False, label="UInt32")}, name="IKEEXT_CERTIFICATE_CRITERIA0", pack=False, align=None), offset=0)}, name="_Anonymous1_e__Struct", pack=False, align=None), "Anonymous2": SimStruct({"outboundEnterpriseStoreArraySize": SimTypeInt(signed=False, label="UInt32"), "outboundEnterpriseStoreCriteria": SimTypePointer(SimStruct({"certData": SimStruct({"size": SimTypeInt(signed=False, label="UInt32"), "data": SimTypePointer(SimTypeChar(label="Byte"), offset=0)}, name="FWP_BYTE_BLOB", pack=False, align=None), "certHash": SimStruct({"size": SimTypeInt(signed=False, label="UInt32"), "data": SimTypePointer(SimTypeChar(label="Byte"), offset=0)}, name="FWP_BYTE_BLOB", pack=False, align=None), "eku": SimTypePointer(SimStruct({"numEku": SimTypeInt(signed=False, label="UInt32"), "eku": SimTypePointer(SimTypePointer(SimTypeChar(label="Byte"), offset=0), offset=0)}, name="IKEEXT_CERT_EKUS0", pack=False, align=None), offset=0), "name": SimTypePointer(SimStruct({"nameType": SimTypeInt(signed=False, label="IKEEXT_CERT_CRITERIA_NAME_TYPE"), "certName": SimTypePointer(SimTypeChar(label="Char"), offset=0)}, name="IKEEXT_CERT_NAME0", pack=False, align=None), offset=0), "flags": SimTypeInt(signed=False, label="UInt32")}, name="IKEEXT_CERTIFICATE_CRITERIA0", pack=False, align=None), offset=0)}, name="_Anonymous2_e__Struct", pack=False, align=None), "Anonymous3": SimStruct({"outboundRootStoreArraySize": SimTypeInt(signed=False, label="UInt32"), "outboundTrustedRootStoreCriteria": SimTypePointer(SimStruct({"certData": SimStruct({"size": SimTypeInt(signed=False, label="UInt32"), "data": SimTypePointer(SimTypeChar(label="Byte"), offset=0)}, name="FWP_BYTE_BLOB", pack=False, align=None), "certHash": SimStruct({"size": SimTypeInt(signed=False, label="UInt32"), "data": SimTypePointer(SimTypeChar(label="Byte"), offset=0)}, name="FWP_BYTE_BLOB", pack=False, align=None), "eku": SimTypePointer(SimStruct({"numEku": SimTypeInt(signed=False, label="UInt32"), "eku": SimTypePointer(SimTypePointer(SimTypeChar(label="Byte"), offset=0), offset=0)}, name="IKEEXT_CERT_EKUS0", pack=False, align=None), offset=0), "name": SimTypePointer(SimStruct({"nameType": SimTypeInt(signed=False, label="IKEEXT_CERT_CRITERIA_NAME_TYPE"), "certName": SimTypePointer(SimTypeChar(label="Char"), offset=0)}, name="IKEEXT_CERT_NAME0", pack=False, align=None), offset=0), "flags": SimTypeInt(signed=False, label="UInt32")}, name="IKEEXT_CERTIFICATE_CRITERIA0", pack=False, align=None), offset=0)}, name="_Anonymous3_e__Struct", pack=False, align=None)}, name="<anon>", label="None"), "flags": SimTypeInt(signed=False, label="IKEEXT_CERT_AUTH"), "localCertLocationUrl": SimStruct({"size": SimTypeInt(signed=False, label="UInt32"), "data": SimTypePointer(SimTypeChar(label="Byte"), offset=0)}, name="FWP_BYTE_BLOB", pack=False, align=None)}, name="IKEEXT_CERTIFICATE_AUTHENTICATION2", pack=False, align=None), "kerberosAuthentication": SimStruct({"flags": SimTypeInt(signed=False, label="IKEEXT_KERBEROS_AUTHENTICATION_FLAGS"), "proxyServer": SimTypePointer(SimTypeChar(label="Char"), offset=0)}, name="IKEEXT_KERBEROS_AUTHENTICATION1", pack=False, align=None), "reservedAuthentication": SimStruct({"flags": SimTypeInt(signed=False, label="IKEEXT_RESERVED_AUTHENTICATION_FLAGS")}, name="IKEEXT_RESERVED_AUTHENTICATION0", pack=False, align=None), "ntlmV2Authentication": SimStruct({"flags": SimTypeInt(signed=False, label="UInt32")}, name="IKEEXT_NTLM_V2_AUTHENTICATION0", pack=False, align=None), "sslAuthentication": SimStruct({"inboundConfigType": SimTypeInt(signed=False, label="IKEEXT_CERT_CONFIG_TYPE"), "Anonymous1": SimUnion({"Anonymous1": SimStruct({"inboundRootArraySize": SimTypeInt(signed=False, label="UInt32"), "inboundRootCriteria": SimTypePointer(SimStruct({"certData": SimStruct({"size": SimTypeInt(signed=False, label="UInt32"), "data": SimTypePointer(SimTypeChar(label="Byte"), offset=0)}, name="FWP_BYTE_BLOB", pack=False, align=None), "certHash": SimStruct({"size": SimTypeInt(signed=False, label="UInt32"), "data": SimTypePointer(SimTypeChar(label="Byte"), offset=0)}, name="FWP_BYTE_BLOB", pack=False, align=None), "eku": SimTypePointer(SimStruct({"numEku": SimTypeInt(signed=False, label="UInt32"), "eku": SimTypePointer(SimTypePointer(SimTypeChar(label="Byte"), offset=0), offset=0)}, name="IKEEXT_CERT_EKUS0", pack=False, align=None), offset=0), "name": SimTypePointer(SimStruct({"nameType": SimTypeInt(signed=False, label="IKEEXT_CERT_CRITERIA_NAME_TYPE"), "certName": SimTypePointer(SimTypeChar(label="Char"), offset=0)}, name="IKEEXT_CERT_NAME0", pack=False, align=None), offset=0), "flags": SimTypeInt(signed=False, label="UInt32")}, name="IKEEXT_CERTIFICATE_CRITERIA0", pack=False, align=None), offset=0)}, name="_Anonymous1_e__Struct", pack=False, align=None), "Anonymous2": SimStruct({"inboundEnterpriseStoreArraySize": SimTypeInt(signed=False, label="UInt32"), "inboundEnterpriseStoreCriteria": SimTypePointer(SimStruct({"certData": SimStruct({"size": SimTypeInt(signed=False, label="UInt32"), "data": SimTypePointer(SimTypeChar(label="Byte"), offset=0)}, name="FWP_BYTE_BLOB", pack=False, align=None), "certHash": SimStruct({"size": SimTypeInt(signed=False, label="UInt32"), "data": SimTypePointer(SimTypeChar(label="Byte"), offset=0)}, name="FWP_BYTE_BLOB", pack=False, align=None), "eku": SimTypePointer(SimStruct({"numEku": SimTypeInt(signed=False, label="UInt32"), "eku": SimTypePointer(SimTypePointer(SimTypeChar(label="Byte"), offset=0), offset=0)}, name="IKEEXT_CERT_EKUS0", pack=False, align=None), offset=0), "name": SimTypePointer(SimStruct({"nameType": SimTypeInt(signed=False, label="IKEEXT_CERT_CRITERIA_NAME_TYPE"), "certName": SimTypePointer(SimTypeChar(label="Char"), offset=0)}, name="IKEEXT_CERT_NAME0", pack=False, align=None), offset=0), "flags": SimTypeInt(signed=False, label="UInt32")}, name="IKEEXT_CERTIFICATE_CRITERIA0", pack=False, align=None), offset=0)}, name="_Anonymous2_e__Struct", pack=False, align=None), "Anonymous3": SimStruct({"inboundRootStoreArraySize": SimTypeInt(signed=False, label="UInt32"), "inboundTrustedRootStoreCriteria": SimTypePointer(SimStruct({"certData": SimStruct({"size": SimTypeInt(signed=False, label="UInt32"), "data": SimTypePointer(SimTypeChar(label="Byte"), offset=0)}, name="FWP_BYTE_BLOB", pack=False, align=None), "certHash": SimStruct({"size": SimTypeInt(signed=False, label="UInt32"), "data": SimTypePointer(SimTypeChar(label="Byte"), offset=0)}, name="FWP_BYTE_BLOB", pack=False, align=None), "eku": SimTypePointer(SimStruct({"numEku": SimTypeInt(signed=False, label="UInt32"), "eku": SimTypePointer(SimTypePointer(SimTypeChar(label="Byte"), offset=0), offset=0)}, name="IKEEXT_CERT_EKUS0", pack=False, align=None), offset=0), "name": SimTypePointer(SimStruct({"nameType": SimTypeInt(signed=False, label="IKEEXT_CERT_CRITERIA_NAME_TYPE"), "certName": SimTypePointer(SimTypeChar(label="Char"), offset=0)}, name="IKEEXT_CERT_NAME0", pack=False, align=None), offset=0), "flags": SimTypeInt(signed=False, label="UInt32")}, name="IKEEXT_CERTIFICATE_CRITERIA0", pack=False, align=None), offset=0)}, name="_Anonymous3_e__Struct", pack=False, align=None)}, name="<anon>", label="None"), "outboundConfigType": SimTypeInt(signed=False, label="IKEEXT_CERT_CONFIG_TYPE"), "Anonymous2": SimUnion({"Anonymous1": SimStruct({"outboundRootArraySize": SimTypeInt(signed=False, label="UInt32"), "outboundRootCriteria": SimTypePointer(SimStruct({"certData": SimStruct({"size": SimTypeInt(signed=False, label="UInt32"), "data": SimTypePointer(SimTypeChar(label="Byte"), offset=0)}, name="FWP_BYTE_BLOB", pack=False, align=None), "certHash": SimStruct({"size": SimTypeInt(signed=False, label="UInt32"), "data": SimTypePointer(SimTypeChar(label="Byte"), offset=0)}, name="FWP_BYTE_BLOB", pack=False, align=None), "eku": SimTypePointer(SimStruct({"numEku": SimTypeInt(signed=False, label="UInt32"), "eku": SimTypePointer(SimTypePointer(SimTypeChar(label="Byte"), offset=0), offset=0)}, name="IKEEXT_CERT_EKUS0", pack=False, align=None), offset=0), "name": SimTypePointer(SimStruct({"nameType": SimTypeInt(signed=False, label="IKEEXT_CERT_CRITERIA_NAME_TYPE"), "certName": SimTypePointer(SimTypeChar(label="Char"), offset=0)}, name="IKEEXT_CERT_NAME0", pack=False, align=None), offset=0), "flags": SimTypeInt(signed=False, label="UInt32")}, name="IKEEXT_CERTIFICATE_CRITERIA0", pack=False, align=None), offset=0)}, name="_Anonymous1_e__Struct", pack=False, align=None), "Anonymous2": SimStruct({"outboundEnterpriseStoreArraySize": SimTypeInt(signed=False, label="UInt32"), "outboundEnterpriseStoreCriteria": SimTypePointer(SimStruct({"certData": SimStruct({"size": SimTypeInt(signed=False, label="UInt32"), "data": SimTypePointer(SimTypeChar(label="Byte"), offset=0)}, name="FWP_BYTE_BLOB", pack=False, align=None), "certHash": SimStruct({"size": SimTypeInt(signed=False, label="UInt32"), "data": SimTypePointer(SimTypeChar(label="Byte"), offset=0)}, name="FWP_BYTE_BLOB", pack=False, align=None), "eku": SimTypePointer(SimStruct({"numEku": SimTypeInt(signed=False, label="UInt32"), "eku": SimTypePointer(SimTypePointer(SimTypeChar(label="Byte"), offset=0), offset=0)}, name="IKEEXT_CERT_EKUS0", pack=False, align=None), offset=0), "name": SimTypePointer(SimStruct({"nameType": SimTypeInt(signed=False, label="IKEEXT_CERT_CRITERIA_NAME_TYPE"), "certName": SimTypePointer(SimTypeChar(label="Char"), offset=0)}, name="IKEEXT_CERT_NAME0", pack=False, align=None), offset=0), "flags": SimTypeInt(signed=False, label="UInt32")}, name="IKEEXT_CERTIFICATE_CRITERIA0", pack=False, align=None), offset=0)}, name="_Anonymous2_e__Struct", pack=False, align=None), "Anonymous3": SimStruct({"outboundRootStoreArraySize": SimTypeInt(signed=False, label="UInt32"), "outboundTrustedRootStoreCriteria": SimTypePointer(SimStruct({"certData": SimStruct({"size": SimTypeInt(signed=False, label="UInt32"), "data": SimTypePointer(SimTypeChar(label="Byte"), offset=0)}, name="FWP_BYTE_BLOB", pack=False, align=None), "certHash": SimStruct({"size": SimTypeInt(signed=False, label="UInt32"), "data": SimTypePointer(SimTypeChar(label="Byte"), offset=0)}, name="FWP_BYTE_BLOB", pack=False, align=None), "eku": SimTypePointer(SimStruct({"numEku": SimTypeInt(signed=False, label="UInt32"), "eku": SimTypePointer(SimTypePointer(SimTypeChar(label="Byte"), offset=0), offset=0)}, name="IKEEXT_CERT_EKUS0", pack=False, align=None), offset=0), "name": SimTypePointer(SimStruct({"nameType": SimTypeInt(signed=False, label="IKEEXT_CERT_CRITERIA_NAME_TYPE"), "certName": SimTypePointer(SimTypeChar(label="Char"), offset=0)}, name="IKEEXT_CERT_NAME0", pack=False, align=None), offset=0), "flags": SimTypeInt(signed=False, label="UInt32")}, name="IKEEXT_CERTIFICATE_CRITERIA0", pack=False, align=None), offset=0)}, name="_Anonymous3_e__Struct", pack=False, align=None)}, name="<anon>", label="None"), "flags": SimTypeInt(signed=False, label="IKEEXT_CERT_AUTH"), "localCertLocationUrl": SimStruct({"size": SimTypeInt(signed=False, label="UInt32"), "data": SimTypePointer(SimTypeChar(label="Byte"), offset=0)}, name="FWP_BYTE_BLOB", pack=False, align=None)}, name="IKEEXT_CERTIFICATE_AUTHENTICATION2", pack=False, align=None), "cgaAuthentication": SimStruct({"keyContainerName": SimTypePointer(SimTypeChar(label="Char"), offset=0), "cspName": SimTypePointer(SimTypeChar(label="Char"), offset=0), "cspType": SimTypeInt(signed=False, label="UInt32"), "cgaModifier": SimStruct({"byteArray16": SimTypeFixedSizeArray(SimTypeChar(label="Byte"), 16)}, name="FWP_BYTE_ARRAY16", pack=False, align=None), "cgaCollisionCount": SimTypeChar(label="Byte")}, name="IKEEXT_IPV6_CGA_AUTHENTICATION0", pack=False, align=None), "eapAuthentication": SimStruct({"flags": SimTypeInt(signed=False, label="IKEEXT_EAP_AUTHENTICATION_FLAGS")}, name="IKEEXT_EAP_AUTHENTICATION0", pack=False, align=None)}, name="<anon>", label="None")}, name="IKEEXT_AUTHENTICATION_METHOD2", pack=False, align=None), offset=0), "initiatorImpersonationType": SimTypeInt(signed=False, label="IKEEXT_AUTHENTICATION_IMPERSONATION_TYPE"), "numIkeProposals": SimTypeInt(signed=False, label="UInt32"), "ikeProposals": SimTypePointer(SimStruct({"cipherAlgorithm": SimStruct({"algoIdentifier": SimTypeInt(signed=False, label="IKEEXT_CIPHER_TYPE"), "keyLen": SimTypeInt(signed=False, label="UInt32"), "rounds": SimTypeInt(signed=False, label="UInt32")}, name="IKEEXT_CIPHER_ALGORITHM0", pack=False, align=None), "integrityAlgorithm": SimStruct({"algoIdentifier": SimTypeInt(signed=False, label="IKEEXT_INTEGRITY_TYPE")}, name="IKEEXT_INTEGRITY_ALGORITHM0", pack=False, align=None), "maxLifetimeSeconds": SimTypeInt(signed=False, label="UInt32"), "dhGroup": SimTypeInt(signed=False, label="IKEEXT_DH_GROUP"), "quickModeLimit": SimTypeInt(signed=False, label="UInt32")}, name="IKEEXT_PROPOSAL0", pack=False, align=None), offset=0), "flags": SimTypeInt(signed=False, label="IKEEXT_POLICY_FLAG"), "maxDynamicFilters": SimTypeInt(signed=False, label="UInt32"), "retransmitDurationSecs": SimTypeInt(signed=False, label="UInt32")}, name="IKEEXT_POLICY2", pack=False, align=None), offset=0), "authIpMmPolicy": SimTypePointer(SimStruct({"softExpirationTime": SimTypeInt(signed=False, label="UInt32"), "numAuthenticationMethods": SimTypeInt(signed=False, label="UInt32"), "authenticationMethods": SimTypePointer(SimStruct({"authenticationMethodType": SimTypeInt(signed=False, label="IKEEXT_AUTHENTICATION_METHOD_TYPE"), "Anonymous": SimUnion({"presharedKeyAuthentication": SimStruct({"presharedKey": SimStruct({"size": SimTypeInt(signed=False, label="UInt32"), "data": SimTypePointer(SimTypeChar(label="Byte"), offset=0)}, name="FWP_BYTE_BLOB", pack=False, align=None), "flags": SimTypeInt(signed=False, label="IKEEXT_PRESHARED_KEY_AUTHENTICATION_FLAGS")}, name="IKEEXT_PRESHARED_KEY_AUTHENTICATION1", pack=False, align=None), "certificateAuthentication": SimStruct({"inboundConfigType": SimTypeInt(signed=False, label="IKEEXT_CERT_CONFIG_TYPE"), "Anonymous1": SimUnion({"Anonymous1": SimStruct({"inboundRootArraySize": SimTypeInt(signed=False, label="UInt32"), "inboundRootCriteria": SimTypePointer(SimStruct({"certData": SimStruct({"size": SimTypeInt(signed=False, label="UInt32"), "data": SimTypePointer(SimTypeChar(label="Byte"), offset=0)}, name="FWP_BYTE_BLOB", pack=False, align=None), "certHash": SimStruct({"size": SimTypeInt(signed=False, label="UInt32"), "data": SimTypePointer(SimTypeChar(label="Byte"), offset=0)}, name="FWP_BYTE_BLOB", pack=False, align=None), "eku": SimTypePointer(SimStruct({"numEku": SimTypeInt(signed=False, label="UInt32"), "eku": SimTypePointer(SimTypePointer(SimTypeChar(label="Byte"), offset=0), offset=0)}, name="IKEEXT_CERT_EKUS0", pack=False, align=None), offset=0), "name": SimTypePointer(SimStruct({"nameType": SimTypeInt(signed=False, label="IKEEXT_CERT_CRITERIA_NAME_TYPE"), "certName": SimTypePointer(SimTypeChar(label="Char"), offset=0)}, name="IKEEXT_CERT_NAME0", pack=False, align=None), offset=0), "flags": SimTypeInt(signed=False, label="UInt32")}, name="IKEEXT_CERTIFICATE_CRITERIA0", pack=False, align=None), offset=0)}, name="_Anonymous1_e__Struct", pack=False, align=None), "Anonymous2": SimStruct({"inboundEnterpriseStoreArraySize": SimTypeInt(signed=False, label="UInt32"), "inboundEnterpriseStoreCriteria": SimTypePointer(SimStruct({"certData": SimStruct({"size": SimTypeInt(signed=False, label="UInt32"), "data": SimTypePointer(SimTypeChar(label="Byte"), offset=0)}, name="FWP_BYTE_BLOB", pack=False, align=None), "certHash": SimStruct({"size": SimTypeInt(signed=False, label="UInt32"), "data": SimTypePointer(SimTypeChar(label="Byte"), offset=0)}, name="FWP_BYTE_BLOB", pack=False, align=None), "eku": SimTypePointer(SimStruct({"numEku": SimTypeInt(signed=False, label="UInt32"), "eku": SimTypePointer(SimTypePointer(SimTypeChar(label="Byte"), offset=0), offset=0)}, name="IKEEXT_CERT_EKUS0", pack=False, align=None), offset=0), "name": SimTypePointer(SimStruct({"nameType": SimTypeInt(signed=False, label="IKEEXT_CERT_CRITERIA_NAME_TYPE"), "certName": SimTypePointer(SimTypeChar(label="Char"), offset=0)}, name="IKEEXT_CERT_NAME0", pack=False, align=None), offset=0), "flags": SimTypeInt(signed=False, label="UInt32")}, name="IKEEXT_CERTIFICATE_CRITERIA0", pack=False, align=None), offset=0)}, name="_Anonymous2_e__Struct", pack=False, align=None), "Anonymous3": SimStruct({"inboundRootStoreArraySize": SimTypeInt(signed=False, label="UInt32"), "inboundTrustedRootStoreCriteria": SimTypePointer(SimStruct({"certData": SimStruct({"size": SimTypeInt(signed=False, label="UInt32"), "data": SimTypePointer(SimTypeChar(label="Byte"), offset=0)}, name="FWP_BYTE_BLOB", pack=False, align=None), "certHash": SimStruct({"size": SimTypeInt(signed=False, label="UInt32"), "data": SimTypePointer(SimTypeChar(label="Byte"), offset=0)}, name="FWP_BYTE_BLOB", pack=False, align=None), "eku": SimTypePointer(SimStruct({"numEku": SimTypeInt(signed=False, label="UInt32"), "eku": SimTypePointer(SimTypePointer(SimTypeChar(label="Byte"), offset=0), offset=0)}, name="IKEEXT_CERT_EKUS0", pack=False, align=None), offset=0), "name": SimTypePointer(SimStruct({"nameType": SimTypeInt(signed=False, label="IKEEXT_CERT_CRITERIA_NAME_TYPE"), "certName": SimTypePointer(SimTypeChar(label="Char"), offset=0)}, name="IKEEXT_CERT_NAME0", pack=False, align=None), offset=0), "flags": SimTypeInt(signed=False, label="UInt32")}, name="IKEEXT_CERTIFICATE_CRITERIA0", pack=False, align=None), offset=0)}, name="_Anonymous3_e__Struct", pack=False, align=None)}, name="<anon>", label="None"), "outboundConfigType": SimTypeInt(signed=False, label="IKEEXT_CERT_CONFIG_TYPE"), "Anonymous2": SimUnion({"Anonymous1": SimStruct({"outboundRootArraySize": SimTypeInt(signed=False, label="UInt32"), "outboundRootCriteria": SimTypePointer(SimStruct({"certData": SimStruct({"size": SimTypeInt(signed=False, label="UInt32"), "data": SimTypePointer(SimTypeChar(label="Byte"), offset=0)}, name="FWP_BYTE_BLOB", pack=False, align=None), "certHash": SimStruct({"size": SimTypeInt(signed=False, label="UInt32"), "data": SimTypePointer(SimTypeChar(label="Byte"), offset=0)}, name="FWP_BYTE_BLOB", pack=False, align=None), "eku": SimTypePointer(SimStruct({"numEku": SimTypeInt(signed=False, label="UInt32"), "eku": SimTypePointer(SimTypePointer(SimTypeChar(label="Byte"), offset=0), offset=0)}, name="IKEEXT_CERT_EKUS0", pack=False, align=None), offset=0), "name": SimTypePointer(SimStruct({"nameType": SimTypeInt(signed=False, label="IKEEXT_CERT_CRITERIA_NAME_TYPE"), "certName": SimTypePointer(SimTypeChar(label="Char"), offset=0)}, name="IKEEXT_CERT_NAME0", pack=False, align=None), offset=0), "flags": SimTypeInt(signed=False, label="UInt32")}, name="IKEEXT_CERTIFICATE_CRITERIA0", pack=False, align=None), offset=0)}, name="_Anonymous1_e__Struct", pack=False, align=None), "Anonymous2": SimStruct({"outboundEnterpriseStoreArraySize": SimTypeInt(signed=False, label="UInt32"), "outboundEnterpriseStoreCriteria": SimTypePointer(SimStruct({"certData": SimStruct({"size": SimTypeInt(signed=False, label="UInt32"), "data": SimTypePointer(SimTypeChar(label="Byte"), offset=0)}, name="FWP_BYTE_BLOB", pack=False, align=None), "certHash": SimStruct({"size": SimTypeInt(signed=False, label="UInt32"), "data": SimTypePointer(SimTypeChar(label="Byte"), offset=0)}, name="FWP_BYTE_BLOB", pack=False, align=None), "eku": SimTypePointer(SimStruct({"numEku": SimTypeInt(signed=False, label="UInt32"), "eku": SimTypePointer(SimTypePointer(SimTypeChar(label="Byte"), offset=0), offset=0)}, name="IKEEXT_CERT_EKUS0", pack=False, align=None), offset=0), "name": SimTypePointer(SimStruct({"nameType": SimTypeInt(signed=False, label="IKEEXT_CERT_CRITERIA_NAME_TYPE"), "certName": SimTypePointer(SimTypeChar(label="Char"), offset=0)}, name="IKEEXT_CERT_NAME0", pack=False, align=None), offset=0), "flags": SimTypeInt(signed=False, label="UInt32")}, name="IKEEXT_CERTIFICATE_CRITERIA0", pack=False, align=None), offset=0)}, name="_Anonymous2_e__Struct", pack=False, align=None), "Anonymous3": SimStruct({"outboundRootStoreArraySize": SimTypeInt(signed=False, label="UInt32"), "outboundTrustedRootStoreCriteria": SimTypePointer(SimStruct({"certData": SimStruct({"size": SimTypeInt(signed=False, label="UInt32"), "data": SimTypePointer(SimTypeChar(label="Byte"), offset=0)}, name="FWP_BYTE_BLOB", pack=False, align=None), "certHash": SimStruct({"size": SimTypeInt(signed=False, label="UInt32"), "data": SimTypePointer(SimTypeChar(label="Byte"), offset=0)}, name="FWP_BYTE_BLOB", pack=False, align=None), "eku": SimTypePointer(SimStruct({"numEku": SimTypeInt(signed=False, label="UInt32"), "eku": SimTypePointer(SimTypePointer(SimTypeChar(label="Byte"), offset=0), offset=0)}, name="IKEEXT_CERT_EKUS0", pack=False, align=None), offset=0), "name": SimTypePointer(SimStruct({"nameType": SimTypeInt(signed=False, label="IKEEXT_CERT_CRITERIA_NAME_TYPE"), "certName": SimTypePointer(SimTypeChar(label="Char"), offset=0)}, name="IKEEXT_CERT_NAME0", pack=False, align=None), offset=0), "flags": SimTypeInt(signed=False, label="UInt32")}, name="IKEEXT_CERTIFICATE_CRITERIA0", pack=False, align=None), offset=0)}, name="_Anonymous3_e__Struct", pack=False, align=None)}, name="<anon>", label="None"), "flags": SimTypeInt(signed=False, label="IKEEXT_CERT_AUTH"), "localCertLocationUrl": SimStruct({"size": SimTypeInt(signed=False, label="UInt32"), "data": SimTypePointer(SimTypeChar(label="Byte"), offset=0)}, name="FWP_BYTE_BLOB", pack=False, align=None)}, name="IKEEXT_CERTIFICATE_AUTHENTICATION2", pack=False, align=None), "kerberosAuthentication": SimStruct({"flags": SimTypeInt(signed=False, label="IKEEXT_KERBEROS_AUTHENTICATION_FLAGS"), "proxyServer": SimTypePointer(SimTypeChar(label="Char"), offset=0)}, name="IKEEXT_KERBEROS_AUTHENTICATION1", pack=False, align=None), "reservedAuthentication": SimStruct({"flags": SimTypeInt(signed=False, label="IKEEXT_RESERVED_AUTHENTICATION_FLAGS")}, name="IKEEXT_RESERVED_AUTHENTICATION0", pack=False, align=None), "ntlmV2Authentication": SimStruct({"flags": SimTypeInt(signed=False, label="UInt32")}, name="IKEEXT_NTLM_V2_AUTHENTICATION0", pack=False, align=None), "sslAuthentication": SimStruct({"inboundConfigType": SimTypeInt(signed=False, label="IKEEXT_CERT_CONFIG_TYPE"), "Anonymous1": SimUnion({"Anonymous1": SimStruct({"inboundRootArraySize": SimTypeInt(signed=False, label="UInt32"), "inboundRootCriteria": SimTypePointer(SimStruct({"certData": SimStruct({"size": SimTypeInt(signed=False, label="UInt32"), "data": SimTypePointer(SimTypeChar(label="Byte"), offset=0)}, name="FWP_BYTE_BLOB", pack=False, align=None), "certHash": SimStruct({"size": SimTypeInt(signed=False, label="UInt32"), "data": SimTypePointer(SimTypeChar(label="Byte"), offset=0)}, name="FWP_BYTE_BLOB", pack=False, align=None), "eku": SimTypePointer(SimStruct({"numEku": SimTypeInt(signed=False, label="UInt32"), "eku": SimTypePointer(SimTypePointer(SimTypeChar(label="Byte"), offset=0), offset=0)}, name="IKEEXT_CERT_EKUS0", pack=False, align=None), offset=0), "name": SimTypePointer(SimStruct({"nameType": SimTypeInt(signed=False, label="IKEEXT_CERT_CRITERIA_NAME_TYPE"), "certName": SimTypePointer(SimTypeChar(label="Char"), offset=0)}, name="IKEEXT_CERT_NAME0", pack=False, align=None), offset=0), "flags": SimTypeInt(signed=False, label="UInt32")}, name="IKEEXT_CERTIFICATE_CRITERIA0", pack=False, align=None), offset=0)}, name="_Anonymous1_e__Struct", pack=False, align=None), "Anonymous2": SimStruct({"inboundEnterpriseStoreArraySize": SimTypeInt(signed=False, label="UInt32"), "inboundEnterpriseStoreCriteria": SimTypePointer(SimStruct({"certData": SimStruct({"size": SimTypeInt(signed=False, label="UInt32"), "data": SimTypePointer(SimTypeChar(label="Byte"), offset=0)}, name="FWP_BYTE_BLOB", pack=False, align=None), "certHash": SimStruct({"size": SimTypeInt(signed=False, label="UInt32"), "data": SimTypePointer(SimTypeChar(label="Byte"), offset=0)}, name="FWP_BYTE_BLOB", pack=False, align=None), "eku": SimTypePointer(SimStruct({"numEku": SimTypeInt(signed=False, label="UInt32"), "eku": SimTypePointer(SimTypePointer(SimTypeChar(label="Byte"), offset=0), offset=0)}, name="IKEEXT_CERT_EKUS0", pack=False, align=None), offset=0), "name": SimTypePointer(SimStruct({"nameType": SimTypeInt(signed=False, label="IKEEXT_CERT_CRITERIA_NAME_TYPE"), "certName": SimTypePointer(SimTypeChar(label="Char"), offset=0)}, name="IKEEXT_CERT_NAME0", pack=False, align=None), offset=0), "flags": SimTypeInt(signed=False, label="UInt32")}, name="IKEEXT_CERTIFICATE_CRITERIA0", pack=False, align=None), offset=0)}, name="_Anonymous2_e__Struct", pack=False, align=None), "Anonymous3": SimStruct({"inboundRootStoreArraySize": SimTypeInt(signed=False, label="UInt32"), "inboundTrustedRootStoreCriteria": SimTypePointer(SimStruct({"certData": SimStruct({"size": SimTypeInt(signed=False, label="UInt32"), "data": SimTypePointer(SimTypeChar(label="Byte"), offset=0)}, name="FWP_BYTE_BLOB", pack=False, align=None), "certHash": SimStruct({"size": SimTypeInt(signed=False, label="UInt32"), "data": SimTypePointer(SimTypeChar(label="Byte"), offset=0)}, name="FWP_BYTE_BLOB", pack=False, align=None), "eku": SimTypePointer(SimStruct({"numEku": SimTypeInt(signed=False, label="UInt32"), "eku": SimTypePointer(SimTypePointer(SimTypeChar(label="Byte"), offset=0), offset=0)}, name="IKEEXT_CERT_EKUS0", pack=False, align=None), offset=0), "name": SimTypePointer(SimStruct({"nameType": SimTypeInt(signed=False, label="IKEEXT_CERT_CRITERIA_NAME_TYPE"), "certName": SimTypePointer(SimTypeChar(label="Char"), offset=0)}, name="IKEEXT_CERT_NAME0", pack=False, align=None), offset=0), "flags": SimTypeInt(signed=False, label="UInt32")}, name="IKEEXT_CERTIFICATE_CRITERIA0", pack=False, align=None), offset=0)}, name="_Anonymous3_e__Struct", pack=False, align=None)}, name="<anon>", label="None"), "outboundConfigType": SimTypeInt(signed=False, label="IKEEXT_CERT_CONFIG_TYPE"), "Anonymous2": SimUnion({"Anonymous1": SimStruct({"outboundRootArraySize": SimTypeInt(signed=False, label="UInt32"), "outboundRootCriteria": SimTypePointer(SimStruct({"certData": SimStruct({"size": SimTypeInt(signed=False, label="UInt32"), "data": SimTypePointer(SimTypeChar(label="Byte"), offset=0)}, name="FWP_BYTE_BLOB", pack=False, align=None), "certHash": SimStruct({"size": SimTypeInt(signed=False, label="UInt32"), "data": SimTypePointer(SimTypeChar(label="Byte"), offset=0)}, name="FWP_BYTE_BLOB", pack=False, align=None), "eku": SimTypePointer(SimStruct({"numEku": SimTypeInt(signed=False, label="UInt32"), "eku": SimTypePointer(SimTypePointer(SimTypeChar(label="Byte"), offset=0), offset=0)}, name="IKEEXT_CERT_EKUS0", pack=False, align=None), offset=0), "name": SimTypePointer(SimStruct({"nameType": SimTypeInt(signed=False, label="IKEEXT_CERT_CRITERIA_NAME_TYPE"), "certName": SimTypePointer(SimTypeChar(label="Char"), offset=0)}, name="IKEEXT_CERT_NAME0", pack=False, align=None), offset=0), "flags": SimTypeInt(signed=False, label="UInt32")}, name="IKEEXT_CERTIFICATE_CRITERIA0", pack=False, align=None), offset=0)}, name="_Anonymous1_e__Struct", pack=False, align=None), "Anonymous2": SimStruct({"outboundEnterpriseStoreArraySize": SimTypeInt(signed=False, label="UInt32"), "outboundEnterpriseStoreCriteria": SimTypePointer(SimStruct({"certData": SimStruct({"size": SimTypeInt(signed=False, label="UInt32"), "data": SimTypePointer(SimTypeChar(label="Byte"), offset=0)}, name="FWP_BYTE_BLOB", pack=False, align=None), "certHash": SimStruct({"size": SimTypeInt(signed=False, label="UInt32"), "data": SimTypePointer(SimTypeChar(label="Byte"), offset=0)}, name="FWP_BYTE_BLOB", pack=False, align=None), "eku": SimTypePointer(SimStruct({"numEku": SimTypeInt(signed=False, label="UInt32"), "eku": SimTypePointer(SimTypePointer(SimTypeChar(label="Byte"), offset=0), offset=0)}, name="IKEEXT_CERT_EKUS0", pack=False, align=None), offset=0), "name": SimTypePointer(SimStruct({"nameType": SimTypeInt(signed=False, label="IKEEXT_CERT_CRITERIA_NAME_TYPE"), "certName": SimTypePointer(SimTypeChar(label="Char"), offset=0)}, name="IKEEXT_CERT_NAME0", pack=False, align=None), offset=0), "flags": SimTypeInt(signed=False, label="UInt32")}, name="IKEEXT_CERTIFICATE_CRITERIA0", pack=False, align=None), offset=0)}, name="_Anonymous2_e__Struct", pack=False, align=None), "Anonymous3": SimStruct({"outboundRootStoreArraySize": SimTypeInt(signed=False, label="UInt32"), "outboundTrustedRootStoreCriteria": SimTypePointer(SimStruct({"certData": SimStruct({"size": SimTypeInt(signed=False, label="UInt32"), "data": SimTypePointer(SimTypeChar(label="Byte"), offset=0)}, name="FWP_BYTE_BLOB", pack=False, align=None), "certHash": SimStruct({"size": SimTypeInt(signed=False, label="UInt32"), "data": SimTypePointer(SimTypeChar(label="Byte"), offset=0)}, name="FWP_BYTE_BLOB", pack=False, align=None), "eku": SimTypePointer(SimStruct({"numEku": SimTypeInt(signed=False, label="UInt32"), "eku": SimTypePointer(SimTypePointer(SimTypeChar(label="Byte"), offset=0), offset=0)}, name="IKEEXT_CERT_EKUS0", pack=False, align=None), offset=0), "name": SimTypePointer(SimStruct({"nameType": SimTypeInt(signed=False, label="IKEEXT_CERT_CRITERIA_NAME_TYPE"), "certName": SimTypePointer(SimTypeChar(label="Char"), offset=0)}, name="IKEEXT_CERT_NAME0", pack=False, align=None), offset=0), "flags": SimTypeInt(signed=False, label="UInt32")}, name="IKEEXT_CERTIFICATE_CRITERIA0", pack=False, align=None), offset=0)}, name="_Anonymous3_e__Struct", pack=False, align=None)}, name="<anon>", label="None"), "flags": SimTypeInt(signed=False, label="IKEEXT_CERT_AUTH"), "localCertLocationUrl": SimStruct({"size": SimTypeInt(signed=False, label="UInt32"), "data": SimTypePointer(SimTypeChar(label="Byte"), offset=0)}, name="FWP_BYTE_BLOB", pack=False, align=None)}, name="IKEEXT_CERTIFICATE_AUTHENTICATION2", pack=False, align=None), "cgaAuthentication": SimStruct({"keyContainerName": SimTypePointer(SimTypeChar(label="Char"), offset=0), "cspName": SimTypePointer(SimTypeChar(label="Char"), offset=0), "cspType": SimTypeInt(signed=False, label="UInt32"), "cgaModifier": SimStruct({"byteArray16": SimTypeFixedSizeArray(SimTypeChar(label="Byte"), 16)}, name="FWP_BYTE_ARRAY16", pack=False, align=None), "cgaCollisionCount": SimTypeChar(label="Byte")}, name="IKEEXT_IPV6_CGA_AUTHENTICATION0", pack=False, align=None), "eapAuthentication": SimStruct({"flags": SimTypeInt(signed=False, label="IKEEXT_EAP_AUTHENTICATION_FLAGS")}, name="IKEEXT_EAP_AUTHENTICATION0", pack=False, align=None)}, name="<anon>", label="None")}, name="IKEEXT_AUTHENTICATION_METHOD2", pack=False, align=None), offset=0), "initiatorImpersonationType": SimTypeInt(signed=False, label="IKEEXT_AUTHENTICATION_IMPERSONATION_TYPE"), "numIkeProposals": SimTypeInt(signed=False, label="UInt32"), "ikeProposals": SimTypePointer(SimStruct({"cipherAlgorithm": SimStruct({"algoIdentifier": SimTypeInt(signed=False, label="IKEEXT_CIPHER_TYPE"), "keyLen": SimTypeInt(signed=False, label="UInt32"), "rounds": SimTypeInt(signed=False, label="UInt32")}, name="IKEEXT_CIPHER_ALGORITHM0", pack=False, align=None), "integrityAlgorithm": SimStruct({"algoIdentifier": SimTypeInt(signed=False, label="IKEEXT_INTEGRITY_TYPE")}, name="IKEEXT_INTEGRITY_ALGORITHM0", pack=False, align=None), "maxLifetimeSeconds": SimTypeInt(signed=False, label="UInt32"), "dhGroup": SimTypeInt(signed=False, label="IKEEXT_DH_GROUP"), "quickModeLimit": SimTypeInt(signed=False, label="UInt32")}, name="IKEEXT_PROPOSAL0", pack=False, align=None), offset=0), "flags": SimTypeInt(signed=False, label="IKEEXT_POLICY_FLAG"), "maxDynamicFilters": SimTypeInt(signed=False, label="UInt32"), "retransmitDurationSecs": SimTypeInt(signed=False, label="UInt32")}, name="IKEEXT_POLICY2", pack=False, align=None), offset=0), "dataBuffer": SimTypePointer(SimStruct({"size": SimTypeInt(signed=False, label="UInt32"), "data": SimTypePointer(SimTypeChar(label="Byte"), offset=0)}, name="FWP_BYTE_BLOB", pack=False, align=None), offset=0), "classifyOptions": SimTypePointer(SimStruct({"numOptions": SimTypeInt(signed=False, label="UInt32"), "options": SimTypePointer(SimStruct({"type": SimTypeInt(signed=False, label="FWP_CLASSIFY_OPTION_TYPE"), "value": SimStruct({"type": SimTypeInt(signed=False, label="FWP_DATA_TYPE"), "Anonymous": SimUnion({"uint8": SimTypeChar(label="Byte"), "uint16": SimTypeShort(signed=False, label="UInt16"), "uint32": SimTypeInt(signed=False, label="UInt32"), "uint64": SimTypePointer(SimTypeLongLong(signed=False, label="UInt64"), offset=0), "int8": SimTypeChar(label="SByte"), "int16": SimTypeShort(signed=True, label="Int16"), "int32": SimTypeInt(signed=True, label="Int32"), "int64": SimTypePointer(SimTypeLongLong(signed=True, label="Int64"), offset=0), "float32": SimTypeFloat(size=32), "double64": SimTypePointer(SimTypeFloat(size=64), offset=0), "byteArray16": SimTypePointer(SimStruct({"byteArray16": SimTypeFixedSizeArray(SimTypeChar(label="Byte"), 16)}, name="FWP_BYTE_ARRAY16", pack=False, align=None), offset=0), "byteBlob": SimTypePointer(SimStruct({"size": SimTypeInt(signed=False, label="UInt32"), "data": SimTypePointer(SimTypeChar(label="Byte"), offset=0)}, name="FWP_BYTE_BLOB", pack=False, align=None), offset=0), "sid": SimTypePointer(SimTypeBottom(label="SID"), offset=0), "sd": SimTypePointer(SimStruct({"size": SimTypeInt(signed=False, label="UInt32"), "data": SimTypePointer(SimTypeChar(label="Byte"), offset=0)}, name="FWP_BYTE_BLOB", pack=False, align=None), offset=0), "tokenInformation": SimTypePointer(SimStruct({"sidCount": SimTypeInt(signed=False, label="UInt32"), "sids": SimTypePointer(SimTypeBottom(label="SID_AND_ATTRIBUTES"), offset=0), "restrictedSidCount": SimTypeInt(signed=False, label="UInt32"), "restrictedSids": SimTypePointer(SimTypeBottom(label="SID_AND_ATTRIBUTES"), offset=0)}, name="FWP_TOKEN_INFORMATION", pack=False, align=None), offset=0), "tokenAccessInformation": SimTypePointer(SimStruct({"size": SimTypeInt(signed=False, label="UInt32"), "data": SimTypePointer(SimTypeChar(label="Byte"), offset=0)}, name="FWP_BYTE_BLOB", pack=False, align=None), offset=0), "unicodeString": SimTypePointer(SimTypeChar(label="Char"), offset=0), "byteArray6": SimTypePointer(SimStruct({"byteArray6": SimTypeFixedSizeArray(SimTypeChar(label="Byte"), 6)}, name="FWP_BYTE_ARRAY6", pack=False, align=None), offset=0), "bitmapArray64": SimTypePointer(SimStruct({"bitmapArray64": SimTypeFixedSizeArray(SimTypeChar(label="Byte"), 8)}, name="FWP_BITMAP_ARRAY64_", pack=False, align=None), offset=0)}, name="<anon>", label="None")}, name="FWP_VALUE0", pack=False, align=None)}, name="FWPM_CLASSIFY_OPTION0", pack=False, align=None), offset=0)}, name="FWPM_CLASSIFY_OPTIONS0", pack=False, align=None), offset=0), "ikeV2QmTunnelPolicy": SimTypePointer(SimStruct({"flags": SimTypeInt(signed=False, label="UInt32"), "numIpsecProposals": SimTypeInt(signed=False, label="UInt32"), "ipsecProposals": SimTypePointer(SimStruct({"lifetime": SimStruct({"lifetimeSeconds": SimTypeInt(signed=False, label="UInt32"), "lifetimeKilobytes": SimTypeInt(signed=False, label="UInt32"), "lifetimePackets": SimTypeInt(signed=False, label="UInt32")}, name="IPSEC_SA_LIFETIME0", pack=False, align=None), "numSaTransforms": SimTypeInt(signed=False, label="UInt32"), "saTransforms": SimTypePointer(SimStruct({"ipsecTransformType": SimTypeInt(signed=False, label="IPSEC_TRANSFORM_TYPE"), "Anonymous": SimUnion({"ahTransform": SimTypePointer(SimStruct({"authTransformId": SimStruct({"authType": SimTypeInt(signed=False, label="IPSEC_AUTH_TYPE"), "authConfig": SimTypeChar(label="Byte")}, name="IPSEC_AUTH_TRANSFORM_ID0", pack=False, align=None), "cryptoModuleId": SimTypePointer(SimTypeBottom(label="Guid"), offset=0)}, name="IPSEC_AUTH_TRANSFORM0", pack=False, align=None), offset=0), "espAuthTransform": SimTypePointer(SimStruct({"authTransformId": SimStruct({"authType": SimTypeInt(signed=False, label="IPSEC_AUTH_TYPE"), "authConfig": SimTypeChar(label="Byte")}, name="IPSEC_AUTH_TRANSFORM_ID0", pack=False, align=None), "cryptoModuleId": SimTypePointer(SimTypeBottom(label="Guid"), offset=0)}, name="IPSEC_AUTH_TRANSFORM0", pack=False, align=None), offset=0), "espCipherTransform": SimTypePointer(SimStruct({"cipherTransformId": SimStruct({"cipherType": SimTypeInt(signed=False, label="IPSEC_CIPHER_TYPE"), "cipherConfig": SimTypeChar(label="Byte")}, name="IPSEC_CIPHER_TRANSFORM_ID0", pack=False, align=None), "cryptoModuleId": SimTypePointer(SimTypeBottom(label="Guid"), offset=0)}, name="IPSEC_CIPHER_TRANSFORM0", pack=False, align=None), offset=0), "espAuthAndCipherTransform": SimTypePointer(SimStruct({"authTransform": SimStruct({"authTransformId": SimStruct({"authType": SimTypeInt(signed=False, label="IPSEC_AUTH_TYPE"), "authConfig": SimTypeChar(label="Byte")}, name="IPSEC_AUTH_TRANSFORM_ID0", pack=False, align=None), "cryptoModuleId": SimTypePointer(SimTypeBottom(label="Guid"), offset=0)}, name="IPSEC_AUTH_TRANSFORM0", pack=False, align=None), "cipherTransform": SimStruct({"cipherTransformId": SimStruct({"cipherType": SimTypeInt(signed=False, label="IPSEC_CIPHER_TYPE"), "cipherConfig": SimTypeChar(label="Byte")}, name="IPSEC_CIPHER_TRANSFORM_ID0", pack=False, align=None), "cryptoModuleId": SimTypePointer(SimTypeBottom(label="Guid"), offset=0)}, name="IPSEC_CIPHER_TRANSFORM0", pack=False, align=None)}, name="IPSEC_AUTH_AND_CIPHER_TRANSFORM0", pack=False, align=None), offset=0), "espAuthFwTransform": SimTypePointer(SimStruct({"authTransformId": SimStruct({"authType": SimTypeInt(signed=False, label="IPSEC_AUTH_TYPE"), "authConfig": SimTypeChar(label="Byte")}, name="IPSEC_AUTH_TRANSFORM_ID0", pack=False, align=None), "cryptoModuleId": SimTypePointer(SimTypeBottom(label="Guid"), offset=0)}, name="IPSEC_AUTH_TRANSFORM0", pack=False, align=None), offset=0)}, name="<anon>", label="None")}, name="IPSEC_SA_TRANSFORM0", pack=False, align=None), offset=0), "pfsGroup": SimTypeInt(signed=False, label="IPSEC_PFS_GROUP")}, name="IPSEC_PROPOSAL0", pack=False, align=None), offset=0), "tunnelEndpoints": SimStruct({"ipVersion": SimTypeInt(signed=False, label="FWP_IP_VERSION"), "Anonymous1": SimUnion({"localV4Address": SimTypeInt(signed=False, label="UInt32"), "localV6Address": SimTypeFixedSizeArray(SimTypeChar(label="Byte"), 16)}, name="<anon>", label="None"), "Anonymous2": SimUnion({"remoteV4Address": SimTypeInt(signed=False, label="UInt32"), "remoteV6Address": SimTypeFixedSizeArray(SimTypeChar(label="Byte"), 16)}, name="<anon>", label="None"), "localIfLuid": SimTypeLongLong(signed=False, label="UInt64"), "remoteFqdn": SimTypePointer(SimTypeChar(label="Char"), offset=0), "numAddresses": SimTypeInt(signed=False, label="UInt32"), "remoteAddresses": SimTypePointer(SimStruct({"ipVersion": SimTypeInt(signed=False, label="FWP_IP_VERSION"), "Anonymous": SimUnion({"v4Address": SimTypeInt(signed=False, label="UInt32"), "v6Address": SimTypeFixedSizeArray(SimTypeChar(label="Byte"), 16)}, name="<anon>", label="None")}, name="IPSEC_TUNNEL_ENDPOINT0", pack=False, align=None), offset=0)}, name="IPSEC_TUNNEL_ENDPOINTS2", pack=False, align=None), "saIdleTimeout": SimStruct({"idleTimeoutSeconds": SimTypeInt(signed=False, label="UInt32"), "idleTimeoutSecondsFailOver": SimTypeInt(signed=False, label="UInt32")}, name="IPSEC_SA_IDLE_TIMEOUT0", pack=False, align=None), "emPolicy": SimTypePointer(SimStruct({"numAuthenticationMethods": SimTypeInt(signed=False, label="UInt32"), "authenticationMethods": SimTypePointer(SimStruct({"authenticationMethodType": SimTypeInt(signed=False, label="IKEEXT_AUTHENTICATION_METHOD_TYPE"), "Anonymous": SimUnion({"presharedKeyAuthentication": SimStruct({"presharedKey": SimStruct({"size": SimTypeInt(signed=False, label="UInt32"), "data": SimTypePointer(SimTypeChar(label="Byte"), offset=0)}, name="FWP_BYTE_BLOB", pack=False, align=None), "flags": SimTypeInt(signed=False, label="IKEEXT_PRESHARED_KEY_AUTHENTICATION_FLAGS")}, name="IKEEXT_PRESHARED_KEY_AUTHENTICATION1", pack=False, align=None), "certificateAuthentication": SimStruct({"inboundConfigType": SimTypeInt(signed=False, label="IKEEXT_CERT_CONFIG_TYPE"), "Anonymous1": SimUnion({"Anonymous1": SimStruct({"inboundRootArraySize": SimTypeInt(signed=False, label="UInt32"), "inboundRootCriteria": SimTypePointer(SimStruct({"certData": SimStruct({"size": SimTypeInt(signed=False, label="UInt32"), "data": SimTypePointer(SimTypeChar(label="Byte"), offset=0)}, name="FWP_BYTE_BLOB", pack=False, align=None), "certHash": SimStruct({"size": SimTypeInt(signed=False, label="UInt32"), "data": SimTypePointer(SimTypeChar(label="Byte"), offset=0)}, name="FWP_BYTE_BLOB", pack=False, align=None), "eku": SimTypePointer(SimStruct({"numEku": SimTypeInt(signed=False, label="UInt32"), "eku": SimTypePointer(SimTypePointer(SimTypeChar(label="Byte"), offset=0), offset=0)}, name="IKEEXT_CERT_EKUS0", pack=False, align=None), offset=0), "name": SimTypePointer(SimStruct({"nameType": SimTypeInt(signed=False, label="IKEEXT_CERT_CRITERIA_NAME_TYPE"), "certName": SimTypePointer(SimTypeChar(label="Char"), offset=0)}, name="IKEEXT_CERT_NAME0", pack=False, align=None), offset=0), "flags": SimTypeInt(signed=False, label="UInt32")}, name="IKEEXT_CERTIFICATE_CRITERIA0", pack=False, align=None), offset=0)}, name="_Anonymous1_e__Struct", pack=False, align=None), "Anonymous2": SimStruct({"inboundEnterpriseStoreArraySize": SimTypeInt(signed=False, label="UInt32"), "inboundEnterpriseStoreCriteria": SimTypePointer(SimStruct({"certData": SimStruct({"size": SimTypeInt(signed=False, label="UInt32"), "data": SimTypePointer(SimTypeChar(label="Byte"), offset=0)}, name="FWP_BYTE_BLOB", pack=False, align=None), "certHash": SimStruct({"size": SimTypeInt(signed=False, label="UInt32"), "data": SimTypePointer(SimTypeChar(label="Byte"), offset=0)}, name="FWP_BYTE_BLOB", pack=False, align=None), "eku": SimTypePointer(SimStruct({"numEku": SimTypeInt(signed=False, label="UInt32"), "eku": SimTypePointer(SimTypePointer(SimTypeChar(label="Byte"), offset=0), offset=0)}, name="IKEEXT_CERT_EKUS0", pack=False, align=None), offset=0), "name": SimTypePointer(SimStruct({"nameType": SimTypeInt(signed=False, label="IKEEXT_CERT_CRITERIA_NAME_TYPE"), "certName": SimTypePointer(SimTypeChar(label="Char"), offset=0)}, name="IKEEXT_CERT_NAME0", pack=False, align=None), offset=0), "flags": SimTypeInt(signed=False, label="UInt32")}, name="IKEEXT_CERTIFICATE_CRITERIA0", pack=False, align=None), offset=0)}, name="_Anonymous2_e__Struct", pack=False, align=None), "Anonymous3": SimStruct({"inboundRootStoreArraySize": SimTypeInt(signed=False, label="UInt32"), "inboundTrustedRootStoreCriteria": SimTypePointer(SimStruct({"certData": SimStruct({"size": SimTypeInt(signed=False, label="UInt32"), "data": SimTypePointer(SimTypeChar(label="Byte"), offset=0)}, name="FWP_BYTE_BLOB", pack=False, align=None), "certHash": SimStruct({"size": SimTypeInt(signed=False, label="UInt32"), "data": SimTypePointer(SimTypeChar(label="Byte"), offset=0)}, name="FWP_BYTE_BLOB", pack=False, align=None), "eku": SimTypePointer(SimStruct({"numEku": SimTypeInt(signed=False, label="UInt32"), "eku": SimTypePointer(SimTypePointer(SimTypeChar(label="Byte"), offset=0), offset=0)}, name="IKEEXT_CERT_EKUS0", pack=False, align=None), offset=0), "name": SimTypePointer(SimStruct({"nameType": SimTypeInt(signed=False, label="IKEEXT_CERT_CRITERIA_NAME_TYPE"), "certName": SimTypePointer(SimTypeChar(label="Char"), offset=0)}, name="IKEEXT_CERT_NAME0", pack=False, align=None), offset=0), "flags": SimTypeInt(signed=False, label="UInt32")}, name="IKEEXT_CERTIFICATE_CRITERIA0", pack=False, align=None), offset=0)}, name="_Anonymous3_e__Struct", pack=False, align=None)}, name="<anon>", label="None"), "outboundConfigType": SimTypeInt(signed=False, label="IKEEXT_CERT_CONFIG_TYPE"), "Anonymous2": SimUnion({"Anonymous1": SimStruct({"outboundRootArraySize": SimTypeInt(signed=False, label="UInt32"), "outboundRootCriteria": SimTypePointer(SimStruct({"certData": SimStruct({"size": SimTypeInt(signed=False, label="UInt32"), "data": SimTypePointer(SimTypeChar(label="Byte"), offset=0)}, name="FWP_BYTE_BLOB", pack=False, align=None), "certHash": SimStruct({"size": SimTypeInt(signed=False, label="UInt32"), "data": SimTypePointer(SimTypeChar(label="Byte"), offset=0)}, name="FWP_BYTE_BLOB", pack=False, align=None), "eku": SimTypePointer(SimStruct({"numEku": SimTypeInt(signed=False, label="UInt32"), "eku": SimTypePointer(SimTypePointer(SimTypeChar(label="Byte"), offset=0), offset=0)}, name="IKEEXT_CERT_EKUS0", pack=False, align=None), offset=0), "name": SimTypePointer(SimStruct({"nameType": SimTypeInt(signed=False, label="IKEEXT_CERT_CRITERIA_NAME_TYPE"), "certName": SimTypePointer(SimTypeChar(label="Char"), offset=0)}, name="IKEEXT_CERT_NAME0", pack=False, align=None), offset=0), "flags": SimTypeInt(signed=False, label="UInt32")}, name="IKEEXT_CERTIFICATE_CRITERIA0", pack=False, align=None), offset=0)}, name="_Anonymous1_e__Struct", pack=False, align=None), "Anonymous2": SimStruct({"outboundEnterpriseStoreArraySize": SimTypeInt(signed=False, label="UInt32"), "outboundEnterpriseStoreCriteria": SimTypePointer(SimStruct({"certData": SimStruct({"size": SimTypeInt(signed=False, label="UInt32"), "data": SimTypePointer(SimTypeChar(label="Byte"), offset=0)}, name="FWP_BYTE_BLOB", pack=False, align=None), "certHash": SimStruct({"size": SimTypeInt(signed=False, label="UInt32"), "data": SimTypePointer(SimTypeChar(label="Byte"), offset=0)}, name="FWP_BYTE_BLOB", pack=False, align=None), "eku": SimTypePointer(SimStruct({"numEku": SimTypeInt(signed=False, label="UInt32"), "eku": SimTypePointer(SimTypePointer(SimTypeChar(label="Byte"), offset=0), offset=0)}, name="IKEEXT_CERT_EKUS0", pack=False, align=None), offset=0), "name": SimTypePointer(SimStruct({"nameType": SimTypeInt(signed=False, label="IKEEXT_CERT_CRITERIA_NAME_TYPE"), "certName": SimTypePointer(SimTypeChar(label="Char"), offset=0)}, name="IKEEXT_CERT_NAME0", pack=False, align=None), offset=0), "flags": SimTypeInt(signed=False, label="UInt32")}, name="IKEEXT_CERTIFICATE_CRITERIA0", pack=False, align=None), offset=0)}, name="_Anonymous2_e__Struct", pack=False, align=None), "Anonymous3": SimStruct({"outboundRootStoreArraySize": SimTypeInt(signed=False, label="UInt32"), "outboundTrustedRootStoreCriteria": SimTypePointer(SimStruct({"certData": SimStruct({"size": SimTypeInt(signed=False, label="UInt32"), "data": SimTypePointer(SimTypeChar(label="Byte"), offset=0)}, name="FWP_BYTE_BLOB", pack=False, align=None), "certHash": SimStruct({"size": SimTypeInt(signed=False, label="UInt32"), "data": SimTypePointer(SimTypeChar(label="Byte"), offset=0)}, name="FWP_BYTE_BLOB", pack=False, align=None), "eku": SimTypePointer(SimStruct({"numEku": SimTypeInt(signed=False, label="UInt32"), "eku": SimTypePointer(SimTypePointer(SimTypeChar(label="Byte"), offset=0), offset=0)}, name="IKEEXT_CERT_EKUS0", pack=False, align=None), offset=0), "name": SimTypePointer(SimStruct({"nameType": SimTypeInt(signed=False, label="IKEEXT_CERT_CRITERIA_NAME_TYPE"), "certName": SimTypePointer(SimTypeChar(label="Char"), offset=0)}, name="IKEEXT_CERT_NAME0", pack=False, align=None), offset=0), "flags": SimTypeInt(signed=False, label="UInt32")}, name="IKEEXT_CERTIFICATE_CRITERIA0", pack=False, align=None), offset=0)}, name="_Anonymous3_e__Struct", pack=False, align=None)}, name="<anon>", label="None"), "flags": SimTypeInt(signed=False, label="IKEEXT_CERT_AUTH"), "localCertLocationUrl": SimStruct({"size": SimTypeInt(signed=False, label="UInt32"), "data": SimTypePointer(SimTypeChar(label="Byte"), offset=0)}, name="FWP_BYTE_BLOB", pack=False, align=None)}, name="IKEEXT_CERTIFICATE_AUTHENTICATION2", pack=False, align=None), "kerberosAuthentication": SimStruct({"flags": SimTypeInt(signed=False, label="IKEEXT_KERBEROS_AUTHENTICATION_FLAGS"), "proxyServer": SimTypePointer(SimTypeChar(label="Char"), offset=0)}, name="IKEEXT_KERBEROS_AUTHENTICATION1", pack=False, align=None), "reservedAuthentication": SimStruct({"flags": SimTypeInt(signed=False, label="IKEEXT_RESERVED_AUTHENTICATION_FLAGS")}, name="IKEEXT_RESERVED_AUTHENTICATION0", pack=False, align=None), "ntlmV2Authentication": SimStruct({"flags": SimTypeInt(signed=False, label="UInt32")}, name="IKEEXT_NTLM_V2_AUTHENTICATION0", pack=False, align=None), "sslAuthentication": SimStruct({"inboundConfigType": SimTypeInt(signed=False, label="IKEEXT_CERT_CONFIG_TYPE"), "Anonymous1": SimUnion({"Anonymous1": SimStruct({"inboundRootArraySize": SimTypeInt(signed=False, label="UInt32"), "inboundRootCriteria": SimTypePointer(SimStruct({"certData": SimStruct({"size": SimTypeInt(signed=False, label="UInt32"), "data": SimTypePointer(SimTypeChar(label="Byte"), offset=0)}, name="FWP_BYTE_BLOB", pack=False, align=None), "certHash": SimStruct({"size": SimTypeInt(signed=False, label="UInt32"), "data": SimTypePointer(SimTypeChar(label="Byte"), offset=0)}, name="FWP_BYTE_BLOB", pack=False, align=None), "eku": SimTypePointer(SimStruct({"numEku": SimTypeInt(signed=False, label="UInt32"), "eku": SimTypePointer(SimTypePointer(SimTypeChar(label="Byte"), offset=0), offset=0)}, name="IKEEXT_CERT_EKUS0", pack=False, align=None), offset=0), "name": SimTypePointer(SimStruct({"nameType": SimTypeInt(signed=False, label="IKEEXT_CERT_CRITERIA_NAME_TYPE"), "certName": SimTypePointer(SimTypeChar(label="Char"), offset=0)}, name="IKEEXT_CERT_NAME0", pack=False, align=None), offset=0), "flags": SimTypeInt(signed=False, label="UInt32")}, name="IKEEXT_CERTIFICATE_CRITERIA0", pack=False, align=None), offset=0)}, name="_Anonymous1_e__Struct", pack=False, align=None), "Anonymous2": SimStruct({"inboundEnterpriseStoreArraySize": SimTypeInt(signed=False, label="UInt32"), "inboundEnterpriseStoreCriteria": SimTypePointer(SimStruct({"certData": SimStruct({"size": SimTypeInt(signed=False, label="UInt32"), "data": SimTypePointer(SimTypeChar(label="Byte"), offset=0)}, name="FWP_BYTE_BLOB", pack=False, align=None), "certHash": SimStruct({"size": SimTypeInt(signed=False, label="UInt32"), "data": SimTypePointer(SimTypeChar(label="Byte"), offset=0)}, name="FWP_BYTE_BLOB", pack=False, align=None), "eku": SimTypePointer(SimStruct({"numEku": SimTypeInt(signed=False, label="UInt32"), "eku": SimTypePointer(SimTypePointer(SimTypeChar(label="Byte"), offset=0), offset=0)}, name="IKEEXT_CERT_EKUS0", pack=False, align=None), offset=0), "name": SimTypePointer(SimStruct({"nameType": SimTypeInt(signed=False, label="IKEEXT_CERT_CRITERIA_NAME_TYPE"), "certName": SimTypePointer(SimTypeChar(label="Char"), offset=0)}, name="IKEEXT_CERT_NAME0", pack=False, align=None), offset=0), "flags": SimTypeInt(signed=False, label="UInt32")}, name="IKEEXT_CERTIFICATE_CRITERIA0", pack=False, align=None), offset=0)}, name="_Anonymous2_e__Struct", pack=False, align=None), "Anonymous3": SimStruct({"inboundRootStoreArraySize": SimTypeInt(signed=False, label="UInt32"), "inboundTrustedRootStoreCriteria": SimTypePointer(SimStruct({"certData": SimStruct({"size": SimTypeInt(signed=False, label="UInt32"), "data": SimTypePointer(SimTypeChar(label="Byte"), offset=0)}, name="FWP_BYTE_BLOB", pack=False, align=None), "certHash": SimStruct({"size": SimTypeInt(signed=False, label="UInt32"), "data": SimTypePointer(SimTypeChar(label="Byte"), offset=0)}, name="FWP_BYTE_BLOB", pack=False, align=None), "eku": SimTypePointer(SimStruct({"numEku": SimTypeInt(signed=False, label="UInt32"), "eku": SimTypePointer(SimTypePointer(SimTypeChar(label="Byte"), offset=0), offset=0)}, name="IKEEXT_CERT_EKUS0", pack=False, align=None), offset=0), "name": SimTypePointer(SimStruct({"nameType": SimTypeInt(signed=False, label="IKEEXT_CERT_CRITERIA_NAME_TYPE"), "certName": SimTypePointer(SimTypeChar(label="Char"), offset=0)}, name="IKEEXT_CERT_NAME0", pack=False, align=None), offset=0), "flags": SimTypeInt(signed=False, label="UInt32")}, name="IKEEXT_CERTIFICATE_CRITERIA0", pack=False, align=None), offset=0)}, name="_Anonymous3_e__Struct", pack=False, align=None)}, name="<anon>", label="None"), "outboundConfigType": SimTypeInt(signed=False, label="IKEEXT_CERT_CONFIG_TYPE"), "Anonymous2": SimUnion({"Anonymous1": SimStruct({"outboundRootArraySize": SimTypeInt(signed=False, label="UInt32"), "outboundRootCriteria": SimTypePointer(SimStruct({"certData": SimStruct({"size": SimTypeInt(signed=False, label="UInt32"), "data": SimTypePointer(SimTypeChar(label="Byte"), offset=0)}, name="FWP_BYTE_BLOB", pack=False, align=None), "certHash": SimStruct({"size": SimTypeInt(signed=False, label="UInt32"), "data": SimTypePointer(SimTypeChar(label="Byte"), offset=0)}, name="FWP_BYTE_BLOB", pack=False, align=None), "eku": SimTypePointer(SimStruct({"numEku": SimTypeInt(signed=False, label="UInt32"), "eku": SimTypePointer(SimTypePointer(SimTypeChar(label="Byte"), offset=0), offset=0)}, name="IKEEXT_CERT_EKUS0", pack=False, align=None), offset=0), "name": SimTypePointer(SimStruct({"nameType": SimTypeInt(signed=False, label="IKEEXT_CERT_CRITERIA_NAME_TYPE"), "certName": SimTypePointer(SimTypeChar(label="Char"), offset=0)}, name="IKEEXT_CERT_NAME0", pack=False, align=None), offset=0), "flags": SimTypeInt(signed=False, label="UInt32")}, name="IKEEXT_CERTIFICATE_CRITERIA0", pack=False, align=None), offset=0)}, name="_Anonymous1_e__Struct", pack=False, align=None), "Anonymous2": SimStruct({"outboundEnterpriseStoreArraySize": SimTypeInt(signed=False, label="UInt32"), "outboundEnterpriseStoreCriteria": SimTypePointer(SimStruct({"certData": SimStruct({"size": SimTypeInt(signed=False, label="UInt32"), "data": SimTypePointer(SimTypeChar(label="Byte"), offset=0)}, name="FWP_BYTE_BLOB", pack=False, align=None), "certHash": SimStruct({"size": SimTypeInt(signed=False, label="UInt32"), "data": SimTypePointer(SimTypeChar(label="Byte"), offset=0)}, name="FWP_BYTE_BLOB", pack=False, align=None), "eku": SimTypePointer(SimStruct({"numEku": SimTypeInt(signed=False, label="UInt32"), "eku": SimTypePointer(SimTypePointer(SimTypeChar(label="Byte"), offset=0), offset=0)}, name="IKEEXT_CERT_EKUS0", pack=False, align=None), offset=0), "name": SimTypePointer(SimStruct({"nameType": SimTypeInt(signed=False, label="IKEEXT_CERT_CRITERIA_NAME_TYPE"), "certName": SimTypePointer(SimTypeChar(label="Char"), offset=0)}, name="IKEEXT_CERT_NAME0", pack=False, align=None), offset=0), "flags": SimTypeInt(signed=False, label="UInt32")}, name="IKEEXT_CERTIFICATE_CRITERIA0", pack=False, align=None), offset=0)}, name="_Anonymous2_e__Struct", pack=False, align=None), "Anonymous3": SimStruct({"outboundRootStoreArraySize": SimTypeInt(signed=False, label="UInt32"), "outboundTrustedRootStoreCriteria": SimTypePointer(SimStruct({"certData": SimStruct({"size": SimTypeInt(signed=False, label="UInt32"), "data": SimTypePointer(SimTypeChar(label="Byte"), offset=0)}, name="FWP_BYTE_BLOB", pack=False, align=None), "certHash": SimStruct({"size": SimTypeInt(signed=False, label="UInt32"), "data": SimTypePointer(SimTypeChar(label="Byte"), offset=0)}, name="FWP_BYTE_BLOB", pack=False, align=None), "eku": SimTypePointer(SimStruct({"numEku": SimTypeInt(signed=False, label="UInt32"), "eku": SimTypePointer(SimTypePointer(SimTypeChar(label="Byte"), offset=0), offset=0)}, name="IKEEXT_CERT_EKUS0", pack=False, align=None), offset=0), "name": SimTypePointer(SimStruct({"nameType": SimTypeInt(signed=False, label="IKEEXT_CERT_CRITERIA_NAME_TYPE"), "certName": SimTypePointer(SimTypeChar(label="Char"), offset=0)}, name="IKEEXT_CERT_NAME0", pack=False, align=None), offset=0), "flags": SimTypeInt(signed=False, label="UInt32")}, name="IKEEXT_CERTIFICATE_CRITERIA0", pack=False, align=None), offset=0)}, name="_Anonymous3_e__Struct", pack=False, align=None)}, name="<anon>", label="None"), "flags": SimTypeInt(signed=False, label="IKEEXT_CERT_AUTH"), "localCertLocationUrl": SimStruct({"size": SimTypeInt(signed=False, label="UInt32"), "data": SimTypePointer(SimTypeChar(label="Byte"), offset=0)}, name="FWP_BYTE_BLOB", pack=False, align=None)}, name="IKEEXT_CERTIFICATE_AUTHENTICATION2", pack=False, align=None), "cgaAuthentication": SimStruct({"keyContainerName": SimTypePointer(SimTypeChar(label="Char"), offset=0), "cspName": SimTypePointer(SimTypeChar(label="Char"), offset=0), "cspType": SimTypeInt(signed=False, label="UInt32"), "cgaModifier": SimStruct({"byteArray16": SimTypeFixedSizeArray(SimTypeChar(label="Byte"), 16)}, name="FWP_BYTE_ARRAY16", pack=False, align=None), "cgaCollisionCount": SimTypeChar(label="Byte")}, name="IKEEXT_IPV6_CGA_AUTHENTICATION0", pack=False, align=None), "eapAuthentication": SimStruct({"flags": SimTypeInt(signed=False, label="IKEEXT_EAP_AUTHENTICATION_FLAGS")}, name="IKEEXT_EAP_AUTHENTICATION0", pack=False, align=None)}, name="<anon>", label="None")}, name="IKEEXT_AUTHENTICATION_METHOD2", pack=False, align=None), offset=0), "initiatorImpersonationType": SimTypeInt(signed=False, label="IKEEXT_AUTHENTICATION_IMPERSONATION_TYPE")}, name="IKEEXT_EM_POLICY2", pack=False, align=None), offset=0), "fwdPathSaLifetime": SimTypeInt(signed=False, label="UInt32"), "compartmentId": SimTypeInt(signed=False, label="UInt32"), "numTrafficSelectorPolicy": SimTypeInt(signed=False, label="UInt32"), "trafficSelectorPolicies": SimTypePointer(SimStruct({"flags": SimTypeInt(signed=False, label="UInt32"), "numLocalTrafficSelectors": SimTypeInt(signed=False, label="UInt32"), "localTrafficSelectors": SimTypePointer(SimStruct({"protocolId": SimTypeChar(label="Byte"), "portStart": SimTypeShort(signed=False, label="UInt16"), "portEnd": SimTypeShort(signed=False, label="UInt16"), "ipVersion": SimTypeInt(signed=False, label="FWP_IP_VERSION"), "Anonymous1": SimUnion({"startV4Address": SimTypeInt(signed=False, label="UInt32"), "startV6Address": SimTypeFixedSizeArray(SimTypeChar(label="Byte"), 16)}, name="<anon>", label="None"), "Anonymous2": SimUnion({"endV4Address": SimTypeInt(signed=False, label="UInt32"), "endV6Address": SimTypeFixedSizeArray(SimTypeChar(label="Byte"), 16)}, name="<anon>", label="None")}, name="IPSEC_TRAFFIC_SELECTOR0_", pack=False, align=None), offset=0), "numRemoteTrafficSelectors": SimTypeInt(signed=False, label="UInt32"), "remoteTrafficSelectors": SimTypePointer(SimStruct({"protocolId": SimTypeChar(label="Byte"), "portStart": SimTypeShort(signed=False, label="UInt16"), "portEnd": SimTypeShort(signed=False, label="UInt16"), "ipVersion": SimTypeInt(signed=False, label="FWP_IP_VERSION"), "Anonymous1": SimUnion({"startV4Address": SimTypeInt(signed=False, label="UInt32"), "startV6Address": SimTypeFixedSizeArray(SimTypeChar(label="Byte"), 16)}, name="<anon>", label="None"), "Anonymous2": SimUnion({"endV4Address": SimTypeInt(signed=False, label="UInt32"), "endV6Address": SimTypeFixedSizeArray(SimTypeChar(label="Byte"), 16)}, name="<anon>", label="None")}, name="IPSEC_TRAFFIC_SELECTOR0_", pack=False, align=None), offset=0)}, name="IPSEC_TRAFFIC_SELECTOR_POLICY0_", pack=False, align=None), offset=0)}, name="IPSEC_TUNNEL_POLICY3_", pack=False, align=None), offset=0), "ikeV2QmTransportPolicy": SimTypePointer(SimStruct({"numIpsecProposals": SimTypeInt(signed=False, label="UInt32"), "ipsecProposals": SimTypePointer(SimStruct({"lifetime": SimStruct({"lifetimeSeconds": SimTypeInt(signed=False, label="UInt32"), "lifetimeKilobytes": SimTypeInt(signed=False, label="UInt32"), "lifetimePackets": SimTypeInt(signed=False, label="UInt32")}, name="IPSEC_SA_LIFETIME0", pack=False, align=None), "numSaTransforms": SimTypeInt(signed=False, label="UInt32"), "saTransforms": SimTypePointer(SimStruct({"ipsecTransformType": SimTypeInt(signed=False, label="IPSEC_TRANSFORM_TYPE"), "Anonymous": SimUnion({"ahTransform": SimTypePointer(SimStruct({"authTransformId": SimStruct({"authType": SimTypeInt(signed=False, label="IPSEC_AUTH_TYPE"), "authConfig": SimTypeChar(label="Byte")}, name="IPSEC_AUTH_TRANSFORM_ID0", pack=False, align=None), "cryptoModuleId": SimTypePointer(SimTypeBottom(label="Guid"), offset=0)}, name="IPSEC_AUTH_TRANSFORM0", pack=False, align=None), offset=0), "espAuthTransform": SimTypePointer(SimStruct({"authTransformId": SimStruct({"authType": SimTypeInt(signed=False, label="IPSEC_AUTH_TYPE"), "authConfig": SimTypeChar(label="Byte")}, name="IPSEC_AUTH_TRANSFORM_ID0", pack=False, align=None), "cryptoModuleId": SimTypePointer(SimTypeBottom(label="Guid"), offset=0)}, name="IPSEC_AUTH_TRANSFORM0", pack=False, align=None), offset=0), "espCipherTransform": SimTypePointer(SimStruct({"cipherTransformId": SimStruct({"cipherType": SimTypeInt(signed=False, label="IPSEC_CIPHER_TYPE"), "cipherConfig": SimTypeChar(label="Byte")}, name="IPSEC_CIPHER_TRANSFORM_ID0", pack=False, align=None), "cryptoModuleId": SimTypePointer(SimTypeBottom(label="Guid"), offset=0)}, name="IPSEC_CIPHER_TRANSFORM0", pack=False, align=None), offset=0), "espAuthAndCipherTransform": SimTypePointer(SimStruct({"authTransform": SimStruct({"authTransformId": SimStruct({"authType": SimTypeInt(signed=False, label="IPSEC_AUTH_TYPE"), "authConfig": SimTypeChar(label="Byte")}, name="IPSEC_AUTH_TRANSFORM_ID0", pack=False, align=None), "cryptoModuleId": SimTypePointer(SimTypeBottom(label="Guid"), offset=0)}, name="IPSEC_AUTH_TRANSFORM0", pack=False, align=None), "cipherTransform": SimStruct({"cipherTransformId": SimStruct({"cipherType": SimTypeInt(signed=False, label="IPSEC_CIPHER_TYPE"), "cipherConfig": SimTypeChar(label="Byte")}, name="IPSEC_CIPHER_TRANSFORM_ID0", pack=False, align=None), "cryptoModuleId": SimTypePointer(SimTypeBottom(label="Guid"), offset=0)}, name="IPSEC_CIPHER_TRANSFORM0", pack=False, align=None)}, name="IPSEC_AUTH_AND_CIPHER_TRANSFORM0", pack=False, align=None), offset=0), "espAuthFwTransform": SimTypePointer(SimStruct({"authTransformId": SimStruct({"authType": SimTypeInt(signed=False, label="IPSEC_AUTH_TYPE"), "authConfig": SimTypeChar(label="Byte")}, name="IPSEC_AUTH_TRANSFORM_ID0", pack=False, align=None), "cryptoModuleId": SimTypePointer(SimTypeBottom(label="Guid"), offset=0)}, name="IPSEC_AUTH_TRANSFORM0", pack=False, align=None), offset=0)}, name="<anon>", label="None")}, name="IPSEC_SA_TRANSFORM0", pack=False, align=None), offset=0), "pfsGroup": SimTypeInt(signed=False, label="IPSEC_PFS_GROUP")}, name="IPSEC_PROPOSAL0", pack=False, align=None), offset=0), "flags": SimTypeInt(signed=False, label="IPSEC_POLICY_FLAG"), "ndAllowClearTimeoutSeconds": SimTypeInt(signed=False, label="UInt32"), "saIdleTimeout": SimStruct({"idleTimeoutSeconds": SimTypeInt(signed=False, label="UInt32"), "idleTimeoutSecondsFailOver": SimTypeInt(signed=False, label="UInt32")}, name="IPSEC_SA_IDLE_TIMEOUT0", pack=False, align=None), "emPolicy": SimTypePointer(SimStruct({"numAuthenticationMethods": SimTypeInt(signed=False, label="UInt32"), "authenticationMethods": SimTypePointer(SimStruct({"authenticationMethodType": SimTypeInt(signed=False, label="IKEEXT_AUTHENTICATION_METHOD_TYPE"), "Anonymous": SimUnion({"presharedKeyAuthentication": SimStruct({"presharedKey": SimStruct({"size": SimTypeInt(signed=False, label="UInt32"), "data": SimTypePointer(SimTypeChar(label="Byte"), offset=0)}, name="FWP_BYTE_BLOB", pack=False, align=None), "flags": SimTypeInt(signed=False, label="IKEEXT_PRESHARED_KEY_AUTHENTICATION_FLAGS")}, name="IKEEXT_PRESHARED_KEY_AUTHENTICATION1", pack=False, align=None), "certificateAuthentication": SimStruct({"inboundConfigType": SimTypeInt(signed=False, label="IKEEXT_CERT_CONFIG_TYPE"), "Anonymous1": SimUnion({"Anonymous1": SimStruct({"inboundRootArraySize": SimTypeInt(signed=False, label="UInt32"), "inboundRootCriteria": SimTypePointer(SimStruct({"certData": SimStruct({"size": SimTypeInt(signed=False, label="UInt32"), "data": SimTypePointer(SimTypeChar(label="Byte"), offset=0)}, name="FWP_BYTE_BLOB", pack=False, align=None), "certHash": SimStruct({"size": SimTypeInt(signed=False, label="UInt32"), "data": SimTypePointer(SimTypeChar(label="Byte"), offset=0)}, name="FWP_BYTE_BLOB", pack=False, align=None), "eku": SimTypePointer(SimStruct({"numEku": SimTypeInt(signed=False, label="UInt32"), "eku": SimTypePointer(SimTypePointer(SimTypeChar(label="Byte"), offset=0), offset=0)}, name="IKEEXT_CERT_EKUS0", pack=False, align=None), offset=0), "name": SimTypePointer(SimStruct({"nameType": SimTypeInt(signed=False, label="IKEEXT_CERT_CRITERIA_NAME_TYPE"), "certName": SimTypePointer(SimTypeChar(label="Char"), offset=0)}, name="IKEEXT_CERT_NAME0", pack=False, align=None), offset=0), "flags": SimTypeInt(signed=False, label="UInt32")}, name="IKEEXT_CERTIFICATE_CRITERIA0", pack=False, align=None), offset=0)}, name="_Anonymous1_e__Struct", pack=False, align=None), "Anonymous2": SimStruct({"inboundEnterpriseStoreArraySize": SimTypeInt(signed=False, label="UInt32"), "inboundEnterpriseStoreCriteria": SimTypePointer(SimStruct({"certData": SimStruct({"size": SimTypeInt(signed=False, label="UInt32"), "data": SimTypePointer(SimTypeChar(label="Byte"), offset=0)}, name="FWP_BYTE_BLOB", pack=False, align=None), "certHash": SimStruct({"size": SimTypeInt(signed=False, label="UInt32"), "data": SimTypePointer(SimTypeChar(label="Byte"), offset=0)}, name="FWP_BYTE_BLOB", pack=False, align=None), "eku": SimTypePointer(SimStruct({"numEku": SimTypeInt(signed=False, label="UInt32"), "eku": SimTypePointer(SimTypePointer(SimTypeChar(label="Byte"), offset=0), offset=0)}, name="IKEEXT_CERT_EKUS0", pack=False, align=None), offset=0), "name": SimTypePointer(SimStruct({"nameType": SimTypeInt(signed=False, label="IKEEXT_CERT_CRITERIA_NAME_TYPE"), "certName": SimTypePointer(SimTypeChar(label="Char"), offset=0)}, name="IKEEXT_CERT_NAME0", pack=False, align=None), offset=0), "flags": SimTypeInt(signed=False, label="UInt32")}, name="IKEEXT_CERTIFICATE_CRITERIA0", pack=False, align=None), offset=0)}, name="_Anonymous2_e__Struct", pack=False, align=None), "Anonymous3": SimStruct({"inboundRootStoreArraySize": SimTypeInt(signed=False, label="UInt32"), "inboundTrustedRootStoreCriteria": SimTypePointer(SimStruct({"certData": SimStruct({"size": SimTypeInt(signed=False, label="UInt32"), "data": SimTypePointer(SimTypeChar(label="Byte"), offset=0)}, name="FWP_BYTE_BLOB", pack=False, align=None), "certHash": SimStruct({"size": SimTypeInt(signed=False, label="UInt32"), "data": SimTypePointer(SimTypeChar(label="Byte"), offset=0)}, name="FWP_BYTE_BLOB", pack=False, align=None), "eku": SimTypePointer(SimStruct({"numEku": SimTypeInt(signed=False, label="UInt32"), "eku": SimTypePointer(SimTypePointer(SimTypeChar(label="Byte"), offset=0), offset=0)}, name="IKEEXT_CERT_EKUS0", pack=False, align=None), offset=0), "name": SimTypePointer(SimStruct({"nameType": SimTypeInt(signed=False, label="IKEEXT_CERT_CRITERIA_NAME_TYPE"), "certName": SimTypePointer(SimTypeChar(label="Char"), offset=0)}, name="IKEEXT_CERT_NAME0", pack=False, align=None), offset=0), "flags": SimTypeInt(signed=False, label="UInt32")}, name="IKEEXT_CERTIFICATE_CRITERIA0", pack=False, align=None), offset=0)}, name="_Anonymous3_e__Struct", pack=False, align=None)}, name="<anon>", label="None"), "outboundConfigType": SimTypeInt(signed=False, label="IKEEXT_CERT_CONFIG_TYPE"), "Anonymous2": SimUnion({"Anonymous1": SimStruct({"outboundRootArraySize": SimTypeInt(signed=False, label="UInt32"), "outboundRootCriteria": SimTypePointer(SimStruct({"certData": SimStruct({"size": SimTypeInt(signed=False, label="UInt32"), "data": SimTypePointer(SimTypeChar(label="Byte"), offset=0)}, name="FWP_BYTE_BLOB", pack=False, align=None), "certHash": SimStruct({"size": SimTypeInt(signed=False, label="UInt32"), "data": SimTypePointer(SimTypeChar(label="Byte"), offset=0)}, name="FWP_BYTE_BLOB", pack=False, align=None), "eku": SimTypePointer(SimStruct({"numEku": SimTypeInt(signed=False, label="UInt32"), "eku": SimTypePointer(SimTypePointer(SimTypeChar(label="Byte"), offset=0), offset=0)}, name="IKEEXT_CERT_EKUS0", pack=False, align=None), offset=0), "name": SimTypePointer(SimStruct({"nameType": SimTypeInt(signed=False, label="IKEEXT_CERT_CRITERIA_NAME_TYPE"), "certName": SimTypePointer(SimTypeChar(label="Char"), offset=0)}, name="IKEEXT_CERT_NAME0", pack=False, align=None), offset=0), "flags": SimTypeInt(signed=False, label="UInt32")}, name="IKEEXT_CERTIFICATE_CRITERIA0", pack=False, align=None), offset=0)}, name="_Anonymous1_e__Struct", pack=False, align=None), "Anonymous2": SimStruct({"outboundEnterpriseStoreArraySize": SimTypeInt(signed=False, label="UInt32"), "outboundEnterpriseStoreCriteria": SimTypePointer(SimStruct({"certData": SimStruct({"size": SimTypeInt(signed=False, label="UInt32"), "data": SimTypePointer(SimTypeChar(label="Byte"), offset=0)}, name="FWP_BYTE_BLOB", pack=False, align=None), "certHash": SimStruct({"size": SimTypeInt(signed=False, label="UInt32"), "data": SimTypePointer(SimTypeChar(label="Byte"), offset=0)}, name="FWP_BYTE_BLOB", pack=False, align=None), "eku": SimTypePointer(SimStruct({"numEku": SimTypeInt(signed=False, label="UInt32"), "eku": SimTypePointer(SimTypePointer(SimTypeChar(label="Byte"), offset=0), offset=0)}, name="IKEEXT_CERT_EKUS0", pack=False, align=None), offset=0), "name": SimTypePointer(SimStruct({"nameType": SimTypeInt(signed=False, label="IKEEXT_CERT_CRITERIA_NAME_TYPE"), "certName": SimTypePointer(SimTypeChar(label="Char"), offset=0)}, name="IKEEXT_CERT_NAME0", pack=False, align=None), offset=0), "flags": SimTypeInt(signed=False, label="UInt32")}, name="IKEEXT_CERTIFICATE_CRITERIA0", pack=False, align=None), offset=0)}, name="_Anonymous2_e__Struct", pack=False, align=None), "Anonymous3": SimStruct({"outboundRootStoreArraySize": SimTypeInt(signed=False, label="UInt32"), "outboundTrustedRootStoreCriteria": SimTypePointer(SimStruct({"certData": SimStruct({"size": SimTypeInt(signed=False, label="UInt32"), "data": SimTypePointer(SimTypeChar(label="Byte"), offset=0)}, name="FWP_BYTE_BLOB", pack=False, align=None), "certHash": SimStruct({"size": SimTypeInt(signed=False, label="UInt32"), "data": SimTypePointer(SimTypeChar(label="Byte"), offset=0)}, name="FWP_BYTE_BLOB", pack=False, align=None), "eku": SimTypePointer(SimStruct({"numEku": SimTypeInt(signed=False, label="UInt32"), "eku": SimTypePointer(SimTypePointer(SimTypeChar(label="Byte"), offset=0), offset=0)}, name="IKEEXT_CERT_EKUS0", pack=False, align=None), offset=0), "name": SimTypePointer(SimStruct({"nameType": SimTypeInt(signed=False, label="IKEEXT_CERT_CRITERIA_NAME_TYPE"), "certName": SimTypePointer(SimTypeChar(label="Char"), offset=0)}, name="IKEEXT_CERT_NAME0", pack=False, align=None), offset=0), "flags": SimTypeInt(signed=False, label="UInt32")}, name="IKEEXT_CERTIFICATE_CRITERIA0", pack=False, align=None), offset=0)}, name="_Anonymous3_e__Struct", pack=False, align=None)}, name="<anon>", label="None"), "flags": SimTypeInt(signed=False, label="IKEEXT_CERT_AUTH"), "localCertLocationUrl": SimStruct({"size": SimTypeInt(signed=False, label="UInt32"), "data": SimTypePointer(SimTypeChar(label="Byte"), offset=0)}, name="FWP_BYTE_BLOB", pack=False, align=None)}, name="IKEEXT_CERTIFICATE_AUTHENTICATION2", pack=False, align=None), "kerberosAuthentication": SimStruct({"flags": SimTypeInt(signed=False, label="IKEEXT_KERBEROS_AUTHENTICATION_FLAGS"), "proxyServer": SimTypePointer(SimTypeChar(label="Char"), offset=0)}, name="IKEEXT_KERBEROS_AUTHENTICATION1", pack=False, align=None), "reservedAuthentication": SimStruct({"flags": SimTypeInt(signed=False, label="IKEEXT_RESERVED_AUTHENTICATION_FLAGS")}, name="IKEEXT_RESERVED_AUTHENTICATION0", pack=False, align=None), "ntlmV2Authentication": SimStruct({"flags": SimTypeInt(signed=False, label="UInt32")}, name="IKEEXT_NTLM_V2_AUTHENTICATION0", pack=False, align=None), "sslAuthentication": SimStruct({"inboundConfigType": SimTypeInt(signed=False, label="IKEEXT_CERT_CONFIG_TYPE"), "Anonymous1": SimUnion({"Anonymous1": SimStruct({"inboundRootArraySize": SimTypeInt(signed=False, label="UInt32"), "inboundRootCriteria": SimTypePointer(SimStruct({"certData": SimStruct({"size": SimTypeInt(signed=False, label="UInt32"), "data": SimTypePointer(SimTypeChar(label="Byte"), offset=0)}, name="FWP_BYTE_BLOB", pack=False, align=None), "certHash": SimStruct({"size": SimTypeInt(signed=False, label="UInt32"), "data": SimTypePointer(SimTypeChar(label="Byte"), offset=0)}, name="FWP_BYTE_BLOB", pack=False, align=None), "eku": SimTypePointer(SimStruct({"numEku": SimTypeInt(signed=False, label="UInt32"), "eku": SimTypePointer(SimTypePointer(SimTypeChar(label="Byte"), offset=0), offset=0)}, name="IKEEXT_CERT_EKUS0", pack=False, align=None), offset=0), "name": SimTypePointer(SimStruct({"nameType": SimTypeInt(signed=False, label="IKEEXT_CERT_CRITERIA_NAME_TYPE"), "certName": SimTypePointer(SimTypeChar(label="Char"), offset=0)}, name="IKEEXT_CERT_NAME0", pack=False, align=None), offset=0), "flags": SimTypeInt(signed=False, label="UInt32")}, name="IKEEXT_CERTIFICATE_CRITERIA0", pack=False, align=None), offset=0)}, name="_Anonymous1_e__Struct", pack=False, align=None), "Anonymous2": SimStruct({"inboundEnterpriseStoreArraySize": SimTypeInt(signed=False, label="UInt32"), "inboundEnterpriseStoreCriteria": SimTypePointer(SimStruct({"certData": SimStruct({"size": SimTypeInt(signed=False, label="UInt32"), "data": SimTypePointer(SimTypeChar(label="Byte"), offset=0)}, name="FWP_BYTE_BLOB", pack=False, align=None), "certHash": SimStruct({"size": SimTypeInt(signed=False, label="UInt32"), "data": SimTypePointer(SimTypeChar(label="Byte"), offset=0)}, name="FWP_BYTE_BLOB", pack=False, align=None), "eku": SimTypePointer(SimStruct({"numEku": SimTypeInt(signed=False, label="UInt32"), "eku": SimTypePointer(SimTypePointer(SimTypeChar(label="Byte"), offset=0), offset=0)}, name="IKEEXT_CERT_EKUS0", pack=False, align=None), offset=0), "name": SimTypePointer(SimStruct({"nameType": SimTypeInt(signed=False, label="IKEEXT_CERT_CRITERIA_NAME_TYPE"), "certName": SimTypePointer(SimTypeChar(label="Char"), offset=0)}, name="IKEEXT_CERT_NAME0", pack=False, align=None), offset=0), "flags": SimTypeInt(signed=False, label="UInt32")}, name="IKEEXT_CERTIFICATE_CRITERIA0", pack=False, align=None), offset=0)}, name="_Anonymous2_e__Struct", pack=False, align=None), "Anonymous3": SimStruct({"inboundRootStoreArraySize": SimTypeInt(signed=False, label="UInt32"), "inboundTrustedRootStoreCriteria": SimTypePointer(SimStruct({"certData": SimStruct({"size": SimTypeInt(signed=False, label="UInt32"), "data": SimTypePointer(SimTypeChar(label="Byte"), offset=0)}, name="FWP_BYTE_BLOB", pack=False, align=None), "certHash": SimStruct({"size": SimTypeInt(signed=False, label="UInt32"), "data": SimTypePointer(SimTypeChar(label="Byte"), offset=0)}, name="FWP_BYTE_BLOB", pack=False, align=None), "eku": SimTypePointer(SimStruct({"numEku": SimTypeInt(signed=False, label="UInt32"), "eku": SimTypePointer(SimTypePointer(SimTypeChar(label="Byte"), offset=0), offset=0)}, name="IKEEXT_CERT_EKUS0", pack=False, align=None), offset=0), "name": SimTypePointer(SimStruct({"nameType": SimTypeInt(signed=False, label="IKEEXT_CERT_CRITERIA_NAME_TYPE"), "certName": SimTypePointer(SimTypeChar(label="Char"), offset=0)}, name="IKEEXT_CERT_NAME0", pack=False, align=None), offset=0), "flags": SimTypeInt(signed=False, label="UInt32")}, name="IKEEXT_CERTIFICATE_CRITERIA0", pack=False, align=None), offset=0)}, name="_Anonymous3_e__Struct", pack=False, align=None)}, name="<anon>", label="None"), "outboundConfigType": SimTypeInt(signed=False, label="IKEEXT_CERT_CONFIG_TYPE"), "Anonymous2": SimUnion({"Anonymous1": SimStruct({"outboundRootArraySize": SimTypeInt(signed=False, label="UInt32"), "outboundRootCriteria": SimTypePointer(SimStruct({"certData": SimStruct({"size": SimTypeInt(signed=False, label="UInt32"), "data": SimTypePointer(SimTypeChar(label="Byte"), offset=0)}, name="FWP_BYTE_BLOB", pack=False, align=None), "certHash": SimStruct({"size": SimTypeInt(signed=False, label="UInt32"), "data": SimTypePointer(SimTypeChar(label="Byte"), offset=0)}, name="FWP_BYTE_BLOB", pack=False, align=None), "eku": SimTypePointer(SimStruct({"numEku": SimTypeInt(signed=False, label="UInt32"), "eku": SimTypePointer(SimTypePointer(SimTypeChar(label="Byte"), offset=0), offset=0)}, name="IKEEXT_CERT_EKUS0", pack=False, align=None), offset=0), "name": SimTypePointer(SimStruct({"nameType": SimTypeInt(signed=False, label="IKEEXT_CERT_CRITERIA_NAME_TYPE"), "certName": SimTypePointer(SimTypeChar(label="Char"), offset=0)}, name="IKEEXT_CERT_NAME0", pack=False, align=None), offset=0), "flags": SimTypeInt(signed=False, label="UInt32")}, name="IKEEXT_CERTIFICATE_CRITERIA0", pack=False, align=None), offset=0)}, name="_Anonymous1_e__Struct", pack=False, align=None), "Anonymous2": SimStruct({"outboundEnterpriseStoreArraySize": SimTypeInt(signed=False, label="UInt32"), "outboundEnterpriseStoreCriteria": SimTypePointer(SimStruct({"certData": SimStruct({"size": SimTypeInt(signed=False, label="UInt32"), "data": SimTypePointer(SimTypeChar(label="Byte"), offset=0)}, name="FWP_BYTE_BLOB", pack=False, align=None), "certHash": SimStruct({"size": SimTypeInt(signed=False, label="UInt32"), "data": SimTypePointer(SimTypeChar(label="Byte"), offset=0)}, name="FWP_BYTE_BLOB", pack=False, align=None), "eku": SimTypePointer(SimStruct({"numEku": SimTypeInt(signed=False, label="UInt32"), "eku": SimTypePointer(SimTypePointer(SimTypeChar(label="Byte"), offset=0), offset=0)}, name="IKEEXT_CERT_EKUS0", pack=False, align=None), offset=0), "name": SimTypePointer(SimStruct({"nameType": SimTypeInt(signed=False, label="IKEEXT_CERT_CRITERIA_NAME_TYPE"), "certName": SimTypePointer(SimTypeChar(label="Char"), offset=0)}, name="IKEEXT_CERT_NAME0", pack=False, align=None), offset=0), "flags": SimTypeInt(signed=False, label="UInt32")}, name="IKEEXT_CERTIFICATE_CRITERIA0", pack=False, align=None), offset=0)}, name="_Anonymous2_e__Struct", pack=False, align=None), "Anonymous3": SimStruct({"outboundRootStoreArraySize": SimTypeInt(signed=False, label="UInt32"), "outboundTrustedRootStoreCriteria": SimTypePointer(SimStruct({"certData": SimStruct({"size": SimTypeInt(signed=False, label="UInt32"), "data": SimTypePointer(SimTypeChar(label="Byte"), offset=0)}, name="FWP_BYTE_BLOB", pack=False, align=None), "certHash": SimStruct({"size": SimTypeInt(signed=False, label="UInt32"), "data": SimTypePointer(SimTypeChar(label="Byte"), offset=0)}, name="FWP_BYTE_BLOB", pack=False, align=None), "eku": SimTypePointer(SimStruct({"numEku": SimTypeInt(signed=False, label="UInt32"), "eku": SimTypePointer(SimTypePointer(SimTypeChar(label="Byte"), offset=0), offset=0)}, name="IKEEXT_CERT_EKUS0", pack=False, align=None), offset=0), "name": SimTypePointer(SimStruct({"nameType": SimTypeInt(signed=False, label="IKEEXT_CERT_CRITERIA_NAME_TYPE"), "certName": SimTypePointer(SimTypeChar(label="Char"), offset=0)}, name="IKEEXT_CERT_NAME0", pack=False, align=None), offset=0), "flags": SimTypeInt(signed=False, label="UInt32")}, name="IKEEXT_CERTIFICATE_CRITERIA0", pack=False, align=None), offset=0)}, name="_Anonymous3_e__Struct", pack=False, align=None)}, name="<anon>", label="None"), "flags": SimTypeInt(signed=False, label="IKEEXT_CERT_AUTH"), "localCertLocationUrl": SimStruct({"size": SimTypeInt(signed=False, label="UInt32"), "data": SimTypePointer(SimTypeChar(label="Byte"), offset=0)}, name="FWP_BYTE_BLOB", pack=False, align=None)}, name="IKEEXT_CERTIFICATE_AUTHENTICATION2", pack=False, align=None), "cgaAuthentication": SimStruct({"keyContainerName": SimTypePointer(SimTypeChar(label="Char"), offset=0), "cspName": SimTypePointer(SimTypeChar(label="Char"), offset=0), "cspType": SimTypeInt(signed=False, label="UInt32"), "cgaModifier": SimStruct({"byteArray16": SimTypeFixedSizeArray(SimTypeChar(label="Byte"), 16)}, name="FWP_BYTE_ARRAY16", pack=False, align=None), "cgaCollisionCount": SimTypeChar(label="Byte")}, name="IKEEXT_IPV6_CGA_AUTHENTICATION0", pack=False, align=None), "eapAuthentication": SimStruct({"flags": SimTypeInt(signed=False, label="IKEEXT_EAP_AUTHENTICATION_FLAGS")}, name="IKEEXT_EAP_AUTHENTICATION0", pack=False, align=None)}, name="<anon>", label="None")}, name="IKEEXT_AUTHENTICATION_METHOD2", pack=False, align=None), offset=0), "initiatorImpersonationType": SimTypeInt(signed=False, label="IKEEXT_AUTHENTICATION_IMPERSONATION_TYPE")}, name="IKEEXT_EM_POLICY2", pack=False, align=None), offset=0)}, name="IPSEC_TRANSPORT_POLICY2", pack=False, align=None), offset=0), "ikeV2MmPolicy": SimTypePointer(SimStruct({"softExpirationTime": SimTypeInt(signed=False, label="UInt32"), "numAuthenticationMethods": SimTypeInt(signed=False, label="UInt32"), "authenticationMethods": SimTypePointer(SimStruct({"authenticationMethodType": SimTypeInt(signed=False, label="IKEEXT_AUTHENTICATION_METHOD_TYPE"), "Anonymous": SimUnion({"presharedKeyAuthentication": SimStruct({"presharedKey": SimStruct({"size": SimTypeInt(signed=False, label="UInt32"), "data": SimTypePointer(SimTypeChar(label="Byte"), offset=0)}, name="FWP_BYTE_BLOB", pack=False, align=None), "flags": SimTypeInt(signed=False, label="IKEEXT_PRESHARED_KEY_AUTHENTICATION_FLAGS")}, name="IKEEXT_PRESHARED_KEY_AUTHENTICATION1", pack=False, align=None), "certificateAuthentication": SimStruct({"inboundConfigType": SimTypeInt(signed=False, label="IKEEXT_CERT_CONFIG_TYPE"), "Anonymous1": SimUnion({"Anonymous1": SimStruct({"inboundRootArraySize": SimTypeInt(signed=False, label="UInt32"), "inboundRootCriteria": SimTypePointer(SimStruct({"certData": SimStruct({"size": SimTypeInt(signed=False, label="UInt32"), "data": SimTypePointer(SimTypeChar(label="Byte"), offset=0)}, name="FWP_BYTE_BLOB", pack=False, align=None), "certHash": SimStruct({"size": SimTypeInt(signed=False, label="UInt32"), "data": SimTypePointer(SimTypeChar(label="Byte"), offset=0)}, name="FWP_BYTE_BLOB", pack=False, align=None), "eku": SimTypePointer(SimStruct({"numEku": SimTypeInt(signed=False, label="UInt32"), "eku": SimTypePointer(SimTypePointer(SimTypeChar(label="Byte"), offset=0), offset=0)}, name="IKEEXT_CERT_EKUS0", pack=False, align=None), offset=0), "name": SimTypePointer(SimStruct({"nameType": SimTypeInt(signed=False, label="IKEEXT_CERT_CRITERIA_NAME_TYPE"), "certName": SimTypePointer(SimTypeChar(label="Char"), offset=0)}, name="IKEEXT_CERT_NAME0", pack=False, align=None), offset=0), "flags": SimTypeInt(signed=False, label="UInt32")}, name="IKEEXT_CERTIFICATE_CRITERIA0", pack=False, align=None), offset=0)}, name="_Anonymous1_e__Struct", pack=False, align=None), "Anonymous2": SimStruct({"inboundEnterpriseStoreArraySize": SimTypeInt(signed=False, label="UInt32"), "inboundEnterpriseStoreCriteria": SimTypePointer(SimStruct({"certData": SimStruct({"size": SimTypeInt(signed=False, label="UInt32"), "data": SimTypePointer(SimTypeChar(label="Byte"), offset=0)}, name="FWP_BYTE_BLOB", pack=False, align=None), "certHash": SimStruct({"size": SimTypeInt(signed=False, label="UInt32"), "data": SimTypePointer(SimTypeChar(label="Byte"), offset=0)}, name="FWP_BYTE_BLOB", pack=False, align=None), "eku": SimTypePointer(SimStruct({"numEku": SimTypeInt(signed=False, label="UInt32"), "eku": SimTypePointer(SimTypePointer(SimTypeChar(label="Byte"), offset=0), offset=0)}, name="IKEEXT_CERT_EKUS0", pack=False, align=None), offset=0), "name": SimTypePointer(SimStruct({"nameType": SimTypeInt(signed=False, label="IKEEXT_CERT_CRITERIA_NAME_TYPE"), "certName": SimTypePointer(SimTypeChar(label="Char"), offset=0)}, name="IKEEXT_CERT_NAME0", pack=False, align=None), offset=0), "flags": SimTypeInt(signed=False, label="UInt32")}, name="IKEEXT_CERTIFICATE_CRITERIA0", pack=False, align=None), offset=0)}, name="_Anonymous2_e__Struct", pack=False, align=None), "Anonymous3": SimStruct({"inboundRootStoreArraySize": SimTypeInt(signed=False, label="UInt32"), "inboundTrustedRootStoreCriteria": SimTypePointer(SimStruct({"certData": SimStruct({"size": SimTypeInt(signed=False, label="UInt32"), "data": SimTypePointer(SimTypeChar(label="Byte"), offset=0)}, name="FWP_BYTE_BLOB", pack=False, align=None), "certHash": SimStruct({"size": SimTypeInt(signed=False, label="UInt32"), "data": SimTypePointer(SimTypeChar(label="Byte"), offset=0)}, name="FWP_BYTE_BLOB", pack=False, align=None), "eku": SimTypePointer(SimStruct({"numEku": SimTypeInt(signed=False, label="UInt32"), "eku": SimTypePointer(SimTypePointer(SimTypeChar(label="Byte"), offset=0), offset=0)}, name="IKEEXT_CERT_EKUS0", pack=False, align=None), offset=0), "name": SimTypePointer(SimStruct({"nameType": SimTypeInt(signed=False, label="IKEEXT_CERT_CRITERIA_NAME_TYPE"), "certName": SimTypePointer(SimTypeChar(label="Char"), offset=0)}, name="IKEEXT_CERT_NAME0", pack=False, align=None), offset=0), "flags": SimTypeInt(signed=False, label="UInt32")}, name="IKEEXT_CERTIFICATE_CRITERIA0", pack=False, align=None), offset=0)}, name="_Anonymous3_e__Struct", pack=False, align=None)}, name="<anon>", label="None"), "outboundConfigType": SimTypeInt(signed=False, label="IKEEXT_CERT_CONFIG_TYPE"), "Anonymous2": SimUnion({"Anonymous1": SimStruct({"outboundRootArraySize": SimTypeInt(signed=False, label="UInt32"), "outboundRootCriteria": SimTypePointer(SimStruct({"certData": SimStruct({"size": SimTypeInt(signed=False, label="UInt32"), "data": SimTypePointer(SimTypeChar(label="Byte"), offset=0)}, name="FWP_BYTE_BLOB", pack=False, align=None), "certHash": SimStruct({"size": SimTypeInt(signed=False, label="UInt32"), "data": SimTypePointer(SimTypeChar(label="Byte"), offset=0)}, name="FWP_BYTE_BLOB", pack=False, align=None), "eku": SimTypePointer(SimStruct({"numEku": SimTypeInt(signed=False, label="UInt32"), "eku": SimTypePointer(SimTypePointer(SimTypeChar(label="Byte"), offset=0), offset=0)}, name="IKEEXT_CERT_EKUS0", pack=False, align=None), offset=0), "name": SimTypePointer(SimStruct({"nameType": SimTypeInt(signed=False, label="IKEEXT_CERT_CRITERIA_NAME_TYPE"), "certName": SimTypePointer(SimTypeChar(label="Char"), offset=0)}, name="IKEEXT_CERT_NAME0", pack=False, align=None), offset=0), "flags": SimTypeInt(signed=False, label="UInt32")}, name="IKEEXT_CERTIFICATE_CRITERIA0", pack=False, align=None), offset=0)}, name="_Anonymous1_e__Struct", pack=False, align=None), "Anonymous2": SimStruct({"outboundEnterpriseStoreArraySize": SimTypeInt(signed=False, label="UInt32"), "outboundEnterpriseStoreCriteria": SimTypePointer(SimStruct({"certData": SimStruct({"size": SimTypeInt(signed=False, label="UInt32"), "data": SimTypePointer(SimTypeChar(label="Byte"), offset=0)}, name="FWP_BYTE_BLOB", pack=False, align=None), "certHash": SimStruct({"size": SimTypeInt(signed=False, label="UInt32"), "data": SimTypePointer(SimTypeChar(label="Byte"), offset=0)}, name="FWP_BYTE_BLOB", pack=False, align=None), "eku": SimTypePointer(SimStruct({"numEku": SimTypeInt(signed=False, label="UInt32"), "eku": SimTypePointer(SimTypePointer(SimTypeChar(label="Byte"), offset=0), offset=0)}, name="IKEEXT_CERT_EKUS0", pack=False, align=None), offset=0), "name": SimTypePointer(SimStruct({"nameType": SimTypeInt(signed=False, label="IKEEXT_CERT_CRITERIA_NAME_TYPE"), "certName": SimTypePointer(SimTypeChar(label="Char"), offset=0)}, name="IKEEXT_CERT_NAME0", pack=False, align=None), offset=0), "flags": SimTypeInt(signed=False, label="UInt32")}, name="IKEEXT_CERTIFICATE_CRITERIA0", pack=False, align=None), offset=0)}, name="_Anonymous2_e__Struct", pack=False, align=None), "Anonymous3": SimStruct({"outboundRootStoreArraySize": SimTypeInt(signed=False, label="UInt32"), "outboundTrustedRootStoreCriteria": SimTypePointer(SimStruct({"certData": SimStruct({"size": SimTypeInt(signed=False, label="UInt32"), "data": SimTypePointer(SimTypeChar(label="Byte"), offset=0)}, name="FWP_BYTE_BLOB", pack=False, align=None), "certHash": SimStruct({"size": SimTypeInt(signed=False, label="UInt32"), "data": SimTypePointer(SimTypeChar(label="Byte"), offset=0)}, name="FWP_BYTE_BLOB", pack=False, align=None), "eku": SimTypePointer(SimStruct({"numEku": SimTypeInt(signed=False, label="UInt32"), "eku": SimTypePointer(SimTypePointer(SimTypeChar(label="Byte"), offset=0), offset=0)}, name="IKEEXT_CERT_EKUS0", pack=False, align=None), offset=0), "name": SimTypePointer(SimStruct({"nameType": SimTypeInt(signed=False, label="IKEEXT_CERT_CRITERIA_NAME_TYPE"), "certName": SimTypePointer(SimTypeChar(label="Char"), offset=0)}, name="IKEEXT_CERT_NAME0", pack=False, align=None), offset=0), "flags": SimTypeInt(signed=False, label="UInt32")}, name="IKEEXT_CERTIFICATE_CRITERIA0", pack=False, align=None), offset=0)}, name="_Anonymous3_e__Struct", pack=False, align=None)}, name="<anon>", label="None"), "flags": SimTypeInt(signed=False, label="IKEEXT_CERT_AUTH"), "localCertLocationUrl": SimStruct({"size": SimTypeInt(signed=False, label="UInt32"), "data": SimTypePointer(SimTypeChar(label="Byte"), offset=0)}, name="FWP_BYTE_BLOB", pack=False, align=None)}, name="IKEEXT_CERTIFICATE_AUTHENTICATION2", pack=False, align=None), "kerberosAuthentication": SimStruct({"flags": SimTypeInt(signed=False, label="IKEEXT_KERBEROS_AUTHENTICATION_FLAGS"), "proxyServer": SimTypePointer(SimTypeChar(label="Char"), offset=0)}, name="IKEEXT_KERBEROS_AUTHENTICATION1", pack=False, align=None), "reservedAuthentication": SimStruct({"flags": SimTypeInt(signed=False, label="IKEEXT_RESERVED_AUTHENTICATION_FLAGS")}, name="IKEEXT_RESERVED_AUTHENTICATION0", pack=False, align=None), "ntlmV2Authentication": SimStruct({"flags": SimTypeInt(signed=False, label="UInt32")}, name="IKEEXT_NTLM_V2_AUTHENTICATION0", pack=False, align=None), "sslAuthentication": SimStruct({"inboundConfigType": SimTypeInt(signed=False, label="IKEEXT_CERT_CONFIG_TYPE"), "Anonymous1": SimUnion({"Anonymous1": SimStruct({"inboundRootArraySize": SimTypeInt(signed=False, label="UInt32"), "inboundRootCriteria": SimTypePointer(SimStruct({"certData": SimStruct({"size": SimTypeInt(signed=False, label="UInt32"), "data": SimTypePointer(SimTypeChar(label="Byte"), offset=0)}, name="FWP_BYTE_BLOB", pack=False, align=None), "certHash": SimStruct({"size": SimTypeInt(signed=False, label="UInt32"), "data": SimTypePointer(SimTypeChar(label="Byte"), offset=0)}, name="FWP_BYTE_BLOB", pack=False, align=None), "eku": SimTypePointer(SimStruct({"numEku": SimTypeInt(signed=False, label="UInt32"), "eku": SimTypePointer(SimTypePointer(SimTypeChar(label="Byte"), offset=0), offset=0)}, name="IKEEXT_CERT_EKUS0", pack=False, align=None), offset=0), "name": SimTypePointer(SimStruct({"nameType": SimTypeInt(signed=False, label="IKEEXT_CERT_CRITERIA_NAME_TYPE"), "certName": SimTypePointer(SimTypeChar(label="Char"), offset=0)}, name="IKEEXT_CERT_NAME0", pack=False, align=None), offset=0), "flags": SimTypeInt(signed=False, label="UInt32")}, name="IKEEXT_CERTIFICATE_CRITERIA0", pack=False, align=None), offset=0)}, name="_Anonymous1_e__Struct", pack=False, align=None), "Anonymous2": SimStruct({"inboundEnterpriseStoreArraySize": SimTypeInt(signed=False, label="UInt32"), "inboundEnterpriseStoreCriteria": SimTypePointer(SimStruct({"certData": SimStruct({"size": SimTypeInt(signed=False, label="UInt32"), "data": SimTypePointer(SimTypeChar(label="Byte"), offset=0)}, name="FWP_BYTE_BLOB", pack=False, align=None), "certHash": SimStruct({"size": SimTypeInt(signed=False, label="UInt32"), "data": SimTypePointer(SimTypeChar(label="Byte"), offset=0)}, name="FWP_BYTE_BLOB", pack=False, align=None), "eku": SimTypePointer(SimStruct({"numEku": SimTypeInt(signed=False, label="UInt32"), "eku": SimTypePointer(SimTypePointer(SimTypeChar(label="Byte"), offset=0), offset=0)}, name="IKEEXT_CERT_EKUS0", pack=False, align=None), offset=0), "name": SimTypePointer(SimStruct({"nameType": SimTypeInt(signed=False, label="IKEEXT_CERT_CRITERIA_NAME_TYPE"), "certName": SimTypePointer(SimTypeChar(label="Char"), offset=0)}, name="IKEEXT_CERT_NAME0", pack=False, align=None), offset=0), "flags": SimTypeInt(signed=False, label="UInt32")}, name="IKEEXT_CERTIFICATE_CRITERIA0", pack=False, align=None), offset=0)}, name="_Anonymous2_e__Struct", pack=False, align=None), "Anonymous3": SimStruct({"inboundRootStoreArraySize": SimTypeInt(signed=False, label="UInt32"), "inboundTrustedRootStoreCriteria": SimTypePointer(SimStruct({"certData": SimStruct({"size": SimTypeInt(signed=False, label="UInt32"), "data": SimTypePointer(SimTypeChar(label="Byte"), offset=0)}, name="FWP_BYTE_BLOB", pack=False, align=None), "certHash": SimStruct({"size": SimTypeInt(signed=False, label="UInt32"), "data": SimTypePointer(SimTypeChar(label="Byte"), offset=0)}, name="FWP_BYTE_BLOB", pack=False, align=None), "eku": SimTypePointer(SimStruct({"numEku": SimTypeInt(signed=False, label="UInt32"), "eku": SimTypePointer(SimTypePointer(SimTypeChar(label="Byte"), offset=0), offset=0)}, name="IKEEXT_CERT_EKUS0", pack=False, align=None), offset=0), "name": SimTypePointer(SimStruct({"nameType": SimTypeInt(signed=False, label="IKEEXT_CERT_CRITERIA_NAME_TYPE"), "certName": SimTypePointer(SimTypeChar(label="Char"), offset=0)}, name="IKEEXT_CERT_NAME0", pack=False, align=None), offset=0), "flags": SimTypeInt(signed=False, label="UInt32")}, name="IKEEXT_CERTIFICATE_CRITERIA0", pack=False, align=None), offset=0)}, name="_Anonymous3_e__Struct", pack=False, align=None)}, name="<anon>", label="None"), "outboundConfigType": SimTypeInt(signed=False, label="IKEEXT_CERT_CONFIG_TYPE"), "Anonymous2": SimUnion({"Anonymous1": SimStruct({"outboundRootArraySize": SimTypeInt(signed=False, label="UInt32"), "outboundRootCriteria": SimTypePointer(SimStruct({"certData": SimStruct({"size": SimTypeInt(signed=False, label="UInt32"), "data": SimTypePointer(SimTypeChar(label="Byte"), offset=0)}, name="FWP_BYTE_BLOB", pack=False, align=None), "certHash": SimStruct({"size": SimTypeInt(signed=False, label="UInt32"), "data": SimTypePointer(SimTypeChar(label="Byte"), offset=0)}, name="FWP_BYTE_BLOB", pack=False, align=None), "eku": SimTypePointer(SimStruct({"numEku": SimTypeInt(signed=False, label="UInt32"), "eku": SimTypePointer(SimTypePointer(SimTypeChar(label="Byte"), offset=0), offset=0)}, name="IKEEXT_CERT_EKUS0", pack=False, align=None), offset=0), "name": SimTypePointer(SimStruct({"nameType": SimTypeInt(signed=False, label="IKEEXT_CERT_CRITERIA_NAME_TYPE"), "certName": SimTypePointer(SimTypeChar(label="Char"), offset=0)}, name="IKEEXT_CERT_NAME0", pack=False, align=None), offset=0), "flags": SimTypeInt(signed=False, label="UInt32")}, name="IKEEXT_CERTIFICATE_CRITERIA0", pack=False, align=None), offset=0)}, name="_Anonymous1_e__Struct", pack=False, align=None), "Anonymous2": SimStruct({"outboundEnterpriseStoreArraySize": SimTypeInt(signed=False, label="UInt32"), "outboundEnterpriseStoreCriteria": SimTypePointer(SimStruct({"certData": SimStruct({"size": SimTypeInt(signed=False, label="UInt32"), "data": SimTypePointer(SimTypeChar(label="Byte"), offset=0)}, name="FWP_BYTE_BLOB", pack=False, align=None), "certHash": SimStruct({"size": SimTypeInt(signed=False, label="UInt32"), "data": SimTypePointer(SimTypeChar(label="Byte"), offset=0)}, name="FWP_BYTE_BLOB", pack=False, align=None), "eku": SimTypePointer(SimStruct({"numEku": SimTypeInt(signed=False, label="UInt32"), "eku": SimTypePointer(SimTypePointer(SimTypeChar(label="Byte"), offset=0), offset=0)}, name="IKEEXT_CERT_EKUS0", pack=False, align=None), offset=0), "name": SimTypePointer(SimStruct({"nameType": SimTypeInt(signed=False, label="IKEEXT_CERT_CRITERIA_NAME_TYPE"), "certName": SimTypePointer(SimTypeChar(label="Char"), offset=0)}, name="IKEEXT_CERT_NAME0", pack=False, align=None), offset=0), "flags": SimTypeInt(signed=False, label="UInt32")}, name="IKEEXT_CERTIFICATE_CRITERIA0", pack=False, align=None), offset=0)}, name="_Anonymous2_e__Struct", pack=False, align=None), "Anonymous3": SimStruct({"outboundRootStoreArraySize": SimTypeInt(signed=False, label="UInt32"), "outboundTrustedRootStoreCriteria": SimTypePointer(SimStruct({"certData": SimStruct({"size": SimTypeInt(signed=False, label="UInt32"), "data": SimTypePointer(SimTypeChar(label="Byte"), offset=0)}, name="FWP_BYTE_BLOB", pack=False, align=None), "certHash": SimStruct({"size": SimTypeInt(signed=False, label="UInt32"), "data": SimTypePointer(SimTypeChar(label="Byte"), offset=0)}, name="FWP_BYTE_BLOB", pack=False, align=None), "eku": SimTypePointer(SimStruct({"numEku": SimTypeInt(signed=False, label="UInt32"), "eku": SimTypePointer(SimTypePointer(SimTypeChar(label="Byte"), offset=0), offset=0)}, name="IKEEXT_CERT_EKUS0", pack=False, align=None), offset=0), "name": SimTypePointer(SimStruct({"nameType": SimTypeInt(signed=False, label="IKEEXT_CERT_CRITERIA_NAME_TYPE"), "certName": SimTypePointer(SimTypeChar(label="Char"), offset=0)}, name="IKEEXT_CERT_NAME0", pack=False, align=None), offset=0), "flags": SimTypeInt(signed=False, label="UInt32")}, name="IKEEXT_CERTIFICATE_CRITERIA0", pack=False, align=None), offset=0)}, name="_Anonymous3_e__Struct", pack=False, align=None)}, name="<anon>", label="None"), "flags": SimTypeInt(signed=False, label="IKEEXT_CERT_AUTH"), "localCertLocationUrl": SimStruct({"size": SimTypeInt(signed=False, label="UInt32"), "data": SimTypePointer(SimTypeChar(label="Byte"), offset=0)}, name="FWP_BYTE_BLOB", pack=False, align=None)}, name="IKEEXT_CERTIFICATE_AUTHENTICATION2", pack=False, align=None), "cgaAuthentication": SimStruct({"keyContainerName": SimTypePointer(SimTypeChar(label="Char"), offset=0), "cspName": SimTypePointer(SimTypeChar(label="Char"), offset=0), "cspType": SimTypeInt(signed=False, label="UInt32"), "cgaModifier": SimStruct({"byteArray16": SimTypeFixedSizeArray(SimTypeChar(label="Byte"), 16)}, name="FWP_BYTE_ARRAY16", pack=False, align=None), "cgaCollisionCount": SimTypeChar(label="Byte")}, name="IKEEXT_IPV6_CGA_AUTHENTICATION0", pack=False, align=None), "eapAuthentication": SimStruct({"flags": SimTypeInt(signed=False, label="IKEEXT_EAP_AUTHENTICATION_FLAGS")}, name="IKEEXT_EAP_AUTHENTICATION0", pack=False, align=None)}, name="<anon>", label="None")}, name="IKEEXT_AUTHENTICATION_METHOD2", pack=False, align=None), offset=0), "initiatorImpersonationType": SimTypeInt(signed=False, label="IKEEXT_AUTHENTICATION_IMPERSONATION_TYPE"), "numIkeProposals": SimTypeInt(signed=False, label="UInt32"), "ikeProposals": SimTypePointer(SimStruct({"cipherAlgorithm": SimStruct({"algoIdentifier": SimTypeInt(signed=False, label="IKEEXT_CIPHER_TYPE"), "keyLen": SimTypeInt(signed=False, label="UInt32"), "rounds": SimTypeInt(signed=False, label="UInt32")}, name="IKEEXT_CIPHER_ALGORITHM0", pack=False, align=None), "integrityAlgorithm": SimStruct({"algoIdentifier": SimTypeInt(signed=False, label="IKEEXT_INTEGRITY_TYPE")}, name="IKEEXT_INTEGRITY_ALGORITHM0", pack=False, align=None), "maxLifetimeSeconds": SimTypeInt(signed=False, label="UInt32"), "dhGroup": SimTypeInt(signed=False, label="IKEEXT_DH_GROUP"), "quickModeLimit": SimTypeInt(signed=False, label="UInt32")}, name="IKEEXT_PROPOSAL0", pack=False, align=None), offset=0), "flags": SimTypeInt(signed=False, label="IKEEXT_POLICY_FLAG"), "maxDynamicFilters": SimTypeInt(signed=False, label="UInt32"), "retransmitDurationSecs": SimTypeInt(signed=False, label="UInt32")}, name="IKEEXT_POLICY2", pack=False, align=None), offset=0), "idpOptions": SimTypePointer(SimStruct({"stateIdleTimeoutSeconds": SimTypeInt(signed=False, label="UInt32"), "perIPRateLimitQueueIdleTimeoutSeconds": SimTypeInt(signed=False, label="UInt32"), "ipV6IPsecUnauthDscp": SimTypeChar(label="Byte"), "ipV6IPsecUnauthRateLimitBytesPerSec": SimTypeInt(signed=False, label="UInt32"), "ipV6IPsecUnauthPerIPRateLimitBytesPerSec": SimTypeInt(signed=False, label="UInt32"), "ipV6IPsecAuthDscp": SimTypeChar(label="Byte"), "ipV6IPsecAuthRateLimitBytesPerSec": SimTypeInt(signed=False, label="UInt32"), "icmpV6Dscp": SimTypeChar(label="Byte"), "icmpV6RateLimitBytesPerSec": SimTypeInt(signed=False, label="UInt32"), "ipV6FilterExemptDscp": SimTypeChar(label="Byte"), "ipV6FilterExemptRateLimitBytesPerSec": SimTypeInt(signed=False, label="UInt32"), "defBlockExemptDscp": SimTypeChar(label="Byte"), "defBlockExemptRateLimitBytesPerSec": SimTypeInt(signed=False, label="UInt32"), "maxStateEntries": SimTypeInt(signed=False, label="UInt32"), "maxPerIPRateLimitQueues": SimTypeInt(signed=False, label="UInt32"), "flags": SimTypeInt(signed=False, label="IPSEC_DOSP_FLAGS"), "numPublicIFLuids": SimTypeInt(signed=False, label="UInt32"), "publicIFLuids": SimTypePointer(SimTypeLongLong(signed=False, label="UInt64"), offset=0), "numInternalIFLuids": SimTypeInt(signed=False, label="UInt32"), "internalIFLuids": SimTypePointer(SimTypeLongLong(signed=False, label="UInt64"), offset=0), "publicV6AddrMask": SimStruct({"addr": SimTypeFixedSizeArray(SimTypeChar(label="Byte"), 16), "prefixLength": SimTypeChar(label="Byte")}, name="FWP_V6_ADDR_AND_MASK", pack=False, align=None), "internalV6AddrMask": SimStruct({"addr": SimTypeFixedSizeArray(SimTypeChar(label="Byte"), 16), "prefixLength": SimTypeChar(label="Byte")}, name="FWP_V6_ADDR_AND_MASK", pack=False, align=None)}, name="IPSEC_DOSP_OPTIONS0", pack=False, align=None), offset=0)}, name="<anon>", label="None"), "providerContextId": SimTypeLongLong(signed=False, label="UInt64")}, name="FWPM_PROVIDER_CONTEXT3_", pack=False, align=None), offset=0), SimTypePointer(SimStruct({"Revision": SimTypeChar(label="Byte"), "Sbz1": SimTypeChar(label="Byte"), "Control": SimTypeShort(signed=False, label="UInt16"), "Owner": SimTypePointer(SimTypeInt(signed=True, label="Int"), label="IntPtr", offset=0), "Group": SimTypePointer(SimTypeInt(signed=True, label="Int"), label="IntPtr", offset=0), "Sacl": SimTypePointer(SimStruct({"AclRevision": SimTypeChar(label="Byte"), "Sbz1": SimTypeChar(label="Byte"), "AclSize": SimTypeShort(signed=False, label="UInt16"), "AceCount": SimTypeShort(signed=False, label="UInt16"), "Sbz2": SimTypeShort(signed=False, label="UInt16")}, name="ACL", pack=False, align=None), offset=0), "Dacl": SimTypePointer(SimStruct({"AclRevision": SimTypeChar(label="Byte"), "Sbz1": SimTypeChar(label="Byte"), "AclSize": SimTypeShort(signed=False, label="UInt16"), "AceCount": SimTypeShort(signed=False, label="UInt16"), "Sbz2": SimTypeShort(signed=False, label="UInt16")}, name="ACL", pack=False, align=None), offset=0)}, name="SECURITY_DESCRIPTOR", pack=False, align=None), offset=0), SimTypePointer(SimTypeLongLong(signed=False, label="UInt64"), offset=0)], SimTypeInt(signed=False, label="UInt32"), arg_names=["engineHandle", "providerContext", "sd", "id"]),
        # 
        'FwpmProviderContextDeleteById0': SimTypeFunction([SimTypePointer(SimTypeInt(signed=True, label="Int"), label="IntPtr", offset=0), SimTypeLongLong(signed=False, label="UInt64")], SimTypeInt(signed=False, label="UInt32"), arg_names=["engineHandle", "id"]),
        # 
        'FwpmProviderContextDeleteByKey0': SimTypeFunction([SimTypePointer(SimTypeInt(signed=True, label="Int"), label="IntPtr", offset=0), SimTypePointer(SimTypeBottom(label="Guid"), offset=0)], SimTypeInt(signed=False, label="UInt32"), arg_names=["engineHandle", "key"]),
        # 
        'FwpmProviderContextGetById0': SimTypeFunction([SimTypePointer(SimTypeInt(signed=True, label="Int"), label="IntPtr", offset=0), SimTypeLongLong(signed=False, label="UInt64"), SimTypePointer(SimTypePointer(SimStruct({"providerContextKey": SimTypeBottom(label="Guid"), "displayData": SimStruct({"name": SimTypePointer(SimTypeChar(label="Char"), offset=0), "description": SimTypePointer(SimTypeChar(label="Char"), offset=0)}, name="FWPM_DISPLAY_DATA0", pack=False, align=None), "flags": SimTypeInt(signed=False, label="UInt32"), "providerKey": SimTypePointer(SimTypeBottom(label="Guid"), offset=0), "providerData": SimStruct({"size": SimTypeInt(signed=False, label="UInt32"), "data": SimTypePointer(SimTypeChar(label="Byte"), offset=0)}, name="FWP_BYTE_BLOB", pack=False, align=None), "type": SimTypeInt(signed=False, label="FWPM_PROVIDER_CONTEXT_TYPE"), "Anonymous": SimUnion({"keyingPolicy": SimTypePointer(SimStruct({"numKeyMods": SimTypeInt(signed=False, label="UInt32"), "keyModKeys": SimTypePointer(SimTypeBottom(label="Guid"), offset=0)}, name="IPSEC_KEYING_POLICY0", pack=False, align=None), offset=0), "ikeQmTransportPolicy": SimTypePointer(SimStruct({"numIpsecProposals": SimTypeInt(signed=False, label="UInt32"), "ipsecProposals": SimTypePointer(SimStruct({"lifetime": SimStruct({"lifetimeSeconds": SimTypeInt(signed=False, label="UInt32"), "lifetimeKilobytes": SimTypeInt(signed=False, label="UInt32"), "lifetimePackets": SimTypeInt(signed=False, label="UInt32")}, name="IPSEC_SA_LIFETIME0", pack=False, align=None), "numSaTransforms": SimTypeInt(signed=False, label="UInt32"), "saTransforms": SimTypePointer(SimStruct({"ipsecTransformType": SimTypeInt(signed=False, label="IPSEC_TRANSFORM_TYPE"), "Anonymous": SimUnion({"ahTransform": SimTypePointer(SimStruct({"authTransformId": SimStruct({"authType": SimTypeInt(signed=False, label="IPSEC_AUTH_TYPE"), "authConfig": SimTypeChar(label="Byte")}, name="IPSEC_AUTH_TRANSFORM_ID0", pack=False, align=None), "cryptoModuleId": SimTypePointer(SimTypeBottom(label="Guid"), offset=0)}, name="IPSEC_AUTH_TRANSFORM0", pack=False, align=None), offset=0), "espAuthTransform": SimTypePointer(SimStruct({"authTransformId": SimStruct({"authType": SimTypeInt(signed=False, label="IPSEC_AUTH_TYPE"), "authConfig": SimTypeChar(label="Byte")}, name="IPSEC_AUTH_TRANSFORM_ID0", pack=False, align=None), "cryptoModuleId": SimTypePointer(SimTypeBottom(label="Guid"), offset=0)}, name="IPSEC_AUTH_TRANSFORM0", pack=False, align=None), offset=0), "espCipherTransform": SimTypePointer(SimStruct({"cipherTransformId": SimStruct({"cipherType": SimTypeInt(signed=False, label="IPSEC_CIPHER_TYPE"), "cipherConfig": SimTypeChar(label="Byte")}, name="IPSEC_CIPHER_TRANSFORM_ID0", pack=False, align=None), "cryptoModuleId": SimTypePointer(SimTypeBottom(label="Guid"), offset=0)}, name="IPSEC_CIPHER_TRANSFORM0", pack=False, align=None), offset=0), "espAuthAndCipherTransform": SimTypePointer(SimStruct({"authTransform": SimStruct({"authTransformId": SimStruct({"authType": SimTypeInt(signed=False, label="IPSEC_AUTH_TYPE"), "authConfig": SimTypeChar(label="Byte")}, name="IPSEC_AUTH_TRANSFORM_ID0", pack=False, align=None), "cryptoModuleId": SimTypePointer(SimTypeBottom(label="Guid"), offset=0)}, name="IPSEC_AUTH_TRANSFORM0", pack=False, align=None), "cipherTransform": SimStruct({"cipherTransformId": SimStruct({"cipherType": SimTypeInt(signed=False, label="IPSEC_CIPHER_TYPE"), "cipherConfig": SimTypeChar(label="Byte")}, name="IPSEC_CIPHER_TRANSFORM_ID0", pack=False, align=None), "cryptoModuleId": SimTypePointer(SimTypeBottom(label="Guid"), offset=0)}, name="IPSEC_CIPHER_TRANSFORM0", pack=False, align=None)}, name="IPSEC_AUTH_AND_CIPHER_TRANSFORM0", pack=False, align=None), offset=0), "espAuthFwTransform": SimTypePointer(SimStruct({"authTransformId": SimStruct({"authType": SimTypeInt(signed=False, label="IPSEC_AUTH_TYPE"), "authConfig": SimTypeChar(label="Byte")}, name="IPSEC_AUTH_TRANSFORM_ID0", pack=False, align=None), "cryptoModuleId": SimTypePointer(SimTypeBottom(label="Guid"), offset=0)}, name="IPSEC_AUTH_TRANSFORM0", pack=False, align=None), offset=0)}, name="<anon>", label="None")}, name="IPSEC_SA_TRANSFORM0", pack=False, align=None), offset=0), "pfsGroup": SimTypeInt(signed=False, label="IPSEC_PFS_GROUP")}, name="IPSEC_PROPOSAL0", pack=False, align=None), offset=0), "flags": SimTypeInt(signed=False, label="IPSEC_POLICY_FLAG"), "ndAllowClearTimeoutSeconds": SimTypeInt(signed=False, label="UInt32"), "saIdleTimeout": SimStruct({"idleTimeoutSeconds": SimTypeInt(signed=False, label="UInt32"), "idleTimeoutSecondsFailOver": SimTypeInt(signed=False, label="UInt32")}, name="IPSEC_SA_IDLE_TIMEOUT0", pack=False, align=None), "emPolicy": SimTypePointer(SimStruct({"numAuthenticationMethods": SimTypeInt(signed=False, label="UInt32"), "authenticationMethods": SimTypePointer(SimStruct({"authenticationMethodType": SimTypeInt(signed=False, label="IKEEXT_AUTHENTICATION_METHOD_TYPE"), "Anonymous": SimUnion({"presharedKeyAuthentication": SimStruct({"presharedKey": SimStruct({"size": SimTypeInt(signed=False, label="UInt32"), "data": SimTypePointer(SimTypeChar(label="Byte"), offset=0)}, name="FWP_BYTE_BLOB", pack=False, align=None)}, name="IKEEXT_PRESHARED_KEY_AUTHENTICATION0", pack=False, align=None), "certificateAuthentication": SimStruct({"inboundConfigType": SimTypeInt(signed=False, label="IKEEXT_CERT_CONFIG_TYPE"), "Anonymous1": SimUnion({"Anonymous": SimStruct({"inboundRootArraySize": SimTypeInt(signed=False, label="UInt32"), "inboundRootArray": SimTypePointer(SimStruct({"certData": SimStruct({"size": SimTypeInt(signed=False, label="UInt32"), "data": SimTypePointer(SimTypeChar(label="Byte"), offset=0)}, name="FWP_BYTE_BLOB", pack=False, align=None), "flags": SimTypeInt(signed=False, label="IKEEXT_CERT_FLAGS")}, name="IKEEXT_CERT_ROOT_CONFIG0", pack=False, align=None), offset=0)}, name="_Anonymous_e__Struct", pack=False, align=None), "inboundEnterpriseStoreConfig": SimTypePointer(SimStruct({"certData": SimStruct({"size": SimTypeInt(signed=False, label="UInt32"), "data": SimTypePointer(SimTypeChar(label="Byte"), offset=0)}, name="FWP_BYTE_BLOB", pack=False, align=None), "flags": SimTypeInt(signed=False, label="IKEEXT_CERT_FLAGS")}, name="IKEEXT_CERT_ROOT_CONFIG0", pack=False, align=None), offset=0), "inboundTrustedRootStoreConfig": SimTypePointer(SimStruct({"certData": SimStruct({"size": SimTypeInt(signed=False, label="UInt32"), "data": SimTypePointer(SimTypeChar(label="Byte"), offset=0)}, name="FWP_BYTE_BLOB", pack=False, align=None), "flags": SimTypeInt(signed=False, label="IKEEXT_CERT_FLAGS")}, name="IKEEXT_CERT_ROOT_CONFIG0", pack=False, align=None), offset=0)}, name="<anon>", label="None"), "outboundConfigType": SimTypeInt(signed=False, label="IKEEXT_CERT_CONFIG_TYPE"), "Anonymous2": SimUnion({"Anonymous": SimStruct({"outboundRootArraySize": SimTypeInt(signed=False, label="UInt32"), "outboundRootArray": SimTypePointer(SimStruct({"certData": SimStruct({"size": SimTypeInt(signed=False, label="UInt32"), "data": SimTypePointer(SimTypeChar(label="Byte"), offset=0)}, name="FWP_BYTE_BLOB", pack=False, align=None), "flags": SimTypeInt(signed=False, label="IKEEXT_CERT_FLAGS")}, name="IKEEXT_CERT_ROOT_CONFIG0", pack=False, align=None), offset=0)}, name="_Anonymous_e__Struct", pack=False, align=None), "outboundEnterpriseStoreConfig": SimTypePointer(SimStruct({"certData": SimStruct({"size": SimTypeInt(signed=False, label="UInt32"), "data": SimTypePointer(SimTypeChar(label="Byte"), offset=0)}, name="FWP_BYTE_BLOB", pack=False, align=None), "flags": SimTypeInt(signed=False, label="IKEEXT_CERT_FLAGS")}, name="IKEEXT_CERT_ROOT_CONFIG0", pack=False, align=None), offset=0), "outboundTrustedRootStoreConfig": SimTypePointer(SimStruct({"certData": SimStruct({"size": SimTypeInt(signed=False, label="UInt32"), "data": SimTypePointer(SimTypeChar(label="Byte"), offset=0)}, name="FWP_BYTE_BLOB", pack=False, align=None), "flags": SimTypeInt(signed=False, label="IKEEXT_CERT_FLAGS")}, name="IKEEXT_CERT_ROOT_CONFIG0", pack=False, align=None), offset=0)}, name="<anon>", label="None"), "flags": SimTypeInt(signed=False, label="IKEEXT_CERT_AUTH")}, name="IKEEXT_CERTIFICATE_AUTHENTICATION0", pack=False, align=None), "kerberosAuthentication": SimStruct({"flags": SimTypeInt(signed=False, label="IKEEXT_KERBEROS_AUTHENTICATION_FLAGS")}, name="IKEEXT_KERBEROS_AUTHENTICATION0", pack=False, align=None), "ntlmV2Authentication": SimStruct({"flags": SimTypeInt(signed=False, label="UInt32")}, name="IKEEXT_NTLM_V2_AUTHENTICATION0", pack=False, align=None), "sslAuthentication": SimStruct({"inboundConfigType": SimTypeInt(signed=False, label="IKEEXT_CERT_CONFIG_TYPE"), "Anonymous1": SimUnion({"Anonymous": SimStruct({"inboundRootArraySize": SimTypeInt(signed=False, label="UInt32"), "inboundRootArray": SimTypePointer(SimStruct({"certData": SimStruct({"size": SimTypeInt(signed=False, label="UInt32"), "data": SimTypePointer(SimTypeChar(label="Byte"), offset=0)}, name="FWP_BYTE_BLOB", pack=False, align=None), "flags": SimTypeInt(signed=False, label="IKEEXT_CERT_FLAGS")}, name="IKEEXT_CERT_ROOT_CONFIG0", pack=False, align=None), offset=0)}, name="_Anonymous_e__Struct", pack=False, align=None), "inboundEnterpriseStoreConfig": SimTypePointer(SimStruct({"certData": SimStruct({"size": SimTypeInt(signed=False, label="UInt32"), "data": SimTypePointer(SimTypeChar(label="Byte"), offset=0)}, name="FWP_BYTE_BLOB", pack=False, align=None), "flags": SimTypeInt(signed=False, label="IKEEXT_CERT_FLAGS")}, name="IKEEXT_CERT_ROOT_CONFIG0", pack=False, align=None), offset=0), "inboundTrustedRootStoreConfig": SimTypePointer(SimStruct({"certData": SimStruct({"size": SimTypeInt(signed=False, label="UInt32"), "data": SimTypePointer(SimTypeChar(label="Byte"), offset=0)}, name="FWP_BYTE_BLOB", pack=False, align=None), "flags": SimTypeInt(signed=False, label="IKEEXT_CERT_FLAGS")}, name="IKEEXT_CERT_ROOT_CONFIG0", pack=False, align=None), offset=0)}, name="<anon>", label="None"), "outboundConfigType": SimTypeInt(signed=False, label="IKEEXT_CERT_CONFIG_TYPE"), "Anonymous2": SimUnion({"Anonymous": SimStruct({"outboundRootArraySize": SimTypeInt(signed=False, label="UInt32"), "outboundRootArray": SimTypePointer(SimStruct({"certData": SimStruct({"size": SimTypeInt(signed=False, label="UInt32"), "data": SimTypePointer(SimTypeChar(label="Byte"), offset=0)}, name="FWP_BYTE_BLOB", pack=False, align=None), "flags": SimTypeInt(signed=False, label="IKEEXT_CERT_FLAGS")}, name="IKEEXT_CERT_ROOT_CONFIG0", pack=False, align=None), offset=0)}, name="_Anonymous_e__Struct", pack=False, align=None), "outboundEnterpriseStoreConfig": SimTypePointer(SimStruct({"certData": SimStruct({"size": SimTypeInt(signed=False, label="UInt32"), "data": SimTypePointer(SimTypeChar(label="Byte"), offset=0)}, name="FWP_BYTE_BLOB", pack=False, align=None), "flags": SimTypeInt(signed=False, label="IKEEXT_CERT_FLAGS")}, name="IKEEXT_CERT_ROOT_CONFIG0", pack=False, align=None), offset=0), "outboundTrustedRootStoreConfig": SimTypePointer(SimStruct({"certData": SimStruct({"size": SimTypeInt(signed=False, label="UInt32"), "data": SimTypePointer(SimTypeChar(label="Byte"), offset=0)}, name="FWP_BYTE_BLOB", pack=False, align=None), "flags": SimTypeInt(signed=False, label="IKEEXT_CERT_FLAGS")}, name="IKEEXT_CERT_ROOT_CONFIG0", pack=False, align=None), offset=0)}, name="<anon>", label="None"), "flags": SimTypeInt(signed=False, label="IKEEXT_CERT_AUTH")}, name="IKEEXT_CERTIFICATE_AUTHENTICATION0", pack=False, align=None), "cgaAuthentication": SimStruct({"keyContainerName": SimTypePointer(SimTypeChar(label="Char"), offset=0), "cspName": SimTypePointer(SimTypeChar(label="Char"), offset=0), "cspType": SimTypeInt(signed=False, label="UInt32"), "cgaModifier": SimStruct({"byteArray16": SimTypeFixedSizeArray(SimTypeChar(label="Byte"), 16)}, name="FWP_BYTE_ARRAY16", pack=False, align=None), "cgaCollisionCount": SimTypeChar(label="Byte")}, name="IKEEXT_IPV6_CGA_AUTHENTICATION0", pack=False, align=None)}, name="<anon>", label="None")}, name="IKEEXT_AUTHENTICATION_METHOD0", pack=False, align=None), offset=0), "initiatorImpersonationType": SimTypeInt(signed=False, label="IKEEXT_AUTHENTICATION_IMPERSONATION_TYPE")}, name="IKEEXT_EM_POLICY0", pack=False, align=None), offset=0)}, name="IPSEC_TRANSPORT_POLICY0", pack=False, align=None), offset=0), "ikeQmTunnelPolicy": SimTypePointer(SimStruct({"flags": SimTypeInt(signed=False, label="IPSEC_POLICY_FLAG"), "numIpsecProposals": SimTypeInt(signed=False, label="UInt32"), "ipsecProposals": SimTypePointer(SimStruct({"lifetime": SimStruct({"lifetimeSeconds": SimTypeInt(signed=False, label="UInt32"), "lifetimeKilobytes": SimTypeInt(signed=False, label="UInt32"), "lifetimePackets": SimTypeInt(signed=False, label="UInt32")}, name="IPSEC_SA_LIFETIME0", pack=False, align=None), "numSaTransforms": SimTypeInt(signed=False, label="UInt32"), "saTransforms": SimTypePointer(SimStruct({"ipsecTransformType": SimTypeInt(signed=False, label="IPSEC_TRANSFORM_TYPE"), "Anonymous": SimUnion({"ahTransform": SimTypePointer(SimStruct({"authTransformId": SimStruct({"authType": SimTypeInt(signed=False, label="IPSEC_AUTH_TYPE"), "authConfig": SimTypeChar(label="Byte")}, name="IPSEC_AUTH_TRANSFORM_ID0", pack=False, align=None), "cryptoModuleId": SimTypePointer(SimTypeBottom(label="Guid"), offset=0)}, name="IPSEC_AUTH_TRANSFORM0", pack=False, align=None), offset=0), "espAuthTransform": SimTypePointer(SimStruct({"authTransformId": SimStruct({"authType": SimTypeInt(signed=False, label="IPSEC_AUTH_TYPE"), "authConfig": SimTypeChar(label="Byte")}, name="IPSEC_AUTH_TRANSFORM_ID0", pack=False, align=None), "cryptoModuleId": SimTypePointer(SimTypeBottom(label="Guid"), offset=0)}, name="IPSEC_AUTH_TRANSFORM0", pack=False, align=None), offset=0), "espCipherTransform": SimTypePointer(SimStruct({"cipherTransformId": SimStruct({"cipherType": SimTypeInt(signed=False, label="IPSEC_CIPHER_TYPE"), "cipherConfig": SimTypeChar(label="Byte")}, name="IPSEC_CIPHER_TRANSFORM_ID0", pack=False, align=None), "cryptoModuleId": SimTypePointer(SimTypeBottom(label="Guid"), offset=0)}, name="IPSEC_CIPHER_TRANSFORM0", pack=False, align=None), offset=0), "espAuthAndCipherTransform": SimTypePointer(SimStruct({"authTransform": SimStruct({"authTransformId": SimStruct({"authType": SimTypeInt(signed=False, label="IPSEC_AUTH_TYPE"), "authConfig": SimTypeChar(label="Byte")}, name="IPSEC_AUTH_TRANSFORM_ID0", pack=False, align=None), "cryptoModuleId": SimTypePointer(SimTypeBottom(label="Guid"), offset=0)}, name="IPSEC_AUTH_TRANSFORM0", pack=False, align=None), "cipherTransform": SimStruct({"cipherTransformId": SimStruct({"cipherType": SimTypeInt(signed=False, label="IPSEC_CIPHER_TYPE"), "cipherConfig": SimTypeChar(label="Byte")}, name="IPSEC_CIPHER_TRANSFORM_ID0", pack=False, align=None), "cryptoModuleId": SimTypePointer(SimTypeBottom(label="Guid"), offset=0)}, name="IPSEC_CIPHER_TRANSFORM0", pack=False, align=None)}, name="IPSEC_AUTH_AND_CIPHER_TRANSFORM0", pack=False, align=None), offset=0), "espAuthFwTransform": SimTypePointer(SimStruct({"authTransformId": SimStruct({"authType": SimTypeInt(signed=False, label="IPSEC_AUTH_TYPE"), "authConfig": SimTypeChar(label="Byte")}, name="IPSEC_AUTH_TRANSFORM_ID0", pack=False, align=None), "cryptoModuleId": SimTypePointer(SimTypeBottom(label="Guid"), offset=0)}, name="IPSEC_AUTH_TRANSFORM0", pack=False, align=None), offset=0)}, name="<anon>", label="None")}, name="IPSEC_SA_TRANSFORM0", pack=False, align=None), offset=0), "pfsGroup": SimTypeInt(signed=False, label="IPSEC_PFS_GROUP")}, name="IPSEC_PROPOSAL0", pack=False, align=None), offset=0), "tunnelEndpoints": SimStruct({"ipVersion": SimTypeInt(signed=False, label="FWP_IP_VERSION"), "Anonymous1": SimUnion({"localV4Address": SimTypeInt(signed=False, label="UInt32"), "localV6Address": SimTypeFixedSizeArray(SimTypeChar(label="Byte"), 16)}, name="<anon>", label="None"), "Anonymous2": SimUnion({"remoteV4Address": SimTypeInt(signed=False, label="UInt32"), "remoteV6Address": SimTypeFixedSizeArray(SimTypeChar(label="Byte"), 16)}, name="<anon>", label="None")}, name="IPSEC_TUNNEL_ENDPOINTS0", pack=False, align=None), "saIdleTimeout": SimStruct({"idleTimeoutSeconds": SimTypeInt(signed=False, label="UInt32"), "idleTimeoutSecondsFailOver": SimTypeInt(signed=False, label="UInt32")}, name="IPSEC_SA_IDLE_TIMEOUT0", pack=False, align=None), "emPolicy": SimTypePointer(SimStruct({"numAuthenticationMethods": SimTypeInt(signed=False, label="UInt32"), "authenticationMethods": SimTypePointer(SimStruct({"authenticationMethodType": SimTypeInt(signed=False, label="IKEEXT_AUTHENTICATION_METHOD_TYPE"), "Anonymous": SimUnion({"presharedKeyAuthentication": SimStruct({"presharedKey": SimStruct({"size": SimTypeInt(signed=False, label="UInt32"), "data": SimTypePointer(SimTypeChar(label="Byte"), offset=0)}, name="FWP_BYTE_BLOB", pack=False, align=None)}, name="IKEEXT_PRESHARED_KEY_AUTHENTICATION0", pack=False, align=None), "certificateAuthentication": SimStruct({"inboundConfigType": SimTypeInt(signed=False, label="IKEEXT_CERT_CONFIG_TYPE"), "Anonymous1": SimUnion({"Anonymous": SimStruct({"inboundRootArraySize": SimTypeInt(signed=False, label="UInt32"), "inboundRootArray": SimTypePointer(SimStruct({"certData": SimStruct({"size": SimTypeInt(signed=False, label="UInt32"), "data": SimTypePointer(SimTypeChar(label="Byte"), offset=0)}, name="FWP_BYTE_BLOB", pack=False, align=None), "flags": SimTypeInt(signed=False, label="IKEEXT_CERT_FLAGS")}, name="IKEEXT_CERT_ROOT_CONFIG0", pack=False, align=None), offset=0)}, name="_Anonymous_e__Struct", pack=False, align=None), "inboundEnterpriseStoreConfig": SimTypePointer(SimStruct({"certData": SimStruct({"size": SimTypeInt(signed=False, label="UInt32"), "data": SimTypePointer(SimTypeChar(label="Byte"), offset=0)}, name="FWP_BYTE_BLOB", pack=False, align=None), "flags": SimTypeInt(signed=False, label="IKEEXT_CERT_FLAGS")}, name="IKEEXT_CERT_ROOT_CONFIG0", pack=False, align=None), offset=0), "inboundTrustedRootStoreConfig": SimTypePointer(SimStruct({"certData": SimStruct({"size": SimTypeInt(signed=False, label="UInt32"), "data": SimTypePointer(SimTypeChar(label="Byte"), offset=0)}, name="FWP_BYTE_BLOB", pack=False, align=None), "flags": SimTypeInt(signed=False, label="IKEEXT_CERT_FLAGS")}, name="IKEEXT_CERT_ROOT_CONFIG0", pack=False, align=None), offset=0)}, name="<anon>", label="None"), "outboundConfigType": SimTypeInt(signed=False, label="IKEEXT_CERT_CONFIG_TYPE"), "Anonymous2": SimUnion({"Anonymous": SimStruct({"outboundRootArraySize": SimTypeInt(signed=False, label="UInt32"), "outboundRootArray": SimTypePointer(SimStruct({"certData": SimStruct({"size": SimTypeInt(signed=False, label="UInt32"), "data": SimTypePointer(SimTypeChar(label="Byte"), offset=0)}, name="FWP_BYTE_BLOB", pack=False, align=None), "flags": SimTypeInt(signed=False, label="IKEEXT_CERT_FLAGS")}, name="IKEEXT_CERT_ROOT_CONFIG0", pack=False, align=None), offset=0)}, name="_Anonymous_e__Struct", pack=False, align=None), "outboundEnterpriseStoreConfig": SimTypePointer(SimStruct({"certData": SimStruct({"size": SimTypeInt(signed=False, label="UInt32"), "data": SimTypePointer(SimTypeChar(label="Byte"), offset=0)}, name="FWP_BYTE_BLOB", pack=False, align=None), "flags": SimTypeInt(signed=False, label="IKEEXT_CERT_FLAGS")}, name="IKEEXT_CERT_ROOT_CONFIG0", pack=False, align=None), offset=0), "outboundTrustedRootStoreConfig": SimTypePointer(SimStruct({"certData": SimStruct({"size": SimTypeInt(signed=False, label="UInt32"), "data": SimTypePointer(SimTypeChar(label="Byte"), offset=0)}, name="FWP_BYTE_BLOB", pack=False, align=None), "flags": SimTypeInt(signed=False, label="IKEEXT_CERT_FLAGS")}, name="IKEEXT_CERT_ROOT_CONFIG0", pack=False, align=None), offset=0)}, name="<anon>", label="None"), "flags": SimTypeInt(signed=False, label="IKEEXT_CERT_AUTH")}, name="IKEEXT_CERTIFICATE_AUTHENTICATION0", pack=False, align=None), "kerberosAuthentication": SimStruct({"flags": SimTypeInt(signed=False, label="IKEEXT_KERBEROS_AUTHENTICATION_FLAGS")}, name="IKEEXT_KERBEROS_AUTHENTICATION0", pack=False, align=None), "ntlmV2Authentication": SimStruct({"flags": SimTypeInt(signed=False, label="UInt32")}, name="IKEEXT_NTLM_V2_AUTHENTICATION0", pack=False, align=None), "sslAuthentication": SimStruct({"inboundConfigType": SimTypeInt(signed=False, label="IKEEXT_CERT_CONFIG_TYPE"), "Anonymous1": SimUnion({"Anonymous": SimStruct({"inboundRootArraySize": SimTypeInt(signed=False, label="UInt32"), "inboundRootArray": SimTypePointer(SimStruct({"certData": SimStruct({"size": SimTypeInt(signed=False, label="UInt32"), "data": SimTypePointer(SimTypeChar(label="Byte"), offset=0)}, name="FWP_BYTE_BLOB", pack=False, align=None), "flags": SimTypeInt(signed=False, label="IKEEXT_CERT_FLAGS")}, name="IKEEXT_CERT_ROOT_CONFIG0", pack=False, align=None), offset=0)}, name="_Anonymous_e__Struct", pack=False, align=None), "inboundEnterpriseStoreConfig": SimTypePointer(SimStruct({"certData": SimStruct({"size": SimTypeInt(signed=False, label="UInt32"), "data": SimTypePointer(SimTypeChar(label="Byte"), offset=0)}, name="FWP_BYTE_BLOB", pack=False, align=None), "flags": SimTypeInt(signed=False, label="IKEEXT_CERT_FLAGS")}, name="IKEEXT_CERT_ROOT_CONFIG0", pack=False, align=None), offset=0), "inboundTrustedRootStoreConfig": SimTypePointer(SimStruct({"certData": SimStruct({"size": SimTypeInt(signed=False, label="UInt32"), "data": SimTypePointer(SimTypeChar(label="Byte"), offset=0)}, name="FWP_BYTE_BLOB", pack=False, align=None), "flags": SimTypeInt(signed=False, label="IKEEXT_CERT_FLAGS")}, name="IKEEXT_CERT_ROOT_CONFIG0", pack=False, align=None), offset=0)}, name="<anon>", label="None"), "outboundConfigType": SimTypeInt(signed=False, label="IKEEXT_CERT_CONFIG_TYPE"), "Anonymous2": SimUnion({"Anonymous": SimStruct({"outboundRootArraySize": SimTypeInt(signed=False, label="UInt32"), "outboundRootArray": SimTypePointer(SimStruct({"certData": SimStruct({"size": SimTypeInt(signed=False, label="UInt32"), "data": SimTypePointer(SimTypeChar(label="Byte"), offset=0)}, name="FWP_BYTE_BLOB", pack=False, align=None), "flags": SimTypeInt(signed=False, label="IKEEXT_CERT_FLAGS")}, name="IKEEXT_CERT_ROOT_CONFIG0", pack=False, align=None), offset=0)}, name="_Anonymous_e__Struct", pack=False, align=None), "outboundEnterpriseStoreConfig": SimTypePointer(SimStruct({"certData": SimStruct({"size": SimTypeInt(signed=False, label="UInt32"), "data": SimTypePointer(SimTypeChar(label="Byte"), offset=0)}, name="FWP_BYTE_BLOB", pack=False, align=None), "flags": SimTypeInt(signed=False, label="IKEEXT_CERT_FLAGS")}, name="IKEEXT_CERT_ROOT_CONFIG0", pack=False, align=None), offset=0), "outboundTrustedRootStoreConfig": SimTypePointer(SimStruct({"certData": SimStruct({"size": SimTypeInt(signed=False, label="UInt32"), "data": SimTypePointer(SimTypeChar(label="Byte"), offset=0)}, name="FWP_BYTE_BLOB", pack=False, align=None), "flags": SimTypeInt(signed=False, label="IKEEXT_CERT_FLAGS")}, name="IKEEXT_CERT_ROOT_CONFIG0", pack=False, align=None), offset=0)}, name="<anon>", label="None"), "flags": SimTypeInt(signed=False, label="IKEEXT_CERT_AUTH")}, name="IKEEXT_CERTIFICATE_AUTHENTICATION0", pack=False, align=None), "cgaAuthentication": SimStruct({"keyContainerName": SimTypePointer(SimTypeChar(label="Char"), offset=0), "cspName": SimTypePointer(SimTypeChar(label="Char"), offset=0), "cspType": SimTypeInt(signed=False, label="UInt32"), "cgaModifier": SimStruct({"byteArray16": SimTypeFixedSizeArray(SimTypeChar(label="Byte"), 16)}, name="FWP_BYTE_ARRAY16", pack=False, align=None), "cgaCollisionCount": SimTypeChar(label="Byte")}, name="IKEEXT_IPV6_CGA_AUTHENTICATION0", pack=False, align=None)}, name="<anon>", label="None")}, name="IKEEXT_AUTHENTICATION_METHOD0", pack=False, align=None), offset=0), "initiatorImpersonationType": SimTypeInt(signed=False, label="IKEEXT_AUTHENTICATION_IMPERSONATION_TYPE")}, name="IKEEXT_EM_POLICY0", pack=False, align=None), offset=0)}, name="IPSEC_TUNNEL_POLICY0", pack=False, align=None), offset=0), "authipQmTransportPolicy": SimTypePointer(SimStruct({"numIpsecProposals": SimTypeInt(signed=False, label="UInt32"), "ipsecProposals": SimTypePointer(SimStruct({"lifetime": SimStruct({"lifetimeSeconds": SimTypeInt(signed=False, label="UInt32"), "lifetimeKilobytes": SimTypeInt(signed=False, label="UInt32"), "lifetimePackets": SimTypeInt(signed=False, label="UInt32")}, name="IPSEC_SA_LIFETIME0", pack=False, align=None), "numSaTransforms": SimTypeInt(signed=False, label="UInt32"), "saTransforms": SimTypePointer(SimStruct({"ipsecTransformType": SimTypeInt(signed=False, label="IPSEC_TRANSFORM_TYPE"), "Anonymous": SimUnion({"ahTransform": SimTypePointer(SimStruct({"authTransformId": SimStruct({"authType": SimTypeInt(signed=False, label="IPSEC_AUTH_TYPE"), "authConfig": SimTypeChar(label="Byte")}, name="IPSEC_AUTH_TRANSFORM_ID0", pack=False, align=None), "cryptoModuleId": SimTypePointer(SimTypeBottom(label="Guid"), offset=0)}, name="IPSEC_AUTH_TRANSFORM0", pack=False, align=None), offset=0), "espAuthTransform": SimTypePointer(SimStruct({"authTransformId": SimStruct({"authType": SimTypeInt(signed=False, label="IPSEC_AUTH_TYPE"), "authConfig": SimTypeChar(label="Byte")}, name="IPSEC_AUTH_TRANSFORM_ID0", pack=False, align=None), "cryptoModuleId": SimTypePointer(SimTypeBottom(label="Guid"), offset=0)}, name="IPSEC_AUTH_TRANSFORM0", pack=False, align=None), offset=0), "espCipherTransform": SimTypePointer(SimStruct({"cipherTransformId": SimStruct({"cipherType": SimTypeInt(signed=False, label="IPSEC_CIPHER_TYPE"), "cipherConfig": SimTypeChar(label="Byte")}, name="IPSEC_CIPHER_TRANSFORM_ID0", pack=False, align=None), "cryptoModuleId": SimTypePointer(SimTypeBottom(label="Guid"), offset=0)}, name="IPSEC_CIPHER_TRANSFORM0", pack=False, align=None), offset=0), "espAuthAndCipherTransform": SimTypePointer(SimStruct({"authTransform": SimStruct({"authTransformId": SimStruct({"authType": SimTypeInt(signed=False, label="IPSEC_AUTH_TYPE"), "authConfig": SimTypeChar(label="Byte")}, name="IPSEC_AUTH_TRANSFORM_ID0", pack=False, align=None), "cryptoModuleId": SimTypePointer(SimTypeBottom(label="Guid"), offset=0)}, name="IPSEC_AUTH_TRANSFORM0", pack=False, align=None), "cipherTransform": SimStruct({"cipherTransformId": SimStruct({"cipherType": SimTypeInt(signed=False, label="IPSEC_CIPHER_TYPE"), "cipherConfig": SimTypeChar(label="Byte")}, name="IPSEC_CIPHER_TRANSFORM_ID0", pack=False, align=None), "cryptoModuleId": SimTypePointer(SimTypeBottom(label="Guid"), offset=0)}, name="IPSEC_CIPHER_TRANSFORM0", pack=False, align=None)}, name="IPSEC_AUTH_AND_CIPHER_TRANSFORM0", pack=False, align=None), offset=0), "espAuthFwTransform": SimTypePointer(SimStruct({"authTransformId": SimStruct({"authType": SimTypeInt(signed=False, label="IPSEC_AUTH_TYPE"), "authConfig": SimTypeChar(label="Byte")}, name="IPSEC_AUTH_TRANSFORM_ID0", pack=False, align=None), "cryptoModuleId": SimTypePointer(SimTypeBottom(label="Guid"), offset=0)}, name="IPSEC_AUTH_TRANSFORM0", pack=False, align=None), offset=0)}, name="<anon>", label="None")}, name="IPSEC_SA_TRANSFORM0", pack=False, align=None), offset=0), "pfsGroup": SimTypeInt(signed=False, label="IPSEC_PFS_GROUP")}, name="IPSEC_PROPOSAL0", pack=False, align=None), offset=0), "flags": SimTypeInt(signed=False, label="IPSEC_POLICY_FLAG"), "ndAllowClearTimeoutSeconds": SimTypeInt(signed=False, label="UInt32"), "saIdleTimeout": SimStruct({"idleTimeoutSeconds": SimTypeInt(signed=False, label="UInt32"), "idleTimeoutSecondsFailOver": SimTypeInt(signed=False, label="UInt32")}, name="IPSEC_SA_IDLE_TIMEOUT0", pack=False, align=None), "emPolicy": SimTypePointer(SimStruct({"numAuthenticationMethods": SimTypeInt(signed=False, label="UInt32"), "authenticationMethods": SimTypePointer(SimStruct({"authenticationMethodType": SimTypeInt(signed=False, label="IKEEXT_AUTHENTICATION_METHOD_TYPE"), "Anonymous": SimUnion({"presharedKeyAuthentication": SimStruct({"presharedKey": SimStruct({"size": SimTypeInt(signed=False, label="UInt32"), "data": SimTypePointer(SimTypeChar(label="Byte"), offset=0)}, name="FWP_BYTE_BLOB", pack=False, align=None)}, name="IKEEXT_PRESHARED_KEY_AUTHENTICATION0", pack=False, align=None), "certificateAuthentication": SimStruct({"inboundConfigType": SimTypeInt(signed=False, label="IKEEXT_CERT_CONFIG_TYPE"), "Anonymous1": SimUnion({"Anonymous": SimStruct({"inboundRootArraySize": SimTypeInt(signed=False, label="UInt32"), "inboundRootArray": SimTypePointer(SimStruct({"certData": SimStruct({"size": SimTypeInt(signed=False, label="UInt32"), "data": SimTypePointer(SimTypeChar(label="Byte"), offset=0)}, name="FWP_BYTE_BLOB", pack=False, align=None), "flags": SimTypeInt(signed=False, label="IKEEXT_CERT_FLAGS")}, name="IKEEXT_CERT_ROOT_CONFIG0", pack=False, align=None), offset=0)}, name="_Anonymous_e__Struct", pack=False, align=None), "inboundEnterpriseStoreConfig": SimTypePointer(SimStruct({"certData": SimStruct({"size": SimTypeInt(signed=False, label="UInt32"), "data": SimTypePointer(SimTypeChar(label="Byte"), offset=0)}, name="FWP_BYTE_BLOB", pack=False, align=None), "flags": SimTypeInt(signed=False, label="IKEEXT_CERT_FLAGS")}, name="IKEEXT_CERT_ROOT_CONFIG0", pack=False, align=None), offset=0), "inboundTrustedRootStoreConfig": SimTypePointer(SimStruct({"certData": SimStruct({"size": SimTypeInt(signed=False, label="UInt32"), "data": SimTypePointer(SimTypeChar(label="Byte"), offset=0)}, name="FWP_BYTE_BLOB", pack=False, align=None), "flags": SimTypeInt(signed=False, label="IKEEXT_CERT_FLAGS")}, name="IKEEXT_CERT_ROOT_CONFIG0", pack=False, align=None), offset=0)}, name="<anon>", label="None"), "outboundConfigType": SimTypeInt(signed=False, label="IKEEXT_CERT_CONFIG_TYPE"), "Anonymous2": SimUnion({"Anonymous": SimStruct({"outboundRootArraySize": SimTypeInt(signed=False, label="UInt32"), "outboundRootArray": SimTypePointer(SimStruct({"certData": SimStruct({"size": SimTypeInt(signed=False, label="UInt32"), "data": SimTypePointer(SimTypeChar(label="Byte"), offset=0)}, name="FWP_BYTE_BLOB", pack=False, align=None), "flags": SimTypeInt(signed=False, label="IKEEXT_CERT_FLAGS")}, name="IKEEXT_CERT_ROOT_CONFIG0", pack=False, align=None), offset=0)}, name="_Anonymous_e__Struct", pack=False, align=None), "outboundEnterpriseStoreConfig": SimTypePointer(SimStruct({"certData": SimStruct({"size": SimTypeInt(signed=False, label="UInt32"), "data": SimTypePointer(SimTypeChar(label="Byte"), offset=0)}, name="FWP_BYTE_BLOB", pack=False, align=None), "flags": SimTypeInt(signed=False, label="IKEEXT_CERT_FLAGS")}, name="IKEEXT_CERT_ROOT_CONFIG0", pack=False, align=None), offset=0), "outboundTrustedRootStoreConfig": SimTypePointer(SimStruct({"certData": SimStruct({"size": SimTypeInt(signed=False, label="UInt32"), "data": SimTypePointer(SimTypeChar(label="Byte"), offset=0)}, name="FWP_BYTE_BLOB", pack=False, align=None), "flags": SimTypeInt(signed=False, label="IKEEXT_CERT_FLAGS")}, name="IKEEXT_CERT_ROOT_CONFIG0", pack=False, align=None), offset=0)}, name="<anon>", label="None"), "flags": SimTypeInt(signed=False, label="IKEEXT_CERT_AUTH")}, name="IKEEXT_CERTIFICATE_AUTHENTICATION0", pack=False, align=None), "kerberosAuthentication": SimStruct({"flags": SimTypeInt(signed=False, label="IKEEXT_KERBEROS_AUTHENTICATION_FLAGS")}, name="IKEEXT_KERBEROS_AUTHENTICATION0", pack=False, align=None), "ntlmV2Authentication": SimStruct({"flags": SimTypeInt(signed=False, label="UInt32")}, name="IKEEXT_NTLM_V2_AUTHENTICATION0", pack=False, align=None), "sslAuthentication": SimStruct({"inboundConfigType": SimTypeInt(signed=False, label="IKEEXT_CERT_CONFIG_TYPE"), "Anonymous1": SimUnion({"Anonymous": SimStruct({"inboundRootArraySize": SimTypeInt(signed=False, label="UInt32"), "inboundRootArray": SimTypePointer(SimStruct({"certData": SimStruct({"size": SimTypeInt(signed=False, label="UInt32"), "data": SimTypePointer(SimTypeChar(label="Byte"), offset=0)}, name="FWP_BYTE_BLOB", pack=False, align=None), "flags": SimTypeInt(signed=False, label="IKEEXT_CERT_FLAGS")}, name="IKEEXT_CERT_ROOT_CONFIG0", pack=False, align=None), offset=0)}, name="_Anonymous_e__Struct", pack=False, align=None), "inboundEnterpriseStoreConfig": SimTypePointer(SimStruct({"certData": SimStruct({"size": SimTypeInt(signed=False, label="UInt32"), "data": SimTypePointer(SimTypeChar(label="Byte"), offset=0)}, name="FWP_BYTE_BLOB", pack=False, align=None), "flags": SimTypeInt(signed=False, label="IKEEXT_CERT_FLAGS")}, name="IKEEXT_CERT_ROOT_CONFIG0", pack=False, align=None), offset=0), "inboundTrustedRootStoreConfig": SimTypePointer(SimStruct({"certData": SimStruct({"size": SimTypeInt(signed=False, label="UInt32"), "data": SimTypePointer(SimTypeChar(label="Byte"), offset=0)}, name="FWP_BYTE_BLOB", pack=False, align=None), "flags": SimTypeInt(signed=False, label="IKEEXT_CERT_FLAGS")}, name="IKEEXT_CERT_ROOT_CONFIG0", pack=False, align=None), offset=0)}, name="<anon>", label="None"), "outboundConfigType": SimTypeInt(signed=False, label="IKEEXT_CERT_CONFIG_TYPE"), "Anonymous2": SimUnion({"Anonymous": SimStruct({"outboundRootArraySize": SimTypeInt(signed=False, label="UInt32"), "outboundRootArray": SimTypePointer(SimStruct({"certData": SimStruct({"size": SimTypeInt(signed=False, label="UInt32"), "data": SimTypePointer(SimTypeChar(label="Byte"), offset=0)}, name="FWP_BYTE_BLOB", pack=False, align=None), "flags": SimTypeInt(signed=False, label="IKEEXT_CERT_FLAGS")}, name="IKEEXT_CERT_ROOT_CONFIG0", pack=False, align=None), offset=0)}, name="_Anonymous_e__Struct", pack=False, align=None), "outboundEnterpriseStoreConfig": SimTypePointer(SimStruct({"certData": SimStruct({"size": SimTypeInt(signed=False, label="UInt32"), "data": SimTypePointer(SimTypeChar(label="Byte"), offset=0)}, name="FWP_BYTE_BLOB", pack=False, align=None), "flags": SimTypeInt(signed=False, label="IKEEXT_CERT_FLAGS")}, name="IKEEXT_CERT_ROOT_CONFIG0", pack=False, align=None), offset=0), "outboundTrustedRootStoreConfig": SimTypePointer(SimStruct({"certData": SimStruct({"size": SimTypeInt(signed=False, label="UInt32"), "data": SimTypePointer(SimTypeChar(label="Byte"), offset=0)}, name="FWP_BYTE_BLOB", pack=False, align=None), "flags": SimTypeInt(signed=False, label="IKEEXT_CERT_FLAGS")}, name="IKEEXT_CERT_ROOT_CONFIG0", pack=False, align=None), offset=0)}, name="<anon>", label="None"), "flags": SimTypeInt(signed=False, label="IKEEXT_CERT_AUTH")}, name="IKEEXT_CERTIFICATE_AUTHENTICATION0", pack=False, align=None), "cgaAuthentication": SimStruct({"keyContainerName": SimTypePointer(SimTypeChar(label="Char"), offset=0), "cspName": SimTypePointer(SimTypeChar(label="Char"), offset=0), "cspType": SimTypeInt(signed=False, label="UInt32"), "cgaModifier": SimStruct({"byteArray16": SimTypeFixedSizeArray(SimTypeChar(label="Byte"), 16)}, name="FWP_BYTE_ARRAY16", pack=False, align=None), "cgaCollisionCount": SimTypeChar(label="Byte")}, name="IKEEXT_IPV6_CGA_AUTHENTICATION0", pack=False, align=None)}, name="<anon>", label="None")}, name="IKEEXT_AUTHENTICATION_METHOD0", pack=False, align=None), offset=0), "initiatorImpersonationType": SimTypeInt(signed=False, label="IKEEXT_AUTHENTICATION_IMPERSONATION_TYPE")}, name="IKEEXT_EM_POLICY0", pack=False, align=None), offset=0)}, name="IPSEC_TRANSPORT_POLICY0", pack=False, align=None), offset=0), "authipQmTunnelPolicy": SimTypePointer(SimStruct({"flags": SimTypeInt(signed=False, label="IPSEC_POLICY_FLAG"), "numIpsecProposals": SimTypeInt(signed=False, label="UInt32"), "ipsecProposals": SimTypePointer(SimStruct({"lifetime": SimStruct({"lifetimeSeconds": SimTypeInt(signed=False, label="UInt32"), "lifetimeKilobytes": SimTypeInt(signed=False, label="UInt32"), "lifetimePackets": SimTypeInt(signed=False, label="UInt32")}, name="IPSEC_SA_LIFETIME0", pack=False, align=None), "numSaTransforms": SimTypeInt(signed=False, label="UInt32"), "saTransforms": SimTypePointer(SimStruct({"ipsecTransformType": SimTypeInt(signed=False, label="IPSEC_TRANSFORM_TYPE"), "Anonymous": SimUnion({"ahTransform": SimTypePointer(SimStruct({"authTransformId": SimStruct({"authType": SimTypeInt(signed=False, label="IPSEC_AUTH_TYPE"), "authConfig": SimTypeChar(label="Byte")}, name="IPSEC_AUTH_TRANSFORM_ID0", pack=False, align=None), "cryptoModuleId": SimTypePointer(SimTypeBottom(label="Guid"), offset=0)}, name="IPSEC_AUTH_TRANSFORM0", pack=False, align=None), offset=0), "espAuthTransform": SimTypePointer(SimStruct({"authTransformId": SimStruct({"authType": SimTypeInt(signed=False, label="IPSEC_AUTH_TYPE"), "authConfig": SimTypeChar(label="Byte")}, name="IPSEC_AUTH_TRANSFORM_ID0", pack=False, align=None), "cryptoModuleId": SimTypePointer(SimTypeBottom(label="Guid"), offset=0)}, name="IPSEC_AUTH_TRANSFORM0", pack=False, align=None), offset=0), "espCipherTransform": SimTypePointer(SimStruct({"cipherTransformId": SimStruct({"cipherType": SimTypeInt(signed=False, label="IPSEC_CIPHER_TYPE"), "cipherConfig": SimTypeChar(label="Byte")}, name="IPSEC_CIPHER_TRANSFORM_ID0", pack=False, align=None), "cryptoModuleId": SimTypePointer(SimTypeBottom(label="Guid"), offset=0)}, name="IPSEC_CIPHER_TRANSFORM0", pack=False, align=None), offset=0), "espAuthAndCipherTransform": SimTypePointer(SimStruct({"authTransform": SimStruct({"authTransformId": SimStruct({"authType": SimTypeInt(signed=False, label="IPSEC_AUTH_TYPE"), "authConfig": SimTypeChar(label="Byte")}, name="IPSEC_AUTH_TRANSFORM_ID0", pack=False, align=None), "cryptoModuleId": SimTypePointer(SimTypeBottom(label="Guid"), offset=0)}, name="IPSEC_AUTH_TRANSFORM0", pack=False, align=None), "cipherTransform": SimStruct({"cipherTransformId": SimStruct({"cipherType": SimTypeInt(signed=False, label="IPSEC_CIPHER_TYPE"), "cipherConfig": SimTypeChar(label="Byte")}, name="IPSEC_CIPHER_TRANSFORM_ID0", pack=False, align=None), "cryptoModuleId": SimTypePointer(SimTypeBottom(label="Guid"), offset=0)}, name="IPSEC_CIPHER_TRANSFORM0", pack=False, align=None)}, name="IPSEC_AUTH_AND_CIPHER_TRANSFORM0", pack=False, align=None), offset=0), "espAuthFwTransform": SimTypePointer(SimStruct({"authTransformId": SimStruct({"authType": SimTypeInt(signed=False, label="IPSEC_AUTH_TYPE"), "authConfig": SimTypeChar(label="Byte")}, name="IPSEC_AUTH_TRANSFORM_ID0", pack=False, align=None), "cryptoModuleId": SimTypePointer(SimTypeBottom(label="Guid"), offset=0)}, name="IPSEC_AUTH_TRANSFORM0", pack=False, align=None), offset=0)}, name="<anon>", label="None")}, name="IPSEC_SA_TRANSFORM0", pack=False, align=None), offset=0), "pfsGroup": SimTypeInt(signed=False, label="IPSEC_PFS_GROUP")}, name="IPSEC_PROPOSAL0", pack=False, align=None), offset=0), "tunnelEndpoints": SimStruct({"ipVersion": SimTypeInt(signed=False, label="FWP_IP_VERSION"), "Anonymous1": SimUnion({"localV4Address": SimTypeInt(signed=False, label="UInt32"), "localV6Address": SimTypeFixedSizeArray(SimTypeChar(label="Byte"), 16)}, name="<anon>", label="None"), "Anonymous2": SimUnion({"remoteV4Address": SimTypeInt(signed=False, label="UInt32"), "remoteV6Address": SimTypeFixedSizeArray(SimTypeChar(label="Byte"), 16)}, name="<anon>", label="None")}, name="IPSEC_TUNNEL_ENDPOINTS0", pack=False, align=None), "saIdleTimeout": SimStruct({"idleTimeoutSeconds": SimTypeInt(signed=False, label="UInt32"), "idleTimeoutSecondsFailOver": SimTypeInt(signed=False, label="UInt32")}, name="IPSEC_SA_IDLE_TIMEOUT0", pack=False, align=None), "emPolicy": SimTypePointer(SimStruct({"numAuthenticationMethods": SimTypeInt(signed=False, label="UInt32"), "authenticationMethods": SimTypePointer(SimStruct({"authenticationMethodType": SimTypeInt(signed=False, label="IKEEXT_AUTHENTICATION_METHOD_TYPE"), "Anonymous": SimUnion({"presharedKeyAuthentication": SimStruct({"presharedKey": SimStruct({"size": SimTypeInt(signed=False, label="UInt32"), "data": SimTypePointer(SimTypeChar(label="Byte"), offset=0)}, name="FWP_BYTE_BLOB", pack=False, align=None)}, name="IKEEXT_PRESHARED_KEY_AUTHENTICATION0", pack=False, align=None), "certificateAuthentication": SimStruct({"inboundConfigType": SimTypeInt(signed=False, label="IKEEXT_CERT_CONFIG_TYPE"), "Anonymous1": SimUnion({"Anonymous": SimStruct({"inboundRootArraySize": SimTypeInt(signed=False, label="UInt32"), "inboundRootArray": SimTypePointer(SimStruct({"certData": SimStruct({"size": SimTypeInt(signed=False, label="UInt32"), "data": SimTypePointer(SimTypeChar(label="Byte"), offset=0)}, name="FWP_BYTE_BLOB", pack=False, align=None), "flags": SimTypeInt(signed=False, label="IKEEXT_CERT_FLAGS")}, name="IKEEXT_CERT_ROOT_CONFIG0", pack=False, align=None), offset=0)}, name="_Anonymous_e__Struct", pack=False, align=None), "inboundEnterpriseStoreConfig": SimTypePointer(SimStruct({"certData": SimStruct({"size": SimTypeInt(signed=False, label="UInt32"), "data": SimTypePointer(SimTypeChar(label="Byte"), offset=0)}, name="FWP_BYTE_BLOB", pack=False, align=None), "flags": SimTypeInt(signed=False, label="IKEEXT_CERT_FLAGS")}, name="IKEEXT_CERT_ROOT_CONFIG0", pack=False, align=None), offset=0), "inboundTrustedRootStoreConfig": SimTypePointer(SimStruct({"certData": SimStruct({"size": SimTypeInt(signed=False, label="UInt32"), "data": SimTypePointer(SimTypeChar(label="Byte"), offset=0)}, name="FWP_BYTE_BLOB", pack=False, align=None), "flags": SimTypeInt(signed=False, label="IKEEXT_CERT_FLAGS")}, name="IKEEXT_CERT_ROOT_CONFIG0", pack=False, align=None), offset=0)}, name="<anon>", label="None"), "outboundConfigType": SimTypeInt(signed=False, label="IKEEXT_CERT_CONFIG_TYPE"), "Anonymous2": SimUnion({"Anonymous": SimStruct({"outboundRootArraySize": SimTypeInt(signed=False, label="UInt32"), "outboundRootArray": SimTypePointer(SimStruct({"certData": SimStruct({"size": SimTypeInt(signed=False, label="UInt32"), "data": SimTypePointer(SimTypeChar(label="Byte"), offset=0)}, name="FWP_BYTE_BLOB", pack=False, align=None), "flags": SimTypeInt(signed=False, label="IKEEXT_CERT_FLAGS")}, name="IKEEXT_CERT_ROOT_CONFIG0", pack=False, align=None), offset=0)}, name="_Anonymous_e__Struct", pack=False, align=None), "outboundEnterpriseStoreConfig": SimTypePointer(SimStruct({"certData": SimStruct({"size": SimTypeInt(signed=False, label="UInt32"), "data": SimTypePointer(SimTypeChar(label="Byte"), offset=0)}, name="FWP_BYTE_BLOB", pack=False, align=None), "flags": SimTypeInt(signed=False, label="IKEEXT_CERT_FLAGS")}, name="IKEEXT_CERT_ROOT_CONFIG0", pack=False, align=None), offset=0), "outboundTrustedRootStoreConfig": SimTypePointer(SimStruct({"certData": SimStruct({"size": SimTypeInt(signed=False, label="UInt32"), "data": SimTypePointer(SimTypeChar(label="Byte"), offset=0)}, name="FWP_BYTE_BLOB", pack=False, align=None), "flags": SimTypeInt(signed=False, label="IKEEXT_CERT_FLAGS")}, name="IKEEXT_CERT_ROOT_CONFIG0", pack=False, align=None), offset=0)}, name="<anon>", label="None"), "flags": SimTypeInt(signed=False, label="IKEEXT_CERT_AUTH")}, name="IKEEXT_CERTIFICATE_AUTHENTICATION0", pack=False, align=None), "kerberosAuthentication": SimStruct({"flags": SimTypeInt(signed=False, label="IKEEXT_KERBEROS_AUTHENTICATION_FLAGS")}, name="IKEEXT_KERBEROS_AUTHENTICATION0", pack=False, align=None), "ntlmV2Authentication": SimStruct({"flags": SimTypeInt(signed=False, label="UInt32")}, name="IKEEXT_NTLM_V2_AUTHENTICATION0", pack=False, align=None), "sslAuthentication": SimStruct({"inboundConfigType": SimTypeInt(signed=False, label="IKEEXT_CERT_CONFIG_TYPE"), "Anonymous1": SimUnion({"Anonymous": SimStruct({"inboundRootArraySize": SimTypeInt(signed=False, label="UInt32"), "inboundRootArray": SimTypePointer(SimStruct({"certData": SimStruct({"size": SimTypeInt(signed=False, label="UInt32"), "data": SimTypePointer(SimTypeChar(label="Byte"), offset=0)}, name="FWP_BYTE_BLOB", pack=False, align=None), "flags": SimTypeInt(signed=False, label="IKEEXT_CERT_FLAGS")}, name="IKEEXT_CERT_ROOT_CONFIG0", pack=False, align=None), offset=0)}, name="_Anonymous_e__Struct", pack=False, align=None), "inboundEnterpriseStoreConfig": SimTypePointer(SimStruct({"certData": SimStruct({"size": SimTypeInt(signed=False, label="UInt32"), "data": SimTypePointer(SimTypeChar(label="Byte"), offset=0)}, name="FWP_BYTE_BLOB", pack=False, align=None), "flags": SimTypeInt(signed=False, label="IKEEXT_CERT_FLAGS")}, name="IKEEXT_CERT_ROOT_CONFIG0", pack=False, align=None), offset=0), "inboundTrustedRootStoreConfig": SimTypePointer(SimStruct({"certData": SimStruct({"size": SimTypeInt(signed=False, label="UInt32"), "data": SimTypePointer(SimTypeChar(label="Byte"), offset=0)}, name="FWP_BYTE_BLOB", pack=False, align=None), "flags": SimTypeInt(signed=False, label="IKEEXT_CERT_FLAGS")}, name="IKEEXT_CERT_ROOT_CONFIG0", pack=False, align=None), offset=0)}, name="<anon>", label="None"), "outboundConfigType": SimTypeInt(signed=False, label="IKEEXT_CERT_CONFIG_TYPE"), "Anonymous2": SimUnion({"Anonymous": SimStruct({"outboundRootArraySize": SimTypeInt(signed=False, label="UInt32"), "outboundRootArray": SimTypePointer(SimStruct({"certData": SimStruct({"size": SimTypeInt(signed=False, label="UInt32"), "data": SimTypePointer(SimTypeChar(label="Byte"), offset=0)}, name="FWP_BYTE_BLOB", pack=False, align=None), "flags": SimTypeInt(signed=False, label="IKEEXT_CERT_FLAGS")}, name="IKEEXT_CERT_ROOT_CONFIG0", pack=False, align=None), offset=0)}, name="_Anonymous_e__Struct", pack=False, align=None), "outboundEnterpriseStoreConfig": SimTypePointer(SimStruct({"certData": SimStruct({"size": SimTypeInt(signed=False, label="UInt32"), "data": SimTypePointer(SimTypeChar(label="Byte"), offset=0)}, name="FWP_BYTE_BLOB", pack=False, align=None), "flags": SimTypeInt(signed=False, label="IKEEXT_CERT_FLAGS")}, name="IKEEXT_CERT_ROOT_CONFIG0", pack=False, align=None), offset=0), "outboundTrustedRootStoreConfig": SimTypePointer(SimStruct({"certData": SimStruct({"size": SimTypeInt(signed=False, label="UInt32"), "data": SimTypePointer(SimTypeChar(label="Byte"), offset=0)}, name="FWP_BYTE_BLOB", pack=False, align=None), "flags": SimTypeInt(signed=False, label="IKEEXT_CERT_FLAGS")}, name="IKEEXT_CERT_ROOT_CONFIG0", pack=False, align=None), offset=0)}, name="<anon>", label="None"), "flags": SimTypeInt(signed=False, label="IKEEXT_CERT_AUTH")}, name="IKEEXT_CERTIFICATE_AUTHENTICATION0", pack=False, align=None), "cgaAuthentication": SimStruct({"keyContainerName": SimTypePointer(SimTypeChar(label="Char"), offset=0), "cspName": SimTypePointer(SimTypeChar(label="Char"), offset=0), "cspType": SimTypeInt(signed=False, label="UInt32"), "cgaModifier": SimStruct({"byteArray16": SimTypeFixedSizeArray(SimTypeChar(label="Byte"), 16)}, name="FWP_BYTE_ARRAY16", pack=False, align=None), "cgaCollisionCount": SimTypeChar(label="Byte")}, name="IKEEXT_IPV6_CGA_AUTHENTICATION0", pack=False, align=None)}, name="<anon>", label="None")}, name="IKEEXT_AUTHENTICATION_METHOD0", pack=False, align=None), offset=0), "initiatorImpersonationType": SimTypeInt(signed=False, label="IKEEXT_AUTHENTICATION_IMPERSONATION_TYPE")}, name="IKEEXT_EM_POLICY0", pack=False, align=None), offset=0)}, name="IPSEC_TUNNEL_POLICY0", pack=False, align=None), offset=0), "ikeMmPolicy": SimTypePointer(SimStruct({"softExpirationTime": SimTypeInt(signed=False, label="UInt32"), "numAuthenticationMethods": SimTypeInt(signed=False, label="UInt32"), "authenticationMethods": SimTypePointer(SimStruct({"authenticationMethodType": SimTypeInt(signed=False, label="IKEEXT_AUTHENTICATION_METHOD_TYPE"), "Anonymous": SimUnion({"presharedKeyAuthentication": SimStruct({"presharedKey": SimStruct({"size": SimTypeInt(signed=False, label="UInt32"), "data": SimTypePointer(SimTypeChar(label="Byte"), offset=0)}, name="FWP_BYTE_BLOB", pack=False, align=None)}, name="IKEEXT_PRESHARED_KEY_AUTHENTICATION0", pack=False, align=None), "certificateAuthentication": SimStruct({"inboundConfigType": SimTypeInt(signed=False, label="IKEEXT_CERT_CONFIG_TYPE"), "Anonymous1": SimUnion({"Anonymous": SimStruct({"inboundRootArraySize": SimTypeInt(signed=False, label="UInt32"), "inboundRootArray": SimTypePointer(SimStruct({"certData": SimStruct({"size": SimTypeInt(signed=False, label="UInt32"), "data": SimTypePointer(SimTypeChar(label="Byte"), offset=0)}, name="FWP_BYTE_BLOB", pack=False, align=None), "flags": SimTypeInt(signed=False, label="IKEEXT_CERT_FLAGS")}, name="IKEEXT_CERT_ROOT_CONFIG0", pack=False, align=None), offset=0)}, name="_Anonymous_e__Struct", pack=False, align=None), "inboundEnterpriseStoreConfig": SimTypePointer(SimStruct({"certData": SimStruct({"size": SimTypeInt(signed=False, label="UInt32"), "data": SimTypePointer(SimTypeChar(label="Byte"), offset=0)}, name="FWP_BYTE_BLOB", pack=False, align=None), "flags": SimTypeInt(signed=False, label="IKEEXT_CERT_FLAGS")}, name="IKEEXT_CERT_ROOT_CONFIG0", pack=False, align=None), offset=0), "inboundTrustedRootStoreConfig": SimTypePointer(SimStruct({"certData": SimStruct({"size": SimTypeInt(signed=False, label="UInt32"), "data": SimTypePointer(SimTypeChar(label="Byte"), offset=0)}, name="FWP_BYTE_BLOB", pack=False, align=None), "flags": SimTypeInt(signed=False, label="IKEEXT_CERT_FLAGS")}, name="IKEEXT_CERT_ROOT_CONFIG0", pack=False, align=None), offset=0)}, name="<anon>", label="None"), "outboundConfigType": SimTypeInt(signed=False, label="IKEEXT_CERT_CONFIG_TYPE"), "Anonymous2": SimUnion({"Anonymous": SimStruct({"outboundRootArraySize": SimTypeInt(signed=False, label="UInt32"), "outboundRootArray": SimTypePointer(SimStruct({"certData": SimStruct({"size": SimTypeInt(signed=False, label="UInt32"), "data": SimTypePointer(SimTypeChar(label="Byte"), offset=0)}, name="FWP_BYTE_BLOB", pack=False, align=None), "flags": SimTypeInt(signed=False, label="IKEEXT_CERT_FLAGS")}, name="IKEEXT_CERT_ROOT_CONFIG0", pack=False, align=None), offset=0)}, name="_Anonymous_e__Struct", pack=False, align=None), "outboundEnterpriseStoreConfig": SimTypePointer(SimStruct({"certData": SimStruct({"size": SimTypeInt(signed=False, label="UInt32"), "data": SimTypePointer(SimTypeChar(label="Byte"), offset=0)}, name="FWP_BYTE_BLOB", pack=False, align=None), "flags": SimTypeInt(signed=False, label="IKEEXT_CERT_FLAGS")}, name="IKEEXT_CERT_ROOT_CONFIG0", pack=False, align=None), offset=0), "outboundTrustedRootStoreConfig": SimTypePointer(SimStruct({"certData": SimStruct({"size": SimTypeInt(signed=False, label="UInt32"), "data": SimTypePointer(SimTypeChar(label="Byte"), offset=0)}, name="FWP_BYTE_BLOB", pack=False, align=None), "flags": SimTypeInt(signed=False, label="IKEEXT_CERT_FLAGS")}, name="IKEEXT_CERT_ROOT_CONFIG0", pack=False, align=None), offset=0)}, name="<anon>", label="None"), "flags": SimTypeInt(signed=False, label="IKEEXT_CERT_AUTH")}, name="IKEEXT_CERTIFICATE_AUTHENTICATION0", pack=False, align=None), "kerberosAuthentication": SimStruct({"flags": SimTypeInt(signed=False, label="IKEEXT_KERBEROS_AUTHENTICATION_FLAGS")}, name="IKEEXT_KERBEROS_AUTHENTICATION0", pack=False, align=None), "ntlmV2Authentication": SimStruct({"flags": SimTypeInt(signed=False, label="UInt32")}, name="IKEEXT_NTLM_V2_AUTHENTICATION0", pack=False, align=None), "sslAuthentication": SimStruct({"inboundConfigType": SimTypeInt(signed=False, label="IKEEXT_CERT_CONFIG_TYPE"), "Anonymous1": SimUnion({"Anonymous": SimStruct({"inboundRootArraySize": SimTypeInt(signed=False, label="UInt32"), "inboundRootArray": SimTypePointer(SimStruct({"certData": SimStruct({"size": SimTypeInt(signed=False, label="UInt32"), "data": SimTypePointer(SimTypeChar(label="Byte"), offset=0)}, name="FWP_BYTE_BLOB", pack=False, align=None), "flags": SimTypeInt(signed=False, label="IKEEXT_CERT_FLAGS")}, name="IKEEXT_CERT_ROOT_CONFIG0", pack=False, align=None), offset=0)}, name="_Anonymous_e__Struct", pack=False, align=None), "inboundEnterpriseStoreConfig": SimTypePointer(SimStruct({"certData": SimStruct({"size": SimTypeInt(signed=False, label="UInt32"), "data": SimTypePointer(SimTypeChar(label="Byte"), offset=0)}, name="FWP_BYTE_BLOB", pack=False, align=None), "flags": SimTypeInt(signed=False, label="IKEEXT_CERT_FLAGS")}, name="IKEEXT_CERT_ROOT_CONFIG0", pack=False, align=None), offset=0), "inboundTrustedRootStoreConfig": SimTypePointer(SimStruct({"certData": SimStruct({"size": SimTypeInt(signed=False, label="UInt32"), "data": SimTypePointer(SimTypeChar(label="Byte"), offset=0)}, name="FWP_BYTE_BLOB", pack=False, align=None), "flags": SimTypeInt(signed=False, label="IKEEXT_CERT_FLAGS")}, name="IKEEXT_CERT_ROOT_CONFIG0", pack=False, align=None), offset=0)}, name="<anon>", label="None"), "outboundConfigType": SimTypeInt(signed=False, label="IKEEXT_CERT_CONFIG_TYPE"), "Anonymous2": SimUnion({"Anonymous": SimStruct({"outboundRootArraySize": SimTypeInt(signed=False, label="UInt32"), "outboundRootArray": SimTypePointer(SimStruct({"certData": SimStruct({"size": SimTypeInt(signed=False, label="UInt32"), "data": SimTypePointer(SimTypeChar(label="Byte"), offset=0)}, name="FWP_BYTE_BLOB", pack=False, align=None), "flags": SimTypeInt(signed=False, label="IKEEXT_CERT_FLAGS")}, name="IKEEXT_CERT_ROOT_CONFIG0", pack=False, align=None), offset=0)}, name="_Anonymous_e__Struct", pack=False, align=None), "outboundEnterpriseStoreConfig": SimTypePointer(SimStruct({"certData": SimStruct({"size": SimTypeInt(signed=False, label="UInt32"), "data": SimTypePointer(SimTypeChar(label="Byte"), offset=0)}, name="FWP_BYTE_BLOB", pack=False, align=None), "flags": SimTypeInt(signed=False, label="IKEEXT_CERT_FLAGS")}, name="IKEEXT_CERT_ROOT_CONFIG0", pack=False, align=None), offset=0), "outboundTrustedRootStoreConfig": SimTypePointer(SimStruct({"certData": SimStruct({"size": SimTypeInt(signed=False, label="UInt32"), "data": SimTypePointer(SimTypeChar(label="Byte"), offset=0)}, name="FWP_BYTE_BLOB", pack=False, align=None), "flags": SimTypeInt(signed=False, label="IKEEXT_CERT_FLAGS")}, name="IKEEXT_CERT_ROOT_CONFIG0", pack=False, align=None), offset=0)}, name="<anon>", label="None"), "flags": SimTypeInt(signed=False, label="IKEEXT_CERT_AUTH")}, name="IKEEXT_CERTIFICATE_AUTHENTICATION0", pack=False, align=None), "cgaAuthentication": SimStruct({"keyContainerName": SimTypePointer(SimTypeChar(label="Char"), offset=0), "cspName": SimTypePointer(SimTypeChar(label="Char"), offset=0), "cspType": SimTypeInt(signed=False, label="UInt32"), "cgaModifier": SimStruct({"byteArray16": SimTypeFixedSizeArray(SimTypeChar(label="Byte"), 16)}, name="FWP_BYTE_ARRAY16", pack=False, align=None), "cgaCollisionCount": SimTypeChar(label="Byte")}, name="IKEEXT_IPV6_CGA_AUTHENTICATION0", pack=False, align=None)}, name="<anon>", label="None")}, name="IKEEXT_AUTHENTICATION_METHOD0", pack=False, align=None), offset=0), "initiatorImpersonationType": SimTypeInt(signed=False, label="IKEEXT_AUTHENTICATION_IMPERSONATION_TYPE"), "numIkeProposals": SimTypeInt(signed=False, label="UInt32"), "ikeProposals": SimTypePointer(SimStruct({"cipherAlgorithm": SimStruct({"algoIdentifier": SimTypeInt(signed=False, label="IKEEXT_CIPHER_TYPE"), "keyLen": SimTypeInt(signed=False, label="UInt32"), "rounds": SimTypeInt(signed=False, label="UInt32")}, name="IKEEXT_CIPHER_ALGORITHM0", pack=False, align=None), "integrityAlgorithm": SimStruct({"algoIdentifier": SimTypeInt(signed=False, label="IKEEXT_INTEGRITY_TYPE")}, name="IKEEXT_INTEGRITY_ALGORITHM0", pack=False, align=None), "maxLifetimeSeconds": SimTypeInt(signed=False, label="UInt32"), "dhGroup": SimTypeInt(signed=False, label="IKEEXT_DH_GROUP"), "quickModeLimit": SimTypeInt(signed=False, label="UInt32")}, name="IKEEXT_PROPOSAL0", pack=False, align=None), offset=0), "flags": SimTypeInt(signed=False, label="IKEEXT_POLICY_FLAG"), "maxDynamicFilters": SimTypeInt(signed=False, label="UInt32")}, name="IKEEXT_POLICY0", pack=False, align=None), offset=0), "authIpMmPolicy": SimTypePointer(SimStruct({"softExpirationTime": SimTypeInt(signed=False, label="UInt32"), "numAuthenticationMethods": SimTypeInt(signed=False, label="UInt32"), "authenticationMethods": SimTypePointer(SimStruct({"authenticationMethodType": SimTypeInt(signed=False, label="IKEEXT_AUTHENTICATION_METHOD_TYPE"), "Anonymous": SimUnion({"presharedKeyAuthentication": SimStruct({"presharedKey": SimStruct({"size": SimTypeInt(signed=False, label="UInt32"), "data": SimTypePointer(SimTypeChar(label="Byte"), offset=0)}, name="FWP_BYTE_BLOB", pack=False, align=None)}, name="IKEEXT_PRESHARED_KEY_AUTHENTICATION0", pack=False, align=None), "certificateAuthentication": SimStruct({"inboundConfigType": SimTypeInt(signed=False, label="IKEEXT_CERT_CONFIG_TYPE"), "Anonymous1": SimUnion({"Anonymous": SimStruct({"inboundRootArraySize": SimTypeInt(signed=False, label="UInt32"), "inboundRootArray": SimTypePointer(SimStruct({"certData": SimStruct({"size": SimTypeInt(signed=False, label="UInt32"), "data": SimTypePointer(SimTypeChar(label="Byte"), offset=0)}, name="FWP_BYTE_BLOB", pack=False, align=None), "flags": SimTypeInt(signed=False, label="IKEEXT_CERT_FLAGS")}, name="IKEEXT_CERT_ROOT_CONFIG0", pack=False, align=None), offset=0)}, name="_Anonymous_e__Struct", pack=False, align=None), "inboundEnterpriseStoreConfig": SimTypePointer(SimStruct({"certData": SimStruct({"size": SimTypeInt(signed=False, label="UInt32"), "data": SimTypePointer(SimTypeChar(label="Byte"), offset=0)}, name="FWP_BYTE_BLOB", pack=False, align=None), "flags": SimTypeInt(signed=False, label="IKEEXT_CERT_FLAGS")}, name="IKEEXT_CERT_ROOT_CONFIG0", pack=False, align=None), offset=0), "inboundTrustedRootStoreConfig": SimTypePointer(SimStruct({"certData": SimStruct({"size": SimTypeInt(signed=False, label="UInt32"), "data": SimTypePointer(SimTypeChar(label="Byte"), offset=0)}, name="FWP_BYTE_BLOB", pack=False, align=None), "flags": SimTypeInt(signed=False, label="IKEEXT_CERT_FLAGS")}, name="IKEEXT_CERT_ROOT_CONFIG0", pack=False, align=None), offset=0)}, name="<anon>", label="None"), "outboundConfigType": SimTypeInt(signed=False, label="IKEEXT_CERT_CONFIG_TYPE"), "Anonymous2": SimUnion({"Anonymous": SimStruct({"outboundRootArraySize": SimTypeInt(signed=False, label="UInt32"), "outboundRootArray": SimTypePointer(SimStruct({"certData": SimStruct({"size": SimTypeInt(signed=False, label="UInt32"), "data": SimTypePointer(SimTypeChar(label="Byte"), offset=0)}, name="FWP_BYTE_BLOB", pack=False, align=None), "flags": SimTypeInt(signed=False, label="IKEEXT_CERT_FLAGS")}, name="IKEEXT_CERT_ROOT_CONFIG0", pack=False, align=None), offset=0)}, name="_Anonymous_e__Struct", pack=False, align=None), "outboundEnterpriseStoreConfig": SimTypePointer(SimStruct({"certData": SimStruct({"size": SimTypeInt(signed=False, label="UInt32"), "data": SimTypePointer(SimTypeChar(label="Byte"), offset=0)}, name="FWP_BYTE_BLOB", pack=False, align=None), "flags": SimTypeInt(signed=False, label="IKEEXT_CERT_FLAGS")}, name="IKEEXT_CERT_ROOT_CONFIG0", pack=False, align=None), offset=0), "outboundTrustedRootStoreConfig": SimTypePointer(SimStruct({"certData": SimStruct({"size": SimTypeInt(signed=False, label="UInt32"), "data": SimTypePointer(SimTypeChar(label="Byte"), offset=0)}, name="FWP_BYTE_BLOB", pack=False, align=None), "flags": SimTypeInt(signed=False, label="IKEEXT_CERT_FLAGS")}, name="IKEEXT_CERT_ROOT_CONFIG0", pack=False, align=None), offset=0)}, name="<anon>", label="None"), "flags": SimTypeInt(signed=False, label="IKEEXT_CERT_AUTH")}, name="IKEEXT_CERTIFICATE_AUTHENTICATION0", pack=False, align=None), "kerberosAuthentication": SimStruct({"flags": SimTypeInt(signed=False, label="IKEEXT_KERBEROS_AUTHENTICATION_FLAGS")}, name="IKEEXT_KERBEROS_AUTHENTICATION0", pack=False, align=None), "ntlmV2Authentication": SimStruct({"flags": SimTypeInt(signed=False, label="UInt32")}, name="IKEEXT_NTLM_V2_AUTHENTICATION0", pack=False, align=None), "sslAuthentication": SimStruct({"inboundConfigType": SimTypeInt(signed=False, label="IKEEXT_CERT_CONFIG_TYPE"), "Anonymous1": SimUnion({"Anonymous": SimStruct({"inboundRootArraySize": SimTypeInt(signed=False, label="UInt32"), "inboundRootArray": SimTypePointer(SimStruct({"certData": SimStruct({"size": SimTypeInt(signed=False, label="UInt32"), "data": SimTypePointer(SimTypeChar(label="Byte"), offset=0)}, name="FWP_BYTE_BLOB", pack=False, align=None), "flags": SimTypeInt(signed=False, label="IKEEXT_CERT_FLAGS")}, name="IKEEXT_CERT_ROOT_CONFIG0", pack=False, align=None), offset=0)}, name="_Anonymous_e__Struct", pack=False, align=None), "inboundEnterpriseStoreConfig": SimTypePointer(SimStruct({"certData": SimStruct({"size": SimTypeInt(signed=False, label="UInt32"), "data": SimTypePointer(SimTypeChar(label="Byte"), offset=0)}, name="FWP_BYTE_BLOB", pack=False, align=None), "flags": SimTypeInt(signed=False, label="IKEEXT_CERT_FLAGS")}, name="IKEEXT_CERT_ROOT_CONFIG0", pack=False, align=None), offset=0), "inboundTrustedRootStoreConfig": SimTypePointer(SimStruct({"certData": SimStruct({"size": SimTypeInt(signed=False, label="UInt32"), "data": SimTypePointer(SimTypeChar(label="Byte"), offset=0)}, name="FWP_BYTE_BLOB", pack=False, align=None), "flags": SimTypeInt(signed=False, label="IKEEXT_CERT_FLAGS")}, name="IKEEXT_CERT_ROOT_CONFIG0", pack=False, align=None), offset=0)}, name="<anon>", label="None"), "outboundConfigType": SimTypeInt(signed=False, label="IKEEXT_CERT_CONFIG_TYPE"), "Anonymous2": SimUnion({"Anonymous": SimStruct({"outboundRootArraySize": SimTypeInt(signed=False, label="UInt32"), "outboundRootArray": SimTypePointer(SimStruct({"certData": SimStruct({"size": SimTypeInt(signed=False, label="UInt32"), "data": SimTypePointer(SimTypeChar(label="Byte"), offset=0)}, name="FWP_BYTE_BLOB", pack=False, align=None), "flags": SimTypeInt(signed=False, label="IKEEXT_CERT_FLAGS")}, name="IKEEXT_CERT_ROOT_CONFIG0", pack=False, align=None), offset=0)}, name="_Anonymous_e__Struct", pack=False, align=None), "outboundEnterpriseStoreConfig": SimTypePointer(SimStruct({"certData": SimStruct({"size": SimTypeInt(signed=False, label="UInt32"), "data": SimTypePointer(SimTypeChar(label="Byte"), offset=0)}, name="FWP_BYTE_BLOB", pack=False, align=None), "flags": SimTypeInt(signed=False, label="IKEEXT_CERT_FLAGS")}, name="IKEEXT_CERT_ROOT_CONFIG0", pack=False, align=None), offset=0), "outboundTrustedRootStoreConfig": SimTypePointer(SimStruct({"certData": SimStruct({"size": SimTypeInt(signed=False, label="UInt32"), "data": SimTypePointer(SimTypeChar(label="Byte"), offset=0)}, name="FWP_BYTE_BLOB", pack=False, align=None), "flags": SimTypeInt(signed=False, label="IKEEXT_CERT_FLAGS")}, name="IKEEXT_CERT_ROOT_CONFIG0", pack=False, align=None), offset=0)}, name="<anon>", label="None"), "flags": SimTypeInt(signed=False, label="IKEEXT_CERT_AUTH")}, name="IKEEXT_CERTIFICATE_AUTHENTICATION0", pack=False, align=None), "cgaAuthentication": SimStruct({"keyContainerName": SimTypePointer(SimTypeChar(label="Char"), offset=0), "cspName": SimTypePointer(SimTypeChar(label="Char"), offset=0), "cspType": SimTypeInt(signed=False, label="UInt32"), "cgaModifier": SimStruct({"byteArray16": SimTypeFixedSizeArray(SimTypeChar(label="Byte"), 16)}, name="FWP_BYTE_ARRAY16", pack=False, align=None), "cgaCollisionCount": SimTypeChar(label="Byte")}, name="IKEEXT_IPV6_CGA_AUTHENTICATION0", pack=False, align=None)}, name="<anon>", label="None")}, name="IKEEXT_AUTHENTICATION_METHOD0", pack=False, align=None), offset=0), "initiatorImpersonationType": SimTypeInt(signed=False, label="IKEEXT_AUTHENTICATION_IMPERSONATION_TYPE"), "numIkeProposals": SimTypeInt(signed=False, label="UInt32"), "ikeProposals": SimTypePointer(SimStruct({"cipherAlgorithm": SimStruct({"algoIdentifier": SimTypeInt(signed=False, label="IKEEXT_CIPHER_TYPE"), "keyLen": SimTypeInt(signed=False, label="UInt32"), "rounds": SimTypeInt(signed=False, label="UInt32")}, name="IKEEXT_CIPHER_ALGORITHM0", pack=False, align=None), "integrityAlgorithm": SimStruct({"algoIdentifier": SimTypeInt(signed=False, label="IKEEXT_INTEGRITY_TYPE")}, name="IKEEXT_INTEGRITY_ALGORITHM0", pack=False, align=None), "maxLifetimeSeconds": SimTypeInt(signed=False, label="UInt32"), "dhGroup": SimTypeInt(signed=False, label="IKEEXT_DH_GROUP"), "quickModeLimit": SimTypeInt(signed=False, label="UInt32")}, name="IKEEXT_PROPOSAL0", pack=False, align=None), offset=0), "flags": SimTypeInt(signed=False, label="IKEEXT_POLICY_FLAG"), "maxDynamicFilters": SimTypeInt(signed=False, label="UInt32")}, name="IKEEXT_POLICY0", pack=False, align=None), offset=0), "dataBuffer": SimTypePointer(SimStruct({"size": SimTypeInt(signed=False, label="UInt32"), "data": SimTypePointer(SimTypeChar(label="Byte"), offset=0)}, name="FWP_BYTE_BLOB", pack=False, align=None), offset=0), "classifyOptions": SimTypePointer(SimStruct({"numOptions": SimTypeInt(signed=False, label="UInt32"), "options": SimTypePointer(SimStruct({"type": SimTypeInt(signed=False, label="FWP_CLASSIFY_OPTION_TYPE"), "value": SimStruct({"type": SimTypeInt(signed=False, label="FWP_DATA_TYPE"), "Anonymous": SimUnion({"uint8": SimTypeChar(label="Byte"), "uint16": SimTypeShort(signed=False, label="UInt16"), "uint32": SimTypeInt(signed=False, label="UInt32"), "uint64": SimTypePointer(SimTypeLongLong(signed=False, label="UInt64"), offset=0), "int8": SimTypeChar(label="SByte"), "int16": SimTypeShort(signed=True, label="Int16"), "int32": SimTypeInt(signed=True, label="Int32"), "int64": SimTypePointer(SimTypeLongLong(signed=True, label="Int64"), offset=0), "float32": SimTypeFloat(size=32), "double64": SimTypePointer(SimTypeFloat(size=64), offset=0), "byteArray16": SimTypePointer(SimStruct({"byteArray16": SimTypeFixedSizeArray(SimTypeChar(label="Byte"), 16)}, name="FWP_BYTE_ARRAY16", pack=False, align=None), offset=0), "byteBlob": SimTypePointer(SimStruct({"size": SimTypeInt(signed=False, label="UInt32"), "data": SimTypePointer(SimTypeChar(label="Byte"), offset=0)}, name="FWP_BYTE_BLOB", pack=False, align=None), offset=0), "sid": SimTypePointer(SimTypeBottom(label="SID"), offset=0), "sd": SimTypePointer(SimStruct({"size": SimTypeInt(signed=False, label="UInt32"), "data": SimTypePointer(SimTypeChar(label="Byte"), offset=0)}, name="FWP_BYTE_BLOB", pack=False, align=None), offset=0), "tokenInformation": SimTypePointer(SimStruct({"sidCount": SimTypeInt(signed=False, label="UInt32"), "sids": SimTypePointer(SimTypeBottom(label="SID_AND_ATTRIBUTES"), offset=0), "restrictedSidCount": SimTypeInt(signed=False, label="UInt32"), "restrictedSids": SimTypePointer(SimTypeBottom(label="SID_AND_ATTRIBUTES"), offset=0)}, name="FWP_TOKEN_INFORMATION", pack=False, align=None), offset=0), "tokenAccessInformation": SimTypePointer(SimStruct({"size": SimTypeInt(signed=False, label="UInt32"), "data": SimTypePointer(SimTypeChar(label="Byte"), offset=0)}, name="FWP_BYTE_BLOB", pack=False, align=None), offset=0), "unicodeString": SimTypePointer(SimTypeChar(label="Char"), offset=0), "byteArray6": SimTypePointer(SimStruct({"byteArray6": SimTypeFixedSizeArray(SimTypeChar(label="Byte"), 6)}, name="FWP_BYTE_ARRAY6", pack=False, align=None), offset=0), "bitmapArray64": SimTypePointer(SimStruct({"bitmapArray64": SimTypeFixedSizeArray(SimTypeChar(label="Byte"), 8)}, name="FWP_BITMAP_ARRAY64_", pack=False, align=None), offset=0)}, name="<anon>", label="None")}, name="FWP_VALUE0", pack=False, align=None)}, name="FWPM_CLASSIFY_OPTION0", pack=False, align=None), offset=0)}, name="FWPM_CLASSIFY_OPTIONS0", pack=False, align=None), offset=0)}, name="<anon>", label="None"), "providerContextId": SimTypeLongLong(signed=False, label="UInt64")}, name="FWPM_PROVIDER_CONTEXT0", pack=False, align=None), offset=0), offset=0)], SimTypeInt(signed=False, label="UInt32"), arg_names=["engineHandle", "id", "providerContext"]),
        # 
        'FwpmProviderContextGetById1': SimTypeFunction([SimTypePointer(SimTypeInt(signed=True, label="Int"), label="IntPtr", offset=0), SimTypeLongLong(signed=False, label="UInt64"), SimTypePointer(SimTypePointer(SimStruct({"providerContextKey": SimTypeBottom(label="Guid"), "displayData": SimStruct({"name": SimTypePointer(SimTypeChar(label="Char"), offset=0), "description": SimTypePointer(SimTypeChar(label="Char"), offset=0)}, name="FWPM_DISPLAY_DATA0", pack=False, align=None), "flags": SimTypeInt(signed=False, label="UInt32"), "providerKey": SimTypePointer(SimTypeBottom(label="Guid"), offset=0), "providerData": SimStruct({"size": SimTypeInt(signed=False, label="UInt32"), "data": SimTypePointer(SimTypeChar(label="Byte"), offset=0)}, name="FWP_BYTE_BLOB", pack=False, align=None), "type": SimTypeInt(signed=False, label="FWPM_PROVIDER_CONTEXT_TYPE"), "Anonymous": SimUnion({"keyingPolicy": SimTypePointer(SimStruct({"numKeyMods": SimTypeInt(signed=False, label="UInt32"), "keyModKeys": SimTypePointer(SimTypeBottom(label="Guid"), offset=0)}, name="IPSEC_KEYING_POLICY0", pack=False, align=None), offset=0), "ikeQmTransportPolicy": SimTypePointer(SimStruct({"numIpsecProposals": SimTypeInt(signed=False, label="UInt32"), "ipsecProposals": SimTypePointer(SimStruct({"lifetime": SimStruct({"lifetimeSeconds": SimTypeInt(signed=False, label="UInt32"), "lifetimeKilobytes": SimTypeInt(signed=False, label="UInt32"), "lifetimePackets": SimTypeInt(signed=False, label="UInt32")}, name="IPSEC_SA_LIFETIME0", pack=False, align=None), "numSaTransforms": SimTypeInt(signed=False, label="UInt32"), "saTransforms": SimTypePointer(SimStruct({"ipsecTransformType": SimTypeInt(signed=False, label="IPSEC_TRANSFORM_TYPE"), "Anonymous": SimUnion({"ahTransform": SimTypePointer(SimStruct({"authTransformId": SimStruct({"authType": SimTypeInt(signed=False, label="IPSEC_AUTH_TYPE"), "authConfig": SimTypeChar(label="Byte")}, name="IPSEC_AUTH_TRANSFORM_ID0", pack=False, align=None), "cryptoModuleId": SimTypePointer(SimTypeBottom(label="Guid"), offset=0)}, name="IPSEC_AUTH_TRANSFORM0", pack=False, align=None), offset=0), "espAuthTransform": SimTypePointer(SimStruct({"authTransformId": SimStruct({"authType": SimTypeInt(signed=False, label="IPSEC_AUTH_TYPE"), "authConfig": SimTypeChar(label="Byte")}, name="IPSEC_AUTH_TRANSFORM_ID0", pack=False, align=None), "cryptoModuleId": SimTypePointer(SimTypeBottom(label="Guid"), offset=0)}, name="IPSEC_AUTH_TRANSFORM0", pack=False, align=None), offset=0), "espCipherTransform": SimTypePointer(SimStruct({"cipherTransformId": SimStruct({"cipherType": SimTypeInt(signed=False, label="IPSEC_CIPHER_TYPE"), "cipherConfig": SimTypeChar(label="Byte")}, name="IPSEC_CIPHER_TRANSFORM_ID0", pack=False, align=None), "cryptoModuleId": SimTypePointer(SimTypeBottom(label="Guid"), offset=0)}, name="IPSEC_CIPHER_TRANSFORM0", pack=False, align=None), offset=0), "espAuthAndCipherTransform": SimTypePointer(SimStruct({"authTransform": SimStruct({"authTransformId": SimStruct({"authType": SimTypeInt(signed=False, label="IPSEC_AUTH_TYPE"), "authConfig": SimTypeChar(label="Byte")}, name="IPSEC_AUTH_TRANSFORM_ID0", pack=False, align=None), "cryptoModuleId": SimTypePointer(SimTypeBottom(label="Guid"), offset=0)}, name="IPSEC_AUTH_TRANSFORM0", pack=False, align=None), "cipherTransform": SimStruct({"cipherTransformId": SimStruct({"cipherType": SimTypeInt(signed=False, label="IPSEC_CIPHER_TYPE"), "cipherConfig": SimTypeChar(label="Byte")}, name="IPSEC_CIPHER_TRANSFORM_ID0", pack=False, align=None), "cryptoModuleId": SimTypePointer(SimTypeBottom(label="Guid"), offset=0)}, name="IPSEC_CIPHER_TRANSFORM0", pack=False, align=None)}, name="IPSEC_AUTH_AND_CIPHER_TRANSFORM0", pack=False, align=None), offset=0), "espAuthFwTransform": SimTypePointer(SimStruct({"authTransformId": SimStruct({"authType": SimTypeInt(signed=False, label="IPSEC_AUTH_TYPE"), "authConfig": SimTypeChar(label="Byte")}, name="IPSEC_AUTH_TRANSFORM_ID0", pack=False, align=None), "cryptoModuleId": SimTypePointer(SimTypeBottom(label="Guid"), offset=0)}, name="IPSEC_AUTH_TRANSFORM0", pack=False, align=None), offset=0)}, name="<anon>", label="None")}, name="IPSEC_SA_TRANSFORM0", pack=False, align=None), offset=0), "pfsGroup": SimTypeInt(signed=False, label="IPSEC_PFS_GROUP")}, name="IPSEC_PROPOSAL0", pack=False, align=None), offset=0), "flags": SimTypeInt(signed=False, label="IPSEC_POLICY_FLAG"), "ndAllowClearTimeoutSeconds": SimTypeInt(signed=False, label="UInt32"), "saIdleTimeout": SimStruct({"idleTimeoutSeconds": SimTypeInt(signed=False, label="UInt32"), "idleTimeoutSecondsFailOver": SimTypeInt(signed=False, label="UInt32")}, name="IPSEC_SA_IDLE_TIMEOUT0", pack=False, align=None), "emPolicy": SimTypePointer(SimStruct({"numAuthenticationMethods": SimTypeInt(signed=False, label="UInt32"), "authenticationMethods": SimTypePointer(SimStruct({"authenticationMethodType": SimTypeInt(signed=False, label="IKEEXT_AUTHENTICATION_METHOD_TYPE"), "Anonymous": SimUnion({"presharedKeyAuthentication": SimStruct({"presharedKey": SimStruct({"size": SimTypeInt(signed=False, label="UInt32"), "data": SimTypePointer(SimTypeChar(label="Byte"), offset=0)}, name="FWP_BYTE_BLOB", pack=False, align=None), "flags": SimTypeInt(signed=False, label="IKEEXT_PRESHARED_KEY_AUTHENTICATION_FLAGS")}, name="IKEEXT_PRESHARED_KEY_AUTHENTICATION1", pack=False, align=None), "certificateAuthentication": SimStruct({"inboundConfigType": SimTypeInt(signed=False, label="IKEEXT_CERT_CONFIG_TYPE"), "Anonymous1": SimUnion({"Anonymous": SimStruct({"inboundRootArraySize": SimTypeInt(signed=False, label="UInt32"), "inboundRootArray": SimTypePointer(SimStruct({"certData": SimStruct({"size": SimTypeInt(signed=False, label="UInt32"), "data": SimTypePointer(SimTypeChar(label="Byte"), offset=0)}, name="FWP_BYTE_BLOB", pack=False, align=None), "flags": SimTypeInt(signed=False, label="IKEEXT_CERT_FLAGS")}, name="IKEEXT_CERT_ROOT_CONFIG0", pack=False, align=None), offset=0)}, name="_Anonymous_e__Struct", pack=False, align=None), "inboundEnterpriseStoreConfig": SimTypePointer(SimStruct({"certData": SimStruct({"size": SimTypeInt(signed=False, label="UInt32"), "data": SimTypePointer(SimTypeChar(label="Byte"), offset=0)}, name="FWP_BYTE_BLOB", pack=False, align=None), "flags": SimTypeInt(signed=False, label="IKEEXT_CERT_FLAGS")}, name="IKEEXT_CERT_ROOT_CONFIG0", pack=False, align=None), offset=0), "inboundTrustedRootStoreConfig": SimTypePointer(SimStruct({"certData": SimStruct({"size": SimTypeInt(signed=False, label="UInt32"), "data": SimTypePointer(SimTypeChar(label="Byte"), offset=0)}, name="FWP_BYTE_BLOB", pack=False, align=None), "flags": SimTypeInt(signed=False, label="IKEEXT_CERT_FLAGS")}, name="IKEEXT_CERT_ROOT_CONFIG0", pack=False, align=None), offset=0)}, name="<anon>", label="None"), "outboundConfigType": SimTypeInt(signed=False, label="IKEEXT_CERT_CONFIG_TYPE"), "Anonymous2": SimUnion({"Anonymous": SimStruct({"outboundRootArraySize": SimTypeInt(signed=False, label="UInt32"), "outboundRootArray": SimTypePointer(SimStruct({"certData": SimStruct({"size": SimTypeInt(signed=False, label="UInt32"), "data": SimTypePointer(SimTypeChar(label="Byte"), offset=0)}, name="FWP_BYTE_BLOB", pack=False, align=None), "flags": SimTypeInt(signed=False, label="IKEEXT_CERT_FLAGS")}, name="IKEEXT_CERT_ROOT_CONFIG0", pack=False, align=None), offset=0)}, name="_Anonymous_e__Struct", pack=False, align=None), "outboundEnterpriseStoreConfig": SimTypePointer(SimStruct({"certData": SimStruct({"size": SimTypeInt(signed=False, label="UInt32"), "data": SimTypePointer(SimTypeChar(label="Byte"), offset=0)}, name="FWP_BYTE_BLOB", pack=False, align=None), "flags": SimTypeInt(signed=False, label="IKEEXT_CERT_FLAGS")}, name="IKEEXT_CERT_ROOT_CONFIG0", pack=False, align=None), offset=0), "outboundTrustedRootStoreConfig": SimTypePointer(SimStruct({"certData": SimStruct({"size": SimTypeInt(signed=False, label="UInt32"), "data": SimTypePointer(SimTypeChar(label="Byte"), offset=0)}, name="FWP_BYTE_BLOB", pack=False, align=None), "flags": SimTypeInt(signed=False, label="IKEEXT_CERT_FLAGS")}, name="IKEEXT_CERT_ROOT_CONFIG0", pack=False, align=None), offset=0)}, name="<anon>", label="None"), "flags": SimTypeInt(signed=False, label="IKEEXT_CERT_AUTH"), "localCertLocationUrl": SimStruct({"size": SimTypeInt(signed=False, label="UInt32"), "data": SimTypePointer(SimTypeChar(label="Byte"), offset=0)}, name="FWP_BYTE_BLOB", pack=False, align=None)}, name="IKEEXT_CERTIFICATE_AUTHENTICATION1", pack=False, align=None), "kerberosAuthentication": SimStruct({"flags": SimTypeInt(signed=False, label="IKEEXT_KERBEROS_AUTHENTICATION_FLAGS")}, name="IKEEXT_KERBEROS_AUTHENTICATION0", pack=False, align=None), "ntlmV2Authentication": SimStruct({"flags": SimTypeInt(signed=False, label="UInt32")}, name="IKEEXT_NTLM_V2_AUTHENTICATION0", pack=False, align=None), "sslAuthentication": SimStruct({"inboundConfigType": SimTypeInt(signed=False, label="IKEEXT_CERT_CONFIG_TYPE"), "Anonymous1": SimUnion({"Anonymous": SimStruct({"inboundRootArraySize": SimTypeInt(signed=False, label="UInt32"), "inboundRootArray": SimTypePointer(SimStruct({"certData": SimStruct({"size": SimTypeInt(signed=False, label="UInt32"), "data": SimTypePointer(SimTypeChar(label="Byte"), offset=0)}, name="FWP_BYTE_BLOB", pack=False, align=None), "flags": SimTypeInt(signed=False, label="IKEEXT_CERT_FLAGS")}, name="IKEEXT_CERT_ROOT_CONFIG0", pack=False, align=None), offset=0)}, name="_Anonymous_e__Struct", pack=False, align=None), "inboundEnterpriseStoreConfig": SimTypePointer(SimStruct({"certData": SimStruct({"size": SimTypeInt(signed=False, label="UInt32"), "data": SimTypePointer(SimTypeChar(label="Byte"), offset=0)}, name="FWP_BYTE_BLOB", pack=False, align=None), "flags": SimTypeInt(signed=False, label="IKEEXT_CERT_FLAGS")}, name="IKEEXT_CERT_ROOT_CONFIG0", pack=False, align=None), offset=0), "inboundTrustedRootStoreConfig": SimTypePointer(SimStruct({"certData": SimStruct({"size": SimTypeInt(signed=False, label="UInt32"), "data": SimTypePointer(SimTypeChar(label="Byte"), offset=0)}, name="FWP_BYTE_BLOB", pack=False, align=None), "flags": SimTypeInt(signed=False, label="IKEEXT_CERT_FLAGS")}, name="IKEEXT_CERT_ROOT_CONFIG0", pack=False, align=None), offset=0)}, name="<anon>", label="None"), "outboundConfigType": SimTypeInt(signed=False, label="IKEEXT_CERT_CONFIG_TYPE"), "Anonymous2": SimUnion({"Anonymous": SimStruct({"outboundRootArraySize": SimTypeInt(signed=False, label="UInt32"), "outboundRootArray": SimTypePointer(SimStruct({"certData": SimStruct({"size": SimTypeInt(signed=False, label="UInt32"), "data": SimTypePointer(SimTypeChar(label="Byte"), offset=0)}, name="FWP_BYTE_BLOB", pack=False, align=None), "flags": SimTypeInt(signed=False, label="IKEEXT_CERT_FLAGS")}, name="IKEEXT_CERT_ROOT_CONFIG0", pack=False, align=None), offset=0)}, name="_Anonymous_e__Struct", pack=False, align=None), "outboundEnterpriseStoreConfig": SimTypePointer(SimStruct({"certData": SimStruct({"size": SimTypeInt(signed=False, label="UInt32"), "data": SimTypePointer(SimTypeChar(label="Byte"), offset=0)}, name="FWP_BYTE_BLOB", pack=False, align=None), "flags": SimTypeInt(signed=False, label="IKEEXT_CERT_FLAGS")}, name="IKEEXT_CERT_ROOT_CONFIG0", pack=False, align=None), offset=0), "outboundTrustedRootStoreConfig": SimTypePointer(SimStruct({"certData": SimStruct({"size": SimTypeInt(signed=False, label="UInt32"), "data": SimTypePointer(SimTypeChar(label="Byte"), offset=0)}, name="FWP_BYTE_BLOB", pack=False, align=None), "flags": SimTypeInt(signed=False, label="IKEEXT_CERT_FLAGS")}, name="IKEEXT_CERT_ROOT_CONFIG0", pack=False, align=None), offset=0)}, name="<anon>", label="None"), "flags": SimTypeInt(signed=False, label="IKEEXT_CERT_AUTH"), "localCertLocationUrl": SimStruct({"size": SimTypeInt(signed=False, label="UInt32"), "data": SimTypePointer(SimTypeChar(label="Byte"), offset=0)}, name="FWP_BYTE_BLOB", pack=False, align=None)}, name="IKEEXT_CERTIFICATE_AUTHENTICATION1", pack=False, align=None), "cgaAuthentication": SimStruct({"keyContainerName": SimTypePointer(SimTypeChar(label="Char"), offset=0), "cspName": SimTypePointer(SimTypeChar(label="Char"), offset=0), "cspType": SimTypeInt(signed=False, label="UInt32"), "cgaModifier": SimStruct({"byteArray16": SimTypeFixedSizeArray(SimTypeChar(label="Byte"), 16)}, name="FWP_BYTE_ARRAY16", pack=False, align=None), "cgaCollisionCount": SimTypeChar(label="Byte")}, name="IKEEXT_IPV6_CGA_AUTHENTICATION0", pack=False, align=None), "eapAuthentication": SimStruct({"flags": SimTypeInt(signed=False, label="IKEEXT_EAP_AUTHENTICATION_FLAGS")}, name="IKEEXT_EAP_AUTHENTICATION0", pack=False, align=None)}, name="<anon>", label="None")}, name="IKEEXT_AUTHENTICATION_METHOD1", pack=False, align=None), offset=0), "initiatorImpersonationType": SimTypeInt(signed=False, label="IKEEXT_AUTHENTICATION_IMPERSONATION_TYPE")}, name="IKEEXT_EM_POLICY1", pack=False, align=None), offset=0)}, name="IPSEC_TRANSPORT_POLICY1", pack=False, align=None), offset=0), "ikeQmTunnelPolicy": SimTypePointer(SimStruct({"flags": SimTypeInt(signed=False, label="IPSEC_POLICY_FLAG"), "numIpsecProposals": SimTypeInt(signed=False, label="UInt32"), "ipsecProposals": SimTypePointer(SimStruct({"lifetime": SimStruct({"lifetimeSeconds": SimTypeInt(signed=False, label="UInt32"), "lifetimeKilobytes": SimTypeInt(signed=False, label="UInt32"), "lifetimePackets": SimTypeInt(signed=False, label="UInt32")}, name="IPSEC_SA_LIFETIME0", pack=False, align=None), "numSaTransforms": SimTypeInt(signed=False, label="UInt32"), "saTransforms": SimTypePointer(SimStruct({"ipsecTransformType": SimTypeInt(signed=False, label="IPSEC_TRANSFORM_TYPE"), "Anonymous": SimUnion({"ahTransform": SimTypePointer(SimStruct({"authTransformId": SimStruct({"authType": SimTypeInt(signed=False, label="IPSEC_AUTH_TYPE"), "authConfig": SimTypeChar(label="Byte")}, name="IPSEC_AUTH_TRANSFORM_ID0", pack=False, align=None), "cryptoModuleId": SimTypePointer(SimTypeBottom(label="Guid"), offset=0)}, name="IPSEC_AUTH_TRANSFORM0", pack=False, align=None), offset=0), "espAuthTransform": SimTypePointer(SimStruct({"authTransformId": SimStruct({"authType": SimTypeInt(signed=False, label="IPSEC_AUTH_TYPE"), "authConfig": SimTypeChar(label="Byte")}, name="IPSEC_AUTH_TRANSFORM_ID0", pack=False, align=None), "cryptoModuleId": SimTypePointer(SimTypeBottom(label="Guid"), offset=0)}, name="IPSEC_AUTH_TRANSFORM0", pack=False, align=None), offset=0), "espCipherTransform": SimTypePointer(SimStruct({"cipherTransformId": SimStruct({"cipherType": SimTypeInt(signed=False, label="IPSEC_CIPHER_TYPE"), "cipherConfig": SimTypeChar(label="Byte")}, name="IPSEC_CIPHER_TRANSFORM_ID0", pack=False, align=None), "cryptoModuleId": SimTypePointer(SimTypeBottom(label="Guid"), offset=0)}, name="IPSEC_CIPHER_TRANSFORM0", pack=False, align=None), offset=0), "espAuthAndCipherTransform": SimTypePointer(SimStruct({"authTransform": SimStruct({"authTransformId": SimStruct({"authType": SimTypeInt(signed=False, label="IPSEC_AUTH_TYPE"), "authConfig": SimTypeChar(label="Byte")}, name="IPSEC_AUTH_TRANSFORM_ID0", pack=False, align=None), "cryptoModuleId": SimTypePointer(SimTypeBottom(label="Guid"), offset=0)}, name="IPSEC_AUTH_TRANSFORM0", pack=False, align=None), "cipherTransform": SimStruct({"cipherTransformId": SimStruct({"cipherType": SimTypeInt(signed=False, label="IPSEC_CIPHER_TYPE"), "cipherConfig": SimTypeChar(label="Byte")}, name="IPSEC_CIPHER_TRANSFORM_ID0", pack=False, align=None), "cryptoModuleId": SimTypePointer(SimTypeBottom(label="Guid"), offset=0)}, name="IPSEC_CIPHER_TRANSFORM0", pack=False, align=None)}, name="IPSEC_AUTH_AND_CIPHER_TRANSFORM0", pack=False, align=None), offset=0), "espAuthFwTransform": SimTypePointer(SimStruct({"authTransformId": SimStruct({"authType": SimTypeInt(signed=False, label="IPSEC_AUTH_TYPE"), "authConfig": SimTypeChar(label="Byte")}, name="IPSEC_AUTH_TRANSFORM_ID0", pack=False, align=None), "cryptoModuleId": SimTypePointer(SimTypeBottom(label="Guid"), offset=0)}, name="IPSEC_AUTH_TRANSFORM0", pack=False, align=None), offset=0)}, name="<anon>", label="None")}, name="IPSEC_SA_TRANSFORM0", pack=False, align=None), offset=0), "pfsGroup": SimTypeInt(signed=False, label="IPSEC_PFS_GROUP")}, name="IPSEC_PROPOSAL0", pack=False, align=None), offset=0), "tunnelEndpoints": SimStruct({"ipVersion": SimTypeInt(signed=False, label="FWP_IP_VERSION"), "Anonymous1": SimUnion({"localV4Address": SimTypeInt(signed=False, label="UInt32"), "localV6Address": SimTypeFixedSizeArray(SimTypeChar(label="Byte"), 16)}, name="<anon>", label="None"), "Anonymous2": SimUnion({"remoteV4Address": SimTypeInt(signed=False, label="UInt32"), "remoteV6Address": SimTypeFixedSizeArray(SimTypeChar(label="Byte"), 16)}, name="<anon>", label="None"), "localIfLuid": SimTypeLongLong(signed=False, label="UInt64")}, name="IPSEC_TUNNEL_ENDPOINTS1", pack=False, align=None), "saIdleTimeout": SimStruct({"idleTimeoutSeconds": SimTypeInt(signed=False, label="UInt32"), "idleTimeoutSecondsFailOver": SimTypeInt(signed=False, label="UInt32")}, name="IPSEC_SA_IDLE_TIMEOUT0", pack=False, align=None), "emPolicy": SimTypePointer(SimStruct({"numAuthenticationMethods": SimTypeInt(signed=False, label="UInt32"), "authenticationMethods": SimTypePointer(SimStruct({"authenticationMethodType": SimTypeInt(signed=False, label="IKEEXT_AUTHENTICATION_METHOD_TYPE"), "Anonymous": SimUnion({"presharedKeyAuthentication": SimStruct({"presharedKey": SimStruct({"size": SimTypeInt(signed=False, label="UInt32"), "data": SimTypePointer(SimTypeChar(label="Byte"), offset=0)}, name="FWP_BYTE_BLOB", pack=False, align=None), "flags": SimTypeInt(signed=False, label="IKEEXT_PRESHARED_KEY_AUTHENTICATION_FLAGS")}, name="IKEEXT_PRESHARED_KEY_AUTHENTICATION1", pack=False, align=None), "certificateAuthentication": SimStruct({"inboundConfigType": SimTypeInt(signed=False, label="IKEEXT_CERT_CONFIG_TYPE"), "Anonymous1": SimUnion({"Anonymous": SimStruct({"inboundRootArraySize": SimTypeInt(signed=False, label="UInt32"), "inboundRootArray": SimTypePointer(SimStruct({"certData": SimStruct({"size": SimTypeInt(signed=False, label="UInt32"), "data": SimTypePointer(SimTypeChar(label="Byte"), offset=0)}, name="FWP_BYTE_BLOB", pack=False, align=None), "flags": SimTypeInt(signed=False, label="IKEEXT_CERT_FLAGS")}, name="IKEEXT_CERT_ROOT_CONFIG0", pack=False, align=None), offset=0)}, name="_Anonymous_e__Struct", pack=False, align=None), "inboundEnterpriseStoreConfig": SimTypePointer(SimStruct({"certData": SimStruct({"size": SimTypeInt(signed=False, label="UInt32"), "data": SimTypePointer(SimTypeChar(label="Byte"), offset=0)}, name="FWP_BYTE_BLOB", pack=False, align=None), "flags": SimTypeInt(signed=False, label="IKEEXT_CERT_FLAGS")}, name="IKEEXT_CERT_ROOT_CONFIG0", pack=False, align=None), offset=0), "inboundTrustedRootStoreConfig": SimTypePointer(SimStruct({"certData": SimStruct({"size": SimTypeInt(signed=False, label="UInt32"), "data": SimTypePointer(SimTypeChar(label="Byte"), offset=0)}, name="FWP_BYTE_BLOB", pack=False, align=None), "flags": SimTypeInt(signed=False, label="IKEEXT_CERT_FLAGS")}, name="IKEEXT_CERT_ROOT_CONFIG0", pack=False, align=None), offset=0)}, name="<anon>", label="None"), "outboundConfigType": SimTypeInt(signed=False, label="IKEEXT_CERT_CONFIG_TYPE"), "Anonymous2": SimUnion({"Anonymous": SimStruct({"outboundRootArraySize": SimTypeInt(signed=False, label="UInt32"), "outboundRootArray": SimTypePointer(SimStruct({"certData": SimStruct({"size": SimTypeInt(signed=False, label="UInt32"), "data": SimTypePointer(SimTypeChar(label="Byte"), offset=0)}, name="FWP_BYTE_BLOB", pack=False, align=None), "flags": SimTypeInt(signed=False, label="IKEEXT_CERT_FLAGS")}, name="IKEEXT_CERT_ROOT_CONFIG0", pack=False, align=None), offset=0)}, name="_Anonymous_e__Struct", pack=False, align=None), "outboundEnterpriseStoreConfig": SimTypePointer(SimStruct({"certData": SimStruct({"size": SimTypeInt(signed=False, label="UInt32"), "data": SimTypePointer(SimTypeChar(label="Byte"), offset=0)}, name="FWP_BYTE_BLOB", pack=False, align=None), "flags": SimTypeInt(signed=False, label="IKEEXT_CERT_FLAGS")}, name="IKEEXT_CERT_ROOT_CONFIG0", pack=False, align=None), offset=0), "outboundTrustedRootStoreConfig": SimTypePointer(SimStruct({"certData": SimStruct({"size": SimTypeInt(signed=False, label="UInt32"), "data": SimTypePointer(SimTypeChar(label="Byte"), offset=0)}, name="FWP_BYTE_BLOB", pack=False, align=None), "flags": SimTypeInt(signed=False, label="IKEEXT_CERT_FLAGS")}, name="IKEEXT_CERT_ROOT_CONFIG0", pack=False, align=None), offset=0)}, name="<anon>", label="None"), "flags": SimTypeInt(signed=False, label="IKEEXT_CERT_AUTH"), "localCertLocationUrl": SimStruct({"size": SimTypeInt(signed=False, label="UInt32"), "data": SimTypePointer(SimTypeChar(label="Byte"), offset=0)}, name="FWP_BYTE_BLOB", pack=False, align=None)}, name="IKEEXT_CERTIFICATE_AUTHENTICATION1", pack=False, align=None), "kerberosAuthentication": SimStruct({"flags": SimTypeInt(signed=False, label="IKEEXT_KERBEROS_AUTHENTICATION_FLAGS")}, name="IKEEXT_KERBEROS_AUTHENTICATION0", pack=False, align=None), "ntlmV2Authentication": SimStruct({"flags": SimTypeInt(signed=False, label="UInt32")}, name="IKEEXT_NTLM_V2_AUTHENTICATION0", pack=False, align=None), "sslAuthentication": SimStruct({"inboundConfigType": SimTypeInt(signed=False, label="IKEEXT_CERT_CONFIG_TYPE"), "Anonymous1": SimUnion({"Anonymous": SimStruct({"inboundRootArraySize": SimTypeInt(signed=False, label="UInt32"), "inboundRootArray": SimTypePointer(SimStruct({"certData": SimStruct({"size": SimTypeInt(signed=False, label="UInt32"), "data": SimTypePointer(SimTypeChar(label="Byte"), offset=0)}, name="FWP_BYTE_BLOB", pack=False, align=None), "flags": SimTypeInt(signed=False, label="IKEEXT_CERT_FLAGS")}, name="IKEEXT_CERT_ROOT_CONFIG0", pack=False, align=None), offset=0)}, name="_Anonymous_e__Struct", pack=False, align=None), "inboundEnterpriseStoreConfig": SimTypePointer(SimStruct({"certData": SimStruct({"size": SimTypeInt(signed=False, label="UInt32"), "data": SimTypePointer(SimTypeChar(label="Byte"), offset=0)}, name="FWP_BYTE_BLOB", pack=False, align=None), "flags": SimTypeInt(signed=False, label="IKEEXT_CERT_FLAGS")}, name="IKEEXT_CERT_ROOT_CONFIG0", pack=False, align=None), offset=0), "inboundTrustedRootStoreConfig": SimTypePointer(SimStruct({"certData": SimStruct({"size": SimTypeInt(signed=False, label="UInt32"), "data": SimTypePointer(SimTypeChar(label="Byte"), offset=0)}, name="FWP_BYTE_BLOB", pack=False, align=None), "flags": SimTypeInt(signed=False, label="IKEEXT_CERT_FLAGS")}, name="IKEEXT_CERT_ROOT_CONFIG0", pack=False, align=None), offset=0)}, name="<anon>", label="None"), "outboundConfigType": SimTypeInt(signed=False, label="IKEEXT_CERT_CONFIG_TYPE"), "Anonymous2": SimUnion({"Anonymous": SimStruct({"outboundRootArraySize": SimTypeInt(signed=False, label="UInt32"), "outboundRootArray": SimTypePointer(SimStruct({"certData": SimStruct({"size": SimTypeInt(signed=False, label="UInt32"), "data": SimTypePointer(SimTypeChar(label="Byte"), offset=0)}, name="FWP_BYTE_BLOB", pack=False, align=None), "flags": SimTypeInt(signed=False, label="IKEEXT_CERT_FLAGS")}, name="IKEEXT_CERT_ROOT_CONFIG0", pack=False, align=None), offset=0)}, name="_Anonymous_e__Struct", pack=False, align=None), "outboundEnterpriseStoreConfig": SimTypePointer(SimStruct({"certData": SimStruct({"size": SimTypeInt(signed=False, label="UInt32"), "data": SimTypePointer(SimTypeChar(label="Byte"), offset=0)}, name="FWP_BYTE_BLOB", pack=False, align=None), "flags": SimTypeInt(signed=False, label="IKEEXT_CERT_FLAGS")}, name="IKEEXT_CERT_ROOT_CONFIG0", pack=False, align=None), offset=0), "outboundTrustedRootStoreConfig": SimTypePointer(SimStruct({"certData": SimStruct({"size": SimTypeInt(signed=False, label="UInt32"), "data": SimTypePointer(SimTypeChar(label="Byte"), offset=0)}, name="FWP_BYTE_BLOB", pack=False, align=None), "flags": SimTypeInt(signed=False, label="IKEEXT_CERT_FLAGS")}, name="IKEEXT_CERT_ROOT_CONFIG0", pack=False, align=None), offset=0)}, name="<anon>", label="None"), "flags": SimTypeInt(signed=False, label="IKEEXT_CERT_AUTH"), "localCertLocationUrl": SimStruct({"size": SimTypeInt(signed=False, label="UInt32"), "data": SimTypePointer(SimTypeChar(label="Byte"), offset=0)}, name="FWP_BYTE_BLOB", pack=False, align=None)}, name="IKEEXT_CERTIFICATE_AUTHENTICATION1", pack=False, align=None), "cgaAuthentication": SimStruct({"keyContainerName": SimTypePointer(SimTypeChar(label="Char"), offset=0), "cspName": SimTypePointer(SimTypeChar(label="Char"), offset=0), "cspType": SimTypeInt(signed=False, label="UInt32"), "cgaModifier": SimStruct({"byteArray16": SimTypeFixedSizeArray(SimTypeChar(label="Byte"), 16)}, name="FWP_BYTE_ARRAY16", pack=False, align=None), "cgaCollisionCount": SimTypeChar(label="Byte")}, name="IKEEXT_IPV6_CGA_AUTHENTICATION0", pack=False, align=None), "eapAuthentication": SimStruct({"flags": SimTypeInt(signed=False, label="IKEEXT_EAP_AUTHENTICATION_FLAGS")}, name="IKEEXT_EAP_AUTHENTICATION0", pack=False, align=None)}, name="<anon>", label="None")}, name="IKEEXT_AUTHENTICATION_METHOD1", pack=False, align=None), offset=0), "initiatorImpersonationType": SimTypeInt(signed=False, label="IKEEXT_AUTHENTICATION_IMPERSONATION_TYPE")}, name="IKEEXT_EM_POLICY1", pack=False, align=None), offset=0)}, name="IPSEC_TUNNEL_POLICY1", pack=False, align=None), offset=0), "authipQmTransportPolicy": SimTypePointer(SimStruct({"numIpsecProposals": SimTypeInt(signed=False, label="UInt32"), "ipsecProposals": SimTypePointer(SimStruct({"lifetime": SimStruct({"lifetimeSeconds": SimTypeInt(signed=False, label="UInt32"), "lifetimeKilobytes": SimTypeInt(signed=False, label="UInt32"), "lifetimePackets": SimTypeInt(signed=False, label="UInt32")}, name="IPSEC_SA_LIFETIME0", pack=False, align=None), "numSaTransforms": SimTypeInt(signed=False, label="UInt32"), "saTransforms": SimTypePointer(SimStruct({"ipsecTransformType": SimTypeInt(signed=False, label="IPSEC_TRANSFORM_TYPE"), "Anonymous": SimUnion({"ahTransform": SimTypePointer(SimStruct({"authTransformId": SimStruct({"authType": SimTypeInt(signed=False, label="IPSEC_AUTH_TYPE"), "authConfig": SimTypeChar(label="Byte")}, name="IPSEC_AUTH_TRANSFORM_ID0", pack=False, align=None), "cryptoModuleId": SimTypePointer(SimTypeBottom(label="Guid"), offset=0)}, name="IPSEC_AUTH_TRANSFORM0", pack=False, align=None), offset=0), "espAuthTransform": SimTypePointer(SimStruct({"authTransformId": SimStruct({"authType": SimTypeInt(signed=False, label="IPSEC_AUTH_TYPE"), "authConfig": SimTypeChar(label="Byte")}, name="IPSEC_AUTH_TRANSFORM_ID0", pack=False, align=None), "cryptoModuleId": SimTypePointer(SimTypeBottom(label="Guid"), offset=0)}, name="IPSEC_AUTH_TRANSFORM0", pack=False, align=None), offset=0), "espCipherTransform": SimTypePointer(SimStruct({"cipherTransformId": SimStruct({"cipherType": SimTypeInt(signed=False, label="IPSEC_CIPHER_TYPE"), "cipherConfig": SimTypeChar(label="Byte")}, name="IPSEC_CIPHER_TRANSFORM_ID0", pack=False, align=None), "cryptoModuleId": SimTypePointer(SimTypeBottom(label="Guid"), offset=0)}, name="IPSEC_CIPHER_TRANSFORM0", pack=False, align=None), offset=0), "espAuthAndCipherTransform": SimTypePointer(SimStruct({"authTransform": SimStruct({"authTransformId": SimStruct({"authType": SimTypeInt(signed=False, label="IPSEC_AUTH_TYPE"), "authConfig": SimTypeChar(label="Byte")}, name="IPSEC_AUTH_TRANSFORM_ID0", pack=False, align=None), "cryptoModuleId": SimTypePointer(SimTypeBottom(label="Guid"), offset=0)}, name="IPSEC_AUTH_TRANSFORM0", pack=False, align=None), "cipherTransform": SimStruct({"cipherTransformId": SimStruct({"cipherType": SimTypeInt(signed=False, label="IPSEC_CIPHER_TYPE"), "cipherConfig": SimTypeChar(label="Byte")}, name="IPSEC_CIPHER_TRANSFORM_ID0", pack=False, align=None), "cryptoModuleId": SimTypePointer(SimTypeBottom(label="Guid"), offset=0)}, name="IPSEC_CIPHER_TRANSFORM0", pack=False, align=None)}, name="IPSEC_AUTH_AND_CIPHER_TRANSFORM0", pack=False, align=None), offset=0), "espAuthFwTransform": SimTypePointer(SimStruct({"authTransformId": SimStruct({"authType": SimTypeInt(signed=False, label="IPSEC_AUTH_TYPE"), "authConfig": SimTypeChar(label="Byte")}, name="IPSEC_AUTH_TRANSFORM_ID0", pack=False, align=None), "cryptoModuleId": SimTypePointer(SimTypeBottom(label="Guid"), offset=0)}, name="IPSEC_AUTH_TRANSFORM0", pack=False, align=None), offset=0)}, name="<anon>", label="None")}, name="IPSEC_SA_TRANSFORM0", pack=False, align=None), offset=0), "pfsGroup": SimTypeInt(signed=False, label="IPSEC_PFS_GROUP")}, name="IPSEC_PROPOSAL0", pack=False, align=None), offset=0), "flags": SimTypeInt(signed=False, label="IPSEC_POLICY_FLAG"), "ndAllowClearTimeoutSeconds": SimTypeInt(signed=False, label="UInt32"), "saIdleTimeout": SimStruct({"idleTimeoutSeconds": SimTypeInt(signed=False, label="UInt32"), "idleTimeoutSecondsFailOver": SimTypeInt(signed=False, label="UInt32")}, name="IPSEC_SA_IDLE_TIMEOUT0", pack=False, align=None), "emPolicy": SimTypePointer(SimStruct({"numAuthenticationMethods": SimTypeInt(signed=False, label="UInt32"), "authenticationMethods": SimTypePointer(SimStruct({"authenticationMethodType": SimTypeInt(signed=False, label="IKEEXT_AUTHENTICATION_METHOD_TYPE"), "Anonymous": SimUnion({"presharedKeyAuthentication": SimStruct({"presharedKey": SimStruct({"size": SimTypeInt(signed=False, label="UInt32"), "data": SimTypePointer(SimTypeChar(label="Byte"), offset=0)}, name="FWP_BYTE_BLOB", pack=False, align=None), "flags": SimTypeInt(signed=False, label="IKEEXT_PRESHARED_KEY_AUTHENTICATION_FLAGS")}, name="IKEEXT_PRESHARED_KEY_AUTHENTICATION1", pack=False, align=None), "certificateAuthentication": SimStruct({"inboundConfigType": SimTypeInt(signed=False, label="IKEEXT_CERT_CONFIG_TYPE"), "Anonymous1": SimUnion({"Anonymous": SimStruct({"inboundRootArraySize": SimTypeInt(signed=False, label="UInt32"), "inboundRootArray": SimTypePointer(SimStruct({"certData": SimStruct({"size": SimTypeInt(signed=False, label="UInt32"), "data": SimTypePointer(SimTypeChar(label="Byte"), offset=0)}, name="FWP_BYTE_BLOB", pack=False, align=None), "flags": SimTypeInt(signed=False, label="IKEEXT_CERT_FLAGS")}, name="IKEEXT_CERT_ROOT_CONFIG0", pack=False, align=None), offset=0)}, name="_Anonymous_e__Struct", pack=False, align=None), "inboundEnterpriseStoreConfig": SimTypePointer(SimStruct({"certData": SimStruct({"size": SimTypeInt(signed=False, label="UInt32"), "data": SimTypePointer(SimTypeChar(label="Byte"), offset=0)}, name="FWP_BYTE_BLOB", pack=False, align=None), "flags": SimTypeInt(signed=False, label="IKEEXT_CERT_FLAGS")}, name="IKEEXT_CERT_ROOT_CONFIG0", pack=False, align=None), offset=0), "inboundTrustedRootStoreConfig": SimTypePointer(SimStruct({"certData": SimStruct({"size": SimTypeInt(signed=False, label="UInt32"), "data": SimTypePointer(SimTypeChar(label="Byte"), offset=0)}, name="FWP_BYTE_BLOB", pack=False, align=None), "flags": SimTypeInt(signed=False, label="IKEEXT_CERT_FLAGS")}, name="IKEEXT_CERT_ROOT_CONFIG0", pack=False, align=None), offset=0)}, name="<anon>", label="None"), "outboundConfigType": SimTypeInt(signed=False, label="IKEEXT_CERT_CONFIG_TYPE"), "Anonymous2": SimUnion({"Anonymous": SimStruct({"outboundRootArraySize": SimTypeInt(signed=False, label="UInt32"), "outboundRootArray": SimTypePointer(SimStruct({"certData": SimStruct({"size": SimTypeInt(signed=False, label="UInt32"), "data": SimTypePointer(SimTypeChar(label="Byte"), offset=0)}, name="FWP_BYTE_BLOB", pack=False, align=None), "flags": SimTypeInt(signed=False, label="IKEEXT_CERT_FLAGS")}, name="IKEEXT_CERT_ROOT_CONFIG0", pack=False, align=None), offset=0)}, name="_Anonymous_e__Struct", pack=False, align=None), "outboundEnterpriseStoreConfig": SimTypePointer(SimStruct({"certData": SimStruct({"size": SimTypeInt(signed=False, label="UInt32"), "data": SimTypePointer(SimTypeChar(label="Byte"), offset=0)}, name="FWP_BYTE_BLOB", pack=False, align=None), "flags": SimTypeInt(signed=False, label="IKEEXT_CERT_FLAGS")}, name="IKEEXT_CERT_ROOT_CONFIG0", pack=False, align=None), offset=0), "outboundTrustedRootStoreConfig": SimTypePointer(SimStruct({"certData": SimStruct({"size": SimTypeInt(signed=False, label="UInt32"), "data": SimTypePointer(SimTypeChar(label="Byte"), offset=0)}, name="FWP_BYTE_BLOB", pack=False, align=None), "flags": SimTypeInt(signed=False, label="IKEEXT_CERT_FLAGS")}, name="IKEEXT_CERT_ROOT_CONFIG0", pack=False, align=None), offset=0)}, name="<anon>", label="None"), "flags": SimTypeInt(signed=False, label="IKEEXT_CERT_AUTH"), "localCertLocationUrl": SimStruct({"size": SimTypeInt(signed=False, label="UInt32"), "data": SimTypePointer(SimTypeChar(label="Byte"), offset=0)}, name="FWP_BYTE_BLOB", pack=False, align=None)}, name="IKEEXT_CERTIFICATE_AUTHENTICATION1", pack=False, align=None), "kerberosAuthentication": SimStruct({"flags": SimTypeInt(signed=False, label="IKEEXT_KERBEROS_AUTHENTICATION_FLAGS")}, name="IKEEXT_KERBEROS_AUTHENTICATION0", pack=False, align=None), "ntlmV2Authentication": SimStruct({"flags": SimTypeInt(signed=False, label="UInt32")}, name="IKEEXT_NTLM_V2_AUTHENTICATION0", pack=False, align=None), "sslAuthentication": SimStruct({"inboundConfigType": SimTypeInt(signed=False, label="IKEEXT_CERT_CONFIG_TYPE"), "Anonymous1": SimUnion({"Anonymous": SimStruct({"inboundRootArraySize": SimTypeInt(signed=False, label="UInt32"), "inboundRootArray": SimTypePointer(SimStruct({"certData": SimStruct({"size": SimTypeInt(signed=False, label="UInt32"), "data": SimTypePointer(SimTypeChar(label="Byte"), offset=0)}, name="FWP_BYTE_BLOB", pack=False, align=None), "flags": SimTypeInt(signed=False, label="IKEEXT_CERT_FLAGS")}, name="IKEEXT_CERT_ROOT_CONFIG0", pack=False, align=None), offset=0)}, name="_Anonymous_e__Struct", pack=False, align=None), "inboundEnterpriseStoreConfig": SimTypePointer(SimStruct({"certData": SimStruct({"size": SimTypeInt(signed=False, label="UInt32"), "data": SimTypePointer(SimTypeChar(label="Byte"), offset=0)}, name="FWP_BYTE_BLOB", pack=False, align=None), "flags": SimTypeInt(signed=False, label="IKEEXT_CERT_FLAGS")}, name="IKEEXT_CERT_ROOT_CONFIG0", pack=False, align=None), offset=0), "inboundTrustedRootStoreConfig": SimTypePointer(SimStruct({"certData": SimStruct({"size": SimTypeInt(signed=False, label="UInt32"), "data": SimTypePointer(SimTypeChar(label="Byte"), offset=0)}, name="FWP_BYTE_BLOB", pack=False, align=None), "flags": SimTypeInt(signed=False, label="IKEEXT_CERT_FLAGS")}, name="IKEEXT_CERT_ROOT_CONFIG0", pack=False, align=None), offset=0)}, name="<anon>", label="None"), "outboundConfigType": SimTypeInt(signed=False, label="IKEEXT_CERT_CONFIG_TYPE"), "Anonymous2": SimUnion({"Anonymous": SimStruct({"outboundRootArraySize": SimTypeInt(signed=False, label="UInt32"), "outboundRootArray": SimTypePointer(SimStruct({"certData": SimStruct({"size": SimTypeInt(signed=False, label="UInt32"), "data": SimTypePointer(SimTypeChar(label="Byte"), offset=0)}, name="FWP_BYTE_BLOB", pack=False, align=None), "flags": SimTypeInt(signed=False, label="IKEEXT_CERT_FLAGS")}, name="IKEEXT_CERT_ROOT_CONFIG0", pack=False, align=None), offset=0)}, name="_Anonymous_e__Struct", pack=False, align=None), "outboundEnterpriseStoreConfig": SimTypePointer(SimStruct({"certData": SimStruct({"size": SimTypeInt(signed=False, label="UInt32"), "data": SimTypePointer(SimTypeChar(label="Byte"), offset=0)}, name="FWP_BYTE_BLOB", pack=False, align=None), "flags": SimTypeInt(signed=False, label="IKEEXT_CERT_FLAGS")}, name="IKEEXT_CERT_ROOT_CONFIG0", pack=False, align=None), offset=0), "outboundTrustedRootStoreConfig": SimTypePointer(SimStruct({"certData": SimStruct({"size": SimTypeInt(signed=False, label="UInt32"), "data": SimTypePointer(SimTypeChar(label="Byte"), offset=0)}, name="FWP_BYTE_BLOB", pack=False, align=None), "flags": SimTypeInt(signed=False, label="IKEEXT_CERT_FLAGS")}, name="IKEEXT_CERT_ROOT_CONFIG0", pack=False, align=None), offset=0)}, name="<anon>", label="None"), "flags": SimTypeInt(signed=False, label="IKEEXT_CERT_AUTH"), "localCertLocationUrl": SimStruct({"size": SimTypeInt(signed=False, label="UInt32"), "data": SimTypePointer(SimTypeChar(label="Byte"), offset=0)}, name="FWP_BYTE_BLOB", pack=False, align=None)}, name="IKEEXT_CERTIFICATE_AUTHENTICATION1", pack=False, align=None), "cgaAuthentication": SimStruct({"keyContainerName": SimTypePointer(SimTypeChar(label="Char"), offset=0), "cspName": SimTypePointer(SimTypeChar(label="Char"), offset=0), "cspType": SimTypeInt(signed=False, label="UInt32"), "cgaModifier": SimStruct({"byteArray16": SimTypeFixedSizeArray(SimTypeChar(label="Byte"), 16)}, name="FWP_BYTE_ARRAY16", pack=False, align=None), "cgaCollisionCount": SimTypeChar(label="Byte")}, name="IKEEXT_IPV6_CGA_AUTHENTICATION0", pack=False, align=None), "eapAuthentication": SimStruct({"flags": SimTypeInt(signed=False, label="IKEEXT_EAP_AUTHENTICATION_FLAGS")}, name="IKEEXT_EAP_AUTHENTICATION0", pack=False, align=None)}, name="<anon>", label="None")}, name="IKEEXT_AUTHENTICATION_METHOD1", pack=False, align=None), offset=0), "initiatorImpersonationType": SimTypeInt(signed=False, label="IKEEXT_AUTHENTICATION_IMPERSONATION_TYPE")}, name="IKEEXT_EM_POLICY1", pack=False, align=None), offset=0)}, name="IPSEC_TRANSPORT_POLICY1", pack=False, align=None), offset=0), "authipQmTunnelPolicy": SimTypePointer(SimStruct({"flags": SimTypeInt(signed=False, label="IPSEC_POLICY_FLAG"), "numIpsecProposals": SimTypeInt(signed=False, label="UInt32"), "ipsecProposals": SimTypePointer(SimStruct({"lifetime": SimStruct({"lifetimeSeconds": SimTypeInt(signed=False, label="UInt32"), "lifetimeKilobytes": SimTypeInt(signed=False, label="UInt32"), "lifetimePackets": SimTypeInt(signed=False, label="UInt32")}, name="IPSEC_SA_LIFETIME0", pack=False, align=None), "numSaTransforms": SimTypeInt(signed=False, label="UInt32"), "saTransforms": SimTypePointer(SimStruct({"ipsecTransformType": SimTypeInt(signed=False, label="IPSEC_TRANSFORM_TYPE"), "Anonymous": SimUnion({"ahTransform": SimTypePointer(SimStruct({"authTransformId": SimStruct({"authType": SimTypeInt(signed=False, label="IPSEC_AUTH_TYPE"), "authConfig": SimTypeChar(label="Byte")}, name="IPSEC_AUTH_TRANSFORM_ID0", pack=False, align=None), "cryptoModuleId": SimTypePointer(SimTypeBottom(label="Guid"), offset=0)}, name="IPSEC_AUTH_TRANSFORM0", pack=False, align=None), offset=0), "espAuthTransform": SimTypePointer(SimStruct({"authTransformId": SimStruct({"authType": SimTypeInt(signed=False, label="IPSEC_AUTH_TYPE"), "authConfig": SimTypeChar(label="Byte")}, name="IPSEC_AUTH_TRANSFORM_ID0", pack=False, align=None), "cryptoModuleId": SimTypePointer(SimTypeBottom(label="Guid"), offset=0)}, name="IPSEC_AUTH_TRANSFORM0", pack=False, align=None), offset=0), "espCipherTransform": SimTypePointer(SimStruct({"cipherTransformId": SimStruct({"cipherType": SimTypeInt(signed=False, label="IPSEC_CIPHER_TYPE"), "cipherConfig": SimTypeChar(label="Byte")}, name="IPSEC_CIPHER_TRANSFORM_ID0", pack=False, align=None), "cryptoModuleId": SimTypePointer(SimTypeBottom(label="Guid"), offset=0)}, name="IPSEC_CIPHER_TRANSFORM0", pack=False, align=None), offset=0), "espAuthAndCipherTransform": SimTypePointer(SimStruct({"authTransform": SimStruct({"authTransformId": SimStruct({"authType": SimTypeInt(signed=False, label="IPSEC_AUTH_TYPE"), "authConfig": SimTypeChar(label="Byte")}, name="IPSEC_AUTH_TRANSFORM_ID0", pack=False, align=None), "cryptoModuleId": SimTypePointer(SimTypeBottom(label="Guid"), offset=0)}, name="IPSEC_AUTH_TRANSFORM0", pack=False, align=None), "cipherTransform": SimStruct({"cipherTransformId": SimStruct({"cipherType": SimTypeInt(signed=False, label="IPSEC_CIPHER_TYPE"), "cipherConfig": SimTypeChar(label="Byte")}, name="IPSEC_CIPHER_TRANSFORM_ID0", pack=False, align=None), "cryptoModuleId": SimTypePointer(SimTypeBottom(label="Guid"), offset=0)}, name="IPSEC_CIPHER_TRANSFORM0", pack=False, align=None)}, name="IPSEC_AUTH_AND_CIPHER_TRANSFORM0", pack=False, align=None), offset=0), "espAuthFwTransform": SimTypePointer(SimStruct({"authTransformId": SimStruct({"authType": SimTypeInt(signed=False, label="IPSEC_AUTH_TYPE"), "authConfig": SimTypeChar(label="Byte")}, name="IPSEC_AUTH_TRANSFORM_ID0", pack=False, align=None), "cryptoModuleId": SimTypePointer(SimTypeBottom(label="Guid"), offset=0)}, name="IPSEC_AUTH_TRANSFORM0", pack=False, align=None), offset=0)}, name="<anon>", label="None")}, name="IPSEC_SA_TRANSFORM0", pack=False, align=None), offset=0), "pfsGroup": SimTypeInt(signed=False, label="IPSEC_PFS_GROUP")}, name="IPSEC_PROPOSAL0", pack=False, align=None), offset=0), "tunnelEndpoints": SimStruct({"ipVersion": SimTypeInt(signed=False, label="FWP_IP_VERSION"), "Anonymous1": SimUnion({"localV4Address": SimTypeInt(signed=False, label="UInt32"), "localV6Address": SimTypeFixedSizeArray(SimTypeChar(label="Byte"), 16)}, name="<anon>", label="None"), "Anonymous2": SimUnion({"remoteV4Address": SimTypeInt(signed=False, label="UInt32"), "remoteV6Address": SimTypeFixedSizeArray(SimTypeChar(label="Byte"), 16)}, name="<anon>", label="None"), "localIfLuid": SimTypeLongLong(signed=False, label="UInt64")}, name="IPSEC_TUNNEL_ENDPOINTS1", pack=False, align=None), "saIdleTimeout": SimStruct({"idleTimeoutSeconds": SimTypeInt(signed=False, label="UInt32"), "idleTimeoutSecondsFailOver": SimTypeInt(signed=False, label="UInt32")}, name="IPSEC_SA_IDLE_TIMEOUT0", pack=False, align=None), "emPolicy": SimTypePointer(SimStruct({"numAuthenticationMethods": SimTypeInt(signed=False, label="UInt32"), "authenticationMethods": SimTypePointer(SimStruct({"authenticationMethodType": SimTypeInt(signed=False, label="IKEEXT_AUTHENTICATION_METHOD_TYPE"), "Anonymous": SimUnion({"presharedKeyAuthentication": SimStruct({"presharedKey": SimStruct({"size": SimTypeInt(signed=False, label="UInt32"), "data": SimTypePointer(SimTypeChar(label="Byte"), offset=0)}, name="FWP_BYTE_BLOB", pack=False, align=None), "flags": SimTypeInt(signed=False, label="IKEEXT_PRESHARED_KEY_AUTHENTICATION_FLAGS")}, name="IKEEXT_PRESHARED_KEY_AUTHENTICATION1", pack=False, align=None), "certificateAuthentication": SimStruct({"inboundConfigType": SimTypeInt(signed=False, label="IKEEXT_CERT_CONFIG_TYPE"), "Anonymous1": SimUnion({"Anonymous": SimStruct({"inboundRootArraySize": SimTypeInt(signed=False, label="UInt32"), "inboundRootArray": SimTypePointer(SimStruct({"certData": SimStruct({"size": SimTypeInt(signed=False, label="UInt32"), "data": SimTypePointer(SimTypeChar(label="Byte"), offset=0)}, name="FWP_BYTE_BLOB", pack=False, align=None), "flags": SimTypeInt(signed=False, label="IKEEXT_CERT_FLAGS")}, name="IKEEXT_CERT_ROOT_CONFIG0", pack=False, align=None), offset=0)}, name="_Anonymous_e__Struct", pack=False, align=None), "inboundEnterpriseStoreConfig": SimTypePointer(SimStruct({"certData": SimStruct({"size": SimTypeInt(signed=False, label="UInt32"), "data": SimTypePointer(SimTypeChar(label="Byte"), offset=0)}, name="FWP_BYTE_BLOB", pack=False, align=None), "flags": SimTypeInt(signed=False, label="IKEEXT_CERT_FLAGS")}, name="IKEEXT_CERT_ROOT_CONFIG0", pack=False, align=None), offset=0), "inboundTrustedRootStoreConfig": SimTypePointer(SimStruct({"certData": SimStruct({"size": SimTypeInt(signed=False, label="UInt32"), "data": SimTypePointer(SimTypeChar(label="Byte"), offset=0)}, name="FWP_BYTE_BLOB", pack=False, align=None), "flags": SimTypeInt(signed=False, label="IKEEXT_CERT_FLAGS")}, name="IKEEXT_CERT_ROOT_CONFIG0", pack=False, align=None), offset=0)}, name="<anon>", label="None"), "outboundConfigType": SimTypeInt(signed=False, label="IKEEXT_CERT_CONFIG_TYPE"), "Anonymous2": SimUnion({"Anonymous": SimStruct({"outboundRootArraySize": SimTypeInt(signed=False, label="UInt32"), "outboundRootArray": SimTypePointer(SimStruct({"certData": SimStruct({"size": SimTypeInt(signed=False, label="UInt32"), "data": SimTypePointer(SimTypeChar(label="Byte"), offset=0)}, name="FWP_BYTE_BLOB", pack=False, align=None), "flags": SimTypeInt(signed=False, label="IKEEXT_CERT_FLAGS")}, name="IKEEXT_CERT_ROOT_CONFIG0", pack=False, align=None), offset=0)}, name="_Anonymous_e__Struct", pack=False, align=None), "outboundEnterpriseStoreConfig": SimTypePointer(SimStruct({"certData": SimStruct({"size": SimTypeInt(signed=False, label="UInt32"), "data": SimTypePointer(SimTypeChar(label="Byte"), offset=0)}, name="FWP_BYTE_BLOB", pack=False, align=None), "flags": SimTypeInt(signed=False, label="IKEEXT_CERT_FLAGS")}, name="IKEEXT_CERT_ROOT_CONFIG0", pack=False, align=None), offset=0), "outboundTrustedRootStoreConfig": SimTypePointer(SimStruct({"certData": SimStruct({"size": SimTypeInt(signed=False, label="UInt32"), "data": SimTypePointer(SimTypeChar(label="Byte"), offset=0)}, name="FWP_BYTE_BLOB", pack=False, align=None), "flags": SimTypeInt(signed=False, label="IKEEXT_CERT_FLAGS")}, name="IKEEXT_CERT_ROOT_CONFIG0", pack=False, align=None), offset=0)}, name="<anon>", label="None"), "flags": SimTypeInt(signed=False, label="IKEEXT_CERT_AUTH"), "localCertLocationUrl": SimStruct({"size": SimTypeInt(signed=False, label="UInt32"), "data": SimTypePointer(SimTypeChar(label="Byte"), offset=0)}, name="FWP_BYTE_BLOB", pack=False, align=None)}, name="IKEEXT_CERTIFICATE_AUTHENTICATION1", pack=False, align=None), "kerberosAuthentication": SimStruct({"flags": SimTypeInt(signed=False, label="IKEEXT_KERBEROS_AUTHENTICATION_FLAGS")}, name="IKEEXT_KERBEROS_AUTHENTICATION0", pack=False, align=None), "ntlmV2Authentication": SimStruct({"flags": SimTypeInt(signed=False, label="UInt32")}, name="IKEEXT_NTLM_V2_AUTHENTICATION0", pack=False, align=None), "sslAuthentication": SimStruct({"inboundConfigType": SimTypeInt(signed=False, label="IKEEXT_CERT_CONFIG_TYPE"), "Anonymous1": SimUnion({"Anonymous": SimStruct({"inboundRootArraySize": SimTypeInt(signed=False, label="UInt32"), "inboundRootArray": SimTypePointer(SimStruct({"certData": SimStruct({"size": SimTypeInt(signed=False, label="UInt32"), "data": SimTypePointer(SimTypeChar(label="Byte"), offset=0)}, name="FWP_BYTE_BLOB", pack=False, align=None), "flags": SimTypeInt(signed=False, label="IKEEXT_CERT_FLAGS")}, name="IKEEXT_CERT_ROOT_CONFIG0", pack=False, align=None), offset=0)}, name="_Anonymous_e__Struct", pack=False, align=None), "inboundEnterpriseStoreConfig": SimTypePointer(SimStruct({"certData": SimStruct({"size": SimTypeInt(signed=False, label="UInt32"), "data": SimTypePointer(SimTypeChar(label="Byte"), offset=0)}, name="FWP_BYTE_BLOB", pack=False, align=None), "flags": SimTypeInt(signed=False, label="IKEEXT_CERT_FLAGS")}, name="IKEEXT_CERT_ROOT_CONFIG0", pack=False, align=None), offset=0), "inboundTrustedRootStoreConfig": SimTypePointer(SimStruct({"certData": SimStruct({"size": SimTypeInt(signed=False, label="UInt32"), "data": SimTypePointer(SimTypeChar(label="Byte"), offset=0)}, name="FWP_BYTE_BLOB", pack=False, align=None), "flags": SimTypeInt(signed=False, label="IKEEXT_CERT_FLAGS")}, name="IKEEXT_CERT_ROOT_CONFIG0", pack=False, align=None), offset=0)}, name="<anon>", label="None"), "outboundConfigType": SimTypeInt(signed=False, label="IKEEXT_CERT_CONFIG_TYPE"), "Anonymous2": SimUnion({"Anonymous": SimStruct({"outboundRootArraySize": SimTypeInt(signed=False, label="UInt32"), "outboundRootArray": SimTypePointer(SimStruct({"certData": SimStruct({"size": SimTypeInt(signed=False, label="UInt32"), "data": SimTypePointer(SimTypeChar(label="Byte"), offset=0)}, name="FWP_BYTE_BLOB", pack=False, align=None), "flags": SimTypeInt(signed=False, label="IKEEXT_CERT_FLAGS")}, name="IKEEXT_CERT_ROOT_CONFIG0", pack=False, align=None), offset=0)}, name="_Anonymous_e__Struct", pack=False, align=None), "outboundEnterpriseStoreConfig": SimTypePointer(SimStruct({"certData": SimStruct({"size": SimTypeInt(signed=False, label="UInt32"), "data": SimTypePointer(SimTypeChar(label="Byte"), offset=0)}, name="FWP_BYTE_BLOB", pack=False, align=None), "flags": SimTypeInt(signed=False, label="IKEEXT_CERT_FLAGS")}, name="IKEEXT_CERT_ROOT_CONFIG0", pack=False, align=None), offset=0), "outboundTrustedRootStoreConfig": SimTypePointer(SimStruct({"certData": SimStruct({"size": SimTypeInt(signed=False, label="UInt32"), "data": SimTypePointer(SimTypeChar(label="Byte"), offset=0)}, name="FWP_BYTE_BLOB", pack=False, align=None), "flags": SimTypeInt(signed=False, label="IKEEXT_CERT_FLAGS")}, name="IKEEXT_CERT_ROOT_CONFIG0", pack=False, align=None), offset=0)}, name="<anon>", label="None"), "flags": SimTypeInt(signed=False, label="IKEEXT_CERT_AUTH"), "localCertLocationUrl": SimStruct({"size": SimTypeInt(signed=False, label="UInt32"), "data": SimTypePointer(SimTypeChar(label="Byte"), offset=0)}, name="FWP_BYTE_BLOB", pack=False, align=None)}, name="IKEEXT_CERTIFICATE_AUTHENTICATION1", pack=False, align=None), "cgaAuthentication": SimStruct({"keyContainerName": SimTypePointer(SimTypeChar(label="Char"), offset=0), "cspName": SimTypePointer(SimTypeChar(label="Char"), offset=0), "cspType": SimTypeInt(signed=False, label="UInt32"), "cgaModifier": SimStruct({"byteArray16": SimTypeFixedSizeArray(SimTypeChar(label="Byte"), 16)}, name="FWP_BYTE_ARRAY16", pack=False, align=None), "cgaCollisionCount": SimTypeChar(label="Byte")}, name="IKEEXT_IPV6_CGA_AUTHENTICATION0", pack=False, align=None), "eapAuthentication": SimStruct({"flags": SimTypeInt(signed=False, label="IKEEXT_EAP_AUTHENTICATION_FLAGS")}, name="IKEEXT_EAP_AUTHENTICATION0", pack=False, align=None)}, name="<anon>", label="None")}, name="IKEEXT_AUTHENTICATION_METHOD1", pack=False, align=None), offset=0), "initiatorImpersonationType": SimTypeInt(signed=False, label="IKEEXT_AUTHENTICATION_IMPERSONATION_TYPE")}, name="IKEEXT_EM_POLICY1", pack=False, align=None), offset=0)}, name="IPSEC_TUNNEL_POLICY1", pack=False, align=None), offset=0), "ikeMmPolicy": SimTypePointer(SimStruct({"softExpirationTime": SimTypeInt(signed=False, label="UInt32"), "numAuthenticationMethods": SimTypeInt(signed=False, label="UInt32"), "authenticationMethods": SimTypePointer(SimStruct({"authenticationMethodType": SimTypeInt(signed=False, label="IKEEXT_AUTHENTICATION_METHOD_TYPE"), "Anonymous": SimUnion({"presharedKeyAuthentication": SimStruct({"presharedKey": SimStruct({"size": SimTypeInt(signed=False, label="UInt32"), "data": SimTypePointer(SimTypeChar(label="Byte"), offset=0)}, name="FWP_BYTE_BLOB", pack=False, align=None), "flags": SimTypeInt(signed=False, label="IKEEXT_PRESHARED_KEY_AUTHENTICATION_FLAGS")}, name="IKEEXT_PRESHARED_KEY_AUTHENTICATION1", pack=False, align=None), "certificateAuthentication": SimStruct({"inboundConfigType": SimTypeInt(signed=False, label="IKEEXT_CERT_CONFIG_TYPE"), "Anonymous1": SimUnion({"Anonymous": SimStruct({"inboundRootArraySize": SimTypeInt(signed=False, label="UInt32"), "inboundRootArray": SimTypePointer(SimStruct({"certData": SimStruct({"size": SimTypeInt(signed=False, label="UInt32"), "data": SimTypePointer(SimTypeChar(label="Byte"), offset=0)}, name="FWP_BYTE_BLOB", pack=False, align=None), "flags": SimTypeInt(signed=False, label="IKEEXT_CERT_FLAGS")}, name="IKEEXT_CERT_ROOT_CONFIG0", pack=False, align=None), offset=0)}, name="_Anonymous_e__Struct", pack=False, align=None), "inboundEnterpriseStoreConfig": SimTypePointer(SimStruct({"certData": SimStruct({"size": SimTypeInt(signed=False, label="UInt32"), "data": SimTypePointer(SimTypeChar(label="Byte"), offset=0)}, name="FWP_BYTE_BLOB", pack=False, align=None), "flags": SimTypeInt(signed=False, label="IKEEXT_CERT_FLAGS")}, name="IKEEXT_CERT_ROOT_CONFIG0", pack=False, align=None), offset=0), "inboundTrustedRootStoreConfig": SimTypePointer(SimStruct({"certData": SimStruct({"size": SimTypeInt(signed=False, label="UInt32"), "data": SimTypePointer(SimTypeChar(label="Byte"), offset=0)}, name="FWP_BYTE_BLOB", pack=False, align=None), "flags": SimTypeInt(signed=False, label="IKEEXT_CERT_FLAGS")}, name="IKEEXT_CERT_ROOT_CONFIG0", pack=False, align=None), offset=0)}, name="<anon>", label="None"), "outboundConfigType": SimTypeInt(signed=False, label="IKEEXT_CERT_CONFIG_TYPE"), "Anonymous2": SimUnion({"Anonymous": SimStruct({"outboundRootArraySize": SimTypeInt(signed=False, label="UInt32"), "outboundRootArray": SimTypePointer(SimStruct({"certData": SimStruct({"size": SimTypeInt(signed=False, label="UInt32"), "data": SimTypePointer(SimTypeChar(label="Byte"), offset=0)}, name="FWP_BYTE_BLOB", pack=False, align=None), "flags": SimTypeInt(signed=False, label="IKEEXT_CERT_FLAGS")}, name="IKEEXT_CERT_ROOT_CONFIG0", pack=False, align=None), offset=0)}, name="_Anonymous_e__Struct", pack=False, align=None), "outboundEnterpriseStoreConfig": SimTypePointer(SimStruct({"certData": SimStruct({"size": SimTypeInt(signed=False, label="UInt32"), "data": SimTypePointer(SimTypeChar(label="Byte"), offset=0)}, name="FWP_BYTE_BLOB", pack=False, align=None), "flags": SimTypeInt(signed=False, label="IKEEXT_CERT_FLAGS")}, name="IKEEXT_CERT_ROOT_CONFIG0", pack=False, align=None), offset=0), "outboundTrustedRootStoreConfig": SimTypePointer(SimStruct({"certData": SimStruct({"size": SimTypeInt(signed=False, label="UInt32"), "data": SimTypePointer(SimTypeChar(label="Byte"), offset=0)}, name="FWP_BYTE_BLOB", pack=False, align=None), "flags": SimTypeInt(signed=False, label="IKEEXT_CERT_FLAGS")}, name="IKEEXT_CERT_ROOT_CONFIG0", pack=False, align=None), offset=0)}, name="<anon>", label="None"), "flags": SimTypeInt(signed=False, label="IKEEXT_CERT_AUTH"), "localCertLocationUrl": SimStruct({"size": SimTypeInt(signed=False, label="UInt32"), "data": SimTypePointer(SimTypeChar(label="Byte"), offset=0)}, name="FWP_BYTE_BLOB", pack=False, align=None)}, name="IKEEXT_CERTIFICATE_AUTHENTICATION1", pack=False, align=None), "kerberosAuthentication": SimStruct({"flags": SimTypeInt(signed=False, label="IKEEXT_KERBEROS_AUTHENTICATION_FLAGS")}, name="IKEEXT_KERBEROS_AUTHENTICATION0", pack=False, align=None), "ntlmV2Authentication": SimStruct({"flags": SimTypeInt(signed=False, label="UInt32")}, name="IKEEXT_NTLM_V2_AUTHENTICATION0", pack=False, align=None), "sslAuthentication": SimStruct({"inboundConfigType": SimTypeInt(signed=False, label="IKEEXT_CERT_CONFIG_TYPE"), "Anonymous1": SimUnion({"Anonymous": SimStruct({"inboundRootArraySize": SimTypeInt(signed=False, label="UInt32"), "inboundRootArray": SimTypePointer(SimStruct({"certData": SimStruct({"size": SimTypeInt(signed=False, label="UInt32"), "data": SimTypePointer(SimTypeChar(label="Byte"), offset=0)}, name="FWP_BYTE_BLOB", pack=False, align=None), "flags": SimTypeInt(signed=False, label="IKEEXT_CERT_FLAGS")}, name="IKEEXT_CERT_ROOT_CONFIG0", pack=False, align=None), offset=0)}, name="_Anonymous_e__Struct", pack=False, align=None), "inboundEnterpriseStoreConfig": SimTypePointer(SimStruct({"certData": SimStruct({"size": SimTypeInt(signed=False, label="UInt32"), "data": SimTypePointer(SimTypeChar(label="Byte"), offset=0)}, name="FWP_BYTE_BLOB", pack=False, align=None), "flags": SimTypeInt(signed=False, label="IKEEXT_CERT_FLAGS")}, name="IKEEXT_CERT_ROOT_CONFIG0", pack=False, align=None), offset=0), "inboundTrustedRootStoreConfig": SimTypePointer(SimStruct({"certData": SimStruct({"size": SimTypeInt(signed=False, label="UInt32"), "data": SimTypePointer(SimTypeChar(label="Byte"), offset=0)}, name="FWP_BYTE_BLOB", pack=False, align=None), "flags": SimTypeInt(signed=False, label="IKEEXT_CERT_FLAGS")}, name="IKEEXT_CERT_ROOT_CONFIG0", pack=False, align=None), offset=0)}, name="<anon>", label="None"), "outboundConfigType": SimTypeInt(signed=False, label="IKEEXT_CERT_CONFIG_TYPE"), "Anonymous2": SimUnion({"Anonymous": SimStruct({"outboundRootArraySize": SimTypeInt(signed=False, label="UInt32"), "outboundRootArray": SimTypePointer(SimStruct({"certData": SimStruct({"size": SimTypeInt(signed=False, label="UInt32"), "data": SimTypePointer(SimTypeChar(label="Byte"), offset=0)}, name="FWP_BYTE_BLOB", pack=False, align=None), "flags": SimTypeInt(signed=False, label="IKEEXT_CERT_FLAGS")}, name="IKEEXT_CERT_ROOT_CONFIG0", pack=False, align=None), offset=0)}, name="_Anonymous_e__Struct", pack=False, align=None), "outboundEnterpriseStoreConfig": SimTypePointer(SimStruct({"certData": SimStruct({"size": SimTypeInt(signed=False, label="UInt32"), "data": SimTypePointer(SimTypeChar(label="Byte"), offset=0)}, name="FWP_BYTE_BLOB", pack=False, align=None), "flags": SimTypeInt(signed=False, label="IKEEXT_CERT_FLAGS")}, name="IKEEXT_CERT_ROOT_CONFIG0", pack=False, align=None), offset=0), "outboundTrustedRootStoreConfig": SimTypePointer(SimStruct({"certData": SimStruct({"size": SimTypeInt(signed=False, label="UInt32"), "data": SimTypePointer(SimTypeChar(label="Byte"), offset=0)}, name="FWP_BYTE_BLOB", pack=False, align=None), "flags": SimTypeInt(signed=False, label="IKEEXT_CERT_FLAGS")}, name="IKEEXT_CERT_ROOT_CONFIG0", pack=False, align=None), offset=0)}, name="<anon>", label="None"), "flags": SimTypeInt(signed=False, label="IKEEXT_CERT_AUTH"), "localCertLocationUrl": SimStruct({"size": SimTypeInt(signed=False, label="UInt32"), "data": SimTypePointer(SimTypeChar(label="Byte"), offset=0)}, name="FWP_BYTE_BLOB", pack=False, align=None)}, name="IKEEXT_CERTIFICATE_AUTHENTICATION1", pack=False, align=None), "cgaAuthentication": SimStruct({"keyContainerName": SimTypePointer(SimTypeChar(label="Char"), offset=0), "cspName": SimTypePointer(SimTypeChar(label="Char"), offset=0), "cspType": SimTypeInt(signed=False, label="UInt32"), "cgaModifier": SimStruct({"byteArray16": SimTypeFixedSizeArray(SimTypeChar(label="Byte"), 16)}, name="FWP_BYTE_ARRAY16", pack=False, align=None), "cgaCollisionCount": SimTypeChar(label="Byte")}, name="IKEEXT_IPV6_CGA_AUTHENTICATION0", pack=False, align=None), "eapAuthentication": SimStruct({"flags": SimTypeInt(signed=False, label="IKEEXT_EAP_AUTHENTICATION_FLAGS")}, name="IKEEXT_EAP_AUTHENTICATION0", pack=False, align=None)}, name="<anon>", label="None")}, name="IKEEXT_AUTHENTICATION_METHOD1", pack=False, align=None), offset=0), "initiatorImpersonationType": SimTypeInt(signed=False, label="IKEEXT_AUTHENTICATION_IMPERSONATION_TYPE"), "numIkeProposals": SimTypeInt(signed=False, label="UInt32"), "ikeProposals": SimTypePointer(SimStruct({"cipherAlgorithm": SimStruct({"algoIdentifier": SimTypeInt(signed=False, label="IKEEXT_CIPHER_TYPE"), "keyLen": SimTypeInt(signed=False, label="UInt32"), "rounds": SimTypeInt(signed=False, label="UInt32")}, name="IKEEXT_CIPHER_ALGORITHM0", pack=False, align=None), "integrityAlgorithm": SimStruct({"algoIdentifier": SimTypeInt(signed=False, label="IKEEXT_INTEGRITY_TYPE")}, name="IKEEXT_INTEGRITY_ALGORITHM0", pack=False, align=None), "maxLifetimeSeconds": SimTypeInt(signed=False, label="UInt32"), "dhGroup": SimTypeInt(signed=False, label="IKEEXT_DH_GROUP"), "quickModeLimit": SimTypeInt(signed=False, label="UInt32")}, name="IKEEXT_PROPOSAL0", pack=False, align=None), offset=0), "flags": SimTypeInt(signed=False, label="IKEEXT_POLICY_FLAG"), "maxDynamicFilters": SimTypeInt(signed=False, label="UInt32"), "retransmitDurationSecs": SimTypeInt(signed=False, label="UInt32")}, name="IKEEXT_POLICY1", pack=False, align=None), offset=0), "authIpMmPolicy": SimTypePointer(SimStruct({"softExpirationTime": SimTypeInt(signed=False, label="UInt32"), "numAuthenticationMethods": SimTypeInt(signed=False, label="UInt32"), "authenticationMethods": SimTypePointer(SimStruct({"authenticationMethodType": SimTypeInt(signed=False, label="IKEEXT_AUTHENTICATION_METHOD_TYPE"), "Anonymous": SimUnion({"presharedKeyAuthentication": SimStruct({"presharedKey": SimStruct({"size": SimTypeInt(signed=False, label="UInt32"), "data": SimTypePointer(SimTypeChar(label="Byte"), offset=0)}, name="FWP_BYTE_BLOB", pack=False, align=None), "flags": SimTypeInt(signed=False, label="IKEEXT_PRESHARED_KEY_AUTHENTICATION_FLAGS")}, name="IKEEXT_PRESHARED_KEY_AUTHENTICATION1", pack=False, align=None), "certificateAuthentication": SimStruct({"inboundConfigType": SimTypeInt(signed=False, label="IKEEXT_CERT_CONFIG_TYPE"), "Anonymous1": SimUnion({"Anonymous": SimStruct({"inboundRootArraySize": SimTypeInt(signed=False, label="UInt32"), "inboundRootArray": SimTypePointer(SimStruct({"certData": SimStruct({"size": SimTypeInt(signed=False, label="UInt32"), "data": SimTypePointer(SimTypeChar(label="Byte"), offset=0)}, name="FWP_BYTE_BLOB", pack=False, align=None), "flags": SimTypeInt(signed=False, label="IKEEXT_CERT_FLAGS")}, name="IKEEXT_CERT_ROOT_CONFIG0", pack=False, align=None), offset=0)}, name="_Anonymous_e__Struct", pack=False, align=None), "inboundEnterpriseStoreConfig": SimTypePointer(SimStruct({"certData": SimStruct({"size": SimTypeInt(signed=False, label="UInt32"), "data": SimTypePointer(SimTypeChar(label="Byte"), offset=0)}, name="FWP_BYTE_BLOB", pack=False, align=None), "flags": SimTypeInt(signed=False, label="IKEEXT_CERT_FLAGS")}, name="IKEEXT_CERT_ROOT_CONFIG0", pack=False, align=None), offset=0), "inboundTrustedRootStoreConfig": SimTypePointer(SimStruct({"certData": SimStruct({"size": SimTypeInt(signed=False, label="UInt32"), "data": SimTypePointer(SimTypeChar(label="Byte"), offset=0)}, name="FWP_BYTE_BLOB", pack=False, align=None), "flags": SimTypeInt(signed=False, label="IKEEXT_CERT_FLAGS")}, name="IKEEXT_CERT_ROOT_CONFIG0", pack=False, align=None), offset=0)}, name="<anon>", label="None"), "outboundConfigType": SimTypeInt(signed=False, label="IKEEXT_CERT_CONFIG_TYPE"), "Anonymous2": SimUnion({"Anonymous": SimStruct({"outboundRootArraySize": SimTypeInt(signed=False, label="UInt32"), "outboundRootArray": SimTypePointer(SimStruct({"certData": SimStruct({"size": SimTypeInt(signed=False, label="UInt32"), "data": SimTypePointer(SimTypeChar(label="Byte"), offset=0)}, name="FWP_BYTE_BLOB", pack=False, align=None), "flags": SimTypeInt(signed=False, label="IKEEXT_CERT_FLAGS")}, name="IKEEXT_CERT_ROOT_CONFIG0", pack=False, align=None), offset=0)}, name="_Anonymous_e__Struct", pack=False, align=None), "outboundEnterpriseStoreConfig": SimTypePointer(SimStruct({"certData": SimStruct({"size": SimTypeInt(signed=False, label="UInt32"), "data": SimTypePointer(SimTypeChar(label="Byte"), offset=0)}, name="FWP_BYTE_BLOB", pack=False, align=None), "flags": SimTypeInt(signed=False, label="IKEEXT_CERT_FLAGS")}, name="IKEEXT_CERT_ROOT_CONFIG0", pack=False, align=None), offset=0), "outboundTrustedRootStoreConfig": SimTypePointer(SimStruct({"certData": SimStruct({"size": SimTypeInt(signed=False, label="UInt32"), "data": SimTypePointer(SimTypeChar(label="Byte"), offset=0)}, name="FWP_BYTE_BLOB", pack=False, align=None), "flags": SimTypeInt(signed=False, label="IKEEXT_CERT_FLAGS")}, name="IKEEXT_CERT_ROOT_CONFIG0", pack=False, align=None), offset=0)}, name="<anon>", label="None"), "flags": SimTypeInt(signed=False, label="IKEEXT_CERT_AUTH"), "localCertLocationUrl": SimStruct({"size": SimTypeInt(signed=False, label="UInt32"), "data": SimTypePointer(SimTypeChar(label="Byte"), offset=0)}, name="FWP_BYTE_BLOB", pack=False, align=None)}, name="IKEEXT_CERTIFICATE_AUTHENTICATION1", pack=False, align=None), "kerberosAuthentication": SimStruct({"flags": SimTypeInt(signed=False, label="IKEEXT_KERBEROS_AUTHENTICATION_FLAGS")}, name="IKEEXT_KERBEROS_AUTHENTICATION0", pack=False, align=None), "ntlmV2Authentication": SimStruct({"flags": SimTypeInt(signed=False, label="UInt32")}, name="IKEEXT_NTLM_V2_AUTHENTICATION0", pack=False, align=None), "sslAuthentication": SimStruct({"inboundConfigType": SimTypeInt(signed=False, label="IKEEXT_CERT_CONFIG_TYPE"), "Anonymous1": SimUnion({"Anonymous": SimStruct({"inboundRootArraySize": SimTypeInt(signed=False, label="UInt32"), "inboundRootArray": SimTypePointer(SimStruct({"certData": SimStruct({"size": SimTypeInt(signed=False, label="UInt32"), "data": SimTypePointer(SimTypeChar(label="Byte"), offset=0)}, name="FWP_BYTE_BLOB", pack=False, align=None), "flags": SimTypeInt(signed=False, label="IKEEXT_CERT_FLAGS")}, name="IKEEXT_CERT_ROOT_CONFIG0", pack=False, align=None), offset=0)}, name="_Anonymous_e__Struct", pack=False, align=None), "inboundEnterpriseStoreConfig": SimTypePointer(SimStruct({"certData": SimStruct({"size": SimTypeInt(signed=False, label="UInt32"), "data": SimTypePointer(SimTypeChar(label="Byte"), offset=0)}, name="FWP_BYTE_BLOB", pack=False, align=None), "flags": SimTypeInt(signed=False, label="IKEEXT_CERT_FLAGS")}, name="IKEEXT_CERT_ROOT_CONFIG0", pack=False, align=None), offset=0), "inboundTrustedRootStoreConfig": SimTypePointer(SimStruct({"certData": SimStruct({"size": SimTypeInt(signed=False, label="UInt32"), "data": SimTypePointer(SimTypeChar(label="Byte"), offset=0)}, name="FWP_BYTE_BLOB", pack=False, align=None), "flags": SimTypeInt(signed=False, label="IKEEXT_CERT_FLAGS")}, name="IKEEXT_CERT_ROOT_CONFIG0", pack=False, align=None), offset=0)}, name="<anon>", label="None"), "outboundConfigType": SimTypeInt(signed=False, label="IKEEXT_CERT_CONFIG_TYPE"), "Anonymous2": SimUnion({"Anonymous": SimStruct({"outboundRootArraySize": SimTypeInt(signed=False, label="UInt32"), "outboundRootArray": SimTypePointer(SimStruct({"certData": SimStruct({"size": SimTypeInt(signed=False, label="UInt32"), "data": SimTypePointer(SimTypeChar(label="Byte"), offset=0)}, name="FWP_BYTE_BLOB", pack=False, align=None), "flags": SimTypeInt(signed=False, label="IKEEXT_CERT_FLAGS")}, name="IKEEXT_CERT_ROOT_CONFIG0", pack=False, align=None), offset=0)}, name="_Anonymous_e__Struct", pack=False, align=None), "outboundEnterpriseStoreConfig": SimTypePointer(SimStruct({"certData": SimStruct({"size": SimTypeInt(signed=False, label="UInt32"), "data": SimTypePointer(SimTypeChar(label="Byte"), offset=0)}, name="FWP_BYTE_BLOB", pack=False, align=None), "flags": SimTypeInt(signed=False, label="IKEEXT_CERT_FLAGS")}, name="IKEEXT_CERT_ROOT_CONFIG0", pack=False, align=None), offset=0), "outboundTrustedRootStoreConfig": SimTypePointer(SimStruct({"certData": SimStruct({"size": SimTypeInt(signed=False, label="UInt32"), "data": SimTypePointer(SimTypeChar(label="Byte"), offset=0)}, name="FWP_BYTE_BLOB", pack=False, align=None), "flags": SimTypeInt(signed=False, label="IKEEXT_CERT_FLAGS")}, name="IKEEXT_CERT_ROOT_CONFIG0", pack=False, align=None), offset=0)}, name="<anon>", label="None"), "flags": SimTypeInt(signed=False, label="IKEEXT_CERT_AUTH"), "localCertLocationUrl": SimStruct({"size": SimTypeInt(signed=False, label="UInt32"), "data": SimTypePointer(SimTypeChar(label="Byte"), offset=0)}, name="FWP_BYTE_BLOB", pack=False, align=None)}, name="IKEEXT_CERTIFICATE_AUTHENTICATION1", pack=False, align=None), "cgaAuthentication": SimStruct({"keyContainerName": SimTypePointer(SimTypeChar(label="Char"), offset=0), "cspName": SimTypePointer(SimTypeChar(label="Char"), offset=0), "cspType": SimTypeInt(signed=False, label="UInt32"), "cgaModifier": SimStruct({"byteArray16": SimTypeFixedSizeArray(SimTypeChar(label="Byte"), 16)}, name="FWP_BYTE_ARRAY16", pack=False, align=None), "cgaCollisionCount": SimTypeChar(label="Byte")}, name="IKEEXT_IPV6_CGA_AUTHENTICATION0", pack=False, align=None), "eapAuthentication": SimStruct({"flags": SimTypeInt(signed=False, label="IKEEXT_EAP_AUTHENTICATION_FLAGS")}, name="IKEEXT_EAP_AUTHENTICATION0", pack=False, align=None)}, name="<anon>", label="None")}, name="IKEEXT_AUTHENTICATION_METHOD1", pack=False, align=None), offset=0), "initiatorImpersonationType": SimTypeInt(signed=False, label="IKEEXT_AUTHENTICATION_IMPERSONATION_TYPE"), "numIkeProposals": SimTypeInt(signed=False, label="UInt32"), "ikeProposals": SimTypePointer(SimStruct({"cipherAlgorithm": SimStruct({"algoIdentifier": SimTypeInt(signed=False, label="IKEEXT_CIPHER_TYPE"), "keyLen": SimTypeInt(signed=False, label="UInt32"), "rounds": SimTypeInt(signed=False, label="UInt32")}, name="IKEEXT_CIPHER_ALGORITHM0", pack=False, align=None), "integrityAlgorithm": SimStruct({"algoIdentifier": SimTypeInt(signed=False, label="IKEEXT_INTEGRITY_TYPE")}, name="IKEEXT_INTEGRITY_ALGORITHM0", pack=False, align=None), "maxLifetimeSeconds": SimTypeInt(signed=False, label="UInt32"), "dhGroup": SimTypeInt(signed=False, label="IKEEXT_DH_GROUP"), "quickModeLimit": SimTypeInt(signed=False, label="UInt32")}, name="IKEEXT_PROPOSAL0", pack=False, align=None), offset=0), "flags": SimTypeInt(signed=False, label="IKEEXT_POLICY_FLAG"), "maxDynamicFilters": SimTypeInt(signed=False, label="UInt32"), "retransmitDurationSecs": SimTypeInt(signed=False, label="UInt32")}, name="IKEEXT_POLICY1", pack=False, align=None), offset=0), "dataBuffer": SimTypePointer(SimStruct({"size": SimTypeInt(signed=False, label="UInt32"), "data": SimTypePointer(SimTypeChar(label="Byte"), offset=0)}, name="FWP_BYTE_BLOB", pack=False, align=None), offset=0), "classifyOptions": SimTypePointer(SimStruct({"numOptions": SimTypeInt(signed=False, label="UInt32"), "options": SimTypePointer(SimStruct({"type": SimTypeInt(signed=False, label="FWP_CLASSIFY_OPTION_TYPE"), "value": SimStruct({"type": SimTypeInt(signed=False, label="FWP_DATA_TYPE"), "Anonymous": SimUnion({"uint8": SimTypeChar(label="Byte"), "uint16": SimTypeShort(signed=False, label="UInt16"), "uint32": SimTypeInt(signed=False, label="UInt32"), "uint64": SimTypePointer(SimTypeLongLong(signed=False, label="UInt64"), offset=0), "int8": SimTypeChar(label="SByte"), "int16": SimTypeShort(signed=True, label="Int16"), "int32": SimTypeInt(signed=True, label="Int32"), "int64": SimTypePointer(SimTypeLongLong(signed=True, label="Int64"), offset=0), "float32": SimTypeFloat(size=32), "double64": SimTypePointer(SimTypeFloat(size=64), offset=0), "byteArray16": SimTypePointer(SimStruct({"byteArray16": SimTypeFixedSizeArray(SimTypeChar(label="Byte"), 16)}, name="FWP_BYTE_ARRAY16", pack=False, align=None), offset=0), "byteBlob": SimTypePointer(SimStruct({"size": SimTypeInt(signed=False, label="UInt32"), "data": SimTypePointer(SimTypeChar(label="Byte"), offset=0)}, name="FWP_BYTE_BLOB", pack=False, align=None), offset=0), "sid": SimTypePointer(SimTypeBottom(label="SID"), offset=0), "sd": SimTypePointer(SimStruct({"size": SimTypeInt(signed=False, label="UInt32"), "data": SimTypePointer(SimTypeChar(label="Byte"), offset=0)}, name="FWP_BYTE_BLOB", pack=False, align=None), offset=0), "tokenInformation": SimTypePointer(SimStruct({"sidCount": SimTypeInt(signed=False, label="UInt32"), "sids": SimTypePointer(SimTypeBottom(label="SID_AND_ATTRIBUTES"), offset=0), "restrictedSidCount": SimTypeInt(signed=False, label="UInt32"), "restrictedSids": SimTypePointer(SimTypeBottom(label="SID_AND_ATTRIBUTES"), offset=0)}, name="FWP_TOKEN_INFORMATION", pack=False, align=None), offset=0), "tokenAccessInformation": SimTypePointer(SimStruct({"size": SimTypeInt(signed=False, label="UInt32"), "data": SimTypePointer(SimTypeChar(label="Byte"), offset=0)}, name="FWP_BYTE_BLOB", pack=False, align=None), offset=0), "unicodeString": SimTypePointer(SimTypeChar(label="Char"), offset=0), "byteArray6": SimTypePointer(SimStruct({"byteArray6": SimTypeFixedSizeArray(SimTypeChar(label="Byte"), 6)}, name="FWP_BYTE_ARRAY6", pack=False, align=None), offset=0), "bitmapArray64": SimTypePointer(SimStruct({"bitmapArray64": SimTypeFixedSizeArray(SimTypeChar(label="Byte"), 8)}, name="FWP_BITMAP_ARRAY64_", pack=False, align=None), offset=0)}, name="<anon>", label="None")}, name="FWP_VALUE0", pack=False, align=None)}, name="FWPM_CLASSIFY_OPTION0", pack=False, align=None), offset=0)}, name="FWPM_CLASSIFY_OPTIONS0", pack=False, align=None), offset=0), "ikeV2QmTunnelPolicy": SimTypePointer(SimStruct({"flags": SimTypeInt(signed=False, label="IPSEC_POLICY_FLAG"), "numIpsecProposals": SimTypeInt(signed=False, label="UInt32"), "ipsecProposals": SimTypePointer(SimStruct({"lifetime": SimStruct({"lifetimeSeconds": SimTypeInt(signed=False, label="UInt32"), "lifetimeKilobytes": SimTypeInt(signed=False, label="UInt32"), "lifetimePackets": SimTypeInt(signed=False, label="UInt32")}, name="IPSEC_SA_LIFETIME0", pack=False, align=None), "numSaTransforms": SimTypeInt(signed=False, label="UInt32"), "saTransforms": SimTypePointer(SimStruct({"ipsecTransformType": SimTypeInt(signed=False, label="IPSEC_TRANSFORM_TYPE"), "Anonymous": SimUnion({"ahTransform": SimTypePointer(SimStruct({"authTransformId": SimStruct({"authType": SimTypeInt(signed=False, label="IPSEC_AUTH_TYPE"), "authConfig": SimTypeChar(label="Byte")}, name="IPSEC_AUTH_TRANSFORM_ID0", pack=False, align=None), "cryptoModuleId": SimTypePointer(SimTypeBottom(label="Guid"), offset=0)}, name="IPSEC_AUTH_TRANSFORM0", pack=False, align=None), offset=0), "espAuthTransform": SimTypePointer(SimStruct({"authTransformId": SimStruct({"authType": SimTypeInt(signed=False, label="IPSEC_AUTH_TYPE"), "authConfig": SimTypeChar(label="Byte")}, name="IPSEC_AUTH_TRANSFORM_ID0", pack=False, align=None), "cryptoModuleId": SimTypePointer(SimTypeBottom(label="Guid"), offset=0)}, name="IPSEC_AUTH_TRANSFORM0", pack=False, align=None), offset=0), "espCipherTransform": SimTypePointer(SimStruct({"cipherTransformId": SimStruct({"cipherType": SimTypeInt(signed=False, label="IPSEC_CIPHER_TYPE"), "cipherConfig": SimTypeChar(label="Byte")}, name="IPSEC_CIPHER_TRANSFORM_ID0", pack=False, align=None), "cryptoModuleId": SimTypePointer(SimTypeBottom(label="Guid"), offset=0)}, name="IPSEC_CIPHER_TRANSFORM0", pack=False, align=None), offset=0), "espAuthAndCipherTransform": SimTypePointer(SimStruct({"authTransform": SimStruct({"authTransformId": SimStruct({"authType": SimTypeInt(signed=False, label="IPSEC_AUTH_TYPE"), "authConfig": SimTypeChar(label="Byte")}, name="IPSEC_AUTH_TRANSFORM_ID0", pack=False, align=None), "cryptoModuleId": SimTypePointer(SimTypeBottom(label="Guid"), offset=0)}, name="IPSEC_AUTH_TRANSFORM0", pack=False, align=None), "cipherTransform": SimStruct({"cipherTransformId": SimStruct({"cipherType": SimTypeInt(signed=False, label="IPSEC_CIPHER_TYPE"), "cipherConfig": SimTypeChar(label="Byte")}, name="IPSEC_CIPHER_TRANSFORM_ID0", pack=False, align=None), "cryptoModuleId": SimTypePointer(SimTypeBottom(label="Guid"), offset=0)}, name="IPSEC_CIPHER_TRANSFORM0", pack=False, align=None)}, name="IPSEC_AUTH_AND_CIPHER_TRANSFORM0", pack=False, align=None), offset=0), "espAuthFwTransform": SimTypePointer(SimStruct({"authTransformId": SimStruct({"authType": SimTypeInt(signed=False, label="IPSEC_AUTH_TYPE"), "authConfig": SimTypeChar(label="Byte")}, name="IPSEC_AUTH_TRANSFORM_ID0", pack=False, align=None), "cryptoModuleId": SimTypePointer(SimTypeBottom(label="Guid"), offset=0)}, name="IPSEC_AUTH_TRANSFORM0", pack=False, align=None), offset=0)}, name="<anon>", label="None")}, name="IPSEC_SA_TRANSFORM0", pack=False, align=None), offset=0), "pfsGroup": SimTypeInt(signed=False, label="IPSEC_PFS_GROUP")}, name="IPSEC_PROPOSAL0", pack=False, align=None), offset=0), "tunnelEndpoints": SimStruct({"ipVersion": SimTypeInt(signed=False, label="FWP_IP_VERSION"), "Anonymous1": SimUnion({"localV4Address": SimTypeInt(signed=False, label="UInt32"), "localV6Address": SimTypeFixedSizeArray(SimTypeChar(label="Byte"), 16)}, name="<anon>", label="None"), "Anonymous2": SimUnion({"remoteV4Address": SimTypeInt(signed=False, label="UInt32"), "remoteV6Address": SimTypeFixedSizeArray(SimTypeChar(label="Byte"), 16)}, name="<anon>", label="None"), "localIfLuid": SimTypeLongLong(signed=False, label="UInt64")}, name="IPSEC_TUNNEL_ENDPOINTS1", pack=False, align=None), "saIdleTimeout": SimStruct({"idleTimeoutSeconds": SimTypeInt(signed=False, label="UInt32"), "idleTimeoutSecondsFailOver": SimTypeInt(signed=False, label="UInt32")}, name="IPSEC_SA_IDLE_TIMEOUT0", pack=False, align=None), "emPolicy": SimTypePointer(SimStruct({"numAuthenticationMethods": SimTypeInt(signed=False, label="UInt32"), "authenticationMethods": SimTypePointer(SimStruct({"authenticationMethodType": SimTypeInt(signed=False, label="IKEEXT_AUTHENTICATION_METHOD_TYPE"), "Anonymous": SimUnion({"presharedKeyAuthentication": SimStruct({"presharedKey": SimStruct({"size": SimTypeInt(signed=False, label="UInt32"), "data": SimTypePointer(SimTypeChar(label="Byte"), offset=0)}, name="FWP_BYTE_BLOB", pack=False, align=None), "flags": SimTypeInt(signed=False, label="IKEEXT_PRESHARED_KEY_AUTHENTICATION_FLAGS")}, name="IKEEXT_PRESHARED_KEY_AUTHENTICATION1", pack=False, align=None), "certificateAuthentication": SimStruct({"inboundConfigType": SimTypeInt(signed=False, label="IKEEXT_CERT_CONFIG_TYPE"), "Anonymous1": SimUnion({"Anonymous": SimStruct({"inboundRootArraySize": SimTypeInt(signed=False, label="UInt32"), "inboundRootArray": SimTypePointer(SimStruct({"certData": SimStruct({"size": SimTypeInt(signed=False, label="UInt32"), "data": SimTypePointer(SimTypeChar(label="Byte"), offset=0)}, name="FWP_BYTE_BLOB", pack=False, align=None), "flags": SimTypeInt(signed=False, label="IKEEXT_CERT_FLAGS")}, name="IKEEXT_CERT_ROOT_CONFIG0", pack=False, align=None), offset=0)}, name="_Anonymous_e__Struct", pack=False, align=None), "inboundEnterpriseStoreConfig": SimTypePointer(SimStruct({"certData": SimStruct({"size": SimTypeInt(signed=False, label="UInt32"), "data": SimTypePointer(SimTypeChar(label="Byte"), offset=0)}, name="FWP_BYTE_BLOB", pack=False, align=None), "flags": SimTypeInt(signed=False, label="IKEEXT_CERT_FLAGS")}, name="IKEEXT_CERT_ROOT_CONFIG0", pack=False, align=None), offset=0), "inboundTrustedRootStoreConfig": SimTypePointer(SimStruct({"certData": SimStruct({"size": SimTypeInt(signed=False, label="UInt32"), "data": SimTypePointer(SimTypeChar(label="Byte"), offset=0)}, name="FWP_BYTE_BLOB", pack=False, align=None), "flags": SimTypeInt(signed=False, label="IKEEXT_CERT_FLAGS")}, name="IKEEXT_CERT_ROOT_CONFIG0", pack=False, align=None), offset=0)}, name="<anon>", label="None"), "outboundConfigType": SimTypeInt(signed=False, label="IKEEXT_CERT_CONFIG_TYPE"), "Anonymous2": SimUnion({"Anonymous": SimStruct({"outboundRootArraySize": SimTypeInt(signed=False, label="UInt32"), "outboundRootArray": SimTypePointer(SimStruct({"certData": SimStruct({"size": SimTypeInt(signed=False, label="UInt32"), "data": SimTypePointer(SimTypeChar(label="Byte"), offset=0)}, name="FWP_BYTE_BLOB", pack=False, align=None), "flags": SimTypeInt(signed=False, label="IKEEXT_CERT_FLAGS")}, name="IKEEXT_CERT_ROOT_CONFIG0", pack=False, align=None), offset=0)}, name="_Anonymous_e__Struct", pack=False, align=None), "outboundEnterpriseStoreConfig": SimTypePointer(SimStruct({"certData": SimStruct({"size": SimTypeInt(signed=False, label="UInt32"), "data": SimTypePointer(SimTypeChar(label="Byte"), offset=0)}, name="FWP_BYTE_BLOB", pack=False, align=None), "flags": SimTypeInt(signed=False, label="IKEEXT_CERT_FLAGS")}, name="IKEEXT_CERT_ROOT_CONFIG0", pack=False, align=None), offset=0), "outboundTrustedRootStoreConfig": SimTypePointer(SimStruct({"certData": SimStruct({"size": SimTypeInt(signed=False, label="UInt32"), "data": SimTypePointer(SimTypeChar(label="Byte"), offset=0)}, name="FWP_BYTE_BLOB", pack=False, align=None), "flags": SimTypeInt(signed=False, label="IKEEXT_CERT_FLAGS")}, name="IKEEXT_CERT_ROOT_CONFIG0", pack=False, align=None), offset=0)}, name="<anon>", label="None"), "flags": SimTypeInt(signed=False, label="IKEEXT_CERT_AUTH"), "localCertLocationUrl": SimStruct({"size": SimTypeInt(signed=False, label="UInt32"), "data": SimTypePointer(SimTypeChar(label="Byte"), offset=0)}, name="FWP_BYTE_BLOB", pack=False, align=None)}, name="IKEEXT_CERTIFICATE_AUTHENTICATION1", pack=False, align=None), "kerberosAuthentication": SimStruct({"flags": SimTypeInt(signed=False, label="IKEEXT_KERBEROS_AUTHENTICATION_FLAGS")}, name="IKEEXT_KERBEROS_AUTHENTICATION0", pack=False, align=None), "ntlmV2Authentication": SimStruct({"flags": SimTypeInt(signed=False, label="UInt32")}, name="IKEEXT_NTLM_V2_AUTHENTICATION0", pack=False, align=None), "sslAuthentication": SimStruct({"inboundConfigType": SimTypeInt(signed=False, label="IKEEXT_CERT_CONFIG_TYPE"), "Anonymous1": SimUnion({"Anonymous": SimStruct({"inboundRootArraySize": SimTypeInt(signed=False, label="UInt32"), "inboundRootArray": SimTypePointer(SimStruct({"certData": SimStruct({"size": SimTypeInt(signed=False, label="UInt32"), "data": SimTypePointer(SimTypeChar(label="Byte"), offset=0)}, name="FWP_BYTE_BLOB", pack=False, align=None), "flags": SimTypeInt(signed=False, label="IKEEXT_CERT_FLAGS")}, name="IKEEXT_CERT_ROOT_CONFIG0", pack=False, align=None), offset=0)}, name="_Anonymous_e__Struct", pack=False, align=None), "inboundEnterpriseStoreConfig": SimTypePointer(SimStruct({"certData": SimStruct({"size": SimTypeInt(signed=False, label="UInt32"), "data": SimTypePointer(SimTypeChar(label="Byte"), offset=0)}, name="FWP_BYTE_BLOB", pack=False, align=None), "flags": SimTypeInt(signed=False, label="IKEEXT_CERT_FLAGS")}, name="IKEEXT_CERT_ROOT_CONFIG0", pack=False, align=None), offset=0), "inboundTrustedRootStoreConfig": SimTypePointer(SimStruct({"certData": SimStruct({"size": SimTypeInt(signed=False, label="UInt32"), "data": SimTypePointer(SimTypeChar(label="Byte"), offset=0)}, name="FWP_BYTE_BLOB", pack=False, align=None), "flags": SimTypeInt(signed=False, label="IKEEXT_CERT_FLAGS")}, name="IKEEXT_CERT_ROOT_CONFIG0", pack=False, align=None), offset=0)}, name="<anon>", label="None"), "outboundConfigType": SimTypeInt(signed=False, label="IKEEXT_CERT_CONFIG_TYPE"), "Anonymous2": SimUnion({"Anonymous": SimStruct({"outboundRootArraySize": SimTypeInt(signed=False, label="UInt32"), "outboundRootArray": SimTypePointer(SimStruct({"certData": SimStruct({"size": SimTypeInt(signed=False, label="UInt32"), "data": SimTypePointer(SimTypeChar(label="Byte"), offset=0)}, name="FWP_BYTE_BLOB", pack=False, align=None), "flags": SimTypeInt(signed=False, label="IKEEXT_CERT_FLAGS")}, name="IKEEXT_CERT_ROOT_CONFIG0", pack=False, align=None), offset=0)}, name="_Anonymous_e__Struct", pack=False, align=None), "outboundEnterpriseStoreConfig": SimTypePointer(SimStruct({"certData": SimStruct({"size": SimTypeInt(signed=False, label="UInt32"), "data": SimTypePointer(SimTypeChar(label="Byte"), offset=0)}, name="FWP_BYTE_BLOB", pack=False, align=None), "flags": SimTypeInt(signed=False, label="IKEEXT_CERT_FLAGS")}, name="IKEEXT_CERT_ROOT_CONFIG0", pack=False, align=None), offset=0), "outboundTrustedRootStoreConfig": SimTypePointer(SimStruct({"certData": SimStruct({"size": SimTypeInt(signed=False, label="UInt32"), "data": SimTypePointer(SimTypeChar(label="Byte"), offset=0)}, name="FWP_BYTE_BLOB", pack=False, align=None), "flags": SimTypeInt(signed=False, label="IKEEXT_CERT_FLAGS")}, name="IKEEXT_CERT_ROOT_CONFIG0", pack=False, align=None), offset=0)}, name="<anon>", label="None"), "flags": SimTypeInt(signed=False, label="IKEEXT_CERT_AUTH"), "localCertLocationUrl": SimStruct({"size": SimTypeInt(signed=False, label="UInt32"), "data": SimTypePointer(SimTypeChar(label="Byte"), offset=0)}, name="FWP_BYTE_BLOB", pack=False, align=None)}, name="IKEEXT_CERTIFICATE_AUTHENTICATION1", pack=False, align=None), "cgaAuthentication": SimStruct({"keyContainerName": SimTypePointer(SimTypeChar(label="Char"), offset=0), "cspName": SimTypePointer(SimTypeChar(label="Char"), offset=0), "cspType": SimTypeInt(signed=False, label="UInt32"), "cgaModifier": SimStruct({"byteArray16": SimTypeFixedSizeArray(SimTypeChar(label="Byte"), 16)}, name="FWP_BYTE_ARRAY16", pack=False, align=None), "cgaCollisionCount": SimTypeChar(label="Byte")}, name="IKEEXT_IPV6_CGA_AUTHENTICATION0", pack=False, align=None), "eapAuthentication": SimStruct({"flags": SimTypeInt(signed=False, label="IKEEXT_EAP_AUTHENTICATION_FLAGS")}, name="IKEEXT_EAP_AUTHENTICATION0", pack=False, align=None)}, name="<anon>", label="None")}, name="IKEEXT_AUTHENTICATION_METHOD1", pack=False, align=None), offset=0), "initiatorImpersonationType": SimTypeInt(signed=False, label="IKEEXT_AUTHENTICATION_IMPERSONATION_TYPE")}, name="IKEEXT_EM_POLICY1", pack=False, align=None), offset=0)}, name="IPSEC_TUNNEL_POLICY1", pack=False, align=None), offset=0), "ikeV2MmPolicy": SimTypePointer(SimStruct({"softExpirationTime": SimTypeInt(signed=False, label="UInt32"), "numAuthenticationMethods": SimTypeInt(signed=False, label="UInt32"), "authenticationMethods": SimTypePointer(SimStruct({"authenticationMethodType": SimTypeInt(signed=False, label="IKEEXT_AUTHENTICATION_METHOD_TYPE"), "Anonymous": SimUnion({"presharedKeyAuthentication": SimStruct({"presharedKey": SimStruct({"size": SimTypeInt(signed=False, label="UInt32"), "data": SimTypePointer(SimTypeChar(label="Byte"), offset=0)}, name="FWP_BYTE_BLOB", pack=False, align=None), "flags": SimTypeInt(signed=False, label="IKEEXT_PRESHARED_KEY_AUTHENTICATION_FLAGS")}, name="IKEEXT_PRESHARED_KEY_AUTHENTICATION1", pack=False, align=None), "certificateAuthentication": SimStruct({"inboundConfigType": SimTypeInt(signed=False, label="IKEEXT_CERT_CONFIG_TYPE"), "Anonymous1": SimUnion({"Anonymous": SimStruct({"inboundRootArraySize": SimTypeInt(signed=False, label="UInt32"), "inboundRootArray": SimTypePointer(SimStruct({"certData": SimStruct({"size": SimTypeInt(signed=False, label="UInt32"), "data": SimTypePointer(SimTypeChar(label="Byte"), offset=0)}, name="FWP_BYTE_BLOB", pack=False, align=None), "flags": SimTypeInt(signed=False, label="IKEEXT_CERT_FLAGS")}, name="IKEEXT_CERT_ROOT_CONFIG0", pack=False, align=None), offset=0)}, name="_Anonymous_e__Struct", pack=False, align=None), "inboundEnterpriseStoreConfig": SimTypePointer(SimStruct({"certData": SimStruct({"size": SimTypeInt(signed=False, label="UInt32"), "data": SimTypePointer(SimTypeChar(label="Byte"), offset=0)}, name="FWP_BYTE_BLOB", pack=False, align=None), "flags": SimTypeInt(signed=False, label="IKEEXT_CERT_FLAGS")}, name="IKEEXT_CERT_ROOT_CONFIG0", pack=False, align=None), offset=0), "inboundTrustedRootStoreConfig": SimTypePointer(SimStruct({"certData": SimStruct({"size": SimTypeInt(signed=False, label="UInt32"), "data": SimTypePointer(SimTypeChar(label="Byte"), offset=0)}, name="FWP_BYTE_BLOB", pack=False, align=None), "flags": SimTypeInt(signed=False, label="IKEEXT_CERT_FLAGS")}, name="IKEEXT_CERT_ROOT_CONFIG0", pack=False, align=None), offset=0)}, name="<anon>", label="None"), "outboundConfigType": SimTypeInt(signed=False, label="IKEEXT_CERT_CONFIG_TYPE"), "Anonymous2": SimUnion({"Anonymous": SimStruct({"outboundRootArraySize": SimTypeInt(signed=False, label="UInt32"), "outboundRootArray": SimTypePointer(SimStruct({"certData": SimStruct({"size": SimTypeInt(signed=False, label="UInt32"), "data": SimTypePointer(SimTypeChar(label="Byte"), offset=0)}, name="FWP_BYTE_BLOB", pack=False, align=None), "flags": SimTypeInt(signed=False, label="IKEEXT_CERT_FLAGS")}, name="IKEEXT_CERT_ROOT_CONFIG0", pack=False, align=None), offset=0)}, name="_Anonymous_e__Struct", pack=False, align=None), "outboundEnterpriseStoreConfig": SimTypePointer(SimStruct({"certData": SimStruct({"size": SimTypeInt(signed=False, label="UInt32"), "data": SimTypePointer(SimTypeChar(label="Byte"), offset=0)}, name="FWP_BYTE_BLOB", pack=False, align=None), "flags": SimTypeInt(signed=False, label="IKEEXT_CERT_FLAGS")}, name="IKEEXT_CERT_ROOT_CONFIG0", pack=False, align=None), offset=0), "outboundTrustedRootStoreConfig": SimTypePointer(SimStruct({"certData": SimStruct({"size": SimTypeInt(signed=False, label="UInt32"), "data": SimTypePointer(SimTypeChar(label="Byte"), offset=0)}, name="FWP_BYTE_BLOB", pack=False, align=None), "flags": SimTypeInt(signed=False, label="IKEEXT_CERT_FLAGS")}, name="IKEEXT_CERT_ROOT_CONFIG0", pack=False, align=None), offset=0)}, name="<anon>", label="None"), "flags": SimTypeInt(signed=False, label="IKEEXT_CERT_AUTH"), "localCertLocationUrl": SimStruct({"size": SimTypeInt(signed=False, label="UInt32"), "data": SimTypePointer(SimTypeChar(label="Byte"), offset=0)}, name="FWP_BYTE_BLOB", pack=False, align=None)}, name="IKEEXT_CERTIFICATE_AUTHENTICATION1", pack=False, align=None), "kerberosAuthentication": SimStruct({"flags": SimTypeInt(signed=False, label="IKEEXT_KERBEROS_AUTHENTICATION_FLAGS")}, name="IKEEXT_KERBEROS_AUTHENTICATION0", pack=False, align=None), "ntlmV2Authentication": SimStruct({"flags": SimTypeInt(signed=False, label="UInt32")}, name="IKEEXT_NTLM_V2_AUTHENTICATION0", pack=False, align=None), "sslAuthentication": SimStruct({"inboundConfigType": SimTypeInt(signed=False, label="IKEEXT_CERT_CONFIG_TYPE"), "Anonymous1": SimUnion({"Anonymous": SimStruct({"inboundRootArraySize": SimTypeInt(signed=False, label="UInt32"), "inboundRootArray": SimTypePointer(SimStruct({"certData": SimStruct({"size": SimTypeInt(signed=False, label="UInt32"), "data": SimTypePointer(SimTypeChar(label="Byte"), offset=0)}, name="FWP_BYTE_BLOB", pack=False, align=None), "flags": SimTypeInt(signed=False, label="IKEEXT_CERT_FLAGS")}, name="IKEEXT_CERT_ROOT_CONFIG0", pack=False, align=None), offset=0)}, name="_Anonymous_e__Struct", pack=False, align=None), "inboundEnterpriseStoreConfig": SimTypePointer(SimStruct({"certData": SimStruct({"size": SimTypeInt(signed=False, label="UInt32"), "data": SimTypePointer(SimTypeChar(label="Byte"), offset=0)}, name="FWP_BYTE_BLOB", pack=False, align=None), "flags": SimTypeInt(signed=False, label="IKEEXT_CERT_FLAGS")}, name="IKEEXT_CERT_ROOT_CONFIG0", pack=False, align=None), offset=0), "inboundTrustedRootStoreConfig": SimTypePointer(SimStruct({"certData": SimStruct({"size": SimTypeInt(signed=False, label="UInt32"), "data": SimTypePointer(SimTypeChar(label="Byte"), offset=0)}, name="FWP_BYTE_BLOB", pack=False, align=None), "flags": SimTypeInt(signed=False, label="IKEEXT_CERT_FLAGS")}, name="IKEEXT_CERT_ROOT_CONFIG0", pack=False, align=None), offset=0)}, name="<anon>", label="None"), "outboundConfigType": SimTypeInt(signed=False, label="IKEEXT_CERT_CONFIG_TYPE"), "Anonymous2": SimUnion({"Anonymous": SimStruct({"outboundRootArraySize": SimTypeInt(signed=False, label="UInt32"), "outboundRootArray": SimTypePointer(SimStruct({"certData": SimStruct({"size": SimTypeInt(signed=False, label="UInt32"), "data": SimTypePointer(SimTypeChar(label="Byte"), offset=0)}, name="FWP_BYTE_BLOB", pack=False, align=None), "flags": SimTypeInt(signed=False, label="IKEEXT_CERT_FLAGS")}, name="IKEEXT_CERT_ROOT_CONFIG0", pack=False, align=None), offset=0)}, name="_Anonymous_e__Struct", pack=False, align=None), "outboundEnterpriseStoreConfig": SimTypePointer(SimStruct({"certData": SimStruct({"size": SimTypeInt(signed=False, label="UInt32"), "data": SimTypePointer(SimTypeChar(label="Byte"), offset=0)}, name="FWP_BYTE_BLOB", pack=False, align=None), "flags": SimTypeInt(signed=False, label="IKEEXT_CERT_FLAGS")}, name="IKEEXT_CERT_ROOT_CONFIG0", pack=False, align=None), offset=0), "outboundTrustedRootStoreConfig": SimTypePointer(SimStruct({"certData": SimStruct({"size": SimTypeInt(signed=False, label="UInt32"), "data": SimTypePointer(SimTypeChar(label="Byte"), offset=0)}, name="FWP_BYTE_BLOB", pack=False, align=None), "flags": SimTypeInt(signed=False, label="IKEEXT_CERT_FLAGS")}, name="IKEEXT_CERT_ROOT_CONFIG0", pack=False, align=None), offset=0)}, name="<anon>", label="None"), "flags": SimTypeInt(signed=False, label="IKEEXT_CERT_AUTH"), "localCertLocationUrl": SimStruct({"size": SimTypeInt(signed=False, label="UInt32"), "data": SimTypePointer(SimTypeChar(label="Byte"), offset=0)}, name="FWP_BYTE_BLOB", pack=False, align=None)}, name="IKEEXT_CERTIFICATE_AUTHENTICATION1", pack=False, align=None), "cgaAuthentication": SimStruct({"keyContainerName": SimTypePointer(SimTypeChar(label="Char"), offset=0), "cspName": SimTypePointer(SimTypeChar(label="Char"), offset=0), "cspType": SimTypeInt(signed=False, label="UInt32"), "cgaModifier": SimStruct({"byteArray16": SimTypeFixedSizeArray(SimTypeChar(label="Byte"), 16)}, name="FWP_BYTE_ARRAY16", pack=False, align=None), "cgaCollisionCount": SimTypeChar(label="Byte")}, name="IKEEXT_IPV6_CGA_AUTHENTICATION0", pack=False, align=None), "eapAuthentication": SimStruct({"flags": SimTypeInt(signed=False, label="IKEEXT_EAP_AUTHENTICATION_FLAGS")}, name="IKEEXT_EAP_AUTHENTICATION0", pack=False, align=None)}, name="<anon>", label="None")}, name="IKEEXT_AUTHENTICATION_METHOD1", pack=False, align=None), offset=0), "initiatorImpersonationType": SimTypeInt(signed=False, label="IKEEXT_AUTHENTICATION_IMPERSONATION_TYPE"), "numIkeProposals": SimTypeInt(signed=False, label="UInt32"), "ikeProposals": SimTypePointer(SimStruct({"cipherAlgorithm": SimStruct({"algoIdentifier": SimTypeInt(signed=False, label="IKEEXT_CIPHER_TYPE"), "keyLen": SimTypeInt(signed=False, label="UInt32"), "rounds": SimTypeInt(signed=False, label="UInt32")}, name="IKEEXT_CIPHER_ALGORITHM0", pack=False, align=None), "integrityAlgorithm": SimStruct({"algoIdentifier": SimTypeInt(signed=False, label="IKEEXT_INTEGRITY_TYPE")}, name="IKEEXT_INTEGRITY_ALGORITHM0", pack=False, align=None), "maxLifetimeSeconds": SimTypeInt(signed=False, label="UInt32"), "dhGroup": SimTypeInt(signed=False, label="IKEEXT_DH_GROUP"), "quickModeLimit": SimTypeInt(signed=False, label="UInt32")}, name="IKEEXT_PROPOSAL0", pack=False, align=None), offset=0), "flags": SimTypeInt(signed=False, label="IKEEXT_POLICY_FLAG"), "maxDynamicFilters": SimTypeInt(signed=False, label="UInt32"), "retransmitDurationSecs": SimTypeInt(signed=False, label="UInt32")}, name="IKEEXT_POLICY1", pack=False, align=None), offset=0), "idpOptions": SimTypePointer(SimStruct({"stateIdleTimeoutSeconds": SimTypeInt(signed=False, label="UInt32"), "perIPRateLimitQueueIdleTimeoutSeconds": SimTypeInt(signed=False, label="UInt32"), "ipV6IPsecUnauthDscp": SimTypeChar(label="Byte"), "ipV6IPsecUnauthRateLimitBytesPerSec": SimTypeInt(signed=False, label="UInt32"), "ipV6IPsecUnauthPerIPRateLimitBytesPerSec": SimTypeInt(signed=False, label="UInt32"), "ipV6IPsecAuthDscp": SimTypeChar(label="Byte"), "ipV6IPsecAuthRateLimitBytesPerSec": SimTypeInt(signed=False, label="UInt32"), "icmpV6Dscp": SimTypeChar(label="Byte"), "icmpV6RateLimitBytesPerSec": SimTypeInt(signed=False, label="UInt32"), "ipV6FilterExemptDscp": SimTypeChar(label="Byte"), "ipV6FilterExemptRateLimitBytesPerSec": SimTypeInt(signed=False, label="UInt32"), "defBlockExemptDscp": SimTypeChar(label="Byte"), "defBlockExemptRateLimitBytesPerSec": SimTypeInt(signed=False, label="UInt32"), "maxStateEntries": SimTypeInt(signed=False, label="UInt32"), "maxPerIPRateLimitQueues": SimTypeInt(signed=False, label="UInt32"), "flags": SimTypeInt(signed=False, label="IPSEC_DOSP_FLAGS"), "numPublicIFLuids": SimTypeInt(signed=False, label="UInt32"), "publicIFLuids": SimTypePointer(SimTypeLongLong(signed=False, label="UInt64"), offset=0), "numInternalIFLuids": SimTypeInt(signed=False, label="UInt32"), "internalIFLuids": SimTypePointer(SimTypeLongLong(signed=False, label="UInt64"), offset=0), "publicV6AddrMask": SimStruct({"addr": SimTypeFixedSizeArray(SimTypeChar(label="Byte"), 16), "prefixLength": SimTypeChar(label="Byte")}, name="FWP_V6_ADDR_AND_MASK", pack=False, align=None), "internalV6AddrMask": SimStruct({"addr": SimTypeFixedSizeArray(SimTypeChar(label="Byte"), 16), "prefixLength": SimTypeChar(label="Byte")}, name="FWP_V6_ADDR_AND_MASK", pack=False, align=None)}, name="IPSEC_DOSP_OPTIONS0", pack=False, align=None), offset=0)}, name="<anon>", label="None"), "providerContextId": SimTypeLongLong(signed=False, label="UInt64")}, name="FWPM_PROVIDER_CONTEXT1", pack=False, align=None), offset=0), offset=0)], SimTypeInt(signed=False, label="UInt32"), arg_names=["engineHandle", "id", "providerContext"]),
        # 
        'FwpmProviderContextGetById2': SimTypeFunction([SimTypePointer(SimTypeInt(signed=True, label="Int"), label="IntPtr", offset=0), SimTypeLongLong(signed=False, label="UInt64"), SimTypePointer(SimTypePointer(SimStruct({"providerContextKey": SimTypeBottom(label="Guid"), "displayData": SimStruct({"name": SimTypePointer(SimTypeChar(label="Char"), offset=0), "description": SimTypePointer(SimTypeChar(label="Char"), offset=0)}, name="FWPM_DISPLAY_DATA0", pack=False, align=None), "flags": SimTypeInt(signed=False, label="UInt32"), "providerKey": SimTypePointer(SimTypeBottom(label="Guid"), offset=0), "providerData": SimStruct({"size": SimTypeInt(signed=False, label="UInt32"), "data": SimTypePointer(SimTypeChar(label="Byte"), offset=0)}, name="FWP_BYTE_BLOB", pack=False, align=None), "type": SimTypeInt(signed=False, label="FWPM_PROVIDER_CONTEXT_TYPE"), "Anonymous": SimUnion({"keyingPolicy": SimTypePointer(SimStruct({"numKeyMods": SimTypeInt(signed=False, label="UInt32"), "keyModKeys": SimTypePointer(SimTypeBottom(label="Guid"), offset=0), "flags": SimTypeInt(signed=False, label="UInt32")}, name="IPSEC_KEYING_POLICY1", pack=False, align=None), offset=0), "ikeQmTransportPolicy": SimTypePointer(SimStruct({"numIpsecProposals": SimTypeInt(signed=False, label="UInt32"), "ipsecProposals": SimTypePointer(SimStruct({"lifetime": SimStruct({"lifetimeSeconds": SimTypeInt(signed=False, label="UInt32"), "lifetimeKilobytes": SimTypeInt(signed=False, label="UInt32"), "lifetimePackets": SimTypeInt(signed=False, label="UInt32")}, name="IPSEC_SA_LIFETIME0", pack=False, align=None), "numSaTransforms": SimTypeInt(signed=False, label="UInt32"), "saTransforms": SimTypePointer(SimStruct({"ipsecTransformType": SimTypeInt(signed=False, label="IPSEC_TRANSFORM_TYPE"), "Anonymous": SimUnion({"ahTransform": SimTypePointer(SimStruct({"authTransformId": SimStruct({"authType": SimTypeInt(signed=False, label="IPSEC_AUTH_TYPE"), "authConfig": SimTypeChar(label="Byte")}, name="IPSEC_AUTH_TRANSFORM_ID0", pack=False, align=None), "cryptoModuleId": SimTypePointer(SimTypeBottom(label="Guid"), offset=0)}, name="IPSEC_AUTH_TRANSFORM0", pack=False, align=None), offset=0), "espAuthTransform": SimTypePointer(SimStruct({"authTransformId": SimStruct({"authType": SimTypeInt(signed=False, label="IPSEC_AUTH_TYPE"), "authConfig": SimTypeChar(label="Byte")}, name="IPSEC_AUTH_TRANSFORM_ID0", pack=False, align=None), "cryptoModuleId": SimTypePointer(SimTypeBottom(label="Guid"), offset=0)}, name="IPSEC_AUTH_TRANSFORM0", pack=False, align=None), offset=0), "espCipherTransform": SimTypePointer(SimStruct({"cipherTransformId": SimStruct({"cipherType": SimTypeInt(signed=False, label="IPSEC_CIPHER_TYPE"), "cipherConfig": SimTypeChar(label="Byte")}, name="IPSEC_CIPHER_TRANSFORM_ID0", pack=False, align=None), "cryptoModuleId": SimTypePointer(SimTypeBottom(label="Guid"), offset=0)}, name="IPSEC_CIPHER_TRANSFORM0", pack=False, align=None), offset=0), "espAuthAndCipherTransform": SimTypePointer(SimStruct({"authTransform": SimStruct({"authTransformId": SimStruct({"authType": SimTypeInt(signed=False, label="IPSEC_AUTH_TYPE"), "authConfig": SimTypeChar(label="Byte")}, name="IPSEC_AUTH_TRANSFORM_ID0", pack=False, align=None), "cryptoModuleId": SimTypePointer(SimTypeBottom(label="Guid"), offset=0)}, name="IPSEC_AUTH_TRANSFORM0", pack=False, align=None), "cipherTransform": SimStruct({"cipherTransformId": SimStruct({"cipherType": SimTypeInt(signed=False, label="IPSEC_CIPHER_TYPE"), "cipherConfig": SimTypeChar(label="Byte")}, name="IPSEC_CIPHER_TRANSFORM_ID0", pack=False, align=None), "cryptoModuleId": SimTypePointer(SimTypeBottom(label="Guid"), offset=0)}, name="IPSEC_CIPHER_TRANSFORM0", pack=False, align=None)}, name="IPSEC_AUTH_AND_CIPHER_TRANSFORM0", pack=False, align=None), offset=0), "espAuthFwTransform": SimTypePointer(SimStruct({"authTransformId": SimStruct({"authType": SimTypeInt(signed=False, label="IPSEC_AUTH_TYPE"), "authConfig": SimTypeChar(label="Byte")}, name="IPSEC_AUTH_TRANSFORM_ID0", pack=False, align=None), "cryptoModuleId": SimTypePointer(SimTypeBottom(label="Guid"), offset=0)}, name="IPSEC_AUTH_TRANSFORM0", pack=False, align=None), offset=0)}, name="<anon>", label="None")}, name="IPSEC_SA_TRANSFORM0", pack=False, align=None), offset=0), "pfsGroup": SimTypeInt(signed=False, label="IPSEC_PFS_GROUP")}, name="IPSEC_PROPOSAL0", pack=False, align=None), offset=0), "flags": SimTypeInt(signed=False, label="IPSEC_POLICY_FLAG"), "ndAllowClearTimeoutSeconds": SimTypeInt(signed=False, label="UInt32"), "saIdleTimeout": SimStruct({"idleTimeoutSeconds": SimTypeInt(signed=False, label="UInt32"), "idleTimeoutSecondsFailOver": SimTypeInt(signed=False, label="UInt32")}, name="IPSEC_SA_IDLE_TIMEOUT0", pack=False, align=None), "emPolicy": SimTypePointer(SimStruct({"numAuthenticationMethods": SimTypeInt(signed=False, label="UInt32"), "authenticationMethods": SimTypePointer(SimStruct({"authenticationMethodType": SimTypeInt(signed=False, label="IKEEXT_AUTHENTICATION_METHOD_TYPE"), "Anonymous": SimUnion({"presharedKeyAuthentication": SimStruct({"presharedKey": SimStruct({"size": SimTypeInt(signed=False, label="UInt32"), "data": SimTypePointer(SimTypeChar(label="Byte"), offset=0)}, name="FWP_BYTE_BLOB", pack=False, align=None), "flags": SimTypeInt(signed=False, label="IKEEXT_PRESHARED_KEY_AUTHENTICATION_FLAGS")}, name="IKEEXT_PRESHARED_KEY_AUTHENTICATION1", pack=False, align=None), "certificateAuthentication": SimStruct({"inboundConfigType": SimTypeInt(signed=False, label="IKEEXT_CERT_CONFIG_TYPE"), "Anonymous1": SimUnion({"Anonymous1": SimStruct({"inboundRootArraySize": SimTypeInt(signed=False, label="UInt32"), "inboundRootCriteria": SimTypePointer(SimStruct({"certData": SimStruct({"size": SimTypeInt(signed=False, label="UInt32"), "data": SimTypePointer(SimTypeChar(label="Byte"), offset=0)}, name="FWP_BYTE_BLOB", pack=False, align=None), "certHash": SimStruct({"size": SimTypeInt(signed=False, label="UInt32"), "data": SimTypePointer(SimTypeChar(label="Byte"), offset=0)}, name="FWP_BYTE_BLOB", pack=False, align=None), "eku": SimTypePointer(SimStruct({"numEku": SimTypeInt(signed=False, label="UInt32"), "eku": SimTypePointer(SimTypePointer(SimTypeChar(label="Byte"), offset=0), offset=0)}, name="IKEEXT_CERT_EKUS0", pack=False, align=None), offset=0), "name": SimTypePointer(SimStruct({"nameType": SimTypeInt(signed=False, label="IKEEXT_CERT_CRITERIA_NAME_TYPE"), "certName": SimTypePointer(SimTypeChar(label="Char"), offset=0)}, name="IKEEXT_CERT_NAME0", pack=False, align=None), offset=0), "flags": SimTypeInt(signed=False, label="UInt32")}, name="IKEEXT_CERTIFICATE_CRITERIA0", pack=False, align=None), offset=0)}, name="_Anonymous1_e__Struct", pack=False, align=None), "Anonymous2": SimStruct({"inboundEnterpriseStoreArraySize": SimTypeInt(signed=False, label="UInt32"), "inboundEnterpriseStoreCriteria": SimTypePointer(SimStruct({"certData": SimStruct({"size": SimTypeInt(signed=False, label="UInt32"), "data": SimTypePointer(SimTypeChar(label="Byte"), offset=0)}, name="FWP_BYTE_BLOB", pack=False, align=None), "certHash": SimStruct({"size": SimTypeInt(signed=False, label="UInt32"), "data": SimTypePointer(SimTypeChar(label="Byte"), offset=0)}, name="FWP_BYTE_BLOB", pack=False, align=None), "eku": SimTypePointer(SimStruct({"numEku": SimTypeInt(signed=False, label="UInt32"), "eku": SimTypePointer(SimTypePointer(SimTypeChar(label="Byte"), offset=0), offset=0)}, name="IKEEXT_CERT_EKUS0", pack=False, align=None), offset=0), "name": SimTypePointer(SimStruct({"nameType": SimTypeInt(signed=False, label="IKEEXT_CERT_CRITERIA_NAME_TYPE"), "certName": SimTypePointer(SimTypeChar(label="Char"), offset=0)}, name="IKEEXT_CERT_NAME0", pack=False, align=None), offset=0), "flags": SimTypeInt(signed=False, label="UInt32")}, name="IKEEXT_CERTIFICATE_CRITERIA0", pack=False, align=None), offset=0)}, name="_Anonymous2_e__Struct", pack=False, align=None), "Anonymous3": SimStruct({"inboundRootStoreArraySize": SimTypeInt(signed=False, label="UInt32"), "inboundTrustedRootStoreCriteria": SimTypePointer(SimStruct({"certData": SimStruct({"size": SimTypeInt(signed=False, label="UInt32"), "data": SimTypePointer(SimTypeChar(label="Byte"), offset=0)}, name="FWP_BYTE_BLOB", pack=False, align=None), "certHash": SimStruct({"size": SimTypeInt(signed=False, label="UInt32"), "data": SimTypePointer(SimTypeChar(label="Byte"), offset=0)}, name="FWP_BYTE_BLOB", pack=False, align=None), "eku": SimTypePointer(SimStruct({"numEku": SimTypeInt(signed=False, label="UInt32"), "eku": SimTypePointer(SimTypePointer(SimTypeChar(label="Byte"), offset=0), offset=0)}, name="IKEEXT_CERT_EKUS0", pack=False, align=None), offset=0), "name": SimTypePointer(SimStruct({"nameType": SimTypeInt(signed=False, label="IKEEXT_CERT_CRITERIA_NAME_TYPE"), "certName": SimTypePointer(SimTypeChar(label="Char"), offset=0)}, name="IKEEXT_CERT_NAME0", pack=False, align=None), offset=0), "flags": SimTypeInt(signed=False, label="UInt32")}, name="IKEEXT_CERTIFICATE_CRITERIA0", pack=False, align=None), offset=0)}, name="_Anonymous3_e__Struct", pack=False, align=None)}, name="<anon>", label="None"), "outboundConfigType": SimTypeInt(signed=False, label="IKEEXT_CERT_CONFIG_TYPE"), "Anonymous2": SimUnion({"Anonymous1": SimStruct({"outboundRootArraySize": SimTypeInt(signed=False, label="UInt32"), "outboundRootCriteria": SimTypePointer(SimStruct({"certData": SimStruct({"size": SimTypeInt(signed=False, label="UInt32"), "data": SimTypePointer(SimTypeChar(label="Byte"), offset=0)}, name="FWP_BYTE_BLOB", pack=False, align=None), "certHash": SimStruct({"size": SimTypeInt(signed=False, label="UInt32"), "data": SimTypePointer(SimTypeChar(label="Byte"), offset=0)}, name="FWP_BYTE_BLOB", pack=False, align=None), "eku": SimTypePointer(SimStruct({"numEku": SimTypeInt(signed=False, label="UInt32"), "eku": SimTypePointer(SimTypePointer(SimTypeChar(label="Byte"), offset=0), offset=0)}, name="IKEEXT_CERT_EKUS0", pack=False, align=None), offset=0), "name": SimTypePointer(SimStruct({"nameType": SimTypeInt(signed=False, label="IKEEXT_CERT_CRITERIA_NAME_TYPE"), "certName": SimTypePointer(SimTypeChar(label="Char"), offset=0)}, name="IKEEXT_CERT_NAME0", pack=False, align=None), offset=0), "flags": SimTypeInt(signed=False, label="UInt32")}, name="IKEEXT_CERTIFICATE_CRITERIA0", pack=False, align=None), offset=0)}, name="_Anonymous1_e__Struct", pack=False, align=None), "Anonymous2": SimStruct({"outboundEnterpriseStoreArraySize": SimTypeInt(signed=False, label="UInt32"), "outboundEnterpriseStoreCriteria": SimTypePointer(SimStruct({"certData": SimStruct({"size": SimTypeInt(signed=False, label="UInt32"), "data": SimTypePointer(SimTypeChar(label="Byte"), offset=0)}, name="FWP_BYTE_BLOB", pack=False, align=None), "certHash": SimStruct({"size": SimTypeInt(signed=False, label="UInt32"), "data": SimTypePointer(SimTypeChar(label="Byte"), offset=0)}, name="FWP_BYTE_BLOB", pack=False, align=None), "eku": SimTypePointer(SimStruct({"numEku": SimTypeInt(signed=False, label="UInt32"), "eku": SimTypePointer(SimTypePointer(SimTypeChar(label="Byte"), offset=0), offset=0)}, name="IKEEXT_CERT_EKUS0", pack=False, align=None), offset=0), "name": SimTypePointer(SimStruct({"nameType": SimTypeInt(signed=False, label="IKEEXT_CERT_CRITERIA_NAME_TYPE"), "certName": SimTypePointer(SimTypeChar(label="Char"), offset=0)}, name="IKEEXT_CERT_NAME0", pack=False, align=None), offset=0), "flags": SimTypeInt(signed=False, label="UInt32")}, name="IKEEXT_CERTIFICATE_CRITERIA0", pack=False, align=None), offset=0)}, name="_Anonymous2_e__Struct", pack=False, align=None), "Anonymous3": SimStruct({"outboundRootStoreArraySize": SimTypeInt(signed=False, label="UInt32"), "outboundTrustedRootStoreCriteria": SimTypePointer(SimStruct({"certData": SimStruct({"size": SimTypeInt(signed=False, label="UInt32"), "data": SimTypePointer(SimTypeChar(label="Byte"), offset=0)}, name="FWP_BYTE_BLOB", pack=False, align=None), "certHash": SimStruct({"size": SimTypeInt(signed=False, label="UInt32"), "data": SimTypePointer(SimTypeChar(label="Byte"), offset=0)}, name="FWP_BYTE_BLOB", pack=False, align=None), "eku": SimTypePointer(SimStruct({"numEku": SimTypeInt(signed=False, label="UInt32"), "eku": SimTypePointer(SimTypePointer(SimTypeChar(label="Byte"), offset=0), offset=0)}, name="IKEEXT_CERT_EKUS0", pack=False, align=None), offset=0), "name": SimTypePointer(SimStruct({"nameType": SimTypeInt(signed=False, label="IKEEXT_CERT_CRITERIA_NAME_TYPE"), "certName": SimTypePointer(SimTypeChar(label="Char"), offset=0)}, name="IKEEXT_CERT_NAME0", pack=False, align=None), offset=0), "flags": SimTypeInt(signed=False, label="UInt32")}, name="IKEEXT_CERTIFICATE_CRITERIA0", pack=False, align=None), offset=0)}, name="_Anonymous3_e__Struct", pack=False, align=None)}, name="<anon>", label="None"), "flags": SimTypeInt(signed=False, label="IKEEXT_CERT_AUTH"), "localCertLocationUrl": SimStruct({"size": SimTypeInt(signed=False, label="UInt32"), "data": SimTypePointer(SimTypeChar(label="Byte"), offset=0)}, name="FWP_BYTE_BLOB", pack=False, align=None)}, name="IKEEXT_CERTIFICATE_AUTHENTICATION2", pack=False, align=None), "kerberosAuthentication": SimStruct({"flags": SimTypeInt(signed=False, label="IKEEXT_KERBEROS_AUTHENTICATION_FLAGS"), "proxyServer": SimTypePointer(SimTypeChar(label="Char"), offset=0)}, name="IKEEXT_KERBEROS_AUTHENTICATION1", pack=False, align=None), "reservedAuthentication": SimStruct({"flags": SimTypeInt(signed=False, label="IKEEXT_RESERVED_AUTHENTICATION_FLAGS")}, name="IKEEXT_RESERVED_AUTHENTICATION0", pack=False, align=None), "ntlmV2Authentication": SimStruct({"flags": SimTypeInt(signed=False, label="UInt32")}, name="IKEEXT_NTLM_V2_AUTHENTICATION0", pack=False, align=None), "sslAuthentication": SimStruct({"inboundConfigType": SimTypeInt(signed=False, label="IKEEXT_CERT_CONFIG_TYPE"), "Anonymous1": SimUnion({"Anonymous1": SimStruct({"inboundRootArraySize": SimTypeInt(signed=False, label="UInt32"), "inboundRootCriteria": SimTypePointer(SimStruct({"certData": SimStruct({"size": SimTypeInt(signed=False, label="UInt32"), "data": SimTypePointer(SimTypeChar(label="Byte"), offset=0)}, name="FWP_BYTE_BLOB", pack=False, align=None), "certHash": SimStruct({"size": SimTypeInt(signed=False, label="UInt32"), "data": SimTypePointer(SimTypeChar(label="Byte"), offset=0)}, name="FWP_BYTE_BLOB", pack=False, align=None), "eku": SimTypePointer(SimStruct({"numEku": SimTypeInt(signed=False, label="UInt32"), "eku": SimTypePointer(SimTypePointer(SimTypeChar(label="Byte"), offset=0), offset=0)}, name="IKEEXT_CERT_EKUS0", pack=False, align=None), offset=0), "name": SimTypePointer(SimStruct({"nameType": SimTypeInt(signed=False, label="IKEEXT_CERT_CRITERIA_NAME_TYPE"), "certName": SimTypePointer(SimTypeChar(label="Char"), offset=0)}, name="IKEEXT_CERT_NAME0", pack=False, align=None), offset=0), "flags": SimTypeInt(signed=False, label="UInt32")}, name="IKEEXT_CERTIFICATE_CRITERIA0", pack=False, align=None), offset=0)}, name="_Anonymous1_e__Struct", pack=False, align=None), "Anonymous2": SimStruct({"inboundEnterpriseStoreArraySize": SimTypeInt(signed=False, label="UInt32"), "inboundEnterpriseStoreCriteria": SimTypePointer(SimStruct({"certData": SimStruct({"size": SimTypeInt(signed=False, label="UInt32"), "data": SimTypePointer(SimTypeChar(label="Byte"), offset=0)}, name="FWP_BYTE_BLOB", pack=False, align=None), "certHash": SimStruct({"size": SimTypeInt(signed=False, label="UInt32"), "data": SimTypePointer(SimTypeChar(label="Byte"), offset=0)}, name="FWP_BYTE_BLOB", pack=False, align=None), "eku": SimTypePointer(SimStruct({"numEku": SimTypeInt(signed=False, label="UInt32"), "eku": SimTypePointer(SimTypePointer(SimTypeChar(label="Byte"), offset=0), offset=0)}, name="IKEEXT_CERT_EKUS0", pack=False, align=None), offset=0), "name": SimTypePointer(SimStruct({"nameType": SimTypeInt(signed=False, label="IKEEXT_CERT_CRITERIA_NAME_TYPE"), "certName": SimTypePointer(SimTypeChar(label="Char"), offset=0)}, name="IKEEXT_CERT_NAME0", pack=False, align=None), offset=0), "flags": SimTypeInt(signed=False, label="UInt32")}, name="IKEEXT_CERTIFICATE_CRITERIA0", pack=False, align=None), offset=0)}, name="_Anonymous2_e__Struct", pack=False, align=None), "Anonymous3": SimStruct({"inboundRootStoreArraySize": SimTypeInt(signed=False, label="UInt32"), "inboundTrustedRootStoreCriteria": SimTypePointer(SimStruct({"certData": SimStruct({"size": SimTypeInt(signed=False, label="UInt32"), "data": SimTypePointer(SimTypeChar(label="Byte"), offset=0)}, name="FWP_BYTE_BLOB", pack=False, align=None), "certHash": SimStruct({"size": SimTypeInt(signed=False, label="UInt32"), "data": SimTypePointer(SimTypeChar(label="Byte"), offset=0)}, name="FWP_BYTE_BLOB", pack=False, align=None), "eku": SimTypePointer(SimStruct({"numEku": SimTypeInt(signed=False, label="UInt32"), "eku": SimTypePointer(SimTypePointer(SimTypeChar(label="Byte"), offset=0), offset=0)}, name="IKEEXT_CERT_EKUS0", pack=False, align=None), offset=0), "name": SimTypePointer(SimStruct({"nameType": SimTypeInt(signed=False, label="IKEEXT_CERT_CRITERIA_NAME_TYPE"), "certName": SimTypePointer(SimTypeChar(label="Char"), offset=0)}, name="IKEEXT_CERT_NAME0", pack=False, align=None), offset=0), "flags": SimTypeInt(signed=False, label="UInt32")}, name="IKEEXT_CERTIFICATE_CRITERIA0", pack=False, align=None), offset=0)}, name="_Anonymous3_e__Struct", pack=False, align=None)}, name="<anon>", label="None"), "outboundConfigType": SimTypeInt(signed=False, label="IKEEXT_CERT_CONFIG_TYPE"), "Anonymous2": SimUnion({"Anonymous1": SimStruct({"outboundRootArraySize": SimTypeInt(signed=False, label="UInt32"), "outboundRootCriteria": SimTypePointer(SimStruct({"certData": SimStruct({"size": SimTypeInt(signed=False, label="UInt32"), "data": SimTypePointer(SimTypeChar(label="Byte"), offset=0)}, name="FWP_BYTE_BLOB", pack=False, align=None), "certHash": SimStruct({"size": SimTypeInt(signed=False, label="UInt32"), "data": SimTypePointer(SimTypeChar(label="Byte"), offset=0)}, name="FWP_BYTE_BLOB", pack=False, align=None), "eku": SimTypePointer(SimStruct({"numEku": SimTypeInt(signed=False, label="UInt32"), "eku": SimTypePointer(SimTypePointer(SimTypeChar(label="Byte"), offset=0), offset=0)}, name="IKEEXT_CERT_EKUS0", pack=False, align=None), offset=0), "name": SimTypePointer(SimStruct({"nameType": SimTypeInt(signed=False, label="IKEEXT_CERT_CRITERIA_NAME_TYPE"), "certName": SimTypePointer(SimTypeChar(label="Char"), offset=0)}, name="IKEEXT_CERT_NAME0", pack=False, align=None), offset=0), "flags": SimTypeInt(signed=False, label="UInt32")}, name="IKEEXT_CERTIFICATE_CRITERIA0", pack=False, align=None), offset=0)}, name="_Anonymous1_e__Struct", pack=False, align=None), "Anonymous2": SimStruct({"outboundEnterpriseStoreArraySize": SimTypeInt(signed=False, label="UInt32"), "outboundEnterpriseStoreCriteria": SimTypePointer(SimStruct({"certData": SimStruct({"size": SimTypeInt(signed=False, label="UInt32"), "data": SimTypePointer(SimTypeChar(label="Byte"), offset=0)}, name="FWP_BYTE_BLOB", pack=False, align=None), "certHash": SimStruct({"size": SimTypeInt(signed=False, label="UInt32"), "data": SimTypePointer(SimTypeChar(label="Byte"), offset=0)}, name="FWP_BYTE_BLOB", pack=False, align=None), "eku": SimTypePointer(SimStruct({"numEku": SimTypeInt(signed=False, label="UInt32"), "eku": SimTypePointer(SimTypePointer(SimTypeChar(label="Byte"), offset=0), offset=0)}, name="IKEEXT_CERT_EKUS0", pack=False, align=None), offset=0), "name": SimTypePointer(SimStruct({"nameType": SimTypeInt(signed=False, label="IKEEXT_CERT_CRITERIA_NAME_TYPE"), "certName": SimTypePointer(SimTypeChar(label="Char"), offset=0)}, name="IKEEXT_CERT_NAME0", pack=False, align=None), offset=0), "flags": SimTypeInt(signed=False, label="UInt32")}, name="IKEEXT_CERTIFICATE_CRITERIA0", pack=False, align=None), offset=0)}, name="_Anonymous2_e__Struct", pack=False, align=None), "Anonymous3": SimStruct({"outboundRootStoreArraySize": SimTypeInt(signed=False, label="UInt32"), "outboundTrustedRootStoreCriteria": SimTypePointer(SimStruct({"certData": SimStruct({"size": SimTypeInt(signed=False, label="UInt32"), "data": SimTypePointer(SimTypeChar(label="Byte"), offset=0)}, name="FWP_BYTE_BLOB", pack=False, align=None), "certHash": SimStruct({"size": SimTypeInt(signed=False, label="UInt32"), "data": SimTypePointer(SimTypeChar(label="Byte"), offset=0)}, name="FWP_BYTE_BLOB", pack=False, align=None), "eku": SimTypePointer(SimStruct({"numEku": SimTypeInt(signed=False, label="UInt32"), "eku": SimTypePointer(SimTypePointer(SimTypeChar(label="Byte"), offset=0), offset=0)}, name="IKEEXT_CERT_EKUS0", pack=False, align=None), offset=0), "name": SimTypePointer(SimStruct({"nameType": SimTypeInt(signed=False, label="IKEEXT_CERT_CRITERIA_NAME_TYPE"), "certName": SimTypePointer(SimTypeChar(label="Char"), offset=0)}, name="IKEEXT_CERT_NAME0", pack=False, align=None), offset=0), "flags": SimTypeInt(signed=False, label="UInt32")}, name="IKEEXT_CERTIFICATE_CRITERIA0", pack=False, align=None), offset=0)}, name="_Anonymous3_e__Struct", pack=False, align=None)}, name="<anon>", label="None"), "flags": SimTypeInt(signed=False, label="IKEEXT_CERT_AUTH"), "localCertLocationUrl": SimStruct({"size": SimTypeInt(signed=False, label="UInt32"), "data": SimTypePointer(SimTypeChar(label="Byte"), offset=0)}, name="FWP_BYTE_BLOB", pack=False, align=None)}, name="IKEEXT_CERTIFICATE_AUTHENTICATION2", pack=False, align=None), "cgaAuthentication": SimStruct({"keyContainerName": SimTypePointer(SimTypeChar(label="Char"), offset=0), "cspName": SimTypePointer(SimTypeChar(label="Char"), offset=0), "cspType": SimTypeInt(signed=False, label="UInt32"), "cgaModifier": SimStruct({"byteArray16": SimTypeFixedSizeArray(SimTypeChar(label="Byte"), 16)}, name="FWP_BYTE_ARRAY16", pack=False, align=None), "cgaCollisionCount": SimTypeChar(label="Byte")}, name="IKEEXT_IPV6_CGA_AUTHENTICATION0", pack=False, align=None), "eapAuthentication": SimStruct({"flags": SimTypeInt(signed=False, label="IKEEXT_EAP_AUTHENTICATION_FLAGS")}, name="IKEEXT_EAP_AUTHENTICATION0", pack=False, align=None)}, name="<anon>", label="None")}, name="IKEEXT_AUTHENTICATION_METHOD2", pack=False, align=None), offset=0), "initiatorImpersonationType": SimTypeInt(signed=False, label="IKEEXT_AUTHENTICATION_IMPERSONATION_TYPE")}, name="IKEEXT_EM_POLICY2", pack=False, align=None), offset=0)}, name="IPSEC_TRANSPORT_POLICY2", pack=False, align=None), offset=0), "ikeQmTunnelPolicy": SimTypePointer(SimStruct({"flags": SimTypeInt(signed=False, label="IPSEC_POLICY_FLAG"), "numIpsecProposals": SimTypeInt(signed=False, label="UInt32"), "ipsecProposals": SimTypePointer(SimStruct({"lifetime": SimStruct({"lifetimeSeconds": SimTypeInt(signed=False, label="UInt32"), "lifetimeKilobytes": SimTypeInt(signed=False, label="UInt32"), "lifetimePackets": SimTypeInt(signed=False, label="UInt32")}, name="IPSEC_SA_LIFETIME0", pack=False, align=None), "numSaTransforms": SimTypeInt(signed=False, label="UInt32"), "saTransforms": SimTypePointer(SimStruct({"ipsecTransformType": SimTypeInt(signed=False, label="IPSEC_TRANSFORM_TYPE"), "Anonymous": SimUnion({"ahTransform": SimTypePointer(SimStruct({"authTransformId": SimStruct({"authType": SimTypeInt(signed=False, label="IPSEC_AUTH_TYPE"), "authConfig": SimTypeChar(label="Byte")}, name="IPSEC_AUTH_TRANSFORM_ID0", pack=False, align=None), "cryptoModuleId": SimTypePointer(SimTypeBottom(label="Guid"), offset=0)}, name="IPSEC_AUTH_TRANSFORM0", pack=False, align=None), offset=0), "espAuthTransform": SimTypePointer(SimStruct({"authTransformId": SimStruct({"authType": SimTypeInt(signed=False, label="IPSEC_AUTH_TYPE"), "authConfig": SimTypeChar(label="Byte")}, name="IPSEC_AUTH_TRANSFORM_ID0", pack=False, align=None), "cryptoModuleId": SimTypePointer(SimTypeBottom(label="Guid"), offset=0)}, name="IPSEC_AUTH_TRANSFORM0", pack=False, align=None), offset=0), "espCipherTransform": SimTypePointer(SimStruct({"cipherTransformId": SimStruct({"cipherType": SimTypeInt(signed=False, label="IPSEC_CIPHER_TYPE"), "cipherConfig": SimTypeChar(label="Byte")}, name="IPSEC_CIPHER_TRANSFORM_ID0", pack=False, align=None), "cryptoModuleId": SimTypePointer(SimTypeBottom(label="Guid"), offset=0)}, name="IPSEC_CIPHER_TRANSFORM0", pack=False, align=None), offset=0), "espAuthAndCipherTransform": SimTypePointer(SimStruct({"authTransform": SimStruct({"authTransformId": SimStruct({"authType": SimTypeInt(signed=False, label="IPSEC_AUTH_TYPE"), "authConfig": SimTypeChar(label="Byte")}, name="IPSEC_AUTH_TRANSFORM_ID0", pack=False, align=None), "cryptoModuleId": SimTypePointer(SimTypeBottom(label="Guid"), offset=0)}, name="IPSEC_AUTH_TRANSFORM0", pack=False, align=None), "cipherTransform": SimStruct({"cipherTransformId": SimStruct({"cipherType": SimTypeInt(signed=False, label="IPSEC_CIPHER_TYPE"), "cipherConfig": SimTypeChar(label="Byte")}, name="IPSEC_CIPHER_TRANSFORM_ID0", pack=False, align=None), "cryptoModuleId": SimTypePointer(SimTypeBottom(label="Guid"), offset=0)}, name="IPSEC_CIPHER_TRANSFORM0", pack=False, align=None)}, name="IPSEC_AUTH_AND_CIPHER_TRANSFORM0", pack=False, align=None), offset=0), "espAuthFwTransform": SimTypePointer(SimStruct({"authTransformId": SimStruct({"authType": SimTypeInt(signed=False, label="IPSEC_AUTH_TYPE"), "authConfig": SimTypeChar(label="Byte")}, name="IPSEC_AUTH_TRANSFORM_ID0", pack=False, align=None), "cryptoModuleId": SimTypePointer(SimTypeBottom(label="Guid"), offset=0)}, name="IPSEC_AUTH_TRANSFORM0", pack=False, align=None), offset=0)}, name="<anon>", label="None")}, name="IPSEC_SA_TRANSFORM0", pack=False, align=None), offset=0), "pfsGroup": SimTypeInt(signed=False, label="IPSEC_PFS_GROUP")}, name="IPSEC_PROPOSAL0", pack=False, align=None), offset=0), "tunnelEndpoints": SimStruct({"ipVersion": SimTypeInt(signed=False, label="FWP_IP_VERSION"), "Anonymous1": SimUnion({"localV4Address": SimTypeInt(signed=False, label="UInt32"), "localV6Address": SimTypeFixedSizeArray(SimTypeChar(label="Byte"), 16)}, name="<anon>", label="None"), "Anonymous2": SimUnion({"remoteV4Address": SimTypeInt(signed=False, label="UInt32"), "remoteV6Address": SimTypeFixedSizeArray(SimTypeChar(label="Byte"), 16)}, name="<anon>", label="None"), "localIfLuid": SimTypeLongLong(signed=False, label="UInt64"), "remoteFqdn": SimTypePointer(SimTypeChar(label="Char"), offset=0), "numAddresses": SimTypeInt(signed=False, label="UInt32"), "remoteAddresses": SimTypePointer(SimStruct({"ipVersion": SimTypeInt(signed=False, label="FWP_IP_VERSION"), "Anonymous": SimUnion({"v4Address": SimTypeInt(signed=False, label="UInt32"), "v6Address": SimTypeFixedSizeArray(SimTypeChar(label="Byte"), 16)}, name="<anon>", label="None")}, name="IPSEC_TUNNEL_ENDPOINT0", pack=False, align=None), offset=0)}, name="IPSEC_TUNNEL_ENDPOINTS2", pack=False, align=None), "saIdleTimeout": SimStruct({"idleTimeoutSeconds": SimTypeInt(signed=False, label="UInt32"), "idleTimeoutSecondsFailOver": SimTypeInt(signed=False, label="UInt32")}, name="IPSEC_SA_IDLE_TIMEOUT0", pack=False, align=None), "emPolicy": SimTypePointer(SimStruct({"numAuthenticationMethods": SimTypeInt(signed=False, label="UInt32"), "authenticationMethods": SimTypePointer(SimStruct({"authenticationMethodType": SimTypeInt(signed=False, label="IKEEXT_AUTHENTICATION_METHOD_TYPE"), "Anonymous": SimUnion({"presharedKeyAuthentication": SimStruct({"presharedKey": SimStruct({"size": SimTypeInt(signed=False, label="UInt32"), "data": SimTypePointer(SimTypeChar(label="Byte"), offset=0)}, name="FWP_BYTE_BLOB", pack=False, align=None), "flags": SimTypeInt(signed=False, label="IKEEXT_PRESHARED_KEY_AUTHENTICATION_FLAGS")}, name="IKEEXT_PRESHARED_KEY_AUTHENTICATION1", pack=False, align=None), "certificateAuthentication": SimStruct({"inboundConfigType": SimTypeInt(signed=False, label="IKEEXT_CERT_CONFIG_TYPE"), "Anonymous1": SimUnion({"Anonymous1": SimStruct({"inboundRootArraySize": SimTypeInt(signed=False, label="UInt32"), "inboundRootCriteria": SimTypePointer(SimStruct({"certData": SimStruct({"size": SimTypeInt(signed=False, label="UInt32"), "data": SimTypePointer(SimTypeChar(label="Byte"), offset=0)}, name="FWP_BYTE_BLOB", pack=False, align=None), "certHash": SimStruct({"size": SimTypeInt(signed=False, label="UInt32"), "data": SimTypePointer(SimTypeChar(label="Byte"), offset=0)}, name="FWP_BYTE_BLOB", pack=False, align=None), "eku": SimTypePointer(SimStruct({"numEku": SimTypeInt(signed=False, label="UInt32"), "eku": SimTypePointer(SimTypePointer(SimTypeChar(label="Byte"), offset=0), offset=0)}, name="IKEEXT_CERT_EKUS0", pack=False, align=None), offset=0), "name": SimTypePointer(SimStruct({"nameType": SimTypeInt(signed=False, label="IKEEXT_CERT_CRITERIA_NAME_TYPE"), "certName": SimTypePointer(SimTypeChar(label="Char"), offset=0)}, name="IKEEXT_CERT_NAME0", pack=False, align=None), offset=0), "flags": SimTypeInt(signed=False, label="UInt32")}, name="IKEEXT_CERTIFICATE_CRITERIA0", pack=False, align=None), offset=0)}, name="_Anonymous1_e__Struct", pack=False, align=None), "Anonymous2": SimStruct({"inboundEnterpriseStoreArraySize": SimTypeInt(signed=False, label="UInt32"), "inboundEnterpriseStoreCriteria": SimTypePointer(SimStruct({"certData": SimStruct({"size": SimTypeInt(signed=False, label="UInt32"), "data": SimTypePointer(SimTypeChar(label="Byte"), offset=0)}, name="FWP_BYTE_BLOB", pack=False, align=None), "certHash": SimStruct({"size": SimTypeInt(signed=False, label="UInt32"), "data": SimTypePointer(SimTypeChar(label="Byte"), offset=0)}, name="FWP_BYTE_BLOB", pack=False, align=None), "eku": SimTypePointer(SimStruct({"numEku": SimTypeInt(signed=False, label="UInt32"), "eku": SimTypePointer(SimTypePointer(SimTypeChar(label="Byte"), offset=0), offset=0)}, name="IKEEXT_CERT_EKUS0", pack=False, align=None), offset=0), "name": SimTypePointer(SimStruct({"nameType": SimTypeInt(signed=False, label="IKEEXT_CERT_CRITERIA_NAME_TYPE"), "certName": SimTypePointer(SimTypeChar(label="Char"), offset=0)}, name="IKEEXT_CERT_NAME0", pack=False, align=None), offset=0), "flags": SimTypeInt(signed=False, label="UInt32")}, name="IKEEXT_CERTIFICATE_CRITERIA0", pack=False, align=None), offset=0)}, name="_Anonymous2_e__Struct", pack=False, align=None), "Anonymous3": SimStruct({"inboundRootStoreArraySize": SimTypeInt(signed=False, label="UInt32"), "inboundTrustedRootStoreCriteria": SimTypePointer(SimStruct({"certData": SimStruct({"size": SimTypeInt(signed=False, label="UInt32"), "data": SimTypePointer(SimTypeChar(label="Byte"), offset=0)}, name="FWP_BYTE_BLOB", pack=False, align=None), "certHash": SimStruct({"size": SimTypeInt(signed=False, label="UInt32"), "data": SimTypePointer(SimTypeChar(label="Byte"), offset=0)}, name="FWP_BYTE_BLOB", pack=False, align=None), "eku": SimTypePointer(SimStruct({"numEku": SimTypeInt(signed=False, label="UInt32"), "eku": SimTypePointer(SimTypePointer(SimTypeChar(label="Byte"), offset=0), offset=0)}, name="IKEEXT_CERT_EKUS0", pack=False, align=None), offset=0), "name": SimTypePointer(SimStruct({"nameType": SimTypeInt(signed=False, label="IKEEXT_CERT_CRITERIA_NAME_TYPE"), "certName": SimTypePointer(SimTypeChar(label="Char"), offset=0)}, name="IKEEXT_CERT_NAME0", pack=False, align=None), offset=0), "flags": SimTypeInt(signed=False, label="UInt32")}, name="IKEEXT_CERTIFICATE_CRITERIA0", pack=False, align=None), offset=0)}, name="_Anonymous3_e__Struct", pack=False, align=None)}, name="<anon>", label="None"), "outboundConfigType": SimTypeInt(signed=False, label="IKEEXT_CERT_CONFIG_TYPE"), "Anonymous2": SimUnion({"Anonymous1": SimStruct({"outboundRootArraySize": SimTypeInt(signed=False, label="UInt32"), "outboundRootCriteria": SimTypePointer(SimStruct({"certData": SimStruct({"size": SimTypeInt(signed=False, label="UInt32"), "data": SimTypePointer(SimTypeChar(label="Byte"), offset=0)}, name="FWP_BYTE_BLOB", pack=False, align=None), "certHash": SimStruct({"size": SimTypeInt(signed=False, label="UInt32"), "data": SimTypePointer(SimTypeChar(label="Byte"), offset=0)}, name="FWP_BYTE_BLOB", pack=False, align=None), "eku": SimTypePointer(SimStruct({"numEku": SimTypeInt(signed=False, label="UInt32"), "eku": SimTypePointer(SimTypePointer(SimTypeChar(label="Byte"), offset=0), offset=0)}, name="IKEEXT_CERT_EKUS0", pack=False, align=None), offset=0), "name": SimTypePointer(SimStruct({"nameType": SimTypeInt(signed=False, label="IKEEXT_CERT_CRITERIA_NAME_TYPE"), "certName": SimTypePointer(SimTypeChar(label="Char"), offset=0)}, name="IKEEXT_CERT_NAME0", pack=False, align=None), offset=0), "flags": SimTypeInt(signed=False, label="UInt32")}, name="IKEEXT_CERTIFICATE_CRITERIA0", pack=False, align=None), offset=0)}, name="_Anonymous1_e__Struct", pack=False, align=None), "Anonymous2": SimStruct({"outboundEnterpriseStoreArraySize": SimTypeInt(signed=False, label="UInt32"), "outboundEnterpriseStoreCriteria": SimTypePointer(SimStruct({"certData": SimStruct({"size": SimTypeInt(signed=False, label="UInt32"), "data": SimTypePointer(SimTypeChar(label="Byte"), offset=0)}, name="FWP_BYTE_BLOB", pack=False, align=None), "certHash": SimStruct({"size": SimTypeInt(signed=False, label="UInt32"), "data": SimTypePointer(SimTypeChar(label="Byte"), offset=0)}, name="FWP_BYTE_BLOB", pack=False, align=None), "eku": SimTypePointer(SimStruct({"numEku": SimTypeInt(signed=False, label="UInt32"), "eku": SimTypePointer(SimTypePointer(SimTypeChar(label="Byte"), offset=0), offset=0)}, name="IKEEXT_CERT_EKUS0", pack=False, align=None), offset=0), "name": SimTypePointer(SimStruct({"nameType": SimTypeInt(signed=False, label="IKEEXT_CERT_CRITERIA_NAME_TYPE"), "certName": SimTypePointer(SimTypeChar(label="Char"), offset=0)}, name="IKEEXT_CERT_NAME0", pack=False, align=None), offset=0), "flags": SimTypeInt(signed=False, label="UInt32")}, name="IKEEXT_CERTIFICATE_CRITERIA0", pack=False, align=None), offset=0)}, name="_Anonymous2_e__Struct", pack=False, align=None), "Anonymous3": SimStruct({"outboundRootStoreArraySize": SimTypeInt(signed=False, label="UInt32"), "outboundTrustedRootStoreCriteria": SimTypePointer(SimStruct({"certData": SimStruct({"size": SimTypeInt(signed=False, label="UInt32"), "data": SimTypePointer(SimTypeChar(label="Byte"), offset=0)}, name="FWP_BYTE_BLOB", pack=False, align=None), "certHash": SimStruct({"size": SimTypeInt(signed=False, label="UInt32"), "data": SimTypePointer(SimTypeChar(label="Byte"), offset=0)}, name="FWP_BYTE_BLOB", pack=False, align=None), "eku": SimTypePointer(SimStruct({"numEku": SimTypeInt(signed=False, label="UInt32"), "eku": SimTypePointer(SimTypePointer(SimTypeChar(label="Byte"), offset=0), offset=0)}, name="IKEEXT_CERT_EKUS0", pack=False, align=None), offset=0), "name": SimTypePointer(SimStruct({"nameType": SimTypeInt(signed=False, label="IKEEXT_CERT_CRITERIA_NAME_TYPE"), "certName": SimTypePointer(SimTypeChar(label="Char"), offset=0)}, name="IKEEXT_CERT_NAME0", pack=False, align=None), offset=0), "flags": SimTypeInt(signed=False, label="UInt32")}, name="IKEEXT_CERTIFICATE_CRITERIA0", pack=False, align=None), offset=0)}, name="_Anonymous3_e__Struct", pack=False, align=None)}, name="<anon>", label="None"), "flags": SimTypeInt(signed=False, label="IKEEXT_CERT_AUTH"), "localCertLocationUrl": SimStruct({"size": SimTypeInt(signed=False, label="UInt32"), "data": SimTypePointer(SimTypeChar(label="Byte"), offset=0)}, name="FWP_BYTE_BLOB", pack=False, align=None)}, name="IKEEXT_CERTIFICATE_AUTHENTICATION2", pack=False, align=None), "kerberosAuthentication": SimStruct({"flags": SimTypeInt(signed=False, label="IKEEXT_KERBEROS_AUTHENTICATION_FLAGS"), "proxyServer": SimTypePointer(SimTypeChar(label="Char"), offset=0)}, name="IKEEXT_KERBEROS_AUTHENTICATION1", pack=False, align=None), "reservedAuthentication": SimStruct({"flags": SimTypeInt(signed=False, label="IKEEXT_RESERVED_AUTHENTICATION_FLAGS")}, name="IKEEXT_RESERVED_AUTHENTICATION0", pack=False, align=None), "ntlmV2Authentication": SimStruct({"flags": SimTypeInt(signed=False, label="UInt32")}, name="IKEEXT_NTLM_V2_AUTHENTICATION0", pack=False, align=None), "sslAuthentication": SimStruct({"inboundConfigType": SimTypeInt(signed=False, label="IKEEXT_CERT_CONFIG_TYPE"), "Anonymous1": SimUnion({"Anonymous1": SimStruct({"inboundRootArraySize": SimTypeInt(signed=False, label="UInt32"), "inboundRootCriteria": SimTypePointer(SimStruct({"certData": SimStruct({"size": SimTypeInt(signed=False, label="UInt32"), "data": SimTypePointer(SimTypeChar(label="Byte"), offset=0)}, name="FWP_BYTE_BLOB", pack=False, align=None), "certHash": SimStruct({"size": SimTypeInt(signed=False, label="UInt32"), "data": SimTypePointer(SimTypeChar(label="Byte"), offset=0)}, name="FWP_BYTE_BLOB", pack=False, align=None), "eku": SimTypePointer(SimStruct({"numEku": SimTypeInt(signed=False, label="UInt32"), "eku": SimTypePointer(SimTypePointer(SimTypeChar(label="Byte"), offset=0), offset=0)}, name="IKEEXT_CERT_EKUS0", pack=False, align=None), offset=0), "name": SimTypePointer(SimStruct({"nameType": SimTypeInt(signed=False, label="IKEEXT_CERT_CRITERIA_NAME_TYPE"), "certName": SimTypePointer(SimTypeChar(label="Char"), offset=0)}, name="IKEEXT_CERT_NAME0", pack=False, align=None), offset=0), "flags": SimTypeInt(signed=False, label="UInt32")}, name="IKEEXT_CERTIFICATE_CRITERIA0", pack=False, align=None), offset=0)}, name="_Anonymous1_e__Struct", pack=False, align=None), "Anonymous2": SimStruct({"inboundEnterpriseStoreArraySize": SimTypeInt(signed=False, label="UInt32"), "inboundEnterpriseStoreCriteria": SimTypePointer(SimStruct({"certData": SimStruct({"size": SimTypeInt(signed=False, label="UInt32"), "data": SimTypePointer(SimTypeChar(label="Byte"), offset=0)}, name="FWP_BYTE_BLOB", pack=False, align=None), "certHash": SimStruct({"size": SimTypeInt(signed=False, label="UInt32"), "data": SimTypePointer(SimTypeChar(label="Byte"), offset=0)}, name="FWP_BYTE_BLOB", pack=False, align=None), "eku": SimTypePointer(SimStruct({"numEku": SimTypeInt(signed=False, label="UInt32"), "eku": SimTypePointer(SimTypePointer(SimTypeChar(label="Byte"), offset=0), offset=0)}, name="IKEEXT_CERT_EKUS0", pack=False, align=None), offset=0), "name": SimTypePointer(SimStruct({"nameType": SimTypeInt(signed=False, label="IKEEXT_CERT_CRITERIA_NAME_TYPE"), "certName": SimTypePointer(SimTypeChar(label="Char"), offset=0)}, name="IKEEXT_CERT_NAME0", pack=False, align=None), offset=0), "flags": SimTypeInt(signed=False, label="UInt32")}, name="IKEEXT_CERTIFICATE_CRITERIA0", pack=False, align=None), offset=0)}, name="_Anonymous2_e__Struct", pack=False, align=None), "Anonymous3": SimStruct({"inboundRootStoreArraySize": SimTypeInt(signed=False, label="UInt32"), "inboundTrustedRootStoreCriteria": SimTypePointer(SimStruct({"certData": SimStruct({"size": SimTypeInt(signed=False, label="UInt32"), "data": SimTypePointer(SimTypeChar(label="Byte"), offset=0)}, name="FWP_BYTE_BLOB", pack=False, align=None), "certHash": SimStruct({"size": SimTypeInt(signed=False, label="UInt32"), "data": SimTypePointer(SimTypeChar(label="Byte"), offset=0)}, name="FWP_BYTE_BLOB", pack=False, align=None), "eku": SimTypePointer(SimStruct({"numEku": SimTypeInt(signed=False, label="UInt32"), "eku": SimTypePointer(SimTypePointer(SimTypeChar(label="Byte"), offset=0), offset=0)}, name="IKEEXT_CERT_EKUS0", pack=False, align=None), offset=0), "name": SimTypePointer(SimStruct({"nameType": SimTypeInt(signed=False, label="IKEEXT_CERT_CRITERIA_NAME_TYPE"), "certName": SimTypePointer(SimTypeChar(label="Char"), offset=0)}, name="IKEEXT_CERT_NAME0", pack=False, align=None), offset=0), "flags": SimTypeInt(signed=False, label="UInt32")}, name="IKEEXT_CERTIFICATE_CRITERIA0", pack=False, align=None), offset=0)}, name="_Anonymous3_e__Struct", pack=False, align=None)}, name="<anon>", label="None"), "outboundConfigType": SimTypeInt(signed=False, label="IKEEXT_CERT_CONFIG_TYPE"), "Anonymous2": SimUnion({"Anonymous1": SimStruct({"outboundRootArraySize": SimTypeInt(signed=False, label="UInt32"), "outboundRootCriteria": SimTypePointer(SimStruct({"certData": SimStruct({"size": SimTypeInt(signed=False, label="UInt32"), "data": SimTypePointer(SimTypeChar(label="Byte"), offset=0)}, name="FWP_BYTE_BLOB", pack=False, align=None), "certHash": SimStruct({"size": SimTypeInt(signed=False, label="UInt32"), "data": SimTypePointer(SimTypeChar(label="Byte"), offset=0)}, name="FWP_BYTE_BLOB", pack=False, align=None), "eku": SimTypePointer(SimStruct({"numEku": SimTypeInt(signed=False, label="UInt32"), "eku": SimTypePointer(SimTypePointer(SimTypeChar(label="Byte"), offset=0), offset=0)}, name="IKEEXT_CERT_EKUS0", pack=False, align=None), offset=0), "name": SimTypePointer(SimStruct({"nameType": SimTypeInt(signed=False, label="IKEEXT_CERT_CRITERIA_NAME_TYPE"), "certName": SimTypePointer(SimTypeChar(label="Char"), offset=0)}, name="IKEEXT_CERT_NAME0", pack=False, align=None), offset=0), "flags": SimTypeInt(signed=False, label="UInt32")}, name="IKEEXT_CERTIFICATE_CRITERIA0", pack=False, align=None), offset=0)}, name="_Anonymous1_e__Struct", pack=False, align=None), "Anonymous2": SimStruct({"outboundEnterpriseStoreArraySize": SimTypeInt(signed=False, label="UInt32"), "outboundEnterpriseStoreCriteria": SimTypePointer(SimStruct({"certData": SimStruct({"size": SimTypeInt(signed=False, label="UInt32"), "data": SimTypePointer(SimTypeChar(label="Byte"), offset=0)}, name="FWP_BYTE_BLOB", pack=False, align=None), "certHash": SimStruct({"size": SimTypeInt(signed=False, label="UInt32"), "data": SimTypePointer(SimTypeChar(label="Byte"), offset=0)}, name="FWP_BYTE_BLOB", pack=False, align=None), "eku": SimTypePointer(SimStruct({"numEku": SimTypeInt(signed=False, label="UInt32"), "eku": SimTypePointer(SimTypePointer(SimTypeChar(label="Byte"), offset=0), offset=0)}, name="IKEEXT_CERT_EKUS0", pack=False, align=None), offset=0), "name": SimTypePointer(SimStruct({"nameType": SimTypeInt(signed=False, label="IKEEXT_CERT_CRITERIA_NAME_TYPE"), "certName": SimTypePointer(SimTypeChar(label="Char"), offset=0)}, name="IKEEXT_CERT_NAME0", pack=False, align=None), offset=0), "flags": SimTypeInt(signed=False, label="UInt32")}, name="IKEEXT_CERTIFICATE_CRITERIA0", pack=False, align=None), offset=0)}, name="_Anonymous2_e__Struct", pack=False, align=None), "Anonymous3": SimStruct({"outboundRootStoreArraySize": SimTypeInt(signed=False, label="UInt32"), "outboundTrustedRootStoreCriteria": SimTypePointer(SimStruct({"certData": SimStruct({"size": SimTypeInt(signed=False, label="UInt32"), "data": SimTypePointer(SimTypeChar(label="Byte"), offset=0)}, name="FWP_BYTE_BLOB", pack=False, align=None), "certHash": SimStruct({"size": SimTypeInt(signed=False, label="UInt32"), "data": SimTypePointer(SimTypeChar(label="Byte"), offset=0)}, name="FWP_BYTE_BLOB", pack=False, align=None), "eku": SimTypePointer(SimStruct({"numEku": SimTypeInt(signed=False, label="UInt32"), "eku": SimTypePointer(SimTypePointer(SimTypeChar(label="Byte"), offset=0), offset=0)}, name="IKEEXT_CERT_EKUS0", pack=False, align=None), offset=0), "name": SimTypePointer(SimStruct({"nameType": SimTypeInt(signed=False, label="IKEEXT_CERT_CRITERIA_NAME_TYPE"), "certName": SimTypePointer(SimTypeChar(label="Char"), offset=0)}, name="IKEEXT_CERT_NAME0", pack=False, align=None), offset=0), "flags": SimTypeInt(signed=False, label="UInt32")}, name="IKEEXT_CERTIFICATE_CRITERIA0", pack=False, align=None), offset=0)}, name="_Anonymous3_e__Struct", pack=False, align=None)}, name="<anon>", label="None"), "flags": SimTypeInt(signed=False, label="IKEEXT_CERT_AUTH"), "localCertLocationUrl": SimStruct({"size": SimTypeInt(signed=False, label="UInt32"), "data": SimTypePointer(SimTypeChar(label="Byte"), offset=0)}, name="FWP_BYTE_BLOB", pack=False, align=None)}, name="IKEEXT_CERTIFICATE_AUTHENTICATION2", pack=False, align=None), "cgaAuthentication": SimStruct({"keyContainerName": SimTypePointer(SimTypeChar(label="Char"), offset=0), "cspName": SimTypePointer(SimTypeChar(label="Char"), offset=0), "cspType": SimTypeInt(signed=False, label="UInt32"), "cgaModifier": SimStruct({"byteArray16": SimTypeFixedSizeArray(SimTypeChar(label="Byte"), 16)}, name="FWP_BYTE_ARRAY16", pack=False, align=None), "cgaCollisionCount": SimTypeChar(label="Byte")}, name="IKEEXT_IPV6_CGA_AUTHENTICATION0", pack=False, align=None), "eapAuthentication": SimStruct({"flags": SimTypeInt(signed=False, label="IKEEXT_EAP_AUTHENTICATION_FLAGS")}, name="IKEEXT_EAP_AUTHENTICATION0", pack=False, align=None)}, name="<anon>", label="None")}, name="IKEEXT_AUTHENTICATION_METHOD2", pack=False, align=None), offset=0), "initiatorImpersonationType": SimTypeInt(signed=False, label="IKEEXT_AUTHENTICATION_IMPERSONATION_TYPE")}, name="IKEEXT_EM_POLICY2", pack=False, align=None), offset=0), "fwdPathSaLifetime": SimTypeInt(signed=False, label="UInt32")}, name="IPSEC_TUNNEL_POLICY2", pack=False, align=None), offset=0), "authipQmTransportPolicy": SimTypePointer(SimStruct({"numIpsecProposals": SimTypeInt(signed=False, label="UInt32"), "ipsecProposals": SimTypePointer(SimStruct({"lifetime": SimStruct({"lifetimeSeconds": SimTypeInt(signed=False, label="UInt32"), "lifetimeKilobytes": SimTypeInt(signed=False, label="UInt32"), "lifetimePackets": SimTypeInt(signed=False, label="UInt32")}, name="IPSEC_SA_LIFETIME0", pack=False, align=None), "numSaTransforms": SimTypeInt(signed=False, label="UInt32"), "saTransforms": SimTypePointer(SimStruct({"ipsecTransformType": SimTypeInt(signed=False, label="IPSEC_TRANSFORM_TYPE"), "Anonymous": SimUnion({"ahTransform": SimTypePointer(SimStruct({"authTransformId": SimStruct({"authType": SimTypeInt(signed=False, label="IPSEC_AUTH_TYPE"), "authConfig": SimTypeChar(label="Byte")}, name="IPSEC_AUTH_TRANSFORM_ID0", pack=False, align=None), "cryptoModuleId": SimTypePointer(SimTypeBottom(label="Guid"), offset=0)}, name="IPSEC_AUTH_TRANSFORM0", pack=False, align=None), offset=0), "espAuthTransform": SimTypePointer(SimStruct({"authTransformId": SimStruct({"authType": SimTypeInt(signed=False, label="IPSEC_AUTH_TYPE"), "authConfig": SimTypeChar(label="Byte")}, name="IPSEC_AUTH_TRANSFORM_ID0", pack=False, align=None), "cryptoModuleId": SimTypePointer(SimTypeBottom(label="Guid"), offset=0)}, name="IPSEC_AUTH_TRANSFORM0", pack=False, align=None), offset=0), "espCipherTransform": SimTypePointer(SimStruct({"cipherTransformId": SimStruct({"cipherType": SimTypeInt(signed=False, label="IPSEC_CIPHER_TYPE"), "cipherConfig": SimTypeChar(label="Byte")}, name="IPSEC_CIPHER_TRANSFORM_ID0", pack=False, align=None), "cryptoModuleId": SimTypePointer(SimTypeBottom(label="Guid"), offset=0)}, name="IPSEC_CIPHER_TRANSFORM0", pack=False, align=None), offset=0), "espAuthAndCipherTransform": SimTypePointer(SimStruct({"authTransform": SimStruct({"authTransformId": SimStruct({"authType": SimTypeInt(signed=False, label="IPSEC_AUTH_TYPE"), "authConfig": SimTypeChar(label="Byte")}, name="IPSEC_AUTH_TRANSFORM_ID0", pack=False, align=None), "cryptoModuleId": SimTypePointer(SimTypeBottom(label="Guid"), offset=0)}, name="IPSEC_AUTH_TRANSFORM0", pack=False, align=None), "cipherTransform": SimStruct({"cipherTransformId": SimStruct({"cipherType": SimTypeInt(signed=False, label="IPSEC_CIPHER_TYPE"), "cipherConfig": SimTypeChar(label="Byte")}, name="IPSEC_CIPHER_TRANSFORM_ID0", pack=False, align=None), "cryptoModuleId": SimTypePointer(SimTypeBottom(label="Guid"), offset=0)}, name="IPSEC_CIPHER_TRANSFORM0", pack=False, align=None)}, name="IPSEC_AUTH_AND_CIPHER_TRANSFORM0", pack=False, align=None), offset=0), "espAuthFwTransform": SimTypePointer(SimStruct({"authTransformId": SimStruct({"authType": SimTypeInt(signed=False, label="IPSEC_AUTH_TYPE"), "authConfig": SimTypeChar(label="Byte")}, name="IPSEC_AUTH_TRANSFORM_ID0", pack=False, align=None), "cryptoModuleId": SimTypePointer(SimTypeBottom(label="Guid"), offset=0)}, name="IPSEC_AUTH_TRANSFORM0", pack=False, align=None), offset=0)}, name="<anon>", label="None")}, name="IPSEC_SA_TRANSFORM0", pack=False, align=None), offset=0), "pfsGroup": SimTypeInt(signed=False, label="IPSEC_PFS_GROUP")}, name="IPSEC_PROPOSAL0", pack=False, align=None), offset=0), "flags": SimTypeInt(signed=False, label="IPSEC_POLICY_FLAG"), "ndAllowClearTimeoutSeconds": SimTypeInt(signed=False, label="UInt32"), "saIdleTimeout": SimStruct({"idleTimeoutSeconds": SimTypeInt(signed=False, label="UInt32"), "idleTimeoutSecondsFailOver": SimTypeInt(signed=False, label="UInt32")}, name="IPSEC_SA_IDLE_TIMEOUT0", pack=False, align=None), "emPolicy": SimTypePointer(SimStruct({"numAuthenticationMethods": SimTypeInt(signed=False, label="UInt32"), "authenticationMethods": SimTypePointer(SimStruct({"authenticationMethodType": SimTypeInt(signed=False, label="IKEEXT_AUTHENTICATION_METHOD_TYPE"), "Anonymous": SimUnion({"presharedKeyAuthentication": SimStruct({"presharedKey": SimStruct({"size": SimTypeInt(signed=False, label="UInt32"), "data": SimTypePointer(SimTypeChar(label="Byte"), offset=0)}, name="FWP_BYTE_BLOB", pack=False, align=None), "flags": SimTypeInt(signed=False, label="IKEEXT_PRESHARED_KEY_AUTHENTICATION_FLAGS")}, name="IKEEXT_PRESHARED_KEY_AUTHENTICATION1", pack=False, align=None), "certificateAuthentication": SimStruct({"inboundConfigType": SimTypeInt(signed=False, label="IKEEXT_CERT_CONFIG_TYPE"), "Anonymous1": SimUnion({"Anonymous1": SimStruct({"inboundRootArraySize": SimTypeInt(signed=False, label="UInt32"), "inboundRootCriteria": SimTypePointer(SimStruct({"certData": SimStruct({"size": SimTypeInt(signed=False, label="UInt32"), "data": SimTypePointer(SimTypeChar(label="Byte"), offset=0)}, name="FWP_BYTE_BLOB", pack=False, align=None), "certHash": SimStruct({"size": SimTypeInt(signed=False, label="UInt32"), "data": SimTypePointer(SimTypeChar(label="Byte"), offset=0)}, name="FWP_BYTE_BLOB", pack=False, align=None), "eku": SimTypePointer(SimStruct({"numEku": SimTypeInt(signed=False, label="UInt32"), "eku": SimTypePointer(SimTypePointer(SimTypeChar(label="Byte"), offset=0), offset=0)}, name="IKEEXT_CERT_EKUS0", pack=False, align=None), offset=0), "name": SimTypePointer(SimStruct({"nameType": SimTypeInt(signed=False, label="IKEEXT_CERT_CRITERIA_NAME_TYPE"), "certName": SimTypePointer(SimTypeChar(label="Char"), offset=0)}, name="IKEEXT_CERT_NAME0", pack=False, align=None), offset=0), "flags": SimTypeInt(signed=False, label="UInt32")}, name="IKEEXT_CERTIFICATE_CRITERIA0", pack=False, align=None), offset=0)}, name="_Anonymous1_e__Struct", pack=False, align=None), "Anonymous2": SimStruct({"inboundEnterpriseStoreArraySize": SimTypeInt(signed=False, label="UInt32"), "inboundEnterpriseStoreCriteria": SimTypePointer(SimStruct({"certData": SimStruct({"size": SimTypeInt(signed=False, label="UInt32"), "data": SimTypePointer(SimTypeChar(label="Byte"), offset=0)}, name="FWP_BYTE_BLOB", pack=False, align=None), "certHash": SimStruct({"size": SimTypeInt(signed=False, label="UInt32"), "data": SimTypePointer(SimTypeChar(label="Byte"), offset=0)}, name="FWP_BYTE_BLOB", pack=False, align=None), "eku": SimTypePointer(SimStruct({"numEku": SimTypeInt(signed=False, label="UInt32"), "eku": SimTypePointer(SimTypePointer(SimTypeChar(label="Byte"), offset=0), offset=0)}, name="IKEEXT_CERT_EKUS0", pack=False, align=None), offset=0), "name": SimTypePointer(SimStruct({"nameType": SimTypeInt(signed=False, label="IKEEXT_CERT_CRITERIA_NAME_TYPE"), "certName": SimTypePointer(SimTypeChar(label="Char"), offset=0)}, name="IKEEXT_CERT_NAME0", pack=False, align=None), offset=0), "flags": SimTypeInt(signed=False, label="UInt32")}, name="IKEEXT_CERTIFICATE_CRITERIA0", pack=False, align=None), offset=0)}, name="_Anonymous2_e__Struct", pack=False, align=None), "Anonymous3": SimStruct({"inboundRootStoreArraySize": SimTypeInt(signed=False, label="UInt32"), "inboundTrustedRootStoreCriteria": SimTypePointer(SimStruct({"certData": SimStruct({"size": SimTypeInt(signed=False, label="UInt32"), "data": SimTypePointer(SimTypeChar(label="Byte"), offset=0)}, name="FWP_BYTE_BLOB", pack=False, align=None), "certHash": SimStruct({"size": SimTypeInt(signed=False, label="UInt32"), "data": SimTypePointer(SimTypeChar(label="Byte"), offset=0)}, name="FWP_BYTE_BLOB", pack=False, align=None), "eku": SimTypePointer(SimStruct({"numEku": SimTypeInt(signed=False, label="UInt32"), "eku": SimTypePointer(SimTypePointer(SimTypeChar(label="Byte"), offset=0), offset=0)}, name="IKEEXT_CERT_EKUS0", pack=False, align=None), offset=0), "name": SimTypePointer(SimStruct({"nameType": SimTypeInt(signed=False, label="IKEEXT_CERT_CRITERIA_NAME_TYPE"), "certName": SimTypePointer(SimTypeChar(label="Char"), offset=0)}, name="IKEEXT_CERT_NAME0", pack=False, align=None), offset=0), "flags": SimTypeInt(signed=False, label="UInt32")}, name="IKEEXT_CERTIFICATE_CRITERIA0", pack=False, align=None), offset=0)}, name="_Anonymous3_e__Struct", pack=False, align=None)}, name="<anon>", label="None"), "outboundConfigType": SimTypeInt(signed=False, label="IKEEXT_CERT_CONFIG_TYPE"), "Anonymous2": SimUnion({"Anonymous1": SimStruct({"outboundRootArraySize": SimTypeInt(signed=False, label="UInt32"), "outboundRootCriteria": SimTypePointer(SimStruct({"certData": SimStruct({"size": SimTypeInt(signed=False, label="UInt32"), "data": SimTypePointer(SimTypeChar(label="Byte"), offset=0)}, name="FWP_BYTE_BLOB", pack=False, align=None), "certHash": SimStruct({"size": SimTypeInt(signed=False, label="UInt32"), "data": SimTypePointer(SimTypeChar(label="Byte"), offset=0)}, name="FWP_BYTE_BLOB", pack=False, align=None), "eku": SimTypePointer(SimStruct({"numEku": SimTypeInt(signed=False, label="UInt32"), "eku": SimTypePointer(SimTypePointer(SimTypeChar(label="Byte"), offset=0), offset=0)}, name="IKEEXT_CERT_EKUS0", pack=False, align=None), offset=0), "name": SimTypePointer(SimStruct({"nameType": SimTypeInt(signed=False, label="IKEEXT_CERT_CRITERIA_NAME_TYPE"), "certName": SimTypePointer(SimTypeChar(label="Char"), offset=0)}, name="IKEEXT_CERT_NAME0", pack=False, align=None), offset=0), "flags": SimTypeInt(signed=False, label="UInt32")}, name="IKEEXT_CERTIFICATE_CRITERIA0", pack=False, align=None), offset=0)}, name="_Anonymous1_e__Struct", pack=False, align=None), "Anonymous2": SimStruct({"outboundEnterpriseStoreArraySize": SimTypeInt(signed=False, label="UInt32"), "outboundEnterpriseStoreCriteria": SimTypePointer(SimStruct({"certData": SimStruct({"size": SimTypeInt(signed=False, label="UInt32"), "data": SimTypePointer(SimTypeChar(label="Byte"), offset=0)}, name="FWP_BYTE_BLOB", pack=False, align=None), "certHash": SimStruct({"size": SimTypeInt(signed=False, label="UInt32"), "data": SimTypePointer(SimTypeChar(label="Byte"), offset=0)}, name="FWP_BYTE_BLOB", pack=False, align=None), "eku": SimTypePointer(SimStruct({"numEku": SimTypeInt(signed=False, label="UInt32"), "eku": SimTypePointer(SimTypePointer(SimTypeChar(label="Byte"), offset=0), offset=0)}, name="IKEEXT_CERT_EKUS0", pack=False, align=None), offset=0), "name": SimTypePointer(SimStruct({"nameType": SimTypeInt(signed=False, label="IKEEXT_CERT_CRITERIA_NAME_TYPE"), "certName": SimTypePointer(SimTypeChar(label="Char"), offset=0)}, name="IKEEXT_CERT_NAME0", pack=False, align=None), offset=0), "flags": SimTypeInt(signed=False, label="UInt32")}, name="IKEEXT_CERTIFICATE_CRITERIA0", pack=False, align=None), offset=0)}, name="_Anonymous2_e__Struct", pack=False, align=None), "Anonymous3": SimStruct({"outboundRootStoreArraySize": SimTypeInt(signed=False, label="UInt32"), "outboundTrustedRootStoreCriteria": SimTypePointer(SimStruct({"certData": SimStruct({"size": SimTypeInt(signed=False, label="UInt32"), "data": SimTypePointer(SimTypeChar(label="Byte"), offset=0)}, name="FWP_BYTE_BLOB", pack=False, align=None), "certHash": SimStruct({"size": SimTypeInt(signed=False, label="UInt32"), "data": SimTypePointer(SimTypeChar(label="Byte"), offset=0)}, name="FWP_BYTE_BLOB", pack=False, align=None), "eku": SimTypePointer(SimStruct({"numEku": SimTypeInt(signed=False, label="UInt32"), "eku": SimTypePointer(SimTypePointer(SimTypeChar(label="Byte"), offset=0), offset=0)}, name="IKEEXT_CERT_EKUS0", pack=False, align=None), offset=0), "name": SimTypePointer(SimStruct({"nameType": SimTypeInt(signed=False, label="IKEEXT_CERT_CRITERIA_NAME_TYPE"), "certName": SimTypePointer(SimTypeChar(label="Char"), offset=0)}, name="IKEEXT_CERT_NAME0", pack=False, align=None), offset=0), "flags": SimTypeInt(signed=False, label="UInt32")}, name="IKEEXT_CERTIFICATE_CRITERIA0", pack=False, align=None), offset=0)}, name="_Anonymous3_e__Struct", pack=False, align=None)}, name="<anon>", label="None"), "flags": SimTypeInt(signed=False, label="IKEEXT_CERT_AUTH"), "localCertLocationUrl": SimStruct({"size": SimTypeInt(signed=False, label="UInt32"), "data": SimTypePointer(SimTypeChar(label="Byte"), offset=0)}, name="FWP_BYTE_BLOB", pack=False, align=None)}, name="IKEEXT_CERTIFICATE_AUTHENTICATION2", pack=False, align=None), "kerberosAuthentication": SimStruct({"flags": SimTypeInt(signed=False, label="IKEEXT_KERBEROS_AUTHENTICATION_FLAGS"), "proxyServer": SimTypePointer(SimTypeChar(label="Char"), offset=0)}, name="IKEEXT_KERBEROS_AUTHENTICATION1", pack=False, align=None), "reservedAuthentication": SimStruct({"flags": SimTypeInt(signed=False, label="IKEEXT_RESERVED_AUTHENTICATION_FLAGS")}, name="IKEEXT_RESERVED_AUTHENTICATION0", pack=False, align=None), "ntlmV2Authentication": SimStruct({"flags": SimTypeInt(signed=False, label="UInt32")}, name="IKEEXT_NTLM_V2_AUTHENTICATION0", pack=False, align=None), "sslAuthentication": SimStruct({"inboundConfigType": SimTypeInt(signed=False, label="IKEEXT_CERT_CONFIG_TYPE"), "Anonymous1": SimUnion({"Anonymous1": SimStruct({"inboundRootArraySize": SimTypeInt(signed=False, label="UInt32"), "inboundRootCriteria": SimTypePointer(SimStruct({"certData": SimStruct({"size": SimTypeInt(signed=False, label="UInt32"), "data": SimTypePointer(SimTypeChar(label="Byte"), offset=0)}, name="FWP_BYTE_BLOB", pack=False, align=None), "certHash": SimStruct({"size": SimTypeInt(signed=False, label="UInt32"), "data": SimTypePointer(SimTypeChar(label="Byte"), offset=0)}, name="FWP_BYTE_BLOB", pack=False, align=None), "eku": SimTypePointer(SimStruct({"numEku": SimTypeInt(signed=False, label="UInt32"), "eku": SimTypePointer(SimTypePointer(SimTypeChar(label="Byte"), offset=0), offset=0)}, name="IKEEXT_CERT_EKUS0", pack=False, align=None), offset=0), "name": SimTypePointer(SimStruct({"nameType": SimTypeInt(signed=False, label="IKEEXT_CERT_CRITERIA_NAME_TYPE"), "certName": SimTypePointer(SimTypeChar(label="Char"), offset=0)}, name="IKEEXT_CERT_NAME0", pack=False, align=None), offset=0), "flags": SimTypeInt(signed=False, label="UInt32")}, name="IKEEXT_CERTIFICATE_CRITERIA0", pack=False, align=None), offset=0)}, name="_Anonymous1_e__Struct", pack=False, align=None), "Anonymous2": SimStruct({"inboundEnterpriseStoreArraySize": SimTypeInt(signed=False, label="UInt32"), "inboundEnterpriseStoreCriteria": SimTypePointer(SimStruct({"certData": SimStruct({"size": SimTypeInt(signed=False, label="UInt32"), "data": SimTypePointer(SimTypeChar(label="Byte"), offset=0)}, name="FWP_BYTE_BLOB", pack=False, align=None), "certHash": SimStruct({"size": SimTypeInt(signed=False, label="UInt32"), "data": SimTypePointer(SimTypeChar(label="Byte"), offset=0)}, name="FWP_BYTE_BLOB", pack=False, align=None), "eku": SimTypePointer(SimStruct({"numEku": SimTypeInt(signed=False, label="UInt32"), "eku": SimTypePointer(SimTypePointer(SimTypeChar(label="Byte"), offset=0), offset=0)}, name="IKEEXT_CERT_EKUS0", pack=False, align=None), offset=0), "name": SimTypePointer(SimStruct({"nameType": SimTypeInt(signed=False, label="IKEEXT_CERT_CRITERIA_NAME_TYPE"), "certName": SimTypePointer(SimTypeChar(label="Char"), offset=0)}, name="IKEEXT_CERT_NAME0", pack=False, align=None), offset=0), "flags": SimTypeInt(signed=False, label="UInt32")}, name="IKEEXT_CERTIFICATE_CRITERIA0", pack=False, align=None), offset=0)}, name="_Anonymous2_e__Struct", pack=False, align=None), "Anonymous3": SimStruct({"inboundRootStoreArraySize": SimTypeInt(signed=False, label="UInt32"), "inboundTrustedRootStoreCriteria": SimTypePointer(SimStruct({"certData": SimStruct({"size": SimTypeInt(signed=False, label="UInt32"), "data": SimTypePointer(SimTypeChar(label="Byte"), offset=0)}, name="FWP_BYTE_BLOB", pack=False, align=None), "certHash": SimStruct({"size": SimTypeInt(signed=False, label="UInt32"), "data": SimTypePointer(SimTypeChar(label="Byte"), offset=0)}, name="FWP_BYTE_BLOB", pack=False, align=None), "eku": SimTypePointer(SimStruct({"numEku": SimTypeInt(signed=False, label="UInt32"), "eku": SimTypePointer(SimTypePointer(SimTypeChar(label="Byte"), offset=0), offset=0)}, name="IKEEXT_CERT_EKUS0", pack=False, align=None), offset=0), "name": SimTypePointer(SimStruct({"nameType": SimTypeInt(signed=False, label="IKEEXT_CERT_CRITERIA_NAME_TYPE"), "certName": SimTypePointer(SimTypeChar(label="Char"), offset=0)}, name="IKEEXT_CERT_NAME0", pack=False, align=None), offset=0), "flags": SimTypeInt(signed=False, label="UInt32")}, name="IKEEXT_CERTIFICATE_CRITERIA0", pack=False, align=None), offset=0)}, name="_Anonymous3_e__Struct", pack=False, align=None)}, name="<anon>", label="None"), "outboundConfigType": SimTypeInt(signed=False, label="IKEEXT_CERT_CONFIG_TYPE"), "Anonymous2": SimUnion({"Anonymous1": SimStruct({"outboundRootArraySize": SimTypeInt(signed=False, label="UInt32"), "outboundRootCriteria": SimTypePointer(SimStruct({"certData": SimStruct({"size": SimTypeInt(signed=False, label="UInt32"), "data": SimTypePointer(SimTypeChar(label="Byte"), offset=0)}, name="FWP_BYTE_BLOB", pack=False, align=None), "certHash": SimStruct({"size": SimTypeInt(signed=False, label="UInt32"), "data": SimTypePointer(SimTypeChar(label="Byte"), offset=0)}, name="FWP_BYTE_BLOB", pack=False, align=None), "eku": SimTypePointer(SimStruct({"numEku": SimTypeInt(signed=False, label="UInt32"), "eku": SimTypePointer(SimTypePointer(SimTypeChar(label="Byte"), offset=0), offset=0)}, name="IKEEXT_CERT_EKUS0", pack=False, align=None), offset=0), "name": SimTypePointer(SimStruct({"nameType": SimTypeInt(signed=False, label="IKEEXT_CERT_CRITERIA_NAME_TYPE"), "certName": SimTypePointer(SimTypeChar(label="Char"), offset=0)}, name="IKEEXT_CERT_NAME0", pack=False, align=None), offset=0), "flags": SimTypeInt(signed=False, label="UInt32")}, name="IKEEXT_CERTIFICATE_CRITERIA0", pack=False, align=None), offset=0)}, name="_Anonymous1_e__Struct", pack=False, align=None), "Anonymous2": SimStruct({"outboundEnterpriseStoreArraySize": SimTypeInt(signed=False, label="UInt32"), "outboundEnterpriseStoreCriteria": SimTypePointer(SimStruct({"certData": SimStruct({"size": SimTypeInt(signed=False, label="UInt32"), "data": SimTypePointer(SimTypeChar(label="Byte"), offset=0)}, name="FWP_BYTE_BLOB", pack=False, align=None), "certHash": SimStruct({"size": SimTypeInt(signed=False, label="UInt32"), "data": SimTypePointer(SimTypeChar(label="Byte"), offset=0)}, name="FWP_BYTE_BLOB", pack=False, align=None), "eku": SimTypePointer(SimStruct({"numEku": SimTypeInt(signed=False, label="UInt32"), "eku": SimTypePointer(SimTypePointer(SimTypeChar(label="Byte"), offset=0), offset=0)}, name="IKEEXT_CERT_EKUS0", pack=False, align=None), offset=0), "name": SimTypePointer(SimStruct({"nameType": SimTypeInt(signed=False, label="IKEEXT_CERT_CRITERIA_NAME_TYPE"), "certName": SimTypePointer(SimTypeChar(label="Char"), offset=0)}, name="IKEEXT_CERT_NAME0", pack=False, align=None), offset=0), "flags": SimTypeInt(signed=False, label="UInt32")}, name="IKEEXT_CERTIFICATE_CRITERIA0", pack=False, align=None), offset=0)}, name="_Anonymous2_e__Struct", pack=False, align=None), "Anonymous3": SimStruct({"outboundRootStoreArraySize": SimTypeInt(signed=False, label="UInt32"), "outboundTrustedRootStoreCriteria": SimTypePointer(SimStruct({"certData": SimStruct({"size": SimTypeInt(signed=False, label="UInt32"), "data": SimTypePointer(SimTypeChar(label="Byte"), offset=0)}, name="FWP_BYTE_BLOB", pack=False, align=None), "certHash": SimStruct({"size": SimTypeInt(signed=False, label="UInt32"), "data": SimTypePointer(SimTypeChar(label="Byte"), offset=0)}, name="FWP_BYTE_BLOB", pack=False, align=None), "eku": SimTypePointer(SimStruct({"numEku": SimTypeInt(signed=False, label="UInt32"), "eku": SimTypePointer(SimTypePointer(SimTypeChar(label="Byte"), offset=0), offset=0)}, name="IKEEXT_CERT_EKUS0", pack=False, align=None), offset=0), "name": SimTypePointer(SimStruct({"nameType": SimTypeInt(signed=False, label="IKEEXT_CERT_CRITERIA_NAME_TYPE"), "certName": SimTypePointer(SimTypeChar(label="Char"), offset=0)}, name="IKEEXT_CERT_NAME0", pack=False, align=None), offset=0), "flags": SimTypeInt(signed=False, label="UInt32")}, name="IKEEXT_CERTIFICATE_CRITERIA0", pack=False, align=None), offset=0)}, name="_Anonymous3_e__Struct", pack=False, align=None)}, name="<anon>", label="None"), "flags": SimTypeInt(signed=False, label="IKEEXT_CERT_AUTH"), "localCertLocationUrl": SimStruct({"size": SimTypeInt(signed=False, label="UInt32"), "data": SimTypePointer(SimTypeChar(label="Byte"), offset=0)}, name="FWP_BYTE_BLOB", pack=False, align=None)}, name="IKEEXT_CERTIFICATE_AUTHENTICATION2", pack=False, align=None), "cgaAuthentication": SimStruct({"keyContainerName": SimTypePointer(SimTypeChar(label="Char"), offset=0), "cspName": SimTypePointer(SimTypeChar(label="Char"), offset=0), "cspType": SimTypeInt(signed=False, label="UInt32"), "cgaModifier": SimStruct({"byteArray16": SimTypeFixedSizeArray(SimTypeChar(label="Byte"), 16)}, name="FWP_BYTE_ARRAY16", pack=False, align=None), "cgaCollisionCount": SimTypeChar(label="Byte")}, name="IKEEXT_IPV6_CGA_AUTHENTICATION0", pack=False, align=None), "eapAuthentication": SimStruct({"flags": SimTypeInt(signed=False, label="IKEEXT_EAP_AUTHENTICATION_FLAGS")}, name="IKEEXT_EAP_AUTHENTICATION0", pack=False, align=None)}, name="<anon>", label="None")}, name="IKEEXT_AUTHENTICATION_METHOD2", pack=False, align=None), offset=0), "initiatorImpersonationType": SimTypeInt(signed=False, label="IKEEXT_AUTHENTICATION_IMPERSONATION_TYPE")}, name="IKEEXT_EM_POLICY2", pack=False, align=None), offset=0)}, name="IPSEC_TRANSPORT_POLICY2", pack=False, align=None), offset=0), "authipQmTunnelPolicy": SimTypePointer(SimStruct({"flags": SimTypeInt(signed=False, label="IPSEC_POLICY_FLAG"), "numIpsecProposals": SimTypeInt(signed=False, label="UInt32"), "ipsecProposals": SimTypePointer(SimStruct({"lifetime": SimStruct({"lifetimeSeconds": SimTypeInt(signed=False, label="UInt32"), "lifetimeKilobytes": SimTypeInt(signed=False, label="UInt32"), "lifetimePackets": SimTypeInt(signed=False, label="UInt32")}, name="IPSEC_SA_LIFETIME0", pack=False, align=None), "numSaTransforms": SimTypeInt(signed=False, label="UInt32"), "saTransforms": SimTypePointer(SimStruct({"ipsecTransformType": SimTypeInt(signed=False, label="IPSEC_TRANSFORM_TYPE"), "Anonymous": SimUnion({"ahTransform": SimTypePointer(SimStruct({"authTransformId": SimStruct({"authType": SimTypeInt(signed=False, label="IPSEC_AUTH_TYPE"), "authConfig": SimTypeChar(label="Byte")}, name="IPSEC_AUTH_TRANSFORM_ID0", pack=False, align=None), "cryptoModuleId": SimTypePointer(SimTypeBottom(label="Guid"), offset=0)}, name="IPSEC_AUTH_TRANSFORM0", pack=False, align=None), offset=0), "espAuthTransform": SimTypePointer(SimStruct({"authTransformId": SimStruct({"authType": SimTypeInt(signed=False, label="IPSEC_AUTH_TYPE"), "authConfig": SimTypeChar(label="Byte")}, name="IPSEC_AUTH_TRANSFORM_ID0", pack=False, align=None), "cryptoModuleId": SimTypePointer(SimTypeBottom(label="Guid"), offset=0)}, name="IPSEC_AUTH_TRANSFORM0", pack=False, align=None), offset=0), "espCipherTransform": SimTypePointer(SimStruct({"cipherTransformId": SimStruct({"cipherType": SimTypeInt(signed=False, label="IPSEC_CIPHER_TYPE"), "cipherConfig": SimTypeChar(label="Byte")}, name="IPSEC_CIPHER_TRANSFORM_ID0", pack=False, align=None), "cryptoModuleId": SimTypePointer(SimTypeBottom(label="Guid"), offset=0)}, name="IPSEC_CIPHER_TRANSFORM0", pack=False, align=None), offset=0), "espAuthAndCipherTransform": SimTypePointer(SimStruct({"authTransform": SimStruct({"authTransformId": SimStruct({"authType": SimTypeInt(signed=False, label="IPSEC_AUTH_TYPE"), "authConfig": SimTypeChar(label="Byte")}, name="IPSEC_AUTH_TRANSFORM_ID0", pack=False, align=None), "cryptoModuleId": SimTypePointer(SimTypeBottom(label="Guid"), offset=0)}, name="IPSEC_AUTH_TRANSFORM0", pack=False, align=None), "cipherTransform": SimStruct({"cipherTransformId": SimStruct({"cipherType": SimTypeInt(signed=False, label="IPSEC_CIPHER_TYPE"), "cipherConfig": SimTypeChar(label="Byte")}, name="IPSEC_CIPHER_TRANSFORM_ID0", pack=False, align=None), "cryptoModuleId": SimTypePointer(SimTypeBottom(label="Guid"), offset=0)}, name="IPSEC_CIPHER_TRANSFORM0", pack=False, align=None)}, name="IPSEC_AUTH_AND_CIPHER_TRANSFORM0", pack=False, align=None), offset=0), "espAuthFwTransform": SimTypePointer(SimStruct({"authTransformId": SimStruct({"authType": SimTypeInt(signed=False, label="IPSEC_AUTH_TYPE"), "authConfig": SimTypeChar(label="Byte")}, name="IPSEC_AUTH_TRANSFORM_ID0", pack=False, align=None), "cryptoModuleId": SimTypePointer(SimTypeBottom(label="Guid"), offset=0)}, name="IPSEC_AUTH_TRANSFORM0", pack=False, align=None), offset=0)}, name="<anon>", label="None")}, name="IPSEC_SA_TRANSFORM0", pack=False, align=None), offset=0), "pfsGroup": SimTypeInt(signed=False, label="IPSEC_PFS_GROUP")}, name="IPSEC_PROPOSAL0", pack=False, align=None), offset=0), "tunnelEndpoints": SimStruct({"ipVersion": SimTypeInt(signed=False, label="FWP_IP_VERSION"), "Anonymous1": SimUnion({"localV4Address": SimTypeInt(signed=False, label="UInt32"), "localV6Address": SimTypeFixedSizeArray(SimTypeChar(label="Byte"), 16)}, name="<anon>", label="None"), "Anonymous2": SimUnion({"remoteV4Address": SimTypeInt(signed=False, label="UInt32"), "remoteV6Address": SimTypeFixedSizeArray(SimTypeChar(label="Byte"), 16)}, name="<anon>", label="None"), "localIfLuid": SimTypeLongLong(signed=False, label="UInt64"), "remoteFqdn": SimTypePointer(SimTypeChar(label="Char"), offset=0), "numAddresses": SimTypeInt(signed=False, label="UInt32"), "remoteAddresses": SimTypePointer(SimStruct({"ipVersion": SimTypeInt(signed=False, label="FWP_IP_VERSION"), "Anonymous": SimUnion({"v4Address": SimTypeInt(signed=False, label="UInt32"), "v6Address": SimTypeFixedSizeArray(SimTypeChar(label="Byte"), 16)}, name="<anon>", label="None")}, name="IPSEC_TUNNEL_ENDPOINT0", pack=False, align=None), offset=0)}, name="IPSEC_TUNNEL_ENDPOINTS2", pack=False, align=None), "saIdleTimeout": SimStruct({"idleTimeoutSeconds": SimTypeInt(signed=False, label="UInt32"), "idleTimeoutSecondsFailOver": SimTypeInt(signed=False, label="UInt32")}, name="IPSEC_SA_IDLE_TIMEOUT0", pack=False, align=None), "emPolicy": SimTypePointer(SimStruct({"numAuthenticationMethods": SimTypeInt(signed=False, label="UInt32"), "authenticationMethods": SimTypePointer(SimStruct({"authenticationMethodType": SimTypeInt(signed=False, label="IKEEXT_AUTHENTICATION_METHOD_TYPE"), "Anonymous": SimUnion({"presharedKeyAuthentication": SimStruct({"presharedKey": SimStruct({"size": SimTypeInt(signed=False, label="UInt32"), "data": SimTypePointer(SimTypeChar(label="Byte"), offset=0)}, name="FWP_BYTE_BLOB", pack=False, align=None), "flags": SimTypeInt(signed=False, label="IKEEXT_PRESHARED_KEY_AUTHENTICATION_FLAGS")}, name="IKEEXT_PRESHARED_KEY_AUTHENTICATION1", pack=False, align=None), "certificateAuthentication": SimStruct({"inboundConfigType": SimTypeInt(signed=False, label="IKEEXT_CERT_CONFIG_TYPE"), "Anonymous1": SimUnion({"Anonymous1": SimStruct({"inboundRootArraySize": SimTypeInt(signed=False, label="UInt32"), "inboundRootCriteria": SimTypePointer(SimStruct({"certData": SimStruct({"size": SimTypeInt(signed=False, label="UInt32"), "data": SimTypePointer(SimTypeChar(label="Byte"), offset=0)}, name="FWP_BYTE_BLOB", pack=False, align=None), "certHash": SimStruct({"size": SimTypeInt(signed=False, label="UInt32"), "data": SimTypePointer(SimTypeChar(label="Byte"), offset=0)}, name="FWP_BYTE_BLOB", pack=False, align=None), "eku": SimTypePointer(SimStruct({"numEku": SimTypeInt(signed=False, label="UInt32"), "eku": SimTypePointer(SimTypePointer(SimTypeChar(label="Byte"), offset=0), offset=0)}, name="IKEEXT_CERT_EKUS0", pack=False, align=None), offset=0), "name": SimTypePointer(SimStruct({"nameType": SimTypeInt(signed=False, label="IKEEXT_CERT_CRITERIA_NAME_TYPE"), "certName": SimTypePointer(SimTypeChar(label="Char"), offset=0)}, name="IKEEXT_CERT_NAME0", pack=False, align=None), offset=0), "flags": SimTypeInt(signed=False, label="UInt32")}, name="IKEEXT_CERTIFICATE_CRITERIA0", pack=False, align=None), offset=0)}, name="_Anonymous1_e__Struct", pack=False, align=None), "Anonymous2": SimStruct({"inboundEnterpriseStoreArraySize": SimTypeInt(signed=False, label="UInt32"), "inboundEnterpriseStoreCriteria": SimTypePointer(SimStruct({"certData": SimStruct({"size": SimTypeInt(signed=False, label="UInt32"), "data": SimTypePointer(SimTypeChar(label="Byte"), offset=0)}, name="FWP_BYTE_BLOB", pack=False, align=None), "certHash": SimStruct({"size": SimTypeInt(signed=False, label="UInt32"), "data": SimTypePointer(SimTypeChar(label="Byte"), offset=0)}, name="FWP_BYTE_BLOB", pack=False, align=None), "eku": SimTypePointer(SimStruct({"numEku": SimTypeInt(signed=False, label="UInt32"), "eku": SimTypePointer(SimTypePointer(SimTypeChar(label="Byte"), offset=0), offset=0)}, name="IKEEXT_CERT_EKUS0", pack=False, align=None), offset=0), "name": SimTypePointer(SimStruct({"nameType": SimTypeInt(signed=False, label="IKEEXT_CERT_CRITERIA_NAME_TYPE"), "certName": SimTypePointer(SimTypeChar(label="Char"), offset=0)}, name="IKEEXT_CERT_NAME0", pack=False, align=None), offset=0), "flags": SimTypeInt(signed=False, label="UInt32")}, name="IKEEXT_CERTIFICATE_CRITERIA0", pack=False, align=None), offset=0)}, name="_Anonymous2_e__Struct", pack=False, align=None), "Anonymous3": SimStruct({"inboundRootStoreArraySize": SimTypeInt(signed=False, label="UInt32"), "inboundTrustedRootStoreCriteria": SimTypePointer(SimStruct({"certData": SimStruct({"size": SimTypeInt(signed=False, label="UInt32"), "data": SimTypePointer(SimTypeChar(label="Byte"), offset=0)}, name="FWP_BYTE_BLOB", pack=False, align=None), "certHash": SimStruct({"size": SimTypeInt(signed=False, label="UInt32"), "data": SimTypePointer(SimTypeChar(label="Byte"), offset=0)}, name="FWP_BYTE_BLOB", pack=False, align=None), "eku": SimTypePointer(SimStruct({"numEku": SimTypeInt(signed=False, label="UInt32"), "eku": SimTypePointer(SimTypePointer(SimTypeChar(label="Byte"), offset=0), offset=0)}, name="IKEEXT_CERT_EKUS0", pack=False, align=None), offset=0), "name": SimTypePointer(SimStruct({"nameType": SimTypeInt(signed=False, label="IKEEXT_CERT_CRITERIA_NAME_TYPE"), "certName": SimTypePointer(SimTypeChar(label="Char"), offset=0)}, name="IKEEXT_CERT_NAME0", pack=False, align=None), offset=0), "flags": SimTypeInt(signed=False, label="UInt32")}, name="IKEEXT_CERTIFICATE_CRITERIA0", pack=False, align=None), offset=0)}, name="_Anonymous3_e__Struct", pack=False, align=None)}, name="<anon>", label="None"), "outboundConfigType": SimTypeInt(signed=False, label="IKEEXT_CERT_CONFIG_TYPE"), "Anonymous2": SimUnion({"Anonymous1": SimStruct({"outboundRootArraySize": SimTypeInt(signed=False, label="UInt32"), "outboundRootCriteria": SimTypePointer(SimStruct({"certData": SimStruct({"size": SimTypeInt(signed=False, label="UInt32"), "data": SimTypePointer(SimTypeChar(label="Byte"), offset=0)}, name="FWP_BYTE_BLOB", pack=False, align=None), "certHash": SimStruct({"size": SimTypeInt(signed=False, label="UInt32"), "data": SimTypePointer(SimTypeChar(label="Byte"), offset=0)}, name="FWP_BYTE_BLOB", pack=False, align=None), "eku": SimTypePointer(SimStruct({"numEku": SimTypeInt(signed=False, label="UInt32"), "eku": SimTypePointer(SimTypePointer(SimTypeChar(label="Byte"), offset=0), offset=0)}, name="IKEEXT_CERT_EKUS0", pack=False, align=None), offset=0), "name": SimTypePointer(SimStruct({"nameType": SimTypeInt(signed=False, label="IKEEXT_CERT_CRITERIA_NAME_TYPE"), "certName": SimTypePointer(SimTypeChar(label="Char"), offset=0)}, name="IKEEXT_CERT_NAME0", pack=False, align=None), offset=0), "flags": SimTypeInt(signed=False, label="UInt32")}, name="IKEEXT_CERTIFICATE_CRITERIA0", pack=False, align=None), offset=0)}, name="_Anonymous1_e__Struct", pack=False, align=None), "Anonymous2": SimStruct({"outboundEnterpriseStoreArraySize": SimTypeInt(signed=False, label="UInt32"), "outboundEnterpriseStoreCriteria": SimTypePointer(SimStruct({"certData": SimStruct({"size": SimTypeInt(signed=False, label="UInt32"), "data": SimTypePointer(SimTypeChar(label="Byte"), offset=0)}, name="FWP_BYTE_BLOB", pack=False, align=None), "certHash": SimStruct({"size": SimTypeInt(signed=False, label="UInt32"), "data": SimTypePointer(SimTypeChar(label="Byte"), offset=0)}, name="FWP_BYTE_BLOB", pack=False, align=None), "eku": SimTypePointer(SimStruct({"numEku": SimTypeInt(signed=False, label="UInt32"), "eku": SimTypePointer(SimTypePointer(SimTypeChar(label="Byte"), offset=0), offset=0)}, name="IKEEXT_CERT_EKUS0", pack=False, align=None), offset=0), "name": SimTypePointer(SimStruct({"nameType": SimTypeInt(signed=False, label="IKEEXT_CERT_CRITERIA_NAME_TYPE"), "certName": SimTypePointer(SimTypeChar(label="Char"), offset=0)}, name="IKEEXT_CERT_NAME0", pack=False, align=None), offset=0), "flags": SimTypeInt(signed=False, label="UInt32")}, name="IKEEXT_CERTIFICATE_CRITERIA0", pack=False, align=None), offset=0)}, name="_Anonymous2_e__Struct", pack=False, align=None), "Anonymous3": SimStruct({"outboundRootStoreArraySize": SimTypeInt(signed=False, label="UInt32"), "outboundTrustedRootStoreCriteria": SimTypePointer(SimStruct({"certData": SimStruct({"size": SimTypeInt(signed=False, label="UInt32"), "data": SimTypePointer(SimTypeChar(label="Byte"), offset=0)}, name="FWP_BYTE_BLOB", pack=False, align=None), "certHash": SimStruct({"size": SimTypeInt(signed=False, label="UInt32"), "data": SimTypePointer(SimTypeChar(label="Byte"), offset=0)}, name="FWP_BYTE_BLOB", pack=False, align=None), "eku": SimTypePointer(SimStruct({"numEku": SimTypeInt(signed=False, label="UInt32"), "eku": SimTypePointer(SimTypePointer(SimTypeChar(label="Byte"), offset=0), offset=0)}, name="IKEEXT_CERT_EKUS0", pack=False, align=None), offset=0), "name": SimTypePointer(SimStruct({"nameType": SimTypeInt(signed=False, label="IKEEXT_CERT_CRITERIA_NAME_TYPE"), "certName": SimTypePointer(SimTypeChar(label="Char"), offset=0)}, name="IKEEXT_CERT_NAME0", pack=False, align=None), offset=0), "flags": SimTypeInt(signed=False, label="UInt32")}, name="IKEEXT_CERTIFICATE_CRITERIA0", pack=False, align=None), offset=0)}, name="_Anonymous3_e__Struct", pack=False, align=None)}, name="<anon>", label="None"), "flags": SimTypeInt(signed=False, label="IKEEXT_CERT_AUTH"), "localCertLocationUrl": SimStruct({"size": SimTypeInt(signed=False, label="UInt32"), "data": SimTypePointer(SimTypeChar(label="Byte"), offset=0)}, name="FWP_BYTE_BLOB", pack=False, align=None)}, name="IKEEXT_CERTIFICATE_AUTHENTICATION2", pack=False, align=None), "kerberosAuthentication": SimStruct({"flags": SimTypeInt(signed=False, label="IKEEXT_KERBEROS_AUTHENTICATION_FLAGS"), "proxyServer": SimTypePointer(SimTypeChar(label="Char"), offset=0)}, name="IKEEXT_KERBEROS_AUTHENTICATION1", pack=False, align=None), "reservedAuthentication": SimStruct({"flags": SimTypeInt(signed=False, label="IKEEXT_RESERVED_AUTHENTICATION_FLAGS")}, name="IKEEXT_RESERVED_AUTHENTICATION0", pack=False, align=None), "ntlmV2Authentication": SimStruct({"flags": SimTypeInt(signed=False, label="UInt32")}, name="IKEEXT_NTLM_V2_AUTHENTICATION0", pack=False, align=None), "sslAuthentication": SimStruct({"inboundConfigType": SimTypeInt(signed=False, label="IKEEXT_CERT_CONFIG_TYPE"), "Anonymous1": SimUnion({"Anonymous1": SimStruct({"inboundRootArraySize": SimTypeInt(signed=False, label="UInt32"), "inboundRootCriteria": SimTypePointer(SimStruct({"certData": SimStruct({"size": SimTypeInt(signed=False, label="UInt32"), "data": SimTypePointer(SimTypeChar(label="Byte"), offset=0)}, name="FWP_BYTE_BLOB", pack=False, align=None), "certHash": SimStruct({"size": SimTypeInt(signed=False, label="UInt32"), "data": SimTypePointer(SimTypeChar(label="Byte"), offset=0)}, name="FWP_BYTE_BLOB", pack=False, align=None), "eku": SimTypePointer(SimStruct({"numEku": SimTypeInt(signed=False, label="UInt32"), "eku": SimTypePointer(SimTypePointer(SimTypeChar(label="Byte"), offset=0), offset=0)}, name="IKEEXT_CERT_EKUS0", pack=False, align=None), offset=0), "name": SimTypePointer(SimStruct({"nameType": SimTypeInt(signed=False, label="IKEEXT_CERT_CRITERIA_NAME_TYPE"), "certName": SimTypePointer(SimTypeChar(label="Char"), offset=0)}, name="IKEEXT_CERT_NAME0", pack=False, align=None), offset=0), "flags": SimTypeInt(signed=False, label="UInt32")}, name="IKEEXT_CERTIFICATE_CRITERIA0", pack=False, align=None), offset=0)}, name="_Anonymous1_e__Struct", pack=False, align=None), "Anonymous2": SimStruct({"inboundEnterpriseStoreArraySize": SimTypeInt(signed=False, label="UInt32"), "inboundEnterpriseStoreCriteria": SimTypePointer(SimStruct({"certData": SimStruct({"size": SimTypeInt(signed=False, label="UInt32"), "data": SimTypePointer(SimTypeChar(label="Byte"), offset=0)}, name="FWP_BYTE_BLOB", pack=False, align=None), "certHash": SimStruct({"size": SimTypeInt(signed=False, label="UInt32"), "data": SimTypePointer(SimTypeChar(label="Byte"), offset=0)}, name="FWP_BYTE_BLOB", pack=False, align=None), "eku": SimTypePointer(SimStruct({"numEku": SimTypeInt(signed=False, label="UInt32"), "eku": SimTypePointer(SimTypePointer(SimTypeChar(label="Byte"), offset=0), offset=0)}, name="IKEEXT_CERT_EKUS0", pack=False, align=None), offset=0), "name": SimTypePointer(SimStruct({"nameType": SimTypeInt(signed=False, label="IKEEXT_CERT_CRITERIA_NAME_TYPE"), "certName": SimTypePointer(SimTypeChar(label="Char"), offset=0)}, name="IKEEXT_CERT_NAME0", pack=False, align=None), offset=0), "flags": SimTypeInt(signed=False, label="UInt32")}, name="IKEEXT_CERTIFICATE_CRITERIA0", pack=False, align=None), offset=0)}, name="_Anonymous2_e__Struct", pack=False, align=None), "Anonymous3": SimStruct({"inboundRootStoreArraySize": SimTypeInt(signed=False, label="UInt32"), "inboundTrustedRootStoreCriteria": SimTypePointer(SimStruct({"certData": SimStruct({"size": SimTypeInt(signed=False, label="UInt32"), "data": SimTypePointer(SimTypeChar(label="Byte"), offset=0)}, name="FWP_BYTE_BLOB", pack=False, align=None), "certHash": SimStruct({"size": SimTypeInt(signed=False, label="UInt32"), "data": SimTypePointer(SimTypeChar(label="Byte"), offset=0)}, name="FWP_BYTE_BLOB", pack=False, align=None), "eku": SimTypePointer(SimStruct({"numEku": SimTypeInt(signed=False, label="UInt32"), "eku": SimTypePointer(SimTypePointer(SimTypeChar(label="Byte"), offset=0), offset=0)}, name="IKEEXT_CERT_EKUS0", pack=False, align=None), offset=0), "name": SimTypePointer(SimStruct({"nameType": SimTypeInt(signed=False, label="IKEEXT_CERT_CRITERIA_NAME_TYPE"), "certName": SimTypePointer(SimTypeChar(label="Char"), offset=0)}, name="IKEEXT_CERT_NAME0", pack=False, align=None), offset=0), "flags": SimTypeInt(signed=False, label="UInt32")}, name="IKEEXT_CERTIFICATE_CRITERIA0", pack=False, align=None), offset=0)}, name="_Anonymous3_e__Struct", pack=False, align=None)}, name="<anon>", label="None"), "outboundConfigType": SimTypeInt(signed=False, label="IKEEXT_CERT_CONFIG_TYPE"), "Anonymous2": SimUnion({"Anonymous1": SimStruct({"outboundRootArraySize": SimTypeInt(signed=False, label="UInt32"), "outboundRootCriteria": SimTypePointer(SimStruct({"certData": SimStruct({"size": SimTypeInt(signed=False, label="UInt32"), "data": SimTypePointer(SimTypeChar(label="Byte"), offset=0)}, name="FWP_BYTE_BLOB", pack=False, align=None), "certHash": SimStruct({"size": SimTypeInt(signed=False, label="UInt32"), "data": SimTypePointer(SimTypeChar(label="Byte"), offset=0)}, name="FWP_BYTE_BLOB", pack=False, align=None), "eku": SimTypePointer(SimStruct({"numEku": SimTypeInt(signed=False, label="UInt32"), "eku": SimTypePointer(SimTypePointer(SimTypeChar(label="Byte"), offset=0), offset=0)}, name="IKEEXT_CERT_EKUS0", pack=False, align=None), offset=0), "name": SimTypePointer(SimStruct({"nameType": SimTypeInt(signed=False, label="IKEEXT_CERT_CRITERIA_NAME_TYPE"), "certName": SimTypePointer(SimTypeChar(label="Char"), offset=0)}, name="IKEEXT_CERT_NAME0", pack=False, align=None), offset=0), "flags": SimTypeInt(signed=False, label="UInt32")}, name="IKEEXT_CERTIFICATE_CRITERIA0", pack=False, align=None), offset=0)}, name="_Anonymous1_e__Struct", pack=False, align=None), "Anonymous2": SimStruct({"outboundEnterpriseStoreArraySize": SimTypeInt(signed=False, label="UInt32"), "outboundEnterpriseStoreCriteria": SimTypePointer(SimStruct({"certData": SimStruct({"size": SimTypeInt(signed=False, label="UInt32"), "data": SimTypePointer(SimTypeChar(label="Byte"), offset=0)}, name="FWP_BYTE_BLOB", pack=False, align=None), "certHash": SimStruct({"size": SimTypeInt(signed=False, label="UInt32"), "data": SimTypePointer(SimTypeChar(label="Byte"), offset=0)}, name="FWP_BYTE_BLOB", pack=False, align=None), "eku": SimTypePointer(SimStruct({"numEku": SimTypeInt(signed=False, label="UInt32"), "eku": SimTypePointer(SimTypePointer(SimTypeChar(label="Byte"), offset=0), offset=0)}, name="IKEEXT_CERT_EKUS0", pack=False, align=None), offset=0), "name": SimTypePointer(SimStruct({"nameType": SimTypeInt(signed=False, label="IKEEXT_CERT_CRITERIA_NAME_TYPE"), "certName": SimTypePointer(SimTypeChar(label="Char"), offset=0)}, name="IKEEXT_CERT_NAME0", pack=False, align=None), offset=0), "flags": SimTypeInt(signed=False, label="UInt32")}, name="IKEEXT_CERTIFICATE_CRITERIA0", pack=False, align=None), offset=0)}, name="_Anonymous2_e__Struct", pack=False, align=None), "Anonymous3": SimStruct({"outboundRootStoreArraySize": SimTypeInt(signed=False, label="UInt32"), "outboundTrustedRootStoreCriteria": SimTypePointer(SimStruct({"certData": SimStruct({"size": SimTypeInt(signed=False, label="UInt32"), "data": SimTypePointer(SimTypeChar(label="Byte"), offset=0)}, name="FWP_BYTE_BLOB", pack=False, align=None), "certHash": SimStruct({"size": SimTypeInt(signed=False, label="UInt32"), "data": SimTypePointer(SimTypeChar(label="Byte"), offset=0)}, name="FWP_BYTE_BLOB", pack=False, align=None), "eku": SimTypePointer(SimStruct({"numEku": SimTypeInt(signed=False, label="UInt32"), "eku": SimTypePointer(SimTypePointer(SimTypeChar(label="Byte"), offset=0), offset=0)}, name="IKEEXT_CERT_EKUS0", pack=False, align=None), offset=0), "name": SimTypePointer(SimStruct({"nameType": SimTypeInt(signed=False, label="IKEEXT_CERT_CRITERIA_NAME_TYPE"), "certName": SimTypePointer(SimTypeChar(label="Char"), offset=0)}, name="IKEEXT_CERT_NAME0", pack=False, align=None), offset=0), "flags": SimTypeInt(signed=False, label="UInt32")}, name="IKEEXT_CERTIFICATE_CRITERIA0", pack=False, align=None), offset=0)}, name="_Anonymous3_e__Struct", pack=False, align=None)}, name="<anon>", label="None"), "flags": SimTypeInt(signed=False, label="IKEEXT_CERT_AUTH"), "localCertLocationUrl": SimStruct({"size": SimTypeInt(signed=False, label="UInt32"), "data": SimTypePointer(SimTypeChar(label="Byte"), offset=0)}, name="FWP_BYTE_BLOB", pack=False, align=None)}, name="IKEEXT_CERTIFICATE_AUTHENTICATION2", pack=False, align=None), "cgaAuthentication": SimStruct({"keyContainerName": SimTypePointer(SimTypeChar(label="Char"), offset=0), "cspName": SimTypePointer(SimTypeChar(label="Char"), offset=0), "cspType": SimTypeInt(signed=False, label="UInt32"), "cgaModifier": SimStruct({"byteArray16": SimTypeFixedSizeArray(SimTypeChar(label="Byte"), 16)}, name="FWP_BYTE_ARRAY16", pack=False, align=None), "cgaCollisionCount": SimTypeChar(label="Byte")}, name="IKEEXT_IPV6_CGA_AUTHENTICATION0", pack=False, align=None), "eapAuthentication": SimStruct({"flags": SimTypeInt(signed=False, label="IKEEXT_EAP_AUTHENTICATION_FLAGS")}, name="IKEEXT_EAP_AUTHENTICATION0", pack=False, align=None)}, name="<anon>", label="None")}, name="IKEEXT_AUTHENTICATION_METHOD2", pack=False, align=None), offset=0), "initiatorImpersonationType": SimTypeInt(signed=False, label="IKEEXT_AUTHENTICATION_IMPERSONATION_TYPE")}, name="IKEEXT_EM_POLICY2", pack=False, align=None), offset=0), "fwdPathSaLifetime": SimTypeInt(signed=False, label="UInt32")}, name="IPSEC_TUNNEL_POLICY2", pack=False, align=None), offset=0), "ikeMmPolicy": SimTypePointer(SimStruct({"softExpirationTime": SimTypeInt(signed=False, label="UInt32"), "numAuthenticationMethods": SimTypeInt(signed=False, label="UInt32"), "authenticationMethods": SimTypePointer(SimStruct({"authenticationMethodType": SimTypeInt(signed=False, label="IKEEXT_AUTHENTICATION_METHOD_TYPE"), "Anonymous": SimUnion({"presharedKeyAuthentication": SimStruct({"presharedKey": SimStruct({"size": SimTypeInt(signed=False, label="UInt32"), "data": SimTypePointer(SimTypeChar(label="Byte"), offset=0)}, name="FWP_BYTE_BLOB", pack=False, align=None), "flags": SimTypeInt(signed=False, label="IKEEXT_PRESHARED_KEY_AUTHENTICATION_FLAGS")}, name="IKEEXT_PRESHARED_KEY_AUTHENTICATION1", pack=False, align=None), "certificateAuthentication": SimStruct({"inboundConfigType": SimTypeInt(signed=False, label="IKEEXT_CERT_CONFIG_TYPE"), "Anonymous1": SimUnion({"Anonymous1": SimStruct({"inboundRootArraySize": SimTypeInt(signed=False, label="UInt32"), "inboundRootCriteria": SimTypePointer(SimStruct({"certData": SimStruct({"size": SimTypeInt(signed=False, label="UInt32"), "data": SimTypePointer(SimTypeChar(label="Byte"), offset=0)}, name="FWP_BYTE_BLOB", pack=False, align=None), "certHash": SimStruct({"size": SimTypeInt(signed=False, label="UInt32"), "data": SimTypePointer(SimTypeChar(label="Byte"), offset=0)}, name="FWP_BYTE_BLOB", pack=False, align=None), "eku": SimTypePointer(SimStruct({"numEku": SimTypeInt(signed=False, label="UInt32"), "eku": SimTypePointer(SimTypePointer(SimTypeChar(label="Byte"), offset=0), offset=0)}, name="IKEEXT_CERT_EKUS0", pack=False, align=None), offset=0), "name": SimTypePointer(SimStruct({"nameType": SimTypeInt(signed=False, label="IKEEXT_CERT_CRITERIA_NAME_TYPE"), "certName": SimTypePointer(SimTypeChar(label="Char"), offset=0)}, name="IKEEXT_CERT_NAME0", pack=False, align=None), offset=0), "flags": SimTypeInt(signed=False, label="UInt32")}, name="IKEEXT_CERTIFICATE_CRITERIA0", pack=False, align=None), offset=0)}, name="_Anonymous1_e__Struct", pack=False, align=None), "Anonymous2": SimStruct({"inboundEnterpriseStoreArraySize": SimTypeInt(signed=False, label="UInt32"), "inboundEnterpriseStoreCriteria": SimTypePointer(SimStruct({"certData": SimStruct({"size": SimTypeInt(signed=False, label="UInt32"), "data": SimTypePointer(SimTypeChar(label="Byte"), offset=0)}, name="FWP_BYTE_BLOB", pack=False, align=None), "certHash": SimStruct({"size": SimTypeInt(signed=False, label="UInt32"), "data": SimTypePointer(SimTypeChar(label="Byte"), offset=0)}, name="FWP_BYTE_BLOB", pack=False, align=None), "eku": SimTypePointer(SimStruct({"numEku": SimTypeInt(signed=False, label="UInt32"), "eku": SimTypePointer(SimTypePointer(SimTypeChar(label="Byte"), offset=0), offset=0)}, name="IKEEXT_CERT_EKUS0", pack=False, align=None), offset=0), "name": SimTypePointer(SimStruct({"nameType": SimTypeInt(signed=False, label="IKEEXT_CERT_CRITERIA_NAME_TYPE"), "certName": SimTypePointer(SimTypeChar(label="Char"), offset=0)}, name="IKEEXT_CERT_NAME0", pack=False, align=None), offset=0), "flags": SimTypeInt(signed=False, label="UInt32")}, name="IKEEXT_CERTIFICATE_CRITERIA0", pack=False, align=None), offset=0)}, name="_Anonymous2_e__Struct", pack=False, align=None), "Anonymous3": SimStruct({"inboundRootStoreArraySize": SimTypeInt(signed=False, label="UInt32"), "inboundTrustedRootStoreCriteria": SimTypePointer(SimStruct({"certData": SimStruct({"size": SimTypeInt(signed=False, label="UInt32"), "data": SimTypePointer(SimTypeChar(label="Byte"), offset=0)}, name="FWP_BYTE_BLOB", pack=False, align=None), "certHash": SimStruct({"size": SimTypeInt(signed=False, label="UInt32"), "data": SimTypePointer(SimTypeChar(label="Byte"), offset=0)}, name="FWP_BYTE_BLOB", pack=False, align=None), "eku": SimTypePointer(SimStruct({"numEku": SimTypeInt(signed=False, label="UInt32"), "eku": SimTypePointer(SimTypePointer(SimTypeChar(label="Byte"), offset=0), offset=0)}, name="IKEEXT_CERT_EKUS0", pack=False, align=None), offset=0), "name": SimTypePointer(SimStruct({"nameType": SimTypeInt(signed=False, label="IKEEXT_CERT_CRITERIA_NAME_TYPE"), "certName": SimTypePointer(SimTypeChar(label="Char"), offset=0)}, name="IKEEXT_CERT_NAME0", pack=False, align=None), offset=0), "flags": SimTypeInt(signed=False, label="UInt32")}, name="IKEEXT_CERTIFICATE_CRITERIA0", pack=False, align=None), offset=0)}, name="_Anonymous3_e__Struct", pack=False, align=None)}, name="<anon>", label="None"), "outboundConfigType": SimTypeInt(signed=False, label="IKEEXT_CERT_CONFIG_TYPE"), "Anonymous2": SimUnion({"Anonymous1": SimStruct({"outboundRootArraySize": SimTypeInt(signed=False, label="UInt32"), "outboundRootCriteria": SimTypePointer(SimStruct({"certData": SimStruct({"size": SimTypeInt(signed=False, label="UInt32"), "data": SimTypePointer(SimTypeChar(label="Byte"), offset=0)}, name="FWP_BYTE_BLOB", pack=False, align=None), "certHash": SimStruct({"size": SimTypeInt(signed=False, label="UInt32"), "data": SimTypePointer(SimTypeChar(label="Byte"), offset=0)}, name="FWP_BYTE_BLOB", pack=False, align=None), "eku": SimTypePointer(SimStruct({"numEku": SimTypeInt(signed=False, label="UInt32"), "eku": SimTypePointer(SimTypePointer(SimTypeChar(label="Byte"), offset=0), offset=0)}, name="IKEEXT_CERT_EKUS0", pack=False, align=None), offset=0), "name": SimTypePointer(SimStruct({"nameType": SimTypeInt(signed=False, label="IKEEXT_CERT_CRITERIA_NAME_TYPE"), "certName": SimTypePointer(SimTypeChar(label="Char"), offset=0)}, name="IKEEXT_CERT_NAME0", pack=False, align=None), offset=0), "flags": SimTypeInt(signed=False, label="UInt32")}, name="IKEEXT_CERTIFICATE_CRITERIA0", pack=False, align=None), offset=0)}, name="_Anonymous1_e__Struct", pack=False, align=None), "Anonymous2": SimStruct({"outboundEnterpriseStoreArraySize": SimTypeInt(signed=False, label="UInt32"), "outboundEnterpriseStoreCriteria": SimTypePointer(SimStruct({"certData": SimStruct({"size": SimTypeInt(signed=False, label="UInt32"), "data": SimTypePointer(SimTypeChar(label="Byte"), offset=0)}, name="FWP_BYTE_BLOB", pack=False, align=None), "certHash": SimStruct({"size": SimTypeInt(signed=False, label="UInt32"), "data": SimTypePointer(SimTypeChar(label="Byte"), offset=0)}, name="FWP_BYTE_BLOB", pack=False, align=None), "eku": SimTypePointer(SimStruct({"numEku": SimTypeInt(signed=False, label="UInt32"), "eku": SimTypePointer(SimTypePointer(SimTypeChar(label="Byte"), offset=0), offset=0)}, name="IKEEXT_CERT_EKUS0", pack=False, align=None), offset=0), "name": SimTypePointer(SimStruct({"nameType": SimTypeInt(signed=False, label="IKEEXT_CERT_CRITERIA_NAME_TYPE"), "certName": SimTypePointer(SimTypeChar(label="Char"), offset=0)}, name="IKEEXT_CERT_NAME0", pack=False, align=None), offset=0), "flags": SimTypeInt(signed=False, label="UInt32")}, name="IKEEXT_CERTIFICATE_CRITERIA0", pack=False, align=None), offset=0)}, name="_Anonymous2_e__Struct", pack=False, align=None), "Anonymous3": SimStruct({"outboundRootStoreArraySize": SimTypeInt(signed=False, label="UInt32"), "outboundTrustedRootStoreCriteria": SimTypePointer(SimStruct({"certData": SimStruct({"size": SimTypeInt(signed=False, label="UInt32"), "data": SimTypePointer(SimTypeChar(label="Byte"), offset=0)}, name="FWP_BYTE_BLOB", pack=False, align=None), "certHash": SimStruct({"size": SimTypeInt(signed=False, label="UInt32"), "data": SimTypePointer(SimTypeChar(label="Byte"), offset=0)}, name="FWP_BYTE_BLOB", pack=False, align=None), "eku": SimTypePointer(SimStruct({"numEku": SimTypeInt(signed=False, label="UInt32"), "eku": SimTypePointer(SimTypePointer(SimTypeChar(label="Byte"), offset=0), offset=0)}, name="IKEEXT_CERT_EKUS0", pack=False, align=None), offset=0), "name": SimTypePointer(SimStruct({"nameType": SimTypeInt(signed=False, label="IKEEXT_CERT_CRITERIA_NAME_TYPE"), "certName": SimTypePointer(SimTypeChar(label="Char"), offset=0)}, name="IKEEXT_CERT_NAME0", pack=False, align=None), offset=0), "flags": SimTypeInt(signed=False, label="UInt32")}, name="IKEEXT_CERTIFICATE_CRITERIA0", pack=False, align=None), offset=0)}, name="_Anonymous3_e__Struct", pack=False, align=None)}, name="<anon>", label="None"), "flags": SimTypeInt(signed=False, label="IKEEXT_CERT_AUTH"), "localCertLocationUrl": SimStruct({"size": SimTypeInt(signed=False, label="UInt32"), "data": SimTypePointer(SimTypeChar(label="Byte"), offset=0)}, name="FWP_BYTE_BLOB", pack=False, align=None)}, name="IKEEXT_CERTIFICATE_AUTHENTICATION2", pack=False, align=None), "kerberosAuthentication": SimStruct({"flags": SimTypeInt(signed=False, label="IKEEXT_KERBEROS_AUTHENTICATION_FLAGS"), "proxyServer": SimTypePointer(SimTypeChar(label="Char"), offset=0)}, name="IKEEXT_KERBEROS_AUTHENTICATION1", pack=False, align=None), "reservedAuthentication": SimStruct({"flags": SimTypeInt(signed=False, label="IKEEXT_RESERVED_AUTHENTICATION_FLAGS")}, name="IKEEXT_RESERVED_AUTHENTICATION0", pack=False, align=None), "ntlmV2Authentication": SimStruct({"flags": SimTypeInt(signed=False, label="UInt32")}, name="IKEEXT_NTLM_V2_AUTHENTICATION0", pack=False, align=None), "sslAuthentication": SimStruct({"inboundConfigType": SimTypeInt(signed=False, label="IKEEXT_CERT_CONFIG_TYPE"), "Anonymous1": SimUnion({"Anonymous1": SimStruct({"inboundRootArraySize": SimTypeInt(signed=False, label="UInt32"), "inboundRootCriteria": SimTypePointer(SimStruct({"certData": SimStruct({"size": SimTypeInt(signed=False, label="UInt32"), "data": SimTypePointer(SimTypeChar(label="Byte"), offset=0)}, name="FWP_BYTE_BLOB", pack=False, align=None), "certHash": SimStruct({"size": SimTypeInt(signed=False, label="UInt32"), "data": SimTypePointer(SimTypeChar(label="Byte"), offset=0)}, name="FWP_BYTE_BLOB", pack=False, align=None), "eku": SimTypePointer(SimStruct({"numEku": SimTypeInt(signed=False, label="UInt32"), "eku": SimTypePointer(SimTypePointer(SimTypeChar(label="Byte"), offset=0), offset=0)}, name="IKEEXT_CERT_EKUS0", pack=False, align=None), offset=0), "name": SimTypePointer(SimStruct({"nameType": SimTypeInt(signed=False, label="IKEEXT_CERT_CRITERIA_NAME_TYPE"), "certName": SimTypePointer(SimTypeChar(label="Char"), offset=0)}, name="IKEEXT_CERT_NAME0", pack=False, align=None), offset=0), "flags": SimTypeInt(signed=False, label="UInt32")}, name="IKEEXT_CERTIFICATE_CRITERIA0", pack=False, align=None), offset=0)}, name="_Anonymous1_e__Struct", pack=False, align=None), "Anonymous2": SimStruct({"inboundEnterpriseStoreArraySize": SimTypeInt(signed=False, label="UInt32"), "inboundEnterpriseStoreCriteria": SimTypePointer(SimStruct({"certData": SimStruct({"size": SimTypeInt(signed=False, label="UInt32"), "data": SimTypePointer(SimTypeChar(label="Byte"), offset=0)}, name="FWP_BYTE_BLOB", pack=False, align=None), "certHash": SimStruct({"size": SimTypeInt(signed=False, label="UInt32"), "data": SimTypePointer(SimTypeChar(label="Byte"), offset=0)}, name="FWP_BYTE_BLOB", pack=False, align=None), "eku": SimTypePointer(SimStruct({"numEku": SimTypeInt(signed=False, label="UInt32"), "eku": SimTypePointer(SimTypePointer(SimTypeChar(label="Byte"), offset=0), offset=0)}, name="IKEEXT_CERT_EKUS0", pack=False, align=None), offset=0), "name": SimTypePointer(SimStruct({"nameType": SimTypeInt(signed=False, label="IKEEXT_CERT_CRITERIA_NAME_TYPE"), "certName": SimTypePointer(SimTypeChar(label="Char"), offset=0)}, name="IKEEXT_CERT_NAME0", pack=False, align=None), offset=0), "flags": SimTypeInt(signed=False, label="UInt32")}, name="IKEEXT_CERTIFICATE_CRITERIA0", pack=False, align=None), offset=0)}, name="_Anonymous2_e__Struct", pack=False, align=None), "Anonymous3": SimStruct({"inboundRootStoreArraySize": SimTypeInt(signed=False, label="UInt32"), "inboundTrustedRootStoreCriteria": SimTypePointer(SimStruct({"certData": SimStruct({"size": SimTypeInt(signed=False, label="UInt32"), "data": SimTypePointer(SimTypeChar(label="Byte"), offset=0)}, name="FWP_BYTE_BLOB", pack=False, align=None), "certHash": SimStruct({"size": SimTypeInt(signed=False, label="UInt32"), "data": SimTypePointer(SimTypeChar(label="Byte"), offset=0)}, name="FWP_BYTE_BLOB", pack=False, align=None), "eku": SimTypePointer(SimStruct({"numEku": SimTypeInt(signed=False, label="UInt32"), "eku": SimTypePointer(SimTypePointer(SimTypeChar(label="Byte"), offset=0), offset=0)}, name="IKEEXT_CERT_EKUS0", pack=False, align=None), offset=0), "name": SimTypePointer(SimStruct({"nameType": SimTypeInt(signed=False, label="IKEEXT_CERT_CRITERIA_NAME_TYPE"), "certName": SimTypePointer(SimTypeChar(label="Char"), offset=0)}, name="IKEEXT_CERT_NAME0", pack=False, align=None), offset=0), "flags": SimTypeInt(signed=False, label="UInt32")}, name="IKEEXT_CERTIFICATE_CRITERIA0", pack=False, align=None), offset=0)}, name="_Anonymous3_e__Struct", pack=False, align=None)}, name="<anon>", label="None"), "outboundConfigType": SimTypeInt(signed=False, label="IKEEXT_CERT_CONFIG_TYPE"), "Anonymous2": SimUnion({"Anonymous1": SimStruct({"outboundRootArraySize": SimTypeInt(signed=False, label="UInt32"), "outboundRootCriteria": SimTypePointer(SimStruct({"certData": SimStruct({"size": SimTypeInt(signed=False, label="UInt32"), "data": SimTypePointer(SimTypeChar(label="Byte"), offset=0)}, name="FWP_BYTE_BLOB", pack=False, align=None), "certHash": SimStruct({"size": SimTypeInt(signed=False, label="UInt32"), "data": SimTypePointer(SimTypeChar(label="Byte"), offset=0)}, name="FWP_BYTE_BLOB", pack=False, align=None), "eku": SimTypePointer(SimStruct({"numEku": SimTypeInt(signed=False, label="UInt32"), "eku": SimTypePointer(SimTypePointer(SimTypeChar(label="Byte"), offset=0), offset=0)}, name="IKEEXT_CERT_EKUS0", pack=False, align=None), offset=0), "name": SimTypePointer(SimStruct({"nameType": SimTypeInt(signed=False, label="IKEEXT_CERT_CRITERIA_NAME_TYPE"), "certName": SimTypePointer(SimTypeChar(label="Char"), offset=0)}, name="IKEEXT_CERT_NAME0", pack=False, align=None), offset=0), "flags": SimTypeInt(signed=False, label="UInt32")}, name="IKEEXT_CERTIFICATE_CRITERIA0", pack=False, align=None), offset=0)}, name="_Anonymous1_e__Struct", pack=False, align=None), "Anonymous2": SimStruct({"outboundEnterpriseStoreArraySize": SimTypeInt(signed=False, label="UInt32"), "outboundEnterpriseStoreCriteria": SimTypePointer(SimStruct({"certData": SimStruct({"size": SimTypeInt(signed=False, label="UInt32"), "data": SimTypePointer(SimTypeChar(label="Byte"), offset=0)}, name="FWP_BYTE_BLOB", pack=False, align=None), "certHash": SimStruct({"size": SimTypeInt(signed=False, label="UInt32"), "data": SimTypePointer(SimTypeChar(label="Byte"), offset=0)}, name="FWP_BYTE_BLOB", pack=False, align=None), "eku": SimTypePointer(SimStruct({"numEku": SimTypeInt(signed=False, label="UInt32"), "eku": SimTypePointer(SimTypePointer(SimTypeChar(label="Byte"), offset=0), offset=0)}, name="IKEEXT_CERT_EKUS0", pack=False, align=None), offset=0), "name": SimTypePointer(SimStruct({"nameType": SimTypeInt(signed=False, label="IKEEXT_CERT_CRITERIA_NAME_TYPE"), "certName": SimTypePointer(SimTypeChar(label="Char"), offset=0)}, name="IKEEXT_CERT_NAME0", pack=False, align=None), offset=0), "flags": SimTypeInt(signed=False, label="UInt32")}, name="IKEEXT_CERTIFICATE_CRITERIA0", pack=False, align=None), offset=0)}, name="_Anonymous2_e__Struct", pack=False, align=None), "Anonymous3": SimStruct({"outboundRootStoreArraySize": SimTypeInt(signed=False, label="UInt32"), "outboundTrustedRootStoreCriteria": SimTypePointer(SimStruct({"certData": SimStruct({"size": SimTypeInt(signed=False, label="UInt32"), "data": SimTypePointer(SimTypeChar(label="Byte"), offset=0)}, name="FWP_BYTE_BLOB", pack=False, align=None), "certHash": SimStruct({"size": SimTypeInt(signed=False, label="UInt32"), "data": SimTypePointer(SimTypeChar(label="Byte"), offset=0)}, name="FWP_BYTE_BLOB", pack=False, align=None), "eku": SimTypePointer(SimStruct({"numEku": SimTypeInt(signed=False, label="UInt32"), "eku": SimTypePointer(SimTypePointer(SimTypeChar(label="Byte"), offset=0), offset=0)}, name="IKEEXT_CERT_EKUS0", pack=False, align=None), offset=0), "name": SimTypePointer(SimStruct({"nameType": SimTypeInt(signed=False, label="IKEEXT_CERT_CRITERIA_NAME_TYPE"), "certName": SimTypePointer(SimTypeChar(label="Char"), offset=0)}, name="IKEEXT_CERT_NAME0", pack=False, align=None), offset=0), "flags": SimTypeInt(signed=False, label="UInt32")}, name="IKEEXT_CERTIFICATE_CRITERIA0", pack=False, align=None), offset=0)}, name="_Anonymous3_e__Struct", pack=False, align=None)}, name="<anon>", label="None"), "flags": SimTypeInt(signed=False, label="IKEEXT_CERT_AUTH"), "localCertLocationUrl": SimStruct({"size": SimTypeInt(signed=False, label="UInt32"), "data": SimTypePointer(SimTypeChar(label="Byte"), offset=0)}, name="FWP_BYTE_BLOB", pack=False, align=None)}, name="IKEEXT_CERTIFICATE_AUTHENTICATION2", pack=False, align=None), "cgaAuthentication": SimStruct({"keyContainerName": SimTypePointer(SimTypeChar(label="Char"), offset=0), "cspName": SimTypePointer(SimTypeChar(label="Char"), offset=0), "cspType": SimTypeInt(signed=False, label="UInt32"), "cgaModifier": SimStruct({"byteArray16": SimTypeFixedSizeArray(SimTypeChar(label="Byte"), 16)}, name="FWP_BYTE_ARRAY16", pack=False, align=None), "cgaCollisionCount": SimTypeChar(label="Byte")}, name="IKEEXT_IPV6_CGA_AUTHENTICATION0", pack=False, align=None), "eapAuthentication": SimStruct({"flags": SimTypeInt(signed=False, label="IKEEXT_EAP_AUTHENTICATION_FLAGS")}, name="IKEEXT_EAP_AUTHENTICATION0", pack=False, align=None)}, name="<anon>", label="None")}, name="IKEEXT_AUTHENTICATION_METHOD2", pack=False, align=None), offset=0), "initiatorImpersonationType": SimTypeInt(signed=False, label="IKEEXT_AUTHENTICATION_IMPERSONATION_TYPE"), "numIkeProposals": SimTypeInt(signed=False, label="UInt32"), "ikeProposals": SimTypePointer(SimStruct({"cipherAlgorithm": SimStruct({"algoIdentifier": SimTypeInt(signed=False, label="IKEEXT_CIPHER_TYPE"), "keyLen": SimTypeInt(signed=False, label="UInt32"), "rounds": SimTypeInt(signed=False, label="UInt32")}, name="IKEEXT_CIPHER_ALGORITHM0", pack=False, align=None), "integrityAlgorithm": SimStruct({"algoIdentifier": SimTypeInt(signed=False, label="IKEEXT_INTEGRITY_TYPE")}, name="IKEEXT_INTEGRITY_ALGORITHM0", pack=False, align=None), "maxLifetimeSeconds": SimTypeInt(signed=False, label="UInt32"), "dhGroup": SimTypeInt(signed=False, label="IKEEXT_DH_GROUP"), "quickModeLimit": SimTypeInt(signed=False, label="UInt32")}, name="IKEEXT_PROPOSAL0", pack=False, align=None), offset=0), "flags": SimTypeInt(signed=False, label="IKEEXT_POLICY_FLAG"), "maxDynamicFilters": SimTypeInt(signed=False, label="UInt32"), "retransmitDurationSecs": SimTypeInt(signed=False, label="UInt32")}, name="IKEEXT_POLICY2", pack=False, align=None), offset=0), "authIpMmPolicy": SimTypePointer(SimStruct({"softExpirationTime": SimTypeInt(signed=False, label="UInt32"), "numAuthenticationMethods": SimTypeInt(signed=False, label="UInt32"), "authenticationMethods": SimTypePointer(SimStruct({"authenticationMethodType": SimTypeInt(signed=False, label="IKEEXT_AUTHENTICATION_METHOD_TYPE"), "Anonymous": SimUnion({"presharedKeyAuthentication": SimStruct({"presharedKey": SimStruct({"size": SimTypeInt(signed=False, label="UInt32"), "data": SimTypePointer(SimTypeChar(label="Byte"), offset=0)}, name="FWP_BYTE_BLOB", pack=False, align=None), "flags": SimTypeInt(signed=False, label="IKEEXT_PRESHARED_KEY_AUTHENTICATION_FLAGS")}, name="IKEEXT_PRESHARED_KEY_AUTHENTICATION1", pack=False, align=None), "certificateAuthentication": SimStruct({"inboundConfigType": SimTypeInt(signed=False, label="IKEEXT_CERT_CONFIG_TYPE"), "Anonymous1": SimUnion({"Anonymous1": SimStruct({"inboundRootArraySize": SimTypeInt(signed=False, label="UInt32"), "inboundRootCriteria": SimTypePointer(SimStruct({"certData": SimStruct({"size": SimTypeInt(signed=False, label="UInt32"), "data": SimTypePointer(SimTypeChar(label="Byte"), offset=0)}, name="FWP_BYTE_BLOB", pack=False, align=None), "certHash": SimStruct({"size": SimTypeInt(signed=False, label="UInt32"), "data": SimTypePointer(SimTypeChar(label="Byte"), offset=0)}, name="FWP_BYTE_BLOB", pack=False, align=None), "eku": SimTypePointer(SimStruct({"numEku": SimTypeInt(signed=False, label="UInt32"), "eku": SimTypePointer(SimTypePointer(SimTypeChar(label="Byte"), offset=0), offset=0)}, name="IKEEXT_CERT_EKUS0", pack=False, align=None), offset=0), "name": SimTypePointer(SimStruct({"nameType": SimTypeInt(signed=False, label="IKEEXT_CERT_CRITERIA_NAME_TYPE"), "certName": SimTypePointer(SimTypeChar(label="Char"), offset=0)}, name="IKEEXT_CERT_NAME0", pack=False, align=None), offset=0), "flags": SimTypeInt(signed=False, label="UInt32")}, name="IKEEXT_CERTIFICATE_CRITERIA0", pack=False, align=None), offset=0)}, name="_Anonymous1_e__Struct", pack=False, align=None), "Anonymous2": SimStruct({"inboundEnterpriseStoreArraySize": SimTypeInt(signed=False, label="UInt32"), "inboundEnterpriseStoreCriteria": SimTypePointer(SimStruct({"certData": SimStruct({"size": SimTypeInt(signed=False, label="UInt32"), "data": SimTypePointer(SimTypeChar(label="Byte"), offset=0)}, name="FWP_BYTE_BLOB", pack=False, align=None), "certHash": SimStruct({"size": SimTypeInt(signed=False, label="UInt32"), "data": SimTypePointer(SimTypeChar(label="Byte"), offset=0)}, name="FWP_BYTE_BLOB", pack=False, align=None), "eku": SimTypePointer(SimStruct({"numEku": SimTypeInt(signed=False, label="UInt32"), "eku": SimTypePointer(SimTypePointer(SimTypeChar(label="Byte"), offset=0), offset=0)}, name="IKEEXT_CERT_EKUS0", pack=False, align=None), offset=0), "name": SimTypePointer(SimStruct({"nameType": SimTypeInt(signed=False, label="IKEEXT_CERT_CRITERIA_NAME_TYPE"), "certName": SimTypePointer(SimTypeChar(label="Char"), offset=0)}, name="IKEEXT_CERT_NAME0", pack=False, align=None), offset=0), "flags": SimTypeInt(signed=False, label="UInt32")}, name="IKEEXT_CERTIFICATE_CRITERIA0", pack=False, align=None), offset=0)}, name="_Anonymous2_e__Struct", pack=False, align=None), "Anonymous3": SimStruct({"inboundRootStoreArraySize": SimTypeInt(signed=False, label="UInt32"), "inboundTrustedRootStoreCriteria": SimTypePointer(SimStruct({"certData": SimStruct({"size": SimTypeInt(signed=False, label="UInt32"), "data": SimTypePointer(SimTypeChar(label="Byte"), offset=0)}, name="FWP_BYTE_BLOB", pack=False, align=None), "certHash": SimStruct({"size": SimTypeInt(signed=False, label="UInt32"), "data": SimTypePointer(SimTypeChar(label="Byte"), offset=0)}, name="FWP_BYTE_BLOB", pack=False, align=None), "eku": SimTypePointer(SimStruct({"numEku": SimTypeInt(signed=False, label="UInt32"), "eku": SimTypePointer(SimTypePointer(SimTypeChar(label="Byte"), offset=0), offset=0)}, name="IKEEXT_CERT_EKUS0", pack=False, align=None), offset=0), "name": SimTypePointer(SimStruct({"nameType": SimTypeInt(signed=False, label="IKEEXT_CERT_CRITERIA_NAME_TYPE"), "certName": SimTypePointer(SimTypeChar(label="Char"), offset=0)}, name="IKEEXT_CERT_NAME0", pack=False, align=None), offset=0), "flags": SimTypeInt(signed=False, label="UInt32")}, name="IKEEXT_CERTIFICATE_CRITERIA0", pack=False, align=None), offset=0)}, name="_Anonymous3_e__Struct", pack=False, align=None)}, name="<anon>", label="None"), "outboundConfigType": SimTypeInt(signed=False, label="IKEEXT_CERT_CONFIG_TYPE"), "Anonymous2": SimUnion({"Anonymous1": SimStruct({"outboundRootArraySize": SimTypeInt(signed=False, label="UInt32"), "outboundRootCriteria": SimTypePointer(SimStruct({"certData": SimStruct({"size": SimTypeInt(signed=False, label="UInt32"), "data": SimTypePointer(SimTypeChar(label="Byte"), offset=0)}, name="FWP_BYTE_BLOB", pack=False, align=None), "certHash": SimStruct({"size": SimTypeInt(signed=False, label="UInt32"), "data": SimTypePointer(SimTypeChar(label="Byte"), offset=0)}, name="FWP_BYTE_BLOB", pack=False, align=None), "eku": SimTypePointer(SimStruct({"numEku": SimTypeInt(signed=False, label="UInt32"), "eku": SimTypePointer(SimTypePointer(SimTypeChar(label="Byte"), offset=0), offset=0)}, name="IKEEXT_CERT_EKUS0", pack=False, align=None), offset=0), "name": SimTypePointer(SimStruct({"nameType": SimTypeInt(signed=False, label="IKEEXT_CERT_CRITERIA_NAME_TYPE"), "certName": SimTypePointer(SimTypeChar(label="Char"), offset=0)}, name="IKEEXT_CERT_NAME0", pack=False, align=None), offset=0), "flags": SimTypeInt(signed=False, label="UInt32")}, name="IKEEXT_CERTIFICATE_CRITERIA0", pack=False, align=None), offset=0)}, name="_Anonymous1_e__Struct", pack=False, align=None), "Anonymous2": SimStruct({"outboundEnterpriseStoreArraySize": SimTypeInt(signed=False, label="UInt32"), "outboundEnterpriseStoreCriteria": SimTypePointer(SimStruct({"certData": SimStruct({"size": SimTypeInt(signed=False, label="UInt32"), "data": SimTypePointer(SimTypeChar(label="Byte"), offset=0)}, name="FWP_BYTE_BLOB", pack=False, align=None), "certHash": SimStruct({"size": SimTypeInt(signed=False, label="UInt32"), "data": SimTypePointer(SimTypeChar(label="Byte"), offset=0)}, name="FWP_BYTE_BLOB", pack=False, align=None), "eku": SimTypePointer(SimStruct({"numEku": SimTypeInt(signed=False, label="UInt32"), "eku": SimTypePointer(SimTypePointer(SimTypeChar(label="Byte"), offset=0), offset=0)}, name="IKEEXT_CERT_EKUS0", pack=False, align=None), offset=0), "name": SimTypePointer(SimStruct({"nameType": SimTypeInt(signed=False, label="IKEEXT_CERT_CRITERIA_NAME_TYPE"), "certName": SimTypePointer(SimTypeChar(label="Char"), offset=0)}, name="IKEEXT_CERT_NAME0", pack=False, align=None), offset=0), "flags": SimTypeInt(signed=False, label="UInt32")}, name="IKEEXT_CERTIFICATE_CRITERIA0", pack=False, align=None), offset=0)}, name="_Anonymous2_e__Struct", pack=False, align=None), "Anonymous3": SimStruct({"outboundRootStoreArraySize": SimTypeInt(signed=False, label="UInt32"), "outboundTrustedRootStoreCriteria": SimTypePointer(SimStruct({"certData": SimStruct({"size": SimTypeInt(signed=False, label="UInt32"), "data": SimTypePointer(SimTypeChar(label="Byte"), offset=0)}, name="FWP_BYTE_BLOB", pack=False, align=None), "certHash": SimStruct({"size": SimTypeInt(signed=False, label="UInt32"), "data": SimTypePointer(SimTypeChar(label="Byte"), offset=0)}, name="FWP_BYTE_BLOB", pack=False, align=None), "eku": SimTypePointer(SimStruct({"numEku": SimTypeInt(signed=False, label="UInt32"), "eku": SimTypePointer(SimTypePointer(SimTypeChar(label="Byte"), offset=0), offset=0)}, name="IKEEXT_CERT_EKUS0", pack=False, align=None), offset=0), "name": SimTypePointer(SimStruct({"nameType": SimTypeInt(signed=False, label="IKEEXT_CERT_CRITERIA_NAME_TYPE"), "certName": SimTypePointer(SimTypeChar(label="Char"), offset=0)}, name="IKEEXT_CERT_NAME0", pack=False, align=None), offset=0), "flags": SimTypeInt(signed=False, label="UInt32")}, name="IKEEXT_CERTIFICATE_CRITERIA0", pack=False, align=None), offset=0)}, name="_Anonymous3_e__Struct", pack=False, align=None)}, name="<anon>", label="None"), "flags": SimTypeInt(signed=False, label="IKEEXT_CERT_AUTH"), "localCertLocationUrl": SimStruct({"size": SimTypeInt(signed=False, label="UInt32"), "data": SimTypePointer(SimTypeChar(label="Byte"), offset=0)}, name="FWP_BYTE_BLOB", pack=False, align=None)}, name="IKEEXT_CERTIFICATE_AUTHENTICATION2", pack=False, align=None), "kerberosAuthentication": SimStruct({"flags": SimTypeInt(signed=False, label="IKEEXT_KERBEROS_AUTHENTICATION_FLAGS"), "proxyServer": SimTypePointer(SimTypeChar(label="Char"), offset=0)}, name="IKEEXT_KERBEROS_AUTHENTICATION1", pack=False, align=None), "reservedAuthentication": SimStruct({"flags": SimTypeInt(signed=False, label="IKEEXT_RESERVED_AUTHENTICATION_FLAGS")}, name="IKEEXT_RESERVED_AUTHENTICATION0", pack=False, align=None), "ntlmV2Authentication": SimStruct({"flags": SimTypeInt(signed=False, label="UInt32")}, name="IKEEXT_NTLM_V2_AUTHENTICATION0", pack=False, align=None), "sslAuthentication": SimStruct({"inboundConfigType": SimTypeInt(signed=False, label="IKEEXT_CERT_CONFIG_TYPE"), "Anonymous1": SimUnion({"Anonymous1": SimStruct({"inboundRootArraySize": SimTypeInt(signed=False, label="UInt32"), "inboundRootCriteria": SimTypePointer(SimStruct({"certData": SimStruct({"size": SimTypeInt(signed=False, label="UInt32"), "data": SimTypePointer(SimTypeChar(label="Byte"), offset=0)}, name="FWP_BYTE_BLOB", pack=False, align=None), "certHash": SimStruct({"size": SimTypeInt(signed=False, label="UInt32"), "data": SimTypePointer(SimTypeChar(label="Byte"), offset=0)}, name="FWP_BYTE_BLOB", pack=False, align=None), "eku": SimTypePointer(SimStruct({"numEku": SimTypeInt(signed=False, label="UInt32"), "eku": SimTypePointer(SimTypePointer(SimTypeChar(label="Byte"), offset=0), offset=0)}, name="IKEEXT_CERT_EKUS0", pack=False, align=None), offset=0), "name": SimTypePointer(SimStruct({"nameType": SimTypeInt(signed=False, label="IKEEXT_CERT_CRITERIA_NAME_TYPE"), "certName": SimTypePointer(SimTypeChar(label="Char"), offset=0)}, name="IKEEXT_CERT_NAME0", pack=False, align=None), offset=0), "flags": SimTypeInt(signed=False, label="UInt32")}, name="IKEEXT_CERTIFICATE_CRITERIA0", pack=False, align=None), offset=0)}, name="_Anonymous1_e__Struct", pack=False, align=None), "Anonymous2": SimStruct({"inboundEnterpriseStoreArraySize": SimTypeInt(signed=False, label="UInt32"), "inboundEnterpriseStoreCriteria": SimTypePointer(SimStruct({"certData": SimStruct({"size": SimTypeInt(signed=False, label="UInt32"), "data": SimTypePointer(SimTypeChar(label="Byte"), offset=0)}, name="FWP_BYTE_BLOB", pack=False, align=None), "certHash": SimStruct({"size": SimTypeInt(signed=False, label="UInt32"), "data": SimTypePointer(SimTypeChar(label="Byte"), offset=0)}, name="FWP_BYTE_BLOB", pack=False, align=None), "eku": SimTypePointer(SimStruct({"numEku": SimTypeInt(signed=False, label="UInt32"), "eku": SimTypePointer(SimTypePointer(SimTypeChar(label="Byte"), offset=0), offset=0)}, name="IKEEXT_CERT_EKUS0", pack=False, align=None), offset=0), "name": SimTypePointer(SimStruct({"nameType": SimTypeInt(signed=False, label="IKEEXT_CERT_CRITERIA_NAME_TYPE"), "certName": SimTypePointer(SimTypeChar(label="Char"), offset=0)}, name="IKEEXT_CERT_NAME0", pack=False, align=None), offset=0), "flags": SimTypeInt(signed=False, label="UInt32")}, name="IKEEXT_CERTIFICATE_CRITERIA0", pack=False, align=None), offset=0)}, name="_Anonymous2_e__Struct", pack=False, align=None), "Anonymous3": SimStruct({"inboundRootStoreArraySize": SimTypeInt(signed=False, label="UInt32"), "inboundTrustedRootStoreCriteria": SimTypePointer(SimStruct({"certData": SimStruct({"size": SimTypeInt(signed=False, label="UInt32"), "data": SimTypePointer(SimTypeChar(label="Byte"), offset=0)}, name="FWP_BYTE_BLOB", pack=False, align=None), "certHash": SimStruct({"size": SimTypeInt(signed=False, label="UInt32"), "data": SimTypePointer(SimTypeChar(label="Byte"), offset=0)}, name="FWP_BYTE_BLOB", pack=False, align=None), "eku": SimTypePointer(SimStruct({"numEku": SimTypeInt(signed=False, label="UInt32"), "eku": SimTypePointer(SimTypePointer(SimTypeChar(label="Byte"), offset=0), offset=0)}, name="IKEEXT_CERT_EKUS0", pack=False, align=None), offset=0), "name": SimTypePointer(SimStruct({"nameType": SimTypeInt(signed=False, label="IKEEXT_CERT_CRITERIA_NAME_TYPE"), "certName": SimTypePointer(SimTypeChar(label="Char"), offset=0)}, name="IKEEXT_CERT_NAME0", pack=False, align=None), offset=0), "flags": SimTypeInt(signed=False, label="UInt32")}, name="IKEEXT_CERTIFICATE_CRITERIA0", pack=False, align=None), offset=0)}, name="_Anonymous3_e__Struct", pack=False, align=None)}, name="<anon>", label="None"), "outboundConfigType": SimTypeInt(signed=False, label="IKEEXT_CERT_CONFIG_TYPE"), "Anonymous2": SimUnion({"Anonymous1": SimStruct({"outboundRootArraySize": SimTypeInt(signed=False, label="UInt32"), "outboundRootCriteria": SimTypePointer(SimStruct({"certData": SimStruct({"size": SimTypeInt(signed=False, label="UInt32"), "data": SimTypePointer(SimTypeChar(label="Byte"), offset=0)}, name="FWP_BYTE_BLOB", pack=False, align=None), "certHash": SimStruct({"size": SimTypeInt(signed=False, label="UInt32"), "data": SimTypePointer(SimTypeChar(label="Byte"), offset=0)}, name="FWP_BYTE_BLOB", pack=False, align=None), "eku": SimTypePointer(SimStruct({"numEku": SimTypeInt(signed=False, label="UInt32"), "eku": SimTypePointer(SimTypePointer(SimTypeChar(label="Byte"), offset=0), offset=0)}, name="IKEEXT_CERT_EKUS0", pack=False, align=None), offset=0), "name": SimTypePointer(SimStruct({"nameType": SimTypeInt(signed=False, label="IKEEXT_CERT_CRITERIA_NAME_TYPE"), "certName": SimTypePointer(SimTypeChar(label="Char"), offset=0)}, name="IKEEXT_CERT_NAME0", pack=False, align=None), offset=0), "flags": SimTypeInt(signed=False, label="UInt32")}, name="IKEEXT_CERTIFICATE_CRITERIA0", pack=False, align=None), offset=0)}, name="_Anonymous1_e__Struct", pack=False, align=None), "Anonymous2": SimStruct({"outboundEnterpriseStoreArraySize": SimTypeInt(signed=False, label="UInt32"), "outboundEnterpriseStoreCriteria": SimTypePointer(SimStruct({"certData": SimStruct({"size": SimTypeInt(signed=False, label="UInt32"), "data": SimTypePointer(SimTypeChar(label="Byte"), offset=0)}, name="FWP_BYTE_BLOB", pack=False, align=None), "certHash": SimStruct({"size": SimTypeInt(signed=False, label="UInt32"), "data": SimTypePointer(SimTypeChar(label="Byte"), offset=0)}, name="FWP_BYTE_BLOB", pack=False, align=None), "eku": SimTypePointer(SimStruct({"numEku": SimTypeInt(signed=False, label="UInt32"), "eku": SimTypePointer(SimTypePointer(SimTypeChar(label="Byte"), offset=0), offset=0)}, name="IKEEXT_CERT_EKUS0", pack=False, align=None), offset=0), "name": SimTypePointer(SimStruct({"nameType": SimTypeInt(signed=False, label="IKEEXT_CERT_CRITERIA_NAME_TYPE"), "certName": SimTypePointer(SimTypeChar(label="Char"), offset=0)}, name="IKEEXT_CERT_NAME0", pack=False, align=None), offset=0), "flags": SimTypeInt(signed=False, label="UInt32")}, name="IKEEXT_CERTIFICATE_CRITERIA0", pack=False, align=None), offset=0)}, name="_Anonymous2_e__Struct", pack=False, align=None), "Anonymous3": SimStruct({"outboundRootStoreArraySize": SimTypeInt(signed=False, label="UInt32"), "outboundTrustedRootStoreCriteria": SimTypePointer(SimStruct({"certData": SimStruct({"size": SimTypeInt(signed=False, label="UInt32"), "data": SimTypePointer(SimTypeChar(label="Byte"), offset=0)}, name="FWP_BYTE_BLOB", pack=False, align=None), "certHash": SimStruct({"size": SimTypeInt(signed=False, label="UInt32"), "data": SimTypePointer(SimTypeChar(label="Byte"), offset=0)}, name="FWP_BYTE_BLOB", pack=False, align=None), "eku": SimTypePointer(SimStruct({"numEku": SimTypeInt(signed=False, label="UInt32"), "eku": SimTypePointer(SimTypePointer(SimTypeChar(label="Byte"), offset=0), offset=0)}, name="IKEEXT_CERT_EKUS0", pack=False, align=None), offset=0), "name": SimTypePointer(SimStruct({"nameType": SimTypeInt(signed=False, label="IKEEXT_CERT_CRITERIA_NAME_TYPE"), "certName": SimTypePointer(SimTypeChar(label="Char"), offset=0)}, name="IKEEXT_CERT_NAME0", pack=False, align=None), offset=0), "flags": SimTypeInt(signed=False, label="UInt32")}, name="IKEEXT_CERTIFICATE_CRITERIA0", pack=False, align=None), offset=0)}, name="_Anonymous3_e__Struct", pack=False, align=None)}, name="<anon>", label="None"), "flags": SimTypeInt(signed=False, label="IKEEXT_CERT_AUTH"), "localCertLocationUrl": SimStruct({"size": SimTypeInt(signed=False, label="UInt32"), "data": SimTypePointer(SimTypeChar(label="Byte"), offset=0)}, name="FWP_BYTE_BLOB", pack=False, align=None)}, name="IKEEXT_CERTIFICATE_AUTHENTICATION2", pack=False, align=None), "cgaAuthentication": SimStruct({"keyContainerName": SimTypePointer(SimTypeChar(label="Char"), offset=0), "cspName": SimTypePointer(SimTypeChar(label="Char"), offset=0), "cspType": SimTypeInt(signed=False, label="UInt32"), "cgaModifier": SimStruct({"byteArray16": SimTypeFixedSizeArray(SimTypeChar(label="Byte"), 16)}, name="FWP_BYTE_ARRAY16", pack=False, align=None), "cgaCollisionCount": SimTypeChar(label="Byte")}, name="IKEEXT_IPV6_CGA_AUTHENTICATION0", pack=False, align=None), "eapAuthentication": SimStruct({"flags": SimTypeInt(signed=False, label="IKEEXT_EAP_AUTHENTICATION_FLAGS")}, name="IKEEXT_EAP_AUTHENTICATION0", pack=False, align=None)}, name="<anon>", label="None")}, name="IKEEXT_AUTHENTICATION_METHOD2", pack=False, align=None), offset=0), "initiatorImpersonationType": SimTypeInt(signed=False, label="IKEEXT_AUTHENTICATION_IMPERSONATION_TYPE"), "numIkeProposals": SimTypeInt(signed=False, label="UInt32"), "ikeProposals": SimTypePointer(SimStruct({"cipherAlgorithm": SimStruct({"algoIdentifier": SimTypeInt(signed=False, label="IKEEXT_CIPHER_TYPE"), "keyLen": SimTypeInt(signed=False, label="UInt32"), "rounds": SimTypeInt(signed=False, label="UInt32")}, name="IKEEXT_CIPHER_ALGORITHM0", pack=False, align=None), "integrityAlgorithm": SimStruct({"algoIdentifier": SimTypeInt(signed=False, label="IKEEXT_INTEGRITY_TYPE")}, name="IKEEXT_INTEGRITY_ALGORITHM0", pack=False, align=None), "maxLifetimeSeconds": SimTypeInt(signed=False, label="UInt32"), "dhGroup": SimTypeInt(signed=False, label="IKEEXT_DH_GROUP"), "quickModeLimit": SimTypeInt(signed=False, label="UInt32")}, name="IKEEXT_PROPOSAL0", pack=False, align=None), offset=0), "flags": SimTypeInt(signed=False, label="IKEEXT_POLICY_FLAG"), "maxDynamicFilters": SimTypeInt(signed=False, label="UInt32"), "retransmitDurationSecs": SimTypeInt(signed=False, label="UInt32")}, name="IKEEXT_POLICY2", pack=False, align=None), offset=0), "dataBuffer": SimTypePointer(SimStruct({"size": SimTypeInt(signed=False, label="UInt32"), "data": SimTypePointer(SimTypeChar(label="Byte"), offset=0)}, name="FWP_BYTE_BLOB", pack=False, align=None), offset=0), "classifyOptions": SimTypePointer(SimStruct({"numOptions": SimTypeInt(signed=False, label="UInt32"), "options": SimTypePointer(SimStruct({"type": SimTypeInt(signed=False, label="FWP_CLASSIFY_OPTION_TYPE"), "value": SimStruct({"type": SimTypeInt(signed=False, label="FWP_DATA_TYPE"), "Anonymous": SimUnion({"uint8": SimTypeChar(label="Byte"), "uint16": SimTypeShort(signed=False, label="UInt16"), "uint32": SimTypeInt(signed=False, label="UInt32"), "uint64": SimTypePointer(SimTypeLongLong(signed=False, label="UInt64"), offset=0), "int8": SimTypeChar(label="SByte"), "int16": SimTypeShort(signed=True, label="Int16"), "int32": SimTypeInt(signed=True, label="Int32"), "int64": SimTypePointer(SimTypeLongLong(signed=True, label="Int64"), offset=0), "float32": SimTypeFloat(size=32), "double64": SimTypePointer(SimTypeFloat(size=64), offset=0), "byteArray16": SimTypePointer(SimStruct({"byteArray16": SimTypeFixedSizeArray(SimTypeChar(label="Byte"), 16)}, name="FWP_BYTE_ARRAY16", pack=False, align=None), offset=0), "byteBlob": SimTypePointer(SimStruct({"size": SimTypeInt(signed=False, label="UInt32"), "data": SimTypePointer(SimTypeChar(label="Byte"), offset=0)}, name="FWP_BYTE_BLOB", pack=False, align=None), offset=0), "sid": SimTypePointer(SimTypeBottom(label="SID"), offset=0), "sd": SimTypePointer(SimStruct({"size": SimTypeInt(signed=False, label="UInt32"), "data": SimTypePointer(SimTypeChar(label="Byte"), offset=0)}, name="FWP_BYTE_BLOB", pack=False, align=None), offset=0), "tokenInformation": SimTypePointer(SimStruct({"sidCount": SimTypeInt(signed=False, label="UInt32"), "sids": SimTypePointer(SimTypeBottom(label="SID_AND_ATTRIBUTES"), offset=0), "restrictedSidCount": SimTypeInt(signed=False, label="UInt32"), "restrictedSids": SimTypePointer(SimTypeBottom(label="SID_AND_ATTRIBUTES"), offset=0)}, name="FWP_TOKEN_INFORMATION", pack=False, align=None), offset=0), "tokenAccessInformation": SimTypePointer(SimStruct({"size": SimTypeInt(signed=False, label="UInt32"), "data": SimTypePointer(SimTypeChar(label="Byte"), offset=0)}, name="FWP_BYTE_BLOB", pack=False, align=None), offset=0), "unicodeString": SimTypePointer(SimTypeChar(label="Char"), offset=0), "byteArray6": SimTypePointer(SimStruct({"byteArray6": SimTypeFixedSizeArray(SimTypeChar(label="Byte"), 6)}, name="FWP_BYTE_ARRAY6", pack=False, align=None), offset=0), "bitmapArray64": SimTypePointer(SimStruct({"bitmapArray64": SimTypeFixedSizeArray(SimTypeChar(label="Byte"), 8)}, name="FWP_BITMAP_ARRAY64_", pack=False, align=None), offset=0)}, name="<anon>", label="None")}, name="FWP_VALUE0", pack=False, align=None)}, name="FWPM_CLASSIFY_OPTION0", pack=False, align=None), offset=0)}, name="FWPM_CLASSIFY_OPTIONS0", pack=False, align=None), offset=0), "ikeV2QmTunnelPolicy": SimTypePointer(SimStruct({"flags": SimTypeInt(signed=False, label="IPSEC_POLICY_FLAG"), "numIpsecProposals": SimTypeInt(signed=False, label="UInt32"), "ipsecProposals": SimTypePointer(SimStruct({"lifetime": SimStruct({"lifetimeSeconds": SimTypeInt(signed=False, label="UInt32"), "lifetimeKilobytes": SimTypeInt(signed=False, label="UInt32"), "lifetimePackets": SimTypeInt(signed=False, label="UInt32")}, name="IPSEC_SA_LIFETIME0", pack=False, align=None), "numSaTransforms": SimTypeInt(signed=False, label="UInt32"), "saTransforms": SimTypePointer(SimStruct({"ipsecTransformType": SimTypeInt(signed=False, label="IPSEC_TRANSFORM_TYPE"), "Anonymous": SimUnion({"ahTransform": SimTypePointer(SimStruct({"authTransformId": SimStruct({"authType": SimTypeInt(signed=False, label="IPSEC_AUTH_TYPE"), "authConfig": SimTypeChar(label="Byte")}, name="IPSEC_AUTH_TRANSFORM_ID0", pack=False, align=None), "cryptoModuleId": SimTypePointer(SimTypeBottom(label="Guid"), offset=0)}, name="IPSEC_AUTH_TRANSFORM0", pack=False, align=None), offset=0), "espAuthTransform": SimTypePointer(SimStruct({"authTransformId": SimStruct({"authType": SimTypeInt(signed=False, label="IPSEC_AUTH_TYPE"), "authConfig": SimTypeChar(label="Byte")}, name="IPSEC_AUTH_TRANSFORM_ID0", pack=False, align=None), "cryptoModuleId": SimTypePointer(SimTypeBottom(label="Guid"), offset=0)}, name="IPSEC_AUTH_TRANSFORM0", pack=False, align=None), offset=0), "espCipherTransform": SimTypePointer(SimStruct({"cipherTransformId": SimStruct({"cipherType": SimTypeInt(signed=False, label="IPSEC_CIPHER_TYPE"), "cipherConfig": SimTypeChar(label="Byte")}, name="IPSEC_CIPHER_TRANSFORM_ID0", pack=False, align=None), "cryptoModuleId": SimTypePointer(SimTypeBottom(label="Guid"), offset=0)}, name="IPSEC_CIPHER_TRANSFORM0", pack=False, align=None), offset=0), "espAuthAndCipherTransform": SimTypePointer(SimStruct({"authTransform": SimStruct({"authTransformId": SimStruct({"authType": SimTypeInt(signed=False, label="IPSEC_AUTH_TYPE"), "authConfig": SimTypeChar(label="Byte")}, name="IPSEC_AUTH_TRANSFORM_ID0", pack=False, align=None), "cryptoModuleId": SimTypePointer(SimTypeBottom(label="Guid"), offset=0)}, name="IPSEC_AUTH_TRANSFORM0", pack=False, align=None), "cipherTransform": SimStruct({"cipherTransformId": SimStruct({"cipherType": SimTypeInt(signed=False, label="IPSEC_CIPHER_TYPE"), "cipherConfig": SimTypeChar(label="Byte")}, name="IPSEC_CIPHER_TRANSFORM_ID0", pack=False, align=None), "cryptoModuleId": SimTypePointer(SimTypeBottom(label="Guid"), offset=0)}, name="IPSEC_CIPHER_TRANSFORM0", pack=False, align=None)}, name="IPSEC_AUTH_AND_CIPHER_TRANSFORM0", pack=False, align=None), offset=0), "espAuthFwTransform": SimTypePointer(SimStruct({"authTransformId": SimStruct({"authType": SimTypeInt(signed=False, label="IPSEC_AUTH_TYPE"), "authConfig": SimTypeChar(label="Byte")}, name="IPSEC_AUTH_TRANSFORM_ID0", pack=False, align=None), "cryptoModuleId": SimTypePointer(SimTypeBottom(label="Guid"), offset=0)}, name="IPSEC_AUTH_TRANSFORM0", pack=False, align=None), offset=0)}, name="<anon>", label="None")}, name="IPSEC_SA_TRANSFORM0", pack=False, align=None), offset=0), "pfsGroup": SimTypeInt(signed=False, label="IPSEC_PFS_GROUP")}, name="IPSEC_PROPOSAL0", pack=False, align=None), offset=0), "tunnelEndpoints": SimStruct({"ipVersion": SimTypeInt(signed=False, label="FWP_IP_VERSION"), "Anonymous1": SimUnion({"localV4Address": SimTypeInt(signed=False, label="UInt32"), "localV6Address": SimTypeFixedSizeArray(SimTypeChar(label="Byte"), 16)}, name="<anon>", label="None"), "Anonymous2": SimUnion({"remoteV4Address": SimTypeInt(signed=False, label="UInt32"), "remoteV6Address": SimTypeFixedSizeArray(SimTypeChar(label="Byte"), 16)}, name="<anon>", label="None"), "localIfLuid": SimTypeLongLong(signed=False, label="UInt64"), "remoteFqdn": SimTypePointer(SimTypeChar(label="Char"), offset=0), "numAddresses": SimTypeInt(signed=False, label="UInt32"), "remoteAddresses": SimTypePointer(SimStruct({"ipVersion": SimTypeInt(signed=False, label="FWP_IP_VERSION"), "Anonymous": SimUnion({"v4Address": SimTypeInt(signed=False, label="UInt32"), "v6Address": SimTypeFixedSizeArray(SimTypeChar(label="Byte"), 16)}, name="<anon>", label="None")}, name="IPSEC_TUNNEL_ENDPOINT0", pack=False, align=None), offset=0)}, name="IPSEC_TUNNEL_ENDPOINTS2", pack=False, align=None), "saIdleTimeout": SimStruct({"idleTimeoutSeconds": SimTypeInt(signed=False, label="UInt32"), "idleTimeoutSecondsFailOver": SimTypeInt(signed=False, label="UInt32")}, name="IPSEC_SA_IDLE_TIMEOUT0", pack=False, align=None), "emPolicy": SimTypePointer(SimStruct({"numAuthenticationMethods": SimTypeInt(signed=False, label="UInt32"), "authenticationMethods": SimTypePointer(SimStruct({"authenticationMethodType": SimTypeInt(signed=False, label="IKEEXT_AUTHENTICATION_METHOD_TYPE"), "Anonymous": SimUnion({"presharedKeyAuthentication": SimStruct({"presharedKey": SimStruct({"size": SimTypeInt(signed=False, label="UInt32"), "data": SimTypePointer(SimTypeChar(label="Byte"), offset=0)}, name="FWP_BYTE_BLOB", pack=False, align=None), "flags": SimTypeInt(signed=False, label="IKEEXT_PRESHARED_KEY_AUTHENTICATION_FLAGS")}, name="IKEEXT_PRESHARED_KEY_AUTHENTICATION1", pack=False, align=None), "certificateAuthentication": SimStruct({"inboundConfigType": SimTypeInt(signed=False, label="IKEEXT_CERT_CONFIG_TYPE"), "Anonymous1": SimUnion({"Anonymous1": SimStruct({"inboundRootArraySize": SimTypeInt(signed=False, label="UInt32"), "inboundRootCriteria": SimTypePointer(SimStruct({"certData": SimStruct({"size": SimTypeInt(signed=False, label="UInt32"), "data": SimTypePointer(SimTypeChar(label="Byte"), offset=0)}, name="FWP_BYTE_BLOB", pack=False, align=None), "certHash": SimStruct({"size": SimTypeInt(signed=False, label="UInt32"), "data": SimTypePointer(SimTypeChar(label="Byte"), offset=0)}, name="FWP_BYTE_BLOB", pack=False, align=None), "eku": SimTypePointer(SimStruct({"numEku": SimTypeInt(signed=False, label="UInt32"), "eku": SimTypePointer(SimTypePointer(SimTypeChar(label="Byte"), offset=0), offset=0)}, name="IKEEXT_CERT_EKUS0", pack=False, align=None), offset=0), "name": SimTypePointer(SimStruct({"nameType": SimTypeInt(signed=False, label="IKEEXT_CERT_CRITERIA_NAME_TYPE"), "certName": SimTypePointer(SimTypeChar(label="Char"), offset=0)}, name="IKEEXT_CERT_NAME0", pack=False, align=None), offset=0), "flags": SimTypeInt(signed=False, label="UInt32")}, name="IKEEXT_CERTIFICATE_CRITERIA0", pack=False, align=None), offset=0)}, name="_Anonymous1_e__Struct", pack=False, align=None), "Anonymous2": SimStruct({"inboundEnterpriseStoreArraySize": SimTypeInt(signed=False, label="UInt32"), "inboundEnterpriseStoreCriteria": SimTypePointer(SimStruct({"certData": SimStruct({"size": SimTypeInt(signed=False, label="UInt32"), "data": SimTypePointer(SimTypeChar(label="Byte"), offset=0)}, name="FWP_BYTE_BLOB", pack=False, align=None), "certHash": SimStruct({"size": SimTypeInt(signed=False, label="UInt32"), "data": SimTypePointer(SimTypeChar(label="Byte"), offset=0)}, name="FWP_BYTE_BLOB", pack=False, align=None), "eku": SimTypePointer(SimStruct({"numEku": SimTypeInt(signed=False, label="UInt32"), "eku": SimTypePointer(SimTypePointer(SimTypeChar(label="Byte"), offset=0), offset=0)}, name="IKEEXT_CERT_EKUS0", pack=False, align=None), offset=0), "name": SimTypePointer(SimStruct({"nameType": SimTypeInt(signed=False, label="IKEEXT_CERT_CRITERIA_NAME_TYPE"), "certName": SimTypePointer(SimTypeChar(label="Char"), offset=0)}, name="IKEEXT_CERT_NAME0", pack=False, align=None), offset=0), "flags": SimTypeInt(signed=False, label="UInt32")}, name="IKEEXT_CERTIFICATE_CRITERIA0", pack=False, align=None), offset=0)}, name="_Anonymous2_e__Struct", pack=False, align=None), "Anonymous3": SimStruct({"inboundRootStoreArraySize": SimTypeInt(signed=False, label="UInt32"), "inboundTrustedRootStoreCriteria": SimTypePointer(SimStruct({"certData": SimStruct({"size": SimTypeInt(signed=False, label="UInt32"), "data": SimTypePointer(SimTypeChar(label="Byte"), offset=0)}, name="FWP_BYTE_BLOB", pack=False, align=None), "certHash": SimStruct({"size": SimTypeInt(signed=False, label="UInt32"), "data": SimTypePointer(SimTypeChar(label="Byte"), offset=0)}, name="FWP_BYTE_BLOB", pack=False, align=None), "eku": SimTypePointer(SimStruct({"numEku": SimTypeInt(signed=False, label="UInt32"), "eku": SimTypePointer(SimTypePointer(SimTypeChar(label="Byte"), offset=0), offset=0)}, name="IKEEXT_CERT_EKUS0", pack=False, align=None), offset=0), "name": SimTypePointer(SimStruct({"nameType": SimTypeInt(signed=False, label="IKEEXT_CERT_CRITERIA_NAME_TYPE"), "certName": SimTypePointer(SimTypeChar(label="Char"), offset=0)}, name="IKEEXT_CERT_NAME0", pack=False, align=None), offset=0), "flags": SimTypeInt(signed=False, label="UInt32")}, name="IKEEXT_CERTIFICATE_CRITERIA0", pack=False, align=None), offset=0)}, name="_Anonymous3_e__Struct", pack=False, align=None)}, name="<anon>", label="None"), "outboundConfigType": SimTypeInt(signed=False, label="IKEEXT_CERT_CONFIG_TYPE"), "Anonymous2": SimUnion({"Anonymous1": SimStruct({"outboundRootArraySize": SimTypeInt(signed=False, label="UInt32"), "outboundRootCriteria": SimTypePointer(SimStruct({"certData": SimStruct({"size": SimTypeInt(signed=False, label="UInt32"), "data": SimTypePointer(SimTypeChar(label="Byte"), offset=0)}, name="FWP_BYTE_BLOB", pack=False, align=None), "certHash": SimStruct({"size": SimTypeInt(signed=False, label="UInt32"), "data": SimTypePointer(SimTypeChar(label="Byte"), offset=0)}, name="FWP_BYTE_BLOB", pack=False, align=None), "eku": SimTypePointer(SimStruct({"numEku": SimTypeInt(signed=False, label="UInt32"), "eku": SimTypePointer(SimTypePointer(SimTypeChar(label="Byte"), offset=0), offset=0)}, name="IKEEXT_CERT_EKUS0", pack=False, align=None), offset=0), "name": SimTypePointer(SimStruct({"nameType": SimTypeInt(signed=False, label="IKEEXT_CERT_CRITERIA_NAME_TYPE"), "certName": SimTypePointer(SimTypeChar(label="Char"), offset=0)}, name="IKEEXT_CERT_NAME0", pack=False, align=None), offset=0), "flags": SimTypeInt(signed=False, label="UInt32")}, name="IKEEXT_CERTIFICATE_CRITERIA0", pack=False, align=None), offset=0)}, name="_Anonymous1_e__Struct", pack=False, align=None), "Anonymous2": SimStruct({"outboundEnterpriseStoreArraySize": SimTypeInt(signed=False, label="UInt32"), "outboundEnterpriseStoreCriteria": SimTypePointer(SimStruct({"certData": SimStruct({"size": SimTypeInt(signed=False, label="UInt32"), "data": SimTypePointer(SimTypeChar(label="Byte"), offset=0)}, name="FWP_BYTE_BLOB", pack=False, align=None), "certHash": SimStruct({"size": SimTypeInt(signed=False, label="UInt32"), "data": SimTypePointer(SimTypeChar(label="Byte"), offset=0)}, name="FWP_BYTE_BLOB", pack=False, align=None), "eku": SimTypePointer(SimStruct({"numEku": SimTypeInt(signed=False, label="UInt32"), "eku": SimTypePointer(SimTypePointer(SimTypeChar(label="Byte"), offset=0), offset=0)}, name="IKEEXT_CERT_EKUS0", pack=False, align=None), offset=0), "name": SimTypePointer(SimStruct({"nameType": SimTypeInt(signed=False, label="IKEEXT_CERT_CRITERIA_NAME_TYPE"), "certName": SimTypePointer(SimTypeChar(label="Char"), offset=0)}, name="IKEEXT_CERT_NAME0", pack=False, align=None), offset=0), "flags": SimTypeInt(signed=False, label="UInt32")}, name="IKEEXT_CERTIFICATE_CRITERIA0", pack=False, align=None), offset=0)}, name="_Anonymous2_e__Struct", pack=False, align=None), "Anonymous3": SimStruct({"outboundRootStoreArraySize": SimTypeInt(signed=False, label="UInt32"), "outboundTrustedRootStoreCriteria": SimTypePointer(SimStruct({"certData": SimStruct({"size": SimTypeInt(signed=False, label="UInt32"), "data": SimTypePointer(SimTypeChar(label="Byte"), offset=0)}, name="FWP_BYTE_BLOB", pack=False, align=None), "certHash": SimStruct({"size": SimTypeInt(signed=False, label="UInt32"), "data": SimTypePointer(SimTypeChar(label="Byte"), offset=0)}, name="FWP_BYTE_BLOB", pack=False, align=None), "eku": SimTypePointer(SimStruct({"numEku": SimTypeInt(signed=False, label="UInt32"), "eku": SimTypePointer(SimTypePointer(SimTypeChar(label="Byte"), offset=0), offset=0)}, name="IKEEXT_CERT_EKUS0", pack=False, align=None), offset=0), "name": SimTypePointer(SimStruct({"nameType": SimTypeInt(signed=False, label="IKEEXT_CERT_CRITERIA_NAME_TYPE"), "certName": SimTypePointer(SimTypeChar(label="Char"), offset=0)}, name="IKEEXT_CERT_NAME0", pack=False, align=None), offset=0), "flags": SimTypeInt(signed=False, label="UInt32")}, name="IKEEXT_CERTIFICATE_CRITERIA0", pack=False, align=None), offset=0)}, name="_Anonymous3_e__Struct", pack=False, align=None)}, name="<anon>", label="None"), "flags": SimTypeInt(signed=False, label="IKEEXT_CERT_AUTH"), "localCertLocationUrl": SimStruct({"size": SimTypeInt(signed=False, label="UInt32"), "data": SimTypePointer(SimTypeChar(label="Byte"), offset=0)}, name="FWP_BYTE_BLOB", pack=False, align=None)}, name="IKEEXT_CERTIFICATE_AUTHENTICATION2", pack=False, align=None), "kerberosAuthentication": SimStruct({"flags": SimTypeInt(signed=False, label="IKEEXT_KERBEROS_AUTHENTICATION_FLAGS"), "proxyServer": SimTypePointer(SimTypeChar(label="Char"), offset=0)}, name="IKEEXT_KERBEROS_AUTHENTICATION1", pack=False, align=None), "reservedAuthentication": SimStruct({"flags": SimTypeInt(signed=False, label="IKEEXT_RESERVED_AUTHENTICATION_FLAGS")}, name="IKEEXT_RESERVED_AUTHENTICATION0", pack=False, align=None), "ntlmV2Authentication": SimStruct({"flags": SimTypeInt(signed=False, label="UInt32")}, name="IKEEXT_NTLM_V2_AUTHENTICATION0", pack=False, align=None), "sslAuthentication": SimStruct({"inboundConfigType": SimTypeInt(signed=False, label="IKEEXT_CERT_CONFIG_TYPE"), "Anonymous1": SimUnion({"Anonymous1": SimStruct({"inboundRootArraySize": SimTypeInt(signed=False, label="UInt32"), "inboundRootCriteria": SimTypePointer(SimStruct({"certData": SimStruct({"size": SimTypeInt(signed=False, label="UInt32"), "data": SimTypePointer(SimTypeChar(label="Byte"), offset=0)}, name="FWP_BYTE_BLOB", pack=False, align=None), "certHash": SimStruct({"size": SimTypeInt(signed=False, label="UInt32"), "data": SimTypePointer(SimTypeChar(label="Byte"), offset=0)}, name="FWP_BYTE_BLOB", pack=False, align=None), "eku": SimTypePointer(SimStruct({"numEku": SimTypeInt(signed=False, label="UInt32"), "eku": SimTypePointer(SimTypePointer(SimTypeChar(label="Byte"), offset=0), offset=0)}, name="IKEEXT_CERT_EKUS0", pack=False, align=None), offset=0), "name": SimTypePointer(SimStruct({"nameType": SimTypeInt(signed=False, label="IKEEXT_CERT_CRITERIA_NAME_TYPE"), "certName": SimTypePointer(SimTypeChar(label="Char"), offset=0)}, name="IKEEXT_CERT_NAME0", pack=False, align=None), offset=0), "flags": SimTypeInt(signed=False, label="UInt32")}, name="IKEEXT_CERTIFICATE_CRITERIA0", pack=False, align=None), offset=0)}, name="_Anonymous1_e__Struct", pack=False, align=None), "Anonymous2": SimStruct({"inboundEnterpriseStoreArraySize": SimTypeInt(signed=False, label="UInt32"), "inboundEnterpriseStoreCriteria": SimTypePointer(SimStruct({"certData": SimStruct({"size": SimTypeInt(signed=False, label="UInt32"), "data": SimTypePointer(SimTypeChar(label="Byte"), offset=0)}, name="FWP_BYTE_BLOB", pack=False, align=None), "certHash": SimStruct({"size": SimTypeInt(signed=False, label="UInt32"), "data": SimTypePointer(SimTypeChar(label="Byte"), offset=0)}, name="FWP_BYTE_BLOB", pack=False, align=None), "eku": SimTypePointer(SimStruct({"numEku": SimTypeInt(signed=False, label="UInt32"), "eku": SimTypePointer(SimTypePointer(SimTypeChar(label="Byte"), offset=0), offset=0)}, name="IKEEXT_CERT_EKUS0", pack=False, align=None), offset=0), "name": SimTypePointer(SimStruct({"nameType": SimTypeInt(signed=False, label="IKEEXT_CERT_CRITERIA_NAME_TYPE"), "certName": SimTypePointer(SimTypeChar(label="Char"), offset=0)}, name="IKEEXT_CERT_NAME0", pack=False, align=None), offset=0), "flags": SimTypeInt(signed=False, label="UInt32")}, name="IKEEXT_CERTIFICATE_CRITERIA0", pack=False, align=None), offset=0)}, name="_Anonymous2_e__Struct", pack=False, align=None), "Anonymous3": SimStruct({"inboundRootStoreArraySize": SimTypeInt(signed=False, label="UInt32"), "inboundTrustedRootStoreCriteria": SimTypePointer(SimStruct({"certData": SimStruct({"size": SimTypeInt(signed=False, label="UInt32"), "data": SimTypePointer(SimTypeChar(label="Byte"), offset=0)}, name="FWP_BYTE_BLOB", pack=False, align=None), "certHash": SimStruct({"size": SimTypeInt(signed=False, label="UInt32"), "data": SimTypePointer(SimTypeChar(label="Byte"), offset=0)}, name="FWP_BYTE_BLOB", pack=False, align=None), "eku": SimTypePointer(SimStruct({"numEku": SimTypeInt(signed=False, label="UInt32"), "eku": SimTypePointer(SimTypePointer(SimTypeChar(label="Byte"), offset=0), offset=0)}, name="IKEEXT_CERT_EKUS0", pack=False, align=None), offset=0), "name": SimTypePointer(SimStruct({"nameType": SimTypeInt(signed=False, label="IKEEXT_CERT_CRITERIA_NAME_TYPE"), "certName": SimTypePointer(SimTypeChar(label="Char"), offset=0)}, name="IKEEXT_CERT_NAME0", pack=False, align=None), offset=0), "flags": SimTypeInt(signed=False, label="UInt32")}, name="IKEEXT_CERTIFICATE_CRITERIA0", pack=False, align=None), offset=0)}, name="_Anonymous3_e__Struct", pack=False, align=None)}, name="<anon>", label="None"), "outboundConfigType": SimTypeInt(signed=False, label="IKEEXT_CERT_CONFIG_TYPE"), "Anonymous2": SimUnion({"Anonymous1": SimStruct({"outboundRootArraySize": SimTypeInt(signed=False, label="UInt32"), "outboundRootCriteria": SimTypePointer(SimStruct({"certData": SimStruct({"size": SimTypeInt(signed=False, label="UInt32"), "data": SimTypePointer(SimTypeChar(label="Byte"), offset=0)}, name="FWP_BYTE_BLOB", pack=False, align=None), "certHash": SimStruct({"size": SimTypeInt(signed=False, label="UInt32"), "data": SimTypePointer(SimTypeChar(label="Byte"), offset=0)}, name="FWP_BYTE_BLOB", pack=False, align=None), "eku": SimTypePointer(SimStruct({"numEku": SimTypeInt(signed=False, label="UInt32"), "eku": SimTypePointer(SimTypePointer(SimTypeChar(label="Byte"), offset=0), offset=0)}, name="IKEEXT_CERT_EKUS0", pack=False, align=None), offset=0), "name": SimTypePointer(SimStruct({"nameType": SimTypeInt(signed=False, label="IKEEXT_CERT_CRITERIA_NAME_TYPE"), "certName": SimTypePointer(SimTypeChar(label="Char"), offset=0)}, name="IKEEXT_CERT_NAME0", pack=False, align=None), offset=0), "flags": SimTypeInt(signed=False, label="UInt32")}, name="IKEEXT_CERTIFICATE_CRITERIA0", pack=False, align=None), offset=0)}, name="_Anonymous1_e__Struct", pack=False, align=None), "Anonymous2": SimStruct({"outboundEnterpriseStoreArraySize": SimTypeInt(signed=False, label="UInt32"), "outboundEnterpriseStoreCriteria": SimTypePointer(SimStruct({"certData": SimStruct({"size": SimTypeInt(signed=False, label="UInt32"), "data": SimTypePointer(SimTypeChar(label="Byte"), offset=0)}, name="FWP_BYTE_BLOB", pack=False, align=None), "certHash": SimStruct({"size": SimTypeInt(signed=False, label="UInt32"), "data": SimTypePointer(SimTypeChar(label="Byte"), offset=0)}, name="FWP_BYTE_BLOB", pack=False, align=None), "eku": SimTypePointer(SimStruct({"numEku": SimTypeInt(signed=False, label="UInt32"), "eku": SimTypePointer(SimTypePointer(SimTypeChar(label="Byte"), offset=0), offset=0)}, name="IKEEXT_CERT_EKUS0", pack=False, align=None), offset=0), "name": SimTypePointer(SimStruct({"nameType": SimTypeInt(signed=False, label="IKEEXT_CERT_CRITERIA_NAME_TYPE"), "certName": SimTypePointer(SimTypeChar(label="Char"), offset=0)}, name="IKEEXT_CERT_NAME0", pack=False, align=None), offset=0), "flags": SimTypeInt(signed=False, label="UInt32")}, name="IKEEXT_CERTIFICATE_CRITERIA0", pack=False, align=None), offset=0)}, name="_Anonymous2_e__Struct", pack=False, align=None), "Anonymous3": SimStruct({"outboundRootStoreArraySize": SimTypeInt(signed=False, label="UInt32"), "outboundTrustedRootStoreCriteria": SimTypePointer(SimStruct({"certData": SimStruct({"size": SimTypeInt(signed=False, label="UInt32"), "data": SimTypePointer(SimTypeChar(label="Byte"), offset=0)}, name="FWP_BYTE_BLOB", pack=False, align=None), "certHash": SimStruct({"size": SimTypeInt(signed=False, label="UInt32"), "data": SimTypePointer(SimTypeChar(label="Byte"), offset=0)}, name="FWP_BYTE_BLOB", pack=False, align=None), "eku": SimTypePointer(SimStruct({"numEku": SimTypeInt(signed=False, label="UInt32"), "eku": SimTypePointer(SimTypePointer(SimTypeChar(label="Byte"), offset=0), offset=0)}, name="IKEEXT_CERT_EKUS0", pack=False, align=None), offset=0), "name": SimTypePointer(SimStruct({"nameType": SimTypeInt(signed=False, label="IKEEXT_CERT_CRITERIA_NAME_TYPE"), "certName": SimTypePointer(SimTypeChar(label="Char"), offset=0)}, name="IKEEXT_CERT_NAME0", pack=False, align=None), offset=0), "flags": SimTypeInt(signed=False, label="UInt32")}, name="IKEEXT_CERTIFICATE_CRITERIA0", pack=False, align=None), offset=0)}, name="_Anonymous3_e__Struct", pack=False, align=None)}, name="<anon>", label="None"), "flags": SimTypeInt(signed=False, label="IKEEXT_CERT_AUTH"), "localCertLocationUrl": SimStruct({"size": SimTypeInt(signed=False, label="UInt32"), "data": SimTypePointer(SimTypeChar(label="Byte"), offset=0)}, name="FWP_BYTE_BLOB", pack=False, align=None)}, name="IKEEXT_CERTIFICATE_AUTHENTICATION2", pack=False, align=None), "cgaAuthentication": SimStruct({"keyContainerName": SimTypePointer(SimTypeChar(label="Char"), offset=0), "cspName": SimTypePointer(SimTypeChar(label="Char"), offset=0), "cspType": SimTypeInt(signed=False, label="UInt32"), "cgaModifier": SimStruct({"byteArray16": SimTypeFixedSizeArray(SimTypeChar(label="Byte"), 16)}, name="FWP_BYTE_ARRAY16", pack=False, align=None), "cgaCollisionCount": SimTypeChar(label="Byte")}, name="IKEEXT_IPV6_CGA_AUTHENTICATION0", pack=False, align=None), "eapAuthentication": SimStruct({"flags": SimTypeInt(signed=False, label="IKEEXT_EAP_AUTHENTICATION_FLAGS")}, name="IKEEXT_EAP_AUTHENTICATION0", pack=False, align=None)}, name="<anon>", label="None")}, name="IKEEXT_AUTHENTICATION_METHOD2", pack=False, align=None), offset=0), "initiatorImpersonationType": SimTypeInt(signed=False, label="IKEEXT_AUTHENTICATION_IMPERSONATION_TYPE")}, name="IKEEXT_EM_POLICY2", pack=False, align=None), offset=0), "fwdPathSaLifetime": SimTypeInt(signed=False, label="UInt32")}, name="IPSEC_TUNNEL_POLICY2", pack=False, align=None), offset=0), "ikeV2QmTransportPolicy": SimTypePointer(SimStruct({"numIpsecProposals": SimTypeInt(signed=False, label="UInt32"), "ipsecProposals": SimTypePointer(SimStruct({"lifetime": SimStruct({"lifetimeSeconds": SimTypeInt(signed=False, label="UInt32"), "lifetimeKilobytes": SimTypeInt(signed=False, label="UInt32"), "lifetimePackets": SimTypeInt(signed=False, label="UInt32")}, name="IPSEC_SA_LIFETIME0", pack=False, align=None), "numSaTransforms": SimTypeInt(signed=False, label="UInt32"), "saTransforms": SimTypePointer(SimStruct({"ipsecTransformType": SimTypeInt(signed=False, label="IPSEC_TRANSFORM_TYPE"), "Anonymous": SimUnion({"ahTransform": SimTypePointer(SimStruct({"authTransformId": SimStruct({"authType": SimTypeInt(signed=False, label="IPSEC_AUTH_TYPE"), "authConfig": SimTypeChar(label="Byte")}, name="IPSEC_AUTH_TRANSFORM_ID0", pack=False, align=None), "cryptoModuleId": SimTypePointer(SimTypeBottom(label="Guid"), offset=0)}, name="IPSEC_AUTH_TRANSFORM0", pack=False, align=None), offset=0), "espAuthTransform": SimTypePointer(SimStruct({"authTransformId": SimStruct({"authType": SimTypeInt(signed=False, label="IPSEC_AUTH_TYPE"), "authConfig": SimTypeChar(label="Byte")}, name="IPSEC_AUTH_TRANSFORM_ID0", pack=False, align=None), "cryptoModuleId": SimTypePointer(SimTypeBottom(label="Guid"), offset=0)}, name="IPSEC_AUTH_TRANSFORM0", pack=False, align=None), offset=0), "espCipherTransform": SimTypePointer(SimStruct({"cipherTransformId": SimStruct({"cipherType": SimTypeInt(signed=False, label="IPSEC_CIPHER_TYPE"), "cipherConfig": SimTypeChar(label="Byte")}, name="IPSEC_CIPHER_TRANSFORM_ID0", pack=False, align=None), "cryptoModuleId": SimTypePointer(SimTypeBottom(label="Guid"), offset=0)}, name="IPSEC_CIPHER_TRANSFORM0", pack=False, align=None), offset=0), "espAuthAndCipherTransform": SimTypePointer(SimStruct({"authTransform": SimStruct({"authTransformId": SimStruct({"authType": SimTypeInt(signed=False, label="IPSEC_AUTH_TYPE"), "authConfig": SimTypeChar(label="Byte")}, name="IPSEC_AUTH_TRANSFORM_ID0", pack=False, align=None), "cryptoModuleId": SimTypePointer(SimTypeBottom(label="Guid"), offset=0)}, name="IPSEC_AUTH_TRANSFORM0", pack=False, align=None), "cipherTransform": SimStruct({"cipherTransformId": SimStruct({"cipherType": SimTypeInt(signed=False, label="IPSEC_CIPHER_TYPE"), "cipherConfig": SimTypeChar(label="Byte")}, name="IPSEC_CIPHER_TRANSFORM_ID0", pack=False, align=None), "cryptoModuleId": SimTypePointer(SimTypeBottom(label="Guid"), offset=0)}, name="IPSEC_CIPHER_TRANSFORM0", pack=False, align=None)}, name="IPSEC_AUTH_AND_CIPHER_TRANSFORM0", pack=False, align=None), offset=0), "espAuthFwTransform": SimTypePointer(SimStruct({"authTransformId": SimStruct({"authType": SimTypeInt(signed=False, label="IPSEC_AUTH_TYPE"), "authConfig": SimTypeChar(label="Byte")}, name="IPSEC_AUTH_TRANSFORM_ID0", pack=False, align=None), "cryptoModuleId": SimTypePointer(SimTypeBottom(label="Guid"), offset=0)}, name="IPSEC_AUTH_TRANSFORM0", pack=False, align=None), offset=0)}, name="<anon>", label="None")}, name="IPSEC_SA_TRANSFORM0", pack=False, align=None), offset=0), "pfsGroup": SimTypeInt(signed=False, label="IPSEC_PFS_GROUP")}, name="IPSEC_PROPOSAL0", pack=False, align=None), offset=0), "flags": SimTypeInt(signed=False, label="IPSEC_POLICY_FLAG"), "ndAllowClearTimeoutSeconds": SimTypeInt(signed=False, label="UInt32"), "saIdleTimeout": SimStruct({"idleTimeoutSeconds": SimTypeInt(signed=False, label="UInt32"), "idleTimeoutSecondsFailOver": SimTypeInt(signed=False, label="UInt32")}, name="IPSEC_SA_IDLE_TIMEOUT0", pack=False, align=None), "emPolicy": SimTypePointer(SimStruct({"numAuthenticationMethods": SimTypeInt(signed=False, label="UInt32"), "authenticationMethods": SimTypePointer(SimStruct({"authenticationMethodType": SimTypeInt(signed=False, label="IKEEXT_AUTHENTICATION_METHOD_TYPE"), "Anonymous": SimUnion({"presharedKeyAuthentication": SimStruct({"presharedKey": SimStruct({"size": SimTypeInt(signed=False, label="UInt32"), "data": SimTypePointer(SimTypeChar(label="Byte"), offset=0)}, name="FWP_BYTE_BLOB", pack=False, align=None), "flags": SimTypeInt(signed=False, label="IKEEXT_PRESHARED_KEY_AUTHENTICATION_FLAGS")}, name="IKEEXT_PRESHARED_KEY_AUTHENTICATION1", pack=False, align=None), "certificateAuthentication": SimStruct({"inboundConfigType": SimTypeInt(signed=False, label="IKEEXT_CERT_CONFIG_TYPE"), "Anonymous1": SimUnion({"Anonymous1": SimStruct({"inboundRootArraySize": SimTypeInt(signed=False, label="UInt32"), "inboundRootCriteria": SimTypePointer(SimStruct({"certData": SimStruct({"size": SimTypeInt(signed=False, label="UInt32"), "data": SimTypePointer(SimTypeChar(label="Byte"), offset=0)}, name="FWP_BYTE_BLOB", pack=False, align=None), "certHash": SimStruct({"size": SimTypeInt(signed=False, label="UInt32"), "data": SimTypePointer(SimTypeChar(label="Byte"), offset=0)}, name="FWP_BYTE_BLOB", pack=False, align=None), "eku": SimTypePointer(SimStruct({"numEku": SimTypeInt(signed=False, label="UInt32"), "eku": SimTypePointer(SimTypePointer(SimTypeChar(label="Byte"), offset=0), offset=0)}, name="IKEEXT_CERT_EKUS0", pack=False, align=None), offset=0), "name": SimTypePointer(SimStruct({"nameType": SimTypeInt(signed=False, label="IKEEXT_CERT_CRITERIA_NAME_TYPE"), "certName": SimTypePointer(SimTypeChar(label="Char"), offset=0)}, name="IKEEXT_CERT_NAME0", pack=False, align=None), offset=0), "flags": SimTypeInt(signed=False, label="UInt32")}, name="IKEEXT_CERTIFICATE_CRITERIA0", pack=False, align=None), offset=0)}, name="_Anonymous1_e__Struct", pack=False, align=None), "Anonymous2": SimStruct({"inboundEnterpriseStoreArraySize": SimTypeInt(signed=False, label="UInt32"), "inboundEnterpriseStoreCriteria": SimTypePointer(SimStruct({"certData": SimStruct({"size": SimTypeInt(signed=False, label="UInt32"), "data": SimTypePointer(SimTypeChar(label="Byte"), offset=0)}, name="FWP_BYTE_BLOB", pack=False, align=None), "certHash": SimStruct({"size": SimTypeInt(signed=False, label="UInt32"), "data": SimTypePointer(SimTypeChar(label="Byte"), offset=0)}, name="FWP_BYTE_BLOB", pack=False, align=None), "eku": SimTypePointer(SimStruct({"numEku": SimTypeInt(signed=False, label="UInt32"), "eku": SimTypePointer(SimTypePointer(SimTypeChar(label="Byte"), offset=0), offset=0)}, name="IKEEXT_CERT_EKUS0", pack=False, align=None), offset=0), "name": SimTypePointer(SimStruct({"nameType": SimTypeInt(signed=False, label="IKEEXT_CERT_CRITERIA_NAME_TYPE"), "certName": SimTypePointer(SimTypeChar(label="Char"), offset=0)}, name="IKEEXT_CERT_NAME0", pack=False, align=None), offset=0), "flags": SimTypeInt(signed=False, label="UInt32")}, name="IKEEXT_CERTIFICATE_CRITERIA0", pack=False, align=None), offset=0)}, name="_Anonymous2_e__Struct", pack=False, align=None), "Anonymous3": SimStruct({"inboundRootStoreArraySize": SimTypeInt(signed=False, label="UInt32"), "inboundTrustedRootStoreCriteria": SimTypePointer(SimStruct({"certData": SimStruct({"size": SimTypeInt(signed=False, label="UInt32"), "data": SimTypePointer(SimTypeChar(label="Byte"), offset=0)}, name="FWP_BYTE_BLOB", pack=False, align=None), "certHash": SimStruct({"size": SimTypeInt(signed=False, label="UInt32"), "data": SimTypePointer(SimTypeChar(label="Byte"), offset=0)}, name="FWP_BYTE_BLOB", pack=False, align=None), "eku": SimTypePointer(SimStruct({"numEku": SimTypeInt(signed=False, label="UInt32"), "eku": SimTypePointer(SimTypePointer(SimTypeChar(label="Byte"), offset=0), offset=0)}, name="IKEEXT_CERT_EKUS0", pack=False, align=None), offset=0), "name": SimTypePointer(SimStruct({"nameType": SimTypeInt(signed=False, label="IKEEXT_CERT_CRITERIA_NAME_TYPE"), "certName": SimTypePointer(SimTypeChar(label="Char"), offset=0)}, name="IKEEXT_CERT_NAME0", pack=False, align=None), offset=0), "flags": SimTypeInt(signed=False, label="UInt32")}, name="IKEEXT_CERTIFICATE_CRITERIA0", pack=False, align=None), offset=0)}, name="_Anonymous3_e__Struct", pack=False, align=None)}, name="<anon>", label="None"), "outboundConfigType": SimTypeInt(signed=False, label="IKEEXT_CERT_CONFIG_TYPE"), "Anonymous2": SimUnion({"Anonymous1": SimStruct({"outboundRootArraySize": SimTypeInt(signed=False, label="UInt32"), "outboundRootCriteria": SimTypePointer(SimStruct({"certData": SimStruct({"size": SimTypeInt(signed=False, label="UInt32"), "data": SimTypePointer(SimTypeChar(label="Byte"), offset=0)}, name="FWP_BYTE_BLOB", pack=False, align=None), "certHash": SimStruct({"size": SimTypeInt(signed=False, label="UInt32"), "data": SimTypePointer(SimTypeChar(label="Byte"), offset=0)}, name="FWP_BYTE_BLOB", pack=False, align=None), "eku": SimTypePointer(SimStruct({"numEku": SimTypeInt(signed=False, label="UInt32"), "eku": SimTypePointer(SimTypePointer(SimTypeChar(label="Byte"), offset=0), offset=0)}, name="IKEEXT_CERT_EKUS0", pack=False, align=None), offset=0), "name": SimTypePointer(SimStruct({"nameType": SimTypeInt(signed=False, label="IKEEXT_CERT_CRITERIA_NAME_TYPE"), "certName": SimTypePointer(SimTypeChar(label="Char"), offset=0)}, name="IKEEXT_CERT_NAME0", pack=False, align=None), offset=0), "flags": SimTypeInt(signed=False, label="UInt32")}, name="IKEEXT_CERTIFICATE_CRITERIA0", pack=False, align=None), offset=0)}, name="_Anonymous1_e__Struct", pack=False, align=None), "Anonymous2": SimStruct({"outboundEnterpriseStoreArraySize": SimTypeInt(signed=False, label="UInt32"), "outboundEnterpriseStoreCriteria": SimTypePointer(SimStruct({"certData": SimStruct({"size": SimTypeInt(signed=False, label="UInt32"), "data": SimTypePointer(SimTypeChar(label="Byte"), offset=0)}, name="FWP_BYTE_BLOB", pack=False, align=None), "certHash": SimStruct({"size": SimTypeInt(signed=False, label="UInt32"), "data": SimTypePointer(SimTypeChar(label="Byte"), offset=0)}, name="FWP_BYTE_BLOB", pack=False, align=None), "eku": SimTypePointer(SimStruct({"numEku": SimTypeInt(signed=False, label="UInt32"), "eku": SimTypePointer(SimTypePointer(SimTypeChar(label="Byte"), offset=0), offset=0)}, name="IKEEXT_CERT_EKUS0", pack=False, align=None), offset=0), "name": SimTypePointer(SimStruct({"nameType": SimTypeInt(signed=False, label="IKEEXT_CERT_CRITERIA_NAME_TYPE"), "certName": SimTypePointer(SimTypeChar(label="Char"), offset=0)}, name="IKEEXT_CERT_NAME0", pack=False, align=None), offset=0), "flags": SimTypeInt(signed=False, label="UInt32")}, name="IKEEXT_CERTIFICATE_CRITERIA0", pack=False, align=None), offset=0)}, name="_Anonymous2_e__Struct", pack=False, align=None), "Anonymous3": SimStruct({"outboundRootStoreArraySize": SimTypeInt(signed=False, label="UInt32"), "outboundTrustedRootStoreCriteria": SimTypePointer(SimStruct({"certData": SimStruct({"size": SimTypeInt(signed=False, label="UInt32"), "data": SimTypePointer(SimTypeChar(label="Byte"), offset=0)}, name="FWP_BYTE_BLOB", pack=False, align=None), "certHash": SimStruct({"size": SimTypeInt(signed=False, label="UInt32"), "data": SimTypePointer(SimTypeChar(label="Byte"), offset=0)}, name="FWP_BYTE_BLOB", pack=False, align=None), "eku": SimTypePointer(SimStruct({"numEku": SimTypeInt(signed=False, label="UInt32"), "eku": SimTypePointer(SimTypePointer(SimTypeChar(label="Byte"), offset=0), offset=0)}, name="IKEEXT_CERT_EKUS0", pack=False, align=None), offset=0), "name": SimTypePointer(SimStruct({"nameType": SimTypeInt(signed=False, label="IKEEXT_CERT_CRITERIA_NAME_TYPE"), "certName": SimTypePointer(SimTypeChar(label="Char"), offset=0)}, name="IKEEXT_CERT_NAME0", pack=False, align=None), offset=0), "flags": SimTypeInt(signed=False, label="UInt32")}, name="IKEEXT_CERTIFICATE_CRITERIA0", pack=False, align=None), offset=0)}, name="_Anonymous3_e__Struct", pack=False, align=None)}, name="<anon>", label="None"), "flags": SimTypeInt(signed=False, label="IKEEXT_CERT_AUTH"), "localCertLocationUrl": SimStruct({"size": SimTypeInt(signed=False, label="UInt32"), "data": SimTypePointer(SimTypeChar(label="Byte"), offset=0)}, name="FWP_BYTE_BLOB", pack=False, align=None)}, name="IKEEXT_CERTIFICATE_AUTHENTICATION2", pack=False, align=None), "kerberosAuthentication": SimStruct({"flags": SimTypeInt(signed=False, label="IKEEXT_KERBEROS_AUTHENTICATION_FLAGS"), "proxyServer": SimTypePointer(SimTypeChar(label="Char"), offset=0)}, name="IKEEXT_KERBEROS_AUTHENTICATION1", pack=False, align=None), "reservedAuthentication": SimStruct({"flags": SimTypeInt(signed=False, label="IKEEXT_RESERVED_AUTHENTICATION_FLAGS")}, name="IKEEXT_RESERVED_AUTHENTICATION0", pack=False, align=None), "ntlmV2Authentication": SimStruct({"flags": SimTypeInt(signed=False, label="UInt32")}, name="IKEEXT_NTLM_V2_AUTHENTICATION0", pack=False, align=None), "sslAuthentication": SimStruct({"inboundConfigType": SimTypeInt(signed=False, label="IKEEXT_CERT_CONFIG_TYPE"), "Anonymous1": SimUnion({"Anonymous1": SimStruct({"inboundRootArraySize": SimTypeInt(signed=False, label="UInt32"), "inboundRootCriteria": SimTypePointer(SimStruct({"certData": SimStruct({"size": SimTypeInt(signed=False, label="UInt32"), "data": SimTypePointer(SimTypeChar(label="Byte"), offset=0)}, name="FWP_BYTE_BLOB", pack=False, align=None), "certHash": SimStruct({"size": SimTypeInt(signed=False, label="UInt32"), "data": SimTypePointer(SimTypeChar(label="Byte"), offset=0)}, name="FWP_BYTE_BLOB", pack=False, align=None), "eku": SimTypePointer(SimStruct({"numEku": SimTypeInt(signed=False, label="UInt32"), "eku": SimTypePointer(SimTypePointer(SimTypeChar(label="Byte"), offset=0), offset=0)}, name="IKEEXT_CERT_EKUS0", pack=False, align=None), offset=0), "name": SimTypePointer(SimStruct({"nameType": SimTypeInt(signed=False, label="IKEEXT_CERT_CRITERIA_NAME_TYPE"), "certName": SimTypePointer(SimTypeChar(label="Char"), offset=0)}, name="IKEEXT_CERT_NAME0", pack=False, align=None), offset=0), "flags": SimTypeInt(signed=False, label="UInt32")}, name="IKEEXT_CERTIFICATE_CRITERIA0", pack=False, align=None), offset=0)}, name="_Anonymous1_e__Struct", pack=False, align=None), "Anonymous2": SimStruct({"inboundEnterpriseStoreArraySize": SimTypeInt(signed=False, label="UInt32"), "inboundEnterpriseStoreCriteria": SimTypePointer(SimStruct({"certData": SimStruct({"size": SimTypeInt(signed=False, label="UInt32"), "data": SimTypePointer(SimTypeChar(label="Byte"), offset=0)}, name="FWP_BYTE_BLOB", pack=False, align=None), "certHash": SimStruct({"size": SimTypeInt(signed=False, label="UInt32"), "data": SimTypePointer(SimTypeChar(label="Byte"), offset=0)}, name="FWP_BYTE_BLOB", pack=False, align=None), "eku": SimTypePointer(SimStruct({"numEku": SimTypeInt(signed=False, label="UInt32"), "eku": SimTypePointer(SimTypePointer(SimTypeChar(label="Byte"), offset=0), offset=0)}, name="IKEEXT_CERT_EKUS0", pack=False, align=None), offset=0), "name": SimTypePointer(SimStruct({"nameType": SimTypeInt(signed=False, label="IKEEXT_CERT_CRITERIA_NAME_TYPE"), "certName": SimTypePointer(SimTypeChar(label="Char"), offset=0)}, name="IKEEXT_CERT_NAME0", pack=False, align=None), offset=0), "flags": SimTypeInt(signed=False, label="UInt32")}, name="IKEEXT_CERTIFICATE_CRITERIA0", pack=False, align=None), offset=0)}, name="_Anonymous2_e__Struct", pack=False, align=None), "Anonymous3": SimStruct({"inboundRootStoreArraySize": SimTypeInt(signed=False, label="UInt32"), "inboundTrustedRootStoreCriteria": SimTypePointer(SimStruct({"certData": SimStruct({"size": SimTypeInt(signed=False, label="UInt32"), "data": SimTypePointer(SimTypeChar(label="Byte"), offset=0)}, name="FWP_BYTE_BLOB", pack=False, align=None), "certHash": SimStruct({"size": SimTypeInt(signed=False, label="UInt32"), "data": SimTypePointer(SimTypeChar(label="Byte"), offset=0)}, name="FWP_BYTE_BLOB", pack=False, align=None), "eku": SimTypePointer(SimStruct({"numEku": SimTypeInt(signed=False, label="UInt32"), "eku": SimTypePointer(SimTypePointer(SimTypeChar(label="Byte"), offset=0), offset=0)}, name="IKEEXT_CERT_EKUS0", pack=False, align=None), offset=0), "name": SimTypePointer(SimStruct({"nameType": SimTypeInt(signed=False, label="IKEEXT_CERT_CRITERIA_NAME_TYPE"), "certName": SimTypePointer(SimTypeChar(label="Char"), offset=0)}, name="IKEEXT_CERT_NAME0", pack=False, align=None), offset=0), "flags": SimTypeInt(signed=False, label="UInt32")}, name="IKEEXT_CERTIFICATE_CRITERIA0", pack=False, align=None), offset=0)}, name="_Anonymous3_e__Struct", pack=False, align=None)}, name="<anon>", label="None"), "outboundConfigType": SimTypeInt(signed=False, label="IKEEXT_CERT_CONFIG_TYPE"), "Anonymous2": SimUnion({"Anonymous1": SimStruct({"outboundRootArraySize": SimTypeInt(signed=False, label="UInt32"), "outboundRootCriteria": SimTypePointer(SimStruct({"certData": SimStruct({"size": SimTypeInt(signed=False, label="UInt32"), "data": SimTypePointer(SimTypeChar(label="Byte"), offset=0)}, name="FWP_BYTE_BLOB", pack=False, align=None), "certHash": SimStruct({"size": SimTypeInt(signed=False, label="UInt32"), "data": SimTypePointer(SimTypeChar(label="Byte"), offset=0)}, name="FWP_BYTE_BLOB", pack=False, align=None), "eku": SimTypePointer(SimStruct({"numEku": SimTypeInt(signed=False, label="UInt32"), "eku": SimTypePointer(SimTypePointer(SimTypeChar(label="Byte"), offset=0), offset=0)}, name="IKEEXT_CERT_EKUS0", pack=False, align=None), offset=0), "name": SimTypePointer(SimStruct({"nameType": SimTypeInt(signed=False, label="IKEEXT_CERT_CRITERIA_NAME_TYPE"), "certName": SimTypePointer(SimTypeChar(label="Char"), offset=0)}, name="IKEEXT_CERT_NAME0", pack=False, align=None), offset=0), "flags": SimTypeInt(signed=False, label="UInt32")}, name="IKEEXT_CERTIFICATE_CRITERIA0", pack=False, align=None), offset=0)}, name="_Anonymous1_e__Struct", pack=False, align=None), "Anonymous2": SimStruct({"outboundEnterpriseStoreArraySize": SimTypeInt(signed=False, label="UInt32"), "outboundEnterpriseStoreCriteria": SimTypePointer(SimStruct({"certData": SimStruct({"size": SimTypeInt(signed=False, label="UInt32"), "data": SimTypePointer(SimTypeChar(label="Byte"), offset=0)}, name="FWP_BYTE_BLOB", pack=False, align=None), "certHash": SimStruct({"size": SimTypeInt(signed=False, label="UInt32"), "data": SimTypePointer(SimTypeChar(label="Byte"), offset=0)}, name="FWP_BYTE_BLOB", pack=False, align=None), "eku": SimTypePointer(SimStruct({"numEku": SimTypeInt(signed=False, label="UInt32"), "eku": SimTypePointer(SimTypePointer(SimTypeChar(label="Byte"), offset=0), offset=0)}, name="IKEEXT_CERT_EKUS0", pack=False, align=None), offset=0), "name": SimTypePointer(SimStruct({"nameType": SimTypeInt(signed=False, label="IKEEXT_CERT_CRITERIA_NAME_TYPE"), "certName": SimTypePointer(SimTypeChar(label="Char"), offset=0)}, name="IKEEXT_CERT_NAME0", pack=False, align=None), offset=0), "flags": SimTypeInt(signed=False, label="UInt32")}, name="IKEEXT_CERTIFICATE_CRITERIA0", pack=False, align=None), offset=0)}, name="_Anonymous2_e__Struct", pack=False, align=None), "Anonymous3": SimStruct({"outboundRootStoreArraySize": SimTypeInt(signed=False, label="UInt32"), "outboundTrustedRootStoreCriteria": SimTypePointer(SimStruct({"certData": SimStruct({"size": SimTypeInt(signed=False, label="UInt32"), "data": SimTypePointer(SimTypeChar(label="Byte"), offset=0)}, name="FWP_BYTE_BLOB", pack=False, align=None), "certHash": SimStruct({"size": SimTypeInt(signed=False, label="UInt32"), "data": SimTypePointer(SimTypeChar(label="Byte"), offset=0)}, name="FWP_BYTE_BLOB", pack=False, align=None), "eku": SimTypePointer(SimStruct({"numEku": SimTypeInt(signed=False, label="UInt32"), "eku": SimTypePointer(SimTypePointer(SimTypeChar(label="Byte"), offset=0), offset=0)}, name="IKEEXT_CERT_EKUS0", pack=False, align=None), offset=0), "name": SimTypePointer(SimStruct({"nameType": SimTypeInt(signed=False, label="IKEEXT_CERT_CRITERIA_NAME_TYPE"), "certName": SimTypePointer(SimTypeChar(label="Char"), offset=0)}, name="IKEEXT_CERT_NAME0", pack=False, align=None), offset=0), "flags": SimTypeInt(signed=False, label="UInt32")}, name="IKEEXT_CERTIFICATE_CRITERIA0", pack=False, align=None), offset=0)}, name="_Anonymous3_e__Struct", pack=False, align=None)}, name="<anon>", label="None"), "flags": SimTypeInt(signed=False, label="IKEEXT_CERT_AUTH"), "localCertLocationUrl": SimStruct({"size": SimTypeInt(signed=False, label="UInt32"), "data": SimTypePointer(SimTypeChar(label="Byte"), offset=0)}, name="FWP_BYTE_BLOB", pack=False, align=None)}, name="IKEEXT_CERTIFICATE_AUTHENTICATION2", pack=False, align=None), "cgaAuthentication": SimStruct({"keyContainerName": SimTypePointer(SimTypeChar(label="Char"), offset=0), "cspName": SimTypePointer(SimTypeChar(label="Char"), offset=0), "cspType": SimTypeInt(signed=False, label="UInt32"), "cgaModifier": SimStruct({"byteArray16": SimTypeFixedSizeArray(SimTypeChar(label="Byte"), 16)}, name="FWP_BYTE_ARRAY16", pack=False, align=None), "cgaCollisionCount": SimTypeChar(label="Byte")}, name="IKEEXT_IPV6_CGA_AUTHENTICATION0", pack=False, align=None), "eapAuthentication": SimStruct({"flags": SimTypeInt(signed=False, label="IKEEXT_EAP_AUTHENTICATION_FLAGS")}, name="IKEEXT_EAP_AUTHENTICATION0", pack=False, align=None)}, name="<anon>", label="None")}, name="IKEEXT_AUTHENTICATION_METHOD2", pack=False, align=None), offset=0), "initiatorImpersonationType": SimTypeInt(signed=False, label="IKEEXT_AUTHENTICATION_IMPERSONATION_TYPE")}, name="IKEEXT_EM_POLICY2", pack=False, align=None), offset=0)}, name="IPSEC_TRANSPORT_POLICY2", pack=False, align=None), offset=0), "ikeV2MmPolicy": SimTypePointer(SimStruct({"softExpirationTime": SimTypeInt(signed=False, label="UInt32"), "numAuthenticationMethods": SimTypeInt(signed=False, label="UInt32"), "authenticationMethods": SimTypePointer(SimStruct({"authenticationMethodType": SimTypeInt(signed=False, label="IKEEXT_AUTHENTICATION_METHOD_TYPE"), "Anonymous": SimUnion({"presharedKeyAuthentication": SimStruct({"presharedKey": SimStruct({"size": SimTypeInt(signed=False, label="UInt32"), "data": SimTypePointer(SimTypeChar(label="Byte"), offset=0)}, name="FWP_BYTE_BLOB", pack=False, align=None), "flags": SimTypeInt(signed=False, label="IKEEXT_PRESHARED_KEY_AUTHENTICATION_FLAGS")}, name="IKEEXT_PRESHARED_KEY_AUTHENTICATION1", pack=False, align=None), "certificateAuthentication": SimStruct({"inboundConfigType": SimTypeInt(signed=False, label="IKEEXT_CERT_CONFIG_TYPE"), "Anonymous1": SimUnion({"Anonymous1": SimStruct({"inboundRootArraySize": SimTypeInt(signed=False, label="UInt32"), "inboundRootCriteria": SimTypePointer(SimStruct({"certData": SimStruct({"size": SimTypeInt(signed=False, label="UInt32"), "data": SimTypePointer(SimTypeChar(label="Byte"), offset=0)}, name="FWP_BYTE_BLOB", pack=False, align=None), "certHash": SimStruct({"size": SimTypeInt(signed=False, label="UInt32"), "data": SimTypePointer(SimTypeChar(label="Byte"), offset=0)}, name="FWP_BYTE_BLOB", pack=False, align=None), "eku": SimTypePointer(SimStruct({"numEku": SimTypeInt(signed=False, label="UInt32"), "eku": SimTypePointer(SimTypePointer(SimTypeChar(label="Byte"), offset=0), offset=0)}, name="IKEEXT_CERT_EKUS0", pack=False, align=None), offset=0), "name": SimTypePointer(SimStruct({"nameType": SimTypeInt(signed=False, label="IKEEXT_CERT_CRITERIA_NAME_TYPE"), "certName": SimTypePointer(SimTypeChar(label="Char"), offset=0)}, name="IKEEXT_CERT_NAME0", pack=False, align=None), offset=0), "flags": SimTypeInt(signed=False, label="UInt32")}, name="IKEEXT_CERTIFICATE_CRITERIA0", pack=False, align=None), offset=0)}, name="_Anonymous1_e__Struct", pack=False, align=None), "Anonymous2": SimStruct({"inboundEnterpriseStoreArraySize": SimTypeInt(signed=False, label="UInt32"), "inboundEnterpriseStoreCriteria": SimTypePointer(SimStruct({"certData": SimStruct({"size": SimTypeInt(signed=False, label="UInt32"), "data": SimTypePointer(SimTypeChar(label="Byte"), offset=0)}, name="FWP_BYTE_BLOB", pack=False, align=None), "certHash": SimStruct({"size": SimTypeInt(signed=False, label="UInt32"), "data": SimTypePointer(SimTypeChar(label="Byte"), offset=0)}, name="FWP_BYTE_BLOB", pack=False, align=None), "eku": SimTypePointer(SimStruct({"numEku": SimTypeInt(signed=False, label="UInt32"), "eku": SimTypePointer(SimTypePointer(SimTypeChar(label="Byte"), offset=0), offset=0)}, name="IKEEXT_CERT_EKUS0", pack=False, align=None), offset=0), "name": SimTypePointer(SimStruct({"nameType": SimTypeInt(signed=False, label="IKEEXT_CERT_CRITERIA_NAME_TYPE"), "certName": SimTypePointer(SimTypeChar(label="Char"), offset=0)}, name="IKEEXT_CERT_NAME0", pack=False, align=None), offset=0), "flags": SimTypeInt(signed=False, label="UInt32")}, name="IKEEXT_CERTIFICATE_CRITERIA0", pack=False, align=None), offset=0)}, name="_Anonymous2_e__Struct", pack=False, align=None), "Anonymous3": SimStruct({"inboundRootStoreArraySize": SimTypeInt(signed=False, label="UInt32"), "inboundTrustedRootStoreCriteria": SimTypePointer(SimStruct({"certData": SimStruct({"size": SimTypeInt(signed=False, label="UInt32"), "data": SimTypePointer(SimTypeChar(label="Byte"), offset=0)}, name="FWP_BYTE_BLOB", pack=False, align=None), "certHash": SimStruct({"size": SimTypeInt(signed=False, label="UInt32"), "data": SimTypePointer(SimTypeChar(label="Byte"), offset=0)}, name="FWP_BYTE_BLOB", pack=False, align=None), "eku": SimTypePointer(SimStruct({"numEku": SimTypeInt(signed=False, label="UInt32"), "eku": SimTypePointer(SimTypePointer(SimTypeChar(label="Byte"), offset=0), offset=0)}, name="IKEEXT_CERT_EKUS0", pack=False, align=None), offset=0), "name": SimTypePointer(SimStruct({"nameType": SimTypeInt(signed=False, label="IKEEXT_CERT_CRITERIA_NAME_TYPE"), "certName": SimTypePointer(SimTypeChar(label="Char"), offset=0)}, name="IKEEXT_CERT_NAME0", pack=False, align=None), offset=0), "flags": SimTypeInt(signed=False, label="UInt32")}, name="IKEEXT_CERTIFICATE_CRITERIA0", pack=False, align=None), offset=0)}, name="_Anonymous3_e__Struct", pack=False, align=None)}, name="<anon>", label="None"), "outboundConfigType": SimTypeInt(signed=False, label="IKEEXT_CERT_CONFIG_TYPE"), "Anonymous2": SimUnion({"Anonymous1": SimStruct({"outboundRootArraySize": SimTypeInt(signed=False, label="UInt32"), "outboundRootCriteria": SimTypePointer(SimStruct({"certData": SimStruct({"size": SimTypeInt(signed=False, label="UInt32"), "data": SimTypePointer(SimTypeChar(label="Byte"), offset=0)}, name="FWP_BYTE_BLOB", pack=False, align=None), "certHash": SimStruct({"size": SimTypeInt(signed=False, label="UInt32"), "data": SimTypePointer(SimTypeChar(label="Byte"), offset=0)}, name="FWP_BYTE_BLOB", pack=False, align=None), "eku": SimTypePointer(SimStruct({"numEku": SimTypeInt(signed=False, label="UInt32"), "eku": SimTypePointer(SimTypePointer(SimTypeChar(label="Byte"), offset=0), offset=0)}, name="IKEEXT_CERT_EKUS0", pack=False, align=None), offset=0), "name": SimTypePointer(SimStruct({"nameType": SimTypeInt(signed=False, label="IKEEXT_CERT_CRITERIA_NAME_TYPE"), "certName": SimTypePointer(SimTypeChar(label="Char"), offset=0)}, name="IKEEXT_CERT_NAME0", pack=False, align=None), offset=0), "flags": SimTypeInt(signed=False, label="UInt32")}, name="IKEEXT_CERTIFICATE_CRITERIA0", pack=False, align=None), offset=0)}, name="_Anonymous1_e__Struct", pack=False, align=None), "Anonymous2": SimStruct({"outboundEnterpriseStoreArraySize": SimTypeInt(signed=False, label="UInt32"), "outboundEnterpriseStoreCriteria": SimTypePointer(SimStruct({"certData": SimStruct({"size": SimTypeInt(signed=False, label="UInt32"), "data": SimTypePointer(SimTypeChar(label="Byte"), offset=0)}, name="FWP_BYTE_BLOB", pack=False, align=None), "certHash": SimStruct({"size": SimTypeInt(signed=False, label="UInt32"), "data": SimTypePointer(SimTypeChar(label="Byte"), offset=0)}, name="FWP_BYTE_BLOB", pack=False, align=None), "eku": SimTypePointer(SimStruct({"numEku": SimTypeInt(signed=False, label="UInt32"), "eku": SimTypePointer(SimTypePointer(SimTypeChar(label="Byte"), offset=0), offset=0)}, name="IKEEXT_CERT_EKUS0", pack=False, align=None), offset=0), "name": SimTypePointer(SimStruct({"nameType": SimTypeInt(signed=False, label="IKEEXT_CERT_CRITERIA_NAME_TYPE"), "certName": SimTypePointer(SimTypeChar(label="Char"), offset=0)}, name="IKEEXT_CERT_NAME0", pack=False, align=None), offset=0), "flags": SimTypeInt(signed=False, label="UInt32")}, name="IKEEXT_CERTIFICATE_CRITERIA0", pack=False, align=None), offset=0)}, name="_Anonymous2_e__Struct", pack=False, align=None), "Anonymous3": SimStruct({"outboundRootStoreArraySize": SimTypeInt(signed=False, label="UInt32"), "outboundTrustedRootStoreCriteria": SimTypePointer(SimStruct({"certData": SimStruct({"size": SimTypeInt(signed=False, label="UInt32"), "data": SimTypePointer(SimTypeChar(label="Byte"), offset=0)}, name="FWP_BYTE_BLOB", pack=False, align=None), "certHash": SimStruct({"size": SimTypeInt(signed=False, label="UInt32"), "data": SimTypePointer(SimTypeChar(label="Byte"), offset=0)}, name="FWP_BYTE_BLOB", pack=False, align=None), "eku": SimTypePointer(SimStruct({"numEku": SimTypeInt(signed=False, label="UInt32"), "eku": SimTypePointer(SimTypePointer(SimTypeChar(label="Byte"), offset=0), offset=0)}, name="IKEEXT_CERT_EKUS0", pack=False, align=None), offset=0), "name": SimTypePointer(SimStruct({"nameType": SimTypeInt(signed=False, label="IKEEXT_CERT_CRITERIA_NAME_TYPE"), "certName": SimTypePointer(SimTypeChar(label="Char"), offset=0)}, name="IKEEXT_CERT_NAME0", pack=False, align=None), offset=0), "flags": SimTypeInt(signed=False, label="UInt32")}, name="IKEEXT_CERTIFICATE_CRITERIA0", pack=False, align=None), offset=0)}, name="_Anonymous3_e__Struct", pack=False, align=None)}, name="<anon>", label="None"), "flags": SimTypeInt(signed=False, label="IKEEXT_CERT_AUTH"), "localCertLocationUrl": SimStruct({"size": SimTypeInt(signed=False, label="UInt32"), "data": SimTypePointer(SimTypeChar(label="Byte"), offset=0)}, name="FWP_BYTE_BLOB", pack=False, align=None)}, name="IKEEXT_CERTIFICATE_AUTHENTICATION2", pack=False, align=None), "kerberosAuthentication": SimStruct({"flags": SimTypeInt(signed=False, label="IKEEXT_KERBEROS_AUTHENTICATION_FLAGS"), "proxyServer": SimTypePointer(SimTypeChar(label="Char"), offset=0)}, name="IKEEXT_KERBEROS_AUTHENTICATION1", pack=False, align=None), "reservedAuthentication": SimStruct({"flags": SimTypeInt(signed=False, label="IKEEXT_RESERVED_AUTHENTICATION_FLAGS")}, name="IKEEXT_RESERVED_AUTHENTICATION0", pack=False, align=None), "ntlmV2Authentication": SimStruct({"flags": SimTypeInt(signed=False, label="UInt32")}, name="IKEEXT_NTLM_V2_AUTHENTICATION0", pack=False, align=None), "sslAuthentication": SimStruct({"inboundConfigType": SimTypeInt(signed=False, label="IKEEXT_CERT_CONFIG_TYPE"), "Anonymous1": SimUnion({"Anonymous1": SimStruct({"inboundRootArraySize": SimTypeInt(signed=False, label="UInt32"), "inboundRootCriteria": SimTypePointer(SimStruct({"certData": SimStruct({"size": SimTypeInt(signed=False, label="UInt32"), "data": SimTypePointer(SimTypeChar(label="Byte"), offset=0)}, name="FWP_BYTE_BLOB", pack=False, align=None), "certHash": SimStruct({"size": SimTypeInt(signed=False, label="UInt32"), "data": SimTypePointer(SimTypeChar(label="Byte"), offset=0)}, name="FWP_BYTE_BLOB", pack=False, align=None), "eku": SimTypePointer(SimStruct({"numEku": SimTypeInt(signed=False, label="UInt32"), "eku": SimTypePointer(SimTypePointer(SimTypeChar(label="Byte"), offset=0), offset=0)}, name="IKEEXT_CERT_EKUS0", pack=False, align=None), offset=0), "name": SimTypePointer(SimStruct({"nameType": SimTypeInt(signed=False, label="IKEEXT_CERT_CRITERIA_NAME_TYPE"), "certName": SimTypePointer(SimTypeChar(label="Char"), offset=0)}, name="IKEEXT_CERT_NAME0", pack=False, align=None), offset=0), "flags": SimTypeInt(signed=False, label="UInt32")}, name="IKEEXT_CERTIFICATE_CRITERIA0", pack=False, align=None), offset=0)}, name="_Anonymous1_e__Struct", pack=False, align=None), "Anonymous2": SimStruct({"inboundEnterpriseStoreArraySize": SimTypeInt(signed=False, label="UInt32"), "inboundEnterpriseStoreCriteria": SimTypePointer(SimStruct({"certData": SimStruct({"size": SimTypeInt(signed=False, label="UInt32"), "data": SimTypePointer(SimTypeChar(label="Byte"), offset=0)}, name="FWP_BYTE_BLOB", pack=False, align=None), "certHash": SimStruct({"size": SimTypeInt(signed=False, label="UInt32"), "data": SimTypePointer(SimTypeChar(label="Byte"), offset=0)}, name="FWP_BYTE_BLOB", pack=False, align=None), "eku": SimTypePointer(SimStruct({"numEku": SimTypeInt(signed=False, label="UInt32"), "eku": SimTypePointer(SimTypePointer(SimTypeChar(label="Byte"), offset=0), offset=0)}, name="IKEEXT_CERT_EKUS0", pack=False, align=None), offset=0), "name": SimTypePointer(SimStruct({"nameType": SimTypeInt(signed=False, label="IKEEXT_CERT_CRITERIA_NAME_TYPE"), "certName": SimTypePointer(SimTypeChar(label="Char"), offset=0)}, name="IKEEXT_CERT_NAME0", pack=False, align=None), offset=0), "flags": SimTypeInt(signed=False, label="UInt32")}, name="IKEEXT_CERTIFICATE_CRITERIA0", pack=False, align=None), offset=0)}, name="_Anonymous2_e__Struct", pack=False, align=None), "Anonymous3": SimStruct({"inboundRootStoreArraySize": SimTypeInt(signed=False, label="UInt32"), "inboundTrustedRootStoreCriteria": SimTypePointer(SimStruct({"certData": SimStruct({"size": SimTypeInt(signed=False, label="UInt32"), "data": SimTypePointer(SimTypeChar(label="Byte"), offset=0)}, name="FWP_BYTE_BLOB", pack=False, align=None), "certHash": SimStruct({"size": SimTypeInt(signed=False, label="UInt32"), "data": SimTypePointer(SimTypeChar(label="Byte"), offset=0)}, name="FWP_BYTE_BLOB", pack=False, align=None), "eku": SimTypePointer(SimStruct({"numEku": SimTypeInt(signed=False, label="UInt32"), "eku": SimTypePointer(SimTypePointer(SimTypeChar(label="Byte"), offset=0), offset=0)}, name="IKEEXT_CERT_EKUS0", pack=False, align=None), offset=0), "name": SimTypePointer(SimStruct({"nameType": SimTypeInt(signed=False, label="IKEEXT_CERT_CRITERIA_NAME_TYPE"), "certName": SimTypePointer(SimTypeChar(label="Char"), offset=0)}, name="IKEEXT_CERT_NAME0", pack=False, align=None), offset=0), "flags": SimTypeInt(signed=False, label="UInt32")}, name="IKEEXT_CERTIFICATE_CRITERIA0", pack=False, align=None), offset=0)}, name="_Anonymous3_e__Struct", pack=False, align=None)}, name="<anon>", label="None"), "outboundConfigType": SimTypeInt(signed=False, label="IKEEXT_CERT_CONFIG_TYPE"), "Anonymous2": SimUnion({"Anonymous1": SimStruct({"outboundRootArraySize": SimTypeInt(signed=False, label="UInt32"), "outboundRootCriteria": SimTypePointer(SimStruct({"certData": SimStruct({"size": SimTypeInt(signed=False, label="UInt32"), "data": SimTypePointer(SimTypeChar(label="Byte"), offset=0)}, name="FWP_BYTE_BLOB", pack=False, align=None), "certHash": SimStruct({"size": SimTypeInt(signed=False, label="UInt32"), "data": SimTypePointer(SimTypeChar(label="Byte"), offset=0)}, name="FWP_BYTE_BLOB", pack=False, align=None), "eku": SimTypePointer(SimStruct({"numEku": SimTypeInt(signed=False, label="UInt32"), "eku": SimTypePointer(SimTypePointer(SimTypeChar(label="Byte"), offset=0), offset=0)}, name="IKEEXT_CERT_EKUS0", pack=False, align=None), offset=0), "name": SimTypePointer(SimStruct({"nameType": SimTypeInt(signed=False, label="IKEEXT_CERT_CRITERIA_NAME_TYPE"), "certName": SimTypePointer(SimTypeChar(label="Char"), offset=0)}, name="IKEEXT_CERT_NAME0", pack=False, align=None), offset=0), "flags": SimTypeInt(signed=False, label="UInt32")}, name="IKEEXT_CERTIFICATE_CRITERIA0", pack=False, align=None), offset=0)}, name="_Anonymous1_e__Struct", pack=False, align=None), "Anonymous2": SimStruct({"outboundEnterpriseStoreArraySize": SimTypeInt(signed=False, label="UInt32"), "outboundEnterpriseStoreCriteria": SimTypePointer(SimStruct({"certData": SimStruct({"size": SimTypeInt(signed=False, label="UInt32"), "data": SimTypePointer(SimTypeChar(label="Byte"), offset=0)}, name="FWP_BYTE_BLOB", pack=False, align=None), "certHash": SimStruct({"size": SimTypeInt(signed=False, label="UInt32"), "data": SimTypePointer(SimTypeChar(label="Byte"), offset=0)}, name="FWP_BYTE_BLOB", pack=False, align=None), "eku": SimTypePointer(SimStruct({"numEku": SimTypeInt(signed=False, label="UInt32"), "eku": SimTypePointer(SimTypePointer(SimTypeChar(label="Byte"), offset=0), offset=0)}, name="IKEEXT_CERT_EKUS0", pack=False, align=None), offset=0), "name": SimTypePointer(SimStruct({"nameType": SimTypeInt(signed=False, label="IKEEXT_CERT_CRITERIA_NAME_TYPE"), "certName": SimTypePointer(SimTypeChar(label="Char"), offset=0)}, name="IKEEXT_CERT_NAME0", pack=False, align=None), offset=0), "flags": SimTypeInt(signed=False, label="UInt32")}, name="IKEEXT_CERTIFICATE_CRITERIA0", pack=False, align=None), offset=0)}, name="_Anonymous2_e__Struct", pack=False, align=None), "Anonymous3": SimStruct({"outboundRootStoreArraySize": SimTypeInt(signed=False, label="UInt32"), "outboundTrustedRootStoreCriteria": SimTypePointer(SimStruct({"certData": SimStruct({"size": SimTypeInt(signed=False, label="UInt32"), "data": SimTypePointer(SimTypeChar(label="Byte"), offset=0)}, name="FWP_BYTE_BLOB", pack=False, align=None), "certHash": SimStruct({"size": SimTypeInt(signed=False, label="UInt32"), "data": SimTypePointer(SimTypeChar(label="Byte"), offset=0)}, name="FWP_BYTE_BLOB", pack=False, align=None), "eku": SimTypePointer(SimStruct({"numEku": SimTypeInt(signed=False, label="UInt32"), "eku": SimTypePointer(SimTypePointer(SimTypeChar(label="Byte"), offset=0), offset=0)}, name="IKEEXT_CERT_EKUS0", pack=False, align=None), offset=0), "name": SimTypePointer(SimStruct({"nameType": SimTypeInt(signed=False, label="IKEEXT_CERT_CRITERIA_NAME_TYPE"), "certName": SimTypePointer(SimTypeChar(label="Char"), offset=0)}, name="IKEEXT_CERT_NAME0", pack=False, align=None), offset=0), "flags": SimTypeInt(signed=False, label="UInt32")}, name="IKEEXT_CERTIFICATE_CRITERIA0", pack=False, align=None), offset=0)}, name="_Anonymous3_e__Struct", pack=False, align=None)}, name="<anon>", label="None"), "flags": SimTypeInt(signed=False, label="IKEEXT_CERT_AUTH"), "localCertLocationUrl": SimStruct({"size": SimTypeInt(signed=False, label="UInt32"), "data": SimTypePointer(SimTypeChar(label="Byte"), offset=0)}, name="FWP_BYTE_BLOB", pack=False, align=None)}, name="IKEEXT_CERTIFICATE_AUTHENTICATION2", pack=False, align=None), "cgaAuthentication": SimStruct({"keyContainerName": SimTypePointer(SimTypeChar(label="Char"), offset=0), "cspName": SimTypePointer(SimTypeChar(label="Char"), offset=0), "cspType": SimTypeInt(signed=False, label="UInt32"), "cgaModifier": SimStruct({"byteArray16": SimTypeFixedSizeArray(SimTypeChar(label="Byte"), 16)}, name="FWP_BYTE_ARRAY16", pack=False, align=None), "cgaCollisionCount": SimTypeChar(label="Byte")}, name="IKEEXT_IPV6_CGA_AUTHENTICATION0", pack=False, align=None), "eapAuthentication": SimStruct({"flags": SimTypeInt(signed=False, label="IKEEXT_EAP_AUTHENTICATION_FLAGS")}, name="IKEEXT_EAP_AUTHENTICATION0", pack=False, align=None)}, name="<anon>", label="None")}, name="IKEEXT_AUTHENTICATION_METHOD2", pack=False, align=None), offset=0), "initiatorImpersonationType": SimTypeInt(signed=False, label="IKEEXT_AUTHENTICATION_IMPERSONATION_TYPE"), "numIkeProposals": SimTypeInt(signed=False, label="UInt32"), "ikeProposals": SimTypePointer(SimStruct({"cipherAlgorithm": SimStruct({"algoIdentifier": SimTypeInt(signed=False, label="IKEEXT_CIPHER_TYPE"), "keyLen": SimTypeInt(signed=False, label="UInt32"), "rounds": SimTypeInt(signed=False, label="UInt32")}, name="IKEEXT_CIPHER_ALGORITHM0", pack=False, align=None), "integrityAlgorithm": SimStruct({"algoIdentifier": SimTypeInt(signed=False, label="IKEEXT_INTEGRITY_TYPE")}, name="IKEEXT_INTEGRITY_ALGORITHM0", pack=False, align=None), "maxLifetimeSeconds": SimTypeInt(signed=False, label="UInt32"), "dhGroup": SimTypeInt(signed=False, label="IKEEXT_DH_GROUP"), "quickModeLimit": SimTypeInt(signed=False, label="UInt32")}, name="IKEEXT_PROPOSAL0", pack=False, align=None), offset=0), "flags": SimTypeInt(signed=False, label="IKEEXT_POLICY_FLAG"), "maxDynamicFilters": SimTypeInt(signed=False, label="UInt32"), "retransmitDurationSecs": SimTypeInt(signed=False, label="UInt32")}, name="IKEEXT_POLICY2", pack=False, align=None), offset=0), "idpOptions": SimTypePointer(SimStruct({"stateIdleTimeoutSeconds": SimTypeInt(signed=False, label="UInt32"), "perIPRateLimitQueueIdleTimeoutSeconds": SimTypeInt(signed=False, label="UInt32"), "ipV6IPsecUnauthDscp": SimTypeChar(label="Byte"), "ipV6IPsecUnauthRateLimitBytesPerSec": SimTypeInt(signed=False, label="UInt32"), "ipV6IPsecUnauthPerIPRateLimitBytesPerSec": SimTypeInt(signed=False, label="UInt32"), "ipV6IPsecAuthDscp": SimTypeChar(label="Byte"), "ipV6IPsecAuthRateLimitBytesPerSec": SimTypeInt(signed=False, label="UInt32"), "icmpV6Dscp": SimTypeChar(label="Byte"), "icmpV6RateLimitBytesPerSec": SimTypeInt(signed=False, label="UInt32"), "ipV6FilterExemptDscp": SimTypeChar(label="Byte"), "ipV6FilterExemptRateLimitBytesPerSec": SimTypeInt(signed=False, label="UInt32"), "defBlockExemptDscp": SimTypeChar(label="Byte"), "defBlockExemptRateLimitBytesPerSec": SimTypeInt(signed=False, label="UInt32"), "maxStateEntries": SimTypeInt(signed=False, label="UInt32"), "maxPerIPRateLimitQueues": SimTypeInt(signed=False, label="UInt32"), "flags": SimTypeInt(signed=False, label="IPSEC_DOSP_FLAGS"), "numPublicIFLuids": SimTypeInt(signed=False, label="UInt32"), "publicIFLuids": SimTypePointer(SimTypeLongLong(signed=False, label="UInt64"), offset=0), "numInternalIFLuids": SimTypeInt(signed=False, label="UInt32"), "internalIFLuids": SimTypePointer(SimTypeLongLong(signed=False, label="UInt64"), offset=0), "publicV6AddrMask": SimStruct({"addr": SimTypeFixedSizeArray(SimTypeChar(label="Byte"), 16), "prefixLength": SimTypeChar(label="Byte")}, name="FWP_V6_ADDR_AND_MASK", pack=False, align=None), "internalV6AddrMask": SimStruct({"addr": SimTypeFixedSizeArray(SimTypeChar(label="Byte"), 16), "prefixLength": SimTypeChar(label="Byte")}, name="FWP_V6_ADDR_AND_MASK", pack=False, align=None)}, name="IPSEC_DOSP_OPTIONS0", pack=False, align=None), offset=0)}, name="<anon>", label="None"), "providerContextId": SimTypeLongLong(signed=False, label="UInt64")}, name="FWPM_PROVIDER_CONTEXT2", pack=False, align=None), offset=0), offset=0)], SimTypeInt(signed=False, label="UInt32"), arg_names=["engineHandle", "id", "providerContext"]),
        # 
        'FwpmProviderContextGetById3': SimTypeFunction([SimTypePointer(SimTypeInt(signed=True, label="Int"), label="IntPtr", offset=0), SimTypeLongLong(signed=False, label="UInt64"), SimTypePointer(SimTypePointer(SimStruct({"providerContextKey": SimTypeBottom(label="Guid"), "displayData": SimStruct({"name": SimTypePointer(SimTypeChar(label="Char"), offset=0), "description": SimTypePointer(SimTypeChar(label="Char"), offset=0)}, name="FWPM_DISPLAY_DATA0", pack=False, align=None), "flags": SimTypeInt(signed=False, label="UInt32"), "providerKey": SimTypePointer(SimTypeBottom(label="Guid"), offset=0), "providerData": SimStruct({"size": SimTypeInt(signed=False, label="UInt32"), "data": SimTypePointer(SimTypeChar(label="Byte"), offset=0)}, name="FWP_BYTE_BLOB", pack=False, align=None), "type": SimTypeInt(signed=False, label="FWPM_PROVIDER_CONTEXT_TYPE"), "Anonymous": SimUnion({"keyingPolicy": SimTypePointer(SimStruct({"numKeyMods": SimTypeInt(signed=False, label="UInt32"), "keyModKeys": SimTypePointer(SimTypeBottom(label="Guid"), offset=0), "flags": SimTypeInt(signed=False, label="UInt32")}, name="IPSEC_KEYING_POLICY1", pack=False, align=None), offset=0), "ikeQmTransportPolicy": SimTypePointer(SimStruct({"numIpsecProposals": SimTypeInt(signed=False, label="UInt32"), "ipsecProposals": SimTypePointer(SimStruct({"lifetime": SimStruct({"lifetimeSeconds": SimTypeInt(signed=False, label="UInt32"), "lifetimeKilobytes": SimTypeInt(signed=False, label="UInt32"), "lifetimePackets": SimTypeInt(signed=False, label="UInt32")}, name="IPSEC_SA_LIFETIME0", pack=False, align=None), "numSaTransforms": SimTypeInt(signed=False, label="UInt32"), "saTransforms": SimTypePointer(SimStruct({"ipsecTransformType": SimTypeInt(signed=False, label="IPSEC_TRANSFORM_TYPE"), "Anonymous": SimUnion({"ahTransform": SimTypePointer(SimStruct({"authTransformId": SimStruct({"authType": SimTypeInt(signed=False, label="IPSEC_AUTH_TYPE"), "authConfig": SimTypeChar(label="Byte")}, name="IPSEC_AUTH_TRANSFORM_ID0", pack=False, align=None), "cryptoModuleId": SimTypePointer(SimTypeBottom(label="Guid"), offset=0)}, name="IPSEC_AUTH_TRANSFORM0", pack=False, align=None), offset=0), "espAuthTransform": SimTypePointer(SimStruct({"authTransformId": SimStruct({"authType": SimTypeInt(signed=False, label="IPSEC_AUTH_TYPE"), "authConfig": SimTypeChar(label="Byte")}, name="IPSEC_AUTH_TRANSFORM_ID0", pack=False, align=None), "cryptoModuleId": SimTypePointer(SimTypeBottom(label="Guid"), offset=0)}, name="IPSEC_AUTH_TRANSFORM0", pack=False, align=None), offset=0), "espCipherTransform": SimTypePointer(SimStruct({"cipherTransformId": SimStruct({"cipherType": SimTypeInt(signed=False, label="IPSEC_CIPHER_TYPE"), "cipherConfig": SimTypeChar(label="Byte")}, name="IPSEC_CIPHER_TRANSFORM_ID0", pack=False, align=None), "cryptoModuleId": SimTypePointer(SimTypeBottom(label="Guid"), offset=0)}, name="IPSEC_CIPHER_TRANSFORM0", pack=False, align=None), offset=0), "espAuthAndCipherTransform": SimTypePointer(SimStruct({"authTransform": SimStruct({"authTransformId": SimStruct({"authType": SimTypeInt(signed=False, label="IPSEC_AUTH_TYPE"), "authConfig": SimTypeChar(label="Byte")}, name="IPSEC_AUTH_TRANSFORM_ID0", pack=False, align=None), "cryptoModuleId": SimTypePointer(SimTypeBottom(label="Guid"), offset=0)}, name="IPSEC_AUTH_TRANSFORM0", pack=False, align=None), "cipherTransform": SimStruct({"cipherTransformId": SimStruct({"cipherType": SimTypeInt(signed=False, label="IPSEC_CIPHER_TYPE"), "cipherConfig": SimTypeChar(label="Byte")}, name="IPSEC_CIPHER_TRANSFORM_ID0", pack=False, align=None), "cryptoModuleId": SimTypePointer(SimTypeBottom(label="Guid"), offset=0)}, name="IPSEC_CIPHER_TRANSFORM0", pack=False, align=None)}, name="IPSEC_AUTH_AND_CIPHER_TRANSFORM0", pack=False, align=None), offset=0), "espAuthFwTransform": SimTypePointer(SimStruct({"authTransformId": SimStruct({"authType": SimTypeInt(signed=False, label="IPSEC_AUTH_TYPE"), "authConfig": SimTypeChar(label="Byte")}, name="IPSEC_AUTH_TRANSFORM_ID0", pack=False, align=None), "cryptoModuleId": SimTypePointer(SimTypeBottom(label="Guid"), offset=0)}, name="IPSEC_AUTH_TRANSFORM0", pack=False, align=None), offset=0)}, name="<anon>", label="None")}, name="IPSEC_SA_TRANSFORM0", pack=False, align=None), offset=0), "pfsGroup": SimTypeInt(signed=False, label="IPSEC_PFS_GROUP")}, name="IPSEC_PROPOSAL0", pack=False, align=None), offset=0), "flags": SimTypeInt(signed=False, label="IPSEC_POLICY_FLAG"), "ndAllowClearTimeoutSeconds": SimTypeInt(signed=False, label="UInt32"), "saIdleTimeout": SimStruct({"idleTimeoutSeconds": SimTypeInt(signed=False, label="UInt32"), "idleTimeoutSecondsFailOver": SimTypeInt(signed=False, label="UInt32")}, name="IPSEC_SA_IDLE_TIMEOUT0", pack=False, align=None), "emPolicy": SimTypePointer(SimStruct({"numAuthenticationMethods": SimTypeInt(signed=False, label="UInt32"), "authenticationMethods": SimTypePointer(SimStruct({"authenticationMethodType": SimTypeInt(signed=False, label="IKEEXT_AUTHENTICATION_METHOD_TYPE"), "Anonymous": SimUnion({"presharedKeyAuthentication": SimStruct({"presharedKey": SimStruct({"size": SimTypeInt(signed=False, label="UInt32"), "data": SimTypePointer(SimTypeChar(label="Byte"), offset=0)}, name="FWP_BYTE_BLOB", pack=False, align=None), "flags": SimTypeInt(signed=False, label="IKEEXT_PRESHARED_KEY_AUTHENTICATION_FLAGS")}, name="IKEEXT_PRESHARED_KEY_AUTHENTICATION1", pack=False, align=None), "certificateAuthentication": SimStruct({"inboundConfigType": SimTypeInt(signed=False, label="IKEEXT_CERT_CONFIG_TYPE"), "Anonymous1": SimUnion({"Anonymous1": SimStruct({"inboundRootArraySize": SimTypeInt(signed=False, label="UInt32"), "inboundRootCriteria": SimTypePointer(SimStruct({"certData": SimStruct({"size": SimTypeInt(signed=False, label="UInt32"), "data": SimTypePointer(SimTypeChar(label="Byte"), offset=0)}, name="FWP_BYTE_BLOB", pack=False, align=None), "certHash": SimStruct({"size": SimTypeInt(signed=False, label="UInt32"), "data": SimTypePointer(SimTypeChar(label="Byte"), offset=0)}, name="FWP_BYTE_BLOB", pack=False, align=None), "eku": SimTypePointer(SimStruct({"numEku": SimTypeInt(signed=False, label="UInt32"), "eku": SimTypePointer(SimTypePointer(SimTypeChar(label="Byte"), offset=0), offset=0)}, name="IKEEXT_CERT_EKUS0", pack=False, align=None), offset=0), "name": SimTypePointer(SimStruct({"nameType": SimTypeInt(signed=False, label="IKEEXT_CERT_CRITERIA_NAME_TYPE"), "certName": SimTypePointer(SimTypeChar(label="Char"), offset=0)}, name="IKEEXT_CERT_NAME0", pack=False, align=None), offset=0), "flags": SimTypeInt(signed=False, label="UInt32")}, name="IKEEXT_CERTIFICATE_CRITERIA0", pack=False, align=None), offset=0)}, name="_Anonymous1_e__Struct", pack=False, align=None), "Anonymous2": SimStruct({"inboundEnterpriseStoreArraySize": SimTypeInt(signed=False, label="UInt32"), "inboundEnterpriseStoreCriteria": SimTypePointer(SimStruct({"certData": SimStruct({"size": SimTypeInt(signed=False, label="UInt32"), "data": SimTypePointer(SimTypeChar(label="Byte"), offset=0)}, name="FWP_BYTE_BLOB", pack=False, align=None), "certHash": SimStruct({"size": SimTypeInt(signed=False, label="UInt32"), "data": SimTypePointer(SimTypeChar(label="Byte"), offset=0)}, name="FWP_BYTE_BLOB", pack=False, align=None), "eku": SimTypePointer(SimStruct({"numEku": SimTypeInt(signed=False, label="UInt32"), "eku": SimTypePointer(SimTypePointer(SimTypeChar(label="Byte"), offset=0), offset=0)}, name="IKEEXT_CERT_EKUS0", pack=False, align=None), offset=0), "name": SimTypePointer(SimStruct({"nameType": SimTypeInt(signed=False, label="IKEEXT_CERT_CRITERIA_NAME_TYPE"), "certName": SimTypePointer(SimTypeChar(label="Char"), offset=0)}, name="IKEEXT_CERT_NAME0", pack=False, align=None), offset=0), "flags": SimTypeInt(signed=False, label="UInt32")}, name="IKEEXT_CERTIFICATE_CRITERIA0", pack=False, align=None), offset=0)}, name="_Anonymous2_e__Struct", pack=False, align=None), "Anonymous3": SimStruct({"inboundRootStoreArraySize": SimTypeInt(signed=False, label="UInt32"), "inboundTrustedRootStoreCriteria": SimTypePointer(SimStruct({"certData": SimStruct({"size": SimTypeInt(signed=False, label="UInt32"), "data": SimTypePointer(SimTypeChar(label="Byte"), offset=0)}, name="FWP_BYTE_BLOB", pack=False, align=None), "certHash": SimStruct({"size": SimTypeInt(signed=False, label="UInt32"), "data": SimTypePointer(SimTypeChar(label="Byte"), offset=0)}, name="FWP_BYTE_BLOB", pack=False, align=None), "eku": SimTypePointer(SimStruct({"numEku": SimTypeInt(signed=False, label="UInt32"), "eku": SimTypePointer(SimTypePointer(SimTypeChar(label="Byte"), offset=0), offset=0)}, name="IKEEXT_CERT_EKUS0", pack=False, align=None), offset=0), "name": SimTypePointer(SimStruct({"nameType": SimTypeInt(signed=False, label="IKEEXT_CERT_CRITERIA_NAME_TYPE"), "certName": SimTypePointer(SimTypeChar(label="Char"), offset=0)}, name="IKEEXT_CERT_NAME0", pack=False, align=None), offset=0), "flags": SimTypeInt(signed=False, label="UInt32")}, name="IKEEXT_CERTIFICATE_CRITERIA0", pack=False, align=None), offset=0)}, name="_Anonymous3_e__Struct", pack=False, align=None)}, name="<anon>", label="None"), "outboundConfigType": SimTypeInt(signed=False, label="IKEEXT_CERT_CONFIG_TYPE"), "Anonymous2": SimUnion({"Anonymous1": SimStruct({"outboundRootArraySize": SimTypeInt(signed=False, label="UInt32"), "outboundRootCriteria": SimTypePointer(SimStruct({"certData": SimStruct({"size": SimTypeInt(signed=False, label="UInt32"), "data": SimTypePointer(SimTypeChar(label="Byte"), offset=0)}, name="FWP_BYTE_BLOB", pack=False, align=None), "certHash": SimStruct({"size": SimTypeInt(signed=False, label="UInt32"), "data": SimTypePointer(SimTypeChar(label="Byte"), offset=0)}, name="FWP_BYTE_BLOB", pack=False, align=None), "eku": SimTypePointer(SimStruct({"numEku": SimTypeInt(signed=False, label="UInt32"), "eku": SimTypePointer(SimTypePointer(SimTypeChar(label="Byte"), offset=0), offset=0)}, name="IKEEXT_CERT_EKUS0", pack=False, align=None), offset=0), "name": SimTypePointer(SimStruct({"nameType": SimTypeInt(signed=False, label="IKEEXT_CERT_CRITERIA_NAME_TYPE"), "certName": SimTypePointer(SimTypeChar(label="Char"), offset=0)}, name="IKEEXT_CERT_NAME0", pack=False, align=None), offset=0), "flags": SimTypeInt(signed=False, label="UInt32")}, name="IKEEXT_CERTIFICATE_CRITERIA0", pack=False, align=None), offset=0)}, name="_Anonymous1_e__Struct", pack=False, align=None), "Anonymous2": SimStruct({"outboundEnterpriseStoreArraySize": SimTypeInt(signed=False, label="UInt32"), "outboundEnterpriseStoreCriteria": SimTypePointer(SimStruct({"certData": SimStruct({"size": SimTypeInt(signed=False, label="UInt32"), "data": SimTypePointer(SimTypeChar(label="Byte"), offset=0)}, name="FWP_BYTE_BLOB", pack=False, align=None), "certHash": SimStruct({"size": SimTypeInt(signed=False, label="UInt32"), "data": SimTypePointer(SimTypeChar(label="Byte"), offset=0)}, name="FWP_BYTE_BLOB", pack=False, align=None), "eku": SimTypePointer(SimStruct({"numEku": SimTypeInt(signed=False, label="UInt32"), "eku": SimTypePointer(SimTypePointer(SimTypeChar(label="Byte"), offset=0), offset=0)}, name="IKEEXT_CERT_EKUS0", pack=False, align=None), offset=0), "name": SimTypePointer(SimStruct({"nameType": SimTypeInt(signed=False, label="IKEEXT_CERT_CRITERIA_NAME_TYPE"), "certName": SimTypePointer(SimTypeChar(label="Char"), offset=0)}, name="IKEEXT_CERT_NAME0", pack=False, align=None), offset=0), "flags": SimTypeInt(signed=False, label="UInt32")}, name="IKEEXT_CERTIFICATE_CRITERIA0", pack=False, align=None), offset=0)}, name="_Anonymous2_e__Struct", pack=False, align=None), "Anonymous3": SimStruct({"outboundRootStoreArraySize": SimTypeInt(signed=False, label="UInt32"), "outboundTrustedRootStoreCriteria": SimTypePointer(SimStruct({"certData": SimStruct({"size": SimTypeInt(signed=False, label="UInt32"), "data": SimTypePointer(SimTypeChar(label="Byte"), offset=0)}, name="FWP_BYTE_BLOB", pack=False, align=None), "certHash": SimStruct({"size": SimTypeInt(signed=False, label="UInt32"), "data": SimTypePointer(SimTypeChar(label="Byte"), offset=0)}, name="FWP_BYTE_BLOB", pack=False, align=None), "eku": SimTypePointer(SimStruct({"numEku": SimTypeInt(signed=False, label="UInt32"), "eku": SimTypePointer(SimTypePointer(SimTypeChar(label="Byte"), offset=0), offset=0)}, name="IKEEXT_CERT_EKUS0", pack=False, align=None), offset=0), "name": SimTypePointer(SimStruct({"nameType": SimTypeInt(signed=False, label="IKEEXT_CERT_CRITERIA_NAME_TYPE"), "certName": SimTypePointer(SimTypeChar(label="Char"), offset=0)}, name="IKEEXT_CERT_NAME0", pack=False, align=None), offset=0), "flags": SimTypeInt(signed=False, label="UInt32")}, name="IKEEXT_CERTIFICATE_CRITERIA0", pack=False, align=None), offset=0)}, name="_Anonymous3_e__Struct", pack=False, align=None)}, name="<anon>", label="None"), "flags": SimTypeInt(signed=False, label="IKEEXT_CERT_AUTH"), "localCertLocationUrl": SimStruct({"size": SimTypeInt(signed=False, label="UInt32"), "data": SimTypePointer(SimTypeChar(label="Byte"), offset=0)}, name="FWP_BYTE_BLOB", pack=False, align=None)}, name="IKEEXT_CERTIFICATE_AUTHENTICATION2", pack=False, align=None), "kerberosAuthentication": SimStruct({"flags": SimTypeInt(signed=False, label="IKEEXT_KERBEROS_AUTHENTICATION_FLAGS"), "proxyServer": SimTypePointer(SimTypeChar(label="Char"), offset=0)}, name="IKEEXT_KERBEROS_AUTHENTICATION1", pack=False, align=None), "reservedAuthentication": SimStruct({"flags": SimTypeInt(signed=False, label="IKEEXT_RESERVED_AUTHENTICATION_FLAGS")}, name="IKEEXT_RESERVED_AUTHENTICATION0", pack=False, align=None), "ntlmV2Authentication": SimStruct({"flags": SimTypeInt(signed=False, label="UInt32")}, name="IKEEXT_NTLM_V2_AUTHENTICATION0", pack=False, align=None), "sslAuthentication": SimStruct({"inboundConfigType": SimTypeInt(signed=False, label="IKEEXT_CERT_CONFIG_TYPE"), "Anonymous1": SimUnion({"Anonymous1": SimStruct({"inboundRootArraySize": SimTypeInt(signed=False, label="UInt32"), "inboundRootCriteria": SimTypePointer(SimStruct({"certData": SimStruct({"size": SimTypeInt(signed=False, label="UInt32"), "data": SimTypePointer(SimTypeChar(label="Byte"), offset=0)}, name="FWP_BYTE_BLOB", pack=False, align=None), "certHash": SimStruct({"size": SimTypeInt(signed=False, label="UInt32"), "data": SimTypePointer(SimTypeChar(label="Byte"), offset=0)}, name="FWP_BYTE_BLOB", pack=False, align=None), "eku": SimTypePointer(SimStruct({"numEku": SimTypeInt(signed=False, label="UInt32"), "eku": SimTypePointer(SimTypePointer(SimTypeChar(label="Byte"), offset=0), offset=0)}, name="IKEEXT_CERT_EKUS0", pack=False, align=None), offset=0), "name": SimTypePointer(SimStruct({"nameType": SimTypeInt(signed=False, label="IKEEXT_CERT_CRITERIA_NAME_TYPE"), "certName": SimTypePointer(SimTypeChar(label="Char"), offset=0)}, name="IKEEXT_CERT_NAME0", pack=False, align=None), offset=0), "flags": SimTypeInt(signed=False, label="UInt32")}, name="IKEEXT_CERTIFICATE_CRITERIA0", pack=False, align=None), offset=0)}, name="_Anonymous1_e__Struct", pack=False, align=None), "Anonymous2": SimStruct({"inboundEnterpriseStoreArraySize": SimTypeInt(signed=False, label="UInt32"), "inboundEnterpriseStoreCriteria": SimTypePointer(SimStruct({"certData": SimStruct({"size": SimTypeInt(signed=False, label="UInt32"), "data": SimTypePointer(SimTypeChar(label="Byte"), offset=0)}, name="FWP_BYTE_BLOB", pack=False, align=None), "certHash": SimStruct({"size": SimTypeInt(signed=False, label="UInt32"), "data": SimTypePointer(SimTypeChar(label="Byte"), offset=0)}, name="FWP_BYTE_BLOB", pack=False, align=None), "eku": SimTypePointer(SimStruct({"numEku": SimTypeInt(signed=False, label="UInt32"), "eku": SimTypePointer(SimTypePointer(SimTypeChar(label="Byte"), offset=0), offset=0)}, name="IKEEXT_CERT_EKUS0", pack=False, align=None), offset=0), "name": SimTypePointer(SimStruct({"nameType": SimTypeInt(signed=False, label="IKEEXT_CERT_CRITERIA_NAME_TYPE"), "certName": SimTypePointer(SimTypeChar(label="Char"), offset=0)}, name="IKEEXT_CERT_NAME0", pack=False, align=None), offset=0), "flags": SimTypeInt(signed=False, label="UInt32")}, name="IKEEXT_CERTIFICATE_CRITERIA0", pack=False, align=None), offset=0)}, name="_Anonymous2_e__Struct", pack=False, align=None), "Anonymous3": SimStruct({"inboundRootStoreArraySize": SimTypeInt(signed=False, label="UInt32"), "inboundTrustedRootStoreCriteria": SimTypePointer(SimStruct({"certData": SimStruct({"size": SimTypeInt(signed=False, label="UInt32"), "data": SimTypePointer(SimTypeChar(label="Byte"), offset=0)}, name="FWP_BYTE_BLOB", pack=False, align=None), "certHash": SimStruct({"size": SimTypeInt(signed=False, label="UInt32"), "data": SimTypePointer(SimTypeChar(label="Byte"), offset=0)}, name="FWP_BYTE_BLOB", pack=False, align=None), "eku": SimTypePointer(SimStruct({"numEku": SimTypeInt(signed=False, label="UInt32"), "eku": SimTypePointer(SimTypePointer(SimTypeChar(label="Byte"), offset=0), offset=0)}, name="IKEEXT_CERT_EKUS0", pack=False, align=None), offset=0), "name": SimTypePointer(SimStruct({"nameType": SimTypeInt(signed=False, label="IKEEXT_CERT_CRITERIA_NAME_TYPE"), "certName": SimTypePointer(SimTypeChar(label="Char"), offset=0)}, name="IKEEXT_CERT_NAME0", pack=False, align=None), offset=0), "flags": SimTypeInt(signed=False, label="UInt32")}, name="IKEEXT_CERTIFICATE_CRITERIA0", pack=False, align=None), offset=0)}, name="_Anonymous3_e__Struct", pack=False, align=None)}, name="<anon>", label="None"), "outboundConfigType": SimTypeInt(signed=False, label="IKEEXT_CERT_CONFIG_TYPE"), "Anonymous2": SimUnion({"Anonymous1": SimStruct({"outboundRootArraySize": SimTypeInt(signed=False, label="UInt32"), "outboundRootCriteria": SimTypePointer(SimStruct({"certData": SimStruct({"size": SimTypeInt(signed=False, label="UInt32"), "data": SimTypePointer(SimTypeChar(label="Byte"), offset=0)}, name="FWP_BYTE_BLOB", pack=False, align=None), "certHash": SimStruct({"size": SimTypeInt(signed=False, label="UInt32"), "data": SimTypePointer(SimTypeChar(label="Byte"), offset=0)}, name="FWP_BYTE_BLOB", pack=False, align=None), "eku": SimTypePointer(SimStruct({"numEku": SimTypeInt(signed=False, label="UInt32"), "eku": SimTypePointer(SimTypePointer(SimTypeChar(label="Byte"), offset=0), offset=0)}, name="IKEEXT_CERT_EKUS0", pack=False, align=None), offset=0), "name": SimTypePointer(SimStruct({"nameType": SimTypeInt(signed=False, label="IKEEXT_CERT_CRITERIA_NAME_TYPE"), "certName": SimTypePointer(SimTypeChar(label="Char"), offset=0)}, name="IKEEXT_CERT_NAME0", pack=False, align=None), offset=0), "flags": SimTypeInt(signed=False, label="UInt32")}, name="IKEEXT_CERTIFICATE_CRITERIA0", pack=False, align=None), offset=0)}, name="_Anonymous1_e__Struct", pack=False, align=None), "Anonymous2": SimStruct({"outboundEnterpriseStoreArraySize": SimTypeInt(signed=False, label="UInt32"), "outboundEnterpriseStoreCriteria": SimTypePointer(SimStruct({"certData": SimStruct({"size": SimTypeInt(signed=False, label="UInt32"), "data": SimTypePointer(SimTypeChar(label="Byte"), offset=0)}, name="FWP_BYTE_BLOB", pack=False, align=None), "certHash": SimStruct({"size": SimTypeInt(signed=False, label="UInt32"), "data": SimTypePointer(SimTypeChar(label="Byte"), offset=0)}, name="FWP_BYTE_BLOB", pack=False, align=None), "eku": SimTypePointer(SimStruct({"numEku": SimTypeInt(signed=False, label="UInt32"), "eku": SimTypePointer(SimTypePointer(SimTypeChar(label="Byte"), offset=0), offset=0)}, name="IKEEXT_CERT_EKUS0", pack=False, align=None), offset=0), "name": SimTypePointer(SimStruct({"nameType": SimTypeInt(signed=False, label="IKEEXT_CERT_CRITERIA_NAME_TYPE"), "certName": SimTypePointer(SimTypeChar(label="Char"), offset=0)}, name="IKEEXT_CERT_NAME0", pack=False, align=None), offset=0), "flags": SimTypeInt(signed=False, label="UInt32")}, name="IKEEXT_CERTIFICATE_CRITERIA0", pack=False, align=None), offset=0)}, name="_Anonymous2_e__Struct", pack=False, align=None), "Anonymous3": SimStruct({"outboundRootStoreArraySize": SimTypeInt(signed=False, label="UInt32"), "outboundTrustedRootStoreCriteria": SimTypePointer(SimStruct({"certData": SimStruct({"size": SimTypeInt(signed=False, label="UInt32"), "data": SimTypePointer(SimTypeChar(label="Byte"), offset=0)}, name="FWP_BYTE_BLOB", pack=False, align=None), "certHash": SimStruct({"size": SimTypeInt(signed=False, label="UInt32"), "data": SimTypePointer(SimTypeChar(label="Byte"), offset=0)}, name="FWP_BYTE_BLOB", pack=False, align=None), "eku": SimTypePointer(SimStruct({"numEku": SimTypeInt(signed=False, label="UInt32"), "eku": SimTypePointer(SimTypePointer(SimTypeChar(label="Byte"), offset=0), offset=0)}, name="IKEEXT_CERT_EKUS0", pack=False, align=None), offset=0), "name": SimTypePointer(SimStruct({"nameType": SimTypeInt(signed=False, label="IKEEXT_CERT_CRITERIA_NAME_TYPE"), "certName": SimTypePointer(SimTypeChar(label="Char"), offset=0)}, name="IKEEXT_CERT_NAME0", pack=False, align=None), offset=0), "flags": SimTypeInt(signed=False, label="UInt32")}, name="IKEEXT_CERTIFICATE_CRITERIA0", pack=False, align=None), offset=0)}, name="_Anonymous3_e__Struct", pack=False, align=None)}, name="<anon>", label="None"), "flags": SimTypeInt(signed=False, label="IKEEXT_CERT_AUTH"), "localCertLocationUrl": SimStruct({"size": SimTypeInt(signed=False, label="UInt32"), "data": SimTypePointer(SimTypeChar(label="Byte"), offset=0)}, name="FWP_BYTE_BLOB", pack=False, align=None)}, name="IKEEXT_CERTIFICATE_AUTHENTICATION2", pack=False, align=None), "cgaAuthentication": SimStruct({"keyContainerName": SimTypePointer(SimTypeChar(label="Char"), offset=0), "cspName": SimTypePointer(SimTypeChar(label="Char"), offset=0), "cspType": SimTypeInt(signed=False, label="UInt32"), "cgaModifier": SimStruct({"byteArray16": SimTypeFixedSizeArray(SimTypeChar(label="Byte"), 16)}, name="FWP_BYTE_ARRAY16", pack=False, align=None), "cgaCollisionCount": SimTypeChar(label="Byte")}, name="IKEEXT_IPV6_CGA_AUTHENTICATION0", pack=False, align=None), "eapAuthentication": SimStruct({"flags": SimTypeInt(signed=False, label="IKEEXT_EAP_AUTHENTICATION_FLAGS")}, name="IKEEXT_EAP_AUTHENTICATION0", pack=False, align=None)}, name="<anon>", label="None")}, name="IKEEXT_AUTHENTICATION_METHOD2", pack=False, align=None), offset=0), "initiatorImpersonationType": SimTypeInt(signed=False, label="IKEEXT_AUTHENTICATION_IMPERSONATION_TYPE")}, name="IKEEXT_EM_POLICY2", pack=False, align=None), offset=0)}, name="IPSEC_TRANSPORT_POLICY2", pack=False, align=None), offset=0), "ikeQmTunnelPolicy": SimTypePointer(SimStruct({"flags": SimTypeInt(signed=False, label="UInt32"), "numIpsecProposals": SimTypeInt(signed=False, label="UInt32"), "ipsecProposals": SimTypePointer(SimStruct({"lifetime": SimStruct({"lifetimeSeconds": SimTypeInt(signed=False, label="UInt32"), "lifetimeKilobytes": SimTypeInt(signed=False, label="UInt32"), "lifetimePackets": SimTypeInt(signed=False, label="UInt32")}, name="IPSEC_SA_LIFETIME0", pack=False, align=None), "numSaTransforms": SimTypeInt(signed=False, label="UInt32"), "saTransforms": SimTypePointer(SimStruct({"ipsecTransformType": SimTypeInt(signed=False, label="IPSEC_TRANSFORM_TYPE"), "Anonymous": SimUnion({"ahTransform": SimTypePointer(SimStruct({"authTransformId": SimStruct({"authType": SimTypeInt(signed=False, label="IPSEC_AUTH_TYPE"), "authConfig": SimTypeChar(label="Byte")}, name="IPSEC_AUTH_TRANSFORM_ID0", pack=False, align=None), "cryptoModuleId": SimTypePointer(SimTypeBottom(label="Guid"), offset=0)}, name="IPSEC_AUTH_TRANSFORM0", pack=False, align=None), offset=0), "espAuthTransform": SimTypePointer(SimStruct({"authTransformId": SimStruct({"authType": SimTypeInt(signed=False, label="IPSEC_AUTH_TYPE"), "authConfig": SimTypeChar(label="Byte")}, name="IPSEC_AUTH_TRANSFORM_ID0", pack=False, align=None), "cryptoModuleId": SimTypePointer(SimTypeBottom(label="Guid"), offset=0)}, name="IPSEC_AUTH_TRANSFORM0", pack=False, align=None), offset=0), "espCipherTransform": SimTypePointer(SimStruct({"cipherTransformId": SimStruct({"cipherType": SimTypeInt(signed=False, label="IPSEC_CIPHER_TYPE"), "cipherConfig": SimTypeChar(label="Byte")}, name="IPSEC_CIPHER_TRANSFORM_ID0", pack=False, align=None), "cryptoModuleId": SimTypePointer(SimTypeBottom(label="Guid"), offset=0)}, name="IPSEC_CIPHER_TRANSFORM0", pack=False, align=None), offset=0), "espAuthAndCipherTransform": SimTypePointer(SimStruct({"authTransform": SimStruct({"authTransformId": SimStruct({"authType": SimTypeInt(signed=False, label="IPSEC_AUTH_TYPE"), "authConfig": SimTypeChar(label="Byte")}, name="IPSEC_AUTH_TRANSFORM_ID0", pack=False, align=None), "cryptoModuleId": SimTypePointer(SimTypeBottom(label="Guid"), offset=0)}, name="IPSEC_AUTH_TRANSFORM0", pack=False, align=None), "cipherTransform": SimStruct({"cipherTransformId": SimStruct({"cipherType": SimTypeInt(signed=False, label="IPSEC_CIPHER_TYPE"), "cipherConfig": SimTypeChar(label="Byte")}, name="IPSEC_CIPHER_TRANSFORM_ID0", pack=False, align=None), "cryptoModuleId": SimTypePointer(SimTypeBottom(label="Guid"), offset=0)}, name="IPSEC_CIPHER_TRANSFORM0", pack=False, align=None)}, name="IPSEC_AUTH_AND_CIPHER_TRANSFORM0", pack=False, align=None), offset=0), "espAuthFwTransform": SimTypePointer(SimStruct({"authTransformId": SimStruct({"authType": SimTypeInt(signed=False, label="IPSEC_AUTH_TYPE"), "authConfig": SimTypeChar(label="Byte")}, name="IPSEC_AUTH_TRANSFORM_ID0", pack=False, align=None), "cryptoModuleId": SimTypePointer(SimTypeBottom(label="Guid"), offset=0)}, name="IPSEC_AUTH_TRANSFORM0", pack=False, align=None), offset=0)}, name="<anon>", label="None")}, name="IPSEC_SA_TRANSFORM0", pack=False, align=None), offset=0), "pfsGroup": SimTypeInt(signed=False, label="IPSEC_PFS_GROUP")}, name="IPSEC_PROPOSAL0", pack=False, align=None), offset=0), "tunnelEndpoints": SimStruct({"ipVersion": SimTypeInt(signed=False, label="FWP_IP_VERSION"), "Anonymous1": SimUnion({"localV4Address": SimTypeInt(signed=False, label="UInt32"), "localV6Address": SimTypeFixedSizeArray(SimTypeChar(label="Byte"), 16)}, name="<anon>", label="None"), "Anonymous2": SimUnion({"remoteV4Address": SimTypeInt(signed=False, label="UInt32"), "remoteV6Address": SimTypeFixedSizeArray(SimTypeChar(label="Byte"), 16)}, name="<anon>", label="None"), "localIfLuid": SimTypeLongLong(signed=False, label="UInt64"), "remoteFqdn": SimTypePointer(SimTypeChar(label="Char"), offset=0), "numAddresses": SimTypeInt(signed=False, label="UInt32"), "remoteAddresses": SimTypePointer(SimStruct({"ipVersion": SimTypeInt(signed=False, label="FWP_IP_VERSION"), "Anonymous": SimUnion({"v4Address": SimTypeInt(signed=False, label="UInt32"), "v6Address": SimTypeFixedSizeArray(SimTypeChar(label="Byte"), 16)}, name="<anon>", label="None")}, name="IPSEC_TUNNEL_ENDPOINT0", pack=False, align=None), offset=0)}, name="IPSEC_TUNNEL_ENDPOINTS2", pack=False, align=None), "saIdleTimeout": SimStruct({"idleTimeoutSeconds": SimTypeInt(signed=False, label="UInt32"), "idleTimeoutSecondsFailOver": SimTypeInt(signed=False, label="UInt32")}, name="IPSEC_SA_IDLE_TIMEOUT0", pack=False, align=None), "emPolicy": SimTypePointer(SimStruct({"numAuthenticationMethods": SimTypeInt(signed=False, label="UInt32"), "authenticationMethods": SimTypePointer(SimStruct({"authenticationMethodType": SimTypeInt(signed=False, label="IKEEXT_AUTHENTICATION_METHOD_TYPE"), "Anonymous": SimUnion({"presharedKeyAuthentication": SimStruct({"presharedKey": SimStruct({"size": SimTypeInt(signed=False, label="UInt32"), "data": SimTypePointer(SimTypeChar(label="Byte"), offset=0)}, name="FWP_BYTE_BLOB", pack=False, align=None), "flags": SimTypeInt(signed=False, label="IKEEXT_PRESHARED_KEY_AUTHENTICATION_FLAGS")}, name="IKEEXT_PRESHARED_KEY_AUTHENTICATION1", pack=False, align=None), "certificateAuthentication": SimStruct({"inboundConfigType": SimTypeInt(signed=False, label="IKEEXT_CERT_CONFIG_TYPE"), "Anonymous1": SimUnion({"Anonymous1": SimStruct({"inboundRootArraySize": SimTypeInt(signed=False, label="UInt32"), "inboundRootCriteria": SimTypePointer(SimStruct({"certData": SimStruct({"size": SimTypeInt(signed=False, label="UInt32"), "data": SimTypePointer(SimTypeChar(label="Byte"), offset=0)}, name="FWP_BYTE_BLOB", pack=False, align=None), "certHash": SimStruct({"size": SimTypeInt(signed=False, label="UInt32"), "data": SimTypePointer(SimTypeChar(label="Byte"), offset=0)}, name="FWP_BYTE_BLOB", pack=False, align=None), "eku": SimTypePointer(SimStruct({"numEku": SimTypeInt(signed=False, label="UInt32"), "eku": SimTypePointer(SimTypePointer(SimTypeChar(label="Byte"), offset=0), offset=0)}, name="IKEEXT_CERT_EKUS0", pack=False, align=None), offset=0), "name": SimTypePointer(SimStruct({"nameType": SimTypeInt(signed=False, label="IKEEXT_CERT_CRITERIA_NAME_TYPE"), "certName": SimTypePointer(SimTypeChar(label="Char"), offset=0)}, name="IKEEXT_CERT_NAME0", pack=False, align=None), offset=0), "flags": SimTypeInt(signed=False, label="UInt32")}, name="IKEEXT_CERTIFICATE_CRITERIA0", pack=False, align=None), offset=0)}, name="_Anonymous1_e__Struct", pack=False, align=None), "Anonymous2": SimStruct({"inboundEnterpriseStoreArraySize": SimTypeInt(signed=False, label="UInt32"), "inboundEnterpriseStoreCriteria": SimTypePointer(SimStruct({"certData": SimStruct({"size": SimTypeInt(signed=False, label="UInt32"), "data": SimTypePointer(SimTypeChar(label="Byte"), offset=0)}, name="FWP_BYTE_BLOB", pack=False, align=None), "certHash": SimStruct({"size": SimTypeInt(signed=False, label="UInt32"), "data": SimTypePointer(SimTypeChar(label="Byte"), offset=0)}, name="FWP_BYTE_BLOB", pack=False, align=None), "eku": SimTypePointer(SimStruct({"numEku": SimTypeInt(signed=False, label="UInt32"), "eku": SimTypePointer(SimTypePointer(SimTypeChar(label="Byte"), offset=0), offset=0)}, name="IKEEXT_CERT_EKUS0", pack=False, align=None), offset=0), "name": SimTypePointer(SimStruct({"nameType": SimTypeInt(signed=False, label="IKEEXT_CERT_CRITERIA_NAME_TYPE"), "certName": SimTypePointer(SimTypeChar(label="Char"), offset=0)}, name="IKEEXT_CERT_NAME0", pack=False, align=None), offset=0), "flags": SimTypeInt(signed=False, label="UInt32")}, name="IKEEXT_CERTIFICATE_CRITERIA0", pack=False, align=None), offset=0)}, name="_Anonymous2_e__Struct", pack=False, align=None), "Anonymous3": SimStruct({"inboundRootStoreArraySize": SimTypeInt(signed=False, label="UInt32"), "inboundTrustedRootStoreCriteria": SimTypePointer(SimStruct({"certData": SimStruct({"size": SimTypeInt(signed=False, label="UInt32"), "data": SimTypePointer(SimTypeChar(label="Byte"), offset=0)}, name="FWP_BYTE_BLOB", pack=False, align=None), "certHash": SimStruct({"size": SimTypeInt(signed=False, label="UInt32"), "data": SimTypePointer(SimTypeChar(label="Byte"), offset=0)}, name="FWP_BYTE_BLOB", pack=False, align=None), "eku": SimTypePointer(SimStruct({"numEku": SimTypeInt(signed=False, label="UInt32"), "eku": SimTypePointer(SimTypePointer(SimTypeChar(label="Byte"), offset=0), offset=0)}, name="IKEEXT_CERT_EKUS0", pack=False, align=None), offset=0), "name": SimTypePointer(SimStruct({"nameType": SimTypeInt(signed=False, label="IKEEXT_CERT_CRITERIA_NAME_TYPE"), "certName": SimTypePointer(SimTypeChar(label="Char"), offset=0)}, name="IKEEXT_CERT_NAME0", pack=False, align=None), offset=0), "flags": SimTypeInt(signed=False, label="UInt32")}, name="IKEEXT_CERTIFICATE_CRITERIA0", pack=False, align=None), offset=0)}, name="_Anonymous3_e__Struct", pack=False, align=None)}, name="<anon>", label="None"), "outboundConfigType": SimTypeInt(signed=False, label="IKEEXT_CERT_CONFIG_TYPE"), "Anonymous2": SimUnion({"Anonymous1": SimStruct({"outboundRootArraySize": SimTypeInt(signed=False, label="UInt32"), "outboundRootCriteria": SimTypePointer(SimStruct({"certData": SimStruct({"size": SimTypeInt(signed=False, label="UInt32"), "data": SimTypePointer(SimTypeChar(label="Byte"), offset=0)}, name="FWP_BYTE_BLOB", pack=False, align=None), "certHash": SimStruct({"size": SimTypeInt(signed=False, label="UInt32"), "data": SimTypePointer(SimTypeChar(label="Byte"), offset=0)}, name="FWP_BYTE_BLOB", pack=False, align=None), "eku": SimTypePointer(SimStruct({"numEku": SimTypeInt(signed=False, label="UInt32"), "eku": SimTypePointer(SimTypePointer(SimTypeChar(label="Byte"), offset=0), offset=0)}, name="IKEEXT_CERT_EKUS0", pack=False, align=None), offset=0), "name": SimTypePointer(SimStruct({"nameType": SimTypeInt(signed=False, label="IKEEXT_CERT_CRITERIA_NAME_TYPE"), "certName": SimTypePointer(SimTypeChar(label="Char"), offset=0)}, name="IKEEXT_CERT_NAME0", pack=False, align=None), offset=0), "flags": SimTypeInt(signed=False, label="UInt32")}, name="IKEEXT_CERTIFICATE_CRITERIA0", pack=False, align=None), offset=0)}, name="_Anonymous1_e__Struct", pack=False, align=None), "Anonymous2": SimStruct({"outboundEnterpriseStoreArraySize": SimTypeInt(signed=False, label="UInt32"), "outboundEnterpriseStoreCriteria": SimTypePointer(SimStruct({"certData": SimStruct({"size": SimTypeInt(signed=False, label="UInt32"), "data": SimTypePointer(SimTypeChar(label="Byte"), offset=0)}, name="FWP_BYTE_BLOB", pack=False, align=None), "certHash": SimStruct({"size": SimTypeInt(signed=False, label="UInt32"), "data": SimTypePointer(SimTypeChar(label="Byte"), offset=0)}, name="FWP_BYTE_BLOB", pack=False, align=None), "eku": SimTypePointer(SimStruct({"numEku": SimTypeInt(signed=False, label="UInt32"), "eku": SimTypePointer(SimTypePointer(SimTypeChar(label="Byte"), offset=0), offset=0)}, name="IKEEXT_CERT_EKUS0", pack=False, align=None), offset=0), "name": SimTypePointer(SimStruct({"nameType": SimTypeInt(signed=False, label="IKEEXT_CERT_CRITERIA_NAME_TYPE"), "certName": SimTypePointer(SimTypeChar(label="Char"), offset=0)}, name="IKEEXT_CERT_NAME0", pack=False, align=None), offset=0), "flags": SimTypeInt(signed=False, label="UInt32")}, name="IKEEXT_CERTIFICATE_CRITERIA0", pack=False, align=None), offset=0)}, name="_Anonymous2_e__Struct", pack=False, align=None), "Anonymous3": SimStruct({"outboundRootStoreArraySize": SimTypeInt(signed=False, label="UInt32"), "outboundTrustedRootStoreCriteria": SimTypePointer(SimStruct({"certData": SimStruct({"size": SimTypeInt(signed=False, label="UInt32"), "data": SimTypePointer(SimTypeChar(label="Byte"), offset=0)}, name="FWP_BYTE_BLOB", pack=False, align=None), "certHash": SimStruct({"size": SimTypeInt(signed=False, label="UInt32"), "data": SimTypePointer(SimTypeChar(label="Byte"), offset=0)}, name="FWP_BYTE_BLOB", pack=False, align=None), "eku": SimTypePointer(SimStruct({"numEku": SimTypeInt(signed=False, label="UInt32"), "eku": SimTypePointer(SimTypePointer(SimTypeChar(label="Byte"), offset=0), offset=0)}, name="IKEEXT_CERT_EKUS0", pack=False, align=None), offset=0), "name": SimTypePointer(SimStruct({"nameType": SimTypeInt(signed=False, label="IKEEXT_CERT_CRITERIA_NAME_TYPE"), "certName": SimTypePointer(SimTypeChar(label="Char"), offset=0)}, name="IKEEXT_CERT_NAME0", pack=False, align=None), offset=0), "flags": SimTypeInt(signed=False, label="UInt32")}, name="IKEEXT_CERTIFICATE_CRITERIA0", pack=False, align=None), offset=0)}, name="_Anonymous3_e__Struct", pack=False, align=None)}, name="<anon>", label="None"), "flags": SimTypeInt(signed=False, label="IKEEXT_CERT_AUTH"), "localCertLocationUrl": SimStruct({"size": SimTypeInt(signed=False, label="UInt32"), "data": SimTypePointer(SimTypeChar(label="Byte"), offset=0)}, name="FWP_BYTE_BLOB", pack=False, align=None)}, name="IKEEXT_CERTIFICATE_AUTHENTICATION2", pack=False, align=None), "kerberosAuthentication": SimStruct({"flags": SimTypeInt(signed=False, label="IKEEXT_KERBEROS_AUTHENTICATION_FLAGS"), "proxyServer": SimTypePointer(SimTypeChar(label="Char"), offset=0)}, name="IKEEXT_KERBEROS_AUTHENTICATION1", pack=False, align=None), "reservedAuthentication": SimStruct({"flags": SimTypeInt(signed=False, label="IKEEXT_RESERVED_AUTHENTICATION_FLAGS")}, name="IKEEXT_RESERVED_AUTHENTICATION0", pack=False, align=None), "ntlmV2Authentication": SimStruct({"flags": SimTypeInt(signed=False, label="UInt32")}, name="IKEEXT_NTLM_V2_AUTHENTICATION0", pack=False, align=None), "sslAuthentication": SimStruct({"inboundConfigType": SimTypeInt(signed=False, label="IKEEXT_CERT_CONFIG_TYPE"), "Anonymous1": SimUnion({"Anonymous1": SimStruct({"inboundRootArraySize": SimTypeInt(signed=False, label="UInt32"), "inboundRootCriteria": SimTypePointer(SimStruct({"certData": SimStruct({"size": SimTypeInt(signed=False, label="UInt32"), "data": SimTypePointer(SimTypeChar(label="Byte"), offset=0)}, name="FWP_BYTE_BLOB", pack=False, align=None), "certHash": SimStruct({"size": SimTypeInt(signed=False, label="UInt32"), "data": SimTypePointer(SimTypeChar(label="Byte"), offset=0)}, name="FWP_BYTE_BLOB", pack=False, align=None), "eku": SimTypePointer(SimStruct({"numEku": SimTypeInt(signed=False, label="UInt32"), "eku": SimTypePointer(SimTypePointer(SimTypeChar(label="Byte"), offset=0), offset=0)}, name="IKEEXT_CERT_EKUS0", pack=False, align=None), offset=0), "name": SimTypePointer(SimStruct({"nameType": SimTypeInt(signed=False, label="IKEEXT_CERT_CRITERIA_NAME_TYPE"), "certName": SimTypePointer(SimTypeChar(label="Char"), offset=0)}, name="IKEEXT_CERT_NAME0", pack=False, align=None), offset=0), "flags": SimTypeInt(signed=False, label="UInt32")}, name="IKEEXT_CERTIFICATE_CRITERIA0", pack=False, align=None), offset=0)}, name="_Anonymous1_e__Struct", pack=False, align=None), "Anonymous2": SimStruct({"inboundEnterpriseStoreArraySize": SimTypeInt(signed=False, label="UInt32"), "inboundEnterpriseStoreCriteria": SimTypePointer(SimStruct({"certData": SimStruct({"size": SimTypeInt(signed=False, label="UInt32"), "data": SimTypePointer(SimTypeChar(label="Byte"), offset=0)}, name="FWP_BYTE_BLOB", pack=False, align=None), "certHash": SimStruct({"size": SimTypeInt(signed=False, label="UInt32"), "data": SimTypePointer(SimTypeChar(label="Byte"), offset=0)}, name="FWP_BYTE_BLOB", pack=False, align=None), "eku": SimTypePointer(SimStruct({"numEku": SimTypeInt(signed=False, label="UInt32"), "eku": SimTypePointer(SimTypePointer(SimTypeChar(label="Byte"), offset=0), offset=0)}, name="IKEEXT_CERT_EKUS0", pack=False, align=None), offset=0), "name": SimTypePointer(SimStruct({"nameType": SimTypeInt(signed=False, label="IKEEXT_CERT_CRITERIA_NAME_TYPE"), "certName": SimTypePointer(SimTypeChar(label="Char"), offset=0)}, name="IKEEXT_CERT_NAME0", pack=False, align=None), offset=0), "flags": SimTypeInt(signed=False, label="UInt32")}, name="IKEEXT_CERTIFICATE_CRITERIA0", pack=False, align=None), offset=0)}, name="_Anonymous2_e__Struct", pack=False, align=None), "Anonymous3": SimStruct({"inboundRootStoreArraySize": SimTypeInt(signed=False, label="UInt32"), "inboundTrustedRootStoreCriteria": SimTypePointer(SimStruct({"certData": SimStruct({"size": SimTypeInt(signed=False, label="UInt32"), "data": SimTypePointer(SimTypeChar(label="Byte"), offset=0)}, name="FWP_BYTE_BLOB", pack=False, align=None), "certHash": SimStruct({"size": SimTypeInt(signed=False, label="UInt32"), "data": SimTypePointer(SimTypeChar(label="Byte"), offset=0)}, name="FWP_BYTE_BLOB", pack=False, align=None), "eku": SimTypePointer(SimStruct({"numEku": SimTypeInt(signed=False, label="UInt32"), "eku": SimTypePointer(SimTypePointer(SimTypeChar(label="Byte"), offset=0), offset=0)}, name="IKEEXT_CERT_EKUS0", pack=False, align=None), offset=0), "name": SimTypePointer(SimStruct({"nameType": SimTypeInt(signed=False, label="IKEEXT_CERT_CRITERIA_NAME_TYPE"), "certName": SimTypePointer(SimTypeChar(label="Char"), offset=0)}, name="IKEEXT_CERT_NAME0", pack=False, align=None), offset=0), "flags": SimTypeInt(signed=False, label="UInt32")}, name="IKEEXT_CERTIFICATE_CRITERIA0", pack=False, align=None), offset=0)}, name="_Anonymous3_e__Struct", pack=False, align=None)}, name="<anon>", label="None"), "outboundConfigType": SimTypeInt(signed=False, label="IKEEXT_CERT_CONFIG_TYPE"), "Anonymous2": SimUnion({"Anonymous1": SimStruct({"outboundRootArraySize": SimTypeInt(signed=False, label="UInt32"), "outboundRootCriteria": SimTypePointer(SimStruct({"certData": SimStruct({"size": SimTypeInt(signed=False, label="UInt32"), "data": SimTypePointer(SimTypeChar(label="Byte"), offset=0)}, name="FWP_BYTE_BLOB", pack=False, align=None), "certHash": SimStruct({"size": SimTypeInt(signed=False, label="UInt32"), "data": SimTypePointer(SimTypeChar(label="Byte"), offset=0)}, name="FWP_BYTE_BLOB", pack=False, align=None), "eku": SimTypePointer(SimStruct({"numEku": SimTypeInt(signed=False, label="UInt32"), "eku": SimTypePointer(SimTypePointer(SimTypeChar(label="Byte"), offset=0), offset=0)}, name="IKEEXT_CERT_EKUS0", pack=False, align=None), offset=0), "name": SimTypePointer(SimStruct({"nameType": SimTypeInt(signed=False, label="IKEEXT_CERT_CRITERIA_NAME_TYPE"), "certName": SimTypePointer(SimTypeChar(label="Char"), offset=0)}, name="IKEEXT_CERT_NAME0", pack=False, align=None), offset=0), "flags": SimTypeInt(signed=False, label="UInt32")}, name="IKEEXT_CERTIFICATE_CRITERIA0", pack=False, align=None), offset=0)}, name="_Anonymous1_e__Struct", pack=False, align=None), "Anonymous2": SimStruct({"outboundEnterpriseStoreArraySize": SimTypeInt(signed=False, label="UInt32"), "outboundEnterpriseStoreCriteria": SimTypePointer(SimStruct({"certData": SimStruct({"size": SimTypeInt(signed=False, label="UInt32"), "data": SimTypePointer(SimTypeChar(label="Byte"), offset=0)}, name="FWP_BYTE_BLOB", pack=False, align=None), "certHash": SimStruct({"size": SimTypeInt(signed=False, label="UInt32"), "data": SimTypePointer(SimTypeChar(label="Byte"), offset=0)}, name="FWP_BYTE_BLOB", pack=False, align=None), "eku": SimTypePointer(SimStruct({"numEku": SimTypeInt(signed=False, label="UInt32"), "eku": SimTypePointer(SimTypePointer(SimTypeChar(label="Byte"), offset=0), offset=0)}, name="IKEEXT_CERT_EKUS0", pack=False, align=None), offset=0), "name": SimTypePointer(SimStruct({"nameType": SimTypeInt(signed=False, label="IKEEXT_CERT_CRITERIA_NAME_TYPE"), "certName": SimTypePointer(SimTypeChar(label="Char"), offset=0)}, name="IKEEXT_CERT_NAME0", pack=False, align=None), offset=0), "flags": SimTypeInt(signed=False, label="UInt32")}, name="IKEEXT_CERTIFICATE_CRITERIA0", pack=False, align=None), offset=0)}, name="_Anonymous2_e__Struct", pack=False, align=None), "Anonymous3": SimStruct({"outboundRootStoreArraySize": SimTypeInt(signed=False, label="UInt32"), "outboundTrustedRootStoreCriteria": SimTypePointer(SimStruct({"certData": SimStruct({"size": SimTypeInt(signed=False, label="UInt32"), "data": SimTypePointer(SimTypeChar(label="Byte"), offset=0)}, name="FWP_BYTE_BLOB", pack=False, align=None), "certHash": SimStruct({"size": SimTypeInt(signed=False, label="UInt32"), "data": SimTypePointer(SimTypeChar(label="Byte"), offset=0)}, name="FWP_BYTE_BLOB", pack=False, align=None), "eku": SimTypePointer(SimStruct({"numEku": SimTypeInt(signed=False, label="UInt32"), "eku": SimTypePointer(SimTypePointer(SimTypeChar(label="Byte"), offset=0), offset=0)}, name="IKEEXT_CERT_EKUS0", pack=False, align=None), offset=0), "name": SimTypePointer(SimStruct({"nameType": SimTypeInt(signed=False, label="IKEEXT_CERT_CRITERIA_NAME_TYPE"), "certName": SimTypePointer(SimTypeChar(label="Char"), offset=0)}, name="IKEEXT_CERT_NAME0", pack=False, align=None), offset=0), "flags": SimTypeInt(signed=False, label="UInt32")}, name="IKEEXT_CERTIFICATE_CRITERIA0", pack=False, align=None), offset=0)}, name="_Anonymous3_e__Struct", pack=False, align=None)}, name="<anon>", label="None"), "flags": SimTypeInt(signed=False, label="IKEEXT_CERT_AUTH"), "localCertLocationUrl": SimStruct({"size": SimTypeInt(signed=False, label="UInt32"), "data": SimTypePointer(SimTypeChar(label="Byte"), offset=0)}, name="FWP_BYTE_BLOB", pack=False, align=None)}, name="IKEEXT_CERTIFICATE_AUTHENTICATION2", pack=False, align=None), "cgaAuthentication": SimStruct({"keyContainerName": SimTypePointer(SimTypeChar(label="Char"), offset=0), "cspName": SimTypePointer(SimTypeChar(label="Char"), offset=0), "cspType": SimTypeInt(signed=False, label="UInt32"), "cgaModifier": SimStruct({"byteArray16": SimTypeFixedSizeArray(SimTypeChar(label="Byte"), 16)}, name="FWP_BYTE_ARRAY16", pack=False, align=None), "cgaCollisionCount": SimTypeChar(label="Byte")}, name="IKEEXT_IPV6_CGA_AUTHENTICATION0", pack=False, align=None), "eapAuthentication": SimStruct({"flags": SimTypeInt(signed=False, label="IKEEXT_EAP_AUTHENTICATION_FLAGS")}, name="IKEEXT_EAP_AUTHENTICATION0", pack=False, align=None)}, name="<anon>", label="None")}, name="IKEEXT_AUTHENTICATION_METHOD2", pack=False, align=None), offset=0), "initiatorImpersonationType": SimTypeInt(signed=False, label="IKEEXT_AUTHENTICATION_IMPERSONATION_TYPE")}, name="IKEEXT_EM_POLICY2", pack=False, align=None), offset=0), "fwdPathSaLifetime": SimTypeInt(signed=False, label="UInt32"), "compartmentId": SimTypeInt(signed=False, label="UInt32"), "numTrafficSelectorPolicy": SimTypeInt(signed=False, label="UInt32"), "trafficSelectorPolicies": SimTypePointer(SimStruct({"flags": SimTypeInt(signed=False, label="UInt32"), "numLocalTrafficSelectors": SimTypeInt(signed=False, label="UInt32"), "localTrafficSelectors": SimTypePointer(SimStruct({"protocolId": SimTypeChar(label="Byte"), "portStart": SimTypeShort(signed=False, label="UInt16"), "portEnd": SimTypeShort(signed=False, label="UInt16"), "ipVersion": SimTypeInt(signed=False, label="FWP_IP_VERSION"), "Anonymous1": SimUnion({"startV4Address": SimTypeInt(signed=False, label="UInt32"), "startV6Address": SimTypeFixedSizeArray(SimTypeChar(label="Byte"), 16)}, name="<anon>", label="None"), "Anonymous2": SimUnion({"endV4Address": SimTypeInt(signed=False, label="UInt32"), "endV6Address": SimTypeFixedSizeArray(SimTypeChar(label="Byte"), 16)}, name="<anon>", label="None")}, name="IPSEC_TRAFFIC_SELECTOR0_", pack=False, align=None), offset=0), "numRemoteTrafficSelectors": SimTypeInt(signed=False, label="UInt32"), "remoteTrafficSelectors": SimTypePointer(SimStruct({"protocolId": SimTypeChar(label="Byte"), "portStart": SimTypeShort(signed=False, label="UInt16"), "portEnd": SimTypeShort(signed=False, label="UInt16"), "ipVersion": SimTypeInt(signed=False, label="FWP_IP_VERSION"), "Anonymous1": SimUnion({"startV4Address": SimTypeInt(signed=False, label="UInt32"), "startV6Address": SimTypeFixedSizeArray(SimTypeChar(label="Byte"), 16)}, name="<anon>", label="None"), "Anonymous2": SimUnion({"endV4Address": SimTypeInt(signed=False, label="UInt32"), "endV6Address": SimTypeFixedSizeArray(SimTypeChar(label="Byte"), 16)}, name="<anon>", label="None")}, name="IPSEC_TRAFFIC_SELECTOR0_", pack=False, align=None), offset=0)}, name="IPSEC_TRAFFIC_SELECTOR_POLICY0_", pack=False, align=None), offset=0)}, name="IPSEC_TUNNEL_POLICY3_", pack=False, align=None), offset=0), "authipQmTransportPolicy": SimTypePointer(SimStruct({"numIpsecProposals": SimTypeInt(signed=False, label="UInt32"), "ipsecProposals": SimTypePointer(SimStruct({"lifetime": SimStruct({"lifetimeSeconds": SimTypeInt(signed=False, label="UInt32"), "lifetimeKilobytes": SimTypeInt(signed=False, label="UInt32"), "lifetimePackets": SimTypeInt(signed=False, label="UInt32")}, name="IPSEC_SA_LIFETIME0", pack=False, align=None), "numSaTransforms": SimTypeInt(signed=False, label="UInt32"), "saTransforms": SimTypePointer(SimStruct({"ipsecTransformType": SimTypeInt(signed=False, label="IPSEC_TRANSFORM_TYPE"), "Anonymous": SimUnion({"ahTransform": SimTypePointer(SimStruct({"authTransformId": SimStruct({"authType": SimTypeInt(signed=False, label="IPSEC_AUTH_TYPE"), "authConfig": SimTypeChar(label="Byte")}, name="IPSEC_AUTH_TRANSFORM_ID0", pack=False, align=None), "cryptoModuleId": SimTypePointer(SimTypeBottom(label="Guid"), offset=0)}, name="IPSEC_AUTH_TRANSFORM0", pack=False, align=None), offset=0), "espAuthTransform": SimTypePointer(SimStruct({"authTransformId": SimStruct({"authType": SimTypeInt(signed=False, label="IPSEC_AUTH_TYPE"), "authConfig": SimTypeChar(label="Byte")}, name="IPSEC_AUTH_TRANSFORM_ID0", pack=False, align=None), "cryptoModuleId": SimTypePointer(SimTypeBottom(label="Guid"), offset=0)}, name="IPSEC_AUTH_TRANSFORM0", pack=False, align=None), offset=0), "espCipherTransform": SimTypePointer(SimStruct({"cipherTransformId": SimStruct({"cipherType": SimTypeInt(signed=False, label="IPSEC_CIPHER_TYPE"), "cipherConfig": SimTypeChar(label="Byte")}, name="IPSEC_CIPHER_TRANSFORM_ID0", pack=False, align=None), "cryptoModuleId": SimTypePointer(SimTypeBottom(label="Guid"), offset=0)}, name="IPSEC_CIPHER_TRANSFORM0", pack=False, align=None), offset=0), "espAuthAndCipherTransform": SimTypePointer(SimStruct({"authTransform": SimStruct({"authTransformId": SimStruct({"authType": SimTypeInt(signed=False, label="IPSEC_AUTH_TYPE"), "authConfig": SimTypeChar(label="Byte")}, name="IPSEC_AUTH_TRANSFORM_ID0", pack=False, align=None), "cryptoModuleId": SimTypePointer(SimTypeBottom(label="Guid"), offset=0)}, name="IPSEC_AUTH_TRANSFORM0", pack=False, align=None), "cipherTransform": SimStruct({"cipherTransformId": SimStruct({"cipherType": SimTypeInt(signed=False, label="IPSEC_CIPHER_TYPE"), "cipherConfig": SimTypeChar(label="Byte")}, name="IPSEC_CIPHER_TRANSFORM_ID0", pack=False, align=None), "cryptoModuleId": SimTypePointer(SimTypeBottom(label="Guid"), offset=0)}, name="IPSEC_CIPHER_TRANSFORM0", pack=False, align=None)}, name="IPSEC_AUTH_AND_CIPHER_TRANSFORM0", pack=False, align=None), offset=0), "espAuthFwTransform": SimTypePointer(SimStruct({"authTransformId": SimStruct({"authType": SimTypeInt(signed=False, label="IPSEC_AUTH_TYPE"), "authConfig": SimTypeChar(label="Byte")}, name="IPSEC_AUTH_TRANSFORM_ID0", pack=False, align=None), "cryptoModuleId": SimTypePointer(SimTypeBottom(label="Guid"), offset=0)}, name="IPSEC_AUTH_TRANSFORM0", pack=False, align=None), offset=0)}, name="<anon>", label="None")}, name="IPSEC_SA_TRANSFORM0", pack=False, align=None), offset=0), "pfsGroup": SimTypeInt(signed=False, label="IPSEC_PFS_GROUP")}, name="IPSEC_PROPOSAL0", pack=False, align=None), offset=0), "flags": SimTypeInt(signed=False, label="IPSEC_POLICY_FLAG"), "ndAllowClearTimeoutSeconds": SimTypeInt(signed=False, label="UInt32"), "saIdleTimeout": SimStruct({"idleTimeoutSeconds": SimTypeInt(signed=False, label="UInt32"), "idleTimeoutSecondsFailOver": SimTypeInt(signed=False, label="UInt32")}, name="IPSEC_SA_IDLE_TIMEOUT0", pack=False, align=None), "emPolicy": SimTypePointer(SimStruct({"numAuthenticationMethods": SimTypeInt(signed=False, label="UInt32"), "authenticationMethods": SimTypePointer(SimStruct({"authenticationMethodType": SimTypeInt(signed=False, label="IKEEXT_AUTHENTICATION_METHOD_TYPE"), "Anonymous": SimUnion({"presharedKeyAuthentication": SimStruct({"presharedKey": SimStruct({"size": SimTypeInt(signed=False, label="UInt32"), "data": SimTypePointer(SimTypeChar(label="Byte"), offset=0)}, name="FWP_BYTE_BLOB", pack=False, align=None), "flags": SimTypeInt(signed=False, label="IKEEXT_PRESHARED_KEY_AUTHENTICATION_FLAGS")}, name="IKEEXT_PRESHARED_KEY_AUTHENTICATION1", pack=False, align=None), "certificateAuthentication": SimStruct({"inboundConfigType": SimTypeInt(signed=False, label="IKEEXT_CERT_CONFIG_TYPE"), "Anonymous1": SimUnion({"Anonymous1": SimStruct({"inboundRootArraySize": SimTypeInt(signed=False, label="UInt32"), "inboundRootCriteria": SimTypePointer(SimStruct({"certData": SimStruct({"size": SimTypeInt(signed=False, label="UInt32"), "data": SimTypePointer(SimTypeChar(label="Byte"), offset=0)}, name="FWP_BYTE_BLOB", pack=False, align=None), "certHash": SimStruct({"size": SimTypeInt(signed=False, label="UInt32"), "data": SimTypePointer(SimTypeChar(label="Byte"), offset=0)}, name="FWP_BYTE_BLOB", pack=False, align=None), "eku": SimTypePointer(SimStruct({"numEku": SimTypeInt(signed=False, label="UInt32"), "eku": SimTypePointer(SimTypePointer(SimTypeChar(label="Byte"), offset=0), offset=0)}, name="IKEEXT_CERT_EKUS0", pack=False, align=None), offset=0), "name": SimTypePointer(SimStruct({"nameType": SimTypeInt(signed=False, label="IKEEXT_CERT_CRITERIA_NAME_TYPE"), "certName": SimTypePointer(SimTypeChar(label="Char"), offset=0)}, name="IKEEXT_CERT_NAME0", pack=False, align=None), offset=0), "flags": SimTypeInt(signed=False, label="UInt32")}, name="IKEEXT_CERTIFICATE_CRITERIA0", pack=False, align=None), offset=0)}, name="_Anonymous1_e__Struct", pack=False, align=None), "Anonymous2": SimStruct({"inboundEnterpriseStoreArraySize": SimTypeInt(signed=False, label="UInt32"), "inboundEnterpriseStoreCriteria": SimTypePointer(SimStruct({"certData": SimStruct({"size": SimTypeInt(signed=False, label="UInt32"), "data": SimTypePointer(SimTypeChar(label="Byte"), offset=0)}, name="FWP_BYTE_BLOB", pack=False, align=None), "certHash": SimStruct({"size": SimTypeInt(signed=False, label="UInt32"), "data": SimTypePointer(SimTypeChar(label="Byte"), offset=0)}, name="FWP_BYTE_BLOB", pack=False, align=None), "eku": SimTypePointer(SimStruct({"numEku": SimTypeInt(signed=False, label="UInt32"), "eku": SimTypePointer(SimTypePointer(SimTypeChar(label="Byte"), offset=0), offset=0)}, name="IKEEXT_CERT_EKUS0", pack=False, align=None), offset=0), "name": SimTypePointer(SimStruct({"nameType": SimTypeInt(signed=False, label="IKEEXT_CERT_CRITERIA_NAME_TYPE"), "certName": SimTypePointer(SimTypeChar(label="Char"), offset=0)}, name="IKEEXT_CERT_NAME0", pack=False, align=None), offset=0), "flags": SimTypeInt(signed=False, label="UInt32")}, name="IKEEXT_CERTIFICATE_CRITERIA0", pack=False, align=None), offset=0)}, name="_Anonymous2_e__Struct", pack=False, align=None), "Anonymous3": SimStruct({"inboundRootStoreArraySize": SimTypeInt(signed=False, label="UInt32"), "inboundTrustedRootStoreCriteria": SimTypePointer(SimStruct({"certData": SimStruct({"size": SimTypeInt(signed=False, label="UInt32"), "data": SimTypePointer(SimTypeChar(label="Byte"), offset=0)}, name="FWP_BYTE_BLOB", pack=False, align=None), "certHash": SimStruct({"size": SimTypeInt(signed=False, label="UInt32"), "data": SimTypePointer(SimTypeChar(label="Byte"), offset=0)}, name="FWP_BYTE_BLOB", pack=False, align=None), "eku": SimTypePointer(SimStruct({"numEku": SimTypeInt(signed=False, label="UInt32"), "eku": SimTypePointer(SimTypePointer(SimTypeChar(label="Byte"), offset=0), offset=0)}, name="IKEEXT_CERT_EKUS0", pack=False, align=None), offset=0), "name": SimTypePointer(SimStruct({"nameType": SimTypeInt(signed=False, label="IKEEXT_CERT_CRITERIA_NAME_TYPE"), "certName": SimTypePointer(SimTypeChar(label="Char"), offset=0)}, name="IKEEXT_CERT_NAME0", pack=False, align=None), offset=0), "flags": SimTypeInt(signed=False, label="UInt32")}, name="IKEEXT_CERTIFICATE_CRITERIA0", pack=False, align=None), offset=0)}, name="_Anonymous3_e__Struct", pack=False, align=None)}, name="<anon>", label="None"), "outboundConfigType": SimTypeInt(signed=False, label="IKEEXT_CERT_CONFIG_TYPE"), "Anonymous2": SimUnion({"Anonymous1": SimStruct({"outboundRootArraySize": SimTypeInt(signed=False, label="UInt32"), "outboundRootCriteria": SimTypePointer(SimStruct({"certData": SimStruct({"size": SimTypeInt(signed=False, label="UInt32"), "data": SimTypePointer(SimTypeChar(label="Byte"), offset=0)}, name="FWP_BYTE_BLOB", pack=False, align=None), "certHash": SimStruct({"size": SimTypeInt(signed=False, label="UInt32"), "data": SimTypePointer(SimTypeChar(label="Byte"), offset=0)}, name="FWP_BYTE_BLOB", pack=False, align=None), "eku": SimTypePointer(SimStruct({"numEku": SimTypeInt(signed=False, label="UInt32"), "eku": SimTypePointer(SimTypePointer(SimTypeChar(label="Byte"), offset=0), offset=0)}, name="IKEEXT_CERT_EKUS0", pack=False, align=None), offset=0), "name": SimTypePointer(SimStruct({"nameType": SimTypeInt(signed=False, label="IKEEXT_CERT_CRITERIA_NAME_TYPE"), "certName": SimTypePointer(SimTypeChar(label="Char"), offset=0)}, name="IKEEXT_CERT_NAME0", pack=False, align=None), offset=0), "flags": SimTypeInt(signed=False, label="UInt32")}, name="IKEEXT_CERTIFICATE_CRITERIA0", pack=False, align=None), offset=0)}, name="_Anonymous1_e__Struct", pack=False, align=None), "Anonymous2": SimStruct({"outboundEnterpriseStoreArraySize": SimTypeInt(signed=False, label="UInt32"), "outboundEnterpriseStoreCriteria": SimTypePointer(SimStruct({"certData": SimStruct({"size": SimTypeInt(signed=False, label="UInt32"), "data": SimTypePointer(SimTypeChar(label="Byte"), offset=0)}, name="FWP_BYTE_BLOB", pack=False, align=None), "certHash": SimStruct({"size": SimTypeInt(signed=False, label="UInt32"), "data": SimTypePointer(SimTypeChar(label="Byte"), offset=0)}, name="FWP_BYTE_BLOB", pack=False, align=None), "eku": SimTypePointer(SimStruct({"numEku": SimTypeInt(signed=False, label="UInt32"), "eku": SimTypePointer(SimTypePointer(SimTypeChar(label="Byte"), offset=0), offset=0)}, name="IKEEXT_CERT_EKUS0", pack=False, align=None), offset=0), "name": SimTypePointer(SimStruct({"nameType": SimTypeInt(signed=False, label="IKEEXT_CERT_CRITERIA_NAME_TYPE"), "certName": SimTypePointer(SimTypeChar(label="Char"), offset=0)}, name="IKEEXT_CERT_NAME0", pack=False, align=None), offset=0), "flags": SimTypeInt(signed=False, label="UInt32")}, name="IKEEXT_CERTIFICATE_CRITERIA0", pack=False, align=None), offset=0)}, name="_Anonymous2_e__Struct", pack=False, align=None), "Anonymous3": SimStruct({"outboundRootStoreArraySize": SimTypeInt(signed=False, label="UInt32"), "outboundTrustedRootStoreCriteria": SimTypePointer(SimStruct({"certData": SimStruct({"size": SimTypeInt(signed=False, label="UInt32"), "data": SimTypePointer(SimTypeChar(label="Byte"), offset=0)}, name="FWP_BYTE_BLOB", pack=False, align=None), "certHash": SimStruct({"size": SimTypeInt(signed=False, label="UInt32"), "data": SimTypePointer(SimTypeChar(label="Byte"), offset=0)}, name="FWP_BYTE_BLOB", pack=False, align=None), "eku": SimTypePointer(SimStruct({"numEku": SimTypeInt(signed=False, label="UInt32"), "eku": SimTypePointer(SimTypePointer(SimTypeChar(label="Byte"), offset=0), offset=0)}, name="IKEEXT_CERT_EKUS0", pack=False, align=None), offset=0), "name": SimTypePointer(SimStruct({"nameType": SimTypeInt(signed=False, label="IKEEXT_CERT_CRITERIA_NAME_TYPE"), "certName": SimTypePointer(SimTypeChar(label="Char"), offset=0)}, name="IKEEXT_CERT_NAME0", pack=False, align=None), offset=0), "flags": SimTypeInt(signed=False, label="UInt32")}, name="IKEEXT_CERTIFICATE_CRITERIA0", pack=False, align=None), offset=0)}, name="_Anonymous3_e__Struct", pack=False, align=None)}, name="<anon>", label="None"), "flags": SimTypeInt(signed=False, label="IKEEXT_CERT_AUTH"), "localCertLocationUrl": SimStruct({"size": SimTypeInt(signed=False, label="UInt32"), "data": SimTypePointer(SimTypeChar(label="Byte"), offset=0)}, name="FWP_BYTE_BLOB", pack=False, align=None)}, name="IKEEXT_CERTIFICATE_AUTHENTICATION2", pack=False, align=None), "kerberosAuthentication": SimStruct({"flags": SimTypeInt(signed=False, label="IKEEXT_KERBEROS_AUTHENTICATION_FLAGS"), "proxyServer": SimTypePointer(SimTypeChar(label="Char"), offset=0)}, name="IKEEXT_KERBEROS_AUTHENTICATION1", pack=False, align=None), "reservedAuthentication": SimStruct({"flags": SimTypeInt(signed=False, label="IKEEXT_RESERVED_AUTHENTICATION_FLAGS")}, name="IKEEXT_RESERVED_AUTHENTICATION0", pack=False, align=None), "ntlmV2Authentication": SimStruct({"flags": SimTypeInt(signed=False, label="UInt32")}, name="IKEEXT_NTLM_V2_AUTHENTICATION0", pack=False, align=None), "sslAuthentication": SimStruct({"inboundConfigType": SimTypeInt(signed=False, label="IKEEXT_CERT_CONFIG_TYPE"), "Anonymous1": SimUnion({"Anonymous1": SimStruct({"inboundRootArraySize": SimTypeInt(signed=False, label="UInt32"), "inboundRootCriteria": SimTypePointer(SimStruct({"certData": SimStruct({"size": SimTypeInt(signed=False, label="UInt32"), "data": SimTypePointer(SimTypeChar(label="Byte"), offset=0)}, name="FWP_BYTE_BLOB", pack=False, align=None), "certHash": SimStruct({"size": SimTypeInt(signed=False, label="UInt32"), "data": SimTypePointer(SimTypeChar(label="Byte"), offset=0)}, name="FWP_BYTE_BLOB", pack=False, align=None), "eku": SimTypePointer(SimStruct({"numEku": SimTypeInt(signed=False, label="UInt32"), "eku": SimTypePointer(SimTypePointer(SimTypeChar(label="Byte"), offset=0), offset=0)}, name="IKEEXT_CERT_EKUS0", pack=False, align=None), offset=0), "name": SimTypePointer(SimStruct({"nameType": SimTypeInt(signed=False, label="IKEEXT_CERT_CRITERIA_NAME_TYPE"), "certName": SimTypePointer(SimTypeChar(label="Char"), offset=0)}, name="IKEEXT_CERT_NAME0", pack=False, align=None), offset=0), "flags": SimTypeInt(signed=False, label="UInt32")}, name="IKEEXT_CERTIFICATE_CRITERIA0", pack=False, align=None), offset=0)}, name="_Anonymous1_e__Struct", pack=False, align=None), "Anonymous2": SimStruct({"inboundEnterpriseStoreArraySize": SimTypeInt(signed=False, label="UInt32"), "inboundEnterpriseStoreCriteria": SimTypePointer(SimStruct({"certData": SimStruct({"size": SimTypeInt(signed=False, label="UInt32"), "data": SimTypePointer(SimTypeChar(label="Byte"), offset=0)}, name="FWP_BYTE_BLOB", pack=False, align=None), "certHash": SimStruct({"size": SimTypeInt(signed=False, label="UInt32"), "data": SimTypePointer(SimTypeChar(label="Byte"), offset=0)}, name="FWP_BYTE_BLOB", pack=False, align=None), "eku": SimTypePointer(SimStruct({"numEku": SimTypeInt(signed=False, label="UInt32"), "eku": SimTypePointer(SimTypePointer(SimTypeChar(label="Byte"), offset=0), offset=0)}, name="IKEEXT_CERT_EKUS0", pack=False, align=None), offset=0), "name": SimTypePointer(SimStruct({"nameType": SimTypeInt(signed=False, label="IKEEXT_CERT_CRITERIA_NAME_TYPE"), "certName": SimTypePointer(SimTypeChar(label="Char"), offset=0)}, name="IKEEXT_CERT_NAME0", pack=False, align=None), offset=0), "flags": SimTypeInt(signed=False, label="UInt32")}, name="IKEEXT_CERTIFICATE_CRITERIA0", pack=False, align=None), offset=0)}, name="_Anonymous2_e__Struct", pack=False, align=None), "Anonymous3": SimStruct({"inboundRootStoreArraySize": SimTypeInt(signed=False, label="UInt32"), "inboundTrustedRootStoreCriteria": SimTypePointer(SimStruct({"certData": SimStruct({"size": SimTypeInt(signed=False, label="UInt32"), "data": SimTypePointer(SimTypeChar(label="Byte"), offset=0)}, name="FWP_BYTE_BLOB", pack=False, align=None), "certHash": SimStruct({"size": SimTypeInt(signed=False, label="UInt32"), "data": SimTypePointer(SimTypeChar(label="Byte"), offset=0)}, name="FWP_BYTE_BLOB", pack=False, align=None), "eku": SimTypePointer(SimStruct({"numEku": SimTypeInt(signed=False, label="UInt32"), "eku": SimTypePointer(SimTypePointer(SimTypeChar(label="Byte"), offset=0), offset=0)}, name="IKEEXT_CERT_EKUS0", pack=False, align=None), offset=0), "name": SimTypePointer(SimStruct({"nameType": SimTypeInt(signed=False, label="IKEEXT_CERT_CRITERIA_NAME_TYPE"), "certName": SimTypePointer(SimTypeChar(label="Char"), offset=0)}, name="IKEEXT_CERT_NAME0", pack=False, align=None), offset=0), "flags": SimTypeInt(signed=False, label="UInt32")}, name="IKEEXT_CERTIFICATE_CRITERIA0", pack=False, align=None), offset=0)}, name="_Anonymous3_e__Struct", pack=False, align=None)}, name="<anon>", label="None"), "outboundConfigType": SimTypeInt(signed=False, label="IKEEXT_CERT_CONFIG_TYPE"), "Anonymous2": SimUnion({"Anonymous1": SimStruct({"outboundRootArraySize": SimTypeInt(signed=False, label="UInt32"), "outboundRootCriteria": SimTypePointer(SimStruct({"certData": SimStruct({"size": SimTypeInt(signed=False, label="UInt32"), "data": SimTypePointer(SimTypeChar(label="Byte"), offset=0)}, name="FWP_BYTE_BLOB", pack=False, align=None), "certHash": SimStruct({"size": SimTypeInt(signed=False, label="UInt32"), "data": SimTypePointer(SimTypeChar(label="Byte"), offset=0)}, name="FWP_BYTE_BLOB", pack=False, align=None), "eku": SimTypePointer(SimStruct({"numEku": SimTypeInt(signed=False, label="UInt32"), "eku": SimTypePointer(SimTypePointer(SimTypeChar(label="Byte"), offset=0), offset=0)}, name="IKEEXT_CERT_EKUS0", pack=False, align=None), offset=0), "name": SimTypePointer(SimStruct({"nameType": SimTypeInt(signed=False, label="IKEEXT_CERT_CRITERIA_NAME_TYPE"), "certName": SimTypePointer(SimTypeChar(label="Char"), offset=0)}, name="IKEEXT_CERT_NAME0", pack=False, align=None), offset=0), "flags": SimTypeInt(signed=False, label="UInt32")}, name="IKEEXT_CERTIFICATE_CRITERIA0", pack=False, align=None), offset=0)}, name="_Anonymous1_e__Struct", pack=False, align=None), "Anonymous2": SimStruct({"outboundEnterpriseStoreArraySize": SimTypeInt(signed=False, label="UInt32"), "outboundEnterpriseStoreCriteria": SimTypePointer(SimStruct({"certData": SimStruct({"size": SimTypeInt(signed=False, label="UInt32"), "data": SimTypePointer(SimTypeChar(label="Byte"), offset=0)}, name="FWP_BYTE_BLOB", pack=False, align=None), "certHash": SimStruct({"size": SimTypeInt(signed=False, label="UInt32"), "data": SimTypePointer(SimTypeChar(label="Byte"), offset=0)}, name="FWP_BYTE_BLOB", pack=False, align=None), "eku": SimTypePointer(SimStruct({"numEku": SimTypeInt(signed=False, label="UInt32"), "eku": SimTypePointer(SimTypePointer(SimTypeChar(label="Byte"), offset=0), offset=0)}, name="IKEEXT_CERT_EKUS0", pack=False, align=None), offset=0), "name": SimTypePointer(SimStruct({"nameType": SimTypeInt(signed=False, label="IKEEXT_CERT_CRITERIA_NAME_TYPE"), "certName": SimTypePointer(SimTypeChar(label="Char"), offset=0)}, name="IKEEXT_CERT_NAME0", pack=False, align=None), offset=0), "flags": SimTypeInt(signed=False, label="UInt32")}, name="IKEEXT_CERTIFICATE_CRITERIA0", pack=False, align=None), offset=0)}, name="_Anonymous2_e__Struct", pack=False, align=None), "Anonymous3": SimStruct({"outboundRootStoreArraySize": SimTypeInt(signed=False, label="UInt32"), "outboundTrustedRootStoreCriteria": SimTypePointer(SimStruct({"certData": SimStruct({"size": SimTypeInt(signed=False, label="UInt32"), "data": SimTypePointer(SimTypeChar(label="Byte"), offset=0)}, name="FWP_BYTE_BLOB", pack=False, align=None), "certHash": SimStruct({"size": SimTypeInt(signed=False, label="UInt32"), "data": SimTypePointer(SimTypeChar(label="Byte"), offset=0)}, name="FWP_BYTE_BLOB", pack=False, align=None), "eku": SimTypePointer(SimStruct({"numEku": SimTypeInt(signed=False, label="UInt32"), "eku": SimTypePointer(SimTypePointer(SimTypeChar(label="Byte"), offset=0), offset=0)}, name="IKEEXT_CERT_EKUS0", pack=False, align=None), offset=0), "name": SimTypePointer(SimStruct({"nameType": SimTypeInt(signed=False, label="IKEEXT_CERT_CRITERIA_NAME_TYPE"), "certName": SimTypePointer(SimTypeChar(label="Char"), offset=0)}, name="IKEEXT_CERT_NAME0", pack=False, align=None), offset=0), "flags": SimTypeInt(signed=False, label="UInt32")}, name="IKEEXT_CERTIFICATE_CRITERIA0", pack=False, align=None), offset=0)}, name="_Anonymous3_e__Struct", pack=False, align=None)}, name="<anon>", label="None"), "flags": SimTypeInt(signed=False, label="IKEEXT_CERT_AUTH"), "localCertLocationUrl": SimStruct({"size": SimTypeInt(signed=False, label="UInt32"), "data": SimTypePointer(SimTypeChar(label="Byte"), offset=0)}, name="FWP_BYTE_BLOB", pack=False, align=None)}, name="IKEEXT_CERTIFICATE_AUTHENTICATION2", pack=False, align=None), "cgaAuthentication": SimStruct({"keyContainerName": SimTypePointer(SimTypeChar(label="Char"), offset=0), "cspName": SimTypePointer(SimTypeChar(label="Char"), offset=0), "cspType": SimTypeInt(signed=False, label="UInt32"), "cgaModifier": SimStruct({"byteArray16": SimTypeFixedSizeArray(SimTypeChar(label="Byte"), 16)}, name="FWP_BYTE_ARRAY16", pack=False, align=None), "cgaCollisionCount": SimTypeChar(label="Byte")}, name="IKEEXT_IPV6_CGA_AUTHENTICATION0", pack=False, align=None), "eapAuthentication": SimStruct({"flags": SimTypeInt(signed=False, label="IKEEXT_EAP_AUTHENTICATION_FLAGS")}, name="IKEEXT_EAP_AUTHENTICATION0", pack=False, align=None)}, name="<anon>", label="None")}, name="IKEEXT_AUTHENTICATION_METHOD2", pack=False, align=None), offset=0), "initiatorImpersonationType": SimTypeInt(signed=False, label="IKEEXT_AUTHENTICATION_IMPERSONATION_TYPE")}, name="IKEEXT_EM_POLICY2", pack=False, align=None), offset=0)}, name="IPSEC_TRANSPORT_POLICY2", pack=False, align=None), offset=0), "authipQmTunnelPolicy": SimTypePointer(SimStruct({"flags": SimTypeInt(signed=False, label="UInt32"), "numIpsecProposals": SimTypeInt(signed=False, label="UInt32"), "ipsecProposals": SimTypePointer(SimStruct({"lifetime": SimStruct({"lifetimeSeconds": SimTypeInt(signed=False, label="UInt32"), "lifetimeKilobytes": SimTypeInt(signed=False, label="UInt32"), "lifetimePackets": SimTypeInt(signed=False, label="UInt32")}, name="IPSEC_SA_LIFETIME0", pack=False, align=None), "numSaTransforms": SimTypeInt(signed=False, label="UInt32"), "saTransforms": SimTypePointer(SimStruct({"ipsecTransformType": SimTypeInt(signed=False, label="IPSEC_TRANSFORM_TYPE"), "Anonymous": SimUnion({"ahTransform": SimTypePointer(SimStruct({"authTransformId": SimStruct({"authType": SimTypeInt(signed=False, label="IPSEC_AUTH_TYPE"), "authConfig": SimTypeChar(label="Byte")}, name="IPSEC_AUTH_TRANSFORM_ID0", pack=False, align=None), "cryptoModuleId": SimTypePointer(SimTypeBottom(label="Guid"), offset=0)}, name="IPSEC_AUTH_TRANSFORM0", pack=False, align=None), offset=0), "espAuthTransform": SimTypePointer(SimStruct({"authTransformId": SimStruct({"authType": SimTypeInt(signed=False, label="IPSEC_AUTH_TYPE"), "authConfig": SimTypeChar(label="Byte")}, name="IPSEC_AUTH_TRANSFORM_ID0", pack=False, align=None), "cryptoModuleId": SimTypePointer(SimTypeBottom(label="Guid"), offset=0)}, name="IPSEC_AUTH_TRANSFORM0", pack=False, align=None), offset=0), "espCipherTransform": SimTypePointer(SimStruct({"cipherTransformId": SimStruct({"cipherType": SimTypeInt(signed=False, label="IPSEC_CIPHER_TYPE"), "cipherConfig": SimTypeChar(label="Byte")}, name="IPSEC_CIPHER_TRANSFORM_ID0", pack=False, align=None), "cryptoModuleId": SimTypePointer(SimTypeBottom(label="Guid"), offset=0)}, name="IPSEC_CIPHER_TRANSFORM0", pack=False, align=None), offset=0), "espAuthAndCipherTransform": SimTypePointer(SimStruct({"authTransform": SimStruct({"authTransformId": SimStruct({"authType": SimTypeInt(signed=False, label="IPSEC_AUTH_TYPE"), "authConfig": SimTypeChar(label="Byte")}, name="IPSEC_AUTH_TRANSFORM_ID0", pack=False, align=None), "cryptoModuleId": SimTypePointer(SimTypeBottom(label="Guid"), offset=0)}, name="IPSEC_AUTH_TRANSFORM0", pack=False, align=None), "cipherTransform": SimStruct({"cipherTransformId": SimStruct({"cipherType": SimTypeInt(signed=False, label="IPSEC_CIPHER_TYPE"), "cipherConfig": SimTypeChar(label="Byte")}, name="IPSEC_CIPHER_TRANSFORM_ID0", pack=False, align=None), "cryptoModuleId": SimTypePointer(SimTypeBottom(label="Guid"), offset=0)}, name="IPSEC_CIPHER_TRANSFORM0", pack=False, align=None)}, name="IPSEC_AUTH_AND_CIPHER_TRANSFORM0", pack=False, align=None), offset=0), "espAuthFwTransform": SimTypePointer(SimStruct({"authTransformId": SimStruct({"authType": SimTypeInt(signed=False, label="IPSEC_AUTH_TYPE"), "authConfig": SimTypeChar(label="Byte")}, name="IPSEC_AUTH_TRANSFORM_ID0", pack=False, align=None), "cryptoModuleId": SimTypePointer(SimTypeBottom(label="Guid"), offset=0)}, name="IPSEC_AUTH_TRANSFORM0", pack=False, align=None), offset=0)}, name="<anon>", label="None")}, name="IPSEC_SA_TRANSFORM0", pack=False, align=None), offset=0), "pfsGroup": SimTypeInt(signed=False, label="IPSEC_PFS_GROUP")}, name="IPSEC_PROPOSAL0", pack=False, align=None), offset=0), "tunnelEndpoints": SimStruct({"ipVersion": SimTypeInt(signed=False, label="FWP_IP_VERSION"), "Anonymous1": SimUnion({"localV4Address": SimTypeInt(signed=False, label="UInt32"), "localV6Address": SimTypeFixedSizeArray(SimTypeChar(label="Byte"), 16)}, name="<anon>", label="None"), "Anonymous2": SimUnion({"remoteV4Address": SimTypeInt(signed=False, label="UInt32"), "remoteV6Address": SimTypeFixedSizeArray(SimTypeChar(label="Byte"), 16)}, name="<anon>", label="None"), "localIfLuid": SimTypeLongLong(signed=False, label="UInt64"), "remoteFqdn": SimTypePointer(SimTypeChar(label="Char"), offset=0), "numAddresses": SimTypeInt(signed=False, label="UInt32"), "remoteAddresses": SimTypePointer(SimStruct({"ipVersion": SimTypeInt(signed=False, label="FWP_IP_VERSION"), "Anonymous": SimUnion({"v4Address": SimTypeInt(signed=False, label="UInt32"), "v6Address": SimTypeFixedSizeArray(SimTypeChar(label="Byte"), 16)}, name="<anon>", label="None")}, name="IPSEC_TUNNEL_ENDPOINT0", pack=False, align=None), offset=0)}, name="IPSEC_TUNNEL_ENDPOINTS2", pack=False, align=None), "saIdleTimeout": SimStruct({"idleTimeoutSeconds": SimTypeInt(signed=False, label="UInt32"), "idleTimeoutSecondsFailOver": SimTypeInt(signed=False, label="UInt32")}, name="IPSEC_SA_IDLE_TIMEOUT0", pack=False, align=None), "emPolicy": SimTypePointer(SimStruct({"numAuthenticationMethods": SimTypeInt(signed=False, label="UInt32"), "authenticationMethods": SimTypePointer(SimStruct({"authenticationMethodType": SimTypeInt(signed=False, label="IKEEXT_AUTHENTICATION_METHOD_TYPE"), "Anonymous": SimUnion({"presharedKeyAuthentication": SimStruct({"presharedKey": SimStruct({"size": SimTypeInt(signed=False, label="UInt32"), "data": SimTypePointer(SimTypeChar(label="Byte"), offset=0)}, name="FWP_BYTE_BLOB", pack=False, align=None), "flags": SimTypeInt(signed=False, label="IKEEXT_PRESHARED_KEY_AUTHENTICATION_FLAGS")}, name="IKEEXT_PRESHARED_KEY_AUTHENTICATION1", pack=False, align=None), "certificateAuthentication": SimStruct({"inboundConfigType": SimTypeInt(signed=False, label="IKEEXT_CERT_CONFIG_TYPE"), "Anonymous1": SimUnion({"Anonymous1": SimStruct({"inboundRootArraySize": SimTypeInt(signed=False, label="UInt32"), "inboundRootCriteria": SimTypePointer(SimStruct({"certData": SimStruct({"size": SimTypeInt(signed=False, label="UInt32"), "data": SimTypePointer(SimTypeChar(label="Byte"), offset=0)}, name="FWP_BYTE_BLOB", pack=False, align=None), "certHash": SimStruct({"size": SimTypeInt(signed=False, label="UInt32"), "data": SimTypePointer(SimTypeChar(label="Byte"), offset=0)}, name="FWP_BYTE_BLOB", pack=False, align=None), "eku": SimTypePointer(SimStruct({"numEku": SimTypeInt(signed=False, label="UInt32"), "eku": SimTypePointer(SimTypePointer(SimTypeChar(label="Byte"), offset=0), offset=0)}, name="IKEEXT_CERT_EKUS0", pack=False, align=None), offset=0), "name": SimTypePointer(SimStruct({"nameType": SimTypeInt(signed=False, label="IKEEXT_CERT_CRITERIA_NAME_TYPE"), "certName": SimTypePointer(SimTypeChar(label="Char"), offset=0)}, name="IKEEXT_CERT_NAME0", pack=False, align=None), offset=0), "flags": SimTypeInt(signed=False, label="UInt32")}, name="IKEEXT_CERTIFICATE_CRITERIA0", pack=False, align=None), offset=0)}, name="_Anonymous1_e__Struct", pack=False, align=None), "Anonymous2": SimStruct({"inboundEnterpriseStoreArraySize": SimTypeInt(signed=False, label="UInt32"), "inboundEnterpriseStoreCriteria": SimTypePointer(SimStruct({"certData": SimStruct({"size": SimTypeInt(signed=False, label="UInt32"), "data": SimTypePointer(SimTypeChar(label="Byte"), offset=0)}, name="FWP_BYTE_BLOB", pack=False, align=None), "certHash": SimStruct({"size": SimTypeInt(signed=False, label="UInt32"), "data": SimTypePointer(SimTypeChar(label="Byte"), offset=0)}, name="FWP_BYTE_BLOB", pack=False, align=None), "eku": SimTypePointer(SimStruct({"numEku": SimTypeInt(signed=False, label="UInt32"), "eku": SimTypePointer(SimTypePointer(SimTypeChar(label="Byte"), offset=0), offset=0)}, name="IKEEXT_CERT_EKUS0", pack=False, align=None), offset=0), "name": SimTypePointer(SimStruct({"nameType": SimTypeInt(signed=False, label="IKEEXT_CERT_CRITERIA_NAME_TYPE"), "certName": SimTypePointer(SimTypeChar(label="Char"), offset=0)}, name="IKEEXT_CERT_NAME0", pack=False, align=None), offset=0), "flags": SimTypeInt(signed=False, label="UInt32")}, name="IKEEXT_CERTIFICATE_CRITERIA0", pack=False, align=None), offset=0)}, name="_Anonymous2_e__Struct", pack=False, align=None), "Anonymous3": SimStruct({"inboundRootStoreArraySize": SimTypeInt(signed=False, label="UInt32"), "inboundTrustedRootStoreCriteria": SimTypePointer(SimStruct({"certData": SimStruct({"size": SimTypeInt(signed=False, label="UInt32"), "data": SimTypePointer(SimTypeChar(label="Byte"), offset=0)}, name="FWP_BYTE_BLOB", pack=False, align=None), "certHash": SimStruct({"size": SimTypeInt(signed=False, label="UInt32"), "data": SimTypePointer(SimTypeChar(label="Byte"), offset=0)}, name="FWP_BYTE_BLOB", pack=False, align=None), "eku": SimTypePointer(SimStruct({"numEku": SimTypeInt(signed=False, label="UInt32"), "eku": SimTypePointer(SimTypePointer(SimTypeChar(label="Byte"), offset=0), offset=0)}, name="IKEEXT_CERT_EKUS0", pack=False, align=None), offset=0), "name": SimTypePointer(SimStruct({"nameType": SimTypeInt(signed=False, label="IKEEXT_CERT_CRITERIA_NAME_TYPE"), "certName": SimTypePointer(SimTypeChar(label="Char"), offset=0)}, name="IKEEXT_CERT_NAME0", pack=False, align=None), offset=0), "flags": SimTypeInt(signed=False, label="UInt32")}, name="IKEEXT_CERTIFICATE_CRITERIA0", pack=False, align=None), offset=0)}, name="_Anonymous3_e__Struct", pack=False, align=None)}, name="<anon>", label="None"), "outboundConfigType": SimTypeInt(signed=False, label="IKEEXT_CERT_CONFIG_TYPE"), "Anonymous2": SimUnion({"Anonymous1": SimStruct({"outboundRootArraySize": SimTypeInt(signed=False, label="UInt32"), "outboundRootCriteria": SimTypePointer(SimStruct({"certData": SimStruct({"size": SimTypeInt(signed=False, label="UInt32"), "data": SimTypePointer(SimTypeChar(label="Byte"), offset=0)}, name="FWP_BYTE_BLOB", pack=False, align=None), "certHash": SimStruct({"size": SimTypeInt(signed=False, label="UInt32"), "data": SimTypePointer(SimTypeChar(label="Byte"), offset=0)}, name="FWP_BYTE_BLOB", pack=False, align=None), "eku": SimTypePointer(SimStruct({"numEku": SimTypeInt(signed=False, label="UInt32"), "eku": SimTypePointer(SimTypePointer(SimTypeChar(label="Byte"), offset=0), offset=0)}, name="IKEEXT_CERT_EKUS0", pack=False, align=None), offset=0), "name": SimTypePointer(SimStruct({"nameType": SimTypeInt(signed=False, label="IKEEXT_CERT_CRITERIA_NAME_TYPE"), "certName": SimTypePointer(SimTypeChar(label="Char"), offset=0)}, name="IKEEXT_CERT_NAME0", pack=False, align=None), offset=0), "flags": SimTypeInt(signed=False, label="UInt32")}, name="IKEEXT_CERTIFICATE_CRITERIA0", pack=False, align=None), offset=0)}, name="_Anonymous1_e__Struct", pack=False, align=None), "Anonymous2": SimStruct({"outboundEnterpriseStoreArraySize": SimTypeInt(signed=False, label="UInt32"), "outboundEnterpriseStoreCriteria": SimTypePointer(SimStruct({"certData": SimStruct({"size": SimTypeInt(signed=False, label="UInt32"), "data": SimTypePointer(SimTypeChar(label="Byte"), offset=0)}, name="FWP_BYTE_BLOB", pack=False, align=None), "certHash": SimStruct({"size": SimTypeInt(signed=False, label="UInt32"), "data": SimTypePointer(SimTypeChar(label="Byte"), offset=0)}, name="FWP_BYTE_BLOB", pack=False, align=None), "eku": SimTypePointer(SimStruct({"numEku": SimTypeInt(signed=False, label="UInt32"), "eku": SimTypePointer(SimTypePointer(SimTypeChar(label="Byte"), offset=0), offset=0)}, name="IKEEXT_CERT_EKUS0", pack=False, align=None), offset=0), "name": SimTypePointer(SimStruct({"nameType": SimTypeInt(signed=False, label="IKEEXT_CERT_CRITERIA_NAME_TYPE"), "certName": SimTypePointer(SimTypeChar(label="Char"), offset=0)}, name="IKEEXT_CERT_NAME0", pack=False, align=None), offset=0), "flags": SimTypeInt(signed=False, label="UInt32")}, name="IKEEXT_CERTIFICATE_CRITERIA0", pack=False, align=None), offset=0)}, name="_Anonymous2_e__Struct", pack=False, align=None), "Anonymous3": SimStruct({"outboundRootStoreArraySize": SimTypeInt(signed=False, label="UInt32"), "outboundTrustedRootStoreCriteria": SimTypePointer(SimStruct({"certData": SimStruct({"size": SimTypeInt(signed=False, label="UInt32"), "data": SimTypePointer(SimTypeChar(label="Byte"), offset=0)}, name="FWP_BYTE_BLOB", pack=False, align=None), "certHash": SimStruct({"size": SimTypeInt(signed=False, label="UInt32"), "data": SimTypePointer(SimTypeChar(label="Byte"), offset=0)}, name="FWP_BYTE_BLOB", pack=False, align=None), "eku": SimTypePointer(SimStruct({"numEku": SimTypeInt(signed=False, label="UInt32"), "eku": SimTypePointer(SimTypePointer(SimTypeChar(label="Byte"), offset=0), offset=0)}, name="IKEEXT_CERT_EKUS0", pack=False, align=None), offset=0), "name": SimTypePointer(SimStruct({"nameType": SimTypeInt(signed=False, label="IKEEXT_CERT_CRITERIA_NAME_TYPE"), "certName": SimTypePointer(SimTypeChar(label="Char"), offset=0)}, name="IKEEXT_CERT_NAME0", pack=False, align=None), offset=0), "flags": SimTypeInt(signed=False, label="UInt32")}, name="IKEEXT_CERTIFICATE_CRITERIA0", pack=False, align=None), offset=0)}, name="_Anonymous3_e__Struct", pack=False, align=None)}, name="<anon>", label="None"), "flags": SimTypeInt(signed=False, label="IKEEXT_CERT_AUTH"), "localCertLocationUrl": SimStruct({"size": SimTypeInt(signed=False, label="UInt32"), "data": SimTypePointer(SimTypeChar(label="Byte"), offset=0)}, name="FWP_BYTE_BLOB", pack=False, align=None)}, name="IKEEXT_CERTIFICATE_AUTHENTICATION2", pack=False, align=None), "kerberosAuthentication": SimStruct({"flags": SimTypeInt(signed=False, label="IKEEXT_KERBEROS_AUTHENTICATION_FLAGS"), "proxyServer": SimTypePointer(SimTypeChar(label="Char"), offset=0)}, name="IKEEXT_KERBEROS_AUTHENTICATION1", pack=False, align=None), "reservedAuthentication": SimStruct({"flags": SimTypeInt(signed=False, label="IKEEXT_RESERVED_AUTHENTICATION_FLAGS")}, name="IKEEXT_RESERVED_AUTHENTICATION0", pack=False, align=None), "ntlmV2Authentication": SimStruct({"flags": SimTypeInt(signed=False, label="UInt32")}, name="IKEEXT_NTLM_V2_AUTHENTICATION0", pack=False, align=None), "sslAuthentication": SimStruct({"inboundConfigType": SimTypeInt(signed=False, label="IKEEXT_CERT_CONFIG_TYPE"), "Anonymous1": SimUnion({"Anonymous1": SimStruct({"inboundRootArraySize": SimTypeInt(signed=False, label="UInt32"), "inboundRootCriteria": SimTypePointer(SimStruct({"certData": SimStruct({"size": SimTypeInt(signed=False, label="UInt32"), "data": SimTypePointer(SimTypeChar(label="Byte"), offset=0)}, name="FWP_BYTE_BLOB", pack=False, align=None), "certHash": SimStruct({"size": SimTypeInt(signed=False, label="UInt32"), "data": SimTypePointer(SimTypeChar(label="Byte"), offset=0)}, name="FWP_BYTE_BLOB", pack=False, align=None), "eku": SimTypePointer(SimStruct({"numEku": SimTypeInt(signed=False, label="UInt32"), "eku": SimTypePointer(SimTypePointer(SimTypeChar(label="Byte"), offset=0), offset=0)}, name="IKEEXT_CERT_EKUS0", pack=False, align=None), offset=0), "name": SimTypePointer(SimStruct({"nameType": SimTypeInt(signed=False, label="IKEEXT_CERT_CRITERIA_NAME_TYPE"), "certName": SimTypePointer(SimTypeChar(label="Char"), offset=0)}, name="IKEEXT_CERT_NAME0", pack=False, align=None), offset=0), "flags": SimTypeInt(signed=False, label="UInt32")}, name="IKEEXT_CERTIFICATE_CRITERIA0", pack=False, align=None), offset=0)}, name="_Anonymous1_e__Struct", pack=False, align=None), "Anonymous2": SimStruct({"inboundEnterpriseStoreArraySize": SimTypeInt(signed=False, label="UInt32"), "inboundEnterpriseStoreCriteria": SimTypePointer(SimStruct({"certData": SimStruct({"size": SimTypeInt(signed=False, label="UInt32"), "data": SimTypePointer(SimTypeChar(label="Byte"), offset=0)}, name="FWP_BYTE_BLOB", pack=False, align=None), "certHash": SimStruct({"size": SimTypeInt(signed=False, label="UInt32"), "data": SimTypePointer(SimTypeChar(label="Byte"), offset=0)}, name="FWP_BYTE_BLOB", pack=False, align=None), "eku": SimTypePointer(SimStruct({"numEku": SimTypeInt(signed=False, label="UInt32"), "eku": SimTypePointer(SimTypePointer(SimTypeChar(label="Byte"), offset=0), offset=0)}, name="IKEEXT_CERT_EKUS0", pack=False, align=None), offset=0), "name": SimTypePointer(SimStruct({"nameType": SimTypeInt(signed=False, label="IKEEXT_CERT_CRITERIA_NAME_TYPE"), "certName": SimTypePointer(SimTypeChar(label="Char"), offset=0)}, name="IKEEXT_CERT_NAME0", pack=False, align=None), offset=0), "flags": SimTypeInt(signed=False, label="UInt32")}, name="IKEEXT_CERTIFICATE_CRITERIA0", pack=False, align=None), offset=0)}, name="_Anonymous2_e__Struct", pack=False, align=None), "Anonymous3": SimStruct({"inboundRootStoreArraySize": SimTypeInt(signed=False, label="UInt32"), "inboundTrustedRootStoreCriteria": SimTypePointer(SimStruct({"certData": SimStruct({"size": SimTypeInt(signed=False, label="UInt32"), "data": SimTypePointer(SimTypeChar(label="Byte"), offset=0)}, name="FWP_BYTE_BLOB", pack=False, align=None), "certHash": SimStruct({"size": SimTypeInt(signed=False, label="UInt32"), "data": SimTypePointer(SimTypeChar(label="Byte"), offset=0)}, name="FWP_BYTE_BLOB", pack=False, align=None), "eku": SimTypePointer(SimStruct({"numEku": SimTypeInt(signed=False, label="UInt32"), "eku": SimTypePointer(SimTypePointer(SimTypeChar(label="Byte"), offset=0), offset=0)}, name="IKEEXT_CERT_EKUS0", pack=False, align=None), offset=0), "name": SimTypePointer(SimStruct({"nameType": SimTypeInt(signed=False, label="IKEEXT_CERT_CRITERIA_NAME_TYPE"), "certName": SimTypePointer(SimTypeChar(label="Char"), offset=0)}, name="IKEEXT_CERT_NAME0", pack=False, align=None), offset=0), "flags": SimTypeInt(signed=False, label="UInt32")}, name="IKEEXT_CERTIFICATE_CRITERIA0", pack=False, align=None), offset=0)}, name="_Anonymous3_e__Struct", pack=False, align=None)}, name="<anon>", label="None"), "outboundConfigType": SimTypeInt(signed=False, label="IKEEXT_CERT_CONFIG_TYPE"), "Anonymous2": SimUnion({"Anonymous1": SimStruct({"outboundRootArraySize": SimTypeInt(signed=False, label="UInt32"), "outboundRootCriteria": SimTypePointer(SimStruct({"certData": SimStruct({"size": SimTypeInt(signed=False, label="UInt32"), "data": SimTypePointer(SimTypeChar(label="Byte"), offset=0)}, name="FWP_BYTE_BLOB", pack=False, align=None), "certHash": SimStruct({"size": SimTypeInt(signed=False, label="UInt32"), "data": SimTypePointer(SimTypeChar(label="Byte"), offset=0)}, name="FWP_BYTE_BLOB", pack=False, align=None), "eku": SimTypePointer(SimStruct({"numEku": SimTypeInt(signed=False, label="UInt32"), "eku": SimTypePointer(SimTypePointer(SimTypeChar(label="Byte"), offset=0), offset=0)}, name="IKEEXT_CERT_EKUS0", pack=False, align=None), offset=0), "name": SimTypePointer(SimStruct({"nameType": SimTypeInt(signed=False, label="IKEEXT_CERT_CRITERIA_NAME_TYPE"), "certName": SimTypePointer(SimTypeChar(label="Char"), offset=0)}, name="IKEEXT_CERT_NAME0", pack=False, align=None), offset=0), "flags": SimTypeInt(signed=False, label="UInt32")}, name="IKEEXT_CERTIFICATE_CRITERIA0", pack=False, align=None), offset=0)}, name="_Anonymous1_e__Struct", pack=False, align=None), "Anonymous2": SimStruct({"outboundEnterpriseStoreArraySize": SimTypeInt(signed=False, label="UInt32"), "outboundEnterpriseStoreCriteria": SimTypePointer(SimStruct({"certData": SimStruct({"size": SimTypeInt(signed=False, label="UInt32"), "data": SimTypePointer(SimTypeChar(label="Byte"), offset=0)}, name="FWP_BYTE_BLOB", pack=False, align=None), "certHash": SimStruct({"size": SimTypeInt(signed=False, label="UInt32"), "data": SimTypePointer(SimTypeChar(label="Byte"), offset=0)}, name="FWP_BYTE_BLOB", pack=False, align=None), "eku": SimTypePointer(SimStruct({"numEku": SimTypeInt(signed=False, label="UInt32"), "eku": SimTypePointer(SimTypePointer(SimTypeChar(label="Byte"), offset=0), offset=0)}, name="IKEEXT_CERT_EKUS0", pack=False, align=None), offset=0), "name": SimTypePointer(SimStruct({"nameType": SimTypeInt(signed=False, label="IKEEXT_CERT_CRITERIA_NAME_TYPE"), "certName": SimTypePointer(SimTypeChar(label="Char"), offset=0)}, name="IKEEXT_CERT_NAME0", pack=False, align=None), offset=0), "flags": SimTypeInt(signed=False, label="UInt32")}, name="IKEEXT_CERTIFICATE_CRITERIA0", pack=False, align=None), offset=0)}, name="_Anonymous2_e__Struct", pack=False, align=None), "Anonymous3": SimStruct({"outboundRootStoreArraySize": SimTypeInt(signed=False, label="UInt32"), "outboundTrustedRootStoreCriteria": SimTypePointer(SimStruct({"certData": SimStruct({"size": SimTypeInt(signed=False, label="UInt32"), "data": SimTypePointer(SimTypeChar(label="Byte"), offset=0)}, name="FWP_BYTE_BLOB", pack=False, align=None), "certHash": SimStruct({"size": SimTypeInt(signed=False, label="UInt32"), "data": SimTypePointer(SimTypeChar(label="Byte"), offset=0)}, name="FWP_BYTE_BLOB", pack=False, align=None), "eku": SimTypePointer(SimStruct({"numEku": SimTypeInt(signed=False, label="UInt32"), "eku": SimTypePointer(SimTypePointer(SimTypeChar(label="Byte"), offset=0), offset=0)}, name="IKEEXT_CERT_EKUS0", pack=False, align=None), offset=0), "name": SimTypePointer(SimStruct({"nameType": SimTypeInt(signed=False, label="IKEEXT_CERT_CRITERIA_NAME_TYPE"), "certName": SimTypePointer(SimTypeChar(label="Char"), offset=0)}, name="IKEEXT_CERT_NAME0", pack=False, align=None), offset=0), "flags": SimTypeInt(signed=False, label="UInt32")}, name="IKEEXT_CERTIFICATE_CRITERIA0", pack=False, align=None), offset=0)}, name="_Anonymous3_e__Struct", pack=False, align=None)}, name="<anon>", label="None"), "flags": SimTypeInt(signed=False, label="IKEEXT_CERT_AUTH"), "localCertLocationUrl": SimStruct({"size": SimTypeInt(signed=False, label="UInt32"), "data": SimTypePointer(SimTypeChar(label="Byte"), offset=0)}, name="FWP_BYTE_BLOB", pack=False, align=None)}, name="IKEEXT_CERTIFICATE_AUTHENTICATION2", pack=False, align=None), "cgaAuthentication": SimStruct({"keyContainerName": SimTypePointer(SimTypeChar(label="Char"), offset=0), "cspName": SimTypePointer(SimTypeChar(label="Char"), offset=0), "cspType": SimTypeInt(signed=False, label="UInt32"), "cgaModifier": SimStruct({"byteArray16": SimTypeFixedSizeArray(SimTypeChar(label="Byte"), 16)}, name="FWP_BYTE_ARRAY16", pack=False, align=None), "cgaCollisionCount": SimTypeChar(label="Byte")}, name="IKEEXT_IPV6_CGA_AUTHENTICATION0", pack=False, align=None), "eapAuthentication": SimStruct({"flags": SimTypeInt(signed=False, label="IKEEXT_EAP_AUTHENTICATION_FLAGS")}, name="IKEEXT_EAP_AUTHENTICATION0", pack=False, align=None)}, name="<anon>", label="None")}, name="IKEEXT_AUTHENTICATION_METHOD2", pack=False, align=None), offset=0), "initiatorImpersonationType": SimTypeInt(signed=False, label="IKEEXT_AUTHENTICATION_IMPERSONATION_TYPE")}, name="IKEEXT_EM_POLICY2", pack=False, align=None), offset=0), "fwdPathSaLifetime": SimTypeInt(signed=False, label="UInt32"), "compartmentId": SimTypeInt(signed=False, label="UInt32"), "numTrafficSelectorPolicy": SimTypeInt(signed=False, label="UInt32"), "trafficSelectorPolicies": SimTypePointer(SimStruct({"flags": SimTypeInt(signed=False, label="UInt32"), "numLocalTrafficSelectors": SimTypeInt(signed=False, label="UInt32"), "localTrafficSelectors": SimTypePointer(SimStruct({"protocolId": SimTypeChar(label="Byte"), "portStart": SimTypeShort(signed=False, label="UInt16"), "portEnd": SimTypeShort(signed=False, label="UInt16"), "ipVersion": SimTypeInt(signed=False, label="FWP_IP_VERSION"), "Anonymous1": SimUnion({"startV4Address": SimTypeInt(signed=False, label="UInt32"), "startV6Address": SimTypeFixedSizeArray(SimTypeChar(label="Byte"), 16)}, name="<anon>", label="None"), "Anonymous2": SimUnion({"endV4Address": SimTypeInt(signed=False, label="UInt32"), "endV6Address": SimTypeFixedSizeArray(SimTypeChar(label="Byte"), 16)}, name="<anon>", label="None")}, name="IPSEC_TRAFFIC_SELECTOR0_", pack=False, align=None), offset=0), "numRemoteTrafficSelectors": SimTypeInt(signed=False, label="UInt32"), "remoteTrafficSelectors": SimTypePointer(SimStruct({"protocolId": SimTypeChar(label="Byte"), "portStart": SimTypeShort(signed=False, label="UInt16"), "portEnd": SimTypeShort(signed=False, label="UInt16"), "ipVersion": SimTypeInt(signed=False, label="FWP_IP_VERSION"), "Anonymous1": SimUnion({"startV4Address": SimTypeInt(signed=False, label="UInt32"), "startV6Address": SimTypeFixedSizeArray(SimTypeChar(label="Byte"), 16)}, name="<anon>", label="None"), "Anonymous2": SimUnion({"endV4Address": SimTypeInt(signed=False, label="UInt32"), "endV6Address": SimTypeFixedSizeArray(SimTypeChar(label="Byte"), 16)}, name="<anon>", label="None")}, name="IPSEC_TRAFFIC_SELECTOR0_", pack=False, align=None), offset=0)}, name="IPSEC_TRAFFIC_SELECTOR_POLICY0_", pack=False, align=None), offset=0)}, name="IPSEC_TUNNEL_POLICY3_", pack=False, align=None), offset=0), "ikeMmPolicy": SimTypePointer(SimStruct({"softExpirationTime": SimTypeInt(signed=False, label="UInt32"), "numAuthenticationMethods": SimTypeInt(signed=False, label="UInt32"), "authenticationMethods": SimTypePointer(SimStruct({"authenticationMethodType": SimTypeInt(signed=False, label="IKEEXT_AUTHENTICATION_METHOD_TYPE"), "Anonymous": SimUnion({"presharedKeyAuthentication": SimStruct({"presharedKey": SimStruct({"size": SimTypeInt(signed=False, label="UInt32"), "data": SimTypePointer(SimTypeChar(label="Byte"), offset=0)}, name="FWP_BYTE_BLOB", pack=False, align=None), "flags": SimTypeInt(signed=False, label="IKEEXT_PRESHARED_KEY_AUTHENTICATION_FLAGS")}, name="IKEEXT_PRESHARED_KEY_AUTHENTICATION1", pack=False, align=None), "certificateAuthentication": SimStruct({"inboundConfigType": SimTypeInt(signed=False, label="IKEEXT_CERT_CONFIG_TYPE"), "Anonymous1": SimUnion({"Anonymous1": SimStruct({"inboundRootArraySize": SimTypeInt(signed=False, label="UInt32"), "inboundRootCriteria": SimTypePointer(SimStruct({"certData": SimStruct({"size": SimTypeInt(signed=False, label="UInt32"), "data": SimTypePointer(SimTypeChar(label="Byte"), offset=0)}, name="FWP_BYTE_BLOB", pack=False, align=None), "certHash": SimStruct({"size": SimTypeInt(signed=False, label="UInt32"), "data": SimTypePointer(SimTypeChar(label="Byte"), offset=0)}, name="FWP_BYTE_BLOB", pack=False, align=None), "eku": SimTypePointer(SimStruct({"numEku": SimTypeInt(signed=False, label="UInt32"), "eku": SimTypePointer(SimTypePointer(SimTypeChar(label="Byte"), offset=0), offset=0)}, name="IKEEXT_CERT_EKUS0", pack=False, align=None), offset=0), "name": SimTypePointer(SimStruct({"nameType": SimTypeInt(signed=False, label="IKEEXT_CERT_CRITERIA_NAME_TYPE"), "certName": SimTypePointer(SimTypeChar(label="Char"), offset=0)}, name="IKEEXT_CERT_NAME0", pack=False, align=None), offset=0), "flags": SimTypeInt(signed=False, label="UInt32")}, name="IKEEXT_CERTIFICATE_CRITERIA0", pack=False, align=None), offset=0)}, name="_Anonymous1_e__Struct", pack=False, align=None), "Anonymous2": SimStruct({"inboundEnterpriseStoreArraySize": SimTypeInt(signed=False, label="UInt32"), "inboundEnterpriseStoreCriteria": SimTypePointer(SimStruct({"certData": SimStruct({"size": SimTypeInt(signed=False, label="UInt32"), "data": SimTypePointer(SimTypeChar(label="Byte"), offset=0)}, name="FWP_BYTE_BLOB", pack=False, align=None), "certHash": SimStruct({"size": SimTypeInt(signed=False, label="UInt32"), "data": SimTypePointer(SimTypeChar(label="Byte"), offset=0)}, name="FWP_BYTE_BLOB", pack=False, align=None), "eku": SimTypePointer(SimStruct({"numEku": SimTypeInt(signed=False, label="UInt32"), "eku": SimTypePointer(SimTypePointer(SimTypeChar(label="Byte"), offset=0), offset=0)}, name="IKEEXT_CERT_EKUS0", pack=False, align=None), offset=0), "name": SimTypePointer(SimStruct({"nameType": SimTypeInt(signed=False, label="IKEEXT_CERT_CRITERIA_NAME_TYPE"), "certName": SimTypePointer(SimTypeChar(label="Char"), offset=0)}, name="IKEEXT_CERT_NAME0", pack=False, align=None), offset=0), "flags": SimTypeInt(signed=False, label="UInt32")}, name="IKEEXT_CERTIFICATE_CRITERIA0", pack=False, align=None), offset=0)}, name="_Anonymous2_e__Struct", pack=False, align=None), "Anonymous3": SimStruct({"inboundRootStoreArraySize": SimTypeInt(signed=False, label="UInt32"), "inboundTrustedRootStoreCriteria": SimTypePointer(SimStruct({"certData": SimStruct({"size": SimTypeInt(signed=False, label="UInt32"), "data": SimTypePointer(SimTypeChar(label="Byte"), offset=0)}, name="FWP_BYTE_BLOB", pack=False, align=None), "certHash": SimStruct({"size": SimTypeInt(signed=False, label="UInt32"), "data": SimTypePointer(SimTypeChar(label="Byte"), offset=0)}, name="FWP_BYTE_BLOB", pack=False, align=None), "eku": SimTypePointer(SimStruct({"numEku": SimTypeInt(signed=False, label="UInt32"), "eku": SimTypePointer(SimTypePointer(SimTypeChar(label="Byte"), offset=0), offset=0)}, name="IKEEXT_CERT_EKUS0", pack=False, align=None), offset=0), "name": SimTypePointer(SimStruct({"nameType": SimTypeInt(signed=False, label="IKEEXT_CERT_CRITERIA_NAME_TYPE"), "certName": SimTypePointer(SimTypeChar(label="Char"), offset=0)}, name="IKEEXT_CERT_NAME0", pack=False, align=None), offset=0), "flags": SimTypeInt(signed=False, label="UInt32")}, name="IKEEXT_CERTIFICATE_CRITERIA0", pack=False, align=None), offset=0)}, name="_Anonymous3_e__Struct", pack=False, align=None)}, name="<anon>", label="None"), "outboundConfigType": SimTypeInt(signed=False, label="IKEEXT_CERT_CONFIG_TYPE"), "Anonymous2": SimUnion({"Anonymous1": SimStruct({"outboundRootArraySize": SimTypeInt(signed=False, label="UInt32"), "outboundRootCriteria": SimTypePointer(SimStruct({"certData": SimStruct({"size": SimTypeInt(signed=False, label="UInt32"), "data": SimTypePointer(SimTypeChar(label="Byte"), offset=0)}, name="FWP_BYTE_BLOB", pack=False, align=None), "certHash": SimStruct({"size": SimTypeInt(signed=False, label="UInt32"), "data": SimTypePointer(SimTypeChar(label="Byte"), offset=0)}, name="FWP_BYTE_BLOB", pack=False, align=None), "eku": SimTypePointer(SimStruct({"numEku": SimTypeInt(signed=False, label="UInt32"), "eku": SimTypePointer(SimTypePointer(SimTypeChar(label="Byte"), offset=0), offset=0)}, name="IKEEXT_CERT_EKUS0", pack=False, align=None), offset=0), "name": SimTypePointer(SimStruct({"nameType": SimTypeInt(signed=False, label="IKEEXT_CERT_CRITERIA_NAME_TYPE"), "certName": SimTypePointer(SimTypeChar(label="Char"), offset=0)}, name="IKEEXT_CERT_NAME0", pack=False, align=None), offset=0), "flags": SimTypeInt(signed=False, label="UInt32")}, name="IKEEXT_CERTIFICATE_CRITERIA0", pack=False, align=None), offset=0)}, name="_Anonymous1_e__Struct", pack=False, align=None), "Anonymous2": SimStruct({"outboundEnterpriseStoreArraySize": SimTypeInt(signed=False, label="UInt32"), "outboundEnterpriseStoreCriteria": SimTypePointer(SimStruct({"certData": SimStruct({"size": SimTypeInt(signed=False, label="UInt32"), "data": SimTypePointer(SimTypeChar(label="Byte"), offset=0)}, name="FWP_BYTE_BLOB", pack=False, align=None), "certHash": SimStruct({"size": SimTypeInt(signed=False, label="UInt32"), "data": SimTypePointer(SimTypeChar(label="Byte"), offset=0)}, name="FWP_BYTE_BLOB", pack=False, align=None), "eku": SimTypePointer(SimStruct({"numEku": SimTypeInt(signed=False, label="UInt32"), "eku": SimTypePointer(SimTypePointer(SimTypeChar(label="Byte"), offset=0), offset=0)}, name="IKEEXT_CERT_EKUS0", pack=False, align=None), offset=0), "name": SimTypePointer(SimStruct({"nameType": SimTypeInt(signed=False, label="IKEEXT_CERT_CRITERIA_NAME_TYPE"), "certName": SimTypePointer(SimTypeChar(label="Char"), offset=0)}, name="IKEEXT_CERT_NAME0", pack=False, align=None), offset=0), "flags": SimTypeInt(signed=False, label="UInt32")}, name="IKEEXT_CERTIFICATE_CRITERIA0", pack=False, align=None), offset=0)}, name="_Anonymous2_e__Struct", pack=False, align=None), "Anonymous3": SimStruct({"outboundRootStoreArraySize": SimTypeInt(signed=False, label="UInt32"), "outboundTrustedRootStoreCriteria": SimTypePointer(SimStruct({"certData": SimStruct({"size": SimTypeInt(signed=False, label="UInt32"), "data": SimTypePointer(SimTypeChar(label="Byte"), offset=0)}, name="FWP_BYTE_BLOB", pack=False, align=None), "certHash": SimStruct({"size": SimTypeInt(signed=False, label="UInt32"), "data": SimTypePointer(SimTypeChar(label="Byte"), offset=0)}, name="FWP_BYTE_BLOB", pack=False, align=None), "eku": SimTypePointer(SimStruct({"numEku": SimTypeInt(signed=False, label="UInt32"), "eku": SimTypePointer(SimTypePointer(SimTypeChar(label="Byte"), offset=0), offset=0)}, name="IKEEXT_CERT_EKUS0", pack=False, align=None), offset=0), "name": SimTypePointer(SimStruct({"nameType": SimTypeInt(signed=False, label="IKEEXT_CERT_CRITERIA_NAME_TYPE"), "certName": SimTypePointer(SimTypeChar(label="Char"), offset=0)}, name="IKEEXT_CERT_NAME0", pack=False, align=None), offset=0), "flags": SimTypeInt(signed=False, label="UInt32")}, name="IKEEXT_CERTIFICATE_CRITERIA0", pack=False, align=None), offset=0)}, name="_Anonymous3_e__Struct", pack=False, align=None)}, name="<anon>", label="None"), "flags": SimTypeInt(signed=False, label="IKEEXT_CERT_AUTH"), "localCertLocationUrl": SimStruct({"size": SimTypeInt(signed=False, label="UInt32"), "data": SimTypePointer(SimTypeChar(label="Byte"), offset=0)}, name="FWP_BYTE_BLOB", pack=False, align=None)}, name="IKEEXT_CERTIFICATE_AUTHENTICATION2", pack=False, align=None), "kerberosAuthentication": SimStruct({"flags": SimTypeInt(signed=False, label="IKEEXT_KERBEROS_AUTHENTICATION_FLAGS"), "proxyServer": SimTypePointer(SimTypeChar(label="Char"), offset=0)}, name="IKEEXT_KERBEROS_AUTHENTICATION1", pack=False, align=None), "reservedAuthentication": SimStruct({"flags": SimTypeInt(signed=False, label="IKEEXT_RESERVED_AUTHENTICATION_FLAGS")}, name="IKEEXT_RESERVED_AUTHENTICATION0", pack=False, align=None), "ntlmV2Authentication": SimStruct({"flags": SimTypeInt(signed=False, label="UInt32")}, name="IKEEXT_NTLM_V2_AUTHENTICATION0", pack=False, align=None), "sslAuthentication": SimStruct({"inboundConfigType": SimTypeInt(signed=False, label="IKEEXT_CERT_CONFIG_TYPE"), "Anonymous1": SimUnion({"Anonymous1": SimStruct({"inboundRootArraySize": SimTypeInt(signed=False, label="UInt32"), "inboundRootCriteria": SimTypePointer(SimStruct({"certData": SimStruct({"size": SimTypeInt(signed=False, label="UInt32"), "data": SimTypePointer(SimTypeChar(label="Byte"), offset=0)}, name="FWP_BYTE_BLOB", pack=False, align=None), "certHash": SimStruct({"size": SimTypeInt(signed=False, label="UInt32"), "data": SimTypePointer(SimTypeChar(label="Byte"), offset=0)}, name="FWP_BYTE_BLOB", pack=False, align=None), "eku": SimTypePointer(SimStruct({"numEku": SimTypeInt(signed=False, label="UInt32"), "eku": SimTypePointer(SimTypePointer(SimTypeChar(label="Byte"), offset=0), offset=0)}, name="IKEEXT_CERT_EKUS0", pack=False, align=None), offset=0), "name": SimTypePointer(SimStruct({"nameType": SimTypeInt(signed=False, label="IKEEXT_CERT_CRITERIA_NAME_TYPE"), "certName": SimTypePointer(SimTypeChar(label="Char"), offset=0)}, name="IKEEXT_CERT_NAME0", pack=False, align=None), offset=0), "flags": SimTypeInt(signed=False, label="UInt32")}, name="IKEEXT_CERTIFICATE_CRITERIA0", pack=False, align=None), offset=0)}, name="_Anonymous1_e__Struct", pack=False, align=None), "Anonymous2": SimStruct({"inboundEnterpriseStoreArraySize": SimTypeInt(signed=False, label="UInt32"), "inboundEnterpriseStoreCriteria": SimTypePointer(SimStruct({"certData": SimStruct({"size": SimTypeInt(signed=False, label="UInt32"), "data": SimTypePointer(SimTypeChar(label="Byte"), offset=0)}, name="FWP_BYTE_BLOB", pack=False, align=None), "certHash": SimStruct({"size": SimTypeInt(signed=False, label="UInt32"), "data": SimTypePointer(SimTypeChar(label="Byte"), offset=0)}, name="FWP_BYTE_BLOB", pack=False, align=None), "eku": SimTypePointer(SimStruct({"numEku": SimTypeInt(signed=False, label="UInt32"), "eku": SimTypePointer(SimTypePointer(SimTypeChar(label="Byte"), offset=0), offset=0)}, name="IKEEXT_CERT_EKUS0", pack=False, align=None), offset=0), "name": SimTypePointer(SimStruct({"nameType": SimTypeInt(signed=False, label="IKEEXT_CERT_CRITERIA_NAME_TYPE"), "certName": SimTypePointer(SimTypeChar(label="Char"), offset=0)}, name="IKEEXT_CERT_NAME0", pack=False, align=None), offset=0), "flags": SimTypeInt(signed=False, label="UInt32")}, name="IKEEXT_CERTIFICATE_CRITERIA0", pack=False, align=None), offset=0)}, name="_Anonymous2_e__Struct", pack=False, align=None), "Anonymous3": SimStruct({"inboundRootStoreArraySize": SimTypeInt(signed=False, label="UInt32"), "inboundTrustedRootStoreCriteria": SimTypePointer(SimStruct({"certData": SimStruct({"size": SimTypeInt(signed=False, label="UInt32"), "data": SimTypePointer(SimTypeChar(label="Byte"), offset=0)}, name="FWP_BYTE_BLOB", pack=False, align=None), "certHash": SimStruct({"size": SimTypeInt(signed=False, label="UInt32"), "data": SimTypePointer(SimTypeChar(label="Byte"), offset=0)}, name="FWP_BYTE_BLOB", pack=False, align=None), "eku": SimTypePointer(SimStruct({"numEku": SimTypeInt(signed=False, label="UInt32"), "eku": SimTypePointer(SimTypePointer(SimTypeChar(label="Byte"), offset=0), offset=0)}, name="IKEEXT_CERT_EKUS0", pack=False, align=None), offset=0), "name": SimTypePointer(SimStruct({"nameType": SimTypeInt(signed=False, label="IKEEXT_CERT_CRITERIA_NAME_TYPE"), "certName": SimTypePointer(SimTypeChar(label="Char"), offset=0)}, name="IKEEXT_CERT_NAME0", pack=False, align=None), offset=0), "flags": SimTypeInt(signed=False, label="UInt32")}, name="IKEEXT_CERTIFICATE_CRITERIA0", pack=False, align=None), offset=0)}, name="_Anonymous3_e__Struct", pack=False, align=None)}, name="<anon>", label="None"), "outboundConfigType": SimTypeInt(signed=False, label="IKEEXT_CERT_CONFIG_TYPE"), "Anonymous2": SimUnion({"Anonymous1": SimStruct({"outboundRootArraySize": SimTypeInt(signed=False, label="UInt32"), "outboundRootCriteria": SimTypePointer(SimStruct({"certData": SimStruct({"size": SimTypeInt(signed=False, label="UInt32"), "data": SimTypePointer(SimTypeChar(label="Byte"), offset=0)}, name="FWP_BYTE_BLOB", pack=False, align=None), "certHash": SimStruct({"size": SimTypeInt(signed=False, label="UInt32"), "data": SimTypePointer(SimTypeChar(label="Byte"), offset=0)}, name="FWP_BYTE_BLOB", pack=False, align=None), "eku": SimTypePointer(SimStruct({"numEku": SimTypeInt(signed=False, label="UInt32"), "eku": SimTypePointer(SimTypePointer(SimTypeChar(label="Byte"), offset=0), offset=0)}, name="IKEEXT_CERT_EKUS0", pack=False, align=None), offset=0), "name": SimTypePointer(SimStruct({"nameType": SimTypeInt(signed=False, label="IKEEXT_CERT_CRITERIA_NAME_TYPE"), "certName": SimTypePointer(SimTypeChar(label="Char"), offset=0)}, name="IKEEXT_CERT_NAME0", pack=False, align=None), offset=0), "flags": SimTypeInt(signed=False, label="UInt32")}, name="IKEEXT_CERTIFICATE_CRITERIA0", pack=False, align=None), offset=0)}, name="_Anonymous1_e__Struct", pack=False, align=None), "Anonymous2": SimStruct({"outboundEnterpriseStoreArraySize": SimTypeInt(signed=False, label="UInt32"), "outboundEnterpriseStoreCriteria": SimTypePointer(SimStruct({"certData": SimStruct({"size": SimTypeInt(signed=False, label="UInt32"), "data": SimTypePointer(SimTypeChar(label="Byte"), offset=0)}, name="FWP_BYTE_BLOB", pack=False, align=None), "certHash": SimStruct({"size": SimTypeInt(signed=False, label="UInt32"), "data": SimTypePointer(SimTypeChar(label="Byte"), offset=0)}, name="FWP_BYTE_BLOB", pack=False, align=None), "eku": SimTypePointer(SimStruct({"numEku": SimTypeInt(signed=False, label="UInt32"), "eku": SimTypePointer(SimTypePointer(SimTypeChar(label="Byte"), offset=0), offset=0)}, name="IKEEXT_CERT_EKUS0", pack=False, align=None), offset=0), "name": SimTypePointer(SimStruct({"nameType": SimTypeInt(signed=False, label="IKEEXT_CERT_CRITERIA_NAME_TYPE"), "certName": SimTypePointer(SimTypeChar(label="Char"), offset=0)}, name="IKEEXT_CERT_NAME0", pack=False, align=None), offset=0), "flags": SimTypeInt(signed=False, label="UInt32")}, name="IKEEXT_CERTIFICATE_CRITERIA0", pack=False, align=None), offset=0)}, name="_Anonymous2_e__Struct", pack=False, align=None), "Anonymous3": SimStruct({"outboundRootStoreArraySize": SimTypeInt(signed=False, label="UInt32"), "outboundTrustedRootStoreCriteria": SimTypePointer(SimStruct({"certData": SimStruct({"size": SimTypeInt(signed=False, label="UInt32"), "data": SimTypePointer(SimTypeChar(label="Byte"), offset=0)}, name="FWP_BYTE_BLOB", pack=False, align=None), "certHash": SimStruct({"size": SimTypeInt(signed=False, label="UInt32"), "data": SimTypePointer(SimTypeChar(label="Byte"), offset=0)}, name="FWP_BYTE_BLOB", pack=False, align=None), "eku": SimTypePointer(SimStruct({"numEku": SimTypeInt(signed=False, label="UInt32"), "eku": SimTypePointer(SimTypePointer(SimTypeChar(label="Byte"), offset=0), offset=0)}, name="IKEEXT_CERT_EKUS0", pack=False, align=None), offset=0), "name": SimTypePointer(SimStruct({"nameType": SimTypeInt(signed=False, label="IKEEXT_CERT_CRITERIA_NAME_TYPE"), "certName": SimTypePointer(SimTypeChar(label="Char"), offset=0)}, name="IKEEXT_CERT_NAME0", pack=False, align=None), offset=0), "flags": SimTypeInt(signed=False, label="UInt32")}, name="IKEEXT_CERTIFICATE_CRITERIA0", pack=False, align=None), offset=0)}, name="_Anonymous3_e__Struct", pack=False, align=None)}, name="<anon>", label="None"), "flags": SimTypeInt(signed=False, label="IKEEXT_CERT_AUTH"), "localCertLocationUrl": SimStruct({"size": SimTypeInt(signed=False, label="UInt32"), "data": SimTypePointer(SimTypeChar(label="Byte"), offset=0)}, name="FWP_BYTE_BLOB", pack=False, align=None)}, name="IKEEXT_CERTIFICATE_AUTHENTICATION2", pack=False, align=None), "cgaAuthentication": SimStruct({"keyContainerName": SimTypePointer(SimTypeChar(label="Char"), offset=0), "cspName": SimTypePointer(SimTypeChar(label="Char"), offset=0), "cspType": SimTypeInt(signed=False, label="UInt32"), "cgaModifier": SimStruct({"byteArray16": SimTypeFixedSizeArray(SimTypeChar(label="Byte"), 16)}, name="FWP_BYTE_ARRAY16", pack=False, align=None), "cgaCollisionCount": SimTypeChar(label="Byte")}, name="IKEEXT_IPV6_CGA_AUTHENTICATION0", pack=False, align=None), "eapAuthentication": SimStruct({"flags": SimTypeInt(signed=False, label="IKEEXT_EAP_AUTHENTICATION_FLAGS")}, name="IKEEXT_EAP_AUTHENTICATION0", pack=False, align=None)}, name="<anon>", label="None")}, name="IKEEXT_AUTHENTICATION_METHOD2", pack=False, align=None), offset=0), "initiatorImpersonationType": SimTypeInt(signed=False, label="IKEEXT_AUTHENTICATION_IMPERSONATION_TYPE"), "numIkeProposals": SimTypeInt(signed=False, label="UInt32"), "ikeProposals": SimTypePointer(SimStruct({"cipherAlgorithm": SimStruct({"algoIdentifier": SimTypeInt(signed=False, label="IKEEXT_CIPHER_TYPE"), "keyLen": SimTypeInt(signed=False, label="UInt32"), "rounds": SimTypeInt(signed=False, label="UInt32")}, name="IKEEXT_CIPHER_ALGORITHM0", pack=False, align=None), "integrityAlgorithm": SimStruct({"algoIdentifier": SimTypeInt(signed=False, label="IKEEXT_INTEGRITY_TYPE")}, name="IKEEXT_INTEGRITY_ALGORITHM0", pack=False, align=None), "maxLifetimeSeconds": SimTypeInt(signed=False, label="UInt32"), "dhGroup": SimTypeInt(signed=False, label="IKEEXT_DH_GROUP"), "quickModeLimit": SimTypeInt(signed=False, label="UInt32")}, name="IKEEXT_PROPOSAL0", pack=False, align=None), offset=0), "flags": SimTypeInt(signed=False, label="IKEEXT_POLICY_FLAG"), "maxDynamicFilters": SimTypeInt(signed=False, label="UInt32"), "retransmitDurationSecs": SimTypeInt(signed=False, label="UInt32")}, name="IKEEXT_POLICY2", pack=False, align=None), offset=0), "authIpMmPolicy": SimTypePointer(SimStruct({"softExpirationTime": SimTypeInt(signed=False, label="UInt32"), "numAuthenticationMethods": SimTypeInt(signed=False, label="UInt32"), "authenticationMethods": SimTypePointer(SimStruct({"authenticationMethodType": SimTypeInt(signed=False, label="IKEEXT_AUTHENTICATION_METHOD_TYPE"), "Anonymous": SimUnion({"presharedKeyAuthentication": SimStruct({"presharedKey": SimStruct({"size": SimTypeInt(signed=False, label="UInt32"), "data": SimTypePointer(SimTypeChar(label="Byte"), offset=0)}, name="FWP_BYTE_BLOB", pack=False, align=None), "flags": SimTypeInt(signed=False, label="IKEEXT_PRESHARED_KEY_AUTHENTICATION_FLAGS")}, name="IKEEXT_PRESHARED_KEY_AUTHENTICATION1", pack=False, align=None), "certificateAuthentication": SimStruct({"inboundConfigType": SimTypeInt(signed=False, label="IKEEXT_CERT_CONFIG_TYPE"), "Anonymous1": SimUnion({"Anonymous1": SimStruct({"inboundRootArraySize": SimTypeInt(signed=False, label="UInt32"), "inboundRootCriteria": SimTypePointer(SimStruct({"certData": SimStruct({"size": SimTypeInt(signed=False, label="UInt32"), "data": SimTypePointer(SimTypeChar(label="Byte"), offset=0)}, name="FWP_BYTE_BLOB", pack=False, align=None), "certHash": SimStruct({"size": SimTypeInt(signed=False, label="UInt32"), "data": SimTypePointer(SimTypeChar(label="Byte"), offset=0)}, name="FWP_BYTE_BLOB", pack=False, align=None), "eku": SimTypePointer(SimStruct({"numEku": SimTypeInt(signed=False, label="UInt32"), "eku": SimTypePointer(SimTypePointer(SimTypeChar(label="Byte"), offset=0), offset=0)}, name="IKEEXT_CERT_EKUS0", pack=False, align=None), offset=0), "name": SimTypePointer(SimStruct({"nameType": SimTypeInt(signed=False, label="IKEEXT_CERT_CRITERIA_NAME_TYPE"), "certName": SimTypePointer(SimTypeChar(label="Char"), offset=0)}, name="IKEEXT_CERT_NAME0", pack=False, align=None), offset=0), "flags": SimTypeInt(signed=False, label="UInt32")}, name="IKEEXT_CERTIFICATE_CRITERIA0", pack=False, align=None), offset=0)}, name="_Anonymous1_e__Struct", pack=False, align=None), "Anonymous2": SimStruct({"inboundEnterpriseStoreArraySize": SimTypeInt(signed=False, label="UInt32"), "inboundEnterpriseStoreCriteria": SimTypePointer(SimStruct({"certData": SimStruct({"size": SimTypeInt(signed=False, label="UInt32"), "data": SimTypePointer(SimTypeChar(label="Byte"), offset=0)}, name="FWP_BYTE_BLOB", pack=False, align=None), "certHash": SimStruct({"size": SimTypeInt(signed=False, label="UInt32"), "data": SimTypePointer(SimTypeChar(label="Byte"), offset=0)}, name="FWP_BYTE_BLOB", pack=False, align=None), "eku": SimTypePointer(SimStruct({"numEku": SimTypeInt(signed=False, label="UInt32"), "eku": SimTypePointer(SimTypePointer(SimTypeChar(label="Byte"), offset=0), offset=0)}, name="IKEEXT_CERT_EKUS0", pack=False, align=None), offset=0), "name": SimTypePointer(SimStruct({"nameType": SimTypeInt(signed=False, label="IKEEXT_CERT_CRITERIA_NAME_TYPE"), "certName": SimTypePointer(SimTypeChar(label="Char"), offset=0)}, name="IKEEXT_CERT_NAME0", pack=False, align=None), offset=0), "flags": SimTypeInt(signed=False, label="UInt32")}, name="IKEEXT_CERTIFICATE_CRITERIA0", pack=False, align=None), offset=0)}, name="_Anonymous2_e__Struct", pack=False, align=None), "Anonymous3": SimStruct({"inboundRootStoreArraySize": SimTypeInt(signed=False, label="UInt32"), "inboundTrustedRootStoreCriteria": SimTypePointer(SimStruct({"certData": SimStruct({"size": SimTypeInt(signed=False, label="UInt32"), "data": SimTypePointer(SimTypeChar(label="Byte"), offset=0)}, name="FWP_BYTE_BLOB", pack=False, align=None), "certHash": SimStruct({"size": SimTypeInt(signed=False, label="UInt32"), "data": SimTypePointer(SimTypeChar(label="Byte"), offset=0)}, name="FWP_BYTE_BLOB", pack=False, align=None), "eku": SimTypePointer(SimStruct({"numEku": SimTypeInt(signed=False, label="UInt32"), "eku": SimTypePointer(SimTypePointer(SimTypeChar(label="Byte"), offset=0), offset=0)}, name="IKEEXT_CERT_EKUS0", pack=False, align=None), offset=0), "name": SimTypePointer(SimStruct({"nameType": SimTypeInt(signed=False, label="IKEEXT_CERT_CRITERIA_NAME_TYPE"), "certName": SimTypePointer(SimTypeChar(label="Char"), offset=0)}, name="IKEEXT_CERT_NAME0", pack=False, align=None), offset=0), "flags": SimTypeInt(signed=False, label="UInt32")}, name="IKEEXT_CERTIFICATE_CRITERIA0", pack=False, align=None), offset=0)}, name="_Anonymous3_e__Struct", pack=False, align=None)}, name="<anon>", label="None"), "outboundConfigType": SimTypeInt(signed=False, label="IKEEXT_CERT_CONFIG_TYPE"), "Anonymous2": SimUnion({"Anonymous1": SimStruct({"outboundRootArraySize": SimTypeInt(signed=False, label="UInt32"), "outboundRootCriteria": SimTypePointer(SimStruct({"certData": SimStruct({"size": SimTypeInt(signed=False, label="UInt32"), "data": SimTypePointer(SimTypeChar(label="Byte"), offset=0)}, name="FWP_BYTE_BLOB", pack=False, align=None), "certHash": SimStruct({"size": SimTypeInt(signed=False, label="UInt32"), "data": SimTypePointer(SimTypeChar(label="Byte"), offset=0)}, name="FWP_BYTE_BLOB", pack=False, align=None), "eku": SimTypePointer(SimStruct({"numEku": SimTypeInt(signed=False, label="UInt32"), "eku": SimTypePointer(SimTypePointer(SimTypeChar(label="Byte"), offset=0), offset=0)}, name="IKEEXT_CERT_EKUS0", pack=False, align=None), offset=0), "name": SimTypePointer(SimStruct({"nameType": SimTypeInt(signed=False, label="IKEEXT_CERT_CRITERIA_NAME_TYPE"), "certName": SimTypePointer(SimTypeChar(label="Char"), offset=0)}, name="IKEEXT_CERT_NAME0", pack=False, align=None), offset=0), "flags": SimTypeInt(signed=False, label="UInt32")}, name="IKEEXT_CERTIFICATE_CRITERIA0", pack=False, align=None), offset=0)}, name="_Anonymous1_e__Struct", pack=False, align=None), "Anonymous2": SimStruct({"outboundEnterpriseStoreArraySize": SimTypeInt(signed=False, label="UInt32"), "outboundEnterpriseStoreCriteria": SimTypePointer(SimStruct({"certData": SimStruct({"size": SimTypeInt(signed=False, label="UInt32"), "data": SimTypePointer(SimTypeChar(label="Byte"), offset=0)}, name="FWP_BYTE_BLOB", pack=False, align=None), "certHash": SimStruct({"size": SimTypeInt(signed=False, label="UInt32"), "data": SimTypePointer(SimTypeChar(label="Byte"), offset=0)}, name="FWP_BYTE_BLOB", pack=False, align=None), "eku": SimTypePointer(SimStruct({"numEku": SimTypeInt(signed=False, label="UInt32"), "eku": SimTypePointer(SimTypePointer(SimTypeChar(label="Byte"), offset=0), offset=0)}, name="IKEEXT_CERT_EKUS0", pack=False, align=None), offset=0), "name": SimTypePointer(SimStruct({"nameType": SimTypeInt(signed=False, label="IKEEXT_CERT_CRITERIA_NAME_TYPE"), "certName": SimTypePointer(SimTypeChar(label="Char"), offset=0)}, name="IKEEXT_CERT_NAME0", pack=False, align=None), offset=0), "flags": SimTypeInt(signed=False, label="UInt32")}, name="IKEEXT_CERTIFICATE_CRITERIA0", pack=False, align=None), offset=0)}, name="_Anonymous2_e__Struct", pack=False, align=None), "Anonymous3": SimStruct({"outboundRootStoreArraySize": SimTypeInt(signed=False, label="UInt32"), "outboundTrustedRootStoreCriteria": SimTypePointer(SimStruct({"certData": SimStruct({"size": SimTypeInt(signed=False, label="UInt32"), "data": SimTypePointer(SimTypeChar(label="Byte"), offset=0)}, name="FWP_BYTE_BLOB", pack=False, align=None), "certHash": SimStruct({"size": SimTypeInt(signed=False, label="UInt32"), "data": SimTypePointer(SimTypeChar(label="Byte"), offset=0)}, name="FWP_BYTE_BLOB", pack=False, align=None), "eku": SimTypePointer(SimStruct({"numEku": SimTypeInt(signed=False, label="UInt32"), "eku": SimTypePointer(SimTypePointer(SimTypeChar(label="Byte"), offset=0), offset=0)}, name="IKEEXT_CERT_EKUS0", pack=False, align=None), offset=0), "name": SimTypePointer(SimStruct({"nameType": SimTypeInt(signed=False, label="IKEEXT_CERT_CRITERIA_NAME_TYPE"), "certName": SimTypePointer(SimTypeChar(label="Char"), offset=0)}, name="IKEEXT_CERT_NAME0", pack=False, align=None), offset=0), "flags": SimTypeInt(signed=False, label="UInt32")}, name="IKEEXT_CERTIFICATE_CRITERIA0", pack=False, align=None), offset=0)}, name="_Anonymous3_e__Struct", pack=False, align=None)}, name="<anon>", label="None"), "flags": SimTypeInt(signed=False, label="IKEEXT_CERT_AUTH"), "localCertLocationUrl": SimStruct({"size": SimTypeInt(signed=False, label="UInt32"), "data": SimTypePointer(SimTypeChar(label="Byte"), offset=0)}, name="FWP_BYTE_BLOB", pack=False, align=None)}, name="IKEEXT_CERTIFICATE_AUTHENTICATION2", pack=False, align=None), "kerberosAuthentication": SimStruct({"flags": SimTypeInt(signed=False, label="IKEEXT_KERBEROS_AUTHENTICATION_FLAGS"), "proxyServer": SimTypePointer(SimTypeChar(label="Char"), offset=0)}, name="IKEEXT_KERBEROS_AUTHENTICATION1", pack=False, align=None), "reservedAuthentication": SimStruct({"flags": SimTypeInt(signed=False, label="IKEEXT_RESERVED_AUTHENTICATION_FLAGS")}, name="IKEEXT_RESERVED_AUTHENTICATION0", pack=False, align=None), "ntlmV2Authentication": SimStruct({"flags": SimTypeInt(signed=False, label="UInt32")}, name="IKEEXT_NTLM_V2_AUTHENTICATION0", pack=False, align=None), "sslAuthentication": SimStruct({"inboundConfigType": SimTypeInt(signed=False, label="IKEEXT_CERT_CONFIG_TYPE"), "Anonymous1": SimUnion({"Anonymous1": SimStruct({"inboundRootArraySize": SimTypeInt(signed=False, label="UInt32"), "inboundRootCriteria": SimTypePointer(SimStruct({"certData": SimStruct({"size": SimTypeInt(signed=False, label="UInt32"), "data": SimTypePointer(SimTypeChar(label="Byte"), offset=0)}, name="FWP_BYTE_BLOB", pack=False, align=None), "certHash": SimStruct({"size": SimTypeInt(signed=False, label="UInt32"), "data": SimTypePointer(SimTypeChar(label="Byte"), offset=0)}, name="FWP_BYTE_BLOB", pack=False, align=None), "eku": SimTypePointer(SimStruct({"numEku": SimTypeInt(signed=False, label="UInt32"), "eku": SimTypePointer(SimTypePointer(SimTypeChar(label="Byte"), offset=0), offset=0)}, name="IKEEXT_CERT_EKUS0", pack=False, align=None), offset=0), "name": SimTypePointer(SimStruct({"nameType": SimTypeInt(signed=False, label="IKEEXT_CERT_CRITERIA_NAME_TYPE"), "certName": SimTypePointer(SimTypeChar(label="Char"), offset=0)}, name="IKEEXT_CERT_NAME0", pack=False, align=None), offset=0), "flags": SimTypeInt(signed=False, label="UInt32")}, name="IKEEXT_CERTIFICATE_CRITERIA0", pack=False, align=None), offset=0)}, name="_Anonymous1_e__Struct", pack=False, align=None), "Anonymous2": SimStruct({"inboundEnterpriseStoreArraySize": SimTypeInt(signed=False, label="UInt32"), "inboundEnterpriseStoreCriteria": SimTypePointer(SimStruct({"certData": SimStruct({"size": SimTypeInt(signed=False, label="UInt32"), "data": SimTypePointer(SimTypeChar(label="Byte"), offset=0)}, name="FWP_BYTE_BLOB", pack=False, align=None), "certHash": SimStruct({"size": SimTypeInt(signed=False, label="UInt32"), "data": SimTypePointer(SimTypeChar(label="Byte"), offset=0)}, name="FWP_BYTE_BLOB", pack=False, align=None), "eku": SimTypePointer(SimStruct({"numEku": SimTypeInt(signed=False, label="UInt32"), "eku": SimTypePointer(SimTypePointer(SimTypeChar(label="Byte"), offset=0), offset=0)}, name="IKEEXT_CERT_EKUS0", pack=False, align=None), offset=0), "name": SimTypePointer(SimStruct({"nameType": SimTypeInt(signed=False, label="IKEEXT_CERT_CRITERIA_NAME_TYPE"), "certName": SimTypePointer(SimTypeChar(label="Char"), offset=0)}, name="IKEEXT_CERT_NAME0", pack=False, align=None), offset=0), "flags": SimTypeInt(signed=False, label="UInt32")}, name="IKEEXT_CERTIFICATE_CRITERIA0", pack=False, align=None), offset=0)}, name="_Anonymous2_e__Struct", pack=False, align=None), "Anonymous3": SimStruct({"inboundRootStoreArraySize": SimTypeInt(signed=False, label="UInt32"), "inboundTrustedRootStoreCriteria": SimTypePointer(SimStruct({"certData": SimStruct({"size": SimTypeInt(signed=False, label="UInt32"), "data": SimTypePointer(SimTypeChar(label="Byte"), offset=0)}, name="FWP_BYTE_BLOB", pack=False, align=None), "certHash": SimStruct({"size": SimTypeInt(signed=False, label="UInt32"), "data": SimTypePointer(SimTypeChar(label="Byte"), offset=0)}, name="FWP_BYTE_BLOB", pack=False, align=None), "eku": SimTypePointer(SimStruct({"numEku": SimTypeInt(signed=False, label="UInt32"), "eku": SimTypePointer(SimTypePointer(SimTypeChar(label="Byte"), offset=0), offset=0)}, name="IKEEXT_CERT_EKUS0", pack=False, align=None), offset=0), "name": SimTypePointer(SimStruct({"nameType": SimTypeInt(signed=False, label="IKEEXT_CERT_CRITERIA_NAME_TYPE"), "certName": SimTypePointer(SimTypeChar(label="Char"), offset=0)}, name="IKEEXT_CERT_NAME0", pack=False, align=None), offset=0), "flags": SimTypeInt(signed=False, label="UInt32")}, name="IKEEXT_CERTIFICATE_CRITERIA0", pack=False, align=None), offset=0)}, name="_Anonymous3_e__Struct", pack=False, align=None)}, name="<anon>", label="None"), "outboundConfigType": SimTypeInt(signed=False, label="IKEEXT_CERT_CONFIG_TYPE"), "Anonymous2": SimUnion({"Anonymous1": SimStruct({"outboundRootArraySize": SimTypeInt(signed=False, label="UInt32"), "outboundRootCriteria": SimTypePointer(SimStruct({"certData": SimStruct({"size": SimTypeInt(signed=False, label="UInt32"), "data": SimTypePointer(SimTypeChar(label="Byte"), offset=0)}, name="FWP_BYTE_BLOB", pack=False, align=None), "certHash": SimStruct({"size": SimTypeInt(signed=False, label="UInt32"), "data": SimTypePointer(SimTypeChar(label="Byte"), offset=0)}, name="FWP_BYTE_BLOB", pack=False, align=None), "eku": SimTypePointer(SimStruct({"numEku": SimTypeInt(signed=False, label="UInt32"), "eku": SimTypePointer(SimTypePointer(SimTypeChar(label="Byte"), offset=0), offset=0)}, name="IKEEXT_CERT_EKUS0", pack=False, align=None), offset=0), "name": SimTypePointer(SimStruct({"nameType": SimTypeInt(signed=False, label="IKEEXT_CERT_CRITERIA_NAME_TYPE"), "certName": SimTypePointer(SimTypeChar(label="Char"), offset=0)}, name="IKEEXT_CERT_NAME0", pack=False, align=None), offset=0), "flags": SimTypeInt(signed=False, label="UInt32")}, name="IKEEXT_CERTIFICATE_CRITERIA0", pack=False, align=None), offset=0)}, name="_Anonymous1_e__Struct", pack=False, align=None), "Anonymous2": SimStruct({"outboundEnterpriseStoreArraySize": SimTypeInt(signed=False, label="UInt32"), "outboundEnterpriseStoreCriteria": SimTypePointer(SimStruct({"certData": SimStruct({"size": SimTypeInt(signed=False, label="UInt32"), "data": SimTypePointer(SimTypeChar(label="Byte"), offset=0)}, name="FWP_BYTE_BLOB", pack=False, align=None), "certHash": SimStruct({"size": SimTypeInt(signed=False, label="UInt32"), "data": SimTypePointer(SimTypeChar(label="Byte"), offset=0)}, name="FWP_BYTE_BLOB", pack=False, align=None), "eku": SimTypePointer(SimStruct({"numEku": SimTypeInt(signed=False, label="UInt32"), "eku": SimTypePointer(SimTypePointer(SimTypeChar(label="Byte"), offset=0), offset=0)}, name="IKEEXT_CERT_EKUS0", pack=False, align=None), offset=0), "name": SimTypePointer(SimStruct({"nameType": SimTypeInt(signed=False, label="IKEEXT_CERT_CRITERIA_NAME_TYPE"), "certName": SimTypePointer(SimTypeChar(label="Char"), offset=0)}, name="IKEEXT_CERT_NAME0", pack=False, align=None), offset=0), "flags": SimTypeInt(signed=False, label="UInt32")}, name="IKEEXT_CERTIFICATE_CRITERIA0", pack=False, align=None), offset=0)}, name="_Anonymous2_e__Struct", pack=False, align=None), "Anonymous3": SimStruct({"outboundRootStoreArraySize": SimTypeInt(signed=False, label="UInt32"), "outboundTrustedRootStoreCriteria": SimTypePointer(SimStruct({"certData": SimStruct({"size": SimTypeInt(signed=False, label="UInt32"), "data": SimTypePointer(SimTypeChar(label="Byte"), offset=0)}, name="FWP_BYTE_BLOB", pack=False, align=None), "certHash": SimStruct({"size": SimTypeInt(signed=False, label="UInt32"), "data": SimTypePointer(SimTypeChar(label="Byte"), offset=0)}, name="FWP_BYTE_BLOB", pack=False, align=None), "eku": SimTypePointer(SimStruct({"numEku": SimTypeInt(signed=False, label="UInt32"), "eku": SimTypePointer(SimTypePointer(SimTypeChar(label="Byte"), offset=0), offset=0)}, name="IKEEXT_CERT_EKUS0", pack=False, align=None), offset=0), "name": SimTypePointer(SimStruct({"nameType": SimTypeInt(signed=False, label="IKEEXT_CERT_CRITERIA_NAME_TYPE"), "certName": SimTypePointer(SimTypeChar(label="Char"), offset=0)}, name="IKEEXT_CERT_NAME0", pack=False, align=None), offset=0), "flags": SimTypeInt(signed=False, label="UInt32")}, name="IKEEXT_CERTIFICATE_CRITERIA0", pack=False, align=None), offset=0)}, name="_Anonymous3_e__Struct", pack=False, align=None)}, name="<anon>", label="None"), "flags": SimTypeInt(signed=False, label="IKEEXT_CERT_AUTH"), "localCertLocationUrl": SimStruct({"size": SimTypeInt(signed=False, label="UInt32"), "data": SimTypePointer(SimTypeChar(label="Byte"), offset=0)}, name="FWP_BYTE_BLOB", pack=False, align=None)}, name="IKEEXT_CERTIFICATE_AUTHENTICATION2", pack=False, align=None), "cgaAuthentication": SimStruct({"keyContainerName": SimTypePointer(SimTypeChar(label="Char"), offset=0), "cspName": SimTypePointer(SimTypeChar(label="Char"), offset=0), "cspType": SimTypeInt(signed=False, label="UInt32"), "cgaModifier": SimStruct({"byteArray16": SimTypeFixedSizeArray(SimTypeChar(label="Byte"), 16)}, name="FWP_BYTE_ARRAY16", pack=False, align=None), "cgaCollisionCount": SimTypeChar(label="Byte")}, name="IKEEXT_IPV6_CGA_AUTHENTICATION0", pack=False, align=None), "eapAuthentication": SimStruct({"flags": SimTypeInt(signed=False, label="IKEEXT_EAP_AUTHENTICATION_FLAGS")}, name="IKEEXT_EAP_AUTHENTICATION0", pack=False, align=None)}, name="<anon>", label="None")}, name="IKEEXT_AUTHENTICATION_METHOD2", pack=False, align=None), offset=0), "initiatorImpersonationType": SimTypeInt(signed=False, label="IKEEXT_AUTHENTICATION_IMPERSONATION_TYPE"), "numIkeProposals": SimTypeInt(signed=False, label="UInt32"), "ikeProposals": SimTypePointer(SimStruct({"cipherAlgorithm": SimStruct({"algoIdentifier": SimTypeInt(signed=False, label="IKEEXT_CIPHER_TYPE"), "keyLen": SimTypeInt(signed=False, label="UInt32"), "rounds": SimTypeInt(signed=False, label="UInt32")}, name="IKEEXT_CIPHER_ALGORITHM0", pack=False, align=None), "integrityAlgorithm": SimStruct({"algoIdentifier": SimTypeInt(signed=False, label="IKEEXT_INTEGRITY_TYPE")}, name="IKEEXT_INTEGRITY_ALGORITHM0", pack=False, align=None), "maxLifetimeSeconds": SimTypeInt(signed=False, label="UInt32"), "dhGroup": SimTypeInt(signed=False, label="IKEEXT_DH_GROUP"), "quickModeLimit": SimTypeInt(signed=False, label="UInt32")}, name="IKEEXT_PROPOSAL0", pack=False, align=None), offset=0), "flags": SimTypeInt(signed=False, label="IKEEXT_POLICY_FLAG"), "maxDynamicFilters": SimTypeInt(signed=False, label="UInt32"), "retransmitDurationSecs": SimTypeInt(signed=False, label="UInt32")}, name="IKEEXT_POLICY2", pack=False, align=None), offset=0), "dataBuffer": SimTypePointer(SimStruct({"size": SimTypeInt(signed=False, label="UInt32"), "data": SimTypePointer(SimTypeChar(label="Byte"), offset=0)}, name="FWP_BYTE_BLOB", pack=False, align=None), offset=0), "classifyOptions": SimTypePointer(SimStruct({"numOptions": SimTypeInt(signed=False, label="UInt32"), "options": SimTypePointer(SimStruct({"type": SimTypeInt(signed=False, label="FWP_CLASSIFY_OPTION_TYPE"), "value": SimStruct({"type": SimTypeInt(signed=False, label="FWP_DATA_TYPE"), "Anonymous": SimUnion({"uint8": SimTypeChar(label="Byte"), "uint16": SimTypeShort(signed=False, label="UInt16"), "uint32": SimTypeInt(signed=False, label="UInt32"), "uint64": SimTypePointer(SimTypeLongLong(signed=False, label="UInt64"), offset=0), "int8": SimTypeChar(label="SByte"), "int16": SimTypeShort(signed=True, label="Int16"), "int32": SimTypeInt(signed=True, label="Int32"), "int64": SimTypePointer(SimTypeLongLong(signed=True, label="Int64"), offset=0), "float32": SimTypeFloat(size=32), "double64": SimTypePointer(SimTypeFloat(size=64), offset=0), "byteArray16": SimTypePointer(SimStruct({"byteArray16": SimTypeFixedSizeArray(SimTypeChar(label="Byte"), 16)}, name="FWP_BYTE_ARRAY16", pack=False, align=None), offset=0), "byteBlob": SimTypePointer(SimStruct({"size": SimTypeInt(signed=False, label="UInt32"), "data": SimTypePointer(SimTypeChar(label="Byte"), offset=0)}, name="FWP_BYTE_BLOB", pack=False, align=None), offset=0), "sid": SimTypePointer(SimTypeBottom(label="SID"), offset=0), "sd": SimTypePointer(SimStruct({"size": SimTypeInt(signed=False, label="UInt32"), "data": SimTypePointer(SimTypeChar(label="Byte"), offset=0)}, name="FWP_BYTE_BLOB", pack=False, align=None), offset=0), "tokenInformation": SimTypePointer(SimStruct({"sidCount": SimTypeInt(signed=False, label="UInt32"), "sids": SimTypePointer(SimTypeBottom(label="SID_AND_ATTRIBUTES"), offset=0), "restrictedSidCount": SimTypeInt(signed=False, label="UInt32"), "restrictedSids": SimTypePointer(SimTypeBottom(label="SID_AND_ATTRIBUTES"), offset=0)}, name="FWP_TOKEN_INFORMATION", pack=False, align=None), offset=0), "tokenAccessInformation": SimTypePointer(SimStruct({"size": SimTypeInt(signed=False, label="UInt32"), "data": SimTypePointer(SimTypeChar(label="Byte"), offset=0)}, name="FWP_BYTE_BLOB", pack=False, align=None), offset=0), "unicodeString": SimTypePointer(SimTypeChar(label="Char"), offset=0), "byteArray6": SimTypePointer(SimStruct({"byteArray6": SimTypeFixedSizeArray(SimTypeChar(label="Byte"), 6)}, name="FWP_BYTE_ARRAY6", pack=False, align=None), offset=0), "bitmapArray64": SimTypePointer(SimStruct({"bitmapArray64": SimTypeFixedSizeArray(SimTypeChar(label="Byte"), 8)}, name="FWP_BITMAP_ARRAY64_", pack=False, align=None), offset=0)}, name="<anon>", label="None")}, name="FWP_VALUE0", pack=False, align=None)}, name="FWPM_CLASSIFY_OPTION0", pack=False, align=None), offset=0)}, name="FWPM_CLASSIFY_OPTIONS0", pack=False, align=None), offset=0), "ikeV2QmTunnelPolicy": SimTypePointer(SimStruct({"flags": SimTypeInt(signed=False, label="UInt32"), "numIpsecProposals": SimTypeInt(signed=False, label="UInt32"), "ipsecProposals": SimTypePointer(SimStruct({"lifetime": SimStruct({"lifetimeSeconds": SimTypeInt(signed=False, label="UInt32"), "lifetimeKilobytes": SimTypeInt(signed=False, label="UInt32"), "lifetimePackets": SimTypeInt(signed=False, label="UInt32")}, name="IPSEC_SA_LIFETIME0", pack=False, align=None), "numSaTransforms": SimTypeInt(signed=False, label="UInt32"), "saTransforms": SimTypePointer(SimStruct({"ipsecTransformType": SimTypeInt(signed=False, label="IPSEC_TRANSFORM_TYPE"), "Anonymous": SimUnion({"ahTransform": SimTypePointer(SimStruct({"authTransformId": SimStruct({"authType": SimTypeInt(signed=False, label="IPSEC_AUTH_TYPE"), "authConfig": SimTypeChar(label="Byte")}, name="IPSEC_AUTH_TRANSFORM_ID0", pack=False, align=None), "cryptoModuleId": SimTypePointer(SimTypeBottom(label="Guid"), offset=0)}, name="IPSEC_AUTH_TRANSFORM0", pack=False, align=None), offset=0), "espAuthTransform": SimTypePointer(SimStruct({"authTransformId": SimStruct({"authType": SimTypeInt(signed=False, label="IPSEC_AUTH_TYPE"), "authConfig": SimTypeChar(label="Byte")}, name="IPSEC_AUTH_TRANSFORM_ID0", pack=False, align=None), "cryptoModuleId": SimTypePointer(SimTypeBottom(label="Guid"), offset=0)}, name="IPSEC_AUTH_TRANSFORM0", pack=False, align=None), offset=0), "espCipherTransform": SimTypePointer(SimStruct({"cipherTransformId": SimStruct({"cipherType": SimTypeInt(signed=False, label="IPSEC_CIPHER_TYPE"), "cipherConfig": SimTypeChar(label="Byte")}, name="IPSEC_CIPHER_TRANSFORM_ID0", pack=False, align=None), "cryptoModuleId": SimTypePointer(SimTypeBottom(label="Guid"), offset=0)}, name="IPSEC_CIPHER_TRANSFORM0", pack=False, align=None), offset=0), "espAuthAndCipherTransform": SimTypePointer(SimStruct({"authTransform": SimStruct({"authTransformId": SimStruct({"authType": SimTypeInt(signed=False, label="IPSEC_AUTH_TYPE"), "authConfig": SimTypeChar(label="Byte")}, name="IPSEC_AUTH_TRANSFORM_ID0", pack=False, align=None), "cryptoModuleId": SimTypePointer(SimTypeBottom(label="Guid"), offset=0)}, name="IPSEC_AUTH_TRANSFORM0", pack=False, align=None), "cipherTransform": SimStruct({"cipherTransformId": SimStruct({"cipherType": SimTypeInt(signed=False, label="IPSEC_CIPHER_TYPE"), "cipherConfig": SimTypeChar(label="Byte")}, name="IPSEC_CIPHER_TRANSFORM_ID0", pack=False, align=None), "cryptoModuleId": SimTypePointer(SimTypeBottom(label="Guid"), offset=0)}, name="IPSEC_CIPHER_TRANSFORM0", pack=False, align=None)}, name="IPSEC_AUTH_AND_CIPHER_TRANSFORM0", pack=False, align=None), offset=0), "espAuthFwTransform": SimTypePointer(SimStruct({"authTransformId": SimStruct({"authType": SimTypeInt(signed=False, label="IPSEC_AUTH_TYPE"), "authConfig": SimTypeChar(label="Byte")}, name="IPSEC_AUTH_TRANSFORM_ID0", pack=False, align=None), "cryptoModuleId": SimTypePointer(SimTypeBottom(label="Guid"), offset=0)}, name="IPSEC_AUTH_TRANSFORM0", pack=False, align=None), offset=0)}, name="<anon>", label="None")}, name="IPSEC_SA_TRANSFORM0", pack=False, align=None), offset=0), "pfsGroup": SimTypeInt(signed=False, label="IPSEC_PFS_GROUP")}, name="IPSEC_PROPOSAL0", pack=False, align=None), offset=0), "tunnelEndpoints": SimStruct({"ipVersion": SimTypeInt(signed=False, label="FWP_IP_VERSION"), "Anonymous1": SimUnion({"localV4Address": SimTypeInt(signed=False, label="UInt32"), "localV6Address": SimTypeFixedSizeArray(SimTypeChar(label="Byte"), 16)}, name="<anon>", label="None"), "Anonymous2": SimUnion({"remoteV4Address": SimTypeInt(signed=False, label="UInt32"), "remoteV6Address": SimTypeFixedSizeArray(SimTypeChar(label="Byte"), 16)}, name="<anon>", label="None"), "localIfLuid": SimTypeLongLong(signed=False, label="UInt64"), "remoteFqdn": SimTypePointer(SimTypeChar(label="Char"), offset=0), "numAddresses": SimTypeInt(signed=False, label="UInt32"), "remoteAddresses": SimTypePointer(SimStruct({"ipVersion": SimTypeInt(signed=False, label="FWP_IP_VERSION"), "Anonymous": SimUnion({"v4Address": SimTypeInt(signed=False, label="UInt32"), "v6Address": SimTypeFixedSizeArray(SimTypeChar(label="Byte"), 16)}, name="<anon>", label="None")}, name="IPSEC_TUNNEL_ENDPOINT0", pack=False, align=None), offset=0)}, name="IPSEC_TUNNEL_ENDPOINTS2", pack=False, align=None), "saIdleTimeout": SimStruct({"idleTimeoutSeconds": SimTypeInt(signed=False, label="UInt32"), "idleTimeoutSecondsFailOver": SimTypeInt(signed=False, label="UInt32")}, name="IPSEC_SA_IDLE_TIMEOUT0", pack=False, align=None), "emPolicy": SimTypePointer(SimStruct({"numAuthenticationMethods": SimTypeInt(signed=False, label="UInt32"), "authenticationMethods": SimTypePointer(SimStruct({"authenticationMethodType": SimTypeInt(signed=False, label="IKEEXT_AUTHENTICATION_METHOD_TYPE"), "Anonymous": SimUnion({"presharedKeyAuthentication": SimStruct({"presharedKey": SimStruct({"size": SimTypeInt(signed=False, label="UInt32"), "data": SimTypePointer(SimTypeChar(label="Byte"), offset=0)}, name="FWP_BYTE_BLOB", pack=False, align=None), "flags": SimTypeInt(signed=False, label="IKEEXT_PRESHARED_KEY_AUTHENTICATION_FLAGS")}, name="IKEEXT_PRESHARED_KEY_AUTHENTICATION1", pack=False, align=None), "certificateAuthentication": SimStruct({"inboundConfigType": SimTypeInt(signed=False, label="IKEEXT_CERT_CONFIG_TYPE"), "Anonymous1": SimUnion({"Anonymous1": SimStruct({"inboundRootArraySize": SimTypeInt(signed=False, label="UInt32"), "inboundRootCriteria": SimTypePointer(SimStruct({"certData": SimStruct({"size": SimTypeInt(signed=False, label="UInt32"), "data": SimTypePointer(SimTypeChar(label="Byte"), offset=0)}, name="FWP_BYTE_BLOB", pack=False, align=None), "certHash": SimStruct({"size": SimTypeInt(signed=False, label="UInt32"), "data": SimTypePointer(SimTypeChar(label="Byte"), offset=0)}, name="FWP_BYTE_BLOB", pack=False, align=None), "eku": SimTypePointer(SimStruct({"numEku": SimTypeInt(signed=False, label="UInt32"), "eku": SimTypePointer(SimTypePointer(SimTypeChar(label="Byte"), offset=0), offset=0)}, name="IKEEXT_CERT_EKUS0", pack=False, align=None), offset=0), "name": SimTypePointer(SimStruct({"nameType": SimTypeInt(signed=False, label="IKEEXT_CERT_CRITERIA_NAME_TYPE"), "certName": SimTypePointer(SimTypeChar(label="Char"), offset=0)}, name="IKEEXT_CERT_NAME0", pack=False, align=None), offset=0), "flags": SimTypeInt(signed=False, label="UInt32")}, name="IKEEXT_CERTIFICATE_CRITERIA0", pack=False, align=None), offset=0)}, name="_Anonymous1_e__Struct", pack=False, align=None), "Anonymous2": SimStruct({"inboundEnterpriseStoreArraySize": SimTypeInt(signed=False, label="UInt32"), "inboundEnterpriseStoreCriteria": SimTypePointer(SimStruct({"certData": SimStruct({"size": SimTypeInt(signed=False, label="UInt32"), "data": SimTypePointer(SimTypeChar(label="Byte"), offset=0)}, name="FWP_BYTE_BLOB", pack=False, align=None), "certHash": SimStruct({"size": SimTypeInt(signed=False, label="UInt32"), "data": SimTypePointer(SimTypeChar(label="Byte"), offset=0)}, name="FWP_BYTE_BLOB", pack=False, align=None), "eku": SimTypePointer(SimStruct({"numEku": SimTypeInt(signed=False, label="UInt32"), "eku": SimTypePointer(SimTypePointer(SimTypeChar(label="Byte"), offset=0), offset=0)}, name="IKEEXT_CERT_EKUS0", pack=False, align=None), offset=0), "name": SimTypePointer(SimStruct({"nameType": SimTypeInt(signed=False, label="IKEEXT_CERT_CRITERIA_NAME_TYPE"), "certName": SimTypePointer(SimTypeChar(label="Char"), offset=0)}, name="IKEEXT_CERT_NAME0", pack=False, align=None), offset=0), "flags": SimTypeInt(signed=False, label="UInt32")}, name="IKEEXT_CERTIFICATE_CRITERIA0", pack=False, align=None), offset=0)}, name="_Anonymous2_e__Struct", pack=False, align=None), "Anonymous3": SimStruct({"inboundRootStoreArraySize": SimTypeInt(signed=False, label="UInt32"), "inboundTrustedRootStoreCriteria": SimTypePointer(SimStruct({"certData": SimStruct({"size": SimTypeInt(signed=False, label="UInt32"), "data": SimTypePointer(SimTypeChar(label="Byte"), offset=0)}, name="FWP_BYTE_BLOB", pack=False, align=None), "certHash": SimStruct({"size": SimTypeInt(signed=False, label="UInt32"), "data": SimTypePointer(SimTypeChar(label="Byte"), offset=0)}, name="FWP_BYTE_BLOB", pack=False, align=None), "eku": SimTypePointer(SimStruct({"numEku": SimTypeInt(signed=False, label="UInt32"), "eku": SimTypePointer(SimTypePointer(SimTypeChar(label="Byte"), offset=0), offset=0)}, name="IKEEXT_CERT_EKUS0", pack=False, align=None), offset=0), "name": SimTypePointer(SimStruct({"nameType": SimTypeInt(signed=False, label="IKEEXT_CERT_CRITERIA_NAME_TYPE"), "certName": SimTypePointer(SimTypeChar(label="Char"), offset=0)}, name="IKEEXT_CERT_NAME0", pack=False, align=None), offset=0), "flags": SimTypeInt(signed=False, label="UInt32")}, name="IKEEXT_CERTIFICATE_CRITERIA0", pack=False, align=None), offset=0)}, name="_Anonymous3_e__Struct", pack=False, align=None)}, name="<anon>", label="None"), "outboundConfigType": SimTypeInt(signed=False, label="IKEEXT_CERT_CONFIG_TYPE"), "Anonymous2": SimUnion({"Anonymous1": SimStruct({"outboundRootArraySize": SimTypeInt(signed=False, label="UInt32"), "outboundRootCriteria": SimTypePointer(SimStruct({"certData": SimStruct({"size": SimTypeInt(signed=False, label="UInt32"), "data": SimTypePointer(SimTypeChar(label="Byte"), offset=0)}, name="FWP_BYTE_BLOB", pack=False, align=None), "certHash": SimStruct({"size": SimTypeInt(signed=False, label="UInt32"), "data": SimTypePointer(SimTypeChar(label="Byte"), offset=0)}, name="FWP_BYTE_BLOB", pack=False, align=None), "eku": SimTypePointer(SimStruct({"numEku": SimTypeInt(signed=False, label="UInt32"), "eku": SimTypePointer(SimTypePointer(SimTypeChar(label="Byte"), offset=0), offset=0)}, name="IKEEXT_CERT_EKUS0", pack=False, align=None), offset=0), "name": SimTypePointer(SimStruct({"nameType": SimTypeInt(signed=False, label="IKEEXT_CERT_CRITERIA_NAME_TYPE"), "certName": SimTypePointer(SimTypeChar(label="Char"), offset=0)}, name="IKEEXT_CERT_NAME0", pack=False, align=None), offset=0), "flags": SimTypeInt(signed=False, label="UInt32")}, name="IKEEXT_CERTIFICATE_CRITERIA0", pack=False, align=None), offset=0)}, name="_Anonymous1_e__Struct", pack=False, align=None), "Anonymous2": SimStruct({"outboundEnterpriseStoreArraySize": SimTypeInt(signed=False, label="UInt32"), "outboundEnterpriseStoreCriteria": SimTypePointer(SimStruct({"certData": SimStruct({"size": SimTypeInt(signed=False, label="UInt32"), "data": SimTypePointer(SimTypeChar(label="Byte"), offset=0)}, name="FWP_BYTE_BLOB", pack=False, align=None), "certHash": SimStruct({"size": SimTypeInt(signed=False, label="UInt32"), "data": SimTypePointer(SimTypeChar(label="Byte"), offset=0)}, name="FWP_BYTE_BLOB", pack=False, align=None), "eku": SimTypePointer(SimStruct({"numEku": SimTypeInt(signed=False, label="UInt32"), "eku": SimTypePointer(SimTypePointer(SimTypeChar(label="Byte"), offset=0), offset=0)}, name="IKEEXT_CERT_EKUS0", pack=False, align=None), offset=0), "name": SimTypePointer(SimStruct({"nameType": SimTypeInt(signed=False, label="IKEEXT_CERT_CRITERIA_NAME_TYPE"), "certName": SimTypePointer(SimTypeChar(label="Char"), offset=0)}, name="IKEEXT_CERT_NAME0", pack=False, align=None), offset=0), "flags": SimTypeInt(signed=False, label="UInt32")}, name="IKEEXT_CERTIFICATE_CRITERIA0", pack=False, align=None), offset=0)}, name="_Anonymous2_e__Struct", pack=False, align=None), "Anonymous3": SimStruct({"outboundRootStoreArraySize": SimTypeInt(signed=False, label="UInt32"), "outboundTrustedRootStoreCriteria": SimTypePointer(SimStruct({"certData": SimStruct({"size": SimTypeInt(signed=False, label="UInt32"), "data": SimTypePointer(SimTypeChar(label="Byte"), offset=0)}, name="FWP_BYTE_BLOB", pack=False, align=None), "certHash": SimStruct({"size": SimTypeInt(signed=False, label="UInt32"), "data": SimTypePointer(SimTypeChar(label="Byte"), offset=0)}, name="FWP_BYTE_BLOB", pack=False, align=None), "eku": SimTypePointer(SimStruct({"numEku": SimTypeInt(signed=False, label="UInt32"), "eku": SimTypePointer(SimTypePointer(SimTypeChar(label="Byte"), offset=0), offset=0)}, name="IKEEXT_CERT_EKUS0", pack=False, align=None), offset=0), "name": SimTypePointer(SimStruct({"nameType": SimTypeInt(signed=False, label="IKEEXT_CERT_CRITERIA_NAME_TYPE"), "certName": SimTypePointer(SimTypeChar(label="Char"), offset=0)}, name="IKEEXT_CERT_NAME0", pack=False, align=None), offset=0), "flags": SimTypeInt(signed=False, label="UInt32")}, name="IKEEXT_CERTIFICATE_CRITERIA0", pack=False, align=None), offset=0)}, name="_Anonymous3_e__Struct", pack=False, align=None)}, name="<anon>", label="None"), "flags": SimTypeInt(signed=False, label="IKEEXT_CERT_AUTH"), "localCertLocationUrl": SimStruct({"size": SimTypeInt(signed=False, label="UInt32"), "data": SimTypePointer(SimTypeChar(label="Byte"), offset=0)}, name="FWP_BYTE_BLOB", pack=False, align=None)}, name="IKEEXT_CERTIFICATE_AUTHENTICATION2", pack=False, align=None), "kerberosAuthentication": SimStruct({"flags": SimTypeInt(signed=False, label="IKEEXT_KERBEROS_AUTHENTICATION_FLAGS"), "proxyServer": SimTypePointer(SimTypeChar(label="Char"), offset=0)}, name="IKEEXT_KERBEROS_AUTHENTICATION1", pack=False, align=None), "reservedAuthentication": SimStruct({"flags": SimTypeInt(signed=False, label="IKEEXT_RESERVED_AUTHENTICATION_FLAGS")}, name="IKEEXT_RESERVED_AUTHENTICATION0", pack=False, align=None), "ntlmV2Authentication": SimStruct({"flags": SimTypeInt(signed=False, label="UInt32")}, name="IKEEXT_NTLM_V2_AUTHENTICATION0", pack=False, align=None), "sslAuthentication": SimStruct({"inboundConfigType": SimTypeInt(signed=False, label="IKEEXT_CERT_CONFIG_TYPE"), "Anonymous1": SimUnion({"Anonymous1": SimStruct({"inboundRootArraySize": SimTypeInt(signed=False, label="UInt32"), "inboundRootCriteria": SimTypePointer(SimStruct({"certData": SimStruct({"size": SimTypeInt(signed=False, label="UInt32"), "data": SimTypePointer(SimTypeChar(label="Byte"), offset=0)}, name="FWP_BYTE_BLOB", pack=False, align=None), "certHash": SimStruct({"size": SimTypeInt(signed=False, label="UInt32"), "data": SimTypePointer(SimTypeChar(label="Byte"), offset=0)}, name="FWP_BYTE_BLOB", pack=False, align=None), "eku": SimTypePointer(SimStruct({"numEku": SimTypeInt(signed=False, label="UInt32"), "eku": SimTypePointer(SimTypePointer(SimTypeChar(label="Byte"), offset=0), offset=0)}, name="IKEEXT_CERT_EKUS0", pack=False, align=None), offset=0), "name": SimTypePointer(SimStruct({"nameType": SimTypeInt(signed=False, label="IKEEXT_CERT_CRITERIA_NAME_TYPE"), "certName": SimTypePointer(SimTypeChar(label="Char"), offset=0)}, name="IKEEXT_CERT_NAME0", pack=False, align=None), offset=0), "flags": SimTypeInt(signed=False, label="UInt32")}, name="IKEEXT_CERTIFICATE_CRITERIA0", pack=False, align=None), offset=0)}, name="_Anonymous1_e__Struct", pack=False, align=None), "Anonymous2": SimStruct({"inboundEnterpriseStoreArraySize": SimTypeInt(signed=False, label="UInt32"), "inboundEnterpriseStoreCriteria": SimTypePointer(SimStruct({"certData": SimStruct({"size": SimTypeInt(signed=False, label="UInt32"), "data": SimTypePointer(SimTypeChar(label="Byte"), offset=0)}, name="FWP_BYTE_BLOB", pack=False, align=None), "certHash": SimStruct({"size": SimTypeInt(signed=False, label="UInt32"), "data": SimTypePointer(SimTypeChar(label="Byte"), offset=0)}, name="FWP_BYTE_BLOB", pack=False, align=None), "eku": SimTypePointer(SimStruct({"numEku": SimTypeInt(signed=False, label="UInt32"), "eku": SimTypePointer(SimTypePointer(SimTypeChar(label="Byte"), offset=0), offset=0)}, name="IKEEXT_CERT_EKUS0", pack=False, align=None), offset=0), "name": SimTypePointer(SimStruct({"nameType": SimTypeInt(signed=False, label="IKEEXT_CERT_CRITERIA_NAME_TYPE"), "certName": SimTypePointer(SimTypeChar(label="Char"), offset=0)}, name="IKEEXT_CERT_NAME0", pack=False, align=None), offset=0), "flags": SimTypeInt(signed=False, label="UInt32")}, name="IKEEXT_CERTIFICATE_CRITERIA0", pack=False, align=None), offset=0)}, name="_Anonymous2_e__Struct", pack=False, align=None), "Anonymous3": SimStruct({"inboundRootStoreArraySize": SimTypeInt(signed=False, label="UInt32"), "inboundTrustedRootStoreCriteria": SimTypePointer(SimStruct({"certData": SimStruct({"size": SimTypeInt(signed=False, label="UInt32"), "data": SimTypePointer(SimTypeChar(label="Byte"), offset=0)}, name="FWP_BYTE_BLOB", pack=False, align=None), "certHash": SimStruct({"size": SimTypeInt(signed=False, label="UInt32"), "data": SimTypePointer(SimTypeChar(label="Byte"), offset=0)}, name="FWP_BYTE_BLOB", pack=False, align=None), "eku": SimTypePointer(SimStruct({"numEku": SimTypeInt(signed=False, label="UInt32"), "eku": SimTypePointer(SimTypePointer(SimTypeChar(label="Byte"), offset=0), offset=0)}, name="IKEEXT_CERT_EKUS0", pack=False, align=None), offset=0), "name": SimTypePointer(SimStruct({"nameType": SimTypeInt(signed=False, label="IKEEXT_CERT_CRITERIA_NAME_TYPE"), "certName": SimTypePointer(SimTypeChar(label="Char"), offset=0)}, name="IKEEXT_CERT_NAME0", pack=False, align=None), offset=0), "flags": SimTypeInt(signed=False, label="UInt32")}, name="IKEEXT_CERTIFICATE_CRITERIA0", pack=False, align=None), offset=0)}, name="_Anonymous3_e__Struct", pack=False, align=None)}, name="<anon>", label="None"), "outboundConfigType": SimTypeInt(signed=False, label="IKEEXT_CERT_CONFIG_TYPE"), "Anonymous2": SimUnion({"Anonymous1": SimStruct({"outboundRootArraySize": SimTypeInt(signed=False, label="UInt32"), "outboundRootCriteria": SimTypePointer(SimStruct({"certData": SimStruct({"size": SimTypeInt(signed=False, label="UInt32"), "data": SimTypePointer(SimTypeChar(label="Byte"), offset=0)}, name="FWP_BYTE_BLOB", pack=False, align=None), "certHash": SimStruct({"size": SimTypeInt(signed=False, label="UInt32"), "data": SimTypePointer(SimTypeChar(label="Byte"), offset=0)}, name="FWP_BYTE_BLOB", pack=False, align=None), "eku": SimTypePointer(SimStruct({"numEku": SimTypeInt(signed=False, label="UInt32"), "eku": SimTypePointer(SimTypePointer(SimTypeChar(label="Byte"), offset=0), offset=0)}, name="IKEEXT_CERT_EKUS0", pack=False, align=None), offset=0), "name": SimTypePointer(SimStruct({"nameType": SimTypeInt(signed=False, label="IKEEXT_CERT_CRITERIA_NAME_TYPE"), "certName": SimTypePointer(SimTypeChar(label="Char"), offset=0)}, name="IKEEXT_CERT_NAME0", pack=False, align=None), offset=0), "flags": SimTypeInt(signed=False, label="UInt32")}, name="IKEEXT_CERTIFICATE_CRITERIA0", pack=False, align=None), offset=0)}, name="_Anonymous1_e__Struct", pack=False, align=None), "Anonymous2": SimStruct({"outboundEnterpriseStoreArraySize": SimTypeInt(signed=False, label="UInt32"), "outboundEnterpriseStoreCriteria": SimTypePointer(SimStruct({"certData": SimStruct({"size": SimTypeInt(signed=False, label="UInt32"), "data": SimTypePointer(SimTypeChar(label="Byte"), offset=0)}, name="FWP_BYTE_BLOB", pack=False, align=None), "certHash": SimStruct({"size": SimTypeInt(signed=False, label="UInt32"), "data": SimTypePointer(SimTypeChar(label="Byte"), offset=0)}, name="FWP_BYTE_BLOB", pack=False, align=None), "eku": SimTypePointer(SimStruct({"numEku": SimTypeInt(signed=False, label="UInt32"), "eku": SimTypePointer(SimTypePointer(SimTypeChar(label="Byte"), offset=0), offset=0)}, name="IKEEXT_CERT_EKUS0", pack=False, align=None), offset=0), "name": SimTypePointer(SimStruct({"nameType": SimTypeInt(signed=False, label="IKEEXT_CERT_CRITERIA_NAME_TYPE"), "certName": SimTypePointer(SimTypeChar(label="Char"), offset=0)}, name="IKEEXT_CERT_NAME0", pack=False, align=None), offset=0), "flags": SimTypeInt(signed=False, label="UInt32")}, name="IKEEXT_CERTIFICATE_CRITERIA0", pack=False, align=None), offset=0)}, name="_Anonymous2_e__Struct", pack=False, align=None), "Anonymous3": SimStruct({"outboundRootStoreArraySize": SimTypeInt(signed=False, label="UInt32"), "outboundTrustedRootStoreCriteria": SimTypePointer(SimStruct({"certData": SimStruct({"size": SimTypeInt(signed=False, label="UInt32"), "data": SimTypePointer(SimTypeChar(label="Byte"), offset=0)}, name="FWP_BYTE_BLOB", pack=False, align=None), "certHash": SimStruct({"size": SimTypeInt(signed=False, label="UInt32"), "data": SimTypePointer(SimTypeChar(label="Byte"), offset=0)}, name="FWP_BYTE_BLOB", pack=False, align=None), "eku": SimTypePointer(SimStruct({"numEku": SimTypeInt(signed=False, label="UInt32"), "eku": SimTypePointer(SimTypePointer(SimTypeChar(label="Byte"), offset=0), offset=0)}, name="IKEEXT_CERT_EKUS0", pack=False, align=None), offset=0), "name": SimTypePointer(SimStruct({"nameType": SimTypeInt(signed=False, label="IKEEXT_CERT_CRITERIA_NAME_TYPE"), "certName": SimTypePointer(SimTypeChar(label="Char"), offset=0)}, name="IKEEXT_CERT_NAME0", pack=False, align=None), offset=0), "flags": SimTypeInt(signed=False, label="UInt32")}, name="IKEEXT_CERTIFICATE_CRITERIA0", pack=False, align=None), offset=0)}, name="_Anonymous3_e__Struct", pack=False, align=None)}, name="<anon>", label="None"), "flags": SimTypeInt(signed=False, label="IKEEXT_CERT_AUTH"), "localCertLocationUrl": SimStruct({"size": SimTypeInt(signed=False, label="UInt32"), "data": SimTypePointer(SimTypeChar(label="Byte"), offset=0)}, name="FWP_BYTE_BLOB", pack=False, align=None)}, name="IKEEXT_CERTIFICATE_AUTHENTICATION2", pack=False, align=None), "cgaAuthentication": SimStruct({"keyContainerName": SimTypePointer(SimTypeChar(label="Char"), offset=0), "cspName": SimTypePointer(SimTypeChar(label="Char"), offset=0), "cspType": SimTypeInt(signed=False, label="UInt32"), "cgaModifier": SimStruct({"byteArray16": SimTypeFixedSizeArray(SimTypeChar(label="Byte"), 16)}, name="FWP_BYTE_ARRAY16", pack=False, align=None), "cgaCollisionCount": SimTypeChar(label="Byte")}, name="IKEEXT_IPV6_CGA_AUTHENTICATION0", pack=False, align=None), "eapAuthentication": SimStruct({"flags": SimTypeInt(signed=False, label="IKEEXT_EAP_AUTHENTICATION_FLAGS")}, name="IKEEXT_EAP_AUTHENTICATION0", pack=False, align=None)}, name="<anon>", label="None")}, name="IKEEXT_AUTHENTICATION_METHOD2", pack=False, align=None), offset=0), "initiatorImpersonationType": SimTypeInt(signed=False, label="IKEEXT_AUTHENTICATION_IMPERSONATION_TYPE")}, name="IKEEXT_EM_POLICY2", pack=False, align=None), offset=0), "fwdPathSaLifetime": SimTypeInt(signed=False, label="UInt32"), "compartmentId": SimTypeInt(signed=False, label="UInt32"), "numTrafficSelectorPolicy": SimTypeInt(signed=False, label="UInt32"), "trafficSelectorPolicies": SimTypePointer(SimStruct({"flags": SimTypeInt(signed=False, label="UInt32"), "numLocalTrafficSelectors": SimTypeInt(signed=False, label="UInt32"), "localTrafficSelectors": SimTypePointer(SimStruct({"protocolId": SimTypeChar(label="Byte"), "portStart": SimTypeShort(signed=False, label="UInt16"), "portEnd": SimTypeShort(signed=False, label="UInt16"), "ipVersion": SimTypeInt(signed=False, label="FWP_IP_VERSION"), "Anonymous1": SimUnion({"startV4Address": SimTypeInt(signed=False, label="UInt32"), "startV6Address": SimTypeFixedSizeArray(SimTypeChar(label="Byte"), 16)}, name="<anon>", label="None"), "Anonymous2": SimUnion({"endV4Address": SimTypeInt(signed=False, label="UInt32"), "endV6Address": SimTypeFixedSizeArray(SimTypeChar(label="Byte"), 16)}, name="<anon>", label="None")}, name="IPSEC_TRAFFIC_SELECTOR0_", pack=False, align=None), offset=0), "numRemoteTrafficSelectors": SimTypeInt(signed=False, label="UInt32"), "remoteTrafficSelectors": SimTypePointer(SimStruct({"protocolId": SimTypeChar(label="Byte"), "portStart": SimTypeShort(signed=False, label="UInt16"), "portEnd": SimTypeShort(signed=False, label="UInt16"), "ipVersion": SimTypeInt(signed=False, label="FWP_IP_VERSION"), "Anonymous1": SimUnion({"startV4Address": SimTypeInt(signed=False, label="UInt32"), "startV6Address": SimTypeFixedSizeArray(SimTypeChar(label="Byte"), 16)}, name="<anon>", label="None"), "Anonymous2": SimUnion({"endV4Address": SimTypeInt(signed=False, label="UInt32"), "endV6Address": SimTypeFixedSizeArray(SimTypeChar(label="Byte"), 16)}, name="<anon>", label="None")}, name="IPSEC_TRAFFIC_SELECTOR0_", pack=False, align=None), offset=0)}, name="IPSEC_TRAFFIC_SELECTOR_POLICY0_", pack=False, align=None), offset=0)}, name="IPSEC_TUNNEL_POLICY3_", pack=False, align=None), offset=0), "ikeV2QmTransportPolicy": SimTypePointer(SimStruct({"numIpsecProposals": SimTypeInt(signed=False, label="UInt32"), "ipsecProposals": SimTypePointer(SimStruct({"lifetime": SimStruct({"lifetimeSeconds": SimTypeInt(signed=False, label="UInt32"), "lifetimeKilobytes": SimTypeInt(signed=False, label="UInt32"), "lifetimePackets": SimTypeInt(signed=False, label="UInt32")}, name="IPSEC_SA_LIFETIME0", pack=False, align=None), "numSaTransforms": SimTypeInt(signed=False, label="UInt32"), "saTransforms": SimTypePointer(SimStruct({"ipsecTransformType": SimTypeInt(signed=False, label="IPSEC_TRANSFORM_TYPE"), "Anonymous": SimUnion({"ahTransform": SimTypePointer(SimStruct({"authTransformId": SimStruct({"authType": SimTypeInt(signed=False, label="IPSEC_AUTH_TYPE"), "authConfig": SimTypeChar(label="Byte")}, name="IPSEC_AUTH_TRANSFORM_ID0", pack=False, align=None), "cryptoModuleId": SimTypePointer(SimTypeBottom(label="Guid"), offset=0)}, name="IPSEC_AUTH_TRANSFORM0", pack=False, align=None), offset=0), "espAuthTransform": SimTypePointer(SimStruct({"authTransformId": SimStruct({"authType": SimTypeInt(signed=False, label="IPSEC_AUTH_TYPE"), "authConfig": SimTypeChar(label="Byte")}, name="IPSEC_AUTH_TRANSFORM_ID0", pack=False, align=None), "cryptoModuleId": SimTypePointer(SimTypeBottom(label="Guid"), offset=0)}, name="IPSEC_AUTH_TRANSFORM0", pack=False, align=None), offset=0), "espCipherTransform": SimTypePointer(SimStruct({"cipherTransformId": SimStruct({"cipherType": SimTypeInt(signed=False, label="IPSEC_CIPHER_TYPE"), "cipherConfig": SimTypeChar(label="Byte")}, name="IPSEC_CIPHER_TRANSFORM_ID0", pack=False, align=None), "cryptoModuleId": SimTypePointer(SimTypeBottom(label="Guid"), offset=0)}, name="IPSEC_CIPHER_TRANSFORM0", pack=False, align=None), offset=0), "espAuthAndCipherTransform": SimTypePointer(SimStruct({"authTransform": SimStruct({"authTransformId": SimStruct({"authType": SimTypeInt(signed=False, label="IPSEC_AUTH_TYPE"), "authConfig": SimTypeChar(label="Byte")}, name="IPSEC_AUTH_TRANSFORM_ID0", pack=False, align=None), "cryptoModuleId": SimTypePointer(SimTypeBottom(label="Guid"), offset=0)}, name="IPSEC_AUTH_TRANSFORM0", pack=False, align=None), "cipherTransform": SimStruct({"cipherTransformId": SimStruct({"cipherType": SimTypeInt(signed=False, label="IPSEC_CIPHER_TYPE"), "cipherConfig": SimTypeChar(label="Byte")}, name="IPSEC_CIPHER_TRANSFORM_ID0", pack=False, align=None), "cryptoModuleId": SimTypePointer(SimTypeBottom(label="Guid"), offset=0)}, name="IPSEC_CIPHER_TRANSFORM0", pack=False, align=None)}, name="IPSEC_AUTH_AND_CIPHER_TRANSFORM0", pack=False, align=None), offset=0), "espAuthFwTransform": SimTypePointer(SimStruct({"authTransformId": SimStruct({"authType": SimTypeInt(signed=False, label="IPSEC_AUTH_TYPE"), "authConfig": SimTypeChar(label="Byte")}, name="IPSEC_AUTH_TRANSFORM_ID0", pack=False, align=None), "cryptoModuleId": SimTypePointer(SimTypeBottom(label="Guid"), offset=0)}, name="IPSEC_AUTH_TRANSFORM0", pack=False, align=None), offset=0)}, name="<anon>", label="None")}, name="IPSEC_SA_TRANSFORM0", pack=False, align=None), offset=0), "pfsGroup": SimTypeInt(signed=False, label="IPSEC_PFS_GROUP")}, name="IPSEC_PROPOSAL0", pack=False, align=None), offset=0), "flags": SimTypeInt(signed=False, label="IPSEC_POLICY_FLAG"), "ndAllowClearTimeoutSeconds": SimTypeInt(signed=False, label="UInt32"), "saIdleTimeout": SimStruct({"idleTimeoutSeconds": SimTypeInt(signed=False, label="UInt32"), "idleTimeoutSecondsFailOver": SimTypeInt(signed=False, label="UInt32")}, name="IPSEC_SA_IDLE_TIMEOUT0", pack=False, align=None), "emPolicy": SimTypePointer(SimStruct({"numAuthenticationMethods": SimTypeInt(signed=False, label="UInt32"), "authenticationMethods": SimTypePointer(SimStruct({"authenticationMethodType": SimTypeInt(signed=False, label="IKEEXT_AUTHENTICATION_METHOD_TYPE"), "Anonymous": SimUnion({"presharedKeyAuthentication": SimStruct({"presharedKey": SimStruct({"size": SimTypeInt(signed=False, label="UInt32"), "data": SimTypePointer(SimTypeChar(label="Byte"), offset=0)}, name="FWP_BYTE_BLOB", pack=False, align=None), "flags": SimTypeInt(signed=False, label="IKEEXT_PRESHARED_KEY_AUTHENTICATION_FLAGS")}, name="IKEEXT_PRESHARED_KEY_AUTHENTICATION1", pack=False, align=None), "certificateAuthentication": SimStruct({"inboundConfigType": SimTypeInt(signed=False, label="IKEEXT_CERT_CONFIG_TYPE"), "Anonymous1": SimUnion({"Anonymous1": SimStruct({"inboundRootArraySize": SimTypeInt(signed=False, label="UInt32"), "inboundRootCriteria": SimTypePointer(SimStruct({"certData": SimStruct({"size": SimTypeInt(signed=False, label="UInt32"), "data": SimTypePointer(SimTypeChar(label="Byte"), offset=0)}, name="FWP_BYTE_BLOB", pack=False, align=None), "certHash": SimStruct({"size": SimTypeInt(signed=False, label="UInt32"), "data": SimTypePointer(SimTypeChar(label="Byte"), offset=0)}, name="FWP_BYTE_BLOB", pack=False, align=None), "eku": SimTypePointer(SimStruct({"numEku": SimTypeInt(signed=False, label="UInt32"), "eku": SimTypePointer(SimTypePointer(SimTypeChar(label="Byte"), offset=0), offset=0)}, name="IKEEXT_CERT_EKUS0", pack=False, align=None), offset=0), "name": SimTypePointer(SimStruct({"nameType": SimTypeInt(signed=False, label="IKEEXT_CERT_CRITERIA_NAME_TYPE"), "certName": SimTypePointer(SimTypeChar(label="Char"), offset=0)}, name="IKEEXT_CERT_NAME0", pack=False, align=None), offset=0), "flags": SimTypeInt(signed=False, label="UInt32")}, name="IKEEXT_CERTIFICATE_CRITERIA0", pack=False, align=None), offset=0)}, name="_Anonymous1_e__Struct", pack=False, align=None), "Anonymous2": SimStruct({"inboundEnterpriseStoreArraySize": SimTypeInt(signed=False, label="UInt32"), "inboundEnterpriseStoreCriteria": SimTypePointer(SimStruct({"certData": SimStruct({"size": SimTypeInt(signed=False, label="UInt32"), "data": SimTypePointer(SimTypeChar(label="Byte"), offset=0)}, name="FWP_BYTE_BLOB", pack=False, align=None), "certHash": SimStruct({"size": SimTypeInt(signed=False, label="UInt32"), "data": SimTypePointer(SimTypeChar(label="Byte"), offset=0)}, name="FWP_BYTE_BLOB", pack=False, align=None), "eku": SimTypePointer(SimStruct({"numEku": SimTypeInt(signed=False, label="UInt32"), "eku": SimTypePointer(SimTypePointer(SimTypeChar(label="Byte"), offset=0), offset=0)}, name="IKEEXT_CERT_EKUS0", pack=False, align=None), offset=0), "name": SimTypePointer(SimStruct({"nameType": SimTypeInt(signed=False, label="IKEEXT_CERT_CRITERIA_NAME_TYPE"), "certName": SimTypePointer(SimTypeChar(label="Char"), offset=0)}, name="IKEEXT_CERT_NAME0", pack=False, align=None), offset=0), "flags": SimTypeInt(signed=False, label="UInt32")}, name="IKEEXT_CERTIFICATE_CRITERIA0", pack=False, align=None), offset=0)}, name="_Anonymous2_e__Struct", pack=False, align=None), "Anonymous3": SimStruct({"inboundRootStoreArraySize": SimTypeInt(signed=False, label="UInt32"), "inboundTrustedRootStoreCriteria": SimTypePointer(SimStruct({"certData": SimStruct({"size": SimTypeInt(signed=False, label="UInt32"), "data": SimTypePointer(SimTypeChar(label="Byte"), offset=0)}, name="FWP_BYTE_BLOB", pack=False, align=None), "certHash": SimStruct({"size": SimTypeInt(signed=False, label="UInt32"), "data": SimTypePointer(SimTypeChar(label="Byte"), offset=0)}, name="FWP_BYTE_BLOB", pack=False, align=None), "eku": SimTypePointer(SimStruct({"numEku": SimTypeInt(signed=False, label="UInt32"), "eku": SimTypePointer(SimTypePointer(SimTypeChar(label="Byte"), offset=0), offset=0)}, name="IKEEXT_CERT_EKUS0", pack=False, align=None), offset=0), "name": SimTypePointer(SimStruct({"nameType": SimTypeInt(signed=False, label="IKEEXT_CERT_CRITERIA_NAME_TYPE"), "certName": SimTypePointer(SimTypeChar(label="Char"), offset=0)}, name="IKEEXT_CERT_NAME0", pack=False, align=None), offset=0), "flags": SimTypeInt(signed=False, label="UInt32")}, name="IKEEXT_CERTIFICATE_CRITERIA0", pack=False, align=None), offset=0)}, name="_Anonymous3_e__Struct", pack=False, align=None)}, name="<anon>", label="None"), "outboundConfigType": SimTypeInt(signed=False, label="IKEEXT_CERT_CONFIG_TYPE"), "Anonymous2": SimUnion({"Anonymous1": SimStruct({"outboundRootArraySize": SimTypeInt(signed=False, label="UInt32"), "outboundRootCriteria": SimTypePointer(SimStruct({"certData": SimStruct({"size": SimTypeInt(signed=False, label="UInt32"), "data": SimTypePointer(SimTypeChar(label="Byte"), offset=0)}, name="FWP_BYTE_BLOB", pack=False, align=None), "certHash": SimStruct({"size": SimTypeInt(signed=False, label="UInt32"), "data": SimTypePointer(SimTypeChar(label="Byte"), offset=0)}, name="FWP_BYTE_BLOB", pack=False, align=None), "eku": SimTypePointer(SimStruct({"numEku": SimTypeInt(signed=False, label="UInt32"), "eku": SimTypePointer(SimTypePointer(SimTypeChar(label="Byte"), offset=0), offset=0)}, name="IKEEXT_CERT_EKUS0", pack=False, align=None), offset=0), "name": SimTypePointer(SimStruct({"nameType": SimTypeInt(signed=False, label="IKEEXT_CERT_CRITERIA_NAME_TYPE"), "certName": SimTypePointer(SimTypeChar(label="Char"), offset=0)}, name="IKEEXT_CERT_NAME0", pack=False, align=None), offset=0), "flags": SimTypeInt(signed=False, label="UInt32")}, name="IKEEXT_CERTIFICATE_CRITERIA0", pack=False, align=None), offset=0)}, name="_Anonymous1_e__Struct", pack=False, align=None), "Anonymous2": SimStruct({"outboundEnterpriseStoreArraySize": SimTypeInt(signed=False, label="UInt32"), "outboundEnterpriseStoreCriteria": SimTypePointer(SimStruct({"certData": SimStruct({"size": SimTypeInt(signed=False, label="UInt32"), "data": SimTypePointer(SimTypeChar(label="Byte"), offset=0)}, name="FWP_BYTE_BLOB", pack=False, align=None), "certHash": SimStruct({"size": SimTypeInt(signed=False, label="UInt32"), "data": SimTypePointer(SimTypeChar(label="Byte"), offset=0)}, name="FWP_BYTE_BLOB", pack=False, align=None), "eku": SimTypePointer(SimStruct({"numEku": SimTypeInt(signed=False, label="UInt32"), "eku": SimTypePointer(SimTypePointer(SimTypeChar(label="Byte"), offset=0), offset=0)}, name="IKEEXT_CERT_EKUS0", pack=False, align=None), offset=0), "name": SimTypePointer(SimStruct({"nameType": SimTypeInt(signed=False, label="IKEEXT_CERT_CRITERIA_NAME_TYPE"), "certName": SimTypePointer(SimTypeChar(label="Char"), offset=0)}, name="IKEEXT_CERT_NAME0", pack=False, align=None), offset=0), "flags": SimTypeInt(signed=False, label="UInt32")}, name="IKEEXT_CERTIFICATE_CRITERIA0", pack=False, align=None), offset=0)}, name="_Anonymous2_e__Struct", pack=False, align=None), "Anonymous3": SimStruct({"outboundRootStoreArraySize": SimTypeInt(signed=False, label="UInt32"), "outboundTrustedRootStoreCriteria": SimTypePointer(SimStruct({"certData": SimStruct({"size": SimTypeInt(signed=False, label="UInt32"), "data": SimTypePointer(SimTypeChar(label="Byte"), offset=0)}, name="FWP_BYTE_BLOB", pack=False, align=None), "certHash": SimStruct({"size": SimTypeInt(signed=False, label="UInt32"), "data": SimTypePointer(SimTypeChar(label="Byte"), offset=0)}, name="FWP_BYTE_BLOB", pack=False, align=None), "eku": SimTypePointer(SimStruct({"numEku": SimTypeInt(signed=False, label="UInt32"), "eku": SimTypePointer(SimTypePointer(SimTypeChar(label="Byte"), offset=0), offset=0)}, name="IKEEXT_CERT_EKUS0", pack=False, align=None), offset=0), "name": SimTypePointer(SimStruct({"nameType": SimTypeInt(signed=False, label="IKEEXT_CERT_CRITERIA_NAME_TYPE"), "certName": SimTypePointer(SimTypeChar(label="Char"), offset=0)}, name="IKEEXT_CERT_NAME0", pack=False, align=None), offset=0), "flags": SimTypeInt(signed=False, label="UInt32")}, name="IKEEXT_CERTIFICATE_CRITERIA0", pack=False, align=None), offset=0)}, name="_Anonymous3_e__Struct", pack=False, align=None)}, name="<anon>", label="None"), "flags": SimTypeInt(signed=False, label="IKEEXT_CERT_AUTH"), "localCertLocationUrl": SimStruct({"size": SimTypeInt(signed=False, label="UInt32"), "data": SimTypePointer(SimTypeChar(label="Byte"), offset=0)}, name="FWP_BYTE_BLOB", pack=False, align=None)}, name="IKEEXT_CERTIFICATE_AUTHENTICATION2", pack=False, align=None), "kerberosAuthentication": SimStruct({"flags": SimTypeInt(signed=False, label="IKEEXT_KERBEROS_AUTHENTICATION_FLAGS"), "proxyServer": SimTypePointer(SimTypeChar(label="Char"), offset=0)}, name="IKEEXT_KERBEROS_AUTHENTICATION1", pack=False, align=None), "reservedAuthentication": SimStruct({"flags": SimTypeInt(signed=False, label="IKEEXT_RESERVED_AUTHENTICATION_FLAGS")}, name="IKEEXT_RESERVED_AUTHENTICATION0", pack=False, align=None), "ntlmV2Authentication": SimStruct({"flags": SimTypeInt(signed=False, label="UInt32")}, name="IKEEXT_NTLM_V2_AUTHENTICATION0", pack=False, align=None), "sslAuthentication": SimStruct({"inboundConfigType": SimTypeInt(signed=False, label="IKEEXT_CERT_CONFIG_TYPE"), "Anonymous1": SimUnion({"Anonymous1": SimStruct({"inboundRootArraySize": SimTypeInt(signed=False, label="UInt32"), "inboundRootCriteria": SimTypePointer(SimStruct({"certData": SimStruct({"size": SimTypeInt(signed=False, label="UInt32"), "data": SimTypePointer(SimTypeChar(label="Byte"), offset=0)}, name="FWP_BYTE_BLOB", pack=False, align=None), "certHash": SimStruct({"size": SimTypeInt(signed=False, label="UInt32"), "data": SimTypePointer(SimTypeChar(label="Byte"), offset=0)}, name="FWP_BYTE_BLOB", pack=False, align=None), "eku": SimTypePointer(SimStruct({"numEku": SimTypeInt(signed=False, label="UInt32"), "eku": SimTypePointer(SimTypePointer(SimTypeChar(label="Byte"), offset=0), offset=0)}, name="IKEEXT_CERT_EKUS0", pack=False, align=None), offset=0), "name": SimTypePointer(SimStruct({"nameType": SimTypeInt(signed=False, label="IKEEXT_CERT_CRITERIA_NAME_TYPE"), "certName": SimTypePointer(SimTypeChar(label="Char"), offset=0)}, name="IKEEXT_CERT_NAME0", pack=False, align=None), offset=0), "flags": SimTypeInt(signed=False, label="UInt32")}, name="IKEEXT_CERTIFICATE_CRITERIA0", pack=False, align=None), offset=0)}, name="_Anonymous1_e__Struct", pack=False, align=None), "Anonymous2": SimStruct({"inboundEnterpriseStoreArraySize": SimTypeInt(signed=False, label="UInt32"), "inboundEnterpriseStoreCriteria": SimTypePointer(SimStruct({"certData": SimStruct({"size": SimTypeInt(signed=False, label="UInt32"), "data": SimTypePointer(SimTypeChar(label="Byte"), offset=0)}, name="FWP_BYTE_BLOB", pack=False, align=None), "certHash": SimStruct({"size": SimTypeInt(signed=False, label="UInt32"), "data": SimTypePointer(SimTypeChar(label="Byte"), offset=0)}, name="FWP_BYTE_BLOB", pack=False, align=None), "eku": SimTypePointer(SimStruct({"numEku": SimTypeInt(signed=False, label="UInt32"), "eku": SimTypePointer(SimTypePointer(SimTypeChar(label="Byte"), offset=0), offset=0)}, name="IKEEXT_CERT_EKUS0", pack=False, align=None), offset=0), "name": SimTypePointer(SimStruct({"nameType": SimTypeInt(signed=False, label="IKEEXT_CERT_CRITERIA_NAME_TYPE"), "certName": SimTypePointer(SimTypeChar(label="Char"), offset=0)}, name="IKEEXT_CERT_NAME0", pack=False, align=None), offset=0), "flags": SimTypeInt(signed=False, label="UInt32")}, name="IKEEXT_CERTIFICATE_CRITERIA0", pack=False, align=None), offset=0)}, name="_Anonymous2_e__Struct", pack=False, align=None), "Anonymous3": SimStruct({"inboundRootStoreArraySize": SimTypeInt(signed=False, label="UInt32"), "inboundTrustedRootStoreCriteria": SimTypePointer(SimStruct({"certData": SimStruct({"size": SimTypeInt(signed=False, label="UInt32"), "data": SimTypePointer(SimTypeChar(label="Byte"), offset=0)}, name="FWP_BYTE_BLOB", pack=False, align=None), "certHash": SimStruct({"size": SimTypeInt(signed=False, label="UInt32"), "data": SimTypePointer(SimTypeChar(label="Byte"), offset=0)}, name="FWP_BYTE_BLOB", pack=False, align=None), "eku": SimTypePointer(SimStruct({"numEku": SimTypeInt(signed=False, label="UInt32"), "eku": SimTypePointer(SimTypePointer(SimTypeChar(label="Byte"), offset=0), offset=0)}, name="IKEEXT_CERT_EKUS0", pack=False, align=None), offset=0), "name": SimTypePointer(SimStruct({"nameType": SimTypeInt(signed=False, label="IKEEXT_CERT_CRITERIA_NAME_TYPE"), "certName": SimTypePointer(SimTypeChar(label="Char"), offset=0)}, name="IKEEXT_CERT_NAME0", pack=False, align=None), offset=0), "flags": SimTypeInt(signed=False, label="UInt32")}, name="IKEEXT_CERTIFICATE_CRITERIA0", pack=False, align=None), offset=0)}, name="_Anonymous3_e__Struct", pack=False, align=None)}, name="<anon>", label="None"), "outboundConfigType": SimTypeInt(signed=False, label="IKEEXT_CERT_CONFIG_TYPE"), "Anonymous2": SimUnion({"Anonymous1": SimStruct({"outboundRootArraySize": SimTypeInt(signed=False, label="UInt32"), "outboundRootCriteria": SimTypePointer(SimStruct({"certData": SimStruct({"size": SimTypeInt(signed=False, label="UInt32"), "data": SimTypePointer(SimTypeChar(label="Byte"), offset=0)}, name="FWP_BYTE_BLOB", pack=False, align=None), "certHash": SimStruct({"size": SimTypeInt(signed=False, label="UInt32"), "data": SimTypePointer(SimTypeChar(label="Byte"), offset=0)}, name="FWP_BYTE_BLOB", pack=False, align=None), "eku": SimTypePointer(SimStruct({"numEku": SimTypeInt(signed=False, label="UInt32"), "eku": SimTypePointer(SimTypePointer(SimTypeChar(label="Byte"), offset=0), offset=0)}, name="IKEEXT_CERT_EKUS0", pack=False, align=None), offset=0), "name": SimTypePointer(SimStruct({"nameType": SimTypeInt(signed=False, label="IKEEXT_CERT_CRITERIA_NAME_TYPE"), "certName": SimTypePointer(SimTypeChar(label="Char"), offset=0)}, name="IKEEXT_CERT_NAME0", pack=False, align=None), offset=0), "flags": SimTypeInt(signed=False, label="UInt32")}, name="IKEEXT_CERTIFICATE_CRITERIA0", pack=False, align=None), offset=0)}, name="_Anonymous1_e__Struct", pack=False, align=None), "Anonymous2": SimStruct({"outboundEnterpriseStoreArraySize": SimTypeInt(signed=False, label="UInt32"), "outboundEnterpriseStoreCriteria": SimTypePointer(SimStruct({"certData": SimStruct({"size": SimTypeInt(signed=False, label="UInt32"), "data": SimTypePointer(SimTypeChar(label="Byte"), offset=0)}, name="FWP_BYTE_BLOB", pack=False, align=None), "certHash": SimStruct({"size": SimTypeInt(signed=False, label="UInt32"), "data": SimTypePointer(SimTypeChar(label="Byte"), offset=0)}, name="FWP_BYTE_BLOB", pack=False, align=None), "eku": SimTypePointer(SimStruct({"numEku": SimTypeInt(signed=False, label="UInt32"), "eku": SimTypePointer(SimTypePointer(SimTypeChar(label="Byte"), offset=0), offset=0)}, name="IKEEXT_CERT_EKUS0", pack=False, align=None), offset=0), "name": SimTypePointer(SimStruct({"nameType": SimTypeInt(signed=False, label="IKEEXT_CERT_CRITERIA_NAME_TYPE"), "certName": SimTypePointer(SimTypeChar(label="Char"), offset=0)}, name="IKEEXT_CERT_NAME0", pack=False, align=None), offset=0), "flags": SimTypeInt(signed=False, label="UInt32")}, name="IKEEXT_CERTIFICATE_CRITERIA0", pack=False, align=None), offset=0)}, name="_Anonymous2_e__Struct", pack=False, align=None), "Anonymous3": SimStruct({"outboundRootStoreArraySize": SimTypeInt(signed=False, label="UInt32"), "outboundTrustedRootStoreCriteria": SimTypePointer(SimStruct({"certData": SimStruct({"size": SimTypeInt(signed=False, label="UInt32"), "data": SimTypePointer(SimTypeChar(label="Byte"), offset=0)}, name="FWP_BYTE_BLOB", pack=False, align=None), "certHash": SimStruct({"size": SimTypeInt(signed=False, label="UInt32"), "data": SimTypePointer(SimTypeChar(label="Byte"), offset=0)}, name="FWP_BYTE_BLOB", pack=False, align=None), "eku": SimTypePointer(SimStruct({"numEku": SimTypeInt(signed=False, label="UInt32"), "eku": SimTypePointer(SimTypePointer(SimTypeChar(label="Byte"), offset=0), offset=0)}, name="IKEEXT_CERT_EKUS0", pack=False, align=None), offset=0), "name": SimTypePointer(SimStruct({"nameType": SimTypeInt(signed=False, label="IKEEXT_CERT_CRITERIA_NAME_TYPE"), "certName": SimTypePointer(SimTypeChar(label="Char"), offset=0)}, name="IKEEXT_CERT_NAME0", pack=False, align=None), offset=0), "flags": SimTypeInt(signed=False, label="UInt32")}, name="IKEEXT_CERTIFICATE_CRITERIA0", pack=False, align=None), offset=0)}, name="_Anonymous3_e__Struct", pack=False, align=None)}, name="<anon>", label="None"), "flags": SimTypeInt(signed=False, label="IKEEXT_CERT_AUTH"), "localCertLocationUrl": SimStruct({"size": SimTypeInt(signed=False, label="UInt32"), "data": SimTypePointer(SimTypeChar(label="Byte"), offset=0)}, name="FWP_BYTE_BLOB", pack=False, align=None)}, name="IKEEXT_CERTIFICATE_AUTHENTICATION2", pack=False, align=None), "cgaAuthentication": SimStruct({"keyContainerName": SimTypePointer(SimTypeChar(label="Char"), offset=0), "cspName": SimTypePointer(SimTypeChar(label="Char"), offset=0), "cspType": SimTypeInt(signed=False, label="UInt32"), "cgaModifier": SimStruct({"byteArray16": SimTypeFixedSizeArray(SimTypeChar(label="Byte"), 16)}, name="FWP_BYTE_ARRAY16", pack=False, align=None), "cgaCollisionCount": SimTypeChar(label="Byte")}, name="IKEEXT_IPV6_CGA_AUTHENTICATION0", pack=False, align=None), "eapAuthentication": SimStruct({"flags": SimTypeInt(signed=False, label="IKEEXT_EAP_AUTHENTICATION_FLAGS")}, name="IKEEXT_EAP_AUTHENTICATION0", pack=False, align=None)}, name="<anon>", label="None")}, name="IKEEXT_AUTHENTICATION_METHOD2", pack=False, align=None), offset=0), "initiatorImpersonationType": SimTypeInt(signed=False, label="IKEEXT_AUTHENTICATION_IMPERSONATION_TYPE")}, name="IKEEXT_EM_POLICY2", pack=False, align=None), offset=0)}, name="IPSEC_TRANSPORT_POLICY2", pack=False, align=None), offset=0), "ikeV2MmPolicy": SimTypePointer(SimStruct({"softExpirationTime": SimTypeInt(signed=False, label="UInt32"), "numAuthenticationMethods": SimTypeInt(signed=False, label="UInt32"), "authenticationMethods": SimTypePointer(SimStruct({"authenticationMethodType": SimTypeInt(signed=False, label="IKEEXT_AUTHENTICATION_METHOD_TYPE"), "Anonymous": SimUnion({"presharedKeyAuthentication": SimStruct({"presharedKey": SimStruct({"size": SimTypeInt(signed=False, label="UInt32"), "data": SimTypePointer(SimTypeChar(label="Byte"), offset=0)}, name="FWP_BYTE_BLOB", pack=False, align=None), "flags": SimTypeInt(signed=False, label="IKEEXT_PRESHARED_KEY_AUTHENTICATION_FLAGS")}, name="IKEEXT_PRESHARED_KEY_AUTHENTICATION1", pack=False, align=None), "certificateAuthentication": SimStruct({"inboundConfigType": SimTypeInt(signed=False, label="IKEEXT_CERT_CONFIG_TYPE"), "Anonymous1": SimUnion({"Anonymous1": SimStruct({"inboundRootArraySize": SimTypeInt(signed=False, label="UInt32"), "inboundRootCriteria": SimTypePointer(SimStruct({"certData": SimStruct({"size": SimTypeInt(signed=False, label="UInt32"), "data": SimTypePointer(SimTypeChar(label="Byte"), offset=0)}, name="FWP_BYTE_BLOB", pack=False, align=None), "certHash": SimStruct({"size": SimTypeInt(signed=False, label="UInt32"), "data": SimTypePointer(SimTypeChar(label="Byte"), offset=0)}, name="FWP_BYTE_BLOB", pack=False, align=None), "eku": SimTypePointer(SimStruct({"numEku": SimTypeInt(signed=False, label="UInt32"), "eku": SimTypePointer(SimTypePointer(SimTypeChar(label="Byte"), offset=0), offset=0)}, name="IKEEXT_CERT_EKUS0", pack=False, align=None), offset=0), "name": SimTypePointer(SimStruct({"nameType": SimTypeInt(signed=False, label="IKEEXT_CERT_CRITERIA_NAME_TYPE"), "certName": SimTypePointer(SimTypeChar(label="Char"), offset=0)}, name="IKEEXT_CERT_NAME0", pack=False, align=None), offset=0), "flags": SimTypeInt(signed=False, label="UInt32")}, name="IKEEXT_CERTIFICATE_CRITERIA0", pack=False, align=None), offset=0)}, name="_Anonymous1_e__Struct", pack=False, align=None), "Anonymous2": SimStruct({"inboundEnterpriseStoreArraySize": SimTypeInt(signed=False, label="UInt32"), "inboundEnterpriseStoreCriteria": SimTypePointer(SimStruct({"certData": SimStruct({"size": SimTypeInt(signed=False, label="UInt32"), "data": SimTypePointer(SimTypeChar(label="Byte"), offset=0)}, name="FWP_BYTE_BLOB", pack=False, align=None), "certHash": SimStruct({"size": SimTypeInt(signed=False, label="UInt32"), "data": SimTypePointer(SimTypeChar(label="Byte"), offset=0)}, name="FWP_BYTE_BLOB", pack=False, align=None), "eku": SimTypePointer(SimStruct({"numEku": SimTypeInt(signed=False, label="UInt32"), "eku": SimTypePointer(SimTypePointer(SimTypeChar(label="Byte"), offset=0), offset=0)}, name="IKEEXT_CERT_EKUS0", pack=False, align=None), offset=0), "name": SimTypePointer(SimStruct({"nameType": SimTypeInt(signed=False, label="IKEEXT_CERT_CRITERIA_NAME_TYPE"), "certName": SimTypePointer(SimTypeChar(label="Char"), offset=0)}, name="IKEEXT_CERT_NAME0", pack=False, align=None), offset=0), "flags": SimTypeInt(signed=False, label="UInt32")}, name="IKEEXT_CERTIFICATE_CRITERIA0", pack=False, align=None), offset=0)}, name="_Anonymous2_e__Struct", pack=False, align=None), "Anonymous3": SimStruct({"inboundRootStoreArraySize": SimTypeInt(signed=False, label="UInt32"), "inboundTrustedRootStoreCriteria": SimTypePointer(SimStruct({"certData": SimStruct({"size": SimTypeInt(signed=False, label="UInt32"), "data": SimTypePointer(SimTypeChar(label="Byte"), offset=0)}, name="FWP_BYTE_BLOB", pack=False, align=None), "certHash": SimStruct({"size": SimTypeInt(signed=False, label="UInt32"), "data": SimTypePointer(SimTypeChar(label="Byte"), offset=0)}, name="FWP_BYTE_BLOB", pack=False, align=None), "eku": SimTypePointer(SimStruct({"numEku": SimTypeInt(signed=False, label="UInt32"), "eku": SimTypePointer(SimTypePointer(SimTypeChar(label="Byte"), offset=0), offset=0)}, name="IKEEXT_CERT_EKUS0", pack=False, align=None), offset=0), "name": SimTypePointer(SimStruct({"nameType": SimTypeInt(signed=False, label="IKEEXT_CERT_CRITERIA_NAME_TYPE"), "certName": SimTypePointer(SimTypeChar(label="Char"), offset=0)}, name="IKEEXT_CERT_NAME0", pack=False, align=None), offset=0), "flags": SimTypeInt(signed=False, label="UInt32")}, name="IKEEXT_CERTIFICATE_CRITERIA0", pack=False, align=None), offset=0)}, name="_Anonymous3_e__Struct", pack=False, align=None)}, name="<anon>", label="None"), "outboundConfigType": SimTypeInt(signed=False, label="IKEEXT_CERT_CONFIG_TYPE"), "Anonymous2": SimUnion({"Anonymous1": SimStruct({"outboundRootArraySize": SimTypeInt(signed=False, label="UInt32"), "outboundRootCriteria": SimTypePointer(SimStruct({"certData": SimStruct({"size": SimTypeInt(signed=False, label="UInt32"), "data": SimTypePointer(SimTypeChar(label="Byte"), offset=0)}, name="FWP_BYTE_BLOB", pack=False, align=None), "certHash": SimStruct({"size": SimTypeInt(signed=False, label="UInt32"), "data": SimTypePointer(SimTypeChar(label="Byte"), offset=0)}, name="FWP_BYTE_BLOB", pack=False, align=None), "eku": SimTypePointer(SimStruct({"numEku": SimTypeInt(signed=False, label="UInt32"), "eku": SimTypePointer(SimTypePointer(SimTypeChar(label="Byte"), offset=0), offset=0)}, name="IKEEXT_CERT_EKUS0", pack=False, align=None), offset=0), "name": SimTypePointer(SimStruct({"nameType": SimTypeInt(signed=False, label="IKEEXT_CERT_CRITERIA_NAME_TYPE"), "certName": SimTypePointer(SimTypeChar(label="Char"), offset=0)}, name="IKEEXT_CERT_NAME0", pack=False, align=None), offset=0), "flags": SimTypeInt(signed=False, label="UInt32")}, name="IKEEXT_CERTIFICATE_CRITERIA0", pack=False, align=None), offset=0)}, name="_Anonymous1_e__Struct", pack=False, align=None), "Anonymous2": SimStruct({"outboundEnterpriseStoreArraySize": SimTypeInt(signed=False, label="UInt32"), "outboundEnterpriseStoreCriteria": SimTypePointer(SimStruct({"certData": SimStruct({"size": SimTypeInt(signed=False, label="UInt32"), "data": SimTypePointer(SimTypeChar(label="Byte"), offset=0)}, name="FWP_BYTE_BLOB", pack=False, align=None), "certHash": SimStruct({"size": SimTypeInt(signed=False, label="UInt32"), "data": SimTypePointer(SimTypeChar(label="Byte"), offset=0)}, name="FWP_BYTE_BLOB", pack=False, align=None), "eku": SimTypePointer(SimStruct({"numEku": SimTypeInt(signed=False, label="UInt32"), "eku": SimTypePointer(SimTypePointer(SimTypeChar(label="Byte"), offset=0), offset=0)}, name="IKEEXT_CERT_EKUS0", pack=False, align=None), offset=0), "name": SimTypePointer(SimStruct({"nameType": SimTypeInt(signed=False, label="IKEEXT_CERT_CRITERIA_NAME_TYPE"), "certName": SimTypePointer(SimTypeChar(label="Char"), offset=0)}, name="IKEEXT_CERT_NAME0", pack=False, align=None), offset=0), "flags": SimTypeInt(signed=False, label="UInt32")}, name="IKEEXT_CERTIFICATE_CRITERIA0", pack=False, align=None), offset=0)}, name="_Anonymous2_e__Struct", pack=False, align=None), "Anonymous3": SimStruct({"outboundRootStoreArraySize": SimTypeInt(signed=False, label="UInt32"), "outboundTrustedRootStoreCriteria": SimTypePointer(SimStruct({"certData": SimStruct({"size": SimTypeInt(signed=False, label="UInt32"), "data": SimTypePointer(SimTypeChar(label="Byte"), offset=0)}, name="FWP_BYTE_BLOB", pack=False, align=None), "certHash": SimStruct({"size": SimTypeInt(signed=False, label="UInt32"), "data": SimTypePointer(SimTypeChar(label="Byte"), offset=0)}, name="FWP_BYTE_BLOB", pack=False, align=None), "eku": SimTypePointer(SimStruct({"numEku": SimTypeInt(signed=False, label="UInt32"), "eku": SimTypePointer(SimTypePointer(SimTypeChar(label="Byte"), offset=0), offset=0)}, name="IKEEXT_CERT_EKUS0", pack=False, align=None), offset=0), "name": SimTypePointer(SimStruct({"nameType": SimTypeInt(signed=False, label="IKEEXT_CERT_CRITERIA_NAME_TYPE"), "certName": SimTypePointer(SimTypeChar(label="Char"), offset=0)}, name="IKEEXT_CERT_NAME0", pack=False, align=None), offset=0), "flags": SimTypeInt(signed=False, label="UInt32")}, name="IKEEXT_CERTIFICATE_CRITERIA0", pack=False, align=None), offset=0)}, name="_Anonymous3_e__Struct", pack=False, align=None)}, name="<anon>", label="None"), "flags": SimTypeInt(signed=False, label="IKEEXT_CERT_AUTH"), "localCertLocationUrl": SimStruct({"size": SimTypeInt(signed=False, label="UInt32"), "data": SimTypePointer(SimTypeChar(label="Byte"), offset=0)}, name="FWP_BYTE_BLOB", pack=False, align=None)}, name="IKEEXT_CERTIFICATE_AUTHENTICATION2", pack=False, align=None), "kerberosAuthentication": SimStruct({"flags": SimTypeInt(signed=False, label="IKEEXT_KERBEROS_AUTHENTICATION_FLAGS"), "proxyServer": SimTypePointer(SimTypeChar(label="Char"), offset=0)}, name="IKEEXT_KERBEROS_AUTHENTICATION1", pack=False, align=None), "reservedAuthentication": SimStruct({"flags": SimTypeInt(signed=False, label="IKEEXT_RESERVED_AUTHENTICATION_FLAGS")}, name="IKEEXT_RESERVED_AUTHENTICATION0", pack=False, align=None), "ntlmV2Authentication": SimStruct({"flags": SimTypeInt(signed=False, label="UInt32")}, name="IKEEXT_NTLM_V2_AUTHENTICATION0", pack=False, align=None), "sslAuthentication": SimStruct({"inboundConfigType": SimTypeInt(signed=False, label="IKEEXT_CERT_CONFIG_TYPE"), "Anonymous1": SimUnion({"Anonymous1": SimStruct({"inboundRootArraySize": SimTypeInt(signed=False, label="UInt32"), "inboundRootCriteria": SimTypePointer(SimStruct({"certData": SimStruct({"size": SimTypeInt(signed=False, label="UInt32"), "data": SimTypePointer(SimTypeChar(label="Byte"), offset=0)}, name="FWP_BYTE_BLOB", pack=False, align=None), "certHash": SimStruct({"size": SimTypeInt(signed=False, label="UInt32"), "data": SimTypePointer(SimTypeChar(label="Byte"), offset=0)}, name="FWP_BYTE_BLOB", pack=False, align=None), "eku": SimTypePointer(SimStruct({"numEku": SimTypeInt(signed=False, label="UInt32"), "eku": SimTypePointer(SimTypePointer(SimTypeChar(label="Byte"), offset=0), offset=0)}, name="IKEEXT_CERT_EKUS0", pack=False, align=None), offset=0), "name": SimTypePointer(SimStruct({"nameType": SimTypeInt(signed=False, label="IKEEXT_CERT_CRITERIA_NAME_TYPE"), "certName": SimTypePointer(SimTypeChar(label="Char"), offset=0)}, name="IKEEXT_CERT_NAME0", pack=False, align=None), offset=0), "flags": SimTypeInt(signed=False, label="UInt32")}, name="IKEEXT_CERTIFICATE_CRITERIA0", pack=False, align=None), offset=0)}, name="_Anonymous1_e__Struct", pack=False, align=None), "Anonymous2": SimStruct({"inboundEnterpriseStoreArraySize": SimTypeInt(signed=False, label="UInt32"), "inboundEnterpriseStoreCriteria": SimTypePointer(SimStruct({"certData": SimStruct({"size": SimTypeInt(signed=False, label="UInt32"), "data": SimTypePointer(SimTypeChar(label="Byte"), offset=0)}, name="FWP_BYTE_BLOB", pack=False, align=None), "certHash": SimStruct({"size": SimTypeInt(signed=False, label="UInt32"), "data": SimTypePointer(SimTypeChar(label="Byte"), offset=0)}, name="FWP_BYTE_BLOB", pack=False, align=None), "eku": SimTypePointer(SimStruct({"numEku": SimTypeInt(signed=False, label="UInt32"), "eku": SimTypePointer(SimTypePointer(SimTypeChar(label="Byte"), offset=0), offset=0)}, name="IKEEXT_CERT_EKUS0", pack=False, align=None), offset=0), "name": SimTypePointer(SimStruct({"nameType": SimTypeInt(signed=False, label="IKEEXT_CERT_CRITERIA_NAME_TYPE"), "certName": SimTypePointer(SimTypeChar(label="Char"), offset=0)}, name="IKEEXT_CERT_NAME0", pack=False, align=None), offset=0), "flags": SimTypeInt(signed=False, label="UInt32")}, name="IKEEXT_CERTIFICATE_CRITERIA0", pack=False, align=None), offset=0)}, name="_Anonymous2_e__Struct", pack=False, align=None), "Anonymous3": SimStruct({"inboundRootStoreArraySize": SimTypeInt(signed=False, label="UInt32"), "inboundTrustedRootStoreCriteria": SimTypePointer(SimStruct({"certData": SimStruct({"size": SimTypeInt(signed=False, label="UInt32"), "data": SimTypePointer(SimTypeChar(label="Byte"), offset=0)}, name="FWP_BYTE_BLOB", pack=False, align=None), "certHash": SimStruct({"size": SimTypeInt(signed=False, label="UInt32"), "data": SimTypePointer(SimTypeChar(label="Byte"), offset=0)}, name="FWP_BYTE_BLOB", pack=False, align=None), "eku": SimTypePointer(SimStruct({"numEku": SimTypeInt(signed=False, label="UInt32"), "eku": SimTypePointer(SimTypePointer(SimTypeChar(label="Byte"), offset=0), offset=0)}, name="IKEEXT_CERT_EKUS0", pack=False, align=None), offset=0), "name": SimTypePointer(SimStruct({"nameType": SimTypeInt(signed=False, label="IKEEXT_CERT_CRITERIA_NAME_TYPE"), "certName": SimTypePointer(SimTypeChar(label="Char"), offset=0)}, name="IKEEXT_CERT_NAME0", pack=False, align=None), offset=0), "flags": SimTypeInt(signed=False, label="UInt32")}, name="IKEEXT_CERTIFICATE_CRITERIA0", pack=False, align=None), offset=0)}, name="_Anonymous3_e__Struct", pack=False, align=None)}, name="<anon>", label="None"), "outboundConfigType": SimTypeInt(signed=False, label="IKEEXT_CERT_CONFIG_TYPE"), "Anonymous2": SimUnion({"Anonymous1": SimStruct({"outboundRootArraySize": SimTypeInt(signed=False, label="UInt32"), "outboundRootCriteria": SimTypePointer(SimStruct({"certData": SimStruct({"size": SimTypeInt(signed=False, label="UInt32"), "data": SimTypePointer(SimTypeChar(label="Byte"), offset=0)}, name="FWP_BYTE_BLOB", pack=False, align=None), "certHash": SimStruct({"size": SimTypeInt(signed=False, label="UInt32"), "data": SimTypePointer(SimTypeChar(label="Byte"), offset=0)}, name="FWP_BYTE_BLOB", pack=False, align=None), "eku": SimTypePointer(SimStruct({"numEku": SimTypeInt(signed=False, label="UInt32"), "eku": SimTypePointer(SimTypePointer(SimTypeChar(label="Byte"), offset=0), offset=0)}, name="IKEEXT_CERT_EKUS0", pack=False, align=None), offset=0), "name": SimTypePointer(SimStruct({"nameType": SimTypeInt(signed=False, label="IKEEXT_CERT_CRITERIA_NAME_TYPE"), "certName": SimTypePointer(SimTypeChar(label="Char"), offset=0)}, name="IKEEXT_CERT_NAME0", pack=False, align=None), offset=0), "flags": SimTypeInt(signed=False, label="UInt32")}, name="IKEEXT_CERTIFICATE_CRITERIA0", pack=False, align=None), offset=0)}, name="_Anonymous1_e__Struct", pack=False, align=None), "Anonymous2": SimStruct({"outboundEnterpriseStoreArraySize": SimTypeInt(signed=False, label="UInt32"), "outboundEnterpriseStoreCriteria": SimTypePointer(SimStruct({"certData": SimStruct({"size": SimTypeInt(signed=False, label="UInt32"), "data": SimTypePointer(SimTypeChar(label="Byte"), offset=0)}, name="FWP_BYTE_BLOB", pack=False, align=None), "certHash": SimStruct({"size": SimTypeInt(signed=False, label="UInt32"), "data": SimTypePointer(SimTypeChar(label="Byte"), offset=0)}, name="FWP_BYTE_BLOB", pack=False, align=None), "eku": SimTypePointer(SimStruct({"numEku": SimTypeInt(signed=False, label="UInt32"), "eku": SimTypePointer(SimTypePointer(SimTypeChar(label="Byte"), offset=0), offset=0)}, name="IKEEXT_CERT_EKUS0", pack=False, align=None), offset=0), "name": SimTypePointer(SimStruct({"nameType": SimTypeInt(signed=False, label="IKEEXT_CERT_CRITERIA_NAME_TYPE"), "certName": SimTypePointer(SimTypeChar(label="Char"), offset=0)}, name="IKEEXT_CERT_NAME0", pack=False, align=None), offset=0), "flags": SimTypeInt(signed=False, label="UInt32")}, name="IKEEXT_CERTIFICATE_CRITERIA0", pack=False, align=None), offset=0)}, name="_Anonymous2_e__Struct", pack=False, align=None), "Anonymous3": SimStruct({"outboundRootStoreArraySize": SimTypeInt(signed=False, label="UInt32"), "outboundTrustedRootStoreCriteria": SimTypePointer(SimStruct({"certData": SimStruct({"size": SimTypeInt(signed=False, label="UInt32"), "data": SimTypePointer(SimTypeChar(label="Byte"), offset=0)}, name="FWP_BYTE_BLOB", pack=False, align=None), "certHash": SimStruct({"size": SimTypeInt(signed=False, label="UInt32"), "data": SimTypePointer(SimTypeChar(label="Byte"), offset=0)}, name="FWP_BYTE_BLOB", pack=False, align=None), "eku": SimTypePointer(SimStruct({"numEku": SimTypeInt(signed=False, label="UInt32"), "eku": SimTypePointer(SimTypePointer(SimTypeChar(label="Byte"), offset=0), offset=0)}, name="IKEEXT_CERT_EKUS0", pack=False, align=None), offset=0), "name": SimTypePointer(SimStruct({"nameType": SimTypeInt(signed=False, label="IKEEXT_CERT_CRITERIA_NAME_TYPE"), "certName": SimTypePointer(SimTypeChar(label="Char"), offset=0)}, name="IKEEXT_CERT_NAME0", pack=False, align=None), offset=0), "flags": SimTypeInt(signed=False, label="UInt32")}, name="IKEEXT_CERTIFICATE_CRITERIA0", pack=False, align=None), offset=0)}, name="_Anonymous3_e__Struct", pack=False, align=None)}, name="<anon>", label="None"), "flags": SimTypeInt(signed=False, label="IKEEXT_CERT_AUTH"), "localCertLocationUrl": SimStruct({"size": SimTypeInt(signed=False, label="UInt32"), "data": SimTypePointer(SimTypeChar(label="Byte"), offset=0)}, name="FWP_BYTE_BLOB", pack=False, align=None)}, name="IKEEXT_CERTIFICATE_AUTHENTICATION2", pack=False, align=None), "cgaAuthentication": SimStruct({"keyContainerName": SimTypePointer(SimTypeChar(label="Char"), offset=0), "cspName": SimTypePointer(SimTypeChar(label="Char"), offset=0), "cspType": SimTypeInt(signed=False, label="UInt32"), "cgaModifier": SimStruct({"byteArray16": SimTypeFixedSizeArray(SimTypeChar(label="Byte"), 16)}, name="FWP_BYTE_ARRAY16", pack=False, align=None), "cgaCollisionCount": SimTypeChar(label="Byte")}, name="IKEEXT_IPV6_CGA_AUTHENTICATION0", pack=False, align=None), "eapAuthentication": SimStruct({"flags": SimTypeInt(signed=False, label="IKEEXT_EAP_AUTHENTICATION_FLAGS")}, name="IKEEXT_EAP_AUTHENTICATION0", pack=False, align=None)}, name="<anon>", label="None")}, name="IKEEXT_AUTHENTICATION_METHOD2", pack=False, align=None), offset=0), "initiatorImpersonationType": SimTypeInt(signed=False, label="IKEEXT_AUTHENTICATION_IMPERSONATION_TYPE"), "numIkeProposals": SimTypeInt(signed=False, label="UInt32"), "ikeProposals": SimTypePointer(SimStruct({"cipherAlgorithm": SimStruct({"algoIdentifier": SimTypeInt(signed=False, label="IKEEXT_CIPHER_TYPE"), "keyLen": SimTypeInt(signed=False, label="UInt32"), "rounds": SimTypeInt(signed=False, label="UInt32")}, name="IKEEXT_CIPHER_ALGORITHM0", pack=False, align=None), "integrityAlgorithm": SimStruct({"algoIdentifier": SimTypeInt(signed=False, label="IKEEXT_INTEGRITY_TYPE")}, name="IKEEXT_INTEGRITY_ALGORITHM0", pack=False, align=None), "maxLifetimeSeconds": SimTypeInt(signed=False, label="UInt32"), "dhGroup": SimTypeInt(signed=False, label="IKEEXT_DH_GROUP"), "quickModeLimit": SimTypeInt(signed=False, label="UInt32")}, name="IKEEXT_PROPOSAL0", pack=False, align=None), offset=0), "flags": SimTypeInt(signed=False, label="IKEEXT_POLICY_FLAG"), "maxDynamicFilters": SimTypeInt(signed=False, label="UInt32"), "retransmitDurationSecs": SimTypeInt(signed=False, label="UInt32")}, name="IKEEXT_POLICY2", pack=False, align=None), offset=0), "idpOptions": SimTypePointer(SimStruct({"stateIdleTimeoutSeconds": SimTypeInt(signed=False, label="UInt32"), "perIPRateLimitQueueIdleTimeoutSeconds": SimTypeInt(signed=False, label="UInt32"), "ipV6IPsecUnauthDscp": SimTypeChar(label="Byte"), "ipV6IPsecUnauthRateLimitBytesPerSec": SimTypeInt(signed=False, label="UInt32"), "ipV6IPsecUnauthPerIPRateLimitBytesPerSec": SimTypeInt(signed=False, label="UInt32"), "ipV6IPsecAuthDscp": SimTypeChar(label="Byte"), "ipV6IPsecAuthRateLimitBytesPerSec": SimTypeInt(signed=False, label="UInt32"), "icmpV6Dscp": SimTypeChar(label="Byte"), "icmpV6RateLimitBytesPerSec": SimTypeInt(signed=False, label="UInt32"), "ipV6FilterExemptDscp": SimTypeChar(label="Byte"), "ipV6FilterExemptRateLimitBytesPerSec": SimTypeInt(signed=False, label="UInt32"), "defBlockExemptDscp": SimTypeChar(label="Byte"), "defBlockExemptRateLimitBytesPerSec": SimTypeInt(signed=False, label="UInt32"), "maxStateEntries": SimTypeInt(signed=False, label="UInt32"), "maxPerIPRateLimitQueues": SimTypeInt(signed=False, label="UInt32"), "flags": SimTypeInt(signed=False, label="IPSEC_DOSP_FLAGS"), "numPublicIFLuids": SimTypeInt(signed=False, label="UInt32"), "publicIFLuids": SimTypePointer(SimTypeLongLong(signed=False, label="UInt64"), offset=0), "numInternalIFLuids": SimTypeInt(signed=False, label="UInt32"), "internalIFLuids": SimTypePointer(SimTypeLongLong(signed=False, label="UInt64"), offset=0), "publicV6AddrMask": SimStruct({"addr": SimTypeFixedSizeArray(SimTypeChar(label="Byte"), 16), "prefixLength": SimTypeChar(label="Byte")}, name="FWP_V6_ADDR_AND_MASK", pack=False, align=None), "internalV6AddrMask": SimStruct({"addr": SimTypeFixedSizeArray(SimTypeChar(label="Byte"), 16), "prefixLength": SimTypeChar(label="Byte")}, name="FWP_V6_ADDR_AND_MASK", pack=False, align=None)}, name="IPSEC_DOSP_OPTIONS0", pack=False, align=None), offset=0)}, name="<anon>", label="None"), "providerContextId": SimTypeLongLong(signed=False, label="UInt64")}, name="FWPM_PROVIDER_CONTEXT3_", pack=False, align=None), offset=0), offset=0)], SimTypeInt(signed=False, label="UInt32"), arg_names=["engineHandle", "id", "providerContext"]),
        # 
        'FwpmProviderContextGetByKey0': SimTypeFunction([SimTypePointer(SimTypeInt(signed=True, label="Int"), label="IntPtr", offset=0), SimTypePointer(SimTypeBottom(label="Guid"), offset=0), SimTypePointer(SimTypePointer(SimStruct({"providerContextKey": SimTypeBottom(label="Guid"), "displayData": SimStruct({"name": SimTypePointer(SimTypeChar(label="Char"), offset=0), "description": SimTypePointer(SimTypeChar(label="Char"), offset=0)}, name="FWPM_DISPLAY_DATA0", pack=False, align=None), "flags": SimTypeInt(signed=False, label="UInt32"), "providerKey": SimTypePointer(SimTypeBottom(label="Guid"), offset=0), "providerData": SimStruct({"size": SimTypeInt(signed=False, label="UInt32"), "data": SimTypePointer(SimTypeChar(label="Byte"), offset=0)}, name="FWP_BYTE_BLOB", pack=False, align=None), "type": SimTypeInt(signed=False, label="FWPM_PROVIDER_CONTEXT_TYPE"), "Anonymous": SimUnion({"keyingPolicy": SimTypePointer(SimStruct({"numKeyMods": SimTypeInt(signed=False, label="UInt32"), "keyModKeys": SimTypePointer(SimTypeBottom(label="Guid"), offset=0)}, name="IPSEC_KEYING_POLICY0", pack=False, align=None), offset=0), "ikeQmTransportPolicy": SimTypePointer(SimStruct({"numIpsecProposals": SimTypeInt(signed=False, label="UInt32"), "ipsecProposals": SimTypePointer(SimStruct({"lifetime": SimStruct({"lifetimeSeconds": SimTypeInt(signed=False, label="UInt32"), "lifetimeKilobytes": SimTypeInt(signed=False, label="UInt32"), "lifetimePackets": SimTypeInt(signed=False, label="UInt32")}, name="IPSEC_SA_LIFETIME0", pack=False, align=None), "numSaTransforms": SimTypeInt(signed=False, label="UInt32"), "saTransforms": SimTypePointer(SimStruct({"ipsecTransformType": SimTypeInt(signed=False, label="IPSEC_TRANSFORM_TYPE"), "Anonymous": SimUnion({"ahTransform": SimTypePointer(SimStruct({"authTransformId": SimStruct({"authType": SimTypeInt(signed=False, label="IPSEC_AUTH_TYPE"), "authConfig": SimTypeChar(label="Byte")}, name="IPSEC_AUTH_TRANSFORM_ID0", pack=False, align=None), "cryptoModuleId": SimTypePointer(SimTypeBottom(label="Guid"), offset=0)}, name="IPSEC_AUTH_TRANSFORM0", pack=False, align=None), offset=0), "espAuthTransform": SimTypePointer(SimStruct({"authTransformId": SimStruct({"authType": SimTypeInt(signed=False, label="IPSEC_AUTH_TYPE"), "authConfig": SimTypeChar(label="Byte")}, name="IPSEC_AUTH_TRANSFORM_ID0", pack=False, align=None), "cryptoModuleId": SimTypePointer(SimTypeBottom(label="Guid"), offset=0)}, name="IPSEC_AUTH_TRANSFORM0", pack=False, align=None), offset=0), "espCipherTransform": SimTypePointer(SimStruct({"cipherTransformId": SimStruct({"cipherType": SimTypeInt(signed=False, label="IPSEC_CIPHER_TYPE"), "cipherConfig": SimTypeChar(label="Byte")}, name="IPSEC_CIPHER_TRANSFORM_ID0", pack=False, align=None), "cryptoModuleId": SimTypePointer(SimTypeBottom(label="Guid"), offset=0)}, name="IPSEC_CIPHER_TRANSFORM0", pack=False, align=None), offset=0), "espAuthAndCipherTransform": SimTypePointer(SimStruct({"authTransform": SimStruct({"authTransformId": SimStruct({"authType": SimTypeInt(signed=False, label="IPSEC_AUTH_TYPE"), "authConfig": SimTypeChar(label="Byte")}, name="IPSEC_AUTH_TRANSFORM_ID0", pack=False, align=None), "cryptoModuleId": SimTypePointer(SimTypeBottom(label="Guid"), offset=0)}, name="IPSEC_AUTH_TRANSFORM0", pack=False, align=None), "cipherTransform": SimStruct({"cipherTransformId": SimStruct({"cipherType": SimTypeInt(signed=False, label="IPSEC_CIPHER_TYPE"), "cipherConfig": SimTypeChar(label="Byte")}, name="IPSEC_CIPHER_TRANSFORM_ID0", pack=False, align=None), "cryptoModuleId": SimTypePointer(SimTypeBottom(label="Guid"), offset=0)}, name="IPSEC_CIPHER_TRANSFORM0", pack=False, align=None)}, name="IPSEC_AUTH_AND_CIPHER_TRANSFORM0", pack=False, align=None), offset=0), "espAuthFwTransform": SimTypePointer(SimStruct({"authTransformId": SimStruct({"authType": SimTypeInt(signed=False, label="IPSEC_AUTH_TYPE"), "authConfig": SimTypeChar(label="Byte")}, name="IPSEC_AUTH_TRANSFORM_ID0", pack=False, align=None), "cryptoModuleId": SimTypePointer(SimTypeBottom(label="Guid"), offset=0)}, name="IPSEC_AUTH_TRANSFORM0", pack=False, align=None), offset=0)}, name="<anon>", label="None")}, name="IPSEC_SA_TRANSFORM0", pack=False, align=None), offset=0), "pfsGroup": SimTypeInt(signed=False, label="IPSEC_PFS_GROUP")}, name="IPSEC_PROPOSAL0", pack=False, align=None), offset=0), "flags": SimTypeInt(signed=False, label="IPSEC_POLICY_FLAG"), "ndAllowClearTimeoutSeconds": SimTypeInt(signed=False, label="UInt32"), "saIdleTimeout": SimStruct({"idleTimeoutSeconds": SimTypeInt(signed=False, label="UInt32"), "idleTimeoutSecondsFailOver": SimTypeInt(signed=False, label="UInt32")}, name="IPSEC_SA_IDLE_TIMEOUT0", pack=False, align=None), "emPolicy": SimTypePointer(SimStruct({"numAuthenticationMethods": SimTypeInt(signed=False, label="UInt32"), "authenticationMethods": SimTypePointer(SimStruct({"authenticationMethodType": SimTypeInt(signed=False, label="IKEEXT_AUTHENTICATION_METHOD_TYPE"), "Anonymous": SimUnion({"presharedKeyAuthentication": SimStruct({"presharedKey": SimStruct({"size": SimTypeInt(signed=False, label="UInt32"), "data": SimTypePointer(SimTypeChar(label="Byte"), offset=0)}, name="FWP_BYTE_BLOB", pack=False, align=None)}, name="IKEEXT_PRESHARED_KEY_AUTHENTICATION0", pack=False, align=None), "certificateAuthentication": SimStruct({"inboundConfigType": SimTypeInt(signed=False, label="IKEEXT_CERT_CONFIG_TYPE"), "Anonymous1": SimUnion({"Anonymous": SimStruct({"inboundRootArraySize": SimTypeInt(signed=False, label="UInt32"), "inboundRootArray": SimTypePointer(SimStruct({"certData": SimStruct({"size": SimTypeInt(signed=False, label="UInt32"), "data": SimTypePointer(SimTypeChar(label="Byte"), offset=0)}, name="FWP_BYTE_BLOB", pack=False, align=None), "flags": SimTypeInt(signed=False, label="IKEEXT_CERT_FLAGS")}, name="IKEEXT_CERT_ROOT_CONFIG0", pack=False, align=None), offset=0)}, name="_Anonymous_e__Struct", pack=False, align=None), "inboundEnterpriseStoreConfig": SimTypePointer(SimStruct({"certData": SimStruct({"size": SimTypeInt(signed=False, label="UInt32"), "data": SimTypePointer(SimTypeChar(label="Byte"), offset=0)}, name="FWP_BYTE_BLOB", pack=False, align=None), "flags": SimTypeInt(signed=False, label="IKEEXT_CERT_FLAGS")}, name="IKEEXT_CERT_ROOT_CONFIG0", pack=False, align=None), offset=0), "inboundTrustedRootStoreConfig": SimTypePointer(SimStruct({"certData": SimStruct({"size": SimTypeInt(signed=False, label="UInt32"), "data": SimTypePointer(SimTypeChar(label="Byte"), offset=0)}, name="FWP_BYTE_BLOB", pack=False, align=None), "flags": SimTypeInt(signed=False, label="IKEEXT_CERT_FLAGS")}, name="IKEEXT_CERT_ROOT_CONFIG0", pack=False, align=None), offset=0)}, name="<anon>", label="None"), "outboundConfigType": SimTypeInt(signed=False, label="IKEEXT_CERT_CONFIG_TYPE"), "Anonymous2": SimUnion({"Anonymous": SimStruct({"outboundRootArraySize": SimTypeInt(signed=False, label="UInt32"), "outboundRootArray": SimTypePointer(SimStruct({"certData": SimStruct({"size": SimTypeInt(signed=False, label="UInt32"), "data": SimTypePointer(SimTypeChar(label="Byte"), offset=0)}, name="FWP_BYTE_BLOB", pack=False, align=None), "flags": SimTypeInt(signed=False, label="IKEEXT_CERT_FLAGS")}, name="IKEEXT_CERT_ROOT_CONFIG0", pack=False, align=None), offset=0)}, name="_Anonymous_e__Struct", pack=False, align=None), "outboundEnterpriseStoreConfig": SimTypePointer(SimStruct({"certData": SimStruct({"size": SimTypeInt(signed=False, label="UInt32"), "data": SimTypePointer(SimTypeChar(label="Byte"), offset=0)}, name="FWP_BYTE_BLOB", pack=False, align=None), "flags": SimTypeInt(signed=False, label="IKEEXT_CERT_FLAGS")}, name="IKEEXT_CERT_ROOT_CONFIG0", pack=False, align=None), offset=0), "outboundTrustedRootStoreConfig": SimTypePointer(SimStruct({"certData": SimStruct({"size": SimTypeInt(signed=False, label="UInt32"), "data": SimTypePointer(SimTypeChar(label="Byte"), offset=0)}, name="FWP_BYTE_BLOB", pack=False, align=None), "flags": SimTypeInt(signed=False, label="IKEEXT_CERT_FLAGS")}, name="IKEEXT_CERT_ROOT_CONFIG0", pack=False, align=None), offset=0)}, name="<anon>", label="None"), "flags": SimTypeInt(signed=False, label="IKEEXT_CERT_AUTH")}, name="IKEEXT_CERTIFICATE_AUTHENTICATION0", pack=False, align=None), "kerberosAuthentication": SimStruct({"flags": SimTypeInt(signed=False, label="IKEEXT_KERBEROS_AUTHENTICATION_FLAGS")}, name="IKEEXT_KERBEROS_AUTHENTICATION0", pack=False, align=None), "ntlmV2Authentication": SimStruct({"flags": SimTypeInt(signed=False, label="UInt32")}, name="IKEEXT_NTLM_V2_AUTHENTICATION0", pack=False, align=None), "sslAuthentication": SimStruct({"inboundConfigType": SimTypeInt(signed=False, label="IKEEXT_CERT_CONFIG_TYPE"), "Anonymous1": SimUnion({"Anonymous": SimStruct({"inboundRootArraySize": SimTypeInt(signed=False, label="UInt32"), "inboundRootArray": SimTypePointer(SimStruct({"certData": SimStruct({"size": SimTypeInt(signed=False, label="UInt32"), "data": SimTypePointer(SimTypeChar(label="Byte"), offset=0)}, name="FWP_BYTE_BLOB", pack=False, align=None), "flags": SimTypeInt(signed=False, label="IKEEXT_CERT_FLAGS")}, name="IKEEXT_CERT_ROOT_CONFIG0", pack=False, align=None), offset=0)}, name="_Anonymous_e__Struct", pack=False, align=None), "inboundEnterpriseStoreConfig": SimTypePointer(SimStruct({"certData": SimStruct({"size": SimTypeInt(signed=False, label="UInt32"), "data": SimTypePointer(SimTypeChar(label="Byte"), offset=0)}, name="FWP_BYTE_BLOB", pack=False, align=None), "flags": SimTypeInt(signed=False, label="IKEEXT_CERT_FLAGS")}, name="IKEEXT_CERT_ROOT_CONFIG0", pack=False, align=None), offset=0), "inboundTrustedRootStoreConfig": SimTypePointer(SimStruct({"certData": SimStruct({"size": SimTypeInt(signed=False, label="UInt32"), "data": SimTypePointer(SimTypeChar(label="Byte"), offset=0)}, name="FWP_BYTE_BLOB", pack=False, align=None), "flags": SimTypeInt(signed=False, label="IKEEXT_CERT_FLAGS")}, name="IKEEXT_CERT_ROOT_CONFIG0", pack=False, align=None), offset=0)}, name="<anon>", label="None"), "outboundConfigType": SimTypeInt(signed=False, label="IKEEXT_CERT_CONFIG_TYPE"), "Anonymous2": SimUnion({"Anonymous": SimStruct({"outboundRootArraySize": SimTypeInt(signed=False, label="UInt32"), "outboundRootArray": SimTypePointer(SimStruct({"certData": SimStruct({"size": SimTypeInt(signed=False, label="UInt32"), "data": SimTypePointer(SimTypeChar(label="Byte"), offset=0)}, name="FWP_BYTE_BLOB", pack=False, align=None), "flags": SimTypeInt(signed=False, label="IKEEXT_CERT_FLAGS")}, name="IKEEXT_CERT_ROOT_CONFIG0", pack=False, align=None), offset=0)}, name="_Anonymous_e__Struct", pack=False, align=None), "outboundEnterpriseStoreConfig": SimTypePointer(SimStruct({"certData": SimStruct({"size": SimTypeInt(signed=False, label="UInt32"), "data": SimTypePointer(SimTypeChar(label="Byte"), offset=0)}, name="FWP_BYTE_BLOB", pack=False, align=None), "flags": SimTypeInt(signed=False, label="IKEEXT_CERT_FLAGS")}, name="IKEEXT_CERT_ROOT_CONFIG0", pack=False, align=None), offset=0), "outboundTrustedRootStoreConfig": SimTypePointer(SimStruct({"certData": SimStruct({"size": SimTypeInt(signed=False, label="UInt32"), "data": SimTypePointer(SimTypeChar(label="Byte"), offset=0)}, name="FWP_BYTE_BLOB", pack=False, align=None), "flags": SimTypeInt(signed=False, label="IKEEXT_CERT_FLAGS")}, name="IKEEXT_CERT_ROOT_CONFIG0", pack=False, align=None), offset=0)}, name="<anon>", label="None"), "flags": SimTypeInt(signed=False, label="IKEEXT_CERT_AUTH")}, name="IKEEXT_CERTIFICATE_AUTHENTICATION0", pack=False, align=None), "cgaAuthentication": SimStruct({"keyContainerName": SimTypePointer(SimTypeChar(label="Char"), offset=0), "cspName": SimTypePointer(SimTypeChar(label="Char"), offset=0), "cspType": SimTypeInt(signed=False, label="UInt32"), "cgaModifier": SimStruct({"byteArray16": SimTypeFixedSizeArray(SimTypeChar(label="Byte"), 16)}, name="FWP_BYTE_ARRAY16", pack=False, align=None), "cgaCollisionCount": SimTypeChar(label="Byte")}, name="IKEEXT_IPV6_CGA_AUTHENTICATION0", pack=False, align=None)}, name="<anon>", label="None")}, name="IKEEXT_AUTHENTICATION_METHOD0", pack=False, align=None), offset=0), "initiatorImpersonationType": SimTypeInt(signed=False, label="IKEEXT_AUTHENTICATION_IMPERSONATION_TYPE")}, name="IKEEXT_EM_POLICY0", pack=False, align=None), offset=0)}, name="IPSEC_TRANSPORT_POLICY0", pack=False, align=None), offset=0), "ikeQmTunnelPolicy": SimTypePointer(SimStruct({"flags": SimTypeInt(signed=False, label="IPSEC_POLICY_FLAG"), "numIpsecProposals": SimTypeInt(signed=False, label="UInt32"), "ipsecProposals": SimTypePointer(SimStruct({"lifetime": SimStruct({"lifetimeSeconds": SimTypeInt(signed=False, label="UInt32"), "lifetimeKilobytes": SimTypeInt(signed=False, label="UInt32"), "lifetimePackets": SimTypeInt(signed=False, label="UInt32")}, name="IPSEC_SA_LIFETIME0", pack=False, align=None), "numSaTransforms": SimTypeInt(signed=False, label="UInt32"), "saTransforms": SimTypePointer(SimStruct({"ipsecTransformType": SimTypeInt(signed=False, label="IPSEC_TRANSFORM_TYPE"), "Anonymous": SimUnion({"ahTransform": SimTypePointer(SimStruct({"authTransformId": SimStruct({"authType": SimTypeInt(signed=False, label="IPSEC_AUTH_TYPE"), "authConfig": SimTypeChar(label="Byte")}, name="IPSEC_AUTH_TRANSFORM_ID0", pack=False, align=None), "cryptoModuleId": SimTypePointer(SimTypeBottom(label="Guid"), offset=0)}, name="IPSEC_AUTH_TRANSFORM0", pack=False, align=None), offset=0), "espAuthTransform": SimTypePointer(SimStruct({"authTransformId": SimStruct({"authType": SimTypeInt(signed=False, label="IPSEC_AUTH_TYPE"), "authConfig": SimTypeChar(label="Byte")}, name="IPSEC_AUTH_TRANSFORM_ID0", pack=False, align=None), "cryptoModuleId": SimTypePointer(SimTypeBottom(label="Guid"), offset=0)}, name="IPSEC_AUTH_TRANSFORM0", pack=False, align=None), offset=0), "espCipherTransform": SimTypePointer(SimStruct({"cipherTransformId": SimStruct({"cipherType": SimTypeInt(signed=False, label="IPSEC_CIPHER_TYPE"), "cipherConfig": SimTypeChar(label="Byte")}, name="IPSEC_CIPHER_TRANSFORM_ID0", pack=False, align=None), "cryptoModuleId": SimTypePointer(SimTypeBottom(label="Guid"), offset=0)}, name="IPSEC_CIPHER_TRANSFORM0", pack=False, align=None), offset=0), "espAuthAndCipherTransform": SimTypePointer(SimStruct({"authTransform": SimStruct({"authTransformId": SimStruct({"authType": SimTypeInt(signed=False, label="IPSEC_AUTH_TYPE"), "authConfig": SimTypeChar(label="Byte")}, name="IPSEC_AUTH_TRANSFORM_ID0", pack=False, align=None), "cryptoModuleId": SimTypePointer(SimTypeBottom(label="Guid"), offset=0)}, name="IPSEC_AUTH_TRANSFORM0", pack=False, align=None), "cipherTransform": SimStruct({"cipherTransformId": SimStruct({"cipherType": SimTypeInt(signed=False, label="IPSEC_CIPHER_TYPE"), "cipherConfig": SimTypeChar(label="Byte")}, name="IPSEC_CIPHER_TRANSFORM_ID0", pack=False, align=None), "cryptoModuleId": SimTypePointer(SimTypeBottom(label="Guid"), offset=0)}, name="IPSEC_CIPHER_TRANSFORM0", pack=False, align=None)}, name="IPSEC_AUTH_AND_CIPHER_TRANSFORM0", pack=False, align=None), offset=0), "espAuthFwTransform": SimTypePointer(SimStruct({"authTransformId": SimStruct({"authType": SimTypeInt(signed=False, label="IPSEC_AUTH_TYPE"), "authConfig": SimTypeChar(label="Byte")}, name="IPSEC_AUTH_TRANSFORM_ID0", pack=False, align=None), "cryptoModuleId": SimTypePointer(SimTypeBottom(label="Guid"), offset=0)}, name="IPSEC_AUTH_TRANSFORM0", pack=False, align=None), offset=0)}, name="<anon>", label="None")}, name="IPSEC_SA_TRANSFORM0", pack=False, align=None), offset=0), "pfsGroup": SimTypeInt(signed=False, label="IPSEC_PFS_GROUP")}, name="IPSEC_PROPOSAL0", pack=False, align=None), offset=0), "tunnelEndpoints": SimStruct({"ipVersion": SimTypeInt(signed=False, label="FWP_IP_VERSION"), "Anonymous1": SimUnion({"localV4Address": SimTypeInt(signed=False, label="UInt32"), "localV6Address": SimTypeFixedSizeArray(SimTypeChar(label="Byte"), 16)}, name="<anon>", label="None"), "Anonymous2": SimUnion({"remoteV4Address": SimTypeInt(signed=False, label="UInt32"), "remoteV6Address": SimTypeFixedSizeArray(SimTypeChar(label="Byte"), 16)}, name="<anon>", label="None")}, name="IPSEC_TUNNEL_ENDPOINTS0", pack=False, align=None), "saIdleTimeout": SimStruct({"idleTimeoutSeconds": SimTypeInt(signed=False, label="UInt32"), "idleTimeoutSecondsFailOver": SimTypeInt(signed=False, label="UInt32")}, name="IPSEC_SA_IDLE_TIMEOUT0", pack=False, align=None), "emPolicy": SimTypePointer(SimStruct({"numAuthenticationMethods": SimTypeInt(signed=False, label="UInt32"), "authenticationMethods": SimTypePointer(SimStruct({"authenticationMethodType": SimTypeInt(signed=False, label="IKEEXT_AUTHENTICATION_METHOD_TYPE"), "Anonymous": SimUnion({"presharedKeyAuthentication": SimStruct({"presharedKey": SimStruct({"size": SimTypeInt(signed=False, label="UInt32"), "data": SimTypePointer(SimTypeChar(label="Byte"), offset=0)}, name="FWP_BYTE_BLOB", pack=False, align=None)}, name="IKEEXT_PRESHARED_KEY_AUTHENTICATION0", pack=False, align=None), "certificateAuthentication": SimStruct({"inboundConfigType": SimTypeInt(signed=False, label="IKEEXT_CERT_CONFIG_TYPE"), "Anonymous1": SimUnion({"Anonymous": SimStruct({"inboundRootArraySize": SimTypeInt(signed=False, label="UInt32"), "inboundRootArray": SimTypePointer(SimStruct({"certData": SimStruct({"size": SimTypeInt(signed=False, label="UInt32"), "data": SimTypePointer(SimTypeChar(label="Byte"), offset=0)}, name="FWP_BYTE_BLOB", pack=False, align=None), "flags": SimTypeInt(signed=False, label="IKEEXT_CERT_FLAGS")}, name="IKEEXT_CERT_ROOT_CONFIG0", pack=False, align=None), offset=0)}, name="_Anonymous_e__Struct", pack=False, align=None), "inboundEnterpriseStoreConfig": SimTypePointer(SimStruct({"certData": SimStruct({"size": SimTypeInt(signed=False, label="UInt32"), "data": SimTypePointer(SimTypeChar(label="Byte"), offset=0)}, name="FWP_BYTE_BLOB", pack=False, align=None), "flags": SimTypeInt(signed=False, label="IKEEXT_CERT_FLAGS")}, name="IKEEXT_CERT_ROOT_CONFIG0", pack=False, align=None), offset=0), "inboundTrustedRootStoreConfig": SimTypePointer(SimStruct({"certData": SimStruct({"size": SimTypeInt(signed=False, label="UInt32"), "data": SimTypePointer(SimTypeChar(label="Byte"), offset=0)}, name="FWP_BYTE_BLOB", pack=False, align=None), "flags": SimTypeInt(signed=False, label="IKEEXT_CERT_FLAGS")}, name="IKEEXT_CERT_ROOT_CONFIG0", pack=False, align=None), offset=0)}, name="<anon>", label="None"), "outboundConfigType": SimTypeInt(signed=False, label="IKEEXT_CERT_CONFIG_TYPE"), "Anonymous2": SimUnion({"Anonymous": SimStruct({"outboundRootArraySize": SimTypeInt(signed=False, label="UInt32"), "outboundRootArray": SimTypePointer(SimStruct({"certData": SimStruct({"size": SimTypeInt(signed=False, label="UInt32"), "data": SimTypePointer(SimTypeChar(label="Byte"), offset=0)}, name="FWP_BYTE_BLOB", pack=False, align=None), "flags": SimTypeInt(signed=False, label="IKEEXT_CERT_FLAGS")}, name="IKEEXT_CERT_ROOT_CONFIG0", pack=False, align=None), offset=0)}, name="_Anonymous_e__Struct", pack=False, align=None), "outboundEnterpriseStoreConfig": SimTypePointer(SimStruct({"certData": SimStruct({"size": SimTypeInt(signed=False, label="UInt32"), "data": SimTypePointer(SimTypeChar(label="Byte"), offset=0)}, name="FWP_BYTE_BLOB", pack=False, align=None), "flags": SimTypeInt(signed=False, label="IKEEXT_CERT_FLAGS")}, name="IKEEXT_CERT_ROOT_CONFIG0", pack=False, align=None), offset=0), "outboundTrustedRootStoreConfig": SimTypePointer(SimStruct({"certData": SimStruct({"size": SimTypeInt(signed=False, label="UInt32"), "data": SimTypePointer(SimTypeChar(label="Byte"), offset=0)}, name="FWP_BYTE_BLOB", pack=False, align=None), "flags": SimTypeInt(signed=False, label="IKEEXT_CERT_FLAGS")}, name="IKEEXT_CERT_ROOT_CONFIG0", pack=False, align=None), offset=0)}, name="<anon>", label="None"), "flags": SimTypeInt(signed=False, label="IKEEXT_CERT_AUTH")}, name="IKEEXT_CERTIFICATE_AUTHENTICATION0", pack=False, align=None), "kerberosAuthentication": SimStruct({"flags": SimTypeInt(signed=False, label="IKEEXT_KERBEROS_AUTHENTICATION_FLAGS")}, name="IKEEXT_KERBEROS_AUTHENTICATION0", pack=False, align=None), "ntlmV2Authentication": SimStruct({"flags": SimTypeInt(signed=False, label="UInt32")}, name="IKEEXT_NTLM_V2_AUTHENTICATION0", pack=False, align=None), "sslAuthentication": SimStruct({"inboundConfigType": SimTypeInt(signed=False, label="IKEEXT_CERT_CONFIG_TYPE"), "Anonymous1": SimUnion({"Anonymous": SimStruct({"inboundRootArraySize": SimTypeInt(signed=False, label="UInt32"), "inboundRootArray": SimTypePointer(SimStruct({"certData": SimStruct({"size": SimTypeInt(signed=False, label="UInt32"), "data": SimTypePointer(SimTypeChar(label="Byte"), offset=0)}, name="FWP_BYTE_BLOB", pack=False, align=None), "flags": SimTypeInt(signed=False, label="IKEEXT_CERT_FLAGS")}, name="IKEEXT_CERT_ROOT_CONFIG0", pack=False, align=None), offset=0)}, name="_Anonymous_e__Struct", pack=False, align=None), "inboundEnterpriseStoreConfig": SimTypePointer(SimStruct({"certData": SimStruct({"size": SimTypeInt(signed=False, label="UInt32"), "data": SimTypePointer(SimTypeChar(label="Byte"), offset=0)}, name="FWP_BYTE_BLOB", pack=False, align=None), "flags": SimTypeInt(signed=False, label="IKEEXT_CERT_FLAGS")}, name="IKEEXT_CERT_ROOT_CONFIG0", pack=False, align=None), offset=0), "inboundTrustedRootStoreConfig": SimTypePointer(SimStruct({"certData": SimStruct({"size": SimTypeInt(signed=False, label="UInt32"), "data": SimTypePointer(SimTypeChar(label="Byte"), offset=0)}, name="FWP_BYTE_BLOB", pack=False, align=None), "flags": SimTypeInt(signed=False, label="IKEEXT_CERT_FLAGS")}, name="IKEEXT_CERT_ROOT_CONFIG0", pack=False, align=None), offset=0)}, name="<anon>", label="None"), "outboundConfigType": SimTypeInt(signed=False, label="IKEEXT_CERT_CONFIG_TYPE"), "Anonymous2": SimUnion({"Anonymous": SimStruct({"outboundRootArraySize": SimTypeInt(signed=False, label="UInt32"), "outboundRootArray": SimTypePointer(SimStruct({"certData": SimStruct({"size": SimTypeInt(signed=False, label="UInt32"), "data": SimTypePointer(SimTypeChar(label="Byte"), offset=0)}, name="FWP_BYTE_BLOB", pack=False, align=None), "flags": SimTypeInt(signed=False, label="IKEEXT_CERT_FLAGS")}, name="IKEEXT_CERT_ROOT_CONFIG0", pack=False, align=None), offset=0)}, name="_Anonymous_e__Struct", pack=False, align=None), "outboundEnterpriseStoreConfig": SimTypePointer(SimStruct({"certData": SimStruct({"size": SimTypeInt(signed=False, label="UInt32"), "data": SimTypePointer(SimTypeChar(label="Byte"), offset=0)}, name="FWP_BYTE_BLOB", pack=False, align=None), "flags": SimTypeInt(signed=False, label="IKEEXT_CERT_FLAGS")}, name="IKEEXT_CERT_ROOT_CONFIG0", pack=False, align=None), offset=0), "outboundTrustedRootStoreConfig": SimTypePointer(SimStruct({"certData": SimStruct({"size": SimTypeInt(signed=False, label="UInt32"), "data": SimTypePointer(SimTypeChar(label="Byte"), offset=0)}, name="FWP_BYTE_BLOB", pack=False, align=None), "flags": SimTypeInt(signed=False, label="IKEEXT_CERT_FLAGS")}, name="IKEEXT_CERT_ROOT_CONFIG0", pack=False, align=None), offset=0)}, name="<anon>", label="None"), "flags": SimTypeInt(signed=False, label="IKEEXT_CERT_AUTH")}, name="IKEEXT_CERTIFICATE_AUTHENTICATION0", pack=False, align=None), "cgaAuthentication": SimStruct({"keyContainerName": SimTypePointer(SimTypeChar(label="Char"), offset=0), "cspName": SimTypePointer(SimTypeChar(label="Char"), offset=0), "cspType": SimTypeInt(signed=False, label="UInt32"), "cgaModifier": SimStruct({"byteArray16": SimTypeFixedSizeArray(SimTypeChar(label="Byte"), 16)}, name="FWP_BYTE_ARRAY16", pack=False, align=None), "cgaCollisionCount": SimTypeChar(label="Byte")}, name="IKEEXT_IPV6_CGA_AUTHENTICATION0", pack=False, align=None)}, name="<anon>", label="None")}, name="IKEEXT_AUTHENTICATION_METHOD0", pack=False, align=None), offset=0), "initiatorImpersonationType": SimTypeInt(signed=False, label="IKEEXT_AUTHENTICATION_IMPERSONATION_TYPE")}, name="IKEEXT_EM_POLICY0", pack=False, align=None), offset=0)}, name="IPSEC_TUNNEL_POLICY0", pack=False, align=None), offset=0), "authipQmTransportPolicy": SimTypePointer(SimStruct({"numIpsecProposals": SimTypeInt(signed=False, label="UInt32"), "ipsecProposals": SimTypePointer(SimStruct({"lifetime": SimStruct({"lifetimeSeconds": SimTypeInt(signed=False, label="UInt32"), "lifetimeKilobytes": SimTypeInt(signed=False, label="UInt32"), "lifetimePackets": SimTypeInt(signed=False, label="UInt32")}, name="IPSEC_SA_LIFETIME0", pack=False, align=None), "numSaTransforms": SimTypeInt(signed=False, label="UInt32"), "saTransforms": SimTypePointer(SimStruct({"ipsecTransformType": SimTypeInt(signed=False, label="IPSEC_TRANSFORM_TYPE"), "Anonymous": SimUnion({"ahTransform": SimTypePointer(SimStruct({"authTransformId": SimStruct({"authType": SimTypeInt(signed=False, label="IPSEC_AUTH_TYPE"), "authConfig": SimTypeChar(label="Byte")}, name="IPSEC_AUTH_TRANSFORM_ID0", pack=False, align=None), "cryptoModuleId": SimTypePointer(SimTypeBottom(label="Guid"), offset=0)}, name="IPSEC_AUTH_TRANSFORM0", pack=False, align=None), offset=0), "espAuthTransform": SimTypePointer(SimStruct({"authTransformId": SimStruct({"authType": SimTypeInt(signed=False, label="IPSEC_AUTH_TYPE"), "authConfig": SimTypeChar(label="Byte")}, name="IPSEC_AUTH_TRANSFORM_ID0", pack=False, align=None), "cryptoModuleId": SimTypePointer(SimTypeBottom(label="Guid"), offset=0)}, name="IPSEC_AUTH_TRANSFORM0", pack=False, align=None), offset=0), "espCipherTransform": SimTypePointer(SimStruct({"cipherTransformId": SimStruct({"cipherType": SimTypeInt(signed=False, label="IPSEC_CIPHER_TYPE"), "cipherConfig": SimTypeChar(label="Byte")}, name="IPSEC_CIPHER_TRANSFORM_ID0", pack=False, align=None), "cryptoModuleId": SimTypePointer(SimTypeBottom(label="Guid"), offset=0)}, name="IPSEC_CIPHER_TRANSFORM0", pack=False, align=None), offset=0), "espAuthAndCipherTransform": SimTypePointer(SimStruct({"authTransform": SimStruct({"authTransformId": SimStruct({"authType": SimTypeInt(signed=False, label="IPSEC_AUTH_TYPE"), "authConfig": SimTypeChar(label="Byte")}, name="IPSEC_AUTH_TRANSFORM_ID0", pack=False, align=None), "cryptoModuleId": SimTypePointer(SimTypeBottom(label="Guid"), offset=0)}, name="IPSEC_AUTH_TRANSFORM0", pack=False, align=None), "cipherTransform": SimStruct({"cipherTransformId": SimStruct({"cipherType": SimTypeInt(signed=False, label="IPSEC_CIPHER_TYPE"), "cipherConfig": SimTypeChar(label="Byte")}, name="IPSEC_CIPHER_TRANSFORM_ID0", pack=False, align=None), "cryptoModuleId": SimTypePointer(SimTypeBottom(label="Guid"), offset=0)}, name="IPSEC_CIPHER_TRANSFORM0", pack=False, align=None)}, name="IPSEC_AUTH_AND_CIPHER_TRANSFORM0", pack=False, align=None), offset=0), "espAuthFwTransform": SimTypePointer(SimStruct({"authTransformId": SimStruct({"authType": SimTypeInt(signed=False, label="IPSEC_AUTH_TYPE"), "authConfig": SimTypeChar(label="Byte")}, name="IPSEC_AUTH_TRANSFORM_ID0", pack=False, align=None), "cryptoModuleId": SimTypePointer(SimTypeBottom(label="Guid"), offset=0)}, name="IPSEC_AUTH_TRANSFORM0", pack=False, align=None), offset=0)}, name="<anon>", label="None")}, name="IPSEC_SA_TRANSFORM0", pack=False, align=None), offset=0), "pfsGroup": SimTypeInt(signed=False, label="IPSEC_PFS_GROUP")}, name="IPSEC_PROPOSAL0", pack=False, align=None), offset=0), "flags": SimTypeInt(signed=False, label="IPSEC_POLICY_FLAG"), "ndAllowClearTimeoutSeconds": SimTypeInt(signed=False, label="UInt32"), "saIdleTimeout": SimStruct({"idleTimeoutSeconds": SimTypeInt(signed=False, label="UInt32"), "idleTimeoutSecondsFailOver": SimTypeInt(signed=False, label="UInt32")}, name="IPSEC_SA_IDLE_TIMEOUT0", pack=False, align=None), "emPolicy": SimTypePointer(SimStruct({"numAuthenticationMethods": SimTypeInt(signed=False, label="UInt32"), "authenticationMethods": SimTypePointer(SimStruct({"authenticationMethodType": SimTypeInt(signed=False, label="IKEEXT_AUTHENTICATION_METHOD_TYPE"), "Anonymous": SimUnion({"presharedKeyAuthentication": SimStruct({"presharedKey": SimStruct({"size": SimTypeInt(signed=False, label="UInt32"), "data": SimTypePointer(SimTypeChar(label="Byte"), offset=0)}, name="FWP_BYTE_BLOB", pack=False, align=None)}, name="IKEEXT_PRESHARED_KEY_AUTHENTICATION0", pack=False, align=None), "certificateAuthentication": SimStruct({"inboundConfigType": SimTypeInt(signed=False, label="IKEEXT_CERT_CONFIG_TYPE"), "Anonymous1": SimUnion({"Anonymous": SimStruct({"inboundRootArraySize": SimTypeInt(signed=False, label="UInt32"), "inboundRootArray": SimTypePointer(SimStruct({"certData": SimStruct({"size": SimTypeInt(signed=False, label="UInt32"), "data": SimTypePointer(SimTypeChar(label="Byte"), offset=0)}, name="FWP_BYTE_BLOB", pack=False, align=None), "flags": SimTypeInt(signed=False, label="IKEEXT_CERT_FLAGS")}, name="IKEEXT_CERT_ROOT_CONFIG0", pack=False, align=None), offset=0)}, name="_Anonymous_e__Struct", pack=False, align=None), "inboundEnterpriseStoreConfig": SimTypePointer(SimStruct({"certData": SimStruct({"size": SimTypeInt(signed=False, label="UInt32"), "data": SimTypePointer(SimTypeChar(label="Byte"), offset=0)}, name="FWP_BYTE_BLOB", pack=False, align=None), "flags": SimTypeInt(signed=False, label="IKEEXT_CERT_FLAGS")}, name="IKEEXT_CERT_ROOT_CONFIG0", pack=False, align=None), offset=0), "inboundTrustedRootStoreConfig": SimTypePointer(SimStruct({"certData": SimStruct({"size": SimTypeInt(signed=False, label="UInt32"), "data": SimTypePointer(SimTypeChar(label="Byte"), offset=0)}, name="FWP_BYTE_BLOB", pack=False, align=None), "flags": SimTypeInt(signed=False, label="IKEEXT_CERT_FLAGS")}, name="IKEEXT_CERT_ROOT_CONFIG0", pack=False, align=None), offset=0)}, name="<anon>", label="None"), "outboundConfigType": SimTypeInt(signed=False, label="IKEEXT_CERT_CONFIG_TYPE"), "Anonymous2": SimUnion({"Anonymous": SimStruct({"outboundRootArraySize": SimTypeInt(signed=False, label="UInt32"), "outboundRootArray": SimTypePointer(SimStruct({"certData": SimStruct({"size": SimTypeInt(signed=False, label="UInt32"), "data": SimTypePointer(SimTypeChar(label="Byte"), offset=0)}, name="FWP_BYTE_BLOB", pack=False, align=None), "flags": SimTypeInt(signed=False, label="IKEEXT_CERT_FLAGS")}, name="IKEEXT_CERT_ROOT_CONFIG0", pack=False, align=None), offset=0)}, name="_Anonymous_e__Struct", pack=False, align=None), "outboundEnterpriseStoreConfig": SimTypePointer(SimStruct({"certData": SimStruct({"size": SimTypeInt(signed=False, label="UInt32"), "data": SimTypePointer(SimTypeChar(label="Byte"), offset=0)}, name="FWP_BYTE_BLOB", pack=False, align=None), "flags": SimTypeInt(signed=False, label="IKEEXT_CERT_FLAGS")}, name="IKEEXT_CERT_ROOT_CONFIG0", pack=False, align=None), offset=0), "outboundTrustedRootStoreConfig": SimTypePointer(SimStruct({"certData": SimStruct({"size": SimTypeInt(signed=False, label="UInt32"), "data": SimTypePointer(SimTypeChar(label="Byte"), offset=0)}, name="FWP_BYTE_BLOB", pack=False, align=None), "flags": SimTypeInt(signed=False, label="IKEEXT_CERT_FLAGS")}, name="IKEEXT_CERT_ROOT_CONFIG0", pack=False, align=None), offset=0)}, name="<anon>", label="None"), "flags": SimTypeInt(signed=False, label="IKEEXT_CERT_AUTH")}, name="IKEEXT_CERTIFICATE_AUTHENTICATION0", pack=False, align=None), "kerberosAuthentication": SimStruct({"flags": SimTypeInt(signed=False, label="IKEEXT_KERBEROS_AUTHENTICATION_FLAGS")}, name="IKEEXT_KERBEROS_AUTHENTICATION0", pack=False, align=None), "ntlmV2Authentication": SimStruct({"flags": SimTypeInt(signed=False, label="UInt32")}, name="IKEEXT_NTLM_V2_AUTHENTICATION0", pack=False, align=None), "sslAuthentication": SimStruct({"inboundConfigType": SimTypeInt(signed=False, label="IKEEXT_CERT_CONFIG_TYPE"), "Anonymous1": SimUnion({"Anonymous": SimStruct({"inboundRootArraySize": SimTypeInt(signed=False, label="UInt32"), "inboundRootArray": SimTypePointer(SimStruct({"certData": SimStruct({"size": SimTypeInt(signed=False, label="UInt32"), "data": SimTypePointer(SimTypeChar(label="Byte"), offset=0)}, name="FWP_BYTE_BLOB", pack=False, align=None), "flags": SimTypeInt(signed=False, label="IKEEXT_CERT_FLAGS")}, name="IKEEXT_CERT_ROOT_CONFIG0", pack=False, align=None), offset=0)}, name="_Anonymous_e__Struct", pack=False, align=None), "inboundEnterpriseStoreConfig": SimTypePointer(SimStruct({"certData": SimStruct({"size": SimTypeInt(signed=False, label="UInt32"), "data": SimTypePointer(SimTypeChar(label="Byte"), offset=0)}, name="FWP_BYTE_BLOB", pack=False, align=None), "flags": SimTypeInt(signed=False, label="IKEEXT_CERT_FLAGS")}, name="IKEEXT_CERT_ROOT_CONFIG0", pack=False, align=None), offset=0), "inboundTrustedRootStoreConfig": SimTypePointer(SimStruct({"certData": SimStruct({"size": SimTypeInt(signed=False, label="UInt32"), "data": SimTypePointer(SimTypeChar(label="Byte"), offset=0)}, name="FWP_BYTE_BLOB", pack=False, align=None), "flags": SimTypeInt(signed=False, label="IKEEXT_CERT_FLAGS")}, name="IKEEXT_CERT_ROOT_CONFIG0", pack=False, align=None), offset=0)}, name="<anon>", label="None"), "outboundConfigType": SimTypeInt(signed=False, label="IKEEXT_CERT_CONFIG_TYPE"), "Anonymous2": SimUnion({"Anonymous": SimStruct({"outboundRootArraySize": SimTypeInt(signed=False, label="UInt32"), "outboundRootArray": SimTypePointer(SimStruct({"certData": SimStruct({"size": SimTypeInt(signed=False, label="UInt32"), "data": SimTypePointer(SimTypeChar(label="Byte"), offset=0)}, name="FWP_BYTE_BLOB", pack=False, align=None), "flags": SimTypeInt(signed=False, label="IKEEXT_CERT_FLAGS")}, name="IKEEXT_CERT_ROOT_CONFIG0", pack=False, align=None), offset=0)}, name="_Anonymous_e__Struct", pack=False, align=None), "outboundEnterpriseStoreConfig": SimTypePointer(SimStruct({"certData": SimStruct({"size": SimTypeInt(signed=False, label="UInt32"), "data": SimTypePointer(SimTypeChar(label="Byte"), offset=0)}, name="FWP_BYTE_BLOB", pack=False, align=None), "flags": SimTypeInt(signed=False, label="IKEEXT_CERT_FLAGS")}, name="IKEEXT_CERT_ROOT_CONFIG0", pack=False, align=None), offset=0), "outboundTrustedRootStoreConfig": SimTypePointer(SimStruct({"certData": SimStruct({"size": SimTypeInt(signed=False, label="UInt32"), "data": SimTypePointer(SimTypeChar(label="Byte"), offset=0)}, name="FWP_BYTE_BLOB", pack=False, align=None), "flags": SimTypeInt(signed=False, label="IKEEXT_CERT_FLAGS")}, name="IKEEXT_CERT_ROOT_CONFIG0", pack=False, align=None), offset=0)}, name="<anon>", label="None"), "flags": SimTypeInt(signed=False, label="IKEEXT_CERT_AUTH")}, name="IKEEXT_CERTIFICATE_AUTHENTICATION0", pack=False, align=None), "cgaAuthentication": SimStruct({"keyContainerName": SimTypePointer(SimTypeChar(label="Char"), offset=0), "cspName": SimTypePointer(SimTypeChar(label="Char"), offset=0), "cspType": SimTypeInt(signed=False, label="UInt32"), "cgaModifier": SimStruct({"byteArray16": SimTypeFixedSizeArray(SimTypeChar(label="Byte"), 16)}, name="FWP_BYTE_ARRAY16", pack=False, align=None), "cgaCollisionCount": SimTypeChar(label="Byte")}, name="IKEEXT_IPV6_CGA_AUTHENTICATION0", pack=False, align=None)}, name="<anon>", label="None")}, name="IKEEXT_AUTHENTICATION_METHOD0", pack=False, align=None), offset=0), "initiatorImpersonationType": SimTypeInt(signed=False, label="IKEEXT_AUTHENTICATION_IMPERSONATION_TYPE")}, name="IKEEXT_EM_POLICY0", pack=False, align=None), offset=0)}, name="IPSEC_TRANSPORT_POLICY0", pack=False, align=None), offset=0), "authipQmTunnelPolicy": SimTypePointer(SimStruct({"flags": SimTypeInt(signed=False, label="IPSEC_POLICY_FLAG"), "numIpsecProposals": SimTypeInt(signed=False, label="UInt32"), "ipsecProposals": SimTypePointer(SimStruct({"lifetime": SimStruct({"lifetimeSeconds": SimTypeInt(signed=False, label="UInt32"), "lifetimeKilobytes": SimTypeInt(signed=False, label="UInt32"), "lifetimePackets": SimTypeInt(signed=False, label="UInt32")}, name="IPSEC_SA_LIFETIME0", pack=False, align=None), "numSaTransforms": SimTypeInt(signed=False, label="UInt32"), "saTransforms": SimTypePointer(SimStruct({"ipsecTransformType": SimTypeInt(signed=False, label="IPSEC_TRANSFORM_TYPE"), "Anonymous": SimUnion({"ahTransform": SimTypePointer(SimStruct({"authTransformId": SimStruct({"authType": SimTypeInt(signed=False, label="IPSEC_AUTH_TYPE"), "authConfig": SimTypeChar(label="Byte")}, name="IPSEC_AUTH_TRANSFORM_ID0", pack=False, align=None), "cryptoModuleId": SimTypePointer(SimTypeBottom(label="Guid"), offset=0)}, name="IPSEC_AUTH_TRANSFORM0", pack=False, align=None), offset=0), "espAuthTransform": SimTypePointer(SimStruct({"authTransformId": SimStruct({"authType": SimTypeInt(signed=False, label="IPSEC_AUTH_TYPE"), "authConfig": SimTypeChar(label="Byte")}, name="IPSEC_AUTH_TRANSFORM_ID0", pack=False, align=None), "cryptoModuleId": SimTypePointer(SimTypeBottom(label="Guid"), offset=0)}, name="IPSEC_AUTH_TRANSFORM0", pack=False, align=None), offset=0), "espCipherTransform": SimTypePointer(SimStruct({"cipherTransformId": SimStruct({"cipherType": SimTypeInt(signed=False, label="IPSEC_CIPHER_TYPE"), "cipherConfig": SimTypeChar(label="Byte")}, name="IPSEC_CIPHER_TRANSFORM_ID0", pack=False, align=None), "cryptoModuleId": SimTypePointer(SimTypeBottom(label="Guid"), offset=0)}, name="IPSEC_CIPHER_TRANSFORM0", pack=False, align=None), offset=0), "espAuthAndCipherTransform": SimTypePointer(SimStruct({"authTransform": SimStruct({"authTransformId": SimStruct({"authType": SimTypeInt(signed=False, label="IPSEC_AUTH_TYPE"), "authConfig": SimTypeChar(label="Byte")}, name="IPSEC_AUTH_TRANSFORM_ID0", pack=False, align=None), "cryptoModuleId": SimTypePointer(SimTypeBottom(label="Guid"), offset=0)}, name="IPSEC_AUTH_TRANSFORM0", pack=False, align=None), "cipherTransform": SimStruct({"cipherTransformId": SimStruct({"cipherType": SimTypeInt(signed=False, label="IPSEC_CIPHER_TYPE"), "cipherConfig": SimTypeChar(label="Byte")}, name="IPSEC_CIPHER_TRANSFORM_ID0", pack=False, align=None), "cryptoModuleId": SimTypePointer(SimTypeBottom(label="Guid"), offset=0)}, name="IPSEC_CIPHER_TRANSFORM0", pack=False, align=None)}, name="IPSEC_AUTH_AND_CIPHER_TRANSFORM0", pack=False, align=None), offset=0), "espAuthFwTransform": SimTypePointer(SimStruct({"authTransformId": SimStruct({"authType": SimTypeInt(signed=False, label="IPSEC_AUTH_TYPE"), "authConfig": SimTypeChar(label="Byte")}, name="IPSEC_AUTH_TRANSFORM_ID0", pack=False, align=None), "cryptoModuleId": SimTypePointer(SimTypeBottom(label="Guid"), offset=0)}, name="IPSEC_AUTH_TRANSFORM0", pack=False, align=None), offset=0)}, name="<anon>", label="None")}, name="IPSEC_SA_TRANSFORM0", pack=False, align=None), offset=0), "pfsGroup": SimTypeInt(signed=False, label="IPSEC_PFS_GROUP")}, name="IPSEC_PROPOSAL0", pack=False, align=None), offset=0), "tunnelEndpoints": SimStruct({"ipVersion": SimTypeInt(signed=False, label="FWP_IP_VERSION"), "Anonymous1": SimUnion({"localV4Address": SimTypeInt(signed=False, label="UInt32"), "localV6Address": SimTypeFixedSizeArray(SimTypeChar(label="Byte"), 16)}, name="<anon>", label="None"), "Anonymous2": SimUnion({"remoteV4Address": SimTypeInt(signed=False, label="UInt32"), "remoteV6Address": SimTypeFixedSizeArray(SimTypeChar(label="Byte"), 16)}, name="<anon>", label="None")}, name="IPSEC_TUNNEL_ENDPOINTS0", pack=False, align=None), "saIdleTimeout": SimStruct({"idleTimeoutSeconds": SimTypeInt(signed=False, label="UInt32"), "idleTimeoutSecondsFailOver": SimTypeInt(signed=False, label="UInt32")}, name="IPSEC_SA_IDLE_TIMEOUT0", pack=False, align=None), "emPolicy": SimTypePointer(SimStruct({"numAuthenticationMethods": SimTypeInt(signed=False, label="UInt32"), "authenticationMethods": SimTypePointer(SimStruct({"authenticationMethodType": SimTypeInt(signed=False, label="IKEEXT_AUTHENTICATION_METHOD_TYPE"), "Anonymous": SimUnion({"presharedKeyAuthentication": SimStruct({"presharedKey": SimStruct({"size": SimTypeInt(signed=False, label="UInt32"), "data": SimTypePointer(SimTypeChar(label="Byte"), offset=0)}, name="FWP_BYTE_BLOB", pack=False, align=None)}, name="IKEEXT_PRESHARED_KEY_AUTHENTICATION0", pack=False, align=None), "certificateAuthentication": SimStruct({"inboundConfigType": SimTypeInt(signed=False, label="IKEEXT_CERT_CONFIG_TYPE"), "Anonymous1": SimUnion({"Anonymous": SimStruct({"inboundRootArraySize": SimTypeInt(signed=False, label="UInt32"), "inboundRootArray": SimTypePointer(SimStruct({"certData": SimStruct({"size": SimTypeInt(signed=False, label="UInt32"), "data": SimTypePointer(SimTypeChar(label="Byte"), offset=0)}, name="FWP_BYTE_BLOB", pack=False, align=None), "flags": SimTypeInt(signed=False, label="IKEEXT_CERT_FLAGS")}, name="IKEEXT_CERT_ROOT_CONFIG0", pack=False, align=None), offset=0)}, name="_Anonymous_e__Struct", pack=False, align=None), "inboundEnterpriseStoreConfig": SimTypePointer(SimStruct({"certData": SimStruct({"size": SimTypeInt(signed=False, label="UInt32"), "data": SimTypePointer(SimTypeChar(label="Byte"), offset=0)}, name="FWP_BYTE_BLOB", pack=False, align=None), "flags": SimTypeInt(signed=False, label="IKEEXT_CERT_FLAGS")}, name="IKEEXT_CERT_ROOT_CONFIG0", pack=False, align=None), offset=0), "inboundTrustedRootStoreConfig": SimTypePointer(SimStruct({"certData": SimStruct({"size": SimTypeInt(signed=False, label="UInt32"), "data": SimTypePointer(SimTypeChar(label="Byte"), offset=0)}, name="FWP_BYTE_BLOB", pack=False, align=None), "flags": SimTypeInt(signed=False, label="IKEEXT_CERT_FLAGS")}, name="IKEEXT_CERT_ROOT_CONFIG0", pack=False, align=None), offset=0)}, name="<anon>", label="None"), "outboundConfigType": SimTypeInt(signed=False, label="IKEEXT_CERT_CONFIG_TYPE"), "Anonymous2": SimUnion({"Anonymous": SimStruct({"outboundRootArraySize": SimTypeInt(signed=False, label="UInt32"), "outboundRootArray": SimTypePointer(SimStruct({"certData": SimStruct({"size": SimTypeInt(signed=False, label="UInt32"), "data": SimTypePointer(SimTypeChar(label="Byte"), offset=0)}, name="FWP_BYTE_BLOB", pack=False, align=None), "flags": SimTypeInt(signed=False, label="IKEEXT_CERT_FLAGS")}, name="IKEEXT_CERT_ROOT_CONFIG0", pack=False, align=None), offset=0)}, name="_Anonymous_e__Struct", pack=False, align=None), "outboundEnterpriseStoreConfig": SimTypePointer(SimStruct({"certData": SimStruct({"size": SimTypeInt(signed=False, label="UInt32"), "data": SimTypePointer(SimTypeChar(label="Byte"), offset=0)}, name="FWP_BYTE_BLOB", pack=False, align=None), "flags": SimTypeInt(signed=False, label="IKEEXT_CERT_FLAGS")}, name="IKEEXT_CERT_ROOT_CONFIG0", pack=False, align=None), offset=0), "outboundTrustedRootStoreConfig": SimTypePointer(SimStruct({"certData": SimStruct({"size": SimTypeInt(signed=False, label="UInt32"), "data": SimTypePointer(SimTypeChar(label="Byte"), offset=0)}, name="FWP_BYTE_BLOB", pack=False, align=None), "flags": SimTypeInt(signed=False, label="IKEEXT_CERT_FLAGS")}, name="IKEEXT_CERT_ROOT_CONFIG0", pack=False, align=None), offset=0)}, name="<anon>", label="None"), "flags": SimTypeInt(signed=False, label="IKEEXT_CERT_AUTH")}, name="IKEEXT_CERTIFICATE_AUTHENTICATION0", pack=False, align=None), "kerberosAuthentication": SimStruct({"flags": SimTypeInt(signed=False, label="IKEEXT_KERBEROS_AUTHENTICATION_FLAGS")}, name="IKEEXT_KERBEROS_AUTHENTICATION0", pack=False, align=None), "ntlmV2Authentication": SimStruct({"flags": SimTypeInt(signed=False, label="UInt32")}, name="IKEEXT_NTLM_V2_AUTHENTICATION0", pack=False, align=None), "sslAuthentication": SimStruct({"inboundConfigType": SimTypeInt(signed=False, label="IKEEXT_CERT_CONFIG_TYPE"), "Anonymous1": SimUnion({"Anonymous": SimStruct({"inboundRootArraySize": SimTypeInt(signed=False, label="UInt32"), "inboundRootArray": SimTypePointer(SimStruct({"certData": SimStruct({"size": SimTypeInt(signed=False, label="UInt32"), "data": SimTypePointer(SimTypeChar(label="Byte"), offset=0)}, name="FWP_BYTE_BLOB", pack=False, align=None), "flags": SimTypeInt(signed=False, label="IKEEXT_CERT_FLAGS")}, name="IKEEXT_CERT_ROOT_CONFIG0", pack=False, align=None), offset=0)}, name="_Anonymous_e__Struct", pack=False, align=None), "inboundEnterpriseStoreConfig": SimTypePointer(SimStruct({"certData": SimStruct({"size": SimTypeInt(signed=False, label="UInt32"), "data": SimTypePointer(SimTypeChar(label="Byte"), offset=0)}, name="FWP_BYTE_BLOB", pack=False, align=None), "flags": SimTypeInt(signed=False, label="IKEEXT_CERT_FLAGS")}, name="IKEEXT_CERT_ROOT_CONFIG0", pack=False, align=None), offset=0), "inboundTrustedRootStoreConfig": SimTypePointer(SimStruct({"certData": SimStruct({"size": SimTypeInt(signed=False, label="UInt32"), "data": SimTypePointer(SimTypeChar(label="Byte"), offset=0)}, name="FWP_BYTE_BLOB", pack=False, align=None), "flags": SimTypeInt(signed=False, label="IKEEXT_CERT_FLAGS")}, name="IKEEXT_CERT_ROOT_CONFIG0", pack=False, align=None), offset=0)}, name="<anon>", label="None"), "outboundConfigType": SimTypeInt(signed=False, label="IKEEXT_CERT_CONFIG_TYPE"), "Anonymous2": SimUnion({"Anonymous": SimStruct({"outboundRootArraySize": SimTypeInt(signed=False, label="UInt32"), "outboundRootArray": SimTypePointer(SimStruct({"certData": SimStruct({"size": SimTypeInt(signed=False, label="UInt32"), "data": SimTypePointer(SimTypeChar(label="Byte"), offset=0)}, name="FWP_BYTE_BLOB", pack=False, align=None), "flags": SimTypeInt(signed=False, label="IKEEXT_CERT_FLAGS")}, name="IKEEXT_CERT_ROOT_CONFIG0", pack=False, align=None), offset=0)}, name="_Anonymous_e__Struct", pack=False, align=None), "outboundEnterpriseStoreConfig": SimTypePointer(SimStruct({"certData": SimStruct({"size": SimTypeInt(signed=False, label="UInt32"), "data": SimTypePointer(SimTypeChar(label="Byte"), offset=0)}, name="FWP_BYTE_BLOB", pack=False, align=None), "flags": SimTypeInt(signed=False, label="IKEEXT_CERT_FLAGS")}, name="IKEEXT_CERT_ROOT_CONFIG0", pack=False, align=None), offset=0), "outboundTrustedRootStoreConfig": SimTypePointer(SimStruct({"certData": SimStruct({"size": SimTypeInt(signed=False, label="UInt32"), "data": SimTypePointer(SimTypeChar(label="Byte"), offset=0)}, name="FWP_BYTE_BLOB", pack=False, align=None), "flags": SimTypeInt(signed=False, label="IKEEXT_CERT_FLAGS")}, name="IKEEXT_CERT_ROOT_CONFIG0", pack=False, align=None), offset=0)}, name="<anon>", label="None"), "flags": SimTypeInt(signed=False, label="IKEEXT_CERT_AUTH")}, name="IKEEXT_CERTIFICATE_AUTHENTICATION0", pack=False, align=None), "cgaAuthentication": SimStruct({"keyContainerName": SimTypePointer(SimTypeChar(label="Char"), offset=0), "cspName": SimTypePointer(SimTypeChar(label="Char"), offset=0), "cspType": SimTypeInt(signed=False, label="UInt32"), "cgaModifier": SimStruct({"byteArray16": SimTypeFixedSizeArray(SimTypeChar(label="Byte"), 16)}, name="FWP_BYTE_ARRAY16", pack=False, align=None), "cgaCollisionCount": SimTypeChar(label="Byte")}, name="IKEEXT_IPV6_CGA_AUTHENTICATION0", pack=False, align=None)}, name="<anon>", label="None")}, name="IKEEXT_AUTHENTICATION_METHOD0", pack=False, align=None), offset=0), "initiatorImpersonationType": SimTypeInt(signed=False, label="IKEEXT_AUTHENTICATION_IMPERSONATION_TYPE")}, name="IKEEXT_EM_POLICY0", pack=False, align=None), offset=0)}, name="IPSEC_TUNNEL_POLICY0", pack=False, align=None), offset=0), "ikeMmPolicy": SimTypePointer(SimStruct({"softExpirationTime": SimTypeInt(signed=False, label="UInt32"), "numAuthenticationMethods": SimTypeInt(signed=False, label="UInt32"), "authenticationMethods": SimTypePointer(SimStruct({"authenticationMethodType": SimTypeInt(signed=False, label="IKEEXT_AUTHENTICATION_METHOD_TYPE"), "Anonymous": SimUnion({"presharedKeyAuthentication": SimStruct({"presharedKey": SimStruct({"size": SimTypeInt(signed=False, label="UInt32"), "data": SimTypePointer(SimTypeChar(label="Byte"), offset=0)}, name="FWP_BYTE_BLOB", pack=False, align=None)}, name="IKEEXT_PRESHARED_KEY_AUTHENTICATION0", pack=False, align=None), "certificateAuthentication": SimStruct({"inboundConfigType": SimTypeInt(signed=False, label="IKEEXT_CERT_CONFIG_TYPE"), "Anonymous1": SimUnion({"Anonymous": SimStruct({"inboundRootArraySize": SimTypeInt(signed=False, label="UInt32"), "inboundRootArray": SimTypePointer(SimStruct({"certData": SimStruct({"size": SimTypeInt(signed=False, label="UInt32"), "data": SimTypePointer(SimTypeChar(label="Byte"), offset=0)}, name="FWP_BYTE_BLOB", pack=False, align=None), "flags": SimTypeInt(signed=False, label="IKEEXT_CERT_FLAGS")}, name="IKEEXT_CERT_ROOT_CONFIG0", pack=False, align=None), offset=0)}, name="_Anonymous_e__Struct", pack=False, align=None), "inboundEnterpriseStoreConfig": SimTypePointer(SimStruct({"certData": SimStruct({"size": SimTypeInt(signed=False, label="UInt32"), "data": SimTypePointer(SimTypeChar(label="Byte"), offset=0)}, name="FWP_BYTE_BLOB", pack=False, align=None), "flags": SimTypeInt(signed=False, label="IKEEXT_CERT_FLAGS")}, name="IKEEXT_CERT_ROOT_CONFIG0", pack=False, align=None), offset=0), "inboundTrustedRootStoreConfig": SimTypePointer(SimStruct({"certData": SimStruct({"size": SimTypeInt(signed=False, label="UInt32"), "data": SimTypePointer(SimTypeChar(label="Byte"), offset=0)}, name="FWP_BYTE_BLOB", pack=False, align=None), "flags": SimTypeInt(signed=False, label="IKEEXT_CERT_FLAGS")}, name="IKEEXT_CERT_ROOT_CONFIG0", pack=False, align=None), offset=0)}, name="<anon>", label="None"), "outboundConfigType": SimTypeInt(signed=False, label="IKEEXT_CERT_CONFIG_TYPE"), "Anonymous2": SimUnion({"Anonymous": SimStruct({"outboundRootArraySize": SimTypeInt(signed=False, label="UInt32"), "outboundRootArray": SimTypePointer(SimStruct({"certData": SimStruct({"size": SimTypeInt(signed=False, label="UInt32"), "data": SimTypePointer(SimTypeChar(label="Byte"), offset=0)}, name="FWP_BYTE_BLOB", pack=False, align=None), "flags": SimTypeInt(signed=False, label="IKEEXT_CERT_FLAGS")}, name="IKEEXT_CERT_ROOT_CONFIG0", pack=False, align=None), offset=0)}, name="_Anonymous_e__Struct", pack=False, align=None), "outboundEnterpriseStoreConfig": SimTypePointer(SimStruct({"certData": SimStruct({"size": SimTypeInt(signed=False, label="UInt32"), "data": SimTypePointer(SimTypeChar(label="Byte"), offset=0)}, name="FWP_BYTE_BLOB", pack=False, align=None), "flags": SimTypeInt(signed=False, label="IKEEXT_CERT_FLAGS")}, name="IKEEXT_CERT_ROOT_CONFIG0", pack=False, align=None), offset=0), "outboundTrustedRootStoreConfig": SimTypePointer(SimStruct({"certData": SimStruct({"size": SimTypeInt(signed=False, label="UInt32"), "data": SimTypePointer(SimTypeChar(label="Byte"), offset=0)}, name="FWP_BYTE_BLOB", pack=False, align=None), "flags": SimTypeInt(signed=False, label="IKEEXT_CERT_FLAGS")}, name="IKEEXT_CERT_ROOT_CONFIG0", pack=False, align=None), offset=0)}, name="<anon>", label="None"), "flags": SimTypeInt(signed=False, label="IKEEXT_CERT_AUTH")}, name="IKEEXT_CERTIFICATE_AUTHENTICATION0", pack=False, align=None), "kerberosAuthentication": SimStruct({"flags": SimTypeInt(signed=False, label="IKEEXT_KERBEROS_AUTHENTICATION_FLAGS")}, name="IKEEXT_KERBEROS_AUTHENTICATION0", pack=False, align=None), "ntlmV2Authentication": SimStruct({"flags": SimTypeInt(signed=False, label="UInt32")}, name="IKEEXT_NTLM_V2_AUTHENTICATION0", pack=False, align=None), "sslAuthentication": SimStruct({"inboundConfigType": SimTypeInt(signed=False, label="IKEEXT_CERT_CONFIG_TYPE"), "Anonymous1": SimUnion({"Anonymous": SimStruct({"inboundRootArraySize": SimTypeInt(signed=False, label="UInt32"), "inboundRootArray": SimTypePointer(SimStruct({"certData": SimStruct({"size": SimTypeInt(signed=False, label="UInt32"), "data": SimTypePointer(SimTypeChar(label="Byte"), offset=0)}, name="FWP_BYTE_BLOB", pack=False, align=None), "flags": SimTypeInt(signed=False, label="IKEEXT_CERT_FLAGS")}, name="IKEEXT_CERT_ROOT_CONFIG0", pack=False, align=None), offset=0)}, name="_Anonymous_e__Struct", pack=False, align=None), "inboundEnterpriseStoreConfig": SimTypePointer(SimStruct({"certData": SimStruct({"size": SimTypeInt(signed=False, label="UInt32"), "data": SimTypePointer(SimTypeChar(label="Byte"), offset=0)}, name="FWP_BYTE_BLOB", pack=False, align=None), "flags": SimTypeInt(signed=False, label="IKEEXT_CERT_FLAGS")}, name="IKEEXT_CERT_ROOT_CONFIG0", pack=False, align=None), offset=0), "inboundTrustedRootStoreConfig": SimTypePointer(SimStruct({"certData": SimStruct({"size": SimTypeInt(signed=False, label="UInt32"), "data": SimTypePointer(SimTypeChar(label="Byte"), offset=0)}, name="FWP_BYTE_BLOB", pack=False, align=None), "flags": SimTypeInt(signed=False, label="IKEEXT_CERT_FLAGS")}, name="IKEEXT_CERT_ROOT_CONFIG0", pack=False, align=None), offset=0)}, name="<anon>", label="None"), "outboundConfigType": SimTypeInt(signed=False, label="IKEEXT_CERT_CONFIG_TYPE"), "Anonymous2": SimUnion({"Anonymous": SimStruct({"outboundRootArraySize": SimTypeInt(signed=False, label="UInt32"), "outboundRootArray": SimTypePointer(SimStruct({"certData": SimStruct({"size": SimTypeInt(signed=False, label="UInt32"), "data": SimTypePointer(SimTypeChar(label="Byte"), offset=0)}, name="FWP_BYTE_BLOB", pack=False, align=None), "flags": SimTypeInt(signed=False, label="IKEEXT_CERT_FLAGS")}, name="IKEEXT_CERT_ROOT_CONFIG0", pack=False, align=None), offset=0)}, name="_Anonymous_e__Struct", pack=False, align=None), "outboundEnterpriseStoreConfig": SimTypePointer(SimStruct({"certData": SimStruct({"size": SimTypeInt(signed=False, label="UInt32"), "data": SimTypePointer(SimTypeChar(label="Byte"), offset=0)}, name="FWP_BYTE_BLOB", pack=False, align=None), "flags": SimTypeInt(signed=False, label="IKEEXT_CERT_FLAGS")}, name="IKEEXT_CERT_ROOT_CONFIG0", pack=False, align=None), offset=0), "outboundTrustedRootStoreConfig": SimTypePointer(SimStruct({"certData": SimStruct({"size": SimTypeInt(signed=False, label="UInt32"), "data": SimTypePointer(SimTypeChar(label="Byte"), offset=0)}, name="FWP_BYTE_BLOB", pack=False, align=None), "flags": SimTypeInt(signed=False, label="IKEEXT_CERT_FLAGS")}, name="IKEEXT_CERT_ROOT_CONFIG0", pack=False, align=None), offset=0)}, name="<anon>", label="None"), "flags": SimTypeInt(signed=False, label="IKEEXT_CERT_AUTH")}, name="IKEEXT_CERTIFICATE_AUTHENTICATION0", pack=False, align=None), "cgaAuthentication": SimStruct({"keyContainerName": SimTypePointer(SimTypeChar(label="Char"), offset=0), "cspName": SimTypePointer(SimTypeChar(label="Char"), offset=0), "cspType": SimTypeInt(signed=False, label="UInt32"), "cgaModifier": SimStruct({"byteArray16": SimTypeFixedSizeArray(SimTypeChar(label="Byte"), 16)}, name="FWP_BYTE_ARRAY16", pack=False, align=None), "cgaCollisionCount": SimTypeChar(label="Byte")}, name="IKEEXT_IPV6_CGA_AUTHENTICATION0", pack=False, align=None)}, name="<anon>", label="None")}, name="IKEEXT_AUTHENTICATION_METHOD0", pack=False, align=None), offset=0), "initiatorImpersonationType": SimTypeInt(signed=False, label="IKEEXT_AUTHENTICATION_IMPERSONATION_TYPE"), "numIkeProposals": SimTypeInt(signed=False, label="UInt32"), "ikeProposals": SimTypePointer(SimStruct({"cipherAlgorithm": SimStruct({"algoIdentifier": SimTypeInt(signed=False, label="IKEEXT_CIPHER_TYPE"), "keyLen": SimTypeInt(signed=False, label="UInt32"), "rounds": SimTypeInt(signed=False, label="UInt32")}, name="IKEEXT_CIPHER_ALGORITHM0", pack=False, align=None), "integrityAlgorithm": SimStruct({"algoIdentifier": SimTypeInt(signed=False, label="IKEEXT_INTEGRITY_TYPE")}, name="IKEEXT_INTEGRITY_ALGORITHM0", pack=False, align=None), "maxLifetimeSeconds": SimTypeInt(signed=False, label="UInt32"), "dhGroup": SimTypeInt(signed=False, label="IKEEXT_DH_GROUP"), "quickModeLimit": SimTypeInt(signed=False, label="UInt32")}, name="IKEEXT_PROPOSAL0", pack=False, align=None), offset=0), "flags": SimTypeInt(signed=False, label="IKEEXT_POLICY_FLAG"), "maxDynamicFilters": SimTypeInt(signed=False, label="UInt32")}, name="IKEEXT_POLICY0", pack=False, align=None), offset=0), "authIpMmPolicy": SimTypePointer(SimStruct({"softExpirationTime": SimTypeInt(signed=False, label="UInt32"), "numAuthenticationMethods": SimTypeInt(signed=False, label="UInt32"), "authenticationMethods": SimTypePointer(SimStruct({"authenticationMethodType": SimTypeInt(signed=False, label="IKEEXT_AUTHENTICATION_METHOD_TYPE"), "Anonymous": SimUnion({"presharedKeyAuthentication": SimStruct({"presharedKey": SimStruct({"size": SimTypeInt(signed=False, label="UInt32"), "data": SimTypePointer(SimTypeChar(label="Byte"), offset=0)}, name="FWP_BYTE_BLOB", pack=False, align=None)}, name="IKEEXT_PRESHARED_KEY_AUTHENTICATION0", pack=False, align=None), "certificateAuthentication": SimStruct({"inboundConfigType": SimTypeInt(signed=False, label="IKEEXT_CERT_CONFIG_TYPE"), "Anonymous1": SimUnion({"Anonymous": SimStruct({"inboundRootArraySize": SimTypeInt(signed=False, label="UInt32"), "inboundRootArray": SimTypePointer(SimStruct({"certData": SimStruct({"size": SimTypeInt(signed=False, label="UInt32"), "data": SimTypePointer(SimTypeChar(label="Byte"), offset=0)}, name="FWP_BYTE_BLOB", pack=False, align=None), "flags": SimTypeInt(signed=False, label="IKEEXT_CERT_FLAGS")}, name="IKEEXT_CERT_ROOT_CONFIG0", pack=False, align=None), offset=0)}, name="_Anonymous_e__Struct", pack=False, align=None), "inboundEnterpriseStoreConfig": SimTypePointer(SimStruct({"certData": SimStruct({"size": SimTypeInt(signed=False, label="UInt32"), "data": SimTypePointer(SimTypeChar(label="Byte"), offset=0)}, name="FWP_BYTE_BLOB", pack=False, align=None), "flags": SimTypeInt(signed=False, label="IKEEXT_CERT_FLAGS")}, name="IKEEXT_CERT_ROOT_CONFIG0", pack=False, align=None), offset=0), "inboundTrustedRootStoreConfig": SimTypePointer(SimStruct({"certData": SimStruct({"size": SimTypeInt(signed=False, label="UInt32"), "data": SimTypePointer(SimTypeChar(label="Byte"), offset=0)}, name="FWP_BYTE_BLOB", pack=False, align=None), "flags": SimTypeInt(signed=False, label="IKEEXT_CERT_FLAGS")}, name="IKEEXT_CERT_ROOT_CONFIG0", pack=False, align=None), offset=0)}, name="<anon>", label="None"), "outboundConfigType": SimTypeInt(signed=False, label="IKEEXT_CERT_CONFIG_TYPE"), "Anonymous2": SimUnion({"Anonymous": SimStruct({"outboundRootArraySize": SimTypeInt(signed=False, label="UInt32"), "outboundRootArray": SimTypePointer(SimStruct({"certData": SimStruct({"size": SimTypeInt(signed=False, label="UInt32"), "data": SimTypePointer(SimTypeChar(label="Byte"), offset=0)}, name="FWP_BYTE_BLOB", pack=False, align=None), "flags": SimTypeInt(signed=False, label="IKEEXT_CERT_FLAGS")}, name="IKEEXT_CERT_ROOT_CONFIG0", pack=False, align=None), offset=0)}, name="_Anonymous_e__Struct", pack=False, align=None), "outboundEnterpriseStoreConfig": SimTypePointer(SimStruct({"certData": SimStruct({"size": SimTypeInt(signed=False, label="UInt32"), "data": SimTypePointer(SimTypeChar(label="Byte"), offset=0)}, name="FWP_BYTE_BLOB", pack=False, align=None), "flags": SimTypeInt(signed=False, label="IKEEXT_CERT_FLAGS")}, name="IKEEXT_CERT_ROOT_CONFIG0", pack=False, align=None), offset=0), "outboundTrustedRootStoreConfig": SimTypePointer(SimStruct({"certData": SimStruct({"size": SimTypeInt(signed=False, label="UInt32"), "data": SimTypePointer(SimTypeChar(label="Byte"), offset=0)}, name="FWP_BYTE_BLOB", pack=False, align=None), "flags": SimTypeInt(signed=False, label="IKEEXT_CERT_FLAGS")}, name="IKEEXT_CERT_ROOT_CONFIG0", pack=False, align=None), offset=0)}, name="<anon>", label="None"), "flags": SimTypeInt(signed=False, label="IKEEXT_CERT_AUTH")}, name="IKEEXT_CERTIFICATE_AUTHENTICATION0", pack=False, align=None), "kerberosAuthentication": SimStruct({"flags": SimTypeInt(signed=False, label="IKEEXT_KERBEROS_AUTHENTICATION_FLAGS")}, name="IKEEXT_KERBEROS_AUTHENTICATION0", pack=False, align=None), "ntlmV2Authentication": SimStruct({"flags": SimTypeInt(signed=False, label="UInt32")}, name="IKEEXT_NTLM_V2_AUTHENTICATION0", pack=False, align=None), "sslAuthentication": SimStruct({"inboundConfigType": SimTypeInt(signed=False, label="IKEEXT_CERT_CONFIG_TYPE"), "Anonymous1": SimUnion({"Anonymous": SimStruct({"inboundRootArraySize": SimTypeInt(signed=False, label="UInt32"), "inboundRootArray": SimTypePointer(SimStruct({"certData": SimStruct({"size": SimTypeInt(signed=False, label="UInt32"), "data": SimTypePointer(SimTypeChar(label="Byte"), offset=0)}, name="FWP_BYTE_BLOB", pack=False, align=None), "flags": SimTypeInt(signed=False, label="IKEEXT_CERT_FLAGS")}, name="IKEEXT_CERT_ROOT_CONFIG0", pack=False, align=None), offset=0)}, name="_Anonymous_e__Struct", pack=False, align=None), "inboundEnterpriseStoreConfig": SimTypePointer(SimStruct({"certData": SimStruct({"size": SimTypeInt(signed=False, label="UInt32"), "data": SimTypePointer(SimTypeChar(label="Byte"), offset=0)}, name="FWP_BYTE_BLOB", pack=False, align=None), "flags": SimTypeInt(signed=False, label="IKEEXT_CERT_FLAGS")}, name="IKEEXT_CERT_ROOT_CONFIG0", pack=False, align=None), offset=0), "inboundTrustedRootStoreConfig": SimTypePointer(SimStruct({"certData": SimStruct({"size": SimTypeInt(signed=False, label="UInt32"), "data": SimTypePointer(SimTypeChar(label="Byte"), offset=0)}, name="FWP_BYTE_BLOB", pack=False, align=None), "flags": SimTypeInt(signed=False, label="IKEEXT_CERT_FLAGS")}, name="IKEEXT_CERT_ROOT_CONFIG0", pack=False, align=None), offset=0)}, name="<anon>", label="None"), "outboundConfigType": SimTypeInt(signed=False, label="IKEEXT_CERT_CONFIG_TYPE"), "Anonymous2": SimUnion({"Anonymous": SimStruct({"outboundRootArraySize": SimTypeInt(signed=False, label="UInt32"), "outboundRootArray": SimTypePointer(SimStruct({"certData": SimStruct({"size": SimTypeInt(signed=False, label="UInt32"), "data": SimTypePointer(SimTypeChar(label="Byte"), offset=0)}, name="FWP_BYTE_BLOB", pack=False, align=None), "flags": SimTypeInt(signed=False, label="IKEEXT_CERT_FLAGS")}, name="IKEEXT_CERT_ROOT_CONFIG0", pack=False, align=None), offset=0)}, name="_Anonymous_e__Struct", pack=False, align=None), "outboundEnterpriseStoreConfig": SimTypePointer(SimStruct({"certData": SimStruct({"size": SimTypeInt(signed=False, label="UInt32"), "data": SimTypePointer(SimTypeChar(label="Byte"), offset=0)}, name="FWP_BYTE_BLOB", pack=False, align=None), "flags": SimTypeInt(signed=False, label="IKEEXT_CERT_FLAGS")}, name="IKEEXT_CERT_ROOT_CONFIG0", pack=False, align=None), offset=0), "outboundTrustedRootStoreConfig": SimTypePointer(SimStruct({"certData": SimStruct({"size": SimTypeInt(signed=False, label="UInt32"), "data": SimTypePointer(SimTypeChar(label="Byte"), offset=0)}, name="FWP_BYTE_BLOB", pack=False, align=None), "flags": SimTypeInt(signed=False, label="IKEEXT_CERT_FLAGS")}, name="IKEEXT_CERT_ROOT_CONFIG0", pack=False, align=None), offset=0)}, name="<anon>", label="None"), "flags": SimTypeInt(signed=False, label="IKEEXT_CERT_AUTH")}, name="IKEEXT_CERTIFICATE_AUTHENTICATION0", pack=False, align=None), "cgaAuthentication": SimStruct({"keyContainerName": SimTypePointer(SimTypeChar(label="Char"), offset=0), "cspName": SimTypePointer(SimTypeChar(label="Char"), offset=0), "cspType": SimTypeInt(signed=False, label="UInt32"), "cgaModifier": SimStruct({"byteArray16": SimTypeFixedSizeArray(SimTypeChar(label="Byte"), 16)}, name="FWP_BYTE_ARRAY16", pack=False, align=None), "cgaCollisionCount": SimTypeChar(label="Byte")}, name="IKEEXT_IPV6_CGA_AUTHENTICATION0", pack=False, align=None)}, name="<anon>", label="None")}, name="IKEEXT_AUTHENTICATION_METHOD0", pack=False, align=None), offset=0), "initiatorImpersonationType": SimTypeInt(signed=False, label="IKEEXT_AUTHENTICATION_IMPERSONATION_TYPE"), "numIkeProposals": SimTypeInt(signed=False, label="UInt32"), "ikeProposals": SimTypePointer(SimStruct({"cipherAlgorithm": SimStruct({"algoIdentifier": SimTypeInt(signed=False, label="IKEEXT_CIPHER_TYPE"), "keyLen": SimTypeInt(signed=False, label="UInt32"), "rounds": SimTypeInt(signed=False, label="UInt32")}, name="IKEEXT_CIPHER_ALGORITHM0", pack=False, align=None), "integrityAlgorithm": SimStruct({"algoIdentifier": SimTypeInt(signed=False, label="IKEEXT_INTEGRITY_TYPE")}, name="IKEEXT_INTEGRITY_ALGORITHM0", pack=False, align=None), "maxLifetimeSeconds": SimTypeInt(signed=False, label="UInt32"), "dhGroup": SimTypeInt(signed=False, label="IKEEXT_DH_GROUP"), "quickModeLimit": SimTypeInt(signed=False, label="UInt32")}, name="IKEEXT_PROPOSAL0", pack=False, align=None), offset=0), "flags": SimTypeInt(signed=False, label="IKEEXT_POLICY_FLAG"), "maxDynamicFilters": SimTypeInt(signed=False, label="UInt32")}, name="IKEEXT_POLICY0", pack=False, align=None), offset=0), "dataBuffer": SimTypePointer(SimStruct({"size": SimTypeInt(signed=False, label="UInt32"), "data": SimTypePointer(SimTypeChar(label="Byte"), offset=0)}, name="FWP_BYTE_BLOB", pack=False, align=None), offset=0), "classifyOptions": SimTypePointer(SimStruct({"numOptions": SimTypeInt(signed=False, label="UInt32"), "options": SimTypePointer(SimStruct({"type": SimTypeInt(signed=False, label="FWP_CLASSIFY_OPTION_TYPE"), "value": SimStruct({"type": SimTypeInt(signed=False, label="FWP_DATA_TYPE"), "Anonymous": SimUnion({"uint8": SimTypeChar(label="Byte"), "uint16": SimTypeShort(signed=False, label="UInt16"), "uint32": SimTypeInt(signed=False, label="UInt32"), "uint64": SimTypePointer(SimTypeLongLong(signed=False, label="UInt64"), offset=0), "int8": SimTypeChar(label="SByte"), "int16": SimTypeShort(signed=True, label="Int16"), "int32": SimTypeInt(signed=True, label="Int32"), "int64": SimTypePointer(SimTypeLongLong(signed=True, label="Int64"), offset=0), "float32": SimTypeFloat(size=32), "double64": SimTypePointer(SimTypeFloat(size=64), offset=0), "byteArray16": SimTypePointer(SimStruct({"byteArray16": SimTypeFixedSizeArray(SimTypeChar(label="Byte"), 16)}, name="FWP_BYTE_ARRAY16", pack=False, align=None), offset=0), "byteBlob": SimTypePointer(SimStruct({"size": SimTypeInt(signed=False, label="UInt32"), "data": SimTypePointer(SimTypeChar(label="Byte"), offset=0)}, name="FWP_BYTE_BLOB", pack=False, align=None), offset=0), "sid": SimTypePointer(SimTypeBottom(label="SID"), offset=0), "sd": SimTypePointer(SimStruct({"size": SimTypeInt(signed=False, label="UInt32"), "data": SimTypePointer(SimTypeChar(label="Byte"), offset=0)}, name="FWP_BYTE_BLOB", pack=False, align=None), offset=0), "tokenInformation": SimTypePointer(SimStruct({"sidCount": SimTypeInt(signed=False, label="UInt32"), "sids": SimTypePointer(SimTypeBottom(label="SID_AND_ATTRIBUTES"), offset=0), "restrictedSidCount": SimTypeInt(signed=False, label="UInt32"), "restrictedSids": SimTypePointer(SimTypeBottom(label="SID_AND_ATTRIBUTES"), offset=0)}, name="FWP_TOKEN_INFORMATION", pack=False, align=None), offset=0), "tokenAccessInformation": SimTypePointer(SimStruct({"size": SimTypeInt(signed=False, label="UInt32"), "data": SimTypePointer(SimTypeChar(label="Byte"), offset=0)}, name="FWP_BYTE_BLOB", pack=False, align=None), offset=0), "unicodeString": SimTypePointer(SimTypeChar(label="Char"), offset=0), "byteArray6": SimTypePointer(SimStruct({"byteArray6": SimTypeFixedSizeArray(SimTypeChar(label="Byte"), 6)}, name="FWP_BYTE_ARRAY6", pack=False, align=None), offset=0), "bitmapArray64": SimTypePointer(SimStruct({"bitmapArray64": SimTypeFixedSizeArray(SimTypeChar(label="Byte"), 8)}, name="FWP_BITMAP_ARRAY64_", pack=False, align=None), offset=0)}, name="<anon>", label="None")}, name="FWP_VALUE0", pack=False, align=None)}, name="FWPM_CLASSIFY_OPTION0", pack=False, align=None), offset=0)}, name="FWPM_CLASSIFY_OPTIONS0", pack=False, align=None), offset=0)}, name="<anon>", label="None"), "providerContextId": SimTypeLongLong(signed=False, label="UInt64")}, name="FWPM_PROVIDER_CONTEXT0", pack=False, align=None), offset=0), offset=0)], SimTypeInt(signed=False, label="UInt32"), arg_names=["engineHandle", "key", "providerContext"]),
        # 
        'FwpmProviderContextGetByKey1': SimTypeFunction([SimTypePointer(SimTypeInt(signed=True, label="Int"), label="IntPtr", offset=0), SimTypePointer(SimTypeBottom(label="Guid"), offset=0), SimTypePointer(SimTypePointer(SimStruct({"providerContextKey": SimTypeBottom(label="Guid"), "displayData": SimStruct({"name": SimTypePointer(SimTypeChar(label="Char"), offset=0), "description": SimTypePointer(SimTypeChar(label="Char"), offset=0)}, name="FWPM_DISPLAY_DATA0", pack=False, align=None), "flags": SimTypeInt(signed=False, label="UInt32"), "providerKey": SimTypePointer(SimTypeBottom(label="Guid"), offset=0), "providerData": SimStruct({"size": SimTypeInt(signed=False, label="UInt32"), "data": SimTypePointer(SimTypeChar(label="Byte"), offset=0)}, name="FWP_BYTE_BLOB", pack=False, align=None), "type": SimTypeInt(signed=False, label="FWPM_PROVIDER_CONTEXT_TYPE"), "Anonymous": SimUnion({"keyingPolicy": SimTypePointer(SimStruct({"numKeyMods": SimTypeInt(signed=False, label="UInt32"), "keyModKeys": SimTypePointer(SimTypeBottom(label="Guid"), offset=0)}, name="IPSEC_KEYING_POLICY0", pack=False, align=None), offset=0), "ikeQmTransportPolicy": SimTypePointer(SimStruct({"numIpsecProposals": SimTypeInt(signed=False, label="UInt32"), "ipsecProposals": SimTypePointer(SimStruct({"lifetime": SimStruct({"lifetimeSeconds": SimTypeInt(signed=False, label="UInt32"), "lifetimeKilobytes": SimTypeInt(signed=False, label="UInt32"), "lifetimePackets": SimTypeInt(signed=False, label="UInt32")}, name="IPSEC_SA_LIFETIME0", pack=False, align=None), "numSaTransforms": SimTypeInt(signed=False, label="UInt32"), "saTransforms": SimTypePointer(SimStruct({"ipsecTransformType": SimTypeInt(signed=False, label="IPSEC_TRANSFORM_TYPE"), "Anonymous": SimUnion({"ahTransform": SimTypePointer(SimStruct({"authTransformId": SimStruct({"authType": SimTypeInt(signed=False, label="IPSEC_AUTH_TYPE"), "authConfig": SimTypeChar(label="Byte")}, name="IPSEC_AUTH_TRANSFORM_ID0", pack=False, align=None), "cryptoModuleId": SimTypePointer(SimTypeBottom(label="Guid"), offset=0)}, name="IPSEC_AUTH_TRANSFORM0", pack=False, align=None), offset=0), "espAuthTransform": SimTypePointer(SimStruct({"authTransformId": SimStruct({"authType": SimTypeInt(signed=False, label="IPSEC_AUTH_TYPE"), "authConfig": SimTypeChar(label="Byte")}, name="IPSEC_AUTH_TRANSFORM_ID0", pack=False, align=None), "cryptoModuleId": SimTypePointer(SimTypeBottom(label="Guid"), offset=0)}, name="IPSEC_AUTH_TRANSFORM0", pack=False, align=None), offset=0), "espCipherTransform": SimTypePointer(SimStruct({"cipherTransformId": SimStruct({"cipherType": SimTypeInt(signed=False, label="IPSEC_CIPHER_TYPE"), "cipherConfig": SimTypeChar(label="Byte")}, name="IPSEC_CIPHER_TRANSFORM_ID0", pack=False, align=None), "cryptoModuleId": SimTypePointer(SimTypeBottom(label="Guid"), offset=0)}, name="IPSEC_CIPHER_TRANSFORM0", pack=False, align=None), offset=0), "espAuthAndCipherTransform": SimTypePointer(SimStruct({"authTransform": SimStruct({"authTransformId": SimStruct({"authType": SimTypeInt(signed=False, label="IPSEC_AUTH_TYPE"), "authConfig": SimTypeChar(label="Byte")}, name="IPSEC_AUTH_TRANSFORM_ID0", pack=False, align=None), "cryptoModuleId": SimTypePointer(SimTypeBottom(label="Guid"), offset=0)}, name="IPSEC_AUTH_TRANSFORM0", pack=False, align=None), "cipherTransform": SimStruct({"cipherTransformId": SimStruct({"cipherType": SimTypeInt(signed=False, label="IPSEC_CIPHER_TYPE"), "cipherConfig": SimTypeChar(label="Byte")}, name="IPSEC_CIPHER_TRANSFORM_ID0", pack=False, align=None), "cryptoModuleId": SimTypePointer(SimTypeBottom(label="Guid"), offset=0)}, name="IPSEC_CIPHER_TRANSFORM0", pack=False, align=None)}, name="IPSEC_AUTH_AND_CIPHER_TRANSFORM0", pack=False, align=None), offset=0), "espAuthFwTransform": SimTypePointer(SimStruct({"authTransformId": SimStruct({"authType": SimTypeInt(signed=False, label="IPSEC_AUTH_TYPE"), "authConfig": SimTypeChar(label="Byte")}, name="IPSEC_AUTH_TRANSFORM_ID0", pack=False, align=None), "cryptoModuleId": SimTypePointer(SimTypeBottom(label="Guid"), offset=0)}, name="IPSEC_AUTH_TRANSFORM0", pack=False, align=None), offset=0)}, name="<anon>", label="None")}, name="IPSEC_SA_TRANSFORM0", pack=False, align=None), offset=0), "pfsGroup": SimTypeInt(signed=False, label="IPSEC_PFS_GROUP")}, name="IPSEC_PROPOSAL0", pack=False, align=None), offset=0), "flags": SimTypeInt(signed=False, label="IPSEC_POLICY_FLAG"), "ndAllowClearTimeoutSeconds": SimTypeInt(signed=False, label="UInt32"), "saIdleTimeout": SimStruct({"idleTimeoutSeconds": SimTypeInt(signed=False, label="UInt32"), "idleTimeoutSecondsFailOver": SimTypeInt(signed=False, label="UInt32")}, name="IPSEC_SA_IDLE_TIMEOUT0", pack=False, align=None), "emPolicy": SimTypePointer(SimStruct({"numAuthenticationMethods": SimTypeInt(signed=False, label="UInt32"), "authenticationMethods": SimTypePointer(SimStruct({"authenticationMethodType": SimTypeInt(signed=False, label="IKEEXT_AUTHENTICATION_METHOD_TYPE"), "Anonymous": SimUnion({"presharedKeyAuthentication": SimStruct({"presharedKey": SimStruct({"size": SimTypeInt(signed=False, label="UInt32"), "data": SimTypePointer(SimTypeChar(label="Byte"), offset=0)}, name="FWP_BYTE_BLOB", pack=False, align=None), "flags": SimTypeInt(signed=False, label="IKEEXT_PRESHARED_KEY_AUTHENTICATION_FLAGS")}, name="IKEEXT_PRESHARED_KEY_AUTHENTICATION1", pack=False, align=None), "certificateAuthentication": SimStruct({"inboundConfigType": SimTypeInt(signed=False, label="IKEEXT_CERT_CONFIG_TYPE"), "Anonymous1": SimUnion({"Anonymous": SimStruct({"inboundRootArraySize": SimTypeInt(signed=False, label="UInt32"), "inboundRootArray": SimTypePointer(SimStruct({"certData": SimStruct({"size": SimTypeInt(signed=False, label="UInt32"), "data": SimTypePointer(SimTypeChar(label="Byte"), offset=0)}, name="FWP_BYTE_BLOB", pack=False, align=None), "flags": SimTypeInt(signed=False, label="IKEEXT_CERT_FLAGS")}, name="IKEEXT_CERT_ROOT_CONFIG0", pack=False, align=None), offset=0)}, name="_Anonymous_e__Struct", pack=False, align=None), "inboundEnterpriseStoreConfig": SimTypePointer(SimStruct({"certData": SimStruct({"size": SimTypeInt(signed=False, label="UInt32"), "data": SimTypePointer(SimTypeChar(label="Byte"), offset=0)}, name="FWP_BYTE_BLOB", pack=False, align=None), "flags": SimTypeInt(signed=False, label="IKEEXT_CERT_FLAGS")}, name="IKEEXT_CERT_ROOT_CONFIG0", pack=False, align=None), offset=0), "inboundTrustedRootStoreConfig": SimTypePointer(SimStruct({"certData": SimStruct({"size": SimTypeInt(signed=False, label="UInt32"), "data": SimTypePointer(SimTypeChar(label="Byte"), offset=0)}, name="FWP_BYTE_BLOB", pack=False, align=None), "flags": SimTypeInt(signed=False, label="IKEEXT_CERT_FLAGS")}, name="IKEEXT_CERT_ROOT_CONFIG0", pack=False, align=None), offset=0)}, name="<anon>", label="None"), "outboundConfigType": SimTypeInt(signed=False, label="IKEEXT_CERT_CONFIG_TYPE"), "Anonymous2": SimUnion({"Anonymous": SimStruct({"outboundRootArraySize": SimTypeInt(signed=False, label="UInt32"), "outboundRootArray": SimTypePointer(SimStruct({"certData": SimStruct({"size": SimTypeInt(signed=False, label="UInt32"), "data": SimTypePointer(SimTypeChar(label="Byte"), offset=0)}, name="FWP_BYTE_BLOB", pack=False, align=None), "flags": SimTypeInt(signed=False, label="IKEEXT_CERT_FLAGS")}, name="IKEEXT_CERT_ROOT_CONFIG0", pack=False, align=None), offset=0)}, name="_Anonymous_e__Struct", pack=False, align=None), "outboundEnterpriseStoreConfig": SimTypePointer(SimStruct({"certData": SimStruct({"size": SimTypeInt(signed=False, label="UInt32"), "data": SimTypePointer(SimTypeChar(label="Byte"), offset=0)}, name="FWP_BYTE_BLOB", pack=False, align=None), "flags": SimTypeInt(signed=False, label="IKEEXT_CERT_FLAGS")}, name="IKEEXT_CERT_ROOT_CONFIG0", pack=False, align=None), offset=0), "outboundTrustedRootStoreConfig": SimTypePointer(SimStruct({"certData": SimStruct({"size": SimTypeInt(signed=False, label="UInt32"), "data": SimTypePointer(SimTypeChar(label="Byte"), offset=0)}, name="FWP_BYTE_BLOB", pack=False, align=None), "flags": SimTypeInt(signed=False, label="IKEEXT_CERT_FLAGS")}, name="IKEEXT_CERT_ROOT_CONFIG0", pack=False, align=None), offset=0)}, name="<anon>", label="None"), "flags": SimTypeInt(signed=False, label="IKEEXT_CERT_AUTH"), "localCertLocationUrl": SimStruct({"size": SimTypeInt(signed=False, label="UInt32"), "data": SimTypePointer(SimTypeChar(label="Byte"), offset=0)}, name="FWP_BYTE_BLOB", pack=False, align=None)}, name="IKEEXT_CERTIFICATE_AUTHENTICATION1", pack=False, align=None), "kerberosAuthentication": SimStruct({"flags": SimTypeInt(signed=False, label="IKEEXT_KERBEROS_AUTHENTICATION_FLAGS")}, name="IKEEXT_KERBEROS_AUTHENTICATION0", pack=False, align=None), "ntlmV2Authentication": SimStruct({"flags": SimTypeInt(signed=False, label="UInt32")}, name="IKEEXT_NTLM_V2_AUTHENTICATION0", pack=False, align=None), "sslAuthentication": SimStruct({"inboundConfigType": SimTypeInt(signed=False, label="IKEEXT_CERT_CONFIG_TYPE"), "Anonymous1": SimUnion({"Anonymous": SimStruct({"inboundRootArraySize": SimTypeInt(signed=False, label="UInt32"), "inboundRootArray": SimTypePointer(SimStruct({"certData": SimStruct({"size": SimTypeInt(signed=False, label="UInt32"), "data": SimTypePointer(SimTypeChar(label="Byte"), offset=0)}, name="FWP_BYTE_BLOB", pack=False, align=None), "flags": SimTypeInt(signed=False, label="IKEEXT_CERT_FLAGS")}, name="IKEEXT_CERT_ROOT_CONFIG0", pack=False, align=None), offset=0)}, name="_Anonymous_e__Struct", pack=False, align=None), "inboundEnterpriseStoreConfig": SimTypePointer(SimStruct({"certData": SimStruct({"size": SimTypeInt(signed=False, label="UInt32"), "data": SimTypePointer(SimTypeChar(label="Byte"), offset=0)}, name="FWP_BYTE_BLOB", pack=False, align=None), "flags": SimTypeInt(signed=False, label="IKEEXT_CERT_FLAGS")}, name="IKEEXT_CERT_ROOT_CONFIG0", pack=False, align=None), offset=0), "inboundTrustedRootStoreConfig": SimTypePointer(SimStruct({"certData": SimStruct({"size": SimTypeInt(signed=False, label="UInt32"), "data": SimTypePointer(SimTypeChar(label="Byte"), offset=0)}, name="FWP_BYTE_BLOB", pack=False, align=None), "flags": SimTypeInt(signed=False, label="IKEEXT_CERT_FLAGS")}, name="IKEEXT_CERT_ROOT_CONFIG0", pack=False, align=None), offset=0)}, name="<anon>", label="None"), "outboundConfigType": SimTypeInt(signed=False, label="IKEEXT_CERT_CONFIG_TYPE"), "Anonymous2": SimUnion({"Anonymous": SimStruct({"outboundRootArraySize": SimTypeInt(signed=False, label="UInt32"), "outboundRootArray": SimTypePointer(SimStruct({"certData": SimStruct({"size": SimTypeInt(signed=False, label="UInt32"), "data": SimTypePointer(SimTypeChar(label="Byte"), offset=0)}, name="FWP_BYTE_BLOB", pack=False, align=None), "flags": SimTypeInt(signed=False, label="IKEEXT_CERT_FLAGS")}, name="IKEEXT_CERT_ROOT_CONFIG0", pack=False, align=None), offset=0)}, name="_Anonymous_e__Struct", pack=False, align=None), "outboundEnterpriseStoreConfig": SimTypePointer(SimStruct({"certData": SimStruct({"size": SimTypeInt(signed=False, label="UInt32"), "data": SimTypePointer(SimTypeChar(label="Byte"), offset=0)}, name="FWP_BYTE_BLOB", pack=False, align=None), "flags": SimTypeInt(signed=False, label="IKEEXT_CERT_FLAGS")}, name="IKEEXT_CERT_ROOT_CONFIG0", pack=False, align=None), offset=0), "outboundTrustedRootStoreConfig": SimTypePointer(SimStruct({"certData": SimStruct({"size": SimTypeInt(signed=False, label="UInt32"), "data": SimTypePointer(SimTypeChar(label="Byte"), offset=0)}, name="FWP_BYTE_BLOB", pack=False, align=None), "flags": SimTypeInt(signed=False, label="IKEEXT_CERT_FLAGS")}, name="IKEEXT_CERT_ROOT_CONFIG0", pack=False, align=None), offset=0)}, name="<anon>", label="None"), "flags": SimTypeInt(signed=False, label="IKEEXT_CERT_AUTH"), "localCertLocationUrl": SimStruct({"size": SimTypeInt(signed=False, label="UInt32"), "data": SimTypePointer(SimTypeChar(label="Byte"), offset=0)}, name="FWP_BYTE_BLOB", pack=False, align=None)}, name="IKEEXT_CERTIFICATE_AUTHENTICATION1", pack=False, align=None), "cgaAuthentication": SimStruct({"keyContainerName": SimTypePointer(SimTypeChar(label="Char"), offset=0), "cspName": SimTypePointer(SimTypeChar(label="Char"), offset=0), "cspType": SimTypeInt(signed=False, label="UInt32"), "cgaModifier": SimStruct({"byteArray16": SimTypeFixedSizeArray(SimTypeChar(label="Byte"), 16)}, name="FWP_BYTE_ARRAY16", pack=False, align=None), "cgaCollisionCount": SimTypeChar(label="Byte")}, name="IKEEXT_IPV6_CGA_AUTHENTICATION0", pack=False, align=None), "eapAuthentication": SimStruct({"flags": SimTypeInt(signed=False, label="IKEEXT_EAP_AUTHENTICATION_FLAGS")}, name="IKEEXT_EAP_AUTHENTICATION0", pack=False, align=None)}, name="<anon>", label="None")}, name="IKEEXT_AUTHENTICATION_METHOD1", pack=False, align=None), offset=0), "initiatorImpersonationType": SimTypeInt(signed=False, label="IKEEXT_AUTHENTICATION_IMPERSONATION_TYPE")}, name="IKEEXT_EM_POLICY1", pack=False, align=None), offset=0)}, name="IPSEC_TRANSPORT_POLICY1", pack=False, align=None), offset=0), "ikeQmTunnelPolicy": SimTypePointer(SimStruct({"flags": SimTypeInt(signed=False, label="IPSEC_POLICY_FLAG"), "numIpsecProposals": SimTypeInt(signed=False, label="UInt32"), "ipsecProposals": SimTypePointer(SimStruct({"lifetime": SimStruct({"lifetimeSeconds": SimTypeInt(signed=False, label="UInt32"), "lifetimeKilobytes": SimTypeInt(signed=False, label="UInt32"), "lifetimePackets": SimTypeInt(signed=False, label="UInt32")}, name="IPSEC_SA_LIFETIME0", pack=False, align=None), "numSaTransforms": SimTypeInt(signed=False, label="UInt32"), "saTransforms": SimTypePointer(SimStruct({"ipsecTransformType": SimTypeInt(signed=False, label="IPSEC_TRANSFORM_TYPE"), "Anonymous": SimUnion({"ahTransform": SimTypePointer(SimStruct({"authTransformId": SimStruct({"authType": SimTypeInt(signed=False, label="IPSEC_AUTH_TYPE"), "authConfig": SimTypeChar(label="Byte")}, name="IPSEC_AUTH_TRANSFORM_ID0", pack=False, align=None), "cryptoModuleId": SimTypePointer(SimTypeBottom(label="Guid"), offset=0)}, name="IPSEC_AUTH_TRANSFORM0", pack=False, align=None), offset=0), "espAuthTransform": SimTypePointer(SimStruct({"authTransformId": SimStruct({"authType": SimTypeInt(signed=False, label="IPSEC_AUTH_TYPE"), "authConfig": SimTypeChar(label="Byte")}, name="IPSEC_AUTH_TRANSFORM_ID0", pack=False, align=None), "cryptoModuleId": SimTypePointer(SimTypeBottom(label="Guid"), offset=0)}, name="IPSEC_AUTH_TRANSFORM0", pack=False, align=None), offset=0), "espCipherTransform": SimTypePointer(SimStruct({"cipherTransformId": SimStruct({"cipherType": SimTypeInt(signed=False, label="IPSEC_CIPHER_TYPE"), "cipherConfig": SimTypeChar(label="Byte")}, name="IPSEC_CIPHER_TRANSFORM_ID0", pack=False, align=None), "cryptoModuleId": SimTypePointer(SimTypeBottom(label="Guid"), offset=0)}, name="IPSEC_CIPHER_TRANSFORM0", pack=False, align=None), offset=0), "espAuthAndCipherTransform": SimTypePointer(SimStruct({"authTransform": SimStruct({"authTransformId": SimStruct({"authType": SimTypeInt(signed=False, label="IPSEC_AUTH_TYPE"), "authConfig": SimTypeChar(label="Byte")}, name="IPSEC_AUTH_TRANSFORM_ID0", pack=False, align=None), "cryptoModuleId": SimTypePointer(SimTypeBottom(label="Guid"), offset=0)}, name="IPSEC_AUTH_TRANSFORM0", pack=False, align=None), "cipherTransform": SimStruct({"cipherTransformId": SimStruct({"cipherType": SimTypeInt(signed=False, label="IPSEC_CIPHER_TYPE"), "cipherConfig": SimTypeChar(label="Byte")}, name="IPSEC_CIPHER_TRANSFORM_ID0", pack=False, align=None), "cryptoModuleId": SimTypePointer(SimTypeBottom(label="Guid"), offset=0)}, name="IPSEC_CIPHER_TRANSFORM0", pack=False, align=None)}, name="IPSEC_AUTH_AND_CIPHER_TRANSFORM0", pack=False, align=None), offset=0), "espAuthFwTransform": SimTypePointer(SimStruct({"authTransformId": SimStruct({"authType": SimTypeInt(signed=False, label="IPSEC_AUTH_TYPE"), "authConfig": SimTypeChar(label="Byte")}, name="IPSEC_AUTH_TRANSFORM_ID0", pack=False, align=None), "cryptoModuleId": SimTypePointer(SimTypeBottom(label="Guid"), offset=0)}, name="IPSEC_AUTH_TRANSFORM0", pack=False, align=None), offset=0)}, name="<anon>", label="None")}, name="IPSEC_SA_TRANSFORM0", pack=False, align=None), offset=0), "pfsGroup": SimTypeInt(signed=False, label="IPSEC_PFS_GROUP")}, name="IPSEC_PROPOSAL0", pack=False, align=None), offset=0), "tunnelEndpoints": SimStruct({"ipVersion": SimTypeInt(signed=False, label="FWP_IP_VERSION"), "Anonymous1": SimUnion({"localV4Address": SimTypeInt(signed=False, label="UInt32"), "localV6Address": SimTypeFixedSizeArray(SimTypeChar(label="Byte"), 16)}, name="<anon>", label="None"), "Anonymous2": SimUnion({"remoteV4Address": SimTypeInt(signed=False, label="UInt32"), "remoteV6Address": SimTypeFixedSizeArray(SimTypeChar(label="Byte"), 16)}, name="<anon>", label="None"), "localIfLuid": SimTypeLongLong(signed=False, label="UInt64")}, name="IPSEC_TUNNEL_ENDPOINTS1", pack=False, align=None), "saIdleTimeout": SimStruct({"idleTimeoutSeconds": SimTypeInt(signed=False, label="UInt32"), "idleTimeoutSecondsFailOver": SimTypeInt(signed=False, label="UInt32")}, name="IPSEC_SA_IDLE_TIMEOUT0", pack=False, align=None), "emPolicy": SimTypePointer(SimStruct({"numAuthenticationMethods": SimTypeInt(signed=False, label="UInt32"), "authenticationMethods": SimTypePointer(SimStruct({"authenticationMethodType": SimTypeInt(signed=False, label="IKEEXT_AUTHENTICATION_METHOD_TYPE"), "Anonymous": SimUnion({"presharedKeyAuthentication": SimStruct({"presharedKey": SimStruct({"size": SimTypeInt(signed=False, label="UInt32"), "data": SimTypePointer(SimTypeChar(label="Byte"), offset=0)}, name="FWP_BYTE_BLOB", pack=False, align=None), "flags": SimTypeInt(signed=False, label="IKEEXT_PRESHARED_KEY_AUTHENTICATION_FLAGS")}, name="IKEEXT_PRESHARED_KEY_AUTHENTICATION1", pack=False, align=None), "certificateAuthentication": SimStruct({"inboundConfigType": SimTypeInt(signed=False, label="IKEEXT_CERT_CONFIG_TYPE"), "Anonymous1": SimUnion({"Anonymous": SimStruct({"inboundRootArraySize": SimTypeInt(signed=False, label="UInt32"), "inboundRootArray": SimTypePointer(SimStruct({"certData": SimStruct({"size": SimTypeInt(signed=False, label="UInt32"), "data": SimTypePointer(SimTypeChar(label="Byte"), offset=0)}, name="FWP_BYTE_BLOB", pack=False, align=None), "flags": SimTypeInt(signed=False, label="IKEEXT_CERT_FLAGS")}, name="IKEEXT_CERT_ROOT_CONFIG0", pack=False, align=None), offset=0)}, name="_Anonymous_e__Struct", pack=False, align=None), "inboundEnterpriseStoreConfig": SimTypePointer(SimStruct({"certData": SimStruct({"size": SimTypeInt(signed=False, label="UInt32"), "data": SimTypePointer(SimTypeChar(label="Byte"), offset=0)}, name="FWP_BYTE_BLOB", pack=False, align=None), "flags": SimTypeInt(signed=False, label="IKEEXT_CERT_FLAGS")}, name="IKEEXT_CERT_ROOT_CONFIG0", pack=False, align=None), offset=0), "inboundTrustedRootStoreConfig": SimTypePointer(SimStruct({"certData": SimStruct({"size": SimTypeInt(signed=False, label="UInt32"), "data": SimTypePointer(SimTypeChar(label="Byte"), offset=0)}, name="FWP_BYTE_BLOB", pack=False, align=None), "flags": SimTypeInt(signed=False, label="IKEEXT_CERT_FLAGS")}, name="IKEEXT_CERT_ROOT_CONFIG0", pack=False, align=None), offset=0)}, name="<anon>", label="None"), "outboundConfigType": SimTypeInt(signed=False, label="IKEEXT_CERT_CONFIG_TYPE"), "Anonymous2": SimUnion({"Anonymous": SimStruct({"outboundRootArraySize": SimTypeInt(signed=False, label="UInt32"), "outboundRootArray": SimTypePointer(SimStruct({"certData": SimStruct({"size": SimTypeInt(signed=False, label="UInt32"), "data": SimTypePointer(SimTypeChar(label="Byte"), offset=0)}, name="FWP_BYTE_BLOB", pack=False, align=None), "flags": SimTypeInt(signed=False, label="IKEEXT_CERT_FLAGS")}, name="IKEEXT_CERT_ROOT_CONFIG0", pack=False, align=None), offset=0)}, name="_Anonymous_e__Struct", pack=False, align=None), "outboundEnterpriseStoreConfig": SimTypePointer(SimStruct({"certData": SimStruct({"size": SimTypeInt(signed=False, label="UInt32"), "data": SimTypePointer(SimTypeChar(label="Byte"), offset=0)}, name="FWP_BYTE_BLOB", pack=False, align=None), "flags": SimTypeInt(signed=False, label="IKEEXT_CERT_FLAGS")}, name="IKEEXT_CERT_ROOT_CONFIG0", pack=False, align=None), offset=0), "outboundTrustedRootStoreConfig": SimTypePointer(SimStruct({"certData": SimStruct({"size": SimTypeInt(signed=False, label="UInt32"), "data": SimTypePointer(SimTypeChar(label="Byte"), offset=0)}, name="FWP_BYTE_BLOB", pack=False, align=None), "flags": SimTypeInt(signed=False, label="IKEEXT_CERT_FLAGS")}, name="IKEEXT_CERT_ROOT_CONFIG0", pack=False, align=None), offset=0)}, name="<anon>", label="None"), "flags": SimTypeInt(signed=False, label="IKEEXT_CERT_AUTH"), "localCertLocationUrl": SimStruct({"size": SimTypeInt(signed=False, label="UInt32"), "data": SimTypePointer(SimTypeChar(label="Byte"), offset=0)}, name="FWP_BYTE_BLOB", pack=False, align=None)}, name="IKEEXT_CERTIFICATE_AUTHENTICATION1", pack=False, align=None), "kerberosAuthentication": SimStruct({"flags": SimTypeInt(signed=False, label="IKEEXT_KERBEROS_AUTHENTICATION_FLAGS")}, name="IKEEXT_KERBEROS_AUTHENTICATION0", pack=False, align=None), "ntlmV2Authentication": SimStruct({"flags": SimTypeInt(signed=False, label="UInt32")}, name="IKEEXT_NTLM_V2_AUTHENTICATION0", pack=False, align=None), "sslAuthentication": SimStruct({"inboundConfigType": SimTypeInt(signed=False, label="IKEEXT_CERT_CONFIG_TYPE"), "Anonymous1": SimUnion({"Anonymous": SimStruct({"inboundRootArraySize": SimTypeInt(signed=False, label="UInt32"), "inboundRootArray": SimTypePointer(SimStruct({"certData": SimStruct({"size": SimTypeInt(signed=False, label="UInt32"), "data": SimTypePointer(SimTypeChar(label="Byte"), offset=0)}, name="FWP_BYTE_BLOB", pack=False, align=None), "flags": SimTypeInt(signed=False, label="IKEEXT_CERT_FLAGS")}, name="IKEEXT_CERT_ROOT_CONFIG0", pack=False, align=None), offset=0)}, name="_Anonymous_e__Struct", pack=False, align=None), "inboundEnterpriseStoreConfig": SimTypePointer(SimStruct({"certData": SimStruct({"size": SimTypeInt(signed=False, label="UInt32"), "data": SimTypePointer(SimTypeChar(label="Byte"), offset=0)}, name="FWP_BYTE_BLOB", pack=False, align=None), "flags": SimTypeInt(signed=False, label="IKEEXT_CERT_FLAGS")}, name="IKEEXT_CERT_ROOT_CONFIG0", pack=False, align=None), offset=0), "inboundTrustedRootStoreConfig": SimTypePointer(SimStruct({"certData": SimStruct({"size": SimTypeInt(signed=False, label="UInt32"), "data": SimTypePointer(SimTypeChar(label="Byte"), offset=0)}, name="FWP_BYTE_BLOB", pack=False, align=None), "flags": SimTypeInt(signed=False, label="IKEEXT_CERT_FLAGS")}, name="IKEEXT_CERT_ROOT_CONFIG0", pack=False, align=None), offset=0)}, name="<anon>", label="None"), "outboundConfigType": SimTypeInt(signed=False, label="IKEEXT_CERT_CONFIG_TYPE"), "Anonymous2": SimUnion({"Anonymous": SimStruct({"outboundRootArraySize": SimTypeInt(signed=False, label="UInt32"), "outboundRootArray": SimTypePointer(SimStruct({"certData": SimStruct({"size": SimTypeInt(signed=False, label="UInt32"), "data": SimTypePointer(SimTypeChar(label="Byte"), offset=0)}, name="FWP_BYTE_BLOB", pack=False, align=None), "flags": SimTypeInt(signed=False, label="IKEEXT_CERT_FLAGS")}, name="IKEEXT_CERT_ROOT_CONFIG0", pack=False, align=None), offset=0)}, name="_Anonymous_e__Struct", pack=False, align=None), "outboundEnterpriseStoreConfig": SimTypePointer(SimStruct({"certData": SimStruct({"size": SimTypeInt(signed=False, label="UInt32"), "data": SimTypePointer(SimTypeChar(label="Byte"), offset=0)}, name="FWP_BYTE_BLOB", pack=False, align=None), "flags": SimTypeInt(signed=False, label="IKEEXT_CERT_FLAGS")}, name="IKEEXT_CERT_ROOT_CONFIG0", pack=False, align=None), offset=0), "outboundTrustedRootStoreConfig": SimTypePointer(SimStruct({"certData": SimStruct({"size": SimTypeInt(signed=False, label="UInt32"), "data": SimTypePointer(SimTypeChar(label="Byte"), offset=0)}, name="FWP_BYTE_BLOB", pack=False, align=None), "flags": SimTypeInt(signed=False, label="IKEEXT_CERT_FLAGS")}, name="IKEEXT_CERT_ROOT_CONFIG0", pack=False, align=None), offset=0)}, name="<anon>", label="None"), "flags": SimTypeInt(signed=False, label="IKEEXT_CERT_AUTH"), "localCertLocationUrl": SimStruct({"size": SimTypeInt(signed=False, label="UInt32"), "data": SimTypePointer(SimTypeChar(label="Byte"), offset=0)}, name="FWP_BYTE_BLOB", pack=False, align=None)}, name="IKEEXT_CERTIFICATE_AUTHENTICATION1", pack=False, align=None), "cgaAuthentication": SimStruct({"keyContainerName": SimTypePointer(SimTypeChar(label="Char"), offset=0), "cspName": SimTypePointer(SimTypeChar(label="Char"), offset=0), "cspType": SimTypeInt(signed=False, label="UInt32"), "cgaModifier": SimStruct({"byteArray16": SimTypeFixedSizeArray(SimTypeChar(label="Byte"), 16)}, name="FWP_BYTE_ARRAY16", pack=False, align=None), "cgaCollisionCount": SimTypeChar(label="Byte")}, name="IKEEXT_IPV6_CGA_AUTHENTICATION0", pack=False, align=None), "eapAuthentication": SimStruct({"flags": SimTypeInt(signed=False, label="IKEEXT_EAP_AUTHENTICATION_FLAGS")}, name="IKEEXT_EAP_AUTHENTICATION0", pack=False, align=None)}, name="<anon>", label="None")}, name="IKEEXT_AUTHENTICATION_METHOD1", pack=False, align=None), offset=0), "initiatorImpersonationType": SimTypeInt(signed=False, label="IKEEXT_AUTHENTICATION_IMPERSONATION_TYPE")}, name="IKEEXT_EM_POLICY1", pack=False, align=None), offset=0)}, name="IPSEC_TUNNEL_POLICY1", pack=False, align=None), offset=0), "authipQmTransportPolicy": SimTypePointer(SimStruct({"numIpsecProposals": SimTypeInt(signed=False, label="UInt32"), "ipsecProposals": SimTypePointer(SimStruct({"lifetime": SimStruct({"lifetimeSeconds": SimTypeInt(signed=False, label="UInt32"), "lifetimeKilobytes": SimTypeInt(signed=False, label="UInt32"), "lifetimePackets": SimTypeInt(signed=False, label="UInt32")}, name="IPSEC_SA_LIFETIME0", pack=False, align=None), "numSaTransforms": SimTypeInt(signed=False, label="UInt32"), "saTransforms": SimTypePointer(SimStruct({"ipsecTransformType": SimTypeInt(signed=False, label="IPSEC_TRANSFORM_TYPE"), "Anonymous": SimUnion({"ahTransform": SimTypePointer(SimStruct({"authTransformId": SimStruct({"authType": SimTypeInt(signed=False, label="IPSEC_AUTH_TYPE"), "authConfig": SimTypeChar(label="Byte")}, name="IPSEC_AUTH_TRANSFORM_ID0", pack=False, align=None), "cryptoModuleId": SimTypePointer(SimTypeBottom(label="Guid"), offset=0)}, name="IPSEC_AUTH_TRANSFORM0", pack=False, align=None), offset=0), "espAuthTransform": SimTypePointer(SimStruct({"authTransformId": SimStruct({"authType": SimTypeInt(signed=False, label="IPSEC_AUTH_TYPE"), "authConfig": SimTypeChar(label="Byte")}, name="IPSEC_AUTH_TRANSFORM_ID0", pack=False, align=None), "cryptoModuleId": SimTypePointer(SimTypeBottom(label="Guid"), offset=0)}, name="IPSEC_AUTH_TRANSFORM0", pack=False, align=None), offset=0), "espCipherTransform": SimTypePointer(SimStruct({"cipherTransformId": SimStruct({"cipherType": SimTypeInt(signed=False, label="IPSEC_CIPHER_TYPE"), "cipherConfig": SimTypeChar(label="Byte")}, name="IPSEC_CIPHER_TRANSFORM_ID0", pack=False, align=None), "cryptoModuleId": SimTypePointer(SimTypeBottom(label="Guid"), offset=0)}, name="IPSEC_CIPHER_TRANSFORM0", pack=False, align=None), offset=0), "espAuthAndCipherTransform": SimTypePointer(SimStruct({"authTransform": SimStruct({"authTransformId": SimStruct({"authType": SimTypeInt(signed=False, label="IPSEC_AUTH_TYPE"), "authConfig": SimTypeChar(label="Byte")}, name="IPSEC_AUTH_TRANSFORM_ID0", pack=False, align=None), "cryptoModuleId": SimTypePointer(SimTypeBottom(label="Guid"), offset=0)}, name="IPSEC_AUTH_TRANSFORM0", pack=False, align=None), "cipherTransform": SimStruct({"cipherTransformId": SimStruct({"cipherType": SimTypeInt(signed=False, label="IPSEC_CIPHER_TYPE"), "cipherConfig": SimTypeChar(label="Byte")}, name="IPSEC_CIPHER_TRANSFORM_ID0", pack=False, align=None), "cryptoModuleId": SimTypePointer(SimTypeBottom(label="Guid"), offset=0)}, name="IPSEC_CIPHER_TRANSFORM0", pack=False, align=None)}, name="IPSEC_AUTH_AND_CIPHER_TRANSFORM0", pack=False, align=None), offset=0), "espAuthFwTransform": SimTypePointer(SimStruct({"authTransformId": SimStruct({"authType": SimTypeInt(signed=False, label="IPSEC_AUTH_TYPE"), "authConfig": SimTypeChar(label="Byte")}, name="IPSEC_AUTH_TRANSFORM_ID0", pack=False, align=None), "cryptoModuleId": SimTypePointer(SimTypeBottom(label="Guid"), offset=0)}, name="IPSEC_AUTH_TRANSFORM0", pack=False, align=None), offset=0)}, name="<anon>", label="None")}, name="IPSEC_SA_TRANSFORM0", pack=False, align=None), offset=0), "pfsGroup": SimTypeInt(signed=False, label="IPSEC_PFS_GROUP")}, name="IPSEC_PROPOSAL0", pack=False, align=None), offset=0), "flags": SimTypeInt(signed=False, label="IPSEC_POLICY_FLAG"), "ndAllowClearTimeoutSeconds": SimTypeInt(signed=False, label="UInt32"), "saIdleTimeout": SimStruct({"idleTimeoutSeconds": SimTypeInt(signed=False, label="UInt32"), "idleTimeoutSecondsFailOver": SimTypeInt(signed=False, label="UInt32")}, name="IPSEC_SA_IDLE_TIMEOUT0", pack=False, align=None), "emPolicy": SimTypePointer(SimStruct({"numAuthenticationMethods": SimTypeInt(signed=False, label="UInt32"), "authenticationMethods": SimTypePointer(SimStruct({"authenticationMethodType": SimTypeInt(signed=False, label="IKEEXT_AUTHENTICATION_METHOD_TYPE"), "Anonymous": SimUnion({"presharedKeyAuthentication": SimStruct({"presharedKey": SimStruct({"size": SimTypeInt(signed=False, label="UInt32"), "data": SimTypePointer(SimTypeChar(label="Byte"), offset=0)}, name="FWP_BYTE_BLOB", pack=False, align=None), "flags": SimTypeInt(signed=False, label="IKEEXT_PRESHARED_KEY_AUTHENTICATION_FLAGS")}, name="IKEEXT_PRESHARED_KEY_AUTHENTICATION1", pack=False, align=None), "certificateAuthentication": SimStruct({"inboundConfigType": SimTypeInt(signed=False, label="IKEEXT_CERT_CONFIG_TYPE"), "Anonymous1": SimUnion({"Anonymous": SimStruct({"inboundRootArraySize": SimTypeInt(signed=False, label="UInt32"), "inboundRootArray": SimTypePointer(SimStruct({"certData": SimStruct({"size": SimTypeInt(signed=False, label="UInt32"), "data": SimTypePointer(SimTypeChar(label="Byte"), offset=0)}, name="FWP_BYTE_BLOB", pack=False, align=None), "flags": SimTypeInt(signed=False, label="IKEEXT_CERT_FLAGS")}, name="IKEEXT_CERT_ROOT_CONFIG0", pack=False, align=None), offset=0)}, name="_Anonymous_e__Struct", pack=False, align=None), "inboundEnterpriseStoreConfig": SimTypePointer(SimStruct({"certData": SimStruct({"size": SimTypeInt(signed=False, label="UInt32"), "data": SimTypePointer(SimTypeChar(label="Byte"), offset=0)}, name="FWP_BYTE_BLOB", pack=False, align=None), "flags": SimTypeInt(signed=False, label="IKEEXT_CERT_FLAGS")}, name="IKEEXT_CERT_ROOT_CONFIG0", pack=False, align=None), offset=0), "inboundTrustedRootStoreConfig": SimTypePointer(SimStruct({"certData": SimStruct({"size": SimTypeInt(signed=False, label="UInt32"), "data": SimTypePointer(SimTypeChar(label="Byte"), offset=0)}, name="FWP_BYTE_BLOB", pack=False, align=None), "flags": SimTypeInt(signed=False, label="IKEEXT_CERT_FLAGS")}, name="IKEEXT_CERT_ROOT_CONFIG0", pack=False, align=None), offset=0)}, name="<anon>", label="None"), "outboundConfigType": SimTypeInt(signed=False, label="IKEEXT_CERT_CONFIG_TYPE"), "Anonymous2": SimUnion({"Anonymous": SimStruct({"outboundRootArraySize": SimTypeInt(signed=False, label="UInt32"), "outboundRootArray": SimTypePointer(SimStruct({"certData": SimStruct({"size": SimTypeInt(signed=False, label="UInt32"), "data": SimTypePointer(SimTypeChar(label="Byte"), offset=0)}, name="FWP_BYTE_BLOB", pack=False, align=None), "flags": SimTypeInt(signed=False, label="IKEEXT_CERT_FLAGS")}, name="IKEEXT_CERT_ROOT_CONFIG0", pack=False, align=None), offset=0)}, name="_Anonymous_e__Struct", pack=False, align=None), "outboundEnterpriseStoreConfig": SimTypePointer(SimStruct({"certData": SimStruct({"size": SimTypeInt(signed=False, label="UInt32"), "data": SimTypePointer(SimTypeChar(label="Byte"), offset=0)}, name="FWP_BYTE_BLOB", pack=False, align=None), "flags": SimTypeInt(signed=False, label="IKEEXT_CERT_FLAGS")}, name="IKEEXT_CERT_ROOT_CONFIG0", pack=False, align=None), offset=0), "outboundTrustedRootStoreConfig": SimTypePointer(SimStruct({"certData": SimStruct({"size": SimTypeInt(signed=False, label="UInt32"), "data": SimTypePointer(SimTypeChar(label="Byte"), offset=0)}, name="FWP_BYTE_BLOB", pack=False, align=None), "flags": SimTypeInt(signed=False, label="IKEEXT_CERT_FLAGS")}, name="IKEEXT_CERT_ROOT_CONFIG0", pack=False, align=None), offset=0)}, name="<anon>", label="None"), "flags": SimTypeInt(signed=False, label="IKEEXT_CERT_AUTH"), "localCertLocationUrl": SimStruct({"size": SimTypeInt(signed=False, label="UInt32"), "data": SimTypePointer(SimTypeChar(label="Byte"), offset=0)}, name="FWP_BYTE_BLOB", pack=False, align=None)}, name="IKEEXT_CERTIFICATE_AUTHENTICATION1", pack=False, align=None), "kerberosAuthentication": SimStruct({"flags": SimTypeInt(signed=False, label="IKEEXT_KERBEROS_AUTHENTICATION_FLAGS")}, name="IKEEXT_KERBEROS_AUTHENTICATION0", pack=False, align=None), "ntlmV2Authentication": SimStruct({"flags": SimTypeInt(signed=False, label="UInt32")}, name="IKEEXT_NTLM_V2_AUTHENTICATION0", pack=False, align=None), "sslAuthentication": SimStruct({"inboundConfigType": SimTypeInt(signed=False, label="IKEEXT_CERT_CONFIG_TYPE"), "Anonymous1": SimUnion({"Anonymous": SimStruct({"inboundRootArraySize": SimTypeInt(signed=False, label="UInt32"), "inboundRootArray": SimTypePointer(SimStruct({"certData": SimStruct({"size": SimTypeInt(signed=False, label="UInt32"), "data": SimTypePointer(SimTypeChar(label="Byte"), offset=0)}, name="FWP_BYTE_BLOB", pack=False, align=None), "flags": SimTypeInt(signed=False, label="IKEEXT_CERT_FLAGS")}, name="IKEEXT_CERT_ROOT_CONFIG0", pack=False, align=None), offset=0)}, name="_Anonymous_e__Struct", pack=False, align=None), "inboundEnterpriseStoreConfig": SimTypePointer(SimStruct({"certData": SimStruct({"size": SimTypeInt(signed=False, label="UInt32"), "data": SimTypePointer(SimTypeChar(label="Byte"), offset=0)}, name="FWP_BYTE_BLOB", pack=False, align=None), "flags": SimTypeInt(signed=False, label="IKEEXT_CERT_FLAGS")}, name="IKEEXT_CERT_ROOT_CONFIG0", pack=False, align=None), offset=0), "inboundTrustedRootStoreConfig": SimTypePointer(SimStruct({"certData": SimStruct({"size": SimTypeInt(signed=False, label="UInt32"), "data": SimTypePointer(SimTypeChar(label="Byte"), offset=0)}, name="FWP_BYTE_BLOB", pack=False, align=None), "flags": SimTypeInt(signed=False, label="IKEEXT_CERT_FLAGS")}, name="IKEEXT_CERT_ROOT_CONFIG0", pack=False, align=None), offset=0)}, name="<anon>", label="None"), "outboundConfigType": SimTypeInt(signed=False, label="IKEEXT_CERT_CONFIG_TYPE"), "Anonymous2": SimUnion({"Anonymous": SimStruct({"outboundRootArraySize": SimTypeInt(signed=False, label="UInt32"), "outboundRootArray": SimTypePointer(SimStruct({"certData": SimStruct({"size": SimTypeInt(signed=False, label="UInt32"), "data": SimTypePointer(SimTypeChar(label="Byte"), offset=0)}, name="FWP_BYTE_BLOB", pack=False, align=None), "flags": SimTypeInt(signed=False, label="IKEEXT_CERT_FLAGS")}, name="IKEEXT_CERT_ROOT_CONFIG0", pack=False, align=None), offset=0)}, name="_Anonymous_e__Struct", pack=False, align=None), "outboundEnterpriseStoreConfig": SimTypePointer(SimStruct({"certData": SimStruct({"size": SimTypeInt(signed=False, label="UInt32"), "data": SimTypePointer(SimTypeChar(label="Byte"), offset=0)}, name="FWP_BYTE_BLOB", pack=False, align=None), "flags": SimTypeInt(signed=False, label="IKEEXT_CERT_FLAGS")}, name="IKEEXT_CERT_ROOT_CONFIG0", pack=False, align=None), offset=0), "outboundTrustedRootStoreConfig": SimTypePointer(SimStruct({"certData": SimStruct({"size": SimTypeInt(signed=False, label="UInt32"), "data": SimTypePointer(SimTypeChar(label="Byte"), offset=0)}, name="FWP_BYTE_BLOB", pack=False, align=None), "flags": SimTypeInt(signed=False, label="IKEEXT_CERT_FLAGS")}, name="IKEEXT_CERT_ROOT_CONFIG0", pack=False, align=None), offset=0)}, name="<anon>", label="None"), "flags": SimTypeInt(signed=False, label="IKEEXT_CERT_AUTH"), "localCertLocationUrl": SimStruct({"size": SimTypeInt(signed=False, label="UInt32"), "data": SimTypePointer(SimTypeChar(label="Byte"), offset=0)}, name="FWP_BYTE_BLOB", pack=False, align=None)}, name="IKEEXT_CERTIFICATE_AUTHENTICATION1", pack=False, align=None), "cgaAuthentication": SimStruct({"keyContainerName": SimTypePointer(SimTypeChar(label="Char"), offset=0), "cspName": SimTypePointer(SimTypeChar(label="Char"), offset=0), "cspType": SimTypeInt(signed=False, label="UInt32"), "cgaModifier": SimStruct({"byteArray16": SimTypeFixedSizeArray(SimTypeChar(label="Byte"), 16)}, name="FWP_BYTE_ARRAY16", pack=False, align=None), "cgaCollisionCount": SimTypeChar(label="Byte")}, name="IKEEXT_IPV6_CGA_AUTHENTICATION0", pack=False, align=None), "eapAuthentication": SimStruct({"flags": SimTypeInt(signed=False, label="IKEEXT_EAP_AUTHENTICATION_FLAGS")}, name="IKEEXT_EAP_AUTHENTICATION0", pack=False, align=None)}, name="<anon>", label="None")}, name="IKEEXT_AUTHENTICATION_METHOD1", pack=False, align=None), offset=0), "initiatorImpersonationType": SimTypeInt(signed=False, label="IKEEXT_AUTHENTICATION_IMPERSONATION_TYPE")}, name="IKEEXT_EM_POLICY1", pack=False, align=None), offset=0)}, name="IPSEC_TRANSPORT_POLICY1", pack=False, align=None), offset=0), "authipQmTunnelPolicy": SimTypePointer(SimStruct({"flags": SimTypeInt(signed=False, label="IPSEC_POLICY_FLAG"), "numIpsecProposals": SimTypeInt(signed=False, label="UInt32"), "ipsecProposals": SimTypePointer(SimStruct({"lifetime": SimStruct({"lifetimeSeconds": SimTypeInt(signed=False, label="UInt32"), "lifetimeKilobytes": SimTypeInt(signed=False, label="UInt32"), "lifetimePackets": SimTypeInt(signed=False, label="UInt32")}, name="IPSEC_SA_LIFETIME0", pack=False, align=None), "numSaTransforms": SimTypeInt(signed=False, label="UInt32"), "saTransforms": SimTypePointer(SimStruct({"ipsecTransformType": SimTypeInt(signed=False, label="IPSEC_TRANSFORM_TYPE"), "Anonymous": SimUnion({"ahTransform": SimTypePointer(SimStruct({"authTransformId": SimStruct({"authType": SimTypeInt(signed=False, label="IPSEC_AUTH_TYPE"), "authConfig": SimTypeChar(label="Byte")}, name="IPSEC_AUTH_TRANSFORM_ID0", pack=False, align=None), "cryptoModuleId": SimTypePointer(SimTypeBottom(label="Guid"), offset=0)}, name="IPSEC_AUTH_TRANSFORM0", pack=False, align=None), offset=0), "espAuthTransform": SimTypePointer(SimStruct({"authTransformId": SimStruct({"authType": SimTypeInt(signed=False, label="IPSEC_AUTH_TYPE"), "authConfig": SimTypeChar(label="Byte")}, name="IPSEC_AUTH_TRANSFORM_ID0", pack=False, align=None), "cryptoModuleId": SimTypePointer(SimTypeBottom(label="Guid"), offset=0)}, name="IPSEC_AUTH_TRANSFORM0", pack=False, align=None), offset=0), "espCipherTransform": SimTypePointer(SimStruct({"cipherTransformId": SimStruct({"cipherType": SimTypeInt(signed=False, label="IPSEC_CIPHER_TYPE"), "cipherConfig": SimTypeChar(label="Byte")}, name="IPSEC_CIPHER_TRANSFORM_ID0", pack=False, align=None), "cryptoModuleId": SimTypePointer(SimTypeBottom(label="Guid"), offset=0)}, name="IPSEC_CIPHER_TRANSFORM0", pack=False, align=None), offset=0), "espAuthAndCipherTransform": SimTypePointer(SimStruct({"authTransform": SimStruct({"authTransformId": SimStruct({"authType": SimTypeInt(signed=False, label="IPSEC_AUTH_TYPE"), "authConfig": SimTypeChar(label="Byte")}, name="IPSEC_AUTH_TRANSFORM_ID0", pack=False, align=None), "cryptoModuleId": SimTypePointer(SimTypeBottom(label="Guid"), offset=0)}, name="IPSEC_AUTH_TRANSFORM0", pack=False, align=None), "cipherTransform": SimStruct({"cipherTransformId": SimStruct({"cipherType": SimTypeInt(signed=False, label="IPSEC_CIPHER_TYPE"), "cipherConfig": SimTypeChar(label="Byte")}, name="IPSEC_CIPHER_TRANSFORM_ID0", pack=False, align=None), "cryptoModuleId": SimTypePointer(SimTypeBottom(label="Guid"), offset=0)}, name="IPSEC_CIPHER_TRANSFORM0", pack=False, align=None)}, name="IPSEC_AUTH_AND_CIPHER_TRANSFORM0", pack=False, align=None), offset=0), "espAuthFwTransform": SimTypePointer(SimStruct({"authTransformId": SimStruct({"authType": SimTypeInt(signed=False, label="IPSEC_AUTH_TYPE"), "authConfig": SimTypeChar(label="Byte")}, name="IPSEC_AUTH_TRANSFORM_ID0", pack=False, align=None), "cryptoModuleId": SimTypePointer(SimTypeBottom(label="Guid"), offset=0)}, name="IPSEC_AUTH_TRANSFORM0", pack=False, align=None), offset=0)}, name="<anon>", label="None")}, name="IPSEC_SA_TRANSFORM0", pack=False, align=None), offset=0), "pfsGroup": SimTypeInt(signed=False, label="IPSEC_PFS_GROUP")}, name="IPSEC_PROPOSAL0", pack=False, align=None), offset=0), "tunnelEndpoints": SimStruct({"ipVersion": SimTypeInt(signed=False, label="FWP_IP_VERSION"), "Anonymous1": SimUnion({"localV4Address": SimTypeInt(signed=False, label="UInt32"), "localV6Address": SimTypeFixedSizeArray(SimTypeChar(label="Byte"), 16)}, name="<anon>", label="None"), "Anonymous2": SimUnion({"remoteV4Address": SimTypeInt(signed=False, label="UInt32"), "remoteV6Address": SimTypeFixedSizeArray(SimTypeChar(label="Byte"), 16)}, name="<anon>", label="None"), "localIfLuid": SimTypeLongLong(signed=False, label="UInt64")}, name="IPSEC_TUNNEL_ENDPOINTS1", pack=False, align=None), "saIdleTimeout": SimStruct({"idleTimeoutSeconds": SimTypeInt(signed=False, label="UInt32"), "idleTimeoutSecondsFailOver": SimTypeInt(signed=False, label="UInt32")}, name="IPSEC_SA_IDLE_TIMEOUT0", pack=False, align=None), "emPolicy": SimTypePointer(SimStruct({"numAuthenticationMethods": SimTypeInt(signed=False, label="UInt32"), "authenticationMethods": SimTypePointer(SimStruct({"authenticationMethodType": SimTypeInt(signed=False, label="IKEEXT_AUTHENTICATION_METHOD_TYPE"), "Anonymous": SimUnion({"presharedKeyAuthentication": SimStruct({"presharedKey": SimStruct({"size": SimTypeInt(signed=False, label="UInt32"), "data": SimTypePointer(SimTypeChar(label="Byte"), offset=0)}, name="FWP_BYTE_BLOB", pack=False, align=None), "flags": SimTypeInt(signed=False, label="IKEEXT_PRESHARED_KEY_AUTHENTICATION_FLAGS")}, name="IKEEXT_PRESHARED_KEY_AUTHENTICATION1", pack=False, align=None), "certificateAuthentication": SimStruct({"inboundConfigType": SimTypeInt(signed=False, label="IKEEXT_CERT_CONFIG_TYPE"), "Anonymous1": SimUnion({"Anonymous": SimStruct({"inboundRootArraySize": SimTypeInt(signed=False, label="UInt32"), "inboundRootArray": SimTypePointer(SimStruct({"certData": SimStruct({"size": SimTypeInt(signed=False, label="UInt32"), "data": SimTypePointer(SimTypeChar(label="Byte"), offset=0)}, name="FWP_BYTE_BLOB", pack=False, align=None), "flags": SimTypeInt(signed=False, label="IKEEXT_CERT_FLAGS")}, name="IKEEXT_CERT_ROOT_CONFIG0", pack=False, align=None), offset=0)}, name="_Anonymous_e__Struct", pack=False, align=None), "inboundEnterpriseStoreConfig": SimTypePointer(SimStruct({"certData": SimStruct({"size": SimTypeInt(signed=False, label="UInt32"), "data": SimTypePointer(SimTypeChar(label="Byte"), offset=0)}, name="FWP_BYTE_BLOB", pack=False, align=None), "flags": SimTypeInt(signed=False, label="IKEEXT_CERT_FLAGS")}, name="IKEEXT_CERT_ROOT_CONFIG0", pack=False, align=None), offset=0), "inboundTrustedRootStoreConfig": SimTypePointer(SimStruct({"certData": SimStruct({"size": SimTypeInt(signed=False, label="UInt32"), "data": SimTypePointer(SimTypeChar(label="Byte"), offset=0)}, name="FWP_BYTE_BLOB", pack=False, align=None), "flags": SimTypeInt(signed=False, label="IKEEXT_CERT_FLAGS")}, name="IKEEXT_CERT_ROOT_CONFIG0", pack=False, align=None), offset=0)}, name="<anon>", label="None"), "outboundConfigType": SimTypeInt(signed=False, label="IKEEXT_CERT_CONFIG_TYPE"), "Anonymous2": SimUnion({"Anonymous": SimStruct({"outboundRootArraySize": SimTypeInt(signed=False, label="UInt32"), "outboundRootArray": SimTypePointer(SimStruct({"certData": SimStruct({"size": SimTypeInt(signed=False, label="UInt32"), "data": SimTypePointer(SimTypeChar(label="Byte"), offset=0)}, name="FWP_BYTE_BLOB", pack=False, align=None), "flags": SimTypeInt(signed=False, label="IKEEXT_CERT_FLAGS")}, name="IKEEXT_CERT_ROOT_CONFIG0", pack=False, align=None), offset=0)}, name="_Anonymous_e__Struct", pack=False, align=None), "outboundEnterpriseStoreConfig": SimTypePointer(SimStruct({"certData": SimStruct({"size": SimTypeInt(signed=False, label="UInt32"), "data": SimTypePointer(SimTypeChar(label="Byte"), offset=0)}, name="FWP_BYTE_BLOB", pack=False, align=None), "flags": SimTypeInt(signed=False, label="IKEEXT_CERT_FLAGS")}, name="IKEEXT_CERT_ROOT_CONFIG0", pack=False, align=None), offset=0), "outboundTrustedRootStoreConfig": SimTypePointer(SimStruct({"certData": SimStruct({"size": SimTypeInt(signed=False, label="UInt32"), "data": SimTypePointer(SimTypeChar(label="Byte"), offset=0)}, name="FWP_BYTE_BLOB", pack=False, align=None), "flags": SimTypeInt(signed=False, label="IKEEXT_CERT_FLAGS")}, name="IKEEXT_CERT_ROOT_CONFIG0", pack=False, align=None), offset=0)}, name="<anon>", label="None"), "flags": SimTypeInt(signed=False, label="IKEEXT_CERT_AUTH"), "localCertLocationUrl": SimStruct({"size": SimTypeInt(signed=False, label="UInt32"), "data": SimTypePointer(SimTypeChar(label="Byte"), offset=0)}, name="FWP_BYTE_BLOB", pack=False, align=None)}, name="IKEEXT_CERTIFICATE_AUTHENTICATION1", pack=False, align=None), "kerberosAuthentication": SimStruct({"flags": SimTypeInt(signed=False, label="IKEEXT_KERBEROS_AUTHENTICATION_FLAGS")}, name="IKEEXT_KERBEROS_AUTHENTICATION0", pack=False, align=None), "ntlmV2Authentication": SimStruct({"flags": SimTypeInt(signed=False, label="UInt32")}, name="IKEEXT_NTLM_V2_AUTHENTICATION0", pack=False, align=None), "sslAuthentication": SimStruct({"inboundConfigType": SimTypeInt(signed=False, label="IKEEXT_CERT_CONFIG_TYPE"), "Anonymous1": SimUnion({"Anonymous": SimStruct({"inboundRootArraySize": SimTypeInt(signed=False, label="UInt32"), "inboundRootArray": SimTypePointer(SimStruct({"certData": SimStruct({"size": SimTypeInt(signed=False, label="UInt32"), "data": SimTypePointer(SimTypeChar(label="Byte"), offset=0)}, name="FWP_BYTE_BLOB", pack=False, align=None), "flags": SimTypeInt(signed=False, label="IKEEXT_CERT_FLAGS")}, name="IKEEXT_CERT_ROOT_CONFIG0", pack=False, align=None), offset=0)}, name="_Anonymous_e__Struct", pack=False, align=None), "inboundEnterpriseStoreConfig": SimTypePointer(SimStruct({"certData": SimStruct({"size": SimTypeInt(signed=False, label="UInt32"), "data": SimTypePointer(SimTypeChar(label="Byte"), offset=0)}, name="FWP_BYTE_BLOB", pack=False, align=None), "flags": SimTypeInt(signed=False, label="IKEEXT_CERT_FLAGS")}, name="IKEEXT_CERT_ROOT_CONFIG0", pack=False, align=None), offset=0), "inboundTrustedRootStoreConfig": SimTypePointer(SimStruct({"certData": SimStruct({"size": SimTypeInt(signed=False, label="UInt32"), "data": SimTypePointer(SimTypeChar(label="Byte"), offset=0)}, name="FWP_BYTE_BLOB", pack=False, align=None), "flags": SimTypeInt(signed=False, label="IKEEXT_CERT_FLAGS")}, name="IKEEXT_CERT_ROOT_CONFIG0", pack=False, align=None), offset=0)}, name="<anon>", label="None"), "outboundConfigType": SimTypeInt(signed=False, label="IKEEXT_CERT_CONFIG_TYPE"), "Anonymous2": SimUnion({"Anonymous": SimStruct({"outboundRootArraySize": SimTypeInt(signed=False, label="UInt32"), "outboundRootArray": SimTypePointer(SimStruct({"certData": SimStruct({"size": SimTypeInt(signed=False, label="UInt32"), "data": SimTypePointer(SimTypeChar(label="Byte"), offset=0)}, name="FWP_BYTE_BLOB", pack=False, align=None), "flags": SimTypeInt(signed=False, label="IKEEXT_CERT_FLAGS")}, name="IKEEXT_CERT_ROOT_CONFIG0", pack=False, align=None), offset=0)}, name="_Anonymous_e__Struct", pack=False, align=None), "outboundEnterpriseStoreConfig": SimTypePointer(SimStruct({"certData": SimStruct({"size": SimTypeInt(signed=False, label="UInt32"), "data": SimTypePointer(SimTypeChar(label="Byte"), offset=0)}, name="FWP_BYTE_BLOB", pack=False, align=None), "flags": SimTypeInt(signed=False, label="IKEEXT_CERT_FLAGS")}, name="IKEEXT_CERT_ROOT_CONFIG0", pack=False, align=None), offset=0), "outboundTrustedRootStoreConfig": SimTypePointer(SimStruct({"certData": SimStruct({"size": SimTypeInt(signed=False, label="UInt32"), "data": SimTypePointer(SimTypeChar(label="Byte"), offset=0)}, name="FWP_BYTE_BLOB", pack=False, align=None), "flags": SimTypeInt(signed=False, label="IKEEXT_CERT_FLAGS")}, name="IKEEXT_CERT_ROOT_CONFIG0", pack=False, align=None), offset=0)}, name="<anon>", label="None"), "flags": SimTypeInt(signed=False, label="IKEEXT_CERT_AUTH"), "localCertLocationUrl": SimStruct({"size": SimTypeInt(signed=False, label="UInt32"), "data": SimTypePointer(SimTypeChar(label="Byte"), offset=0)}, name="FWP_BYTE_BLOB", pack=False, align=None)}, name="IKEEXT_CERTIFICATE_AUTHENTICATION1", pack=False, align=None), "cgaAuthentication": SimStruct({"keyContainerName": SimTypePointer(SimTypeChar(label="Char"), offset=0), "cspName": SimTypePointer(SimTypeChar(label="Char"), offset=0), "cspType": SimTypeInt(signed=False, label="UInt32"), "cgaModifier": SimStruct({"byteArray16": SimTypeFixedSizeArray(SimTypeChar(label="Byte"), 16)}, name="FWP_BYTE_ARRAY16", pack=False, align=None), "cgaCollisionCount": SimTypeChar(label="Byte")}, name="IKEEXT_IPV6_CGA_AUTHENTICATION0", pack=False, align=None), "eapAuthentication": SimStruct({"flags": SimTypeInt(signed=False, label="IKEEXT_EAP_AUTHENTICATION_FLAGS")}, name="IKEEXT_EAP_AUTHENTICATION0", pack=False, align=None)}, name="<anon>", label="None")}, name="IKEEXT_AUTHENTICATION_METHOD1", pack=False, align=None), offset=0), "initiatorImpersonationType": SimTypeInt(signed=False, label="IKEEXT_AUTHENTICATION_IMPERSONATION_TYPE")}, name="IKEEXT_EM_POLICY1", pack=False, align=None), offset=0)}, name="IPSEC_TUNNEL_POLICY1", pack=False, align=None), offset=0), "ikeMmPolicy": SimTypePointer(SimStruct({"softExpirationTime": SimTypeInt(signed=False, label="UInt32"), "numAuthenticationMethods": SimTypeInt(signed=False, label="UInt32"), "authenticationMethods": SimTypePointer(SimStruct({"authenticationMethodType": SimTypeInt(signed=False, label="IKEEXT_AUTHENTICATION_METHOD_TYPE"), "Anonymous": SimUnion({"presharedKeyAuthentication": SimStruct({"presharedKey": SimStruct({"size": SimTypeInt(signed=False, label="UInt32"), "data": SimTypePointer(SimTypeChar(label="Byte"), offset=0)}, name="FWP_BYTE_BLOB", pack=False, align=None), "flags": SimTypeInt(signed=False, label="IKEEXT_PRESHARED_KEY_AUTHENTICATION_FLAGS")}, name="IKEEXT_PRESHARED_KEY_AUTHENTICATION1", pack=False, align=None), "certificateAuthentication": SimStruct({"inboundConfigType": SimTypeInt(signed=False, label="IKEEXT_CERT_CONFIG_TYPE"), "Anonymous1": SimUnion({"Anonymous": SimStruct({"inboundRootArraySize": SimTypeInt(signed=False, label="UInt32"), "inboundRootArray": SimTypePointer(SimStruct({"certData": SimStruct({"size": SimTypeInt(signed=False, label="UInt32"), "data": SimTypePointer(SimTypeChar(label="Byte"), offset=0)}, name="FWP_BYTE_BLOB", pack=False, align=None), "flags": SimTypeInt(signed=False, label="IKEEXT_CERT_FLAGS")}, name="IKEEXT_CERT_ROOT_CONFIG0", pack=False, align=None), offset=0)}, name="_Anonymous_e__Struct", pack=False, align=None), "inboundEnterpriseStoreConfig": SimTypePointer(SimStruct({"certData": SimStruct({"size": SimTypeInt(signed=False, label="UInt32"), "data": SimTypePointer(SimTypeChar(label="Byte"), offset=0)}, name="FWP_BYTE_BLOB", pack=False, align=None), "flags": SimTypeInt(signed=False, label="IKEEXT_CERT_FLAGS")}, name="IKEEXT_CERT_ROOT_CONFIG0", pack=False, align=None), offset=0), "inboundTrustedRootStoreConfig": SimTypePointer(SimStruct({"certData": SimStruct({"size": SimTypeInt(signed=False, label="UInt32"), "data": SimTypePointer(SimTypeChar(label="Byte"), offset=0)}, name="FWP_BYTE_BLOB", pack=False, align=None), "flags": SimTypeInt(signed=False, label="IKEEXT_CERT_FLAGS")}, name="IKEEXT_CERT_ROOT_CONFIG0", pack=False, align=None), offset=0)}, name="<anon>", label="None"), "outboundConfigType": SimTypeInt(signed=False, label="IKEEXT_CERT_CONFIG_TYPE"), "Anonymous2": SimUnion({"Anonymous": SimStruct({"outboundRootArraySize": SimTypeInt(signed=False, label="UInt32"), "outboundRootArray": SimTypePointer(SimStruct({"certData": SimStruct({"size": SimTypeInt(signed=False, label="UInt32"), "data": SimTypePointer(SimTypeChar(label="Byte"), offset=0)}, name="FWP_BYTE_BLOB", pack=False, align=None), "flags": SimTypeInt(signed=False, label="IKEEXT_CERT_FLAGS")}, name="IKEEXT_CERT_ROOT_CONFIG0", pack=False, align=None), offset=0)}, name="_Anonymous_e__Struct", pack=False, align=None), "outboundEnterpriseStoreConfig": SimTypePointer(SimStruct({"certData": SimStruct({"size": SimTypeInt(signed=False, label="UInt32"), "data": SimTypePointer(SimTypeChar(label="Byte"), offset=0)}, name="FWP_BYTE_BLOB", pack=False, align=None), "flags": SimTypeInt(signed=False, label="IKEEXT_CERT_FLAGS")}, name="IKEEXT_CERT_ROOT_CONFIG0", pack=False, align=None), offset=0), "outboundTrustedRootStoreConfig": SimTypePointer(SimStruct({"certData": SimStruct({"size": SimTypeInt(signed=False, label="UInt32"), "data": SimTypePointer(SimTypeChar(label="Byte"), offset=0)}, name="FWP_BYTE_BLOB", pack=False, align=None), "flags": SimTypeInt(signed=False, label="IKEEXT_CERT_FLAGS")}, name="IKEEXT_CERT_ROOT_CONFIG0", pack=False, align=None), offset=0)}, name="<anon>", label="None"), "flags": SimTypeInt(signed=False, label="IKEEXT_CERT_AUTH"), "localCertLocationUrl": SimStruct({"size": SimTypeInt(signed=False, label="UInt32"), "data": SimTypePointer(SimTypeChar(label="Byte"), offset=0)}, name="FWP_BYTE_BLOB", pack=False, align=None)}, name="IKEEXT_CERTIFICATE_AUTHENTICATION1", pack=False, align=None), "kerberosAuthentication": SimStruct({"flags": SimTypeInt(signed=False, label="IKEEXT_KERBEROS_AUTHENTICATION_FLAGS")}, name="IKEEXT_KERBEROS_AUTHENTICATION0", pack=False, align=None), "ntlmV2Authentication": SimStruct({"flags": SimTypeInt(signed=False, label="UInt32")}, name="IKEEXT_NTLM_V2_AUTHENTICATION0", pack=False, align=None), "sslAuthentication": SimStruct({"inboundConfigType": SimTypeInt(signed=False, label="IKEEXT_CERT_CONFIG_TYPE"), "Anonymous1": SimUnion({"Anonymous": SimStruct({"inboundRootArraySize": SimTypeInt(signed=False, label="UInt32"), "inboundRootArray": SimTypePointer(SimStruct({"certData": SimStruct({"size": SimTypeInt(signed=False, label="UInt32"), "data": SimTypePointer(SimTypeChar(label="Byte"), offset=0)}, name="FWP_BYTE_BLOB", pack=False, align=None), "flags": SimTypeInt(signed=False, label="IKEEXT_CERT_FLAGS")}, name="IKEEXT_CERT_ROOT_CONFIG0", pack=False, align=None), offset=0)}, name="_Anonymous_e__Struct", pack=False, align=None), "inboundEnterpriseStoreConfig": SimTypePointer(SimStruct({"certData": SimStruct({"size": SimTypeInt(signed=False, label="UInt32"), "data": SimTypePointer(SimTypeChar(label="Byte"), offset=0)}, name="FWP_BYTE_BLOB", pack=False, align=None), "flags": SimTypeInt(signed=False, label="IKEEXT_CERT_FLAGS")}, name="IKEEXT_CERT_ROOT_CONFIG0", pack=False, align=None), offset=0), "inboundTrustedRootStoreConfig": SimTypePointer(SimStruct({"certData": SimStruct({"size": SimTypeInt(signed=False, label="UInt32"), "data": SimTypePointer(SimTypeChar(label="Byte"), offset=0)}, name="FWP_BYTE_BLOB", pack=False, align=None), "flags": SimTypeInt(signed=False, label="IKEEXT_CERT_FLAGS")}, name="IKEEXT_CERT_ROOT_CONFIG0", pack=False, align=None), offset=0)}, name="<anon>", label="None"), "outboundConfigType": SimTypeInt(signed=False, label="IKEEXT_CERT_CONFIG_TYPE"), "Anonymous2": SimUnion({"Anonymous": SimStruct({"outboundRootArraySize": SimTypeInt(signed=False, label="UInt32"), "outboundRootArray": SimTypePointer(SimStruct({"certData": SimStruct({"size": SimTypeInt(signed=False, label="UInt32"), "data": SimTypePointer(SimTypeChar(label="Byte"), offset=0)}, name="FWP_BYTE_BLOB", pack=False, align=None), "flags": SimTypeInt(signed=False, label="IKEEXT_CERT_FLAGS")}, name="IKEEXT_CERT_ROOT_CONFIG0", pack=False, align=None), offset=0)}, name="_Anonymous_e__Struct", pack=False, align=None), "outboundEnterpriseStoreConfig": SimTypePointer(SimStruct({"certData": SimStruct({"size": SimTypeInt(signed=False, label="UInt32"), "data": SimTypePointer(SimTypeChar(label="Byte"), offset=0)}, name="FWP_BYTE_BLOB", pack=False, align=None), "flags": SimTypeInt(signed=False, label="IKEEXT_CERT_FLAGS")}, name="IKEEXT_CERT_ROOT_CONFIG0", pack=False, align=None), offset=0), "outboundTrustedRootStoreConfig": SimTypePointer(SimStruct({"certData": SimStruct({"size": SimTypeInt(signed=False, label="UInt32"), "data": SimTypePointer(SimTypeChar(label="Byte"), offset=0)}, name="FWP_BYTE_BLOB", pack=False, align=None), "flags": SimTypeInt(signed=False, label="IKEEXT_CERT_FLAGS")}, name="IKEEXT_CERT_ROOT_CONFIG0", pack=False, align=None), offset=0)}, name="<anon>", label="None"), "flags": SimTypeInt(signed=False, label="IKEEXT_CERT_AUTH"), "localCertLocationUrl": SimStruct({"size": SimTypeInt(signed=False, label="UInt32"), "data": SimTypePointer(SimTypeChar(label="Byte"), offset=0)}, name="FWP_BYTE_BLOB", pack=False, align=None)}, name="IKEEXT_CERTIFICATE_AUTHENTICATION1", pack=False, align=None), "cgaAuthentication": SimStruct({"keyContainerName": SimTypePointer(SimTypeChar(label="Char"), offset=0), "cspName": SimTypePointer(SimTypeChar(label="Char"), offset=0), "cspType": SimTypeInt(signed=False, label="UInt32"), "cgaModifier": SimStruct({"byteArray16": SimTypeFixedSizeArray(SimTypeChar(label="Byte"), 16)}, name="FWP_BYTE_ARRAY16", pack=False, align=None), "cgaCollisionCount": SimTypeChar(label="Byte")}, name="IKEEXT_IPV6_CGA_AUTHENTICATION0", pack=False, align=None), "eapAuthentication": SimStruct({"flags": SimTypeInt(signed=False, label="IKEEXT_EAP_AUTHENTICATION_FLAGS")}, name="IKEEXT_EAP_AUTHENTICATION0", pack=False, align=None)}, name="<anon>", label="None")}, name="IKEEXT_AUTHENTICATION_METHOD1", pack=False, align=None), offset=0), "initiatorImpersonationType": SimTypeInt(signed=False, label="IKEEXT_AUTHENTICATION_IMPERSONATION_TYPE"), "numIkeProposals": SimTypeInt(signed=False, label="UInt32"), "ikeProposals": SimTypePointer(SimStruct({"cipherAlgorithm": SimStruct({"algoIdentifier": SimTypeInt(signed=False, label="IKEEXT_CIPHER_TYPE"), "keyLen": SimTypeInt(signed=False, label="UInt32"), "rounds": SimTypeInt(signed=False, label="UInt32")}, name="IKEEXT_CIPHER_ALGORITHM0", pack=False, align=None), "integrityAlgorithm": SimStruct({"algoIdentifier": SimTypeInt(signed=False, label="IKEEXT_INTEGRITY_TYPE")}, name="IKEEXT_INTEGRITY_ALGORITHM0", pack=False, align=None), "maxLifetimeSeconds": SimTypeInt(signed=False, label="UInt32"), "dhGroup": SimTypeInt(signed=False, label="IKEEXT_DH_GROUP"), "quickModeLimit": SimTypeInt(signed=False, label="UInt32")}, name="IKEEXT_PROPOSAL0", pack=False, align=None), offset=0), "flags": SimTypeInt(signed=False, label="IKEEXT_POLICY_FLAG"), "maxDynamicFilters": SimTypeInt(signed=False, label="UInt32"), "retransmitDurationSecs": SimTypeInt(signed=False, label="UInt32")}, name="IKEEXT_POLICY1", pack=False, align=None), offset=0), "authIpMmPolicy": SimTypePointer(SimStruct({"softExpirationTime": SimTypeInt(signed=False, label="UInt32"), "numAuthenticationMethods": SimTypeInt(signed=False, label="UInt32"), "authenticationMethods": SimTypePointer(SimStruct({"authenticationMethodType": SimTypeInt(signed=False, label="IKEEXT_AUTHENTICATION_METHOD_TYPE"), "Anonymous": SimUnion({"presharedKeyAuthentication": SimStruct({"presharedKey": SimStruct({"size": SimTypeInt(signed=False, label="UInt32"), "data": SimTypePointer(SimTypeChar(label="Byte"), offset=0)}, name="FWP_BYTE_BLOB", pack=False, align=None), "flags": SimTypeInt(signed=False, label="IKEEXT_PRESHARED_KEY_AUTHENTICATION_FLAGS")}, name="IKEEXT_PRESHARED_KEY_AUTHENTICATION1", pack=False, align=None), "certificateAuthentication": SimStruct({"inboundConfigType": SimTypeInt(signed=False, label="IKEEXT_CERT_CONFIG_TYPE"), "Anonymous1": SimUnion({"Anonymous": SimStruct({"inboundRootArraySize": SimTypeInt(signed=False, label="UInt32"), "inboundRootArray": SimTypePointer(SimStruct({"certData": SimStruct({"size": SimTypeInt(signed=False, label="UInt32"), "data": SimTypePointer(SimTypeChar(label="Byte"), offset=0)}, name="FWP_BYTE_BLOB", pack=False, align=None), "flags": SimTypeInt(signed=False, label="IKEEXT_CERT_FLAGS")}, name="IKEEXT_CERT_ROOT_CONFIG0", pack=False, align=None), offset=0)}, name="_Anonymous_e__Struct", pack=False, align=None), "inboundEnterpriseStoreConfig": SimTypePointer(SimStruct({"certData": SimStruct({"size": SimTypeInt(signed=False, label="UInt32"), "data": SimTypePointer(SimTypeChar(label="Byte"), offset=0)}, name="FWP_BYTE_BLOB", pack=False, align=None), "flags": SimTypeInt(signed=False, label="IKEEXT_CERT_FLAGS")}, name="IKEEXT_CERT_ROOT_CONFIG0", pack=False, align=None), offset=0), "inboundTrustedRootStoreConfig": SimTypePointer(SimStruct({"certData": SimStruct({"size": SimTypeInt(signed=False, label="UInt32"), "data": SimTypePointer(SimTypeChar(label="Byte"), offset=0)}, name="FWP_BYTE_BLOB", pack=False, align=None), "flags": SimTypeInt(signed=False, label="IKEEXT_CERT_FLAGS")}, name="IKEEXT_CERT_ROOT_CONFIG0", pack=False, align=None), offset=0)}, name="<anon>", label="None"), "outboundConfigType": SimTypeInt(signed=False, label="IKEEXT_CERT_CONFIG_TYPE"), "Anonymous2": SimUnion({"Anonymous": SimStruct({"outboundRootArraySize": SimTypeInt(signed=False, label="UInt32"), "outboundRootArray": SimTypePointer(SimStruct({"certData": SimStruct({"size": SimTypeInt(signed=False, label="UInt32"), "data": SimTypePointer(SimTypeChar(label="Byte"), offset=0)}, name="FWP_BYTE_BLOB", pack=False, align=None), "flags": SimTypeInt(signed=False, label="IKEEXT_CERT_FLAGS")}, name="IKEEXT_CERT_ROOT_CONFIG0", pack=False, align=None), offset=0)}, name="_Anonymous_e__Struct", pack=False, align=None), "outboundEnterpriseStoreConfig": SimTypePointer(SimStruct({"certData": SimStruct({"size": SimTypeInt(signed=False, label="UInt32"), "data": SimTypePointer(SimTypeChar(label="Byte"), offset=0)}, name="FWP_BYTE_BLOB", pack=False, align=None), "flags": SimTypeInt(signed=False, label="IKEEXT_CERT_FLAGS")}, name="IKEEXT_CERT_ROOT_CONFIG0", pack=False, align=None), offset=0), "outboundTrustedRootStoreConfig": SimTypePointer(SimStruct({"certData": SimStruct({"size": SimTypeInt(signed=False, label="UInt32"), "data": SimTypePointer(SimTypeChar(label="Byte"), offset=0)}, name="FWP_BYTE_BLOB", pack=False, align=None), "flags": SimTypeInt(signed=False, label="IKEEXT_CERT_FLAGS")}, name="IKEEXT_CERT_ROOT_CONFIG0", pack=False, align=None), offset=0)}, name="<anon>", label="None"), "flags": SimTypeInt(signed=False, label="IKEEXT_CERT_AUTH"), "localCertLocationUrl": SimStruct({"size": SimTypeInt(signed=False, label="UInt32"), "data": SimTypePointer(SimTypeChar(label="Byte"), offset=0)}, name="FWP_BYTE_BLOB", pack=False, align=None)}, name="IKEEXT_CERTIFICATE_AUTHENTICATION1", pack=False, align=None), "kerberosAuthentication": SimStruct({"flags": SimTypeInt(signed=False, label="IKEEXT_KERBEROS_AUTHENTICATION_FLAGS")}, name="IKEEXT_KERBEROS_AUTHENTICATION0", pack=False, align=None), "ntlmV2Authentication": SimStruct({"flags": SimTypeInt(signed=False, label="UInt32")}, name="IKEEXT_NTLM_V2_AUTHENTICATION0", pack=False, align=None), "sslAuthentication": SimStruct({"inboundConfigType": SimTypeInt(signed=False, label="IKEEXT_CERT_CONFIG_TYPE"), "Anonymous1": SimUnion({"Anonymous": SimStruct({"inboundRootArraySize": SimTypeInt(signed=False, label="UInt32"), "inboundRootArray": SimTypePointer(SimStruct({"certData": SimStruct({"size": SimTypeInt(signed=False, label="UInt32"), "data": SimTypePointer(SimTypeChar(label="Byte"), offset=0)}, name="FWP_BYTE_BLOB", pack=False, align=None), "flags": SimTypeInt(signed=False, label="IKEEXT_CERT_FLAGS")}, name="IKEEXT_CERT_ROOT_CONFIG0", pack=False, align=None), offset=0)}, name="_Anonymous_e__Struct", pack=False, align=None), "inboundEnterpriseStoreConfig": SimTypePointer(SimStruct({"certData": SimStruct({"size": SimTypeInt(signed=False, label="UInt32"), "data": SimTypePointer(SimTypeChar(label="Byte"), offset=0)}, name="FWP_BYTE_BLOB", pack=False, align=None), "flags": SimTypeInt(signed=False, label="IKEEXT_CERT_FLAGS")}, name="IKEEXT_CERT_ROOT_CONFIG0", pack=False, align=None), offset=0), "inboundTrustedRootStoreConfig": SimTypePointer(SimStruct({"certData": SimStruct({"size": SimTypeInt(signed=False, label="UInt32"), "data": SimTypePointer(SimTypeChar(label="Byte"), offset=0)}, name="FWP_BYTE_BLOB", pack=False, align=None), "flags": SimTypeInt(signed=False, label="IKEEXT_CERT_FLAGS")}, name="IKEEXT_CERT_ROOT_CONFIG0", pack=False, align=None), offset=0)}, name="<anon>", label="None"), "outboundConfigType": SimTypeInt(signed=False, label="IKEEXT_CERT_CONFIG_TYPE"), "Anonymous2": SimUnion({"Anonymous": SimStruct({"outboundRootArraySize": SimTypeInt(signed=False, label="UInt32"), "outboundRootArray": SimTypePointer(SimStruct({"certData": SimStruct({"size": SimTypeInt(signed=False, label="UInt32"), "data": SimTypePointer(SimTypeChar(label="Byte"), offset=0)}, name="FWP_BYTE_BLOB", pack=False, align=None), "flags": SimTypeInt(signed=False, label="IKEEXT_CERT_FLAGS")}, name="IKEEXT_CERT_ROOT_CONFIG0", pack=False, align=None), offset=0)}, name="_Anonymous_e__Struct", pack=False, align=None), "outboundEnterpriseStoreConfig": SimTypePointer(SimStruct({"certData": SimStruct({"size": SimTypeInt(signed=False, label="UInt32"), "data": SimTypePointer(SimTypeChar(label="Byte"), offset=0)}, name="FWP_BYTE_BLOB", pack=False, align=None), "flags": SimTypeInt(signed=False, label="IKEEXT_CERT_FLAGS")}, name="IKEEXT_CERT_ROOT_CONFIG0", pack=False, align=None), offset=0), "outboundTrustedRootStoreConfig": SimTypePointer(SimStruct({"certData": SimStruct({"size": SimTypeInt(signed=False, label="UInt32"), "data": SimTypePointer(SimTypeChar(label="Byte"), offset=0)}, name="FWP_BYTE_BLOB", pack=False, align=None), "flags": SimTypeInt(signed=False, label="IKEEXT_CERT_FLAGS")}, name="IKEEXT_CERT_ROOT_CONFIG0", pack=False, align=None), offset=0)}, name="<anon>", label="None"), "flags": SimTypeInt(signed=False, label="IKEEXT_CERT_AUTH"), "localCertLocationUrl": SimStruct({"size": SimTypeInt(signed=False, label="UInt32"), "data": SimTypePointer(SimTypeChar(label="Byte"), offset=0)}, name="FWP_BYTE_BLOB", pack=False, align=None)}, name="IKEEXT_CERTIFICATE_AUTHENTICATION1", pack=False, align=None), "cgaAuthentication": SimStruct({"keyContainerName": SimTypePointer(SimTypeChar(label="Char"), offset=0), "cspName": SimTypePointer(SimTypeChar(label="Char"), offset=0), "cspType": SimTypeInt(signed=False, label="UInt32"), "cgaModifier": SimStruct({"byteArray16": SimTypeFixedSizeArray(SimTypeChar(label="Byte"), 16)}, name="FWP_BYTE_ARRAY16", pack=False, align=None), "cgaCollisionCount": SimTypeChar(label="Byte")}, name="IKEEXT_IPV6_CGA_AUTHENTICATION0", pack=False, align=None), "eapAuthentication": SimStruct({"flags": SimTypeInt(signed=False, label="IKEEXT_EAP_AUTHENTICATION_FLAGS")}, name="IKEEXT_EAP_AUTHENTICATION0", pack=False, align=None)}, name="<anon>", label="None")}, name="IKEEXT_AUTHENTICATION_METHOD1", pack=False, align=None), offset=0), "initiatorImpersonationType": SimTypeInt(signed=False, label="IKEEXT_AUTHENTICATION_IMPERSONATION_TYPE"), "numIkeProposals": SimTypeInt(signed=False, label="UInt32"), "ikeProposals": SimTypePointer(SimStruct({"cipherAlgorithm": SimStruct({"algoIdentifier": SimTypeInt(signed=False, label="IKEEXT_CIPHER_TYPE"), "keyLen": SimTypeInt(signed=False, label="UInt32"), "rounds": SimTypeInt(signed=False, label="UInt32")}, name="IKEEXT_CIPHER_ALGORITHM0", pack=False, align=None), "integrityAlgorithm": SimStruct({"algoIdentifier": SimTypeInt(signed=False, label="IKEEXT_INTEGRITY_TYPE")}, name="IKEEXT_INTEGRITY_ALGORITHM0", pack=False, align=None), "maxLifetimeSeconds": SimTypeInt(signed=False, label="UInt32"), "dhGroup": SimTypeInt(signed=False, label="IKEEXT_DH_GROUP"), "quickModeLimit": SimTypeInt(signed=False, label="UInt32")}, name="IKEEXT_PROPOSAL0", pack=False, align=None), offset=0), "flags": SimTypeInt(signed=False, label="IKEEXT_POLICY_FLAG"), "maxDynamicFilters": SimTypeInt(signed=False, label="UInt32"), "retransmitDurationSecs": SimTypeInt(signed=False, label="UInt32")}, name="IKEEXT_POLICY1", pack=False, align=None), offset=0), "dataBuffer": SimTypePointer(SimStruct({"size": SimTypeInt(signed=False, label="UInt32"), "data": SimTypePointer(SimTypeChar(label="Byte"), offset=0)}, name="FWP_BYTE_BLOB", pack=False, align=None), offset=0), "classifyOptions": SimTypePointer(SimStruct({"numOptions": SimTypeInt(signed=False, label="UInt32"), "options": SimTypePointer(SimStruct({"type": SimTypeInt(signed=False, label="FWP_CLASSIFY_OPTION_TYPE"), "value": SimStruct({"type": SimTypeInt(signed=False, label="FWP_DATA_TYPE"), "Anonymous": SimUnion({"uint8": SimTypeChar(label="Byte"), "uint16": SimTypeShort(signed=False, label="UInt16"), "uint32": SimTypeInt(signed=False, label="UInt32"), "uint64": SimTypePointer(SimTypeLongLong(signed=False, label="UInt64"), offset=0), "int8": SimTypeChar(label="SByte"), "int16": SimTypeShort(signed=True, label="Int16"), "int32": SimTypeInt(signed=True, label="Int32"), "int64": SimTypePointer(SimTypeLongLong(signed=True, label="Int64"), offset=0), "float32": SimTypeFloat(size=32), "double64": SimTypePointer(SimTypeFloat(size=64), offset=0), "byteArray16": SimTypePointer(SimStruct({"byteArray16": SimTypeFixedSizeArray(SimTypeChar(label="Byte"), 16)}, name="FWP_BYTE_ARRAY16", pack=False, align=None), offset=0), "byteBlob": SimTypePointer(SimStruct({"size": SimTypeInt(signed=False, label="UInt32"), "data": SimTypePointer(SimTypeChar(label="Byte"), offset=0)}, name="FWP_BYTE_BLOB", pack=False, align=None), offset=0), "sid": SimTypePointer(SimTypeBottom(label="SID"), offset=0), "sd": SimTypePointer(SimStruct({"size": SimTypeInt(signed=False, label="UInt32"), "data": SimTypePointer(SimTypeChar(label="Byte"), offset=0)}, name="FWP_BYTE_BLOB", pack=False, align=None), offset=0), "tokenInformation": SimTypePointer(SimStruct({"sidCount": SimTypeInt(signed=False, label="UInt32"), "sids": SimTypePointer(SimTypeBottom(label="SID_AND_ATTRIBUTES"), offset=0), "restrictedSidCount": SimTypeInt(signed=False, label="UInt32"), "restrictedSids": SimTypePointer(SimTypeBottom(label="SID_AND_ATTRIBUTES"), offset=0)}, name="FWP_TOKEN_INFORMATION", pack=False, align=None), offset=0), "tokenAccessInformation": SimTypePointer(SimStruct({"size": SimTypeInt(signed=False, label="UInt32"), "data": SimTypePointer(SimTypeChar(label="Byte"), offset=0)}, name="FWP_BYTE_BLOB", pack=False, align=None), offset=0), "unicodeString": SimTypePointer(SimTypeChar(label="Char"), offset=0), "byteArray6": SimTypePointer(SimStruct({"byteArray6": SimTypeFixedSizeArray(SimTypeChar(label="Byte"), 6)}, name="FWP_BYTE_ARRAY6", pack=False, align=None), offset=0), "bitmapArray64": SimTypePointer(SimStruct({"bitmapArray64": SimTypeFixedSizeArray(SimTypeChar(label="Byte"), 8)}, name="FWP_BITMAP_ARRAY64_", pack=False, align=None), offset=0)}, name="<anon>", label="None")}, name="FWP_VALUE0", pack=False, align=None)}, name="FWPM_CLASSIFY_OPTION0", pack=False, align=None), offset=0)}, name="FWPM_CLASSIFY_OPTIONS0", pack=False, align=None), offset=0), "ikeV2QmTunnelPolicy": SimTypePointer(SimStruct({"flags": SimTypeInt(signed=False, label="IPSEC_POLICY_FLAG"), "numIpsecProposals": SimTypeInt(signed=False, label="UInt32"), "ipsecProposals": SimTypePointer(SimStruct({"lifetime": SimStruct({"lifetimeSeconds": SimTypeInt(signed=False, label="UInt32"), "lifetimeKilobytes": SimTypeInt(signed=False, label="UInt32"), "lifetimePackets": SimTypeInt(signed=False, label="UInt32")}, name="IPSEC_SA_LIFETIME0", pack=False, align=None), "numSaTransforms": SimTypeInt(signed=False, label="UInt32"), "saTransforms": SimTypePointer(SimStruct({"ipsecTransformType": SimTypeInt(signed=False, label="IPSEC_TRANSFORM_TYPE"), "Anonymous": SimUnion({"ahTransform": SimTypePointer(SimStruct({"authTransformId": SimStruct({"authType": SimTypeInt(signed=False, label="IPSEC_AUTH_TYPE"), "authConfig": SimTypeChar(label="Byte")}, name="IPSEC_AUTH_TRANSFORM_ID0", pack=False, align=None), "cryptoModuleId": SimTypePointer(SimTypeBottom(label="Guid"), offset=0)}, name="IPSEC_AUTH_TRANSFORM0", pack=False, align=None), offset=0), "espAuthTransform": SimTypePointer(SimStruct({"authTransformId": SimStruct({"authType": SimTypeInt(signed=False, label="IPSEC_AUTH_TYPE"), "authConfig": SimTypeChar(label="Byte")}, name="IPSEC_AUTH_TRANSFORM_ID0", pack=False, align=None), "cryptoModuleId": SimTypePointer(SimTypeBottom(label="Guid"), offset=0)}, name="IPSEC_AUTH_TRANSFORM0", pack=False, align=None), offset=0), "espCipherTransform": SimTypePointer(SimStruct({"cipherTransformId": SimStruct({"cipherType": SimTypeInt(signed=False, label="IPSEC_CIPHER_TYPE"), "cipherConfig": SimTypeChar(label="Byte")}, name="IPSEC_CIPHER_TRANSFORM_ID0", pack=False, align=None), "cryptoModuleId": SimTypePointer(SimTypeBottom(label="Guid"), offset=0)}, name="IPSEC_CIPHER_TRANSFORM0", pack=False, align=None), offset=0), "espAuthAndCipherTransform": SimTypePointer(SimStruct({"authTransform": SimStruct({"authTransformId": SimStruct({"authType": SimTypeInt(signed=False, label="IPSEC_AUTH_TYPE"), "authConfig": SimTypeChar(label="Byte")}, name="IPSEC_AUTH_TRANSFORM_ID0", pack=False, align=None), "cryptoModuleId": SimTypePointer(SimTypeBottom(label="Guid"), offset=0)}, name="IPSEC_AUTH_TRANSFORM0", pack=False, align=None), "cipherTransform": SimStruct({"cipherTransformId": SimStruct({"cipherType": SimTypeInt(signed=False, label="IPSEC_CIPHER_TYPE"), "cipherConfig": SimTypeChar(label="Byte")}, name="IPSEC_CIPHER_TRANSFORM_ID0", pack=False, align=None), "cryptoModuleId": SimTypePointer(SimTypeBottom(label="Guid"), offset=0)}, name="IPSEC_CIPHER_TRANSFORM0", pack=False, align=None)}, name="IPSEC_AUTH_AND_CIPHER_TRANSFORM0", pack=False, align=None), offset=0), "espAuthFwTransform": SimTypePointer(SimStruct({"authTransformId": SimStruct({"authType": SimTypeInt(signed=False, label="IPSEC_AUTH_TYPE"), "authConfig": SimTypeChar(label="Byte")}, name="IPSEC_AUTH_TRANSFORM_ID0", pack=False, align=None), "cryptoModuleId": SimTypePointer(SimTypeBottom(label="Guid"), offset=0)}, name="IPSEC_AUTH_TRANSFORM0", pack=False, align=None), offset=0)}, name="<anon>", label="None")}, name="IPSEC_SA_TRANSFORM0", pack=False, align=None), offset=0), "pfsGroup": SimTypeInt(signed=False, label="IPSEC_PFS_GROUP")}, name="IPSEC_PROPOSAL0", pack=False, align=None), offset=0), "tunnelEndpoints": SimStruct({"ipVersion": SimTypeInt(signed=False, label="FWP_IP_VERSION"), "Anonymous1": SimUnion({"localV4Address": SimTypeInt(signed=False, label="UInt32"), "localV6Address": SimTypeFixedSizeArray(SimTypeChar(label="Byte"), 16)}, name="<anon>", label="None"), "Anonymous2": SimUnion({"remoteV4Address": SimTypeInt(signed=False, label="UInt32"), "remoteV6Address": SimTypeFixedSizeArray(SimTypeChar(label="Byte"), 16)}, name="<anon>", label="None"), "localIfLuid": SimTypeLongLong(signed=False, label="UInt64")}, name="IPSEC_TUNNEL_ENDPOINTS1", pack=False, align=None), "saIdleTimeout": SimStruct({"idleTimeoutSeconds": SimTypeInt(signed=False, label="UInt32"), "idleTimeoutSecondsFailOver": SimTypeInt(signed=False, label="UInt32")}, name="IPSEC_SA_IDLE_TIMEOUT0", pack=False, align=None), "emPolicy": SimTypePointer(SimStruct({"numAuthenticationMethods": SimTypeInt(signed=False, label="UInt32"), "authenticationMethods": SimTypePointer(SimStruct({"authenticationMethodType": SimTypeInt(signed=False, label="IKEEXT_AUTHENTICATION_METHOD_TYPE"), "Anonymous": SimUnion({"presharedKeyAuthentication": SimStruct({"presharedKey": SimStruct({"size": SimTypeInt(signed=False, label="UInt32"), "data": SimTypePointer(SimTypeChar(label="Byte"), offset=0)}, name="FWP_BYTE_BLOB", pack=False, align=None), "flags": SimTypeInt(signed=False, label="IKEEXT_PRESHARED_KEY_AUTHENTICATION_FLAGS")}, name="IKEEXT_PRESHARED_KEY_AUTHENTICATION1", pack=False, align=None), "certificateAuthentication": SimStruct({"inboundConfigType": SimTypeInt(signed=False, label="IKEEXT_CERT_CONFIG_TYPE"), "Anonymous1": SimUnion({"Anonymous": SimStruct({"inboundRootArraySize": SimTypeInt(signed=False, label="UInt32"), "inboundRootArray": SimTypePointer(SimStruct({"certData": SimStruct({"size": SimTypeInt(signed=False, label="UInt32"), "data": SimTypePointer(SimTypeChar(label="Byte"), offset=0)}, name="FWP_BYTE_BLOB", pack=False, align=None), "flags": SimTypeInt(signed=False, label="IKEEXT_CERT_FLAGS")}, name="IKEEXT_CERT_ROOT_CONFIG0", pack=False, align=None), offset=0)}, name="_Anonymous_e__Struct", pack=False, align=None), "inboundEnterpriseStoreConfig": SimTypePointer(SimStruct({"certData": SimStruct({"size": SimTypeInt(signed=False, label="UInt32"), "data": SimTypePointer(SimTypeChar(label="Byte"), offset=0)}, name="FWP_BYTE_BLOB", pack=False, align=None), "flags": SimTypeInt(signed=False, label="IKEEXT_CERT_FLAGS")}, name="IKEEXT_CERT_ROOT_CONFIG0", pack=False, align=None), offset=0), "inboundTrustedRootStoreConfig": SimTypePointer(SimStruct({"certData": SimStruct({"size": SimTypeInt(signed=False, label="UInt32"), "data": SimTypePointer(SimTypeChar(label="Byte"), offset=0)}, name="FWP_BYTE_BLOB", pack=False, align=None), "flags": SimTypeInt(signed=False, label="IKEEXT_CERT_FLAGS")}, name="IKEEXT_CERT_ROOT_CONFIG0", pack=False, align=None), offset=0)}, name="<anon>", label="None"), "outboundConfigType": SimTypeInt(signed=False, label="IKEEXT_CERT_CONFIG_TYPE"), "Anonymous2": SimUnion({"Anonymous": SimStruct({"outboundRootArraySize": SimTypeInt(signed=False, label="UInt32"), "outboundRootArray": SimTypePointer(SimStruct({"certData": SimStruct({"size": SimTypeInt(signed=False, label="UInt32"), "data": SimTypePointer(SimTypeChar(label="Byte"), offset=0)}, name="FWP_BYTE_BLOB", pack=False, align=None), "flags": SimTypeInt(signed=False, label="IKEEXT_CERT_FLAGS")}, name="IKEEXT_CERT_ROOT_CONFIG0", pack=False, align=None), offset=0)}, name="_Anonymous_e__Struct", pack=False, align=None), "outboundEnterpriseStoreConfig": SimTypePointer(SimStruct({"certData": SimStruct({"size": SimTypeInt(signed=False, label="UInt32"), "data": SimTypePointer(SimTypeChar(label="Byte"), offset=0)}, name="FWP_BYTE_BLOB", pack=False, align=None), "flags": SimTypeInt(signed=False, label="IKEEXT_CERT_FLAGS")}, name="IKEEXT_CERT_ROOT_CONFIG0", pack=False, align=None), offset=0), "outboundTrustedRootStoreConfig": SimTypePointer(SimStruct({"certData": SimStruct({"size": SimTypeInt(signed=False, label="UInt32"), "data": SimTypePointer(SimTypeChar(label="Byte"), offset=0)}, name="FWP_BYTE_BLOB", pack=False, align=None), "flags": SimTypeInt(signed=False, label="IKEEXT_CERT_FLAGS")}, name="IKEEXT_CERT_ROOT_CONFIG0", pack=False, align=None), offset=0)}, name="<anon>", label="None"), "flags": SimTypeInt(signed=False, label="IKEEXT_CERT_AUTH"), "localCertLocationUrl": SimStruct({"size": SimTypeInt(signed=False, label="UInt32"), "data": SimTypePointer(SimTypeChar(label="Byte"), offset=0)}, name="FWP_BYTE_BLOB", pack=False, align=None)}, name="IKEEXT_CERTIFICATE_AUTHENTICATION1", pack=False, align=None), "kerberosAuthentication": SimStruct({"flags": SimTypeInt(signed=False, label="IKEEXT_KERBEROS_AUTHENTICATION_FLAGS")}, name="IKEEXT_KERBEROS_AUTHENTICATION0", pack=False, align=None), "ntlmV2Authentication": SimStruct({"flags": SimTypeInt(signed=False, label="UInt32")}, name="IKEEXT_NTLM_V2_AUTHENTICATION0", pack=False, align=None), "sslAuthentication": SimStruct({"inboundConfigType": SimTypeInt(signed=False, label="IKEEXT_CERT_CONFIG_TYPE"), "Anonymous1": SimUnion({"Anonymous": SimStruct({"inboundRootArraySize": SimTypeInt(signed=False, label="UInt32"), "inboundRootArray": SimTypePointer(SimStruct({"certData": SimStruct({"size": SimTypeInt(signed=False, label="UInt32"), "data": SimTypePointer(SimTypeChar(label="Byte"), offset=0)}, name="FWP_BYTE_BLOB", pack=False, align=None), "flags": SimTypeInt(signed=False, label="IKEEXT_CERT_FLAGS")}, name="IKEEXT_CERT_ROOT_CONFIG0", pack=False, align=None), offset=0)}, name="_Anonymous_e__Struct", pack=False, align=None), "inboundEnterpriseStoreConfig": SimTypePointer(SimStruct({"certData": SimStruct({"size": SimTypeInt(signed=False, label="UInt32"), "data": SimTypePointer(SimTypeChar(label="Byte"), offset=0)}, name="FWP_BYTE_BLOB", pack=False, align=None), "flags": SimTypeInt(signed=False, label="IKEEXT_CERT_FLAGS")}, name="IKEEXT_CERT_ROOT_CONFIG0", pack=False, align=None), offset=0), "inboundTrustedRootStoreConfig": SimTypePointer(SimStruct({"certData": SimStruct({"size": SimTypeInt(signed=False, label="UInt32"), "data": SimTypePointer(SimTypeChar(label="Byte"), offset=0)}, name="FWP_BYTE_BLOB", pack=False, align=None), "flags": SimTypeInt(signed=False, label="IKEEXT_CERT_FLAGS")}, name="IKEEXT_CERT_ROOT_CONFIG0", pack=False, align=None), offset=0)}, name="<anon>", label="None"), "outboundConfigType": SimTypeInt(signed=False, label="IKEEXT_CERT_CONFIG_TYPE"), "Anonymous2": SimUnion({"Anonymous": SimStruct({"outboundRootArraySize": SimTypeInt(signed=False, label="UInt32"), "outboundRootArray": SimTypePointer(SimStruct({"certData": SimStruct({"size": SimTypeInt(signed=False, label="UInt32"), "data": SimTypePointer(SimTypeChar(label="Byte"), offset=0)}, name="FWP_BYTE_BLOB", pack=False, align=None), "flags": SimTypeInt(signed=False, label="IKEEXT_CERT_FLAGS")}, name="IKEEXT_CERT_ROOT_CONFIG0", pack=False, align=None), offset=0)}, name="_Anonymous_e__Struct", pack=False, align=None), "outboundEnterpriseStoreConfig": SimTypePointer(SimStruct({"certData": SimStruct({"size": SimTypeInt(signed=False, label="UInt32"), "data": SimTypePointer(SimTypeChar(label="Byte"), offset=0)}, name="FWP_BYTE_BLOB", pack=False, align=None), "flags": SimTypeInt(signed=False, label="IKEEXT_CERT_FLAGS")}, name="IKEEXT_CERT_ROOT_CONFIG0", pack=False, align=None), offset=0), "outboundTrustedRootStoreConfig": SimTypePointer(SimStruct({"certData": SimStruct({"size": SimTypeInt(signed=False, label="UInt32"), "data": SimTypePointer(SimTypeChar(label="Byte"), offset=0)}, name="FWP_BYTE_BLOB", pack=False, align=None), "flags": SimTypeInt(signed=False, label="IKEEXT_CERT_FLAGS")}, name="IKEEXT_CERT_ROOT_CONFIG0", pack=False, align=None), offset=0)}, name="<anon>", label="None"), "flags": SimTypeInt(signed=False, label="IKEEXT_CERT_AUTH"), "localCertLocationUrl": SimStruct({"size": SimTypeInt(signed=False, label="UInt32"), "data": SimTypePointer(SimTypeChar(label="Byte"), offset=0)}, name="FWP_BYTE_BLOB", pack=False, align=None)}, name="IKEEXT_CERTIFICATE_AUTHENTICATION1", pack=False, align=None), "cgaAuthentication": SimStruct({"keyContainerName": SimTypePointer(SimTypeChar(label="Char"), offset=0), "cspName": SimTypePointer(SimTypeChar(label="Char"), offset=0), "cspType": SimTypeInt(signed=False, label="UInt32"), "cgaModifier": SimStruct({"byteArray16": SimTypeFixedSizeArray(SimTypeChar(label="Byte"), 16)}, name="FWP_BYTE_ARRAY16", pack=False, align=None), "cgaCollisionCount": SimTypeChar(label="Byte")}, name="IKEEXT_IPV6_CGA_AUTHENTICATION0", pack=False, align=None), "eapAuthentication": SimStruct({"flags": SimTypeInt(signed=False, label="IKEEXT_EAP_AUTHENTICATION_FLAGS")}, name="IKEEXT_EAP_AUTHENTICATION0", pack=False, align=None)}, name="<anon>", label="None")}, name="IKEEXT_AUTHENTICATION_METHOD1", pack=False, align=None), offset=0), "initiatorImpersonationType": SimTypeInt(signed=False, label="IKEEXT_AUTHENTICATION_IMPERSONATION_TYPE")}, name="IKEEXT_EM_POLICY1", pack=False, align=None), offset=0)}, name="IPSEC_TUNNEL_POLICY1", pack=False, align=None), offset=0), "ikeV2MmPolicy": SimTypePointer(SimStruct({"softExpirationTime": SimTypeInt(signed=False, label="UInt32"), "numAuthenticationMethods": SimTypeInt(signed=False, label="UInt32"), "authenticationMethods": SimTypePointer(SimStruct({"authenticationMethodType": SimTypeInt(signed=False, label="IKEEXT_AUTHENTICATION_METHOD_TYPE"), "Anonymous": SimUnion({"presharedKeyAuthentication": SimStruct({"presharedKey": SimStruct({"size": SimTypeInt(signed=False, label="UInt32"), "data": SimTypePointer(SimTypeChar(label="Byte"), offset=0)}, name="FWP_BYTE_BLOB", pack=False, align=None), "flags": SimTypeInt(signed=False, label="IKEEXT_PRESHARED_KEY_AUTHENTICATION_FLAGS")}, name="IKEEXT_PRESHARED_KEY_AUTHENTICATION1", pack=False, align=None), "certificateAuthentication": SimStruct({"inboundConfigType": SimTypeInt(signed=False, label="IKEEXT_CERT_CONFIG_TYPE"), "Anonymous1": SimUnion({"Anonymous": SimStruct({"inboundRootArraySize": SimTypeInt(signed=False, label="UInt32"), "inboundRootArray": SimTypePointer(SimStruct({"certData": SimStruct({"size": SimTypeInt(signed=False, label="UInt32"), "data": SimTypePointer(SimTypeChar(label="Byte"), offset=0)}, name="FWP_BYTE_BLOB", pack=False, align=None), "flags": SimTypeInt(signed=False, label="IKEEXT_CERT_FLAGS")}, name="IKEEXT_CERT_ROOT_CONFIG0", pack=False, align=None), offset=0)}, name="_Anonymous_e__Struct", pack=False, align=None), "inboundEnterpriseStoreConfig": SimTypePointer(SimStruct({"certData": SimStruct({"size": SimTypeInt(signed=False, label="UInt32"), "data": SimTypePointer(SimTypeChar(label="Byte"), offset=0)}, name="FWP_BYTE_BLOB", pack=False, align=None), "flags": SimTypeInt(signed=False, label="IKEEXT_CERT_FLAGS")}, name="IKEEXT_CERT_ROOT_CONFIG0", pack=False, align=None), offset=0), "inboundTrustedRootStoreConfig": SimTypePointer(SimStruct({"certData": SimStruct({"size": SimTypeInt(signed=False, label="UInt32"), "data": SimTypePointer(SimTypeChar(label="Byte"), offset=0)}, name="FWP_BYTE_BLOB", pack=False, align=None), "flags": SimTypeInt(signed=False, label="IKEEXT_CERT_FLAGS")}, name="IKEEXT_CERT_ROOT_CONFIG0", pack=False, align=None), offset=0)}, name="<anon>", label="None"), "outboundConfigType": SimTypeInt(signed=False, label="IKEEXT_CERT_CONFIG_TYPE"), "Anonymous2": SimUnion({"Anonymous": SimStruct({"outboundRootArraySize": SimTypeInt(signed=False, label="UInt32"), "outboundRootArray": SimTypePointer(SimStruct({"certData": SimStruct({"size": SimTypeInt(signed=False, label="UInt32"), "data": SimTypePointer(SimTypeChar(label="Byte"), offset=0)}, name="FWP_BYTE_BLOB", pack=False, align=None), "flags": SimTypeInt(signed=False, label="IKEEXT_CERT_FLAGS")}, name="IKEEXT_CERT_ROOT_CONFIG0", pack=False, align=None), offset=0)}, name="_Anonymous_e__Struct", pack=False, align=None), "outboundEnterpriseStoreConfig": SimTypePointer(SimStruct({"certData": SimStruct({"size": SimTypeInt(signed=False, label="UInt32"), "data": SimTypePointer(SimTypeChar(label="Byte"), offset=0)}, name="FWP_BYTE_BLOB", pack=False, align=None), "flags": SimTypeInt(signed=False, label="IKEEXT_CERT_FLAGS")}, name="IKEEXT_CERT_ROOT_CONFIG0", pack=False, align=None), offset=0), "outboundTrustedRootStoreConfig": SimTypePointer(SimStruct({"certData": SimStruct({"size": SimTypeInt(signed=False, label="UInt32"), "data": SimTypePointer(SimTypeChar(label="Byte"), offset=0)}, name="FWP_BYTE_BLOB", pack=False, align=None), "flags": SimTypeInt(signed=False, label="IKEEXT_CERT_FLAGS")}, name="IKEEXT_CERT_ROOT_CONFIG0", pack=False, align=None), offset=0)}, name="<anon>", label="None"), "flags": SimTypeInt(signed=False, label="IKEEXT_CERT_AUTH"), "localCertLocationUrl": SimStruct({"size": SimTypeInt(signed=False, label="UInt32"), "data": SimTypePointer(SimTypeChar(label="Byte"), offset=0)}, name="FWP_BYTE_BLOB", pack=False, align=None)}, name="IKEEXT_CERTIFICATE_AUTHENTICATION1", pack=False, align=None), "kerberosAuthentication": SimStruct({"flags": SimTypeInt(signed=False, label="IKEEXT_KERBEROS_AUTHENTICATION_FLAGS")}, name="IKEEXT_KERBEROS_AUTHENTICATION0", pack=False, align=None), "ntlmV2Authentication": SimStruct({"flags": SimTypeInt(signed=False, label="UInt32")}, name="IKEEXT_NTLM_V2_AUTHENTICATION0", pack=False, align=None), "sslAuthentication": SimStruct({"inboundConfigType": SimTypeInt(signed=False, label="IKEEXT_CERT_CONFIG_TYPE"), "Anonymous1": SimUnion({"Anonymous": SimStruct({"inboundRootArraySize": SimTypeInt(signed=False, label="UInt32"), "inboundRootArray": SimTypePointer(SimStruct({"certData": SimStruct({"size": SimTypeInt(signed=False, label="UInt32"), "data": SimTypePointer(SimTypeChar(label="Byte"), offset=0)}, name="FWP_BYTE_BLOB", pack=False, align=None), "flags": SimTypeInt(signed=False, label="IKEEXT_CERT_FLAGS")}, name="IKEEXT_CERT_ROOT_CONFIG0", pack=False, align=None), offset=0)}, name="_Anonymous_e__Struct", pack=False, align=None), "inboundEnterpriseStoreConfig": SimTypePointer(SimStruct({"certData": SimStruct({"size": SimTypeInt(signed=False, label="UInt32"), "data": SimTypePointer(SimTypeChar(label="Byte"), offset=0)}, name="FWP_BYTE_BLOB", pack=False, align=None), "flags": SimTypeInt(signed=False, label="IKEEXT_CERT_FLAGS")}, name="IKEEXT_CERT_ROOT_CONFIG0", pack=False, align=None), offset=0), "inboundTrustedRootStoreConfig": SimTypePointer(SimStruct({"certData": SimStruct({"size": SimTypeInt(signed=False, label="UInt32"), "data": SimTypePointer(SimTypeChar(label="Byte"), offset=0)}, name="FWP_BYTE_BLOB", pack=False, align=None), "flags": SimTypeInt(signed=False, label="IKEEXT_CERT_FLAGS")}, name="IKEEXT_CERT_ROOT_CONFIG0", pack=False, align=None), offset=0)}, name="<anon>", label="None"), "outboundConfigType": SimTypeInt(signed=False, label="IKEEXT_CERT_CONFIG_TYPE"), "Anonymous2": SimUnion({"Anonymous": SimStruct({"outboundRootArraySize": SimTypeInt(signed=False, label="UInt32"), "outboundRootArray": SimTypePointer(SimStruct({"certData": SimStruct({"size": SimTypeInt(signed=False, label="UInt32"), "data": SimTypePointer(SimTypeChar(label="Byte"), offset=0)}, name="FWP_BYTE_BLOB", pack=False, align=None), "flags": SimTypeInt(signed=False, label="IKEEXT_CERT_FLAGS")}, name="IKEEXT_CERT_ROOT_CONFIG0", pack=False, align=None), offset=0)}, name="_Anonymous_e__Struct", pack=False, align=None), "outboundEnterpriseStoreConfig": SimTypePointer(SimStruct({"certData": SimStruct({"size": SimTypeInt(signed=False, label="UInt32"), "data": SimTypePointer(SimTypeChar(label="Byte"), offset=0)}, name="FWP_BYTE_BLOB", pack=False, align=None), "flags": SimTypeInt(signed=False, label="IKEEXT_CERT_FLAGS")}, name="IKEEXT_CERT_ROOT_CONFIG0", pack=False, align=None), offset=0), "outboundTrustedRootStoreConfig": SimTypePointer(SimStruct({"certData": SimStruct({"size": SimTypeInt(signed=False, label="UInt32"), "data": SimTypePointer(SimTypeChar(label="Byte"), offset=0)}, name="FWP_BYTE_BLOB", pack=False, align=None), "flags": SimTypeInt(signed=False, label="IKEEXT_CERT_FLAGS")}, name="IKEEXT_CERT_ROOT_CONFIG0", pack=False, align=None), offset=0)}, name="<anon>", label="None"), "flags": SimTypeInt(signed=False, label="IKEEXT_CERT_AUTH"), "localCertLocationUrl": SimStruct({"size": SimTypeInt(signed=False, label="UInt32"), "data": SimTypePointer(SimTypeChar(label="Byte"), offset=0)}, name="FWP_BYTE_BLOB", pack=False, align=None)}, name="IKEEXT_CERTIFICATE_AUTHENTICATION1", pack=False, align=None), "cgaAuthentication": SimStruct({"keyContainerName": SimTypePointer(SimTypeChar(label="Char"), offset=0), "cspName": SimTypePointer(SimTypeChar(label="Char"), offset=0), "cspType": SimTypeInt(signed=False, label="UInt32"), "cgaModifier": SimStruct({"byteArray16": SimTypeFixedSizeArray(SimTypeChar(label="Byte"), 16)}, name="FWP_BYTE_ARRAY16", pack=False, align=None), "cgaCollisionCount": SimTypeChar(label="Byte")}, name="IKEEXT_IPV6_CGA_AUTHENTICATION0", pack=False, align=None), "eapAuthentication": SimStruct({"flags": SimTypeInt(signed=False, label="IKEEXT_EAP_AUTHENTICATION_FLAGS")}, name="IKEEXT_EAP_AUTHENTICATION0", pack=False, align=None)}, name="<anon>", label="None")}, name="IKEEXT_AUTHENTICATION_METHOD1", pack=False, align=None), offset=0), "initiatorImpersonationType": SimTypeInt(signed=False, label="IKEEXT_AUTHENTICATION_IMPERSONATION_TYPE"), "numIkeProposals": SimTypeInt(signed=False, label="UInt32"), "ikeProposals": SimTypePointer(SimStruct({"cipherAlgorithm": SimStruct({"algoIdentifier": SimTypeInt(signed=False, label="IKEEXT_CIPHER_TYPE"), "keyLen": SimTypeInt(signed=False, label="UInt32"), "rounds": SimTypeInt(signed=False, label="UInt32")}, name="IKEEXT_CIPHER_ALGORITHM0", pack=False, align=None), "integrityAlgorithm": SimStruct({"algoIdentifier": SimTypeInt(signed=False, label="IKEEXT_INTEGRITY_TYPE")}, name="IKEEXT_INTEGRITY_ALGORITHM0", pack=False, align=None), "maxLifetimeSeconds": SimTypeInt(signed=False, label="UInt32"), "dhGroup": SimTypeInt(signed=False, label="IKEEXT_DH_GROUP"), "quickModeLimit": SimTypeInt(signed=False, label="UInt32")}, name="IKEEXT_PROPOSAL0", pack=False, align=None), offset=0), "flags": SimTypeInt(signed=False, label="IKEEXT_POLICY_FLAG"), "maxDynamicFilters": SimTypeInt(signed=False, label="UInt32"), "retransmitDurationSecs": SimTypeInt(signed=False, label="UInt32")}, name="IKEEXT_POLICY1", pack=False, align=None), offset=0), "idpOptions": SimTypePointer(SimStruct({"stateIdleTimeoutSeconds": SimTypeInt(signed=False, label="UInt32"), "perIPRateLimitQueueIdleTimeoutSeconds": SimTypeInt(signed=False, label="UInt32"), "ipV6IPsecUnauthDscp": SimTypeChar(label="Byte"), "ipV6IPsecUnauthRateLimitBytesPerSec": SimTypeInt(signed=False, label="UInt32"), "ipV6IPsecUnauthPerIPRateLimitBytesPerSec": SimTypeInt(signed=False, label="UInt32"), "ipV6IPsecAuthDscp": SimTypeChar(label="Byte"), "ipV6IPsecAuthRateLimitBytesPerSec": SimTypeInt(signed=False, label="UInt32"), "icmpV6Dscp": SimTypeChar(label="Byte"), "icmpV6RateLimitBytesPerSec": SimTypeInt(signed=False, label="UInt32"), "ipV6FilterExemptDscp": SimTypeChar(label="Byte"), "ipV6FilterExemptRateLimitBytesPerSec": SimTypeInt(signed=False, label="UInt32"), "defBlockExemptDscp": SimTypeChar(label="Byte"), "defBlockExemptRateLimitBytesPerSec": SimTypeInt(signed=False, label="UInt32"), "maxStateEntries": SimTypeInt(signed=False, label="UInt32"), "maxPerIPRateLimitQueues": SimTypeInt(signed=False, label="UInt32"), "flags": SimTypeInt(signed=False, label="IPSEC_DOSP_FLAGS"), "numPublicIFLuids": SimTypeInt(signed=False, label="UInt32"), "publicIFLuids": SimTypePointer(SimTypeLongLong(signed=False, label="UInt64"), offset=0), "numInternalIFLuids": SimTypeInt(signed=False, label="UInt32"), "internalIFLuids": SimTypePointer(SimTypeLongLong(signed=False, label="UInt64"), offset=0), "publicV6AddrMask": SimStruct({"addr": SimTypeFixedSizeArray(SimTypeChar(label="Byte"), 16), "prefixLength": SimTypeChar(label="Byte")}, name="FWP_V6_ADDR_AND_MASK", pack=False, align=None), "internalV6AddrMask": SimStruct({"addr": SimTypeFixedSizeArray(SimTypeChar(label="Byte"), 16), "prefixLength": SimTypeChar(label="Byte")}, name="FWP_V6_ADDR_AND_MASK", pack=False, align=None)}, name="IPSEC_DOSP_OPTIONS0", pack=False, align=None), offset=0)}, name="<anon>", label="None"), "providerContextId": SimTypeLongLong(signed=False, label="UInt64")}, name="FWPM_PROVIDER_CONTEXT1", pack=False, align=None), offset=0), offset=0)], SimTypeInt(signed=False, label="UInt32"), arg_names=["engineHandle", "key", "providerContext"]),
        # 
        'FwpmProviderContextGetByKey2': SimTypeFunction([SimTypePointer(SimTypeInt(signed=True, label="Int"), label="IntPtr", offset=0), SimTypePointer(SimTypeBottom(label="Guid"), offset=0), SimTypePointer(SimTypePointer(SimStruct({"providerContextKey": SimTypeBottom(label="Guid"), "displayData": SimStruct({"name": SimTypePointer(SimTypeChar(label="Char"), offset=0), "description": SimTypePointer(SimTypeChar(label="Char"), offset=0)}, name="FWPM_DISPLAY_DATA0", pack=False, align=None), "flags": SimTypeInt(signed=False, label="UInt32"), "providerKey": SimTypePointer(SimTypeBottom(label="Guid"), offset=0), "providerData": SimStruct({"size": SimTypeInt(signed=False, label="UInt32"), "data": SimTypePointer(SimTypeChar(label="Byte"), offset=0)}, name="FWP_BYTE_BLOB", pack=False, align=None), "type": SimTypeInt(signed=False, label="FWPM_PROVIDER_CONTEXT_TYPE"), "Anonymous": SimUnion({"keyingPolicy": SimTypePointer(SimStruct({"numKeyMods": SimTypeInt(signed=False, label="UInt32"), "keyModKeys": SimTypePointer(SimTypeBottom(label="Guid"), offset=0), "flags": SimTypeInt(signed=False, label="UInt32")}, name="IPSEC_KEYING_POLICY1", pack=False, align=None), offset=0), "ikeQmTransportPolicy": SimTypePointer(SimStruct({"numIpsecProposals": SimTypeInt(signed=False, label="UInt32"), "ipsecProposals": SimTypePointer(SimStruct({"lifetime": SimStruct({"lifetimeSeconds": SimTypeInt(signed=False, label="UInt32"), "lifetimeKilobytes": SimTypeInt(signed=False, label="UInt32"), "lifetimePackets": SimTypeInt(signed=False, label="UInt32")}, name="IPSEC_SA_LIFETIME0", pack=False, align=None), "numSaTransforms": SimTypeInt(signed=False, label="UInt32"), "saTransforms": SimTypePointer(SimStruct({"ipsecTransformType": SimTypeInt(signed=False, label="IPSEC_TRANSFORM_TYPE"), "Anonymous": SimUnion({"ahTransform": SimTypePointer(SimStruct({"authTransformId": SimStruct({"authType": SimTypeInt(signed=False, label="IPSEC_AUTH_TYPE"), "authConfig": SimTypeChar(label="Byte")}, name="IPSEC_AUTH_TRANSFORM_ID0", pack=False, align=None), "cryptoModuleId": SimTypePointer(SimTypeBottom(label="Guid"), offset=0)}, name="IPSEC_AUTH_TRANSFORM0", pack=False, align=None), offset=0), "espAuthTransform": SimTypePointer(SimStruct({"authTransformId": SimStruct({"authType": SimTypeInt(signed=False, label="IPSEC_AUTH_TYPE"), "authConfig": SimTypeChar(label="Byte")}, name="IPSEC_AUTH_TRANSFORM_ID0", pack=False, align=None), "cryptoModuleId": SimTypePointer(SimTypeBottom(label="Guid"), offset=0)}, name="IPSEC_AUTH_TRANSFORM0", pack=False, align=None), offset=0), "espCipherTransform": SimTypePointer(SimStruct({"cipherTransformId": SimStruct({"cipherType": SimTypeInt(signed=False, label="IPSEC_CIPHER_TYPE"), "cipherConfig": SimTypeChar(label="Byte")}, name="IPSEC_CIPHER_TRANSFORM_ID0", pack=False, align=None), "cryptoModuleId": SimTypePointer(SimTypeBottom(label="Guid"), offset=0)}, name="IPSEC_CIPHER_TRANSFORM0", pack=False, align=None), offset=0), "espAuthAndCipherTransform": SimTypePointer(SimStruct({"authTransform": SimStruct({"authTransformId": SimStruct({"authType": SimTypeInt(signed=False, label="IPSEC_AUTH_TYPE"), "authConfig": SimTypeChar(label="Byte")}, name="IPSEC_AUTH_TRANSFORM_ID0", pack=False, align=None), "cryptoModuleId": SimTypePointer(SimTypeBottom(label="Guid"), offset=0)}, name="IPSEC_AUTH_TRANSFORM0", pack=False, align=None), "cipherTransform": SimStruct({"cipherTransformId": SimStruct({"cipherType": SimTypeInt(signed=False, label="IPSEC_CIPHER_TYPE"), "cipherConfig": SimTypeChar(label="Byte")}, name="IPSEC_CIPHER_TRANSFORM_ID0", pack=False, align=None), "cryptoModuleId": SimTypePointer(SimTypeBottom(label="Guid"), offset=0)}, name="IPSEC_CIPHER_TRANSFORM0", pack=False, align=None)}, name="IPSEC_AUTH_AND_CIPHER_TRANSFORM0", pack=False, align=None), offset=0), "espAuthFwTransform": SimTypePointer(SimStruct({"authTransformId": SimStruct({"authType": SimTypeInt(signed=False, label="IPSEC_AUTH_TYPE"), "authConfig": SimTypeChar(label="Byte")}, name="IPSEC_AUTH_TRANSFORM_ID0", pack=False, align=None), "cryptoModuleId": SimTypePointer(SimTypeBottom(label="Guid"), offset=0)}, name="IPSEC_AUTH_TRANSFORM0", pack=False, align=None), offset=0)}, name="<anon>", label="None")}, name="IPSEC_SA_TRANSFORM0", pack=False, align=None), offset=0), "pfsGroup": SimTypeInt(signed=False, label="IPSEC_PFS_GROUP")}, name="IPSEC_PROPOSAL0", pack=False, align=None), offset=0), "flags": SimTypeInt(signed=False, label="IPSEC_POLICY_FLAG"), "ndAllowClearTimeoutSeconds": SimTypeInt(signed=False, label="UInt32"), "saIdleTimeout": SimStruct({"idleTimeoutSeconds": SimTypeInt(signed=False, label="UInt32"), "idleTimeoutSecondsFailOver": SimTypeInt(signed=False, label="UInt32")}, name="IPSEC_SA_IDLE_TIMEOUT0", pack=False, align=None), "emPolicy": SimTypePointer(SimStruct({"numAuthenticationMethods": SimTypeInt(signed=False, label="UInt32"), "authenticationMethods": SimTypePointer(SimStruct({"authenticationMethodType": SimTypeInt(signed=False, label="IKEEXT_AUTHENTICATION_METHOD_TYPE"), "Anonymous": SimUnion({"presharedKeyAuthentication": SimStruct({"presharedKey": SimStruct({"size": SimTypeInt(signed=False, label="UInt32"), "data": SimTypePointer(SimTypeChar(label="Byte"), offset=0)}, name="FWP_BYTE_BLOB", pack=False, align=None), "flags": SimTypeInt(signed=False, label="IKEEXT_PRESHARED_KEY_AUTHENTICATION_FLAGS")}, name="IKEEXT_PRESHARED_KEY_AUTHENTICATION1", pack=False, align=None), "certificateAuthentication": SimStruct({"inboundConfigType": SimTypeInt(signed=False, label="IKEEXT_CERT_CONFIG_TYPE"), "Anonymous1": SimUnion({"Anonymous1": SimStruct({"inboundRootArraySize": SimTypeInt(signed=False, label="UInt32"), "inboundRootCriteria": SimTypePointer(SimStruct({"certData": SimStruct({"size": SimTypeInt(signed=False, label="UInt32"), "data": SimTypePointer(SimTypeChar(label="Byte"), offset=0)}, name="FWP_BYTE_BLOB", pack=False, align=None), "certHash": SimStruct({"size": SimTypeInt(signed=False, label="UInt32"), "data": SimTypePointer(SimTypeChar(label="Byte"), offset=0)}, name="FWP_BYTE_BLOB", pack=False, align=None), "eku": SimTypePointer(SimStruct({"numEku": SimTypeInt(signed=False, label="UInt32"), "eku": SimTypePointer(SimTypePointer(SimTypeChar(label="Byte"), offset=0), offset=0)}, name="IKEEXT_CERT_EKUS0", pack=False, align=None), offset=0), "name": SimTypePointer(SimStruct({"nameType": SimTypeInt(signed=False, label="IKEEXT_CERT_CRITERIA_NAME_TYPE"), "certName": SimTypePointer(SimTypeChar(label="Char"), offset=0)}, name="IKEEXT_CERT_NAME0", pack=False, align=None), offset=0), "flags": SimTypeInt(signed=False, label="UInt32")}, name="IKEEXT_CERTIFICATE_CRITERIA0", pack=False, align=None), offset=0)}, name="_Anonymous1_e__Struct", pack=False, align=None), "Anonymous2": SimStruct({"inboundEnterpriseStoreArraySize": SimTypeInt(signed=False, label="UInt32"), "inboundEnterpriseStoreCriteria": SimTypePointer(SimStruct({"certData": SimStruct({"size": SimTypeInt(signed=False, label="UInt32"), "data": SimTypePointer(SimTypeChar(label="Byte"), offset=0)}, name="FWP_BYTE_BLOB", pack=False, align=None), "certHash": SimStruct({"size": SimTypeInt(signed=False, label="UInt32"), "data": SimTypePointer(SimTypeChar(label="Byte"), offset=0)}, name="FWP_BYTE_BLOB", pack=False, align=None), "eku": SimTypePointer(SimStruct({"numEku": SimTypeInt(signed=False, label="UInt32"), "eku": SimTypePointer(SimTypePointer(SimTypeChar(label="Byte"), offset=0), offset=0)}, name="IKEEXT_CERT_EKUS0", pack=False, align=None), offset=0), "name": SimTypePointer(SimStruct({"nameType": SimTypeInt(signed=False, label="IKEEXT_CERT_CRITERIA_NAME_TYPE"), "certName": SimTypePointer(SimTypeChar(label="Char"), offset=0)}, name="IKEEXT_CERT_NAME0", pack=False, align=None), offset=0), "flags": SimTypeInt(signed=False, label="UInt32")}, name="IKEEXT_CERTIFICATE_CRITERIA0", pack=False, align=None), offset=0)}, name="_Anonymous2_e__Struct", pack=False, align=None), "Anonymous3": SimStruct({"inboundRootStoreArraySize": SimTypeInt(signed=False, label="UInt32"), "inboundTrustedRootStoreCriteria": SimTypePointer(SimStruct({"certData": SimStruct({"size": SimTypeInt(signed=False, label="UInt32"), "data": SimTypePointer(SimTypeChar(label="Byte"), offset=0)}, name="FWP_BYTE_BLOB", pack=False, align=None), "certHash": SimStruct({"size": SimTypeInt(signed=False, label="UInt32"), "data": SimTypePointer(SimTypeChar(label="Byte"), offset=0)}, name="FWP_BYTE_BLOB", pack=False, align=None), "eku": SimTypePointer(SimStruct({"numEku": SimTypeInt(signed=False, label="UInt32"), "eku": SimTypePointer(SimTypePointer(SimTypeChar(label="Byte"), offset=0), offset=0)}, name="IKEEXT_CERT_EKUS0", pack=False, align=None), offset=0), "name": SimTypePointer(SimStruct({"nameType": SimTypeInt(signed=False, label="IKEEXT_CERT_CRITERIA_NAME_TYPE"), "certName": SimTypePointer(SimTypeChar(label="Char"), offset=0)}, name="IKEEXT_CERT_NAME0", pack=False, align=None), offset=0), "flags": SimTypeInt(signed=False, label="UInt32")}, name="IKEEXT_CERTIFICATE_CRITERIA0", pack=False, align=None), offset=0)}, name="_Anonymous3_e__Struct", pack=False, align=None)}, name="<anon>", label="None"), "outboundConfigType": SimTypeInt(signed=False, label="IKEEXT_CERT_CONFIG_TYPE"), "Anonymous2": SimUnion({"Anonymous1": SimStruct({"outboundRootArraySize": SimTypeInt(signed=False, label="UInt32"), "outboundRootCriteria": SimTypePointer(SimStruct({"certData": SimStruct({"size": SimTypeInt(signed=False, label="UInt32"), "data": SimTypePointer(SimTypeChar(label="Byte"), offset=0)}, name="FWP_BYTE_BLOB", pack=False, align=None), "certHash": SimStruct({"size": SimTypeInt(signed=False, label="UInt32"), "data": SimTypePointer(SimTypeChar(label="Byte"), offset=0)}, name="FWP_BYTE_BLOB", pack=False, align=None), "eku": SimTypePointer(SimStruct({"numEku": SimTypeInt(signed=False, label="UInt32"), "eku": SimTypePointer(SimTypePointer(SimTypeChar(label="Byte"), offset=0), offset=0)}, name="IKEEXT_CERT_EKUS0", pack=False, align=None), offset=0), "name": SimTypePointer(SimStruct({"nameType": SimTypeInt(signed=False, label="IKEEXT_CERT_CRITERIA_NAME_TYPE"), "certName": SimTypePointer(SimTypeChar(label="Char"), offset=0)}, name="IKEEXT_CERT_NAME0", pack=False, align=None), offset=0), "flags": SimTypeInt(signed=False, label="UInt32")}, name="IKEEXT_CERTIFICATE_CRITERIA0", pack=False, align=None), offset=0)}, name="_Anonymous1_e__Struct", pack=False, align=None), "Anonymous2": SimStruct({"outboundEnterpriseStoreArraySize": SimTypeInt(signed=False, label="UInt32"), "outboundEnterpriseStoreCriteria": SimTypePointer(SimStruct({"certData": SimStruct({"size": SimTypeInt(signed=False, label="UInt32"), "data": SimTypePointer(SimTypeChar(label="Byte"), offset=0)}, name="FWP_BYTE_BLOB", pack=False, align=None), "certHash": SimStruct({"size": SimTypeInt(signed=False, label="UInt32"), "data": SimTypePointer(SimTypeChar(label="Byte"), offset=0)}, name="FWP_BYTE_BLOB", pack=False, align=None), "eku": SimTypePointer(SimStruct({"numEku": SimTypeInt(signed=False, label="UInt32"), "eku": SimTypePointer(SimTypePointer(SimTypeChar(label="Byte"), offset=0), offset=0)}, name="IKEEXT_CERT_EKUS0", pack=False, align=None), offset=0), "name": SimTypePointer(SimStruct({"nameType": SimTypeInt(signed=False, label="IKEEXT_CERT_CRITERIA_NAME_TYPE"), "certName": SimTypePointer(SimTypeChar(label="Char"), offset=0)}, name="IKEEXT_CERT_NAME0", pack=False, align=None), offset=0), "flags": SimTypeInt(signed=False, label="UInt32")}, name="IKEEXT_CERTIFICATE_CRITERIA0", pack=False, align=None), offset=0)}, name="_Anonymous2_e__Struct", pack=False, align=None), "Anonymous3": SimStruct({"outboundRootStoreArraySize": SimTypeInt(signed=False, label="UInt32"), "outboundTrustedRootStoreCriteria": SimTypePointer(SimStruct({"certData": SimStruct({"size": SimTypeInt(signed=False, label="UInt32"), "data": SimTypePointer(SimTypeChar(label="Byte"), offset=0)}, name="FWP_BYTE_BLOB", pack=False, align=None), "certHash": SimStruct({"size": SimTypeInt(signed=False, label="UInt32"), "data": SimTypePointer(SimTypeChar(label="Byte"), offset=0)}, name="FWP_BYTE_BLOB", pack=False, align=None), "eku": SimTypePointer(SimStruct({"numEku": SimTypeInt(signed=False, label="UInt32"), "eku": SimTypePointer(SimTypePointer(SimTypeChar(label="Byte"), offset=0), offset=0)}, name="IKEEXT_CERT_EKUS0", pack=False, align=None), offset=0), "name": SimTypePointer(SimStruct({"nameType": SimTypeInt(signed=False, label="IKEEXT_CERT_CRITERIA_NAME_TYPE"), "certName": SimTypePointer(SimTypeChar(label="Char"), offset=0)}, name="IKEEXT_CERT_NAME0", pack=False, align=None), offset=0), "flags": SimTypeInt(signed=False, label="UInt32")}, name="IKEEXT_CERTIFICATE_CRITERIA0", pack=False, align=None), offset=0)}, name="_Anonymous3_e__Struct", pack=False, align=None)}, name="<anon>", label="None"), "flags": SimTypeInt(signed=False, label="IKEEXT_CERT_AUTH"), "localCertLocationUrl": SimStruct({"size": SimTypeInt(signed=False, label="UInt32"), "data": SimTypePointer(SimTypeChar(label="Byte"), offset=0)}, name="FWP_BYTE_BLOB", pack=False, align=None)}, name="IKEEXT_CERTIFICATE_AUTHENTICATION2", pack=False, align=None), "kerberosAuthentication": SimStruct({"flags": SimTypeInt(signed=False, label="IKEEXT_KERBEROS_AUTHENTICATION_FLAGS"), "proxyServer": SimTypePointer(SimTypeChar(label="Char"), offset=0)}, name="IKEEXT_KERBEROS_AUTHENTICATION1", pack=False, align=None), "reservedAuthentication": SimStruct({"flags": SimTypeInt(signed=False, label="IKEEXT_RESERVED_AUTHENTICATION_FLAGS")}, name="IKEEXT_RESERVED_AUTHENTICATION0", pack=False, align=None), "ntlmV2Authentication": SimStruct({"flags": SimTypeInt(signed=False, label="UInt32")}, name="IKEEXT_NTLM_V2_AUTHENTICATION0", pack=False, align=None), "sslAuthentication": SimStruct({"inboundConfigType": SimTypeInt(signed=False, label="IKEEXT_CERT_CONFIG_TYPE"), "Anonymous1": SimUnion({"Anonymous1": SimStruct({"inboundRootArraySize": SimTypeInt(signed=False, label="UInt32"), "inboundRootCriteria": SimTypePointer(SimStruct({"certData": SimStruct({"size": SimTypeInt(signed=False, label="UInt32"), "data": SimTypePointer(SimTypeChar(label="Byte"), offset=0)}, name="FWP_BYTE_BLOB", pack=False, align=None), "certHash": SimStruct({"size": SimTypeInt(signed=False, label="UInt32"), "data": SimTypePointer(SimTypeChar(label="Byte"), offset=0)}, name="FWP_BYTE_BLOB", pack=False, align=None), "eku": SimTypePointer(SimStruct({"numEku": SimTypeInt(signed=False, label="UInt32"), "eku": SimTypePointer(SimTypePointer(SimTypeChar(label="Byte"), offset=0), offset=0)}, name="IKEEXT_CERT_EKUS0", pack=False, align=None), offset=0), "name": SimTypePointer(SimStruct({"nameType": SimTypeInt(signed=False, label="IKEEXT_CERT_CRITERIA_NAME_TYPE"), "certName": SimTypePointer(SimTypeChar(label="Char"), offset=0)}, name="IKEEXT_CERT_NAME0", pack=False, align=None), offset=0), "flags": SimTypeInt(signed=False, label="UInt32")}, name="IKEEXT_CERTIFICATE_CRITERIA0", pack=False, align=None), offset=0)}, name="_Anonymous1_e__Struct", pack=False, align=None), "Anonymous2": SimStruct({"inboundEnterpriseStoreArraySize": SimTypeInt(signed=False, label="UInt32"), "inboundEnterpriseStoreCriteria": SimTypePointer(SimStruct({"certData": SimStruct({"size": SimTypeInt(signed=False, label="UInt32"), "data": SimTypePointer(SimTypeChar(label="Byte"), offset=0)}, name="FWP_BYTE_BLOB", pack=False, align=None), "certHash": SimStruct({"size": SimTypeInt(signed=False, label="UInt32"), "data": SimTypePointer(SimTypeChar(label="Byte"), offset=0)}, name="FWP_BYTE_BLOB", pack=False, align=None), "eku": SimTypePointer(SimStruct({"numEku": SimTypeInt(signed=False, label="UInt32"), "eku": SimTypePointer(SimTypePointer(SimTypeChar(label="Byte"), offset=0), offset=0)}, name="IKEEXT_CERT_EKUS0", pack=False, align=None), offset=0), "name": SimTypePointer(SimStruct({"nameType": SimTypeInt(signed=False, label="IKEEXT_CERT_CRITERIA_NAME_TYPE"), "certName": SimTypePointer(SimTypeChar(label="Char"), offset=0)}, name="IKEEXT_CERT_NAME0", pack=False, align=None), offset=0), "flags": SimTypeInt(signed=False, label="UInt32")}, name="IKEEXT_CERTIFICATE_CRITERIA0", pack=False, align=None), offset=0)}, name="_Anonymous2_e__Struct", pack=False, align=None), "Anonymous3": SimStruct({"inboundRootStoreArraySize": SimTypeInt(signed=False, label="UInt32"), "inboundTrustedRootStoreCriteria": SimTypePointer(SimStruct({"certData": SimStruct({"size": SimTypeInt(signed=False, label="UInt32"), "data": SimTypePointer(SimTypeChar(label="Byte"), offset=0)}, name="FWP_BYTE_BLOB", pack=False, align=None), "certHash": SimStruct({"size": SimTypeInt(signed=False, label="UInt32"), "data": SimTypePointer(SimTypeChar(label="Byte"), offset=0)}, name="FWP_BYTE_BLOB", pack=False, align=None), "eku": SimTypePointer(SimStruct({"numEku": SimTypeInt(signed=False, label="UInt32"), "eku": SimTypePointer(SimTypePointer(SimTypeChar(label="Byte"), offset=0), offset=0)}, name="IKEEXT_CERT_EKUS0", pack=False, align=None), offset=0), "name": SimTypePointer(SimStruct({"nameType": SimTypeInt(signed=False, label="IKEEXT_CERT_CRITERIA_NAME_TYPE"), "certName": SimTypePointer(SimTypeChar(label="Char"), offset=0)}, name="IKEEXT_CERT_NAME0", pack=False, align=None), offset=0), "flags": SimTypeInt(signed=False, label="UInt32")}, name="IKEEXT_CERTIFICATE_CRITERIA0", pack=False, align=None), offset=0)}, name="_Anonymous3_e__Struct", pack=False, align=None)}, name="<anon>", label="None"), "outboundConfigType": SimTypeInt(signed=False, label="IKEEXT_CERT_CONFIG_TYPE"), "Anonymous2": SimUnion({"Anonymous1": SimStruct({"outboundRootArraySize": SimTypeInt(signed=False, label="UInt32"), "outboundRootCriteria": SimTypePointer(SimStruct({"certData": SimStruct({"size": SimTypeInt(signed=False, label="UInt32"), "data": SimTypePointer(SimTypeChar(label="Byte"), offset=0)}, name="FWP_BYTE_BLOB", pack=False, align=None), "certHash": SimStruct({"size": SimTypeInt(signed=False, label="UInt32"), "data": SimTypePointer(SimTypeChar(label="Byte"), offset=0)}, name="FWP_BYTE_BLOB", pack=False, align=None), "eku": SimTypePointer(SimStruct({"numEku": SimTypeInt(signed=False, label="UInt32"), "eku": SimTypePointer(SimTypePointer(SimTypeChar(label="Byte"), offset=0), offset=0)}, name="IKEEXT_CERT_EKUS0", pack=False, align=None), offset=0), "name": SimTypePointer(SimStruct({"nameType": SimTypeInt(signed=False, label="IKEEXT_CERT_CRITERIA_NAME_TYPE"), "certName": SimTypePointer(SimTypeChar(label="Char"), offset=0)}, name="IKEEXT_CERT_NAME0", pack=False, align=None), offset=0), "flags": SimTypeInt(signed=False, label="UInt32")}, name="IKEEXT_CERTIFICATE_CRITERIA0", pack=False, align=None), offset=0)}, name="_Anonymous1_e__Struct", pack=False, align=None), "Anonymous2": SimStruct({"outboundEnterpriseStoreArraySize": SimTypeInt(signed=False, label="UInt32"), "outboundEnterpriseStoreCriteria": SimTypePointer(SimStruct({"certData": SimStruct({"size": SimTypeInt(signed=False, label="UInt32"), "data": SimTypePointer(SimTypeChar(label="Byte"), offset=0)}, name="FWP_BYTE_BLOB", pack=False, align=None), "certHash": SimStruct({"size": SimTypeInt(signed=False, label="UInt32"), "data": SimTypePointer(SimTypeChar(label="Byte"), offset=0)}, name="FWP_BYTE_BLOB", pack=False, align=None), "eku": SimTypePointer(SimStruct({"numEku": SimTypeInt(signed=False, label="UInt32"), "eku": SimTypePointer(SimTypePointer(SimTypeChar(label="Byte"), offset=0), offset=0)}, name="IKEEXT_CERT_EKUS0", pack=False, align=None), offset=0), "name": SimTypePointer(SimStruct({"nameType": SimTypeInt(signed=False, label="IKEEXT_CERT_CRITERIA_NAME_TYPE"), "certName": SimTypePointer(SimTypeChar(label="Char"), offset=0)}, name="IKEEXT_CERT_NAME0", pack=False, align=None), offset=0), "flags": SimTypeInt(signed=False, label="UInt32")}, name="IKEEXT_CERTIFICATE_CRITERIA0", pack=False, align=None), offset=0)}, name="_Anonymous2_e__Struct", pack=False, align=None), "Anonymous3": SimStruct({"outboundRootStoreArraySize": SimTypeInt(signed=False, label="UInt32"), "outboundTrustedRootStoreCriteria": SimTypePointer(SimStruct({"certData": SimStruct({"size": SimTypeInt(signed=False, label="UInt32"), "data": SimTypePointer(SimTypeChar(label="Byte"), offset=0)}, name="FWP_BYTE_BLOB", pack=False, align=None), "certHash": SimStruct({"size": SimTypeInt(signed=False, label="UInt32"), "data": SimTypePointer(SimTypeChar(label="Byte"), offset=0)}, name="FWP_BYTE_BLOB", pack=False, align=None), "eku": SimTypePointer(SimStruct({"numEku": SimTypeInt(signed=False, label="UInt32"), "eku": SimTypePointer(SimTypePointer(SimTypeChar(label="Byte"), offset=0), offset=0)}, name="IKEEXT_CERT_EKUS0", pack=False, align=None), offset=0), "name": SimTypePointer(SimStruct({"nameType": SimTypeInt(signed=False, label="IKEEXT_CERT_CRITERIA_NAME_TYPE"), "certName": SimTypePointer(SimTypeChar(label="Char"), offset=0)}, name="IKEEXT_CERT_NAME0", pack=False, align=None), offset=0), "flags": SimTypeInt(signed=False, label="UInt32")}, name="IKEEXT_CERTIFICATE_CRITERIA0", pack=False, align=None), offset=0)}, name="_Anonymous3_e__Struct", pack=False, align=None)}, name="<anon>", label="None"), "flags": SimTypeInt(signed=False, label="IKEEXT_CERT_AUTH"), "localCertLocationUrl": SimStruct({"size": SimTypeInt(signed=False, label="UInt32"), "data": SimTypePointer(SimTypeChar(label="Byte"), offset=0)}, name="FWP_BYTE_BLOB", pack=False, align=None)}, name="IKEEXT_CERTIFICATE_AUTHENTICATION2", pack=False, align=None), "cgaAuthentication": SimStruct({"keyContainerName": SimTypePointer(SimTypeChar(label="Char"), offset=0), "cspName": SimTypePointer(SimTypeChar(label="Char"), offset=0), "cspType": SimTypeInt(signed=False, label="UInt32"), "cgaModifier": SimStruct({"byteArray16": SimTypeFixedSizeArray(SimTypeChar(label="Byte"), 16)}, name="FWP_BYTE_ARRAY16", pack=False, align=None), "cgaCollisionCount": SimTypeChar(label="Byte")}, name="IKEEXT_IPV6_CGA_AUTHENTICATION0", pack=False, align=None), "eapAuthentication": SimStruct({"flags": SimTypeInt(signed=False, label="IKEEXT_EAP_AUTHENTICATION_FLAGS")}, name="IKEEXT_EAP_AUTHENTICATION0", pack=False, align=None)}, name="<anon>", label="None")}, name="IKEEXT_AUTHENTICATION_METHOD2", pack=False, align=None), offset=0), "initiatorImpersonationType": SimTypeInt(signed=False, label="IKEEXT_AUTHENTICATION_IMPERSONATION_TYPE")}, name="IKEEXT_EM_POLICY2", pack=False, align=None), offset=0)}, name="IPSEC_TRANSPORT_POLICY2", pack=False, align=None), offset=0), "ikeQmTunnelPolicy": SimTypePointer(SimStruct({"flags": SimTypeInt(signed=False, label="IPSEC_POLICY_FLAG"), "numIpsecProposals": SimTypeInt(signed=False, label="UInt32"), "ipsecProposals": SimTypePointer(SimStruct({"lifetime": SimStruct({"lifetimeSeconds": SimTypeInt(signed=False, label="UInt32"), "lifetimeKilobytes": SimTypeInt(signed=False, label="UInt32"), "lifetimePackets": SimTypeInt(signed=False, label="UInt32")}, name="IPSEC_SA_LIFETIME0", pack=False, align=None), "numSaTransforms": SimTypeInt(signed=False, label="UInt32"), "saTransforms": SimTypePointer(SimStruct({"ipsecTransformType": SimTypeInt(signed=False, label="IPSEC_TRANSFORM_TYPE"), "Anonymous": SimUnion({"ahTransform": SimTypePointer(SimStruct({"authTransformId": SimStruct({"authType": SimTypeInt(signed=False, label="IPSEC_AUTH_TYPE"), "authConfig": SimTypeChar(label="Byte")}, name="IPSEC_AUTH_TRANSFORM_ID0", pack=False, align=None), "cryptoModuleId": SimTypePointer(SimTypeBottom(label="Guid"), offset=0)}, name="IPSEC_AUTH_TRANSFORM0", pack=False, align=None), offset=0), "espAuthTransform": SimTypePointer(SimStruct({"authTransformId": SimStruct({"authType": SimTypeInt(signed=False, label="IPSEC_AUTH_TYPE"), "authConfig": SimTypeChar(label="Byte")}, name="IPSEC_AUTH_TRANSFORM_ID0", pack=False, align=None), "cryptoModuleId": SimTypePointer(SimTypeBottom(label="Guid"), offset=0)}, name="IPSEC_AUTH_TRANSFORM0", pack=False, align=None), offset=0), "espCipherTransform": SimTypePointer(SimStruct({"cipherTransformId": SimStruct({"cipherType": SimTypeInt(signed=False, label="IPSEC_CIPHER_TYPE"), "cipherConfig": SimTypeChar(label="Byte")}, name="IPSEC_CIPHER_TRANSFORM_ID0", pack=False, align=None), "cryptoModuleId": SimTypePointer(SimTypeBottom(label="Guid"), offset=0)}, name="IPSEC_CIPHER_TRANSFORM0", pack=False, align=None), offset=0), "espAuthAndCipherTransform": SimTypePointer(SimStruct({"authTransform": SimStruct({"authTransformId": SimStruct({"authType": SimTypeInt(signed=False, label="IPSEC_AUTH_TYPE"), "authConfig": SimTypeChar(label="Byte")}, name="IPSEC_AUTH_TRANSFORM_ID0", pack=False, align=None), "cryptoModuleId": SimTypePointer(SimTypeBottom(label="Guid"), offset=0)}, name="IPSEC_AUTH_TRANSFORM0", pack=False, align=None), "cipherTransform": SimStruct({"cipherTransformId": SimStruct({"cipherType": SimTypeInt(signed=False, label="IPSEC_CIPHER_TYPE"), "cipherConfig": SimTypeChar(label="Byte")}, name="IPSEC_CIPHER_TRANSFORM_ID0", pack=False, align=None), "cryptoModuleId": SimTypePointer(SimTypeBottom(label="Guid"), offset=0)}, name="IPSEC_CIPHER_TRANSFORM0", pack=False, align=None)}, name="IPSEC_AUTH_AND_CIPHER_TRANSFORM0", pack=False, align=None), offset=0), "espAuthFwTransform": SimTypePointer(SimStruct({"authTransformId": SimStruct({"authType": SimTypeInt(signed=False, label="IPSEC_AUTH_TYPE"), "authConfig": SimTypeChar(label="Byte")}, name="IPSEC_AUTH_TRANSFORM_ID0", pack=False, align=None), "cryptoModuleId": SimTypePointer(SimTypeBottom(label="Guid"), offset=0)}, name="IPSEC_AUTH_TRANSFORM0", pack=False, align=None), offset=0)}, name="<anon>", label="None")}, name="IPSEC_SA_TRANSFORM0", pack=False, align=None), offset=0), "pfsGroup": SimTypeInt(signed=False, label="IPSEC_PFS_GROUP")}, name="IPSEC_PROPOSAL0", pack=False, align=None), offset=0), "tunnelEndpoints": SimStruct({"ipVersion": SimTypeInt(signed=False, label="FWP_IP_VERSION"), "Anonymous1": SimUnion({"localV4Address": SimTypeInt(signed=False, label="UInt32"), "localV6Address": SimTypeFixedSizeArray(SimTypeChar(label="Byte"), 16)}, name="<anon>", label="None"), "Anonymous2": SimUnion({"remoteV4Address": SimTypeInt(signed=False, label="UInt32"), "remoteV6Address": SimTypeFixedSizeArray(SimTypeChar(label="Byte"), 16)}, name="<anon>", label="None"), "localIfLuid": SimTypeLongLong(signed=False, label="UInt64"), "remoteFqdn": SimTypePointer(SimTypeChar(label="Char"), offset=0), "numAddresses": SimTypeInt(signed=False, label="UInt32"), "remoteAddresses": SimTypePointer(SimStruct({"ipVersion": SimTypeInt(signed=False, label="FWP_IP_VERSION"), "Anonymous": SimUnion({"v4Address": SimTypeInt(signed=False, label="UInt32"), "v6Address": SimTypeFixedSizeArray(SimTypeChar(label="Byte"), 16)}, name="<anon>", label="None")}, name="IPSEC_TUNNEL_ENDPOINT0", pack=False, align=None), offset=0)}, name="IPSEC_TUNNEL_ENDPOINTS2", pack=False, align=None), "saIdleTimeout": SimStruct({"idleTimeoutSeconds": SimTypeInt(signed=False, label="UInt32"), "idleTimeoutSecondsFailOver": SimTypeInt(signed=False, label="UInt32")}, name="IPSEC_SA_IDLE_TIMEOUT0", pack=False, align=None), "emPolicy": SimTypePointer(SimStruct({"numAuthenticationMethods": SimTypeInt(signed=False, label="UInt32"), "authenticationMethods": SimTypePointer(SimStruct({"authenticationMethodType": SimTypeInt(signed=False, label="IKEEXT_AUTHENTICATION_METHOD_TYPE"), "Anonymous": SimUnion({"presharedKeyAuthentication": SimStruct({"presharedKey": SimStruct({"size": SimTypeInt(signed=False, label="UInt32"), "data": SimTypePointer(SimTypeChar(label="Byte"), offset=0)}, name="FWP_BYTE_BLOB", pack=False, align=None), "flags": SimTypeInt(signed=False, label="IKEEXT_PRESHARED_KEY_AUTHENTICATION_FLAGS")}, name="IKEEXT_PRESHARED_KEY_AUTHENTICATION1", pack=False, align=None), "certificateAuthentication": SimStruct({"inboundConfigType": SimTypeInt(signed=False, label="IKEEXT_CERT_CONFIG_TYPE"), "Anonymous1": SimUnion({"Anonymous1": SimStruct({"inboundRootArraySize": SimTypeInt(signed=False, label="UInt32"), "inboundRootCriteria": SimTypePointer(SimStruct({"certData": SimStruct({"size": SimTypeInt(signed=False, label="UInt32"), "data": SimTypePointer(SimTypeChar(label="Byte"), offset=0)}, name="FWP_BYTE_BLOB", pack=False, align=None), "certHash": SimStruct({"size": SimTypeInt(signed=False, label="UInt32"), "data": SimTypePointer(SimTypeChar(label="Byte"), offset=0)}, name="FWP_BYTE_BLOB", pack=False, align=None), "eku": SimTypePointer(SimStruct({"numEku": SimTypeInt(signed=False, label="UInt32"), "eku": SimTypePointer(SimTypePointer(SimTypeChar(label="Byte"), offset=0), offset=0)}, name="IKEEXT_CERT_EKUS0", pack=False, align=None), offset=0), "name": SimTypePointer(SimStruct({"nameType": SimTypeInt(signed=False, label="IKEEXT_CERT_CRITERIA_NAME_TYPE"), "certName": SimTypePointer(SimTypeChar(label="Char"), offset=0)}, name="IKEEXT_CERT_NAME0", pack=False, align=None), offset=0), "flags": SimTypeInt(signed=False, label="UInt32")}, name="IKEEXT_CERTIFICATE_CRITERIA0", pack=False, align=None), offset=0)}, name="_Anonymous1_e__Struct", pack=False, align=None), "Anonymous2": SimStruct({"inboundEnterpriseStoreArraySize": SimTypeInt(signed=False, label="UInt32"), "inboundEnterpriseStoreCriteria": SimTypePointer(SimStruct({"certData": SimStruct({"size": SimTypeInt(signed=False, label="UInt32"), "data": SimTypePointer(SimTypeChar(label="Byte"), offset=0)}, name="FWP_BYTE_BLOB", pack=False, align=None), "certHash": SimStruct({"size": SimTypeInt(signed=False, label="UInt32"), "data": SimTypePointer(SimTypeChar(label="Byte"), offset=0)}, name="FWP_BYTE_BLOB", pack=False, align=None), "eku": SimTypePointer(SimStruct({"numEku": SimTypeInt(signed=False, label="UInt32"), "eku": SimTypePointer(SimTypePointer(SimTypeChar(label="Byte"), offset=0), offset=0)}, name="IKEEXT_CERT_EKUS0", pack=False, align=None), offset=0), "name": SimTypePointer(SimStruct({"nameType": SimTypeInt(signed=False, label="IKEEXT_CERT_CRITERIA_NAME_TYPE"), "certName": SimTypePointer(SimTypeChar(label="Char"), offset=0)}, name="IKEEXT_CERT_NAME0", pack=False, align=None), offset=0), "flags": SimTypeInt(signed=False, label="UInt32")}, name="IKEEXT_CERTIFICATE_CRITERIA0", pack=False, align=None), offset=0)}, name="_Anonymous2_e__Struct", pack=False, align=None), "Anonymous3": SimStruct({"inboundRootStoreArraySize": SimTypeInt(signed=False, label="UInt32"), "inboundTrustedRootStoreCriteria": SimTypePointer(SimStruct({"certData": SimStruct({"size": SimTypeInt(signed=False, label="UInt32"), "data": SimTypePointer(SimTypeChar(label="Byte"), offset=0)}, name="FWP_BYTE_BLOB", pack=False, align=None), "certHash": SimStruct({"size": SimTypeInt(signed=False, label="UInt32"), "data": SimTypePointer(SimTypeChar(label="Byte"), offset=0)}, name="FWP_BYTE_BLOB", pack=False, align=None), "eku": SimTypePointer(SimStruct({"numEku": SimTypeInt(signed=False, label="UInt32"), "eku": SimTypePointer(SimTypePointer(SimTypeChar(label="Byte"), offset=0), offset=0)}, name="IKEEXT_CERT_EKUS0", pack=False, align=None), offset=0), "name": SimTypePointer(SimStruct({"nameType": SimTypeInt(signed=False, label="IKEEXT_CERT_CRITERIA_NAME_TYPE"), "certName": SimTypePointer(SimTypeChar(label="Char"), offset=0)}, name="IKEEXT_CERT_NAME0", pack=False, align=None), offset=0), "flags": SimTypeInt(signed=False, label="UInt32")}, name="IKEEXT_CERTIFICATE_CRITERIA0", pack=False, align=None), offset=0)}, name="_Anonymous3_e__Struct", pack=False, align=None)}, name="<anon>", label="None"), "outboundConfigType": SimTypeInt(signed=False, label="IKEEXT_CERT_CONFIG_TYPE"), "Anonymous2": SimUnion({"Anonymous1": SimStruct({"outboundRootArraySize": SimTypeInt(signed=False, label="UInt32"), "outboundRootCriteria": SimTypePointer(SimStruct({"certData": SimStruct({"size": SimTypeInt(signed=False, label="UInt32"), "data": SimTypePointer(SimTypeChar(label="Byte"), offset=0)}, name="FWP_BYTE_BLOB", pack=False, align=None), "certHash": SimStruct({"size": SimTypeInt(signed=False, label="UInt32"), "data": SimTypePointer(SimTypeChar(label="Byte"), offset=0)}, name="FWP_BYTE_BLOB", pack=False, align=None), "eku": SimTypePointer(SimStruct({"numEku": SimTypeInt(signed=False, label="UInt32"), "eku": SimTypePointer(SimTypePointer(SimTypeChar(label="Byte"), offset=0), offset=0)}, name="IKEEXT_CERT_EKUS0", pack=False, align=None), offset=0), "name": SimTypePointer(SimStruct({"nameType": SimTypeInt(signed=False, label="IKEEXT_CERT_CRITERIA_NAME_TYPE"), "certName": SimTypePointer(SimTypeChar(label="Char"), offset=0)}, name="IKEEXT_CERT_NAME0", pack=False, align=None), offset=0), "flags": SimTypeInt(signed=False, label="UInt32")}, name="IKEEXT_CERTIFICATE_CRITERIA0", pack=False, align=None), offset=0)}, name="_Anonymous1_e__Struct", pack=False, align=None), "Anonymous2": SimStruct({"outboundEnterpriseStoreArraySize": SimTypeInt(signed=False, label="UInt32"), "outboundEnterpriseStoreCriteria": SimTypePointer(SimStruct({"certData": SimStruct({"size": SimTypeInt(signed=False, label="UInt32"), "data": SimTypePointer(SimTypeChar(label="Byte"), offset=0)}, name="FWP_BYTE_BLOB", pack=False, align=None), "certHash": SimStruct({"size": SimTypeInt(signed=False, label="UInt32"), "data": SimTypePointer(SimTypeChar(label="Byte"), offset=0)}, name="FWP_BYTE_BLOB", pack=False, align=None), "eku": SimTypePointer(SimStruct({"numEku": SimTypeInt(signed=False, label="UInt32"), "eku": SimTypePointer(SimTypePointer(SimTypeChar(label="Byte"), offset=0), offset=0)}, name="IKEEXT_CERT_EKUS0", pack=False, align=None), offset=0), "name": SimTypePointer(SimStruct({"nameType": SimTypeInt(signed=False, label="IKEEXT_CERT_CRITERIA_NAME_TYPE"), "certName": SimTypePointer(SimTypeChar(label="Char"), offset=0)}, name="IKEEXT_CERT_NAME0", pack=False, align=None), offset=0), "flags": SimTypeInt(signed=False, label="UInt32")}, name="IKEEXT_CERTIFICATE_CRITERIA0", pack=False, align=None), offset=0)}, name="_Anonymous2_e__Struct", pack=False, align=None), "Anonymous3": SimStruct({"outboundRootStoreArraySize": SimTypeInt(signed=False, label="UInt32"), "outboundTrustedRootStoreCriteria": SimTypePointer(SimStruct({"certData": SimStruct({"size": SimTypeInt(signed=False, label="UInt32"), "data": SimTypePointer(SimTypeChar(label="Byte"), offset=0)}, name="FWP_BYTE_BLOB", pack=False, align=None), "certHash": SimStruct({"size": SimTypeInt(signed=False, label="UInt32"), "data": SimTypePointer(SimTypeChar(label="Byte"), offset=0)}, name="FWP_BYTE_BLOB", pack=False, align=None), "eku": SimTypePointer(SimStruct({"numEku": SimTypeInt(signed=False, label="UInt32"), "eku": SimTypePointer(SimTypePointer(SimTypeChar(label="Byte"), offset=0), offset=0)}, name="IKEEXT_CERT_EKUS0", pack=False, align=None), offset=0), "name": SimTypePointer(SimStruct({"nameType": SimTypeInt(signed=False, label="IKEEXT_CERT_CRITERIA_NAME_TYPE"), "certName": SimTypePointer(SimTypeChar(label="Char"), offset=0)}, name="IKEEXT_CERT_NAME0", pack=False, align=None), offset=0), "flags": SimTypeInt(signed=False, label="UInt32")}, name="IKEEXT_CERTIFICATE_CRITERIA0", pack=False, align=None), offset=0)}, name="_Anonymous3_e__Struct", pack=False, align=None)}, name="<anon>", label="None"), "flags": SimTypeInt(signed=False, label="IKEEXT_CERT_AUTH"), "localCertLocationUrl": SimStruct({"size": SimTypeInt(signed=False, label="UInt32"), "data": SimTypePointer(SimTypeChar(label="Byte"), offset=0)}, name="FWP_BYTE_BLOB", pack=False, align=None)}, name="IKEEXT_CERTIFICATE_AUTHENTICATION2", pack=False, align=None), "kerberosAuthentication": SimStruct({"flags": SimTypeInt(signed=False, label="IKEEXT_KERBEROS_AUTHENTICATION_FLAGS"), "proxyServer": SimTypePointer(SimTypeChar(label="Char"), offset=0)}, name="IKEEXT_KERBEROS_AUTHENTICATION1", pack=False, align=None), "reservedAuthentication": SimStruct({"flags": SimTypeInt(signed=False, label="IKEEXT_RESERVED_AUTHENTICATION_FLAGS")}, name="IKEEXT_RESERVED_AUTHENTICATION0", pack=False, align=None), "ntlmV2Authentication": SimStruct({"flags": SimTypeInt(signed=False, label="UInt32")}, name="IKEEXT_NTLM_V2_AUTHENTICATION0", pack=False, align=None), "sslAuthentication": SimStruct({"inboundConfigType": SimTypeInt(signed=False, label="IKEEXT_CERT_CONFIG_TYPE"), "Anonymous1": SimUnion({"Anonymous1": SimStruct({"inboundRootArraySize": SimTypeInt(signed=False, label="UInt32"), "inboundRootCriteria": SimTypePointer(SimStruct({"certData": SimStruct({"size": SimTypeInt(signed=False, label="UInt32"), "data": SimTypePointer(SimTypeChar(label="Byte"), offset=0)}, name="FWP_BYTE_BLOB", pack=False, align=None), "certHash": SimStruct({"size": SimTypeInt(signed=False, label="UInt32"), "data": SimTypePointer(SimTypeChar(label="Byte"), offset=0)}, name="FWP_BYTE_BLOB", pack=False, align=None), "eku": SimTypePointer(SimStruct({"numEku": SimTypeInt(signed=False, label="UInt32"), "eku": SimTypePointer(SimTypePointer(SimTypeChar(label="Byte"), offset=0), offset=0)}, name="IKEEXT_CERT_EKUS0", pack=False, align=None), offset=0), "name": SimTypePointer(SimStruct({"nameType": SimTypeInt(signed=False, label="IKEEXT_CERT_CRITERIA_NAME_TYPE"), "certName": SimTypePointer(SimTypeChar(label="Char"), offset=0)}, name="IKEEXT_CERT_NAME0", pack=False, align=None), offset=0), "flags": SimTypeInt(signed=False, label="UInt32")}, name="IKEEXT_CERTIFICATE_CRITERIA0", pack=False, align=None), offset=0)}, name="_Anonymous1_e__Struct", pack=False, align=None), "Anonymous2": SimStruct({"inboundEnterpriseStoreArraySize": SimTypeInt(signed=False, label="UInt32"), "inboundEnterpriseStoreCriteria": SimTypePointer(SimStruct({"certData": SimStruct({"size": SimTypeInt(signed=False, label="UInt32"), "data": SimTypePointer(SimTypeChar(label="Byte"), offset=0)}, name="FWP_BYTE_BLOB", pack=False, align=None), "certHash": SimStruct({"size": SimTypeInt(signed=False, label="UInt32"), "data": SimTypePointer(SimTypeChar(label="Byte"), offset=0)}, name="FWP_BYTE_BLOB", pack=False, align=None), "eku": SimTypePointer(SimStruct({"numEku": SimTypeInt(signed=False, label="UInt32"), "eku": SimTypePointer(SimTypePointer(SimTypeChar(label="Byte"), offset=0), offset=0)}, name="IKEEXT_CERT_EKUS0", pack=False, align=None), offset=0), "name": SimTypePointer(SimStruct({"nameType": SimTypeInt(signed=False, label="IKEEXT_CERT_CRITERIA_NAME_TYPE"), "certName": SimTypePointer(SimTypeChar(label="Char"), offset=0)}, name="IKEEXT_CERT_NAME0", pack=False, align=None), offset=0), "flags": SimTypeInt(signed=False, label="UInt32")}, name="IKEEXT_CERTIFICATE_CRITERIA0", pack=False, align=None), offset=0)}, name="_Anonymous2_e__Struct", pack=False, align=None), "Anonymous3": SimStruct({"inboundRootStoreArraySize": SimTypeInt(signed=False, label="UInt32"), "inboundTrustedRootStoreCriteria": SimTypePointer(SimStruct({"certData": SimStruct({"size": SimTypeInt(signed=False, label="UInt32"), "data": SimTypePointer(SimTypeChar(label="Byte"), offset=0)}, name="FWP_BYTE_BLOB", pack=False, align=None), "certHash": SimStruct({"size": SimTypeInt(signed=False, label="UInt32"), "data": SimTypePointer(SimTypeChar(label="Byte"), offset=0)}, name="FWP_BYTE_BLOB", pack=False, align=None), "eku": SimTypePointer(SimStruct({"numEku": SimTypeInt(signed=False, label="UInt32"), "eku": SimTypePointer(SimTypePointer(SimTypeChar(label="Byte"), offset=0), offset=0)}, name="IKEEXT_CERT_EKUS0", pack=False, align=None), offset=0), "name": SimTypePointer(SimStruct({"nameType": SimTypeInt(signed=False, label="IKEEXT_CERT_CRITERIA_NAME_TYPE"), "certName": SimTypePointer(SimTypeChar(label="Char"), offset=0)}, name="IKEEXT_CERT_NAME0", pack=False, align=None), offset=0), "flags": SimTypeInt(signed=False, label="UInt32")}, name="IKEEXT_CERTIFICATE_CRITERIA0", pack=False, align=None), offset=0)}, name="_Anonymous3_e__Struct", pack=False, align=None)}, name="<anon>", label="None"), "outboundConfigType": SimTypeInt(signed=False, label="IKEEXT_CERT_CONFIG_TYPE"), "Anonymous2": SimUnion({"Anonymous1": SimStruct({"outboundRootArraySize": SimTypeInt(signed=False, label="UInt32"), "outboundRootCriteria": SimTypePointer(SimStruct({"certData": SimStruct({"size": SimTypeInt(signed=False, label="UInt32"), "data": SimTypePointer(SimTypeChar(label="Byte"), offset=0)}, name="FWP_BYTE_BLOB", pack=False, align=None), "certHash": SimStruct({"size": SimTypeInt(signed=False, label="UInt32"), "data": SimTypePointer(SimTypeChar(label="Byte"), offset=0)}, name="FWP_BYTE_BLOB", pack=False, align=None), "eku": SimTypePointer(SimStruct({"numEku": SimTypeInt(signed=False, label="UInt32"), "eku": SimTypePointer(SimTypePointer(SimTypeChar(label="Byte"), offset=0), offset=0)}, name="IKEEXT_CERT_EKUS0", pack=False, align=None), offset=0), "name": SimTypePointer(SimStruct({"nameType": SimTypeInt(signed=False, label="IKEEXT_CERT_CRITERIA_NAME_TYPE"), "certName": SimTypePointer(SimTypeChar(label="Char"), offset=0)}, name="IKEEXT_CERT_NAME0", pack=False, align=None), offset=0), "flags": SimTypeInt(signed=False, label="UInt32")}, name="IKEEXT_CERTIFICATE_CRITERIA0", pack=False, align=None), offset=0)}, name="_Anonymous1_e__Struct", pack=False, align=None), "Anonymous2": SimStruct({"outboundEnterpriseStoreArraySize": SimTypeInt(signed=False, label="UInt32"), "outboundEnterpriseStoreCriteria": SimTypePointer(SimStruct({"certData": SimStruct({"size": SimTypeInt(signed=False, label="UInt32"), "data": SimTypePointer(SimTypeChar(label="Byte"), offset=0)}, name="FWP_BYTE_BLOB", pack=False, align=None), "certHash": SimStruct({"size": SimTypeInt(signed=False, label="UInt32"), "data": SimTypePointer(SimTypeChar(label="Byte"), offset=0)}, name="FWP_BYTE_BLOB", pack=False, align=None), "eku": SimTypePointer(SimStruct({"numEku": SimTypeInt(signed=False, label="UInt32"), "eku": SimTypePointer(SimTypePointer(SimTypeChar(label="Byte"), offset=0), offset=0)}, name="IKEEXT_CERT_EKUS0", pack=False, align=None), offset=0), "name": SimTypePointer(SimStruct({"nameType": SimTypeInt(signed=False, label="IKEEXT_CERT_CRITERIA_NAME_TYPE"), "certName": SimTypePointer(SimTypeChar(label="Char"), offset=0)}, name="IKEEXT_CERT_NAME0", pack=False, align=None), offset=0), "flags": SimTypeInt(signed=False, label="UInt32")}, name="IKEEXT_CERTIFICATE_CRITERIA0", pack=False, align=None), offset=0)}, name="_Anonymous2_e__Struct", pack=False, align=None), "Anonymous3": SimStruct({"outboundRootStoreArraySize": SimTypeInt(signed=False, label="UInt32"), "outboundTrustedRootStoreCriteria": SimTypePointer(SimStruct({"certData": SimStruct({"size": SimTypeInt(signed=False, label="UInt32"), "data": SimTypePointer(SimTypeChar(label="Byte"), offset=0)}, name="FWP_BYTE_BLOB", pack=False, align=None), "certHash": SimStruct({"size": SimTypeInt(signed=False, label="UInt32"), "data": SimTypePointer(SimTypeChar(label="Byte"), offset=0)}, name="FWP_BYTE_BLOB", pack=False, align=None), "eku": SimTypePointer(SimStruct({"numEku": SimTypeInt(signed=False, label="UInt32"), "eku": SimTypePointer(SimTypePointer(SimTypeChar(label="Byte"), offset=0), offset=0)}, name="IKEEXT_CERT_EKUS0", pack=False, align=None), offset=0), "name": SimTypePointer(SimStruct({"nameType": SimTypeInt(signed=False, label="IKEEXT_CERT_CRITERIA_NAME_TYPE"), "certName": SimTypePointer(SimTypeChar(label="Char"), offset=0)}, name="IKEEXT_CERT_NAME0", pack=False, align=None), offset=0), "flags": SimTypeInt(signed=False, label="UInt32")}, name="IKEEXT_CERTIFICATE_CRITERIA0", pack=False, align=None), offset=0)}, name="_Anonymous3_e__Struct", pack=False, align=None)}, name="<anon>", label="None"), "flags": SimTypeInt(signed=False, label="IKEEXT_CERT_AUTH"), "localCertLocationUrl": SimStruct({"size": SimTypeInt(signed=False, label="UInt32"), "data": SimTypePointer(SimTypeChar(label="Byte"), offset=0)}, name="FWP_BYTE_BLOB", pack=False, align=None)}, name="IKEEXT_CERTIFICATE_AUTHENTICATION2", pack=False, align=None), "cgaAuthentication": SimStruct({"keyContainerName": SimTypePointer(SimTypeChar(label="Char"), offset=0), "cspName": SimTypePointer(SimTypeChar(label="Char"), offset=0), "cspType": SimTypeInt(signed=False, label="UInt32"), "cgaModifier": SimStruct({"byteArray16": SimTypeFixedSizeArray(SimTypeChar(label="Byte"), 16)}, name="FWP_BYTE_ARRAY16", pack=False, align=None), "cgaCollisionCount": SimTypeChar(label="Byte")}, name="IKEEXT_IPV6_CGA_AUTHENTICATION0", pack=False, align=None), "eapAuthentication": SimStruct({"flags": SimTypeInt(signed=False, label="IKEEXT_EAP_AUTHENTICATION_FLAGS")}, name="IKEEXT_EAP_AUTHENTICATION0", pack=False, align=None)}, name="<anon>", label="None")}, name="IKEEXT_AUTHENTICATION_METHOD2", pack=False, align=None), offset=0), "initiatorImpersonationType": SimTypeInt(signed=False, label="IKEEXT_AUTHENTICATION_IMPERSONATION_TYPE")}, name="IKEEXT_EM_POLICY2", pack=False, align=None), offset=0), "fwdPathSaLifetime": SimTypeInt(signed=False, label="UInt32")}, name="IPSEC_TUNNEL_POLICY2", pack=False, align=None), offset=0), "authipQmTransportPolicy": SimTypePointer(SimStruct({"numIpsecProposals": SimTypeInt(signed=False, label="UInt32"), "ipsecProposals": SimTypePointer(SimStruct({"lifetime": SimStruct({"lifetimeSeconds": SimTypeInt(signed=False, label="UInt32"), "lifetimeKilobytes": SimTypeInt(signed=False, label="UInt32"), "lifetimePackets": SimTypeInt(signed=False, label="UInt32")}, name="IPSEC_SA_LIFETIME0", pack=False, align=None), "numSaTransforms": SimTypeInt(signed=False, label="UInt32"), "saTransforms": SimTypePointer(SimStruct({"ipsecTransformType": SimTypeInt(signed=False, label="IPSEC_TRANSFORM_TYPE"), "Anonymous": SimUnion({"ahTransform": SimTypePointer(SimStruct({"authTransformId": SimStruct({"authType": SimTypeInt(signed=False, label="IPSEC_AUTH_TYPE"), "authConfig": SimTypeChar(label="Byte")}, name="IPSEC_AUTH_TRANSFORM_ID0", pack=False, align=None), "cryptoModuleId": SimTypePointer(SimTypeBottom(label="Guid"), offset=0)}, name="IPSEC_AUTH_TRANSFORM0", pack=False, align=None), offset=0), "espAuthTransform": SimTypePointer(SimStruct({"authTransformId": SimStruct({"authType": SimTypeInt(signed=False, label="IPSEC_AUTH_TYPE"), "authConfig": SimTypeChar(label="Byte")}, name="IPSEC_AUTH_TRANSFORM_ID0", pack=False, align=None), "cryptoModuleId": SimTypePointer(SimTypeBottom(label="Guid"), offset=0)}, name="IPSEC_AUTH_TRANSFORM0", pack=False, align=None), offset=0), "espCipherTransform": SimTypePointer(SimStruct({"cipherTransformId": SimStruct({"cipherType": SimTypeInt(signed=False, label="IPSEC_CIPHER_TYPE"), "cipherConfig": SimTypeChar(label="Byte")}, name="IPSEC_CIPHER_TRANSFORM_ID0", pack=False, align=None), "cryptoModuleId": SimTypePointer(SimTypeBottom(label="Guid"), offset=0)}, name="IPSEC_CIPHER_TRANSFORM0", pack=False, align=None), offset=0), "espAuthAndCipherTransform": SimTypePointer(SimStruct({"authTransform": SimStruct({"authTransformId": SimStruct({"authType": SimTypeInt(signed=False, label="IPSEC_AUTH_TYPE"), "authConfig": SimTypeChar(label="Byte")}, name="IPSEC_AUTH_TRANSFORM_ID0", pack=False, align=None), "cryptoModuleId": SimTypePointer(SimTypeBottom(label="Guid"), offset=0)}, name="IPSEC_AUTH_TRANSFORM0", pack=False, align=None), "cipherTransform": SimStruct({"cipherTransformId": SimStruct({"cipherType": SimTypeInt(signed=False, label="IPSEC_CIPHER_TYPE"), "cipherConfig": SimTypeChar(label="Byte")}, name="IPSEC_CIPHER_TRANSFORM_ID0", pack=False, align=None), "cryptoModuleId": SimTypePointer(SimTypeBottom(label="Guid"), offset=0)}, name="IPSEC_CIPHER_TRANSFORM0", pack=False, align=None)}, name="IPSEC_AUTH_AND_CIPHER_TRANSFORM0", pack=False, align=None), offset=0), "espAuthFwTransform": SimTypePointer(SimStruct({"authTransformId": SimStruct({"authType": SimTypeInt(signed=False, label="IPSEC_AUTH_TYPE"), "authConfig": SimTypeChar(label="Byte")}, name="IPSEC_AUTH_TRANSFORM_ID0", pack=False, align=None), "cryptoModuleId": SimTypePointer(SimTypeBottom(label="Guid"), offset=0)}, name="IPSEC_AUTH_TRANSFORM0", pack=False, align=None), offset=0)}, name="<anon>", label="None")}, name="IPSEC_SA_TRANSFORM0", pack=False, align=None), offset=0), "pfsGroup": SimTypeInt(signed=False, label="IPSEC_PFS_GROUP")}, name="IPSEC_PROPOSAL0", pack=False, align=None), offset=0), "flags": SimTypeInt(signed=False, label="IPSEC_POLICY_FLAG"), "ndAllowClearTimeoutSeconds": SimTypeInt(signed=False, label="UInt32"), "saIdleTimeout": SimStruct({"idleTimeoutSeconds": SimTypeInt(signed=False, label="UInt32"), "idleTimeoutSecondsFailOver": SimTypeInt(signed=False, label="UInt32")}, name="IPSEC_SA_IDLE_TIMEOUT0", pack=False, align=None), "emPolicy": SimTypePointer(SimStruct({"numAuthenticationMethods": SimTypeInt(signed=False, label="UInt32"), "authenticationMethods": SimTypePointer(SimStruct({"authenticationMethodType": SimTypeInt(signed=False, label="IKEEXT_AUTHENTICATION_METHOD_TYPE"), "Anonymous": SimUnion({"presharedKeyAuthentication": SimStruct({"presharedKey": SimStruct({"size": SimTypeInt(signed=False, label="UInt32"), "data": SimTypePointer(SimTypeChar(label="Byte"), offset=0)}, name="FWP_BYTE_BLOB", pack=False, align=None), "flags": SimTypeInt(signed=False, label="IKEEXT_PRESHARED_KEY_AUTHENTICATION_FLAGS")}, name="IKEEXT_PRESHARED_KEY_AUTHENTICATION1", pack=False, align=None), "certificateAuthentication": SimStruct({"inboundConfigType": SimTypeInt(signed=False, label="IKEEXT_CERT_CONFIG_TYPE"), "Anonymous1": SimUnion({"Anonymous1": SimStruct({"inboundRootArraySize": SimTypeInt(signed=False, label="UInt32"), "inboundRootCriteria": SimTypePointer(SimStruct({"certData": SimStruct({"size": SimTypeInt(signed=False, label="UInt32"), "data": SimTypePointer(SimTypeChar(label="Byte"), offset=0)}, name="FWP_BYTE_BLOB", pack=False, align=None), "certHash": SimStruct({"size": SimTypeInt(signed=False, label="UInt32"), "data": SimTypePointer(SimTypeChar(label="Byte"), offset=0)}, name="FWP_BYTE_BLOB", pack=False, align=None), "eku": SimTypePointer(SimStruct({"numEku": SimTypeInt(signed=False, label="UInt32"), "eku": SimTypePointer(SimTypePointer(SimTypeChar(label="Byte"), offset=0), offset=0)}, name="IKEEXT_CERT_EKUS0", pack=False, align=None), offset=0), "name": SimTypePointer(SimStruct({"nameType": SimTypeInt(signed=False, label="IKEEXT_CERT_CRITERIA_NAME_TYPE"), "certName": SimTypePointer(SimTypeChar(label="Char"), offset=0)}, name="IKEEXT_CERT_NAME0", pack=False, align=None), offset=0), "flags": SimTypeInt(signed=False, label="UInt32")}, name="IKEEXT_CERTIFICATE_CRITERIA0", pack=False, align=None), offset=0)}, name="_Anonymous1_e__Struct", pack=False, align=None), "Anonymous2": SimStruct({"inboundEnterpriseStoreArraySize": SimTypeInt(signed=False, label="UInt32"), "inboundEnterpriseStoreCriteria": SimTypePointer(SimStruct({"certData": SimStruct({"size": SimTypeInt(signed=False, label="UInt32"), "data": SimTypePointer(SimTypeChar(label="Byte"), offset=0)}, name="FWP_BYTE_BLOB", pack=False, align=None), "certHash": SimStruct({"size": SimTypeInt(signed=False, label="UInt32"), "data": SimTypePointer(SimTypeChar(label="Byte"), offset=0)}, name="FWP_BYTE_BLOB", pack=False, align=None), "eku": SimTypePointer(SimStruct({"numEku": SimTypeInt(signed=False, label="UInt32"), "eku": SimTypePointer(SimTypePointer(SimTypeChar(label="Byte"), offset=0), offset=0)}, name="IKEEXT_CERT_EKUS0", pack=False, align=None), offset=0), "name": SimTypePointer(SimStruct({"nameType": SimTypeInt(signed=False, label="IKEEXT_CERT_CRITERIA_NAME_TYPE"), "certName": SimTypePointer(SimTypeChar(label="Char"), offset=0)}, name="IKEEXT_CERT_NAME0", pack=False, align=None), offset=0), "flags": SimTypeInt(signed=False, label="UInt32")}, name="IKEEXT_CERTIFICATE_CRITERIA0", pack=False, align=None), offset=0)}, name="_Anonymous2_e__Struct", pack=False, align=None), "Anonymous3": SimStruct({"inboundRootStoreArraySize": SimTypeInt(signed=False, label="UInt32"), "inboundTrustedRootStoreCriteria": SimTypePointer(SimStruct({"certData": SimStruct({"size": SimTypeInt(signed=False, label="UInt32"), "data": SimTypePointer(SimTypeChar(label="Byte"), offset=0)}, name="FWP_BYTE_BLOB", pack=False, align=None), "certHash": SimStruct({"size": SimTypeInt(signed=False, label="UInt32"), "data": SimTypePointer(SimTypeChar(label="Byte"), offset=0)}, name="FWP_BYTE_BLOB", pack=False, align=None), "eku": SimTypePointer(SimStruct({"numEku": SimTypeInt(signed=False, label="UInt32"), "eku": SimTypePointer(SimTypePointer(SimTypeChar(label="Byte"), offset=0), offset=0)}, name="IKEEXT_CERT_EKUS0", pack=False, align=None), offset=0), "name": SimTypePointer(SimStruct({"nameType": SimTypeInt(signed=False, label="IKEEXT_CERT_CRITERIA_NAME_TYPE"), "certName": SimTypePointer(SimTypeChar(label="Char"), offset=0)}, name="IKEEXT_CERT_NAME0", pack=False, align=None), offset=0), "flags": SimTypeInt(signed=False, label="UInt32")}, name="IKEEXT_CERTIFICATE_CRITERIA0", pack=False, align=None), offset=0)}, name="_Anonymous3_e__Struct", pack=False, align=None)}, name="<anon>", label="None"), "outboundConfigType": SimTypeInt(signed=False, label="IKEEXT_CERT_CONFIG_TYPE"), "Anonymous2": SimUnion({"Anonymous1": SimStruct({"outboundRootArraySize": SimTypeInt(signed=False, label="UInt32"), "outboundRootCriteria": SimTypePointer(SimStruct({"certData": SimStruct({"size": SimTypeInt(signed=False, label="UInt32"), "data": SimTypePointer(SimTypeChar(label="Byte"), offset=0)}, name="FWP_BYTE_BLOB", pack=False, align=None), "certHash": SimStruct({"size": SimTypeInt(signed=False, label="UInt32"), "data": SimTypePointer(SimTypeChar(label="Byte"), offset=0)}, name="FWP_BYTE_BLOB", pack=False, align=None), "eku": SimTypePointer(SimStruct({"numEku": SimTypeInt(signed=False, label="UInt32"), "eku": SimTypePointer(SimTypePointer(SimTypeChar(label="Byte"), offset=0), offset=0)}, name="IKEEXT_CERT_EKUS0", pack=False, align=None), offset=0), "name": SimTypePointer(SimStruct({"nameType": SimTypeInt(signed=False, label="IKEEXT_CERT_CRITERIA_NAME_TYPE"), "certName": SimTypePointer(SimTypeChar(label="Char"), offset=0)}, name="IKEEXT_CERT_NAME0", pack=False, align=None), offset=0), "flags": SimTypeInt(signed=False, label="UInt32")}, name="IKEEXT_CERTIFICATE_CRITERIA0", pack=False, align=None), offset=0)}, name="_Anonymous1_e__Struct", pack=False, align=None), "Anonymous2": SimStruct({"outboundEnterpriseStoreArraySize": SimTypeInt(signed=False, label="UInt32"), "outboundEnterpriseStoreCriteria": SimTypePointer(SimStruct({"certData": SimStruct({"size": SimTypeInt(signed=False, label="UInt32"), "data": SimTypePointer(SimTypeChar(label="Byte"), offset=0)}, name="FWP_BYTE_BLOB", pack=False, align=None), "certHash": SimStruct({"size": SimTypeInt(signed=False, label="UInt32"), "data": SimTypePointer(SimTypeChar(label="Byte"), offset=0)}, name="FWP_BYTE_BLOB", pack=False, align=None), "eku": SimTypePointer(SimStruct({"numEku": SimTypeInt(signed=False, label="UInt32"), "eku": SimTypePointer(SimTypePointer(SimTypeChar(label="Byte"), offset=0), offset=0)}, name="IKEEXT_CERT_EKUS0", pack=False, align=None), offset=0), "name": SimTypePointer(SimStruct({"nameType": SimTypeInt(signed=False, label="IKEEXT_CERT_CRITERIA_NAME_TYPE"), "certName": SimTypePointer(SimTypeChar(label="Char"), offset=0)}, name="IKEEXT_CERT_NAME0", pack=False, align=None), offset=0), "flags": SimTypeInt(signed=False, label="UInt32")}, name="IKEEXT_CERTIFICATE_CRITERIA0", pack=False, align=None), offset=0)}, name="_Anonymous2_e__Struct", pack=False, align=None), "Anonymous3": SimStruct({"outboundRootStoreArraySize": SimTypeInt(signed=False, label="UInt32"), "outboundTrustedRootStoreCriteria": SimTypePointer(SimStruct({"certData": SimStruct({"size": SimTypeInt(signed=False, label="UInt32"), "data": SimTypePointer(SimTypeChar(label="Byte"), offset=0)}, name="FWP_BYTE_BLOB", pack=False, align=None), "certHash": SimStruct({"size": SimTypeInt(signed=False, label="UInt32"), "data": SimTypePointer(SimTypeChar(label="Byte"), offset=0)}, name="FWP_BYTE_BLOB", pack=False, align=None), "eku": SimTypePointer(SimStruct({"numEku": SimTypeInt(signed=False, label="UInt32"), "eku": SimTypePointer(SimTypePointer(SimTypeChar(label="Byte"), offset=0), offset=0)}, name="IKEEXT_CERT_EKUS0", pack=False, align=None), offset=0), "name": SimTypePointer(SimStruct({"nameType": SimTypeInt(signed=False, label="IKEEXT_CERT_CRITERIA_NAME_TYPE"), "certName": SimTypePointer(SimTypeChar(label="Char"), offset=0)}, name="IKEEXT_CERT_NAME0", pack=False, align=None), offset=0), "flags": SimTypeInt(signed=False, label="UInt32")}, name="IKEEXT_CERTIFICATE_CRITERIA0", pack=False, align=None), offset=0)}, name="_Anonymous3_e__Struct", pack=False, align=None)}, name="<anon>", label="None"), "flags": SimTypeInt(signed=False, label="IKEEXT_CERT_AUTH"), "localCertLocationUrl": SimStruct({"size": SimTypeInt(signed=False, label="UInt32"), "data": SimTypePointer(SimTypeChar(label="Byte"), offset=0)}, name="FWP_BYTE_BLOB", pack=False, align=None)}, name="IKEEXT_CERTIFICATE_AUTHENTICATION2", pack=False, align=None), "kerberosAuthentication": SimStruct({"flags": SimTypeInt(signed=False, label="IKEEXT_KERBEROS_AUTHENTICATION_FLAGS"), "proxyServer": SimTypePointer(SimTypeChar(label="Char"), offset=0)}, name="IKEEXT_KERBEROS_AUTHENTICATION1", pack=False, align=None), "reservedAuthentication": SimStruct({"flags": SimTypeInt(signed=False, label="IKEEXT_RESERVED_AUTHENTICATION_FLAGS")}, name="IKEEXT_RESERVED_AUTHENTICATION0", pack=False, align=None), "ntlmV2Authentication": SimStruct({"flags": SimTypeInt(signed=False, label="UInt32")}, name="IKEEXT_NTLM_V2_AUTHENTICATION0", pack=False, align=None), "sslAuthentication": SimStruct({"inboundConfigType": SimTypeInt(signed=False, label="IKEEXT_CERT_CONFIG_TYPE"), "Anonymous1": SimUnion({"Anonymous1": SimStruct({"inboundRootArraySize": SimTypeInt(signed=False, label="UInt32"), "inboundRootCriteria": SimTypePointer(SimStruct({"certData": SimStruct({"size": SimTypeInt(signed=False, label="UInt32"), "data": SimTypePointer(SimTypeChar(label="Byte"), offset=0)}, name="FWP_BYTE_BLOB", pack=False, align=None), "certHash": SimStruct({"size": SimTypeInt(signed=False, label="UInt32"), "data": SimTypePointer(SimTypeChar(label="Byte"), offset=0)}, name="FWP_BYTE_BLOB", pack=False, align=None), "eku": SimTypePointer(SimStruct({"numEku": SimTypeInt(signed=False, label="UInt32"), "eku": SimTypePointer(SimTypePointer(SimTypeChar(label="Byte"), offset=0), offset=0)}, name="IKEEXT_CERT_EKUS0", pack=False, align=None), offset=0), "name": SimTypePointer(SimStruct({"nameType": SimTypeInt(signed=False, label="IKEEXT_CERT_CRITERIA_NAME_TYPE"), "certName": SimTypePointer(SimTypeChar(label="Char"), offset=0)}, name="IKEEXT_CERT_NAME0", pack=False, align=None), offset=0), "flags": SimTypeInt(signed=False, label="UInt32")}, name="IKEEXT_CERTIFICATE_CRITERIA0", pack=False, align=None), offset=0)}, name="_Anonymous1_e__Struct", pack=False, align=None), "Anonymous2": SimStruct({"inboundEnterpriseStoreArraySize": SimTypeInt(signed=False, label="UInt32"), "inboundEnterpriseStoreCriteria": SimTypePointer(SimStruct({"certData": SimStruct({"size": SimTypeInt(signed=False, label="UInt32"), "data": SimTypePointer(SimTypeChar(label="Byte"), offset=0)}, name="FWP_BYTE_BLOB", pack=False, align=None), "certHash": SimStruct({"size": SimTypeInt(signed=False, label="UInt32"), "data": SimTypePointer(SimTypeChar(label="Byte"), offset=0)}, name="FWP_BYTE_BLOB", pack=False, align=None), "eku": SimTypePointer(SimStruct({"numEku": SimTypeInt(signed=False, label="UInt32"), "eku": SimTypePointer(SimTypePointer(SimTypeChar(label="Byte"), offset=0), offset=0)}, name="IKEEXT_CERT_EKUS0", pack=False, align=None), offset=0), "name": SimTypePointer(SimStruct({"nameType": SimTypeInt(signed=False, label="IKEEXT_CERT_CRITERIA_NAME_TYPE"), "certName": SimTypePointer(SimTypeChar(label="Char"), offset=0)}, name="IKEEXT_CERT_NAME0", pack=False, align=None), offset=0), "flags": SimTypeInt(signed=False, label="UInt32")}, name="IKEEXT_CERTIFICATE_CRITERIA0", pack=False, align=None), offset=0)}, name="_Anonymous2_e__Struct", pack=False, align=None), "Anonymous3": SimStruct({"inboundRootStoreArraySize": SimTypeInt(signed=False, label="UInt32"), "inboundTrustedRootStoreCriteria": SimTypePointer(SimStruct({"certData": SimStruct({"size": SimTypeInt(signed=False, label="UInt32"), "data": SimTypePointer(SimTypeChar(label="Byte"), offset=0)}, name="FWP_BYTE_BLOB", pack=False, align=None), "certHash": SimStruct({"size": SimTypeInt(signed=False, label="UInt32"), "data": SimTypePointer(SimTypeChar(label="Byte"), offset=0)}, name="FWP_BYTE_BLOB", pack=False, align=None), "eku": SimTypePointer(SimStruct({"numEku": SimTypeInt(signed=False, label="UInt32"), "eku": SimTypePointer(SimTypePointer(SimTypeChar(label="Byte"), offset=0), offset=0)}, name="IKEEXT_CERT_EKUS0", pack=False, align=None), offset=0), "name": SimTypePointer(SimStruct({"nameType": SimTypeInt(signed=False, label="IKEEXT_CERT_CRITERIA_NAME_TYPE"), "certName": SimTypePointer(SimTypeChar(label="Char"), offset=0)}, name="IKEEXT_CERT_NAME0", pack=False, align=None), offset=0), "flags": SimTypeInt(signed=False, label="UInt32")}, name="IKEEXT_CERTIFICATE_CRITERIA0", pack=False, align=None), offset=0)}, name="_Anonymous3_e__Struct", pack=False, align=None)}, name="<anon>", label="None"), "outboundConfigType": SimTypeInt(signed=False, label="IKEEXT_CERT_CONFIG_TYPE"), "Anonymous2": SimUnion({"Anonymous1": SimStruct({"outboundRootArraySize": SimTypeInt(signed=False, label="UInt32"), "outboundRootCriteria": SimTypePointer(SimStruct({"certData": SimStruct({"size": SimTypeInt(signed=False, label="UInt32"), "data": SimTypePointer(SimTypeChar(label="Byte"), offset=0)}, name="FWP_BYTE_BLOB", pack=False, align=None), "certHash": SimStruct({"size": SimTypeInt(signed=False, label="UInt32"), "data": SimTypePointer(SimTypeChar(label="Byte"), offset=0)}, name="FWP_BYTE_BLOB", pack=False, align=None), "eku": SimTypePointer(SimStruct({"numEku": SimTypeInt(signed=False, label="UInt32"), "eku": SimTypePointer(SimTypePointer(SimTypeChar(label="Byte"), offset=0), offset=0)}, name="IKEEXT_CERT_EKUS0", pack=False, align=None), offset=0), "name": SimTypePointer(SimStruct({"nameType": SimTypeInt(signed=False, label="IKEEXT_CERT_CRITERIA_NAME_TYPE"), "certName": SimTypePointer(SimTypeChar(label="Char"), offset=0)}, name="IKEEXT_CERT_NAME0", pack=False, align=None), offset=0), "flags": SimTypeInt(signed=False, label="UInt32")}, name="IKEEXT_CERTIFICATE_CRITERIA0", pack=False, align=None), offset=0)}, name="_Anonymous1_e__Struct", pack=False, align=None), "Anonymous2": SimStruct({"outboundEnterpriseStoreArraySize": SimTypeInt(signed=False, label="UInt32"), "outboundEnterpriseStoreCriteria": SimTypePointer(SimStruct({"certData": SimStruct({"size": SimTypeInt(signed=False, label="UInt32"), "data": SimTypePointer(SimTypeChar(label="Byte"), offset=0)}, name="FWP_BYTE_BLOB", pack=False, align=None), "certHash": SimStruct({"size": SimTypeInt(signed=False, label="UInt32"), "data": SimTypePointer(SimTypeChar(label="Byte"), offset=0)}, name="FWP_BYTE_BLOB", pack=False, align=None), "eku": SimTypePointer(SimStruct({"numEku": SimTypeInt(signed=False, label="UInt32"), "eku": SimTypePointer(SimTypePointer(SimTypeChar(label="Byte"), offset=0), offset=0)}, name="IKEEXT_CERT_EKUS0", pack=False, align=None), offset=0), "name": SimTypePointer(SimStruct({"nameType": SimTypeInt(signed=False, label="IKEEXT_CERT_CRITERIA_NAME_TYPE"), "certName": SimTypePointer(SimTypeChar(label="Char"), offset=0)}, name="IKEEXT_CERT_NAME0", pack=False, align=None), offset=0), "flags": SimTypeInt(signed=False, label="UInt32")}, name="IKEEXT_CERTIFICATE_CRITERIA0", pack=False, align=None), offset=0)}, name="_Anonymous2_e__Struct", pack=False, align=None), "Anonymous3": SimStruct({"outboundRootStoreArraySize": SimTypeInt(signed=False, label="UInt32"), "outboundTrustedRootStoreCriteria": SimTypePointer(SimStruct({"certData": SimStruct({"size": SimTypeInt(signed=False, label="UInt32"), "data": SimTypePointer(SimTypeChar(label="Byte"), offset=0)}, name="FWP_BYTE_BLOB", pack=False, align=None), "certHash": SimStruct({"size": SimTypeInt(signed=False, label="UInt32"), "data": SimTypePointer(SimTypeChar(label="Byte"), offset=0)}, name="FWP_BYTE_BLOB", pack=False, align=None), "eku": SimTypePointer(SimStruct({"numEku": SimTypeInt(signed=False, label="UInt32"), "eku": SimTypePointer(SimTypePointer(SimTypeChar(label="Byte"), offset=0), offset=0)}, name="IKEEXT_CERT_EKUS0", pack=False, align=None), offset=0), "name": SimTypePointer(SimStruct({"nameType": SimTypeInt(signed=False, label="IKEEXT_CERT_CRITERIA_NAME_TYPE"), "certName": SimTypePointer(SimTypeChar(label="Char"), offset=0)}, name="IKEEXT_CERT_NAME0", pack=False, align=None), offset=0), "flags": SimTypeInt(signed=False, label="UInt32")}, name="IKEEXT_CERTIFICATE_CRITERIA0", pack=False, align=None), offset=0)}, name="_Anonymous3_e__Struct", pack=False, align=None)}, name="<anon>", label="None"), "flags": SimTypeInt(signed=False, label="IKEEXT_CERT_AUTH"), "localCertLocationUrl": SimStruct({"size": SimTypeInt(signed=False, label="UInt32"), "data": SimTypePointer(SimTypeChar(label="Byte"), offset=0)}, name="FWP_BYTE_BLOB", pack=False, align=None)}, name="IKEEXT_CERTIFICATE_AUTHENTICATION2", pack=False, align=None), "cgaAuthentication": SimStruct({"keyContainerName": SimTypePointer(SimTypeChar(label="Char"), offset=0), "cspName": SimTypePointer(SimTypeChar(label="Char"), offset=0), "cspType": SimTypeInt(signed=False, label="UInt32"), "cgaModifier": SimStruct({"byteArray16": SimTypeFixedSizeArray(SimTypeChar(label="Byte"), 16)}, name="FWP_BYTE_ARRAY16", pack=False, align=None), "cgaCollisionCount": SimTypeChar(label="Byte")}, name="IKEEXT_IPV6_CGA_AUTHENTICATION0", pack=False, align=None), "eapAuthentication": SimStruct({"flags": SimTypeInt(signed=False, label="IKEEXT_EAP_AUTHENTICATION_FLAGS")}, name="IKEEXT_EAP_AUTHENTICATION0", pack=False, align=None)}, name="<anon>", label="None")}, name="IKEEXT_AUTHENTICATION_METHOD2", pack=False, align=None), offset=0), "initiatorImpersonationType": SimTypeInt(signed=False, label="IKEEXT_AUTHENTICATION_IMPERSONATION_TYPE")}, name="IKEEXT_EM_POLICY2", pack=False, align=None), offset=0)}, name="IPSEC_TRANSPORT_POLICY2", pack=False, align=None), offset=0), "authipQmTunnelPolicy": SimTypePointer(SimStruct({"flags": SimTypeInt(signed=False, label="IPSEC_POLICY_FLAG"), "numIpsecProposals": SimTypeInt(signed=False, label="UInt32"), "ipsecProposals": SimTypePointer(SimStruct({"lifetime": SimStruct({"lifetimeSeconds": SimTypeInt(signed=False, label="UInt32"), "lifetimeKilobytes": SimTypeInt(signed=False, label="UInt32"), "lifetimePackets": SimTypeInt(signed=False, label="UInt32")}, name="IPSEC_SA_LIFETIME0", pack=False, align=None), "numSaTransforms": SimTypeInt(signed=False, label="UInt32"), "saTransforms": SimTypePointer(SimStruct({"ipsecTransformType": SimTypeInt(signed=False, label="IPSEC_TRANSFORM_TYPE"), "Anonymous": SimUnion({"ahTransform": SimTypePointer(SimStruct({"authTransformId": SimStruct({"authType": SimTypeInt(signed=False, label="IPSEC_AUTH_TYPE"), "authConfig": SimTypeChar(label="Byte")}, name="IPSEC_AUTH_TRANSFORM_ID0", pack=False, align=None), "cryptoModuleId": SimTypePointer(SimTypeBottom(label="Guid"), offset=0)}, name="IPSEC_AUTH_TRANSFORM0", pack=False, align=None), offset=0), "espAuthTransform": SimTypePointer(SimStruct({"authTransformId": SimStruct({"authType": SimTypeInt(signed=False, label="IPSEC_AUTH_TYPE"), "authConfig": SimTypeChar(label="Byte")}, name="IPSEC_AUTH_TRANSFORM_ID0", pack=False, align=None), "cryptoModuleId": SimTypePointer(SimTypeBottom(label="Guid"), offset=0)}, name="IPSEC_AUTH_TRANSFORM0", pack=False, align=None), offset=0), "espCipherTransform": SimTypePointer(SimStruct({"cipherTransformId": SimStruct({"cipherType": SimTypeInt(signed=False, label="IPSEC_CIPHER_TYPE"), "cipherConfig": SimTypeChar(label="Byte")}, name="IPSEC_CIPHER_TRANSFORM_ID0", pack=False, align=None), "cryptoModuleId": SimTypePointer(SimTypeBottom(label="Guid"), offset=0)}, name="IPSEC_CIPHER_TRANSFORM0", pack=False, align=None), offset=0), "espAuthAndCipherTransform": SimTypePointer(SimStruct({"authTransform": SimStruct({"authTransformId": SimStruct({"authType": SimTypeInt(signed=False, label="IPSEC_AUTH_TYPE"), "authConfig": SimTypeChar(label="Byte")}, name="IPSEC_AUTH_TRANSFORM_ID0", pack=False, align=None), "cryptoModuleId": SimTypePointer(SimTypeBottom(label="Guid"), offset=0)}, name="IPSEC_AUTH_TRANSFORM0", pack=False, align=None), "cipherTransform": SimStruct({"cipherTransformId": SimStruct({"cipherType": SimTypeInt(signed=False, label="IPSEC_CIPHER_TYPE"), "cipherConfig": SimTypeChar(label="Byte")}, name="IPSEC_CIPHER_TRANSFORM_ID0", pack=False, align=None), "cryptoModuleId": SimTypePointer(SimTypeBottom(label="Guid"), offset=0)}, name="IPSEC_CIPHER_TRANSFORM0", pack=False, align=None)}, name="IPSEC_AUTH_AND_CIPHER_TRANSFORM0", pack=False, align=None), offset=0), "espAuthFwTransform": SimTypePointer(SimStruct({"authTransformId": SimStruct({"authType": SimTypeInt(signed=False, label="IPSEC_AUTH_TYPE"), "authConfig": SimTypeChar(label="Byte")}, name="IPSEC_AUTH_TRANSFORM_ID0", pack=False, align=None), "cryptoModuleId": SimTypePointer(SimTypeBottom(label="Guid"), offset=0)}, name="IPSEC_AUTH_TRANSFORM0", pack=False, align=None), offset=0)}, name="<anon>", label="None")}, name="IPSEC_SA_TRANSFORM0", pack=False, align=None), offset=0), "pfsGroup": SimTypeInt(signed=False, label="IPSEC_PFS_GROUP")}, name="IPSEC_PROPOSAL0", pack=False, align=None), offset=0), "tunnelEndpoints": SimStruct({"ipVersion": SimTypeInt(signed=False, label="FWP_IP_VERSION"), "Anonymous1": SimUnion({"localV4Address": SimTypeInt(signed=False, label="UInt32"), "localV6Address": SimTypeFixedSizeArray(SimTypeChar(label="Byte"), 16)}, name="<anon>", label="None"), "Anonymous2": SimUnion({"remoteV4Address": SimTypeInt(signed=False, label="UInt32"), "remoteV6Address": SimTypeFixedSizeArray(SimTypeChar(label="Byte"), 16)}, name="<anon>", label="None"), "localIfLuid": SimTypeLongLong(signed=False, label="UInt64"), "remoteFqdn": SimTypePointer(SimTypeChar(label="Char"), offset=0), "numAddresses": SimTypeInt(signed=False, label="UInt32"), "remoteAddresses": SimTypePointer(SimStruct({"ipVersion": SimTypeInt(signed=False, label="FWP_IP_VERSION"), "Anonymous": SimUnion({"v4Address": SimTypeInt(signed=False, label="UInt32"), "v6Address": SimTypeFixedSizeArray(SimTypeChar(label="Byte"), 16)}, name="<anon>", label="None")}, name="IPSEC_TUNNEL_ENDPOINT0", pack=False, align=None), offset=0)}, name="IPSEC_TUNNEL_ENDPOINTS2", pack=False, align=None), "saIdleTimeout": SimStruct({"idleTimeoutSeconds": SimTypeInt(signed=False, label="UInt32"), "idleTimeoutSecondsFailOver": SimTypeInt(signed=False, label="UInt32")}, name="IPSEC_SA_IDLE_TIMEOUT0", pack=False, align=None), "emPolicy": SimTypePointer(SimStruct({"numAuthenticationMethods": SimTypeInt(signed=False, label="UInt32"), "authenticationMethods": SimTypePointer(SimStruct({"authenticationMethodType": SimTypeInt(signed=False, label="IKEEXT_AUTHENTICATION_METHOD_TYPE"), "Anonymous": SimUnion({"presharedKeyAuthentication": SimStruct({"presharedKey": SimStruct({"size": SimTypeInt(signed=False, label="UInt32"), "data": SimTypePointer(SimTypeChar(label="Byte"), offset=0)}, name="FWP_BYTE_BLOB", pack=False, align=None), "flags": SimTypeInt(signed=False, label="IKEEXT_PRESHARED_KEY_AUTHENTICATION_FLAGS")}, name="IKEEXT_PRESHARED_KEY_AUTHENTICATION1", pack=False, align=None), "certificateAuthentication": SimStruct({"inboundConfigType": SimTypeInt(signed=False, label="IKEEXT_CERT_CONFIG_TYPE"), "Anonymous1": SimUnion({"Anonymous1": SimStruct({"inboundRootArraySize": SimTypeInt(signed=False, label="UInt32"), "inboundRootCriteria": SimTypePointer(SimStruct({"certData": SimStruct({"size": SimTypeInt(signed=False, label="UInt32"), "data": SimTypePointer(SimTypeChar(label="Byte"), offset=0)}, name="FWP_BYTE_BLOB", pack=False, align=None), "certHash": SimStruct({"size": SimTypeInt(signed=False, label="UInt32"), "data": SimTypePointer(SimTypeChar(label="Byte"), offset=0)}, name="FWP_BYTE_BLOB", pack=False, align=None), "eku": SimTypePointer(SimStruct({"numEku": SimTypeInt(signed=False, label="UInt32"), "eku": SimTypePointer(SimTypePointer(SimTypeChar(label="Byte"), offset=0), offset=0)}, name="IKEEXT_CERT_EKUS0", pack=False, align=None), offset=0), "name": SimTypePointer(SimStruct({"nameType": SimTypeInt(signed=False, label="IKEEXT_CERT_CRITERIA_NAME_TYPE"), "certName": SimTypePointer(SimTypeChar(label="Char"), offset=0)}, name="IKEEXT_CERT_NAME0", pack=False, align=None), offset=0), "flags": SimTypeInt(signed=False, label="UInt32")}, name="IKEEXT_CERTIFICATE_CRITERIA0", pack=False, align=None), offset=0)}, name="_Anonymous1_e__Struct", pack=False, align=None), "Anonymous2": SimStruct({"inboundEnterpriseStoreArraySize": SimTypeInt(signed=False, label="UInt32"), "inboundEnterpriseStoreCriteria": SimTypePointer(SimStruct({"certData": SimStruct({"size": SimTypeInt(signed=False, label="UInt32"), "data": SimTypePointer(SimTypeChar(label="Byte"), offset=0)}, name="FWP_BYTE_BLOB", pack=False, align=None), "certHash": SimStruct({"size": SimTypeInt(signed=False, label="UInt32"), "data": SimTypePointer(SimTypeChar(label="Byte"), offset=0)}, name="FWP_BYTE_BLOB", pack=False, align=None), "eku": SimTypePointer(SimStruct({"numEku": SimTypeInt(signed=False, label="UInt32"), "eku": SimTypePointer(SimTypePointer(SimTypeChar(label="Byte"), offset=0), offset=0)}, name="IKEEXT_CERT_EKUS0", pack=False, align=None), offset=0), "name": SimTypePointer(SimStruct({"nameType": SimTypeInt(signed=False, label="IKEEXT_CERT_CRITERIA_NAME_TYPE"), "certName": SimTypePointer(SimTypeChar(label="Char"), offset=0)}, name="IKEEXT_CERT_NAME0", pack=False, align=None), offset=0), "flags": SimTypeInt(signed=False, label="UInt32")}, name="IKEEXT_CERTIFICATE_CRITERIA0", pack=False, align=None), offset=0)}, name="_Anonymous2_e__Struct", pack=False, align=None), "Anonymous3": SimStruct({"inboundRootStoreArraySize": SimTypeInt(signed=False, label="UInt32"), "inboundTrustedRootStoreCriteria": SimTypePointer(SimStruct({"certData": SimStruct({"size": SimTypeInt(signed=False, label="UInt32"), "data": SimTypePointer(SimTypeChar(label="Byte"), offset=0)}, name="FWP_BYTE_BLOB", pack=False, align=None), "certHash": SimStruct({"size": SimTypeInt(signed=False, label="UInt32"), "data": SimTypePointer(SimTypeChar(label="Byte"), offset=0)}, name="FWP_BYTE_BLOB", pack=False, align=None), "eku": SimTypePointer(SimStruct({"numEku": SimTypeInt(signed=False, label="UInt32"), "eku": SimTypePointer(SimTypePointer(SimTypeChar(label="Byte"), offset=0), offset=0)}, name="IKEEXT_CERT_EKUS0", pack=False, align=None), offset=0), "name": SimTypePointer(SimStruct({"nameType": SimTypeInt(signed=False, label="IKEEXT_CERT_CRITERIA_NAME_TYPE"), "certName": SimTypePointer(SimTypeChar(label="Char"), offset=0)}, name="IKEEXT_CERT_NAME0", pack=False, align=None), offset=0), "flags": SimTypeInt(signed=False, label="UInt32")}, name="IKEEXT_CERTIFICATE_CRITERIA0", pack=False, align=None), offset=0)}, name="_Anonymous3_e__Struct", pack=False, align=None)}, name="<anon>", label="None"), "outboundConfigType": SimTypeInt(signed=False, label="IKEEXT_CERT_CONFIG_TYPE"), "Anonymous2": SimUnion({"Anonymous1": SimStruct({"outboundRootArraySize": SimTypeInt(signed=False, label="UInt32"), "outboundRootCriteria": SimTypePointer(SimStruct({"certData": SimStruct({"size": SimTypeInt(signed=False, label="UInt32"), "data": SimTypePointer(SimTypeChar(label="Byte"), offset=0)}, name="FWP_BYTE_BLOB", pack=False, align=None), "certHash": SimStruct({"size": SimTypeInt(signed=False, label="UInt32"), "data": SimTypePointer(SimTypeChar(label="Byte"), offset=0)}, name="FWP_BYTE_BLOB", pack=False, align=None), "eku": SimTypePointer(SimStruct({"numEku": SimTypeInt(signed=False, label="UInt32"), "eku": SimTypePointer(SimTypePointer(SimTypeChar(label="Byte"), offset=0), offset=0)}, name="IKEEXT_CERT_EKUS0", pack=False, align=None), offset=0), "name": SimTypePointer(SimStruct({"nameType": SimTypeInt(signed=False, label="IKEEXT_CERT_CRITERIA_NAME_TYPE"), "certName": SimTypePointer(SimTypeChar(label="Char"), offset=0)}, name="IKEEXT_CERT_NAME0", pack=False, align=None), offset=0), "flags": SimTypeInt(signed=False, label="UInt32")}, name="IKEEXT_CERTIFICATE_CRITERIA0", pack=False, align=None), offset=0)}, name="_Anonymous1_e__Struct", pack=False, align=None), "Anonymous2": SimStruct({"outboundEnterpriseStoreArraySize": SimTypeInt(signed=False, label="UInt32"), "outboundEnterpriseStoreCriteria": SimTypePointer(SimStruct({"certData": SimStruct({"size": SimTypeInt(signed=False, label="UInt32"), "data": SimTypePointer(SimTypeChar(label="Byte"), offset=0)}, name="FWP_BYTE_BLOB", pack=False, align=None), "certHash": SimStruct({"size": SimTypeInt(signed=False, label="UInt32"), "data": SimTypePointer(SimTypeChar(label="Byte"), offset=0)}, name="FWP_BYTE_BLOB", pack=False, align=None), "eku": SimTypePointer(SimStruct({"numEku": SimTypeInt(signed=False, label="UInt32"), "eku": SimTypePointer(SimTypePointer(SimTypeChar(label="Byte"), offset=0), offset=0)}, name="IKEEXT_CERT_EKUS0", pack=False, align=None), offset=0), "name": SimTypePointer(SimStruct({"nameType": SimTypeInt(signed=False, label="IKEEXT_CERT_CRITERIA_NAME_TYPE"), "certName": SimTypePointer(SimTypeChar(label="Char"), offset=0)}, name="IKEEXT_CERT_NAME0", pack=False, align=None), offset=0), "flags": SimTypeInt(signed=False, label="UInt32")}, name="IKEEXT_CERTIFICATE_CRITERIA0", pack=False, align=None), offset=0)}, name="_Anonymous2_e__Struct", pack=False, align=None), "Anonymous3": SimStruct({"outboundRootStoreArraySize": SimTypeInt(signed=False, label="UInt32"), "outboundTrustedRootStoreCriteria": SimTypePointer(SimStruct({"certData": SimStruct({"size": SimTypeInt(signed=False, label="UInt32"), "data": SimTypePointer(SimTypeChar(label="Byte"), offset=0)}, name="FWP_BYTE_BLOB", pack=False, align=None), "certHash": SimStruct({"size": SimTypeInt(signed=False, label="UInt32"), "data": SimTypePointer(SimTypeChar(label="Byte"), offset=0)}, name="FWP_BYTE_BLOB", pack=False, align=None), "eku": SimTypePointer(SimStruct({"numEku": SimTypeInt(signed=False, label="UInt32"), "eku": SimTypePointer(SimTypePointer(SimTypeChar(label="Byte"), offset=0), offset=0)}, name="IKEEXT_CERT_EKUS0", pack=False, align=None), offset=0), "name": SimTypePointer(SimStruct({"nameType": SimTypeInt(signed=False, label="IKEEXT_CERT_CRITERIA_NAME_TYPE"), "certName": SimTypePointer(SimTypeChar(label="Char"), offset=0)}, name="IKEEXT_CERT_NAME0", pack=False, align=None), offset=0), "flags": SimTypeInt(signed=False, label="UInt32")}, name="IKEEXT_CERTIFICATE_CRITERIA0", pack=False, align=None), offset=0)}, name="_Anonymous3_e__Struct", pack=False, align=None)}, name="<anon>", label="None"), "flags": SimTypeInt(signed=False, label="IKEEXT_CERT_AUTH"), "localCertLocationUrl": SimStruct({"size": SimTypeInt(signed=False, label="UInt32"), "data": SimTypePointer(SimTypeChar(label="Byte"), offset=0)}, name="FWP_BYTE_BLOB", pack=False, align=None)}, name="IKEEXT_CERTIFICATE_AUTHENTICATION2", pack=False, align=None), "kerberosAuthentication": SimStruct({"flags": SimTypeInt(signed=False, label="IKEEXT_KERBEROS_AUTHENTICATION_FLAGS"), "proxyServer": SimTypePointer(SimTypeChar(label="Char"), offset=0)}, name="IKEEXT_KERBEROS_AUTHENTICATION1", pack=False, align=None), "reservedAuthentication": SimStruct({"flags": SimTypeInt(signed=False, label="IKEEXT_RESERVED_AUTHENTICATION_FLAGS")}, name="IKEEXT_RESERVED_AUTHENTICATION0", pack=False, align=None), "ntlmV2Authentication": SimStruct({"flags": SimTypeInt(signed=False, label="UInt32")}, name="IKEEXT_NTLM_V2_AUTHENTICATION0", pack=False, align=None), "sslAuthentication": SimStruct({"inboundConfigType": SimTypeInt(signed=False, label="IKEEXT_CERT_CONFIG_TYPE"), "Anonymous1": SimUnion({"Anonymous1": SimStruct({"inboundRootArraySize": SimTypeInt(signed=False, label="UInt32"), "inboundRootCriteria": SimTypePointer(SimStruct({"certData": SimStruct({"size": SimTypeInt(signed=False, label="UInt32"), "data": SimTypePointer(SimTypeChar(label="Byte"), offset=0)}, name="FWP_BYTE_BLOB", pack=False, align=None), "certHash": SimStruct({"size": SimTypeInt(signed=False, label="UInt32"), "data": SimTypePointer(SimTypeChar(label="Byte"), offset=0)}, name="FWP_BYTE_BLOB", pack=False, align=None), "eku": SimTypePointer(SimStruct({"numEku": SimTypeInt(signed=False, label="UInt32"), "eku": SimTypePointer(SimTypePointer(SimTypeChar(label="Byte"), offset=0), offset=0)}, name="IKEEXT_CERT_EKUS0", pack=False, align=None), offset=0), "name": SimTypePointer(SimStruct({"nameType": SimTypeInt(signed=False, label="IKEEXT_CERT_CRITERIA_NAME_TYPE"), "certName": SimTypePointer(SimTypeChar(label="Char"), offset=0)}, name="IKEEXT_CERT_NAME0", pack=False, align=None), offset=0), "flags": SimTypeInt(signed=False, label="UInt32")}, name="IKEEXT_CERTIFICATE_CRITERIA0", pack=False, align=None), offset=0)}, name="_Anonymous1_e__Struct", pack=False, align=None), "Anonymous2": SimStruct({"inboundEnterpriseStoreArraySize": SimTypeInt(signed=False, label="UInt32"), "inboundEnterpriseStoreCriteria": SimTypePointer(SimStruct({"certData": SimStruct({"size": SimTypeInt(signed=False, label="UInt32"), "data": SimTypePointer(SimTypeChar(label="Byte"), offset=0)}, name="FWP_BYTE_BLOB", pack=False, align=None), "certHash": SimStruct({"size": SimTypeInt(signed=False, label="UInt32"), "data": SimTypePointer(SimTypeChar(label="Byte"), offset=0)}, name="FWP_BYTE_BLOB", pack=False, align=None), "eku": SimTypePointer(SimStruct({"numEku": SimTypeInt(signed=False, label="UInt32"), "eku": SimTypePointer(SimTypePointer(SimTypeChar(label="Byte"), offset=0), offset=0)}, name="IKEEXT_CERT_EKUS0", pack=False, align=None), offset=0), "name": SimTypePointer(SimStruct({"nameType": SimTypeInt(signed=False, label="IKEEXT_CERT_CRITERIA_NAME_TYPE"), "certName": SimTypePointer(SimTypeChar(label="Char"), offset=0)}, name="IKEEXT_CERT_NAME0", pack=False, align=None), offset=0), "flags": SimTypeInt(signed=False, label="UInt32")}, name="IKEEXT_CERTIFICATE_CRITERIA0", pack=False, align=None), offset=0)}, name="_Anonymous2_e__Struct", pack=False, align=None), "Anonymous3": SimStruct({"inboundRootStoreArraySize": SimTypeInt(signed=False, label="UInt32"), "inboundTrustedRootStoreCriteria": SimTypePointer(SimStruct({"certData": SimStruct({"size": SimTypeInt(signed=False, label="UInt32"), "data": SimTypePointer(SimTypeChar(label="Byte"), offset=0)}, name="FWP_BYTE_BLOB", pack=False, align=None), "certHash": SimStruct({"size": SimTypeInt(signed=False, label="UInt32"), "data": SimTypePointer(SimTypeChar(label="Byte"), offset=0)}, name="FWP_BYTE_BLOB", pack=False, align=None), "eku": SimTypePointer(SimStruct({"numEku": SimTypeInt(signed=False, label="UInt32"), "eku": SimTypePointer(SimTypePointer(SimTypeChar(label="Byte"), offset=0), offset=0)}, name="IKEEXT_CERT_EKUS0", pack=False, align=None), offset=0), "name": SimTypePointer(SimStruct({"nameType": SimTypeInt(signed=False, label="IKEEXT_CERT_CRITERIA_NAME_TYPE"), "certName": SimTypePointer(SimTypeChar(label="Char"), offset=0)}, name="IKEEXT_CERT_NAME0", pack=False, align=None), offset=0), "flags": SimTypeInt(signed=False, label="UInt32")}, name="IKEEXT_CERTIFICATE_CRITERIA0", pack=False, align=None), offset=0)}, name="_Anonymous3_e__Struct", pack=False, align=None)}, name="<anon>", label="None"), "outboundConfigType": SimTypeInt(signed=False, label="IKEEXT_CERT_CONFIG_TYPE"), "Anonymous2": SimUnion({"Anonymous1": SimStruct({"outboundRootArraySize": SimTypeInt(signed=False, label="UInt32"), "outboundRootCriteria": SimTypePointer(SimStruct({"certData": SimStruct({"size": SimTypeInt(signed=False, label="UInt32"), "data": SimTypePointer(SimTypeChar(label="Byte"), offset=0)}, name="FWP_BYTE_BLOB", pack=False, align=None), "certHash": SimStruct({"size": SimTypeInt(signed=False, label="UInt32"), "data": SimTypePointer(SimTypeChar(label="Byte"), offset=0)}, name="FWP_BYTE_BLOB", pack=False, align=None), "eku": SimTypePointer(SimStruct({"numEku": SimTypeInt(signed=False, label="UInt32"), "eku": SimTypePointer(SimTypePointer(SimTypeChar(label="Byte"), offset=0), offset=0)}, name="IKEEXT_CERT_EKUS0", pack=False, align=None), offset=0), "name": SimTypePointer(SimStruct({"nameType": SimTypeInt(signed=False, label="IKEEXT_CERT_CRITERIA_NAME_TYPE"), "certName": SimTypePointer(SimTypeChar(label="Char"), offset=0)}, name="IKEEXT_CERT_NAME0", pack=False, align=None), offset=0), "flags": SimTypeInt(signed=False, label="UInt32")}, name="IKEEXT_CERTIFICATE_CRITERIA0", pack=False, align=None), offset=0)}, name="_Anonymous1_e__Struct", pack=False, align=None), "Anonymous2": SimStruct({"outboundEnterpriseStoreArraySize": SimTypeInt(signed=False, label="UInt32"), "outboundEnterpriseStoreCriteria": SimTypePointer(SimStruct({"certData": SimStruct({"size": SimTypeInt(signed=False, label="UInt32"), "data": SimTypePointer(SimTypeChar(label="Byte"), offset=0)}, name="FWP_BYTE_BLOB", pack=False, align=None), "certHash": SimStruct({"size": SimTypeInt(signed=False, label="UInt32"), "data": SimTypePointer(SimTypeChar(label="Byte"), offset=0)}, name="FWP_BYTE_BLOB", pack=False, align=None), "eku": SimTypePointer(SimStruct({"numEku": SimTypeInt(signed=False, label="UInt32"), "eku": SimTypePointer(SimTypePointer(SimTypeChar(label="Byte"), offset=0), offset=0)}, name="IKEEXT_CERT_EKUS0", pack=False, align=None), offset=0), "name": SimTypePointer(SimStruct({"nameType": SimTypeInt(signed=False, label="IKEEXT_CERT_CRITERIA_NAME_TYPE"), "certName": SimTypePointer(SimTypeChar(label="Char"), offset=0)}, name="IKEEXT_CERT_NAME0", pack=False, align=None), offset=0), "flags": SimTypeInt(signed=False, label="UInt32")}, name="IKEEXT_CERTIFICATE_CRITERIA0", pack=False, align=None), offset=0)}, name="_Anonymous2_e__Struct", pack=False, align=None), "Anonymous3": SimStruct({"outboundRootStoreArraySize": SimTypeInt(signed=False, label="UInt32"), "outboundTrustedRootStoreCriteria": SimTypePointer(SimStruct({"certData": SimStruct({"size": SimTypeInt(signed=False, label="UInt32"), "data": SimTypePointer(SimTypeChar(label="Byte"), offset=0)}, name="FWP_BYTE_BLOB", pack=False, align=None), "certHash": SimStruct({"size": SimTypeInt(signed=False, label="UInt32"), "data": SimTypePointer(SimTypeChar(label="Byte"), offset=0)}, name="FWP_BYTE_BLOB", pack=False, align=None), "eku": SimTypePointer(SimStruct({"numEku": SimTypeInt(signed=False, label="UInt32"), "eku": SimTypePointer(SimTypePointer(SimTypeChar(label="Byte"), offset=0), offset=0)}, name="IKEEXT_CERT_EKUS0", pack=False, align=None), offset=0), "name": SimTypePointer(SimStruct({"nameType": SimTypeInt(signed=False, label="IKEEXT_CERT_CRITERIA_NAME_TYPE"), "certName": SimTypePointer(SimTypeChar(label="Char"), offset=0)}, name="IKEEXT_CERT_NAME0", pack=False, align=None), offset=0), "flags": SimTypeInt(signed=False, label="UInt32")}, name="IKEEXT_CERTIFICATE_CRITERIA0", pack=False, align=None), offset=0)}, name="_Anonymous3_e__Struct", pack=False, align=None)}, name="<anon>", label="None"), "flags": SimTypeInt(signed=False, label="IKEEXT_CERT_AUTH"), "localCertLocationUrl": SimStruct({"size": SimTypeInt(signed=False, label="UInt32"), "data": SimTypePointer(SimTypeChar(label="Byte"), offset=0)}, name="FWP_BYTE_BLOB", pack=False, align=None)}, name="IKEEXT_CERTIFICATE_AUTHENTICATION2", pack=False, align=None), "cgaAuthentication": SimStruct({"keyContainerName": SimTypePointer(SimTypeChar(label="Char"), offset=0), "cspName": SimTypePointer(SimTypeChar(label="Char"), offset=0), "cspType": SimTypeInt(signed=False, label="UInt32"), "cgaModifier": SimStruct({"byteArray16": SimTypeFixedSizeArray(SimTypeChar(label="Byte"), 16)}, name="FWP_BYTE_ARRAY16", pack=False, align=None), "cgaCollisionCount": SimTypeChar(label="Byte")}, name="IKEEXT_IPV6_CGA_AUTHENTICATION0", pack=False, align=None), "eapAuthentication": SimStruct({"flags": SimTypeInt(signed=False, label="IKEEXT_EAP_AUTHENTICATION_FLAGS")}, name="IKEEXT_EAP_AUTHENTICATION0", pack=False, align=None)}, name="<anon>", label="None")}, name="IKEEXT_AUTHENTICATION_METHOD2", pack=False, align=None), offset=0), "initiatorImpersonationType": SimTypeInt(signed=False, label="IKEEXT_AUTHENTICATION_IMPERSONATION_TYPE")}, name="IKEEXT_EM_POLICY2", pack=False, align=None), offset=0), "fwdPathSaLifetime": SimTypeInt(signed=False, label="UInt32")}, name="IPSEC_TUNNEL_POLICY2", pack=False, align=None), offset=0), "ikeMmPolicy": SimTypePointer(SimStruct({"softExpirationTime": SimTypeInt(signed=False, label="UInt32"), "numAuthenticationMethods": SimTypeInt(signed=False, label="UInt32"), "authenticationMethods": SimTypePointer(SimStruct({"authenticationMethodType": SimTypeInt(signed=False, label="IKEEXT_AUTHENTICATION_METHOD_TYPE"), "Anonymous": SimUnion({"presharedKeyAuthentication": SimStruct({"presharedKey": SimStruct({"size": SimTypeInt(signed=False, label="UInt32"), "data": SimTypePointer(SimTypeChar(label="Byte"), offset=0)}, name="FWP_BYTE_BLOB", pack=False, align=None), "flags": SimTypeInt(signed=False, label="IKEEXT_PRESHARED_KEY_AUTHENTICATION_FLAGS")}, name="IKEEXT_PRESHARED_KEY_AUTHENTICATION1", pack=False, align=None), "certificateAuthentication": SimStruct({"inboundConfigType": SimTypeInt(signed=False, label="IKEEXT_CERT_CONFIG_TYPE"), "Anonymous1": SimUnion({"Anonymous1": SimStruct({"inboundRootArraySize": SimTypeInt(signed=False, label="UInt32"), "inboundRootCriteria": SimTypePointer(SimStruct({"certData": SimStruct({"size": SimTypeInt(signed=False, label="UInt32"), "data": SimTypePointer(SimTypeChar(label="Byte"), offset=0)}, name="FWP_BYTE_BLOB", pack=False, align=None), "certHash": SimStruct({"size": SimTypeInt(signed=False, label="UInt32"), "data": SimTypePointer(SimTypeChar(label="Byte"), offset=0)}, name="FWP_BYTE_BLOB", pack=False, align=None), "eku": SimTypePointer(SimStruct({"numEku": SimTypeInt(signed=False, label="UInt32"), "eku": SimTypePointer(SimTypePointer(SimTypeChar(label="Byte"), offset=0), offset=0)}, name="IKEEXT_CERT_EKUS0", pack=False, align=None), offset=0), "name": SimTypePointer(SimStruct({"nameType": SimTypeInt(signed=False, label="IKEEXT_CERT_CRITERIA_NAME_TYPE"), "certName": SimTypePointer(SimTypeChar(label="Char"), offset=0)}, name="IKEEXT_CERT_NAME0", pack=False, align=None), offset=0), "flags": SimTypeInt(signed=False, label="UInt32")}, name="IKEEXT_CERTIFICATE_CRITERIA0", pack=False, align=None), offset=0)}, name="_Anonymous1_e__Struct", pack=False, align=None), "Anonymous2": SimStruct({"inboundEnterpriseStoreArraySize": SimTypeInt(signed=False, label="UInt32"), "inboundEnterpriseStoreCriteria": SimTypePointer(SimStruct({"certData": SimStruct({"size": SimTypeInt(signed=False, label="UInt32"), "data": SimTypePointer(SimTypeChar(label="Byte"), offset=0)}, name="FWP_BYTE_BLOB", pack=False, align=None), "certHash": SimStruct({"size": SimTypeInt(signed=False, label="UInt32"), "data": SimTypePointer(SimTypeChar(label="Byte"), offset=0)}, name="FWP_BYTE_BLOB", pack=False, align=None), "eku": SimTypePointer(SimStruct({"numEku": SimTypeInt(signed=False, label="UInt32"), "eku": SimTypePointer(SimTypePointer(SimTypeChar(label="Byte"), offset=0), offset=0)}, name="IKEEXT_CERT_EKUS0", pack=False, align=None), offset=0), "name": SimTypePointer(SimStruct({"nameType": SimTypeInt(signed=False, label="IKEEXT_CERT_CRITERIA_NAME_TYPE"), "certName": SimTypePointer(SimTypeChar(label="Char"), offset=0)}, name="IKEEXT_CERT_NAME0", pack=False, align=None), offset=0), "flags": SimTypeInt(signed=False, label="UInt32")}, name="IKEEXT_CERTIFICATE_CRITERIA0", pack=False, align=None), offset=0)}, name="_Anonymous2_e__Struct", pack=False, align=None), "Anonymous3": SimStruct({"inboundRootStoreArraySize": SimTypeInt(signed=False, label="UInt32"), "inboundTrustedRootStoreCriteria": SimTypePointer(SimStruct({"certData": SimStruct({"size": SimTypeInt(signed=False, label="UInt32"), "data": SimTypePointer(SimTypeChar(label="Byte"), offset=0)}, name="FWP_BYTE_BLOB", pack=False, align=None), "certHash": SimStruct({"size": SimTypeInt(signed=False, label="UInt32"), "data": SimTypePointer(SimTypeChar(label="Byte"), offset=0)}, name="FWP_BYTE_BLOB", pack=False, align=None), "eku": SimTypePointer(SimStruct({"numEku": SimTypeInt(signed=False, label="UInt32"), "eku": SimTypePointer(SimTypePointer(SimTypeChar(label="Byte"), offset=0), offset=0)}, name="IKEEXT_CERT_EKUS0", pack=False, align=None), offset=0), "name": SimTypePointer(SimStruct({"nameType": SimTypeInt(signed=False, label="IKEEXT_CERT_CRITERIA_NAME_TYPE"), "certName": SimTypePointer(SimTypeChar(label="Char"), offset=0)}, name="IKEEXT_CERT_NAME0", pack=False, align=None), offset=0), "flags": SimTypeInt(signed=False, label="UInt32")}, name="IKEEXT_CERTIFICATE_CRITERIA0", pack=False, align=None), offset=0)}, name="_Anonymous3_e__Struct", pack=False, align=None)}, name="<anon>", label="None"), "outboundConfigType": SimTypeInt(signed=False, label="IKEEXT_CERT_CONFIG_TYPE"), "Anonymous2": SimUnion({"Anonymous1": SimStruct({"outboundRootArraySize": SimTypeInt(signed=False, label="UInt32"), "outboundRootCriteria": SimTypePointer(SimStruct({"certData": SimStruct({"size": SimTypeInt(signed=False, label="UInt32"), "data": SimTypePointer(SimTypeChar(label="Byte"), offset=0)}, name="FWP_BYTE_BLOB", pack=False, align=None), "certHash": SimStruct({"size": SimTypeInt(signed=False, label="UInt32"), "data": SimTypePointer(SimTypeChar(label="Byte"), offset=0)}, name="FWP_BYTE_BLOB", pack=False, align=None), "eku": SimTypePointer(SimStruct({"numEku": SimTypeInt(signed=False, label="UInt32"), "eku": SimTypePointer(SimTypePointer(SimTypeChar(label="Byte"), offset=0), offset=0)}, name="IKEEXT_CERT_EKUS0", pack=False, align=None), offset=0), "name": SimTypePointer(SimStruct({"nameType": SimTypeInt(signed=False, label="IKEEXT_CERT_CRITERIA_NAME_TYPE"), "certName": SimTypePointer(SimTypeChar(label="Char"), offset=0)}, name="IKEEXT_CERT_NAME0", pack=False, align=None), offset=0), "flags": SimTypeInt(signed=False, label="UInt32")}, name="IKEEXT_CERTIFICATE_CRITERIA0", pack=False, align=None), offset=0)}, name="_Anonymous1_e__Struct", pack=False, align=None), "Anonymous2": SimStruct({"outboundEnterpriseStoreArraySize": SimTypeInt(signed=False, label="UInt32"), "outboundEnterpriseStoreCriteria": SimTypePointer(SimStruct({"certData": SimStruct({"size": SimTypeInt(signed=False, label="UInt32"), "data": SimTypePointer(SimTypeChar(label="Byte"), offset=0)}, name="FWP_BYTE_BLOB", pack=False, align=None), "certHash": SimStruct({"size": SimTypeInt(signed=False, label="UInt32"), "data": SimTypePointer(SimTypeChar(label="Byte"), offset=0)}, name="FWP_BYTE_BLOB", pack=False, align=None), "eku": SimTypePointer(SimStruct({"numEku": SimTypeInt(signed=False, label="UInt32"), "eku": SimTypePointer(SimTypePointer(SimTypeChar(label="Byte"), offset=0), offset=0)}, name="IKEEXT_CERT_EKUS0", pack=False, align=None), offset=0), "name": SimTypePointer(SimStruct({"nameType": SimTypeInt(signed=False, label="IKEEXT_CERT_CRITERIA_NAME_TYPE"), "certName": SimTypePointer(SimTypeChar(label="Char"), offset=0)}, name="IKEEXT_CERT_NAME0", pack=False, align=None), offset=0), "flags": SimTypeInt(signed=False, label="UInt32")}, name="IKEEXT_CERTIFICATE_CRITERIA0", pack=False, align=None), offset=0)}, name="_Anonymous2_e__Struct", pack=False, align=None), "Anonymous3": SimStruct({"outboundRootStoreArraySize": SimTypeInt(signed=False, label="UInt32"), "outboundTrustedRootStoreCriteria": SimTypePointer(SimStruct({"certData": SimStruct({"size": SimTypeInt(signed=False, label="UInt32"), "data": SimTypePointer(SimTypeChar(label="Byte"), offset=0)}, name="FWP_BYTE_BLOB", pack=False, align=None), "certHash": SimStruct({"size": SimTypeInt(signed=False, label="UInt32"), "data": SimTypePointer(SimTypeChar(label="Byte"), offset=0)}, name="FWP_BYTE_BLOB", pack=False, align=None), "eku": SimTypePointer(SimStruct({"numEku": SimTypeInt(signed=False, label="UInt32"), "eku": SimTypePointer(SimTypePointer(SimTypeChar(label="Byte"), offset=0), offset=0)}, name="IKEEXT_CERT_EKUS0", pack=False, align=None), offset=0), "name": SimTypePointer(SimStruct({"nameType": SimTypeInt(signed=False, label="IKEEXT_CERT_CRITERIA_NAME_TYPE"), "certName": SimTypePointer(SimTypeChar(label="Char"), offset=0)}, name="IKEEXT_CERT_NAME0", pack=False, align=None), offset=0), "flags": SimTypeInt(signed=False, label="UInt32")}, name="IKEEXT_CERTIFICATE_CRITERIA0", pack=False, align=None), offset=0)}, name="_Anonymous3_e__Struct", pack=False, align=None)}, name="<anon>", label="None"), "flags": SimTypeInt(signed=False, label="IKEEXT_CERT_AUTH"), "localCertLocationUrl": SimStruct({"size": SimTypeInt(signed=False, label="UInt32"), "data": SimTypePointer(SimTypeChar(label="Byte"), offset=0)}, name="FWP_BYTE_BLOB", pack=False, align=None)}, name="IKEEXT_CERTIFICATE_AUTHENTICATION2", pack=False, align=None), "kerberosAuthentication": SimStruct({"flags": SimTypeInt(signed=False, label="IKEEXT_KERBEROS_AUTHENTICATION_FLAGS"), "proxyServer": SimTypePointer(SimTypeChar(label="Char"), offset=0)}, name="IKEEXT_KERBEROS_AUTHENTICATION1", pack=False, align=None), "reservedAuthentication": SimStruct({"flags": SimTypeInt(signed=False, label="IKEEXT_RESERVED_AUTHENTICATION_FLAGS")}, name="IKEEXT_RESERVED_AUTHENTICATION0", pack=False, align=None), "ntlmV2Authentication": SimStruct({"flags": SimTypeInt(signed=False, label="UInt32")}, name="IKEEXT_NTLM_V2_AUTHENTICATION0", pack=False, align=None), "sslAuthentication": SimStruct({"inboundConfigType": SimTypeInt(signed=False, label="IKEEXT_CERT_CONFIG_TYPE"), "Anonymous1": SimUnion({"Anonymous1": SimStruct({"inboundRootArraySize": SimTypeInt(signed=False, label="UInt32"), "inboundRootCriteria": SimTypePointer(SimStruct({"certData": SimStruct({"size": SimTypeInt(signed=False, label="UInt32"), "data": SimTypePointer(SimTypeChar(label="Byte"), offset=0)}, name="FWP_BYTE_BLOB", pack=False, align=None), "certHash": SimStruct({"size": SimTypeInt(signed=False, label="UInt32"), "data": SimTypePointer(SimTypeChar(label="Byte"), offset=0)}, name="FWP_BYTE_BLOB", pack=False, align=None), "eku": SimTypePointer(SimStruct({"numEku": SimTypeInt(signed=False, label="UInt32"), "eku": SimTypePointer(SimTypePointer(SimTypeChar(label="Byte"), offset=0), offset=0)}, name="IKEEXT_CERT_EKUS0", pack=False, align=None), offset=0), "name": SimTypePointer(SimStruct({"nameType": SimTypeInt(signed=False, label="IKEEXT_CERT_CRITERIA_NAME_TYPE"), "certName": SimTypePointer(SimTypeChar(label="Char"), offset=0)}, name="IKEEXT_CERT_NAME0", pack=False, align=None), offset=0), "flags": SimTypeInt(signed=False, label="UInt32")}, name="IKEEXT_CERTIFICATE_CRITERIA0", pack=False, align=None), offset=0)}, name="_Anonymous1_e__Struct", pack=False, align=None), "Anonymous2": SimStruct({"inboundEnterpriseStoreArraySize": SimTypeInt(signed=False, label="UInt32"), "inboundEnterpriseStoreCriteria": SimTypePointer(SimStruct({"certData": SimStruct({"size": SimTypeInt(signed=False, label="UInt32"), "data": SimTypePointer(SimTypeChar(label="Byte"), offset=0)}, name="FWP_BYTE_BLOB", pack=False, align=None), "certHash": SimStruct({"size": SimTypeInt(signed=False, label="UInt32"), "data": SimTypePointer(SimTypeChar(label="Byte"), offset=0)}, name="FWP_BYTE_BLOB", pack=False, align=None), "eku": SimTypePointer(SimStruct({"numEku": SimTypeInt(signed=False, label="UInt32"), "eku": SimTypePointer(SimTypePointer(SimTypeChar(label="Byte"), offset=0), offset=0)}, name="IKEEXT_CERT_EKUS0", pack=False, align=None), offset=0), "name": SimTypePointer(SimStruct({"nameType": SimTypeInt(signed=False, label="IKEEXT_CERT_CRITERIA_NAME_TYPE"), "certName": SimTypePointer(SimTypeChar(label="Char"), offset=0)}, name="IKEEXT_CERT_NAME0", pack=False, align=None), offset=0), "flags": SimTypeInt(signed=False, label="UInt32")}, name="IKEEXT_CERTIFICATE_CRITERIA0", pack=False, align=None), offset=0)}, name="_Anonymous2_e__Struct", pack=False, align=None), "Anonymous3": SimStruct({"inboundRootStoreArraySize": SimTypeInt(signed=False, label="UInt32"), "inboundTrustedRootStoreCriteria": SimTypePointer(SimStruct({"certData": SimStruct({"size": SimTypeInt(signed=False, label="UInt32"), "data": SimTypePointer(SimTypeChar(label="Byte"), offset=0)}, name="FWP_BYTE_BLOB", pack=False, align=None), "certHash": SimStruct({"size": SimTypeInt(signed=False, label="UInt32"), "data": SimTypePointer(SimTypeChar(label="Byte"), offset=0)}, name="FWP_BYTE_BLOB", pack=False, align=None), "eku": SimTypePointer(SimStruct({"numEku": SimTypeInt(signed=False, label="UInt32"), "eku": SimTypePointer(SimTypePointer(SimTypeChar(label="Byte"), offset=0), offset=0)}, name="IKEEXT_CERT_EKUS0", pack=False, align=None), offset=0), "name": SimTypePointer(SimStruct({"nameType": SimTypeInt(signed=False, label="IKEEXT_CERT_CRITERIA_NAME_TYPE"), "certName": SimTypePointer(SimTypeChar(label="Char"), offset=0)}, name="IKEEXT_CERT_NAME0", pack=False, align=None), offset=0), "flags": SimTypeInt(signed=False, label="UInt32")}, name="IKEEXT_CERTIFICATE_CRITERIA0", pack=False, align=None), offset=0)}, name="_Anonymous3_e__Struct", pack=False, align=None)}, name="<anon>", label="None"), "outboundConfigType": SimTypeInt(signed=False, label="IKEEXT_CERT_CONFIG_TYPE"), "Anonymous2": SimUnion({"Anonymous1": SimStruct({"outboundRootArraySize": SimTypeInt(signed=False, label="UInt32"), "outboundRootCriteria": SimTypePointer(SimStruct({"certData": SimStruct({"size": SimTypeInt(signed=False, label="UInt32"), "data": SimTypePointer(SimTypeChar(label="Byte"), offset=0)}, name="FWP_BYTE_BLOB", pack=False, align=None), "certHash": SimStruct({"size": SimTypeInt(signed=False, label="UInt32"), "data": SimTypePointer(SimTypeChar(label="Byte"), offset=0)}, name="FWP_BYTE_BLOB", pack=False, align=None), "eku": SimTypePointer(SimStruct({"numEku": SimTypeInt(signed=False, label="UInt32"), "eku": SimTypePointer(SimTypePointer(SimTypeChar(label="Byte"), offset=0), offset=0)}, name="IKEEXT_CERT_EKUS0", pack=False, align=None), offset=0), "name": SimTypePointer(SimStruct({"nameType": SimTypeInt(signed=False, label="IKEEXT_CERT_CRITERIA_NAME_TYPE"), "certName": SimTypePointer(SimTypeChar(label="Char"), offset=0)}, name="IKEEXT_CERT_NAME0", pack=False, align=None), offset=0), "flags": SimTypeInt(signed=False, label="UInt32")}, name="IKEEXT_CERTIFICATE_CRITERIA0", pack=False, align=None), offset=0)}, name="_Anonymous1_e__Struct", pack=False, align=None), "Anonymous2": SimStruct({"outboundEnterpriseStoreArraySize": SimTypeInt(signed=False, label="UInt32"), "outboundEnterpriseStoreCriteria": SimTypePointer(SimStruct({"certData": SimStruct({"size": SimTypeInt(signed=False, label="UInt32"), "data": SimTypePointer(SimTypeChar(label="Byte"), offset=0)}, name="FWP_BYTE_BLOB", pack=False, align=None), "certHash": SimStruct({"size": SimTypeInt(signed=False, label="UInt32"), "data": SimTypePointer(SimTypeChar(label="Byte"), offset=0)}, name="FWP_BYTE_BLOB", pack=False, align=None), "eku": SimTypePointer(SimStruct({"numEku": SimTypeInt(signed=False, label="UInt32"), "eku": SimTypePointer(SimTypePointer(SimTypeChar(label="Byte"), offset=0), offset=0)}, name="IKEEXT_CERT_EKUS0", pack=False, align=None), offset=0), "name": SimTypePointer(SimStruct({"nameType": SimTypeInt(signed=False, label="IKEEXT_CERT_CRITERIA_NAME_TYPE"), "certName": SimTypePointer(SimTypeChar(label="Char"), offset=0)}, name="IKEEXT_CERT_NAME0", pack=False, align=None), offset=0), "flags": SimTypeInt(signed=False, label="UInt32")}, name="IKEEXT_CERTIFICATE_CRITERIA0", pack=False, align=None), offset=0)}, name="_Anonymous2_e__Struct", pack=False, align=None), "Anonymous3": SimStruct({"outboundRootStoreArraySize": SimTypeInt(signed=False, label="UInt32"), "outboundTrustedRootStoreCriteria": SimTypePointer(SimStruct({"certData": SimStruct({"size": SimTypeInt(signed=False, label="UInt32"), "data": SimTypePointer(SimTypeChar(label="Byte"), offset=0)}, name="FWP_BYTE_BLOB", pack=False, align=None), "certHash": SimStruct({"size": SimTypeInt(signed=False, label="UInt32"), "data": SimTypePointer(SimTypeChar(label="Byte"), offset=0)}, name="FWP_BYTE_BLOB", pack=False, align=None), "eku": SimTypePointer(SimStruct({"numEku": SimTypeInt(signed=False, label="UInt32"), "eku": SimTypePointer(SimTypePointer(SimTypeChar(label="Byte"), offset=0), offset=0)}, name="IKEEXT_CERT_EKUS0", pack=False, align=None), offset=0), "name": SimTypePointer(SimStruct({"nameType": SimTypeInt(signed=False, label="IKEEXT_CERT_CRITERIA_NAME_TYPE"), "certName": SimTypePointer(SimTypeChar(label="Char"), offset=0)}, name="IKEEXT_CERT_NAME0", pack=False, align=None), offset=0), "flags": SimTypeInt(signed=False, label="UInt32")}, name="IKEEXT_CERTIFICATE_CRITERIA0", pack=False, align=None), offset=0)}, name="_Anonymous3_e__Struct", pack=False, align=None)}, name="<anon>", label="None"), "flags": SimTypeInt(signed=False, label="IKEEXT_CERT_AUTH"), "localCertLocationUrl": SimStruct({"size": SimTypeInt(signed=False, label="UInt32"), "data": SimTypePointer(SimTypeChar(label="Byte"), offset=0)}, name="FWP_BYTE_BLOB", pack=False, align=None)}, name="IKEEXT_CERTIFICATE_AUTHENTICATION2", pack=False, align=None), "cgaAuthentication": SimStruct({"keyContainerName": SimTypePointer(SimTypeChar(label="Char"), offset=0), "cspName": SimTypePointer(SimTypeChar(label="Char"), offset=0), "cspType": SimTypeInt(signed=False, label="UInt32"), "cgaModifier": SimStruct({"byteArray16": SimTypeFixedSizeArray(SimTypeChar(label="Byte"), 16)}, name="FWP_BYTE_ARRAY16", pack=False, align=None), "cgaCollisionCount": SimTypeChar(label="Byte")}, name="IKEEXT_IPV6_CGA_AUTHENTICATION0", pack=False, align=None), "eapAuthentication": SimStruct({"flags": SimTypeInt(signed=False, label="IKEEXT_EAP_AUTHENTICATION_FLAGS")}, name="IKEEXT_EAP_AUTHENTICATION0", pack=False, align=None)}, name="<anon>", label="None")}, name="IKEEXT_AUTHENTICATION_METHOD2", pack=False, align=None), offset=0), "initiatorImpersonationType": SimTypeInt(signed=False, label="IKEEXT_AUTHENTICATION_IMPERSONATION_TYPE"), "numIkeProposals": SimTypeInt(signed=False, label="UInt32"), "ikeProposals": SimTypePointer(SimStruct({"cipherAlgorithm": SimStruct({"algoIdentifier": SimTypeInt(signed=False, label="IKEEXT_CIPHER_TYPE"), "keyLen": SimTypeInt(signed=False, label="UInt32"), "rounds": SimTypeInt(signed=False, label="UInt32")}, name="IKEEXT_CIPHER_ALGORITHM0", pack=False, align=None), "integrityAlgorithm": SimStruct({"algoIdentifier": SimTypeInt(signed=False, label="IKEEXT_INTEGRITY_TYPE")}, name="IKEEXT_INTEGRITY_ALGORITHM0", pack=False, align=None), "maxLifetimeSeconds": SimTypeInt(signed=False, label="UInt32"), "dhGroup": SimTypeInt(signed=False, label="IKEEXT_DH_GROUP"), "quickModeLimit": SimTypeInt(signed=False, label="UInt32")}, name="IKEEXT_PROPOSAL0", pack=False, align=None), offset=0), "flags": SimTypeInt(signed=False, label="IKEEXT_POLICY_FLAG"), "maxDynamicFilters": SimTypeInt(signed=False, label="UInt32"), "retransmitDurationSecs": SimTypeInt(signed=False, label="UInt32")}, name="IKEEXT_POLICY2", pack=False, align=None), offset=0), "authIpMmPolicy": SimTypePointer(SimStruct({"softExpirationTime": SimTypeInt(signed=False, label="UInt32"), "numAuthenticationMethods": SimTypeInt(signed=False, label="UInt32"), "authenticationMethods": SimTypePointer(SimStruct({"authenticationMethodType": SimTypeInt(signed=False, label="IKEEXT_AUTHENTICATION_METHOD_TYPE"), "Anonymous": SimUnion({"presharedKeyAuthentication": SimStruct({"presharedKey": SimStruct({"size": SimTypeInt(signed=False, label="UInt32"), "data": SimTypePointer(SimTypeChar(label="Byte"), offset=0)}, name="FWP_BYTE_BLOB", pack=False, align=None), "flags": SimTypeInt(signed=False, label="IKEEXT_PRESHARED_KEY_AUTHENTICATION_FLAGS")}, name="IKEEXT_PRESHARED_KEY_AUTHENTICATION1", pack=False, align=None), "certificateAuthentication": SimStruct({"inboundConfigType": SimTypeInt(signed=False, label="IKEEXT_CERT_CONFIG_TYPE"), "Anonymous1": SimUnion({"Anonymous1": SimStruct({"inboundRootArraySize": SimTypeInt(signed=False, label="UInt32"), "inboundRootCriteria": SimTypePointer(SimStruct({"certData": SimStruct({"size": SimTypeInt(signed=False, label="UInt32"), "data": SimTypePointer(SimTypeChar(label="Byte"), offset=0)}, name="FWP_BYTE_BLOB", pack=False, align=None), "certHash": SimStruct({"size": SimTypeInt(signed=False, label="UInt32"), "data": SimTypePointer(SimTypeChar(label="Byte"), offset=0)}, name="FWP_BYTE_BLOB", pack=False, align=None), "eku": SimTypePointer(SimStruct({"numEku": SimTypeInt(signed=False, label="UInt32"), "eku": SimTypePointer(SimTypePointer(SimTypeChar(label="Byte"), offset=0), offset=0)}, name="IKEEXT_CERT_EKUS0", pack=False, align=None), offset=0), "name": SimTypePointer(SimStruct({"nameType": SimTypeInt(signed=False, label="IKEEXT_CERT_CRITERIA_NAME_TYPE"), "certName": SimTypePointer(SimTypeChar(label="Char"), offset=0)}, name="IKEEXT_CERT_NAME0", pack=False, align=None), offset=0), "flags": SimTypeInt(signed=False, label="UInt32")}, name="IKEEXT_CERTIFICATE_CRITERIA0", pack=False, align=None), offset=0)}, name="_Anonymous1_e__Struct", pack=False, align=None), "Anonymous2": SimStruct({"inboundEnterpriseStoreArraySize": SimTypeInt(signed=False, label="UInt32"), "inboundEnterpriseStoreCriteria": SimTypePointer(SimStruct({"certData": SimStruct({"size": SimTypeInt(signed=False, label="UInt32"), "data": SimTypePointer(SimTypeChar(label="Byte"), offset=0)}, name="FWP_BYTE_BLOB", pack=False, align=None), "certHash": SimStruct({"size": SimTypeInt(signed=False, label="UInt32"), "data": SimTypePointer(SimTypeChar(label="Byte"), offset=0)}, name="FWP_BYTE_BLOB", pack=False, align=None), "eku": SimTypePointer(SimStruct({"numEku": SimTypeInt(signed=False, label="UInt32"), "eku": SimTypePointer(SimTypePointer(SimTypeChar(label="Byte"), offset=0), offset=0)}, name="IKEEXT_CERT_EKUS0", pack=False, align=None), offset=0), "name": SimTypePointer(SimStruct({"nameType": SimTypeInt(signed=False, label="IKEEXT_CERT_CRITERIA_NAME_TYPE"), "certName": SimTypePointer(SimTypeChar(label="Char"), offset=0)}, name="IKEEXT_CERT_NAME0", pack=False, align=None), offset=0), "flags": SimTypeInt(signed=False, label="UInt32")}, name="IKEEXT_CERTIFICATE_CRITERIA0", pack=False, align=None), offset=0)}, name="_Anonymous2_e__Struct", pack=False, align=None), "Anonymous3": SimStruct({"inboundRootStoreArraySize": SimTypeInt(signed=False, label="UInt32"), "inboundTrustedRootStoreCriteria": SimTypePointer(SimStruct({"certData": SimStruct({"size": SimTypeInt(signed=False, label="UInt32"), "data": SimTypePointer(SimTypeChar(label="Byte"), offset=0)}, name="FWP_BYTE_BLOB", pack=False, align=None), "certHash": SimStruct({"size": SimTypeInt(signed=False, label="UInt32"), "data": SimTypePointer(SimTypeChar(label="Byte"), offset=0)}, name="FWP_BYTE_BLOB", pack=False, align=None), "eku": SimTypePointer(SimStruct({"numEku": SimTypeInt(signed=False, label="UInt32"), "eku": SimTypePointer(SimTypePointer(SimTypeChar(label="Byte"), offset=0), offset=0)}, name="IKEEXT_CERT_EKUS0", pack=False, align=None), offset=0), "name": SimTypePointer(SimStruct({"nameType": SimTypeInt(signed=False, label="IKEEXT_CERT_CRITERIA_NAME_TYPE"), "certName": SimTypePointer(SimTypeChar(label="Char"), offset=0)}, name="IKEEXT_CERT_NAME0", pack=False, align=None), offset=0), "flags": SimTypeInt(signed=False, label="UInt32")}, name="IKEEXT_CERTIFICATE_CRITERIA0", pack=False, align=None), offset=0)}, name="_Anonymous3_e__Struct", pack=False, align=None)}, name="<anon>", label="None"), "outboundConfigType": SimTypeInt(signed=False, label="IKEEXT_CERT_CONFIG_TYPE"), "Anonymous2": SimUnion({"Anonymous1": SimStruct({"outboundRootArraySize": SimTypeInt(signed=False, label="UInt32"), "outboundRootCriteria": SimTypePointer(SimStruct({"certData": SimStruct({"size": SimTypeInt(signed=False, label="UInt32"), "data": SimTypePointer(SimTypeChar(label="Byte"), offset=0)}, name="FWP_BYTE_BLOB", pack=False, align=None), "certHash": SimStruct({"size": SimTypeInt(signed=False, label="UInt32"), "data": SimTypePointer(SimTypeChar(label="Byte"), offset=0)}, name="FWP_BYTE_BLOB", pack=False, align=None), "eku": SimTypePointer(SimStruct({"numEku": SimTypeInt(signed=False, label="UInt32"), "eku": SimTypePointer(SimTypePointer(SimTypeChar(label="Byte"), offset=0), offset=0)}, name="IKEEXT_CERT_EKUS0", pack=False, align=None), offset=0), "name": SimTypePointer(SimStruct({"nameType": SimTypeInt(signed=False, label="IKEEXT_CERT_CRITERIA_NAME_TYPE"), "certName": SimTypePointer(SimTypeChar(label="Char"), offset=0)}, name="IKEEXT_CERT_NAME0", pack=False, align=None), offset=0), "flags": SimTypeInt(signed=False, label="UInt32")}, name="IKEEXT_CERTIFICATE_CRITERIA0", pack=False, align=None), offset=0)}, name="_Anonymous1_e__Struct", pack=False, align=None), "Anonymous2": SimStruct({"outboundEnterpriseStoreArraySize": SimTypeInt(signed=False, label="UInt32"), "outboundEnterpriseStoreCriteria": SimTypePointer(SimStruct({"certData": SimStruct({"size": SimTypeInt(signed=False, label="UInt32"), "data": SimTypePointer(SimTypeChar(label="Byte"), offset=0)}, name="FWP_BYTE_BLOB", pack=False, align=None), "certHash": SimStruct({"size": SimTypeInt(signed=False, label="UInt32"), "data": SimTypePointer(SimTypeChar(label="Byte"), offset=0)}, name="FWP_BYTE_BLOB", pack=False, align=None), "eku": SimTypePointer(SimStruct({"numEku": SimTypeInt(signed=False, label="UInt32"), "eku": SimTypePointer(SimTypePointer(SimTypeChar(label="Byte"), offset=0), offset=0)}, name="IKEEXT_CERT_EKUS0", pack=False, align=None), offset=0), "name": SimTypePointer(SimStruct({"nameType": SimTypeInt(signed=False, label="IKEEXT_CERT_CRITERIA_NAME_TYPE"), "certName": SimTypePointer(SimTypeChar(label="Char"), offset=0)}, name="IKEEXT_CERT_NAME0", pack=False, align=None), offset=0), "flags": SimTypeInt(signed=False, label="UInt32")}, name="IKEEXT_CERTIFICATE_CRITERIA0", pack=False, align=None), offset=0)}, name="_Anonymous2_e__Struct", pack=False, align=None), "Anonymous3": SimStruct({"outboundRootStoreArraySize": SimTypeInt(signed=False, label="UInt32"), "outboundTrustedRootStoreCriteria": SimTypePointer(SimStruct({"certData": SimStruct({"size": SimTypeInt(signed=False, label="UInt32"), "data": SimTypePointer(SimTypeChar(label="Byte"), offset=0)}, name="FWP_BYTE_BLOB", pack=False, align=None), "certHash": SimStruct({"size": SimTypeInt(signed=False, label="UInt32"), "data": SimTypePointer(SimTypeChar(label="Byte"), offset=0)}, name="FWP_BYTE_BLOB", pack=False, align=None), "eku": SimTypePointer(SimStruct({"numEku": SimTypeInt(signed=False, label="UInt32"), "eku": SimTypePointer(SimTypePointer(SimTypeChar(label="Byte"), offset=0), offset=0)}, name="IKEEXT_CERT_EKUS0", pack=False, align=None), offset=0), "name": SimTypePointer(SimStruct({"nameType": SimTypeInt(signed=False, label="IKEEXT_CERT_CRITERIA_NAME_TYPE"), "certName": SimTypePointer(SimTypeChar(label="Char"), offset=0)}, name="IKEEXT_CERT_NAME0", pack=False, align=None), offset=0), "flags": SimTypeInt(signed=False, label="UInt32")}, name="IKEEXT_CERTIFICATE_CRITERIA0", pack=False, align=None), offset=0)}, name="_Anonymous3_e__Struct", pack=False, align=None)}, name="<anon>", label="None"), "flags": SimTypeInt(signed=False, label="IKEEXT_CERT_AUTH"), "localCertLocationUrl": SimStruct({"size": SimTypeInt(signed=False, label="UInt32"), "data": SimTypePointer(SimTypeChar(label="Byte"), offset=0)}, name="FWP_BYTE_BLOB", pack=False, align=None)}, name="IKEEXT_CERTIFICATE_AUTHENTICATION2", pack=False, align=None), "kerberosAuthentication": SimStruct({"flags": SimTypeInt(signed=False, label="IKEEXT_KERBEROS_AUTHENTICATION_FLAGS"), "proxyServer": SimTypePointer(SimTypeChar(label="Char"), offset=0)}, name="IKEEXT_KERBEROS_AUTHENTICATION1", pack=False, align=None), "reservedAuthentication": SimStruct({"flags": SimTypeInt(signed=False, label="IKEEXT_RESERVED_AUTHENTICATION_FLAGS")}, name="IKEEXT_RESERVED_AUTHENTICATION0", pack=False, align=None), "ntlmV2Authentication": SimStruct({"flags": SimTypeInt(signed=False, label="UInt32")}, name="IKEEXT_NTLM_V2_AUTHENTICATION0", pack=False, align=None), "sslAuthentication": SimStruct({"inboundConfigType": SimTypeInt(signed=False, label="IKEEXT_CERT_CONFIG_TYPE"), "Anonymous1": SimUnion({"Anonymous1": SimStruct({"inboundRootArraySize": SimTypeInt(signed=False, label="UInt32"), "inboundRootCriteria": SimTypePointer(SimStruct({"certData": SimStruct({"size": SimTypeInt(signed=False, label="UInt32"), "data": SimTypePointer(SimTypeChar(label="Byte"), offset=0)}, name="FWP_BYTE_BLOB", pack=False, align=None), "certHash": SimStruct({"size": SimTypeInt(signed=False, label="UInt32"), "data": SimTypePointer(SimTypeChar(label="Byte"), offset=0)}, name="FWP_BYTE_BLOB", pack=False, align=None), "eku": SimTypePointer(SimStruct({"numEku": SimTypeInt(signed=False, label="UInt32"), "eku": SimTypePointer(SimTypePointer(SimTypeChar(label="Byte"), offset=0), offset=0)}, name="IKEEXT_CERT_EKUS0", pack=False, align=None), offset=0), "name": SimTypePointer(SimStruct({"nameType": SimTypeInt(signed=False, label="IKEEXT_CERT_CRITERIA_NAME_TYPE"), "certName": SimTypePointer(SimTypeChar(label="Char"), offset=0)}, name="IKEEXT_CERT_NAME0", pack=False, align=None), offset=0), "flags": SimTypeInt(signed=False, label="UInt32")}, name="IKEEXT_CERTIFICATE_CRITERIA0", pack=False, align=None), offset=0)}, name="_Anonymous1_e__Struct", pack=False, align=None), "Anonymous2": SimStruct({"inboundEnterpriseStoreArraySize": SimTypeInt(signed=False, label="UInt32"), "inboundEnterpriseStoreCriteria": SimTypePointer(SimStruct({"certData": SimStruct({"size": SimTypeInt(signed=False, label="UInt32"), "data": SimTypePointer(SimTypeChar(label="Byte"), offset=0)}, name="FWP_BYTE_BLOB", pack=False, align=None), "certHash": SimStruct({"size": SimTypeInt(signed=False, label="UInt32"), "data": SimTypePointer(SimTypeChar(label="Byte"), offset=0)}, name="FWP_BYTE_BLOB", pack=False, align=None), "eku": SimTypePointer(SimStruct({"numEku": SimTypeInt(signed=False, label="UInt32"), "eku": SimTypePointer(SimTypePointer(SimTypeChar(label="Byte"), offset=0), offset=0)}, name="IKEEXT_CERT_EKUS0", pack=False, align=None), offset=0), "name": SimTypePointer(SimStruct({"nameType": SimTypeInt(signed=False, label="IKEEXT_CERT_CRITERIA_NAME_TYPE"), "certName": SimTypePointer(SimTypeChar(label="Char"), offset=0)}, name="IKEEXT_CERT_NAME0", pack=False, align=None), offset=0), "flags": SimTypeInt(signed=False, label="UInt32")}, name="IKEEXT_CERTIFICATE_CRITERIA0", pack=False, align=None), offset=0)}, name="_Anonymous2_e__Struct", pack=False, align=None), "Anonymous3": SimStruct({"inboundRootStoreArraySize": SimTypeInt(signed=False, label="UInt32"), "inboundTrustedRootStoreCriteria": SimTypePointer(SimStruct({"certData": SimStruct({"size": SimTypeInt(signed=False, label="UInt32"), "data": SimTypePointer(SimTypeChar(label="Byte"), offset=0)}, name="FWP_BYTE_BLOB", pack=False, align=None), "certHash": SimStruct({"size": SimTypeInt(signed=False, label="UInt32"), "data": SimTypePointer(SimTypeChar(label="Byte"), offset=0)}, name="FWP_BYTE_BLOB", pack=False, align=None), "eku": SimTypePointer(SimStruct({"numEku": SimTypeInt(signed=False, label="UInt32"), "eku": SimTypePointer(SimTypePointer(SimTypeChar(label="Byte"), offset=0), offset=0)}, name="IKEEXT_CERT_EKUS0", pack=False, align=None), offset=0), "name": SimTypePointer(SimStruct({"nameType": SimTypeInt(signed=False, label="IKEEXT_CERT_CRITERIA_NAME_TYPE"), "certName": SimTypePointer(SimTypeChar(label="Char"), offset=0)}, name="IKEEXT_CERT_NAME0", pack=False, align=None), offset=0), "flags": SimTypeInt(signed=False, label="UInt32")}, name="IKEEXT_CERTIFICATE_CRITERIA0", pack=False, align=None), offset=0)}, name="_Anonymous3_e__Struct", pack=False, align=None)}, name="<anon>", label="None"), "outboundConfigType": SimTypeInt(signed=False, label="IKEEXT_CERT_CONFIG_TYPE"), "Anonymous2": SimUnion({"Anonymous1": SimStruct({"outboundRootArraySize": SimTypeInt(signed=False, label="UInt32"), "outboundRootCriteria": SimTypePointer(SimStruct({"certData": SimStruct({"size": SimTypeInt(signed=False, label="UInt32"), "data": SimTypePointer(SimTypeChar(label="Byte"), offset=0)}, name="FWP_BYTE_BLOB", pack=False, align=None), "certHash": SimStruct({"size": SimTypeInt(signed=False, label="UInt32"), "data": SimTypePointer(SimTypeChar(label="Byte"), offset=0)}, name="FWP_BYTE_BLOB", pack=False, align=None), "eku": SimTypePointer(SimStruct({"numEku": SimTypeInt(signed=False, label="UInt32"), "eku": SimTypePointer(SimTypePointer(SimTypeChar(label="Byte"), offset=0), offset=0)}, name="IKEEXT_CERT_EKUS0", pack=False, align=None), offset=0), "name": SimTypePointer(SimStruct({"nameType": SimTypeInt(signed=False, label="IKEEXT_CERT_CRITERIA_NAME_TYPE"), "certName": SimTypePointer(SimTypeChar(label="Char"), offset=0)}, name="IKEEXT_CERT_NAME0", pack=False, align=None), offset=0), "flags": SimTypeInt(signed=False, label="UInt32")}, name="IKEEXT_CERTIFICATE_CRITERIA0", pack=False, align=None), offset=0)}, name="_Anonymous1_e__Struct", pack=False, align=None), "Anonymous2": SimStruct({"outboundEnterpriseStoreArraySize": SimTypeInt(signed=False, label="UInt32"), "outboundEnterpriseStoreCriteria": SimTypePointer(SimStruct({"certData": SimStruct({"size": SimTypeInt(signed=False, label="UInt32"), "data": SimTypePointer(SimTypeChar(label="Byte"), offset=0)}, name="FWP_BYTE_BLOB", pack=False, align=None), "certHash": SimStruct({"size": SimTypeInt(signed=False, label="UInt32"), "data": SimTypePointer(SimTypeChar(label="Byte"), offset=0)}, name="FWP_BYTE_BLOB", pack=False, align=None), "eku": SimTypePointer(SimStruct({"numEku": SimTypeInt(signed=False, label="UInt32"), "eku": SimTypePointer(SimTypePointer(SimTypeChar(label="Byte"), offset=0), offset=0)}, name="IKEEXT_CERT_EKUS0", pack=False, align=None), offset=0), "name": SimTypePointer(SimStruct({"nameType": SimTypeInt(signed=False, label="IKEEXT_CERT_CRITERIA_NAME_TYPE"), "certName": SimTypePointer(SimTypeChar(label="Char"), offset=0)}, name="IKEEXT_CERT_NAME0", pack=False, align=None), offset=0), "flags": SimTypeInt(signed=False, label="UInt32")}, name="IKEEXT_CERTIFICATE_CRITERIA0", pack=False, align=None), offset=0)}, name="_Anonymous2_e__Struct", pack=False, align=None), "Anonymous3": SimStruct({"outboundRootStoreArraySize": SimTypeInt(signed=False, label="UInt32"), "outboundTrustedRootStoreCriteria": SimTypePointer(SimStruct({"certData": SimStruct({"size": SimTypeInt(signed=False, label="UInt32"), "data": SimTypePointer(SimTypeChar(label="Byte"), offset=0)}, name="FWP_BYTE_BLOB", pack=False, align=None), "certHash": SimStruct({"size": SimTypeInt(signed=False, label="UInt32"), "data": SimTypePointer(SimTypeChar(label="Byte"), offset=0)}, name="FWP_BYTE_BLOB", pack=False, align=None), "eku": SimTypePointer(SimStruct({"numEku": SimTypeInt(signed=False, label="UInt32"), "eku": SimTypePointer(SimTypePointer(SimTypeChar(label="Byte"), offset=0), offset=0)}, name="IKEEXT_CERT_EKUS0", pack=False, align=None), offset=0), "name": SimTypePointer(SimStruct({"nameType": SimTypeInt(signed=False, label="IKEEXT_CERT_CRITERIA_NAME_TYPE"), "certName": SimTypePointer(SimTypeChar(label="Char"), offset=0)}, name="IKEEXT_CERT_NAME0", pack=False, align=None), offset=0), "flags": SimTypeInt(signed=False, label="UInt32")}, name="IKEEXT_CERTIFICATE_CRITERIA0", pack=False, align=None), offset=0)}, name="_Anonymous3_e__Struct", pack=False, align=None)}, name="<anon>", label="None"), "flags": SimTypeInt(signed=False, label="IKEEXT_CERT_AUTH"), "localCertLocationUrl": SimStruct({"size": SimTypeInt(signed=False, label="UInt32"), "data": SimTypePointer(SimTypeChar(label="Byte"), offset=0)}, name="FWP_BYTE_BLOB", pack=False, align=None)}, name="IKEEXT_CERTIFICATE_AUTHENTICATION2", pack=False, align=None), "cgaAuthentication": SimStruct({"keyContainerName": SimTypePointer(SimTypeChar(label="Char"), offset=0), "cspName": SimTypePointer(SimTypeChar(label="Char"), offset=0), "cspType": SimTypeInt(signed=False, label="UInt32"), "cgaModifier": SimStruct({"byteArray16": SimTypeFixedSizeArray(SimTypeChar(label="Byte"), 16)}, name="FWP_BYTE_ARRAY16", pack=False, align=None), "cgaCollisionCount": SimTypeChar(label="Byte")}, name="IKEEXT_IPV6_CGA_AUTHENTICATION0", pack=False, align=None), "eapAuthentication": SimStruct({"flags": SimTypeInt(signed=False, label="IKEEXT_EAP_AUTHENTICATION_FLAGS")}, name="IKEEXT_EAP_AUTHENTICATION0", pack=False, align=None)}, name="<anon>", label="None")}, name="IKEEXT_AUTHENTICATION_METHOD2", pack=False, align=None), offset=0), "initiatorImpersonationType": SimTypeInt(signed=False, label="IKEEXT_AUTHENTICATION_IMPERSONATION_TYPE"), "numIkeProposals": SimTypeInt(signed=False, label="UInt32"), "ikeProposals": SimTypePointer(SimStruct({"cipherAlgorithm": SimStruct({"algoIdentifier": SimTypeInt(signed=False, label="IKEEXT_CIPHER_TYPE"), "keyLen": SimTypeInt(signed=False, label="UInt32"), "rounds": SimTypeInt(signed=False, label="UInt32")}, name="IKEEXT_CIPHER_ALGORITHM0", pack=False, align=None), "integrityAlgorithm": SimStruct({"algoIdentifier": SimTypeInt(signed=False, label="IKEEXT_INTEGRITY_TYPE")}, name="IKEEXT_INTEGRITY_ALGORITHM0", pack=False, align=None), "maxLifetimeSeconds": SimTypeInt(signed=False, label="UInt32"), "dhGroup": SimTypeInt(signed=False, label="IKEEXT_DH_GROUP"), "quickModeLimit": SimTypeInt(signed=False, label="UInt32")}, name="IKEEXT_PROPOSAL0", pack=False, align=None), offset=0), "flags": SimTypeInt(signed=False, label="IKEEXT_POLICY_FLAG"), "maxDynamicFilters": SimTypeInt(signed=False, label="UInt32"), "retransmitDurationSecs": SimTypeInt(signed=False, label="UInt32")}, name="IKEEXT_POLICY2", pack=False, align=None), offset=0), "dataBuffer": SimTypePointer(SimStruct({"size": SimTypeInt(signed=False, label="UInt32"), "data": SimTypePointer(SimTypeChar(label="Byte"), offset=0)}, name="FWP_BYTE_BLOB", pack=False, align=None), offset=0), "classifyOptions": SimTypePointer(SimStruct({"numOptions": SimTypeInt(signed=False, label="UInt32"), "options": SimTypePointer(SimStruct({"type": SimTypeInt(signed=False, label="FWP_CLASSIFY_OPTION_TYPE"), "value": SimStruct({"type": SimTypeInt(signed=False, label="FWP_DATA_TYPE"), "Anonymous": SimUnion({"uint8": SimTypeChar(label="Byte"), "uint16": SimTypeShort(signed=False, label="UInt16"), "uint32": SimTypeInt(signed=False, label="UInt32"), "uint64": SimTypePointer(SimTypeLongLong(signed=False, label="UInt64"), offset=0), "int8": SimTypeChar(label="SByte"), "int16": SimTypeShort(signed=True, label="Int16"), "int32": SimTypeInt(signed=True, label="Int32"), "int64": SimTypePointer(SimTypeLongLong(signed=True, label="Int64"), offset=0), "float32": SimTypeFloat(size=32), "double64": SimTypePointer(SimTypeFloat(size=64), offset=0), "byteArray16": SimTypePointer(SimStruct({"byteArray16": SimTypeFixedSizeArray(SimTypeChar(label="Byte"), 16)}, name="FWP_BYTE_ARRAY16", pack=False, align=None), offset=0), "byteBlob": SimTypePointer(SimStruct({"size": SimTypeInt(signed=False, label="UInt32"), "data": SimTypePointer(SimTypeChar(label="Byte"), offset=0)}, name="FWP_BYTE_BLOB", pack=False, align=None), offset=0), "sid": SimTypePointer(SimTypeBottom(label="SID"), offset=0), "sd": SimTypePointer(SimStruct({"size": SimTypeInt(signed=False, label="UInt32"), "data": SimTypePointer(SimTypeChar(label="Byte"), offset=0)}, name="FWP_BYTE_BLOB", pack=False, align=None), offset=0), "tokenInformation": SimTypePointer(SimStruct({"sidCount": SimTypeInt(signed=False, label="UInt32"), "sids": SimTypePointer(SimTypeBottom(label="SID_AND_ATTRIBUTES"), offset=0), "restrictedSidCount": SimTypeInt(signed=False, label="UInt32"), "restrictedSids": SimTypePointer(SimTypeBottom(label="SID_AND_ATTRIBUTES"), offset=0)}, name="FWP_TOKEN_INFORMATION", pack=False, align=None), offset=0), "tokenAccessInformation": SimTypePointer(SimStruct({"size": SimTypeInt(signed=False, label="UInt32"), "data": SimTypePointer(SimTypeChar(label="Byte"), offset=0)}, name="FWP_BYTE_BLOB", pack=False, align=None), offset=0), "unicodeString": SimTypePointer(SimTypeChar(label="Char"), offset=0), "byteArray6": SimTypePointer(SimStruct({"byteArray6": SimTypeFixedSizeArray(SimTypeChar(label="Byte"), 6)}, name="FWP_BYTE_ARRAY6", pack=False, align=None), offset=0), "bitmapArray64": SimTypePointer(SimStruct({"bitmapArray64": SimTypeFixedSizeArray(SimTypeChar(label="Byte"), 8)}, name="FWP_BITMAP_ARRAY64_", pack=False, align=None), offset=0)}, name="<anon>", label="None")}, name="FWP_VALUE0", pack=False, align=None)}, name="FWPM_CLASSIFY_OPTION0", pack=False, align=None), offset=0)}, name="FWPM_CLASSIFY_OPTIONS0", pack=False, align=None), offset=0), "ikeV2QmTunnelPolicy": SimTypePointer(SimStruct({"flags": SimTypeInt(signed=False, label="IPSEC_POLICY_FLAG"), "numIpsecProposals": SimTypeInt(signed=False, label="UInt32"), "ipsecProposals": SimTypePointer(SimStruct({"lifetime": SimStruct({"lifetimeSeconds": SimTypeInt(signed=False, label="UInt32"), "lifetimeKilobytes": SimTypeInt(signed=False, label="UInt32"), "lifetimePackets": SimTypeInt(signed=False, label="UInt32")}, name="IPSEC_SA_LIFETIME0", pack=False, align=None), "numSaTransforms": SimTypeInt(signed=False, label="UInt32"), "saTransforms": SimTypePointer(SimStruct({"ipsecTransformType": SimTypeInt(signed=False, label="IPSEC_TRANSFORM_TYPE"), "Anonymous": SimUnion({"ahTransform": SimTypePointer(SimStruct({"authTransformId": SimStruct({"authType": SimTypeInt(signed=False, label="IPSEC_AUTH_TYPE"), "authConfig": SimTypeChar(label="Byte")}, name="IPSEC_AUTH_TRANSFORM_ID0", pack=False, align=None), "cryptoModuleId": SimTypePointer(SimTypeBottom(label="Guid"), offset=0)}, name="IPSEC_AUTH_TRANSFORM0", pack=False, align=None), offset=0), "espAuthTransform": SimTypePointer(SimStruct({"authTransformId": SimStruct({"authType": SimTypeInt(signed=False, label="IPSEC_AUTH_TYPE"), "authConfig": SimTypeChar(label="Byte")}, name="IPSEC_AUTH_TRANSFORM_ID0", pack=False, align=None), "cryptoModuleId": SimTypePointer(SimTypeBottom(label="Guid"), offset=0)}, name="IPSEC_AUTH_TRANSFORM0", pack=False, align=None), offset=0), "espCipherTransform": SimTypePointer(SimStruct({"cipherTransformId": SimStruct({"cipherType": SimTypeInt(signed=False, label="IPSEC_CIPHER_TYPE"), "cipherConfig": SimTypeChar(label="Byte")}, name="IPSEC_CIPHER_TRANSFORM_ID0", pack=False, align=None), "cryptoModuleId": SimTypePointer(SimTypeBottom(label="Guid"), offset=0)}, name="IPSEC_CIPHER_TRANSFORM0", pack=False, align=None), offset=0), "espAuthAndCipherTransform": SimTypePointer(SimStruct({"authTransform": SimStruct({"authTransformId": SimStruct({"authType": SimTypeInt(signed=False, label="IPSEC_AUTH_TYPE"), "authConfig": SimTypeChar(label="Byte")}, name="IPSEC_AUTH_TRANSFORM_ID0", pack=False, align=None), "cryptoModuleId": SimTypePointer(SimTypeBottom(label="Guid"), offset=0)}, name="IPSEC_AUTH_TRANSFORM0", pack=False, align=None), "cipherTransform": SimStruct({"cipherTransformId": SimStruct({"cipherType": SimTypeInt(signed=False, label="IPSEC_CIPHER_TYPE"), "cipherConfig": SimTypeChar(label="Byte")}, name="IPSEC_CIPHER_TRANSFORM_ID0", pack=False, align=None), "cryptoModuleId": SimTypePointer(SimTypeBottom(label="Guid"), offset=0)}, name="IPSEC_CIPHER_TRANSFORM0", pack=False, align=None)}, name="IPSEC_AUTH_AND_CIPHER_TRANSFORM0", pack=False, align=None), offset=0), "espAuthFwTransform": SimTypePointer(SimStruct({"authTransformId": SimStruct({"authType": SimTypeInt(signed=False, label="IPSEC_AUTH_TYPE"), "authConfig": SimTypeChar(label="Byte")}, name="IPSEC_AUTH_TRANSFORM_ID0", pack=False, align=None), "cryptoModuleId": SimTypePointer(SimTypeBottom(label="Guid"), offset=0)}, name="IPSEC_AUTH_TRANSFORM0", pack=False, align=None), offset=0)}, name="<anon>", label="None")}, name="IPSEC_SA_TRANSFORM0", pack=False, align=None), offset=0), "pfsGroup": SimTypeInt(signed=False, label="IPSEC_PFS_GROUP")}, name="IPSEC_PROPOSAL0", pack=False, align=None), offset=0), "tunnelEndpoints": SimStruct({"ipVersion": SimTypeInt(signed=False, label="FWP_IP_VERSION"), "Anonymous1": SimUnion({"localV4Address": SimTypeInt(signed=False, label="UInt32"), "localV6Address": SimTypeFixedSizeArray(SimTypeChar(label="Byte"), 16)}, name="<anon>", label="None"), "Anonymous2": SimUnion({"remoteV4Address": SimTypeInt(signed=False, label="UInt32"), "remoteV6Address": SimTypeFixedSizeArray(SimTypeChar(label="Byte"), 16)}, name="<anon>", label="None"), "localIfLuid": SimTypeLongLong(signed=False, label="UInt64"), "remoteFqdn": SimTypePointer(SimTypeChar(label="Char"), offset=0), "numAddresses": SimTypeInt(signed=False, label="UInt32"), "remoteAddresses": SimTypePointer(SimStruct({"ipVersion": SimTypeInt(signed=False, label="FWP_IP_VERSION"), "Anonymous": SimUnion({"v4Address": SimTypeInt(signed=False, label="UInt32"), "v6Address": SimTypeFixedSizeArray(SimTypeChar(label="Byte"), 16)}, name="<anon>", label="None")}, name="IPSEC_TUNNEL_ENDPOINT0", pack=False, align=None), offset=0)}, name="IPSEC_TUNNEL_ENDPOINTS2", pack=False, align=None), "saIdleTimeout": SimStruct({"idleTimeoutSeconds": SimTypeInt(signed=False, label="UInt32"), "idleTimeoutSecondsFailOver": SimTypeInt(signed=False, label="UInt32")}, name="IPSEC_SA_IDLE_TIMEOUT0", pack=False, align=None), "emPolicy": SimTypePointer(SimStruct({"numAuthenticationMethods": SimTypeInt(signed=False, label="UInt32"), "authenticationMethods": SimTypePointer(SimStruct({"authenticationMethodType": SimTypeInt(signed=False, label="IKEEXT_AUTHENTICATION_METHOD_TYPE"), "Anonymous": SimUnion({"presharedKeyAuthentication": SimStruct({"presharedKey": SimStruct({"size": SimTypeInt(signed=False, label="UInt32"), "data": SimTypePointer(SimTypeChar(label="Byte"), offset=0)}, name="FWP_BYTE_BLOB", pack=False, align=None), "flags": SimTypeInt(signed=False, label="IKEEXT_PRESHARED_KEY_AUTHENTICATION_FLAGS")}, name="IKEEXT_PRESHARED_KEY_AUTHENTICATION1", pack=False, align=None), "certificateAuthentication": SimStruct({"inboundConfigType": SimTypeInt(signed=False, label="IKEEXT_CERT_CONFIG_TYPE"), "Anonymous1": SimUnion({"Anonymous1": SimStruct({"inboundRootArraySize": SimTypeInt(signed=False, label="UInt32"), "inboundRootCriteria": SimTypePointer(SimStruct({"certData": SimStruct({"size": SimTypeInt(signed=False, label="UInt32"), "data": SimTypePointer(SimTypeChar(label="Byte"), offset=0)}, name="FWP_BYTE_BLOB", pack=False, align=None), "certHash": SimStruct({"size": SimTypeInt(signed=False, label="UInt32"), "data": SimTypePointer(SimTypeChar(label="Byte"), offset=0)}, name="FWP_BYTE_BLOB", pack=False, align=None), "eku": SimTypePointer(SimStruct({"numEku": SimTypeInt(signed=False, label="UInt32"), "eku": SimTypePointer(SimTypePointer(SimTypeChar(label="Byte"), offset=0), offset=0)}, name="IKEEXT_CERT_EKUS0", pack=False, align=None), offset=0), "name": SimTypePointer(SimStruct({"nameType": SimTypeInt(signed=False, label="IKEEXT_CERT_CRITERIA_NAME_TYPE"), "certName": SimTypePointer(SimTypeChar(label="Char"), offset=0)}, name="IKEEXT_CERT_NAME0", pack=False, align=None), offset=0), "flags": SimTypeInt(signed=False, label="UInt32")}, name="IKEEXT_CERTIFICATE_CRITERIA0", pack=False, align=None), offset=0)}, name="_Anonymous1_e__Struct", pack=False, align=None), "Anonymous2": SimStruct({"inboundEnterpriseStoreArraySize": SimTypeInt(signed=False, label="UInt32"), "inboundEnterpriseStoreCriteria": SimTypePointer(SimStruct({"certData": SimStruct({"size": SimTypeInt(signed=False, label="UInt32"), "data": SimTypePointer(SimTypeChar(label="Byte"), offset=0)}, name="FWP_BYTE_BLOB", pack=False, align=None), "certHash": SimStruct({"size": SimTypeInt(signed=False, label="UInt32"), "data": SimTypePointer(SimTypeChar(label="Byte"), offset=0)}, name="FWP_BYTE_BLOB", pack=False, align=None), "eku": SimTypePointer(SimStruct({"numEku": SimTypeInt(signed=False, label="UInt32"), "eku": SimTypePointer(SimTypePointer(SimTypeChar(label="Byte"), offset=0), offset=0)}, name="IKEEXT_CERT_EKUS0", pack=False, align=None), offset=0), "name": SimTypePointer(SimStruct({"nameType": SimTypeInt(signed=False, label="IKEEXT_CERT_CRITERIA_NAME_TYPE"), "certName": SimTypePointer(SimTypeChar(label="Char"), offset=0)}, name="IKEEXT_CERT_NAME0", pack=False, align=None), offset=0), "flags": SimTypeInt(signed=False, label="UInt32")}, name="IKEEXT_CERTIFICATE_CRITERIA0", pack=False, align=None), offset=0)}, name="_Anonymous2_e__Struct", pack=False, align=None), "Anonymous3": SimStruct({"inboundRootStoreArraySize": SimTypeInt(signed=False, label="UInt32"), "inboundTrustedRootStoreCriteria": SimTypePointer(SimStruct({"certData": SimStruct({"size": SimTypeInt(signed=False, label="UInt32"), "data": SimTypePointer(SimTypeChar(label="Byte"), offset=0)}, name="FWP_BYTE_BLOB", pack=False, align=None), "certHash": SimStruct({"size": SimTypeInt(signed=False, label="UInt32"), "data": SimTypePointer(SimTypeChar(label="Byte"), offset=0)}, name="FWP_BYTE_BLOB", pack=False, align=None), "eku": SimTypePointer(SimStruct({"numEku": SimTypeInt(signed=False, label="UInt32"), "eku": SimTypePointer(SimTypePointer(SimTypeChar(label="Byte"), offset=0), offset=0)}, name="IKEEXT_CERT_EKUS0", pack=False, align=None), offset=0), "name": SimTypePointer(SimStruct({"nameType": SimTypeInt(signed=False, label="IKEEXT_CERT_CRITERIA_NAME_TYPE"), "certName": SimTypePointer(SimTypeChar(label="Char"), offset=0)}, name="IKEEXT_CERT_NAME0", pack=False, align=None), offset=0), "flags": SimTypeInt(signed=False, label="UInt32")}, name="IKEEXT_CERTIFICATE_CRITERIA0", pack=False, align=None), offset=0)}, name="_Anonymous3_e__Struct", pack=False, align=None)}, name="<anon>", label="None"), "outboundConfigType": SimTypeInt(signed=False, label="IKEEXT_CERT_CONFIG_TYPE"), "Anonymous2": SimUnion({"Anonymous1": SimStruct({"outboundRootArraySize": SimTypeInt(signed=False, label="UInt32"), "outboundRootCriteria": SimTypePointer(SimStruct({"certData": SimStruct({"size": SimTypeInt(signed=False, label="UInt32"), "data": SimTypePointer(SimTypeChar(label="Byte"), offset=0)}, name="FWP_BYTE_BLOB", pack=False, align=None), "certHash": SimStruct({"size": SimTypeInt(signed=False, label="UInt32"), "data": SimTypePointer(SimTypeChar(label="Byte"), offset=0)}, name="FWP_BYTE_BLOB", pack=False, align=None), "eku": SimTypePointer(SimStruct({"numEku": SimTypeInt(signed=False, label="UInt32"), "eku": SimTypePointer(SimTypePointer(SimTypeChar(label="Byte"), offset=0), offset=0)}, name="IKEEXT_CERT_EKUS0", pack=False, align=None), offset=0), "name": SimTypePointer(SimStruct({"nameType": SimTypeInt(signed=False, label="IKEEXT_CERT_CRITERIA_NAME_TYPE"), "certName": SimTypePointer(SimTypeChar(label="Char"), offset=0)}, name="IKEEXT_CERT_NAME0", pack=False, align=None), offset=0), "flags": SimTypeInt(signed=False, label="UInt32")}, name="IKEEXT_CERTIFICATE_CRITERIA0", pack=False, align=None), offset=0)}, name="_Anonymous1_e__Struct", pack=False, align=None), "Anonymous2": SimStruct({"outboundEnterpriseStoreArraySize": SimTypeInt(signed=False, label="UInt32"), "outboundEnterpriseStoreCriteria": SimTypePointer(SimStruct({"certData": SimStruct({"size": SimTypeInt(signed=False, label="UInt32"), "data": SimTypePointer(SimTypeChar(label="Byte"), offset=0)}, name="FWP_BYTE_BLOB", pack=False, align=None), "certHash": SimStruct({"size": SimTypeInt(signed=False, label="UInt32"), "data": SimTypePointer(SimTypeChar(label="Byte"), offset=0)}, name="FWP_BYTE_BLOB", pack=False, align=None), "eku": SimTypePointer(SimStruct({"numEku": SimTypeInt(signed=False, label="UInt32"), "eku": SimTypePointer(SimTypePointer(SimTypeChar(label="Byte"), offset=0), offset=0)}, name="IKEEXT_CERT_EKUS0", pack=False, align=None), offset=0), "name": SimTypePointer(SimStruct({"nameType": SimTypeInt(signed=False, label="IKEEXT_CERT_CRITERIA_NAME_TYPE"), "certName": SimTypePointer(SimTypeChar(label="Char"), offset=0)}, name="IKEEXT_CERT_NAME0", pack=False, align=None), offset=0), "flags": SimTypeInt(signed=False, label="UInt32")}, name="IKEEXT_CERTIFICATE_CRITERIA0", pack=False, align=None), offset=0)}, name="_Anonymous2_e__Struct", pack=False, align=None), "Anonymous3": SimStruct({"outboundRootStoreArraySize": SimTypeInt(signed=False, label="UInt32"), "outboundTrustedRootStoreCriteria": SimTypePointer(SimStruct({"certData": SimStruct({"size": SimTypeInt(signed=False, label="UInt32"), "data": SimTypePointer(SimTypeChar(label="Byte"), offset=0)}, name="FWP_BYTE_BLOB", pack=False, align=None), "certHash": SimStruct({"size": SimTypeInt(signed=False, label="UInt32"), "data": SimTypePointer(SimTypeChar(label="Byte"), offset=0)}, name="FWP_BYTE_BLOB", pack=False, align=None), "eku": SimTypePointer(SimStruct({"numEku": SimTypeInt(signed=False, label="UInt32"), "eku": SimTypePointer(SimTypePointer(SimTypeChar(label="Byte"), offset=0), offset=0)}, name="IKEEXT_CERT_EKUS0", pack=False, align=None), offset=0), "name": SimTypePointer(SimStruct({"nameType": SimTypeInt(signed=False, label="IKEEXT_CERT_CRITERIA_NAME_TYPE"), "certName": SimTypePointer(SimTypeChar(label="Char"), offset=0)}, name="IKEEXT_CERT_NAME0", pack=False, align=None), offset=0), "flags": SimTypeInt(signed=False, label="UInt32")}, name="IKEEXT_CERTIFICATE_CRITERIA0", pack=False, align=None), offset=0)}, name="_Anonymous3_e__Struct", pack=False, align=None)}, name="<anon>", label="None"), "flags": SimTypeInt(signed=False, label="IKEEXT_CERT_AUTH"), "localCertLocationUrl": SimStruct({"size": SimTypeInt(signed=False, label="UInt32"), "data": SimTypePointer(SimTypeChar(label="Byte"), offset=0)}, name="FWP_BYTE_BLOB", pack=False, align=None)}, name="IKEEXT_CERTIFICATE_AUTHENTICATION2", pack=False, align=None), "kerberosAuthentication": SimStruct({"flags": SimTypeInt(signed=False, label="IKEEXT_KERBEROS_AUTHENTICATION_FLAGS"), "proxyServer": SimTypePointer(SimTypeChar(label="Char"), offset=0)}, name="IKEEXT_KERBEROS_AUTHENTICATION1", pack=False, align=None), "reservedAuthentication": SimStruct({"flags": SimTypeInt(signed=False, label="IKEEXT_RESERVED_AUTHENTICATION_FLAGS")}, name="IKEEXT_RESERVED_AUTHENTICATION0", pack=False, align=None), "ntlmV2Authentication": SimStruct({"flags": SimTypeInt(signed=False, label="UInt32")}, name="IKEEXT_NTLM_V2_AUTHENTICATION0", pack=False, align=None), "sslAuthentication": SimStruct({"inboundConfigType": SimTypeInt(signed=False, label="IKEEXT_CERT_CONFIG_TYPE"), "Anonymous1": SimUnion({"Anonymous1": SimStruct({"inboundRootArraySize": SimTypeInt(signed=False, label="UInt32"), "inboundRootCriteria": SimTypePointer(SimStruct({"certData": SimStruct({"size": SimTypeInt(signed=False, label="UInt32"), "data": SimTypePointer(SimTypeChar(label="Byte"), offset=0)}, name="FWP_BYTE_BLOB", pack=False, align=None), "certHash": SimStruct({"size": SimTypeInt(signed=False, label="UInt32"), "data": SimTypePointer(SimTypeChar(label="Byte"), offset=0)}, name="FWP_BYTE_BLOB", pack=False, align=None), "eku": SimTypePointer(SimStruct({"numEku": SimTypeInt(signed=False, label="UInt32"), "eku": SimTypePointer(SimTypePointer(SimTypeChar(label="Byte"), offset=0), offset=0)}, name="IKEEXT_CERT_EKUS0", pack=False, align=None), offset=0), "name": SimTypePointer(SimStruct({"nameType": SimTypeInt(signed=False, label="IKEEXT_CERT_CRITERIA_NAME_TYPE"), "certName": SimTypePointer(SimTypeChar(label="Char"), offset=0)}, name="IKEEXT_CERT_NAME0", pack=False, align=None), offset=0), "flags": SimTypeInt(signed=False, label="UInt32")}, name="IKEEXT_CERTIFICATE_CRITERIA0", pack=False, align=None), offset=0)}, name="_Anonymous1_e__Struct", pack=False, align=None), "Anonymous2": SimStruct({"inboundEnterpriseStoreArraySize": SimTypeInt(signed=False, label="UInt32"), "inboundEnterpriseStoreCriteria": SimTypePointer(SimStruct({"certData": SimStruct({"size": SimTypeInt(signed=False, label="UInt32"), "data": SimTypePointer(SimTypeChar(label="Byte"), offset=0)}, name="FWP_BYTE_BLOB", pack=False, align=None), "certHash": SimStruct({"size": SimTypeInt(signed=False, label="UInt32"), "data": SimTypePointer(SimTypeChar(label="Byte"), offset=0)}, name="FWP_BYTE_BLOB", pack=False, align=None), "eku": SimTypePointer(SimStruct({"numEku": SimTypeInt(signed=False, label="UInt32"), "eku": SimTypePointer(SimTypePointer(SimTypeChar(label="Byte"), offset=0), offset=0)}, name="IKEEXT_CERT_EKUS0", pack=False, align=None), offset=0), "name": SimTypePointer(SimStruct({"nameType": SimTypeInt(signed=False, label="IKEEXT_CERT_CRITERIA_NAME_TYPE"), "certName": SimTypePointer(SimTypeChar(label="Char"), offset=0)}, name="IKEEXT_CERT_NAME0", pack=False, align=None), offset=0), "flags": SimTypeInt(signed=False, label="UInt32")}, name="IKEEXT_CERTIFICATE_CRITERIA0", pack=False, align=None), offset=0)}, name="_Anonymous2_e__Struct", pack=False, align=None), "Anonymous3": SimStruct({"inboundRootStoreArraySize": SimTypeInt(signed=False, label="UInt32"), "inboundTrustedRootStoreCriteria": SimTypePointer(SimStruct({"certData": SimStruct({"size": SimTypeInt(signed=False, label="UInt32"), "data": SimTypePointer(SimTypeChar(label="Byte"), offset=0)}, name="FWP_BYTE_BLOB", pack=False, align=None), "certHash": SimStruct({"size": SimTypeInt(signed=False, label="UInt32"), "data": SimTypePointer(SimTypeChar(label="Byte"), offset=0)}, name="FWP_BYTE_BLOB", pack=False, align=None), "eku": SimTypePointer(SimStruct({"numEku": SimTypeInt(signed=False, label="UInt32"), "eku": SimTypePointer(SimTypePointer(SimTypeChar(label="Byte"), offset=0), offset=0)}, name="IKEEXT_CERT_EKUS0", pack=False, align=None), offset=0), "name": SimTypePointer(SimStruct({"nameType": SimTypeInt(signed=False, label="IKEEXT_CERT_CRITERIA_NAME_TYPE"), "certName": SimTypePointer(SimTypeChar(label="Char"), offset=0)}, name="IKEEXT_CERT_NAME0", pack=False, align=None), offset=0), "flags": SimTypeInt(signed=False, label="UInt32")}, name="IKEEXT_CERTIFICATE_CRITERIA0", pack=False, align=None), offset=0)}, name="_Anonymous3_e__Struct", pack=False, align=None)}, name="<anon>", label="None"), "outboundConfigType": SimTypeInt(signed=False, label="IKEEXT_CERT_CONFIG_TYPE"), "Anonymous2": SimUnion({"Anonymous1": SimStruct({"outboundRootArraySize": SimTypeInt(signed=False, label="UInt32"), "outboundRootCriteria": SimTypePointer(SimStruct({"certData": SimStruct({"size": SimTypeInt(signed=False, label="UInt32"), "data": SimTypePointer(SimTypeChar(label="Byte"), offset=0)}, name="FWP_BYTE_BLOB", pack=False, align=None), "certHash": SimStruct({"size": SimTypeInt(signed=False, label="UInt32"), "data": SimTypePointer(SimTypeChar(label="Byte"), offset=0)}, name="FWP_BYTE_BLOB", pack=False, align=None), "eku": SimTypePointer(SimStruct({"numEku": SimTypeInt(signed=False, label="UInt32"), "eku": SimTypePointer(SimTypePointer(SimTypeChar(label="Byte"), offset=0), offset=0)}, name="IKEEXT_CERT_EKUS0", pack=False, align=None), offset=0), "name": SimTypePointer(SimStruct({"nameType": SimTypeInt(signed=False, label="IKEEXT_CERT_CRITERIA_NAME_TYPE"), "certName": SimTypePointer(SimTypeChar(label="Char"), offset=0)}, name="IKEEXT_CERT_NAME0", pack=False, align=None), offset=0), "flags": SimTypeInt(signed=False, label="UInt32")}, name="IKEEXT_CERTIFICATE_CRITERIA0", pack=False, align=None), offset=0)}, name="_Anonymous1_e__Struct", pack=False, align=None), "Anonymous2": SimStruct({"outboundEnterpriseStoreArraySize": SimTypeInt(signed=False, label="UInt32"), "outboundEnterpriseStoreCriteria": SimTypePointer(SimStruct({"certData": SimStruct({"size": SimTypeInt(signed=False, label="UInt32"), "data": SimTypePointer(SimTypeChar(label="Byte"), offset=0)}, name="FWP_BYTE_BLOB", pack=False, align=None), "certHash": SimStruct({"size": SimTypeInt(signed=False, label="UInt32"), "data": SimTypePointer(SimTypeChar(label="Byte"), offset=0)}, name="FWP_BYTE_BLOB", pack=False, align=None), "eku": SimTypePointer(SimStruct({"numEku": SimTypeInt(signed=False, label="UInt32"), "eku": SimTypePointer(SimTypePointer(SimTypeChar(label="Byte"), offset=0), offset=0)}, name="IKEEXT_CERT_EKUS0", pack=False, align=None), offset=0), "name": SimTypePointer(SimStruct({"nameType": SimTypeInt(signed=False, label="IKEEXT_CERT_CRITERIA_NAME_TYPE"), "certName": SimTypePointer(SimTypeChar(label="Char"), offset=0)}, name="IKEEXT_CERT_NAME0", pack=False, align=None), offset=0), "flags": SimTypeInt(signed=False, label="UInt32")}, name="IKEEXT_CERTIFICATE_CRITERIA0", pack=False, align=None), offset=0)}, name="_Anonymous2_e__Struct", pack=False, align=None), "Anonymous3": SimStruct({"outboundRootStoreArraySize": SimTypeInt(signed=False, label="UInt32"), "outboundTrustedRootStoreCriteria": SimTypePointer(SimStruct({"certData": SimStruct({"size": SimTypeInt(signed=False, label="UInt32"), "data": SimTypePointer(SimTypeChar(label="Byte"), offset=0)}, name="FWP_BYTE_BLOB", pack=False, align=None), "certHash": SimStruct({"size": SimTypeInt(signed=False, label="UInt32"), "data": SimTypePointer(SimTypeChar(label="Byte"), offset=0)}, name="FWP_BYTE_BLOB", pack=False, align=None), "eku": SimTypePointer(SimStruct({"numEku": SimTypeInt(signed=False, label="UInt32"), "eku": SimTypePointer(SimTypePointer(SimTypeChar(label="Byte"), offset=0), offset=0)}, name="IKEEXT_CERT_EKUS0", pack=False, align=None), offset=0), "name": SimTypePointer(SimStruct({"nameType": SimTypeInt(signed=False, label="IKEEXT_CERT_CRITERIA_NAME_TYPE"), "certName": SimTypePointer(SimTypeChar(label="Char"), offset=0)}, name="IKEEXT_CERT_NAME0", pack=False, align=None), offset=0), "flags": SimTypeInt(signed=False, label="UInt32")}, name="IKEEXT_CERTIFICATE_CRITERIA0", pack=False, align=None), offset=0)}, name="_Anonymous3_e__Struct", pack=False, align=None)}, name="<anon>", label="None"), "flags": SimTypeInt(signed=False, label="IKEEXT_CERT_AUTH"), "localCertLocationUrl": SimStruct({"size": SimTypeInt(signed=False, label="UInt32"), "data": SimTypePointer(SimTypeChar(label="Byte"), offset=0)}, name="FWP_BYTE_BLOB", pack=False, align=None)}, name="IKEEXT_CERTIFICATE_AUTHENTICATION2", pack=False, align=None), "cgaAuthentication": SimStruct({"keyContainerName": SimTypePointer(SimTypeChar(label="Char"), offset=0), "cspName": SimTypePointer(SimTypeChar(label="Char"), offset=0), "cspType": SimTypeInt(signed=False, label="UInt32"), "cgaModifier": SimStruct({"byteArray16": SimTypeFixedSizeArray(SimTypeChar(label="Byte"), 16)}, name="FWP_BYTE_ARRAY16", pack=False, align=None), "cgaCollisionCount": SimTypeChar(label="Byte")}, name="IKEEXT_IPV6_CGA_AUTHENTICATION0", pack=False, align=None), "eapAuthentication": SimStruct({"flags": SimTypeInt(signed=False, label="IKEEXT_EAP_AUTHENTICATION_FLAGS")}, name="IKEEXT_EAP_AUTHENTICATION0", pack=False, align=None)}, name="<anon>", label="None")}, name="IKEEXT_AUTHENTICATION_METHOD2", pack=False, align=None), offset=0), "initiatorImpersonationType": SimTypeInt(signed=False, label="IKEEXT_AUTHENTICATION_IMPERSONATION_TYPE")}, name="IKEEXT_EM_POLICY2", pack=False, align=None), offset=0), "fwdPathSaLifetime": SimTypeInt(signed=False, label="UInt32")}, name="IPSEC_TUNNEL_POLICY2", pack=False, align=None), offset=0), "ikeV2QmTransportPolicy": SimTypePointer(SimStruct({"numIpsecProposals": SimTypeInt(signed=False, label="UInt32"), "ipsecProposals": SimTypePointer(SimStruct({"lifetime": SimStruct({"lifetimeSeconds": SimTypeInt(signed=False, label="UInt32"), "lifetimeKilobytes": SimTypeInt(signed=False, label="UInt32"), "lifetimePackets": SimTypeInt(signed=False, label="UInt32")}, name="IPSEC_SA_LIFETIME0", pack=False, align=None), "numSaTransforms": SimTypeInt(signed=False, label="UInt32"), "saTransforms": SimTypePointer(SimStruct({"ipsecTransformType": SimTypeInt(signed=False, label="IPSEC_TRANSFORM_TYPE"), "Anonymous": SimUnion({"ahTransform": SimTypePointer(SimStruct({"authTransformId": SimStruct({"authType": SimTypeInt(signed=False, label="IPSEC_AUTH_TYPE"), "authConfig": SimTypeChar(label="Byte")}, name="IPSEC_AUTH_TRANSFORM_ID0", pack=False, align=None), "cryptoModuleId": SimTypePointer(SimTypeBottom(label="Guid"), offset=0)}, name="IPSEC_AUTH_TRANSFORM0", pack=False, align=None), offset=0), "espAuthTransform": SimTypePointer(SimStruct({"authTransformId": SimStruct({"authType": SimTypeInt(signed=False, label="IPSEC_AUTH_TYPE"), "authConfig": SimTypeChar(label="Byte")}, name="IPSEC_AUTH_TRANSFORM_ID0", pack=False, align=None), "cryptoModuleId": SimTypePointer(SimTypeBottom(label="Guid"), offset=0)}, name="IPSEC_AUTH_TRANSFORM0", pack=False, align=None), offset=0), "espCipherTransform": SimTypePointer(SimStruct({"cipherTransformId": SimStruct({"cipherType": SimTypeInt(signed=False, label="IPSEC_CIPHER_TYPE"), "cipherConfig": SimTypeChar(label="Byte")}, name="IPSEC_CIPHER_TRANSFORM_ID0", pack=False, align=None), "cryptoModuleId": SimTypePointer(SimTypeBottom(label="Guid"), offset=0)}, name="IPSEC_CIPHER_TRANSFORM0", pack=False, align=None), offset=0), "espAuthAndCipherTransform": SimTypePointer(SimStruct({"authTransform": SimStruct({"authTransformId": SimStruct({"authType": SimTypeInt(signed=False, label="IPSEC_AUTH_TYPE"), "authConfig": SimTypeChar(label="Byte")}, name="IPSEC_AUTH_TRANSFORM_ID0", pack=False, align=None), "cryptoModuleId": SimTypePointer(SimTypeBottom(label="Guid"), offset=0)}, name="IPSEC_AUTH_TRANSFORM0", pack=False, align=None), "cipherTransform": SimStruct({"cipherTransformId": SimStruct({"cipherType": SimTypeInt(signed=False, label="IPSEC_CIPHER_TYPE"), "cipherConfig": SimTypeChar(label="Byte")}, name="IPSEC_CIPHER_TRANSFORM_ID0", pack=False, align=None), "cryptoModuleId": SimTypePointer(SimTypeBottom(label="Guid"), offset=0)}, name="IPSEC_CIPHER_TRANSFORM0", pack=False, align=None)}, name="IPSEC_AUTH_AND_CIPHER_TRANSFORM0", pack=False, align=None), offset=0), "espAuthFwTransform": SimTypePointer(SimStruct({"authTransformId": SimStruct({"authType": SimTypeInt(signed=False, label="IPSEC_AUTH_TYPE"), "authConfig": SimTypeChar(label="Byte")}, name="IPSEC_AUTH_TRANSFORM_ID0", pack=False, align=None), "cryptoModuleId": SimTypePointer(SimTypeBottom(label="Guid"), offset=0)}, name="IPSEC_AUTH_TRANSFORM0", pack=False, align=None), offset=0)}, name="<anon>", label="None")}, name="IPSEC_SA_TRANSFORM0", pack=False, align=None), offset=0), "pfsGroup": SimTypeInt(signed=False, label="IPSEC_PFS_GROUP")}, name="IPSEC_PROPOSAL0", pack=False, align=None), offset=0), "flags": SimTypeInt(signed=False, label="IPSEC_POLICY_FLAG"), "ndAllowClearTimeoutSeconds": SimTypeInt(signed=False, label="UInt32"), "saIdleTimeout": SimStruct({"idleTimeoutSeconds": SimTypeInt(signed=False, label="UInt32"), "idleTimeoutSecondsFailOver": SimTypeInt(signed=False, label="UInt32")}, name="IPSEC_SA_IDLE_TIMEOUT0", pack=False, align=None), "emPolicy": SimTypePointer(SimStruct({"numAuthenticationMethods": SimTypeInt(signed=False, label="UInt32"), "authenticationMethods": SimTypePointer(SimStruct({"authenticationMethodType": SimTypeInt(signed=False, label="IKEEXT_AUTHENTICATION_METHOD_TYPE"), "Anonymous": SimUnion({"presharedKeyAuthentication": SimStruct({"presharedKey": SimStruct({"size": SimTypeInt(signed=False, label="UInt32"), "data": SimTypePointer(SimTypeChar(label="Byte"), offset=0)}, name="FWP_BYTE_BLOB", pack=False, align=None), "flags": SimTypeInt(signed=False, label="IKEEXT_PRESHARED_KEY_AUTHENTICATION_FLAGS")}, name="IKEEXT_PRESHARED_KEY_AUTHENTICATION1", pack=False, align=None), "certificateAuthentication": SimStruct({"inboundConfigType": SimTypeInt(signed=False, label="IKEEXT_CERT_CONFIG_TYPE"), "Anonymous1": SimUnion({"Anonymous1": SimStruct({"inboundRootArraySize": SimTypeInt(signed=False, label="UInt32"), "inboundRootCriteria": SimTypePointer(SimStruct({"certData": SimStruct({"size": SimTypeInt(signed=False, label="UInt32"), "data": SimTypePointer(SimTypeChar(label="Byte"), offset=0)}, name="FWP_BYTE_BLOB", pack=False, align=None), "certHash": SimStruct({"size": SimTypeInt(signed=False, label="UInt32"), "data": SimTypePointer(SimTypeChar(label="Byte"), offset=0)}, name="FWP_BYTE_BLOB", pack=False, align=None), "eku": SimTypePointer(SimStruct({"numEku": SimTypeInt(signed=False, label="UInt32"), "eku": SimTypePointer(SimTypePointer(SimTypeChar(label="Byte"), offset=0), offset=0)}, name="IKEEXT_CERT_EKUS0", pack=False, align=None), offset=0), "name": SimTypePointer(SimStruct({"nameType": SimTypeInt(signed=False, label="IKEEXT_CERT_CRITERIA_NAME_TYPE"), "certName": SimTypePointer(SimTypeChar(label="Char"), offset=0)}, name="IKEEXT_CERT_NAME0", pack=False, align=None), offset=0), "flags": SimTypeInt(signed=False, label="UInt32")}, name="IKEEXT_CERTIFICATE_CRITERIA0", pack=False, align=None), offset=0)}, name="_Anonymous1_e__Struct", pack=False, align=None), "Anonymous2": SimStruct({"inboundEnterpriseStoreArraySize": SimTypeInt(signed=False, label="UInt32"), "inboundEnterpriseStoreCriteria": SimTypePointer(SimStruct({"certData": SimStruct({"size": SimTypeInt(signed=False, label="UInt32"), "data": SimTypePointer(SimTypeChar(label="Byte"), offset=0)}, name="FWP_BYTE_BLOB", pack=False, align=None), "certHash": SimStruct({"size": SimTypeInt(signed=False, label="UInt32"), "data": SimTypePointer(SimTypeChar(label="Byte"), offset=0)}, name="FWP_BYTE_BLOB", pack=False, align=None), "eku": SimTypePointer(SimStruct({"numEku": SimTypeInt(signed=False, label="UInt32"), "eku": SimTypePointer(SimTypePointer(SimTypeChar(label="Byte"), offset=0), offset=0)}, name="IKEEXT_CERT_EKUS0", pack=False, align=None), offset=0), "name": SimTypePointer(SimStruct({"nameType": SimTypeInt(signed=False, label="IKEEXT_CERT_CRITERIA_NAME_TYPE"), "certName": SimTypePointer(SimTypeChar(label="Char"), offset=0)}, name="IKEEXT_CERT_NAME0", pack=False, align=None), offset=0), "flags": SimTypeInt(signed=False, label="UInt32")}, name="IKEEXT_CERTIFICATE_CRITERIA0", pack=False, align=None), offset=0)}, name="_Anonymous2_e__Struct", pack=False, align=None), "Anonymous3": SimStruct({"inboundRootStoreArraySize": SimTypeInt(signed=False, label="UInt32"), "inboundTrustedRootStoreCriteria": SimTypePointer(SimStruct({"certData": SimStruct({"size": SimTypeInt(signed=False, label="UInt32"), "data": SimTypePointer(SimTypeChar(label="Byte"), offset=0)}, name="FWP_BYTE_BLOB", pack=False, align=None), "certHash": SimStruct({"size": SimTypeInt(signed=False, label="UInt32"), "data": SimTypePointer(SimTypeChar(label="Byte"), offset=0)}, name="FWP_BYTE_BLOB", pack=False, align=None), "eku": SimTypePointer(SimStruct({"numEku": SimTypeInt(signed=False, label="UInt32"), "eku": SimTypePointer(SimTypePointer(SimTypeChar(label="Byte"), offset=0), offset=0)}, name="IKEEXT_CERT_EKUS0", pack=False, align=None), offset=0), "name": SimTypePointer(SimStruct({"nameType": SimTypeInt(signed=False, label="IKEEXT_CERT_CRITERIA_NAME_TYPE"), "certName": SimTypePointer(SimTypeChar(label="Char"), offset=0)}, name="IKEEXT_CERT_NAME0", pack=False, align=None), offset=0), "flags": SimTypeInt(signed=False, label="UInt32")}, name="IKEEXT_CERTIFICATE_CRITERIA0", pack=False, align=None), offset=0)}, name="_Anonymous3_e__Struct", pack=False, align=None)}, name="<anon>", label="None"), "outboundConfigType": SimTypeInt(signed=False, label="IKEEXT_CERT_CONFIG_TYPE"), "Anonymous2": SimUnion({"Anonymous1": SimStruct({"outboundRootArraySize": SimTypeInt(signed=False, label="UInt32"), "outboundRootCriteria": SimTypePointer(SimStruct({"certData": SimStruct({"size": SimTypeInt(signed=False, label="UInt32"), "data": SimTypePointer(SimTypeChar(label="Byte"), offset=0)}, name="FWP_BYTE_BLOB", pack=False, align=None), "certHash": SimStruct({"size": SimTypeInt(signed=False, label="UInt32"), "data": SimTypePointer(SimTypeChar(label="Byte"), offset=0)}, name="FWP_BYTE_BLOB", pack=False, align=None), "eku": SimTypePointer(SimStruct({"numEku": SimTypeInt(signed=False, label="UInt32"), "eku": SimTypePointer(SimTypePointer(SimTypeChar(label="Byte"), offset=0), offset=0)}, name="IKEEXT_CERT_EKUS0", pack=False, align=None), offset=0), "name": SimTypePointer(SimStruct({"nameType": SimTypeInt(signed=False, label="IKEEXT_CERT_CRITERIA_NAME_TYPE"), "certName": SimTypePointer(SimTypeChar(label="Char"), offset=0)}, name="IKEEXT_CERT_NAME0", pack=False, align=None), offset=0), "flags": SimTypeInt(signed=False, label="UInt32")}, name="IKEEXT_CERTIFICATE_CRITERIA0", pack=False, align=None), offset=0)}, name="_Anonymous1_e__Struct", pack=False, align=None), "Anonymous2": SimStruct({"outboundEnterpriseStoreArraySize": SimTypeInt(signed=False, label="UInt32"), "outboundEnterpriseStoreCriteria": SimTypePointer(SimStruct({"certData": SimStruct({"size": SimTypeInt(signed=False, label="UInt32"), "data": SimTypePointer(SimTypeChar(label="Byte"), offset=0)}, name="FWP_BYTE_BLOB", pack=False, align=None), "certHash": SimStruct({"size": SimTypeInt(signed=False, label="UInt32"), "data": SimTypePointer(SimTypeChar(label="Byte"), offset=0)}, name="FWP_BYTE_BLOB", pack=False, align=None), "eku": SimTypePointer(SimStruct({"numEku": SimTypeInt(signed=False, label="UInt32"), "eku": SimTypePointer(SimTypePointer(SimTypeChar(label="Byte"), offset=0), offset=0)}, name="IKEEXT_CERT_EKUS0", pack=False, align=None), offset=0), "name": SimTypePointer(SimStruct({"nameType": SimTypeInt(signed=False, label="IKEEXT_CERT_CRITERIA_NAME_TYPE"), "certName": SimTypePointer(SimTypeChar(label="Char"), offset=0)}, name="IKEEXT_CERT_NAME0", pack=False, align=None), offset=0), "flags": SimTypeInt(signed=False, label="UInt32")}, name="IKEEXT_CERTIFICATE_CRITERIA0", pack=False, align=None), offset=0)}, name="_Anonymous2_e__Struct", pack=False, align=None), "Anonymous3": SimStruct({"outboundRootStoreArraySize": SimTypeInt(signed=False, label="UInt32"), "outboundTrustedRootStoreCriteria": SimTypePointer(SimStruct({"certData": SimStruct({"size": SimTypeInt(signed=False, label="UInt32"), "data": SimTypePointer(SimTypeChar(label="Byte"), offset=0)}, name="FWP_BYTE_BLOB", pack=False, align=None), "certHash": SimStruct({"size": SimTypeInt(signed=False, label="UInt32"), "data": SimTypePointer(SimTypeChar(label="Byte"), offset=0)}, name="FWP_BYTE_BLOB", pack=False, align=None), "eku": SimTypePointer(SimStruct({"numEku": SimTypeInt(signed=False, label="UInt32"), "eku": SimTypePointer(SimTypePointer(SimTypeChar(label="Byte"), offset=0), offset=0)}, name="IKEEXT_CERT_EKUS0", pack=False, align=None), offset=0), "name": SimTypePointer(SimStruct({"nameType": SimTypeInt(signed=False, label="IKEEXT_CERT_CRITERIA_NAME_TYPE"), "certName": SimTypePointer(SimTypeChar(label="Char"), offset=0)}, name="IKEEXT_CERT_NAME0", pack=False, align=None), offset=0), "flags": SimTypeInt(signed=False, label="UInt32")}, name="IKEEXT_CERTIFICATE_CRITERIA0", pack=False, align=None), offset=0)}, name="_Anonymous3_e__Struct", pack=False, align=None)}, name="<anon>", label="None"), "flags": SimTypeInt(signed=False, label="IKEEXT_CERT_AUTH"), "localCertLocationUrl": SimStruct({"size": SimTypeInt(signed=False, label="UInt32"), "data": SimTypePointer(SimTypeChar(label="Byte"), offset=0)}, name="FWP_BYTE_BLOB", pack=False, align=None)}, name="IKEEXT_CERTIFICATE_AUTHENTICATION2", pack=False, align=None), "kerberosAuthentication": SimStruct({"flags": SimTypeInt(signed=False, label="IKEEXT_KERBEROS_AUTHENTICATION_FLAGS"), "proxyServer": SimTypePointer(SimTypeChar(label="Char"), offset=0)}, name="IKEEXT_KERBEROS_AUTHENTICATION1", pack=False, align=None), "reservedAuthentication": SimStruct({"flags": SimTypeInt(signed=False, label="IKEEXT_RESERVED_AUTHENTICATION_FLAGS")}, name="IKEEXT_RESERVED_AUTHENTICATION0", pack=False, align=None), "ntlmV2Authentication": SimStruct({"flags": SimTypeInt(signed=False, label="UInt32")}, name="IKEEXT_NTLM_V2_AUTHENTICATION0", pack=False, align=None), "sslAuthentication": SimStruct({"inboundConfigType": SimTypeInt(signed=False, label="IKEEXT_CERT_CONFIG_TYPE"), "Anonymous1": SimUnion({"Anonymous1": SimStruct({"inboundRootArraySize": SimTypeInt(signed=False, label="UInt32"), "inboundRootCriteria": SimTypePointer(SimStruct({"certData": SimStruct({"size": SimTypeInt(signed=False, label="UInt32"), "data": SimTypePointer(SimTypeChar(label="Byte"), offset=0)}, name="FWP_BYTE_BLOB", pack=False, align=None), "certHash": SimStruct({"size": SimTypeInt(signed=False, label="UInt32"), "data": SimTypePointer(SimTypeChar(label="Byte"), offset=0)}, name="FWP_BYTE_BLOB", pack=False, align=None), "eku": SimTypePointer(SimStruct({"numEku": SimTypeInt(signed=False, label="UInt32"), "eku": SimTypePointer(SimTypePointer(SimTypeChar(label="Byte"), offset=0), offset=0)}, name="IKEEXT_CERT_EKUS0", pack=False, align=None), offset=0), "name": SimTypePointer(SimStruct({"nameType": SimTypeInt(signed=False, label="IKEEXT_CERT_CRITERIA_NAME_TYPE"), "certName": SimTypePointer(SimTypeChar(label="Char"), offset=0)}, name="IKEEXT_CERT_NAME0", pack=False, align=None), offset=0), "flags": SimTypeInt(signed=False, label="UInt32")}, name="IKEEXT_CERTIFICATE_CRITERIA0", pack=False, align=None), offset=0)}, name="_Anonymous1_e__Struct", pack=False, align=None), "Anonymous2": SimStruct({"inboundEnterpriseStoreArraySize": SimTypeInt(signed=False, label="UInt32"), "inboundEnterpriseStoreCriteria": SimTypePointer(SimStruct({"certData": SimStruct({"size": SimTypeInt(signed=False, label="UInt32"), "data": SimTypePointer(SimTypeChar(label="Byte"), offset=0)}, name="FWP_BYTE_BLOB", pack=False, align=None), "certHash": SimStruct({"size": SimTypeInt(signed=False, label="UInt32"), "data": SimTypePointer(SimTypeChar(label="Byte"), offset=0)}, name="FWP_BYTE_BLOB", pack=False, align=None), "eku": SimTypePointer(SimStruct({"numEku": SimTypeInt(signed=False, label="UInt32"), "eku": SimTypePointer(SimTypePointer(SimTypeChar(label="Byte"), offset=0), offset=0)}, name="IKEEXT_CERT_EKUS0", pack=False, align=None), offset=0), "name": SimTypePointer(SimStruct({"nameType": SimTypeInt(signed=False, label="IKEEXT_CERT_CRITERIA_NAME_TYPE"), "certName": SimTypePointer(SimTypeChar(label="Char"), offset=0)}, name="IKEEXT_CERT_NAME0", pack=False, align=None), offset=0), "flags": SimTypeInt(signed=False, label="UInt32")}, name="IKEEXT_CERTIFICATE_CRITERIA0", pack=False, align=None), offset=0)}, name="_Anonymous2_e__Struct", pack=False, align=None), "Anonymous3": SimStruct({"inboundRootStoreArraySize": SimTypeInt(signed=False, label="UInt32"), "inboundTrustedRootStoreCriteria": SimTypePointer(SimStruct({"certData": SimStruct({"size": SimTypeInt(signed=False, label="UInt32"), "data": SimTypePointer(SimTypeChar(label="Byte"), offset=0)}, name="FWP_BYTE_BLOB", pack=False, align=None), "certHash": SimStruct({"size": SimTypeInt(signed=False, label="UInt32"), "data": SimTypePointer(SimTypeChar(label="Byte"), offset=0)}, name="FWP_BYTE_BLOB", pack=False, align=None), "eku": SimTypePointer(SimStruct({"numEku": SimTypeInt(signed=False, label="UInt32"), "eku": SimTypePointer(SimTypePointer(SimTypeChar(label="Byte"), offset=0), offset=0)}, name="IKEEXT_CERT_EKUS0", pack=False, align=None), offset=0), "name": SimTypePointer(SimStruct({"nameType": SimTypeInt(signed=False, label="IKEEXT_CERT_CRITERIA_NAME_TYPE"), "certName": SimTypePointer(SimTypeChar(label="Char"), offset=0)}, name="IKEEXT_CERT_NAME0", pack=False, align=None), offset=0), "flags": SimTypeInt(signed=False, label="UInt32")}, name="IKEEXT_CERTIFICATE_CRITERIA0", pack=False, align=None), offset=0)}, name="_Anonymous3_e__Struct", pack=False, align=None)}, name="<anon>", label="None"), "outboundConfigType": SimTypeInt(signed=False, label="IKEEXT_CERT_CONFIG_TYPE"), "Anonymous2": SimUnion({"Anonymous1": SimStruct({"outboundRootArraySize": SimTypeInt(signed=False, label="UInt32"), "outboundRootCriteria": SimTypePointer(SimStruct({"certData": SimStruct({"size": SimTypeInt(signed=False, label="UInt32"), "data": SimTypePointer(SimTypeChar(label="Byte"), offset=0)}, name="FWP_BYTE_BLOB", pack=False, align=None), "certHash": SimStruct({"size": SimTypeInt(signed=False, label="UInt32"), "data": SimTypePointer(SimTypeChar(label="Byte"), offset=0)}, name="FWP_BYTE_BLOB", pack=False, align=None), "eku": SimTypePointer(SimStruct({"numEku": SimTypeInt(signed=False, label="UInt32"), "eku": SimTypePointer(SimTypePointer(SimTypeChar(label="Byte"), offset=0), offset=0)}, name="IKEEXT_CERT_EKUS0", pack=False, align=None), offset=0), "name": SimTypePointer(SimStruct({"nameType": SimTypeInt(signed=False, label="IKEEXT_CERT_CRITERIA_NAME_TYPE"), "certName": SimTypePointer(SimTypeChar(label="Char"), offset=0)}, name="IKEEXT_CERT_NAME0", pack=False, align=None), offset=0), "flags": SimTypeInt(signed=False, label="UInt32")}, name="IKEEXT_CERTIFICATE_CRITERIA0", pack=False, align=None), offset=0)}, name="_Anonymous1_e__Struct", pack=False, align=None), "Anonymous2": SimStruct({"outboundEnterpriseStoreArraySize": SimTypeInt(signed=False, label="UInt32"), "outboundEnterpriseStoreCriteria": SimTypePointer(SimStruct({"certData": SimStruct({"size": SimTypeInt(signed=False, label="UInt32"), "data": SimTypePointer(SimTypeChar(label="Byte"), offset=0)}, name="FWP_BYTE_BLOB", pack=False, align=None), "certHash": SimStruct({"size": SimTypeInt(signed=False, label="UInt32"), "data": SimTypePointer(SimTypeChar(label="Byte"), offset=0)}, name="FWP_BYTE_BLOB", pack=False, align=None), "eku": SimTypePointer(SimStruct({"numEku": SimTypeInt(signed=False, label="UInt32"), "eku": SimTypePointer(SimTypePointer(SimTypeChar(label="Byte"), offset=0), offset=0)}, name="IKEEXT_CERT_EKUS0", pack=False, align=None), offset=0), "name": SimTypePointer(SimStruct({"nameType": SimTypeInt(signed=False, label="IKEEXT_CERT_CRITERIA_NAME_TYPE"), "certName": SimTypePointer(SimTypeChar(label="Char"), offset=0)}, name="IKEEXT_CERT_NAME0", pack=False, align=None), offset=0), "flags": SimTypeInt(signed=False, label="UInt32")}, name="IKEEXT_CERTIFICATE_CRITERIA0", pack=False, align=None), offset=0)}, name="_Anonymous2_e__Struct", pack=False, align=None), "Anonymous3": SimStruct({"outboundRootStoreArraySize": SimTypeInt(signed=False, label="UInt32"), "outboundTrustedRootStoreCriteria": SimTypePointer(SimStruct({"certData": SimStruct({"size": SimTypeInt(signed=False, label="UInt32"), "data": SimTypePointer(SimTypeChar(label="Byte"), offset=0)}, name="FWP_BYTE_BLOB", pack=False, align=None), "certHash": SimStruct({"size": SimTypeInt(signed=False, label="UInt32"), "data": SimTypePointer(SimTypeChar(label="Byte"), offset=0)}, name="FWP_BYTE_BLOB", pack=False, align=None), "eku": SimTypePointer(SimStruct({"numEku": SimTypeInt(signed=False, label="UInt32"), "eku": SimTypePointer(SimTypePointer(SimTypeChar(label="Byte"), offset=0), offset=0)}, name="IKEEXT_CERT_EKUS0", pack=False, align=None), offset=0), "name": SimTypePointer(SimStruct({"nameType": SimTypeInt(signed=False, label="IKEEXT_CERT_CRITERIA_NAME_TYPE"), "certName": SimTypePointer(SimTypeChar(label="Char"), offset=0)}, name="IKEEXT_CERT_NAME0", pack=False, align=None), offset=0), "flags": SimTypeInt(signed=False, label="UInt32")}, name="IKEEXT_CERTIFICATE_CRITERIA0", pack=False, align=None), offset=0)}, name="_Anonymous3_e__Struct", pack=False, align=None)}, name="<anon>", label="None"), "flags": SimTypeInt(signed=False, label="IKEEXT_CERT_AUTH"), "localCertLocationUrl": SimStruct({"size": SimTypeInt(signed=False, label="UInt32"), "data": SimTypePointer(SimTypeChar(label="Byte"), offset=0)}, name="FWP_BYTE_BLOB", pack=False, align=None)}, name="IKEEXT_CERTIFICATE_AUTHENTICATION2", pack=False, align=None), "cgaAuthentication": SimStruct({"keyContainerName": SimTypePointer(SimTypeChar(label="Char"), offset=0), "cspName": SimTypePointer(SimTypeChar(label="Char"), offset=0), "cspType": SimTypeInt(signed=False, label="UInt32"), "cgaModifier": SimStruct({"byteArray16": SimTypeFixedSizeArray(SimTypeChar(label="Byte"), 16)}, name="FWP_BYTE_ARRAY16", pack=False, align=None), "cgaCollisionCount": SimTypeChar(label="Byte")}, name="IKEEXT_IPV6_CGA_AUTHENTICATION0", pack=False, align=None), "eapAuthentication": SimStruct({"flags": SimTypeInt(signed=False, label="IKEEXT_EAP_AUTHENTICATION_FLAGS")}, name="IKEEXT_EAP_AUTHENTICATION0", pack=False, align=None)}, name="<anon>", label="None")}, name="IKEEXT_AUTHENTICATION_METHOD2", pack=False, align=None), offset=0), "initiatorImpersonationType": SimTypeInt(signed=False, label="IKEEXT_AUTHENTICATION_IMPERSONATION_TYPE")}, name="IKEEXT_EM_POLICY2", pack=False, align=None), offset=0)}, name="IPSEC_TRANSPORT_POLICY2", pack=False, align=None), offset=0), "ikeV2MmPolicy": SimTypePointer(SimStruct({"softExpirationTime": SimTypeInt(signed=False, label="UInt32"), "numAuthenticationMethods": SimTypeInt(signed=False, label="UInt32"), "authenticationMethods": SimTypePointer(SimStruct({"authenticationMethodType": SimTypeInt(signed=False, label="IKEEXT_AUTHENTICATION_METHOD_TYPE"), "Anonymous": SimUnion({"presharedKeyAuthentication": SimStruct({"presharedKey": SimStruct({"size": SimTypeInt(signed=False, label="UInt32"), "data": SimTypePointer(SimTypeChar(label="Byte"), offset=0)}, name="FWP_BYTE_BLOB", pack=False, align=None), "flags": SimTypeInt(signed=False, label="IKEEXT_PRESHARED_KEY_AUTHENTICATION_FLAGS")}, name="IKEEXT_PRESHARED_KEY_AUTHENTICATION1", pack=False, align=None), "certificateAuthentication": SimStruct({"inboundConfigType": SimTypeInt(signed=False, label="IKEEXT_CERT_CONFIG_TYPE"), "Anonymous1": SimUnion({"Anonymous1": SimStruct({"inboundRootArraySize": SimTypeInt(signed=False, label="UInt32"), "inboundRootCriteria": SimTypePointer(SimStruct({"certData": SimStruct({"size": SimTypeInt(signed=False, label="UInt32"), "data": SimTypePointer(SimTypeChar(label="Byte"), offset=0)}, name="FWP_BYTE_BLOB", pack=False, align=None), "certHash": SimStruct({"size": SimTypeInt(signed=False, label="UInt32"), "data": SimTypePointer(SimTypeChar(label="Byte"), offset=0)}, name="FWP_BYTE_BLOB", pack=False, align=None), "eku": SimTypePointer(SimStruct({"numEku": SimTypeInt(signed=False, label="UInt32"), "eku": SimTypePointer(SimTypePointer(SimTypeChar(label="Byte"), offset=0), offset=0)}, name="IKEEXT_CERT_EKUS0", pack=False, align=None), offset=0), "name": SimTypePointer(SimStruct({"nameType": SimTypeInt(signed=False, label="IKEEXT_CERT_CRITERIA_NAME_TYPE"), "certName": SimTypePointer(SimTypeChar(label="Char"), offset=0)}, name="IKEEXT_CERT_NAME0", pack=False, align=None), offset=0), "flags": SimTypeInt(signed=False, label="UInt32")}, name="IKEEXT_CERTIFICATE_CRITERIA0", pack=False, align=None), offset=0)}, name="_Anonymous1_e__Struct", pack=False, align=None), "Anonymous2": SimStruct({"inboundEnterpriseStoreArraySize": SimTypeInt(signed=False, label="UInt32"), "inboundEnterpriseStoreCriteria": SimTypePointer(SimStruct({"certData": SimStruct({"size": SimTypeInt(signed=False, label="UInt32"), "data": SimTypePointer(SimTypeChar(label="Byte"), offset=0)}, name="FWP_BYTE_BLOB", pack=False, align=None), "certHash": SimStruct({"size": SimTypeInt(signed=False, label="UInt32"), "data": SimTypePointer(SimTypeChar(label="Byte"), offset=0)}, name="FWP_BYTE_BLOB", pack=False, align=None), "eku": SimTypePointer(SimStruct({"numEku": SimTypeInt(signed=False, label="UInt32"), "eku": SimTypePointer(SimTypePointer(SimTypeChar(label="Byte"), offset=0), offset=0)}, name="IKEEXT_CERT_EKUS0", pack=False, align=None), offset=0), "name": SimTypePointer(SimStruct({"nameType": SimTypeInt(signed=False, label="IKEEXT_CERT_CRITERIA_NAME_TYPE"), "certName": SimTypePointer(SimTypeChar(label="Char"), offset=0)}, name="IKEEXT_CERT_NAME0", pack=False, align=None), offset=0), "flags": SimTypeInt(signed=False, label="UInt32")}, name="IKEEXT_CERTIFICATE_CRITERIA0", pack=False, align=None), offset=0)}, name="_Anonymous2_e__Struct", pack=False, align=None), "Anonymous3": SimStruct({"inboundRootStoreArraySize": SimTypeInt(signed=False, label="UInt32"), "inboundTrustedRootStoreCriteria": SimTypePointer(SimStruct({"certData": SimStruct({"size": SimTypeInt(signed=False, label="UInt32"), "data": SimTypePointer(SimTypeChar(label="Byte"), offset=0)}, name="FWP_BYTE_BLOB", pack=False, align=None), "certHash": SimStruct({"size": SimTypeInt(signed=False, label="UInt32"), "data": SimTypePointer(SimTypeChar(label="Byte"), offset=0)}, name="FWP_BYTE_BLOB", pack=False, align=None), "eku": SimTypePointer(SimStruct({"numEku": SimTypeInt(signed=False, label="UInt32"), "eku": SimTypePointer(SimTypePointer(SimTypeChar(label="Byte"), offset=0), offset=0)}, name="IKEEXT_CERT_EKUS0", pack=False, align=None), offset=0), "name": SimTypePointer(SimStruct({"nameType": SimTypeInt(signed=False, label="IKEEXT_CERT_CRITERIA_NAME_TYPE"), "certName": SimTypePointer(SimTypeChar(label="Char"), offset=0)}, name="IKEEXT_CERT_NAME0", pack=False, align=None), offset=0), "flags": SimTypeInt(signed=False, label="UInt32")}, name="IKEEXT_CERTIFICATE_CRITERIA0", pack=False, align=None), offset=0)}, name="_Anonymous3_e__Struct", pack=False, align=None)}, name="<anon>", label="None"), "outboundConfigType": SimTypeInt(signed=False, label="IKEEXT_CERT_CONFIG_TYPE"), "Anonymous2": SimUnion({"Anonymous1": SimStruct({"outboundRootArraySize": SimTypeInt(signed=False, label="UInt32"), "outboundRootCriteria": SimTypePointer(SimStruct({"certData": SimStruct({"size": SimTypeInt(signed=False, label="UInt32"), "data": SimTypePointer(SimTypeChar(label="Byte"), offset=0)}, name="FWP_BYTE_BLOB", pack=False, align=None), "certHash": SimStruct({"size": SimTypeInt(signed=False, label="UInt32"), "data": SimTypePointer(SimTypeChar(label="Byte"), offset=0)}, name="FWP_BYTE_BLOB", pack=False, align=None), "eku": SimTypePointer(SimStruct({"numEku": SimTypeInt(signed=False, label="UInt32"), "eku": SimTypePointer(SimTypePointer(SimTypeChar(label="Byte"), offset=0), offset=0)}, name="IKEEXT_CERT_EKUS0", pack=False, align=None), offset=0), "name": SimTypePointer(SimStruct({"nameType": SimTypeInt(signed=False, label="IKEEXT_CERT_CRITERIA_NAME_TYPE"), "certName": SimTypePointer(SimTypeChar(label="Char"), offset=0)}, name="IKEEXT_CERT_NAME0", pack=False, align=None), offset=0), "flags": SimTypeInt(signed=False, label="UInt32")}, name="IKEEXT_CERTIFICATE_CRITERIA0", pack=False, align=None), offset=0)}, name="_Anonymous1_e__Struct", pack=False, align=None), "Anonymous2": SimStruct({"outboundEnterpriseStoreArraySize": SimTypeInt(signed=False, label="UInt32"), "outboundEnterpriseStoreCriteria": SimTypePointer(SimStruct({"certData": SimStruct({"size": SimTypeInt(signed=False, label="UInt32"), "data": SimTypePointer(SimTypeChar(label="Byte"), offset=0)}, name="FWP_BYTE_BLOB", pack=False, align=None), "certHash": SimStruct({"size": SimTypeInt(signed=False, label="UInt32"), "data": SimTypePointer(SimTypeChar(label="Byte"), offset=0)}, name="FWP_BYTE_BLOB", pack=False, align=None), "eku": SimTypePointer(SimStruct({"numEku": SimTypeInt(signed=False, label="UInt32"), "eku": SimTypePointer(SimTypePointer(SimTypeChar(label="Byte"), offset=0), offset=0)}, name="IKEEXT_CERT_EKUS0", pack=False, align=None), offset=0), "name": SimTypePointer(SimStruct({"nameType": SimTypeInt(signed=False, label="IKEEXT_CERT_CRITERIA_NAME_TYPE"), "certName": SimTypePointer(SimTypeChar(label="Char"), offset=0)}, name="IKEEXT_CERT_NAME0", pack=False, align=None), offset=0), "flags": SimTypeInt(signed=False, label="UInt32")}, name="IKEEXT_CERTIFICATE_CRITERIA0", pack=False, align=None), offset=0)}, name="_Anonymous2_e__Struct", pack=False, align=None), "Anonymous3": SimStruct({"outboundRootStoreArraySize": SimTypeInt(signed=False, label="UInt32"), "outboundTrustedRootStoreCriteria": SimTypePointer(SimStruct({"certData": SimStruct({"size": SimTypeInt(signed=False, label="UInt32"), "data": SimTypePointer(SimTypeChar(label="Byte"), offset=0)}, name="FWP_BYTE_BLOB", pack=False, align=None), "certHash": SimStruct({"size": SimTypeInt(signed=False, label="UInt32"), "data": SimTypePointer(SimTypeChar(label="Byte"), offset=0)}, name="FWP_BYTE_BLOB", pack=False, align=None), "eku": SimTypePointer(SimStruct({"numEku": SimTypeInt(signed=False, label="UInt32"), "eku": SimTypePointer(SimTypePointer(SimTypeChar(label="Byte"), offset=0), offset=0)}, name="IKEEXT_CERT_EKUS0", pack=False, align=None), offset=0), "name": SimTypePointer(SimStruct({"nameType": SimTypeInt(signed=False, label="IKEEXT_CERT_CRITERIA_NAME_TYPE"), "certName": SimTypePointer(SimTypeChar(label="Char"), offset=0)}, name="IKEEXT_CERT_NAME0", pack=False, align=None), offset=0), "flags": SimTypeInt(signed=False, label="UInt32")}, name="IKEEXT_CERTIFICATE_CRITERIA0", pack=False, align=None), offset=0)}, name="_Anonymous3_e__Struct", pack=False, align=None)}, name="<anon>", label="None"), "flags": SimTypeInt(signed=False, label="IKEEXT_CERT_AUTH"), "localCertLocationUrl": SimStruct({"size": SimTypeInt(signed=False, label="UInt32"), "data": SimTypePointer(SimTypeChar(label="Byte"), offset=0)}, name="FWP_BYTE_BLOB", pack=False, align=None)}, name="IKEEXT_CERTIFICATE_AUTHENTICATION2", pack=False, align=None), "kerberosAuthentication": SimStruct({"flags": SimTypeInt(signed=False, label="IKEEXT_KERBEROS_AUTHENTICATION_FLAGS"), "proxyServer": SimTypePointer(SimTypeChar(label="Char"), offset=0)}, name="IKEEXT_KERBEROS_AUTHENTICATION1", pack=False, align=None), "reservedAuthentication": SimStruct({"flags": SimTypeInt(signed=False, label="IKEEXT_RESERVED_AUTHENTICATION_FLAGS")}, name="IKEEXT_RESERVED_AUTHENTICATION0", pack=False, align=None), "ntlmV2Authentication": SimStruct({"flags": SimTypeInt(signed=False, label="UInt32")}, name="IKEEXT_NTLM_V2_AUTHENTICATION0", pack=False, align=None), "sslAuthentication": SimStruct({"inboundConfigType": SimTypeInt(signed=False, label="IKEEXT_CERT_CONFIG_TYPE"), "Anonymous1": SimUnion({"Anonymous1": SimStruct({"inboundRootArraySize": SimTypeInt(signed=False, label="UInt32"), "inboundRootCriteria": SimTypePointer(SimStruct({"certData": SimStruct({"size": SimTypeInt(signed=False, label="UInt32"), "data": SimTypePointer(SimTypeChar(label="Byte"), offset=0)}, name="FWP_BYTE_BLOB", pack=False, align=None), "certHash": SimStruct({"size": SimTypeInt(signed=False, label="UInt32"), "data": SimTypePointer(SimTypeChar(label="Byte"), offset=0)}, name="FWP_BYTE_BLOB", pack=False, align=None), "eku": SimTypePointer(SimStruct({"numEku": SimTypeInt(signed=False, label="UInt32"), "eku": SimTypePointer(SimTypePointer(SimTypeChar(label="Byte"), offset=0), offset=0)}, name="IKEEXT_CERT_EKUS0", pack=False, align=None), offset=0), "name": SimTypePointer(SimStruct({"nameType": SimTypeInt(signed=False, label="IKEEXT_CERT_CRITERIA_NAME_TYPE"), "certName": SimTypePointer(SimTypeChar(label="Char"), offset=0)}, name="IKEEXT_CERT_NAME0", pack=False, align=None), offset=0), "flags": SimTypeInt(signed=False, label="UInt32")}, name="IKEEXT_CERTIFICATE_CRITERIA0", pack=False, align=None), offset=0)}, name="_Anonymous1_e__Struct", pack=False, align=None), "Anonymous2": SimStruct({"inboundEnterpriseStoreArraySize": SimTypeInt(signed=False, label="UInt32"), "inboundEnterpriseStoreCriteria": SimTypePointer(SimStruct({"certData": SimStruct({"size": SimTypeInt(signed=False, label="UInt32"), "data": SimTypePointer(SimTypeChar(label="Byte"), offset=0)}, name="FWP_BYTE_BLOB", pack=False, align=None), "certHash": SimStruct({"size": SimTypeInt(signed=False, label="UInt32"), "data": SimTypePointer(SimTypeChar(label="Byte"), offset=0)}, name="FWP_BYTE_BLOB", pack=False, align=None), "eku": SimTypePointer(SimStruct({"numEku": SimTypeInt(signed=False, label="UInt32"), "eku": SimTypePointer(SimTypePointer(SimTypeChar(label="Byte"), offset=0), offset=0)}, name="IKEEXT_CERT_EKUS0", pack=False, align=None), offset=0), "name": SimTypePointer(SimStruct({"nameType": SimTypeInt(signed=False, label="IKEEXT_CERT_CRITERIA_NAME_TYPE"), "certName": SimTypePointer(SimTypeChar(label="Char"), offset=0)}, name="IKEEXT_CERT_NAME0", pack=False, align=None), offset=0), "flags": SimTypeInt(signed=False, label="UInt32")}, name="IKEEXT_CERTIFICATE_CRITERIA0", pack=False, align=None), offset=0)}, name="_Anonymous2_e__Struct", pack=False, align=None), "Anonymous3": SimStruct({"inboundRootStoreArraySize": SimTypeInt(signed=False, label="UInt32"), "inboundTrustedRootStoreCriteria": SimTypePointer(SimStruct({"certData": SimStruct({"size": SimTypeInt(signed=False, label="UInt32"), "data": SimTypePointer(SimTypeChar(label="Byte"), offset=0)}, name="FWP_BYTE_BLOB", pack=False, align=None), "certHash": SimStruct({"size": SimTypeInt(signed=False, label="UInt32"), "data": SimTypePointer(SimTypeChar(label="Byte"), offset=0)}, name="FWP_BYTE_BLOB", pack=False, align=None), "eku": SimTypePointer(SimStruct({"numEku": SimTypeInt(signed=False, label="UInt32"), "eku": SimTypePointer(SimTypePointer(SimTypeChar(label="Byte"), offset=0), offset=0)}, name="IKEEXT_CERT_EKUS0", pack=False, align=None), offset=0), "name": SimTypePointer(SimStruct({"nameType": SimTypeInt(signed=False, label="IKEEXT_CERT_CRITERIA_NAME_TYPE"), "certName": SimTypePointer(SimTypeChar(label="Char"), offset=0)}, name="IKEEXT_CERT_NAME0", pack=False, align=None), offset=0), "flags": SimTypeInt(signed=False, label="UInt32")}, name="IKEEXT_CERTIFICATE_CRITERIA0", pack=False, align=None), offset=0)}, name="_Anonymous3_e__Struct", pack=False, align=None)}, name="<anon>", label="None"), "outboundConfigType": SimTypeInt(signed=False, label="IKEEXT_CERT_CONFIG_TYPE"), "Anonymous2": SimUnion({"Anonymous1": SimStruct({"outboundRootArraySize": SimTypeInt(signed=False, label="UInt32"), "outboundRootCriteria": SimTypePointer(SimStruct({"certData": SimStruct({"size": SimTypeInt(signed=False, label="UInt32"), "data": SimTypePointer(SimTypeChar(label="Byte"), offset=0)}, name="FWP_BYTE_BLOB", pack=False, align=None), "certHash": SimStruct({"size": SimTypeInt(signed=False, label="UInt32"), "data": SimTypePointer(SimTypeChar(label="Byte"), offset=0)}, name="FWP_BYTE_BLOB", pack=False, align=None), "eku": SimTypePointer(SimStruct({"numEku": SimTypeInt(signed=False, label="UInt32"), "eku": SimTypePointer(SimTypePointer(SimTypeChar(label="Byte"), offset=0), offset=0)}, name="IKEEXT_CERT_EKUS0", pack=False, align=None), offset=0), "name": SimTypePointer(SimStruct({"nameType": SimTypeInt(signed=False, label="IKEEXT_CERT_CRITERIA_NAME_TYPE"), "certName": SimTypePointer(SimTypeChar(label="Char"), offset=0)}, name="IKEEXT_CERT_NAME0", pack=False, align=None), offset=0), "flags": SimTypeInt(signed=False, label="UInt32")}, name="IKEEXT_CERTIFICATE_CRITERIA0", pack=False, align=None), offset=0)}, name="_Anonymous1_e__Struct", pack=False, align=None), "Anonymous2": SimStruct({"outboundEnterpriseStoreArraySize": SimTypeInt(signed=False, label="UInt32"), "outboundEnterpriseStoreCriteria": SimTypePointer(SimStruct({"certData": SimStruct({"size": SimTypeInt(signed=False, label="UInt32"), "data": SimTypePointer(SimTypeChar(label="Byte"), offset=0)}, name="FWP_BYTE_BLOB", pack=False, align=None), "certHash": SimStruct({"size": SimTypeInt(signed=False, label="UInt32"), "data": SimTypePointer(SimTypeChar(label="Byte"), offset=0)}, name="FWP_BYTE_BLOB", pack=False, align=None), "eku": SimTypePointer(SimStruct({"numEku": SimTypeInt(signed=False, label="UInt32"), "eku": SimTypePointer(SimTypePointer(SimTypeChar(label="Byte"), offset=0), offset=0)}, name="IKEEXT_CERT_EKUS0", pack=False, align=None), offset=0), "name": SimTypePointer(SimStruct({"nameType": SimTypeInt(signed=False, label="IKEEXT_CERT_CRITERIA_NAME_TYPE"), "certName": SimTypePointer(SimTypeChar(label="Char"), offset=0)}, name="IKEEXT_CERT_NAME0", pack=False, align=None), offset=0), "flags": SimTypeInt(signed=False, label="UInt32")}, name="IKEEXT_CERTIFICATE_CRITERIA0", pack=False, align=None), offset=0)}, name="_Anonymous2_e__Struct", pack=False, align=None), "Anonymous3": SimStruct({"outboundRootStoreArraySize": SimTypeInt(signed=False, label="UInt32"), "outboundTrustedRootStoreCriteria": SimTypePointer(SimStruct({"certData": SimStruct({"size": SimTypeInt(signed=False, label="UInt32"), "data": SimTypePointer(SimTypeChar(label="Byte"), offset=0)}, name="FWP_BYTE_BLOB", pack=False, align=None), "certHash": SimStruct({"size": SimTypeInt(signed=False, label="UInt32"), "data": SimTypePointer(SimTypeChar(label="Byte"), offset=0)}, name="FWP_BYTE_BLOB", pack=False, align=None), "eku": SimTypePointer(SimStruct({"numEku": SimTypeInt(signed=False, label="UInt32"), "eku": SimTypePointer(SimTypePointer(SimTypeChar(label="Byte"), offset=0), offset=0)}, name="IKEEXT_CERT_EKUS0", pack=False, align=None), offset=0), "name": SimTypePointer(SimStruct({"nameType": SimTypeInt(signed=False, label="IKEEXT_CERT_CRITERIA_NAME_TYPE"), "certName": SimTypePointer(SimTypeChar(label="Char"), offset=0)}, name="IKEEXT_CERT_NAME0", pack=False, align=None), offset=0), "flags": SimTypeInt(signed=False, label="UInt32")}, name="IKEEXT_CERTIFICATE_CRITERIA0", pack=False, align=None), offset=0)}, name="_Anonymous3_e__Struct", pack=False, align=None)}, name="<anon>", label="None"), "flags": SimTypeInt(signed=False, label="IKEEXT_CERT_AUTH"), "localCertLocationUrl": SimStruct({"size": SimTypeInt(signed=False, label="UInt32"), "data": SimTypePointer(SimTypeChar(label="Byte"), offset=0)}, name="FWP_BYTE_BLOB", pack=False, align=None)}, name="IKEEXT_CERTIFICATE_AUTHENTICATION2", pack=False, align=None), "cgaAuthentication": SimStruct({"keyContainerName": SimTypePointer(SimTypeChar(label="Char"), offset=0), "cspName": SimTypePointer(SimTypeChar(label="Char"), offset=0), "cspType": SimTypeInt(signed=False, label="UInt32"), "cgaModifier": SimStruct({"byteArray16": SimTypeFixedSizeArray(SimTypeChar(label="Byte"), 16)}, name="FWP_BYTE_ARRAY16", pack=False, align=None), "cgaCollisionCount": SimTypeChar(label="Byte")}, name="IKEEXT_IPV6_CGA_AUTHENTICATION0", pack=False, align=None), "eapAuthentication": SimStruct({"flags": SimTypeInt(signed=False, label="IKEEXT_EAP_AUTHENTICATION_FLAGS")}, name="IKEEXT_EAP_AUTHENTICATION0", pack=False, align=None)}, name="<anon>", label="None")}, name="IKEEXT_AUTHENTICATION_METHOD2", pack=False, align=None), offset=0), "initiatorImpersonationType": SimTypeInt(signed=False, label="IKEEXT_AUTHENTICATION_IMPERSONATION_TYPE"), "numIkeProposals": SimTypeInt(signed=False, label="UInt32"), "ikeProposals": SimTypePointer(SimStruct({"cipherAlgorithm": SimStruct({"algoIdentifier": SimTypeInt(signed=False, label="IKEEXT_CIPHER_TYPE"), "keyLen": SimTypeInt(signed=False, label="UInt32"), "rounds": SimTypeInt(signed=False, label="UInt32")}, name="IKEEXT_CIPHER_ALGORITHM0", pack=False, align=None), "integrityAlgorithm": SimStruct({"algoIdentifier": SimTypeInt(signed=False, label="IKEEXT_INTEGRITY_TYPE")}, name="IKEEXT_INTEGRITY_ALGORITHM0", pack=False, align=None), "maxLifetimeSeconds": SimTypeInt(signed=False, label="UInt32"), "dhGroup": SimTypeInt(signed=False, label="IKEEXT_DH_GROUP"), "quickModeLimit": SimTypeInt(signed=False, label="UInt32")}, name="IKEEXT_PROPOSAL0", pack=False, align=None), offset=0), "flags": SimTypeInt(signed=False, label="IKEEXT_POLICY_FLAG"), "maxDynamicFilters": SimTypeInt(signed=False, label="UInt32"), "retransmitDurationSecs": SimTypeInt(signed=False, label="UInt32")}, name="IKEEXT_POLICY2", pack=False, align=None), offset=0), "idpOptions": SimTypePointer(SimStruct({"stateIdleTimeoutSeconds": SimTypeInt(signed=False, label="UInt32"), "perIPRateLimitQueueIdleTimeoutSeconds": SimTypeInt(signed=False, label="UInt32"), "ipV6IPsecUnauthDscp": SimTypeChar(label="Byte"), "ipV6IPsecUnauthRateLimitBytesPerSec": SimTypeInt(signed=False, label="UInt32"), "ipV6IPsecUnauthPerIPRateLimitBytesPerSec": SimTypeInt(signed=False, label="UInt32"), "ipV6IPsecAuthDscp": SimTypeChar(label="Byte"), "ipV6IPsecAuthRateLimitBytesPerSec": SimTypeInt(signed=False, label="UInt32"), "icmpV6Dscp": SimTypeChar(label="Byte"), "icmpV6RateLimitBytesPerSec": SimTypeInt(signed=False, label="UInt32"), "ipV6FilterExemptDscp": SimTypeChar(label="Byte"), "ipV6FilterExemptRateLimitBytesPerSec": SimTypeInt(signed=False, label="UInt32"), "defBlockExemptDscp": SimTypeChar(label="Byte"), "defBlockExemptRateLimitBytesPerSec": SimTypeInt(signed=False, label="UInt32"), "maxStateEntries": SimTypeInt(signed=False, label="UInt32"), "maxPerIPRateLimitQueues": SimTypeInt(signed=False, label="UInt32"), "flags": SimTypeInt(signed=False, label="IPSEC_DOSP_FLAGS"), "numPublicIFLuids": SimTypeInt(signed=False, label="UInt32"), "publicIFLuids": SimTypePointer(SimTypeLongLong(signed=False, label="UInt64"), offset=0), "numInternalIFLuids": SimTypeInt(signed=False, label="UInt32"), "internalIFLuids": SimTypePointer(SimTypeLongLong(signed=False, label="UInt64"), offset=0), "publicV6AddrMask": SimStruct({"addr": SimTypeFixedSizeArray(SimTypeChar(label="Byte"), 16), "prefixLength": SimTypeChar(label="Byte")}, name="FWP_V6_ADDR_AND_MASK", pack=False, align=None), "internalV6AddrMask": SimStruct({"addr": SimTypeFixedSizeArray(SimTypeChar(label="Byte"), 16), "prefixLength": SimTypeChar(label="Byte")}, name="FWP_V6_ADDR_AND_MASK", pack=False, align=None)}, name="IPSEC_DOSP_OPTIONS0", pack=False, align=None), offset=0)}, name="<anon>", label="None"), "providerContextId": SimTypeLongLong(signed=False, label="UInt64")}, name="FWPM_PROVIDER_CONTEXT2", pack=False, align=None), offset=0), offset=0)], SimTypeInt(signed=False, label="UInt32"), arg_names=["engineHandle", "key", "providerContext"]),
        # 
        'FwpmProviderContextGetByKey3': SimTypeFunction([SimTypePointer(SimTypeInt(signed=True, label="Int"), label="IntPtr", offset=0), SimTypePointer(SimTypeBottom(label="Guid"), offset=0), SimTypePointer(SimTypePointer(SimStruct({"providerContextKey": SimTypeBottom(label="Guid"), "displayData": SimStruct({"name": SimTypePointer(SimTypeChar(label="Char"), offset=0), "description": SimTypePointer(SimTypeChar(label="Char"), offset=0)}, name="FWPM_DISPLAY_DATA0", pack=False, align=None), "flags": SimTypeInt(signed=False, label="UInt32"), "providerKey": SimTypePointer(SimTypeBottom(label="Guid"), offset=0), "providerData": SimStruct({"size": SimTypeInt(signed=False, label="UInt32"), "data": SimTypePointer(SimTypeChar(label="Byte"), offset=0)}, name="FWP_BYTE_BLOB", pack=False, align=None), "type": SimTypeInt(signed=False, label="FWPM_PROVIDER_CONTEXT_TYPE"), "Anonymous": SimUnion({"keyingPolicy": SimTypePointer(SimStruct({"numKeyMods": SimTypeInt(signed=False, label="UInt32"), "keyModKeys": SimTypePointer(SimTypeBottom(label="Guid"), offset=0), "flags": SimTypeInt(signed=False, label="UInt32")}, name="IPSEC_KEYING_POLICY1", pack=False, align=None), offset=0), "ikeQmTransportPolicy": SimTypePointer(SimStruct({"numIpsecProposals": SimTypeInt(signed=False, label="UInt32"), "ipsecProposals": SimTypePointer(SimStruct({"lifetime": SimStruct({"lifetimeSeconds": SimTypeInt(signed=False, label="UInt32"), "lifetimeKilobytes": SimTypeInt(signed=False, label="UInt32"), "lifetimePackets": SimTypeInt(signed=False, label="UInt32")}, name="IPSEC_SA_LIFETIME0", pack=False, align=None), "numSaTransforms": SimTypeInt(signed=False, label="UInt32"), "saTransforms": SimTypePointer(SimStruct({"ipsecTransformType": SimTypeInt(signed=False, label="IPSEC_TRANSFORM_TYPE"), "Anonymous": SimUnion({"ahTransform": SimTypePointer(SimStruct({"authTransformId": SimStruct({"authType": SimTypeInt(signed=False, label="IPSEC_AUTH_TYPE"), "authConfig": SimTypeChar(label="Byte")}, name="IPSEC_AUTH_TRANSFORM_ID0", pack=False, align=None), "cryptoModuleId": SimTypePointer(SimTypeBottom(label="Guid"), offset=0)}, name="IPSEC_AUTH_TRANSFORM0", pack=False, align=None), offset=0), "espAuthTransform": SimTypePointer(SimStruct({"authTransformId": SimStruct({"authType": SimTypeInt(signed=False, label="IPSEC_AUTH_TYPE"), "authConfig": SimTypeChar(label="Byte")}, name="IPSEC_AUTH_TRANSFORM_ID0", pack=False, align=None), "cryptoModuleId": SimTypePointer(SimTypeBottom(label="Guid"), offset=0)}, name="IPSEC_AUTH_TRANSFORM0", pack=False, align=None), offset=0), "espCipherTransform": SimTypePointer(SimStruct({"cipherTransformId": SimStruct({"cipherType": SimTypeInt(signed=False, label="IPSEC_CIPHER_TYPE"), "cipherConfig": SimTypeChar(label="Byte")}, name="IPSEC_CIPHER_TRANSFORM_ID0", pack=False, align=None), "cryptoModuleId": SimTypePointer(SimTypeBottom(label="Guid"), offset=0)}, name="IPSEC_CIPHER_TRANSFORM0", pack=False, align=None), offset=0), "espAuthAndCipherTransform": SimTypePointer(SimStruct({"authTransform": SimStruct({"authTransformId": SimStruct({"authType": SimTypeInt(signed=False, label="IPSEC_AUTH_TYPE"), "authConfig": SimTypeChar(label="Byte")}, name="IPSEC_AUTH_TRANSFORM_ID0", pack=False, align=None), "cryptoModuleId": SimTypePointer(SimTypeBottom(label="Guid"), offset=0)}, name="IPSEC_AUTH_TRANSFORM0", pack=False, align=None), "cipherTransform": SimStruct({"cipherTransformId": SimStruct({"cipherType": SimTypeInt(signed=False, label="IPSEC_CIPHER_TYPE"), "cipherConfig": SimTypeChar(label="Byte")}, name="IPSEC_CIPHER_TRANSFORM_ID0", pack=False, align=None), "cryptoModuleId": SimTypePointer(SimTypeBottom(label="Guid"), offset=0)}, name="IPSEC_CIPHER_TRANSFORM0", pack=False, align=None)}, name="IPSEC_AUTH_AND_CIPHER_TRANSFORM0", pack=False, align=None), offset=0), "espAuthFwTransform": SimTypePointer(SimStruct({"authTransformId": SimStruct({"authType": SimTypeInt(signed=False, label="IPSEC_AUTH_TYPE"), "authConfig": SimTypeChar(label="Byte")}, name="IPSEC_AUTH_TRANSFORM_ID0", pack=False, align=None), "cryptoModuleId": SimTypePointer(SimTypeBottom(label="Guid"), offset=0)}, name="IPSEC_AUTH_TRANSFORM0", pack=False, align=None), offset=0)}, name="<anon>", label="None")}, name="IPSEC_SA_TRANSFORM0", pack=False, align=None), offset=0), "pfsGroup": SimTypeInt(signed=False, label="IPSEC_PFS_GROUP")}, name="IPSEC_PROPOSAL0", pack=False, align=None), offset=0), "flags": SimTypeInt(signed=False, label="IPSEC_POLICY_FLAG"), "ndAllowClearTimeoutSeconds": SimTypeInt(signed=False, label="UInt32"), "saIdleTimeout": SimStruct({"idleTimeoutSeconds": SimTypeInt(signed=False, label="UInt32"), "idleTimeoutSecondsFailOver": SimTypeInt(signed=False, label="UInt32")}, name="IPSEC_SA_IDLE_TIMEOUT0", pack=False, align=None), "emPolicy": SimTypePointer(SimStruct({"numAuthenticationMethods": SimTypeInt(signed=False, label="UInt32"), "authenticationMethods": SimTypePointer(SimStruct({"authenticationMethodType": SimTypeInt(signed=False, label="IKEEXT_AUTHENTICATION_METHOD_TYPE"), "Anonymous": SimUnion({"presharedKeyAuthentication": SimStruct({"presharedKey": SimStruct({"size": SimTypeInt(signed=False, label="UInt32"), "data": SimTypePointer(SimTypeChar(label="Byte"), offset=0)}, name="FWP_BYTE_BLOB", pack=False, align=None), "flags": SimTypeInt(signed=False, label="IKEEXT_PRESHARED_KEY_AUTHENTICATION_FLAGS")}, name="IKEEXT_PRESHARED_KEY_AUTHENTICATION1", pack=False, align=None), "certificateAuthentication": SimStruct({"inboundConfigType": SimTypeInt(signed=False, label="IKEEXT_CERT_CONFIG_TYPE"), "Anonymous1": SimUnion({"Anonymous1": SimStruct({"inboundRootArraySize": SimTypeInt(signed=False, label="UInt32"), "inboundRootCriteria": SimTypePointer(SimStruct({"certData": SimStruct({"size": SimTypeInt(signed=False, label="UInt32"), "data": SimTypePointer(SimTypeChar(label="Byte"), offset=0)}, name="FWP_BYTE_BLOB", pack=False, align=None), "certHash": SimStruct({"size": SimTypeInt(signed=False, label="UInt32"), "data": SimTypePointer(SimTypeChar(label="Byte"), offset=0)}, name="FWP_BYTE_BLOB", pack=False, align=None), "eku": SimTypePointer(SimStruct({"numEku": SimTypeInt(signed=False, label="UInt32"), "eku": SimTypePointer(SimTypePointer(SimTypeChar(label="Byte"), offset=0), offset=0)}, name="IKEEXT_CERT_EKUS0", pack=False, align=None), offset=0), "name": SimTypePointer(SimStruct({"nameType": SimTypeInt(signed=False, label="IKEEXT_CERT_CRITERIA_NAME_TYPE"), "certName": SimTypePointer(SimTypeChar(label="Char"), offset=0)}, name="IKEEXT_CERT_NAME0", pack=False, align=None), offset=0), "flags": SimTypeInt(signed=False, label="UInt32")}, name="IKEEXT_CERTIFICATE_CRITERIA0", pack=False, align=None), offset=0)}, name="_Anonymous1_e__Struct", pack=False, align=None), "Anonymous2": SimStruct({"inboundEnterpriseStoreArraySize": SimTypeInt(signed=False, label="UInt32"), "inboundEnterpriseStoreCriteria": SimTypePointer(SimStruct({"certData": SimStruct({"size": SimTypeInt(signed=False, label="UInt32"), "data": SimTypePointer(SimTypeChar(label="Byte"), offset=0)}, name="FWP_BYTE_BLOB", pack=False, align=None), "certHash": SimStruct({"size": SimTypeInt(signed=False, label="UInt32"), "data": SimTypePointer(SimTypeChar(label="Byte"), offset=0)}, name="FWP_BYTE_BLOB", pack=False, align=None), "eku": SimTypePointer(SimStruct({"numEku": SimTypeInt(signed=False, label="UInt32"), "eku": SimTypePointer(SimTypePointer(SimTypeChar(label="Byte"), offset=0), offset=0)}, name="IKEEXT_CERT_EKUS0", pack=False, align=None), offset=0), "name": SimTypePointer(SimStruct({"nameType": SimTypeInt(signed=False, label="IKEEXT_CERT_CRITERIA_NAME_TYPE"), "certName": SimTypePointer(SimTypeChar(label="Char"), offset=0)}, name="IKEEXT_CERT_NAME0", pack=False, align=None), offset=0), "flags": SimTypeInt(signed=False, label="UInt32")}, name="IKEEXT_CERTIFICATE_CRITERIA0", pack=False, align=None), offset=0)}, name="_Anonymous2_e__Struct", pack=False, align=None), "Anonymous3": SimStruct({"inboundRootStoreArraySize": SimTypeInt(signed=False, label="UInt32"), "inboundTrustedRootStoreCriteria": SimTypePointer(SimStruct({"certData": SimStruct({"size": SimTypeInt(signed=False, label="UInt32"), "data": SimTypePointer(SimTypeChar(label="Byte"), offset=0)}, name="FWP_BYTE_BLOB", pack=False, align=None), "certHash": SimStruct({"size": SimTypeInt(signed=False, label="UInt32"), "data": SimTypePointer(SimTypeChar(label="Byte"), offset=0)}, name="FWP_BYTE_BLOB", pack=False, align=None), "eku": SimTypePointer(SimStruct({"numEku": SimTypeInt(signed=False, label="UInt32"), "eku": SimTypePointer(SimTypePointer(SimTypeChar(label="Byte"), offset=0), offset=0)}, name="IKEEXT_CERT_EKUS0", pack=False, align=None), offset=0), "name": SimTypePointer(SimStruct({"nameType": SimTypeInt(signed=False, label="IKEEXT_CERT_CRITERIA_NAME_TYPE"), "certName": SimTypePointer(SimTypeChar(label="Char"), offset=0)}, name="IKEEXT_CERT_NAME0", pack=False, align=None), offset=0), "flags": SimTypeInt(signed=False, label="UInt32")}, name="IKEEXT_CERTIFICATE_CRITERIA0", pack=False, align=None), offset=0)}, name="_Anonymous3_e__Struct", pack=False, align=None)}, name="<anon>", label="None"), "outboundConfigType": SimTypeInt(signed=False, label="IKEEXT_CERT_CONFIG_TYPE"), "Anonymous2": SimUnion({"Anonymous1": SimStruct({"outboundRootArraySize": SimTypeInt(signed=False, label="UInt32"), "outboundRootCriteria": SimTypePointer(SimStruct({"certData": SimStruct({"size": SimTypeInt(signed=False, label="UInt32"), "data": SimTypePointer(SimTypeChar(label="Byte"), offset=0)}, name="FWP_BYTE_BLOB", pack=False, align=None), "certHash": SimStruct({"size": SimTypeInt(signed=False, label="UInt32"), "data": SimTypePointer(SimTypeChar(label="Byte"), offset=0)}, name="FWP_BYTE_BLOB", pack=False, align=None), "eku": SimTypePointer(SimStruct({"numEku": SimTypeInt(signed=False, label="UInt32"), "eku": SimTypePointer(SimTypePointer(SimTypeChar(label="Byte"), offset=0), offset=0)}, name="IKEEXT_CERT_EKUS0", pack=False, align=None), offset=0), "name": SimTypePointer(SimStruct({"nameType": SimTypeInt(signed=False, label="IKEEXT_CERT_CRITERIA_NAME_TYPE"), "certName": SimTypePointer(SimTypeChar(label="Char"), offset=0)}, name="IKEEXT_CERT_NAME0", pack=False, align=None), offset=0), "flags": SimTypeInt(signed=False, label="UInt32")}, name="IKEEXT_CERTIFICATE_CRITERIA0", pack=False, align=None), offset=0)}, name="_Anonymous1_e__Struct", pack=False, align=None), "Anonymous2": SimStruct({"outboundEnterpriseStoreArraySize": SimTypeInt(signed=False, label="UInt32"), "outboundEnterpriseStoreCriteria": SimTypePointer(SimStruct({"certData": SimStruct({"size": SimTypeInt(signed=False, label="UInt32"), "data": SimTypePointer(SimTypeChar(label="Byte"), offset=0)}, name="FWP_BYTE_BLOB", pack=False, align=None), "certHash": SimStruct({"size": SimTypeInt(signed=False, label="UInt32"), "data": SimTypePointer(SimTypeChar(label="Byte"), offset=0)}, name="FWP_BYTE_BLOB", pack=False, align=None), "eku": SimTypePointer(SimStruct({"numEku": SimTypeInt(signed=False, label="UInt32"), "eku": SimTypePointer(SimTypePointer(SimTypeChar(label="Byte"), offset=0), offset=0)}, name="IKEEXT_CERT_EKUS0", pack=False, align=None), offset=0), "name": SimTypePointer(SimStruct({"nameType": SimTypeInt(signed=False, label="IKEEXT_CERT_CRITERIA_NAME_TYPE"), "certName": SimTypePointer(SimTypeChar(label="Char"), offset=0)}, name="IKEEXT_CERT_NAME0", pack=False, align=None), offset=0), "flags": SimTypeInt(signed=False, label="UInt32")}, name="IKEEXT_CERTIFICATE_CRITERIA0", pack=False, align=None), offset=0)}, name="_Anonymous2_e__Struct", pack=False, align=None), "Anonymous3": SimStruct({"outboundRootStoreArraySize": SimTypeInt(signed=False, label="UInt32"), "outboundTrustedRootStoreCriteria": SimTypePointer(SimStruct({"certData": SimStruct({"size": SimTypeInt(signed=False, label="UInt32"), "data": SimTypePointer(SimTypeChar(label="Byte"), offset=0)}, name="FWP_BYTE_BLOB", pack=False, align=None), "certHash": SimStruct({"size": SimTypeInt(signed=False, label="UInt32"), "data": SimTypePointer(SimTypeChar(label="Byte"), offset=0)}, name="FWP_BYTE_BLOB", pack=False, align=None), "eku": SimTypePointer(SimStruct({"numEku": SimTypeInt(signed=False, label="UInt32"), "eku": SimTypePointer(SimTypePointer(SimTypeChar(label="Byte"), offset=0), offset=0)}, name="IKEEXT_CERT_EKUS0", pack=False, align=None), offset=0), "name": SimTypePointer(SimStruct({"nameType": SimTypeInt(signed=False, label="IKEEXT_CERT_CRITERIA_NAME_TYPE"), "certName": SimTypePointer(SimTypeChar(label="Char"), offset=0)}, name="IKEEXT_CERT_NAME0", pack=False, align=None), offset=0), "flags": SimTypeInt(signed=False, label="UInt32")}, name="IKEEXT_CERTIFICATE_CRITERIA0", pack=False, align=None), offset=0)}, name="_Anonymous3_e__Struct", pack=False, align=None)}, name="<anon>", label="None"), "flags": SimTypeInt(signed=False, label="IKEEXT_CERT_AUTH"), "localCertLocationUrl": SimStruct({"size": SimTypeInt(signed=False, label="UInt32"), "data": SimTypePointer(SimTypeChar(label="Byte"), offset=0)}, name="FWP_BYTE_BLOB", pack=False, align=None)}, name="IKEEXT_CERTIFICATE_AUTHENTICATION2", pack=False, align=None), "kerberosAuthentication": SimStruct({"flags": SimTypeInt(signed=False, label="IKEEXT_KERBEROS_AUTHENTICATION_FLAGS"), "proxyServer": SimTypePointer(SimTypeChar(label="Char"), offset=0)}, name="IKEEXT_KERBEROS_AUTHENTICATION1", pack=False, align=None), "reservedAuthentication": SimStruct({"flags": SimTypeInt(signed=False, label="IKEEXT_RESERVED_AUTHENTICATION_FLAGS")}, name="IKEEXT_RESERVED_AUTHENTICATION0", pack=False, align=None), "ntlmV2Authentication": SimStruct({"flags": SimTypeInt(signed=False, label="UInt32")}, name="IKEEXT_NTLM_V2_AUTHENTICATION0", pack=False, align=None), "sslAuthentication": SimStruct({"inboundConfigType": SimTypeInt(signed=False, label="IKEEXT_CERT_CONFIG_TYPE"), "Anonymous1": SimUnion({"Anonymous1": SimStruct({"inboundRootArraySize": SimTypeInt(signed=False, label="UInt32"), "inboundRootCriteria": SimTypePointer(SimStruct({"certData": SimStruct({"size": SimTypeInt(signed=False, label="UInt32"), "data": SimTypePointer(SimTypeChar(label="Byte"), offset=0)}, name="FWP_BYTE_BLOB", pack=False, align=None), "certHash": SimStruct({"size": SimTypeInt(signed=False, label="UInt32"), "data": SimTypePointer(SimTypeChar(label="Byte"), offset=0)}, name="FWP_BYTE_BLOB", pack=False, align=None), "eku": SimTypePointer(SimStruct({"numEku": SimTypeInt(signed=False, label="UInt32"), "eku": SimTypePointer(SimTypePointer(SimTypeChar(label="Byte"), offset=0), offset=0)}, name="IKEEXT_CERT_EKUS0", pack=False, align=None), offset=0), "name": SimTypePointer(SimStruct({"nameType": SimTypeInt(signed=False, label="IKEEXT_CERT_CRITERIA_NAME_TYPE"), "certName": SimTypePointer(SimTypeChar(label="Char"), offset=0)}, name="IKEEXT_CERT_NAME0", pack=False, align=None), offset=0), "flags": SimTypeInt(signed=False, label="UInt32")}, name="IKEEXT_CERTIFICATE_CRITERIA0", pack=False, align=None), offset=0)}, name="_Anonymous1_e__Struct", pack=False, align=None), "Anonymous2": SimStruct({"inboundEnterpriseStoreArraySize": SimTypeInt(signed=False, label="UInt32"), "inboundEnterpriseStoreCriteria": SimTypePointer(SimStruct({"certData": SimStruct({"size": SimTypeInt(signed=False, label="UInt32"), "data": SimTypePointer(SimTypeChar(label="Byte"), offset=0)}, name="FWP_BYTE_BLOB", pack=False, align=None), "certHash": SimStruct({"size": SimTypeInt(signed=False, label="UInt32"), "data": SimTypePointer(SimTypeChar(label="Byte"), offset=0)}, name="FWP_BYTE_BLOB", pack=False, align=None), "eku": SimTypePointer(SimStruct({"numEku": SimTypeInt(signed=False, label="UInt32"), "eku": SimTypePointer(SimTypePointer(SimTypeChar(label="Byte"), offset=0), offset=0)}, name="IKEEXT_CERT_EKUS0", pack=False, align=None), offset=0), "name": SimTypePointer(SimStruct({"nameType": SimTypeInt(signed=False, label="IKEEXT_CERT_CRITERIA_NAME_TYPE"), "certName": SimTypePointer(SimTypeChar(label="Char"), offset=0)}, name="IKEEXT_CERT_NAME0", pack=False, align=None), offset=0), "flags": SimTypeInt(signed=False, label="UInt32")}, name="IKEEXT_CERTIFICATE_CRITERIA0", pack=False, align=None), offset=0)}, name="_Anonymous2_e__Struct", pack=False, align=None), "Anonymous3": SimStruct({"inboundRootStoreArraySize": SimTypeInt(signed=False, label="UInt32"), "inboundTrustedRootStoreCriteria": SimTypePointer(SimStruct({"certData": SimStruct({"size": SimTypeInt(signed=False, label="UInt32"), "data": SimTypePointer(SimTypeChar(label="Byte"), offset=0)}, name="FWP_BYTE_BLOB", pack=False, align=None), "certHash": SimStruct({"size": SimTypeInt(signed=False, label="UInt32"), "data": SimTypePointer(SimTypeChar(label="Byte"), offset=0)}, name="FWP_BYTE_BLOB", pack=False, align=None), "eku": SimTypePointer(SimStruct({"numEku": SimTypeInt(signed=False, label="UInt32"), "eku": SimTypePointer(SimTypePointer(SimTypeChar(label="Byte"), offset=0), offset=0)}, name="IKEEXT_CERT_EKUS0", pack=False, align=None), offset=0), "name": SimTypePointer(SimStruct({"nameType": SimTypeInt(signed=False, label="IKEEXT_CERT_CRITERIA_NAME_TYPE"), "certName": SimTypePointer(SimTypeChar(label="Char"), offset=0)}, name="IKEEXT_CERT_NAME0", pack=False, align=None), offset=0), "flags": SimTypeInt(signed=False, label="UInt32")}, name="IKEEXT_CERTIFICATE_CRITERIA0", pack=False, align=None), offset=0)}, name="_Anonymous3_e__Struct", pack=False, align=None)}, name="<anon>", label="None"), "outboundConfigType": SimTypeInt(signed=False, label="IKEEXT_CERT_CONFIG_TYPE"), "Anonymous2": SimUnion({"Anonymous1": SimStruct({"outboundRootArraySize": SimTypeInt(signed=False, label="UInt32"), "outboundRootCriteria": SimTypePointer(SimStruct({"certData": SimStruct({"size": SimTypeInt(signed=False, label="UInt32"), "data": SimTypePointer(SimTypeChar(label="Byte"), offset=0)}, name="FWP_BYTE_BLOB", pack=False, align=None), "certHash": SimStruct({"size": SimTypeInt(signed=False, label="UInt32"), "data": SimTypePointer(SimTypeChar(label="Byte"), offset=0)}, name="FWP_BYTE_BLOB", pack=False, align=None), "eku": SimTypePointer(SimStruct({"numEku": SimTypeInt(signed=False, label="UInt32"), "eku": SimTypePointer(SimTypePointer(SimTypeChar(label="Byte"), offset=0), offset=0)}, name="IKEEXT_CERT_EKUS0", pack=False, align=None), offset=0), "name": SimTypePointer(SimStruct({"nameType": SimTypeInt(signed=False, label="IKEEXT_CERT_CRITERIA_NAME_TYPE"), "certName": SimTypePointer(SimTypeChar(label="Char"), offset=0)}, name="IKEEXT_CERT_NAME0", pack=False, align=None), offset=0), "flags": SimTypeInt(signed=False, label="UInt32")}, name="IKEEXT_CERTIFICATE_CRITERIA0", pack=False, align=None), offset=0)}, name="_Anonymous1_e__Struct", pack=False, align=None), "Anonymous2": SimStruct({"outboundEnterpriseStoreArraySize": SimTypeInt(signed=False, label="UInt32"), "outboundEnterpriseStoreCriteria": SimTypePointer(SimStruct({"certData": SimStruct({"size": SimTypeInt(signed=False, label="UInt32"), "data": SimTypePointer(SimTypeChar(label="Byte"), offset=0)}, name="FWP_BYTE_BLOB", pack=False, align=None), "certHash": SimStruct({"size": SimTypeInt(signed=False, label="UInt32"), "data": SimTypePointer(SimTypeChar(label="Byte"), offset=0)}, name="FWP_BYTE_BLOB", pack=False, align=None), "eku": SimTypePointer(SimStruct({"numEku": SimTypeInt(signed=False, label="UInt32"), "eku": SimTypePointer(SimTypePointer(SimTypeChar(label="Byte"), offset=0), offset=0)}, name="IKEEXT_CERT_EKUS0", pack=False, align=None), offset=0), "name": SimTypePointer(SimStruct({"nameType": SimTypeInt(signed=False, label="IKEEXT_CERT_CRITERIA_NAME_TYPE"), "certName": SimTypePointer(SimTypeChar(label="Char"), offset=0)}, name="IKEEXT_CERT_NAME0", pack=False, align=None), offset=0), "flags": SimTypeInt(signed=False, label="UInt32")}, name="IKEEXT_CERTIFICATE_CRITERIA0", pack=False, align=None), offset=0)}, name="_Anonymous2_e__Struct", pack=False, align=None), "Anonymous3": SimStruct({"outboundRootStoreArraySize": SimTypeInt(signed=False, label="UInt32"), "outboundTrustedRootStoreCriteria": SimTypePointer(SimStruct({"certData": SimStruct({"size": SimTypeInt(signed=False, label="UInt32"), "data": SimTypePointer(SimTypeChar(label="Byte"), offset=0)}, name="FWP_BYTE_BLOB", pack=False, align=None), "certHash": SimStruct({"size": SimTypeInt(signed=False, label="UInt32"), "data": SimTypePointer(SimTypeChar(label="Byte"), offset=0)}, name="FWP_BYTE_BLOB", pack=False, align=None), "eku": SimTypePointer(SimStruct({"numEku": SimTypeInt(signed=False, label="UInt32"), "eku": SimTypePointer(SimTypePointer(SimTypeChar(label="Byte"), offset=0), offset=0)}, name="IKEEXT_CERT_EKUS0", pack=False, align=None), offset=0), "name": SimTypePointer(SimStruct({"nameType": SimTypeInt(signed=False, label="IKEEXT_CERT_CRITERIA_NAME_TYPE"), "certName": SimTypePointer(SimTypeChar(label="Char"), offset=0)}, name="IKEEXT_CERT_NAME0", pack=False, align=None), offset=0), "flags": SimTypeInt(signed=False, label="UInt32")}, name="IKEEXT_CERTIFICATE_CRITERIA0", pack=False, align=None), offset=0)}, name="_Anonymous3_e__Struct", pack=False, align=None)}, name="<anon>", label="None"), "flags": SimTypeInt(signed=False, label="IKEEXT_CERT_AUTH"), "localCertLocationUrl": SimStruct({"size": SimTypeInt(signed=False, label="UInt32"), "data": SimTypePointer(SimTypeChar(label="Byte"), offset=0)}, name="FWP_BYTE_BLOB", pack=False, align=None)}, name="IKEEXT_CERTIFICATE_AUTHENTICATION2", pack=False, align=None), "cgaAuthentication": SimStruct({"keyContainerName": SimTypePointer(SimTypeChar(label="Char"), offset=0), "cspName": SimTypePointer(SimTypeChar(label="Char"), offset=0), "cspType": SimTypeInt(signed=False, label="UInt32"), "cgaModifier": SimStruct({"byteArray16": SimTypeFixedSizeArray(SimTypeChar(label="Byte"), 16)}, name="FWP_BYTE_ARRAY16", pack=False, align=None), "cgaCollisionCount": SimTypeChar(label="Byte")}, name="IKEEXT_IPV6_CGA_AUTHENTICATION0", pack=False, align=None), "eapAuthentication": SimStruct({"flags": SimTypeInt(signed=False, label="IKEEXT_EAP_AUTHENTICATION_FLAGS")}, name="IKEEXT_EAP_AUTHENTICATION0", pack=False, align=None)}, name="<anon>", label="None")}, name="IKEEXT_AUTHENTICATION_METHOD2", pack=False, align=None), offset=0), "initiatorImpersonationType": SimTypeInt(signed=False, label="IKEEXT_AUTHENTICATION_IMPERSONATION_TYPE")}, name="IKEEXT_EM_POLICY2", pack=False, align=None), offset=0)}, name="IPSEC_TRANSPORT_POLICY2", pack=False, align=None), offset=0), "ikeQmTunnelPolicy": SimTypePointer(SimStruct({"flags": SimTypeInt(signed=False, label="UInt32"), "numIpsecProposals": SimTypeInt(signed=False, label="UInt32"), "ipsecProposals": SimTypePointer(SimStruct({"lifetime": SimStruct({"lifetimeSeconds": SimTypeInt(signed=False, label="UInt32"), "lifetimeKilobytes": SimTypeInt(signed=False, label="UInt32"), "lifetimePackets": SimTypeInt(signed=False, label="UInt32")}, name="IPSEC_SA_LIFETIME0", pack=False, align=None), "numSaTransforms": SimTypeInt(signed=False, label="UInt32"), "saTransforms": SimTypePointer(SimStruct({"ipsecTransformType": SimTypeInt(signed=False, label="IPSEC_TRANSFORM_TYPE"), "Anonymous": SimUnion({"ahTransform": SimTypePointer(SimStruct({"authTransformId": SimStruct({"authType": SimTypeInt(signed=False, label="IPSEC_AUTH_TYPE"), "authConfig": SimTypeChar(label="Byte")}, name="IPSEC_AUTH_TRANSFORM_ID0", pack=False, align=None), "cryptoModuleId": SimTypePointer(SimTypeBottom(label="Guid"), offset=0)}, name="IPSEC_AUTH_TRANSFORM0", pack=False, align=None), offset=0), "espAuthTransform": SimTypePointer(SimStruct({"authTransformId": SimStruct({"authType": SimTypeInt(signed=False, label="IPSEC_AUTH_TYPE"), "authConfig": SimTypeChar(label="Byte")}, name="IPSEC_AUTH_TRANSFORM_ID0", pack=False, align=None), "cryptoModuleId": SimTypePointer(SimTypeBottom(label="Guid"), offset=0)}, name="IPSEC_AUTH_TRANSFORM0", pack=False, align=None), offset=0), "espCipherTransform": SimTypePointer(SimStruct({"cipherTransformId": SimStruct({"cipherType": SimTypeInt(signed=False, label="IPSEC_CIPHER_TYPE"), "cipherConfig": SimTypeChar(label="Byte")}, name="IPSEC_CIPHER_TRANSFORM_ID0", pack=False, align=None), "cryptoModuleId": SimTypePointer(SimTypeBottom(label="Guid"), offset=0)}, name="IPSEC_CIPHER_TRANSFORM0", pack=False, align=None), offset=0), "espAuthAndCipherTransform": SimTypePointer(SimStruct({"authTransform": SimStruct({"authTransformId": SimStruct({"authType": SimTypeInt(signed=False, label="IPSEC_AUTH_TYPE"), "authConfig": SimTypeChar(label="Byte")}, name="IPSEC_AUTH_TRANSFORM_ID0", pack=False, align=None), "cryptoModuleId": SimTypePointer(SimTypeBottom(label="Guid"), offset=0)}, name="IPSEC_AUTH_TRANSFORM0", pack=False, align=None), "cipherTransform": SimStruct({"cipherTransformId": SimStruct({"cipherType": SimTypeInt(signed=False, label="IPSEC_CIPHER_TYPE"), "cipherConfig": SimTypeChar(label="Byte")}, name="IPSEC_CIPHER_TRANSFORM_ID0", pack=False, align=None), "cryptoModuleId": SimTypePointer(SimTypeBottom(label="Guid"), offset=0)}, name="IPSEC_CIPHER_TRANSFORM0", pack=False, align=None)}, name="IPSEC_AUTH_AND_CIPHER_TRANSFORM0", pack=False, align=None), offset=0), "espAuthFwTransform": SimTypePointer(SimStruct({"authTransformId": SimStruct({"authType": SimTypeInt(signed=False, label="IPSEC_AUTH_TYPE"), "authConfig": SimTypeChar(label="Byte")}, name="IPSEC_AUTH_TRANSFORM_ID0", pack=False, align=None), "cryptoModuleId": SimTypePointer(SimTypeBottom(label="Guid"), offset=0)}, name="IPSEC_AUTH_TRANSFORM0", pack=False, align=None), offset=0)}, name="<anon>", label="None")}, name="IPSEC_SA_TRANSFORM0", pack=False, align=None), offset=0), "pfsGroup": SimTypeInt(signed=False, label="IPSEC_PFS_GROUP")}, name="IPSEC_PROPOSAL0", pack=False, align=None), offset=0), "tunnelEndpoints": SimStruct({"ipVersion": SimTypeInt(signed=False, label="FWP_IP_VERSION"), "Anonymous1": SimUnion({"localV4Address": SimTypeInt(signed=False, label="UInt32"), "localV6Address": SimTypeFixedSizeArray(SimTypeChar(label="Byte"), 16)}, name="<anon>", label="None"), "Anonymous2": SimUnion({"remoteV4Address": SimTypeInt(signed=False, label="UInt32"), "remoteV6Address": SimTypeFixedSizeArray(SimTypeChar(label="Byte"), 16)}, name="<anon>", label="None"), "localIfLuid": SimTypeLongLong(signed=False, label="UInt64"), "remoteFqdn": SimTypePointer(SimTypeChar(label="Char"), offset=0), "numAddresses": SimTypeInt(signed=False, label="UInt32"), "remoteAddresses": SimTypePointer(SimStruct({"ipVersion": SimTypeInt(signed=False, label="FWP_IP_VERSION"), "Anonymous": SimUnion({"v4Address": SimTypeInt(signed=False, label="UInt32"), "v6Address": SimTypeFixedSizeArray(SimTypeChar(label="Byte"), 16)}, name="<anon>", label="None")}, name="IPSEC_TUNNEL_ENDPOINT0", pack=False, align=None), offset=0)}, name="IPSEC_TUNNEL_ENDPOINTS2", pack=False, align=None), "saIdleTimeout": SimStruct({"idleTimeoutSeconds": SimTypeInt(signed=False, label="UInt32"), "idleTimeoutSecondsFailOver": SimTypeInt(signed=False, label="UInt32")}, name="IPSEC_SA_IDLE_TIMEOUT0", pack=False, align=None), "emPolicy": SimTypePointer(SimStruct({"numAuthenticationMethods": SimTypeInt(signed=False, label="UInt32"), "authenticationMethods": SimTypePointer(SimStruct({"authenticationMethodType": SimTypeInt(signed=False, label="IKEEXT_AUTHENTICATION_METHOD_TYPE"), "Anonymous": SimUnion({"presharedKeyAuthentication": SimStruct({"presharedKey": SimStruct({"size": SimTypeInt(signed=False, label="UInt32"), "data": SimTypePointer(SimTypeChar(label="Byte"), offset=0)}, name="FWP_BYTE_BLOB", pack=False, align=None), "flags": SimTypeInt(signed=False, label="IKEEXT_PRESHARED_KEY_AUTHENTICATION_FLAGS")}, name="IKEEXT_PRESHARED_KEY_AUTHENTICATION1", pack=False, align=None), "certificateAuthentication": SimStruct({"inboundConfigType": SimTypeInt(signed=False, label="IKEEXT_CERT_CONFIG_TYPE"), "Anonymous1": SimUnion({"Anonymous1": SimStruct({"inboundRootArraySize": SimTypeInt(signed=False, label="UInt32"), "inboundRootCriteria": SimTypePointer(SimStruct({"certData": SimStruct({"size": SimTypeInt(signed=False, label="UInt32"), "data": SimTypePointer(SimTypeChar(label="Byte"), offset=0)}, name="FWP_BYTE_BLOB", pack=False, align=None), "certHash": SimStruct({"size": SimTypeInt(signed=False, label="UInt32"), "data": SimTypePointer(SimTypeChar(label="Byte"), offset=0)}, name="FWP_BYTE_BLOB", pack=False, align=None), "eku": SimTypePointer(SimStruct({"numEku": SimTypeInt(signed=False, label="UInt32"), "eku": SimTypePointer(SimTypePointer(SimTypeChar(label="Byte"), offset=0), offset=0)}, name="IKEEXT_CERT_EKUS0", pack=False, align=None), offset=0), "name": SimTypePointer(SimStruct({"nameType": SimTypeInt(signed=False, label="IKEEXT_CERT_CRITERIA_NAME_TYPE"), "certName": SimTypePointer(SimTypeChar(label="Char"), offset=0)}, name="IKEEXT_CERT_NAME0", pack=False, align=None), offset=0), "flags": SimTypeInt(signed=False, label="UInt32")}, name="IKEEXT_CERTIFICATE_CRITERIA0", pack=False, align=None), offset=0)}, name="_Anonymous1_e__Struct", pack=False, align=None), "Anonymous2": SimStruct({"inboundEnterpriseStoreArraySize": SimTypeInt(signed=False, label="UInt32"), "inboundEnterpriseStoreCriteria": SimTypePointer(SimStruct({"certData": SimStruct({"size": SimTypeInt(signed=False, label="UInt32"), "data": SimTypePointer(SimTypeChar(label="Byte"), offset=0)}, name="FWP_BYTE_BLOB", pack=False, align=None), "certHash": SimStruct({"size": SimTypeInt(signed=False, label="UInt32"), "data": SimTypePointer(SimTypeChar(label="Byte"), offset=0)}, name="FWP_BYTE_BLOB", pack=False, align=None), "eku": SimTypePointer(SimStruct({"numEku": SimTypeInt(signed=False, label="UInt32"), "eku": SimTypePointer(SimTypePointer(SimTypeChar(label="Byte"), offset=0), offset=0)}, name="IKEEXT_CERT_EKUS0", pack=False, align=None), offset=0), "name": SimTypePointer(SimStruct({"nameType": SimTypeInt(signed=False, label="IKEEXT_CERT_CRITERIA_NAME_TYPE"), "certName": SimTypePointer(SimTypeChar(label="Char"), offset=0)}, name="IKEEXT_CERT_NAME0", pack=False, align=None), offset=0), "flags": SimTypeInt(signed=False, label="UInt32")}, name="IKEEXT_CERTIFICATE_CRITERIA0", pack=False, align=None), offset=0)}, name="_Anonymous2_e__Struct", pack=False, align=None), "Anonymous3": SimStruct({"inboundRootStoreArraySize": SimTypeInt(signed=False, label="UInt32"), "inboundTrustedRootStoreCriteria": SimTypePointer(SimStruct({"certData": SimStruct({"size": SimTypeInt(signed=False, label="UInt32"), "data": SimTypePointer(SimTypeChar(label="Byte"), offset=0)}, name="FWP_BYTE_BLOB", pack=False, align=None), "certHash": SimStruct({"size": SimTypeInt(signed=False, label="UInt32"), "data": SimTypePointer(SimTypeChar(label="Byte"), offset=0)}, name="FWP_BYTE_BLOB", pack=False, align=None), "eku": SimTypePointer(SimStruct({"numEku": SimTypeInt(signed=False, label="UInt32"), "eku": SimTypePointer(SimTypePointer(SimTypeChar(label="Byte"), offset=0), offset=0)}, name="IKEEXT_CERT_EKUS0", pack=False, align=None), offset=0), "name": SimTypePointer(SimStruct({"nameType": SimTypeInt(signed=False, label="IKEEXT_CERT_CRITERIA_NAME_TYPE"), "certName": SimTypePointer(SimTypeChar(label="Char"), offset=0)}, name="IKEEXT_CERT_NAME0", pack=False, align=None), offset=0), "flags": SimTypeInt(signed=False, label="UInt32")}, name="IKEEXT_CERTIFICATE_CRITERIA0", pack=False, align=None), offset=0)}, name="_Anonymous3_e__Struct", pack=False, align=None)}, name="<anon>", label="None"), "outboundConfigType": SimTypeInt(signed=False, label="IKEEXT_CERT_CONFIG_TYPE"), "Anonymous2": SimUnion({"Anonymous1": SimStruct({"outboundRootArraySize": SimTypeInt(signed=False, label="UInt32"), "outboundRootCriteria": SimTypePointer(SimStruct({"certData": SimStruct({"size": SimTypeInt(signed=False, label="UInt32"), "data": SimTypePointer(SimTypeChar(label="Byte"), offset=0)}, name="FWP_BYTE_BLOB", pack=False, align=None), "certHash": SimStruct({"size": SimTypeInt(signed=False, label="UInt32"), "data": SimTypePointer(SimTypeChar(label="Byte"), offset=0)}, name="FWP_BYTE_BLOB", pack=False, align=None), "eku": SimTypePointer(SimStruct({"numEku": SimTypeInt(signed=False, label="UInt32"), "eku": SimTypePointer(SimTypePointer(SimTypeChar(label="Byte"), offset=0), offset=0)}, name="IKEEXT_CERT_EKUS0", pack=False, align=None), offset=0), "name": SimTypePointer(SimStruct({"nameType": SimTypeInt(signed=False, label="IKEEXT_CERT_CRITERIA_NAME_TYPE"), "certName": SimTypePointer(SimTypeChar(label="Char"), offset=0)}, name="IKEEXT_CERT_NAME0", pack=False, align=None), offset=0), "flags": SimTypeInt(signed=False, label="UInt32")}, name="IKEEXT_CERTIFICATE_CRITERIA0", pack=False, align=None), offset=0)}, name="_Anonymous1_e__Struct", pack=False, align=None), "Anonymous2": SimStruct({"outboundEnterpriseStoreArraySize": SimTypeInt(signed=False, label="UInt32"), "outboundEnterpriseStoreCriteria": SimTypePointer(SimStruct({"certData": SimStruct({"size": SimTypeInt(signed=False, label="UInt32"), "data": SimTypePointer(SimTypeChar(label="Byte"), offset=0)}, name="FWP_BYTE_BLOB", pack=False, align=None), "certHash": SimStruct({"size": SimTypeInt(signed=False, label="UInt32"), "data": SimTypePointer(SimTypeChar(label="Byte"), offset=0)}, name="FWP_BYTE_BLOB", pack=False, align=None), "eku": SimTypePointer(SimStruct({"numEku": SimTypeInt(signed=False, label="UInt32"), "eku": SimTypePointer(SimTypePointer(SimTypeChar(label="Byte"), offset=0), offset=0)}, name="IKEEXT_CERT_EKUS0", pack=False, align=None), offset=0), "name": SimTypePointer(SimStruct({"nameType": SimTypeInt(signed=False, label="IKEEXT_CERT_CRITERIA_NAME_TYPE"), "certName": SimTypePointer(SimTypeChar(label="Char"), offset=0)}, name="IKEEXT_CERT_NAME0", pack=False, align=None), offset=0), "flags": SimTypeInt(signed=False, label="UInt32")}, name="IKEEXT_CERTIFICATE_CRITERIA0", pack=False, align=None), offset=0)}, name="_Anonymous2_e__Struct", pack=False, align=None), "Anonymous3": SimStruct({"outboundRootStoreArraySize": SimTypeInt(signed=False, label="UInt32"), "outboundTrustedRootStoreCriteria": SimTypePointer(SimStruct({"certData": SimStruct({"size": SimTypeInt(signed=False, label="UInt32"), "data": SimTypePointer(SimTypeChar(label="Byte"), offset=0)}, name="FWP_BYTE_BLOB", pack=False, align=None), "certHash": SimStruct({"size": SimTypeInt(signed=False, label="UInt32"), "data": SimTypePointer(SimTypeChar(label="Byte"), offset=0)}, name="FWP_BYTE_BLOB", pack=False, align=None), "eku": SimTypePointer(SimStruct({"numEku": SimTypeInt(signed=False, label="UInt32"), "eku": SimTypePointer(SimTypePointer(SimTypeChar(label="Byte"), offset=0), offset=0)}, name="IKEEXT_CERT_EKUS0", pack=False, align=None), offset=0), "name": SimTypePointer(SimStruct({"nameType": SimTypeInt(signed=False, label="IKEEXT_CERT_CRITERIA_NAME_TYPE"), "certName": SimTypePointer(SimTypeChar(label="Char"), offset=0)}, name="IKEEXT_CERT_NAME0", pack=False, align=None), offset=0), "flags": SimTypeInt(signed=False, label="UInt32")}, name="IKEEXT_CERTIFICATE_CRITERIA0", pack=False, align=None), offset=0)}, name="_Anonymous3_e__Struct", pack=False, align=None)}, name="<anon>", label="None"), "flags": SimTypeInt(signed=False, label="IKEEXT_CERT_AUTH"), "localCertLocationUrl": SimStruct({"size": SimTypeInt(signed=False, label="UInt32"), "data": SimTypePointer(SimTypeChar(label="Byte"), offset=0)}, name="FWP_BYTE_BLOB", pack=False, align=None)}, name="IKEEXT_CERTIFICATE_AUTHENTICATION2", pack=False, align=None), "kerberosAuthentication": SimStruct({"flags": SimTypeInt(signed=False, label="IKEEXT_KERBEROS_AUTHENTICATION_FLAGS"), "proxyServer": SimTypePointer(SimTypeChar(label="Char"), offset=0)}, name="IKEEXT_KERBEROS_AUTHENTICATION1", pack=False, align=None), "reservedAuthentication": SimStruct({"flags": SimTypeInt(signed=False, label="IKEEXT_RESERVED_AUTHENTICATION_FLAGS")}, name="IKEEXT_RESERVED_AUTHENTICATION0", pack=False, align=None), "ntlmV2Authentication": SimStruct({"flags": SimTypeInt(signed=False, label="UInt32")}, name="IKEEXT_NTLM_V2_AUTHENTICATION0", pack=False, align=None), "sslAuthentication": SimStruct({"inboundConfigType": SimTypeInt(signed=False, label="IKEEXT_CERT_CONFIG_TYPE"), "Anonymous1": SimUnion({"Anonymous1": SimStruct({"inboundRootArraySize": SimTypeInt(signed=False, label="UInt32"), "inboundRootCriteria": SimTypePointer(SimStruct({"certData": SimStruct({"size": SimTypeInt(signed=False, label="UInt32"), "data": SimTypePointer(SimTypeChar(label="Byte"), offset=0)}, name="FWP_BYTE_BLOB", pack=False, align=None), "certHash": SimStruct({"size": SimTypeInt(signed=False, label="UInt32"), "data": SimTypePointer(SimTypeChar(label="Byte"), offset=0)}, name="FWP_BYTE_BLOB", pack=False, align=None), "eku": SimTypePointer(SimStruct({"numEku": SimTypeInt(signed=False, label="UInt32"), "eku": SimTypePointer(SimTypePointer(SimTypeChar(label="Byte"), offset=0), offset=0)}, name="IKEEXT_CERT_EKUS0", pack=False, align=None), offset=0), "name": SimTypePointer(SimStruct({"nameType": SimTypeInt(signed=False, label="IKEEXT_CERT_CRITERIA_NAME_TYPE"), "certName": SimTypePointer(SimTypeChar(label="Char"), offset=0)}, name="IKEEXT_CERT_NAME0", pack=False, align=None), offset=0), "flags": SimTypeInt(signed=False, label="UInt32")}, name="IKEEXT_CERTIFICATE_CRITERIA0", pack=False, align=None), offset=0)}, name="_Anonymous1_e__Struct", pack=False, align=None), "Anonymous2": SimStruct({"inboundEnterpriseStoreArraySize": SimTypeInt(signed=False, label="UInt32"), "inboundEnterpriseStoreCriteria": SimTypePointer(SimStruct({"certData": SimStruct({"size": SimTypeInt(signed=False, label="UInt32"), "data": SimTypePointer(SimTypeChar(label="Byte"), offset=0)}, name="FWP_BYTE_BLOB", pack=False, align=None), "certHash": SimStruct({"size": SimTypeInt(signed=False, label="UInt32"), "data": SimTypePointer(SimTypeChar(label="Byte"), offset=0)}, name="FWP_BYTE_BLOB", pack=False, align=None), "eku": SimTypePointer(SimStruct({"numEku": SimTypeInt(signed=False, label="UInt32"), "eku": SimTypePointer(SimTypePointer(SimTypeChar(label="Byte"), offset=0), offset=0)}, name="IKEEXT_CERT_EKUS0", pack=False, align=None), offset=0), "name": SimTypePointer(SimStruct({"nameType": SimTypeInt(signed=False, label="IKEEXT_CERT_CRITERIA_NAME_TYPE"), "certName": SimTypePointer(SimTypeChar(label="Char"), offset=0)}, name="IKEEXT_CERT_NAME0", pack=False, align=None), offset=0), "flags": SimTypeInt(signed=False, label="UInt32")}, name="IKEEXT_CERTIFICATE_CRITERIA0", pack=False, align=None), offset=0)}, name="_Anonymous2_e__Struct", pack=False, align=None), "Anonymous3": SimStruct({"inboundRootStoreArraySize": SimTypeInt(signed=False, label="UInt32"), "inboundTrustedRootStoreCriteria": SimTypePointer(SimStruct({"certData": SimStruct({"size": SimTypeInt(signed=False, label="UInt32"), "data": SimTypePointer(SimTypeChar(label="Byte"), offset=0)}, name="FWP_BYTE_BLOB", pack=False, align=None), "certHash": SimStruct({"size": SimTypeInt(signed=False, label="UInt32"), "data": SimTypePointer(SimTypeChar(label="Byte"), offset=0)}, name="FWP_BYTE_BLOB", pack=False, align=None), "eku": SimTypePointer(SimStruct({"numEku": SimTypeInt(signed=False, label="UInt32"), "eku": SimTypePointer(SimTypePointer(SimTypeChar(label="Byte"), offset=0), offset=0)}, name="IKEEXT_CERT_EKUS0", pack=False, align=None), offset=0), "name": SimTypePointer(SimStruct({"nameType": SimTypeInt(signed=False, label="IKEEXT_CERT_CRITERIA_NAME_TYPE"), "certName": SimTypePointer(SimTypeChar(label="Char"), offset=0)}, name="IKEEXT_CERT_NAME0", pack=False, align=None), offset=0), "flags": SimTypeInt(signed=False, label="UInt32")}, name="IKEEXT_CERTIFICATE_CRITERIA0", pack=False, align=None), offset=0)}, name="_Anonymous3_e__Struct", pack=False, align=None)}, name="<anon>", label="None"), "outboundConfigType": SimTypeInt(signed=False, label="IKEEXT_CERT_CONFIG_TYPE"), "Anonymous2": SimUnion({"Anonymous1": SimStruct({"outboundRootArraySize": SimTypeInt(signed=False, label="UInt32"), "outboundRootCriteria": SimTypePointer(SimStruct({"certData": SimStruct({"size": SimTypeInt(signed=False, label="UInt32"), "data": SimTypePointer(SimTypeChar(label="Byte"), offset=0)}, name="FWP_BYTE_BLOB", pack=False, align=None), "certHash": SimStruct({"size": SimTypeInt(signed=False, label="UInt32"), "data": SimTypePointer(SimTypeChar(label="Byte"), offset=0)}, name="FWP_BYTE_BLOB", pack=False, align=None), "eku": SimTypePointer(SimStruct({"numEku": SimTypeInt(signed=False, label="UInt32"), "eku": SimTypePointer(SimTypePointer(SimTypeChar(label="Byte"), offset=0), offset=0)}, name="IKEEXT_CERT_EKUS0", pack=False, align=None), offset=0), "name": SimTypePointer(SimStruct({"nameType": SimTypeInt(signed=False, label="IKEEXT_CERT_CRITERIA_NAME_TYPE"), "certName": SimTypePointer(SimTypeChar(label="Char"), offset=0)}, name="IKEEXT_CERT_NAME0", pack=False, align=None), offset=0), "flags": SimTypeInt(signed=False, label="UInt32")}, name="IKEEXT_CERTIFICATE_CRITERIA0", pack=False, align=None), offset=0)}, name="_Anonymous1_e__Struct", pack=False, align=None), "Anonymous2": SimStruct({"outboundEnterpriseStoreArraySize": SimTypeInt(signed=False, label="UInt32"), "outboundEnterpriseStoreCriteria": SimTypePointer(SimStruct({"certData": SimStruct({"size": SimTypeInt(signed=False, label="UInt32"), "data": SimTypePointer(SimTypeChar(label="Byte"), offset=0)}, name="FWP_BYTE_BLOB", pack=False, align=None), "certHash": SimStruct({"size": SimTypeInt(signed=False, label="UInt32"), "data": SimTypePointer(SimTypeChar(label="Byte"), offset=0)}, name="FWP_BYTE_BLOB", pack=False, align=None), "eku": SimTypePointer(SimStruct({"numEku": SimTypeInt(signed=False, label="UInt32"), "eku": SimTypePointer(SimTypePointer(SimTypeChar(label="Byte"), offset=0), offset=0)}, name="IKEEXT_CERT_EKUS0", pack=False, align=None), offset=0), "name": SimTypePointer(SimStruct({"nameType": SimTypeInt(signed=False, label="IKEEXT_CERT_CRITERIA_NAME_TYPE"), "certName": SimTypePointer(SimTypeChar(label="Char"), offset=0)}, name="IKEEXT_CERT_NAME0", pack=False, align=None), offset=0), "flags": SimTypeInt(signed=False, label="UInt32")}, name="IKEEXT_CERTIFICATE_CRITERIA0", pack=False, align=None), offset=0)}, name="_Anonymous2_e__Struct", pack=False, align=None), "Anonymous3": SimStruct({"outboundRootStoreArraySize": SimTypeInt(signed=False, label="UInt32"), "outboundTrustedRootStoreCriteria": SimTypePointer(SimStruct({"certData": SimStruct({"size": SimTypeInt(signed=False, label="UInt32"), "data": SimTypePointer(SimTypeChar(label="Byte"), offset=0)}, name="FWP_BYTE_BLOB", pack=False, align=None), "certHash": SimStruct({"size": SimTypeInt(signed=False, label="UInt32"), "data": SimTypePointer(SimTypeChar(label="Byte"), offset=0)}, name="FWP_BYTE_BLOB", pack=False, align=None), "eku": SimTypePointer(SimStruct({"numEku": SimTypeInt(signed=False, label="UInt32"), "eku": SimTypePointer(SimTypePointer(SimTypeChar(label="Byte"), offset=0), offset=0)}, name="IKEEXT_CERT_EKUS0", pack=False, align=None), offset=0), "name": SimTypePointer(SimStruct({"nameType": SimTypeInt(signed=False, label="IKEEXT_CERT_CRITERIA_NAME_TYPE"), "certName": SimTypePointer(SimTypeChar(label="Char"), offset=0)}, name="IKEEXT_CERT_NAME0", pack=False, align=None), offset=0), "flags": SimTypeInt(signed=False, label="UInt32")}, name="IKEEXT_CERTIFICATE_CRITERIA0", pack=False, align=None), offset=0)}, name="_Anonymous3_e__Struct", pack=False, align=None)}, name="<anon>", label="None"), "flags": SimTypeInt(signed=False, label="IKEEXT_CERT_AUTH"), "localCertLocationUrl": SimStruct({"size": SimTypeInt(signed=False, label="UInt32"), "data": SimTypePointer(SimTypeChar(label="Byte"), offset=0)}, name="FWP_BYTE_BLOB", pack=False, align=None)}, name="IKEEXT_CERTIFICATE_AUTHENTICATION2", pack=False, align=None), "cgaAuthentication": SimStruct({"keyContainerName": SimTypePointer(SimTypeChar(label="Char"), offset=0), "cspName": SimTypePointer(SimTypeChar(label="Char"), offset=0), "cspType": SimTypeInt(signed=False, label="UInt32"), "cgaModifier": SimStruct({"byteArray16": SimTypeFixedSizeArray(SimTypeChar(label="Byte"), 16)}, name="FWP_BYTE_ARRAY16", pack=False, align=None), "cgaCollisionCount": SimTypeChar(label="Byte")}, name="IKEEXT_IPV6_CGA_AUTHENTICATION0", pack=False, align=None), "eapAuthentication": SimStruct({"flags": SimTypeInt(signed=False, label="IKEEXT_EAP_AUTHENTICATION_FLAGS")}, name="IKEEXT_EAP_AUTHENTICATION0", pack=False, align=None)}, name="<anon>", label="None")}, name="IKEEXT_AUTHENTICATION_METHOD2", pack=False, align=None), offset=0), "initiatorImpersonationType": SimTypeInt(signed=False, label="IKEEXT_AUTHENTICATION_IMPERSONATION_TYPE")}, name="IKEEXT_EM_POLICY2", pack=False, align=None), offset=0), "fwdPathSaLifetime": SimTypeInt(signed=False, label="UInt32"), "compartmentId": SimTypeInt(signed=False, label="UInt32"), "numTrafficSelectorPolicy": SimTypeInt(signed=False, label="UInt32"), "trafficSelectorPolicies": SimTypePointer(SimStruct({"flags": SimTypeInt(signed=False, label="UInt32"), "numLocalTrafficSelectors": SimTypeInt(signed=False, label="UInt32"), "localTrafficSelectors": SimTypePointer(SimStruct({"protocolId": SimTypeChar(label="Byte"), "portStart": SimTypeShort(signed=False, label="UInt16"), "portEnd": SimTypeShort(signed=False, label="UInt16"), "ipVersion": SimTypeInt(signed=False, label="FWP_IP_VERSION"), "Anonymous1": SimUnion({"startV4Address": SimTypeInt(signed=False, label="UInt32"), "startV6Address": SimTypeFixedSizeArray(SimTypeChar(label="Byte"), 16)}, name="<anon>", label="None"), "Anonymous2": SimUnion({"endV4Address": SimTypeInt(signed=False, label="UInt32"), "endV6Address": SimTypeFixedSizeArray(SimTypeChar(label="Byte"), 16)}, name="<anon>", label="None")}, name="IPSEC_TRAFFIC_SELECTOR0_", pack=False, align=None), offset=0), "numRemoteTrafficSelectors": SimTypeInt(signed=False, label="UInt32"), "remoteTrafficSelectors": SimTypePointer(SimStruct({"protocolId": SimTypeChar(label="Byte"), "portStart": SimTypeShort(signed=False, label="UInt16"), "portEnd": SimTypeShort(signed=False, label="UInt16"), "ipVersion": SimTypeInt(signed=False, label="FWP_IP_VERSION"), "Anonymous1": SimUnion({"startV4Address": SimTypeInt(signed=False, label="UInt32"), "startV6Address": SimTypeFixedSizeArray(SimTypeChar(label="Byte"), 16)}, name="<anon>", label="None"), "Anonymous2": SimUnion({"endV4Address": SimTypeInt(signed=False, label="UInt32"), "endV6Address": SimTypeFixedSizeArray(SimTypeChar(label="Byte"), 16)}, name="<anon>", label="None")}, name="IPSEC_TRAFFIC_SELECTOR0_", pack=False, align=None), offset=0)}, name="IPSEC_TRAFFIC_SELECTOR_POLICY0_", pack=False, align=None), offset=0)}, name="IPSEC_TUNNEL_POLICY3_", pack=False, align=None), offset=0), "authipQmTransportPolicy": SimTypePointer(SimStruct({"numIpsecProposals": SimTypeInt(signed=False, label="UInt32"), "ipsecProposals": SimTypePointer(SimStruct({"lifetime": SimStruct({"lifetimeSeconds": SimTypeInt(signed=False, label="UInt32"), "lifetimeKilobytes": SimTypeInt(signed=False, label="UInt32"), "lifetimePackets": SimTypeInt(signed=False, label="UInt32")}, name="IPSEC_SA_LIFETIME0", pack=False, align=None), "numSaTransforms": SimTypeInt(signed=False, label="UInt32"), "saTransforms": SimTypePointer(SimStruct({"ipsecTransformType": SimTypeInt(signed=False, label="IPSEC_TRANSFORM_TYPE"), "Anonymous": SimUnion({"ahTransform": SimTypePointer(SimStruct({"authTransformId": SimStruct({"authType": SimTypeInt(signed=False, label="IPSEC_AUTH_TYPE"), "authConfig": SimTypeChar(label="Byte")}, name="IPSEC_AUTH_TRANSFORM_ID0", pack=False, align=None), "cryptoModuleId": SimTypePointer(SimTypeBottom(label="Guid"), offset=0)}, name="IPSEC_AUTH_TRANSFORM0", pack=False, align=None), offset=0), "espAuthTransform": SimTypePointer(SimStruct({"authTransformId": SimStruct({"authType": SimTypeInt(signed=False, label="IPSEC_AUTH_TYPE"), "authConfig": SimTypeChar(label="Byte")}, name="IPSEC_AUTH_TRANSFORM_ID0", pack=False, align=None), "cryptoModuleId": SimTypePointer(SimTypeBottom(label="Guid"), offset=0)}, name="IPSEC_AUTH_TRANSFORM0", pack=False, align=None), offset=0), "espCipherTransform": SimTypePointer(SimStruct({"cipherTransformId": SimStruct({"cipherType": SimTypeInt(signed=False, label="IPSEC_CIPHER_TYPE"), "cipherConfig": SimTypeChar(label="Byte")}, name="IPSEC_CIPHER_TRANSFORM_ID0", pack=False, align=None), "cryptoModuleId": SimTypePointer(SimTypeBottom(label="Guid"), offset=0)}, name="IPSEC_CIPHER_TRANSFORM0", pack=False, align=None), offset=0), "espAuthAndCipherTransform": SimTypePointer(SimStruct({"authTransform": SimStruct({"authTransformId": SimStruct({"authType": SimTypeInt(signed=False, label="IPSEC_AUTH_TYPE"), "authConfig": SimTypeChar(label="Byte")}, name="IPSEC_AUTH_TRANSFORM_ID0", pack=False, align=None), "cryptoModuleId": SimTypePointer(SimTypeBottom(label="Guid"), offset=0)}, name="IPSEC_AUTH_TRANSFORM0", pack=False, align=None), "cipherTransform": SimStruct({"cipherTransformId": SimStruct({"cipherType": SimTypeInt(signed=False, label="IPSEC_CIPHER_TYPE"), "cipherConfig": SimTypeChar(label="Byte")}, name="IPSEC_CIPHER_TRANSFORM_ID0", pack=False, align=None), "cryptoModuleId": SimTypePointer(SimTypeBottom(label="Guid"), offset=0)}, name="IPSEC_CIPHER_TRANSFORM0", pack=False, align=None)}, name="IPSEC_AUTH_AND_CIPHER_TRANSFORM0", pack=False, align=None), offset=0), "espAuthFwTransform": SimTypePointer(SimStruct({"authTransformId": SimStruct({"authType": SimTypeInt(signed=False, label="IPSEC_AUTH_TYPE"), "authConfig": SimTypeChar(label="Byte")}, name="IPSEC_AUTH_TRANSFORM_ID0", pack=False, align=None), "cryptoModuleId": SimTypePointer(SimTypeBottom(label="Guid"), offset=0)}, name="IPSEC_AUTH_TRANSFORM0", pack=False, align=None), offset=0)}, name="<anon>", label="None")}, name="IPSEC_SA_TRANSFORM0", pack=False, align=None), offset=0), "pfsGroup": SimTypeInt(signed=False, label="IPSEC_PFS_GROUP")}, name="IPSEC_PROPOSAL0", pack=False, align=None), offset=0), "flags": SimTypeInt(signed=False, label="IPSEC_POLICY_FLAG"), "ndAllowClearTimeoutSeconds": SimTypeInt(signed=False, label="UInt32"), "saIdleTimeout": SimStruct({"idleTimeoutSeconds": SimTypeInt(signed=False, label="UInt32"), "idleTimeoutSecondsFailOver": SimTypeInt(signed=False, label="UInt32")}, name="IPSEC_SA_IDLE_TIMEOUT0", pack=False, align=None), "emPolicy": SimTypePointer(SimStruct({"numAuthenticationMethods": SimTypeInt(signed=False, label="UInt32"), "authenticationMethods": SimTypePointer(SimStruct({"authenticationMethodType": SimTypeInt(signed=False, label="IKEEXT_AUTHENTICATION_METHOD_TYPE"), "Anonymous": SimUnion({"presharedKeyAuthentication": SimStruct({"presharedKey": SimStruct({"size": SimTypeInt(signed=False, label="UInt32"), "data": SimTypePointer(SimTypeChar(label="Byte"), offset=0)}, name="FWP_BYTE_BLOB", pack=False, align=None), "flags": SimTypeInt(signed=False, label="IKEEXT_PRESHARED_KEY_AUTHENTICATION_FLAGS")}, name="IKEEXT_PRESHARED_KEY_AUTHENTICATION1", pack=False, align=None), "certificateAuthentication": SimStruct({"inboundConfigType": SimTypeInt(signed=False, label="IKEEXT_CERT_CONFIG_TYPE"), "Anonymous1": SimUnion({"Anonymous1": SimStruct({"inboundRootArraySize": SimTypeInt(signed=False, label="UInt32"), "inboundRootCriteria": SimTypePointer(SimStruct({"certData": SimStruct({"size": SimTypeInt(signed=False, label="UInt32"), "data": SimTypePointer(SimTypeChar(label="Byte"), offset=0)}, name="FWP_BYTE_BLOB", pack=False, align=None), "certHash": SimStruct({"size": SimTypeInt(signed=False, label="UInt32"), "data": SimTypePointer(SimTypeChar(label="Byte"), offset=0)}, name="FWP_BYTE_BLOB", pack=False, align=None), "eku": SimTypePointer(SimStruct({"numEku": SimTypeInt(signed=False, label="UInt32"), "eku": SimTypePointer(SimTypePointer(SimTypeChar(label="Byte"), offset=0), offset=0)}, name="IKEEXT_CERT_EKUS0", pack=False, align=None), offset=0), "name": SimTypePointer(SimStruct({"nameType": SimTypeInt(signed=False, label="IKEEXT_CERT_CRITERIA_NAME_TYPE"), "certName": SimTypePointer(SimTypeChar(label="Char"), offset=0)}, name="IKEEXT_CERT_NAME0", pack=False, align=None), offset=0), "flags": SimTypeInt(signed=False, label="UInt32")}, name="IKEEXT_CERTIFICATE_CRITERIA0", pack=False, align=None), offset=0)}, name="_Anonymous1_e__Struct", pack=False, align=None), "Anonymous2": SimStruct({"inboundEnterpriseStoreArraySize": SimTypeInt(signed=False, label="UInt32"), "inboundEnterpriseStoreCriteria": SimTypePointer(SimStruct({"certData": SimStruct({"size": SimTypeInt(signed=False, label="UInt32"), "data": SimTypePointer(SimTypeChar(label="Byte"), offset=0)}, name="FWP_BYTE_BLOB", pack=False, align=None), "certHash": SimStruct({"size": SimTypeInt(signed=False, label="UInt32"), "data": SimTypePointer(SimTypeChar(label="Byte"), offset=0)}, name="FWP_BYTE_BLOB", pack=False, align=None), "eku": SimTypePointer(SimStruct({"numEku": SimTypeInt(signed=False, label="UInt32"), "eku": SimTypePointer(SimTypePointer(SimTypeChar(label="Byte"), offset=0), offset=0)}, name="IKEEXT_CERT_EKUS0", pack=False, align=None), offset=0), "name": SimTypePointer(SimStruct({"nameType": SimTypeInt(signed=False, label="IKEEXT_CERT_CRITERIA_NAME_TYPE"), "certName": SimTypePointer(SimTypeChar(label="Char"), offset=0)}, name="IKEEXT_CERT_NAME0", pack=False, align=None), offset=0), "flags": SimTypeInt(signed=False, label="UInt32")}, name="IKEEXT_CERTIFICATE_CRITERIA0", pack=False, align=None), offset=0)}, name="_Anonymous2_e__Struct", pack=False, align=None), "Anonymous3": SimStruct({"inboundRootStoreArraySize": SimTypeInt(signed=False, label="UInt32"), "inboundTrustedRootStoreCriteria": SimTypePointer(SimStruct({"certData": SimStruct({"size": SimTypeInt(signed=False, label="UInt32"), "data": SimTypePointer(SimTypeChar(label="Byte"), offset=0)}, name="FWP_BYTE_BLOB", pack=False, align=None), "certHash": SimStruct({"size": SimTypeInt(signed=False, label="UInt32"), "data": SimTypePointer(SimTypeChar(label="Byte"), offset=0)}, name="FWP_BYTE_BLOB", pack=False, align=None), "eku": SimTypePointer(SimStruct({"numEku": SimTypeInt(signed=False, label="UInt32"), "eku": SimTypePointer(SimTypePointer(SimTypeChar(label="Byte"), offset=0), offset=0)}, name="IKEEXT_CERT_EKUS0", pack=False, align=None), offset=0), "name": SimTypePointer(SimStruct({"nameType": SimTypeInt(signed=False, label="IKEEXT_CERT_CRITERIA_NAME_TYPE"), "certName": SimTypePointer(SimTypeChar(label="Char"), offset=0)}, name="IKEEXT_CERT_NAME0", pack=False, align=None), offset=0), "flags": SimTypeInt(signed=False, label="UInt32")}, name="IKEEXT_CERTIFICATE_CRITERIA0", pack=False, align=None), offset=0)}, name="_Anonymous3_e__Struct", pack=False, align=None)}, name="<anon>", label="None"), "outboundConfigType": SimTypeInt(signed=False, label="IKEEXT_CERT_CONFIG_TYPE"), "Anonymous2": SimUnion({"Anonymous1": SimStruct({"outboundRootArraySize": SimTypeInt(signed=False, label="UInt32"), "outboundRootCriteria": SimTypePointer(SimStruct({"certData": SimStruct({"size": SimTypeInt(signed=False, label="UInt32"), "data": SimTypePointer(SimTypeChar(label="Byte"), offset=0)}, name="FWP_BYTE_BLOB", pack=False, align=None), "certHash": SimStruct({"size": SimTypeInt(signed=False, label="UInt32"), "data": SimTypePointer(SimTypeChar(label="Byte"), offset=0)}, name="FWP_BYTE_BLOB", pack=False, align=None), "eku": SimTypePointer(SimStruct({"numEku": SimTypeInt(signed=False, label="UInt32"), "eku": SimTypePointer(SimTypePointer(SimTypeChar(label="Byte"), offset=0), offset=0)}, name="IKEEXT_CERT_EKUS0", pack=False, align=None), offset=0), "name": SimTypePointer(SimStruct({"nameType": SimTypeInt(signed=False, label="IKEEXT_CERT_CRITERIA_NAME_TYPE"), "certName": SimTypePointer(SimTypeChar(label="Char"), offset=0)}, name="IKEEXT_CERT_NAME0", pack=False, align=None), offset=0), "flags": SimTypeInt(signed=False, label="UInt32")}, name="IKEEXT_CERTIFICATE_CRITERIA0", pack=False, align=None), offset=0)}, name="_Anonymous1_e__Struct", pack=False, align=None), "Anonymous2": SimStruct({"outboundEnterpriseStoreArraySize": SimTypeInt(signed=False, label="UInt32"), "outboundEnterpriseStoreCriteria": SimTypePointer(SimStruct({"certData": SimStruct({"size": SimTypeInt(signed=False, label="UInt32"), "data": SimTypePointer(SimTypeChar(label="Byte"), offset=0)}, name="FWP_BYTE_BLOB", pack=False, align=None), "certHash": SimStruct({"size": SimTypeInt(signed=False, label="UInt32"), "data": SimTypePointer(SimTypeChar(label="Byte"), offset=0)}, name="FWP_BYTE_BLOB", pack=False, align=None), "eku": SimTypePointer(SimStruct({"numEku": SimTypeInt(signed=False, label="UInt32"), "eku": SimTypePointer(SimTypePointer(SimTypeChar(label="Byte"), offset=0), offset=0)}, name="IKEEXT_CERT_EKUS0", pack=False, align=None), offset=0), "name": SimTypePointer(SimStruct({"nameType": SimTypeInt(signed=False, label="IKEEXT_CERT_CRITERIA_NAME_TYPE"), "certName": SimTypePointer(SimTypeChar(label="Char"), offset=0)}, name="IKEEXT_CERT_NAME0", pack=False, align=None), offset=0), "flags": SimTypeInt(signed=False, label="UInt32")}, name="IKEEXT_CERTIFICATE_CRITERIA0", pack=False, align=None), offset=0)}, name="_Anonymous2_e__Struct", pack=False, align=None), "Anonymous3": SimStruct({"outboundRootStoreArraySize": SimTypeInt(signed=False, label="UInt32"), "outboundTrustedRootStoreCriteria": SimTypePointer(SimStruct({"certData": SimStruct({"size": SimTypeInt(signed=False, label="UInt32"), "data": SimTypePointer(SimTypeChar(label="Byte"), offset=0)}, name="FWP_BYTE_BLOB", pack=False, align=None), "certHash": SimStruct({"size": SimTypeInt(signed=False, label="UInt32"), "data": SimTypePointer(SimTypeChar(label="Byte"), offset=0)}, name="FWP_BYTE_BLOB", pack=False, align=None), "eku": SimTypePointer(SimStruct({"numEku": SimTypeInt(signed=False, label="UInt32"), "eku": SimTypePointer(SimTypePointer(SimTypeChar(label="Byte"), offset=0), offset=0)}, name="IKEEXT_CERT_EKUS0", pack=False, align=None), offset=0), "name": SimTypePointer(SimStruct({"nameType": SimTypeInt(signed=False, label="IKEEXT_CERT_CRITERIA_NAME_TYPE"), "certName": SimTypePointer(SimTypeChar(label="Char"), offset=0)}, name="IKEEXT_CERT_NAME0", pack=False, align=None), offset=0), "flags": SimTypeInt(signed=False, label="UInt32")}, name="IKEEXT_CERTIFICATE_CRITERIA0", pack=False, align=None), offset=0)}, name="_Anonymous3_e__Struct", pack=False, align=None)}, name="<anon>", label="None"), "flags": SimTypeInt(signed=False, label="IKEEXT_CERT_AUTH"), "localCertLocationUrl": SimStruct({"size": SimTypeInt(signed=False, label="UInt32"), "data": SimTypePointer(SimTypeChar(label="Byte"), offset=0)}, name="FWP_BYTE_BLOB", pack=False, align=None)}, name="IKEEXT_CERTIFICATE_AUTHENTICATION2", pack=False, align=None), "kerberosAuthentication": SimStruct({"flags": SimTypeInt(signed=False, label="IKEEXT_KERBEROS_AUTHENTICATION_FLAGS"), "proxyServer": SimTypePointer(SimTypeChar(label="Char"), offset=0)}, name="IKEEXT_KERBEROS_AUTHENTICATION1", pack=False, align=None), "reservedAuthentication": SimStruct({"flags": SimTypeInt(signed=False, label="IKEEXT_RESERVED_AUTHENTICATION_FLAGS")}, name="IKEEXT_RESERVED_AUTHENTICATION0", pack=False, align=None), "ntlmV2Authentication": SimStruct({"flags": SimTypeInt(signed=False, label="UInt32")}, name="IKEEXT_NTLM_V2_AUTHENTICATION0", pack=False, align=None), "sslAuthentication": SimStruct({"inboundConfigType": SimTypeInt(signed=False, label="IKEEXT_CERT_CONFIG_TYPE"), "Anonymous1": SimUnion({"Anonymous1": SimStruct({"inboundRootArraySize": SimTypeInt(signed=False, label="UInt32"), "inboundRootCriteria": SimTypePointer(SimStruct({"certData": SimStruct({"size": SimTypeInt(signed=False, label="UInt32"), "data": SimTypePointer(SimTypeChar(label="Byte"), offset=0)}, name="FWP_BYTE_BLOB", pack=False, align=None), "certHash": SimStruct({"size": SimTypeInt(signed=False, label="UInt32"), "data": SimTypePointer(SimTypeChar(label="Byte"), offset=0)}, name="FWP_BYTE_BLOB", pack=False, align=None), "eku": SimTypePointer(SimStruct({"numEku": SimTypeInt(signed=False, label="UInt32"), "eku": SimTypePointer(SimTypePointer(SimTypeChar(label="Byte"), offset=0), offset=0)}, name="IKEEXT_CERT_EKUS0", pack=False, align=None), offset=0), "name": SimTypePointer(SimStruct({"nameType": SimTypeInt(signed=False, label="IKEEXT_CERT_CRITERIA_NAME_TYPE"), "certName": SimTypePointer(SimTypeChar(label="Char"), offset=0)}, name="IKEEXT_CERT_NAME0", pack=False, align=None), offset=0), "flags": SimTypeInt(signed=False, label="UInt32")}, name="IKEEXT_CERTIFICATE_CRITERIA0", pack=False, align=None), offset=0)}, name="_Anonymous1_e__Struct", pack=False, align=None), "Anonymous2": SimStruct({"inboundEnterpriseStoreArraySize": SimTypeInt(signed=False, label="UInt32"), "inboundEnterpriseStoreCriteria": SimTypePointer(SimStruct({"certData": SimStruct({"size": SimTypeInt(signed=False, label="UInt32"), "data": SimTypePointer(SimTypeChar(label="Byte"), offset=0)}, name="FWP_BYTE_BLOB", pack=False, align=None), "certHash": SimStruct({"size": SimTypeInt(signed=False, label="UInt32"), "data": SimTypePointer(SimTypeChar(label="Byte"), offset=0)}, name="FWP_BYTE_BLOB", pack=False, align=None), "eku": SimTypePointer(SimStruct({"numEku": SimTypeInt(signed=False, label="UInt32"), "eku": SimTypePointer(SimTypePointer(SimTypeChar(label="Byte"), offset=0), offset=0)}, name="IKEEXT_CERT_EKUS0", pack=False, align=None), offset=0), "name": SimTypePointer(SimStruct({"nameType": SimTypeInt(signed=False, label="IKEEXT_CERT_CRITERIA_NAME_TYPE"), "certName": SimTypePointer(SimTypeChar(label="Char"), offset=0)}, name="IKEEXT_CERT_NAME0", pack=False, align=None), offset=0), "flags": SimTypeInt(signed=False, label="UInt32")}, name="IKEEXT_CERTIFICATE_CRITERIA0", pack=False, align=None), offset=0)}, name="_Anonymous2_e__Struct", pack=False, align=None), "Anonymous3": SimStruct({"inboundRootStoreArraySize": SimTypeInt(signed=False, label="UInt32"), "inboundTrustedRootStoreCriteria": SimTypePointer(SimStruct({"certData": SimStruct({"size": SimTypeInt(signed=False, label="UInt32"), "data": SimTypePointer(SimTypeChar(label="Byte"), offset=0)}, name="FWP_BYTE_BLOB", pack=False, align=None), "certHash": SimStruct({"size": SimTypeInt(signed=False, label="UInt32"), "data": SimTypePointer(SimTypeChar(label="Byte"), offset=0)}, name="FWP_BYTE_BLOB", pack=False, align=None), "eku": SimTypePointer(SimStruct({"numEku": SimTypeInt(signed=False, label="UInt32"), "eku": SimTypePointer(SimTypePointer(SimTypeChar(label="Byte"), offset=0), offset=0)}, name="IKEEXT_CERT_EKUS0", pack=False, align=None), offset=0), "name": SimTypePointer(SimStruct({"nameType": SimTypeInt(signed=False, label="IKEEXT_CERT_CRITERIA_NAME_TYPE"), "certName": SimTypePointer(SimTypeChar(label="Char"), offset=0)}, name="IKEEXT_CERT_NAME0", pack=False, align=None), offset=0), "flags": SimTypeInt(signed=False, label="UInt32")}, name="IKEEXT_CERTIFICATE_CRITERIA0", pack=False, align=None), offset=0)}, name="_Anonymous3_e__Struct", pack=False, align=None)}, name="<anon>", label="None"), "outboundConfigType": SimTypeInt(signed=False, label="IKEEXT_CERT_CONFIG_TYPE"), "Anonymous2": SimUnion({"Anonymous1": SimStruct({"outboundRootArraySize": SimTypeInt(signed=False, label="UInt32"), "outboundRootCriteria": SimTypePointer(SimStruct({"certData": SimStruct({"size": SimTypeInt(signed=False, label="UInt32"), "data": SimTypePointer(SimTypeChar(label="Byte"), offset=0)}, name="FWP_BYTE_BLOB", pack=False, align=None), "certHash": SimStruct({"size": SimTypeInt(signed=False, label="UInt32"), "data": SimTypePointer(SimTypeChar(label="Byte"), offset=0)}, name="FWP_BYTE_BLOB", pack=False, align=None), "eku": SimTypePointer(SimStruct({"numEku": SimTypeInt(signed=False, label="UInt32"), "eku": SimTypePointer(SimTypePointer(SimTypeChar(label="Byte"), offset=0), offset=0)}, name="IKEEXT_CERT_EKUS0", pack=False, align=None), offset=0), "name": SimTypePointer(SimStruct({"nameType": SimTypeInt(signed=False, label="IKEEXT_CERT_CRITERIA_NAME_TYPE"), "certName": SimTypePointer(SimTypeChar(label="Char"), offset=0)}, name="IKEEXT_CERT_NAME0", pack=False, align=None), offset=0), "flags": SimTypeInt(signed=False, label="UInt32")}, name="IKEEXT_CERTIFICATE_CRITERIA0", pack=False, align=None), offset=0)}, name="_Anonymous1_e__Struct", pack=False, align=None), "Anonymous2": SimStruct({"outboundEnterpriseStoreArraySize": SimTypeInt(signed=False, label="UInt32"), "outboundEnterpriseStoreCriteria": SimTypePointer(SimStruct({"certData": SimStruct({"size": SimTypeInt(signed=False, label="UInt32"), "data": SimTypePointer(SimTypeChar(label="Byte"), offset=0)}, name="FWP_BYTE_BLOB", pack=False, align=None), "certHash": SimStruct({"size": SimTypeInt(signed=False, label="UInt32"), "data": SimTypePointer(SimTypeChar(label="Byte"), offset=0)}, name="FWP_BYTE_BLOB", pack=False, align=None), "eku": SimTypePointer(SimStruct({"numEku": SimTypeInt(signed=False, label="UInt32"), "eku": SimTypePointer(SimTypePointer(SimTypeChar(label="Byte"), offset=0), offset=0)}, name="IKEEXT_CERT_EKUS0", pack=False, align=None), offset=0), "name": SimTypePointer(SimStruct({"nameType": SimTypeInt(signed=False, label="IKEEXT_CERT_CRITERIA_NAME_TYPE"), "certName": SimTypePointer(SimTypeChar(label="Char"), offset=0)}, name="IKEEXT_CERT_NAME0", pack=False, align=None), offset=0), "flags": SimTypeInt(signed=False, label="UInt32")}, name="IKEEXT_CERTIFICATE_CRITERIA0", pack=False, align=None), offset=0)}, name="_Anonymous2_e__Struct", pack=False, align=None), "Anonymous3": SimStruct({"outboundRootStoreArraySize": SimTypeInt(signed=False, label="UInt32"), "outboundTrustedRootStoreCriteria": SimTypePointer(SimStruct({"certData": SimStruct({"size": SimTypeInt(signed=False, label="UInt32"), "data": SimTypePointer(SimTypeChar(label="Byte"), offset=0)}, name="FWP_BYTE_BLOB", pack=False, align=None), "certHash": SimStruct({"size": SimTypeInt(signed=False, label="UInt32"), "data": SimTypePointer(SimTypeChar(label="Byte"), offset=0)}, name="FWP_BYTE_BLOB", pack=False, align=None), "eku": SimTypePointer(SimStruct({"numEku": SimTypeInt(signed=False, label="UInt32"), "eku": SimTypePointer(SimTypePointer(SimTypeChar(label="Byte"), offset=0), offset=0)}, name="IKEEXT_CERT_EKUS0", pack=False, align=None), offset=0), "name": SimTypePointer(SimStruct({"nameType": SimTypeInt(signed=False, label="IKEEXT_CERT_CRITERIA_NAME_TYPE"), "certName": SimTypePointer(SimTypeChar(label="Char"), offset=0)}, name="IKEEXT_CERT_NAME0", pack=False, align=None), offset=0), "flags": SimTypeInt(signed=False, label="UInt32")}, name="IKEEXT_CERTIFICATE_CRITERIA0", pack=False, align=None), offset=0)}, name="_Anonymous3_e__Struct", pack=False, align=None)}, name="<anon>", label="None"), "flags": SimTypeInt(signed=False, label="IKEEXT_CERT_AUTH"), "localCertLocationUrl": SimStruct({"size": SimTypeInt(signed=False, label="UInt32"), "data": SimTypePointer(SimTypeChar(label="Byte"), offset=0)}, name="FWP_BYTE_BLOB", pack=False, align=None)}, name="IKEEXT_CERTIFICATE_AUTHENTICATION2", pack=False, align=None), "cgaAuthentication": SimStruct({"keyContainerName": SimTypePointer(SimTypeChar(label="Char"), offset=0), "cspName": SimTypePointer(SimTypeChar(label="Char"), offset=0), "cspType": SimTypeInt(signed=False, label="UInt32"), "cgaModifier": SimStruct({"byteArray16": SimTypeFixedSizeArray(SimTypeChar(label="Byte"), 16)}, name="FWP_BYTE_ARRAY16", pack=False, align=None), "cgaCollisionCount": SimTypeChar(label="Byte")}, name="IKEEXT_IPV6_CGA_AUTHENTICATION0", pack=False, align=None), "eapAuthentication": SimStruct({"flags": SimTypeInt(signed=False, label="IKEEXT_EAP_AUTHENTICATION_FLAGS")}, name="IKEEXT_EAP_AUTHENTICATION0", pack=False, align=None)}, name="<anon>", label="None")}, name="IKEEXT_AUTHENTICATION_METHOD2", pack=False, align=None), offset=0), "initiatorImpersonationType": SimTypeInt(signed=False, label="IKEEXT_AUTHENTICATION_IMPERSONATION_TYPE")}, name="IKEEXT_EM_POLICY2", pack=False, align=None), offset=0)}, name="IPSEC_TRANSPORT_POLICY2", pack=False, align=None), offset=0), "authipQmTunnelPolicy": SimTypePointer(SimStruct({"flags": SimTypeInt(signed=False, label="UInt32"), "numIpsecProposals": SimTypeInt(signed=False, label="UInt32"), "ipsecProposals": SimTypePointer(SimStruct({"lifetime": SimStruct({"lifetimeSeconds": SimTypeInt(signed=False, label="UInt32"), "lifetimeKilobytes": SimTypeInt(signed=False, label="UInt32"), "lifetimePackets": SimTypeInt(signed=False, label="UInt32")}, name="IPSEC_SA_LIFETIME0", pack=False, align=None), "numSaTransforms": SimTypeInt(signed=False, label="UInt32"), "saTransforms": SimTypePointer(SimStruct({"ipsecTransformType": SimTypeInt(signed=False, label="IPSEC_TRANSFORM_TYPE"), "Anonymous": SimUnion({"ahTransform": SimTypePointer(SimStruct({"authTransformId": SimStruct({"authType": SimTypeInt(signed=False, label="IPSEC_AUTH_TYPE"), "authConfig": SimTypeChar(label="Byte")}, name="IPSEC_AUTH_TRANSFORM_ID0", pack=False, align=None), "cryptoModuleId": SimTypePointer(SimTypeBottom(label="Guid"), offset=0)}, name="IPSEC_AUTH_TRANSFORM0", pack=False, align=None), offset=0), "espAuthTransform": SimTypePointer(SimStruct({"authTransformId": SimStruct({"authType": SimTypeInt(signed=False, label="IPSEC_AUTH_TYPE"), "authConfig": SimTypeChar(label="Byte")}, name="IPSEC_AUTH_TRANSFORM_ID0", pack=False, align=None), "cryptoModuleId": SimTypePointer(SimTypeBottom(label="Guid"), offset=0)}, name="IPSEC_AUTH_TRANSFORM0", pack=False, align=None), offset=0), "espCipherTransform": SimTypePointer(SimStruct({"cipherTransformId": SimStruct({"cipherType": SimTypeInt(signed=False, label="IPSEC_CIPHER_TYPE"), "cipherConfig": SimTypeChar(label="Byte")}, name="IPSEC_CIPHER_TRANSFORM_ID0", pack=False, align=None), "cryptoModuleId": SimTypePointer(SimTypeBottom(label="Guid"), offset=0)}, name="IPSEC_CIPHER_TRANSFORM0", pack=False, align=None), offset=0), "espAuthAndCipherTransform": SimTypePointer(SimStruct({"authTransform": SimStruct({"authTransformId": SimStruct({"authType": SimTypeInt(signed=False, label="IPSEC_AUTH_TYPE"), "authConfig": SimTypeChar(label="Byte")}, name="IPSEC_AUTH_TRANSFORM_ID0", pack=False, align=None), "cryptoModuleId": SimTypePointer(SimTypeBottom(label="Guid"), offset=0)}, name="IPSEC_AUTH_TRANSFORM0", pack=False, align=None), "cipherTransform": SimStruct({"cipherTransformId": SimStruct({"cipherType": SimTypeInt(signed=False, label="IPSEC_CIPHER_TYPE"), "cipherConfig": SimTypeChar(label="Byte")}, name="IPSEC_CIPHER_TRANSFORM_ID0", pack=False, align=None), "cryptoModuleId": SimTypePointer(SimTypeBottom(label="Guid"), offset=0)}, name="IPSEC_CIPHER_TRANSFORM0", pack=False, align=None)}, name="IPSEC_AUTH_AND_CIPHER_TRANSFORM0", pack=False, align=None), offset=0), "espAuthFwTransform": SimTypePointer(SimStruct({"authTransformId": SimStruct({"authType": SimTypeInt(signed=False, label="IPSEC_AUTH_TYPE"), "authConfig": SimTypeChar(label="Byte")}, name="IPSEC_AUTH_TRANSFORM_ID0", pack=False, align=None), "cryptoModuleId": SimTypePointer(SimTypeBottom(label="Guid"), offset=0)}, name="IPSEC_AUTH_TRANSFORM0", pack=False, align=None), offset=0)}, name="<anon>", label="None")}, name="IPSEC_SA_TRANSFORM0", pack=False, align=None), offset=0), "pfsGroup": SimTypeInt(signed=False, label="IPSEC_PFS_GROUP")}, name="IPSEC_PROPOSAL0", pack=False, align=None), offset=0), "tunnelEndpoints": SimStruct({"ipVersion": SimTypeInt(signed=False, label="FWP_IP_VERSION"), "Anonymous1": SimUnion({"localV4Address": SimTypeInt(signed=False, label="UInt32"), "localV6Address": SimTypeFixedSizeArray(SimTypeChar(label="Byte"), 16)}, name="<anon>", label="None"), "Anonymous2": SimUnion({"remoteV4Address": SimTypeInt(signed=False, label="UInt32"), "remoteV6Address": SimTypeFixedSizeArray(SimTypeChar(label="Byte"), 16)}, name="<anon>", label="None"), "localIfLuid": SimTypeLongLong(signed=False, label="UInt64"), "remoteFqdn": SimTypePointer(SimTypeChar(label="Char"), offset=0), "numAddresses": SimTypeInt(signed=False, label="UInt32"), "remoteAddresses": SimTypePointer(SimStruct({"ipVersion": SimTypeInt(signed=False, label="FWP_IP_VERSION"), "Anonymous": SimUnion({"v4Address": SimTypeInt(signed=False, label="UInt32"), "v6Address": SimTypeFixedSizeArray(SimTypeChar(label="Byte"), 16)}, name="<anon>", label="None")}, name="IPSEC_TUNNEL_ENDPOINT0", pack=False, align=None), offset=0)}, name="IPSEC_TUNNEL_ENDPOINTS2", pack=False, align=None), "saIdleTimeout": SimStruct({"idleTimeoutSeconds": SimTypeInt(signed=False, label="UInt32"), "idleTimeoutSecondsFailOver": SimTypeInt(signed=False, label="UInt32")}, name="IPSEC_SA_IDLE_TIMEOUT0", pack=False, align=None), "emPolicy": SimTypePointer(SimStruct({"numAuthenticationMethods": SimTypeInt(signed=False, label="UInt32"), "authenticationMethods": SimTypePointer(SimStruct({"authenticationMethodType": SimTypeInt(signed=False, label="IKEEXT_AUTHENTICATION_METHOD_TYPE"), "Anonymous": SimUnion({"presharedKeyAuthentication": SimStruct({"presharedKey": SimStruct({"size": SimTypeInt(signed=False, label="UInt32"), "data": SimTypePointer(SimTypeChar(label="Byte"), offset=0)}, name="FWP_BYTE_BLOB", pack=False, align=None), "flags": SimTypeInt(signed=False, label="IKEEXT_PRESHARED_KEY_AUTHENTICATION_FLAGS")}, name="IKEEXT_PRESHARED_KEY_AUTHENTICATION1", pack=False, align=None), "certificateAuthentication": SimStruct({"inboundConfigType": SimTypeInt(signed=False, label="IKEEXT_CERT_CONFIG_TYPE"), "Anonymous1": SimUnion({"Anonymous1": SimStruct({"inboundRootArraySize": SimTypeInt(signed=False, label="UInt32"), "inboundRootCriteria": SimTypePointer(SimStruct({"certData": SimStruct({"size": SimTypeInt(signed=False, label="UInt32"), "data": SimTypePointer(SimTypeChar(label="Byte"), offset=0)}, name="FWP_BYTE_BLOB", pack=False, align=None), "certHash": SimStruct({"size": SimTypeInt(signed=False, label="UInt32"), "data": SimTypePointer(SimTypeChar(label="Byte"), offset=0)}, name="FWP_BYTE_BLOB", pack=False, align=None), "eku": SimTypePointer(SimStruct({"numEku": SimTypeInt(signed=False, label="UInt32"), "eku": SimTypePointer(SimTypePointer(SimTypeChar(label="Byte"), offset=0), offset=0)}, name="IKEEXT_CERT_EKUS0", pack=False, align=None), offset=0), "name": SimTypePointer(SimStruct({"nameType": SimTypeInt(signed=False, label="IKEEXT_CERT_CRITERIA_NAME_TYPE"), "certName": SimTypePointer(SimTypeChar(label="Char"), offset=0)}, name="IKEEXT_CERT_NAME0", pack=False, align=None), offset=0), "flags": SimTypeInt(signed=False, label="UInt32")}, name="IKEEXT_CERTIFICATE_CRITERIA0", pack=False, align=None), offset=0)}, name="_Anonymous1_e__Struct", pack=False, align=None), "Anonymous2": SimStruct({"inboundEnterpriseStoreArraySize": SimTypeInt(signed=False, label="UInt32"), "inboundEnterpriseStoreCriteria": SimTypePointer(SimStruct({"certData": SimStruct({"size": SimTypeInt(signed=False, label="UInt32"), "data": SimTypePointer(SimTypeChar(label="Byte"), offset=0)}, name="FWP_BYTE_BLOB", pack=False, align=None), "certHash": SimStruct({"size": SimTypeInt(signed=False, label="UInt32"), "data": SimTypePointer(SimTypeChar(label="Byte"), offset=0)}, name="FWP_BYTE_BLOB", pack=False, align=None), "eku": SimTypePointer(SimStruct({"numEku": SimTypeInt(signed=False, label="UInt32"), "eku": SimTypePointer(SimTypePointer(SimTypeChar(label="Byte"), offset=0), offset=0)}, name="IKEEXT_CERT_EKUS0", pack=False, align=None), offset=0), "name": SimTypePointer(SimStruct({"nameType": SimTypeInt(signed=False, label="IKEEXT_CERT_CRITERIA_NAME_TYPE"), "certName": SimTypePointer(SimTypeChar(label="Char"), offset=0)}, name="IKEEXT_CERT_NAME0", pack=False, align=None), offset=0), "flags": SimTypeInt(signed=False, label="UInt32")}, name="IKEEXT_CERTIFICATE_CRITERIA0", pack=False, align=None), offset=0)}, name="_Anonymous2_e__Struct", pack=False, align=None), "Anonymous3": SimStruct({"inboundRootStoreArraySize": SimTypeInt(signed=False, label="UInt32"), "inboundTrustedRootStoreCriteria": SimTypePointer(SimStruct({"certData": SimStruct({"size": SimTypeInt(signed=False, label="UInt32"), "data": SimTypePointer(SimTypeChar(label="Byte"), offset=0)}, name="FWP_BYTE_BLOB", pack=False, align=None), "certHash": SimStruct({"size": SimTypeInt(signed=False, label="UInt32"), "data": SimTypePointer(SimTypeChar(label="Byte"), offset=0)}, name="FWP_BYTE_BLOB", pack=False, align=None), "eku": SimTypePointer(SimStruct({"numEku": SimTypeInt(signed=False, label="UInt32"), "eku": SimTypePointer(SimTypePointer(SimTypeChar(label="Byte"), offset=0), offset=0)}, name="IKEEXT_CERT_EKUS0", pack=False, align=None), offset=0), "name": SimTypePointer(SimStruct({"nameType": SimTypeInt(signed=False, label="IKEEXT_CERT_CRITERIA_NAME_TYPE"), "certName": SimTypePointer(SimTypeChar(label="Char"), offset=0)}, name="IKEEXT_CERT_NAME0", pack=False, align=None), offset=0), "flags": SimTypeInt(signed=False, label="UInt32")}, name="IKEEXT_CERTIFICATE_CRITERIA0", pack=False, align=None), offset=0)}, name="_Anonymous3_e__Struct", pack=False, align=None)}, name="<anon>", label="None"), "outboundConfigType": SimTypeInt(signed=False, label="IKEEXT_CERT_CONFIG_TYPE"), "Anonymous2": SimUnion({"Anonymous1": SimStruct({"outboundRootArraySize": SimTypeInt(signed=False, label="UInt32"), "outboundRootCriteria": SimTypePointer(SimStruct({"certData": SimStruct({"size": SimTypeInt(signed=False, label="UInt32"), "data": SimTypePointer(SimTypeChar(label="Byte"), offset=0)}, name="FWP_BYTE_BLOB", pack=False, align=None), "certHash": SimStruct({"size": SimTypeInt(signed=False, label="UInt32"), "data": SimTypePointer(SimTypeChar(label="Byte"), offset=0)}, name="FWP_BYTE_BLOB", pack=False, align=None), "eku": SimTypePointer(SimStruct({"numEku": SimTypeInt(signed=False, label="UInt32"), "eku": SimTypePointer(SimTypePointer(SimTypeChar(label="Byte"), offset=0), offset=0)}, name="IKEEXT_CERT_EKUS0", pack=False, align=None), offset=0), "name": SimTypePointer(SimStruct({"nameType": SimTypeInt(signed=False, label="IKEEXT_CERT_CRITERIA_NAME_TYPE"), "certName": SimTypePointer(SimTypeChar(label="Char"), offset=0)}, name="IKEEXT_CERT_NAME0", pack=False, align=None), offset=0), "flags": SimTypeInt(signed=False, label="UInt32")}, name="IKEEXT_CERTIFICATE_CRITERIA0", pack=False, align=None), offset=0)}, name="_Anonymous1_e__Struct", pack=False, align=None), "Anonymous2": SimStruct({"outboundEnterpriseStoreArraySize": SimTypeInt(signed=False, label="UInt32"), "outboundEnterpriseStoreCriteria": SimTypePointer(SimStruct({"certData": SimStruct({"size": SimTypeInt(signed=False, label="UInt32"), "data": SimTypePointer(SimTypeChar(label="Byte"), offset=0)}, name="FWP_BYTE_BLOB", pack=False, align=None), "certHash": SimStruct({"size": SimTypeInt(signed=False, label="UInt32"), "data": SimTypePointer(SimTypeChar(label="Byte"), offset=0)}, name="FWP_BYTE_BLOB", pack=False, align=None), "eku": SimTypePointer(SimStruct({"numEku": SimTypeInt(signed=False, label="UInt32"), "eku": SimTypePointer(SimTypePointer(SimTypeChar(label="Byte"), offset=0), offset=0)}, name="IKEEXT_CERT_EKUS0", pack=False, align=None), offset=0), "name": SimTypePointer(SimStruct({"nameType": SimTypeInt(signed=False, label="IKEEXT_CERT_CRITERIA_NAME_TYPE"), "certName": SimTypePointer(SimTypeChar(label="Char"), offset=0)}, name="IKEEXT_CERT_NAME0", pack=False, align=None), offset=0), "flags": SimTypeInt(signed=False, label="UInt32")}, name="IKEEXT_CERTIFICATE_CRITERIA0", pack=False, align=None), offset=0)}, name="_Anonymous2_e__Struct", pack=False, align=None), "Anonymous3": SimStruct({"outboundRootStoreArraySize": SimTypeInt(signed=False, label="UInt32"), "outboundTrustedRootStoreCriteria": SimTypePointer(SimStruct({"certData": SimStruct({"size": SimTypeInt(signed=False, label="UInt32"), "data": SimTypePointer(SimTypeChar(label="Byte"), offset=0)}, name="FWP_BYTE_BLOB", pack=False, align=None), "certHash": SimStruct({"size": SimTypeInt(signed=False, label="UInt32"), "data": SimTypePointer(SimTypeChar(label="Byte"), offset=0)}, name="FWP_BYTE_BLOB", pack=False, align=None), "eku": SimTypePointer(SimStruct({"numEku": SimTypeInt(signed=False, label="UInt32"), "eku": SimTypePointer(SimTypePointer(SimTypeChar(label="Byte"), offset=0), offset=0)}, name="IKEEXT_CERT_EKUS0", pack=False, align=None), offset=0), "name": SimTypePointer(SimStruct({"nameType": SimTypeInt(signed=False, label="IKEEXT_CERT_CRITERIA_NAME_TYPE"), "certName": SimTypePointer(SimTypeChar(label="Char"), offset=0)}, name="IKEEXT_CERT_NAME0", pack=False, align=None), offset=0), "flags": SimTypeInt(signed=False, label="UInt32")}, name="IKEEXT_CERTIFICATE_CRITERIA0", pack=False, align=None), offset=0)}, name="_Anonymous3_e__Struct", pack=False, align=None)}, name="<anon>", label="None"), "flags": SimTypeInt(signed=False, label="IKEEXT_CERT_AUTH"), "localCertLocationUrl": SimStruct({"size": SimTypeInt(signed=False, label="UInt32"), "data": SimTypePointer(SimTypeChar(label="Byte"), offset=0)}, name="FWP_BYTE_BLOB", pack=False, align=None)}, name="IKEEXT_CERTIFICATE_AUTHENTICATION2", pack=False, align=None), "kerberosAuthentication": SimStruct({"flags": SimTypeInt(signed=False, label="IKEEXT_KERBEROS_AUTHENTICATION_FLAGS"), "proxyServer": SimTypePointer(SimTypeChar(label="Char"), offset=0)}, name="IKEEXT_KERBEROS_AUTHENTICATION1", pack=False, align=None), "reservedAuthentication": SimStruct({"flags": SimTypeInt(signed=False, label="IKEEXT_RESERVED_AUTHENTICATION_FLAGS")}, name="IKEEXT_RESERVED_AUTHENTICATION0", pack=False, align=None), "ntlmV2Authentication": SimStruct({"flags": SimTypeInt(signed=False, label="UInt32")}, name="IKEEXT_NTLM_V2_AUTHENTICATION0", pack=False, align=None), "sslAuthentication": SimStruct({"inboundConfigType": SimTypeInt(signed=False, label="IKEEXT_CERT_CONFIG_TYPE"), "Anonymous1": SimUnion({"Anonymous1": SimStruct({"inboundRootArraySize": SimTypeInt(signed=False, label="UInt32"), "inboundRootCriteria": SimTypePointer(SimStruct({"certData": SimStruct({"size": SimTypeInt(signed=False, label="UInt32"), "data": SimTypePointer(SimTypeChar(label="Byte"), offset=0)}, name="FWP_BYTE_BLOB", pack=False, align=None), "certHash": SimStruct({"size": SimTypeInt(signed=False, label="UInt32"), "data": SimTypePointer(SimTypeChar(label="Byte"), offset=0)}, name="FWP_BYTE_BLOB", pack=False, align=None), "eku": SimTypePointer(SimStruct({"numEku": SimTypeInt(signed=False, label="UInt32"), "eku": SimTypePointer(SimTypePointer(SimTypeChar(label="Byte"), offset=0), offset=0)}, name="IKEEXT_CERT_EKUS0", pack=False, align=None), offset=0), "name": SimTypePointer(SimStruct({"nameType": SimTypeInt(signed=False, label="IKEEXT_CERT_CRITERIA_NAME_TYPE"), "certName": SimTypePointer(SimTypeChar(label="Char"), offset=0)}, name="IKEEXT_CERT_NAME0", pack=False, align=None), offset=0), "flags": SimTypeInt(signed=False, label="UInt32")}, name="IKEEXT_CERTIFICATE_CRITERIA0", pack=False, align=None), offset=0)}, name="_Anonymous1_e__Struct", pack=False, align=None), "Anonymous2": SimStruct({"inboundEnterpriseStoreArraySize": SimTypeInt(signed=False, label="UInt32"), "inboundEnterpriseStoreCriteria": SimTypePointer(SimStruct({"certData": SimStruct({"size": SimTypeInt(signed=False, label="UInt32"), "data": SimTypePointer(SimTypeChar(label="Byte"), offset=0)}, name="FWP_BYTE_BLOB", pack=False, align=None), "certHash": SimStruct({"size": SimTypeInt(signed=False, label="UInt32"), "data": SimTypePointer(SimTypeChar(label="Byte"), offset=0)}, name="FWP_BYTE_BLOB", pack=False, align=None), "eku": SimTypePointer(SimStruct({"numEku": SimTypeInt(signed=False, label="UInt32"), "eku": SimTypePointer(SimTypePointer(SimTypeChar(label="Byte"), offset=0), offset=0)}, name="IKEEXT_CERT_EKUS0", pack=False, align=None), offset=0), "name": SimTypePointer(SimStruct({"nameType": SimTypeInt(signed=False, label="IKEEXT_CERT_CRITERIA_NAME_TYPE"), "certName": SimTypePointer(SimTypeChar(label="Char"), offset=0)}, name="IKEEXT_CERT_NAME0", pack=False, align=None), offset=0), "flags": SimTypeInt(signed=False, label="UInt32")}, name="IKEEXT_CERTIFICATE_CRITERIA0", pack=False, align=None), offset=0)}, name="_Anonymous2_e__Struct", pack=False, align=None), "Anonymous3": SimStruct({"inboundRootStoreArraySize": SimTypeInt(signed=False, label="UInt32"), "inboundTrustedRootStoreCriteria": SimTypePointer(SimStruct({"certData": SimStruct({"size": SimTypeInt(signed=False, label="UInt32"), "data": SimTypePointer(SimTypeChar(label="Byte"), offset=0)}, name="FWP_BYTE_BLOB", pack=False, align=None), "certHash": SimStruct({"size": SimTypeInt(signed=False, label="UInt32"), "data": SimTypePointer(SimTypeChar(label="Byte"), offset=0)}, name="FWP_BYTE_BLOB", pack=False, align=None), "eku": SimTypePointer(SimStruct({"numEku": SimTypeInt(signed=False, label="UInt32"), "eku": SimTypePointer(SimTypePointer(SimTypeChar(label="Byte"), offset=0), offset=0)}, name="IKEEXT_CERT_EKUS0", pack=False, align=None), offset=0), "name": SimTypePointer(SimStruct({"nameType": SimTypeInt(signed=False, label="IKEEXT_CERT_CRITERIA_NAME_TYPE"), "certName": SimTypePointer(SimTypeChar(label="Char"), offset=0)}, name="IKEEXT_CERT_NAME0", pack=False, align=None), offset=0), "flags": SimTypeInt(signed=False, label="UInt32")}, name="IKEEXT_CERTIFICATE_CRITERIA0", pack=False, align=None), offset=0)}, name="_Anonymous3_e__Struct", pack=False, align=None)}, name="<anon>", label="None"), "outboundConfigType": SimTypeInt(signed=False, label="IKEEXT_CERT_CONFIG_TYPE"), "Anonymous2": SimUnion({"Anonymous1": SimStruct({"outboundRootArraySize": SimTypeInt(signed=False, label="UInt32"), "outboundRootCriteria": SimTypePointer(SimStruct({"certData": SimStruct({"size": SimTypeInt(signed=False, label="UInt32"), "data": SimTypePointer(SimTypeChar(label="Byte"), offset=0)}, name="FWP_BYTE_BLOB", pack=False, align=None), "certHash": SimStruct({"size": SimTypeInt(signed=False, label="UInt32"), "data": SimTypePointer(SimTypeChar(label="Byte"), offset=0)}, name="FWP_BYTE_BLOB", pack=False, align=None), "eku": SimTypePointer(SimStruct({"numEku": SimTypeInt(signed=False, label="UInt32"), "eku": SimTypePointer(SimTypePointer(SimTypeChar(label="Byte"), offset=0), offset=0)}, name="IKEEXT_CERT_EKUS0", pack=False, align=None), offset=0), "name": SimTypePointer(SimStruct({"nameType": SimTypeInt(signed=False, label="IKEEXT_CERT_CRITERIA_NAME_TYPE"), "certName": SimTypePointer(SimTypeChar(label="Char"), offset=0)}, name="IKEEXT_CERT_NAME0", pack=False, align=None), offset=0), "flags": SimTypeInt(signed=False, label="UInt32")}, name="IKEEXT_CERTIFICATE_CRITERIA0", pack=False, align=None), offset=0)}, name="_Anonymous1_e__Struct", pack=False, align=None), "Anonymous2": SimStruct({"outboundEnterpriseStoreArraySize": SimTypeInt(signed=False, label="UInt32"), "outboundEnterpriseStoreCriteria": SimTypePointer(SimStruct({"certData": SimStruct({"size": SimTypeInt(signed=False, label="UInt32"), "data": SimTypePointer(SimTypeChar(label="Byte"), offset=0)}, name="FWP_BYTE_BLOB", pack=False, align=None), "certHash": SimStruct({"size": SimTypeInt(signed=False, label="UInt32"), "data": SimTypePointer(SimTypeChar(label="Byte"), offset=0)}, name="FWP_BYTE_BLOB", pack=False, align=None), "eku": SimTypePointer(SimStruct({"numEku": SimTypeInt(signed=False, label="UInt32"), "eku": SimTypePointer(SimTypePointer(SimTypeChar(label="Byte"), offset=0), offset=0)}, name="IKEEXT_CERT_EKUS0", pack=False, align=None), offset=0), "name": SimTypePointer(SimStruct({"nameType": SimTypeInt(signed=False, label="IKEEXT_CERT_CRITERIA_NAME_TYPE"), "certName": SimTypePointer(SimTypeChar(label="Char"), offset=0)}, name="IKEEXT_CERT_NAME0", pack=False, align=None), offset=0), "flags": SimTypeInt(signed=False, label="UInt32")}, name="IKEEXT_CERTIFICATE_CRITERIA0", pack=False, align=None), offset=0)}, name="_Anonymous2_e__Struct", pack=False, align=None), "Anonymous3": SimStruct({"outboundRootStoreArraySize": SimTypeInt(signed=False, label="UInt32"), "outboundTrustedRootStoreCriteria": SimTypePointer(SimStruct({"certData": SimStruct({"size": SimTypeInt(signed=False, label="UInt32"), "data": SimTypePointer(SimTypeChar(label="Byte"), offset=0)}, name="FWP_BYTE_BLOB", pack=False, align=None), "certHash": SimStruct({"size": SimTypeInt(signed=False, label="UInt32"), "data": SimTypePointer(SimTypeChar(label="Byte"), offset=0)}, name="FWP_BYTE_BLOB", pack=False, align=None), "eku": SimTypePointer(SimStruct({"numEku": SimTypeInt(signed=False, label="UInt32"), "eku": SimTypePointer(SimTypePointer(SimTypeChar(label="Byte"), offset=0), offset=0)}, name="IKEEXT_CERT_EKUS0", pack=False, align=None), offset=0), "name": SimTypePointer(SimStruct({"nameType": SimTypeInt(signed=False, label="IKEEXT_CERT_CRITERIA_NAME_TYPE"), "certName": SimTypePointer(SimTypeChar(label="Char"), offset=0)}, name="IKEEXT_CERT_NAME0", pack=False, align=None), offset=0), "flags": SimTypeInt(signed=False, label="UInt32")}, name="IKEEXT_CERTIFICATE_CRITERIA0", pack=False, align=None), offset=0)}, name="_Anonymous3_e__Struct", pack=False, align=None)}, name="<anon>", label="None"), "flags": SimTypeInt(signed=False, label="IKEEXT_CERT_AUTH"), "localCertLocationUrl": SimStruct({"size": SimTypeInt(signed=False, label="UInt32"), "data": SimTypePointer(SimTypeChar(label="Byte"), offset=0)}, name="FWP_BYTE_BLOB", pack=False, align=None)}, name="IKEEXT_CERTIFICATE_AUTHENTICATION2", pack=False, align=None), "cgaAuthentication": SimStruct({"keyContainerName": SimTypePointer(SimTypeChar(label="Char"), offset=0), "cspName": SimTypePointer(SimTypeChar(label="Char"), offset=0), "cspType": SimTypeInt(signed=False, label="UInt32"), "cgaModifier": SimStruct({"byteArray16": SimTypeFixedSizeArray(SimTypeChar(label="Byte"), 16)}, name="FWP_BYTE_ARRAY16", pack=False, align=None), "cgaCollisionCount": SimTypeChar(label="Byte")}, name="IKEEXT_IPV6_CGA_AUTHENTICATION0", pack=False, align=None), "eapAuthentication": SimStruct({"flags": SimTypeInt(signed=False, label="IKEEXT_EAP_AUTHENTICATION_FLAGS")}, name="IKEEXT_EAP_AUTHENTICATION0", pack=False, align=None)}, name="<anon>", label="None")}, name="IKEEXT_AUTHENTICATION_METHOD2", pack=False, align=None), offset=0), "initiatorImpersonationType": SimTypeInt(signed=False, label="IKEEXT_AUTHENTICATION_IMPERSONATION_TYPE")}, name="IKEEXT_EM_POLICY2", pack=False, align=None), offset=0), "fwdPathSaLifetime": SimTypeInt(signed=False, label="UInt32"), "compartmentId": SimTypeInt(signed=False, label="UInt32"), "numTrafficSelectorPolicy": SimTypeInt(signed=False, label="UInt32"), "trafficSelectorPolicies": SimTypePointer(SimStruct({"flags": SimTypeInt(signed=False, label="UInt32"), "numLocalTrafficSelectors": SimTypeInt(signed=False, label="UInt32"), "localTrafficSelectors": SimTypePointer(SimStruct({"protocolId": SimTypeChar(label="Byte"), "portStart": SimTypeShort(signed=False, label="UInt16"), "portEnd": SimTypeShort(signed=False, label="UInt16"), "ipVersion": SimTypeInt(signed=False, label="FWP_IP_VERSION"), "Anonymous1": SimUnion({"startV4Address": SimTypeInt(signed=False, label="UInt32"), "startV6Address": SimTypeFixedSizeArray(SimTypeChar(label="Byte"), 16)}, name="<anon>", label="None"), "Anonymous2": SimUnion({"endV4Address": SimTypeInt(signed=False, label="UInt32"), "endV6Address": SimTypeFixedSizeArray(SimTypeChar(label="Byte"), 16)}, name="<anon>", label="None")}, name="IPSEC_TRAFFIC_SELECTOR0_", pack=False, align=None), offset=0), "numRemoteTrafficSelectors": SimTypeInt(signed=False, label="UInt32"), "remoteTrafficSelectors": SimTypePointer(SimStruct({"protocolId": SimTypeChar(label="Byte"), "portStart": SimTypeShort(signed=False, label="UInt16"), "portEnd": SimTypeShort(signed=False, label="UInt16"), "ipVersion": SimTypeInt(signed=False, label="FWP_IP_VERSION"), "Anonymous1": SimUnion({"startV4Address": SimTypeInt(signed=False, label="UInt32"), "startV6Address": SimTypeFixedSizeArray(SimTypeChar(label="Byte"), 16)}, name="<anon>", label="None"), "Anonymous2": SimUnion({"endV4Address": SimTypeInt(signed=False, label="UInt32"), "endV6Address": SimTypeFixedSizeArray(SimTypeChar(label="Byte"), 16)}, name="<anon>", label="None")}, name="IPSEC_TRAFFIC_SELECTOR0_", pack=False, align=None), offset=0)}, name="IPSEC_TRAFFIC_SELECTOR_POLICY0_", pack=False, align=None), offset=0)}, name="IPSEC_TUNNEL_POLICY3_", pack=False, align=None), offset=0), "ikeMmPolicy": SimTypePointer(SimStruct({"softExpirationTime": SimTypeInt(signed=False, label="UInt32"), "numAuthenticationMethods": SimTypeInt(signed=False, label="UInt32"), "authenticationMethods": SimTypePointer(SimStruct({"authenticationMethodType": SimTypeInt(signed=False, label="IKEEXT_AUTHENTICATION_METHOD_TYPE"), "Anonymous": SimUnion({"presharedKeyAuthentication": SimStruct({"presharedKey": SimStruct({"size": SimTypeInt(signed=False, label="UInt32"), "data": SimTypePointer(SimTypeChar(label="Byte"), offset=0)}, name="FWP_BYTE_BLOB", pack=False, align=None), "flags": SimTypeInt(signed=False, label="IKEEXT_PRESHARED_KEY_AUTHENTICATION_FLAGS")}, name="IKEEXT_PRESHARED_KEY_AUTHENTICATION1", pack=False, align=None), "certificateAuthentication": SimStruct({"inboundConfigType": SimTypeInt(signed=False, label="IKEEXT_CERT_CONFIG_TYPE"), "Anonymous1": SimUnion({"Anonymous1": SimStruct({"inboundRootArraySize": SimTypeInt(signed=False, label="UInt32"), "inboundRootCriteria": SimTypePointer(SimStruct({"certData": SimStruct({"size": SimTypeInt(signed=False, label="UInt32"), "data": SimTypePointer(SimTypeChar(label="Byte"), offset=0)}, name="FWP_BYTE_BLOB", pack=False, align=None), "certHash": SimStruct({"size": SimTypeInt(signed=False, label="UInt32"), "data": SimTypePointer(SimTypeChar(label="Byte"), offset=0)}, name="FWP_BYTE_BLOB", pack=False, align=None), "eku": SimTypePointer(SimStruct({"numEku": SimTypeInt(signed=False, label="UInt32"), "eku": SimTypePointer(SimTypePointer(SimTypeChar(label="Byte"), offset=0), offset=0)}, name="IKEEXT_CERT_EKUS0", pack=False, align=None), offset=0), "name": SimTypePointer(SimStruct({"nameType": SimTypeInt(signed=False, label="IKEEXT_CERT_CRITERIA_NAME_TYPE"), "certName": SimTypePointer(SimTypeChar(label="Char"), offset=0)}, name="IKEEXT_CERT_NAME0", pack=False, align=None), offset=0), "flags": SimTypeInt(signed=False, label="UInt32")}, name="IKEEXT_CERTIFICATE_CRITERIA0", pack=False, align=None), offset=0)}, name="_Anonymous1_e__Struct", pack=False, align=None), "Anonymous2": SimStruct({"inboundEnterpriseStoreArraySize": SimTypeInt(signed=False, label="UInt32"), "inboundEnterpriseStoreCriteria": SimTypePointer(SimStruct({"certData": SimStruct({"size": SimTypeInt(signed=False, label="UInt32"), "data": SimTypePointer(SimTypeChar(label="Byte"), offset=0)}, name="FWP_BYTE_BLOB", pack=False, align=None), "certHash": SimStruct({"size": SimTypeInt(signed=False, label="UInt32"), "data": SimTypePointer(SimTypeChar(label="Byte"), offset=0)}, name="FWP_BYTE_BLOB", pack=False, align=None), "eku": SimTypePointer(SimStruct({"numEku": SimTypeInt(signed=False, label="UInt32"), "eku": SimTypePointer(SimTypePointer(SimTypeChar(label="Byte"), offset=0), offset=0)}, name="IKEEXT_CERT_EKUS0", pack=False, align=None), offset=0), "name": SimTypePointer(SimStruct({"nameType": SimTypeInt(signed=False, label="IKEEXT_CERT_CRITERIA_NAME_TYPE"), "certName": SimTypePointer(SimTypeChar(label="Char"), offset=0)}, name="IKEEXT_CERT_NAME0", pack=False, align=None), offset=0), "flags": SimTypeInt(signed=False, label="UInt32")}, name="IKEEXT_CERTIFICATE_CRITERIA0", pack=False, align=None), offset=0)}, name="_Anonymous2_e__Struct", pack=False, align=None), "Anonymous3": SimStruct({"inboundRootStoreArraySize": SimTypeInt(signed=False, label="UInt32"), "inboundTrustedRootStoreCriteria": SimTypePointer(SimStruct({"certData": SimStruct({"size": SimTypeInt(signed=False, label="UInt32"), "data": SimTypePointer(SimTypeChar(label="Byte"), offset=0)}, name="FWP_BYTE_BLOB", pack=False, align=None), "certHash": SimStruct({"size": SimTypeInt(signed=False, label="UInt32"), "data": SimTypePointer(SimTypeChar(label="Byte"), offset=0)}, name="FWP_BYTE_BLOB", pack=False, align=None), "eku": SimTypePointer(SimStruct({"numEku": SimTypeInt(signed=False, label="UInt32"), "eku": SimTypePointer(SimTypePointer(SimTypeChar(label="Byte"), offset=0), offset=0)}, name="IKEEXT_CERT_EKUS0", pack=False, align=None), offset=0), "name": SimTypePointer(SimStruct({"nameType": SimTypeInt(signed=False, label="IKEEXT_CERT_CRITERIA_NAME_TYPE"), "certName": SimTypePointer(SimTypeChar(label="Char"), offset=0)}, name="IKEEXT_CERT_NAME0", pack=False, align=None), offset=0), "flags": SimTypeInt(signed=False, label="UInt32")}, name="IKEEXT_CERTIFICATE_CRITERIA0", pack=False, align=None), offset=0)}, name="_Anonymous3_e__Struct", pack=False, align=None)}, name="<anon>", label="None"), "outboundConfigType": SimTypeInt(signed=False, label="IKEEXT_CERT_CONFIG_TYPE"), "Anonymous2": SimUnion({"Anonymous1": SimStruct({"outboundRootArraySize": SimTypeInt(signed=False, label="UInt32"), "outboundRootCriteria": SimTypePointer(SimStruct({"certData": SimStruct({"size": SimTypeInt(signed=False, label="UInt32"), "data": SimTypePointer(SimTypeChar(label="Byte"), offset=0)}, name="FWP_BYTE_BLOB", pack=False, align=None), "certHash": SimStruct({"size": SimTypeInt(signed=False, label="UInt32"), "data": SimTypePointer(SimTypeChar(label="Byte"), offset=0)}, name="FWP_BYTE_BLOB", pack=False, align=None), "eku": SimTypePointer(SimStruct({"numEku": SimTypeInt(signed=False, label="UInt32"), "eku": SimTypePointer(SimTypePointer(SimTypeChar(label="Byte"), offset=0), offset=0)}, name="IKEEXT_CERT_EKUS0", pack=False, align=None), offset=0), "name": SimTypePointer(SimStruct({"nameType": SimTypeInt(signed=False, label="IKEEXT_CERT_CRITERIA_NAME_TYPE"), "certName": SimTypePointer(SimTypeChar(label="Char"), offset=0)}, name="IKEEXT_CERT_NAME0", pack=False, align=None), offset=0), "flags": SimTypeInt(signed=False, label="UInt32")}, name="IKEEXT_CERTIFICATE_CRITERIA0", pack=False, align=None), offset=0)}, name="_Anonymous1_e__Struct", pack=False, align=None), "Anonymous2": SimStruct({"outboundEnterpriseStoreArraySize": SimTypeInt(signed=False, label="UInt32"), "outboundEnterpriseStoreCriteria": SimTypePointer(SimStruct({"certData": SimStruct({"size": SimTypeInt(signed=False, label="UInt32"), "data": SimTypePointer(SimTypeChar(label="Byte"), offset=0)}, name="FWP_BYTE_BLOB", pack=False, align=None), "certHash": SimStruct({"size": SimTypeInt(signed=False, label="UInt32"), "data": SimTypePointer(SimTypeChar(label="Byte"), offset=0)}, name="FWP_BYTE_BLOB", pack=False, align=None), "eku": SimTypePointer(SimStruct({"numEku": SimTypeInt(signed=False, label="UInt32"), "eku": SimTypePointer(SimTypePointer(SimTypeChar(label="Byte"), offset=0), offset=0)}, name="IKEEXT_CERT_EKUS0", pack=False, align=None), offset=0), "name": SimTypePointer(SimStruct({"nameType": SimTypeInt(signed=False, label="IKEEXT_CERT_CRITERIA_NAME_TYPE"), "certName": SimTypePointer(SimTypeChar(label="Char"), offset=0)}, name="IKEEXT_CERT_NAME0", pack=False, align=None), offset=0), "flags": SimTypeInt(signed=False, label="UInt32")}, name="IKEEXT_CERTIFICATE_CRITERIA0", pack=False, align=None), offset=0)}, name="_Anonymous2_e__Struct", pack=False, align=None), "Anonymous3": SimStruct({"outboundRootStoreArraySize": SimTypeInt(signed=False, label="UInt32"), "outboundTrustedRootStoreCriteria": SimTypePointer(SimStruct({"certData": SimStruct({"size": SimTypeInt(signed=False, label="UInt32"), "data": SimTypePointer(SimTypeChar(label="Byte"), offset=0)}, name="FWP_BYTE_BLOB", pack=False, align=None), "certHash": SimStruct({"size": SimTypeInt(signed=False, label="UInt32"), "data": SimTypePointer(SimTypeChar(label="Byte"), offset=0)}, name="FWP_BYTE_BLOB", pack=False, align=None), "eku": SimTypePointer(SimStruct({"numEku": SimTypeInt(signed=False, label="UInt32"), "eku": SimTypePointer(SimTypePointer(SimTypeChar(label="Byte"), offset=0), offset=0)}, name="IKEEXT_CERT_EKUS0", pack=False, align=None), offset=0), "name": SimTypePointer(SimStruct({"nameType": SimTypeInt(signed=False, label="IKEEXT_CERT_CRITERIA_NAME_TYPE"), "certName": SimTypePointer(SimTypeChar(label="Char"), offset=0)}, name="IKEEXT_CERT_NAME0", pack=False, align=None), offset=0), "flags": SimTypeInt(signed=False, label="UInt32")}, name="IKEEXT_CERTIFICATE_CRITERIA0", pack=False, align=None), offset=0)}, name="_Anonymous3_e__Struct", pack=False, align=None)}, name="<anon>", label="None"), "flags": SimTypeInt(signed=False, label="IKEEXT_CERT_AUTH"), "localCertLocationUrl": SimStruct({"size": SimTypeInt(signed=False, label="UInt32"), "data": SimTypePointer(SimTypeChar(label="Byte"), offset=0)}, name="FWP_BYTE_BLOB", pack=False, align=None)}, name="IKEEXT_CERTIFICATE_AUTHENTICATION2", pack=False, align=None), "kerberosAuthentication": SimStruct({"flags": SimTypeInt(signed=False, label="IKEEXT_KERBEROS_AUTHENTICATION_FLAGS"), "proxyServer": SimTypePointer(SimTypeChar(label="Char"), offset=0)}, name="IKEEXT_KERBEROS_AUTHENTICATION1", pack=False, align=None), "reservedAuthentication": SimStruct({"flags": SimTypeInt(signed=False, label="IKEEXT_RESERVED_AUTHENTICATION_FLAGS")}, name="IKEEXT_RESERVED_AUTHENTICATION0", pack=False, align=None), "ntlmV2Authentication": SimStruct({"flags": SimTypeInt(signed=False, label="UInt32")}, name="IKEEXT_NTLM_V2_AUTHENTICATION0", pack=False, align=None), "sslAuthentication": SimStruct({"inboundConfigType": SimTypeInt(signed=False, label="IKEEXT_CERT_CONFIG_TYPE"), "Anonymous1": SimUnion({"Anonymous1": SimStruct({"inboundRootArraySize": SimTypeInt(signed=False, label="UInt32"), "inboundRootCriteria": SimTypePointer(SimStruct({"certData": SimStruct({"size": SimTypeInt(signed=False, label="UInt32"), "data": SimTypePointer(SimTypeChar(label="Byte"), offset=0)}, name="FWP_BYTE_BLOB", pack=False, align=None), "certHash": SimStruct({"size": SimTypeInt(signed=False, label="UInt32"), "data": SimTypePointer(SimTypeChar(label="Byte"), offset=0)}, name="FWP_BYTE_BLOB", pack=False, align=None), "eku": SimTypePointer(SimStruct({"numEku": SimTypeInt(signed=False, label="UInt32"), "eku": SimTypePointer(SimTypePointer(SimTypeChar(label="Byte"), offset=0), offset=0)}, name="IKEEXT_CERT_EKUS0", pack=False, align=None), offset=0), "name": SimTypePointer(SimStruct({"nameType": SimTypeInt(signed=False, label="IKEEXT_CERT_CRITERIA_NAME_TYPE"), "certName": SimTypePointer(SimTypeChar(label="Char"), offset=0)}, name="IKEEXT_CERT_NAME0", pack=False, align=None), offset=0), "flags": SimTypeInt(signed=False, label="UInt32")}, name="IKEEXT_CERTIFICATE_CRITERIA0", pack=False, align=None), offset=0)}, name="_Anonymous1_e__Struct", pack=False, align=None), "Anonymous2": SimStruct({"inboundEnterpriseStoreArraySize": SimTypeInt(signed=False, label="UInt32"), "inboundEnterpriseStoreCriteria": SimTypePointer(SimStruct({"certData": SimStruct({"size": SimTypeInt(signed=False, label="UInt32"), "data": SimTypePointer(SimTypeChar(label="Byte"), offset=0)}, name="FWP_BYTE_BLOB", pack=False, align=None), "certHash": SimStruct({"size": SimTypeInt(signed=False, label="UInt32"), "data": SimTypePointer(SimTypeChar(label="Byte"), offset=0)}, name="FWP_BYTE_BLOB", pack=False, align=None), "eku": SimTypePointer(SimStruct({"numEku": SimTypeInt(signed=False, label="UInt32"), "eku": SimTypePointer(SimTypePointer(SimTypeChar(label="Byte"), offset=0), offset=0)}, name="IKEEXT_CERT_EKUS0", pack=False, align=None), offset=0), "name": SimTypePointer(SimStruct({"nameType": SimTypeInt(signed=False, label="IKEEXT_CERT_CRITERIA_NAME_TYPE"), "certName": SimTypePointer(SimTypeChar(label="Char"), offset=0)}, name="IKEEXT_CERT_NAME0", pack=False, align=None), offset=0), "flags": SimTypeInt(signed=False, label="UInt32")}, name="IKEEXT_CERTIFICATE_CRITERIA0", pack=False, align=None), offset=0)}, name="_Anonymous2_e__Struct", pack=False, align=None), "Anonymous3": SimStruct({"inboundRootStoreArraySize": SimTypeInt(signed=False, label="UInt32"), "inboundTrustedRootStoreCriteria": SimTypePointer(SimStruct({"certData": SimStruct({"size": SimTypeInt(signed=False, label="UInt32"), "data": SimTypePointer(SimTypeChar(label="Byte"), offset=0)}, name="FWP_BYTE_BLOB", pack=False, align=None), "certHash": SimStruct({"size": SimTypeInt(signed=False, label="UInt32"), "data": SimTypePointer(SimTypeChar(label="Byte"), offset=0)}, name="FWP_BYTE_BLOB", pack=False, align=None), "eku": SimTypePointer(SimStruct({"numEku": SimTypeInt(signed=False, label="UInt32"), "eku": SimTypePointer(SimTypePointer(SimTypeChar(label="Byte"), offset=0), offset=0)}, name="IKEEXT_CERT_EKUS0", pack=False, align=None), offset=0), "name": SimTypePointer(SimStruct({"nameType": SimTypeInt(signed=False, label="IKEEXT_CERT_CRITERIA_NAME_TYPE"), "certName": SimTypePointer(SimTypeChar(label="Char"), offset=0)}, name="IKEEXT_CERT_NAME0", pack=False, align=None), offset=0), "flags": SimTypeInt(signed=False, label="UInt32")}, name="IKEEXT_CERTIFICATE_CRITERIA0", pack=False, align=None), offset=0)}, name="_Anonymous3_e__Struct", pack=False, align=None)}, name="<anon>", label="None"), "outboundConfigType": SimTypeInt(signed=False, label="IKEEXT_CERT_CONFIG_TYPE"), "Anonymous2": SimUnion({"Anonymous1": SimStruct({"outboundRootArraySize": SimTypeInt(signed=False, label="UInt32"), "outboundRootCriteria": SimTypePointer(SimStruct({"certData": SimStruct({"size": SimTypeInt(signed=False, label="UInt32"), "data": SimTypePointer(SimTypeChar(label="Byte"), offset=0)}, name="FWP_BYTE_BLOB", pack=False, align=None), "certHash": SimStruct({"size": SimTypeInt(signed=False, label="UInt32"), "data": SimTypePointer(SimTypeChar(label="Byte"), offset=0)}, name="FWP_BYTE_BLOB", pack=False, align=None), "eku": SimTypePointer(SimStruct({"numEku": SimTypeInt(signed=False, label="UInt32"), "eku": SimTypePointer(SimTypePointer(SimTypeChar(label="Byte"), offset=0), offset=0)}, name="IKEEXT_CERT_EKUS0", pack=False, align=None), offset=0), "name": SimTypePointer(SimStruct({"nameType": SimTypeInt(signed=False, label="IKEEXT_CERT_CRITERIA_NAME_TYPE"), "certName": SimTypePointer(SimTypeChar(label="Char"), offset=0)}, name="IKEEXT_CERT_NAME0", pack=False, align=None), offset=0), "flags": SimTypeInt(signed=False, label="UInt32")}, name="IKEEXT_CERTIFICATE_CRITERIA0", pack=False, align=None), offset=0)}, name="_Anonymous1_e__Struct", pack=False, align=None), "Anonymous2": SimStruct({"outboundEnterpriseStoreArraySize": SimTypeInt(signed=False, label="UInt32"), "outboundEnterpriseStoreCriteria": SimTypePointer(SimStruct({"certData": SimStruct({"size": SimTypeInt(signed=False, label="UInt32"), "data": SimTypePointer(SimTypeChar(label="Byte"), offset=0)}, name="FWP_BYTE_BLOB", pack=False, align=None), "certHash": SimStruct({"size": SimTypeInt(signed=False, label="UInt32"), "data": SimTypePointer(SimTypeChar(label="Byte"), offset=0)}, name="FWP_BYTE_BLOB", pack=False, align=None), "eku": SimTypePointer(SimStruct({"numEku": SimTypeInt(signed=False, label="UInt32"), "eku": SimTypePointer(SimTypePointer(SimTypeChar(label="Byte"), offset=0), offset=0)}, name="IKEEXT_CERT_EKUS0", pack=False, align=None), offset=0), "name": SimTypePointer(SimStruct({"nameType": SimTypeInt(signed=False, label="IKEEXT_CERT_CRITERIA_NAME_TYPE"), "certName": SimTypePointer(SimTypeChar(label="Char"), offset=0)}, name="IKEEXT_CERT_NAME0", pack=False, align=None), offset=0), "flags": SimTypeInt(signed=False, label="UInt32")}, name="IKEEXT_CERTIFICATE_CRITERIA0", pack=False, align=None), offset=0)}, name="_Anonymous2_e__Struct", pack=False, align=None), "Anonymous3": SimStruct({"outboundRootStoreArraySize": SimTypeInt(signed=False, label="UInt32"), "outboundTrustedRootStoreCriteria": SimTypePointer(SimStruct({"certData": SimStruct({"size": SimTypeInt(signed=False, label="UInt32"), "data": SimTypePointer(SimTypeChar(label="Byte"), offset=0)}, name="FWP_BYTE_BLOB", pack=False, align=None), "certHash": SimStruct({"size": SimTypeInt(signed=False, label="UInt32"), "data": SimTypePointer(SimTypeChar(label="Byte"), offset=0)}, name="FWP_BYTE_BLOB", pack=False, align=None), "eku": SimTypePointer(SimStruct({"numEku": SimTypeInt(signed=False, label="UInt32"), "eku": SimTypePointer(SimTypePointer(SimTypeChar(label="Byte"), offset=0), offset=0)}, name="IKEEXT_CERT_EKUS0", pack=False, align=None), offset=0), "name": SimTypePointer(SimStruct({"nameType": SimTypeInt(signed=False, label="IKEEXT_CERT_CRITERIA_NAME_TYPE"), "certName": SimTypePointer(SimTypeChar(label="Char"), offset=0)}, name="IKEEXT_CERT_NAME0", pack=False, align=None), offset=0), "flags": SimTypeInt(signed=False, label="UInt32")}, name="IKEEXT_CERTIFICATE_CRITERIA0", pack=False, align=None), offset=0)}, name="_Anonymous3_e__Struct", pack=False, align=None)}, name="<anon>", label="None"), "flags": SimTypeInt(signed=False, label="IKEEXT_CERT_AUTH"), "localCertLocationUrl": SimStruct({"size": SimTypeInt(signed=False, label="UInt32"), "data": SimTypePointer(SimTypeChar(label="Byte"), offset=0)}, name="FWP_BYTE_BLOB", pack=False, align=None)}, name="IKEEXT_CERTIFICATE_AUTHENTICATION2", pack=False, align=None), "cgaAuthentication": SimStruct({"keyContainerName": SimTypePointer(SimTypeChar(label="Char"), offset=0), "cspName": SimTypePointer(SimTypeChar(label="Char"), offset=0), "cspType": SimTypeInt(signed=False, label="UInt32"), "cgaModifier": SimStruct({"byteArray16": SimTypeFixedSizeArray(SimTypeChar(label="Byte"), 16)}, name="FWP_BYTE_ARRAY16", pack=False, align=None), "cgaCollisionCount": SimTypeChar(label="Byte")}, name="IKEEXT_IPV6_CGA_AUTHENTICATION0", pack=False, align=None), "eapAuthentication": SimStruct({"flags": SimTypeInt(signed=False, label="IKEEXT_EAP_AUTHENTICATION_FLAGS")}, name="IKEEXT_EAP_AUTHENTICATION0", pack=False, align=None)}, name="<anon>", label="None")}, name="IKEEXT_AUTHENTICATION_METHOD2", pack=False, align=None), offset=0), "initiatorImpersonationType": SimTypeInt(signed=False, label="IKEEXT_AUTHENTICATION_IMPERSONATION_TYPE"), "numIkeProposals": SimTypeInt(signed=False, label="UInt32"), "ikeProposals": SimTypePointer(SimStruct({"cipherAlgorithm": SimStruct({"algoIdentifier": SimTypeInt(signed=False, label="IKEEXT_CIPHER_TYPE"), "keyLen": SimTypeInt(signed=False, label="UInt32"), "rounds": SimTypeInt(signed=False, label="UInt32")}, name="IKEEXT_CIPHER_ALGORITHM0", pack=False, align=None), "integrityAlgorithm": SimStruct({"algoIdentifier": SimTypeInt(signed=False, label="IKEEXT_INTEGRITY_TYPE")}, name="IKEEXT_INTEGRITY_ALGORITHM0", pack=False, align=None), "maxLifetimeSeconds": SimTypeInt(signed=False, label="UInt32"), "dhGroup": SimTypeInt(signed=False, label="IKEEXT_DH_GROUP"), "quickModeLimit": SimTypeInt(signed=False, label="UInt32")}, name="IKEEXT_PROPOSAL0", pack=False, align=None), offset=0), "flags": SimTypeInt(signed=False, label="IKEEXT_POLICY_FLAG"), "maxDynamicFilters": SimTypeInt(signed=False, label="UInt32"), "retransmitDurationSecs": SimTypeInt(signed=False, label="UInt32")}, name="IKEEXT_POLICY2", pack=False, align=None), offset=0), "authIpMmPolicy": SimTypePointer(SimStruct({"softExpirationTime": SimTypeInt(signed=False, label="UInt32"), "numAuthenticationMethods": SimTypeInt(signed=False, label="UInt32"), "authenticationMethods": SimTypePointer(SimStruct({"authenticationMethodType": SimTypeInt(signed=False, label="IKEEXT_AUTHENTICATION_METHOD_TYPE"), "Anonymous": SimUnion({"presharedKeyAuthentication": SimStruct({"presharedKey": SimStruct({"size": SimTypeInt(signed=False, label="UInt32"), "data": SimTypePointer(SimTypeChar(label="Byte"), offset=0)}, name="FWP_BYTE_BLOB", pack=False, align=None), "flags": SimTypeInt(signed=False, label="IKEEXT_PRESHARED_KEY_AUTHENTICATION_FLAGS")}, name="IKEEXT_PRESHARED_KEY_AUTHENTICATION1", pack=False, align=None), "certificateAuthentication": SimStruct({"inboundConfigType": SimTypeInt(signed=False, label="IKEEXT_CERT_CONFIG_TYPE"), "Anonymous1": SimUnion({"Anonymous1": SimStruct({"inboundRootArraySize": SimTypeInt(signed=False, label="UInt32"), "inboundRootCriteria": SimTypePointer(SimStruct({"certData": SimStruct({"size": SimTypeInt(signed=False, label="UInt32"), "data": SimTypePointer(SimTypeChar(label="Byte"), offset=0)}, name="FWP_BYTE_BLOB", pack=False, align=None), "certHash": SimStruct({"size": SimTypeInt(signed=False, label="UInt32"), "data": SimTypePointer(SimTypeChar(label="Byte"), offset=0)}, name="FWP_BYTE_BLOB", pack=False, align=None), "eku": SimTypePointer(SimStruct({"numEku": SimTypeInt(signed=False, label="UInt32"), "eku": SimTypePointer(SimTypePointer(SimTypeChar(label="Byte"), offset=0), offset=0)}, name="IKEEXT_CERT_EKUS0", pack=False, align=None), offset=0), "name": SimTypePointer(SimStruct({"nameType": SimTypeInt(signed=False, label="IKEEXT_CERT_CRITERIA_NAME_TYPE"), "certName": SimTypePointer(SimTypeChar(label="Char"), offset=0)}, name="IKEEXT_CERT_NAME0", pack=False, align=None), offset=0), "flags": SimTypeInt(signed=False, label="UInt32")}, name="IKEEXT_CERTIFICATE_CRITERIA0", pack=False, align=None), offset=0)}, name="_Anonymous1_e__Struct", pack=False, align=None), "Anonymous2": SimStruct({"inboundEnterpriseStoreArraySize": SimTypeInt(signed=False, label="UInt32"), "inboundEnterpriseStoreCriteria": SimTypePointer(SimStruct({"certData": SimStruct({"size": SimTypeInt(signed=False, label="UInt32"), "data": SimTypePointer(SimTypeChar(label="Byte"), offset=0)}, name="FWP_BYTE_BLOB", pack=False, align=None), "certHash": SimStruct({"size": SimTypeInt(signed=False, label="UInt32"), "data": SimTypePointer(SimTypeChar(label="Byte"), offset=0)}, name="FWP_BYTE_BLOB", pack=False, align=None), "eku": SimTypePointer(SimStruct({"numEku": SimTypeInt(signed=False, label="UInt32"), "eku": SimTypePointer(SimTypePointer(SimTypeChar(label="Byte"), offset=0), offset=0)}, name="IKEEXT_CERT_EKUS0", pack=False, align=None), offset=0), "name": SimTypePointer(SimStruct({"nameType": SimTypeInt(signed=False, label="IKEEXT_CERT_CRITERIA_NAME_TYPE"), "certName": SimTypePointer(SimTypeChar(label="Char"), offset=0)}, name="IKEEXT_CERT_NAME0", pack=False, align=None), offset=0), "flags": SimTypeInt(signed=False, label="UInt32")}, name="IKEEXT_CERTIFICATE_CRITERIA0", pack=False, align=None), offset=0)}, name="_Anonymous2_e__Struct", pack=False, align=None), "Anonymous3": SimStruct({"inboundRootStoreArraySize": SimTypeInt(signed=False, label="UInt32"), "inboundTrustedRootStoreCriteria": SimTypePointer(SimStruct({"certData": SimStruct({"size": SimTypeInt(signed=False, label="UInt32"), "data": SimTypePointer(SimTypeChar(label="Byte"), offset=0)}, name="FWP_BYTE_BLOB", pack=False, align=None), "certHash": SimStruct({"size": SimTypeInt(signed=False, label="UInt32"), "data": SimTypePointer(SimTypeChar(label="Byte"), offset=0)}, name="FWP_BYTE_BLOB", pack=False, align=None), "eku": SimTypePointer(SimStruct({"numEku": SimTypeInt(signed=False, label="UInt32"), "eku": SimTypePointer(SimTypePointer(SimTypeChar(label="Byte"), offset=0), offset=0)}, name="IKEEXT_CERT_EKUS0", pack=False, align=None), offset=0), "name": SimTypePointer(SimStruct({"nameType": SimTypeInt(signed=False, label="IKEEXT_CERT_CRITERIA_NAME_TYPE"), "certName": SimTypePointer(SimTypeChar(label="Char"), offset=0)}, name="IKEEXT_CERT_NAME0", pack=False, align=None), offset=0), "flags": SimTypeInt(signed=False, label="UInt32")}, name="IKEEXT_CERTIFICATE_CRITERIA0", pack=False, align=None), offset=0)}, name="_Anonymous3_e__Struct", pack=False, align=None)}, name="<anon>", label="None"), "outboundConfigType": SimTypeInt(signed=False, label="IKEEXT_CERT_CONFIG_TYPE"), "Anonymous2": SimUnion({"Anonymous1": SimStruct({"outboundRootArraySize": SimTypeInt(signed=False, label="UInt32"), "outboundRootCriteria": SimTypePointer(SimStruct({"certData": SimStruct({"size": SimTypeInt(signed=False, label="UInt32"), "data": SimTypePointer(SimTypeChar(label="Byte"), offset=0)}, name="FWP_BYTE_BLOB", pack=False, align=None), "certHash": SimStruct({"size": SimTypeInt(signed=False, label="UInt32"), "data": SimTypePointer(SimTypeChar(label="Byte"), offset=0)}, name="FWP_BYTE_BLOB", pack=False, align=None), "eku": SimTypePointer(SimStruct({"numEku": SimTypeInt(signed=False, label="UInt32"), "eku": SimTypePointer(SimTypePointer(SimTypeChar(label="Byte"), offset=0), offset=0)}, name="IKEEXT_CERT_EKUS0", pack=False, align=None), offset=0), "name": SimTypePointer(SimStruct({"nameType": SimTypeInt(signed=False, label="IKEEXT_CERT_CRITERIA_NAME_TYPE"), "certName": SimTypePointer(SimTypeChar(label="Char"), offset=0)}, name="IKEEXT_CERT_NAME0", pack=False, align=None), offset=0), "flags": SimTypeInt(signed=False, label="UInt32")}, name="IKEEXT_CERTIFICATE_CRITERIA0", pack=False, align=None), offset=0)}, name="_Anonymous1_e__Struct", pack=False, align=None), "Anonymous2": SimStruct({"outboundEnterpriseStoreArraySize": SimTypeInt(signed=False, label="UInt32"), "outboundEnterpriseStoreCriteria": SimTypePointer(SimStruct({"certData": SimStruct({"size": SimTypeInt(signed=False, label="UInt32"), "data": SimTypePointer(SimTypeChar(label="Byte"), offset=0)}, name="FWP_BYTE_BLOB", pack=False, align=None), "certHash": SimStruct({"size": SimTypeInt(signed=False, label="UInt32"), "data": SimTypePointer(SimTypeChar(label="Byte"), offset=0)}, name="FWP_BYTE_BLOB", pack=False, align=None), "eku": SimTypePointer(SimStruct({"numEku": SimTypeInt(signed=False, label="UInt32"), "eku": SimTypePointer(SimTypePointer(SimTypeChar(label="Byte"), offset=0), offset=0)}, name="IKEEXT_CERT_EKUS0", pack=False, align=None), offset=0), "name": SimTypePointer(SimStruct({"nameType": SimTypeInt(signed=False, label="IKEEXT_CERT_CRITERIA_NAME_TYPE"), "certName": SimTypePointer(SimTypeChar(label="Char"), offset=0)}, name="IKEEXT_CERT_NAME0", pack=False, align=None), offset=0), "flags": SimTypeInt(signed=False, label="UInt32")}, name="IKEEXT_CERTIFICATE_CRITERIA0", pack=False, align=None), offset=0)}, name="_Anonymous2_e__Struct", pack=False, align=None), "Anonymous3": SimStruct({"outboundRootStoreArraySize": SimTypeInt(signed=False, label="UInt32"), "outboundTrustedRootStoreCriteria": SimTypePointer(SimStruct({"certData": SimStruct({"size": SimTypeInt(signed=False, label="UInt32"), "data": SimTypePointer(SimTypeChar(label="Byte"), offset=0)}, name="FWP_BYTE_BLOB", pack=False, align=None), "certHash": SimStruct({"size": SimTypeInt(signed=False, label="UInt32"), "data": SimTypePointer(SimTypeChar(label="Byte"), offset=0)}, name="FWP_BYTE_BLOB", pack=False, align=None), "eku": SimTypePointer(SimStruct({"numEku": SimTypeInt(signed=False, label="UInt32"), "eku": SimTypePointer(SimTypePointer(SimTypeChar(label="Byte"), offset=0), offset=0)}, name="IKEEXT_CERT_EKUS0", pack=False, align=None), offset=0), "name": SimTypePointer(SimStruct({"nameType": SimTypeInt(signed=False, label="IKEEXT_CERT_CRITERIA_NAME_TYPE"), "certName": SimTypePointer(SimTypeChar(label="Char"), offset=0)}, name="IKEEXT_CERT_NAME0", pack=False, align=None), offset=0), "flags": SimTypeInt(signed=False, label="UInt32")}, name="IKEEXT_CERTIFICATE_CRITERIA0", pack=False, align=None), offset=0)}, name="_Anonymous3_e__Struct", pack=False, align=None)}, name="<anon>", label="None"), "flags": SimTypeInt(signed=False, label="IKEEXT_CERT_AUTH"), "localCertLocationUrl": SimStruct({"size": SimTypeInt(signed=False, label="UInt32"), "data": SimTypePointer(SimTypeChar(label="Byte"), offset=0)}, name="FWP_BYTE_BLOB", pack=False, align=None)}, name="IKEEXT_CERTIFICATE_AUTHENTICATION2", pack=False, align=None), "kerberosAuthentication": SimStruct({"flags": SimTypeInt(signed=False, label="IKEEXT_KERBEROS_AUTHENTICATION_FLAGS"), "proxyServer": SimTypePointer(SimTypeChar(label="Char"), offset=0)}, name="IKEEXT_KERBEROS_AUTHENTICATION1", pack=False, align=None), "reservedAuthentication": SimStruct({"flags": SimTypeInt(signed=False, label="IKEEXT_RESERVED_AUTHENTICATION_FLAGS")}, name="IKEEXT_RESERVED_AUTHENTICATION0", pack=False, align=None), "ntlmV2Authentication": SimStruct({"flags": SimTypeInt(signed=False, label="UInt32")}, name="IKEEXT_NTLM_V2_AUTHENTICATION0", pack=False, align=None), "sslAuthentication": SimStruct({"inboundConfigType": SimTypeInt(signed=False, label="IKEEXT_CERT_CONFIG_TYPE"), "Anonymous1": SimUnion({"Anonymous1": SimStruct({"inboundRootArraySize": SimTypeInt(signed=False, label="UInt32"), "inboundRootCriteria": SimTypePointer(SimStruct({"certData": SimStruct({"size": SimTypeInt(signed=False, label="UInt32"), "data": SimTypePointer(SimTypeChar(label="Byte"), offset=0)}, name="FWP_BYTE_BLOB", pack=False, align=None), "certHash": SimStruct({"size": SimTypeInt(signed=False, label="UInt32"), "data": SimTypePointer(SimTypeChar(label="Byte"), offset=0)}, name="FWP_BYTE_BLOB", pack=False, align=None), "eku": SimTypePointer(SimStruct({"numEku": SimTypeInt(signed=False, label="UInt32"), "eku": SimTypePointer(SimTypePointer(SimTypeChar(label="Byte"), offset=0), offset=0)}, name="IKEEXT_CERT_EKUS0", pack=False, align=None), offset=0), "name": SimTypePointer(SimStruct({"nameType": SimTypeInt(signed=False, label="IKEEXT_CERT_CRITERIA_NAME_TYPE"), "certName": SimTypePointer(SimTypeChar(label="Char"), offset=0)}, name="IKEEXT_CERT_NAME0", pack=False, align=None), offset=0), "flags": SimTypeInt(signed=False, label="UInt32")}, name="IKEEXT_CERTIFICATE_CRITERIA0", pack=False, align=None), offset=0)}, name="_Anonymous1_e__Struct", pack=False, align=None), "Anonymous2": SimStruct({"inboundEnterpriseStoreArraySize": SimTypeInt(signed=False, label="UInt32"), "inboundEnterpriseStoreCriteria": SimTypePointer(SimStruct({"certData": SimStruct({"size": SimTypeInt(signed=False, label="UInt32"), "data": SimTypePointer(SimTypeChar(label="Byte"), offset=0)}, name="FWP_BYTE_BLOB", pack=False, align=None), "certHash": SimStruct({"size": SimTypeInt(signed=False, label="UInt32"), "data": SimTypePointer(SimTypeChar(label="Byte"), offset=0)}, name="FWP_BYTE_BLOB", pack=False, align=None), "eku": SimTypePointer(SimStruct({"numEku": SimTypeInt(signed=False, label="UInt32"), "eku": SimTypePointer(SimTypePointer(SimTypeChar(label="Byte"), offset=0), offset=0)}, name="IKEEXT_CERT_EKUS0", pack=False, align=None), offset=0), "name": SimTypePointer(SimStruct({"nameType": SimTypeInt(signed=False, label="IKEEXT_CERT_CRITERIA_NAME_TYPE"), "certName": SimTypePointer(SimTypeChar(label="Char"), offset=0)}, name="IKEEXT_CERT_NAME0", pack=False, align=None), offset=0), "flags": SimTypeInt(signed=False, label="UInt32")}, name="IKEEXT_CERTIFICATE_CRITERIA0", pack=False, align=None), offset=0)}, name="_Anonymous2_e__Struct", pack=False, align=None), "Anonymous3": SimStruct({"inboundRootStoreArraySize": SimTypeInt(signed=False, label="UInt32"), "inboundTrustedRootStoreCriteria": SimTypePointer(SimStruct({"certData": SimStruct({"size": SimTypeInt(signed=False, label="UInt32"), "data": SimTypePointer(SimTypeChar(label="Byte"), offset=0)}, name="FWP_BYTE_BLOB", pack=False, align=None), "certHash": SimStruct({"size": SimTypeInt(signed=False, label="UInt32"), "data": SimTypePointer(SimTypeChar(label="Byte"), offset=0)}, name="FWP_BYTE_BLOB", pack=False, align=None), "eku": SimTypePointer(SimStruct({"numEku": SimTypeInt(signed=False, label="UInt32"), "eku": SimTypePointer(SimTypePointer(SimTypeChar(label="Byte"), offset=0), offset=0)}, name="IKEEXT_CERT_EKUS0", pack=False, align=None), offset=0), "name": SimTypePointer(SimStruct({"nameType": SimTypeInt(signed=False, label="IKEEXT_CERT_CRITERIA_NAME_TYPE"), "certName": SimTypePointer(SimTypeChar(label="Char"), offset=0)}, name="IKEEXT_CERT_NAME0", pack=False, align=None), offset=0), "flags": SimTypeInt(signed=False, label="UInt32")}, name="IKEEXT_CERTIFICATE_CRITERIA0", pack=False, align=None), offset=0)}, name="_Anonymous3_e__Struct", pack=False, align=None)}, name="<anon>", label="None"), "outboundConfigType": SimTypeInt(signed=False, label="IKEEXT_CERT_CONFIG_TYPE"), "Anonymous2": SimUnion({"Anonymous1": SimStruct({"outboundRootArraySize": SimTypeInt(signed=False, label="UInt32"), "outboundRootCriteria": SimTypePointer(SimStruct({"certData": SimStruct({"size": SimTypeInt(signed=False, label="UInt32"), "data": SimTypePointer(SimTypeChar(label="Byte"), offset=0)}, name="FWP_BYTE_BLOB", pack=False, align=None), "certHash": SimStruct({"size": SimTypeInt(signed=False, label="UInt32"), "data": SimTypePointer(SimTypeChar(label="Byte"), offset=0)}, name="FWP_BYTE_BLOB", pack=False, align=None), "eku": SimTypePointer(SimStruct({"numEku": SimTypeInt(signed=False, label="UInt32"), "eku": SimTypePointer(SimTypePointer(SimTypeChar(label="Byte"), offset=0), offset=0)}, name="IKEEXT_CERT_EKUS0", pack=False, align=None), offset=0), "name": SimTypePointer(SimStruct({"nameType": SimTypeInt(signed=False, label="IKEEXT_CERT_CRITERIA_NAME_TYPE"), "certName": SimTypePointer(SimTypeChar(label="Char"), offset=0)}, name="IKEEXT_CERT_NAME0", pack=False, align=None), offset=0), "flags": SimTypeInt(signed=False, label="UInt32")}, name="IKEEXT_CERTIFICATE_CRITERIA0", pack=False, align=None), offset=0)}, name="_Anonymous1_e__Struct", pack=False, align=None), "Anonymous2": SimStruct({"outboundEnterpriseStoreArraySize": SimTypeInt(signed=False, label="UInt32"), "outboundEnterpriseStoreCriteria": SimTypePointer(SimStruct({"certData": SimStruct({"size": SimTypeInt(signed=False, label="UInt32"), "data": SimTypePointer(SimTypeChar(label="Byte"), offset=0)}, name="FWP_BYTE_BLOB", pack=False, align=None), "certHash": SimStruct({"size": SimTypeInt(signed=False, label="UInt32"), "data": SimTypePointer(SimTypeChar(label="Byte"), offset=0)}, name="FWP_BYTE_BLOB", pack=False, align=None), "eku": SimTypePointer(SimStruct({"numEku": SimTypeInt(signed=False, label="UInt32"), "eku": SimTypePointer(SimTypePointer(SimTypeChar(label="Byte"), offset=0), offset=0)}, name="IKEEXT_CERT_EKUS0", pack=False, align=None), offset=0), "name": SimTypePointer(SimStruct({"nameType": SimTypeInt(signed=False, label="IKEEXT_CERT_CRITERIA_NAME_TYPE"), "certName": SimTypePointer(SimTypeChar(label="Char"), offset=0)}, name="IKEEXT_CERT_NAME0", pack=False, align=None), offset=0), "flags": SimTypeInt(signed=False, label="UInt32")}, name="IKEEXT_CERTIFICATE_CRITERIA0", pack=False, align=None), offset=0)}, name="_Anonymous2_e__Struct", pack=False, align=None), "Anonymous3": SimStruct({"outboundRootStoreArraySize": SimTypeInt(signed=False, label="UInt32"), "outboundTrustedRootStoreCriteria": SimTypePointer(SimStruct({"certData": SimStruct({"size": SimTypeInt(signed=False, label="UInt32"), "data": SimTypePointer(SimTypeChar(label="Byte"), offset=0)}, name="FWP_BYTE_BLOB", pack=False, align=None), "certHash": SimStruct({"size": SimTypeInt(signed=False, label="UInt32"), "data": SimTypePointer(SimTypeChar(label="Byte"), offset=0)}, name="FWP_BYTE_BLOB", pack=False, align=None), "eku": SimTypePointer(SimStruct({"numEku": SimTypeInt(signed=False, label="UInt32"), "eku": SimTypePointer(SimTypePointer(SimTypeChar(label="Byte"), offset=0), offset=0)}, name="IKEEXT_CERT_EKUS0", pack=False, align=None), offset=0), "name": SimTypePointer(SimStruct({"nameType": SimTypeInt(signed=False, label="IKEEXT_CERT_CRITERIA_NAME_TYPE"), "certName": SimTypePointer(SimTypeChar(label="Char"), offset=0)}, name="IKEEXT_CERT_NAME0", pack=False, align=None), offset=0), "flags": SimTypeInt(signed=False, label="UInt32")}, name="IKEEXT_CERTIFICATE_CRITERIA0", pack=False, align=None), offset=0)}, name="_Anonymous3_e__Struct", pack=False, align=None)}, name="<anon>", label="None"), "flags": SimTypeInt(signed=False, label="IKEEXT_CERT_AUTH"), "localCertLocationUrl": SimStruct({"size": SimTypeInt(signed=False, label="UInt32"), "data": SimTypePointer(SimTypeChar(label="Byte"), offset=0)}, name="FWP_BYTE_BLOB", pack=False, align=None)}, name="IKEEXT_CERTIFICATE_AUTHENTICATION2", pack=False, align=None), "cgaAuthentication": SimStruct({"keyContainerName": SimTypePointer(SimTypeChar(label="Char"), offset=0), "cspName": SimTypePointer(SimTypeChar(label="Char"), offset=0), "cspType": SimTypeInt(signed=False, label="UInt32"), "cgaModifier": SimStruct({"byteArray16": SimTypeFixedSizeArray(SimTypeChar(label="Byte"), 16)}, name="FWP_BYTE_ARRAY16", pack=False, align=None), "cgaCollisionCount": SimTypeChar(label="Byte")}, name="IKEEXT_IPV6_CGA_AUTHENTICATION0", pack=False, align=None), "eapAuthentication": SimStruct({"flags": SimTypeInt(signed=False, label="IKEEXT_EAP_AUTHENTICATION_FLAGS")}, name="IKEEXT_EAP_AUTHENTICATION0", pack=False, align=None)}, name="<anon>", label="None")}, name="IKEEXT_AUTHENTICATION_METHOD2", pack=False, align=None), offset=0), "initiatorImpersonationType": SimTypeInt(signed=False, label="IKEEXT_AUTHENTICATION_IMPERSONATION_TYPE"), "numIkeProposals": SimTypeInt(signed=False, label="UInt32"), "ikeProposals": SimTypePointer(SimStruct({"cipherAlgorithm": SimStruct({"algoIdentifier": SimTypeInt(signed=False, label="IKEEXT_CIPHER_TYPE"), "keyLen": SimTypeInt(signed=False, label="UInt32"), "rounds": SimTypeInt(signed=False, label="UInt32")}, name="IKEEXT_CIPHER_ALGORITHM0", pack=False, align=None), "integrityAlgorithm": SimStruct({"algoIdentifier": SimTypeInt(signed=False, label="IKEEXT_INTEGRITY_TYPE")}, name="IKEEXT_INTEGRITY_ALGORITHM0", pack=False, align=None), "maxLifetimeSeconds": SimTypeInt(signed=False, label="UInt32"), "dhGroup": SimTypeInt(signed=False, label="IKEEXT_DH_GROUP"), "quickModeLimit": SimTypeInt(signed=False, label="UInt32")}, name="IKEEXT_PROPOSAL0", pack=False, align=None), offset=0), "flags": SimTypeInt(signed=False, label="IKEEXT_POLICY_FLAG"), "maxDynamicFilters": SimTypeInt(signed=False, label="UInt32"), "retransmitDurationSecs": SimTypeInt(signed=False, label="UInt32")}, name="IKEEXT_POLICY2", pack=False, align=None), offset=0), "dataBuffer": SimTypePointer(SimStruct({"size": SimTypeInt(signed=False, label="UInt32"), "data": SimTypePointer(SimTypeChar(label="Byte"), offset=0)}, name="FWP_BYTE_BLOB", pack=False, align=None), offset=0), "classifyOptions": SimTypePointer(SimStruct({"numOptions": SimTypeInt(signed=False, label="UInt32"), "options": SimTypePointer(SimStruct({"type": SimTypeInt(signed=False, label="FWP_CLASSIFY_OPTION_TYPE"), "value": SimStruct({"type": SimTypeInt(signed=False, label="FWP_DATA_TYPE"), "Anonymous": SimUnion({"uint8": SimTypeChar(label="Byte"), "uint16": SimTypeShort(signed=False, label="UInt16"), "uint32": SimTypeInt(signed=False, label="UInt32"), "uint64": SimTypePointer(SimTypeLongLong(signed=False, label="UInt64"), offset=0), "int8": SimTypeChar(label="SByte"), "int16": SimTypeShort(signed=True, label="Int16"), "int32": SimTypeInt(signed=True, label="Int32"), "int64": SimTypePointer(SimTypeLongLong(signed=True, label="Int64"), offset=0), "float32": SimTypeFloat(size=32), "double64": SimTypePointer(SimTypeFloat(size=64), offset=0), "byteArray16": SimTypePointer(SimStruct({"byteArray16": SimTypeFixedSizeArray(SimTypeChar(label="Byte"), 16)}, name="FWP_BYTE_ARRAY16", pack=False, align=None), offset=0), "byteBlob": SimTypePointer(SimStruct({"size": SimTypeInt(signed=False, label="UInt32"), "data": SimTypePointer(SimTypeChar(label="Byte"), offset=0)}, name="FWP_BYTE_BLOB", pack=False, align=None), offset=0), "sid": SimTypePointer(SimTypeBottom(label="SID"), offset=0), "sd": SimTypePointer(SimStruct({"size": SimTypeInt(signed=False, label="UInt32"), "data": SimTypePointer(SimTypeChar(label="Byte"), offset=0)}, name="FWP_BYTE_BLOB", pack=False, align=None), offset=0), "tokenInformation": SimTypePointer(SimStruct({"sidCount": SimTypeInt(signed=False, label="UInt32"), "sids": SimTypePointer(SimTypeBottom(label="SID_AND_ATTRIBUTES"), offset=0), "restrictedSidCount": SimTypeInt(signed=False, label="UInt32"), "restrictedSids": SimTypePointer(SimTypeBottom(label="SID_AND_ATTRIBUTES"), offset=0)}, name="FWP_TOKEN_INFORMATION", pack=False, align=None), offset=0), "tokenAccessInformation": SimTypePointer(SimStruct({"size": SimTypeInt(signed=False, label="UInt32"), "data": SimTypePointer(SimTypeChar(label="Byte"), offset=0)}, name="FWP_BYTE_BLOB", pack=False, align=None), offset=0), "unicodeString": SimTypePointer(SimTypeChar(label="Char"), offset=0), "byteArray6": SimTypePointer(SimStruct({"byteArray6": SimTypeFixedSizeArray(SimTypeChar(label="Byte"), 6)}, name="FWP_BYTE_ARRAY6", pack=False, align=None), offset=0), "bitmapArray64": SimTypePointer(SimStruct({"bitmapArray64": SimTypeFixedSizeArray(SimTypeChar(label="Byte"), 8)}, name="FWP_BITMAP_ARRAY64_", pack=False, align=None), offset=0)}, name="<anon>", label="None")}, name="FWP_VALUE0", pack=False, align=None)}, name="FWPM_CLASSIFY_OPTION0", pack=False, align=None), offset=0)}, name="FWPM_CLASSIFY_OPTIONS0", pack=False, align=None), offset=0), "ikeV2QmTunnelPolicy": SimTypePointer(SimStruct({"flags": SimTypeInt(signed=False, label="UInt32"), "numIpsecProposals": SimTypeInt(signed=False, label="UInt32"), "ipsecProposals": SimTypePointer(SimStruct({"lifetime": SimStruct({"lifetimeSeconds": SimTypeInt(signed=False, label="UInt32"), "lifetimeKilobytes": SimTypeInt(signed=False, label="UInt32"), "lifetimePackets": SimTypeInt(signed=False, label="UInt32")}, name="IPSEC_SA_LIFETIME0", pack=False, align=None), "numSaTransforms": SimTypeInt(signed=False, label="UInt32"), "saTransforms": SimTypePointer(SimStruct({"ipsecTransformType": SimTypeInt(signed=False, label="IPSEC_TRANSFORM_TYPE"), "Anonymous": SimUnion({"ahTransform": SimTypePointer(SimStruct({"authTransformId": SimStruct({"authType": SimTypeInt(signed=False, label="IPSEC_AUTH_TYPE"), "authConfig": SimTypeChar(label="Byte")}, name="IPSEC_AUTH_TRANSFORM_ID0", pack=False, align=None), "cryptoModuleId": SimTypePointer(SimTypeBottom(label="Guid"), offset=0)}, name="IPSEC_AUTH_TRANSFORM0", pack=False, align=None), offset=0), "espAuthTransform": SimTypePointer(SimStruct({"authTransformId": SimStruct({"authType": SimTypeInt(signed=False, label="IPSEC_AUTH_TYPE"), "authConfig": SimTypeChar(label="Byte")}, name="IPSEC_AUTH_TRANSFORM_ID0", pack=False, align=None), "cryptoModuleId": SimTypePointer(SimTypeBottom(label="Guid"), offset=0)}, name="IPSEC_AUTH_TRANSFORM0", pack=False, align=None), offset=0), "espCipherTransform": SimTypePointer(SimStruct({"cipherTransformId": SimStruct({"cipherType": SimTypeInt(signed=False, label="IPSEC_CIPHER_TYPE"), "cipherConfig": SimTypeChar(label="Byte")}, name="IPSEC_CIPHER_TRANSFORM_ID0", pack=False, align=None), "cryptoModuleId": SimTypePointer(SimTypeBottom(label="Guid"), offset=0)}, name="IPSEC_CIPHER_TRANSFORM0", pack=False, align=None), offset=0), "espAuthAndCipherTransform": SimTypePointer(SimStruct({"authTransform": SimStruct({"authTransformId": SimStruct({"authType": SimTypeInt(signed=False, label="IPSEC_AUTH_TYPE"), "authConfig": SimTypeChar(label="Byte")}, name="IPSEC_AUTH_TRANSFORM_ID0", pack=False, align=None), "cryptoModuleId": SimTypePointer(SimTypeBottom(label="Guid"), offset=0)}, name="IPSEC_AUTH_TRANSFORM0", pack=False, align=None), "cipherTransform": SimStruct({"cipherTransformId": SimStruct({"cipherType": SimTypeInt(signed=False, label="IPSEC_CIPHER_TYPE"), "cipherConfig": SimTypeChar(label="Byte")}, name="IPSEC_CIPHER_TRANSFORM_ID0", pack=False, align=None), "cryptoModuleId": SimTypePointer(SimTypeBottom(label="Guid"), offset=0)}, name="IPSEC_CIPHER_TRANSFORM0", pack=False, align=None)}, name="IPSEC_AUTH_AND_CIPHER_TRANSFORM0", pack=False, align=None), offset=0), "espAuthFwTransform": SimTypePointer(SimStruct({"authTransformId": SimStruct({"authType": SimTypeInt(signed=False, label="IPSEC_AUTH_TYPE"), "authConfig": SimTypeChar(label="Byte")}, name="IPSEC_AUTH_TRANSFORM_ID0", pack=False, align=None), "cryptoModuleId": SimTypePointer(SimTypeBottom(label="Guid"), offset=0)}, name="IPSEC_AUTH_TRANSFORM0", pack=False, align=None), offset=0)}, name="<anon>", label="None")}, name="IPSEC_SA_TRANSFORM0", pack=False, align=None), offset=0), "pfsGroup": SimTypeInt(signed=False, label="IPSEC_PFS_GROUP")}, name="IPSEC_PROPOSAL0", pack=False, align=None), offset=0), "tunnelEndpoints": SimStruct({"ipVersion": SimTypeInt(signed=False, label="FWP_IP_VERSION"), "Anonymous1": SimUnion({"localV4Address": SimTypeInt(signed=False, label="UInt32"), "localV6Address": SimTypeFixedSizeArray(SimTypeChar(label="Byte"), 16)}, name="<anon>", label="None"), "Anonymous2": SimUnion({"remoteV4Address": SimTypeInt(signed=False, label="UInt32"), "remoteV6Address": SimTypeFixedSizeArray(SimTypeChar(label="Byte"), 16)}, name="<anon>", label="None"), "localIfLuid": SimTypeLongLong(signed=False, label="UInt64"), "remoteFqdn": SimTypePointer(SimTypeChar(label="Char"), offset=0), "numAddresses": SimTypeInt(signed=False, label="UInt32"), "remoteAddresses": SimTypePointer(SimStruct({"ipVersion": SimTypeInt(signed=False, label="FWP_IP_VERSION"), "Anonymous": SimUnion({"v4Address": SimTypeInt(signed=False, label="UInt32"), "v6Address": SimTypeFixedSizeArray(SimTypeChar(label="Byte"), 16)}, name="<anon>", label="None")}, name="IPSEC_TUNNEL_ENDPOINT0", pack=False, align=None), offset=0)}, name="IPSEC_TUNNEL_ENDPOINTS2", pack=False, align=None), "saIdleTimeout": SimStruct({"idleTimeoutSeconds": SimTypeInt(signed=False, label="UInt32"), "idleTimeoutSecondsFailOver": SimTypeInt(signed=False, label="UInt32")}, name="IPSEC_SA_IDLE_TIMEOUT0", pack=False, align=None), "emPolicy": SimTypePointer(SimStruct({"numAuthenticationMethods": SimTypeInt(signed=False, label="UInt32"), "authenticationMethods": SimTypePointer(SimStruct({"authenticationMethodType": SimTypeInt(signed=False, label="IKEEXT_AUTHENTICATION_METHOD_TYPE"), "Anonymous": SimUnion({"presharedKeyAuthentication": SimStruct({"presharedKey": SimStruct({"size": SimTypeInt(signed=False, label="UInt32"), "data": SimTypePointer(SimTypeChar(label="Byte"), offset=0)}, name="FWP_BYTE_BLOB", pack=False, align=None), "flags": SimTypeInt(signed=False, label="IKEEXT_PRESHARED_KEY_AUTHENTICATION_FLAGS")}, name="IKEEXT_PRESHARED_KEY_AUTHENTICATION1", pack=False, align=None), "certificateAuthentication": SimStruct({"inboundConfigType": SimTypeInt(signed=False, label="IKEEXT_CERT_CONFIG_TYPE"), "Anonymous1": SimUnion({"Anonymous1": SimStruct({"inboundRootArraySize": SimTypeInt(signed=False, label="UInt32"), "inboundRootCriteria": SimTypePointer(SimStruct({"certData": SimStruct({"size": SimTypeInt(signed=False, label="UInt32"), "data": SimTypePointer(SimTypeChar(label="Byte"), offset=0)}, name="FWP_BYTE_BLOB", pack=False, align=None), "certHash": SimStruct({"size": SimTypeInt(signed=False, label="UInt32"), "data": SimTypePointer(SimTypeChar(label="Byte"), offset=0)}, name="FWP_BYTE_BLOB", pack=False, align=None), "eku": SimTypePointer(SimStruct({"numEku": SimTypeInt(signed=False, label="UInt32"), "eku": SimTypePointer(SimTypePointer(SimTypeChar(label="Byte"), offset=0), offset=0)}, name="IKEEXT_CERT_EKUS0", pack=False, align=None), offset=0), "name": SimTypePointer(SimStruct({"nameType": SimTypeInt(signed=False, label="IKEEXT_CERT_CRITERIA_NAME_TYPE"), "certName": SimTypePointer(SimTypeChar(label="Char"), offset=0)}, name="IKEEXT_CERT_NAME0", pack=False, align=None), offset=0), "flags": SimTypeInt(signed=False, label="UInt32")}, name="IKEEXT_CERTIFICATE_CRITERIA0", pack=False, align=None), offset=0)}, name="_Anonymous1_e__Struct", pack=False, align=None), "Anonymous2": SimStruct({"inboundEnterpriseStoreArraySize": SimTypeInt(signed=False, label="UInt32"), "inboundEnterpriseStoreCriteria": SimTypePointer(SimStruct({"certData": SimStruct({"size": SimTypeInt(signed=False, label="UInt32"), "data": SimTypePointer(SimTypeChar(label="Byte"), offset=0)}, name="FWP_BYTE_BLOB", pack=False, align=None), "certHash": SimStruct({"size": SimTypeInt(signed=False, label="UInt32"), "data": SimTypePointer(SimTypeChar(label="Byte"), offset=0)}, name="FWP_BYTE_BLOB", pack=False, align=None), "eku": SimTypePointer(SimStruct({"numEku": SimTypeInt(signed=False, label="UInt32"), "eku": SimTypePointer(SimTypePointer(SimTypeChar(label="Byte"), offset=0), offset=0)}, name="IKEEXT_CERT_EKUS0", pack=False, align=None), offset=0), "name": SimTypePointer(SimStruct({"nameType": SimTypeInt(signed=False, label="IKEEXT_CERT_CRITERIA_NAME_TYPE"), "certName": SimTypePointer(SimTypeChar(label="Char"), offset=0)}, name="IKEEXT_CERT_NAME0", pack=False, align=None), offset=0), "flags": SimTypeInt(signed=False, label="UInt32")}, name="IKEEXT_CERTIFICATE_CRITERIA0", pack=False, align=None), offset=0)}, name="_Anonymous2_e__Struct", pack=False, align=None), "Anonymous3": SimStruct({"inboundRootStoreArraySize": SimTypeInt(signed=False, label="UInt32"), "inboundTrustedRootStoreCriteria": SimTypePointer(SimStruct({"certData": SimStruct({"size": SimTypeInt(signed=False, label="UInt32"), "data": SimTypePointer(SimTypeChar(label="Byte"), offset=0)}, name="FWP_BYTE_BLOB", pack=False, align=None), "certHash": SimStruct({"size": SimTypeInt(signed=False, label="UInt32"), "data": SimTypePointer(SimTypeChar(label="Byte"), offset=0)}, name="FWP_BYTE_BLOB", pack=False, align=None), "eku": SimTypePointer(SimStruct({"numEku": SimTypeInt(signed=False, label="UInt32"), "eku": SimTypePointer(SimTypePointer(SimTypeChar(label="Byte"), offset=0), offset=0)}, name="IKEEXT_CERT_EKUS0", pack=False, align=None), offset=0), "name": SimTypePointer(SimStruct({"nameType": SimTypeInt(signed=False, label="IKEEXT_CERT_CRITERIA_NAME_TYPE"), "certName": SimTypePointer(SimTypeChar(label="Char"), offset=0)}, name="IKEEXT_CERT_NAME0", pack=False, align=None), offset=0), "flags": SimTypeInt(signed=False, label="UInt32")}, name="IKEEXT_CERTIFICATE_CRITERIA0", pack=False, align=None), offset=0)}, name="_Anonymous3_e__Struct", pack=False, align=None)}, name="<anon>", label="None"), "outboundConfigType": SimTypeInt(signed=False, label="IKEEXT_CERT_CONFIG_TYPE"), "Anonymous2": SimUnion({"Anonymous1": SimStruct({"outboundRootArraySize": SimTypeInt(signed=False, label="UInt32"), "outboundRootCriteria": SimTypePointer(SimStruct({"certData": SimStruct({"size": SimTypeInt(signed=False, label="UInt32"), "data": SimTypePointer(SimTypeChar(label="Byte"), offset=0)}, name="FWP_BYTE_BLOB", pack=False, align=None), "certHash": SimStruct({"size": SimTypeInt(signed=False, label="UInt32"), "data": SimTypePointer(SimTypeChar(label="Byte"), offset=0)}, name="FWP_BYTE_BLOB", pack=False, align=None), "eku": SimTypePointer(SimStruct({"numEku": SimTypeInt(signed=False, label="UInt32"), "eku": SimTypePointer(SimTypePointer(SimTypeChar(label="Byte"), offset=0), offset=0)}, name="IKEEXT_CERT_EKUS0", pack=False, align=None), offset=0), "name": SimTypePointer(SimStruct({"nameType": SimTypeInt(signed=False, label="IKEEXT_CERT_CRITERIA_NAME_TYPE"), "certName": SimTypePointer(SimTypeChar(label="Char"), offset=0)}, name="IKEEXT_CERT_NAME0", pack=False, align=None), offset=0), "flags": SimTypeInt(signed=False, label="UInt32")}, name="IKEEXT_CERTIFICATE_CRITERIA0", pack=False, align=None), offset=0)}, name="_Anonymous1_e__Struct", pack=False, align=None), "Anonymous2": SimStruct({"outboundEnterpriseStoreArraySize": SimTypeInt(signed=False, label="UInt32"), "outboundEnterpriseStoreCriteria": SimTypePointer(SimStruct({"certData": SimStruct({"size": SimTypeInt(signed=False, label="UInt32"), "data": SimTypePointer(SimTypeChar(label="Byte"), offset=0)}, name="FWP_BYTE_BLOB", pack=False, align=None), "certHash": SimStruct({"size": SimTypeInt(signed=False, label="UInt32"), "data": SimTypePointer(SimTypeChar(label="Byte"), offset=0)}, name="FWP_BYTE_BLOB", pack=False, align=None), "eku": SimTypePointer(SimStruct({"numEku": SimTypeInt(signed=False, label="UInt32"), "eku": SimTypePointer(SimTypePointer(SimTypeChar(label="Byte"), offset=0), offset=0)}, name="IKEEXT_CERT_EKUS0", pack=False, align=None), offset=0), "name": SimTypePointer(SimStruct({"nameType": SimTypeInt(signed=False, label="IKEEXT_CERT_CRITERIA_NAME_TYPE"), "certName": SimTypePointer(SimTypeChar(label="Char"), offset=0)}, name="IKEEXT_CERT_NAME0", pack=False, align=None), offset=0), "flags": SimTypeInt(signed=False, label="UInt32")}, name="IKEEXT_CERTIFICATE_CRITERIA0", pack=False, align=None), offset=0)}, name="_Anonymous2_e__Struct", pack=False, align=None), "Anonymous3": SimStruct({"outboundRootStoreArraySize": SimTypeInt(signed=False, label="UInt32"), "outboundTrustedRootStoreCriteria": SimTypePointer(SimStruct({"certData": SimStruct({"size": SimTypeInt(signed=False, label="UInt32"), "data": SimTypePointer(SimTypeChar(label="Byte"), offset=0)}, name="FWP_BYTE_BLOB", pack=False, align=None), "certHash": SimStruct({"size": SimTypeInt(signed=False, label="UInt32"), "data": SimTypePointer(SimTypeChar(label="Byte"), offset=0)}, name="FWP_BYTE_BLOB", pack=False, align=None), "eku": SimTypePointer(SimStruct({"numEku": SimTypeInt(signed=False, label="UInt32"), "eku": SimTypePointer(SimTypePointer(SimTypeChar(label="Byte"), offset=0), offset=0)}, name="IKEEXT_CERT_EKUS0", pack=False, align=None), offset=0), "name": SimTypePointer(SimStruct({"nameType": SimTypeInt(signed=False, label="IKEEXT_CERT_CRITERIA_NAME_TYPE"), "certName": SimTypePointer(SimTypeChar(label="Char"), offset=0)}, name="IKEEXT_CERT_NAME0", pack=False, align=None), offset=0), "flags": SimTypeInt(signed=False, label="UInt32")}, name="IKEEXT_CERTIFICATE_CRITERIA0", pack=False, align=None), offset=0)}, name="_Anonymous3_e__Struct", pack=False, align=None)}, name="<anon>", label="None"), "flags": SimTypeInt(signed=False, label="IKEEXT_CERT_AUTH"), "localCertLocationUrl": SimStruct({"size": SimTypeInt(signed=False, label="UInt32"), "data": SimTypePointer(SimTypeChar(label="Byte"), offset=0)}, name="FWP_BYTE_BLOB", pack=False, align=None)}, name="IKEEXT_CERTIFICATE_AUTHENTICATION2", pack=False, align=None), "kerberosAuthentication": SimStruct({"flags": SimTypeInt(signed=False, label="IKEEXT_KERBEROS_AUTHENTICATION_FLAGS"), "proxyServer": SimTypePointer(SimTypeChar(label="Char"), offset=0)}, name="IKEEXT_KERBEROS_AUTHENTICATION1", pack=False, align=None), "reservedAuthentication": SimStruct({"flags": SimTypeInt(signed=False, label="IKEEXT_RESERVED_AUTHENTICATION_FLAGS")}, name="IKEEXT_RESERVED_AUTHENTICATION0", pack=False, align=None), "ntlmV2Authentication": SimStruct({"flags": SimTypeInt(signed=False, label="UInt32")}, name="IKEEXT_NTLM_V2_AUTHENTICATION0", pack=False, align=None), "sslAuthentication": SimStruct({"inboundConfigType": SimTypeInt(signed=False, label="IKEEXT_CERT_CONFIG_TYPE"), "Anonymous1": SimUnion({"Anonymous1": SimStruct({"inboundRootArraySize": SimTypeInt(signed=False, label="UInt32"), "inboundRootCriteria": SimTypePointer(SimStruct({"certData": SimStruct({"size": SimTypeInt(signed=False, label="UInt32"), "data": SimTypePointer(SimTypeChar(label="Byte"), offset=0)}, name="FWP_BYTE_BLOB", pack=False, align=None), "certHash": SimStruct({"size": SimTypeInt(signed=False, label="UInt32"), "data": SimTypePointer(SimTypeChar(label="Byte"), offset=0)}, name="FWP_BYTE_BLOB", pack=False, align=None), "eku": SimTypePointer(SimStruct({"numEku": SimTypeInt(signed=False, label="UInt32"), "eku": SimTypePointer(SimTypePointer(SimTypeChar(label="Byte"), offset=0), offset=0)}, name="IKEEXT_CERT_EKUS0", pack=False, align=None), offset=0), "name": SimTypePointer(SimStruct({"nameType": SimTypeInt(signed=False, label="IKEEXT_CERT_CRITERIA_NAME_TYPE"), "certName": SimTypePointer(SimTypeChar(label="Char"), offset=0)}, name="IKEEXT_CERT_NAME0", pack=False, align=None), offset=0), "flags": SimTypeInt(signed=False, label="UInt32")}, name="IKEEXT_CERTIFICATE_CRITERIA0", pack=False, align=None), offset=0)}, name="_Anonymous1_e__Struct", pack=False, align=None), "Anonymous2": SimStruct({"inboundEnterpriseStoreArraySize": SimTypeInt(signed=False, label="UInt32"), "inboundEnterpriseStoreCriteria": SimTypePointer(SimStruct({"certData": SimStruct({"size": SimTypeInt(signed=False, label="UInt32"), "data": SimTypePointer(SimTypeChar(label="Byte"), offset=0)}, name="FWP_BYTE_BLOB", pack=False, align=None), "certHash": SimStruct({"size": SimTypeInt(signed=False, label="UInt32"), "data": SimTypePointer(SimTypeChar(label="Byte"), offset=0)}, name="FWP_BYTE_BLOB", pack=False, align=None), "eku": SimTypePointer(SimStruct({"numEku": SimTypeInt(signed=False, label="UInt32"), "eku": SimTypePointer(SimTypePointer(SimTypeChar(label="Byte"), offset=0), offset=0)}, name="IKEEXT_CERT_EKUS0", pack=False, align=None), offset=0), "name": SimTypePointer(SimStruct({"nameType": SimTypeInt(signed=False, label="IKEEXT_CERT_CRITERIA_NAME_TYPE"), "certName": SimTypePointer(SimTypeChar(label="Char"), offset=0)}, name="IKEEXT_CERT_NAME0", pack=False, align=None), offset=0), "flags": SimTypeInt(signed=False, label="UInt32")}, name="IKEEXT_CERTIFICATE_CRITERIA0", pack=False, align=None), offset=0)}, name="_Anonymous2_e__Struct", pack=False, align=None), "Anonymous3": SimStruct({"inboundRootStoreArraySize": SimTypeInt(signed=False, label="UInt32"), "inboundTrustedRootStoreCriteria": SimTypePointer(SimStruct({"certData": SimStruct({"size": SimTypeInt(signed=False, label="UInt32"), "data": SimTypePointer(SimTypeChar(label="Byte"), offset=0)}, name="FWP_BYTE_BLOB", pack=False, align=None), "certHash": SimStruct({"size": SimTypeInt(signed=False, label="UInt32"), "data": SimTypePointer(SimTypeChar(label="Byte"), offset=0)}, name="FWP_BYTE_BLOB", pack=False, align=None), "eku": SimTypePointer(SimStruct({"numEku": SimTypeInt(signed=False, label="UInt32"), "eku": SimTypePointer(SimTypePointer(SimTypeChar(label="Byte"), offset=0), offset=0)}, name="IKEEXT_CERT_EKUS0", pack=False, align=None), offset=0), "name": SimTypePointer(SimStruct({"nameType": SimTypeInt(signed=False, label="IKEEXT_CERT_CRITERIA_NAME_TYPE"), "certName": SimTypePointer(SimTypeChar(label="Char"), offset=0)}, name="IKEEXT_CERT_NAME0", pack=False, align=None), offset=0), "flags": SimTypeInt(signed=False, label="UInt32")}, name="IKEEXT_CERTIFICATE_CRITERIA0", pack=False, align=None), offset=0)}, name="_Anonymous3_e__Struct", pack=False, align=None)}, name="<anon>", label="None"), "outboundConfigType": SimTypeInt(signed=False, label="IKEEXT_CERT_CONFIG_TYPE"), "Anonymous2": SimUnion({"Anonymous1": SimStruct({"outboundRootArraySize": SimTypeInt(signed=False, label="UInt32"), "outboundRootCriteria": SimTypePointer(SimStruct({"certData": SimStruct({"size": SimTypeInt(signed=False, label="UInt32"), "data": SimTypePointer(SimTypeChar(label="Byte"), offset=0)}, name="FWP_BYTE_BLOB", pack=False, align=None), "certHash": SimStruct({"size": SimTypeInt(signed=False, label="UInt32"), "data": SimTypePointer(SimTypeChar(label="Byte"), offset=0)}, name="FWP_BYTE_BLOB", pack=False, align=None), "eku": SimTypePointer(SimStruct({"numEku": SimTypeInt(signed=False, label="UInt32"), "eku": SimTypePointer(SimTypePointer(SimTypeChar(label="Byte"), offset=0), offset=0)}, name="IKEEXT_CERT_EKUS0", pack=False, align=None), offset=0), "name": SimTypePointer(SimStruct({"nameType": SimTypeInt(signed=False, label="IKEEXT_CERT_CRITERIA_NAME_TYPE"), "certName": SimTypePointer(SimTypeChar(label="Char"), offset=0)}, name="IKEEXT_CERT_NAME0", pack=False, align=None), offset=0), "flags": SimTypeInt(signed=False, label="UInt32")}, name="IKEEXT_CERTIFICATE_CRITERIA0", pack=False, align=None), offset=0)}, name="_Anonymous1_e__Struct", pack=False, align=None), "Anonymous2": SimStruct({"outboundEnterpriseStoreArraySize": SimTypeInt(signed=False, label="UInt32"), "outboundEnterpriseStoreCriteria": SimTypePointer(SimStruct({"certData": SimStruct({"size": SimTypeInt(signed=False, label="UInt32"), "data": SimTypePointer(SimTypeChar(label="Byte"), offset=0)}, name="FWP_BYTE_BLOB", pack=False, align=None), "certHash": SimStruct({"size": SimTypeInt(signed=False, label="UInt32"), "data": SimTypePointer(SimTypeChar(label="Byte"), offset=0)}, name="FWP_BYTE_BLOB", pack=False, align=None), "eku": SimTypePointer(SimStruct({"numEku": SimTypeInt(signed=False, label="UInt32"), "eku": SimTypePointer(SimTypePointer(SimTypeChar(label="Byte"), offset=0), offset=0)}, name="IKEEXT_CERT_EKUS0", pack=False, align=None), offset=0), "name": SimTypePointer(SimStruct({"nameType": SimTypeInt(signed=False, label="IKEEXT_CERT_CRITERIA_NAME_TYPE"), "certName": SimTypePointer(SimTypeChar(label="Char"), offset=0)}, name="IKEEXT_CERT_NAME0", pack=False, align=None), offset=0), "flags": SimTypeInt(signed=False, label="UInt32")}, name="IKEEXT_CERTIFICATE_CRITERIA0", pack=False, align=None), offset=0)}, name="_Anonymous2_e__Struct", pack=False, align=None), "Anonymous3": SimStruct({"outboundRootStoreArraySize": SimTypeInt(signed=False, label="UInt32"), "outboundTrustedRootStoreCriteria": SimTypePointer(SimStruct({"certData": SimStruct({"size": SimTypeInt(signed=False, label="UInt32"), "data": SimTypePointer(SimTypeChar(label="Byte"), offset=0)}, name="FWP_BYTE_BLOB", pack=False, align=None), "certHash": SimStruct({"size": SimTypeInt(signed=False, label="UInt32"), "data": SimTypePointer(SimTypeChar(label="Byte"), offset=0)}, name="FWP_BYTE_BLOB", pack=False, align=None), "eku": SimTypePointer(SimStruct({"numEku": SimTypeInt(signed=False, label="UInt32"), "eku": SimTypePointer(SimTypePointer(SimTypeChar(label="Byte"), offset=0), offset=0)}, name="IKEEXT_CERT_EKUS0", pack=False, align=None), offset=0), "name": SimTypePointer(SimStruct({"nameType": SimTypeInt(signed=False, label="IKEEXT_CERT_CRITERIA_NAME_TYPE"), "certName": SimTypePointer(SimTypeChar(label="Char"), offset=0)}, name="IKEEXT_CERT_NAME0", pack=False, align=None), offset=0), "flags": SimTypeInt(signed=False, label="UInt32")}, name="IKEEXT_CERTIFICATE_CRITERIA0", pack=False, align=None), offset=0)}, name="_Anonymous3_e__Struct", pack=False, align=None)}, name="<anon>", label="None"), "flags": SimTypeInt(signed=False, label="IKEEXT_CERT_AUTH"), "localCertLocationUrl": SimStruct({"size": SimTypeInt(signed=False, label="UInt32"), "data": SimTypePointer(SimTypeChar(label="Byte"), offset=0)}, name="FWP_BYTE_BLOB", pack=False, align=None)}, name="IKEEXT_CERTIFICATE_AUTHENTICATION2", pack=False, align=None), "cgaAuthentication": SimStruct({"keyContainerName": SimTypePointer(SimTypeChar(label="Char"), offset=0), "cspName": SimTypePointer(SimTypeChar(label="Char"), offset=0), "cspType": SimTypeInt(signed=False, label="UInt32"), "cgaModifier": SimStruct({"byteArray16": SimTypeFixedSizeArray(SimTypeChar(label="Byte"), 16)}, name="FWP_BYTE_ARRAY16", pack=False, align=None), "cgaCollisionCount": SimTypeChar(label="Byte")}, name="IKEEXT_IPV6_CGA_AUTHENTICATION0", pack=False, align=None), "eapAuthentication": SimStruct({"flags": SimTypeInt(signed=False, label="IKEEXT_EAP_AUTHENTICATION_FLAGS")}, name="IKEEXT_EAP_AUTHENTICATION0", pack=False, align=None)}, name="<anon>", label="None")}, name="IKEEXT_AUTHENTICATION_METHOD2", pack=False, align=None), offset=0), "initiatorImpersonationType": SimTypeInt(signed=False, label="IKEEXT_AUTHENTICATION_IMPERSONATION_TYPE")}, name="IKEEXT_EM_POLICY2", pack=False, align=None), offset=0), "fwdPathSaLifetime": SimTypeInt(signed=False, label="UInt32"), "compartmentId": SimTypeInt(signed=False, label="UInt32"), "numTrafficSelectorPolicy": SimTypeInt(signed=False, label="UInt32"), "trafficSelectorPolicies": SimTypePointer(SimStruct({"flags": SimTypeInt(signed=False, label="UInt32"), "numLocalTrafficSelectors": SimTypeInt(signed=False, label="UInt32"), "localTrafficSelectors": SimTypePointer(SimStruct({"protocolId": SimTypeChar(label="Byte"), "portStart": SimTypeShort(signed=False, label="UInt16"), "portEnd": SimTypeShort(signed=False, label="UInt16"), "ipVersion": SimTypeInt(signed=False, label="FWP_IP_VERSION"), "Anonymous1": SimUnion({"startV4Address": SimTypeInt(signed=False, label="UInt32"), "startV6Address": SimTypeFixedSizeArray(SimTypeChar(label="Byte"), 16)}, name="<anon>", label="None"), "Anonymous2": SimUnion({"endV4Address": SimTypeInt(signed=False, label="UInt32"), "endV6Address": SimTypeFixedSizeArray(SimTypeChar(label="Byte"), 16)}, name="<anon>", label="None")}, name="IPSEC_TRAFFIC_SELECTOR0_", pack=False, align=None), offset=0), "numRemoteTrafficSelectors": SimTypeInt(signed=False, label="UInt32"), "remoteTrafficSelectors": SimTypePointer(SimStruct({"protocolId": SimTypeChar(label="Byte"), "portStart": SimTypeShort(signed=False, label="UInt16"), "portEnd": SimTypeShort(signed=False, label="UInt16"), "ipVersion": SimTypeInt(signed=False, label="FWP_IP_VERSION"), "Anonymous1": SimUnion({"startV4Address": SimTypeInt(signed=False, label="UInt32"), "startV6Address": SimTypeFixedSizeArray(SimTypeChar(label="Byte"), 16)}, name="<anon>", label="None"), "Anonymous2": SimUnion({"endV4Address": SimTypeInt(signed=False, label="UInt32"), "endV6Address": SimTypeFixedSizeArray(SimTypeChar(label="Byte"), 16)}, name="<anon>", label="None")}, name="IPSEC_TRAFFIC_SELECTOR0_", pack=False, align=None), offset=0)}, name="IPSEC_TRAFFIC_SELECTOR_POLICY0_", pack=False, align=None), offset=0)}, name="IPSEC_TUNNEL_POLICY3_", pack=False, align=None), offset=0), "ikeV2QmTransportPolicy": SimTypePointer(SimStruct({"numIpsecProposals": SimTypeInt(signed=False, label="UInt32"), "ipsecProposals": SimTypePointer(SimStruct({"lifetime": SimStruct({"lifetimeSeconds": SimTypeInt(signed=False, label="UInt32"), "lifetimeKilobytes": SimTypeInt(signed=False, label="UInt32"), "lifetimePackets": SimTypeInt(signed=False, label="UInt32")}, name="IPSEC_SA_LIFETIME0", pack=False, align=None), "numSaTransforms": SimTypeInt(signed=False, label="UInt32"), "saTransforms": SimTypePointer(SimStruct({"ipsecTransformType": SimTypeInt(signed=False, label="IPSEC_TRANSFORM_TYPE"), "Anonymous": SimUnion({"ahTransform": SimTypePointer(SimStruct({"authTransformId": SimStruct({"authType": SimTypeInt(signed=False, label="IPSEC_AUTH_TYPE"), "authConfig": SimTypeChar(label="Byte")}, name="IPSEC_AUTH_TRANSFORM_ID0", pack=False, align=None), "cryptoModuleId": SimTypePointer(SimTypeBottom(label="Guid"), offset=0)}, name="IPSEC_AUTH_TRANSFORM0", pack=False, align=None), offset=0), "espAuthTransform": SimTypePointer(SimStruct({"authTransformId": SimStruct({"authType": SimTypeInt(signed=False, label="IPSEC_AUTH_TYPE"), "authConfig": SimTypeChar(label="Byte")}, name="IPSEC_AUTH_TRANSFORM_ID0", pack=False, align=None), "cryptoModuleId": SimTypePointer(SimTypeBottom(label="Guid"), offset=0)}, name="IPSEC_AUTH_TRANSFORM0", pack=False, align=None), offset=0), "espCipherTransform": SimTypePointer(SimStruct({"cipherTransformId": SimStruct({"cipherType": SimTypeInt(signed=False, label="IPSEC_CIPHER_TYPE"), "cipherConfig": SimTypeChar(label="Byte")}, name="IPSEC_CIPHER_TRANSFORM_ID0", pack=False, align=None), "cryptoModuleId": SimTypePointer(SimTypeBottom(label="Guid"), offset=0)}, name="IPSEC_CIPHER_TRANSFORM0", pack=False, align=None), offset=0), "espAuthAndCipherTransform": SimTypePointer(SimStruct({"authTransform": SimStruct({"authTransformId": SimStruct({"authType": SimTypeInt(signed=False, label="IPSEC_AUTH_TYPE"), "authConfig": SimTypeChar(label="Byte")}, name="IPSEC_AUTH_TRANSFORM_ID0", pack=False, align=None), "cryptoModuleId": SimTypePointer(SimTypeBottom(label="Guid"), offset=0)}, name="IPSEC_AUTH_TRANSFORM0", pack=False, align=None), "cipherTransform": SimStruct({"cipherTransformId": SimStruct({"cipherType": SimTypeInt(signed=False, label="IPSEC_CIPHER_TYPE"), "cipherConfig": SimTypeChar(label="Byte")}, name="IPSEC_CIPHER_TRANSFORM_ID0", pack=False, align=None), "cryptoModuleId": SimTypePointer(SimTypeBottom(label="Guid"), offset=0)}, name="IPSEC_CIPHER_TRANSFORM0", pack=False, align=None)}, name="IPSEC_AUTH_AND_CIPHER_TRANSFORM0", pack=False, align=None), offset=0), "espAuthFwTransform": SimTypePointer(SimStruct({"authTransformId": SimStruct({"authType": SimTypeInt(signed=False, label="IPSEC_AUTH_TYPE"), "authConfig": SimTypeChar(label="Byte")}, name="IPSEC_AUTH_TRANSFORM_ID0", pack=False, align=None), "cryptoModuleId": SimTypePointer(SimTypeBottom(label="Guid"), offset=0)}, name="IPSEC_AUTH_TRANSFORM0", pack=False, align=None), offset=0)}, name="<anon>", label="None")}, name="IPSEC_SA_TRANSFORM0", pack=False, align=None), offset=0), "pfsGroup": SimTypeInt(signed=False, label="IPSEC_PFS_GROUP")}, name="IPSEC_PROPOSAL0", pack=False, align=None), offset=0), "flags": SimTypeInt(signed=False, label="IPSEC_POLICY_FLAG"), "ndAllowClearTimeoutSeconds": SimTypeInt(signed=False, label="UInt32"), "saIdleTimeout": SimStruct({"idleTimeoutSeconds": SimTypeInt(signed=False, label="UInt32"), "idleTimeoutSecondsFailOver": SimTypeInt(signed=False, label="UInt32")}, name="IPSEC_SA_IDLE_TIMEOUT0", pack=False, align=None), "emPolicy": SimTypePointer(SimStruct({"numAuthenticationMethods": SimTypeInt(signed=False, label="UInt32"), "authenticationMethods": SimTypePointer(SimStruct({"authenticationMethodType": SimTypeInt(signed=False, label="IKEEXT_AUTHENTICATION_METHOD_TYPE"), "Anonymous": SimUnion({"presharedKeyAuthentication": SimStruct({"presharedKey": SimStruct({"size": SimTypeInt(signed=False, label="UInt32"), "data": SimTypePointer(SimTypeChar(label="Byte"), offset=0)}, name="FWP_BYTE_BLOB", pack=False, align=None), "flags": SimTypeInt(signed=False, label="IKEEXT_PRESHARED_KEY_AUTHENTICATION_FLAGS")}, name="IKEEXT_PRESHARED_KEY_AUTHENTICATION1", pack=False, align=None), "certificateAuthentication": SimStruct({"inboundConfigType": SimTypeInt(signed=False, label="IKEEXT_CERT_CONFIG_TYPE"), "Anonymous1": SimUnion({"Anonymous1": SimStruct({"inboundRootArraySize": SimTypeInt(signed=False, label="UInt32"), "inboundRootCriteria": SimTypePointer(SimStruct({"certData": SimStruct({"size": SimTypeInt(signed=False, label="UInt32"), "data": SimTypePointer(SimTypeChar(label="Byte"), offset=0)}, name="FWP_BYTE_BLOB", pack=False, align=None), "certHash": SimStruct({"size": SimTypeInt(signed=False, label="UInt32"), "data": SimTypePointer(SimTypeChar(label="Byte"), offset=0)}, name="FWP_BYTE_BLOB", pack=False, align=None), "eku": SimTypePointer(SimStruct({"numEku": SimTypeInt(signed=False, label="UInt32"), "eku": SimTypePointer(SimTypePointer(SimTypeChar(label="Byte"), offset=0), offset=0)}, name="IKEEXT_CERT_EKUS0", pack=False, align=None), offset=0), "name": SimTypePointer(SimStruct({"nameType": SimTypeInt(signed=False, label="IKEEXT_CERT_CRITERIA_NAME_TYPE"), "certName": SimTypePointer(SimTypeChar(label="Char"), offset=0)}, name="IKEEXT_CERT_NAME0", pack=False, align=None), offset=0), "flags": SimTypeInt(signed=False, label="UInt32")}, name="IKEEXT_CERTIFICATE_CRITERIA0", pack=False, align=None), offset=0)}, name="_Anonymous1_e__Struct", pack=False, align=None), "Anonymous2": SimStruct({"inboundEnterpriseStoreArraySize": SimTypeInt(signed=False, label="UInt32"), "inboundEnterpriseStoreCriteria": SimTypePointer(SimStruct({"certData": SimStruct({"size": SimTypeInt(signed=False, label="UInt32"), "data": SimTypePointer(SimTypeChar(label="Byte"), offset=0)}, name="FWP_BYTE_BLOB", pack=False, align=None), "certHash": SimStruct({"size": SimTypeInt(signed=False, label="UInt32"), "data": SimTypePointer(SimTypeChar(label="Byte"), offset=0)}, name="FWP_BYTE_BLOB", pack=False, align=None), "eku": SimTypePointer(SimStruct({"numEku": SimTypeInt(signed=False, label="UInt32"), "eku": SimTypePointer(SimTypePointer(SimTypeChar(label="Byte"), offset=0), offset=0)}, name="IKEEXT_CERT_EKUS0", pack=False, align=None), offset=0), "name": SimTypePointer(SimStruct({"nameType": SimTypeInt(signed=False, label="IKEEXT_CERT_CRITERIA_NAME_TYPE"), "certName": SimTypePointer(SimTypeChar(label="Char"), offset=0)}, name="IKEEXT_CERT_NAME0", pack=False, align=None), offset=0), "flags": SimTypeInt(signed=False, label="UInt32")}, name="IKEEXT_CERTIFICATE_CRITERIA0", pack=False, align=None), offset=0)}, name="_Anonymous2_e__Struct", pack=False, align=None), "Anonymous3": SimStruct({"inboundRootStoreArraySize": SimTypeInt(signed=False, label="UInt32"), "inboundTrustedRootStoreCriteria": SimTypePointer(SimStruct({"certData": SimStruct({"size": SimTypeInt(signed=False, label="UInt32"), "data": SimTypePointer(SimTypeChar(label="Byte"), offset=0)}, name="FWP_BYTE_BLOB", pack=False, align=None), "certHash": SimStruct({"size": SimTypeInt(signed=False, label="UInt32"), "data": SimTypePointer(SimTypeChar(label="Byte"), offset=0)}, name="FWP_BYTE_BLOB", pack=False, align=None), "eku": SimTypePointer(SimStruct({"numEku": SimTypeInt(signed=False, label="UInt32"), "eku": SimTypePointer(SimTypePointer(SimTypeChar(label="Byte"), offset=0), offset=0)}, name="IKEEXT_CERT_EKUS0", pack=False, align=None), offset=0), "name": SimTypePointer(SimStruct({"nameType": SimTypeInt(signed=False, label="IKEEXT_CERT_CRITERIA_NAME_TYPE"), "certName": SimTypePointer(SimTypeChar(label="Char"), offset=0)}, name="IKEEXT_CERT_NAME0", pack=False, align=None), offset=0), "flags": SimTypeInt(signed=False, label="UInt32")}, name="IKEEXT_CERTIFICATE_CRITERIA0", pack=False, align=None), offset=0)}, name="_Anonymous3_e__Struct", pack=False, align=None)}, name="<anon>", label="None"), "outboundConfigType": SimTypeInt(signed=False, label="IKEEXT_CERT_CONFIG_TYPE"), "Anonymous2": SimUnion({"Anonymous1": SimStruct({"outboundRootArraySize": SimTypeInt(signed=False, label="UInt32"), "outboundRootCriteria": SimTypePointer(SimStruct({"certData": SimStruct({"size": SimTypeInt(signed=False, label="UInt32"), "data": SimTypePointer(SimTypeChar(label="Byte"), offset=0)}, name="FWP_BYTE_BLOB", pack=False, align=None), "certHash": SimStruct({"size": SimTypeInt(signed=False, label="UInt32"), "data": SimTypePointer(SimTypeChar(label="Byte"), offset=0)}, name="FWP_BYTE_BLOB", pack=False, align=None), "eku": SimTypePointer(SimStruct({"numEku": SimTypeInt(signed=False, label="UInt32"), "eku": SimTypePointer(SimTypePointer(SimTypeChar(label="Byte"), offset=0), offset=0)}, name="IKEEXT_CERT_EKUS0", pack=False, align=None), offset=0), "name": SimTypePointer(SimStruct({"nameType": SimTypeInt(signed=False, label="IKEEXT_CERT_CRITERIA_NAME_TYPE"), "certName": SimTypePointer(SimTypeChar(label="Char"), offset=0)}, name="IKEEXT_CERT_NAME0", pack=False, align=None), offset=0), "flags": SimTypeInt(signed=False, label="UInt32")}, name="IKEEXT_CERTIFICATE_CRITERIA0", pack=False, align=None), offset=0)}, name="_Anonymous1_e__Struct", pack=False, align=None), "Anonymous2": SimStruct({"outboundEnterpriseStoreArraySize": SimTypeInt(signed=False, label="UInt32"), "outboundEnterpriseStoreCriteria": SimTypePointer(SimStruct({"certData": SimStruct({"size": SimTypeInt(signed=False, label="UInt32"), "data": SimTypePointer(SimTypeChar(label="Byte"), offset=0)}, name="FWP_BYTE_BLOB", pack=False, align=None), "certHash": SimStruct({"size": SimTypeInt(signed=False, label="UInt32"), "data": SimTypePointer(SimTypeChar(label="Byte"), offset=0)}, name="FWP_BYTE_BLOB", pack=False, align=None), "eku": SimTypePointer(SimStruct({"numEku": SimTypeInt(signed=False, label="UInt32"), "eku": SimTypePointer(SimTypePointer(SimTypeChar(label="Byte"), offset=0), offset=0)}, name="IKEEXT_CERT_EKUS0", pack=False, align=None), offset=0), "name": SimTypePointer(SimStruct({"nameType": SimTypeInt(signed=False, label="IKEEXT_CERT_CRITERIA_NAME_TYPE"), "certName": SimTypePointer(SimTypeChar(label="Char"), offset=0)}, name="IKEEXT_CERT_NAME0", pack=False, align=None), offset=0), "flags": SimTypeInt(signed=False, label="UInt32")}, name="IKEEXT_CERTIFICATE_CRITERIA0", pack=False, align=None), offset=0)}, name="_Anonymous2_e__Struct", pack=False, align=None), "Anonymous3": SimStruct({"outboundRootStoreArraySize": SimTypeInt(signed=False, label="UInt32"), "outboundTrustedRootStoreCriteria": SimTypePointer(SimStruct({"certData": SimStruct({"size": SimTypeInt(signed=False, label="UInt32"), "data": SimTypePointer(SimTypeChar(label="Byte"), offset=0)}, name="FWP_BYTE_BLOB", pack=False, align=None), "certHash": SimStruct({"size": SimTypeInt(signed=False, label="UInt32"), "data": SimTypePointer(SimTypeChar(label="Byte"), offset=0)}, name="FWP_BYTE_BLOB", pack=False, align=None), "eku": SimTypePointer(SimStruct({"numEku": SimTypeInt(signed=False, label="UInt32"), "eku": SimTypePointer(SimTypePointer(SimTypeChar(label="Byte"), offset=0), offset=0)}, name="IKEEXT_CERT_EKUS0", pack=False, align=None), offset=0), "name": SimTypePointer(SimStruct({"nameType": SimTypeInt(signed=False, label="IKEEXT_CERT_CRITERIA_NAME_TYPE"), "certName": SimTypePointer(SimTypeChar(label="Char"), offset=0)}, name="IKEEXT_CERT_NAME0", pack=False, align=None), offset=0), "flags": SimTypeInt(signed=False, label="UInt32")}, name="IKEEXT_CERTIFICATE_CRITERIA0", pack=False, align=None), offset=0)}, name="_Anonymous3_e__Struct", pack=False, align=None)}, name="<anon>", label="None"), "flags": SimTypeInt(signed=False, label="IKEEXT_CERT_AUTH"), "localCertLocationUrl": SimStruct({"size": SimTypeInt(signed=False, label="UInt32"), "data": SimTypePointer(SimTypeChar(label="Byte"), offset=0)}, name="FWP_BYTE_BLOB", pack=False, align=None)}, name="IKEEXT_CERTIFICATE_AUTHENTICATION2", pack=False, align=None), "kerberosAuthentication": SimStruct({"flags": SimTypeInt(signed=False, label="IKEEXT_KERBEROS_AUTHENTICATION_FLAGS"), "proxyServer": SimTypePointer(SimTypeChar(label="Char"), offset=0)}, name="IKEEXT_KERBEROS_AUTHENTICATION1", pack=False, align=None), "reservedAuthentication": SimStruct({"flags": SimTypeInt(signed=False, label="IKEEXT_RESERVED_AUTHENTICATION_FLAGS")}, name="IKEEXT_RESERVED_AUTHENTICATION0", pack=False, align=None), "ntlmV2Authentication": SimStruct({"flags": SimTypeInt(signed=False, label="UInt32")}, name="IKEEXT_NTLM_V2_AUTHENTICATION0", pack=False, align=None), "sslAuthentication": SimStruct({"inboundConfigType": SimTypeInt(signed=False, label="IKEEXT_CERT_CONFIG_TYPE"), "Anonymous1": SimUnion({"Anonymous1": SimStruct({"inboundRootArraySize": SimTypeInt(signed=False, label="UInt32"), "inboundRootCriteria": SimTypePointer(SimStruct({"certData": SimStruct({"size": SimTypeInt(signed=False, label="UInt32"), "data": SimTypePointer(SimTypeChar(label="Byte"), offset=0)}, name="FWP_BYTE_BLOB", pack=False, align=None), "certHash": SimStruct({"size": SimTypeInt(signed=False, label="UInt32"), "data": SimTypePointer(SimTypeChar(label="Byte"), offset=0)}, name="FWP_BYTE_BLOB", pack=False, align=None), "eku": SimTypePointer(SimStruct({"numEku": SimTypeInt(signed=False, label="UInt32"), "eku": SimTypePointer(SimTypePointer(SimTypeChar(label="Byte"), offset=0), offset=0)}, name="IKEEXT_CERT_EKUS0", pack=False, align=None), offset=0), "name": SimTypePointer(SimStruct({"nameType": SimTypeInt(signed=False, label="IKEEXT_CERT_CRITERIA_NAME_TYPE"), "certName": SimTypePointer(SimTypeChar(label="Char"), offset=0)}, name="IKEEXT_CERT_NAME0", pack=False, align=None), offset=0), "flags": SimTypeInt(signed=False, label="UInt32")}, name="IKEEXT_CERTIFICATE_CRITERIA0", pack=False, align=None), offset=0)}, name="_Anonymous1_e__Struct", pack=False, align=None), "Anonymous2": SimStruct({"inboundEnterpriseStoreArraySize": SimTypeInt(signed=False, label="UInt32"), "inboundEnterpriseStoreCriteria": SimTypePointer(SimStruct({"certData": SimStruct({"size": SimTypeInt(signed=False, label="UInt32"), "data": SimTypePointer(SimTypeChar(label="Byte"), offset=0)}, name="FWP_BYTE_BLOB", pack=False, align=None), "certHash": SimStruct({"size": SimTypeInt(signed=False, label="UInt32"), "data": SimTypePointer(SimTypeChar(label="Byte"), offset=0)}, name="FWP_BYTE_BLOB", pack=False, align=None), "eku": SimTypePointer(SimStruct({"numEku": SimTypeInt(signed=False, label="UInt32"), "eku": SimTypePointer(SimTypePointer(SimTypeChar(label="Byte"), offset=0), offset=0)}, name="IKEEXT_CERT_EKUS0", pack=False, align=None), offset=0), "name": SimTypePointer(SimStruct({"nameType": SimTypeInt(signed=False, label="IKEEXT_CERT_CRITERIA_NAME_TYPE"), "certName": SimTypePointer(SimTypeChar(label="Char"), offset=0)}, name="IKEEXT_CERT_NAME0", pack=False, align=None), offset=0), "flags": SimTypeInt(signed=False, label="UInt32")}, name="IKEEXT_CERTIFICATE_CRITERIA0", pack=False, align=None), offset=0)}, name="_Anonymous2_e__Struct", pack=False, align=None), "Anonymous3": SimStruct({"inboundRootStoreArraySize": SimTypeInt(signed=False, label="UInt32"), "inboundTrustedRootStoreCriteria": SimTypePointer(SimStruct({"certData": SimStruct({"size": SimTypeInt(signed=False, label="UInt32"), "data": SimTypePointer(SimTypeChar(label="Byte"), offset=0)}, name="FWP_BYTE_BLOB", pack=False, align=None), "certHash": SimStruct({"size": SimTypeInt(signed=False, label="UInt32"), "data": SimTypePointer(SimTypeChar(label="Byte"), offset=0)}, name="FWP_BYTE_BLOB", pack=False, align=None), "eku": SimTypePointer(SimStruct({"numEku": SimTypeInt(signed=False, label="UInt32"), "eku": SimTypePointer(SimTypePointer(SimTypeChar(label="Byte"), offset=0), offset=0)}, name="IKEEXT_CERT_EKUS0", pack=False, align=None), offset=0), "name": SimTypePointer(SimStruct({"nameType": SimTypeInt(signed=False, label="IKEEXT_CERT_CRITERIA_NAME_TYPE"), "certName": SimTypePointer(SimTypeChar(label="Char"), offset=0)}, name="IKEEXT_CERT_NAME0", pack=False, align=None), offset=0), "flags": SimTypeInt(signed=False, label="UInt32")}, name="IKEEXT_CERTIFICATE_CRITERIA0", pack=False, align=None), offset=0)}, name="_Anonymous3_e__Struct", pack=False, align=None)}, name="<anon>", label="None"), "outboundConfigType": SimTypeInt(signed=False, label="IKEEXT_CERT_CONFIG_TYPE"), "Anonymous2": SimUnion({"Anonymous1": SimStruct({"outboundRootArraySize": SimTypeInt(signed=False, label="UInt32"), "outboundRootCriteria": SimTypePointer(SimStruct({"certData": SimStruct({"size": SimTypeInt(signed=False, label="UInt32"), "data": SimTypePointer(SimTypeChar(label="Byte"), offset=0)}, name="FWP_BYTE_BLOB", pack=False, align=None), "certHash": SimStruct({"size": SimTypeInt(signed=False, label="UInt32"), "data": SimTypePointer(SimTypeChar(label="Byte"), offset=0)}, name="FWP_BYTE_BLOB", pack=False, align=None), "eku": SimTypePointer(SimStruct({"numEku": SimTypeInt(signed=False, label="UInt32"), "eku": SimTypePointer(SimTypePointer(SimTypeChar(label="Byte"), offset=0), offset=0)}, name="IKEEXT_CERT_EKUS0", pack=False, align=None), offset=0), "name": SimTypePointer(SimStruct({"nameType": SimTypeInt(signed=False, label="IKEEXT_CERT_CRITERIA_NAME_TYPE"), "certName": SimTypePointer(SimTypeChar(label="Char"), offset=0)}, name="IKEEXT_CERT_NAME0", pack=False, align=None), offset=0), "flags": SimTypeInt(signed=False, label="UInt32")}, name="IKEEXT_CERTIFICATE_CRITERIA0", pack=False, align=None), offset=0)}, name="_Anonymous1_e__Struct", pack=False, align=None), "Anonymous2": SimStruct({"outboundEnterpriseStoreArraySize": SimTypeInt(signed=False, label="UInt32"), "outboundEnterpriseStoreCriteria": SimTypePointer(SimStruct({"certData": SimStruct({"size": SimTypeInt(signed=False, label="UInt32"), "data": SimTypePointer(SimTypeChar(label="Byte"), offset=0)}, name="FWP_BYTE_BLOB", pack=False, align=None), "certHash": SimStruct({"size": SimTypeInt(signed=False, label="UInt32"), "data": SimTypePointer(SimTypeChar(label="Byte"), offset=0)}, name="FWP_BYTE_BLOB", pack=False, align=None), "eku": SimTypePointer(SimStruct({"numEku": SimTypeInt(signed=False, label="UInt32"), "eku": SimTypePointer(SimTypePointer(SimTypeChar(label="Byte"), offset=0), offset=0)}, name="IKEEXT_CERT_EKUS0", pack=False, align=None), offset=0), "name": SimTypePointer(SimStruct({"nameType": SimTypeInt(signed=False, label="IKEEXT_CERT_CRITERIA_NAME_TYPE"), "certName": SimTypePointer(SimTypeChar(label="Char"), offset=0)}, name="IKEEXT_CERT_NAME0", pack=False, align=None), offset=0), "flags": SimTypeInt(signed=False, label="UInt32")}, name="IKEEXT_CERTIFICATE_CRITERIA0", pack=False, align=None), offset=0)}, name="_Anonymous2_e__Struct", pack=False, align=None), "Anonymous3": SimStruct({"outboundRootStoreArraySize": SimTypeInt(signed=False, label="UInt32"), "outboundTrustedRootStoreCriteria": SimTypePointer(SimStruct({"certData": SimStruct({"size": SimTypeInt(signed=False, label="UInt32"), "data": SimTypePointer(SimTypeChar(label="Byte"), offset=0)}, name="FWP_BYTE_BLOB", pack=False, align=None), "certHash": SimStruct({"size": SimTypeInt(signed=False, label="UInt32"), "data": SimTypePointer(SimTypeChar(label="Byte"), offset=0)}, name="FWP_BYTE_BLOB", pack=False, align=None), "eku": SimTypePointer(SimStruct({"numEku": SimTypeInt(signed=False, label="UInt32"), "eku": SimTypePointer(SimTypePointer(SimTypeChar(label="Byte"), offset=0), offset=0)}, name="IKEEXT_CERT_EKUS0", pack=False, align=None), offset=0), "name": SimTypePointer(SimStruct({"nameType": SimTypeInt(signed=False, label="IKEEXT_CERT_CRITERIA_NAME_TYPE"), "certName": SimTypePointer(SimTypeChar(label="Char"), offset=0)}, name="IKEEXT_CERT_NAME0", pack=False, align=None), offset=0), "flags": SimTypeInt(signed=False, label="UInt32")}, name="IKEEXT_CERTIFICATE_CRITERIA0", pack=False, align=None), offset=0)}, name="_Anonymous3_e__Struct", pack=False, align=None)}, name="<anon>", label="None"), "flags": SimTypeInt(signed=False, label="IKEEXT_CERT_AUTH"), "localCertLocationUrl": SimStruct({"size": SimTypeInt(signed=False, label="UInt32"), "data": SimTypePointer(SimTypeChar(label="Byte"), offset=0)}, name="FWP_BYTE_BLOB", pack=False, align=None)}, name="IKEEXT_CERTIFICATE_AUTHENTICATION2", pack=False, align=None), "cgaAuthentication": SimStruct({"keyContainerName": SimTypePointer(SimTypeChar(label="Char"), offset=0), "cspName": SimTypePointer(SimTypeChar(label="Char"), offset=0), "cspType": SimTypeInt(signed=False, label="UInt32"), "cgaModifier": SimStruct({"byteArray16": SimTypeFixedSizeArray(SimTypeChar(label="Byte"), 16)}, name="FWP_BYTE_ARRAY16", pack=False, align=None), "cgaCollisionCount": SimTypeChar(label="Byte")}, name="IKEEXT_IPV6_CGA_AUTHENTICATION0", pack=False, align=None), "eapAuthentication": SimStruct({"flags": SimTypeInt(signed=False, label="IKEEXT_EAP_AUTHENTICATION_FLAGS")}, name="IKEEXT_EAP_AUTHENTICATION0", pack=False, align=None)}, name="<anon>", label="None")}, name="IKEEXT_AUTHENTICATION_METHOD2", pack=False, align=None), offset=0), "initiatorImpersonationType": SimTypeInt(signed=False, label="IKEEXT_AUTHENTICATION_IMPERSONATION_TYPE")}, name="IKEEXT_EM_POLICY2", pack=False, align=None), offset=0)}, name="IPSEC_TRANSPORT_POLICY2", pack=False, align=None), offset=0), "ikeV2MmPolicy": SimTypePointer(SimStruct({"softExpirationTime": SimTypeInt(signed=False, label="UInt32"), "numAuthenticationMethods": SimTypeInt(signed=False, label="UInt32"), "authenticationMethods": SimTypePointer(SimStruct({"authenticationMethodType": SimTypeInt(signed=False, label="IKEEXT_AUTHENTICATION_METHOD_TYPE"), "Anonymous": SimUnion({"presharedKeyAuthentication": SimStruct({"presharedKey": SimStruct({"size": SimTypeInt(signed=False, label="UInt32"), "data": SimTypePointer(SimTypeChar(label="Byte"), offset=0)}, name="FWP_BYTE_BLOB", pack=False, align=None), "flags": SimTypeInt(signed=False, label="IKEEXT_PRESHARED_KEY_AUTHENTICATION_FLAGS")}, name="IKEEXT_PRESHARED_KEY_AUTHENTICATION1", pack=False, align=None), "certificateAuthentication": SimStruct({"inboundConfigType": SimTypeInt(signed=False, label="IKEEXT_CERT_CONFIG_TYPE"), "Anonymous1": SimUnion({"Anonymous1": SimStruct({"inboundRootArraySize": SimTypeInt(signed=False, label="UInt32"), "inboundRootCriteria": SimTypePointer(SimStruct({"certData": SimStruct({"size": SimTypeInt(signed=False, label="UInt32"), "data": SimTypePointer(SimTypeChar(label="Byte"), offset=0)}, name="FWP_BYTE_BLOB", pack=False, align=None), "certHash": SimStruct({"size": SimTypeInt(signed=False, label="UInt32"), "data": SimTypePointer(SimTypeChar(label="Byte"), offset=0)}, name="FWP_BYTE_BLOB", pack=False, align=None), "eku": SimTypePointer(SimStruct({"numEku": SimTypeInt(signed=False, label="UInt32"), "eku": SimTypePointer(SimTypePointer(SimTypeChar(label="Byte"), offset=0), offset=0)}, name="IKEEXT_CERT_EKUS0", pack=False, align=None), offset=0), "name": SimTypePointer(SimStruct({"nameType": SimTypeInt(signed=False, label="IKEEXT_CERT_CRITERIA_NAME_TYPE"), "certName": SimTypePointer(SimTypeChar(label="Char"), offset=0)}, name="IKEEXT_CERT_NAME0", pack=False, align=None), offset=0), "flags": SimTypeInt(signed=False, label="UInt32")}, name="IKEEXT_CERTIFICATE_CRITERIA0", pack=False, align=None), offset=0)}, name="_Anonymous1_e__Struct", pack=False, align=None), "Anonymous2": SimStruct({"inboundEnterpriseStoreArraySize": SimTypeInt(signed=False, label="UInt32"), "inboundEnterpriseStoreCriteria": SimTypePointer(SimStruct({"certData": SimStruct({"size": SimTypeInt(signed=False, label="UInt32"), "data": SimTypePointer(SimTypeChar(label="Byte"), offset=0)}, name="FWP_BYTE_BLOB", pack=False, align=None), "certHash": SimStruct({"size": SimTypeInt(signed=False, label="UInt32"), "data": SimTypePointer(SimTypeChar(label="Byte"), offset=0)}, name="FWP_BYTE_BLOB", pack=False, align=None), "eku": SimTypePointer(SimStruct({"numEku": SimTypeInt(signed=False, label="UInt32"), "eku": SimTypePointer(SimTypePointer(SimTypeChar(label="Byte"), offset=0), offset=0)}, name="IKEEXT_CERT_EKUS0", pack=False, align=None), offset=0), "name": SimTypePointer(SimStruct({"nameType": SimTypeInt(signed=False, label="IKEEXT_CERT_CRITERIA_NAME_TYPE"), "certName": SimTypePointer(SimTypeChar(label="Char"), offset=0)}, name="IKEEXT_CERT_NAME0", pack=False, align=None), offset=0), "flags": SimTypeInt(signed=False, label="UInt32")}, name="IKEEXT_CERTIFICATE_CRITERIA0", pack=False, align=None), offset=0)}, name="_Anonymous2_e__Struct", pack=False, align=None), "Anonymous3": SimStruct({"inboundRootStoreArraySize": SimTypeInt(signed=False, label="UInt32"), "inboundTrustedRootStoreCriteria": SimTypePointer(SimStruct({"certData": SimStruct({"size": SimTypeInt(signed=False, label="UInt32"), "data": SimTypePointer(SimTypeChar(label="Byte"), offset=0)}, name="FWP_BYTE_BLOB", pack=False, align=None), "certHash": SimStruct({"size": SimTypeInt(signed=False, label="UInt32"), "data": SimTypePointer(SimTypeChar(label="Byte"), offset=0)}, name="FWP_BYTE_BLOB", pack=False, align=None), "eku": SimTypePointer(SimStruct({"numEku": SimTypeInt(signed=False, label="UInt32"), "eku": SimTypePointer(SimTypePointer(SimTypeChar(label="Byte"), offset=0), offset=0)}, name="IKEEXT_CERT_EKUS0", pack=False, align=None), offset=0), "name": SimTypePointer(SimStruct({"nameType": SimTypeInt(signed=False, label="IKEEXT_CERT_CRITERIA_NAME_TYPE"), "certName": SimTypePointer(SimTypeChar(label="Char"), offset=0)}, name="IKEEXT_CERT_NAME0", pack=False, align=None), offset=0), "flags": SimTypeInt(signed=False, label="UInt32")}, name="IKEEXT_CERTIFICATE_CRITERIA0", pack=False, align=None), offset=0)}, name="_Anonymous3_e__Struct", pack=False, align=None)}, name="<anon>", label="None"), "outboundConfigType": SimTypeInt(signed=False, label="IKEEXT_CERT_CONFIG_TYPE"), "Anonymous2": SimUnion({"Anonymous1": SimStruct({"outboundRootArraySize": SimTypeInt(signed=False, label="UInt32"), "outboundRootCriteria": SimTypePointer(SimStruct({"certData": SimStruct({"size": SimTypeInt(signed=False, label="UInt32"), "data": SimTypePointer(SimTypeChar(label="Byte"), offset=0)}, name="FWP_BYTE_BLOB", pack=False, align=None), "certHash": SimStruct({"size": SimTypeInt(signed=False, label="UInt32"), "data": SimTypePointer(SimTypeChar(label="Byte"), offset=0)}, name="FWP_BYTE_BLOB", pack=False, align=None), "eku": SimTypePointer(SimStruct({"numEku": SimTypeInt(signed=False, label="UInt32"), "eku": SimTypePointer(SimTypePointer(SimTypeChar(label="Byte"), offset=0), offset=0)}, name="IKEEXT_CERT_EKUS0", pack=False, align=None), offset=0), "name": SimTypePointer(SimStruct({"nameType": SimTypeInt(signed=False, label="IKEEXT_CERT_CRITERIA_NAME_TYPE"), "certName": SimTypePointer(SimTypeChar(label="Char"), offset=0)}, name="IKEEXT_CERT_NAME0", pack=False, align=None), offset=0), "flags": SimTypeInt(signed=False, label="UInt32")}, name="IKEEXT_CERTIFICATE_CRITERIA0", pack=False, align=None), offset=0)}, name="_Anonymous1_e__Struct", pack=False, align=None), "Anonymous2": SimStruct({"outboundEnterpriseStoreArraySize": SimTypeInt(signed=False, label="UInt32"), "outboundEnterpriseStoreCriteria": SimTypePointer(SimStruct({"certData": SimStruct({"size": SimTypeInt(signed=False, label="UInt32"), "data": SimTypePointer(SimTypeChar(label="Byte"), offset=0)}, name="FWP_BYTE_BLOB", pack=False, align=None), "certHash": SimStruct({"size": SimTypeInt(signed=False, label="UInt32"), "data": SimTypePointer(SimTypeChar(label="Byte"), offset=0)}, name="FWP_BYTE_BLOB", pack=False, align=None), "eku": SimTypePointer(SimStruct({"numEku": SimTypeInt(signed=False, label="UInt32"), "eku": SimTypePointer(SimTypePointer(SimTypeChar(label="Byte"), offset=0), offset=0)}, name="IKEEXT_CERT_EKUS0", pack=False, align=None), offset=0), "name": SimTypePointer(SimStruct({"nameType": SimTypeInt(signed=False, label="IKEEXT_CERT_CRITERIA_NAME_TYPE"), "certName": SimTypePointer(SimTypeChar(label="Char"), offset=0)}, name="IKEEXT_CERT_NAME0", pack=False, align=None), offset=0), "flags": SimTypeInt(signed=False, label="UInt32")}, name="IKEEXT_CERTIFICATE_CRITERIA0", pack=False, align=None), offset=0)}, name="_Anonymous2_e__Struct", pack=False, align=None), "Anonymous3": SimStruct({"outboundRootStoreArraySize": SimTypeInt(signed=False, label="UInt32"), "outboundTrustedRootStoreCriteria": SimTypePointer(SimStruct({"certData": SimStruct({"size": SimTypeInt(signed=False, label="UInt32"), "data": SimTypePointer(SimTypeChar(label="Byte"), offset=0)}, name="FWP_BYTE_BLOB", pack=False, align=None), "certHash": SimStruct({"size": SimTypeInt(signed=False, label="UInt32"), "data": SimTypePointer(SimTypeChar(label="Byte"), offset=0)}, name="FWP_BYTE_BLOB", pack=False, align=None), "eku": SimTypePointer(SimStruct({"numEku": SimTypeInt(signed=False, label="UInt32"), "eku": SimTypePointer(SimTypePointer(SimTypeChar(label="Byte"), offset=0), offset=0)}, name="IKEEXT_CERT_EKUS0", pack=False, align=None), offset=0), "name": SimTypePointer(SimStruct({"nameType": SimTypeInt(signed=False, label="IKEEXT_CERT_CRITERIA_NAME_TYPE"), "certName": SimTypePointer(SimTypeChar(label="Char"), offset=0)}, name="IKEEXT_CERT_NAME0", pack=False, align=None), offset=0), "flags": SimTypeInt(signed=False, label="UInt32")}, name="IKEEXT_CERTIFICATE_CRITERIA0", pack=False, align=None), offset=0)}, name="_Anonymous3_e__Struct", pack=False, align=None)}, name="<anon>", label="None"), "flags": SimTypeInt(signed=False, label="IKEEXT_CERT_AUTH"), "localCertLocationUrl": SimStruct({"size": SimTypeInt(signed=False, label="UInt32"), "data": SimTypePointer(SimTypeChar(label="Byte"), offset=0)}, name="FWP_BYTE_BLOB", pack=False, align=None)}, name="IKEEXT_CERTIFICATE_AUTHENTICATION2", pack=False, align=None), "kerberosAuthentication": SimStruct({"flags": SimTypeInt(signed=False, label="IKEEXT_KERBEROS_AUTHENTICATION_FLAGS"), "proxyServer": SimTypePointer(SimTypeChar(label="Char"), offset=0)}, name="IKEEXT_KERBEROS_AUTHENTICATION1", pack=False, align=None), "reservedAuthentication": SimStruct({"flags": SimTypeInt(signed=False, label="IKEEXT_RESERVED_AUTHENTICATION_FLAGS")}, name="IKEEXT_RESERVED_AUTHENTICATION0", pack=False, align=None), "ntlmV2Authentication": SimStruct({"flags": SimTypeInt(signed=False, label="UInt32")}, name="IKEEXT_NTLM_V2_AUTHENTICATION0", pack=False, align=None), "sslAuthentication": SimStruct({"inboundConfigType": SimTypeInt(signed=False, label="IKEEXT_CERT_CONFIG_TYPE"), "Anonymous1": SimUnion({"Anonymous1": SimStruct({"inboundRootArraySize": SimTypeInt(signed=False, label="UInt32"), "inboundRootCriteria": SimTypePointer(SimStruct({"certData": SimStruct({"size": SimTypeInt(signed=False, label="UInt32"), "data": SimTypePointer(SimTypeChar(label="Byte"), offset=0)}, name="FWP_BYTE_BLOB", pack=False, align=None), "certHash": SimStruct({"size": SimTypeInt(signed=False, label="UInt32"), "data": SimTypePointer(SimTypeChar(label="Byte"), offset=0)}, name="FWP_BYTE_BLOB", pack=False, align=None), "eku": SimTypePointer(SimStruct({"numEku": SimTypeInt(signed=False, label="UInt32"), "eku": SimTypePointer(SimTypePointer(SimTypeChar(label="Byte"), offset=0), offset=0)}, name="IKEEXT_CERT_EKUS0", pack=False, align=None), offset=0), "name": SimTypePointer(SimStruct({"nameType": SimTypeInt(signed=False, label="IKEEXT_CERT_CRITERIA_NAME_TYPE"), "certName": SimTypePointer(SimTypeChar(label="Char"), offset=0)}, name="IKEEXT_CERT_NAME0", pack=False, align=None), offset=0), "flags": SimTypeInt(signed=False, label="UInt32")}, name="IKEEXT_CERTIFICATE_CRITERIA0", pack=False, align=None), offset=0)}, name="_Anonymous1_e__Struct", pack=False, align=None), "Anonymous2": SimStruct({"inboundEnterpriseStoreArraySize": SimTypeInt(signed=False, label="UInt32"), "inboundEnterpriseStoreCriteria": SimTypePointer(SimStruct({"certData": SimStruct({"size": SimTypeInt(signed=False, label="UInt32"), "data": SimTypePointer(SimTypeChar(label="Byte"), offset=0)}, name="FWP_BYTE_BLOB", pack=False, align=None), "certHash": SimStruct({"size": SimTypeInt(signed=False, label="UInt32"), "data": SimTypePointer(SimTypeChar(label="Byte"), offset=0)}, name="FWP_BYTE_BLOB", pack=False, align=None), "eku": SimTypePointer(SimStruct({"numEku": SimTypeInt(signed=False, label="UInt32"), "eku": SimTypePointer(SimTypePointer(SimTypeChar(label="Byte"), offset=0), offset=0)}, name="IKEEXT_CERT_EKUS0", pack=False, align=None), offset=0), "name": SimTypePointer(SimStruct({"nameType": SimTypeInt(signed=False, label="IKEEXT_CERT_CRITERIA_NAME_TYPE"), "certName": SimTypePointer(SimTypeChar(label="Char"), offset=0)}, name="IKEEXT_CERT_NAME0", pack=False, align=None), offset=0), "flags": SimTypeInt(signed=False, label="UInt32")}, name="IKEEXT_CERTIFICATE_CRITERIA0", pack=False, align=None), offset=0)}, name="_Anonymous2_e__Struct", pack=False, align=None), "Anonymous3": SimStruct({"inboundRootStoreArraySize": SimTypeInt(signed=False, label="UInt32"), "inboundTrustedRootStoreCriteria": SimTypePointer(SimStruct({"certData": SimStruct({"size": SimTypeInt(signed=False, label="UInt32"), "data": SimTypePointer(SimTypeChar(label="Byte"), offset=0)}, name="FWP_BYTE_BLOB", pack=False, align=None), "certHash": SimStruct({"size": SimTypeInt(signed=False, label="UInt32"), "data": SimTypePointer(SimTypeChar(label="Byte"), offset=0)}, name="FWP_BYTE_BLOB", pack=False, align=None), "eku": SimTypePointer(SimStruct({"numEku": SimTypeInt(signed=False, label="UInt32"), "eku": SimTypePointer(SimTypePointer(SimTypeChar(label="Byte"), offset=0), offset=0)}, name="IKEEXT_CERT_EKUS0", pack=False, align=None), offset=0), "name": SimTypePointer(SimStruct({"nameType": SimTypeInt(signed=False, label="IKEEXT_CERT_CRITERIA_NAME_TYPE"), "certName": SimTypePointer(SimTypeChar(label="Char"), offset=0)}, name="IKEEXT_CERT_NAME0", pack=False, align=None), offset=0), "flags": SimTypeInt(signed=False, label="UInt32")}, name="IKEEXT_CERTIFICATE_CRITERIA0", pack=False, align=None), offset=0)}, name="_Anonymous3_e__Struct", pack=False, align=None)}, name="<anon>", label="None"), "outboundConfigType": SimTypeInt(signed=False, label="IKEEXT_CERT_CONFIG_TYPE"), "Anonymous2": SimUnion({"Anonymous1": SimStruct({"outboundRootArraySize": SimTypeInt(signed=False, label="UInt32"), "outboundRootCriteria": SimTypePointer(SimStruct({"certData": SimStruct({"size": SimTypeInt(signed=False, label="UInt32"), "data": SimTypePointer(SimTypeChar(label="Byte"), offset=0)}, name="FWP_BYTE_BLOB", pack=False, align=None), "certHash": SimStruct({"size": SimTypeInt(signed=False, label="UInt32"), "data": SimTypePointer(SimTypeChar(label="Byte"), offset=0)}, name="FWP_BYTE_BLOB", pack=False, align=None), "eku": SimTypePointer(SimStruct({"numEku": SimTypeInt(signed=False, label="UInt32"), "eku": SimTypePointer(SimTypePointer(SimTypeChar(label="Byte"), offset=0), offset=0)}, name="IKEEXT_CERT_EKUS0", pack=False, align=None), offset=0), "name": SimTypePointer(SimStruct({"nameType": SimTypeInt(signed=False, label="IKEEXT_CERT_CRITERIA_NAME_TYPE"), "certName": SimTypePointer(SimTypeChar(label="Char"), offset=0)}, name="IKEEXT_CERT_NAME0", pack=False, align=None), offset=0), "flags": SimTypeInt(signed=False, label="UInt32")}, name="IKEEXT_CERTIFICATE_CRITERIA0", pack=False, align=None), offset=0)}, name="_Anonymous1_e__Struct", pack=False, align=None), "Anonymous2": SimStruct({"outboundEnterpriseStoreArraySize": SimTypeInt(signed=False, label="UInt32"), "outboundEnterpriseStoreCriteria": SimTypePointer(SimStruct({"certData": SimStruct({"size": SimTypeInt(signed=False, label="UInt32"), "data": SimTypePointer(SimTypeChar(label="Byte"), offset=0)}, name="FWP_BYTE_BLOB", pack=False, align=None), "certHash": SimStruct({"size": SimTypeInt(signed=False, label="UInt32"), "data": SimTypePointer(SimTypeChar(label="Byte"), offset=0)}, name="FWP_BYTE_BLOB", pack=False, align=None), "eku": SimTypePointer(SimStruct({"numEku": SimTypeInt(signed=False, label="UInt32"), "eku": SimTypePointer(SimTypePointer(SimTypeChar(label="Byte"), offset=0), offset=0)}, name="IKEEXT_CERT_EKUS0", pack=False, align=None), offset=0), "name": SimTypePointer(SimStruct({"nameType": SimTypeInt(signed=False, label="IKEEXT_CERT_CRITERIA_NAME_TYPE"), "certName": SimTypePointer(SimTypeChar(label="Char"), offset=0)}, name="IKEEXT_CERT_NAME0", pack=False, align=None), offset=0), "flags": SimTypeInt(signed=False, label="UInt32")}, name="IKEEXT_CERTIFICATE_CRITERIA0", pack=False, align=None), offset=0)}, name="_Anonymous2_e__Struct", pack=False, align=None), "Anonymous3": SimStruct({"outboundRootStoreArraySize": SimTypeInt(signed=False, label="UInt32"), "outboundTrustedRootStoreCriteria": SimTypePointer(SimStruct({"certData": SimStruct({"size": SimTypeInt(signed=False, label="UInt32"), "data": SimTypePointer(SimTypeChar(label="Byte"), offset=0)}, name="FWP_BYTE_BLOB", pack=False, align=None), "certHash": SimStruct({"size": SimTypeInt(signed=False, label="UInt32"), "data": SimTypePointer(SimTypeChar(label="Byte"), offset=0)}, name="FWP_BYTE_BLOB", pack=False, align=None), "eku": SimTypePointer(SimStruct({"numEku": SimTypeInt(signed=False, label="UInt32"), "eku": SimTypePointer(SimTypePointer(SimTypeChar(label="Byte"), offset=0), offset=0)}, name="IKEEXT_CERT_EKUS0", pack=False, align=None), offset=0), "name": SimTypePointer(SimStruct({"nameType": SimTypeInt(signed=False, label="IKEEXT_CERT_CRITERIA_NAME_TYPE"), "certName": SimTypePointer(SimTypeChar(label="Char"), offset=0)}, name="IKEEXT_CERT_NAME0", pack=False, align=None), offset=0), "flags": SimTypeInt(signed=False, label="UInt32")}, name="IKEEXT_CERTIFICATE_CRITERIA0", pack=False, align=None), offset=0)}, name="_Anonymous3_e__Struct", pack=False, align=None)}, name="<anon>", label="None"), "flags": SimTypeInt(signed=False, label="IKEEXT_CERT_AUTH"), "localCertLocationUrl": SimStruct({"size": SimTypeInt(signed=False, label="UInt32"), "data": SimTypePointer(SimTypeChar(label="Byte"), offset=0)}, name="FWP_BYTE_BLOB", pack=False, align=None)}, name="IKEEXT_CERTIFICATE_AUTHENTICATION2", pack=False, align=None), "cgaAuthentication": SimStruct({"keyContainerName": SimTypePointer(SimTypeChar(label="Char"), offset=0), "cspName": SimTypePointer(SimTypeChar(label="Char"), offset=0), "cspType": SimTypeInt(signed=False, label="UInt32"), "cgaModifier": SimStruct({"byteArray16": SimTypeFixedSizeArray(SimTypeChar(label="Byte"), 16)}, name="FWP_BYTE_ARRAY16", pack=False, align=None), "cgaCollisionCount": SimTypeChar(label="Byte")}, name="IKEEXT_IPV6_CGA_AUTHENTICATION0", pack=False, align=None), "eapAuthentication": SimStruct({"flags": SimTypeInt(signed=False, label="IKEEXT_EAP_AUTHENTICATION_FLAGS")}, name="IKEEXT_EAP_AUTHENTICATION0", pack=False, align=None)}, name="<anon>", label="None")}, name="IKEEXT_AUTHENTICATION_METHOD2", pack=False, align=None), offset=0), "initiatorImpersonationType": SimTypeInt(signed=False, label="IKEEXT_AUTHENTICATION_IMPERSONATION_TYPE"), "numIkeProposals": SimTypeInt(signed=False, label="UInt32"), "ikeProposals": SimTypePointer(SimStruct({"cipherAlgorithm": SimStruct({"algoIdentifier": SimTypeInt(signed=False, label="IKEEXT_CIPHER_TYPE"), "keyLen": SimTypeInt(signed=False, label="UInt32"), "rounds": SimTypeInt(signed=False, label="UInt32")}, name="IKEEXT_CIPHER_ALGORITHM0", pack=False, align=None), "integrityAlgorithm": SimStruct({"algoIdentifier": SimTypeInt(signed=False, label="IKEEXT_INTEGRITY_TYPE")}, name="IKEEXT_INTEGRITY_ALGORITHM0", pack=False, align=None), "maxLifetimeSeconds": SimTypeInt(signed=False, label="UInt32"), "dhGroup": SimTypeInt(signed=False, label="IKEEXT_DH_GROUP"), "quickModeLimit": SimTypeInt(signed=False, label="UInt32")}, name="IKEEXT_PROPOSAL0", pack=False, align=None), offset=0), "flags": SimTypeInt(signed=False, label="IKEEXT_POLICY_FLAG"), "maxDynamicFilters": SimTypeInt(signed=False, label="UInt32"), "retransmitDurationSecs": SimTypeInt(signed=False, label="UInt32")}, name="IKEEXT_POLICY2", pack=False, align=None), offset=0), "idpOptions": SimTypePointer(SimStruct({"stateIdleTimeoutSeconds": SimTypeInt(signed=False, label="UInt32"), "perIPRateLimitQueueIdleTimeoutSeconds": SimTypeInt(signed=False, label="UInt32"), "ipV6IPsecUnauthDscp": SimTypeChar(label="Byte"), "ipV6IPsecUnauthRateLimitBytesPerSec": SimTypeInt(signed=False, label="UInt32"), "ipV6IPsecUnauthPerIPRateLimitBytesPerSec": SimTypeInt(signed=False, label="UInt32"), "ipV6IPsecAuthDscp": SimTypeChar(label="Byte"), "ipV6IPsecAuthRateLimitBytesPerSec": SimTypeInt(signed=False, label="UInt32"), "icmpV6Dscp": SimTypeChar(label="Byte"), "icmpV6RateLimitBytesPerSec": SimTypeInt(signed=False, label="UInt32"), "ipV6FilterExemptDscp": SimTypeChar(label="Byte"), "ipV6FilterExemptRateLimitBytesPerSec": SimTypeInt(signed=False, label="UInt32"), "defBlockExemptDscp": SimTypeChar(label="Byte"), "defBlockExemptRateLimitBytesPerSec": SimTypeInt(signed=False, label="UInt32"), "maxStateEntries": SimTypeInt(signed=False, label="UInt32"), "maxPerIPRateLimitQueues": SimTypeInt(signed=False, label="UInt32"), "flags": SimTypeInt(signed=False, label="IPSEC_DOSP_FLAGS"), "numPublicIFLuids": SimTypeInt(signed=False, label="UInt32"), "publicIFLuids": SimTypePointer(SimTypeLongLong(signed=False, label="UInt64"), offset=0), "numInternalIFLuids": SimTypeInt(signed=False, label="UInt32"), "internalIFLuids": SimTypePointer(SimTypeLongLong(signed=False, label="UInt64"), offset=0), "publicV6AddrMask": SimStruct({"addr": SimTypeFixedSizeArray(SimTypeChar(label="Byte"), 16), "prefixLength": SimTypeChar(label="Byte")}, name="FWP_V6_ADDR_AND_MASK", pack=False, align=None), "internalV6AddrMask": SimStruct({"addr": SimTypeFixedSizeArray(SimTypeChar(label="Byte"), 16), "prefixLength": SimTypeChar(label="Byte")}, name="FWP_V6_ADDR_AND_MASK", pack=False, align=None)}, name="IPSEC_DOSP_OPTIONS0", pack=False, align=None), offset=0)}, name="<anon>", label="None"), "providerContextId": SimTypeLongLong(signed=False, label="UInt64")}, name="FWPM_PROVIDER_CONTEXT3_", pack=False, align=None), offset=0), offset=0)], SimTypeInt(signed=False, label="UInt32"), arg_names=["engineHandle", "key", "providerContext"]),
        # 
        'FwpmProviderContextCreateEnumHandle0': SimTypeFunction([SimTypePointer(SimTypeInt(signed=True, label="Int"), label="IntPtr", offset=0), SimTypePointer(SimStruct({"providerKey": SimTypePointer(SimTypeBottom(label="Guid"), offset=0), "providerContextType": SimTypeInt(signed=False, label="FWPM_PROVIDER_CONTEXT_TYPE")}, name="FWPM_PROVIDER_CONTEXT_ENUM_TEMPLATE0", pack=False, align=None), offset=0), SimTypePointer(SimTypePointer(SimTypeInt(signed=True, label="Int"), label="IntPtr", offset=0), offset=0)], SimTypeInt(signed=False, label="UInt32"), arg_names=["engineHandle", "enumTemplate", "enumHandle"]),
        # 
        'FwpmProviderContextEnum0': SimTypeFunction([SimTypePointer(SimTypeInt(signed=True, label="Int"), label="IntPtr", offset=0), SimTypePointer(SimTypeInt(signed=True, label="Int"), label="IntPtr", offset=0), SimTypeInt(signed=False, label="UInt32"), SimTypePointer(SimTypePointer(SimTypePointer(SimStruct({"providerContextKey": SimTypeBottom(label="Guid"), "displayData": SimStruct({"name": SimTypePointer(SimTypeChar(label="Char"), offset=0), "description": SimTypePointer(SimTypeChar(label="Char"), offset=0)}, name="FWPM_DISPLAY_DATA0", pack=False, align=None), "flags": SimTypeInt(signed=False, label="UInt32"), "providerKey": SimTypePointer(SimTypeBottom(label="Guid"), offset=0), "providerData": SimStruct({"size": SimTypeInt(signed=False, label="UInt32"), "data": SimTypePointer(SimTypeChar(label="Byte"), offset=0)}, name="FWP_BYTE_BLOB", pack=False, align=None), "type": SimTypeInt(signed=False, label="FWPM_PROVIDER_CONTEXT_TYPE"), "Anonymous": SimUnion({"keyingPolicy": SimTypePointer(SimStruct({"numKeyMods": SimTypeInt(signed=False, label="UInt32"), "keyModKeys": SimTypePointer(SimTypeBottom(label="Guid"), offset=0)}, name="IPSEC_KEYING_POLICY0", pack=False, align=None), offset=0), "ikeQmTransportPolicy": SimTypePointer(SimStruct({"numIpsecProposals": SimTypeInt(signed=False, label="UInt32"), "ipsecProposals": SimTypePointer(SimStruct({"lifetime": SimStruct({"lifetimeSeconds": SimTypeInt(signed=False, label="UInt32"), "lifetimeKilobytes": SimTypeInt(signed=False, label="UInt32"), "lifetimePackets": SimTypeInt(signed=False, label="UInt32")}, name="IPSEC_SA_LIFETIME0", pack=False, align=None), "numSaTransforms": SimTypeInt(signed=False, label="UInt32"), "saTransforms": SimTypePointer(SimStruct({"ipsecTransformType": SimTypeInt(signed=False, label="IPSEC_TRANSFORM_TYPE"), "Anonymous": SimUnion({"ahTransform": SimTypePointer(SimStruct({"authTransformId": SimStruct({"authType": SimTypeInt(signed=False, label="IPSEC_AUTH_TYPE"), "authConfig": SimTypeChar(label="Byte")}, name="IPSEC_AUTH_TRANSFORM_ID0", pack=False, align=None), "cryptoModuleId": SimTypePointer(SimTypeBottom(label="Guid"), offset=0)}, name="IPSEC_AUTH_TRANSFORM0", pack=False, align=None), offset=0), "espAuthTransform": SimTypePointer(SimStruct({"authTransformId": SimStruct({"authType": SimTypeInt(signed=False, label="IPSEC_AUTH_TYPE"), "authConfig": SimTypeChar(label="Byte")}, name="IPSEC_AUTH_TRANSFORM_ID0", pack=False, align=None), "cryptoModuleId": SimTypePointer(SimTypeBottom(label="Guid"), offset=0)}, name="IPSEC_AUTH_TRANSFORM0", pack=False, align=None), offset=0), "espCipherTransform": SimTypePointer(SimStruct({"cipherTransformId": SimStruct({"cipherType": SimTypeInt(signed=False, label="IPSEC_CIPHER_TYPE"), "cipherConfig": SimTypeChar(label="Byte")}, name="IPSEC_CIPHER_TRANSFORM_ID0", pack=False, align=None), "cryptoModuleId": SimTypePointer(SimTypeBottom(label="Guid"), offset=0)}, name="IPSEC_CIPHER_TRANSFORM0", pack=False, align=None), offset=0), "espAuthAndCipherTransform": SimTypePointer(SimStruct({"authTransform": SimStruct({"authTransformId": SimStruct({"authType": SimTypeInt(signed=False, label="IPSEC_AUTH_TYPE"), "authConfig": SimTypeChar(label="Byte")}, name="IPSEC_AUTH_TRANSFORM_ID0", pack=False, align=None), "cryptoModuleId": SimTypePointer(SimTypeBottom(label="Guid"), offset=0)}, name="IPSEC_AUTH_TRANSFORM0", pack=False, align=None), "cipherTransform": SimStruct({"cipherTransformId": SimStruct({"cipherType": SimTypeInt(signed=False, label="IPSEC_CIPHER_TYPE"), "cipherConfig": SimTypeChar(label="Byte")}, name="IPSEC_CIPHER_TRANSFORM_ID0", pack=False, align=None), "cryptoModuleId": SimTypePointer(SimTypeBottom(label="Guid"), offset=0)}, name="IPSEC_CIPHER_TRANSFORM0", pack=False, align=None)}, name="IPSEC_AUTH_AND_CIPHER_TRANSFORM0", pack=False, align=None), offset=0), "espAuthFwTransform": SimTypePointer(SimStruct({"authTransformId": SimStruct({"authType": SimTypeInt(signed=False, label="IPSEC_AUTH_TYPE"), "authConfig": SimTypeChar(label="Byte")}, name="IPSEC_AUTH_TRANSFORM_ID0", pack=False, align=None), "cryptoModuleId": SimTypePointer(SimTypeBottom(label="Guid"), offset=0)}, name="IPSEC_AUTH_TRANSFORM0", pack=False, align=None), offset=0)}, name="<anon>", label="None")}, name="IPSEC_SA_TRANSFORM0", pack=False, align=None), offset=0), "pfsGroup": SimTypeInt(signed=False, label="IPSEC_PFS_GROUP")}, name="IPSEC_PROPOSAL0", pack=False, align=None), offset=0), "flags": SimTypeInt(signed=False, label="IPSEC_POLICY_FLAG"), "ndAllowClearTimeoutSeconds": SimTypeInt(signed=False, label="UInt32"), "saIdleTimeout": SimStruct({"idleTimeoutSeconds": SimTypeInt(signed=False, label="UInt32"), "idleTimeoutSecondsFailOver": SimTypeInt(signed=False, label="UInt32")}, name="IPSEC_SA_IDLE_TIMEOUT0", pack=False, align=None), "emPolicy": SimTypePointer(SimStruct({"numAuthenticationMethods": SimTypeInt(signed=False, label="UInt32"), "authenticationMethods": SimTypePointer(SimStruct({"authenticationMethodType": SimTypeInt(signed=False, label="IKEEXT_AUTHENTICATION_METHOD_TYPE"), "Anonymous": SimUnion({"presharedKeyAuthentication": SimStruct({"presharedKey": SimStruct({"size": SimTypeInt(signed=False, label="UInt32"), "data": SimTypePointer(SimTypeChar(label="Byte"), offset=0)}, name="FWP_BYTE_BLOB", pack=False, align=None)}, name="IKEEXT_PRESHARED_KEY_AUTHENTICATION0", pack=False, align=None), "certificateAuthentication": SimStruct({"inboundConfigType": SimTypeInt(signed=False, label="IKEEXT_CERT_CONFIG_TYPE"), "Anonymous1": SimUnion({"Anonymous": SimStruct({"inboundRootArraySize": SimTypeInt(signed=False, label="UInt32"), "inboundRootArray": SimTypePointer(SimStruct({"certData": SimStruct({"size": SimTypeInt(signed=False, label="UInt32"), "data": SimTypePointer(SimTypeChar(label="Byte"), offset=0)}, name="FWP_BYTE_BLOB", pack=False, align=None), "flags": SimTypeInt(signed=False, label="IKEEXT_CERT_FLAGS")}, name="IKEEXT_CERT_ROOT_CONFIG0", pack=False, align=None), offset=0)}, name="_Anonymous_e__Struct", pack=False, align=None), "inboundEnterpriseStoreConfig": SimTypePointer(SimStruct({"certData": SimStruct({"size": SimTypeInt(signed=False, label="UInt32"), "data": SimTypePointer(SimTypeChar(label="Byte"), offset=0)}, name="FWP_BYTE_BLOB", pack=False, align=None), "flags": SimTypeInt(signed=False, label="IKEEXT_CERT_FLAGS")}, name="IKEEXT_CERT_ROOT_CONFIG0", pack=False, align=None), offset=0), "inboundTrustedRootStoreConfig": SimTypePointer(SimStruct({"certData": SimStruct({"size": SimTypeInt(signed=False, label="UInt32"), "data": SimTypePointer(SimTypeChar(label="Byte"), offset=0)}, name="FWP_BYTE_BLOB", pack=False, align=None), "flags": SimTypeInt(signed=False, label="IKEEXT_CERT_FLAGS")}, name="IKEEXT_CERT_ROOT_CONFIG0", pack=False, align=None), offset=0)}, name="<anon>", label="None"), "outboundConfigType": SimTypeInt(signed=False, label="IKEEXT_CERT_CONFIG_TYPE"), "Anonymous2": SimUnion({"Anonymous": SimStruct({"outboundRootArraySize": SimTypeInt(signed=False, label="UInt32"), "outboundRootArray": SimTypePointer(SimStruct({"certData": SimStruct({"size": SimTypeInt(signed=False, label="UInt32"), "data": SimTypePointer(SimTypeChar(label="Byte"), offset=0)}, name="FWP_BYTE_BLOB", pack=False, align=None), "flags": SimTypeInt(signed=False, label="IKEEXT_CERT_FLAGS")}, name="IKEEXT_CERT_ROOT_CONFIG0", pack=False, align=None), offset=0)}, name="_Anonymous_e__Struct", pack=False, align=None), "outboundEnterpriseStoreConfig": SimTypePointer(SimStruct({"certData": SimStruct({"size": SimTypeInt(signed=False, label="UInt32"), "data": SimTypePointer(SimTypeChar(label="Byte"), offset=0)}, name="FWP_BYTE_BLOB", pack=False, align=None), "flags": SimTypeInt(signed=False, label="IKEEXT_CERT_FLAGS")}, name="IKEEXT_CERT_ROOT_CONFIG0", pack=False, align=None), offset=0), "outboundTrustedRootStoreConfig": SimTypePointer(SimStruct({"certData": SimStruct({"size": SimTypeInt(signed=False, label="UInt32"), "data": SimTypePointer(SimTypeChar(label="Byte"), offset=0)}, name="FWP_BYTE_BLOB", pack=False, align=None), "flags": SimTypeInt(signed=False, label="IKEEXT_CERT_FLAGS")}, name="IKEEXT_CERT_ROOT_CONFIG0", pack=False, align=None), offset=0)}, name="<anon>", label="None"), "flags": SimTypeInt(signed=False, label="IKEEXT_CERT_AUTH")}, name="IKEEXT_CERTIFICATE_AUTHENTICATION0", pack=False, align=None), "kerberosAuthentication": SimStruct({"flags": SimTypeInt(signed=False, label="IKEEXT_KERBEROS_AUTHENTICATION_FLAGS")}, name="IKEEXT_KERBEROS_AUTHENTICATION0", pack=False, align=None), "ntlmV2Authentication": SimStruct({"flags": SimTypeInt(signed=False, label="UInt32")}, name="IKEEXT_NTLM_V2_AUTHENTICATION0", pack=False, align=None), "sslAuthentication": SimStruct({"inboundConfigType": SimTypeInt(signed=False, label="IKEEXT_CERT_CONFIG_TYPE"), "Anonymous1": SimUnion({"Anonymous": SimStruct({"inboundRootArraySize": SimTypeInt(signed=False, label="UInt32"), "inboundRootArray": SimTypePointer(SimStruct({"certData": SimStruct({"size": SimTypeInt(signed=False, label="UInt32"), "data": SimTypePointer(SimTypeChar(label="Byte"), offset=0)}, name="FWP_BYTE_BLOB", pack=False, align=None), "flags": SimTypeInt(signed=False, label="IKEEXT_CERT_FLAGS")}, name="IKEEXT_CERT_ROOT_CONFIG0", pack=False, align=None), offset=0)}, name="_Anonymous_e__Struct", pack=False, align=None), "inboundEnterpriseStoreConfig": SimTypePointer(SimStruct({"certData": SimStruct({"size": SimTypeInt(signed=False, label="UInt32"), "data": SimTypePointer(SimTypeChar(label="Byte"), offset=0)}, name="FWP_BYTE_BLOB", pack=False, align=None), "flags": SimTypeInt(signed=False, label="IKEEXT_CERT_FLAGS")}, name="IKEEXT_CERT_ROOT_CONFIG0", pack=False, align=None), offset=0), "inboundTrustedRootStoreConfig": SimTypePointer(SimStruct({"certData": SimStruct({"size": SimTypeInt(signed=False, label="UInt32"), "data": SimTypePointer(SimTypeChar(label="Byte"), offset=0)}, name="FWP_BYTE_BLOB", pack=False, align=None), "flags": SimTypeInt(signed=False, label="IKEEXT_CERT_FLAGS")}, name="IKEEXT_CERT_ROOT_CONFIG0", pack=False, align=None), offset=0)}, name="<anon>", label="None"), "outboundConfigType": SimTypeInt(signed=False, label="IKEEXT_CERT_CONFIG_TYPE"), "Anonymous2": SimUnion({"Anonymous": SimStruct({"outboundRootArraySize": SimTypeInt(signed=False, label="UInt32"), "outboundRootArray": SimTypePointer(SimStruct({"certData": SimStruct({"size": SimTypeInt(signed=False, label="UInt32"), "data": SimTypePointer(SimTypeChar(label="Byte"), offset=0)}, name="FWP_BYTE_BLOB", pack=False, align=None), "flags": SimTypeInt(signed=False, label="IKEEXT_CERT_FLAGS")}, name="IKEEXT_CERT_ROOT_CONFIG0", pack=False, align=None), offset=0)}, name="_Anonymous_e__Struct", pack=False, align=None), "outboundEnterpriseStoreConfig": SimTypePointer(SimStruct({"certData": SimStruct({"size": SimTypeInt(signed=False, label="UInt32"), "data": SimTypePointer(SimTypeChar(label="Byte"), offset=0)}, name="FWP_BYTE_BLOB", pack=False, align=None), "flags": SimTypeInt(signed=False, label="IKEEXT_CERT_FLAGS")}, name="IKEEXT_CERT_ROOT_CONFIG0", pack=False, align=None), offset=0), "outboundTrustedRootStoreConfig": SimTypePointer(SimStruct({"certData": SimStruct({"size": SimTypeInt(signed=False, label="UInt32"), "data": SimTypePointer(SimTypeChar(label="Byte"), offset=0)}, name="FWP_BYTE_BLOB", pack=False, align=None), "flags": SimTypeInt(signed=False, label="IKEEXT_CERT_FLAGS")}, name="IKEEXT_CERT_ROOT_CONFIG0", pack=False, align=None), offset=0)}, name="<anon>", label="None"), "flags": SimTypeInt(signed=False, label="IKEEXT_CERT_AUTH")}, name="IKEEXT_CERTIFICATE_AUTHENTICATION0", pack=False, align=None), "cgaAuthentication": SimStruct({"keyContainerName": SimTypePointer(SimTypeChar(label="Char"), offset=0), "cspName": SimTypePointer(SimTypeChar(label="Char"), offset=0), "cspType": SimTypeInt(signed=False, label="UInt32"), "cgaModifier": SimStruct({"byteArray16": SimTypeFixedSizeArray(SimTypeChar(label="Byte"), 16)}, name="FWP_BYTE_ARRAY16", pack=False, align=None), "cgaCollisionCount": SimTypeChar(label="Byte")}, name="IKEEXT_IPV6_CGA_AUTHENTICATION0", pack=False, align=None)}, name="<anon>", label="None")}, name="IKEEXT_AUTHENTICATION_METHOD0", pack=False, align=None), offset=0), "initiatorImpersonationType": SimTypeInt(signed=False, label="IKEEXT_AUTHENTICATION_IMPERSONATION_TYPE")}, name="IKEEXT_EM_POLICY0", pack=False, align=None), offset=0)}, name="IPSEC_TRANSPORT_POLICY0", pack=False, align=None), offset=0), "ikeQmTunnelPolicy": SimTypePointer(SimStruct({"flags": SimTypeInt(signed=False, label="IPSEC_POLICY_FLAG"), "numIpsecProposals": SimTypeInt(signed=False, label="UInt32"), "ipsecProposals": SimTypePointer(SimStruct({"lifetime": SimStruct({"lifetimeSeconds": SimTypeInt(signed=False, label="UInt32"), "lifetimeKilobytes": SimTypeInt(signed=False, label="UInt32"), "lifetimePackets": SimTypeInt(signed=False, label="UInt32")}, name="IPSEC_SA_LIFETIME0", pack=False, align=None), "numSaTransforms": SimTypeInt(signed=False, label="UInt32"), "saTransforms": SimTypePointer(SimStruct({"ipsecTransformType": SimTypeInt(signed=False, label="IPSEC_TRANSFORM_TYPE"), "Anonymous": SimUnion({"ahTransform": SimTypePointer(SimStruct({"authTransformId": SimStruct({"authType": SimTypeInt(signed=False, label="IPSEC_AUTH_TYPE"), "authConfig": SimTypeChar(label="Byte")}, name="IPSEC_AUTH_TRANSFORM_ID0", pack=False, align=None), "cryptoModuleId": SimTypePointer(SimTypeBottom(label="Guid"), offset=0)}, name="IPSEC_AUTH_TRANSFORM0", pack=False, align=None), offset=0), "espAuthTransform": SimTypePointer(SimStruct({"authTransformId": SimStruct({"authType": SimTypeInt(signed=False, label="IPSEC_AUTH_TYPE"), "authConfig": SimTypeChar(label="Byte")}, name="IPSEC_AUTH_TRANSFORM_ID0", pack=False, align=None), "cryptoModuleId": SimTypePointer(SimTypeBottom(label="Guid"), offset=0)}, name="IPSEC_AUTH_TRANSFORM0", pack=False, align=None), offset=0), "espCipherTransform": SimTypePointer(SimStruct({"cipherTransformId": SimStruct({"cipherType": SimTypeInt(signed=False, label="IPSEC_CIPHER_TYPE"), "cipherConfig": SimTypeChar(label="Byte")}, name="IPSEC_CIPHER_TRANSFORM_ID0", pack=False, align=None), "cryptoModuleId": SimTypePointer(SimTypeBottom(label="Guid"), offset=0)}, name="IPSEC_CIPHER_TRANSFORM0", pack=False, align=None), offset=0), "espAuthAndCipherTransform": SimTypePointer(SimStruct({"authTransform": SimStruct({"authTransformId": SimStruct({"authType": SimTypeInt(signed=False, label="IPSEC_AUTH_TYPE"), "authConfig": SimTypeChar(label="Byte")}, name="IPSEC_AUTH_TRANSFORM_ID0", pack=False, align=None), "cryptoModuleId": SimTypePointer(SimTypeBottom(label="Guid"), offset=0)}, name="IPSEC_AUTH_TRANSFORM0", pack=False, align=None), "cipherTransform": SimStruct({"cipherTransformId": SimStruct({"cipherType": SimTypeInt(signed=False, label="IPSEC_CIPHER_TYPE"), "cipherConfig": SimTypeChar(label="Byte")}, name="IPSEC_CIPHER_TRANSFORM_ID0", pack=False, align=None), "cryptoModuleId": SimTypePointer(SimTypeBottom(label="Guid"), offset=0)}, name="IPSEC_CIPHER_TRANSFORM0", pack=False, align=None)}, name="IPSEC_AUTH_AND_CIPHER_TRANSFORM0", pack=False, align=None), offset=0), "espAuthFwTransform": SimTypePointer(SimStruct({"authTransformId": SimStruct({"authType": SimTypeInt(signed=False, label="IPSEC_AUTH_TYPE"), "authConfig": SimTypeChar(label="Byte")}, name="IPSEC_AUTH_TRANSFORM_ID0", pack=False, align=None), "cryptoModuleId": SimTypePointer(SimTypeBottom(label="Guid"), offset=0)}, name="IPSEC_AUTH_TRANSFORM0", pack=False, align=None), offset=0)}, name="<anon>", label="None")}, name="IPSEC_SA_TRANSFORM0", pack=False, align=None), offset=0), "pfsGroup": SimTypeInt(signed=False, label="IPSEC_PFS_GROUP")}, name="IPSEC_PROPOSAL0", pack=False, align=None), offset=0), "tunnelEndpoints": SimStruct({"ipVersion": SimTypeInt(signed=False, label="FWP_IP_VERSION"), "Anonymous1": SimUnion({"localV4Address": SimTypeInt(signed=False, label="UInt32"), "localV6Address": SimTypeFixedSizeArray(SimTypeChar(label="Byte"), 16)}, name="<anon>", label="None"), "Anonymous2": SimUnion({"remoteV4Address": SimTypeInt(signed=False, label="UInt32"), "remoteV6Address": SimTypeFixedSizeArray(SimTypeChar(label="Byte"), 16)}, name="<anon>", label="None")}, name="IPSEC_TUNNEL_ENDPOINTS0", pack=False, align=None), "saIdleTimeout": SimStruct({"idleTimeoutSeconds": SimTypeInt(signed=False, label="UInt32"), "idleTimeoutSecondsFailOver": SimTypeInt(signed=False, label="UInt32")}, name="IPSEC_SA_IDLE_TIMEOUT0", pack=False, align=None), "emPolicy": SimTypePointer(SimStruct({"numAuthenticationMethods": SimTypeInt(signed=False, label="UInt32"), "authenticationMethods": SimTypePointer(SimStruct({"authenticationMethodType": SimTypeInt(signed=False, label="IKEEXT_AUTHENTICATION_METHOD_TYPE"), "Anonymous": SimUnion({"presharedKeyAuthentication": SimStruct({"presharedKey": SimStruct({"size": SimTypeInt(signed=False, label="UInt32"), "data": SimTypePointer(SimTypeChar(label="Byte"), offset=0)}, name="FWP_BYTE_BLOB", pack=False, align=None)}, name="IKEEXT_PRESHARED_KEY_AUTHENTICATION0", pack=False, align=None), "certificateAuthentication": SimStruct({"inboundConfigType": SimTypeInt(signed=False, label="IKEEXT_CERT_CONFIG_TYPE"), "Anonymous1": SimUnion({"Anonymous": SimStruct({"inboundRootArraySize": SimTypeInt(signed=False, label="UInt32"), "inboundRootArray": SimTypePointer(SimStruct({"certData": SimStruct({"size": SimTypeInt(signed=False, label="UInt32"), "data": SimTypePointer(SimTypeChar(label="Byte"), offset=0)}, name="FWP_BYTE_BLOB", pack=False, align=None), "flags": SimTypeInt(signed=False, label="IKEEXT_CERT_FLAGS")}, name="IKEEXT_CERT_ROOT_CONFIG0", pack=False, align=None), offset=0)}, name="_Anonymous_e__Struct", pack=False, align=None), "inboundEnterpriseStoreConfig": SimTypePointer(SimStruct({"certData": SimStruct({"size": SimTypeInt(signed=False, label="UInt32"), "data": SimTypePointer(SimTypeChar(label="Byte"), offset=0)}, name="FWP_BYTE_BLOB", pack=False, align=None), "flags": SimTypeInt(signed=False, label="IKEEXT_CERT_FLAGS")}, name="IKEEXT_CERT_ROOT_CONFIG0", pack=False, align=None), offset=0), "inboundTrustedRootStoreConfig": SimTypePointer(SimStruct({"certData": SimStruct({"size": SimTypeInt(signed=False, label="UInt32"), "data": SimTypePointer(SimTypeChar(label="Byte"), offset=0)}, name="FWP_BYTE_BLOB", pack=False, align=None), "flags": SimTypeInt(signed=False, label="IKEEXT_CERT_FLAGS")}, name="IKEEXT_CERT_ROOT_CONFIG0", pack=False, align=None), offset=0)}, name="<anon>", label="None"), "outboundConfigType": SimTypeInt(signed=False, label="IKEEXT_CERT_CONFIG_TYPE"), "Anonymous2": SimUnion({"Anonymous": SimStruct({"outboundRootArraySize": SimTypeInt(signed=False, label="UInt32"), "outboundRootArray": SimTypePointer(SimStruct({"certData": SimStruct({"size": SimTypeInt(signed=False, label="UInt32"), "data": SimTypePointer(SimTypeChar(label="Byte"), offset=0)}, name="FWP_BYTE_BLOB", pack=False, align=None), "flags": SimTypeInt(signed=False, label="IKEEXT_CERT_FLAGS")}, name="IKEEXT_CERT_ROOT_CONFIG0", pack=False, align=None), offset=0)}, name="_Anonymous_e__Struct", pack=False, align=None), "outboundEnterpriseStoreConfig": SimTypePointer(SimStruct({"certData": SimStruct({"size": SimTypeInt(signed=False, label="UInt32"), "data": SimTypePointer(SimTypeChar(label="Byte"), offset=0)}, name="FWP_BYTE_BLOB", pack=False, align=None), "flags": SimTypeInt(signed=False, label="IKEEXT_CERT_FLAGS")}, name="IKEEXT_CERT_ROOT_CONFIG0", pack=False, align=None), offset=0), "outboundTrustedRootStoreConfig": SimTypePointer(SimStruct({"certData": SimStruct({"size": SimTypeInt(signed=False, label="UInt32"), "data": SimTypePointer(SimTypeChar(label="Byte"), offset=0)}, name="FWP_BYTE_BLOB", pack=False, align=None), "flags": SimTypeInt(signed=False, label="IKEEXT_CERT_FLAGS")}, name="IKEEXT_CERT_ROOT_CONFIG0", pack=False, align=None), offset=0)}, name="<anon>", label="None"), "flags": SimTypeInt(signed=False, label="IKEEXT_CERT_AUTH")}, name="IKEEXT_CERTIFICATE_AUTHENTICATION0", pack=False, align=None), "kerberosAuthentication": SimStruct({"flags": SimTypeInt(signed=False, label="IKEEXT_KERBEROS_AUTHENTICATION_FLAGS")}, name="IKEEXT_KERBEROS_AUTHENTICATION0", pack=False, align=None), "ntlmV2Authentication": SimStruct({"flags": SimTypeInt(signed=False, label="UInt32")}, name="IKEEXT_NTLM_V2_AUTHENTICATION0", pack=False, align=None), "sslAuthentication": SimStruct({"inboundConfigType": SimTypeInt(signed=False, label="IKEEXT_CERT_CONFIG_TYPE"), "Anonymous1": SimUnion({"Anonymous": SimStruct({"inboundRootArraySize": SimTypeInt(signed=False, label="UInt32"), "inboundRootArray": SimTypePointer(SimStruct({"certData": SimStruct({"size": SimTypeInt(signed=False, label="UInt32"), "data": SimTypePointer(SimTypeChar(label="Byte"), offset=0)}, name="FWP_BYTE_BLOB", pack=False, align=None), "flags": SimTypeInt(signed=False, label="IKEEXT_CERT_FLAGS")}, name="IKEEXT_CERT_ROOT_CONFIG0", pack=False, align=None), offset=0)}, name="_Anonymous_e__Struct", pack=False, align=None), "inboundEnterpriseStoreConfig": SimTypePointer(SimStruct({"certData": SimStruct({"size": SimTypeInt(signed=False, label="UInt32"), "data": SimTypePointer(SimTypeChar(label="Byte"), offset=0)}, name="FWP_BYTE_BLOB", pack=False, align=None), "flags": SimTypeInt(signed=False, label="IKEEXT_CERT_FLAGS")}, name="IKEEXT_CERT_ROOT_CONFIG0", pack=False, align=None), offset=0), "inboundTrustedRootStoreConfig": SimTypePointer(SimStruct({"certData": SimStruct({"size": SimTypeInt(signed=False, label="UInt32"), "data": SimTypePointer(SimTypeChar(label="Byte"), offset=0)}, name="FWP_BYTE_BLOB", pack=False, align=None), "flags": SimTypeInt(signed=False, label="IKEEXT_CERT_FLAGS")}, name="IKEEXT_CERT_ROOT_CONFIG0", pack=False, align=None), offset=0)}, name="<anon>", label="None"), "outboundConfigType": SimTypeInt(signed=False, label="IKEEXT_CERT_CONFIG_TYPE"), "Anonymous2": SimUnion({"Anonymous": SimStruct({"outboundRootArraySize": SimTypeInt(signed=False, label="UInt32"), "outboundRootArray": SimTypePointer(SimStruct({"certData": SimStruct({"size": SimTypeInt(signed=False, label="UInt32"), "data": SimTypePointer(SimTypeChar(label="Byte"), offset=0)}, name="FWP_BYTE_BLOB", pack=False, align=None), "flags": SimTypeInt(signed=False, label="IKEEXT_CERT_FLAGS")}, name="IKEEXT_CERT_ROOT_CONFIG0", pack=False, align=None), offset=0)}, name="_Anonymous_e__Struct", pack=False, align=None), "outboundEnterpriseStoreConfig": SimTypePointer(SimStruct({"certData": SimStruct({"size": SimTypeInt(signed=False, label="UInt32"), "data": SimTypePointer(SimTypeChar(label="Byte"), offset=0)}, name="FWP_BYTE_BLOB", pack=False, align=None), "flags": SimTypeInt(signed=False, label="IKEEXT_CERT_FLAGS")}, name="IKEEXT_CERT_ROOT_CONFIG0", pack=False, align=None), offset=0), "outboundTrustedRootStoreConfig": SimTypePointer(SimStruct({"certData": SimStruct({"size": SimTypeInt(signed=False, label="UInt32"), "data": SimTypePointer(SimTypeChar(label="Byte"), offset=0)}, name="FWP_BYTE_BLOB", pack=False, align=None), "flags": SimTypeInt(signed=False, label="IKEEXT_CERT_FLAGS")}, name="IKEEXT_CERT_ROOT_CONFIG0", pack=False, align=None), offset=0)}, name="<anon>", label="None"), "flags": SimTypeInt(signed=False, label="IKEEXT_CERT_AUTH")}, name="IKEEXT_CERTIFICATE_AUTHENTICATION0", pack=False, align=None), "cgaAuthentication": SimStruct({"keyContainerName": SimTypePointer(SimTypeChar(label="Char"), offset=0), "cspName": SimTypePointer(SimTypeChar(label="Char"), offset=0), "cspType": SimTypeInt(signed=False, label="UInt32"), "cgaModifier": SimStruct({"byteArray16": SimTypeFixedSizeArray(SimTypeChar(label="Byte"), 16)}, name="FWP_BYTE_ARRAY16", pack=False, align=None), "cgaCollisionCount": SimTypeChar(label="Byte")}, name="IKEEXT_IPV6_CGA_AUTHENTICATION0", pack=False, align=None)}, name="<anon>", label="None")}, name="IKEEXT_AUTHENTICATION_METHOD0", pack=False, align=None), offset=0), "initiatorImpersonationType": SimTypeInt(signed=False, label="IKEEXT_AUTHENTICATION_IMPERSONATION_TYPE")}, name="IKEEXT_EM_POLICY0", pack=False, align=None), offset=0)}, name="IPSEC_TUNNEL_POLICY0", pack=False, align=None), offset=0), "authipQmTransportPolicy": SimTypePointer(SimStruct({"numIpsecProposals": SimTypeInt(signed=False, label="UInt32"), "ipsecProposals": SimTypePointer(SimStruct({"lifetime": SimStruct({"lifetimeSeconds": SimTypeInt(signed=False, label="UInt32"), "lifetimeKilobytes": SimTypeInt(signed=False, label="UInt32"), "lifetimePackets": SimTypeInt(signed=False, label="UInt32")}, name="IPSEC_SA_LIFETIME0", pack=False, align=None), "numSaTransforms": SimTypeInt(signed=False, label="UInt32"), "saTransforms": SimTypePointer(SimStruct({"ipsecTransformType": SimTypeInt(signed=False, label="IPSEC_TRANSFORM_TYPE"), "Anonymous": SimUnion({"ahTransform": SimTypePointer(SimStruct({"authTransformId": SimStruct({"authType": SimTypeInt(signed=False, label="IPSEC_AUTH_TYPE"), "authConfig": SimTypeChar(label="Byte")}, name="IPSEC_AUTH_TRANSFORM_ID0", pack=False, align=None), "cryptoModuleId": SimTypePointer(SimTypeBottom(label="Guid"), offset=0)}, name="IPSEC_AUTH_TRANSFORM0", pack=False, align=None), offset=0), "espAuthTransform": SimTypePointer(SimStruct({"authTransformId": SimStruct({"authType": SimTypeInt(signed=False, label="IPSEC_AUTH_TYPE"), "authConfig": SimTypeChar(label="Byte")}, name="IPSEC_AUTH_TRANSFORM_ID0", pack=False, align=None), "cryptoModuleId": SimTypePointer(SimTypeBottom(label="Guid"), offset=0)}, name="IPSEC_AUTH_TRANSFORM0", pack=False, align=None), offset=0), "espCipherTransform": SimTypePointer(SimStruct({"cipherTransformId": SimStruct({"cipherType": SimTypeInt(signed=False, label="IPSEC_CIPHER_TYPE"), "cipherConfig": SimTypeChar(label="Byte")}, name="IPSEC_CIPHER_TRANSFORM_ID0", pack=False, align=None), "cryptoModuleId": SimTypePointer(SimTypeBottom(label="Guid"), offset=0)}, name="IPSEC_CIPHER_TRANSFORM0", pack=False, align=None), offset=0), "espAuthAndCipherTransform": SimTypePointer(SimStruct({"authTransform": SimStruct({"authTransformId": SimStruct({"authType": SimTypeInt(signed=False, label="IPSEC_AUTH_TYPE"), "authConfig": SimTypeChar(label="Byte")}, name="IPSEC_AUTH_TRANSFORM_ID0", pack=False, align=None), "cryptoModuleId": SimTypePointer(SimTypeBottom(label="Guid"), offset=0)}, name="IPSEC_AUTH_TRANSFORM0", pack=False, align=None), "cipherTransform": SimStruct({"cipherTransformId": SimStruct({"cipherType": SimTypeInt(signed=False, label="IPSEC_CIPHER_TYPE"), "cipherConfig": SimTypeChar(label="Byte")}, name="IPSEC_CIPHER_TRANSFORM_ID0", pack=False, align=None), "cryptoModuleId": SimTypePointer(SimTypeBottom(label="Guid"), offset=0)}, name="IPSEC_CIPHER_TRANSFORM0", pack=False, align=None)}, name="IPSEC_AUTH_AND_CIPHER_TRANSFORM0", pack=False, align=None), offset=0), "espAuthFwTransform": SimTypePointer(SimStruct({"authTransformId": SimStruct({"authType": SimTypeInt(signed=False, label="IPSEC_AUTH_TYPE"), "authConfig": SimTypeChar(label="Byte")}, name="IPSEC_AUTH_TRANSFORM_ID0", pack=False, align=None), "cryptoModuleId": SimTypePointer(SimTypeBottom(label="Guid"), offset=0)}, name="IPSEC_AUTH_TRANSFORM0", pack=False, align=None), offset=0)}, name="<anon>", label="None")}, name="IPSEC_SA_TRANSFORM0", pack=False, align=None), offset=0), "pfsGroup": SimTypeInt(signed=False, label="IPSEC_PFS_GROUP")}, name="IPSEC_PROPOSAL0", pack=False, align=None), offset=0), "flags": SimTypeInt(signed=False, label="IPSEC_POLICY_FLAG"), "ndAllowClearTimeoutSeconds": SimTypeInt(signed=False, label="UInt32"), "saIdleTimeout": SimStruct({"idleTimeoutSeconds": SimTypeInt(signed=False, label="UInt32"), "idleTimeoutSecondsFailOver": SimTypeInt(signed=False, label="UInt32")}, name="IPSEC_SA_IDLE_TIMEOUT0", pack=False, align=None), "emPolicy": SimTypePointer(SimStruct({"numAuthenticationMethods": SimTypeInt(signed=False, label="UInt32"), "authenticationMethods": SimTypePointer(SimStruct({"authenticationMethodType": SimTypeInt(signed=False, label="IKEEXT_AUTHENTICATION_METHOD_TYPE"), "Anonymous": SimUnion({"presharedKeyAuthentication": SimStruct({"presharedKey": SimStruct({"size": SimTypeInt(signed=False, label="UInt32"), "data": SimTypePointer(SimTypeChar(label="Byte"), offset=0)}, name="FWP_BYTE_BLOB", pack=False, align=None)}, name="IKEEXT_PRESHARED_KEY_AUTHENTICATION0", pack=False, align=None), "certificateAuthentication": SimStruct({"inboundConfigType": SimTypeInt(signed=False, label="IKEEXT_CERT_CONFIG_TYPE"), "Anonymous1": SimUnion({"Anonymous": SimStruct({"inboundRootArraySize": SimTypeInt(signed=False, label="UInt32"), "inboundRootArray": SimTypePointer(SimStruct({"certData": SimStruct({"size": SimTypeInt(signed=False, label="UInt32"), "data": SimTypePointer(SimTypeChar(label="Byte"), offset=0)}, name="FWP_BYTE_BLOB", pack=False, align=None), "flags": SimTypeInt(signed=False, label="IKEEXT_CERT_FLAGS")}, name="IKEEXT_CERT_ROOT_CONFIG0", pack=False, align=None), offset=0)}, name="_Anonymous_e__Struct", pack=False, align=None), "inboundEnterpriseStoreConfig": SimTypePointer(SimStruct({"certData": SimStruct({"size": SimTypeInt(signed=False, label="UInt32"), "data": SimTypePointer(SimTypeChar(label="Byte"), offset=0)}, name="FWP_BYTE_BLOB", pack=False, align=None), "flags": SimTypeInt(signed=False, label="IKEEXT_CERT_FLAGS")}, name="IKEEXT_CERT_ROOT_CONFIG0", pack=False, align=None), offset=0), "inboundTrustedRootStoreConfig": SimTypePointer(SimStruct({"certData": SimStruct({"size": SimTypeInt(signed=False, label="UInt32"), "data": SimTypePointer(SimTypeChar(label="Byte"), offset=0)}, name="FWP_BYTE_BLOB", pack=False, align=None), "flags": SimTypeInt(signed=False, label="IKEEXT_CERT_FLAGS")}, name="IKEEXT_CERT_ROOT_CONFIG0", pack=False, align=None), offset=0)}, name="<anon>", label="None"), "outboundConfigType": SimTypeInt(signed=False, label="IKEEXT_CERT_CONFIG_TYPE"), "Anonymous2": SimUnion({"Anonymous": SimStruct({"outboundRootArraySize": SimTypeInt(signed=False, label="UInt32"), "outboundRootArray": SimTypePointer(SimStruct({"certData": SimStruct({"size": SimTypeInt(signed=False, label="UInt32"), "data": SimTypePointer(SimTypeChar(label="Byte"), offset=0)}, name="FWP_BYTE_BLOB", pack=False, align=None), "flags": SimTypeInt(signed=False, label="IKEEXT_CERT_FLAGS")}, name="IKEEXT_CERT_ROOT_CONFIG0", pack=False, align=None), offset=0)}, name="_Anonymous_e__Struct", pack=False, align=None), "outboundEnterpriseStoreConfig": SimTypePointer(SimStruct({"certData": SimStruct({"size": SimTypeInt(signed=False, label="UInt32"), "data": SimTypePointer(SimTypeChar(label="Byte"), offset=0)}, name="FWP_BYTE_BLOB", pack=False, align=None), "flags": SimTypeInt(signed=False, label="IKEEXT_CERT_FLAGS")}, name="IKEEXT_CERT_ROOT_CONFIG0", pack=False, align=None), offset=0), "outboundTrustedRootStoreConfig": SimTypePointer(SimStruct({"certData": SimStruct({"size": SimTypeInt(signed=False, label="UInt32"), "data": SimTypePointer(SimTypeChar(label="Byte"), offset=0)}, name="FWP_BYTE_BLOB", pack=False, align=None), "flags": SimTypeInt(signed=False, label="IKEEXT_CERT_FLAGS")}, name="IKEEXT_CERT_ROOT_CONFIG0", pack=False, align=None), offset=0)}, name="<anon>", label="None"), "flags": SimTypeInt(signed=False, label="IKEEXT_CERT_AUTH")}, name="IKEEXT_CERTIFICATE_AUTHENTICATION0", pack=False, align=None), "kerberosAuthentication": SimStruct({"flags": SimTypeInt(signed=False, label="IKEEXT_KERBEROS_AUTHENTICATION_FLAGS")}, name="IKEEXT_KERBEROS_AUTHENTICATION0", pack=False, align=None), "ntlmV2Authentication": SimStruct({"flags": SimTypeInt(signed=False, label="UInt32")}, name="IKEEXT_NTLM_V2_AUTHENTICATION0", pack=False, align=None), "sslAuthentication": SimStruct({"inboundConfigType": SimTypeInt(signed=False, label="IKEEXT_CERT_CONFIG_TYPE"), "Anonymous1": SimUnion({"Anonymous": SimStruct({"inboundRootArraySize": SimTypeInt(signed=False, label="UInt32"), "inboundRootArray": SimTypePointer(SimStruct({"certData": SimStruct({"size": SimTypeInt(signed=False, label="UInt32"), "data": SimTypePointer(SimTypeChar(label="Byte"), offset=0)}, name="FWP_BYTE_BLOB", pack=False, align=None), "flags": SimTypeInt(signed=False, label="IKEEXT_CERT_FLAGS")}, name="IKEEXT_CERT_ROOT_CONFIG0", pack=False, align=None), offset=0)}, name="_Anonymous_e__Struct", pack=False, align=None), "inboundEnterpriseStoreConfig": SimTypePointer(SimStruct({"certData": SimStruct({"size": SimTypeInt(signed=False, label="UInt32"), "data": SimTypePointer(SimTypeChar(label="Byte"), offset=0)}, name="FWP_BYTE_BLOB", pack=False, align=None), "flags": SimTypeInt(signed=False, label="IKEEXT_CERT_FLAGS")}, name="IKEEXT_CERT_ROOT_CONFIG0", pack=False, align=None), offset=0), "inboundTrustedRootStoreConfig": SimTypePointer(SimStruct({"certData": SimStruct({"size": SimTypeInt(signed=False, label="UInt32"), "data": SimTypePointer(SimTypeChar(label="Byte"), offset=0)}, name="FWP_BYTE_BLOB", pack=False, align=None), "flags": SimTypeInt(signed=False, label="IKEEXT_CERT_FLAGS")}, name="IKEEXT_CERT_ROOT_CONFIG0", pack=False, align=None), offset=0)}, name="<anon>", label="None"), "outboundConfigType": SimTypeInt(signed=False, label="IKEEXT_CERT_CONFIG_TYPE"), "Anonymous2": SimUnion({"Anonymous": SimStruct({"outboundRootArraySize": SimTypeInt(signed=False, label="UInt32"), "outboundRootArray": SimTypePointer(SimStruct({"certData": SimStruct({"size": SimTypeInt(signed=False, label="UInt32"), "data": SimTypePointer(SimTypeChar(label="Byte"), offset=0)}, name="FWP_BYTE_BLOB", pack=False, align=None), "flags": SimTypeInt(signed=False, label="IKEEXT_CERT_FLAGS")}, name="IKEEXT_CERT_ROOT_CONFIG0", pack=False, align=None), offset=0)}, name="_Anonymous_e__Struct", pack=False, align=None), "outboundEnterpriseStoreConfig": SimTypePointer(SimStruct({"certData": SimStruct({"size": SimTypeInt(signed=False, label="UInt32"), "data": SimTypePointer(SimTypeChar(label="Byte"), offset=0)}, name="FWP_BYTE_BLOB", pack=False, align=None), "flags": SimTypeInt(signed=False, label="IKEEXT_CERT_FLAGS")}, name="IKEEXT_CERT_ROOT_CONFIG0", pack=False, align=None), offset=0), "outboundTrustedRootStoreConfig": SimTypePointer(SimStruct({"certData": SimStruct({"size": SimTypeInt(signed=False, label="UInt32"), "data": SimTypePointer(SimTypeChar(label="Byte"), offset=0)}, name="FWP_BYTE_BLOB", pack=False, align=None), "flags": SimTypeInt(signed=False, label="IKEEXT_CERT_FLAGS")}, name="IKEEXT_CERT_ROOT_CONFIG0", pack=False, align=None), offset=0)}, name="<anon>", label="None"), "flags": SimTypeInt(signed=False, label="IKEEXT_CERT_AUTH")}, name="IKEEXT_CERTIFICATE_AUTHENTICATION0", pack=False, align=None), "cgaAuthentication": SimStruct({"keyContainerName": SimTypePointer(SimTypeChar(label="Char"), offset=0), "cspName": SimTypePointer(SimTypeChar(label="Char"), offset=0), "cspType": SimTypeInt(signed=False, label="UInt32"), "cgaModifier": SimStruct({"byteArray16": SimTypeFixedSizeArray(SimTypeChar(label="Byte"), 16)}, name="FWP_BYTE_ARRAY16", pack=False, align=None), "cgaCollisionCount": SimTypeChar(label="Byte")}, name="IKEEXT_IPV6_CGA_AUTHENTICATION0", pack=False, align=None)}, name="<anon>", label="None")}, name="IKEEXT_AUTHENTICATION_METHOD0", pack=False, align=None), offset=0), "initiatorImpersonationType": SimTypeInt(signed=False, label="IKEEXT_AUTHENTICATION_IMPERSONATION_TYPE")}, name="IKEEXT_EM_POLICY0", pack=False, align=None), offset=0)}, name="IPSEC_TRANSPORT_POLICY0", pack=False, align=None), offset=0), "authipQmTunnelPolicy": SimTypePointer(SimStruct({"flags": SimTypeInt(signed=False, label="IPSEC_POLICY_FLAG"), "numIpsecProposals": SimTypeInt(signed=False, label="UInt32"), "ipsecProposals": SimTypePointer(SimStruct({"lifetime": SimStruct({"lifetimeSeconds": SimTypeInt(signed=False, label="UInt32"), "lifetimeKilobytes": SimTypeInt(signed=False, label="UInt32"), "lifetimePackets": SimTypeInt(signed=False, label="UInt32")}, name="IPSEC_SA_LIFETIME0", pack=False, align=None), "numSaTransforms": SimTypeInt(signed=False, label="UInt32"), "saTransforms": SimTypePointer(SimStruct({"ipsecTransformType": SimTypeInt(signed=False, label="IPSEC_TRANSFORM_TYPE"), "Anonymous": SimUnion({"ahTransform": SimTypePointer(SimStruct({"authTransformId": SimStruct({"authType": SimTypeInt(signed=False, label="IPSEC_AUTH_TYPE"), "authConfig": SimTypeChar(label="Byte")}, name="IPSEC_AUTH_TRANSFORM_ID0", pack=False, align=None), "cryptoModuleId": SimTypePointer(SimTypeBottom(label="Guid"), offset=0)}, name="IPSEC_AUTH_TRANSFORM0", pack=False, align=None), offset=0), "espAuthTransform": SimTypePointer(SimStruct({"authTransformId": SimStruct({"authType": SimTypeInt(signed=False, label="IPSEC_AUTH_TYPE"), "authConfig": SimTypeChar(label="Byte")}, name="IPSEC_AUTH_TRANSFORM_ID0", pack=False, align=None), "cryptoModuleId": SimTypePointer(SimTypeBottom(label="Guid"), offset=0)}, name="IPSEC_AUTH_TRANSFORM0", pack=False, align=None), offset=0), "espCipherTransform": SimTypePointer(SimStruct({"cipherTransformId": SimStruct({"cipherType": SimTypeInt(signed=False, label="IPSEC_CIPHER_TYPE"), "cipherConfig": SimTypeChar(label="Byte")}, name="IPSEC_CIPHER_TRANSFORM_ID0", pack=False, align=None), "cryptoModuleId": SimTypePointer(SimTypeBottom(label="Guid"), offset=0)}, name="IPSEC_CIPHER_TRANSFORM0", pack=False, align=None), offset=0), "espAuthAndCipherTransform": SimTypePointer(SimStruct({"authTransform": SimStruct({"authTransformId": SimStruct({"authType": SimTypeInt(signed=False, label="IPSEC_AUTH_TYPE"), "authConfig": SimTypeChar(label="Byte")}, name="IPSEC_AUTH_TRANSFORM_ID0", pack=False, align=None), "cryptoModuleId": SimTypePointer(SimTypeBottom(label="Guid"), offset=0)}, name="IPSEC_AUTH_TRANSFORM0", pack=False, align=None), "cipherTransform": SimStruct({"cipherTransformId": SimStruct({"cipherType": SimTypeInt(signed=False, label="IPSEC_CIPHER_TYPE"), "cipherConfig": SimTypeChar(label="Byte")}, name="IPSEC_CIPHER_TRANSFORM_ID0", pack=False, align=None), "cryptoModuleId": SimTypePointer(SimTypeBottom(label="Guid"), offset=0)}, name="IPSEC_CIPHER_TRANSFORM0", pack=False, align=None)}, name="IPSEC_AUTH_AND_CIPHER_TRANSFORM0", pack=False, align=None), offset=0), "espAuthFwTransform": SimTypePointer(SimStruct({"authTransformId": SimStruct({"authType": SimTypeInt(signed=False, label="IPSEC_AUTH_TYPE"), "authConfig": SimTypeChar(label="Byte")}, name="IPSEC_AUTH_TRANSFORM_ID0", pack=False, align=None), "cryptoModuleId": SimTypePointer(SimTypeBottom(label="Guid"), offset=0)}, name="IPSEC_AUTH_TRANSFORM0", pack=False, align=None), offset=0)}, name="<anon>", label="None")}, name="IPSEC_SA_TRANSFORM0", pack=False, align=None), offset=0), "pfsGroup": SimTypeInt(signed=False, label="IPSEC_PFS_GROUP")}, name="IPSEC_PROPOSAL0", pack=False, align=None), offset=0), "tunnelEndpoints": SimStruct({"ipVersion": SimTypeInt(signed=False, label="FWP_IP_VERSION"), "Anonymous1": SimUnion({"localV4Address": SimTypeInt(signed=False, label="UInt32"), "localV6Address": SimTypeFixedSizeArray(SimTypeChar(label="Byte"), 16)}, name="<anon>", label="None"), "Anonymous2": SimUnion({"remoteV4Address": SimTypeInt(signed=False, label="UInt32"), "remoteV6Address": SimTypeFixedSizeArray(SimTypeChar(label="Byte"), 16)}, name="<anon>", label="None")}, name="IPSEC_TUNNEL_ENDPOINTS0", pack=False, align=None), "saIdleTimeout": SimStruct({"idleTimeoutSeconds": SimTypeInt(signed=False, label="UInt32"), "idleTimeoutSecondsFailOver": SimTypeInt(signed=False, label="UInt32")}, name="IPSEC_SA_IDLE_TIMEOUT0", pack=False, align=None), "emPolicy": SimTypePointer(SimStruct({"numAuthenticationMethods": SimTypeInt(signed=False, label="UInt32"), "authenticationMethods": SimTypePointer(SimStruct({"authenticationMethodType": SimTypeInt(signed=False, label="IKEEXT_AUTHENTICATION_METHOD_TYPE"), "Anonymous": SimUnion({"presharedKeyAuthentication": SimStruct({"presharedKey": SimStruct({"size": SimTypeInt(signed=False, label="UInt32"), "data": SimTypePointer(SimTypeChar(label="Byte"), offset=0)}, name="FWP_BYTE_BLOB", pack=False, align=None)}, name="IKEEXT_PRESHARED_KEY_AUTHENTICATION0", pack=False, align=None), "certificateAuthentication": SimStruct({"inboundConfigType": SimTypeInt(signed=False, label="IKEEXT_CERT_CONFIG_TYPE"), "Anonymous1": SimUnion({"Anonymous": SimStruct({"inboundRootArraySize": SimTypeInt(signed=False, label="UInt32"), "inboundRootArray": SimTypePointer(SimStruct({"certData": SimStruct({"size": SimTypeInt(signed=False, label="UInt32"), "data": SimTypePointer(SimTypeChar(label="Byte"), offset=0)}, name="FWP_BYTE_BLOB", pack=False, align=None), "flags": SimTypeInt(signed=False, label="IKEEXT_CERT_FLAGS")}, name="IKEEXT_CERT_ROOT_CONFIG0", pack=False, align=None), offset=0)}, name="_Anonymous_e__Struct", pack=False, align=None), "inboundEnterpriseStoreConfig": SimTypePointer(SimStruct({"certData": SimStruct({"size": SimTypeInt(signed=False, label="UInt32"), "data": SimTypePointer(SimTypeChar(label="Byte"), offset=0)}, name="FWP_BYTE_BLOB", pack=False, align=None), "flags": SimTypeInt(signed=False, label="IKEEXT_CERT_FLAGS")}, name="IKEEXT_CERT_ROOT_CONFIG0", pack=False, align=None), offset=0), "inboundTrustedRootStoreConfig": SimTypePointer(SimStruct({"certData": SimStruct({"size": SimTypeInt(signed=False, label="UInt32"), "data": SimTypePointer(SimTypeChar(label="Byte"), offset=0)}, name="FWP_BYTE_BLOB", pack=False, align=None), "flags": SimTypeInt(signed=False, label="IKEEXT_CERT_FLAGS")}, name="IKEEXT_CERT_ROOT_CONFIG0", pack=False, align=None), offset=0)}, name="<anon>", label="None"), "outboundConfigType": SimTypeInt(signed=False, label="IKEEXT_CERT_CONFIG_TYPE"), "Anonymous2": SimUnion({"Anonymous": SimStruct({"outboundRootArraySize": SimTypeInt(signed=False, label="UInt32"), "outboundRootArray": SimTypePointer(SimStruct({"certData": SimStruct({"size": SimTypeInt(signed=False, label="UInt32"), "data": SimTypePointer(SimTypeChar(label="Byte"), offset=0)}, name="FWP_BYTE_BLOB", pack=False, align=None), "flags": SimTypeInt(signed=False, label="IKEEXT_CERT_FLAGS")}, name="IKEEXT_CERT_ROOT_CONFIG0", pack=False, align=None), offset=0)}, name="_Anonymous_e__Struct", pack=False, align=None), "outboundEnterpriseStoreConfig": SimTypePointer(SimStruct({"certData": SimStruct({"size": SimTypeInt(signed=False, label="UInt32"), "data": SimTypePointer(SimTypeChar(label="Byte"), offset=0)}, name="FWP_BYTE_BLOB", pack=False, align=None), "flags": SimTypeInt(signed=False, label="IKEEXT_CERT_FLAGS")}, name="IKEEXT_CERT_ROOT_CONFIG0", pack=False, align=None), offset=0), "outboundTrustedRootStoreConfig": SimTypePointer(SimStruct({"certData": SimStruct({"size": SimTypeInt(signed=False, label="UInt32"), "data": SimTypePointer(SimTypeChar(label="Byte"), offset=0)}, name="FWP_BYTE_BLOB", pack=False, align=None), "flags": SimTypeInt(signed=False, label="IKEEXT_CERT_FLAGS")}, name="IKEEXT_CERT_ROOT_CONFIG0", pack=False, align=None), offset=0)}, name="<anon>", label="None"), "flags": SimTypeInt(signed=False, label="IKEEXT_CERT_AUTH")}, name="IKEEXT_CERTIFICATE_AUTHENTICATION0", pack=False, align=None), "kerberosAuthentication": SimStruct({"flags": SimTypeInt(signed=False, label="IKEEXT_KERBEROS_AUTHENTICATION_FLAGS")}, name="IKEEXT_KERBEROS_AUTHENTICATION0", pack=False, align=None), "ntlmV2Authentication": SimStruct({"flags": SimTypeInt(signed=False, label="UInt32")}, name="IKEEXT_NTLM_V2_AUTHENTICATION0", pack=False, align=None), "sslAuthentication": SimStruct({"inboundConfigType": SimTypeInt(signed=False, label="IKEEXT_CERT_CONFIG_TYPE"), "Anonymous1": SimUnion({"Anonymous": SimStruct({"inboundRootArraySize": SimTypeInt(signed=False, label="UInt32"), "inboundRootArray": SimTypePointer(SimStruct({"certData": SimStruct({"size": SimTypeInt(signed=False, label="UInt32"), "data": SimTypePointer(SimTypeChar(label="Byte"), offset=0)}, name="FWP_BYTE_BLOB", pack=False, align=None), "flags": SimTypeInt(signed=False, label="IKEEXT_CERT_FLAGS")}, name="IKEEXT_CERT_ROOT_CONFIG0", pack=False, align=None), offset=0)}, name="_Anonymous_e__Struct", pack=False, align=None), "inboundEnterpriseStoreConfig": SimTypePointer(SimStruct({"certData": SimStruct({"size": SimTypeInt(signed=False, label="UInt32"), "data": SimTypePointer(SimTypeChar(label="Byte"), offset=0)}, name="FWP_BYTE_BLOB", pack=False, align=None), "flags": SimTypeInt(signed=False, label="IKEEXT_CERT_FLAGS")}, name="IKEEXT_CERT_ROOT_CONFIG0", pack=False, align=None), offset=0), "inboundTrustedRootStoreConfig": SimTypePointer(SimStruct({"certData": SimStruct({"size": SimTypeInt(signed=False, label="UInt32"), "data": SimTypePointer(SimTypeChar(label="Byte"), offset=0)}, name="FWP_BYTE_BLOB", pack=False, align=None), "flags": SimTypeInt(signed=False, label="IKEEXT_CERT_FLAGS")}, name="IKEEXT_CERT_ROOT_CONFIG0", pack=False, align=None), offset=0)}, name="<anon>", label="None"), "outboundConfigType": SimTypeInt(signed=False, label="IKEEXT_CERT_CONFIG_TYPE"), "Anonymous2": SimUnion({"Anonymous": SimStruct({"outboundRootArraySize": SimTypeInt(signed=False, label="UInt32"), "outboundRootArray": SimTypePointer(SimStruct({"certData": SimStruct({"size": SimTypeInt(signed=False, label="UInt32"), "data": SimTypePointer(SimTypeChar(label="Byte"), offset=0)}, name="FWP_BYTE_BLOB", pack=False, align=None), "flags": SimTypeInt(signed=False, label="IKEEXT_CERT_FLAGS")}, name="IKEEXT_CERT_ROOT_CONFIG0", pack=False, align=None), offset=0)}, name="_Anonymous_e__Struct", pack=False, align=None), "outboundEnterpriseStoreConfig": SimTypePointer(SimStruct({"certData": SimStruct({"size": SimTypeInt(signed=False, label="UInt32"), "data": SimTypePointer(SimTypeChar(label="Byte"), offset=0)}, name="FWP_BYTE_BLOB", pack=False, align=None), "flags": SimTypeInt(signed=False, label="IKEEXT_CERT_FLAGS")}, name="IKEEXT_CERT_ROOT_CONFIG0", pack=False, align=None), offset=0), "outboundTrustedRootStoreConfig": SimTypePointer(SimStruct({"certData": SimStruct({"size": SimTypeInt(signed=False, label="UInt32"), "data": SimTypePointer(SimTypeChar(label="Byte"), offset=0)}, name="FWP_BYTE_BLOB", pack=False, align=None), "flags": SimTypeInt(signed=False, label="IKEEXT_CERT_FLAGS")}, name="IKEEXT_CERT_ROOT_CONFIG0", pack=False, align=None), offset=0)}, name="<anon>", label="None"), "flags": SimTypeInt(signed=False, label="IKEEXT_CERT_AUTH")}, name="IKEEXT_CERTIFICATE_AUTHENTICATION0", pack=False, align=None), "cgaAuthentication": SimStruct({"keyContainerName": SimTypePointer(SimTypeChar(label="Char"), offset=0), "cspName": SimTypePointer(SimTypeChar(label="Char"), offset=0), "cspType": SimTypeInt(signed=False, label="UInt32"), "cgaModifier": SimStruct({"byteArray16": SimTypeFixedSizeArray(SimTypeChar(label="Byte"), 16)}, name="FWP_BYTE_ARRAY16", pack=False, align=None), "cgaCollisionCount": SimTypeChar(label="Byte")}, name="IKEEXT_IPV6_CGA_AUTHENTICATION0", pack=False, align=None)}, name="<anon>", label="None")}, name="IKEEXT_AUTHENTICATION_METHOD0", pack=False, align=None), offset=0), "initiatorImpersonationType": SimTypeInt(signed=False, label="IKEEXT_AUTHENTICATION_IMPERSONATION_TYPE")}, name="IKEEXT_EM_POLICY0", pack=False, align=None), offset=0)}, name="IPSEC_TUNNEL_POLICY0", pack=False, align=None), offset=0), "ikeMmPolicy": SimTypePointer(SimStruct({"softExpirationTime": SimTypeInt(signed=False, label="UInt32"), "numAuthenticationMethods": SimTypeInt(signed=False, label="UInt32"), "authenticationMethods": SimTypePointer(SimStruct({"authenticationMethodType": SimTypeInt(signed=False, label="IKEEXT_AUTHENTICATION_METHOD_TYPE"), "Anonymous": SimUnion({"presharedKeyAuthentication": SimStruct({"presharedKey": SimStruct({"size": SimTypeInt(signed=False, label="UInt32"), "data": SimTypePointer(SimTypeChar(label="Byte"), offset=0)}, name="FWP_BYTE_BLOB", pack=False, align=None)}, name="IKEEXT_PRESHARED_KEY_AUTHENTICATION0", pack=False, align=None), "certificateAuthentication": SimStruct({"inboundConfigType": SimTypeInt(signed=False, label="IKEEXT_CERT_CONFIG_TYPE"), "Anonymous1": SimUnion({"Anonymous": SimStruct({"inboundRootArraySize": SimTypeInt(signed=False, label="UInt32"), "inboundRootArray": SimTypePointer(SimStruct({"certData": SimStruct({"size": SimTypeInt(signed=False, label="UInt32"), "data": SimTypePointer(SimTypeChar(label="Byte"), offset=0)}, name="FWP_BYTE_BLOB", pack=False, align=None), "flags": SimTypeInt(signed=False, label="IKEEXT_CERT_FLAGS")}, name="IKEEXT_CERT_ROOT_CONFIG0", pack=False, align=None), offset=0)}, name="_Anonymous_e__Struct", pack=False, align=None), "inboundEnterpriseStoreConfig": SimTypePointer(SimStruct({"certData": SimStruct({"size": SimTypeInt(signed=False, label="UInt32"), "data": SimTypePointer(SimTypeChar(label="Byte"), offset=0)}, name="FWP_BYTE_BLOB", pack=False, align=None), "flags": SimTypeInt(signed=False, label="IKEEXT_CERT_FLAGS")}, name="IKEEXT_CERT_ROOT_CONFIG0", pack=False, align=None), offset=0), "inboundTrustedRootStoreConfig": SimTypePointer(SimStruct({"certData": SimStruct({"size": SimTypeInt(signed=False, label="UInt32"), "data": SimTypePointer(SimTypeChar(label="Byte"), offset=0)}, name="FWP_BYTE_BLOB", pack=False, align=None), "flags": SimTypeInt(signed=False, label="IKEEXT_CERT_FLAGS")}, name="IKEEXT_CERT_ROOT_CONFIG0", pack=False, align=None), offset=0)}, name="<anon>", label="None"), "outboundConfigType": SimTypeInt(signed=False, label="IKEEXT_CERT_CONFIG_TYPE"), "Anonymous2": SimUnion({"Anonymous": SimStruct({"outboundRootArraySize": SimTypeInt(signed=False, label="UInt32"), "outboundRootArray": SimTypePointer(SimStruct({"certData": SimStruct({"size": SimTypeInt(signed=False, label="UInt32"), "data": SimTypePointer(SimTypeChar(label="Byte"), offset=0)}, name="FWP_BYTE_BLOB", pack=False, align=None), "flags": SimTypeInt(signed=False, label="IKEEXT_CERT_FLAGS")}, name="IKEEXT_CERT_ROOT_CONFIG0", pack=False, align=None), offset=0)}, name="_Anonymous_e__Struct", pack=False, align=None), "outboundEnterpriseStoreConfig": SimTypePointer(SimStruct({"certData": SimStruct({"size": SimTypeInt(signed=False, label="UInt32"), "data": SimTypePointer(SimTypeChar(label="Byte"), offset=0)}, name="FWP_BYTE_BLOB", pack=False, align=None), "flags": SimTypeInt(signed=False, label="IKEEXT_CERT_FLAGS")}, name="IKEEXT_CERT_ROOT_CONFIG0", pack=False, align=None), offset=0), "outboundTrustedRootStoreConfig": SimTypePointer(SimStruct({"certData": SimStruct({"size": SimTypeInt(signed=False, label="UInt32"), "data": SimTypePointer(SimTypeChar(label="Byte"), offset=0)}, name="FWP_BYTE_BLOB", pack=False, align=None), "flags": SimTypeInt(signed=False, label="IKEEXT_CERT_FLAGS")}, name="IKEEXT_CERT_ROOT_CONFIG0", pack=False, align=None), offset=0)}, name="<anon>", label="None"), "flags": SimTypeInt(signed=False, label="IKEEXT_CERT_AUTH")}, name="IKEEXT_CERTIFICATE_AUTHENTICATION0", pack=False, align=None), "kerberosAuthentication": SimStruct({"flags": SimTypeInt(signed=False, label="IKEEXT_KERBEROS_AUTHENTICATION_FLAGS")}, name="IKEEXT_KERBEROS_AUTHENTICATION0", pack=False, align=None), "ntlmV2Authentication": SimStruct({"flags": SimTypeInt(signed=False, label="UInt32")}, name="IKEEXT_NTLM_V2_AUTHENTICATION0", pack=False, align=None), "sslAuthentication": SimStruct({"inboundConfigType": SimTypeInt(signed=False, label="IKEEXT_CERT_CONFIG_TYPE"), "Anonymous1": SimUnion({"Anonymous": SimStruct({"inboundRootArraySize": SimTypeInt(signed=False, label="UInt32"), "inboundRootArray": SimTypePointer(SimStruct({"certData": SimStruct({"size": SimTypeInt(signed=False, label="UInt32"), "data": SimTypePointer(SimTypeChar(label="Byte"), offset=0)}, name="FWP_BYTE_BLOB", pack=False, align=None), "flags": SimTypeInt(signed=False, label="IKEEXT_CERT_FLAGS")}, name="IKEEXT_CERT_ROOT_CONFIG0", pack=False, align=None), offset=0)}, name="_Anonymous_e__Struct", pack=False, align=None), "inboundEnterpriseStoreConfig": SimTypePointer(SimStruct({"certData": SimStruct({"size": SimTypeInt(signed=False, label="UInt32"), "data": SimTypePointer(SimTypeChar(label="Byte"), offset=0)}, name="FWP_BYTE_BLOB", pack=False, align=None), "flags": SimTypeInt(signed=False, label="IKEEXT_CERT_FLAGS")}, name="IKEEXT_CERT_ROOT_CONFIG0", pack=False, align=None), offset=0), "inboundTrustedRootStoreConfig": SimTypePointer(SimStruct({"certData": SimStruct({"size": SimTypeInt(signed=False, label="UInt32"), "data": SimTypePointer(SimTypeChar(label="Byte"), offset=0)}, name="FWP_BYTE_BLOB", pack=False, align=None), "flags": SimTypeInt(signed=False, label="IKEEXT_CERT_FLAGS")}, name="IKEEXT_CERT_ROOT_CONFIG0", pack=False, align=None), offset=0)}, name="<anon>", label="None"), "outboundConfigType": SimTypeInt(signed=False, label="IKEEXT_CERT_CONFIG_TYPE"), "Anonymous2": SimUnion({"Anonymous": SimStruct({"outboundRootArraySize": SimTypeInt(signed=False, label="UInt32"), "outboundRootArray": SimTypePointer(SimStruct({"certData": SimStruct({"size": SimTypeInt(signed=False, label="UInt32"), "data": SimTypePointer(SimTypeChar(label="Byte"), offset=0)}, name="FWP_BYTE_BLOB", pack=False, align=None), "flags": SimTypeInt(signed=False, label="IKEEXT_CERT_FLAGS")}, name="IKEEXT_CERT_ROOT_CONFIG0", pack=False, align=None), offset=0)}, name="_Anonymous_e__Struct", pack=False, align=None), "outboundEnterpriseStoreConfig": SimTypePointer(SimStruct({"certData": SimStruct({"size": SimTypeInt(signed=False, label="UInt32"), "data": SimTypePointer(SimTypeChar(label="Byte"), offset=0)}, name="FWP_BYTE_BLOB", pack=False, align=None), "flags": SimTypeInt(signed=False, label="IKEEXT_CERT_FLAGS")}, name="IKEEXT_CERT_ROOT_CONFIG0", pack=False, align=None), offset=0), "outboundTrustedRootStoreConfig": SimTypePointer(SimStruct({"certData": SimStruct({"size": SimTypeInt(signed=False, label="UInt32"), "data": SimTypePointer(SimTypeChar(label="Byte"), offset=0)}, name="FWP_BYTE_BLOB", pack=False, align=None), "flags": SimTypeInt(signed=False, label="IKEEXT_CERT_FLAGS")}, name="IKEEXT_CERT_ROOT_CONFIG0", pack=False, align=None), offset=0)}, name="<anon>", label="None"), "flags": SimTypeInt(signed=False, label="IKEEXT_CERT_AUTH")}, name="IKEEXT_CERTIFICATE_AUTHENTICATION0", pack=False, align=None), "cgaAuthentication": SimStruct({"keyContainerName": SimTypePointer(SimTypeChar(label="Char"), offset=0), "cspName": SimTypePointer(SimTypeChar(label="Char"), offset=0), "cspType": SimTypeInt(signed=False, label="UInt32"), "cgaModifier": SimStruct({"byteArray16": SimTypeFixedSizeArray(SimTypeChar(label="Byte"), 16)}, name="FWP_BYTE_ARRAY16", pack=False, align=None), "cgaCollisionCount": SimTypeChar(label="Byte")}, name="IKEEXT_IPV6_CGA_AUTHENTICATION0", pack=False, align=None)}, name="<anon>", label="None")}, name="IKEEXT_AUTHENTICATION_METHOD0", pack=False, align=None), offset=0), "initiatorImpersonationType": SimTypeInt(signed=False, label="IKEEXT_AUTHENTICATION_IMPERSONATION_TYPE"), "numIkeProposals": SimTypeInt(signed=False, label="UInt32"), "ikeProposals": SimTypePointer(SimStruct({"cipherAlgorithm": SimStruct({"algoIdentifier": SimTypeInt(signed=False, label="IKEEXT_CIPHER_TYPE"), "keyLen": SimTypeInt(signed=False, label="UInt32"), "rounds": SimTypeInt(signed=False, label="UInt32")}, name="IKEEXT_CIPHER_ALGORITHM0", pack=False, align=None), "integrityAlgorithm": SimStruct({"algoIdentifier": SimTypeInt(signed=False, label="IKEEXT_INTEGRITY_TYPE")}, name="IKEEXT_INTEGRITY_ALGORITHM0", pack=False, align=None), "maxLifetimeSeconds": SimTypeInt(signed=False, label="UInt32"), "dhGroup": SimTypeInt(signed=False, label="IKEEXT_DH_GROUP"), "quickModeLimit": SimTypeInt(signed=False, label="UInt32")}, name="IKEEXT_PROPOSAL0", pack=False, align=None), offset=0), "flags": SimTypeInt(signed=False, label="IKEEXT_POLICY_FLAG"), "maxDynamicFilters": SimTypeInt(signed=False, label="UInt32")}, name="IKEEXT_POLICY0", pack=False, align=None), offset=0), "authIpMmPolicy": SimTypePointer(SimStruct({"softExpirationTime": SimTypeInt(signed=False, label="UInt32"), "numAuthenticationMethods": SimTypeInt(signed=False, label="UInt32"), "authenticationMethods": SimTypePointer(SimStruct({"authenticationMethodType": SimTypeInt(signed=False, label="IKEEXT_AUTHENTICATION_METHOD_TYPE"), "Anonymous": SimUnion({"presharedKeyAuthentication": SimStruct({"presharedKey": SimStruct({"size": SimTypeInt(signed=False, label="UInt32"), "data": SimTypePointer(SimTypeChar(label="Byte"), offset=0)}, name="FWP_BYTE_BLOB", pack=False, align=None)}, name="IKEEXT_PRESHARED_KEY_AUTHENTICATION0", pack=False, align=None), "certificateAuthentication": SimStruct({"inboundConfigType": SimTypeInt(signed=False, label="IKEEXT_CERT_CONFIG_TYPE"), "Anonymous1": SimUnion({"Anonymous": SimStruct({"inboundRootArraySize": SimTypeInt(signed=False, label="UInt32"), "inboundRootArray": SimTypePointer(SimStruct({"certData": SimStruct({"size": SimTypeInt(signed=False, label="UInt32"), "data": SimTypePointer(SimTypeChar(label="Byte"), offset=0)}, name="FWP_BYTE_BLOB", pack=False, align=None), "flags": SimTypeInt(signed=False, label="IKEEXT_CERT_FLAGS")}, name="IKEEXT_CERT_ROOT_CONFIG0", pack=False, align=None), offset=0)}, name="_Anonymous_e__Struct", pack=False, align=None), "inboundEnterpriseStoreConfig": SimTypePointer(SimStruct({"certData": SimStruct({"size": SimTypeInt(signed=False, label="UInt32"), "data": SimTypePointer(SimTypeChar(label="Byte"), offset=0)}, name="FWP_BYTE_BLOB", pack=False, align=None), "flags": SimTypeInt(signed=False, label="IKEEXT_CERT_FLAGS")}, name="IKEEXT_CERT_ROOT_CONFIG0", pack=False, align=None), offset=0), "inboundTrustedRootStoreConfig": SimTypePointer(SimStruct({"certData": SimStruct({"size": SimTypeInt(signed=False, label="UInt32"), "data": SimTypePointer(SimTypeChar(label="Byte"), offset=0)}, name="FWP_BYTE_BLOB", pack=False, align=None), "flags": SimTypeInt(signed=False, label="IKEEXT_CERT_FLAGS")}, name="IKEEXT_CERT_ROOT_CONFIG0", pack=False, align=None), offset=0)}, name="<anon>", label="None"), "outboundConfigType": SimTypeInt(signed=False, label="IKEEXT_CERT_CONFIG_TYPE"), "Anonymous2": SimUnion({"Anonymous": SimStruct({"outboundRootArraySize": SimTypeInt(signed=False, label="UInt32"), "outboundRootArray": SimTypePointer(SimStruct({"certData": SimStruct({"size": SimTypeInt(signed=False, label="UInt32"), "data": SimTypePointer(SimTypeChar(label="Byte"), offset=0)}, name="FWP_BYTE_BLOB", pack=False, align=None), "flags": SimTypeInt(signed=False, label="IKEEXT_CERT_FLAGS")}, name="IKEEXT_CERT_ROOT_CONFIG0", pack=False, align=None), offset=0)}, name="_Anonymous_e__Struct", pack=False, align=None), "outboundEnterpriseStoreConfig": SimTypePointer(SimStruct({"certData": SimStruct({"size": SimTypeInt(signed=False, label="UInt32"), "data": SimTypePointer(SimTypeChar(label="Byte"), offset=0)}, name="FWP_BYTE_BLOB", pack=False, align=None), "flags": SimTypeInt(signed=False, label="IKEEXT_CERT_FLAGS")}, name="IKEEXT_CERT_ROOT_CONFIG0", pack=False, align=None), offset=0), "outboundTrustedRootStoreConfig": SimTypePointer(SimStruct({"certData": SimStruct({"size": SimTypeInt(signed=False, label="UInt32"), "data": SimTypePointer(SimTypeChar(label="Byte"), offset=0)}, name="FWP_BYTE_BLOB", pack=False, align=None), "flags": SimTypeInt(signed=False, label="IKEEXT_CERT_FLAGS")}, name="IKEEXT_CERT_ROOT_CONFIG0", pack=False, align=None), offset=0)}, name="<anon>", label="None"), "flags": SimTypeInt(signed=False, label="IKEEXT_CERT_AUTH")}, name="IKEEXT_CERTIFICATE_AUTHENTICATION0", pack=False, align=None), "kerberosAuthentication": SimStruct({"flags": SimTypeInt(signed=False, label="IKEEXT_KERBEROS_AUTHENTICATION_FLAGS")}, name="IKEEXT_KERBEROS_AUTHENTICATION0", pack=False, align=None), "ntlmV2Authentication": SimStruct({"flags": SimTypeInt(signed=False, label="UInt32")}, name="IKEEXT_NTLM_V2_AUTHENTICATION0", pack=False, align=None), "sslAuthentication": SimStruct({"inboundConfigType": SimTypeInt(signed=False, label="IKEEXT_CERT_CONFIG_TYPE"), "Anonymous1": SimUnion({"Anonymous": SimStruct({"inboundRootArraySize": SimTypeInt(signed=False, label="UInt32"), "inboundRootArray": SimTypePointer(SimStruct({"certData": SimStruct({"size": SimTypeInt(signed=False, label="UInt32"), "data": SimTypePointer(SimTypeChar(label="Byte"), offset=0)}, name="FWP_BYTE_BLOB", pack=False, align=None), "flags": SimTypeInt(signed=False, label="IKEEXT_CERT_FLAGS")}, name="IKEEXT_CERT_ROOT_CONFIG0", pack=False, align=None), offset=0)}, name="_Anonymous_e__Struct", pack=False, align=None), "inboundEnterpriseStoreConfig": SimTypePointer(SimStruct({"certData": SimStruct({"size": SimTypeInt(signed=False, label="UInt32"), "data": SimTypePointer(SimTypeChar(label="Byte"), offset=0)}, name="FWP_BYTE_BLOB", pack=False, align=None), "flags": SimTypeInt(signed=False, label="IKEEXT_CERT_FLAGS")}, name="IKEEXT_CERT_ROOT_CONFIG0", pack=False, align=None), offset=0), "inboundTrustedRootStoreConfig": SimTypePointer(SimStruct({"certData": SimStruct({"size": SimTypeInt(signed=False, label="UInt32"), "data": SimTypePointer(SimTypeChar(label="Byte"), offset=0)}, name="FWP_BYTE_BLOB", pack=False, align=None), "flags": SimTypeInt(signed=False, label="IKEEXT_CERT_FLAGS")}, name="IKEEXT_CERT_ROOT_CONFIG0", pack=False, align=None), offset=0)}, name="<anon>", label="None"), "outboundConfigType": SimTypeInt(signed=False, label="IKEEXT_CERT_CONFIG_TYPE"), "Anonymous2": SimUnion({"Anonymous": SimStruct({"outboundRootArraySize": SimTypeInt(signed=False, label="UInt32"), "outboundRootArray": SimTypePointer(SimStruct({"certData": SimStruct({"size": SimTypeInt(signed=False, label="UInt32"), "data": SimTypePointer(SimTypeChar(label="Byte"), offset=0)}, name="FWP_BYTE_BLOB", pack=False, align=None), "flags": SimTypeInt(signed=False, label="IKEEXT_CERT_FLAGS")}, name="IKEEXT_CERT_ROOT_CONFIG0", pack=False, align=None), offset=0)}, name="_Anonymous_e__Struct", pack=False, align=None), "outboundEnterpriseStoreConfig": SimTypePointer(SimStruct({"certData": SimStruct({"size": SimTypeInt(signed=False, label="UInt32"), "data": SimTypePointer(SimTypeChar(label="Byte"), offset=0)}, name="FWP_BYTE_BLOB", pack=False, align=None), "flags": SimTypeInt(signed=False, label="IKEEXT_CERT_FLAGS")}, name="IKEEXT_CERT_ROOT_CONFIG0", pack=False, align=None), offset=0), "outboundTrustedRootStoreConfig": SimTypePointer(SimStruct({"certData": SimStruct({"size": SimTypeInt(signed=False, label="UInt32"), "data": SimTypePointer(SimTypeChar(label="Byte"), offset=0)}, name="FWP_BYTE_BLOB", pack=False, align=None), "flags": SimTypeInt(signed=False, label="IKEEXT_CERT_FLAGS")}, name="IKEEXT_CERT_ROOT_CONFIG0", pack=False, align=None), offset=0)}, name="<anon>", label="None"), "flags": SimTypeInt(signed=False, label="IKEEXT_CERT_AUTH")}, name="IKEEXT_CERTIFICATE_AUTHENTICATION0", pack=False, align=None), "cgaAuthentication": SimStruct({"keyContainerName": SimTypePointer(SimTypeChar(label="Char"), offset=0), "cspName": SimTypePointer(SimTypeChar(label="Char"), offset=0), "cspType": SimTypeInt(signed=False, label="UInt32"), "cgaModifier": SimStruct({"byteArray16": SimTypeFixedSizeArray(SimTypeChar(label="Byte"), 16)}, name="FWP_BYTE_ARRAY16", pack=False, align=None), "cgaCollisionCount": SimTypeChar(label="Byte")}, name="IKEEXT_IPV6_CGA_AUTHENTICATION0", pack=False, align=None)}, name="<anon>", label="None")}, name="IKEEXT_AUTHENTICATION_METHOD0", pack=False, align=None), offset=0), "initiatorImpersonationType": SimTypeInt(signed=False, label="IKEEXT_AUTHENTICATION_IMPERSONATION_TYPE"), "numIkeProposals": SimTypeInt(signed=False, label="UInt32"), "ikeProposals": SimTypePointer(SimStruct({"cipherAlgorithm": SimStruct({"algoIdentifier": SimTypeInt(signed=False, label="IKEEXT_CIPHER_TYPE"), "keyLen": SimTypeInt(signed=False, label="UInt32"), "rounds": SimTypeInt(signed=False, label="UInt32")}, name="IKEEXT_CIPHER_ALGORITHM0", pack=False, align=None), "integrityAlgorithm": SimStruct({"algoIdentifier": SimTypeInt(signed=False, label="IKEEXT_INTEGRITY_TYPE")}, name="IKEEXT_INTEGRITY_ALGORITHM0", pack=False, align=None), "maxLifetimeSeconds": SimTypeInt(signed=False, label="UInt32"), "dhGroup": SimTypeInt(signed=False, label="IKEEXT_DH_GROUP"), "quickModeLimit": SimTypeInt(signed=False, label="UInt32")}, name="IKEEXT_PROPOSAL0", pack=False, align=None), offset=0), "flags": SimTypeInt(signed=False, label="IKEEXT_POLICY_FLAG"), "maxDynamicFilters": SimTypeInt(signed=False, label="UInt32")}, name="IKEEXT_POLICY0", pack=False, align=None), offset=0), "dataBuffer": SimTypePointer(SimStruct({"size": SimTypeInt(signed=False, label="UInt32"), "data": SimTypePointer(SimTypeChar(label="Byte"), offset=0)}, name="FWP_BYTE_BLOB", pack=False, align=None), offset=0), "classifyOptions": SimTypePointer(SimStruct({"numOptions": SimTypeInt(signed=False, label="UInt32"), "options": SimTypePointer(SimStruct({"type": SimTypeInt(signed=False, label="FWP_CLASSIFY_OPTION_TYPE"), "value": SimStruct({"type": SimTypeInt(signed=False, label="FWP_DATA_TYPE"), "Anonymous": SimUnion({"uint8": SimTypeChar(label="Byte"), "uint16": SimTypeShort(signed=False, label="UInt16"), "uint32": SimTypeInt(signed=False, label="UInt32"), "uint64": SimTypePointer(SimTypeLongLong(signed=False, label="UInt64"), offset=0), "int8": SimTypeChar(label="SByte"), "int16": SimTypeShort(signed=True, label="Int16"), "int32": SimTypeInt(signed=True, label="Int32"), "int64": SimTypePointer(SimTypeLongLong(signed=True, label="Int64"), offset=0), "float32": SimTypeFloat(size=32), "double64": SimTypePointer(SimTypeFloat(size=64), offset=0), "byteArray16": SimTypePointer(SimStruct({"byteArray16": SimTypeFixedSizeArray(SimTypeChar(label="Byte"), 16)}, name="FWP_BYTE_ARRAY16", pack=False, align=None), offset=0), "byteBlob": SimTypePointer(SimStruct({"size": SimTypeInt(signed=False, label="UInt32"), "data": SimTypePointer(SimTypeChar(label="Byte"), offset=0)}, name="FWP_BYTE_BLOB", pack=False, align=None), offset=0), "sid": SimTypePointer(SimTypeBottom(label="SID"), offset=0), "sd": SimTypePointer(SimStruct({"size": SimTypeInt(signed=False, label="UInt32"), "data": SimTypePointer(SimTypeChar(label="Byte"), offset=0)}, name="FWP_BYTE_BLOB", pack=False, align=None), offset=0), "tokenInformation": SimTypePointer(SimStruct({"sidCount": SimTypeInt(signed=False, label="UInt32"), "sids": SimTypePointer(SimTypeBottom(label="SID_AND_ATTRIBUTES"), offset=0), "restrictedSidCount": SimTypeInt(signed=False, label="UInt32"), "restrictedSids": SimTypePointer(SimTypeBottom(label="SID_AND_ATTRIBUTES"), offset=0)}, name="FWP_TOKEN_INFORMATION", pack=False, align=None), offset=0), "tokenAccessInformation": SimTypePointer(SimStruct({"size": SimTypeInt(signed=False, label="UInt32"), "data": SimTypePointer(SimTypeChar(label="Byte"), offset=0)}, name="FWP_BYTE_BLOB", pack=False, align=None), offset=0), "unicodeString": SimTypePointer(SimTypeChar(label="Char"), offset=0), "byteArray6": SimTypePointer(SimStruct({"byteArray6": SimTypeFixedSizeArray(SimTypeChar(label="Byte"), 6)}, name="FWP_BYTE_ARRAY6", pack=False, align=None), offset=0), "bitmapArray64": SimTypePointer(SimStruct({"bitmapArray64": SimTypeFixedSizeArray(SimTypeChar(label="Byte"), 8)}, name="FWP_BITMAP_ARRAY64_", pack=False, align=None), offset=0)}, name="<anon>", label="None")}, name="FWP_VALUE0", pack=False, align=None)}, name="FWPM_CLASSIFY_OPTION0", pack=False, align=None), offset=0)}, name="FWPM_CLASSIFY_OPTIONS0", pack=False, align=None), offset=0)}, name="<anon>", label="None"), "providerContextId": SimTypeLongLong(signed=False, label="UInt64")}, name="FWPM_PROVIDER_CONTEXT0", pack=False, align=None), offset=0), offset=0), offset=0), SimTypePointer(SimTypeInt(signed=False, label="UInt32"), offset=0)], SimTypeInt(signed=False, label="UInt32"), arg_names=["engineHandle", "enumHandle", "numEntriesRequested", "entries", "numEntriesReturned"]),
        # 
        'FwpmProviderContextEnum1': SimTypeFunction([SimTypePointer(SimTypeInt(signed=True, label="Int"), label="IntPtr", offset=0), SimTypePointer(SimTypeInt(signed=True, label="Int"), label="IntPtr", offset=0), SimTypeInt(signed=False, label="UInt32"), SimTypePointer(SimTypePointer(SimTypePointer(SimStruct({"providerContextKey": SimTypeBottom(label="Guid"), "displayData": SimStruct({"name": SimTypePointer(SimTypeChar(label="Char"), offset=0), "description": SimTypePointer(SimTypeChar(label="Char"), offset=0)}, name="FWPM_DISPLAY_DATA0", pack=False, align=None), "flags": SimTypeInt(signed=False, label="UInt32"), "providerKey": SimTypePointer(SimTypeBottom(label="Guid"), offset=0), "providerData": SimStruct({"size": SimTypeInt(signed=False, label="UInt32"), "data": SimTypePointer(SimTypeChar(label="Byte"), offset=0)}, name="FWP_BYTE_BLOB", pack=False, align=None), "type": SimTypeInt(signed=False, label="FWPM_PROVIDER_CONTEXT_TYPE"), "Anonymous": SimUnion({"keyingPolicy": SimTypePointer(SimStruct({"numKeyMods": SimTypeInt(signed=False, label="UInt32"), "keyModKeys": SimTypePointer(SimTypeBottom(label="Guid"), offset=0)}, name="IPSEC_KEYING_POLICY0", pack=False, align=None), offset=0), "ikeQmTransportPolicy": SimTypePointer(SimStruct({"numIpsecProposals": SimTypeInt(signed=False, label="UInt32"), "ipsecProposals": SimTypePointer(SimStruct({"lifetime": SimStruct({"lifetimeSeconds": SimTypeInt(signed=False, label="UInt32"), "lifetimeKilobytes": SimTypeInt(signed=False, label="UInt32"), "lifetimePackets": SimTypeInt(signed=False, label="UInt32")}, name="IPSEC_SA_LIFETIME0", pack=False, align=None), "numSaTransforms": SimTypeInt(signed=False, label="UInt32"), "saTransforms": SimTypePointer(SimStruct({"ipsecTransformType": SimTypeInt(signed=False, label="IPSEC_TRANSFORM_TYPE"), "Anonymous": SimUnion({"ahTransform": SimTypePointer(SimStruct({"authTransformId": SimStruct({"authType": SimTypeInt(signed=False, label="IPSEC_AUTH_TYPE"), "authConfig": SimTypeChar(label="Byte")}, name="IPSEC_AUTH_TRANSFORM_ID0", pack=False, align=None), "cryptoModuleId": SimTypePointer(SimTypeBottom(label="Guid"), offset=0)}, name="IPSEC_AUTH_TRANSFORM0", pack=False, align=None), offset=0), "espAuthTransform": SimTypePointer(SimStruct({"authTransformId": SimStruct({"authType": SimTypeInt(signed=False, label="IPSEC_AUTH_TYPE"), "authConfig": SimTypeChar(label="Byte")}, name="IPSEC_AUTH_TRANSFORM_ID0", pack=False, align=None), "cryptoModuleId": SimTypePointer(SimTypeBottom(label="Guid"), offset=0)}, name="IPSEC_AUTH_TRANSFORM0", pack=False, align=None), offset=0), "espCipherTransform": SimTypePointer(SimStruct({"cipherTransformId": SimStruct({"cipherType": SimTypeInt(signed=False, label="IPSEC_CIPHER_TYPE"), "cipherConfig": SimTypeChar(label="Byte")}, name="IPSEC_CIPHER_TRANSFORM_ID0", pack=False, align=None), "cryptoModuleId": SimTypePointer(SimTypeBottom(label="Guid"), offset=0)}, name="IPSEC_CIPHER_TRANSFORM0", pack=False, align=None), offset=0), "espAuthAndCipherTransform": SimTypePointer(SimStruct({"authTransform": SimStruct({"authTransformId": SimStruct({"authType": SimTypeInt(signed=False, label="IPSEC_AUTH_TYPE"), "authConfig": SimTypeChar(label="Byte")}, name="IPSEC_AUTH_TRANSFORM_ID0", pack=False, align=None), "cryptoModuleId": SimTypePointer(SimTypeBottom(label="Guid"), offset=0)}, name="IPSEC_AUTH_TRANSFORM0", pack=False, align=None), "cipherTransform": SimStruct({"cipherTransformId": SimStruct({"cipherType": SimTypeInt(signed=False, label="IPSEC_CIPHER_TYPE"), "cipherConfig": SimTypeChar(label="Byte")}, name="IPSEC_CIPHER_TRANSFORM_ID0", pack=False, align=None), "cryptoModuleId": SimTypePointer(SimTypeBottom(label="Guid"), offset=0)}, name="IPSEC_CIPHER_TRANSFORM0", pack=False, align=None)}, name="IPSEC_AUTH_AND_CIPHER_TRANSFORM0", pack=False, align=None), offset=0), "espAuthFwTransform": SimTypePointer(SimStruct({"authTransformId": SimStruct({"authType": SimTypeInt(signed=False, label="IPSEC_AUTH_TYPE"), "authConfig": SimTypeChar(label="Byte")}, name="IPSEC_AUTH_TRANSFORM_ID0", pack=False, align=None), "cryptoModuleId": SimTypePointer(SimTypeBottom(label="Guid"), offset=0)}, name="IPSEC_AUTH_TRANSFORM0", pack=False, align=None), offset=0)}, name="<anon>", label="None")}, name="IPSEC_SA_TRANSFORM0", pack=False, align=None), offset=0), "pfsGroup": SimTypeInt(signed=False, label="IPSEC_PFS_GROUP")}, name="IPSEC_PROPOSAL0", pack=False, align=None), offset=0), "flags": SimTypeInt(signed=False, label="IPSEC_POLICY_FLAG"), "ndAllowClearTimeoutSeconds": SimTypeInt(signed=False, label="UInt32"), "saIdleTimeout": SimStruct({"idleTimeoutSeconds": SimTypeInt(signed=False, label="UInt32"), "idleTimeoutSecondsFailOver": SimTypeInt(signed=False, label="UInt32")}, name="IPSEC_SA_IDLE_TIMEOUT0", pack=False, align=None), "emPolicy": SimTypePointer(SimStruct({"numAuthenticationMethods": SimTypeInt(signed=False, label="UInt32"), "authenticationMethods": SimTypePointer(SimStruct({"authenticationMethodType": SimTypeInt(signed=False, label="IKEEXT_AUTHENTICATION_METHOD_TYPE"), "Anonymous": SimUnion({"presharedKeyAuthentication": SimStruct({"presharedKey": SimStruct({"size": SimTypeInt(signed=False, label="UInt32"), "data": SimTypePointer(SimTypeChar(label="Byte"), offset=0)}, name="FWP_BYTE_BLOB", pack=False, align=None), "flags": SimTypeInt(signed=False, label="IKEEXT_PRESHARED_KEY_AUTHENTICATION_FLAGS")}, name="IKEEXT_PRESHARED_KEY_AUTHENTICATION1", pack=False, align=None), "certificateAuthentication": SimStruct({"inboundConfigType": SimTypeInt(signed=False, label="IKEEXT_CERT_CONFIG_TYPE"), "Anonymous1": SimUnion({"Anonymous": SimStruct({"inboundRootArraySize": SimTypeInt(signed=False, label="UInt32"), "inboundRootArray": SimTypePointer(SimStruct({"certData": SimStruct({"size": SimTypeInt(signed=False, label="UInt32"), "data": SimTypePointer(SimTypeChar(label="Byte"), offset=0)}, name="FWP_BYTE_BLOB", pack=False, align=None), "flags": SimTypeInt(signed=False, label="IKEEXT_CERT_FLAGS")}, name="IKEEXT_CERT_ROOT_CONFIG0", pack=False, align=None), offset=0)}, name="_Anonymous_e__Struct", pack=False, align=None), "inboundEnterpriseStoreConfig": SimTypePointer(SimStruct({"certData": SimStruct({"size": SimTypeInt(signed=False, label="UInt32"), "data": SimTypePointer(SimTypeChar(label="Byte"), offset=0)}, name="FWP_BYTE_BLOB", pack=False, align=None), "flags": SimTypeInt(signed=False, label="IKEEXT_CERT_FLAGS")}, name="IKEEXT_CERT_ROOT_CONFIG0", pack=False, align=None), offset=0), "inboundTrustedRootStoreConfig": SimTypePointer(SimStruct({"certData": SimStruct({"size": SimTypeInt(signed=False, label="UInt32"), "data": SimTypePointer(SimTypeChar(label="Byte"), offset=0)}, name="FWP_BYTE_BLOB", pack=False, align=None), "flags": SimTypeInt(signed=False, label="IKEEXT_CERT_FLAGS")}, name="IKEEXT_CERT_ROOT_CONFIG0", pack=False, align=None), offset=0)}, name="<anon>", label="None"), "outboundConfigType": SimTypeInt(signed=False, label="IKEEXT_CERT_CONFIG_TYPE"), "Anonymous2": SimUnion({"Anonymous": SimStruct({"outboundRootArraySize": SimTypeInt(signed=False, label="UInt32"), "outboundRootArray": SimTypePointer(SimStruct({"certData": SimStruct({"size": SimTypeInt(signed=False, label="UInt32"), "data": SimTypePointer(SimTypeChar(label="Byte"), offset=0)}, name="FWP_BYTE_BLOB", pack=False, align=None), "flags": SimTypeInt(signed=False, label="IKEEXT_CERT_FLAGS")}, name="IKEEXT_CERT_ROOT_CONFIG0", pack=False, align=None), offset=0)}, name="_Anonymous_e__Struct", pack=False, align=None), "outboundEnterpriseStoreConfig": SimTypePointer(SimStruct({"certData": SimStruct({"size": SimTypeInt(signed=False, label="UInt32"), "data": SimTypePointer(SimTypeChar(label="Byte"), offset=0)}, name="FWP_BYTE_BLOB", pack=False, align=None), "flags": SimTypeInt(signed=False, label="IKEEXT_CERT_FLAGS")}, name="IKEEXT_CERT_ROOT_CONFIG0", pack=False, align=None), offset=0), "outboundTrustedRootStoreConfig": SimTypePointer(SimStruct({"certData": SimStruct({"size": SimTypeInt(signed=False, label="UInt32"), "data": SimTypePointer(SimTypeChar(label="Byte"), offset=0)}, name="FWP_BYTE_BLOB", pack=False, align=None), "flags": SimTypeInt(signed=False, label="IKEEXT_CERT_FLAGS")}, name="IKEEXT_CERT_ROOT_CONFIG0", pack=False, align=None), offset=0)}, name="<anon>", label="None"), "flags": SimTypeInt(signed=False, label="IKEEXT_CERT_AUTH"), "localCertLocationUrl": SimStruct({"size": SimTypeInt(signed=False, label="UInt32"), "data": SimTypePointer(SimTypeChar(label="Byte"), offset=0)}, name="FWP_BYTE_BLOB", pack=False, align=None)}, name="IKEEXT_CERTIFICATE_AUTHENTICATION1", pack=False, align=None), "kerberosAuthentication": SimStruct({"flags": SimTypeInt(signed=False, label="IKEEXT_KERBEROS_AUTHENTICATION_FLAGS")}, name="IKEEXT_KERBEROS_AUTHENTICATION0", pack=False, align=None), "ntlmV2Authentication": SimStruct({"flags": SimTypeInt(signed=False, label="UInt32")}, name="IKEEXT_NTLM_V2_AUTHENTICATION0", pack=False, align=None), "sslAuthentication": SimStruct({"inboundConfigType": SimTypeInt(signed=False, label="IKEEXT_CERT_CONFIG_TYPE"), "Anonymous1": SimUnion({"Anonymous": SimStruct({"inboundRootArraySize": SimTypeInt(signed=False, label="UInt32"), "inboundRootArray": SimTypePointer(SimStruct({"certData": SimStruct({"size": SimTypeInt(signed=False, label="UInt32"), "data": SimTypePointer(SimTypeChar(label="Byte"), offset=0)}, name="FWP_BYTE_BLOB", pack=False, align=None), "flags": SimTypeInt(signed=False, label="IKEEXT_CERT_FLAGS")}, name="IKEEXT_CERT_ROOT_CONFIG0", pack=False, align=None), offset=0)}, name="_Anonymous_e__Struct", pack=False, align=None), "inboundEnterpriseStoreConfig": SimTypePointer(SimStruct({"certData": SimStruct({"size": SimTypeInt(signed=False, label="UInt32"), "data": SimTypePointer(SimTypeChar(label="Byte"), offset=0)}, name="FWP_BYTE_BLOB", pack=False, align=None), "flags": SimTypeInt(signed=False, label="IKEEXT_CERT_FLAGS")}, name="IKEEXT_CERT_ROOT_CONFIG0", pack=False, align=None), offset=0), "inboundTrustedRootStoreConfig": SimTypePointer(SimStruct({"certData": SimStruct({"size": SimTypeInt(signed=False, label="UInt32"), "data": SimTypePointer(SimTypeChar(label="Byte"), offset=0)}, name="FWP_BYTE_BLOB", pack=False, align=None), "flags": SimTypeInt(signed=False, label="IKEEXT_CERT_FLAGS")}, name="IKEEXT_CERT_ROOT_CONFIG0", pack=False, align=None), offset=0)}, name="<anon>", label="None"), "outboundConfigType": SimTypeInt(signed=False, label="IKEEXT_CERT_CONFIG_TYPE"), "Anonymous2": SimUnion({"Anonymous": SimStruct({"outboundRootArraySize": SimTypeInt(signed=False, label="UInt32"), "outboundRootArray": SimTypePointer(SimStruct({"certData": SimStruct({"size": SimTypeInt(signed=False, label="UInt32"), "data": SimTypePointer(SimTypeChar(label="Byte"), offset=0)}, name="FWP_BYTE_BLOB", pack=False, align=None), "flags": SimTypeInt(signed=False, label="IKEEXT_CERT_FLAGS")}, name="IKEEXT_CERT_ROOT_CONFIG0", pack=False, align=None), offset=0)}, name="_Anonymous_e__Struct", pack=False, align=None), "outboundEnterpriseStoreConfig": SimTypePointer(SimStruct({"certData": SimStruct({"size": SimTypeInt(signed=False, label="UInt32"), "data": SimTypePointer(SimTypeChar(label="Byte"), offset=0)}, name="FWP_BYTE_BLOB", pack=False, align=None), "flags": SimTypeInt(signed=False, label="IKEEXT_CERT_FLAGS")}, name="IKEEXT_CERT_ROOT_CONFIG0", pack=False, align=None), offset=0), "outboundTrustedRootStoreConfig": SimTypePointer(SimStruct({"certData": SimStruct({"size": SimTypeInt(signed=False, label="UInt32"), "data": SimTypePointer(SimTypeChar(label="Byte"), offset=0)}, name="FWP_BYTE_BLOB", pack=False, align=None), "flags": SimTypeInt(signed=False, label="IKEEXT_CERT_FLAGS")}, name="IKEEXT_CERT_ROOT_CONFIG0", pack=False, align=None), offset=0)}, name="<anon>", label="None"), "flags": SimTypeInt(signed=False, label="IKEEXT_CERT_AUTH"), "localCertLocationUrl": SimStruct({"size": SimTypeInt(signed=False, label="UInt32"), "data": SimTypePointer(SimTypeChar(label="Byte"), offset=0)}, name="FWP_BYTE_BLOB", pack=False, align=None)}, name="IKEEXT_CERTIFICATE_AUTHENTICATION1", pack=False, align=None), "cgaAuthentication": SimStruct({"keyContainerName": SimTypePointer(SimTypeChar(label="Char"), offset=0), "cspName": SimTypePointer(SimTypeChar(label="Char"), offset=0), "cspType": SimTypeInt(signed=False, label="UInt32"), "cgaModifier": SimStruct({"byteArray16": SimTypeFixedSizeArray(SimTypeChar(label="Byte"), 16)}, name="FWP_BYTE_ARRAY16", pack=False, align=None), "cgaCollisionCount": SimTypeChar(label="Byte")}, name="IKEEXT_IPV6_CGA_AUTHENTICATION0", pack=False, align=None), "eapAuthentication": SimStruct({"flags": SimTypeInt(signed=False, label="IKEEXT_EAP_AUTHENTICATION_FLAGS")}, name="IKEEXT_EAP_AUTHENTICATION0", pack=False, align=None)}, name="<anon>", label="None")}, name="IKEEXT_AUTHENTICATION_METHOD1", pack=False, align=None), offset=0), "initiatorImpersonationType": SimTypeInt(signed=False, label="IKEEXT_AUTHENTICATION_IMPERSONATION_TYPE")}, name="IKEEXT_EM_POLICY1", pack=False, align=None), offset=0)}, name="IPSEC_TRANSPORT_POLICY1", pack=False, align=None), offset=0), "ikeQmTunnelPolicy": SimTypePointer(SimStruct({"flags": SimTypeInt(signed=False, label="IPSEC_POLICY_FLAG"), "numIpsecProposals": SimTypeInt(signed=False, label="UInt32"), "ipsecProposals": SimTypePointer(SimStruct({"lifetime": SimStruct({"lifetimeSeconds": SimTypeInt(signed=False, label="UInt32"), "lifetimeKilobytes": SimTypeInt(signed=False, label="UInt32"), "lifetimePackets": SimTypeInt(signed=False, label="UInt32")}, name="IPSEC_SA_LIFETIME0", pack=False, align=None), "numSaTransforms": SimTypeInt(signed=False, label="UInt32"), "saTransforms": SimTypePointer(SimStruct({"ipsecTransformType": SimTypeInt(signed=False, label="IPSEC_TRANSFORM_TYPE"), "Anonymous": SimUnion({"ahTransform": SimTypePointer(SimStruct({"authTransformId": SimStruct({"authType": SimTypeInt(signed=False, label="IPSEC_AUTH_TYPE"), "authConfig": SimTypeChar(label="Byte")}, name="IPSEC_AUTH_TRANSFORM_ID0", pack=False, align=None), "cryptoModuleId": SimTypePointer(SimTypeBottom(label="Guid"), offset=0)}, name="IPSEC_AUTH_TRANSFORM0", pack=False, align=None), offset=0), "espAuthTransform": SimTypePointer(SimStruct({"authTransformId": SimStruct({"authType": SimTypeInt(signed=False, label="IPSEC_AUTH_TYPE"), "authConfig": SimTypeChar(label="Byte")}, name="IPSEC_AUTH_TRANSFORM_ID0", pack=False, align=None), "cryptoModuleId": SimTypePointer(SimTypeBottom(label="Guid"), offset=0)}, name="IPSEC_AUTH_TRANSFORM0", pack=False, align=None), offset=0), "espCipherTransform": SimTypePointer(SimStruct({"cipherTransformId": SimStruct({"cipherType": SimTypeInt(signed=False, label="IPSEC_CIPHER_TYPE"), "cipherConfig": SimTypeChar(label="Byte")}, name="IPSEC_CIPHER_TRANSFORM_ID0", pack=False, align=None), "cryptoModuleId": SimTypePointer(SimTypeBottom(label="Guid"), offset=0)}, name="IPSEC_CIPHER_TRANSFORM0", pack=False, align=None), offset=0), "espAuthAndCipherTransform": SimTypePointer(SimStruct({"authTransform": SimStruct({"authTransformId": SimStruct({"authType": SimTypeInt(signed=False, label="IPSEC_AUTH_TYPE"), "authConfig": SimTypeChar(label="Byte")}, name="IPSEC_AUTH_TRANSFORM_ID0", pack=False, align=None), "cryptoModuleId": SimTypePointer(SimTypeBottom(label="Guid"), offset=0)}, name="IPSEC_AUTH_TRANSFORM0", pack=False, align=None), "cipherTransform": SimStruct({"cipherTransformId": SimStruct({"cipherType": SimTypeInt(signed=False, label="IPSEC_CIPHER_TYPE"), "cipherConfig": SimTypeChar(label="Byte")}, name="IPSEC_CIPHER_TRANSFORM_ID0", pack=False, align=None), "cryptoModuleId": SimTypePointer(SimTypeBottom(label="Guid"), offset=0)}, name="IPSEC_CIPHER_TRANSFORM0", pack=False, align=None)}, name="IPSEC_AUTH_AND_CIPHER_TRANSFORM0", pack=False, align=None), offset=0), "espAuthFwTransform": SimTypePointer(SimStruct({"authTransformId": SimStruct({"authType": SimTypeInt(signed=False, label="IPSEC_AUTH_TYPE"), "authConfig": SimTypeChar(label="Byte")}, name="IPSEC_AUTH_TRANSFORM_ID0", pack=False, align=None), "cryptoModuleId": SimTypePointer(SimTypeBottom(label="Guid"), offset=0)}, name="IPSEC_AUTH_TRANSFORM0", pack=False, align=None), offset=0)}, name="<anon>", label="None")}, name="IPSEC_SA_TRANSFORM0", pack=False, align=None), offset=0), "pfsGroup": SimTypeInt(signed=False, label="IPSEC_PFS_GROUP")}, name="IPSEC_PROPOSAL0", pack=False, align=None), offset=0), "tunnelEndpoints": SimStruct({"ipVersion": SimTypeInt(signed=False, label="FWP_IP_VERSION"), "Anonymous1": SimUnion({"localV4Address": SimTypeInt(signed=False, label="UInt32"), "localV6Address": SimTypeFixedSizeArray(SimTypeChar(label="Byte"), 16)}, name="<anon>", label="None"), "Anonymous2": SimUnion({"remoteV4Address": SimTypeInt(signed=False, label="UInt32"), "remoteV6Address": SimTypeFixedSizeArray(SimTypeChar(label="Byte"), 16)}, name="<anon>", label="None"), "localIfLuid": SimTypeLongLong(signed=False, label="UInt64")}, name="IPSEC_TUNNEL_ENDPOINTS1", pack=False, align=None), "saIdleTimeout": SimStruct({"idleTimeoutSeconds": SimTypeInt(signed=False, label="UInt32"), "idleTimeoutSecondsFailOver": SimTypeInt(signed=False, label="UInt32")}, name="IPSEC_SA_IDLE_TIMEOUT0", pack=False, align=None), "emPolicy": SimTypePointer(SimStruct({"numAuthenticationMethods": SimTypeInt(signed=False, label="UInt32"), "authenticationMethods": SimTypePointer(SimStruct({"authenticationMethodType": SimTypeInt(signed=False, label="IKEEXT_AUTHENTICATION_METHOD_TYPE"), "Anonymous": SimUnion({"presharedKeyAuthentication": SimStruct({"presharedKey": SimStruct({"size": SimTypeInt(signed=False, label="UInt32"), "data": SimTypePointer(SimTypeChar(label="Byte"), offset=0)}, name="FWP_BYTE_BLOB", pack=False, align=None), "flags": SimTypeInt(signed=False, label="IKEEXT_PRESHARED_KEY_AUTHENTICATION_FLAGS")}, name="IKEEXT_PRESHARED_KEY_AUTHENTICATION1", pack=False, align=None), "certificateAuthentication": SimStruct({"inboundConfigType": SimTypeInt(signed=False, label="IKEEXT_CERT_CONFIG_TYPE"), "Anonymous1": SimUnion({"Anonymous": SimStruct({"inboundRootArraySize": SimTypeInt(signed=False, label="UInt32"), "inboundRootArray": SimTypePointer(SimStruct({"certData": SimStruct({"size": SimTypeInt(signed=False, label="UInt32"), "data": SimTypePointer(SimTypeChar(label="Byte"), offset=0)}, name="FWP_BYTE_BLOB", pack=False, align=None), "flags": SimTypeInt(signed=False, label="IKEEXT_CERT_FLAGS")}, name="IKEEXT_CERT_ROOT_CONFIG0", pack=False, align=None), offset=0)}, name="_Anonymous_e__Struct", pack=False, align=None), "inboundEnterpriseStoreConfig": SimTypePointer(SimStruct({"certData": SimStruct({"size": SimTypeInt(signed=False, label="UInt32"), "data": SimTypePointer(SimTypeChar(label="Byte"), offset=0)}, name="FWP_BYTE_BLOB", pack=False, align=None), "flags": SimTypeInt(signed=False, label="IKEEXT_CERT_FLAGS")}, name="IKEEXT_CERT_ROOT_CONFIG0", pack=False, align=None), offset=0), "inboundTrustedRootStoreConfig": SimTypePointer(SimStruct({"certData": SimStruct({"size": SimTypeInt(signed=False, label="UInt32"), "data": SimTypePointer(SimTypeChar(label="Byte"), offset=0)}, name="FWP_BYTE_BLOB", pack=False, align=None), "flags": SimTypeInt(signed=False, label="IKEEXT_CERT_FLAGS")}, name="IKEEXT_CERT_ROOT_CONFIG0", pack=False, align=None), offset=0)}, name="<anon>", label="None"), "outboundConfigType": SimTypeInt(signed=False, label="IKEEXT_CERT_CONFIG_TYPE"), "Anonymous2": SimUnion({"Anonymous": SimStruct({"outboundRootArraySize": SimTypeInt(signed=False, label="UInt32"), "outboundRootArray": SimTypePointer(SimStruct({"certData": SimStruct({"size": SimTypeInt(signed=False, label="UInt32"), "data": SimTypePointer(SimTypeChar(label="Byte"), offset=0)}, name="FWP_BYTE_BLOB", pack=False, align=None), "flags": SimTypeInt(signed=False, label="IKEEXT_CERT_FLAGS")}, name="IKEEXT_CERT_ROOT_CONFIG0", pack=False, align=None), offset=0)}, name="_Anonymous_e__Struct", pack=False, align=None), "outboundEnterpriseStoreConfig": SimTypePointer(SimStruct({"certData": SimStruct({"size": SimTypeInt(signed=False, label="UInt32"), "data": SimTypePointer(SimTypeChar(label="Byte"), offset=0)}, name="FWP_BYTE_BLOB", pack=False, align=None), "flags": SimTypeInt(signed=False, label="IKEEXT_CERT_FLAGS")}, name="IKEEXT_CERT_ROOT_CONFIG0", pack=False, align=None), offset=0), "outboundTrustedRootStoreConfig": SimTypePointer(SimStruct({"certData": SimStruct({"size": SimTypeInt(signed=False, label="UInt32"), "data": SimTypePointer(SimTypeChar(label="Byte"), offset=0)}, name="FWP_BYTE_BLOB", pack=False, align=None), "flags": SimTypeInt(signed=False, label="IKEEXT_CERT_FLAGS")}, name="IKEEXT_CERT_ROOT_CONFIG0", pack=False, align=None), offset=0)}, name="<anon>", label="None"), "flags": SimTypeInt(signed=False, label="IKEEXT_CERT_AUTH"), "localCertLocationUrl": SimStruct({"size": SimTypeInt(signed=False, label="UInt32"), "data": SimTypePointer(SimTypeChar(label="Byte"), offset=0)}, name="FWP_BYTE_BLOB", pack=False, align=None)}, name="IKEEXT_CERTIFICATE_AUTHENTICATION1", pack=False, align=None), "kerberosAuthentication": SimStruct({"flags": SimTypeInt(signed=False, label="IKEEXT_KERBEROS_AUTHENTICATION_FLAGS")}, name="IKEEXT_KERBEROS_AUTHENTICATION0", pack=False, align=None), "ntlmV2Authentication": SimStruct({"flags": SimTypeInt(signed=False, label="UInt32")}, name="IKEEXT_NTLM_V2_AUTHENTICATION0", pack=False, align=None), "sslAuthentication": SimStruct({"inboundConfigType": SimTypeInt(signed=False, label="IKEEXT_CERT_CONFIG_TYPE"), "Anonymous1": SimUnion({"Anonymous": SimStruct({"inboundRootArraySize": SimTypeInt(signed=False, label="UInt32"), "inboundRootArray": SimTypePointer(SimStruct({"certData": SimStruct({"size": SimTypeInt(signed=False, label="UInt32"), "data": SimTypePointer(SimTypeChar(label="Byte"), offset=0)}, name="FWP_BYTE_BLOB", pack=False, align=None), "flags": SimTypeInt(signed=False, label="IKEEXT_CERT_FLAGS")}, name="IKEEXT_CERT_ROOT_CONFIG0", pack=False, align=None), offset=0)}, name="_Anonymous_e__Struct", pack=False, align=None), "inboundEnterpriseStoreConfig": SimTypePointer(SimStruct({"certData": SimStruct({"size": SimTypeInt(signed=False, label="UInt32"), "data": SimTypePointer(SimTypeChar(label="Byte"), offset=0)}, name="FWP_BYTE_BLOB", pack=False, align=None), "flags": SimTypeInt(signed=False, label="IKEEXT_CERT_FLAGS")}, name="IKEEXT_CERT_ROOT_CONFIG0", pack=False, align=None), offset=0), "inboundTrustedRootStoreConfig": SimTypePointer(SimStruct({"certData": SimStruct({"size": SimTypeInt(signed=False, label="UInt32"), "data": SimTypePointer(SimTypeChar(label="Byte"), offset=0)}, name="FWP_BYTE_BLOB", pack=False, align=None), "flags": SimTypeInt(signed=False, label="IKEEXT_CERT_FLAGS")}, name="IKEEXT_CERT_ROOT_CONFIG0", pack=False, align=None), offset=0)}, name="<anon>", label="None"), "outboundConfigType": SimTypeInt(signed=False, label="IKEEXT_CERT_CONFIG_TYPE"), "Anonymous2": SimUnion({"Anonymous": SimStruct({"outboundRootArraySize": SimTypeInt(signed=False, label="UInt32"), "outboundRootArray": SimTypePointer(SimStruct({"certData": SimStruct({"size": SimTypeInt(signed=False, label="UInt32"), "data": SimTypePointer(SimTypeChar(label="Byte"), offset=0)}, name="FWP_BYTE_BLOB", pack=False, align=None), "flags": SimTypeInt(signed=False, label="IKEEXT_CERT_FLAGS")}, name="IKEEXT_CERT_ROOT_CONFIG0", pack=False, align=None), offset=0)}, name="_Anonymous_e__Struct", pack=False, align=None), "outboundEnterpriseStoreConfig": SimTypePointer(SimStruct({"certData": SimStruct({"size": SimTypeInt(signed=False, label="UInt32"), "data": SimTypePointer(SimTypeChar(label="Byte"), offset=0)}, name="FWP_BYTE_BLOB", pack=False, align=None), "flags": SimTypeInt(signed=False, label="IKEEXT_CERT_FLAGS")}, name="IKEEXT_CERT_ROOT_CONFIG0", pack=False, align=None), offset=0), "outboundTrustedRootStoreConfig": SimTypePointer(SimStruct({"certData": SimStruct({"size": SimTypeInt(signed=False, label="UInt32"), "data": SimTypePointer(SimTypeChar(label="Byte"), offset=0)}, name="FWP_BYTE_BLOB", pack=False, align=None), "flags": SimTypeInt(signed=False, label="IKEEXT_CERT_FLAGS")}, name="IKEEXT_CERT_ROOT_CONFIG0", pack=False, align=None), offset=0)}, name="<anon>", label="None"), "flags": SimTypeInt(signed=False, label="IKEEXT_CERT_AUTH"), "localCertLocationUrl": SimStruct({"size": SimTypeInt(signed=False, label="UInt32"), "data": SimTypePointer(SimTypeChar(label="Byte"), offset=0)}, name="FWP_BYTE_BLOB", pack=False, align=None)}, name="IKEEXT_CERTIFICATE_AUTHENTICATION1", pack=False, align=None), "cgaAuthentication": SimStruct({"keyContainerName": SimTypePointer(SimTypeChar(label="Char"), offset=0), "cspName": SimTypePointer(SimTypeChar(label="Char"), offset=0), "cspType": SimTypeInt(signed=False, label="UInt32"), "cgaModifier": SimStruct({"byteArray16": SimTypeFixedSizeArray(SimTypeChar(label="Byte"), 16)}, name="FWP_BYTE_ARRAY16", pack=False, align=None), "cgaCollisionCount": SimTypeChar(label="Byte")}, name="IKEEXT_IPV6_CGA_AUTHENTICATION0", pack=False, align=None), "eapAuthentication": SimStruct({"flags": SimTypeInt(signed=False, label="IKEEXT_EAP_AUTHENTICATION_FLAGS")}, name="IKEEXT_EAP_AUTHENTICATION0", pack=False, align=None)}, name="<anon>", label="None")}, name="IKEEXT_AUTHENTICATION_METHOD1", pack=False, align=None), offset=0), "initiatorImpersonationType": SimTypeInt(signed=False, label="IKEEXT_AUTHENTICATION_IMPERSONATION_TYPE")}, name="IKEEXT_EM_POLICY1", pack=False, align=None), offset=0)}, name="IPSEC_TUNNEL_POLICY1", pack=False, align=None), offset=0), "authipQmTransportPolicy": SimTypePointer(SimStruct({"numIpsecProposals": SimTypeInt(signed=False, label="UInt32"), "ipsecProposals": SimTypePointer(SimStruct({"lifetime": SimStruct({"lifetimeSeconds": SimTypeInt(signed=False, label="UInt32"), "lifetimeKilobytes": SimTypeInt(signed=False, label="UInt32"), "lifetimePackets": SimTypeInt(signed=False, label="UInt32")}, name="IPSEC_SA_LIFETIME0", pack=False, align=None), "numSaTransforms": SimTypeInt(signed=False, label="UInt32"), "saTransforms": SimTypePointer(SimStruct({"ipsecTransformType": SimTypeInt(signed=False, label="IPSEC_TRANSFORM_TYPE"), "Anonymous": SimUnion({"ahTransform": SimTypePointer(SimStruct({"authTransformId": SimStruct({"authType": SimTypeInt(signed=False, label="IPSEC_AUTH_TYPE"), "authConfig": SimTypeChar(label="Byte")}, name="IPSEC_AUTH_TRANSFORM_ID0", pack=False, align=None), "cryptoModuleId": SimTypePointer(SimTypeBottom(label="Guid"), offset=0)}, name="IPSEC_AUTH_TRANSFORM0", pack=False, align=None), offset=0), "espAuthTransform": SimTypePointer(SimStruct({"authTransformId": SimStruct({"authType": SimTypeInt(signed=False, label="IPSEC_AUTH_TYPE"), "authConfig": SimTypeChar(label="Byte")}, name="IPSEC_AUTH_TRANSFORM_ID0", pack=False, align=None), "cryptoModuleId": SimTypePointer(SimTypeBottom(label="Guid"), offset=0)}, name="IPSEC_AUTH_TRANSFORM0", pack=False, align=None), offset=0), "espCipherTransform": SimTypePointer(SimStruct({"cipherTransformId": SimStruct({"cipherType": SimTypeInt(signed=False, label="IPSEC_CIPHER_TYPE"), "cipherConfig": SimTypeChar(label="Byte")}, name="IPSEC_CIPHER_TRANSFORM_ID0", pack=False, align=None), "cryptoModuleId": SimTypePointer(SimTypeBottom(label="Guid"), offset=0)}, name="IPSEC_CIPHER_TRANSFORM0", pack=False, align=None), offset=0), "espAuthAndCipherTransform": SimTypePointer(SimStruct({"authTransform": SimStruct({"authTransformId": SimStruct({"authType": SimTypeInt(signed=False, label="IPSEC_AUTH_TYPE"), "authConfig": SimTypeChar(label="Byte")}, name="IPSEC_AUTH_TRANSFORM_ID0", pack=False, align=None), "cryptoModuleId": SimTypePointer(SimTypeBottom(label="Guid"), offset=0)}, name="IPSEC_AUTH_TRANSFORM0", pack=False, align=None), "cipherTransform": SimStruct({"cipherTransformId": SimStruct({"cipherType": SimTypeInt(signed=False, label="IPSEC_CIPHER_TYPE"), "cipherConfig": SimTypeChar(label="Byte")}, name="IPSEC_CIPHER_TRANSFORM_ID0", pack=False, align=None), "cryptoModuleId": SimTypePointer(SimTypeBottom(label="Guid"), offset=0)}, name="IPSEC_CIPHER_TRANSFORM0", pack=False, align=None)}, name="IPSEC_AUTH_AND_CIPHER_TRANSFORM0", pack=False, align=None), offset=0), "espAuthFwTransform": SimTypePointer(SimStruct({"authTransformId": SimStruct({"authType": SimTypeInt(signed=False, label="IPSEC_AUTH_TYPE"), "authConfig": SimTypeChar(label="Byte")}, name="IPSEC_AUTH_TRANSFORM_ID0", pack=False, align=None), "cryptoModuleId": SimTypePointer(SimTypeBottom(label="Guid"), offset=0)}, name="IPSEC_AUTH_TRANSFORM0", pack=False, align=None), offset=0)}, name="<anon>", label="None")}, name="IPSEC_SA_TRANSFORM0", pack=False, align=None), offset=0), "pfsGroup": SimTypeInt(signed=False, label="IPSEC_PFS_GROUP")}, name="IPSEC_PROPOSAL0", pack=False, align=None), offset=0), "flags": SimTypeInt(signed=False, label="IPSEC_POLICY_FLAG"), "ndAllowClearTimeoutSeconds": SimTypeInt(signed=False, label="UInt32"), "saIdleTimeout": SimStruct({"idleTimeoutSeconds": SimTypeInt(signed=False, label="UInt32"), "idleTimeoutSecondsFailOver": SimTypeInt(signed=False, label="UInt32")}, name="IPSEC_SA_IDLE_TIMEOUT0", pack=False, align=None), "emPolicy": SimTypePointer(SimStruct({"numAuthenticationMethods": SimTypeInt(signed=False, label="UInt32"), "authenticationMethods": SimTypePointer(SimStruct({"authenticationMethodType": SimTypeInt(signed=False, label="IKEEXT_AUTHENTICATION_METHOD_TYPE"), "Anonymous": SimUnion({"presharedKeyAuthentication": SimStruct({"presharedKey": SimStruct({"size": SimTypeInt(signed=False, label="UInt32"), "data": SimTypePointer(SimTypeChar(label="Byte"), offset=0)}, name="FWP_BYTE_BLOB", pack=False, align=None), "flags": SimTypeInt(signed=False, label="IKEEXT_PRESHARED_KEY_AUTHENTICATION_FLAGS")}, name="IKEEXT_PRESHARED_KEY_AUTHENTICATION1", pack=False, align=None), "certificateAuthentication": SimStruct({"inboundConfigType": SimTypeInt(signed=False, label="IKEEXT_CERT_CONFIG_TYPE"), "Anonymous1": SimUnion({"Anonymous": SimStruct({"inboundRootArraySize": SimTypeInt(signed=False, label="UInt32"), "inboundRootArray": SimTypePointer(SimStruct({"certData": SimStruct({"size": SimTypeInt(signed=False, label="UInt32"), "data": SimTypePointer(SimTypeChar(label="Byte"), offset=0)}, name="FWP_BYTE_BLOB", pack=False, align=None), "flags": SimTypeInt(signed=False, label="IKEEXT_CERT_FLAGS")}, name="IKEEXT_CERT_ROOT_CONFIG0", pack=False, align=None), offset=0)}, name="_Anonymous_e__Struct", pack=False, align=None), "inboundEnterpriseStoreConfig": SimTypePointer(SimStruct({"certData": SimStruct({"size": SimTypeInt(signed=False, label="UInt32"), "data": SimTypePointer(SimTypeChar(label="Byte"), offset=0)}, name="FWP_BYTE_BLOB", pack=False, align=None), "flags": SimTypeInt(signed=False, label="IKEEXT_CERT_FLAGS")}, name="IKEEXT_CERT_ROOT_CONFIG0", pack=False, align=None), offset=0), "inboundTrustedRootStoreConfig": SimTypePointer(SimStruct({"certData": SimStruct({"size": SimTypeInt(signed=False, label="UInt32"), "data": SimTypePointer(SimTypeChar(label="Byte"), offset=0)}, name="FWP_BYTE_BLOB", pack=False, align=None), "flags": SimTypeInt(signed=False, label="IKEEXT_CERT_FLAGS")}, name="IKEEXT_CERT_ROOT_CONFIG0", pack=False, align=None), offset=0)}, name="<anon>", label="None"), "outboundConfigType": SimTypeInt(signed=False, label="IKEEXT_CERT_CONFIG_TYPE"), "Anonymous2": SimUnion({"Anonymous": SimStruct({"outboundRootArraySize": SimTypeInt(signed=False, label="UInt32"), "outboundRootArray": SimTypePointer(SimStruct({"certData": SimStruct({"size": SimTypeInt(signed=False, label="UInt32"), "data": SimTypePointer(SimTypeChar(label="Byte"), offset=0)}, name="FWP_BYTE_BLOB", pack=False, align=None), "flags": SimTypeInt(signed=False, label="IKEEXT_CERT_FLAGS")}, name="IKEEXT_CERT_ROOT_CONFIG0", pack=False, align=None), offset=0)}, name="_Anonymous_e__Struct", pack=False, align=None), "outboundEnterpriseStoreConfig": SimTypePointer(SimStruct({"certData": SimStruct({"size": SimTypeInt(signed=False, label="UInt32"), "data": SimTypePointer(SimTypeChar(label="Byte"), offset=0)}, name="FWP_BYTE_BLOB", pack=False, align=None), "flags": SimTypeInt(signed=False, label="IKEEXT_CERT_FLAGS")}, name="IKEEXT_CERT_ROOT_CONFIG0", pack=False, align=None), offset=0), "outboundTrustedRootStoreConfig": SimTypePointer(SimStruct({"certData": SimStruct({"size": SimTypeInt(signed=False, label="UInt32"), "data": SimTypePointer(SimTypeChar(label="Byte"), offset=0)}, name="FWP_BYTE_BLOB", pack=False, align=None), "flags": SimTypeInt(signed=False, label="IKEEXT_CERT_FLAGS")}, name="IKEEXT_CERT_ROOT_CONFIG0", pack=False, align=None), offset=0)}, name="<anon>", label="None"), "flags": SimTypeInt(signed=False, label="IKEEXT_CERT_AUTH"), "localCertLocationUrl": SimStruct({"size": SimTypeInt(signed=False, label="UInt32"), "data": SimTypePointer(SimTypeChar(label="Byte"), offset=0)}, name="FWP_BYTE_BLOB", pack=False, align=None)}, name="IKEEXT_CERTIFICATE_AUTHENTICATION1", pack=False, align=None), "kerberosAuthentication": SimStruct({"flags": SimTypeInt(signed=False, label="IKEEXT_KERBEROS_AUTHENTICATION_FLAGS")}, name="IKEEXT_KERBEROS_AUTHENTICATION0", pack=False, align=None), "ntlmV2Authentication": SimStruct({"flags": SimTypeInt(signed=False, label="UInt32")}, name="IKEEXT_NTLM_V2_AUTHENTICATION0", pack=False, align=None), "sslAuthentication": SimStruct({"inboundConfigType": SimTypeInt(signed=False, label="IKEEXT_CERT_CONFIG_TYPE"), "Anonymous1": SimUnion({"Anonymous": SimStruct({"inboundRootArraySize": SimTypeInt(signed=False, label="UInt32"), "inboundRootArray": SimTypePointer(SimStruct({"certData": SimStruct({"size": SimTypeInt(signed=False, label="UInt32"), "data": SimTypePointer(SimTypeChar(label="Byte"), offset=0)}, name="FWP_BYTE_BLOB", pack=False, align=None), "flags": SimTypeInt(signed=False, label="IKEEXT_CERT_FLAGS")}, name="IKEEXT_CERT_ROOT_CONFIG0", pack=False, align=None), offset=0)}, name="_Anonymous_e__Struct", pack=False, align=None), "inboundEnterpriseStoreConfig": SimTypePointer(SimStruct({"certData": SimStruct({"size": SimTypeInt(signed=False, label="UInt32"), "data": SimTypePointer(SimTypeChar(label="Byte"), offset=0)}, name="FWP_BYTE_BLOB", pack=False, align=None), "flags": SimTypeInt(signed=False, label="IKEEXT_CERT_FLAGS")}, name="IKEEXT_CERT_ROOT_CONFIG0", pack=False, align=None), offset=0), "inboundTrustedRootStoreConfig": SimTypePointer(SimStruct({"certData": SimStruct({"size": SimTypeInt(signed=False, label="UInt32"), "data": SimTypePointer(SimTypeChar(label="Byte"), offset=0)}, name="FWP_BYTE_BLOB", pack=False, align=None), "flags": SimTypeInt(signed=False, label="IKEEXT_CERT_FLAGS")}, name="IKEEXT_CERT_ROOT_CONFIG0", pack=False, align=None), offset=0)}, name="<anon>", label="None"), "outboundConfigType": SimTypeInt(signed=False, label="IKEEXT_CERT_CONFIG_TYPE"), "Anonymous2": SimUnion({"Anonymous": SimStruct({"outboundRootArraySize": SimTypeInt(signed=False, label="UInt32"), "outboundRootArray": SimTypePointer(SimStruct({"certData": SimStruct({"size": SimTypeInt(signed=False, label="UInt32"), "data": SimTypePointer(SimTypeChar(label="Byte"), offset=0)}, name="FWP_BYTE_BLOB", pack=False, align=None), "flags": SimTypeInt(signed=False, label="IKEEXT_CERT_FLAGS")}, name="IKEEXT_CERT_ROOT_CONFIG0", pack=False, align=None), offset=0)}, name="_Anonymous_e__Struct", pack=False, align=None), "outboundEnterpriseStoreConfig": SimTypePointer(SimStruct({"certData": SimStruct({"size": SimTypeInt(signed=False, label="UInt32"), "data": SimTypePointer(SimTypeChar(label="Byte"), offset=0)}, name="FWP_BYTE_BLOB", pack=False, align=None), "flags": SimTypeInt(signed=False, label="IKEEXT_CERT_FLAGS")}, name="IKEEXT_CERT_ROOT_CONFIG0", pack=False, align=None), offset=0), "outboundTrustedRootStoreConfig": SimTypePointer(SimStruct({"certData": SimStruct({"size": SimTypeInt(signed=False, label="UInt32"), "data": SimTypePointer(SimTypeChar(label="Byte"), offset=0)}, name="FWP_BYTE_BLOB", pack=False, align=None), "flags": SimTypeInt(signed=False, label="IKEEXT_CERT_FLAGS")}, name="IKEEXT_CERT_ROOT_CONFIG0", pack=False, align=None), offset=0)}, name="<anon>", label="None"), "flags": SimTypeInt(signed=False, label="IKEEXT_CERT_AUTH"), "localCertLocationUrl": SimStruct({"size": SimTypeInt(signed=False, label="UInt32"), "data": SimTypePointer(SimTypeChar(label="Byte"), offset=0)}, name="FWP_BYTE_BLOB", pack=False, align=None)}, name="IKEEXT_CERTIFICATE_AUTHENTICATION1", pack=False, align=None), "cgaAuthentication": SimStruct({"keyContainerName": SimTypePointer(SimTypeChar(label="Char"), offset=0), "cspName": SimTypePointer(SimTypeChar(label="Char"), offset=0), "cspType": SimTypeInt(signed=False, label="UInt32"), "cgaModifier": SimStruct({"byteArray16": SimTypeFixedSizeArray(SimTypeChar(label="Byte"), 16)}, name="FWP_BYTE_ARRAY16", pack=False, align=None), "cgaCollisionCount": SimTypeChar(label="Byte")}, name="IKEEXT_IPV6_CGA_AUTHENTICATION0", pack=False, align=None), "eapAuthentication": SimStruct({"flags": SimTypeInt(signed=False, label="IKEEXT_EAP_AUTHENTICATION_FLAGS")}, name="IKEEXT_EAP_AUTHENTICATION0", pack=False, align=None)}, name="<anon>", label="None")}, name="IKEEXT_AUTHENTICATION_METHOD1", pack=False, align=None), offset=0), "initiatorImpersonationType": SimTypeInt(signed=False, label="IKEEXT_AUTHENTICATION_IMPERSONATION_TYPE")}, name="IKEEXT_EM_POLICY1", pack=False, align=None), offset=0)}, name="IPSEC_TRANSPORT_POLICY1", pack=False, align=None), offset=0), "authipQmTunnelPolicy": SimTypePointer(SimStruct({"flags": SimTypeInt(signed=False, label="IPSEC_POLICY_FLAG"), "numIpsecProposals": SimTypeInt(signed=False, label="UInt32"), "ipsecProposals": SimTypePointer(SimStruct({"lifetime": SimStruct({"lifetimeSeconds": SimTypeInt(signed=False, label="UInt32"), "lifetimeKilobytes": SimTypeInt(signed=False, label="UInt32"), "lifetimePackets": SimTypeInt(signed=False, label="UInt32")}, name="IPSEC_SA_LIFETIME0", pack=False, align=None), "numSaTransforms": SimTypeInt(signed=False, label="UInt32"), "saTransforms": SimTypePointer(SimStruct({"ipsecTransformType": SimTypeInt(signed=False, label="IPSEC_TRANSFORM_TYPE"), "Anonymous": SimUnion({"ahTransform": SimTypePointer(SimStruct({"authTransformId": SimStruct({"authType": SimTypeInt(signed=False, label="IPSEC_AUTH_TYPE"), "authConfig": SimTypeChar(label="Byte")}, name="IPSEC_AUTH_TRANSFORM_ID0", pack=False, align=None), "cryptoModuleId": SimTypePointer(SimTypeBottom(label="Guid"), offset=0)}, name="IPSEC_AUTH_TRANSFORM0", pack=False, align=None), offset=0), "espAuthTransform": SimTypePointer(SimStruct({"authTransformId": SimStruct({"authType": SimTypeInt(signed=False, label="IPSEC_AUTH_TYPE"), "authConfig": SimTypeChar(label="Byte")}, name="IPSEC_AUTH_TRANSFORM_ID0", pack=False, align=None), "cryptoModuleId": SimTypePointer(SimTypeBottom(label="Guid"), offset=0)}, name="IPSEC_AUTH_TRANSFORM0", pack=False, align=None), offset=0), "espCipherTransform": SimTypePointer(SimStruct({"cipherTransformId": SimStruct({"cipherType": SimTypeInt(signed=False, label="IPSEC_CIPHER_TYPE"), "cipherConfig": SimTypeChar(label="Byte")}, name="IPSEC_CIPHER_TRANSFORM_ID0", pack=False, align=None), "cryptoModuleId": SimTypePointer(SimTypeBottom(label="Guid"), offset=0)}, name="IPSEC_CIPHER_TRANSFORM0", pack=False, align=None), offset=0), "espAuthAndCipherTransform": SimTypePointer(SimStruct({"authTransform": SimStruct({"authTransformId": SimStruct({"authType": SimTypeInt(signed=False, label="IPSEC_AUTH_TYPE"), "authConfig": SimTypeChar(label="Byte")}, name="IPSEC_AUTH_TRANSFORM_ID0", pack=False, align=None), "cryptoModuleId": SimTypePointer(SimTypeBottom(label="Guid"), offset=0)}, name="IPSEC_AUTH_TRANSFORM0", pack=False, align=None), "cipherTransform": SimStruct({"cipherTransformId": SimStruct({"cipherType": SimTypeInt(signed=False, label="IPSEC_CIPHER_TYPE"), "cipherConfig": SimTypeChar(label="Byte")}, name="IPSEC_CIPHER_TRANSFORM_ID0", pack=False, align=None), "cryptoModuleId": SimTypePointer(SimTypeBottom(label="Guid"), offset=0)}, name="IPSEC_CIPHER_TRANSFORM0", pack=False, align=None)}, name="IPSEC_AUTH_AND_CIPHER_TRANSFORM0", pack=False, align=None), offset=0), "espAuthFwTransform": SimTypePointer(SimStruct({"authTransformId": SimStruct({"authType": SimTypeInt(signed=False, label="IPSEC_AUTH_TYPE"), "authConfig": SimTypeChar(label="Byte")}, name="IPSEC_AUTH_TRANSFORM_ID0", pack=False, align=None), "cryptoModuleId": SimTypePointer(SimTypeBottom(label="Guid"), offset=0)}, name="IPSEC_AUTH_TRANSFORM0", pack=False, align=None), offset=0)}, name="<anon>", label="None")}, name="IPSEC_SA_TRANSFORM0", pack=False, align=None), offset=0), "pfsGroup": SimTypeInt(signed=False, label="IPSEC_PFS_GROUP")}, name="IPSEC_PROPOSAL0", pack=False, align=None), offset=0), "tunnelEndpoints": SimStruct({"ipVersion": SimTypeInt(signed=False, label="FWP_IP_VERSION"), "Anonymous1": SimUnion({"localV4Address": SimTypeInt(signed=False, label="UInt32"), "localV6Address": SimTypeFixedSizeArray(SimTypeChar(label="Byte"), 16)}, name="<anon>", label="None"), "Anonymous2": SimUnion({"remoteV4Address": SimTypeInt(signed=False, label="UInt32"), "remoteV6Address": SimTypeFixedSizeArray(SimTypeChar(label="Byte"), 16)}, name="<anon>", label="None"), "localIfLuid": SimTypeLongLong(signed=False, label="UInt64")}, name="IPSEC_TUNNEL_ENDPOINTS1", pack=False, align=None), "saIdleTimeout": SimStruct({"idleTimeoutSeconds": SimTypeInt(signed=False, label="UInt32"), "idleTimeoutSecondsFailOver": SimTypeInt(signed=False, label="UInt32")}, name="IPSEC_SA_IDLE_TIMEOUT0", pack=False, align=None), "emPolicy": SimTypePointer(SimStruct({"numAuthenticationMethods": SimTypeInt(signed=False, label="UInt32"), "authenticationMethods": SimTypePointer(SimStruct({"authenticationMethodType": SimTypeInt(signed=False, label="IKEEXT_AUTHENTICATION_METHOD_TYPE"), "Anonymous": SimUnion({"presharedKeyAuthentication": SimStruct({"presharedKey": SimStruct({"size": SimTypeInt(signed=False, label="UInt32"), "data": SimTypePointer(SimTypeChar(label="Byte"), offset=0)}, name="FWP_BYTE_BLOB", pack=False, align=None), "flags": SimTypeInt(signed=False, label="IKEEXT_PRESHARED_KEY_AUTHENTICATION_FLAGS")}, name="IKEEXT_PRESHARED_KEY_AUTHENTICATION1", pack=False, align=None), "certificateAuthentication": SimStruct({"inboundConfigType": SimTypeInt(signed=False, label="IKEEXT_CERT_CONFIG_TYPE"), "Anonymous1": SimUnion({"Anonymous": SimStruct({"inboundRootArraySize": SimTypeInt(signed=False, label="UInt32"), "inboundRootArray": SimTypePointer(SimStruct({"certData": SimStruct({"size": SimTypeInt(signed=False, label="UInt32"), "data": SimTypePointer(SimTypeChar(label="Byte"), offset=0)}, name="FWP_BYTE_BLOB", pack=False, align=None), "flags": SimTypeInt(signed=False, label="IKEEXT_CERT_FLAGS")}, name="IKEEXT_CERT_ROOT_CONFIG0", pack=False, align=None), offset=0)}, name="_Anonymous_e__Struct", pack=False, align=None), "inboundEnterpriseStoreConfig": SimTypePointer(SimStruct({"certData": SimStruct({"size": SimTypeInt(signed=False, label="UInt32"), "data": SimTypePointer(SimTypeChar(label="Byte"), offset=0)}, name="FWP_BYTE_BLOB", pack=False, align=None), "flags": SimTypeInt(signed=False, label="IKEEXT_CERT_FLAGS")}, name="IKEEXT_CERT_ROOT_CONFIG0", pack=False, align=None), offset=0), "inboundTrustedRootStoreConfig": SimTypePointer(SimStruct({"certData": SimStruct({"size": SimTypeInt(signed=False, label="UInt32"), "data": SimTypePointer(SimTypeChar(label="Byte"), offset=0)}, name="FWP_BYTE_BLOB", pack=False, align=None), "flags": SimTypeInt(signed=False, label="IKEEXT_CERT_FLAGS")}, name="IKEEXT_CERT_ROOT_CONFIG0", pack=False, align=None), offset=0)}, name="<anon>", label="None"), "outboundConfigType": SimTypeInt(signed=False, label="IKEEXT_CERT_CONFIG_TYPE"), "Anonymous2": SimUnion({"Anonymous": SimStruct({"outboundRootArraySize": SimTypeInt(signed=False, label="UInt32"), "outboundRootArray": SimTypePointer(SimStruct({"certData": SimStruct({"size": SimTypeInt(signed=False, label="UInt32"), "data": SimTypePointer(SimTypeChar(label="Byte"), offset=0)}, name="FWP_BYTE_BLOB", pack=False, align=None), "flags": SimTypeInt(signed=False, label="IKEEXT_CERT_FLAGS")}, name="IKEEXT_CERT_ROOT_CONFIG0", pack=False, align=None), offset=0)}, name="_Anonymous_e__Struct", pack=False, align=None), "outboundEnterpriseStoreConfig": SimTypePointer(SimStruct({"certData": SimStruct({"size": SimTypeInt(signed=False, label="UInt32"), "data": SimTypePointer(SimTypeChar(label="Byte"), offset=0)}, name="FWP_BYTE_BLOB", pack=False, align=None), "flags": SimTypeInt(signed=False, label="IKEEXT_CERT_FLAGS")}, name="IKEEXT_CERT_ROOT_CONFIG0", pack=False, align=None), offset=0), "outboundTrustedRootStoreConfig": SimTypePointer(SimStruct({"certData": SimStruct({"size": SimTypeInt(signed=False, label="UInt32"), "data": SimTypePointer(SimTypeChar(label="Byte"), offset=0)}, name="FWP_BYTE_BLOB", pack=False, align=None), "flags": SimTypeInt(signed=False, label="IKEEXT_CERT_FLAGS")}, name="IKEEXT_CERT_ROOT_CONFIG0", pack=False, align=None), offset=0)}, name="<anon>", label="None"), "flags": SimTypeInt(signed=False, label="IKEEXT_CERT_AUTH"), "localCertLocationUrl": SimStruct({"size": SimTypeInt(signed=False, label="UInt32"), "data": SimTypePointer(SimTypeChar(label="Byte"), offset=0)}, name="FWP_BYTE_BLOB", pack=False, align=None)}, name="IKEEXT_CERTIFICATE_AUTHENTICATION1", pack=False, align=None), "kerberosAuthentication": SimStruct({"flags": SimTypeInt(signed=False, label="IKEEXT_KERBEROS_AUTHENTICATION_FLAGS")}, name="IKEEXT_KERBEROS_AUTHENTICATION0", pack=False, align=None), "ntlmV2Authentication": SimStruct({"flags": SimTypeInt(signed=False, label="UInt32")}, name="IKEEXT_NTLM_V2_AUTHENTICATION0", pack=False, align=None), "sslAuthentication": SimStruct({"inboundConfigType": SimTypeInt(signed=False, label="IKEEXT_CERT_CONFIG_TYPE"), "Anonymous1": SimUnion({"Anonymous": SimStruct({"inboundRootArraySize": SimTypeInt(signed=False, label="UInt32"), "inboundRootArray": SimTypePointer(SimStruct({"certData": SimStruct({"size": SimTypeInt(signed=False, label="UInt32"), "data": SimTypePointer(SimTypeChar(label="Byte"), offset=0)}, name="FWP_BYTE_BLOB", pack=False, align=None), "flags": SimTypeInt(signed=False, label="IKEEXT_CERT_FLAGS")}, name="IKEEXT_CERT_ROOT_CONFIG0", pack=False, align=None), offset=0)}, name="_Anonymous_e__Struct", pack=False, align=None), "inboundEnterpriseStoreConfig": SimTypePointer(SimStruct({"certData": SimStruct({"size": SimTypeInt(signed=False, label="UInt32"), "data": SimTypePointer(SimTypeChar(label="Byte"), offset=0)}, name="FWP_BYTE_BLOB", pack=False, align=None), "flags": SimTypeInt(signed=False, label="IKEEXT_CERT_FLAGS")}, name="IKEEXT_CERT_ROOT_CONFIG0", pack=False, align=None), offset=0), "inboundTrustedRootStoreConfig": SimTypePointer(SimStruct({"certData": SimStruct({"size": SimTypeInt(signed=False, label="UInt32"), "data": SimTypePointer(SimTypeChar(label="Byte"), offset=0)}, name="FWP_BYTE_BLOB", pack=False, align=None), "flags": SimTypeInt(signed=False, label="IKEEXT_CERT_FLAGS")}, name="IKEEXT_CERT_ROOT_CONFIG0", pack=False, align=None), offset=0)}, name="<anon>", label="None"), "outboundConfigType": SimTypeInt(signed=False, label="IKEEXT_CERT_CONFIG_TYPE"), "Anonymous2": SimUnion({"Anonymous": SimStruct({"outboundRootArraySize": SimTypeInt(signed=False, label="UInt32"), "outboundRootArray": SimTypePointer(SimStruct({"certData": SimStruct({"size": SimTypeInt(signed=False, label="UInt32"), "data": SimTypePointer(SimTypeChar(label="Byte"), offset=0)}, name="FWP_BYTE_BLOB", pack=False, align=None), "flags": SimTypeInt(signed=False, label="IKEEXT_CERT_FLAGS")}, name="IKEEXT_CERT_ROOT_CONFIG0", pack=False, align=None), offset=0)}, name="_Anonymous_e__Struct", pack=False, align=None), "outboundEnterpriseStoreConfig": SimTypePointer(SimStruct({"certData": SimStruct({"size": SimTypeInt(signed=False, label="UInt32"), "data": SimTypePointer(SimTypeChar(label="Byte"), offset=0)}, name="FWP_BYTE_BLOB", pack=False, align=None), "flags": SimTypeInt(signed=False, label="IKEEXT_CERT_FLAGS")}, name="IKEEXT_CERT_ROOT_CONFIG0", pack=False, align=None), offset=0), "outboundTrustedRootStoreConfig": SimTypePointer(SimStruct({"certData": SimStruct({"size": SimTypeInt(signed=False, label="UInt32"), "data": SimTypePointer(SimTypeChar(label="Byte"), offset=0)}, name="FWP_BYTE_BLOB", pack=False, align=None), "flags": SimTypeInt(signed=False, label="IKEEXT_CERT_FLAGS")}, name="IKEEXT_CERT_ROOT_CONFIG0", pack=False, align=None), offset=0)}, name="<anon>", label="None"), "flags": SimTypeInt(signed=False, label="IKEEXT_CERT_AUTH"), "localCertLocationUrl": SimStruct({"size": SimTypeInt(signed=False, label="UInt32"), "data": SimTypePointer(SimTypeChar(label="Byte"), offset=0)}, name="FWP_BYTE_BLOB", pack=False, align=None)}, name="IKEEXT_CERTIFICATE_AUTHENTICATION1", pack=False, align=None), "cgaAuthentication": SimStruct({"keyContainerName": SimTypePointer(SimTypeChar(label="Char"), offset=0), "cspName": SimTypePointer(SimTypeChar(label="Char"), offset=0), "cspType": SimTypeInt(signed=False, label="UInt32"), "cgaModifier": SimStruct({"byteArray16": SimTypeFixedSizeArray(SimTypeChar(label="Byte"), 16)}, name="FWP_BYTE_ARRAY16", pack=False, align=None), "cgaCollisionCount": SimTypeChar(label="Byte")}, name="IKEEXT_IPV6_CGA_AUTHENTICATION0", pack=False, align=None), "eapAuthentication": SimStruct({"flags": SimTypeInt(signed=False, label="IKEEXT_EAP_AUTHENTICATION_FLAGS")}, name="IKEEXT_EAP_AUTHENTICATION0", pack=False, align=None)}, name="<anon>", label="None")}, name="IKEEXT_AUTHENTICATION_METHOD1", pack=False, align=None), offset=0), "initiatorImpersonationType": SimTypeInt(signed=False, label="IKEEXT_AUTHENTICATION_IMPERSONATION_TYPE")}, name="IKEEXT_EM_POLICY1", pack=False, align=None), offset=0)}, name="IPSEC_TUNNEL_POLICY1", pack=False, align=None), offset=0), "ikeMmPolicy": SimTypePointer(SimStruct({"softExpirationTime": SimTypeInt(signed=False, label="UInt32"), "numAuthenticationMethods": SimTypeInt(signed=False, label="UInt32"), "authenticationMethods": SimTypePointer(SimStruct({"authenticationMethodType": SimTypeInt(signed=False, label="IKEEXT_AUTHENTICATION_METHOD_TYPE"), "Anonymous": SimUnion({"presharedKeyAuthentication": SimStruct({"presharedKey": SimStruct({"size": SimTypeInt(signed=False, label="UInt32"), "data": SimTypePointer(SimTypeChar(label="Byte"), offset=0)}, name="FWP_BYTE_BLOB", pack=False, align=None), "flags": SimTypeInt(signed=False, label="IKEEXT_PRESHARED_KEY_AUTHENTICATION_FLAGS")}, name="IKEEXT_PRESHARED_KEY_AUTHENTICATION1", pack=False, align=None), "certificateAuthentication": SimStruct({"inboundConfigType": SimTypeInt(signed=False, label="IKEEXT_CERT_CONFIG_TYPE"), "Anonymous1": SimUnion({"Anonymous": SimStruct({"inboundRootArraySize": SimTypeInt(signed=False, label="UInt32"), "inboundRootArray": SimTypePointer(SimStruct({"certData": SimStruct({"size": SimTypeInt(signed=False, label="UInt32"), "data": SimTypePointer(SimTypeChar(label="Byte"), offset=0)}, name="FWP_BYTE_BLOB", pack=False, align=None), "flags": SimTypeInt(signed=False, label="IKEEXT_CERT_FLAGS")}, name="IKEEXT_CERT_ROOT_CONFIG0", pack=False, align=None), offset=0)}, name="_Anonymous_e__Struct", pack=False, align=None), "inboundEnterpriseStoreConfig": SimTypePointer(SimStruct({"certData": SimStruct({"size": SimTypeInt(signed=False, label="UInt32"), "data": SimTypePointer(SimTypeChar(label="Byte"), offset=0)}, name="FWP_BYTE_BLOB", pack=False, align=None), "flags": SimTypeInt(signed=False, label="IKEEXT_CERT_FLAGS")}, name="IKEEXT_CERT_ROOT_CONFIG0", pack=False, align=None), offset=0), "inboundTrustedRootStoreConfig": SimTypePointer(SimStruct({"certData": SimStruct({"size": SimTypeInt(signed=False, label="UInt32"), "data": SimTypePointer(SimTypeChar(label="Byte"), offset=0)}, name="FWP_BYTE_BLOB", pack=False, align=None), "flags": SimTypeInt(signed=False, label="IKEEXT_CERT_FLAGS")}, name="IKEEXT_CERT_ROOT_CONFIG0", pack=False, align=None), offset=0)}, name="<anon>", label="None"), "outboundConfigType": SimTypeInt(signed=False, label="IKEEXT_CERT_CONFIG_TYPE"), "Anonymous2": SimUnion({"Anonymous": SimStruct({"outboundRootArraySize": SimTypeInt(signed=False, label="UInt32"), "outboundRootArray": SimTypePointer(SimStruct({"certData": SimStruct({"size": SimTypeInt(signed=False, label="UInt32"), "data": SimTypePointer(SimTypeChar(label="Byte"), offset=0)}, name="FWP_BYTE_BLOB", pack=False, align=None), "flags": SimTypeInt(signed=False, label="IKEEXT_CERT_FLAGS")}, name="IKEEXT_CERT_ROOT_CONFIG0", pack=False, align=None), offset=0)}, name="_Anonymous_e__Struct", pack=False, align=None), "outboundEnterpriseStoreConfig": SimTypePointer(SimStruct({"certData": SimStruct({"size": SimTypeInt(signed=False, label="UInt32"), "data": SimTypePointer(SimTypeChar(label="Byte"), offset=0)}, name="FWP_BYTE_BLOB", pack=False, align=None), "flags": SimTypeInt(signed=False, label="IKEEXT_CERT_FLAGS")}, name="IKEEXT_CERT_ROOT_CONFIG0", pack=False, align=None), offset=0), "outboundTrustedRootStoreConfig": SimTypePointer(SimStruct({"certData": SimStruct({"size": SimTypeInt(signed=False, label="UInt32"), "data": SimTypePointer(SimTypeChar(label="Byte"), offset=0)}, name="FWP_BYTE_BLOB", pack=False, align=None), "flags": SimTypeInt(signed=False, label="IKEEXT_CERT_FLAGS")}, name="IKEEXT_CERT_ROOT_CONFIG0", pack=False, align=None), offset=0)}, name="<anon>", label="None"), "flags": SimTypeInt(signed=False, label="IKEEXT_CERT_AUTH"), "localCertLocationUrl": SimStruct({"size": SimTypeInt(signed=False, label="UInt32"), "data": SimTypePointer(SimTypeChar(label="Byte"), offset=0)}, name="FWP_BYTE_BLOB", pack=False, align=None)}, name="IKEEXT_CERTIFICATE_AUTHENTICATION1", pack=False, align=None), "kerberosAuthentication": SimStruct({"flags": SimTypeInt(signed=False, label="IKEEXT_KERBEROS_AUTHENTICATION_FLAGS")}, name="IKEEXT_KERBEROS_AUTHENTICATION0", pack=False, align=None), "ntlmV2Authentication": SimStruct({"flags": SimTypeInt(signed=False, label="UInt32")}, name="IKEEXT_NTLM_V2_AUTHENTICATION0", pack=False, align=None), "sslAuthentication": SimStruct({"inboundConfigType": SimTypeInt(signed=False, label="IKEEXT_CERT_CONFIG_TYPE"), "Anonymous1": SimUnion({"Anonymous": SimStruct({"inboundRootArraySize": SimTypeInt(signed=False, label="UInt32"), "inboundRootArray": SimTypePointer(SimStruct({"certData": SimStruct({"size": SimTypeInt(signed=False, label="UInt32"), "data": SimTypePointer(SimTypeChar(label="Byte"), offset=0)}, name="FWP_BYTE_BLOB", pack=False, align=None), "flags": SimTypeInt(signed=False, label="IKEEXT_CERT_FLAGS")}, name="IKEEXT_CERT_ROOT_CONFIG0", pack=False, align=None), offset=0)}, name="_Anonymous_e__Struct", pack=False, align=None), "inboundEnterpriseStoreConfig": SimTypePointer(SimStruct({"certData": SimStruct({"size": SimTypeInt(signed=False, label="UInt32"), "data": SimTypePointer(SimTypeChar(label="Byte"), offset=0)}, name="FWP_BYTE_BLOB", pack=False, align=None), "flags": SimTypeInt(signed=False, label="IKEEXT_CERT_FLAGS")}, name="IKEEXT_CERT_ROOT_CONFIG0", pack=False, align=None), offset=0), "inboundTrustedRootStoreConfig": SimTypePointer(SimStruct({"certData": SimStruct({"size": SimTypeInt(signed=False, label="UInt32"), "data": SimTypePointer(SimTypeChar(label="Byte"), offset=0)}, name="FWP_BYTE_BLOB", pack=False, align=None), "flags": SimTypeInt(signed=False, label="IKEEXT_CERT_FLAGS")}, name="IKEEXT_CERT_ROOT_CONFIG0", pack=False, align=None), offset=0)}, name="<anon>", label="None"), "outboundConfigType": SimTypeInt(signed=False, label="IKEEXT_CERT_CONFIG_TYPE"), "Anonymous2": SimUnion({"Anonymous": SimStruct({"outboundRootArraySize": SimTypeInt(signed=False, label="UInt32"), "outboundRootArray": SimTypePointer(SimStruct({"certData": SimStruct({"size": SimTypeInt(signed=False, label="UInt32"), "data": SimTypePointer(SimTypeChar(label="Byte"), offset=0)}, name="FWP_BYTE_BLOB", pack=False, align=None), "flags": SimTypeInt(signed=False, label="IKEEXT_CERT_FLAGS")}, name="IKEEXT_CERT_ROOT_CONFIG0", pack=False, align=None), offset=0)}, name="_Anonymous_e__Struct", pack=False, align=None), "outboundEnterpriseStoreConfig": SimTypePointer(SimStruct({"certData": SimStruct({"size": SimTypeInt(signed=False, label="UInt32"), "data": SimTypePointer(SimTypeChar(label="Byte"), offset=0)}, name="FWP_BYTE_BLOB", pack=False, align=None), "flags": SimTypeInt(signed=False, label="IKEEXT_CERT_FLAGS")}, name="IKEEXT_CERT_ROOT_CONFIG0", pack=False, align=None), offset=0), "outboundTrustedRootStoreConfig": SimTypePointer(SimStruct({"certData": SimStruct({"size": SimTypeInt(signed=False, label="UInt32"), "data": SimTypePointer(SimTypeChar(label="Byte"), offset=0)}, name="FWP_BYTE_BLOB", pack=False, align=None), "flags": SimTypeInt(signed=False, label="IKEEXT_CERT_FLAGS")}, name="IKEEXT_CERT_ROOT_CONFIG0", pack=False, align=None), offset=0)}, name="<anon>", label="None"), "flags": SimTypeInt(signed=False, label="IKEEXT_CERT_AUTH"), "localCertLocationUrl": SimStruct({"size": SimTypeInt(signed=False, label="UInt32"), "data": SimTypePointer(SimTypeChar(label="Byte"), offset=0)}, name="FWP_BYTE_BLOB", pack=False, align=None)}, name="IKEEXT_CERTIFICATE_AUTHENTICATION1", pack=False, align=None), "cgaAuthentication": SimStruct({"keyContainerName": SimTypePointer(SimTypeChar(label="Char"), offset=0), "cspName": SimTypePointer(SimTypeChar(label="Char"), offset=0), "cspType": SimTypeInt(signed=False, label="UInt32"), "cgaModifier": SimStruct({"byteArray16": SimTypeFixedSizeArray(SimTypeChar(label="Byte"), 16)}, name="FWP_BYTE_ARRAY16", pack=False, align=None), "cgaCollisionCount": SimTypeChar(label="Byte")}, name="IKEEXT_IPV6_CGA_AUTHENTICATION0", pack=False, align=None), "eapAuthentication": SimStruct({"flags": SimTypeInt(signed=False, label="IKEEXT_EAP_AUTHENTICATION_FLAGS")}, name="IKEEXT_EAP_AUTHENTICATION0", pack=False, align=None)}, name="<anon>", label="None")}, name="IKEEXT_AUTHENTICATION_METHOD1", pack=False, align=None), offset=0), "initiatorImpersonationType": SimTypeInt(signed=False, label="IKEEXT_AUTHENTICATION_IMPERSONATION_TYPE"), "numIkeProposals": SimTypeInt(signed=False, label="UInt32"), "ikeProposals": SimTypePointer(SimStruct({"cipherAlgorithm": SimStruct({"algoIdentifier": SimTypeInt(signed=False, label="IKEEXT_CIPHER_TYPE"), "keyLen": SimTypeInt(signed=False, label="UInt32"), "rounds": SimTypeInt(signed=False, label="UInt32")}, name="IKEEXT_CIPHER_ALGORITHM0", pack=False, align=None), "integrityAlgorithm": SimStruct({"algoIdentifier": SimTypeInt(signed=False, label="IKEEXT_INTEGRITY_TYPE")}, name="IKEEXT_INTEGRITY_ALGORITHM0", pack=False, align=None), "maxLifetimeSeconds": SimTypeInt(signed=False, label="UInt32"), "dhGroup": SimTypeInt(signed=False, label="IKEEXT_DH_GROUP"), "quickModeLimit": SimTypeInt(signed=False, label="UInt32")}, name="IKEEXT_PROPOSAL0", pack=False, align=None), offset=0), "flags": SimTypeInt(signed=False, label="IKEEXT_POLICY_FLAG"), "maxDynamicFilters": SimTypeInt(signed=False, label="UInt32"), "retransmitDurationSecs": SimTypeInt(signed=False, label="UInt32")}, name="IKEEXT_POLICY1", pack=False, align=None), offset=0), "authIpMmPolicy": SimTypePointer(SimStruct({"softExpirationTime": SimTypeInt(signed=False, label="UInt32"), "numAuthenticationMethods": SimTypeInt(signed=False, label="UInt32"), "authenticationMethods": SimTypePointer(SimStruct({"authenticationMethodType": SimTypeInt(signed=False, label="IKEEXT_AUTHENTICATION_METHOD_TYPE"), "Anonymous": SimUnion({"presharedKeyAuthentication": SimStruct({"presharedKey": SimStruct({"size": SimTypeInt(signed=False, label="UInt32"), "data": SimTypePointer(SimTypeChar(label="Byte"), offset=0)}, name="FWP_BYTE_BLOB", pack=False, align=None), "flags": SimTypeInt(signed=False, label="IKEEXT_PRESHARED_KEY_AUTHENTICATION_FLAGS")}, name="IKEEXT_PRESHARED_KEY_AUTHENTICATION1", pack=False, align=None), "certificateAuthentication": SimStruct({"inboundConfigType": SimTypeInt(signed=False, label="IKEEXT_CERT_CONFIG_TYPE"), "Anonymous1": SimUnion({"Anonymous": SimStruct({"inboundRootArraySize": SimTypeInt(signed=False, label="UInt32"), "inboundRootArray": SimTypePointer(SimStruct({"certData": SimStruct({"size": SimTypeInt(signed=False, label="UInt32"), "data": SimTypePointer(SimTypeChar(label="Byte"), offset=0)}, name="FWP_BYTE_BLOB", pack=False, align=None), "flags": SimTypeInt(signed=False, label="IKEEXT_CERT_FLAGS")}, name="IKEEXT_CERT_ROOT_CONFIG0", pack=False, align=None), offset=0)}, name="_Anonymous_e__Struct", pack=False, align=None), "inboundEnterpriseStoreConfig": SimTypePointer(SimStruct({"certData": SimStruct({"size": SimTypeInt(signed=False, label="UInt32"), "data": SimTypePointer(SimTypeChar(label="Byte"), offset=0)}, name="FWP_BYTE_BLOB", pack=False, align=None), "flags": SimTypeInt(signed=False, label="IKEEXT_CERT_FLAGS")}, name="IKEEXT_CERT_ROOT_CONFIG0", pack=False, align=None), offset=0), "inboundTrustedRootStoreConfig": SimTypePointer(SimStruct({"certData": SimStruct({"size": SimTypeInt(signed=False, label="UInt32"), "data": SimTypePointer(SimTypeChar(label="Byte"), offset=0)}, name="FWP_BYTE_BLOB", pack=False, align=None), "flags": SimTypeInt(signed=False, label="IKEEXT_CERT_FLAGS")}, name="IKEEXT_CERT_ROOT_CONFIG0", pack=False, align=None), offset=0)}, name="<anon>", label="None"), "outboundConfigType": SimTypeInt(signed=False, label="IKEEXT_CERT_CONFIG_TYPE"), "Anonymous2": SimUnion({"Anonymous": SimStruct({"outboundRootArraySize": SimTypeInt(signed=False, label="UInt32"), "outboundRootArray": SimTypePointer(SimStruct({"certData": SimStruct({"size": SimTypeInt(signed=False, label="UInt32"), "data": SimTypePointer(SimTypeChar(label="Byte"), offset=0)}, name="FWP_BYTE_BLOB", pack=False, align=None), "flags": SimTypeInt(signed=False, label="IKEEXT_CERT_FLAGS")}, name="IKEEXT_CERT_ROOT_CONFIG0", pack=False, align=None), offset=0)}, name="_Anonymous_e__Struct", pack=False, align=None), "outboundEnterpriseStoreConfig": SimTypePointer(SimStruct({"certData": SimStruct({"size": SimTypeInt(signed=False, label="UInt32"), "data": SimTypePointer(SimTypeChar(label="Byte"), offset=0)}, name="FWP_BYTE_BLOB", pack=False, align=None), "flags": SimTypeInt(signed=False, label="IKEEXT_CERT_FLAGS")}, name="IKEEXT_CERT_ROOT_CONFIG0", pack=False, align=None), offset=0), "outboundTrustedRootStoreConfig": SimTypePointer(SimStruct({"certData": SimStruct({"size": SimTypeInt(signed=False, label="UInt32"), "data": SimTypePointer(SimTypeChar(label="Byte"), offset=0)}, name="FWP_BYTE_BLOB", pack=False, align=None), "flags": SimTypeInt(signed=False, label="IKEEXT_CERT_FLAGS")}, name="IKEEXT_CERT_ROOT_CONFIG0", pack=False, align=None), offset=0)}, name="<anon>", label="None"), "flags": SimTypeInt(signed=False, label="IKEEXT_CERT_AUTH"), "localCertLocationUrl": SimStruct({"size": SimTypeInt(signed=False, label="UInt32"), "data": SimTypePointer(SimTypeChar(label="Byte"), offset=0)}, name="FWP_BYTE_BLOB", pack=False, align=None)}, name="IKEEXT_CERTIFICATE_AUTHENTICATION1", pack=False, align=None), "kerberosAuthentication": SimStruct({"flags": SimTypeInt(signed=False, label="IKEEXT_KERBEROS_AUTHENTICATION_FLAGS")}, name="IKEEXT_KERBEROS_AUTHENTICATION0", pack=False, align=None), "ntlmV2Authentication": SimStruct({"flags": SimTypeInt(signed=False, label="UInt32")}, name="IKEEXT_NTLM_V2_AUTHENTICATION0", pack=False, align=None), "sslAuthentication": SimStruct({"inboundConfigType": SimTypeInt(signed=False, label="IKEEXT_CERT_CONFIG_TYPE"), "Anonymous1": SimUnion({"Anonymous": SimStruct({"inboundRootArraySize": SimTypeInt(signed=False, label="UInt32"), "inboundRootArray": SimTypePointer(SimStruct({"certData": SimStruct({"size": SimTypeInt(signed=False, label="UInt32"), "data": SimTypePointer(SimTypeChar(label="Byte"), offset=0)}, name="FWP_BYTE_BLOB", pack=False, align=None), "flags": SimTypeInt(signed=False, label="IKEEXT_CERT_FLAGS")}, name="IKEEXT_CERT_ROOT_CONFIG0", pack=False, align=None), offset=0)}, name="_Anonymous_e__Struct", pack=False, align=None), "inboundEnterpriseStoreConfig": SimTypePointer(SimStruct({"certData": SimStruct({"size": SimTypeInt(signed=False, label="UInt32"), "data": SimTypePointer(SimTypeChar(label="Byte"), offset=0)}, name="FWP_BYTE_BLOB", pack=False, align=None), "flags": SimTypeInt(signed=False, label="IKEEXT_CERT_FLAGS")}, name="IKEEXT_CERT_ROOT_CONFIG0", pack=False, align=None), offset=0), "inboundTrustedRootStoreConfig": SimTypePointer(SimStruct({"certData": SimStruct({"size": SimTypeInt(signed=False, label="UInt32"), "data": SimTypePointer(SimTypeChar(label="Byte"), offset=0)}, name="FWP_BYTE_BLOB", pack=False, align=None), "flags": SimTypeInt(signed=False, label="IKEEXT_CERT_FLAGS")}, name="IKEEXT_CERT_ROOT_CONFIG0", pack=False, align=None), offset=0)}, name="<anon>", label="None"), "outboundConfigType": SimTypeInt(signed=False, label="IKEEXT_CERT_CONFIG_TYPE"), "Anonymous2": SimUnion({"Anonymous": SimStruct({"outboundRootArraySize": SimTypeInt(signed=False, label="UInt32"), "outboundRootArray": SimTypePointer(SimStruct({"certData": SimStruct({"size": SimTypeInt(signed=False, label="UInt32"), "data": SimTypePointer(SimTypeChar(label="Byte"), offset=0)}, name="FWP_BYTE_BLOB", pack=False, align=None), "flags": SimTypeInt(signed=False, label="IKEEXT_CERT_FLAGS")}, name="IKEEXT_CERT_ROOT_CONFIG0", pack=False, align=None), offset=0)}, name="_Anonymous_e__Struct", pack=False, align=None), "outboundEnterpriseStoreConfig": SimTypePointer(SimStruct({"certData": SimStruct({"size": SimTypeInt(signed=False, label="UInt32"), "data": SimTypePointer(SimTypeChar(label="Byte"), offset=0)}, name="FWP_BYTE_BLOB", pack=False, align=None), "flags": SimTypeInt(signed=False, label="IKEEXT_CERT_FLAGS")}, name="IKEEXT_CERT_ROOT_CONFIG0", pack=False, align=None), offset=0), "outboundTrustedRootStoreConfig": SimTypePointer(SimStruct({"certData": SimStruct({"size": SimTypeInt(signed=False, label="UInt32"), "data": SimTypePointer(SimTypeChar(label="Byte"), offset=0)}, name="FWP_BYTE_BLOB", pack=False, align=None), "flags": SimTypeInt(signed=False, label="IKEEXT_CERT_FLAGS")}, name="IKEEXT_CERT_ROOT_CONFIG0", pack=False, align=None), offset=0)}, name="<anon>", label="None"), "flags": SimTypeInt(signed=False, label="IKEEXT_CERT_AUTH"), "localCertLocationUrl": SimStruct({"size": SimTypeInt(signed=False, label="UInt32"), "data": SimTypePointer(SimTypeChar(label="Byte"), offset=0)}, name="FWP_BYTE_BLOB", pack=False, align=None)}, name="IKEEXT_CERTIFICATE_AUTHENTICATION1", pack=False, align=None), "cgaAuthentication": SimStruct({"keyContainerName": SimTypePointer(SimTypeChar(label="Char"), offset=0), "cspName": SimTypePointer(SimTypeChar(label="Char"), offset=0), "cspType": SimTypeInt(signed=False, label="UInt32"), "cgaModifier": SimStruct({"byteArray16": SimTypeFixedSizeArray(SimTypeChar(label="Byte"), 16)}, name="FWP_BYTE_ARRAY16", pack=False, align=None), "cgaCollisionCount": SimTypeChar(label="Byte")}, name="IKEEXT_IPV6_CGA_AUTHENTICATION0", pack=False, align=None), "eapAuthentication": SimStruct({"flags": SimTypeInt(signed=False, label="IKEEXT_EAP_AUTHENTICATION_FLAGS")}, name="IKEEXT_EAP_AUTHENTICATION0", pack=False, align=None)}, name="<anon>", label="None")}, name="IKEEXT_AUTHENTICATION_METHOD1", pack=False, align=None), offset=0), "initiatorImpersonationType": SimTypeInt(signed=False, label="IKEEXT_AUTHENTICATION_IMPERSONATION_TYPE"), "numIkeProposals": SimTypeInt(signed=False, label="UInt32"), "ikeProposals": SimTypePointer(SimStruct({"cipherAlgorithm": SimStruct({"algoIdentifier": SimTypeInt(signed=False, label="IKEEXT_CIPHER_TYPE"), "keyLen": SimTypeInt(signed=False, label="UInt32"), "rounds": SimTypeInt(signed=False, label="UInt32")}, name="IKEEXT_CIPHER_ALGORITHM0", pack=False, align=None), "integrityAlgorithm": SimStruct({"algoIdentifier": SimTypeInt(signed=False, label="IKEEXT_INTEGRITY_TYPE")}, name="IKEEXT_INTEGRITY_ALGORITHM0", pack=False, align=None), "maxLifetimeSeconds": SimTypeInt(signed=False, label="UInt32"), "dhGroup": SimTypeInt(signed=False, label="IKEEXT_DH_GROUP"), "quickModeLimit": SimTypeInt(signed=False, label="UInt32")}, name="IKEEXT_PROPOSAL0", pack=False, align=None), offset=0), "flags": SimTypeInt(signed=False, label="IKEEXT_POLICY_FLAG"), "maxDynamicFilters": SimTypeInt(signed=False, label="UInt32"), "retransmitDurationSecs": SimTypeInt(signed=False, label="UInt32")}, name="IKEEXT_POLICY1", pack=False, align=None), offset=0), "dataBuffer": SimTypePointer(SimStruct({"size": SimTypeInt(signed=False, label="UInt32"), "data": SimTypePointer(SimTypeChar(label="Byte"), offset=0)}, name="FWP_BYTE_BLOB", pack=False, align=None), offset=0), "classifyOptions": SimTypePointer(SimStruct({"numOptions": SimTypeInt(signed=False, label="UInt32"), "options": SimTypePointer(SimStruct({"type": SimTypeInt(signed=False, label="FWP_CLASSIFY_OPTION_TYPE"), "value": SimStruct({"type": SimTypeInt(signed=False, label="FWP_DATA_TYPE"), "Anonymous": SimUnion({"uint8": SimTypeChar(label="Byte"), "uint16": SimTypeShort(signed=False, label="UInt16"), "uint32": SimTypeInt(signed=False, label="UInt32"), "uint64": SimTypePointer(SimTypeLongLong(signed=False, label="UInt64"), offset=0), "int8": SimTypeChar(label="SByte"), "int16": SimTypeShort(signed=True, label="Int16"), "int32": SimTypeInt(signed=True, label="Int32"), "int64": SimTypePointer(SimTypeLongLong(signed=True, label="Int64"), offset=0), "float32": SimTypeFloat(size=32), "double64": SimTypePointer(SimTypeFloat(size=64), offset=0), "byteArray16": SimTypePointer(SimStruct({"byteArray16": SimTypeFixedSizeArray(SimTypeChar(label="Byte"), 16)}, name="FWP_BYTE_ARRAY16", pack=False, align=None), offset=0), "byteBlob": SimTypePointer(SimStruct({"size": SimTypeInt(signed=False, label="UInt32"), "data": SimTypePointer(SimTypeChar(label="Byte"), offset=0)}, name="FWP_BYTE_BLOB", pack=False, align=None), offset=0), "sid": SimTypePointer(SimTypeBottom(label="SID"), offset=0), "sd": SimTypePointer(SimStruct({"size": SimTypeInt(signed=False, label="UInt32"), "data": SimTypePointer(SimTypeChar(label="Byte"), offset=0)}, name="FWP_BYTE_BLOB", pack=False, align=None), offset=0), "tokenInformation": SimTypePointer(SimStruct({"sidCount": SimTypeInt(signed=False, label="UInt32"), "sids": SimTypePointer(SimTypeBottom(label="SID_AND_ATTRIBUTES"), offset=0), "restrictedSidCount": SimTypeInt(signed=False, label="UInt32"), "restrictedSids": SimTypePointer(SimTypeBottom(label="SID_AND_ATTRIBUTES"), offset=0)}, name="FWP_TOKEN_INFORMATION", pack=False, align=None), offset=0), "tokenAccessInformation": SimTypePointer(SimStruct({"size": SimTypeInt(signed=False, label="UInt32"), "data": SimTypePointer(SimTypeChar(label="Byte"), offset=0)}, name="FWP_BYTE_BLOB", pack=False, align=None), offset=0), "unicodeString": SimTypePointer(SimTypeChar(label="Char"), offset=0), "byteArray6": SimTypePointer(SimStruct({"byteArray6": SimTypeFixedSizeArray(SimTypeChar(label="Byte"), 6)}, name="FWP_BYTE_ARRAY6", pack=False, align=None), offset=0), "bitmapArray64": SimTypePointer(SimStruct({"bitmapArray64": SimTypeFixedSizeArray(SimTypeChar(label="Byte"), 8)}, name="FWP_BITMAP_ARRAY64_", pack=False, align=None), offset=0)}, name="<anon>", label="None")}, name="FWP_VALUE0", pack=False, align=None)}, name="FWPM_CLASSIFY_OPTION0", pack=False, align=None), offset=0)}, name="FWPM_CLASSIFY_OPTIONS0", pack=False, align=None), offset=0), "ikeV2QmTunnelPolicy": SimTypePointer(SimStruct({"flags": SimTypeInt(signed=False, label="IPSEC_POLICY_FLAG"), "numIpsecProposals": SimTypeInt(signed=False, label="UInt32"), "ipsecProposals": SimTypePointer(SimStruct({"lifetime": SimStruct({"lifetimeSeconds": SimTypeInt(signed=False, label="UInt32"), "lifetimeKilobytes": SimTypeInt(signed=False, label="UInt32"), "lifetimePackets": SimTypeInt(signed=False, label="UInt32")}, name="IPSEC_SA_LIFETIME0", pack=False, align=None), "numSaTransforms": SimTypeInt(signed=False, label="UInt32"), "saTransforms": SimTypePointer(SimStruct({"ipsecTransformType": SimTypeInt(signed=False, label="IPSEC_TRANSFORM_TYPE"), "Anonymous": SimUnion({"ahTransform": SimTypePointer(SimStruct({"authTransformId": SimStruct({"authType": SimTypeInt(signed=False, label="IPSEC_AUTH_TYPE"), "authConfig": SimTypeChar(label="Byte")}, name="IPSEC_AUTH_TRANSFORM_ID0", pack=False, align=None), "cryptoModuleId": SimTypePointer(SimTypeBottom(label="Guid"), offset=0)}, name="IPSEC_AUTH_TRANSFORM0", pack=False, align=None), offset=0), "espAuthTransform": SimTypePointer(SimStruct({"authTransformId": SimStruct({"authType": SimTypeInt(signed=False, label="IPSEC_AUTH_TYPE"), "authConfig": SimTypeChar(label="Byte")}, name="IPSEC_AUTH_TRANSFORM_ID0", pack=False, align=None), "cryptoModuleId": SimTypePointer(SimTypeBottom(label="Guid"), offset=0)}, name="IPSEC_AUTH_TRANSFORM0", pack=False, align=None), offset=0), "espCipherTransform": SimTypePointer(SimStruct({"cipherTransformId": SimStruct({"cipherType": SimTypeInt(signed=False, label="IPSEC_CIPHER_TYPE"), "cipherConfig": SimTypeChar(label="Byte")}, name="IPSEC_CIPHER_TRANSFORM_ID0", pack=False, align=None), "cryptoModuleId": SimTypePointer(SimTypeBottom(label="Guid"), offset=0)}, name="IPSEC_CIPHER_TRANSFORM0", pack=False, align=None), offset=0), "espAuthAndCipherTransform": SimTypePointer(SimStruct({"authTransform": SimStruct({"authTransformId": SimStruct({"authType": SimTypeInt(signed=False, label="IPSEC_AUTH_TYPE"), "authConfig": SimTypeChar(label="Byte")}, name="IPSEC_AUTH_TRANSFORM_ID0", pack=False, align=None), "cryptoModuleId": SimTypePointer(SimTypeBottom(label="Guid"), offset=0)}, name="IPSEC_AUTH_TRANSFORM0", pack=False, align=None), "cipherTransform": SimStruct({"cipherTransformId": SimStruct({"cipherType": SimTypeInt(signed=False, label="IPSEC_CIPHER_TYPE"), "cipherConfig": SimTypeChar(label="Byte")}, name="IPSEC_CIPHER_TRANSFORM_ID0", pack=False, align=None), "cryptoModuleId": SimTypePointer(SimTypeBottom(label="Guid"), offset=0)}, name="IPSEC_CIPHER_TRANSFORM0", pack=False, align=None)}, name="IPSEC_AUTH_AND_CIPHER_TRANSFORM0", pack=False, align=None), offset=0), "espAuthFwTransform": SimTypePointer(SimStruct({"authTransformId": SimStruct({"authType": SimTypeInt(signed=False, label="IPSEC_AUTH_TYPE"), "authConfig": SimTypeChar(label="Byte")}, name="IPSEC_AUTH_TRANSFORM_ID0", pack=False, align=None), "cryptoModuleId": SimTypePointer(SimTypeBottom(label="Guid"), offset=0)}, name="IPSEC_AUTH_TRANSFORM0", pack=False, align=None), offset=0)}, name="<anon>", label="None")}, name="IPSEC_SA_TRANSFORM0", pack=False, align=None), offset=0), "pfsGroup": SimTypeInt(signed=False, label="IPSEC_PFS_GROUP")}, name="IPSEC_PROPOSAL0", pack=False, align=None), offset=0), "tunnelEndpoints": SimStruct({"ipVersion": SimTypeInt(signed=False, label="FWP_IP_VERSION"), "Anonymous1": SimUnion({"localV4Address": SimTypeInt(signed=False, label="UInt32"), "localV6Address": SimTypeFixedSizeArray(SimTypeChar(label="Byte"), 16)}, name="<anon>", label="None"), "Anonymous2": SimUnion({"remoteV4Address": SimTypeInt(signed=False, label="UInt32"), "remoteV6Address": SimTypeFixedSizeArray(SimTypeChar(label="Byte"), 16)}, name="<anon>", label="None"), "localIfLuid": SimTypeLongLong(signed=False, label="UInt64")}, name="IPSEC_TUNNEL_ENDPOINTS1", pack=False, align=None), "saIdleTimeout": SimStruct({"idleTimeoutSeconds": SimTypeInt(signed=False, label="UInt32"), "idleTimeoutSecondsFailOver": SimTypeInt(signed=False, label="UInt32")}, name="IPSEC_SA_IDLE_TIMEOUT0", pack=False, align=None), "emPolicy": SimTypePointer(SimStruct({"numAuthenticationMethods": SimTypeInt(signed=False, label="UInt32"), "authenticationMethods": SimTypePointer(SimStruct({"authenticationMethodType": SimTypeInt(signed=False, label="IKEEXT_AUTHENTICATION_METHOD_TYPE"), "Anonymous": SimUnion({"presharedKeyAuthentication": SimStruct({"presharedKey": SimStruct({"size": SimTypeInt(signed=False, label="UInt32"), "data": SimTypePointer(SimTypeChar(label="Byte"), offset=0)}, name="FWP_BYTE_BLOB", pack=False, align=None), "flags": SimTypeInt(signed=False, label="IKEEXT_PRESHARED_KEY_AUTHENTICATION_FLAGS")}, name="IKEEXT_PRESHARED_KEY_AUTHENTICATION1", pack=False, align=None), "certificateAuthentication": SimStruct({"inboundConfigType": SimTypeInt(signed=False, label="IKEEXT_CERT_CONFIG_TYPE"), "Anonymous1": SimUnion({"Anonymous": SimStruct({"inboundRootArraySize": SimTypeInt(signed=False, label="UInt32"), "inboundRootArray": SimTypePointer(SimStruct({"certData": SimStruct({"size": SimTypeInt(signed=False, label="UInt32"), "data": SimTypePointer(SimTypeChar(label="Byte"), offset=0)}, name="FWP_BYTE_BLOB", pack=False, align=None), "flags": SimTypeInt(signed=False, label="IKEEXT_CERT_FLAGS")}, name="IKEEXT_CERT_ROOT_CONFIG0", pack=False, align=None), offset=0)}, name="_Anonymous_e__Struct", pack=False, align=None), "inboundEnterpriseStoreConfig": SimTypePointer(SimStruct({"certData": SimStruct({"size": SimTypeInt(signed=False, label="UInt32"), "data": SimTypePointer(SimTypeChar(label="Byte"), offset=0)}, name="FWP_BYTE_BLOB", pack=False, align=None), "flags": SimTypeInt(signed=False, label="IKEEXT_CERT_FLAGS")}, name="IKEEXT_CERT_ROOT_CONFIG0", pack=False, align=None), offset=0), "inboundTrustedRootStoreConfig": SimTypePointer(SimStruct({"certData": SimStruct({"size": SimTypeInt(signed=False, label="UInt32"), "data": SimTypePointer(SimTypeChar(label="Byte"), offset=0)}, name="FWP_BYTE_BLOB", pack=False, align=None), "flags": SimTypeInt(signed=False, label="IKEEXT_CERT_FLAGS")}, name="IKEEXT_CERT_ROOT_CONFIG0", pack=False, align=None), offset=0)}, name="<anon>", label="None"), "outboundConfigType": SimTypeInt(signed=False, label="IKEEXT_CERT_CONFIG_TYPE"), "Anonymous2": SimUnion({"Anonymous": SimStruct({"outboundRootArraySize": SimTypeInt(signed=False, label="UInt32"), "outboundRootArray": SimTypePointer(SimStruct({"certData": SimStruct({"size": SimTypeInt(signed=False, label="UInt32"), "data": SimTypePointer(SimTypeChar(label="Byte"), offset=0)}, name="FWP_BYTE_BLOB", pack=False, align=None), "flags": SimTypeInt(signed=False, label="IKEEXT_CERT_FLAGS")}, name="IKEEXT_CERT_ROOT_CONFIG0", pack=False, align=None), offset=0)}, name="_Anonymous_e__Struct", pack=False, align=None), "outboundEnterpriseStoreConfig": SimTypePointer(SimStruct({"certData": SimStruct({"size": SimTypeInt(signed=False, label="UInt32"), "data": SimTypePointer(SimTypeChar(label="Byte"), offset=0)}, name="FWP_BYTE_BLOB", pack=False, align=None), "flags": SimTypeInt(signed=False, label="IKEEXT_CERT_FLAGS")}, name="IKEEXT_CERT_ROOT_CONFIG0", pack=False, align=None), offset=0), "outboundTrustedRootStoreConfig": SimTypePointer(SimStruct({"certData": SimStruct({"size": SimTypeInt(signed=False, label="UInt32"), "data": SimTypePointer(SimTypeChar(label="Byte"), offset=0)}, name="FWP_BYTE_BLOB", pack=False, align=None), "flags": SimTypeInt(signed=False, label="IKEEXT_CERT_FLAGS")}, name="IKEEXT_CERT_ROOT_CONFIG0", pack=False, align=None), offset=0)}, name="<anon>", label="None"), "flags": SimTypeInt(signed=False, label="IKEEXT_CERT_AUTH"), "localCertLocationUrl": SimStruct({"size": SimTypeInt(signed=False, label="UInt32"), "data": SimTypePointer(SimTypeChar(label="Byte"), offset=0)}, name="FWP_BYTE_BLOB", pack=False, align=None)}, name="IKEEXT_CERTIFICATE_AUTHENTICATION1", pack=False, align=None), "kerberosAuthentication": SimStruct({"flags": SimTypeInt(signed=False, label="IKEEXT_KERBEROS_AUTHENTICATION_FLAGS")}, name="IKEEXT_KERBEROS_AUTHENTICATION0", pack=False, align=None), "ntlmV2Authentication": SimStruct({"flags": SimTypeInt(signed=False, label="UInt32")}, name="IKEEXT_NTLM_V2_AUTHENTICATION0", pack=False, align=None), "sslAuthentication": SimStruct({"inboundConfigType": SimTypeInt(signed=False, label="IKEEXT_CERT_CONFIG_TYPE"), "Anonymous1": SimUnion({"Anonymous": SimStruct({"inboundRootArraySize": SimTypeInt(signed=False, label="UInt32"), "inboundRootArray": SimTypePointer(SimStruct({"certData": SimStruct({"size": SimTypeInt(signed=False, label="UInt32"), "data": SimTypePointer(SimTypeChar(label="Byte"), offset=0)}, name="FWP_BYTE_BLOB", pack=False, align=None), "flags": SimTypeInt(signed=False, label="IKEEXT_CERT_FLAGS")}, name="IKEEXT_CERT_ROOT_CONFIG0", pack=False, align=None), offset=0)}, name="_Anonymous_e__Struct", pack=False, align=None), "inboundEnterpriseStoreConfig": SimTypePointer(SimStruct({"certData": SimStruct({"size": SimTypeInt(signed=False, label="UInt32"), "data": SimTypePointer(SimTypeChar(label="Byte"), offset=0)}, name="FWP_BYTE_BLOB", pack=False, align=None), "flags": SimTypeInt(signed=False, label="IKEEXT_CERT_FLAGS")}, name="IKEEXT_CERT_ROOT_CONFIG0", pack=False, align=None), offset=0), "inboundTrustedRootStoreConfig": SimTypePointer(SimStruct({"certData": SimStruct({"size": SimTypeInt(signed=False, label="UInt32"), "data": SimTypePointer(SimTypeChar(label="Byte"), offset=0)}, name="FWP_BYTE_BLOB", pack=False, align=None), "flags": SimTypeInt(signed=False, label="IKEEXT_CERT_FLAGS")}, name="IKEEXT_CERT_ROOT_CONFIG0", pack=False, align=None), offset=0)}, name="<anon>", label="None"), "outboundConfigType": SimTypeInt(signed=False, label="IKEEXT_CERT_CONFIG_TYPE"), "Anonymous2": SimUnion({"Anonymous": SimStruct({"outboundRootArraySize": SimTypeInt(signed=False, label="UInt32"), "outboundRootArray": SimTypePointer(SimStruct({"certData": SimStruct({"size": SimTypeInt(signed=False, label="UInt32"), "data": SimTypePointer(SimTypeChar(label="Byte"), offset=0)}, name="FWP_BYTE_BLOB", pack=False, align=None), "flags": SimTypeInt(signed=False, label="IKEEXT_CERT_FLAGS")}, name="IKEEXT_CERT_ROOT_CONFIG0", pack=False, align=None), offset=0)}, name="_Anonymous_e__Struct", pack=False, align=None), "outboundEnterpriseStoreConfig": SimTypePointer(SimStruct({"certData": SimStruct({"size": SimTypeInt(signed=False, label="UInt32"), "data": SimTypePointer(SimTypeChar(label="Byte"), offset=0)}, name="FWP_BYTE_BLOB", pack=False, align=None), "flags": SimTypeInt(signed=False, label="IKEEXT_CERT_FLAGS")}, name="IKEEXT_CERT_ROOT_CONFIG0", pack=False, align=None), offset=0), "outboundTrustedRootStoreConfig": SimTypePointer(SimStruct({"certData": SimStruct({"size": SimTypeInt(signed=False, label="UInt32"), "data": SimTypePointer(SimTypeChar(label="Byte"), offset=0)}, name="FWP_BYTE_BLOB", pack=False, align=None), "flags": SimTypeInt(signed=False, label="IKEEXT_CERT_FLAGS")}, name="IKEEXT_CERT_ROOT_CONFIG0", pack=False, align=None), offset=0)}, name="<anon>", label="None"), "flags": SimTypeInt(signed=False, label="IKEEXT_CERT_AUTH"), "localCertLocationUrl": SimStruct({"size": SimTypeInt(signed=False, label="UInt32"), "data": SimTypePointer(SimTypeChar(label="Byte"), offset=0)}, name="FWP_BYTE_BLOB", pack=False, align=None)}, name="IKEEXT_CERTIFICATE_AUTHENTICATION1", pack=False, align=None), "cgaAuthentication": SimStruct({"keyContainerName": SimTypePointer(SimTypeChar(label="Char"), offset=0), "cspName": SimTypePointer(SimTypeChar(label="Char"), offset=0), "cspType": SimTypeInt(signed=False, label="UInt32"), "cgaModifier": SimStruct({"byteArray16": SimTypeFixedSizeArray(SimTypeChar(label="Byte"), 16)}, name="FWP_BYTE_ARRAY16", pack=False, align=None), "cgaCollisionCount": SimTypeChar(label="Byte")}, name="IKEEXT_IPV6_CGA_AUTHENTICATION0", pack=False, align=None), "eapAuthentication": SimStruct({"flags": SimTypeInt(signed=False, label="IKEEXT_EAP_AUTHENTICATION_FLAGS")}, name="IKEEXT_EAP_AUTHENTICATION0", pack=False, align=None)}, name="<anon>", label="None")}, name="IKEEXT_AUTHENTICATION_METHOD1", pack=False, align=None), offset=0), "initiatorImpersonationType": SimTypeInt(signed=False, label="IKEEXT_AUTHENTICATION_IMPERSONATION_TYPE")}, name="IKEEXT_EM_POLICY1", pack=False, align=None), offset=0)}, name="IPSEC_TUNNEL_POLICY1", pack=False, align=None), offset=0), "ikeV2MmPolicy": SimTypePointer(SimStruct({"softExpirationTime": SimTypeInt(signed=False, label="UInt32"), "numAuthenticationMethods": SimTypeInt(signed=False, label="UInt32"), "authenticationMethods": SimTypePointer(SimStruct({"authenticationMethodType": SimTypeInt(signed=False, label="IKEEXT_AUTHENTICATION_METHOD_TYPE"), "Anonymous": SimUnion({"presharedKeyAuthentication": SimStruct({"presharedKey": SimStruct({"size": SimTypeInt(signed=False, label="UInt32"), "data": SimTypePointer(SimTypeChar(label="Byte"), offset=0)}, name="FWP_BYTE_BLOB", pack=False, align=None), "flags": SimTypeInt(signed=False, label="IKEEXT_PRESHARED_KEY_AUTHENTICATION_FLAGS")}, name="IKEEXT_PRESHARED_KEY_AUTHENTICATION1", pack=False, align=None), "certificateAuthentication": SimStruct({"inboundConfigType": SimTypeInt(signed=False, label="IKEEXT_CERT_CONFIG_TYPE"), "Anonymous1": SimUnion({"Anonymous": SimStruct({"inboundRootArraySize": SimTypeInt(signed=False, label="UInt32"), "inboundRootArray": SimTypePointer(SimStruct({"certData": SimStruct({"size": SimTypeInt(signed=False, label="UInt32"), "data": SimTypePointer(SimTypeChar(label="Byte"), offset=0)}, name="FWP_BYTE_BLOB", pack=False, align=None), "flags": SimTypeInt(signed=False, label="IKEEXT_CERT_FLAGS")}, name="IKEEXT_CERT_ROOT_CONFIG0", pack=False, align=None), offset=0)}, name="_Anonymous_e__Struct", pack=False, align=None), "inboundEnterpriseStoreConfig": SimTypePointer(SimStruct({"certData": SimStruct({"size": SimTypeInt(signed=False, label="UInt32"), "data": SimTypePointer(SimTypeChar(label="Byte"), offset=0)}, name="FWP_BYTE_BLOB", pack=False, align=None), "flags": SimTypeInt(signed=False, label="IKEEXT_CERT_FLAGS")}, name="IKEEXT_CERT_ROOT_CONFIG0", pack=False, align=None), offset=0), "inboundTrustedRootStoreConfig": SimTypePointer(SimStruct({"certData": SimStruct({"size": SimTypeInt(signed=False, label="UInt32"), "data": SimTypePointer(SimTypeChar(label="Byte"), offset=0)}, name="FWP_BYTE_BLOB", pack=False, align=None), "flags": SimTypeInt(signed=False, label="IKEEXT_CERT_FLAGS")}, name="IKEEXT_CERT_ROOT_CONFIG0", pack=False, align=None), offset=0)}, name="<anon>", label="None"), "outboundConfigType": SimTypeInt(signed=False, label="IKEEXT_CERT_CONFIG_TYPE"), "Anonymous2": SimUnion({"Anonymous": SimStruct({"outboundRootArraySize": SimTypeInt(signed=False, label="UInt32"), "outboundRootArray": SimTypePointer(SimStruct({"certData": SimStruct({"size": SimTypeInt(signed=False, label="UInt32"), "data": SimTypePointer(SimTypeChar(label="Byte"), offset=0)}, name="FWP_BYTE_BLOB", pack=False, align=None), "flags": SimTypeInt(signed=False, label="IKEEXT_CERT_FLAGS")}, name="IKEEXT_CERT_ROOT_CONFIG0", pack=False, align=None), offset=0)}, name="_Anonymous_e__Struct", pack=False, align=None), "outboundEnterpriseStoreConfig": SimTypePointer(SimStruct({"certData": SimStruct({"size": SimTypeInt(signed=False, label="UInt32"), "data": SimTypePointer(SimTypeChar(label="Byte"), offset=0)}, name="FWP_BYTE_BLOB", pack=False, align=None), "flags": SimTypeInt(signed=False, label="IKEEXT_CERT_FLAGS")}, name="IKEEXT_CERT_ROOT_CONFIG0", pack=False, align=None), offset=0), "outboundTrustedRootStoreConfig": SimTypePointer(SimStruct({"certData": SimStruct({"size": SimTypeInt(signed=False, label="UInt32"), "data": SimTypePointer(SimTypeChar(label="Byte"), offset=0)}, name="FWP_BYTE_BLOB", pack=False, align=None), "flags": SimTypeInt(signed=False, label="IKEEXT_CERT_FLAGS")}, name="IKEEXT_CERT_ROOT_CONFIG0", pack=False, align=None), offset=0)}, name="<anon>", label="None"), "flags": SimTypeInt(signed=False, label="IKEEXT_CERT_AUTH"), "localCertLocationUrl": SimStruct({"size": SimTypeInt(signed=False, label="UInt32"), "data": SimTypePointer(SimTypeChar(label="Byte"), offset=0)}, name="FWP_BYTE_BLOB", pack=False, align=None)}, name="IKEEXT_CERTIFICATE_AUTHENTICATION1", pack=False, align=None), "kerberosAuthentication": SimStruct({"flags": SimTypeInt(signed=False, label="IKEEXT_KERBEROS_AUTHENTICATION_FLAGS")}, name="IKEEXT_KERBEROS_AUTHENTICATION0", pack=False, align=None), "ntlmV2Authentication": SimStruct({"flags": SimTypeInt(signed=False, label="UInt32")}, name="IKEEXT_NTLM_V2_AUTHENTICATION0", pack=False, align=None), "sslAuthentication": SimStruct({"inboundConfigType": SimTypeInt(signed=False, label="IKEEXT_CERT_CONFIG_TYPE"), "Anonymous1": SimUnion({"Anonymous": SimStruct({"inboundRootArraySize": SimTypeInt(signed=False, label="UInt32"), "inboundRootArray": SimTypePointer(SimStruct({"certData": SimStruct({"size": SimTypeInt(signed=False, label="UInt32"), "data": SimTypePointer(SimTypeChar(label="Byte"), offset=0)}, name="FWP_BYTE_BLOB", pack=False, align=None), "flags": SimTypeInt(signed=False, label="IKEEXT_CERT_FLAGS")}, name="IKEEXT_CERT_ROOT_CONFIG0", pack=False, align=None), offset=0)}, name="_Anonymous_e__Struct", pack=False, align=None), "inboundEnterpriseStoreConfig": SimTypePointer(SimStruct({"certData": SimStruct({"size": SimTypeInt(signed=False, label="UInt32"), "data": SimTypePointer(SimTypeChar(label="Byte"), offset=0)}, name="FWP_BYTE_BLOB", pack=False, align=None), "flags": SimTypeInt(signed=False, label="IKEEXT_CERT_FLAGS")}, name="IKEEXT_CERT_ROOT_CONFIG0", pack=False, align=None), offset=0), "inboundTrustedRootStoreConfig": SimTypePointer(SimStruct({"certData": SimStruct({"size": SimTypeInt(signed=False, label="UInt32"), "data": SimTypePointer(SimTypeChar(label="Byte"), offset=0)}, name="FWP_BYTE_BLOB", pack=False, align=None), "flags": SimTypeInt(signed=False, label="IKEEXT_CERT_FLAGS")}, name="IKEEXT_CERT_ROOT_CONFIG0", pack=False, align=None), offset=0)}, name="<anon>", label="None"), "outboundConfigType": SimTypeInt(signed=False, label="IKEEXT_CERT_CONFIG_TYPE"), "Anonymous2": SimUnion({"Anonymous": SimStruct({"outboundRootArraySize": SimTypeInt(signed=False, label="UInt32"), "outboundRootArray": SimTypePointer(SimStruct({"certData": SimStruct({"size": SimTypeInt(signed=False, label="UInt32"), "data": SimTypePointer(SimTypeChar(label="Byte"), offset=0)}, name="FWP_BYTE_BLOB", pack=False, align=None), "flags": SimTypeInt(signed=False, label="IKEEXT_CERT_FLAGS")}, name="IKEEXT_CERT_ROOT_CONFIG0", pack=False, align=None), offset=0)}, name="_Anonymous_e__Struct", pack=False, align=None), "outboundEnterpriseStoreConfig": SimTypePointer(SimStruct({"certData": SimStruct({"size": SimTypeInt(signed=False, label="UInt32"), "data": SimTypePointer(SimTypeChar(label="Byte"), offset=0)}, name="FWP_BYTE_BLOB", pack=False, align=None), "flags": SimTypeInt(signed=False, label="IKEEXT_CERT_FLAGS")}, name="IKEEXT_CERT_ROOT_CONFIG0", pack=False, align=None), offset=0), "outboundTrustedRootStoreConfig": SimTypePointer(SimStruct({"certData": SimStruct({"size": SimTypeInt(signed=False, label="UInt32"), "data": SimTypePointer(SimTypeChar(label="Byte"), offset=0)}, name="FWP_BYTE_BLOB", pack=False, align=None), "flags": SimTypeInt(signed=False, label="IKEEXT_CERT_FLAGS")}, name="IKEEXT_CERT_ROOT_CONFIG0", pack=False, align=None), offset=0)}, name="<anon>", label="None"), "flags": SimTypeInt(signed=False, label="IKEEXT_CERT_AUTH"), "localCertLocationUrl": SimStruct({"size": SimTypeInt(signed=False, label="UInt32"), "data": SimTypePointer(SimTypeChar(label="Byte"), offset=0)}, name="FWP_BYTE_BLOB", pack=False, align=None)}, name="IKEEXT_CERTIFICATE_AUTHENTICATION1", pack=False, align=None), "cgaAuthentication": SimStruct({"keyContainerName": SimTypePointer(SimTypeChar(label="Char"), offset=0), "cspName": SimTypePointer(SimTypeChar(label="Char"), offset=0), "cspType": SimTypeInt(signed=False, label="UInt32"), "cgaModifier": SimStruct({"byteArray16": SimTypeFixedSizeArray(SimTypeChar(label="Byte"), 16)}, name="FWP_BYTE_ARRAY16", pack=False, align=None), "cgaCollisionCount": SimTypeChar(label="Byte")}, name="IKEEXT_IPV6_CGA_AUTHENTICATION0", pack=False, align=None), "eapAuthentication": SimStruct({"flags": SimTypeInt(signed=False, label="IKEEXT_EAP_AUTHENTICATION_FLAGS")}, name="IKEEXT_EAP_AUTHENTICATION0", pack=False, align=None)}, name="<anon>", label="None")}, name="IKEEXT_AUTHENTICATION_METHOD1", pack=False, align=None), offset=0), "initiatorImpersonationType": SimTypeInt(signed=False, label="IKEEXT_AUTHENTICATION_IMPERSONATION_TYPE"), "numIkeProposals": SimTypeInt(signed=False, label="UInt32"), "ikeProposals": SimTypePointer(SimStruct({"cipherAlgorithm": SimStruct({"algoIdentifier": SimTypeInt(signed=False, label="IKEEXT_CIPHER_TYPE"), "keyLen": SimTypeInt(signed=False, label="UInt32"), "rounds": SimTypeInt(signed=False, label="UInt32")}, name="IKEEXT_CIPHER_ALGORITHM0", pack=False, align=None), "integrityAlgorithm": SimStruct({"algoIdentifier": SimTypeInt(signed=False, label="IKEEXT_INTEGRITY_TYPE")}, name="IKEEXT_INTEGRITY_ALGORITHM0", pack=False, align=None), "maxLifetimeSeconds": SimTypeInt(signed=False, label="UInt32"), "dhGroup": SimTypeInt(signed=False, label="IKEEXT_DH_GROUP"), "quickModeLimit": SimTypeInt(signed=False, label="UInt32")}, name="IKEEXT_PROPOSAL0", pack=False, align=None), offset=0), "flags": SimTypeInt(signed=False, label="IKEEXT_POLICY_FLAG"), "maxDynamicFilters": SimTypeInt(signed=False, label="UInt32"), "retransmitDurationSecs": SimTypeInt(signed=False, label="UInt32")}, name="IKEEXT_POLICY1", pack=False, align=None), offset=0), "idpOptions": SimTypePointer(SimStruct({"stateIdleTimeoutSeconds": SimTypeInt(signed=False, label="UInt32"), "perIPRateLimitQueueIdleTimeoutSeconds": SimTypeInt(signed=False, label="UInt32"), "ipV6IPsecUnauthDscp": SimTypeChar(label="Byte"), "ipV6IPsecUnauthRateLimitBytesPerSec": SimTypeInt(signed=False, label="UInt32"), "ipV6IPsecUnauthPerIPRateLimitBytesPerSec": SimTypeInt(signed=False, label="UInt32"), "ipV6IPsecAuthDscp": SimTypeChar(label="Byte"), "ipV6IPsecAuthRateLimitBytesPerSec": SimTypeInt(signed=False, label="UInt32"), "icmpV6Dscp": SimTypeChar(label="Byte"), "icmpV6RateLimitBytesPerSec": SimTypeInt(signed=False, label="UInt32"), "ipV6FilterExemptDscp": SimTypeChar(label="Byte"), "ipV6FilterExemptRateLimitBytesPerSec": SimTypeInt(signed=False, label="UInt32"), "defBlockExemptDscp": SimTypeChar(label="Byte"), "defBlockExemptRateLimitBytesPerSec": SimTypeInt(signed=False, label="UInt32"), "maxStateEntries": SimTypeInt(signed=False, label="UInt32"), "maxPerIPRateLimitQueues": SimTypeInt(signed=False, label="UInt32"), "flags": SimTypeInt(signed=False, label="IPSEC_DOSP_FLAGS"), "numPublicIFLuids": SimTypeInt(signed=False, label="UInt32"), "publicIFLuids": SimTypePointer(SimTypeLongLong(signed=False, label="UInt64"), offset=0), "numInternalIFLuids": SimTypeInt(signed=False, label="UInt32"), "internalIFLuids": SimTypePointer(SimTypeLongLong(signed=False, label="UInt64"), offset=0), "publicV6AddrMask": SimStruct({"addr": SimTypeFixedSizeArray(SimTypeChar(label="Byte"), 16), "prefixLength": SimTypeChar(label="Byte")}, name="FWP_V6_ADDR_AND_MASK", pack=False, align=None), "internalV6AddrMask": SimStruct({"addr": SimTypeFixedSizeArray(SimTypeChar(label="Byte"), 16), "prefixLength": SimTypeChar(label="Byte")}, name="FWP_V6_ADDR_AND_MASK", pack=False, align=None)}, name="IPSEC_DOSP_OPTIONS0", pack=False, align=None), offset=0)}, name="<anon>", label="None"), "providerContextId": SimTypeLongLong(signed=False, label="UInt64")}, name="FWPM_PROVIDER_CONTEXT1", pack=False, align=None), offset=0), offset=0), offset=0), SimTypePointer(SimTypeInt(signed=False, label="UInt32"), offset=0)], SimTypeInt(signed=False, label="UInt32"), arg_names=["engineHandle", "enumHandle", "numEntriesRequested", "entries", "numEntriesReturned"]),
        # 
        'FwpmProviderContextEnum2': SimTypeFunction([SimTypePointer(SimTypeInt(signed=True, label="Int"), label="IntPtr", offset=0), SimTypePointer(SimTypeInt(signed=True, label="Int"), label="IntPtr", offset=0), SimTypeInt(signed=False, label="UInt32"), SimTypePointer(SimTypePointer(SimTypePointer(SimStruct({"providerContextKey": SimTypeBottom(label="Guid"), "displayData": SimStruct({"name": SimTypePointer(SimTypeChar(label="Char"), offset=0), "description": SimTypePointer(SimTypeChar(label="Char"), offset=0)}, name="FWPM_DISPLAY_DATA0", pack=False, align=None), "flags": SimTypeInt(signed=False, label="UInt32"), "providerKey": SimTypePointer(SimTypeBottom(label="Guid"), offset=0), "providerData": SimStruct({"size": SimTypeInt(signed=False, label="UInt32"), "data": SimTypePointer(SimTypeChar(label="Byte"), offset=0)}, name="FWP_BYTE_BLOB", pack=False, align=None), "type": SimTypeInt(signed=False, label="FWPM_PROVIDER_CONTEXT_TYPE"), "Anonymous": SimUnion({"keyingPolicy": SimTypePointer(SimStruct({"numKeyMods": SimTypeInt(signed=False, label="UInt32"), "keyModKeys": SimTypePointer(SimTypeBottom(label="Guid"), offset=0), "flags": SimTypeInt(signed=False, label="UInt32")}, name="IPSEC_KEYING_POLICY1", pack=False, align=None), offset=0), "ikeQmTransportPolicy": SimTypePointer(SimStruct({"numIpsecProposals": SimTypeInt(signed=False, label="UInt32"), "ipsecProposals": SimTypePointer(SimStruct({"lifetime": SimStruct({"lifetimeSeconds": SimTypeInt(signed=False, label="UInt32"), "lifetimeKilobytes": SimTypeInt(signed=False, label="UInt32"), "lifetimePackets": SimTypeInt(signed=False, label="UInt32")}, name="IPSEC_SA_LIFETIME0", pack=False, align=None), "numSaTransforms": SimTypeInt(signed=False, label="UInt32"), "saTransforms": SimTypePointer(SimStruct({"ipsecTransformType": SimTypeInt(signed=False, label="IPSEC_TRANSFORM_TYPE"), "Anonymous": SimUnion({"ahTransform": SimTypePointer(SimStruct({"authTransformId": SimStruct({"authType": SimTypeInt(signed=False, label="IPSEC_AUTH_TYPE"), "authConfig": SimTypeChar(label="Byte")}, name="IPSEC_AUTH_TRANSFORM_ID0", pack=False, align=None), "cryptoModuleId": SimTypePointer(SimTypeBottom(label="Guid"), offset=0)}, name="IPSEC_AUTH_TRANSFORM0", pack=False, align=None), offset=0), "espAuthTransform": SimTypePointer(SimStruct({"authTransformId": SimStruct({"authType": SimTypeInt(signed=False, label="IPSEC_AUTH_TYPE"), "authConfig": SimTypeChar(label="Byte")}, name="IPSEC_AUTH_TRANSFORM_ID0", pack=False, align=None), "cryptoModuleId": SimTypePointer(SimTypeBottom(label="Guid"), offset=0)}, name="IPSEC_AUTH_TRANSFORM0", pack=False, align=None), offset=0), "espCipherTransform": SimTypePointer(SimStruct({"cipherTransformId": SimStruct({"cipherType": SimTypeInt(signed=False, label="IPSEC_CIPHER_TYPE"), "cipherConfig": SimTypeChar(label="Byte")}, name="IPSEC_CIPHER_TRANSFORM_ID0", pack=False, align=None), "cryptoModuleId": SimTypePointer(SimTypeBottom(label="Guid"), offset=0)}, name="IPSEC_CIPHER_TRANSFORM0", pack=False, align=None), offset=0), "espAuthAndCipherTransform": SimTypePointer(SimStruct({"authTransform": SimStruct({"authTransformId": SimStruct({"authType": SimTypeInt(signed=False, label="IPSEC_AUTH_TYPE"), "authConfig": SimTypeChar(label="Byte")}, name="IPSEC_AUTH_TRANSFORM_ID0", pack=False, align=None), "cryptoModuleId": SimTypePointer(SimTypeBottom(label="Guid"), offset=0)}, name="IPSEC_AUTH_TRANSFORM0", pack=False, align=None), "cipherTransform": SimStruct({"cipherTransformId": SimStruct({"cipherType": SimTypeInt(signed=False, label="IPSEC_CIPHER_TYPE"), "cipherConfig": SimTypeChar(label="Byte")}, name="IPSEC_CIPHER_TRANSFORM_ID0", pack=False, align=None), "cryptoModuleId": SimTypePointer(SimTypeBottom(label="Guid"), offset=0)}, name="IPSEC_CIPHER_TRANSFORM0", pack=False, align=None)}, name="IPSEC_AUTH_AND_CIPHER_TRANSFORM0", pack=False, align=None), offset=0), "espAuthFwTransform": SimTypePointer(SimStruct({"authTransformId": SimStruct({"authType": SimTypeInt(signed=False, label="IPSEC_AUTH_TYPE"), "authConfig": SimTypeChar(label="Byte")}, name="IPSEC_AUTH_TRANSFORM_ID0", pack=False, align=None), "cryptoModuleId": SimTypePointer(SimTypeBottom(label="Guid"), offset=0)}, name="IPSEC_AUTH_TRANSFORM0", pack=False, align=None), offset=0)}, name="<anon>", label="None")}, name="IPSEC_SA_TRANSFORM0", pack=False, align=None), offset=0), "pfsGroup": SimTypeInt(signed=False, label="IPSEC_PFS_GROUP")}, name="IPSEC_PROPOSAL0", pack=False, align=None), offset=0), "flags": SimTypeInt(signed=False, label="IPSEC_POLICY_FLAG"), "ndAllowClearTimeoutSeconds": SimTypeInt(signed=False, label="UInt32"), "saIdleTimeout": SimStruct({"idleTimeoutSeconds": SimTypeInt(signed=False, label="UInt32"), "idleTimeoutSecondsFailOver": SimTypeInt(signed=False, label="UInt32")}, name="IPSEC_SA_IDLE_TIMEOUT0", pack=False, align=None), "emPolicy": SimTypePointer(SimStruct({"numAuthenticationMethods": SimTypeInt(signed=False, label="UInt32"), "authenticationMethods": SimTypePointer(SimStruct({"authenticationMethodType": SimTypeInt(signed=False, label="IKEEXT_AUTHENTICATION_METHOD_TYPE"), "Anonymous": SimUnion({"presharedKeyAuthentication": SimStruct({"presharedKey": SimStruct({"size": SimTypeInt(signed=False, label="UInt32"), "data": SimTypePointer(SimTypeChar(label="Byte"), offset=0)}, name="FWP_BYTE_BLOB", pack=False, align=None), "flags": SimTypeInt(signed=False, label="IKEEXT_PRESHARED_KEY_AUTHENTICATION_FLAGS")}, name="IKEEXT_PRESHARED_KEY_AUTHENTICATION1", pack=False, align=None), "certificateAuthentication": SimStruct({"inboundConfigType": SimTypeInt(signed=False, label="IKEEXT_CERT_CONFIG_TYPE"), "Anonymous1": SimUnion({"Anonymous1": SimStruct({"inboundRootArraySize": SimTypeInt(signed=False, label="UInt32"), "inboundRootCriteria": SimTypePointer(SimStruct({"certData": SimStruct({"size": SimTypeInt(signed=False, label="UInt32"), "data": SimTypePointer(SimTypeChar(label="Byte"), offset=0)}, name="FWP_BYTE_BLOB", pack=False, align=None), "certHash": SimStruct({"size": SimTypeInt(signed=False, label="UInt32"), "data": SimTypePointer(SimTypeChar(label="Byte"), offset=0)}, name="FWP_BYTE_BLOB", pack=False, align=None), "eku": SimTypePointer(SimStruct({"numEku": SimTypeInt(signed=False, label="UInt32"), "eku": SimTypePointer(SimTypePointer(SimTypeChar(label="Byte"), offset=0), offset=0)}, name="IKEEXT_CERT_EKUS0", pack=False, align=None), offset=0), "name": SimTypePointer(SimStruct({"nameType": SimTypeInt(signed=False, label="IKEEXT_CERT_CRITERIA_NAME_TYPE"), "certName": SimTypePointer(SimTypeChar(label="Char"), offset=0)}, name="IKEEXT_CERT_NAME0", pack=False, align=None), offset=0), "flags": SimTypeInt(signed=False, label="UInt32")}, name="IKEEXT_CERTIFICATE_CRITERIA0", pack=False, align=None), offset=0)}, name="_Anonymous1_e__Struct", pack=False, align=None), "Anonymous2": SimStruct({"inboundEnterpriseStoreArraySize": SimTypeInt(signed=False, label="UInt32"), "inboundEnterpriseStoreCriteria": SimTypePointer(SimStruct({"certData": SimStruct({"size": SimTypeInt(signed=False, label="UInt32"), "data": SimTypePointer(SimTypeChar(label="Byte"), offset=0)}, name="FWP_BYTE_BLOB", pack=False, align=None), "certHash": SimStruct({"size": SimTypeInt(signed=False, label="UInt32"), "data": SimTypePointer(SimTypeChar(label="Byte"), offset=0)}, name="FWP_BYTE_BLOB", pack=False, align=None), "eku": SimTypePointer(SimStruct({"numEku": SimTypeInt(signed=False, label="UInt32"), "eku": SimTypePointer(SimTypePointer(SimTypeChar(label="Byte"), offset=0), offset=0)}, name="IKEEXT_CERT_EKUS0", pack=False, align=None), offset=0), "name": SimTypePointer(SimStruct({"nameType": SimTypeInt(signed=False, label="IKEEXT_CERT_CRITERIA_NAME_TYPE"), "certName": SimTypePointer(SimTypeChar(label="Char"), offset=0)}, name="IKEEXT_CERT_NAME0", pack=False, align=None), offset=0), "flags": SimTypeInt(signed=False, label="UInt32")}, name="IKEEXT_CERTIFICATE_CRITERIA0", pack=False, align=None), offset=0)}, name="_Anonymous2_e__Struct", pack=False, align=None), "Anonymous3": SimStruct({"inboundRootStoreArraySize": SimTypeInt(signed=False, label="UInt32"), "inboundTrustedRootStoreCriteria": SimTypePointer(SimStruct({"certData": SimStruct({"size": SimTypeInt(signed=False, label="UInt32"), "data": SimTypePointer(SimTypeChar(label="Byte"), offset=0)}, name="FWP_BYTE_BLOB", pack=False, align=None), "certHash": SimStruct({"size": SimTypeInt(signed=False, label="UInt32"), "data": SimTypePointer(SimTypeChar(label="Byte"), offset=0)}, name="FWP_BYTE_BLOB", pack=False, align=None), "eku": SimTypePointer(SimStruct({"numEku": SimTypeInt(signed=False, label="UInt32"), "eku": SimTypePointer(SimTypePointer(SimTypeChar(label="Byte"), offset=0), offset=0)}, name="IKEEXT_CERT_EKUS0", pack=False, align=None), offset=0), "name": SimTypePointer(SimStruct({"nameType": SimTypeInt(signed=False, label="IKEEXT_CERT_CRITERIA_NAME_TYPE"), "certName": SimTypePointer(SimTypeChar(label="Char"), offset=0)}, name="IKEEXT_CERT_NAME0", pack=False, align=None), offset=0), "flags": SimTypeInt(signed=False, label="UInt32")}, name="IKEEXT_CERTIFICATE_CRITERIA0", pack=False, align=None), offset=0)}, name="_Anonymous3_e__Struct", pack=False, align=None)}, name="<anon>", label="None"), "outboundConfigType": SimTypeInt(signed=False, label="IKEEXT_CERT_CONFIG_TYPE"), "Anonymous2": SimUnion({"Anonymous1": SimStruct({"outboundRootArraySize": SimTypeInt(signed=False, label="UInt32"), "outboundRootCriteria": SimTypePointer(SimStruct({"certData": SimStruct({"size": SimTypeInt(signed=False, label="UInt32"), "data": SimTypePointer(SimTypeChar(label="Byte"), offset=0)}, name="FWP_BYTE_BLOB", pack=False, align=None), "certHash": SimStruct({"size": SimTypeInt(signed=False, label="UInt32"), "data": SimTypePointer(SimTypeChar(label="Byte"), offset=0)}, name="FWP_BYTE_BLOB", pack=False, align=None), "eku": SimTypePointer(SimStruct({"numEku": SimTypeInt(signed=False, label="UInt32"), "eku": SimTypePointer(SimTypePointer(SimTypeChar(label="Byte"), offset=0), offset=0)}, name="IKEEXT_CERT_EKUS0", pack=False, align=None), offset=0), "name": SimTypePointer(SimStruct({"nameType": SimTypeInt(signed=False, label="IKEEXT_CERT_CRITERIA_NAME_TYPE"), "certName": SimTypePointer(SimTypeChar(label="Char"), offset=0)}, name="IKEEXT_CERT_NAME0", pack=False, align=None), offset=0), "flags": SimTypeInt(signed=False, label="UInt32")}, name="IKEEXT_CERTIFICATE_CRITERIA0", pack=False, align=None), offset=0)}, name="_Anonymous1_e__Struct", pack=False, align=None), "Anonymous2": SimStruct({"outboundEnterpriseStoreArraySize": SimTypeInt(signed=False, label="UInt32"), "outboundEnterpriseStoreCriteria": SimTypePointer(SimStruct({"certData": SimStruct({"size": SimTypeInt(signed=False, label="UInt32"), "data": SimTypePointer(SimTypeChar(label="Byte"), offset=0)}, name="FWP_BYTE_BLOB", pack=False, align=None), "certHash": SimStruct({"size": SimTypeInt(signed=False, label="UInt32"), "data": SimTypePointer(SimTypeChar(label="Byte"), offset=0)}, name="FWP_BYTE_BLOB", pack=False, align=None), "eku": SimTypePointer(SimStruct({"numEku": SimTypeInt(signed=False, label="UInt32"), "eku": SimTypePointer(SimTypePointer(SimTypeChar(label="Byte"), offset=0), offset=0)}, name="IKEEXT_CERT_EKUS0", pack=False, align=None), offset=0), "name": SimTypePointer(SimStruct({"nameType": SimTypeInt(signed=False, label="IKEEXT_CERT_CRITERIA_NAME_TYPE"), "certName": SimTypePointer(SimTypeChar(label="Char"), offset=0)}, name="IKEEXT_CERT_NAME0", pack=False, align=None), offset=0), "flags": SimTypeInt(signed=False, label="UInt32")}, name="IKEEXT_CERTIFICATE_CRITERIA0", pack=False, align=None), offset=0)}, name="_Anonymous2_e__Struct", pack=False, align=None), "Anonymous3": SimStruct({"outboundRootStoreArraySize": SimTypeInt(signed=False, label="UInt32"), "outboundTrustedRootStoreCriteria": SimTypePointer(SimStruct({"certData": SimStruct({"size": SimTypeInt(signed=False, label="UInt32"), "data": SimTypePointer(SimTypeChar(label="Byte"), offset=0)}, name="FWP_BYTE_BLOB", pack=False, align=None), "certHash": SimStruct({"size": SimTypeInt(signed=False, label="UInt32"), "data": SimTypePointer(SimTypeChar(label="Byte"), offset=0)}, name="FWP_BYTE_BLOB", pack=False, align=None), "eku": SimTypePointer(SimStruct({"numEku": SimTypeInt(signed=False, label="UInt32"), "eku": SimTypePointer(SimTypePointer(SimTypeChar(label="Byte"), offset=0), offset=0)}, name="IKEEXT_CERT_EKUS0", pack=False, align=None), offset=0), "name": SimTypePointer(SimStruct({"nameType": SimTypeInt(signed=False, label="IKEEXT_CERT_CRITERIA_NAME_TYPE"), "certName": SimTypePointer(SimTypeChar(label="Char"), offset=0)}, name="IKEEXT_CERT_NAME0", pack=False, align=None), offset=0), "flags": SimTypeInt(signed=False, label="UInt32")}, name="IKEEXT_CERTIFICATE_CRITERIA0", pack=False, align=None), offset=0)}, name="_Anonymous3_e__Struct", pack=False, align=None)}, name="<anon>", label="None"), "flags": SimTypeInt(signed=False, label="IKEEXT_CERT_AUTH"), "localCertLocationUrl": SimStruct({"size": SimTypeInt(signed=False, label="UInt32"), "data": SimTypePointer(SimTypeChar(label="Byte"), offset=0)}, name="FWP_BYTE_BLOB", pack=False, align=None)}, name="IKEEXT_CERTIFICATE_AUTHENTICATION2", pack=False, align=None), "kerberosAuthentication": SimStruct({"flags": SimTypeInt(signed=False, label="IKEEXT_KERBEROS_AUTHENTICATION_FLAGS"), "proxyServer": SimTypePointer(SimTypeChar(label="Char"), offset=0)}, name="IKEEXT_KERBEROS_AUTHENTICATION1", pack=False, align=None), "reservedAuthentication": SimStruct({"flags": SimTypeInt(signed=False, label="IKEEXT_RESERVED_AUTHENTICATION_FLAGS")}, name="IKEEXT_RESERVED_AUTHENTICATION0", pack=False, align=None), "ntlmV2Authentication": SimStruct({"flags": SimTypeInt(signed=False, label="UInt32")}, name="IKEEXT_NTLM_V2_AUTHENTICATION0", pack=False, align=None), "sslAuthentication": SimStruct({"inboundConfigType": SimTypeInt(signed=False, label="IKEEXT_CERT_CONFIG_TYPE"), "Anonymous1": SimUnion({"Anonymous1": SimStruct({"inboundRootArraySize": SimTypeInt(signed=False, label="UInt32"), "inboundRootCriteria": SimTypePointer(SimStruct({"certData": SimStruct({"size": SimTypeInt(signed=False, label="UInt32"), "data": SimTypePointer(SimTypeChar(label="Byte"), offset=0)}, name="FWP_BYTE_BLOB", pack=False, align=None), "certHash": SimStruct({"size": SimTypeInt(signed=False, label="UInt32"), "data": SimTypePointer(SimTypeChar(label="Byte"), offset=0)}, name="FWP_BYTE_BLOB", pack=False, align=None), "eku": SimTypePointer(SimStruct({"numEku": SimTypeInt(signed=False, label="UInt32"), "eku": SimTypePointer(SimTypePointer(SimTypeChar(label="Byte"), offset=0), offset=0)}, name="IKEEXT_CERT_EKUS0", pack=False, align=None), offset=0), "name": SimTypePointer(SimStruct({"nameType": SimTypeInt(signed=False, label="IKEEXT_CERT_CRITERIA_NAME_TYPE"), "certName": SimTypePointer(SimTypeChar(label="Char"), offset=0)}, name="IKEEXT_CERT_NAME0", pack=False, align=None), offset=0), "flags": SimTypeInt(signed=False, label="UInt32")}, name="IKEEXT_CERTIFICATE_CRITERIA0", pack=False, align=None), offset=0)}, name="_Anonymous1_e__Struct", pack=False, align=None), "Anonymous2": SimStruct({"inboundEnterpriseStoreArraySize": SimTypeInt(signed=False, label="UInt32"), "inboundEnterpriseStoreCriteria": SimTypePointer(SimStruct({"certData": SimStruct({"size": SimTypeInt(signed=False, label="UInt32"), "data": SimTypePointer(SimTypeChar(label="Byte"), offset=0)}, name="FWP_BYTE_BLOB", pack=False, align=None), "certHash": SimStruct({"size": SimTypeInt(signed=False, label="UInt32"), "data": SimTypePointer(SimTypeChar(label="Byte"), offset=0)}, name="FWP_BYTE_BLOB", pack=False, align=None), "eku": SimTypePointer(SimStruct({"numEku": SimTypeInt(signed=False, label="UInt32"), "eku": SimTypePointer(SimTypePointer(SimTypeChar(label="Byte"), offset=0), offset=0)}, name="IKEEXT_CERT_EKUS0", pack=False, align=None), offset=0), "name": SimTypePointer(SimStruct({"nameType": SimTypeInt(signed=False, label="IKEEXT_CERT_CRITERIA_NAME_TYPE"), "certName": SimTypePointer(SimTypeChar(label="Char"), offset=0)}, name="IKEEXT_CERT_NAME0", pack=False, align=None), offset=0), "flags": SimTypeInt(signed=False, label="UInt32")}, name="IKEEXT_CERTIFICATE_CRITERIA0", pack=False, align=None), offset=0)}, name="_Anonymous2_e__Struct", pack=False, align=None), "Anonymous3": SimStruct({"inboundRootStoreArraySize": SimTypeInt(signed=False, label="UInt32"), "inboundTrustedRootStoreCriteria": SimTypePointer(SimStruct({"certData": SimStruct({"size": SimTypeInt(signed=False, label="UInt32"), "data": SimTypePointer(SimTypeChar(label="Byte"), offset=0)}, name="FWP_BYTE_BLOB", pack=False, align=None), "certHash": SimStruct({"size": SimTypeInt(signed=False, label="UInt32"), "data": SimTypePointer(SimTypeChar(label="Byte"), offset=0)}, name="FWP_BYTE_BLOB", pack=False, align=None), "eku": SimTypePointer(SimStruct({"numEku": SimTypeInt(signed=False, label="UInt32"), "eku": SimTypePointer(SimTypePointer(SimTypeChar(label="Byte"), offset=0), offset=0)}, name="IKEEXT_CERT_EKUS0", pack=False, align=None), offset=0), "name": SimTypePointer(SimStruct({"nameType": SimTypeInt(signed=False, label="IKEEXT_CERT_CRITERIA_NAME_TYPE"), "certName": SimTypePointer(SimTypeChar(label="Char"), offset=0)}, name="IKEEXT_CERT_NAME0", pack=False, align=None), offset=0), "flags": SimTypeInt(signed=False, label="UInt32")}, name="IKEEXT_CERTIFICATE_CRITERIA0", pack=False, align=None), offset=0)}, name="_Anonymous3_e__Struct", pack=False, align=None)}, name="<anon>", label="None"), "outboundConfigType": SimTypeInt(signed=False, label="IKEEXT_CERT_CONFIG_TYPE"), "Anonymous2": SimUnion({"Anonymous1": SimStruct({"outboundRootArraySize": SimTypeInt(signed=False, label="UInt32"), "outboundRootCriteria": SimTypePointer(SimStruct({"certData": SimStruct({"size": SimTypeInt(signed=False, label="UInt32"), "data": SimTypePointer(SimTypeChar(label="Byte"), offset=0)}, name="FWP_BYTE_BLOB", pack=False, align=None), "certHash": SimStruct({"size": SimTypeInt(signed=False, label="UInt32"), "data": SimTypePointer(SimTypeChar(label="Byte"), offset=0)}, name="FWP_BYTE_BLOB", pack=False, align=None), "eku": SimTypePointer(SimStruct({"numEku": SimTypeInt(signed=False, label="UInt32"), "eku": SimTypePointer(SimTypePointer(SimTypeChar(label="Byte"), offset=0), offset=0)}, name="IKEEXT_CERT_EKUS0", pack=False, align=None), offset=0), "name": SimTypePointer(SimStruct({"nameType": SimTypeInt(signed=False, label="IKEEXT_CERT_CRITERIA_NAME_TYPE"), "certName": SimTypePointer(SimTypeChar(label="Char"), offset=0)}, name="IKEEXT_CERT_NAME0", pack=False, align=None), offset=0), "flags": SimTypeInt(signed=False, label="UInt32")}, name="IKEEXT_CERTIFICATE_CRITERIA0", pack=False, align=None), offset=0)}, name="_Anonymous1_e__Struct", pack=False, align=None), "Anonymous2": SimStruct({"outboundEnterpriseStoreArraySize": SimTypeInt(signed=False, label="UInt32"), "outboundEnterpriseStoreCriteria": SimTypePointer(SimStruct({"certData": SimStruct({"size": SimTypeInt(signed=False, label="UInt32"), "data": SimTypePointer(SimTypeChar(label="Byte"), offset=0)}, name="FWP_BYTE_BLOB", pack=False, align=None), "certHash": SimStruct({"size": SimTypeInt(signed=False, label="UInt32"), "data": SimTypePointer(SimTypeChar(label="Byte"), offset=0)}, name="FWP_BYTE_BLOB", pack=False, align=None), "eku": SimTypePointer(SimStruct({"numEku": SimTypeInt(signed=False, label="UInt32"), "eku": SimTypePointer(SimTypePointer(SimTypeChar(label="Byte"), offset=0), offset=0)}, name="IKEEXT_CERT_EKUS0", pack=False, align=None), offset=0), "name": SimTypePointer(SimStruct({"nameType": SimTypeInt(signed=False, label="IKEEXT_CERT_CRITERIA_NAME_TYPE"), "certName": SimTypePointer(SimTypeChar(label="Char"), offset=0)}, name="IKEEXT_CERT_NAME0", pack=False, align=None), offset=0), "flags": SimTypeInt(signed=False, label="UInt32")}, name="IKEEXT_CERTIFICATE_CRITERIA0", pack=False, align=None), offset=0)}, name="_Anonymous2_e__Struct", pack=False, align=None), "Anonymous3": SimStruct({"outboundRootStoreArraySize": SimTypeInt(signed=False, label="UInt32"), "outboundTrustedRootStoreCriteria": SimTypePointer(SimStruct({"certData": SimStruct({"size": SimTypeInt(signed=False, label="UInt32"), "data": SimTypePointer(SimTypeChar(label="Byte"), offset=0)}, name="FWP_BYTE_BLOB", pack=False, align=None), "certHash": SimStruct({"size": SimTypeInt(signed=False, label="UInt32"), "data": SimTypePointer(SimTypeChar(label="Byte"), offset=0)}, name="FWP_BYTE_BLOB", pack=False, align=None), "eku": SimTypePointer(SimStruct({"numEku": SimTypeInt(signed=False, label="UInt32"), "eku": SimTypePointer(SimTypePointer(SimTypeChar(label="Byte"), offset=0), offset=0)}, name="IKEEXT_CERT_EKUS0", pack=False, align=None), offset=0), "name": SimTypePointer(SimStruct({"nameType": SimTypeInt(signed=False, label="IKEEXT_CERT_CRITERIA_NAME_TYPE"), "certName": SimTypePointer(SimTypeChar(label="Char"), offset=0)}, name="IKEEXT_CERT_NAME0", pack=False, align=None), offset=0), "flags": SimTypeInt(signed=False, label="UInt32")}, name="IKEEXT_CERTIFICATE_CRITERIA0", pack=False, align=None), offset=0)}, name="_Anonymous3_e__Struct", pack=False, align=None)}, name="<anon>", label="None"), "flags": SimTypeInt(signed=False, label="IKEEXT_CERT_AUTH"), "localCertLocationUrl": SimStruct({"size": SimTypeInt(signed=False, label="UInt32"), "data": SimTypePointer(SimTypeChar(label="Byte"), offset=0)}, name="FWP_BYTE_BLOB", pack=False, align=None)}, name="IKEEXT_CERTIFICATE_AUTHENTICATION2", pack=False, align=None), "cgaAuthentication": SimStruct({"keyContainerName": SimTypePointer(SimTypeChar(label="Char"), offset=0), "cspName": SimTypePointer(SimTypeChar(label="Char"), offset=0), "cspType": SimTypeInt(signed=False, label="UInt32"), "cgaModifier": SimStruct({"byteArray16": SimTypeFixedSizeArray(SimTypeChar(label="Byte"), 16)}, name="FWP_BYTE_ARRAY16", pack=False, align=None), "cgaCollisionCount": SimTypeChar(label="Byte")}, name="IKEEXT_IPV6_CGA_AUTHENTICATION0", pack=False, align=None), "eapAuthentication": SimStruct({"flags": SimTypeInt(signed=False, label="IKEEXT_EAP_AUTHENTICATION_FLAGS")}, name="IKEEXT_EAP_AUTHENTICATION0", pack=False, align=None)}, name="<anon>", label="None")}, name="IKEEXT_AUTHENTICATION_METHOD2", pack=False, align=None), offset=0), "initiatorImpersonationType": SimTypeInt(signed=False, label="IKEEXT_AUTHENTICATION_IMPERSONATION_TYPE")}, name="IKEEXT_EM_POLICY2", pack=False, align=None), offset=0)}, name="IPSEC_TRANSPORT_POLICY2", pack=False, align=None), offset=0), "ikeQmTunnelPolicy": SimTypePointer(SimStruct({"flags": SimTypeInt(signed=False, label="IPSEC_POLICY_FLAG"), "numIpsecProposals": SimTypeInt(signed=False, label="UInt32"), "ipsecProposals": SimTypePointer(SimStruct({"lifetime": SimStruct({"lifetimeSeconds": SimTypeInt(signed=False, label="UInt32"), "lifetimeKilobytes": SimTypeInt(signed=False, label="UInt32"), "lifetimePackets": SimTypeInt(signed=False, label="UInt32")}, name="IPSEC_SA_LIFETIME0", pack=False, align=None), "numSaTransforms": SimTypeInt(signed=False, label="UInt32"), "saTransforms": SimTypePointer(SimStruct({"ipsecTransformType": SimTypeInt(signed=False, label="IPSEC_TRANSFORM_TYPE"), "Anonymous": SimUnion({"ahTransform": SimTypePointer(SimStruct({"authTransformId": SimStruct({"authType": SimTypeInt(signed=False, label="IPSEC_AUTH_TYPE"), "authConfig": SimTypeChar(label="Byte")}, name="IPSEC_AUTH_TRANSFORM_ID0", pack=False, align=None), "cryptoModuleId": SimTypePointer(SimTypeBottom(label="Guid"), offset=0)}, name="IPSEC_AUTH_TRANSFORM0", pack=False, align=None), offset=0), "espAuthTransform": SimTypePointer(SimStruct({"authTransformId": SimStruct({"authType": SimTypeInt(signed=False, label="IPSEC_AUTH_TYPE"), "authConfig": SimTypeChar(label="Byte")}, name="IPSEC_AUTH_TRANSFORM_ID0", pack=False, align=None), "cryptoModuleId": SimTypePointer(SimTypeBottom(label="Guid"), offset=0)}, name="IPSEC_AUTH_TRANSFORM0", pack=False, align=None), offset=0), "espCipherTransform": SimTypePointer(SimStruct({"cipherTransformId": SimStruct({"cipherType": SimTypeInt(signed=False, label="IPSEC_CIPHER_TYPE"), "cipherConfig": SimTypeChar(label="Byte")}, name="IPSEC_CIPHER_TRANSFORM_ID0", pack=False, align=None), "cryptoModuleId": SimTypePointer(SimTypeBottom(label="Guid"), offset=0)}, name="IPSEC_CIPHER_TRANSFORM0", pack=False, align=None), offset=0), "espAuthAndCipherTransform": SimTypePointer(SimStruct({"authTransform": SimStruct({"authTransformId": SimStruct({"authType": SimTypeInt(signed=False, label="IPSEC_AUTH_TYPE"), "authConfig": SimTypeChar(label="Byte")}, name="IPSEC_AUTH_TRANSFORM_ID0", pack=False, align=None), "cryptoModuleId": SimTypePointer(SimTypeBottom(label="Guid"), offset=0)}, name="IPSEC_AUTH_TRANSFORM0", pack=False, align=None), "cipherTransform": SimStruct({"cipherTransformId": SimStruct({"cipherType": SimTypeInt(signed=False, label="IPSEC_CIPHER_TYPE"), "cipherConfig": SimTypeChar(label="Byte")}, name="IPSEC_CIPHER_TRANSFORM_ID0", pack=False, align=None), "cryptoModuleId": SimTypePointer(SimTypeBottom(label="Guid"), offset=0)}, name="IPSEC_CIPHER_TRANSFORM0", pack=False, align=None)}, name="IPSEC_AUTH_AND_CIPHER_TRANSFORM0", pack=False, align=None), offset=0), "espAuthFwTransform": SimTypePointer(SimStruct({"authTransformId": SimStruct({"authType": SimTypeInt(signed=False, label="IPSEC_AUTH_TYPE"), "authConfig": SimTypeChar(label="Byte")}, name="IPSEC_AUTH_TRANSFORM_ID0", pack=False, align=None), "cryptoModuleId": SimTypePointer(SimTypeBottom(label="Guid"), offset=0)}, name="IPSEC_AUTH_TRANSFORM0", pack=False, align=None), offset=0)}, name="<anon>", label="None")}, name="IPSEC_SA_TRANSFORM0", pack=False, align=None), offset=0), "pfsGroup": SimTypeInt(signed=False, label="IPSEC_PFS_GROUP")}, name="IPSEC_PROPOSAL0", pack=False, align=None), offset=0), "tunnelEndpoints": SimStruct({"ipVersion": SimTypeInt(signed=False, label="FWP_IP_VERSION"), "Anonymous1": SimUnion({"localV4Address": SimTypeInt(signed=False, label="UInt32"), "localV6Address": SimTypeFixedSizeArray(SimTypeChar(label="Byte"), 16)}, name="<anon>", label="None"), "Anonymous2": SimUnion({"remoteV4Address": SimTypeInt(signed=False, label="UInt32"), "remoteV6Address": SimTypeFixedSizeArray(SimTypeChar(label="Byte"), 16)}, name="<anon>", label="None"), "localIfLuid": SimTypeLongLong(signed=False, label="UInt64"), "remoteFqdn": SimTypePointer(SimTypeChar(label="Char"), offset=0), "numAddresses": SimTypeInt(signed=False, label="UInt32"), "remoteAddresses": SimTypePointer(SimStruct({"ipVersion": SimTypeInt(signed=False, label="FWP_IP_VERSION"), "Anonymous": SimUnion({"v4Address": SimTypeInt(signed=False, label="UInt32"), "v6Address": SimTypeFixedSizeArray(SimTypeChar(label="Byte"), 16)}, name="<anon>", label="None")}, name="IPSEC_TUNNEL_ENDPOINT0", pack=False, align=None), offset=0)}, name="IPSEC_TUNNEL_ENDPOINTS2", pack=False, align=None), "saIdleTimeout": SimStruct({"idleTimeoutSeconds": SimTypeInt(signed=False, label="UInt32"), "idleTimeoutSecondsFailOver": SimTypeInt(signed=False, label="UInt32")}, name="IPSEC_SA_IDLE_TIMEOUT0", pack=False, align=None), "emPolicy": SimTypePointer(SimStruct({"numAuthenticationMethods": SimTypeInt(signed=False, label="UInt32"), "authenticationMethods": SimTypePointer(SimStruct({"authenticationMethodType": SimTypeInt(signed=False, label="IKEEXT_AUTHENTICATION_METHOD_TYPE"), "Anonymous": SimUnion({"presharedKeyAuthentication": SimStruct({"presharedKey": SimStruct({"size": SimTypeInt(signed=False, label="UInt32"), "data": SimTypePointer(SimTypeChar(label="Byte"), offset=0)}, name="FWP_BYTE_BLOB", pack=False, align=None), "flags": SimTypeInt(signed=False, label="IKEEXT_PRESHARED_KEY_AUTHENTICATION_FLAGS")}, name="IKEEXT_PRESHARED_KEY_AUTHENTICATION1", pack=False, align=None), "certificateAuthentication": SimStruct({"inboundConfigType": SimTypeInt(signed=False, label="IKEEXT_CERT_CONFIG_TYPE"), "Anonymous1": SimUnion({"Anonymous1": SimStruct({"inboundRootArraySize": SimTypeInt(signed=False, label="UInt32"), "inboundRootCriteria": SimTypePointer(SimStruct({"certData": SimStruct({"size": SimTypeInt(signed=False, label="UInt32"), "data": SimTypePointer(SimTypeChar(label="Byte"), offset=0)}, name="FWP_BYTE_BLOB", pack=False, align=None), "certHash": SimStruct({"size": SimTypeInt(signed=False, label="UInt32"), "data": SimTypePointer(SimTypeChar(label="Byte"), offset=0)}, name="FWP_BYTE_BLOB", pack=False, align=None), "eku": SimTypePointer(SimStruct({"numEku": SimTypeInt(signed=False, label="UInt32"), "eku": SimTypePointer(SimTypePointer(SimTypeChar(label="Byte"), offset=0), offset=0)}, name="IKEEXT_CERT_EKUS0", pack=False, align=None), offset=0), "name": SimTypePointer(SimStruct({"nameType": SimTypeInt(signed=False, label="IKEEXT_CERT_CRITERIA_NAME_TYPE"), "certName": SimTypePointer(SimTypeChar(label="Char"), offset=0)}, name="IKEEXT_CERT_NAME0", pack=False, align=None), offset=0), "flags": SimTypeInt(signed=False, label="UInt32")}, name="IKEEXT_CERTIFICATE_CRITERIA0", pack=False, align=None), offset=0)}, name="_Anonymous1_e__Struct", pack=False, align=None), "Anonymous2": SimStruct({"inboundEnterpriseStoreArraySize": SimTypeInt(signed=False, label="UInt32"), "inboundEnterpriseStoreCriteria": SimTypePointer(SimStruct({"certData": SimStruct({"size": SimTypeInt(signed=False, label="UInt32"), "data": SimTypePointer(SimTypeChar(label="Byte"), offset=0)}, name="FWP_BYTE_BLOB", pack=False, align=None), "certHash": SimStruct({"size": SimTypeInt(signed=False, label="UInt32"), "data": SimTypePointer(SimTypeChar(label="Byte"), offset=0)}, name="FWP_BYTE_BLOB", pack=False, align=None), "eku": SimTypePointer(SimStruct({"numEku": SimTypeInt(signed=False, label="UInt32"), "eku": SimTypePointer(SimTypePointer(SimTypeChar(label="Byte"), offset=0), offset=0)}, name="IKEEXT_CERT_EKUS0", pack=False, align=None), offset=0), "name": SimTypePointer(SimStruct({"nameType": SimTypeInt(signed=False, label="IKEEXT_CERT_CRITERIA_NAME_TYPE"), "certName": SimTypePointer(SimTypeChar(label="Char"), offset=0)}, name="IKEEXT_CERT_NAME0", pack=False, align=None), offset=0), "flags": SimTypeInt(signed=False, label="UInt32")}, name="IKEEXT_CERTIFICATE_CRITERIA0", pack=False, align=None), offset=0)}, name="_Anonymous2_e__Struct", pack=False, align=None), "Anonymous3": SimStruct({"inboundRootStoreArraySize": SimTypeInt(signed=False, label="UInt32"), "inboundTrustedRootStoreCriteria": SimTypePointer(SimStruct({"certData": SimStruct({"size": SimTypeInt(signed=False, label="UInt32"), "data": SimTypePointer(SimTypeChar(label="Byte"), offset=0)}, name="FWP_BYTE_BLOB", pack=False, align=None), "certHash": SimStruct({"size": SimTypeInt(signed=False, label="UInt32"), "data": SimTypePointer(SimTypeChar(label="Byte"), offset=0)}, name="FWP_BYTE_BLOB", pack=False, align=None), "eku": SimTypePointer(SimStruct({"numEku": SimTypeInt(signed=False, label="UInt32"), "eku": SimTypePointer(SimTypePointer(SimTypeChar(label="Byte"), offset=0), offset=0)}, name="IKEEXT_CERT_EKUS0", pack=False, align=None), offset=0), "name": SimTypePointer(SimStruct({"nameType": SimTypeInt(signed=False, label="IKEEXT_CERT_CRITERIA_NAME_TYPE"), "certName": SimTypePointer(SimTypeChar(label="Char"), offset=0)}, name="IKEEXT_CERT_NAME0", pack=False, align=None), offset=0), "flags": SimTypeInt(signed=False, label="UInt32")}, name="IKEEXT_CERTIFICATE_CRITERIA0", pack=False, align=None), offset=0)}, name="_Anonymous3_e__Struct", pack=False, align=None)}, name="<anon>", label="None"), "outboundConfigType": SimTypeInt(signed=False, label="IKEEXT_CERT_CONFIG_TYPE"), "Anonymous2": SimUnion({"Anonymous1": SimStruct({"outboundRootArraySize": SimTypeInt(signed=False, label="UInt32"), "outboundRootCriteria": SimTypePointer(SimStruct({"certData": SimStruct({"size": SimTypeInt(signed=False, label="UInt32"), "data": SimTypePointer(SimTypeChar(label="Byte"), offset=0)}, name="FWP_BYTE_BLOB", pack=False, align=None), "certHash": SimStruct({"size": SimTypeInt(signed=False, label="UInt32"), "data": SimTypePointer(SimTypeChar(label="Byte"), offset=0)}, name="FWP_BYTE_BLOB", pack=False, align=None), "eku": SimTypePointer(SimStruct({"numEku": SimTypeInt(signed=False, label="UInt32"), "eku": SimTypePointer(SimTypePointer(SimTypeChar(label="Byte"), offset=0), offset=0)}, name="IKEEXT_CERT_EKUS0", pack=False, align=None), offset=0), "name": SimTypePointer(SimStruct({"nameType": SimTypeInt(signed=False, label="IKEEXT_CERT_CRITERIA_NAME_TYPE"), "certName": SimTypePointer(SimTypeChar(label="Char"), offset=0)}, name="IKEEXT_CERT_NAME0", pack=False, align=None), offset=0), "flags": SimTypeInt(signed=False, label="UInt32")}, name="IKEEXT_CERTIFICATE_CRITERIA0", pack=False, align=None), offset=0)}, name="_Anonymous1_e__Struct", pack=False, align=None), "Anonymous2": SimStruct({"outboundEnterpriseStoreArraySize": SimTypeInt(signed=False, label="UInt32"), "outboundEnterpriseStoreCriteria": SimTypePointer(SimStruct({"certData": SimStruct({"size": SimTypeInt(signed=False, label="UInt32"), "data": SimTypePointer(SimTypeChar(label="Byte"), offset=0)}, name="FWP_BYTE_BLOB", pack=False, align=None), "certHash": SimStruct({"size": SimTypeInt(signed=False, label="UInt32"), "data": SimTypePointer(SimTypeChar(label="Byte"), offset=0)}, name="FWP_BYTE_BLOB", pack=False, align=None), "eku": SimTypePointer(SimStruct({"numEku": SimTypeInt(signed=False, label="UInt32"), "eku": SimTypePointer(SimTypePointer(SimTypeChar(label="Byte"), offset=0), offset=0)}, name="IKEEXT_CERT_EKUS0", pack=False, align=None), offset=0), "name": SimTypePointer(SimStruct({"nameType": SimTypeInt(signed=False, label="IKEEXT_CERT_CRITERIA_NAME_TYPE"), "certName": SimTypePointer(SimTypeChar(label="Char"), offset=0)}, name="IKEEXT_CERT_NAME0", pack=False, align=None), offset=0), "flags": SimTypeInt(signed=False, label="UInt32")}, name="IKEEXT_CERTIFICATE_CRITERIA0", pack=False, align=None), offset=0)}, name="_Anonymous2_e__Struct", pack=False, align=None), "Anonymous3": SimStruct({"outboundRootStoreArraySize": SimTypeInt(signed=False, label="UInt32"), "outboundTrustedRootStoreCriteria": SimTypePointer(SimStruct({"certData": SimStruct({"size": SimTypeInt(signed=False, label="UInt32"), "data": SimTypePointer(SimTypeChar(label="Byte"), offset=0)}, name="FWP_BYTE_BLOB", pack=False, align=None), "certHash": SimStruct({"size": SimTypeInt(signed=False, label="UInt32"), "data": SimTypePointer(SimTypeChar(label="Byte"), offset=0)}, name="FWP_BYTE_BLOB", pack=False, align=None), "eku": SimTypePointer(SimStruct({"numEku": SimTypeInt(signed=False, label="UInt32"), "eku": SimTypePointer(SimTypePointer(SimTypeChar(label="Byte"), offset=0), offset=0)}, name="IKEEXT_CERT_EKUS0", pack=False, align=None), offset=0), "name": SimTypePointer(SimStruct({"nameType": SimTypeInt(signed=False, label="IKEEXT_CERT_CRITERIA_NAME_TYPE"), "certName": SimTypePointer(SimTypeChar(label="Char"), offset=0)}, name="IKEEXT_CERT_NAME0", pack=False, align=None), offset=0), "flags": SimTypeInt(signed=False, label="UInt32")}, name="IKEEXT_CERTIFICATE_CRITERIA0", pack=False, align=None), offset=0)}, name="_Anonymous3_e__Struct", pack=False, align=None)}, name="<anon>", label="None"), "flags": SimTypeInt(signed=False, label="IKEEXT_CERT_AUTH"), "localCertLocationUrl": SimStruct({"size": SimTypeInt(signed=False, label="UInt32"), "data": SimTypePointer(SimTypeChar(label="Byte"), offset=0)}, name="FWP_BYTE_BLOB", pack=False, align=None)}, name="IKEEXT_CERTIFICATE_AUTHENTICATION2", pack=False, align=None), "kerberosAuthentication": SimStruct({"flags": SimTypeInt(signed=False, label="IKEEXT_KERBEROS_AUTHENTICATION_FLAGS"), "proxyServer": SimTypePointer(SimTypeChar(label="Char"), offset=0)}, name="IKEEXT_KERBEROS_AUTHENTICATION1", pack=False, align=None), "reservedAuthentication": SimStruct({"flags": SimTypeInt(signed=False, label="IKEEXT_RESERVED_AUTHENTICATION_FLAGS")}, name="IKEEXT_RESERVED_AUTHENTICATION0", pack=False, align=None), "ntlmV2Authentication": SimStruct({"flags": SimTypeInt(signed=False, label="UInt32")}, name="IKEEXT_NTLM_V2_AUTHENTICATION0", pack=False, align=None), "sslAuthentication": SimStruct({"inboundConfigType": SimTypeInt(signed=False, label="IKEEXT_CERT_CONFIG_TYPE"), "Anonymous1": SimUnion({"Anonymous1": SimStruct({"inboundRootArraySize": SimTypeInt(signed=False, label="UInt32"), "inboundRootCriteria": SimTypePointer(SimStruct({"certData": SimStruct({"size": SimTypeInt(signed=False, label="UInt32"), "data": SimTypePointer(SimTypeChar(label="Byte"), offset=0)}, name="FWP_BYTE_BLOB", pack=False, align=None), "certHash": SimStruct({"size": SimTypeInt(signed=False, label="UInt32"), "data": SimTypePointer(SimTypeChar(label="Byte"), offset=0)}, name="FWP_BYTE_BLOB", pack=False, align=None), "eku": SimTypePointer(SimStruct({"numEku": SimTypeInt(signed=False, label="UInt32"), "eku": SimTypePointer(SimTypePointer(SimTypeChar(label="Byte"), offset=0), offset=0)}, name="IKEEXT_CERT_EKUS0", pack=False, align=None), offset=0), "name": SimTypePointer(SimStruct({"nameType": SimTypeInt(signed=False, label="IKEEXT_CERT_CRITERIA_NAME_TYPE"), "certName": SimTypePointer(SimTypeChar(label="Char"), offset=0)}, name="IKEEXT_CERT_NAME0", pack=False, align=None), offset=0), "flags": SimTypeInt(signed=False, label="UInt32")}, name="IKEEXT_CERTIFICATE_CRITERIA0", pack=False, align=None), offset=0)}, name="_Anonymous1_e__Struct", pack=False, align=None), "Anonymous2": SimStruct({"inboundEnterpriseStoreArraySize": SimTypeInt(signed=False, label="UInt32"), "inboundEnterpriseStoreCriteria": SimTypePointer(SimStruct({"certData": SimStruct({"size": SimTypeInt(signed=False, label="UInt32"), "data": SimTypePointer(SimTypeChar(label="Byte"), offset=0)}, name="FWP_BYTE_BLOB", pack=False, align=None), "certHash": SimStruct({"size": SimTypeInt(signed=False, label="UInt32"), "data": SimTypePointer(SimTypeChar(label="Byte"), offset=0)}, name="FWP_BYTE_BLOB", pack=False, align=None), "eku": SimTypePointer(SimStruct({"numEku": SimTypeInt(signed=False, label="UInt32"), "eku": SimTypePointer(SimTypePointer(SimTypeChar(label="Byte"), offset=0), offset=0)}, name="IKEEXT_CERT_EKUS0", pack=False, align=None), offset=0), "name": SimTypePointer(SimStruct({"nameType": SimTypeInt(signed=False, label="IKEEXT_CERT_CRITERIA_NAME_TYPE"), "certName": SimTypePointer(SimTypeChar(label="Char"), offset=0)}, name="IKEEXT_CERT_NAME0", pack=False, align=None), offset=0), "flags": SimTypeInt(signed=False, label="UInt32")}, name="IKEEXT_CERTIFICATE_CRITERIA0", pack=False, align=None), offset=0)}, name="_Anonymous2_e__Struct", pack=False, align=None), "Anonymous3": SimStruct({"inboundRootStoreArraySize": SimTypeInt(signed=False, label="UInt32"), "inboundTrustedRootStoreCriteria": SimTypePointer(SimStruct({"certData": SimStruct({"size": SimTypeInt(signed=False, label="UInt32"), "data": SimTypePointer(SimTypeChar(label="Byte"), offset=0)}, name="FWP_BYTE_BLOB", pack=False, align=None), "certHash": SimStruct({"size": SimTypeInt(signed=False, label="UInt32"), "data": SimTypePointer(SimTypeChar(label="Byte"), offset=0)}, name="FWP_BYTE_BLOB", pack=False, align=None), "eku": SimTypePointer(SimStruct({"numEku": SimTypeInt(signed=False, label="UInt32"), "eku": SimTypePointer(SimTypePointer(SimTypeChar(label="Byte"), offset=0), offset=0)}, name="IKEEXT_CERT_EKUS0", pack=False, align=None), offset=0), "name": SimTypePointer(SimStruct({"nameType": SimTypeInt(signed=False, label="IKEEXT_CERT_CRITERIA_NAME_TYPE"), "certName": SimTypePointer(SimTypeChar(label="Char"), offset=0)}, name="IKEEXT_CERT_NAME0", pack=False, align=None), offset=0), "flags": SimTypeInt(signed=False, label="UInt32")}, name="IKEEXT_CERTIFICATE_CRITERIA0", pack=False, align=None), offset=0)}, name="_Anonymous3_e__Struct", pack=False, align=None)}, name="<anon>", label="None"), "outboundConfigType": SimTypeInt(signed=False, label="IKEEXT_CERT_CONFIG_TYPE"), "Anonymous2": SimUnion({"Anonymous1": SimStruct({"outboundRootArraySize": SimTypeInt(signed=False, label="UInt32"), "outboundRootCriteria": SimTypePointer(SimStruct({"certData": SimStruct({"size": SimTypeInt(signed=False, label="UInt32"), "data": SimTypePointer(SimTypeChar(label="Byte"), offset=0)}, name="FWP_BYTE_BLOB", pack=False, align=None), "certHash": SimStruct({"size": SimTypeInt(signed=False, label="UInt32"), "data": SimTypePointer(SimTypeChar(label="Byte"), offset=0)}, name="FWP_BYTE_BLOB", pack=False, align=None), "eku": SimTypePointer(SimStruct({"numEku": SimTypeInt(signed=False, label="UInt32"), "eku": SimTypePointer(SimTypePointer(SimTypeChar(label="Byte"), offset=0), offset=0)}, name="IKEEXT_CERT_EKUS0", pack=False, align=None), offset=0), "name": SimTypePointer(SimStruct({"nameType": SimTypeInt(signed=False, label="IKEEXT_CERT_CRITERIA_NAME_TYPE"), "certName": SimTypePointer(SimTypeChar(label="Char"), offset=0)}, name="IKEEXT_CERT_NAME0", pack=False, align=None), offset=0), "flags": SimTypeInt(signed=False, label="UInt32")}, name="IKEEXT_CERTIFICATE_CRITERIA0", pack=False, align=None), offset=0)}, name="_Anonymous1_e__Struct", pack=False, align=None), "Anonymous2": SimStruct({"outboundEnterpriseStoreArraySize": SimTypeInt(signed=False, label="UInt32"), "outboundEnterpriseStoreCriteria": SimTypePointer(SimStruct({"certData": SimStruct({"size": SimTypeInt(signed=False, label="UInt32"), "data": SimTypePointer(SimTypeChar(label="Byte"), offset=0)}, name="FWP_BYTE_BLOB", pack=False, align=None), "certHash": SimStruct({"size": SimTypeInt(signed=False, label="UInt32"), "data": SimTypePointer(SimTypeChar(label="Byte"), offset=0)}, name="FWP_BYTE_BLOB", pack=False, align=None), "eku": SimTypePointer(SimStruct({"numEku": SimTypeInt(signed=False, label="UInt32"), "eku": SimTypePointer(SimTypePointer(SimTypeChar(label="Byte"), offset=0), offset=0)}, name="IKEEXT_CERT_EKUS0", pack=False, align=None), offset=0), "name": SimTypePointer(SimStruct({"nameType": SimTypeInt(signed=False, label="IKEEXT_CERT_CRITERIA_NAME_TYPE"), "certName": SimTypePointer(SimTypeChar(label="Char"), offset=0)}, name="IKEEXT_CERT_NAME0", pack=False, align=None), offset=0), "flags": SimTypeInt(signed=False, label="UInt32")}, name="IKEEXT_CERTIFICATE_CRITERIA0", pack=False, align=None), offset=0)}, name="_Anonymous2_e__Struct", pack=False, align=None), "Anonymous3": SimStruct({"outboundRootStoreArraySize": SimTypeInt(signed=False, label="UInt32"), "outboundTrustedRootStoreCriteria": SimTypePointer(SimStruct({"certData": SimStruct({"size": SimTypeInt(signed=False, label="UInt32"), "data": SimTypePointer(SimTypeChar(label="Byte"), offset=0)}, name="FWP_BYTE_BLOB", pack=False, align=None), "certHash": SimStruct({"size": SimTypeInt(signed=False, label="UInt32"), "data": SimTypePointer(SimTypeChar(label="Byte"), offset=0)}, name="FWP_BYTE_BLOB", pack=False, align=None), "eku": SimTypePointer(SimStruct({"numEku": SimTypeInt(signed=False, label="UInt32"), "eku": SimTypePointer(SimTypePointer(SimTypeChar(label="Byte"), offset=0), offset=0)}, name="IKEEXT_CERT_EKUS0", pack=False, align=None), offset=0), "name": SimTypePointer(SimStruct({"nameType": SimTypeInt(signed=False, label="IKEEXT_CERT_CRITERIA_NAME_TYPE"), "certName": SimTypePointer(SimTypeChar(label="Char"), offset=0)}, name="IKEEXT_CERT_NAME0", pack=False, align=None), offset=0), "flags": SimTypeInt(signed=False, label="UInt32")}, name="IKEEXT_CERTIFICATE_CRITERIA0", pack=False, align=None), offset=0)}, name="_Anonymous3_e__Struct", pack=False, align=None)}, name="<anon>", label="None"), "flags": SimTypeInt(signed=False, label="IKEEXT_CERT_AUTH"), "localCertLocationUrl": SimStruct({"size": SimTypeInt(signed=False, label="UInt32"), "data": SimTypePointer(SimTypeChar(label="Byte"), offset=0)}, name="FWP_BYTE_BLOB", pack=False, align=None)}, name="IKEEXT_CERTIFICATE_AUTHENTICATION2", pack=False, align=None), "cgaAuthentication": SimStruct({"keyContainerName": SimTypePointer(SimTypeChar(label="Char"), offset=0), "cspName": SimTypePointer(SimTypeChar(label="Char"), offset=0), "cspType": SimTypeInt(signed=False, label="UInt32"), "cgaModifier": SimStruct({"byteArray16": SimTypeFixedSizeArray(SimTypeChar(label="Byte"), 16)}, name="FWP_BYTE_ARRAY16", pack=False, align=None), "cgaCollisionCount": SimTypeChar(label="Byte")}, name="IKEEXT_IPV6_CGA_AUTHENTICATION0", pack=False, align=None), "eapAuthentication": SimStruct({"flags": SimTypeInt(signed=False, label="IKEEXT_EAP_AUTHENTICATION_FLAGS")}, name="IKEEXT_EAP_AUTHENTICATION0", pack=False, align=None)}, name="<anon>", label="None")}, name="IKEEXT_AUTHENTICATION_METHOD2", pack=False, align=None), offset=0), "initiatorImpersonationType": SimTypeInt(signed=False, label="IKEEXT_AUTHENTICATION_IMPERSONATION_TYPE")}, name="IKEEXT_EM_POLICY2", pack=False, align=None), offset=0), "fwdPathSaLifetime": SimTypeInt(signed=False, label="UInt32")}, name="IPSEC_TUNNEL_POLICY2", pack=False, align=None), offset=0), "authipQmTransportPolicy": SimTypePointer(SimStruct({"numIpsecProposals": SimTypeInt(signed=False, label="UInt32"), "ipsecProposals": SimTypePointer(SimStruct({"lifetime": SimStruct({"lifetimeSeconds": SimTypeInt(signed=False, label="UInt32"), "lifetimeKilobytes": SimTypeInt(signed=False, label="UInt32"), "lifetimePackets": SimTypeInt(signed=False, label="UInt32")}, name="IPSEC_SA_LIFETIME0", pack=False, align=None), "numSaTransforms": SimTypeInt(signed=False, label="UInt32"), "saTransforms": SimTypePointer(SimStruct({"ipsecTransformType": SimTypeInt(signed=False, label="IPSEC_TRANSFORM_TYPE"), "Anonymous": SimUnion({"ahTransform": SimTypePointer(SimStruct({"authTransformId": SimStruct({"authType": SimTypeInt(signed=False, label="IPSEC_AUTH_TYPE"), "authConfig": SimTypeChar(label="Byte")}, name="IPSEC_AUTH_TRANSFORM_ID0", pack=False, align=None), "cryptoModuleId": SimTypePointer(SimTypeBottom(label="Guid"), offset=0)}, name="IPSEC_AUTH_TRANSFORM0", pack=False, align=None), offset=0), "espAuthTransform": SimTypePointer(SimStruct({"authTransformId": SimStruct({"authType": SimTypeInt(signed=False, label="IPSEC_AUTH_TYPE"), "authConfig": SimTypeChar(label="Byte")}, name="IPSEC_AUTH_TRANSFORM_ID0", pack=False, align=None), "cryptoModuleId": SimTypePointer(SimTypeBottom(label="Guid"), offset=0)}, name="IPSEC_AUTH_TRANSFORM0", pack=False, align=None), offset=0), "espCipherTransform": SimTypePointer(SimStruct({"cipherTransformId": SimStruct({"cipherType": SimTypeInt(signed=False, label="IPSEC_CIPHER_TYPE"), "cipherConfig": SimTypeChar(label="Byte")}, name="IPSEC_CIPHER_TRANSFORM_ID0", pack=False, align=None), "cryptoModuleId": SimTypePointer(SimTypeBottom(label="Guid"), offset=0)}, name="IPSEC_CIPHER_TRANSFORM0", pack=False, align=None), offset=0), "espAuthAndCipherTransform": SimTypePointer(SimStruct({"authTransform": SimStruct({"authTransformId": SimStruct({"authType": SimTypeInt(signed=False, label="IPSEC_AUTH_TYPE"), "authConfig": SimTypeChar(label="Byte")}, name="IPSEC_AUTH_TRANSFORM_ID0", pack=False, align=None), "cryptoModuleId": SimTypePointer(SimTypeBottom(label="Guid"), offset=0)}, name="IPSEC_AUTH_TRANSFORM0", pack=False, align=None), "cipherTransform": SimStruct({"cipherTransformId": SimStruct({"cipherType": SimTypeInt(signed=False, label="IPSEC_CIPHER_TYPE"), "cipherConfig": SimTypeChar(label="Byte")}, name="IPSEC_CIPHER_TRANSFORM_ID0", pack=False, align=None), "cryptoModuleId": SimTypePointer(SimTypeBottom(label="Guid"), offset=0)}, name="IPSEC_CIPHER_TRANSFORM0", pack=False, align=None)}, name="IPSEC_AUTH_AND_CIPHER_TRANSFORM0", pack=False, align=None), offset=0), "espAuthFwTransform": SimTypePointer(SimStruct({"authTransformId": SimStruct({"authType": SimTypeInt(signed=False, label="IPSEC_AUTH_TYPE"), "authConfig": SimTypeChar(label="Byte")}, name="IPSEC_AUTH_TRANSFORM_ID0", pack=False, align=None), "cryptoModuleId": SimTypePointer(SimTypeBottom(label="Guid"), offset=0)}, name="IPSEC_AUTH_TRANSFORM0", pack=False, align=None), offset=0)}, name="<anon>", label="None")}, name="IPSEC_SA_TRANSFORM0", pack=False, align=None), offset=0), "pfsGroup": SimTypeInt(signed=False, label="IPSEC_PFS_GROUP")}, name="IPSEC_PROPOSAL0", pack=False, align=None), offset=0), "flags": SimTypeInt(signed=False, label="IPSEC_POLICY_FLAG"), "ndAllowClearTimeoutSeconds": SimTypeInt(signed=False, label="UInt32"), "saIdleTimeout": SimStruct({"idleTimeoutSeconds": SimTypeInt(signed=False, label="UInt32"), "idleTimeoutSecondsFailOver": SimTypeInt(signed=False, label="UInt32")}, name="IPSEC_SA_IDLE_TIMEOUT0", pack=False, align=None), "emPolicy": SimTypePointer(SimStruct({"numAuthenticationMethods": SimTypeInt(signed=False, label="UInt32"), "authenticationMethods": SimTypePointer(SimStruct({"authenticationMethodType": SimTypeInt(signed=False, label="IKEEXT_AUTHENTICATION_METHOD_TYPE"), "Anonymous": SimUnion({"presharedKeyAuthentication": SimStruct({"presharedKey": SimStruct({"size": SimTypeInt(signed=False, label="UInt32"), "data": SimTypePointer(SimTypeChar(label="Byte"), offset=0)}, name="FWP_BYTE_BLOB", pack=False, align=None), "flags": SimTypeInt(signed=False, label="IKEEXT_PRESHARED_KEY_AUTHENTICATION_FLAGS")}, name="IKEEXT_PRESHARED_KEY_AUTHENTICATION1", pack=False, align=None), "certificateAuthentication": SimStruct({"inboundConfigType": SimTypeInt(signed=False, label="IKEEXT_CERT_CONFIG_TYPE"), "Anonymous1": SimUnion({"Anonymous1": SimStruct({"inboundRootArraySize": SimTypeInt(signed=False, label="UInt32"), "inboundRootCriteria": SimTypePointer(SimStruct({"certData": SimStruct({"size": SimTypeInt(signed=False, label="UInt32"), "data": SimTypePointer(SimTypeChar(label="Byte"), offset=0)}, name="FWP_BYTE_BLOB", pack=False, align=None), "certHash": SimStruct({"size": SimTypeInt(signed=False, label="UInt32"), "data": SimTypePointer(SimTypeChar(label="Byte"), offset=0)}, name="FWP_BYTE_BLOB", pack=False, align=None), "eku": SimTypePointer(SimStruct({"numEku": SimTypeInt(signed=False, label="UInt32"), "eku": SimTypePointer(SimTypePointer(SimTypeChar(label="Byte"), offset=0), offset=0)}, name="IKEEXT_CERT_EKUS0", pack=False, align=None), offset=0), "name": SimTypePointer(SimStruct({"nameType": SimTypeInt(signed=False, label="IKEEXT_CERT_CRITERIA_NAME_TYPE"), "certName": SimTypePointer(SimTypeChar(label="Char"), offset=0)}, name="IKEEXT_CERT_NAME0", pack=False, align=None), offset=0), "flags": SimTypeInt(signed=False, label="UInt32")}, name="IKEEXT_CERTIFICATE_CRITERIA0", pack=False, align=None), offset=0)}, name="_Anonymous1_e__Struct", pack=False, align=None), "Anonymous2": SimStruct({"inboundEnterpriseStoreArraySize": SimTypeInt(signed=False, label="UInt32"), "inboundEnterpriseStoreCriteria": SimTypePointer(SimStruct({"certData": SimStruct({"size": SimTypeInt(signed=False, label="UInt32"), "data": SimTypePointer(SimTypeChar(label="Byte"), offset=0)}, name="FWP_BYTE_BLOB", pack=False, align=None), "certHash": SimStruct({"size": SimTypeInt(signed=False, label="UInt32"), "data": SimTypePointer(SimTypeChar(label="Byte"), offset=0)}, name="FWP_BYTE_BLOB", pack=False, align=None), "eku": SimTypePointer(SimStruct({"numEku": SimTypeInt(signed=False, label="UInt32"), "eku": SimTypePointer(SimTypePointer(SimTypeChar(label="Byte"), offset=0), offset=0)}, name="IKEEXT_CERT_EKUS0", pack=False, align=None), offset=0), "name": SimTypePointer(SimStruct({"nameType": SimTypeInt(signed=False, label="IKEEXT_CERT_CRITERIA_NAME_TYPE"), "certName": SimTypePointer(SimTypeChar(label="Char"), offset=0)}, name="IKEEXT_CERT_NAME0", pack=False, align=None), offset=0), "flags": SimTypeInt(signed=False, label="UInt32")}, name="IKEEXT_CERTIFICATE_CRITERIA0", pack=False, align=None), offset=0)}, name="_Anonymous2_e__Struct", pack=False, align=None), "Anonymous3": SimStruct({"inboundRootStoreArraySize": SimTypeInt(signed=False, label="UInt32"), "inboundTrustedRootStoreCriteria": SimTypePointer(SimStruct({"certData": SimStruct({"size": SimTypeInt(signed=False, label="UInt32"), "data": SimTypePointer(SimTypeChar(label="Byte"), offset=0)}, name="FWP_BYTE_BLOB", pack=False, align=None), "certHash": SimStruct({"size": SimTypeInt(signed=False, label="UInt32"), "data": SimTypePointer(SimTypeChar(label="Byte"), offset=0)}, name="FWP_BYTE_BLOB", pack=False, align=None), "eku": SimTypePointer(SimStruct({"numEku": SimTypeInt(signed=False, label="UInt32"), "eku": SimTypePointer(SimTypePointer(SimTypeChar(label="Byte"), offset=0), offset=0)}, name="IKEEXT_CERT_EKUS0", pack=False, align=None), offset=0), "name": SimTypePointer(SimStruct({"nameType": SimTypeInt(signed=False, label="IKEEXT_CERT_CRITERIA_NAME_TYPE"), "certName": SimTypePointer(SimTypeChar(label="Char"), offset=0)}, name="IKEEXT_CERT_NAME0", pack=False, align=None), offset=0), "flags": SimTypeInt(signed=False, label="UInt32")}, name="IKEEXT_CERTIFICATE_CRITERIA0", pack=False, align=None), offset=0)}, name="_Anonymous3_e__Struct", pack=False, align=None)}, name="<anon>", label="None"), "outboundConfigType": SimTypeInt(signed=False, label="IKEEXT_CERT_CONFIG_TYPE"), "Anonymous2": SimUnion({"Anonymous1": SimStruct({"outboundRootArraySize": SimTypeInt(signed=False, label="UInt32"), "outboundRootCriteria": SimTypePointer(SimStruct({"certData": SimStruct({"size": SimTypeInt(signed=False, label="UInt32"), "data": SimTypePointer(SimTypeChar(label="Byte"), offset=0)}, name="FWP_BYTE_BLOB", pack=False, align=None), "certHash": SimStruct({"size": SimTypeInt(signed=False, label="UInt32"), "data": SimTypePointer(SimTypeChar(label="Byte"), offset=0)}, name="FWP_BYTE_BLOB", pack=False, align=None), "eku": SimTypePointer(SimStruct({"numEku": SimTypeInt(signed=False, label="UInt32"), "eku": SimTypePointer(SimTypePointer(SimTypeChar(label="Byte"), offset=0), offset=0)}, name="IKEEXT_CERT_EKUS0", pack=False, align=None), offset=0), "name": SimTypePointer(SimStruct({"nameType": SimTypeInt(signed=False, label="IKEEXT_CERT_CRITERIA_NAME_TYPE"), "certName": SimTypePointer(SimTypeChar(label="Char"), offset=0)}, name="IKEEXT_CERT_NAME0", pack=False, align=None), offset=0), "flags": SimTypeInt(signed=False, label="UInt32")}, name="IKEEXT_CERTIFICATE_CRITERIA0", pack=False, align=None), offset=0)}, name="_Anonymous1_e__Struct", pack=False, align=None), "Anonymous2": SimStruct({"outboundEnterpriseStoreArraySize": SimTypeInt(signed=False, label="UInt32"), "outboundEnterpriseStoreCriteria": SimTypePointer(SimStruct({"certData": SimStruct({"size": SimTypeInt(signed=False, label="UInt32"), "data": SimTypePointer(SimTypeChar(label="Byte"), offset=0)}, name="FWP_BYTE_BLOB", pack=False, align=None), "certHash": SimStruct({"size": SimTypeInt(signed=False, label="UInt32"), "data": SimTypePointer(SimTypeChar(label="Byte"), offset=0)}, name="FWP_BYTE_BLOB", pack=False, align=None), "eku": SimTypePointer(SimStruct({"numEku": SimTypeInt(signed=False, label="UInt32"), "eku": SimTypePointer(SimTypePointer(SimTypeChar(label="Byte"), offset=0), offset=0)}, name="IKEEXT_CERT_EKUS0", pack=False, align=None), offset=0), "name": SimTypePointer(SimStruct({"nameType": SimTypeInt(signed=False, label="IKEEXT_CERT_CRITERIA_NAME_TYPE"), "certName": SimTypePointer(SimTypeChar(label="Char"), offset=0)}, name="IKEEXT_CERT_NAME0", pack=False, align=None), offset=0), "flags": SimTypeInt(signed=False, label="UInt32")}, name="IKEEXT_CERTIFICATE_CRITERIA0", pack=False, align=None), offset=0)}, name="_Anonymous2_e__Struct", pack=False, align=None), "Anonymous3": SimStruct({"outboundRootStoreArraySize": SimTypeInt(signed=False, label="UInt32"), "outboundTrustedRootStoreCriteria": SimTypePointer(SimStruct({"certData": SimStruct({"size": SimTypeInt(signed=False, label="UInt32"), "data": SimTypePointer(SimTypeChar(label="Byte"), offset=0)}, name="FWP_BYTE_BLOB", pack=False, align=None), "certHash": SimStruct({"size": SimTypeInt(signed=False, label="UInt32"), "data": SimTypePointer(SimTypeChar(label="Byte"), offset=0)}, name="FWP_BYTE_BLOB", pack=False, align=None), "eku": SimTypePointer(SimStruct({"numEku": SimTypeInt(signed=False, label="UInt32"), "eku": SimTypePointer(SimTypePointer(SimTypeChar(label="Byte"), offset=0), offset=0)}, name="IKEEXT_CERT_EKUS0", pack=False, align=None), offset=0), "name": SimTypePointer(SimStruct({"nameType": SimTypeInt(signed=False, label="IKEEXT_CERT_CRITERIA_NAME_TYPE"), "certName": SimTypePointer(SimTypeChar(label="Char"), offset=0)}, name="IKEEXT_CERT_NAME0", pack=False, align=None), offset=0), "flags": SimTypeInt(signed=False, label="UInt32")}, name="IKEEXT_CERTIFICATE_CRITERIA0", pack=False, align=None), offset=0)}, name="_Anonymous3_e__Struct", pack=False, align=None)}, name="<anon>", label="None"), "flags": SimTypeInt(signed=False, label="IKEEXT_CERT_AUTH"), "localCertLocationUrl": SimStruct({"size": SimTypeInt(signed=False, label="UInt32"), "data": SimTypePointer(SimTypeChar(label="Byte"), offset=0)}, name="FWP_BYTE_BLOB", pack=False, align=None)}, name="IKEEXT_CERTIFICATE_AUTHENTICATION2", pack=False, align=None), "kerberosAuthentication": SimStruct({"flags": SimTypeInt(signed=False, label="IKEEXT_KERBEROS_AUTHENTICATION_FLAGS"), "proxyServer": SimTypePointer(SimTypeChar(label="Char"), offset=0)}, name="IKEEXT_KERBEROS_AUTHENTICATION1", pack=False, align=None), "reservedAuthentication": SimStruct({"flags": SimTypeInt(signed=False, label="IKEEXT_RESERVED_AUTHENTICATION_FLAGS")}, name="IKEEXT_RESERVED_AUTHENTICATION0", pack=False, align=None), "ntlmV2Authentication": SimStruct({"flags": SimTypeInt(signed=False, label="UInt32")}, name="IKEEXT_NTLM_V2_AUTHENTICATION0", pack=False, align=None), "sslAuthentication": SimStruct({"inboundConfigType": SimTypeInt(signed=False, label="IKEEXT_CERT_CONFIG_TYPE"), "Anonymous1": SimUnion({"Anonymous1": SimStruct({"inboundRootArraySize": SimTypeInt(signed=False, label="UInt32"), "inboundRootCriteria": SimTypePointer(SimStruct({"certData": SimStruct({"size": SimTypeInt(signed=False, label="UInt32"), "data": SimTypePointer(SimTypeChar(label="Byte"), offset=0)}, name="FWP_BYTE_BLOB", pack=False, align=None), "certHash": SimStruct({"size": SimTypeInt(signed=False, label="UInt32"), "data": SimTypePointer(SimTypeChar(label="Byte"), offset=0)}, name="FWP_BYTE_BLOB", pack=False, align=None), "eku": SimTypePointer(SimStruct({"numEku": SimTypeInt(signed=False, label="UInt32"), "eku": SimTypePointer(SimTypePointer(SimTypeChar(label="Byte"), offset=0), offset=0)}, name="IKEEXT_CERT_EKUS0", pack=False, align=None), offset=0), "name": SimTypePointer(SimStruct({"nameType": SimTypeInt(signed=False, label="IKEEXT_CERT_CRITERIA_NAME_TYPE"), "certName": SimTypePointer(SimTypeChar(label="Char"), offset=0)}, name="IKEEXT_CERT_NAME0", pack=False, align=None), offset=0), "flags": SimTypeInt(signed=False, label="UInt32")}, name="IKEEXT_CERTIFICATE_CRITERIA0", pack=False, align=None), offset=0)}, name="_Anonymous1_e__Struct", pack=False, align=None), "Anonymous2": SimStruct({"inboundEnterpriseStoreArraySize": SimTypeInt(signed=False, label="UInt32"), "inboundEnterpriseStoreCriteria": SimTypePointer(SimStruct({"certData": SimStruct({"size": SimTypeInt(signed=False, label="UInt32"), "data": SimTypePointer(SimTypeChar(label="Byte"), offset=0)}, name="FWP_BYTE_BLOB", pack=False, align=None), "certHash": SimStruct({"size": SimTypeInt(signed=False, label="UInt32"), "data": SimTypePointer(SimTypeChar(label="Byte"), offset=0)}, name="FWP_BYTE_BLOB", pack=False, align=None), "eku": SimTypePointer(SimStruct({"numEku": SimTypeInt(signed=False, label="UInt32"), "eku": SimTypePointer(SimTypePointer(SimTypeChar(label="Byte"), offset=0), offset=0)}, name="IKEEXT_CERT_EKUS0", pack=False, align=None), offset=0), "name": SimTypePointer(SimStruct({"nameType": SimTypeInt(signed=False, label="IKEEXT_CERT_CRITERIA_NAME_TYPE"), "certName": SimTypePointer(SimTypeChar(label="Char"), offset=0)}, name="IKEEXT_CERT_NAME0", pack=False, align=None), offset=0), "flags": SimTypeInt(signed=False, label="UInt32")}, name="IKEEXT_CERTIFICATE_CRITERIA0", pack=False, align=None), offset=0)}, name="_Anonymous2_e__Struct", pack=False, align=None), "Anonymous3": SimStruct({"inboundRootStoreArraySize": SimTypeInt(signed=False, label="UInt32"), "inboundTrustedRootStoreCriteria": SimTypePointer(SimStruct({"certData": SimStruct({"size": SimTypeInt(signed=False, label="UInt32"), "data": SimTypePointer(SimTypeChar(label="Byte"), offset=0)}, name="FWP_BYTE_BLOB", pack=False, align=None), "certHash": SimStruct({"size": SimTypeInt(signed=False, label="UInt32"), "data": SimTypePointer(SimTypeChar(label="Byte"), offset=0)}, name="FWP_BYTE_BLOB", pack=False, align=None), "eku": SimTypePointer(SimStruct({"numEku": SimTypeInt(signed=False, label="UInt32"), "eku": SimTypePointer(SimTypePointer(SimTypeChar(label="Byte"), offset=0), offset=0)}, name="IKEEXT_CERT_EKUS0", pack=False, align=None), offset=0), "name": SimTypePointer(SimStruct({"nameType": SimTypeInt(signed=False, label="IKEEXT_CERT_CRITERIA_NAME_TYPE"), "certName": SimTypePointer(SimTypeChar(label="Char"), offset=0)}, name="IKEEXT_CERT_NAME0", pack=False, align=None), offset=0), "flags": SimTypeInt(signed=False, label="UInt32")}, name="IKEEXT_CERTIFICATE_CRITERIA0", pack=False, align=None), offset=0)}, name="_Anonymous3_e__Struct", pack=False, align=None)}, name="<anon>", label="None"), "outboundConfigType": SimTypeInt(signed=False, label="IKEEXT_CERT_CONFIG_TYPE"), "Anonymous2": SimUnion({"Anonymous1": SimStruct({"outboundRootArraySize": SimTypeInt(signed=False, label="UInt32"), "outboundRootCriteria": SimTypePointer(SimStruct({"certData": SimStruct({"size": SimTypeInt(signed=False, label="UInt32"), "data": SimTypePointer(SimTypeChar(label="Byte"), offset=0)}, name="FWP_BYTE_BLOB", pack=False, align=None), "certHash": SimStruct({"size": SimTypeInt(signed=False, label="UInt32"), "data": SimTypePointer(SimTypeChar(label="Byte"), offset=0)}, name="FWP_BYTE_BLOB", pack=False, align=None), "eku": SimTypePointer(SimStruct({"numEku": SimTypeInt(signed=False, label="UInt32"), "eku": SimTypePointer(SimTypePointer(SimTypeChar(label="Byte"), offset=0), offset=0)}, name="IKEEXT_CERT_EKUS0", pack=False, align=None), offset=0), "name": SimTypePointer(SimStruct({"nameType": SimTypeInt(signed=False, label="IKEEXT_CERT_CRITERIA_NAME_TYPE"), "certName": SimTypePointer(SimTypeChar(label="Char"), offset=0)}, name="IKEEXT_CERT_NAME0", pack=False, align=None), offset=0), "flags": SimTypeInt(signed=False, label="UInt32")}, name="IKEEXT_CERTIFICATE_CRITERIA0", pack=False, align=None), offset=0)}, name="_Anonymous1_e__Struct", pack=False, align=None), "Anonymous2": SimStruct({"outboundEnterpriseStoreArraySize": SimTypeInt(signed=False, label="UInt32"), "outboundEnterpriseStoreCriteria": SimTypePointer(SimStruct({"certData": SimStruct({"size": SimTypeInt(signed=False, label="UInt32"), "data": SimTypePointer(SimTypeChar(label="Byte"), offset=0)}, name="FWP_BYTE_BLOB", pack=False, align=None), "certHash": SimStruct({"size": SimTypeInt(signed=False, label="UInt32"), "data": SimTypePointer(SimTypeChar(label="Byte"), offset=0)}, name="FWP_BYTE_BLOB", pack=False, align=None), "eku": SimTypePointer(SimStruct({"numEku": SimTypeInt(signed=False, label="UInt32"), "eku": SimTypePointer(SimTypePointer(SimTypeChar(label="Byte"), offset=0), offset=0)}, name="IKEEXT_CERT_EKUS0", pack=False, align=None), offset=0), "name": SimTypePointer(SimStruct({"nameType": SimTypeInt(signed=False, label="IKEEXT_CERT_CRITERIA_NAME_TYPE"), "certName": SimTypePointer(SimTypeChar(label="Char"), offset=0)}, name="IKEEXT_CERT_NAME0", pack=False, align=None), offset=0), "flags": SimTypeInt(signed=False, label="UInt32")}, name="IKEEXT_CERTIFICATE_CRITERIA0", pack=False, align=None), offset=0)}, name="_Anonymous2_e__Struct", pack=False, align=None), "Anonymous3": SimStruct({"outboundRootStoreArraySize": SimTypeInt(signed=False, label="UInt32"), "outboundTrustedRootStoreCriteria": SimTypePointer(SimStruct({"certData": SimStruct({"size": SimTypeInt(signed=False, label="UInt32"), "data": SimTypePointer(SimTypeChar(label="Byte"), offset=0)}, name="FWP_BYTE_BLOB", pack=False, align=None), "certHash": SimStruct({"size": SimTypeInt(signed=False, label="UInt32"), "data": SimTypePointer(SimTypeChar(label="Byte"), offset=0)}, name="FWP_BYTE_BLOB", pack=False, align=None), "eku": SimTypePointer(SimStruct({"numEku": SimTypeInt(signed=False, label="UInt32"), "eku": SimTypePointer(SimTypePointer(SimTypeChar(label="Byte"), offset=0), offset=0)}, name="IKEEXT_CERT_EKUS0", pack=False, align=None), offset=0), "name": SimTypePointer(SimStruct({"nameType": SimTypeInt(signed=False, label="IKEEXT_CERT_CRITERIA_NAME_TYPE"), "certName": SimTypePointer(SimTypeChar(label="Char"), offset=0)}, name="IKEEXT_CERT_NAME0", pack=False, align=None), offset=0), "flags": SimTypeInt(signed=False, label="UInt32")}, name="IKEEXT_CERTIFICATE_CRITERIA0", pack=False, align=None), offset=0)}, name="_Anonymous3_e__Struct", pack=False, align=None)}, name="<anon>", label="None"), "flags": SimTypeInt(signed=False, label="IKEEXT_CERT_AUTH"), "localCertLocationUrl": SimStruct({"size": SimTypeInt(signed=False, label="UInt32"), "data": SimTypePointer(SimTypeChar(label="Byte"), offset=0)}, name="FWP_BYTE_BLOB", pack=False, align=None)}, name="IKEEXT_CERTIFICATE_AUTHENTICATION2", pack=False, align=None), "cgaAuthentication": SimStruct({"keyContainerName": SimTypePointer(SimTypeChar(label="Char"), offset=0), "cspName": SimTypePointer(SimTypeChar(label="Char"), offset=0), "cspType": SimTypeInt(signed=False, label="UInt32"), "cgaModifier": SimStruct({"byteArray16": SimTypeFixedSizeArray(SimTypeChar(label="Byte"), 16)}, name="FWP_BYTE_ARRAY16", pack=False, align=None), "cgaCollisionCount": SimTypeChar(label="Byte")}, name="IKEEXT_IPV6_CGA_AUTHENTICATION0", pack=False, align=None), "eapAuthentication": SimStruct({"flags": SimTypeInt(signed=False, label="IKEEXT_EAP_AUTHENTICATION_FLAGS")}, name="IKEEXT_EAP_AUTHENTICATION0", pack=False, align=None)}, name="<anon>", label="None")}, name="IKEEXT_AUTHENTICATION_METHOD2", pack=False, align=None), offset=0), "initiatorImpersonationType": SimTypeInt(signed=False, label="IKEEXT_AUTHENTICATION_IMPERSONATION_TYPE")}, name="IKEEXT_EM_POLICY2", pack=False, align=None), offset=0)}, name="IPSEC_TRANSPORT_POLICY2", pack=False, align=None), offset=0), "authipQmTunnelPolicy": SimTypePointer(SimStruct({"flags": SimTypeInt(signed=False, label="IPSEC_POLICY_FLAG"), "numIpsecProposals": SimTypeInt(signed=False, label="UInt32"), "ipsecProposals": SimTypePointer(SimStruct({"lifetime": SimStruct({"lifetimeSeconds": SimTypeInt(signed=False, label="UInt32"), "lifetimeKilobytes": SimTypeInt(signed=False, label="UInt32"), "lifetimePackets": SimTypeInt(signed=False, label="UInt32")}, name="IPSEC_SA_LIFETIME0", pack=False, align=None), "numSaTransforms": SimTypeInt(signed=False, label="UInt32"), "saTransforms": SimTypePointer(SimStruct({"ipsecTransformType": SimTypeInt(signed=False, label="IPSEC_TRANSFORM_TYPE"), "Anonymous": SimUnion({"ahTransform": SimTypePointer(SimStruct({"authTransformId": SimStruct({"authType": SimTypeInt(signed=False, label="IPSEC_AUTH_TYPE"), "authConfig": SimTypeChar(label="Byte")}, name="IPSEC_AUTH_TRANSFORM_ID0", pack=False, align=None), "cryptoModuleId": SimTypePointer(SimTypeBottom(label="Guid"), offset=0)}, name="IPSEC_AUTH_TRANSFORM0", pack=False, align=None), offset=0), "espAuthTransform": SimTypePointer(SimStruct({"authTransformId": SimStruct({"authType": SimTypeInt(signed=False, label="IPSEC_AUTH_TYPE"), "authConfig": SimTypeChar(label="Byte")}, name="IPSEC_AUTH_TRANSFORM_ID0", pack=False, align=None), "cryptoModuleId": SimTypePointer(SimTypeBottom(label="Guid"), offset=0)}, name="IPSEC_AUTH_TRANSFORM0", pack=False, align=None), offset=0), "espCipherTransform": SimTypePointer(SimStruct({"cipherTransformId": SimStruct({"cipherType": SimTypeInt(signed=False, label="IPSEC_CIPHER_TYPE"), "cipherConfig": SimTypeChar(label="Byte")}, name="IPSEC_CIPHER_TRANSFORM_ID0", pack=False, align=None), "cryptoModuleId": SimTypePointer(SimTypeBottom(label="Guid"), offset=0)}, name="IPSEC_CIPHER_TRANSFORM0", pack=False, align=None), offset=0), "espAuthAndCipherTransform": SimTypePointer(SimStruct({"authTransform": SimStruct({"authTransformId": SimStruct({"authType": SimTypeInt(signed=False, label="IPSEC_AUTH_TYPE"), "authConfig": SimTypeChar(label="Byte")}, name="IPSEC_AUTH_TRANSFORM_ID0", pack=False, align=None), "cryptoModuleId": SimTypePointer(SimTypeBottom(label="Guid"), offset=0)}, name="IPSEC_AUTH_TRANSFORM0", pack=False, align=None), "cipherTransform": SimStruct({"cipherTransformId": SimStruct({"cipherType": SimTypeInt(signed=False, label="IPSEC_CIPHER_TYPE"), "cipherConfig": SimTypeChar(label="Byte")}, name="IPSEC_CIPHER_TRANSFORM_ID0", pack=False, align=None), "cryptoModuleId": SimTypePointer(SimTypeBottom(label="Guid"), offset=0)}, name="IPSEC_CIPHER_TRANSFORM0", pack=False, align=None)}, name="IPSEC_AUTH_AND_CIPHER_TRANSFORM0", pack=False, align=None), offset=0), "espAuthFwTransform": SimTypePointer(SimStruct({"authTransformId": SimStruct({"authType": SimTypeInt(signed=False, label="IPSEC_AUTH_TYPE"), "authConfig": SimTypeChar(label="Byte")}, name="IPSEC_AUTH_TRANSFORM_ID0", pack=False, align=None), "cryptoModuleId": SimTypePointer(SimTypeBottom(label="Guid"), offset=0)}, name="IPSEC_AUTH_TRANSFORM0", pack=False, align=None), offset=0)}, name="<anon>", label="None")}, name="IPSEC_SA_TRANSFORM0", pack=False, align=None), offset=0), "pfsGroup": SimTypeInt(signed=False, label="IPSEC_PFS_GROUP")}, name="IPSEC_PROPOSAL0", pack=False, align=None), offset=0), "tunnelEndpoints": SimStruct({"ipVersion": SimTypeInt(signed=False, label="FWP_IP_VERSION"), "Anonymous1": SimUnion({"localV4Address": SimTypeInt(signed=False, label="UInt32"), "localV6Address": SimTypeFixedSizeArray(SimTypeChar(label="Byte"), 16)}, name="<anon>", label="None"), "Anonymous2": SimUnion({"remoteV4Address": SimTypeInt(signed=False, label="UInt32"), "remoteV6Address": SimTypeFixedSizeArray(SimTypeChar(label="Byte"), 16)}, name="<anon>", label="None"), "localIfLuid": SimTypeLongLong(signed=False, label="UInt64"), "remoteFqdn": SimTypePointer(SimTypeChar(label="Char"), offset=0), "numAddresses": SimTypeInt(signed=False, label="UInt32"), "remoteAddresses": SimTypePointer(SimStruct({"ipVersion": SimTypeInt(signed=False, label="FWP_IP_VERSION"), "Anonymous": SimUnion({"v4Address": SimTypeInt(signed=False, label="UInt32"), "v6Address": SimTypeFixedSizeArray(SimTypeChar(label="Byte"), 16)}, name="<anon>", label="None")}, name="IPSEC_TUNNEL_ENDPOINT0", pack=False, align=None), offset=0)}, name="IPSEC_TUNNEL_ENDPOINTS2", pack=False, align=None), "saIdleTimeout": SimStruct({"idleTimeoutSeconds": SimTypeInt(signed=False, label="UInt32"), "idleTimeoutSecondsFailOver": SimTypeInt(signed=False, label="UInt32")}, name="IPSEC_SA_IDLE_TIMEOUT0", pack=False, align=None), "emPolicy": SimTypePointer(SimStruct({"numAuthenticationMethods": SimTypeInt(signed=False, label="UInt32"), "authenticationMethods": SimTypePointer(SimStruct({"authenticationMethodType": SimTypeInt(signed=False, label="IKEEXT_AUTHENTICATION_METHOD_TYPE"), "Anonymous": SimUnion({"presharedKeyAuthentication": SimStruct({"presharedKey": SimStruct({"size": SimTypeInt(signed=False, label="UInt32"), "data": SimTypePointer(SimTypeChar(label="Byte"), offset=0)}, name="FWP_BYTE_BLOB", pack=False, align=None), "flags": SimTypeInt(signed=False, label="IKEEXT_PRESHARED_KEY_AUTHENTICATION_FLAGS")}, name="IKEEXT_PRESHARED_KEY_AUTHENTICATION1", pack=False, align=None), "certificateAuthentication": SimStruct({"inboundConfigType": SimTypeInt(signed=False, label="IKEEXT_CERT_CONFIG_TYPE"), "Anonymous1": SimUnion({"Anonymous1": SimStruct({"inboundRootArraySize": SimTypeInt(signed=False, label="UInt32"), "inboundRootCriteria": SimTypePointer(SimStruct({"certData": SimStruct({"size": SimTypeInt(signed=False, label="UInt32"), "data": SimTypePointer(SimTypeChar(label="Byte"), offset=0)}, name="FWP_BYTE_BLOB", pack=False, align=None), "certHash": SimStruct({"size": SimTypeInt(signed=False, label="UInt32"), "data": SimTypePointer(SimTypeChar(label="Byte"), offset=0)}, name="FWP_BYTE_BLOB", pack=False, align=None), "eku": SimTypePointer(SimStruct({"numEku": SimTypeInt(signed=False, label="UInt32"), "eku": SimTypePointer(SimTypePointer(SimTypeChar(label="Byte"), offset=0), offset=0)}, name="IKEEXT_CERT_EKUS0", pack=False, align=None), offset=0), "name": SimTypePointer(SimStruct({"nameType": SimTypeInt(signed=False, label="IKEEXT_CERT_CRITERIA_NAME_TYPE"), "certName": SimTypePointer(SimTypeChar(label="Char"), offset=0)}, name="IKEEXT_CERT_NAME0", pack=False, align=None), offset=0), "flags": SimTypeInt(signed=False, label="UInt32")}, name="IKEEXT_CERTIFICATE_CRITERIA0", pack=False, align=None), offset=0)}, name="_Anonymous1_e__Struct", pack=False, align=None), "Anonymous2": SimStruct({"inboundEnterpriseStoreArraySize": SimTypeInt(signed=False, label="UInt32"), "inboundEnterpriseStoreCriteria": SimTypePointer(SimStruct({"certData": SimStruct({"size": SimTypeInt(signed=False, label="UInt32"), "data": SimTypePointer(SimTypeChar(label="Byte"), offset=0)}, name="FWP_BYTE_BLOB", pack=False, align=None), "certHash": SimStruct({"size": SimTypeInt(signed=False, label="UInt32"), "data": SimTypePointer(SimTypeChar(label="Byte"), offset=0)}, name="FWP_BYTE_BLOB", pack=False, align=None), "eku": SimTypePointer(SimStruct({"numEku": SimTypeInt(signed=False, label="UInt32"), "eku": SimTypePointer(SimTypePointer(SimTypeChar(label="Byte"), offset=0), offset=0)}, name="IKEEXT_CERT_EKUS0", pack=False, align=None), offset=0), "name": SimTypePointer(SimStruct({"nameType": SimTypeInt(signed=False, label="IKEEXT_CERT_CRITERIA_NAME_TYPE"), "certName": SimTypePointer(SimTypeChar(label="Char"), offset=0)}, name="IKEEXT_CERT_NAME0", pack=False, align=None), offset=0), "flags": SimTypeInt(signed=False, label="UInt32")}, name="IKEEXT_CERTIFICATE_CRITERIA0", pack=False, align=None), offset=0)}, name="_Anonymous2_e__Struct", pack=False, align=None), "Anonymous3": SimStruct({"inboundRootStoreArraySize": SimTypeInt(signed=False, label="UInt32"), "inboundTrustedRootStoreCriteria": SimTypePointer(SimStruct({"certData": SimStruct({"size": SimTypeInt(signed=False, label="UInt32"), "data": SimTypePointer(SimTypeChar(label="Byte"), offset=0)}, name="FWP_BYTE_BLOB", pack=False, align=None), "certHash": SimStruct({"size": SimTypeInt(signed=False, label="UInt32"), "data": SimTypePointer(SimTypeChar(label="Byte"), offset=0)}, name="FWP_BYTE_BLOB", pack=False, align=None), "eku": SimTypePointer(SimStruct({"numEku": SimTypeInt(signed=False, label="UInt32"), "eku": SimTypePointer(SimTypePointer(SimTypeChar(label="Byte"), offset=0), offset=0)}, name="IKEEXT_CERT_EKUS0", pack=False, align=None), offset=0), "name": SimTypePointer(SimStruct({"nameType": SimTypeInt(signed=False, label="IKEEXT_CERT_CRITERIA_NAME_TYPE"), "certName": SimTypePointer(SimTypeChar(label="Char"), offset=0)}, name="IKEEXT_CERT_NAME0", pack=False, align=None), offset=0), "flags": SimTypeInt(signed=False, label="UInt32")}, name="IKEEXT_CERTIFICATE_CRITERIA0", pack=False, align=None), offset=0)}, name="_Anonymous3_e__Struct", pack=False, align=None)}, name="<anon>", label="None"), "outboundConfigType": SimTypeInt(signed=False, label="IKEEXT_CERT_CONFIG_TYPE"), "Anonymous2": SimUnion({"Anonymous1": SimStruct({"outboundRootArraySize": SimTypeInt(signed=False, label="UInt32"), "outboundRootCriteria": SimTypePointer(SimStruct({"certData": SimStruct({"size": SimTypeInt(signed=False, label="UInt32"), "data": SimTypePointer(SimTypeChar(label="Byte"), offset=0)}, name="FWP_BYTE_BLOB", pack=False, align=None), "certHash": SimStruct({"size": SimTypeInt(signed=False, label="UInt32"), "data": SimTypePointer(SimTypeChar(label="Byte"), offset=0)}, name="FWP_BYTE_BLOB", pack=False, align=None), "eku": SimTypePointer(SimStruct({"numEku": SimTypeInt(signed=False, label="UInt32"), "eku": SimTypePointer(SimTypePointer(SimTypeChar(label="Byte"), offset=0), offset=0)}, name="IKEEXT_CERT_EKUS0", pack=False, align=None), offset=0), "name": SimTypePointer(SimStruct({"nameType": SimTypeInt(signed=False, label="IKEEXT_CERT_CRITERIA_NAME_TYPE"), "certName": SimTypePointer(SimTypeChar(label="Char"), offset=0)}, name="IKEEXT_CERT_NAME0", pack=False, align=None), offset=0), "flags": SimTypeInt(signed=False, label="UInt32")}, name="IKEEXT_CERTIFICATE_CRITERIA0", pack=False, align=None), offset=0)}, name="_Anonymous1_e__Struct", pack=False, align=None), "Anonymous2": SimStruct({"outboundEnterpriseStoreArraySize": SimTypeInt(signed=False, label="UInt32"), "outboundEnterpriseStoreCriteria": SimTypePointer(SimStruct({"certData": SimStruct({"size": SimTypeInt(signed=False, label="UInt32"), "data": SimTypePointer(SimTypeChar(label="Byte"), offset=0)}, name="FWP_BYTE_BLOB", pack=False, align=None), "certHash": SimStruct({"size": SimTypeInt(signed=False, label="UInt32"), "data": SimTypePointer(SimTypeChar(label="Byte"), offset=0)}, name="FWP_BYTE_BLOB", pack=False, align=None), "eku": SimTypePointer(SimStruct({"numEku": SimTypeInt(signed=False, label="UInt32"), "eku": SimTypePointer(SimTypePointer(SimTypeChar(label="Byte"), offset=0), offset=0)}, name="IKEEXT_CERT_EKUS0", pack=False, align=None), offset=0), "name": SimTypePointer(SimStruct({"nameType": SimTypeInt(signed=False, label="IKEEXT_CERT_CRITERIA_NAME_TYPE"), "certName": SimTypePointer(SimTypeChar(label="Char"), offset=0)}, name="IKEEXT_CERT_NAME0", pack=False, align=None), offset=0), "flags": SimTypeInt(signed=False, label="UInt32")}, name="IKEEXT_CERTIFICATE_CRITERIA0", pack=False, align=None), offset=0)}, name="_Anonymous2_e__Struct", pack=False, align=None), "Anonymous3": SimStruct({"outboundRootStoreArraySize": SimTypeInt(signed=False, label="UInt32"), "outboundTrustedRootStoreCriteria": SimTypePointer(SimStruct({"certData": SimStruct({"size": SimTypeInt(signed=False, label="UInt32"), "data": SimTypePointer(SimTypeChar(label="Byte"), offset=0)}, name="FWP_BYTE_BLOB", pack=False, align=None), "certHash": SimStruct({"size": SimTypeInt(signed=False, label="UInt32"), "data": SimTypePointer(SimTypeChar(label="Byte"), offset=0)}, name="FWP_BYTE_BLOB", pack=False, align=None), "eku": SimTypePointer(SimStruct({"numEku": SimTypeInt(signed=False, label="UInt32"), "eku": SimTypePointer(SimTypePointer(SimTypeChar(label="Byte"), offset=0), offset=0)}, name="IKEEXT_CERT_EKUS0", pack=False, align=None), offset=0), "name": SimTypePointer(SimStruct({"nameType": SimTypeInt(signed=False, label="IKEEXT_CERT_CRITERIA_NAME_TYPE"), "certName": SimTypePointer(SimTypeChar(label="Char"), offset=0)}, name="IKEEXT_CERT_NAME0", pack=False, align=None), offset=0), "flags": SimTypeInt(signed=False, label="UInt32")}, name="IKEEXT_CERTIFICATE_CRITERIA0", pack=False, align=None), offset=0)}, name="_Anonymous3_e__Struct", pack=False, align=None)}, name="<anon>", label="None"), "flags": SimTypeInt(signed=False, label="IKEEXT_CERT_AUTH"), "localCertLocationUrl": SimStruct({"size": SimTypeInt(signed=False, label="UInt32"), "data": SimTypePointer(SimTypeChar(label="Byte"), offset=0)}, name="FWP_BYTE_BLOB", pack=False, align=None)}, name="IKEEXT_CERTIFICATE_AUTHENTICATION2", pack=False, align=None), "kerberosAuthentication": SimStruct({"flags": SimTypeInt(signed=False, label="IKEEXT_KERBEROS_AUTHENTICATION_FLAGS"), "proxyServer": SimTypePointer(SimTypeChar(label="Char"), offset=0)}, name="IKEEXT_KERBEROS_AUTHENTICATION1", pack=False, align=None), "reservedAuthentication": SimStruct({"flags": SimTypeInt(signed=False, label="IKEEXT_RESERVED_AUTHENTICATION_FLAGS")}, name="IKEEXT_RESERVED_AUTHENTICATION0", pack=False, align=None), "ntlmV2Authentication": SimStruct({"flags": SimTypeInt(signed=False, label="UInt32")}, name="IKEEXT_NTLM_V2_AUTHENTICATION0", pack=False, align=None), "sslAuthentication": SimStruct({"inboundConfigType": SimTypeInt(signed=False, label="IKEEXT_CERT_CONFIG_TYPE"), "Anonymous1": SimUnion({"Anonymous1": SimStruct({"inboundRootArraySize": SimTypeInt(signed=False, label="UInt32"), "inboundRootCriteria": SimTypePointer(SimStruct({"certData": SimStruct({"size": SimTypeInt(signed=False, label="UInt32"), "data": SimTypePointer(SimTypeChar(label="Byte"), offset=0)}, name="FWP_BYTE_BLOB", pack=False, align=None), "certHash": SimStruct({"size": SimTypeInt(signed=False, label="UInt32"), "data": SimTypePointer(SimTypeChar(label="Byte"), offset=0)}, name="FWP_BYTE_BLOB", pack=False, align=None), "eku": SimTypePointer(SimStruct({"numEku": SimTypeInt(signed=False, label="UInt32"), "eku": SimTypePointer(SimTypePointer(SimTypeChar(label="Byte"), offset=0), offset=0)}, name="IKEEXT_CERT_EKUS0", pack=False, align=None), offset=0), "name": SimTypePointer(SimStruct({"nameType": SimTypeInt(signed=False, label="IKEEXT_CERT_CRITERIA_NAME_TYPE"), "certName": SimTypePointer(SimTypeChar(label="Char"), offset=0)}, name="IKEEXT_CERT_NAME0", pack=False, align=None), offset=0), "flags": SimTypeInt(signed=False, label="UInt32")}, name="IKEEXT_CERTIFICATE_CRITERIA0", pack=False, align=None), offset=0)}, name="_Anonymous1_e__Struct", pack=False, align=None), "Anonymous2": SimStruct({"inboundEnterpriseStoreArraySize": SimTypeInt(signed=False, label="UInt32"), "inboundEnterpriseStoreCriteria": SimTypePointer(SimStruct({"certData": SimStruct({"size": SimTypeInt(signed=False, label="UInt32"), "data": SimTypePointer(SimTypeChar(label="Byte"), offset=0)}, name="FWP_BYTE_BLOB", pack=False, align=None), "certHash": SimStruct({"size": SimTypeInt(signed=False, label="UInt32"), "data": SimTypePointer(SimTypeChar(label="Byte"), offset=0)}, name="FWP_BYTE_BLOB", pack=False, align=None), "eku": SimTypePointer(SimStruct({"numEku": SimTypeInt(signed=False, label="UInt32"), "eku": SimTypePointer(SimTypePointer(SimTypeChar(label="Byte"), offset=0), offset=0)}, name="IKEEXT_CERT_EKUS0", pack=False, align=None), offset=0), "name": SimTypePointer(SimStruct({"nameType": SimTypeInt(signed=False, label="IKEEXT_CERT_CRITERIA_NAME_TYPE"), "certName": SimTypePointer(SimTypeChar(label="Char"), offset=0)}, name="IKEEXT_CERT_NAME0", pack=False, align=None), offset=0), "flags": SimTypeInt(signed=False, label="UInt32")}, name="IKEEXT_CERTIFICATE_CRITERIA0", pack=False, align=None), offset=0)}, name="_Anonymous2_e__Struct", pack=False, align=None), "Anonymous3": SimStruct({"inboundRootStoreArraySize": SimTypeInt(signed=False, label="UInt32"), "inboundTrustedRootStoreCriteria": SimTypePointer(SimStruct({"certData": SimStruct({"size": SimTypeInt(signed=False, label="UInt32"), "data": SimTypePointer(SimTypeChar(label="Byte"), offset=0)}, name="FWP_BYTE_BLOB", pack=False, align=None), "certHash": SimStruct({"size": SimTypeInt(signed=False, label="UInt32"), "data": SimTypePointer(SimTypeChar(label="Byte"), offset=0)}, name="FWP_BYTE_BLOB", pack=False, align=None), "eku": SimTypePointer(SimStruct({"numEku": SimTypeInt(signed=False, label="UInt32"), "eku": SimTypePointer(SimTypePointer(SimTypeChar(label="Byte"), offset=0), offset=0)}, name="IKEEXT_CERT_EKUS0", pack=False, align=None), offset=0), "name": SimTypePointer(SimStruct({"nameType": SimTypeInt(signed=False, label="IKEEXT_CERT_CRITERIA_NAME_TYPE"), "certName": SimTypePointer(SimTypeChar(label="Char"), offset=0)}, name="IKEEXT_CERT_NAME0", pack=False, align=None), offset=0), "flags": SimTypeInt(signed=False, label="UInt32")}, name="IKEEXT_CERTIFICATE_CRITERIA0", pack=False, align=None), offset=0)}, name="_Anonymous3_e__Struct", pack=False, align=None)}, name="<anon>", label="None"), "outboundConfigType": SimTypeInt(signed=False, label="IKEEXT_CERT_CONFIG_TYPE"), "Anonymous2": SimUnion({"Anonymous1": SimStruct({"outboundRootArraySize": SimTypeInt(signed=False, label="UInt32"), "outboundRootCriteria": SimTypePointer(SimStruct({"certData": SimStruct({"size": SimTypeInt(signed=False, label="UInt32"), "data": SimTypePointer(SimTypeChar(label="Byte"), offset=0)}, name="FWP_BYTE_BLOB", pack=False, align=None), "certHash": SimStruct({"size": SimTypeInt(signed=False, label="UInt32"), "data": SimTypePointer(SimTypeChar(label="Byte"), offset=0)}, name="FWP_BYTE_BLOB", pack=False, align=None), "eku": SimTypePointer(SimStruct({"numEku": SimTypeInt(signed=False, label="UInt32"), "eku": SimTypePointer(SimTypePointer(SimTypeChar(label="Byte"), offset=0), offset=0)}, name="IKEEXT_CERT_EKUS0", pack=False, align=None), offset=0), "name": SimTypePointer(SimStruct({"nameType": SimTypeInt(signed=False, label="IKEEXT_CERT_CRITERIA_NAME_TYPE"), "certName": SimTypePointer(SimTypeChar(label="Char"), offset=0)}, name="IKEEXT_CERT_NAME0", pack=False, align=None), offset=0), "flags": SimTypeInt(signed=False, label="UInt32")}, name="IKEEXT_CERTIFICATE_CRITERIA0", pack=False, align=None), offset=0)}, name="_Anonymous1_e__Struct", pack=False, align=None), "Anonymous2": SimStruct({"outboundEnterpriseStoreArraySize": SimTypeInt(signed=False, label="UInt32"), "outboundEnterpriseStoreCriteria": SimTypePointer(SimStruct({"certData": SimStruct({"size": SimTypeInt(signed=False, label="UInt32"), "data": SimTypePointer(SimTypeChar(label="Byte"), offset=0)}, name="FWP_BYTE_BLOB", pack=False, align=None), "certHash": SimStruct({"size": SimTypeInt(signed=False, label="UInt32"), "data": SimTypePointer(SimTypeChar(label="Byte"), offset=0)}, name="FWP_BYTE_BLOB", pack=False, align=None), "eku": SimTypePointer(SimStruct({"numEku": SimTypeInt(signed=False, label="UInt32"), "eku": SimTypePointer(SimTypePointer(SimTypeChar(label="Byte"), offset=0), offset=0)}, name="IKEEXT_CERT_EKUS0", pack=False, align=None), offset=0), "name": SimTypePointer(SimStruct({"nameType": SimTypeInt(signed=False, label="IKEEXT_CERT_CRITERIA_NAME_TYPE"), "certName": SimTypePointer(SimTypeChar(label="Char"), offset=0)}, name="IKEEXT_CERT_NAME0", pack=False, align=None), offset=0), "flags": SimTypeInt(signed=False, label="UInt32")}, name="IKEEXT_CERTIFICATE_CRITERIA0", pack=False, align=None), offset=0)}, name="_Anonymous2_e__Struct", pack=False, align=None), "Anonymous3": SimStruct({"outboundRootStoreArraySize": SimTypeInt(signed=False, label="UInt32"), "outboundTrustedRootStoreCriteria": SimTypePointer(SimStruct({"certData": SimStruct({"size": SimTypeInt(signed=False, label="UInt32"), "data": SimTypePointer(SimTypeChar(label="Byte"), offset=0)}, name="FWP_BYTE_BLOB", pack=False, align=None), "certHash": SimStruct({"size": SimTypeInt(signed=False, label="UInt32"), "data": SimTypePointer(SimTypeChar(label="Byte"), offset=0)}, name="FWP_BYTE_BLOB", pack=False, align=None), "eku": SimTypePointer(SimStruct({"numEku": SimTypeInt(signed=False, label="UInt32"), "eku": SimTypePointer(SimTypePointer(SimTypeChar(label="Byte"), offset=0), offset=0)}, name="IKEEXT_CERT_EKUS0", pack=False, align=None), offset=0), "name": SimTypePointer(SimStruct({"nameType": SimTypeInt(signed=False, label="IKEEXT_CERT_CRITERIA_NAME_TYPE"), "certName": SimTypePointer(SimTypeChar(label="Char"), offset=0)}, name="IKEEXT_CERT_NAME0", pack=False, align=None), offset=0), "flags": SimTypeInt(signed=False, label="UInt32")}, name="IKEEXT_CERTIFICATE_CRITERIA0", pack=False, align=None), offset=0)}, name="_Anonymous3_e__Struct", pack=False, align=None)}, name="<anon>", label="None"), "flags": SimTypeInt(signed=False, label="IKEEXT_CERT_AUTH"), "localCertLocationUrl": SimStruct({"size": SimTypeInt(signed=False, label="UInt32"), "data": SimTypePointer(SimTypeChar(label="Byte"), offset=0)}, name="FWP_BYTE_BLOB", pack=False, align=None)}, name="IKEEXT_CERTIFICATE_AUTHENTICATION2", pack=False, align=None), "cgaAuthentication": SimStruct({"keyContainerName": SimTypePointer(SimTypeChar(label="Char"), offset=0), "cspName": SimTypePointer(SimTypeChar(label="Char"), offset=0), "cspType": SimTypeInt(signed=False, label="UInt32"), "cgaModifier": SimStruct({"byteArray16": SimTypeFixedSizeArray(SimTypeChar(label="Byte"), 16)}, name="FWP_BYTE_ARRAY16", pack=False, align=None), "cgaCollisionCount": SimTypeChar(label="Byte")}, name="IKEEXT_IPV6_CGA_AUTHENTICATION0", pack=False, align=None), "eapAuthentication": SimStruct({"flags": SimTypeInt(signed=False, label="IKEEXT_EAP_AUTHENTICATION_FLAGS")}, name="IKEEXT_EAP_AUTHENTICATION0", pack=False, align=None)}, name="<anon>", label="None")}, name="IKEEXT_AUTHENTICATION_METHOD2", pack=False, align=None), offset=0), "initiatorImpersonationType": SimTypeInt(signed=False, label="IKEEXT_AUTHENTICATION_IMPERSONATION_TYPE")}, name="IKEEXT_EM_POLICY2", pack=False, align=None), offset=0), "fwdPathSaLifetime": SimTypeInt(signed=False, label="UInt32")}, name="IPSEC_TUNNEL_POLICY2", pack=False, align=None), offset=0), "ikeMmPolicy": SimTypePointer(SimStruct({"softExpirationTime": SimTypeInt(signed=False, label="UInt32"), "numAuthenticationMethods": SimTypeInt(signed=False, label="UInt32"), "authenticationMethods": SimTypePointer(SimStruct({"authenticationMethodType": SimTypeInt(signed=False, label="IKEEXT_AUTHENTICATION_METHOD_TYPE"), "Anonymous": SimUnion({"presharedKeyAuthentication": SimStruct({"presharedKey": SimStruct({"size": SimTypeInt(signed=False, label="UInt32"), "data": SimTypePointer(SimTypeChar(label="Byte"), offset=0)}, name="FWP_BYTE_BLOB", pack=False, align=None), "flags": SimTypeInt(signed=False, label="IKEEXT_PRESHARED_KEY_AUTHENTICATION_FLAGS")}, name="IKEEXT_PRESHARED_KEY_AUTHENTICATION1", pack=False, align=None), "certificateAuthentication": SimStruct({"inboundConfigType": SimTypeInt(signed=False, label="IKEEXT_CERT_CONFIG_TYPE"), "Anonymous1": SimUnion({"Anonymous1": SimStruct({"inboundRootArraySize": SimTypeInt(signed=False, label="UInt32"), "inboundRootCriteria": SimTypePointer(SimStruct({"certData": SimStruct({"size": SimTypeInt(signed=False, label="UInt32"), "data": SimTypePointer(SimTypeChar(label="Byte"), offset=0)}, name="FWP_BYTE_BLOB", pack=False, align=None), "certHash": SimStruct({"size": SimTypeInt(signed=False, label="UInt32"), "data": SimTypePointer(SimTypeChar(label="Byte"), offset=0)}, name="FWP_BYTE_BLOB", pack=False, align=None), "eku": SimTypePointer(SimStruct({"numEku": SimTypeInt(signed=False, label="UInt32"), "eku": SimTypePointer(SimTypePointer(SimTypeChar(label="Byte"), offset=0), offset=0)}, name="IKEEXT_CERT_EKUS0", pack=False, align=None), offset=0), "name": SimTypePointer(SimStruct({"nameType": SimTypeInt(signed=False, label="IKEEXT_CERT_CRITERIA_NAME_TYPE"), "certName": SimTypePointer(SimTypeChar(label="Char"), offset=0)}, name="IKEEXT_CERT_NAME0", pack=False, align=None), offset=0), "flags": SimTypeInt(signed=False, label="UInt32")}, name="IKEEXT_CERTIFICATE_CRITERIA0", pack=False, align=None), offset=0)}, name="_Anonymous1_e__Struct", pack=False, align=None), "Anonymous2": SimStruct({"inboundEnterpriseStoreArraySize": SimTypeInt(signed=False, label="UInt32"), "inboundEnterpriseStoreCriteria": SimTypePointer(SimStruct({"certData": SimStruct({"size": SimTypeInt(signed=False, label="UInt32"), "data": SimTypePointer(SimTypeChar(label="Byte"), offset=0)}, name="FWP_BYTE_BLOB", pack=False, align=None), "certHash": SimStruct({"size": SimTypeInt(signed=False, label="UInt32"), "data": SimTypePointer(SimTypeChar(label="Byte"), offset=0)}, name="FWP_BYTE_BLOB", pack=False, align=None), "eku": SimTypePointer(SimStruct({"numEku": SimTypeInt(signed=False, label="UInt32"), "eku": SimTypePointer(SimTypePointer(SimTypeChar(label="Byte"), offset=0), offset=0)}, name="IKEEXT_CERT_EKUS0", pack=False, align=None), offset=0), "name": SimTypePointer(SimStruct({"nameType": SimTypeInt(signed=False, label="IKEEXT_CERT_CRITERIA_NAME_TYPE"), "certName": SimTypePointer(SimTypeChar(label="Char"), offset=0)}, name="IKEEXT_CERT_NAME0", pack=False, align=None), offset=0), "flags": SimTypeInt(signed=False, label="UInt32")}, name="IKEEXT_CERTIFICATE_CRITERIA0", pack=False, align=None), offset=0)}, name="_Anonymous2_e__Struct", pack=False, align=None), "Anonymous3": SimStruct({"inboundRootStoreArraySize": SimTypeInt(signed=False, label="UInt32"), "inboundTrustedRootStoreCriteria": SimTypePointer(SimStruct({"certData": SimStruct({"size": SimTypeInt(signed=False, label="UInt32"), "data": SimTypePointer(SimTypeChar(label="Byte"), offset=0)}, name="FWP_BYTE_BLOB", pack=False, align=None), "certHash": SimStruct({"size": SimTypeInt(signed=False, label="UInt32"), "data": SimTypePointer(SimTypeChar(label="Byte"), offset=0)}, name="FWP_BYTE_BLOB", pack=False, align=None), "eku": SimTypePointer(SimStruct({"numEku": SimTypeInt(signed=False, label="UInt32"), "eku": SimTypePointer(SimTypePointer(SimTypeChar(label="Byte"), offset=0), offset=0)}, name="IKEEXT_CERT_EKUS0", pack=False, align=None), offset=0), "name": SimTypePointer(SimStruct({"nameType": SimTypeInt(signed=False, label="IKEEXT_CERT_CRITERIA_NAME_TYPE"), "certName": SimTypePointer(SimTypeChar(label="Char"), offset=0)}, name="IKEEXT_CERT_NAME0", pack=False, align=None), offset=0), "flags": SimTypeInt(signed=False, label="UInt32")}, name="IKEEXT_CERTIFICATE_CRITERIA0", pack=False, align=None), offset=0)}, name="_Anonymous3_e__Struct", pack=False, align=None)}, name="<anon>", label="None"), "outboundConfigType": SimTypeInt(signed=False, label="IKEEXT_CERT_CONFIG_TYPE"), "Anonymous2": SimUnion({"Anonymous1": SimStruct({"outboundRootArraySize": SimTypeInt(signed=False, label="UInt32"), "outboundRootCriteria": SimTypePointer(SimStruct({"certData": SimStruct({"size": SimTypeInt(signed=False, label="UInt32"), "data": SimTypePointer(SimTypeChar(label="Byte"), offset=0)}, name="FWP_BYTE_BLOB", pack=False, align=None), "certHash": SimStruct({"size": SimTypeInt(signed=False, label="UInt32"), "data": SimTypePointer(SimTypeChar(label="Byte"), offset=0)}, name="FWP_BYTE_BLOB", pack=False, align=None), "eku": SimTypePointer(SimStruct({"numEku": SimTypeInt(signed=False, label="UInt32"), "eku": SimTypePointer(SimTypePointer(SimTypeChar(label="Byte"), offset=0), offset=0)}, name="IKEEXT_CERT_EKUS0", pack=False, align=None), offset=0), "name": SimTypePointer(SimStruct({"nameType": SimTypeInt(signed=False, label="IKEEXT_CERT_CRITERIA_NAME_TYPE"), "certName": SimTypePointer(SimTypeChar(label="Char"), offset=0)}, name="IKEEXT_CERT_NAME0", pack=False, align=None), offset=0), "flags": SimTypeInt(signed=False, label="UInt32")}, name="IKEEXT_CERTIFICATE_CRITERIA0", pack=False, align=None), offset=0)}, name="_Anonymous1_e__Struct", pack=False, align=None), "Anonymous2": SimStruct({"outboundEnterpriseStoreArraySize": SimTypeInt(signed=False, label="UInt32"), "outboundEnterpriseStoreCriteria": SimTypePointer(SimStruct({"certData": SimStruct({"size": SimTypeInt(signed=False, label="UInt32"), "data": SimTypePointer(SimTypeChar(label="Byte"), offset=0)}, name="FWP_BYTE_BLOB", pack=False, align=None), "certHash": SimStruct({"size": SimTypeInt(signed=False, label="UInt32"), "data": SimTypePointer(SimTypeChar(label="Byte"), offset=0)}, name="FWP_BYTE_BLOB", pack=False, align=None), "eku": SimTypePointer(SimStruct({"numEku": SimTypeInt(signed=False, label="UInt32"), "eku": SimTypePointer(SimTypePointer(SimTypeChar(label="Byte"), offset=0), offset=0)}, name="IKEEXT_CERT_EKUS0", pack=False, align=None), offset=0), "name": SimTypePointer(SimStruct({"nameType": SimTypeInt(signed=False, label="IKEEXT_CERT_CRITERIA_NAME_TYPE"), "certName": SimTypePointer(SimTypeChar(label="Char"), offset=0)}, name="IKEEXT_CERT_NAME0", pack=False, align=None), offset=0), "flags": SimTypeInt(signed=False, label="UInt32")}, name="IKEEXT_CERTIFICATE_CRITERIA0", pack=False, align=None), offset=0)}, name="_Anonymous2_e__Struct", pack=False, align=None), "Anonymous3": SimStruct({"outboundRootStoreArraySize": SimTypeInt(signed=False, label="UInt32"), "outboundTrustedRootStoreCriteria": SimTypePointer(SimStruct({"certData": SimStruct({"size": SimTypeInt(signed=False, label="UInt32"), "data": SimTypePointer(SimTypeChar(label="Byte"), offset=0)}, name="FWP_BYTE_BLOB", pack=False, align=None), "certHash": SimStruct({"size": SimTypeInt(signed=False, label="UInt32"), "data": SimTypePointer(SimTypeChar(label="Byte"), offset=0)}, name="FWP_BYTE_BLOB", pack=False, align=None), "eku": SimTypePointer(SimStruct({"numEku": SimTypeInt(signed=False, label="UInt32"), "eku": SimTypePointer(SimTypePointer(SimTypeChar(label="Byte"), offset=0), offset=0)}, name="IKEEXT_CERT_EKUS0", pack=False, align=None), offset=0), "name": SimTypePointer(SimStruct({"nameType": SimTypeInt(signed=False, label="IKEEXT_CERT_CRITERIA_NAME_TYPE"), "certName": SimTypePointer(SimTypeChar(label="Char"), offset=0)}, name="IKEEXT_CERT_NAME0", pack=False, align=None), offset=0), "flags": SimTypeInt(signed=False, label="UInt32")}, name="IKEEXT_CERTIFICATE_CRITERIA0", pack=False, align=None), offset=0)}, name="_Anonymous3_e__Struct", pack=False, align=None)}, name="<anon>", label="None"), "flags": SimTypeInt(signed=False, label="IKEEXT_CERT_AUTH"), "localCertLocationUrl": SimStruct({"size": SimTypeInt(signed=False, label="UInt32"), "data": SimTypePointer(SimTypeChar(label="Byte"), offset=0)}, name="FWP_BYTE_BLOB", pack=False, align=None)}, name="IKEEXT_CERTIFICATE_AUTHENTICATION2", pack=False, align=None), "kerberosAuthentication": SimStruct({"flags": SimTypeInt(signed=False, label="IKEEXT_KERBEROS_AUTHENTICATION_FLAGS"), "proxyServer": SimTypePointer(SimTypeChar(label="Char"), offset=0)}, name="IKEEXT_KERBEROS_AUTHENTICATION1", pack=False, align=None), "reservedAuthentication": SimStruct({"flags": SimTypeInt(signed=False, label="IKEEXT_RESERVED_AUTHENTICATION_FLAGS")}, name="IKEEXT_RESERVED_AUTHENTICATION0", pack=False, align=None), "ntlmV2Authentication": SimStruct({"flags": SimTypeInt(signed=False, label="UInt32")}, name="IKEEXT_NTLM_V2_AUTHENTICATION0", pack=False, align=None), "sslAuthentication": SimStruct({"inboundConfigType": SimTypeInt(signed=False, label="IKEEXT_CERT_CONFIG_TYPE"), "Anonymous1": SimUnion({"Anonymous1": SimStruct({"inboundRootArraySize": SimTypeInt(signed=False, label="UInt32"), "inboundRootCriteria": SimTypePointer(SimStruct({"certData": SimStruct({"size": SimTypeInt(signed=False, label="UInt32"), "data": SimTypePointer(SimTypeChar(label="Byte"), offset=0)}, name="FWP_BYTE_BLOB", pack=False, align=None), "certHash": SimStruct({"size": SimTypeInt(signed=False, label="UInt32"), "data": SimTypePointer(SimTypeChar(label="Byte"), offset=0)}, name="FWP_BYTE_BLOB", pack=False, align=None), "eku": SimTypePointer(SimStruct({"numEku": SimTypeInt(signed=False, label="UInt32"), "eku": SimTypePointer(SimTypePointer(SimTypeChar(label="Byte"), offset=0), offset=0)}, name="IKEEXT_CERT_EKUS0", pack=False, align=None), offset=0), "name": SimTypePointer(SimStruct({"nameType": SimTypeInt(signed=False, label="IKEEXT_CERT_CRITERIA_NAME_TYPE"), "certName": SimTypePointer(SimTypeChar(label="Char"), offset=0)}, name="IKEEXT_CERT_NAME0", pack=False, align=None), offset=0), "flags": SimTypeInt(signed=False, label="UInt32")}, name="IKEEXT_CERTIFICATE_CRITERIA0", pack=False, align=None), offset=0)}, name="_Anonymous1_e__Struct", pack=False, align=None), "Anonymous2": SimStruct({"inboundEnterpriseStoreArraySize": SimTypeInt(signed=False, label="UInt32"), "inboundEnterpriseStoreCriteria": SimTypePointer(SimStruct({"certData": SimStruct({"size": SimTypeInt(signed=False, label="UInt32"), "data": SimTypePointer(SimTypeChar(label="Byte"), offset=0)}, name="FWP_BYTE_BLOB", pack=False, align=None), "certHash": SimStruct({"size": SimTypeInt(signed=False, label="UInt32"), "data": SimTypePointer(SimTypeChar(label="Byte"), offset=0)}, name="FWP_BYTE_BLOB", pack=False, align=None), "eku": SimTypePointer(SimStruct({"numEku": SimTypeInt(signed=False, label="UInt32"), "eku": SimTypePointer(SimTypePointer(SimTypeChar(label="Byte"), offset=0), offset=0)}, name="IKEEXT_CERT_EKUS0", pack=False, align=None), offset=0), "name": SimTypePointer(SimStruct({"nameType": SimTypeInt(signed=False, label="IKEEXT_CERT_CRITERIA_NAME_TYPE"), "certName": SimTypePointer(SimTypeChar(label="Char"), offset=0)}, name="IKEEXT_CERT_NAME0", pack=False, align=None), offset=0), "flags": SimTypeInt(signed=False, label="UInt32")}, name="IKEEXT_CERTIFICATE_CRITERIA0", pack=False, align=None), offset=0)}, name="_Anonymous2_e__Struct", pack=False, align=None), "Anonymous3": SimStruct({"inboundRootStoreArraySize": SimTypeInt(signed=False, label="UInt32"), "inboundTrustedRootStoreCriteria": SimTypePointer(SimStruct({"certData": SimStruct({"size": SimTypeInt(signed=False, label="UInt32"), "data": SimTypePointer(SimTypeChar(label="Byte"), offset=0)}, name="FWP_BYTE_BLOB", pack=False, align=None), "certHash": SimStruct({"size": SimTypeInt(signed=False, label="UInt32"), "data": SimTypePointer(SimTypeChar(label="Byte"), offset=0)}, name="FWP_BYTE_BLOB", pack=False, align=None), "eku": SimTypePointer(SimStruct({"numEku": SimTypeInt(signed=False, label="UInt32"), "eku": SimTypePointer(SimTypePointer(SimTypeChar(label="Byte"), offset=0), offset=0)}, name="IKEEXT_CERT_EKUS0", pack=False, align=None), offset=0), "name": SimTypePointer(SimStruct({"nameType": SimTypeInt(signed=False, label="IKEEXT_CERT_CRITERIA_NAME_TYPE"), "certName": SimTypePointer(SimTypeChar(label="Char"), offset=0)}, name="IKEEXT_CERT_NAME0", pack=False, align=None), offset=0), "flags": SimTypeInt(signed=False, label="UInt32")}, name="IKEEXT_CERTIFICATE_CRITERIA0", pack=False, align=None), offset=0)}, name="_Anonymous3_e__Struct", pack=False, align=None)}, name="<anon>", label="None"), "outboundConfigType": SimTypeInt(signed=False, label="IKEEXT_CERT_CONFIG_TYPE"), "Anonymous2": SimUnion({"Anonymous1": SimStruct({"outboundRootArraySize": SimTypeInt(signed=False, label="UInt32"), "outboundRootCriteria": SimTypePointer(SimStruct({"certData": SimStruct({"size": SimTypeInt(signed=False, label="UInt32"), "data": SimTypePointer(SimTypeChar(label="Byte"), offset=0)}, name="FWP_BYTE_BLOB", pack=False, align=None), "certHash": SimStruct({"size": SimTypeInt(signed=False, label="UInt32"), "data": SimTypePointer(SimTypeChar(label="Byte"), offset=0)}, name="FWP_BYTE_BLOB", pack=False, align=None), "eku": SimTypePointer(SimStruct({"numEku": SimTypeInt(signed=False, label="UInt32"), "eku": SimTypePointer(SimTypePointer(SimTypeChar(label="Byte"), offset=0), offset=0)}, name="IKEEXT_CERT_EKUS0", pack=False, align=None), offset=0), "name": SimTypePointer(SimStruct({"nameType": SimTypeInt(signed=False, label="IKEEXT_CERT_CRITERIA_NAME_TYPE"), "certName": SimTypePointer(SimTypeChar(label="Char"), offset=0)}, name="IKEEXT_CERT_NAME0", pack=False, align=None), offset=0), "flags": SimTypeInt(signed=False, label="UInt32")}, name="IKEEXT_CERTIFICATE_CRITERIA0", pack=False, align=None), offset=0)}, name="_Anonymous1_e__Struct", pack=False, align=None), "Anonymous2": SimStruct({"outboundEnterpriseStoreArraySize": SimTypeInt(signed=False, label="UInt32"), "outboundEnterpriseStoreCriteria": SimTypePointer(SimStruct({"certData": SimStruct({"size": SimTypeInt(signed=False, label="UInt32"), "data": SimTypePointer(SimTypeChar(label="Byte"), offset=0)}, name="FWP_BYTE_BLOB", pack=False, align=None), "certHash": SimStruct({"size": SimTypeInt(signed=False, label="UInt32"), "data": SimTypePointer(SimTypeChar(label="Byte"), offset=0)}, name="FWP_BYTE_BLOB", pack=False, align=None), "eku": SimTypePointer(SimStruct({"numEku": SimTypeInt(signed=False, label="UInt32"), "eku": SimTypePointer(SimTypePointer(SimTypeChar(label="Byte"), offset=0), offset=0)}, name="IKEEXT_CERT_EKUS0", pack=False, align=None), offset=0), "name": SimTypePointer(SimStruct({"nameType": SimTypeInt(signed=False, label="IKEEXT_CERT_CRITERIA_NAME_TYPE"), "certName": SimTypePointer(SimTypeChar(label="Char"), offset=0)}, name="IKEEXT_CERT_NAME0", pack=False, align=None), offset=0), "flags": SimTypeInt(signed=False, label="UInt32")}, name="IKEEXT_CERTIFICATE_CRITERIA0", pack=False, align=None), offset=0)}, name="_Anonymous2_e__Struct", pack=False, align=None), "Anonymous3": SimStruct({"outboundRootStoreArraySize": SimTypeInt(signed=False, label="UInt32"), "outboundTrustedRootStoreCriteria": SimTypePointer(SimStruct({"certData": SimStruct({"size": SimTypeInt(signed=False, label="UInt32"), "data": SimTypePointer(SimTypeChar(label="Byte"), offset=0)}, name="FWP_BYTE_BLOB", pack=False, align=None), "certHash": SimStruct({"size": SimTypeInt(signed=False, label="UInt32"), "data": SimTypePointer(SimTypeChar(label="Byte"), offset=0)}, name="FWP_BYTE_BLOB", pack=False, align=None), "eku": SimTypePointer(SimStruct({"numEku": SimTypeInt(signed=False, label="UInt32"), "eku": SimTypePointer(SimTypePointer(SimTypeChar(label="Byte"), offset=0), offset=0)}, name="IKEEXT_CERT_EKUS0", pack=False, align=None), offset=0), "name": SimTypePointer(SimStruct({"nameType": SimTypeInt(signed=False, label="IKEEXT_CERT_CRITERIA_NAME_TYPE"), "certName": SimTypePointer(SimTypeChar(label="Char"), offset=0)}, name="IKEEXT_CERT_NAME0", pack=False, align=None), offset=0), "flags": SimTypeInt(signed=False, label="UInt32")}, name="IKEEXT_CERTIFICATE_CRITERIA0", pack=False, align=None), offset=0)}, name="_Anonymous3_e__Struct", pack=False, align=None)}, name="<anon>", label="None"), "flags": SimTypeInt(signed=False, label="IKEEXT_CERT_AUTH"), "localCertLocationUrl": SimStruct({"size": SimTypeInt(signed=False, label="UInt32"), "data": SimTypePointer(SimTypeChar(label="Byte"), offset=0)}, name="FWP_BYTE_BLOB", pack=False, align=None)}, name="IKEEXT_CERTIFICATE_AUTHENTICATION2", pack=False, align=None), "cgaAuthentication": SimStruct({"keyContainerName": SimTypePointer(SimTypeChar(label="Char"), offset=0), "cspName": SimTypePointer(SimTypeChar(label="Char"), offset=0), "cspType": SimTypeInt(signed=False, label="UInt32"), "cgaModifier": SimStruct({"byteArray16": SimTypeFixedSizeArray(SimTypeChar(label="Byte"), 16)}, name="FWP_BYTE_ARRAY16", pack=False, align=None), "cgaCollisionCount": SimTypeChar(label="Byte")}, name="IKEEXT_IPV6_CGA_AUTHENTICATION0", pack=False, align=None), "eapAuthentication": SimStruct({"flags": SimTypeInt(signed=False, label="IKEEXT_EAP_AUTHENTICATION_FLAGS")}, name="IKEEXT_EAP_AUTHENTICATION0", pack=False, align=None)}, name="<anon>", label="None")}, name="IKEEXT_AUTHENTICATION_METHOD2", pack=False, align=None), offset=0), "initiatorImpersonationType": SimTypeInt(signed=False, label="IKEEXT_AUTHENTICATION_IMPERSONATION_TYPE"), "numIkeProposals": SimTypeInt(signed=False, label="UInt32"), "ikeProposals": SimTypePointer(SimStruct({"cipherAlgorithm": SimStruct({"algoIdentifier": SimTypeInt(signed=False, label="IKEEXT_CIPHER_TYPE"), "keyLen": SimTypeInt(signed=False, label="UInt32"), "rounds": SimTypeInt(signed=False, label="UInt32")}, name="IKEEXT_CIPHER_ALGORITHM0", pack=False, align=None), "integrityAlgorithm": SimStruct({"algoIdentifier": SimTypeInt(signed=False, label="IKEEXT_INTEGRITY_TYPE")}, name="IKEEXT_INTEGRITY_ALGORITHM0", pack=False, align=None), "maxLifetimeSeconds": SimTypeInt(signed=False, label="UInt32"), "dhGroup": SimTypeInt(signed=False, label="IKEEXT_DH_GROUP"), "quickModeLimit": SimTypeInt(signed=False, label="UInt32")}, name="IKEEXT_PROPOSAL0", pack=False, align=None), offset=0), "flags": SimTypeInt(signed=False, label="IKEEXT_POLICY_FLAG"), "maxDynamicFilters": SimTypeInt(signed=False, label="UInt32"), "retransmitDurationSecs": SimTypeInt(signed=False, label="UInt32")}, name="IKEEXT_POLICY2", pack=False, align=None), offset=0), "authIpMmPolicy": SimTypePointer(SimStruct({"softExpirationTime": SimTypeInt(signed=False, label="UInt32"), "numAuthenticationMethods": SimTypeInt(signed=False, label="UInt32"), "authenticationMethods": SimTypePointer(SimStruct({"authenticationMethodType": SimTypeInt(signed=False, label="IKEEXT_AUTHENTICATION_METHOD_TYPE"), "Anonymous": SimUnion({"presharedKeyAuthentication": SimStruct({"presharedKey": SimStruct({"size": SimTypeInt(signed=False, label="UInt32"), "data": SimTypePointer(SimTypeChar(label="Byte"), offset=0)}, name="FWP_BYTE_BLOB", pack=False, align=None), "flags": SimTypeInt(signed=False, label="IKEEXT_PRESHARED_KEY_AUTHENTICATION_FLAGS")}, name="IKEEXT_PRESHARED_KEY_AUTHENTICATION1", pack=False, align=None), "certificateAuthentication": SimStruct({"inboundConfigType": SimTypeInt(signed=False, label="IKEEXT_CERT_CONFIG_TYPE"), "Anonymous1": SimUnion({"Anonymous1": SimStruct({"inboundRootArraySize": SimTypeInt(signed=False, label="UInt32"), "inboundRootCriteria": SimTypePointer(SimStruct({"certData": SimStruct({"size": SimTypeInt(signed=False, label="UInt32"), "data": SimTypePointer(SimTypeChar(label="Byte"), offset=0)}, name="FWP_BYTE_BLOB", pack=False, align=None), "certHash": SimStruct({"size": SimTypeInt(signed=False, label="UInt32"), "data": SimTypePointer(SimTypeChar(label="Byte"), offset=0)}, name="FWP_BYTE_BLOB", pack=False, align=None), "eku": SimTypePointer(SimStruct({"numEku": SimTypeInt(signed=False, label="UInt32"), "eku": SimTypePointer(SimTypePointer(SimTypeChar(label="Byte"), offset=0), offset=0)}, name="IKEEXT_CERT_EKUS0", pack=False, align=None), offset=0), "name": SimTypePointer(SimStruct({"nameType": SimTypeInt(signed=False, label="IKEEXT_CERT_CRITERIA_NAME_TYPE"), "certName": SimTypePointer(SimTypeChar(label="Char"), offset=0)}, name="IKEEXT_CERT_NAME0", pack=False, align=None), offset=0), "flags": SimTypeInt(signed=False, label="UInt32")}, name="IKEEXT_CERTIFICATE_CRITERIA0", pack=False, align=None), offset=0)}, name="_Anonymous1_e__Struct", pack=False, align=None), "Anonymous2": SimStruct({"inboundEnterpriseStoreArraySize": SimTypeInt(signed=False, label="UInt32"), "inboundEnterpriseStoreCriteria": SimTypePointer(SimStruct({"certData": SimStruct({"size": SimTypeInt(signed=False, label="UInt32"), "data": SimTypePointer(SimTypeChar(label="Byte"), offset=0)}, name="FWP_BYTE_BLOB", pack=False, align=None), "certHash": SimStruct({"size": SimTypeInt(signed=False, label="UInt32"), "data": SimTypePointer(SimTypeChar(label="Byte"), offset=0)}, name="FWP_BYTE_BLOB", pack=False, align=None), "eku": SimTypePointer(SimStruct({"numEku": SimTypeInt(signed=False, label="UInt32"), "eku": SimTypePointer(SimTypePointer(SimTypeChar(label="Byte"), offset=0), offset=0)}, name="IKEEXT_CERT_EKUS0", pack=False, align=None), offset=0), "name": SimTypePointer(SimStruct({"nameType": SimTypeInt(signed=False, label="IKEEXT_CERT_CRITERIA_NAME_TYPE"), "certName": SimTypePointer(SimTypeChar(label="Char"), offset=0)}, name="IKEEXT_CERT_NAME0", pack=False, align=None), offset=0), "flags": SimTypeInt(signed=False, label="UInt32")}, name="IKEEXT_CERTIFICATE_CRITERIA0", pack=False, align=None), offset=0)}, name="_Anonymous2_e__Struct", pack=False, align=None), "Anonymous3": SimStruct({"inboundRootStoreArraySize": SimTypeInt(signed=False, label="UInt32"), "inboundTrustedRootStoreCriteria": SimTypePointer(SimStruct({"certData": SimStruct({"size": SimTypeInt(signed=False, label="UInt32"), "data": SimTypePointer(SimTypeChar(label="Byte"), offset=0)}, name="FWP_BYTE_BLOB", pack=False, align=None), "certHash": SimStruct({"size": SimTypeInt(signed=False, label="UInt32"), "data": SimTypePointer(SimTypeChar(label="Byte"), offset=0)}, name="FWP_BYTE_BLOB", pack=False, align=None), "eku": SimTypePointer(SimStruct({"numEku": SimTypeInt(signed=False, label="UInt32"), "eku": SimTypePointer(SimTypePointer(SimTypeChar(label="Byte"), offset=0), offset=0)}, name="IKEEXT_CERT_EKUS0", pack=False, align=None), offset=0), "name": SimTypePointer(SimStruct({"nameType": SimTypeInt(signed=False, label="IKEEXT_CERT_CRITERIA_NAME_TYPE"), "certName": SimTypePointer(SimTypeChar(label="Char"), offset=0)}, name="IKEEXT_CERT_NAME0", pack=False, align=None), offset=0), "flags": SimTypeInt(signed=False, label="UInt32")}, name="IKEEXT_CERTIFICATE_CRITERIA0", pack=False, align=None), offset=0)}, name="_Anonymous3_e__Struct", pack=False, align=None)}, name="<anon>", label="None"), "outboundConfigType": SimTypeInt(signed=False, label="IKEEXT_CERT_CONFIG_TYPE"), "Anonymous2": SimUnion({"Anonymous1": SimStruct({"outboundRootArraySize": SimTypeInt(signed=False, label="UInt32"), "outboundRootCriteria": SimTypePointer(SimStruct({"certData": SimStruct({"size": SimTypeInt(signed=False, label="UInt32"), "data": SimTypePointer(SimTypeChar(label="Byte"), offset=0)}, name="FWP_BYTE_BLOB", pack=False, align=None), "certHash": SimStruct({"size": SimTypeInt(signed=False, label="UInt32"), "data": SimTypePointer(SimTypeChar(label="Byte"), offset=0)}, name="FWP_BYTE_BLOB", pack=False, align=None), "eku": SimTypePointer(SimStruct({"numEku": SimTypeInt(signed=False, label="UInt32"), "eku": SimTypePointer(SimTypePointer(SimTypeChar(label="Byte"), offset=0), offset=0)}, name="IKEEXT_CERT_EKUS0", pack=False, align=None), offset=0), "name": SimTypePointer(SimStruct({"nameType": SimTypeInt(signed=False, label="IKEEXT_CERT_CRITERIA_NAME_TYPE"), "certName": SimTypePointer(SimTypeChar(label="Char"), offset=0)}, name="IKEEXT_CERT_NAME0", pack=False, align=None), offset=0), "flags": SimTypeInt(signed=False, label="UInt32")}, name="IKEEXT_CERTIFICATE_CRITERIA0", pack=False, align=None), offset=0)}, name="_Anonymous1_e__Struct", pack=False, align=None), "Anonymous2": SimStruct({"outboundEnterpriseStoreArraySize": SimTypeInt(signed=False, label="UInt32"), "outboundEnterpriseStoreCriteria": SimTypePointer(SimStruct({"certData": SimStruct({"size": SimTypeInt(signed=False, label="UInt32"), "data": SimTypePointer(SimTypeChar(label="Byte"), offset=0)}, name="FWP_BYTE_BLOB", pack=False, align=None), "certHash": SimStruct({"size": SimTypeInt(signed=False, label="UInt32"), "data": SimTypePointer(SimTypeChar(label="Byte"), offset=0)}, name="FWP_BYTE_BLOB", pack=False, align=None), "eku": SimTypePointer(SimStruct({"numEku": SimTypeInt(signed=False, label="UInt32"), "eku": SimTypePointer(SimTypePointer(SimTypeChar(label="Byte"), offset=0), offset=0)}, name="IKEEXT_CERT_EKUS0", pack=False, align=None), offset=0), "name": SimTypePointer(SimStruct({"nameType": SimTypeInt(signed=False, label="IKEEXT_CERT_CRITERIA_NAME_TYPE"), "certName": SimTypePointer(SimTypeChar(label="Char"), offset=0)}, name="IKEEXT_CERT_NAME0", pack=False, align=None), offset=0), "flags": SimTypeInt(signed=False, label="UInt32")}, name="IKEEXT_CERTIFICATE_CRITERIA0", pack=False, align=None), offset=0)}, name="_Anonymous2_e__Struct", pack=False, align=None), "Anonymous3": SimStruct({"outboundRootStoreArraySize": SimTypeInt(signed=False, label="UInt32"), "outboundTrustedRootStoreCriteria": SimTypePointer(SimStruct({"certData": SimStruct({"size": SimTypeInt(signed=False, label="UInt32"), "data": SimTypePointer(SimTypeChar(label="Byte"), offset=0)}, name="FWP_BYTE_BLOB", pack=False, align=None), "certHash": SimStruct({"size": SimTypeInt(signed=False, label="UInt32"), "data": SimTypePointer(SimTypeChar(label="Byte"), offset=0)}, name="FWP_BYTE_BLOB", pack=False, align=None), "eku": SimTypePointer(SimStruct({"numEku": SimTypeInt(signed=False, label="UInt32"), "eku": SimTypePointer(SimTypePointer(SimTypeChar(label="Byte"), offset=0), offset=0)}, name="IKEEXT_CERT_EKUS0", pack=False, align=None), offset=0), "name": SimTypePointer(SimStruct({"nameType": SimTypeInt(signed=False, label="IKEEXT_CERT_CRITERIA_NAME_TYPE"), "certName": SimTypePointer(SimTypeChar(label="Char"), offset=0)}, name="IKEEXT_CERT_NAME0", pack=False, align=None), offset=0), "flags": SimTypeInt(signed=False, label="UInt32")}, name="IKEEXT_CERTIFICATE_CRITERIA0", pack=False, align=None), offset=0)}, name="_Anonymous3_e__Struct", pack=False, align=None)}, name="<anon>", label="None"), "flags": SimTypeInt(signed=False, label="IKEEXT_CERT_AUTH"), "localCertLocationUrl": SimStruct({"size": SimTypeInt(signed=False, label="UInt32"), "data": SimTypePointer(SimTypeChar(label="Byte"), offset=0)}, name="FWP_BYTE_BLOB", pack=False, align=None)}, name="IKEEXT_CERTIFICATE_AUTHENTICATION2", pack=False, align=None), "kerberosAuthentication": SimStruct({"flags": SimTypeInt(signed=False, label="IKEEXT_KERBEROS_AUTHENTICATION_FLAGS"), "proxyServer": SimTypePointer(SimTypeChar(label="Char"), offset=0)}, name="IKEEXT_KERBEROS_AUTHENTICATION1", pack=False, align=None), "reservedAuthentication": SimStruct({"flags": SimTypeInt(signed=False, label="IKEEXT_RESERVED_AUTHENTICATION_FLAGS")}, name="IKEEXT_RESERVED_AUTHENTICATION0", pack=False, align=None), "ntlmV2Authentication": SimStruct({"flags": SimTypeInt(signed=False, label="UInt32")}, name="IKEEXT_NTLM_V2_AUTHENTICATION0", pack=False, align=None), "sslAuthentication": SimStruct({"inboundConfigType": SimTypeInt(signed=False, label="IKEEXT_CERT_CONFIG_TYPE"), "Anonymous1": SimUnion({"Anonymous1": SimStruct({"inboundRootArraySize": SimTypeInt(signed=False, label="UInt32"), "inboundRootCriteria": SimTypePointer(SimStruct({"certData": SimStruct({"size": SimTypeInt(signed=False, label="UInt32"), "data": SimTypePointer(SimTypeChar(label="Byte"), offset=0)}, name="FWP_BYTE_BLOB", pack=False, align=None), "certHash": SimStruct({"size": SimTypeInt(signed=False, label="UInt32"), "data": SimTypePointer(SimTypeChar(label="Byte"), offset=0)}, name="FWP_BYTE_BLOB", pack=False, align=None), "eku": SimTypePointer(SimStruct({"numEku": SimTypeInt(signed=False, label="UInt32"), "eku": SimTypePointer(SimTypePointer(SimTypeChar(label="Byte"), offset=0), offset=0)}, name="IKEEXT_CERT_EKUS0", pack=False, align=None), offset=0), "name": SimTypePointer(SimStruct({"nameType": SimTypeInt(signed=False, label="IKEEXT_CERT_CRITERIA_NAME_TYPE"), "certName": SimTypePointer(SimTypeChar(label="Char"), offset=0)}, name="IKEEXT_CERT_NAME0", pack=False, align=None), offset=0), "flags": SimTypeInt(signed=False, label="UInt32")}, name="IKEEXT_CERTIFICATE_CRITERIA0", pack=False, align=None), offset=0)}, name="_Anonymous1_e__Struct", pack=False, align=None), "Anonymous2": SimStruct({"inboundEnterpriseStoreArraySize": SimTypeInt(signed=False, label="UInt32"), "inboundEnterpriseStoreCriteria": SimTypePointer(SimStruct({"certData": SimStruct({"size": SimTypeInt(signed=False, label="UInt32"), "data": SimTypePointer(SimTypeChar(label="Byte"), offset=0)}, name="FWP_BYTE_BLOB", pack=False, align=None), "certHash": SimStruct({"size": SimTypeInt(signed=False, label="UInt32"), "data": SimTypePointer(SimTypeChar(label="Byte"), offset=0)}, name="FWP_BYTE_BLOB", pack=False, align=None), "eku": SimTypePointer(SimStruct({"numEku": SimTypeInt(signed=False, label="UInt32"), "eku": SimTypePointer(SimTypePointer(SimTypeChar(label="Byte"), offset=0), offset=0)}, name="IKEEXT_CERT_EKUS0", pack=False, align=None), offset=0), "name": SimTypePointer(SimStruct({"nameType": SimTypeInt(signed=False, label="IKEEXT_CERT_CRITERIA_NAME_TYPE"), "certName": SimTypePointer(SimTypeChar(label="Char"), offset=0)}, name="IKEEXT_CERT_NAME0", pack=False, align=None), offset=0), "flags": SimTypeInt(signed=False, label="UInt32")}, name="IKEEXT_CERTIFICATE_CRITERIA0", pack=False, align=None), offset=0)}, name="_Anonymous2_e__Struct", pack=False, align=None), "Anonymous3": SimStruct({"inboundRootStoreArraySize": SimTypeInt(signed=False, label="UInt32"), "inboundTrustedRootStoreCriteria": SimTypePointer(SimStruct({"certData": SimStruct({"size": SimTypeInt(signed=False, label="UInt32"), "data": SimTypePointer(SimTypeChar(label="Byte"), offset=0)}, name="FWP_BYTE_BLOB", pack=False, align=None), "certHash": SimStruct({"size": SimTypeInt(signed=False, label="UInt32"), "data": SimTypePointer(SimTypeChar(label="Byte"), offset=0)}, name="FWP_BYTE_BLOB", pack=False, align=None), "eku": SimTypePointer(SimStruct({"numEku": SimTypeInt(signed=False, label="UInt32"), "eku": SimTypePointer(SimTypePointer(SimTypeChar(label="Byte"), offset=0), offset=0)}, name="IKEEXT_CERT_EKUS0", pack=False, align=None), offset=0), "name": SimTypePointer(SimStruct({"nameType": SimTypeInt(signed=False, label="IKEEXT_CERT_CRITERIA_NAME_TYPE"), "certName": SimTypePointer(SimTypeChar(label="Char"), offset=0)}, name="IKEEXT_CERT_NAME0", pack=False, align=None), offset=0), "flags": SimTypeInt(signed=False, label="UInt32")}, name="IKEEXT_CERTIFICATE_CRITERIA0", pack=False, align=None), offset=0)}, name="_Anonymous3_e__Struct", pack=False, align=None)}, name="<anon>", label="None"), "outboundConfigType": SimTypeInt(signed=False, label="IKEEXT_CERT_CONFIG_TYPE"), "Anonymous2": SimUnion({"Anonymous1": SimStruct({"outboundRootArraySize": SimTypeInt(signed=False, label="UInt32"), "outboundRootCriteria": SimTypePointer(SimStruct({"certData": SimStruct({"size": SimTypeInt(signed=False, label="UInt32"), "data": SimTypePointer(SimTypeChar(label="Byte"), offset=0)}, name="FWP_BYTE_BLOB", pack=False, align=None), "certHash": SimStruct({"size": SimTypeInt(signed=False, label="UInt32"), "data": SimTypePointer(SimTypeChar(label="Byte"), offset=0)}, name="FWP_BYTE_BLOB", pack=False, align=None), "eku": SimTypePointer(SimStruct({"numEku": SimTypeInt(signed=False, label="UInt32"), "eku": SimTypePointer(SimTypePointer(SimTypeChar(label="Byte"), offset=0), offset=0)}, name="IKEEXT_CERT_EKUS0", pack=False, align=None), offset=0), "name": SimTypePointer(SimStruct({"nameType": SimTypeInt(signed=False, label="IKEEXT_CERT_CRITERIA_NAME_TYPE"), "certName": SimTypePointer(SimTypeChar(label="Char"), offset=0)}, name="IKEEXT_CERT_NAME0", pack=False, align=None), offset=0), "flags": SimTypeInt(signed=False, label="UInt32")}, name="IKEEXT_CERTIFICATE_CRITERIA0", pack=False, align=None), offset=0)}, name="_Anonymous1_e__Struct", pack=False, align=None), "Anonymous2": SimStruct({"outboundEnterpriseStoreArraySize": SimTypeInt(signed=False, label="UInt32"), "outboundEnterpriseStoreCriteria": SimTypePointer(SimStruct({"certData": SimStruct({"size": SimTypeInt(signed=False, label="UInt32"), "data": SimTypePointer(SimTypeChar(label="Byte"), offset=0)}, name="FWP_BYTE_BLOB", pack=False, align=None), "certHash": SimStruct({"size": SimTypeInt(signed=False, label="UInt32"), "data": SimTypePointer(SimTypeChar(label="Byte"), offset=0)}, name="FWP_BYTE_BLOB", pack=False, align=None), "eku": SimTypePointer(SimStruct({"numEku": SimTypeInt(signed=False, label="UInt32"), "eku": SimTypePointer(SimTypePointer(SimTypeChar(label="Byte"), offset=0), offset=0)}, name="IKEEXT_CERT_EKUS0", pack=False, align=None), offset=0), "name": SimTypePointer(SimStruct({"nameType": SimTypeInt(signed=False, label="IKEEXT_CERT_CRITERIA_NAME_TYPE"), "certName": SimTypePointer(SimTypeChar(label="Char"), offset=0)}, name="IKEEXT_CERT_NAME0", pack=False, align=None), offset=0), "flags": SimTypeInt(signed=False, label="UInt32")}, name="IKEEXT_CERTIFICATE_CRITERIA0", pack=False, align=None), offset=0)}, name="_Anonymous2_e__Struct", pack=False, align=None), "Anonymous3": SimStruct({"outboundRootStoreArraySize": SimTypeInt(signed=False, label="UInt32"), "outboundTrustedRootStoreCriteria": SimTypePointer(SimStruct({"certData": SimStruct({"size": SimTypeInt(signed=False, label="UInt32"), "data": SimTypePointer(SimTypeChar(label="Byte"), offset=0)}, name="FWP_BYTE_BLOB", pack=False, align=None), "certHash": SimStruct({"size": SimTypeInt(signed=False, label="UInt32"), "data": SimTypePointer(SimTypeChar(label="Byte"), offset=0)}, name="FWP_BYTE_BLOB", pack=False, align=None), "eku": SimTypePointer(SimStruct({"numEku": SimTypeInt(signed=False, label="UInt32"), "eku": SimTypePointer(SimTypePointer(SimTypeChar(label="Byte"), offset=0), offset=0)}, name="IKEEXT_CERT_EKUS0", pack=False, align=None), offset=0), "name": SimTypePointer(SimStruct({"nameType": SimTypeInt(signed=False, label="IKEEXT_CERT_CRITERIA_NAME_TYPE"), "certName": SimTypePointer(SimTypeChar(label="Char"), offset=0)}, name="IKEEXT_CERT_NAME0", pack=False, align=None), offset=0), "flags": SimTypeInt(signed=False, label="UInt32")}, name="IKEEXT_CERTIFICATE_CRITERIA0", pack=False, align=None), offset=0)}, name="_Anonymous3_e__Struct", pack=False, align=None)}, name="<anon>", label="None"), "flags": SimTypeInt(signed=False, label="IKEEXT_CERT_AUTH"), "localCertLocationUrl": SimStruct({"size": SimTypeInt(signed=False, label="UInt32"), "data": SimTypePointer(SimTypeChar(label="Byte"), offset=0)}, name="FWP_BYTE_BLOB", pack=False, align=None)}, name="IKEEXT_CERTIFICATE_AUTHENTICATION2", pack=False, align=None), "cgaAuthentication": SimStruct({"keyContainerName": SimTypePointer(SimTypeChar(label="Char"), offset=0), "cspName": SimTypePointer(SimTypeChar(label="Char"), offset=0), "cspType": SimTypeInt(signed=False, label="UInt32"), "cgaModifier": SimStruct({"byteArray16": SimTypeFixedSizeArray(SimTypeChar(label="Byte"), 16)}, name="FWP_BYTE_ARRAY16", pack=False, align=None), "cgaCollisionCount": SimTypeChar(label="Byte")}, name="IKEEXT_IPV6_CGA_AUTHENTICATION0", pack=False, align=None), "eapAuthentication": SimStruct({"flags": SimTypeInt(signed=False, label="IKEEXT_EAP_AUTHENTICATION_FLAGS")}, name="IKEEXT_EAP_AUTHENTICATION0", pack=False, align=None)}, name="<anon>", label="None")}, name="IKEEXT_AUTHENTICATION_METHOD2", pack=False, align=None), offset=0), "initiatorImpersonationType": SimTypeInt(signed=False, label="IKEEXT_AUTHENTICATION_IMPERSONATION_TYPE"), "numIkeProposals": SimTypeInt(signed=False, label="UInt32"), "ikeProposals": SimTypePointer(SimStruct({"cipherAlgorithm": SimStruct({"algoIdentifier": SimTypeInt(signed=False, label="IKEEXT_CIPHER_TYPE"), "keyLen": SimTypeInt(signed=False, label="UInt32"), "rounds": SimTypeInt(signed=False, label="UInt32")}, name="IKEEXT_CIPHER_ALGORITHM0", pack=False, align=None), "integrityAlgorithm": SimStruct({"algoIdentifier": SimTypeInt(signed=False, label="IKEEXT_INTEGRITY_TYPE")}, name="IKEEXT_INTEGRITY_ALGORITHM0", pack=False, align=None), "maxLifetimeSeconds": SimTypeInt(signed=False, label="UInt32"), "dhGroup": SimTypeInt(signed=False, label="IKEEXT_DH_GROUP"), "quickModeLimit": SimTypeInt(signed=False, label="UInt32")}, name="IKEEXT_PROPOSAL0", pack=False, align=None), offset=0), "flags": SimTypeInt(signed=False, label="IKEEXT_POLICY_FLAG"), "maxDynamicFilters": SimTypeInt(signed=False, label="UInt32"), "retransmitDurationSecs": SimTypeInt(signed=False, label="UInt32")}, name="IKEEXT_POLICY2", pack=False, align=None), offset=0), "dataBuffer": SimTypePointer(SimStruct({"size": SimTypeInt(signed=False, label="UInt32"), "data": SimTypePointer(SimTypeChar(label="Byte"), offset=0)}, name="FWP_BYTE_BLOB", pack=False, align=None), offset=0), "classifyOptions": SimTypePointer(SimStruct({"numOptions": SimTypeInt(signed=False, label="UInt32"), "options": SimTypePointer(SimStruct({"type": SimTypeInt(signed=False, label="FWP_CLASSIFY_OPTION_TYPE"), "value": SimStruct({"type": SimTypeInt(signed=False, label="FWP_DATA_TYPE"), "Anonymous": SimUnion({"uint8": SimTypeChar(label="Byte"), "uint16": SimTypeShort(signed=False, label="UInt16"), "uint32": SimTypeInt(signed=False, label="UInt32"), "uint64": SimTypePointer(SimTypeLongLong(signed=False, label="UInt64"), offset=0), "int8": SimTypeChar(label="SByte"), "int16": SimTypeShort(signed=True, label="Int16"), "int32": SimTypeInt(signed=True, label="Int32"), "int64": SimTypePointer(SimTypeLongLong(signed=True, label="Int64"), offset=0), "float32": SimTypeFloat(size=32), "double64": SimTypePointer(SimTypeFloat(size=64), offset=0), "byteArray16": SimTypePointer(SimStruct({"byteArray16": SimTypeFixedSizeArray(SimTypeChar(label="Byte"), 16)}, name="FWP_BYTE_ARRAY16", pack=False, align=None), offset=0), "byteBlob": SimTypePointer(SimStruct({"size": SimTypeInt(signed=False, label="UInt32"), "data": SimTypePointer(SimTypeChar(label="Byte"), offset=0)}, name="FWP_BYTE_BLOB", pack=False, align=None), offset=0), "sid": SimTypePointer(SimTypeBottom(label="SID"), offset=0), "sd": SimTypePointer(SimStruct({"size": SimTypeInt(signed=False, label="UInt32"), "data": SimTypePointer(SimTypeChar(label="Byte"), offset=0)}, name="FWP_BYTE_BLOB", pack=False, align=None), offset=0), "tokenInformation": SimTypePointer(SimStruct({"sidCount": SimTypeInt(signed=False, label="UInt32"), "sids": SimTypePointer(SimTypeBottom(label="SID_AND_ATTRIBUTES"), offset=0), "restrictedSidCount": SimTypeInt(signed=False, label="UInt32"), "restrictedSids": SimTypePointer(SimTypeBottom(label="SID_AND_ATTRIBUTES"), offset=0)}, name="FWP_TOKEN_INFORMATION", pack=False, align=None), offset=0), "tokenAccessInformation": SimTypePointer(SimStruct({"size": SimTypeInt(signed=False, label="UInt32"), "data": SimTypePointer(SimTypeChar(label="Byte"), offset=0)}, name="FWP_BYTE_BLOB", pack=False, align=None), offset=0), "unicodeString": SimTypePointer(SimTypeChar(label="Char"), offset=0), "byteArray6": SimTypePointer(SimStruct({"byteArray6": SimTypeFixedSizeArray(SimTypeChar(label="Byte"), 6)}, name="FWP_BYTE_ARRAY6", pack=False, align=None), offset=0), "bitmapArray64": SimTypePointer(SimStruct({"bitmapArray64": SimTypeFixedSizeArray(SimTypeChar(label="Byte"), 8)}, name="FWP_BITMAP_ARRAY64_", pack=False, align=None), offset=0)}, name="<anon>", label="None")}, name="FWP_VALUE0", pack=False, align=None)}, name="FWPM_CLASSIFY_OPTION0", pack=False, align=None), offset=0)}, name="FWPM_CLASSIFY_OPTIONS0", pack=False, align=None), offset=0), "ikeV2QmTunnelPolicy": SimTypePointer(SimStruct({"flags": SimTypeInt(signed=False, label="IPSEC_POLICY_FLAG"), "numIpsecProposals": SimTypeInt(signed=False, label="UInt32"), "ipsecProposals": SimTypePointer(SimStruct({"lifetime": SimStruct({"lifetimeSeconds": SimTypeInt(signed=False, label="UInt32"), "lifetimeKilobytes": SimTypeInt(signed=False, label="UInt32"), "lifetimePackets": SimTypeInt(signed=False, label="UInt32")}, name="IPSEC_SA_LIFETIME0", pack=False, align=None), "numSaTransforms": SimTypeInt(signed=False, label="UInt32"), "saTransforms": SimTypePointer(SimStruct({"ipsecTransformType": SimTypeInt(signed=False, label="IPSEC_TRANSFORM_TYPE"), "Anonymous": SimUnion({"ahTransform": SimTypePointer(SimStruct({"authTransformId": SimStruct({"authType": SimTypeInt(signed=False, label="IPSEC_AUTH_TYPE"), "authConfig": SimTypeChar(label="Byte")}, name="IPSEC_AUTH_TRANSFORM_ID0", pack=False, align=None), "cryptoModuleId": SimTypePointer(SimTypeBottom(label="Guid"), offset=0)}, name="IPSEC_AUTH_TRANSFORM0", pack=False, align=None), offset=0), "espAuthTransform": SimTypePointer(SimStruct({"authTransformId": SimStruct({"authType": SimTypeInt(signed=False, label="IPSEC_AUTH_TYPE"), "authConfig": SimTypeChar(label="Byte")}, name="IPSEC_AUTH_TRANSFORM_ID0", pack=False, align=None), "cryptoModuleId": SimTypePointer(SimTypeBottom(label="Guid"), offset=0)}, name="IPSEC_AUTH_TRANSFORM0", pack=False, align=None), offset=0), "espCipherTransform": SimTypePointer(SimStruct({"cipherTransformId": SimStruct({"cipherType": SimTypeInt(signed=False, label="IPSEC_CIPHER_TYPE"), "cipherConfig": SimTypeChar(label="Byte")}, name="IPSEC_CIPHER_TRANSFORM_ID0", pack=False, align=None), "cryptoModuleId": SimTypePointer(SimTypeBottom(label="Guid"), offset=0)}, name="IPSEC_CIPHER_TRANSFORM0", pack=False, align=None), offset=0), "espAuthAndCipherTransform": SimTypePointer(SimStruct({"authTransform": SimStruct({"authTransformId": SimStruct({"authType": SimTypeInt(signed=False, label="IPSEC_AUTH_TYPE"), "authConfig": SimTypeChar(label="Byte")}, name="IPSEC_AUTH_TRANSFORM_ID0", pack=False, align=None), "cryptoModuleId": SimTypePointer(SimTypeBottom(label="Guid"), offset=0)}, name="IPSEC_AUTH_TRANSFORM0", pack=False, align=None), "cipherTransform": SimStruct({"cipherTransformId": SimStruct({"cipherType": SimTypeInt(signed=False, label="IPSEC_CIPHER_TYPE"), "cipherConfig": SimTypeChar(label="Byte")}, name="IPSEC_CIPHER_TRANSFORM_ID0", pack=False, align=None), "cryptoModuleId": SimTypePointer(SimTypeBottom(label="Guid"), offset=0)}, name="IPSEC_CIPHER_TRANSFORM0", pack=False, align=None)}, name="IPSEC_AUTH_AND_CIPHER_TRANSFORM0", pack=False, align=None), offset=0), "espAuthFwTransform": SimTypePointer(SimStruct({"authTransformId": SimStruct({"authType": SimTypeInt(signed=False, label="IPSEC_AUTH_TYPE"), "authConfig": SimTypeChar(label="Byte")}, name="IPSEC_AUTH_TRANSFORM_ID0", pack=False, align=None), "cryptoModuleId": SimTypePointer(SimTypeBottom(label="Guid"), offset=0)}, name="IPSEC_AUTH_TRANSFORM0", pack=False, align=None), offset=0)}, name="<anon>", label="None")}, name="IPSEC_SA_TRANSFORM0", pack=False, align=None), offset=0), "pfsGroup": SimTypeInt(signed=False, label="IPSEC_PFS_GROUP")}, name="IPSEC_PROPOSAL0", pack=False, align=None), offset=0), "tunnelEndpoints": SimStruct({"ipVersion": SimTypeInt(signed=False, label="FWP_IP_VERSION"), "Anonymous1": SimUnion({"localV4Address": SimTypeInt(signed=False, label="UInt32"), "localV6Address": SimTypeFixedSizeArray(SimTypeChar(label="Byte"), 16)}, name="<anon>", label="None"), "Anonymous2": SimUnion({"remoteV4Address": SimTypeInt(signed=False, label="UInt32"), "remoteV6Address": SimTypeFixedSizeArray(SimTypeChar(label="Byte"), 16)}, name="<anon>", label="None"), "localIfLuid": SimTypeLongLong(signed=False, label="UInt64"), "remoteFqdn": SimTypePointer(SimTypeChar(label="Char"), offset=0), "numAddresses": SimTypeInt(signed=False, label="UInt32"), "remoteAddresses": SimTypePointer(SimStruct({"ipVersion": SimTypeInt(signed=False, label="FWP_IP_VERSION"), "Anonymous": SimUnion({"v4Address": SimTypeInt(signed=False, label="UInt32"), "v6Address": SimTypeFixedSizeArray(SimTypeChar(label="Byte"), 16)}, name="<anon>", label="None")}, name="IPSEC_TUNNEL_ENDPOINT0", pack=False, align=None), offset=0)}, name="IPSEC_TUNNEL_ENDPOINTS2", pack=False, align=None), "saIdleTimeout": SimStruct({"idleTimeoutSeconds": SimTypeInt(signed=False, label="UInt32"), "idleTimeoutSecondsFailOver": SimTypeInt(signed=False, label="UInt32")}, name="IPSEC_SA_IDLE_TIMEOUT0", pack=False, align=None), "emPolicy": SimTypePointer(SimStruct({"numAuthenticationMethods": SimTypeInt(signed=False, label="UInt32"), "authenticationMethods": SimTypePointer(SimStruct({"authenticationMethodType": SimTypeInt(signed=False, label="IKEEXT_AUTHENTICATION_METHOD_TYPE"), "Anonymous": SimUnion({"presharedKeyAuthentication": SimStruct({"presharedKey": SimStruct({"size": SimTypeInt(signed=False, label="UInt32"), "data": SimTypePointer(SimTypeChar(label="Byte"), offset=0)}, name="FWP_BYTE_BLOB", pack=False, align=None), "flags": SimTypeInt(signed=False, label="IKEEXT_PRESHARED_KEY_AUTHENTICATION_FLAGS")}, name="IKEEXT_PRESHARED_KEY_AUTHENTICATION1", pack=False, align=None), "certificateAuthentication": SimStruct({"inboundConfigType": SimTypeInt(signed=False, label="IKEEXT_CERT_CONFIG_TYPE"), "Anonymous1": SimUnion({"Anonymous1": SimStruct({"inboundRootArraySize": SimTypeInt(signed=False, label="UInt32"), "inboundRootCriteria": SimTypePointer(SimStruct({"certData": SimStruct({"size": SimTypeInt(signed=False, label="UInt32"), "data": SimTypePointer(SimTypeChar(label="Byte"), offset=0)}, name="FWP_BYTE_BLOB", pack=False, align=None), "certHash": SimStruct({"size": SimTypeInt(signed=False, label="UInt32"), "data": SimTypePointer(SimTypeChar(label="Byte"), offset=0)}, name="FWP_BYTE_BLOB", pack=False, align=None), "eku": SimTypePointer(SimStruct({"numEku": SimTypeInt(signed=False, label="UInt32"), "eku": SimTypePointer(SimTypePointer(SimTypeChar(label="Byte"), offset=0), offset=0)}, name="IKEEXT_CERT_EKUS0", pack=False, align=None), offset=0), "name": SimTypePointer(SimStruct({"nameType": SimTypeInt(signed=False, label="IKEEXT_CERT_CRITERIA_NAME_TYPE"), "certName": SimTypePointer(SimTypeChar(label="Char"), offset=0)}, name="IKEEXT_CERT_NAME0", pack=False, align=None), offset=0), "flags": SimTypeInt(signed=False, label="UInt32")}, name="IKEEXT_CERTIFICATE_CRITERIA0", pack=False, align=None), offset=0)}, name="_Anonymous1_e__Struct", pack=False, align=None), "Anonymous2": SimStruct({"inboundEnterpriseStoreArraySize": SimTypeInt(signed=False, label="UInt32"), "inboundEnterpriseStoreCriteria": SimTypePointer(SimStruct({"certData": SimStruct({"size": SimTypeInt(signed=False, label="UInt32"), "data": SimTypePointer(SimTypeChar(label="Byte"), offset=0)}, name="FWP_BYTE_BLOB", pack=False, align=None), "certHash": SimStruct({"size": SimTypeInt(signed=False, label="UInt32"), "data": SimTypePointer(SimTypeChar(label="Byte"), offset=0)}, name="FWP_BYTE_BLOB", pack=False, align=None), "eku": SimTypePointer(SimStruct({"numEku": SimTypeInt(signed=False, label="UInt32"), "eku": SimTypePointer(SimTypePointer(SimTypeChar(label="Byte"), offset=0), offset=0)}, name="IKEEXT_CERT_EKUS0", pack=False, align=None), offset=0), "name": SimTypePointer(SimStruct({"nameType": SimTypeInt(signed=False, label="IKEEXT_CERT_CRITERIA_NAME_TYPE"), "certName": SimTypePointer(SimTypeChar(label="Char"), offset=0)}, name="IKEEXT_CERT_NAME0", pack=False, align=None), offset=0), "flags": SimTypeInt(signed=False, label="UInt32")}, name="IKEEXT_CERTIFICATE_CRITERIA0", pack=False, align=None), offset=0)}, name="_Anonymous2_e__Struct", pack=False, align=None), "Anonymous3": SimStruct({"inboundRootStoreArraySize": SimTypeInt(signed=False, label="UInt32"), "inboundTrustedRootStoreCriteria": SimTypePointer(SimStruct({"certData": SimStruct({"size": SimTypeInt(signed=False, label="UInt32"), "data": SimTypePointer(SimTypeChar(label="Byte"), offset=0)}, name="FWP_BYTE_BLOB", pack=False, align=None), "certHash": SimStruct({"size": SimTypeInt(signed=False, label="UInt32"), "data": SimTypePointer(SimTypeChar(label="Byte"), offset=0)}, name="FWP_BYTE_BLOB", pack=False, align=None), "eku": SimTypePointer(SimStruct({"numEku": SimTypeInt(signed=False, label="UInt32"), "eku": SimTypePointer(SimTypePointer(SimTypeChar(label="Byte"), offset=0), offset=0)}, name="IKEEXT_CERT_EKUS0", pack=False, align=None), offset=0), "name": SimTypePointer(SimStruct({"nameType": SimTypeInt(signed=False, label="IKEEXT_CERT_CRITERIA_NAME_TYPE"), "certName": SimTypePointer(SimTypeChar(label="Char"), offset=0)}, name="IKEEXT_CERT_NAME0", pack=False, align=None), offset=0), "flags": SimTypeInt(signed=False, label="UInt32")}, name="IKEEXT_CERTIFICATE_CRITERIA0", pack=False, align=None), offset=0)}, name="_Anonymous3_e__Struct", pack=False, align=None)}, name="<anon>", label="None"), "outboundConfigType": SimTypeInt(signed=False, label="IKEEXT_CERT_CONFIG_TYPE"), "Anonymous2": SimUnion({"Anonymous1": SimStruct({"outboundRootArraySize": SimTypeInt(signed=False, label="UInt32"), "outboundRootCriteria": SimTypePointer(SimStruct({"certData": SimStruct({"size": SimTypeInt(signed=False, label="UInt32"), "data": SimTypePointer(SimTypeChar(label="Byte"), offset=0)}, name="FWP_BYTE_BLOB", pack=False, align=None), "certHash": SimStruct({"size": SimTypeInt(signed=False, label="UInt32"), "data": SimTypePointer(SimTypeChar(label="Byte"), offset=0)}, name="FWP_BYTE_BLOB", pack=False, align=None), "eku": SimTypePointer(SimStruct({"numEku": SimTypeInt(signed=False, label="UInt32"), "eku": SimTypePointer(SimTypePointer(SimTypeChar(label="Byte"), offset=0), offset=0)}, name="IKEEXT_CERT_EKUS0", pack=False, align=None), offset=0), "name": SimTypePointer(SimStruct({"nameType": SimTypeInt(signed=False, label="IKEEXT_CERT_CRITERIA_NAME_TYPE"), "certName": SimTypePointer(SimTypeChar(label="Char"), offset=0)}, name="IKEEXT_CERT_NAME0", pack=False, align=None), offset=0), "flags": SimTypeInt(signed=False, label="UInt32")}, name="IKEEXT_CERTIFICATE_CRITERIA0", pack=False, align=None), offset=0)}, name="_Anonymous1_e__Struct", pack=False, align=None), "Anonymous2": SimStruct({"outboundEnterpriseStoreArraySize": SimTypeInt(signed=False, label="UInt32"), "outboundEnterpriseStoreCriteria": SimTypePointer(SimStruct({"certData": SimStruct({"size": SimTypeInt(signed=False, label="UInt32"), "data": SimTypePointer(SimTypeChar(label="Byte"), offset=0)}, name="FWP_BYTE_BLOB", pack=False, align=None), "certHash": SimStruct({"size": SimTypeInt(signed=False, label="UInt32"), "data": SimTypePointer(SimTypeChar(label="Byte"), offset=0)}, name="FWP_BYTE_BLOB", pack=False, align=None), "eku": SimTypePointer(SimStruct({"numEku": SimTypeInt(signed=False, label="UInt32"), "eku": SimTypePointer(SimTypePointer(SimTypeChar(label="Byte"), offset=0), offset=0)}, name="IKEEXT_CERT_EKUS0", pack=False, align=None), offset=0), "name": SimTypePointer(SimStruct({"nameType": SimTypeInt(signed=False, label="IKEEXT_CERT_CRITERIA_NAME_TYPE"), "certName": SimTypePointer(SimTypeChar(label="Char"), offset=0)}, name="IKEEXT_CERT_NAME0", pack=False, align=None), offset=0), "flags": SimTypeInt(signed=False, label="UInt32")}, name="IKEEXT_CERTIFICATE_CRITERIA0", pack=False, align=None), offset=0)}, name="_Anonymous2_e__Struct", pack=False, align=None), "Anonymous3": SimStruct({"outboundRootStoreArraySize": SimTypeInt(signed=False, label="UInt32"), "outboundTrustedRootStoreCriteria": SimTypePointer(SimStruct({"certData": SimStruct({"size": SimTypeInt(signed=False, label="UInt32"), "data": SimTypePointer(SimTypeChar(label="Byte"), offset=0)}, name="FWP_BYTE_BLOB", pack=False, align=None), "certHash": SimStruct({"size": SimTypeInt(signed=False, label="UInt32"), "data": SimTypePointer(SimTypeChar(label="Byte"), offset=0)}, name="FWP_BYTE_BLOB", pack=False, align=None), "eku": SimTypePointer(SimStruct({"numEku": SimTypeInt(signed=False, label="UInt32"), "eku": SimTypePointer(SimTypePointer(SimTypeChar(label="Byte"), offset=0), offset=0)}, name="IKEEXT_CERT_EKUS0", pack=False, align=None), offset=0), "name": SimTypePointer(SimStruct({"nameType": SimTypeInt(signed=False, label="IKEEXT_CERT_CRITERIA_NAME_TYPE"), "certName": SimTypePointer(SimTypeChar(label="Char"), offset=0)}, name="IKEEXT_CERT_NAME0", pack=False, align=None), offset=0), "flags": SimTypeInt(signed=False, label="UInt32")}, name="IKEEXT_CERTIFICATE_CRITERIA0", pack=False, align=None), offset=0)}, name="_Anonymous3_e__Struct", pack=False, align=None)}, name="<anon>", label="None"), "flags": SimTypeInt(signed=False, label="IKEEXT_CERT_AUTH"), "localCertLocationUrl": SimStruct({"size": SimTypeInt(signed=False, label="UInt32"), "data": SimTypePointer(SimTypeChar(label="Byte"), offset=0)}, name="FWP_BYTE_BLOB", pack=False, align=None)}, name="IKEEXT_CERTIFICATE_AUTHENTICATION2", pack=False, align=None), "kerberosAuthentication": SimStruct({"flags": SimTypeInt(signed=False, label="IKEEXT_KERBEROS_AUTHENTICATION_FLAGS"), "proxyServer": SimTypePointer(SimTypeChar(label="Char"), offset=0)}, name="IKEEXT_KERBEROS_AUTHENTICATION1", pack=False, align=None), "reservedAuthentication": SimStruct({"flags": SimTypeInt(signed=False, label="IKEEXT_RESERVED_AUTHENTICATION_FLAGS")}, name="IKEEXT_RESERVED_AUTHENTICATION0", pack=False, align=None), "ntlmV2Authentication": SimStruct({"flags": SimTypeInt(signed=False, label="UInt32")}, name="IKEEXT_NTLM_V2_AUTHENTICATION0", pack=False, align=None), "sslAuthentication": SimStruct({"inboundConfigType": SimTypeInt(signed=False, label="IKEEXT_CERT_CONFIG_TYPE"), "Anonymous1": SimUnion({"Anonymous1": SimStruct({"inboundRootArraySize": SimTypeInt(signed=False, label="UInt32"), "inboundRootCriteria": SimTypePointer(SimStruct({"certData": SimStruct({"size": SimTypeInt(signed=False, label="UInt32"), "data": SimTypePointer(SimTypeChar(label="Byte"), offset=0)}, name="FWP_BYTE_BLOB", pack=False, align=None), "certHash": SimStruct({"size": SimTypeInt(signed=False, label="UInt32"), "data": SimTypePointer(SimTypeChar(label="Byte"), offset=0)}, name="FWP_BYTE_BLOB", pack=False, align=None), "eku": SimTypePointer(SimStruct({"numEku": SimTypeInt(signed=False, label="UInt32"), "eku": SimTypePointer(SimTypePointer(SimTypeChar(label="Byte"), offset=0), offset=0)}, name="IKEEXT_CERT_EKUS0", pack=False, align=None), offset=0), "name": SimTypePointer(SimStruct({"nameType": SimTypeInt(signed=False, label="IKEEXT_CERT_CRITERIA_NAME_TYPE"), "certName": SimTypePointer(SimTypeChar(label="Char"), offset=0)}, name="IKEEXT_CERT_NAME0", pack=False, align=None), offset=0), "flags": SimTypeInt(signed=False, label="UInt32")}, name="IKEEXT_CERTIFICATE_CRITERIA0", pack=False, align=None), offset=0)}, name="_Anonymous1_e__Struct", pack=False, align=None), "Anonymous2": SimStruct({"inboundEnterpriseStoreArraySize": SimTypeInt(signed=False, label="UInt32"), "inboundEnterpriseStoreCriteria": SimTypePointer(SimStruct({"certData": SimStruct({"size": SimTypeInt(signed=False, label="UInt32"), "data": SimTypePointer(SimTypeChar(label="Byte"), offset=0)}, name="FWP_BYTE_BLOB", pack=False, align=None), "certHash": SimStruct({"size": SimTypeInt(signed=False, label="UInt32"), "data": SimTypePointer(SimTypeChar(label="Byte"), offset=0)}, name="FWP_BYTE_BLOB", pack=False, align=None), "eku": SimTypePointer(SimStruct({"numEku": SimTypeInt(signed=False, label="UInt32"), "eku": SimTypePointer(SimTypePointer(SimTypeChar(label="Byte"), offset=0), offset=0)}, name="IKEEXT_CERT_EKUS0", pack=False, align=None), offset=0), "name": SimTypePointer(SimStruct({"nameType": SimTypeInt(signed=False, label="IKEEXT_CERT_CRITERIA_NAME_TYPE"), "certName": SimTypePointer(SimTypeChar(label="Char"), offset=0)}, name="IKEEXT_CERT_NAME0", pack=False, align=None), offset=0), "flags": SimTypeInt(signed=False, label="UInt32")}, name="IKEEXT_CERTIFICATE_CRITERIA0", pack=False, align=None), offset=0)}, name="_Anonymous2_e__Struct", pack=False, align=None), "Anonymous3": SimStruct({"inboundRootStoreArraySize": SimTypeInt(signed=False, label="UInt32"), "inboundTrustedRootStoreCriteria": SimTypePointer(SimStruct({"certData": SimStruct({"size": SimTypeInt(signed=False, label="UInt32"), "data": SimTypePointer(SimTypeChar(label="Byte"), offset=0)}, name="FWP_BYTE_BLOB", pack=False, align=None), "certHash": SimStruct({"size": SimTypeInt(signed=False, label="UInt32"), "data": SimTypePointer(SimTypeChar(label="Byte"), offset=0)}, name="FWP_BYTE_BLOB", pack=False, align=None), "eku": SimTypePointer(SimStruct({"numEku": SimTypeInt(signed=False, label="UInt32"), "eku": SimTypePointer(SimTypePointer(SimTypeChar(label="Byte"), offset=0), offset=0)}, name="IKEEXT_CERT_EKUS0", pack=False, align=None), offset=0), "name": SimTypePointer(SimStruct({"nameType": SimTypeInt(signed=False, label="IKEEXT_CERT_CRITERIA_NAME_TYPE"), "certName": SimTypePointer(SimTypeChar(label="Char"), offset=0)}, name="IKEEXT_CERT_NAME0", pack=False, align=None), offset=0), "flags": SimTypeInt(signed=False, label="UInt32")}, name="IKEEXT_CERTIFICATE_CRITERIA0", pack=False, align=None), offset=0)}, name="_Anonymous3_e__Struct", pack=False, align=None)}, name="<anon>", label="None"), "outboundConfigType": SimTypeInt(signed=False, label="IKEEXT_CERT_CONFIG_TYPE"), "Anonymous2": SimUnion({"Anonymous1": SimStruct({"outboundRootArraySize": SimTypeInt(signed=False, label="UInt32"), "outboundRootCriteria": SimTypePointer(SimStruct({"certData": SimStruct({"size": SimTypeInt(signed=False, label="UInt32"), "data": SimTypePointer(SimTypeChar(label="Byte"), offset=0)}, name="FWP_BYTE_BLOB", pack=False, align=None), "certHash": SimStruct({"size": SimTypeInt(signed=False, label="UInt32"), "data": SimTypePointer(SimTypeChar(label="Byte"), offset=0)}, name="FWP_BYTE_BLOB", pack=False, align=None), "eku": SimTypePointer(SimStruct({"numEku": SimTypeInt(signed=False, label="UInt32"), "eku": SimTypePointer(SimTypePointer(SimTypeChar(label="Byte"), offset=0), offset=0)}, name="IKEEXT_CERT_EKUS0", pack=False, align=None), offset=0), "name": SimTypePointer(SimStruct({"nameType": SimTypeInt(signed=False, label="IKEEXT_CERT_CRITERIA_NAME_TYPE"), "certName": SimTypePointer(SimTypeChar(label="Char"), offset=0)}, name="IKEEXT_CERT_NAME0", pack=False, align=None), offset=0), "flags": SimTypeInt(signed=False, label="UInt32")}, name="IKEEXT_CERTIFICATE_CRITERIA0", pack=False, align=None), offset=0)}, name="_Anonymous1_e__Struct", pack=False, align=None), "Anonymous2": SimStruct({"outboundEnterpriseStoreArraySize": SimTypeInt(signed=False, label="UInt32"), "outboundEnterpriseStoreCriteria": SimTypePointer(SimStruct({"certData": SimStruct({"size": SimTypeInt(signed=False, label="UInt32"), "data": SimTypePointer(SimTypeChar(label="Byte"), offset=0)}, name="FWP_BYTE_BLOB", pack=False, align=None), "certHash": SimStruct({"size": SimTypeInt(signed=False, label="UInt32"), "data": SimTypePointer(SimTypeChar(label="Byte"), offset=0)}, name="FWP_BYTE_BLOB", pack=False, align=None), "eku": SimTypePointer(SimStruct({"numEku": SimTypeInt(signed=False, label="UInt32"), "eku": SimTypePointer(SimTypePointer(SimTypeChar(label="Byte"), offset=0), offset=0)}, name="IKEEXT_CERT_EKUS0", pack=False, align=None), offset=0), "name": SimTypePointer(SimStruct({"nameType": SimTypeInt(signed=False, label="IKEEXT_CERT_CRITERIA_NAME_TYPE"), "certName": SimTypePointer(SimTypeChar(label="Char"), offset=0)}, name="IKEEXT_CERT_NAME0", pack=False, align=None), offset=0), "flags": SimTypeInt(signed=False, label="UInt32")}, name="IKEEXT_CERTIFICATE_CRITERIA0", pack=False, align=None), offset=0)}, name="_Anonymous2_e__Struct", pack=False, align=None), "Anonymous3": SimStruct({"outboundRootStoreArraySize": SimTypeInt(signed=False, label="UInt32"), "outboundTrustedRootStoreCriteria": SimTypePointer(SimStruct({"certData": SimStruct({"size": SimTypeInt(signed=False, label="UInt32"), "data": SimTypePointer(SimTypeChar(label="Byte"), offset=0)}, name="FWP_BYTE_BLOB", pack=False, align=None), "certHash": SimStruct({"size": SimTypeInt(signed=False, label="UInt32"), "data": SimTypePointer(SimTypeChar(label="Byte"), offset=0)}, name="FWP_BYTE_BLOB", pack=False, align=None), "eku": SimTypePointer(SimStruct({"numEku": SimTypeInt(signed=False, label="UInt32"), "eku": SimTypePointer(SimTypePointer(SimTypeChar(label="Byte"), offset=0), offset=0)}, name="IKEEXT_CERT_EKUS0", pack=False, align=None), offset=0), "name": SimTypePointer(SimStruct({"nameType": SimTypeInt(signed=False, label="IKEEXT_CERT_CRITERIA_NAME_TYPE"), "certName": SimTypePointer(SimTypeChar(label="Char"), offset=0)}, name="IKEEXT_CERT_NAME0", pack=False, align=None), offset=0), "flags": SimTypeInt(signed=False, label="UInt32")}, name="IKEEXT_CERTIFICATE_CRITERIA0", pack=False, align=None), offset=0)}, name="_Anonymous3_e__Struct", pack=False, align=None)}, name="<anon>", label="None"), "flags": SimTypeInt(signed=False, label="IKEEXT_CERT_AUTH"), "localCertLocationUrl": SimStruct({"size": SimTypeInt(signed=False, label="UInt32"), "data": SimTypePointer(SimTypeChar(label="Byte"), offset=0)}, name="FWP_BYTE_BLOB", pack=False, align=None)}, name="IKEEXT_CERTIFICATE_AUTHENTICATION2", pack=False, align=None), "cgaAuthentication": SimStruct({"keyContainerName": SimTypePointer(SimTypeChar(label="Char"), offset=0), "cspName": SimTypePointer(SimTypeChar(label="Char"), offset=0), "cspType": SimTypeInt(signed=False, label="UInt32"), "cgaModifier": SimStruct({"byteArray16": SimTypeFixedSizeArray(SimTypeChar(label="Byte"), 16)}, name="FWP_BYTE_ARRAY16", pack=False, align=None), "cgaCollisionCount": SimTypeChar(label="Byte")}, name="IKEEXT_IPV6_CGA_AUTHENTICATION0", pack=False, align=None), "eapAuthentication": SimStruct({"flags": SimTypeInt(signed=False, label="IKEEXT_EAP_AUTHENTICATION_FLAGS")}, name="IKEEXT_EAP_AUTHENTICATION0", pack=False, align=None)}, name="<anon>", label="None")}, name="IKEEXT_AUTHENTICATION_METHOD2", pack=False, align=None), offset=0), "initiatorImpersonationType": SimTypeInt(signed=False, label="IKEEXT_AUTHENTICATION_IMPERSONATION_TYPE")}, name="IKEEXT_EM_POLICY2", pack=False, align=None), offset=0), "fwdPathSaLifetime": SimTypeInt(signed=False, label="UInt32")}, name="IPSEC_TUNNEL_POLICY2", pack=False, align=None), offset=0), "ikeV2QmTransportPolicy": SimTypePointer(SimStruct({"numIpsecProposals": SimTypeInt(signed=False, label="UInt32"), "ipsecProposals": SimTypePointer(SimStruct({"lifetime": SimStruct({"lifetimeSeconds": SimTypeInt(signed=False, label="UInt32"), "lifetimeKilobytes": SimTypeInt(signed=False, label="UInt32"), "lifetimePackets": SimTypeInt(signed=False, label="UInt32")}, name="IPSEC_SA_LIFETIME0", pack=False, align=None), "numSaTransforms": SimTypeInt(signed=False, label="UInt32"), "saTransforms": SimTypePointer(SimStruct({"ipsecTransformType": SimTypeInt(signed=False, label="IPSEC_TRANSFORM_TYPE"), "Anonymous": SimUnion({"ahTransform": SimTypePointer(SimStruct({"authTransformId": SimStruct({"authType": SimTypeInt(signed=False, label="IPSEC_AUTH_TYPE"), "authConfig": SimTypeChar(label="Byte")}, name="IPSEC_AUTH_TRANSFORM_ID0", pack=False, align=None), "cryptoModuleId": SimTypePointer(SimTypeBottom(label="Guid"), offset=0)}, name="IPSEC_AUTH_TRANSFORM0", pack=False, align=None), offset=0), "espAuthTransform": SimTypePointer(SimStruct({"authTransformId": SimStruct({"authType": SimTypeInt(signed=False, label="IPSEC_AUTH_TYPE"), "authConfig": SimTypeChar(label="Byte")}, name="IPSEC_AUTH_TRANSFORM_ID0", pack=False, align=None), "cryptoModuleId": SimTypePointer(SimTypeBottom(label="Guid"), offset=0)}, name="IPSEC_AUTH_TRANSFORM0", pack=False, align=None), offset=0), "espCipherTransform": SimTypePointer(SimStruct({"cipherTransformId": SimStruct({"cipherType": SimTypeInt(signed=False, label="IPSEC_CIPHER_TYPE"), "cipherConfig": SimTypeChar(label="Byte")}, name="IPSEC_CIPHER_TRANSFORM_ID0", pack=False, align=None), "cryptoModuleId": SimTypePointer(SimTypeBottom(label="Guid"), offset=0)}, name="IPSEC_CIPHER_TRANSFORM0", pack=False, align=None), offset=0), "espAuthAndCipherTransform": SimTypePointer(SimStruct({"authTransform": SimStruct({"authTransformId": SimStruct({"authType": SimTypeInt(signed=False, label="IPSEC_AUTH_TYPE"), "authConfig": SimTypeChar(label="Byte")}, name="IPSEC_AUTH_TRANSFORM_ID0", pack=False, align=None), "cryptoModuleId": SimTypePointer(SimTypeBottom(label="Guid"), offset=0)}, name="IPSEC_AUTH_TRANSFORM0", pack=False, align=None), "cipherTransform": SimStruct({"cipherTransformId": SimStruct({"cipherType": SimTypeInt(signed=False, label="IPSEC_CIPHER_TYPE"), "cipherConfig": SimTypeChar(label="Byte")}, name="IPSEC_CIPHER_TRANSFORM_ID0", pack=False, align=None), "cryptoModuleId": SimTypePointer(SimTypeBottom(label="Guid"), offset=0)}, name="IPSEC_CIPHER_TRANSFORM0", pack=False, align=None)}, name="IPSEC_AUTH_AND_CIPHER_TRANSFORM0", pack=False, align=None), offset=0), "espAuthFwTransform": SimTypePointer(SimStruct({"authTransformId": SimStruct({"authType": SimTypeInt(signed=False, label="IPSEC_AUTH_TYPE"), "authConfig": SimTypeChar(label="Byte")}, name="IPSEC_AUTH_TRANSFORM_ID0", pack=False, align=None), "cryptoModuleId": SimTypePointer(SimTypeBottom(label="Guid"), offset=0)}, name="IPSEC_AUTH_TRANSFORM0", pack=False, align=None), offset=0)}, name="<anon>", label="None")}, name="IPSEC_SA_TRANSFORM0", pack=False, align=None), offset=0), "pfsGroup": SimTypeInt(signed=False, label="IPSEC_PFS_GROUP")}, name="IPSEC_PROPOSAL0", pack=False, align=None), offset=0), "flags": SimTypeInt(signed=False, label="IPSEC_POLICY_FLAG"), "ndAllowClearTimeoutSeconds": SimTypeInt(signed=False, label="UInt32"), "saIdleTimeout": SimStruct({"idleTimeoutSeconds": SimTypeInt(signed=False, label="UInt32"), "idleTimeoutSecondsFailOver": SimTypeInt(signed=False, label="UInt32")}, name="IPSEC_SA_IDLE_TIMEOUT0", pack=False, align=None), "emPolicy": SimTypePointer(SimStruct({"numAuthenticationMethods": SimTypeInt(signed=False, label="UInt32"), "authenticationMethods": SimTypePointer(SimStruct({"authenticationMethodType": SimTypeInt(signed=False, label="IKEEXT_AUTHENTICATION_METHOD_TYPE"), "Anonymous": SimUnion({"presharedKeyAuthentication": SimStruct({"presharedKey": SimStruct({"size": SimTypeInt(signed=False, label="UInt32"), "data": SimTypePointer(SimTypeChar(label="Byte"), offset=0)}, name="FWP_BYTE_BLOB", pack=False, align=None), "flags": SimTypeInt(signed=False, label="IKEEXT_PRESHARED_KEY_AUTHENTICATION_FLAGS")}, name="IKEEXT_PRESHARED_KEY_AUTHENTICATION1", pack=False, align=None), "certificateAuthentication": SimStruct({"inboundConfigType": SimTypeInt(signed=False, label="IKEEXT_CERT_CONFIG_TYPE"), "Anonymous1": SimUnion({"Anonymous1": SimStruct({"inboundRootArraySize": SimTypeInt(signed=False, label="UInt32"), "inboundRootCriteria": SimTypePointer(SimStruct({"certData": SimStruct({"size": SimTypeInt(signed=False, label="UInt32"), "data": SimTypePointer(SimTypeChar(label="Byte"), offset=0)}, name="FWP_BYTE_BLOB", pack=False, align=None), "certHash": SimStruct({"size": SimTypeInt(signed=False, label="UInt32"), "data": SimTypePointer(SimTypeChar(label="Byte"), offset=0)}, name="FWP_BYTE_BLOB", pack=False, align=None), "eku": SimTypePointer(SimStruct({"numEku": SimTypeInt(signed=False, label="UInt32"), "eku": SimTypePointer(SimTypePointer(SimTypeChar(label="Byte"), offset=0), offset=0)}, name="IKEEXT_CERT_EKUS0", pack=False, align=None), offset=0), "name": SimTypePointer(SimStruct({"nameType": SimTypeInt(signed=False, label="IKEEXT_CERT_CRITERIA_NAME_TYPE"), "certName": SimTypePointer(SimTypeChar(label="Char"), offset=0)}, name="IKEEXT_CERT_NAME0", pack=False, align=None), offset=0), "flags": SimTypeInt(signed=False, label="UInt32")}, name="IKEEXT_CERTIFICATE_CRITERIA0", pack=False, align=None), offset=0)}, name="_Anonymous1_e__Struct", pack=False, align=None), "Anonymous2": SimStruct({"inboundEnterpriseStoreArraySize": SimTypeInt(signed=False, label="UInt32"), "inboundEnterpriseStoreCriteria": SimTypePointer(SimStruct({"certData": SimStruct({"size": SimTypeInt(signed=False, label="UInt32"), "data": SimTypePointer(SimTypeChar(label="Byte"), offset=0)}, name="FWP_BYTE_BLOB", pack=False, align=None), "certHash": SimStruct({"size": SimTypeInt(signed=False, label="UInt32"), "data": SimTypePointer(SimTypeChar(label="Byte"), offset=0)}, name="FWP_BYTE_BLOB", pack=False, align=None), "eku": SimTypePointer(SimStruct({"numEku": SimTypeInt(signed=False, label="UInt32"), "eku": SimTypePointer(SimTypePointer(SimTypeChar(label="Byte"), offset=0), offset=0)}, name="IKEEXT_CERT_EKUS0", pack=False, align=None), offset=0), "name": SimTypePointer(SimStruct({"nameType": SimTypeInt(signed=False, label="IKEEXT_CERT_CRITERIA_NAME_TYPE"), "certName": SimTypePointer(SimTypeChar(label="Char"), offset=0)}, name="IKEEXT_CERT_NAME0", pack=False, align=None), offset=0), "flags": SimTypeInt(signed=False, label="UInt32")}, name="IKEEXT_CERTIFICATE_CRITERIA0", pack=False, align=None), offset=0)}, name="_Anonymous2_e__Struct", pack=False, align=None), "Anonymous3": SimStruct({"inboundRootStoreArraySize": SimTypeInt(signed=False, label="UInt32"), "inboundTrustedRootStoreCriteria": SimTypePointer(SimStruct({"certData": SimStruct({"size": SimTypeInt(signed=False, label="UInt32"), "data": SimTypePointer(SimTypeChar(label="Byte"), offset=0)}, name="FWP_BYTE_BLOB", pack=False, align=None), "certHash": SimStruct({"size": SimTypeInt(signed=False, label="UInt32"), "data": SimTypePointer(SimTypeChar(label="Byte"), offset=0)}, name="FWP_BYTE_BLOB", pack=False, align=None), "eku": SimTypePointer(SimStruct({"numEku": SimTypeInt(signed=False, label="UInt32"), "eku": SimTypePointer(SimTypePointer(SimTypeChar(label="Byte"), offset=0), offset=0)}, name="IKEEXT_CERT_EKUS0", pack=False, align=None), offset=0), "name": SimTypePointer(SimStruct({"nameType": SimTypeInt(signed=False, label="IKEEXT_CERT_CRITERIA_NAME_TYPE"), "certName": SimTypePointer(SimTypeChar(label="Char"), offset=0)}, name="IKEEXT_CERT_NAME0", pack=False, align=None), offset=0), "flags": SimTypeInt(signed=False, label="UInt32")}, name="IKEEXT_CERTIFICATE_CRITERIA0", pack=False, align=None), offset=0)}, name="_Anonymous3_e__Struct", pack=False, align=None)}, name="<anon>", label="None"), "outboundConfigType": SimTypeInt(signed=False, label="IKEEXT_CERT_CONFIG_TYPE"), "Anonymous2": SimUnion({"Anonymous1": SimStruct({"outboundRootArraySize": SimTypeInt(signed=False, label="UInt32"), "outboundRootCriteria": SimTypePointer(SimStruct({"certData": SimStruct({"size": SimTypeInt(signed=False, label="UInt32"), "data": SimTypePointer(SimTypeChar(label="Byte"), offset=0)}, name="FWP_BYTE_BLOB", pack=False, align=None), "certHash": SimStruct({"size": SimTypeInt(signed=False, label="UInt32"), "data": SimTypePointer(SimTypeChar(label="Byte"), offset=0)}, name="FWP_BYTE_BLOB", pack=False, align=None), "eku": SimTypePointer(SimStruct({"numEku": SimTypeInt(signed=False, label="UInt32"), "eku": SimTypePointer(SimTypePointer(SimTypeChar(label="Byte"), offset=0), offset=0)}, name="IKEEXT_CERT_EKUS0", pack=False, align=None), offset=0), "name": SimTypePointer(SimStruct({"nameType": SimTypeInt(signed=False, label="IKEEXT_CERT_CRITERIA_NAME_TYPE"), "certName": SimTypePointer(SimTypeChar(label="Char"), offset=0)}, name="IKEEXT_CERT_NAME0", pack=False, align=None), offset=0), "flags": SimTypeInt(signed=False, label="UInt32")}, name="IKEEXT_CERTIFICATE_CRITERIA0", pack=False, align=None), offset=0)}, name="_Anonymous1_e__Struct", pack=False, align=None), "Anonymous2": SimStruct({"outboundEnterpriseStoreArraySize": SimTypeInt(signed=False, label="UInt32"), "outboundEnterpriseStoreCriteria": SimTypePointer(SimStruct({"certData": SimStruct({"size": SimTypeInt(signed=False, label="UInt32"), "data": SimTypePointer(SimTypeChar(label="Byte"), offset=0)}, name="FWP_BYTE_BLOB", pack=False, align=None), "certHash": SimStruct({"size": SimTypeInt(signed=False, label="UInt32"), "data": SimTypePointer(SimTypeChar(label="Byte"), offset=0)}, name="FWP_BYTE_BLOB", pack=False, align=None), "eku": SimTypePointer(SimStruct({"numEku": SimTypeInt(signed=False, label="UInt32"), "eku": SimTypePointer(SimTypePointer(SimTypeChar(label="Byte"), offset=0), offset=0)}, name="IKEEXT_CERT_EKUS0", pack=False, align=None), offset=0), "name": SimTypePointer(SimStruct({"nameType": SimTypeInt(signed=False, label="IKEEXT_CERT_CRITERIA_NAME_TYPE"), "certName": SimTypePointer(SimTypeChar(label="Char"), offset=0)}, name="IKEEXT_CERT_NAME0", pack=False, align=None), offset=0), "flags": SimTypeInt(signed=False, label="UInt32")}, name="IKEEXT_CERTIFICATE_CRITERIA0", pack=False, align=None), offset=0)}, name="_Anonymous2_e__Struct", pack=False, align=None), "Anonymous3": SimStruct({"outboundRootStoreArraySize": SimTypeInt(signed=False, label="UInt32"), "outboundTrustedRootStoreCriteria": SimTypePointer(SimStruct({"certData": SimStruct({"size": SimTypeInt(signed=False, label="UInt32"), "data": SimTypePointer(SimTypeChar(label="Byte"), offset=0)}, name="FWP_BYTE_BLOB", pack=False, align=None), "certHash": SimStruct({"size": SimTypeInt(signed=False, label="UInt32"), "data": SimTypePointer(SimTypeChar(label="Byte"), offset=0)}, name="FWP_BYTE_BLOB", pack=False, align=None), "eku": SimTypePointer(SimStruct({"numEku": SimTypeInt(signed=False, label="UInt32"), "eku": SimTypePointer(SimTypePointer(SimTypeChar(label="Byte"), offset=0), offset=0)}, name="IKEEXT_CERT_EKUS0", pack=False, align=None), offset=0), "name": SimTypePointer(SimStruct({"nameType": SimTypeInt(signed=False, label="IKEEXT_CERT_CRITERIA_NAME_TYPE"), "certName": SimTypePointer(SimTypeChar(label="Char"), offset=0)}, name="IKEEXT_CERT_NAME0", pack=False, align=None), offset=0), "flags": SimTypeInt(signed=False, label="UInt32")}, name="IKEEXT_CERTIFICATE_CRITERIA0", pack=False, align=None), offset=0)}, name="_Anonymous3_e__Struct", pack=False, align=None)}, name="<anon>", label="None"), "flags": SimTypeInt(signed=False, label="IKEEXT_CERT_AUTH"), "localCertLocationUrl": SimStruct({"size": SimTypeInt(signed=False, label="UInt32"), "data": SimTypePointer(SimTypeChar(label="Byte"), offset=0)}, name="FWP_BYTE_BLOB", pack=False, align=None)}, name="IKEEXT_CERTIFICATE_AUTHENTICATION2", pack=False, align=None), "kerberosAuthentication": SimStruct({"flags": SimTypeInt(signed=False, label="IKEEXT_KERBEROS_AUTHENTICATION_FLAGS"), "proxyServer": SimTypePointer(SimTypeChar(label="Char"), offset=0)}, name="IKEEXT_KERBEROS_AUTHENTICATION1", pack=False, align=None), "reservedAuthentication": SimStruct({"flags": SimTypeInt(signed=False, label="IKEEXT_RESERVED_AUTHENTICATION_FLAGS")}, name="IKEEXT_RESERVED_AUTHENTICATION0", pack=False, align=None), "ntlmV2Authentication": SimStruct({"flags": SimTypeInt(signed=False, label="UInt32")}, name="IKEEXT_NTLM_V2_AUTHENTICATION0", pack=False, align=None), "sslAuthentication": SimStruct({"inboundConfigType": SimTypeInt(signed=False, label="IKEEXT_CERT_CONFIG_TYPE"), "Anonymous1": SimUnion({"Anonymous1": SimStruct({"inboundRootArraySize": SimTypeInt(signed=False, label="UInt32"), "inboundRootCriteria": SimTypePointer(SimStruct({"certData": SimStruct({"size": SimTypeInt(signed=False, label="UInt32"), "data": SimTypePointer(SimTypeChar(label="Byte"), offset=0)}, name="FWP_BYTE_BLOB", pack=False, align=None), "certHash": SimStruct({"size": SimTypeInt(signed=False, label="UInt32"), "data": SimTypePointer(SimTypeChar(label="Byte"), offset=0)}, name="FWP_BYTE_BLOB", pack=False, align=None), "eku": SimTypePointer(SimStruct({"numEku": SimTypeInt(signed=False, label="UInt32"), "eku": SimTypePointer(SimTypePointer(SimTypeChar(label="Byte"), offset=0), offset=0)}, name="IKEEXT_CERT_EKUS0", pack=False, align=None), offset=0), "name": SimTypePointer(SimStruct({"nameType": SimTypeInt(signed=False, label="IKEEXT_CERT_CRITERIA_NAME_TYPE"), "certName": SimTypePointer(SimTypeChar(label="Char"), offset=0)}, name="IKEEXT_CERT_NAME0", pack=False, align=None), offset=0), "flags": SimTypeInt(signed=False, label="UInt32")}, name="IKEEXT_CERTIFICATE_CRITERIA0", pack=False, align=None), offset=0)}, name="_Anonymous1_e__Struct", pack=False, align=None), "Anonymous2": SimStruct({"inboundEnterpriseStoreArraySize": SimTypeInt(signed=False, label="UInt32"), "inboundEnterpriseStoreCriteria": SimTypePointer(SimStruct({"certData": SimStruct({"size": SimTypeInt(signed=False, label="UInt32"), "data": SimTypePointer(SimTypeChar(label="Byte"), offset=0)}, name="FWP_BYTE_BLOB", pack=False, align=None), "certHash": SimStruct({"size": SimTypeInt(signed=False, label="UInt32"), "data": SimTypePointer(SimTypeChar(label="Byte"), offset=0)}, name="FWP_BYTE_BLOB", pack=False, align=None), "eku": SimTypePointer(SimStruct({"numEku": SimTypeInt(signed=False, label="UInt32"), "eku": SimTypePointer(SimTypePointer(SimTypeChar(label="Byte"), offset=0), offset=0)}, name="IKEEXT_CERT_EKUS0", pack=False, align=None), offset=0), "name": SimTypePointer(SimStruct({"nameType": SimTypeInt(signed=False, label="IKEEXT_CERT_CRITERIA_NAME_TYPE"), "certName": SimTypePointer(SimTypeChar(label="Char"), offset=0)}, name="IKEEXT_CERT_NAME0", pack=False, align=None), offset=0), "flags": SimTypeInt(signed=False, label="UInt32")}, name="IKEEXT_CERTIFICATE_CRITERIA0", pack=False, align=None), offset=0)}, name="_Anonymous2_e__Struct", pack=False, align=None), "Anonymous3": SimStruct({"inboundRootStoreArraySize": SimTypeInt(signed=False, label="UInt32"), "inboundTrustedRootStoreCriteria": SimTypePointer(SimStruct({"certData": SimStruct({"size": SimTypeInt(signed=False, label="UInt32"), "data": SimTypePointer(SimTypeChar(label="Byte"), offset=0)}, name="FWP_BYTE_BLOB", pack=False, align=None), "certHash": SimStruct({"size": SimTypeInt(signed=False, label="UInt32"), "data": SimTypePointer(SimTypeChar(label="Byte"), offset=0)}, name="FWP_BYTE_BLOB", pack=False, align=None), "eku": SimTypePointer(SimStruct({"numEku": SimTypeInt(signed=False, label="UInt32"), "eku": SimTypePointer(SimTypePointer(SimTypeChar(label="Byte"), offset=0), offset=0)}, name="IKEEXT_CERT_EKUS0", pack=False, align=None), offset=0), "name": SimTypePointer(SimStruct({"nameType": SimTypeInt(signed=False, label="IKEEXT_CERT_CRITERIA_NAME_TYPE"), "certName": SimTypePointer(SimTypeChar(label="Char"), offset=0)}, name="IKEEXT_CERT_NAME0", pack=False, align=None), offset=0), "flags": SimTypeInt(signed=False, label="UInt32")}, name="IKEEXT_CERTIFICATE_CRITERIA0", pack=False, align=None), offset=0)}, name="_Anonymous3_e__Struct", pack=False, align=None)}, name="<anon>", label="None"), "outboundConfigType": SimTypeInt(signed=False, label="IKEEXT_CERT_CONFIG_TYPE"), "Anonymous2": SimUnion({"Anonymous1": SimStruct({"outboundRootArraySize": SimTypeInt(signed=False, label="UInt32"), "outboundRootCriteria": SimTypePointer(SimStruct({"certData": SimStruct({"size": SimTypeInt(signed=False, label="UInt32"), "data": SimTypePointer(SimTypeChar(label="Byte"), offset=0)}, name="FWP_BYTE_BLOB", pack=False, align=None), "certHash": SimStruct({"size": SimTypeInt(signed=False, label="UInt32"), "data": SimTypePointer(SimTypeChar(label="Byte"), offset=0)}, name="FWP_BYTE_BLOB", pack=False, align=None), "eku": SimTypePointer(SimStruct({"numEku": SimTypeInt(signed=False, label="UInt32"), "eku": SimTypePointer(SimTypePointer(SimTypeChar(label="Byte"), offset=0), offset=0)}, name="IKEEXT_CERT_EKUS0", pack=False, align=None), offset=0), "name": SimTypePointer(SimStruct({"nameType": SimTypeInt(signed=False, label="IKEEXT_CERT_CRITERIA_NAME_TYPE"), "certName": SimTypePointer(SimTypeChar(label="Char"), offset=0)}, name="IKEEXT_CERT_NAME0", pack=False, align=None), offset=0), "flags": SimTypeInt(signed=False, label="UInt32")}, name="IKEEXT_CERTIFICATE_CRITERIA0", pack=False, align=None), offset=0)}, name="_Anonymous1_e__Struct", pack=False, align=None), "Anonymous2": SimStruct({"outboundEnterpriseStoreArraySize": SimTypeInt(signed=False, label="UInt32"), "outboundEnterpriseStoreCriteria": SimTypePointer(SimStruct({"certData": SimStruct({"size": SimTypeInt(signed=False, label="UInt32"), "data": SimTypePointer(SimTypeChar(label="Byte"), offset=0)}, name="FWP_BYTE_BLOB", pack=False, align=None), "certHash": SimStruct({"size": SimTypeInt(signed=False, label="UInt32"), "data": SimTypePointer(SimTypeChar(label="Byte"), offset=0)}, name="FWP_BYTE_BLOB", pack=False, align=None), "eku": SimTypePointer(SimStruct({"numEku": SimTypeInt(signed=False, label="UInt32"), "eku": SimTypePointer(SimTypePointer(SimTypeChar(label="Byte"), offset=0), offset=0)}, name="IKEEXT_CERT_EKUS0", pack=False, align=None), offset=0), "name": SimTypePointer(SimStruct({"nameType": SimTypeInt(signed=False, label="IKEEXT_CERT_CRITERIA_NAME_TYPE"), "certName": SimTypePointer(SimTypeChar(label="Char"), offset=0)}, name="IKEEXT_CERT_NAME0", pack=False, align=None), offset=0), "flags": SimTypeInt(signed=False, label="UInt32")}, name="IKEEXT_CERTIFICATE_CRITERIA0", pack=False, align=None), offset=0)}, name="_Anonymous2_e__Struct", pack=False, align=None), "Anonymous3": SimStruct({"outboundRootStoreArraySize": SimTypeInt(signed=False, label="UInt32"), "outboundTrustedRootStoreCriteria": SimTypePointer(SimStruct({"certData": SimStruct({"size": SimTypeInt(signed=False, label="UInt32"), "data": SimTypePointer(SimTypeChar(label="Byte"), offset=0)}, name="FWP_BYTE_BLOB", pack=False, align=None), "certHash": SimStruct({"size": SimTypeInt(signed=False, label="UInt32"), "data": SimTypePointer(SimTypeChar(label="Byte"), offset=0)}, name="FWP_BYTE_BLOB", pack=False, align=None), "eku": SimTypePointer(SimStruct({"numEku": SimTypeInt(signed=False, label="UInt32"), "eku": SimTypePointer(SimTypePointer(SimTypeChar(label="Byte"), offset=0), offset=0)}, name="IKEEXT_CERT_EKUS0", pack=False, align=None), offset=0), "name": SimTypePointer(SimStruct({"nameType": SimTypeInt(signed=False, label="IKEEXT_CERT_CRITERIA_NAME_TYPE"), "certName": SimTypePointer(SimTypeChar(label="Char"), offset=0)}, name="IKEEXT_CERT_NAME0", pack=False, align=None), offset=0), "flags": SimTypeInt(signed=False, label="UInt32")}, name="IKEEXT_CERTIFICATE_CRITERIA0", pack=False, align=None), offset=0)}, name="_Anonymous3_e__Struct", pack=False, align=None)}, name="<anon>", label="None"), "flags": SimTypeInt(signed=False, label="IKEEXT_CERT_AUTH"), "localCertLocationUrl": SimStruct({"size": SimTypeInt(signed=False, label="UInt32"), "data": SimTypePointer(SimTypeChar(label="Byte"), offset=0)}, name="FWP_BYTE_BLOB", pack=False, align=None)}, name="IKEEXT_CERTIFICATE_AUTHENTICATION2", pack=False, align=None), "cgaAuthentication": SimStruct({"keyContainerName": SimTypePointer(SimTypeChar(label="Char"), offset=0), "cspName": SimTypePointer(SimTypeChar(label="Char"), offset=0), "cspType": SimTypeInt(signed=False, label="UInt32"), "cgaModifier": SimStruct({"byteArray16": SimTypeFixedSizeArray(SimTypeChar(label="Byte"), 16)}, name="FWP_BYTE_ARRAY16", pack=False, align=None), "cgaCollisionCount": SimTypeChar(label="Byte")}, name="IKEEXT_IPV6_CGA_AUTHENTICATION0", pack=False, align=None), "eapAuthentication": SimStruct({"flags": SimTypeInt(signed=False, label="IKEEXT_EAP_AUTHENTICATION_FLAGS")}, name="IKEEXT_EAP_AUTHENTICATION0", pack=False, align=None)}, name="<anon>", label="None")}, name="IKEEXT_AUTHENTICATION_METHOD2", pack=False, align=None), offset=0), "initiatorImpersonationType": SimTypeInt(signed=False, label="IKEEXT_AUTHENTICATION_IMPERSONATION_TYPE")}, name="IKEEXT_EM_POLICY2", pack=False, align=None), offset=0)}, name="IPSEC_TRANSPORT_POLICY2", pack=False, align=None), offset=0), "ikeV2MmPolicy": SimTypePointer(SimStruct({"softExpirationTime": SimTypeInt(signed=False, label="UInt32"), "numAuthenticationMethods": SimTypeInt(signed=False, label="UInt32"), "authenticationMethods": SimTypePointer(SimStruct({"authenticationMethodType": SimTypeInt(signed=False, label="IKEEXT_AUTHENTICATION_METHOD_TYPE"), "Anonymous": SimUnion({"presharedKeyAuthentication": SimStruct({"presharedKey": SimStruct({"size": SimTypeInt(signed=False, label="UInt32"), "data": SimTypePointer(SimTypeChar(label="Byte"), offset=0)}, name="FWP_BYTE_BLOB", pack=False, align=None), "flags": SimTypeInt(signed=False, label="IKEEXT_PRESHARED_KEY_AUTHENTICATION_FLAGS")}, name="IKEEXT_PRESHARED_KEY_AUTHENTICATION1", pack=False, align=None), "certificateAuthentication": SimStruct({"inboundConfigType": SimTypeInt(signed=False, label="IKEEXT_CERT_CONFIG_TYPE"), "Anonymous1": SimUnion({"Anonymous1": SimStruct({"inboundRootArraySize": SimTypeInt(signed=False, label="UInt32"), "inboundRootCriteria": SimTypePointer(SimStruct({"certData": SimStruct({"size": SimTypeInt(signed=False, label="UInt32"), "data": SimTypePointer(SimTypeChar(label="Byte"), offset=0)}, name="FWP_BYTE_BLOB", pack=False, align=None), "certHash": SimStruct({"size": SimTypeInt(signed=False, label="UInt32"), "data": SimTypePointer(SimTypeChar(label="Byte"), offset=0)}, name="FWP_BYTE_BLOB", pack=False, align=None), "eku": SimTypePointer(SimStruct({"numEku": SimTypeInt(signed=False, label="UInt32"), "eku": SimTypePointer(SimTypePointer(SimTypeChar(label="Byte"), offset=0), offset=0)}, name="IKEEXT_CERT_EKUS0", pack=False, align=None), offset=0), "name": SimTypePointer(SimStruct({"nameType": SimTypeInt(signed=False, label="IKEEXT_CERT_CRITERIA_NAME_TYPE"), "certName": SimTypePointer(SimTypeChar(label="Char"), offset=0)}, name="IKEEXT_CERT_NAME0", pack=False, align=None), offset=0), "flags": SimTypeInt(signed=False, label="UInt32")}, name="IKEEXT_CERTIFICATE_CRITERIA0", pack=False, align=None), offset=0)}, name="_Anonymous1_e__Struct", pack=False, align=None), "Anonymous2": SimStruct({"inboundEnterpriseStoreArraySize": SimTypeInt(signed=False, label="UInt32"), "inboundEnterpriseStoreCriteria": SimTypePointer(SimStruct({"certData": SimStruct({"size": SimTypeInt(signed=False, label="UInt32"), "data": SimTypePointer(SimTypeChar(label="Byte"), offset=0)}, name="FWP_BYTE_BLOB", pack=False, align=None), "certHash": SimStruct({"size": SimTypeInt(signed=False, label="UInt32"), "data": SimTypePointer(SimTypeChar(label="Byte"), offset=0)}, name="FWP_BYTE_BLOB", pack=False, align=None), "eku": SimTypePointer(SimStruct({"numEku": SimTypeInt(signed=False, label="UInt32"), "eku": SimTypePointer(SimTypePointer(SimTypeChar(label="Byte"), offset=0), offset=0)}, name="IKEEXT_CERT_EKUS0", pack=False, align=None), offset=0), "name": SimTypePointer(SimStruct({"nameType": SimTypeInt(signed=False, label="IKEEXT_CERT_CRITERIA_NAME_TYPE"), "certName": SimTypePointer(SimTypeChar(label="Char"), offset=0)}, name="IKEEXT_CERT_NAME0", pack=False, align=None), offset=0), "flags": SimTypeInt(signed=False, label="UInt32")}, name="IKEEXT_CERTIFICATE_CRITERIA0", pack=False, align=None), offset=0)}, name="_Anonymous2_e__Struct", pack=False, align=None), "Anonymous3": SimStruct({"inboundRootStoreArraySize": SimTypeInt(signed=False, label="UInt32"), "inboundTrustedRootStoreCriteria": SimTypePointer(SimStruct({"certData": SimStruct({"size": SimTypeInt(signed=False, label="UInt32"), "data": SimTypePointer(SimTypeChar(label="Byte"), offset=0)}, name="FWP_BYTE_BLOB", pack=False, align=None), "certHash": SimStruct({"size": SimTypeInt(signed=False, label="UInt32"), "data": SimTypePointer(SimTypeChar(label="Byte"), offset=0)}, name="FWP_BYTE_BLOB", pack=False, align=None), "eku": SimTypePointer(SimStruct({"numEku": SimTypeInt(signed=False, label="UInt32"), "eku": SimTypePointer(SimTypePointer(SimTypeChar(label="Byte"), offset=0), offset=0)}, name="IKEEXT_CERT_EKUS0", pack=False, align=None), offset=0), "name": SimTypePointer(SimStruct({"nameType": SimTypeInt(signed=False, label="IKEEXT_CERT_CRITERIA_NAME_TYPE"), "certName": SimTypePointer(SimTypeChar(label="Char"), offset=0)}, name="IKEEXT_CERT_NAME0", pack=False, align=None), offset=0), "flags": SimTypeInt(signed=False, label="UInt32")}, name="IKEEXT_CERTIFICATE_CRITERIA0", pack=False, align=None), offset=0)}, name="_Anonymous3_e__Struct", pack=False, align=None)}, name="<anon>", label="None"), "outboundConfigType": SimTypeInt(signed=False, label="IKEEXT_CERT_CONFIG_TYPE"), "Anonymous2": SimUnion({"Anonymous1": SimStruct({"outboundRootArraySize": SimTypeInt(signed=False, label="UInt32"), "outboundRootCriteria": SimTypePointer(SimStruct({"certData": SimStruct({"size": SimTypeInt(signed=False, label="UInt32"), "data": SimTypePointer(SimTypeChar(label="Byte"), offset=0)}, name="FWP_BYTE_BLOB", pack=False, align=None), "certHash": SimStruct({"size": SimTypeInt(signed=False, label="UInt32"), "data": SimTypePointer(SimTypeChar(label="Byte"), offset=0)}, name="FWP_BYTE_BLOB", pack=False, align=None), "eku": SimTypePointer(SimStruct({"numEku": SimTypeInt(signed=False, label="UInt32"), "eku": SimTypePointer(SimTypePointer(SimTypeChar(label="Byte"), offset=0), offset=0)}, name="IKEEXT_CERT_EKUS0", pack=False, align=None), offset=0), "name": SimTypePointer(SimStruct({"nameType": SimTypeInt(signed=False, label="IKEEXT_CERT_CRITERIA_NAME_TYPE"), "certName": SimTypePointer(SimTypeChar(label="Char"), offset=0)}, name="IKEEXT_CERT_NAME0", pack=False, align=None), offset=0), "flags": SimTypeInt(signed=False, label="UInt32")}, name="IKEEXT_CERTIFICATE_CRITERIA0", pack=False, align=None), offset=0)}, name="_Anonymous1_e__Struct", pack=False, align=None), "Anonymous2": SimStruct({"outboundEnterpriseStoreArraySize": SimTypeInt(signed=False, label="UInt32"), "outboundEnterpriseStoreCriteria": SimTypePointer(SimStruct({"certData": SimStruct({"size": SimTypeInt(signed=False, label="UInt32"), "data": SimTypePointer(SimTypeChar(label="Byte"), offset=0)}, name="FWP_BYTE_BLOB", pack=False, align=None), "certHash": SimStruct({"size": SimTypeInt(signed=False, label="UInt32"), "data": SimTypePointer(SimTypeChar(label="Byte"), offset=0)}, name="FWP_BYTE_BLOB", pack=False, align=None), "eku": SimTypePointer(SimStruct({"numEku": SimTypeInt(signed=False, label="UInt32"), "eku": SimTypePointer(SimTypePointer(SimTypeChar(label="Byte"), offset=0), offset=0)}, name="IKEEXT_CERT_EKUS0", pack=False, align=None), offset=0), "name": SimTypePointer(SimStruct({"nameType": SimTypeInt(signed=False, label="IKEEXT_CERT_CRITERIA_NAME_TYPE"), "certName": SimTypePointer(SimTypeChar(label="Char"), offset=0)}, name="IKEEXT_CERT_NAME0", pack=False, align=None), offset=0), "flags": SimTypeInt(signed=False, label="UInt32")}, name="IKEEXT_CERTIFICATE_CRITERIA0", pack=False, align=None), offset=0)}, name="_Anonymous2_e__Struct", pack=False, align=None), "Anonymous3": SimStruct({"outboundRootStoreArraySize": SimTypeInt(signed=False, label="UInt32"), "outboundTrustedRootStoreCriteria": SimTypePointer(SimStruct({"certData": SimStruct({"size": SimTypeInt(signed=False, label="UInt32"), "data": SimTypePointer(SimTypeChar(label="Byte"), offset=0)}, name="FWP_BYTE_BLOB", pack=False, align=None), "certHash": SimStruct({"size": SimTypeInt(signed=False, label="UInt32"), "data": SimTypePointer(SimTypeChar(label="Byte"), offset=0)}, name="FWP_BYTE_BLOB", pack=False, align=None), "eku": SimTypePointer(SimStruct({"numEku": SimTypeInt(signed=False, label="UInt32"), "eku": SimTypePointer(SimTypePointer(SimTypeChar(label="Byte"), offset=0), offset=0)}, name="IKEEXT_CERT_EKUS0", pack=False, align=None), offset=0), "name": SimTypePointer(SimStruct({"nameType": SimTypeInt(signed=False, label="IKEEXT_CERT_CRITERIA_NAME_TYPE"), "certName": SimTypePointer(SimTypeChar(label="Char"), offset=0)}, name="IKEEXT_CERT_NAME0", pack=False, align=None), offset=0), "flags": SimTypeInt(signed=False, label="UInt32")}, name="IKEEXT_CERTIFICATE_CRITERIA0", pack=False, align=None), offset=0)}, name="_Anonymous3_e__Struct", pack=False, align=None)}, name="<anon>", label="None"), "flags": SimTypeInt(signed=False, label="IKEEXT_CERT_AUTH"), "localCertLocationUrl": SimStruct({"size": SimTypeInt(signed=False, label="UInt32"), "data": SimTypePointer(SimTypeChar(label="Byte"), offset=0)}, name="FWP_BYTE_BLOB", pack=False, align=None)}, name="IKEEXT_CERTIFICATE_AUTHENTICATION2", pack=False, align=None), "kerberosAuthentication": SimStruct({"flags": SimTypeInt(signed=False, label="IKEEXT_KERBEROS_AUTHENTICATION_FLAGS"), "proxyServer": SimTypePointer(SimTypeChar(label="Char"), offset=0)}, name="IKEEXT_KERBEROS_AUTHENTICATION1", pack=False, align=None), "reservedAuthentication": SimStruct({"flags": SimTypeInt(signed=False, label="IKEEXT_RESERVED_AUTHENTICATION_FLAGS")}, name="IKEEXT_RESERVED_AUTHENTICATION0", pack=False, align=None), "ntlmV2Authentication": SimStruct({"flags": SimTypeInt(signed=False, label="UInt32")}, name="IKEEXT_NTLM_V2_AUTHENTICATION0", pack=False, align=None), "sslAuthentication": SimStruct({"inboundConfigType": SimTypeInt(signed=False, label="IKEEXT_CERT_CONFIG_TYPE"), "Anonymous1": SimUnion({"Anonymous1": SimStruct({"inboundRootArraySize": SimTypeInt(signed=False, label="UInt32"), "inboundRootCriteria": SimTypePointer(SimStruct({"certData": SimStruct({"size": SimTypeInt(signed=False, label="UInt32"), "data": SimTypePointer(SimTypeChar(label="Byte"), offset=0)}, name="FWP_BYTE_BLOB", pack=False, align=None), "certHash": SimStruct({"size": SimTypeInt(signed=False, label="UInt32"), "data": SimTypePointer(SimTypeChar(label="Byte"), offset=0)}, name="FWP_BYTE_BLOB", pack=False, align=None), "eku": SimTypePointer(SimStruct({"numEku": SimTypeInt(signed=False, label="UInt32"), "eku": SimTypePointer(SimTypePointer(SimTypeChar(label="Byte"), offset=0), offset=0)}, name="IKEEXT_CERT_EKUS0", pack=False, align=None), offset=0), "name": SimTypePointer(SimStruct({"nameType": SimTypeInt(signed=False, label="IKEEXT_CERT_CRITERIA_NAME_TYPE"), "certName": SimTypePointer(SimTypeChar(label="Char"), offset=0)}, name="IKEEXT_CERT_NAME0", pack=False, align=None), offset=0), "flags": SimTypeInt(signed=False, label="UInt32")}, name="IKEEXT_CERTIFICATE_CRITERIA0", pack=False, align=None), offset=0)}, name="_Anonymous1_e__Struct", pack=False, align=None), "Anonymous2": SimStruct({"inboundEnterpriseStoreArraySize": SimTypeInt(signed=False, label="UInt32"), "inboundEnterpriseStoreCriteria": SimTypePointer(SimStruct({"certData": SimStruct({"size": SimTypeInt(signed=False, label="UInt32"), "data": SimTypePointer(SimTypeChar(label="Byte"), offset=0)}, name="FWP_BYTE_BLOB", pack=False, align=None), "certHash": SimStruct({"size": SimTypeInt(signed=False, label="UInt32"), "data": SimTypePointer(SimTypeChar(label="Byte"), offset=0)}, name="FWP_BYTE_BLOB", pack=False, align=None), "eku": SimTypePointer(SimStruct({"numEku": SimTypeInt(signed=False, label="UInt32"), "eku": SimTypePointer(SimTypePointer(SimTypeChar(label="Byte"), offset=0), offset=0)}, name="IKEEXT_CERT_EKUS0", pack=False, align=None), offset=0), "name": SimTypePointer(SimStruct({"nameType": SimTypeInt(signed=False, label="IKEEXT_CERT_CRITERIA_NAME_TYPE"), "certName": SimTypePointer(SimTypeChar(label="Char"), offset=0)}, name="IKEEXT_CERT_NAME0", pack=False, align=None), offset=0), "flags": SimTypeInt(signed=False, label="UInt32")}, name="IKEEXT_CERTIFICATE_CRITERIA0", pack=False, align=None), offset=0)}, name="_Anonymous2_e__Struct", pack=False, align=None), "Anonymous3": SimStruct({"inboundRootStoreArraySize": SimTypeInt(signed=False, label="UInt32"), "inboundTrustedRootStoreCriteria": SimTypePointer(SimStruct({"certData": SimStruct({"size": SimTypeInt(signed=False, label="UInt32"), "data": SimTypePointer(SimTypeChar(label="Byte"), offset=0)}, name="FWP_BYTE_BLOB", pack=False, align=None), "certHash": SimStruct({"size": SimTypeInt(signed=False, label="UInt32"), "data": SimTypePointer(SimTypeChar(label="Byte"), offset=0)}, name="FWP_BYTE_BLOB", pack=False, align=None), "eku": SimTypePointer(SimStruct({"numEku": SimTypeInt(signed=False, label="UInt32"), "eku": SimTypePointer(SimTypePointer(SimTypeChar(label="Byte"), offset=0), offset=0)}, name="IKEEXT_CERT_EKUS0", pack=False, align=None), offset=0), "name": SimTypePointer(SimStruct({"nameType": SimTypeInt(signed=False, label="IKEEXT_CERT_CRITERIA_NAME_TYPE"), "certName": SimTypePointer(SimTypeChar(label="Char"), offset=0)}, name="IKEEXT_CERT_NAME0", pack=False, align=None), offset=0), "flags": SimTypeInt(signed=False, label="UInt32")}, name="IKEEXT_CERTIFICATE_CRITERIA0", pack=False, align=None), offset=0)}, name="_Anonymous3_e__Struct", pack=False, align=None)}, name="<anon>", label="None"), "outboundConfigType": SimTypeInt(signed=False, label="IKEEXT_CERT_CONFIG_TYPE"), "Anonymous2": SimUnion({"Anonymous1": SimStruct({"outboundRootArraySize": SimTypeInt(signed=False, label="UInt32"), "outboundRootCriteria": SimTypePointer(SimStruct({"certData": SimStruct({"size": SimTypeInt(signed=False, label="UInt32"), "data": SimTypePointer(SimTypeChar(label="Byte"), offset=0)}, name="FWP_BYTE_BLOB", pack=False, align=None), "certHash": SimStruct({"size": SimTypeInt(signed=False, label="UInt32"), "data": SimTypePointer(SimTypeChar(label="Byte"), offset=0)}, name="FWP_BYTE_BLOB", pack=False, align=None), "eku": SimTypePointer(SimStruct({"numEku": SimTypeInt(signed=False, label="UInt32"), "eku": SimTypePointer(SimTypePointer(SimTypeChar(label="Byte"), offset=0), offset=0)}, name="IKEEXT_CERT_EKUS0", pack=False, align=None), offset=0), "name": SimTypePointer(SimStruct({"nameType": SimTypeInt(signed=False, label="IKEEXT_CERT_CRITERIA_NAME_TYPE"), "certName": SimTypePointer(SimTypeChar(label="Char"), offset=0)}, name="IKEEXT_CERT_NAME0", pack=False, align=None), offset=0), "flags": SimTypeInt(signed=False, label="UInt32")}, name="IKEEXT_CERTIFICATE_CRITERIA0", pack=False, align=None), offset=0)}, name="_Anonymous1_e__Struct", pack=False, align=None), "Anonymous2": SimStruct({"outboundEnterpriseStoreArraySize": SimTypeInt(signed=False, label="UInt32"), "outboundEnterpriseStoreCriteria": SimTypePointer(SimStruct({"certData": SimStruct({"size": SimTypeInt(signed=False, label="UInt32"), "data": SimTypePointer(SimTypeChar(label="Byte"), offset=0)}, name="FWP_BYTE_BLOB", pack=False, align=None), "certHash": SimStruct({"size": SimTypeInt(signed=False, label="UInt32"), "data": SimTypePointer(SimTypeChar(label="Byte"), offset=0)}, name="FWP_BYTE_BLOB", pack=False, align=None), "eku": SimTypePointer(SimStruct({"numEku": SimTypeInt(signed=False, label="UInt32"), "eku": SimTypePointer(SimTypePointer(SimTypeChar(label="Byte"), offset=0), offset=0)}, name="IKEEXT_CERT_EKUS0", pack=False, align=None), offset=0), "name": SimTypePointer(SimStruct({"nameType": SimTypeInt(signed=False, label="IKEEXT_CERT_CRITERIA_NAME_TYPE"), "certName": SimTypePointer(SimTypeChar(label="Char"), offset=0)}, name="IKEEXT_CERT_NAME0", pack=False, align=None), offset=0), "flags": SimTypeInt(signed=False, label="UInt32")}, name="IKEEXT_CERTIFICATE_CRITERIA0", pack=False, align=None), offset=0)}, name="_Anonymous2_e__Struct", pack=False, align=None), "Anonymous3": SimStruct({"outboundRootStoreArraySize": SimTypeInt(signed=False, label="UInt32"), "outboundTrustedRootStoreCriteria": SimTypePointer(SimStruct({"certData": SimStruct({"size": SimTypeInt(signed=False, label="UInt32"), "data": SimTypePointer(SimTypeChar(label="Byte"), offset=0)}, name="FWP_BYTE_BLOB", pack=False, align=None), "certHash": SimStruct({"size": SimTypeInt(signed=False, label="UInt32"), "data": SimTypePointer(SimTypeChar(label="Byte"), offset=0)}, name="FWP_BYTE_BLOB", pack=False, align=None), "eku": SimTypePointer(SimStruct({"numEku": SimTypeInt(signed=False, label="UInt32"), "eku": SimTypePointer(SimTypePointer(SimTypeChar(label="Byte"), offset=0), offset=0)}, name="IKEEXT_CERT_EKUS0", pack=False, align=None), offset=0), "name": SimTypePointer(SimStruct({"nameType": SimTypeInt(signed=False, label="IKEEXT_CERT_CRITERIA_NAME_TYPE"), "certName": SimTypePointer(SimTypeChar(label="Char"), offset=0)}, name="IKEEXT_CERT_NAME0", pack=False, align=None), offset=0), "flags": SimTypeInt(signed=False, label="UInt32")}, name="IKEEXT_CERTIFICATE_CRITERIA0", pack=False, align=None), offset=0)}, name="_Anonymous3_e__Struct", pack=False, align=None)}, name="<anon>", label="None"), "flags": SimTypeInt(signed=False, label="IKEEXT_CERT_AUTH"), "localCertLocationUrl": SimStruct({"size": SimTypeInt(signed=False, label="UInt32"), "data": SimTypePointer(SimTypeChar(label="Byte"), offset=0)}, name="FWP_BYTE_BLOB", pack=False, align=None)}, name="IKEEXT_CERTIFICATE_AUTHENTICATION2", pack=False, align=None), "cgaAuthentication": SimStruct({"keyContainerName": SimTypePointer(SimTypeChar(label="Char"), offset=0), "cspName": SimTypePointer(SimTypeChar(label="Char"), offset=0), "cspType": SimTypeInt(signed=False, label="UInt32"), "cgaModifier": SimStruct({"byteArray16": SimTypeFixedSizeArray(SimTypeChar(label="Byte"), 16)}, name="FWP_BYTE_ARRAY16", pack=False, align=None), "cgaCollisionCount": SimTypeChar(label="Byte")}, name="IKEEXT_IPV6_CGA_AUTHENTICATION0", pack=False, align=None), "eapAuthentication": SimStruct({"flags": SimTypeInt(signed=False, label="IKEEXT_EAP_AUTHENTICATION_FLAGS")}, name="IKEEXT_EAP_AUTHENTICATION0", pack=False, align=None)}, name="<anon>", label="None")}, name="IKEEXT_AUTHENTICATION_METHOD2", pack=False, align=None), offset=0), "initiatorImpersonationType": SimTypeInt(signed=False, label="IKEEXT_AUTHENTICATION_IMPERSONATION_TYPE"), "numIkeProposals": SimTypeInt(signed=False, label="UInt32"), "ikeProposals": SimTypePointer(SimStruct({"cipherAlgorithm": SimStruct({"algoIdentifier": SimTypeInt(signed=False, label="IKEEXT_CIPHER_TYPE"), "keyLen": SimTypeInt(signed=False, label="UInt32"), "rounds": SimTypeInt(signed=False, label="UInt32")}, name="IKEEXT_CIPHER_ALGORITHM0", pack=False, align=None), "integrityAlgorithm": SimStruct({"algoIdentifier": SimTypeInt(signed=False, label="IKEEXT_INTEGRITY_TYPE")}, name="IKEEXT_INTEGRITY_ALGORITHM0", pack=False, align=None), "maxLifetimeSeconds": SimTypeInt(signed=False, label="UInt32"), "dhGroup": SimTypeInt(signed=False, label="IKEEXT_DH_GROUP"), "quickModeLimit": SimTypeInt(signed=False, label="UInt32")}, name="IKEEXT_PROPOSAL0", pack=False, align=None), offset=0), "flags": SimTypeInt(signed=False, label="IKEEXT_POLICY_FLAG"), "maxDynamicFilters": SimTypeInt(signed=False, label="UInt32"), "retransmitDurationSecs": SimTypeInt(signed=False, label="UInt32")}, name="IKEEXT_POLICY2", pack=False, align=None), offset=0), "idpOptions": SimTypePointer(SimStruct({"stateIdleTimeoutSeconds": SimTypeInt(signed=False, label="UInt32"), "perIPRateLimitQueueIdleTimeoutSeconds": SimTypeInt(signed=False, label="UInt32"), "ipV6IPsecUnauthDscp": SimTypeChar(label="Byte"), "ipV6IPsecUnauthRateLimitBytesPerSec": SimTypeInt(signed=False, label="UInt32"), "ipV6IPsecUnauthPerIPRateLimitBytesPerSec": SimTypeInt(signed=False, label="UInt32"), "ipV6IPsecAuthDscp": SimTypeChar(label="Byte"), "ipV6IPsecAuthRateLimitBytesPerSec": SimTypeInt(signed=False, label="UInt32"), "icmpV6Dscp": SimTypeChar(label="Byte"), "icmpV6RateLimitBytesPerSec": SimTypeInt(signed=False, label="UInt32"), "ipV6FilterExemptDscp": SimTypeChar(label="Byte"), "ipV6FilterExemptRateLimitBytesPerSec": SimTypeInt(signed=False, label="UInt32"), "defBlockExemptDscp": SimTypeChar(label="Byte"), "defBlockExemptRateLimitBytesPerSec": SimTypeInt(signed=False, label="UInt32"), "maxStateEntries": SimTypeInt(signed=False, label="UInt32"), "maxPerIPRateLimitQueues": SimTypeInt(signed=False, label="UInt32"), "flags": SimTypeInt(signed=False, label="IPSEC_DOSP_FLAGS"), "numPublicIFLuids": SimTypeInt(signed=False, label="UInt32"), "publicIFLuids": SimTypePointer(SimTypeLongLong(signed=False, label="UInt64"), offset=0), "numInternalIFLuids": SimTypeInt(signed=False, label="UInt32"), "internalIFLuids": SimTypePointer(SimTypeLongLong(signed=False, label="UInt64"), offset=0), "publicV6AddrMask": SimStruct({"addr": SimTypeFixedSizeArray(SimTypeChar(label="Byte"), 16), "prefixLength": SimTypeChar(label="Byte")}, name="FWP_V6_ADDR_AND_MASK", pack=False, align=None), "internalV6AddrMask": SimStruct({"addr": SimTypeFixedSizeArray(SimTypeChar(label="Byte"), 16), "prefixLength": SimTypeChar(label="Byte")}, name="FWP_V6_ADDR_AND_MASK", pack=False, align=None)}, name="IPSEC_DOSP_OPTIONS0", pack=False, align=None), offset=0)}, name="<anon>", label="None"), "providerContextId": SimTypeLongLong(signed=False, label="UInt64")}, name="FWPM_PROVIDER_CONTEXT2", pack=False, align=None), offset=0), offset=0), offset=0), SimTypePointer(SimTypeInt(signed=False, label="UInt32"), offset=0)], SimTypeInt(signed=False, label="UInt32"), arg_names=["engineHandle", "enumHandle", "numEntriesRequested", "entries", "numEntriesReturned"]),
        # 
        'FwpmProviderContextEnum3': SimTypeFunction([SimTypePointer(SimTypeInt(signed=True, label="Int"), label="IntPtr", offset=0), SimTypePointer(SimTypeInt(signed=True, label="Int"), label="IntPtr", offset=0), SimTypeInt(signed=False, label="UInt32"), SimTypePointer(SimTypePointer(SimTypePointer(SimStruct({"providerContextKey": SimTypeBottom(label="Guid"), "displayData": SimStruct({"name": SimTypePointer(SimTypeChar(label="Char"), offset=0), "description": SimTypePointer(SimTypeChar(label="Char"), offset=0)}, name="FWPM_DISPLAY_DATA0", pack=False, align=None), "flags": SimTypeInt(signed=False, label="UInt32"), "providerKey": SimTypePointer(SimTypeBottom(label="Guid"), offset=0), "providerData": SimStruct({"size": SimTypeInt(signed=False, label="UInt32"), "data": SimTypePointer(SimTypeChar(label="Byte"), offset=0)}, name="FWP_BYTE_BLOB", pack=False, align=None), "type": SimTypeInt(signed=False, label="FWPM_PROVIDER_CONTEXT_TYPE"), "Anonymous": SimUnion({"keyingPolicy": SimTypePointer(SimStruct({"numKeyMods": SimTypeInt(signed=False, label="UInt32"), "keyModKeys": SimTypePointer(SimTypeBottom(label="Guid"), offset=0), "flags": SimTypeInt(signed=False, label="UInt32")}, name="IPSEC_KEYING_POLICY1", pack=False, align=None), offset=0), "ikeQmTransportPolicy": SimTypePointer(SimStruct({"numIpsecProposals": SimTypeInt(signed=False, label="UInt32"), "ipsecProposals": SimTypePointer(SimStruct({"lifetime": SimStruct({"lifetimeSeconds": SimTypeInt(signed=False, label="UInt32"), "lifetimeKilobytes": SimTypeInt(signed=False, label="UInt32"), "lifetimePackets": SimTypeInt(signed=False, label="UInt32")}, name="IPSEC_SA_LIFETIME0", pack=False, align=None), "numSaTransforms": SimTypeInt(signed=False, label="UInt32"), "saTransforms": SimTypePointer(SimStruct({"ipsecTransformType": SimTypeInt(signed=False, label="IPSEC_TRANSFORM_TYPE"), "Anonymous": SimUnion({"ahTransform": SimTypePointer(SimStruct({"authTransformId": SimStruct({"authType": SimTypeInt(signed=False, label="IPSEC_AUTH_TYPE"), "authConfig": SimTypeChar(label="Byte")}, name="IPSEC_AUTH_TRANSFORM_ID0", pack=False, align=None), "cryptoModuleId": SimTypePointer(SimTypeBottom(label="Guid"), offset=0)}, name="IPSEC_AUTH_TRANSFORM0", pack=False, align=None), offset=0), "espAuthTransform": SimTypePointer(SimStruct({"authTransformId": SimStruct({"authType": SimTypeInt(signed=False, label="IPSEC_AUTH_TYPE"), "authConfig": SimTypeChar(label="Byte")}, name="IPSEC_AUTH_TRANSFORM_ID0", pack=False, align=None), "cryptoModuleId": SimTypePointer(SimTypeBottom(label="Guid"), offset=0)}, name="IPSEC_AUTH_TRANSFORM0", pack=False, align=None), offset=0), "espCipherTransform": SimTypePointer(SimStruct({"cipherTransformId": SimStruct({"cipherType": SimTypeInt(signed=False, label="IPSEC_CIPHER_TYPE"), "cipherConfig": SimTypeChar(label="Byte")}, name="IPSEC_CIPHER_TRANSFORM_ID0", pack=False, align=None), "cryptoModuleId": SimTypePointer(SimTypeBottom(label="Guid"), offset=0)}, name="IPSEC_CIPHER_TRANSFORM0", pack=False, align=None), offset=0), "espAuthAndCipherTransform": SimTypePointer(SimStruct({"authTransform": SimStruct({"authTransformId": SimStruct({"authType": SimTypeInt(signed=False, label="IPSEC_AUTH_TYPE"), "authConfig": SimTypeChar(label="Byte")}, name="IPSEC_AUTH_TRANSFORM_ID0", pack=False, align=None), "cryptoModuleId": SimTypePointer(SimTypeBottom(label="Guid"), offset=0)}, name="IPSEC_AUTH_TRANSFORM0", pack=False, align=None), "cipherTransform": SimStruct({"cipherTransformId": SimStruct({"cipherType": SimTypeInt(signed=False, label="IPSEC_CIPHER_TYPE"), "cipherConfig": SimTypeChar(label="Byte")}, name="IPSEC_CIPHER_TRANSFORM_ID0", pack=False, align=None), "cryptoModuleId": SimTypePointer(SimTypeBottom(label="Guid"), offset=0)}, name="IPSEC_CIPHER_TRANSFORM0", pack=False, align=None)}, name="IPSEC_AUTH_AND_CIPHER_TRANSFORM0", pack=False, align=None), offset=0), "espAuthFwTransform": SimTypePointer(SimStruct({"authTransformId": SimStruct({"authType": SimTypeInt(signed=False, label="IPSEC_AUTH_TYPE"), "authConfig": SimTypeChar(label="Byte")}, name="IPSEC_AUTH_TRANSFORM_ID0", pack=False, align=None), "cryptoModuleId": SimTypePointer(SimTypeBottom(label="Guid"), offset=0)}, name="IPSEC_AUTH_TRANSFORM0", pack=False, align=None), offset=0)}, name="<anon>", label="None")}, name="IPSEC_SA_TRANSFORM0", pack=False, align=None), offset=0), "pfsGroup": SimTypeInt(signed=False, label="IPSEC_PFS_GROUP")}, name="IPSEC_PROPOSAL0", pack=False, align=None), offset=0), "flags": SimTypeInt(signed=False, label="IPSEC_POLICY_FLAG"), "ndAllowClearTimeoutSeconds": SimTypeInt(signed=False, label="UInt32"), "saIdleTimeout": SimStruct({"idleTimeoutSeconds": SimTypeInt(signed=False, label="UInt32"), "idleTimeoutSecondsFailOver": SimTypeInt(signed=False, label="UInt32")}, name="IPSEC_SA_IDLE_TIMEOUT0", pack=False, align=None), "emPolicy": SimTypePointer(SimStruct({"numAuthenticationMethods": SimTypeInt(signed=False, label="UInt32"), "authenticationMethods": SimTypePointer(SimStruct({"authenticationMethodType": SimTypeInt(signed=False, label="IKEEXT_AUTHENTICATION_METHOD_TYPE"), "Anonymous": SimUnion({"presharedKeyAuthentication": SimStruct({"presharedKey": SimStruct({"size": SimTypeInt(signed=False, label="UInt32"), "data": SimTypePointer(SimTypeChar(label="Byte"), offset=0)}, name="FWP_BYTE_BLOB", pack=False, align=None), "flags": SimTypeInt(signed=False, label="IKEEXT_PRESHARED_KEY_AUTHENTICATION_FLAGS")}, name="IKEEXT_PRESHARED_KEY_AUTHENTICATION1", pack=False, align=None), "certificateAuthentication": SimStruct({"inboundConfigType": SimTypeInt(signed=False, label="IKEEXT_CERT_CONFIG_TYPE"), "Anonymous1": SimUnion({"Anonymous1": SimStruct({"inboundRootArraySize": SimTypeInt(signed=False, label="UInt32"), "inboundRootCriteria": SimTypePointer(SimStruct({"certData": SimStruct({"size": SimTypeInt(signed=False, label="UInt32"), "data": SimTypePointer(SimTypeChar(label="Byte"), offset=0)}, name="FWP_BYTE_BLOB", pack=False, align=None), "certHash": SimStruct({"size": SimTypeInt(signed=False, label="UInt32"), "data": SimTypePointer(SimTypeChar(label="Byte"), offset=0)}, name="FWP_BYTE_BLOB", pack=False, align=None), "eku": SimTypePointer(SimStruct({"numEku": SimTypeInt(signed=False, label="UInt32"), "eku": SimTypePointer(SimTypePointer(SimTypeChar(label="Byte"), offset=0), offset=0)}, name="IKEEXT_CERT_EKUS0", pack=False, align=None), offset=0), "name": SimTypePointer(SimStruct({"nameType": SimTypeInt(signed=False, label="IKEEXT_CERT_CRITERIA_NAME_TYPE"), "certName": SimTypePointer(SimTypeChar(label="Char"), offset=0)}, name="IKEEXT_CERT_NAME0", pack=False, align=None), offset=0), "flags": SimTypeInt(signed=False, label="UInt32")}, name="IKEEXT_CERTIFICATE_CRITERIA0", pack=False, align=None), offset=0)}, name="_Anonymous1_e__Struct", pack=False, align=None), "Anonymous2": SimStruct({"inboundEnterpriseStoreArraySize": SimTypeInt(signed=False, label="UInt32"), "inboundEnterpriseStoreCriteria": SimTypePointer(SimStruct({"certData": SimStruct({"size": SimTypeInt(signed=False, label="UInt32"), "data": SimTypePointer(SimTypeChar(label="Byte"), offset=0)}, name="FWP_BYTE_BLOB", pack=False, align=None), "certHash": SimStruct({"size": SimTypeInt(signed=False, label="UInt32"), "data": SimTypePointer(SimTypeChar(label="Byte"), offset=0)}, name="FWP_BYTE_BLOB", pack=False, align=None), "eku": SimTypePointer(SimStruct({"numEku": SimTypeInt(signed=False, label="UInt32"), "eku": SimTypePointer(SimTypePointer(SimTypeChar(label="Byte"), offset=0), offset=0)}, name="IKEEXT_CERT_EKUS0", pack=False, align=None), offset=0), "name": SimTypePointer(SimStruct({"nameType": SimTypeInt(signed=False, label="IKEEXT_CERT_CRITERIA_NAME_TYPE"), "certName": SimTypePointer(SimTypeChar(label="Char"), offset=0)}, name="IKEEXT_CERT_NAME0", pack=False, align=None), offset=0), "flags": SimTypeInt(signed=False, label="UInt32")}, name="IKEEXT_CERTIFICATE_CRITERIA0", pack=False, align=None), offset=0)}, name="_Anonymous2_e__Struct", pack=False, align=None), "Anonymous3": SimStruct({"inboundRootStoreArraySize": SimTypeInt(signed=False, label="UInt32"), "inboundTrustedRootStoreCriteria": SimTypePointer(SimStruct({"certData": SimStruct({"size": SimTypeInt(signed=False, label="UInt32"), "data": SimTypePointer(SimTypeChar(label="Byte"), offset=0)}, name="FWP_BYTE_BLOB", pack=False, align=None), "certHash": SimStruct({"size": SimTypeInt(signed=False, label="UInt32"), "data": SimTypePointer(SimTypeChar(label="Byte"), offset=0)}, name="FWP_BYTE_BLOB", pack=False, align=None), "eku": SimTypePointer(SimStruct({"numEku": SimTypeInt(signed=False, label="UInt32"), "eku": SimTypePointer(SimTypePointer(SimTypeChar(label="Byte"), offset=0), offset=0)}, name="IKEEXT_CERT_EKUS0", pack=False, align=None), offset=0), "name": SimTypePointer(SimStruct({"nameType": SimTypeInt(signed=False, label="IKEEXT_CERT_CRITERIA_NAME_TYPE"), "certName": SimTypePointer(SimTypeChar(label="Char"), offset=0)}, name="IKEEXT_CERT_NAME0", pack=False, align=None), offset=0), "flags": SimTypeInt(signed=False, label="UInt32")}, name="IKEEXT_CERTIFICATE_CRITERIA0", pack=False, align=None), offset=0)}, name="_Anonymous3_e__Struct", pack=False, align=None)}, name="<anon>", label="None"), "outboundConfigType": SimTypeInt(signed=False, label="IKEEXT_CERT_CONFIG_TYPE"), "Anonymous2": SimUnion({"Anonymous1": SimStruct({"outboundRootArraySize": SimTypeInt(signed=False, label="UInt32"), "outboundRootCriteria": SimTypePointer(SimStruct({"certData": SimStruct({"size": SimTypeInt(signed=False, label="UInt32"), "data": SimTypePointer(SimTypeChar(label="Byte"), offset=0)}, name="FWP_BYTE_BLOB", pack=False, align=None), "certHash": SimStruct({"size": SimTypeInt(signed=False, label="UInt32"), "data": SimTypePointer(SimTypeChar(label="Byte"), offset=0)}, name="FWP_BYTE_BLOB", pack=False, align=None), "eku": SimTypePointer(SimStruct({"numEku": SimTypeInt(signed=False, label="UInt32"), "eku": SimTypePointer(SimTypePointer(SimTypeChar(label="Byte"), offset=0), offset=0)}, name="IKEEXT_CERT_EKUS0", pack=False, align=None), offset=0), "name": SimTypePointer(SimStruct({"nameType": SimTypeInt(signed=False, label="IKEEXT_CERT_CRITERIA_NAME_TYPE"), "certName": SimTypePointer(SimTypeChar(label="Char"), offset=0)}, name="IKEEXT_CERT_NAME0", pack=False, align=None), offset=0), "flags": SimTypeInt(signed=False, label="UInt32")}, name="IKEEXT_CERTIFICATE_CRITERIA0", pack=False, align=None), offset=0)}, name="_Anonymous1_e__Struct", pack=False, align=None), "Anonymous2": SimStruct({"outboundEnterpriseStoreArraySize": SimTypeInt(signed=False, label="UInt32"), "outboundEnterpriseStoreCriteria": SimTypePointer(SimStruct({"certData": SimStruct({"size": SimTypeInt(signed=False, label="UInt32"), "data": SimTypePointer(SimTypeChar(label="Byte"), offset=0)}, name="FWP_BYTE_BLOB", pack=False, align=None), "certHash": SimStruct({"size": SimTypeInt(signed=False, label="UInt32"), "data": SimTypePointer(SimTypeChar(label="Byte"), offset=0)}, name="FWP_BYTE_BLOB", pack=False, align=None), "eku": SimTypePointer(SimStruct({"numEku": SimTypeInt(signed=False, label="UInt32"), "eku": SimTypePointer(SimTypePointer(SimTypeChar(label="Byte"), offset=0), offset=0)}, name="IKEEXT_CERT_EKUS0", pack=False, align=None), offset=0), "name": SimTypePointer(SimStruct({"nameType": SimTypeInt(signed=False, label="IKEEXT_CERT_CRITERIA_NAME_TYPE"), "certName": SimTypePointer(SimTypeChar(label="Char"), offset=0)}, name="IKEEXT_CERT_NAME0", pack=False, align=None), offset=0), "flags": SimTypeInt(signed=False, label="UInt32")}, name="IKEEXT_CERTIFICATE_CRITERIA0", pack=False, align=None), offset=0)}, name="_Anonymous2_e__Struct", pack=False, align=None), "Anonymous3": SimStruct({"outboundRootStoreArraySize": SimTypeInt(signed=False, label="UInt32"), "outboundTrustedRootStoreCriteria": SimTypePointer(SimStruct({"certData": SimStruct({"size": SimTypeInt(signed=False, label="UInt32"), "data": SimTypePointer(SimTypeChar(label="Byte"), offset=0)}, name="FWP_BYTE_BLOB", pack=False, align=None), "certHash": SimStruct({"size": SimTypeInt(signed=False, label="UInt32"), "data": SimTypePointer(SimTypeChar(label="Byte"), offset=0)}, name="FWP_BYTE_BLOB", pack=False, align=None), "eku": SimTypePointer(SimStruct({"numEku": SimTypeInt(signed=False, label="UInt32"), "eku": SimTypePointer(SimTypePointer(SimTypeChar(label="Byte"), offset=0), offset=0)}, name="IKEEXT_CERT_EKUS0", pack=False, align=None), offset=0), "name": SimTypePointer(SimStruct({"nameType": SimTypeInt(signed=False, label="IKEEXT_CERT_CRITERIA_NAME_TYPE"), "certName": SimTypePointer(SimTypeChar(label="Char"), offset=0)}, name="IKEEXT_CERT_NAME0", pack=False, align=None), offset=0), "flags": SimTypeInt(signed=False, label="UInt32")}, name="IKEEXT_CERTIFICATE_CRITERIA0", pack=False, align=None), offset=0)}, name="_Anonymous3_e__Struct", pack=False, align=None)}, name="<anon>", label="None"), "flags": SimTypeInt(signed=False, label="IKEEXT_CERT_AUTH"), "localCertLocationUrl": SimStruct({"size": SimTypeInt(signed=False, label="UInt32"), "data": SimTypePointer(SimTypeChar(label="Byte"), offset=0)}, name="FWP_BYTE_BLOB", pack=False, align=None)}, name="IKEEXT_CERTIFICATE_AUTHENTICATION2", pack=False, align=None), "kerberosAuthentication": SimStruct({"flags": SimTypeInt(signed=False, label="IKEEXT_KERBEROS_AUTHENTICATION_FLAGS"), "proxyServer": SimTypePointer(SimTypeChar(label="Char"), offset=0)}, name="IKEEXT_KERBEROS_AUTHENTICATION1", pack=False, align=None), "reservedAuthentication": SimStruct({"flags": SimTypeInt(signed=False, label="IKEEXT_RESERVED_AUTHENTICATION_FLAGS")}, name="IKEEXT_RESERVED_AUTHENTICATION0", pack=False, align=None), "ntlmV2Authentication": SimStruct({"flags": SimTypeInt(signed=False, label="UInt32")}, name="IKEEXT_NTLM_V2_AUTHENTICATION0", pack=False, align=None), "sslAuthentication": SimStruct({"inboundConfigType": SimTypeInt(signed=False, label="IKEEXT_CERT_CONFIG_TYPE"), "Anonymous1": SimUnion({"Anonymous1": SimStruct({"inboundRootArraySize": SimTypeInt(signed=False, label="UInt32"), "inboundRootCriteria": SimTypePointer(SimStruct({"certData": SimStruct({"size": SimTypeInt(signed=False, label="UInt32"), "data": SimTypePointer(SimTypeChar(label="Byte"), offset=0)}, name="FWP_BYTE_BLOB", pack=False, align=None), "certHash": SimStruct({"size": SimTypeInt(signed=False, label="UInt32"), "data": SimTypePointer(SimTypeChar(label="Byte"), offset=0)}, name="FWP_BYTE_BLOB", pack=False, align=None), "eku": SimTypePointer(SimStruct({"numEku": SimTypeInt(signed=False, label="UInt32"), "eku": SimTypePointer(SimTypePointer(SimTypeChar(label="Byte"), offset=0), offset=0)}, name="IKEEXT_CERT_EKUS0", pack=False, align=None), offset=0), "name": SimTypePointer(SimStruct({"nameType": SimTypeInt(signed=False, label="IKEEXT_CERT_CRITERIA_NAME_TYPE"), "certName": SimTypePointer(SimTypeChar(label="Char"), offset=0)}, name="IKEEXT_CERT_NAME0", pack=False, align=None), offset=0), "flags": SimTypeInt(signed=False, label="UInt32")}, name="IKEEXT_CERTIFICATE_CRITERIA0", pack=False, align=None), offset=0)}, name="_Anonymous1_e__Struct", pack=False, align=None), "Anonymous2": SimStruct({"inboundEnterpriseStoreArraySize": SimTypeInt(signed=False, label="UInt32"), "inboundEnterpriseStoreCriteria": SimTypePointer(SimStruct({"certData": SimStruct({"size": SimTypeInt(signed=False, label="UInt32"), "data": SimTypePointer(SimTypeChar(label="Byte"), offset=0)}, name="FWP_BYTE_BLOB", pack=False, align=None), "certHash": SimStruct({"size": SimTypeInt(signed=False, label="UInt32"), "data": SimTypePointer(SimTypeChar(label="Byte"), offset=0)}, name="FWP_BYTE_BLOB", pack=False, align=None), "eku": SimTypePointer(SimStruct({"numEku": SimTypeInt(signed=False, label="UInt32"), "eku": SimTypePointer(SimTypePointer(SimTypeChar(label="Byte"), offset=0), offset=0)}, name="IKEEXT_CERT_EKUS0", pack=False, align=None), offset=0), "name": SimTypePointer(SimStruct({"nameType": SimTypeInt(signed=False, label="IKEEXT_CERT_CRITERIA_NAME_TYPE"), "certName": SimTypePointer(SimTypeChar(label="Char"), offset=0)}, name="IKEEXT_CERT_NAME0", pack=False, align=None), offset=0), "flags": SimTypeInt(signed=False, label="UInt32")}, name="IKEEXT_CERTIFICATE_CRITERIA0", pack=False, align=None), offset=0)}, name="_Anonymous2_e__Struct", pack=False, align=None), "Anonymous3": SimStruct({"inboundRootStoreArraySize": SimTypeInt(signed=False, label="UInt32"), "inboundTrustedRootStoreCriteria": SimTypePointer(SimStruct({"certData": SimStruct({"size": SimTypeInt(signed=False, label="UInt32"), "data": SimTypePointer(SimTypeChar(label="Byte"), offset=0)}, name="FWP_BYTE_BLOB", pack=False, align=None), "certHash": SimStruct({"size": SimTypeInt(signed=False, label="UInt32"), "data": SimTypePointer(SimTypeChar(label="Byte"), offset=0)}, name="FWP_BYTE_BLOB", pack=False, align=None), "eku": SimTypePointer(SimStruct({"numEku": SimTypeInt(signed=False, label="UInt32"), "eku": SimTypePointer(SimTypePointer(SimTypeChar(label="Byte"), offset=0), offset=0)}, name="IKEEXT_CERT_EKUS0", pack=False, align=None), offset=0), "name": SimTypePointer(SimStruct({"nameType": SimTypeInt(signed=False, label="IKEEXT_CERT_CRITERIA_NAME_TYPE"), "certName": SimTypePointer(SimTypeChar(label="Char"), offset=0)}, name="IKEEXT_CERT_NAME0", pack=False, align=None), offset=0), "flags": SimTypeInt(signed=False, label="UInt32")}, name="IKEEXT_CERTIFICATE_CRITERIA0", pack=False, align=None), offset=0)}, name="_Anonymous3_e__Struct", pack=False, align=None)}, name="<anon>", label="None"), "outboundConfigType": SimTypeInt(signed=False, label="IKEEXT_CERT_CONFIG_TYPE"), "Anonymous2": SimUnion({"Anonymous1": SimStruct({"outboundRootArraySize": SimTypeInt(signed=False, label="UInt32"), "outboundRootCriteria": SimTypePointer(SimStruct({"certData": SimStruct({"size": SimTypeInt(signed=False, label="UInt32"), "data": SimTypePointer(SimTypeChar(label="Byte"), offset=0)}, name="FWP_BYTE_BLOB", pack=False, align=None), "certHash": SimStruct({"size": SimTypeInt(signed=False, label="UInt32"), "data": SimTypePointer(SimTypeChar(label="Byte"), offset=0)}, name="FWP_BYTE_BLOB", pack=False, align=None), "eku": SimTypePointer(SimStruct({"numEku": SimTypeInt(signed=False, label="UInt32"), "eku": SimTypePointer(SimTypePointer(SimTypeChar(label="Byte"), offset=0), offset=0)}, name="IKEEXT_CERT_EKUS0", pack=False, align=None), offset=0), "name": SimTypePointer(SimStruct({"nameType": SimTypeInt(signed=False, label="IKEEXT_CERT_CRITERIA_NAME_TYPE"), "certName": SimTypePointer(SimTypeChar(label="Char"), offset=0)}, name="IKEEXT_CERT_NAME0", pack=False, align=None), offset=0), "flags": SimTypeInt(signed=False, label="UInt32")}, name="IKEEXT_CERTIFICATE_CRITERIA0", pack=False, align=None), offset=0)}, name="_Anonymous1_e__Struct", pack=False, align=None), "Anonymous2": SimStruct({"outboundEnterpriseStoreArraySize": SimTypeInt(signed=False, label="UInt32"), "outboundEnterpriseStoreCriteria": SimTypePointer(SimStruct({"certData": SimStruct({"size": SimTypeInt(signed=False, label="UInt32"), "data": SimTypePointer(SimTypeChar(label="Byte"), offset=0)}, name="FWP_BYTE_BLOB", pack=False, align=None), "certHash": SimStruct({"size": SimTypeInt(signed=False, label="UInt32"), "data": SimTypePointer(SimTypeChar(label="Byte"), offset=0)}, name="FWP_BYTE_BLOB", pack=False, align=None), "eku": SimTypePointer(SimStruct({"numEku": SimTypeInt(signed=False, label="UInt32"), "eku": SimTypePointer(SimTypePointer(SimTypeChar(label="Byte"), offset=0), offset=0)}, name="IKEEXT_CERT_EKUS0", pack=False, align=None), offset=0), "name": SimTypePointer(SimStruct({"nameType": SimTypeInt(signed=False, label="IKEEXT_CERT_CRITERIA_NAME_TYPE"), "certName": SimTypePointer(SimTypeChar(label="Char"), offset=0)}, name="IKEEXT_CERT_NAME0", pack=False, align=None), offset=0), "flags": SimTypeInt(signed=False, label="UInt32")}, name="IKEEXT_CERTIFICATE_CRITERIA0", pack=False, align=None), offset=0)}, name="_Anonymous2_e__Struct", pack=False, align=None), "Anonymous3": SimStruct({"outboundRootStoreArraySize": SimTypeInt(signed=False, label="UInt32"), "outboundTrustedRootStoreCriteria": SimTypePointer(SimStruct({"certData": SimStruct({"size": SimTypeInt(signed=False, label="UInt32"), "data": SimTypePointer(SimTypeChar(label="Byte"), offset=0)}, name="FWP_BYTE_BLOB", pack=False, align=None), "certHash": SimStruct({"size": SimTypeInt(signed=False, label="UInt32"), "data": SimTypePointer(SimTypeChar(label="Byte"), offset=0)}, name="FWP_BYTE_BLOB", pack=False, align=None), "eku": SimTypePointer(SimStruct({"numEku": SimTypeInt(signed=False, label="UInt32"), "eku": SimTypePointer(SimTypePointer(SimTypeChar(label="Byte"), offset=0), offset=0)}, name="IKEEXT_CERT_EKUS0", pack=False, align=None), offset=0), "name": SimTypePointer(SimStruct({"nameType": SimTypeInt(signed=False, label="IKEEXT_CERT_CRITERIA_NAME_TYPE"), "certName": SimTypePointer(SimTypeChar(label="Char"), offset=0)}, name="IKEEXT_CERT_NAME0", pack=False, align=None), offset=0), "flags": SimTypeInt(signed=False, label="UInt32")}, name="IKEEXT_CERTIFICATE_CRITERIA0", pack=False, align=None), offset=0)}, name="_Anonymous3_e__Struct", pack=False, align=None)}, name="<anon>", label="None"), "flags": SimTypeInt(signed=False, label="IKEEXT_CERT_AUTH"), "localCertLocationUrl": SimStruct({"size": SimTypeInt(signed=False, label="UInt32"), "data": SimTypePointer(SimTypeChar(label="Byte"), offset=0)}, name="FWP_BYTE_BLOB", pack=False, align=None)}, name="IKEEXT_CERTIFICATE_AUTHENTICATION2", pack=False, align=None), "cgaAuthentication": SimStruct({"keyContainerName": SimTypePointer(SimTypeChar(label="Char"), offset=0), "cspName": SimTypePointer(SimTypeChar(label="Char"), offset=0), "cspType": SimTypeInt(signed=False, label="UInt32"), "cgaModifier": SimStruct({"byteArray16": SimTypeFixedSizeArray(SimTypeChar(label="Byte"), 16)}, name="FWP_BYTE_ARRAY16", pack=False, align=None), "cgaCollisionCount": SimTypeChar(label="Byte")}, name="IKEEXT_IPV6_CGA_AUTHENTICATION0", pack=False, align=None), "eapAuthentication": SimStruct({"flags": SimTypeInt(signed=False, label="IKEEXT_EAP_AUTHENTICATION_FLAGS")}, name="IKEEXT_EAP_AUTHENTICATION0", pack=False, align=None)}, name="<anon>", label="None")}, name="IKEEXT_AUTHENTICATION_METHOD2", pack=False, align=None), offset=0), "initiatorImpersonationType": SimTypeInt(signed=False, label="IKEEXT_AUTHENTICATION_IMPERSONATION_TYPE")}, name="IKEEXT_EM_POLICY2", pack=False, align=None), offset=0)}, name="IPSEC_TRANSPORT_POLICY2", pack=False, align=None), offset=0), "ikeQmTunnelPolicy": SimTypePointer(SimStruct({"flags": SimTypeInt(signed=False, label="UInt32"), "numIpsecProposals": SimTypeInt(signed=False, label="UInt32"), "ipsecProposals": SimTypePointer(SimStruct({"lifetime": SimStruct({"lifetimeSeconds": SimTypeInt(signed=False, label="UInt32"), "lifetimeKilobytes": SimTypeInt(signed=False, label="UInt32"), "lifetimePackets": SimTypeInt(signed=False, label="UInt32")}, name="IPSEC_SA_LIFETIME0", pack=False, align=None), "numSaTransforms": SimTypeInt(signed=False, label="UInt32"), "saTransforms": SimTypePointer(SimStruct({"ipsecTransformType": SimTypeInt(signed=False, label="IPSEC_TRANSFORM_TYPE"), "Anonymous": SimUnion({"ahTransform": SimTypePointer(SimStruct({"authTransformId": SimStruct({"authType": SimTypeInt(signed=False, label="IPSEC_AUTH_TYPE"), "authConfig": SimTypeChar(label="Byte")}, name="IPSEC_AUTH_TRANSFORM_ID0", pack=False, align=None), "cryptoModuleId": SimTypePointer(SimTypeBottom(label="Guid"), offset=0)}, name="IPSEC_AUTH_TRANSFORM0", pack=False, align=None), offset=0), "espAuthTransform": SimTypePointer(SimStruct({"authTransformId": SimStruct({"authType": SimTypeInt(signed=False, label="IPSEC_AUTH_TYPE"), "authConfig": SimTypeChar(label="Byte")}, name="IPSEC_AUTH_TRANSFORM_ID0", pack=False, align=None), "cryptoModuleId": SimTypePointer(SimTypeBottom(label="Guid"), offset=0)}, name="IPSEC_AUTH_TRANSFORM0", pack=False, align=None), offset=0), "espCipherTransform": SimTypePointer(SimStruct({"cipherTransformId": SimStruct({"cipherType": SimTypeInt(signed=False, label="IPSEC_CIPHER_TYPE"), "cipherConfig": SimTypeChar(label="Byte")}, name="IPSEC_CIPHER_TRANSFORM_ID0", pack=False, align=None), "cryptoModuleId": SimTypePointer(SimTypeBottom(label="Guid"), offset=0)}, name="IPSEC_CIPHER_TRANSFORM0", pack=False, align=None), offset=0), "espAuthAndCipherTransform": SimTypePointer(SimStruct({"authTransform": SimStruct({"authTransformId": SimStruct({"authType": SimTypeInt(signed=False, label="IPSEC_AUTH_TYPE"), "authConfig": SimTypeChar(label="Byte")}, name="IPSEC_AUTH_TRANSFORM_ID0", pack=False, align=None), "cryptoModuleId": SimTypePointer(SimTypeBottom(label="Guid"), offset=0)}, name="IPSEC_AUTH_TRANSFORM0", pack=False, align=None), "cipherTransform": SimStruct({"cipherTransformId": SimStruct({"cipherType": SimTypeInt(signed=False, label="IPSEC_CIPHER_TYPE"), "cipherConfig": SimTypeChar(label="Byte")}, name="IPSEC_CIPHER_TRANSFORM_ID0", pack=False, align=None), "cryptoModuleId": SimTypePointer(SimTypeBottom(label="Guid"), offset=0)}, name="IPSEC_CIPHER_TRANSFORM0", pack=False, align=None)}, name="IPSEC_AUTH_AND_CIPHER_TRANSFORM0", pack=False, align=None), offset=0), "espAuthFwTransform": SimTypePointer(SimStruct({"authTransformId": SimStruct({"authType": SimTypeInt(signed=False, label="IPSEC_AUTH_TYPE"), "authConfig": SimTypeChar(label="Byte")}, name="IPSEC_AUTH_TRANSFORM_ID0", pack=False, align=None), "cryptoModuleId": SimTypePointer(SimTypeBottom(label="Guid"), offset=0)}, name="IPSEC_AUTH_TRANSFORM0", pack=False, align=None), offset=0)}, name="<anon>", label="None")}, name="IPSEC_SA_TRANSFORM0", pack=False, align=None), offset=0), "pfsGroup": SimTypeInt(signed=False, label="IPSEC_PFS_GROUP")}, name="IPSEC_PROPOSAL0", pack=False, align=None), offset=0), "tunnelEndpoints": SimStruct({"ipVersion": SimTypeInt(signed=False, label="FWP_IP_VERSION"), "Anonymous1": SimUnion({"localV4Address": SimTypeInt(signed=False, label="UInt32"), "localV6Address": SimTypeFixedSizeArray(SimTypeChar(label="Byte"), 16)}, name="<anon>", label="None"), "Anonymous2": SimUnion({"remoteV4Address": SimTypeInt(signed=False, label="UInt32"), "remoteV6Address": SimTypeFixedSizeArray(SimTypeChar(label="Byte"), 16)}, name="<anon>", label="None"), "localIfLuid": SimTypeLongLong(signed=False, label="UInt64"), "remoteFqdn": SimTypePointer(SimTypeChar(label="Char"), offset=0), "numAddresses": SimTypeInt(signed=False, label="UInt32"), "remoteAddresses": SimTypePointer(SimStruct({"ipVersion": SimTypeInt(signed=False, label="FWP_IP_VERSION"), "Anonymous": SimUnion({"v4Address": SimTypeInt(signed=False, label="UInt32"), "v6Address": SimTypeFixedSizeArray(SimTypeChar(label="Byte"), 16)}, name="<anon>", label="None")}, name="IPSEC_TUNNEL_ENDPOINT0", pack=False, align=None), offset=0)}, name="IPSEC_TUNNEL_ENDPOINTS2", pack=False, align=None), "saIdleTimeout": SimStruct({"idleTimeoutSeconds": SimTypeInt(signed=False, label="UInt32"), "idleTimeoutSecondsFailOver": SimTypeInt(signed=False, label="UInt32")}, name="IPSEC_SA_IDLE_TIMEOUT0", pack=False, align=None), "emPolicy": SimTypePointer(SimStruct({"numAuthenticationMethods": SimTypeInt(signed=False, label="UInt32"), "authenticationMethods": SimTypePointer(SimStruct({"authenticationMethodType": SimTypeInt(signed=False, label="IKEEXT_AUTHENTICATION_METHOD_TYPE"), "Anonymous": SimUnion({"presharedKeyAuthentication": SimStruct({"presharedKey": SimStruct({"size": SimTypeInt(signed=False, label="UInt32"), "data": SimTypePointer(SimTypeChar(label="Byte"), offset=0)}, name="FWP_BYTE_BLOB", pack=False, align=None), "flags": SimTypeInt(signed=False, label="IKEEXT_PRESHARED_KEY_AUTHENTICATION_FLAGS")}, name="IKEEXT_PRESHARED_KEY_AUTHENTICATION1", pack=False, align=None), "certificateAuthentication": SimStruct({"inboundConfigType": SimTypeInt(signed=False, label="IKEEXT_CERT_CONFIG_TYPE"), "Anonymous1": SimUnion({"Anonymous1": SimStruct({"inboundRootArraySize": SimTypeInt(signed=False, label="UInt32"), "inboundRootCriteria": SimTypePointer(SimStruct({"certData": SimStruct({"size": SimTypeInt(signed=False, label="UInt32"), "data": SimTypePointer(SimTypeChar(label="Byte"), offset=0)}, name="FWP_BYTE_BLOB", pack=False, align=None), "certHash": SimStruct({"size": SimTypeInt(signed=False, label="UInt32"), "data": SimTypePointer(SimTypeChar(label="Byte"), offset=0)}, name="FWP_BYTE_BLOB", pack=False, align=None), "eku": SimTypePointer(SimStruct({"numEku": SimTypeInt(signed=False, label="UInt32"), "eku": SimTypePointer(SimTypePointer(SimTypeChar(label="Byte"), offset=0), offset=0)}, name="IKEEXT_CERT_EKUS0", pack=False, align=None), offset=0), "name": SimTypePointer(SimStruct({"nameType": SimTypeInt(signed=False, label="IKEEXT_CERT_CRITERIA_NAME_TYPE"), "certName": SimTypePointer(SimTypeChar(label="Char"), offset=0)}, name="IKEEXT_CERT_NAME0", pack=False, align=None), offset=0), "flags": SimTypeInt(signed=False, label="UInt32")}, name="IKEEXT_CERTIFICATE_CRITERIA0", pack=False, align=None), offset=0)}, name="_Anonymous1_e__Struct", pack=False, align=None), "Anonymous2": SimStruct({"inboundEnterpriseStoreArraySize": SimTypeInt(signed=False, label="UInt32"), "inboundEnterpriseStoreCriteria": SimTypePointer(SimStruct({"certData": SimStruct({"size": SimTypeInt(signed=False, label="UInt32"), "data": SimTypePointer(SimTypeChar(label="Byte"), offset=0)}, name="FWP_BYTE_BLOB", pack=False, align=None), "certHash": SimStruct({"size": SimTypeInt(signed=False, label="UInt32"), "data": SimTypePointer(SimTypeChar(label="Byte"), offset=0)}, name="FWP_BYTE_BLOB", pack=False, align=None), "eku": SimTypePointer(SimStruct({"numEku": SimTypeInt(signed=False, label="UInt32"), "eku": SimTypePointer(SimTypePointer(SimTypeChar(label="Byte"), offset=0), offset=0)}, name="IKEEXT_CERT_EKUS0", pack=False, align=None), offset=0), "name": SimTypePointer(SimStruct({"nameType": SimTypeInt(signed=False, label="IKEEXT_CERT_CRITERIA_NAME_TYPE"), "certName": SimTypePointer(SimTypeChar(label="Char"), offset=0)}, name="IKEEXT_CERT_NAME0", pack=False, align=None), offset=0), "flags": SimTypeInt(signed=False, label="UInt32")}, name="IKEEXT_CERTIFICATE_CRITERIA0", pack=False, align=None), offset=0)}, name="_Anonymous2_e__Struct", pack=False, align=None), "Anonymous3": SimStruct({"inboundRootStoreArraySize": SimTypeInt(signed=False, label="UInt32"), "inboundTrustedRootStoreCriteria": SimTypePointer(SimStruct({"certData": SimStruct({"size": SimTypeInt(signed=False, label="UInt32"), "data": SimTypePointer(SimTypeChar(label="Byte"), offset=0)}, name="FWP_BYTE_BLOB", pack=False, align=None), "certHash": SimStruct({"size": SimTypeInt(signed=False, label="UInt32"), "data": SimTypePointer(SimTypeChar(label="Byte"), offset=0)}, name="FWP_BYTE_BLOB", pack=False, align=None), "eku": SimTypePointer(SimStruct({"numEku": SimTypeInt(signed=False, label="UInt32"), "eku": SimTypePointer(SimTypePointer(SimTypeChar(label="Byte"), offset=0), offset=0)}, name="IKEEXT_CERT_EKUS0", pack=False, align=None), offset=0), "name": SimTypePointer(SimStruct({"nameType": SimTypeInt(signed=False, label="IKEEXT_CERT_CRITERIA_NAME_TYPE"), "certName": SimTypePointer(SimTypeChar(label="Char"), offset=0)}, name="IKEEXT_CERT_NAME0", pack=False, align=None), offset=0), "flags": SimTypeInt(signed=False, label="UInt32")}, name="IKEEXT_CERTIFICATE_CRITERIA0", pack=False, align=None), offset=0)}, name="_Anonymous3_e__Struct", pack=False, align=None)}, name="<anon>", label="None"), "outboundConfigType": SimTypeInt(signed=False, label="IKEEXT_CERT_CONFIG_TYPE"), "Anonymous2": SimUnion({"Anonymous1": SimStruct({"outboundRootArraySize": SimTypeInt(signed=False, label="UInt32"), "outboundRootCriteria": SimTypePointer(SimStruct({"certData": SimStruct({"size": SimTypeInt(signed=False, label="UInt32"), "data": SimTypePointer(SimTypeChar(label="Byte"), offset=0)}, name="FWP_BYTE_BLOB", pack=False, align=None), "certHash": SimStruct({"size": SimTypeInt(signed=False, label="UInt32"), "data": SimTypePointer(SimTypeChar(label="Byte"), offset=0)}, name="FWP_BYTE_BLOB", pack=False, align=None), "eku": SimTypePointer(SimStruct({"numEku": SimTypeInt(signed=False, label="UInt32"), "eku": SimTypePointer(SimTypePointer(SimTypeChar(label="Byte"), offset=0), offset=0)}, name="IKEEXT_CERT_EKUS0", pack=False, align=None), offset=0), "name": SimTypePointer(SimStruct({"nameType": SimTypeInt(signed=False, label="IKEEXT_CERT_CRITERIA_NAME_TYPE"), "certName": SimTypePointer(SimTypeChar(label="Char"), offset=0)}, name="IKEEXT_CERT_NAME0", pack=False, align=None), offset=0), "flags": SimTypeInt(signed=False, label="UInt32")}, name="IKEEXT_CERTIFICATE_CRITERIA0", pack=False, align=None), offset=0)}, name="_Anonymous1_e__Struct", pack=False, align=None), "Anonymous2": SimStruct({"outboundEnterpriseStoreArraySize": SimTypeInt(signed=False, label="UInt32"), "outboundEnterpriseStoreCriteria": SimTypePointer(SimStruct({"certData": SimStruct({"size": SimTypeInt(signed=False, label="UInt32"), "data": SimTypePointer(SimTypeChar(label="Byte"), offset=0)}, name="FWP_BYTE_BLOB", pack=False, align=None), "certHash": SimStruct({"size": SimTypeInt(signed=False, label="UInt32"), "data": SimTypePointer(SimTypeChar(label="Byte"), offset=0)}, name="FWP_BYTE_BLOB", pack=False, align=None), "eku": SimTypePointer(SimStruct({"numEku": SimTypeInt(signed=False, label="UInt32"), "eku": SimTypePointer(SimTypePointer(SimTypeChar(label="Byte"), offset=0), offset=0)}, name="IKEEXT_CERT_EKUS0", pack=False, align=None), offset=0), "name": SimTypePointer(SimStruct({"nameType": SimTypeInt(signed=False, label="IKEEXT_CERT_CRITERIA_NAME_TYPE"), "certName": SimTypePointer(SimTypeChar(label="Char"), offset=0)}, name="IKEEXT_CERT_NAME0", pack=False, align=None), offset=0), "flags": SimTypeInt(signed=False, label="UInt32")}, name="IKEEXT_CERTIFICATE_CRITERIA0", pack=False, align=None), offset=0)}, name="_Anonymous2_e__Struct", pack=False, align=None), "Anonymous3": SimStruct({"outboundRootStoreArraySize": SimTypeInt(signed=False, label="UInt32"), "outboundTrustedRootStoreCriteria": SimTypePointer(SimStruct({"certData": SimStruct({"size": SimTypeInt(signed=False, label="UInt32"), "data": SimTypePointer(SimTypeChar(label="Byte"), offset=0)}, name="FWP_BYTE_BLOB", pack=False, align=None), "certHash": SimStruct({"size": SimTypeInt(signed=False, label="UInt32"), "data": SimTypePointer(SimTypeChar(label="Byte"), offset=0)}, name="FWP_BYTE_BLOB", pack=False, align=None), "eku": SimTypePointer(SimStruct({"numEku": SimTypeInt(signed=False, label="UInt32"), "eku": SimTypePointer(SimTypePointer(SimTypeChar(label="Byte"), offset=0), offset=0)}, name="IKEEXT_CERT_EKUS0", pack=False, align=None), offset=0), "name": SimTypePointer(SimStruct({"nameType": SimTypeInt(signed=False, label="IKEEXT_CERT_CRITERIA_NAME_TYPE"), "certName": SimTypePointer(SimTypeChar(label="Char"), offset=0)}, name="IKEEXT_CERT_NAME0", pack=False, align=None), offset=0), "flags": SimTypeInt(signed=False, label="UInt32")}, name="IKEEXT_CERTIFICATE_CRITERIA0", pack=False, align=None), offset=0)}, name="_Anonymous3_e__Struct", pack=False, align=None)}, name="<anon>", label="None"), "flags": SimTypeInt(signed=False, label="IKEEXT_CERT_AUTH"), "localCertLocationUrl": SimStruct({"size": SimTypeInt(signed=False, label="UInt32"), "data": SimTypePointer(SimTypeChar(label="Byte"), offset=0)}, name="FWP_BYTE_BLOB", pack=False, align=None)}, name="IKEEXT_CERTIFICATE_AUTHENTICATION2", pack=False, align=None), "kerberosAuthentication": SimStruct({"flags": SimTypeInt(signed=False, label="IKEEXT_KERBEROS_AUTHENTICATION_FLAGS"), "proxyServer": SimTypePointer(SimTypeChar(label="Char"), offset=0)}, name="IKEEXT_KERBEROS_AUTHENTICATION1", pack=False, align=None), "reservedAuthentication": SimStruct({"flags": SimTypeInt(signed=False, label="IKEEXT_RESERVED_AUTHENTICATION_FLAGS")}, name="IKEEXT_RESERVED_AUTHENTICATION0", pack=False, align=None), "ntlmV2Authentication": SimStruct({"flags": SimTypeInt(signed=False, label="UInt32")}, name="IKEEXT_NTLM_V2_AUTHENTICATION0", pack=False, align=None), "sslAuthentication": SimStruct({"inboundConfigType": SimTypeInt(signed=False, label="IKEEXT_CERT_CONFIG_TYPE"), "Anonymous1": SimUnion({"Anonymous1": SimStruct({"inboundRootArraySize": SimTypeInt(signed=False, label="UInt32"), "inboundRootCriteria": SimTypePointer(SimStruct({"certData": SimStruct({"size": SimTypeInt(signed=False, label="UInt32"), "data": SimTypePointer(SimTypeChar(label="Byte"), offset=0)}, name="FWP_BYTE_BLOB", pack=False, align=None), "certHash": SimStruct({"size": SimTypeInt(signed=False, label="UInt32"), "data": SimTypePointer(SimTypeChar(label="Byte"), offset=0)}, name="FWP_BYTE_BLOB", pack=False, align=None), "eku": SimTypePointer(SimStruct({"numEku": SimTypeInt(signed=False, label="UInt32"), "eku": SimTypePointer(SimTypePointer(SimTypeChar(label="Byte"), offset=0), offset=0)}, name="IKEEXT_CERT_EKUS0", pack=False, align=None), offset=0), "name": SimTypePointer(SimStruct({"nameType": SimTypeInt(signed=False, label="IKEEXT_CERT_CRITERIA_NAME_TYPE"), "certName": SimTypePointer(SimTypeChar(label="Char"), offset=0)}, name="IKEEXT_CERT_NAME0", pack=False, align=None), offset=0), "flags": SimTypeInt(signed=False, label="UInt32")}, name="IKEEXT_CERTIFICATE_CRITERIA0", pack=False, align=None), offset=0)}, name="_Anonymous1_e__Struct", pack=False, align=None), "Anonymous2": SimStruct({"inboundEnterpriseStoreArraySize": SimTypeInt(signed=False, label="UInt32"), "inboundEnterpriseStoreCriteria": SimTypePointer(SimStruct({"certData": SimStruct({"size": SimTypeInt(signed=False, label="UInt32"), "data": SimTypePointer(SimTypeChar(label="Byte"), offset=0)}, name="FWP_BYTE_BLOB", pack=False, align=None), "certHash": SimStruct({"size": SimTypeInt(signed=False, label="UInt32"), "data": SimTypePointer(SimTypeChar(label="Byte"), offset=0)}, name="FWP_BYTE_BLOB", pack=False, align=None), "eku": SimTypePointer(SimStruct({"numEku": SimTypeInt(signed=False, label="UInt32"), "eku": SimTypePointer(SimTypePointer(SimTypeChar(label="Byte"), offset=0), offset=0)}, name="IKEEXT_CERT_EKUS0", pack=False, align=None), offset=0), "name": SimTypePointer(SimStruct({"nameType": SimTypeInt(signed=False, label="IKEEXT_CERT_CRITERIA_NAME_TYPE"), "certName": SimTypePointer(SimTypeChar(label="Char"), offset=0)}, name="IKEEXT_CERT_NAME0", pack=False, align=None), offset=0), "flags": SimTypeInt(signed=False, label="UInt32")}, name="IKEEXT_CERTIFICATE_CRITERIA0", pack=False, align=None), offset=0)}, name="_Anonymous2_e__Struct", pack=False, align=None), "Anonymous3": SimStruct({"inboundRootStoreArraySize": SimTypeInt(signed=False, label="UInt32"), "inboundTrustedRootStoreCriteria": SimTypePointer(SimStruct({"certData": SimStruct({"size": SimTypeInt(signed=False, label="UInt32"), "data": SimTypePointer(SimTypeChar(label="Byte"), offset=0)}, name="FWP_BYTE_BLOB", pack=False, align=None), "certHash": SimStruct({"size": SimTypeInt(signed=False, label="UInt32"), "data": SimTypePointer(SimTypeChar(label="Byte"), offset=0)}, name="FWP_BYTE_BLOB", pack=False, align=None), "eku": SimTypePointer(SimStruct({"numEku": SimTypeInt(signed=False, label="UInt32"), "eku": SimTypePointer(SimTypePointer(SimTypeChar(label="Byte"), offset=0), offset=0)}, name="IKEEXT_CERT_EKUS0", pack=False, align=None), offset=0), "name": SimTypePointer(SimStruct({"nameType": SimTypeInt(signed=False, label="IKEEXT_CERT_CRITERIA_NAME_TYPE"), "certName": SimTypePointer(SimTypeChar(label="Char"), offset=0)}, name="IKEEXT_CERT_NAME0", pack=False, align=None), offset=0), "flags": SimTypeInt(signed=False, label="UInt32")}, name="IKEEXT_CERTIFICATE_CRITERIA0", pack=False, align=None), offset=0)}, name="_Anonymous3_e__Struct", pack=False, align=None)}, name="<anon>", label="None"), "outboundConfigType": SimTypeInt(signed=False, label="IKEEXT_CERT_CONFIG_TYPE"), "Anonymous2": SimUnion({"Anonymous1": SimStruct({"outboundRootArraySize": SimTypeInt(signed=False, label="UInt32"), "outboundRootCriteria": SimTypePointer(SimStruct({"certData": SimStruct({"size": SimTypeInt(signed=False, label="UInt32"), "data": SimTypePointer(SimTypeChar(label="Byte"), offset=0)}, name="FWP_BYTE_BLOB", pack=False, align=None), "certHash": SimStruct({"size": SimTypeInt(signed=False, label="UInt32"), "data": SimTypePointer(SimTypeChar(label="Byte"), offset=0)}, name="FWP_BYTE_BLOB", pack=False, align=None), "eku": SimTypePointer(SimStruct({"numEku": SimTypeInt(signed=False, label="UInt32"), "eku": SimTypePointer(SimTypePointer(SimTypeChar(label="Byte"), offset=0), offset=0)}, name="IKEEXT_CERT_EKUS0", pack=False, align=None), offset=0), "name": SimTypePointer(SimStruct({"nameType": SimTypeInt(signed=False, label="IKEEXT_CERT_CRITERIA_NAME_TYPE"), "certName": SimTypePointer(SimTypeChar(label="Char"), offset=0)}, name="IKEEXT_CERT_NAME0", pack=False, align=None), offset=0), "flags": SimTypeInt(signed=False, label="UInt32")}, name="IKEEXT_CERTIFICATE_CRITERIA0", pack=False, align=None), offset=0)}, name="_Anonymous1_e__Struct", pack=False, align=None), "Anonymous2": SimStruct({"outboundEnterpriseStoreArraySize": SimTypeInt(signed=False, label="UInt32"), "outboundEnterpriseStoreCriteria": SimTypePointer(SimStruct({"certData": SimStruct({"size": SimTypeInt(signed=False, label="UInt32"), "data": SimTypePointer(SimTypeChar(label="Byte"), offset=0)}, name="FWP_BYTE_BLOB", pack=False, align=None), "certHash": SimStruct({"size": SimTypeInt(signed=False, label="UInt32"), "data": SimTypePointer(SimTypeChar(label="Byte"), offset=0)}, name="FWP_BYTE_BLOB", pack=False, align=None), "eku": SimTypePointer(SimStruct({"numEku": SimTypeInt(signed=False, label="UInt32"), "eku": SimTypePointer(SimTypePointer(SimTypeChar(label="Byte"), offset=0), offset=0)}, name="IKEEXT_CERT_EKUS0", pack=False, align=None), offset=0), "name": SimTypePointer(SimStruct({"nameType": SimTypeInt(signed=False, label="IKEEXT_CERT_CRITERIA_NAME_TYPE"), "certName": SimTypePointer(SimTypeChar(label="Char"), offset=0)}, name="IKEEXT_CERT_NAME0", pack=False, align=None), offset=0), "flags": SimTypeInt(signed=False, label="UInt32")}, name="IKEEXT_CERTIFICATE_CRITERIA0", pack=False, align=None), offset=0)}, name="_Anonymous2_e__Struct", pack=False, align=None), "Anonymous3": SimStruct({"outboundRootStoreArraySize": SimTypeInt(signed=False, label="UInt32"), "outboundTrustedRootStoreCriteria": SimTypePointer(SimStruct({"certData": SimStruct({"size": SimTypeInt(signed=False, label="UInt32"), "data": SimTypePointer(SimTypeChar(label="Byte"), offset=0)}, name="FWP_BYTE_BLOB", pack=False, align=None), "certHash": SimStruct({"size": SimTypeInt(signed=False, label="UInt32"), "data": SimTypePointer(SimTypeChar(label="Byte"), offset=0)}, name="FWP_BYTE_BLOB", pack=False, align=None), "eku": SimTypePointer(SimStruct({"numEku": SimTypeInt(signed=False, label="UInt32"), "eku": SimTypePointer(SimTypePointer(SimTypeChar(label="Byte"), offset=0), offset=0)}, name="IKEEXT_CERT_EKUS0", pack=False, align=None), offset=0), "name": SimTypePointer(SimStruct({"nameType": SimTypeInt(signed=False, label="IKEEXT_CERT_CRITERIA_NAME_TYPE"), "certName": SimTypePointer(SimTypeChar(label="Char"), offset=0)}, name="IKEEXT_CERT_NAME0", pack=False, align=None), offset=0), "flags": SimTypeInt(signed=False, label="UInt32")}, name="IKEEXT_CERTIFICATE_CRITERIA0", pack=False, align=None), offset=0)}, name="_Anonymous3_e__Struct", pack=False, align=None)}, name="<anon>", label="None"), "flags": SimTypeInt(signed=False, label="IKEEXT_CERT_AUTH"), "localCertLocationUrl": SimStruct({"size": SimTypeInt(signed=False, label="UInt32"), "data": SimTypePointer(SimTypeChar(label="Byte"), offset=0)}, name="FWP_BYTE_BLOB", pack=False, align=None)}, name="IKEEXT_CERTIFICATE_AUTHENTICATION2", pack=False, align=None), "cgaAuthentication": SimStruct({"keyContainerName": SimTypePointer(SimTypeChar(label="Char"), offset=0), "cspName": SimTypePointer(SimTypeChar(label="Char"), offset=0), "cspType": SimTypeInt(signed=False, label="UInt32"), "cgaModifier": SimStruct({"byteArray16": SimTypeFixedSizeArray(SimTypeChar(label="Byte"), 16)}, name="FWP_BYTE_ARRAY16", pack=False, align=None), "cgaCollisionCount": SimTypeChar(label="Byte")}, name="IKEEXT_IPV6_CGA_AUTHENTICATION0", pack=False, align=None), "eapAuthentication": SimStruct({"flags": SimTypeInt(signed=False, label="IKEEXT_EAP_AUTHENTICATION_FLAGS")}, name="IKEEXT_EAP_AUTHENTICATION0", pack=False, align=None)}, name="<anon>", label="None")}, name="IKEEXT_AUTHENTICATION_METHOD2", pack=False, align=None), offset=0), "initiatorImpersonationType": SimTypeInt(signed=False, label="IKEEXT_AUTHENTICATION_IMPERSONATION_TYPE")}, name="IKEEXT_EM_POLICY2", pack=False, align=None), offset=0), "fwdPathSaLifetime": SimTypeInt(signed=False, label="UInt32"), "compartmentId": SimTypeInt(signed=False, label="UInt32"), "numTrafficSelectorPolicy": SimTypeInt(signed=False, label="UInt32"), "trafficSelectorPolicies": SimTypePointer(SimStruct({"flags": SimTypeInt(signed=False, label="UInt32"), "numLocalTrafficSelectors": SimTypeInt(signed=False, label="UInt32"), "localTrafficSelectors": SimTypePointer(SimStruct({"protocolId": SimTypeChar(label="Byte"), "portStart": SimTypeShort(signed=False, label="UInt16"), "portEnd": SimTypeShort(signed=False, label="UInt16"), "ipVersion": SimTypeInt(signed=False, label="FWP_IP_VERSION"), "Anonymous1": SimUnion({"startV4Address": SimTypeInt(signed=False, label="UInt32"), "startV6Address": SimTypeFixedSizeArray(SimTypeChar(label="Byte"), 16)}, name="<anon>", label="None"), "Anonymous2": SimUnion({"endV4Address": SimTypeInt(signed=False, label="UInt32"), "endV6Address": SimTypeFixedSizeArray(SimTypeChar(label="Byte"), 16)}, name="<anon>", label="None")}, name="IPSEC_TRAFFIC_SELECTOR0_", pack=False, align=None), offset=0), "numRemoteTrafficSelectors": SimTypeInt(signed=False, label="UInt32"), "remoteTrafficSelectors": SimTypePointer(SimStruct({"protocolId": SimTypeChar(label="Byte"), "portStart": SimTypeShort(signed=False, label="UInt16"), "portEnd": SimTypeShort(signed=False, label="UInt16"), "ipVersion": SimTypeInt(signed=False, label="FWP_IP_VERSION"), "Anonymous1": SimUnion({"startV4Address": SimTypeInt(signed=False, label="UInt32"), "startV6Address": SimTypeFixedSizeArray(SimTypeChar(label="Byte"), 16)}, name="<anon>", label="None"), "Anonymous2": SimUnion({"endV4Address": SimTypeInt(signed=False, label="UInt32"), "endV6Address": SimTypeFixedSizeArray(SimTypeChar(label="Byte"), 16)}, name="<anon>", label="None")}, name="IPSEC_TRAFFIC_SELECTOR0_", pack=False, align=None), offset=0)}, name="IPSEC_TRAFFIC_SELECTOR_POLICY0_", pack=False, align=None), offset=0)}, name="IPSEC_TUNNEL_POLICY3_", pack=False, align=None), offset=0), "authipQmTransportPolicy": SimTypePointer(SimStruct({"numIpsecProposals": SimTypeInt(signed=False, label="UInt32"), "ipsecProposals": SimTypePointer(SimStruct({"lifetime": SimStruct({"lifetimeSeconds": SimTypeInt(signed=False, label="UInt32"), "lifetimeKilobytes": SimTypeInt(signed=False, label="UInt32"), "lifetimePackets": SimTypeInt(signed=False, label="UInt32")}, name="IPSEC_SA_LIFETIME0", pack=False, align=None), "numSaTransforms": SimTypeInt(signed=False, label="UInt32"), "saTransforms": SimTypePointer(SimStruct({"ipsecTransformType": SimTypeInt(signed=False, label="IPSEC_TRANSFORM_TYPE"), "Anonymous": SimUnion({"ahTransform": SimTypePointer(SimStruct({"authTransformId": SimStruct({"authType": SimTypeInt(signed=False, label="IPSEC_AUTH_TYPE"), "authConfig": SimTypeChar(label="Byte")}, name="IPSEC_AUTH_TRANSFORM_ID0", pack=False, align=None), "cryptoModuleId": SimTypePointer(SimTypeBottom(label="Guid"), offset=0)}, name="IPSEC_AUTH_TRANSFORM0", pack=False, align=None), offset=0), "espAuthTransform": SimTypePointer(SimStruct({"authTransformId": SimStruct({"authType": SimTypeInt(signed=False, label="IPSEC_AUTH_TYPE"), "authConfig": SimTypeChar(label="Byte")}, name="IPSEC_AUTH_TRANSFORM_ID0", pack=False, align=None), "cryptoModuleId": SimTypePointer(SimTypeBottom(label="Guid"), offset=0)}, name="IPSEC_AUTH_TRANSFORM0", pack=False, align=None), offset=0), "espCipherTransform": SimTypePointer(SimStruct({"cipherTransformId": SimStruct({"cipherType": SimTypeInt(signed=False, label="IPSEC_CIPHER_TYPE"), "cipherConfig": SimTypeChar(label="Byte")}, name="IPSEC_CIPHER_TRANSFORM_ID0", pack=False, align=None), "cryptoModuleId": SimTypePointer(SimTypeBottom(label="Guid"), offset=0)}, name="IPSEC_CIPHER_TRANSFORM0", pack=False, align=None), offset=0), "espAuthAndCipherTransform": SimTypePointer(SimStruct({"authTransform": SimStruct({"authTransformId": SimStruct({"authType": SimTypeInt(signed=False, label="IPSEC_AUTH_TYPE"), "authConfig": SimTypeChar(label="Byte")}, name="IPSEC_AUTH_TRANSFORM_ID0", pack=False, align=None), "cryptoModuleId": SimTypePointer(SimTypeBottom(label="Guid"), offset=0)}, name="IPSEC_AUTH_TRANSFORM0", pack=False, align=None), "cipherTransform": SimStruct({"cipherTransformId": SimStruct({"cipherType": SimTypeInt(signed=False, label="IPSEC_CIPHER_TYPE"), "cipherConfig": SimTypeChar(label="Byte")}, name="IPSEC_CIPHER_TRANSFORM_ID0", pack=False, align=None), "cryptoModuleId": SimTypePointer(SimTypeBottom(label="Guid"), offset=0)}, name="IPSEC_CIPHER_TRANSFORM0", pack=False, align=None)}, name="IPSEC_AUTH_AND_CIPHER_TRANSFORM0", pack=False, align=None), offset=0), "espAuthFwTransform": SimTypePointer(SimStruct({"authTransformId": SimStruct({"authType": SimTypeInt(signed=False, label="IPSEC_AUTH_TYPE"), "authConfig": SimTypeChar(label="Byte")}, name="IPSEC_AUTH_TRANSFORM_ID0", pack=False, align=None), "cryptoModuleId": SimTypePointer(SimTypeBottom(label="Guid"), offset=0)}, name="IPSEC_AUTH_TRANSFORM0", pack=False, align=None), offset=0)}, name="<anon>", label="None")}, name="IPSEC_SA_TRANSFORM0", pack=False, align=None), offset=0), "pfsGroup": SimTypeInt(signed=False, label="IPSEC_PFS_GROUP")}, name="IPSEC_PROPOSAL0", pack=False, align=None), offset=0), "flags": SimTypeInt(signed=False, label="IPSEC_POLICY_FLAG"), "ndAllowClearTimeoutSeconds": SimTypeInt(signed=False, label="UInt32"), "saIdleTimeout": SimStruct({"idleTimeoutSeconds": SimTypeInt(signed=False, label="UInt32"), "idleTimeoutSecondsFailOver": SimTypeInt(signed=False, label="UInt32")}, name="IPSEC_SA_IDLE_TIMEOUT0", pack=False, align=None), "emPolicy": SimTypePointer(SimStruct({"numAuthenticationMethods": SimTypeInt(signed=False, label="UInt32"), "authenticationMethods": SimTypePointer(SimStruct({"authenticationMethodType": SimTypeInt(signed=False, label="IKEEXT_AUTHENTICATION_METHOD_TYPE"), "Anonymous": SimUnion({"presharedKeyAuthentication": SimStruct({"presharedKey": SimStruct({"size": SimTypeInt(signed=False, label="UInt32"), "data": SimTypePointer(SimTypeChar(label="Byte"), offset=0)}, name="FWP_BYTE_BLOB", pack=False, align=None), "flags": SimTypeInt(signed=False, label="IKEEXT_PRESHARED_KEY_AUTHENTICATION_FLAGS")}, name="IKEEXT_PRESHARED_KEY_AUTHENTICATION1", pack=False, align=None), "certificateAuthentication": SimStruct({"inboundConfigType": SimTypeInt(signed=False, label="IKEEXT_CERT_CONFIG_TYPE"), "Anonymous1": SimUnion({"Anonymous1": SimStruct({"inboundRootArraySize": SimTypeInt(signed=False, label="UInt32"), "inboundRootCriteria": SimTypePointer(SimStruct({"certData": SimStruct({"size": SimTypeInt(signed=False, label="UInt32"), "data": SimTypePointer(SimTypeChar(label="Byte"), offset=0)}, name="FWP_BYTE_BLOB", pack=False, align=None), "certHash": SimStruct({"size": SimTypeInt(signed=False, label="UInt32"), "data": SimTypePointer(SimTypeChar(label="Byte"), offset=0)}, name="FWP_BYTE_BLOB", pack=False, align=None), "eku": SimTypePointer(SimStruct({"numEku": SimTypeInt(signed=False, label="UInt32"), "eku": SimTypePointer(SimTypePointer(SimTypeChar(label="Byte"), offset=0), offset=0)}, name="IKEEXT_CERT_EKUS0", pack=False, align=None), offset=0), "name": SimTypePointer(SimStruct({"nameType": SimTypeInt(signed=False, label="IKEEXT_CERT_CRITERIA_NAME_TYPE"), "certName": SimTypePointer(SimTypeChar(label="Char"), offset=0)}, name="IKEEXT_CERT_NAME0", pack=False, align=None), offset=0), "flags": SimTypeInt(signed=False, label="UInt32")}, name="IKEEXT_CERTIFICATE_CRITERIA0", pack=False, align=None), offset=0)}, name="_Anonymous1_e__Struct", pack=False, align=None), "Anonymous2": SimStruct({"inboundEnterpriseStoreArraySize": SimTypeInt(signed=False, label="UInt32"), "inboundEnterpriseStoreCriteria": SimTypePointer(SimStruct({"certData": SimStruct({"size": SimTypeInt(signed=False, label="UInt32"), "data": SimTypePointer(SimTypeChar(label="Byte"), offset=0)}, name="FWP_BYTE_BLOB", pack=False, align=None), "certHash": SimStruct({"size": SimTypeInt(signed=False, label="UInt32"), "data": SimTypePointer(SimTypeChar(label="Byte"), offset=0)}, name="FWP_BYTE_BLOB", pack=False, align=None), "eku": SimTypePointer(SimStruct({"numEku": SimTypeInt(signed=False, label="UInt32"), "eku": SimTypePointer(SimTypePointer(SimTypeChar(label="Byte"), offset=0), offset=0)}, name="IKEEXT_CERT_EKUS0", pack=False, align=None), offset=0), "name": SimTypePointer(SimStruct({"nameType": SimTypeInt(signed=False, label="IKEEXT_CERT_CRITERIA_NAME_TYPE"), "certName": SimTypePointer(SimTypeChar(label="Char"), offset=0)}, name="IKEEXT_CERT_NAME0", pack=False, align=None), offset=0), "flags": SimTypeInt(signed=False, label="UInt32")}, name="IKEEXT_CERTIFICATE_CRITERIA0", pack=False, align=None), offset=0)}, name="_Anonymous2_e__Struct", pack=False, align=None), "Anonymous3": SimStruct({"inboundRootStoreArraySize": SimTypeInt(signed=False, label="UInt32"), "inboundTrustedRootStoreCriteria": SimTypePointer(SimStruct({"certData": SimStruct({"size": SimTypeInt(signed=False, label="UInt32"), "data": SimTypePointer(SimTypeChar(label="Byte"), offset=0)}, name="FWP_BYTE_BLOB", pack=False, align=None), "certHash": SimStruct({"size": SimTypeInt(signed=False, label="UInt32"), "data": SimTypePointer(SimTypeChar(label="Byte"), offset=0)}, name="FWP_BYTE_BLOB", pack=False, align=None), "eku": SimTypePointer(SimStruct({"numEku": SimTypeInt(signed=False, label="UInt32"), "eku": SimTypePointer(SimTypePointer(SimTypeChar(label="Byte"), offset=0), offset=0)}, name="IKEEXT_CERT_EKUS0", pack=False, align=None), offset=0), "name": SimTypePointer(SimStruct({"nameType": SimTypeInt(signed=False, label="IKEEXT_CERT_CRITERIA_NAME_TYPE"), "certName": SimTypePointer(SimTypeChar(label="Char"), offset=0)}, name="IKEEXT_CERT_NAME0", pack=False, align=None), offset=0), "flags": SimTypeInt(signed=False, label="UInt32")}, name="IKEEXT_CERTIFICATE_CRITERIA0", pack=False, align=None), offset=0)}, name="_Anonymous3_e__Struct", pack=False, align=None)}, name="<anon>", label="None"), "outboundConfigType": SimTypeInt(signed=False, label="IKEEXT_CERT_CONFIG_TYPE"), "Anonymous2": SimUnion({"Anonymous1": SimStruct({"outboundRootArraySize": SimTypeInt(signed=False, label="UInt32"), "outboundRootCriteria": SimTypePointer(SimStruct({"certData": SimStruct({"size": SimTypeInt(signed=False, label="UInt32"), "data": SimTypePointer(SimTypeChar(label="Byte"), offset=0)}, name="FWP_BYTE_BLOB", pack=False, align=None), "certHash": SimStruct({"size": SimTypeInt(signed=False, label="UInt32"), "data": SimTypePointer(SimTypeChar(label="Byte"), offset=0)}, name="FWP_BYTE_BLOB", pack=False, align=None), "eku": SimTypePointer(SimStruct({"numEku": SimTypeInt(signed=False, label="UInt32"), "eku": SimTypePointer(SimTypePointer(SimTypeChar(label="Byte"), offset=0), offset=0)}, name="IKEEXT_CERT_EKUS0", pack=False, align=None), offset=0), "name": SimTypePointer(SimStruct({"nameType": SimTypeInt(signed=False, label="IKEEXT_CERT_CRITERIA_NAME_TYPE"), "certName": SimTypePointer(SimTypeChar(label="Char"), offset=0)}, name="IKEEXT_CERT_NAME0", pack=False, align=None), offset=0), "flags": SimTypeInt(signed=False, label="UInt32")}, name="IKEEXT_CERTIFICATE_CRITERIA0", pack=False, align=None), offset=0)}, name="_Anonymous1_e__Struct", pack=False, align=None), "Anonymous2": SimStruct({"outboundEnterpriseStoreArraySize": SimTypeInt(signed=False, label="UInt32"), "outboundEnterpriseStoreCriteria": SimTypePointer(SimStruct({"certData": SimStruct({"size": SimTypeInt(signed=False, label="UInt32"), "data": SimTypePointer(SimTypeChar(label="Byte"), offset=0)}, name="FWP_BYTE_BLOB", pack=False, align=None), "certHash": SimStruct({"size": SimTypeInt(signed=False, label="UInt32"), "data": SimTypePointer(SimTypeChar(label="Byte"), offset=0)}, name="FWP_BYTE_BLOB", pack=False, align=None), "eku": SimTypePointer(SimStruct({"numEku": SimTypeInt(signed=False, label="UInt32"), "eku": SimTypePointer(SimTypePointer(SimTypeChar(label="Byte"), offset=0), offset=0)}, name="IKEEXT_CERT_EKUS0", pack=False, align=None), offset=0), "name": SimTypePointer(SimStruct({"nameType": SimTypeInt(signed=False, label="IKEEXT_CERT_CRITERIA_NAME_TYPE"), "certName": SimTypePointer(SimTypeChar(label="Char"), offset=0)}, name="IKEEXT_CERT_NAME0", pack=False, align=None), offset=0), "flags": SimTypeInt(signed=False, label="UInt32")}, name="IKEEXT_CERTIFICATE_CRITERIA0", pack=False, align=None), offset=0)}, name="_Anonymous2_e__Struct", pack=False, align=None), "Anonymous3": SimStruct({"outboundRootStoreArraySize": SimTypeInt(signed=False, label="UInt32"), "outboundTrustedRootStoreCriteria": SimTypePointer(SimStruct({"certData": SimStruct({"size": SimTypeInt(signed=False, label="UInt32"), "data": SimTypePointer(SimTypeChar(label="Byte"), offset=0)}, name="FWP_BYTE_BLOB", pack=False, align=None), "certHash": SimStruct({"size": SimTypeInt(signed=False, label="UInt32"), "data": SimTypePointer(SimTypeChar(label="Byte"), offset=0)}, name="FWP_BYTE_BLOB", pack=False, align=None), "eku": SimTypePointer(SimStruct({"numEku": SimTypeInt(signed=False, label="UInt32"), "eku": SimTypePointer(SimTypePointer(SimTypeChar(label="Byte"), offset=0), offset=0)}, name="IKEEXT_CERT_EKUS0", pack=False, align=None), offset=0), "name": SimTypePointer(SimStruct({"nameType": SimTypeInt(signed=False, label="IKEEXT_CERT_CRITERIA_NAME_TYPE"), "certName": SimTypePointer(SimTypeChar(label="Char"), offset=0)}, name="IKEEXT_CERT_NAME0", pack=False, align=None), offset=0), "flags": SimTypeInt(signed=False, label="UInt32")}, name="IKEEXT_CERTIFICATE_CRITERIA0", pack=False, align=None), offset=0)}, name="_Anonymous3_e__Struct", pack=False, align=None)}, name="<anon>", label="None"), "flags": SimTypeInt(signed=False, label="IKEEXT_CERT_AUTH"), "localCertLocationUrl": SimStruct({"size": SimTypeInt(signed=False, label="UInt32"), "data": SimTypePointer(SimTypeChar(label="Byte"), offset=0)}, name="FWP_BYTE_BLOB", pack=False, align=None)}, name="IKEEXT_CERTIFICATE_AUTHENTICATION2", pack=False, align=None), "kerberosAuthentication": SimStruct({"flags": SimTypeInt(signed=False, label="IKEEXT_KERBEROS_AUTHENTICATION_FLAGS"), "proxyServer": SimTypePointer(SimTypeChar(label="Char"), offset=0)}, name="IKEEXT_KERBEROS_AUTHENTICATION1", pack=False, align=None), "reservedAuthentication": SimStruct({"flags": SimTypeInt(signed=False, label="IKEEXT_RESERVED_AUTHENTICATION_FLAGS")}, name="IKEEXT_RESERVED_AUTHENTICATION0", pack=False, align=None), "ntlmV2Authentication": SimStruct({"flags": SimTypeInt(signed=False, label="UInt32")}, name="IKEEXT_NTLM_V2_AUTHENTICATION0", pack=False, align=None), "sslAuthentication": SimStruct({"inboundConfigType": SimTypeInt(signed=False, label="IKEEXT_CERT_CONFIG_TYPE"), "Anonymous1": SimUnion({"Anonymous1": SimStruct({"inboundRootArraySize": SimTypeInt(signed=False, label="UInt32"), "inboundRootCriteria": SimTypePointer(SimStruct({"certData": SimStruct({"size": SimTypeInt(signed=False, label="UInt32"), "data": SimTypePointer(SimTypeChar(label="Byte"), offset=0)}, name="FWP_BYTE_BLOB", pack=False, align=None), "certHash": SimStruct({"size": SimTypeInt(signed=False, label="UInt32"), "data": SimTypePointer(SimTypeChar(label="Byte"), offset=0)}, name="FWP_BYTE_BLOB", pack=False, align=None), "eku": SimTypePointer(SimStruct({"numEku": SimTypeInt(signed=False, label="UInt32"), "eku": SimTypePointer(SimTypePointer(SimTypeChar(label="Byte"), offset=0), offset=0)}, name="IKEEXT_CERT_EKUS0", pack=False, align=None), offset=0), "name": SimTypePointer(SimStruct({"nameType": SimTypeInt(signed=False, label="IKEEXT_CERT_CRITERIA_NAME_TYPE"), "certName": SimTypePointer(SimTypeChar(label="Char"), offset=0)}, name="IKEEXT_CERT_NAME0", pack=False, align=None), offset=0), "flags": SimTypeInt(signed=False, label="UInt32")}, name="IKEEXT_CERTIFICATE_CRITERIA0", pack=False, align=None), offset=0)}, name="_Anonymous1_e__Struct", pack=False, align=None), "Anonymous2": SimStruct({"inboundEnterpriseStoreArraySize": SimTypeInt(signed=False, label="UInt32"), "inboundEnterpriseStoreCriteria": SimTypePointer(SimStruct({"certData": SimStruct({"size": SimTypeInt(signed=False, label="UInt32"), "data": SimTypePointer(SimTypeChar(label="Byte"), offset=0)}, name="FWP_BYTE_BLOB", pack=False, align=None), "certHash": SimStruct({"size": SimTypeInt(signed=False, label="UInt32"), "data": SimTypePointer(SimTypeChar(label="Byte"), offset=0)}, name="FWP_BYTE_BLOB", pack=False, align=None), "eku": SimTypePointer(SimStruct({"numEku": SimTypeInt(signed=False, label="UInt32"), "eku": SimTypePointer(SimTypePointer(SimTypeChar(label="Byte"), offset=0), offset=0)}, name="IKEEXT_CERT_EKUS0", pack=False, align=None), offset=0), "name": SimTypePointer(SimStruct({"nameType": SimTypeInt(signed=False, label="IKEEXT_CERT_CRITERIA_NAME_TYPE"), "certName": SimTypePointer(SimTypeChar(label="Char"), offset=0)}, name="IKEEXT_CERT_NAME0", pack=False, align=None), offset=0), "flags": SimTypeInt(signed=False, label="UInt32")}, name="IKEEXT_CERTIFICATE_CRITERIA0", pack=False, align=None), offset=0)}, name="_Anonymous2_e__Struct", pack=False, align=None), "Anonymous3": SimStruct({"inboundRootStoreArraySize": SimTypeInt(signed=False, label="UInt32"), "inboundTrustedRootStoreCriteria": SimTypePointer(SimStruct({"certData": SimStruct({"size": SimTypeInt(signed=False, label="UInt32"), "data": SimTypePointer(SimTypeChar(label="Byte"), offset=0)}, name="FWP_BYTE_BLOB", pack=False, align=None), "certHash": SimStruct({"size": SimTypeInt(signed=False, label="UInt32"), "data": SimTypePointer(SimTypeChar(label="Byte"), offset=0)}, name="FWP_BYTE_BLOB", pack=False, align=None), "eku": SimTypePointer(SimStruct({"numEku": SimTypeInt(signed=False, label="UInt32"), "eku": SimTypePointer(SimTypePointer(SimTypeChar(label="Byte"), offset=0), offset=0)}, name="IKEEXT_CERT_EKUS0", pack=False, align=None), offset=0), "name": SimTypePointer(SimStruct({"nameType": SimTypeInt(signed=False, label="IKEEXT_CERT_CRITERIA_NAME_TYPE"), "certName": SimTypePointer(SimTypeChar(label="Char"), offset=0)}, name="IKEEXT_CERT_NAME0", pack=False, align=None), offset=0), "flags": SimTypeInt(signed=False, label="UInt32")}, name="IKEEXT_CERTIFICATE_CRITERIA0", pack=False, align=None), offset=0)}, name="_Anonymous3_e__Struct", pack=False, align=None)}, name="<anon>", label="None"), "outboundConfigType": SimTypeInt(signed=False, label="IKEEXT_CERT_CONFIG_TYPE"), "Anonymous2": SimUnion({"Anonymous1": SimStruct({"outboundRootArraySize": SimTypeInt(signed=False, label="UInt32"), "outboundRootCriteria": SimTypePointer(SimStruct({"certData": SimStruct({"size": SimTypeInt(signed=False, label="UInt32"), "data": SimTypePointer(SimTypeChar(label="Byte"), offset=0)}, name="FWP_BYTE_BLOB", pack=False, align=None), "certHash": SimStruct({"size": SimTypeInt(signed=False, label="UInt32"), "data": SimTypePointer(SimTypeChar(label="Byte"), offset=0)}, name="FWP_BYTE_BLOB", pack=False, align=None), "eku": SimTypePointer(SimStruct({"numEku": SimTypeInt(signed=False, label="UInt32"), "eku": SimTypePointer(SimTypePointer(SimTypeChar(label="Byte"), offset=0), offset=0)}, name="IKEEXT_CERT_EKUS0", pack=False, align=None), offset=0), "name": SimTypePointer(SimStruct({"nameType": SimTypeInt(signed=False, label="IKEEXT_CERT_CRITERIA_NAME_TYPE"), "certName": SimTypePointer(SimTypeChar(label="Char"), offset=0)}, name="IKEEXT_CERT_NAME0", pack=False, align=None), offset=0), "flags": SimTypeInt(signed=False, label="UInt32")}, name="IKEEXT_CERTIFICATE_CRITERIA0", pack=False, align=None), offset=0)}, name="_Anonymous1_e__Struct", pack=False, align=None), "Anonymous2": SimStruct({"outboundEnterpriseStoreArraySize": SimTypeInt(signed=False, label="UInt32"), "outboundEnterpriseStoreCriteria": SimTypePointer(SimStruct({"certData": SimStruct({"size": SimTypeInt(signed=False, label="UInt32"), "data": SimTypePointer(SimTypeChar(label="Byte"), offset=0)}, name="FWP_BYTE_BLOB", pack=False, align=None), "certHash": SimStruct({"size": SimTypeInt(signed=False, label="UInt32"), "data": SimTypePointer(SimTypeChar(label="Byte"), offset=0)}, name="FWP_BYTE_BLOB", pack=False, align=None), "eku": SimTypePointer(SimStruct({"numEku": SimTypeInt(signed=False, label="UInt32"), "eku": SimTypePointer(SimTypePointer(SimTypeChar(label="Byte"), offset=0), offset=0)}, name="IKEEXT_CERT_EKUS0", pack=False, align=None), offset=0), "name": SimTypePointer(SimStruct({"nameType": SimTypeInt(signed=False, label="IKEEXT_CERT_CRITERIA_NAME_TYPE"), "certName": SimTypePointer(SimTypeChar(label="Char"), offset=0)}, name="IKEEXT_CERT_NAME0", pack=False, align=None), offset=0), "flags": SimTypeInt(signed=False, label="UInt32")}, name="IKEEXT_CERTIFICATE_CRITERIA0", pack=False, align=None), offset=0)}, name="_Anonymous2_e__Struct", pack=False, align=None), "Anonymous3": SimStruct({"outboundRootStoreArraySize": SimTypeInt(signed=False, label="UInt32"), "outboundTrustedRootStoreCriteria": SimTypePointer(SimStruct({"certData": SimStruct({"size": SimTypeInt(signed=False, label="UInt32"), "data": SimTypePointer(SimTypeChar(label="Byte"), offset=0)}, name="FWP_BYTE_BLOB", pack=False, align=None), "certHash": SimStruct({"size": SimTypeInt(signed=False, label="UInt32"), "data": SimTypePointer(SimTypeChar(label="Byte"), offset=0)}, name="FWP_BYTE_BLOB", pack=False, align=None), "eku": SimTypePointer(SimStruct({"numEku": SimTypeInt(signed=False, label="UInt32"), "eku": SimTypePointer(SimTypePointer(SimTypeChar(label="Byte"), offset=0), offset=0)}, name="IKEEXT_CERT_EKUS0", pack=False, align=None), offset=0), "name": SimTypePointer(SimStruct({"nameType": SimTypeInt(signed=False, label="IKEEXT_CERT_CRITERIA_NAME_TYPE"), "certName": SimTypePointer(SimTypeChar(label="Char"), offset=0)}, name="IKEEXT_CERT_NAME0", pack=False, align=None), offset=0), "flags": SimTypeInt(signed=False, label="UInt32")}, name="IKEEXT_CERTIFICATE_CRITERIA0", pack=False, align=None), offset=0)}, name="_Anonymous3_e__Struct", pack=False, align=None)}, name="<anon>", label="None"), "flags": SimTypeInt(signed=False, label="IKEEXT_CERT_AUTH"), "localCertLocationUrl": SimStruct({"size": SimTypeInt(signed=False, label="UInt32"), "data": SimTypePointer(SimTypeChar(label="Byte"), offset=0)}, name="FWP_BYTE_BLOB", pack=False, align=None)}, name="IKEEXT_CERTIFICATE_AUTHENTICATION2", pack=False, align=None), "cgaAuthentication": SimStruct({"keyContainerName": SimTypePointer(SimTypeChar(label="Char"), offset=0), "cspName": SimTypePointer(SimTypeChar(label="Char"), offset=0), "cspType": SimTypeInt(signed=False, label="UInt32"), "cgaModifier": SimStruct({"byteArray16": SimTypeFixedSizeArray(SimTypeChar(label="Byte"), 16)}, name="FWP_BYTE_ARRAY16", pack=False, align=None), "cgaCollisionCount": SimTypeChar(label="Byte")}, name="IKEEXT_IPV6_CGA_AUTHENTICATION0", pack=False, align=None), "eapAuthentication": SimStruct({"flags": SimTypeInt(signed=False, label="IKEEXT_EAP_AUTHENTICATION_FLAGS")}, name="IKEEXT_EAP_AUTHENTICATION0", pack=False, align=None)}, name="<anon>", label="None")}, name="IKEEXT_AUTHENTICATION_METHOD2", pack=False, align=None), offset=0), "initiatorImpersonationType": SimTypeInt(signed=False, label="IKEEXT_AUTHENTICATION_IMPERSONATION_TYPE")}, name="IKEEXT_EM_POLICY2", pack=False, align=None), offset=0)}, name="IPSEC_TRANSPORT_POLICY2", pack=False, align=None), offset=0), "authipQmTunnelPolicy": SimTypePointer(SimStruct({"flags": SimTypeInt(signed=False, label="UInt32"), "numIpsecProposals": SimTypeInt(signed=False, label="UInt32"), "ipsecProposals": SimTypePointer(SimStruct({"lifetime": SimStruct({"lifetimeSeconds": SimTypeInt(signed=False, label="UInt32"), "lifetimeKilobytes": SimTypeInt(signed=False, label="UInt32"), "lifetimePackets": SimTypeInt(signed=False, label="UInt32")}, name="IPSEC_SA_LIFETIME0", pack=False, align=None), "numSaTransforms": SimTypeInt(signed=False, label="UInt32"), "saTransforms": SimTypePointer(SimStruct({"ipsecTransformType": SimTypeInt(signed=False, label="IPSEC_TRANSFORM_TYPE"), "Anonymous": SimUnion({"ahTransform": SimTypePointer(SimStruct({"authTransformId": SimStruct({"authType": SimTypeInt(signed=False, label="IPSEC_AUTH_TYPE"), "authConfig": SimTypeChar(label="Byte")}, name="IPSEC_AUTH_TRANSFORM_ID0", pack=False, align=None), "cryptoModuleId": SimTypePointer(SimTypeBottom(label="Guid"), offset=0)}, name="IPSEC_AUTH_TRANSFORM0", pack=False, align=None), offset=0), "espAuthTransform": SimTypePointer(SimStruct({"authTransformId": SimStruct({"authType": SimTypeInt(signed=False, label="IPSEC_AUTH_TYPE"), "authConfig": SimTypeChar(label="Byte")}, name="IPSEC_AUTH_TRANSFORM_ID0", pack=False, align=None), "cryptoModuleId": SimTypePointer(SimTypeBottom(label="Guid"), offset=0)}, name="IPSEC_AUTH_TRANSFORM0", pack=False, align=None), offset=0), "espCipherTransform": SimTypePointer(SimStruct({"cipherTransformId": SimStruct({"cipherType": SimTypeInt(signed=False, label="IPSEC_CIPHER_TYPE"), "cipherConfig": SimTypeChar(label="Byte")}, name="IPSEC_CIPHER_TRANSFORM_ID0", pack=False, align=None), "cryptoModuleId": SimTypePointer(SimTypeBottom(label="Guid"), offset=0)}, name="IPSEC_CIPHER_TRANSFORM0", pack=False, align=None), offset=0), "espAuthAndCipherTransform": SimTypePointer(SimStruct({"authTransform": SimStruct({"authTransformId": SimStruct({"authType": SimTypeInt(signed=False, label="IPSEC_AUTH_TYPE"), "authConfig": SimTypeChar(label="Byte")}, name="IPSEC_AUTH_TRANSFORM_ID0", pack=False, align=None), "cryptoModuleId": SimTypePointer(SimTypeBottom(label="Guid"), offset=0)}, name="IPSEC_AUTH_TRANSFORM0", pack=False, align=None), "cipherTransform": SimStruct({"cipherTransformId": SimStruct({"cipherType": SimTypeInt(signed=False, label="IPSEC_CIPHER_TYPE"), "cipherConfig": SimTypeChar(label="Byte")}, name="IPSEC_CIPHER_TRANSFORM_ID0", pack=False, align=None), "cryptoModuleId": SimTypePointer(SimTypeBottom(label="Guid"), offset=0)}, name="IPSEC_CIPHER_TRANSFORM0", pack=False, align=None)}, name="IPSEC_AUTH_AND_CIPHER_TRANSFORM0", pack=False, align=None), offset=0), "espAuthFwTransform": SimTypePointer(SimStruct({"authTransformId": SimStruct({"authType": SimTypeInt(signed=False, label="IPSEC_AUTH_TYPE"), "authConfig": SimTypeChar(label="Byte")}, name="IPSEC_AUTH_TRANSFORM_ID0", pack=False, align=None), "cryptoModuleId": SimTypePointer(SimTypeBottom(label="Guid"), offset=0)}, name="IPSEC_AUTH_TRANSFORM0", pack=False, align=None), offset=0)}, name="<anon>", label="None")}, name="IPSEC_SA_TRANSFORM0", pack=False, align=None), offset=0), "pfsGroup": SimTypeInt(signed=False, label="IPSEC_PFS_GROUP")}, name="IPSEC_PROPOSAL0", pack=False, align=None), offset=0), "tunnelEndpoints": SimStruct({"ipVersion": SimTypeInt(signed=False, label="FWP_IP_VERSION"), "Anonymous1": SimUnion({"localV4Address": SimTypeInt(signed=False, label="UInt32"), "localV6Address": SimTypeFixedSizeArray(SimTypeChar(label="Byte"), 16)}, name="<anon>", label="None"), "Anonymous2": SimUnion({"remoteV4Address": SimTypeInt(signed=False, label="UInt32"), "remoteV6Address": SimTypeFixedSizeArray(SimTypeChar(label="Byte"), 16)}, name="<anon>", label="None"), "localIfLuid": SimTypeLongLong(signed=False, label="UInt64"), "remoteFqdn": SimTypePointer(SimTypeChar(label="Char"), offset=0), "numAddresses": SimTypeInt(signed=False, label="UInt32"), "remoteAddresses": SimTypePointer(SimStruct({"ipVersion": SimTypeInt(signed=False, label="FWP_IP_VERSION"), "Anonymous": SimUnion({"v4Address": SimTypeInt(signed=False, label="UInt32"), "v6Address": SimTypeFixedSizeArray(SimTypeChar(label="Byte"), 16)}, name="<anon>", label="None")}, name="IPSEC_TUNNEL_ENDPOINT0", pack=False, align=None), offset=0)}, name="IPSEC_TUNNEL_ENDPOINTS2", pack=False, align=None), "saIdleTimeout": SimStruct({"idleTimeoutSeconds": SimTypeInt(signed=False, label="UInt32"), "idleTimeoutSecondsFailOver": SimTypeInt(signed=False, label="UInt32")}, name="IPSEC_SA_IDLE_TIMEOUT0", pack=False, align=None), "emPolicy": SimTypePointer(SimStruct({"numAuthenticationMethods": SimTypeInt(signed=False, label="UInt32"), "authenticationMethods": SimTypePointer(SimStruct({"authenticationMethodType": SimTypeInt(signed=False, label="IKEEXT_AUTHENTICATION_METHOD_TYPE"), "Anonymous": SimUnion({"presharedKeyAuthentication": SimStruct({"presharedKey": SimStruct({"size": SimTypeInt(signed=False, label="UInt32"), "data": SimTypePointer(SimTypeChar(label="Byte"), offset=0)}, name="FWP_BYTE_BLOB", pack=False, align=None), "flags": SimTypeInt(signed=False, label="IKEEXT_PRESHARED_KEY_AUTHENTICATION_FLAGS")}, name="IKEEXT_PRESHARED_KEY_AUTHENTICATION1", pack=False, align=None), "certificateAuthentication": SimStruct({"inboundConfigType": SimTypeInt(signed=False, label="IKEEXT_CERT_CONFIG_TYPE"), "Anonymous1": SimUnion({"Anonymous1": SimStruct({"inboundRootArraySize": SimTypeInt(signed=False, label="UInt32"), "inboundRootCriteria": SimTypePointer(SimStruct({"certData": SimStruct({"size": SimTypeInt(signed=False, label="UInt32"), "data": SimTypePointer(SimTypeChar(label="Byte"), offset=0)}, name="FWP_BYTE_BLOB", pack=False, align=None), "certHash": SimStruct({"size": SimTypeInt(signed=False, label="UInt32"), "data": SimTypePointer(SimTypeChar(label="Byte"), offset=0)}, name="FWP_BYTE_BLOB", pack=False, align=None), "eku": SimTypePointer(SimStruct({"numEku": SimTypeInt(signed=False, label="UInt32"), "eku": SimTypePointer(SimTypePointer(SimTypeChar(label="Byte"), offset=0), offset=0)}, name="IKEEXT_CERT_EKUS0", pack=False, align=None), offset=0), "name": SimTypePointer(SimStruct({"nameType": SimTypeInt(signed=False, label="IKEEXT_CERT_CRITERIA_NAME_TYPE"), "certName": SimTypePointer(SimTypeChar(label="Char"), offset=0)}, name="IKEEXT_CERT_NAME0", pack=False, align=None), offset=0), "flags": SimTypeInt(signed=False, label="UInt32")}, name="IKEEXT_CERTIFICATE_CRITERIA0", pack=False, align=None), offset=0)}, name="_Anonymous1_e__Struct", pack=False, align=None), "Anonymous2": SimStruct({"inboundEnterpriseStoreArraySize": SimTypeInt(signed=False, label="UInt32"), "inboundEnterpriseStoreCriteria": SimTypePointer(SimStruct({"certData": SimStruct({"size": SimTypeInt(signed=False, label="UInt32"), "data": SimTypePointer(SimTypeChar(label="Byte"), offset=0)}, name="FWP_BYTE_BLOB", pack=False, align=None), "certHash": SimStruct({"size": SimTypeInt(signed=False, label="UInt32"), "data": SimTypePointer(SimTypeChar(label="Byte"), offset=0)}, name="FWP_BYTE_BLOB", pack=False, align=None), "eku": SimTypePointer(SimStruct({"numEku": SimTypeInt(signed=False, label="UInt32"), "eku": SimTypePointer(SimTypePointer(SimTypeChar(label="Byte"), offset=0), offset=0)}, name="IKEEXT_CERT_EKUS0", pack=False, align=None), offset=0), "name": SimTypePointer(SimStruct({"nameType": SimTypeInt(signed=False, label="IKEEXT_CERT_CRITERIA_NAME_TYPE"), "certName": SimTypePointer(SimTypeChar(label="Char"), offset=0)}, name="IKEEXT_CERT_NAME0", pack=False, align=None), offset=0), "flags": SimTypeInt(signed=False, label="UInt32")}, name="IKEEXT_CERTIFICATE_CRITERIA0", pack=False, align=None), offset=0)}, name="_Anonymous2_e__Struct", pack=False, align=None), "Anonymous3": SimStruct({"inboundRootStoreArraySize": SimTypeInt(signed=False, label="UInt32"), "inboundTrustedRootStoreCriteria": SimTypePointer(SimStruct({"certData": SimStruct({"size": SimTypeInt(signed=False, label="UInt32"), "data": SimTypePointer(SimTypeChar(label="Byte"), offset=0)}, name="FWP_BYTE_BLOB", pack=False, align=None), "certHash": SimStruct({"size": SimTypeInt(signed=False, label="UInt32"), "data": SimTypePointer(SimTypeChar(label="Byte"), offset=0)}, name="FWP_BYTE_BLOB", pack=False, align=None), "eku": SimTypePointer(SimStruct({"numEku": SimTypeInt(signed=False, label="UInt32"), "eku": SimTypePointer(SimTypePointer(SimTypeChar(label="Byte"), offset=0), offset=0)}, name="IKEEXT_CERT_EKUS0", pack=False, align=None), offset=0), "name": SimTypePointer(SimStruct({"nameType": SimTypeInt(signed=False, label="IKEEXT_CERT_CRITERIA_NAME_TYPE"), "certName": SimTypePointer(SimTypeChar(label="Char"), offset=0)}, name="IKEEXT_CERT_NAME0", pack=False, align=None), offset=0), "flags": SimTypeInt(signed=False, label="UInt32")}, name="IKEEXT_CERTIFICATE_CRITERIA0", pack=False, align=None), offset=0)}, name="_Anonymous3_e__Struct", pack=False, align=None)}, name="<anon>", label="None"), "outboundConfigType": SimTypeInt(signed=False, label="IKEEXT_CERT_CONFIG_TYPE"), "Anonymous2": SimUnion({"Anonymous1": SimStruct({"outboundRootArraySize": SimTypeInt(signed=False, label="UInt32"), "outboundRootCriteria": SimTypePointer(SimStruct({"certData": SimStruct({"size": SimTypeInt(signed=False, label="UInt32"), "data": SimTypePointer(SimTypeChar(label="Byte"), offset=0)}, name="FWP_BYTE_BLOB", pack=False, align=None), "certHash": SimStruct({"size": SimTypeInt(signed=False, label="UInt32"), "data": SimTypePointer(SimTypeChar(label="Byte"), offset=0)}, name="FWP_BYTE_BLOB", pack=False, align=None), "eku": SimTypePointer(SimStruct({"numEku": SimTypeInt(signed=False, label="UInt32"), "eku": SimTypePointer(SimTypePointer(SimTypeChar(label="Byte"), offset=0), offset=0)}, name="IKEEXT_CERT_EKUS0", pack=False, align=None), offset=0), "name": SimTypePointer(SimStruct({"nameType": SimTypeInt(signed=False, label="IKEEXT_CERT_CRITERIA_NAME_TYPE"), "certName": SimTypePointer(SimTypeChar(label="Char"), offset=0)}, name="IKEEXT_CERT_NAME0", pack=False, align=None), offset=0), "flags": SimTypeInt(signed=False, label="UInt32")}, name="IKEEXT_CERTIFICATE_CRITERIA0", pack=False, align=None), offset=0)}, name="_Anonymous1_e__Struct", pack=False, align=None), "Anonymous2": SimStruct({"outboundEnterpriseStoreArraySize": SimTypeInt(signed=False, label="UInt32"), "outboundEnterpriseStoreCriteria": SimTypePointer(SimStruct({"certData": SimStruct({"size": SimTypeInt(signed=False, label="UInt32"), "data": SimTypePointer(SimTypeChar(label="Byte"), offset=0)}, name="FWP_BYTE_BLOB", pack=False, align=None), "certHash": SimStruct({"size": SimTypeInt(signed=False, label="UInt32"), "data": SimTypePointer(SimTypeChar(label="Byte"), offset=0)}, name="FWP_BYTE_BLOB", pack=False, align=None), "eku": SimTypePointer(SimStruct({"numEku": SimTypeInt(signed=False, label="UInt32"), "eku": SimTypePointer(SimTypePointer(SimTypeChar(label="Byte"), offset=0), offset=0)}, name="IKEEXT_CERT_EKUS0", pack=False, align=None), offset=0), "name": SimTypePointer(SimStruct({"nameType": SimTypeInt(signed=False, label="IKEEXT_CERT_CRITERIA_NAME_TYPE"), "certName": SimTypePointer(SimTypeChar(label="Char"), offset=0)}, name="IKEEXT_CERT_NAME0", pack=False, align=None), offset=0), "flags": SimTypeInt(signed=False, label="UInt32")}, name="IKEEXT_CERTIFICATE_CRITERIA0", pack=False, align=None), offset=0)}, name="_Anonymous2_e__Struct", pack=False, align=None), "Anonymous3": SimStruct({"outboundRootStoreArraySize": SimTypeInt(signed=False, label="UInt32"), "outboundTrustedRootStoreCriteria": SimTypePointer(SimStruct({"certData": SimStruct({"size": SimTypeInt(signed=False, label="UInt32"), "data": SimTypePointer(SimTypeChar(label="Byte"), offset=0)}, name="FWP_BYTE_BLOB", pack=False, align=None), "certHash": SimStruct({"size": SimTypeInt(signed=False, label="UInt32"), "data": SimTypePointer(SimTypeChar(label="Byte"), offset=0)}, name="FWP_BYTE_BLOB", pack=False, align=None), "eku": SimTypePointer(SimStruct({"numEku": SimTypeInt(signed=False, label="UInt32"), "eku": SimTypePointer(SimTypePointer(SimTypeChar(label="Byte"), offset=0), offset=0)}, name="IKEEXT_CERT_EKUS0", pack=False, align=None), offset=0), "name": SimTypePointer(SimStruct({"nameType": SimTypeInt(signed=False, label="IKEEXT_CERT_CRITERIA_NAME_TYPE"), "certName": SimTypePointer(SimTypeChar(label="Char"), offset=0)}, name="IKEEXT_CERT_NAME0", pack=False, align=None), offset=0), "flags": SimTypeInt(signed=False, label="UInt32")}, name="IKEEXT_CERTIFICATE_CRITERIA0", pack=False, align=None), offset=0)}, name="_Anonymous3_e__Struct", pack=False, align=None)}, name="<anon>", label="None"), "flags": SimTypeInt(signed=False, label="IKEEXT_CERT_AUTH"), "localCertLocationUrl": SimStruct({"size": SimTypeInt(signed=False, label="UInt32"), "data": SimTypePointer(SimTypeChar(label="Byte"), offset=0)}, name="FWP_BYTE_BLOB", pack=False, align=None)}, name="IKEEXT_CERTIFICATE_AUTHENTICATION2", pack=False, align=None), "kerberosAuthentication": SimStruct({"flags": SimTypeInt(signed=False, label="IKEEXT_KERBEROS_AUTHENTICATION_FLAGS"), "proxyServer": SimTypePointer(SimTypeChar(label="Char"), offset=0)}, name="IKEEXT_KERBEROS_AUTHENTICATION1", pack=False, align=None), "reservedAuthentication": SimStruct({"flags": SimTypeInt(signed=False, label="IKEEXT_RESERVED_AUTHENTICATION_FLAGS")}, name="IKEEXT_RESERVED_AUTHENTICATION0", pack=False, align=None), "ntlmV2Authentication": SimStruct({"flags": SimTypeInt(signed=False, label="UInt32")}, name="IKEEXT_NTLM_V2_AUTHENTICATION0", pack=False, align=None), "sslAuthentication": SimStruct({"inboundConfigType": SimTypeInt(signed=False, label="IKEEXT_CERT_CONFIG_TYPE"), "Anonymous1": SimUnion({"Anonymous1": SimStruct({"inboundRootArraySize": SimTypeInt(signed=False, label="UInt32"), "inboundRootCriteria": SimTypePointer(SimStruct({"certData": SimStruct({"size": SimTypeInt(signed=False, label="UInt32"), "data": SimTypePointer(SimTypeChar(label="Byte"), offset=0)}, name="FWP_BYTE_BLOB", pack=False, align=None), "certHash": SimStruct({"size": SimTypeInt(signed=False, label="UInt32"), "data": SimTypePointer(SimTypeChar(label="Byte"), offset=0)}, name="FWP_BYTE_BLOB", pack=False, align=None), "eku": SimTypePointer(SimStruct({"numEku": SimTypeInt(signed=False, label="UInt32"), "eku": SimTypePointer(SimTypePointer(SimTypeChar(label="Byte"), offset=0), offset=0)}, name="IKEEXT_CERT_EKUS0", pack=False, align=None), offset=0), "name": SimTypePointer(SimStruct({"nameType": SimTypeInt(signed=False, label="IKEEXT_CERT_CRITERIA_NAME_TYPE"), "certName": SimTypePointer(SimTypeChar(label="Char"), offset=0)}, name="IKEEXT_CERT_NAME0", pack=False, align=None), offset=0), "flags": SimTypeInt(signed=False, label="UInt32")}, name="IKEEXT_CERTIFICATE_CRITERIA0", pack=False, align=None), offset=0)}, name="_Anonymous1_e__Struct", pack=False, align=None), "Anonymous2": SimStruct({"inboundEnterpriseStoreArraySize": SimTypeInt(signed=False, label="UInt32"), "inboundEnterpriseStoreCriteria": SimTypePointer(SimStruct({"certData": SimStruct({"size": SimTypeInt(signed=False, label="UInt32"), "data": SimTypePointer(SimTypeChar(label="Byte"), offset=0)}, name="FWP_BYTE_BLOB", pack=False, align=None), "certHash": SimStruct({"size": SimTypeInt(signed=False, label="UInt32"), "data": SimTypePointer(SimTypeChar(label="Byte"), offset=0)}, name="FWP_BYTE_BLOB", pack=False, align=None), "eku": SimTypePointer(SimStruct({"numEku": SimTypeInt(signed=False, label="UInt32"), "eku": SimTypePointer(SimTypePointer(SimTypeChar(label="Byte"), offset=0), offset=0)}, name="IKEEXT_CERT_EKUS0", pack=False, align=None), offset=0), "name": SimTypePointer(SimStruct({"nameType": SimTypeInt(signed=False, label="IKEEXT_CERT_CRITERIA_NAME_TYPE"), "certName": SimTypePointer(SimTypeChar(label="Char"), offset=0)}, name="IKEEXT_CERT_NAME0", pack=False, align=None), offset=0), "flags": SimTypeInt(signed=False, label="UInt32")}, name="IKEEXT_CERTIFICATE_CRITERIA0", pack=False, align=None), offset=0)}, name="_Anonymous2_e__Struct", pack=False, align=None), "Anonymous3": SimStruct({"inboundRootStoreArraySize": SimTypeInt(signed=False, label="UInt32"), "inboundTrustedRootStoreCriteria": SimTypePointer(SimStruct({"certData": SimStruct({"size": SimTypeInt(signed=False, label="UInt32"), "data": SimTypePointer(SimTypeChar(label="Byte"), offset=0)}, name="FWP_BYTE_BLOB", pack=False, align=None), "certHash": SimStruct({"size": SimTypeInt(signed=False, label="UInt32"), "data": SimTypePointer(SimTypeChar(label="Byte"), offset=0)}, name="FWP_BYTE_BLOB", pack=False, align=None), "eku": SimTypePointer(SimStruct({"numEku": SimTypeInt(signed=False, label="UInt32"), "eku": SimTypePointer(SimTypePointer(SimTypeChar(label="Byte"), offset=0), offset=0)}, name="IKEEXT_CERT_EKUS0", pack=False, align=None), offset=0), "name": SimTypePointer(SimStruct({"nameType": SimTypeInt(signed=False, label="IKEEXT_CERT_CRITERIA_NAME_TYPE"), "certName": SimTypePointer(SimTypeChar(label="Char"), offset=0)}, name="IKEEXT_CERT_NAME0", pack=False, align=None), offset=0), "flags": SimTypeInt(signed=False, label="UInt32")}, name="IKEEXT_CERTIFICATE_CRITERIA0", pack=False, align=None), offset=0)}, name="_Anonymous3_e__Struct", pack=False, align=None)}, name="<anon>", label="None"), "outboundConfigType": SimTypeInt(signed=False, label="IKEEXT_CERT_CONFIG_TYPE"), "Anonymous2": SimUnion({"Anonymous1": SimStruct({"outboundRootArraySize": SimTypeInt(signed=False, label="UInt32"), "outboundRootCriteria": SimTypePointer(SimStruct({"certData": SimStruct({"size": SimTypeInt(signed=False, label="UInt32"), "data": SimTypePointer(SimTypeChar(label="Byte"), offset=0)}, name="FWP_BYTE_BLOB", pack=False, align=None), "certHash": SimStruct({"size": SimTypeInt(signed=False, label="UInt32"), "data": SimTypePointer(SimTypeChar(label="Byte"), offset=0)}, name="FWP_BYTE_BLOB", pack=False, align=None), "eku": SimTypePointer(SimStruct({"numEku": SimTypeInt(signed=False, label="UInt32"), "eku": SimTypePointer(SimTypePointer(SimTypeChar(label="Byte"), offset=0), offset=0)}, name="IKEEXT_CERT_EKUS0", pack=False, align=None), offset=0), "name": SimTypePointer(SimStruct({"nameType": SimTypeInt(signed=False, label="IKEEXT_CERT_CRITERIA_NAME_TYPE"), "certName": SimTypePointer(SimTypeChar(label="Char"), offset=0)}, name="IKEEXT_CERT_NAME0", pack=False, align=None), offset=0), "flags": SimTypeInt(signed=False, label="UInt32")}, name="IKEEXT_CERTIFICATE_CRITERIA0", pack=False, align=None), offset=0)}, name="_Anonymous1_e__Struct", pack=False, align=None), "Anonymous2": SimStruct({"outboundEnterpriseStoreArraySize": SimTypeInt(signed=False, label="UInt32"), "outboundEnterpriseStoreCriteria": SimTypePointer(SimStruct({"certData": SimStruct({"size": SimTypeInt(signed=False, label="UInt32"), "data": SimTypePointer(SimTypeChar(label="Byte"), offset=0)}, name="FWP_BYTE_BLOB", pack=False, align=None), "certHash": SimStruct({"size": SimTypeInt(signed=False, label="UInt32"), "data": SimTypePointer(SimTypeChar(label="Byte"), offset=0)}, name="FWP_BYTE_BLOB", pack=False, align=None), "eku": SimTypePointer(SimStruct({"numEku": SimTypeInt(signed=False, label="UInt32"), "eku": SimTypePointer(SimTypePointer(SimTypeChar(label="Byte"), offset=0), offset=0)}, name="IKEEXT_CERT_EKUS0", pack=False, align=None), offset=0), "name": SimTypePointer(SimStruct({"nameType": SimTypeInt(signed=False, label="IKEEXT_CERT_CRITERIA_NAME_TYPE"), "certName": SimTypePointer(SimTypeChar(label="Char"), offset=0)}, name="IKEEXT_CERT_NAME0", pack=False, align=None), offset=0), "flags": SimTypeInt(signed=False, label="UInt32")}, name="IKEEXT_CERTIFICATE_CRITERIA0", pack=False, align=None), offset=0)}, name="_Anonymous2_e__Struct", pack=False, align=None), "Anonymous3": SimStruct({"outboundRootStoreArraySize": SimTypeInt(signed=False, label="UInt32"), "outboundTrustedRootStoreCriteria": SimTypePointer(SimStruct({"certData": SimStruct({"size": SimTypeInt(signed=False, label="UInt32"), "data": SimTypePointer(SimTypeChar(label="Byte"), offset=0)}, name="FWP_BYTE_BLOB", pack=False, align=None), "certHash": SimStruct({"size": SimTypeInt(signed=False, label="UInt32"), "data": SimTypePointer(SimTypeChar(label="Byte"), offset=0)}, name="FWP_BYTE_BLOB", pack=False, align=None), "eku": SimTypePointer(SimStruct({"numEku": SimTypeInt(signed=False, label="UInt32"), "eku": SimTypePointer(SimTypePointer(SimTypeChar(label="Byte"), offset=0), offset=0)}, name="IKEEXT_CERT_EKUS0", pack=False, align=None), offset=0), "name": SimTypePointer(SimStruct({"nameType": SimTypeInt(signed=False, label="IKEEXT_CERT_CRITERIA_NAME_TYPE"), "certName": SimTypePointer(SimTypeChar(label="Char"), offset=0)}, name="IKEEXT_CERT_NAME0", pack=False, align=None), offset=0), "flags": SimTypeInt(signed=False, label="UInt32")}, name="IKEEXT_CERTIFICATE_CRITERIA0", pack=False, align=None), offset=0)}, name="_Anonymous3_e__Struct", pack=False, align=None)}, name="<anon>", label="None"), "flags": SimTypeInt(signed=False, label="IKEEXT_CERT_AUTH"), "localCertLocationUrl": SimStruct({"size": SimTypeInt(signed=False, label="UInt32"), "data": SimTypePointer(SimTypeChar(label="Byte"), offset=0)}, name="FWP_BYTE_BLOB", pack=False, align=None)}, name="IKEEXT_CERTIFICATE_AUTHENTICATION2", pack=False, align=None), "cgaAuthentication": SimStruct({"keyContainerName": SimTypePointer(SimTypeChar(label="Char"), offset=0), "cspName": SimTypePointer(SimTypeChar(label="Char"), offset=0), "cspType": SimTypeInt(signed=False, label="UInt32"), "cgaModifier": SimStruct({"byteArray16": SimTypeFixedSizeArray(SimTypeChar(label="Byte"), 16)}, name="FWP_BYTE_ARRAY16", pack=False, align=None), "cgaCollisionCount": SimTypeChar(label="Byte")}, name="IKEEXT_IPV6_CGA_AUTHENTICATION0", pack=False, align=None), "eapAuthentication": SimStruct({"flags": SimTypeInt(signed=False, label="IKEEXT_EAP_AUTHENTICATION_FLAGS")}, name="IKEEXT_EAP_AUTHENTICATION0", pack=False, align=None)}, name="<anon>", label="None")}, name="IKEEXT_AUTHENTICATION_METHOD2", pack=False, align=None), offset=0), "initiatorImpersonationType": SimTypeInt(signed=False, label="IKEEXT_AUTHENTICATION_IMPERSONATION_TYPE")}, name="IKEEXT_EM_POLICY2", pack=False, align=None), offset=0), "fwdPathSaLifetime": SimTypeInt(signed=False, label="UInt32"), "compartmentId": SimTypeInt(signed=False, label="UInt32"), "numTrafficSelectorPolicy": SimTypeInt(signed=False, label="UInt32"), "trafficSelectorPolicies": SimTypePointer(SimStruct({"flags": SimTypeInt(signed=False, label="UInt32"), "numLocalTrafficSelectors": SimTypeInt(signed=False, label="UInt32"), "localTrafficSelectors": SimTypePointer(SimStruct({"protocolId": SimTypeChar(label="Byte"), "portStart": SimTypeShort(signed=False, label="UInt16"), "portEnd": SimTypeShort(signed=False, label="UInt16"), "ipVersion": SimTypeInt(signed=False, label="FWP_IP_VERSION"), "Anonymous1": SimUnion({"startV4Address": SimTypeInt(signed=False, label="UInt32"), "startV6Address": SimTypeFixedSizeArray(SimTypeChar(label="Byte"), 16)}, name="<anon>", label="None"), "Anonymous2": SimUnion({"endV4Address": SimTypeInt(signed=False, label="UInt32"), "endV6Address": SimTypeFixedSizeArray(SimTypeChar(label="Byte"), 16)}, name="<anon>", label="None")}, name="IPSEC_TRAFFIC_SELECTOR0_", pack=False, align=None), offset=0), "numRemoteTrafficSelectors": SimTypeInt(signed=False, label="UInt32"), "remoteTrafficSelectors": SimTypePointer(SimStruct({"protocolId": SimTypeChar(label="Byte"), "portStart": SimTypeShort(signed=False, label="UInt16"), "portEnd": SimTypeShort(signed=False, label="UInt16"), "ipVersion": SimTypeInt(signed=False, label="FWP_IP_VERSION"), "Anonymous1": SimUnion({"startV4Address": SimTypeInt(signed=False, label="UInt32"), "startV6Address": SimTypeFixedSizeArray(SimTypeChar(label="Byte"), 16)}, name="<anon>", label="None"), "Anonymous2": SimUnion({"endV4Address": SimTypeInt(signed=False, label="UInt32"), "endV6Address": SimTypeFixedSizeArray(SimTypeChar(label="Byte"), 16)}, name="<anon>", label="None")}, name="IPSEC_TRAFFIC_SELECTOR0_", pack=False, align=None), offset=0)}, name="IPSEC_TRAFFIC_SELECTOR_POLICY0_", pack=False, align=None), offset=0)}, name="IPSEC_TUNNEL_POLICY3_", pack=False, align=None), offset=0), "ikeMmPolicy": SimTypePointer(SimStruct({"softExpirationTime": SimTypeInt(signed=False, label="UInt32"), "numAuthenticationMethods": SimTypeInt(signed=False, label="UInt32"), "authenticationMethods": SimTypePointer(SimStruct({"authenticationMethodType": SimTypeInt(signed=False, label="IKEEXT_AUTHENTICATION_METHOD_TYPE"), "Anonymous": SimUnion({"presharedKeyAuthentication": SimStruct({"presharedKey": SimStruct({"size": SimTypeInt(signed=False, label="UInt32"), "data": SimTypePointer(SimTypeChar(label="Byte"), offset=0)}, name="FWP_BYTE_BLOB", pack=False, align=None), "flags": SimTypeInt(signed=False, label="IKEEXT_PRESHARED_KEY_AUTHENTICATION_FLAGS")}, name="IKEEXT_PRESHARED_KEY_AUTHENTICATION1", pack=False, align=None), "certificateAuthentication": SimStruct({"inboundConfigType": SimTypeInt(signed=False, label="IKEEXT_CERT_CONFIG_TYPE"), "Anonymous1": SimUnion({"Anonymous1": SimStruct({"inboundRootArraySize": SimTypeInt(signed=False, label="UInt32"), "inboundRootCriteria": SimTypePointer(SimStruct({"certData": SimStruct({"size": SimTypeInt(signed=False, label="UInt32"), "data": SimTypePointer(SimTypeChar(label="Byte"), offset=0)}, name="FWP_BYTE_BLOB", pack=False, align=None), "certHash": SimStruct({"size": SimTypeInt(signed=False, label="UInt32"), "data": SimTypePointer(SimTypeChar(label="Byte"), offset=0)}, name="FWP_BYTE_BLOB", pack=False, align=None), "eku": SimTypePointer(SimStruct({"numEku": SimTypeInt(signed=False, label="UInt32"), "eku": SimTypePointer(SimTypePointer(SimTypeChar(label="Byte"), offset=0), offset=0)}, name="IKEEXT_CERT_EKUS0", pack=False, align=None), offset=0), "name": SimTypePointer(SimStruct({"nameType": SimTypeInt(signed=False, label="IKEEXT_CERT_CRITERIA_NAME_TYPE"), "certName": SimTypePointer(SimTypeChar(label="Char"), offset=0)}, name="IKEEXT_CERT_NAME0", pack=False, align=None), offset=0), "flags": SimTypeInt(signed=False, label="UInt32")}, name="IKEEXT_CERTIFICATE_CRITERIA0", pack=False, align=None), offset=0)}, name="_Anonymous1_e__Struct", pack=False, align=None), "Anonymous2": SimStruct({"inboundEnterpriseStoreArraySize": SimTypeInt(signed=False, label="UInt32"), "inboundEnterpriseStoreCriteria": SimTypePointer(SimStruct({"certData": SimStruct({"size": SimTypeInt(signed=False, label="UInt32"), "data": SimTypePointer(SimTypeChar(label="Byte"), offset=0)}, name="FWP_BYTE_BLOB", pack=False, align=None), "certHash": SimStruct({"size": SimTypeInt(signed=False, label="UInt32"), "data": SimTypePointer(SimTypeChar(label="Byte"), offset=0)}, name="FWP_BYTE_BLOB", pack=False, align=None), "eku": SimTypePointer(SimStruct({"numEku": SimTypeInt(signed=False, label="UInt32"), "eku": SimTypePointer(SimTypePointer(SimTypeChar(label="Byte"), offset=0), offset=0)}, name="IKEEXT_CERT_EKUS0", pack=False, align=None), offset=0), "name": SimTypePointer(SimStruct({"nameType": SimTypeInt(signed=False, label="IKEEXT_CERT_CRITERIA_NAME_TYPE"), "certName": SimTypePointer(SimTypeChar(label="Char"), offset=0)}, name="IKEEXT_CERT_NAME0", pack=False, align=None), offset=0), "flags": SimTypeInt(signed=False, label="UInt32")}, name="IKEEXT_CERTIFICATE_CRITERIA0", pack=False, align=None), offset=0)}, name="_Anonymous2_e__Struct", pack=False, align=None), "Anonymous3": SimStruct({"inboundRootStoreArraySize": SimTypeInt(signed=False, label="UInt32"), "inboundTrustedRootStoreCriteria": SimTypePointer(SimStruct({"certData": SimStruct({"size": SimTypeInt(signed=False, label="UInt32"), "data": SimTypePointer(SimTypeChar(label="Byte"), offset=0)}, name="FWP_BYTE_BLOB", pack=False, align=None), "certHash": SimStruct({"size": SimTypeInt(signed=False, label="UInt32"), "data": SimTypePointer(SimTypeChar(label="Byte"), offset=0)}, name="FWP_BYTE_BLOB", pack=False, align=None), "eku": SimTypePointer(SimStruct({"numEku": SimTypeInt(signed=False, label="UInt32"), "eku": SimTypePointer(SimTypePointer(SimTypeChar(label="Byte"), offset=0), offset=0)}, name="IKEEXT_CERT_EKUS0", pack=False, align=None), offset=0), "name": SimTypePointer(SimStruct({"nameType": SimTypeInt(signed=False, label="IKEEXT_CERT_CRITERIA_NAME_TYPE"), "certName": SimTypePointer(SimTypeChar(label="Char"), offset=0)}, name="IKEEXT_CERT_NAME0", pack=False, align=None), offset=0), "flags": SimTypeInt(signed=False, label="UInt32")}, name="IKEEXT_CERTIFICATE_CRITERIA0", pack=False, align=None), offset=0)}, name="_Anonymous3_e__Struct", pack=False, align=None)}, name="<anon>", label="None"), "outboundConfigType": SimTypeInt(signed=False, label="IKEEXT_CERT_CONFIG_TYPE"), "Anonymous2": SimUnion({"Anonymous1": SimStruct({"outboundRootArraySize": SimTypeInt(signed=False, label="UInt32"), "outboundRootCriteria": SimTypePointer(SimStruct({"certData": SimStruct({"size": SimTypeInt(signed=False, label="UInt32"), "data": SimTypePointer(SimTypeChar(label="Byte"), offset=0)}, name="FWP_BYTE_BLOB", pack=False, align=None), "certHash": SimStruct({"size": SimTypeInt(signed=False, label="UInt32"), "data": SimTypePointer(SimTypeChar(label="Byte"), offset=0)}, name="FWP_BYTE_BLOB", pack=False, align=None), "eku": SimTypePointer(SimStruct({"numEku": SimTypeInt(signed=False, label="UInt32"), "eku": SimTypePointer(SimTypePointer(SimTypeChar(label="Byte"), offset=0), offset=0)}, name="IKEEXT_CERT_EKUS0", pack=False, align=None), offset=0), "name": SimTypePointer(SimStruct({"nameType": SimTypeInt(signed=False, label="IKEEXT_CERT_CRITERIA_NAME_TYPE"), "certName": SimTypePointer(SimTypeChar(label="Char"), offset=0)}, name="IKEEXT_CERT_NAME0", pack=False, align=None), offset=0), "flags": SimTypeInt(signed=False, label="UInt32")}, name="IKEEXT_CERTIFICATE_CRITERIA0", pack=False, align=None), offset=0)}, name="_Anonymous1_e__Struct", pack=False, align=None), "Anonymous2": SimStruct({"outboundEnterpriseStoreArraySize": SimTypeInt(signed=False, label="UInt32"), "outboundEnterpriseStoreCriteria": SimTypePointer(SimStruct({"certData": SimStruct({"size": SimTypeInt(signed=False, label="UInt32"), "data": SimTypePointer(SimTypeChar(label="Byte"), offset=0)}, name="FWP_BYTE_BLOB", pack=False, align=None), "certHash": SimStruct({"size": SimTypeInt(signed=False, label="UInt32"), "data": SimTypePointer(SimTypeChar(label="Byte"), offset=0)}, name="FWP_BYTE_BLOB", pack=False, align=None), "eku": SimTypePointer(SimStruct({"numEku": SimTypeInt(signed=False, label="UInt32"), "eku": SimTypePointer(SimTypePointer(SimTypeChar(label="Byte"), offset=0), offset=0)}, name="IKEEXT_CERT_EKUS0", pack=False, align=None), offset=0), "name": SimTypePointer(SimStruct({"nameType": SimTypeInt(signed=False, label="IKEEXT_CERT_CRITERIA_NAME_TYPE"), "certName": SimTypePointer(SimTypeChar(label="Char"), offset=0)}, name="IKEEXT_CERT_NAME0", pack=False, align=None), offset=0), "flags": SimTypeInt(signed=False, label="UInt32")}, name="IKEEXT_CERTIFICATE_CRITERIA0", pack=False, align=None), offset=0)}, name="_Anonymous2_e__Struct", pack=False, align=None), "Anonymous3": SimStruct({"outboundRootStoreArraySize": SimTypeInt(signed=False, label="UInt32"), "outboundTrustedRootStoreCriteria": SimTypePointer(SimStruct({"certData": SimStruct({"size": SimTypeInt(signed=False, label="UInt32"), "data": SimTypePointer(SimTypeChar(label="Byte"), offset=0)}, name="FWP_BYTE_BLOB", pack=False, align=None), "certHash": SimStruct({"size": SimTypeInt(signed=False, label="UInt32"), "data": SimTypePointer(SimTypeChar(label="Byte"), offset=0)}, name="FWP_BYTE_BLOB", pack=False, align=None), "eku": SimTypePointer(SimStruct({"numEku": SimTypeInt(signed=False, label="UInt32"), "eku": SimTypePointer(SimTypePointer(SimTypeChar(label="Byte"), offset=0), offset=0)}, name="IKEEXT_CERT_EKUS0", pack=False, align=None), offset=0), "name": SimTypePointer(SimStruct({"nameType": SimTypeInt(signed=False, label="IKEEXT_CERT_CRITERIA_NAME_TYPE"), "certName": SimTypePointer(SimTypeChar(label="Char"), offset=0)}, name="IKEEXT_CERT_NAME0", pack=False, align=None), offset=0), "flags": SimTypeInt(signed=False, label="UInt32")}, name="IKEEXT_CERTIFICATE_CRITERIA0", pack=False, align=None), offset=0)}, name="_Anonymous3_e__Struct", pack=False, align=None)}, name="<anon>", label="None"), "flags": SimTypeInt(signed=False, label="IKEEXT_CERT_AUTH"), "localCertLocationUrl": SimStruct({"size": SimTypeInt(signed=False, label="UInt32"), "data": SimTypePointer(SimTypeChar(label="Byte"), offset=0)}, name="FWP_BYTE_BLOB", pack=False, align=None)}, name="IKEEXT_CERTIFICATE_AUTHENTICATION2", pack=False, align=None), "kerberosAuthentication": SimStruct({"flags": SimTypeInt(signed=False, label="IKEEXT_KERBEROS_AUTHENTICATION_FLAGS"), "proxyServer": SimTypePointer(SimTypeChar(label="Char"), offset=0)}, name="IKEEXT_KERBEROS_AUTHENTICATION1", pack=False, align=None), "reservedAuthentication": SimStruct({"flags": SimTypeInt(signed=False, label="IKEEXT_RESERVED_AUTHENTICATION_FLAGS")}, name="IKEEXT_RESERVED_AUTHENTICATION0", pack=False, align=None), "ntlmV2Authentication": SimStruct({"flags": SimTypeInt(signed=False, label="UInt32")}, name="IKEEXT_NTLM_V2_AUTHENTICATION0", pack=False, align=None), "sslAuthentication": SimStruct({"inboundConfigType": SimTypeInt(signed=False, label="IKEEXT_CERT_CONFIG_TYPE"), "Anonymous1": SimUnion({"Anonymous1": SimStruct({"inboundRootArraySize": SimTypeInt(signed=False, label="UInt32"), "inboundRootCriteria": SimTypePointer(SimStruct({"certData": SimStruct({"size": SimTypeInt(signed=False, label="UInt32"), "data": SimTypePointer(SimTypeChar(label="Byte"), offset=0)}, name="FWP_BYTE_BLOB", pack=False, align=None), "certHash": SimStruct({"size": SimTypeInt(signed=False, label="UInt32"), "data": SimTypePointer(SimTypeChar(label="Byte"), offset=0)}, name="FWP_BYTE_BLOB", pack=False, align=None), "eku": SimTypePointer(SimStruct({"numEku": SimTypeInt(signed=False, label="UInt32"), "eku": SimTypePointer(SimTypePointer(SimTypeChar(label="Byte"), offset=0), offset=0)}, name="IKEEXT_CERT_EKUS0", pack=False, align=None), offset=0), "name": SimTypePointer(SimStruct({"nameType": SimTypeInt(signed=False, label="IKEEXT_CERT_CRITERIA_NAME_TYPE"), "certName": SimTypePointer(SimTypeChar(label="Char"), offset=0)}, name="IKEEXT_CERT_NAME0", pack=False, align=None), offset=0), "flags": SimTypeInt(signed=False, label="UInt32")}, name="IKEEXT_CERTIFICATE_CRITERIA0", pack=False, align=None), offset=0)}, name="_Anonymous1_e__Struct", pack=False, align=None), "Anonymous2": SimStruct({"inboundEnterpriseStoreArraySize": SimTypeInt(signed=False, label="UInt32"), "inboundEnterpriseStoreCriteria": SimTypePointer(SimStruct({"certData": SimStruct({"size": SimTypeInt(signed=False, label="UInt32"), "data": SimTypePointer(SimTypeChar(label="Byte"), offset=0)}, name="FWP_BYTE_BLOB", pack=False, align=None), "certHash": SimStruct({"size": SimTypeInt(signed=False, label="UInt32"), "data": SimTypePointer(SimTypeChar(label="Byte"), offset=0)}, name="FWP_BYTE_BLOB", pack=False, align=None), "eku": SimTypePointer(SimStruct({"numEku": SimTypeInt(signed=False, label="UInt32"), "eku": SimTypePointer(SimTypePointer(SimTypeChar(label="Byte"), offset=0), offset=0)}, name="IKEEXT_CERT_EKUS0", pack=False, align=None), offset=0), "name": SimTypePointer(SimStruct({"nameType": SimTypeInt(signed=False, label="IKEEXT_CERT_CRITERIA_NAME_TYPE"), "certName": SimTypePointer(SimTypeChar(label="Char"), offset=0)}, name="IKEEXT_CERT_NAME0", pack=False, align=None), offset=0), "flags": SimTypeInt(signed=False, label="UInt32")}, name="IKEEXT_CERTIFICATE_CRITERIA0", pack=False, align=None), offset=0)}, name="_Anonymous2_e__Struct", pack=False, align=None), "Anonymous3": SimStruct({"inboundRootStoreArraySize": SimTypeInt(signed=False, label="UInt32"), "inboundTrustedRootStoreCriteria": SimTypePointer(SimStruct({"certData": SimStruct({"size": SimTypeInt(signed=False, label="UInt32"), "data": SimTypePointer(SimTypeChar(label="Byte"), offset=0)}, name="FWP_BYTE_BLOB", pack=False, align=None), "certHash": SimStruct({"size": SimTypeInt(signed=False, label="UInt32"), "data": SimTypePointer(SimTypeChar(label="Byte"), offset=0)}, name="FWP_BYTE_BLOB", pack=False, align=None), "eku": SimTypePointer(SimStruct({"numEku": SimTypeInt(signed=False, label="UInt32"), "eku": SimTypePointer(SimTypePointer(SimTypeChar(label="Byte"), offset=0), offset=0)}, name="IKEEXT_CERT_EKUS0", pack=False, align=None), offset=0), "name": SimTypePointer(SimStruct({"nameType": SimTypeInt(signed=False, label="IKEEXT_CERT_CRITERIA_NAME_TYPE"), "certName": SimTypePointer(SimTypeChar(label="Char"), offset=0)}, name="IKEEXT_CERT_NAME0", pack=False, align=None), offset=0), "flags": SimTypeInt(signed=False, label="UInt32")}, name="IKEEXT_CERTIFICATE_CRITERIA0", pack=False, align=None), offset=0)}, name="_Anonymous3_e__Struct", pack=False, align=None)}, name="<anon>", label="None"), "outboundConfigType": SimTypeInt(signed=False, label="IKEEXT_CERT_CONFIG_TYPE"), "Anonymous2": SimUnion({"Anonymous1": SimStruct({"outboundRootArraySize": SimTypeInt(signed=False, label="UInt32"), "outboundRootCriteria": SimTypePointer(SimStruct({"certData": SimStruct({"size": SimTypeInt(signed=False, label="UInt32"), "data": SimTypePointer(SimTypeChar(label="Byte"), offset=0)}, name="FWP_BYTE_BLOB", pack=False, align=None), "certHash": SimStruct({"size": SimTypeInt(signed=False, label="UInt32"), "data": SimTypePointer(SimTypeChar(label="Byte"), offset=0)}, name="FWP_BYTE_BLOB", pack=False, align=None), "eku": SimTypePointer(SimStruct({"numEku": SimTypeInt(signed=False, label="UInt32"), "eku": SimTypePointer(SimTypePointer(SimTypeChar(label="Byte"), offset=0), offset=0)}, name="IKEEXT_CERT_EKUS0", pack=False, align=None), offset=0), "name": SimTypePointer(SimStruct({"nameType": SimTypeInt(signed=False, label="IKEEXT_CERT_CRITERIA_NAME_TYPE"), "certName": SimTypePointer(SimTypeChar(label="Char"), offset=0)}, name="IKEEXT_CERT_NAME0", pack=False, align=None), offset=0), "flags": SimTypeInt(signed=False, label="UInt32")}, name="IKEEXT_CERTIFICATE_CRITERIA0", pack=False, align=None), offset=0)}, name="_Anonymous1_e__Struct", pack=False, align=None), "Anonymous2": SimStruct({"outboundEnterpriseStoreArraySize": SimTypeInt(signed=False, label="UInt32"), "outboundEnterpriseStoreCriteria": SimTypePointer(SimStruct({"certData": SimStruct({"size": SimTypeInt(signed=False, label="UInt32"), "data": SimTypePointer(SimTypeChar(label="Byte"), offset=0)}, name="FWP_BYTE_BLOB", pack=False, align=None), "certHash": SimStruct({"size": SimTypeInt(signed=False, label="UInt32"), "data": SimTypePointer(SimTypeChar(label="Byte"), offset=0)}, name="FWP_BYTE_BLOB", pack=False, align=None), "eku": SimTypePointer(SimStruct({"numEku": SimTypeInt(signed=False, label="UInt32"), "eku": SimTypePointer(SimTypePointer(SimTypeChar(label="Byte"), offset=0), offset=0)}, name="IKEEXT_CERT_EKUS0", pack=False, align=None), offset=0), "name": SimTypePointer(SimStruct({"nameType": SimTypeInt(signed=False, label="IKEEXT_CERT_CRITERIA_NAME_TYPE"), "certName": SimTypePointer(SimTypeChar(label="Char"), offset=0)}, name="IKEEXT_CERT_NAME0", pack=False, align=None), offset=0), "flags": SimTypeInt(signed=False, label="UInt32")}, name="IKEEXT_CERTIFICATE_CRITERIA0", pack=False, align=None), offset=0)}, name="_Anonymous2_e__Struct", pack=False, align=None), "Anonymous3": SimStruct({"outboundRootStoreArraySize": SimTypeInt(signed=False, label="UInt32"), "outboundTrustedRootStoreCriteria": SimTypePointer(SimStruct({"certData": SimStruct({"size": SimTypeInt(signed=False, label="UInt32"), "data": SimTypePointer(SimTypeChar(label="Byte"), offset=0)}, name="FWP_BYTE_BLOB", pack=False, align=None), "certHash": SimStruct({"size": SimTypeInt(signed=False, label="UInt32"), "data": SimTypePointer(SimTypeChar(label="Byte"), offset=0)}, name="FWP_BYTE_BLOB", pack=False, align=None), "eku": SimTypePointer(SimStruct({"numEku": SimTypeInt(signed=False, label="UInt32"), "eku": SimTypePointer(SimTypePointer(SimTypeChar(label="Byte"), offset=0), offset=0)}, name="IKEEXT_CERT_EKUS0", pack=False, align=None), offset=0), "name": SimTypePointer(SimStruct({"nameType": SimTypeInt(signed=False, label="IKEEXT_CERT_CRITERIA_NAME_TYPE"), "certName": SimTypePointer(SimTypeChar(label="Char"), offset=0)}, name="IKEEXT_CERT_NAME0", pack=False, align=None), offset=0), "flags": SimTypeInt(signed=False, label="UInt32")}, name="IKEEXT_CERTIFICATE_CRITERIA0", pack=False, align=None), offset=0)}, name="_Anonymous3_e__Struct", pack=False, align=None)}, name="<anon>", label="None"), "flags": SimTypeInt(signed=False, label="IKEEXT_CERT_AUTH"), "localCertLocationUrl": SimStruct({"size": SimTypeInt(signed=False, label="UInt32"), "data": SimTypePointer(SimTypeChar(label="Byte"), offset=0)}, name="FWP_BYTE_BLOB", pack=False, align=None)}, name="IKEEXT_CERTIFICATE_AUTHENTICATION2", pack=False, align=None), "cgaAuthentication": SimStruct({"keyContainerName": SimTypePointer(SimTypeChar(label="Char"), offset=0), "cspName": SimTypePointer(SimTypeChar(label="Char"), offset=0), "cspType": SimTypeInt(signed=False, label="UInt32"), "cgaModifier": SimStruct({"byteArray16": SimTypeFixedSizeArray(SimTypeChar(label="Byte"), 16)}, name="FWP_BYTE_ARRAY16", pack=False, align=None), "cgaCollisionCount": SimTypeChar(label="Byte")}, name="IKEEXT_IPV6_CGA_AUTHENTICATION0", pack=False, align=None), "eapAuthentication": SimStruct({"flags": SimTypeInt(signed=False, label="IKEEXT_EAP_AUTHENTICATION_FLAGS")}, name="IKEEXT_EAP_AUTHENTICATION0", pack=False, align=None)}, name="<anon>", label="None")}, name="IKEEXT_AUTHENTICATION_METHOD2", pack=False, align=None), offset=0), "initiatorImpersonationType": SimTypeInt(signed=False, label="IKEEXT_AUTHENTICATION_IMPERSONATION_TYPE"), "numIkeProposals": SimTypeInt(signed=False, label="UInt32"), "ikeProposals": SimTypePointer(SimStruct({"cipherAlgorithm": SimStruct({"algoIdentifier": SimTypeInt(signed=False, label="IKEEXT_CIPHER_TYPE"), "keyLen": SimTypeInt(signed=False, label="UInt32"), "rounds": SimTypeInt(signed=False, label="UInt32")}, name="IKEEXT_CIPHER_ALGORITHM0", pack=False, align=None), "integrityAlgorithm": SimStruct({"algoIdentifier": SimTypeInt(signed=False, label="IKEEXT_INTEGRITY_TYPE")}, name="IKEEXT_INTEGRITY_ALGORITHM0", pack=False, align=None), "maxLifetimeSeconds": SimTypeInt(signed=False, label="UInt32"), "dhGroup": SimTypeInt(signed=False, label="IKEEXT_DH_GROUP"), "quickModeLimit": SimTypeInt(signed=False, label="UInt32")}, name="IKEEXT_PROPOSAL0", pack=False, align=None), offset=0), "flags": SimTypeInt(signed=False, label="IKEEXT_POLICY_FLAG"), "maxDynamicFilters": SimTypeInt(signed=False, label="UInt32"), "retransmitDurationSecs": SimTypeInt(signed=False, label="UInt32")}, name="IKEEXT_POLICY2", pack=False, align=None), offset=0), "authIpMmPolicy": SimTypePointer(SimStruct({"softExpirationTime": SimTypeInt(signed=False, label="UInt32"), "numAuthenticationMethods": SimTypeInt(signed=False, label="UInt32"), "authenticationMethods": SimTypePointer(SimStruct({"authenticationMethodType": SimTypeInt(signed=False, label="IKEEXT_AUTHENTICATION_METHOD_TYPE"), "Anonymous": SimUnion({"presharedKeyAuthentication": SimStruct({"presharedKey": SimStruct({"size": SimTypeInt(signed=False, label="UInt32"), "data": SimTypePointer(SimTypeChar(label="Byte"), offset=0)}, name="FWP_BYTE_BLOB", pack=False, align=None), "flags": SimTypeInt(signed=False, label="IKEEXT_PRESHARED_KEY_AUTHENTICATION_FLAGS")}, name="IKEEXT_PRESHARED_KEY_AUTHENTICATION1", pack=False, align=None), "certificateAuthentication": SimStruct({"inboundConfigType": SimTypeInt(signed=False, label="IKEEXT_CERT_CONFIG_TYPE"), "Anonymous1": SimUnion({"Anonymous1": SimStruct({"inboundRootArraySize": SimTypeInt(signed=False, label="UInt32"), "inboundRootCriteria": SimTypePointer(SimStruct({"certData": SimStruct({"size": SimTypeInt(signed=False, label="UInt32"), "data": SimTypePointer(SimTypeChar(label="Byte"), offset=0)}, name="FWP_BYTE_BLOB", pack=False, align=None), "certHash": SimStruct({"size": SimTypeInt(signed=False, label="UInt32"), "data": SimTypePointer(SimTypeChar(label="Byte"), offset=0)}, name="FWP_BYTE_BLOB", pack=False, align=None), "eku": SimTypePointer(SimStruct({"numEku": SimTypeInt(signed=False, label="UInt32"), "eku": SimTypePointer(SimTypePointer(SimTypeChar(label="Byte"), offset=0), offset=0)}, name="IKEEXT_CERT_EKUS0", pack=False, align=None), offset=0), "name": SimTypePointer(SimStruct({"nameType": SimTypeInt(signed=False, label="IKEEXT_CERT_CRITERIA_NAME_TYPE"), "certName": SimTypePointer(SimTypeChar(label="Char"), offset=0)}, name="IKEEXT_CERT_NAME0", pack=False, align=None), offset=0), "flags": SimTypeInt(signed=False, label="UInt32")}, name="IKEEXT_CERTIFICATE_CRITERIA0", pack=False, align=None), offset=0)}, name="_Anonymous1_e__Struct", pack=False, align=None), "Anonymous2": SimStruct({"inboundEnterpriseStoreArraySize": SimTypeInt(signed=False, label="UInt32"), "inboundEnterpriseStoreCriteria": SimTypePointer(SimStruct({"certData": SimStruct({"size": SimTypeInt(signed=False, label="UInt32"), "data": SimTypePointer(SimTypeChar(label="Byte"), offset=0)}, name="FWP_BYTE_BLOB", pack=False, align=None), "certHash": SimStruct({"size": SimTypeInt(signed=False, label="UInt32"), "data": SimTypePointer(SimTypeChar(label="Byte"), offset=0)}, name="FWP_BYTE_BLOB", pack=False, align=None), "eku": SimTypePointer(SimStruct({"numEku": SimTypeInt(signed=False, label="UInt32"), "eku": SimTypePointer(SimTypePointer(SimTypeChar(label="Byte"), offset=0), offset=0)}, name="IKEEXT_CERT_EKUS0", pack=False, align=None), offset=0), "name": SimTypePointer(SimStruct({"nameType": SimTypeInt(signed=False, label="IKEEXT_CERT_CRITERIA_NAME_TYPE"), "certName": SimTypePointer(SimTypeChar(label="Char"), offset=0)}, name="IKEEXT_CERT_NAME0", pack=False, align=None), offset=0), "flags": SimTypeInt(signed=False, label="UInt32")}, name="IKEEXT_CERTIFICATE_CRITERIA0", pack=False, align=None), offset=0)}, name="_Anonymous2_e__Struct", pack=False, align=None), "Anonymous3": SimStruct({"inboundRootStoreArraySize": SimTypeInt(signed=False, label="UInt32"), "inboundTrustedRootStoreCriteria": SimTypePointer(SimStruct({"certData": SimStruct({"size": SimTypeInt(signed=False, label="UInt32"), "data": SimTypePointer(SimTypeChar(label="Byte"), offset=0)}, name="FWP_BYTE_BLOB", pack=False, align=None), "certHash": SimStruct({"size": SimTypeInt(signed=False, label="UInt32"), "data": SimTypePointer(SimTypeChar(label="Byte"), offset=0)}, name="FWP_BYTE_BLOB", pack=False, align=None), "eku": SimTypePointer(SimStruct({"numEku": SimTypeInt(signed=False, label="UInt32"), "eku": SimTypePointer(SimTypePointer(SimTypeChar(label="Byte"), offset=0), offset=0)}, name="IKEEXT_CERT_EKUS0", pack=False, align=None), offset=0), "name": SimTypePointer(SimStruct({"nameType": SimTypeInt(signed=False, label="IKEEXT_CERT_CRITERIA_NAME_TYPE"), "certName": SimTypePointer(SimTypeChar(label="Char"), offset=0)}, name="IKEEXT_CERT_NAME0", pack=False, align=None), offset=0), "flags": SimTypeInt(signed=False, label="UInt32")}, name="IKEEXT_CERTIFICATE_CRITERIA0", pack=False, align=None), offset=0)}, name="_Anonymous3_e__Struct", pack=False, align=None)}, name="<anon>", label="None"), "outboundConfigType": SimTypeInt(signed=False, label="IKEEXT_CERT_CONFIG_TYPE"), "Anonymous2": SimUnion({"Anonymous1": SimStruct({"outboundRootArraySize": SimTypeInt(signed=False, label="UInt32"), "outboundRootCriteria": SimTypePointer(SimStruct({"certData": SimStruct({"size": SimTypeInt(signed=False, label="UInt32"), "data": SimTypePointer(SimTypeChar(label="Byte"), offset=0)}, name="FWP_BYTE_BLOB", pack=False, align=None), "certHash": SimStruct({"size": SimTypeInt(signed=False, label="UInt32"), "data": SimTypePointer(SimTypeChar(label="Byte"), offset=0)}, name="FWP_BYTE_BLOB", pack=False, align=None), "eku": SimTypePointer(SimStruct({"numEku": SimTypeInt(signed=False, label="UInt32"), "eku": SimTypePointer(SimTypePointer(SimTypeChar(label="Byte"), offset=0), offset=0)}, name="IKEEXT_CERT_EKUS0", pack=False, align=None), offset=0), "name": SimTypePointer(SimStruct({"nameType": SimTypeInt(signed=False, label="IKEEXT_CERT_CRITERIA_NAME_TYPE"), "certName": SimTypePointer(SimTypeChar(label="Char"), offset=0)}, name="IKEEXT_CERT_NAME0", pack=False, align=None), offset=0), "flags": SimTypeInt(signed=False, label="UInt32")}, name="IKEEXT_CERTIFICATE_CRITERIA0", pack=False, align=None), offset=0)}, name="_Anonymous1_e__Struct", pack=False, align=None), "Anonymous2": SimStruct({"outboundEnterpriseStoreArraySize": SimTypeInt(signed=False, label="UInt32"), "outboundEnterpriseStoreCriteria": SimTypePointer(SimStruct({"certData": SimStruct({"size": SimTypeInt(signed=False, label="UInt32"), "data": SimTypePointer(SimTypeChar(label="Byte"), offset=0)}, name="FWP_BYTE_BLOB", pack=False, align=None), "certHash": SimStruct({"size": SimTypeInt(signed=False, label="UInt32"), "data": SimTypePointer(SimTypeChar(label="Byte"), offset=0)}, name="FWP_BYTE_BLOB", pack=False, align=None), "eku": SimTypePointer(SimStruct({"numEku": SimTypeInt(signed=False, label="UInt32"), "eku": SimTypePointer(SimTypePointer(SimTypeChar(label="Byte"), offset=0), offset=0)}, name="IKEEXT_CERT_EKUS0", pack=False, align=None), offset=0), "name": SimTypePointer(SimStruct({"nameType": SimTypeInt(signed=False, label="IKEEXT_CERT_CRITERIA_NAME_TYPE"), "certName": SimTypePointer(SimTypeChar(label="Char"), offset=0)}, name="IKEEXT_CERT_NAME0", pack=False, align=None), offset=0), "flags": SimTypeInt(signed=False, label="UInt32")}, name="IKEEXT_CERTIFICATE_CRITERIA0", pack=False, align=None), offset=0)}, name="_Anonymous2_e__Struct", pack=False, align=None), "Anonymous3": SimStruct({"outboundRootStoreArraySize": SimTypeInt(signed=False, label="UInt32"), "outboundTrustedRootStoreCriteria": SimTypePointer(SimStruct({"certData": SimStruct({"size": SimTypeInt(signed=False, label="UInt32"), "data": SimTypePointer(SimTypeChar(label="Byte"), offset=0)}, name="FWP_BYTE_BLOB", pack=False, align=None), "certHash": SimStruct({"size": SimTypeInt(signed=False, label="UInt32"), "data": SimTypePointer(SimTypeChar(label="Byte"), offset=0)}, name="FWP_BYTE_BLOB", pack=False, align=None), "eku": SimTypePointer(SimStruct({"numEku": SimTypeInt(signed=False, label="UInt32"), "eku": SimTypePointer(SimTypePointer(SimTypeChar(label="Byte"), offset=0), offset=0)}, name="IKEEXT_CERT_EKUS0", pack=False, align=None), offset=0), "name": SimTypePointer(SimStruct({"nameType": SimTypeInt(signed=False, label="IKEEXT_CERT_CRITERIA_NAME_TYPE"), "certName": SimTypePointer(SimTypeChar(label="Char"), offset=0)}, name="IKEEXT_CERT_NAME0", pack=False, align=None), offset=0), "flags": SimTypeInt(signed=False, label="UInt32")}, name="IKEEXT_CERTIFICATE_CRITERIA0", pack=False, align=None), offset=0)}, name="_Anonymous3_e__Struct", pack=False, align=None)}, name="<anon>", label="None"), "flags": SimTypeInt(signed=False, label="IKEEXT_CERT_AUTH"), "localCertLocationUrl": SimStruct({"size": SimTypeInt(signed=False, label="UInt32"), "data": SimTypePointer(SimTypeChar(label="Byte"), offset=0)}, name="FWP_BYTE_BLOB", pack=False, align=None)}, name="IKEEXT_CERTIFICATE_AUTHENTICATION2", pack=False, align=None), "kerberosAuthentication": SimStruct({"flags": SimTypeInt(signed=False, label="IKEEXT_KERBEROS_AUTHENTICATION_FLAGS"), "proxyServer": SimTypePointer(SimTypeChar(label="Char"), offset=0)}, name="IKEEXT_KERBEROS_AUTHENTICATION1", pack=False, align=None), "reservedAuthentication": SimStruct({"flags": SimTypeInt(signed=False, label="IKEEXT_RESERVED_AUTHENTICATION_FLAGS")}, name="IKEEXT_RESERVED_AUTHENTICATION0", pack=False, align=None), "ntlmV2Authentication": SimStruct({"flags": SimTypeInt(signed=False, label="UInt32")}, name="IKEEXT_NTLM_V2_AUTHENTICATION0", pack=False, align=None), "sslAuthentication": SimStruct({"inboundConfigType": SimTypeInt(signed=False, label="IKEEXT_CERT_CONFIG_TYPE"), "Anonymous1": SimUnion({"Anonymous1": SimStruct({"inboundRootArraySize": SimTypeInt(signed=False, label="UInt32"), "inboundRootCriteria": SimTypePointer(SimStruct({"certData": SimStruct({"size": SimTypeInt(signed=False, label="UInt32"), "data": SimTypePointer(SimTypeChar(label="Byte"), offset=0)}, name="FWP_BYTE_BLOB", pack=False, align=None), "certHash": SimStruct({"size": SimTypeInt(signed=False, label="UInt32"), "data": SimTypePointer(SimTypeChar(label="Byte"), offset=0)}, name="FWP_BYTE_BLOB", pack=False, align=None), "eku": SimTypePointer(SimStruct({"numEku": SimTypeInt(signed=False, label="UInt32"), "eku": SimTypePointer(SimTypePointer(SimTypeChar(label="Byte"), offset=0), offset=0)}, name="IKEEXT_CERT_EKUS0", pack=False, align=None), offset=0), "name": SimTypePointer(SimStruct({"nameType": SimTypeInt(signed=False, label="IKEEXT_CERT_CRITERIA_NAME_TYPE"), "certName": SimTypePointer(SimTypeChar(label="Char"), offset=0)}, name="IKEEXT_CERT_NAME0", pack=False, align=None), offset=0), "flags": SimTypeInt(signed=False, label="UInt32")}, name="IKEEXT_CERTIFICATE_CRITERIA0", pack=False, align=None), offset=0)}, name="_Anonymous1_e__Struct", pack=False, align=None), "Anonymous2": SimStruct({"inboundEnterpriseStoreArraySize": SimTypeInt(signed=False, label="UInt32"), "inboundEnterpriseStoreCriteria": SimTypePointer(SimStruct({"certData": SimStruct({"size": SimTypeInt(signed=False, label="UInt32"), "data": SimTypePointer(SimTypeChar(label="Byte"), offset=0)}, name="FWP_BYTE_BLOB", pack=False, align=None), "certHash": SimStruct({"size": SimTypeInt(signed=False, label="UInt32"), "data": SimTypePointer(SimTypeChar(label="Byte"), offset=0)}, name="FWP_BYTE_BLOB", pack=False, align=None), "eku": SimTypePointer(SimStruct({"numEku": SimTypeInt(signed=False, label="UInt32"), "eku": SimTypePointer(SimTypePointer(SimTypeChar(label="Byte"), offset=0), offset=0)}, name="IKEEXT_CERT_EKUS0", pack=False, align=None), offset=0), "name": SimTypePointer(SimStruct({"nameType": SimTypeInt(signed=False, label="IKEEXT_CERT_CRITERIA_NAME_TYPE"), "certName": SimTypePointer(SimTypeChar(label="Char"), offset=0)}, name="IKEEXT_CERT_NAME0", pack=False, align=None), offset=0), "flags": SimTypeInt(signed=False, label="UInt32")}, name="IKEEXT_CERTIFICATE_CRITERIA0", pack=False, align=None), offset=0)}, name="_Anonymous2_e__Struct", pack=False, align=None), "Anonymous3": SimStruct({"inboundRootStoreArraySize": SimTypeInt(signed=False, label="UInt32"), "inboundTrustedRootStoreCriteria": SimTypePointer(SimStruct({"certData": SimStruct({"size": SimTypeInt(signed=False, label="UInt32"), "data": SimTypePointer(SimTypeChar(label="Byte"), offset=0)}, name="FWP_BYTE_BLOB", pack=False, align=None), "certHash": SimStruct({"size": SimTypeInt(signed=False, label="UInt32"), "data": SimTypePointer(SimTypeChar(label="Byte"), offset=0)}, name="FWP_BYTE_BLOB", pack=False, align=None), "eku": SimTypePointer(SimStruct({"numEku": SimTypeInt(signed=False, label="UInt32"), "eku": SimTypePointer(SimTypePointer(SimTypeChar(label="Byte"), offset=0), offset=0)}, name="IKEEXT_CERT_EKUS0", pack=False, align=None), offset=0), "name": SimTypePointer(SimStruct({"nameType": SimTypeInt(signed=False, label="IKEEXT_CERT_CRITERIA_NAME_TYPE"), "certName": SimTypePointer(SimTypeChar(label="Char"), offset=0)}, name="IKEEXT_CERT_NAME0", pack=False, align=None), offset=0), "flags": SimTypeInt(signed=False, label="UInt32")}, name="IKEEXT_CERTIFICATE_CRITERIA0", pack=False, align=None), offset=0)}, name="_Anonymous3_e__Struct", pack=False, align=None)}, name="<anon>", label="None"), "outboundConfigType": SimTypeInt(signed=False, label="IKEEXT_CERT_CONFIG_TYPE"), "Anonymous2": SimUnion({"Anonymous1": SimStruct({"outboundRootArraySize": SimTypeInt(signed=False, label="UInt32"), "outboundRootCriteria": SimTypePointer(SimStruct({"certData": SimStruct({"size": SimTypeInt(signed=False, label="UInt32"), "data": SimTypePointer(SimTypeChar(label="Byte"), offset=0)}, name="FWP_BYTE_BLOB", pack=False, align=None), "certHash": SimStruct({"size": SimTypeInt(signed=False, label="UInt32"), "data": SimTypePointer(SimTypeChar(label="Byte"), offset=0)}, name="FWP_BYTE_BLOB", pack=False, align=None), "eku": SimTypePointer(SimStruct({"numEku": SimTypeInt(signed=False, label="UInt32"), "eku": SimTypePointer(SimTypePointer(SimTypeChar(label="Byte"), offset=0), offset=0)}, name="IKEEXT_CERT_EKUS0", pack=False, align=None), offset=0), "name": SimTypePointer(SimStruct({"nameType": SimTypeInt(signed=False, label="IKEEXT_CERT_CRITERIA_NAME_TYPE"), "certName": SimTypePointer(SimTypeChar(label="Char"), offset=0)}, name="IKEEXT_CERT_NAME0", pack=False, align=None), offset=0), "flags": SimTypeInt(signed=False, label="UInt32")}, name="IKEEXT_CERTIFICATE_CRITERIA0", pack=False, align=None), offset=0)}, name="_Anonymous1_e__Struct", pack=False, align=None), "Anonymous2": SimStruct({"outboundEnterpriseStoreArraySize": SimTypeInt(signed=False, label="UInt32"), "outboundEnterpriseStoreCriteria": SimTypePointer(SimStruct({"certData": SimStruct({"size": SimTypeInt(signed=False, label="UInt32"), "data": SimTypePointer(SimTypeChar(label="Byte"), offset=0)}, name="FWP_BYTE_BLOB", pack=False, align=None), "certHash": SimStruct({"size": SimTypeInt(signed=False, label="UInt32"), "data": SimTypePointer(SimTypeChar(label="Byte"), offset=0)}, name="FWP_BYTE_BLOB", pack=False, align=None), "eku": SimTypePointer(SimStruct({"numEku": SimTypeInt(signed=False, label="UInt32"), "eku": SimTypePointer(SimTypePointer(SimTypeChar(label="Byte"), offset=0), offset=0)}, name="IKEEXT_CERT_EKUS0", pack=False, align=None), offset=0), "name": SimTypePointer(SimStruct({"nameType": SimTypeInt(signed=False, label="IKEEXT_CERT_CRITERIA_NAME_TYPE"), "certName": SimTypePointer(SimTypeChar(label="Char"), offset=0)}, name="IKEEXT_CERT_NAME0", pack=False, align=None), offset=0), "flags": SimTypeInt(signed=False, label="UInt32")}, name="IKEEXT_CERTIFICATE_CRITERIA0", pack=False, align=None), offset=0)}, name="_Anonymous2_e__Struct", pack=False, align=None), "Anonymous3": SimStruct({"outboundRootStoreArraySize": SimTypeInt(signed=False, label="UInt32"), "outboundTrustedRootStoreCriteria": SimTypePointer(SimStruct({"certData": SimStruct({"size": SimTypeInt(signed=False, label="UInt32"), "data": SimTypePointer(SimTypeChar(label="Byte"), offset=0)}, name="FWP_BYTE_BLOB", pack=False, align=None), "certHash": SimStruct({"size": SimTypeInt(signed=False, label="UInt32"), "data": SimTypePointer(SimTypeChar(label="Byte"), offset=0)}, name="FWP_BYTE_BLOB", pack=False, align=None), "eku": SimTypePointer(SimStruct({"numEku": SimTypeInt(signed=False, label="UInt32"), "eku": SimTypePointer(SimTypePointer(SimTypeChar(label="Byte"), offset=0), offset=0)}, name="IKEEXT_CERT_EKUS0", pack=False, align=None), offset=0), "name": SimTypePointer(SimStruct({"nameType": SimTypeInt(signed=False, label="IKEEXT_CERT_CRITERIA_NAME_TYPE"), "certName": SimTypePointer(SimTypeChar(label="Char"), offset=0)}, name="IKEEXT_CERT_NAME0", pack=False, align=None), offset=0), "flags": SimTypeInt(signed=False, label="UInt32")}, name="IKEEXT_CERTIFICATE_CRITERIA0", pack=False, align=None), offset=0)}, name="_Anonymous3_e__Struct", pack=False, align=None)}, name="<anon>", label="None"), "flags": SimTypeInt(signed=False, label="IKEEXT_CERT_AUTH"), "localCertLocationUrl": SimStruct({"size": SimTypeInt(signed=False, label="UInt32"), "data": SimTypePointer(SimTypeChar(label="Byte"), offset=0)}, name="FWP_BYTE_BLOB", pack=False, align=None)}, name="IKEEXT_CERTIFICATE_AUTHENTICATION2", pack=False, align=None), "cgaAuthentication": SimStruct({"keyContainerName": SimTypePointer(SimTypeChar(label="Char"), offset=0), "cspName": SimTypePointer(SimTypeChar(label="Char"), offset=0), "cspType": SimTypeInt(signed=False, label="UInt32"), "cgaModifier": SimStruct({"byteArray16": SimTypeFixedSizeArray(SimTypeChar(label="Byte"), 16)}, name="FWP_BYTE_ARRAY16", pack=False, align=None), "cgaCollisionCount": SimTypeChar(label="Byte")}, name="IKEEXT_IPV6_CGA_AUTHENTICATION0", pack=False, align=None), "eapAuthentication": SimStruct({"flags": SimTypeInt(signed=False, label="IKEEXT_EAP_AUTHENTICATION_FLAGS")}, name="IKEEXT_EAP_AUTHENTICATION0", pack=False, align=None)}, name="<anon>", label="None")}, name="IKEEXT_AUTHENTICATION_METHOD2", pack=False, align=None), offset=0), "initiatorImpersonationType": SimTypeInt(signed=False, label="IKEEXT_AUTHENTICATION_IMPERSONATION_TYPE"), "numIkeProposals": SimTypeInt(signed=False, label="UInt32"), "ikeProposals": SimTypePointer(SimStruct({"cipherAlgorithm": SimStruct({"algoIdentifier": SimTypeInt(signed=False, label="IKEEXT_CIPHER_TYPE"), "keyLen": SimTypeInt(signed=False, label="UInt32"), "rounds": SimTypeInt(signed=False, label="UInt32")}, name="IKEEXT_CIPHER_ALGORITHM0", pack=False, align=None), "integrityAlgorithm": SimStruct({"algoIdentifier": SimTypeInt(signed=False, label="IKEEXT_INTEGRITY_TYPE")}, name="IKEEXT_INTEGRITY_ALGORITHM0", pack=False, align=None), "maxLifetimeSeconds": SimTypeInt(signed=False, label="UInt32"), "dhGroup": SimTypeInt(signed=False, label="IKEEXT_DH_GROUP"), "quickModeLimit": SimTypeInt(signed=False, label="UInt32")}, name="IKEEXT_PROPOSAL0", pack=False, align=None), offset=0), "flags": SimTypeInt(signed=False, label="IKEEXT_POLICY_FLAG"), "maxDynamicFilters": SimTypeInt(signed=False, label="UInt32"), "retransmitDurationSecs": SimTypeInt(signed=False, label="UInt32")}, name="IKEEXT_POLICY2", pack=False, align=None), offset=0), "dataBuffer": SimTypePointer(SimStruct({"size": SimTypeInt(signed=False, label="UInt32"), "data": SimTypePointer(SimTypeChar(label="Byte"), offset=0)}, name="FWP_BYTE_BLOB", pack=False, align=None), offset=0), "classifyOptions": SimTypePointer(SimStruct({"numOptions": SimTypeInt(signed=False, label="UInt32"), "options": SimTypePointer(SimStruct({"type": SimTypeInt(signed=False, label="FWP_CLASSIFY_OPTION_TYPE"), "value": SimStruct({"type": SimTypeInt(signed=False, label="FWP_DATA_TYPE"), "Anonymous": SimUnion({"uint8": SimTypeChar(label="Byte"), "uint16": SimTypeShort(signed=False, label="UInt16"), "uint32": SimTypeInt(signed=False, label="UInt32"), "uint64": SimTypePointer(SimTypeLongLong(signed=False, label="UInt64"), offset=0), "int8": SimTypeChar(label="SByte"), "int16": SimTypeShort(signed=True, label="Int16"), "int32": SimTypeInt(signed=True, label="Int32"), "int64": SimTypePointer(SimTypeLongLong(signed=True, label="Int64"), offset=0), "float32": SimTypeFloat(size=32), "double64": SimTypePointer(SimTypeFloat(size=64), offset=0), "byteArray16": SimTypePointer(SimStruct({"byteArray16": SimTypeFixedSizeArray(SimTypeChar(label="Byte"), 16)}, name="FWP_BYTE_ARRAY16", pack=False, align=None), offset=0), "byteBlob": SimTypePointer(SimStruct({"size": SimTypeInt(signed=False, label="UInt32"), "data": SimTypePointer(SimTypeChar(label="Byte"), offset=0)}, name="FWP_BYTE_BLOB", pack=False, align=None), offset=0), "sid": SimTypePointer(SimTypeBottom(label="SID"), offset=0), "sd": SimTypePointer(SimStruct({"size": SimTypeInt(signed=False, label="UInt32"), "data": SimTypePointer(SimTypeChar(label="Byte"), offset=0)}, name="FWP_BYTE_BLOB", pack=False, align=None), offset=0), "tokenInformation": SimTypePointer(SimStruct({"sidCount": SimTypeInt(signed=False, label="UInt32"), "sids": SimTypePointer(SimTypeBottom(label="SID_AND_ATTRIBUTES"), offset=0), "restrictedSidCount": SimTypeInt(signed=False, label="UInt32"), "restrictedSids": SimTypePointer(SimTypeBottom(label="SID_AND_ATTRIBUTES"), offset=0)}, name="FWP_TOKEN_INFORMATION", pack=False, align=None), offset=0), "tokenAccessInformation": SimTypePointer(SimStruct({"size": SimTypeInt(signed=False, label="UInt32"), "data": SimTypePointer(SimTypeChar(label="Byte"), offset=0)}, name="FWP_BYTE_BLOB", pack=False, align=None), offset=0), "unicodeString": SimTypePointer(SimTypeChar(label="Char"), offset=0), "byteArray6": SimTypePointer(SimStruct({"byteArray6": SimTypeFixedSizeArray(SimTypeChar(label="Byte"), 6)}, name="FWP_BYTE_ARRAY6", pack=False, align=None), offset=0), "bitmapArray64": SimTypePointer(SimStruct({"bitmapArray64": SimTypeFixedSizeArray(SimTypeChar(label="Byte"), 8)}, name="FWP_BITMAP_ARRAY64_", pack=False, align=None), offset=0)}, name="<anon>", label="None")}, name="FWP_VALUE0", pack=False, align=None)}, name="FWPM_CLASSIFY_OPTION0", pack=False, align=None), offset=0)}, name="FWPM_CLASSIFY_OPTIONS0", pack=False, align=None), offset=0), "ikeV2QmTunnelPolicy": SimTypePointer(SimStruct({"flags": SimTypeInt(signed=False, label="UInt32"), "numIpsecProposals": SimTypeInt(signed=False, label="UInt32"), "ipsecProposals": SimTypePointer(SimStruct({"lifetime": SimStruct({"lifetimeSeconds": SimTypeInt(signed=False, label="UInt32"), "lifetimeKilobytes": SimTypeInt(signed=False, label="UInt32"), "lifetimePackets": SimTypeInt(signed=False, label="UInt32")}, name="IPSEC_SA_LIFETIME0", pack=False, align=None), "numSaTransforms": SimTypeInt(signed=False, label="UInt32"), "saTransforms": SimTypePointer(SimStruct({"ipsecTransformType": SimTypeInt(signed=False, label="IPSEC_TRANSFORM_TYPE"), "Anonymous": SimUnion({"ahTransform": SimTypePointer(SimStruct({"authTransformId": SimStruct({"authType": SimTypeInt(signed=False, label="IPSEC_AUTH_TYPE"), "authConfig": SimTypeChar(label="Byte")}, name="IPSEC_AUTH_TRANSFORM_ID0", pack=False, align=None), "cryptoModuleId": SimTypePointer(SimTypeBottom(label="Guid"), offset=0)}, name="IPSEC_AUTH_TRANSFORM0", pack=False, align=None), offset=0), "espAuthTransform": SimTypePointer(SimStruct({"authTransformId": SimStruct({"authType": SimTypeInt(signed=False, label="IPSEC_AUTH_TYPE"), "authConfig": SimTypeChar(label="Byte")}, name="IPSEC_AUTH_TRANSFORM_ID0", pack=False, align=None), "cryptoModuleId": SimTypePointer(SimTypeBottom(label="Guid"), offset=0)}, name="IPSEC_AUTH_TRANSFORM0", pack=False, align=None), offset=0), "espCipherTransform": SimTypePointer(SimStruct({"cipherTransformId": SimStruct({"cipherType": SimTypeInt(signed=False, label="IPSEC_CIPHER_TYPE"), "cipherConfig": SimTypeChar(label="Byte")}, name="IPSEC_CIPHER_TRANSFORM_ID0", pack=False, align=None), "cryptoModuleId": SimTypePointer(SimTypeBottom(label="Guid"), offset=0)}, name="IPSEC_CIPHER_TRANSFORM0", pack=False, align=None), offset=0), "espAuthAndCipherTransform": SimTypePointer(SimStruct({"authTransform": SimStruct({"authTransformId": SimStruct({"authType": SimTypeInt(signed=False, label="IPSEC_AUTH_TYPE"), "authConfig": SimTypeChar(label="Byte")}, name="IPSEC_AUTH_TRANSFORM_ID0", pack=False, align=None), "cryptoModuleId": SimTypePointer(SimTypeBottom(label="Guid"), offset=0)}, name="IPSEC_AUTH_TRANSFORM0", pack=False, align=None), "cipherTransform": SimStruct({"cipherTransformId": SimStruct({"cipherType": SimTypeInt(signed=False, label="IPSEC_CIPHER_TYPE"), "cipherConfig": SimTypeChar(label="Byte")}, name="IPSEC_CIPHER_TRANSFORM_ID0", pack=False, align=None), "cryptoModuleId": SimTypePointer(SimTypeBottom(label="Guid"), offset=0)}, name="IPSEC_CIPHER_TRANSFORM0", pack=False, align=None)}, name="IPSEC_AUTH_AND_CIPHER_TRANSFORM0", pack=False, align=None), offset=0), "espAuthFwTransform": SimTypePointer(SimStruct({"authTransformId": SimStruct({"authType": SimTypeInt(signed=False, label="IPSEC_AUTH_TYPE"), "authConfig": SimTypeChar(label="Byte")}, name="IPSEC_AUTH_TRANSFORM_ID0", pack=False, align=None), "cryptoModuleId": SimTypePointer(SimTypeBottom(label="Guid"), offset=0)}, name="IPSEC_AUTH_TRANSFORM0", pack=False, align=None), offset=0)}, name="<anon>", label="None")}, name="IPSEC_SA_TRANSFORM0", pack=False, align=None), offset=0), "pfsGroup": SimTypeInt(signed=False, label="IPSEC_PFS_GROUP")}, name="IPSEC_PROPOSAL0", pack=False, align=None), offset=0), "tunnelEndpoints": SimStruct({"ipVersion": SimTypeInt(signed=False, label="FWP_IP_VERSION"), "Anonymous1": SimUnion({"localV4Address": SimTypeInt(signed=False, label="UInt32"), "localV6Address": SimTypeFixedSizeArray(SimTypeChar(label="Byte"), 16)}, name="<anon>", label="None"), "Anonymous2": SimUnion({"remoteV4Address": SimTypeInt(signed=False, label="UInt32"), "remoteV6Address": SimTypeFixedSizeArray(SimTypeChar(label="Byte"), 16)}, name="<anon>", label="None"), "localIfLuid": SimTypeLongLong(signed=False, label="UInt64"), "remoteFqdn": SimTypePointer(SimTypeChar(label="Char"), offset=0), "numAddresses": SimTypeInt(signed=False, label="UInt32"), "remoteAddresses": SimTypePointer(SimStruct({"ipVersion": SimTypeInt(signed=False, label="FWP_IP_VERSION"), "Anonymous": SimUnion({"v4Address": SimTypeInt(signed=False, label="UInt32"), "v6Address": SimTypeFixedSizeArray(SimTypeChar(label="Byte"), 16)}, name="<anon>", label="None")}, name="IPSEC_TUNNEL_ENDPOINT0", pack=False, align=None), offset=0)}, name="IPSEC_TUNNEL_ENDPOINTS2", pack=False, align=None), "saIdleTimeout": SimStruct({"idleTimeoutSeconds": SimTypeInt(signed=False, label="UInt32"), "idleTimeoutSecondsFailOver": SimTypeInt(signed=False, label="UInt32")}, name="IPSEC_SA_IDLE_TIMEOUT0", pack=False, align=None), "emPolicy": SimTypePointer(SimStruct({"numAuthenticationMethods": SimTypeInt(signed=False, label="UInt32"), "authenticationMethods": SimTypePointer(SimStruct({"authenticationMethodType": SimTypeInt(signed=False, label="IKEEXT_AUTHENTICATION_METHOD_TYPE"), "Anonymous": SimUnion({"presharedKeyAuthentication": SimStruct({"presharedKey": SimStruct({"size": SimTypeInt(signed=False, label="UInt32"), "data": SimTypePointer(SimTypeChar(label="Byte"), offset=0)}, name="FWP_BYTE_BLOB", pack=False, align=None), "flags": SimTypeInt(signed=False, label="IKEEXT_PRESHARED_KEY_AUTHENTICATION_FLAGS")}, name="IKEEXT_PRESHARED_KEY_AUTHENTICATION1", pack=False, align=None), "certificateAuthentication": SimStruct({"inboundConfigType": SimTypeInt(signed=False, label="IKEEXT_CERT_CONFIG_TYPE"), "Anonymous1": SimUnion({"Anonymous1": SimStruct({"inboundRootArraySize": SimTypeInt(signed=False, label="UInt32"), "inboundRootCriteria": SimTypePointer(SimStruct({"certData": SimStruct({"size": SimTypeInt(signed=False, label="UInt32"), "data": SimTypePointer(SimTypeChar(label="Byte"), offset=0)}, name="FWP_BYTE_BLOB", pack=False, align=None), "certHash": SimStruct({"size": SimTypeInt(signed=False, label="UInt32"), "data": SimTypePointer(SimTypeChar(label="Byte"), offset=0)}, name="FWP_BYTE_BLOB", pack=False, align=None), "eku": SimTypePointer(SimStruct({"numEku": SimTypeInt(signed=False, label="UInt32"), "eku": SimTypePointer(SimTypePointer(SimTypeChar(label="Byte"), offset=0), offset=0)}, name="IKEEXT_CERT_EKUS0", pack=False, align=None), offset=0), "name": SimTypePointer(SimStruct({"nameType": SimTypeInt(signed=False, label="IKEEXT_CERT_CRITERIA_NAME_TYPE"), "certName": SimTypePointer(SimTypeChar(label="Char"), offset=0)}, name="IKEEXT_CERT_NAME0", pack=False, align=None), offset=0), "flags": SimTypeInt(signed=False, label="UInt32")}, name="IKEEXT_CERTIFICATE_CRITERIA0", pack=False, align=None), offset=0)}, name="_Anonymous1_e__Struct", pack=False, align=None), "Anonymous2": SimStruct({"inboundEnterpriseStoreArraySize": SimTypeInt(signed=False, label="UInt32"), "inboundEnterpriseStoreCriteria": SimTypePointer(SimStruct({"certData": SimStruct({"size": SimTypeInt(signed=False, label="UInt32"), "data": SimTypePointer(SimTypeChar(label="Byte"), offset=0)}, name="FWP_BYTE_BLOB", pack=False, align=None), "certHash": SimStruct({"size": SimTypeInt(signed=False, label="UInt32"), "data": SimTypePointer(SimTypeChar(label="Byte"), offset=0)}, name="FWP_BYTE_BLOB", pack=False, align=None), "eku": SimTypePointer(SimStruct({"numEku": SimTypeInt(signed=False, label="UInt32"), "eku": SimTypePointer(SimTypePointer(SimTypeChar(label="Byte"), offset=0), offset=0)}, name="IKEEXT_CERT_EKUS0", pack=False, align=None), offset=0), "name": SimTypePointer(SimStruct({"nameType": SimTypeInt(signed=False, label="IKEEXT_CERT_CRITERIA_NAME_TYPE"), "certName": SimTypePointer(SimTypeChar(label="Char"), offset=0)}, name="IKEEXT_CERT_NAME0", pack=False, align=None), offset=0), "flags": SimTypeInt(signed=False, label="UInt32")}, name="IKEEXT_CERTIFICATE_CRITERIA0", pack=False, align=None), offset=0)}, name="_Anonymous2_e__Struct", pack=False, align=None), "Anonymous3": SimStruct({"inboundRootStoreArraySize": SimTypeInt(signed=False, label="UInt32"), "inboundTrustedRootStoreCriteria": SimTypePointer(SimStruct({"certData": SimStruct({"size": SimTypeInt(signed=False, label="UInt32"), "data": SimTypePointer(SimTypeChar(label="Byte"), offset=0)}, name="FWP_BYTE_BLOB", pack=False, align=None), "certHash": SimStruct({"size": SimTypeInt(signed=False, label="UInt32"), "data": SimTypePointer(SimTypeChar(label="Byte"), offset=0)}, name="FWP_BYTE_BLOB", pack=False, align=None), "eku": SimTypePointer(SimStruct({"numEku": SimTypeInt(signed=False, label="UInt32"), "eku": SimTypePointer(SimTypePointer(SimTypeChar(label="Byte"), offset=0), offset=0)}, name="IKEEXT_CERT_EKUS0", pack=False, align=None), offset=0), "name": SimTypePointer(SimStruct({"nameType": SimTypeInt(signed=False, label="IKEEXT_CERT_CRITERIA_NAME_TYPE"), "certName": SimTypePointer(SimTypeChar(label="Char"), offset=0)}, name="IKEEXT_CERT_NAME0", pack=False, align=None), offset=0), "flags": SimTypeInt(signed=False, label="UInt32")}, name="IKEEXT_CERTIFICATE_CRITERIA0", pack=False, align=None), offset=0)}, name="_Anonymous3_e__Struct", pack=False, align=None)}, name="<anon>", label="None"), "outboundConfigType": SimTypeInt(signed=False, label="IKEEXT_CERT_CONFIG_TYPE"), "Anonymous2": SimUnion({"Anonymous1": SimStruct({"outboundRootArraySize": SimTypeInt(signed=False, label="UInt32"), "outboundRootCriteria": SimTypePointer(SimStruct({"certData": SimStruct({"size": SimTypeInt(signed=False, label="UInt32"), "data": SimTypePointer(SimTypeChar(label="Byte"), offset=0)}, name="FWP_BYTE_BLOB", pack=False, align=None), "certHash": SimStruct({"size": SimTypeInt(signed=False, label="UInt32"), "data": SimTypePointer(SimTypeChar(label="Byte"), offset=0)}, name="FWP_BYTE_BLOB", pack=False, align=None), "eku": SimTypePointer(SimStruct({"numEku": SimTypeInt(signed=False, label="UInt32"), "eku": SimTypePointer(SimTypePointer(SimTypeChar(label="Byte"), offset=0), offset=0)}, name="IKEEXT_CERT_EKUS0", pack=False, align=None), offset=0), "name": SimTypePointer(SimStruct({"nameType": SimTypeInt(signed=False, label="IKEEXT_CERT_CRITERIA_NAME_TYPE"), "certName": SimTypePointer(SimTypeChar(label="Char"), offset=0)}, name="IKEEXT_CERT_NAME0", pack=False, align=None), offset=0), "flags": SimTypeInt(signed=False, label="UInt32")}, name="IKEEXT_CERTIFICATE_CRITERIA0", pack=False, align=None), offset=0)}, name="_Anonymous1_e__Struct", pack=False, align=None), "Anonymous2": SimStruct({"outboundEnterpriseStoreArraySize": SimTypeInt(signed=False, label="UInt32"), "outboundEnterpriseStoreCriteria": SimTypePointer(SimStruct({"certData": SimStruct({"size": SimTypeInt(signed=False, label="UInt32"), "data": SimTypePointer(SimTypeChar(label="Byte"), offset=0)}, name="FWP_BYTE_BLOB", pack=False, align=None), "certHash": SimStruct({"size": SimTypeInt(signed=False, label="UInt32"), "data": SimTypePointer(SimTypeChar(label="Byte"), offset=0)}, name="FWP_BYTE_BLOB", pack=False, align=None), "eku": SimTypePointer(SimStruct({"numEku": SimTypeInt(signed=False, label="UInt32"), "eku": SimTypePointer(SimTypePointer(SimTypeChar(label="Byte"), offset=0), offset=0)}, name="IKEEXT_CERT_EKUS0", pack=False, align=None), offset=0), "name": SimTypePointer(SimStruct({"nameType": SimTypeInt(signed=False, label="IKEEXT_CERT_CRITERIA_NAME_TYPE"), "certName": SimTypePointer(SimTypeChar(label="Char"), offset=0)}, name="IKEEXT_CERT_NAME0", pack=False, align=None), offset=0), "flags": SimTypeInt(signed=False, label="UInt32")}, name="IKEEXT_CERTIFICATE_CRITERIA0", pack=False, align=None), offset=0)}, name="_Anonymous2_e__Struct", pack=False, align=None), "Anonymous3": SimStruct({"outboundRootStoreArraySize": SimTypeInt(signed=False, label="UInt32"), "outboundTrustedRootStoreCriteria": SimTypePointer(SimStruct({"certData": SimStruct({"size": SimTypeInt(signed=False, label="UInt32"), "data": SimTypePointer(SimTypeChar(label="Byte"), offset=0)}, name="FWP_BYTE_BLOB", pack=False, align=None), "certHash": SimStruct({"size": SimTypeInt(signed=False, label="UInt32"), "data": SimTypePointer(SimTypeChar(label="Byte"), offset=0)}, name="FWP_BYTE_BLOB", pack=False, align=None), "eku": SimTypePointer(SimStruct({"numEku": SimTypeInt(signed=False, label="UInt32"), "eku": SimTypePointer(SimTypePointer(SimTypeChar(label="Byte"), offset=0), offset=0)}, name="IKEEXT_CERT_EKUS0", pack=False, align=None), offset=0), "name": SimTypePointer(SimStruct({"nameType": SimTypeInt(signed=False, label="IKEEXT_CERT_CRITERIA_NAME_TYPE"), "certName": SimTypePointer(SimTypeChar(label="Char"), offset=0)}, name="IKEEXT_CERT_NAME0", pack=False, align=None), offset=0), "flags": SimTypeInt(signed=False, label="UInt32")}, name="IKEEXT_CERTIFICATE_CRITERIA0", pack=False, align=None), offset=0)}, name="_Anonymous3_e__Struct", pack=False, align=None)}, name="<anon>", label="None"), "flags": SimTypeInt(signed=False, label="IKEEXT_CERT_AUTH"), "localCertLocationUrl": SimStruct({"size": SimTypeInt(signed=False, label="UInt32"), "data": SimTypePointer(SimTypeChar(label="Byte"), offset=0)}, name="FWP_BYTE_BLOB", pack=False, align=None)}, name="IKEEXT_CERTIFICATE_AUTHENTICATION2", pack=False, align=None), "kerberosAuthentication": SimStruct({"flags": SimTypeInt(signed=False, label="IKEEXT_KERBEROS_AUTHENTICATION_FLAGS"), "proxyServer": SimTypePointer(SimTypeChar(label="Char"), offset=0)}, name="IKEEXT_KERBEROS_AUTHENTICATION1", pack=False, align=None), "reservedAuthentication": SimStruct({"flags": SimTypeInt(signed=False, label="IKEEXT_RESERVED_AUTHENTICATION_FLAGS")}, name="IKEEXT_RESERVED_AUTHENTICATION0", pack=False, align=None), "ntlmV2Authentication": SimStruct({"flags": SimTypeInt(signed=False, label="UInt32")}, name="IKEEXT_NTLM_V2_AUTHENTICATION0", pack=False, align=None), "sslAuthentication": SimStruct({"inboundConfigType": SimTypeInt(signed=False, label="IKEEXT_CERT_CONFIG_TYPE"), "Anonymous1": SimUnion({"Anonymous1": SimStruct({"inboundRootArraySize": SimTypeInt(signed=False, label="UInt32"), "inboundRootCriteria": SimTypePointer(SimStruct({"certData": SimStruct({"size": SimTypeInt(signed=False, label="UInt32"), "data": SimTypePointer(SimTypeChar(label="Byte"), offset=0)}, name="FWP_BYTE_BLOB", pack=False, align=None), "certHash": SimStruct({"size": SimTypeInt(signed=False, label="UInt32"), "data": SimTypePointer(SimTypeChar(label="Byte"), offset=0)}, name="FWP_BYTE_BLOB", pack=False, align=None), "eku": SimTypePointer(SimStruct({"numEku": SimTypeInt(signed=False, label="UInt32"), "eku": SimTypePointer(SimTypePointer(SimTypeChar(label="Byte"), offset=0), offset=0)}, name="IKEEXT_CERT_EKUS0", pack=False, align=None), offset=0), "name": SimTypePointer(SimStruct({"nameType": SimTypeInt(signed=False, label="IKEEXT_CERT_CRITERIA_NAME_TYPE"), "certName": SimTypePointer(SimTypeChar(label="Char"), offset=0)}, name="IKEEXT_CERT_NAME0", pack=False, align=None), offset=0), "flags": SimTypeInt(signed=False, label="UInt32")}, name="IKEEXT_CERTIFICATE_CRITERIA0", pack=False, align=None), offset=0)}, name="_Anonymous1_e__Struct", pack=False, align=None), "Anonymous2": SimStruct({"inboundEnterpriseStoreArraySize": SimTypeInt(signed=False, label="UInt32"), "inboundEnterpriseStoreCriteria": SimTypePointer(SimStruct({"certData": SimStruct({"size": SimTypeInt(signed=False, label="UInt32"), "data": SimTypePointer(SimTypeChar(label="Byte"), offset=0)}, name="FWP_BYTE_BLOB", pack=False, align=None), "certHash": SimStruct({"size": SimTypeInt(signed=False, label="UInt32"), "data": SimTypePointer(SimTypeChar(label="Byte"), offset=0)}, name="FWP_BYTE_BLOB", pack=False, align=None), "eku": SimTypePointer(SimStruct({"numEku": SimTypeInt(signed=False, label="UInt32"), "eku": SimTypePointer(SimTypePointer(SimTypeChar(label="Byte"), offset=0), offset=0)}, name="IKEEXT_CERT_EKUS0", pack=False, align=None), offset=0), "name": SimTypePointer(SimStruct({"nameType": SimTypeInt(signed=False, label="IKEEXT_CERT_CRITERIA_NAME_TYPE"), "certName": SimTypePointer(SimTypeChar(label="Char"), offset=0)}, name="IKEEXT_CERT_NAME0", pack=False, align=None), offset=0), "flags": SimTypeInt(signed=False, label="UInt32")}, name="IKEEXT_CERTIFICATE_CRITERIA0", pack=False, align=None), offset=0)}, name="_Anonymous2_e__Struct", pack=False, align=None), "Anonymous3": SimStruct({"inboundRootStoreArraySize": SimTypeInt(signed=False, label="UInt32"), "inboundTrustedRootStoreCriteria": SimTypePointer(SimStruct({"certData": SimStruct({"size": SimTypeInt(signed=False, label="UInt32"), "data": SimTypePointer(SimTypeChar(label="Byte"), offset=0)}, name="FWP_BYTE_BLOB", pack=False, align=None), "certHash": SimStruct({"size": SimTypeInt(signed=False, label="UInt32"), "data": SimTypePointer(SimTypeChar(label="Byte"), offset=0)}, name="FWP_BYTE_BLOB", pack=False, align=None), "eku": SimTypePointer(SimStruct({"numEku": SimTypeInt(signed=False, label="UInt32"), "eku": SimTypePointer(SimTypePointer(SimTypeChar(label="Byte"), offset=0), offset=0)}, name="IKEEXT_CERT_EKUS0", pack=False, align=None), offset=0), "name": SimTypePointer(SimStruct({"nameType": SimTypeInt(signed=False, label="IKEEXT_CERT_CRITERIA_NAME_TYPE"), "certName": SimTypePointer(SimTypeChar(label="Char"), offset=0)}, name="IKEEXT_CERT_NAME0", pack=False, align=None), offset=0), "flags": SimTypeInt(signed=False, label="UInt32")}, name="IKEEXT_CERTIFICATE_CRITERIA0", pack=False, align=None), offset=0)}, name="_Anonymous3_e__Struct", pack=False, align=None)}, name="<anon>", label="None"), "outboundConfigType": SimTypeInt(signed=False, label="IKEEXT_CERT_CONFIG_TYPE"), "Anonymous2": SimUnion({"Anonymous1": SimStruct({"outboundRootArraySize": SimTypeInt(signed=False, label="UInt32"), "outboundRootCriteria": SimTypePointer(SimStruct({"certData": SimStruct({"size": SimTypeInt(signed=False, label="UInt32"), "data": SimTypePointer(SimTypeChar(label="Byte"), offset=0)}, name="FWP_BYTE_BLOB", pack=False, align=None), "certHash": SimStruct({"size": SimTypeInt(signed=False, label="UInt32"), "data": SimTypePointer(SimTypeChar(label="Byte"), offset=0)}, name="FWP_BYTE_BLOB", pack=False, align=None), "eku": SimTypePointer(SimStruct({"numEku": SimTypeInt(signed=False, label="UInt32"), "eku": SimTypePointer(SimTypePointer(SimTypeChar(label="Byte"), offset=0), offset=0)}, name="IKEEXT_CERT_EKUS0", pack=False, align=None), offset=0), "name": SimTypePointer(SimStruct({"nameType": SimTypeInt(signed=False, label="IKEEXT_CERT_CRITERIA_NAME_TYPE"), "certName": SimTypePointer(SimTypeChar(label="Char"), offset=0)}, name="IKEEXT_CERT_NAME0", pack=False, align=None), offset=0), "flags": SimTypeInt(signed=False, label="UInt32")}, name="IKEEXT_CERTIFICATE_CRITERIA0", pack=False, align=None), offset=0)}, name="_Anonymous1_e__Struct", pack=False, align=None), "Anonymous2": SimStruct({"outboundEnterpriseStoreArraySize": SimTypeInt(signed=False, label="UInt32"), "outboundEnterpriseStoreCriteria": SimTypePointer(SimStruct({"certData": SimStruct({"size": SimTypeInt(signed=False, label="UInt32"), "data": SimTypePointer(SimTypeChar(label="Byte"), offset=0)}, name="FWP_BYTE_BLOB", pack=False, align=None), "certHash": SimStruct({"size": SimTypeInt(signed=False, label="UInt32"), "data": SimTypePointer(SimTypeChar(label="Byte"), offset=0)}, name="FWP_BYTE_BLOB", pack=False, align=None), "eku": SimTypePointer(SimStruct({"numEku": SimTypeInt(signed=False, label="UInt32"), "eku": SimTypePointer(SimTypePointer(SimTypeChar(label="Byte"), offset=0), offset=0)}, name="IKEEXT_CERT_EKUS0", pack=False, align=None), offset=0), "name": SimTypePointer(SimStruct({"nameType": SimTypeInt(signed=False, label="IKEEXT_CERT_CRITERIA_NAME_TYPE"), "certName": SimTypePointer(SimTypeChar(label="Char"), offset=0)}, name="IKEEXT_CERT_NAME0", pack=False, align=None), offset=0), "flags": SimTypeInt(signed=False, label="UInt32")}, name="IKEEXT_CERTIFICATE_CRITERIA0", pack=False, align=None), offset=0)}, name="_Anonymous2_e__Struct", pack=False, align=None), "Anonymous3": SimStruct({"outboundRootStoreArraySize": SimTypeInt(signed=False, label="UInt32"), "outboundTrustedRootStoreCriteria": SimTypePointer(SimStruct({"certData": SimStruct({"size": SimTypeInt(signed=False, label="UInt32"), "data": SimTypePointer(SimTypeChar(label="Byte"), offset=0)}, name="FWP_BYTE_BLOB", pack=False, align=None), "certHash": SimStruct({"size": SimTypeInt(signed=False, label="UInt32"), "data": SimTypePointer(SimTypeChar(label="Byte"), offset=0)}, name="FWP_BYTE_BLOB", pack=False, align=None), "eku": SimTypePointer(SimStruct({"numEku": SimTypeInt(signed=False, label="UInt32"), "eku": SimTypePointer(SimTypePointer(SimTypeChar(label="Byte"), offset=0), offset=0)}, name="IKEEXT_CERT_EKUS0", pack=False, align=None), offset=0), "name": SimTypePointer(SimStruct({"nameType": SimTypeInt(signed=False, label="IKEEXT_CERT_CRITERIA_NAME_TYPE"), "certName": SimTypePointer(SimTypeChar(label="Char"), offset=0)}, name="IKEEXT_CERT_NAME0", pack=False, align=None), offset=0), "flags": SimTypeInt(signed=False, label="UInt32")}, name="IKEEXT_CERTIFICATE_CRITERIA0", pack=False, align=None), offset=0)}, name="_Anonymous3_e__Struct", pack=False, align=None)}, name="<anon>", label="None"), "flags": SimTypeInt(signed=False, label="IKEEXT_CERT_AUTH"), "localCertLocationUrl": SimStruct({"size": SimTypeInt(signed=False, label="UInt32"), "data": SimTypePointer(SimTypeChar(label="Byte"), offset=0)}, name="FWP_BYTE_BLOB", pack=False, align=None)}, name="IKEEXT_CERTIFICATE_AUTHENTICATION2", pack=False, align=None), "cgaAuthentication": SimStruct({"keyContainerName": SimTypePointer(SimTypeChar(label="Char"), offset=0), "cspName": SimTypePointer(SimTypeChar(label="Char"), offset=0), "cspType": SimTypeInt(signed=False, label="UInt32"), "cgaModifier": SimStruct({"byteArray16": SimTypeFixedSizeArray(SimTypeChar(label="Byte"), 16)}, name="FWP_BYTE_ARRAY16", pack=False, align=None), "cgaCollisionCount": SimTypeChar(label="Byte")}, name="IKEEXT_IPV6_CGA_AUTHENTICATION0", pack=False, align=None), "eapAuthentication": SimStruct({"flags": SimTypeInt(signed=False, label="IKEEXT_EAP_AUTHENTICATION_FLAGS")}, name="IKEEXT_EAP_AUTHENTICATION0", pack=False, align=None)}, name="<anon>", label="None")}, name="IKEEXT_AUTHENTICATION_METHOD2", pack=False, align=None), offset=0), "initiatorImpersonationType": SimTypeInt(signed=False, label="IKEEXT_AUTHENTICATION_IMPERSONATION_TYPE")}, name="IKEEXT_EM_POLICY2", pack=False, align=None), offset=0), "fwdPathSaLifetime": SimTypeInt(signed=False, label="UInt32"), "compartmentId": SimTypeInt(signed=False, label="UInt32"), "numTrafficSelectorPolicy": SimTypeInt(signed=False, label="UInt32"), "trafficSelectorPolicies": SimTypePointer(SimStruct({"flags": SimTypeInt(signed=False, label="UInt32"), "numLocalTrafficSelectors": SimTypeInt(signed=False, label="UInt32"), "localTrafficSelectors": SimTypePointer(SimStruct({"protocolId": SimTypeChar(label="Byte"), "portStart": SimTypeShort(signed=False, label="UInt16"), "portEnd": SimTypeShort(signed=False, label="UInt16"), "ipVersion": SimTypeInt(signed=False, label="FWP_IP_VERSION"), "Anonymous1": SimUnion({"startV4Address": SimTypeInt(signed=False, label="UInt32"), "startV6Address": SimTypeFixedSizeArray(SimTypeChar(label="Byte"), 16)}, name="<anon>", label="None"), "Anonymous2": SimUnion({"endV4Address": SimTypeInt(signed=False, label="UInt32"), "endV6Address": SimTypeFixedSizeArray(SimTypeChar(label="Byte"), 16)}, name="<anon>", label="None")}, name="IPSEC_TRAFFIC_SELECTOR0_", pack=False, align=None), offset=0), "numRemoteTrafficSelectors": SimTypeInt(signed=False, label="UInt32"), "remoteTrafficSelectors": SimTypePointer(SimStruct({"protocolId": SimTypeChar(label="Byte"), "portStart": SimTypeShort(signed=False, label="UInt16"), "portEnd": SimTypeShort(signed=False, label="UInt16"), "ipVersion": SimTypeInt(signed=False, label="FWP_IP_VERSION"), "Anonymous1": SimUnion({"startV4Address": SimTypeInt(signed=False, label="UInt32"), "startV6Address": SimTypeFixedSizeArray(SimTypeChar(label="Byte"), 16)}, name="<anon>", label="None"), "Anonymous2": SimUnion({"endV4Address": SimTypeInt(signed=False, label="UInt32"), "endV6Address": SimTypeFixedSizeArray(SimTypeChar(label="Byte"), 16)}, name="<anon>", label="None")}, name="IPSEC_TRAFFIC_SELECTOR0_", pack=False, align=None), offset=0)}, name="IPSEC_TRAFFIC_SELECTOR_POLICY0_", pack=False, align=None), offset=0)}, name="IPSEC_TUNNEL_POLICY3_", pack=False, align=None), offset=0), "ikeV2QmTransportPolicy": SimTypePointer(SimStruct({"numIpsecProposals": SimTypeInt(signed=False, label="UInt32"), "ipsecProposals": SimTypePointer(SimStruct({"lifetime": SimStruct({"lifetimeSeconds": SimTypeInt(signed=False, label="UInt32"), "lifetimeKilobytes": SimTypeInt(signed=False, label="UInt32"), "lifetimePackets": SimTypeInt(signed=False, label="UInt32")}, name="IPSEC_SA_LIFETIME0", pack=False, align=None), "numSaTransforms": SimTypeInt(signed=False, label="UInt32"), "saTransforms": SimTypePointer(SimStruct({"ipsecTransformType": SimTypeInt(signed=False, label="IPSEC_TRANSFORM_TYPE"), "Anonymous": SimUnion({"ahTransform": SimTypePointer(SimStruct({"authTransformId": SimStruct({"authType": SimTypeInt(signed=False, label="IPSEC_AUTH_TYPE"), "authConfig": SimTypeChar(label="Byte")}, name="IPSEC_AUTH_TRANSFORM_ID0", pack=False, align=None), "cryptoModuleId": SimTypePointer(SimTypeBottom(label="Guid"), offset=0)}, name="IPSEC_AUTH_TRANSFORM0", pack=False, align=None), offset=0), "espAuthTransform": SimTypePointer(SimStruct({"authTransformId": SimStruct({"authType": SimTypeInt(signed=False, label="IPSEC_AUTH_TYPE"), "authConfig": SimTypeChar(label="Byte")}, name="IPSEC_AUTH_TRANSFORM_ID0", pack=False, align=None), "cryptoModuleId": SimTypePointer(SimTypeBottom(label="Guid"), offset=0)}, name="IPSEC_AUTH_TRANSFORM0", pack=False, align=None), offset=0), "espCipherTransform": SimTypePointer(SimStruct({"cipherTransformId": SimStruct({"cipherType": SimTypeInt(signed=False, label="IPSEC_CIPHER_TYPE"), "cipherConfig": SimTypeChar(label="Byte")}, name="IPSEC_CIPHER_TRANSFORM_ID0", pack=False, align=None), "cryptoModuleId": SimTypePointer(SimTypeBottom(label="Guid"), offset=0)}, name="IPSEC_CIPHER_TRANSFORM0", pack=False, align=None), offset=0), "espAuthAndCipherTransform": SimTypePointer(SimStruct({"authTransform": SimStruct({"authTransformId": SimStruct({"authType": SimTypeInt(signed=False, label="IPSEC_AUTH_TYPE"), "authConfig": SimTypeChar(label="Byte")}, name="IPSEC_AUTH_TRANSFORM_ID0", pack=False, align=None), "cryptoModuleId": SimTypePointer(SimTypeBottom(label="Guid"), offset=0)}, name="IPSEC_AUTH_TRANSFORM0", pack=False, align=None), "cipherTransform": SimStruct({"cipherTransformId": SimStruct({"cipherType": SimTypeInt(signed=False, label="IPSEC_CIPHER_TYPE"), "cipherConfig": SimTypeChar(label="Byte")}, name="IPSEC_CIPHER_TRANSFORM_ID0", pack=False, align=None), "cryptoModuleId": SimTypePointer(SimTypeBottom(label="Guid"), offset=0)}, name="IPSEC_CIPHER_TRANSFORM0", pack=False, align=None)}, name="IPSEC_AUTH_AND_CIPHER_TRANSFORM0", pack=False, align=None), offset=0), "espAuthFwTransform": SimTypePointer(SimStruct({"authTransformId": SimStruct({"authType": SimTypeInt(signed=False, label="IPSEC_AUTH_TYPE"), "authConfig": SimTypeChar(label="Byte")}, name="IPSEC_AUTH_TRANSFORM_ID0", pack=False, align=None), "cryptoModuleId": SimTypePointer(SimTypeBottom(label="Guid"), offset=0)}, name="IPSEC_AUTH_TRANSFORM0", pack=False, align=None), offset=0)}, name="<anon>", label="None")}, name="IPSEC_SA_TRANSFORM0", pack=False, align=None), offset=0), "pfsGroup": SimTypeInt(signed=False, label="IPSEC_PFS_GROUP")}, name="IPSEC_PROPOSAL0", pack=False, align=None), offset=0), "flags": SimTypeInt(signed=False, label="IPSEC_POLICY_FLAG"), "ndAllowClearTimeoutSeconds": SimTypeInt(signed=False, label="UInt32"), "saIdleTimeout": SimStruct({"idleTimeoutSeconds": SimTypeInt(signed=False, label="UInt32"), "idleTimeoutSecondsFailOver": SimTypeInt(signed=False, label="UInt32")}, name="IPSEC_SA_IDLE_TIMEOUT0", pack=False, align=None), "emPolicy": SimTypePointer(SimStruct({"numAuthenticationMethods": SimTypeInt(signed=False, label="UInt32"), "authenticationMethods": SimTypePointer(SimStruct({"authenticationMethodType": SimTypeInt(signed=False, label="IKEEXT_AUTHENTICATION_METHOD_TYPE"), "Anonymous": SimUnion({"presharedKeyAuthentication": SimStruct({"presharedKey": SimStruct({"size": SimTypeInt(signed=False, label="UInt32"), "data": SimTypePointer(SimTypeChar(label="Byte"), offset=0)}, name="FWP_BYTE_BLOB", pack=False, align=None), "flags": SimTypeInt(signed=False, label="IKEEXT_PRESHARED_KEY_AUTHENTICATION_FLAGS")}, name="IKEEXT_PRESHARED_KEY_AUTHENTICATION1", pack=False, align=None), "certificateAuthentication": SimStruct({"inboundConfigType": SimTypeInt(signed=False, label="IKEEXT_CERT_CONFIG_TYPE"), "Anonymous1": SimUnion({"Anonymous1": SimStruct({"inboundRootArraySize": SimTypeInt(signed=False, label="UInt32"), "inboundRootCriteria": SimTypePointer(SimStruct({"certData": SimStruct({"size": SimTypeInt(signed=False, label="UInt32"), "data": SimTypePointer(SimTypeChar(label="Byte"), offset=0)}, name="FWP_BYTE_BLOB", pack=False, align=None), "certHash": SimStruct({"size": SimTypeInt(signed=False, label="UInt32"), "data": SimTypePointer(SimTypeChar(label="Byte"), offset=0)}, name="FWP_BYTE_BLOB", pack=False, align=None), "eku": SimTypePointer(SimStruct({"numEku": SimTypeInt(signed=False, label="UInt32"), "eku": SimTypePointer(SimTypePointer(SimTypeChar(label="Byte"), offset=0), offset=0)}, name="IKEEXT_CERT_EKUS0", pack=False, align=None), offset=0), "name": SimTypePointer(SimStruct({"nameType": SimTypeInt(signed=False, label="IKEEXT_CERT_CRITERIA_NAME_TYPE"), "certName": SimTypePointer(SimTypeChar(label="Char"), offset=0)}, name="IKEEXT_CERT_NAME0", pack=False, align=None), offset=0), "flags": SimTypeInt(signed=False, label="UInt32")}, name="IKEEXT_CERTIFICATE_CRITERIA0", pack=False, align=None), offset=0)}, name="_Anonymous1_e__Struct", pack=False, align=None), "Anonymous2": SimStruct({"inboundEnterpriseStoreArraySize": SimTypeInt(signed=False, label="UInt32"), "inboundEnterpriseStoreCriteria": SimTypePointer(SimStruct({"certData": SimStruct({"size": SimTypeInt(signed=False, label="UInt32"), "data": SimTypePointer(SimTypeChar(label="Byte"), offset=0)}, name="FWP_BYTE_BLOB", pack=False, align=None), "certHash": SimStruct({"size": SimTypeInt(signed=False, label="UInt32"), "data": SimTypePointer(SimTypeChar(label="Byte"), offset=0)}, name="FWP_BYTE_BLOB", pack=False, align=None), "eku": SimTypePointer(SimStruct({"numEku": SimTypeInt(signed=False, label="UInt32"), "eku": SimTypePointer(SimTypePointer(SimTypeChar(label="Byte"), offset=0), offset=0)}, name="IKEEXT_CERT_EKUS0", pack=False, align=None), offset=0), "name": SimTypePointer(SimStruct({"nameType": SimTypeInt(signed=False, label="IKEEXT_CERT_CRITERIA_NAME_TYPE"), "certName": SimTypePointer(SimTypeChar(label="Char"), offset=0)}, name="IKEEXT_CERT_NAME0", pack=False, align=None), offset=0), "flags": SimTypeInt(signed=False, label="UInt32")}, name="IKEEXT_CERTIFICATE_CRITERIA0", pack=False, align=None), offset=0)}, name="_Anonymous2_e__Struct", pack=False, align=None), "Anonymous3": SimStruct({"inboundRootStoreArraySize": SimTypeInt(signed=False, label="UInt32"), "inboundTrustedRootStoreCriteria": SimTypePointer(SimStruct({"certData": SimStruct({"size": SimTypeInt(signed=False, label="UInt32"), "data": SimTypePointer(SimTypeChar(label="Byte"), offset=0)}, name="FWP_BYTE_BLOB", pack=False, align=None), "certHash": SimStruct({"size": SimTypeInt(signed=False, label="UInt32"), "data": SimTypePointer(SimTypeChar(label="Byte"), offset=0)}, name="FWP_BYTE_BLOB", pack=False, align=None), "eku": SimTypePointer(SimStruct({"numEku": SimTypeInt(signed=False, label="UInt32"), "eku": SimTypePointer(SimTypePointer(SimTypeChar(label="Byte"), offset=0), offset=0)}, name="IKEEXT_CERT_EKUS0", pack=False, align=None), offset=0), "name": SimTypePointer(SimStruct({"nameType": SimTypeInt(signed=False, label="IKEEXT_CERT_CRITERIA_NAME_TYPE"), "certName": SimTypePointer(SimTypeChar(label="Char"), offset=0)}, name="IKEEXT_CERT_NAME0", pack=False, align=None), offset=0), "flags": SimTypeInt(signed=False, label="UInt32")}, name="IKEEXT_CERTIFICATE_CRITERIA0", pack=False, align=None), offset=0)}, name="_Anonymous3_e__Struct", pack=False, align=None)}, name="<anon>", label="None"), "outboundConfigType": SimTypeInt(signed=False, label="IKEEXT_CERT_CONFIG_TYPE"), "Anonymous2": SimUnion({"Anonymous1": SimStruct({"outboundRootArraySize": SimTypeInt(signed=False, label="UInt32"), "outboundRootCriteria": SimTypePointer(SimStruct({"certData": SimStruct({"size": SimTypeInt(signed=False, label="UInt32"), "data": SimTypePointer(SimTypeChar(label="Byte"), offset=0)}, name="FWP_BYTE_BLOB", pack=False, align=None), "certHash": SimStruct({"size": SimTypeInt(signed=False, label="UInt32"), "data": SimTypePointer(SimTypeChar(label="Byte"), offset=0)}, name="FWP_BYTE_BLOB", pack=False, align=None), "eku": SimTypePointer(SimStruct({"numEku": SimTypeInt(signed=False, label="UInt32"), "eku": SimTypePointer(SimTypePointer(SimTypeChar(label="Byte"), offset=0), offset=0)}, name="IKEEXT_CERT_EKUS0", pack=False, align=None), offset=0), "name": SimTypePointer(SimStruct({"nameType": SimTypeInt(signed=False, label="IKEEXT_CERT_CRITERIA_NAME_TYPE"), "certName": SimTypePointer(SimTypeChar(label="Char"), offset=0)}, name="IKEEXT_CERT_NAME0", pack=False, align=None), offset=0), "flags": SimTypeInt(signed=False, label="UInt32")}, name="IKEEXT_CERTIFICATE_CRITERIA0", pack=False, align=None), offset=0)}, name="_Anonymous1_e__Struct", pack=False, align=None), "Anonymous2": SimStruct({"outboundEnterpriseStoreArraySize": SimTypeInt(signed=False, label="UInt32"), "outboundEnterpriseStoreCriteria": SimTypePointer(SimStruct({"certData": SimStruct({"size": SimTypeInt(signed=False, label="UInt32"), "data": SimTypePointer(SimTypeChar(label="Byte"), offset=0)}, name="FWP_BYTE_BLOB", pack=False, align=None), "certHash": SimStruct({"size": SimTypeInt(signed=False, label="UInt32"), "data": SimTypePointer(SimTypeChar(label="Byte"), offset=0)}, name="FWP_BYTE_BLOB", pack=False, align=None), "eku": SimTypePointer(SimStruct({"numEku": SimTypeInt(signed=False, label="UInt32"), "eku": SimTypePointer(SimTypePointer(SimTypeChar(label="Byte"), offset=0), offset=0)}, name="IKEEXT_CERT_EKUS0", pack=False, align=None), offset=0), "name": SimTypePointer(SimStruct({"nameType": SimTypeInt(signed=False, label="IKEEXT_CERT_CRITERIA_NAME_TYPE"), "certName": SimTypePointer(SimTypeChar(label="Char"), offset=0)}, name="IKEEXT_CERT_NAME0", pack=False, align=None), offset=0), "flags": SimTypeInt(signed=False, label="UInt32")}, name="IKEEXT_CERTIFICATE_CRITERIA0", pack=False, align=None), offset=0)}, name="_Anonymous2_e__Struct", pack=False, align=None), "Anonymous3": SimStruct({"outboundRootStoreArraySize": SimTypeInt(signed=False, label="UInt32"), "outboundTrustedRootStoreCriteria": SimTypePointer(SimStruct({"certData": SimStruct({"size": SimTypeInt(signed=False, label="UInt32"), "data": SimTypePointer(SimTypeChar(label="Byte"), offset=0)}, name="FWP_BYTE_BLOB", pack=False, align=None), "certHash": SimStruct({"size": SimTypeInt(signed=False, label="UInt32"), "data": SimTypePointer(SimTypeChar(label="Byte"), offset=0)}, name="FWP_BYTE_BLOB", pack=False, align=None), "eku": SimTypePointer(SimStruct({"numEku": SimTypeInt(signed=False, label="UInt32"), "eku": SimTypePointer(SimTypePointer(SimTypeChar(label="Byte"), offset=0), offset=0)}, name="IKEEXT_CERT_EKUS0", pack=False, align=None), offset=0), "name": SimTypePointer(SimStruct({"nameType": SimTypeInt(signed=False, label="IKEEXT_CERT_CRITERIA_NAME_TYPE"), "certName": SimTypePointer(SimTypeChar(label="Char"), offset=0)}, name="IKEEXT_CERT_NAME0", pack=False, align=None), offset=0), "flags": SimTypeInt(signed=False, label="UInt32")}, name="IKEEXT_CERTIFICATE_CRITERIA0", pack=False, align=None), offset=0)}, name="_Anonymous3_e__Struct", pack=False, align=None)}, name="<anon>", label="None"), "flags": SimTypeInt(signed=False, label="IKEEXT_CERT_AUTH"), "localCertLocationUrl": SimStruct({"size": SimTypeInt(signed=False, label="UInt32"), "data": SimTypePointer(SimTypeChar(label="Byte"), offset=0)}, name="FWP_BYTE_BLOB", pack=False, align=None)}, name="IKEEXT_CERTIFICATE_AUTHENTICATION2", pack=False, align=None), "kerberosAuthentication": SimStruct({"flags": SimTypeInt(signed=False, label="IKEEXT_KERBEROS_AUTHENTICATION_FLAGS"), "proxyServer": SimTypePointer(SimTypeChar(label="Char"), offset=0)}, name="IKEEXT_KERBEROS_AUTHENTICATION1", pack=False, align=None), "reservedAuthentication": SimStruct({"flags": SimTypeInt(signed=False, label="IKEEXT_RESERVED_AUTHENTICATION_FLAGS")}, name="IKEEXT_RESERVED_AUTHENTICATION0", pack=False, align=None), "ntlmV2Authentication": SimStruct({"flags": SimTypeInt(signed=False, label="UInt32")}, name="IKEEXT_NTLM_V2_AUTHENTICATION0", pack=False, align=None), "sslAuthentication": SimStruct({"inboundConfigType": SimTypeInt(signed=False, label="IKEEXT_CERT_CONFIG_TYPE"), "Anonymous1": SimUnion({"Anonymous1": SimStruct({"inboundRootArraySize": SimTypeInt(signed=False, label="UInt32"), "inboundRootCriteria": SimTypePointer(SimStruct({"certData": SimStruct({"size": SimTypeInt(signed=False, label="UInt32"), "data": SimTypePointer(SimTypeChar(label="Byte"), offset=0)}, name="FWP_BYTE_BLOB", pack=False, align=None), "certHash": SimStruct({"size": SimTypeInt(signed=False, label="UInt32"), "data": SimTypePointer(SimTypeChar(label="Byte"), offset=0)}, name="FWP_BYTE_BLOB", pack=False, align=None), "eku": SimTypePointer(SimStruct({"numEku": SimTypeInt(signed=False, label="UInt32"), "eku": SimTypePointer(SimTypePointer(SimTypeChar(label="Byte"), offset=0), offset=0)}, name="IKEEXT_CERT_EKUS0", pack=False, align=None), offset=0), "name": SimTypePointer(SimStruct({"nameType": SimTypeInt(signed=False, label="IKEEXT_CERT_CRITERIA_NAME_TYPE"), "certName": SimTypePointer(SimTypeChar(label="Char"), offset=0)}, name="IKEEXT_CERT_NAME0", pack=False, align=None), offset=0), "flags": SimTypeInt(signed=False, label="UInt32")}, name="IKEEXT_CERTIFICATE_CRITERIA0", pack=False, align=None), offset=0)}, name="_Anonymous1_e__Struct", pack=False, align=None), "Anonymous2": SimStruct({"inboundEnterpriseStoreArraySize": SimTypeInt(signed=False, label="UInt32"), "inboundEnterpriseStoreCriteria": SimTypePointer(SimStruct({"certData": SimStruct({"size": SimTypeInt(signed=False, label="UInt32"), "data": SimTypePointer(SimTypeChar(label="Byte"), offset=0)}, name="FWP_BYTE_BLOB", pack=False, align=None), "certHash": SimStruct({"size": SimTypeInt(signed=False, label="UInt32"), "data": SimTypePointer(SimTypeChar(label="Byte"), offset=0)}, name="FWP_BYTE_BLOB", pack=False, align=None), "eku": SimTypePointer(SimStruct({"numEku": SimTypeInt(signed=False, label="UInt32"), "eku": SimTypePointer(SimTypePointer(SimTypeChar(label="Byte"), offset=0), offset=0)}, name="IKEEXT_CERT_EKUS0", pack=False, align=None), offset=0), "name": SimTypePointer(SimStruct({"nameType": SimTypeInt(signed=False, label="IKEEXT_CERT_CRITERIA_NAME_TYPE"), "certName": SimTypePointer(SimTypeChar(label="Char"), offset=0)}, name="IKEEXT_CERT_NAME0", pack=False, align=None), offset=0), "flags": SimTypeInt(signed=False, label="UInt32")}, name="IKEEXT_CERTIFICATE_CRITERIA0", pack=False, align=None), offset=0)}, name="_Anonymous2_e__Struct", pack=False, align=None), "Anonymous3": SimStruct({"inboundRootStoreArraySize": SimTypeInt(signed=False, label="UInt32"), "inboundTrustedRootStoreCriteria": SimTypePointer(SimStruct({"certData": SimStruct({"size": SimTypeInt(signed=False, label="UInt32"), "data": SimTypePointer(SimTypeChar(label="Byte"), offset=0)}, name="FWP_BYTE_BLOB", pack=False, align=None), "certHash": SimStruct({"size": SimTypeInt(signed=False, label="UInt32"), "data": SimTypePointer(SimTypeChar(label="Byte"), offset=0)}, name="FWP_BYTE_BLOB", pack=False, align=None), "eku": SimTypePointer(SimStruct({"numEku": SimTypeInt(signed=False, label="UInt32"), "eku": SimTypePointer(SimTypePointer(SimTypeChar(label="Byte"), offset=0), offset=0)}, name="IKEEXT_CERT_EKUS0", pack=False, align=None), offset=0), "name": SimTypePointer(SimStruct({"nameType": SimTypeInt(signed=False, label="IKEEXT_CERT_CRITERIA_NAME_TYPE"), "certName": SimTypePointer(SimTypeChar(label="Char"), offset=0)}, name="IKEEXT_CERT_NAME0", pack=False, align=None), offset=0), "flags": SimTypeInt(signed=False, label="UInt32")}, name="IKEEXT_CERTIFICATE_CRITERIA0", pack=False, align=None), offset=0)}, name="_Anonymous3_e__Struct", pack=False, align=None)}, name="<anon>", label="None"), "outboundConfigType": SimTypeInt(signed=False, label="IKEEXT_CERT_CONFIG_TYPE"), "Anonymous2": SimUnion({"Anonymous1": SimStruct({"outboundRootArraySize": SimTypeInt(signed=False, label="UInt32"), "outboundRootCriteria": SimTypePointer(SimStruct({"certData": SimStruct({"size": SimTypeInt(signed=False, label="UInt32"), "data": SimTypePointer(SimTypeChar(label="Byte"), offset=0)}, name="FWP_BYTE_BLOB", pack=False, align=None), "certHash": SimStruct({"size": SimTypeInt(signed=False, label="UInt32"), "data": SimTypePointer(SimTypeChar(label="Byte"), offset=0)}, name="FWP_BYTE_BLOB", pack=False, align=None), "eku": SimTypePointer(SimStruct({"numEku": SimTypeInt(signed=False, label="UInt32"), "eku": SimTypePointer(SimTypePointer(SimTypeChar(label="Byte"), offset=0), offset=0)}, name="IKEEXT_CERT_EKUS0", pack=False, align=None), offset=0), "name": SimTypePointer(SimStruct({"nameType": SimTypeInt(signed=False, label="IKEEXT_CERT_CRITERIA_NAME_TYPE"), "certName": SimTypePointer(SimTypeChar(label="Char"), offset=0)}, name="IKEEXT_CERT_NAME0", pack=False, align=None), offset=0), "flags": SimTypeInt(signed=False, label="UInt32")}, name="IKEEXT_CERTIFICATE_CRITERIA0", pack=False, align=None), offset=0)}, name="_Anonymous1_e__Struct", pack=False, align=None), "Anonymous2": SimStruct({"outboundEnterpriseStoreArraySize": SimTypeInt(signed=False, label="UInt32"), "outboundEnterpriseStoreCriteria": SimTypePointer(SimStruct({"certData": SimStruct({"size": SimTypeInt(signed=False, label="UInt32"), "data": SimTypePointer(SimTypeChar(label="Byte"), offset=0)}, name="FWP_BYTE_BLOB", pack=False, align=None), "certHash": SimStruct({"size": SimTypeInt(signed=False, label="UInt32"), "data": SimTypePointer(SimTypeChar(label="Byte"), offset=0)}, name="FWP_BYTE_BLOB", pack=False, align=None), "eku": SimTypePointer(SimStruct({"numEku": SimTypeInt(signed=False, label="UInt32"), "eku": SimTypePointer(SimTypePointer(SimTypeChar(label="Byte"), offset=0), offset=0)}, name="IKEEXT_CERT_EKUS0", pack=False, align=None), offset=0), "name": SimTypePointer(SimStruct({"nameType": SimTypeInt(signed=False, label="IKEEXT_CERT_CRITERIA_NAME_TYPE"), "certName": SimTypePointer(SimTypeChar(label="Char"), offset=0)}, name="IKEEXT_CERT_NAME0", pack=False, align=None), offset=0), "flags": SimTypeInt(signed=False, label="UInt32")}, name="IKEEXT_CERTIFICATE_CRITERIA0", pack=False, align=None), offset=0)}, name="_Anonymous2_e__Struct", pack=False, align=None), "Anonymous3": SimStruct({"outboundRootStoreArraySize": SimTypeInt(signed=False, label="UInt32"), "outboundTrustedRootStoreCriteria": SimTypePointer(SimStruct({"certData": SimStruct({"size": SimTypeInt(signed=False, label="UInt32"), "data": SimTypePointer(SimTypeChar(label="Byte"), offset=0)}, name="FWP_BYTE_BLOB", pack=False, align=None), "certHash": SimStruct({"size": SimTypeInt(signed=False, label="UInt32"), "data": SimTypePointer(SimTypeChar(label="Byte"), offset=0)}, name="FWP_BYTE_BLOB", pack=False, align=None), "eku": SimTypePointer(SimStruct({"numEku": SimTypeInt(signed=False, label="UInt32"), "eku": SimTypePointer(SimTypePointer(SimTypeChar(label="Byte"), offset=0), offset=0)}, name="IKEEXT_CERT_EKUS0", pack=False, align=None), offset=0), "name": SimTypePointer(SimStruct({"nameType": SimTypeInt(signed=False, label="IKEEXT_CERT_CRITERIA_NAME_TYPE"), "certName": SimTypePointer(SimTypeChar(label="Char"), offset=0)}, name="IKEEXT_CERT_NAME0", pack=False, align=None), offset=0), "flags": SimTypeInt(signed=False, label="UInt32")}, name="IKEEXT_CERTIFICATE_CRITERIA0", pack=False, align=None), offset=0)}, name="_Anonymous3_e__Struct", pack=False, align=None)}, name="<anon>", label="None"), "flags": SimTypeInt(signed=False, label="IKEEXT_CERT_AUTH"), "localCertLocationUrl": SimStruct({"size": SimTypeInt(signed=False, label="UInt32"), "data": SimTypePointer(SimTypeChar(label="Byte"), offset=0)}, name="FWP_BYTE_BLOB", pack=False, align=None)}, name="IKEEXT_CERTIFICATE_AUTHENTICATION2", pack=False, align=None), "cgaAuthentication": SimStruct({"keyContainerName": SimTypePointer(SimTypeChar(label="Char"), offset=0), "cspName": SimTypePointer(SimTypeChar(label="Char"), offset=0), "cspType": SimTypeInt(signed=False, label="UInt32"), "cgaModifier": SimStruct({"byteArray16": SimTypeFixedSizeArray(SimTypeChar(label="Byte"), 16)}, name="FWP_BYTE_ARRAY16", pack=False, align=None), "cgaCollisionCount": SimTypeChar(label="Byte")}, name="IKEEXT_IPV6_CGA_AUTHENTICATION0", pack=False, align=None), "eapAuthentication": SimStruct({"flags": SimTypeInt(signed=False, label="IKEEXT_EAP_AUTHENTICATION_FLAGS")}, name="IKEEXT_EAP_AUTHENTICATION0", pack=False, align=None)}, name="<anon>", label="None")}, name="IKEEXT_AUTHENTICATION_METHOD2", pack=False, align=None), offset=0), "initiatorImpersonationType": SimTypeInt(signed=False, label="IKEEXT_AUTHENTICATION_IMPERSONATION_TYPE")}, name="IKEEXT_EM_POLICY2", pack=False, align=None), offset=0)}, name="IPSEC_TRANSPORT_POLICY2", pack=False, align=None), offset=0), "ikeV2MmPolicy": SimTypePointer(SimStruct({"softExpirationTime": SimTypeInt(signed=False, label="UInt32"), "numAuthenticationMethods": SimTypeInt(signed=False, label="UInt32"), "authenticationMethods": SimTypePointer(SimStruct({"authenticationMethodType": SimTypeInt(signed=False, label="IKEEXT_AUTHENTICATION_METHOD_TYPE"), "Anonymous": SimUnion({"presharedKeyAuthentication": SimStruct({"presharedKey": SimStruct({"size": SimTypeInt(signed=False, label="UInt32"), "data": SimTypePointer(SimTypeChar(label="Byte"), offset=0)}, name="FWP_BYTE_BLOB", pack=False, align=None), "flags": SimTypeInt(signed=False, label="IKEEXT_PRESHARED_KEY_AUTHENTICATION_FLAGS")}, name="IKEEXT_PRESHARED_KEY_AUTHENTICATION1", pack=False, align=None), "certificateAuthentication": SimStruct({"inboundConfigType": SimTypeInt(signed=False, label="IKEEXT_CERT_CONFIG_TYPE"), "Anonymous1": SimUnion({"Anonymous1": SimStruct({"inboundRootArraySize": SimTypeInt(signed=False, label="UInt32"), "inboundRootCriteria": SimTypePointer(SimStruct({"certData": SimStruct({"size": SimTypeInt(signed=False, label="UInt32"), "data": SimTypePointer(SimTypeChar(label="Byte"), offset=0)}, name="FWP_BYTE_BLOB", pack=False, align=None), "certHash": SimStruct({"size": SimTypeInt(signed=False, label="UInt32"), "data": SimTypePointer(SimTypeChar(label="Byte"), offset=0)}, name="FWP_BYTE_BLOB", pack=False, align=None), "eku": SimTypePointer(SimStruct({"numEku": SimTypeInt(signed=False, label="UInt32"), "eku": SimTypePointer(SimTypePointer(SimTypeChar(label="Byte"), offset=0), offset=0)}, name="IKEEXT_CERT_EKUS0", pack=False, align=None), offset=0), "name": SimTypePointer(SimStruct({"nameType": SimTypeInt(signed=False, label="IKEEXT_CERT_CRITERIA_NAME_TYPE"), "certName": SimTypePointer(SimTypeChar(label="Char"), offset=0)}, name="IKEEXT_CERT_NAME0", pack=False, align=None), offset=0), "flags": SimTypeInt(signed=False, label="UInt32")}, name="IKEEXT_CERTIFICATE_CRITERIA0", pack=False, align=None), offset=0)}, name="_Anonymous1_e__Struct", pack=False, align=None), "Anonymous2": SimStruct({"inboundEnterpriseStoreArraySize": SimTypeInt(signed=False, label="UInt32"), "inboundEnterpriseStoreCriteria": SimTypePointer(SimStruct({"certData": SimStruct({"size": SimTypeInt(signed=False, label="UInt32"), "data": SimTypePointer(SimTypeChar(label="Byte"), offset=0)}, name="FWP_BYTE_BLOB", pack=False, align=None), "certHash": SimStruct({"size": SimTypeInt(signed=False, label="UInt32"), "data": SimTypePointer(SimTypeChar(label="Byte"), offset=0)}, name="FWP_BYTE_BLOB", pack=False, align=None), "eku": SimTypePointer(SimStruct({"numEku": SimTypeInt(signed=False, label="UInt32"), "eku": SimTypePointer(SimTypePointer(SimTypeChar(label="Byte"), offset=0), offset=0)}, name="IKEEXT_CERT_EKUS0", pack=False, align=None), offset=0), "name": SimTypePointer(SimStruct({"nameType": SimTypeInt(signed=False, label="IKEEXT_CERT_CRITERIA_NAME_TYPE"), "certName": SimTypePointer(SimTypeChar(label="Char"), offset=0)}, name="IKEEXT_CERT_NAME0", pack=False, align=None), offset=0), "flags": SimTypeInt(signed=False, label="UInt32")}, name="IKEEXT_CERTIFICATE_CRITERIA0", pack=False, align=None), offset=0)}, name="_Anonymous2_e__Struct", pack=False, align=None), "Anonymous3": SimStruct({"inboundRootStoreArraySize": SimTypeInt(signed=False, label="UInt32"), "inboundTrustedRootStoreCriteria": SimTypePointer(SimStruct({"certData": SimStruct({"size": SimTypeInt(signed=False, label="UInt32"), "data": SimTypePointer(SimTypeChar(label="Byte"), offset=0)}, name="FWP_BYTE_BLOB", pack=False, align=None), "certHash": SimStruct({"size": SimTypeInt(signed=False, label="UInt32"), "data": SimTypePointer(SimTypeChar(label="Byte"), offset=0)}, name="FWP_BYTE_BLOB", pack=False, align=None), "eku": SimTypePointer(SimStruct({"numEku": SimTypeInt(signed=False, label="UInt32"), "eku": SimTypePointer(SimTypePointer(SimTypeChar(label="Byte"), offset=0), offset=0)}, name="IKEEXT_CERT_EKUS0", pack=False, align=None), offset=0), "name": SimTypePointer(SimStruct({"nameType": SimTypeInt(signed=False, label="IKEEXT_CERT_CRITERIA_NAME_TYPE"), "certName": SimTypePointer(SimTypeChar(label="Char"), offset=0)}, name="IKEEXT_CERT_NAME0", pack=False, align=None), offset=0), "flags": SimTypeInt(signed=False, label="UInt32")}, name="IKEEXT_CERTIFICATE_CRITERIA0", pack=False, align=None), offset=0)}, name="_Anonymous3_e__Struct", pack=False, align=None)}, name="<anon>", label="None"), "outboundConfigType": SimTypeInt(signed=False, label="IKEEXT_CERT_CONFIG_TYPE"), "Anonymous2": SimUnion({"Anonymous1": SimStruct({"outboundRootArraySize": SimTypeInt(signed=False, label="UInt32"), "outboundRootCriteria": SimTypePointer(SimStruct({"certData": SimStruct({"size": SimTypeInt(signed=False, label="UInt32"), "data": SimTypePointer(SimTypeChar(label="Byte"), offset=0)}, name="FWP_BYTE_BLOB", pack=False, align=None), "certHash": SimStruct({"size": SimTypeInt(signed=False, label="UInt32"), "data": SimTypePointer(SimTypeChar(label="Byte"), offset=0)}, name="FWP_BYTE_BLOB", pack=False, align=None), "eku": SimTypePointer(SimStruct({"numEku": SimTypeInt(signed=False, label="UInt32"), "eku": SimTypePointer(SimTypePointer(SimTypeChar(label="Byte"), offset=0), offset=0)}, name="IKEEXT_CERT_EKUS0", pack=False, align=None), offset=0), "name": SimTypePointer(SimStruct({"nameType": SimTypeInt(signed=False, label="IKEEXT_CERT_CRITERIA_NAME_TYPE"), "certName": SimTypePointer(SimTypeChar(label="Char"), offset=0)}, name="IKEEXT_CERT_NAME0", pack=False, align=None), offset=0), "flags": SimTypeInt(signed=False, label="UInt32")}, name="IKEEXT_CERTIFICATE_CRITERIA0", pack=False, align=None), offset=0)}, name="_Anonymous1_e__Struct", pack=False, align=None), "Anonymous2": SimStruct({"outboundEnterpriseStoreArraySize": SimTypeInt(signed=False, label="UInt32"), "outboundEnterpriseStoreCriteria": SimTypePointer(SimStruct({"certData": SimStruct({"size": SimTypeInt(signed=False, label="UInt32"), "data": SimTypePointer(SimTypeChar(label="Byte"), offset=0)}, name="FWP_BYTE_BLOB", pack=False, align=None), "certHash": SimStruct({"size": SimTypeInt(signed=False, label="UInt32"), "data": SimTypePointer(SimTypeChar(label="Byte"), offset=0)}, name="FWP_BYTE_BLOB", pack=False, align=None), "eku": SimTypePointer(SimStruct({"numEku": SimTypeInt(signed=False, label="UInt32"), "eku": SimTypePointer(SimTypePointer(SimTypeChar(label="Byte"), offset=0), offset=0)}, name="IKEEXT_CERT_EKUS0", pack=False, align=None), offset=0), "name": SimTypePointer(SimStruct({"nameType": SimTypeInt(signed=False, label="IKEEXT_CERT_CRITERIA_NAME_TYPE"), "certName": SimTypePointer(SimTypeChar(label="Char"), offset=0)}, name="IKEEXT_CERT_NAME0", pack=False, align=None), offset=0), "flags": SimTypeInt(signed=False, label="UInt32")}, name="IKEEXT_CERTIFICATE_CRITERIA0", pack=False, align=None), offset=0)}, name="_Anonymous2_e__Struct", pack=False, align=None), "Anonymous3": SimStruct({"outboundRootStoreArraySize": SimTypeInt(signed=False, label="UInt32"), "outboundTrustedRootStoreCriteria": SimTypePointer(SimStruct({"certData": SimStruct({"size": SimTypeInt(signed=False, label="UInt32"), "data": SimTypePointer(SimTypeChar(label="Byte"), offset=0)}, name="FWP_BYTE_BLOB", pack=False, align=None), "certHash": SimStruct({"size": SimTypeInt(signed=False, label="UInt32"), "data": SimTypePointer(SimTypeChar(label="Byte"), offset=0)}, name="FWP_BYTE_BLOB", pack=False, align=None), "eku": SimTypePointer(SimStruct({"numEku": SimTypeInt(signed=False, label="UInt32"), "eku": SimTypePointer(SimTypePointer(SimTypeChar(label="Byte"), offset=0), offset=0)}, name="IKEEXT_CERT_EKUS0", pack=False, align=None), offset=0), "name": SimTypePointer(SimStruct({"nameType": SimTypeInt(signed=False, label="IKEEXT_CERT_CRITERIA_NAME_TYPE"), "certName": SimTypePointer(SimTypeChar(label="Char"), offset=0)}, name="IKEEXT_CERT_NAME0", pack=False, align=None), offset=0), "flags": SimTypeInt(signed=False, label="UInt32")}, name="IKEEXT_CERTIFICATE_CRITERIA0", pack=False, align=None), offset=0)}, name="_Anonymous3_e__Struct", pack=False, align=None)}, name="<anon>", label="None"), "flags": SimTypeInt(signed=False, label="IKEEXT_CERT_AUTH"), "localCertLocationUrl": SimStruct({"size": SimTypeInt(signed=False, label="UInt32"), "data": SimTypePointer(SimTypeChar(label="Byte"), offset=0)}, name="FWP_BYTE_BLOB", pack=False, align=None)}, name="IKEEXT_CERTIFICATE_AUTHENTICATION2", pack=False, align=None), "kerberosAuthentication": SimStruct({"flags": SimTypeInt(signed=False, label="IKEEXT_KERBEROS_AUTHENTICATION_FLAGS"), "proxyServer": SimTypePointer(SimTypeChar(label="Char"), offset=0)}, name="IKEEXT_KERBEROS_AUTHENTICATION1", pack=False, align=None), "reservedAuthentication": SimStruct({"flags": SimTypeInt(signed=False, label="IKEEXT_RESERVED_AUTHENTICATION_FLAGS")}, name="IKEEXT_RESERVED_AUTHENTICATION0", pack=False, align=None), "ntlmV2Authentication": SimStruct({"flags": SimTypeInt(signed=False, label="UInt32")}, name="IKEEXT_NTLM_V2_AUTHENTICATION0", pack=False, align=None), "sslAuthentication": SimStruct({"inboundConfigType": SimTypeInt(signed=False, label="IKEEXT_CERT_CONFIG_TYPE"), "Anonymous1": SimUnion({"Anonymous1": SimStruct({"inboundRootArraySize": SimTypeInt(signed=False, label="UInt32"), "inboundRootCriteria": SimTypePointer(SimStruct({"certData": SimStruct({"size": SimTypeInt(signed=False, label="UInt32"), "data": SimTypePointer(SimTypeChar(label="Byte"), offset=0)}, name="FWP_BYTE_BLOB", pack=False, align=None), "certHash": SimStruct({"size": SimTypeInt(signed=False, label="UInt32"), "data": SimTypePointer(SimTypeChar(label="Byte"), offset=0)}, name="FWP_BYTE_BLOB", pack=False, align=None), "eku": SimTypePointer(SimStruct({"numEku": SimTypeInt(signed=False, label="UInt32"), "eku": SimTypePointer(SimTypePointer(SimTypeChar(label="Byte"), offset=0), offset=0)}, name="IKEEXT_CERT_EKUS0", pack=False, align=None), offset=0), "name": SimTypePointer(SimStruct({"nameType": SimTypeInt(signed=False, label="IKEEXT_CERT_CRITERIA_NAME_TYPE"), "certName": SimTypePointer(SimTypeChar(label="Char"), offset=0)}, name="IKEEXT_CERT_NAME0", pack=False, align=None), offset=0), "flags": SimTypeInt(signed=False, label="UInt32")}, name="IKEEXT_CERTIFICATE_CRITERIA0", pack=False, align=None), offset=0)}, name="_Anonymous1_e__Struct", pack=False, align=None), "Anonymous2": SimStruct({"inboundEnterpriseStoreArraySize": SimTypeInt(signed=False, label="UInt32"), "inboundEnterpriseStoreCriteria": SimTypePointer(SimStruct({"certData": SimStruct({"size": SimTypeInt(signed=False, label="UInt32"), "data": SimTypePointer(SimTypeChar(label="Byte"), offset=0)}, name="FWP_BYTE_BLOB", pack=False, align=None), "certHash": SimStruct({"size": SimTypeInt(signed=False, label="UInt32"), "data": SimTypePointer(SimTypeChar(label="Byte"), offset=0)}, name="FWP_BYTE_BLOB", pack=False, align=None), "eku": SimTypePointer(SimStruct({"numEku": SimTypeInt(signed=False, label="UInt32"), "eku": SimTypePointer(SimTypePointer(SimTypeChar(label="Byte"), offset=0), offset=0)}, name="IKEEXT_CERT_EKUS0", pack=False, align=None), offset=0), "name": SimTypePointer(SimStruct({"nameType": SimTypeInt(signed=False, label="IKEEXT_CERT_CRITERIA_NAME_TYPE"), "certName": SimTypePointer(SimTypeChar(label="Char"), offset=0)}, name="IKEEXT_CERT_NAME0", pack=False, align=None), offset=0), "flags": SimTypeInt(signed=False, label="UInt32")}, name="IKEEXT_CERTIFICATE_CRITERIA0", pack=False, align=None), offset=0)}, name="_Anonymous2_e__Struct", pack=False, align=None), "Anonymous3": SimStruct({"inboundRootStoreArraySize": SimTypeInt(signed=False, label="UInt32"), "inboundTrustedRootStoreCriteria": SimTypePointer(SimStruct({"certData": SimStruct({"size": SimTypeInt(signed=False, label="UInt32"), "data": SimTypePointer(SimTypeChar(label="Byte"), offset=0)}, name="FWP_BYTE_BLOB", pack=False, align=None), "certHash": SimStruct({"size": SimTypeInt(signed=False, label="UInt32"), "data": SimTypePointer(SimTypeChar(label="Byte"), offset=0)}, name="FWP_BYTE_BLOB", pack=False, align=None), "eku": SimTypePointer(SimStruct({"numEku": SimTypeInt(signed=False, label="UInt32"), "eku": SimTypePointer(SimTypePointer(SimTypeChar(label="Byte"), offset=0), offset=0)}, name="IKEEXT_CERT_EKUS0", pack=False, align=None), offset=0), "name": SimTypePointer(SimStruct({"nameType": SimTypeInt(signed=False, label="IKEEXT_CERT_CRITERIA_NAME_TYPE"), "certName": SimTypePointer(SimTypeChar(label="Char"), offset=0)}, name="IKEEXT_CERT_NAME0", pack=False, align=None), offset=0), "flags": SimTypeInt(signed=False, label="UInt32")}, name="IKEEXT_CERTIFICATE_CRITERIA0", pack=False, align=None), offset=0)}, name="_Anonymous3_e__Struct", pack=False, align=None)}, name="<anon>", label="None"), "outboundConfigType": SimTypeInt(signed=False, label="IKEEXT_CERT_CONFIG_TYPE"), "Anonymous2": SimUnion({"Anonymous1": SimStruct({"outboundRootArraySize": SimTypeInt(signed=False, label="UInt32"), "outboundRootCriteria": SimTypePointer(SimStruct({"certData": SimStruct({"size": SimTypeInt(signed=False, label="UInt32"), "data": SimTypePointer(SimTypeChar(label="Byte"), offset=0)}, name="FWP_BYTE_BLOB", pack=False, align=None), "certHash": SimStruct({"size": SimTypeInt(signed=False, label="UInt32"), "data": SimTypePointer(SimTypeChar(label="Byte"), offset=0)}, name="FWP_BYTE_BLOB", pack=False, align=None), "eku": SimTypePointer(SimStruct({"numEku": SimTypeInt(signed=False, label="UInt32"), "eku": SimTypePointer(SimTypePointer(SimTypeChar(label="Byte"), offset=0), offset=0)}, name="IKEEXT_CERT_EKUS0", pack=False, align=None), offset=0), "name": SimTypePointer(SimStruct({"nameType": SimTypeInt(signed=False, label="IKEEXT_CERT_CRITERIA_NAME_TYPE"), "certName": SimTypePointer(SimTypeChar(label="Char"), offset=0)}, name="IKEEXT_CERT_NAME0", pack=False, align=None), offset=0), "flags": SimTypeInt(signed=False, label="UInt32")}, name="IKEEXT_CERTIFICATE_CRITERIA0", pack=False, align=None), offset=0)}, name="_Anonymous1_e__Struct", pack=False, align=None), "Anonymous2": SimStruct({"outboundEnterpriseStoreArraySize": SimTypeInt(signed=False, label="UInt32"), "outboundEnterpriseStoreCriteria": SimTypePointer(SimStruct({"certData": SimStruct({"size": SimTypeInt(signed=False, label="UInt32"), "data": SimTypePointer(SimTypeChar(label="Byte"), offset=0)}, name="FWP_BYTE_BLOB", pack=False, align=None), "certHash": SimStruct({"size": SimTypeInt(signed=False, label="UInt32"), "data": SimTypePointer(SimTypeChar(label="Byte"), offset=0)}, name="FWP_BYTE_BLOB", pack=False, align=None), "eku": SimTypePointer(SimStruct({"numEku": SimTypeInt(signed=False, label="UInt32"), "eku": SimTypePointer(SimTypePointer(SimTypeChar(label="Byte"), offset=0), offset=0)}, name="IKEEXT_CERT_EKUS0", pack=False, align=None), offset=0), "name": SimTypePointer(SimStruct({"nameType": SimTypeInt(signed=False, label="IKEEXT_CERT_CRITERIA_NAME_TYPE"), "certName": SimTypePointer(SimTypeChar(label="Char"), offset=0)}, name="IKEEXT_CERT_NAME0", pack=False, align=None), offset=0), "flags": SimTypeInt(signed=False, label="UInt32")}, name="IKEEXT_CERTIFICATE_CRITERIA0", pack=False, align=None), offset=0)}, name="_Anonymous2_e__Struct", pack=False, align=None), "Anonymous3": SimStruct({"outboundRootStoreArraySize": SimTypeInt(signed=False, label="UInt32"), "outboundTrustedRootStoreCriteria": SimTypePointer(SimStruct({"certData": SimStruct({"size": SimTypeInt(signed=False, label="UInt32"), "data": SimTypePointer(SimTypeChar(label="Byte"), offset=0)}, name="FWP_BYTE_BLOB", pack=False, align=None), "certHash": SimStruct({"size": SimTypeInt(signed=False, label="UInt32"), "data": SimTypePointer(SimTypeChar(label="Byte"), offset=0)}, name="FWP_BYTE_BLOB", pack=False, align=None), "eku": SimTypePointer(SimStruct({"numEku": SimTypeInt(signed=False, label="UInt32"), "eku": SimTypePointer(SimTypePointer(SimTypeChar(label="Byte"), offset=0), offset=0)}, name="IKEEXT_CERT_EKUS0", pack=False, align=None), offset=0), "name": SimTypePointer(SimStruct({"nameType": SimTypeInt(signed=False, label="IKEEXT_CERT_CRITERIA_NAME_TYPE"), "certName": SimTypePointer(SimTypeChar(label="Char"), offset=0)}, name="IKEEXT_CERT_NAME0", pack=False, align=None), offset=0), "flags": SimTypeInt(signed=False, label="UInt32")}, name="IKEEXT_CERTIFICATE_CRITERIA0", pack=False, align=None), offset=0)}, name="_Anonymous3_e__Struct", pack=False, align=None)}, name="<anon>", label="None"), "flags": SimTypeInt(signed=False, label="IKEEXT_CERT_AUTH"), "localCertLocationUrl": SimStruct({"size": SimTypeInt(signed=False, label="UInt32"), "data": SimTypePointer(SimTypeChar(label="Byte"), offset=0)}, name="FWP_BYTE_BLOB", pack=False, align=None)}, name="IKEEXT_CERTIFICATE_AUTHENTICATION2", pack=False, align=None), "cgaAuthentication": SimStruct({"keyContainerName": SimTypePointer(SimTypeChar(label="Char"), offset=0), "cspName": SimTypePointer(SimTypeChar(label="Char"), offset=0), "cspType": SimTypeInt(signed=False, label="UInt32"), "cgaModifier": SimStruct({"byteArray16": SimTypeFixedSizeArray(SimTypeChar(label="Byte"), 16)}, name="FWP_BYTE_ARRAY16", pack=False, align=None), "cgaCollisionCount": SimTypeChar(label="Byte")}, name="IKEEXT_IPV6_CGA_AUTHENTICATION0", pack=False, align=None), "eapAuthentication": SimStruct({"flags": SimTypeInt(signed=False, label="IKEEXT_EAP_AUTHENTICATION_FLAGS")}, name="IKEEXT_EAP_AUTHENTICATION0", pack=False, align=None)}, name="<anon>", label="None")}, name="IKEEXT_AUTHENTICATION_METHOD2", pack=False, align=None), offset=0), "initiatorImpersonationType": SimTypeInt(signed=False, label="IKEEXT_AUTHENTICATION_IMPERSONATION_TYPE"), "numIkeProposals": SimTypeInt(signed=False, label="UInt32"), "ikeProposals": SimTypePointer(SimStruct({"cipherAlgorithm": SimStruct({"algoIdentifier": SimTypeInt(signed=False, label="IKEEXT_CIPHER_TYPE"), "keyLen": SimTypeInt(signed=False, label="UInt32"), "rounds": SimTypeInt(signed=False, label="UInt32")}, name="IKEEXT_CIPHER_ALGORITHM0", pack=False, align=None), "integrityAlgorithm": SimStruct({"algoIdentifier": SimTypeInt(signed=False, label="IKEEXT_INTEGRITY_TYPE")}, name="IKEEXT_INTEGRITY_ALGORITHM0", pack=False, align=None), "maxLifetimeSeconds": SimTypeInt(signed=False, label="UInt32"), "dhGroup": SimTypeInt(signed=False, label="IKEEXT_DH_GROUP"), "quickModeLimit": SimTypeInt(signed=False, label="UInt32")}, name="IKEEXT_PROPOSAL0", pack=False, align=None), offset=0), "flags": SimTypeInt(signed=False, label="IKEEXT_POLICY_FLAG"), "maxDynamicFilters": SimTypeInt(signed=False, label="UInt32"), "retransmitDurationSecs": SimTypeInt(signed=False, label="UInt32")}, name="IKEEXT_POLICY2", pack=False, align=None), offset=0), "idpOptions": SimTypePointer(SimStruct({"stateIdleTimeoutSeconds": SimTypeInt(signed=False, label="UInt32"), "perIPRateLimitQueueIdleTimeoutSeconds": SimTypeInt(signed=False, label="UInt32"), "ipV6IPsecUnauthDscp": SimTypeChar(label="Byte"), "ipV6IPsecUnauthRateLimitBytesPerSec": SimTypeInt(signed=False, label="UInt32"), "ipV6IPsecUnauthPerIPRateLimitBytesPerSec": SimTypeInt(signed=False, label="UInt32"), "ipV6IPsecAuthDscp": SimTypeChar(label="Byte"), "ipV6IPsecAuthRateLimitBytesPerSec": SimTypeInt(signed=False, label="UInt32"), "icmpV6Dscp": SimTypeChar(label="Byte"), "icmpV6RateLimitBytesPerSec": SimTypeInt(signed=False, label="UInt32"), "ipV6FilterExemptDscp": SimTypeChar(label="Byte"), "ipV6FilterExemptRateLimitBytesPerSec": SimTypeInt(signed=False, label="UInt32"), "defBlockExemptDscp": SimTypeChar(label="Byte"), "defBlockExemptRateLimitBytesPerSec": SimTypeInt(signed=False, label="UInt32"), "maxStateEntries": SimTypeInt(signed=False, label="UInt32"), "maxPerIPRateLimitQueues": SimTypeInt(signed=False, label="UInt32"), "flags": SimTypeInt(signed=False, label="IPSEC_DOSP_FLAGS"), "numPublicIFLuids": SimTypeInt(signed=False, label="UInt32"), "publicIFLuids": SimTypePointer(SimTypeLongLong(signed=False, label="UInt64"), offset=0), "numInternalIFLuids": SimTypeInt(signed=False, label="UInt32"), "internalIFLuids": SimTypePointer(SimTypeLongLong(signed=False, label="UInt64"), offset=0), "publicV6AddrMask": SimStruct({"addr": SimTypeFixedSizeArray(SimTypeChar(label="Byte"), 16), "prefixLength": SimTypeChar(label="Byte")}, name="FWP_V6_ADDR_AND_MASK", pack=False, align=None), "internalV6AddrMask": SimStruct({"addr": SimTypeFixedSizeArray(SimTypeChar(label="Byte"), 16), "prefixLength": SimTypeChar(label="Byte")}, name="FWP_V6_ADDR_AND_MASK", pack=False, align=None)}, name="IPSEC_DOSP_OPTIONS0", pack=False, align=None), offset=0)}, name="<anon>", label="None"), "providerContextId": SimTypeLongLong(signed=False, label="UInt64")}, name="FWPM_PROVIDER_CONTEXT3_", pack=False, align=None), offset=0), offset=0), offset=0), SimTypePointer(SimTypeInt(signed=False, label="UInt32"), offset=0)], SimTypeInt(signed=False, label="UInt32"), arg_names=["engineHandle", "enumHandle", "numEntriesRequested", "entries", "numEntriesReturned"]),
        # 
        'FwpmProviderContextDestroyEnumHandle0': SimTypeFunction([SimTypePointer(SimTypeInt(signed=True, label="Int"), label="IntPtr", offset=0), SimTypePointer(SimTypeInt(signed=True, label="Int"), label="IntPtr", offset=0)], SimTypeInt(signed=False, label="UInt32"), arg_names=["engineHandle", "enumHandle"]),
        # 
        'FwpmProviderContextGetSecurityInfoByKey0': SimTypeFunction([SimTypePointer(SimTypeInt(signed=True, label="Int"), label="IntPtr", offset=0), SimTypePointer(SimTypeBottom(label="Guid"), offset=0), SimTypeInt(signed=False, label="UInt32"), SimTypePointer(SimTypePointer(SimTypeInt(signed=True, label="Int"), label="IntPtr", offset=0), offset=0), SimTypePointer(SimTypePointer(SimTypeInt(signed=True, label="Int"), label="IntPtr", offset=0), offset=0), SimTypePointer(SimTypePointer(SimStruct({"AclRevision": SimTypeChar(label="Byte"), "Sbz1": SimTypeChar(label="Byte"), "AclSize": SimTypeShort(signed=False, label="UInt16"), "AceCount": SimTypeShort(signed=False, label="UInt16"), "Sbz2": SimTypeShort(signed=False, label="UInt16")}, name="ACL", pack=False, align=None), offset=0), offset=0), SimTypePointer(SimTypePointer(SimStruct({"AclRevision": SimTypeChar(label="Byte"), "Sbz1": SimTypeChar(label="Byte"), "AclSize": SimTypeShort(signed=False, label="UInt16"), "AceCount": SimTypeShort(signed=False, label="UInt16"), "Sbz2": SimTypeShort(signed=False, label="UInt16")}, name="ACL", pack=False, align=None), offset=0), offset=0), SimTypePointer(SimTypePointer(SimStruct({"Revision": SimTypeChar(label="Byte"), "Sbz1": SimTypeChar(label="Byte"), "Control": SimTypeShort(signed=False, label="UInt16"), "Owner": SimTypePointer(SimTypeInt(signed=True, label="Int"), label="IntPtr", offset=0), "Group": SimTypePointer(SimTypeInt(signed=True, label="Int"), label="IntPtr", offset=0), "Sacl": SimTypePointer(SimStruct({"AclRevision": SimTypeChar(label="Byte"), "Sbz1": SimTypeChar(label="Byte"), "AclSize": SimTypeShort(signed=False, label="UInt16"), "AceCount": SimTypeShort(signed=False, label="UInt16"), "Sbz2": SimTypeShort(signed=False, label="UInt16")}, name="ACL", pack=False, align=None), offset=0), "Dacl": SimTypePointer(SimStruct({"AclRevision": SimTypeChar(label="Byte"), "Sbz1": SimTypeChar(label="Byte"), "AclSize": SimTypeShort(signed=False, label="UInt16"), "AceCount": SimTypeShort(signed=False, label="UInt16"), "Sbz2": SimTypeShort(signed=False, label="UInt16")}, name="ACL", pack=False, align=None), offset=0)}, name="SECURITY_DESCRIPTOR", pack=False, align=None), offset=0), offset=0)], SimTypeInt(signed=False, label="UInt32"), arg_names=["engineHandle", "key", "securityInfo", "sidOwner", "sidGroup", "dacl", "sacl", "securityDescriptor"]),
        # 
        'FwpmProviderContextSetSecurityInfoByKey0': SimTypeFunction([SimTypePointer(SimTypeInt(signed=True, label="Int"), label="IntPtr", offset=0), SimTypePointer(SimTypeBottom(label="Guid"), offset=0), SimTypeInt(signed=False, label="UInt32"), SimTypePointer(SimStruct({"Revision": SimTypeChar(label="Byte"), "SubAuthorityCount": SimTypeChar(label="Byte"), "IdentifierAuthority": SimStruct({"Value": SimTypeFixedSizeArray(SimTypeChar(label="Byte"), 6)}, name="SID_IDENTIFIER_AUTHORITY", pack=False, align=None), "SubAuthority": SimTypePointer(SimTypeInt(signed=False, label="UInt32"), offset=0)}, name="SID", pack=False, align=None), offset=0), SimTypePointer(SimStruct({"Revision": SimTypeChar(label="Byte"), "SubAuthorityCount": SimTypeChar(label="Byte"), "IdentifierAuthority": SimStruct({"Value": SimTypeFixedSizeArray(SimTypeChar(label="Byte"), 6)}, name="SID_IDENTIFIER_AUTHORITY", pack=False, align=None), "SubAuthority": SimTypePointer(SimTypeInt(signed=False, label="UInt32"), offset=0)}, name="SID", pack=False, align=None), offset=0), SimTypePointer(SimStruct({"AclRevision": SimTypeChar(label="Byte"), "Sbz1": SimTypeChar(label="Byte"), "AclSize": SimTypeShort(signed=False, label="UInt16"), "AceCount": SimTypeShort(signed=False, label="UInt16"), "Sbz2": SimTypeShort(signed=False, label="UInt16")}, name="ACL", pack=False, align=None), offset=0), SimTypePointer(SimStruct({"AclRevision": SimTypeChar(label="Byte"), "Sbz1": SimTypeChar(label="Byte"), "AclSize": SimTypeShort(signed=False, label="UInt16"), "AceCount": SimTypeShort(signed=False, label="UInt16"), "Sbz2": SimTypeShort(signed=False, label="UInt16")}, name="ACL", pack=False, align=None), offset=0)], SimTypeInt(signed=False, label="UInt32"), arg_names=["engineHandle", "key", "securityInfo", "sidOwner", "sidGroup", "dacl", "sacl"]),
        # 
        'FwpmProviderContextSubscribeChanges0': SimTypeFunction([SimTypePointer(SimTypeInt(signed=True, label="Int"), label="IntPtr", offset=0), SimTypePointer(SimStruct({"enumTemplate": SimTypePointer(SimStruct({"providerKey": SimTypePointer(SimTypeBottom(label="Guid"), offset=0), "providerContextType": SimTypeInt(signed=False, label="FWPM_PROVIDER_CONTEXT_TYPE")}, name="FWPM_PROVIDER_CONTEXT_ENUM_TEMPLATE0", pack=False, align=None), offset=0), "flags": SimTypeInt(signed=False, label="FWPM_SUBSCRIPTION_FLAGS"), "sessionKey": SimTypeBottom(label="Guid")}, name="FWPM_PROVIDER_CONTEXT_SUBSCRIPTION0", pack=False, align=None), offset=0), SimTypePointer(SimTypeFunction([SimTypePointer(SimTypeBottom(label="Void"), offset=0), SimTypePointer(SimStruct({"changeType": SimTypeInt(signed=False, label="FWPM_CHANGE_TYPE"), "providerContextKey": SimTypeBottom(label="Guid"), "providerContextId": SimTypeLongLong(signed=False, label="UInt64")}, name="FWPM_PROVIDER_CONTEXT_CHANGE0", pack=False, align=None), offset=0)], SimTypeBottom(label="Void"), arg_names=["context", "change"]), offset=0), SimTypePointer(SimTypeBottom(label="Void"), offset=0), SimTypePointer(SimTypePointer(SimTypeInt(signed=True, label="Int"), label="IntPtr", offset=0), offset=0)], SimTypeInt(signed=False, label="UInt32"), arg_names=["engineHandle", "subscription", "callback", "context", "changeHandle"]),
        # 
        'FwpmProviderContextUnsubscribeChanges0': SimTypeFunction([SimTypePointer(SimTypeInt(signed=True, label="Int"), label="IntPtr", offset=0), SimTypePointer(SimTypeInt(signed=True, label="Int"), label="IntPtr", offset=0)], SimTypeInt(signed=False, label="UInt32"), arg_names=["engineHandle", "changeHandle"]),
        # 
        'FwpmProviderContextSubscriptionsGet0': SimTypeFunction([SimTypePointer(SimTypeInt(signed=True, label="Int"), label="IntPtr", offset=0), SimTypePointer(SimTypePointer(SimTypePointer(SimStruct({"enumTemplate": SimTypePointer(SimStruct({"providerKey": SimTypePointer(SimTypeBottom(label="Guid"), offset=0), "providerContextType": SimTypeInt(signed=False, label="FWPM_PROVIDER_CONTEXT_TYPE")}, name="FWPM_PROVIDER_CONTEXT_ENUM_TEMPLATE0", pack=False, align=None), offset=0), "flags": SimTypeInt(signed=False, label="FWPM_SUBSCRIPTION_FLAGS"), "sessionKey": SimTypeBottom(label="Guid")}, name="FWPM_PROVIDER_CONTEXT_SUBSCRIPTION0", pack=False, align=None), offset=0), offset=0), offset=0), SimTypePointer(SimTypeInt(signed=False, label="UInt32"), offset=0)], SimTypeInt(signed=False, label="UInt32"), arg_names=["engineHandle", "entries", "numEntries"]),
        # 
        'FwpmSubLayerAdd0': SimTypeFunction([SimTypePointer(SimTypeInt(signed=True, label="Int"), label="IntPtr", offset=0), SimTypePointer(SimStruct({"subLayerKey": SimTypeBottom(label="Guid"), "displayData": SimStruct({"name": SimTypePointer(SimTypeChar(label="Char"), offset=0), "description": SimTypePointer(SimTypeChar(label="Char"), offset=0)}, name="FWPM_DISPLAY_DATA0", pack=False, align=None), "flags": SimTypeInt(signed=False, label="UInt32"), "providerKey": SimTypePointer(SimTypeBottom(label="Guid"), offset=0), "providerData": SimStruct({"size": SimTypeInt(signed=False, label="UInt32"), "data": SimTypePointer(SimTypeChar(label="Byte"), offset=0)}, name="FWP_BYTE_BLOB", pack=False, align=None), "weight": SimTypeShort(signed=False, label="UInt16")}, name="FWPM_SUBLAYER0", pack=False, align=None), offset=0), SimTypePointer(SimStruct({"Revision": SimTypeChar(label="Byte"), "Sbz1": SimTypeChar(label="Byte"), "Control": SimTypeShort(signed=False, label="UInt16"), "Owner": SimTypePointer(SimTypeInt(signed=True, label="Int"), label="IntPtr", offset=0), "Group": SimTypePointer(SimTypeInt(signed=True, label="Int"), label="IntPtr", offset=0), "Sacl": SimTypePointer(SimStruct({"AclRevision": SimTypeChar(label="Byte"), "Sbz1": SimTypeChar(label="Byte"), "AclSize": SimTypeShort(signed=False, label="UInt16"), "AceCount": SimTypeShort(signed=False, label="UInt16"), "Sbz2": SimTypeShort(signed=False, label="UInt16")}, name="ACL", pack=False, align=None), offset=0), "Dacl": SimTypePointer(SimStruct({"AclRevision": SimTypeChar(label="Byte"), "Sbz1": SimTypeChar(label="Byte"), "AclSize": SimTypeShort(signed=False, label="UInt16"), "AceCount": SimTypeShort(signed=False, label="UInt16"), "Sbz2": SimTypeShort(signed=False, label="UInt16")}, name="ACL", pack=False, align=None), offset=0)}, name="SECURITY_DESCRIPTOR", pack=False, align=None), offset=0)], SimTypeInt(signed=False, label="UInt32"), arg_names=["engineHandle", "subLayer", "sd"]),
        # 
        'FwpmSubLayerDeleteByKey0': SimTypeFunction([SimTypePointer(SimTypeInt(signed=True, label="Int"), label="IntPtr", offset=0), SimTypePointer(SimTypeBottom(label="Guid"), offset=0)], SimTypeInt(signed=False, label="UInt32"), arg_names=["engineHandle", "key"]),
        # 
        'FwpmSubLayerGetByKey0': SimTypeFunction([SimTypePointer(SimTypeInt(signed=True, label="Int"), label="IntPtr", offset=0), SimTypePointer(SimTypeBottom(label="Guid"), offset=0), SimTypePointer(SimTypePointer(SimStruct({"subLayerKey": SimTypeBottom(label="Guid"), "displayData": SimStruct({"name": SimTypePointer(SimTypeChar(label="Char"), offset=0), "description": SimTypePointer(SimTypeChar(label="Char"), offset=0)}, name="FWPM_DISPLAY_DATA0", pack=False, align=None), "flags": SimTypeInt(signed=False, label="UInt32"), "providerKey": SimTypePointer(SimTypeBottom(label="Guid"), offset=0), "providerData": SimStruct({"size": SimTypeInt(signed=False, label="UInt32"), "data": SimTypePointer(SimTypeChar(label="Byte"), offset=0)}, name="FWP_BYTE_BLOB", pack=False, align=None), "weight": SimTypeShort(signed=False, label="UInt16")}, name="FWPM_SUBLAYER0", pack=False, align=None), offset=0), offset=0)], SimTypeInt(signed=False, label="UInt32"), arg_names=["engineHandle", "key", "subLayer"]),
        # 
        'FwpmSubLayerCreateEnumHandle0': SimTypeFunction([SimTypePointer(SimTypeInt(signed=True, label="Int"), label="IntPtr", offset=0), SimTypePointer(SimStruct({"providerKey": SimTypePointer(SimTypeBottom(label="Guid"), offset=0)}, name="FWPM_SUBLAYER_ENUM_TEMPLATE0", pack=False, align=None), offset=0), SimTypePointer(SimTypePointer(SimTypeInt(signed=True, label="Int"), label="IntPtr", offset=0), offset=0)], SimTypeInt(signed=False, label="UInt32"), arg_names=["engineHandle", "enumTemplate", "enumHandle"]),
        # 
        'FwpmSubLayerEnum0': SimTypeFunction([SimTypePointer(SimTypeInt(signed=True, label="Int"), label="IntPtr", offset=0), SimTypePointer(SimTypeInt(signed=True, label="Int"), label="IntPtr", offset=0), SimTypeInt(signed=False, label="UInt32"), SimTypePointer(SimTypePointer(SimTypePointer(SimStruct({"subLayerKey": SimTypeBottom(label="Guid"), "displayData": SimStruct({"name": SimTypePointer(SimTypeChar(label="Char"), offset=0), "description": SimTypePointer(SimTypeChar(label="Char"), offset=0)}, name="FWPM_DISPLAY_DATA0", pack=False, align=None), "flags": SimTypeInt(signed=False, label="UInt32"), "providerKey": SimTypePointer(SimTypeBottom(label="Guid"), offset=0), "providerData": SimStruct({"size": SimTypeInt(signed=False, label="UInt32"), "data": SimTypePointer(SimTypeChar(label="Byte"), offset=0)}, name="FWP_BYTE_BLOB", pack=False, align=None), "weight": SimTypeShort(signed=False, label="UInt16")}, name="FWPM_SUBLAYER0", pack=False, align=None), offset=0), offset=0), offset=0), SimTypePointer(SimTypeInt(signed=False, label="UInt32"), offset=0)], SimTypeInt(signed=False, label="UInt32"), arg_names=["engineHandle", "enumHandle", "numEntriesRequested", "entries", "numEntriesReturned"]),
        # 
        'FwpmSubLayerDestroyEnumHandle0': SimTypeFunction([SimTypePointer(SimTypeInt(signed=True, label="Int"), label="IntPtr", offset=0), SimTypePointer(SimTypeInt(signed=True, label="Int"), label="IntPtr", offset=0)], SimTypeInt(signed=False, label="UInt32"), arg_names=["engineHandle", "enumHandle"]),
        # 
        'FwpmSubLayerGetSecurityInfoByKey0': SimTypeFunction([SimTypePointer(SimTypeInt(signed=True, label="Int"), label="IntPtr", offset=0), SimTypePointer(SimTypeBottom(label="Guid"), offset=0), SimTypeInt(signed=False, label="UInt32"), SimTypePointer(SimTypePointer(SimTypeInt(signed=True, label="Int"), label="IntPtr", offset=0), offset=0), SimTypePointer(SimTypePointer(SimTypeInt(signed=True, label="Int"), label="IntPtr", offset=0), offset=0), SimTypePointer(SimTypePointer(SimStruct({"AclRevision": SimTypeChar(label="Byte"), "Sbz1": SimTypeChar(label="Byte"), "AclSize": SimTypeShort(signed=False, label="UInt16"), "AceCount": SimTypeShort(signed=False, label="UInt16"), "Sbz2": SimTypeShort(signed=False, label="UInt16")}, name="ACL", pack=False, align=None), offset=0), offset=0), SimTypePointer(SimTypePointer(SimStruct({"AclRevision": SimTypeChar(label="Byte"), "Sbz1": SimTypeChar(label="Byte"), "AclSize": SimTypeShort(signed=False, label="UInt16"), "AceCount": SimTypeShort(signed=False, label="UInt16"), "Sbz2": SimTypeShort(signed=False, label="UInt16")}, name="ACL", pack=False, align=None), offset=0), offset=0), SimTypePointer(SimTypePointer(SimStruct({"Revision": SimTypeChar(label="Byte"), "Sbz1": SimTypeChar(label="Byte"), "Control": SimTypeShort(signed=False, label="UInt16"), "Owner": SimTypePointer(SimTypeInt(signed=True, label="Int"), label="IntPtr", offset=0), "Group": SimTypePointer(SimTypeInt(signed=True, label="Int"), label="IntPtr", offset=0), "Sacl": SimTypePointer(SimStruct({"AclRevision": SimTypeChar(label="Byte"), "Sbz1": SimTypeChar(label="Byte"), "AclSize": SimTypeShort(signed=False, label="UInt16"), "AceCount": SimTypeShort(signed=False, label="UInt16"), "Sbz2": SimTypeShort(signed=False, label="UInt16")}, name="ACL", pack=False, align=None), offset=0), "Dacl": SimTypePointer(SimStruct({"AclRevision": SimTypeChar(label="Byte"), "Sbz1": SimTypeChar(label="Byte"), "AclSize": SimTypeShort(signed=False, label="UInt16"), "AceCount": SimTypeShort(signed=False, label="UInt16"), "Sbz2": SimTypeShort(signed=False, label="UInt16")}, name="ACL", pack=False, align=None), offset=0)}, name="SECURITY_DESCRIPTOR", pack=False, align=None), offset=0), offset=0)], SimTypeInt(signed=False, label="UInt32"), arg_names=["engineHandle", "key", "securityInfo", "sidOwner", "sidGroup", "dacl", "sacl", "securityDescriptor"]),
        # 
        'FwpmSubLayerSetSecurityInfoByKey0': SimTypeFunction([SimTypePointer(SimTypeInt(signed=True, label="Int"), label="IntPtr", offset=0), SimTypePointer(SimTypeBottom(label="Guid"), offset=0), SimTypeInt(signed=False, label="UInt32"), SimTypePointer(SimStruct({"Revision": SimTypeChar(label="Byte"), "SubAuthorityCount": SimTypeChar(label="Byte"), "IdentifierAuthority": SimStruct({"Value": SimTypeFixedSizeArray(SimTypeChar(label="Byte"), 6)}, name="SID_IDENTIFIER_AUTHORITY", pack=False, align=None), "SubAuthority": SimTypePointer(SimTypeInt(signed=False, label="UInt32"), offset=0)}, name="SID", pack=False, align=None), offset=0), SimTypePointer(SimStruct({"Revision": SimTypeChar(label="Byte"), "SubAuthorityCount": SimTypeChar(label="Byte"), "IdentifierAuthority": SimStruct({"Value": SimTypeFixedSizeArray(SimTypeChar(label="Byte"), 6)}, name="SID_IDENTIFIER_AUTHORITY", pack=False, align=None), "SubAuthority": SimTypePointer(SimTypeInt(signed=False, label="UInt32"), offset=0)}, name="SID", pack=False, align=None), offset=0), SimTypePointer(SimStruct({"AclRevision": SimTypeChar(label="Byte"), "Sbz1": SimTypeChar(label="Byte"), "AclSize": SimTypeShort(signed=False, label="UInt16"), "AceCount": SimTypeShort(signed=False, label="UInt16"), "Sbz2": SimTypeShort(signed=False, label="UInt16")}, name="ACL", pack=False, align=None), offset=0), SimTypePointer(SimStruct({"AclRevision": SimTypeChar(label="Byte"), "Sbz1": SimTypeChar(label="Byte"), "AclSize": SimTypeShort(signed=False, label="UInt16"), "AceCount": SimTypeShort(signed=False, label="UInt16"), "Sbz2": SimTypeShort(signed=False, label="UInt16")}, name="ACL", pack=False, align=None), offset=0)], SimTypeInt(signed=False, label="UInt32"), arg_names=["engineHandle", "key", "securityInfo", "sidOwner", "sidGroup", "dacl", "sacl"]),
        # 
        'FwpmSubLayerSubscribeChanges0': SimTypeFunction([SimTypePointer(SimTypeInt(signed=True, label="Int"), label="IntPtr", offset=0), SimTypePointer(SimStruct({"enumTemplate": SimTypePointer(SimStruct({"providerKey": SimTypePointer(SimTypeBottom(label="Guid"), offset=0)}, name="FWPM_SUBLAYER_ENUM_TEMPLATE0", pack=False, align=None), offset=0), "flags": SimTypeInt(signed=False, label="FWPM_SUBSCRIPTION_FLAGS"), "sessionKey": SimTypeBottom(label="Guid")}, name="FWPM_SUBLAYER_SUBSCRIPTION0", pack=False, align=None), offset=0), SimTypePointer(SimTypeFunction([SimTypePointer(SimTypeBottom(label="Void"), offset=0), SimTypePointer(SimStruct({"changeType": SimTypeInt(signed=False, label="FWPM_CHANGE_TYPE"), "subLayerKey": SimTypeBottom(label="Guid")}, name="FWPM_SUBLAYER_CHANGE0", pack=False, align=None), offset=0)], SimTypeBottom(label="Void"), arg_names=["context", "change"]), offset=0), SimTypePointer(SimTypeBottom(label="Void"), offset=0), SimTypePointer(SimTypePointer(SimTypeInt(signed=True, label="Int"), label="IntPtr", offset=0), offset=0)], SimTypeInt(signed=False, label="UInt32"), arg_names=["engineHandle", "subscription", "callback", "context", "changeHandle"]),
        # 
        'FwpmSubLayerUnsubscribeChanges0': SimTypeFunction([SimTypePointer(SimTypeInt(signed=True, label="Int"), label="IntPtr", offset=0), SimTypePointer(SimTypeInt(signed=True, label="Int"), label="IntPtr", offset=0)], SimTypeInt(signed=False, label="UInt32"), arg_names=["engineHandle", "changeHandle"]),
        # 
        'FwpmSubLayerSubscriptionsGet0': SimTypeFunction([SimTypePointer(SimTypeInt(signed=True, label="Int"), label="IntPtr", offset=0), SimTypePointer(SimTypePointer(SimTypePointer(SimStruct({"enumTemplate": SimTypePointer(SimStruct({"providerKey": SimTypePointer(SimTypeBottom(label="Guid"), offset=0)}, name="FWPM_SUBLAYER_ENUM_TEMPLATE0", pack=False, align=None), offset=0), "flags": SimTypeInt(signed=False, label="FWPM_SUBSCRIPTION_FLAGS"), "sessionKey": SimTypeBottom(label="Guid")}, name="FWPM_SUBLAYER_SUBSCRIPTION0", pack=False, align=None), offset=0), offset=0), offset=0), SimTypePointer(SimTypeInt(signed=False, label="UInt32"), offset=0)], SimTypeInt(signed=False, label="UInt32"), arg_names=["engineHandle", "entries", "numEntries"]),
        # 
        'FwpmLayerGetById0': SimTypeFunction([SimTypePointer(SimTypeInt(signed=True, label="Int"), label="IntPtr", offset=0), SimTypeShort(signed=False, label="UInt16"), SimTypePointer(SimTypePointer(SimStruct({"layerKey": SimTypeBottom(label="Guid"), "displayData": SimStruct({"name": SimTypePointer(SimTypeChar(label="Char"), offset=0), "description": SimTypePointer(SimTypeChar(label="Char"), offset=0)}, name="FWPM_DISPLAY_DATA0", pack=False, align=None), "flags": SimTypeInt(signed=False, label="UInt32"), "numFields": SimTypeInt(signed=False, label="UInt32"), "field": SimTypePointer(SimStruct({"fieldKey": SimTypePointer(SimTypeBottom(label="Guid"), offset=0), "type": SimTypeInt(signed=False, label="FWPM_FIELD_TYPE"), "dataType": SimTypeInt(signed=False, label="FWP_DATA_TYPE")}, name="FWPM_FIELD0", pack=False, align=None), offset=0), "defaultSubLayerKey": SimTypeBottom(label="Guid"), "layerId": SimTypeShort(signed=False, label="UInt16")}, name="FWPM_LAYER0", pack=False, align=None), offset=0), offset=0)], SimTypeInt(signed=False, label="UInt32"), arg_names=["engineHandle", "id", "layer"]),
        # 
        'FwpmLayerGetByKey0': SimTypeFunction([SimTypePointer(SimTypeInt(signed=True, label="Int"), label="IntPtr", offset=0), SimTypePointer(SimTypeBottom(label="Guid"), offset=0), SimTypePointer(SimTypePointer(SimStruct({"layerKey": SimTypeBottom(label="Guid"), "displayData": SimStruct({"name": SimTypePointer(SimTypeChar(label="Char"), offset=0), "description": SimTypePointer(SimTypeChar(label="Char"), offset=0)}, name="FWPM_DISPLAY_DATA0", pack=False, align=None), "flags": SimTypeInt(signed=False, label="UInt32"), "numFields": SimTypeInt(signed=False, label="UInt32"), "field": SimTypePointer(SimStruct({"fieldKey": SimTypePointer(SimTypeBottom(label="Guid"), offset=0), "type": SimTypeInt(signed=False, label="FWPM_FIELD_TYPE"), "dataType": SimTypeInt(signed=False, label="FWP_DATA_TYPE")}, name="FWPM_FIELD0", pack=False, align=None), offset=0), "defaultSubLayerKey": SimTypeBottom(label="Guid"), "layerId": SimTypeShort(signed=False, label="UInt16")}, name="FWPM_LAYER0", pack=False, align=None), offset=0), offset=0)], SimTypeInt(signed=False, label="UInt32"), arg_names=["engineHandle", "key", "layer"]),
        # 
        'FwpmLayerCreateEnumHandle0': SimTypeFunction([SimTypePointer(SimTypeInt(signed=True, label="Int"), label="IntPtr", offset=0), SimTypePointer(SimStruct({"reserved": SimTypeLongLong(signed=False, label="UInt64")}, name="FWPM_LAYER_ENUM_TEMPLATE0", pack=False, align=None), offset=0), SimTypePointer(SimTypePointer(SimTypeInt(signed=True, label="Int"), label="IntPtr", offset=0), offset=0)], SimTypeInt(signed=False, label="UInt32"), arg_names=["engineHandle", "enumTemplate", "enumHandle"]),
        # 
        'FwpmLayerEnum0': SimTypeFunction([SimTypePointer(SimTypeInt(signed=True, label="Int"), label="IntPtr", offset=0), SimTypePointer(SimTypeInt(signed=True, label="Int"), label="IntPtr", offset=0), SimTypeInt(signed=False, label="UInt32"), SimTypePointer(SimTypePointer(SimTypePointer(SimStruct({"layerKey": SimTypeBottom(label="Guid"), "displayData": SimStruct({"name": SimTypePointer(SimTypeChar(label="Char"), offset=0), "description": SimTypePointer(SimTypeChar(label="Char"), offset=0)}, name="FWPM_DISPLAY_DATA0", pack=False, align=None), "flags": SimTypeInt(signed=False, label="UInt32"), "numFields": SimTypeInt(signed=False, label="UInt32"), "field": SimTypePointer(SimStruct({"fieldKey": SimTypePointer(SimTypeBottom(label="Guid"), offset=0), "type": SimTypeInt(signed=False, label="FWPM_FIELD_TYPE"), "dataType": SimTypeInt(signed=False, label="FWP_DATA_TYPE")}, name="FWPM_FIELD0", pack=False, align=None), offset=0), "defaultSubLayerKey": SimTypeBottom(label="Guid"), "layerId": SimTypeShort(signed=False, label="UInt16")}, name="FWPM_LAYER0", pack=False, align=None), offset=0), offset=0), offset=0), SimTypePointer(SimTypeInt(signed=False, label="UInt32"), offset=0)], SimTypeInt(signed=False, label="UInt32"), arg_names=["engineHandle", "enumHandle", "numEntriesRequested", "entries", "numEntriesReturned"]),
        # 
        'FwpmLayerDestroyEnumHandle0': SimTypeFunction([SimTypePointer(SimTypeInt(signed=True, label="Int"), label="IntPtr", offset=0), SimTypePointer(SimTypeInt(signed=True, label="Int"), label="IntPtr", offset=0)], SimTypeInt(signed=False, label="UInt32"), arg_names=["engineHandle", "enumHandle"]),
        # 
        'FwpmLayerGetSecurityInfoByKey0': SimTypeFunction([SimTypePointer(SimTypeInt(signed=True, label="Int"), label="IntPtr", offset=0), SimTypePointer(SimTypeBottom(label="Guid"), offset=0), SimTypeInt(signed=False, label="UInt32"), SimTypePointer(SimTypePointer(SimTypeInt(signed=True, label="Int"), label="IntPtr", offset=0), offset=0), SimTypePointer(SimTypePointer(SimTypeInt(signed=True, label="Int"), label="IntPtr", offset=0), offset=0), SimTypePointer(SimTypePointer(SimStruct({"AclRevision": SimTypeChar(label="Byte"), "Sbz1": SimTypeChar(label="Byte"), "AclSize": SimTypeShort(signed=False, label="UInt16"), "AceCount": SimTypeShort(signed=False, label="UInt16"), "Sbz2": SimTypeShort(signed=False, label="UInt16")}, name="ACL", pack=False, align=None), offset=0), offset=0), SimTypePointer(SimTypePointer(SimStruct({"AclRevision": SimTypeChar(label="Byte"), "Sbz1": SimTypeChar(label="Byte"), "AclSize": SimTypeShort(signed=False, label="UInt16"), "AceCount": SimTypeShort(signed=False, label="UInt16"), "Sbz2": SimTypeShort(signed=False, label="UInt16")}, name="ACL", pack=False, align=None), offset=0), offset=0), SimTypePointer(SimTypePointer(SimStruct({"Revision": SimTypeChar(label="Byte"), "Sbz1": SimTypeChar(label="Byte"), "Control": SimTypeShort(signed=False, label="UInt16"), "Owner": SimTypePointer(SimTypeInt(signed=True, label="Int"), label="IntPtr", offset=0), "Group": SimTypePointer(SimTypeInt(signed=True, label="Int"), label="IntPtr", offset=0), "Sacl": SimTypePointer(SimStruct({"AclRevision": SimTypeChar(label="Byte"), "Sbz1": SimTypeChar(label="Byte"), "AclSize": SimTypeShort(signed=False, label="UInt16"), "AceCount": SimTypeShort(signed=False, label="UInt16"), "Sbz2": SimTypeShort(signed=False, label="UInt16")}, name="ACL", pack=False, align=None), offset=0), "Dacl": SimTypePointer(SimStruct({"AclRevision": SimTypeChar(label="Byte"), "Sbz1": SimTypeChar(label="Byte"), "AclSize": SimTypeShort(signed=False, label="UInt16"), "AceCount": SimTypeShort(signed=False, label="UInt16"), "Sbz2": SimTypeShort(signed=False, label="UInt16")}, name="ACL", pack=False, align=None), offset=0)}, name="SECURITY_DESCRIPTOR", pack=False, align=None), offset=0), offset=0)], SimTypeInt(signed=False, label="UInt32"), arg_names=["engineHandle", "key", "securityInfo", "sidOwner", "sidGroup", "dacl", "sacl", "securityDescriptor"]),
        # 
        'FwpmLayerSetSecurityInfoByKey0': SimTypeFunction([SimTypePointer(SimTypeInt(signed=True, label="Int"), label="IntPtr", offset=0), SimTypePointer(SimTypeBottom(label="Guid"), offset=0), SimTypeInt(signed=False, label="UInt32"), SimTypePointer(SimStruct({"Revision": SimTypeChar(label="Byte"), "SubAuthorityCount": SimTypeChar(label="Byte"), "IdentifierAuthority": SimStruct({"Value": SimTypeFixedSizeArray(SimTypeChar(label="Byte"), 6)}, name="SID_IDENTIFIER_AUTHORITY", pack=False, align=None), "SubAuthority": SimTypePointer(SimTypeInt(signed=False, label="UInt32"), offset=0)}, name="SID", pack=False, align=None), offset=0), SimTypePointer(SimStruct({"Revision": SimTypeChar(label="Byte"), "SubAuthorityCount": SimTypeChar(label="Byte"), "IdentifierAuthority": SimStruct({"Value": SimTypeFixedSizeArray(SimTypeChar(label="Byte"), 6)}, name="SID_IDENTIFIER_AUTHORITY", pack=False, align=None), "SubAuthority": SimTypePointer(SimTypeInt(signed=False, label="UInt32"), offset=0)}, name="SID", pack=False, align=None), offset=0), SimTypePointer(SimStruct({"AclRevision": SimTypeChar(label="Byte"), "Sbz1": SimTypeChar(label="Byte"), "AclSize": SimTypeShort(signed=False, label="UInt16"), "AceCount": SimTypeShort(signed=False, label="UInt16"), "Sbz2": SimTypeShort(signed=False, label="UInt16")}, name="ACL", pack=False, align=None), offset=0), SimTypePointer(SimStruct({"AclRevision": SimTypeChar(label="Byte"), "Sbz1": SimTypeChar(label="Byte"), "AclSize": SimTypeShort(signed=False, label="UInt16"), "AceCount": SimTypeShort(signed=False, label="UInt16"), "Sbz2": SimTypeShort(signed=False, label="UInt16")}, name="ACL", pack=False, align=None), offset=0)], SimTypeInt(signed=False, label="UInt32"), arg_names=["engineHandle", "key", "securityInfo", "sidOwner", "sidGroup", "dacl", "sacl"]),
        # 
        'FwpmCalloutAdd0': SimTypeFunction([SimTypePointer(SimTypeInt(signed=True, label="Int"), label="IntPtr", offset=0), SimTypePointer(SimStruct({"calloutKey": SimTypeBottom(label="Guid"), "displayData": SimStruct({"name": SimTypePointer(SimTypeChar(label="Char"), offset=0), "description": SimTypePointer(SimTypeChar(label="Char"), offset=0)}, name="FWPM_DISPLAY_DATA0", pack=False, align=None), "flags": SimTypeInt(signed=False, label="UInt32"), "providerKey": SimTypePointer(SimTypeBottom(label="Guid"), offset=0), "providerData": SimStruct({"size": SimTypeInt(signed=False, label="UInt32"), "data": SimTypePointer(SimTypeChar(label="Byte"), offset=0)}, name="FWP_BYTE_BLOB", pack=False, align=None), "applicableLayer": SimTypeBottom(label="Guid"), "calloutId": SimTypeInt(signed=False, label="UInt32")}, name="FWPM_CALLOUT0", pack=False, align=None), offset=0), SimTypePointer(SimStruct({"Revision": SimTypeChar(label="Byte"), "Sbz1": SimTypeChar(label="Byte"), "Control": SimTypeShort(signed=False, label="UInt16"), "Owner": SimTypePointer(SimTypeInt(signed=True, label="Int"), label="IntPtr", offset=0), "Group": SimTypePointer(SimTypeInt(signed=True, label="Int"), label="IntPtr", offset=0), "Sacl": SimTypePointer(SimStruct({"AclRevision": SimTypeChar(label="Byte"), "Sbz1": SimTypeChar(label="Byte"), "AclSize": SimTypeShort(signed=False, label="UInt16"), "AceCount": SimTypeShort(signed=False, label="UInt16"), "Sbz2": SimTypeShort(signed=False, label="UInt16")}, name="ACL", pack=False, align=None), offset=0), "Dacl": SimTypePointer(SimStruct({"AclRevision": SimTypeChar(label="Byte"), "Sbz1": SimTypeChar(label="Byte"), "AclSize": SimTypeShort(signed=False, label="UInt16"), "AceCount": SimTypeShort(signed=False, label="UInt16"), "Sbz2": SimTypeShort(signed=False, label="UInt16")}, name="ACL", pack=False, align=None), offset=0)}, name="SECURITY_DESCRIPTOR", pack=False, align=None), offset=0), SimTypePointer(SimTypeInt(signed=False, label="UInt32"), offset=0)], SimTypeInt(signed=False, label="UInt32"), arg_names=["engineHandle", "callout", "sd", "id"]),
        # 
        'FwpmCalloutDeleteById0': SimTypeFunction([SimTypePointer(SimTypeInt(signed=True, label="Int"), label="IntPtr", offset=0), SimTypeInt(signed=False, label="UInt32")], SimTypeInt(signed=False, label="UInt32"), arg_names=["engineHandle", "id"]),
        # 
        'FwpmCalloutDeleteByKey0': SimTypeFunction([SimTypePointer(SimTypeInt(signed=True, label="Int"), label="IntPtr", offset=0), SimTypePointer(SimTypeBottom(label="Guid"), offset=0)], SimTypeInt(signed=False, label="UInt32"), arg_names=["engineHandle", "key"]),
        # 
        'FwpmCalloutGetById0': SimTypeFunction([SimTypePointer(SimTypeInt(signed=True, label="Int"), label="IntPtr", offset=0), SimTypeInt(signed=False, label="UInt32"), SimTypePointer(SimTypePointer(SimStruct({"calloutKey": SimTypeBottom(label="Guid"), "displayData": SimStruct({"name": SimTypePointer(SimTypeChar(label="Char"), offset=0), "description": SimTypePointer(SimTypeChar(label="Char"), offset=0)}, name="FWPM_DISPLAY_DATA0", pack=False, align=None), "flags": SimTypeInt(signed=False, label="UInt32"), "providerKey": SimTypePointer(SimTypeBottom(label="Guid"), offset=0), "providerData": SimStruct({"size": SimTypeInt(signed=False, label="UInt32"), "data": SimTypePointer(SimTypeChar(label="Byte"), offset=0)}, name="FWP_BYTE_BLOB", pack=False, align=None), "applicableLayer": SimTypeBottom(label="Guid"), "calloutId": SimTypeInt(signed=False, label="UInt32")}, name="FWPM_CALLOUT0", pack=False, align=None), offset=0), offset=0)], SimTypeInt(signed=False, label="UInt32"), arg_names=["engineHandle", "id", "callout"]),
        # 
        'FwpmCalloutGetByKey0': SimTypeFunction([SimTypePointer(SimTypeInt(signed=True, label="Int"), label="IntPtr", offset=0), SimTypePointer(SimTypeBottom(label="Guid"), offset=0), SimTypePointer(SimTypePointer(SimStruct({"calloutKey": SimTypeBottom(label="Guid"), "displayData": SimStruct({"name": SimTypePointer(SimTypeChar(label="Char"), offset=0), "description": SimTypePointer(SimTypeChar(label="Char"), offset=0)}, name="FWPM_DISPLAY_DATA0", pack=False, align=None), "flags": SimTypeInt(signed=False, label="UInt32"), "providerKey": SimTypePointer(SimTypeBottom(label="Guid"), offset=0), "providerData": SimStruct({"size": SimTypeInt(signed=False, label="UInt32"), "data": SimTypePointer(SimTypeChar(label="Byte"), offset=0)}, name="FWP_BYTE_BLOB", pack=False, align=None), "applicableLayer": SimTypeBottom(label="Guid"), "calloutId": SimTypeInt(signed=False, label="UInt32")}, name="FWPM_CALLOUT0", pack=False, align=None), offset=0), offset=0)], SimTypeInt(signed=False, label="UInt32"), arg_names=["engineHandle", "key", "callout"]),
        # 
        'FwpmCalloutCreateEnumHandle0': SimTypeFunction([SimTypePointer(SimTypeInt(signed=True, label="Int"), label="IntPtr", offset=0), SimTypePointer(SimStruct({"providerKey": SimTypePointer(SimTypeBottom(label="Guid"), offset=0), "layerKey": SimTypeBottom(label="Guid")}, name="FWPM_CALLOUT_ENUM_TEMPLATE0", pack=False, align=None), offset=0), SimTypePointer(SimTypePointer(SimTypeInt(signed=True, label="Int"), label="IntPtr", offset=0), offset=0)], SimTypeInt(signed=False, label="UInt32"), arg_names=["engineHandle", "enumTemplate", "enumHandle"]),
        # 
        'FwpmCalloutEnum0': SimTypeFunction([SimTypePointer(SimTypeInt(signed=True, label="Int"), label="IntPtr", offset=0), SimTypePointer(SimTypeInt(signed=True, label="Int"), label="IntPtr", offset=0), SimTypeInt(signed=False, label="UInt32"), SimTypePointer(SimTypePointer(SimTypePointer(SimStruct({"calloutKey": SimTypeBottom(label="Guid"), "displayData": SimStruct({"name": SimTypePointer(SimTypeChar(label="Char"), offset=0), "description": SimTypePointer(SimTypeChar(label="Char"), offset=0)}, name="FWPM_DISPLAY_DATA0", pack=False, align=None), "flags": SimTypeInt(signed=False, label="UInt32"), "providerKey": SimTypePointer(SimTypeBottom(label="Guid"), offset=0), "providerData": SimStruct({"size": SimTypeInt(signed=False, label="UInt32"), "data": SimTypePointer(SimTypeChar(label="Byte"), offset=0)}, name="FWP_BYTE_BLOB", pack=False, align=None), "applicableLayer": SimTypeBottom(label="Guid"), "calloutId": SimTypeInt(signed=False, label="UInt32")}, name="FWPM_CALLOUT0", pack=False, align=None), offset=0), offset=0), offset=0), SimTypePointer(SimTypeInt(signed=False, label="UInt32"), offset=0)], SimTypeInt(signed=False, label="UInt32"), arg_names=["engineHandle", "enumHandle", "numEntriesRequested", "entries", "numEntriesReturned"]),
        # 
        'FwpmCalloutDestroyEnumHandle0': SimTypeFunction([SimTypePointer(SimTypeInt(signed=True, label="Int"), label="IntPtr", offset=0), SimTypePointer(SimTypeInt(signed=True, label="Int"), label="IntPtr", offset=0)], SimTypeInt(signed=False, label="UInt32"), arg_names=["engineHandle", "enumHandle"]),
        # 
        'FwpmCalloutGetSecurityInfoByKey0': SimTypeFunction([SimTypePointer(SimTypeInt(signed=True, label="Int"), label="IntPtr", offset=0), SimTypePointer(SimTypeBottom(label="Guid"), offset=0), SimTypeInt(signed=False, label="UInt32"), SimTypePointer(SimTypePointer(SimTypeInt(signed=True, label="Int"), label="IntPtr", offset=0), offset=0), SimTypePointer(SimTypePointer(SimTypeInt(signed=True, label="Int"), label="IntPtr", offset=0), offset=0), SimTypePointer(SimTypePointer(SimStruct({"AclRevision": SimTypeChar(label="Byte"), "Sbz1": SimTypeChar(label="Byte"), "AclSize": SimTypeShort(signed=False, label="UInt16"), "AceCount": SimTypeShort(signed=False, label="UInt16"), "Sbz2": SimTypeShort(signed=False, label="UInt16")}, name="ACL", pack=False, align=None), offset=0), offset=0), SimTypePointer(SimTypePointer(SimStruct({"AclRevision": SimTypeChar(label="Byte"), "Sbz1": SimTypeChar(label="Byte"), "AclSize": SimTypeShort(signed=False, label="UInt16"), "AceCount": SimTypeShort(signed=False, label="UInt16"), "Sbz2": SimTypeShort(signed=False, label="UInt16")}, name="ACL", pack=False, align=None), offset=0), offset=0), SimTypePointer(SimTypePointer(SimStruct({"Revision": SimTypeChar(label="Byte"), "Sbz1": SimTypeChar(label="Byte"), "Control": SimTypeShort(signed=False, label="UInt16"), "Owner": SimTypePointer(SimTypeInt(signed=True, label="Int"), label="IntPtr", offset=0), "Group": SimTypePointer(SimTypeInt(signed=True, label="Int"), label="IntPtr", offset=0), "Sacl": SimTypePointer(SimStruct({"AclRevision": SimTypeChar(label="Byte"), "Sbz1": SimTypeChar(label="Byte"), "AclSize": SimTypeShort(signed=False, label="UInt16"), "AceCount": SimTypeShort(signed=False, label="UInt16"), "Sbz2": SimTypeShort(signed=False, label="UInt16")}, name="ACL", pack=False, align=None), offset=0), "Dacl": SimTypePointer(SimStruct({"AclRevision": SimTypeChar(label="Byte"), "Sbz1": SimTypeChar(label="Byte"), "AclSize": SimTypeShort(signed=False, label="UInt16"), "AceCount": SimTypeShort(signed=False, label="UInt16"), "Sbz2": SimTypeShort(signed=False, label="UInt16")}, name="ACL", pack=False, align=None), offset=0)}, name="SECURITY_DESCRIPTOR", pack=False, align=None), offset=0), offset=0)], SimTypeInt(signed=False, label="UInt32"), arg_names=["engineHandle", "key", "securityInfo", "sidOwner", "sidGroup", "dacl", "sacl", "securityDescriptor"]),
        # 
        'FwpmCalloutSetSecurityInfoByKey0': SimTypeFunction([SimTypePointer(SimTypeInt(signed=True, label="Int"), label="IntPtr", offset=0), SimTypePointer(SimTypeBottom(label="Guid"), offset=0), SimTypeInt(signed=False, label="UInt32"), SimTypePointer(SimStruct({"Revision": SimTypeChar(label="Byte"), "SubAuthorityCount": SimTypeChar(label="Byte"), "IdentifierAuthority": SimStruct({"Value": SimTypeFixedSizeArray(SimTypeChar(label="Byte"), 6)}, name="SID_IDENTIFIER_AUTHORITY", pack=False, align=None), "SubAuthority": SimTypePointer(SimTypeInt(signed=False, label="UInt32"), offset=0)}, name="SID", pack=False, align=None), offset=0), SimTypePointer(SimStruct({"Revision": SimTypeChar(label="Byte"), "SubAuthorityCount": SimTypeChar(label="Byte"), "IdentifierAuthority": SimStruct({"Value": SimTypeFixedSizeArray(SimTypeChar(label="Byte"), 6)}, name="SID_IDENTIFIER_AUTHORITY", pack=False, align=None), "SubAuthority": SimTypePointer(SimTypeInt(signed=False, label="UInt32"), offset=0)}, name="SID", pack=False, align=None), offset=0), SimTypePointer(SimStruct({"AclRevision": SimTypeChar(label="Byte"), "Sbz1": SimTypeChar(label="Byte"), "AclSize": SimTypeShort(signed=False, label="UInt16"), "AceCount": SimTypeShort(signed=False, label="UInt16"), "Sbz2": SimTypeShort(signed=False, label="UInt16")}, name="ACL", pack=False, align=None), offset=0), SimTypePointer(SimStruct({"AclRevision": SimTypeChar(label="Byte"), "Sbz1": SimTypeChar(label="Byte"), "AclSize": SimTypeShort(signed=False, label="UInt16"), "AceCount": SimTypeShort(signed=False, label="UInt16"), "Sbz2": SimTypeShort(signed=False, label="UInt16")}, name="ACL", pack=False, align=None), offset=0)], SimTypeInt(signed=False, label="UInt32"), arg_names=["engineHandle", "key", "securityInfo", "sidOwner", "sidGroup", "dacl", "sacl"]),
        # 
        'FwpmCalloutSubscribeChanges0': SimTypeFunction([SimTypePointer(SimTypeInt(signed=True, label="Int"), label="IntPtr", offset=0), SimTypePointer(SimStruct({"enumTemplate": SimTypePointer(SimStruct({"providerKey": SimTypePointer(SimTypeBottom(label="Guid"), offset=0), "layerKey": SimTypeBottom(label="Guid")}, name="FWPM_CALLOUT_ENUM_TEMPLATE0", pack=False, align=None), offset=0), "flags": SimTypeInt(signed=False, label="UInt32"), "sessionKey": SimTypeBottom(label="Guid")}, name="FWPM_CALLOUT_SUBSCRIPTION0", pack=False, align=None), offset=0), SimTypePointer(SimTypeFunction([SimTypePointer(SimTypeBottom(label="Void"), offset=0), SimTypePointer(SimStruct({"changeType": SimTypeInt(signed=False, label="FWPM_CHANGE_TYPE"), "calloutKey": SimTypeBottom(label="Guid"), "calloutId": SimTypeInt(signed=False, label="UInt32")}, name="FWPM_CALLOUT_CHANGE0", pack=False, align=None), offset=0)], SimTypeBottom(label="Void"), arg_names=["context", "change"]), offset=0), SimTypePointer(SimTypeBottom(label="Void"), offset=0), SimTypePointer(SimTypePointer(SimTypeInt(signed=True, label="Int"), label="IntPtr", offset=0), offset=0)], SimTypeInt(signed=False, label="UInt32"), arg_names=["engineHandle", "subscription", "callback", "context", "changeHandle"]),
        # 
        'FwpmCalloutUnsubscribeChanges0': SimTypeFunction([SimTypePointer(SimTypeInt(signed=True, label="Int"), label="IntPtr", offset=0), SimTypePointer(SimTypeInt(signed=True, label="Int"), label="IntPtr", offset=0)], SimTypeInt(signed=False, label="UInt32"), arg_names=["engineHandle", "changeHandle"]),
        # 
        'FwpmCalloutSubscriptionsGet0': SimTypeFunction([SimTypePointer(SimTypeInt(signed=True, label="Int"), label="IntPtr", offset=0), SimTypePointer(SimTypePointer(SimTypePointer(SimStruct({"enumTemplate": SimTypePointer(SimStruct({"providerKey": SimTypePointer(SimTypeBottom(label="Guid"), offset=0), "layerKey": SimTypeBottom(label="Guid")}, name="FWPM_CALLOUT_ENUM_TEMPLATE0", pack=False, align=None), offset=0), "flags": SimTypeInt(signed=False, label="UInt32"), "sessionKey": SimTypeBottom(label="Guid")}, name="FWPM_CALLOUT_SUBSCRIPTION0", pack=False, align=None), offset=0), offset=0), offset=0), SimTypePointer(SimTypeInt(signed=False, label="UInt32"), offset=0)], SimTypeInt(signed=False, label="UInt32"), arg_names=["engineHandle", "entries", "numEntries"]),
        # 
        'FwpmFilterAdd0': SimTypeFunction([SimTypePointer(SimTypeInt(signed=True, label="Int"), label="IntPtr", offset=0), SimTypePointer(SimStruct({"filterKey": SimTypeBottom(label="Guid"), "displayData": SimStruct({"name": SimTypePointer(SimTypeChar(label="Char"), offset=0), "description": SimTypePointer(SimTypeChar(label="Char"), offset=0)}, name="FWPM_DISPLAY_DATA0", pack=False, align=None), "flags": SimTypeInt(signed=False, label="FWPM_FILTER_FLAGS"), "providerKey": SimTypePointer(SimTypeBottom(label="Guid"), offset=0), "providerData": SimStruct({"size": SimTypeInt(signed=False, label="UInt32"), "data": SimTypePointer(SimTypeChar(label="Byte"), offset=0)}, name="FWP_BYTE_BLOB", pack=False, align=None), "layerKey": SimTypeBottom(label="Guid"), "subLayerKey": SimTypeBottom(label="Guid"), "weight": SimStruct({"type": SimTypeInt(signed=False, label="FWP_DATA_TYPE"), "Anonymous": SimUnion({"uint8": SimTypeChar(label="Byte"), "uint16": SimTypeShort(signed=False, label="UInt16"), "uint32": SimTypeInt(signed=False, label="UInt32"), "uint64": SimTypePointer(SimTypeLongLong(signed=False, label="UInt64"), offset=0), "int8": SimTypeChar(label="SByte"), "int16": SimTypeShort(signed=True, label="Int16"), "int32": SimTypeInt(signed=True, label="Int32"), "int64": SimTypePointer(SimTypeLongLong(signed=True, label="Int64"), offset=0), "float32": SimTypeFloat(size=32), "double64": SimTypePointer(SimTypeFloat(size=64), offset=0), "byteArray16": SimTypePointer(SimStruct({"byteArray16": SimTypeFixedSizeArray(SimTypeChar(label="Byte"), 16)}, name="FWP_BYTE_ARRAY16", pack=False, align=None), offset=0), "byteBlob": SimTypePointer(SimStruct({"size": SimTypeInt(signed=False, label="UInt32"), "data": SimTypePointer(SimTypeChar(label="Byte"), offset=0)}, name="FWP_BYTE_BLOB", pack=False, align=None), offset=0), "sid": SimTypePointer(SimTypeBottom(label="SID"), offset=0), "sd": SimTypePointer(SimStruct({"size": SimTypeInt(signed=False, label="UInt32"), "data": SimTypePointer(SimTypeChar(label="Byte"), offset=0)}, name="FWP_BYTE_BLOB", pack=False, align=None), offset=0), "tokenInformation": SimTypePointer(SimStruct({"sidCount": SimTypeInt(signed=False, label="UInt32"), "sids": SimTypePointer(SimTypeBottom(label="SID_AND_ATTRIBUTES"), offset=0), "restrictedSidCount": SimTypeInt(signed=False, label="UInt32"), "restrictedSids": SimTypePointer(SimTypeBottom(label="SID_AND_ATTRIBUTES"), offset=0)}, name="FWP_TOKEN_INFORMATION", pack=False, align=None), offset=0), "tokenAccessInformation": SimTypePointer(SimStruct({"size": SimTypeInt(signed=False, label="UInt32"), "data": SimTypePointer(SimTypeChar(label="Byte"), offset=0)}, name="FWP_BYTE_BLOB", pack=False, align=None), offset=0), "unicodeString": SimTypePointer(SimTypeChar(label="Char"), offset=0), "byteArray6": SimTypePointer(SimStruct({"byteArray6": SimTypeFixedSizeArray(SimTypeChar(label="Byte"), 6)}, name="FWP_BYTE_ARRAY6", pack=False, align=None), offset=0), "bitmapArray64": SimTypePointer(SimStruct({"bitmapArray64": SimTypeFixedSizeArray(SimTypeChar(label="Byte"), 8)}, name="FWP_BITMAP_ARRAY64_", pack=False, align=None), offset=0)}, name="<anon>", label="None")}, name="FWP_VALUE0", pack=False, align=None), "numFilterConditions": SimTypeInt(signed=False, label="UInt32"), "filterCondition": SimTypePointer(SimStruct({"fieldKey": SimTypeBottom(label="Guid"), "matchType": SimTypeInt(signed=False, label="FWP_MATCH_TYPE"), "conditionValue": SimStruct({"type": SimTypeInt(signed=False, label="FWP_DATA_TYPE"), "Anonymous": SimUnion({"uint8": SimTypeChar(label="Byte"), "uint16": SimTypeShort(signed=False, label="UInt16"), "uint32": SimTypeInt(signed=False, label="UInt32"), "uint64": SimTypePointer(SimTypeLongLong(signed=False, label="UInt64"), offset=0), "int8": SimTypeChar(label="SByte"), "int16": SimTypeShort(signed=True, label="Int16"), "int32": SimTypeInt(signed=True, label="Int32"), "int64": SimTypePointer(SimTypeLongLong(signed=True, label="Int64"), offset=0), "float32": SimTypeFloat(size=32), "double64": SimTypePointer(SimTypeFloat(size=64), offset=0), "byteArray16": SimTypePointer(SimStruct({"byteArray16": SimTypeFixedSizeArray(SimTypeChar(label="Byte"), 16)}, name="FWP_BYTE_ARRAY16", pack=False, align=None), offset=0), "byteBlob": SimTypePointer(SimStruct({"size": SimTypeInt(signed=False, label="UInt32"), "data": SimTypePointer(SimTypeChar(label="Byte"), offset=0)}, name="FWP_BYTE_BLOB", pack=False, align=None), offset=0), "sid": SimTypePointer(SimTypeBottom(label="SID"), offset=0), "sd": SimTypePointer(SimStruct({"size": SimTypeInt(signed=False, label="UInt32"), "data": SimTypePointer(SimTypeChar(label="Byte"), offset=0)}, name="FWP_BYTE_BLOB", pack=False, align=None), offset=0), "tokenInformation": SimTypePointer(SimStruct({"sidCount": SimTypeInt(signed=False, label="UInt32"), "sids": SimTypePointer(SimTypeBottom(label="SID_AND_ATTRIBUTES"), offset=0), "restrictedSidCount": SimTypeInt(signed=False, label="UInt32"), "restrictedSids": SimTypePointer(SimTypeBottom(label="SID_AND_ATTRIBUTES"), offset=0)}, name="FWP_TOKEN_INFORMATION", pack=False, align=None), offset=0), "tokenAccessInformation": SimTypePointer(SimStruct({"size": SimTypeInt(signed=False, label="UInt32"), "data": SimTypePointer(SimTypeChar(label="Byte"), offset=0)}, name="FWP_BYTE_BLOB", pack=False, align=None), offset=0), "unicodeString": SimTypePointer(SimTypeChar(label="Char"), offset=0), "byteArray6": SimTypePointer(SimStruct({"byteArray6": SimTypeFixedSizeArray(SimTypeChar(label="Byte"), 6)}, name="FWP_BYTE_ARRAY6", pack=False, align=None), offset=0), "bitmapArray64": SimTypePointer(SimStruct({"bitmapArray64": SimTypeFixedSizeArray(SimTypeChar(label="Byte"), 8)}, name="FWP_BITMAP_ARRAY64_", pack=False, align=None), offset=0), "v4AddrMask": SimTypePointer(SimStruct({"addr": SimTypeInt(signed=False, label="UInt32"), "mask": SimTypeInt(signed=False, label="UInt32")}, name="FWP_V4_ADDR_AND_MASK", pack=False, align=None), offset=0), "v6AddrMask": SimTypePointer(SimStruct({"addr": SimTypeFixedSizeArray(SimTypeChar(label="Byte"), 16), "prefixLength": SimTypeChar(label="Byte")}, name="FWP_V6_ADDR_AND_MASK", pack=False, align=None), offset=0), "rangeValue": SimTypePointer(SimStruct({"valueLow": SimStruct({"type": SimTypeInt(signed=False, label="FWP_DATA_TYPE"), "Anonymous": SimUnion({"uint8": SimTypeChar(label="Byte"), "uint16": SimTypeShort(signed=False, label="UInt16"), "uint32": SimTypeInt(signed=False, label="UInt32"), "uint64": SimTypePointer(SimTypeLongLong(signed=False, label="UInt64"), offset=0), "int8": SimTypeChar(label="SByte"), "int16": SimTypeShort(signed=True, label="Int16"), "int32": SimTypeInt(signed=True, label="Int32"), "int64": SimTypePointer(SimTypeLongLong(signed=True, label="Int64"), offset=0), "float32": SimTypeFloat(size=32), "double64": SimTypePointer(SimTypeFloat(size=64), offset=0), "byteArray16": SimTypePointer(SimStruct({"byteArray16": SimTypeFixedSizeArray(SimTypeChar(label="Byte"), 16)}, name="FWP_BYTE_ARRAY16", pack=False, align=None), offset=0), "byteBlob": SimTypePointer(SimStruct({"size": SimTypeInt(signed=False, label="UInt32"), "data": SimTypePointer(SimTypeChar(label="Byte"), offset=0)}, name="FWP_BYTE_BLOB", pack=False, align=None), offset=0), "sid": SimTypePointer(SimTypeBottom(label="SID"), offset=0), "sd": SimTypePointer(SimStruct({"size": SimTypeInt(signed=False, label="UInt32"), "data": SimTypePointer(SimTypeChar(label="Byte"), offset=0)}, name="FWP_BYTE_BLOB", pack=False, align=None), offset=0), "tokenInformation": SimTypePointer(SimStruct({"sidCount": SimTypeInt(signed=False, label="UInt32"), "sids": SimTypePointer(SimTypeBottom(label="SID_AND_ATTRIBUTES"), offset=0), "restrictedSidCount": SimTypeInt(signed=False, label="UInt32"), "restrictedSids": SimTypePointer(SimTypeBottom(label="SID_AND_ATTRIBUTES"), offset=0)}, name="FWP_TOKEN_INFORMATION", pack=False, align=None), offset=0), "tokenAccessInformation": SimTypePointer(SimStruct({"size": SimTypeInt(signed=False, label="UInt32"), "data": SimTypePointer(SimTypeChar(label="Byte"), offset=0)}, name="FWP_BYTE_BLOB", pack=False, align=None), offset=0), "unicodeString": SimTypePointer(SimTypeChar(label="Char"), offset=0), "byteArray6": SimTypePointer(SimStruct({"byteArray6": SimTypeFixedSizeArray(SimTypeChar(label="Byte"), 6)}, name="FWP_BYTE_ARRAY6", pack=False, align=None), offset=0), "bitmapArray64": SimTypePointer(SimStruct({"bitmapArray64": SimTypeFixedSizeArray(SimTypeChar(label="Byte"), 8)}, name="FWP_BITMAP_ARRAY64_", pack=False, align=None), offset=0)}, name="<anon>", label="None")}, name="FWP_VALUE0", pack=False, align=None), "valueHigh": SimStruct({"type": SimTypeInt(signed=False, label="FWP_DATA_TYPE"), "Anonymous": SimUnion({"uint8": SimTypeChar(label="Byte"), "uint16": SimTypeShort(signed=False, label="UInt16"), "uint32": SimTypeInt(signed=False, label="UInt32"), "uint64": SimTypePointer(SimTypeLongLong(signed=False, label="UInt64"), offset=0), "int8": SimTypeChar(label="SByte"), "int16": SimTypeShort(signed=True, label="Int16"), "int32": SimTypeInt(signed=True, label="Int32"), "int64": SimTypePointer(SimTypeLongLong(signed=True, label="Int64"), offset=0), "float32": SimTypeFloat(size=32), "double64": SimTypePointer(SimTypeFloat(size=64), offset=0), "byteArray16": SimTypePointer(SimStruct({"byteArray16": SimTypeFixedSizeArray(SimTypeChar(label="Byte"), 16)}, name="FWP_BYTE_ARRAY16", pack=False, align=None), offset=0), "byteBlob": SimTypePointer(SimStruct({"size": SimTypeInt(signed=False, label="UInt32"), "data": SimTypePointer(SimTypeChar(label="Byte"), offset=0)}, name="FWP_BYTE_BLOB", pack=False, align=None), offset=0), "sid": SimTypePointer(SimTypeBottom(label="SID"), offset=0), "sd": SimTypePointer(SimStruct({"size": SimTypeInt(signed=False, label="UInt32"), "data": SimTypePointer(SimTypeChar(label="Byte"), offset=0)}, name="FWP_BYTE_BLOB", pack=False, align=None), offset=0), "tokenInformation": SimTypePointer(SimStruct({"sidCount": SimTypeInt(signed=False, label="UInt32"), "sids": SimTypePointer(SimTypeBottom(label="SID_AND_ATTRIBUTES"), offset=0), "restrictedSidCount": SimTypeInt(signed=False, label="UInt32"), "restrictedSids": SimTypePointer(SimTypeBottom(label="SID_AND_ATTRIBUTES"), offset=0)}, name="FWP_TOKEN_INFORMATION", pack=False, align=None), offset=0), "tokenAccessInformation": SimTypePointer(SimStruct({"size": SimTypeInt(signed=False, label="UInt32"), "data": SimTypePointer(SimTypeChar(label="Byte"), offset=0)}, name="FWP_BYTE_BLOB", pack=False, align=None), offset=0), "unicodeString": SimTypePointer(SimTypeChar(label="Char"), offset=0), "byteArray6": SimTypePointer(SimStruct({"byteArray6": SimTypeFixedSizeArray(SimTypeChar(label="Byte"), 6)}, name="FWP_BYTE_ARRAY6", pack=False, align=None), offset=0), "bitmapArray64": SimTypePointer(SimStruct({"bitmapArray64": SimTypeFixedSizeArray(SimTypeChar(label="Byte"), 8)}, name="FWP_BITMAP_ARRAY64_", pack=False, align=None), offset=0)}, name="<anon>", label="None")}, name="FWP_VALUE0", pack=False, align=None)}, name="FWP_RANGE0", pack=False, align=None), offset=0)}, name="<anon>", label="None")}, name="FWP_CONDITION_VALUE0", pack=False, align=None)}, name="FWPM_FILTER_CONDITION0", pack=False, align=None), offset=0), "action": SimStruct({"type": SimTypeInt(signed=False, label="UInt32"), "Anonymous": SimUnion({"filterType": SimTypeBottom(label="Guid"), "calloutKey": SimTypeBottom(label="Guid"), "bitmapIndex": SimTypeChar(label="Byte")}, name="<anon>", label="None")}, name="FWPM_ACTION0", pack=False, align=None), "Anonymous": SimUnion({"rawContext": SimTypeLongLong(signed=False, label="UInt64"), "providerContextKey": SimTypeBottom(label="Guid")}, name="<anon>", label="None"), "reserved": SimTypePointer(SimTypeBottom(label="Guid"), offset=0), "filterId": SimTypeLongLong(signed=False, label="UInt64"), "effectiveWeight": SimStruct({"type": SimTypeInt(signed=False, label="FWP_DATA_TYPE"), "Anonymous": SimUnion({"uint8": SimTypeChar(label="Byte"), "uint16": SimTypeShort(signed=False, label="UInt16"), "uint32": SimTypeInt(signed=False, label="UInt32"), "uint64": SimTypePointer(SimTypeLongLong(signed=False, label="UInt64"), offset=0), "int8": SimTypeChar(label="SByte"), "int16": SimTypeShort(signed=True, label="Int16"), "int32": SimTypeInt(signed=True, label="Int32"), "int64": SimTypePointer(SimTypeLongLong(signed=True, label="Int64"), offset=0), "float32": SimTypeFloat(size=32), "double64": SimTypePointer(SimTypeFloat(size=64), offset=0), "byteArray16": SimTypePointer(SimStruct({"byteArray16": SimTypeFixedSizeArray(SimTypeChar(label="Byte"), 16)}, name="FWP_BYTE_ARRAY16", pack=False, align=None), offset=0), "byteBlob": SimTypePointer(SimStruct({"size": SimTypeInt(signed=False, label="UInt32"), "data": SimTypePointer(SimTypeChar(label="Byte"), offset=0)}, name="FWP_BYTE_BLOB", pack=False, align=None), offset=0), "sid": SimTypePointer(SimTypeBottom(label="SID"), offset=0), "sd": SimTypePointer(SimStruct({"size": SimTypeInt(signed=False, label="UInt32"), "data": SimTypePointer(SimTypeChar(label="Byte"), offset=0)}, name="FWP_BYTE_BLOB", pack=False, align=None), offset=0), "tokenInformation": SimTypePointer(SimStruct({"sidCount": SimTypeInt(signed=False, label="UInt32"), "sids": SimTypePointer(SimTypeBottom(label="SID_AND_ATTRIBUTES"), offset=0), "restrictedSidCount": SimTypeInt(signed=False, label="UInt32"), "restrictedSids": SimTypePointer(SimTypeBottom(label="SID_AND_ATTRIBUTES"), offset=0)}, name="FWP_TOKEN_INFORMATION", pack=False, align=None), offset=0), "tokenAccessInformation": SimTypePointer(SimStruct({"size": SimTypeInt(signed=False, label="UInt32"), "data": SimTypePointer(SimTypeChar(label="Byte"), offset=0)}, name="FWP_BYTE_BLOB", pack=False, align=None), offset=0), "unicodeString": SimTypePointer(SimTypeChar(label="Char"), offset=0), "byteArray6": SimTypePointer(SimStruct({"byteArray6": SimTypeFixedSizeArray(SimTypeChar(label="Byte"), 6)}, name="FWP_BYTE_ARRAY6", pack=False, align=None), offset=0), "bitmapArray64": SimTypePointer(SimStruct({"bitmapArray64": SimTypeFixedSizeArray(SimTypeChar(label="Byte"), 8)}, name="FWP_BITMAP_ARRAY64_", pack=False, align=None), offset=0)}, name="<anon>", label="None")}, name="FWP_VALUE0", pack=False, align=None)}, name="FWPM_FILTER0", pack=False, align=None), offset=0), SimTypePointer(SimStruct({"Revision": SimTypeChar(label="Byte"), "Sbz1": SimTypeChar(label="Byte"), "Control": SimTypeShort(signed=False, label="UInt16"), "Owner": SimTypePointer(SimTypeInt(signed=True, label="Int"), label="IntPtr", offset=0), "Group": SimTypePointer(SimTypeInt(signed=True, label="Int"), label="IntPtr", offset=0), "Sacl": SimTypePointer(SimStruct({"AclRevision": SimTypeChar(label="Byte"), "Sbz1": SimTypeChar(label="Byte"), "AclSize": SimTypeShort(signed=False, label="UInt16"), "AceCount": SimTypeShort(signed=False, label="UInt16"), "Sbz2": SimTypeShort(signed=False, label="UInt16")}, name="ACL", pack=False, align=None), offset=0), "Dacl": SimTypePointer(SimStruct({"AclRevision": SimTypeChar(label="Byte"), "Sbz1": SimTypeChar(label="Byte"), "AclSize": SimTypeShort(signed=False, label="UInt16"), "AceCount": SimTypeShort(signed=False, label="UInt16"), "Sbz2": SimTypeShort(signed=False, label="UInt16")}, name="ACL", pack=False, align=None), offset=0)}, name="SECURITY_DESCRIPTOR", pack=False, align=None), offset=0), SimTypePointer(SimTypeLongLong(signed=False, label="UInt64"), offset=0)], SimTypeInt(signed=False, label="UInt32"), arg_names=["engineHandle", "filter", "sd", "id"]),
        # 
        'FwpmFilterDeleteById0': SimTypeFunction([SimTypePointer(SimTypeInt(signed=True, label="Int"), label="IntPtr", offset=0), SimTypeLongLong(signed=False, label="UInt64")], SimTypeInt(signed=False, label="UInt32"), arg_names=["engineHandle", "id"]),
        # 
        'FwpmFilterDeleteByKey0': SimTypeFunction([SimTypePointer(SimTypeInt(signed=True, label="Int"), label="IntPtr", offset=0), SimTypePointer(SimTypeBottom(label="Guid"), offset=0)], SimTypeInt(signed=False, label="UInt32"), arg_names=["engineHandle", "key"]),
        # 
        'FwpmFilterGetById0': SimTypeFunction([SimTypePointer(SimTypeInt(signed=True, label="Int"), label="IntPtr", offset=0), SimTypeLongLong(signed=False, label="UInt64"), SimTypePointer(SimTypePointer(SimStruct({"filterKey": SimTypeBottom(label="Guid"), "displayData": SimStruct({"name": SimTypePointer(SimTypeChar(label="Char"), offset=0), "description": SimTypePointer(SimTypeChar(label="Char"), offset=0)}, name="FWPM_DISPLAY_DATA0", pack=False, align=None), "flags": SimTypeInt(signed=False, label="FWPM_FILTER_FLAGS"), "providerKey": SimTypePointer(SimTypeBottom(label="Guid"), offset=0), "providerData": SimStruct({"size": SimTypeInt(signed=False, label="UInt32"), "data": SimTypePointer(SimTypeChar(label="Byte"), offset=0)}, name="FWP_BYTE_BLOB", pack=False, align=None), "layerKey": SimTypeBottom(label="Guid"), "subLayerKey": SimTypeBottom(label="Guid"), "weight": SimStruct({"type": SimTypeInt(signed=False, label="FWP_DATA_TYPE"), "Anonymous": SimUnion({"uint8": SimTypeChar(label="Byte"), "uint16": SimTypeShort(signed=False, label="UInt16"), "uint32": SimTypeInt(signed=False, label="UInt32"), "uint64": SimTypePointer(SimTypeLongLong(signed=False, label="UInt64"), offset=0), "int8": SimTypeChar(label="SByte"), "int16": SimTypeShort(signed=True, label="Int16"), "int32": SimTypeInt(signed=True, label="Int32"), "int64": SimTypePointer(SimTypeLongLong(signed=True, label="Int64"), offset=0), "float32": SimTypeFloat(size=32), "double64": SimTypePointer(SimTypeFloat(size=64), offset=0), "byteArray16": SimTypePointer(SimStruct({"byteArray16": SimTypeFixedSizeArray(SimTypeChar(label="Byte"), 16)}, name="FWP_BYTE_ARRAY16", pack=False, align=None), offset=0), "byteBlob": SimTypePointer(SimStruct({"size": SimTypeInt(signed=False, label="UInt32"), "data": SimTypePointer(SimTypeChar(label="Byte"), offset=0)}, name="FWP_BYTE_BLOB", pack=False, align=None), offset=0), "sid": SimTypePointer(SimTypeBottom(label="SID"), offset=0), "sd": SimTypePointer(SimStruct({"size": SimTypeInt(signed=False, label="UInt32"), "data": SimTypePointer(SimTypeChar(label="Byte"), offset=0)}, name="FWP_BYTE_BLOB", pack=False, align=None), offset=0), "tokenInformation": SimTypePointer(SimStruct({"sidCount": SimTypeInt(signed=False, label="UInt32"), "sids": SimTypePointer(SimTypeBottom(label="SID_AND_ATTRIBUTES"), offset=0), "restrictedSidCount": SimTypeInt(signed=False, label="UInt32"), "restrictedSids": SimTypePointer(SimTypeBottom(label="SID_AND_ATTRIBUTES"), offset=0)}, name="FWP_TOKEN_INFORMATION", pack=False, align=None), offset=0), "tokenAccessInformation": SimTypePointer(SimStruct({"size": SimTypeInt(signed=False, label="UInt32"), "data": SimTypePointer(SimTypeChar(label="Byte"), offset=0)}, name="FWP_BYTE_BLOB", pack=False, align=None), offset=0), "unicodeString": SimTypePointer(SimTypeChar(label="Char"), offset=0), "byteArray6": SimTypePointer(SimStruct({"byteArray6": SimTypeFixedSizeArray(SimTypeChar(label="Byte"), 6)}, name="FWP_BYTE_ARRAY6", pack=False, align=None), offset=0), "bitmapArray64": SimTypePointer(SimStruct({"bitmapArray64": SimTypeFixedSizeArray(SimTypeChar(label="Byte"), 8)}, name="FWP_BITMAP_ARRAY64_", pack=False, align=None), offset=0)}, name="<anon>", label="None")}, name="FWP_VALUE0", pack=False, align=None), "numFilterConditions": SimTypeInt(signed=False, label="UInt32"), "filterCondition": SimTypePointer(SimStruct({"fieldKey": SimTypeBottom(label="Guid"), "matchType": SimTypeInt(signed=False, label="FWP_MATCH_TYPE"), "conditionValue": SimStruct({"type": SimTypeInt(signed=False, label="FWP_DATA_TYPE"), "Anonymous": SimUnion({"uint8": SimTypeChar(label="Byte"), "uint16": SimTypeShort(signed=False, label="UInt16"), "uint32": SimTypeInt(signed=False, label="UInt32"), "uint64": SimTypePointer(SimTypeLongLong(signed=False, label="UInt64"), offset=0), "int8": SimTypeChar(label="SByte"), "int16": SimTypeShort(signed=True, label="Int16"), "int32": SimTypeInt(signed=True, label="Int32"), "int64": SimTypePointer(SimTypeLongLong(signed=True, label="Int64"), offset=0), "float32": SimTypeFloat(size=32), "double64": SimTypePointer(SimTypeFloat(size=64), offset=0), "byteArray16": SimTypePointer(SimStruct({"byteArray16": SimTypeFixedSizeArray(SimTypeChar(label="Byte"), 16)}, name="FWP_BYTE_ARRAY16", pack=False, align=None), offset=0), "byteBlob": SimTypePointer(SimStruct({"size": SimTypeInt(signed=False, label="UInt32"), "data": SimTypePointer(SimTypeChar(label="Byte"), offset=0)}, name="FWP_BYTE_BLOB", pack=False, align=None), offset=0), "sid": SimTypePointer(SimTypeBottom(label="SID"), offset=0), "sd": SimTypePointer(SimStruct({"size": SimTypeInt(signed=False, label="UInt32"), "data": SimTypePointer(SimTypeChar(label="Byte"), offset=0)}, name="FWP_BYTE_BLOB", pack=False, align=None), offset=0), "tokenInformation": SimTypePointer(SimStruct({"sidCount": SimTypeInt(signed=False, label="UInt32"), "sids": SimTypePointer(SimTypeBottom(label="SID_AND_ATTRIBUTES"), offset=0), "restrictedSidCount": SimTypeInt(signed=False, label="UInt32"), "restrictedSids": SimTypePointer(SimTypeBottom(label="SID_AND_ATTRIBUTES"), offset=0)}, name="FWP_TOKEN_INFORMATION", pack=False, align=None), offset=0), "tokenAccessInformation": SimTypePointer(SimStruct({"size": SimTypeInt(signed=False, label="UInt32"), "data": SimTypePointer(SimTypeChar(label="Byte"), offset=0)}, name="FWP_BYTE_BLOB", pack=False, align=None), offset=0), "unicodeString": SimTypePointer(SimTypeChar(label="Char"), offset=0), "byteArray6": SimTypePointer(SimStruct({"byteArray6": SimTypeFixedSizeArray(SimTypeChar(label="Byte"), 6)}, name="FWP_BYTE_ARRAY6", pack=False, align=None), offset=0), "bitmapArray64": SimTypePointer(SimStruct({"bitmapArray64": SimTypeFixedSizeArray(SimTypeChar(label="Byte"), 8)}, name="FWP_BITMAP_ARRAY64_", pack=False, align=None), offset=0), "v4AddrMask": SimTypePointer(SimStruct({"addr": SimTypeInt(signed=False, label="UInt32"), "mask": SimTypeInt(signed=False, label="UInt32")}, name="FWP_V4_ADDR_AND_MASK", pack=False, align=None), offset=0), "v6AddrMask": SimTypePointer(SimStruct({"addr": SimTypeFixedSizeArray(SimTypeChar(label="Byte"), 16), "prefixLength": SimTypeChar(label="Byte")}, name="FWP_V6_ADDR_AND_MASK", pack=False, align=None), offset=0), "rangeValue": SimTypePointer(SimStruct({"valueLow": SimStruct({"type": SimTypeInt(signed=False, label="FWP_DATA_TYPE"), "Anonymous": SimUnion({"uint8": SimTypeChar(label="Byte"), "uint16": SimTypeShort(signed=False, label="UInt16"), "uint32": SimTypeInt(signed=False, label="UInt32"), "uint64": SimTypePointer(SimTypeLongLong(signed=False, label="UInt64"), offset=0), "int8": SimTypeChar(label="SByte"), "int16": SimTypeShort(signed=True, label="Int16"), "int32": SimTypeInt(signed=True, label="Int32"), "int64": SimTypePointer(SimTypeLongLong(signed=True, label="Int64"), offset=0), "float32": SimTypeFloat(size=32), "double64": SimTypePointer(SimTypeFloat(size=64), offset=0), "byteArray16": SimTypePointer(SimStruct({"byteArray16": SimTypeFixedSizeArray(SimTypeChar(label="Byte"), 16)}, name="FWP_BYTE_ARRAY16", pack=False, align=None), offset=0), "byteBlob": SimTypePointer(SimStruct({"size": SimTypeInt(signed=False, label="UInt32"), "data": SimTypePointer(SimTypeChar(label="Byte"), offset=0)}, name="FWP_BYTE_BLOB", pack=False, align=None), offset=0), "sid": SimTypePointer(SimTypeBottom(label="SID"), offset=0), "sd": SimTypePointer(SimStruct({"size": SimTypeInt(signed=False, label="UInt32"), "data": SimTypePointer(SimTypeChar(label="Byte"), offset=0)}, name="FWP_BYTE_BLOB", pack=False, align=None), offset=0), "tokenInformation": SimTypePointer(SimStruct({"sidCount": SimTypeInt(signed=False, label="UInt32"), "sids": SimTypePointer(SimTypeBottom(label="SID_AND_ATTRIBUTES"), offset=0), "restrictedSidCount": SimTypeInt(signed=False, label="UInt32"), "restrictedSids": SimTypePointer(SimTypeBottom(label="SID_AND_ATTRIBUTES"), offset=0)}, name="FWP_TOKEN_INFORMATION", pack=False, align=None), offset=0), "tokenAccessInformation": SimTypePointer(SimStruct({"size": SimTypeInt(signed=False, label="UInt32"), "data": SimTypePointer(SimTypeChar(label="Byte"), offset=0)}, name="FWP_BYTE_BLOB", pack=False, align=None), offset=0), "unicodeString": SimTypePointer(SimTypeChar(label="Char"), offset=0), "byteArray6": SimTypePointer(SimStruct({"byteArray6": SimTypeFixedSizeArray(SimTypeChar(label="Byte"), 6)}, name="FWP_BYTE_ARRAY6", pack=False, align=None), offset=0), "bitmapArray64": SimTypePointer(SimStruct({"bitmapArray64": SimTypeFixedSizeArray(SimTypeChar(label="Byte"), 8)}, name="FWP_BITMAP_ARRAY64_", pack=False, align=None), offset=0)}, name="<anon>", label="None")}, name="FWP_VALUE0", pack=False, align=None), "valueHigh": SimStruct({"type": SimTypeInt(signed=False, label="FWP_DATA_TYPE"), "Anonymous": SimUnion({"uint8": SimTypeChar(label="Byte"), "uint16": SimTypeShort(signed=False, label="UInt16"), "uint32": SimTypeInt(signed=False, label="UInt32"), "uint64": SimTypePointer(SimTypeLongLong(signed=False, label="UInt64"), offset=0), "int8": SimTypeChar(label="SByte"), "int16": SimTypeShort(signed=True, label="Int16"), "int32": SimTypeInt(signed=True, label="Int32"), "int64": SimTypePointer(SimTypeLongLong(signed=True, label="Int64"), offset=0), "float32": SimTypeFloat(size=32), "double64": SimTypePointer(SimTypeFloat(size=64), offset=0), "byteArray16": SimTypePointer(SimStruct({"byteArray16": SimTypeFixedSizeArray(SimTypeChar(label="Byte"), 16)}, name="FWP_BYTE_ARRAY16", pack=False, align=None), offset=0), "byteBlob": SimTypePointer(SimStruct({"size": SimTypeInt(signed=False, label="UInt32"), "data": SimTypePointer(SimTypeChar(label="Byte"), offset=0)}, name="FWP_BYTE_BLOB", pack=False, align=None), offset=0), "sid": SimTypePointer(SimTypeBottom(label="SID"), offset=0), "sd": SimTypePointer(SimStruct({"size": SimTypeInt(signed=False, label="UInt32"), "data": SimTypePointer(SimTypeChar(label="Byte"), offset=0)}, name="FWP_BYTE_BLOB", pack=False, align=None), offset=0), "tokenInformation": SimTypePointer(SimStruct({"sidCount": SimTypeInt(signed=False, label="UInt32"), "sids": SimTypePointer(SimTypeBottom(label="SID_AND_ATTRIBUTES"), offset=0), "restrictedSidCount": SimTypeInt(signed=False, label="UInt32"), "restrictedSids": SimTypePointer(SimTypeBottom(label="SID_AND_ATTRIBUTES"), offset=0)}, name="FWP_TOKEN_INFORMATION", pack=False, align=None), offset=0), "tokenAccessInformation": SimTypePointer(SimStruct({"size": SimTypeInt(signed=False, label="UInt32"), "data": SimTypePointer(SimTypeChar(label="Byte"), offset=0)}, name="FWP_BYTE_BLOB", pack=False, align=None), offset=0), "unicodeString": SimTypePointer(SimTypeChar(label="Char"), offset=0), "byteArray6": SimTypePointer(SimStruct({"byteArray6": SimTypeFixedSizeArray(SimTypeChar(label="Byte"), 6)}, name="FWP_BYTE_ARRAY6", pack=False, align=None), offset=0), "bitmapArray64": SimTypePointer(SimStruct({"bitmapArray64": SimTypeFixedSizeArray(SimTypeChar(label="Byte"), 8)}, name="FWP_BITMAP_ARRAY64_", pack=False, align=None), offset=0)}, name="<anon>", label="None")}, name="FWP_VALUE0", pack=False, align=None)}, name="FWP_RANGE0", pack=False, align=None), offset=0)}, name="<anon>", label="None")}, name="FWP_CONDITION_VALUE0", pack=False, align=None)}, name="FWPM_FILTER_CONDITION0", pack=False, align=None), offset=0), "action": SimStruct({"type": SimTypeInt(signed=False, label="UInt32"), "Anonymous": SimUnion({"filterType": SimTypeBottom(label="Guid"), "calloutKey": SimTypeBottom(label="Guid"), "bitmapIndex": SimTypeChar(label="Byte")}, name="<anon>", label="None")}, name="FWPM_ACTION0", pack=False, align=None), "Anonymous": SimUnion({"rawContext": SimTypeLongLong(signed=False, label="UInt64"), "providerContextKey": SimTypeBottom(label="Guid")}, name="<anon>", label="None"), "reserved": SimTypePointer(SimTypeBottom(label="Guid"), offset=0), "filterId": SimTypeLongLong(signed=False, label="UInt64"), "effectiveWeight": SimStruct({"type": SimTypeInt(signed=False, label="FWP_DATA_TYPE"), "Anonymous": SimUnion({"uint8": SimTypeChar(label="Byte"), "uint16": SimTypeShort(signed=False, label="UInt16"), "uint32": SimTypeInt(signed=False, label="UInt32"), "uint64": SimTypePointer(SimTypeLongLong(signed=False, label="UInt64"), offset=0), "int8": SimTypeChar(label="SByte"), "int16": SimTypeShort(signed=True, label="Int16"), "int32": SimTypeInt(signed=True, label="Int32"), "int64": SimTypePointer(SimTypeLongLong(signed=True, label="Int64"), offset=0), "float32": SimTypeFloat(size=32), "double64": SimTypePointer(SimTypeFloat(size=64), offset=0), "byteArray16": SimTypePointer(SimStruct({"byteArray16": SimTypeFixedSizeArray(SimTypeChar(label="Byte"), 16)}, name="FWP_BYTE_ARRAY16", pack=False, align=None), offset=0), "byteBlob": SimTypePointer(SimStruct({"size": SimTypeInt(signed=False, label="UInt32"), "data": SimTypePointer(SimTypeChar(label="Byte"), offset=0)}, name="FWP_BYTE_BLOB", pack=False, align=None), offset=0), "sid": SimTypePointer(SimTypeBottom(label="SID"), offset=0), "sd": SimTypePointer(SimStruct({"size": SimTypeInt(signed=False, label="UInt32"), "data": SimTypePointer(SimTypeChar(label="Byte"), offset=0)}, name="FWP_BYTE_BLOB", pack=False, align=None), offset=0), "tokenInformation": SimTypePointer(SimStruct({"sidCount": SimTypeInt(signed=False, label="UInt32"), "sids": SimTypePointer(SimTypeBottom(label="SID_AND_ATTRIBUTES"), offset=0), "restrictedSidCount": SimTypeInt(signed=False, label="UInt32"), "restrictedSids": SimTypePointer(SimTypeBottom(label="SID_AND_ATTRIBUTES"), offset=0)}, name="FWP_TOKEN_INFORMATION", pack=False, align=None), offset=0), "tokenAccessInformation": SimTypePointer(SimStruct({"size": SimTypeInt(signed=False, label="UInt32"), "data": SimTypePointer(SimTypeChar(label="Byte"), offset=0)}, name="FWP_BYTE_BLOB", pack=False, align=None), offset=0), "unicodeString": SimTypePointer(SimTypeChar(label="Char"), offset=0), "byteArray6": SimTypePointer(SimStruct({"byteArray6": SimTypeFixedSizeArray(SimTypeChar(label="Byte"), 6)}, name="FWP_BYTE_ARRAY6", pack=False, align=None), offset=0), "bitmapArray64": SimTypePointer(SimStruct({"bitmapArray64": SimTypeFixedSizeArray(SimTypeChar(label="Byte"), 8)}, name="FWP_BITMAP_ARRAY64_", pack=False, align=None), offset=0)}, name="<anon>", label="None")}, name="FWP_VALUE0", pack=False, align=None)}, name="FWPM_FILTER0", pack=False, align=None), offset=0), offset=0)], SimTypeInt(signed=False, label="UInt32"), arg_names=["engineHandle", "id", "filter"]),
        # 
        'FwpmFilterGetByKey0': SimTypeFunction([SimTypePointer(SimTypeInt(signed=True, label="Int"), label="IntPtr", offset=0), SimTypePointer(SimTypeBottom(label="Guid"), offset=0), SimTypePointer(SimTypePointer(SimStruct({"filterKey": SimTypeBottom(label="Guid"), "displayData": SimStruct({"name": SimTypePointer(SimTypeChar(label="Char"), offset=0), "description": SimTypePointer(SimTypeChar(label="Char"), offset=0)}, name="FWPM_DISPLAY_DATA0", pack=False, align=None), "flags": SimTypeInt(signed=False, label="FWPM_FILTER_FLAGS"), "providerKey": SimTypePointer(SimTypeBottom(label="Guid"), offset=0), "providerData": SimStruct({"size": SimTypeInt(signed=False, label="UInt32"), "data": SimTypePointer(SimTypeChar(label="Byte"), offset=0)}, name="FWP_BYTE_BLOB", pack=False, align=None), "layerKey": SimTypeBottom(label="Guid"), "subLayerKey": SimTypeBottom(label="Guid"), "weight": SimStruct({"type": SimTypeInt(signed=False, label="FWP_DATA_TYPE"), "Anonymous": SimUnion({"uint8": SimTypeChar(label="Byte"), "uint16": SimTypeShort(signed=False, label="UInt16"), "uint32": SimTypeInt(signed=False, label="UInt32"), "uint64": SimTypePointer(SimTypeLongLong(signed=False, label="UInt64"), offset=0), "int8": SimTypeChar(label="SByte"), "int16": SimTypeShort(signed=True, label="Int16"), "int32": SimTypeInt(signed=True, label="Int32"), "int64": SimTypePointer(SimTypeLongLong(signed=True, label="Int64"), offset=0), "float32": SimTypeFloat(size=32), "double64": SimTypePointer(SimTypeFloat(size=64), offset=0), "byteArray16": SimTypePointer(SimStruct({"byteArray16": SimTypeFixedSizeArray(SimTypeChar(label="Byte"), 16)}, name="FWP_BYTE_ARRAY16", pack=False, align=None), offset=0), "byteBlob": SimTypePointer(SimStruct({"size": SimTypeInt(signed=False, label="UInt32"), "data": SimTypePointer(SimTypeChar(label="Byte"), offset=0)}, name="FWP_BYTE_BLOB", pack=False, align=None), offset=0), "sid": SimTypePointer(SimTypeBottom(label="SID"), offset=0), "sd": SimTypePointer(SimStruct({"size": SimTypeInt(signed=False, label="UInt32"), "data": SimTypePointer(SimTypeChar(label="Byte"), offset=0)}, name="FWP_BYTE_BLOB", pack=False, align=None), offset=0), "tokenInformation": SimTypePointer(SimStruct({"sidCount": SimTypeInt(signed=False, label="UInt32"), "sids": SimTypePointer(SimTypeBottom(label="SID_AND_ATTRIBUTES"), offset=0), "restrictedSidCount": SimTypeInt(signed=False, label="UInt32"), "restrictedSids": SimTypePointer(SimTypeBottom(label="SID_AND_ATTRIBUTES"), offset=0)}, name="FWP_TOKEN_INFORMATION", pack=False, align=None), offset=0), "tokenAccessInformation": SimTypePointer(SimStruct({"size": SimTypeInt(signed=False, label="UInt32"), "data": SimTypePointer(SimTypeChar(label="Byte"), offset=0)}, name="FWP_BYTE_BLOB", pack=False, align=None), offset=0), "unicodeString": SimTypePointer(SimTypeChar(label="Char"), offset=0), "byteArray6": SimTypePointer(SimStruct({"byteArray6": SimTypeFixedSizeArray(SimTypeChar(label="Byte"), 6)}, name="FWP_BYTE_ARRAY6", pack=False, align=None), offset=0), "bitmapArray64": SimTypePointer(SimStruct({"bitmapArray64": SimTypeFixedSizeArray(SimTypeChar(label="Byte"), 8)}, name="FWP_BITMAP_ARRAY64_", pack=False, align=None), offset=0)}, name="<anon>", label="None")}, name="FWP_VALUE0", pack=False, align=None), "numFilterConditions": SimTypeInt(signed=False, label="UInt32"), "filterCondition": SimTypePointer(SimStruct({"fieldKey": SimTypeBottom(label="Guid"), "matchType": SimTypeInt(signed=False, label="FWP_MATCH_TYPE"), "conditionValue": SimStruct({"type": SimTypeInt(signed=False, label="FWP_DATA_TYPE"), "Anonymous": SimUnion({"uint8": SimTypeChar(label="Byte"), "uint16": SimTypeShort(signed=False, label="UInt16"), "uint32": SimTypeInt(signed=False, label="UInt32"), "uint64": SimTypePointer(SimTypeLongLong(signed=False, label="UInt64"), offset=0), "int8": SimTypeChar(label="SByte"), "int16": SimTypeShort(signed=True, label="Int16"), "int32": SimTypeInt(signed=True, label="Int32"), "int64": SimTypePointer(SimTypeLongLong(signed=True, label="Int64"), offset=0), "float32": SimTypeFloat(size=32), "double64": SimTypePointer(SimTypeFloat(size=64), offset=0), "byteArray16": SimTypePointer(SimStruct({"byteArray16": SimTypeFixedSizeArray(SimTypeChar(label="Byte"), 16)}, name="FWP_BYTE_ARRAY16", pack=False, align=None), offset=0), "byteBlob": SimTypePointer(SimStruct({"size": SimTypeInt(signed=False, label="UInt32"), "data": SimTypePointer(SimTypeChar(label="Byte"), offset=0)}, name="FWP_BYTE_BLOB", pack=False, align=None), offset=0), "sid": SimTypePointer(SimTypeBottom(label="SID"), offset=0), "sd": SimTypePointer(SimStruct({"size": SimTypeInt(signed=False, label="UInt32"), "data": SimTypePointer(SimTypeChar(label="Byte"), offset=0)}, name="FWP_BYTE_BLOB", pack=False, align=None), offset=0), "tokenInformation": SimTypePointer(SimStruct({"sidCount": SimTypeInt(signed=False, label="UInt32"), "sids": SimTypePointer(SimTypeBottom(label="SID_AND_ATTRIBUTES"), offset=0), "restrictedSidCount": SimTypeInt(signed=False, label="UInt32"), "restrictedSids": SimTypePointer(SimTypeBottom(label="SID_AND_ATTRIBUTES"), offset=0)}, name="FWP_TOKEN_INFORMATION", pack=False, align=None), offset=0), "tokenAccessInformation": SimTypePointer(SimStruct({"size": SimTypeInt(signed=False, label="UInt32"), "data": SimTypePointer(SimTypeChar(label="Byte"), offset=0)}, name="FWP_BYTE_BLOB", pack=False, align=None), offset=0), "unicodeString": SimTypePointer(SimTypeChar(label="Char"), offset=0), "byteArray6": SimTypePointer(SimStruct({"byteArray6": SimTypeFixedSizeArray(SimTypeChar(label="Byte"), 6)}, name="FWP_BYTE_ARRAY6", pack=False, align=None), offset=0), "bitmapArray64": SimTypePointer(SimStruct({"bitmapArray64": SimTypeFixedSizeArray(SimTypeChar(label="Byte"), 8)}, name="FWP_BITMAP_ARRAY64_", pack=False, align=None), offset=0), "v4AddrMask": SimTypePointer(SimStruct({"addr": SimTypeInt(signed=False, label="UInt32"), "mask": SimTypeInt(signed=False, label="UInt32")}, name="FWP_V4_ADDR_AND_MASK", pack=False, align=None), offset=0), "v6AddrMask": SimTypePointer(SimStruct({"addr": SimTypeFixedSizeArray(SimTypeChar(label="Byte"), 16), "prefixLength": SimTypeChar(label="Byte")}, name="FWP_V6_ADDR_AND_MASK", pack=False, align=None), offset=0), "rangeValue": SimTypePointer(SimStruct({"valueLow": SimStruct({"type": SimTypeInt(signed=False, label="FWP_DATA_TYPE"), "Anonymous": SimUnion({"uint8": SimTypeChar(label="Byte"), "uint16": SimTypeShort(signed=False, label="UInt16"), "uint32": SimTypeInt(signed=False, label="UInt32"), "uint64": SimTypePointer(SimTypeLongLong(signed=False, label="UInt64"), offset=0), "int8": SimTypeChar(label="SByte"), "int16": SimTypeShort(signed=True, label="Int16"), "int32": SimTypeInt(signed=True, label="Int32"), "int64": SimTypePointer(SimTypeLongLong(signed=True, label="Int64"), offset=0), "float32": SimTypeFloat(size=32), "double64": SimTypePointer(SimTypeFloat(size=64), offset=0), "byteArray16": SimTypePointer(SimStruct({"byteArray16": SimTypeFixedSizeArray(SimTypeChar(label="Byte"), 16)}, name="FWP_BYTE_ARRAY16", pack=False, align=None), offset=0), "byteBlob": SimTypePointer(SimStruct({"size": SimTypeInt(signed=False, label="UInt32"), "data": SimTypePointer(SimTypeChar(label="Byte"), offset=0)}, name="FWP_BYTE_BLOB", pack=False, align=None), offset=0), "sid": SimTypePointer(SimTypeBottom(label="SID"), offset=0), "sd": SimTypePointer(SimStruct({"size": SimTypeInt(signed=False, label="UInt32"), "data": SimTypePointer(SimTypeChar(label="Byte"), offset=0)}, name="FWP_BYTE_BLOB", pack=False, align=None), offset=0), "tokenInformation": SimTypePointer(SimStruct({"sidCount": SimTypeInt(signed=False, label="UInt32"), "sids": SimTypePointer(SimTypeBottom(label="SID_AND_ATTRIBUTES"), offset=0), "restrictedSidCount": SimTypeInt(signed=False, label="UInt32"), "restrictedSids": SimTypePointer(SimTypeBottom(label="SID_AND_ATTRIBUTES"), offset=0)}, name="FWP_TOKEN_INFORMATION", pack=False, align=None), offset=0), "tokenAccessInformation": SimTypePointer(SimStruct({"size": SimTypeInt(signed=False, label="UInt32"), "data": SimTypePointer(SimTypeChar(label="Byte"), offset=0)}, name="FWP_BYTE_BLOB", pack=False, align=None), offset=0), "unicodeString": SimTypePointer(SimTypeChar(label="Char"), offset=0), "byteArray6": SimTypePointer(SimStruct({"byteArray6": SimTypeFixedSizeArray(SimTypeChar(label="Byte"), 6)}, name="FWP_BYTE_ARRAY6", pack=False, align=None), offset=0), "bitmapArray64": SimTypePointer(SimStruct({"bitmapArray64": SimTypeFixedSizeArray(SimTypeChar(label="Byte"), 8)}, name="FWP_BITMAP_ARRAY64_", pack=False, align=None), offset=0)}, name="<anon>", label="None")}, name="FWP_VALUE0", pack=False, align=None), "valueHigh": SimStruct({"type": SimTypeInt(signed=False, label="FWP_DATA_TYPE"), "Anonymous": SimUnion({"uint8": SimTypeChar(label="Byte"), "uint16": SimTypeShort(signed=False, label="UInt16"), "uint32": SimTypeInt(signed=False, label="UInt32"), "uint64": SimTypePointer(SimTypeLongLong(signed=False, label="UInt64"), offset=0), "int8": SimTypeChar(label="SByte"), "int16": SimTypeShort(signed=True, label="Int16"), "int32": SimTypeInt(signed=True, label="Int32"), "int64": SimTypePointer(SimTypeLongLong(signed=True, label="Int64"), offset=0), "float32": SimTypeFloat(size=32), "double64": SimTypePointer(SimTypeFloat(size=64), offset=0), "byteArray16": SimTypePointer(SimStruct({"byteArray16": SimTypeFixedSizeArray(SimTypeChar(label="Byte"), 16)}, name="FWP_BYTE_ARRAY16", pack=False, align=None), offset=0), "byteBlob": SimTypePointer(SimStruct({"size": SimTypeInt(signed=False, label="UInt32"), "data": SimTypePointer(SimTypeChar(label="Byte"), offset=0)}, name="FWP_BYTE_BLOB", pack=False, align=None), offset=0), "sid": SimTypePointer(SimTypeBottom(label="SID"), offset=0), "sd": SimTypePointer(SimStruct({"size": SimTypeInt(signed=False, label="UInt32"), "data": SimTypePointer(SimTypeChar(label="Byte"), offset=0)}, name="FWP_BYTE_BLOB", pack=False, align=None), offset=0), "tokenInformation": SimTypePointer(SimStruct({"sidCount": SimTypeInt(signed=False, label="UInt32"), "sids": SimTypePointer(SimTypeBottom(label="SID_AND_ATTRIBUTES"), offset=0), "restrictedSidCount": SimTypeInt(signed=False, label="UInt32"), "restrictedSids": SimTypePointer(SimTypeBottom(label="SID_AND_ATTRIBUTES"), offset=0)}, name="FWP_TOKEN_INFORMATION", pack=False, align=None), offset=0), "tokenAccessInformation": SimTypePointer(SimStruct({"size": SimTypeInt(signed=False, label="UInt32"), "data": SimTypePointer(SimTypeChar(label="Byte"), offset=0)}, name="FWP_BYTE_BLOB", pack=False, align=None), offset=0), "unicodeString": SimTypePointer(SimTypeChar(label="Char"), offset=0), "byteArray6": SimTypePointer(SimStruct({"byteArray6": SimTypeFixedSizeArray(SimTypeChar(label="Byte"), 6)}, name="FWP_BYTE_ARRAY6", pack=False, align=None), offset=0), "bitmapArray64": SimTypePointer(SimStruct({"bitmapArray64": SimTypeFixedSizeArray(SimTypeChar(label="Byte"), 8)}, name="FWP_BITMAP_ARRAY64_", pack=False, align=None), offset=0)}, name="<anon>", label="None")}, name="FWP_VALUE0", pack=False, align=None)}, name="FWP_RANGE0", pack=False, align=None), offset=0)}, name="<anon>", label="None")}, name="FWP_CONDITION_VALUE0", pack=False, align=None)}, name="FWPM_FILTER_CONDITION0", pack=False, align=None), offset=0), "action": SimStruct({"type": SimTypeInt(signed=False, label="UInt32"), "Anonymous": SimUnion({"filterType": SimTypeBottom(label="Guid"), "calloutKey": SimTypeBottom(label="Guid"), "bitmapIndex": SimTypeChar(label="Byte")}, name="<anon>", label="None")}, name="FWPM_ACTION0", pack=False, align=None), "Anonymous": SimUnion({"rawContext": SimTypeLongLong(signed=False, label="UInt64"), "providerContextKey": SimTypeBottom(label="Guid")}, name="<anon>", label="None"), "reserved": SimTypePointer(SimTypeBottom(label="Guid"), offset=0), "filterId": SimTypeLongLong(signed=False, label="UInt64"), "effectiveWeight": SimStruct({"type": SimTypeInt(signed=False, label="FWP_DATA_TYPE"), "Anonymous": SimUnion({"uint8": SimTypeChar(label="Byte"), "uint16": SimTypeShort(signed=False, label="UInt16"), "uint32": SimTypeInt(signed=False, label="UInt32"), "uint64": SimTypePointer(SimTypeLongLong(signed=False, label="UInt64"), offset=0), "int8": SimTypeChar(label="SByte"), "int16": SimTypeShort(signed=True, label="Int16"), "int32": SimTypeInt(signed=True, label="Int32"), "int64": SimTypePointer(SimTypeLongLong(signed=True, label="Int64"), offset=0), "float32": SimTypeFloat(size=32), "double64": SimTypePointer(SimTypeFloat(size=64), offset=0), "byteArray16": SimTypePointer(SimStruct({"byteArray16": SimTypeFixedSizeArray(SimTypeChar(label="Byte"), 16)}, name="FWP_BYTE_ARRAY16", pack=False, align=None), offset=0), "byteBlob": SimTypePointer(SimStruct({"size": SimTypeInt(signed=False, label="UInt32"), "data": SimTypePointer(SimTypeChar(label="Byte"), offset=0)}, name="FWP_BYTE_BLOB", pack=False, align=None), offset=0), "sid": SimTypePointer(SimTypeBottom(label="SID"), offset=0), "sd": SimTypePointer(SimStruct({"size": SimTypeInt(signed=False, label="UInt32"), "data": SimTypePointer(SimTypeChar(label="Byte"), offset=0)}, name="FWP_BYTE_BLOB", pack=False, align=None), offset=0), "tokenInformation": SimTypePointer(SimStruct({"sidCount": SimTypeInt(signed=False, label="UInt32"), "sids": SimTypePointer(SimTypeBottom(label="SID_AND_ATTRIBUTES"), offset=0), "restrictedSidCount": SimTypeInt(signed=False, label="UInt32"), "restrictedSids": SimTypePointer(SimTypeBottom(label="SID_AND_ATTRIBUTES"), offset=0)}, name="FWP_TOKEN_INFORMATION", pack=False, align=None), offset=0), "tokenAccessInformation": SimTypePointer(SimStruct({"size": SimTypeInt(signed=False, label="UInt32"), "data": SimTypePointer(SimTypeChar(label="Byte"), offset=0)}, name="FWP_BYTE_BLOB", pack=False, align=None), offset=0), "unicodeString": SimTypePointer(SimTypeChar(label="Char"), offset=0), "byteArray6": SimTypePointer(SimStruct({"byteArray6": SimTypeFixedSizeArray(SimTypeChar(label="Byte"), 6)}, name="FWP_BYTE_ARRAY6", pack=False, align=None), offset=0), "bitmapArray64": SimTypePointer(SimStruct({"bitmapArray64": SimTypeFixedSizeArray(SimTypeChar(label="Byte"), 8)}, name="FWP_BITMAP_ARRAY64_", pack=False, align=None), offset=0)}, name="<anon>", label="None")}, name="FWP_VALUE0", pack=False, align=None)}, name="FWPM_FILTER0", pack=False, align=None), offset=0), offset=0)], SimTypeInt(signed=False, label="UInt32"), arg_names=["engineHandle", "key", "filter"]),
        # 
        'FwpmFilterCreateEnumHandle0': SimTypeFunction([SimTypePointer(SimTypeInt(signed=True, label="Int"), label="IntPtr", offset=0), SimTypePointer(SimStruct({"providerKey": SimTypePointer(SimTypeBottom(label="Guid"), offset=0), "layerKey": SimTypeBottom(label="Guid"), "enumType": SimTypeInt(signed=False, label="FWP_FILTER_ENUM_TYPE"), "flags": SimTypeInt(signed=False, label="UInt32"), "providerContextTemplate": SimTypePointer(SimStruct({"providerKey": SimTypePointer(SimTypeBottom(label="Guid"), offset=0), "providerContextType": SimTypeInt(signed=False, label="FWPM_PROVIDER_CONTEXT_TYPE")}, name="FWPM_PROVIDER_CONTEXT_ENUM_TEMPLATE0", pack=False, align=None), offset=0), "numFilterConditions": SimTypeInt(signed=False, label="UInt32"), "filterCondition": SimTypePointer(SimStruct({"fieldKey": SimTypeBottom(label="Guid"), "matchType": SimTypeInt(signed=False, label="FWP_MATCH_TYPE"), "conditionValue": SimStruct({"type": SimTypeInt(signed=False, label="FWP_DATA_TYPE"), "Anonymous": SimUnion({"uint8": SimTypeChar(label="Byte"), "uint16": SimTypeShort(signed=False, label="UInt16"), "uint32": SimTypeInt(signed=False, label="UInt32"), "uint64": SimTypePointer(SimTypeLongLong(signed=False, label="UInt64"), offset=0), "int8": SimTypeChar(label="SByte"), "int16": SimTypeShort(signed=True, label="Int16"), "int32": SimTypeInt(signed=True, label="Int32"), "int64": SimTypePointer(SimTypeLongLong(signed=True, label="Int64"), offset=0), "float32": SimTypeFloat(size=32), "double64": SimTypePointer(SimTypeFloat(size=64), offset=0), "byteArray16": SimTypePointer(SimStruct({"byteArray16": SimTypeFixedSizeArray(SimTypeChar(label="Byte"), 16)}, name="FWP_BYTE_ARRAY16", pack=False, align=None), offset=0), "byteBlob": SimTypePointer(SimStruct({"size": SimTypeInt(signed=False, label="UInt32"), "data": SimTypePointer(SimTypeChar(label="Byte"), offset=0)}, name="FWP_BYTE_BLOB", pack=False, align=None), offset=0), "sid": SimTypePointer(SimTypeBottom(label="SID"), offset=0), "sd": SimTypePointer(SimStruct({"size": SimTypeInt(signed=False, label="UInt32"), "data": SimTypePointer(SimTypeChar(label="Byte"), offset=0)}, name="FWP_BYTE_BLOB", pack=False, align=None), offset=0), "tokenInformation": SimTypePointer(SimStruct({"sidCount": SimTypeInt(signed=False, label="UInt32"), "sids": SimTypePointer(SimTypeBottom(label="SID_AND_ATTRIBUTES"), offset=0), "restrictedSidCount": SimTypeInt(signed=False, label="UInt32"), "restrictedSids": SimTypePointer(SimTypeBottom(label="SID_AND_ATTRIBUTES"), offset=0)}, name="FWP_TOKEN_INFORMATION", pack=False, align=None), offset=0), "tokenAccessInformation": SimTypePointer(SimStruct({"size": SimTypeInt(signed=False, label="UInt32"), "data": SimTypePointer(SimTypeChar(label="Byte"), offset=0)}, name="FWP_BYTE_BLOB", pack=False, align=None), offset=0), "unicodeString": SimTypePointer(SimTypeChar(label="Char"), offset=0), "byteArray6": SimTypePointer(SimStruct({"byteArray6": SimTypeFixedSizeArray(SimTypeChar(label="Byte"), 6)}, name="FWP_BYTE_ARRAY6", pack=False, align=None), offset=0), "bitmapArray64": SimTypePointer(SimStruct({"bitmapArray64": SimTypeFixedSizeArray(SimTypeChar(label="Byte"), 8)}, name="FWP_BITMAP_ARRAY64_", pack=False, align=None), offset=0), "v4AddrMask": SimTypePointer(SimStruct({"addr": SimTypeInt(signed=False, label="UInt32"), "mask": SimTypeInt(signed=False, label="UInt32")}, name="FWP_V4_ADDR_AND_MASK", pack=False, align=None), offset=0), "v6AddrMask": SimTypePointer(SimStruct({"addr": SimTypeFixedSizeArray(SimTypeChar(label="Byte"), 16), "prefixLength": SimTypeChar(label="Byte")}, name="FWP_V6_ADDR_AND_MASK", pack=False, align=None), offset=0), "rangeValue": SimTypePointer(SimStruct({"valueLow": SimStruct({"type": SimTypeInt(signed=False, label="FWP_DATA_TYPE"), "Anonymous": SimUnion({"uint8": SimTypeChar(label="Byte"), "uint16": SimTypeShort(signed=False, label="UInt16"), "uint32": SimTypeInt(signed=False, label="UInt32"), "uint64": SimTypePointer(SimTypeLongLong(signed=False, label="UInt64"), offset=0), "int8": SimTypeChar(label="SByte"), "int16": SimTypeShort(signed=True, label="Int16"), "int32": SimTypeInt(signed=True, label="Int32"), "int64": SimTypePointer(SimTypeLongLong(signed=True, label="Int64"), offset=0), "float32": SimTypeFloat(size=32), "double64": SimTypePointer(SimTypeFloat(size=64), offset=0), "byteArray16": SimTypePointer(SimStruct({"byteArray16": SimTypeFixedSizeArray(SimTypeChar(label="Byte"), 16)}, name="FWP_BYTE_ARRAY16", pack=False, align=None), offset=0), "byteBlob": SimTypePointer(SimStruct({"size": SimTypeInt(signed=False, label="UInt32"), "data": SimTypePointer(SimTypeChar(label="Byte"), offset=0)}, name="FWP_BYTE_BLOB", pack=False, align=None), offset=0), "sid": SimTypePointer(SimTypeBottom(label="SID"), offset=0), "sd": SimTypePointer(SimStruct({"size": SimTypeInt(signed=False, label="UInt32"), "data": SimTypePointer(SimTypeChar(label="Byte"), offset=0)}, name="FWP_BYTE_BLOB", pack=False, align=None), offset=0), "tokenInformation": SimTypePointer(SimStruct({"sidCount": SimTypeInt(signed=False, label="UInt32"), "sids": SimTypePointer(SimTypeBottom(label="SID_AND_ATTRIBUTES"), offset=0), "restrictedSidCount": SimTypeInt(signed=False, label="UInt32"), "restrictedSids": SimTypePointer(SimTypeBottom(label="SID_AND_ATTRIBUTES"), offset=0)}, name="FWP_TOKEN_INFORMATION", pack=False, align=None), offset=0), "tokenAccessInformation": SimTypePointer(SimStruct({"size": SimTypeInt(signed=False, label="UInt32"), "data": SimTypePointer(SimTypeChar(label="Byte"), offset=0)}, name="FWP_BYTE_BLOB", pack=False, align=None), offset=0), "unicodeString": SimTypePointer(SimTypeChar(label="Char"), offset=0), "byteArray6": SimTypePointer(SimStruct({"byteArray6": SimTypeFixedSizeArray(SimTypeChar(label="Byte"), 6)}, name="FWP_BYTE_ARRAY6", pack=False, align=None), offset=0), "bitmapArray64": SimTypePointer(SimStruct({"bitmapArray64": SimTypeFixedSizeArray(SimTypeChar(label="Byte"), 8)}, name="FWP_BITMAP_ARRAY64_", pack=False, align=None), offset=0)}, name="<anon>", label="None")}, name="FWP_VALUE0", pack=False, align=None), "valueHigh": SimStruct({"type": SimTypeInt(signed=False, label="FWP_DATA_TYPE"), "Anonymous": SimUnion({"uint8": SimTypeChar(label="Byte"), "uint16": SimTypeShort(signed=False, label="UInt16"), "uint32": SimTypeInt(signed=False, label="UInt32"), "uint64": SimTypePointer(SimTypeLongLong(signed=False, label="UInt64"), offset=0), "int8": SimTypeChar(label="SByte"), "int16": SimTypeShort(signed=True, label="Int16"), "int32": SimTypeInt(signed=True, label="Int32"), "int64": SimTypePointer(SimTypeLongLong(signed=True, label="Int64"), offset=0), "float32": SimTypeFloat(size=32), "double64": SimTypePointer(SimTypeFloat(size=64), offset=0), "byteArray16": SimTypePointer(SimStruct({"byteArray16": SimTypeFixedSizeArray(SimTypeChar(label="Byte"), 16)}, name="FWP_BYTE_ARRAY16", pack=False, align=None), offset=0), "byteBlob": SimTypePointer(SimStruct({"size": SimTypeInt(signed=False, label="UInt32"), "data": SimTypePointer(SimTypeChar(label="Byte"), offset=0)}, name="FWP_BYTE_BLOB", pack=False, align=None), offset=0), "sid": SimTypePointer(SimTypeBottom(label="SID"), offset=0), "sd": SimTypePointer(SimStruct({"size": SimTypeInt(signed=False, label="UInt32"), "data": SimTypePointer(SimTypeChar(label="Byte"), offset=0)}, name="FWP_BYTE_BLOB", pack=False, align=None), offset=0), "tokenInformation": SimTypePointer(SimStruct({"sidCount": SimTypeInt(signed=False, label="UInt32"), "sids": SimTypePointer(SimTypeBottom(label="SID_AND_ATTRIBUTES"), offset=0), "restrictedSidCount": SimTypeInt(signed=False, label="UInt32"), "restrictedSids": SimTypePointer(SimTypeBottom(label="SID_AND_ATTRIBUTES"), offset=0)}, name="FWP_TOKEN_INFORMATION", pack=False, align=None), offset=0), "tokenAccessInformation": SimTypePointer(SimStruct({"size": SimTypeInt(signed=False, label="UInt32"), "data": SimTypePointer(SimTypeChar(label="Byte"), offset=0)}, name="FWP_BYTE_BLOB", pack=False, align=None), offset=0), "unicodeString": SimTypePointer(SimTypeChar(label="Char"), offset=0), "byteArray6": SimTypePointer(SimStruct({"byteArray6": SimTypeFixedSizeArray(SimTypeChar(label="Byte"), 6)}, name="FWP_BYTE_ARRAY6", pack=False, align=None), offset=0), "bitmapArray64": SimTypePointer(SimStruct({"bitmapArray64": SimTypeFixedSizeArray(SimTypeChar(label="Byte"), 8)}, name="FWP_BITMAP_ARRAY64_", pack=False, align=None), offset=0)}, name="<anon>", label="None")}, name="FWP_VALUE0", pack=False, align=None)}, name="FWP_RANGE0", pack=False, align=None), offset=0)}, name="<anon>", label="None")}, name="FWP_CONDITION_VALUE0", pack=False, align=None)}, name="FWPM_FILTER_CONDITION0", pack=False, align=None), offset=0), "actionMask": SimTypeInt(signed=False, label="UInt32"), "calloutKey": SimTypePointer(SimTypeBottom(label="Guid"), offset=0)}, name="FWPM_FILTER_ENUM_TEMPLATE0", pack=False, align=None), offset=0), SimTypePointer(SimTypePointer(SimTypeInt(signed=True, label="Int"), label="IntPtr", offset=0), offset=0)], SimTypeInt(signed=False, label="UInt32"), arg_names=["engineHandle", "enumTemplate", "enumHandle"]),
        # 
        'FwpmFilterEnum0': SimTypeFunction([SimTypePointer(SimTypeInt(signed=True, label="Int"), label="IntPtr", offset=0), SimTypePointer(SimTypeInt(signed=True, label="Int"), label="IntPtr", offset=0), SimTypeInt(signed=False, label="UInt32"), SimTypePointer(SimTypePointer(SimTypePointer(SimStruct({"filterKey": SimTypeBottom(label="Guid"), "displayData": SimStruct({"name": SimTypePointer(SimTypeChar(label="Char"), offset=0), "description": SimTypePointer(SimTypeChar(label="Char"), offset=0)}, name="FWPM_DISPLAY_DATA0", pack=False, align=None), "flags": SimTypeInt(signed=False, label="FWPM_FILTER_FLAGS"), "providerKey": SimTypePointer(SimTypeBottom(label="Guid"), offset=0), "providerData": SimStruct({"size": SimTypeInt(signed=False, label="UInt32"), "data": SimTypePointer(SimTypeChar(label="Byte"), offset=0)}, name="FWP_BYTE_BLOB", pack=False, align=None), "layerKey": SimTypeBottom(label="Guid"), "subLayerKey": SimTypeBottom(label="Guid"), "weight": SimStruct({"type": SimTypeInt(signed=False, label="FWP_DATA_TYPE"), "Anonymous": SimUnion({"uint8": SimTypeChar(label="Byte"), "uint16": SimTypeShort(signed=False, label="UInt16"), "uint32": SimTypeInt(signed=False, label="UInt32"), "uint64": SimTypePointer(SimTypeLongLong(signed=False, label="UInt64"), offset=0), "int8": SimTypeChar(label="SByte"), "int16": SimTypeShort(signed=True, label="Int16"), "int32": SimTypeInt(signed=True, label="Int32"), "int64": SimTypePointer(SimTypeLongLong(signed=True, label="Int64"), offset=0), "float32": SimTypeFloat(size=32), "double64": SimTypePointer(SimTypeFloat(size=64), offset=0), "byteArray16": SimTypePointer(SimStruct({"byteArray16": SimTypeFixedSizeArray(SimTypeChar(label="Byte"), 16)}, name="FWP_BYTE_ARRAY16", pack=False, align=None), offset=0), "byteBlob": SimTypePointer(SimStruct({"size": SimTypeInt(signed=False, label="UInt32"), "data": SimTypePointer(SimTypeChar(label="Byte"), offset=0)}, name="FWP_BYTE_BLOB", pack=False, align=None), offset=0), "sid": SimTypePointer(SimTypeBottom(label="SID"), offset=0), "sd": SimTypePointer(SimStruct({"size": SimTypeInt(signed=False, label="UInt32"), "data": SimTypePointer(SimTypeChar(label="Byte"), offset=0)}, name="FWP_BYTE_BLOB", pack=False, align=None), offset=0), "tokenInformation": SimTypePointer(SimStruct({"sidCount": SimTypeInt(signed=False, label="UInt32"), "sids": SimTypePointer(SimTypeBottom(label="SID_AND_ATTRIBUTES"), offset=0), "restrictedSidCount": SimTypeInt(signed=False, label="UInt32"), "restrictedSids": SimTypePointer(SimTypeBottom(label="SID_AND_ATTRIBUTES"), offset=0)}, name="FWP_TOKEN_INFORMATION", pack=False, align=None), offset=0), "tokenAccessInformation": SimTypePointer(SimStruct({"size": SimTypeInt(signed=False, label="UInt32"), "data": SimTypePointer(SimTypeChar(label="Byte"), offset=0)}, name="FWP_BYTE_BLOB", pack=False, align=None), offset=0), "unicodeString": SimTypePointer(SimTypeChar(label="Char"), offset=0), "byteArray6": SimTypePointer(SimStruct({"byteArray6": SimTypeFixedSizeArray(SimTypeChar(label="Byte"), 6)}, name="FWP_BYTE_ARRAY6", pack=False, align=None), offset=0), "bitmapArray64": SimTypePointer(SimStruct({"bitmapArray64": SimTypeFixedSizeArray(SimTypeChar(label="Byte"), 8)}, name="FWP_BITMAP_ARRAY64_", pack=False, align=None), offset=0)}, name="<anon>", label="None")}, name="FWP_VALUE0", pack=False, align=None), "numFilterConditions": SimTypeInt(signed=False, label="UInt32"), "filterCondition": SimTypePointer(SimStruct({"fieldKey": SimTypeBottom(label="Guid"), "matchType": SimTypeInt(signed=False, label="FWP_MATCH_TYPE"), "conditionValue": SimStruct({"type": SimTypeInt(signed=False, label="FWP_DATA_TYPE"), "Anonymous": SimUnion({"uint8": SimTypeChar(label="Byte"), "uint16": SimTypeShort(signed=False, label="UInt16"), "uint32": SimTypeInt(signed=False, label="UInt32"), "uint64": SimTypePointer(SimTypeLongLong(signed=False, label="UInt64"), offset=0), "int8": SimTypeChar(label="SByte"), "int16": SimTypeShort(signed=True, label="Int16"), "int32": SimTypeInt(signed=True, label="Int32"), "int64": SimTypePointer(SimTypeLongLong(signed=True, label="Int64"), offset=0), "float32": SimTypeFloat(size=32), "double64": SimTypePointer(SimTypeFloat(size=64), offset=0), "byteArray16": SimTypePointer(SimStruct({"byteArray16": SimTypeFixedSizeArray(SimTypeChar(label="Byte"), 16)}, name="FWP_BYTE_ARRAY16", pack=False, align=None), offset=0), "byteBlob": SimTypePointer(SimStruct({"size": SimTypeInt(signed=False, label="UInt32"), "data": SimTypePointer(SimTypeChar(label="Byte"), offset=0)}, name="FWP_BYTE_BLOB", pack=False, align=None), offset=0), "sid": SimTypePointer(SimTypeBottom(label="SID"), offset=0), "sd": SimTypePointer(SimStruct({"size": SimTypeInt(signed=False, label="UInt32"), "data": SimTypePointer(SimTypeChar(label="Byte"), offset=0)}, name="FWP_BYTE_BLOB", pack=False, align=None), offset=0), "tokenInformation": SimTypePointer(SimStruct({"sidCount": SimTypeInt(signed=False, label="UInt32"), "sids": SimTypePointer(SimTypeBottom(label="SID_AND_ATTRIBUTES"), offset=0), "restrictedSidCount": SimTypeInt(signed=False, label="UInt32"), "restrictedSids": SimTypePointer(SimTypeBottom(label="SID_AND_ATTRIBUTES"), offset=0)}, name="FWP_TOKEN_INFORMATION", pack=False, align=None), offset=0), "tokenAccessInformation": SimTypePointer(SimStruct({"size": SimTypeInt(signed=False, label="UInt32"), "data": SimTypePointer(SimTypeChar(label="Byte"), offset=0)}, name="FWP_BYTE_BLOB", pack=False, align=None), offset=0), "unicodeString": SimTypePointer(SimTypeChar(label="Char"), offset=0), "byteArray6": SimTypePointer(SimStruct({"byteArray6": SimTypeFixedSizeArray(SimTypeChar(label="Byte"), 6)}, name="FWP_BYTE_ARRAY6", pack=False, align=None), offset=0), "bitmapArray64": SimTypePointer(SimStruct({"bitmapArray64": SimTypeFixedSizeArray(SimTypeChar(label="Byte"), 8)}, name="FWP_BITMAP_ARRAY64_", pack=False, align=None), offset=0), "v4AddrMask": SimTypePointer(SimStruct({"addr": SimTypeInt(signed=False, label="UInt32"), "mask": SimTypeInt(signed=False, label="UInt32")}, name="FWP_V4_ADDR_AND_MASK", pack=False, align=None), offset=0), "v6AddrMask": SimTypePointer(SimStruct({"addr": SimTypeFixedSizeArray(SimTypeChar(label="Byte"), 16), "prefixLength": SimTypeChar(label="Byte")}, name="FWP_V6_ADDR_AND_MASK", pack=False, align=None), offset=0), "rangeValue": SimTypePointer(SimStruct({"valueLow": SimStruct({"type": SimTypeInt(signed=False, label="FWP_DATA_TYPE"), "Anonymous": SimUnion({"uint8": SimTypeChar(label="Byte"), "uint16": SimTypeShort(signed=False, label="UInt16"), "uint32": SimTypeInt(signed=False, label="UInt32"), "uint64": SimTypePointer(SimTypeLongLong(signed=False, label="UInt64"), offset=0), "int8": SimTypeChar(label="SByte"), "int16": SimTypeShort(signed=True, label="Int16"), "int32": SimTypeInt(signed=True, label="Int32"), "int64": SimTypePointer(SimTypeLongLong(signed=True, label="Int64"), offset=0), "float32": SimTypeFloat(size=32), "double64": SimTypePointer(SimTypeFloat(size=64), offset=0), "byteArray16": SimTypePointer(SimStruct({"byteArray16": SimTypeFixedSizeArray(SimTypeChar(label="Byte"), 16)}, name="FWP_BYTE_ARRAY16", pack=False, align=None), offset=0), "byteBlob": SimTypePointer(SimStruct({"size": SimTypeInt(signed=False, label="UInt32"), "data": SimTypePointer(SimTypeChar(label="Byte"), offset=0)}, name="FWP_BYTE_BLOB", pack=False, align=None), offset=0), "sid": SimTypePointer(SimTypeBottom(label="SID"), offset=0), "sd": SimTypePointer(SimStruct({"size": SimTypeInt(signed=False, label="UInt32"), "data": SimTypePointer(SimTypeChar(label="Byte"), offset=0)}, name="FWP_BYTE_BLOB", pack=False, align=None), offset=0), "tokenInformation": SimTypePointer(SimStruct({"sidCount": SimTypeInt(signed=False, label="UInt32"), "sids": SimTypePointer(SimTypeBottom(label="SID_AND_ATTRIBUTES"), offset=0), "restrictedSidCount": SimTypeInt(signed=False, label="UInt32"), "restrictedSids": SimTypePointer(SimTypeBottom(label="SID_AND_ATTRIBUTES"), offset=0)}, name="FWP_TOKEN_INFORMATION", pack=False, align=None), offset=0), "tokenAccessInformation": SimTypePointer(SimStruct({"size": SimTypeInt(signed=False, label="UInt32"), "data": SimTypePointer(SimTypeChar(label="Byte"), offset=0)}, name="FWP_BYTE_BLOB", pack=False, align=None), offset=0), "unicodeString": SimTypePointer(SimTypeChar(label="Char"), offset=0), "byteArray6": SimTypePointer(SimStruct({"byteArray6": SimTypeFixedSizeArray(SimTypeChar(label="Byte"), 6)}, name="FWP_BYTE_ARRAY6", pack=False, align=None), offset=0), "bitmapArray64": SimTypePointer(SimStruct({"bitmapArray64": SimTypeFixedSizeArray(SimTypeChar(label="Byte"), 8)}, name="FWP_BITMAP_ARRAY64_", pack=False, align=None), offset=0)}, name="<anon>", label="None")}, name="FWP_VALUE0", pack=False, align=None), "valueHigh": SimStruct({"type": SimTypeInt(signed=False, label="FWP_DATA_TYPE"), "Anonymous": SimUnion({"uint8": SimTypeChar(label="Byte"), "uint16": SimTypeShort(signed=False, label="UInt16"), "uint32": SimTypeInt(signed=False, label="UInt32"), "uint64": SimTypePointer(SimTypeLongLong(signed=False, label="UInt64"), offset=0), "int8": SimTypeChar(label="SByte"), "int16": SimTypeShort(signed=True, label="Int16"), "int32": SimTypeInt(signed=True, label="Int32"), "int64": SimTypePointer(SimTypeLongLong(signed=True, label="Int64"), offset=0), "float32": SimTypeFloat(size=32), "double64": SimTypePointer(SimTypeFloat(size=64), offset=0), "byteArray16": SimTypePointer(SimStruct({"byteArray16": SimTypeFixedSizeArray(SimTypeChar(label="Byte"), 16)}, name="FWP_BYTE_ARRAY16", pack=False, align=None), offset=0), "byteBlob": SimTypePointer(SimStruct({"size": SimTypeInt(signed=False, label="UInt32"), "data": SimTypePointer(SimTypeChar(label="Byte"), offset=0)}, name="FWP_BYTE_BLOB", pack=False, align=None), offset=0), "sid": SimTypePointer(SimTypeBottom(label="SID"), offset=0), "sd": SimTypePointer(SimStruct({"size": SimTypeInt(signed=False, label="UInt32"), "data": SimTypePointer(SimTypeChar(label="Byte"), offset=0)}, name="FWP_BYTE_BLOB", pack=False, align=None), offset=0), "tokenInformation": SimTypePointer(SimStruct({"sidCount": SimTypeInt(signed=False, label="UInt32"), "sids": SimTypePointer(SimTypeBottom(label="SID_AND_ATTRIBUTES"), offset=0), "restrictedSidCount": SimTypeInt(signed=False, label="UInt32"), "restrictedSids": SimTypePointer(SimTypeBottom(label="SID_AND_ATTRIBUTES"), offset=0)}, name="FWP_TOKEN_INFORMATION", pack=False, align=None), offset=0), "tokenAccessInformation": SimTypePointer(SimStruct({"size": SimTypeInt(signed=False, label="UInt32"), "data": SimTypePointer(SimTypeChar(label="Byte"), offset=0)}, name="FWP_BYTE_BLOB", pack=False, align=None), offset=0), "unicodeString": SimTypePointer(SimTypeChar(label="Char"), offset=0), "byteArray6": SimTypePointer(SimStruct({"byteArray6": SimTypeFixedSizeArray(SimTypeChar(label="Byte"), 6)}, name="FWP_BYTE_ARRAY6", pack=False, align=None), offset=0), "bitmapArray64": SimTypePointer(SimStruct({"bitmapArray64": SimTypeFixedSizeArray(SimTypeChar(label="Byte"), 8)}, name="FWP_BITMAP_ARRAY64_", pack=False, align=None), offset=0)}, name="<anon>", label="None")}, name="FWP_VALUE0", pack=False, align=None)}, name="FWP_RANGE0", pack=False, align=None), offset=0)}, name="<anon>", label="None")}, name="FWP_CONDITION_VALUE0", pack=False, align=None)}, name="FWPM_FILTER_CONDITION0", pack=False, align=None), offset=0), "action": SimStruct({"type": SimTypeInt(signed=False, label="UInt32"), "Anonymous": SimUnion({"filterType": SimTypeBottom(label="Guid"), "calloutKey": SimTypeBottom(label="Guid"), "bitmapIndex": SimTypeChar(label="Byte")}, name="<anon>", label="None")}, name="FWPM_ACTION0", pack=False, align=None), "Anonymous": SimUnion({"rawContext": SimTypeLongLong(signed=False, label="UInt64"), "providerContextKey": SimTypeBottom(label="Guid")}, name="<anon>", label="None"), "reserved": SimTypePointer(SimTypeBottom(label="Guid"), offset=0), "filterId": SimTypeLongLong(signed=False, label="UInt64"), "effectiveWeight": SimStruct({"type": SimTypeInt(signed=False, label="FWP_DATA_TYPE"), "Anonymous": SimUnion({"uint8": SimTypeChar(label="Byte"), "uint16": SimTypeShort(signed=False, label="UInt16"), "uint32": SimTypeInt(signed=False, label="UInt32"), "uint64": SimTypePointer(SimTypeLongLong(signed=False, label="UInt64"), offset=0), "int8": SimTypeChar(label="SByte"), "int16": SimTypeShort(signed=True, label="Int16"), "int32": SimTypeInt(signed=True, label="Int32"), "int64": SimTypePointer(SimTypeLongLong(signed=True, label="Int64"), offset=0), "float32": SimTypeFloat(size=32), "double64": SimTypePointer(SimTypeFloat(size=64), offset=0), "byteArray16": SimTypePointer(SimStruct({"byteArray16": SimTypeFixedSizeArray(SimTypeChar(label="Byte"), 16)}, name="FWP_BYTE_ARRAY16", pack=False, align=None), offset=0), "byteBlob": SimTypePointer(SimStruct({"size": SimTypeInt(signed=False, label="UInt32"), "data": SimTypePointer(SimTypeChar(label="Byte"), offset=0)}, name="FWP_BYTE_BLOB", pack=False, align=None), offset=0), "sid": SimTypePointer(SimTypeBottom(label="SID"), offset=0), "sd": SimTypePointer(SimStruct({"size": SimTypeInt(signed=False, label="UInt32"), "data": SimTypePointer(SimTypeChar(label="Byte"), offset=0)}, name="FWP_BYTE_BLOB", pack=False, align=None), offset=0), "tokenInformation": SimTypePointer(SimStruct({"sidCount": SimTypeInt(signed=False, label="UInt32"), "sids": SimTypePointer(SimTypeBottom(label="SID_AND_ATTRIBUTES"), offset=0), "restrictedSidCount": SimTypeInt(signed=False, label="UInt32"), "restrictedSids": SimTypePointer(SimTypeBottom(label="SID_AND_ATTRIBUTES"), offset=0)}, name="FWP_TOKEN_INFORMATION", pack=False, align=None), offset=0), "tokenAccessInformation": SimTypePointer(SimStruct({"size": SimTypeInt(signed=False, label="UInt32"), "data": SimTypePointer(SimTypeChar(label="Byte"), offset=0)}, name="FWP_BYTE_BLOB", pack=False, align=None), offset=0), "unicodeString": SimTypePointer(SimTypeChar(label="Char"), offset=0), "byteArray6": SimTypePointer(SimStruct({"byteArray6": SimTypeFixedSizeArray(SimTypeChar(label="Byte"), 6)}, name="FWP_BYTE_ARRAY6", pack=False, align=None), offset=0), "bitmapArray64": SimTypePointer(SimStruct({"bitmapArray64": SimTypeFixedSizeArray(SimTypeChar(label="Byte"), 8)}, name="FWP_BITMAP_ARRAY64_", pack=False, align=None), offset=0)}, name="<anon>", label="None")}, name="FWP_VALUE0", pack=False, align=None)}, name="FWPM_FILTER0", pack=False, align=None), offset=0), offset=0), offset=0), SimTypePointer(SimTypeInt(signed=False, label="UInt32"), offset=0)], SimTypeInt(signed=False, label="UInt32"), arg_names=["engineHandle", "enumHandle", "numEntriesRequested", "entries", "numEntriesReturned"]),
        # 
        'FwpmFilterDestroyEnumHandle0': SimTypeFunction([SimTypePointer(SimTypeInt(signed=True, label="Int"), label="IntPtr", offset=0), SimTypePointer(SimTypeInt(signed=True, label="Int"), label="IntPtr", offset=0)], SimTypeInt(signed=False, label="UInt32"), arg_names=["engineHandle", "enumHandle"]),
        # 
        'FwpmFilterGetSecurityInfoByKey0': SimTypeFunction([SimTypePointer(SimTypeInt(signed=True, label="Int"), label="IntPtr", offset=0), SimTypePointer(SimTypeBottom(label="Guid"), offset=0), SimTypeInt(signed=False, label="UInt32"), SimTypePointer(SimTypePointer(SimTypeInt(signed=True, label="Int"), label="IntPtr", offset=0), offset=0), SimTypePointer(SimTypePointer(SimTypeInt(signed=True, label="Int"), label="IntPtr", offset=0), offset=0), SimTypePointer(SimTypePointer(SimStruct({"AclRevision": SimTypeChar(label="Byte"), "Sbz1": SimTypeChar(label="Byte"), "AclSize": SimTypeShort(signed=False, label="UInt16"), "AceCount": SimTypeShort(signed=False, label="UInt16"), "Sbz2": SimTypeShort(signed=False, label="UInt16")}, name="ACL", pack=False, align=None), offset=0), offset=0), SimTypePointer(SimTypePointer(SimStruct({"AclRevision": SimTypeChar(label="Byte"), "Sbz1": SimTypeChar(label="Byte"), "AclSize": SimTypeShort(signed=False, label="UInt16"), "AceCount": SimTypeShort(signed=False, label="UInt16"), "Sbz2": SimTypeShort(signed=False, label="UInt16")}, name="ACL", pack=False, align=None), offset=0), offset=0), SimTypePointer(SimTypePointer(SimStruct({"Revision": SimTypeChar(label="Byte"), "Sbz1": SimTypeChar(label="Byte"), "Control": SimTypeShort(signed=False, label="UInt16"), "Owner": SimTypePointer(SimTypeInt(signed=True, label="Int"), label="IntPtr", offset=0), "Group": SimTypePointer(SimTypeInt(signed=True, label="Int"), label="IntPtr", offset=0), "Sacl": SimTypePointer(SimStruct({"AclRevision": SimTypeChar(label="Byte"), "Sbz1": SimTypeChar(label="Byte"), "AclSize": SimTypeShort(signed=False, label="UInt16"), "AceCount": SimTypeShort(signed=False, label="UInt16"), "Sbz2": SimTypeShort(signed=False, label="UInt16")}, name="ACL", pack=False, align=None), offset=0), "Dacl": SimTypePointer(SimStruct({"AclRevision": SimTypeChar(label="Byte"), "Sbz1": SimTypeChar(label="Byte"), "AclSize": SimTypeShort(signed=False, label="UInt16"), "AceCount": SimTypeShort(signed=False, label="UInt16"), "Sbz2": SimTypeShort(signed=False, label="UInt16")}, name="ACL", pack=False, align=None), offset=0)}, name="SECURITY_DESCRIPTOR", pack=False, align=None), offset=0), offset=0)], SimTypeInt(signed=False, label="UInt32"), arg_names=["engineHandle", "key", "securityInfo", "sidOwner", "sidGroup", "dacl", "sacl", "securityDescriptor"]),
        # 
        'FwpmFilterSetSecurityInfoByKey0': SimTypeFunction([SimTypePointer(SimTypeInt(signed=True, label="Int"), label="IntPtr", offset=0), SimTypePointer(SimTypeBottom(label="Guid"), offset=0), SimTypeInt(signed=False, label="UInt32"), SimTypePointer(SimStruct({"Revision": SimTypeChar(label="Byte"), "SubAuthorityCount": SimTypeChar(label="Byte"), "IdentifierAuthority": SimStruct({"Value": SimTypeFixedSizeArray(SimTypeChar(label="Byte"), 6)}, name="SID_IDENTIFIER_AUTHORITY", pack=False, align=None), "SubAuthority": SimTypePointer(SimTypeInt(signed=False, label="UInt32"), offset=0)}, name="SID", pack=False, align=None), offset=0), SimTypePointer(SimStruct({"Revision": SimTypeChar(label="Byte"), "SubAuthorityCount": SimTypeChar(label="Byte"), "IdentifierAuthority": SimStruct({"Value": SimTypeFixedSizeArray(SimTypeChar(label="Byte"), 6)}, name="SID_IDENTIFIER_AUTHORITY", pack=False, align=None), "SubAuthority": SimTypePointer(SimTypeInt(signed=False, label="UInt32"), offset=0)}, name="SID", pack=False, align=None), offset=0), SimTypePointer(SimStruct({"AclRevision": SimTypeChar(label="Byte"), "Sbz1": SimTypeChar(label="Byte"), "AclSize": SimTypeShort(signed=False, label="UInt16"), "AceCount": SimTypeShort(signed=False, label="UInt16"), "Sbz2": SimTypeShort(signed=False, label="UInt16")}, name="ACL", pack=False, align=None), offset=0), SimTypePointer(SimStruct({"AclRevision": SimTypeChar(label="Byte"), "Sbz1": SimTypeChar(label="Byte"), "AclSize": SimTypeShort(signed=False, label="UInt16"), "AceCount": SimTypeShort(signed=False, label="UInt16"), "Sbz2": SimTypeShort(signed=False, label="UInt16")}, name="ACL", pack=False, align=None), offset=0)], SimTypeInt(signed=False, label="UInt32"), arg_names=["engineHandle", "key", "securityInfo", "sidOwner", "sidGroup", "dacl", "sacl"]),
        # 
        'FwpmFilterSubscribeChanges0': SimTypeFunction([SimTypePointer(SimTypeInt(signed=True, label="Int"), label="IntPtr", offset=0), SimTypePointer(SimStruct({"enumTemplate": SimTypePointer(SimStruct({"providerKey": SimTypePointer(SimTypeBottom(label="Guid"), offset=0), "layerKey": SimTypeBottom(label="Guid"), "enumType": SimTypeInt(signed=False, label="FWP_FILTER_ENUM_TYPE"), "flags": SimTypeInt(signed=False, label="UInt32"), "providerContextTemplate": SimTypePointer(SimStruct({"providerKey": SimTypePointer(SimTypeBottom(label="Guid"), offset=0), "providerContextType": SimTypeInt(signed=False, label="FWPM_PROVIDER_CONTEXT_TYPE")}, name="FWPM_PROVIDER_CONTEXT_ENUM_TEMPLATE0", pack=False, align=None), offset=0), "numFilterConditions": SimTypeInt(signed=False, label="UInt32"), "filterCondition": SimTypePointer(SimStruct({"fieldKey": SimTypeBottom(label="Guid"), "matchType": SimTypeInt(signed=False, label="FWP_MATCH_TYPE"), "conditionValue": SimStruct({"type": SimTypeInt(signed=False, label="FWP_DATA_TYPE"), "Anonymous": SimUnion({"uint8": SimTypeChar(label="Byte"), "uint16": SimTypeShort(signed=False, label="UInt16"), "uint32": SimTypeInt(signed=False, label="UInt32"), "uint64": SimTypePointer(SimTypeLongLong(signed=False, label="UInt64"), offset=0), "int8": SimTypeChar(label="SByte"), "int16": SimTypeShort(signed=True, label="Int16"), "int32": SimTypeInt(signed=True, label="Int32"), "int64": SimTypePointer(SimTypeLongLong(signed=True, label="Int64"), offset=0), "float32": SimTypeFloat(size=32), "double64": SimTypePointer(SimTypeFloat(size=64), offset=0), "byteArray16": SimTypePointer(SimStruct({"byteArray16": SimTypeFixedSizeArray(SimTypeChar(label="Byte"), 16)}, name="FWP_BYTE_ARRAY16", pack=False, align=None), offset=0), "byteBlob": SimTypePointer(SimStruct({"size": SimTypeInt(signed=False, label="UInt32"), "data": SimTypePointer(SimTypeChar(label="Byte"), offset=0)}, name="FWP_BYTE_BLOB", pack=False, align=None), offset=0), "sid": SimTypePointer(SimTypeBottom(label="SID"), offset=0), "sd": SimTypePointer(SimStruct({"size": SimTypeInt(signed=False, label="UInt32"), "data": SimTypePointer(SimTypeChar(label="Byte"), offset=0)}, name="FWP_BYTE_BLOB", pack=False, align=None), offset=0), "tokenInformation": SimTypePointer(SimStruct({"sidCount": SimTypeInt(signed=False, label="UInt32"), "sids": SimTypePointer(SimTypeBottom(label="SID_AND_ATTRIBUTES"), offset=0), "restrictedSidCount": SimTypeInt(signed=False, label="UInt32"), "restrictedSids": SimTypePointer(SimTypeBottom(label="SID_AND_ATTRIBUTES"), offset=0)}, name="FWP_TOKEN_INFORMATION", pack=False, align=None), offset=0), "tokenAccessInformation": SimTypePointer(SimStruct({"size": SimTypeInt(signed=False, label="UInt32"), "data": SimTypePointer(SimTypeChar(label="Byte"), offset=0)}, name="FWP_BYTE_BLOB", pack=False, align=None), offset=0), "unicodeString": SimTypePointer(SimTypeChar(label="Char"), offset=0), "byteArray6": SimTypePointer(SimStruct({"byteArray6": SimTypeFixedSizeArray(SimTypeChar(label="Byte"), 6)}, name="FWP_BYTE_ARRAY6", pack=False, align=None), offset=0), "bitmapArray64": SimTypePointer(SimStruct({"bitmapArray64": SimTypeFixedSizeArray(SimTypeChar(label="Byte"), 8)}, name="FWP_BITMAP_ARRAY64_", pack=False, align=None), offset=0), "v4AddrMask": SimTypePointer(SimStruct({"addr": SimTypeInt(signed=False, label="UInt32"), "mask": SimTypeInt(signed=False, label="UInt32")}, name="FWP_V4_ADDR_AND_MASK", pack=False, align=None), offset=0), "v6AddrMask": SimTypePointer(SimStruct({"addr": SimTypeFixedSizeArray(SimTypeChar(label="Byte"), 16), "prefixLength": SimTypeChar(label="Byte")}, name="FWP_V6_ADDR_AND_MASK", pack=False, align=None), offset=0), "rangeValue": SimTypePointer(SimStruct({"valueLow": SimStruct({"type": SimTypeInt(signed=False, label="FWP_DATA_TYPE"), "Anonymous": SimUnion({"uint8": SimTypeChar(label="Byte"), "uint16": SimTypeShort(signed=False, label="UInt16"), "uint32": SimTypeInt(signed=False, label="UInt32"), "uint64": SimTypePointer(SimTypeLongLong(signed=False, label="UInt64"), offset=0), "int8": SimTypeChar(label="SByte"), "int16": SimTypeShort(signed=True, label="Int16"), "int32": SimTypeInt(signed=True, label="Int32"), "int64": SimTypePointer(SimTypeLongLong(signed=True, label="Int64"), offset=0), "float32": SimTypeFloat(size=32), "double64": SimTypePointer(SimTypeFloat(size=64), offset=0), "byteArray16": SimTypePointer(SimStruct({"byteArray16": SimTypeFixedSizeArray(SimTypeChar(label="Byte"), 16)}, name="FWP_BYTE_ARRAY16", pack=False, align=None), offset=0), "byteBlob": SimTypePointer(SimStruct({"size": SimTypeInt(signed=False, label="UInt32"), "data": SimTypePointer(SimTypeChar(label="Byte"), offset=0)}, name="FWP_BYTE_BLOB", pack=False, align=None), offset=0), "sid": SimTypePointer(SimTypeBottom(label="SID"), offset=0), "sd": SimTypePointer(SimStruct({"size": SimTypeInt(signed=False, label="UInt32"), "data": SimTypePointer(SimTypeChar(label="Byte"), offset=0)}, name="FWP_BYTE_BLOB", pack=False, align=None), offset=0), "tokenInformation": SimTypePointer(SimStruct({"sidCount": SimTypeInt(signed=False, label="UInt32"), "sids": SimTypePointer(SimTypeBottom(label="SID_AND_ATTRIBUTES"), offset=0), "restrictedSidCount": SimTypeInt(signed=False, label="UInt32"), "restrictedSids": SimTypePointer(SimTypeBottom(label="SID_AND_ATTRIBUTES"), offset=0)}, name="FWP_TOKEN_INFORMATION", pack=False, align=None), offset=0), "tokenAccessInformation": SimTypePointer(SimStruct({"size": SimTypeInt(signed=False, label="UInt32"), "data": SimTypePointer(SimTypeChar(label="Byte"), offset=0)}, name="FWP_BYTE_BLOB", pack=False, align=None), offset=0), "unicodeString": SimTypePointer(SimTypeChar(label="Char"), offset=0), "byteArray6": SimTypePointer(SimStruct({"byteArray6": SimTypeFixedSizeArray(SimTypeChar(label="Byte"), 6)}, name="FWP_BYTE_ARRAY6", pack=False, align=None), offset=0), "bitmapArray64": SimTypePointer(SimStruct({"bitmapArray64": SimTypeFixedSizeArray(SimTypeChar(label="Byte"), 8)}, name="FWP_BITMAP_ARRAY64_", pack=False, align=None), offset=0)}, name="<anon>", label="None")}, name="FWP_VALUE0", pack=False, align=None), "valueHigh": SimStruct({"type": SimTypeInt(signed=False, label="FWP_DATA_TYPE"), "Anonymous": SimUnion({"uint8": SimTypeChar(label="Byte"), "uint16": SimTypeShort(signed=False, label="UInt16"), "uint32": SimTypeInt(signed=False, label="UInt32"), "uint64": SimTypePointer(SimTypeLongLong(signed=False, label="UInt64"), offset=0), "int8": SimTypeChar(label="SByte"), "int16": SimTypeShort(signed=True, label="Int16"), "int32": SimTypeInt(signed=True, label="Int32"), "int64": SimTypePointer(SimTypeLongLong(signed=True, label="Int64"), offset=0), "float32": SimTypeFloat(size=32), "double64": SimTypePointer(SimTypeFloat(size=64), offset=0), "byteArray16": SimTypePointer(SimStruct({"byteArray16": SimTypeFixedSizeArray(SimTypeChar(label="Byte"), 16)}, name="FWP_BYTE_ARRAY16", pack=False, align=None), offset=0), "byteBlob": SimTypePointer(SimStruct({"size": SimTypeInt(signed=False, label="UInt32"), "data": SimTypePointer(SimTypeChar(label="Byte"), offset=0)}, name="FWP_BYTE_BLOB", pack=False, align=None), offset=0), "sid": SimTypePointer(SimTypeBottom(label="SID"), offset=0), "sd": SimTypePointer(SimStruct({"size": SimTypeInt(signed=False, label="UInt32"), "data": SimTypePointer(SimTypeChar(label="Byte"), offset=0)}, name="FWP_BYTE_BLOB", pack=False, align=None), offset=0), "tokenInformation": SimTypePointer(SimStruct({"sidCount": SimTypeInt(signed=False, label="UInt32"), "sids": SimTypePointer(SimTypeBottom(label="SID_AND_ATTRIBUTES"), offset=0), "restrictedSidCount": SimTypeInt(signed=False, label="UInt32"), "restrictedSids": SimTypePointer(SimTypeBottom(label="SID_AND_ATTRIBUTES"), offset=0)}, name="FWP_TOKEN_INFORMATION", pack=False, align=None), offset=0), "tokenAccessInformation": SimTypePointer(SimStruct({"size": SimTypeInt(signed=False, label="UInt32"), "data": SimTypePointer(SimTypeChar(label="Byte"), offset=0)}, name="FWP_BYTE_BLOB", pack=False, align=None), offset=0), "unicodeString": SimTypePointer(SimTypeChar(label="Char"), offset=0), "byteArray6": SimTypePointer(SimStruct({"byteArray6": SimTypeFixedSizeArray(SimTypeChar(label="Byte"), 6)}, name="FWP_BYTE_ARRAY6", pack=False, align=None), offset=0), "bitmapArray64": SimTypePointer(SimStruct({"bitmapArray64": SimTypeFixedSizeArray(SimTypeChar(label="Byte"), 8)}, name="FWP_BITMAP_ARRAY64_", pack=False, align=None), offset=0)}, name="<anon>", label="None")}, name="FWP_VALUE0", pack=False, align=None)}, name="FWP_RANGE0", pack=False, align=None), offset=0)}, name="<anon>", label="None")}, name="FWP_CONDITION_VALUE0", pack=False, align=None)}, name="FWPM_FILTER_CONDITION0", pack=False, align=None), offset=0), "actionMask": SimTypeInt(signed=False, label="UInt32"), "calloutKey": SimTypePointer(SimTypeBottom(label="Guid"), offset=0)}, name="FWPM_FILTER_ENUM_TEMPLATE0", pack=False, align=None), offset=0), "flags": SimTypeInt(signed=False, label="UInt32"), "sessionKey": SimTypeBottom(label="Guid")}, name="FWPM_FILTER_SUBSCRIPTION0", pack=False, align=None), offset=0), SimTypePointer(SimTypeFunction([SimTypePointer(SimTypeBottom(label="Void"), offset=0), SimTypePointer(SimStruct({"changeType": SimTypeInt(signed=False, label="FWPM_CHANGE_TYPE"), "filterKey": SimTypeBottom(label="Guid"), "filterId": SimTypeLongLong(signed=False, label="UInt64")}, name="FWPM_FILTER_CHANGE0", pack=False, align=None), offset=0)], SimTypeBottom(label="Void"), arg_names=["context", "change"]), offset=0), SimTypePointer(SimTypeBottom(label="Void"), offset=0), SimTypePointer(SimTypePointer(SimTypeInt(signed=True, label="Int"), label="IntPtr", offset=0), offset=0)], SimTypeInt(signed=False, label="UInt32"), arg_names=["engineHandle", "subscription", "callback", "context", "changeHandle"]),
        # 
        'FwpmFilterUnsubscribeChanges0': SimTypeFunction([SimTypePointer(SimTypeInt(signed=True, label="Int"), label="IntPtr", offset=0), SimTypePointer(SimTypeInt(signed=True, label="Int"), label="IntPtr", offset=0)], SimTypeInt(signed=False, label="UInt32"), arg_names=["engineHandle", "changeHandle"]),
        # 
        'FwpmFilterSubscriptionsGet0': SimTypeFunction([SimTypePointer(SimTypeInt(signed=True, label="Int"), label="IntPtr", offset=0), SimTypePointer(SimTypePointer(SimTypePointer(SimStruct({"enumTemplate": SimTypePointer(SimStruct({"providerKey": SimTypePointer(SimTypeBottom(label="Guid"), offset=0), "layerKey": SimTypeBottom(label="Guid"), "enumType": SimTypeInt(signed=False, label="FWP_FILTER_ENUM_TYPE"), "flags": SimTypeInt(signed=False, label="UInt32"), "providerContextTemplate": SimTypePointer(SimStruct({"providerKey": SimTypePointer(SimTypeBottom(label="Guid"), offset=0), "providerContextType": SimTypeInt(signed=False, label="FWPM_PROVIDER_CONTEXT_TYPE")}, name="FWPM_PROVIDER_CONTEXT_ENUM_TEMPLATE0", pack=False, align=None), offset=0), "numFilterConditions": SimTypeInt(signed=False, label="UInt32"), "filterCondition": SimTypePointer(SimStruct({"fieldKey": SimTypeBottom(label="Guid"), "matchType": SimTypeInt(signed=False, label="FWP_MATCH_TYPE"), "conditionValue": SimStruct({"type": SimTypeInt(signed=False, label="FWP_DATA_TYPE"), "Anonymous": SimUnion({"uint8": SimTypeChar(label="Byte"), "uint16": SimTypeShort(signed=False, label="UInt16"), "uint32": SimTypeInt(signed=False, label="UInt32"), "uint64": SimTypePointer(SimTypeLongLong(signed=False, label="UInt64"), offset=0), "int8": SimTypeChar(label="SByte"), "int16": SimTypeShort(signed=True, label="Int16"), "int32": SimTypeInt(signed=True, label="Int32"), "int64": SimTypePointer(SimTypeLongLong(signed=True, label="Int64"), offset=0), "float32": SimTypeFloat(size=32), "double64": SimTypePointer(SimTypeFloat(size=64), offset=0), "byteArray16": SimTypePointer(SimStruct({"byteArray16": SimTypeFixedSizeArray(SimTypeChar(label="Byte"), 16)}, name="FWP_BYTE_ARRAY16", pack=False, align=None), offset=0), "byteBlob": SimTypePointer(SimStruct({"size": SimTypeInt(signed=False, label="UInt32"), "data": SimTypePointer(SimTypeChar(label="Byte"), offset=0)}, name="FWP_BYTE_BLOB", pack=False, align=None), offset=0), "sid": SimTypePointer(SimTypeBottom(label="SID"), offset=0), "sd": SimTypePointer(SimStruct({"size": SimTypeInt(signed=False, label="UInt32"), "data": SimTypePointer(SimTypeChar(label="Byte"), offset=0)}, name="FWP_BYTE_BLOB", pack=False, align=None), offset=0), "tokenInformation": SimTypePointer(SimStruct({"sidCount": SimTypeInt(signed=False, label="UInt32"), "sids": SimTypePointer(SimTypeBottom(label="SID_AND_ATTRIBUTES"), offset=0), "restrictedSidCount": SimTypeInt(signed=False, label="UInt32"), "restrictedSids": SimTypePointer(SimTypeBottom(label="SID_AND_ATTRIBUTES"), offset=0)}, name="FWP_TOKEN_INFORMATION", pack=False, align=None), offset=0), "tokenAccessInformation": SimTypePointer(SimStruct({"size": SimTypeInt(signed=False, label="UInt32"), "data": SimTypePointer(SimTypeChar(label="Byte"), offset=0)}, name="FWP_BYTE_BLOB", pack=False, align=None), offset=0), "unicodeString": SimTypePointer(SimTypeChar(label="Char"), offset=0), "byteArray6": SimTypePointer(SimStruct({"byteArray6": SimTypeFixedSizeArray(SimTypeChar(label="Byte"), 6)}, name="FWP_BYTE_ARRAY6", pack=False, align=None), offset=0), "bitmapArray64": SimTypePointer(SimStruct({"bitmapArray64": SimTypeFixedSizeArray(SimTypeChar(label="Byte"), 8)}, name="FWP_BITMAP_ARRAY64_", pack=False, align=None), offset=0), "v4AddrMask": SimTypePointer(SimStruct({"addr": SimTypeInt(signed=False, label="UInt32"), "mask": SimTypeInt(signed=False, label="UInt32")}, name="FWP_V4_ADDR_AND_MASK", pack=False, align=None), offset=0), "v6AddrMask": SimTypePointer(SimStruct({"addr": SimTypeFixedSizeArray(SimTypeChar(label="Byte"), 16), "prefixLength": SimTypeChar(label="Byte")}, name="FWP_V6_ADDR_AND_MASK", pack=False, align=None), offset=0), "rangeValue": SimTypePointer(SimStruct({"valueLow": SimStruct({"type": SimTypeInt(signed=False, label="FWP_DATA_TYPE"), "Anonymous": SimUnion({"uint8": SimTypeChar(label="Byte"), "uint16": SimTypeShort(signed=False, label="UInt16"), "uint32": SimTypeInt(signed=False, label="UInt32"), "uint64": SimTypePointer(SimTypeLongLong(signed=False, label="UInt64"), offset=0), "int8": SimTypeChar(label="SByte"), "int16": SimTypeShort(signed=True, label="Int16"), "int32": SimTypeInt(signed=True, label="Int32"), "int64": SimTypePointer(SimTypeLongLong(signed=True, label="Int64"), offset=0), "float32": SimTypeFloat(size=32), "double64": SimTypePointer(SimTypeFloat(size=64), offset=0), "byteArray16": SimTypePointer(SimStruct({"byteArray16": SimTypeFixedSizeArray(SimTypeChar(label="Byte"), 16)}, name="FWP_BYTE_ARRAY16", pack=False, align=None), offset=0), "byteBlob": SimTypePointer(SimStruct({"size": SimTypeInt(signed=False, label="UInt32"), "data": SimTypePointer(SimTypeChar(label="Byte"), offset=0)}, name="FWP_BYTE_BLOB", pack=False, align=None), offset=0), "sid": SimTypePointer(SimTypeBottom(label="SID"), offset=0), "sd": SimTypePointer(SimStruct({"size": SimTypeInt(signed=False, label="UInt32"), "data": SimTypePointer(SimTypeChar(label="Byte"), offset=0)}, name="FWP_BYTE_BLOB", pack=False, align=None), offset=0), "tokenInformation": SimTypePointer(SimStruct({"sidCount": SimTypeInt(signed=False, label="UInt32"), "sids": SimTypePointer(SimTypeBottom(label="SID_AND_ATTRIBUTES"), offset=0), "restrictedSidCount": SimTypeInt(signed=False, label="UInt32"), "restrictedSids": SimTypePointer(SimTypeBottom(label="SID_AND_ATTRIBUTES"), offset=0)}, name="FWP_TOKEN_INFORMATION", pack=False, align=None), offset=0), "tokenAccessInformation": SimTypePointer(SimStruct({"size": SimTypeInt(signed=False, label="UInt32"), "data": SimTypePointer(SimTypeChar(label="Byte"), offset=0)}, name="FWP_BYTE_BLOB", pack=False, align=None), offset=0), "unicodeString": SimTypePointer(SimTypeChar(label="Char"), offset=0), "byteArray6": SimTypePointer(SimStruct({"byteArray6": SimTypeFixedSizeArray(SimTypeChar(label="Byte"), 6)}, name="FWP_BYTE_ARRAY6", pack=False, align=None), offset=0), "bitmapArray64": SimTypePointer(SimStruct({"bitmapArray64": SimTypeFixedSizeArray(SimTypeChar(label="Byte"), 8)}, name="FWP_BITMAP_ARRAY64_", pack=False, align=None), offset=0)}, name="<anon>", label="None")}, name="FWP_VALUE0", pack=False, align=None), "valueHigh": SimStruct({"type": SimTypeInt(signed=False, label="FWP_DATA_TYPE"), "Anonymous": SimUnion({"uint8": SimTypeChar(label="Byte"), "uint16": SimTypeShort(signed=False, label="UInt16"), "uint32": SimTypeInt(signed=False, label="UInt32"), "uint64": SimTypePointer(SimTypeLongLong(signed=False, label="UInt64"), offset=0), "int8": SimTypeChar(label="SByte"), "int16": SimTypeShort(signed=True, label="Int16"), "int32": SimTypeInt(signed=True, label="Int32"), "int64": SimTypePointer(SimTypeLongLong(signed=True, label="Int64"), offset=0), "float32": SimTypeFloat(size=32), "double64": SimTypePointer(SimTypeFloat(size=64), offset=0), "byteArray16": SimTypePointer(SimStruct({"byteArray16": SimTypeFixedSizeArray(SimTypeChar(label="Byte"), 16)}, name="FWP_BYTE_ARRAY16", pack=False, align=None), offset=0), "byteBlob": SimTypePointer(SimStruct({"size": SimTypeInt(signed=False, label="UInt32"), "data": SimTypePointer(SimTypeChar(label="Byte"), offset=0)}, name="FWP_BYTE_BLOB", pack=False, align=None), offset=0), "sid": SimTypePointer(SimTypeBottom(label="SID"), offset=0), "sd": SimTypePointer(SimStruct({"size": SimTypeInt(signed=False, label="UInt32"), "data": SimTypePointer(SimTypeChar(label="Byte"), offset=0)}, name="FWP_BYTE_BLOB", pack=False, align=None), offset=0), "tokenInformation": SimTypePointer(SimStruct({"sidCount": SimTypeInt(signed=False, label="UInt32"), "sids": SimTypePointer(SimTypeBottom(label="SID_AND_ATTRIBUTES"), offset=0), "restrictedSidCount": SimTypeInt(signed=False, label="UInt32"), "restrictedSids": SimTypePointer(SimTypeBottom(label="SID_AND_ATTRIBUTES"), offset=0)}, name="FWP_TOKEN_INFORMATION", pack=False, align=None), offset=0), "tokenAccessInformation": SimTypePointer(SimStruct({"size": SimTypeInt(signed=False, label="UInt32"), "data": SimTypePointer(SimTypeChar(label="Byte"), offset=0)}, name="FWP_BYTE_BLOB", pack=False, align=None), offset=0), "unicodeString": SimTypePointer(SimTypeChar(label="Char"), offset=0), "byteArray6": SimTypePointer(SimStruct({"byteArray6": SimTypeFixedSizeArray(SimTypeChar(label="Byte"), 6)}, name="FWP_BYTE_ARRAY6", pack=False, align=None), offset=0), "bitmapArray64": SimTypePointer(SimStruct({"bitmapArray64": SimTypeFixedSizeArray(SimTypeChar(label="Byte"), 8)}, name="FWP_BITMAP_ARRAY64_", pack=False, align=None), offset=0)}, name="<anon>", label="None")}, name="FWP_VALUE0", pack=False, align=None)}, name="FWP_RANGE0", pack=False, align=None), offset=0)}, name="<anon>", label="None")}, name="FWP_CONDITION_VALUE0", pack=False, align=None)}, name="FWPM_FILTER_CONDITION0", pack=False, align=None), offset=0), "actionMask": SimTypeInt(signed=False, label="UInt32"), "calloutKey": SimTypePointer(SimTypeBottom(label="Guid"), offset=0)}, name="FWPM_FILTER_ENUM_TEMPLATE0", pack=False, align=None), offset=0), "flags": SimTypeInt(signed=False, label="UInt32"), "sessionKey": SimTypeBottom(label="Guid")}, name="FWPM_FILTER_SUBSCRIPTION0", pack=False, align=None), offset=0), offset=0), offset=0), SimTypePointer(SimTypeInt(signed=False, label="UInt32"), offset=0)], SimTypeInt(signed=False, label="UInt32"), arg_names=["engineHandle", "entries", "numEntries"]),
        # 
        'FwpmGetAppIdFromFileName0': SimTypeFunction([SimTypePointer(SimTypeChar(label="Char"), offset=0), SimTypePointer(SimTypePointer(SimStruct({"size": SimTypeInt(signed=False, label="UInt32"), "data": SimTypePointer(SimTypeChar(label="Byte"), offset=0)}, name="FWP_BYTE_BLOB", pack=False, align=None), offset=0), offset=0)], SimTypeInt(signed=False, label="UInt32"), arg_names=["fileName", "appId"]),
        # 
        'FwpmBitmapIndexGet0': SimTypeFunction([SimTypePointer(SimTypeInt(signed=True, label="Int"), label="IntPtr", offset=0), SimTypePointer(SimTypeBottom(label="Guid"), offset=0), SimTypePointer(SimTypeChar(label="Byte"), offset=0)], SimTypeInt(signed=False, label="UInt32"), arg_names=["engineHandle", "fieldId", "idx"]),
        # 
        'FwpmBitmapIndexFree0': SimTypeFunction([SimTypePointer(SimTypeInt(signed=True, label="Int"), label="IntPtr", offset=0), SimTypePointer(SimTypeBottom(label="Guid"), offset=0), SimTypePointer(SimTypeChar(label="Byte"), offset=0)], SimTypeInt(signed=False, label="UInt32"), arg_names=["engineHandle", "fieldId", "idx"]),
        # 
        'FwpmIPsecTunnelAdd0': SimTypeFunction([SimTypePointer(SimTypeInt(signed=True, label="Int"), label="IntPtr", offset=0), SimTypeInt(signed=False, label="UInt32"), SimTypePointer(SimStruct({"providerContextKey": SimTypeBottom(label="Guid"), "displayData": SimStruct({"name": SimTypePointer(SimTypeChar(label="Char"), offset=0), "description": SimTypePointer(SimTypeChar(label="Char"), offset=0)}, name="FWPM_DISPLAY_DATA0", pack=False, align=None), "flags": SimTypeInt(signed=False, label="UInt32"), "providerKey": SimTypePointer(SimTypeBottom(label="Guid"), offset=0), "providerData": SimStruct({"size": SimTypeInt(signed=False, label="UInt32"), "data": SimTypePointer(SimTypeChar(label="Byte"), offset=0)}, name="FWP_BYTE_BLOB", pack=False, align=None), "type": SimTypeInt(signed=False, label="FWPM_PROVIDER_CONTEXT_TYPE"), "Anonymous": SimUnion({"keyingPolicy": SimTypePointer(SimStruct({"numKeyMods": SimTypeInt(signed=False, label="UInt32"), "keyModKeys": SimTypePointer(SimTypeBottom(label="Guid"), offset=0)}, name="IPSEC_KEYING_POLICY0", pack=False, align=None), offset=0), "ikeQmTransportPolicy": SimTypePointer(SimStruct({"numIpsecProposals": SimTypeInt(signed=False, label="UInt32"), "ipsecProposals": SimTypePointer(SimStruct({"lifetime": SimStruct({"lifetimeSeconds": SimTypeInt(signed=False, label="UInt32"), "lifetimeKilobytes": SimTypeInt(signed=False, label="UInt32"), "lifetimePackets": SimTypeInt(signed=False, label="UInt32")}, name="IPSEC_SA_LIFETIME0", pack=False, align=None), "numSaTransforms": SimTypeInt(signed=False, label="UInt32"), "saTransforms": SimTypePointer(SimStruct({"ipsecTransformType": SimTypeInt(signed=False, label="IPSEC_TRANSFORM_TYPE"), "Anonymous": SimUnion({"ahTransform": SimTypePointer(SimStruct({"authTransformId": SimStruct({"authType": SimTypeInt(signed=False, label="IPSEC_AUTH_TYPE"), "authConfig": SimTypeChar(label="Byte")}, name="IPSEC_AUTH_TRANSFORM_ID0", pack=False, align=None), "cryptoModuleId": SimTypePointer(SimTypeBottom(label="Guid"), offset=0)}, name="IPSEC_AUTH_TRANSFORM0", pack=False, align=None), offset=0), "espAuthTransform": SimTypePointer(SimStruct({"authTransformId": SimStruct({"authType": SimTypeInt(signed=False, label="IPSEC_AUTH_TYPE"), "authConfig": SimTypeChar(label="Byte")}, name="IPSEC_AUTH_TRANSFORM_ID0", pack=False, align=None), "cryptoModuleId": SimTypePointer(SimTypeBottom(label="Guid"), offset=0)}, name="IPSEC_AUTH_TRANSFORM0", pack=False, align=None), offset=0), "espCipherTransform": SimTypePointer(SimStruct({"cipherTransformId": SimStruct({"cipherType": SimTypeInt(signed=False, label="IPSEC_CIPHER_TYPE"), "cipherConfig": SimTypeChar(label="Byte")}, name="IPSEC_CIPHER_TRANSFORM_ID0", pack=False, align=None), "cryptoModuleId": SimTypePointer(SimTypeBottom(label="Guid"), offset=0)}, name="IPSEC_CIPHER_TRANSFORM0", pack=False, align=None), offset=0), "espAuthAndCipherTransform": SimTypePointer(SimStruct({"authTransform": SimStruct({"authTransformId": SimStruct({"authType": SimTypeInt(signed=False, label="IPSEC_AUTH_TYPE"), "authConfig": SimTypeChar(label="Byte")}, name="IPSEC_AUTH_TRANSFORM_ID0", pack=False, align=None), "cryptoModuleId": SimTypePointer(SimTypeBottom(label="Guid"), offset=0)}, name="IPSEC_AUTH_TRANSFORM0", pack=False, align=None), "cipherTransform": SimStruct({"cipherTransformId": SimStruct({"cipherType": SimTypeInt(signed=False, label="IPSEC_CIPHER_TYPE"), "cipherConfig": SimTypeChar(label="Byte")}, name="IPSEC_CIPHER_TRANSFORM_ID0", pack=False, align=None), "cryptoModuleId": SimTypePointer(SimTypeBottom(label="Guid"), offset=0)}, name="IPSEC_CIPHER_TRANSFORM0", pack=False, align=None)}, name="IPSEC_AUTH_AND_CIPHER_TRANSFORM0", pack=False, align=None), offset=0), "espAuthFwTransform": SimTypePointer(SimStruct({"authTransformId": SimStruct({"authType": SimTypeInt(signed=False, label="IPSEC_AUTH_TYPE"), "authConfig": SimTypeChar(label="Byte")}, name="IPSEC_AUTH_TRANSFORM_ID0", pack=False, align=None), "cryptoModuleId": SimTypePointer(SimTypeBottom(label="Guid"), offset=0)}, name="IPSEC_AUTH_TRANSFORM0", pack=False, align=None), offset=0)}, name="<anon>", label="None")}, name="IPSEC_SA_TRANSFORM0", pack=False, align=None), offset=0), "pfsGroup": SimTypeInt(signed=False, label="IPSEC_PFS_GROUP")}, name="IPSEC_PROPOSAL0", pack=False, align=None), offset=0), "flags": SimTypeInt(signed=False, label="IPSEC_POLICY_FLAG"), "ndAllowClearTimeoutSeconds": SimTypeInt(signed=False, label="UInt32"), "saIdleTimeout": SimStruct({"idleTimeoutSeconds": SimTypeInt(signed=False, label="UInt32"), "idleTimeoutSecondsFailOver": SimTypeInt(signed=False, label="UInt32")}, name="IPSEC_SA_IDLE_TIMEOUT0", pack=False, align=None), "emPolicy": SimTypePointer(SimStruct({"numAuthenticationMethods": SimTypeInt(signed=False, label="UInt32"), "authenticationMethods": SimTypePointer(SimStruct({"authenticationMethodType": SimTypeInt(signed=False, label="IKEEXT_AUTHENTICATION_METHOD_TYPE"), "Anonymous": SimUnion({"presharedKeyAuthentication": SimStruct({"presharedKey": SimStruct({"size": SimTypeInt(signed=False, label="UInt32"), "data": SimTypePointer(SimTypeChar(label="Byte"), offset=0)}, name="FWP_BYTE_BLOB", pack=False, align=None)}, name="IKEEXT_PRESHARED_KEY_AUTHENTICATION0", pack=False, align=None), "certificateAuthentication": SimStruct({"inboundConfigType": SimTypeInt(signed=False, label="IKEEXT_CERT_CONFIG_TYPE"), "Anonymous1": SimUnion({"Anonymous": SimStruct({"inboundRootArraySize": SimTypeInt(signed=False, label="UInt32"), "inboundRootArray": SimTypePointer(SimStruct({"certData": SimStruct({"size": SimTypeInt(signed=False, label="UInt32"), "data": SimTypePointer(SimTypeChar(label="Byte"), offset=0)}, name="FWP_BYTE_BLOB", pack=False, align=None), "flags": SimTypeInt(signed=False, label="IKEEXT_CERT_FLAGS")}, name="IKEEXT_CERT_ROOT_CONFIG0", pack=False, align=None), offset=0)}, name="_Anonymous_e__Struct", pack=False, align=None), "inboundEnterpriseStoreConfig": SimTypePointer(SimStruct({"certData": SimStruct({"size": SimTypeInt(signed=False, label="UInt32"), "data": SimTypePointer(SimTypeChar(label="Byte"), offset=0)}, name="FWP_BYTE_BLOB", pack=False, align=None), "flags": SimTypeInt(signed=False, label="IKEEXT_CERT_FLAGS")}, name="IKEEXT_CERT_ROOT_CONFIG0", pack=False, align=None), offset=0), "inboundTrustedRootStoreConfig": SimTypePointer(SimStruct({"certData": SimStruct({"size": SimTypeInt(signed=False, label="UInt32"), "data": SimTypePointer(SimTypeChar(label="Byte"), offset=0)}, name="FWP_BYTE_BLOB", pack=False, align=None), "flags": SimTypeInt(signed=False, label="IKEEXT_CERT_FLAGS")}, name="IKEEXT_CERT_ROOT_CONFIG0", pack=False, align=None), offset=0)}, name="<anon>", label="None"), "outboundConfigType": SimTypeInt(signed=False, label="IKEEXT_CERT_CONFIG_TYPE"), "Anonymous2": SimUnion({"Anonymous": SimStruct({"outboundRootArraySize": SimTypeInt(signed=False, label="UInt32"), "outboundRootArray": SimTypePointer(SimStruct({"certData": SimStruct({"size": SimTypeInt(signed=False, label="UInt32"), "data": SimTypePointer(SimTypeChar(label="Byte"), offset=0)}, name="FWP_BYTE_BLOB", pack=False, align=None), "flags": SimTypeInt(signed=False, label="IKEEXT_CERT_FLAGS")}, name="IKEEXT_CERT_ROOT_CONFIG0", pack=False, align=None), offset=0)}, name="_Anonymous_e__Struct", pack=False, align=None), "outboundEnterpriseStoreConfig": SimTypePointer(SimStruct({"certData": SimStruct({"size": SimTypeInt(signed=False, label="UInt32"), "data": SimTypePointer(SimTypeChar(label="Byte"), offset=0)}, name="FWP_BYTE_BLOB", pack=False, align=None), "flags": SimTypeInt(signed=False, label="IKEEXT_CERT_FLAGS")}, name="IKEEXT_CERT_ROOT_CONFIG0", pack=False, align=None), offset=0), "outboundTrustedRootStoreConfig": SimTypePointer(SimStruct({"certData": SimStruct({"size": SimTypeInt(signed=False, label="UInt32"), "data": SimTypePointer(SimTypeChar(label="Byte"), offset=0)}, name="FWP_BYTE_BLOB", pack=False, align=None), "flags": SimTypeInt(signed=False, label="IKEEXT_CERT_FLAGS")}, name="IKEEXT_CERT_ROOT_CONFIG0", pack=False, align=None), offset=0)}, name="<anon>", label="None"), "flags": SimTypeInt(signed=False, label="IKEEXT_CERT_AUTH")}, name="IKEEXT_CERTIFICATE_AUTHENTICATION0", pack=False, align=None), "kerberosAuthentication": SimStruct({"flags": SimTypeInt(signed=False, label="IKEEXT_KERBEROS_AUTHENTICATION_FLAGS")}, name="IKEEXT_KERBEROS_AUTHENTICATION0", pack=False, align=None), "ntlmV2Authentication": SimStruct({"flags": SimTypeInt(signed=False, label="UInt32")}, name="IKEEXT_NTLM_V2_AUTHENTICATION0", pack=False, align=None), "sslAuthentication": SimStruct({"inboundConfigType": SimTypeInt(signed=False, label="IKEEXT_CERT_CONFIG_TYPE"), "Anonymous1": SimUnion({"Anonymous": SimStruct({"inboundRootArraySize": SimTypeInt(signed=False, label="UInt32"), "inboundRootArray": SimTypePointer(SimStruct({"certData": SimStruct({"size": SimTypeInt(signed=False, label="UInt32"), "data": SimTypePointer(SimTypeChar(label="Byte"), offset=0)}, name="FWP_BYTE_BLOB", pack=False, align=None), "flags": SimTypeInt(signed=False, label="IKEEXT_CERT_FLAGS")}, name="IKEEXT_CERT_ROOT_CONFIG0", pack=False, align=None), offset=0)}, name="_Anonymous_e__Struct", pack=False, align=None), "inboundEnterpriseStoreConfig": SimTypePointer(SimStruct({"certData": SimStruct({"size": SimTypeInt(signed=False, label="UInt32"), "data": SimTypePointer(SimTypeChar(label="Byte"), offset=0)}, name="FWP_BYTE_BLOB", pack=False, align=None), "flags": SimTypeInt(signed=False, label="IKEEXT_CERT_FLAGS")}, name="IKEEXT_CERT_ROOT_CONFIG0", pack=False, align=None), offset=0), "inboundTrustedRootStoreConfig": SimTypePointer(SimStruct({"certData": SimStruct({"size": SimTypeInt(signed=False, label="UInt32"), "data": SimTypePointer(SimTypeChar(label="Byte"), offset=0)}, name="FWP_BYTE_BLOB", pack=False, align=None), "flags": SimTypeInt(signed=False, label="IKEEXT_CERT_FLAGS")}, name="IKEEXT_CERT_ROOT_CONFIG0", pack=False, align=None), offset=0)}, name="<anon>", label="None"), "outboundConfigType": SimTypeInt(signed=False, label="IKEEXT_CERT_CONFIG_TYPE"), "Anonymous2": SimUnion({"Anonymous": SimStruct({"outboundRootArraySize": SimTypeInt(signed=False, label="UInt32"), "outboundRootArray": SimTypePointer(SimStruct({"certData": SimStruct({"size": SimTypeInt(signed=False, label="UInt32"), "data": SimTypePointer(SimTypeChar(label="Byte"), offset=0)}, name="FWP_BYTE_BLOB", pack=False, align=None), "flags": SimTypeInt(signed=False, label="IKEEXT_CERT_FLAGS")}, name="IKEEXT_CERT_ROOT_CONFIG0", pack=False, align=None), offset=0)}, name="_Anonymous_e__Struct", pack=False, align=None), "outboundEnterpriseStoreConfig": SimTypePointer(SimStruct({"certData": SimStruct({"size": SimTypeInt(signed=False, label="UInt32"), "data": SimTypePointer(SimTypeChar(label="Byte"), offset=0)}, name="FWP_BYTE_BLOB", pack=False, align=None), "flags": SimTypeInt(signed=False, label="IKEEXT_CERT_FLAGS")}, name="IKEEXT_CERT_ROOT_CONFIG0", pack=False, align=None), offset=0), "outboundTrustedRootStoreConfig": SimTypePointer(SimStruct({"certData": SimStruct({"size": SimTypeInt(signed=False, label="UInt32"), "data": SimTypePointer(SimTypeChar(label="Byte"), offset=0)}, name="FWP_BYTE_BLOB", pack=False, align=None), "flags": SimTypeInt(signed=False, label="IKEEXT_CERT_FLAGS")}, name="IKEEXT_CERT_ROOT_CONFIG0", pack=False, align=None), offset=0)}, name="<anon>", label="None"), "flags": SimTypeInt(signed=False, label="IKEEXT_CERT_AUTH")}, name="IKEEXT_CERTIFICATE_AUTHENTICATION0", pack=False, align=None), "cgaAuthentication": SimStruct({"keyContainerName": SimTypePointer(SimTypeChar(label="Char"), offset=0), "cspName": SimTypePointer(SimTypeChar(label="Char"), offset=0), "cspType": SimTypeInt(signed=False, label="UInt32"), "cgaModifier": SimStruct({"byteArray16": SimTypeFixedSizeArray(SimTypeChar(label="Byte"), 16)}, name="FWP_BYTE_ARRAY16", pack=False, align=None), "cgaCollisionCount": SimTypeChar(label="Byte")}, name="IKEEXT_IPV6_CGA_AUTHENTICATION0", pack=False, align=None)}, name="<anon>", label="None")}, name="IKEEXT_AUTHENTICATION_METHOD0", pack=False, align=None), offset=0), "initiatorImpersonationType": SimTypeInt(signed=False, label="IKEEXT_AUTHENTICATION_IMPERSONATION_TYPE")}, name="IKEEXT_EM_POLICY0", pack=False, align=None), offset=0)}, name="IPSEC_TRANSPORT_POLICY0", pack=False, align=None), offset=0), "ikeQmTunnelPolicy": SimTypePointer(SimStruct({"flags": SimTypeInt(signed=False, label="IPSEC_POLICY_FLAG"), "numIpsecProposals": SimTypeInt(signed=False, label="UInt32"), "ipsecProposals": SimTypePointer(SimStruct({"lifetime": SimStruct({"lifetimeSeconds": SimTypeInt(signed=False, label="UInt32"), "lifetimeKilobytes": SimTypeInt(signed=False, label="UInt32"), "lifetimePackets": SimTypeInt(signed=False, label="UInt32")}, name="IPSEC_SA_LIFETIME0", pack=False, align=None), "numSaTransforms": SimTypeInt(signed=False, label="UInt32"), "saTransforms": SimTypePointer(SimStruct({"ipsecTransformType": SimTypeInt(signed=False, label="IPSEC_TRANSFORM_TYPE"), "Anonymous": SimUnion({"ahTransform": SimTypePointer(SimStruct({"authTransformId": SimStruct({"authType": SimTypeInt(signed=False, label="IPSEC_AUTH_TYPE"), "authConfig": SimTypeChar(label="Byte")}, name="IPSEC_AUTH_TRANSFORM_ID0", pack=False, align=None), "cryptoModuleId": SimTypePointer(SimTypeBottom(label="Guid"), offset=0)}, name="IPSEC_AUTH_TRANSFORM0", pack=False, align=None), offset=0), "espAuthTransform": SimTypePointer(SimStruct({"authTransformId": SimStruct({"authType": SimTypeInt(signed=False, label="IPSEC_AUTH_TYPE"), "authConfig": SimTypeChar(label="Byte")}, name="IPSEC_AUTH_TRANSFORM_ID0", pack=False, align=None), "cryptoModuleId": SimTypePointer(SimTypeBottom(label="Guid"), offset=0)}, name="IPSEC_AUTH_TRANSFORM0", pack=False, align=None), offset=0), "espCipherTransform": SimTypePointer(SimStruct({"cipherTransformId": SimStruct({"cipherType": SimTypeInt(signed=False, label="IPSEC_CIPHER_TYPE"), "cipherConfig": SimTypeChar(label="Byte")}, name="IPSEC_CIPHER_TRANSFORM_ID0", pack=False, align=None), "cryptoModuleId": SimTypePointer(SimTypeBottom(label="Guid"), offset=0)}, name="IPSEC_CIPHER_TRANSFORM0", pack=False, align=None), offset=0), "espAuthAndCipherTransform": SimTypePointer(SimStruct({"authTransform": SimStruct({"authTransformId": SimStruct({"authType": SimTypeInt(signed=False, label="IPSEC_AUTH_TYPE"), "authConfig": SimTypeChar(label="Byte")}, name="IPSEC_AUTH_TRANSFORM_ID0", pack=False, align=None), "cryptoModuleId": SimTypePointer(SimTypeBottom(label="Guid"), offset=0)}, name="IPSEC_AUTH_TRANSFORM0", pack=False, align=None), "cipherTransform": SimStruct({"cipherTransformId": SimStruct({"cipherType": SimTypeInt(signed=False, label="IPSEC_CIPHER_TYPE"), "cipherConfig": SimTypeChar(label="Byte")}, name="IPSEC_CIPHER_TRANSFORM_ID0", pack=False, align=None), "cryptoModuleId": SimTypePointer(SimTypeBottom(label="Guid"), offset=0)}, name="IPSEC_CIPHER_TRANSFORM0", pack=False, align=None)}, name="IPSEC_AUTH_AND_CIPHER_TRANSFORM0", pack=False, align=None), offset=0), "espAuthFwTransform": SimTypePointer(SimStruct({"authTransformId": SimStruct({"authType": SimTypeInt(signed=False, label="IPSEC_AUTH_TYPE"), "authConfig": SimTypeChar(label="Byte")}, name="IPSEC_AUTH_TRANSFORM_ID0", pack=False, align=None), "cryptoModuleId": SimTypePointer(SimTypeBottom(label="Guid"), offset=0)}, name="IPSEC_AUTH_TRANSFORM0", pack=False, align=None), offset=0)}, name="<anon>", label="None")}, name="IPSEC_SA_TRANSFORM0", pack=False, align=None), offset=0), "pfsGroup": SimTypeInt(signed=False, label="IPSEC_PFS_GROUP")}, name="IPSEC_PROPOSAL0", pack=False, align=None), offset=0), "tunnelEndpoints": SimStruct({"ipVersion": SimTypeInt(signed=False, label="FWP_IP_VERSION"), "Anonymous1": SimUnion({"localV4Address": SimTypeInt(signed=False, label="UInt32"), "localV6Address": SimTypeFixedSizeArray(SimTypeChar(label="Byte"), 16)}, name="<anon>", label="None"), "Anonymous2": SimUnion({"remoteV4Address": SimTypeInt(signed=False, label="UInt32"), "remoteV6Address": SimTypeFixedSizeArray(SimTypeChar(label="Byte"), 16)}, name="<anon>", label="None")}, name="IPSEC_TUNNEL_ENDPOINTS0", pack=False, align=None), "saIdleTimeout": SimStruct({"idleTimeoutSeconds": SimTypeInt(signed=False, label="UInt32"), "idleTimeoutSecondsFailOver": SimTypeInt(signed=False, label="UInt32")}, name="IPSEC_SA_IDLE_TIMEOUT0", pack=False, align=None), "emPolicy": SimTypePointer(SimStruct({"numAuthenticationMethods": SimTypeInt(signed=False, label="UInt32"), "authenticationMethods": SimTypePointer(SimStruct({"authenticationMethodType": SimTypeInt(signed=False, label="IKEEXT_AUTHENTICATION_METHOD_TYPE"), "Anonymous": SimUnion({"presharedKeyAuthentication": SimStruct({"presharedKey": SimStruct({"size": SimTypeInt(signed=False, label="UInt32"), "data": SimTypePointer(SimTypeChar(label="Byte"), offset=0)}, name="FWP_BYTE_BLOB", pack=False, align=None)}, name="IKEEXT_PRESHARED_KEY_AUTHENTICATION0", pack=False, align=None), "certificateAuthentication": SimStruct({"inboundConfigType": SimTypeInt(signed=False, label="IKEEXT_CERT_CONFIG_TYPE"), "Anonymous1": SimUnion({"Anonymous": SimStruct({"inboundRootArraySize": SimTypeInt(signed=False, label="UInt32"), "inboundRootArray": SimTypePointer(SimStruct({"certData": SimStruct({"size": SimTypeInt(signed=False, label="UInt32"), "data": SimTypePointer(SimTypeChar(label="Byte"), offset=0)}, name="FWP_BYTE_BLOB", pack=False, align=None), "flags": SimTypeInt(signed=False, label="IKEEXT_CERT_FLAGS")}, name="IKEEXT_CERT_ROOT_CONFIG0", pack=False, align=None), offset=0)}, name="_Anonymous_e__Struct", pack=False, align=None), "inboundEnterpriseStoreConfig": SimTypePointer(SimStruct({"certData": SimStruct({"size": SimTypeInt(signed=False, label="UInt32"), "data": SimTypePointer(SimTypeChar(label="Byte"), offset=0)}, name="FWP_BYTE_BLOB", pack=False, align=None), "flags": SimTypeInt(signed=False, label="IKEEXT_CERT_FLAGS")}, name="IKEEXT_CERT_ROOT_CONFIG0", pack=False, align=None), offset=0), "inboundTrustedRootStoreConfig": SimTypePointer(SimStruct({"certData": SimStruct({"size": SimTypeInt(signed=False, label="UInt32"), "data": SimTypePointer(SimTypeChar(label="Byte"), offset=0)}, name="FWP_BYTE_BLOB", pack=False, align=None), "flags": SimTypeInt(signed=False, label="IKEEXT_CERT_FLAGS")}, name="IKEEXT_CERT_ROOT_CONFIG0", pack=False, align=None), offset=0)}, name="<anon>", label="None"), "outboundConfigType": SimTypeInt(signed=False, label="IKEEXT_CERT_CONFIG_TYPE"), "Anonymous2": SimUnion({"Anonymous": SimStruct({"outboundRootArraySize": SimTypeInt(signed=False, label="UInt32"), "outboundRootArray": SimTypePointer(SimStruct({"certData": SimStruct({"size": SimTypeInt(signed=False, label="UInt32"), "data": SimTypePointer(SimTypeChar(label="Byte"), offset=0)}, name="FWP_BYTE_BLOB", pack=False, align=None), "flags": SimTypeInt(signed=False, label="IKEEXT_CERT_FLAGS")}, name="IKEEXT_CERT_ROOT_CONFIG0", pack=False, align=None), offset=0)}, name="_Anonymous_e__Struct", pack=False, align=None), "outboundEnterpriseStoreConfig": SimTypePointer(SimStruct({"certData": SimStruct({"size": SimTypeInt(signed=False, label="UInt32"), "data": SimTypePointer(SimTypeChar(label="Byte"), offset=0)}, name="FWP_BYTE_BLOB", pack=False, align=None), "flags": SimTypeInt(signed=False, label="IKEEXT_CERT_FLAGS")}, name="IKEEXT_CERT_ROOT_CONFIG0", pack=False, align=None), offset=0), "outboundTrustedRootStoreConfig": SimTypePointer(SimStruct({"certData": SimStruct({"size": SimTypeInt(signed=False, label="UInt32"), "data": SimTypePointer(SimTypeChar(label="Byte"), offset=0)}, name="FWP_BYTE_BLOB", pack=False, align=None), "flags": SimTypeInt(signed=False, label="IKEEXT_CERT_FLAGS")}, name="IKEEXT_CERT_ROOT_CONFIG0", pack=False, align=None), offset=0)}, name="<anon>", label="None"), "flags": SimTypeInt(signed=False, label="IKEEXT_CERT_AUTH")}, name="IKEEXT_CERTIFICATE_AUTHENTICATION0", pack=False, align=None), "kerberosAuthentication": SimStruct({"flags": SimTypeInt(signed=False, label="IKEEXT_KERBEROS_AUTHENTICATION_FLAGS")}, name="IKEEXT_KERBEROS_AUTHENTICATION0", pack=False, align=None), "ntlmV2Authentication": SimStruct({"flags": SimTypeInt(signed=False, label="UInt32")}, name="IKEEXT_NTLM_V2_AUTHENTICATION0", pack=False, align=None), "sslAuthentication": SimStruct({"inboundConfigType": SimTypeInt(signed=False, label="IKEEXT_CERT_CONFIG_TYPE"), "Anonymous1": SimUnion({"Anonymous": SimStruct({"inboundRootArraySize": SimTypeInt(signed=False, label="UInt32"), "inboundRootArray": SimTypePointer(SimStruct({"certData": SimStruct({"size": SimTypeInt(signed=False, label="UInt32"), "data": SimTypePointer(SimTypeChar(label="Byte"), offset=0)}, name="FWP_BYTE_BLOB", pack=False, align=None), "flags": SimTypeInt(signed=False, label="IKEEXT_CERT_FLAGS")}, name="IKEEXT_CERT_ROOT_CONFIG0", pack=False, align=None), offset=0)}, name="_Anonymous_e__Struct", pack=False, align=None), "inboundEnterpriseStoreConfig": SimTypePointer(SimStruct({"certData": SimStruct({"size": SimTypeInt(signed=False, label="UInt32"), "data": SimTypePointer(SimTypeChar(label="Byte"), offset=0)}, name="FWP_BYTE_BLOB", pack=False, align=None), "flags": SimTypeInt(signed=False, label="IKEEXT_CERT_FLAGS")}, name="IKEEXT_CERT_ROOT_CONFIG0", pack=False, align=None), offset=0), "inboundTrustedRootStoreConfig": SimTypePointer(SimStruct({"certData": SimStruct({"size": SimTypeInt(signed=False, label="UInt32"), "data": SimTypePointer(SimTypeChar(label="Byte"), offset=0)}, name="FWP_BYTE_BLOB", pack=False, align=None), "flags": SimTypeInt(signed=False, label="IKEEXT_CERT_FLAGS")}, name="IKEEXT_CERT_ROOT_CONFIG0", pack=False, align=None), offset=0)}, name="<anon>", label="None"), "outboundConfigType": SimTypeInt(signed=False, label="IKEEXT_CERT_CONFIG_TYPE"), "Anonymous2": SimUnion({"Anonymous": SimStruct({"outboundRootArraySize": SimTypeInt(signed=False, label="UInt32"), "outboundRootArray": SimTypePointer(SimStruct({"certData": SimStruct({"size": SimTypeInt(signed=False, label="UInt32"), "data": SimTypePointer(SimTypeChar(label="Byte"), offset=0)}, name="FWP_BYTE_BLOB", pack=False, align=None), "flags": SimTypeInt(signed=False, label="IKEEXT_CERT_FLAGS")}, name="IKEEXT_CERT_ROOT_CONFIG0", pack=False, align=None), offset=0)}, name="_Anonymous_e__Struct", pack=False, align=None), "outboundEnterpriseStoreConfig": SimTypePointer(SimStruct({"certData": SimStruct({"size": SimTypeInt(signed=False, label="UInt32"), "data": SimTypePointer(SimTypeChar(label="Byte"), offset=0)}, name="FWP_BYTE_BLOB", pack=False, align=None), "flags": SimTypeInt(signed=False, label="IKEEXT_CERT_FLAGS")}, name="IKEEXT_CERT_ROOT_CONFIG0", pack=False, align=None), offset=0), "outboundTrustedRootStoreConfig": SimTypePointer(SimStruct({"certData": SimStruct({"size": SimTypeInt(signed=False, label="UInt32"), "data": SimTypePointer(SimTypeChar(label="Byte"), offset=0)}, name="FWP_BYTE_BLOB", pack=False, align=None), "flags": SimTypeInt(signed=False, label="IKEEXT_CERT_FLAGS")}, name="IKEEXT_CERT_ROOT_CONFIG0", pack=False, align=None), offset=0)}, name="<anon>", label="None"), "flags": SimTypeInt(signed=False, label="IKEEXT_CERT_AUTH")}, name="IKEEXT_CERTIFICATE_AUTHENTICATION0", pack=False, align=None), "cgaAuthentication": SimStruct({"keyContainerName": SimTypePointer(SimTypeChar(label="Char"), offset=0), "cspName": SimTypePointer(SimTypeChar(label="Char"), offset=0), "cspType": SimTypeInt(signed=False, label="UInt32"), "cgaModifier": SimStruct({"byteArray16": SimTypeFixedSizeArray(SimTypeChar(label="Byte"), 16)}, name="FWP_BYTE_ARRAY16", pack=False, align=None), "cgaCollisionCount": SimTypeChar(label="Byte")}, name="IKEEXT_IPV6_CGA_AUTHENTICATION0", pack=False, align=None)}, name="<anon>", label="None")}, name="IKEEXT_AUTHENTICATION_METHOD0", pack=False, align=None), offset=0), "initiatorImpersonationType": SimTypeInt(signed=False, label="IKEEXT_AUTHENTICATION_IMPERSONATION_TYPE")}, name="IKEEXT_EM_POLICY0", pack=False, align=None), offset=0)}, name="IPSEC_TUNNEL_POLICY0", pack=False, align=None), offset=0), "authipQmTransportPolicy": SimTypePointer(SimStruct({"numIpsecProposals": SimTypeInt(signed=False, label="UInt32"), "ipsecProposals": SimTypePointer(SimStruct({"lifetime": SimStruct({"lifetimeSeconds": SimTypeInt(signed=False, label="UInt32"), "lifetimeKilobytes": SimTypeInt(signed=False, label="UInt32"), "lifetimePackets": SimTypeInt(signed=False, label="UInt32")}, name="IPSEC_SA_LIFETIME0", pack=False, align=None), "numSaTransforms": SimTypeInt(signed=False, label="UInt32"), "saTransforms": SimTypePointer(SimStruct({"ipsecTransformType": SimTypeInt(signed=False, label="IPSEC_TRANSFORM_TYPE"), "Anonymous": SimUnion({"ahTransform": SimTypePointer(SimStruct({"authTransformId": SimStruct({"authType": SimTypeInt(signed=False, label="IPSEC_AUTH_TYPE"), "authConfig": SimTypeChar(label="Byte")}, name="IPSEC_AUTH_TRANSFORM_ID0", pack=False, align=None), "cryptoModuleId": SimTypePointer(SimTypeBottom(label="Guid"), offset=0)}, name="IPSEC_AUTH_TRANSFORM0", pack=False, align=None), offset=0), "espAuthTransform": SimTypePointer(SimStruct({"authTransformId": SimStruct({"authType": SimTypeInt(signed=False, label="IPSEC_AUTH_TYPE"), "authConfig": SimTypeChar(label="Byte")}, name="IPSEC_AUTH_TRANSFORM_ID0", pack=False, align=None), "cryptoModuleId": SimTypePointer(SimTypeBottom(label="Guid"), offset=0)}, name="IPSEC_AUTH_TRANSFORM0", pack=False, align=None), offset=0), "espCipherTransform": SimTypePointer(SimStruct({"cipherTransformId": SimStruct({"cipherType": SimTypeInt(signed=False, label="IPSEC_CIPHER_TYPE"), "cipherConfig": SimTypeChar(label="Byte")}, name="IPSEC_CIPHER_TRANSFORM_ID0", pack=False, align=None), "cryptoModuleId": SimTypePointer(SimTypeBottom(label="Guid"), offset=0)}, name="IPSEC_CIPHER_TRANSFORM0", pack=False, align=None), offset=0), "espAuthAndCipherTransform": SimTypePointer(SimStruct({"authTransform": SimStruct({"authTransformId": SimStruct({"authType": SimTypeInt(signed=False, label="IPSEC_AUTH_TYPE"), "authConfig": SimTypeChar(label="Byte")}, name="IPSEC_AUTH_TRANSFORM_ID0", pack=False, align=None), "cryptoModuleId": SimTypePointer(SimTypeBottom(label="Guid"), offset=0)}, name="IPSEC_AUTH_TRANSFORM0", pack=False, align=None), "cipherTransform": SimStruct({"cipherTransformId": SimStruct({"cipherType": SimTypeInt(signed=False, label="IPSEC_CIPHER_TYPE"), "cipherConfig": SimTypeChar(label="Byte")}, name="IPSEC_CIPHER_TRANSFORM_ID0", pack=False, align=None), "cryptoModuleId": SimTypePointer(SimTypeBottom(label="Guid"), offset=0)}, name="IPSEC_CIPHER_TRANSFORM0", pack=False, align=None)}, name="IPSEC_AUTH_AND_CIPHER_TRANSFORM0", pack=False, align=None), offset=0), "espAuthFwTransform": SimTypePointer(SimStruct({"authTransformId": SimStruct({"authType": SimTypeInt(signed=False, label="IPSEC_AUTH_TYPE"), "authConfig": SimTypeChar(label="Byte")}, name="IPSEC_AUTH_TRANSFORM_ID0", pack=False, align=None), "cryptoModuleId": SimTypePointer(SimTypeBottom(label="Guid"), offset=0)}, name="IPSEC_AUTH_TRANSFORM0", pack=False, align=None), offset=0)}, name="<anon>", label="None")}, name="IPSEC_SA_TRANSFORM0", pack=False, align=None), offset=0), "pfsGroup": SimTypeInt(signed=False, label="IPSEC_PFS_GROUP")}, name="IPSEC_PROPOSAL0", pack=False, align=None), offset=0), "flags": SimTypeInt(signed=False, label="IPSEC_POLICY_FLAG"), "ndAllowClearTimeoutSeconds": SimTypeInt(signed=False, label="UInt32"), "saIdleTimeout": SimStruct({"idleTimeoutSeconds": SimTypeInt(signed=False, label="UInt32"), "idleTimeoutSecondsFailOver": SimTypeInt(signed=False, label="UInt32")}, name="IPSEC_SA_IDLE_TIMEOUT0", pack=False, align=None), "emPolicy": SimTypePointer(SimStruct({"numAuthenticationMethods": SimTypeInt(signed=False, label="UInt32"), "authenticationMethods": SimTypePointer(SimStruct({"authenticationMethodType": SimTypeInt(signed=False, label="IKEEXT_AUTHENTICATION_METHOD_TYPE"), "Anonymous": SimUnion({"presharedKeyAuthentication": SimStruct({"presharedKey": SimStruct({"size": SimTypeInt(signed=False, label="UInt32"), "data": SimTypePointer(SimTypeChar(label="Byte"), offset=0)}, name="FWP_BYTE_BLOB", pack=False, align=None)}, name="IKEEXT_PRESHARED_KEY_AUTHENTICATION0", pack=False, align=None), "certificateAuthentication": SimStruct({"inboundConfigType": SimTypeInt(signed=False, label="IKEEXT_CERT_CONFIG_TYPE"), "Anonymous1": SimUnion({"Anonymous": SimStruct({"inboundRootArraySize": SimTypeInt(signed=False, label="UInt32"), "inboundRootArray": SimTypePointer(SimStruct({"certData": SimStruct({"size": SimTypeInt(signed=False, label="UInt32"), "data": SimTypePointer(SimTypeChar(label="Byte"), offset=0)}, name="FWP_BYTE_BLOB", pack=False, align=None), "flags": SimTypeInt(signed=False, label="IKEEXT_CERT_FLAGS")}, name="IKEEXT_CERT_ROOT_CONFIG0", pack=False, align=None), offset=0)}, name="_Anonymous_e__Struct", pack=False, align=None), "inboundEnterpriseStoreConfig": SimTypePointer(SimStruct({"certData": SimStruct({"size": SimTypeInt(signed=False, label="UInt32"), "data": SimTypePointer(SimTypeChar(label="Byte"), offset=0)}, name="FWP_BYTE_BLOB", pack=False, align=None), "flags": SimTypeInt(signed=False, label="IKEEXT_CERT_FLAGS")}, name="IKEEXT_CERT_ROOT_CONFIG0", pack=False, align=None), offset=0), "inboundTrustedRootStoreConfig": SimTypePointer(SimStruct({"certData": SimStruct({"size": SimTypeInt(signed=False, label="UInt32"), "data": SimTypePointer(SimTypeChar(label="Byte"), offset=0)}, name="FWP_BYTE_BLOB", pack=False, align=None), "flags": SimTypeInt(signed=False, label="IKEEXT_CERT_FLAGS")}, name="IKEEXT_CERT_ROOT_CONFIG0", pack=False, align=None), offset=0)}, name="<anon>", label="None"), "outboundConfigType": SimTypeInt(signed=False, label="IKEEXT_CERT_CONFIG_TYPE"), "Anonymous2": SimUnion({"Anonymous": SimStruct({"outboundRootArraySize": SimTypeInt(signed=False, label="UInt32"), "outboundRootArray": SimTypePointer(SimStruct({"certData": SimStruct({"size": SimTypeInt(signed=False, label="UInt32"), "data": SimTypePointer(SimTypeChar(label="Byte"), offset=0)}, name="FWP_BYTE_BLOB", pack=False, align=None), "flags": SimTypeInt(signed=False, label="IKEEXT_CERT_FLAGS")}, name="IKEEXT_CERT_ROOT_CONFIG0", pack=False, align=None), offset=0)}, name="_Anonymous_e__Struct", pack=False, align=None), "outboundEnterpriseStoreConfig": SimTypePointer(SimStruct({"certData": SimStruct({"size": SimTypeInt(signed=False, label="UInt32"), "data": SimTypePointer(SimTypeChar(label="Byte"), offset=0)}, name="FWP_BYTE_BLOB", pack=False, align=None), "flags": SimTypeInt(signed=False, label="IKEEXT_CERT_FLAGS")}, name="IKEEXT_CERT_ROOT_CONFIG0", pack=False, align=None), offset=0), "outboundTrustedRootStoreConfig": SimTypePointer(SimStruct({"certData": SimStruct({"size": SimTypeInt(signed=False, label="UInt32"), "data": SimTypePointer(SimTypeChar(label="Byte"), offset=0)}, name="FWP_BYTE_BLOB", pack=False, align=None), "flags": SimTypeInt(signed=False, label="IKEEXT_CERT_FLAGS")}, name="IKEEXT_CERT_ROOT_CONFIG0", pack=False, align=None), offset=0)}, name="<anon>", label="None"), "flags": SimTypeInt(signed=False, label="IKEEXT_CERT_AUTH")}, name="IKEEXT_CERTIFICATE_AUTHENTICATION0", pack=False, align=None), "kerberosAuthentication": SimStruct({"flags": SimTypeInt(signed=False, label="IKEEXT_KERBEROS_AUTHENTICATION_FLAGS")}, name="IKEEXT_KERBEROS_AUTHENTICATION0", pack=False, align=None), "ntlmV2Authentication": SimStruct({"flags": SimTypeInt(signed=False, label="UInt32")}, name="IKEEXT_NTLM_V2_AUTHENTICATION0", pack=False, align=None), "sslAuthentication": SimStruct({"inboundConfigType": SimTypeInt(signed=False, label="IKEEXT_CERT_CONFIG_TYPE"), "Anonymous1": SimUnion({"Anonymous": SimStruct({"inboundRootArraySize": SimTypeInt(signed=False, label="UInt32"), "inboundRootArray": SimTypePointer(SimStruct({"certData": SimStruct({"size": SimTypeInt(signed=False, label="UInt32"), "data": SimTypePointer(SimTypeChar(label="Byte"), offset=0)}, name="FWP_BYTE_BLOB", pack=False, align=None), "flags": SimTypeInt(signed=False, label="IKEEXT_CERT_FLAGS")}, name="IKEEXT_CERT_ROOT_CONFIG0", pack=False, align=None), offset=0)}, name="_Anonymous_e__Struct", pack=False, align=None), "inboundEnterpriseStoreConfig": SimTypePointer(SimStruct({"certData": SimStruct({"size": SimTypeInt(signed=False, label="UInt32"), "data": SimTypePointer(SimTypeChar(label="Byte"), offset=0)}, name="FWP_BYTE_BLOB", pack=False, align=None), "flags": SimTypeInt(signed=False, label="IKEEXT_CERT_FLAGS")}, name="IKEEXT_CERT_ROOT_CONFIG0", pack=False, align=None), offset=0), "inboundTrustedRootStoreConfig": SimTypePointer(SimStruct({"certData": SimStruct({"size": SimTypeInt(signed=False, label="UInt32"), "data": SimTypePointer(SimTypeChar(label="Byte"), offset=0)}, name="FWP_BYTE_BLOB", pack=False, align=None), "flags": SimTypeInt(signed=False, label="IKEEXT_CERT_FLAGS")}, name="IKEEXT_CERT_ROOT_CONFIG0", pack=False, align=None), offset=0)}, name="<anon>", label="None"), "outboundConfigType": SimTypeInt(signed=False, label="IKEEXT_CERT_CONFIG_TYPE"), "Anonymous2": SimUnion({"Anonymous": SimStruct({"outboundRootArraySize": SimTypeInt(signed=False, label="UInt32"), "outboundRootArray": SimTypePointer(SimStruct({"certData": SimStruct({"size": SimTypeInt(signed=False, label="UInt32"), "data": SimTypePointer(SimTypeChar(label="Byte"), offset=0)}, name="FWP_BYTE_BLOB", pack=False, align=None), "flags": SimTypeInt(signed=False, label="IKEEXT_CERT_FLAGS")}, name="IKEEXT_CERT_ROOT_CONFIG0", pack=False, align=None), offset=0)}, name="_Anonymous_e__Struct", pack=False, align=None), "outboundEnterpriseStoreConfig": SimTypePointer(SimStruct({"certData": SimStruct({"size": SimTypeInt(signed=False, label="UInt32"), "data": SimTypePointer(SimTypeChar(label="Byte"), offset=0)}, name="FWP_BYTE_BLOB", pack=False, align=None), "flags": SimTypeInt(signed=False, label="IKEEXT_CERT_FLAGS")}, name="IKEEXT_CERT_ROOT_CONFIG0", pack=False, align=None), offset=0), "outboundTrustedRootStoreConfig": SimTypePointer(SimStruct({"certData": SimStruct({"size": SimTypeInt(signed=False, label="UInt32"), "data": SimTypePointer(SimTypeChar(label="Byte"), offset=0)}, name="FWP_BYTE_BLOB", pack=False, align=None), "flags": SimTypeInt(signed=False, label="IKEEXT_CERT_FLAGS")}, name="IKEEXT_CERT_ROOT_CONFIG0", pack=False, align=None), offset=0)}, name="<anon>", label="None"), "flags": SimTypeInt(signed=False, label="IKEEXT_CERT_AUTH")}, name="IKEEXT_CERTIFICATE_AUTHENTICATION0", pack=False, align=None), "cgaAuthentication": SimStruct({"keyContainerName": SimTypePointer(SimTypeChar(label="Char"), offset=0), "cspName": SimTypePointer(SimTypeChar(label="Char"), offset=0), "cspType": SimTypeInt(signed=False, label="UInt32"), "cgaModifier": SimStruct({"byteArray16": SimTypeFixedSizeArray(SimTypeChar(label="Byte"), 16)}, name="FWP_BYTE_ARRAY16", pack=False, align=None), "cgaCollisionCount": SimTypeChar(label="Byte")}, name="IKEEXT_IPV6_CGA_AUTHENTICATION0", pack=False, align=None)}, name="<anon>", label="None")}, name="IKEEXT_AUTHENTICATION_METHOD0", pack=False, align=None), offset=0), "initiatorImpersonationType": SimTypeInt(signed=False, label="IKEEXT_AUTHENTICATION_IMPERSONATION_TYPE")}, name="IKEEXT_EM_POLICY0", pack=False, align=None), offset=0)}, name="IPSEC_TRANSPORT_POLICY0", pack=False, align=None), offset=0), "authipQmTunnelPolicy": SimTypePointer(SimStruct({"flags": SimTypeInt(signed=False, label="IPSEC_POLICY_FLAG"), "numIpsecProposals": SimTypeInt(signed=False, label="UInt32"), "ipsecProposals": SimTypePointer(SimStruct({"lifetime": SimStruct({"lifetimeSeconds": SimTypeInt(signed=False, label="UInt32"), "lifetimeKilobytes": SimTypeInt(signed=False, label="UInt32"), "lifetimePackets": SimTypeInt(signed=False, label="UInt32")}, name="IPSEC_SA_LIFETIME0", pack=False, align=None), "numSaTransforms": SimTypeInt(signed=False, label="UInt32"), "saTransforms": SimTypePointer(SimStruct({"ipsecTransformType": SimTypeInt(signed=False, label="IPSEC_TRANSFORM_TYPE"), "Anonymous": SimUnion({"ahTransform": SimTypePointer(SimStruct({"authTransformId": SimStruct({"authType": SimTypeInt(signed=False, label="IPSEC_AUTH_TYPE"), "authConfig": SimTypeChar(label="Byte")}, name="IPSEC_AUTH_TRANSFORM_ID0", pack=False, align=None), "cryptoModuleId": SimTypePointer(SimTypeBottom(label="Guid"), offset=0)}, name="IPSEC_AUTH_TRANSFORM0", pack=False, align=None), offset=0), "espAuthTransform": SimTypePointer(SimStruct({"authTransformId": SimStruct({"authType": SimTypeInt(signed=False, label="IPSEC_AUTH_TYPE"), "authConfig": SimTypeChar(label="Byte")}, name="IPSEC_AUTH_TRANSFORM_ID0", pack=False, align=None), "cryptoModuleId": SimTypePointer(SimTypeBottom(label="Guid"), offset=0)}, name="IPSEC_AUTH_TRANSFORM0", pack=False, align=None), offset=0), "espCipherTransform": SimTypePointer(SimStruct({"cipherTransformId": SimStruct({"cipherType": SimTypeInt(signed=False, label="IPSEC_CIPHER_TYPE"), "cipherConfig": SimTypeChar(label="Byte")}, name="IPSEC_CIPHER_TRANSFORM_ID0", pack=False, align=None), "cryptoModuleId": SimTypePointer(SimTypeBottom(label="Guid"), offset=0)}, name="IPSEC_CIPHER_TRANSFORM0", pack=False, align=None), offset=0), "espAuthAndCipherTransform": SimTypePointer(SimStruct({"authTransform": SimStruct({"authTransformId": SimStruct({"authType": SimTypeInt(signed=False, label="IPSEC_AUTH_TYPE"), "authConfig": SimTypeChar(label="Byte")}, name="IPSEC_AUTH_TRANSFORM_ID0", pack=False, align=None), "cryptoModuleId": SimTypePointer(SimTypeBottom(label="Guid"), offset=0)}, name="IPSEC_AUTH_TRANSFORM0", pack=False, align=None), "cipherTransform": SimStruct({"cipherTransformId": SimStruct({"cipherType": SimTypeInt(signed=False, label="IPSEC_CIPHER_TYPE"), "cipherConfig": SimTypeChar(label="Byte")}, name="IPSEC_CIPHER_TRANSFORM_ID0", pack=False, align=None), "cryptoModuleId": SimTypePointer(SimTypeBottom(label="Guid"), offset=0)}, name="IPSEC_CIPHER_TRANSFORM0", pack=False, align=None)}, name="IPSEC_AUTH_AND_CIPHER_TRANSFORM0", pack=False, align=None), offset=0), "espAuthFwTransform": SimTypePointer(SimStruct({"authTransformId": SimStruct({"authType": SimTypeInt(signed=False, label="IPSEC_AUTH_TYPE"), "authConfig": SimTypeChar(label="Byte")}, name="IPSEC_AUTH_TRANSFORM_ID0", pack=False, align=None), "cryptoModuleId": SimTypePointer(SimTypeBottom(label="Guid"), offset=0)}, name="IPSEC_AUTH_TRANSFORM0", pack=False, align=None), offset=0)}, name="<anon>", label="None")}, name="IPSEC_SA_TRANSFORM0", pack=False, align=None), offset=0), "pfsGroup": SimTypeInt(signed=False, label="IPSEC_PFS_GROUP")}, name="IPSEC_PROPOSAL0", pack=False, align=None), offset=0), "tunnelEndpoints": SimStruct({"ipVersion": SimTypeInt(signed=False, label="FWP_IP_VERSION"), "Anonymous1": SimUnion({"localV4Address": SimTypeInt(signed=False, label="UInt32"), "localV6Address": SimTypeFixedSizeArray(SimTypeChar(label="Byte"), 16)}, name="<anon>", label="None"), "Anonymous2": SimUnion({"remoteV4Address": SimTypeInt(signed=False, label="UInt32"), "remoteV6Address": SimTypeFixedSizeArray(SimTypeChar(label="Byte"), 16)}, name="<anon>", label="None")}, name="IPSEC_TUNNEL_ENDPOINTS0", pack=False, align=None), "saIdleTimeout": SimStruct({"idleTimeoutSeconds": SimTypeInt(signed=False, label="UInt32"), "idleTimeoutSecondsFailOver": SimTypeInt(signed=False, label="UInt32")}, name="IPSEC_SA_IDLE_TIMEOUT0", pack=False, align=None), "emPolicy": SimTypePointer(SimStruct({"numAuthenticationMethods": SimTypeInt(signed=False, label="UInt32"), "authenticationMethods": SimTypePointer(SimStruct({"authenticationMethodType": SimTypeInt(signed=False, label="IKEEXT_AUTHENTICATION_METHOD_TYPE"), "Anonymous": SimUnion({"presharedKeyAuthentication": SimStruct({"presharedKey": SimStruct({"size": SimTypeInt(signed=False, label="UInt32"), "data": SimTypePointer(SimTypeChar(label="Byte"), offset=0)}, name="FWP_BYTE_BLOB", pack=False, align=None)}, name="IKEEXT_PRESHARED_KEY_AUTHENTICATION0", pack=False, align=None), "certificateAuthentication": SimStruct({"inboundConfigType": SimTypeInt(signed=False, label="IKEEXT_CERT_CONFIG_TYPE"), "Anonymous1": SimUnion({"Anonymous": SimStruct({"inboundRootArraySize": SimTypeInt(signed=False, label="UInt32"), "inboundRootArray": SimTypePointer(SimStruct({"certData": SimStruct({"size": SimTypeInt(signed=False, label="UInt32"), "data": SimTypePointer(SimTypeChar(label="Byte"), offset=0)}, name="FWP_BYTE_BLOB", pack=False, align=None), "flags": SimTypeInt(signed=False, label="IKEEXT_CERT_FLAGS")}, name="IKEEXT_CERT_ROOT_CONFIG0", pack=False, align=None), offset=0)}, name="_Anonymous_e__Struct", pack=False, align=None), "inboundEnterpriseStoreConfig": SimTypePointer(SimStruct({"certData": SimStruct({"size": SimTypeInt(signed=False, label="UInt32"), "data": SimTypePointer(SimTypeChar(label="Byte"), offset=0)}, name="FWP_BYTE_BLOB", pack=False, align=None), "flags": SimTypeInt(signed=False, label="IKEEXT_CERT_FLAGS")}, name="IKEEXT_CERT_ROOT_CONFIG0", pack=False, align=None), offset=0), "inboundTrustedRootStoreConfig": SimTypePointer(SimStruct({"certData": SimStruct({"size": SimTypeInt(signed=False, label="UInt32"), "data": SimTypePointer(SimTypeChar(label="Byte"), offset=0)}, name="FWP_BYTE_BLOB", pack=False, align=None), "flags": SimTypeInt(signed=False, label="IKEEXT_CERT_FLAGS")}, name="IKEEXT_CERT_ROOT_CONFIG0", pack=False, align=None), offset=0)}, name="<anon>", label="None"), "outboundConfigType": SimTypeInt(signed=False, label="IKEEXT_CERT_CONFIG_TYPE"), "Anonymous2": SimUnion({"Anonymous": SimStruct({"outboundRootArraySize": SimTypeInt(signed=False, label="UInt32"), "outboundRootArray": SimTypePointer(SimStruct({"certData": SimStruct({"size": SimTypeInt(signed=False, label="UInt32"), "data": SimTypePointer(SimTypeChar(label="Byte"), offset=0)}, name="FWP_BYTE_BLOB", pack=False, align=None), "flags": SimTypeInt(signed=False, label="IKEEXT_CERT_FLAGS")}, name="IKEEXT_CERT_ROOT_CONFIG0", pack=False, align=None), offset=0)}, name="_Anonymous_e__Struct", pack=False, align=None), "outboundEnterpriseStoreConfig": SimTypePointer(SimStruct({"certData": SimStruct({"size": SimTypeInt(signed=False, label="UInt32"), "data": SimTypePointer(SimTypeChar(label="Byte"), offset=0)}, name="FWP_BYTE_BLOB", pack=False, align=None), "flags": SimTypeInt(signed=False, label="IKEEXT_CERT_FLAGS")}, name="IKEEXT_CERT_ROOT_CONFIG0", pack=False, align=None), offset=0), "outboundTrustedRootStoreConfig": SimTypePointer(SimStruct({"certData": SimStruct({"size": SimTypeInt(signed=False, label="UInt32"), "data": SimTypePointer(SimTypeChar(label="Byte"), offset=0)}, name="FWP_BYTE_BLOB", pack=False, align=None), "flags": SimTypeInt(signed=False, label="IKEEXT_CERT_FLAGS")}, name="IKEEXT_CERT_ROOT_CONFIG0", pack=False, align=None), offset=0)}, name="<anon>", label="None"), "flags": SimTypeInt(signed=False, label="IKEEXT_CERT_AUTH")}, name="IKEEXT_CERTIFICATE_AUTHENTICATION0", pack=False, align=None), "kerberosAuthentication": SimStruct({"flags": SimTypeInt(signed=False, label="IKEEXT_KERBEROS_AUTHENTICATION_FLAGS")}, name="IKEEXT_KERBEROS_AUTHENTICATION0", pack=False, align=None), "ntlmV2Authentication": SimStruct({"flags": SimTypeInt(signed=False, label="UInt32")}, name="IKEEXT_NTLM_V2_AUTHENTICATION0", pack=False, align=None), "sslAuthentication": SimStruct({"inboundConfigType": SimTypeInt(signed=False, label="IKEEXT_CERT_CONFIG_TYPE"), "Anonymous1": SimUnion({"Anonymous": SimStruct({"inboundRootArraySize": SimTypeInt(signed=False, label="UInt32"), "inboundRootArray": SimTypePointer(SimStruct({"certData": SimStruct({"size": SimTypeInt(signed=False, label="UInt32"), "data": SimTypePointer(SimTypeChar(label="Byte"), offset=0)}, name="FWP_BYTE_BLOB", pack=False, align=None), "flags": SimTypeInt(signed=False, label="IKEEXT_CERT_FLAGS")}, name="IKEEXT_CERT_ROOT_CONFIG0", pack=False, align=None), offset=0)}, name="_Anonymous_e__Struct", pack=False, align=None), "inboundEnterpriseStoreConfig": SimTypePointer(SimStruct({"certData": SimStruct({"size": SimTypeInt(signed=False, label="UInt32"), "data": SimTypePointer(SimTypeChar(label="Byte"), offset=0)}, name="FWP_BYTE_BLOB", pack=False, align=None), "flags": SimTypeInt(signed=False, label="IKEEXT_CERT_FLAGS")}, name="IKEEXT_CERT_ROOT_CONFIG0", pack=False, align=None), offset=0), "inboundTrustedRootStoreConfig": SimTypePointer(SimStruct({"certData": SimStruct({"size": SimTypeInt(signed=False, label="UInt32"), "data": SimTypePointer(SimTypeChar(label="Byte"), offset=0)}, name="FWP_BYTE_BLOB", pack=False, align=None), "flags": SimTypeInt(signed=False, label="IKEEXT_CERT_FLAGS")}, name="IKEEXT_CERT_ROOT_CONFIG0", pack=False, align=None), offset=0)}, name="<anon>", label="None"), "outboundConfigType": SimTypeInt(signed=False, label="IKEEXT_CERT_CONFIG_TYPE"), "Anonymous2": SimUnion({"Anonymous": SimStruct({"outboundRootArraySize": SimTypeInt(signed=False, label="UInt32"), "outboundRootArray": SimTypePointer(SimStruct({"certData": SimStruct({"size": SimTypeInt(signed=False, label="UInt32"), "data": SimTypePointer(SimTypeChar(label="Byte"), offset=0)}, name="FWP_BYTE_BLOB", pack=False, align=None), "flags": SimTypeInt(signed=False, label="IKEEXT_CERT_FLAGS")}, name="IKEEXT_CERT_ROOT_CONFIG0", pack=False, align=None), offset=0)}, name="_Anonymous_e__Struct", pack=False, align=None), "outboundEnterpriseStoreConfig": SimTypePointer(SimStruct({"certData": SimStruct({"size": SimTypeInt(signed=False, label="UInt32"), "data": SimTypePointer(SimTypeChar(label="Byte"), offset=0)}, name="FWP_BYTE_BLOB", pack=False, align=None), "flags": SimTypeInt(signed=False, label="IKEEXT_CERT_FLAGS")}, name="IKEEXT_CERT_ROOT_CONFIG0", pack=False, align=None), offset=0), "outboundTrustedRootStoreConfig": SimTypePointer(SimStruct({"certData": SimStruct({"size": SimTypeInt(signed=False, label="UInt32"), "data": SimTypePointer(SimTypeChar(label="Byte"), offset=0)}, name="FWP_BYTE_BLOB", pack=False, align=None), "flags": SimTypeInt(signed=False, label="IKEEXT_CERT_FLAGS")}, name="IKEEXT_CERT_ROOT_CONFIG0", pack=False, align=None), offset=0)}, name="<anon>", label="None"), "flags": SimTypeInt(signed=False, label="IKEEXT_CERT_AUTH")}, name="IKEEXT_CERTIFICATE_AUTHENTICATION0", pack=False, align=None), "cgaAuthentication": SimStruct({"keyContainerName": SimTypePointer(SimTypeChar(label="Char"), offset=0), "cspName": SimTypePointer(SimTypeChar(label="Char"), offset=0), "cspType": SimTypeInt(signed=False, label="UInt32"), "cgaModifier": SimStruct({"byteArray16": SimTypeFixedSizeArray(SimTypeChar(label="Byte"), 16)}, name="FWP_BYTE_ARRAY16", pack=False, align=None), "cgaCollisionCount": SimTypeChar(label="Byte")}, name="IKEEXT_IPV6_CGA_AUTHENTICATION0", pack=False, align=None)}, name="<anon>", label="None")}, name="IKEEXT_AUTHENTICATION_METHOD0", pack=False, align=None), offset=0), "initiatorImpersonationType": SimTypeInt(signed=False, label="IKEEXT_AUTHENTICATION_IMPERSONATION_TYPE")}, name="IKEEXT_EM_POLICY0", pack=False, align=None), offset=0)}, name="IPSEC_TUNNEL_POLICY0", pack=False, align=None), offset=0), "ikeMmPolicy": SimTypePointer(SimStruct({"softExpirationTime": SimTypeInt(signed=False, label="UInt32"), "numAuthenticationMethods": SimTypeInt(signed=False, label="UInt32"), "authenticationMethods": SimTypePointer(SimStruct({"authenticationMethodType": SimTypeInt(signed=False, label="IKEEXT_AUTHENTICATION_METHOD_TYPE"), "Anonymous": SimUnion({"presharedKeyAuthentication": SimStruct({"presharedKey": SimStruct({"size": SimTypeInt(signed=False, label="UInt32"), "data": SimTypePointer(SimTypeChar(label="Byte"), offset=0)}, name="FWP_BYTE_BLOB", pack=False, align=None)}, name="IKEEXT_PRESHARED_KEY_AUTHENTICATION0", pack=False, align=None), "certificateAuthentication": SimStruct({"inboundConfigType": SimTypeInt(signed=False, label="IKEEXT_CERT_CONFIG_TYPE"), "Anonymous1": SimUnion({"Anonymous": SimStruct({"inboundRootArraySize": SimTypeInt(signed=False, label="UInt32"), "inboundRootArray": SimTypePointer(SimStruct({"certData": SimStruct({"size": SimTypeInt(signed=False, label="UInt32"), "data": SimTypePointer(SimTypeChar(label="Byte"), offset=0)}, name="FWP_BYTE_BLOB", pack=False, align=None), "flags": SimTypeInt(signed=False, label="IKEEXT_CERT_FLAGS")}, name="IKEEXT_CERT_ROOT_CONFIG0", pack=False, align=None), offset=0)}, name="_Anonymous_e__Struct", pack=False, align=None), "inboundEnterpriseStoreConfig": SimTypePointer(SimStruct({"certData": SimStruct({"size": SimTypeInt(signed=False, label="UInt32"), "data": SimTypePointer(SimTypeChar(label="Byte"), offset=0)}, name="FWP_BYTE_BLOB", pack=False, align=None), "flags": SimTypeInt(signed=False, label="IKEEXT_CERT_FLAGS")}, name="IKEEXT_CERT_ROOT_CONFIG0", pack=False, align=None), offset=0), "inboundTrustedRootStoreConfig": SimTypePointer(SimStruct({"certData": SimStruct({"size": SimTypeInt(signed=False, label="UInt32"), "data": SimTypePointer(SimTypeChar(label="Byte"), offset=0)}, name="FWP_BYTE_BLOB", pack=False, align=None), "flags": SimTypeInt(signed=False, label="IKEEXT_CERT_FLAGS")}, name="IKEEXT_CERT_ROOT_CONFIG0", pack=False, align=None), offset=0)}, name="<anon>", label="None"), "outboundConfigType": SimTypeInt(signed=False, label="IKEEXT_CERT_CONFIG_TYPE"), "Anonymous2": SimUnion({"Anonymous": SimStruct({"outboundRootArraySize": SimTypeInt(signed=False, label="UInt32"), "outboundRootArray": SimTypePointer(SimStruct({"certData": SimStruct({"size": SimTypeInt(signed=False, label="UInt32"), "data": SimTypePointer(SimTypeChar(label="Byte"), offset=0)}, name="FWP_BYTE_BLOB", pack=False, align=None), "flags": SimTypeInt(signed=False, label="IKEEXT_CERT_FLAGS")}, name="IKEEXT_CERT_ROOT_CONFIG0", pack=False, align=None), offset=0)}, name="_Anonymous_e__Struct", pack=False, align=None), "outboundEnterpriseStoreConfig": SimTypePointer(SimStruct({"certData": SimStruct({"size": SimTypeInt(signed=False, label="UInt32"), "data": SimTypePointer(SimTypeChar(label="Byte"), offset=0)}, name="FWP_BYTE_BLOB", pack=False, align=None), "flags": SimTypeInt(signed=False, label="IKEEXT_CERT_FLAGS")}, name="IKEEXT_CERT_ROOT_CONFIG0", pack=False, align=None), offset=0), "outboundTrustedRootStoreConfig": SimTypePointer(SimStruct({"certData": SimStruct({"size": SimTypeInt(signed=False, label="UInt32"), "data": SimTypePointer(SimTypeChar(label="Byte"), offset=0)}, name="FWP_BYTE_BLOB", pack=False, align=None), "flags": SimTypeInt(signed=False, label="IKEEXT_CERT_FLAGS")}, name="IKEEXT_CERT_ROOT_CONFIG0", pack=False, align=None), offset=0)}, name="<anon>", label="None"), "flags": SimTypeInt(signed=False, label="IKEEXT_CERT_AUTH")}, name="IKEEXT_CERTIFICATE_AUTHENTICATION0", pack=False, align=None), "kerberosAuthentication": SimStruct({"flags": SimTypeInt(signed=False, label="IKEEXT_KERBEROS_AUTHENTICATION_FLAGS")}, name="IKEEXT_KERBEROS_AUTHENTICATION0", pack=False, align=None), "ntlmV2Authentication": SimStruct({"flags": SimTypeInt(signed=False, label="UInt32")}, name="IKEEXT_NTLM_V2_AUTHENTICATION0", pack=False, align=None), "sslAuthentication": SimStruct({"inboundConfigType": SimTypeInt(signed=False, label="IKEEXT_CERT_CONFIG_TYPE"), "Anonymous1": SimUnion({"Anonymous": SimStruct({"inboundRootArraySize": SimTypeInt(signed=False, label="UInt32"), "inboundRootArray": SimTypePointer(SimStruct({"certData": SimStruct({"size": SimTypeInt(signed=False, label="UInt32"), "data": SimTypePointer(SimTypeChar(label="Byte"), offset=0)}, name="FWP_BYTE_BLOB", pack=False, align=None), "flags": SimTypeInt(signed=False, label="IKEEXT_CERT_FLAGS")}, name="IKEEXT_CERT_ROOT_CONFIG0", pack=False, align=None), offset=0)}, name="_Anonymous_e__Struct", pack=False, align=None), "inboundEnterpriseStoreConfig": SimTypePointer(SimStruct({"certData": SimStruct({"size": SimTypeInt(signed=False, label="UInt32"), "data": SimTypePointer(SimTypeChar(label="Byte"), offset=0)}, name="FWP_BYTE_BLOB", pack=False, align=None), "flags": SimTypeInt(signed=False, label="IKEEXT_CERT_FLAGS")}, name="IKEEXT_CERT_ROOT_CONFIG0", pack=False, align=None), offset=0), "inboundTrustedRootStoreConfig": SimTypePointer(SimStruct({"certData": SimStruct({"size": SimTypeInt(signed=False, label="UInt32"), "data": SimTypePointer(SimTypeChar(label="Byte"), offset=0)}, name="FWP_BYTE_BLOB", pack=False, align=None), "flags": SimTypeInt(signed=False, label="IKEEXT_CERT_FLAGS")}, name="IKEEXT_CERT_ROOT_CONFIG0", pack=False, align=None), offset=0)}, name="<anon>", label="None"), "outboundConfigType": SimTypeInt(signed=False, label="IKEEXT_CERT_CONFIG_TYPE"), "Anonymous2": SimUnion({"Anonymous": SimStruct({"outboundRootArraySize": SimTypeInt(signed=False, label="UInt32"), "outboundRootArray": SimTypePointer(SimStruct({"certData": SimStruct({"size": SimTypeInt(signed=False, label="UInt32"), "data": SimTypePointer(SimTypeChar(label="Byte"), offset=0)}, name="FWP_BYTE_BLOB", pack=False, align=None), "flags": SimTypeInt(signed=False, label="IKEEXT_CERT_FLAGS")}, name="IKEEXT_CERT_ROOT_CONFIG0", pack=False, align=None), offset=0)}, name="_Anonymous_e__Struct", pack=False, align=None), "outboundEnterpriseStoreConfig": SimTypePointer(SimStruct({"certData": SimStruct({"size": SimTypeInt(signed=False, label="UInt32"), "data": SimTypePointer(SimTypeChar(label="Byte"), offset=0)}, name="FWP_BYTE_BLOB", pack=False, align=None), "flags": SimTypeInt(signed=False, label="IKEEXT_CERT_FLAGS")}, name="IKEEXT_CERT_ROOT_CONFIG0", pack=False, align=None), offset=0), "outboundTrustedRootStoreConfig": SimTypePointer(SimStruct({"certData": SimStruct({"size": SimTypeInt(signed=False, label="UInt32"), "data": SimTypePointer(SimTypeChar(label="Byte"), offset=0)}, name="FWP_BYTE_BLOB", pack=False, align=None), "flags": SimTypeInt(signed=False, label="IKEEXT_CERT_FLAGS")}, name="IKEEXT_CERT_ROOT_CONFIG0", pack=False, align=None), offset=0)}, name="<anon>", label="None"), "flags": SimTypeInt(signed=False, label="IKEEXT_CERT_AUTH")}, name="IKEEXT_CERTIFICATE_AUTHENTICATION0", pack=False, align=None), "cgaAuthentication": SimStruct({"keyContainerName": SimTypePointer(SimTypeChar(label="Char"), offset=0), "cspName": SimTypePointer(SimTypeChar(label="Char"), offset=0), "cspType": SimTypeInt(signed=False, label="UInt32"), "cgaModifier": SimStruct({"byteArray16": SimTypeFixedSizeArray(SimTypeChar(label="Byte"), 16)}, name="FWP_BYTE_ARRAY16", pack=False, align=None), "cgaCollisionCount": SimTypeChar(label="Byte")}, name="IKEEXT_IPV6_CGA_AUTHENTICATION0", pack=False, align=None)}, name="<anon>", label="None")}, name="IKEEXT_AUTHENTICATION_METHOD0", pack=False, align=None), offset=0), "initiatorImpersonationType": SimTypeInt(signed=False, label="IKEEXT_AUTHENTICATION_IMPERSONATION_TYPE"), "numIkeProposals": SimTypeInt(signed=False, label="UInt32"), "ikeProposals": SimTypePointer(SimStruct({"cipherAlgorithm": SimStruct({"algoIdentifier": SimTypeInt(signed=False, label="IKEEXT_CIPHER_TYPE"), "keyLen": SimTypeInt(signed=False, label="UInt32"), "rounds": SimTypeInt(signed=False, label="UInt32")}, name="IKEEXT_CIPHER_ALGORITHM0", pack=False, align=None), "integrityAlgorithm": SimStruct({"algoIdentifier": SimTypeInt(signed=False, label="IKEEXT_INTEGRITY_TYPE")}, name="IKEEXT_INTEGRITY_ALGORITHM0", pack=False, align=None), "maxLifetimeSeconds": SimTypeInt(signed=False, label="UInt32"), "dhGroup": SimTypeInt(signed=False, label="IKEEXT_DH_GROUP"), "quickModeLimit": SimTypeInt(signed=False, label="UInt32")}, name="IKEEXT_PROPOSAL0", pack=False, align=None), offset=0), "flags": SimTypeInt(signed=False, label="IKEEXT_POLICY_FLAG"), "maxDynamicFilters": SimTypeInt(signed=False, label="UInt32")}, name="IKEEXT_POLICY0", pack=False, align=None), offset=0), "authIpMmPolicy": SimTypePointer(SimStruct({"softExpirationTime": SimTypeInt(signed=False, label="UInt32"), "numAuthenticationMethods": SimTypeInt(signed=False, label="UInt32"), "authenticationMethods": SimTypePointer(SimStruct({"authenticationMethodType": SimTypeInt(signed=False, label="IKEEXT_AUTHENTICATION_METHOD_TYPE"), "Anonymous": SimUnion({"presharedKeyAuthentication": SimStruct({"presharedKey": SimStruct({"size": SimTypeInt(signed=False, label="UInt32"), "data": SimTypePointer(SimTypeChar(label="Byte"), offset=0)}, name="FWP_BYTE_BLOB", pack=False, align=None)}, name="IKEEXT_PRESHARED_KEY_AUTHENTICATION0", pack=False, align=None), "certificateAuthentication": SimStruct({"inboundConfigType": SimTypeInt(signed=False, label="IKEEXT_CERT_CONFIG_TYPE"), "Anonymous1": SimUnion({"Anonymous": SimStruct({"inboundRootArraySize": SimTypeInt(signed=False, label="UInt32"), "inboundRootArray": SimTypePointer(SimStruct({"certData": SimStruct({"size": SimTypeInt(signed=False, label="UInt32"), "data": SimTypePointer(SimTypeChar(label="Byte"), offset=0)}, name="FWP_BYTE_BLOB", pack=False, align=None), "flags": SimTypeInt(signed=False, label="IKEEXT_CERT_FLAGS")}, name="IKEEXT_CERT_ROOT_CONFIG0", pack=False, align=None), offset=0)}, name="_Anonymous_e__Struct", pack=False, align=None), "inboundEnterpriseStoreConfig": SimTypePointer(SimStruct({"certData": SimStruct({"size": SimTypeInt(signed=False, label="UInt32"), "data": SimTypePointer(SimTypeChar(label="Byte"), offset=0)}, name="FWP_BYTE_BLOB", pack=False, align=None), "flags": SimTypeInt(signed=False, label="IKEEXT_CERT_FLAGS")}, name="IKEEXT_CERT_ROOT_CONFIG0", pack=False, align=None), offset=0), "inboundTrustedRootStoreConfig": SimTypePointer(SimStruct({"certData": SimStruct({"size": SimTypeInt(signed=False, label="UInt32"), "data": SimTypePointer(SimTypeChar(label="Byte"), offset=0)}, name="FWP_BYTE_BLOB", pack=False, align=None), "flags": SimTypeInt(signed=False, label="IKEEXT_CERT_FLAGS")}, name="IKEEXT_CERT_ROOT_CONFIG0", pack=False, align=None), offset=0)}, name="<anon>", label="None"), "outboundConfigType": SimTypeInt(signed=False, label="IKEEXT_CERT_CONFIG_TYPE"), "Anonymous2": SimUnion({"Anonymous": SimStruct({"outboundRootArraySize": SimTypeInt(signed=False, label="UInt32"), "outboundRootArray": SimTypePointer(SimStruct({"certData": SimStruct({"size": SimTypeInt(signed=False, label="UInt32"), "data": SimTypePointer(SimTypeChar(label="Byte"), offset=0)}, name="FWP_BYTE_BLOB", pack=False, align=None), "flags": SimTypeInt(signed=False, label="IKEEXT_CERT_FLAGS")}, name="IKEEXT_CERT_ROOT_CONFIG0", pack=False, align=None), offset=0)}, name="_Anonymous_e__Struct", pack=False, align=None), "outboundEnterpriseStoreConfig": SimTypePointer(SimStruct({"certData": SimStruct({"size": SimTypeInt(signed=False, label="UInt32"), "data": SimTypePointer(SimTypeChar(label="Byte"), offset=0)}, name="FWP_BYTE_BLOB", pack=False, align=None), "flags": SimTypeInt(signed=False, label="IKEEXT_CERT_FLAGS")}, name="IKEEXT_CERT_ROOT_CONFIG0", pack=False, align=None), offset=0), "outboundTrustedRootStoreConfig": SimTypePointer(SimStruct({"certData": SimStruct({"size": SimTypeInt(signed=False, label="UInt32"), "data": SimTypePointer(SimTypeChar(label="Byte"), offset=0)}, name="FWP_BYTE_BLOB", pack=False, align=None), "flags": SimTypeInt(signed=False, label="IKEEXT_CERT_FLAGS")}, name="IKEEXT_CERT_ROOT_CONFIG0", pack=False, align=None), offset=0)}, name="<anon>", label="None"), "flags": SimTypeInt(signed=False, label="IKEEXT_CERT_AUTH")}, name="IKEEXT_CERTIFICATE_AUTHENTICATION0", pack=False, align=None), "kerberosAuthentication": SimStruct({"flags": SimTypeInt(signed=False, label="IKEEXT_KERBEROS_AUTHENTICATION_FLAGS")}, name="IKEEXT_KERBEROS_AUTHENTICATION0", pack=False, align=None), "ntlmV2Authentication": SimStruct({"flags": SimTypeInt(signed=False, label="UInt32")}, name="IKEEXT_NTLM_V2_AUTHENTICATION0", pack=False, align=None), "sslAuthentication": SimStruct({"inboundConfigType": SimTypeInt(signed=False, label="IKEEXT_CERT_CONFIG_TYPE"), "Anonymous1": SimUnion({"Anonymous": SimStruct({"inboundRootArraySize": SimTypeInt(signed=False, label="UInt32"), "inboundRootArray": SimTypePointer(SimStruct({"certData": SimStruct({"size": SimTypeInt(signed=False, label="UInt32"), "data": SimTypePointer(SimTypeChar(label="Byte"), offset=0)}, name="FWP_BYTE_BLOB", pack=False, align=None), "flags": SimTypeInt(signed=False, label="IKEEXT_CERT_FLAGS")}, name="IKEEXT_CERT_ROOT_CONFIG0", pack=False, align=None), offset=0)}, name="_Anonymous_e__Struct", pack=False, align=None), "inboundEnterpriseStoreConfig": SimTypePointer(SimStruct({"certData": SimStruct({"size": SimTypeInt(signed=False, label="UInt32"), "data": SimTypePointer(SimTypeChar(label="Byte"), offset=0)}, name="FWP_BYTE_BLOB", pack=False, align=None), "flags": SimTypeInt(signed=False, label="IKEEXT_CERT_FLAGS")}, name="IKEEXT_CERT_ROOT_CONFIG0", pack=False, align=None), offset=0), "inboundTrustedRootStoreConfig": SimTypePointer(SimStruct({"certData": SimStruct({"size": SimTypeInt(signed=False, label="UInt32"), "data": SimTypePointer(SimTypeChar(label="Byte"), offset=0)}, name="FWP_BYTE_BLOB", pack=False, align=None), "flags": SimTypeInt(signed=False, label="IKEEXT_CERT_FLAGS")}, name="IKEEXT_CERT_ROOT_CONFIG0", pack=False, align=None), offset=0)}, name="<anon>", label="None"), "outboundConfigType": SimTypeInt(signed=False, label="IKEEXT_CERT_CONFIG_TYPE"), "Anonymous2": SimUnion({"Anonymous": SimStruct({"outboundRootArraySize": SimTypeInt(signed=False, label="UInt32"), "outboundRootArray": SimTypePointer(SimStruct({"certData": SimStruct({"size": SimTypeInt(signed=False, label="UInt32"), "data": SimTypePointer(SimTypeChar(label="Byte"), offset=0)}, name="FWP_BYTE_BLOB", pack=False, align=None), "flags": SimTypeInt(signed=False, label="IKEEXT_CERT_FLAGS")}, name="IKEEXT_CERT_ROOT_CONFIG0", pack=False, align=None), offset=0)}, name="_Anonymous_e__Struct", pack=False, align=None), "outboundEnterpriseStoreConfig": SimTypePointer(SimStruct({"certData": SimStruct({"size": SimTypeInt(signed=False, label="UInt32"), "data": SimTypePointer(SimTypeChar(label="Byte"), offset=0)}, name="FWP_BYTE_BLOB", pack=False, align=None), "flags": SimTypeInt(signed=False, label="IKEEXT_CERT_FLAGS")}, name="IKEEXT_CERT_ROOT_CONFIG0", pack=False, align=None), offset=0), "outboundTrustedRootStoreConfig": SimTypePointer(SimStruct({"certData": SimStruct({"size": SimTypeInt(signed=False, label="UInt32"), "data": SimTypePointer(SimTypeChar(label="Byte"), offset=0)}, name="FWP_BYTE_BLOB", pack=False, align=None), "flags": SimTypeInt(signed=False, label="IKEEXT_CERT_FLAGS")}, name="IKEEXT_CERT_ROOT_CONFIG0", pack=False, align=None), offset=0)}, name="<anon>", label="None"), "flags": SimTypeInt(signed=False, label="IKEEXT_CERT_AUTH")}, name="IKEEXT_CERTIFICATE_AUTHENTICATION0", pack=False, align=None), "cgaAuthentication": SimStruct({"keyContainerName": SimTypePointer(SimTypeChar(label="Char"), offset=0), "cspName": SimTypePointer(SimTypeChar(label="Char"), offset=0), "cspType": SimTypeInt(signed=False, label="UInt32"), "cgaModifier": SimStruct({"byteArray16": SimTypeFixedSizeArray(SimTypeChar(label="Byte"), 16)}, name="FWP_BYTE_ARRAY16", pack=False, align=None), "cgaCollisionCount": SimTypeChar(label="Byte")}, name="IKEEXT_IPV6_CGA_AUTHENTICATION0", pack=False, align=None)}, name="<anon>", label="None")}, name="IKEEXT_AUTHENTICATION_METHOD0", pack=False, align=None), offset=0), "initiatorImpersonationType": SimTypeInt(signed=False, label="IKEEXT_AUTHENTICATION_IMPERSONATION_TYPE"), "numIkeProposals": SimTypeInt(signed=False, label="UInt32"), "ikeProposals": SimTypePointer(SimStruct({"cipherAlgorithm": SimStruct({"algoIdentifier": SimTypeInt(signed=False, label="IKEEXT_CIPHER_TYPE"), "keyLen": SimTypeInt(signed=False, label="UInt32"), "rounds": SimTypeInt(signed=False, label="UInt32")}, name="IKEEXT_CIPHER_ALGORITHM0", pack=False, align=None), "integrityAlgorithm": SimStruct({"algoIdentifier": SimTypeInt(signed=False, label="IKEEXT_INTEGRITY_TYPE")}, name="IKEEXT_INTEGRITY_ALGORITHM0", pack=False, align=None), "maxLifetimeSeconds": SimTypeInt(signed=False, label="UInt32"), "dhGroup": SimTypeInt(signed=False, label="IKEEXT_DH_GROUP"), "quickModeLimit": SimTypeInt(signed=False, label="UInt32")}, name="IKEEXT_PROPOSAL0", pack=False, align=None), offset=0), "flags": SimTypeInt(signed=False, label="IKEEXT_POLICY_FLAG"), "maxDynamicFilters": SimTypeInt(signed=False, label="UInt32")}, name="IKEEXT_POLICY0", pack=False, align=None), offset=0), "dataBuffer": SimTypePointer(SimStruct({"size": SimTypeInt(signed=False, label="UInt32"), "data": SimTypePointer(SimTypeChar(label="Byte"), offset=0)}, name="FWP_BYTE_BLOB", pack=False, align=None), offset=0), "classifyOptions": SimTypePointer(SimStruct({"numOptions": SimTypeInt(signed=False, label="UInt32"), "options": SimTypePointer(SimStruct({"type": SimTypeInt(signed=False, label="FWP_CLASSIFY_OPTION_TYPE"), "value": SimStruct({"type": SimTypeInt(signed=False, label="FWP_DATA_TYPE"), "Anonymous": SimUnion({"uint8": SimTypeChar(label="Byte"), "uint16": SimTypeShort(signed=False, label="UInt16"), "uint32": SimTypeInt(signed=False, label="UInt32"), "uint64": SimTypePointer(SimTypeLongLong(signed=False, label="UInt64"), offset=0), "int8": SimTypeChar(label="SByte"), "int16": SimTypeShort(signed=True, label="Int16"), "int32": SimTypeInt(signed=True, label="Int32"), "int64": SimTypePointer(SimTypeLongLong(signed=True, label="Int64"), offset=0), "float32": SimTypeFloat(size=32), "double64": SimTypePointer(SimTypeFloat(size=64), offset=0), "byteArray16": SimTypePointer(SimStruct({"byteArray16": SimTypeFixedSizeArray(SimTypeChar(label="Byte"), 16)}, name="FWP_BYTE_ARRAY16", pack=False, align=None), offset=0), "byteBlob": SimTypePointer(SimStruct({"size": SimTypeInt(signed=False, label="UInt32"), "data": SimTypePointer(SimTypeChar(label="Byte"), offset=0)}, name="FWP_BYTE_BLOB", pack=False, align=None), offset=0), "sid": SimTypePointer(SimTypeBottom(label="SID"), offset=0), "sd": SimTypePointer(SimStruct({"size": SimTypeInt(signed=False, label="UInt32"), "data": SimTypePointer(SimTypeChar(label="Byte"), offset=0)}, name="FWP_BYTE_BLOB", pack=False, align=None), offset=0), "tokenInformation": SimTypePointer(SimStruct({"sidCount": SimTypeInt(signed=False, label="UInt32"), "sids": SimTypePointer(SimTypeBottom(label="SID_AND_ATTRIBUTES"), offset=0), "restrictedSidCount": SimTypeInt(signed=False, label="UInt32"), "restrictedSids": SimTypePointer(SimTypeBottom(label="SID_AND_ATTRIBUTES"), offset=0)}, name="FWP_TOKEN_INFORMATION", pack=False, align=None), offset=0), "tokenAccessInformation": SimTypePointer(SimStruct({"size": SimTypeInt(signed=False, label="UInt32"), "data": SimTypePointer(SimTypeChar(label="Byte"), offset=0)}, name="FWP_BYTE_BLOB", pack=False, align=None), offset=0), "unicodeString": SimTypePointer(SimTypeChar(label="Char"), offset=0), "byteArray6": SimTypePointer(SimStruct({"byteArray6": SimTypeFixedSizeArray(SimTypeChar(label="Byte"), 6)}, name="FWP_BYTE_ARRAY6", pack=False, align=None), offset=0), "bitmapArray64": SimTypePointer(SimStruct({"bitmapArray64": SimTypeFixedSizeArray(SimTypeChar(label="Byte"), 8)}, name="FWP_BITMAP_ARRAY64_", pack=False, align=None), offset=0)}, name="<anon>", label="None")}, name="FWP_VALUE0", pack=False, align=None)}, name="FWPM_CLASSIFY_OPTION0", pack=False, align=None), offset=0)}, name="FWPM_CLASSIFY_OPTIONS0", pack=False, align=None), offset=0)}, name="<anon>", label="None"), "providerContextId": SimTypeLongLong(signed=False, label="UInt64")}, name="FWPM_PROVIDER_CONTEXT0", pack=False, align=None), offset=0), SimTypePointer(SimStruct({"providerContextKey": SimTypeBottom(label="Guid"), "displayData": SimStruct({"name": SimTypePointer(SimTypeChar(label="Char"), offset=0), "description": SimTypePointer(SimTypeChar(label="Char"), offset=0)}, name="FWPM_DISPLAY_DATA0", pack=False, align=None), "flags": SimTypeInt(signed=False, label="UInt32"), "providerKey": SimTypePointer(SimTypeBottom(label="Guid"), offset=0), "providerData": SimStruct({"size": SimTypeInt(signed=False, label="UInt32"), "data": SimTypePointer(SimTypeChar(label="Byte"), offset=0)}, name="FWP_BYTE_BLOB", pack=False, align=None), "type": SimTypeInt(signed=False, label="FWPM_PROVIDER_CONTEXT_TYPE"), "Anonymous": SimUnion({"keyingPolicy": SimTypePointer(SimStruct({"numKeyMods": SimTypeInt(signed=False, label="UInt32"), "keyModKeys": SimTypePointer(SimTypeBottom(label="Guid"), offset=0)}, name="IPSEC_KEYING_POLICY0", pack=False, align=None), offset=0), "ikeQmTransportPolicy": SimTypePointer(SimStruct({"numIpsecProposals": SimTypeInt(signed=False, label="UInt32"), "ipsecProposals": SimTypePointer(SimStruct({"lifetime": SimStruct({"lifetimeSeconds": SimTypeInt(signed=False, label="UInt32"), "lifetimeKilobytes": SimTypeInt(signed=False, label="UInt32"), "lifetimePackets": SimTypeInt(signed=False, label="UInt32")}, name="IPSEC_SA_LIFETIME0", pack=False, align=None), "numSaTransforms": SimTypeInt(signed=False, label="UInt32"), "saTransforms": SimTypePointer(SimStruct({"ipsecTransformType": SimTypeInt(signed=False, label="IPSEC_TRANSFORM_TYPE"), "Anonymous": SimUnion({"ahTransform": SimTypePointer(SimStruct({"authTransformId": SimStruct({"authType": SimTypeInt(signed=False, label="IPSEC_AUTH_TYPE"), "authConfig": SimTypeChar(label="Byte")}, name="IPSEC_AUTH_TRANSFORM_ID0", pack=False, align=None), "cryptoModuleId": SimTypePointer(SimTypeBottom(label="Guid"), offset=0)}, name="IPSEC_AUTH_TRANSFORM0", pack=False, align=None), offset=0), "espAuthTransform": SimTypePointer(SimStruct({"authTransformId": SimStruct({"authType": SimTypeInt(signed=False, label="IPSEC_AUTH_TYPE"), "authConfig": SimTypeChar(label="Byte")}, name="IPSEC_AUTH_TRANSFORM_ID0", pack=False, align=None), "cryptoModuleId": SimTypePointer(SimTypeBottom(label="Guid"), offset=0)}, name="IPSEC_AUTH_TRANSFORM0", pack=False, align=None), offset=0), "espCipherTransform": SimTypePointer(SimStruct({"cipherTransformId": SimStruct({"cipherType": SimTypeInt(signed=False, label="IPSEC_CIPHER_TYPE"), "cipherConfig": SimTypeChar(label="Byte")}, name="IPSEC_CIPHER_TRANSFORM_ID0", pack=False, align=None), "cryptoModuleId": SimTypePointer(SimTypeBottom(label="Guid"), offset=0)}, name="IPSEC_CIPHER_TRANSFORM0", pack=False, align=None), offset=0), "espAuthAndCipherTransform": SimTypePointer(SimStruct({"authTransform": SimStruct({"authTransformId": SimStruct({"authType": SimTypeInt(signed=False, label="IPSEC_AUTH_TYPE"), "authConfig": SimTypeChar(label="Byte")}, name="IPSEC_AUTH_TRANSFORM_ID0", pack=False, align=None), "cryptoModuleId": SimTypePointer(SimTypeBottom(label="Guid"), offset=0)}, name="IPSEC_AUTH_TRANSFORM0", pack=False, align=None), "cipherTransform": SimStruct({"cipherTransformId": SimStruct({"cipherType": SimTypeInt(signed=False, label="IPSEC_CIPHER_TYPE"), "cipherConfig": SimTypeChar(label="Byte")}, name="IPSEC_CIPHER_TRANSFORM_ID0", pack=False, align=None), "cryptoModuleId": SimTypePointer(SimTypeBottom(label="Guid"), offset=0)}, name="IPSEC_CIPHER_TRANSFORM0", pack=False, align=None)}, name="IPSEC_AUTH_AND_CIPHER_TRANSFORM0", pack=False, align=None), offset=0), "espAuthFwTransform": SimTypePointer(SimStruct({"authTransformId": SimStruct({"authType": SimTypeInt(signed=False, label="IPSEC_AUTH_TYPE"), "authConfig": SimTypeChar(label="Byte")}, name="IPSEC_AUTH_TRANSFORM_ID0", pack=False, align=None), "cryptoModuleId": SimTypePointer(SimTypeBottom(label="Guid"), offset=0)}, name="IPSEC_AUTH_TRANSFORM0", pack=False, align=None), offset=0)}, name="<anon>", label="None")}, name="IPSEC_SA_TRANSFORM0", pack=False, align=None), offset=0), "pfsGroup": SimTypeInt(signed=False, label="IPSEC_PFS_GROUP")}, name="IPSEC_PROPOSAL0", pack=False, align=None), offset=0), "flags": SimTypeInt(signed=False, label="IPSEC_POLICY_FLAG"), "ndAllowClearTimeoutSeconds": SimTypeInt(signed=False, label="UInt32"), "saIdleTimeout": SimStruct({"idleTimeoutSeconds": SimTypeInt(signed=False, label="UInt32"), "idleTimeoutSecondsFailOver": SimTypeInt(signed=False, label="UInt32")}, name="IPSEC_SA_IDLE_TIMEOUT0", pack=False, align=None), "emPolicy": SimTypePointer(SimStruct({"numAuthenticationMethods": SimTypeInt(signed=False, label="UInt32"), "authenticationMethods": SimTypePointer(SimStruct({"authenticationMethodType": SimTypeInt(signed=False, label="IKEEXT_AUTHENTICATION_METHOD_TYPE"), "Anonymous": SimUnion({"presharedKeyAuthentication": SimStruct({"presharedKey": SimStruct({"size": SimTypeInt(signed=False, label="UInt32"), "data": SimTypePointer(SimTypeChar(label="Byte"), offset=0)}, name="FWP_BYTE_BLOB", pack=False, align=None)}, name="IKEEXT_PRESHARED_KEY_AUTHENTICATION0", pack=False, align=None), "certificateAuthentication": SimStruct({"inboundConfigType": SimTypeInt(signed=False, label="IKEEXT_CERT_CONFIG_TYPE"), "Anonymous1": SimUnion({"Anonymous": SimStruct({"inboundRootArraySize": SimTypeInt(signed=False, label="UInt32"), "inboundRootArray": SimTypePointer(SimStruct({"certData": SimStruct({"size": SimTypeInt(signed=False, label="UInt32"), "data": SimTypePointer(SimTypeChar(label="Byte"), offset=0)}, name="FWP_BYTE_BLOB", pack=False, align=None), "flags": SimTypeInt(signed=False, label="IKEEXT_CERT_FLAGS")}, name="IKEEXT_CERT_ROOT_CONFIG0", pack=False, align=None), offset=0)}, name="_Anonymous_e__Struct", pack=False, align=None), "inboundEnterpriseStoreConfig": SimTypePointer(SimStruct({"certData": SimStruct({"size": SimTypeInt(signed=False, label="UInt32"), "data": SimTypePointer(SimTypeChar(label="Byte"), offset=0)}, name="FWP_BYTE_BLOB", pack=False, align=None), "flags": SimTypeInt(signed=False, label="IKEEXT_CERT_FLAGS")}, name="IKEEXT_CERT_ROOT_CONFIG0", pack=False, align=None), offset=0), "inboundTrustedRootStoreConfig": SimTypePointer(SimStruct({"certData": SimStruct({"size": SimTypeInt(signed=False, label="UInt32"), "data": SimTypePointer(SimTypeChar(label="Byte"), offset=0)}, name="FWP_BYTE_BLOB", pack=False, align=None), "flags": SimTypeInt(signed=False, label="IKEEXT_CERT_FLAGS")}, name="IKEEXT_CERT_ROOT_CONFIG0", pack=False, align=None), offset=0)}, name="<anon>", label="None"), "outboundConfigType": SimTypeInt(signed=False, label="IKEEXT_CERT_CONFIG_TYPE"), "Anonymous2": SimUnion({"Anonymous": SimStruct({"outboundRootArraySize": SimTypeInt(signed=False, label="UInt32"), "outboundRootArray": SimTypePointer(SimStruct({"certData": SimStruct({"size": SimTypeInt(signed=False, label="UInt32"), "data": SimTypePointer(SimTypeChar(label="Byte"), offset=0)}, name="FWP_BYTE_BLOB", pack=False, align=None), "flags": SimTypeInt(signed=False, label="IKEEXT_CERT_FLAGS")}, name="IKEEXT_CERT_ROOT_CONFIG0", pack=False, align=None), offset=0)}, name="_Anonymous_e__Struct", pack=False, align=None), "outboundEnterpriseStoreConfig": SimTypePointer(SimStruct({"certData": SimStruct({"size": SimTypeInt(signed=False, label="UInt32"), "data": SimTypePointer(SimTypeChar(label="Byte"), offset=0)}, name="FWP_BYTE_BLOB", pack=False, align=None), "flags": SimTypeInt(signed=False, label="IKEEXT_CERT_FLAGS")}, name="IKEEXT_CERT_ROOT_CONFIG0", pack=False, align=None), offset=0), "outboundTrustedRootStoreConfig": SimTypePointer(SimStruct({"certData": SimStruct({"size": SimTypeInt(signed=False, label="UInt32"), "data": SimTypePointer(SimTypeChar(label="Byte"), offset=0)}, name="FWP_BYTE_BLOB", pack=False, align=None), "flags": SimTypeInt(signed=False, label="IKEEXT_CERT_FLAGS")}, name="IKEEXT_CERT_ROOT_CONFIG0", pack=False, align=None), offset=0)}, name="<anon>", label="None"), "flags": SimTypeInt(signed=False, label="IKEEXT_CERT_AUTH")}, name="IKEEXT_CERTIFICATE_AUTHENTICATION0", pack=False, align=None), "kerberosAuthentication": SimStruct({"flags": SimTypeInt(signed=False, label="IKEEXT_KERBEROS_AUTHENTICATION_FLAGS")}, name="IKEEXT_KERBEROS_AUTHENTICATION0", pack=False, align=None), "ntlmV2Authentication": SimStruct({"flags": SimTypeInt(signed=False, label="UInt32")}, name="IKEEXT_NTLM_V2_AUTHENTICATION0", pack=False, align=None), "sslAuthentication": SimStruct({"inboundConfigType": SimTypeInt(signed=False, label="IKEEXT_CERT_CONFIG_TYPE"), "Anonymous1": SimUnion({"Anonymous": SimStruct({"inboundRootArraySize": SimTypeInt(signed=False, label="UInt32"), "inboundRootArray": SimTypePointer(SimStruct({"certData": SimStruct({"size": SimTypeInt(signed=False, label="UInt32"), "data": SimTypePointer(SimTypeChar(label="Byte"), offset=0)}, name="FWP_BYTE_BLOB", pack=False, align=None), "flags": SimTypeInt(signed=False, label="IKEEXT_CERT_FLAGS")}, name="IKEEXT_CERT_ROOT_CONFIG0", pack=False, align=None), offset=0)}, name="_Anonymous_e__Struct", pack=False, align=None), "inboundEnterpriseStoreConfig": SimTypePointer(SimStruct({"certData": SimStruct({"size": SimTypeInt(signed=False, label="UInt32"), "data": SimTypePointer(SimTypeChar(label="Byte"), offset=0)}, name="FWP_BYTE_BLOB", pack=False, align=None), "flags": SimTypeInt(signed=False, label="IKEEXT_CERT_FLAGS")}, name="IKEEXT_CERT_ROOT_CONFIG0", pack=False, align=None), offset=0), "inboundTrustedRootStoreConfig": SimTypePointer(SimStruct({"certData": SimStruct({"size": SimTypeInt(signed=False, label="UInt32"), "data": SimTypePointer(SimTypeChar(label="Byte"), offset=0)}, name="FWP_BYTE_BLOB", pack=False, align=None), "flags": SimTypeInt(signed=False, label="IKEEXT_CERT_FLAGS")}, name="IKEEXT_CERT_ROOT_CONFIG0", pack=False, align=None), offset=0)}, name="<anon>", label="None"), "outboundConfigType": SimTypeInt(signed=False, label="IKEEXT_CERT_CONFIG_TYPE"), "Anonymous2": SimUnion({"Anonymous": SimStruct({"outboundRootArraySize": SimTypeInt(signed=False, label="UInt32"), "outboundRootArray": SimTypePointer(SimStruct({"certData": SimStruct({"size": SimTypeInt(signed=False, label="UInt32"), "data": SimTypePointer(SimTypeChar(label="Byte"), offset=0)}, name="FWP_BYTE_BLOB", pack=False, align=None), "flags": SimTypeInt(signed=False, label="IKEEXT_CERT_FLAGS")}, name="IKEEXT_CERT_ROOT_CONFIG0", pack=False, align=None), offset=0)}, name="_Anonymous_e__Struct", pack=False, align=None), "outboundEnterpriseStoreConfig": SimTypePointer(SimStruct({"certData": SimStruct({"size": SimTypeInt(signed=False, label="UInt32"), "data": SimTypePointer(SimTypeChar(label="Byte"), offset=0)}, name="FWP_BYTE_BLOB", pack=False, align=None), "flags": SimTypeInt(signed=False, label="IKEEXT_CERT_FLAGS")}, name="IKEEXT_CERT_ROOT_CONFIG0", pack=False, align=None), offset=0), "outboundTrustedRootStoreConfig": SimTypePointer(SimStruct({"certData": SimStruct({"size": SimTypeInt(signed=False, label="UInt32"), "data": SimTypePointer(SimTypeChar(label="Byte"), offset=0)}, name="FWP_BYTE_BLOB", pack=False, align=None), "flags": SimTypeInt(signed=False, label="IKEEXT_CERT_FLAGS")}, name="IKEEXT_CERT_ROOT_CONFIG0", pack=False, align=None), offset=0)}, name="<anon>", label="None"), "flags": SimTypeInt(signed=False, label="IKEEXT_CERT_AUTH")}, name="IKEEXT_CERTIFICATE_AUTHENTICATION0", pack=False, align=None), "cgaAuthentication": SimStruct({"keyContainerName": SimTypePointer(SimTypeChar(label="Char"), offset=0), "cspName": SimTypePointer(SimTypeChar(label="Char"), offset=0), "cspType": SimTypeInt(signed=False, label="UInt32"), "cgaModifier": SimStruct({"byteArray16": SimTypeFixedSizeArray(SimTypeChar(label="Byte"), 16)}, name="FWP_BYTE_ARRAY16", pack=False, align=None), "cgaCollisionCount": SimTypeChar(label="Byte")}, name="IKEEXT_IPV6_CGA_AUTHENTICATION0", pack=False, align=None)}, name="<anon>", label="None")}, name="IKEEXT_AUTHENTICATION_METHOD0", pack=False, align=None), offset=0), "initiatorImpersonationType": SimTypeInt(signed=False, label="IKEEXT_AUTHENTICATION_IMPERSONATION_TYPE")}, name="IKEEXT_EM_POLICY0", pack=False, align=None), offset=0)}, name="IPSEC_TRANSPORT_POLICY0", pack=False, align=None), offset=0), "ikeQmTunnelPolicy": SimTypePointer(SimStruct({"flags": SimTypeInt(signed=False, label="IPSEC_POLICY_FLAG"), "numIpsecProposals": SimTypeInt(signed=False, label="UInt32"), "ipsecProposals": SimTypePointer(SimStruct({"lifetime": SimStruct({"lifetimeSeconds": SimTypeInt(signed=False, label="UInt32"), "lifetimeKilobytes": SimTypeInt(signed=False, label="UInt32"), "lifetimePackets": SimTypeInt(signed=False, label="UInt32")}, name="IPSEC_SA_LIFETIME0", pack=False, align=None), "numSaTransforms": SimTypeInt(signed=False, label="UInt32"), "saTransforms": SimTypePointer(SimStruct({"ipsecTransformType": SimTypeInt(signed=False, label="IPSEC_TRANSFORM_TYPE"), "Anonymous": SimUnion({"ahTransform": SimTypePointer(SimStruct({"authTransformId": SimStruct({"authType": SimTypeInt(signed=False, label="IPSEC_AUTH_TYPE"), "authConfig": SimTypeChar(label="Byte")}, name="IPSEC_AUTH_TRANSFORM_ID0", pack=False, align=None), "cryptoModuleId": SimTypePointer(SimTypeBottom(label="Guid"), offset=0)}, name="IPSEC_AUTH_TRANSFORM0", pack=False, align=None), offset=0), "espAuthTransform": SimTypePointer(SimStruct({"authTransformId": SimStruct({"authType": SimTypeInt(signed=False, label="IPSEC_AUTH_TYPE"), "authConfig": SimTypeChar(label="Byte")}, name="IPSEC_AUTH_TRANSFORM_ID0", pack=False, align=None), "cryptoModuleId": SimTypePointer(SimTypeBottom(label="Guid"), offset=0)}, name="IPSEC_AUTH_TRANSFORM0", pack=False, align=None), offset=0), "espCipherTransform": SimTypePointer(SimStruct({"cipherTransformId": SimStruct({"cipherType": SimTypeInt(signed=False, label="IPSEC_CIPHER_TYPE"), "cipherConfig": SimTypeChar(label="Byte")}, name="IPSEC_CIPHER_TRANSFORM_ID0", pack=False, align=None), "cryptoModuleId": SimTypePointer(SimTypeBottom(label="Guid"), offset=0)}, name="IPSEC_CIPHER_TRANSFORM0", pack=False, align=None), offset=0), "espAuthAndCipherTransform": SimTypePointer(SimStruct({"authTransform": SimStruct({"authTransformId": SimStruct({"authType": SimTypeInt(signed=False, label="IPSEC_AUTH_TYPE"), "authConfig": SimTypeChar(label="Byte")}, name="IPSEC_AUTH_TRANSFORM_ID0", pack=False, align=None), "cryptoModuleId": SimTypePointer(SimTypeBottom(label="Guid"), offset=0)}, name="IPSEC_AUTH_TRANSFORM0", pack=False, align=None), "cipherTransform": SimStruct({"cipherTransformId": SimStruct({"cipherType": SimTypeInt(signed=False, label="IPSEC_CIPHER_TYPE"), "cipherConfig": SimTypeChar(label="Byte")}, name="IPSEC_CIPHER_TRANSFORM_ID0", pack=False, align=None), "cryptoModuleId": SimTypePointer(SimTypeBottom(label="Guid"), offset=0)}, name="IPSEC_CIPHER_TRANSFORM0", pack=False, align=None)}, name="IPSEC_AUTH_AND_CIPHER_TRANSFORM0", pack=False, align=None), offset=0), "espAuthFwTransform": SimTypePointer(SimStruct({"authTransformId": SimStruct({"authType": SimTypeInt(signed=False, label="IPSEC_AUTH_TYPE"), "authConfig": SimTypeChar(label="Byte")}, name="IPSEC_AUTH_TRANSFORM_ID0", pack=False, align=None), "cryptoModuleId": SimTypePointer(SimTypeBottom(label="Guid"), offset=0)}, name="IPSEC_AUTH_TRANSFORM0", pack=False, align=None), offset=0)}, name="<anon>", label="None")}, name="IPSEC_SA_TRANSFORM0", pack=False, align=None), offset=0), "pfsGroup": SimTypeInt(signed=False, label="IPSEC_PFS_GROUP")}, name="IPSEC_PROPOSAL0", pack=False, align=None), offset=0), "tunnelEndpoints": SimStruct({"ipVersion": SimTypeInt(signed=False, label="FWP_IP_VERSION"), "Anonymous1": SimUnion({"localV4Address": SimTypeInt(signed=False, label="UInt32"), "localV6Address": SimTypeFixedSizeArray(SimTypeChar(label="Byte"), 16)}, name="<anon>", label="None"), "Anonymous2": SimUnion({"remoteV4Address": SimTypeInt(signed=False, label="UInt32"), "remoteV6Address": SimTypeFixedSizeArray(SimTypeChar(label="Byte"), 16)}, name="<anon>", label="None")}, name="IPSEC_TUNNEL_ENDPOINTS0", pack=False, align=None), "saIdleTimeout": SimStruct({"idleTimeoutSeconds": SimTypeInt(signed=False, label="UInt32"), "idleTimeoutSecondsFailOver": SimTypeInt(signed=False, label="UInt32")}, name="IPSEC_SA_IDLE_TIMEOUT0", pack=False, align=None), "emPolicy": SimTypePointer(SimStruct({"numAuthenticationMethods": SimTypeInt(signed=False, label="UInt32"), "authenticationMethods": SimTypePointer(SimStruct({"authenticationMethodType": SimTypeInt(signed=False, label="IKEEXT_AUTHENTICATION_METHOD_TYPE"), "Anonymous": SimUnion({"presharedKeyAuthentication": SimStruct({"presharedKey": SimStruct({"size": SimTypeInt(signed=False, label="UInt32"), "data": SimTypePointer(SimTypeChar(label="Byte"), offset=0)}, name="FWP_BYTE_BLOB", pack=False, align=None)}, name="IKEEXT_PRESHARED_KEY_AUTHENTICATION0", pack=False, align=None), "certificateAuthentication": SimStruct({"inboundConfigType": SimTypeInt(signed=False, label="IKEEXT_CERT_CONFIG_TYPE"), "Anonymous1": SimUnion({"Anonymous": SimStruct({"inboundRootArraySize": SimTypeInt(signed=False, label="UInt32"), "inboundRootArray": SimTypePointer(SimStruct({"certData": SimStruct({"size": SimTypeInt(signed=False, label="UInt32"), "data": SimTypePointer(SimTypeChar(label="Byte"), offset=0)}, name="FWP_BYTE_BLOB", pack=False, align=None), "flags": SimTypeInt(signed=False, label="IKEEXT_CERT_FLAGS")}, name="IKEEXT_CERT_ROOT_CONFIG0", pack=False, align=None), offset=0)}, name="_Anonymous_e__Struct", pack=False, align=None), "inboundEnterpriseStoreConfig": SimTypePointer(SimStruct({"certData": SimStruct({"size": SimTypeInt(signed=False, label="UInt32"), "data": SimTypePointer(SimTypeChar(label="Byte"), offset=0)}, name="FWP_BYTE_BLOB", pack=False, align=None), "flags": SimTypeInt(signed=False, label="IKEEXT_CERT_FLAGS")}, name="IKEEXT_CERT_ROOT_CONFIG0", pack=False, align=None), offset=0), "inboundTrustedRootStoreConfig": SimTypePointer(SimStruct({"certData": SimStruct({"size": SimTypeInt(signed=False, label="UInt32"), "data": SimTypePointer(SimTypeChar(label="Byte"), offset=0)}, name="FWP_BYTE_BLOB", pack=False, align=None), "flags": SimTypeInt(signed=False, label="IKEEXT_CERT_FLAGS")}, name="IKEEXT_CERT_ROOT_CONFIG0", pack=False, align=None), offset=0)}, name="<anon>", label="None"), "outboundConfigType": SimTypeInt(signed=False, label="IKEEXT_CERT_CONFIG_TYPE"), "Anonymous2": SimUnion({"Anonymous": SimStruct({"outboundRootArraySize": SimTypeInt(signed=False, label="UInt32"), "outboundRootArray": SimTypePointer(SimStruct({"certData": SimStruct({"size": SimTypeInt(signed=False, label="UInt32"), "data": SimTypePointer(SimTypeChar(label="Byte"), offset=0)}, name="FWP_BYTE_BLOB", pack=False, align=None), "flags": SimTypeInt(signed=False, label="IKEEXT_CERT_FLAGS")}, name="IKEEXT_CERT_ROOT_CONFIG0", pack=False, align=None), offset=0)}, name="_Anonymous_e__Struct", pack=False, align=None), "outboundEnterpriseStoreConfig": SimTypePointer(SimStruct({"certData": SimStruct({"size": SimTypeInt(signed=False, label="UInt32"), "data": SimTypePointer(SimTypeChar(label="Byte"), offset=0)}, name="FWP_BYTE_BLOB", pack=False, align=None), "flags": SimTypeInt(signed=False, label="IKEEXT_CERT_FLAGS")}, name="IKEEXT_CERT_ROOT_CONFIG0", pack=False, align=None), offset=0), "outboundTrustedRootStoreConfig": SimTypePointer(SimStruct({"certData": SimStruct({"size": SimTypeInt(signed=False, label="UInt32"), "data": SimTypePointer(SimTypeChar(label="Byte"), offset=0)}, name="FWP_BYTE_BLOB", pack=False, align=None), "flags": SimTypeInt(signed=False, label="IKEEXT_CERT_FLAGS")}, name="IKEEXT_CERT_ROOT_CONFIG0", pack=False, align=None), offset=0)}, name="<anon>", label="None"), "flags": SimTypeInt(signed=False, label="IKEEXT_CERT_AUTH")}, name="IKEEXT_CERTIFICATE_AUTHENTICATION0", pack=False, align=None), "kerberosAuthentication": SimStruct({"flags": SimTypeInt(signed=False, label="IKEEXT_KERBEROS_AUTHENTICATION_FLAGS")}, name="IKEEXT_KERBEROS_AUTHENTICATION0", pack=False, align=None), "ntlmV2Authentication": SimStruct({"flags": SimTypeInt(signed=False, label="UInt32")}, name="IKEEXT_NTLM_V2_AUTHENTICATION0", pack=False, align=None), "sslAuthentication": SimStruct({"inboundConfigType": SimTypeInt(signed=False, label="IKEEXT_CERT_CONFIG_TYPE"), "Anonymous1": SimUnion({"Anonymous": SimStruct({"inboundRootArraySize": SimTypeInt(signed=False, label="UInt32"), "inboundRootArray": SimTypePointer(SimStruct({"certData": SimStruct({"size": SimTypeInt(signed=False, label="UInt32"), "data": SimTypePointer(SimTypeChar(label="Byte"), offset=0)}, name="FWP_BYTE_BLOB", pack=False, align=None), "flags": SimTypeInt(signed=False, label="IKEEXT_CERT_FLAGS")}, name="IKEEXT_CERT_ROOT_CONFIG0", pack=False, align=None), offset=0)}, name="_Anonymous_e__Struct", pack=False, align=None), "inboundEnterpriseStoreConfig": SimTypePointer(SimStruct({"certData": SimStruct({"size": SimTypeInt(signed=False, label="UInt32"), "data": SimTypePointer(SimTypeChar(label="Byte"), offset=0)}, name="FWP_BYTE_BLOB", pack=False, align=None), "flags": SimTypeInt(signed=False, label="IKEEXT_CERT_FLAGS")}, name="IKEEXT_CERT_ROOT_CONFIG0", pack=False, align=None), offset=0), "inboundTrustedRootStoreConfig": SimTypePointer(SimStruct({"certData": SimStruct({"size": SimTypeInt(signed=False, label="UInt32"), "data": SimTypePointer(SimTypeChar(label="Byte"), offset=0)}, name="FWP_BYTE_BLOB", pack=False, align=None), "flags": SimTypeInt(signed=False, label="IKEEXT_CERT_FLAGS")}, name="IKEEXT_CERT_ROOT_CONFIG0", pack=False, align=None), offset=0)}, name="<anon>", label="None"), "outboundConfigType": SimTypeInt(signed=False, label="IKEEXT_CERT_CONFIG_TYPE"), "Anonymous2": SimUnion({"Anonymous": SimStruct({"outboundRootArraySize": SimTypeInt(signed=False, label="UInt32"), "outboundRootArray": SimTypePointer(SimStruct({"certData": SimStruct({"size": SimTypeInt(signed=False, label="UInt32"), "data": SimTypePointer(SimTypeChar(label="Byte"), offset=0)}, name="FWP_BYTE_BLOB", pack=False, align=None), "flags": SimTypeInt(signed=False, label="IKEEXT_CERT_FLAGS")}, name="IKEEXT_CERT_ROOT_CONFIG0", pack=False, align=None), offset=0)}, name="_Anonymous_e__Struct", pack=False, align=None), "outboundEnterpriseStoreConfig": SimTypePointer(SimStruct({"certData": SimStruct({"size": SimTypeInt(signed=False, label="UInt32"), "data": SimTypePointer(SimTypeChar(label="Byte"), offset=0)}, name="FWP_BYTE_BLOB", pack=False, align=None), "flags": SimTypeInt(signed=False, label="IKEEXT_CERT_FLAGS")}, name="IKEEXT_CERT_ROOT_CONFIG0", pack=False, align=None), offset=0), "outboundTrustedRootStoreConfig": SimTypePointer(SimStruct({"certData": SimStruct({"size": SimTypeInt(signed=False, label="UInt32"), "data": SimTypePointer(SimTypeChar(label="Byte"), offset=0)}, name="FWP_BYTE_BLOB", pack=False, align=None), "flags": SimTypeInt(signed=False, label="IKEEXT_CERT_FLAGS")}, name="IKEEXT_CERT_ROOT_CONFIG0", pack=False, align=None), offset=0)}, name="<anon>", label="None"), "flags": SimTypeInt(signed=False, label="IKEEXT_CERT_AUTH")}, name="IKEEXT_CERTIFICATE_AUTHENTICATION0", pack=False, align=None), "cgaAuthentication": SimStruct({"keyContainerName": SimTypePointer(SimTypeChar(label="Char"), offset=0), "cspName": SimTypePointer(SimTypeChar(label="Char"), offset=0), "cspType": SimTypeInt(signed=False, label="UInt32"), "cgaModifier": SimStruct({"byteArray16": SimTypeFixedSizeArray(SimTypeChar(label="Byte"), 16)}, name="FWP_BYTE_ARRAY16", pack=False, align=None), "cgaCollisionCount": SimTypeChar(label="Byte")}, name="IKEEXT_IPV6_CGA_AUTHENTICATION0", pack=False, align=None)}, name="<anon>", label="None")}, name="IKEEXT_AUTHENTICATION_METHOD0", pack=False, align=None), offset=0), "initiatorImpersonationType": SimTypeInt(signed=False, label="IKEEXT_AUTHENTICATION_IMPERSONATION_TYPE")}, name="IKEEXT_EM_POLICY0", pack=False, align=None), offset=0)}, name="IPSEC_TUNNEL_POLICY0", pack=False, align=None), offset=0), "authipQmTransportPolicy": SimTypePointer(SimStruct({"numIpsecProposals": SimTypeInt(signed=False, label="UInt32"), "ipsecProposals": SimTypePointer(SimStruct({"lifetime": SimStruct({"lifetimeSeconds": SimTypeInt(signed=False, label="UInt32"), "lifetimeKilobytes": SimTypeInt(signed=False, label="UInt32"), "lifetimePackets": SimTypeInt(signed=False, label="UInt32")}, name="IPSEC_SA_LIFETIME0", pack=False, align=None), "numSaTransforms": SimTypeInt(signed=False, label="UInt32"), "saTransforms": SimTypePointer(SimStruct({"ipsecTransformType": SimTypeInt(signed=False, label="IPSEC_TRANSFORM_TYPE"), "Anonymous": SimUnion({"ahTransform": SimTypePointer(SimStruct({"authTransformId": SimStruct({"authType": SimTypeInt(signed=False, label="IPSEC_AUTH_TYPE"), "authConfig": SimTypeChar(label="Byte")}, name="IPSEC_AUTH_TRANSFORM_ID0", pack=False, align=None), "cryptoModuleId": SimTypePointer(SimTypeBottom(label="Guid"), offset=0)}, name="IPSEC_AUTH_TRANSFORM0", pack=False, align=None), offset=0), "espAuthTransform": SimTypePointer(SimStruct({"authTransformId": SimStruct({"authType": SimTypeInt(signed=False, label="IPSEC_AUTH_TYPE"), "authConfig": SimTypeChar(label="Byte")}, name="IPSEC_AUTH_TRANSFORM_ID0", pack=False, align=None), "cryptoModuleId": SimTypePointer(SimTypeBottom(label="Guid"), offset=0)}, name="IPSEC_AUTH_TRANSFORM0", pack=False, align=None), offset=0), "espCipherTransform": SimTypePointer(SimStruct({"cipherTransformId": SimStruct({"cipherType": SimTypeInt(signed=False, label="IPSEC_CIPHER_TYPE"), "cipherConfig": SimTypeChar(label="Byte")}, name="IPSEC_CIPHER_TRANSFORM_ID0", pack=False, align=None), "cryptoModuleId": SimTypePointer(SimTypeBottom(label="Guid"), offset=0)}, name="IPSEC_CIPHER_TRANSFORM0", pack=False, align=None), offset=0), "espAuthAndCipherTransform": SimTypePointer(SimStruct({"authTransform": SimStruct({"authTransformId": SimStruct({"authType": SimTypeInt(signed=False, label="IPSEC_AUTH_TYPE"), "authConfig": SimTypeChar(label="Byte")}, name="IPSEC_AUTH_TRANSFORM_ID0", pack=False, align=None), "cryptoModuleId": SimTypePointer(SimTypeBottom(label="Guid"), offset=0)}, name="IPSEC_AUTH_TRANSFORM0", pack=False, align=None), "cipherTransform": SimStruct({"cipherTransformId": SimStruct({"cipherType": SimTypeInt(signed=False, label="IPSEC_CIPHER_TYPE"), "cipherConfig": SimTypeChar(label="Byte")}, name="IPSEC_CIPHER_TRANSFORM_ID0", pack=False, align=None), "cryptoModuleId": SimTypePointer(SimTypeBottom(label="Guid"), offset=0)}, name="IPSEC_CIPHER_TRANSFORM0", pack=False, align=None)}, name="IPSEC_AUTH_AND_CIPHER_TRANSFORM0", pack=False, align=None), offset=0), "espAuthFwTransform": SimTypePointer(SimStruct({"authTransformId": SimStruct({"authType": SimTypeInt(signed=False, label="IPSEC_AUTH_TYPE"), "authConfig": SimTypeChar(label="Byte")}, name="IPSEC_AUTH_TRANSFORM_ID0", pack=False, align=None), "cryptoModuleId": SimTypePointer(SimTypeBottom(label="Guid"), offset=0)}, name="IPSEC_AUTH_TRANSFORM0", pack=False, align=None), offset=0)}, name="<anon>", label="None")}, name="IPSEC_SA_TRANSFORM0", pack=False, align=None), offset=0), "pfsGroup": SimTypeInt(signed=False, label="IPSEC_PFS_GROUP")}, name="IPSEC_PROPOSAL0", pack=False, align=None), offset=0), "flags": SimTypeInt(signed=False, label="IPSEC_POLICY_FLAG"), "ndAllowClearTimeoutSeconds": SimTypeInt(signed=False, label="UInt32"), "saIdleTimeout": SimStruct({"idleTimeoutSeconds": SimTypeInt(signed=False, label="UInt32"), "idleTimeoutSecondsFailOver": SimTypeInt(signed=False, label="UInt32")}, name="IPSEC_SA_IDLE_TIMEOUT0", pack=False, align=None), "emPolicy": SimTypePointer(SimStruct({"numAuthenticationMethods": SimTypeInt(signed=False, label="UInt32"), "authenticationMethods": SimTypePointer(SimStruct({"authenticationMethodType": SimTypeInt(signed=False, label="IKEEXT_AUTHENTICATION_METHOD_TYPE"), "Anonymous": SimUnion({"presharedKeyAuthentication": SimStruct({"presharedKey": SimStruct({"size": SimTypeInt(signed=False, label="UInt32"), "data": SimTypePointer(SimTypeChar(label="Byte"), offset=0)}, name="FWP_BYTE_BLOB", pack=False, align=None)}, name="IKEEXT_PRESHARED_KEY_AUTHENTICATION0", pack=False, align=None), "certificateAuthentication": SimStruct({"inboundConfigType": SimTypeInt(signed=False, label="IKEEXT_CERT_CONFIG_TYPE"), "Anonymous1": SimUnion({"Anonymous": SimStruct({"inboundRootArraySize": SimTypeInt(signed=False, label="UInt32"), "inboundRootArray": SimTypePointer(SimStruct({"certData": SimStruct({"size": SimTypeInt(signed=False, label="UInt32"), "data": SimTypePointer(SimTypeChar(label="Byte"), offset=0)}, name="FWP_BYTE_BLOB", pack=False, align=None), "flags": SimTypeInt(signed=False, label="IKEEXT_CERT_FLAGS")}, name="IKEEXT_CERT_ROOT_CONFIG0", pack=False, align=None), offset=0)}, name="_Anonymous_e__Struct", pack=False, align=None), "inboundEnterpriseStoreConfig": SimTypePointer(SimStruct({"certData": SimStruct({"size": SimTypeInt(signed=False, label="UInt32"), "data": SimTypePointer(SimTypeChar(label="Byte"), offset=0)}, name="FWP_BYTE_BLOB", pack=False, align=None), "flags": SimTypeInt(signed=False, label="IKEEXT_CERT_FLAGS")}, name="IKEEXT_CERT_ROOT_CONFIG0", pack=False, align=None), offset=0), "inboundTrustedRootStoreConfig": SimTypePointer(SimStruct({"certData": SimStruct({"size": SimTypeInt(signed=False, label="UInt32"), "data": SimTypePointer(SimTypeChar(label="Byte"), offset=0)}, name="FWP_BYTE_BLOB", pack=False, align=None), "flags": SimTypeInt(signed=False, label="IKEEXT_CERT_FLAGS")}, name="IKEEXT_CERT_ROOT_CONFIG0", pack=False, align=None), offset=0)}, name="<anon>", label="None"), "outboundConfigType": SimTypeInt(signed=False, label="IKEEXT_CERT_CONFIG_TYPE"), "Anonymous2": SimUnion({"Anonymous": SimStruct({"outboundRootArraySize": SimTypeInt(signed=False, label="UInt32"), "outboundRootArray": SimTypePointer(SimStruct({"certData": SimStruct({"size": SimTypeInt(signed=False, label="UInt32"), "data": SimTypePointer(SimTypeChar(label="Byte"), offset=0)}, name="FWP_BYTE_BLOB", pack=False, align=None), "flags": SimTypeInt(signed=False, label="IKEEXT_CERT_FLAGS")}, name="IKEEXT_CERT_ROOT_CONFIG0", pack=False, align=None), offset=0)}, name="_Anonymous_e__Struct", pack=False, align=None), "outboundEnterpriseStoreConfig": SimTypePointer(SimStruct({"certData": SimStruct({"size": SimTypeInt(signed=False, label="UInt32"), "data": SimTypePointer(SimTypeChar(label="Byte"), offset=0)}, name="FWP_BYTE_BLOB", pack=False, align=None), "flags": SimTypeInt(signed=False, label="IKEEXT_CERT_FLAGS")}, name="IKEEXT_CERT_ROOT_CONFIG0", pack=False, align=None), offset=0), "outboundTrustedRootStoreConfig": SimTypePointer(SimStruct({"certData": SimStruct({"size": SimTypeInt(signed=False, label="UInt32"), "data": SimTypePointer(SimTypeChar(label="Byte"), offset=0)}, name="FWP_BYTE_BLOB", pack=False, align=None), "flags": SimTypeInt(signed=False, label="IKEEXT_CERT_FLAGS")}, name="IKEEXT_CERT_ROOT_CONFIG0", pack=False, align=None), offset=0)}, name="<anon>", label="None"), "flags": SimTypeInt(signed=False, label="IKEEXT_CERT_AUTH")}, name="IKEEXT_CERTIFICATE_AUTHENTICATION0", pack=False, align=None), "kerberosAuthentication": SimStruct({"flags": SimTypeInt(signed=False, label="IKEEXT_KERBEROS_AUTHENTICATION_FLAGS")}, name="IKEEXT_KERBEROS_AUTHENTICATION0", pack=False, align=None), "ntlmV2Authentication": SimStruct({"flags": SimTypeInt(signed=False, label="UInt32")}, name="IKEEXT_NTLM_V2_AUTHENTICATION0", pack=False, align=None), "sslAuthentication": SimStruct({"inboundConfigType": SimTypeInt(signed=False, label="IKEEXT_CERT_CONFIG_TYPE"), "Anonymous1": SimUnion({"Anonymous": SimStruct({"inboundRootArraySize": SimTypeInt(signed=False, label="UInt32"), "inboundRootArray": SimTypePointer(SimStruct({"certData": SimStruct({"size": SimTypeInt(signed=False, label="UInt32"), "data": SimTypePointer(SimTypeChar(label="Byte"), offset=0)}, name="FWP_BYTE_BLOB", pack=False, align=None), "flags": SimTypeInt(signed=False, label="IKEEXT_CERT_FLAGS")}, name="IKEEXT_CERT_ROOT_CONFIG0", pack=False, align=None), offset=0)}, name="_Anonymous_e__Struct", pack=False, align=None), "inboundEnterpriseStoreConfig": SimTypePointer(SimStruct({"certData": SimStruct({"size": SimTypeInt(signed=False, label="UInt32"), "data": SimTypePointer(SimTypeChar(label="Byte"), offset=0)}, name="FWP_BYTE_BLOB", pack=False, align=None), "flags": SimTypeInt(signed=False, label="IKEEXT_CERT_FLAGS")}, name="IKEEXT_CERT_ROOT_CONFIG0", pack=False, align=None), offset=0), "inboundTrustedRootStoreConfig": SimTypePointer(SimStruct({"certData": SimStruct({"size": SimTypeInt(signed=False, label="UInt32"), "data": SimTypePointer(SimTypeChar(label="Byte"), offset=0)}, name="FWP_BYTE_BLOB", pack=False, align=None), "flags": SimTypeInt(signed=False, label="IKEEXT_CERT_FLAGS")}, name="IKEEXT_CERT_ROOT_CONFIG0", pack=False, align=None), offset=0)}, name="<anon>", label="None"), "outboundConfigType": SimTypeInt(signed=False, label="IKEEXT_CERT_CONFIG_TYPE"), "Anonymous2": SimUnion({"Anonymous": SimStruct({"outboundRootArraySize": SimTypeInt(signed=False, label="UInt32"), "outboundRootArray": SimTypePointer(SimStruct({"certData": SimStruct({"size": SimTypeInt(signed=False, label="UInt32"), "data": SimTypePointer(SimTypeChar(label="Byte"), offset=0)}, name="FWP_BYTE_BLOB", pack=False, align=None), "flags": SimTypeInt(signed=False, label="IKEEXT_CERT_FLAGS")}, name="IKEEXT_CERT_ROOT_CONFIG0", pack=False, align=None), offset=0)}, name="_Anonymous_e__Struct", pack=False, align=None), "outboundEnterpriseStoreConfig": SimTypePointer(SimStruct({"certData": SimStruct({"size": SimTypeInt(signed=False, label="UInt32"), "data": SimTypePointer(SimTypeChar(label="Byte"), offset=0)}, name="FWP_BYTE_BLOB", pack=False, align=None), "flags": SimTypeInt(signed=False, label="IKEEXT_CERT_FLAGS")}, name="IKEEXT_CERT_ROOT_CONFIG0", pack=False, align=None), offset=0), "outboundTrustedRootStoreConfig": SimTypePointer(SimStruct({"certData": SimStruct({"size": SimTypeInt(signed=False, label="UInt32"), "data": SimTypePointer(SimTypeChar(label="Byte"), offset=0)}, name="FWP_BYTE_BLOB", pack=False, align=None), "flags": SimTypeInt(signed=False, label="IKEEXT_CERT_FLAGS")}, name="IKEEXT_CERT_ROOT_CONFIG0", pack=False, align=None), offset=0)}, name="<anon>", label="None"), "flags": SimTypeInt(signed=False, label="IKEEXT_CERT_AUTH")}, name="IKEEXT_CERTIFICATE_AUTHENTICATION0", pack=False, align=None), "cgaAuthentication": SimStruct({"keyContainerName": SimTypePointer(SimTypeChar(label="Char"), offset=0), "cspName": SimTypePointer(SimTypeChar(label="Char"), offset=0), "cspType": SimTypeInt(signed=False, label="UInt32"), "cgaModifier": SimStruct({"byteArray16": SimTypeFixedSizeArray(SimTypeChar(label="Byte"), 16)}, name="FWP_BYTE_ARRAY16", pack=False, align=None), "cgaCollisionCount": SimTypeChar(label="Byte")}, name="IKEEXT_IPV6_CGA_AUTHENTICATION0", pack=False, align=None)}, name="<anon>", label="None")}, name="IKEEXT_AUTHENTICATION_METHOD0", pack=False, align=None), offset=0), "initiatorImpersonationType": SimTypeInt(signed=False, label="IKEEXT_AUTHENTICATION_IMPERSONATION_TYPE")}, name="IKEEXT_EM_POLICY0", pack=False, align=None), offset=0)}, name="IPSEC_TRANSPORT_POLICY0", pack=False, align=None), offset=0), "authipQmTunnelPolicy": SimTypePointer(SimStruct({"flags": SimTypeInt(signed=False, label="IPSEC_POLICY_FLAG"), "numIpsecProposals": SimTypeInt(signed=False, label="UInt32"), "ipsecProposals": SimTypePointer(SimStruct({"lifetime": SimStruct({"lifetimeSeconds": SimTypeInt(signed=False, label="UInt32"), "lifetimeKilobytes": SimTypeInt(signed=False, label="UInt32"), "lifetimePackets": SimTypeInt(signed=False, label="UInt32")}, name="IPSEC_SA_LIFETIME0", pack=False, align=None), "numSaTransforms": SimTypeInt(signed=False, label="UInt32"), "saTransforms": SimTypePointer(SimStruct({"ipsecTransformType": SimTypeInt(signed=False, label="IPSEC_TRANSFORM_TYPE"), "Anonymous": SimUnion({"ahTransform": SimTypePointer(SimStruct({"authTransformId": SimStruct({"authType": SimTypeInt(signed=False, label="IPSEC_AUTH_TYPE"), "authConfig": SimTypeChar(label="Byte")}, name="IPSEC_AUTH_TRANSFORM_ID0", pack=False, align=None), "cryptoModuleId": SimTypePointer(SimTypeBottom(label="Guid"), offset=0)}, name="IPSEC_AUTH_TRANSFORM0", pack=False, align=None), offset=0), "espAuthTransform": SimTypePointer(SimStruct({"authTransformId": SimStruct({"authType": SimTypeInt(signed=False, label="IPSEC_AUTH_TYPE"), "authConfig": SimTypeChar(label="Byte")}, name="IPSEC_AUTH_TRANSFORM_ID0", pack=False, align=None), "cryptoModuleId": SimTypePointer(SimTypeBottom(label="Guid"), offset=0)}, name="IPSEC_AUTH_TRANSFORM0", pack=False, align=None), offset=0), "espCipherTransform": SimTypePointer(SimStruct({"cipherTransformId": SimStruct({"cipherType": SimTypeInt(signed=False, label="IPSEC_CIPHER_TYPE"), "cipherConfig": SimTypeChar(label="Byte")}, name="IPSEC_CIPHER_TRANSFORM_ID0", pack=False, align=None), "cryptoModuleId": SimTypePointer(SimTypeBottom(label="Guid"), offset=0)}, name="IPSEC_CIPHER_TRANSFORM0", pack=False, align=None), offset=0), "espAuthAndCipherTransform": SimTypePointer(SimStruct({"authTransform": SimStruct({"authTransformId": SimStruct({"authType": SimTypeInt(signed=False, label="IPSEC_AUTH_TYPE"), "authConfig": SimTypeChar(label="Byte")}, name="IPSEC_AUTH_TRANSFORM_ID0", pack=False, align=None), "cryptoModuleId": SimTypePointer(SimTypeBottom(label="Guid"), offset=0)}, name="IPSEC_AUTH_TRANSFORM0", pack=False, align=None), "cipherTransform": SimStruct({"cipherTransformId": SimStruct({"cipherType": SimTypeInt(signed=False, label="IPSEC_CIPHER_TYPE"), "cipherConfig": SimTypeChar(label="Byte")}, name="IPSEC_CIPHER_TRANSFORM_ID0", pack=False, align=None), "cryptoModuleId": SimTypePointer(SimTypeBottom(label="Guid"), offset=0)}, name="IPSEC_CIPHER_TRANSFORM0", pack=False, align=None)}, name="IPSEC_AUTH_AND_CIPHER_TRANSFORM0", pack=False, align=None), offset=0), "espAuthFwTransform": SimTypePointer(SimStruct({"authTransformId": SimStruct({"authType": SimTypeInt(signed=False, label="IPSEC_AUTH_TYPE"), "authConfig": SimTypeChar(label="Byte")}, name="IPSEC_AUTH_TRANSFORM_ID0", pack=False, align=None), "cryptoModuleId": SimTypePointer(SimTypeBottom(label="Guid"), offset=0)}, name="IPSEC_AUTH_TRANSFORM0", pack=False, align=None), offset=0)}, name="<anon>", label="None")}, name="IPSEC_SA_TRANSFORM0", pack=False, align=None), offset=0), "pfsGroup": SimTypeInt(signed=False, label="IPSEC_PFS_GROUP")}, name="IPSEC_PROPOSAL0", pack=False, align=None), offset=0), "tunnelEndpoints": SimStruct({"ipVersion": SimTypeInt(signed=False, label="FWP_IP_VERSION"), "Anonymous1": SimUnion({"localV4Address": SimTypeInt(signed=False, label="UInt32"), "localV6Address": SimTypeFixedSizeArray(SimTypeChar(label="Byte"), 16)}, name="<anon>", label="None"), "Anonymous2": SimUnion({"remoteV4Address": SimTypeInt(signed=False, label="UInt32"), "remoteV6Address": SimTypeFixedSizeArray(SimTypeChar(label="Byte"), 16)}, name="<anon>", label="None")}, name="IPSEC_TUNNEL_ENDPOINTS0", pack=False, align=None), "saIdleTimeout": SimStruct({"idleTimeoutSeconds": SimTypeInt(signed=False, label="UInt32"), "idleTimeoutSecondsFailOver": SimTypeInt(signed=False, label="UInt32")}, name="IPSEC_SA_IDLE_TIMEOUT0", pack=False, align=None), "emPolicy": SimTypePointer(SimStruct({"numAuthenticationMethods": SimTypeInt(signed=False, label="UInt32"), "authenticationMethods": SimTypePointer(SimStruct({"authenticationMethodType": SimTypeInt(signed=False, label="IKEEXT_AUTHENTICATION_METHOD_TYPE"), "Anonymous": SimUnion({"presharedKeyAuthentication": SimStruct({"presharedKey": SimStruct({"size": SimTypeInt(signed=False, label="UInt32"), "data": SimTypePointer(SimTypeChar(label="Byte"), offset=0)}, name="FWP_BYTE_BLOB", pack=False, align=None)}, name="IKEEXT_PRESHARED_KEY_AUTHENTICATION0", pack=False, align=None), "certificateAuthentication": SimStruct({"inboundConfigType": SimTypeInt(signed=False, label="IKEEXT_CERT_CONFIG_TYPE"), "Anonymous1": SimUnion({"Anonymous": SimStruct({"inboundRootArraySize": SimTypeInt(signed=False, label="UInt32"), "inboundRootArray": SimTypePointer(SimStruct({"certData": SimStruct({"size": SimTypeInt(signed=False, label="UInt32"), "data": SimTypePointer(SimTypeChar(label="Byte"), offset=0)}, name="FWP_BYTE_BLOB", pack=False, align=None), "flags": SimTypeInt(signed=False, label="IKEEXT_CERT_FLAGS")}, name="IKEEXT_CERT_ROOT_CONFIG0", pack=False, align=None), offset=0)}, name="_Anonymous_e__Struct", pack=False, align=None), "inboundEnterpriseStoreConfig": SimTypePointer(SimStruct({"certData": SimStruct({"size": SimTypeInt(signed=False, label="UInt32"), "data": SimTypePointer(SimTypeChar(label="Byte"), offset=0)}, name="FWP_BYTE_BLOB", pack=False, align=None), "flags": SimTypeInt(signed=False, label="IKEEXT_CERT_FLAGS")}, name="IKEEXT_CERT_ROOT_CONFIG0", pack=False, align=None), offset=0), "inboundTrustedRootStoreConfig": SimTypePointer(SimStruct({"certData": SimStruct({"size": SimTypeInt(signed=False, label="UInt32"), "data": SimTypePointer(SimTypeChar(label="Byte"), offset=0)}, name="FWP_BYTE_BLOB", pack=False, align=None), "flags": SimTypeInt(signed=False, label="IKEEXT_CERT_FLAGS")}, name="IKEEXT_CERT_ROOT_CONFIG0", pack=False, align=None), offset=0)}, name="<anon>", label="None"), "outboundConfigType": SimTypeInt(signed=False, label="IKEEXT_CERT_CONFIG_TYPE"), "Anonymous2": SimUnion({"Anonymous": SimStruct({"outboundRootArraySize": SimTypeInt(signed=False, label="UInt32"), "outboundRootArray": SimTypePointer(SimStruct({"certData": SimStruct({"size": SimTypeInt(signed=False, label="UInt32"), "data": SimTypePointer(SimTypeChar(label="Byte"), offset=0)}, name="FWP_BYTE_BLOB", pack=False, align=None), "flags": SimTypeInt(signed=False, label="IKEEXT_CERT_FLAGS")}, name="IKEEXT_CERT_ROOT_CONFIG0", pack=False, align=None), offset=0)}, name="_Anonymous_e__Struct", pack=False, align=None), "outboundEnterpriseStoreConfig": SimTypePointer(SimStruct({"certData": SimStruct({"size": SimTypeInt(signed=False, label="UInt32"), "data": SimTypePointer(SimTypeChar(label="Byte"), offset=0)}, name="FWP_BYTE_BLOB", pack=False, align=None), "flags": SimTypeInt(signed=False, label="IKEEXT_CERT_FLAGS")}, name="IKEEXT_CERT_ROOT_CONFIG0", pack=False, align=None), offset=0), "outboundTrustedRootStoreConfig": SimTypePointer(SimStruct({"certData": SimStruct({"size": SimTypeInt(signed=False, label="UInt32"), "data": SimTypePointer(SimTypeChar(label="Byte"), offset=0)}, name="FWP_BYTE_BLOB", pack=False, align=None), "flags": SimTypeInt(signed=False, label="IKEEXT_CERT_FLAGS")}, name="IKEEXT_CERT_ROOT_CONFIG0", pack=False, align=None), offset=0)}, name="<anon>", label="None"), "flags": SimTypeInt(signed=False, label="IKEEXT_CERT_AUTH")}, name="IKEEXT_CERTIFICATE_AUTHENTICATION0", pack=False, align=None), "kerberosAuthentication": SimStruct({"flags": SimTypeInt(signed=False, label="IKEEXT_KERBEROS_AUTHENTICATION_FLAGS")}, name="IKEEXT_KERBEROS_AUTHENTICATION0", pack=False, align=None), "ntlmV2Authentication": SimStruct({"flags": SimTypeInt(signed=False, label="UInt32")}, name="IKEEXT_NTLM_V2_AUTHENTICATION0", pack=False, align=None), "sslAuthentication": SimStruct({"inboundConfigType": SimTypeInt(signed=False, label="IKEEXT_CERT_CONFIG_TYPE"), "Anonymous1": SimUnion({"Anonymous": SimStruct({"inboundRootArraySize": SimTypeInt(signed=False, label="UInt32"), "inboundRootArray": SimTypePointer(SimStruct({"certData": SimStruct({"size": SimTypeInt(signed=False, label="UInt32"), "data": SimTypePointer(SimTypeChar(label="Byte"), offset=0)}, name="FWP_BYTE_BLOB", pack=False, align=None), "flags": SimTypeInt(signed=False, label="IKEEXT_CERT_FLAGS")}, name="IKEEXT_CERT_ROOT_CONFIG0", pack=False, align=None), offset=0)}, name="_Anonymous_e__Struct", pack=False, align=None), "inboundEnterpriseStoreConfig": SimTypePointer(SimStruct({"certData": SimStruct({"size": SimTypeInt(signed=False, label="UInt32"), "data": SimTypePointer(SimTypeChar(label="Byte"), offset=0)}, name="FWP_BYTE_BLOB", pack=False, align=None), "flags": SimTypeInt(signed=False, label="IKEEXT_CERT_FLAGS")}, name="IKEEXT_CERT_ROOT_CONFIG0", pack=False, align=None), offset=0), "inboundTrustedRootStoreConfig": SimTypePointer(SimStruct({"certData": SimStruct({"size": SimTypeInt(signed=False, label="UInt32"), "data": SimTypePointer(SimTypeChar(label="Byte"), offset=0)}, name="FWP_BYTE_BLOB", pack=False, align=None), "flags": SimTypeInt(signed=False, label="IKEEXT_CERT_FLAGS")}, name="IKEEXT_CERT_ROOT_CONFIG0", pack=False, align=None), offset=0)}, name="<anon>", label="None"), "outboundConfigType": SimTypeInt(signed=False, label="IKEEXT_CERT_CONFIG_TYPE"), "Anonymous2": SimUnion({"Anonymous": SimStruct({"outboundRootArraySize": SimTypeInt(signed=False, label="UInt32"), "outboundRootArray": SimTypePointer(SimStruct({"certData": SimStruct({"size": SimTypeInt(signed=False, label="UInt32"), "data": SimTypePointer(SimTypeChar(label="Byte"), offset=0)}, name="FWP_BYTE_BLOB", pack=False, align=None), "flags": SimTypeInt(signed=False, label="IKEEXT_CERT_FLAGS")}, name="IKEEXT_CERT_ROOT_CONFIG0", pack=False, align=None), offset=0)}, name="_Anonymous_e__Struct", pack=False, align=None), "outboundEnterpriseStoreConfig": SimTypePointer(SimStruct({"certData": SimStruct({"size": SimTypeInt(signed=False, label="UInt32"), "data": SimTypePointer(SimTypeChar(label="Byte"), offset=0)}, name="FWP_BYTE_BLOB", pack=False, align=None), "flags": SimTypeInt(signed=False, label="IKEEXT_CERT_FLAGS")}, name="IKEEXT_CERT_ROOT_CONFIG0", pack=False, align=None), offset=0), "outboundTrustedRootStoreConfig": SimTypePointer(SimStruct({"certData": SimStruct({"size": SimTypeInt(signed=False, label="UInt32"), "data": SimTypePointer(SimTypeChar(label="Byte"), offset=0)}, name="FWP_BYTE_BLOB", pack=False, align=None), "flags": SimTypeInt(signed=False, label="IKEEXT_CERT_FLAGS")}, name="IKEEXT_CERT_ROOT_CONFIG0", pack=False, align=None), offset=0)}, name="<anon>", label="None"), "flags": SimTypeInt(signed=False, label="IKEEXT_CERT_AUTH")}, name="IKEEXT_CERTIFICATE_AUTHENTICATION0", pack=False, align=None), "cgaAuthentication": SimStruct({"keyContainerName": SimTypePointer(SimTypeChar(label="Char"), offset=0), "cspName": SimTypePointer(SimTypeChar(label="Char"), offset=0), "cspType": SimTypeInt(signed=False, label="UInt32"), "cgaModifier": SimStruct({"byteArray16": SimTypeFixedSizeArray(SimTypeChar(label="Byte"), 16)}, name="FWP_BYTE_ARRAY16", pack=False, align=None), "cgaCollisionCount": SimTypeChar(label="Byte")}, name="IKEEXT_IPV6_CGA_AUTHENTICATION0", pack=False, align=None)}, name="<anon>", label="None")}, name="IKEEXT_AUTHENTICATION_METHOD0", pack=False, align=None), offset=0), "initiatorImpersonationType": SimTypeInt(signed=False, label="IKEEXT_AUTHENTICATION_IMPERSONATION_TYPE")}, name="IKEEXT_EM_POLICY0", pack=False, align=None), offset=0)}, name="IPSEC_TUNNEL_POLICY0", pack=False, align=None), offset=0), "ikeMmPolicy": SimTypePointer(SimStruct({"softExpirationTime": SimTypeInt(signed=False, label="UInt32"), "numAuthenticationMethods": SimTypeInt(signed=False, label="UInt32"), "authenticationMethods": SimTypePointer(SimStruct({"authenticationMethodType": SimTypeInt(signed=False, label="IKEEXT_AUTHENTICATION_METHOD_TYPE"), "Anonymous": SimUnion({"presharedKeyAuthentication": SimStruct({"presharedKey": SimStruct({"size": SimTypeInt(signed=False, label="UInt32"), "data": SimTypePointer(SimTypeChar(label="Byte"), offset=0)}, name="FWP_BYTE_BLOB", pack=False, align=None)}, name="IKEEXT_PRESHARED_KEY_AUTHENTICATION0", pack=False, align=None), "certificateAuthentication": SimStruct({"inboundConfigType": SimTypeInt(signed=False, label="IKEEXT_CERT_CONFIG_TYPE"), "Anonymous1": SimUnion({"Anonymous": SimStruct({"inboundRootArraySize": SimTypeInt(signed=False, label="UInt32"), "inboundRootArray": SimTypePointer(SimStruct({"certData": SimStruct({"size": SimTypeInt(signed=False, label="UInt32"), "data": SimTypePointer(SimTypeChar(label="Byte"), offset=0)}, name="FWP_BYTE_BLOB", pack=False, align=None), "flags": SimTypeInt(signed=False, label="IKEEXT_CERT_FLAGS")}, name="IKEEXT_CERT_ROOT_CONFIG0", pack=False, align=None), offset=0)}, name="_Anonymous_e__Struct", pack=False, align=None), "inboundEnterpriseStoreConfig": SimTypePointer(SimStruct({"certData": SimStruct({"size": SimTypeInt(signed=False, label="UInt32"), "data": SimTypePointer(SimTypeChar(label="Byte"), offset=0)}, name="FWP_BYTE_BLOB", pack=False, align=None), "flags": SimTypeInt(signed=False, label="IKEEXT_CERT_FLAGS")}, name="IKEEXT_CERT_ROOT_CONFIG0", pack=False, align=None), offset=0), "inboundTrustedRootStoreConfig": SimTypePointer(SimStruct({"certData": SimStruct({"size": SimTypeInt(signed=False, label="UInt32"), "data": SimTypePointer(SimTypeChar(label="Byte"), offset=0)}, name="FWP_BYTE_BLOB", pack=False, align=None), "flags": SimTypeInt(signed=False, label="IKEEXT_CERT_FLAGS")}, name="IKEEXT_CERT_ROOT_CONFIG0", pack=False, align=None), offset=0)}, name="<anon>", label="None"), "outboundConfigType": SimTypeInt(signed=False, label="IKEEXT_CERT_CONFIG_TYPE"), "Anonymous2": SimUnion({"Anonymous": SimStruct({"outboundRootArraySize": SimTypeInt(signed=False, label="UInt32"), "outboundRootArray": SimTypePointer(SimStruct({"certData": SimStruct({"size": SimTypeInt(signed=False, label="UInt32"), "data": SimTypePointer(SimTypeChar(label="Byte"), offset=0)}, name="FWP_BYTE_BLOB", pack=False, align=None), "flags": SimTypeInt(signed=False, label="IKEEXT_CERT_FLAGS")}, name="IKEEXT_CERT_ROOT_CONFIG0", pack=False, align=None), offset=0)}, name="_Anonymous_e__Struct", pack=False, align=None), "outboundEnterpriseStoreConfig": SimTypePointer(SimStruct({"certData": SimStruct({"size": SimTypeInt(signed=False, label="UInt32"), "data": SimTypePointer(SimTypeChar(label="Byte"), offset=0)}, name="FWP_BYTE_BLOB", pack=False, align=None), "flags": SimTypeInt(signed=False, label="IKEEXT_CERT_FLAGS")}, name="IKEEXT_CERT_ROOT_CONFIG0", pack=False, align=None), offset=0), "outboundTrustedRootStoreConfig": SimTypePointer(SimStruct({"certData": SimStruct({"size": SimTypeInt(signed=False, label="UInt32"), "data": SimTypePointer(SimTypeChar(label="Byte"), offset=0)}, name="FWP_BYTE_BLOB", pack=False, align=None), "flags": SimTypeInt(signed=False, label="IKEEXT_CERT_FLAGS")}, name="IKEEXT_CERT_ROOT_CONFIG0", pack=False, align=None), offset=0)}, name="<anon>", label="None"), "flags": SimTypeInt(signed=False, label="IKEEXT_CERT_AUTH")}, name="IKEEXT_CERTIFICATE_AUTHENTICATION0", pack=False, align=None), "kerberosAuthentication": SimStruct({"flags": SimTypeInt(signed=False, label="IKEEXT_KERBEROS_AUTHENTICATION_FLAGS")}, name="IKEEXT_KERBEROS_AUTHENTICATION0", pack=False, align=None), "ntlmV2Authentication": SimStruct({"flags": SimTypeInt(signed=False, label="UInt32")}, name="IKEEXT_NTLM_V2_AUTHENTICATION0", pack=False, align=None), "sslAuthentication": SimStruct({"inboundConfigType": SimTypeInt(signed=False, label="IKEEXT_CERT_CONFIG_TYPE"), "Anonymous1": SimUnion({"Anonymous": SimStruct({"inboundRootArraySize": SimTypeInt(signed=False, label="UInt32"), "inboundRootArray": SimTypePointer(SimStruct({"certData": SimStruct({"size": SimTypeInt(signed=False, label="UInt32"), "data": SimTypePointer(SimTypeChar(label="Byte"), offset=0)}, name="FWP_BYTE_BLOB", pack=False, align=None), "flags": SimTypeInt(signed=False, label="IKEEXT_CERT_FLAGS")}, name="IKEEXT_CERT_ROOT_CONFIG0", pack=False, align=None), offset=0)}, name="_Anonymous_e__Struct", pack=False, align=None), "inboundEnterpriseStoreConfig": SimTypePointer(SimStruct({"certData": SimStruct({"size": SimTypeInt(signed=False, label="UInt32"), "data": SimTypePointer(SimTypeChar(label="Byte"), offset=0)}, name="FWP_BYTE_BLOB", pack=False, align=None), "flags": SimTypeInt(signed=False, label="IKEEXT_CERT_FLAGS")}, name="IKEEXT_CERT_ROOT_CONFIG0", pack=False, align=None), offset=0), "inboundTrustedRootStoreConfig": SimTypePointer(SimStruct({"certData": SimStruct({"size": SimTypeInt(signed=False, label="UInt32"), "data": SimTypePointer(SimTypeChar(label="Byte"), offset=0)}, name="FWP_BYTE_BLOB", pack=False, align=None), "flags": SimTypeInt(signed=False, label="IKEEXT_CERT_FLAGS")}, name="IKEEXT_CERT_ROOT_CONFIG0", pack=False, align=None), offset=0)}, name="<anon>", label="None"), "outboundConfigType": SimTypeInt(signed=False, label="IKEEXT_CERT_CONFIG_TYPE"), "Anonymous2": SimUnion({"Anonymous": SimStruct({"outboundRootArraySize": SimTypeInt(signed=False, label="UInt32"), "outboundRootArray": SimTypePointer(SimStruct({"certData": SimStruct({"size": SimTypeInt(signed=False, label="UInt32"), "data": SimTypePointer(SimTypeChar(label="Byte"), offset=0)}, name="FWP_BYTE_BLOB", pack=False, align=None), "flags": SimTypeInt(signed=False, label="IKEEXT_CERT_FLAGS")}, name="IKEEXT_CERT_ROOT_CONFIG0", pack=False, align=None), offset=0)}, name="_Anonymous_e__Struct", pack=False, align=None), "outboundEnterpriseStoreConfig": SimTypePointer(SimStruct({"certData": SimStruct({"size": SimTypeInt(signed=False, label="UInt32"), "data": SimTypePointer(SimTypeChar(label="Byte"), offset=0)}, name="FWP_BYTE_BLOB", pack=False, align=None), "flags": SimTypeInt(signed=False, label="IKEEXT_CERT_FLAGS")}, name="IKEEXT_CERT_ROOT_CONFIG0", pack=False, align=None), offset=0), "outboundTrustedRootStoreConfig": SimTypePointer(SimStruct({"certData": SimStruct({"size": SimTypeInt(signed=False, label="UInt32"), "data": SimTypePointer(SimTypeChar(label="Byte"), offset=0)}, name="FWP_BYTE_BLOB", pack=False, align=None), "flags": SimTypeInt(signed=False, label="IKEEXT_CERT_FLAGS")}, name="IKEEXT_CERT_ROOT_CONFIG0", pack=False, align=None), offset=0)}, name="<anon>", label="None"), "flags": SimTypeInt(signed=False, label="IKEEXT_CERT_AUTH")}, name="IKEEXT_CERTIFICATE_AUTHENTICATION0", pack=False, align=None), "cgaAuthentication": SimStruct({"keyContainerName": SimTypePointer(SimTypeChar(label="Char"), offset=0), "cspName": SimTypePointer(SimTypeChar(label="Char"), offset=0), "cspType": SimTypeInt(signed=False, label="UInt32"), "cgaModifier": SimStruct({"byteArray16": SimTypeFixedSizeArray(SimTypeChar(label="Byte"), 16)}, name="FWP_BYTE_ARRAY16", pack=False, align=None), "cgaCollisionCount": SimTypeChar(label="Byte")}, name="IKEEXT_IPV6_CGA_AUTHENTICATION0", pack=False, align=None)}, name="<anon>", label="None")}, name="IKEEXT_AUTHENTICATION_METHOD0", pack=False, align=None), offset=0), "initiatorImpersonationType": SimTypeInt(signed=False, label="IKEEXT_AUTHENTICATION_IMPERSONATION_TYPE"), "numIkeProposals": SimTypeInt(signed=False, label="UInt32"), "ikeProposals": SimTypePointer(SimStruct({"cipherAlgorithm": SimStruct({"algoIdentifier": SimTypeInt(signed=False, label="IKEEXT_CIPHER_TYPE"), "keyLen": SimTypeInt(signed=False, label="UInt32"), "rounds": SimTypeInt(signed=False, label="UInt32")}, name="IKEEXT_CIPHER_ALGORITHM0", pack=False, align=None), "integrityAlgorithm": SimStruct({"algoIdentifier": SimTypeInt(signed=False, label="IKEEXT_INTEGRITY_TYPE")}, name="IKEEXT_INTEGRITY_ALGORITHM0", pack=False, align=None), "maxLifetimeSeconds": SimTypeInt(signed=False, label="UInt32"), "dhGroup": SimTypeInt(signed=False, label="IKEEXT_DH_GROUP"), "quickModeLimit": SimTypeInt(signed=False, label="UInt32")}, name="IKEEXT_PROPOSAL0", pack=False, align=None), offset=0), "flags": SimTypeInt(signed=False, label="IKEEXT_POLICY_FLAG"), "maxDynamicFilters": SimTypeInt(signed=False, label="UInt32")}, name="IKEEXT_POLICY0", pack=False, align=None), offset=0), "authIpMmPolicy": SimTypePointer(SimStruct({"softExpirationTime": SimTypeInt(signed=False, label="UInt32"), "numAuthenticationMethods": SimTypeInt(signed=False, label="UInt32"), "authenticationMethods": SimTypePointer(SimStruct({"authenticationMethodType": SimTypeInt(signed=False, label="IKEEXT_AUTHENTICATION_METHOD_TYPE"), "Anonymous": SimUnion({"presharedKeyAuthentication": SimStruct({"presharedKey": SimStruct({"size": SimTypeInt(signed=False, label="UInt32"), "data": SimTypePointer(SimTypeChar(label="Byte"), offset=0)}, name="FWP_BYTE_BLOB", pack=False, align=None)}, name="IKEEXT_PRESHARED_KEY_AUTHENTICATION0", pack=False, align=None), "certificateAuthentication": SimStruct({"inboundConfigType": SimTypeInt(signed=False, label="IKEEXT_CERT_CONFIG_TYPE"), "Anonymous1": SimUnion({"Anonymous": SimStruct({"inboundRootArraySize": SimTypeInt(signed=False, label="UInt32"), "inboundRootArray": SimTypePointer(SimStruct({"certData": SimStruct({"size": SimTypeInt(signed=False, label="UInt32"), "data": SimTypePointer(SimTypeChar(label="Byte"), offset=0)}, name="FWP_BYTE_BLOB", pack=False, align=None), "flags": SimTypeInt(signed=False, label="IKEEXT_CERT_FLAGS")}, name="IKEEXT_CERT_ROOT_CONFIG0", pack=False, align=None), offset=0)}, name="_Anonymous_e__Struct", pack=False, align=None), "inboundEnterpriseStoreConfig": SimTypePointer(SimStruct({"certData": SimStruct({"size": SimTypeInt(signed=False, label="UInt32"), "data": SimTypePointer(SimTypeChar(label="Byte"), offset=0)}, name="FWP_BYTE_BLOB", pack=False, align=None), "flags": SimTypeInt(signed=False, label="IKEEXT_CERT_FLAGS")}, name="IKEEXT_CERT_ROOT_CONFIG0", pack=False, align=None), offset=0), "inboundTrustedRootStoreConfig": SimTypePointer(SimStruct({"certData": SimStruct({"size": SimTypeInt(signed=False, label="UInt32"), "data": SimTypePointer(SimTypeChar(label="Byte"), offset=0)}, name="FWP_BYTE_BLOB", pack=False, align=None), "flags": SimTypeInt(signed=False, label="IKEEXT_CERT_FLAGS")}, name="IKEEXT_CERT_ROOT_CONFIG0", pack=False, align=None), offset=0)}, name="<anon>", label="None"), "outboundConfigType": SimTypeInt(signed=False, label="IKEEXT_CERT_CONFIG_TYPE"), "Anonymous2": SimUnion({"Anonymous": SimStruct({"outboundRootArraySize": SimTypeInt(signed=False, label="UInt32"), "outboundRootArray": SimTypePointer(SimStruct({"certData": SimStruct({"size": SimTypeInt(signed=False, label="UInt32"), "data": SimTypePointer(SimTypeChar(label="Byte"), offset=0)}, name="FWP_BYTE_BLOB", pack=False, align=None), "flags": SimTypeInt(signed=False, label="IKEEXT_CERT_FLAGS")}, name="IKEEXT_CERT_ROOT_CONFIG0", pack=False, align=None), offset=0)}, name="_Anonymous_e__Struct", pack=False, align=None), "outboundEnterpriseStoreConfig": SimTypePointer(SimStruct({"certData": SimStruct({"size": SimTypeInt(signed=False, label="UInt32"), "data": SimTypePointer(SimTypeChar(label="Byte"), offset=0)}, name="FWP_BYTE_BLOB", pack=False, align=None), "flags": SimTypeInt(signed=False, label="IKEEXT_CERT_FLAGS")}, name="IKEEXT_CERT_ROOT_CONFIG0", pack=False, align=None), offset=0), "outboundTrustedRootStoreConfig": SimTypePointer(SimStruct({"certData": SimStruct({"size": SimTypeInt(signed=False, label="UInt32"), "data": SimTypePointer(SimTypeChar(label="Byte"), offset=0)}, name="FWP_BYTE_BLOB", pack=False, align=None), "flags": SimTypeInt(signed=False, label="IKEEXT_CERT_FLAGS")}, name="IKEEXT_CERT_ROOT_CONFIG0", pack=False, align=None), offset=0)}, name="<anon>", label="None"), "flags": SimTypeInt(signed=False, label="IKEEXT_CERT_AUTH")}, name="IKEEXT_CERTIFICATE_AUTHENTICATION0", pack=False, align=None), "kerberosAuthentication": SimStruct({"flags": SimTypeInt(signed=False, label="IKEEXT_KERBEROS_AUTHENTICATION_FLAGS")}, name="IKEEXT_KERBEROS_AUTHENTICATION0", pack=False, align=None), "ntlmV2Authentication": SimStruct({"flags": SimTypeInt(signed=False, label="UInt32")}, name="IKEEXT_NTLM_V2_AUTHENTICATION0", pack=False, align=None), "sslAuthentication": SimStruct({"inboundConfigType": SimTypeInt(signed=False, label="IKEEXT_CERT_CONFIG_TYPE"), "Anonymous1": SimUnion({"Anonymous": SimStruct({"inboundRootArraySize": SimTypeInt(signed=False, label="UInt32"), "inboundRootArray": SimTypePointer(SimStruct({"certData": SimStruct({"size": SimTypeInt(signed=False, label="UInt32"), "data": SimTypePointer(SimTypeChar(label="Byte"), offset=0)}, name="FWP_BYTE_BLOB", pack=False, align=None), "flags": SimTypeInt(signed=False, label="IKEEXT_CERT_FLAGS")}, name="IKEEXT_CERT_ROOT_CONFIG0", pack=False, align=None), offset=0)}, name="_Anonymous_e__Struct", pack=False, align=None), "inboundEnterpriseStoreConfig": SimTypePointer(SimStruct({"certData": SimStruct({"size": SimTypeInt(signed=False, label="UInt32"), "data": SimTypePointer(SimTypeChar(label="Byte"), offset=0)}, name="FWP_BYTE_BLOB", pack=False, align=None), "flags": SimTypeInt(signed=False, label="IKEEXT_CERT_FLAGS")}, name="IKEEXT_CERT_ROOT_CONFIG0", pack=False, align=None), offset=0), "inboundTrustedRootStoreConfig": SimTypePointer(SimStruct({"certData": SimStruct({"size": SimTypeInt(signed=False, label="UInt32"), "data": SimTypePointer(SimTypeChar(label="Byte"), offset=0)}, name="FWP_BYTE_BLOB", pack=False, align=None), "flags": SimTypeInt(signed=False, label="IKEEXT_CERT_FLAGS")}, name="IKEEXT_CERT_ROOT_CONFIG0", pack=False, align=None), offset=0)}, name="<anon>", label="None"), "outboundConfigType": SimTypeInt(signed=False, label="IKEEXT_CERT_CONFIG_TYPE"), "Anonymous2": SimUnion({"Anonymous": SimStruct({"outboundRootArraySize": SimTypeInt(signed=False, label="UInt32"), "outboundRootArray": SimTypePointer(SimStruct({"certData": SimStruct({"size": SimTypeInt(signed=False, label="UInt32"), "data": SimTypePointer(SimTypeChar(label="Byte"), offset=0)}, name="FWP_BYTE_BLOB", pack=False, align=None), "flags": SimTypeInt(signed=False, label="IKEEXT_CERT_FLAGS")}, name="IKEEXT_CERT_ROOT_CONFIG0", pack=False, align=None), offset=0)}, name="_Anonymous_e__Struct", pack=False, align=None), "outboundEnterpriseStoreConfig": SimTypePointer(SimStruct({"certData": SimStruct({"size": SimTypeInt(signed=False, label="UInt32"), "data": SimTypePointer(SimTypeChar(label="Byte"), offset=0)}, name="FWP_BYTE_BLOB", pack=False, align=None), "flags": SimTypeInt(signed=False, label="IKEEXT_CERT_FLAGS")}, name="IKEEXT_CERT_ROOT_CONFIG0", pack=False, align=None), offset=0), "outboundTrustedRootStoreConfig": SimTypePointer(SimStruct({"certData": SimStruct({"size": SimTypeInt(signed=False, label="UInt32"), "data": SimTypePointer(SimTypeChar(label="Byte"), offset=0)}, name="FWP_BYTE_BLOB", pack=False, align=None), "flags": SimTypeInt(signed=False, label="IKEEXT_CERT_FLAGS")}, name="IKEEXT_CERT_ROOT_CONFIG0", pack=False, align=None), offset=0)}, name="<anon>", label="None"), "flags": SimTypeInt(signed=False, label="IKEEXT_CERT_AUTH")}, name="IKEEXT_CERTIFICATE_AUTHENTICATION0", pack=False, align=None), "cgaAuthentication": SimStruct({"keyContainerName": SimTypePointer(SimTypeChar(label="Char"), offset=0), "cspName": SimTypePointer(SimTypeChar(label="Char"), offset=0), "cspType": SimTypeInt(signed=False, label="UInt32"), "cgaModifier": SimStruct({"byteArray16": SimTypeFixedSizeArray(SimTypeChar(label="Byte"), 16)}, name="FWP_BYTE_ARRAY16", pack=False, align=None), "cgaCollisionCount": SimTypeChar(label="Byte")}, name="IKEEXT_IPV6_CGA_AUTHENTICATION0", pack=False, align=None)}, name="<anon>", label="None")}, name="IKEEXT_AUTHENTICATION_METHOD0", pack=False, align=None), offset=0), "initiatorImpersonationType": SimTypeInt(signed=False, label="IKEEXT_AUTHENTICATION_IMPERSONATION_TYPE"), "numIkeProposals": SimTypeInt(signed=False, label="UInt32"), "ikeProposals": SimTypePointer(SimStruct({"cipherAlgorithm": SimStruct({"algoIdentifier": SimTypeInt(signed=False, label="IKEEXT_CIPHER_TYPE"), "keyLen": SimTypeInt(signed=False, label="UInt32"), "rounds": SimTypeInt(signed=False, label="UInt32")}, name="IKEEXT_CIPHER_ALGORITHM0", pack=False, align=None), "integrityAlgorithm": SimStruct({"algoIdentifier": SimTypeInt(signed=False, label="IKEEXT_INTEGRITY_TYPE")}, name="IKEEXT_INTEGRITY_ALGORITHM0", pack=False, align=None), "maxLifetimeSeconds": SimTypeInt(signed=False, label="UInt32"), "dhGroup": SimTypeInt(signed=False, label="IKEEXT_DH_GROUP"), "quickModeLimit": SimTypeInt(signed=False, label="UInt32")}, name="IKEEXT_PROPOSAL0", pack=False, align=None), offset=0), "flags": SimTypeInt(signed=False, label="IKEEXT_POLICY_FLAG"), "maxDynamicFilters": SimTypeInt(signed=False, label="UInt32")}, name="IKEEXT_POLICY0", pack=False, align=None), offset=0), "dataBuffer": SimTypePointer(SimStruct({"size": SimTypeInt(signed=False, label="UInt32"), "data": SimTypePointer(SimTypeChar(label="Byte"), offset=0)}, name="FWP_BYTE_BLOB", pack=False, align=None), offset=0), "classifyOptions": SimTypePointer(SimStruct({"numOptions": SimTypeInt(signed=False, label="UInt32"), "options": SimTypePointer(SimStruct({"type": SimTypeInt(signed=False, label="FWP_CLASSIFY_OPTION_TYPE"), "value": SimStruct({"type": SimTypeInt(signed=False, label="FWP_DATA_TYPE"), "Anonymous": SimUnion({"uint8": SimTypeChar(label="Byte"), "uint16": SimTypeShort(signed=False, label="UInt16"), "uint32": SimTypeInt(signed=False, label="UInt32"), "uint64": SimTypePointer(SimTypeLongLong(signed=False, label="UInt64"), offset=0), "int8": SimTypeChar(label="SByte"), "int16": SimTypeShort(signed=True, label="Int16"), "int32": SimTypeInt(signed=True, label="Int32"), "int64": SimTypePointer(SimTypeLongLong(signed=True, label="Int64"), offset=0), "float32": SimTypeFloat(size=32), "double64": SimTypePointer(SimTypeFloat(size=64), offset=0), "byteArray16": SimTypePointer(SimStruct({"byteArray16": SimTypeFixedSizeArray(SimTypeChar(label="Byte"), 16)}, name="FWP_BYTE_ARRAY16", pack=False, align=None), offset=0), "byteBlob": SimTypePointer(SimStruct({"size": SimTypeInt(signed=False, label="UInt32"), "data": SimTypePointer(SimTypeChar(label="Byte"), offset=0)}, name="FWP_BYTE_BLOB", pack=False, align=None), offset=0), "sid": SimTypePointer(SimTypeBottom(label="SID"), offset=0), "sd": SimTypePointer(SimStruct({"size": SimTypeInt(signed=False, label="UInt32"), "data": SimTypePointer(SimTypeChar(label="Byte"), offset=0)}, name="FWP_BYTE_BLOB", pack=False, align=None), offset=0), "tokenInformation": SimTypePointer(SimStruct({"sidCount": SimTypeInt(signed=False, label="UInt32"), "sids": SimTypePointer(SimTypeBottom(label="SID_AND_ATTRIBUTES"), offset=0), "restrictedSidCount": SimTypeInt(signed=False, label="UInt32"), "restrictedSids": SimTypePointer(SimTypeBottom(label="SID_AND_ATTRIBUTES"), offset=0)}, name="FWP_TOKEN_INFORMATION", pack=False, align=None), offset=0), "tokenAccessInformation": SimTypePointer(SimStruct({"size": SimTypeInt(signed=False, label="UInt32"), "data": SimTypePointer(SimTypeChar(label="Byte"), offset=0)}, name="FWP_BYTE_BLOB", pack=False, align=None), offset=0), "unicodeString": SimTypePointer(SimTypeChar(label="Char"), offset=0), "byteArray6": SimTypePointer(SimStruct({"byteArray6": SimTypeFixedSizeArray(SimTypeChar(label="Byte"), 6)}, name="FWP_BYTE_ARRAY6", pack=False, align=None), offset=0), "bitmapArray64": SimTypePointer(SimStruct({"bitmapArray64": SimTypeFixedSizeArray(SimTypeChar(label="Byte"), 8)}, name="FWP_BITMAP_ARRAY64_", pack=False, align=None), offset=0)}, name="<anon>", label="None")}, name="FWP_VALUE0", pack=False, align=None)}, name="FWPM_CLASSIFY_OPTION0", pack=False, align=None), offset=0)}, name="FWPM_CLASSIFY_OPTIONS0", pack=False, align=None), offset=0)}, name="<anon>", label="None"), "providerContextId": SimTypeLongLong(signed=False, label="UInt64")}, name="FWPM_PROVIDER_CONTEXT0", pack=False, align=None), offset=0), SimTypeInt(signed=False, label="UInt32"), SimTypePointer(SimStruct({"fieldKey": SimTypeBottom(label="Guid"), "matchType": SimTypeInt(signed=False, label="FWP_MATCH_TYPE"), "conditionValue": SimStruct({"type": SimTypeInt(signed=False, label="FWP_DATA_TYPE"), "Anonymous": SimUnion({"uint8": SimTypeChar(label="Byte"), "uint16": SimTypeShort(signed=False, label="UInt16"), "uint32": SimTypeInt(signed=False, label="UInt32"), "uint64": SimTypePointer(SimTypeLongLong(signed=False, label="UInt64"), offset=0), "int8": SimTypeChar(label="SByte"), "int16": SimTypeShort(signed=True, label="Int16"), "int32": SimTypeInt(signed=True, label="Int32"), "int64": SimTypePointer(SimTypeLongLong(signed=True, label="Int64"), offset=0), "float32": SimTypeFloat(size=32), "double64": SimTypePointer(SimTypeFloat(size=64), offset=0), "byteArray16": SimTypePointer(SimStruct({"byteArray16": SimTypeFixedSizeArray(SimTypeChar(label="Byte"), 16)}, name="FWP_BYTE_ARRAY16", pack=False, align=None), offset=0), "byteBlob": SimTypePointer(SimStruct({"size": SimTypeInt(signed=False, label="UInt32"), "data": SimTypePointer(SimTypeChar(label="Byte"), offset=0)}, name="FWP_BYTE_BLOB", pack=False, align=None), offset=0), "sid": SimTypePointer(SimTypeBottom(label="SID"), offset=0), "sd": SimTypePointer(SimStruct({"size": SimTypeInt(signed=False, label="UInt32"), "data": SimTypePointer(SimTypeChar(label="Byte"), offset=0)}, name="FWP_BYTE_BLOB", pack=False, align=None), offset=0), "tokenInformation": SimTypePointer(SimStruct({"sidCount": SimTypeInt(signed=False, label="UInt32"), "sids": SimTypePointer(SimTypeBottom(label="SID_AND_ATTRIBUTES"), offset=0), "restrictedSidCount": SimTypeInt(signed=False, label="UInt32"), "restrictedSids": SimTypePointer(SimTypeBottom(label="SID_AND_ATTRIBUTES"), offset=0)}, name="FWP_TOKEN_INFORMATION", pack=False, align=None), offset=0), "tokenAccessInformation": SimTypePointer(SimStruct({"size": SimTypeInt(signed=False, label="UInt32"), "data": SimTypePointer(SimTypeChar(label="Byte"), offset=0)}, name="FWP_BYTE_BLOB", pack=False, align=None), offset=0), "unicodeString": SimTypePointer(SimTypeChar(label="Char"), offset=0), "byteArray6": SimTypePointer(SimStruct({"byteArray6": SimTypeFixedSizeArray(SimTypeChar(label="Byte"), 6)}, name="FWP_BYTE_ARRAY6", pack=False, align=None), offset=0), "bitmapArray64": SimTypePointer(SimStruct({"bitmapArray64": SimTypeFixedSizeArray(SimTypeChar(label="Byte"), 8)}, name="FWP_BITMAP_ARRAY64_", pack=False, align=None), offset=0), "v4AddrMask": SimTypePointer(SimStruct({"addr": SimTypeInt(signed=False, label="UInt32"), "mask": SimTypeInt(signed=False, label="UInt32")}, name="FWP_V4_ADDR_AND_MASK", pack=False, align=None), offset=0), "v6AddrMask": SimTypePointer(SimStruct({"addr": SimTypeFixedSizeArray(SimTypeChar(label="Byte"), 16), "prefixLength": SimTypeChar(label="Byte")}, name="FWP_V6_ADDR_AND_MASK", pack=False, align=None), offset=0), "rangeValue": SimTypePointer(SimStruct({"valueLow": SimStruct({"type": SimTypeInt(signed=False, label="FWP_DATA_TYPE"), "Anonymous": SimUnion({"uint8": SimTypeChar(label="Byte"), "uint16": SimTypeShort(signed=False, label="UInt16"), "uint32": SimTypeInt(signed=False, label="UInt32"), "uint64": SimTypePointer(SimTypeLongLong(signed=False, label="UInt64"), offset=0), "int8": SimTypeChar(label="SByte"), "int16": SimTypeShort(signed=True, label="Int16"), "int32": SimTypeInt(signed=True, label="Int32"), "int64": SimTypePointer(SimTypeLongLong(signed=True, label="Int64"), offset=0), "float32": SimTypeFloat(size=32), "double64": SimTypePointer(SimTypeFloat(size=64), offset=0), "byteArray16": SimTypePointer(SimStruct({"byteArray16": SimTypeFixedSizeArray(SimTypeChar(label="Byte"), 16)}, name="FWP_BYTE_ARRAY16", pack=False, align=None), offset=0), "byteBlob": SimTypePointer(SimStruct({"size": SimTypeInt(signed=False, label="UInt32"), "data": SimTypePointer(SimTypeChar(label="Byte"), offset=0)}, name="FWP_BYTE_BLOB", pack=False, align=None), offset=0), "sid": SimTypePointer(SimTypeBottom(label="SID"), offset=0), "sd": SimTypePointer(SimStruct({"size": SimTypeInt(signed=False, label="UInt32"), "data": SimTypePointer(SimTypeChar(label="Byte"), offset=0)}, name="FWP_BYTE_BLOB", pack=False, align=None), offset=0), "tokenInformation": SimTypePointer(SimStruct({"sidCount": SimTypeInt(signed=False, label="UInt32"), "sids": SimTypePointer(SimTypeBottom(label="SID_AND_ATTRIBUTES"), offset=0), "restrictedSidCount": SimTypeInt(signed=False, label="UInt32"), "restrictedSids": SimTypePointer(SimTypeBottom(label="SID_AND_ATTRIBUTES"), offset=0)}, name="FWP_TOKEN_INFORMATION", pack=False, align=None), offset=0), "tokenAccessInformation": SimTypePointer(SimStruct({"size": SimTypeInt(signed=False, label="UInt32"), "data": SimTypePointer(SimTypeChar(label="Byte"), offset=0)}, name="FWP_BYTE_BLOB", pack=False, align=None), offset=0), "unicodeString": SimTypePointer(SimTypeChar(label="Char"), offset=0), "byteArray6": SimTypePointer(SimStruct({"byteArray6": SimTypeFixedSizeArray(SimTypeChar(label="Byte"), 6)}, name="FWP_BYTE_ARRAY6", pack=False, align=None), offset=0), "bitmapArray64": SimTypePointer(SimStruct({"bitmapArray64": SimTypeFixedSizeArray(SimTypeChar(label="Byte"), 8)}, name="FWP_BITMAP_ARRAY64_", pack=False, align=None), offset=0)}, name="<anon>", label="None")}, name="FWP_VALUE0", pack=False, align=None), "valueHigh": SimStruct({"type": SimTypeInt(signed=False, label="FWP_DATA_TYPE"), "Anonymous": SimUnion({"uint8": SimTypeChar(label="Byte"), "uint16": SimTypeShort(signed=False, label="UInt16"), "uint32": SimTypeInt(signed=False, label="UInt32"), "uint64": SimTypePointer(SimTypeLongLong(signed=False, label="UInt64"), offset=0), "int8": SimTypeChar(label="SByte"), "int16": SimTypeShort(signed=True, label="Int16"), "int32": SimTypeInt(signed=True, label="Int32"), "int64": SimTypePointer(SimTypeLongLong(signed=True, label="Int64"), offset=0), "float32": SimTypeFloat(size=32), "double64": SimTypePointer(SimTypeFloat(size=64), offset=0), "byteArray16": SimTypePointer(SimStruct({"byteArray16": SimTypeFixedSizeArray(SimTypeChar(label="Byte"), 16)}, name="FWP_BYTE_ARRAY16", pack=False, align=None), offset=0), "byteBlob": SimTypePointer(SimStruct({"size": SimTypeInt(signed=False, label="UInt32"), "data": SimTypePointer(SimTypeChar(label="Byte"), offset=0)}, name="FWP_BYTE_BLOB", pack=False, align=None), offset=0), "sid": SimTypePointer(SimTypeBottom(label="SID"), offset=0), "sd": SimTypePointer(SimStruct({"size": SimTypeInt(signed=False, label="UInt32"), "data": SimTypePointer(SimTypeChar(label="Byte"), offset=0)}, name="FWP_BYTE_BLOB", pack=False, align=None), offset=0), "tokenInformation": SimTypePointer(SimStruct({"sidCount": SimTypeInt(signed=False, label="UInt32"), "sids": SimTypePointer(SimTypeBottom(label="SID_AND_ATTRIBUTES"), offset=0), "restrictedSidCount": SimTypeInt(signed=False, label="UInt32"), "restrictedSids": SimTypePointer(SimTypeBottom(label="SID_AND_ATTRIBUTES"), offset=0)}, name="FWP_TOKEN_INFORMATION", pack=False, align=None), offset=0), "tokenAccessInformation": SimTypePointer(SimStruct({"size": SimTypeInt(signed=False, label="UInt32"), "data": SimTypePointer(SimTypeChar(label="Byte"), offset=0)}, name="FWP_BYTE_BLOB", pack=False, align=None), offset=0), "unicodeString": SimTypePointer(SimTypeChar(label="Char"), offset=0), "byteArray6": SimTypePointer(SimStruct({"byteArray6": SimTypeFixedSizeArray(SimTypeChar(label="Byte"), 6)}, name="FWP_BYTE_ARRAY6", pack=False, align=None), offset=0), "bitmapArray64": SimTypePointer(SimStruct({"bitmapArray64": SimTypeFixedSizeArray(SimTypeChar(label="Byte"), 8)}, name="FWP_BITMAP_ARRAY64_", pack=False, align=None), offset=0)}, name="<anon>", label="None")}, name="FWP_VALUE0", pack=False, align=None)}, name="FWP_RANGE0", pack=False, align=None), offset=0)}, name="<anon>", label="None")}, name="FWP_CONDITION_VALUE0", pack=False, align=None)}, name="FWPM_FILTER_CONDITION0", pack=False, align=None), label="LPArray", offset=0), SimTypePointer(SimStruct({"Revision": SimTypeChar(label="Byte"), "Sbz1": SimTypeChar(label="Byte"), "Control": SimTypeShort(signed=False, label="UInt16"), "Owner": SimTypePointer(SimTypeInt(signed=True, label="Int"), label="IntPtr", offset=0), "Group": SimTypePointer(SimTypeInt(signed=True, label="Int"), label="IntPtr", offset=0), "Sacl": SimTypePointer(SimStruct({"AclRevision": SimTypeChar(label="Byte"), "Sbz1": SimTypeChar(label="Byte"), "AclSize": SimTypeShort(signed=False, label="UInt16"), "AceCount": SimTypeShort(signed=False, label="UInt16"), "Sbz2": SimTypeShort(signed=False, label="UInt16")}, name="ACL", pack=False, align=None), offset=0), "Dacl": SimTypePointer(SimStruct({"AclRevision": SimTypeChar(label="Byte"), "Sbz1": SimTypeChar(label="Byte"), "AclSize": SimTypeShort(signed=False, label="UInt16"), "AceCount": SimTypeShort(signed=False, label="UInt16"), "Sbz2": SimTypeShort(signed=False, label="UInt16")}, name="ACL", pack=False, align=None), offset=0)}, name="SECURITY_DESCRIPTOR", pack=False, align=None), offset=0)], SimTypeInt(signed=False, label="UInt32"), arg_names=["engineHandle", "flags", "mainModePolicy", "tunnelPolicy", "numFilterConditions", "filterConditions", "sd"]),
        # 
        'FwpmIPsecTunnelAdd1': SimTypeFunction([SimTypePointer(SimTypeInt(signed=True, label="Int"), label="IntPtr", offset=0), SimTypeInt(signed=False, label="UInt32"), SimTypePointer(SimStruct({"providerContextKey": SimTypeBottom(label="Guid"), "displayData": SimStruct({"name": SimTypePointer(SimTypeChar(label="Char"), offset=0), "description": SimTypePointer(SimTypeChar(label="Char"), offset=0)}, name="FWPM_DISPLAY_DATA0", pack=False, align=None), "flags": SimTypeInt(signed=False, label="UInt32"), "providerKey": SimTypePointer(SimTypeBottom(label="Guid"), offset=0), "providerData": SimStruct({"size": SimTypeInt(signed=False, label="UInt32"), "data": SimTypePointer(SimTypeChar(label="Byte"), offset=0)}, name="FWP_BYTE_BLOB", pack=False, align=None), "type": SimTypeInt(signed=False, label="FWPM_PROVIDER_CONTEXT_TYPE"), "Anonymous": SimUnion({"keyingPolicy": SimTypePointer(SimStruct({"numKeyMods": SimTypeInt(signed=False, label="UInt32"), "keyModKeys": SimTypePointer(SimTypeBottom(label="Guid"), offset=0)}, name="IPSEC_KEYING_POLICY0", pack=False, align=None), offset=0), "ikeQmTransportPolicy": SimTypePointer(SimStruct({"numIpsecProposals": SimTypeInt(signed=False, label="UInt32"), "ipsecProposals": SimTypePointer(SimStruct({"lifetime": SimStruct({"lifetimeSeconds": SimTypeInt(signed=False, label="UInt32"), "lifetimeKilobytes": SimTypeInt(signed=False, label="UInt32"), "lifetimePackets": SimTypeInt(signed=False, label="UInt32")}, name="IPSEC_SA_LIFETIME0", pack=False, align=None), "numSaTransforms": SimTypeInt(signed=False, label="UInt32"), "saTransforms": SimTypePointer(SimStruct({"ipsecTransformType": SimTypeInt(signed=False, label="IPSEC_TRANSFORM_TYPE"), "Anonymous": SimUnion({"ahTransform": SimTypePointer(SimStruct({"authTransformId": SimStruct({"authType": SimTypeInt(signed=False, label="IPSEC_AUTH_TYPE"), "authConfig": SimTypeChar(label="Byte")}, name="IPSEC_AUTH_TRANSFORM_ID0", pack=False, align=None), "cryptoModuleId": SimTypePointer(SimTypeBottom(label="Guid"), offset=0)}, name="IPSEC_AUTH_TRANSFORM0", pack=False, align=None), offset=0), "espAuthTransform": SimTypePointer(SimStruct({"authTransformId": SimStruct({"authType": SimTypeInt(signed=False, label="IPSEC_AUTH_TYPE"), "authConfig": SimTypeChar(label="Byte")}, name="IPSEC_AUTH_TRANSFORM_ID0", pack=False, align=None), "cryptoModuleId": SimTypePointer(SimTypeBottom(label="Guid"), offset=0)}, name="IPSEC_AUTH_TRANSFORM0", pack=False, align=None), offset=0), "espCipherTransform": SimTypePointer(SimStruct({"cipherTransformId": SimStruct({"cipherType": SimTypeInt(signed=False, label="IPSEC_CIPHER_TYPE"), "cipherConfig": SimTypeChar(label="Byte")}, name="IPSEC_CIPHER_TRANSFORM_ID0", pack=False, align=None), "cryptoModuleId": SimTypePointer(SimTypeBottom(label="Guid"), offset=0)}, name="IPSEC_CIPHER_TRANSFORM0", pack=False, align=None), offset=0), "espAuthAndCipherTransform": SimTypePointer(SimStruct({"authTransform": SimStruct({"authTransformId": SimStruct({"authType": SimTypeInt(signed=False, label="IPSEC_AUTH_TYPE"), "authConfig": SimTypeChar(label="Byte")}, name="IPSEC_AUTH_TRANSFORM_ID0", pack=False, align=None), "cryptoModuleId": SimTypePointer(SimTypeBottom(label="Guid"), offset=0)}, name="IPSEC_AUTH_TRANSFORM0", pack=False, align=None), "cipherTransform": SimStruct({"cipherTransformId": SimStruct({"cipherType": SimTypeInt(signed=False, label="IPSEC_CIPHER_TYPE"), "cipherConfig": SimTypeChar(label="Byte")}, name="IPSEC_CIPHER_TRANSFORM_ID0", pack=False, align=None), "cryptoModuleId": SimTypePointer(SimTypeBottom(label="Guid"), offset=0)}, name="IPSEC_CIPHER_TRANSFORM0", pack=False, align=None)}, name="IPSEC_AUTH_AND_CIPHER_TRANSFORM0", pack=False, align=None), offset=0), "espAuthFwTransform": SimTypePointer(SimStruct({"authTransformId": SimStruct({"authType": SimTypeInt(signed=False, label="IPSEC_AUTH_TYPE"), "authConfig": SimTypeChar(label="Byte")}, name="IPSEC_AUTH_TRANSFORM_ID0", pack=False, align=None), "cryptoModuleId": SimTypePointer(SimTypeBottom(label="Guid"), offset=0)}, name="IPSEC_AUTH_TRANSFORM0", pack=False, align=None), offset=0)}, name="<anon>", label="None")}, name="IPSEC_SA_TRANSFORM0", pack=False, align=None), offset=0), "pfsGroup": SimTypeInt(signed=False, label="IPSEC_PFS_GROUP")}, name="IPSEC_PROPOSAL0", pack=False, align=None), offset=0), "flags": SimTypeInt(signed=False, label="IPSEC_POLICY_FLAG"), "ndAllowClearTimeoutSeconds": SimTypeInt(signed=False, label="UInt32"), "saIdleTimeout": SimStruct({"idleTimeoutSeconds": SimTypeInt(signed=False, label="UInt32"), "idleTimeoutSecondsFailOver": SimTypeInt(signed=False, label="UInt32")}, name="IPSEC_SA_IDLE_TIMEOUT0", pack=False, align=None), "emPolicy": SimTypePointer(SimStruct({"numAuthenticationMethods": SimTypeInt(signed=False, label="UInt32"), "authenticationMethods": SimTypePointer(SimStruct({"authenticationMethodType": SimTypeInt(signed=False, label="IKEEXT_AUTHENTICATION_METHOD_TYPE"), "Anonymous": SimUnion({"presharedKeyAuthentication": SimStruct({"presharedKey": SimStruct({"size": SimTypeInt(signed=False, label="UInt32"), "data": SimTypePointer(SimTypeChar(label="Byte"), offset=0)}, name="FWP_BYTE_BLOB", pack=False, align=None), "flags": SimTypeInt(signed=False, label="IKEEXT_PRESHARED_KEY_AUTHENTICATION_FLAGS")}, name="IKEEXT_PRESHARED_KEY_AUTHENTICATION1", pack=False, align=None), "certificateAuthentication": SimStruct({"inboundConfigType": SimTypeInt(signed=False, label="IKEEXT_CERT_CONFIG_TYPE"), "Anonymous1": SimUnion({"Anonymous": SimStruct({"inboundRootArraySize": SimTypeInt(signed=False, label="UInt32"), "inboundRootArray": SimTypePointer(SimStruct({"certData": SimStruct({"size": SimTypeInt(signed=False, label="UInt32"), "data": SimTypePointer(SimTypeChar(label="Byte"), offset=0)}, name="FWP_BYTE_BLOB", pack=False, align=None), "flags": SimTypeInt(signed=False, label="IKEEXT_CERT_FLAGS")}, name="IKEEXT_CERT_ROOT_CONFIG0", pack=False, align=None), offset=0)}, name="_Anonymous_e__Struct", pack=False, align=None), "inboundEnterpriseStoreConfig": SimTypePointer(SimStruct({"certData": SimStruct({"size": SimTypeInt(signed=False, label="UInt32"), "data": SimTypePointer(SimTypeChar(label="Byte"), offset=0)}, name="FWP_BYTE_BLOB", pack=False, align=None), "flags": SimTypeInt(signed=False, label="IKEEXT_CERT_FLAGS")}, name="IKEEXT_CERT_ROOT_CONFIG0", pack=False, align=None), offset=0), "inboundTrustedRootStoreConfig": SimTypePointer(SimStruct({"certData": SimStruct({"size": SimTypeInt(signed=False, label="UInt32"), "data": SimTypePointer(SimTypeChar(label="Byte"), offset=0)}, name="FWP_BYTE_BLOB", pack=False, align=None), "flags": SimTypeInt(signed=False, label="IKEEXT_CERT_FLAGS")}, name="IKEEXT_CERT_ROOT_CONFIG0", pack=False, align=None), offset=0)}, name="<anon>", label="None"), "outboundConfigType": SimTypeInt(signed=False, label="IKEEXT_CERT_CONFIG_TYPE"), "Anonymous2": SimUnion({"Anonymous": SimStruct({"outboundRootArraySize": SimTypeInt(signed=False, label="UInt32"), "outboundRootArray": SimTypePointer(SimStruct({"certData": SimStruct({"size": SimTypeInt(signed=False, label="UInt32"), "data": SimTypePointer(SimTypeChar(label="Byte"), offset=0)}, name="FWP_BYTE_BLOB", pack=False, align=None), "flags": SimTypeInt(signed=False, label="IKEEXT_CERT_FLAGS")}, name="IKEEXT_CERT_ROOT_CONFIG0", pack=False, align=None), offset=0)}, name="_Anonymous_e__Struct", pack=False, align=None), "outboundEnterpriseStoreConfig": SimTypePointer(SimStruct({"certData": SimStruct({"size": SimTypeInt(signed=False, label="UInt32"), "data": SimTypePointer(SimTypeChar(label="Byte"), offset=0)}, name="FWP_BYTE_BLOB", pack=False, align=None), "flags": SimTypeInt(signed=False, label="IKEEXT_CERT_FLAGS")}, name="IKEEXT_CERT_ROOT_CONFIG0", pack=False, align=None), offset=0), "outboundTrustedRootStoreConfig": SimTypePointer(SimStruct({"certData": SimStruct({"size": SimTypeInt(signed=False, label="UInt32"), "data": SimTypePointer(SimTypeChar(label="Byte"), offset=0)}, name="FWP_BYTE_BLOB", pack=False, align=None), "flags": SimTypeInt(signed=False, label="IKEEXT_CERT_FLAGS")}, name="IKEEXT_CERT_ROOT_CONFIG0", pack=False, align=None), offset=0)}, name="<anon>", label="None"), "flags": SimTypeInt(signed=False, label="IKEEXT_CERT_AUTH"), "localCertLocationUrl": SimStruct({"size": SimTypeInt(signed=False, label="UInt32"), "data": SimTypePointer(SimTypeChar(label="Byte"), offset=0)}, name="FWP_BYTE_BLOB", pack=False, align=None)}, name="IKEEXT_CERTIFICATE_AUTHENTICATION1", pack=False, align=None), "kerberosAuthentication": SimStruct({"flags": SimTypeInt(signed=False, label="IKEEXT_KERBEROS_AUTHENTICATION_FLAGS")}, name="IKEEXT_KERBEROS_AUTHENTICATION0", pack=False, align=None), "ntlmV2Authentication": SimStruct({"flags": SimTypeInt(signed=False, label="UInt32")}, name="IKEEXT_NTLM_V2_AUTHENTICATION0", pack=False, align=None), "sslAuthentication": SimStruct({"inboundConfigType": SimTypeInt(signed=False, label="IKEEXT_CERT_CONFIG_TYPE"), "Anonymous1": SimUnion({"Anonymous": SimStruct({"inboundRootArraySize": SimTypeInt(signed=False, label="UInt32"), "inboundRootArray": SimTypePointer(SimStruct({"certData": SimStruct({"size": SimTypeInt(signed=False, label="UInt32"), "data": SimTypePointer(SimTypeChar(label="Byte"), offset=0)}, name="FWP_BYTE_BLOB", pack=False, align=None), "flags": SimTypeInt(signed=False, label="IKEEXT_CERT_FLAGS")}, name="IKEEXT_CERT_ROOT_CONFIG0", pack=False, align=None), offset=0)}, name="_Anonymous_e__Struct", pack=False, align=None), "inboundEnterpriseStoreConfig": SimTypePointer(SimStruct({"certData": SimStruct({"size": SimTypeInt(signed=False, label="UInt32"), "data": SimTypePointer(SimTypeChar(label="Byte"), offset=0)}, name="FWP_BYTE_BLOB", pack=False, align=None), "flags": SimTypeInt(signed=False, label="IKEEXT_CERT_FLAGS")}, name="IKEEXT_CERT_ROOT_CONFIG0", pack=False, align=None), offset=0), "inboundTrustedRootStoreConfig": SimTypePointer(SimStruct({"certData": SimStruct({"size": SimTypeInt(signed=False, label="UInt32"), "data": SimTypePointer(SimTypeChar(label="Byte"), offset=0)}, name="FWP_BYTE_BLOB", pack=False, align=None), "flags": SimTypeInt(signed=False, label="IKEEXT_CERT_FLAGS")}, name="IKEEXT_CERT_ROOT_CONFIG0", pack=False, align=None), offset=0)}, name="<anon>", label="None"), "outboundConfigType": SimTypeInt(signed=False, label="IKEEXT_CERT_CONFIG_TYPE"), "Anonymous2": SimUnion({"Anonymous": SimStruct({"outboundRootArraySize": SimTypeInt(signed=False, label="UInt32"), "outboundRootArray": SimTypePointer(SimStruct({"certData": SimStruct({"size": SimTypeInt(signed=False, label="UInt32"), "data": SimTypePointer(SimTypeChar(label="Byte"), offset=0)}, name="FWP_BYTE_BLOB", pack=False, align=None), "flags": SimTypeInt(signed=False, label="IKEEXT_CERT_FLAGS")}, name="IKEEXT_CERT_ROOT_CONFIG0", pack=False, align=None), offset=0)}, name="_Anonymous_e__Struct", pack=False, align=None), "outboundEnterpriseStoreConfig": SimTypePointer(SimStruct({"certData": SimStruct({"size": SimTypeInt(signed=False, label="UInt32"), "data": SimTypePointer(SimTypeChar(label="Byte"), offset=0)}, name="FWP_BYTE_BLOB", pack=False, align=None), "flags": SimTypeInt(signed=False, label="IKEEXT_CERT_FLAGS")}, name="IKEEXT_CERT_ROOT_CONFIG0", pack=False, align=None), offset=0), "outboundTrustedRootStoreConfig": SimTypePointer(SimStruct({"certData": SimStruct({"size": SimTypeInt(signed=False, label="UInt32"), "data": SimTypePointer(SimTypeChar(label="Byte"), offset=0)}, name="FWP_BYTE_BLOB", pack=False, align=None), "flags": SimTypeInt(signed=False, label="IKEEXT_CERT_FLAGS")}, name="IKEEXT_CERT_ROOT_CONFIG0", pack=False, align=None), offset=0)}, name="<anon>", label="None"), "flags": SimTypeInt(signed=False, label="IKEEXT_CERT_AUTH"), "localCertLocationUrl": SimStruct({"size": SimTypeInt(signed=False, label="UInt32"), "data": SimTypePointer(SimTypeChar(label="Byte"), offset=0)}, name="FWP_BYTE_BLOB", pack=False, align=None)}, name="IKEEXT_CERTIFICATE_AUTHENTICATION1", pack=False, align=None), "cgaAuthentication": SimStruct({"keyContainerName": SimTypePointer(SimTypeChar(label="Char"), offset=0), "cspName": SimTypePointer(SimTypeChar(label="Char"), offset=0), "cspType": SimTypeInt(signed=False, label="UInt32"), "cgaModifier": SimStruct({"byteArray16": SimTypeFixedSizeArray(SimTypeChar(label="Byte"), 16)}, name="FWP_BYTE_ARRAY16", pack=False, align=None), "cgaCollisionCount": SimTypeChar(label="Byte")}, name="IKEEXT_IPV6_CGA_AUTHENTICATION0", pack=False, align=None), "eapAuthentication": SimStruct({"flags": SimTypeInt(signed=False, label="IKEEXT_EAP_AUTHENTICATION_FLAGS")}, name="IKEEXT_EAP_AUTHENTICATION0", pack=False, align=None)}, name="<anon>", label="None")}, name="IKEEXT_AUTHENTICATION_METHOD1", pack=False, align=None), offset=0), "initiatorImpersonationType": SimTypeInt(signed=False, label="IKEEXT_AUTHENTICATION_IMPERSONATION_TYPE")}, name="IKEEXT_EM_POLICY1", pack=False, align=None), offset=0)}, name="IPSEC_TRANSPORT_POLICY1", pack=False, align=None), offset=0), "ikeQmTunnelPolicy": SimTypePointer(SimStruct({"flags": SimTypeInt(signed=False, label="IPSEC_POLICY_FLAG"), "numIpsecProposals": SimTypeInt(signed=False, label="UInt32"), "ipsecProposals": SimTypePointer(SimStruct({"lifetime": SimStruct({"lifetimeSeconds": SimTypeInt(signed=False, label="UInt32"), "lifetimeKilobytes": SimTypeInt(signed=False, label="UInt32"), "lifetimePackets": SimTypeInt(signed=False, label="UInt32")}, name="IPSEC_SA_LIFETIME0", pack=False, align=None), "numSaTransforms": SimTypeInt(signed=False, label="UInt32"), "saTransforms": SimTypePointer(SimStruct({"ipsecTransformType": SimTypeInt(signed=False, label="IPSEC_TRANSFORM_TYPE"), "Anonymous": SimUnion({"ahTransform": SimTypePointer(SimStruct({"authTransformId": SimStruct({"authType": SimTypeInt(signed=False, label="IPSEC_AUTH_TYPE"), "authConfig": SimTypeChar(label="Byte")}, name="IPSEC_AUTH_TRANSFORM_ID0", pack=False, align=None), "cryptoModuleId": SimTypePointer(SimTypeBottom(label="Guid"), offset=0)}, name="IPSEC_AUTH_TRANSFORM0", pack=False, align=None), offset=0), "espAuthTransform": SimTypePointer(SimStruct({"authTransformId": SimStruct({"authType": SimTypeInt(signed=False, label="IPSEC_AUTH_TYPE"), "authConfig": SimTypeChar(label="Byte")}, name="IPSEC_AUTH_TRANSFORM_ID0", pack=False, align=None), "cryptoModuleId": SimTypePointer(SimTypeBottom(label="Guid"), offset=0)}, name="IPSEC_AUTH_TRANSFORM0", pack=False, align=None), offset=0), "espCipherTransform": SimTypePointer(SimStruct({"cipherTransformId": SimStruct({"cipherType": SimTypeInt(signed=False, label="IPSEC_CIPHER_TYPE"), "cipherConfig": SimTypeChar(label="Byte")}, name="IPSEC_CIPHER_TRANSFORM_ID0", pack=False, align=None), "cryptoModuleId": SimTypePointer(SimTypeBottom(label="Guid"), offset=0)}, name="IPSEC_CIPHER_TRANSFORM0", pack=False, align=None), offset=0), "espAuthAndCipherTransform": SimTypePointer(SimStruct({"authTransform": SimStruct({"authTransformId": SimStruct({"authType": SimTypeInt(signed=False, label="IPSEC_AUTH_TYPE"), "authConfig": SimTypeChar(label="Byte")}, name="IPSEC_AUTH_TRANSFORM_ID0", pack=False, align=None), "cryptoModuleId": SimTypePointer(SimTypeBottom(label="Guid"), offset=0)}, name="IPSEC_AUTH_TRANSFORM0", pack=False, align=None), "cipherTransform": SimStruct({"cipherTransformId": SimStruct({"cipherType": SimTypeInt(signed=False, label="IPSEC_CIPHER_TYPE"), "cipherConfig": SimTypeChar(label="Byte")}, name="IPSEC_CIPHER_TRANSFORM_ID0", pack=False, align=None), "cryptoModuleId": SimTypePointer(SimTypeBottom(label="Guid"), offset=0)}, name="IPSEC_CIPHER_TRANSFORM0", pack=False, align=None)}, name="IPSEC_AUTH_AND_CIPHER_TRANSFORM0", pack=False, align=None), offset=0), "espAuthFwTransform": SimTypePointer(SimStruct({"authTransformId": SimStruct({"authType": SimTypeInt(signed=False, label="IPSEC_AUTH_TYPE"), "authConfig": SimTypeChar(label="Byte")}, name="IPSEC_AUTH_TRANSFORM_ID0", pack=False, align=None), "cryptoModuleId": SimTypePointer(SimTypeBottom(label="Guid"), offset=0)}, name="IPSEC_AUTH_TRANSFORM0", pack=False, align=None), offset=0)}, name="<anon>", label="None")}, name="IPSEC_SA_TRANSFORM0", pack=False, align=None), offset=0), "pfsGroup": SimTypeInt(signed=False, label="IPSEC_PFS_GROUP")}, name="IPSEC_PROPOSAL0", pack=False, align=None), offset=0), "tunnelEndpoints": SimStruct({"ipVersion": SimTypeInt(signed=False, label="FWP_IP_VERSION"), "Anonymous1": SimUnion({"localV4Address": SimTypeInt(signed=False, label="UInt32"), "localV6Address": SimTypeFixedSizeArray(SimTypeChar(label="Byte"), 16)}, name="<anon>", label="None"), "Anonymous2": SimUnion({"remoteV4Address": SimTypeInt(signed=False, label="UInt32"), "remoteV6Address": SimTypeFixedSizeArray(SimTypeChar(label="Byte"), 16)}, name="<anon>", label="None"), "localIfLuid": SimTypeLongLong(signed=False, label="UInt64")}, name="IPSEC_TUNNEL_ENDPOINTS1", pack=False, align=None), "saIdleTimeout": SimStruct({"idleTimeoutSeconds": SimTypeInt(signed=False, label="UInt32"), "idleTimeoutSecondsFailOver": SimTypeInt(signed=False, label="UInt32")}, name="IPSEC_SA_IDLE_TIMEOUT0", pack=False, align=None), "emPolicy": SimTypePointer(SimStruct({"numAuthenticationMethods": SimTypeInt(signed=False, label="UInt32"), "authenticationMethods": SimTypePointer(SimStruct({"authenticationMethodType": SimTypeInt(signed=False, label="IKEEXT_AUTHENTICATION_METHOD_TYPE"), "Anonymous": SimUnion({"presharedKeyAuthentication": SimStruct({"presharedKey": SimStruct({"size": SimTypeInt(signed=False, label="UInt32"), "data": SimTypePointer(SimTypeChar(label="Byte"), offset=0)}, name="FWP_BYTE_BLOB", pack=False, align=None), "flags": SimTypeInt(signed=False, label="IKEEXT_PRESHARED_KEY_AUTHENTICATION_FLAGS")}, name="IKEEXT_PRESHARED_KEY_AUTHENTICATION1", pack=False, align=None), "certificateAuthentication": SimStruct({"inboundConfigType": SimTypeInt(signed=False, label="IKEEXT_CERT_CONFIG_TYPE"), "Anonymous1": SimUnion({"Anonymous": SimStruct({"inboundRootArraySize": SimTypeInt(signed=False, label="UInt32"), "inboundRootArray": SimTypePointer(SimStruct({"certData": SimStruct({"size": SimTypeInt(signed=False, label="UInt32"), "data": SimTypePointer(SimTypeChar(label="Byte"), offset=0)}, name="FWP_BYTE_BLOB", pack=False, align=None), "flags": SimTypeInt(signed=False, label="IKEEXT_CERT_FLAGS")}, name="IKEEXT_CERT_ROOT_CONFIG0", pack=False, align=None), offset=0)}, name="_Anonymous_e__Struct", pack=False, align=None), "inboundEnterpriseStoreConfig": SimTypePointer(SimStruct({"certData": SimStruct({"size": SimTypeInt(signed=False, label="UInt32"), "data": SimTypePointer(SimTypeChar(label="Byte"), offset=0)}, name="FWP_BYTE_BLOB", pack=False, align=None), "flags": SimTypeInt(signed=False, label="IKEEXT_CERT_FLAGS")}, name="IKEEXT_CERT_ROOT_CONFIG0", pack=False, align=None), offset=0), "inboundTrustedRootStoreConfig": SimTypePointer(SimStruct({"certData": SimStruct({"size": SimTypeInt(signed=False, label="UInt32"), "data": SimTypePointer(SimTypeChar(label="Byte"), offset=0)}, name="FWP_BYTE_BLOB", pack=False, align=None), "flags": SimTypeInt(signed=False, label="IKEEXT_CERT_FLAGS")}, name="IKEEXT_CERT_ROOT_CONFIG0", pack=False, align=None), offset=0)}, name="<anon>", label="None"), "outboundConfigType": SimTypeInt(signed=False, label="IKEEXT_CERT_CONFIG_TYPE"), "Anonymous2": SimUnion({"Anonymous": SimStruct({"outboundRootArraySize": SimTypeInt(signed=False, label="UInt32"), "outboundRootArray": SimTypePointer(SimStruct({"certData": SimStruct({"size": SimTypeInt(signed=False, label="UInt32"), "data": SimTypePointer(SimTypeChar(label="Byte"), offset=0)}, name="FWP_BYTE_BLOB", pack=False, align=None), "flags": SimTypeInt(signed=False, label="IKEEXT_CERT_FLAGS")}, name="IKEEXT_CERT_ROOT_CONFIG0", pack=False, align=None), offset=0)}, name="_Anonymous_e__Struct", pack=False, align=None), "outboundEnterpriseStoreConfig": SimTypePointer(SimStruct({"certData": SimStruct({"size": SimTypeInt(signed=False, label="UInt32"), "data": SimTypePointer(SimTypeChar(label="Byte"), offset=0)}, name="FWP_BYTE_BLOB", pack=False, align=None), "flags": SimTypeInt(signed=False, label="IKEEXT_CERT_FLAGS")}, name="IKEEXT_CERT_ROOT_CONFIG0", pack=False, align=None), offset=0), "outboundTrustedRootStoreConfig": SimTypePointer(SimStruct({"certData": SimStruct({"size": SimTypeInt(signed=False, label="UInt32"), "data": SimTypePointer(SimTypeChar(label="Byte"), offset=0)}, name="FWP_BYTE_BLOB", pack=False, align=None), "flags": SimTypeInt(signed=False, label="IKEEXT_CERT_FLAGS")}, name="IKEEXT_CERT_ROOT_CONFIG0", pack=False, align=None), offset=0)}, name="<anon>", label="None"), "flags": SimTypeInt(signed=False, label="IKEEXT_CERT_AUTH"), "localCertLocationUrl": SimStruct({"size": SimTypeInt(signed=False, label="UInt32"), "data": SimTypePointer(SimTypeChar(label="Byte"), offset=0)}, name="FWP_BYTE_BLOB", pack=False, align=None)}, name="IKEEXT_CERTIFICATE_AUTHENTICATION1", pack=False, align=None), "kerberosAuthentication": SimStruct({"flags": SimTypeInt(signed=False, label="IKEEXT_KERBEROS_AUTHENTICATION_FLAGS")}, name="IKEEXT_KERBEROS_AUTHENTICATION0", pack=False, align=None), "ntlmV2Authentication": SimStruct({"flags": SimTypeInt(signed=False, label="UInt32")}, name="IKEEXT_NTLM_V2_AUTHENTICATION0", pack=False, align=None), "sslAuthentication": SimStruct({"inboundConfigType": SimTypeInt(signed=False, label="IKEEXT_CERT_CONFIG_TYPE"), "Anonymous1": SimUnion({"Anonymous": SimStruct({"inboundRootArraySize": SimTypeInt(signed=False, label="UInt32"), "inboundRootArray": SimTypePointer(SimStruct({"certData": SimStruct({"size": SimTypeInt(signed=False, label="UInt32"), "data": SimTypePointer(SimTypeChar(label="Byte"), offset=0)}, name="FWP_BYTE_BLOB", pack=False, align=None), "flags": SimTypeInt(signed=False, label="IKEEXT_CERT_FLAGS")}, name="IKEEXT_CERT_ROOT_CONFIG0", pack=False, align=None), offset=0)}, name="_Anonymous_e__Struct", pack=False, align=None), "inboundEnterpriseStoreConfig": SimTypePointer(SimStruct({"certData": SimStruct({"size": SimTypeInt(signed=False, label="UInt32"), "data": SimTypePointer(SimTypeChar(label="Byte"), offset=0)}, name="FWP_BYTE_BLOB", pack=False, align=None), "flags": SimTypeInt(signed=False, label="IKEEXT_CERT_FLAGS")}, name="IKEEXT_CERT_ROOT_CONFIG0", pack=False, align=None), offset=0), "inboundTrustedRootStoreConfig": SimTypePointer(SimStruct({"certData": SimStruct({"size": SimTypeInt(signed=False, label="UInt32"), "data": SimTypePointer(SimTypeChar(label="Byte"), offset=0)}, name="FWP_BYTE_BLOB", pack=False, align=None), "flags": SimTypeInt(signed=False, label="IKEEXT_CERT_FLAGS")}, name="IKEEXT_CERT_ROOT_CONFIG0", pack=False, align=None), offset=0)}, name="<anon>", label="None"), "outboundConfigType": SimTypeInt(signed=False, label="IKEEXT_CERT_CONFIG_TYPE"), "Anonymous2": SimUnion({"Anonymous": SimStruct({"outboundRootArraySize": SimTypeInt(signed=False, label="UInt32"), "outboundRootArray": SimTypePointer(SimStruct({"certData": SimStruct({"size": SimTypeInt(signed=False, label="UInt32"), "data": SimTypePointer(SimTypeChar(label="Byte"), offset=0)}, name="FWP_BYTE_BLOB", pack=False, align=None), "flags": SimTypeInt(signed=False, label="IKEEXT_CERT_FLAGS")}, name="IKEEXT_CERT_ROOT_CONFIG0", pack=False, align=None), offset=0)}, name="_Anonymous_e__Struct", pack=False, align=None), "outboundEnterpriseStoreConfig": SimTypePointer(SimStruct({"certData": SimStruct({"size": SimTypeInt(signed=False, label="UInt32"), "data": SimTypePointer(SimTypeChar(label="Byte"), offset=0)}, name="FWP_BYTE_BLOB", pack=False, align=None), "flags": SimTypeInt(signed=False, label="IKEEXT_CERT_FLAGS")}, name="IKEEXT_CERT_ROOT_CONFIG0", pack=False, align=None), offset=0), "outboundTrustedRootStoreConfig": SimTypePointer(SimStruct({"certData": SimStruct({"size": SimTypeInt(signed=False, label="UInt32"), "data": SimTypePointer(SimTypeChar(label="Byte"), offset=0)}, name="FWP_BYTE_BLOB", pack=False, align=None), "flags": SimTypeInt(signed=False, label="IKEEXT_CERT_FLAGS")}, name="IKEEXT_CERT_ROOT_CONFIG0", pack=False, align=None), offset=0)}, name="<anon>", label="None"), "flags": SimTypeInt(signed=False, label="IKEEXT_CERT_AUTH"), "localCertLocationUrl": SimStruct({"size": SimTypeInt(signed=False, label="UInt32"), "data": SimTypePointer(SimTypeChar(label="Byte"), offset=0)}, name="FWP_BYTE_BLOB", pack=False, align=None)}, name="IKEEXT_CERTIFICATE_AUTHENTICATION1", pack=False, align=None), "cgaAuthentication": SimStruct({"keyContainerName": SimTypePointer(SimTypeChar(label="Char"), offset=0), "cspName": SimTypePointer(SimTypeChar(label="Char"), offset=0), "cspType": SimTypeInt(signed=False, label="UInt32"), "cgaModifier": SimStruct({"byteArray16": SimTypeFixedSizeArray(SimTypeChar(label="Byte"), 16)}, name="FWP_BYTE_ARRAY16", pack=False, align=None), "cgaCollisionCount": SimTypeChar(label="Byte")}, name="IKEEXT_IPV6_CGA_AUTHENTICATION0", pack=False, align=None), "eapAuthentication": SimStruct({"flags": SimTypeInt(signed=False, label="IKEEXT_EAP_AUTHENTICATION_FLAGS")}, name="IKEEXT_EAP_AUTHENTICATION0", pack=False, align=None)}, name="<anon>", label="None")}, name="IKEEXT_AUTHENTICATION_METHOD1", pack=False, align=None), offset=0), "initiatorImpersonationType": SimTypeInt(signed=False, label="IKEEXT_AUTHENTICATION_IMPERSONATION_TYPE")}, name="IKEEXT_EM_POLICY1", pack=False, align=None), offset=0)}, name="IPSEC_TUNNEL_POLICY1", pack=False, align=None), offset=0), "authipQmTransportPolicy": SimTypePointer(SimStruct({"numIpsecProposals": SimTypeInt(signed=False, label="UInt32"), "ipsecProposals": SimTypePointer(SimStruct({"lifetime": SimStruct({"lifetimeSeconds": SimTypeInt(signed=False, label="UInt32"), "lifetimeKilobytes": SimTypeInt(signed=False, label="UInt32"), "lifetimePackets": SimTypeInt(signed=False, label="UInt32")}, name="IPSEC_SA_LIFETIME0", pack=False, align=None), "numSaTransforms": SimTypeInt(signed=False, label="UInt32"), "saTransforms": SimTypePointer(SimStruct({"ipsecTransformType": SimTypeInt(signed=False, label="IPSEC_TRANSFORM_TYPE"), "Anonymous": SimUnion({"ahTransform": SimTypePointer(SimStruct({"authTransformId": SimStruct({"authType": SimTypeInt(signed=False, label="IPSEC_AUTH_TYPE"), "authConfig": SimTypeChar(label="Byte")}, name="IPSEC_AUTH_TRANSFORM_ID0", pack=False, align=None), "cryptoModuleId": SimTypePointer(SimTypeBottom(label="Guid"), offset=0)}, name="IPSEC_AUTH_TRANSFORM0", pack=False, align=None), offset=0), "espAuthTransform": SimTypePointer(SimStruct({"authTransformId": SimStruct({"authType": SimTypeInt(signed=False, label="IPSEC_AUTH_TYPE"), "authConfig": SimTypeChar(label="Byte")}, name="IPSEC_AUTH_TRANSFORM_ID0", pack=False, align=None), "cryptoModuleId": SimTypePointer(SimTypeBottom(label="Guid"), offset=0)}, name="IPSEC_AUTH_TRANSFORM0", pack=False, align=None), offset=0), "espCipherTransform": SimTypePointer(SimStruct({"cipherTransformId": SimStruct({"cipherType": SimTypeInt(signed=False, label="IPSEC_CIPHER_TYPE"), "cipherConfig": SimTypeChar(label="Byte")}, name="IPSEC_CIPHER_TRANSFORM_ID0", pack=False, align=None), "cryptoModuleId": SimTypePointer(SimTypeBottom(label="Guid"), offset=0)}, name="IPSEC_CIPHER_TRANSFORM0", pack=False, align=None), offset=0), "espAuthAndCipherTransform": SimTypePointer(SimStruct({"authTransform": SimStruct({"authTransformId": SimStruct({"authType": SimTypeInt(signed=False, label="IPSEC_AUTH_TYPE"), "authConfig": SimTypeChar(label="Byte")}, name="IPSEC_AUTH_TRANSFORM_ID0", pack=False, align=None), "cryptoModuleId": SimTypePointer(SimTypeBottom(label="Guid"), offset=0)}, name="IPSEC_AUTH_TRANSFORM0", pack=False, align=None), "cipherTransform": SimStruct({"cipherTransformId": SimStruct({"cipherType": SimTypeInt(signed=False, label="IPSEC_CIPHER_TYPE"), "cipherConfig": SimTypeChar(label="Byte")}, name="IPSEC_CIPHER_TRANSFORM_ID0", pack=False, align=None), "cryptoModuleId": SimTypePointer(SimTypeBottom(label="Guid"), offset=0)}, name="IPSEC_CIPHER_TRANSFORM0", pack=False, align=None)}, name="IPSEC_AUTH_AND_CIPHER_TRANSFORM0", pack=False, align=None), offset=0), "espAuthFwTransform": SimTypePointer(SimStruct({"authTransformId": SimStruct({"authType": SimTypeInt(signed=False, label="IPSEC_AUTH_TYPE"), "authConfig": SimTypeChar(label="Byte")}, name="IPSEC_AUTH_TRANSFORM_ID0", pack=False, align=None), "cryptoModuleId": SimTypePointer(SimTypeBottom(label="Guid"), offset=0)}, name="IPSEC_AUTH_TRANSFORM0", pack=False, align=None), offset=0)}, name="<anon>", label="None")}, name="IPSEC_SA_TRANSFORM0", pack=False, align=None), offset=0), "pfsGroup": SimTypeInt(signed=False, label="IPSEC_PFS_GROUP")}, name="IPSEC_PROPOSAL0", pack=False, align=None), offset=0), "flags": SimTypeInt(signed=False, label="IPSEC_POLICY_FLAG"), "ndAllowClearTimeoutSeconds": SimTypeInt(signed=False, label="UInt32"), "saIdleTimeout": SimStruct({"idleTimeoutSeconds": SimTypeInt(signed=False, label="UInt32"), "idleTimeoutSecondsFailOver": SimTypeInt(signed=False, label="UInt32")}, name="IPSEC_SA_IDLE_TIMEOUT0", pack=False, align=None), "emPolicy": SimTypePointer(SimStruct({"numAuthenticationMethods": SimTypeInt(signed=False, label="UInt32"), "authenticationMethods": SimTypePointer(SimStruct({"authenticationMethodType": SimTypeInt(signed=False, label="IKEEXT_AUTHENTICATION_METHOD_TYPE"), "Anonymous": SimUnion({"presharedKeyAuthentication": SimStruct({"presharedKey": SimStruct({"size": SimTypeInt(signed=False, label="UInt32"), "data": SimTypePointer(SimTypeChar(label="Byte"), offset=0)}, name="FWP_BYTE_BLOB", pack=False, align=None), "flags": SimTypeInt(signed=False, label="IKEEXT_PRESHARED_KEY_AUTHENTICATION_FLAGS")}, name="IKEEXT_PRESHARED_KEY_AUTHENTICATION1", pack=False, align=None), "certificateAuthentication": SimStruct({"inboundConfigType": SimTypeInt(signed=False, label="IKEEXT_CERT_CONFIG_TYPE"), "Anonymous1": SimUnion({"Anonymous": SimStruct({"inboundRootArraySize": SimTypeInt(signed=False, label="UInt32"), "inboundRootArray": SimTypePointer(SimStruct({"certData": SimStruct({"size": SimTypeInt(signed=False, label="UInt32"), "data": SimTypePointer(SimTypeChar(label="Byte"), offset=0)}, name="FWP_BYTE_BLOB", pack=False, align=None), "flags": SimTypeInt(signed=False, label="IKEEXT_CERT_FLAGS")}, name="IKEEXT_CERT_ROOT_CONFIG0", pack=False, align=None), offset=0)}, name="_Anonymous_e__Struct", pack=False, align=None), "inboundEnterpriseStoreConfig": SimTypePointer(SimStruct({"certData": SimStruct({"size": SimTypeInt(signed=False, label="UInt32"), "data": SimTypePointer(SimTypeChar(label="Byte"), offset=0)}, name="FWP_BYTE_BLOB", pack=False, align=None), "flags": SimTypeInt(signed=False, label="IKEEXT_CERT_FLAGS")}, name="IKEEXT_CERT_ROOT_CONFIG0", pack=False, align=None), offset=0), "inboundTrustedRootStoreConfig": SimTypePointer(SimStruct({"certData": SimStruct({"size": SimTypeInt(signed=False, label="UInt32"), "data": SimTypePointer(SimTypeChar(label="Byte"), offset=0)}, name="FWP_BYTE_BLOB", pack=False, align=None), "flags": SimTypeInt(signed=False, label="IKEEXT_CERT_FLAGS")}, name="IKEEXT_CERT_ROOT_CONFIG0", pack=False, align=None), offset=0)}, name="<anon>", label="None"), "outboundConfigType": SimTypeInt(signed=False, label="IKEEXT_CERT_CONFIG_TYPE"), "Anonymous2": SimUnion({"Anonymous": SimStruct({"outboundRootArraySize": SimTypeInt(signed=False, label="UInt32"), "outboundRootArray": SimTypePointer(SimStruct({"certData": SimStruct({"size": SimTypeInt(signed=False, label="UInt32"), "data": SimTypePointer(SimTypeChar(label="Byte"), offset=0)}, name="FWP_BYTE_BLOB", pack=False, align=None), "flags": SimTypeInt(signed=False, label="IKEEXT_CERT_FLAGS")}, name="IKEEXT_CERT_ROOT_CONFIG0", pack=False, align=None), offset=0)}, name="_Anonymous_e__Struct", pack=False, align=None), "outboundEnterpriseStoreConfig": SimTypePointer(SimStruct({"certData": SimStruct({"size": SimTypeInt(signed=False, label="UInt32"), "data": SimTypePointer(SimTypeChar(label="Byte"), offset=0)}, name="FWP_BYTE_BLOB", pack=False, align=None), "flags": SimTypeInt(signed=False, label="IKEEXT_CERT_FLAGS")}, name="IKEEXT_CERT_ROOT_CONFIG0", pack=False, align=None), offset=0), "outboundTrustedRootStoreConfig": SimTypePointer(SimStruct({"certData": SimStruct({"size": SimTypeInt(signed=False, label="UInt32"), "data": SimTypePointer(SimTypeChar(label="Byte"), offset=0)}, name="FWP_BYTE_BLOB", pack=False, align=None), "flags": SimTypeInt(signed=False, label="IKEEXT_CERT_FLAGS")}, name="IKEEXT_CERT_ROOT_CONFIG0", pack=False, align=None), offset=0)}, name="<anon>", label="None"), "flags": SimTypeInt(signed=False, label="IKEEXT_CERT_AUTH"), "localCertLocationUrl": SimStruct({"size": SimTypeInt(signed=False, label="UInt32"), "data": SimTypePointer(SimTypeChar(label="Byte"), offset=0)}, name="FWP_BYTE_BLOB", pack=False, align=None)}, name="IKEEXT_CERTIFICATE_AUTHENTICATION1", pack=False, align=None), "kerberosAuthentication": SimStruct({"flags": SimTypeInt(signed=False, label="IKEEXT_KERBEROS_AUTHENTICATION_FLAGS")}, name="IKEEXT_KERBEROS_AUTHENTICATION0", pack=False, align=None), "ntlmV2Authentication": SimStruct({"flags": SimTypeInt(signed=False, label="UInt32")}, name="IKEEXT_NTLM_V2_AUTHENTICATION0", pack=False, align=None), "sslAuthentication": SimStruct({"inboundConfigType": SimTypeInt(signed=False, label="IKEEXT_CERT_CONFIG_TYPE"), "Anonymous1": SimUnion({"Anonymous": SimStruct({"inboundRootArraySize": SimTypeInt(signed=False, label="UInt32"), "inboundRootArray": SimTypePointer(SimStruct({"certData": SimStruct({"size": SimTypeInt(signed=False, label="UInt32"), "data": SimTypePointer(SimTypeChar(label="Byte"), offset=0)}, name="FWP_BYTE_BLOB", pack=False, align=None), "flags": SimTypeInt(signed=False, label="IKEEXT_CERT_FLAGS")}, name="IKEEXT_CERT_ROOT_CONFIG0", pack=False, align=None), offset=0)}, name="_Anonymous_e__Struct", pack=False, align=None), "inboundEnterpriseStoreConfig": SimTypePointer(SimStruct({"certData": SimStruct({"size": SimTypeInt(signed=False, label="UInt32"), "data": SimTypePointer(SimTypeChar(label="Byte"), offset=0)}, name="FWP_BYTE_BLOB", pack=False, align=None), "flags": SimTypeInt(signed=False, label="IKEEXT_CERT_FLAGS")}, name="IKEEXT_CERT_ROOT_CONFIG0", pack=False, align=None), offset=0), "inboundTrustedRootStoreConfig": SimTypePointer(SimStruct({"certData": SimStruct({"size": SimTypeInt(signed=False, label="UInt32"), "data": SimTypePointer(SimTypeChar(label="Byte"), offset=0)}, name="FWP_BYTE_BLOB", pack=False, align=None), "flags": SimTypeInt(signed=False, label="IKEEXT_CERT_FLAGS")}, name="IKEEXT_CERT_ROOT_CONFIG0", pack=False, align=None), offset=0)}, name="<anon>", label="None"), "outboundConfigType": SimTypeInt(signed=False, label="IKEEXT_CERT_CONFIG_TYPE"), "Anonymous2": SimUnion({"Anonymous": SimStruct({"outboundRootArraySize": SimTypeInt(signed=False, label="UInt32"), "outboundRootArray": SimTypePointer(SimStruct({"certData": SimStruct({"size": SimTypeInt(signed=False, label="UInt32"), "data": SimTypePointer(SimTypeChar(label="Byte"), offset=0)}, name="FWP_BYTE_BLOB", pack=False, align=None), "flags": SimTypeInt(signed=False, label="IKEEXT_CERT_FLAGS")}, name="IKEEXT_CERT_ROOT_CONFIG0", pack=False, align=None), offset=0)}, name="_Anonymous_e__Struct", pack=False, align=None), "outboundEnterpriseStoreConfig": SimTypePointer(SimStruct({"certData": SimStruct({"size": SimTypeInt(signed=False, label="UInt32"), "data": SimTypePointer(SimTypeChar(label="Byte"), offset=0)}, name="FWP_BYTE_BLOB", pack=False, align=None), "flags": SimTypeInt(signed=False, label="IKEEXT_CERT_FLAGS")}, name="IKEEXT_CERT_ROOT_CONFIG0", pack=False, align=None), offset=0), "outboundTrustedRootStoreConfig": SimTypePointer(SimStruct({"certData": SimStruct({"size": SimTypeInt(signed=False, label="UInt32"), "data": SimTypePointer(SimTypeChar(label="Byte"), offset=0)}, name="FWP_BYTE_BLOB", pack=False, align=None), "flags": SimTypeInt(signed=False, label="IKEEXT_CERT_FLAGS")}, name="IKEEXT_CERT_ROOT_CONFIG0", pack=False, align=None), offset=0)}, name="<anon>", label="None"), "flags": SimTypeInt(signed=False, label="IKEEXT_CERT_AUTH"), "localCertLocationUrl": SimStruct({"size": SimTypeInt(signed=False, label="UInt32"), "data": SimTypePointer(SimTypeChar(label="Byte"), offset=0)}, name="FWP_BYTE_BLOB", pack=False, align=None)}, name="IKEEXT_CERTIFICATE_AUTHENTICATION1", pack=False, align=None), "cgaAuthentication": SimStruct({"keyContainerName": SimTypePointer(SimTypeChar(label="Char"), offset=0), "cspName": SimTypePointer(SimTypeChar(label="Char"), offset=0), "cspType": SimTypeInt(signed=False, label="UInt32"), "cgaModifier": SimStruct({"byteArray16": SimTypeFixedSizeArray(SimTypeChar(label="Byte"), 16)}, name="FWP_BYTE_ARRAY16", pack=False, align=None), "cgaCollisionCount": SimTypeChar(label="Byte")}, name="IKEEXT_IPV6_CGA_AUTHENTICATION0", pack=False, align=None), "eapAuthentication": SimStruct({"flags": SimTypeInt(signed=False, label="IKEEXT_EAP_AUTHENTICATION_FLAGS")}, name="IKEEXT_EAP_AUTHENTICATION0", pack=False, align=None)}, name="<anon>", label="None")}, name="IKEEXT_AUTHENTICATION_METHOD1", pack=False, align=None), offset=0), "initiatorImpersonationType": SimTypeInt(signed=False, label="IKEEXT_AUTHENTICATION_IMPERSONATION_TYPE")}, name="IKEEXT_EM_POLICY1", pack=False, align=None), offset=0)}, name="IPSEC_TRANSPORT_POLICY1", pack=False, align=None), offset=0), "authipQmTunnelPolicy": SimTypePointer(SimStruct({"flags": SimTypeInt(signed=False, label="IPSEC_POLICY_FLAG"), "numIpsecProposals": SimTypeInt(signed=False, label="UInt32"), "ipsecProposals": SimTypePointer(SimStruct({"lifetime": SimStruct({"lifetimeSeconds": SimTypeInt(signed=False, label="UInt32"), "lifetimeKilobytes": SimTypeInt(signed=False, label="UInt32"), "lifetimePackets": SimTypeInt(signed=False, label="UInt32")}, name="IPSEC_SA_LIFETIME0", pack=False, align=None), "numSaTransforms": SimTypeInt(signed=False, label="UInt32"), "saTransforms": SimTypePointer(SimStruct({"ipsecTransformType": SimTypeInt(signed=False, label="IPSEC_TRANSFORM_TYPE"), "Anonymous": SimUnion({"ahTransform": SimTypePointer(SimStruct({"authTransformId": SimStruct({"authType": SimTypeInt(signed=False, label="IPSEC_AUTH_TYPE"), "authConfig": SimTypeChar(label="Byte")}, name="IPSEC_AUTH_TRANSFORM_ID0", pack=False, align=None), "cryptoModuleId": SimTypePointer(SimTypeBottom(label="Guid"), offset=0)}, name="IPSEC_AUTH_TRANSFORM0", pack=False, align=None), offset=0), "espAuthTransform": SimTypePointer(SimStruct({"authTransformId": SimStruct({"authType": SimTypeInt(signed=False, label="IPSEC_AUTH_TYPE"), "authConfig": SimTypeChar(label="Byte")}, name="IPSEC_AUTH_TRANSFORM_ID0", pack=False, align=None), "cryptoModuleId": SimTypePointer(SimTypeBottom(label="Guid"), offset=0)}, name="IPSEC_AUTH_TRANSFORM0", pack=False, align=None), offset=0), "espCipherTransform": SimTypePointer(SimStruct({"cipherTransformId": SimStruct({"cipherType": SimTypeInt(signed=False, label="IPSEC_CIPHER_TYPE"), "cipherConfig": SimTypeChar(label="Byte")}, name="IPSEC_CIPHER_TRANSFORM_ID0", pack=False, align=None), "cryptoModuleId": SimTypePointer(SimTypeBottom(label="Guid"), offset=0)}, name="IPSEC_CIPHER_TRANSFORM0", pack=False, align=None), offset=0), "espAuthAndCipherTransform": SimTypePointer(SimStruct({"authTransform": SimStruct({"authTransformId": SimStruct({"authType": SimTypeInt(signed=False, label="IPSEC_AUTH_TYPE"), "authConfig": SimTypeChar(label="Byte")}, name="IPSEC_AUTH_TRANSFORM_ID0", pack=False, align=None), "cryptoModuleId": SimTypePointer(SimTypeBottom(label="Guid"), offset=0)}, name="IPSEC_AUTH_TRANSFORM0", pack=False, align=None), "cipherTransform": SimStruct({"cipherTransformId": SimStruct({"cipherType": SimTypeInt(signed=False, label="IPSEC_CIPHER_TYPE"), "cipherConfig": SimTypeChar(label="Byte")}, name="IPSEC_CIPHER_TRANSFORM_ID0", pack=False, align=None), "cryptoModuleId": SimTypePointer(SimTypeBottom(label="Guid"), offset=0)}, name="IPSEC_CIPHER_TRANSFORM0", pack=False, align=None)}, name="IPSEC_AUTH_AND_CIPHER_TRANSFORM0", pack=False, align=None), offset=0), "espAuthFwTransform": SimTypePointer(SimStruct({"authTransformId": SimStruct({"authType": SimTypeInt(signed=False, label="IPSEC_AUTH_TYPE"), "authConfig": SimTypeChar(label="Byte")}, name="IPSEC_AUTH_TRANSFORM_ID0", pack=False, align=None), "cryptoModuleId": SimTypePointer(SimTypeBottom(label="Guid"), offset=0)}, name="IPSEC_AUTH_TRANSFORM0", pack=False, align=None), offset=0)}, name="<anon>", label="None")}, name="IPSEC_SA_TRANSFORM0", pack=False, align=None), offset=0), "pfsGroup": SimTypeInt(signed=False, label="IPSEC_PFS_GROUP")}, name="IPSEC_PROPOSAL0", pack=False, align=None), offset=0), "tunnelEndpoints": SimStruct({"ipVersion": SimTypeInt(signed=False, label="FWP_IP_VERSION"), "Anonymous1": SimUnion({"localV4Address": SimTypeInt(signed=False, label="UInt32"), "localV6Address": SimTypeFixedSizeArray(SimTypeChar(label="Byte"), 16)}, name="<anon>", label="None"), "Anonymous2": SimUnion({"remoteV4Address": SimTypeInt(signed=False, label="UInt32"), "remoteV6Address": SimTypeFixedSizeArray(SimTypeChar(label="Byte"), 16)}, name="<anon>", label="None"), "localIfLuid": SimTypeLongLong(signed=False, label="UInt64")}, name="IPSEC_TUNNEL_ENDPOINTS1", pack=False, align=None), "saIdleTimeout": SimStruct({"idleTimeoutSeconds": SimTypeInt(signed=False, label="UInt32"), "idleTimeoutSecondsFailOver": SimTypeInt(signed=False, label="UInt32")}, name="IPSEC_SA_IDLE_TIMEOUT0", pack=False, align=None), "emPolicy": SimTypePointer(SimStruct({"numAuthenticationMethods": SimTypeInt(signed=False, label="UInt32"), "authenticationMethods": SimTypePointer(SimStruct({"authenticationMethodType": SimTypeInt(signed=False, label="IKEEXT_AUTHENTICATION_METHOD_TYPE"), "Anonymous": SimUnion({"presharedKeyAuthentication": SimStruct({"presharedKey": SimStruct({"size": SimTypeInt(signed=False, label="UInt32"), "data": SimTypePointer(SimTypeChar(label="Byte"), offset=0)}, name="FWP_BYTE_BLOB", pack=False, align=None), "flags": SimTypeInt(signed=False, label="IKEEXT_PRESHARED_KEY_AUTHENTICATION_FLAGS")}, name="IKEEXT_PRESHARED_KEY_AUTHENTICATION1", pack=False, align=None), "certificateAuthentication": SimStruct({"inboundConfigType": SimTypeInt(signed=False, label="IKEEXT_CERT_CONFIG_TYPE"), "Anonymous1": SimUnion({"Anonymous": SimStruct({"inboundRootArraySize": SimTypeInt(signed=False, label="UInt32"), "inboundRootArray": SimTypePointer(SimStruct({"certData": SimStruct({"size": SimTypeInt(signed=False, label="UInt32"), "data": SimTypePointer(SimTypeChar(label="Byte"), offset=0)}, name="FWP_BYTE_BLOB", pack=False, align=None), "flags": SimTypeInt(signed=False, label="IKEEXT_CERT_FLAGS")}, name="IKEEXT_CERT_ROOT_CONFIG0", pack=False, align=None), offset=0)}, name="_Anonymous_e__Struct", pack=False, align=None), "inboundEnterpriseStoreConfig": SimTypePointer(SimStruct({"certData": SimStruct({"size": SimTypeInt(signed=False, label="UInt32"), "data": SimTypePointer(SimTypeChar(label="Byte"), offset=0)}, name="FWP_BYTE_BLOB", pack=False, align=None), "flags": SimTypeInt(signed=False, label="IKEEXT_CERT_FLAGS")}, name="IKEEXT_CERT_ROOT_CONFIG0", pack=False, align=None), offset=0), "inboundTrustedRootStoreConfig": SimTypePointer(SimStruct({"certData": SimStruct({"size": SimTypeInt(signed=False, label="UInt32"), "data": SimTypePointer(SimTypeChar(label="Byte"), offset=0)}, name="FWP_BYTE_BLOB", pack=False, align=None), "flags": SimTypeInt(signed=False, label="IKEEXT_CERT_FLAGS")}, name="IKEEXT_CERT_ROOT_CONFIG0", pack=False, align=None), offset=0)}, name="<anon>", label="None"), "outboundConfigType": SimTypeInt(signed=False, label="IKEEXT_CERT_CONFIG_TYPE"), "Anonymous2": SimUnion({"Anonymous": SimStruct({"outboundRootArraySize": SimTypeInt(signed=False, label="UInt32"), "outboundRootArray": SimTypePointer(SimStruct({"certData": SimStruct({"size": SimTypeInt(signed=False, label="UInt32"), "data": SimTypePointer(SimTypeChar(label="Byte"), offset=0)}, name="FWP_BYTE_BLOB", pack=False, align=None), "flags": SimTypeInt(signed=False, label="IKEEXT_CERT_FLAGS")}, name="IKEEXT_CERT_ROOT_CONFIG0", pack=False, align=None), offset=0)}, name="_Anonymous_e__Struct", pack=False, align=None), "outboundEnterpriseStoreConfig": SimTypePointer(SimStruct({"certData": SimStruct({"size": SimTypeInt(signed=False, label="UInt32"), "data": SimTypePointer(SimTypeChar(label="Byte"), offset=0)}, name="FWP_BYTE_BLOB", pack=False, align=None), "flags": SimTypeInt(signed=False, label="IKEEXT_CERT_FLAGS")}, name="IKEEXT_CERT_ROOT_CONFIG0", pack=False, align=None), offset=0), "outboundTrustedRootStoreConfig": SimTypePointer(SimStruct({"certData": SimStruct({"size": SimTypeInt(signed=False, label="UInt32"), "data": SimTypePointer(SimTypeChar(label="Byte"), offset=0)}, name="FWP_BYTE_BLOB", pack=False, align=None), "flags": SimTypeInt(signed=False, label="IKEEXT_CERT_FLAGS")}, name="IKEEXT_CERT_ROOT_CONFIG0", pack=False, align=None), offset=0)}, name="<anon>", label="None"), "flags": SimTypeInt(signed=False, label="IKEEXT_CERT_AUTH"), "localCertLocationUrl": SimStruct({"size": SimTypeInt(signed=False, label="UInt32"), "data": SimTypePointer(SimTypeChar(label="Byte"), offset=0)}, name="FWP_BYTE_BLOB", pack=False, align=None)}, name="IKEEXT_CERTIFICATE_AUTHENTICATION1", pack=False, align=None), "kerberosAuthentication": SimStruct({"flags": SimTypeInt(signed=False, label="IKEEXT_KERBEROS_AUTHENTICATION_FLAGS")}, name="IKEEXT_KERBEROS_AUTHENTICATION0", pack=False, align=None), "ntlmV2Authentication": SimStruct({"flags": SimTypeInt(signed=False, label="UInt32")}, name="IKEEXT_NTLM_V2_AUTHENTICATION0", pack=False, align=None), "sslAuthentication": SimStruct({"inboundConfigType": SimTypeInt(signed=False, label="IKEEXT_CERT_CONFIG_TYPE"), "Anonymous1": SimUnion({"Anonymous": SimStruct({"inboundRootArraySize": SimTypeInt(signed=False, label="UInt32"), "inboundRootArray": SimTypePointer(SimStruct({"certData": SimStruct({"size": SimTypeInt(signed=False, label="UInt32"), "data": SimTypePointer(SimTypeChar(label="Byte"), offset=0)}, name="FWP_BYTE_BLOB", pack=False, align=None), "flags": SimTypeInt(signed=False, label="IKEEXT_CERT_FLAGS")}, name="IKEEXT_CERT_ROOT_CONFIG0", pack=False, align=None), offset=0)}, name="_Anonymous_e__Struct", pack=False, align=None), "inboundEnterpriseStoreConfig": SimTypePointer(SimStruct({"certData": SimStruct({"size": SimTypeInt(signed=False, label="UInt32"), "data": SimTypePointer(SimTypeChar(label="Byte"), offset=0)}, name="FWP_BYTE_BLOB", pack=False, align=None), "flags": SimTypeInt(signed=False, label="IKEEXT_CERT_FLAGS")}, name="IKEEXT_CERT_ROOT_CONFIG0", pack=False, align=None), offset=0), "inboundTrustedRootStoreConfig": SimTypePointer(SimStruct({"certData": SimStruct({"size": SimTypeInt(signed=False, label="UInt32"), "data": SimTypePointer(SimTypeChar(label="Byte"), offset=0)}, name="FWP_BYTE_BLOB", pack=False, align=None), "flags": SimTypeInt(signed=False, label="IKEEXT_CERT_FLAGS")}, name="IKEEXT_CERT_ROOT_CONFIG0", pack=False, align=None), offset=0)}, name="<anon>", label="None"), "outboundConfigType": SimTypeInt(signed=False, label="IKEEXT_CERT_CONFIG_TYPE"), "Anonymous2": SimUnion({"Anonymous": SimStruct({"outboundRootArraySize": SimTypeInt(signed=False, label="UInt32"), "outboundRootArray": SimTypePointer(SimStruct({"certData": SimStruct({"size": SimTypeInt(signed=False, label="UInt32"), "data": SimTypePointer(SimTypeChar(label="Byte"), offset=0)}, name="FWP_BYTE_BLOB", pack=False, align=None), "flags": SimTypeInt(signed=False, label="IKEEXT_CERT_FLAGS")}, name="IKEEXT_CERT_ROOT_CONFIG0", pack=False, align=None), offset=0)}, name="_Anonymous_e__Struct", pack=False, align=None), "outboundEnterpriseStoreConfig": SimTypePointer(SimStruct({"certData": SimStruct({"size": SimTypeInt(signed=False, label="UInt32"), "data": SimTypePointer(SimTypeChar(label="Byte"), offset=0)}, name="FWP_BYTE_BLOB", pack=False, align=None), "flags": SimTypeInt(signed=False, label="IKEEXT_CERT_FLAGS")}, name="IKEEXT_CERT_ROOT_CONFIG0", pack=False, align=None), offset=0), "outboundTrustedRootStoreConfig": SimTypePointer(SimStruct({"certData": SimStruct({"size": SimTypeInt(signed=False, label="UInt32"), "data": SimTypePointer(SimTypeChar(label="Byte"), offset=0)}, name="FWP_BYTE_BLOB", pack=False, align=None), "flags": SimTypeInt(signed=False, label="IKEEXT_CERT_FLAGS")}, name="IKEEXT_CERT_ROOT_CONFIG0", pack=False, align=None), offset=0)}, name="<anon>", label="None"), "flags": SimTypeInt(signed=False, label="IKEEXT_CERT_AUTH"), "localCertLocationUrl": SimStruct({"size": SimTypeInt(signed=False, label="UInt32"), "data": SimTypePointer(SimTypeChar(label="Byte"), offset=0)}, name="FWP_BYTE_BLOB", pack=False, align=None)}, name="IKEEXT_CERTIFICATE_AUTHENTICATION1", pack=False, align=None), "cgaAuthentication": SimStruct({"keyContainerName": SimTypePointer(SimTypeChar(label="Char"), offset=0), "cspName": SimTypePointer(SimTypeChar(label="Char"), offset=0), "cspType": SimTypeInt(signed=False, label="UInt32"), "cgaModifier": SimStruct({"byteArray16": SimTypeFixedSizeArray(SimTypeChar(label="Byte"), 16)}, name="FWP_BYTE_ARRAY16", pack=False, align=None), "cgaCollisionCount": SimTypeChar(label="Byte")}, name="IKEEXT_IPV6_CGA_AUTHENTICATION0", pack=False, align=None), "eapAuthentication": SimStruct({"flags": SimTypeInt(signed=False, label="IKEEXT_EAP_AUTHENTICATION_FLAGS")}, name="IKEEXT_EAP_AUTHENTICATION0", pack=False, align=None)}, name="<anon>", label="None")}, name="IKEEXT_AUTHENTICATION_METHOD1", pack=False, align=None), offset=0), "initiatorImpersonationType": SimTypeInt(signed=False, label="IKEEXT_AUTHENTICATION_IMPERSONATION_TYPE")}, name="IKEEXT_EM_POLICY1", pack=False, align=None), offset=0)}, name="IPSEC_TUNNEL_POLICY1", pack=False, align=None), offset=0), "ikeMmPolicy": SimTypePointer(SimStruct({"softExpirationTime": SimTypeInt(signed=False, label="UInt32"), "numAuthenticationMethods": SimTypeInt(signed=False, label="UInt32"), "authenticationMethods": SimTypePointer(SimStruct({"authenticationMethodType": SimTypeInt(signed=False, label="IKEEXT_AUTHENTICATION_METHOD_TYPE"), "Anonymous": SimUnion({"presharedKeyAuthentication": SimStruct({"presharedKey": SimStruct({"size": SimTypeInt(signed=False, label="UInt32"), "data": SimTypePointer(SimTypeChar(label="Byte"), offset=0)}, name="FWP_BYTE_BLOB", pack=False, align=None), "flags": SimTypeInt(signed=False, label="IKEEXT_PRESHARED_KEY_AUTHENTICATION_FLAGS")}, name="IKEEXT_PRESHARED_KEY_AUTHENTICATION1", pack=False, align=None), "certificateAuthentication": SimStruct({"inboundConfigType": SimTypeInt(signed=False, label="IKEEXT_CERT_CONFIG_TYPE"), "Anonymous1": SimUnion({"Anonymous": SimStruct({"inboundRootArraySize": SimTypeInt(signed=False, label="UInt32"), "inboundRootArray": SimTypePointer(SimStruct({"certData": SimStruct({"size": SimTypeInt(signed=False, label="UInt32"), "data": SimTypePointer(SimTypeChar(label="Byte"), offset=0)}, name="FWP_BYTE_BLOB", pack=False, align=None), "flags": SimTypeInt(signed=False, label="IKEEXT_CERT_FLAGS")}, name="IKEEXT_CERT_ROOT_CONFIG0", pack=False, align=None), offset=0)}, name="_Anonymous_e__Struct", pack=False, align=None), "inboundEnterpriseStoreConfig": SimTypePointer(SimStruct({"certData": SimStruct({"size": SimTypeInt(signed=False, label="UInt32"), "data": SimTypePointer(SimTypeChar(label="Byte"), offset=0)}, name="FWP_BYTE_BLOB", pack=False, align=None), "flags": SimTypeInt(signed=False, label="IKEEXT_CERT_FLAGS")}, name="IKEEXT_CERT_ROOT_CONFIG0", pack=False, align=None), offset=0), "inboundTrustedRootStoreConfig": SimTypePointer(SimStruct({"certData": SimStruct({"size": SimTypeInt(signed=False, label="UInt32"), "data": SimTypePointer(SimTypeChar(label="Byte"), offset=0)}, name="FWP_BYTE_BLOB", pack=False, align=None), "flags": SimTypeInt(signed=False, label="IKEEXT_CERT_FLAGS")}, name="IKEEXT_CERT_ROOT_CONFIG0", pack=False, align=None), offset=0)}, name="<anon>", label="None"), "outboundConfigType": SimTypeInt(signed=False, label="IKEEXT_CERT_CONFIG_TYPE"), "Anonymous2": SimUnion({"Anonymous": SimStruct({"outboundRootArraySize": SimTypeInt(signed=False, label="UInt32"), "outboundRootArray": SimTypePointer(SimStruct({"certData": SimStruct({"size": SimTypeInt(signed=False, label="UInt32"), "data": SimTypePointer(SimTypeChar(label="Byte"), offset=0)}, name="FWP_BYTE_BLOB", pack=False, align=None), "flags": SimTypeInt(signed=False, label="IKEEXT_CERT_FLAGS")}, name="IKEEXT_CERT_ROOT_CONFIG0", pack=False, align=None), offset=0)}, name="_Anonymous_e__Struct", pack=False, align=None), "outboundEnterpriseStoreConfig": SimTypePointer(SimStruct({"certData": SimStruct({"size": SimTypeInt(signed=False, label="UInt32"), "data": SimTypePointer(SimTypeChar(label="Byte"), offset=0)}, name="FWP_BYTE_BLOB", pack=False, align=None), "flags": SimTypeInt(signed=False, label="IKEEXT_CERT_FLAGS")}, name="IKEEXT_CERT_ROOT_CONFIG0", pack=False, align=None), offset=0), "outboundTrustedRootStoreConfig": SimTypePointer(SimStruct({"certData": SimStruct({"size": SimTypeInt(signed=False, label="UInt32"), "data": SimTypePointer(SimTypeChar(label="Byte"), offset=0)}, name="FWP_BYTE_BLOB", pack=False, align=None), "flags": SimTypeInt(signed=False, label="IKEEXT_CERT_FLAGS")}, name="IKEEXT_CERT_ROOT_CONFIG0", pack=False, align=None), offset=0)}, name="<anon>", label="None"), "flags": SimTypeInt(signed=False, label="IKEEXT_CERT_AUTH"), "localCertLocationUrl": SimStruct({"size": SimTypeInt(signed=False, label="UInt32"), "data": SimTypePointer(SimTypeChar(label="Byte"), offset=0)}, name="FWP_BYTE_BLOB", pack=False, align=None)}, name="IKEEXT_CERTIFICATE_AUTHENTICATION1", pack=False, align=None), "kerberosAuthentication": SimStruct({"flags": SimTypeInt(signed=False, label="IKEEXT_KERBEROS_AUTHENTICATION_FLAGS")}, name="IKEEXT_KERBEROS_AUTHENTICATION0", pack=False, align=None), "ntlmV2Authentication": SimStruct({"flags": SimTypeInt(signed=False, label="UInt32")}, name="IKEEXT_NTLM_V2_AUTHENTICATION0", pack=False, align=None), "sslAuthentication": SimStruct({"inboundConfigType": SimTypeInt(signed=False, label="IKEEXT_CERT_CONFIG_TYPE"), "Anonymous1": SimUnion({"Anonymous": SimStruct({"inboundRootArraySize": SimTypeInt(signed=False, label="UInt32"), "inboundRootArray": SimTypePointer(SimStruct({"certData": SimStruct({"size": SimTypeInt(signed=False, label="UInt32"), "data": SimTypePointer(SimTypeChar(label="Byte"), offset=0)}, name="FWP_BYTE_BLOB", pack=False, align=None), "flags": SimTypeInt(signed=False, label="IKEEXT_CERT_FLAGS")}, name="IKEEXT_CERT_ROOT_CONFIG0", pack=False, align=None), offset=0)}, name="_Anonymous_e__Struct", pack=False, align=None), "inboundEnterpriseStoreConfig": SimTypePointer(SimStruct({"certData": SimStruct({"size": SimTypeInt(signed=False, label="UInt32"), "data": SimTypePointer(SimTypeChar(label="Byte"), offset=0)}, name="FWP_BYTE_BLOB", pack=False, align=None), "flags": SimTypeInt(signed=False, label="IKEEXT_CERT_FLAGS")}, name="IKEEXT_CERT_ROOT_CONFIG0", pack=False, align=None), offset=0), "inboundTrustedRootStoreConfig": SimTypePointer(SimStruct({"certData": SimStruct({"size": SimTypeInt(signed=False, label="UInt32"), "data": SimTypePointer(SimTypeChar(label="Byte"), offset=0)}, name="FWP_BYTE_BLOB", pack=False, align=None), "flags": SimTypeInt(signed=False, label="IKEEXT_CERT_FLAGS")}, name="IKEEXT_CERT_ROOT_CONFIG0", pack=False, align=None), offset=0)}, name="<anon>", label="None"), "outboundConfigType": SimTypeInt(signed=False, label="IKEEXT_CERT_CONFIG_TYPE"), "Anonymous2": SimUnion({"Anonymous": SimStruct({"outboundRootArraySize": SimTypeInt(signed=False, label="UInt32"), "outboundRootArray": SimTypePointer(SimStruct({"certData": SimStruct({"size": SimTypeInt(signed=False, label="UInt32"), "data": SimTypePointer(SimTypeChar(label="Byte"), offset=0)}, name="FWP_BYTE_BLOB", pack=False, align=None), "flags": SimTypeInt(signed=False, label="IKEEXT_CERT_FLAGS")}, name="IKEEXT_CERT_ROOT_CONFIG0", pack=False, align=None), offset=0)}, name="_Anonymous_e__Struct", pack=False, align=None), "outboundEnterpriseStoreConfig": SimTypePointer(SimStruct({"certData": SimStruct({"size": SimTypeInt(signed=False, label="UInt32"), "data": SimTypePointer(SimTypeChar(label="Byte"), offset=0)}, name="FWP_BYTE_BLOB", pack=False, align=None), "flags": SimTypeInt(signed=False, label="IKEEXT_CERT_FLAGS")}, name="IKEEXT_CERT_ROOT_CONFIG0", pack=False, align=None), offset=0), "outboundTrustedRootStoreConfig": SimTypePointer(SimStruct({"certData": SimStruct({"size": SimTypeInt(signed=False, label="UInt32"), "data": SimTypePointer(SimTypeChar(label="Byte"), offset=0)}, name="FWP_BYTE_BLOB", pack=False, align=None), "flags": SimTypeInt(signed=False, label="IKEEXT_CERT_FLAGS")}, name="IKEEXT_CERT_ROOT_CONFIG0", pack=False, align=None), offset=0)}, name="<anon>", label="None"), "flags": SimTypeInt(signed=False, label="IKEEXT_CERT_AUTH"), "localCertLocationUrl": SimStruct({"size": SimTypeInt(signed=False, label="UInt32"), "data": SimTypePointer(SimTypeChar(label="Byte"), offset=0)}, name="FWP_BYTE_BLOB", pack=False, align=None)}, name="IKEEXT_CERTIFICATE_AUTHENTICATION1", pack=False, align=None), "cgaAuthentication": SimStruct({"keyContainerName": SimTypePointer(SimTypeChar(label="Char"), offset=0), "cspName": SimTypePointer(SimTypeChar(label="Char"), offset=0), "cspType": SimTypeInt(signed=False, label="UInt32"), "cgaModifier": SimStruct({"byteArray16": SimTypeFixedSizeArray(SimTypeChar(label="Byte"), 16)}, name="FWP_BYTE_ARRAY16", pack=False, align=None), "cgaCollisionCount": SimTypeChar(label="Byte")}, name="IKEEXT_IPV6_CGA_AUTHENTICATION0", pack=False, align=None), "eapAuthentication": SimStruct({"flags": SimTypeInt(signed=False, label="IKEEXT_EAP_AUTHENTICATION_FLAGS")}, name="IKEEXT_EAP_AUTHENTICATION0", pack=False, align=None)}, name="<anon>", label="None")}, name="IKEEXT_AUTHENTICATION_METHOD1", pack=False, align=None), offset=0), "initiatorImpersonationType": SimTypeInt(signed=False, label="IKEEXT_AUTHENTICATION_IMPERSONATION_TYPE"), "numIkeProposals": SimTypeInt(signed=False, label="UInt32"), "ikeProposals": SimTypePointer(SimStruct({"cipherAlgorithm": SimStruct({"algoIdentifier": SimTypeInt(signed=False, label="IKEEXT_CIPHER_TYPE"), "keyLen": SimTypeInt(signed=False, label="UInt32"), "rounds": SimTypeInt(signed=False, label="UInt32")}, name="IKEEXT_CIPHER_ALGORITHM0", pack=False, align=None), "integrityAlgorithm": SimStruct({"algoIdentifier": SimTypeInt(signed=False, label="IKEEXT_INTEGRITY_TYPE")}, name="IKEEXT_INTEGRITY_ALGORITHM0", pack=False, align=None), "maxLifetimeSeconds": SimTypeInt(signed=False, label="UInt32"), "dhGroup": SimTypeInt(signed=False, label="IKEEXT_DH_GROUP"), "quickModeLimit": SimTypeInt(signed=False, label="UInt32")}, name="IKEEXT_PROPOSAL0", pack=False, align=None), offset=0), "flags": SimTypeInt(signed=False, label="IKEEXT_POLICY_FLAG"), "maxDynamicFilters": SimTypeInt(signed=False, label="UInt32"), "retransmitDurationSecs": SimTypeInt(signed=False, label="UInt32")}, name="IKEEXT_POLICY1", pack=False, align=None), offset=0), "authIpMmPolicy": SimTypePointer(SimStruct({"softExpirationTime": SimTypeInt(signed=False, label="UInt32"), "numAuthenticationMethods": SimTypeInt(signed=False, label="UInt32"), "authenticationMethods": SimTypePointer(SimStruct({"authenticationMethodType": SimTypeInt(signed=False, label="IKEEXT_AUTHENTICATION_METHOD_TYPE"), "Anonymous": SimUnion({"presharedKeyAuthentication": SimStruct({"presharedKey": SimStruct({"size": SimTypeInt(signed=False, label="UInt32"), "data": SimTypePointer(SimTypeChar(label="Byte"), offset=0)}, name="FWP_BYTE_BLOB", pack=False, align=None), "flags": SimTypeInt(signed=False, label="IKEEXT_PRESHARED_KEY_AUTHENTICATION_FLAGS")}, name="IKEEXT_PRESHARED_KEY_AUTHENTICATION1", pack=False, align=None), "certificateAuthentication": SimStruct({"inboundConfigType": SimTypeInt(signed=False, label="IKEEXT_CERT_CONFIG_TYPE"), "Anonymous1": SimUnion({"Anonymous": SimStruct({"inboundRootArraySize": SimTypeInt(signed=False, label="UInt32"), "inboundRootArray": SimTypePointer(SimStruct({"certData": SimStruct({"size": SimTypeInt(signed=False, label="UInt32"), "data": SimTypePointer(SimTypeChar(label="Byte"), offset=0)}, name="FWP_BYTE_BLOB", pack=False, align=None), "flags": SimTypeInt(signed=False, label="IKEEXT_CERT_FLAGS")}, name="IKEEXT_CERT_ROOT_CONFIG0", pack=False, align=None), offset=0)}, name="_Anonymous_e__Struct", pack=False, align=None), "inboundEnterpriseStoreConfig": SimTypePointer(SimStruct({"certData": SimStruct({"size": SimTypeInt(signed=False, label="UInt32"), "data": SimTypePointer(SimTypeChar(label="Byte"), offset=0)}, name="FWP_BYTE_BLOB", pack=False, align=None), "flags": SimTypeInt(signed=False, label="IKEEXT_CERT_FLAGS")}, name="IKEEXT_CERT_ROOT_CONFIG0", pack=False, align=None), offset=0), "inboundTrustedRootStoreConfig": SimTypePointer(SimStruct({"certData": SimStruct({"size": SimTypeInt(signed=False, label="UInt32"), "data": SimTypePointer(SimTypeChar(label="Byte"), offset=0)}, name="FWP_BYTE_BLOB", pack=False, align=None), "flags": SimTypeInt(signed=False, label="IKEEXT_CERT_FLAGS")}, name="IKEEXT_CERT_ROOT_CONFIG0", pack=False, align=None), offset=0)}, name="<anon>", label="None"), "outboundConfigType": SimTypeInt(signed=False, label="IKEEXT_CERT_CONFIG_TYPE"), "Anonymous2": SimUnion({"Anonymous": SimStruct({"outboundRootArraySize": SimTypeInt(signed=False, label="UInt32"), "outboundRootArray": SimTypePointer(SimStruct({"certData": SimStruct({"size": SimTypeInt(signed=False, label="UInt32"), "data": SimTypePointer(SimTypeChar(label="Byte"), offset=0)}, name="FWP_BYTE_BLOB", pack=False, align=None), "flags": SimTypeInt(signed=False, label="IKEEXT_CERT_FLAGS")}, name="IKEEXT_CERT_ROOT_CONFIG0", pack=False, align=None), offset=0)}, name="_Anonymous_e__Struct", pack=False, align=None), "outboundEnterpriseStoreConfig": SimTypePointer(SimStruct({"certData": SimStruct({"size": SimTypeInt(signed=False, label="UInt32"), "data": SimTypePointer(SimTypeChar(label="Byte"), offset=0)}, name="FWP_BYTE_BLOB", pack=False, align=None), "flags": SimTypeInt(signed=False, label="IKEEXT_CERT_FLAGS")}, name="IKEEXT_CERT_ROOT_CONFIG0", pack=False, align=None), offset=0), "outboundTrustedRootStoreConfig": SimTypePointer(SimStruct({"certData": SimStruct({"size": SimTypeInt(signed=False, label="UInt32"), "data": SimTypePointer(SimTypeChar(label="Byte"), offset=0)}, name="FWP_BYTE_BLOB", pack=False, align=None), "flags": SimTypeInt(signed=False, label="IKEEXT_CERT_FLAGS")}, name="IKEEXT_CERT_ROOT_CONFIG0", pack=False, align=None), offset=0)}, name="<anon>", label="None"), "flags": SimTypeInt(signed=False, label="IKEEXT_CERT_AUTH"), "localCertLocationUrl": SimStruct({"size": SimTypeInt(signed=False, label="UInt32"), "data": SimTypePointer(SimTypeChar(label="Byte"), offset=0)}, name="FWP_BYTE_BLOB", pack=False, align=None)}, name="IKEEXT_CERTIFICATE_AUTHENTICATION1", pack=False, align=None), "kerberosAuthentication": SimStruct({"flags": SimTypeInt(signed=False, label="IKEEXT_KERBEROS_AUTHENTICATION_FLAGS")}, name="IKEEXT_KERBEROS_AUTHENTICATION0", pack=False, align=None), "ntlmV2Authentication": SimStruct({"flags": SimTypeInt(signed=False, label="UInt32")}, name="IKEEXT_NTLM_V2_AUTHENTICATION0", pack=False, align=None), "sslAuthentication": SimStruct({"inboundConfigType": SimTypeInt(signed=False, label="IKEEXT_CERT_CONFIG_TYPE"), "Anonymous1": SimUnion({"Anonymous": SimStruct({"inboundRootArraySize": SimTypeInt(signed=False, label="UInt32"), "inboundRootArray": SimTypePointer(SimStruct({"certData": SimStruct({"size": SimTypeInt(signed=False, label="UInt32"), "data": SimTypePointer(SimTypeChar(label="Byte"), offset=0)}, name="FWP_BYTE_BLOB", pack=False, align=None), "flags": SimTypeInt(signed=False, label="IKEEXT_CERT_FLAGS")}, name="IKEEXT_CERT_ROOT_CONFIG0", pack=False, align=None), offset=0)}, name="_Anonymous_e__Struct", pack=False, align=None), "inboundEnterpriseStoreConfig": SimTypePointer(SimStruct({"certData": SimStruct({"size": SimTypeInt(signed=False, label="UInt32"), "data": SimTypePointer(SimTypeChar(label="Byte"), offset=0)}, name="FWP_BYTE_BLOB", pack=False, align=None), "flags": SimTypeInt(signed=False, label="IKEEXT_CERT_FLAGS")}, name="IKEEXT_CERT_ROOT_CONFIG0", pack=False, align=None), offset=0), "inboundTrustedRootStoreConfig": SimTypePointer(SimStruct({"certData": SimStruct({"size": SimTypeInt(signed=False, label="UInt32"), "data": SimTypePointer(SimTypeChar(label="Byte"), offset=0)}, name="FWP_BYTE_BLOB", pack=False, align=None), "flags": SimTypeInt(signed=False, label="IKEEXT_CERT_FLAGS")}, name="IKEEXT_CERT_ROOT_CONFIG0", pack=False, align=None), offset=0)}, name="<anon>", label="None"), "outboundConfigType": SimTypeInt(signed=False, label="IKEEXT_CERT_CONFIG_TYPE"), "Anonymous2": SimUnion({"Anonymous": SimStruct({"outboundRootArraySize": SimTypeInt(signed=False, label="UInt32"), "outboundRootArray": SimTypePointer(SimStruct({"certData": SimStruct({"size": SimTypeInt(signed=False, label="UInt32"), "data": SimTypePointer(SimTypeChar(label="Byte"), offset=0)}, name="FWP_BYTE_BLOB", pack=False, align=None), "flags": SimTypeInt(signed=False, label="IKEEXT_CERT_FLAGS")}, name="IKEEXT_CERT_ROOT_CONFIG0", pack=False, align=None), offset=0)}, name="_Anonymous_e__Struct", pack=False, align=None), "outboundEnterpriseStoreConfig": SimTypePointer(SimStruct({"certData": SimStruct({"size": SimTypeInt(signed=False, label="UInt32"), "data": SimTypePointer(SimTypeChar(label="Byte"), offset=0)}, name="FWP_BYTE_BLOB", pack=False, align=None), "flags": SimTypeInt(signed=False, label="IKEEXT_CERT_FLAGS")}, name="IKEEXT_CERT_ROOT_CONFIG0", pack=False, align=None), offset=0), "outboundTrustedRootStoreConfig": SimTypePointer(SimStruct({"certData": SimStruct({"size": SimTypeInt(signed=False, label="UInt32"), "data": SimTypePointer(SimTypeChar(label="Byte"), offset=0)}, name="FWP_BYTE_BLOB", pack=False, align=None), "flags": SimTypeInt(signed=False, label="IKEEXT_CERT_FLAGS")}, name="IKEEXT_CERT_ROOT_CONFIG0", pack=False, align=None), offset=0)}, name="<anon>", label="None"), "flags": SimTypeInt(signed=False, label="IKEEXT_CERT_AUTH"), "localCertLocationUrl": SimStruct({"size": SimTypeInt(signed=False, label="UInt32"), "data": SimTypePointer(SimTypeChar(label="Byte"), offset=0)}, name="FWP_BYTE_BLOB", pack=False, align=None)}, name="IKEEXT_CERTIFICATE_AUTHENTICATION1", pack=False, align=None), "cgaAuthentication": SimStruct({"keyContainerName": SimTypePointer(SimTypeChar(label="Char"), offset=0), "cspName": SimTypePointer(SimTypeChar(label="Char"), offset=0), "cspType": SimTypeInt(signed=False, label="UInt32"), "cgaModifier": SimStruct({"byteArray16": SimTypeFixedSizeArray(SimTypeChar(label="Byte"), 16)}, name="FWP_BYTE_ARRAY16", pack=False, align=None), "cgaCollisionCount": SimTypeChar(label="Byte")}, name="IKEEXT_IPV6_CGA_AUTHENTICATION0", pack=False, align=None), "eapAuthentication": SimStruct({"flags": SimTypeInt(signed=False, label="IKEEXT_EAP_AUTHENTICATION_FLAGS")}, name="IKEEXT_EAP_AUTHENTICATION0", pack=False, align=None)}, name="<anon>", label="None")}, name="IKEEXT_AUTHENTICATION_METHOD1", pack=False, align=None), offset=0), "initiatorImpersonationType": SimTypeInt(signed=False, label="IKEEXT_AUTHENTICATION_IMPERSONATION_TYPE"), "numIkeProposals": SimTypeInt(signed=False, label="UInt32"), "ikeProposals": SimTypePointer(SimStruct({"cipherAlgorithm": SimStruct({"algoIdentifier": SimTypeInt(signed=False, label="IKEEXT_CIPHER_TYPE"), "keyLen": SimTypeInt(signed=False, label="UInt32"), "rounds": SimTypeInt(signed=False, label="UInt32")}, name="IKEEXT_CIPHER_ALGORITHM0", pack=False, align=None), "integrityAlgorithm": SimStruct({"algoIdentifier": SimTypeInt(signed=False, label="IKEEXT_INTEGRITY_TYPE")}, name="IKEEXT_INTEGRITY_ALGORITHM0", pack=False, align=None), "maxLifetimeSeconds": SimTypeInt(signed=False, label="UInt32"), "dhGroup": SimTypeInt(signed=False, label="IKEEXT_DH_GROUP"), "quickModeLimit": SimTypeInt(signed=False, label="UInt32")}, name="IKEEXT_PROPOSAL0", pack=False, align=None), offset=0), "flags": SimTypeInt(signed=False, label="IKEEXT_POLICY_FLAG"), "maxDynamicFilters": SimTypeInt(signed=False, label="UInt32"), "retransmitDurationSecs": SimTypeInt(signed=False, label="UInt32")}, name="IKEEXT_POLICY1", pack=False, align=None), offset=0), "dataBuffer": SimTypePointer(SimStruct({"size": SimTypeInt(signed=False, label="UInt32"), "data": SimTypePointer(SimTypeChar(label="Byte"), offset=0)}, name="FWP_BYTE_BLOB", pack=False, align=None), offset=0), "classifyOptions": SimTypePointer(SimStruct({"numOptions": SimTypeInt(signed=False, label="UInt32"), "options": SimTypePointer(SimStruct({"type": SimTypeInt(signed=False, label="FWP_CLASSIFY_OPTION_TYPE"), "value": SimStruct({"type": SimTypeInt(signed=False, label="FWP_DATA_TYPE"), "Anonymous": SimUnion({"uint8": SimTypeChar(label="Byte"), "uint16": SimTypeShort(signed=False, label="UInt16"), "uint32": SimTypeInt(signed=False, label="UInt32"), "uint64": SimTypePointer(SimTypeLongLong(signed=False, label="UInt64"), offset=0), "int8": SimTypeChar(label="SByte"), "int16": SimTypeShort(signed=True, label="Int16"), "int32": SimTypeInt(signed=True, label="Int32"), "int64": SimTypePointer(SimTypeLongLong(signed=True, label="Int64"), offset=0), "float32": SimTypeFloat(size=32), "double64": SimTypePointer(SimTypeFloat(size=64), offset=0), "byteArray16": SimTypePointer(SimStruct({"byteArray16": SimTypeFixedSizeArray(SimTypeChar(label="Byte"), 16)}, name="FWP_BYTE_ARRAY16", pack=False, align=None), offset=0), "byteBlob": SimTypePointer(SimStruct({"size": SimTypeInt(signed=False, label="UInt32"), "data": SimTypePointer(SimTypeChar(label="Byte"), offset=0)}, name="FWP_BYTE_BLOB", pack=False, align=None), offset=0), "sid": SimTypePointer(SimTypeBottom(label="SID"), offset=0), "sd": SimTypePointer(SimStruct({"size": SimTypeInt(signed=False, label="UInt32"), "data": SimTypePointer(SimTypeChar(label="Byte"), offset=0)}, name="FWP_BYTE_BLOB", pack=False, align=None), offset=0), "tokenInformation": SimTypePointer(SimStruct({"sidCount": SimTypeInt(signed=False, label="UInt32"), "sids": SimTypePointer(SimTypeBottom(label="SID_AND_ATTRIBUTES"), offset=0), "restrictedSidCount": SimTypeInt(signed=False, label="UInt32"), "restrictedSids": SimTypePointer(SimTypeBottom(label="SID_AND_ATTRIBUTES"), offset=0)}, name="FWP_TOKEN_INFORMATION", pack=False, align=None), offset=0), "tokenAccessInformation": SimTypePointer(SimStruct({"size": SimTypeInt(signed=False, label="UInt32"), "data": SimTypePointer(SimTypeChar(label="Byte"), offset=0)}, name="FWP_BYTE_BLOB", pack=False, align=None), offset=0), "unicodeString": SimTypePointer(SimTypeChar(label="Char"), offset=0), "byteArray6": SimTypePointer(SimStruct({"byteArray6": SimTypeFixedSizeArray(SimTypeChar(label="Byte"), 6)}, name="FWP_BYTE_ARRAY6", pack=False, align=None), offset=0), "bitmapArray64": SimTypePointer(SimStruct({"bitmapArray64": SimTypeFixedSizeArray(SimTypeChar(label="Byte"), 8)}, name="FWP_BITMAP_ARRAY64_", pack=False, align=None), offset=0)}, name="<anon>", label="None")}, name="FWP_VALUE0", pack=False, align=None)}, name="FWPM_CLASSIFY_OPTION0", pack=False, align=None), offset=0)}, name="FWPM_CLASSIFY_OPTIONS0", pack=False, align=None), offset=0), "ikeV2QmTunnelPolicy": SimTypePointer(SimStruct({"flags": SimTypeInt(signed=False, label="IPSEC_POLICY_FLAG"), "numIpsecProposals": SimTypeInt(signed=False, label="UInt32"), "ipsecProposals": SimTypePointer(SimStruct({"lifetime": SimStruct({"lifetimeSeconds": SimTypeInt(signed=False, label="UInt32"), "lifetimeKilobytes": SimTypeInt(signed=False, label="UInt32"), "lifetimePackets": SimTypeInt(signed=False, label="UInt32")}, name="IPSEC_SA_LIFETIME0", pack=False, align=None), "numSaTransforms": SimTypeInt(signed=False, label="UInt32"), "saTransforms": SimTypePointer(SimStruct({"ipsecTransformType": SimTypeInt(signed=False, label="IPSEC_TRANSFORM_TYPE"), "Anonymous": SimUnion({"ahTransform": SimTypePointer(SimStruct({"authTransformId": SimStruct({"authType": SimTypeInt(signed=False, label="IPSEC_AUTH_TYPE"), "authConfig": SimTypeChar(label="Byte")}, name="IPSEC_AUTH_TRANSFORM_ID0", pack=False, align=None), "cryptoModuleId": SimTypePointer(SimTypeBottom(label="Guid"), offset=0)}, name="IPSEC_AUTH_TRANSFORM0", pack=False, align=None), offset=0), "espAuthTransform": SimTypePointer(SimStruct({"authTransformId": SimStruct({"authType": SimTypeInt(signed=False, label="IPSEC_AUTH_TYPE"), "authConfig": SimTypeChar(label="Byte")}, name="IPSEC_AUTH_TRANSFORM_ID0", pack=False, align=None), "cryptoModuleId": SimTypePointer(SimTypeBottom(label="Guid"), offset=0)}, name="IPSEC_AUTH_TRANSFORM0", pack=False, align=None), offset=0), "espCipherTransform": SimTypePointer(SimStruct({"cipherTransformId": SimStruct({"cipherType": SimTypeInt(signed=False, label="IPSEC_CIPHER_TYPE"), "cipherConfig": SimTypeChar(label="Byte")}, name="IPSEC_CIPHER_TRANSFORM_ID0", pack=False, align=None), "cryptoModuleId": SimTypePointer(SimTypeBottom(label="Guid"), offset=0)}, name="IPSEC_CIPHER_TRANSFORM0", pack=False, align=None), offset=0), "espAuthAndCipherTransform": SimTypePointer(SimStruct({"authTransform": SimStruct({"authTransformId": SimStruct({"authType": SimTypeInt(signed=False, label="IPSEC_AUTH_TYPE"), "authConfig": SimTypeChar(label="Byte")}, name="IPSEC_AUTH_TRANSFORM_ID0", pack=False, align=None), "cryptoModuleId": SimTypePointer(SimTypeBottom(label="Guid"), offset=0)}, name="IPSEC_AUTH_TRANSFORM0", pack=False, align=None), "cipherTransform": SimStruct({"cipherTransformId": SimStruct({"cipherType": SimTypeInt(signed=False, label="IPSEC_CIPHER_TYPE"), "cipherConfig": SimTypeChar(label="Byte")}, name="IPSEC_CIPHER_TRANSFORM_ID0", pack=False, align=None), "cryptoModuleId": SimTypePointer(SimTypeBottom(label="Guid"), offset=0)}, name="IPSEC_CIPHER_TRANSFORM0", pack=False, align=None)}, name="IPSEC_AUTH_AND_CIPHER_TRANSFORM0", pack=False, align=None), offset=0), "espAuthFwTransform": SimTypePointer(SimStruct({"authTransformId": SimStruct({"authType": SimTypeInt(signed=False, label="IPSEC_AUTH_TYPE"), "authConfig": SimTypeChar(label="Byte")}, name="IPSEC_AUTH_TRANSFORM_ID0", pack=False, align=None), "cryptoModuleId": SimTypePointer(SimTypeBottom(label="Guid"), offset=0)}, name="IPSEC_AUTH_TRANSFORM0", pack=False, align=None), offset=0)}, name="<anon>", label="None")}, name="IPSEC_SA_TRANSFORM0", pack=False, align=None), offset=0), "pfsGroup": SimTypeInt(signed=False, label="IPSEC_PFS_GROUP")}, name="IPSEC_PROPOSAL0", pack=False, align=None), offset=0), "tunnelEndpoints": SimStruct({"ipVersion": SimTypeInt(signed=False, label="FWP_IP_VERSION"), "Anonymous1": SimUnion({"localV4Address": SimTypeInt(signed=False, label="UInt32"), "localV6Address": SimTypeFixedSizeArray(SimTypeChar(label="Byte"), 16)}, name="<anon>", label="None"), "Anonymous2": SimUnion({"remoteV4Address": SimTypeInt(signed=False, label="UInt32"), "remoteV6Address": SimTypeFixedSizeArray(SimTypeChar(label="Byte"), 16)}, name="<anon>", label="None"), "localIfLuid": SimTypeLongLong(signed=False, label="UInt64")}, name="IPSEC_TUNNEL_ENDPOINTS1", pack=False, align=None), "saIdleTimeout": SimStruct({"idleTimeoutSeconds": SimTypeInt(signed=False, label="UInt32"), "idleTimeoutSecondsFailOver": SimTypeInt(signed=False, label="UInt32")}, name="IPSEC_SA_IDLE_TIMEOUT0", pack=False, align=None), "emPolicy": SimTypePointer(SimStruct({"numAuthenticationMethods": SimTypeInt(signed=False, label="UInt32"), "authenticationMethods": SimTypePointer(SimStruct({"authenticationMethodType": SimTypeInt(signed=False, label="IKEEXT_AUTHENTICATION_METHOD_TYPE"), "Anonymous": SimUnion({"presharedKeyAuthentication": SimStruct({"presharedKey": SimStruct({"size": SimTypeInt(signed=False, label="UInt32"), "data": SimTypePointer(SimTypeChar(label="Byte"), offset=0)}, name="FWP_BYTE_BLOB", pack=False, align=None), "flags": SimTypeInt(signed=False, label="IKEEXT_PRESHARED_KEY_AUTHENTICATION_FLAGS")}, name="IKEEXT_PRESHARED_KEY_AUTHENTICATION1", pack=False, align=None), "certificateAuthentication": SimStruct({"inboundConfigType": SimTypeInt(signed=False, label="IKEEXT_CERT_CONFIG_TYPE"), "Anonymous1": SimUnion({"Anonymous": SimStruct({"inboundRootArraySize": SimTypeInt(signed=False, label="UInt32"), "inboundRootArray": SimTypePointer(SimStruct({"certData": SimStruct({"size": SimTypeInt(signed=False, label="UInt32"), "data": SimTypePointer(SimTypeChar(label="Byte"), offset=0)}, name="FWP_BYTE_BLOB", pack=False, align=None), "flags": SimTypeInt(signed=False, label="IKEEXT_CERT_FLAGS")}, name="IKEEXT_CERT_ROOT_CONFIG0", pack=False, align=None), offset=0)}, name="_Anonymous_e__Struct", pack=False, align=None), "inboundEnterpriseStoreConfig": SimTypePointer(SimStruct({"certData": SimStruct({"size": SimTypeInt(signed=False, label="UInt32"), "data": SimTypePointer(SimTypeChar(label="Byte"), offset=0)}, name="FWP_BYTE_BLOB", pack=False, align=None), "flags": SimTypeInt(signed=False, label="IKEEXT_CERT_FLAGS")}, name="IKEEXT_CERT_ROOT_CONFIG0", pack=False, align=None), offset=0), "inboundTrustedRootStoreConfig": SimTypePointer(SimStruct({"certData": SimStruct({"size": SimTypeInt(signed=False, label="UInt32"), "data": SimTypePointer(SimTypeChar(label="Byte"), offset=0)}, name="FWP_BYTE_BLOB", pack=False, align=None), "flags": SimTypeInt(signed=False, label="IKEEXT_CERT_FLAGS")}, name="IKEEXT_CERT_ROOT_CONFIG0", pack=False, align=None), offset=0)}, name="<anon>", label="None"), "outboundConfigType": SimTypeInt(signed=False, label="IKEEXT_CERT_CONFIG_TYPE"), "Anonymous2": SimUnion({"Anonymous": SimStruct({"outboundRootArraySize": SimTypeInt(signed=False, label="UInt32"), "outboundRootArray": SimTypePointer(SimStruct({"certData": SimStruct({"size": SimTypeInt(signed=False, label="UInt32"), "data": SimTypePointer(SimTypeChar(label="Byte"), offset=0)}, name="FWP_BYTE_BLOB", pack=False, align=None), "flags": SimTypeInt(signed=False, label="IKEEXT_CERT_FLAGS")}, name="IKEEXT_CERT_ROOT_CONFIG0", pack=False, align=None), offset=0)}, name="_Anonymous_e__Struct", pack=False, align=None), "outboundEnterpriseStoreConfig": SimTypePointer(SimStruct({"certData": SimStruct({"size": SimTypeInt(signed=False, label="UInt32"), "data": SimTypePointer(SimTypeChar(label="Byte"), offset=0)}, name="FWP_BYTE_BLOB", pack=False, align=None), "flags": SimTypeInt(signed=False, label="IKEEXT_CERT_FLAGS")}, name="IKEEXT_CERT_ROOT_CONFIG0", pack=False, align=None), offset=0), "outboundTrustedRootStoreConfig": SimTypePointer(SimStruct({"certData": SimStruct({"size": SimTypeInt(signed=False, label="UInt32"), "data": SimTypePointer(SimTypeChar(label="Byte"), offset=0)}, name="FWP_BYTE_BLOB", pack=False, align=None), "flags": SimTypeInt(signed=False, label="IKEEXT_CERT_FLAGS")}, name="IKEEXT_CERT_ROOT_CONFIG0", pack=False, align=None), offset=0)}, name="<anon>", label="None"), "flags": SimTypeInt(signed=False, label="IKEEXT_CERT_AUTH"), "localCertLocationUrl": SimStruct({"size": SimTypeInt(signed=False, label="UInt32"), "data": SimTypePointer(SimTypeChar(label="Byte"), offset=0)}, name="FWP_BYTE_BLOB", pack=False, align=None)}, name="IKEEXT_CERTIFICATE_AUTHENTICATION1", pack=False, align=None), "kerberosAuthentication": SimStruct({"flags": SimTypeInt(signed=False, label="IKEEXT_KERBEROS_AUTHENTICATION_FLAGS")}, name="IKEEXT_KERBEROS_AUTHENTICATION0", pack=False, align=None), "ntlmV2Authentication": SimStruct({"flags": SimTypeInt(signed=False, label="UInt32")}, name="IKEEXT_NTLM_V2_AUTHENTICATION0", pack=False, align=None), "sslAuthentication": SimStruct({"inboundConfigType": SimTypeInt(signed=False, label="IKEEXT_CERT_CONFIG_TYPE"), "Anonymous1": SimUnion({"Anonymous": SimStruct({"inboundRootArraySize": SimTypeInt(signed=False, label="UInt32"), "inboundRootArray": SimTypePointer(SimStruct({"certData": SimStruct({"size": SimTypeInt(signed=False, label="UInt32"), "data": SimTypePointer(SimTypeChar(label="Byte"), offset=0)}, name="FWP_BYTE_BLOB", pack=False, align=None), "flags": SimTypeInt(signed=False, label="IKEEXT_CERT_FLAGS")}, name="IKEEXT_CERT_ROOT_CONFIG0", pack=False, align=None), offset=0)}, name="_Anonymous_e__Struct", pack=False, align=None), "inboundEnterpriseStoreConfig": SimTypePointer(SimStruct({"certData": SimStruct({"size": SimTypeInt(signed=False, label="UInt32"), "data": SimTypePointer(SimTypeChar(label="Byte"), offset=0)}, name="FWP_BYTE_BLOB", pack=False, align=None), "flags": SimTypeInt(signed=False, label="IKEEXT_CERT_FLAGS")}, name="IKEEXT_CERT_ROOT_CONFIG0", pack=False, align=None), offset=0), "inboundTrustedRootStoreConfig": SimTypePointer(SimStruct({"certData": SimStruct({"size": SimTypeInt(signed=False, label="UInt32"), "data": SimTypePointer(SimTypeChar(label="Byte"), offset=0)}, name="FWP_BYTE_BLOB", pack=False, align=None), "flags": SimTypeInt(signed=False, label="IKEEXT_CERT_FLAGS")}, name="IKEEXT_CERT_ROOT_CONFIG0", pack=False, align=None), offset=0)}, name="<anon>", label="None"), "outboundConfigType": SimTypeInt(signed=False, label="IKEEXT_CERT_CONFIG_TYPE"), "Anonymous2": SimUnion({"Anonymous": SimStruct({"outboundRootArraySize": SimTypeInt(signed=False, label="UInt32"), "outboundRootArray": SimTypePointer(SimStruct({"certData": SimStruct({"size": SimTypeInt(signed=False, label="UInt32"), "data": SimTypePointer(SimTypeChar(label="Byte"), offset=0)}, name="FWP_BYTE_BLOB", pack=False, align=None), "flags": SimTypeInt(signed=False, label="IKEEXT_CERT_FLAGS")}, name="IKEEXT_CERT_ROOT_CONFIG0", pack=False, align=None), offset=0)}, name="_Anonymous_e__Struct", pack=False, align=None), "outboundEnterpriseStoreConfig": SimTypePointer(SimStruct({"certData": SimStruct({"size": SimTypeInt(signed=False, label="UInt32"), "data": SimTypePointer(SimTypeChar(label="Byte"), offset=0)}, name="FWP_BYTE_BLOB", pack=False, align=None), "flags": SimTypeInt(signed=False, label="IKEEXT_CERT_FLAGS")}, name="IKEEXT_CERT_ROOT_CONFIG0", pack=False, align=None), offset=0), "outboundTrustedRootStoreConfig": SimTypePointer(SimStruct({"certData": SimStruct({"size": SimTypeInt(signed=False, label="UInt32"), "data": SimTypePointer(SimTypeChar(label="Byte"), offset=0)}, name="FWP_BYTE_BLOB", pack=False, align=None), "flags": SimTypeInt(signed=False, label="IKEEXT_CERT_FLAGS")}, name="IKEEXT_CERT_ROOT_CONFIG0", pack=False, align=None), offset=0)}, name="<anon>", label="None"), "flags": SimTypeInt(signed=False, label="IKEEXT_CERT_AUTH"), "localCertLocationUrl": SimStruct({"size": SimTypeInt(signed=False, label="UInt32"), "data": SimTypePointer(SimTypeChar(label="Byte"), offset=0)}, name="FWP_BYTE_BLOB", pack=False, align=None)}, name="IKEEXT_CERTIFICATE_AUTHENTICATION1", pack=False, align=None), "cgaAuthentication": SimStruct({"keyContainerName": SimTypePointer(SimTypeChar(label="Char"), offset=0), "cspName": SimTypePointer(SimTypeChar(label="Char"), offset=0), "cspType": SimTypeInt(signed=False, label="UInt32"), "cgaModifier": SimStruct({"byteArray16": SimTypeFixedSizeArray(SimTypeChar(label="Byte"), 16)}, name="FWP_BYTE_ARRAY16", pack=False, align=None), "cgaCollisionCount": SimTypeChar(label="Byte")}, name="IKEEXT_IPV6_CGA_AUTHENTICATION0", pack=False, align=None), "eapAuthentication": SimStruct({"flags": SimTypeInt(signed=False, label="IKEEXT_EAP_AUTHENTICATION_FLAGS")}, name="IKEEXT_EAP_AUTHENTICATION0", pack=False, align=None)}, name="<anon>", label="None")}, name="IKEEXT_AUTHENTICATION_METHOD1", pack=False, align=None), offset=0), "initiatorImpersonationType": SimTypeInt(signed=False, label="IKEEXT_AUTHENTICATION_IMPERSONATION_TYPE")}, name="IKEEXT_EM_POLICY1", pack=False, align=None), offset=0)}, name="IPSEC_TUNNEL_POLICY1", pack=False, align=None), offset=0), "ikeV2MmPolicy": SimTypePointer(SimStruct({"softExpirationTime": SimTypeInt(signed=False, label="UInt32"), "numAuthenticationMethods": SimTypeInt(signed=False, label="UInt32"), "authenticationMethods": SimTypePointer(SimStruct({"authenticationMethodType": SimTypeInt(signed=False, label="IKEEXT_AUTHENTICATION_METHOD_TYPE"), "Anonymous": SimUnion({"presharedKeyAuthentication": SimStruct({"presharedKey": SimStruct({"size": SimTypeInt(signed=False, label="UInt32"), "data": SimTypePointer(SimTypeChar(label="Byte"), offset=0)}, name="FWP_BYTE_BLOB", pack=False, align=None), "flags": SimTypeInt(signed=False, label="IKEEXT_PRESHARED_KEY_AUTHENTICATION_FLAGS")}, name="IKEEXT_PRESHARED_KEY_AUTHENTICATION1", pack=False, align=None), "certificateAuthentication": SimStruct({"inboundConfigType": SimTypeInt(signed=False, label="IKEEXT_CERT_CONFIG_TYPE"), "Anonymous1": SimUnion({"Anonymous": SimStruct({"inboundRootArraySize": SimTypeInt(signed=False, label="UInt32"), "inboundRootArray": SimTypePointer(SimStruct({"certData": SimStruct({"size": SimTypeInt(signed=False, label="UInt32"), "data": SimTypePointer(SimTypeChar(label="Byte"), offset=0)}, name="FWP_BYTE_BLOB", pack=False, align=None), "flags": SimTypeInt(signed=False, label="IKEEXT_CERT_FLAGS")}, name="IKEEXT_CERT_ROOT_CONFIG0", pack=False, align=None), offset=0)}, name="_Anonymous_e__Struct", pack=False, align=None), "inboundEnterpriseStoreConfig": SimTypePointer(SimStruct({"certData": SimStruct({"size": SimTypeInt(signed=False, label="UInt32"), "data": SimTypePointer(SimTypeChar(label="Byte"), offset=0)}, name="FWP_BYTE_BLOB", pack=False, align=None), "flags": SimTypeInt(signed=False, label="IKEEXT_CERT_FLAGS")}, name="IKEEXT_CERT_ROOT_CONFIG0", pack=False, align=None), offset=0), "inboundTrustedRootStoreConfig": SimTypePointer(SimStruct({"certData": SimStruct({"size": SimTypeInt(signed=False, label="UInt32"), "data": SimTypePointer(SimTypeChar(label="Byte"), offset=0)}, name="FWP_BYTE_BLOB", pack=False, align=None), "flags": SimTypeInt(signed=False, label="IKEEXT_CERT_FLAGS")}, name="IKEEXT_CERT_ROOT_CONFIG0", pack=False, align=None), offset=0)}, name="<anon>", label="None"), "outboundConfigType": SimTypeInt(signed=False, label="IKEEXT_CERT_CONFIG_TYPE"), "Anonymous2": SimUnion({"Anonymous": SimStruct({"outboundRootArraySize": SimTypeInt(signed=False, label="UInt32"), "outboundRootArray": SimTypePointer(SimStruct({"certData": SimStruct({"size": SimTypeInt(signed=False, label="UInt32"), "data": SimTypePointer(SimTypeChar(label="Byte"), offset=0)}, name="FWP_BYTE_BLOB", pack=False, align=None), "flags": SimTypeInt(signed=False, label="IKEEXT_CERT_FLAGS")}, name="IKEEXT_CERT_ROOT_CONFIG0", pack=False, align=None), offset=0)}, name="_Anonymous_e__Struct", pack=False, align=None), "outboundEnterpriseStoreConfig": SimTypePointer(SimStruct({"certData": SimStruct({"size": SimTypeInt(signed=False, label="UInt32"), "data": SimTypePointer(SimTypeChar(label="Byte"), offset=0)}, name="FWP_BYTE_BLOB", pack=False, align=None), "flags": SimTypeInt(signed=False, label="IKEEXT_CERT_FLAGS")}, name="IKEEXT_CERT_ROOT_CONFIG0", pack=False, align=None), offset=0), "outboundTrustedRootStoreConfig": SimTypePointer(SimStruct({"certData": SimStruct({"size": SimTypeInt(signed=False, label="UInt32"), "data": SimTypePointer(SimTypeChar(label="Byte"), offset=0)}, name="FWP_BYTE_BLOB", pack=False, align=None), "flags": SimTypeInt(signed=False, label="IKEEXT_CERT_FLAGS")}, name="IKEEXT_CERT_ROOT_CONFIG0", pack=False, align=None), offset=0)}, name="<anon>", label="None"), "flags": SimTypeInt(signed=False, label="IKEEXT_CERT_AUTH"), "localCertLocationUrl": SimStruct({"size": SimTypeInt(signed=False, label="UInt32"), "data": SimTypePointer(SimTypeChar(label="Byte"), offset=0)}, name="FWP_BYTE_BLOB", pack=False, align=None)}, name="IKEEXT_CERTIFICATE_AUTHENTICATION1", pack=False, align=None), "kerberosAuthentication": SimStruct({"flags": SimTypeInt(signed=False, label="IKEEXT_KERBEROS_AUTHENTICATION_FLAGS")}, name="IKEEXT_KERBEROS_AUTHENTICATION0", pack=False, align=None), "ntlmV2Authentication": SimStruct({"flags": SimTypeInt(signed=False, label="UInt32")}, name="IKEEXT_NTLM_V2_AUTHENTICATION0", pack=False, align=None), "sslAuthentication": SimStruct({"inboundConfigType": SimTypeInt(signed=False, label="IKEEXT_CERT_CONFIG_TYPE"), "Anonymous1": SimUnion({"Anonymous": SimStruct({"inboundRootArraySize": SimTypeInt(signed=False, label="UInt32"), "inboundRootArray": SimTypePointer(SimStruct({"certData": SimStruct({"size": SimTypeInt(signed=False, label="UInt32"), "data": SimTypePointer(SimTypeChar(label="Byte"), offset=0)}, name="FWP_BYTE_BLOB", pack=False, align=None), "flags": SimTypeInt(signed=False, label="IKEEXT_CERT_FLAGS")}, name="IKEEXT_CERT_ROOT_CONFIG0", pack=False, align=None), offset=0)}, name="_Anonymous_e__Struct", pack=False, align=None), "inboundEnterpriseStoreConfig": SimTypePointer(SimStruct({"certData": SimStruct({"size": SimTypeInt(signed=False, label="UInt32"), "data": SimTypePointer(SimTypeChar(label="Byte"), offset=0)}, name="FWP_BYTE_BLOB", pack=False, align=None), "flags": SimTypeInt(signed=False, label="IKEEXT_CERT_FLAGS")}, name="IKEEXT_CERT_ROOT_CONFIG0", pack=False, align=None), offset=0), "inboundTrustedRootStoreConfig": SimTypePointer(SimStruct({"certData": SimStruct({"size": SimTypeInt(signed=False, label="UInt32"), "data": SimTypePointer(SimTypeChar(label="Byte"), offset=0)}, name="FWP_BYTE_BLOB", pack=False, align=None), "flags": SimTypeInt(signed=False, label="IKEEXT_CERT_FLAGS")}, name="IKEEXT_CERT_ROOT_CONFIG0", pack=False, align=None), offset=0)}, name="<anon>", label="None"), "outboundConfigType": SimTypeInt(signed=False, label="IKEEXT_CERT_CONFIG_TYPE"), "Anonymous2": SimUnion({"Anonymous": SimStruct({"outboundRootArraySize": SimTypeInt(signed=False, label="UInt32"), "outboundRootArray": SimTypePointer(SimStruct({"certData": SimStruct({"size": SimTypeInt(signed=False, label="UInt32"), "data": SimTypePointer(SimTypeChar(label="Byte"), offset=0)}, name="FWP_BYTE_BLOB", pack=False, align=None), "flags": SimTypeInt(signed=False, label="IKEEXT_CERT_FLAGS")}, name="IKEEXT_CERT_ROOT_CONFIG0", pack=False, align=None), offset=0)}, name="_Anonymous_e__Struct", pack=False, align=None), "outboundEnterpriseStoreConfig": SimTypePointer(SimStruct({"certData": SimStruct({"size": SimTypeInt(signed=False, label="UInt32"), "data": SimTypePointer(SimTypeChar(label="Byte"), offset=0)}, name="FWP_BYTE_BLOB", pack=False, align=None), "flags": SimTypeInt(signed=False, label="IKEEXT_CERT_FLAGS")}, name="IKEEXT_CERT_ROOT_CONFIG0", pack=False, align=None), offset=0), "outboundTrustedRootStoreConfig": SimTypePointer(SimStruct({"certData": SimStruct({"size": SimTypeInt(signed=False, label="UInt32"), "data": SimTypePointer(SimTypeChar(label="Byte"), offset=0)}, name="FWP_BYTE_BLOB", pack=False, align=None), "flags": SimTypeInt(signed=False, label="IKEEXT_CERT_FLAGS")}, name="IKEEXT_CERT_ROOT_CONFIG0", pack=False, align=None), offset=0)}, name="<anon>", label="None"), "flags": SimTypeInt(signed=False, label="IKEEXT_CERT_AUTH"), "localCertLocationUrl": SimStruct({"size": SimTypeInt(signed=False, label="UInt32"), "data": SimTypePointer(SimTypeChar(label="Byte"), offset=0)}, name="FWP_BYTE_BLOB", pack=False, align=None)}, name="IKEEXT_CERTIFICATE_AUTHENTICATION1", pack=False, align=None), "cgaAuthentication": SimStruct({"keyContainerName": SimTypePointer(SimTypeChar(label="Char"), offset=0), "cspName": SimTypePointer(SimTypeChar(label="Char"), offset=0), "cspType": SimTypeInt(signed=False, label="UInt32"), "cgaModifier": SimStruct({"byteArray16": SimTypeFixedSizeArray(SimTypeChar(label="Byte"), 16)}, name="FWP_BYTE_ARRAY16", pack=False, align=None), "cgaCollisionCount": SimTypeChar(label="Byte")}, name="IKEEXT_IPV6_CGA_AUTHENTICATION0", pack=False, align=None), "eapAuthentication": SimStruct({"flags": SimTypeInt(signed=False, label="IKEEXT_EAP_AUTHENTICATION_FLAGS")}, name="IKEEXT_EAP_AUTHENTICATION0", pack=False, align=None)}, name="<anon>", label="None")}, name="IKEEXT_AUTHENTICATION_METHOD1", pack=False, align=None), offset=0), "initiatorImpersonationType": SimTypeInt(signed=False, label="IKEEXT_AUTHENTICATION_IMPERSONATION_TYPE"), "numIkeProposals": SimTypeInt(signed=False, label="UInt32"), "ikeProposals": SimTypePointer(SimStruct({"cipherAlgorithm": SimStruct({"algoIdentifier": SimTypeInt(signed=False, label="IKEEXT_CIPHER_TYPE"), "keyLen": SimTypeInt(signed=False, label="UInt32"), "rounds": SimTypeInt(signed=False, label="UInt32")}, name="IKEEXT_CIPHER_ALGORITHM0", pack=False, align=None), "integrityAlgorithm": SimStruct({"algoIdentifier": SimTypeInt(signed=False, label="IKEEXT_INTEGRITY_TYPE")}, name="IKEEXT_INTEGRITY_ALGORITHM0", pack=False, align=None), "maxLifetimeSeconds": SimTypeInt(signed=False, label="UInt32"), "dhGroup": SimTypeInt(signed=False, label="IKEEXT_DH_GROUP"), "quickModeLimit": SimTypeInt(signed=False, label="UInt32")}, name="IKEEXT_PROPOSAL0", pack=False, align=None), offset=0), "flags": SimTypeInt(signed=False, label="IKEEXT_POLICY_FLAG"), "maxDynamicFilters": SimTypeInt(signed=False, label="UInt32"), "retransmitDurationSecs": SimTypeInt(signed=False, label="UInt32")}, name="IKEEXT_POLICY1", pack=False, align=None), offset=0), "idpOptions": SimTypePointer(SimStruct({"stateIdleTimeoutSeconds": SimTypeInt(signed=False, label="UInt32"), "perIPRateLimitQueueIdleTimeoutSeconds": SimTypeInt(signed=False, label="UInt32"), "ipV6IPsecUnauthDscp": SimTypeChar(label="Byte"), "ipV6IPsecUnauthRateLimitBytesPerSec": SimTypeInt(signed=False, label="UInt32"), "ipV6IPsecUnauthPerIPRateLimitBytesPerSec": SimTypeInt(signed=False, label="UInt32"), "ipV6IPsecAuthDscp": SimTypeChar(label="Byte"), "ipV6IPsecAuthRateLimitBytesPerSec": SimTypeInt(signed=False, label="UInt32"), "icmpV6Dscp": SimTypeChar(label="Byte"), "icmpV6RateLimitBytesPerSec": SimTypeInt(signed=False, label="UInt32"), "ipV6FilterExemptDscp": SimTypeChar(label="Byte"), "ipV6FilterExemptRateLimitBytesPerSec": SimTypeInt(signed=False, label="UInt32"), "defBlockExemptDscp": SimTypeChar(label="Byte"), "defBlockExemptRateLimitBytesPerSec": SimTypeInt(signed=False, label="UInt32"), "maxStateEntries": SimTypeInt(signed=False, label="UInt32"), "maxPerIPRateLimitQueues": SimTypeInt(signed=False, label="UInt32"), "flags": SimTypeInt(signed=False, label="IPSEC_DOSP_FLAGS"), "numPublicIFLuids": SimTypeInt(signed=False, label="UInt32"), "publicIFLuids": SimTypePointer(SimTypeLongLong(signed=False, label="UInt64"), offset=0), "numInternalIFLuids": SimTypeInt(signed=False, label="UInt32"), "internalIFLuids": SimTypePointer(SimTypeLongLong(signed=False, label="UInt64"), offset=0), "publicV6AddrMask": SimStruct({"addr": SimTypeFixedSizeArray(SimTypeChar(label="Byte"), 16), "prefixLength": SimTypeChar(label="Byte")}, name="FWP_V6_ADDR_AND_MASK", pack=False, align=None), "internalV6AddrMask": SimStruct({"addr": SimTypeFixedSizeArray(SimTypeChar(label="Byte"), 16), "prefixLength": SimTypeChar(label="Byte")}, name="FWP_V6_ADDR_AND_MASK", pack=False, align=None)}, name="IPSEC_DOSP_OPTIONS0", pack=False, align=None), offset=0)}, name="<anon>", label="None"), "providerContextId": SimTypeLongLong(signed=False, label="UInt64")}, name="FWPM_PROVIDER_CONTEXT1", pack=False, align=None), offset=0), SimTypePointer(SimStruct({"providerContextKey": SimTypeBottom(label="Guid"), "displayData": SimStruct({"name": SimTypePointer(SimTypeChar(label="Char"), offset=0), "description": SimTypePointer(SimTypeChar(label="Char"), offset=0)}, name="FWPM_DISPLAY_DATA0", pack=False, align=None), "flags": SimTypeInt(signed=False, label="UInt32"), "providerKey": SimTypePointer(SimTypeBottom(label="Guid"), offset=0), "providerData": SimStruct({"size": SimTypeInt(signed=False, label="UInt32"), "data": SimTypePointer(SimTypeChar(label="Byte"), offset=0)}, name="FWP_BYTE_BLOB", pack=False, align=None), "type": SimTypeInt(signed=False, label="FWPM_PROVIDER_CONTEXT_TYPE"), "Anonymous": SimUnion({"keyingPolicy": SimTypePointer(SimStruct({"numKeyMods": SimTypeInt(signed=False, label="UInt32"), "keyModKeys": SimTypePointer(SimTypeBottom(label="Guid"), offset=0)}, name="IPSEC_KEYING_POLICY0", pack=False, align=None), offset=0), "ikeQmTransportPolicy": SimTypePointer(SimStruct({"numIpsecProposals": SimTypeInt(signed=False, label="UInt32"), "ipsecProposals": SimTypePointer(SimStruct({"lifetime": SimStruct({"lifetimeSeconds": SimTypeInt(signed=False, label="UInt32"), "lifetimeKilobytes": SimTypeInt(signed=False, label="UInt32"), "lifetimePackets": SimTypeInt(signed=False, label="UInt32")}, name="IPSEC_SA_LIFETIME0", pack=False, align=None), "numSaTransforms": SimTypeInt(signed=False, label="UInt32"), "saTransforms": SimTypePointer(SimStruct({"ipsecTransformType": SimTypeInt(signed=False, label="IPSEC_TRANSFORM_TYPE"), "Anonymous": SimUnion({"ahTransform": SimTypePointer(SimStruct({"authTransformId": SimStruct({"authType": SimTypeInt(signed=False, label="IPSEC_AUTH_TYPE"), "authConfig": SimTypeChar(label="Byte")}, name="IPSEC_AUTH_TRANSFORM_ID0", pack=False, align=None), "cryptoModuleId": SimTypePointer(SimTypeBottom(label="Guid"), offset=0)}, name="IPSEC_AUTH_TRANSFORM0", pack=False, align=None), offset=0), "espAuthTransform": SimTypePointer(SimStruct({"authTransformId": SimStruct({"authType": SimTypeInt(signed=False, label="IPSEC_AUTH_TYPE"), "authConfig": SimTypeChar(label="Byte")}, name="IPSEC_AUTH_TRANSFORM_ID0", pack=False, align=None), "cryptoModuleId": SimTypePointer(SimTypeBottom(label="Guid"), offset=0)}, name="IPSEC_AUTH_TRANSFORM0", pack=False, align=None), offset=0), "espCipherTransform": SimTypePointer(SimStruct({"cipherTransformId": SimStruct({"cipherType": SimTypeInt(signed=False, label="IPSEC_CIPHER_TYPE"), "cipherConfig": SimTypeChar(label="Byte")}, name="IPSEC_CIPHER_TRANSFORM_ID0", pack=False, align=None), "cryptoModuleId": SimTypePointer(SimTypeBottom(label="Guid"), offset=0)}, name="IPSEC_CIPHER_TRANSFORM0", pack=False, align=None), offset=0), "espAuthAndCipherTransform": SimTypePointer(SimStruct({"authTransform": SimStruct({"authTransformId": SimStruct({"authType": SimTypeInt(signed=False, label="IPSEC_AUTH_TYPE"), "authConfig": SimTypeChar(label="Byte")}, name="IPSEC_AUTH_TRANSFORM_ID0", pack=False, align=None), "cryptoModuleId": SimTypePointer(SimTypeBottom(label="Guid"), offset=0)}, name="IPSEC_AUTH_TRANSFORM0", pack=False, align=None), "cipherTransform": SimStruct({"cipherTransformId": SimStruct({"cipherType": SimTypeInt(signed=False, label="IPSEC_CIPHER_TYPE"), "cipherConfig": SimTypeChar(label="Byte")}, name="IPSEC_CIPHER_TRANSFORM_ID0", pack=False, align=None), "cryptoModuleId": SimTypePointer(SimTypeBottom(label="Guid"), offset=0)}, name="IPSEC_CIPHER_TRANSFORM0", pack=False, align=None)}, name="IPSEC_AUTH_AND_CIPHER_TRANSFORM0", pack=False, align=None), offset=0), "espAuthFwTransform": SimTypePointer(SimStruct({"authTransformId": SimStruct({"authType": SimTypeInt(signed=False, label="IPSEC_AUTH_TYPE"), "authConfig": SimTypeChar(label="Byte")}, name="IPSEC_AUTH_TRANSFORM_ID0", pack=False, align=None), "cryptoModuleId": SimTypePointer(SimTypeBottom(label="Guid"), offset=0)}, name="IPSEC_AUTH_TRANSFORM0", pack=False, align=None), offset=0)}, name="<anon>", label="None")}, name="IPSEC_SA_TRANSFORM0", pack=False, align=None), offset=0), "pfsGroup": SimTypeInt(signed=False, label="IPSEC_PFS_GROUP")}, name="IPSEC_PROPOSAL0", pack=False, align=None), offset=0), "flags": SimTypeInt(signed=False, label="IPSEC_POLICY_FLAG"), "ndAllowClearTimeoutSeconds": SimTypeInt(signed=False, label="UInt32"), "saIdleTimeout": SimStruct({"idleTimeoutSeconds": SimTypeInt(signed=False, label="UInt32"), "idleTimeoutSecondsFailOver": SimTypeInt(signed=False, label="UInt32")}, name="IPSEC_SA_IDLE_TIMEOUT0", pack=False, align=None), "emPolicy": SimTypePointer(SimStruct({"numAuthenticationMethods": SimTypeInt(signed=False, label="UInt32"), "authenticationMethods": SimTypePointer(SimStruct({"authenticationMethodType": SimTypeInt(signed=False, label="IKEEXT_AUTHENTICATION_METHOD_TYPE"), "Anonymous": SimUnion({"presharedKeyAuthentication": SimStruct({"presharedKey": SimStruct({"size": SimTypeInt(signed=False, label="UInt32"), "data": SimTypePointer(SimTypeChar(label="Byte"), offset=0)}, name="FWP_BYTE_BLOB", pack=False, align=None), "flags": SimTypeInt(signed=False, label="IKEEXT_PRESHARED_KEY_AUTHENTICATION_FLAGS")}, name="IKEEXT_PRESHARED_KEY_AUTHENTICATION1", pack=False, align=None), "certificateAuthentication": SimStruct({"inboundConfigType": SimTypeInt(signed=False, label="IKEEXT_CERT_CONFIG_TYPE"), "Anonymous1": SimUnion({"Anonymous": SimStruct({"inboundRootArraySize": SimTypeInt(signed=False, label="UInt32"), "inboundRootArray": SimTypePointer(SimStruct({"certData": SimStruct({"size": SimTypeInt(signed=False, label="UInt32"), "data": SimTypePointer(SimTypeChar(label="Byte"), offset=0)}, name="FWP_BYTE_BLOB", pack=False, align=None), "flags": SimTypeInt(signed=False, label="IKEEXT_CERT_FLAGS")}, name="IKEEXT_CERT_ROOT_CONFIG0", pack=False, align=None), offset=0)}, name="_Anonymous_e__Struct", pack=False, align=None), "inboundEnterpriseStoreConfig": SimTypePointer(SimStruct({"certData": SimStruct({"size": SimTypeInt(signed=False, label="UInt32"), "data": SimTypePointer(SimTypeChar(label="Byte"), offset=0)}, name="FWP_BYTE_BLOB", pack=False, align=None), "flags": SimTypeInt(signed=False, label="IKEEXT_CERT_FLAGS")}, name="IKEEXT_CERT_ROOT_CONFIG0", pack=False, align=None), offset=0), "inboundTrustedRootStoreConfig": SimTypePointer(SimStruct({"certData": SimStruct({"size": SimTypeInt(signed=False, label="UInt32"), "data": SimTypePointer(SimTypeChar(label="Byte"), offset=0)}, name="FWP_BYTE_BLOB", pack=False, align=None), "flags": SimTypeInt(signed=False, label="IKEEXT_CERT_FLAGS")}, name="IKEEXT_CERT_ROOT_CONFIG0", pack=False, align=None), offset=0)}, name="<anon>", label="None"), "outboundConfigType": SimTypeInt(signed=False, label="IKEEXT_CERT_CONFIG_TYPE"), "Anonymous2": SimUnion({"Anonymous": SimStruct({"outboundRootArraySize": SimTypeInt(signed=False, label="UInt32"), "outboundRootArray": SimTypePointer(SimStruct({"certData": SimStruct({"size": SimTypeInt(signed=False, label="UInt32"), "data": SimTypePointer(SimTypeChar(label="Byte"), offset=0)}, name="FWP_BYTE_BLOB", pack=False, align=None), "flags": SimTypeInt(signed=False, label="IKEEXT_CERT_FLAGS")}, name="IKEEXT_CERT_ROOT_CONFIG0", pack=False, align=None), offset=0)}, name="_Anonymous_e__Struct", pack=False, align=None), "outboundEnterpriseStoreConfig": SimTypePointer(SimStruct({"certData": SimStruct({"size": SimTypeInt(signed=False, label="UInt32"), "data": SimTypePointer(SimTypeChar(label="Byte"), offset=0)}, name="FWP_BYTE_BLOB", pack=False, align=None), "flags": SimTypeInt(signed=False, label="IKEEXT_CERT_FLAGS")}, name="IKEEXT_CERT_ROOT_CONFIG0", pack=False, align=None), offset=0), "outboundTrustedRootStoreConfig": SimTypePointer(SimStruct({"certData": SimStruct({"size": SimTypeInt(signed=False, label="UInt32"), "data": SimTypePointer(SimTypeChar(label="Byte"), offset=0)}, name="FWP_BYTE_BLOB", pack=False, align=None), "flags": SimTypeInt(signed=False, label="IKEEXT_CERT_FLAGS")}, name="IKEEXT_CERT_ROOT_CONFIG0", pack=False, align=None), offset=0)}, name="<anon>", label="None"), "flags": SimTypeInt(signed=False, label="IKEEXT_CERT_AUTH"), "localCertLocationUrl": SimStruct({"size": SimTypeInt(signed=False, label="UInt32"), "data": SimTypePointer(SimTypeChar(label="Byte"), offset=0)}, name="FWP_BYTE_BLOB", pack=False, align=None)}, name="IKEEXT_CERTIFICATE_AUTHENTICATION1", pack=False, align=None), "kerberosAuthentication": SimStruct({"flags": SimTypeInt(signed=False, label="IKEEXT_KERBEROS_AUTHENTICATION_FLAGS")}, name="IKEEXT_KERBEROS_AUTHENTICATION0", pack=False, align=None), "ntlmV2Authentication": SimStruct({"flags": SimTypeInt(signed=False, label="UInt32")}, name="IKEEXT_NTLM_V2_AUTHENTICATION0", pack=False, align=None), "sslAuthentication": SimStruct({"inboundConfigType": SimTypeInt(signed=False, label="IKEEXT_CERT_CONFIG_TYPE"), "Anonymous1": SimUnion({"Anonymous": SimStruct({"inboundRootArraySize": SimTypeInt(signed=False, label="UInt32"), "inboundRootArray": SimTypePointer(SimStruct({"certData": SimStruct({"size": SimTypeInt(signed=False, label="UInt32"), "data": SimTypePointer(SimTypeChar(label="Byte"), offset=0)}, name="FWP_BYTE_BLOB", pack=False, align=None), "flags": SimTypeInt(signed=False, label="IKEEXT_CERT_FLAGS")}, name="IKEEXT_CERT_ROOT_CONFIG0", pack=False, align=None), offset=0)}, name="_Anonymous_e__Struct", pack=False, align=None), "inboundEnterpriseStoreConfig": SimTypePointer(SimStruct({"certData": SimStruct({"size": SimTypeInt(signed=False, label="UInt32"), "data": SimTypePointer(SimTypeChar(label="Byte"), offset=0)}, name="FWP_BYTE_BLOB", pack=False, align=None), "flags": SimTypeInt(signed=False, label="IKEEXT_CERT_FLAGS")}, name="IKEEXT_CERT_ROOT_CONFIG0", pack=False, align=None), offset=0), "inboundTrustedRootStoreConfig": SimTypePointer(SimStruct({"certData": SimStruct({"size": SimTypeInt(signed=False, label="UInt32"), "data": SimTypePointer(SimTypeChar(label="Byte"), offset=0)}, name="FWP_BYTE_BLOB", pack=False, align=None), "flags": SimTypeInt(signed=False, label="IKEEXT_CERT_FLAGS")}, name="IKEEXT_CERT_ROOT_CONFIG0", pack=False, align=None), offset=0)}, name="<anon>", label="None"), "outboundConfigType": SimTypeInt(signed=False, label="IKEEXT_CERT_CONFIG_TYPE"), "Anonymous2": SimUnion({"Anonymous": SimStruct({"outboundRootArraySize": SimTypeInt(signed=False, label="UInt32"), "outboundRootArray": SimTypePointer(SimStruct({"certData": SimStruct({"size": SimTypeInt(signed=False, label="UInt32"), "data": SimTypePointer(SimTypeChar(label="Byte"), offset=0)}, name="FWP_BYTE_BLOB", pack=False, align=None), "flags": SimTypeInt(signed=False, label="IKEEXT_CERT_FLAGS")}, name="IKEEXT_CERT_ROOT_CONFIG0", pack=False, align=None), offset=0)}, name="_Anonymous_e__Struct", pack=False, align=None), "outboundEnterpriseStoreConfig": SimTypePointer(SimStruct({"certData": SimStruct({"size": SimTypeInt(signed=False, label="UInt32"), "data": SimTypePointer(SimTypeChar(label="Byte"), offset=0)}, name="FWP_BYTE_BLOB", pack=False, align=None), "flags": SimTypeInt(signed=False, label="IKEEXT_CERT_FLAGS")}, name="IKEEXT_CERT_ROOT_CONFIG0", pack=False, align=None), offset=0), "outboundTrustedRootStoreConfig": SimTypePointer(SimStruct({"certData": SimStruct({"size": SimTypeInt(signed=False, label="UInt32"), "data": SimTypePointer(SimTypeChar(label="Byte"), offset=0)}, name="FWP_BYTE_BLOB", pack=False, align=None), "flags": SimTypeInt(signed=False, label="IKEEXT_CERT_FLAGS")}, name="IKEEXT_CERT_ROOT_CONFIG0", pack=False, align=None), offset=0)}, name="<anon>", label="None"), "flags": SimTypeInt(signed=False, label="IKEEXT_CERT_AUTH"), "localCertLocationUrl": SimStruct({"size": SimTypeInt(signed=False, label="UInt32"), "data": SimTypePointer(SimTypeChar(label="Byte"), offset=0)}, name="FWP_BYTE_BLOB", pack=False, align=None)}, name="IKEEXT_CERTIFICATE_AUTHENTICATION1", pack=False, align=None), "cgaAuthentication": SimStruct({"keyContainerName": SimTypePointer(SimTypeChar(label="Char"), offset=0), "cspName": SimTypePointer(SimTypeChar(label="Char"), offset=0), "cspType": SimTypeInt(signed=False, label="UInt32"), "cgaModifier": SimStruct({"byteArray16": SimTypeFixedSizeArray(SimTypeChar(label="Byte"), 16)}, name="FWP_BYTE_ARRAY16", pack=False, align=None), "cgaCollisionCount": SimTypeChar(label="Byte")}, name="IKEEXT_IPV6_CGA_AUTHENTICATION0", pack=False, align=None), "eapAuthentication": SimStruct({"flags": SimTypeInt(signed=False, label="IKEEXT_EAP_AUTHENTICATION_FLAGS")}, name="IKEEXT_EAP_AUTHENTICATION0", pack=False, align=None)}, name="<anon>", label="None")}, name="IKEEXT_AUTHENTICATION_METHOD1", pack=False, align=None), offset=0), "initiatorImpersonationType": SimTypeInt(signed=False, label="IKEEXT_AUTHENTICATION_IMPERSONATION_TYPE")}, name="IKEEXT_EM_POLICY1", pack=False, align=None), offset=0)}, name="IPSEC_TRANSPORT_POLICY1", pack=False, align=None), offset=0), "ikeQmTunnelPolicy": SimTypePointer(SimStruct({"flags": SimTypeInt(signed=False, label="IPSEC_POLICY_FLAG"), "numIpsecProposals": SimTypeInt(signed=False, label="UInt32"), "ipsecProposals": SimTypePointer(SimStruct({"lifetime": SimStruct({"lifetimeSeconds": SimTypeInt(signed=False, label="UInt32"), "lifetimeKilobytes": SimTypeInt(signed=False, label="UInt32"), "lifetimePackets": SimTypeInt(signed=False, label="UInt32")}, name="IPSEC_SA_LIFETIME0", pack=False, align=None), "numSaTransforms": SimTypeInt(signed=False, label="UInt32"), "saTransforms": SimTypePointer(SimStruct({"ipsecTransformType": SimTypeInt(signed=False, label="IPSEC_TRANSFORM_TYPE"), "Anonymous": SimUnion({"ahTransform": SimTypePointer(SimStruct({"authTransformId": SimStruct({"authType": SimTypeInt(signed=False, label="IPSEC_AUTH_TYPE"), "authConfig": SimTypeChar(label="Byte")}, name="IPSEC_AUTH_TRANSFORM_ID0", pack=False, align=None), "cryptoModuleId": SimTypePointer(SimTypeBottom(label="Guid"), offset=0)}, name="IPSEC_AUTH_TRANSFORM0", pack=False, align=None), offset=0), "espAuthTransform": SimTypePointer(SimStruct({"authTransformId": SimStruct({"authType": SimTypeInt(signed=False, label="IPSEC_AUTH_TYPE"), "authConfig": SimTypeChar(label="Byte")}, name="IPSEC_AUTH_TRANSFORM_ID0", pack=False, align=None), "cryptoModuleId": SimTypePointer(SimTypeBottom(label="Guid"), offset=0)}, name="IPSEC_AUTH_TRANSFORM0", pack=False, align=None), offset=0), "espCipherTransform": SimTypePointer(SimStruct({"cipherTransformId": SimStruct({"cipherType": SimTypeInt(signed=False, label="IPSEC_CIPHER_TYPE"), "cipherConfig": SimTypeChar(label="Byte")}, name="IPSEC_CIPHER_TRANSFORM_ID0", pack=False, align=None), "cryptoModuleId": SimTypePointer(SimTypeBottom(label="Guid"), offset=0)}, name="IPSEC_CIPHER_TRANSFORM0", pack=False, align=None), offset=0), "espAuthAndCipherTransform": SimTypePointer(SimStruct({"authTransform": SimStruct({"authTransformId": SimStruct({"authType": SimTypeInt(signed=False, label="IPSEC_AUTH_TYPE"), "authConfig": SimTypeChar(label="Byte")}, name="IPSEC_AUTH_TRANSFORM_ID0", pack=False, align=None), "cryptoModuleId": SimTypePointer(SimTypeBottom(label="Guid"), offset=0)}, name="IPSEC_AUTH_TRANSFORM0", pack=False, align=None), "cipherTransform": SimStruct({"cipherTransformId": SimStruct({"cipherType": SimTypeInt(signed=False, label="IPSEC_CIPHER_TYPE"), "cipherConfig": SimTypeChar(label="Byte")}, name="IPSEC_CIPHER_TRANSFORM_ID0", pack=False, align=None), "cryptoModuleId": SimTypePointer(SimTypeBottom(label="Guid"), offset=0)}, name="IPSEC_CIPHER_TRANSFORM0", pack=False, align=None)}, name="IPSEC_AUTH_AND_CIPHER_TRANSFORM0", pack=False, align=None), offset=0), "espAuthFwTransform": SimTypePointer(SimStruct({"authTransformId": SimStruct({"authType": SimTypeInt(signed=False, label="IPSEC_AUTH_TYPE"), "authConfig": SimTypeChar(label="Byte")}, name="IPSEC_AUTH_TRANSFORM_ID0", pack=False, align=None), "cryptoModuleId": SimTypePointer(SimTypeBottom(label="Guid"), offset=0)}, name="IPSEC_AUTH_TRANSFORM0", pack=False, align=None), offset=0)}, name="<anon>", label="None")}, name="IPSEC_SA_TRANSFORM0", pack=False, align=None), offset=0), "pfsGroup": SimTypeInt(signed=False, label="IPSEC_PFS_GROUP")}, name="IPSEC_PROPOSAL0", pack=False, align=None), offset=0), "tunnelEndpoints": SimStruct({"ipVersion": SimTypeInt(signed=False, label="FWP_IP_VERSION"), "Anonymous1": SimUnion({"localV4Address": SimTypeInt(signed=False, label="UInt32"), "localV6Address": SimTypeFixedSizeArray(SimTypeChar(label="Byte"), 16)}, name="<anon>", label="None"), "Anonymous2": SimUnion({"remoteV4Address": SimTypeInt(signed=False, label="UInt32"), "remoteV6Address": SimTypeFixedSizeArray(SimTypeChar(label="Byte"), 16)}, name="<anon>", label="None"), "localIfLuid": SimTypeLongLong(signed=False, label="UInt64")}, name="IPSEC_TUNNEL_ENDPOINTS1", pack=False, align=None), "saIdleTimeout": SimStruct({"idleTimeoutSeconds": SimTypeInt(signed=False, label="UInt32"), "idleTimeoutSecondsFailOver": SimTypeInt(signed=False, label="UInt32")}, name="IPSEC_SA_IDLE_TIMEOUT0", pack=False, align=None), "emPolicy": SimTypePointer(SimStruct({"numAuthenticationMethods": SimTypeInt(signed=False, label="UInt32"), "authenticationMethods": SimTypePointer(SimStruct({"authenticationMethodType": SimTypeInt(signed=False, label="IKEEXT_AUTHENTICATION_METHOD_TYPE"), "Anonymous": SimUnion({"presharedKeyAuthentication": SimStruct({"presharedKey": SimStruct({"size": SimTypeInt(signed=False, label="UInt32"), "data": SimTypePointer(SimTypeChar(label="Byte"), offset=0)}, name="FWP_BYTE_BLOB", pack=False, align=None), "flags": SimTypeInt(signed=False, label="IKEEXT_PRESHARED_KEY_AUTHENTICATION_FLAGS")}, name="IKEEXT_PRESHARED_KEY_AUTHENTICATION1", pack=False, align=None), "certificateAuthentication": SimStruct({"inboundConfigType": SimTypeInt(signed=False, label="IKEEXT_CERT_CONFIG_TYPE"), "Anonymous1": SimUnion({"Anonymous": SimStruct({"inboundRootArraySize": SimTypeInt(signed=False, label="UInt32"), "inboundRootArray": SimTypePointer(SimStruct({"certData": SimStruct({"size": SimTypeInt(signed=False, label="UInt32"), "data": SimTypePointer(SimTypeChar(label="Byte"), offset=0)}, name="FWP_BYTE_BLOB", pack=False, align=None), "flags": SimTypeInt(signed=False, label="IKEEXT_CERT_FLAGS")}, name="IKEEXT_CERT_ROOT_CONFIG0", pack=False, align=None), offset=0)}, name="_Anonymous_e__Struct", pack=False, align=None), "inboundEnterpriseStoreConfig": SimTypePointer(SimStruct({"certData": SimStruct({"size": SimTypeInt(signed=False, label="UInt32"), "data": SimTypePointer(SimTypeChar(label="Byte"), offset=0)}, name="FWP_BYTE_BLOB", pack=False, align=None), "flags": SimTypeInt(signed=False, label="IKEEXT_CERT_FLAGS")}, name="IKEEXT_CERT_ROOT_CONFIG0", pack=False, align=None), offset=0), "inboundTrustedRootStoreConfig": SimTypePointer(SimStruct({"certData": SimStruct({"size": SimTypeInt(signed=False, label="UInt32"), "data": SimTypePointer(SimTypeChar(label="Byte"), offset=0)}, name="FWP_BYTE_BLOB", pack=False, align=None), "flags": SimTypeInt(signed=False, label="IKEEXT_CERT_FLAGS")}, name="IKEEXT_CERT_ROOT_CONFIG0", pack=False, align=None), offset=0)}, name="<anon>", label="None"), "outboundConfigType": SimTypeInt(signed=False, label="IKEEXT_CERT_CONFIG_TYPE"), "Anonymous2": SimUnion({"Anonymous": SimStruct({"outboundRootArraySize": SimTypeInt(signed=False, label="UInt32"), "outboundRootArray": SimTypePointer(SimStruct({"certData": SimStruct({"size": SimTypeInt(signed=False, label="UInt32"), "data": SimTypePointer(SimTypeChar(label="Byte"), offset=0)}, name="FWP_BYTE_BLOB", pack=False, align=None), "flags": SimTypeInt(signed=False, label="IKEEXT_CERT_FLAGS")}, name="IKEEXT_CERT_ROOT_CONFIG0", pack=False, align=None), offset=0)}, name="_Anonymous_e__Struct", pack=False, align=None), "outboundEnterpriseStoreConfig": SimTypePointer(SimStruct({"certData": SimStruct({"size": SimTypeInt(signed=False, label="UInt32"), "data": SimTypePointer(SimTypeChar(label="Byte"), offset=0)}, name="FWP_BYTE_BLOB", pack=False, align=None), "flags": SimTypeInt(signed=False, label="IKEEXT_CERT_FLAGS")}, name="IKEEXT_CERT_ROOT_CONFIG0", pack=False, align=None), offset=0), "outboundTrustedRootStoreConfig": SimTypePointer(SimStruct({"certData": SimStruct({"size": SimTypeInt(signed=False, label="UInt32"), "data": SimTypePointer(SimTypeChar(label="Byte"), offset=0)}, name="FWP_BYTE_BLOB", pack=False, align=None), "flags": SimTypeInt(signed=False, label="IKEEXT_CERT_FLAGS")}, name="IKEEXT_CERT_ROOT_CONFIG0", pack=False, align=None), offset=0)}, name="<anon>", label="None"), "flags": SimTypeInt(signed=False, label="IKEEXT_CERT_AUTH"), "localCertLocationUrl": SimStruct({"size": SimTypeInt(signed=False, label="UInt32"), "data": SimTypePointer(SimTypeChar(label="Byte"), offset=0)}, name="FWP_BYTE_BLOB", pack=False, align=None)}, name="IKEEXT_CERTIFICATE_AUTHENTICATION1", pack=False, align=None), "kerberosAuthentication": SimStruct({"flags": SimTypeInt(signed=False, label="IKEEXT_KERBEROS_AUTHENTICATION_FLAGS")}, name="IKEEXT_KERBEROS_AUTHENTICATION0", pack=False, align=None), "ntlmV2Authentication": SimStruct({"flags": SimTypeInt(signed=False, label="UInt32")}, name="IKEEXT_NTLM_V2_AUTHENTICATION0", pack=False, align=None), "sslAuthentication": SimStruct({"inboundConfigType": SimTypeInt(signed=False, label="IKEEXT_CERT_CONFIG_TYPE"), "Anonymous1": SimUnion({"Anonymous": SimStruct({"inboundRootArraySize": SimTypeInt(signed=False, label="UInt32"), "inboundRootArray": SimTypePointer(SimStruct({"certData": SimStruct({"size": SimTypeInt(signed=False, label="UInt32"), "data": SimTypePointer(SimTypeChar(label="Byte"), offset=0)}, name="FWP_BYTE_BLOB", pack=False, align=None), "flags": SimTypeInt(signed=False, label="IKEEXT_CERT_FLAGS")}, name="IKEEXT_CERT_ROOT_CONFIG0", pack=False, align=None), offset=0)}, name="_Anonymous_e__Struct", pack=False, align=None), "inboundEnterpriseStoreConfig": SimTypePointer(SimStruct({"certData": SimStruct({"size": SimTypeInt(signed=False, label="UInt32"), "data": SimTypePointer(SimTypeChar(label="Byte"), offset=0)}, name="FWP_BYTE_BLOB", pack=False, align=None), "flags": SimTypeInt(signed=False, label="IKEEXT_CERT_FLAGS")}, name="IKEEXT_CERT_ROOT_CONFIG0", pack=False, align=None), offset=0), "inboundTrustedRootStoreConfig": SimTypePointer(SimStruct({"certData": SimStruct({"size": SimTypeInt(signed=False, label="UInt32"), "data": SimTypePointer(SimTypeChar(label="Byte"), offset=0)}, name="FWP_BYTE_BLOB", pack=False, align=None), "flags": SimTypeInt(signed=False, label="IKEEXT_CERT_FLAGS")}, name="IKEEXT_CERT_ROOT_CONFIG0", pack=False, align=None), offset=0)}, name="<anon>", label="None"), "outboundConfigType": SimTypeInt(signed=False, label="IKEEXT_CERT_CONFIG_TYPE"), "Anonymous2": SimUnion({"Anonymous": SimStruct({"outboundRootArraySize": SimTypeInt(signed=False, label="UInt32"), "outboundRootArray": SimTypePointer(SimStruct({"certData": SimStruct({"size": SimTypeInt(signed=False, label="UInt32"), "data": SimTypePointer(SimTypeChar(label="Byte"), offset=0)}, name="FWP_BYTE_BLOB", pack=False, align=None), "flags": SimTypeInt(signed=False, label="IKEEXT_CERT_FLAGS")}, name="IKEEXT_CERT_ROOT_CONFIG0", pack=False, align=None), offset=0)}, name="_Anonymous_e__Struct", pack=False, align=None), "outboundEnterpriseStoreConfig": SimTypePointer(SimStruct({"certData": SimStruct({"size": SimTypeInt(signed=False, label="UInt32"), "data": SimTypePointer(SimTypeChar(label="Byte"), offset=0)}, name="FWP_BYTE_BLOB", pack=False, align=None), "flags": SimTypeInt(signed=False, label="IKEEXT_CERT_FLAGS")}, name="IKEEXT_CERT_ROOT_CONFIG0", pack=False, align=None), offset=0), "outboundTrustedRootStoreConfig": SimTypePointer(SimStruct({"certData": SimStruct({"size": SimTypeInt(signed=False, label="UInt32"), "data": SimTypePointer(SimTypeChar(label="Byte"), offset=0)}, name="FWP_BYTE_BLOB", pack=False, align=None), "flags": SimTypeInt(signed=False, label="IKEEXT_CERT_FLAGS")}, name="IKEEXT_CERT_ROOT_CONFIG0", pack=False, align=None), offset=0)}, name="<anon>", label="None"), "flags": SimTypeInt(signed=False, label="IKEEXT_CERT_AUTH"), "localCertLocationUrl": SimStruct({"size": SimTypeInt(signed=False, label="UInt32"), "data": SimTypePointer(SimTypeChar(label="Byte"), offset=0)}, name="FWP_BYTE_BLOB", pack=False, align=None)}, name="IKEEXT_CERTIFICATE_AUTHENTICATION1", pack=False, align=None), "cgaAuthentication": SimStruct({"keyContainerName": SimTypePointer(SimTypeChar(label="Char"), offset=0), "cspName": SimTypePointer(SimTypeChar(label="Char"), offset=0), "cspType": SimTypeInt(signed=False, label="UInt32"), "cgaModifier": SimStruct({"byteArray16": SimTypeFixedSizeArray(SimTypeChar(label="Byte"), 16)}, name="FWP_BYTE_ARRAY16", pack=False, align=None), "cgaCollisionCount": SimTypeChar(label="Byte")}, name="IKEEXT_IPV6_CGA_AUTHENTICATION0", pack=False, align=None), "eapAuthentication": SimStruct({"flags": SimTypeInt(signed=False, label="IKEEXT_EAP_AUTHENTICATION_FLAGS")}, name="IKEEXT_EAP_AUTHENTICATION0", pack=False, align=None)}, name="<anon>", label="None")}, name="IKEEXT_AUTHENTICATION_METHOD1", pack=False, align=None), offset=0), "initiatorImpersonationType": SimTypeInt(signed=False, label="IKEEXT_AUTHENTICATION_IMPERSONATION_TYPE")}, name="IKEEXT_EM_POLICY1", pack=False, align=None), offset=0)}, name="IPSEC_TUNNEL_POLICY1", pack=False, align=None), offset=0), "authipQmTransportPolicy": SimTypePointer(SimStruct({"numIpsecProposals": SimTypeInt(signed=False, label="UInt32"), "ipsecProposals": SimTypePointer(SimStruct({"lifetime": SimStruct({"lifetimeSeconds": SimTypeInt(signed=False, label="UInt32"), "lifetimeKilobytes": SimTypeInt(signed=False, label="UInt32"), "lifetimePackets": SimTypeInt(signed=False, label="UInt32")}, name="IPSEC_SA_LIFETIME0", pack=False, align=None), "numSaTransforms": SimTypeInt(signed=False, label="UInt32"), "saTransforms": SimTypePointer(SimStruct({"ipsecTransformType": SimTypeInt(signed=False, label="IPSEC_TRANSFORM_TYPE"), "Anonymous": SimUnion({"ahTransform": SimTypePointer(SimStruct({"authTransformId": SimStruct({"authType": SimTypeInt(signed=False, label="IPSEC_AUTH_TYPE"), "authConfig": SimTypeChar(label="Byte")}, name="IPSEC_AUTH_TRANSFORM_ID0", pack=False, align=None), "cryptoModuleId": SimTypePointer(SimTypeBottom(label="Guid"), offset=0)}, name="IPSEC_AUTH_TRANSFORM0", pack=False, align=None), offset=0), "espAuthTransform": SimTypePointer(SimStruct({"authTransformId": SimStruct({"authType": SimTypeInt(signed=False, label="IPSEC_AUTH_TYPE"), "authConfig": SimTypeChar(label="Byte")}, name="IPSEC_AUTH_TRANSFORM_ID0", pack=False, align=None), "cryptoModuleId": SimTypePointer(SimTypeBottom(label="Guid"), offset=0)}, name="IPSEC_AUTH_TRANSFORM0", pack=False, align=None), offset=0), "espCipherTransform": SimTypePointer(SimStruct({"cipherTransformId": SimStruct({"cipherType": SimTypeInt(signed=False, label="IPSEC_CIPHER_TYPE"), "cipherConfig": SimTypeChar(label="Byte")}, name="IPSEC_CIPHER_TRANSFORM_ID0", pack=False, align=None), "cryptoModuleId": SimTypePointer(SimTypeBottom(label="Guid"), offset=0)}, name="IPSEC_CIPHER_TRANSFORM0", pack=False, align=None), offset=0), "espAuthAndCipherTransform": SimTypePointer(SimStruct({"authTransform": SimStruct({"authTransformId": SimStruct({"authType": SimTypeInt(signed=False, label="IPSEC_AUTH_TYPE"), "authConfig": SimTypeChar(label="Byte")}, name="IPSEC_AUTH_TRANSFORM_ID0", pack=False, align=None), "cryptoModuleId": SimTypePointer(SimTypeBottom(label="Guid"), offset=0)}, name="IPSEC_AUTH_TRANSFORM0", pack=False, align=None), "cipherTransform": SimStruct({"cipherTransformId": SimStruct({"cipherType": SimTypeInt(signed=False, label="IPSEC_CIPHER_TYPE"), "cipherConfig": SimTypeChar(label="Byte")}, name="IPSEC_CIPHER_TRANSFORM_ID0", pack=False, align=None), "cryptoModuleId": SimTypePointer(SimTypeBottom(label="Guid"), offset=0)}, name="IPSEC_CIPHER_TRANSFORM0", pack=False, align=None)}, name="IPSEC_AUTH_AND_CIPHER_TRANSFORM0", pack=False, align=None), offset=0), "espAuthFwTransform": SimTypePointer(SimStruct({"authTransformId": SimStruct({"authType": SimTypeInt(signed=False, label="IPSEC_AUTH_TYPE"), "authConfig": SimTypeChar(label="Byte")}, name="IPSEC_AUTH_TRANSFORM_ID0", pack=False, align=None), "cryptoModuleId": SimTypePointer(SimTypeBottom(label="Guid"), offset=0)}, name="IPSEC_AUTH_TRANSFORM0", pack=False, align=None), offset=0)}, name="<anon>", label="None")}, name="IPSEC_SA_TRANSFORM0", pack=False, align=None), offset=0), "pfsGroup": SimTypeInt(signed=False, label="IPSEC_PFS_GROUP")}, name="IPSEC_PROPOSAL0", pack=False, align=None), offset=0), "flags": SimTypeInt(signed=False, label="IPSEC_POLICY_FLAG"), "ndAllowClearTimeoutSeconds": SimTypeInt(signed=False, label="UInt32"), "saIdleTimeout": SimStruct({"idleTimeoutSeconds": SimTypeInt(signed=False, label="UInt32"), "idleTimeoutSecondsFailOver": SimTypeInt(signed=False, label="UInt32")}, name="IPSEC_SA_IDLE_TIMEOUT0", pack=False, align=None), "emPolicy": SimTypePointer(SimStruct({"numAuthenticationMethods": SimTypeInt(signed=False, label="UInt32"), "authenticationMethods": SimTypePointer(SimStruct({"authenticationMethodType": SimTypeInt(signed=False, label="IKEEXT_AUTHENTICATION_METHOD_TYPE"), "Anonymous": SimUnion({"presharedKeyAuthentication": SimStruct({"presharedKey": SimStruct({"size": SimTypeInt(signed=False, label="UInt32"), "data": SimTypePointer(SimTypeChar(label="Byte"), offset=0)}, name="FWP_BYTE_BLOB", pack=False, align=None), "flags": SimTypeInt(signed=False, label="IKEEXT_PRESHARED_KEY_AUTHENTICATION_FLAGS")}, name="IKEEXT_PRESHARED_KEY_AUTHENTICATION1", pack=False, align=None), "certificateAuthentication": SimStruct({"inboundConfigType": SimTypeInt(signed=False, label="IKEEXT_CERT_CONFIG_TYPE"), "Anonymous1": SimUnion({"Anonymous": SimStruct({"inboundRootArraySize": SimTypeInt(signed=False, label="UInt32"), "inboundRootArray": SimTypePointer(SimStruct({"certData": SimStruct({"size": SimTypeInt(signed=False, label="UInt32"), "data": SimTypePointer(SimTypeChar(label="Byte"), offset=0)}, name="FWP_BYTE_BLOB", pack=False, align=None), "flags": SimTypeInt(signed=False, label="IKEEXT_CERT_FLAGS")}, name="IKEEXT_CERT_ROOT_CONFIG0", pack=False, align=None), offset=0)}, name="_Anonymous_e__Struct", pack=False, align=None), "inboundEnterpriseStoreConfig": SimTypePointer(SimStruct({"certData": SimStruct({"size": SimTypeInt(signed=False, label="UInt32"), "data": SimTypePointer(SimTypeChar(label="Byte"), offset=0)}, name="FWP_BYTE_BLOB", pack=False, align=None), "flags": SimTypeInt(signed=False, label="IKEEXT_CERT_FLAGS")}, name="IKEEXT_CERT_ROOT_CONFIG0", pack=False, align=None), offset=0), "inboundTrustedRootStoreConfig": SimTypePointer(SimStruct({"certData": SimStruct({"size": SimTypeInt(signed=False, label="UInt32"), "data": SimTypePointer(SimTypeChar(label="Byte"), offset=0)}, name="FWP_BYTE_BLOB", pack=False, align=None), "flags": SimTypeInt(signed=False, label="IKEEXT_CERT_FLAGS")}, name="IKEEXT_CERT_ROOT_CONFIG0", pack=False, align=None), offset=0)}, name="<anon>", label="None"), "outboundConfigType": SimTypeInt(signed=False, label="IKEEXT_CERT_CONFIG_TYPE"), "Anonymous2": SimUnion({"Anonymous": SimStruct({"outboundRootArraySize": SimTypeInt(signed=False, label="UInt32"), "outboundRootArray": SimTypePointer(SimStruct({"certData": SimStruct({"size": SimTypeInt(signed=False, label="UInt32"), "data": SimTypePointer(SimTypeChar(label="Byte"), offset=0)}, name="FWP_BYTE_BLOB", pack=False, align=None), "flags": SimTypeInt(signed=False, label="IKEEXT_CERT_FLAGS")}, name="IKEEXT_CERT_ROOT_CONFIG0", pack=False, align=None), offset=0)}, name="_Anonymous_e__Struct", pack=False, align=None), "outboundEnterpriseStoreConfig": SimTypePointer(SimStruct({"certData": SimStruct({"size": SimTypeInt(signed=False, label="UInt32"), "data": SimTypePointer(SimTypeChar(label="Byte"), offset=0)}, name="FWP_BYTE_BLOB", pack=False, align=None), "flags": SimTypeInt(signed=False, label="IKEEXT_CERT_FLAGS")}, name="IKEEXT_CERT_ROOT_CONFIG0", pack=False, align=None), offset=0), "outboundTrustedRootStoreConfig": SimTypePointer(SimStruct({"certData": SimStruct({"size": SimTypeInt(signed=False, label="UInt32"), "data": SimTypePointer(SimTypeChar(label="Byte"), offset=0)}, name="FWP_BYTE_BLOB", pack=False, align=None), "flags": SimTypeInt(signed=False, label="IKEEXT_CERT_FLAGS")}, name="IKEEXT_CERT_ROOT_CONFIG0", pack=False, align=None), offset=0)}, name="<anon>", label="None"), "flags": SimTypeInt(signed=False, label="IKEEXT_CERT_AUTH"), "localCertLocationUrl": SimStruct({"size": SimTypeInt(signed=False, label="UInt32"), "data": SimTypePointer(SimTypeChar(label="Byte"), offset=0)}, name="FWP_BYTE_BLOB", pack=False, align=None)}, name="IKEEXT_CERTIFICATE_AUTHENTICATION1", pack=False, align=None), "kerberosAuthentication": SimStruct({"flags": SimTypeInt(signed=False, label="IKEEXT_KERBEROS_AUTHENTICATION_FLAGS")}, name="IKEEXT_KERBEROS_AUTHENTICATION0", pack=False, align=None), "ntlmV2Authentication": SimStruct({"flags": SimTypeInt(signed=False, label="UInt32")}, name="IKEEXT_NTLM_V2_AUTHENTICATION0", pack=False, align=None), "sslAuthentication": SimStruct({"inboundConfigType": SimTypeInt(signed=False, label="IKEEXT_CERT_CONFIG_TYPE"), "Anonymous1": SimUnion({"Anonymous": SimStruct({"inboundRootArraySize": SimTypeInt(signed=False, label="UInt32"), "inboundRootArray": SimTypePointer(SimStruct({"certData": SimStruct({"size": SimTypeInt(signed=False, label="UInt32"), "data": SimTypePointer(SimTypeChar(label="Byte"), offset=0)}, name="FWP_BYTE_BLOB", pack=False, align=None), "flags": SimTypeInt(signed=False, label="IKEEXT_CERT_FLAGS")}, name="IKEEXT_CERT_ROOT_CONFIG0", pack=False, align=None), offset=0)}, name="_Anonymous_e__Struct", pack=False, align=None), "inboundEnterpriseStoreConfig": SimTypePointer(SimStruct({"certData": SimStruct({"size": SimTypeInt(signed=False, label="UInt32"), "data": SimTypePointer(SimTypeChar(label="Byte"), offset=0)}, name="FWP_BYTE_BLOB", pack=False, align=None), "flags": SimTypeInt(signed=False, label="IKEEXT_CERT_FLAGS")}, name="IKEEXT_CERT_ROOT_CONFIG0", pack=False, align=None), offset=0), "inboundTrustedRootStoreConfig": SimTypePointer(SimStruct({"certData": SimStruct({"size": SimTypeInt(signed=False, label="UInt32"), "data": SimTypePointer(SimTypeChar(label="Byte"), offset=0)}, name="FWP_BYTE_BLOB", pack=False, align=None), "flags": SimTypeInt(signed=False, label="IKEEXT_CERT_FLAGS")}, name="IKEEXT_CERT_ROOT_CONFIG0", pack=False, align=None), offset=0)}, name="<anon>", label="None"), "outboundConfigType": SimTypeInt(signed=False, label="IKEEXT_CERT_CONFIG_TYPE"), "Anonymous2": SimUnion({"Anonymous": SimStruct({"outboundRootArraySize": SimTypeInt(signed=False, label="UInt32"), "outboundRootArray": SimTypePointer(SimStruct({"certData": SimStruct({"size": SimTypeInt(signed=False, label="UInt32"), "data": SimTypePointer(SimTypeChar(label="Byte"), offset=0)}, name="FWP_BYTE_BLOB", pack=False, align=None), "flags": SimTypeInt(signed=False, label="IKEEXT_CERT_FLAGS")}, name="IKEEXT_CERT_ROOT_CONFIG0", pack=False, align=None), offset=0)}, name="_Anonymous_e__Struct", pack=False, align=None), "outboundEnterpriseStoreConfig": SimTypePointer(SimStruct({"certData": SimStruct({"size": SimTypeInt(signed=False, label="UInt32"), "data": SimTypePointer(SimTypeChar(label="Byte"), offset=0)}, name="FWP_BYTE_BLOB", pack=False, align=None), "flags": SimTypeInt(signed=False, label="IKEEXT_CERT_FLAGS")}, name="IKEEXT_CERT_ROOT_CONFIG0", pack=False, align=None), offset=0), "outboundTrustedRootStoreConfig": SimTypePointer(SimStruct({"certData": SimStruct({"size": SimTypeInt(signed=False, label="UInt32"), "data": SimTypePointer(SimTypeChar(label="Byte"), offset=0)}, name="FWP_BYTE_BLOB", pack=False, align=None), "flags": SimTypeInt(signed=False, label="IKEEXT_CERT_FLAGS")}, name="IKEEXT_CERT_ROOT_CONFIG0", pack=False, align=None), offset=0)}, name="<anon>", label="None"), "flags": SimTypeInt(signed=False, label="IKEEXT_CERT_AUTH"), "localCertLocationUrl": SimStruct({"size": SimTypeInt(signed=False, label="UInt32"), "data": SimTypePointer(SimTypeChar(label="Byte"), offset=0)}, name="FWP_BYTE_BLOB", pack=False, align=None)}, name="IKEEXT_CERTIFICATE_AUTHENTICATION1", pack=False, align=None), "cgaAuthentication": SimStruct({"keyContainerName": SimTypePointer(SimTypeChar(label="Char"), offset=0), "cspName": SimTypePointer(SimTypeChar(label="Char"), offset=0), "cspType": SimTypeInt(signed=False, label="UInt32"), "cgaModifier": SimStruct({"byteArray16": SimTypeFixedSizeArray(SimTypeChar(label="Byte"), 16)}, name="FWP_BYTE_ARRAY16", pack=False, align=None), "cgaCollisionCount": SimTypeChar(label="Byte")}, name="IKEEXT_IPV6_CGA_AUTHENTICATION0", pack=False, align=None), "eapAuthentication": SimStruct({"flags": SimTypeInt(signed=False, label="IKEEXT_EAP_AUTHENTICATION_FLAGS")}, name="IKEEXT_EAP_AUTHENTICATION0", pack=False, align=None)}, name="<anon>", label="None")}, name="IKEEXT_AUTHENTICATION_METHOD1", pack=False, align=None), offset=0), "initiatorImpersonationType": SimTypeInt(signed=False, label="IKEEXT_AUTHENTICATION_IMPERSONATION_TYPE")}, name="IKEEXT_EM_POLICY1", pack=False, align=None), offset=0)}, name="IPSEC_TRANSPORT_POLICY1", pack=False, align=None), offset=0), "authipQmTunnelPolicy": SimTypePointer(SimStruct({"flags": SimTypeInt(signed=False, label="IPSEC_POLICY_FLAG"), "numIpsecProposals": SimTypeInt(signed=False, label="UInt32"), "ipsecProposals": SimTypePointer(SimStruct({"lifetime": SimStruct({"lifetimeSeconds": SimTypeInt(signed=False, label="UInt32"), "lifetimeKilobytes": SimTypeInt(signed=False, label="UInt32"), "lifetimePackets": SimTypeInt(signed=False, label="UInt32")}, name="IPSEC_SA_LIFETIME0", pack=False, align=None), "numSaTransforms": SimTypeInt(signed=False, label="UInt32"), "saTransforms": SimTypePointer(SimStruct({"ipsecTransformType": SimTypeInt(signed=False, label="IPSEC_TRANSFORM_TYPE"), "Anonymous": SimUnion({"ahTransform": SimTypePointer(SimStruct({"authTransformId": SimStruct({"authType": SimTypeInt(signed=False, label="IPSEC_AUTH_TYPE"), "authConfig": SimTypeChar(label="Byte")}, name="IPSEC_AUTH_TRANSFORM_ID0", pack=False, align=None), "cryptoModuleId": SimTypePointer(SimTypeBottom(label="Guid"), offset=0)}, name="IPSEC_AUTH_TRANSFORM0", pack=False, align=None), offset=0), "espAuthTransform": SimTypePointer(SimStruct({"authTransformId": SimStruct({"authType": SimTypeInt(signed=False, label="IPSEC_AUTH_TYPE"), "authConfig": SimTypeChar(label="Byte")}, name="IPSEC_AUTH_TRANSFORM_ID0", pack=False, align=None), "cryptoModuleId": SimTypePointer(SimTypeBottom(label="Guid"), offset=0)}, name="IPSEC_AUTH_TRANSFORM0", pack=False, align=None), offset=0), "espCipherTransform": SimTypePointer(SimStruct({"cipherTransformId": SimStruct({"cipherType": SimTypeInt(signed=False, label="IPSEC_CIPHER_TYPE"), "cipherConfig": SimTypeChar(label="Byte")}, name="IPSEC_CIPHER_TRANSFORM_ID0", pack=False, align=None), "cryptoModuleId": SimTypePointer(SimTypeBottom(label="Guid"), offset=0)}, name="IPSEC_CIPHER_TRANSFORM0", pack=False, align=None), offset=0), "espAuthAndCipherTransform": SimTypePointer(SimStruct({"authTransform": SimStruct({"authTransformId": SimStruct({"authType": SimTypeInt(signed=False, label="IPSEC_AUTH_TYPE"), "authConfig": SimTypeChar(label="Byte")}, name="IPSEC_AUTH_TRANSFORM_ID0", pack=False, align=None), "cryptoModuleId": SimTypePointer(SimTypeBottom(label="Guid"), offset=0)}, name="IPSEC_AUTH_TRANSFORM0", pack=False, align=None), "cipherTransform": SimStruct({"cipherTransformId": SimStruct({"cipherType": SimTypeInt(signed=False, label="IPSEC_CIPHER_TYPE"), "cipherConfig": SimTypeChar(label="Byte")}, name="IPSEC_CIPHER_TRANSFORM_ID0", pack=False, align=None), "cryptoModuleId": SimTypePointer(SimTypeBottom(label="Guid"), offset=0)}, name="IPSEC_CIPHER_TRANSFORM0", pack=False, align=None)}, name="IPSEC_AUTH_AND_CIPHER_TRANSFORM0", pack=False, align=None), offset=0), "espAuthFwTransform": SimTypePointer(SimStruct({"authTransformId": SimStruct({"authType": SimTypeInt(signed=False, label="IPSEC_AUTH_TYPE"), "authConfig": SimTypeChar(label="Byte")}, name="IPSEC_AUTH_TRANSFORM_ID0", pack=False, align=None), "cryptoModuleId": SimTypePointer(SimTypeBottom(label="Guid"), offset=0)}, name="IPSEC_AUTH_TRANSFORM0", pack=False, align=None), offset=0)}, name="<anon>", label="None")}, name="IPSEC_SA_TRANSFORM0", pack=False, align=None), offset=0), "pfsGroup": SimTypeInt(signed=False, label="IPSEC_PFS_GROUP")}, name="IPSEC_PROPOSAL0", pack=False, align=None), offset=0), "tunnelEndpoints": SimStruct({"ipVersion": SimTypeInt(signed=False, label="FWP_IP_VERSION"), "Anonymous1": SimUnion({"localV4Address": SimTypeInt(signed=False, label="UInt32"), "localV6Address": SimTypeFixedSizeArray(SimTypeChar(label="Byte"), 16)}, name="<anon>", label="None"), "Anonymous2": SimUnion({"remoteV4Address": SimTypeInt(signed=False, label="UInt32"), "remoteV6Address": SimTypeFixedSizeArray(SimTypeChar(label="Byte"), 16)}, name="<anon>", label="None"), "localIfLuid": SimTypeLongLong(signed=False, label="UInt64")}, name="IPSEC_TUNNEL_ENDPOINTS1", pack=False, align=None), "saIdleTimeout": SimStruct({"idleTimeoutSeconds": SimTypeInt(signed=False, label="UInt32"), "idleTimeoutSecondsFailOver": SimTypeInt(signed=False, label="UInt32")}, name="IPSEC_SA_IDLE_TIMEOUT0", pack=False, align=None), "emPolicy": SimTypePointer(SimStruct({"numAuthenticationMethods": SimTypeInt(signed=False, label="UInt32"), "authenticationMethods": SimTypePointer(SimStruct({"authenticationMethodType": SimTypeInt(signed=False, label="IKEEXT_AUTHENTICATION_METHOD_TYPE"), "Anonymous": SimUnion({"presharedKeyAuthentication": SimStruct({"presharedKey": SimStruct({"size": SimTypeInt(signed=False, label="UInt32"), "data": SimTypePointer(SimTypeChar(label="Byte"), offset=0)}, name="FWP_BYTE_BLOB", pack=False, align=None), "flags": SimTypeInt(signed=False, label="IKEEXT_PRESHARED_KEY_AUTHENTICATION_FLAGS")}, name="IKEEXT_PRESHARED_KEY_AUTHENTICATION1", pack=False, align=None), "certificateAuthentication": SimStruct({"inboundConfigType": SimTypeInt(signed=False, label="IKEEXT_CERT_CONFIG_TYPE"), "Anonymous1": SimUnion({"Anonymous": SimStruct({"inboundRootArraySize": SimTypeInt(signed=False, label="UInt32"), "inboundRootArray": SimTypePointer(SimStruct({"certData": SimStruct({"size": SimTypeInt(signed=False, label="UInt32"), "data": SimTypePointer(SimTypeChar(label="Byte"), offset=0)}, name="FWP_BYTE_BLOB", pack=False, align=None), "flags": SimTypeInt(signed=False, label="IKEEXT_CERT_FLAGS")}, name="IKEEXT_CERT_ROOT_CONFIG0", pack=False, align=None), offset=0)}, name="_Anonymous_e__Struct", pack=False, align=None), "inboundEnterpriseStoreConfig": SimTypePointer(SimStruct({"certData": SimStruct({"size": SimTypeInt(signed=False, label="UInt32"), "data": SimTypePointer(SimTypeChar(label="Byte"), offset=0)}, name="FWP_BYTE_BLOB", pack=False, align=None), "flags": SimTypeInt(signed=False, label="IKEEXT_CERT_FLAGS")}, name="IKEEXT_CERT_ROOT_CONFIG0", pack=False, align=None), offset=0), "inboundTrustedRootStoreConfig": SimTypePointer(SimStruct({"certData": SimStruct({"size": SimTypeInt(signed=False, label="UInt32"), "data": SimTypePointer(SimTypeChar(label="Byte"), offset=0)}, name="FWP_BYTE_BLOB", pack=False, align=None), "flags": SimTypeInt(signed=False, label="IKEEXT_CERT_FLAGS")}, name="IKEEXT_CERT_ROOT_CONFIG0", pack=False, align=None), offset=0)}, name="<anon>", label="None"), "outboundConfigType": SimTypeInt(signed=False, label="IKEEXT_CERT_CONFIG_TYPE"), "Anonymous2": SimUnion({"Anonymous": SimStruct({"outboundRootArraySize": SimTypeInt(signed=False, label="UInt32"), "outboundRootArray": SimTypePointer(SimStruct({"certData": SimStruct({"size": SimTypeInt(signed=False, label="UInt32"), "data": SimTypePointer(SimTypeChar(label="Byte"), offset=0)}, name="FWP_BYTE_BLOB", pack=False, align=None), "flags": SimTypeInt(signed=False, label="IKEEXT_CERT_FLAGS")}, name="IKEEXT_CERT_ROOT_CONFIG0", pack=False, align=None), offset=0)}, name="_Anonymous_e__Struct", pack=False, align=None), "outboundEnterpriseStoreConfig": SimTypePointer(SimStruct({"certData": SimStruct({"size": SimTypeInt(signed=False, label="UInt32"), "data": SimTypePointer(SimTypeChar(label="Byte"), offset=0)}, name="FWP_BYTE_BLOB", pack=False, align=None), "flags": SimTypeInt(signed=False, label="IKEEXT_CERT_FLAGS")}, name="IKEEXT_CERT_ROOT_CONFIG0", pack=False, align=None), offset=0), "outboundTrustedRootStoreConfig": SimTypePointer(SimStruct({"certData": SimStruct({"size": SimTypeInt(signed=False, label="UInt32"), "data": SimTypePointer(SimTypeChar(label="Byte"), offset=0)}, name="FWP_BYTE_BLOB", pack=False, align=None), "flags": SimTypeInt(signed=False, label="IKEEXT_CERT_FLAGS")}, name="IKEEXT_CERT_ROOT_CONFIG0", pack=False, align=None), offset=0)}, name="<anon>", label="None"), "flags": SimTypeInt(signed=False, label="IKEEXT_CERT_AUTH"), "localCertLocationUrl": SimStruct({"size": SimTypeInt(signed=False, label="UInt32"), "data": SimTypePointer(SimTypeChar(label="Byte"), offset=0)}, name="FWP_BYTE_BLOB", pack=False, align=None)}, name="IKEEXT_CERTIFICATE_AUTHENTICATION1", pack=False, align=None), "kerberosAuthentication": SimStruct({"flags": SimTypeInt(signed=False, label="IKEEXT_KERBEROS_AUTHENTICATION_FLAGS")}, name="IKEEXT_KERBEROS_AUTHENTICATION0", pack=False, align=None), "ntlmV2Authentication": SimStruct({"flags": SimTypeInt(signed=False, label="UInt32")}, name="IKEEXT_NTLM_V2_AUTHENTICATION0", pack=False, align=None), "sslAuthentication": SimStruct({"inboundConfigType": SimTypeInt(signed=False, label="IKEEXT_CERT_CONFIG_TYPE"), "Anonymous1": SimUnion({"Anonymous": SimStruct({"inboundRootArraySize": SimTypeInt(signed=False, label="UInt32"), "inboundRootArray": SimTypePointer(SimStruct({"certData": SimStruct({"size": SimTypeInt(signed=False, label="UInt32"), "data": SimTypePointer(SimTypeChar(label="Byte"), offset=0)}, name="FWP_BYTE_BLOB", pack=False, align=None), "flags": SimTypeInt(signed=False, label="IKEEXT_CERT_FLAGS")}, name="IKEEXT_CERT_ROOT_CONFIG0", pack=False, align=None), offset=0)}, name="_Anonymous_e__Struct", pack=False, align=None), "inboundEnterpriseStoreConfig": SimTypePointer(SimStruct({"certData": SimStruct({"size": SimTypeInt(signed=False, label="UInt32"), "data": SimTypePointer(SimTypeChar(label="Byte"), offset=0)}, name="FWP_BYTE_BLOB", pack=False, align=None), "flags": SimTypeInt(signed=False, label="IKEEXT_CERT_FLAGS")}, name="IKEEXT_CERT_ROOT_CONFIG0", pack=False, align=None), offset=0), "inboundTrustedRootStoreConfig": SimTypePointer(SimStruct({"certData": SimStruct({"size": SimTypeInt(signed=False, label="UInt32"), "data": SimTypePointer(SimTypeChar(label="Byte"), offset=0)}, name="FWP_BYTE_BLOB", pack=False, align=None), "flags": SimTypeInt(signed=False, label="IKEEXT_CERT_FLAGS")}, name="IKEEXT_CERT_ROOT_CONFIG0", pack=False, align=None), offset=0)}, name="<anon>", label="None"), "outboundConfigType": SimTypeInt(signed=False, label="IKEEXT_CERT_CONFIG_TYPE"), "Anonymous2": SimUnion({"Anonymous": SimStruct({"outboundRootArraySize": SimTypeInt(signed=False, label="UInt32"), "outboundRootArray": SimTypePointer(SimStruct({"certData": SimStruct({"size": SimTypeInt(signed=False, label="UInt32"), "data": SimTypePointer(SimTypeChar(label="Byte"), offset=0)}, name="FWP_BYTE_BLOB", pack=False, align=None), "flags": SimTypeInt(signed=False, label="IKEEXT_CERT_FLAGS")}, name="IKEEXT_CERT_ROOT_CONFIG0", pack=False, align=None), offset=0)}, name="_Anonymous_e__Struct", pack=False, align=None), "outboundEnterpriseStoreConfig": SimTypePointer(SimStruct({"certData": SimStruct({"size": SimTypeInt(signed=False, label="UInt32"), "data": SimTypePointer(SimTypeChar(label="Byte"), offset=0)}, name="FWP_BYTE_BLOB", pack=False, align=None), "flags": SimTypeInt(signed=False, label="IKEEXT_CERT_FLAGS")}, name="IKEEXT_CERT_ROOT_CONFIG0", pack=False, align=None), offset=0), "outboundTrustedRootStoreConfig": SimTypePointer(SimStruct({"certData": SimStruct({"size": SimTypeInt(signed=False, label="UInt32"), "data": SimTypePointer(SimTypeChar(label="Byte"), offset=0)}, name="FWP_BYTE_BLOB", pack=False, align=None), "flags": SimTypeInt(signed=False, label="IKEEXT_CERT_FLAGS")}, name="IKEEXT_CERT_ROOT_CONFIG0", pack=False, align=None), offset=0)}, name="<anon>", label="None"), "flags": SimTypeInt(signed=False, label="IKEEXT_CERT_AUTH"), "localCertLocationUrl": SimStruct({"size": SimTypeInt(signed=False, label="UInt32"), "data": SimTypePointer(SimTypeChar(label="Byte"), offset=0)}, name="FWP_BYTE_BLOB", pack=False, align=None)}, name="IKEEXT_CERTIFICATE_AUTHENTICATION1", pack=False, align=None), "cgaAuthentication": SimStruct({"keyContainerName": SimTypePointer(SimTypeChar(label="Char"), offset=0), "cspName": SimTypePointer(SimTypeChar(label="Char"), offset=0), "cspType": SimTypeInt(signed=False, label="UInt32"), "cgaModifier": SimStruct({"byteArray16": SimTypeFixedSizeArray(SimTypeChar(label="Byte"), 16)}, name="FWP_BYTE_ARRAY16", pack=False, align=None), "cgaCollisionCount": SimTypeChar(label="Byte")}, name="IKEEXT_IPV6_CGA_AUTHENTICATION0", pack=False, align=None), "eapAuthentication": SimStruct({"flags": SimTypeInt(signed=False, label="IKEEXT_EAP_AUTHENTICATION_FLAGS")}, name="IKEEXT_EAP_AUTHENTICATION0", pack=False, align=None)}, name="<anon>", label="None")}, name="IKEEXT_AUTHENTICATION_METHOD1", pack=False, align=None), offset=0), "initiatorImpersonationType": SimTypeInt(signed=False, label="IKEEXT_AUTHENTICATION_IMPERSONATION_TYPE")}, name="IKEEXT_EM_POLICY1", pack=False, align=None), offset=0)}, name="IPSEC_TUNNEL_POLICY1", pack=False, align=None), offset=0), "ikeMmPolicy": SimTypePointer(SimStruct({"softExpirationTime": SimTypeInt(signed=False, label="UInt32"), "numAuthenticationMethods": SimTypeInt(signed=False, label="UInt32"), "authenticationMethods": SimTypePointer(SimStruct({"authenticationMethodType": SimTypeInt(signed=False, label="IKEEXT_AUTHENTICATION_METHOD_TYPE"), "Anonymous": SimUnion({"presharedKeyAuthentication": SimStruct({"presharedKey": SimStruct({"size": SimTypeInt(signed=False, label="UInt32"), "data": SimTypePointer(SimTypeChar(label="Byte"), offset=0)}, name="FWP_BYTE_BLOB", pack=False, align=None), "flags": SimTypeInt(signed=False, label="IKEEXT_PRESHARED_KEY_AUTHENTICATION_FLAGS")}, name="IKEEXT_PRESHARED_KEY_AUTHENTICATION1", pack=False, align=None), "certificateAuthentication": SimStruct({"inboundConfigType": SimTypeInt(signed=False, label="IKEEXT_CERT_CONFIG_TYPE"), "Anonymous1": SimUnion({"Anonymous": SimStruct({"inboundRootArraySize": SimTypeInt(signed=False, label="UInt32"), "inboundRootArray": SimTypePointer(SimStruct({"certData": SimStruct({"size": SimTypeInt(signed=False, label="UInt32"), "data": SimTypePointer(SimTypeChar(label="Byte"), offset=0)}, name="FWP_BYTE_BLOB", pack=False, align=None), "flags": SimTypeInt(signed=False, label="IKEEXT_CERT_FLAGS")}, name="IKEEXT_CERT_ROOT_CONFIG0", pack=False, align=None), offset=0)}, name="_Anonymous_e__Struct", pack=False, align=None), "inboundEnterpriseStoreConfig": SimTypePointer(SimStruct({"certData": SimStruct({"size": SimTypeInt(signed=False, label="UInt32"), "data": SimTypePointer(SimTypeChar(label="Byte"), offset=0)}, name="FWP_BYTE_BLOB", pack=False, align=None), "flags": SimTypeInt(signed=False, label="IKEEXT_CERT_FLAGS")}, name="IKEEXT_CERT_ROOT_CONFIG0", pack=False, align=None), offset=0), "inboundTrustedRootStoreConfig": SimTypePointer(SimStruct({"certData": SimStruct({"size": SimTypeInt(signed=False, label="UInt32"), "data": SimTypePointer(SimTypeChar(label="Byte"), offset=0)}, name="FWP_BYTE_BLOB", pack=False, align=None), "flags": SimTypeInt(signed=False, label="IKEEXT_CERT_FLAGS")}, name="IKEEXT_CERT_ROOT_CONFIG0", pack=False, align=None), offset=0)}, name="<anon>", label="None"), "outboundConfigType": SimTypeInt(signed=False, label="IKEEXT_CERT_CONFIG_TYPE"), "Anonymous2": SimUnion({"Anonymous": SimStruct({"outboundRootArraySize": SimTypeInt(signed=False, label="UInt32"), "outboundRootArray": SimTypePointer(SimStruct({"certData": SimStruct({"size": SimTypeInt(signed=False, label="UInt32"), "data": SimTypePointer(SimTypeChar(label="Byte"), offset=0)}, name="FWP_BYTE_BLOB", pack=False, align=None), "flags": SimTypeInt(signed=False, label="IKEEXT_CERT_FLAGS")}, name="IKEEXT_CERT_ROOT_CONFIG0", pack=False, align=None), offset=0)}, name="_Anonymous_e__Struct", pack=False, align=None), "outboundEnterpriseStoreConfig": SimTypePointer(SimStruct({"certData": SimStruct({"size": SimTypeInt(signed=False, label="UInt32"), "data": SimTypePointer(SimTypeChar(label="Byte"), offset=0)}, name="FWP_BYTE_BLOB", pack=False, align=None), "flags": SimTypeInt(signed=False, label="IKEEXT_CERT_FLAGS")}, name="IKEEXT_CERT_ROOT_CONFIG0", pack=False, align=None), offset=0), "outboundTrustedRootStoreConfig": SimTypePointer(SimStruct({"certData": SimStruct({"size": SimTypeInt(signed=False, label="UInt32"), "data": SimTypePointer(SimTypeChar(label="Byte"), offset=0)}, name="FWP_BYTE_BLOB", pack=False, align=None), "flags": SimTypeInt(signed=False, label="IKEEXT_CERT_FLAGS")}, name="IKEEXT_CERT_ROOT_CONFIG0", pack=False, align=None), offset=0)}, name="<anon>", label="None"), "flags": SimTypeInt(signed=False, label="IKEEXT_CERT_AUTH"), "localCertLocationUrl": SimStruct({"size": SimTypeInt(signed=False, label="UInt32"), "data": SimTypePointer(SimTypeChar(label="Byte"), offset=0)}, name="FWP_BYTE_BLOB", pack=False, align=None)}, name="IKEEXT_CERTIFICATE_AUTHENTICATION1", pack=False, align=None), "kerberosAuthentication": SimStruct({"flags": SimTypeInt(signed=False, label="IKEEXT_KERBEROS_AUTHENTICATION_FLAGS")}, name="IKEEXT_KERBEROS_AUTHENTICATION0", pack=False, align=None), "ntlmV2Authentication": SimStruct({"flags": SimTypeInt(signed=False, label="UInt32")}, name="IKEEXT_NTLM_V2_AUTHENTICATION0", pack=False, align=None), "sslAuthentication": SimStruct({"inboundConfigType": SimTypeInt(signed=False, label="IKEEXT_CERT_CONFIG_TYPE"), "Anonymous1": SimUnion({"Anonymous": SimStruct({"inboundRootArraySize": SimTypeInt(signed=False, label="UInt32"), "inboundRootArray": SimTypePointer(SimStruct({"certData": SimStruct({"size": SimTypeInt(signed=False, label="UInt32"), "data": SimTypePointer(SimTypeChar(label="Byte"), offset=0)}, name="FWP_BYTE_BLOB", pack=False, align=None), "flags": SimTypeInt(signed=False, label="IKEEXT_CERT_FLAGS")}, name="IKEEXT_CERT_ROOT_CONFIG0", pack=False, align=None), offset=0)}, name="_Anonymous_e__Struct", pack=False, align=None), "inboundEnterpriseStoreConfig": SimTypePointer(SimStruct({"certData": SimStruct({"size": SimTypeInt(signed=False, label="UInt32"), "data": SimTypePointer(SimTypeChar(label="Byte"), offset=0)}, name="FWP_BYTE_BLOB", pack=False, align=None), "flags": SimTypeInt(signed=False, label="IKEEXT_CERT_FLAGS")}, name="IKEEXT_CERT_ROOT_CONFIG0", pack=False, align=None), offset=0), "inboundTrustedRootStoreConfig": SimTypePointer(SimStruct({"certData": SimStruct({"size": SimTypeInt(signed=False, label="UInt32"), "data": SimTypePointer(SimTypeChar(label="Byte"), offset=0)}, name="FWP_BYTE_BLOB", pack=False, align=None), "flags": SimTypeInt(signed=False, label="IKEEXT_CERT_FLAGS")}, name="IKEEXT_CERT_ROOT_CONFIG0", pack=False, align=None), offset=0)}, name="<anon>", label="None"), "outboundConfigType": SimTypeInt(signed=False, label="IKEEXT_CERT_CONFIG_TYPE"), "Anonymous2": SimUnion({"Anonymous": SimStruct({"outboundRootArraySize": SimTypeInt(signed=False, label="UInt32"), "outboundRootArray": SimTypePointer(SimStruct({"certData": SimStruct({"size": SimTypeInt(signed=False, label="UInt32"), "data": SimTypePointer(SimTypeChar(label="Byte"), offset=0)}, name="FWP_BYTE_BLOB", pack=False, align=None), "flags": SimTypeInt(signed=False, label="IKEEXT_CERT_FLAGS")}, name="IKEEXT_CERT_ROOT_CONFIG0", pack=False, align=None), offset=0)}, name="_Anonymous_e__Struct", pack=False, align=None), "outboundEnterpriseStoreConfig": SimTypePointer(SimStruct({"certData": SimStruct({"size": SimTypeInt(signed=False, label="UInt32"), "data": SimTypePointer(SimTypeChar(label="Byte"), offset=0)}, name="FWP_BYTE_BLOB", pack=False, align=None), "flags": SimTypeInt(signed=False, label="IKEEXT_CERT_FLAGS")}, name="IKEEXT_CERT_ROOT_CONFIG0", pack=False, align=None), offset=0), "outboundTrustedRootStoreConfig": SimTypePointer(SimStruct({"certData": SimStruct({"size": SimTypeInt(signed=False, label="UInt32"), "data": SimTypePointer(SimTypeChar(label="Byte"), offset=0)}, name="FWP_BYTE_BLOB", pack=False, align=None), "flags": SimTypeInt(signed=False, label="IKEEXT_CERT_FLAGS")}, name="IKEEXT_CERT_ROOT_CONFIG0", pack=False, align=None), offset=0)}, name="<anon>", label="None"), "flags": SimTypeInt(signed=False, label="IKEEXT_CERT_AUTH"), "localCertLocationUrl": SimStruct({"size": SimTypeInt(signed=False, label="UInt32"), "data": SimTypePointer(SimTypeChar(label="Byte"), offset=0)}, name="FWP_BYTE_BLOB", pack=False, align=None)}, name="IKEEXT_CERTIFICATE_AUTHENTICATION1", pack=False, align=None), "cgaAuthentication": SimStruct({"keyContainerName": SimTypePointer(SimTypeChar(label="Char"), offset=0), "cspName": SimTypePointer(SimTypeChar(label="Char"), offset=0), "cspType": SimTypeInt(signed=False, label="UInt32"), "cgaModifier": SimStruct({"byteArray16": SimTypeFixedSizeArray(SimTypeChar(label="Byte"), 16)}, name="FWP_BYTE_ARRAY16", pack=False, align=None), "cgaCollisionCount": SimTypeChar(label="Byte")}, name="IKEEXT_IPV6_CGA_AUTHENTICATION0", pack=False, align=None), "eapAuthentication": SimStruct({"flags": SimTypeInt(signed=False, label="IKEEXT_EAP_AUTHENTICATION_FLAGS")}, name="IKEEXT_EAP_AUTHENTICATION0", pack=False, align=None)}, name="<anon>", label="None")}, name="IKEEXT_AUTHENTICATION_METHOD1", pack=False, align=None), offset=0), "initiatorImpersonationType": SimTypeInt(signed=False, label="IKEEXT_AUTHENTICATION_IMPERSONATION_TYPE"), "numIkeProposals": SimTypeInt(signed=False, label="UInt32"), "ikeProposals": SimTypePointer(SimStruct({"cipherAlgorithm": SimStruct({"algoIdentifier": SimTypeInt(signed=False, label="IKEEXT_CIPHER_TYPE"), "keyLen": SimTypeInt(signed=False, label="UInt32"), "rounds": SimTypeInt(signed=False, label="UInt32")}, name="IKEEXT_CIPHER_ALGORITHM0", pack=False, align=None), "integrityAlgorithm": SimStruct({"algoIdentifier": SimTypeInt(signed=False, label="IKEEXT_INTEGRITY_TYPE")}, name="IKEEXT_INTEGRITY_ALGORITHM0", pack=False, align=None), "maxLifetimeSeconds": SimTypeInt(signed=False, label="UInt32"), "dhGroup": SimTypeInt(signed=False, label="IKEEXT_DH_GROUP"), "quickModeLimit": SimTypeInt(signed=False, label="UInt32")}, name="IKEEXT_PROPOSAL0", pack=False, align=None), offset=0), "flags": SimTypeInt(signed=False, label="IKEEXT_POLICY_FLAG"), "maxDynamicFilters": SimTypeInt(signed=False, label="UInt32"), "retransmitDurationSecs": SimTypeInt(signed=False, label="UInt32")}, name="IKEEXT_POLICY1", pack=False, align=None), offset=0), "authIpMmPolicy": SimTypePointer(SimStruct({"softExpirationTime": SimTypeInt(signed=False, label="UInt32"), "numAuthenticationMethods": SimTypeInt(signed=False, label="UInt32"), "authenticationMethods": SimTypePointer(SimStruct({"authenticationMethodType": SimTypeInt(signed=False, label="IKEEXT_AUTHENTICATION_METHOD_TYPE"), "Anonymous": SimUnion({"presharedKeyAuthentication": SimStruct({"presharedKey": SimStruct({"size": SimTypeInt(signed=False, label="UInt32"), "data": SimTypePointer(SimTypeChar(label="Byte"), offset=0)}, name="FWP_BYTE_BLOB", pack=False, align=None), "flags": SimTypeInt(signed=False, label="IKEEXT_PRESHARED_KEY_AUTHENTICATION_FLAGS")}, name="IKEEXT_PRESHARED_KEY_AUTHENTICATION1", pack=False, align=None), "certificateAuthentication": SimStruct({"inboundConfigType": SimTypeInt(signed=False, label="IKEEXT_CERT_CONFIG_TYPE"), "Anonymous1": SimUnion({"Anonymous": SimStruct({"inboundRootArraySize": SimTypeInt(signed=False, label="UInt32"), "inboundRootArray": SimTypePointer(SimStruct({"certData": SimStruct({"size": SimTypeInt(signed=False, label="UInt32"), "data": SimTypePointer(SimTypeChar(label="Byte"), offset=0)}, name="FWP_BYTE_BLOB", pack=False, align=None), "flags": SimTypeInt(signed=False, label="IKEEXT_CERT_FLAGS")}, name="IKEEXT_CERT_ROOT_CONFIG0", pack=False, align=None), offset=0)}, name="_Anonymous_e__Struct", pack=False, align=None), "inboundEnterpriseStoreConfig": SimTypePointer(SimStruct({"certData": SimStruct({"size": SimTypeInt(signed=False, label="UInt32"), "data": SimTypePointer(SimTypeChar(label="Byte"), offset=0)}, name="FWP_BYTE_BLOB", pack=False, align=None), "flags": SimTypeInt(signed=False, label="IKEEXT_CERT_FLAGS")}, name="IKEEXT_CERT_ROOT_CONFIG0", pack=False, align=None), offset=0), "inboundTrustedRootStoreConfig": SimTypePointer(SimStruct({"certData": SimStruct({"size": SimTypeInt(signed=False, label="UInt32"), "data": SimTypePointer(SimTypeChar(label="Byte"), offset=0)}, name="FWP_BYTE_BLOB", pack=False, align=None), "flags": SimTypeInt(signed=False, label="IKEEXT_CERT_FLAGS")}, name="IKEEXT_CERT_ROOT_CONFIG0", pack=False, align=None), offset=0)}, name="<anon>", label="None"), "outboundConfigType": SimTypeInt(signed=False, label="IKEEXT_CERT_CONFIG_TYPE"), "Anonymous2": SimUnion({"Anonymous": SimStruct({"outboundRootArraySize": SimTypeInt(signed=False, label="UInt32"), "outboundRootArray": SimTypePointer(SimStruct({"certData": SimStruct({"size": SimTypeInt(signed=False, label="UInt32"), "data": SimTypePointer(SimTypeChar(label="Byte"), offset=0)}, name="FWP_BYTE_BLOB", pack=False, align=None), "flags": SimTypeInt(signed=False, label="IKEEXT_CERT_FLAGS")}, name="IKEEXT_CERT_ROOT_CONFIG0", pack=False, align=None), offset=0)}, name="_Anonymous_e__Struct", pack=False, align=None), "outboundEnterpriseStoreConfig": SimTypePointer(SimStruct({"certData": SimStruct({"size": SimTypeInt(signed=False, label="UInt32"), "data": SimTypePointer(SimTypeChar(label="Byte"), offset=0)}, name="FWP_BYTE_BLOB", pack=False, align=None), "flags": SimTypeInt(signed=False, label="IKEEXT_CERT_FLAGS")}, name="IKEEXT_CERT_ROOT_CONFIG0", pack=False, align=None), offset=0), "outboundTrustedRootStoreConfig": SimTypePointer(SimStruct({"certData": SimStruct({"size": SimTypeInt(signed=False, label="UInt32"), "data": SimTypePointer(SimTypeChar(label="Byte"), offset=0)}, name="FWP_BYTE_BLOB", pack=False, align=None), "flags": SimTypeInt(signed=False, label="IKEEXT_CERT_FLAGS")}, name="IKEEXT_CERT_ROOT_CONFIG0", pack=False, align=None), offset=0)}, name="<anon>", label="None"), "flags": SimTypeInt(signed=False, label="IKEEXT_CERT_AUTH"), "localCertLocationUrl": SimStruct({"size": SimTypeInt(signed=False, label="UInt32"), "data": SimTypePointer(SimTypeChar(label="Byte"), offset=0)}, name="FWP_BYTE_BLOB", pack=False, align=None)}, name="IKEEXT_CERTIFICATE_AUTHENTICATION1", pack=False, align=None), "kerberosAuthentication": SimStruct({"flags": SimTypeInt(signed=False, label="IKEEXT_KERBEROS_AUTHENTICATION_FLAGS")}, name="IKEEXT_KERBEROS_AUTHENTICATION0", pack=False, align=None), "ntlmV2Authentication": SimStruct({"flags": SimTypeInt(signed=False, label="UInt32")}, name="IKEEXT_NTLM_V2_AUTHENTICATION0", pack=False, align=None), "sslAuthentication": SimStruct({"inboundConfigType": SimTypeInt(signed=False, label="IKEEXT_CERT_CONFIG_TYPE"), "Anonymous1": SimUnion({"Anonymous": SimStruct({"inboundRootArraySize": SimTypeInt(signed=False, label="UInt32"), "inboundRootArray": SimTypePointer(SimStruct({"certData": SimStruct({"size": SimTypeInt(signed=False, label="UInt32"), "data": SimTypePointer(SimTypeChar(label="Byte"), offset=0)}, name="FWP_BYTE_BLOB", pack=False, align=None), "flags": SimTypeInt(signed=False, label="IKEEXT_CERT_FLAGS")}, name="IKEEXT_CERT_ROOT_CONFIG0", pack=False, align=None), offset=0)}, name="_Anonymous_e__Struct", pack=False, align=None), "inboundEnterpriseStoreConfig": SimTypePointer(SimStruct({"certData": SimStruct({"size": SimTypeInt(signed=False, label="UInt32"), "data": SimTypePointer(SimTypeChar(label="Byte"), offset=0)}, name="FWP_BYTE_BLOB", pack=False, align=None), "flags": SimTypeInt(signed=False, label="IKEEXT_CERT_FLAGS")}, name="IKEEXT_CERT_ROOT_CONFIG0", pack=False, align=None), offset=0), "inboundTrustedRootStoreConfig": SimTypePointer(SimStruct({"certData": SimStruct({"size": SimTypeInt(signed=False, label="UInt32"), "data": SimTypePointer(SimTypeChar(label="Byte"), offset=0)}, name="FWP_BYTE_BLOB", pack=False, align=None), "flags": SimTypeInt(signed=False, label="IKEEXT_CERT_FLAGS")}, name="IKEEXT_CERT_ROOT_CONFIG0", pack=False, align=None), offset=0)}, name="<anon>", label="None"), "outboundConfigType": SimTypeInt(signed=False, label="IKEEXT_CERT_CONFIG_TYPE"), "Anonymous2": SimUnion({"Anonymous": SimStruct({"outboundRootArraySize": SimTypeInt(signed=False, label="UInt32"), "outboundRootArray": SimTypePointer(SimStruct({"certData": SimStruct({"size": SimTypeInt(signed=False, label="UInt32"), "data": SimTypePointer(SimTypeChar(label="Byte"), offset=0)}, name="FWP_BYTE_BLOB", pack=False, align=None), "flags": SimTypeInt(signed=False, label="IKEEXT_CERT_FLAGS")}, name="IKEEXT_CERT_ROOT_CONFIG0", pack=False, align=None), offset=0)}, name="_Anonymous_e__Struct", pack=False, align=None), "outboundEnterpriseStoreConfig": SimTypePointer(SimStruct({"certData": SimStruct({"size": SimTypeInt(signed=False, label="UInt32"), "data": SimTypePointer(SimTypeChar(label="Byte"), offset=0)}, name="FWP_BYTE_BLOB", pack=False, align=None), "flags": SimTypeInt(signed=False, label="IKEEXT_CERT_FLAGS")}, name="IKEEXT_CERT_ROOT_CONFIG0", pack=False, align=None), offset=0), "outboundTrustedRootStoreConfig": SimTypePointer(SimStruct({"certData": SimStruct({"size": SimTypeInt(signed=False, label="UInt32"), "data": SimTypePointer(SimTypeChar(label="Byte"), offset=0)}, name="FWP_BYTE_BLOB", pack=False, align=None), "flags": SimTypeInt(signed=False, label="IKEEXT_CERT_FLAGS")}, name="IKEEXT_CERT_ROOT_CONFIG0", pack=False, align=None), offset=0)}, name="<anon>", label="None"), "flags": SimTypeInt(signed=False, label="IKEEXT_CERT_AUTH"), "localCertLocationUrl": SimStruct({"size": SimTypeInt(signed=False, label="UInt32"), "data": SimTypePointer(SimTypeChar(label="Byte"), offset=0)}, name="FWP_BYTE_BLOB", pack=False, align=None)}, name="IKEEXT_CERTIFICATE_AUTHENTICATION1", pack=False, align=None), "cgaAuthentication": SimStruct({"keyContainerName": SimTypePointer(SimTypeChar(label="Char"), offset=0), "cspName": SimTypePointer(SimTypeChar(label="Char"), offset=0), "cspType": SimTypeInt(signed=False, label="UInt32"), "cgaModifier": SimStruct({"byteArray16": SimTypeFixedSizeArray(SimTypeChar(label="Byte"), 16)}, name="FWP_BYTE_ARRAY16", pack=False, align=None), "cgaCollisionCount": SimTypeChar(label="Byte")}, name="IKEEXT_IPV6_CGA_AUTHENTICATION0", pack=False, align=None), "eapAuthentication": SimStruct({"flags": SimTypeInt(signed=False, label="IKEEXT_EAP_AUTHENTICATION_FLAGS")}, name="IKEEXT_EAP_AUTHENTICATION0", pack=False, align=None)}, name="<anon>", label="None")}, name="IKEEXT_AUTHENTICATION_METHOD1", pack=False, align=None), offset=0), "initiatorImpersonationType": SimTypeInt(signed=False, label="IKEEXT_AUTHENTICATION_IMPERSONATION_TYPE"), "numIkeProposals": SimTypeInt(signed=False, label="UInt32"), "ikeProposals": SimTypePointer(SimStruct({"cipherAlgorithm": SimStruct({"algoIdentifier": SimTypeInt(signed=False, label="IKEEXT_CIPHER_TYPE"), "keyLen": SimTypeInt(signed=False, label="UInt32"), "rounds": SimTypeInt(signed=False, label="UInt32")}, name="IKEEXT_CIPHER_ALGORITHM0", pack=False, align=None), "integrityAlgorithm": SimStruct({"algoIdentifier": SimTypeInt(signed=False, label="IKEEXT_INTEGRITY_TYPE")}, name="IKEEXT_INTEGRITY_ALGORITHM0", pack=False, align=None), "maxLifetimeSeconds": SimTypeInt(signed=False, label="UInt32"), "dhGroup": SimTypeInt(signed=False, label="IKEEXT_DH_GROUP"), "quickModeLimit": SimTypeInt(signed=False, label="UInt32")}, name="IKEEXT_PROPOSAL0", pack=False, align=None), offset=0), "flags": SimTypeInt(signed=False, label="IKEEXT_POLICY_FLAG"), "maxDynamicFilters": SimTypeInt(signed=False, label="UInt32"), "retransmitDurationSecs": SimTypeInt(signed=False, label="UInt32")}, name="IKEEXT_POLICY1", pack=False, align=None), offset=0), "dataBuffer": SimTypePointer(SimStruct({"size": SimTypeInt(signed=False, label="UInt32"), "data": SimTypePointer(SimTypeChar(label="Byte"), offset=0)}, name="FWP_BYTE_BLOB", pack=False, align=None), offset=0), "classifyOptions": SimTypePointer(SimStruct({"numOptions": SimTypeInt(signed=False, label="UInt32"), "options": SimTypePointer(SimStruct({"type": SimTypeInt(signed=False, label="FWP_CLASSIFY_OPTION_TYPE"), "value": SimStruct({"type": SimTypeInt(signed=False, label="FWP_DATA_TYPE"), "Anonymous": SimUnion({"uint8": SimTypeChar(label="Byte"), "uint16": SimTypeShort(signed=False, label="UInt16"), "uint32": SimTypeInt(signed=False, label="UInt32"), "uint64": SimTypePointer(SimTypeLongLong(signed=False, label="UInt64"), offset=0), "int8": SimTypeChar(label="SByte"), "int16": SimTypeShort(signed=True, label="Int16"), "int32": SimTypeInt(signed=True, label="Int32"), "int64": SimTypePointer(SimTypeLongLong(signed=True, label="Int64"), offset=0), "float32": SimTypeFloat(size=32), "double64": SimTypePointer(SimTypeFloat(size=64), offset=0), "byteArray16": SimTypePointer(SimStruct({"byteArray16": SimTypeFixedSizeArray(SimTypeChar(label="Byte"), 16)}, name="FWP_BYTE_ARRAY16", pack=False, align=None), offset=0), "byteBlob": SimTypePointer(SimStruct({"size": SimTypeInt(signed=False, label="UInt32"), "data": SimTypePointer(SimTypeChar(label="Byte"), offset=0)}, name="FWP_BYTE_BLOB", pack=False, align=None), offset=0), "sid": SimTypePointer(SimTypeBottom(label="SID"), offset=0), "sd": SimTypePointer(SimStruct({"size": SimTypeInt(signed=False, label="UInt32"), "data": SimTypePointer(SimTypeChar(label="Byte"), offset=0)}, name="FWP_BYTE_BLOB", pack=False, align=None), offset=0), "tokenInformation": SimTypePointer(SimStruct({"sidCount": SimTypeInt(signed=False, label="UInt32"), "sids": SimTypePointer(SimTypeBottom(label="SID_AND_ATTRIBUTES"), offset=0), "restrictedSidCount": SimTypeInt(signed=False, label="UInt32"), "restrictedSids": SimTypePointer(SimTypeBottom(label="SID_AND_ATTRIBUTES"), offset=0)}, name="FWP_TOKEN_INFORMATION", pack=False, align=None), offset=0), "tokenAccessInformation": SimTypePointer(SimStruct({"size": SimTypeInt(signed=False, label="UInt32"), "data": SimTypePointer(SimTypeChar(label="Byte"), offset=0)}, name="FWP_BYTE_BLOB", pack=False, align=None), offset=0), "unicodeString": SimTypePointer(SimTypeChar(label="Char"), offset=0), "byteArray6": SimTypePointer(SimStruct({"byteArray6": SimTypeFixedSizeArray(SimTypeChar(label="Byte"), 6)}, name="FWP_BYTE_ARRAY6", pack=False, align=None), offset=0), "bitmapArray64": SimTypePointer(SimStruct({"bitmapArray64": SimTypeFixedSizeArray(SimTypeChar(label="Byte"), 8)}, name="FWP_BITMAP_ARRAY64_", pack=False, align=None), offset=0)}, name="<anon>", label="None")}, name="FWP_VALUE0", pack=False, align=None)}, name="FWPM_CLASSIFY_OPTION0", pack=False, align=None), offset=0)}, name="FWPM_CLASSIFY_OPTIONS0", pack=False, align=None), offset=0), "ikeV2QmTunnelPolicy": SimTypePointer(SimStruct({"flags": SimTypeInt(signed=False, label="IPSEC_POLICY_FLAG"), "numIpsecProposals": SimTypeInt(signed=False, label="UInt32"), "ipsecProposals": SimTypePointer(SimStruct({"lifetime": SimStruct({"lifetimeSeconds": SimTypeInt(signed=False, label="UInt32"), "lifetimeKilobytes": SimTypeInt(signed=False, label="UInt32"), "lifetimePackets": SimTypeInt(signed=False, label="UInt32")}, name="IPSEC_SA_LIFETIME0", pack=False, align=None), "numSaTransforms": SimTypeInt(signed=False, label="UInt32"), "saTransforms": SimTypePointer(SimStruct({"ipsecTransformType": SimTypeInt(signed=False, label="IPSEC_TRANSFORM_TYPE"), "Anonymous": SimUnion({"ahTransform": SimTypePointer(SimStruct({"authTransformId": SimStruct({"authType": SimTypeInt(signed=False, label="IPSEC_AUTH_TYPE"), "authConfig": SimTypeChar(label="Byte")}, name="IPSEC_AUTH_TRANSFORM_ID0", pack=False, align=None), "cryptoModuleId": SimTypePointer(SimTypeBottom(label="Guid"), offset=0)}, name="IPSEC_AUTH_TRANSFORM0", pack=False, align=None), offset=0), "espAuthTransform": SimTypePointer(SimStruct({"authTransformId": SimStruct({"authType": SimTypeInt(signed=False, label="IPSEC_AUTH_TYPE"), "authConfig": SimTypeChar(label="Byte")}, name="IPSEC_AUTH_TRANSFORM_ID0", pack=False, align=None), "cryptoModuleId": SimTypePointer(SimTypeBottom(label="Guid"), offset=0)}, name="IPSEC_AUTH_TRANSFORM0", pack=False, align=None), offset=0), "espCipherTransform": SimTypePointer(SimStruct({"cipherTransformId": SimStruct({"cipherType": SimTypeInt(signed=False, label="IPSEC_CIPHER_TYPE"), "cipherConfig": SimTypeChar(label="Byte")}, name="IPSEC_CIPHER_TRANSFORM_ID0", pack=False, align=None), "cryptoModuleId": SimTypePointer(SimTypeBottom(label="Guid"), offset=0)}, name="IPSEC_CIPHER_TRANSFORM0", pack=False, align=None), offset=0), "espAuthAndCipherTransform": SimTypePointer(SimStruct({"authTransform": SimStruct({"authTransformId": SimStruct({"authType": SimTypeInt(signed=False, label="IPSEC_AUTH_TYPE"), "authConfig": SimTypeChar(label="Byte")}, name="IPSEC_AUTH_TRANSFORM_ID0", pack=False, align=None), "cryptoModuleId": SimTypePointer(SimTypeBottom(label="Guid"), offset=0)}, name="IPSEC_AUTH_TRANSFORM0", pack=False, align=None), "cipherTransform": SimStruct({"cipherTransformId": SimStruct({"cipherType": SimTypeInt(signed=False, label="IPSEC_CIPHER_TYPE"), "cipherConfig": SimTypeChar(label="Byte")}, name="IPSEC_CIPHER_TRANSFORM_ID0", pack=False, align=None), "cryptoModuleId": SimTypePointer(SimTypeBottom(label="Guid"), offset=0)}, name="IPSEC_CIPHER_TRANSFORM0", pack=False, align=None)}, name="IPSEC_AUTH_AND_CIPHER_TRANSFORM0", pack=False, align=None), offset=0), "espAuthFwTransform": SimTypePointer(SimStruct({"authTransformId": SimStruct({"authType": SimTypeInt(signed=False, label="IPSEC_AUTH_TYPE"), "authConfig": SimTypeChar(label="Byte")}, name="IPSEC_AUTH_TRANSFORM_ID0", pack=False, align=None), "cryptoModuleId": SimTypePointer(SimTypeBottom(label="Guid"), offset=0)}, name="IPSEC_AUTH_TRANSFORM0", pack=False, align=None), offset=0)}, name="<anon>", label="None")}, name="IPSEC_SA_TRANSFORM0", pack=False, align=None), offset=0), "pfsGroup": SimTypeInt(signed=False, label="IPSEC_PFS_GROUP")}, name="IPSEC_PROPOSAL0", pack=False, align=None), offset=0), "tunnelEndpoints": SimStruct({"ipVersion": SimTypeInt(signed=False, label="FWP_IP_VERSION"), "Anonymous1": SimUnion({"localV4Address": SimTypeInt(signed=False, label="UInt32"), "localV6Address": SimTypeFixedSizeArray(SimTypeChar(label="Byte"), 16)}, name="<anon>", label="None"), "Anonymous2": SimUnion({"remoteV4Address": SimTypeInt(signed=False, label="UInt32"), "remoteV6Address": SimTypeFixedSizeArray(SimTypeChar(label="Byte"), 16)}, name="<anon>", label="None"), "localIfLuid": SimTypeLongLong(signed=False, label="UInt64")}, name="IPSEC_TUNNEL_ENDPOINTS1", pack=False, align=None), "saIdleTimeout": SimStruct({"idleTimeoutSeconds": SimTypeInt(signed=False, label="UInt32"), "idleTimeoutSecondsFailOver": SimTypeInt(signed=False, label="UInt32")}, name="IPSEC_SA_IDLE_TIMEOUT0", pack=False, align=None), "emPolicy": SimTypePointer(SimStruct({"numAuthenticationMethods": SimTypeInt(signed=False, label="UInt32"), "authenticationMethods": SimTypePointer(SimStruct({"authenticationMethodType": SimTypeInt(signed=False, label="IKEEXT_AUTHENTICATION_METHOD_TYPE"), "Anonymous": SimUnion({"presharedKeyAuthentication": SimStruct({"presharedKey": SimStruct({"size": SimTypeInt(signed=False, label="UInt32"), "data": SimTypePointer(SimTypeChar(label="Byte"), offset=0)}, name="FWP_BYTE_BLOB", pack=False, align=None), "flags": SimTypeInt(signed=False, label="IKEEXT_PRESHARED_KEY_AUTHENTICATION_FLAGS")}, name="IKEEXT_PRESHARED_KEY_AUTHENTICATION1", pack=False, align=None), "certificateAuthentication": SimStruct({"inboundConfigType": SimTypeInt(signed=False, label="IKEEXT_CERT_CONFIG_TYPE"), "Anonymous1": SimUnion({"Anonymous": SimStruct({"inboundRootArraySize": SimTypeInt(signed=False, label="UInt32"), "inboundRootArray": SimTypePointer(SimStruct({"certData": SimStruct({"size": SimTypeInt(signed=False, label="UInt32"), "data": SimTypePointer(SimTypeChar(label="Byte"), offset=0)}, name="FWP_BYTE_BLOB", pack=False, align=None), "flags": SimTypeInt(signed=False, label="IKEEXT_CERT_FLAGS")}, name="IKEEXT_CERT_ROOT_CONFIG0", pack=False, align=None), offset=0)}, name="_Anonymous_e__Struct", pack=False, align=None), "inboundEnterpriseStoreConfig": SimTypePointer(SimStruct({"certData": SimStruct({"size": SimTypeInt(signed=False, label="UInt32"), "data": SimTypePointer(SimTypeChar(label="Byte"), offset=0)}, name="FWP_BYTE_BLOB", pack=False, align=None), "flags": SimTypeInt(signed=False, label="IKEEXT_CERT_FLAGS")}, name="IKEEXT_CERT_ROOT_CONFIG0", pack=False, align=None), offset=0), "inboundTrustedRootStoreConfig": SimTypePointer(SimStruct({"certData": SimStruct({"size": SimTypeInt(signed=False, label="UInt32"), "data": SimTypePointer(SimTypeChar(label="Byte"), offset=0)}, name="FWP_BYTE_BLOB", pack=False, align=None), "flags": SimTypeInt(signed=False, label="IKEEXT_CERT_FLAGS")}, name="IKEEXT_CERT_ROOT_CONFIG0", pack=False, align=None), offset=0)}, name="<anon>", label="None"), "outboundConfigType": SimTypeInt(signed=False, label="IKEEXT_CERT_CONFIG_TYPE"), "Anonymous2": SimUnion({"Anonymous": SimStruct({"outboundRootArraySize": SimTypeInt(signed=False, label="UInt32"), "outboundRootArray": SimTypePointer(SimStruct({"certData": SimStruct({"size": SimTypeInt(signed=False, label="UInt32"), "data": SimTypePointer(SimTypeChar(label="Byte"), offset=0)}, name="FWP_BYTE_BLOB", pack=False, align=None), "flags": SimTypeInt(signed=False, label="IKEEXT_CERT_FLAGS")}, name="IKEEXT_CERT_ROOT_CONFIG0", pack=False, align=None), offset=0)}, name="_Anonymous_e__Struct", pack=False, align=None), "outboundEnterpriseStoreConfig": SimTypePointer(SimStruct({"certData": SimStruct({"size": SimTypeInt(signed=False, label="UInt32"), "data": SimTypePointer(SimTypeChar(label="Byte"), offset=0)}, name="FWP_BYTE_BLOB", pack=False, align=None), "flags": SimTypeInt(signed=False, label="IKEEXT_CERT_FLAGS")}, name="IKEEXT_CERT_ROOT_CONFIG0", pack=False, align=None), offset=0), "outboundTrustedRootStoreConfig": SimTypePointer(SimStruct({"certData": SimStruct({"size": SimTypeInt(signed=False, label="UInt32"), "data": SimTypePointer(SimTypeChar(label="Byte"), offset=0)}, name="FWP_BYTE_BLOB", pack=False, align=None), "flags": SimTypeInt(signed=False, label="IKEEXT_CERT_FLAGS")}, name="IKEEXT_CERT_ROOT_CONFIG0", pack=False, align=None), offset=0)}, name="<anon>", label="None"), "flags": SimTypeInt(signed=False, label="IKEEXT_CERT_AUTH"), "localCertLocationUrl": SimStruct({"size": SimTypeInt(signed=False, label="UInt32"), "data": SimTypePointer(SimTypeChar(label="Byte"), offset=0)}, name="FWP_BYTE_BLOB", pack=False, align=None)}, name="IKEEXT_CERTIFICATE_AUTHENTICATION1", pack=False, align=None), "kerberosAuthentication": SimStruct({"flags": SimTypeInt(signed=False, label="IKEEXT_KERBEROS_AUTHENTICATION_FLAGS")}, name="IKEEXT_KERBEROS_AUTHENTICATION0", pack=False, align=None), "ntlmV2Authentication": SimStruct({"flags": SimTypeInt(signed=False, label="UInt32")}, name="IKEEXT_NTLM_V2_AUTHENTICATION0", pack=False, align=None), "sslAuthentication": SimStruct({"inboundConfigType": SimTypeInt(signed=False, label="IKEEXT_CERT_CONFIG_TYPE"), "Anonymous1": SimUnion({"Anonymous": SimStruct({"inboundRootArraySize": SimTypeInt(signed=False, label="UInt32"), "inboundRootArray": SimTypePointer(SimStruct({"certData": SimStruct({"size": SimTypeInt(signed=False, label="UInt32"), "data": SimTypePointer(SimTypeChar(label="Byte"), offset=0)}, name="FWP_BYTE_BLOB", pack=False, align=None), "flags": SimTypeInt(signed=False, label="IKEEXT_CERT_FLAGS")}, name="IKEEXT_CERT_ROOT_CONFIG0", pack=False, align=None), offset=0)}, name="_Anonymous_e__Struct", pack=False, align=None), "inboundEnterpriseStoreConfig": SimTypePointer(SimStruct({"certData": SimStruct({"size": SimTypeInt(signed=False, label="UInt32"), "data": SimTypePointer(SimTypeChar(label="Byte"), offset=0)}, name="FWP_BYTE_BLOB", pack=False, align=None), "flags": SimTypeInt(signed=False, label="IKEEXT_CERT_FLAGS")}, name="IKEEXT_CERT_ROOT_CONFIG0", pack=False, align=None), offset=0), "inboundTrustedRootStoreConfig": SimTypePointer(SimStruct({"certData": SimStruct({"size": SimTypeInt(signed=False, label="UInt32"), "data": SimTypePointer(SimTypeChar(label="Byte"), offset=0)}, name="FWP_BYTE_BLOB", pack=False, align=None), "flags": SimTypeInt(signed=False, label="IKEEXT_CERT_FLAGS")}, name="IKEEXT_CERT_ROOT_CONFIG0", pack=False, align=None), offset=0)}, name="<anon>", label="None"), "outboundConfigType": SimTypeInt(signed=False, label="IKEEXT_CERT_CONFIG_TYPE"), "Anonymous2": SimUnion({"Anonymous": SimStruct({"outboundRootArraySize": SimTypeInt(signed=False, label="UInt32"), "outboundRootArray": SimTypePointer(SimStruct({"certData": SimStruct({"size": SimTypeInt(signed=False, label="UInt32"), "data": SimTypePointer(SimTypeChar(label="Byte"), offset=0)}, name="FWP_BYTE_BLOB", pack=False, align=None), "flags": SimTypeInt(signed=False, label="IKEEXT_CERT_FLAGS")}, name="IKEEXT_CERT_ROOT_CONFIG0", pack=False, align=None), offset=0)}, name="_Anonymous_e__Struct", pack=False, align=None), "outboundEnterpriseStoreConfig": SimTypePointer(SimStruct({"certData": SimStruct({"size": SimTypeInt(signed=False, label="UInt32"), "data": SimTypePointer(SimTypeChar(label="Byte"), offset=0)}, name="FWP_BYTE_BLOB", pack=False, align=None), "flags": SimTypeInt(signed=False, label="IKEEXT_CERT_FLAGS")}, name="IKEEXT_CERT_ROOT_CONFIG0", pack=False, align=None), offset=0), "outboundTrustedRootStoreConfig": SimTypePointer(SimStruct({"certData": SimStruct({"size": SimTypeInt(signed=False, label="UInt32"), "data": SimTypePointer(SimTypeChar(label="Byte"), offset=0)}, name="FWP_BYTE_BLOB", pack=False, align=None), "flags": SimTypeInt(signed=False, label="IKEEXT_CERT_FLAGS")}, name="IKEEXT_CERT_ROOT_CONFIG0", pack=False, align=None), offset=0)}, name="<anon>", label="None"), "flags": SimTypeInt(signed=False, label="IKEEXT_CERT_AUTH"), "localCertLocationUrl": SimStruct({"size": SimTypeInt(signed=False, label="UInt32"), "data": SimTypePointer(SimTypeChar(label="Byte"), offset=0)}, name="FWP_BYTE_BLOB", pack=False, align=None)}, name="IKEEXT_CERTIFICATE_AUTHENTICATION1", pack=False, align=None), "cgaAuthentication": SimStruct({"keyContainerName": SimTypePointer(SimTypeChar(label="Char"), offset=0), "cspName": SimTypePointer(SimTypeChar(label="Char"), offset=0), "cspType": SimTypeInt(signed=False, label="UInt32"), "cgaModifier": SimStruct({"byteArray16": SimTypeFixedSizeArray(SimTypeChar(label="Byte"), 16)}, name="FWP_BYTE_ARRAY16", pack=False, align=None), "cgaCollisionCount": SimTypeChar(label="Byte")}, name="IKEEXT_IPV6_CGA_AUTHENTICATION0", pack=False, align=None), "eapAuthentication": SimStruct({"flags": SimTypeInt(signed=False, label="IKEEXT_EAP_AUTHENTICATION_FLAGS")}, name="IKEEXT_EAP_AUTHENTICATION0", pack=False, align=None)}, name="<anon>", label="None")}, name="IKEEXT_AUTHENTICATION_METHOD1", pack=False, align=None), offset=0), "initiatorImpersonationType": SimTypeInt(signed=False, label="IKEEXT_AUTHENTICATION_IMPERSONATION_TYPE")}, name="IKEEXT_EM_POLICY1", pack=False, align=None), offset=0)}, name="IPSEC_TUNNEL_POLICY1", pack=False, align=None), offset=0), "ikeV2MmPolicy": SimTypePointer(SimStruct({"softExpirationTime": SimTypeInt(signed=False, label="UInt32"), "numAuthenticationMethods": SimTypeInt(signed=False, label="UInt32"), "authenticationMethods": SimTypePointer(SimStruct({"authenticationMethodType": SimTypeInt(signed=False, label="IKEEXT_AUTHENTICATION_METHOD_TYPE"), "Anonymous": SimUnion({"presharedKeyAuthentication": SimStruct({"presharedKey": SimStruct({"size": SimTypeInt(signed=False, label="UInt32"), "data": SimTypePointer(SimTypeChar(label="Byte"), offset=0)}, name="FWP_BYTE_BLOB", pack=False, align=None), "flags": SimTypeInt(signed=False, label="IKEEXT_PRESHARED_KEY_AUTHENTICATION_FLAGS")}, name="IKEEXT_PRESHARED_KEY_AUTHENTICATION1", pack=False, align=None), "certificateAuthentication": SimStruct({"inboundConfigType": SimTypeInt(signed=False, label="IKEEXT_CERT_CONFIG_TYPE"), "Anonymous1": SimUnion({"Anonymous": SimStruct({"inboundRootArraySize": SimTypeInt(signed=False, label="UInt32"), "inboundRootArray": SimTypePointer(SimStruct({"certData": SimStruct({"size": SimTypeInt(signed=False, label="UInt32"), "data": SimTypePointer(SimTypeChar(label="Byte"), offset=0)}, name="FWP_BYTE_BLOB", pack=False, align=None), "flags": SimTypeInt(signed=False, label="IKEEXT_CERT_FLAGS")}, name="IKEEXT_CERT_ROOT_CONFIG0", pack=False, align=None), offset=0)}, name="_Anonymous_e__Struct", pack=False, align=None), "inboundEnterpriseStoreConfig": SimTypePointer(SimStruct({"certData": SimStruct({"size": SimTypeInt(signed=False, label="UInt32"), "data": SimTypePointer(SimTypeChar(label="Byte"), offset=0)}, name="FWP_BYTE_BLOB", pack=False, align=None), "flags": SimTypeInt(signed=False, label="IKEEXT_CERT_FLAGS")}, name="IKEEXT_CERT_ROOT_CONFIG0", pack=False, align=None), offset=0), "inboundTrustedRootStoreConfig": SimTypePointer(SimStruct({"certData": SimStruct({"size": SimTypeInt(signed=False, label="UInt32"), "data": SimTypePointer(SimTypeChar(label="Byte"), offset=0)}, name="FWP_BYTE_BLOB", pack=False, align=None), "flags": SimTypeInt(signed=False, label="IKEEXT_CERT_FLAGS")}, name="IKEEXT_CERT_ROOT_CONFIG0", pack=False, align=None), offset=0)}, name="<anon>", label="None"), "outboundConfigType": SimTypeInt(signed=False, label="IKEEXT_CERT_CONFIG_TYPE"), "Anonymous2": SimUnion({"Anonymous": SimStruct({"outboundRootArraySize": SimTypeInt(signed=False, label="UInt32"), "outboundRootArray": SimTypePointer(SimStruct({"certData": SimStruct({"size": SimTypeInt(signed=False, label="UInt32"), "data": SimTypePointer(SimTypeChar(label="Byte"), offset=0)}, name="FWP_BYTE_BLOB", pack=False, align=None), "flags": SimTypeInt(signed=False, label="IKEEXT_CERT_FLAGS")}, name="IKEEXT_CERT_ROOT_CONFIG0", pack=False, align=None), offset=0)}, name="_Anonymous_e__Struct", pack=False, align=None), "outboundEnterpriseStoreConfig": SimTypePointer(SimStruct({"certData": SimStruct({"size": SimTypeInt(signed=False, label="UInt32"), "data": SimTypePointer(SimTypeChar(label="Byte"), offset=0)}, name="FWP_BYTE_BLOB", pack=False, align=None), "flags": SimTypeInt(signed=False, label="IKEEXT_CERT_FLAGS")}, name="IKEEXT_CERT_ROOT_CONFIG0", pack=False, align=None), offset=0), "outboundTrustedRootStoreConfig": SimTypePointer(SimStruct({"certData": SimStruct({"size": SimTypeInt(signed=False, label="UInt32"), "data": SimTypePointer(SimTypeChar(label="Byte"), offset=0)}, name="FWP_BYTE_BLOB", pack=False, align=None), "flags": SimTypeInt(signed=False, label="IKEEXT_CERT_FLAGS")}, name="IKEEXT_CERT_ROOT_CONFIG0", pack=False, align=None), offset=0)}, name="<anon>", label="None"), "flags": SimTypeInt(signed=False, label="IKEEXT_CERT_AUTH"), "localCertLocationUrl": SimStruct({"size": SimTypeInt(signed=False, label="UInt32"), "data": SimTypePointer(SimTypeChar(label="Byte"), offset=0)}, name="FWP_BYTE_BLOB", pack=False, align=None)}, name="IKEEXT_CERTIFICATE_AUTHENTICATION1", pack=False, align=None), "kerberosAuthentication": SimStruct({"flags": SimTypeInt(signed=False, label="IKEEXT_KERBEROS_AUTHENTICATION_FLAGS")}, name="IKEEXT_KERBEROS_AUTHENTICATION0", pack=False, align=None), "ntlmV2Authentication": SimStruct({"flags": SimTypeInt(signed=False, label="UInt32")}, name="IKEEXT_NTLM_V2_AUTHENTICATION0", pack=False, align=None), "sslAuthentication": SimStruct({"inboundConfigType": SimTypeInt(signed=False, label="IKEEXT_CERT_CONFIG_TYPE"), "Anonymous1": SimUnion({"Anonymous": SimStruct({"inboundRootArraySize": SimTypeInt(signed=False, label="UInt32"), "inboundRootArray": SimTypePointer(SimStruct({"certData": SimStruct({"size": SimTypeInt(signed=False, label="UInt32"), "data": SimTypePointer(SimTypeChar(label="Byte"), offset=0)}, name="FWP_BYTE_BLOB", pack=False, align=None), "flags": SimTypeInt(signed=False, label="IKEEXT_CERT_FLAGS")}, name="IKEEXT_CERT_ROOT_CONFIG0", pack=False, align=None), offset=0)}, name="_Anonymous_e__Struct", pack=False, align=None), "inboundEnterpriseStoreConfig": SimTypePointer(SimStruct({"certData": SimStruct({"size": SimTypeInt(signed=False, label="UInt32"), "data": SimTypePointer(SimTypeChar(label="Byte"), offset=0)}, name="FWP_BYTE_BLOB", pack=False, align=None), "flags": SimTypeInt(signed=False, label="IKEEXT_CERT_FLAGS")}, name="IKEEXT_CERT_ROOT_CONFIG0", pack=False, align=None), offset=0), "inboundTrustedRootStoreConfig": SimTypePointer(SimStruct({"certData": SimStruct({"size": SimTypeInt(signed=False, label="UInt32"), "data": SimTypePointer(SimTypeChar(label="Byte"), offset=0)}, name="FWP_BYTE_BLOB", pack=False, align=None), "flags": SimTypeInt(signed=False, label="IKEEXT_CERT_FLAGS")}, name="IKEEXT_CERT_ROOT_CONFIG0", pack=False, align=None), offset=0)}, name="<anon>", label="None"), "outboundConfigType": SimTypeInt(signed=False, label="IKEEXT_CERT_CONFIG_TYPE"), "Anonymous2": SimUnion({"Anonymous": SimStruct({"outboundRootArraySize": SimTypeInt(signed=False, label="UInt32"), "outboundRootArray": SimTypePointer(SimStruct({"certData": SimStruct({"size": SimTypeInt(signed=False, label="UInt32"), "data": SimTypePointer(SimTypeChar(label="Byte"), offset=0)}, name="FWP_BYTE_BLOB", pack=False, align=None), "flags": SimTypeInt(signed=False, label="IKEEXT_CERT_FLAGS")}, name="IKEEXT_CERT_ROOT_CONFIG0", pack=False, align=None), offset=0)}, name="_Anonymous_e__Struct", pack=False, align=None), "outboundEnterpriseStoreConfig": SimTypePointer(SimStruct({"certData": SimStruct({"size": SimTypeInt(signed=False, label="UInt32"), "data": SimTypePointer(SimTypeChar(label="Byte"), offset=0)}, name="FWP_BYTE_BLOB", pack=False, align=None), "flags": SimTypeInt(signed=False, label="IKEEXT_CERT_FLAGS")}, name="IKEEXT_CERT_ROOT_CONFIG0", pack=False, align=None), offset=0), "outboundTrustedRootStoreConfig": SimTypePointer(SimStruct({"certData": SimStruct({"size": SimTypeInt(signed=False, label="UInt32"), "data": SimTypePointer(SimTypeChar(label="Byte"), offset=0)}, name="FWP_BYTE_BLOB", pack=False, align=None), "flags": SimTypeInt(signed=False, label="IKEEXT_CERT_FLAGS")}, name="IKEEXT_CERT_ROOT_CONFIG0", pack=False, align=None), offset=0)}, name="<anon>", label="None"), "flags": SimTypeInt(signed=False, label="IKEEXT_CERT_AUTH"), "localCertLocationUrl": SimStruct({"size": SimTypeInt(signed=False, label="UInt32"), "data": SimTypePointer(SimTypeChar(label="Byte"), offset=0)}, name="FWP_BYTE_BLOB", pack=False, align=None)}, name="IKEEXT_CERTIFICATE_AUTHENTICATION1", pack=False, align=None), "cgaAuthentication": SimStruct({"keyContainerName": SimTypePointer(SimTypeChar(label="Char"), offset=0), "cspName": SimTypePointer(SimTypeChar(label="Char"), offset=0), "cspType": SimTypeInt(signed=False, label="UInt32"), "cgaModifier": SimStruct({"byteArray16": SimTypeFixedSizeArray(SimTypeChar(label="Byte"), 16)}, name="FWP_BYTE_ARRAY16", pack=False, align=None), "cgaCollisionCount": SimTypeChar(label="Byte")}, name="IKEEXT_IPV6_CGA_AUTHENTICATION0", pack=False, align=None), "eapAuthentication": SimStruct({"flags": SimTypeInt(signed=False, label="IKEEXT_EAP_AUTHENTICATION_FLAGS")}, name="IKEEXT_EAP_AUTHENTICATION0", pack=False, align=None)}, name="<anon>", label="None")}, name="IKEEXT_AUTHENTICATION_METHOD1", pack=False, align=None), offset=0), "initiatorImpersonationType": SimTypeInt(signed=False, label="IKEEXT_AUTHENTICATION_IMPERSONATION_TYPE"), "numIkeProposals": SimTypeInt(signed=False, label="UInt32"), "ikeProposals": SimTypePointer(SimStruct({"cipherAlgorithm": SimStruct({"algoIdentifier": SimTypeInt(signed=False, label="IKEEXT_CIPHER_TYPE"), "keyLen": SimTypeInt(signed=False, label="UInt32"), "rounds": SimTypeInt(signed=False, label="UInt32")}, name="IKEEXT_CIPHER_ALGORITHM0", pack=False, align=None), "integrityAlgorithm": SimStruct({"algoIdentifier": SimTypeInt(signed=False, label="IKEEXT_INTEGRITY_TYPE")}, name="IKEEXT_INTEGRITY_ALGORITHM0", pack=False, align=None), "maxLifetimeSeconds": SimTypeInt(signed=False, label="UInt32"), "dhGroup": SimTypeInt(signed=False, label="IKEEXT_DH_GROUP"), "quickModeLimit": SimTypeInt(signed=False, label="UInt32")}, name="IKEEXT_PROPOSAL0", pack=False, align=None), offset=0), "flags": SimTypeInt(signed=False, label="IKEEXT_POLICY_FLAG"), "maxDynamicFilters": SimTypeInt(signed=False, label="UInt32"), "retransmitDurationSecs": SimTypeInt(signed=False, label="UInt32")}, name="IKEEXT_POLICY1", pack=False, align=None), offset=0), "idpOptions": SimTypePointer(SimStruct({"stateIdleTimeoutSeconds": SimTypeInt(signed=False, label="UInt32"), "perIPRateLimitQueueIdleTimeoutSeconds": SimTypeInt(signed=False, label="UInt32"), "ipV6IPsecUnauthDscp": SimTypeChar(label="Byte"), "ipV6IPsecUnauthRateLimitBytesPerSec": SimTypeInt(signed=False, label="UInt32"), "ipV6IPsecUnauthPerIPRateLimitBytesPerSec": SimTypeInt(signed=False, label="UInt32"), "ipV6IPsecAuthDscp": SimTypeChar(label="Byte"), "ipV6IPsecAuthRateLimitBytesPerSec": SimTypeInt(signed=False, label="UInt32"), "icmpV6Dscp": SimTypeChar(label="Byte"), "icmpV6RateLimitBytesPerSec": SimTypeInt(signed=False, label="UInt32"), "ipV6FilterExemptDscp": SimTypeChar(label="Byte"), "ipV6FilterExemptRateLimitBytesPerSec": SimTypeInt(signed=False, label="UInt32"), "defBlockExemptDscp": SimTypeChar(label="Byte"), "defBlockExemptRateLimitBytesPerSec": SimTypeInt(signed=False, label="UInt32"), "maxStateEntries": SimTypeInt(signed=False, label="UInt32"), "maxPerIPRateLimitQueues": SimTypeInt(signed=False, label="UInt32"), "flags": SimTypeInt(signed=False, label="IPSEC_DOSP_FLAGS"), "numPublicIFLuids": SimTypeInt(signed=False, label="UInt32"), "publicIFLuids": SimTypePointer(SimTypeLongLong(signed=False, label="UInt64"), offset=0), "numInternalIFLuids": SimTypeInt(signed=False, label="UInt32"), "internalIFLuids": SimTypePointer(SimTypeLongLong(signed=False, label="UInt64"), offset=0), "publicV6AddrMask": SimStruct({"addr": SimTypeFixedSizeArray(SimTypeChar(label="Byte"), 16), "prefixLength": SimTypeChar(label="Byte")}, name="FWP_V6_ADDR_AND_MASK", pack=False, align=None), "internalV6AddrMask": SimStruct({"addr": SimTypeFixedSizeArray(SimTypeChar(label="Byte"), 16), "prefixLength": SimTypeChar(label="Byte")}, name="FWP_V6_ADDR_AND_MASK", pack=False, align=None)}, name="IPSEC_DOSP_OPTIONS0", pack=False, align=None), offset=0)}, name="<anon>", label="None"), "providerContextId": SimTypeLongLong(signed=False, label="UInt64")}, name="FWPM_PROVIDER_CONTEXT1", pack=False, align=None), offset=0), SimTypeInt(signed=False, label="UInt32"), SimTypePointer(SimStruct({"fieldKey": SimTypeBottom(label="Guid"), "matchType": SimTypeInt(signed=False, label="FWP_MATCH_TYPE"), "conditionValue": SimStruct({"type": SimTypeInt(signed=False, label="FWP_DATA_TYPE"), "Anonymous": SimUnion({"uint8": SimTypeChar(label="Byte"), "uint16": SimTypeShort(signed=False, label="UInt16"), "uint32": SimTypeInt(signed=False, label="UInt32"), "uint64": SimTypePointer(SimTypeLongLong(signed=False, label="UInt64"), offset=0), "int8": SimTypeChar(label="SByte"), "int16": SimTypeShort(signed=True, label="Int16"), "int32": SimTypeInt(signed=True, label="Int32"), "int64": SimTypePointer(SimTypeLongLong(signed=True, label="Int64"), offset=0), "float32": SimTypeFloat(size=32), "double64": SimTypePointer(SimTypeFloat(size=64), offset=0), "byteArray16": SimTypePointer(SimStruct({"byteArray16": SimTypeFixedSizeArray(SimTypeChar(label="Byte"), 16)}, name="FWP_BYTE_ARRAY16", pack=False, align=None), offset=0), "byteBlob": SimTypePointer(SimStruct({"size": SimTypeInt(signed=False, label="UInt32"), "data": SimTypePointer(SimTypeChar(label="Byte"), offset=0)}, name="FWP_BYTE_BLOB", pack=False, align=None), offset=0), "sid": SimTypePointer(SimTypeBottom(label="SID"), offset=0), "sd": SimTypePointer(SimStruct({"size": SimTypeInt(signed=False, label="UInt32"), "data": SimTypePointer(SimTypeChar(label="Byte"), offset=0)}, name="FWP_BYTE_BLOB", pack=False, align=None), offset=0), "tokenInformation": SimTypePointer(SimStruct({"sidCount": SimTypeInt(signed=False, label="UInt32"), "sids": SimTypePointer(SimTypeBottom(label="SID_AND_ATTRIBUTES"), offset=0), "restrictedSidCount": SimTypeInt(signed=False, label="UInt32"), "restrictedSids": SimTypePointer(SimTypeBottom(label="SID_AND_ATTRIBUTES"), offset=0)}, name="FWP_TOKEN_INFORMATION", pack=False, align=None), offset=0), "tokenAccessInformation": SimTypePointer(SimStruct({"size": SimTypeInt(signed=False, label="UInt32"), "data": SimTypePointer(SimTypeChar(label="Byte"), offset=0)}, name="FWP_BYTE_BLOB", pack=False, align=None), offset=0), "unicodeString": SimTypePointer(SimTypeChar(label="Char"), offset=0), "byteArray6": SimTypePointer(SimStruct({"byteArray6": SimTypeFixedSizeArray(SimTypeChar(label="Byte"), 6)}, name="FWP_BYTE_ARRAY6", pack=False, align=None), offset=0), "bitmapArray64": SimTypePointer(SimStruct({"bitmapArray64": SimTypeFixedSizeArray(SimTypeChar(label="Byte"), 8)}, name="FWP_BITMAP_ARRAY64_", pack=False, align=None), offset=0), "v4AddrMask": SimTypePointer(SimStruct({"addr": SimTypeInt(signed=False, label="UInt32"), "mask": SimTypeInt(signed=False, label="UInt32")}, name="FWP_V4_ADDR_AND_MASK", pack=False, align=None), offset=0), "v6AddrMask": SimTypePointer(SimStruct({"addr": SimTypeFixedSizeArray(SimTypeChar(label="Byte"), 16), "prefixLength": SimTypeChar(label="Byte")}, name="FWP_V6_ADDR_AND_MASK", pack=False, align=None), offset=0), "rangeValue": SimTypePointer(SimStruct({"valueLow": SimStruct({"type": SimTypeInt(signed=False, label="FWP_DATA_TYPE"), "Anonymous": SimUnion({"uint8": SimTypeChar(label="Byte"), "uint16": SimTypeShort(signed=False, label="UInt16"), "uint32": SimTypeInt(signed=False, label="UInt32"), "uint64": SimTypePointer(SimTypeLongLong(signed=False, label="UInt64"), offset=0), "int8": SimTypeChar(label="SByte"), "int16": SimTypeShort(signed=True, label="Int16"), "int32": SimTypeInt(signed=True, label="Int32"), "int64": SimTypePointer(SimTypeLongLong(signed=True, label="Int64"), offset=0), "float32": SimTypeFloat(size=32), "double64": SimTypePointer(SimTypeFloat(size=64), offset=0), "byteArray16": SimTypePointer(SimStruct({"byteArray16": SimTypeFixedSizeArray(SimTypeChar(label="Byte"), 16)}, name="FWP_BYTE_ARRAY16", pack=False, align=None), offset=0), "byteBlob": SimTypePointer(SimStruct({"size": SimTypeInt(signed=False, label="UInt32"), "data": SimTypePointer(SimTypeChar(label="Byte"), offset=0)}, name="FWP_BYTE_BLOB", pack=False, align=None), offset=0), "sid": SimTypePointer(SimTypeBottom(label="SID"), offset=0), "sd": SimTypePointer(SimStruct({"size": SimTypeInt(signed=False, label="UInt32"), "data": SimTypePointer(SimTypeChar(label="Byte"), offset=0)}, name="FWP_BYTE_BLOB", pack=False, align=None), offset=0), "tokenInformation": SimTypePointer(SimStruct({"sidCount": SimTypeInt(signed=False, label="UInt32"), "sids": SimTypePointer(SimTypeBottom(label="SID_AND_ATTRIBUTES"), offset=0), "restrictedSidCount": SimTypeInt(signed=False, label="UInt32"), "restrictedSids": SimTypePointer(SimTypeBottom(label="SID_AND_ATTRIBUTES"), offset=0)}, name="FWP_TOKEN_INFORMATION", pack=False, align=None), offset=0), "tokenAccessInformation": SimTypePointer(SimStruct({"size": SimTypeInt(signed=False, label="UInt32"), "data": SimTypePointer(SimTypeChar(label="Byte"), offset=0)}, name="FWP_BYTE_BLOB", pack=False, align=None), offset=0), "unicodeString": SimTypePointer(SimTypeChar(label="Char"), offset=0), "byteArray6": SimTypePointer(SimStruct({"byteArray6": SimTypeFixedSizeArray(SimTypeChar(label="Byte"), 6)}, name="FWP_BYTE_ARRAY6", pack=False, align=None), offset=0), "bitmapArray64": SimTypePointer(SimStruct({"bitmapArray64": SimTypeFixedSizeArray(SimTypeChar(label="Byte"), 8)}, name="FWP_BITMAP_ARRAY64_", pack=False, align=None), offset=0)}, name="<anon>", label="None")}, name="FWP_VALUE0", pack=False, align=None), "valueHigh": SimStruct({"type": SimTypeInt(signed=False, label="FWP_DATA_TYPE"), "Anonymous": SimUnion({"uint8": SimTypeChar(label="Byte"), "uint16": SimTypeShort(signed=False, label="UInt16"), "uint32": SimTypeInt(signed=False, label="UInt32"), "uint64": SimTypePointer(SimTypeLongLong(signed=False, label="UInt64"), offset=0), "int8": SimTypeChar(label="SByte"), "int16": SimTypeShort(signed=True, label="Int16"), "int32": SimTypeInt(signed=True, label="Int32"), "int64": SimTypePointer(SimTypeLongLong(signed=True, label="Int64"), offset=0), "float32": SimTypeFloat(size=32), "double64": SimTypePointer(SimTypeFloat(size=64), offset=0), "byteArray16": SimTypePointer(SimStruct({"byteArray16": SimTypeFixedSizeArray(SimTypeChar(label="Byte"), 16)}, name="FWP_BYTE_ARRAY16", pack=False, align=None), offset=0), "byteBlob": SimTypePointer(SimStruct({"size": SimTypeInt(signed=False, label="UInt32"), "data": SimTypePointer(SimTypeChar(label="Byte"), offset=0)}, name="FWP_BYTE_BLOB", pack=False, align=None), offset=0), "sid": SimTypePointer(SimTypeBottom(label="SID"), offset=0), "sd": SimTypePointer(SimStruct({"size": SimTypeInt(signed=False, label="UInt32"), "data": SimTypePointer(SimTypeChar(label="Byte"), offset=0)}, name="FWP_BYTE_BLOB", pack=False, align=None), offset=0), "tokenInformation": SimTypePointer(SimStruct({"sidCount": SimTypeInt(signed=False, label="UInt32"), "sids": SimTypePointer(SimTypeBottom(label="SID_AND_ATTRIBUTES"), offset=0), "restrictedSidCount": SimTypeInt(signed=False, label="UInt32"), "restrictedSids": SimTypePointer(SimTypeBottom(label="SID_AND_ATTRIBUTES"), offset=0)}, name="FWP_TOKEN_INFORMATION", pack=False, align=None), offset=0), "tokenAccessInformation": SimTypePointer(SimStruct({"size": SimTypeInt(signed=False, label="UInt32"), "data": SimTypePointer(SimTypeChar(label="Byte"), offset=0)}, name="FWP_BYTE_BLOB", pack=False, align=None), offset=0), "unicodeString": SimTypePointer(SimTypeChar(label="Char"), offset=0), "byteArray6": SimTypePointer(SimStruct({"byteArray6": SimTypeFixedSizeArray(SimTypeChar(label="Byte"), 6)}, name="FWP_BYTE_ARRAY6", pack=False, align=None), offset=0), "bitmapArray64": SimTypePointer(SimStruct({"bitmapArray64": SimTypeFixedSizeArray(SimTypeChar(label="Byte"), 8)}, name="FWP_BITMAP_ARRAY64_", pack=False, align=None), offset=0)}, name="<anon>", label="None")}, name="FWP_VALUE0", pack=False, align=None)}, name="FWP_RANGE0", pack=False, align=None), offset=0)}, name="<anon>", label="None")}, name="FWP_CONDITION_VALUE0", pack=False, align=None)}, name="FWPM_FILTER_CONDITION0", pack=False, align=None), label="LPArray", offset=0), SimTypePointer(SimTypeBottom(label="Guid"), offset=0), SimTypePointer(SimStruct({"Revision": SimTypeChar(label="Byte"), "Sbz1": SimTypeChar(label="Byte"), "Control": SimTypeShort(signed=False, label="UInt16"), "Owner": SimTypePointer(SimTypeInt(signed=True, label="Int"), label="IntPtr", offset=0), "Group": SimTypePointer(SimTypeInt(signed=True, label="Int"), label="IntPtr", offset=0), "Sacl": SimTypePointer(SimStruct({"AclRevision": SimTypeChar(label="Byte"), "Sbz1": SimTypeChar(label="Byte"), "AclSize": SimTypeShort(signed=False, label="UInt16"), "AceCount": SimTypeShort(signed=False, label="UInt16"), "Sbz2": SimTypeShort(signed=False, label="UInt16")}, name="ACL", pack=False, align=None), offset=0), "Dacl": SimTypePointer(SimStruct({"AclRevision": SimTypeChar(label="Byte"), "Sbz1": SimTypeChar(label="Byte"), "AclSize": SimTypeShort(signed=False, label="UInt16"), "AceCount": SimTypeShort(signed=False, label="UInt16"), "Sbz2": SimTypeShort(signed=False, label="UInt16")}, name="ACL", pack=False, align=None), offset=0)}, name="SECURITY_DESCRIPTOR", pack=False, align=None), offset=0)], SimTypeInt(signed=False, label="UInt32"), arg_names=["engineHandle", "flags", "mainModePolicy", "tunnelPolicy", "numFilterConditions", "filterConditions", "keyModKey", "sd"]),
        # 
        'FwpmIPsecTunnelAdd2': SimTypeFunction([SimTypePointer(SimTypeInt(signed=True, label="Int"), label="IntPtr", offset=0), SimTypeInt(signed=False, label="UInt32"), SimTypePointer(SimStruct({"providerContextKey": SimTypeBottom(label="Guid"), "displayData": SimStruct({"name": SimTypePointer(SimTypeChar(label="Char"), offset=0), "description": SimTypePointer(SimTypeChar(label="Char"), offset=0)}, name="FWPM_DISPLAY_DATA0", pack=False, align=None), "flags": SimTypeInt(signed=False, label="UInt32"), "providerKey": SimTypePointer(SimTypeBottom(label="Guid"), offset=0), "providerData": SimStruct({"size": SimTypeInt(signed=False, label="UInt32"), "data": SimTypePointer(SimTypeChar(label="Byte"), offset=0)}, name="FWP_BYTE_BLOB", pack=False, align=None), "type": SimTypeInt(signed=False, label="FWPM_PROVIDER_CONTEXT_TYPE"), "Anonymous": SimUnion({"keyingPolicy": SimTypePointer(SimStruct({"numKeyMods": SimTypeInt(signed=False, label="UInt32"), "keyModKeys": SimTypePointer(SimTypeBottom(label="Guid"), offset=0), "flags": SimTypeInt(signed=False, label="UInt32")}, name="IPSEC_KEYING_POLICY1", pack=False, align=None), offset=0), "ikeQmTransportPolicy": SimTypePointer(SimStruct({"numIpsecProposals": SimTypeInt(signed=False, label="UInt32"), "ipsecProposals": SimTypePointer(SimStruct({"lifetime": SimStruct({"lifetimeSeconds": SimTypeInt(signed=False, label="UInt32"), "lifetimeKilobytes": SimTypeInt(signed=False, label="UInt32"), "lifetimePackets": SimTypeInt(signed=False, label="UInt32")}, name="IPSEC_SA_LIFETIME0", pack=False, align=None), "numSaTransforms": SimTypeInt(signed=False, label="UInt32"), "saTransforms": SimTypePointer(SimStruct({"ipsecTransformType": SimTypeInt(signed=False, label="IPSEC_TRANSFORM_TYPE"), "Anonymous": SimUnion({"ahTransform": SimTypePointer(SimStruct({"authTransformId": SimStruct({"authType": SimTypeInt(signed=False, label="IPSEC_AUTH_TYPE"), "authConfig": SimTypeChar(label="Byte")}, name="IPSEC_AUTH_TRANSFORM_ID0", pack=False, align=None), "cryptoModuleId": SimTypePointer(SimTypeBottom(label="Guid"), offset=0)}, name="IPSEC_AUTH_TRANSFORM0", pack=False, align=None), offset=0), "espAuthTransform": SimTypePointer(SimStruct({"authTransformId": SimStruct({"authType": SimTypeInt(signed=False, label="IPSEC_AUTH_TYPE"), "authConfig": SimTypeChar(label="Byte")}, name="IPSEC_AUTH_TRANSFORM_ID0", pack=False, align=None), "cryptoModuleId": SimTypePointer(SimTypeBottom(label="Guid"), offset=0)}, name="IPSEC_AUTH_TRANSFORM0", pack=False, align=None), offset=0), "espCipherTransform": SimTypePointer(SimStruct({"cipherTransformId": SimStruct({"cipherType": SimTypeInt(signed=False, label="IPSEC_CIPHER_TYPE"), "cipherConfig": SimTypeChar(label="Byte")}, name="IPSEC_CIPHER_TRANSFORM_ID0", pack=False, align=None), "cryptoModuleId": SimTypePointer(SimTypeBottom(label="Guid"), offset=0)}, name="IPSEC_CIPHER_TRANSFORM0", pack=False, align=None), offset=0), "espAuthAndCipherTransform": SimTypePointer(SimStruct({"authTransform": SimStruct({"authTransformId": SimStruct({"authType": SimTypeInt(signed=False, label="IPSEC_AUTH_TYPE"), "authConfig": SimTypeChar(label="Byte")}, name="IPSEC_AUTH_TRANSFORM_ID0", pack=False, align=None), "cryptoModuleId": SimTypePointer(SimTypeBottom(label="Guid"), offset=0)}, name="IPSEC_AUTH_TRANSFORM0", pack=False, align=None), "cipherTransform": SimStruct({"cipherTransformId": SimStruct({"cipherType": SimTypeInt(signed=False, label="IPSEC_CIPHER_TYPE"), "cipherConfig": SimTypeChar(label="Byte")}, name="IPSEC_CIPHER_TRANSFORM_ID0", pack=False, align=None), "cryptoModuleId": SimTypePointer(SimTypeBottom(label="Guid"), offset=0)}, name="IPSEC_CIPHER_TRANSFORM0", pack=False, align=None)}, name="IPSEC_AUTH_AND_CIPHER_TRANSFORM0", pack=False, align=None), offset=0), "espAuthFwTransform": SimTypePointer(SimStruct({"authTransformId": SimStruct({"authType": SimTypeInt(signed=False, label="IPSEC_AUTH_TYPE"), "authConfig": SimTypeChar(label="Byte")}, name="IPSEC_AUTH_TRANSFORM_ID0", pack=False, align=None), "cryptoModuleId": SimTypePointer(SimTypeBottom(label="Guid"), offset=0)}, name="IPSEC_AUTH_TRANSFORM0", pack=False, align=None), offset=0)}, name="<anon>", label="None")}, name="IPSEC_SA_TRANSFORM0", pack=False, align=None), offset=0), "pfsGroup": SimTypeInt(signed=False, label="IPSEC_PFS_GROUP")}, name="IPSEC_PROPOSAL0", pack=False, align=None), offset=0), "flags": SimTypeInt(signed=False, label="IPSEC_POLICY_FLAG"), "ndAllowClearTimeoutSeconds": SimTypeInt(signed=False, label="UInt32"), "saIdleTimeout": SimStruct({"idleTimeoutSeconds": SimTypeInt(signed=False, label="UInt32"), "idleTimeoutSecondsFailOver": SimTypeInt(signed=False, label="UInt32")}, name="IPSEC_SA_IDLE_TIMEOUT0", pack=False, align=None), "emPolicy": SimTypePointer(SimStruct({"numAuthenticationMethods": SimTypeInt(signed=False, label="UInt32"), "authenticationMethods": SimTypePointer(SimStruct({"authenticationMethodType": SimTypeInt(signed=False, label="IKEEXT_AUTHENTICATION_METHOD_TYPE"), "Anonymous": SimUnion({"presharedKeyAuthentication": SimStruct({"presharedKey": SimStruct({"size": SimTypeInt(signed=False, label="UInt32"), "data": SimTypePointer(SimTypeChar(label="Byte"), offset=0)}, name="FWP_BYTE_BLOB", pack=False, align=None), "flags": SimTypeInt(signed=False, label="IKEEXT_PRESHARED_KEY_AUTHENTICATION_FLAGS")}, name="IKEEXT_PRESHARED_KEY_AUTHENTICATION1", pack=False, align=None), "certificateAuthentication": SimStruct({"inboundConfigType": SimTypeInt(signed=False, label="IKEEXT_CERT_CONFIG_TYPE"), "Anonymous1": SimUnion({"Anonymous1": SimStruct({"inboundRootArraySize": SimTypeInt(signed=False, label="UInt32"), "inboundRootCriteria": SimTypePointer(SimStruct({"certData": SimStruct({"size": SimTypeInt(signed=False, label="UInt32"), "data": SimTypePointer(SimTypeChar(label="Byte"), offset=0)}, name="FWP_BYTE_BLOB", pack=False, align=None), "certHash": SimStruct({"size": SimTypeInt(signed=False, label="UInt32"), "data": SimTypePointer(SimTypeChar(label="Byte"), offset=0)}, name="FWP_BYTE_BLOB", pack=False, align=None), "eku": SimTypePointer(SimStruct({"numEku": SimTypeInt(signed=False, label="UInt32"), "eku": SimTypePointer(SimTypePointer(SimTypeChar(label="Byte"), offset=0), offset=0)}, name="IKEEXT_CERT_EKUS0", pack=False, align=None), offset=0), "name": SimTypePointer(SimStruct({"nameType": SimTypeInt(signed=False, label="IKEEXT_CERT_CRITERIA_NAME_TYPE"), "certName": SimTypePointer(SimTypeChar(label="Char"), offset=0)}, name="IKEEXT_CERT_NAME0", pack=False, align=None), offset=0), "flags": SimTypeInt(signed=False, label="UInt32")}, name="IKEEXT_CERTIFICATE_CRITERIA0", pack=False, align=None), offset=0)}, name="_Anonymous1_e__Struct", pack=False, align=None), "Anonymous2": SimStruct({"inboundEnterpriseStoreArraySize": SimTypeInt(signed=False, label="UInt32"), "inboundEnterpriseStoreCriteria": SimTypePointer(SimStruct({"certData": SimStruct({"size": SimTypeInt(signed=False, label="UInt32"), "data": SimTypePointer(SimTypeChar(label="Byte"), offset=0)}, name="FWP_BYTE_BLOB", pack=False, align=None), "certHash": SimStruct({"size": SimTypeInt(signed=False, label="UInt32"), "data": SimTypePointer(SimTypeChar(label="Byte"), offset=0)}, name="FWP_BYTE_BLOB", pack=False, align=None), "eku": SimTypePointer(SimStruct({"numEku": SimTypeInt(signed=False, label="UInt32"), "eku": SimTypePointer(SimTypePointer(SimTypeChar(label="Byte"), offset=0), offset=0)}, name="IKEEXT_CERT_EKUS0", pack=False, align=None), offset=0), "name": SimTypePointer(SimStruct({"nameType": SimTypeInt(signed=False, label="IKEEXT_CERT_CRITERIA_NAME_TYPE"), "certName": SimTypePointer(SimTypeChar(label="Char"), offset=0)}, name="IKEEXT_CERT_NAME0", pack=False, align=None), offset=0), "flags": SimTypeInt(signed=False, label="UInt32")}, name="IKEEXT_CERTIFICATE_CRITERIA0", pack=False, align=None), offset=0)}, name="_Anonymous2_e__Struct", pack=False, align=None), "Anonymous3": SimStruct({"inboundRootStoreArraySize": SimTypeInt(signed=False, label="UInt32"), "inboundTrustedRootStoreCriteria": SimTypePointer(SimStruct({"certData": SimStruct({"size": SimTypeInt(signed=False, label="UInt32"), "data": SimTypePointer(SimTypeChar(label="Byte"), offset=0)}, name="FWP_BYTE_BLOB", pack=False, align=None), "certHash": SimStruct({"size": SimTypeInt(signed=False, label="UInt32"), "data": SimTypePointer(SimTypeChar(label="Byte"), offset=0)}, name="FWP_BYTE_BLOB", pack=False, align=None), "eku": SimTypePointer(SimStruct({"numEku": SimTypeInt(signed=False, label="UInt32"), "eku": SimTypePointer(SimTypePointer(SimTypeChar(label="Byte"), offset=0), offset=0)}, name="IKEEXT_CERT_EKUS0", pack=False, align=None), offset=0), "name": SimTypePointer(SimStruct({"nameType": SimTypeInt(signed=False, label="IKEEXT_CERT_CRITERIA_NAME_TYPE"), "certName": SimTypePointer(SimTypeChar(label="Char"), offset=0)}, name="IKEEXT_CERT_NAME0", pack=False, align=None), offset=0), "flags": SimTypeInt(signed=False, label="UInt32")}, name="IKEEXT_CERTIFICATE_CRITERIA0", pack=False, align=None), offset=0)}, name="_Anonymous3_e__Struct", pack=False, align=None)}, name="<anon>", label="None"), "outboundConfigType": SimTypeInt(signed=False, label="IKEEXT_CERT_CONFIG_TYPE"), "Anonymous2": SimUnion({"Anonymous1": SimStruct({"outboundRootArraySize": SimTypeInt(signed=False, label="UInt32"), "outboundRootCriteria": SimTypePointer(SimStruct({"certData": SimStruct({"size": SimTypeInt(signed=False, label="UInt32"), "data": SimTypePointer(SimTypeChar(label="Byte"), offset=0)}, name="FWP_BYTE_BLOB", pack=False, align=None), "certHash": SimStruct({"size": SimTypeInt(signed=False, label="UInt32"), "data": SimTypePointer(SimTypeChar(label="Byte"), offset=0)}, name="FWP_BYTE_BLOB", pack=False, align=None), "eku": SimTypePointer(SimStruct({"numEku": SimTypeInt(signed=False, label="UInt32"), "eku": SimTypePointer(SimTypePointer(SimTypeChar(label="Byte"), offset=0), offset=0)}, name="IKEEXT_CERT_EKUS0", pack=False, align=None), offset=0), "name": SimTypePointer(SimStruct({"nameType": SimTypeInt(signed=False, label="IKEEXT_CERT_CRITERIA_NAME_TYPE"), "certName": SimTypePointer(SimTypeChar(label="Char"), offset=0)}, name="IKEEXT_CERT_NAME0", pack=False, align=None), offset=0), "flags": SimTypeInt(signed=False, label="UInt32")}, name="IKEEXT_CERTIFICATE_CRITERIA0", pack=False, align=None), offset=0)}, name="_Anonymous1_e__Struct", pack=False, align=None), "Anonymous2": SimStruct({"outboundEnterpriseStoreArraySize": SimTypeInt(signed=False, label="UInt32"), "outboundEnterpriseStoreCriteria": SimTypePointer(SimStruct({"certData": SimStruct({"size": SimTypeInt(signed=False, label="UInt32"), "data": SimTypePointer(SimTypeChar(label="Byte"), offset=0)}, name="FWP_BYTE_BLOB", pack=False, align=None), "certHash": SimStruct({"size": SimTypeInt(signed=False, label="UInt32"), "data": SimTypePointer(SimTypeChar(label="Byte"), offset=0)}, name="FWP_BYTE_BLOB", pack=False, align=None), "eku": SimTypePointer(SimStruct({"numEku": SimTypeInt(signed=False, label="UInt32"), "eku": SimTypePointer(SimTypePointer(SimTypeChar(label="Byte"), offset=0), offset=0)}, name="IKEEXT_CERT_EKUS0", pack=False, align=None), offset=0), "name": SimTypePointer(SimStruct({"nameType": SimTypeInt(signed=False, label="IKEEXT_CERT_CRITERIA_NAME_TYPE"), "certName": SimTypePointer(SimTypeChar(label="Char"), offset=0)}, name="IKEEXT_CERT_NAME0", pack=False, align=None), offset=0), "flags": SimTypeInt(signed=False, label="UInt32")}, name="IKEEXT_CERTIFICATE_CRITERIA0", pack=False, align=None), offset=0)}, name="_Anonymous2_e__Struct", pack=False, align=None), "Anonymous3": SimStruct({"outboundRootStoreArraySize": SimTypeInt(signed=False, label="UInt32"), "outboundTrustedRootStoreCriteria": SimTypePointer(SimStruct({"certData": SimStruct({"size": SimTypeInt(signed=False, label="UInt32"), "data": SimTypePointer(SimTypeChar(label="Byte"), offset=0)}, name="FWP_BYTE_BLOB", pack=False, align=None), "certHash": SimStruct({"size": SimTypeInt(signed=False, label="UInt32"), "data": SimTypePointer(SimTypeChar(label="Byte"), offset=0)}, name="FWP_BYTE_BLOB", pack=False, align=None), "eku": SimTypePointer(SimStruct({"numEku": SimTypeInt(signed=False, label="UInt32"), "eku": SimTypePointer(SimTypePointer(SimTypeChar(label="Byte"), offset=0), offset=0)}, name="IKEEXT_CERT_EKUS0", pack=False, align=None), offset=0), "name": SimTypePointer(SimStruct({"nameType": SimTypeInt(signed=False, label="IKEEXT_CERT_CRITERIA_NAME_TYPE"), "certName": SimTypePointer(SimTypeChar(label="Char"), offset=0)}, name="IKEEXT_CERT_NAME0", pack=False, align=None), offset=0), "flags": SimTypeInt(signed=False, label="UInt32")}, name="IKEEXT_CERTIFICATE_CRITERIA0", pack=False, align=None), offset=0)}, name="_Anonymous3_e__Struct", pack=False, align=None)}, name="<anon>", label="None"), "flags": SimTypeInt(signed=False, label="IKEEXT_CERT_AUTH"), "localCertLocationUrl": SimStruct({"size": SimTypeInt(signed=False, label="UInt32"), "data": SimTypePointer(SimTypeChar(label="Byte"), offset=0)}, name="FWP_BYTE_BLOB", pack=False, align=None)}, name="IKEEXT_CERTIFICATE_AUTHENTICATION2", pack=False, align=None), "kerberosAuthentication": SimStruct({"flags": SimTypeInt(signed=False, label="IKEEXT_KERBEROS_AUTHENTICATION_FLAGS"), "proxyServer": SimTypePointer(SimTypeChar(label="Char"), offset=0)}, name="IKEEXT_KERBEROS_AUTHENTICATION1", pack=False, align=None), "reservedAuthentication": SimStruct({"flags": SimTypeInt(signed=False, label="IKEEXT_RESERVED_AUTHENTICATION_FLAGS")}, name="IKEEXT_RESERVED_AUTHENTICATION0", pack=False, align=None), "ntlmV2Authentication": SimStruct({"flags": SimTypeInt(signed=False, label="UInt32")}, name="IKEEXT_NTLM_V2_AUTHENTICATION0", pack=False, align=None), "sslAuthentication": SimStruct({"inboundConfigType": SimTypeInt(signed=False, label="IKEEXT_CERT_CONFIG_TYPE"), "Anonymous1": SimUnion({"Anonymous1": SimStruct({"inboundRootArraySize": SimTypeInt(signed=False, label="UInt32"), "inboundRootCriteria": SimTypePointer(SimStruct({"certData": SimStruct({"size": SimTypeInt(signed=False, label="UInt32"), "data": SimTypePointer(SimTypeChar(label="Byte"), offset=0)}, name="FWP_BYTE_BLOB", pack=False, align=None), "certHash": SimStruct({"size": SimTypeInt(signed=False, label="UInt32"), "data": SimTypePointer(SimTypeChar(label="Byte"), offset=0)}, name="FWP_BYTE_BLOB", pack=False, align=None), "eku": SimTypePointer(SimStruct({"numEku": SimTypeInt(signed=False, label="UInt32"), "eku": SimTypePointer(SimTypePointer(SimTypeChar(label="Byte"), offset=0), offset=0)}, name="IKEEXT_CERT_EKUS0", pack=False, align=None), offset=0), "name": SimTypePointer(SimStruct({"nameType": SimTypeInt(signed=False, label="IKEEXT_CERT_CRITERIA_NAME_TYPE"), "certName": SimTypePointer(SimTypeChar(label="Char"), offset=0)}, name="IKEEXT_CERT_NAME0", pack=False, align=None), offset=0), "flags": SimTypeInt(signed=False, label="UInt32")}, name="IKEEXT_CERTIFICATE_CRITERIA0", pack=False, align=None), offset=0)}, name="_Anonymous1_e__Struct", pack=False, align=None), "Anonymous2": SimStruct({"inboundEnterpriseStoreArraySize": SimTypeInt(signed=False, label="UInt32"), "inboundEnterpriseStoreCriteria": SimTypePointer(SimStruct({"certData": SimStruct({"size": SimTypeInt(signed=False, label="UInt32"), "data": SimTypePointer(SimTypeChar(label="Byte"), offset=0)}, name="FWP_BYTE_BLOB", pack=False, align=None), "certHash": SimStruct({"size": SimTypeInt(signed=False, label="UInt32"), "data": SimTypePointer(SimTypeChar(label="Byte"), offset=0)}, name="FWP_BYTE_BLOB", pack=False, align=None), "eku": SimTypePointer(SimStruct({"numEku": SimTypeInt(signed=False, label="UInt32"), "eku": SimTypePointer(SimTypePointer(SimTypeChar(label="Byte"), offset=0), offset=0)}, name="IKEEXT_CERT_EKUS0", pack=False, align=None), offset=0), "name": SimTypePointer(SimStruct({"nameType": SimTypeInt(signed=False, label="IKEEXT_CERT_CRITERIA_NAME_TYPE"), "certName": SimTypePointer(SimTypeChar(label="Char"), offset=0)}, name="IKEEXT_CERT_NAME0", pack=False, align=None), offset=0), "flags": SimTypeInt(signed=False, label="UInt32")}, name="IKEEXT_CERTIFICATE_CRITERIA0", pack=False, align=None), offset=0)}, name="_Anonymous2_e__Struct", pack=False, align=None), "Anonymous3": SimStruct({"inboundRootStoreArraySize": SimTypeInt(signed=False, label="UInt32"), "inboundTrustedRootStoreCriteria": SimTypePointer(SimStruct({"certData": SimStruct({"size": SimTypeInt(signed=False, label="UInt32"), "data": SimTypePointer(SimTypeChar(label="Byte"), offset=0)}, name="FWP_BYTE_BLOB", pack=False, align=None), "certHash": SimStruct({"size": SimTypeInt(signed=False, label="UInt32"), "data": SimTypePointer(SimTypeChar(label="Byte"), offset=0)}, name="FWP_BYTE_BLOB", pack=False, align=None), "eku": SimTypePointer(SimStruct({"numEku": SimTypeInt(signed=False, label="UInt32"), "eku": SimTypePointer(SimTypePointer(SimTypeChar(label="Byte"), offset=0), offset=0)}, name="IKEEXT_CERT_EKUS0", pack=False, align=None), offset=0), "name": SimTypePointer(SimStruct({"nameType": SimTypeInt(signed=False, label="IKEEXT_CERT_CRITERIA_NAME_TYPE"), "certName": SimTypePointer(SimTypeChar(label="Char"), offset=0)}, name="IKEEXT_CERT_NAME0", pack=False, align=None), offset=0), "flags": SimTypeInt(signed=False, label="UInt32")}, name="IKEEXT_CERTIFICATE_CRITERIA0", pack=False, align=None), offset=0)}, name="_Anonymous3_e__Struct", pack=False, align=None)}, name="<anon>", label="None"), "outboundConfigType": SimTypeInt(signed=False, label="IKEEXT_CERT_CONFIG_TYPE"), "Anonymous2": SimUnion({"Anonymous1": SimStruct({"outboundRootArraySize": SimTypeInt(signed=False, label="UInt32"), "outboundRootCriteria": SimTypePointer(SimStruct({"certData": SimStruct({"size": SimTypeInt(signed=False, label="UInt32"), "data": SimTypePointer(SimTypeChar(label="Byte"), offset=0)}, name="FWP_BYTE_BLOB", pack=False, align=None), "certHash": SimStruct({"size": SimTypeInt(signed=False, label="UInt32"), "data": SimTypePointer(SimTypeChar(label="Byte"), offset=0)}, name="FWP_BYTE_BLOB", pack=False, align=None), "eku": SimTypePointer(SimStruct({"numEku": SimTypeInt(signed=False, label="UInt32"), "eku": SimTypePointer(SimTypePointer(SimTypeChar(label="Byte"), offset=0), offset=0)}, name="IKEEXT_CERT_EKUS0", pack=False, align=None), offset=0), "name": SimTypePointer(SimStruct({"nameType": SimTypeInt(signed=False, label="IKEEXT_CERT_CRITERIA_NAME_TYPE"), "certName": SimTypePointer(SimTypeChar(label="Char"), offset=0)}, name="IKEEXT_CERT_NAME0", pack=False, align=None), offset=0), "flags": SimTypeInt(signed=False, label="UInt32")}, name="IKEEXT_CERTIFICATE_CRITERIA0", pack=False, align=None), offset=0)}, name="_Anonymous1_e__Struct", pack=False, align=None), "Anonymous2": SimStruct({"outboundEnterpriseStoreArraySize": SimTypeInt(signed=False, label="UInt32"), "outboundEnterpriseStoreCriteria": SimTypePointer(SimStruct({"certData": SimStruct({"size": SimTypeInt(signed=False, label="UInt32"), "data": SimTypePointer(SimTypeChar(label="Byte"), offset=0)}, name="FWP_BYTE_BLOB", pack=False, align=None), "certHash": SimStruct({"size": SimTypeInt(signed=False, label="UInt32"), "data": SimTypePointer(SimTypeChar(label="Byte"), offset=0)}, name="FWP_BYTE_BLOB", pack=False, align=None), "eku": SimTypePointer(SimStruct({"numEku": SimTypeInt(signed=False, label="UInt32"), "eku": SimTypePointer(SimTypePointer(SimTypeChar(label="Byte"), offset=0), offset=0)}, name="IKEEXT_CERT_EKUS0", pack=False, align=None), offset=0), "name": SimTypePointer(SimStruct({"nameType": SimTypeInt(signed=False, label="IKEEXT_CERT_CRITERIA_NAME_TYPE"), "certName": SimTypePointer(SimTypeChar(label="Char"), offset=0)}, name="IKEEXT_CERT_NAME0", pack=False, align=None), offset=0), "flags": SimTypeInt(signed=False, label="UInt32")}, name="IKEEXT_CERTIFICATE_CRITERIA0", pack=False, align=None), offset=0)}, name="_Anonymous2_e__Struct", pack=False, align=None), "Anonymous3": SimStruct({"outboundRootStoreArraySize": SimTypeInt(signed=False, label="UInt32"), "outboundTrustedRootStoreCriteria": SimTypePointer(SimStruct({"certData": SimStruct({"size": SimTypeInt(signed=False, label="UInt32"), "data": SimTypePointer(SimTypeChar(label="Byte"), offset=0)}, name="FWP_BYTE_BLOB", pack=False, align=None), "certHash": SimStruct({"size": SimTypeInt(signed=False, label="UInt32"), "data": SimTypePointer(SimTypeChar(label="Byte"), offset=0)}, name="FWP_BYTE_BLOB", pack=False, align=None), "eku": SimTypePointer(SimStruct({"numEku": SimTypeInt(signed=False, label="UInt32"), "eku": SimTypePointer(SimTypePointer(SimTypeChar(label="Byte"), offset=0), offset=0)}, name="IKEEXT_CERT_EKUS0", pack=False, align=None), offset=0), "name": SimTypePointer(SimStruct({"nameType": SimTypeInt(signed=False, label="IKEEXT_CERT_CRITERIA_NAME_TYPE"), "certName": SimTypePointer(SimTypeChar(label="Char"), offset=0)}, name="IKEEXT_CERT_NAME0", pack=False, align=None), offset=0), "flags": SimTypeInt(signed=False, label="UInt32")}, name="IKEEXT_CERTIFICATE_CRITERIA0", pack=False, align=None), offset=0)}, name="_Anonymous3_e__Struct", pack=False, align=None)}, name="<anon>", label="None"), "flags": SimTypeInt(signed=False, label="IKEEXT_CERT_AUTH"), "localCertLocationUrl": SimStruct({"size": SimTypeInt(signed=False, label="UInt32"), "data": SimTypePointer(SimTypeChar(label="Byte"), offset=0)}, name="FWP_BYTE_BLOB", pack=False, align=None)}, name="IKEEXT_CERTIFICATE_AUTHENTICATION2", pack=False, align=None), "cgaAuthentication": SimStruct({"keyContainerName": SimTypePointer(SimTypeChar(label="Char"), offset=0), "cspName": SimTypePointer(SimTypeChar(label="Char"), offset=0), "cspType": SimTypeInt(signed=False, label="UInt32"), "cgaModifier": SimStruct({"byteArray16": SimTypeFixedSizeArray(SimTypeChar(label="Byte"), 16)}, name="FWP_BYTE_ARRAY16", pack=False, align=None), "cgaCollisionCount": SimTypeChar(label="Byte")}, name="IKEEXT_IPV6_CGA_AUTHENTICATION0", pack=False, align=None), "eapAuthentication": SimStruct({"flags": SimTypeInt(signed=False, label="IKEEXT_EAP_AUTHENTICATION_FLAGS")}, name="IKEEXT_EAP_AUTHENTICATION0", pack=False, align=None)}, name="<anon>", label="None")}, name="IKEEXT_AUTHENTICATION_METHOD2", pack=False, align=None), offset=0), "initiatorImpersonationType": SimTypeInt(signed=False, label="IKEEXT_AUTHENTICATION_IMPERSONATION_TYPE")}, name="IKEEXT_EM_POLICY2", pack=False, align=None), offset=0)}, name="IPSEC_TRANSPORT_POLICY2", pack=False, align=None), offset=0), "ikeQmTunnelPolicy": SimTypePointer(SimStruct({"flags": SimTypeInt(signed=False, label="IPSEC_POLICY_FLAG"), "numIpsecProposals": SimTypeInt(signed=False, label="UInt32"), "ipsecProposals": SimTypePointer(SimStruct({"lifetime": SimStruct({"lifetimeSeconds": SimTypeInt(signed=False, label="UInt32"), "lifetimeKilobytes": SimTypeInt(signed=False, label="UInt32"), "lifetimePackets": SimTypeInt(signed=False, label="UInt32")}, name="IPSEC_SA_LIFETIME0", pack=False, align=None), "numSaTransforms": SimTypeInt(signed=False, label="UInt32"), "saTransforms": SimTypePointer(SimStruct({"ipsecTransformType": SimTypeInt(signed=False, label="IPSEC_TRANSFORM_TYPE"), "Anonymous": SimUnion({"ahTransform": SimTypePointer(SimStruct({"authTransformId": SimStruct({"authType": SimTypeInt(signed=False, label="IPSEC_AUTH_TYPE"), "authConfig": SimTypeChar(label="Byte")}, name="IPSEC_AUTH_TRANSFORM_ID0", pack=False, align=None), "cryptoModuleId": SimTypePointer(SimTypeBottom(label="Guid"), offset=0)}, name="IPSEC_AUTH_TRANSFORM0", pack=False, align=None), offset=0), "espAuthTransform": SimTypePointer(SimStruct({"authTransformId": SimStruct({"authType": SimTypeInt(signed=False, label="IPSEC_AUTH_TYPE"), "authConfig": SimTypeChar(label="Byte")}, name="IPSEC_AUTH_TRANSFORM_ID0", pack=False, align=None), "cryptoModuleId": SimTypePointer(SimTypeBottom(label="Guid"), offset=0)}, name="IPSEC_AUTH_TRANSFORM0", pack=False, align=None), offset=0), "espCipherTransform": SimTypePointer(SimStruct({"cipherTransformId": SimStruct({"cipherType": SimTypeInt(signed=False, label="IPSEC_CIPHER_TYPE"), "cipherConfig": SimTypeChar(label="Byte")}, name="IPSEC_CIPHER_TRANSFORM_ID0", pack=False, align=None), "cryptoModuleId": SimTypePointer(SimTypeBottom(label="Guid"), offset=0)}, name="IPSEC_CIPHER_TRANSFORM0", pack=False, align=None), offset=0), "espAuthAndCipherTransform": SimTypePointer(SimStruct({"authTransform": SimStruct({"authTransformId": SimStruct({"authType": SimTypeInt(signed=False, label="IPSEC_AUTH_TYPE"), "authConfig": SimTypeChar(label="Byte")}, name="IPSEC_AUTH_TRANSFORM_ID0", pack=False, align=None), "cryptoModuleId": SimTypePointer(SimTypeBottom(label="Guid"), offset=0)}, name="IPSEC_AUTH_TRANSFORM0", pack=False, align=None), "cipherTransform": SimStruct({"cipherTransformId": SimStruct({"cipherType": SimTypeInt(signed=False, label="IPSEC_CIPHER_TYPE"), "cipherConfig": SimTypeChar(label="Byte")}, name="IPSEC_CIPHER_TRANSFORM_ID0", pack=False, align=None), "cryptoModuleId": SimTypePointer(SimTypeBottom(label="Guid"), offset=0)}, name="IPSEC_CIPHER_TRANSFORM0", pack=False, align=None)}, name="IPSEC_AUTH_AND_CIPHER_TRANSFORM0", pack=False, align=None), offset=0), "espAuthFwTransform": SimTypePointer(SimStruct({"authTransformId": SimStruct({"authType": SimTypeInt(signed=False, label="IPSEC_AUTH_TYPE"), "authConfig": SimTypeChar(label="Byte")}, name="IPSEC_AUTH_TRANSFORM_ID0", pack=False, align=None), "cryptoModuleId": SimTypePointer(SimTypeBottom(label="Guid"), offset=0)}, name="IPSEC_AUTH_TRANSFORM0", pack=False, align=None), offset=0)}, name="<anon>", label="None")}, name="IPSEC_SA_TRANSFORM0", pack=False, align=None), offset=0), "pfsGroup": SimTypeInt(signed=False, label="IPSEC_PFS_GROUP")}, name="IPSEC_PROPOSAL0", pack=False, align=None), offset=0), "tunnelEndpoints": SimStruct({"ipVersion": SimTypeInt(signed=False, label="FWP_IP_VERSION"), "Anonymous1": SimUnion({"localV4Address": SimTypeInt(signed=False, label="UInt32"), "localV6Address": SimTypeFixedSizeArray(SimTypeChar(label="Byte"), 16)}, name="<anon>", label="None"), "Anonymous2": SimUnion({"remoteV4Address": SimTypeInt(signed=False, label="UInt32"), "remoteV6Address": SimTypeFixedSizeArray(SimTypeChar(label="Byte"), 16)}, name="<anon>", label="None"), "localIfLuid": SimTypeLongLong(signed=False, label="UInt64"), "remoteFqdn": SimTypePointer(SimTypeChar(label="Char"), offset=0), "numAddresses": SimTypeInt(signed=False, label="UInt32"), "remoteAddresses": SimTypePointer(SimStruct({"ipVersion": SimTypeInt(signed=False, label="FWP_IP_VERSION"), "Anonymous": SimUnion({"v4Address": SimTypeInt(signed=False, label="UInt32"), "v6Address": SimTypeFixedSizeArray(SimTypeChar(label="Byte"), 16)}, name="<anon>", label="None")}, name="IPSEC_TUNNEL_ENDPOINT0", pack=False, align=None), offset=0)}, name="IPSEC_TUNNEL_ENDPOINTS2", pack=False, align=None), "saIdleTimeout": SimStruct({"idleTimeoutSeconds": SimTypeInt(signed=False, label="UInt32"), "idleTimeoutSecondsFailOver": SimTypeInt(signed=False, label="UInt32")}, name="IPSEC_SA_IDLE_TIMEOUT0", pack=False, align=None), "emPolicy": SimTypePointer(SimStruct({"numAuthenticationMethods": SimTypeInt(signed=False, label="UInt32"), "authenticationMethods": SimTypePointer(SimStruct({"authenticationMethodType": SimTypeInt(signed=False, label="IKEEXT_AUTHENTICATION_METHOD_TYPE"), "Anonymous": SimUnion({"presharedKeyAuthentication": SimStruct({"presharedKey": SimStruct({"size": SimTypeInt(signed=False, label="UInt32"), "data": SimTypePointer(SimTypeChar(label="Byte"), offset=0)}, name="FWP_BYTE_BLOB", pack=False, align=None), "flags": SimTypeInt(signed=False, label="IKEEXT_PRESHARED_KEY_AUTHENTICATION_FLAGS")}, name="IKEEXT_PRESHARED_KEY_AUTHENTICATION1", pack=False, align=None), "certificateAuthentication": SimStruct({"inboundConfigType": SimTypeInt(signed=False, label="IKEEXT_CERT_CONFIG_TYPE"), "Anonymous1": SimUnion({"Anonymous1": SimStruct({"inboundRootArraySize": SimTypeInt(signed=False, label="UInt32"), "inboundRootCriteria": SimTypePointer(SimStruct({"certData": SimStruct({"size": SimTypeInt(signed=False, label="UInt32"), "data": SimTypePointer(SimTypeChar(label="Byte"), offset=0)}, name="FWP_BYTE_BLOB", pack=False, align=None), "certHash": SimStruct({"size": SimTypeInt(signed=False, label="UInt32"), "data": SimTypePointer(SimTypeChar(label="Byte"), offset=0)}, name="FWP_BYTE_BLOB", pack=False, align=None), "eku": SimTypePointer(SimStruct({"numEku": SimTypeInt(signed=False, label="UInt32"), "eku": SimTypePointer(SimTypePointer(SimTypeChar(label="Byte"), offset=0), offset=0)}, name="IKEEXT_CERT_EKUS0", pack=False, align=None), offset=0), "name": SimTypePointer(SimStruct({"nameType": SimTypeInt(signed=False, label="IKEEXT_CERT_CRITERIA_NAME_TYPE"), "certName": SimTypePointer(SimTypeChar(label="Char"), offset=0)}, name="IKEEXT_CERT_NAME0", pack=False, align=None), offset=0), "flags": SimTypeInt(signed=False, label="UInt32")}, name="IKEEXT_CERTIFICATE_CRITERIA0", pack=False, align=None), offset=0)}, name="_Anonymous1_e__Struct", pack=False, align=None), "Anonymous2": SimStruct({"inboundEnterpriseStoreArraySize": SimTypeInt(signed=False, label="UInt32"), "inboundEnterpriseStoreCriteria": SimTypePointer(SimStruct({"certData": SimStruct({"size": SimTypeInt(signed=False, label="UInt32"), "data": SimTypePointer(SimTypeChar(label="Byte"), offset=0)}, name="FWP_BYTE_BLOB", pack=False, align=None), "certHash": SimStruct({"size": SimTypeInt(signed=False, label="UInt32"), "data": SimTypePointer(SimTypeChar(label="Byte"), offset=0)}, name="FWP_BYTE_BLOB", pack=False, align=None), "eku": SimTypePointer(SimStruct({"numEku": SimTypeInt(signed=False, label="UInt32"), "eku": SimTypePointer(SimTypePointer(SimTypeChar(label="Byte"), offset=0), offset=0)}, name="IKEEXT_CERT_EKUS0", pack=False, align=None), offset=0), "name": SimTypePointer(SimStruct({"nameType": SimTypeInt(signed=False, label="IKEEXT_CERT_CRITERIA_NAME_TYPE"), "certName": SimTypePointer(SimTypeChar(label="Char"), offset=0)}, name="IKEEXT_CERT_NAME0", pack=False, align=None), offset=0), "flags": SimTypeInt(signed=False, label="UInt32")}, name="IKEEXT_CERTIFICATE_CRITERIA0", pack=False, align=None), offset=0)}, name="_Anonymous2_e__Struct", pack=False, align=None), "Anonymous3": SimStruct({"inboundRootStoreArraySize": SimTypeInt(signed=False, label="UInt32"), "inboundTrustedRootStoreCriteria": SimTypePointer(SimStruct({"certData": SimStruct({"size": SimTypeInt(signed=False, label="UInt32"), "data": SimTypePointer(SimTypeChar(label="Byte"), offset=0)}, name="FWP_BYTE_BLOB", pack=False, align=None), "certHash": SimStruct({"size": SimTypeInt(signed=False, label="UInt32"), "data": SimTypePointer(SimTypeChar(label="Byte"), offset=0)}, name="FWP_BYTE_BLOB", pack=False, align=None), "eku": SimTypePointer(SimStruct({"numEku": SimTypeInt(signed=False, label="UInt32"), "eku": SimTypePointer(SimTypePointer(SimTypeChar(label="Byte"), offset=0), offset=0)}, name="IKEEXT_CERT_EKUS0", pack=False, align=None), offset=0), "name": SimTypePointer(SimStruct({"nameType": SimTypeInt(signed=False, label="IKEEXT_CERT_CRITERIA_NAME_TYPE"), "certName": SimTypePointer(SimTypeChar(label="Char"), offset=0)}, name="IKEEXT_CERT_NAME0", pack=False, align=None), offset=0), "flags": SimTypeInt(signed=False, label="UInt32")}, name="IKEEXT_CERTIFICATE_CRITERIA0", pack=False, align=None), offset=0)}, name="_Anonymous3_e__Struct", pack=False, align=None)}, name="<anon>", label="None"), "outboundConfigType": SimTypeInt(signed=False, label="IKEEXT_CERT_CONFIG_TYPE"), "Anonymous2": SimUnion({"Anonymous1": SimStruct({"outboundRootArraySize": SimTypeInt(signed=False, label="UInt32"), "outboundRootCriteria": SimTypePointer(SimStruct({"certData": SimStruct({"size": SimTypeInt(signed=False, label="UInt32"), "data": SimTypePointer(SimTypeChar(label="Byte"), offset=0)}, name="FWP_BYTE_BLOB", pack=False, align=None), "certHash": SimStruct({"size": SimTypeInt(signed=False, label="UInt32"), "data": SimTypePointer(SimTypeChar(label="Byte"), offset=0)}, name="FWP_BYTE_BLOB", pack=False, align=None), "eku": SimTypePointer(SimStruct({"numEku": SimTypeInt(signed=False, label="UInt32"), "eku": SimTypePointer(SimTypePointer(SimTypeChar(label="Byte"), offset=0), offset=0)}, name="IKEEXT_CERT_EKUS0", pack=False, align=None), offset=0), "name": SimTypePointer(SimStruct({"nameType": SimTypeInt(signed=False, label="IKEEXT_CERT_CRITERIA_NAME_TYPE"), "certName": SimTypePointer(SimTypeChar(label="Char"), offset=0)}, name="IKEEXT_CERT_NAME0", pack=False, align=None), offset=0), "flags": SimTypeInt(signed=False, label="UInt32")}, name="IKEEXT_CERTIFICATE_CRITERIA0", pack=False, align=None), offset=0)}, name="_Anonymous1_e__Struct", pack=False, align=None), "Anonymous2": SimStruct({"outboundEnterpriseStoreArraySize": SimTypeInt(signed=False, label="UInt32"), "outboundEnterpriseStoreCriteria": SimTypePointer(SimStruct({"certData": SimStruct({"size": SimTypeInt(signed=False, label="UInt32"), "data": SimTypePointer(SimTypeChar(label="Byte"), offset=0)}, name="FWP_BYTE_BLOB", pack=False, align=None), "certHash": SimStruct({"size": SimTypeInt(signed=False, label="UInt32"), "data": SimTypePointer(SimTypeChar(label="Byte"), offset=0)}, name="FWP_BYTE_BLOB", pack=False, align=None), "eku": SimTypePointer(SimStruct({"numEku": SimTypeInt(signed=False, label="UInt32"), "eku": SimTypePointer(SimTypePointer(SimTypeChar(label="Byte"), offset=0), offset=0)}, name="IKEEXT_CERT_EKUS0", pack=False, align=None), offset=0), "name": SimTypePointer(SimStruct({"nameType": SimTypeInt(signed=False, label="IKEEXT_CERT_CRITERIA_NAME_TYPE"), "certName": SimTypePointer(SimTypeChar(label="Char"), offset=0)}, name="IKEEXT_CERT_NAME0", pack=False, align=None), offset=0), "flags": SimTypeInt(signed=False, label="UInt32")}, name="IKEEXT_CERTIFICATE_CRITERIA0", pack=False, align=None), offset=0)}, name="_Anonymous2_e__Struct", pack=False, align=None), "Anonymous3": SimStruct({"outboundRootStoreArraySize": SimTypeInt(signed=False, label="UInt32"), "outboundTrustedRootStoreCriteria": SimTypePointer(SimStruct({"certData": SimStruct({"size": SimTypeInt(signed=False, label="UInt32"), "data": SimTypePointer(SimTypeChar(label="Byte"), offset=0)}, name="FWP_BYTE_BLOB", pack=False, align=None), "certHash": SimStruct({"size": SimTypeInt(signed=False, label="UInt32"), "data": SimTypePointer(SimTypeChar(label="Byte"), offset=0)}, name="FWP_BYTE_BLOB", pack=False, align=None), "eku": SimTypePointer(SimStruct({"numEku": SimTypeInt(signed=False, label="UInt32"), "eku": SimTypePointer(SimTypePointer(SimTypeChar(label="Byte"), offset=0), offset=0)}, name="IKEEXT_CERT_EKUS0", pack=False, align=None), offset=0), "name": SimTypePointer(SimStruct({"nameType": SimTypeInt(signed=False, label="IKEEXT_CERT_CRITERIA_NAME_TYPE"), "certName": SimTypePointer(SimTypeChar(label="Char"), offset=0)}, name="IKEEXT_CERT_NAME0", pack=False, align=None), offset=0), "flags": SimTypeInt(signed=False, label="UInt32")}, name="IKEEXT_CERTIFICATE_CRITERIA0", pack=False, align=None), offset=0)}, name="_Anonymous3_e__Struct", pack=False, align=None)}, name="<anon>", label="None"), "flags": SimTypeInt(signed=False, label="IKEEXT_CERT_AUTH"), "localCertLocationUrl": SimStruct({"size": SimTypeInt(signed=False, label="UInt32"), "data": SimTypePointer(SimTypeChar(label="Byte"), offset=0)}, name="FWP_BYTE_BLOB", pack=False, align=None)}, name="IKEEXT_CERTIFICATE_AUTHENTICATION2", pack=False, align=None), "kerberosAuthentication": SimStruct({"flags": SimTypeInt(signed=False, label="IKEEXT_KERBEROS_AUTHENTICATION_FLAGS"), "proxyServer": SimTypePointer(SimTypeChar(label="Char"), offset=0)}, name="IKEEXT_KERBEROS_AUTHENTICATION1", pack=False, align=None), "reservedAuthentication": SimStruct({"flags": SimTypeInt(signed=False, label="IKEEXT_RESERVED_AUTHENTICATION_FLAGS")}, name="IKEEXT_RESERVED_AUTHENTICATION0", pack=False, align=None), "ntlmV2Authentication": SimStruct({"flags": SimTypeInt(signed=False, label="UInt32")}, name="IKEEXT_NTLM_V2_AUTHENTICATION0", pack=False, align=None), "sslAuthentication": SimStruct({"inboundConfigType": SimTypeInt(signed=False, label="IKEEXT_CERT_CONFIG_TYPE"), "Anonymous1": SimUnion({"Anonymous1": SimStruct({"inboundRootArraySize": SimTypeInt(signed=False, label="UInt32"), "inboundRootCriteria": SimTypePointer(SimStruct({"certData": SimStruct({"size": SimTypeInt(signed=False, label="UInt32"), "data": SimTypePointer(SimTypeChar(label="Byte"), offset=0)}, name="FWP_BYTE_BLOB", pack=False, align=None), "certHash": SimStruct({"size": SimTypeInt(signed=False, label="UInt32"), "data": SimTypePointer(SimTypeChar(label="Byte"), offset=0)}, name="FWP_BYTE_BLOB", pack=False, align=None), "eku": SimTypePointer(SimStruct({"numEku": SimTypeInt(signed=False, label="UInt32"), "eku": SimTypePointer(SimTypePointer(SimTypeChar(label="Byte"), offset=0), offset=0)}, name="IKEEXT_CERT_EKUS0", pack=False, align=None), offset=0), "name": SimTypePointer(SimStruct({"nameType": SimTypeInt(signed=False, label="IKEEXT_CERT_CRITERIA_NAME_TYPE"), "certName": SimTypePointer(SimTypeChar(label="Char"), offset=0)}, name="IKEEXT_CERT_NAME0", pack=False, align=None), offset=0), "flags": SimTypeInt(signed=False, label="UInt32")}, name="IKEEXT_CERTIFICATE_CRITERIA0", pack=False, align=None), offset=0)}, name="_Anonymous1_e__Struct", pack=False, align=None), "Anonymous2": SimStruct({"inboundEnterpriseStoreArraySize": SimTypeInt(signed=False, label="UInt32"), "inboundEnterpriseStoreCriteria": SimTypePointer(SimStruct({"certData": SimStruct({"size": SimTypeInt(signed=False, label="UInt32"), "data": SimTypePointer(SimTypeChar(label="Byte"), offset=0)}, name="FWP_BYTE_BLOB", pack=False, align=None), "certHash": SimStruct({"size": SimTypeInt(signed=False, label="UInt32"), "data": SimTypePointer(SimTypeChar(label="Byte"), offset=0)}, name="FWP_BYTE_BLOB", pack=False, align=None), "eku": SimTypePointer(SimStruct({"numEku": SimTypeInt(signed=False, label="UInt32"), "eku": SimTypePointer(SimTypePointer(SimTypeChar(label="Byte"), offset=0), offset=0)}, name="IKEEXT_CERT_EKUS0", pack=False, align=None), offset=0), "name": SimTypePointer(SimStruct({"nameType": SimTypeInt(signed=False, label="IKEEXT_CERT_CRITERIA_NAME_TYPE"), "certName": SimTypePointer(SimTypeChar(label="Char"), offset=0)}, name="IKEEXT_CERT_NAME0", pack=False, align=None), offset=0), "flags": SimTypeInt(signed=False, label="UInt32")}, name="IKEEXT_CERTIFICATE_CRITERIA0", pack=False, align=None), offset=0)}, name="_Anonymous2_e__Struct", pack=False, align=None), "Anonymous3": SimStruct({"inboundRootStoreArraySize": SimTypeInt(signed=False, label="UInt32"), "inboundTrustedRootStoreCriteria": SimTypePointer(SimStruct({"certData": SimStruct({"size": SimTypeInt(signed=False, label="UInt32"), "data": SimTypePointer(SimTypeChar(label="Byte"), offset=0)}, name="FWP_BYTE_BLOB", pack=False, align=None), "certHash": SimStruct({"size": SimTypeInt(signed=False, label="UInt32"), "data": SimTypePointer(SimTypeChar(label="Byte"), offset=0)}, name="FWP_BYTE_BLOB", pack=False, align=None), "eku": SimTypePointer(SimStruct({"numEku": SimTypeInt(signed=False, label="UInt32"), "eku": SimTypePointer(SimTypePointer(SimTypeChar(label="Byte"), offset=0), offset=0)}, name="IKEEXT_CERT_EKUS0", pack=False, align=None), offset=0), "name": SimTypePointer(SimStruct({"nameType": SimTypeInt(signed=False, label="IKEEXT_CERT_CRITERIA_NAME_TYPE"), "certName": SimTypePointer(SimTypeChar(label="Char"), offset=0)}, name="IKEEXT_CERT_NAME0", pack=False, align=None), offset=0), "flags": SimTypeInt(signed=False, label="UInt32")}, name="IKEEXT_CERTIFICATE_CRITERIA0", pack=False, align=None), offset=0)}, name="_Anonymous3_e__Struct", pack=False, align=None)}, name="<anon>", label="None"), "outboundConfigType": SimTypeInt(signed=False, label="IKEEXT_CERT_CONFIG_TYPE"), "Anonymous2": SimUnion({"Anonymous1": SimStruct({"outboundRootArraySize": SimTypeInt(signed=False, label="UInt32"), "outboundRootCriteria": SimTypePointer(SimStruct({"certData": SimStruct({"size": SimTypeInt(signed=False, label="UInt32"), "data": SimTypePointer(SimTypeChar(label="Byte"), offset=0)}, name="FWP_BYTE_BLOB", pack=False, align=None), "certHash": SimStruct({"size": SimTypeInt(signed=False, label="UInt32"), "data": SimTypePointer(SimTypeChar(label="Byte"), offset=0)}, name="FWP_BYTE_BLOB", pack=False, align=None), "eku": SimTypePointer(SimStruct({"numEku": SimTypeInt(signed=False, label="UInt32"), "eku": SimTypePointer(SimTypePointer(SimTypeChar(label="Byte"), offset=0), offset=0)}, name="IKEEXT_CERT_EKUS0", pack=False, align=None), offset=0), "name": SimTypePointer(SimStruct({"nameType": SimTypeInt(signed=False, label="IKEEXT_CERT_CRITERIA_NAME_TYPE"), "certName": SimTypePointer(SimTypeChar(label="Char"), offset=0)}, name="IKEEXT_CERT_NAME0", pack=False, align=None), offset=0), "flags": SimTypeInt(signed=False, label="UInt32")}, name="IKEEXT_CERTIFICATE_CRITERIA0", pack=False, align=None), offset=0)}, name="_Anonymous1_e__Struct", pack=False, align=None), "Anonymous2": SimStruct({"outboundEnterpriseStoreArraySize": SimTypeInt(signed=False, label="UInt32"), "outboundEnterpriseStoreCriteria": SimTypePointer(SimStruct({"certData": SimStruct({"size": SimTypeInt(signed=False, label="UInt32"), "data": SimTypePointer(SimTypeChar(label="Byte"), offset=0)}, name="FWP_BYTE_BLOB", pack=False, align=None), "certHash": SimStruct({"size": SimTypeInt(signed=False, label="UInt32"), "data": SimTypePointer(SimTypeChar(label="Byte"), offset=0)}, name="FWP_BYTE_BLOB", pack=False, align=None), "eku": SimTypePointer(SimStruct({"numEku": SimTypeInt(signed=False, label="UInt32"), "eku": SimTypePointer(SimTypePointer(SimTypeChar(label="Byte"), offset=0), offset=0)}, name="IKEEXT_CERT_EKUS0", pack=False, align=None), offset=0), "name": SimTypePointer(SimStruct({"nameType": SimTypeInt(signed=False, label="IKEEXT_CERT_CRITERIA_NAME_TYPE"), "certName": SimTypePointer(SimTypeChar(label="Char"), offset=0)}, name="IKEEXT_CERT_NAME0", pack=False, align=None), offset=0), "flags": SimTypeInt(signed=False, label="UInt32")}, name="IKEEXT_CERTIFICATE_CRITERIA0", pack=False, align=None), offset=0)}, name="_Anonymous2_e__Struct", pack=False, align=None), "Anonymous3": SimStruct({"outboundRootStoreArraySize": SimTypeInt(signed=False, label="UInt32"), "outboundTrustedRootStoreCriteria": SimTypePointer(SimStruct({"certData": SimStruct({"size": SimTypeInt(signed=False, label="UInt32"), "data": SimTypePointer(SimTypeChar(label="Byte"), offset=0)}, name="FWP_BYTE_BLOB", pack=False, align=None), "certHash": SimStruct({"size": SimTypeInt(signed=False, label="UInt32"), "data": SimTypePointer(SimTypeChar(label="Byte"), offset=0)}, name="FWP_BYTE_BLOB", pack=False, align=None), "eku": SimTypePointer(SimStruct({"numEku": SimTypeInt(signed=False, label="UInt32"), "eku": SimTypePointer(SimTypePointer(SimTypeChar(label="Byte"), offset=0), offset=0)}, name="IKEEXT_CERT_EKUS0", pack=False, align=None), offset=0), "name": SimTypePointer(SimStruct({"nameType": SimTypeInt(signed=False, label="IKEEXT_CERT_CRITERIA_NAME_TYPE"), "certName": SimTypePointer(SimTypeChar(label="Char"), offset=0)}, name="IKEEXT_CERT_NAME0", pack=False, align=None), offset=0), "flags": SimTypeInt(signed=False, label="UInt32")}, name="IKEEXT_CERTIFICATE_CRITERIA0", pack=False, align=None), offset=0)}, name="_Anonymous3_e__Struct", pack=False, align=None)}, name="<anon>", label="None"), "flags": SimTypeInt(signed=False, label="IKEEXT_CERT_AUTH"), "localCertLocationUrl": SimStruct({"size": SimTypeInt(signed=False, label="UInt32"), "data": SimTypePointer(SimTypeChar(label="Byte"), offset=0)}, name="FWP_BYTE_BLOB", pack=False, align=None)}, name="IKEEXT_CERTIFICATE_AUTHENTICATION2", pack=False, align=None), "cgaAuthentication": SimStruct({"keyContainerName": SimTypePointer(SimTypeChar(label="Char"), offset=0), "cspName": SimTypePointer(SimTypeChar(label="Char"), offset=0), "cspType": SimTypeInt(signed=False, label="UInt32"), "cgaModifier": SimStruct({"byteArray16": SimTypeFixedSizeArray(SimTypeChar(label="Byte"), 16)}, name="FWP_BYTE_ARRAY16", pack=False, align=None), "cgaCollisionCount": SimTypeChar(label="Byte")}, name="IKEEXT_IPV6_CGA_AUTHENTICATION0", pack=False, align=None), "eapAuthentication": SimStruct({"flags": SimTypeInt(signed=False, label="IKEEXT_EAP_AUTHENTICATION_FLAGS")}, name="IKEEXT_EAP_AUTHENTICATION0", pack=False, align=None)}, name="<anon>", label="None")}, name="IKEEXT_AUTHENTICATION_METHOD2", pack=False, align=None), offset=0), "initiatorImpersonationType": SimTypeInt(signed=False, label="IKEEXT_AUTHENTICATION_IMPERSONATION_TYPE")}, name="IKEEXT_EM_POLICY2", pack=False, align=None), offset=0), "fwdPathSaLifetime": SimTypeInt(signed=False, label="UInt32")}, name="IPSEC_TUNNEL_POLICY2", pack=False, align=None), offset=0), "authipQmTransportPolicy": SimTypePointer(SimStruct({"numIpsecProposals": SimTypeInt(signed=False, label="UInt32"), "ipsecProposals": SimTypePointer(SimStruct({"lifetime": SimStruct({"lifetimeSeconds": SimTypeInt(signed=False, label="UInt32"), "lifetimeKilobytes": SimTypeInt(signed=False, label="UInt32"), "lifetimePackets": SimTypeInt(signed=False, label="UInt32")}, name="IPSEC_SA_LIFETIME0", pack=False, align=None), "numSaTransforms": SimTypeInt(signed=False, label="UInt32"), "saTransforms": SimTypePointer(SimStruct({"ipsecTransformType": SimTypeInt(signed=False, label="IPSEC_TRANSFORM_TYPE"), "Anonymous": SimUnion({"ahTransform": SimTypePointer(SimStruct({"authTransformId": SimStruct({"authType": SimTypeInt(signed=False, label="IPSEC_AUTH_TYPE"), "authConfig": SimTypeChar(label="Byte")}, name="IPSEC_AUTH_TRANSFORM_ID0", pack=False, align=None), "cryptoModuleId": SimTypePointer(SimTypeBottom(label="Guid"), offset=0)}, name="IPSEC_AUTH_TRANSFORM0", pack=False, align=None), offset=0), "espAuthTransform": SimTypePointer(SimStruct({"authTransformId": SimStruct({"authType": SimTypeInt(signed=False, label="IPSEC_AUTH_TYPE"), "authConfig": SimTypeChar(label="Byte")}, name="IPSEC_AUTH_TRANSFORM_ID0", pack=False, align=None), "cryptoModuleId": SimTypePointer(SimTypeBottom(label="Guid"), offset=0)}, name="IPSEC_AUTH_TRANSFORM0", pack=False, align=None), offset=0), "espCipherTransform": SimTypePointer(SimStruct({"cipherTransformId": SimStruct({"cipherType": SimTypeInt(signed=False, label="IPSEC_CIPHER_TYPE"), "cipherConfig": SimTypeChar(label="Byte")}, name="IPSEC_CIPHER_TRANSFORM_ID0", pack=False, align=None), "cryptoModuleId": SimTypePointer(SimTypeBottom(label="Guid"), offset=0)}, name="IPSEC_CIPHER_TRANSFORM0", pack=False, align=None), offset=0), "espAuthAndCipherTransform": SimTypePointer(SimStruct({"authTransform": SimStruct({"authTransformId": SimStruct({"authType": SimTypeInt(signed=False, label="IPSEC_AUTH_TYPE"), "authConfig": SimTypeChar(label="Byte")}, name="IPSEC_AUTH_TRANSFORM_ID0", pack=False, align=None), "cryptoModuleId": SimTypePointer(SimTypeBottom(label="Guid"), offset=0)}, name="IPSEC_AUTH_TRANSFORM0", pack=False, align=None), "cipherTransform": SimStruct({"cipherTransformId": SimStruct({"cipherType": SimTypeInt(signed=False, label="IPSEC_CIPHER_TYPE"), "cipherConfig": SimTypeChar(label="Byte")}, name="IPSEC_CIPHER_TRANSFORM_ID0", pack=False, align=None), "cryptoModuleId": SimTypePointer(SimTypeBottom(label="Guid"), offset=0)}, name="IPSEC_CIPHER_TRANSFORM0", pack=False, align=None)}, name="IPSEC_AUTH_AND_CIPHER_TRANSFORM0", pack=False, align=None), offset=0), "espAuthFwTransform": SimTypePointer(SimStruct({"authTransformId": SimStruct({"authType": SimTypeInt(signed=False, label="IPSEC_AUTH_TYPE"), "authConfig": SimTypeChar(label="Byte")}, name="IPSEC_AUTH_TRANSFORM_ID0", pack=False, align=None), "cryptoModuleId": SimTypePointer(SimTypeBottom(label="Guid"), offset=0)}, name="IPSEC_AUTH_TRANSFORM0", pack=False, align=None), offset=0)}, name="<anon>", label="None")}, name="IPSEC_SA_TRANSFORM0", pack=False, align=None), offset=0), "pfsGroup": SimTypeInt(signed=False, label="IPSEC_PFS_GROUP")}, name="IPSEC_PROPOSAL0", pack=False, align=None), offset=0), "flags": SimTypeInt(signed=False, label="IPSEC_POLICY_FLAG"), "ndAllowClearTimeoutSeconds": SimTypeInt(signed=False, label="UInt32"), "saIdleTimeout": SimStruct({"idleTimeoutSeconds": SimTypeInt(signed=False, label="UInt32"), "idleTimeoutSecondsFailOver": SimTypeInt(signed=False, label="UInt32")}, name="IPSEC_SA_IDLE_TIMEOUT0", pack=False, align=None), "emPolicy": SimTypePointer(SimStruct({"numAuthenticationMethods": SimTypeInt(signed=False, label="UInt32"), "authenticationMethods": SimTypePointer(SimStruct({"authenticationMethodType": SimTypeInt(signed=False, label="IKEEXT_AUTHENTICATION_METHOD_TYPE"), "Anonymous": SimUnion({"presharedKeyAuthentication": SimStruct({"presharedKey": SimStruct({"size": SimTypeInt(signed=False, label="UInt32"), "data": SimTypePointer(SimTypeChar(label="Byte"), offset=0)}, name="FWP_BYTE_BLOB", pack=False, align=None), "flags": SimTypeInt(signed=False, label="IKEEXT_PRESHARED_KEY_AUTHENTICATION_FLAGS")}, name="IKEEXT_PRESHARED_KEY_AUTHENTICATION1", pack=False, align=None), "certificateAuthentication": SimStruct({"inboundConfigType": SimTypeInt(signed=False, label="IKEEXT_CERT_CONFIG_TYPE"), "Anonymous1": SimUnion({"Anonymous1": SimStruct({"inboundRootArraySize": SimTypeInt(signed=False, label="UInt32"), "inboundRootCriteria": SimTypePointer(SimStruct({"certData": SimStruct({"size": SimTypeInt(signed=False, label="UInt32"), "data": SimTypePointer(SimTypeChar(label="Byte"), offset=0)}, name="FWP_BYTE_BLOB", pack=False, align=None), "certHash": SimStruct({"size": SimTypeInt(signed=False, label="UInt32"), "data": SimTypePointer(SimTypeChar(label="Byte"), offset=0)}, name="FWP_BYTE_BLOB", pack=False, align=None), "eku": SimTypePointer(SimStruct({"numEku": SimTypeInt(signed=False, label="UInt32"), "eku": SimTypePointer(SimTypePointer(SimTypeChar(label="Byte"), offset=0), offset=0)}, name="IKEEXT_CERT_EKUS0", pack=False, align=None), offset=0), "name": SimTypePointer(SimStruct({"nameType": SimTypeInt(signed=False, label="IKEEXT_CERT_CRITERIA_NAME_TYPE"), "certName": SimTypePointer(SimTypeChar(label="Char"), offset=0)}, name="IKEEXT_CERT_NAME0", pack=False, align=None), offset=0), "flags": SimTypeInt(signed=False, label="UInt32")}, name="IKEEXT_CERTIFICATE_CRITERIA0", pack=False, align=None), offset=0)}, name="_Anonymous1_e__Struct", pack=False, align=None), "Anonymous2": SimStruct({"inboundEnterpriseStoreArraySize": SimTypeInt(signed=False, label="UInt32"), "inboundEnterpriseStoreCriteria": SimTypePointer(SimStruct({"certData": SimStruct({"size": SimTypeInt(signed=False, label="UInt32"), "data": SimTypePointer(SimTypeChar(label="Byte"), offset=0)}, name="FWP_BYTE_BLOB", pack=False, align=None), "certHash": SimStruct({"size": SimTypeInt(signed=False, label="UInt32"), "data": SimTypePointer(SimTypeChar(label="Byte"), offset=0)}, name="FWP_BYTE_BLOB", pack=False, align=None), "eku": SimTypePointer(SimStruct({"numEku": SimTypeInt(signed=False, label="UInt32"), "eku": SimTypePointer(SimTypePointer(SimTypeChar(label="Byte"), offset=0), offset=0)}, name="IKEEXT_CERT_EKUS0", pack=False, align=None), offset=0), "name": SimTypePointer(SimStruct({"nameType": SimTypeInt(signed=False, label="IKEEXT_CERT_CRITERIA_NAME_TYPE"), "certName": SimTypePointer(SimTypeChar(label="Char"), offset=0)}, name="IKEEXT_CERT_NAME0", pack=False, align=None), offset=0), "flags": SimTypeInt(signed=False, label="UInt32")}, name="IKEEXT_CERTIFICATE_CRITERIA0", pack=False, align=None), offset=0)}, name="_Anonymous2_e__Struct", pack=False, align=None), "Anonymous3": SimStruct({"inboundRootStoreArraySize": SimTypeInt(signed=False, label="UInt32"), "inboundTrustedRootStoreCriteria": SimTypePointer(SimStruct({"certData": SimStruct({"size": SimTypeInt(signed=False, label="UInt32"), "data": SimTypePointer(SimTypeChar(label="Byte"), offset=0)}, name="FWP_BYTE_BLOB", pack=False, align=None), "certHash": SimStruct({"size": SimTypeInt(signed=False, label="UInt32"), "data": SimTypePointer(SimTypeChar(label="Byte"), offset=0)}, name="FWP_BYTE_BLOB", pack=False, align=None), "eku": SimTypePointer(SimStruct({"numEku": SimTypeInt(signed=False, label="UInt32"), "eku": SimTypePointer(SimTypePointer(SimTypeChar(label="Byte"), offset=0), offset=0)}, name="IKEEXT_CERT_EKUS0", pack=False, align=None), offset=0), "name": SimTypePointer(SimStruct({"nameType": SimTypeInt(signed=False, label="IKEEXT_CERT_CRITERIA_NAME_TYPE"), "certName": SimTypePointer(SimTypeChar(label="Char"), offset=0)}, name="IKEEXT_CERT_NAME0", pack=False, align=None), offset=0), "flags": SimTypeInt(signed=False, label="UInt32")}, name="IKEEXT_CERTIFICATE_CRITERIA0", pack=False, align=None), offset=0)}, name="_Anonymous3_e__Struct", pack=False, align=None)}, name="<anon>", label="None"), "outboundConfigType": SimTypeInt(signed=False, label="IKEEXT_CERT_CONFIG_TYPE"), "Anonymous2": SimUnion({"Anonymous1": SimStruct({"outboundRootArraySize": SimTypeInt(signed=False, label="UInt32"), "outboundRootCriteria": SimTypePointer(SimStruct({"certData": SimStruct({"size": SimTypeInt(signed=False, label="UInt32"), "data": SimTypePointer(SimTypeChar(label="Byte"), offset=0)}, name="FWP_BYTE_BLOB", pack=False, align=None), "certHash": SimStruct({"size": SimTypeInt(signed=False, label="UInt32"), "data": SimTypePointer(SimTypeChar(label="Byte"), offset=0)}, name="FWP_BYTE_BLOB", pack=False, align=None), "eku": SimTypePointer(SimStruct({"numEku": SimTypeInt(signed=False, label="UInt32"), "eku": SimTypePointer(SimTypePointer(SimTypeChar(label="Byte"), offset=0), offset=0)}, name="IKEEXT_CERT_EKUS0", pack=False, align=None), offset=0), "name": SimTypePointer(SimStruct({"nameType": SimTypeInt(signed=False, label="IKEEXT_CERT_CRITERIA_NAME_TYPE"), "certName": SimTypePointer(SimTypeChar(label="Char"), offset=0)}, name="IKEEXT_CERT_NAME0", pack=False, align=None), offset=0), "flags": SimTypeInt(signed=False, label="UInt32")}, name="IKEEXT_CERTIFICATE_CRITERIA0", pack=False, align=None), offset=0)}, name="_Anonymous1_e__Struct", pack=False, align=None), "Anonymous2": SimStruct({"outboundEnterpriseStoreArraySize": SimTypeInt(signed=False, label="UInt32"), "outboundEnterpriseStoreCriteria": SimTypePointer(SimStruct({"certData": SimStruct({"size": SimTypeInt(signed=False, label="UInt32"), "data": SimTypePointer(SimTypeChar(label="Byte"), offset=0)}, name="FWP_BYTE_BLOB", pack=False, align=None), "certHash": SimStruct({"size": SimTypeInt(signed=False, label="UInt32"), "data": SimTypePointer(SimTypeChar(label="Byte"), offset=0)}, name="FWP_BYTE_BLOB", pack=False, align=None), "eku": SimTypePointer(SimStruct({"numEku": SimTypeInt(signed=False, label="UInt32"), "eku": SimTypePointer(SimTypePointer(SimTypeChar(label="Byte"), offset=0), offset=0)}, name="IKEEXT_CERT_EKUS0", pack=False, align=None), offset=0), "name": SimTypePointer(SimStruct({"nameType": SimTypeInt(signed=False, label="IKEEXT_CERT_CRITERIA_NAME_TYPE"), "certName": SimTypePointer(SimTypeChar(label="Char"), offset=0)}, name="IKEEXT_CERT_NAME0", pack=False, align=None), offset=0), "flags": SimTypeInt(signed=False, label="UInt32")}, name="IKEEXT_CERTIFICATE_CRITERIA0", pack=False, align=None), offset=0)}, name="_Anonymous2_e__Struct", pack=False, align=None), "Anonymous3": SimStruct({"outboundRootStoreArraySize": SimTypeInt(signed=False, label="UInt32"), "outboundTrustedRootStoreCriteria": SimTypePointer(SimStruct({"certData": SimStruct({"size": SimTypeInt(signed=False, label="UInt32"), "data": SimTypePointer(SimTypeChar(label="Byte"), offset=0)}, name="FWP_BYTE_BLOB", pack=False, align=None), "certHash": SimStruct({"size": SimTypeInt(signed=False, label="UInt32"), "data": SimTypePointer(SimTypeChar(label="Byte"), offset=0)}, name="FWP_BYTE_BLOB", pack=False, align=None), "eku": SimTypePointer(SimStruct({"numEku": SimTypeInt(signed=False, label="UInt32"), "eku": SimTypePointer(SimTypePointer(SimTypeChar(label="Byte"), offset=0), offset=0)}, name="IKEEXT_CERT_EKUS0", pack=False, align=None), offset=0), "name": SimTypePointer(SimStruct({"nameType": SimTypeInt(signed=False, label="IKEEXT_CERT_CRITERIA_NAME_TYPE"), "certName": SimTypePointer(SimTypeChar(label="Char"), offset=0)}, name="IKEEXT_CERT_NAME0", pack=False, align=None), offset=0), "flags": SimTypeInt(signed=False, label="UInt32")}, name="IKEEXT_CERTIFICATE_CRITERIA0", pack=False, align=None), offset=0)}, name="_Anonymous3_e__Struct", pack=False, align=None)}, name="<anon>", label="None"), "flags": SimTypeInt(signed=False, label="IKEEXT_CERT_AUTH"), "localCertLocationUrl": SimStruct({"size": SimTypeInt(signed=False, label="UInt32"), "data": SimTypePointer(SimTypeChar(label="Byte"), offset=0)}, name="FWP_BYTE_BLOB", pack=False, align=None)}, name="IKEEXT_CERTIFICATE_AUTHENTICATION2", pack=False, align=None), "kerberosAuthentication": SimStruct({"flags": SimTypeInt(signed=False, label="IKEEXT_KERBEROS_AUTHENTICATION_FLAGS"), "proxyServer": SimTypePointer(SimTypeChar(label="Char"), offset=0)}, name="IKEEXT_KERBEROS_AUTHENTICATION1", pack=False, align=None), "reservedAuthentication": SimStruct({"flags": SimTypeInt(signed=False, label="IKEEXT_RESERVED_AUTHENTICATION_FLAGS")}, name="IKEEXT_RESERVED_AUTHENTICATION0", pack=False, align=None), "ntlmV2Authentication": SimStruct({"flags": SimTypeInt(signed=False, label="UInt32")}, name="IKEEXT_NTLM_V2_AUTHENTICATION0", pack=False, align=None), "sslAuthentication": SimStruct({"inboundConfigType": SimTypeInt(signed=False, label="IKEEXT_CERT_CONFIG_TYPE"), "Anonymous1": SimUnion({"Anonymous1": SimStruct({"inboundRootArraySize": SimTypeInt(signed=False, label="UInt32"), "inboundRootCriteria": SimTypePointer(SimStruct({"certData": SimStruct({"size": SimTypeInt(signed=False, label="UInt32"), "data": SimTypePointer(SimTypeChar(label="Byte"), offset=0)}, name="FWP_BYTE_BLOB", pack=False, align=None), "certHash": SimStruct({"size": SimTypeInt(signed=False, label="UInt32"), "data": SimTypePointer(SimTypeChar(label="Byte"), offset=0)}, name="FWP_BYTE_BLOB", pack=False, align=None), "eku": SimTypePointer(SimStruct({"numEku": SimTypeInt(signed=False, label="UInt32"), "eku": SimTypePointer(SimTypePointer(SimTypeChar(label="Byte"), offset=0), offset=0)}, name="IKEEXT_CERT_EKUS0", pack=False, align=None), offset=0), "name": SimTypePointer(SimStruct({"nameType": SimTypeInt(signed=False, label="IKEEXT_CERT_CRITERIA_NAME_TYPE"), "certName": SimTypePointer(SimTypeChar(label="Char"), offset=0)}, name="IKEEXT_CERT_NAME0", pack=False, align=None), offset=0), "flags": SimTypeInt(signed=False, label="UInt32")}, name="IKEEXT_CERTIFICATE_CRITERIA0", pack=False, align=None), offset=0)}, name="_Anonymous1_e__Struct", pack=False, align=None), "Anonymous2": SimStruct({"inboundEnterpriseStoreArraySize": SimTypeInt(signed=False, label="UInt32"), "inboundEnterpriseStoreCriteria": SimTypePointer(SimStruct({"certData": SimStruct({"size": SimTypeInt(signed=False, label="UInt32"), "data": SimTypePointer(SimTypeChar(label="Byte"), offset=0)}, name="FWP_BYTE_BLOB", pack=False, align=None), "certHash": SimStruct({"size": SimTypeInt(signed=False, label="UInt32"), "data": SimTypePointer(SimTypeChar(label="Byte"), offset=0)}, name="FWP_BYTE_BLOB", pack=False, align=None), "eku": SimTypePointer(SimStruct({"numEku": SimTypeInt(signed=False, label="UInt32"), "eku": SimTypePointer(SimTypePointer(SimTypeChar(label="Byte"), offset=0), offset=0)}, name="IKEEXT_CERT_EKUS0", pack=False, align=None), offset=0), "name": SimTypePointer(SimStruct({"nameType": SimTypeInt(signed=False, label="IKEEXT_CERT_CRITERIA_NAME_TYPE"), "certName": SimTypePointer(SimTypeChar(label="Char"), offset=0)}, name="IKEEXT_CERT_NAME0", pack=False, align=None), offset=0), "flags": SimTypeInt(signed=False, label="UInt32")}, name="IKEEXT_CERTIFICATE_CRITERIA0", pack=False, align=None), offset=0)}, name="_Anonymous2_e__Struct", pack=False, align=None), "Anonymous3": SimStruct({"inboundRootStoreArraySize": SimTypeInt(signed=False, label="UInt32"), "inboundTrustedRootStoreCriteria": SimTypePointer(SimStruct({"certData": SimStruct({"size": SimTypeInt(signed=False, label="UInt32"), "data": SimTypePointer(SimTypeChar(label="Byte"), offset=0)}, name="FWP_BYTE_BLOB", pack=False, align=None), "certHash": SimStruct({"size": SimTypeInt(signed=False, label="UInt32"), "data": SimTypePointer(SimTypeChar(label="Byte"), offset=0)}, name="FWP_BYTE_BLOB", pack=False, align=None), "eku": SimTypePointer(SimStruct({"numEku": SimTypeInt(signed=False, label="UInt32"), "eku": SimTypePointer(SimTypePointer(SimTypeChar(label="Byte"), offset=0), offset=0)}, name="IKEEXT_CERT_EKUS0", pack=False, align=None), offset=0), "name": SimTypePointer(SimStruct({"nameType": SimTypeInt(signed=False, label="IKEEXT_CERT_CRITERIA_NAME_TYPE"), "certName": SimTypePointer(SimTypeChar(label="Char"), offset=0)}, name="IKEEXT_CERT_NAME0", pack=False, align=None), offset=0), "flags": SimTypeInt(signed=False, label="UInt32")}, name="IKEEXT_CERTIFICATE_CRITERIA0", pack=False, align=None), offset=0)}, name="_Anonymous3_e__Struct", pack=False, align=None)}, name="<anon>", label="None"), "outboundConfigType": SimTypeInt(signed=False, label="IKEEXT_CERT_CONFIG_TYPE"), "Anonymous2": SimUnion({"Anonymous1": SimStruct({"outboundRootArraySize": SimTypeInt(signed=False, label="UInt32"), "outboundRootCriteria": SimTypePointer(SimStruct({"certData": SimStruct({"size": SimTypeInt(signed=False, label="UInt32"), "data": SimTypePointer(SimTypeChar(label="Byte"), offset=0)}, name="FWP_BYTE_BLOB", pack=False, align=None), "certHash": SimStruct({"size": SimTypeInt(signed=False, label="UInt32"), "data": SimTypePointer(SimTypeChar(label="Byte"), offset=0)}, name="FWP_BYTE_BLOB", pack=False, align=None), "eku": SimTypePointer(SimStruct({"numEku": SimTypeInt(signed=False, label="UInt32"), "eku": SimTypePointer(SimTypePointer(SimTypeChar(label="Byte"), offset=0), offset=0)}, name="IKEEXT_CERT_EKUS0", pack=False, align=None), offset=0), "name": SimTypePointer(SimStruct({"nameType": SimTypeInt(signed=False, label="IKEEXT_CERT_CRITERIA_NAME_TYPE"), "certName": SimTypePointer(SimTypeChar(label="Char"), offset=0)}, name="IKEEXT_CERT_NAME0", pack=False, align=None), offset=0), "flags": SimTypeInt(signed=False, label="UInt32")}, name="IKEEXT_CERTIFICATE_CRITERIA0", pack=False, align=None), offset=0)}, name="_Anonymous1_e__Struct", pack=False, align=None), "Anonymous2": SimStruct({"outboundEnterpriseStoreArraySize": SimTypeInt(signed=False, label="UInt32"), "outboundEnterpriseStoreCriteria": SimTypePointer(SimStruct({"certData": SimStruct({"size": SimTypeInt(signed=False, label="UInt32"), "data": SimTypePointer(SimTypeChar(label="Byte"), offset=0)}, name="FWP_BYTE_BLOB", pack=False, align=None), "certHash": SimStruct({"size": SimTypeInt(signed=False, label="UInt32"), "data": SimTypePointer(SimTypeChar(label="Byte"), offset=0)}, name="FWP_BYTE_BLOB", pack=False, align=None), "eku": SimTypePointer(SimStruct({"numEku": SimTypeInt(signed=False, label="UInt32"), "eku": SimTypePointer(SimTypePointer(SimTypeChar(label="Byte"), offset=0), offset=0)}, name="IKEEXT_CERT_EKUS0", pack=False, align=None), offset=0), "name": SimTypePointer(SimStruct({"nameType": SimTypeInt(signed=False, label="IKEEXT_CERT_CRITERIA_NAME_TYPE"), "certName": SimTypePointer(SimTypeChar(label="Char"), offset=0)}, name="IKEEXT_CERT_NAME0", pack=False, align=None), offset=0), "flags": SimTypeInt(signed=False, label="UInt32")}, name="IKEEXT_CERTIFICATE_CRITERIA0", pack=False, align=None), offset=0)}, name="_Anonymous2_e__Struct", pack=False, align=None), "Anonymous3": SimStruct({"outboundRootStoreArraySize": SimTypeInt(signed=False, label="UInt32"), "outboundTrustedRootStoreCriteria": SimTypePointer(SimStruct({"certData": SimStruct({"size": SimTypeInt(signed=False, label="UInt32"), "data": SimTypePointer(SimTypeChar(label="Byte"), offset=0)}, name="FWP_BYTE_BLOB", pack=False, align=None), "certHash": SimStruct({"size": SimTypeInt(signed=False, label="UInt32"), "data": SimTypePointer(SimTypeChar(label="Byte"), offset=0)}, name="FWP_BYTE_BLOB", pack=False, align=None), "eku": SimTypePointer(SimStruct({"numEku": SimTypeInt(signed=False, label="UInt32"), "eku": SimTypePointer(SimTypePointer(SimTypeChar(label="Byte"), offset=0), offset=0)}, name="IKEEXT_CERT_EKUS0", pack=False, align=None), offset=0), "name": SimTypePointer(SimStruct({"nameType": SimTypeInt(signed=False, label="IKEEXT_CERT_CRITERIA_NAME_TYPE"), "certName": SimTypePointer(SimTypeChar(label="Char"), offset=0)}, name="IKEEXT_CERT_NAME0", pack=False, align=None), offset=0), "flags": SimTypeInt(signed=False, label="UInt32")}, name="IKEEXT_CERTIFICATE_CRITERIA0", pack=False, align=None), offset=0)}, name="_Anonymous3_e__Struct", pack=False, align=None)}, name="<anon>", label="None"), "flags": SimTypeInt(signed=False, label="IKEEXT_CERT_AUTH"), "localCertLocationUrl": SimStruct({"size": SimTypeInt(signed=False, label="UInt32"), "data": SimTypePointer(SimTypeChar(label="Byte"), offset=0)}, name="FWP_BYTE_BLOB", pack=False, align=None)}, name="IKEEXT_CERTIFICATE_AUTHENTICATION2", pack=False, align=None), "cgaAuthentication": SimStruct({"keyContainerName": SimTypePointer(SimTypeChar(label="Char"), offset=0), "cspName": SimTypePointer(SimTypeChar(label="Char"), offset=0), "cspType": SimTypeInt(signed=False, label="UInt32"), "cgaModifier": SimStruct({"byteArray16": SimTypeFixedSizeArray(SimTypeChar(label="Byte"), 16)}, name="FWP_BYTE_ARRAY16", pack=False, align=None), "cgaCollisionCount": SimTypeChar(label="Byte")}, name="IKEEXT_IPV6_CGA_AUTHENTICATION0", pack=False, align=None), "eapAuthentication": SimStruct({"flags": SimTypeInt(signed=False, label="IKEEXT_EAP_AUTHENTICATION_FLAGS")}, name="IKEEXT_EAP_AUTHENTICATION0", pack=False, align=None)}, name="<anon>", label="None")}, name="IKEEXT_AUTHENTICATION_METHOD2", pack=False, align=None), offset=0), "initiatorImpersonationType": SimTypeInt(signed=False, label="IKEEXT_AUTHENTICATION_IMPERSONATION_TYPE")}, name="IKEEXT_EM_POLICY2", pack=False, align=None), offset=0)}, name="IPSEC_TRANSPORT_POLICY2", pack=False, align=None), offset=0), "authipQmTunnelPolicy": SimTypePointer(SimStruct({"flags": SimTypeInt(signed=False, label="IPSEC_POLICY_FLAG"), "numIpsecProposals": SimTypeInt(signed=False, label="UInt32"), "ipsecProposals": SimTypePointer(SimStruct({"lifetime": SimStruct({"lifetimeSeconds": SimTypeInt(signed=False, label="UInt32"), "lifetimeKilobytes": SimTypeInt(signed=False, label="UInt32"), "lifetimePackets": SimTypeInt(signed=False, label="UInt32")}, name="IPSEC_SA_LIFETIME0", pack=False, align=None), "numSaTransforms": SimTypeInt(signed=False, label="UInt32"), "saTransforms": SimTypePointer(SimStruct({"ipsecTransformType": SimTypeInt(signed=False, label="IPSEC_TRANSFORM_TYPE"), "Anonymous": SimUnion({"ahTransform": SimTypePointer(SimStruct({"authTransformId": SimStruct({"authType": SimTypeInt(signed=False, label="IPSEC_AUTH_TYPE"), "authConfig": SimTypeChar(label="Byte")}, name="IPSEC_AUTH_TRANSFORM_ID0", pack=False, align=None), "cryptoModuleId": SimTypePointer(SimTypeBottom(label="Guid"), offset=0)}, name="IPSEC_AUTH_TRANSFORM0", pack=False, align=None), offset=0), "espAuthTransform": SimTypePointer(SimStruct({"authTransformId": SimStruct({"authType": SimTypeInt(signed=False, label="IPSEC_AUTH_TYPE"), "authConfig": SimTypeChar(label="Byte")}, name="IPSEC_AUTH_TRANSFORM_ID0", pack=False, align=None), "cryptoModuleId": SimTypePointer(SimTypeBottom(label="Guid"), offset=0)}, name="IPSEC_AUTH_TRANSFORM0", pack=False, align=None), offset=0), "espCipherTransform": SimTypePointer(SimStruct({"cipherTransformId": SimStruct({"cipherType": SimTypeInt(signed=False, label="IPSEC_CIPHER_TYPE"), "cipherConfig": SimTypeChar(label="Byte")}, name="IPSEC_CIPHER_TRANSFORM_ID0", pack=False, align=None), "cryptoModuleId": SimTypePointer(SimTypeBottom(label="Guid"), offset=0)}, name="IPSEC_CIPHER_TRANSFORM0", pack=False, align=None), offset=0), "espAuthAndCipherTransform": SimTypePointer(SimStruct({"authTransform": SimStruct({"authTransformId": SimStruct({"authType": SimTypeInt(signed=False, label="IPSEC_AUTH_TYPE"), "authConfig": SimTypeChar(label="Byte")}, name="IPSEC_AUTH_TRANSFORM_ID0", pack=False, align=None), "cryptoModuleId": SimTypePointer(SimTypeBottom(label="Guid"), offset=0)}, name="IPSEC_AUTH_TRANSFORM0", pack=False, align=None), "cipherTransform": SimStruct({"cipherTransformId": SimStruct({"cipherType": SimTypeInt(signed=False, label="IPSEC_CIPHER_TYPE"), "cipherConfig": SimTypeChar(label="Byte")}, name="IPSEC_CIPHER_TRANSFORM_ID0", pack=False, align=None), "cryptoModuleId": SimTypePointer(SimTypeBottom(label="Guid"), offset=0)}, name="IPSEC_CIPHER_TRANSFORM0", pack=False, align=None)}, name="IPSEC_AUTH_AND_CIPHER_TRANSFORM0", pack=False, align=None), offset=0), "espAuthFwTransform": SimTypePointer(SimStruct({"authTransformId": SimStruct({"authType": SimTypeInt(signed=False, label="IPSEC_AUTH_TYPE"), "authConfig": SimTypeChar(label="Byte")}, name="IPSEC_AUTH_TRANSFORM_ID0", pack=False, align=None), "cryptoModuleId": SimTypePointer(SimTypeBottom(label="Guid"), offset=0)}, name="IPSEC_AUTH_TRANSFORM0", pack=False, align=None), offset=0)}, name="<anon>", label="None")}, name="IPSEC_SA_TRANSFORM0", pack=False, align=None), offset=0), "pfsGroup": SimTypeInt(signed=False, label="IPSEC_PFS_GROUP")}, name="IPSEC_PROPOSAL0", pack=False, align=None), offset=0), "tunnelEndpoints": SimStruct({"ipVersion": SimTypeInt(signed=False, label="FWP_IP_VERSION"), "Anonymous1": SimUnion({"localV4Address": SimTypeInt(signed=False, label="UInt32"), "localV6Address": SimTypeFixedSizeArray(SimTypeChar(label="Byte"), 16)}, name="<anon>", label="None"), "Anonymous2": SimUnion({"remoteV4Address": SimTypeInt(signed=False, label="UInt32"), "remoteV6Address": SimTypeFixedSizeArray(SimTypeChar(label="Byte"), 16)}, name="<anon>", label="None"), "localIfLuid": SimTypeLongLong(signed=False, label="UInt64"), "remoteFqdn": SimTypePointer(SimTypeChar(label="Char"), offset=0), "numAddresses": SimTypeInt(signed=False, label="UInt32"), "remoteAddresses": SimTypePointer(SimStruct({"ipVersion": SimTypeInt(signed=False, label="FWP_IP_VERSION"), "Anonymous": SimUnion({"v4Address": SimTypeInt(signed=False, label="UInt32"), "v6Address": SimTypeFixedSizeArray(SimTypeChar(label="Byte"), 16)}, name="<anon>", label="None")}, name="IPSEC_TUNNEL_ENDPOINT0", pack=False, align=None), offset=0)}, name="IPSEC_TUNNEL_ENDPOINTS2", pack=False, align=None), "saIdleTimeout": SimStruct({"idleTimeoutSeconds": SimTypeInt(signed=False, label="UInt32"), "idleTimeoutSecondsFailOver": SimTypeInt(signed=False, label="UInt32")}, name="IPSEC_SA_IDLE_TIMEOUT0", pack=False, align=None), "emPolicy": SimTypePointer(SimStruct({"numAuthenticationMethods": SimTypeInt(signed=False, label="UInt32"), "authenticationMethods": SimTypePointer(SimStruct({"authenticationMethodType": SimTypeInt(signed=False, label="IKEEXT_AUTHENTICATION_METHOD_TYPE"), "Anonymous": SimUnion({"presharedKeyAuthentication": SimStruct({"presharedKey": SimStruct({"size": SimTypeInt(signed=False, label="UInt32"), "data": SimTypePointer(SimTypeChar(label="Byte"), offset=0)}, name="FWP_BYTE_BLOB", pack=False, align=None), "flags": SimTypeInt(signed=False, label="IKEEXT_PRESHARED_KEY_AUTHENTICATION_FLAGS")}, name="IKEEXT_PRESHARED_KEY_AUTHENTICATION1", pack=False, align=None), "certificateAuthentication": SimStruct({"inboundConfigType": SimTypeInt(signed=False, label="IKEEXT_CERT_CONFIG_TYPE"), "Anonymous1": SimUnion({"Anonymous1": SimStruct({"inboundRootArraySize": SimTypeInt(signed=False, label="UInt32"), "inboundRootCriteria": SimTypePointer(SimStruct({"certData": SimStruct({"size": SimTypeInt(signed=False, label="UInt32"), "data": SimTypePointer(SimTypeChar(label="Byte"), offset=0)}, name="FWP_BYTE_BLOB", pack=False, align=None), "certHash": SimStruct({"size": SimTypeInt(signed=False, label="UInt32"), "data": SimTypePointer(SimTypeChar(label="Byte"), offset=0)}, name="FWP_BYTE_BLOB", pack=False, align=None), "eku": SimTypePointer(SimStruct({"numEku": SimTypeInt(signed=False, label="UInt32"), "eku": SimTypePointer(SimTypePointer(SimTypeChar(label="Byte"), offset=0), offset=0)}, name="IKEEXT_CERT_EKUS0", pack=False, align=None), offset=0), "name": SimTypePointer(SimStruct({"nameType": SimTypeInt(signed=False, label="IKEEXT_CERT_CRITERIA_NAME_TYPE"), "certName": SimTypePointer(SimTypeChar(label="Char"), offset=0)}, name="IKEEXT_CERT_NAME0", pack=False, align=None), offset=0), "flags": SimTypeInt(signed=False, label="UInt32")}, name="IKEEXT_CERTIFICATE_CRITERIA0", pack=False, align=None), offset=0)}, name="_Anonymous1_e__Struct", pack=False, align=None), "Anonymous2": SimStruct({"inboundEnterpriseStoreArraySize": SimTypeInt(signed=False, label="UInt32"), "inboundEnterpriseStoreCriteria": SimTypePointer(SimStruct({"certData": SimStruct({"size": SimTypeInt(signed=False, label="UInt32"), "data": SimTypePointer(SimTypeChar(label="Byte"), offset=0)}, name="FWP_BYTE_BLOB", pack=False, align=None), "certHash": SimStruct({"size": SimTypeInt(signed=False, label="UInt32"), "data": SimTypePointer(SimTypeChar(label="Byte"), offset=0)}, name="FWP_BYTE_BLOB", pack=False, align=None), "eku": SimTypePointer(SimStruct({"numEku": SimTypeInt(signed=False, label="UInt32"), "eku": SimTypePointer(SimTypePointer(SimTypeChar(label="Byte"), offset=0), offset=0)}, name="IKEEXT_CERT_EKUS0", pack=False, align=None), offset=0), "name": SimTypePointer(SimStruct({"nameType": SimTypeInt(signed=False, label="IKEEXT_CERT_CRITERIA_NAME_TYPE"), "certName": SimTypePointer(SimTypeChar(label="Char"), offset=0)}, name="IKEEXT_CERT_NAME0", pack=False, align=None), offset=0), "flags": SimTypeInt(signed=False, label="UInt32")}, name="IKEEXT_CERTIFICATE_CRITERIA0", pack=False, align=None), offset=0)}, name="_Anonymous2_e__Struct", pack=False, align=None), "Anonymous3": SimStruct({"inboundRootStoreArraySize": SimTypeInt(signed=False, label="UInt32"), "inboundTrustedRootStoreCriteria": SimTypePointer(SimStruct({"certData": SimStruct({"size": SimTypeInt(signed=False, label="UInt32"), "data": SimTypePointer(SimTypeChar(label="Byte"), offset=0)}, name="FWP_BYTE_BLOB", pack=False, align=None), "certHash": SimStruct({"size": SimTypeInt(signed=False, label="UInt32"), "data": SimTypePointer(SimTypeChar(label="Byte"), offset=0)}, name="FWP_BYTE_BLOB", pack=False, align=None), "eku": SimTypePointer(SimStruct({"numEku": SimTypeInt(signed=False, label="UInt32"), "eku": SimTypePointer(SimTypePointer(SimTypeChar(label="Byte"), offset=0), offset=0)}, name="IKEEXT_CERT_EKUS0", pack=False, align=None), offset=0), "name": SimTypePointer(SimStruct({"nameType": SimTypeInt(signed=False, label="IKEEXT_CERT_CRITERIA_NAME_TYPE"), "certName": SimTypePointer(SimTypeChar(label="Char"), offset=0)}, name="IKEEXT_CERT_NAME0", pack=False, align=None), offset=0), "flags": SimTypeInt(signed=False, label="UInt32")}, name="IKEEXT_CERTIFICATE_CRITERIA0", pack=False, align=None), offset=0)}, name="_Anonymous3_e__Struct", pack=False, align=None)}, name="<anon>", label="None"), "outboundConfigType": SimTypeInt(signed=False, label="IKEEXT_CERT_CONFIG_TYPE"), "Anonymous2": SimUnion({"Anonymous1": SimStruct({"outboundRootArraySize": SimTypeInt(signed=False, label="UInt32"), "outboundRootCriteria": SimTypePointer(SimStruct({"certData": SimStruct({"size": SimTypeInt(signed=False, label="UInt32"), "data": SimTypePointer(SimTypeChar(label="Byte"), offset=0)}, name="FWP_BYTE_BLOB", pack=False, align=None), "certHash": SimStruct({"size": SimTypeInt(signed=False, label="UInt32"), "data": SimTypePointer(SimTypeChar(label="Byte"), offset=0)}, name="FWP_BYTE_BLOB", pack=False, align=None), "eku": SimTypePointer(SimStruct({"numEku": SimTypeInt(signed=False, label="UInt32"), "eku": SimTypePointer(SimTypePointer(SimTypeChar(label="Byte"), offset=0), offset=0)}, name="IKEEXT_CERT_EKUS0", pack=False, align=None), offset=0), "name": SimTypePointer(SimStruct({"nameType": SimTypeInt(signed=False, label="IKEEXT_CERT_CRITERIA_NAME_TYPE"), "certName": SimTypePointer(SimTypeChar(label="Char"), offset=0)}, name="IKEEXT_CERT_NAME0", pack=False, align=None), offset=0), "flags": SimTypeInt(signed=False, label="UInt32")}, name="IKEEXT_CERTIFICATE_CRITERIA0", pack=False, align=None), offset=0)}, name="_Anonymous1_e__Struct", pack=False, align=None), "Anonymous2": SimStruct({"outboundEnterpriseStoreArraySize": SimTypeInt(signed=False, label="UInt32"), "outboundEnterpriseStoreCriteria": SimTypePointer(SimStruct({"certData": SimStruct({"size": SimTypeInt(signed=False, label="UInt32"), "data": SimTypePointer(SimTypeChar(label="Byte"), offset=0)}, name="FWP_BYTE_BLOB", pack=False, align=None), "certHash": SimStruct({"size": SimTypeInt(signed=False, label="UInt32"), "data": SimTypePointer(SimTypeChar(label="Byte"), offset=0)}, name="FWP_BYTE_BLOB", pack=False, align=None), "eku": SimTypePointer(SimStruct({"numEku": SimTypeInt(signed=False, label="UInt32"), "eku": SimTypePointer(SimTypePointer(SimTypeChar(label="Byte"), offset=0), offset=0)}, name="IKEEXT_CERT_EKUS0", pack=False, align=None), offset=0), "name": SimTypePointer(SimStruct({"nameType": SimTypeInt(signed=False, label="IKEEXT_CERT_CRITERIA_NAME_TYPE"), "certName": SimTypePointer(SimTypeChar(label="Char"), offset=0)}, name="IKEEXT_CERT_NAME0", pack=False, align=None), offset=0), "flags": SimTypeInt(signed=False, label="UInt32")}, name="IKEEXT_CERTIFICATE_CRITERIA0", pack=False, align=None), offset=0)}, name="_Anonymous2_e__Struct", pack=False, align=None), "Anonymous3": SimStruct({"outboundRootStoreArraySize": SimTypeInt(signed=False, label="UInt32"), "outboundTrustedRootStoreCriteria": SimTypePointer(SimStruct({"certData": SimStruct({"size": SimTypeInt(signed=False, label="UInt32"), "data": SimTypePointer(SimTypeChar(label="Byte"), offset=0)}, name="FWP_BYTE_BLOB", pack=False, align=None), "certHash": SimStruct({"size": SimTypeInt(signed=False, label="UInt32"), "data": SimTypePointer(SimTypeChar(label="Byte"), offset=0)}, name="FWP_BYTE_BLOB", pack=False, align=None), "eku": SimTypePointer(SimStruct({"numEku": SimTypeInt(signed=False, label="UInt32"), "eku": SimTypePointer(SimTypePointer(SimTypeChar(label="Byte"), offset=0), offset=0)}, name="IKEEXT_CERT_EKUS0", pack=False, align=None), offset=0), "name": SimTypePointer(SimStruct({"nameType": SimTypeInt(signed=False, label="IKEEXT_CERT_CRITERIA_NAME_TYPE"), "certName": SimTypePointer(SimTypeChar(label="Char"), offset=0)}, name="IKEEXT_CERT_NAME0", pack=False, align=None), offset=0), "flags": SimTypeInt(signed=False, label="UInt32")}, name="IKEEXT_CERTIFICATE_CRITERIA0", pack=False, align=None), offset=0)}, name="_Anonymous3_e__Struct", pack=False, align=None)}, name="<anon>", label="None"), "flags": SimTypeInt(signed=False, label="IKEEXT_CERT_AUTH"), "localCertLocationUrl": SimStruct({"size": SimTypeInt(signed=False, label="UInt32"), "data": SimTypePointer(SimTypeChar(label="Byte"), offset=0)}, name="FWP_BYTE_BLOB", pack=False, align=None)}, name="IKEEXT_CERTIFICATE_AUTHENTICATION2", pack=False, align=None), "kerberosAuthentication": SimStruct({"flags": SimTypeInt(signed=False, label="IKEEXT_KERBEROS_AUTHENTICATION_FLAGS"), "proxyServer": SimTypePointer(SimTypeChar(label="Char"), offset=0)}, name="IKEEXT_KERBEROS_AUTHENTICATION1", pack=False, align=None), "reservedAuthentication": SimStruct({"flags": SimTypeInt(signed=False, label="IKEEXT_RESERVED_AUTHENTICATION_FLAGS")}, name="IKEEXT_RESERVED_AUTHENTICATION0", pack=False, align=None), "ntlmV2Authentication": SimStruct({"flags": SimTypeInt(signed=False, label="UInt32")}, name="IKEEXT_NTLM_V2_AUTHENTICATION0", pack=False, align=None), "sslAuthentication": SimStruct({"inboundConfigType": SimTypeInt(signed=False, label="IKEEXT_CERT_CONFIG_TYPE"), "Anonymous1": SimUnion({"Anonymous1": SimStruct({"inboundRootArraySize": SimTypeInt(signed=False, label="UInt32"), "inboundRootCriteria": SimTypePointer(SimStruct({"certData": SimStruct({"size": SimTypeInt(signed=False, label="UInt32"), "data": SimTypePointer(SimTypeChar(label="Byte"), offset=0)}, name="FWP_BYTE_BLOB", pack=False, align=None), "certHash": SimStruct({"size": SimTypeInt(signed=False, label="UInt32"), "data": SimTypePointer(SimTypeChar(label="Byte"), offset=0)}, name="FWP_BYTE_BLOB", pack=False, align=None), "eku": SimTypePointer(SimStruct({"numEku": SimTypeInt(signed=False, label="UInt32"), "eku": SimTypePointer(SimTypePointer(SimTypeChar(label="Byte"), offset=0), offset=0)}, name="IKEEXT_CERT_EKUS0", pack=False, align=None), offset=0), "name": SimTypePointer(SimStruct({"nameType": SimTypeInt(signed=False, label="IKEEXT_CERT_CRITERIA_NAME_TYPE"), "certName": SimTypePointer(SimTypeChar(label="Char"), offset=0)}, name="IKEEXT_CERT_NAME0", pack=False, align=None), offset=0), "flags": SimTypeInt(signed=False, label="UInt32")}, name="IKEEXT_CERTIFICATE_CRITERIA0", pack=False, align=None), offset=0)}, name="_Anonymous1_e__Struct", pack=False, align=None), "Anonymous2": SimStruct({"inboundEnterpriseStoreArraySize": SimTypeInt(signed=False, label="UInt32"), "inboundEnterpriseStoreCriteria": SimTypePointer(SimStruct({"certData": SimStruct({"size": SimTypeInt(signed=False, label="UInt32"), "data": SimTypePointer(SimTypeChar(label="Byte"), offset=0)}, name="FWP_BYTE_BLOB", pack=False, align=None), "certHash": SimStruct({"size": SimTypeInt(signed=False, label="UInt32"), "data": SimTypePointer(SimTypeChar(label="Byte"), offset=0)}, name="FWP_BYTE_BLOB", pack=False, align=None), "eku": SimTypePointer(SimStruct({"numEku": SimTypeInt(signed=False, label="UInt32"), "eku": SimTypePointer(SimTypePointer(SimTypeChar(label="Byte"), offset=0), offset=0)}, name="IKEEXT_CERT_EKUS0", pack=False, align=None), offset=0), "name": SimTypePointer(SimStruct({"nameType": SimTypeInt(signed=False, label="IKEEXT_CERT_CRITERIA_NAME_TYPE"), "certName": SimTypePointer(SimTypeChar(label="Char"), offset=0)}, name="IKEEXT_CERT_NAME0", pack=False, align=None), offset=0), "flags": SimTypeInt(signed=False, label="UInt32")}, name="IKEEXT_CERTIFICATE_CRITERIA0", pack=False, align=None), offset=0)}, name="_Anonymous2_e__Struct", pack=False, align=None), "Anonymous3": SimStruct({"inboundRootStoreArraySize": SimTypeInt(signed=False, label="UInt32"), "inboundTrustedRootStoreCriteria": SimTypePointer(SimStruct({"certData": SimStruct({"size": SimTypeInt(signed=False, label="UInt32"), "data": SimTypePointer(SimTypeChar(label="Byte"), offset=0)}, name="FWP_BYTE_BLOB", pack=False, align=None), "certHash": SimStruct({"size": SimTypeInt(signed=False, label="UInt32"), "data": SimTypePointer(SimTypeChar(label="Byte"), offset=0)}, name="FWP_BYTE_BLOB", pack=False, align=None), "eku": SimTypePointer(SimStruct({"numEku": SimTypeInt(signed=False, label="UInt32"), "eku": SimTypePointer(SimTypePointer(SimTypeChar(label="Byte"), offset=0), offset=0)}, name="IKEEXT_CERT_EKUS0", pack=False, align=None), offset=0), "name": SimTypePointer(SimStruct({"nameType": SimTypeInt(signed=False, label="IKEEXT_CERT_CRITERIA_NAME_TYPE"), "certName": SimTypePointer(SimTypeChar(label="Char"), offset=0)}, name="IKEEXT_CERT_NAME0", pack=False, align=None), offset=0), "flags": SimTypeInt(signed=False, label="UInt32")}, name="IKEEXT_CERTIFICATE_CRITERIA0", pack=False, align=None), offset=0)}, name="_Anonymous3_e__Struct", pack=False, align=None)}, name="<anon>", label="None"), "outboundConfigType": SimTypeInt(signed=False, label="IKEEXT_CERT_CONFIG_TYPE"), "Anonymous2": SimUnion({"Anonymous1": SimStruct({"outboundRootArraySize": SimTypeInt(signed=False, label="UInt32"), "outboundRootCriteria": SimTypePointer(SimStruct({"certData": SimStruct({"size": SimTypeInt(signed=False, label="UInt32"), "data": SimTypePointer(SimTypeChar(label="Byte"), offset=0)}, name="FWP_BYTE_BLOB", pack=False, align=None), "certHash": SimStruct({"size": SimTypeInt(signed=False, label="UInt32"), "data": SimTypePointer(SimTypeChar(label="Byte"), offset=0)}, name="FWP_BYTE_BLOB", pack=False, align=None), "eku": SimTypePointer(SimStruct({"numEku": SimTypeInt(signed=False, label="UInt32"), "eku": SimTypePointer(SimTypePointer(SimTypeChar(label="Byte"), offset=0), offset=0)}, name="IKEEXT_CERT_EKUS0", pack=False, align=None), offset=0), "name": SimTypePointer(SimStruct({"nameType": SimTypeInt(signed=False, label="IKEEXT_CERT_CRITERIA_NAME_TYPE"), "certName": SimTypePointer(SimTypeChar(label="Char"), offset=0)}, name="IKEEXT_CERT_NAME0", pack=False, align=None), offset=0), "flags": SimTypeInt(signed=False, label="UInt32")}, name="IKEEXT_CERTIFICATE_CRITERIA0", pack=False, align=None), offset=0)}, name="_Anonymous1_e__Struct", pack=False, align=None), "Anonymous2": SimStruct({"outboundEnterpriseStoreArraySize": SimTypeInt(signed=False, label="UInt32"), "outboundEnterpriseStoreCriteria": SimTypePointer(SimStruct({"certData": SimStruct({"size": SimTypeInt(signed=False, label="UInt32"), "data": SimTypePointer(SimTypeChar(label="Byte"), offset=0)}, name="FWP_BYTE_BLOB", pack=False, align=None), "certHash": SimStruct({"size": SimTypeInt(signed=False, label="UInt32"), "data": SimTypePointer(SimTypeChar(label="Byte"), offset=0)}, name="FWP_BYTE_BLOB", pack=False, align=None), "eku": SimTypePointer(SimStruct({"numEku": SimTypeInt(signed=False, label="UInt32"), "eku": SimTypePointer(SimTypePointer(SimTypeChar(label="Byte"), offset=0), offset=0)}, name="IKEEXT_CERT_EKUS0", pack=False, align=None), offset=0), "name": SimTypePointer(SimStruct({"nameType": SimTypeInt(signed=False, label="IKEEXT_CERT_CRITERIA_NAME_TYPE"), "certName": SimTypePointer(SimTypeChar(label="Char"), offset=0)}, name="IKEEXT_CERT_NAME0", pack=False, align=None), offset=0), "flags": SimTypeInt(signed=False, label="UInt32")}, name="IKEEXT_CERTIFICATE_CRITERIA0", pack=False, align=None), offset=0)}, name="_Anonymous2_e__Struct", pack=False, align=None), "Anonymous3": SimStruct({"outboundRootStoreArraySize": SimTypeInt(signed=False, label="UInt32"), "outboundTrustedRootStoreCriteria": SimTypePointer(SimStruct({"certData": SimStruct({"size": SimTypeInt(signed=False, label="UInt32"), "data": SimTypePointer(SimTypeChar(label="Byte"), offset=0)}, name="FWP_BYTE_BLOB", pack=False, align=None), "certHash": SimStruct({"size": SimTypeInt(signed=False, label="UInt32"), "data": SimTypePointer(SimTypeChar(label="Byte"), offset=0)}, name="FWP_BYTE_BLOB", pack=False, align=None), "eku": SimTypePointer(SimStruct({"numEku": SimTypeInt(signed=False, label="UInt32"), "eku": SimTypePointer(SimTypePointer(SimTypeChar(label="Byte"), offset=0), offset=0)}, name="IKEEXT_CERT_EKUS0", pack=False, align=None), offset=0), "name": SimTypePointer(SimStruct({"nameType": SimTypeInt(signed=False, label="IKEEXT_CERT_CRITERIA_NAME_TYPE"), "certName": SimTypePointer(SimTypeChar(label="Char"), offset=0)}, name="IKEEXT_CERT_NAME0", pack=False, align=None), offset=0), "flags": SimTypeInt(signed=False, label="UInt32")}, name="IKEEXT_CERTIFICATE_CRITERIA0", pack=False, align=None), offset=0)}, name="_Anonymous3_e__Struct", pack=False, align=None)}, name="<anon>", label="None"), "flags": SimTypeInt(signed=False, label="IKEEXT_CERT_AUTH"), "localCertLocationUrl": SimStruct({"size": SimTypeInt(signed=False, label="UInt32"), "data": SimTypePointer(SimTypeChar(label="Byte"), offset=0)}, name="FWP_BYTE_BLOB", pack=False, align=None)}, name="IKEEXT_CERTIFICATE_AUTHENTICATION2", pack=False, align=None), "cgaAuthentication": SimStruct({"keyContainerName": SimTypePointer(SimTypeChar(label="Char"), offset=0), "cspName": SimTypePointer(SimTypeChar(label="Char"), offset=0), "cspType": SimTypeInt(signed=False, label="UInt32"), "cgaModifier": SimStruct({"byteArray16": SimTypeFixedSizeArray(SimTypeChar(label="Byte"), 16)}, name="FWP_BYTE_ARRAY16", pack=False, align=None), "cgaCollisionCount": SimTypeChar(label="Byte")}, name="IKEEXT_IPV6_CGA_AUTHENTICATION0", pack=False, align=None), "eapAuthentication": SimStruct({"flags": SimTypeInt(signed=False, label="IKEEXT_EAP_AUTHENTICATION_FLAGS")}, name="IKEEXT_EAP_AUTHENTICATION0", pack=False, align=None)}, name="<anon>", label="None")}, name="IKEEXT_AUTHENTICATION_METHOD2", pack=False, align=None), offset=0), "initiatorImpersonationType": SimTypeInt(signed=False, label="IKEEXT_AUTHENTICATION_IMPERSONATION_TYPE")}, name="IKEEXT_EM_POLICY2", pack=False, align=None), offset=0), "fwdPathSaLifetime": SimTypeInt(signed=False, label="UInt32")}, name="IPSEC_TUNNEL_POLICY2", pack=False, align=None), offset=0), "ikeMmPolicy": SimTypePointer(SimStruct({"softExpirationTime": SimTypeInt(signed=False, label="UInt32"), "numAuthenticationMethods": SimTypeInt(signed=False, label="UInt32"), "authenticationMethods": SimTypePointer(SimStruct({"authenticationMethodType": SimTypeInt(signed=False, label="IKEEXT_AUTHENTICATION_METHOD_TYPE"), "Anonymous": SimUnion({"presharedKeyAuthentication": SimStruct({"presharedKey": SimStruct({"size": SimTypeInt(signed=False, label="UInt32"), "data": SimTypePointer(SimTypeChar(label="Byte"), offset=0)}, name="FWP_BYTE_BLOB", pack=False, align=None), "flags": SimTypeInt(signed=False, label="IKEEXT_PRESHARED_KEY_AUTHENTICATION_FLAGS")}, name="IKEEXT_PRESHARED_KEY_AUTHENTICATION1", pack=False, align=None), "certificateAuthentication": SimStruct({"inboundConfigType": SimTypeInt(signed=False, label="IKEEXT_CERT_CONFIG_TYPE"), "Anonymous1": SimUnion({"Anonymous1": SimStruct({"inboundRootArraySize": SimTypeInt(signed=False, label="UInt32"), "inboundRootCriteria": SimTypePointer(SimStruct({"certData": SimStruct({"size": SimTypeInt(signed=False, label="UInt32"), "data": SimTypePointer(SimTypeChar(label="Byte"), offset=0)}, name="FWP_BYTE_BLOB", pack=False, align=None), "certHash": SimStruct({"size": SimTypeInt(signed=False, label="UInt32"), "data": SimTypePointer(SimTypeChar(label="Byte"), offset=0)}, name="FWP_BYTE_BLOB", pack=False, align=None), "eku": SimTypePointer(SimStruct({"numEku": SimTypeInt(signed=False, label="UInt32"), "eku": SimTypePointer(SimTypePointer(SimTypeChar(label="Byte"), offset=0), offset=0)}, name="IKEEXT_CERT_EKUS0", pack=False, align=None), offset=0), "name": SimTypePointer(SimStruct({"nameType": SimTypeInt(signed=False, label="IKEEXT_CERT_CRITERIA_NAME_TYPE"), "certName": SimTypePointer(SimTypeChar(label="Char"), offset=0)}, name="IKEEXT_CERT_NAME0", pack=False, align=None), offset=0), "flags": SimTypeInt(signed=False, label="UInt32")}, name="IKEEXT_CERTIFICATE_CRITERIA0", pack=False, align=None), offset=0)}, name="_Anonymous1_e__Struct", pack=False, align=None), "Anonymous2": SimStruct({"inboundEnterpriseStoreArraySize": SimTypeInt(signed=False, label="UInt32"), "inboundEnterpriseStoreCriteria": SimTypePointer(SimStruct({"certData": SimStruct({"size": SimTypeInt(signed=False, label="UInt32"), "data": SimTypePointer(SimTypeChar(label="Byte"), offset=0)}, name="FWP_BYTE_BLOB", pack=False, align=None), "certHash": SimStruct({"size": SimTypeInt(signed=False, label="UInt32"), "data": SimTypePointer(SimTypeChar(label="Byte"), offset=0)}, name="FWP_BYTE_BLOB", pack=False, align=None), "eku": SimTypePointer(SimStruct({"numEku": SimTypeInt(signed=False, label="UInt32"), "eku": SimTypePointer(SimTypePointer(SimTypeChar(label="Byte"), offset=0), offset=0)}, name="IKEEXT_CERT_EKUS0", pack=False, align=None), offset=0), "name": SimTypePointer(SimStruct({"nameType": SimTypeInt(signed=False, label="IKEEXT_CERT_CRITERIA_NAME_TYPE"), "certName": SimTypePointer(SimTypeChar(label="Char"), offset=0)}, name="IKEEXT_CERT_NAME0", pack=False, align=None), offset=0), "flags": SimTypeInt(signed=False, label="UInt32")}, name="IKEEXT_CERTIFICATE_CRITERIA0", pack=False, align=None), offset=0)}, name="_Anonymous2_e__Struct", pack=False, align=None), "Anonymous3": SimStruct({"inboundRootStoreArraySize": SimTypeInt(signed=False, label="UInt32"), "inboundTrustedRootStoreCriteria": SimTypePointer(SimStruct({"certData": SimStruct({"size": SimTypeInt(signed=False, label="UInt32"), "data": SimTypePointer(SimTypeChar(label="Byte"), offset=0)}, name="FWP_BYTE_BLOB", pack=False, align=None), "certHash": SimStruct({"size": SimTypeInt(signed=False, label="UInt32"), "data": SimTypePointer(SimTypeChar(label="Byte"), offset=0)}, name="FWP_BYTE_BLOB", pack=False, align=None), "eku": SimTypePointer(SimStruct({"numEku": SimTypeInt(signed=False, label="UInt32"), "eku": SimTypePointer(SimTypePointer(SimTypeChar(label="Byte"), offset=0), offset=0)}, name="IKEEXT_CERT_EKUS0", pack=False, align=None), offset=0), "name": SimTypePointer(SimStruct({"nameType": SimTypeInt(signed=False, label="IKEEXT_CERT_CRITERIA_NAME_TYPE"), "certName": SimTypePointer(SimTypeChar(label="Char"), offset=0)}, name="IKEEXT_CERT_NAME0", pack=False, align=None), offset=0), "flags": SimTypeInt(signed=False, label="UInt32")}, name="IKEEXT_CERTIFICATE_CRITERIA0", pack=False, align=None), offset=0)}, name="_Anonymous3_e__Struct", pack=False, align=None)}, name="<anon>", label="None"), "outboundConfigType": SimTypeInt(signed=False, label="IKEEXT_CERT_CONFIG_TYPE"), "Anonymous2": SimUnion({"Anonymous1": SimStruct({"outboundRootArraySize": SimTypeInt(signed=False, label="UInt32"), "outboundRootCriteria": SimTypePointer(SimStruct({"certData": SimStruct({"size": SimTypeInt(signed=False, label="UInt32"), "data": SimTypePointer(SimTypeChar(label="Byte"), offset=0)}, name="FWP_BYTE_BLOB", pack=False, align=None), "certHash": SimStruct({"size": SimTypeInt(signed=False, label="UInt32"), "data": SimTypePointer(SimTypeChar(label="Byte"), offset=0)}, name="FWP_BYTE_BLOB", pack=False, align=None), "eku": SimTypePointer(SimStruct({"numEku": SimTypeInt(signed=False, label="UInt32"), "eku": SimTypePointer(SimTypePointer(SimTypeChar(label="Byte"), offset=0), offset=0)}, name="IKEEXT_CERT_EKUS0", pack=False, align=None), offset=0), "name": SimTypePointer(SimStruct({"nameType": SimTypeInt(signed=False, label="IKEEXT_CERT_CRITERIA_NAME_TYPE"), "certName": SimTypePointer(SimTypeChar(label="Char"), offset=0)}, name="IKEEXT_CERT_NAME0", pack=False, align=None), offset=0), "flags": SimTypeInt(signed=False, label="UInt32")}, name="IKEEXT_CERTIFICATE_CRITERIA0", pack=False, align=None), offset=0)}, name="_Anonymous1_e__Struct", pack=False, align=None), "Anonymous2": SimStruct({"outboundEnterpriseStoreArraySize": SimTypeInt(signed=False, label="UInt32"), "outboundEnterpriseStoreCriteria": SimTypePointer(SimStruct({"certData": SimStruct({"size": SimTypeInt(signed=False, label="UInt32"), "data": SimTypePointer(SimTypeChar(label="Byte"), offset=0)}, name="FWP_BYTE_BLOB", pack=False, align=None), "certHash": SimStruct({"size": SimTypeInt(signed=False, label="UInt32"), "data": SimTypePointer(SimTypeChar(label="Byte"), offset=0)}, name="FWP_BYTE_BLOB", pack=False, align=None), "eku": SimTypePointer(SimStruct({"numEku": SimTypeInt(signed=False, label="UInt32"), "eku": SimTypePointer(SimTypePointer(SimTypeChar(label="Byte"), offset=0), offset=0)}, name="IKEEXT_CERT_EKUS0", pack=False, align=None), offset=0), "name": SimTypePointer(SimStruct({"nameType": SimTypeInt(signed=False, label="IKEEXT_CERT_CRITERIA_NAME_TYPE"), "certName": SimTypePointer(SimTypeChar(label="Char"), offset=0)}, name="IKEEXT_CERT_NAME0", pack=False, align=None), offset=0), "flags": SimTypeInt(signed=False, label="UInt32")}, name="IKEEXT_CERTIFICATE_CRITERIA0", pack=False, align=None), offset=0)}, name="_Anonymous2_e__Struct", pack=False, align=None), "Anonymous3": SimStruct({"outboundRootStoreArraySize": SimTypeInt(signed=False, label="UInt32"), "outboundTrustedRootStoreCriteria": SimTypePointer(SimStruct({"certData": SimStruct({"size": SimTypeInt(signed=False, label="UInt32"), "data": SimTypePointer(SimTypeChar(label="Byte"), offset=0)}, name="FWP_BYTE_BLOB", pack=False, align=None), "certHash": SimStruct({"size": SimTypeInt(signed=False, label="UInt32"), "data": SimTypePointer(SimTypeChar(label="Byte"), offset=0)}, name="FWP_BYTE_BLOB", pack=False, align=None), "eku": SimTypePointer(SimStruct({"numEku": SimTypeInt(signed=False, label="UInt32"), "eku": SimTypePointer(SimTypePointer(SimTypeChar(label="Byte"), offset=0), offset=0)}, name="IKEEXT_CERT_EKUS0", pack=False, align=None), offset=0), "name": SimTypePointer(SimStruct({"nameType": SimTypeInt(signed=False, label="IKEEXT_CERT_CRITERIA_NAME_TYPE"), "certName": SimTypePointer(SimTypeChar(label="Char"), offset=0)}, name="IKEEXT_CERT_NAME0", pack=False, align=None), offset=0), "flags": SimTypeInt(signed=False, label="UInt32")}, name="IKEEXT_CERTIFICATE_CRITERIA0", pack=False, align=None), offset=0)}, name="_Anonymous3_e__Struct", pack=False, align=None)}, name="<anon>", label="None"), "flags": SimTypeInt(signed=False, label="IKEEXT_CERT_AUTH"), "localCertLocationUrl": SimStruct({"size": SimTypeInt(signed=False, label="UInt32"), "data": SimTypePointer(SimTypeChar(label="Byte"), offset=0)}, name="FWP_BYTE_BLOB", pack=False, align=None)}, name="IKEEXT_CERTIFICATE_AUTHENTICATION2", pack=False, align=None), "kerberosAuthentication": SimStruct({"flags": SimTypeInt(signed=False, label="IKEEXT_KERBEROS_AUTHENTICATION_FLAGS"), "proxyServer": SimTypePointer(SimTypeChar(label="Char"), offset=0)}, name="IKEEXT_KERBEROS_AUTHENTICATION1", pack=False, align=None), "reservedAuthentication": SimStruct({"flags": SimTypeInt(signed=False, label="IKEEXT_RESERVED_AUTHENTICATION_FLAGS")}, name="IKEEXT_RESERVED_AUTHENTICATION0", pack=False, align=None), "ntlmV2Authentication": SimStruct({"flags": SimTypeInt(signed=False, label="UInt32")}, name="IKEEXT_NTLM_V2_AUTHENTICATION0", pack=False, align=None), "sslAuthentication": SimStruct({"inboundConfigType": SimTypeInt(signed=False, label="IKEEXT_CERT_CONFIG_TYPE"), "Anonymous1": SimUnion({"Anonymous1": SimStruct({"inboundRootArraySize": SimTypeInt(signed=False, label="UInt32"), "inboundRootCriteria": SimTypePointer(SimStruct({"certData": SimStruct({"size": SimTypeInt(signed=False, label="UInt32"), "data": SimTypePointer(SimTypeChar(label="Byte"), offset=0)}, name="FWP_BYTE_BLOB", pack=False, align=None), "certHash": SimStruct({"size": SimTypeInt(signed=False, label="UInt32"), "data": SimTypePointer(SimTypeChar(label="Byte"), offset=0)}, name="FWP_BYTE_BLOB", pack=False, align=None), "eku": SimTypePointer(SimStruct({"numEku": SimTypeInt(signed=False, label="UInt32"), "eku": SimTypePointer(SimTypePointer(SimTypeChar(label="Byte"), offset=0), offset=0)}, name="IKEEXT_CERT_EKUS0", pack=False, align=None), offset=0), "name": SimTypePointer(SimStruct({"nameType": SimTypeInt(signed=False, label="IKEEXT_CERT_CRITERIA_NAME_TYPE"), "certName": SimTypePointer(SimTypeChar(label="Char"), offset=0)}, name="IKEEXT_CERT_NAME0", pack=False, align=None), offset=0), "flags": SimTypeInt(signed=False, label="UInt32")}, name="IKEEXT_CERTIFICATE_CRITERIA0", pack=False, align=None), offset=0)}, name="_Anonymous1_e__Struct", pack=False, align=None), "Anonymous2": SimStruct({"inboundEnterpriseStoreArraySize": SimTypeInt(signed=False, label="UInt32"), "inboundEnterpriseStoreCriteria": SimTypePointer(SimStruct({"certData": SimStruct({"size": SimTypeInt(signed=False, label="UInt32"), "data": SimTypePointer(SimTypeChar(label="Byte"), offset=0)}, name="FWP_BYTE_BLOB", pack=False, align=None), "certHash": SimStruct({"size": SimTypeInt(signed=False, label="UInt32"), "data": SimTypePointer(SimTypeChar(label="Byte"), offset=0)}, name="FWP_BYTE_BLOB", pack=False, align=None), "eku": SimTypePointer(SimStruct({"numEku": SimTypeInt(signed=False, label="UInt32"), "eku": SimTypePointer(SimTypePointer(SimTypeChar(label="Byte"), offset=0), offset=0)}, name="IKEEXT_CERT_EKUS0", pack=False, align=None), offset=0), "name": SimTypePointer(SimStruct({"nameType": SimTypeInt(signed=False, label="IKEEXT_CERT_CRITERIA_NAME_TYPE"), "certName": SimTypePointer(SimTypeChar(label="Char"), offset=0)}, name="IKEEXT_CERT_NAME0", pack=False, align=None), offset=0), "flags": SimTypeInt(signed=False, label="UInt32")}, name="IKEEXT_CERTIFICATE_CRITERIA0", pack=False, align=None), offset=0)}, name="_Anonymous2_e__Struct", pack=False, align=None), "Anonymous3": SimStruct({"inboundRootStoreArraySize": SimTypeInt(signed=False, label="UInt32"), "inboundTrustedRootStoreCriteria": SimTypePointer(SimStruct({"certData": SimStruct({"size": SimTypeInt(signed=False, label="UInt32"), "data": SimTypePointer(SimTypeChar(label="Byte"), offset=0)}, name="FWP_BYTE_BLOB", pack=False, align=None), "certHash": SimStruct({"size": SimTypeInt(signed=False, label="UInt32"), "data": SimTypePointer(SimTypeChar(label="Byte"), offset=0)}, name="FWP_BYTE_BLOB", pack=False, align=None), "eku": SimTypePointer(SimStruct({"numEku": SimTypeInt(signed=False, label="UInt32"), "eku": SimTypePointer(SimTypePointer(SimTypeChar(label="Byte"), offset=0), offset=0)}, name="IKEEXT_CERT_EKUS0", pack=False, align=None), offset=0), "name": SimTypePointer(SimStruct({"nameType": SimTypeInt(signed=False, label="IKEEXT_CERT_CRITERIA_NAME_TYPE"), "certName": SimTypePointer(SimTypeChar(label="Char"), offset=0)}, name="IKEEXT_CERT_NAME0", pack=False, align=None), offset=0), "flags": SimTypeInt(signed=False, label="UInt32")}, name="IKEEXT_CERTIFICATE_CRITERIA0", pack=False, align=None), offset=0)}, name="_Anonymous3_e__Struct", pack=False, align=None)}, name="<anon>", label="None"), "outboundConfigType": SimTypeInt(signed=False, label="IKEEXT_CERT_CONFIG_TYPE"), "Anonymous2": SimUnion({"Anonymous1": SimStruct({"outboundRootArraySize": SimTypeInt(signed=False, label="UInt32"), "outboundRootCriteria": SimTypePointer(SimStruct({"certData": SimStruct({"size": SimTypeInt(signed=False, label="UInt32"), "data": SimTypePointer(SimTypeChar(label="Byte"), offset=0)}, name="FWP_BYTE_BLOB", pack=False, align=None), "certHash": SimStruct({"size": SimTypeInt(signed=False, label="UInt32"), "data": SimTypePointer(SimTypeChar(label="Byte"), offset=0)}, name="FWP_BYTE_BLOB", pack=False, align=None), "eku": SimTypePointer(SimStruct({"numEku": SimTypeInt(signed=False, label="UInt32"), "eku": SimTypePointer(SimTypePointer(SimTypeChar(label="Byte"), offset=0), offset=0)}, name="IKEEXT_CERT_EKUS0", pack=False, align=None), offset=0), "name": SimTypePointer(SimStruct({"nameType": SimTypeInt(signed=False, label="IKEEXT_CERT_CRITERIA_NAME_TYPE"), "certName": SimTypePointer(SimTypeChar(label="Char"), offset=0)}, name="IKEEXT_CERT_NAME0", pack=False, align=None), offset=0), "flags": SimTypeInt(signed=False, label="UInt32")}, name="IKEEXT_CERTIFICATE_CRITERIA0", pack=False, align=None), offset=0)}, name="_Anonymous1_e__Struct", pack=False, align=None), "Anonymous2": SimStruct({"outboundEnterpriseStoreArraySize": SimTypeInt(signed=False, label="UInt32"), "outboundEnterpriseStoreCriteria": SimTypePointer(SimStruct({"certData": SimStruct({"size": SimTypeInt(signed=False, label="UInt32"), "data": SimTypePointer(SimTypeChar(label="Byte"), offset=0)}, name="FWP_BYTE_BLOB", pack=False, align=None), "certHash": SimStruct({"size": SimTypeInt(signed=False, label="UInt32"), "data": SimTypePointer(SimTypeChar(label="Byte"), offset=0)}, name="FWP_BYTE_BLOB", pack=False, align=None), "eku": SimTypePointer(SimStruct({"numEku": SimTypeInt(signed=False, label="UInt32"), "eku": SimTypePointer(SimTypePointer(SimTypeChar(label="Byte"), offset=0), offset=0)}, name="IKEEXT_CERT_EKUS0", pack=False, align=None), offset=0), "name": SimTypePointer(SimStruct({"nameType": SimTypeInt(signed=False, label="IKEEXT_CERT_CRITERIA_NAME_TYPE"), "certName": SimTypePointer(SimTypeChar(label="Char"), offset=0)}, name="IKEEXT_CERT_NAME0", pack=False, align=None), offset=0), "flags": SimTypeInt(signed=False, label="UInt32")}, name="IKEEXT_CERTIFICATE_CRITERIA0", pack=False, align=None), offset=0)}, name="_Anonymous2_e__Struct", pack=False, align=None), "Anonymous3": SimStruct({"outboundRootStoreArraySize": SimTypeInt(signed=False, label="UInt32"), "outboundTrustedRootStoreCriteria": SimTypePointer(SimStruct({"certData": SimStruct({"size": SimTypeInt(signed=False, label="UInt32"), "data": SimTypePointer(SimTypeChar(label="Byte"), offset=0)}, name="FWP_BYTE_BLOB", pack=False, align=None), "certHash": SimStruct({"size": SimTypeInt(signed=False, label="UInt32"), "data": SimTypePointer(SimTypeChar(label="Byte"), offset=0)}, name="FWP_BYTE_BLOB", pack=False, align=None), "eku": SimTypePointer(SimStruct({"numEku": SimTypeInt(signed=False, label="UInt32"), "eku": SimTypePointer(SimTypePointer(SimTypeChar(label="Byte"), offset=0), offset=0)}, name="IKEEXT_CERT_EKUS0", pack=False, align=None), offset=0), "name": SimTypePointer(SimStruct({"nameType": SimTypeInt(signed=False, label="IKEEXT_CERT_CRITERIA_NAME_TYPE"), "certName": SimTypePointer(SimTypeChar(label="Char"), offset=0)}, name="IKEEXT_CERT_NAME0", pack=False, align=None), offset=0), "flags": SimTypeInt(signed=False, label="UInt32")}, name="IKEEXT_CERTIFICATE_CRITERIA0", pack=False, align=None), offset=0)}, name="_Anonymous3_e__Struct", pack=False, align=None)}, name="<anon>", label="None"), "flags": SimTypeInt(signed=False, label="IKEEXT_CERT_AUTH"), "localCertLocationUrl": SimStruct({"size": SimTypeInt(signed=False, label="UInt32"), "data": SimTypePointer(SimTypeChar(label="Byte"), offset=0)}, name="FWP_BYTE_BLOB", pack=False, align=None)}, name="IKEEXT_CERTIFICATE_AUTHENTICATION2", pack=False, align=None), "cgaAuthentication": SimStruct({"keyContainerName": SimTypePointer(SimTypeChar(label="Char"), offset=0), "cspName": SimTypePointer(SimTypeChar(label="Char"), offset=0), "cspType": SimTypeInt(signed=False, label="UInt32"), "cgaModifier": SimStruct({"byteArray16": SimTypeFixedSizeArray(SimTypeChar(label="Byte"), 16)}, name="FWP_BYTE_ARRAY16", pack=False, align=None), "cgaCollisionCount": SimTypeChar(label="Byte")}, name="IKEEXT_IPV6_CGA_AUTHENTICATION0", pack=False, align=None), "eapAuthentication": SimStruct({"flags": SimTypeInt(signed=False, label="IKEEXT_EAP_AUTHENTICATION_FLAGS")}, name="IKEEXT_EAP_AUTHENTICATION0", pack=False, align=None)}, name="<anon>", label="None")}, name="IKEEXT_AUTHENTICATION_METHOD2", pack=False, align=None), offset=0), "initiatorImpersonationType": SimTypeInt(signed=False, label="IKEEXT_AUTHENTICATION_IMPERSONATION_TYPE"), "numIkeProposals": SimTypeInt(signed=False, label="UInt32"), "ikeProposals": SimTypePointer(SimStruct({"cipherAlgorithm": SimStruct({"algoIdentifier": SimTypeInt(signed=False, label="IKEEXT_CIPHER_TYPE"), "keyLen": SimTypeInt(signed=False, label="UInt32"), "rounds": SimTypeInt(signed=False, label="UInt32")}, name="IKEEXT_CIPHER_ALGORITHM0", pack=False, align=None), "integrityAlgorithm": SimStruct({"algoIdentifier": SimTypeInt(signed=False, label="IKEEXT_INTEGRITY_TYPE")}, name="IKEEXT_INTEGRITY_ALGORITHM0", pack=False, align=None), "maxLifetimeSeconds": SimTypeInt(signed=False, label="UInt32"), "dhGroup": SimTypeInt(signed=False, label="IKEEXT_DH_GROUP"), "quickModeLimit": SimTypeInt(signed=False, label="UInt32")}, name="IKEEXT_PROPOSAL0", pack=False, align=None), offset=0), "flags": SimTypeInt(signed=False, label="IKEEXT_POLICY_FLAG"), "maxDynamicFilters": SimTypeInt(signed=False, label="UInt32"), "retransmitDurationSecs": SimTypeInt(signed=False, label="UInt32")}, name="IKEEXT_POLICY2", pack=False, align=None), offset=0), "authIpMmPolicy": SimTypePointer(SimStruct({"softExpirationTime": SimTypeInt(signed=False, label="UInt32"), "numAuthenticationMethods": SimTypeInt(signed=False, label="UInt32"), "authenticationMethods": SimTypePointer(SimStruct({"authenticationMethodType": SimTypeInt(signed=False, label="IKEEXT_AUTHENTICATION_METHOD_TYPE"), "Anonymous": SimUnion({"presharedKeyAuthentication": SimStruct({"presharedKey": SimStruct({"size": SimTypeInt(signed=False, label="UInt32"), "data": SimTypePointer(SimTypeChar(label="Byte"), offset=0)}, name="FWP_BYTE_BLOB", pack=False, align=None), "flags": SimTypeInt(signed=False, label="IKEEXT_PRESHARED_KEY_AUTHENTICATION_FLAGS")}, name="IKEEXT_PRESHARED_KEY_AUTHENTICATION1", pack=False, align=None), "certificateAuthentication": SimStruct({"inboundConfigType": SimTypeInt(signed=False, label="IKEEXT_CERT_CONFIG_TYPE"), "Anonymous1": SimUnion({"Anonymous1": SimStruct({"inboundRootArraySize": SimTypeInt(signed=False, label="UInt32"), "inboundRootCriteria": SimTypePointer(SimStruct({"certData": SimStruct({"size": SimTypeInt(signed=False, label="UInt32"), "data": SimTypePointer(SimTypeChar(label="Byte"), offset=0)}, name="FWP_BYTE_BLOB", pack=False, align=None), "certHash": SimStruct({"size": SimTypeInt(signed=False, label="UInt32"), "data": SimTypePointer(SimTypeChar(label="Byte"), offset=0)}, name="FWP_BYTE_BLOB", pack=False, align=None), "eku": SimTypePointer(SimStruct({"numEku": SimTypeInt(signed=False, label="UInt32"), "eku": SimTypePointer(SimTypePointer(SimTypeChar(label="Byte"), offset=0), offset=0)}, name="IKEEXT_CERT_EKUS0", pack=False, align=None), offset=0), "name": SimTypePointer(SimStruct({"nameType": SimTypeInt(signed=False, label="IKEEXT_CERT_CRITERIA_NAME_TYPE"), "certName": SimTypePointer(SimTypeChar(label="Char"), offset=0)}, name="IKEEXT_CERT_NAME0", pack=False, align=None), offset=0), "flags": SimTypeInt(signed=False, label="UInt32")}, name="IKEEXT_CERTIFICATE_CRITERIA0", pack=False, align=None), offset=0)}, name="_Anonymous1_e__Struct", pack=False, align=None), "Anonymous2": SimStruct({"inboundEnterpriseStoreArraySize": SimTypeInt(signed=False, label="UInt32"), "inboundEnterpriseStoreCriteria": SimTypePointer(SimStruct({"certData": SimStruct({"size": SimTypeInt(signed=False, label="UInt32"), "data": SimTypePointer(SimTypeChar(label="Byte"), offset=0)}, name="FWP_BYTE_BLOB", pack=False, align=None), "certHash": SimStruct({"size": SimTypeInt(signed=False, label="UInt32"), "data": SimTypePointer(SimTypeChar(label="Byte"), offset=0)}, name="FWP_BYTE_BLOB", pack=False, align=None), "eku": SimTypePointer(SimStruct({"numEku": SimTypeInt(signed=False, label="UInt32"), "eku": SimTypePointer(SimTypePointer(SimTypeChar(label="Byte"), offset=0), offset=0)}, name="IKEEXT_CERT_EKUS0", pack=False, align=None), offset=0), "name": SimTypePointer(SimStruct({"nameType": SimTypeInt(signed=False, label="IKEEXT_CERT_CRITERIA_NAME_TYPE"), "certName": SimTypePointer(SimTypeChar(label="Char"), offset=0)}, name="IKEEXT_CERT_NAME0", pack=False, align=None), offset=0), "flags": SimTypeInt(signed=False, label="UInt32")}, name="IKEEXT_CERTIFICATE_CRITERIA0", pack=False, align=None), offset=0)}, name="_Anonymous2_e__Struct", pack=False, align=None), "Anonymous3": SimStruct({"inboundRootStoreArraySize": SimTypeInt(signed=False, label="UInt32"), "inboundTrustedRootStoreCriteria": SimTypePointer(SimStruct({"certData": SimStruct({"size": SimTypeInt(signed=False, label="UInt32"), "data": SimTypePointer(SimTypeChar(label="Byte"), offset=0)}, name="FWP_BYTE_BLOB", pack=False, align=None), "certHash": SimStruct({"size": SimTypeInt(signed=False, label="UInt32"), "data": SimTypePointer(SimTypeChar(label="Byte"), offset=0)}, name="FWP_BYTE_BLOB", pack=False, align=None), "eku": SimTypePointer(SimStruct({"numEku": SimTypeInt(signed=False, label="UInt32"), "eku": SimTypePointer(SimTypePointer(SimTypeChar(label="Byte"), offset=0), offset=0)}, name="IKEEXT_CERT_EKUS0", pack=False, align=None), offset=0), "name": SimTypePointer(SimStruct({"nameType": SimTypeInt(signed=False, label="IKEEXT_CERT_CRITERIA_NAME_TYPE"), "certName": SimTypePointer(SimTypeChar(label="Char"), offset=0)}, name="IKEEXT_CERT_NAME0", pack=False, align=None), offset=0), "flags": SimTypeInt(signed=False, label="UInt32")}, name="IKEEXT_CERTIFICATE_CRITERIA0", pack=False, align=None), offset=0)}, name="_Anonymous3_e__Struct", pack=False, align=None)}, name="<anon>", label="None"), "outboundConfigType": SimTypeInt(signed=False, label="IKEEXT_CERT_CONFIG_TYPE"), "Anonymous2": SimUnion({"Anonymous1": SimStruct({"outboundRootArraySize": SimTypeInt(signed=False, label="UInt32"), "outboundRootCriteria": SimTypePointer(SimStruct({"certData": SimStruct({"size": SimTypeInt(signed=False, label="UInt32"), "data": SimTypePointer(SimTypeChar(label="Byte"), offset=0)}, name="FWP_BYTE_BLOB", pack=False, align=None), "certHash": SimStruct({"size": SimTypeInt(signed=False, label="UInt32"), "data": SimTypePointer(SimTypeChar(label="Byte"), offset=0)}, name="FWP_BYTE_BLOB", pack=False, align=None), "eku": SimTypePointer(SimStruct({"numEku": SimTypeInt(signed=False, label="UInt32"), "eku": SimTypePointer(SimTypePointer(SimTypeChar(label="Byte"), offset=0), offset=0)}, name="IKEEXT_CERT_EKUS0", pack=False, align=None), offset=0), "name": SimTypePointer(SimStruct({"nameType": SimTypeInt(signed=False, label="IKEEXT_CERT_CRITERIA_NAME_TYPE"), "certName": SimTypePointer(SimTypeChar(label="Char"), offset=0)}, name="IKEEXT_CERT_NAME0", pack=False, align=None), offset=0), "flags": SimTypeInt(signed=False, label="UInt32")}, name="IKEEXT_CERTIFICATE_CRITERIA0", pack=False, align=None), offset=0)}, name="_Anonymous1_e__Struct", pack=False, align=None), "Anonymous2": SimStruct({"outboundEnterpriseStoreArraySize": SimTypeInt(signed=False, label="UInt32"), "outboundEnterpriseStoreCriteria": SimTypePointer(SimStruct({"certData": SimStruct({"size": SimTypeInt(signed=False, label="UInt32"), "data": SimTypePointer(SimTypeChar(label="Byte"), offset=0)}, name="FWP_BYTE_BLOB", pack=False, align=None), "certHash": SimStruct({"size": SimTypeInt(signed=False, label="UInt32"), "data": SimTypePointer(SimTypeChar(label="Byte"), offset=0)}, name="FWP_BYTE_BLOB", pack=False, align=None), "eku": SimTypePointer(SimStruct({"numEku": SimTypeInt(signed=False, label="UInt32"), "eku": SimTypePointer(SimTypePointer(SimTypeChar(label="Byte"), offset=0), offset=0)}, name="IKEEXT_CERT_EKUS0", pack=False, align=None), offset=0), "name": SimTypePointer(SimStruct({"nameType": SimTypeInt(signed=False, label="IKEEXT_CERT_CRITERIA_NAME_TYPE"), "certName": SimTypePointer(SimTypeChar(label="Char"), offset=0)}, name="IKEEXT_CERT_NAME0", pack=False, align=None), offset=0), "flags": SimTypeInt(signed=False, label="UInt32")}, name="IKEEXT_CERTIFICATE_CRITERIA0", pack=False, align=None), offset=0)}, name="_Anonymous2_e__Struct", pack=False, align=None), "Anonymous3": SimStruct({"outboundRootStoreArraySize": SimTypeInt(signed=False, label="UInt32"), "outboundTrustedRootStoreCriteria": SimTypePointer(SimStruct({"certData": SimStruct({"size": SimTypeInt(signed=False, label="UInt32"), "data": SimTypePointer(SimTypeChar(label="Byte"), offset=0)}, name="FWP_BYTE_BLOB", pack=False, align=None), "certHash": SimStruct({"size": SimTypeInt(signed=False, label="UInt32"), "data": SimTypePointer(SimTypeChar(label="Byte"), offset=0)}, name="FWP_BYTE_BLOB", pack=False, align=None), "eku": SimTypePointer(SimStruct({"numEku": SimTypeInt(signed=False, label="UInt32"), "eku": SimTypePointer(SimTypePointer(SimTypeChar(label="Byte"), offset=0), offset=0)}, name="IKEEXT_CERT_EKUS0", pack=False, align=None), offset=0), "name": SimTypePointer(SimStruct({"nameType": SimTypeInt(signed=False, label="IKEEXT_CERT_CRITERIA_NAME_TYPE"), "certName": SimTypePointer(SimTypeChar(label="Char"), offset=0)}, name="IKEEXT_CERT_NAME0", pack=False, align=None), offset=0), "flags": SimTypeInt(signed=False, label="UInt32")}, name="IKEEXT_CERTIFICATE_CRITERIA0", pack=False, align=None), offset=0)}, name="_Anonymous3_e__Struct", pack=False, align=None)}, name="<anon>", label="None"), "flags": SimTypeInt(signed=False, label="IKEEXT_CERT_AUTH"), "localCertLocationUrl": SimStruct({"size": SimTypeInt(signed=False, label="UInt32"), "data": SimTypePointer(SimTypeChar(label="Byte"), offset=0)}, name="FWP_BYTE_BLOB", pack=False, align=None)}, name="IKEEXT_CERTIFICATE_AUTHENTICATION2", pack=False, align=None), "kerberosAuthentication": SimStruct({"flags": SimTypeInt(signed=False, label="IKEEXT_KERBEROS_AUTHENTICATION_FLAGS"), "proxyServer": SimTypePointer(SimTypeChar(label="Char"), offset=0)}, name="IKEEXT_KERBEROS_AUTHENTICATION1", pack=False, align=None), "reservedAuthentication": SimStruct({"flags": SimTypeInt(signed=False, label="IKEEXT_RESERVED_AUTHENTICATION_FLAGS")}, name="IKEEXT_RESERVED_AUTHENTICATION0", pack=False, align=None), "ntlmV2Authentication": SimStruct({"flags": SimTypeInt(signed=False, label="UInt32")}, name="IKEEXT_NTLM_V2_AUTHENTICATION0", pack=False, align=None), "sslAuthentication": SimStruct({"inboundConfigType": SimTypeInt(signed=False, label="IKEEXT_CERT_CONFIG_TYPE"), "Anonymous1": SimUnion({"Anonymous1": SimStruct({"inboundRootArraySize": SimTypeInt(signed=False, label="UInt32"), "inboundRootCriteria": SimTypePointer(SimStruct({"certData": SimStruct({"size": SimTypeInt(signed=False, label="UInt32"), "data": SimTypePointer(SimTypeChar(label="Byte"), offset=0)}, name="FWP_BYTE_BLOB", pack=False, align=None), "certHash": SimStruct({"size": SimTypeInt(signed=False, label="UInt32"), "data": SimTypePointer(SimTypeChar(label="Byte"), offset=0)}, name="FWP_BYTE_BLOB", pack=False, align=None), "eku": SimTypePointer(SimStruct({"numEku": SimTypeInt(signed=False, label="UInt32"), "eku": SimTypePointer(SimTypePointer(SimTypeChar(label="Byte"), offset=0), offset=0)}, name="IKEEXT_CERT_EKUS0", pack=False, align=None), offset=0), "name": SimTypePointer(SimStruct({"nameType": SimTypeInt(signed=False, label="IKEEXT_CERT_CRITERIA_NAME_TYPE"), "certName": SimTypePointer(SimTypeChar(label="Char"), offset=0)}, name="IKEEXT_CERT_NAME0", pack=False, align=None), offset=0), "flags": SimTypeInt(signed=False, label="UInt32")}, name="IKEEXT_CERTIFICATE_CRITERIA0", pack=False, align=None), offset=0)}, name="_Anonymous1_e__Struct", pack=False, align=None), "Anonymous2": SimStruct({"inboundEnterpriseStoreArraySize": SimTypeInt(signed=False, label="UInt32"), "inboundEnterpriseStoreCriteria": SimTypePointer(SimStruct({"certData": SimStruct({"size": SimTypeInt(signed=False, label="UInt32"), "data": SimTypePointer(SimTypeChar(label="Byte"), offset=0)}, name="FWP_BYTE_BLOB", pack=False, align=None), "certHash": SimStruct({"size": SimTypeInt(signed=False, label="UInt32"), "data": SimTypePointer(SimTypeChar(label="Byte"), offset=0)}, name="FWP_BYTE_BLOB", pack=False, align=None), "eku": SimTypePointer(SimStruct({"numEku": SimTypeInt(signed=False, label="UInt32"), "eku": SimTypePointer(SimTypePointer(SimTypeChar(label="Byte"), offset=0), offset=0)}, name="IKEEXT_CERT_EKUS0", pack=False, align=None), offset=0), "name": SimTypePointer(SimStruct({"nameType": SimTypeInt(signed=False, label="IKEEXT_CERT_CRITERIA_NAME_TYPE"), "certName": SimTypePointer(SimTypeChar(label="Char"), offset=0)}, name="IKEEXT_CERT_NAME0", pack=False, align=None), offset=0), "flags": SimTypeInt(signed=False, label="UInt32")}, name="IKEEXT_CERTIFICATE_CRITERIA0", pack=False, align=None), offset=0)}, name="_Anonymous2_e__Struct", pack=False, align=None), "Anonymous3": SimStruct({"inboundRootStoreArraySize": SimTypeInt(signed=False, label="UInt32"), "inboundTrustedRootStoreCriteria": SimTypePointer(SimStruct({"certData": SimStruct({"size": SimTypeInt(signed=False, label="UInt32"), "data": SimTypePointer(SimTypeChar(label="Byte"), offset=0)}, name="FWP_BYTE_BLOB", pack=False, align=None), "certHash": SimStruct({"size": SimTypeInt(signed=False, label="UInt32"), "data": SimTypePointer(SimTypeChar(label="Byte"), offset=0)}, name="FWP_BYTE_BLOB", pack=False, align=None), "eku": SimTypePointer(SimStruct({"numEku": SimTypeInt(signed=False, label="UInt32"), "eku": SimTypePointer(SimTypePointer(SimTypeChar(label="Byte"), offset=0), offset=0)}, name="IKEEXT_CERT_EKUS0", pack=False, align=None), offset=0), "name": SimTypePointer(SimStruct({"nameType": SimTypeInt(signed=False, label="IKEEXT_CERT_CRITERIA_NAME_TYPE"), "certName": SimTypePointer(SimTypeChar(label="Char"), offset=0)}, name="IKEEXT_CERT_NAME0", pack=False, align=None), offset=0), "flags": SimTypeInt(signed=False, label="UInt32")}, name="IKEEXT_CERTIFICATE_CRITERIA0", pack=False, align=None), offset=0)}, name="_Anonymous3_e__Struct", pack=False, align=None)}, name="<anon>", label="None"), "outboundConfigType": SimTypeInt(signed=False, label="IKEEXT_CERT_CONFIG_TYPE"), "Anonymous2": SimUnion({"Anonymous1": SimStruct({"outboundRootArraySize": SimTypeInt(signed=False, label="UInt32"), "outboundRootCriteria": SimTypePointer(SimStruct({"certData": SimStruct({"size": SimTypeInt(signed=False, label="UInt32"), "data": SimTypePointer(SimTypeChar(label="Byte"), offset=0)}, name="FWP_BYTE_BLOB", pack=False, align=None), "certHash": SimStruct({"size": SimTypeInt(signed=False, label="UInt32"), "data": SimTypePointer(SimTypeChar(label="Byte"), offset=0)}, name="FWP_BYTE_BLOB", pack=False, align=None), "eku": SimTypePointer(SimStruct({"numEku": SimTypeInt(signed=False, label="UInt32"), "eku": SimTypePointer(SimTypePointer(SimTypeChar(label="Byte"), offset=0), offset=0)}, name="IKEEXT_CERT_EKUS0", pack=False, align=None), offset=0), "name": SimTypePointer(SimStruct({"nameType": SimTypeInt(signed=False, label="IKEEXT_CERT_CRITERIA_NAME_TYPE"), "certName": SimTypePointer(SimTypeChar(label="Char"), offset=0)}, name="IKEEXT_CERT_NAME0", pack=False, align=None), offset=0), "flags": SimTypeInt(signed=False, label="UInt32")}, name="IKEEXT_CERTIFICATE_CRITERIA0", pack=False, align=None), offset=0)}, name="_Anonymous1_e__Struct", pack=False, align=None), "Anonymous2": SimStruct({"outboundEnterpriseStoreArraySize": SimTypeInt(signed=False, label="UInt32"), "outboundEnterpriseStoreCriteria": SimTypePointer(SimStruct({"certData": SimStruct({"size": SimTypeInt(signed=False, label="UInt32"), "data": SimTypePointer(SimTypeChar(label="Byte"), offset=0)}, name="FWP_BYTE_BLOB", pack=False, align=None), "certHash": SimStruct({"size": SimTypeInt(signed=False, label="UInt32"), "data": SimTypePointer(SimTypeChar(label="Byte"), offset=0)}, name="FWP_BYTE_BLOB", pack=False, align=None), "eku": SimTypePointer(SimStruct({"numEku": SimTypeInt(signed=False, label="UInt32"), "eku": SimTypePointer(SimTypePointer(SimTypeChar(label="Byte"), offset=0), offset=0)}, name="IKEEXT_CERT_EKUS0", pack=False, align=None), offset=0), "name": SimTypePointer(SimStruct({"nameType": SimTypeInt(signed=False, label="IKEEXT_CERT_CRITERIA_NAME_TYPE"), "certName": SimTypePointer(SimTypeChar(label="Char"), offset=0)}, name="IKEEXT_CERT_NAME0", pack=False, align=None), offset=0), "flags": SimTypeInt(signed=False, label="UInt32")}, name="IKEEXT_CERTIFICATE_CRITERIA0", pack=False, align=None), offset=0)}, name="_Anonymous2_e__Struct", pack=False, align=None), "Anonymous3": SimStruct({"outboundRootStoreArraySize": SimTypeInt(signed=False, label="UInt32"), "outboundTrustedRootStoreCriteria": SimTypePointer(SimStruct({"certData": SimStruct({"size": SimTypeInt(signed=False, label="UInt32"), "data": SimTypePointer(SimTypeChar(label="Byte"), offset=0)}, name="FWP_BYTE_BLOB", pack=False, align=None), "certHash": SimStruct({"size": SimTypeInt(signed=False, label="UInt32"), "data": SimTypePointer(SimTypeChar(label="Byte"), offset=0)}, name="FWP_BYTE_BLOB", pack=False, align=None), "eku": SimTypePointer(SimStruct({"numEku": SimTypeInt(signed=False, label="UInt32"), "eku": SimTypePointer(SimTypePointer(SimTypeChar(label="Byte"), offset=0), offset=0)}, name="IKEEXT_CERT_EKUS0", pack=False, align=None), offset=0), "name": SimTypePointer(SimStruct({"nameType": SimTypeInt(signed=False, label="IKEEXT_CERT_CRITERIA_NAME_TYPE"), "certName": SimTypePointer(SimTypeChar(label="Char"), offset=0)}, name="IKEEXT_CERT_NAME0", pack=False, align=None), offset=0), "flags": SimTypeInt(signed=False, label="UInt32")}, name="IKEEXT_CERTIFICATE_CRITERIA0", pack=False, align=None), offset=0)}, name="_Anonymous3_e__Struct", pack=False, align=None)}, name="<anon>", label="None"), "flags": SimTypeInt(signed=False, label="IKEEXT_CERT_AUTH"), "localCertLocationUrl": SimStruct({"size": SimTypeInt(signed=False, label="UInt32"), "data": SimTypePointer(SimTypeChar(label="Byte"), offset=0)}, name="FWP_BYTE_BLOB", pack=False, align=None)}, name="IKEEXT_CERTIFICATE_AUTHENTICATION2", pack=False, align=None), "cgaAuthentication": SimStruct({"keyContainerName": SimTypePointer(SimTypeChar(label="Char"), offset=0), "cspName": SimTypePointer(SimTypeChar(label="Char"), offset=0), "cspType": SimTypeInt(signed=False, label="UInt32"), "cgaModifier": SimStruct({"byteArray16": SimTypeFixedSizeArray(SimTypeChar(label="Byte"), 16)}, name="FWP_BYTE_ARRAY16", pack=False, align=None), "cgaCollisionCount": SimTypeChar(label="Byte")}, name="IKEEXT_IPV6_CGA_AUTHENTICATION0", pack=False, align=None), "eapAuthentication": SimStruct({"flags": SimTypeInt(signed=False, label="IKEEXT_EAP_AUTHENTICATION_FLAGS")}, name="IKEEXT_EAP_AUTHENTICATION0", pack=False, align=None)}, name="<anon>", label="None")}, name="IKEEXT_AUTHENTICATION_METHOD2", pack=False, align=None), offset=0), "initiatorImpersonationType": SimTypeInt(signed=False, label="IKEEXT_AUTHENTICATION_IMPERSONATION_TYPE"), "numIkeProposals": SimTypeInt(signed=False, label="UInt32"), "ikeProposals": SimTypePointer(SimStruct({"cipherAlgorithm": SimStruct({"algoIdentifier": SimTypeInt(signed=False, label="IKEEXT_CIPHER_TYPE"), "keyLen": SimTypeInt(signed=False, label="UInt32"), "rounds": SimTypeInt(signed=False, label="UInt32")}, name="IKEEXT_CIPHER_ALGORITHM0", pack=False, align=None), "integrityAlgorithm": SimStruct({"algoIdentifier": SimTypeInt(signed=False, label="IKEEXT_INTEGRITY_TYPE")}, name="IKEEXT_INTEGRITY_ALGORITHM0", pack=False, align=None), "maxLifetimeSeconds": SimTypeInt(signed=False, label="UInt32"), "dhGroup": SimTypeInt(signed=False, label="IKEEXT_DH_GROUP"), "quickModeLimit": SimTypeInt(signed=False, label="UInt32")}, name="IKEEXT_PROPOSAL0", pack=False, align=None), offset=0), "flags": SimTypeInt(signed=False, label="IKEEXT_POLICY_FLAG"), "maxDynamicFilters": SimTypeInt(signed=False, label="UInt32"), "retransmitDurationSecs": SimTypeInt(signed=False, label="UInt32")}, name="IKEEXT_POLICY2", pack=False, align=None), offset=0), "dataBuffer": SimTypePointer(SimStruct({"size": SimTypeInt(signed=False, label="UInt32"), "data": SimTypePointer(SimTypeChar(label="Byte"), offset=0)}, name="FWP_BYTE_BLOB", pack=False, align=None), offset=0), "classifyOptions": SimTypePointer(SimStruct({"numOptions": SimTypeInt(signed=False, label="UInt32"), "options": SimTypePointer(SimStruct({"type": SimTypeInt(signed=False, label="FWP_CLASSIFY_OPTION_TYPE"), "value": SimStruct({"type": SimTypeInt(signed=False, label="FWP_DATA_TYPE"), "Anonymous": SimUnion({"uint8": SimTypeChar(label="Byte"), "uint16": SimTypeShort(signed=False, label="UInt16"), "uint32": SimTypeInt(signed=False, label="UInt32"), "uint64": SimTypePointer(SimTypeLongLong(signed=False, label="UInt64"), offset=0), "int8": SimTypeChar(label="SByte"), "int16": SimTypeShort(signed=True, label="Int16"), "int32": SimTypeInt(signed=True, label="Int32"), "int64": SimTypePointer(SimTypeLongLong(signed=True, label="Int64"), offset=0), "float32": SimTypeFloat(size=32), "double64": SimTypePointer(SimTypeFloat(size=64), offset=0), "byteArray16": SimTypePointer(SimStruct({"byteArray16": SimTypeFixedSizeArray(SimTypeChar(label="Byte"), 16)}, name="FWP_BYTE_ARRAY16", pack=False, align=None), offset=0), "byteBlob": SimTypePointer(SimStruct({"size": SimTypeInt(signed=False, label="UInt32"), "data": SimTypePointer(SimTypeChar(label="Byte"), offset=0)}, name="FWP_BYTE_BLOB", pack=False, align=None), offset=0), "sid": SimTypePointer(SimTypeBottom(label="SID"), offset=0), "sd": SimTypePointer(SimStruct({"size": SimTypeInt(signed=False, label="UInt32"), "data": SimTypePointer(SimTypeChar(label="Byte"), offset=0)}, name="FWP_BYTE_BLOB", pack=False, align=None), offset=0), "tokenInformation": SimTypePointer(SimStruct({"sidCount": SimTypeInt(signed=False, label="UInt32"), "sids": SimTypePointer(SimTypeBottom(label="SID_AND_ATTRIBUTES"), offset=0), "restrictedSidCount": SimTypeInt(signed=False, label="UInt32"), "restrictedSids": SimTypePointer(SimTypeBottom(label="SID_AND_ATTRIBUTES"), offset=0)}, name="FWP_TOKEN_INFORMATION", pack=False, align=None), offset=0), "tokenAccessInformation": SimTypePointer(SimStruct({"size": SimTypeInt(signed=False, label="UInt32"), "data": SimTypePointer(SimTypeChar(label="Byte"), offset=0)}, name="FWP_BYTE_BLOB", pack=False, align=None), offset=0), "unicodeString": SimTypePointer(SimTypeChar(label="Char"), offset=0), "byteArray6": SimTypePointer(SimStruct({"byteArray6": SimTypeFixedSizeArray(SimTypeChar(label="Byte"), 6)}, name="FWP_BYTE_ARRAY6", pack=False, align=None), offset=0), "bitmapArray64": SimTypePointer(SimStruct({"bitmapArray64": SimTypeFixedSizeArray(SimTypeChar(label="Byte"), 8)}, name="FWP_BITMAP_ARRAY64_", pack=False, align=None), offset=0)}, name="<anon>", label="None")}, name="FWP_VALUE0", pack=False, align=None)}, name="FWPM_CLASSIFY_OPTION0", pack=False, align=None), offset=0)}, name="FWPM_CLASSIFY_OPTIONS0", pack=False, align=None), offset=0), "ikeV2QmTunnelPolicy": SimTypePointer(SimStruct({"flags": SimTypeInt(signed=False, label="IPSEC_POLICY_FLAG"), "numIpsecProposals": SimTypeInt(signed=False, label="UInt32"), "ipsecProposals": SimTypePointer(SimStruct({"lifetime": SimStruct({"lifetimeSeconds": SimTypeInt(signed=False, label="UInt32"), "lifetimeKilobytes": SimTypeInt(signed=False, label="UInt32"), "lifetimePackets": SimTypeInt(signed=False, label="UInt32")}, name="IPSEC_SA_LIFETIME0", pack=False, align=None), "numSaTransforms": SimTypeInt(signed=False, label="UInt32"), "saTransforms": SimTypePointer(SimStruct({"ipsecTransformType": SimTypeInt(signed=False, label="IPSEC_TRANSFORM_TYPE"), "Anonymous": SimUnion({"ahTransform": SimTypePointer(SimStruct({"authTransformId": SimStruct({"authType": SimTypeInt(signed=False, label="IPSEC_AUTH_TYPE"), "authConfig": SimTypeChar(label="Byte")}, name="IPSEC_AUTH_TRANSFORM_ID0", pack=False, align=None), "cryptoModuleId": SimTypePointer(SimTypeBottom(label="Guid"), offset=0)}, name="IPSEC_AUTH_TRANSFORM0", pack=False, align=None), offset=0), "espAuthTransform": SimTypePointer(SimStruct({"authTransformId": SimStruct({"authType": SimTypeInt(signed=False, label="IPSEC_AUTH_TYPE"), "authConfig": SimTypeChar(label="Byte")}, name="IPSEC_AUTH_TRANSFORM_ID0", pack=False, align=None), "cryptoModuleId": SimTypePointer(SimTypeBottom(label="Guid"), offset=0)}, name="IPSEC_AUTH_TRANSFORM0", pack=False, align=None), offset=0), "espCipherTransform": SimTypePointer(SimStruct({"cipherTransformId": SimStruct({"cipherType": SimTypeInt(signed=False, label="IPSEC_CIPHER_TYPE"), "cipherConfig": SimTypeChar(label="Byte")}, name="IPSEC_CIPHER_TRANSFORM_ID0", pack=False, align=None), "cryptoModuleId": SimTypePointer(SimTypeBottom(label="Guid"), offset=0)}, name="IPSEC_CIPHER_TRANSFORM0", pack=False, align=None), offset=0), "espAuthAndCipherTransform": SimTypePointer(SimStruct({"authTransform": SimStruct({"authTransformId": SimStruct({"authType": SimTypeInt(signed=False, label="IPSEC_AUTH_TYPE"), "authConfig": SimTypeChar(label="Byte")}, name="IPSEC_AUTH_TRANSFORM_ID0", pack=False, align=None), "cryptoModuleId": SimTypePointer(SimTypeBottom(label="Guid"), offset=0)}, name="IPSEC_AUTH_TRANSFORM0", pack=False, align=None), "cipherTransform": SimStruct({"cipherTransformId": SimStruct({"cipherType": SimTypeInt(signed=False, label="IPSEC_CIPHER_TYPE"), "cipherConfig": SimTypeChar(label="Byte")}, name="IPSEC_CIPHER_TRANSFORM_ID0", pack=False, align=None), "cryptoModuleId": SimTypePointer(SimTypeBottom(label="Guid"), offset=0)}, name="IPSEC_CIPHER_TRANSFORM0", pack=False, align=None)}, name="IPSEC_AUTH_AND_CIPHER_TRANSFORM0", pack=False, align=None), offset=0), "espAuthFwTransform": SimTypePointer(SimStruct({"authTransformId": SimStruct({"authType": SimTypeInt(signed=False, label="IPSEC_AUTH_TYPE"), "authConfig": SimTypeChar(label="Byte")}, name="IPSEC_AUTH_TRANSFORM_ID0", pack=False, align=None), "cryptoModuleId": SimTypePointer(SimTypeBottom(label="Guid"), offset=0)}, name="IPSEC_AUTH_TRANSFORM0", pack=False, align=None), offset=0)}, name="<anon>", label="None")}, name="IPSEC_SA_TRANSFORM0", pack=False, align=None), offset=0), "pfsGroup": SimTypeInt(signed=False, label="IPSEC_PFS_GROUP")}, name="IPSEC_PROPOSAL0", pack=False, align=None), offset=0), "tunnelEndpoints": SimStruct({"ipVersion": SimTypeInt(signed=False, label="FWP_IP_VERSION"), "Anonymous1": SimUnion({"localV4Address": SimTypeInt(signed=False, label="UInt32"), "localV6Address": SimTypeFixedSizeArray(SimTypeChar(label="Byte"), 16)}, name="<anon>", label="None"), "Anonymous2": SimUnion({"remoteV4Address": SimTypeInt(signed=False, label="UInt32"), "remoteV6Address": SimTypeFixedSizeArray(SimTypeChar(label="Byte"), 16)}, name="<anon>", label="None"), "localIfLuid": SimTypeLongLong(signed=False, label="UInt64"), "remoteFqdn": SimTypePointer(SimTypeChar(label="Char"), offset=0), "numAddresses": SimTypeInt(signed=False, label="UInt32"), "remoteAddresses": SimTypePointer(SimStruct({"ipVersion": SimTypeInt(signed=False, label="FWP_IP_VERSION"), "Anonymous": SimUnion({"v4Address": SimTypeInt(signed=False, label="UInt32"), "v6Address": SimTypeFixedSizeArray(SimTypeChar(label="Byte"), 16)}, name="<anon>", label="None")}, name="IPSEC_TUNNEL_ENDPOINT0", pack=False, align=None), offset=0)}, name="IPSEC_TUNNEL_ENDPOINTS2", pack=False, align=None), "saIdleTimeout": SimStruct({"idleTimeoutSeconds": SimTypeInt(signed=False, label="UInt32"), "idleTimeoutSecondsFailOver": SimTypeInt(signed=False, label="UInt32")}, name="IPSEC_SA_IDLE_TIMEOUT0", pack=False, align=None), "emPolicy": SimTypePointer(SimStruct({"numAuthenticationMethods": SimTypeInt(signed=False, label="UInt32"), "authenticationMethods": SimTypePointer(SimStruct({"authenticationMethodType": SimTypeInt(signed=False, label="IKEEXT_AUTHENTICATION_METHOD_TYPE"), "Anonymous": SimUnion({"presharedKeyAuthentication": SimStruct({"presharedKey": SimStruct({"size": SimTypeInt(signed=False, label="UInt32"), "data": SimTypePointer(SimTypeChar(label="Byte"), offset=0)}, name="FWP_BYTE_BLOB", pack=False, align=None), "flags": SimTypeInt(signed=False, label="IKEEXT_PRESHARED_KEY_AUTHENTICATION_FLAGS")}, name="IKEEXT_PRESHARED_KEY_AUTHENTICATION1", pack=False, align=None), "certificateAuthentication": SimStruct({"inboundConfigType": SimTypeInt(signed=False, label="IKEEXT_CERT_CONFIG_TYPE"), "Anonymous1": SimUnion({"Anonymous1": SimStruct({"inboundRootArraySize": SimTypeInt(signed=False, label="UInt32"), "inboundRootCriteria": SimTypePointer(SimStruct({"certData": SimStruct({"size": SimTypeInt(signed=False, label="UInt32"), "data": SimTypePointer(SimTypeChar(label="Byte"), offset=0)}, name="FWP_BYTE_BLOB", pack=False, align=None), "certHash": SimStruct({"size": SimTypeInt(signed=False, label="UInt32"), "data": SimTypePointer(SimTypeChar(label="Byte"), offset=0)}, name="FWP_BYTE_BLOB", pack=False, align=None), "eku": SimTypePointer(SimStruct({"numEku": SimTypeInt(signed=False, label="UInt32"), "eku": SimTypePointer(SimTypePointer(SimTypeChar(label="Byte"), offset=0), offset=0)}, name="IKEEXT_CERT_EKUS0", pack=False, align=None), offset=0), "name": SimTypePointer(SimStruct({"nameType": SimTypeInt(signed=False, label="IKEEXT_CERT_CRITERIA_NAME_TYPE"), "certName": SimTypePointer(SimTypeChar(label="Char"), offset=0)}, name="IKEEXT_CERT_NAME0", pack=False, align=None), offset=0), "flags": SimTypeInt(signed=False, label="UInt32")}, name="IKEEXT_CERTIFICATE_CRITERIA0", pack=False, align=None), offset=0)}, name="_Anonymous1_e__Struct", pack=False, align=None), "Anonymous2": SimStruct({"inboundEnterpriseStoreArraySize": SimTypeInt(signed=False, label="UInt32"), "inboundEnterpriseStoreCriteria": SimTypePointer(SimStruct({"certData": SimStruct({"size": SimTypeInt(signed=False, label="UInt32"), "data": SimTypePointer(SimTypeChar(label="Byte"), offset=0)}, name="FWP_BYTE_BLOB", pack=False, align=None), "certHash": SimStruct({"size": SimTypeInt(signed=False, label="UInt32"), "data": SimTypePointer(SimTypeChar(label="Byte"), offset=0)}, name="FWP_BYTE_BLOB", pack=False, align=None), "eku": SimTypePointer(SimStruct({"numEku": SimTypeInt(signed=False, label="UInt32"), "eku": SimTypePointer(SimTypePointer(SimTypeChar(label="Byte"), offset=0), offset=0)}, name="IKEEXT_CERT_EKUS0", pack=False, align=None), offset=0), "name": SimTypePointer(SimStruct({"nameType": SimTypeInt(signed=False, label="IKEEXT_CERT_CRITERIA_NAME_TYPE"), "certName": SimTypePointer(SimTypeChar(label="Char"), offset=0)}, name="IKEEXT_CERT_NAME0", pack=False, align=None), offset=0), "flags": SimTypeInt(signed=False, label="UInt32")}, name="IKEEXT_CERTIFICATE_CRITERIA0", pack=False, align=None), offset=0)}, name="_Anonymous2_e__Struct", pack=False, align=None), "Anonymous3": SimStruct({"inboundRootStoreArraySize": SimTypeInt(signed=False, label="UInt32"), "inboundTrustedRootStoreCriteria": SimTypePointer(SimStruct({"certData": SimStruct({"size": SimTypeInt(signed=False, label="UInt32"), "data": SimTypePointer(SimTypeChar(label="Byte"), offset=0)}, name="FWP_BYTE_BLOB", pack=False, align=None), "certHash": SimStruct({"size": SimTypeInt(signed=False, label="UInt32"), "data": SimTypePointer(SimTypeChar(label="Byte"), offset=0)}, name="FWP_BYTE_BLOB", pack=False, align=None), "eku": SimTypePointer(SimStruct({"numEku": SimTypeInt(signed=False, label="UInt32"), "eku": SimTypePointer(SimTypePointer(SimTypeChar(label="Byte"), offset=0), offset=0)}, name="IKEEXT_CERT_EKUS0", pack=False, align=None), offset=0), "name": SimTypePointer(SimStruct({"nameType": SimTypeInt(signed=False, label="IKEEXT_CERT_CRITERIA_NAME_TYPE"), "certName": SimTypePointer(SimTypeChar(label="Char"), offset=0)}, name="IKEEXT_CERT_NAME0", pack=False, align=None), offset=0), "flags": SimTypeInt(signed=False, label="UInt32")}, name="IKEEXT_CERTIFICATE_CRITERIA0", pack=False, align=None), offset=0)}, name="_Anonymous3_e__Struct", pack=False, align=None)}, name="<anon>", label="None"), "outboundConfigType": SimTypeInt(signed=False, label="IKEEXT_CERT_CONFIG_TYPE"), "Anonymous2": SimUnion({"Anonymous1": SimStruct({"outboundRootArraySize": SimTypeInt(signed=False, label="UInt32"), "outboundRootCriteria": SimTypePointer(SimStruct({"certData": SimStruct({"size": SimTypeInt(signed=False, label="UInt32"), "data": SimTypePointer(SimTypeChar(label="Byte"), offset=0)}, name="FWP_BYTE_BLOB", pack=False, align=None), "certHash": SimStruct({"size": SimTypeInt(signed=False, label="UInt32"), "data": SimTypePointer(SimTypeChar(label="Byte"), offset=0)}, name="FWP_BYTE_BLOB", pack=False, align=None), "eku": SimTypePointer(SimStruct({"numEku": SimTypeInt(signed=False, label="UInt32"), "eku": SimTypePointer(SimTypePointer(SimTypeChar(label="Byte"), offset=0), offset=0)}, name="IKEEXT_CERT_EKUS0", pack=False, align=None), offset=0), "name": SimTypePointer(SimStruct({"nameType": SimTypeInt(signed=False, label="IKEEXT_CERT_CRITERIA_NAME_TYPE"), "certName": SimTypePointer(SimTypeChar(label="Char"), offset=0)}, name="IKEEXT_CERT_NAME0", pack=False, align=None), offset=0), "flags": SimTypeInt(signed=False, label="UInt32")}, name="IKEEXT_CERTIFICATE_CRITERIA0", pack=False, align=None), offset=0)}, name="_Anonymous1_e__Struct", pack=False, align=None), "Anonymous2": SimStruct({"outboundEnterpriseStoreArraySize": SimTypeInt(signed=False, label="UInt32"), "outboundEnterpriseStoreCriteria": SimTypePointer(SimStruct({"certData": SimStruct({"size": SimTypeInt(signed=False, label="UInt32"), "data": SimTypePointer(SimTypeChar(label="Byte"), offset=0)}, name="FWP_BYTE_BLOB", pack=False, align=None), "certHash": SimStruct({"size": SimTypeInt(signed=False, label="UInt32"), "data": SimTypePointer(SimTypeChar(label="Byte"), offset=0)}, name="FWP_BYTE_BLOB", pack=False, align=None), "eku": SimTypePointer(SimStruct({"numEku": SimTypeInt(signed=False, label="UInt32"), "eku": SimTypePointer(SimTypePointer(SimTypeChar(label="Byte"), offset=0), offset=0)}, name="IKEEXT_CERT_EKUS0", pack=False, align=None), offset=0), "name": SimTypePointer(SimStruct({"nameType": SimTypeInt(signed=False, label="IKEEXT_CERT_CRITERIA_NAME_TYPE"), "certName": SimTypePointer(SimTypeChar(label="Char"), offset=0)}, name="IKEEXT_CERT_NAME0", pack=False, align=None), offset=0), "flags": SimTypeInt(signed=False, label="UInt32")}, name="IKEEXT_CERTIFICATE_CRITERIA0", pack=False, align=None), offset=0)}, name="_Anonymous2_e__Struct", pack=False, align=None), "Anonymous3": SimStruct({"outboundRootStoreArraySize": SimTypeInt(signed=False, label="UInt32"), "outboundTrustedRootStoreCriteria": SimTypePointer(SimStruct({"certData": SimStruct({"size": SimTypeInt(signed=False, label="UInt32"), "data": SimTypePointer(SimTypeChar(label="Byte"), offset=0)}, name="FWP_BYTE_BLOB", pack=False, align=None), "certHash": SimStruct({"size": SimTypeInt(signed=False, label="UInt32"), "data": SimTypePointer(SimTypeChar(label="Byte"), offset=0)}, name="FWP_BYTE_BLOB", pack=False, align=None), "eku": SimTypePointer(SimStruct({"numEku": SimTypeInt(signed=False, label="UInt32"), "eku": SimTypePointer(SimTypePointer(SimTypeChar(label="Byte"), offset=0), offset=0)}, name="IKEEXT_CERT_EKUS0", pack=False, align=None), offset=0), "name": SimTypePointer(SimStruct({"nameType": SimTypeInt(signed=False, label="IKEEXT_CERT_CRITERIA_NAME_TYPE"), "certName": SimTypePointer(SimTypeChar(label="Char"), offset=0)}, name="IKEEXT_CERT_NAME0", pack=False, align=None), offset=0), "flags": SimTypeInt(signed=False, label="UInt32")}, name="IKEEXT_CERTIFICATE_CRITERIA0", pack=False, align=None), offset=0)}, name="_Anonymous3_e__Struct", pack=False, align=None)}, name="<anon>", label="None"), "flags": SimTypeInt(signed=False, label="IKEEXT_CERT_AUTH"), "localCertLocationUrl": SimStruct({"size": SimTypeInt(signed=False, label="UInt32"), "data": SimTypePointer(SimTypeChar(label="Byte"), offset=0)}, name="FWP_BYTE_BLOB", pack=False, align=None)}, name="IKEEXT_CERTIFICATE_AUTHENTICATION2", pack=False, align=None), "kerberosAuthentication": SimStruct({"flags": SimTypeInt(signed=False, label="IKEEXT_KERBEROS_AUTHENTICATION_FLAGS"), "proxyServer": SimTypePointer(SimTypeChar(label="Char"), offset=0)}, name="IKEEXT_KERBEROS_AUTHENTICATION1", pack=False, align=None), "reservedAuthentication": SimStruct({"flags": SimTypeInt(signed=False, label="IKEEXT_RESERVED_AUTHENTICATION_FLAGS")}, name="IKEEXT_RESERVED_AUTHENTICATION0", pack=False, align=None), "ntlmV2Authentication": SimStruct({"flags": SimTypeInt(signed=False, label="UInt32")}, name="IKEEXT_NTLM_V2_AUTHENTICATION0", pack=False, align=None), "sslAuthentication": SimStruct({"inboundConfigType": SimTypeInt(signed=False, label="IKEEXT_CERT_CONFIG_TYPE"), "Anonymous1": SimUnion({"Anonymous1": SimStruct({"inboundRootArraySize": SimTypeInt(signed=False, label="UInt32"), "inboundRootCriteria": SimTypePointer(SimStruct({"certData": SimStruct({"size": SimTypeInt(signed=False, label="UInt32"), "data": SimTypePointer(SimTypeChar(label="Byte"), offset=0)}, name="FWP_BYTE_BLOB", pack=False, align=None), "certHash": SimStruct({"size": SimTypeInt(signed=False, label="UInt32"), "data": SimTypePointer(SimTypeChar(label="Byte"), offset=0)}, name="FWP_BYTE_BLOB", pack=False, align=None), "eku": SimTypePointer(SimStruct({"numEku": SimTypeInt(signed=False, label="UInt32"), "eku": SimTypePointer(SimTypePointer(SimTypeChar(label="Byte"), offset=0), offset=0)}, name="IKEEXT_CERT_EKUS0", pack=False, align=None), offset=0), "name": SimTypePointer(SimStruct({"nameType": SimTypeInt(signed=False, label="IKEEXT_CERT_CRITERIA_NAME_TYPE"), "certName": SimTypePointer(SimTypeChar(label="Char"), offset=0)}, name="IKEEXT_CERT_NAME0", pack=False, align=None), offset=0), "flags": SimTypeInt(signed=False, label="UInt32")}, name="IKEEXT_CERTIFICATE_CRITERIA0", pack=False, align=None), offset=0)}, name="_Anonymous1_e__Struct", pack=False, align=None), "Anonymous2": SimStruct({"inboundEnterpriseStoreArraySize": SimTypeInt(signed=False, label="UInt32"), "inboundEnterpriseStoreCriteria": SimTypePointer(SimStruct({"certData": SimStruct({"size": SimTypeInt(signed=False, label="UInt32"), "data": SimTypePointer(SimTypeChar(label="Byte"), offset=0)}, name="FWP_BYTE_BLOB", pack=False, align=None), "certHash": SimStruct({"size": SimTypeInt(signed=False, label="UInt32"), "data": SimTypePointer(SimTypeChar(label="Byte"), offset=0)}, name="FWP_BYTE_BLOB", pack=False, align=None), "eku": SimTypePointer(SimStruct({"numEku": SimTypeInt(signed=False, label="UInt32"), "eku": SimTypePointer(SimTypePointer(SimTypeChar(label="Byte"), offset=0), offset=0)}, name="IKEEXT_CERT_EKUS0", pack=False, align=None), offset=0), "name": SimTypePointer(SimStruct({"nameType": SimTypeInt(signed=False, label="IKEEXT_CERT_CRITERIA_NAME_TYPE"), "certName": SimTypePointer(SimTypeChar(label="Char"), offset=0)}, name="IKEEXT_CERT_NAME0", pack=False, align=None), offset=0), "flags": SimTypeInt(signed=False, label="UInt32")}, name="IKEEXT_CERTIFICATE_CRITERIA0", pack=False, align=None), offset=0)}, name="_Anonymous2_e__Struct", pack=False, align=None), "Anonymous3": SimStruct({"inboundRootStoreArraySize": SimTypeInt(signed=False, label="UInt32"), "inboundTrustedRootStoreCriteria": SimTypePointer(SimStruct({"certData": SimStruct({"size": SimTypeInt(signed=False, label="UInt32"), "data": SimTypePointer(SimTypeChar(label="Byte"), offset=0)}, name="FWP_BYTE_BLOB", pack=False, align=None), "certHash": SimStruct({"size": SimTypeInt(signed=False, label="UInt32"), "data": SimTypePointer(SimTypeChar(label="Byte"), offset=0)}, name="FWP_BYTE_BLOB", pack=False, align=None), "eku": SimTypePointer(SimStruct({"numEku": SimTypeInt(signed=False, label="UInt32"), "eku": SimTypePointer(SimTypePointer(SimTypeChar(label="Byte"), offset=0), offset=0)}, name="IKEEXT_CERT_EKUS0", pack=False, align=None), offset=0), "name": SimTypePointer(SimStruct({"nameType": SimTypeInt(signed=False, label="IKEEXT_CERT_CRITERIA_NAME_TYPE"), "certName": SimTypePointer(SimTypeChar(label="Char"), offset=0)}, name="IKEEXT_CERT_NAME0", pack=False, align=None), offset=0), "flags": SimTypeInt(signed=False, label="UInt32")}, name="IKEEXT_CERTIFICATE_CRITERIA0", pack=False, align=None), offset=0)}, name="_Anonymous3_e__Struct", pack=False, align=None)}, name="<anon>", label="None"), "outboundConfigType": SimTypeInt(signed=False, label="IKEEXT_CERT_CONFIG_TYPE"), "Anonymous2": SimUnion({"Anonymous1": SimStruct({"outboundRootArraySize": SimTypeInt(signed=False, label="UInt32"), "outboundRootCriteria": SimTypePointer(SimStruct({"certData": SimStruct({"size": SimTypeInt(signed=False, label="UInt32"), "data": SimTypePointer(SimTypeChar(label="Byte"), offset=0)}, name="FWP_BYTE_BLOB", pack=False, align=None), "certHash": SimStruct({"size": SimTypeInt(signed=False, label="UInt32"), "data": SimTypePointer(SimTypeChar(label="Byte"), offset=0)}, name="FWP_BYTE_BLOB", pack=False, align=None), "eku": SimTypePointer(SimStruct({"numEku": SimTypeInt(signed=False, label="UInt32"), "eku": SimTypePointer(SimTypePointer(SimTypeChar(label="Byte"), offset=0), offset=0)}, name="IKEEXT_CERT_EKUS0", pack=False, align=None), offset=0), "name": SimTypePointer(SimStruct({"nameType": SimTypeInt(signed=False, label="IKEEXT_CERT_CRITERIA_NAME_TYPE"), "certName": SimTypePointer(SimTypeChar(label="Char"), offset=0)}, name="IKEEXT_CERT_NAME0", pack=False, align=None), offset=0), "flags": SimTypeInt(signed=False, label="UInt32")}, name="IKEEXT_CERTIFICATE_CRITERIA0", pack=False, align=None), offset=0)}, name="_Anonymous1_e__Struct", pack=False, align=None), "Anonymous2": SimStruct({"outboundEnterpriseStoreArraySize": SimTypeInt(signed=False, label="UInt32"), "outboundEnterpriseStoreCriteria": SimTypePointer(SimStruct({"certData": SimStruct({"size": SimTypeInt(signed=False, label="UInt32"), "data": SimTypePointer(SimTypeChar(label="Byte"), offset=0)}, name="FWP_BYTE_BLOB", pack=False, align=None), "certHash": SimStruct({"size": SimTypeInt(signed=False, label="UInt32"), "data": SimTypePointer(SimTypeChar(label="Byte"), offset=0)}, name="FWP_BYTE_BLOB", pack=False, align=None), "eku": SimTypePointer(SimStruct({"numEku": SimTypeInt(signed=False, label="UInt32"), "eku": SimTypePointer(SimTypePointer(SimTypeChar(label="Byte"), offset=0), offset=0)}, name="IKEEXT_CERT_EKUS0", pack=False, align=None), offset=0), "name": SimTypePointer(SimStruct({"nameType": SimTypeInt(signed=False, label="IKEEXT_CERT_CRITERIA_NAME_TYPE"), "certName": SimTypePointer(SimTypeChar(label="Char"), offset=0)}, name="IKEEXT_CERT_NAME0", pack=False, align=None), offset=0), "flags": SimTypeInt(signed=False, label="UInt32")}, name="IKEEXT_CERTIFICATE_CRITERIA0", pack=False, align=None), offset=0)}, name="_Anonymous2_e__Struct", pack=False, align=None), "Anonymous3": SimStruct({"outboundRootStoreArraySize": SimTypeInt(signed=False, label="UInt32"), "outboundTrustedRootStoreCriteria": SimTypePointer(SimStruct({"certData": SimStruct({"size": SimTypeInt(signed=False, label="UInt32"), "data": SimTypePointer(SimTypeChar(label="Byte"), offset=0)}, name="FWP_BYTE_BLOB", pack=False, align=None), "certHash": SimStruct({"size": SimTypeInt(signed=False, label="UInt32"), "data": SimTypePointer(SimTypeChar(label="Byte"), offset=0)}, name="FWP_BYTE_BLOB", pack=False, align=None), "eku": SimTypePointer(SimStruct({"numEku": SimTypeInt(signed=False, label="UInt32"), "eku": SimTypePointer(SimTypePointer(SimTypeChar(label="Byte"), offset=0), offset=0)}, name="IKEEXT_CERT_EKUS0", pack=False, align=None), offset=0), "name": SimTypePointer(SimStruct({"nameType": SimTypeInt(signed=False, label="IKEEXT_CERT_CRITERIA_NAME_TYPE"), "certName": SimTypePointer(SimTypeChar(label="Char"), offset=0)}, name="IKEEXT_CERT_NAME0", pack=False, align=None), offset=0), "flags": SimTypeInt(signed=False, label="UInt32")}, name="IKEEXT_CERTIFICATE_CRITERIA0", pack=False, align=None), offset=0)}, name="_Anonymous3_e__Struct", pack=False, align=None)}, name="<anon>", label="None"), "flags": SimTypeInt(signed=False, label="IKEEXT_CERT_AUTH"), "localCertLocationUrl": SimStruct({"size": SimTypeInt(signed=False, label="UInt32"), "data": SimTypePointer(SimTypeChar(label="Byte"), offset=0)}, name="FWP_BYTE_BLOB", pack=False, align=None)}, name="IKEEXT_CERTIFICATE_AUTHENTICATION2", pack=False, align=None), "cgaAuthentication": SimStruct({"keyContainerName": SimTypePointer(SimTypeChar(label="Char"), offset=0), "cspName": SimTypePointer(SimTypeChar(label="Char"), offset=0), "cspType": SimTypeInt(signed=False, label="UInt32"), "cgaModifier": SimStruct({"byteArray16": SimTypeFixedSizeArray(SimTypeChar(label="Byte"), 16)}, name="FWP_BYTE_ARRAY16", pack=False, align=None), "cgaCollisionCount": SimTypeChar(label="Byte")}, name="IKEEXT_IPV6_CGA_AUTHENTICATION0", pack=False, align=None), "eapAuthentication": SimStruct({"flags": SimTypeInt(signed=False, label="IKEEXT_EAP_AUTHENTICATION_FLAGS")}, name="IKEEXT_EAP_AUTHENTICATION0", pack=False, align=None)}, name="<anon>", label="None")}, name="IKEEXT_AUTHENTICATION_METHOD2", pack=False, align=None), offset=0), "initiatorImpersonationType": SimTypeInt(signed=False, label="IKEEXT_AUTHENTICATION_IMPERSONATION_TYPE")}, name="IKEEXT_EM_POLICY2", pack=False, align=None), offset=0), "fwdPathSaLifetime": SimTypeInt(signed=False, label="UInt32")}, name="IPSEC_TUNNEL_POLICY2", pack=False, align=None), offset=0), "ikeV2QmTransportPolicy": SimTypePointer(SimStruct({"numIpsecProposals": SimTypeInt(signed=False, label="UInt32"), "ipsecProposals": SimTypePointer(SimStruct({"lifetime": SimStruct({"lifetimeSeconds": SimTypeInt(signed=False, label="UInt32"), "lifetimeKilobytes": SimTypeInt(signed=False, label="UInt32"), "lifetimePackets": SimTypeInt(signed=False, label="UInt32")}, name="IPSEC_SA_LIFETIME0", pack=False, align=None), "numSaTransforms": SimTypeInt(signed=False, label="UInt32"), "saTransforms": SimTypePointer(SimStruct({"ipsecTransformType": SimTypeInt(signed=False, label="IPSEC_TRANSFORM_TYPE"), "Anonymous": SimUnion({"ahTransform": SimTypePointer(SimStruct({"authTransformId": SimStruct({"authType": SimTypeInt(signed=False, label="IPSEC_AUTH_TYPE"), "authConfig": SimTypeChar(label="Byte")}, name="IPSEC_AUTH_TRANSFORM_ID0", pack=False, align=None), "cryptoModuleId": SimTypePointer(SimTypeBottom(label="Guid"), offset=0)}, name="IPSEC_AUTH_TRANSFORM0", pack=False, align=None), offset=0), "espAuthTransform": SimTypePointer(SimStruct({"authTransformId": SimStruct({"authType": SimTypeInt(signed=False, label="IPSEC_AUTH_TYPE"), "authConfig": SimTypeChar(label="Byte")}, name="IPSEC_AUTH_TRANSFORM_ID0", pack=False, align=None), "cryptoModuleId": SimTypePointer(SimTypeBottom(label="Guid"), offset=0)}, name="IPSEC_AUTH_TRANSFORM0", pack=False, align=None), offset=0), "espCipherTransform": SimTypePointer(SimStruct({"cipherTransformId": SimStruct({"cipherType": SimTypeInt(signed=False, label="IPSEC_CIPHER_TYPE"), "cipherConfig": SimTypeChar(label="Byte")}, name="IPSEC_CIPHER_TRANSFORM_ID0", pack=False, align=None), "cryptoModuleId": SimTypePointer(SimTypeBottom(label="Guid"), offset=0)}, name="IPSEC_CIPHER_TRANSFORM0", pack=False, align=None), offset=0), "espAuthAndCipherTransform": SimTypePointer(SimStruct({"authTransform": SimStruct({"authTransformId": SimStruct({"authType": SimTypeInt(signed=False, label="IPSEC_AUTH_TYPE"), "authConfig": SimTypeChar(label="Byte")}, name="IPSEC_AUTH_TRANSFORM_ID0", pack=False, align=None), "cryptoModuleId": SimTypePointer(SimTypeBottom(label="Guid"), offset=0)}, name="IPSEC_AUTH_TRANSFORM0", pack=False, align=None), "cipherTransform": SimStruct({"cipherTransformId": SimStruct({"cipherType": SimTypeInt(signed=False, label="IPSEC_CIPHER_TYPE"), "cipherConfig": SimTypeChar(label="Byte")}, name="IPSEC_CIPHER_TRANSFORM_ID0", pack=False, align=None), "cryptoModuleId": SimTypePointer(SimTypeBottom(label="Guid"), offset=0)}, name="IPSEC_CIPHER_TRANSFORM0", pack=False, align=None)}, name="IPSEC_AUTH_AND_CIPHER_TRANSFORM0", pack=False, align=None), offset=0), "espAuthFwTransform": SimTypePointer(SimStruct({"authTransformId": SimStruct({"authType": SimTypeInt(signed=False, label="IPSEC_AUTH_TYPE"), "authConfig": SimTypeChar(label="Byte")}, name="IPSEC_AUTH_TRANSFORM_ID0", pack=False, align=None), "cryptoModuleId": SimTypePointer(SimTypeBottom(label="Guid"), offset=0)}, name="IPSEC_AUTH_TRANSFORM0", pack=False, align=None), offset=0)}, name="<anon>", label="None")}, name="IPSEC_SA_TRANSFORM0", pack=False, align=None), offset=0), "pfsGroup": SimTypeInt(signed=False, label="IPSEC_PFS_GROUP")}, name="IPSEC_PROPOSAL0", pack=False, align=None), offset=0), "flags": SimTypeInt(signed=False, label="IPSEC_POLICY_FLAG"), "ndAllowClearTimeoutSeconds": SimTypeInt(signed=False, label="UInt32"), "saIdleTimeout": SimStruct({"idleTimeoutSeconds": SimTypeInt(signed=False, label="UInt32"), "idleTimeoutSecondsFailOver": SimTypeInt(signed=False, label="UInt32")}, name="IPSEC_SA_IDLE_TIMEOUT0", pack=False, align=None), "emPolicy": SimTypePointer(SimStruct({"numAuthenticationMethods": SimTypeInt(signed=False, label="UInt32"), "authenticationMethods": SimTypePointer(SimStruct({"authenticationMethodType": SimTypeInt(signed=False, label="IKEEXT_AUTHENTICATION_METHOD_TYPE"), "Anonymous": SimUnion({"presharedKeyAuthentication": SimStruct({"presharedKey": SimStruct({"size": SimTypeInt(signed=False, label="UInt32"), "data": SimTypePointer(SimTypeChar(label="Byte"), offset=0)}, name="FWP_BYTE_BLOB", pack=False, align=None), "flags": SimTypeInt(signed=False, label="IKEEXT_PRESHARED_KEY_AUTHENTICATION_FLAGS")}, name="IKEEXT_PRESHARED_KEY_AUTHENTICATION1", pack=False, align=None), "certificateAuthentication": SimStruct({"inboundConfigType": SimTypeInt(signed=False, label="IKEEXT_CERT_CONFIG_TYPE"), "Anonymous1": SimUnion({"Anonymous1": SimStruct({"inboundRootArraySize": SimTypeInt(signed=False, label="UInt32"), "inboundRootCriteria": SimTypePointer(SimStruct({"certData": SimStruct({"size": SimTypeInt(signed=False, label="UInt32"), "data": SimTypePointer(SimTypeChar(label="Byte"), offset=0)}, name="FWP_BYTE_BLOB", pack=False, align=None), "certHash": SimStruct({"size": SimTypeInt(signed=False, label="UInt32"), "data": SimTypePointer(SimTypeChar(label="Byte"), offset=0)}, name="FWP_BYTE_BLOB", pack=False, align=None), "eku": SimTypePointer(SimStruct({"numEku": SimTypeInt(signed=False, label="UInt32"), "eku": SimTypePointer(SimTypePointer(SimTypeChar(label="Byte"), offset=0), offset=0)}, name="IKEEXT_CERT_EKUS0", pack=False, align=None), offset=0), "name": SimTypePointer(SimStruct({"nameType": SimTypeInt(signed=False, label="IKEEXT_CERT_CRITERIA_NAME_TYPE"), "certName": SimTypePointer(SimTypeChar(label="Char"), offset=0)}, name="IKEEXT_CERT_NAME0", pack=False, align=None), offset=0), "flags": SimTypeInt(signed=False, label="UInt32")}, name="IKEEXT_CERTIFICATE_CRITERIA0", pack=False, align=None), offset=0)}, name="_Anonymous1_e__Struct", pack=False, align=None), "Anonymous2": SimStruct({"inboundEnterpriseStoreArraySize": SimTypeInt(signed=False, label="UInt32"), "inboundEnterpriseStoreCriteria": SimTypePointer(SimStruct({"certData": SimStruct({"size": SimTypeInt(signed=False, label="UInt32"), "data": SimTypePointer(SimTypeChar(label="Byte"), offset=0)}, name="FWP_BYTE_BLOB", pack=False, align=None), "certHash": SimStruct({"size": SimTypeInt(signed=False, label="UInt32"), "data": SimTypePointer(SimTypeChar(label="Byte"), offset=0)}, name="FWP_BYTE_BLOB", pack=False, align=None), "eku": SimTypePointer(SimStruct({"numEku": SimTypeInt(signed=False, label="UInt32"), "eku": SimTypePointer(SimTypePointer(SimTypeChar(label="Byte"), offset=0), offset=0)}, name="IKEEXT_CERT_EKUS0", pack=False, align=None), offset=0), "name": SimTypePointer(SimStruct({"nameType": SimTypeInt(signed=False, label="IKEEXT_CERT_CRITERIA_NAME_TYPE"), "certName": SimTypePointer(SimTypeChar(label="Char"), offset=0)}, name="IKEEXT_CERT_NAME0", pack=False, align=None), offset=0), "flags": SimTypeInt(signed=False, label="UInt32")}, name="IKEEXT_CERTIFICATE_CRITERIA0", pack=False, align=None), offset=0)}, name="_Anonymous2_e__Struct", pack=False, align=None), "Anonymous3": SimStruct({"inboundRootStoreArraySize": SimTypeInt(signed=False, label="UInt32"), "inboundTrustedRootStoreCriteria": SimTypePointer(SimStruct({"certData": SimStruct({"size": SimTypeInt(signed=False, label="UInt32"), "data": SimTypePointer(SimTypeChar(label="Byte"), offset=0)}, name="FWP_BYTE_BLOB", pack=False, align=None), "certHash": SimStruct({"size": SimTypeInt(signed=False, label="UInt32"), "data": SimTypePointer(SimTypeChar(label="Byte"), offset=0)}, name="FWP_BYTE_BLOB", pack=False, align=None), "eku": SimTypePointer(SimStruct({"numEku": SimTypeInt(signed=False, label="UInt32"), "eku": SimTypePointer(SimTypePointer(SimTypeChar(label="Byte"), offset=0), offset=0)}, name="IKEEXT_CERT_EKUS0", pack=False, align=None), offset=0), "name": SimTypePointer(SimStruct({"nameType": SimTypeInt(signed=False, label="IKEEXT_CERT_CRITERIA_NAME_TYPE"), "certName": SimTypePointer(SimTypeChar(label="Char"), offset=0)}, name="IKEEXT_CERT_NAME0", pack=False, align=None), offset=0), "flags": SimTypeInt(signed=False, label="UInt32")}, name="IKEEXT_CERTIFICATE_CRITERIA0", pack=False, align=None), offset=0)}, name="_Anonymous3_e__Struct", pack=False, align=None)}, name="<anon>", label="None"), "outboundConfigType": SimTypeInt(signed=False, label="IKEEXT_CERT_CONFIG_TYPE"), "Anonymous2": SimUnion({"Anonymous1": SimStruct({"outboundRootArraySize": SimTypeInt(signed=False, label="UInt32"), "outboundRootCriteria": SimTypePointer(SimStruct({"certData": SimStruct({"size": SimTypeInt(signed=False, label="UInt32"), "data": SimTypePointer(SimTypeChar(label="Byte"), offset=0)}, name="FWP_BYTE_BLOB", pack=False, align=None), "certHash": SimStruct({"size": SimTypeInt(signed=False, label="UInt32"), "data": SimTypePointer(SimTypeChar(label="Byte"), offset=0)}, name="FWP_BYTE_BLOB", pack=False, align=None), "eku": SimTypePointer(SimStruct({"numEku": SimTypeInt(signed=False, label="UInt32"), "eku": SimTypePointer(SimTypePointer(SimTypeChar(label="Byte"), offset=0), offset=0)}, name="IKEEXT_CERT_EKUS0", pack=False, align=None), offset=0), "name": SimTypePointer(SimStruct({"nameType": SimTypeInt(signed=False, label="IKEEXT_CERT_CRITERIA_NAME_TYPE"), "certName": SimTypePointer(SimTypeChar(label="Char"), offset=0)}, name="IKEEXT_CERT_NAME0", pack=False, align=None), offset=0), "flags": SimTypeInt(signed=False, label="UInt32")}, name="IKEEXT_CERTIFICATE_CRITERIA0", pack=False, align=None), offset=0)}, name="_Anonymous1_e__Struct", pack=False, align=None), "Anonymous2": SimStruct({"outboundEnterpriseStoreArraySize": SimTypeInt(signed=False, label="UInt32"), "outboundEnterpriseStoreCriteria": SimTypePointer(SimStruct({"certData": SimStruct({"size": SimTypeInt(signed=False, label="UInt32"), "data": SimTypePointer(SimTypeChar(label="Byte"), offset=0)}, name="FWP_BYTE_BLOB", pack=False, align=None), "certHash": SimStruct({"size": SimTypeInt(signed=False, label="UInt32"), "data": SimTypePointer(SimTypeChar(label="Byte"), offset=0)}, name="FWP_BYTE_BLOB", pack=False, align=None), "eku": SimTypePointer(SimStruct({"numEku": SimTypeInt(signed=False, label="UInt32"), "eku": SimTypePointer(SimTypePointer(SimTypeChar(label="Byte"), offset=0), offset=0)}, name="IKEEXT_CERT_EKUS0", pack=False, align=None), offset=0), "name": SimTypePointer(SimStruct({"nameType": SimTypeInt(signed=False, label="IKEEXT_CERT_CRITERIA_NAME_TYPE"), "certName": SimTypePointer(SimTypeChar(label="Char"), offset=0)}, name="IKEEXT_CERT_NAME0", pack=False, align=None), offset=0), "flags": SimTypeInt(signed=False, label="UInt32")}, name="IKEEXT_CERTIFICATE_CRITERIA0", pack=False, align=None), offset=0)}, name="_Anonymous2_e__Struct", pack=False, align=None), "Anonymous3": SimStruct({"outboundRootStoreArraySize": SimTypeInt(signed=False, label="UInt32"), "outboundTrustedRootStoreCriteria": SimTypePointer(SimStruct({"certData": SimStruct({"size": SimTypeInt(signed=False, label="UInt32"), "data": SimTypePointer(SimTypeChar(label="Byte"), offset=0)}, name="FWP_BYTE_BLOB", pack=False, align=None), "certHash": SimStruct({"size": SimTypeInt(signed=False, label="UInt32"), "data": SimTypePointer(SimTypeChar(label="Byte"), offset=0)}, name="FWP_BYTE_BLOB", pack=False, align=None), "eku": SimTypePointer(SimStruct({"numEku": SimTypeInt(signed=False, label="UInt32"), "eku": SimTypePointer(SimTypePointer(SimTypeChar(label="Byte"), offset=0), offset=0)}, name="IKEEXT_CERT_EKUS0", pack=False, align=None), offset=0), "name": SimTypePointer(SimStruct({"nameType": SimTypeInt(signed=False, label="IKEEXT_CERT_CRITERIA_NAME_TYPE"), "certName": SimTypePointer(SimTypeChar(label="Char"), offset=0)}, name="IKEEXT_CERT_NAME0", pack=False, align=None), offset=0), "flags": SimTypeInt(signed=False, label="UInt32")}, name="IKEEXT_CERTIFICATE_CRITERIA0", pack=False, align=None), offset=0)}, name="_Anonymous3_e__Struct", pack=False, align=None)}, name="<anon>", label="None"), "flags": SimTypeInt(signed=False, label="IKEEXT_CERT_AUTH"), "localCertLocationUrl": SimStruct({"size": SimTypeInt(signed=False, label="UInt32"), "data": SimTypePointer(SimTypeChar(label="Byte"), offset=0)}, name="FWP_BYTE_BLOB", pack=False, align=None)}, name="IKEEXT_CERTIFICATE_AUTHENTICATION2", pack=False, align=None), "kerberosAuthentication": SimStruct({"flags": SimTypeInt(signed=False, label="IKEEXT_KERBEROS_AUTHENTICATION_FLAGS"), "proxyServer": SimTypePointer(SimTypeChar(label="Char"), offset=0)}, name="IKEEXT_KERBEROS_AUTHENTICATION1", pack=False, align=None), "reservedAuthentication": SimStruct({"flags": SimTypeInt(signed=False, label="IKEEXT_RESERVED_AUTHENTICATION_FLAGS")}, name="IKEEXT_RESERVED_AUTHENTICATION0", pack=False, align=None), "ntlmV2Authentication": SimStruct({"flags": SimTypeInt(signed=False, label="UInt32")}, name="IKEEXT_NTLM_V2_AUTHENTICATION0", pack=False, align=None), "sslAuthentication": SimStruct({"inboundConfigType": SimTypeInt(signed=False, label="IKEEXT_CERT_CONFIG_TYPE"), "Anonymous1": SimUnion({"Anonymous1": SimStruct({"inboundRootArraySize": SimTypeInt(signed=False, label="UInt32"), "inboundRootCriteria": SimTypePointer(SimStruct({"certData": SimStruct({"size": SimTypeInt(signed=False, label="UInt32"), "data": SimTypePointer(SimTypeChar(label="Byte"), offset=0)}, name="FWP_BYTE_BLOB", pack=False, align=None), "certHash": SimStruct({"size": SimTypeInt(signed=False, label="UInt32"), "data": SimTypePointer(SimTypeChar(label="Byte"), offset=0)}, name="FWP_BYTE_BLOB", pack=False, align=None), "eku": SimTypePointer(SimStruct({"numEku": SimTypeInt(signed=False, label="UInt32"), "eku": SimTypePointer(SimTypePointer(SimTypeChar(label="Byte"), offset=0), offset=0)}, name="IKEEXT_CERT_EKUS0", pack=False, align=None), offset=0), "name": SimTypePointer(SimStruct({"nameType": SimTypeInt(signed=False, label="IKEEXT_CERT_CRITERIA_NAME_TYPE"), "certName": SimTypePointer(SimTypeChar(label="Char"), offset=0)}, name="IKEEXT_CERT_NAME0", pack=False, align=None), offset=0), "flags": SimTypeInt(signed=False, label="UInt32")}, name="IKEEXT_CERTIFICATE_CRITERIA0", pack=False, align=None), offset=0)}, name="_Anonymous1_e__Struct", pack=False, align=None), "Anonymous2": SimStruct({"inboundEnterpriseStoreArraySize": SimTypeInt(signed=False, label="UInt32"), "inboundEnterpriseStoreCriteria": SimTypePointer(SimStruct({"certData": SimStruct({"size": SimTypeInt(signed=False, label="UInt32"), "data": SimTypePointer(SimTypeChar(label="Byte"), offset=0)}, name="FWP_BYTE_BLOB", pack=False, align=None), "certHash": SimStruct({"size": SimTypeInt(signed=False, label="UInt32"), "data": SimTypePointer(SimTypeChar(label="Byte"), offset=0)}, name="FWP_BYTE_BLOB", pack=False, align=None), "eku": SimTypePointer(SimStruct({"numEku": SimTypeInt(signed=False, label="UInt32"), "eku": SimTypePointer(SimTypePointer(SimTypeChar(label="Byte"), offset=0), offset=0)}, name="IKEEXT_CERT_EKUS0", pack=False, align=None), offset=0), "name": SimTypePointer(SimStruct({"nameType": SimTypeInt(signed=False, label="IKEEXT_CERT_CRITERIA_NAME_TYPE"), "certName": SimTypePointer(SimTypeChar(label="Char"), offset=0)}, name="IKEEXT_CERT_NAME0", pack=False, align=None), offset=0), "flags": SimTypeInt(signed=False, label="UInt32")}, name="IKEEXT_CERTIFICATE_CRITERIA0", pack=False, align=None), offset=0)}, name="_Anonymous2_e__Struct", pack=False, align=None), "Anonymous3": SimStruct({"inboundRootStoreArraySize": SimTypeInt(signed=False, label="UInt32"), "inboundTrustedRootStoreCriteria": SimTypePointer(SimStruct({"certData": SimStruct({"size": SimTypeInt(signed=False, label="UInt32"), "data": SimTypePointer(SimTypeChar(label="Byte"), offset=0)}, name="FWP_BYTE_BLOB", pack=False, align=None), "certHash": SimStruct({"size": SimTypeInt(signed=False, label="UInt32"), "data": SimTypePointer(SimTypeChar(label="Byte"), offset=0)}, name="FWP_BYTE_BLOB", pack=False, align=None), "eku": SimTypePointer(SimStruct({"numEku": SimTypeInt(signed=False, label="UInt32"), "eku": SimTypePointer(SimTypePointer(SimTypeChar(label="Byte"), offset=0), offset=0)}, name="IKEEXT_CERT_EKUS0", pack=False, align=None), offset=0), "name": SimTypePointer(SimStruct({"nameType": SimTypeInt(signed=False, label="IKEEXT_CERT_CRITERIA_NAME_TYPE"), "certName": SimTypePointer(SimTypeChar(label="Char"), offset=0)}, name="IKEEXT_CERT_NAME0", pack=False, align=None), offset=0), "flags": SimTypeInt(signed=False, label="UInt32")}, name="IKEEXT_CERTIFICATE_CRITERIA0", pack=False, align=None), offset=0)}, name="_Anonymous3_e__Struct", pack=False, align=None)}, name="<anon>", label="None"), "outboundConfigType": SimTypeInt(signed=False, label="IKEEXT_CERT_CONFIG_TYPE"), "Anonymous2": SimUnion({"Anonymous1": SimStruct({"outboundRootArraySize": SimTypeInt(signed=False, label="UInt32"), "outboundRootCriteria": SimTypePointer(SimStruct({"certData": SimStruct({"size": SimTypeInt(signed=False, label="UInt32"), "data": SimTypePointer(SimTypeChar(label="Byte"), offset=0)}, name="FWP_BYTE_BLOB", pack=False, align=None), "certHash": SimStruct({"size": SimTypeInt(signed=False, label="UInt32"), "data": SimTypePointer(SimTypeChar(label="Byte"), offset=0)}, name="FWP_BYTE_BLOB", pack=False, align=None), "eku": SimTypePointer(SimStruct({"numEku": SimTypeInt(signed=False, label="UInt32"), "eku": SimTypePointer(SimTypePointer(SimTypeChar(label="Byte"), offset=0), offset=0)}, name="IKEEXT_CERT_EKUS0", pack=False, align=None), offset=0), "name": SimTypePointer(SimStruct({"nameType": SimTypeInt(signed=False, label="IKEEXT_CERT_CRITERIA_NAME_TYPE"), "certName": SimTypePointer(SimTypeChar(label="Char"), offset=0)}, name="IKEEXT_CERT_NAME0", pack=False, align=None), offset=0), "flags": SimTypeInt(signed=False, label="UInt32")}, name="IKEEXT_CERTIFICATE_CRITERIA0", pack=False, align=None), offset=0)}, name="_Anonymous1_e__Struct", pack=False, align=None), "Anonymous2": SimStruct({"outboundEnterpriseStoreArraySize": SimTypeInt(signed=False, label="UInt32"), "outboundEnterpriseStoreCriteria": SimTypePointer(SimStruct({"certData": SimStruct({"size": SimTypeInt(signed=False, label="UInt32"), "data": SimTypePointer(SimTypeChar(label="Byte"), offset=0)}, name="FWP_BYTE_BLOB", pack=False, align=None), "certHash": SimStruct({"size": SimTypeInt(signed=False, label="UInt32"), "data": SimTypePointer(SimTypeChar(label="Byte"), offset=0)}, name="FWP_BYTE_BLOB", pack=False, align=None), "eku": SimTypePointer(SimStruct({"numEku": SimTypeInt(signed=False, label="UInt32"), "eku": SimTypePointer(SimTypePointer(SimTypeChar(label="Byte"), offset=0), offset=0)}, name="IKEEXT_CERT_EKUS0", pack=False, align=None), offset=0), "name": SimTypePointer(SimStruct({"nameType": SimTypeInt(signed=False, label="IKEEXT_CERT_CRITERIA_NAME_TYPE"), "certName": SimTypePointer(SimTypeChar(label="Char"), offset=0)}, name="IKEEXT_CERT_NAME0", pack=False, align=None), offset=0), "flags": SimTypeInt(signed=False, label="UInt32")}, name="IKEEXT_CERTIFICATE_CRITERIA0", pack=False, align=None), offset=0)}, name="_Anonymous2_e__Struct", pack=False, align=None), "Anonymous3": SimStruct({"outboundRootStoreArraySize": SimTypeInt(signed=False, label="UInt32"), "outboundTrustedRootStoreCriteria": SimTypePointer(SimStruct({"certData": SimStruct({"size": SimTypeInt(signed=False, label="UInt32"), "data": SimTypePointer(SimTypeChar(label="Byte"), offset=0)}, name="FWP_BYTE_BLOB", pack=False, align=None), "certHash": SimStruct({"size": SimTypeInt(signed=False, label="UInt32"), "data": SimTypePointer(SimTypeChar(label="Byte"), offset=0)}, name="FWP_BYTE_BLOB", pack=False, align=None), "eku": SimTypePointer(SimStruct({"numEku": SimTypeInt(signed=False, label="UInt32"), "eku": SimTypePointer(SimTypePointer(SimTypeChar(label="Byte"), offset=0), offset=0)}, name="IKEEXT_CERT_EKUS0", pack=False, align=None), offset=0), "name": SimTypePointer(SimStruct({"nameType": SimTypeInt(signed=False, label="IKEEXT_CERT_CRITERIA_NAME_TYPE"), "certName": SimTypePointer(SimTypeChar(label="Char"), offset=0)}, name="IKEEXT_CERT_NAME0", pack=False, align=None), offset=0), "flags": SimTypeInt(signed=False, label="UInt32")}, name="IKEEXT_CERTIFICATE_CRITERIA0", pack=False, align=None), offset=0)}, name="_Anonymous3_e__Struct", pack=False, align=None)}, name="<anon>", label="None"), "flags": SimTypeInt(signed=False, label="IKEEXT_CERT_AUTH"), "localCertLocationUrl": SimStruct({"size": SimTypeInt(signed=False, label="UInt32"), "data": SimTypePointer(SimTypeChar(label="Byte"), offset=0)}, name="FWP_BYTE_BLOB", pack=False, align=None)}, name="IKEEXT_CERTIFICATE_AUTHENTICATION2", pack=False, align=None), "cgaAuthentication": SimStruct({"keyContainerName": SimTypePointer(SimTypeChar(label="Char"), offset=0), "cspName": SimTypePointer(SimTypeChar(label="Char"), offset=0), "cspType": SimTypeInt(signed=False, label="UInt32"), "cgaModifier": SimStruct({"byteArray16": SimTypeFixedSizeArray(SimTypeChar(label="Byte"), 16)}, name="FWP_BYTE_ARRAY16", pack=False, align=None), "cgaCollisionCount": SimTypeChar(label="Byte")}, name="IKEEXT_IPV6_CGA_AUTHENTICATION0", pack=False, align=None), "eapAuthentication": SimStruct({"flags": SimTypeInt(signed=False, label="IKEEXT_EAP_AUTHENTICATION_FLAGS")}, name="IKEEXT_EAP_AUTHENTICATION0", pack=False, align=None)}, name="<anon>", label="None")}, name="IKEEXT_AUTHENTICATION_METHOD2", pack=False, align=None), offset=0), "initiatorImpersonationType": SimTypeInt(signed=False, label="IKEEXT_AUTHENTICATION_IMPERSONATION_TYPE")}, name="IKEEXT_EM_POLICY2", pack=False, align=None), offset=0)}, name="IPSEC_TRANSPORT_POLICY2", pack=False, align=None), offset=0), "ikeV2MmPolicy": SimTypePointer(SimStruct({"softExpirationTime": SimTypeInt(signed=False, label="UInt32"), "numAuthenticationMethods": SimTypeInt(signed=False, label="UInt32"), "authenticationMethods": SimTypePointer(SimStruct({"authenticationMethodType": SimTypeInt(signed=False, label="IKEEXT_AUTHENTICATION_METHOD_TYPE"), "Anonymous": SimUnion({"presharedKeyAuthentication": SimStruct({"presharedKey": SimStruct({"size": SimTypeInt(signed=False, label="UInt32"), "data": SimTypePointer(SimTypeChar(label="Byte"), offset=0)}, name="FWP_BYTE_BLOB", pack=False, align=None), "flags": SimTypeInt(signed=False, label="IKEEXT_PRESHARED_KEY_AUTHENTICATION_FLAGS")}, name="IKEEXT_PRESHARED_KEY_AUTHENTICATION1", pack=False, align=None), "certificateAuthentication": SimStruct({"inboundConfigType": SimTypeInt(signed=False, label="IKEEXT_CERT_CONFIG_TYPE"), "Anonymous1": SimUnion({"Anonymous1": SimStruct({"inboundRootArraySize": SimTypeInt(signed=False, label="UInt32"), "inboundRootCriteria": SimTypePointer(SimStruct({"certData": SimStruct({"size": SimTypeInt(signed=False, label="UInt32"), "data": SimTypePointer(SimTypeChar(label="Byte"), offset=0)}, name="FWP_BYTE_BLOB", pack=False, align=None), "certHash": SimStruct({"size": SimTypeInt(signed=False, label="UInt32"), "data": SimTypePointer(SimTypeChar(label="Byte"), offset=0)}, name="FWP_BYTE_BLOB", pack=False, align=None), "eku": SimTypePointer(SimStruct({"numEku": SimTypeInt(signed=False, label="UInt32"), "eku": SimTypePointer(SimTypePointer(SimTypeChar(label="Byte"), offset=0), offset=0)}, name="IKEEXT_CERT_EKUS0", pack=False, align=None), offset=0), "name": SimTypePointer(SimStruct({"nameType": SimTypeInt(signed=False, label="IKEEXT_CERT_CRITERIA_NAME_TYPE"), "certName": SimTypePointer(SimTypeChar(label="Char"), offset=0)}, name="IKEEXT_CERT_NAME0", pack=False, align=None), offset=0), "flags": SimTypeInt(signed=False, label="UInt32")}, name="IKEEXT_CERTIFICATE_CRITERIA0", pack=False, align=None), offset=0)}, name="_Anonymous1_e__Struct", pack=False, align=None), "Anonymous2": SimStruct({"inboundEnterpriseStoreArraySize": SimTypeInt(signed=False, label="UInt32"), "inboundEnterpriseStoreCriteria": SimTypePointer(SimStruct({"certData": SimStruct({"size": SimTypeInt(signed=False, label="UInt32"), "data": SimTypePointer(SimTypeChar(label="Byte"), offset=0)}, name="FWP_BYTE_BLOB", pack=False, align=None), "certHash": SimStruct({"size": SimTypeInt(signed=False, label="UInt32"), "data": SimTypePointer(SimTypeChar(label="Byte"), offset=0)}, name="FWP_BYTE_BLOB", pack=False, align=None), "eku": SimTypePointer(SimStruct({"numEku": SimTypeInt(signed=False, label="UInt32"), "eku": SimTypePointer(SimTypePointer(SimTypeChar(label="Byte"), offset=0), offset=0)}, name="IKEEXT_CERT_EKUS0", pack=False, align=None), offset=0), "name": SimTypePointer(SimStruct({"nameType": SimTypeInt(signed=False, label="IKEEXT_CERT_CRITERIA_NAME_TYPE"), "certName": SimTypePointer(SimTypeChar(label="Char"), offset=0)}, name="IKEEXT_CERT_NAME0", pack=False, align=None), offset=0), "flags": SimTypeInt(signed=False, label="UInt32")}, name="IKEEXT_CERTIFICATE_CRITERIA0", pack=False, align=None), offset=0)}, name="_Anonymous2_e__Struct", pack=False, align=None), "Anonymous3": SimStruct({"inboundRootStoreArraySize": SimTypeInt(signed=False, label="UInt32"), "inboundTrustedRootStoreCriteria": SimTypePointer(SimStruct({"certData": SimStruct({"size": SimTypeInt(signed=False, label="UInt32"), "data": SimTypePointer(SimTypeChar(label="Byte"), offset=0)}, name="FWP_BYTE_BLOB", pack=False, align=None), "certHash": SimStruct({"size": SimTypeInt(signed=False, label="UInt32"), "data": SimTypePointer(SimTypeChar(label="Byte"), offset=0)}, name="FWP_BYTE_BLOB", pack=False, align=None), "eku": SimTypePointer(SimStruct({"numEku": SimTypeInt(signed=False, label="UInt32"), "eku": SimTypePointer(SimTypePointer(SimTypeChar(label="Byte"), offset=0), offset=0)}, name="IKEEXT_CERT_EKUS0", pack=False, align=None), offset=0), "name": SimTypePointer(SimStruct({"nameType": SimTypeInt(signed=False, label="IKEEXT_CERT_CRITERIA_NAME_TYPE"), "certName": SimTypePointer(SimTypeChar(label="Char"), offset=0)}, name="IKEEXT_CERT_NAME0", pack=False, align=None), offset=0), "flags": SimTypeInt(signed=False, label="UInt32")}, name="IKEEXT_CERTIFICATE_CRITERIA0", pack=False, align=None), offset=0)}, name="_Anonymous3_e__Struct", pack=False, align=None)}, name="<anon>", label="None"), "outboundConfigType": SimTypeInt(signed=False, label="IKEEXT_CERT_CONFIG_TYPE"), "Anonymous2": SimUnion({"Anonymous1": SimStruct({"outboundRootArraySize": SimTypeInt(signed=False, label="UInt32"), "outboundRootCriteria": SimTypePointer(SimStruct({"certData": SimStruct({"size": SimTypeInt(signed=False, label="UInt32"), "data": SimTypePointer(SimTypeChar(label="Byte"), offset=0)}, name="FWP_BYTE_BLOB", pack=False, align=None), "certHash": SimStruct({"size": SimTypeInt(signed=False, label="UInt32"), "data": SimTypePointer(SimTypeChar(label="Byte"), offset=0)}, name="FWP_BYTE_BLOB", pack=False, align=None), "eku": SimTypePointer(SimStruct({"numEku": SimTypeInt(signed=False, label="UInt32"), "eku": SimTypePointer(SimTypePointer(SimTypeChar(label="Byte"), offset=0), offset=0)}, name="IKEEXT_CERT_EKUS0", pack=False, align=None), offset=0), "name": SimTypePointer(SimStruct({"nameType": SimTypeInt(signed=False, label="IKEEXT_CERT_CRITERIA_NAME_TYPE"), "certName": SimTypePointer(SimTypeChar(label="Char"), offset=0)}, name="IKEEXT_CERT_NAME0", pack=False, align=None), offset=0), "flags": SimTypeInt(signed=False, label="UInt32")}, name="IKEEXT_CERTIFICATE_CRITERIA0", pack=False, align=None), offset=0)}, name="_Anonymous1_e__Struct", pack=False, align=None), "Anonymous2": SimStruct({"outboundEnterpriseStoreArraySize": SimTypeInt(signed=False, label="UInt32"), "outboundEnterpriseStoreCriteria": SimTypePointer(SimStruct({"certData": SimStruct({"size": SimTypeInt(signed=False, label="UInt32"), "data": SimTypePointer(SimTypeChar(label="Byte"), offset=0)}, name="FWP_BYTE_BLOB", pack=False, align=None), "certHash": SimStruct({"size": SimTypeInt(signed=False, label="UInt32"), "data": SimTypePointer(SimTypeChar(label="Byte"), offset=0)}, name="FWP_BYTE_BLOB", pack=False, align=None), "eku": SimTypePointer(SimStruct({"numEku": SimTypeInt(signed=False, label="UInt32"), "eku": SimTypePointer(SimTypePointer(SimTypeChar(label="Byte"), offset=0), offset=0)}, name="IKEEXT_CERT_EKUS0", pack=False, align=None), offset=0), "name": SimTypePointer(SimStruct({"nameType": SimTypeInt(signed=False, label="IKEEXT_CERT_CRITERIA_NAME_TYPE"), "certName": SimTypePointer(SimTypeChar(label="Char"), offset=0)}, name="IKEEXT_CERT_NAME0", pack=False, align=None), offset=0), "flags": SimTypeInt(signed=False, label="UInt32")}, name="IKEEXT_CERTIFICATE_CRITERIA0", pack=False, align=None), offset=0)}, name="_Anonymous2_e__Struct", pack=False, align=None), "Anonymous3": SimStruct({"outboundRootStoreArraySize": SimTypeInt(signed=False, label="UInt32"), "outboundTrustedRootStoreCriteria": SimTypePointer(SimStruct({"certData": SimStruct({"size": SimTypeInt(signed=False, label="UInt32"), "data": SimTypePointer(SimTypeChar(label="Byte"), offset=0)}, name="FWP_BYTE_BLOB", pack=False, align=None), "certHash": SimStruct({"size": SimTypeInt(signed=False, label="UInt32"), "data": SimTypePointer(SimTypeChar(label="Byte"), offset=0)}, name="FWP_BYTE_BLOB", pack=False, align=None), "eku": SimTypePointer(SimStruct({"numEku": SimTypeInt(signed=False, label="UInt32"), "eku": SimTypePointer(SimTypePointer(SimTypeChar(label="Byte"), offset=0), offset=0)}, name="IKEEXT_CERT_EKUS0", pack=False, align=None), offset=0), "name": SimTypePointer(SimStruct({"nameType": SimTypeInt(signed=False, label="IKEEXT_CERT_CRITERIA_NAME_TYPE"), "certName": SimTypePointer(SimTypeChar(label="Char"), offset=0)}, name="IKEEXT_CERT_NAME0", pack=False, align=None), offset=0), "flags": SimTypeInt(signed=False, label="UInt32")}, name="IKEEXT_CERTIFICATE_CRITERIA0", pack=False, align=None), offset=0)}, name="_Anonymous3_e__Struct", pack=False, align=None)}, name="<anon>", label="None"), "flags": SimTypeInt(signed=False, label="IKEEXT_CERT_AUTH"), "localCertLocationUrl": SimStruct({"size": SimTypeInt(signed=False, label="UInt32"), "data": SimTypePointer(SimTypeChar(label="Byte"), offset=0)}, name="FWP_BYTE_BLOB", pack=False, align=None)}, name="IKEEXT_CERTIFICATE_AUTHENTICATION2", pack=False, align=None), "kerberosAuthentication": SimStruct({"flags": SimTypeInt(signed=False, label="IKEEXT_KERBEROS_AUTHENTICATION_FLAGS"), "proxyServer": SimTypePointer(SimTypeChar(label="Char"), offset=0)}, name="IKEEXT_KERBEROS_AUTHENTICATION1", pack=False, align=None), "reservedAuthentication": SimStruct({"flags": SimTypeInt(signed=False, label="IKEEXT_RESERVED_AUTHENTICATION_FLAGS")}, name="IKEEXT_RESERVED_AUTHENTICATION0", pack=False, align=None), "ntlmV2Authentication": SimStruct({"flags": SimTypeInt(signed=False, label="UInt32")}, name="IKEEXT_NTLM_V2_AUTHENTICATION0", pack=False, align=None), "sslAuthentication": SimStruct({"inboundConfigType": SimTypeInt(signed=False, label="IKEEXT_CERT_CONFIG_TYPE"), "Anonymous1": SimUnion({"Anonymous1": SimStruct({"inboundRootArraySize": SimTypeInt(signed=False, label="UInt32"), "inboundRootCriteria": SimTypePointer(SimStruct({"certData": SimStruct({"size": SimTypeInt(signed=False, label="UInt32"), "data": SimTypePointer(SimTypeChar(label="Byte"), offset=0)}, name="FWP_BYTE_BLOB", pack=False, align=None), "certHash": SimStruct({"size": SimTypeInt(signed=False, label="UInt32"), "data": SimTypePointer(SimTypeChar(label="Byte"), offset=0)}, name="FWP_BYTE_BLOB", pack=False, align=None), "eku": SimTypePointer(SimStruct({"numEku": SimTypeInt(signed=False, label="UInt32"), "eku": SimTypePointer(SimTypePointer(SimTypeChar(label="Byte"), offset=0), offset=0)}, name="IKEEXT_CERT_EKUS0", pack=False, align=None), offset=0), "name": SimTypePointer(SimStruct({"nameType": SimTypeInt(signed=False, label="IKEEXT_CERT_CRITERIA_NAME_TYPE"), "certName": SimTypePointer(SimTypeChar(label="Char"), offset=0)}, name="IKEEXT_CERT_NAME0", pack=False, align=None), offset=0), "flags": SimTypeInt(signed=False, label="UInt32")}, name="IKEEXT_CERTIFICATE_CRITERIA0", pack=False, align=None), offset=0)}, name="_Anonymous1_e__Struct", pack=False, align=None), "Anonymous2": SimStruct({"inboundEnterpriseStoreArraySize": SimTypeInt(signed=False, label="UInt32"), "inboundEnterpriseStoreCriteria": SimTypePointer(SimStruct({"certData": SimStruct({"size": SimTypeInt(signed=False, label="UInt32"), "data": SimTypePointer(SimTypeChar(label="Byte"), offset=0)}, name="FWP_BYTE_BLOB", pack=False, align=None), "certHash": SimStruct({"size": SimTypeInt(signed=False, label="UInt32"), "data": SimTypePointer(SimTypeChar(label="Byte"), offset=0)}, name="FWP_BYTE_BLOB", pack=False, align=None), "eku": SimTypePointer(SimStruct({"numEku": SimTypeInt(signed=False, label="UInt32"), "eku": SimTypePointer(SimTypePointer(SimTypeChar(label="Byte"), offset=0), offset=0)}, name="IKEEXT_CERT_EKUS0", pack=False, align=None), offset=0), "name": SimTypePointer(SimStruct({"nameType": SimTypeInt(signed=False, label="IKEEXT_CERT_CRITERIA_NAME_TYPE"), "certName": SimTypePointer(SimTypeChar(label="Char"), offset=0)}, name="IKEEXT_CERT_NAME0", pack=False, align=None), offset=0), "flags": SimTypeInt(signed=False, label="UInt32")}, name="IKEEXT_CERTIFICATE_CRITERIA0", pack=False, align=None), offset=0)}, name="_Anonymous2_e__Struct", pack=False, align=None), "Anonymous3": SimStruct({"inboundRootStoreArraySize": SimTypeInt(signed=False, label="UInt32"), "inboundTrustedRootStoreCriteria": SimTypePointer(SimStruct({"certData": SimStruct({"size": SimTypeInt(signed=False, label="UInt32"), "data": SimTypePointer(SimTypeChar(label="Byte"), offset=0)}, name="FWP_BYTE_BLOB", pack=False, align=None), "certHash": SimStruct({"size": SimTypeInt(signed=False, label="UInt32"), "data": SimTypePointer(SimTypeChar(label="Byte"), offset=0)}, name="FWP_BYTE_BLOB", pack=False, align=None), "eku": SimTypePointer(SimStruct({"numEku": SimTypeInt(signed=False, label="UInt32"), "eku": SimTypePointer(SimTypePointer(SimTypeChar(label="Byte"), offset=0), offset=0)}, name="IKEEXT_CERT_EKUS0", pack=False, align=None), offset=0), "name": SimTypePointer(SimStruct({"nameType": SimTypeInt(signed=False, label="IKEEXT_CERT_CRITERIA_NAME_TYPE"), "certName": SimTypePointer(SimTypeChar(label="Char"), offset=0)}, name="IKEEXT_CERT_NAME0", pack=False, align=None), offset=0), "flags": SimTypeInt(signed=False, label="UInt32")}, name="IKEEXT_CERTIFICATE_CRITERIA0", pack=False, align=None), offset=0)}, name="_Anonymous3_e__Struct", pack=False, align=None)}, name="<anon>", label="None"), "outboundConfigType": SimTypeInt(signed=False, label="IKEEXT_CERT_CONFIG_TYPE"), "Anonymous2": SimUnion({"Anonymous1": SimStruct({"outboundRootArraySize": SimTypeInt(signed=False, label="UInt32"), "outboundRootCriteria": SimTypePointer(SimStruct({"certData": SimStruct({"size": SimTypeInt(signed=False, label="UInt32"), "data": SimTypePointer(SimTypeChar(label="Byte"), offset=0)}, name="FWP_BYTE_BLOB", pack=False, align=None), "certHash": SimStruct({"size": SimTypeInt(signed=False, label="UInt32"), "data": SimTypePointer(SimTypeChar(label="Byte"), offset=0)}, name="FWP_BYTE_BLOB", pack=False, align=None), "eku": SimTypePointer(SimStruct({"numEku": SimTypeInt(signed=False, label="UInt32"), "eku": SimTypePointer(SimTypePointer(SimTypeChar(label="Byte"), offset=0), offset=0)}, name="IKEEXT_CERT_EKUS0", pack=False, align=None), offset=0), "name": SimTypePointer(SimStruct({"nameType": SimTypeInt(signed=False, label="IKEEXT_CERT_CRITERIA_NAME_TYPE"), "certName": SimTypePointer(SimTypeChar(label="Char"), offset=0)}, name="IKEEXT_CERT_NAME0", pack=False, align=None), offset=0), "flags": SimTypeInt(signed=False, label="UInt32")}, name="IKEEXT_CERTIFICATE_CRITERIA0", pack=False, align=None), offset=0)}, name="_Anonymous1_e__Struct", pack=False, align=None), "Anonymous2": SimStruct({"outboundEnterpriseStoreArraySize": SimTypeInt(signed=False, label="UInt32"), "outboundEnterpriseStoreCriteria": SimTypePointer(SimStruct({"certData": SimStruct({"size": SimTypeInt(signed=False, label="UInt32"), "data": SimTypePointer(SimTypeChar(label="Byte"), offset=0)}, name="FWP_BYTE_BLOB", pack=False, align=None), "certHash": SimStruct({"size": SimTypeInt(signed=False, label="UInt32"), "data": SimTypePointer(SimTypeChar(label="Byte"), offset=0)}, name="FWP_BYTE_BLOB", pack=False, align=None), "eku": SimTypePointer(SimStruct({"numEku": SimTypeInt(signed=False, label="UInt32"), "eku": SimTypePointer(SimTypePointer(SimTypeChar(label="Byte"), offset=0), offset=0)}, name="IKEEXT_CERT_EKUS0", pack=False, align=None), offset=0), "name": SimTypePointer(SimStruct({"nameType": SimTypeInt(signed=False, label="IKEEXT_CERT_CRITERIA_NAME_TYPE"), "certName": SimTypePointer(SimTypeChar(label="Char"), offset=0)}, name="IKEEXT_CERT_NAME0", pack=False, align=None), offset=0), "flags": SimTypeInt(signed=False, label="UInt32")}, name="IKEEXT_CERTIFICATE_CRITERIA0", pack=False, align=None), offset=0)}, name="_Anonymous2_e__Struct", pack=False, align=None), "Anonymous3": SimStruct({"outboundRootStoreArraySize": SimTypeInt(signed=False, label="UInt32"), "outboundTrustedRootStoreCriteria": SimTypePointer(SimStruct({"certData": SimStruct({"size": SimTypeInt(signed=False, label="UInt32"), "data": SimTypePointer(SimTypeChar(label="Byte"), offset=0)}, name="FWP_BYTE_BLOB", pack=False, align=None), "certHash": SimStruct({"size": SimTypeInt(signed=False, label="UInt32"), "data": SimTypePointer(SimTypeChar(label="Byte"), offset=0)}, name="FWP_BYTE_BLOB", pack=False, align=None), "eku": SimTypePointer(SimStruct({"numEku": SimTypeInt(signed=False, label="UInt32"), "eku": SimTypePointer(SimTypePointer(SimTypeChar(label="Byte"), offset=0), offset=0)}, name="IKEEXT_CERT_EKUS0", pack=False, align=None), offset=0), "name": SimTypePointer(SimStruct({"nameType": SimTypeInt(signed=False, label="IKEEXT_CERT_CRITERIA_NAME_TYPE"), "certName": SimTypePointer(SimTypeChar(label="Char"), offset=0)}, name="IKEEXT_CERT_NAME0", pack=False, align=None), offset=0), "flags": SimTypeInt(signed=False, label="UInt32")}, name="IKEEXT_CERTIFICATE_CRITERIA0", pack=False, align=None), offset=0)}, name="_Anonymous3_e__Struct", pack=False, align=None)}, name="<anon>", label="None"), "flags": SimTypeInt(signed=False, label="IKEEXT_CERT_AUTH"), "localCertLocationUrl": SimStruct({"size": SimTypeInt(signed=False, label="UInt32"), "data": SimTypePointer(SimTypeChar(label="Byte"), offset=0)}, name="FWP_BYTE_BLOB", pack=False, align=None)}, name="IKEEXT_CERTIFICATE_AUTHENTICATION2", pack=False, align=None), "cgaAuthentication": SimStruct({"keyContainerName": SimTypePointer(SimTypeChar(label="Char"), offset=0), "cspName": SimTypePointer(SimTypeChar(label="Char"), offset=0), "cspType": SimTypeInt(signed=False, label="UInt32"), "cgaModifier": SimStruct({"byteArray16": SimTypeFixedSizeArray(SimTypeChar(label="Byte"), 16)}, name="FWP_BYTE_ARRAY16", pack=False, align=None), "cgaCollisionCount": SimTypeChar(label="Byte")}, name="IKEEXT_IPV6_CGA_AUTHENTICATION0", pack=False, align=None), "eapAuthentication": SimStruct({"flags": SimTypeInt(signed=False, label="IKEEXT_EAP_AUTHENTICATION_FLAGS")}, name="IKEEXT_EAP_AUTHENTICATION0", pack=False, align=None)}, name="<anon>", label="None")}, name="IKEEXT_AUTHENTICATION_METHOD2", pack=False, align=None), offset=0), "initiatorImpersonationType": SimTypeInt(signed=False, label="IKEEXT_AUTHENTICATION_IMPERSONATION_TYPE"), "numIkeProposals": SimTypeInt(signed=False, label="UInt32"), "ikeProposals": SimTypePointer(SimStruct({"cipherAlgorithm": SimStruct({"algoIdentifier": SimTypeInt(signed=False, label="IKEEXT_CIPHER_TYPE"), "keyLen": SimTypeInt(signed=False, label="UInt32"), "rounds": SimTypeInt(signed=False, label="UInt32")}, name="IKEEXT_CIPHER_ALGORITHM0", pack=False, align=None), "integrityAlgorithm": SimStruct({"algoIdentifier": SimTypeInt(signed=False, label="IKEEXT_INTEGRITY_TYPE")}, name="IKEEXT_INTEGRITY_ALGORITHM0", pack=False, align=None), "maxLifetimeSeconds": SimTypeInt(signed=False, label="UInt32"), "dhGroup": SimTypeInt(signed=False, label="IKEEXT_DH_GROUP"), "quickModeLimit": SimTypeInt(signed=False, label="UInt32")}, name="IKEEXT_PROPOSAL0", pack=False, align=None), offset=0), "flags": SimTypeInt(signed=False, label="IKEEXT_POLICY_FLAG"), "maxDynamicFilters": SimTypeInt(signed=False, label="UInt32"), "retransmitDurationSecs": SimTypeInt(signed=False, label="UInt32")}, name="IKEEXT_POLICY2", pack=False, align=None), offset=0), "idpOptions": SimTypePointer(SimStruct({"stateIdleTimeoutSeconds": SimTypeInt(signed=False, label="UInt32"), "perIPRateLimitQueueIdleTimeoutSeconds": SimTypeInt(signed=False, label="UInt32"), "ipV6IPsecUnauthDscp": SimTypeChar(label="Byte"), "ipV6IPsecUnauthRateLimitBytesPerSec": SimTypeInt(signed=False, label="UInt32"), "ipV6IPsecUnauthPerIPRateLimitBytesPerSec": SimTypeInt(signed=False, label="UInt32"), "ipV6IPsecAuthDscp": SimTypeChar(label="Byte"), "ipV6IPsecAuthRateLimitBytesPerSec": SimTypeInt(signed=False, label="UInt32"), "icmpV6Dscp": SimTypeChar(label="Byte"), "icmpV6RateLimitBytesPerSec": SimTypeInt(signed=False, label="UInt32"), "ipV6FilterExemptDscp": SimTypeChar(label="Byte"), "ipV6FilterExemptRateLimitBytesPerSec": SimTypeInt(signed=False, label="UInt32"), "defBlockExemptDscp": SimTypeChar(label="Byte"), "defBlockExemptRateLimitBytesPerSec": SimTypeInt(signed=False, label="UInt32"), "maxStateEntries": SimTypeInt(signed=False, label="UInt32"), "maxPerIPRateLimitQueues": SimTypeInt(signed=False, label="UInt32"), "flags": SimTypeInt(signed=False, label="IPSEC_DOSP_FLAGS"), "numPublicIFLuids": SimTypeInt(signed=False, label="UInt32"), "publicIFLuids": SimTypePointer(SimTypeLongLong(signed=False, label="UInt64"), offset=0), "numInternalIFLuids": SimTypeInt(signed=False, label="UInt32"), "internalIFLuids": SimTypePointer(SimTypeLongLong(signed=False, label="UInt64"), offset=0), "publicV6AddrMask": SimStruct({"addr": SimTypeFixedSizeArray(SimTypeChar(label="Byte"), 16), "prefixLength": SimTypeChar(label="Byte")}, name="FWP_V6_ADDR_AND_MASK", pack=False, align=None), "internalV6AddrMask": SimStruct({"addr": SimTypeFixedSizeArray(SimTypeChar(label="Byte"), 16), "prefixLength": SimTypeChar(label="Byte")}, name="FWP_V6_ADDR_AND_MASK", pack=False, align=None)}, name="IPSEC_DOSP_OPTIONS0", pack=False, align=None), offset=0)}, name="<anon>", label="None"), "providerContextId": SimTypeLongLong(signed=False, label="UInt64")}, name="FWPM_PROVIDER_CONTEXT2", pack=False, align=None), offset=0), SimTypePointer(SimStruct({"providerContextKey": SimTypeBottom(label="Guid"), "displayData": SimStruct({"name": SimTypePointer(SimTypeChar(label="Char"), offset=0), "description": SimTypePointer(SimTypeChar(label="Char"), offset=0)}, name="FWPM_DISPLAY_DATA0", pack=False, align=None), "flags": SimTypeInt(signed=False, label="UInt32"), "providerKey": SimTypePointer(SimTypeBottom(label="Guid"), offset=0), "providerData": SimStruct({"size": SimTypeInt(signed=False, label="UInt32"), "data": SimTypePointer(SimTypeChar(label="Byte"), offset=0)}, name="FWP_BYTE_BLOB", pack=False, align=None), "type": SimTypeInt(signed=False, label="FWPM_PROVIDER_CONTEXT_TYPE"), "Anonymous": SimUnion({"keyingPolicy": SimTypePointer(SimStruct({"numKeyMods": SimTypeInt(signed=False, label="UInt32"), "keyModKeys": SimTypePointer(SimTypeBottom(label="Guid"), offset=0), "flags": SimTypeInt(signed=False, label="UInt32")}, name="IPSEC_KEYING_POLICY1", pack=False, align=None), offset=0), "ikeQmTransportPolicy": SimTypePointer(SimStruct({"numIpsecProposals": SimTypeInt(signed=False, label="UInt32"), "ipsecProposals": SimTypePointer(SimStruct({"lifetime": SimStruct({"lifetimeSeconds": SimTypeInt(signed=False, label="UInt32"), "lifetimeKilobytes": SimTypeInt(signed=False, label="UInt32"), "lifetimePackets": SimTypeInt(signed=False, label="UInt32")}, name="IPSEC_SA_LIFETIME0", pack=False, align=None), "numSaTransforms": SimTypeInt(signed=False, label="UInt32"), "saTransforms": SimTypePointer(SimStruct({"ipsecTransformType": SimTypeInt(signed=False, label="IPSEC_TRANSFORM_TYPE"), "Anonymous": SimUnion({"ahTransform": SimTypePointer(SimStruct({"authTransformId": SimStruct({"authType": SimTypeInt(signed=False, label="IPSEC_AUTH_TYPE"), "authConfig": SimTypeChar(label="Byte")}, name="IPSEC_AUTH_TRANSFORM_ID0", pack=False, align=None), "cryptoModuleId": SimTypePointer(SimTypeBottom(label="Guid"), offset=0)}, name="IPSEC_AUTH_TRANSFORM0", pack=False, align=None), offset=0), "espAuthTransform": SimTypePointer(SimStruct({"authTransformId": SimStruct({"authType": SimTypeInt(signed=False, label="IPSEC_AUTH_TYPE"), "authConfig": SimTypeChar(label="Byte")}, name="IPSEC_AUTH_TRANSFORM_ID0", pack=False, align=None), "cryptoModuleId": SimTypePointer(SimTypeBottom(label="Guid"), offset=0)}, name="IPSEC_AUTH_TRANSFORM0", pack=False, align=None), offset=0), "espCipherTransform": SimTypePointer(SimStruct({"cipherTransformId": SimStruct({"cipherType": SimTypeInt(signed=False, label="IPSEC_CIPHER_TYPE"), "cipherConfig": SimTypeChar(label="Byte")}, name="IPSEC_CIPHER_TRANSFORM_ID0", pack=False, align=None), "cryptoModuleId": SimTypePointer(SimTypeBottom(label="Guid"), offset=0)}, name="IPSEC_CIPHER_TRANSFORM0", pack=False, align=None), offset=0), "espAuthAndCipherTransform": SimTypePointer(SimStruct({"authTransform": SimStruct({"authTransformId": SimStruct({"authType": SimTypeInt(signed=False, label="IPSEC_AUTH_TYPE"), "authConfig": SimTypeChar(label="Byte")}, name="IPSEC_AUTH_TRANSFORM_ID0", pack=False, align=None), "cryptoModuleId": SimTypePointer(SimTypeBottom(label="Guid"), offset=0)}, name="IPSEC_AUTH_TRANSFORM0", pack=False, align=None), "cipherTransform": SimStruct({"cipherTransformId": SimStruct({"cipherType": SimTypeInt(signed=False, label="IPSEC_CIPHER_TYPE"), "cipherConfig": SimTypeChar(label="Byte")}, name="IPSEC_CIPHER_TRANSFORM_ID0", pack=False, align=None), "cryptoModuleId": SimTypePointer(SimTypeBottom(label="Guid"), offset=0)}, name="IPSEC_CIPHER_TRANSFORM0", pack=False, align=None)}, name="IPSEC_AUTH_AND_CIPHER_TRANSFORM0", pack=False, align=None), offset=0), "espAuthFwTransform": SimTypePointer(SimStruct({"authTransformId": SimStruct({"authType": SimTypeInt(signed=False, label="IPSEC_AUTH_TYPE"), "authConfig": SimTypeChar(label="Byte")}, name="IPSEC_AUTH_TRANSFORM_ID0", pack=False, align=None), "cryptoModuleId": SimTypePointer(SimTypeBottom(label="Guid"), offset=0)}, name="IPSEC_AUTH_TRANSFORM0", pack=False, align=None), offset=0)}, name="<anon>", label="None")}, name="IPSEC_SA_TRANSFORM0", pack=False, align=None), offset=0), "pfsGroup": SimTypeInt(signed=False, label="IPSEC_PFS_GROUP")}, name="IPSEC_PROPOSAL0", pack=False, align=None), offset=0), "flags": SimTypeInt(signed=False, label="IPSEC_POLICY_FLAG"), "ndAllowClearTimeoutSeconds": SimTypeInt(signed=False, label="UInt32"), "saIdleTimeout": SimStruct({"idleTimeoutSeconds": SimTypeInt(signed=False, label="UInt32"), "idleTimeoutSecondsFailOver": SimTypeInt(signed=False, label="UInt32")}, name="IPSEC_SA_IDLE_TIMEOUT0", pack=False, align=None), "emPolicy": SimTypePointer(SimStruct({"numAuthenticationMethods": SimTypeInt(signed=False, label="UInt32"), "authenticationMethods": SimTypePointer(SimStruct({"authenticationMethodType": SimTypeInt(signed=False, label="IKEEXT_AUTHENTICATION_METHOD_TYPE"), "Anonymous": SimUnion({"presharedKeyAuthentication": SimStruct({"presharedKey": SimStruct({"size": SimTypeInt(signed=False, label="UInt32"), "data": SimTypePointer(SimTypeChar(label="Byte"), offset=0)}, name="FWP_BYTE_BLOB", pack=False, align=None), "flags": SimTypeInt(signed=False, label="IKEEXT_PRESHARED_KEY_AUTHENTICATION_FLAGS")}, name="IKEEXT_PRESHARED_KEY_AUTHENTICATION1", pack=False, align=None), "certificateAuthentication": SimStruct({"inboundConfigType": SimTypeInt(signed=False, label="IKEEXT_CERT_CONFIG_TYPE"), "Anonymous1": SimUnion({"Anonymous1": SimStruct({"inboundRootArraySize": SimTypeInt(signed=False, label="UInt32"), "inboundRootCriteria": SimTypePointer(SimStruct({"certData": SimStruct({"size": SimTypeInt(signed=False, label="UInt32"), "data": SimTypePointer(SimTypeChar(label="Byte"), offset=0)}, name="FWP_BYTE_BLOB", pack=False, align=None), "certHash": SimStruct({"size": SimTypeInt(signed=False, label="UInt32"), "data": SimTypePointer(SimTypeChar(label="Byte"), offset=0)}, name="FWP_BYTE_BLOB", pack=False, align=None), "eku": SimTypePointer(SimStruct({"numEku": SimTypeInt(signed=False, label="UInt32"), "eku": SimTypePointer(SimTypePointer(SimTypeChar(label="Byte"), offset=0), offset=0)}, name="IKEEXT_CERT_EKUS0", pack=False, align=None), offset=0), "name": SimTypePointer(SimStruct({"nameType": SimTypeInt(signed=False, label="IKEEXT_CERT_CRITERIA_NAME_TYPE"), "certName": SimTypePointer(SimTypeChar(label="Char"), offset=0)}, name="IKEEXT_CERT_NAME0", pack=False, align=None), offset=0), "flags": SimTypeInt(signed=False, label="UInt32")}, name="IKEEXT_CERTIFICATE_CRITERIA0", pack=False, align=None), offset=0)}, name="_Anonymous1_e__Struct", pack=False, align=None), "Anonymous2": SimStruct({"inboundEnterpriseStoreArraySize": SimTypeInt(signed=False, label="UInt32"), "inboundEnterpriseStoreCriteria": SimTypePointer(SimStruct({"certData": SimStruct({"size": SimTypeInt(signed=False, label="UInt32"), "data": SimTypePointer(SimTypeChar(label="Byte"), offset=0)}, name="FWP_BYTE_BLOB", pack=False, align=None), "certHash": SimStruct({"size": SimTypeInt(signed=False, label="UInt32"), "data": SimTypePointer(SimTypeChar(label="Byte"), offset=0)}, name="FWP_BYTE_BLOB", pack=False, align=None), "eku": SimTypePointer(SimStruct({"numEku": SimTypeInt(signed=False, label="UInt32"), "eku": SimTypePointer(SimTypePointer(SimTypeChar(label="Byte"), offset=0), offset=0)}, name="IKEEXT_CERT_EKUS0", pack=False, align=None), offset=0), "name": SimTypePointer(SimStruct({"nameType": SimTypeInt(signed=False, label="IKEEXT_CERT_CRITERIA_NAME_TYPE"), "certName": SimTypePointer(SimTypeChar(label="Char"), offset=0)}, name="IKEEXT_CERT_NAME0", pack=False, align=None), offset=0), "flags": SimTypeInt(signed=False, label="UInt32")}, name="IKEEXT_CERTIFICATE_CRITERIA0", pack=False, align=None), offset=0)}, name="_Anonymous2_e__Struct", pack=False, align=None), "Anonymous3": SimStruct({"inboundRootStoreArraySize": SimTypeInt(signed=False, label="UInt32"), "inboundTrustedRootStoreCriteria": SimTypePointer(SimStruct({"certData": SimStruct({"size": SimTypeInt(signed=False, label="UInt32"), "data": SimTypePointer(SimTypeChar(label="Byte"), offset=0)}, name="FWP_BYTE_BLOB", pack=False, align=None), "certHash": SimStruct({"size": SimTypeInt(signed=False, label="UInt32"), "data": SimTypePointer(SimTypeChar(label="Byte"), offset=0)}, name="FWP_BYTE_BLOB", pack=False, align=None), "eku": SimTypePointer(SimStruct({"numEku": SimTypeInt(signed=False, label="UInt32"), "eku": SimTypePointer(SimTypePointer(SimTypeChar(label="Byte"), offset=0), offset=0)}, name="IKEEXT_CERT_EKUS0", pack=False, align=None), offset=0), "name": SimTypePointer(SimStruct({"nameType": SimTypeInt(signed=False, label="IKEEXT_CERT_CRITERIA_NAME_TYPE"), "certName": SimTypePointer(SimTypeChar(label="Char"), offset=0)}, name="IKEEXT_CERT_NAME0", pack=False, align=None), offset=0), "flags": SimTypeInt(signed=False, label="UInt32")}, name="IKEEXT_CERTIFICATE_CRITERIA0", pack=False, align=None), offset=0)}, name="_Anonymous3_e__Struct", pack=False, align=None)}, name="<anon>", label="None"), "outboundConfigType": SimTypeInt(signed=False, label="IKEEXT_CERT_CONFIG_TYPE"), "Anonymous2": SimUnion({"Anonymous1": SimStruct({"outboundRootArraySize": SimTypeInt(signed=False, label="UInt32"), "outboundRootCriteria": SimTypePointer(SimStruct({"certData": SimStruct({"size": SimTypeInt(signed=False, label="UInt32"), "data": SimTypePointer(SimTypeChar(label="Byte"), offset=0)}, name="FWP_BYTE_BLOB", pack=False, align=None), "certHash": SimStruct({"size": SimTypeInt(signed=False, label="UInt32"), "data": SimTypePointer(SimTypeChar(label="Byte"), offset=0)}, name="FWP_BYTE_BLOB", pack=False, align=None), "eku": SimTypePointer(SimStruct({"numEku": SimTypeInt(signed=False, label="UInt32"), "eku": SimTypePointer(SimTypePointer(SimTypeChar(label="Byte"), offset=0), offset=0)}, name="IKEEXT_CERT_EKUS0", pack=False, align=None), offset=0), "name": SimTypePointer(SimStruct({"nameType": SimTypeInt(signed=False, label="IKEEXT_CERT_CRITERIA_NAME_TYPE"), "certName": SimTypePointer(SimTypeChar(label="Char"), offset=0)}, name="IKEEXT_CERT_NAME0", pack=False, align=None), offset=0), "flags": SimTypeInt(signed=False, label="UInt32")}, name="IKEEXT_CERTIFICATE_CRITERIA0", pack=False, align=None), offset=0)}, name="_Anonymous1_e__Struct", pack=False, align=None), "Anonymous2": SimStruct({"outboundEnterpriseStoreArraySize": SimTypeInt(signed=False, label="UInt32"), "outboundEnterpriseStoreCriteria": SimTypePointer(SimStruct({"certData": SimStruct({"size": SimTypeInt(signed=False, label="UInt32"), "data": SimTypePointer(SimTypeChar(label="Byte"), offset=0)}, name="FWP_BYTE_BLOB", pack=False, align=None), "certHash": SimStruct({"size": SimTypeInt(signed=False, label="UInt32"), "data": SimTypePointer(SimTypeChar(label="Byte"), offset=0)}, name="FWP_BYTE_BLOB", pack=False, align=None), "eku": SimTypePointer(SimStruct({"numEku": SimTypeInt(signed=False, label="UInt32"), "eku": SimTypePointer(SimTypePointer(SimTypeChar(label="Byte"), offset=0), offset=0)}, name="IKEEXT_CERT_EKUS0", pack=False, align=None), offset=0), "name": SimTypePointer(SimStruct({"nameType": SimTypeInt(signed=False, label="IKEEXT_CERT_CRITERIA_NAME_TYPE"), "certName": SimTypePointer(SimTypeChar(label="Char"), offset=0)}, name="IKEEXT_CERT_NAME0", pack=False, align=None), offset=0), "flags": SimTypeInt(signed=False, label="UInt32")}, name="IKEEXT_CERTIFICATE_CRITERIA0", pack=False, align=None), offset=0)}, name="_Anonymous2_e__Struct", pack=False, align=None), "Anonymous3": SimStruct({"outboundRootStoreArraySize": SimTypeInt(signed=False, label="UInt32"), "outboundTrustedRootStoreCriteria": SimTypePointer(SimStruct({"certData": SimStruct({"size": SimTypeInt(signed=False, label="UInt32"), "data": SimTypePointer(SimTypeChar(label="Byte"), offset=0)}, name="FWP_BYTE_BLOB", pack=False, align=None), "certHash": SimStruct({"size": SimTypeInt(signed=False, label="UInt32"), "data": SimTypePointer(SimTypeChar(label="Byte"), offset=0)}, name="FWP_BYTE_BLOB", pack=False, align=None), "eku": SimTypePointer(SimStruct({"numEku": SimTypeInt(signed=False, label="UInt32"), "eku": SimTypePointer(SimTypePointer(SimTypeChar(label="Byte"), offset=0), offset=0)}, name="IKEEXT_CERT_EKUS0", pack=False, align=None), offset=0), "name": SimTypePointer(SimStruct({"nameType": SimTypeInt(signed=False, label="IKEEXT_CERT_CRITERIA_NAME_TYPE"), "certName": SimTypePointer(SimTypeChar(label="Char"), offset=0)}, name="IKEEXT_CERT_NAME0", pack=False, align=None), offset=0), "flags": SimTypeInt(signed=False, label="UInt32")}, name="IKEEXT_CERTIFICATE_CRITERIA0", pack=False, align=None), offset=0)}, name="_Anonymous3_e__Struct", pack=False, align=None)}, name="<anon>", label="None"), "flags": SimTypeInt(signed=False, label="IKEEXT_CERT_AUTH"), "localCertLocationUrl": SimStruct({"size": SimTypeInt(signed=False, label="UInt32"), "data": SimTypePointer(SimTypeChar(label="Byte"), offset=0)}, name="FWP_BYTE_BLOB", pack=False, align=None)}, name="IKEEXT_CERTIFICATE_AUTHENTICATION2", pack=False, align=None), "kerberosAuthentication": SimStruct({"flags": SimTypeInt(signed=False, label="IKEEXT_KERBEROS_AUTHENTICATION_FLAGS"), "proxyServer": SimTypePointer(SimTypeChar(label="Char"), offset=0)}, name="IKEEXT_KERBEROS_AUTHENTICATION1", pack=False, align=None), "reservedAuthentication": SimStruct({"flags": SimTypeInt(signed=False, label="IKEEXT_RESERVED_AUTHENTICATION_FLAGS")}, name="IKEEXT_RESERVED_AUTHENTICATION0", pack=False, align=None), "ntlmV2Authentication": SimStruct({"flags": SimTypeInt(signed=False, label="UInt32")}, name="IKEEXT_NTLM_V2_AUTHENTICATION0", pack=False, align=None), "sslAuthentication": SimStruct({"inboundConfigType": SimTypeInt(signed=False, label="IKEEXT_CERT_CONFIG_TYPE"), "Anonymous1": SimUnion({"Anonymous1": SimStruct({"inboundRootArraySize": SimTypeInt(signed=False, label="UInt32"), "inboundRootCriteria": SimTypePointer(SimStruct({"certData": SimStruct({"size": SimTypeInt(signed=False, label="UInt32"), "data": SimTypePointer(SimTypeChar(label="Byte"), offset=0)}, name="FWP_BYTE_BLOB", pack=False, align=None), "certHash": SimStruct({"size": SimTypeInt(signed=False, label="UInt32"), "data": SimTypePointer(SimTypeChar(label="Byte"), offset=0)}, name="FWP_BYTE_BLOB", pack=False, align=None), "eku": SimTypePointer(SimStruct({"numEku": SimTypeInt(signed=False, label="UInt32"), "eku": SimTypePointer(SimTypePointer(SimTypeChar(label="Byte"), offset=0), offset=0)}, name="IKEEXT_CERT_EKUS0", pack=False, align=None), offset=0), "name": SimTypePointer(SimStruct({"nameType": SimTypeInt(signed=False, label="IKEEXT_CERT_CRITERIA_NAME_TYPE"), "certName": SimTypePointer(SimTypeChar(label="Char"), offset=0)}, name="IKEEXT_CERT_NAME0", pack=False, align=None), offset=0), "flags": SimTypeInt(signed=False, label="UInt32")}, name="IKEEXT_CERTIFICATE_CRITERIA0", pack=False, align=None), offset=0)}, name="_Anonymous1_e__Struct", pack=False, align=None), "Anonymous2": SimStruct({"inboundEnterpriseStoreArraySize": SimTypeInt(signed=False, label="UInt32"), "inboundEnterpriseStoreCriteria": SimTypePointer(SimStruct({"certData": SimStruct({"size": SimTypeInt(signed=False, label="UInt32"), "data": SimTypePointer(SimTypeChar(label="Byte"), offset=0)}, name="FWP_BYTE_BLOB", pack=False, align=None), "certHash": SimStruct({"size": SimTypeInt(signed=False, label="UInt32"), "data": SimTypePointer(SimTypeChar(label="Byte"), offset=0)}, name="FWP_BYTE_BLOB", pack=False, align=None), "eku": SimTypePointer(SimStruct({"numEku": SimTypeInt(signed=False, label="UInt32"), "eku": SimTypePointer(SimTypePointer(SimTypeChar(label="Byte"), offset=0), offset=0)}, name="IKEEXT_CERT_EKUS0", pack=False, align=None), offset=0), "name": SimTypePointer(SimStruct({"nameType": SimTypeInt(signed=False, label="IKEEXT_CERT_CRITERIA_NAME_TYPE"), "certName": SimTypePointer(SimTypeChar(label="Char"), offset=0)}, name="IKEEXT_CERT_NAME0", pack=False, align=None), offset=0), "flags": SimTypeInt(signed=False, label="UInt32")}, name="IKEEXT_CERTIFICATE_CRITERIA0", pack=False, align=None), offset=0)}, name="_Anonymous2_e__Struct", pack=False, align=None), "Anonymous3": SimStruct({"inboundRootStoreArraySize": SimTypeInt(signed=False, label="UInt32"), "inboundTrustedRootStoreCriteria": SimTypePointer(SimStruct({"certData": SimStruct({"size": SimTypeInt(signed=False, label="UInt32"), "data": SimTypePointer(SimTypeChar(label="Byte"), offset=0)}, name="FWP_BYTE_BLOB", pack=False, align=None), "certHash": SimStruct({"size": SimTypeInt(signed=False, label="UInt32"), "data": SimTypePointer(SimTypeChar(label="Byte"), offset=0)}, name="FWP_BYTE_BLOB", pack=False, align=None), "eku": SimTypePointer(SimStruct({"numEku": SimTypeInt(signed=False, label="UInt32"), "eku": SimTypePointer(SimTypePointer(SimTypeChar(label="Byte"), offset=0), offset=0)}, name="IKEEXT_CERT_EKUS0", pack=False, align=None), offset=0), "name": SimTypePointer(SimStruct({"nameType": SimTypeInt(signed=False, label="IKEEXT_CERT_CRITERIA_NAME_TYPE"), "certName": SimTypePointer(SimTypeChar(label="Char"), offset=0)}, name="IKEEXT_CERT_NAME0", pack=False, align=None), offset=0), "flags": SimTypeInt(signed=False, label="UInt32")}, name="IKEEXT_CERTIFICATE_CRITERIA0", pack=False, align=None), offset=0)}, name="_Anonymous3_e__Struct", pack=False, align=None)}, name="<anon>", label="None"), "outboundConfigType": SimTypeInt(signed=False, label="IKEEXT_CERT_CONFIG_TYPE"), "Anonymous2": SimUnion({"Anonymous1": SimStruct({"outboundRootArraySize": SimTypeInt(signed=False, label="UInt32"), "outboundRootCriteria": SimTypePointer(SimStruct({"certData": SimStruct({"size": SimTypeInt(signed=False, label="UInt32"), "data": SimTypePointer(SimTypeChar(label="Byte"), offset=0)}, name="FWP_BYTE_BLOB", pack=False, align=None), "certHash": SimStruct({"size": SimTypeInt(signed=False, label="UInt32"), "data": SimTypePointer(SimTypeChar(label="Byte"), offset=0)}, name="FWP_BYTE_BLOB", pack=False, align=None), "eku": SimTypePointer(SimStruct({"numEku": SimTypeInt(signed=False, label="UInt32"), "eku": SimTypePointer(SimTypePointer(SimTypeChar(label="Byte"), offset=0), offset=0)}, name="IKEEXT_CERT_EKUS0", pack=False, align=None), offset=0), "name": SimTypePointer(SimStruct({"nameType": SimTypeInt(signed=False, label="IKEEXT_CERT_CRITERIA_NAME_TYPE"), "certName": SimTypePointer(SimTypeChar(label="Char"), offset=0)}, name="IKEEXT_CERT_NAME0", pack=False, align=None), offset=0), "flags": SimTypeInt(signed=False, label="UInt32")}, name="IKEEXT_CERTIFICATE_CRITERIA0", pack=False, align=None), offset=0)}, name="_Anonymous1_e__Struct", pack=False, align=None), "Anonymous2": SimStruct({"outboundEnterpriseStoreArraySize": SimTypeInt(signed=False, label="UInt32"), "outboundEnterpriseStoreCriteria": SimTypePointer(SimStruct({"certData": SimStruct({"size": SimTypeInt(signed=False, label="UInt32"), "data": SimTypePointer(SimTypeChar(label="Byte"), offset=0)}, name="FWP_BYTE_BLOB", pack=False, align=None), "certHash": SimStruct({"size": SimTypeInt(signed=False, label="UInt32"), "data": SimTypePointer(SimTypeChar(label="Byte"), offset=0)}, name="FWP_BYTE_BLOB", pack=False, align=None), "eku": SimTypePointer(SimStruct({"numEku": SimTypeInt(signed=False, label="UInt32"), "eku": SimTypePointer(SimTypePointer(SimTypeChar(label="Byte"), offset=0), offset=0)}, name="IKEEXT_CERT_EKUS0", pack=False, align=None), offset=0), "name": SimTypePointer(SimStruct({"nameType": SimTypeInt(signed=False, label="IKEEXT_CERT_CRITERIA_NAME_TYPE"), "certName": SimTypePointer(SimTypeChar(label="Char"), offset=0)}, name="IKEEXT_CERT_NAME0", pack=False, align=None), offset=0), "flags": SimTypeInt(signed=False, label="UInt32")}, name="IKEEXT_CERTIFICATE_CRITERIA0", pack=False, align=None), offset=0)}, name="_Anonymous2_e__Struct", pack=False, align=None), "Anonymous3": SimStruct({"outboundRootStoreArraySize": SimTypeInt(signed=False, label="UInt32"), "outboundTrustedRootStoreCriteria": SimTypePointer(SimStruct({"certData": SimStruct({"size": SimTypeInt(signed=False, label="UInt32"), "data": SimTypePointer(SimTypeChar(label="Byte"), offset=0)}, name="FWP_BYTE_BLOB", pack=False, align=None), "certHash": SimStruct({"size": SimTypeInt(signed=False, label="UInt32"), "data": SimTypePointer(SimTypeChar(label="Byte"), offset=0)}, name="FWP_BYTE_BLOB", pack=False, align=None), "eku": SimTypePointer(SimStruct({"numEku": SimTypeInt(signed=False, label="UInt32"), "eku": SimTypePointer(SimTypePointer(SimTypeChar(label="Byte"), offset=0), offset=0)}, name="IKEEXT_CERT_EKUS0", pack=False, align=None), offset=0), "name": SimTypePointer(SimStruct({"nameType": SimTypeInt(signed=False, label="IKEEXT_CERT_CRITERIA_NAME_TYPE"), "certName": SimTypePointer(SimTypeChar(label="Char"), offset=0)}, name="IKEEXT_CERT_NAME0", pack=False, align=None), offset=0), "flags": SimTypeInt(signed=False, label="UInt32")}, name="IKEEXT_CERTIFICATE_CRITERIA0", pack=False, align=None), offset=0)}, name="_Anonymous3_e__Struct", pack=False, align=None)}, name="<anon>", label="None"), "flags": SimTypeInt(signed=False, label="IKEEXT_CERT_AUTH"), "localCertLocationUrl": SimStruct({"size": SimTypeInt(signed=False, label="UInt32"), "data": SimTypePointer(SimTypeChar(label="Byte"), offset=0)}, name="FWP_BYTE_BLOB", pack=False, align=None)}, name="IKEEXT_CERTIFICATE_AUTHENTICATION2", pack=False, align=None), "cgaAuthentication": SimStruct({"keyContainerName": SimTypePointer(SimTypeChar(label="Char"), offset=0), "cspName": SimTypePointer(SimTypeChar(label="Char"), offset=0), "cspType": SimTypeInt(signed=False, label="UInt32"), "cgaModifier": SimStruct({"byteArray16": SimTypeFixedSizeArray(SimTypeChar(label="Byte"), 16)}, name="FWP_BYTE_ARRAY16", pack=False, align=None), "cgaCollisionCount": SimTypeChar(label="Byte")}, name="IKEEXT_IPV6_CGA_AUTHENTICATION0", pack=False, align=None), "eapAuthentication": SimStruct({"flags": SimTypeInt(signed=False, label="IKEEXT_EAP_AUTHENTICATION_FLAGS")}, name="IKEEXT_EAP_AUTHENTICATION0", pack=False, align=None)}, name="<anon>", label="None")}, name="IKEEXT_AUTHENTICATION_METHOD2", pack=False, align=None), offset=0), "initiatorImpersonationType": SimTypeInt(signed=False, label="IKEEXT_AUTHENTICATION_IMPERSONATION_TYPE")}, name="IKEEXT_EM_POLICY2", pack=False, align=None), offset=0)}, name="IPSEC_TRANSPORT_POLICY2", pack=False, align=None), offset=0), "ikeQmTunnelPolicy": SimTypePointer(SimStruct({"flags": SimTypeInt(signed=False, label="IPSEC_POLICY_FLAG"), "numIpsecProposals": SimTypeInt(signed=False, label="UInt32"), "ipsecProposals": SimTypePointer(SimStruct({"lifetime": SimStruct({"lifetimeSeconds": SimTypeInt(signed=False, label="UInt32"), "lifetimeKilobytes": SimTypeInt(signed=False, label="UInt32"), "lifetimePackets": SimTypeInt(signed=False, label="UInt32")}, name="IPSEC_SA_LIFETIME0", pack=False, align=None), "numSaTransforms": SimTypeInt(signed=False, label="UInt32"), "saTransforms": SimTypePointer(SimStruct({"ipsecTransformType": SimTypeInt(signed=False, label="IPSEC_TRANSFORM_TYPE"), "Anonymous": SimUnion({"ahTransform": SimTypePointer(SimStruct({"authTransformId": SimStruct({"authType": SimTypeInt(signed=False, label="IPSEC_AUTH_TYPE"), "authConfig": SimTypeChar(label="Byte")}, name="IPSEC_AUTH_TRANSFORM_ID0", pack=False, align=None), "cryptoModuleId": SimTypePointer(SimTypeBottom(label="Guid"), offset=0)}, name="IPSEC_AUTH_TRANSFORM0", pack=False, align=None), offset=0), "espAuthTransform": SimTypePointer(SimStruct({"authTransformId": SimStruct({"authType": SimTypeInt(signed=False, label="IPSEC_AUTH_TYPE"), "authConfig": SimTypeChar(label="Byte")}, name="IPSEC_AUTH_TRANSFORM_ID0", pack=False, align=None), "cryptoModuleId": SimTypePointer(SimTypeBottom(label="Guid"), offset=0)}, name="IPSEC_AUTH_TRANSFORM0", pack=False, align=None), offset=0), "espCipherTransform": SimTypePointer(SimStruct({"cipherTransformId": SimStruct({"cipherType": SimTypeInt(signed=False, label="IPSEC_CIPHER_TYPE"), "cipherConfig": SimTypeChar(label="Byte")}, name="IPSEC_CIPHER_TRANSFORM_ID0", pack=False, align=None), "cryptoModuleId": SimTypePointer(SimTypeBottom(label="Guid"), offset=0)}, name="IPSEC_CIPHER_TRANSFORM0", pack=False, align=None), offset=0), "espAuthAndCipherTransform": SimTypePointer(SimStruct({"authTransform": SimStruct({"authTransformId": SimStruct({"authType": SimTypeInt(signed=False, label="IPSEC_AUTH_TYPE"), "authConfig": SimTypeChar(label="Byte")}, name="IPSEC_AUTH_TRANSFORM_ID0", pack=False, align=None), "cryptoModuleId": SimTypePointer(SimTypeBottom(label="Guid"), offset=0)}, name="IPSEC_AUTH_TRANSFORM0", pack=False, align=None), "cipherTransform": SimStruct({"cipherTransformId": SimStruct({"cipherType": SimTypeInt(signed=False, label="IPSEC_CIPHER_TYPE"), "cipherConfig": SimTypeChar(label="Byte")}, name="IPSEC_CIPHER_TRANSFORM_ID0", pack=False, align=None), "cryptoModuleId": SimTypePointer(SimTypeBottom(label="Guid"), offset=0)}, name="IPSEC_CIPHER_TRANSFORM0", pack=False, align=None)}, name="IPSEC_AUTH_AND_CIPHER_TRANSFORM0", pack=False, align=None), offset=0), "espAuthFwTransform": SimTypePointer(SimStruct({"authTransformId": SimStruct({"authType": SimTypeInt(signed=False, label="IPSEC_AUTH_TYPE"), "authConfig": SimTypeChar(label="Byte")}, name="IPSEC_AUTH_TRANSFORM_ID0", pack=False, align=None), "cryptoModuleId": SimTypePointer(SimTypeBottom(label="Guid"), offset=0)}, name="IPSEC_AUTH_TRANSFORM0", pack=False, align=None), offset=0)}, name="<anon>", label="None")}, name="IPSEC_SA_TRANSFORM0", pack=False, align=None), offset=0), "pfsGroup": SimTypeInt(signed=False, label="IPSEC_PFS_GROUP")}, name="IPSEC_PROPOSAL0", pack=False, align=None), offset=0), "tunnelEndpoints": SimStruct({"ipVersion": SimTypeInt(signed=False, label="FWP_IP_VERSION"), "Anonymous1": SimUnion({"localV4Address": SimTypeInt(signed=False, label="UInt32"), "localV6Address": SimTypeFixedSizeArray(SimTypeChar(label="Byte"), 16)}, name="<anon>", label="None"), "Anonymous2": SimUnion({"remoteV4Address": SimTypeInt(signed=False, label="UInt32"), "remoteV6Address": SimTypeFixedSizeArray(SimTypeChar(label="Byte"), 16)}, name="<anon>", label="None"), "localIfLuid": SimTypeLongLong(signed=False, label="UInt64"), "remoteFqdn": SimTypePointer(SimTypeChar(label="Char"), offset=0), "numAddresses": SimTypeInt(signed=False, label="UInt32"), "remoteAddresses": SimTypePointer(SimStruct({"ipVersion": SimTypeInt(signed=False, label="FWP_IP_VERSION"), "Anonymous": SimUnion({"v4Address": SimTypeInt(signed=False, label="UInt32"), "v6Address": SimTypeFixedSizeArray(SimTypeChar(label="Byte"), 16)}, name="<anon>", label="None")}, name="IPSEC_TUNNEL_ENDPOINT0", pack=False, align=None), offset=0)}, name="IPSEC_TUNNEL_ENDPOINTS2", pack=False, align=None), "saIdleTimeout": SimStruct({"idleTimeoutSeconds": SimTypeInt(signed=False, label="UInt32"), "idleTimeoutSecondsFailOver": SimTypeInt(signed=False, label="UInt32")}, name="IPSEC_SA_IDLE_TIMEOUT0", pack=False, align=None), "emPolicy": SimTypePointer(SimStruct({"numAuthenticationMethods": SimTypeInt(signed=False, label="UInt32"), "authenticationMethods": SimTypePointer(SimStruct({"authenticationMethodType": SimTypeInt(signed=False, label="IKEEXT_AUTHENTICATION_METHOD_TYPE"), "Anonymous": SimUnion({"presharedKeyAuthentication": SimStruct({"presharedKey": SimStruct({"size": SimTypeInt(signed=False, label="UInt32"), "data": SimTypePointer(SimTypeChar(label="Byte"), offset=0)}, name="FWP_BYTE_BLOB", pack=False, align=None), "flags": SimTypeInt(signed=False, label="IKEEXT_PRESHARED_KEY_AUTHENTICATION_FLAGS")}, name="IKEEXT_PRESHARED_KEY_AUTHENTICATION1", pack=False, align=None), "certificateAuthentication": SimStruct({"inboundConfigType": SimTypeInt(signed=False, label="IKEEXT_CERT_CONFIG_TYPE"), "Anonymous1": SimUnion({"Anonymous1": SimStruct({"inboundRootArraySize": SimTypeInt(signed=False, label="UInt32"), "inboundRootCriteria": SimTypePointer(SimStruct({"certData": SimStruct({"size": SimTypeInt(signed=False, label="UInt32"), "data": SimTypePointer(SimTypeChar(label="Byte"), offset=0)}, name="FWP_BYTE_BLOB", pack=False, align=None), "certHash": SimStruct({"size": SimTypeInt(signed=False, label="UInt32"), "data": SimTypePointer(SimTypeChar(label="Byte"), offset=0)}, name="FWP_BYTE_BLOB", pack=False, align=None), "eku": SimTypePointer(SimStruct({"numEku": SimTypeInt(signed=False, label="UInt32"), "eku": SimTypePointer(SimTypePointer(SimTypeChar(label="Byte"), offset=0), offset=0)}, name="IKEEXT_CERT_EKUS0", pack=False, align=None), offset=0), "name": SimTypePointer(SimStruct({"nameType": SimTypeInt(signed=False, label="IKEEXT_CERT_CRITERIA_NAME_TYPE"), "certName": SimTypePointer(SimTypeChar(label="Char"), offset=0)}, name="IKEEXT_CERT_NAME0", pack=False, align=None), offset=0), "flags": SimTypeInt(signed=False, label="UInt32")}, name="IKEEXT_CERTIFICATE_CRITERIA0", pack=False, align=None), offset=0)}, name="_Anonymous1_e__Struct", pack=False, align=None), "Anonymous2": SimStruct({"inboundEnterpriseStoreArraySize": SimTypeInt(signed=False, label="UInt32"), "inboundEnterpriseStoreCriteria": SimTypePointer(SimStruct({"certData": SimStruct({"size": SimTypeInt(signed=False, label="UInt32"), "data": SimTypePointer(SimTypeChar(label="Byte"), offset=0)}, name="FWP_BYTE_BLOB", pack=False, align=None), "certHash": SimStruct({"size": SimTypeInt(signed=False, label="UInt32"), "data": SimTypePointer(SimTypeChar(label="Byte"), offset=0)}, name="FWP_BYTE_BLOB", pack=False, align=None), "eku": SimTypePointer(SimStruct({"numEku": SimTypeInt(signed=False, label="UInt32"), "eku": SimTypePointer(SimTypePointer(SimTypeChar(label="Byte"), offset=0), offset=0)}, name="IKEEXT_CERT_EKUS0", pack=False, align=None), offset=0), "name": SimTypePointer(SimStruct({"nameType": SimTypeInt(signed=False, label="IKEEXT_CERT_CRITERIA_NAME_TYPE"), "certName": SimTypePointer(SimTypeChar(label="Char"), offset=0)}, name="IKEEXT_CERT_NAME0", pack=False, align=None), offset=0), "flags": SimTypeInt(signed=False, label="UInt32")}, name="IKEEXT_CERTIFICATE_CRITERIA0", pack=False, align=None), offset=0)}, name="_Anonymous2_e__Struct", pack=False, align=None), "Anonymous3": SimStruct({"inboundRootStoreArraySize": SimTypeInt(signed=False, label="UInt32"), "inboundTrustedRootStoreCriteria": SimTypePointer(SimStruct({"certData": SimStruct({"size": SimTypeInt(signed=False, label="UInt32"), "data": SimTypePointer(SimTypeChar(label="Byte"), offset=0)}, name="FWP_BYTE_BLOB", pack=False, align=None), "certHash": SimStruct({"size": SimTypeInt(signed=False, label="UInt32"), "data": SimTypePointer(SimTypeChar(label="Byte"), offset=0)}, name="FWP_BYTE_BLOB", pack=False, align=None), "eku": SimTypePointer(SimStruct({"numEku": SimTypeInt(signed=False, label="UInt32"), "eku": SimTypePointer(SimTypePointer(SimTypeChar(label="Byte"), offset=0), offset=0)}, name="IKEEXT_CERT_EKUS0", pack=False, align=None), offset=0), "name": SimTypePointer(SimStruct({"nameType": SimTypeInt(signed=False, label="IKEEXT_CERT_CRITERIA_NAME_TYPE"), "certName": SimTypePointer(SimTypeChar(label="Char"), offset=0)}, name="IKEEXT_CERT_NAME0", pack=False, align=None), offset=0), "flags": SimTypeInt(signed=False, label="UInt32")}, name="IKEEXT_CERTIFICATE_CRITERIA0", pack=False, align=None), offset=0)}, name="_Anonymous3_e__Struct", pack=False, align=None)}, name="<anon>", label="None"), "outboundConfigType": SimTypeInt(signed=False, label="IKEEXT_CERT_CONFIG_TYPE"), "Anonymous2": SimUnion({"Anonymous1": SimStruct({"outboundRootArraySize": SimTypeInt(signed=False, label="UInt32"), "outboundRootCriteria": SimTypePointer(SimStruct({"certData": SimStruct({"size": SimTypeInt(signed=False, label="UInt32"), "data": SimTypePointer(SimTypeChar(label="Byte"), offset=0)}, name="FWP_BYTE_BLOB", pack=False, align=None), "certHash": SimStruct({"size": SimTypeInt(signed=False, label="UInt32"), "data": SimTypePointer(SimTypeChar(label="Byte"), offset=0)}, name="FWP_BYTE_BLOB", pack=False, align=None), "eku": SimTypePointer(SimStruct({"numEku": SimTypeInt(signed=False, label="UInt32"), "eku": SimTypePointer(SimTypePointer(SimTypeChar(label="Byte"), offset=0), offset=0)}, name="IKEEXT_CERT_EKUS0", pack=False, align=None), offset=0), "name": SimTypePointer(SimStruct({"nameType": SimTypeInt(signed=False, label="IKEEXT_CERT_CRITERIA_NAME_TYPE"), "certName": SimTypePointer(SimTypeChar(label="Char"), offset=0)}, name="IKEEXT_CERT_NAME0", pack=False, align=None), offset=0), "flags": SimTypeInt(signed=False, label="UInt32")}, name="IKEEXT_CERTIFICATE_CRITERIA0", pack=False, align=None), offset=0)}, name="_Anonymous1_e__Struct", pack=False, align=None), "Anonymous2": SimStruct({"outboundEnterpriseStoreArraySize": SimTypeInt(signed=False, label="UInt32"), "outboundEnterpriseStoreCriteria": SimTypePointer(SimStruct({"certData": SimStruct({"size": SimTypeInt(signed=False, label="UInt32"), "data": SimTypePointer(SimTypeChar(label="Byte"), offset=0)}, name="FWP_BYTE_BLOB", pack=False, align=None), "certHash": SimStruct({"size": SimTypeInt(signed=False, label="UInt32"), "data": SimTypePointer(SimTypeChar(label="Byte"), offset=0)}, name="FWP_BYTE_BLOB", pack=False, align=None), "eku": SimTypePointer(SimStruct({"numEku": SimTypeInt(signed=False, label="UInt32"), "eku": SimTypePointer(SimTypePointer(SimTypeChar(label="Byte"), offset=0), offset=0)}, name="IKEEXT_CERT_EKUS0", pack=False, align=None), offset=0), "name": SimTypePointer(SimStruct({"nameType": SimTypeInt(signed=False, label="IKEEXT_CERT_CRITERIA_NAME_TYPE"), "certName": SimTypePointer(SimTypeChar(label="Char"), offset=0)}, name="IKEEXT_CERT_NAME0", pack=False, align=None), offset=0), "flags": SimTypeInt(signed=False, label="UInt32")}, name="IKEEXT_CERTIFICATE_CRITERIA0", pack=False, align=None), offset=0)}, name="_Anonymous2_e__Struct", pack=False, align=None), "Anonymous3": SimStruct({"outboundRootStoreArraySize": SimTypeInt(signed=False, label="UInt32"), "outboundTrustedRootStoreCriteria": SimTypePointer(SimStruct({"certData": SimStruct({"size": SimTypeInt(signed=False, label="UInt32"), "data": SimTypePointer(SimTypeChar(label="Byte"), offset=0)}, name="FWP_BYTE_BLOB", pack=False, align=None), "certHash": SimStruct({"size": SimTypeInt(signed=False, label="UInt32"), "data": SimTypePointer(SimTypeChar(label="Byte"), offset=0)}, name="FWP_BYTE_BLOB", pack=False, align=None), "eku": SimTypePointer(SimStruct({"numEku": SimTypeInt(signed=False, label="UInt32"), "eku": SimTypePointer(SimTypePointer(SimTypeChar(label="Byte"), offset=0), offset=0)}, name="IKEEXT_CERT_EKUS0", pack=False, align=None), offset=0), "name": SimTypePointer(SimStruct({"nameType": SimTypeInt(signed=False, label="IKEEXT_CERT_CRITERIA_NAME_TYPE"), "certName": SimTypePointer(SimTypeChar(label="Char"), offset=0)}, name="IKEEXT_CERT_NAME0", pack=False, align=None), offset=0), "flags": SimTypeInt(signed=False, label="UInt32")}, name="IKEEXT_CERTIFICATE_CRITERIA0", pack=False, align=None), offset=0)}, name="_Anonymous3_e__Struct", pack=False, align=None)}, name="<anon>", label="None"), "flags": SimTypeInt(signed=False, label="IKEEXT_CERT_AUTH"), "localCertLocationUrl": SimStruct({"size": SimTypeInt(signed=False, label="UInt32"), "data": SimTypePointer(SimTypeChar(label="Byte"), offset=0)}, name="FWP_BYTE_BLOB", pack=False, align=None)}, name="IKEEXT_CERTIFICATE_AUTHENTICATION2", pack=False, align=None), "kerberosAuthentication": SimStruct({"flags": SimTypeInt(signed=False, label="IKEEXT_KERBEROS_AUTHENTICATION_FLAGS"), "proxyServer": SimTypePointer(SimTypeChar(label="Char"), offset=0)}, name="IKEEXT_KERBEROS_AUTHENTICATION1", pack=False, align=None), "reservedAuthentication": SimStruct({"flags": SimTypeInt(signed=False, label="IKEEXT_RESERVED_AUTHENTICATION_FLAGS")}, name="IKEEXT_RESERVED_AUTHENTICATION0", pack=False, align=None), "ntlmV2Authentication": SimStruct({"flags": SimTypeInt(signed=False, label="UInt32")}, name="IKEEXT_NTLM_V2_AUTHENTICATION0", pack=False, align=None), "sslAuthentication": SimStruct({"inboundConfigType": SimTypeInt(signed=False, label="IKEEXT_CERT_CONFIG_TYPE"), "Anonymous1": SimUnion({"Anonymous1": SimStruct({"inboundRootArraySize": SimTypeInt(signed=False, label="UInt32"), "inboundRootCriteria": SimTypePointer(SimStruct({"certData": SimStruct({"size": SimTypeInt(signed=False, label="UInt32"), "data": SimTypePointer(SimTypeChar(label="Byte"), offset=0)}, name="FWP_BYTE_BLOB", pack=False, align=None), "certHash": SimStruct({"size": SimTypeInt(signed=False, label="UInt32"), "data": SimTypePointer(SimTypeChar(label="Byte"), offset=0)}, name="FWP_BYTE_BLOB", pack=False, align=None), "eku": SimTypePointer(SimStruct({"numEku": SimTypeInt(signed=False, label="UInt32"), "eku": SimTypePointer(SimTypePointer(SimTypeChar(label="Byte"), offset=0), offset=0)}, name="IKEEXT_CERT_EKUS0", pack=False, align=None), offset=0), "name": SimTypePointer(SimStruct({"nameType": SimTypeInt(signed=False, label="IKEEXT_CERT_CRITERIA_NAME_TYPE"), "certName": SimTypePointer(SimTypeChar(label="Char"), offset=0)}, name="IKEEXT_CERT_NAME0", pack=False, align=None), offset=0), "flags": SimTypeInt(signed=False, label="UInt32")}, name="IKEEXT_CERTIFICATE_CRITERIA0", pack=False, align=None), offset=0)}, name="_Anonymous1_e__Struct", pack=False, align=None), "Anonymous2": SimStruct({"inboundEnterpriseStoreArraySize": SimTypeInt(signed=False, label="UInt32"), "inboundEnterpriseStoreCriteria": SimTypePointer(SimStruct({"certData": SimStruct({"size": SimTypeInt(signed=False, label="UInt32"), "data": SimTypePointer(SimTypeChar(label="Byte"), offset=0)}, name="FWP_BYTE_BLOB", pack=False, align=None), "certHash": SimStruct({"size": SimTypeInt(signed=False, label="UInt32"), "data": SimTypePointer(SimTypeChar(label="Byte"), offset=0)}, name="FWP_BYTE_BLOB", pack=False, align=None), "eku": SimTypePointer(SimStruct({"numEku": SimTypeInt(signed=False, label="UInt32"), "eku": SimTypePointer(SimTypePointer(SimTypeChar(label="Byte"), offset=0), offset=0)}, name="IKEEXT_CERT_EKUS0", pack=False, align=None), offset=0), "name": SimTypePointer(SimStruct({"nameType": SimTypeInt(signed=False, label="IKEEXT_CERT_CRITERIA_NAME_TYPE"), "certName": SimTypePointer(SimTypeChar(label="Char"), offset=0)}, name="IKEEXT_CERT_NAME0", pack=False, align=None), offset=0), "flags": SimTypeInt(signed=False, label="UInt32")}, name="IKEEXT_CERTIFICATE_CRITERIA0", pack=False, align=None), offset=0)}, name="_Anonymous2_e__Struct", pack=False, align=None), "Anonymous3": SimStruct({"inboundRootStoreArraySize": SimTypeInt(signed=False, label="UInt32"), "inboundTrustedRootStoreCriteria": SimTypePointer(SimStruct({"certData": SimStruct({"size": SimTypeInt(signed=False, label="UInt32"), "data": SimTypePointer(SimTypeChar(label="Byte"), offset=0)}, name="FWP_BYTE_BLOB", pack=False, align=None), "certHash": SimStruct({"size": SimTypeInt(signed=False, label="UInt32"), "data": SimTypePointer(SimTypeChar(label="Byte"), offset=0)}, name="FWP_BYTE_BLOB", pack=False, align=None), "eku": SimTypePointer(SimStruct({"numEku": SimTypeInt(signed=False, label="UInt32"), "eku": SimTypePointer(SimTypePointer(SimTypeChar(label="Byte"), offset=0), offset=0)}, name="IKEEXT_CERT_EKUS0", pack=False, align=None), offset=0), "name": SimTypePointer(SimStruct({"nameType": SimTypeInt(signed=False, label="IKEEXT_CERT_CRITERIA_NAME_TYPE"), "certName": SimTypePointer(SimTypeChar(label="Char"), offset=0)}, name="IKEEXT_CERT_NAME0", pack=False, align=None), offset=0), "flags": SimTypeInt(signed=False, label="UInt32")}, name="IKEEXT_CERTIFICATE_CRITERIA0", pack=False, align=None), offset=0)}, name="_Anonymous3_e__Struct", pack=False, align=None)}, name="<anon>", label="None"), "outboundConfigType": SimTypeInt(signed=False, label="IKEEXT_CERT_CONFIG_TYPE"), "Anonymous2": SimUnion({"Anonymous1": SimStruct({"outboundRootArraySize": SimTypeInt(signed=False, label="UInt32"), "outboundRootCriteria": SimTypePointer(SimStruct({"certData": SimStruct({"size": SimTypeInt(signed=False, label="UInt32"), "data": SimTypePointer(SimTypeChar(label="Byte"), offset=0)}, name="FWP_BYTE_BLOB", pack=False, align=None), "certHash": SimStruct({"size": SimTypeInt(signed=False, label="UInt32"), "data": SimTypePointer(SimTypeChar(label="Byte"), offset=0)}, name="FWP_BYTE_BLOB", pack=False, align=None), "eku": SimTypePointer(SimStruct({"numEku": SimTypeInt(signed=False, label="UInt32"), "eku": SimTypePointer(SimTypePointer(SimTypeChar(label="Byte"), offset=0), offset=0)}, name="IKEEXT_CERT_EKUS0", pack=False, align=None), offset=0), "name": SimTypePointer(SimStruct({"nameType": SimTypeInt(signed=False, label="IKEEXT_CERT_CRITERIA_NAME_TYPE"), "certName": SimTypePointer(SimTypeChar(label="Char"), offset=0)}, name="IKEEXT_CERT_NAME0", pack=False, align=None), offset=0), "flags": SimTypeInt(signed=False, label="UInt32")}, name="IKEEXT_CERTIFICATE_CRITERIA0", pack=False, align=None), offset=0)}, name="_Anonymous1_e__Struct", pack=False, align=None), "Anonymous2": SimStruct({"outboundEnterpriseStoreArraySize": SimTypeInt(signed=False, label="UInt32"), "outboundEnterpriseStoreCriteria": SimTypePointer(SimStruct({"certData": SimStruct({"size": SimTypeInt(signed=False, label="UInt32"), "data": SimTypePointer(SimTypeChar(label="Byte"), offset=0)}, name="FWP_BYTE_BLOB", pack=False, align=None), "certHash": SimStruct({"size": SimTypeInt(signed=False, label="UInt32"), "data": SimTypePointer(SimTypeChar(label="Byte"), offset=0)}, name="FWP_BYTE_BLOB", pack=False, align=None), "eku": SimTypePointer(SimStruct({"numEku": SimTypeInt(signed=False, label="UInt32"), "eku": SimTypePointer(SimTypePointer(SimTypeChar(label="Byte"), offset=0), offset=0)}, name="IKEEXT_CERT_EKUS0", pack=False, align=None), offset=0), "name": SimTypePointer(SimStruct({"nameType": SimTypeInt(signed=False, label="IKEEXT_CERT_CRITERIA_NAME_TYPE"), "certName": SimTypePointer(SimTypeChar(label="Char"), offset=0)}, name="IKEEXT_CERT_NAME0", pack=False, align=None), offset=0), "flags": SimTypeInt(signed=False, label="UInt32")}, name="IKEEXT_CERTIFICATE_CRITERIA0", pack=False, align=None), offset=0)}, name="_Anonymous2_e__Struct", pack=False, align=None), "Anonymous3": SimStruct({"outboundRootStoreArraySize": SimTypeInt(signed=False, label="UInt32"), "outboundTrustedRootStoreCriteria": SimTypePointer(SimStruct({"certData": SimStruct({"size": SimTypeInt(signed=False, label="UInt32"), "data": SimTypePointer(SimTypeChar(label="Byte"), offset=0)}, name="FWP_BYTE_BLOB", pack=False, align=None), "certHash": SimStruct({"size": SimTypeInt(signed=False, label="UInt32"), "data": SimTypePointer(SimTypeChar(label="Byte"), offset=0)}, name="FWP_BYTE_BLOB", pack=False, align=None), "eku": SimTypePointer(SimStruct({"numEku": SimTypeInt(signed=False, label="UInt32"), "eku": SimTypePointer(SimTypePointer(SimTypeChar(label="Byte"), offset=0), offset=0)}, name="IKEEXT_CERT_EKUS0", pack=False, align=None), offset=0), "name": SimTypePointer(SimStruct({"nameType": SimTypeInt(signed=False, label="IKEEXT_CERT_CRITERIA_NAME_TYPE"), "certName": SimTypePointer(SimTypeChar(label="Char"), offset=0)}, name="IKEEXT_CERT_NAME0", pack=False, align=None), offset=0), "flags": SimTypeInt(signed=False, label="UInt32")}, name="IKEEXT_CERTIFICATE_CRITERIA0", pack=False, align=None), offset=0)}, name="_Anonymous3_e__Struct", pack=False, align=None)}, name="<anon>", label="None"), "flags": SimTypeInt(signed=False, label="IKEEXT_CERT_AUTH"), "localCertLocationUrl": SimStruct({"size": SimTypeInt(signed=False, label="UInt32"), "data": SimTypePointer(SimTypeChar(label="Byte"), offset=0)}, name="FWP_BYTE_BLOB", pack=False, align=None)}, name="IKEEXT_CERTIFICATE_AUTHENTICATION2", pack=False, align=None), "cgaAuthentication": SimStruct({"keyContainerName": SimTypePointer(SimTypeChar(label="Char"), offset=0), "cspName": SimTypePointer(SimTypeChar(label="Char"), offset=0), "cspType": SimTypeInt(signed=False, label="UInt32"), "cgaModifier": SimStruct({"byteArray16": SimTypeFixedSizeArray(SimTypeChar(label="Byte"), 16)}, name="FWP_BYTE_ARRAY16", pack=False, align=None), "cgaCollisionCount": SimTypeChar(label="Byte")}, name="IKEEXT_IPV6_CGA_AUTHENTICATION0", pack=False, align=None), "eapAuthentication": SimStruct({"flags": SimTypeInt(signed=False, label="IKEEXT_EAP_AUTHENTICATION_FLAGS")}, name="IKEEXT_EAP_AUTHENTICATION0", pack=False, align=None)}, name="<anon>", label="None")}, name="IKEEXT_AUTHENTICATION_METHOD2", pack=False, align=None), offset=0), "initiatorImpersonationType": SimTypeInt(signed=False, label="IKEEXT_AUTHENTICATION_IMPERSONATION_TYPE")}, name="IKEEXT_EM_POLICY2", pack=False, align=None), offset=0), "fwdPathSaLifetime": SimTypeInt(signed=False, label="UInt32")}, name="IPSEC_TUNNEL_POLICY2", pack=False, align=None), offset=0), "authipQmTransportPolicy": SimTypePointer(SimStruct({"numIpsecProposals": SimTypeInt(signed=False, label="UInt32"), "ipsecProposals": SimTypePointer(SimStruct({"lifetime": SimStruct({"lifetimeSeconds": SimTypeInt(signed=False, label="UInt32"), "lifetimeKilobytes": SimTypeInt(signed=False, label="UInt32"), "lifetimePackets": SimTypeInt(signed=False, label="UInt32")}, name="IPSEC_SA_LIFETIME0", pack=False, align=None), "numSaTransforms": SimTypeInt(signed=False, label="UInt32"), "saTransforms": SimTypePointer(SimStruct({"ipsecTransformType": SimTypeInt(signed=False, label="IPSEC_TRANSFORM_TYPE"), "Anonymous": SimUnion({"ahTransform": SimTypePointer(SimStruct({"authTransformId": SimStruct({"authType": SimTypeInt(signed=False, label="IPSEC_AUTH_TYPE"), "authConfig": SimTypeChar(label="Byte")}, name="IPSEC_AUTH_TRANSFORM_ID0", pack=False, align=None), "cryptoModuleId": SimTypePointer(SimTypeBottom(label="Guid"), offset=0)}, name="IPSEC_AUTH_TRANSFORM0", pack=False, align=None), offset=0), "espAuthTransform": SimTypePointer(SimStruct({"authTransformId": SimStruct({"authType": SimTypeInt(signed=False, label="IPSEC_AUTH_TYPE"), "authConfig": SimTypeChar(label="Byte")}, name="IPSEC_AUTH_TRANSFORM_ID0", pack=False, align=None), "cryptoModuleId": SimTypePointer(SimTypeBottom(label="Guid"), offset=0)}, name="IPSEC_AUTH_TRANSFORM0", pack=False, align=None), offset=0), "espCipherTransform": SimTypePointer(SimStruct({"cipherTransformId": SimStruct({"cipherType": SimTypeInt(signed=False, label="IPSEC_CIPHER_TYPE"), "cipherConfig": SimTypeChar(label="Byte")}, name="IPSEC_CIPHER_TRANSFORM_ID0", pack=False, align=None), "cryptoModuleId": SimTypePointer(SimTypeBottom(label="Guid"), offset=0)}, name="IPSEC_CIPHER_TRANSFORM0", pack=False, align=None), offset=0), "espAuthAndCipherTransform": SimTypePointer(SimStruct({"authTransform": SimStruct({"authTransformId": SimStruct({"authType": SimTypeInt(signed=False, label="IPSEC_AUTH_TYPE"), "authConfig": SimTypeChar(label="Byte")}, name="IPSEC_AUTH_TRANSFORM_ID0", pack=False, align=None), "cryptoModuleId": SimTypePointer(SimTypeBottom(label="Guid"), offset=0)}, name="IPSEC_AUTH_TRANSFORM0", pack=False, align=None), "cipherTransform": SimStruct({"cipherTransformId": SimStruct({"cipherType": SimTypeInt(signed=False, label="IPSEC_CIPHER_TYPE"), "cipherConfig": SimTypeChar(label="Byte")}, name="IPSEC_CIPHER_TRANSFORM_ID0", pack=False, align=None), "cryptoModuleId": SimTypePointer(SimTypeBottom(label="Guid"), offset=0)}, name="IPSEC_CIPHER_TRANSFORM0", pack=False, align=None)}, name="IPSEC_AUTH_AND_CIPHER_TRANSFORM0", pack=False, align=None), offset=0), "espAuthFwTransform": SimTypePointer(SimStruct({"authTransformId": SimStruct({"authType": SimTypeInt(signed=False, label="IPSEC_AUTH_TYPE"), "authConfig": SimTypeChar(label="Byte")}, name="IPSEC_AUTH_TRANSFORM_ID0", pack=False, align=None), "cryptoModuleId": SimTypePointer(SimTypeBottom(label="Guid"), offset=0)}, name="IPSEC_AUTH_TRANSFORM0", pack=False, align=None), offset=0)}, name="<anon>", label="None")}, name="IPSEC_SA_TRANSFORM0", pack=False, align=None), offset=0), "pfsGroup": SimTypeInt(signed=False, label="IPSEC_PFS_GROUP")}, name="IPSEC_PROPOSAL0", pack=False, align=None), offset=0), "flags": SimTypeInt(signed=False, label="IPSEC_POLICY_FLAG"), "ndAllowClearTimeoutSeconds": SimTypeInt(signed=False, label="UInt32"), "saIdleTimeout": SimStruct({"idleTimeoutSeconds": SimTypeInt(signed=False, label="UInt32"), "idleTimeoutSecondsFailOver": SimTypeInt(signed=False, label="UInt32")}, name="IPSEC_SA_IDLE_TIMEOUT0", pack=False, align=None), "emPolicy": SimTypePointer(SimStruct({"numAuthenticationMethods": SimTypeInt(signed=False, label="UInt32"), "authenticationMethods": SimTypePointer(SimStruct({"authenticationMethodType": SimTypeInt(signed=False, label="IKEEXT_AUTHENTICATION_METHOD_TYPE"), "Anonymous": SimUnion({"presharedKeyAuthentication": SimStruct({"presharedKey": SimStruct({"size": SimTypeInt(signed=False, label="UInt32"), "data": SimTypePointer(SimTypeChar(label="Byte"), offset=0)}, name="FWP_BYTE_BLOB", pack=False, align=None), "flags": SimTypeInt(signed=False, label="IKEEXT_PRESHARED_KEY_AUTHENTICATION_FLAGS")}, name="IKEEXT_PRESHARED_KEY_AUTHENTICATION1", pack=False, align=None), "certificateAuthentication": SimStruct({"inboundConfigType": SimTypeInt(signed=False, label="IKEEXT_CERT_CONFIG_TYPE"), "Anonymous1": SimUnion({"Anonymous1": SimStruct({"inboundRootArraySize": SimTypeInt(signed=False, label="UInt32"), "inboundRootCriteria": SimTypePointer(SimStruct({"certData": SimStruct({"size": SimTypeInt(signed=False, label="UInt32"), "data": SimTypePointer(SimTypeChar(label="Byte"), offset=0)}, name="FWP_BYTE_BLOB", pack=False, align=None), "certHash": SimStruct({"size": SimTypeInt(signed=False, label="UInt32"), "data": SimTypePointer(SimTypeChar(label="Byte"), offset=0)}, name="FWP_BYTE_BLOB", pack=False, align=None), "eku": SimTypePointer(SimStruct({"numEku": SimTypeInt(signed=False, label="UInt32"), "eku": SimTypePointer(SimTypePointer(SimTypeChar(label="Byte"), offset=0), offset=0)}, name="IKEEXT_CERT_EKUS0", pack=False, align=None), offset=0), "name": SimTypePointer(SimStruct({"nameType": SimTypeInt(signed=False, label="IKEEXT_CERT_CRITERIA_NAME_TYPE"), "certName": SimTypePointer(SimTypeChar(label="Char"), offset=0)}, name="IKEEXT_CERT_NAME0", pack=False, align=None), offset=0), "flags": SimTypeInt(signed=False, label="UInt32")}, name="IKEEXT_CERTIFICATE_CRITERIA0", pack=False, align=None), offset=0)}, name="_Anonymous1_e__Struct", pack=False, align=None), "Anonymous2": SimStruct({"inboundEnterpriseStoreArraySize": SimTypeInt(signed=False, label="UInt32"), "inboundEnterpriseStoreCriteria": SimTypePointer(SimStruct({"certData": SimStruct({"size": SimTypeInt(signed=False, label="UInt32"), "data": SimTypePointer(SimTypeChar(label="Byte"), offset=0)}, name="FWP_BYTE_BLOB", pack=False, align=None), "certHash": SimStruct({"size": SimTypeInt(signed=False, label="UInt32"), "data": SimTypePointer(SimTypeChar(label="Byte"), offset=0)}, name="FWP_BYTE_BLOB", pack=False, align=None), "eku": SimTypePointer(SimStruct({"numEku": SimTypeInt(signed=False, label="UInt32"), "eku": SimTypePointer(SimTypePointer(SimTypeChar(label="Byte"), offset=0), offset=0)}, name="IKEEXT_CERT_EKUS0", pack=False, align=None), offset=0), "name": SimTypePointer(SimStruct({"nameType": SimTypeInt(signed=False, label="IKEEXT_CERT_CRITERIA_NAME_TYPE"), "certName": SimTypePointer(SimTypeChar(label="Char"), offset=0)}, name="IKEEXT_CERT_NAME0", pack=False, align=None), offset=0), "flags": SimTypeInt(signed=False, label="UInt32")}, name="IKEEXT_CERTIFICATE_CRITERIA0", pack=False, align=None), offset=0)}, name="_Anonymous2_e__Struct", pack=False, align=None), "Anonymous3": SimStruct({"inboundRootStoreArraySize": SimTypeInt(signed=False, label="UInt32"), "inboundTrustedRootStoreCriteria": SimTypePointer(SimStruct({"certData": SimStruct({"size": SimTypeInt(signed=False, label="UInt32"), "data": SimTypePointer(SimTypeChar(label="Byte"), offset=0)}, name="FWP_BYTE_BLOB", pack=False, align=None), "certHash": SimStruct({"size": SimTypeInt(signed=False, label="UInt32"), "data": SimTypePointer(SimTypeChar(label="Byte"), offset=0)}, name="FWP_BYTE_BLOB", pack=False, align=None), "eku": SimTypePointer(SimStruct({"numEku": SimTypeInt(signed=False, label="UInt32"), "eku": SimTypePointer(SimTypePointer(SimTypeChar(label="Byte"), offset=0), offset=0)}, name="IKEEXT_CERT_EKUS0", pack=False, align=None), offset=0), "name": SimTypePointer(SimStruct({"nameType": SimTypeInt(signed=False, label="IKEEXT_CERT_CRITERIA_NAME_TYPE"), "certName": SimTypePointer(SimTypeChar(label="Char"), offset=0)}, name="IKEEXT_CERT_NAME0", pack=False, align=None), offset=0), "flags": SimTypeInt(signed=False, label="UInt32")}, name="IKEEXT_CERTIFICATE_CRITERIA0", pack=False, align=None), offset=0)}, name="_Anonymous3_e__Struct", pack=False, align=None)}, name="<anon>", label="None"), "outboundConfigType": SimTypeInt(signed=False, label="IKEEXT_CERT_CONFIG_TYPE"), "Anonymous2": SimUnion({"Anonymous1": SimStruct({"outboundRootArraySize": SimTypeInt(signed=False, label="UInt32"), "outboundRootCriteria": SimTypePointer(SimStruct({"certData": SimStruct({"size": SimTypeInt(signed=False, label="UInt32"), "data": SimTypePointer(SimTypeChar(label="Byte"), offset=0)}, name="FWP_BYTE_BLOB", pack=False, align=None), "certHash": SimStruct({"size": SimTypeInt(signed=False, label="UInt32"), "data": SimTypePointer(SimTypeChar(label="Byte"), offset=0)}, name="FWP_BYTE_BLOB", pack=False, align=None), "eku": SimTypePointer(SimStruct({"numEku": SimTypeInt(signed=False, label="UInt32"), "eku": SimTypePointer(SimTypePointer(SimTypeChar(label="Byte"), offset=0), offset=0)}, name="IKEEXT_CERT_EKUS0", pack=False, align=None), offset=0), "name": SimTypePointer(SimStruct({"nameType": SimTypeInt(signed=False, label="IKEEXT_CERT_CRITERIA_NAME_TYPE"), "certName": SimTypePointer(SimTypeChar(label="Char"), offset=0)}, name="IKEEXT_CERT_NAME0", pack=False, align=None), offset=0), "flags": SimTypeInt(signed=False, label="UInt32")}, name="IKEEXT_CERTIFICATE_CRITERIA0", pack=False, align=None), offset=0)}, name="_Anonymous1_e__Struct", pack=False, align=None), "Anonymous2": SimStruct({"outboundEnterpriseStoreArraySize": SimTypeInt(signed=False, label="UInt32"), "outboundEnterpriseStoreCriteria": SimTypePointer(SimStruct({"certData": SimStruct({"size": SimTypeInt(signed=False, label="UInt32"), "data": SimTypePointer(SimTypeChar(label="Byte"), offset=0)}, name="FWP_BYTE_BLOB", pack=False, align=None), "certHash": SimStruct({"size": SimTypeInt(signed=False, label="UInt32"), "data": SimTypePointer(SimTypeChar(label="Byte"), offset=0)}, name="FWP_BYTE_BLOB", pack=False, align=None), "eku": SimTypePointer(SimStruct({"numEku": SimTypeInt(signed=False, label="UInt32"), "eku": SimTypePointer(SimTypePointer(SimTypeChar(label="Byte"), offset=0), offset=0)}, name="IKEEXT_CERT_EKUS0", pack=False, align=None), offset=0), "name": SimTypePointer(SimStruct({"nameType": SimTypeInt(signed=False, label="IKEEXT_CERT_CRITERIA_NAME_TYPE"), "certName": SimTypePointer(SimTypeChar(label="Char"), offset=0)}, name="IKEEXT_CERT_NAME0", pack=False, align=None), offset=0), "flags": SimTypeInt(signed=False, label="UInt32")}, name="IKEEXT_CERTIFICATE_CRITERIA0", pack=False, align=None), offset=0)}, name="_Anonymous2_e__Struct", pack=False, align=None), "Anonymous3": SimStruct({"outboundRootStoreArraySize": SimTypeInt(signed=False, label="UInt32"), "outboundTrustedRootStoreCriteria": SimTypePointer(SimStruct({"certData": SimStruct({"size": SimTypeInt(signed=False, label="UInt32"), "data": SimTypePointer(SimTypeChar(label="Byte"), offset=0)}, name="FWP_BYTE_BLOB", pack=False, align=None), "certHash": SimStruct({"size": SimTypeInt(signed=False, label="UInt32"), "data": SimTypePointer(SimTypeChar(label="Byte"), offset=0)}, name="FWP_BYTE_BLOB", pack=False, align=None), "eku": SimTypePointer(SimStruct({"numEku": SimTypeInt(signed=False, label="UInt32"), "eku": SimTypePointer(SimTypePointer(SimTypeChar(label="Byte"), offset=0), offset=0)}, name="IKEEXT_CERT_EKUS0", pack=False, align=None), offset=0), "name": SimTypePointer(SimStruct({"nameType": SimTypeInt(signed=False, label="IKEEXT_CERT_CRITERIA_NAME_TYPE"), "certName": SimTypePointer(SimTypeChar(label="Char"), offset=0)}, name="IKEEXT_CERT_NAME0", pack=False, align=None), offset=0), "flags": SimTypeInt(signed=False, label="UInt32")}, name="IKEEXT_CERTIFICATE_CRITERIA0", pack=False, align=None), offset=0)}, name="_Anonymous3_e__Struct", pack=False, align=None)}, name="<anon>", label="None"), "flags": SimTypeInt(signed=False, label="IKEEXT_CERT_AUTH"), "localCertLocationUrl": SimStruct({"size": SimTypeInt(signed=False, label="UInt32"), "data": SimTypePointer(SimTypeChar(label="Byte"), offset=0)}, name="FWP_BYTE_BLOB", pack=False, align=None)}, name="IKEEXT_CERTIFICATE_AUTHENTICATION2", pack=False, align=None), "kerberosAuthentication": SimStruct({"flags": SimTypeInt(signed=False, label="IKEEXT_KERBEROS_AUTHENTICATION_FLAGS"), "proxyServer": SimTypePointer(SimTypeChar(label="Char"), offset=0)}, name="IKEEXT_KERBEROS_AUTHENTICATION1", pack=False, align=None), "reservedAuthentication": SimStruct({"flags": SimTypeInt(signed=False, label="IKEEXT_RESERVED_AUTHENTICATION_FLAGS")}, name="IKEEXT_RESERVED_AUTHENTICATION0", pack=False, align=None), "ntlmV2Authentication": SimStruct({"flags": SimTypeInt(signed=False, label="UInt32")}, name="IKEEXT_NTLM_V2_AUTHENTICATION0", pack=False, align=None), "sslAuthentication": SimStruct({"inboundConfigType": SimTypeInt(signed=False, label="IKEEXT_CERT_CONFIG_TYPE"), "Anonymous1": SimUnion({"Anonymous1": SimStruct({"inboundRootArraySize": SimTypeInt(signed=False, label="UInt32"), "inboundRootCriteria": SimTypePointer(SimStruct({"certData": SimStruct({"size": SimTypeInt(signed=False, label="UInt32"), "data": SimTypePointer(SimTypeChar(label="Byte"), offset=0)}, name="FWP_BYTE_BLOB", pack=False, align=None), "certHash": SimStruct({"size": SimTypeInt(signed=False, label="UInt32"), "data": SimTypePointer(SimTypeChar(label="Byte"), offset=0)}, name="FWP_BYTE_BLOB", pack=False, align=None), "eku": SimTypePointer(SimStruct({"numEku": SimTypeInt(signed=False, label="UInt32"), "eku": SimTypePointer(SimTypePointer(SimTypeChar(label="Byte"), offset=0), offset=0)}, name="IKEEXT_CERT_EKUS0", pack=False, align=None), offset=0), "name": SimTypePointer(SimStruct({"nameType": SimTypeInt(signed=False, label="IKEEXT_CERT_CRITERIA_NAME_TYPE"), "certName": SimTypePointer(SimTypeChar(label="Char"), offset=0)}, name="IKEEXT_CERT_NAME0", pack=False, align=None), offset=0), "flags": SimTypeInt(signed=False, label="UInt32")}, name="IKEEXT_CERTIFICATE_CRITERIA0", pack=False, align=None), offset=0)}, name="_Anonymous1_e__Struct", pack=False, align=None), "Anonymous2": SimStruct({"inboundEnterpriseStoreArraySize": SimTypeInt(signed=False, label="UInt32"), "inboundEnterpriseStoreCriteria": SimTypePointer(SimStruct({"certData": SimStruct({"size": SimTypeInt(signed=False, label="UInt32"), "data": SimTypePointer(SimTypeChar(label="Byte"), offset=0)}, name="FWP_BYTE_BLOB", pack=False, align=None), "certHash": SimStruct({"size": SimTypeInt(signed=False, label="UInt32"), "data": SimTypePointer(SimTypeChar(label="Byte"), offset=0)}, name="FWP_BYTE_BLOB", pack=False, align=None), "eku": SimTypePointer(SimStruct({"numEku": SimTypeInt(signed=False, label="UInt32"), "eku": SimTypePointer(SimTypePointer(SimTypeChar(label="Byte"), offset=0), offset=0)}, name="IKEEXT_CERT_EKUS0", pack=False, align=None), offset=0), "name": SimTypePointer(SimStruct({"nameType": SimTypeInt(signed=False, label="IKEEXT_CERT_CRITERIA_NAME_TYPE"), "certName": SimTypePointer(SimTypeChar(label="Char"), offset=0)}, name="IKEEXT_CERT_NAME0", pack=False, align=None), offset=0), "flags": SimTypeInt(signed=False, label="UInt32")}, name="IKEEXT_CERTIFICATE_CRITERIA0", pack=False, align=None), offset=0)}, name="_Anonymous2_e__Struct", pack=False, align=None), "Anonymous3": SimStruct({"inboundRootStoreArraySize": SimTypeInt(signed=False, label="UInt32"), "inboundTrustedRootStoreCriteria": SimTypePointer(SimStruct({"certData": SimStruct({"size": SimTypeInt(signed=False, label="UInt32"), "data": SimTypePointer(SimTypeChar(label="Byte"), offset=0)}, name="FWP_BYTE_BLOB", pack=False, align=None), "certHash": SimStruct({"size": SimTypeInt(signed=False, label="UInt32"), "data": SimTypePointer(SimTypeChar(label="Byte"), offset=0)}, name="FWP_BYTE_BLOB", pack=False, align=None), "eku": SimTypePointer(SimStruct({"numEku": SimTypeInt(signed=False, label="UInt32"), "eku": SimTypePointer(SimTypePointer(SimTypeChar(label="Byte"), offset=0), offset=0)}, name="IKEEXT_CERT_EKUS0", pack=False, align=None), offset=0), "name": SimTypePointer(SimStruct({"nameType": SimTypeInt(signed=False, label="IKEEXT_CERT_CRITERIA_NAME_TYPE"), "certName": SimTypePointer(SimTypeChar(label="Char"), offset=0)}, name="IKEEXT_CERT_NAME0", pack=False, align=None), offset=0), "flags": SimTypeInt(signed=False, label="UInt32")}, name="IKEEXT_CERTIFICATE_CRITERIA0", pack=False, align=None), offset=0)}, name="_Anonymous3_e__Struct", pack=False, align=None)}, name="<anon>", label="None"), "outboundConfigType": SimTypeInt(signed=False, label="IKEEXT_CERT_CONFIG_TYPE"), "Anonymous2": SimUnion({"Anonymous1": SimStruct({"outboundRootArraySize": SimTypeInt(signed=False, label="UInt32"), "outboundRootCriteria": SimTypePointer(SimStruct({"certData": SimStruct({"size": SimTypeInt(signed=False, label="UInt32"), "data": SimTypePointer(SimTypeChar(label="Byte"), offset=0)}, name="FWP_BYTE_BLOB", pack=False, align=None), "certHash": SimStruct({"size": SimTypeInt(signed=False, label="UInt32"), "data": SimTypePointer(SimTypeChar(label="Byte"), offset=0)}, name="FWP_BYTE_BLOB", pack=False, align=None), "eku": SimTypePointer(SimStruct({"numEku": SimTypeInt(signed=False, label="UInt32"), "eku": SimTypePointer(SimTypePointer(SimTypeChar(label="Byte"), offset=0), offset=0)}, name="IKEEXT_CERT_EKUS0", pack=False, align=None), offset=0), "name": SimTypePointer(SimStruct({"nameType": SimTypeInt(signed=False, label="IKEEXT_CERT_CRITERIA_NAME_TYPE"), "certName": SimTypePointer(SimTypeChar(label="Char"), offset=0)}, name="IKEEXT_CERT_NAME0", pack=False, align=None), offset=0), "flags": SimTypeInt(signed=False, label="UInt32")}, name="IKEEXT_CERTIFICATE_CRITERIA0", pack=False, align=None), offset=0)}, name="_Anonymous1_e__Struct", pack=False, align=None), "Anonymous2": SimStruct({"outboundEnterpriseStoreArraySize": SimTypeInt(signed=False, label="UInt32"), "outboundEnterpriseStoreCriteria": SimTypePointer(SimStruct({"certData": SimStruct({"size": SimTypeInt(signed=False, label="UInt32"), "data": SimTypePointer(SimTypeChar(label="Byte"), offset=0)}, name="FWP_BYTE_BLOB", pack=False, align=None), "certHash": SimStruct({"size": SimTypeInt(signed=False, label="UInt32"), "data": SimTypePointer(SimTypeChar(label="Byte"), offset=0)}, name="FWP_BYTE_BLOB", pack=False, align=None), "eku": SimTypePointer(SimStruct({"numEku": SimTypeInt(signed=False, label="UInt32"), "eku": SimTypePointer(SimTypePointer(SimTypeChar(label="Byte"), offset=0), offset=0)}, name="IKEEXT_CERT_EKUS0", pack=False, align=None), offset=0), "name": SimTypePointer(SimStruct({"nameType": SimTypeInt(signed=False, label="IKEEXT_CERT_CRITERIA_NAME_TYPE"), "certName": SimTypePointer(SimTypeChar(label="Char"), offset=0)}, name="IKEEXT_CERT_NAME0", pack=False, align=None), offset=0), "flags": SimTypeInt(signed=False, label="UInt32")}, name="IKEEXT_CERTIFICATE_CRITERIA0", pack=False, align=None), offset=0)}, name="_Anonymous2_e__Struct", pack=False, align=None), "Anonymous3": SimStruct({"outboundRootStoreArraySize": SimTypeInt(signed=False, label="UInt32"), "outboundTrustedRootStoreCriteria": SimTypePointer(SimStruct({"certData": SimStruct({"size": SimTypeInt(signed=False, label="UInt32"), "data": SimTypePointer(SimTypeChar(label="Byte"), offset=0)}, name="FWP_BYTE_BLOB", pack=False, align=None), "certHash": SimStruct({"size": SimTypeInt(signed=False, label="UInt32"), "data": SimTypePointer(SimTypeChar(label="Byte"), offset=0)}, name="FWP_BYTE_BLOB", pack=False, align=None), "eku": SimTypePointer(SimStruct({"numEku": SimTypeInt(signed=False, label="UInt32"), "eku": SimTypePointer(SimTypePointer(SimTypeChar(label="Byte"), offset=0), offset=0)}, name="IKEEXT_CERT_EKUS0", pack=False, align=None), offset=0), "name": SimTypePointer(SimStruct({"nameType": SimTypeInt(signed=False, label="IKEEXT_CERT_CRITERIA_NAME_TYPE"), "certName": SimTypePointer(SimTypeChar(label="Char"), offset=0)}, name="IKEEXT_CERT_NAME0", pack=False, align=None), offset=0), "flags": SimTypeInt(signed=False, label="UInt32")}, name="IKEEXT_CERTIFICATE_CRITERIA0", pack=False, align=None), offset=0)}, name="_Anonymous3_e__Struct", pack=False, align=None)}, name="<anon>", label="None"), "flags": SimTypeInt(signed=False, label="IKEEXT_CERT_AUTH"), "localCertLocationUrl": SimStruct({"size": SimTypeInt(signed=False, label="UInt32"), "data": SimTypePointer(SimTypeChar(label="Byte"), offset=0)}, name="FWP_BYTE_BLOB", pack=False, align=None)}, name="IKEEXT_CERTIFICATE_AUTHENTICATION2", pack=False, align=None), "cgaAuthentication": SimStruct({"keyContainerName": SimTypePointer(SimTypeChar(label="Char"), offset=0), "cspName": SimTypePointer(SimTypeChar(label="Char"), offset=0), "cspType": SimTypeInt(signed=False, label="UInt32"), "cgaModifier": SimStruct({"byteArray16": SimTypeFixedSizeArray(SimTypeChar(label="Byte"), 16)}, name="FWP_BYTE_ARRAY16", pack=False, align=None), "cgaCollisionCount": SimTypeChar(label="Byte")}, name="IKEEXT_IPV6_CGA_AUTHENTICATION0", pack=False, align=None), "eapAuthentication": SimStruct({"flags": SimTypeInt(signed=False, label="IKEEXT_EAP_AUTHENTICATION_FLAGS")}, name="IKEEXT_EAP_AUTHENTICATION0", pack=False, align=None)}, name="<anon>", label="None")}, name="IKEEXT_AUTHENTICATION_METHOD2", pack=False, align=None), offset=0), "initiatorImpersonationType": SimTypeInt(signed=False, label="IKEEXT_AUTHENTICATION_IMPERSONATION_TYPE")}, name="IKEEXT_EM_POLICY2", pack=False, align=None), offset=0)}, name="IPSEC_TRANSPORT_POLICY2", pack=False, align=None), offset=0), "authipQmTunnelPolicy": SimTypePointer(SimStruct({"flags": SimTypeInt(signed=False, label="IPSEC_POLICY_FLAG"), "numIpsecProposals": SimTypeInt(signed=False, label="UInt32"), "ipsecProposals": SimTypePointer(SimStruct({"lifetime": SimStruct({"lifetimeSeconds": SimTypeInt(signed=False, label="UInt32"), "lifetimeKilobytes": SimTypeInt(signed=False, label="UInt32"), "lifetimePackets": SimTypeInt(signed=False, label="UInt32")}, name="IPSEC_SA_LIFETIME0", pack=False, align=None), "numSaTransforms": SimTypeInt(signed=False, label="UInt32"), "saTransforms": SimTypePointer(SimStruct({"ipsecTransformType": SimTypeInt(signed=False, label="IPSEC_TRANSFORM_TYPE"), "Anonymous": SimUnion({"ahTransform": SimTypePointer(SimStruct({"authTransformId": SimStruct({"authType": SimTypeInt(signed=False, label="IPSEC_AUTH_TYPE"), "authConfig": SimTypeChar(label="Byte")}, name="IPSEC_AUTH_TRANSFORM_ID0", pack=False, align=None), "cryptoModuleId": SimTypePointer(SimTypeBottom(label="Guid"), offset=0)}, name="IPSEC_AUTH_TRANSFORM0", pack=False, align=None), offset=0), "espAuthTransform": SimTypePointer(SimStruct({"authTransformId": SimStruct({"authType": SimTypeInt(signed=False, label="IPSEC_AUTH_TYPE"), "authConfig": SimTypeChar(label="Byte")}, name="IPSEC_AUTH_TRANSFORM_ID0", pack=False, align=None), "cryptoModuleId": SimTypePointer(SimTypeBottom(label="Guid"), offset=0)}, name="IPSEC_AUTH_TRANSFORM0", pack=False, align=None), offset=0), "espCipherTransform": SimTypePointer(SimStruct({"cipherTransformId": SimStruct({"cipherType": SimTypeInt(signed=False, label="IPSEC_CIPHER_TYPE"), "cipherConfig": SimTypeChar(label="Byte")}, name="IPSEC_CIPHER_TRANSFORM_ID0", pack=False, align=None), "cryptoModuleId": SimTypePointer(SimTypeBottom(label="Guid"), offset=0)}, name="IPSEC_CIPHER_TRANSFORM0", pack=False, align=None), offset=0), "espAuthAndCipherTransform": SimTypePointer(SimStruct({"authTransform": SimStruct({"authTransformId": SimStruct({"authType": SimTypeInt(signed=False, label="IPSEC_AUTH_TYPE"), "authConfig": SimTypeChar(label="Byte")}, name="IPSEC_AUTH_TRANSFORM_ID0", pack=False, align=None), "cryptoModuleId": SimTypePointer(SimTypeBottom(label="Guid"), offset=0)}, name="IPSEC_AUTH_TRANSFORM0", pack=False, align=None), "cipherTransform": SimStruct({"cipherTransformId": SimStruct({"cipherType": SimTypeInt(signed=False, label="IPSEC_CIPHER_TYPE"), "cipherConfig": SimTypeChar(label="Byte")}, name="IPSEC_CIPHER_TRANSFORM_ID0", pack=False, align=None), "cryptoModuleId": SimTypePointer(SimTypeBottom(label="Guid"), offset=0)}, name="IPSEC_CIPHER_TRANSFORM0", pack=False, align=None)}, name="IPSEC_AUTH_AND_CIPHER_TRANSFORM0", pack=False, align=None), offset=0), "espAuthFwTransform": SimTypePointer(SimStruct({"authTransformId": SimStruct({"authType": SimTypeInt(signed=False, label="IPSEC_AUTH_TYPE"), "authConfig": SimTypeChar(label="Byte")}, name="IPSEC_AUTH_TRANSFORM_ID0", pack=False, align=None), "cryptoModuleId": SimTypePointer(SimTypeBottom(label="Guid"), offset=0)}, name="IPSEC_AUTH_TRANSFORM0", pack=False, align=None), offset=0)}, name="<anon>", label="None")}, name="IPSEC_SA_TRANSFORM0", pack=False, align=None), offset=0), "pfsGroup": SimTypeInt(signed=False, label="IPSEC_PFS_GROUP")}, name="IPSEC_PROPOSAL0", pack=False, align=None), offset=0), "tunnelEndpoints": SimStruct({"ipVersion": SimTypeInt(signed=False, label="FWP_IP_VERSION"), "Anonymous1": SimUnion({"localV4Address": SimTypeInt(signed=False, label="UInt32"), "localV6Address": SimTypeFixedSizeArray(SimTypeChar(label="Byte"), 16)}, name="<anon>", label="None"), "Anonymous2": SimUnion({"remoteV4Address": SimTypeInt(signed=False, label="UInt32"), "remoteV6Address": SimTypeFixedSizeArray(SimTypeChar(label="Byte"), 16)}, name="<anon>", label="None"), "localIfLuid": SimTypeLongLong(signed=False, label="UInt64"), "remoteFqdn": SimTypePointer(SimTypeChar(label="Char"), offset=0), "numAddresses": SimTypeInt(signed=False, label="UInt32"), "remoteAddresses": SimTypePointer(SimStruct({"ipVersion": SimTypeInt(signed=False, label="FWP_IP_VERSION"), "Anonymous": SimUnion({"v4Address": SimTypeInt(signed=False, label="UInt32"), "v6Address": SimTypeFixedSizeArray(SimTypeChar(label="Byte"), 16)}, name="<anon>", label="None")}, name="IPSEC_TUNNEL_ENDPOINT0", pack=False, align=None), offset=0)}, name="IPSEC_TUNNEL_ENDPOINTS2", pack=False, align=None), "saIdleTimeout": SimStruct({"idleTimeoutSeconds": SimTypeInt(signed=False, label="UInt32"), "idleTimeoutSecondsFailOver": SimTypeInt(signed=False, label="UInt32")}, name="IPSEC_SA_IDLE_TIMEOUT0", pack=False, align=None), "emPolicy": SimTypePointer(SimStruct({"numAuthenticationMethods": SimTypeInt(signed=False, label="UInt32"), "authenticationMethods": SimTypePointer(SimStruct({"authenticationMethodType": SimTypeInt(signed=False, label="IKEEXT_AUTHENTICATION_METHOD_TYPE"), "Anonymous": SimUnion({"presharedKeyAuthentication": SimStruct({"presharedKey": SimStruct({"size": SimTypeInt(signed=False, label="UInt32"), "data": SimTypePointer(SimTypeChar(label="Byte"), offset=0)}, name="FWP_BYTE_BLOB", pack=False, align=None), "flags": SimTypeInt(signed=False, label="IKEEXT_PRESHARED_KEY_AUTHENTICATION_FLAGS")}, name="IKEEXT_PRESHARED_KEY_AUTHENTICATION1", pack=False, align=None), "certificateAuthentication": SimStruct({"inboundConfigType": SimTypeInt(signed=False, label="IKEEXT_CERT_CONFIG_TYPE"), "Anonymous1": SimUnion({"Anonymous1": SimStruct({"inboundRootArraySize": SimTypeInt(signed=False, label="UInt32"), "inboundRootCriteria": SimTypePointer(SimStruct({"certData": SimStruct({"size": SimTypeInt(signed=False, label="UInt32"), "data": SimTypePointer(SimTypeChar(label="Byte"), offset=0)}, name="FWP_BYTE_BLOB", pack=False, align=None), "certHash": SimStruct({"size": SimTypeInt(signed=False, label="UInt32"), "data": SimTypePointer(SimTypeChar(label="Byte"), offset=0)}, name="FWP_BYTE_BLOB", pack=False, align=None), "eku": SimTypePointer(SimStruct({"numEku": SimTypeInt(signed=False, label="UInt32"), "eku": SimTypePointer(SimTypePointer(SimTypeChar(label="Byte"), offset=0), offset=0)}, name="IKEEXT_CERT_EKUS0", pack=False, align=None), offset=0), "name": SimTypePointer(SimStruct({"nameType": SimTypeInt(signed=False, label="IKEEXT_CERT_CRITERIA_NAME_TYPE"), "certName": SimTypePointer(SimTypeChar(label="Char"), offset=0)}, name="IKEEXT_CERT_NAME0", pack=False, align=None), offset=0), "flags": SimTypeInt(signed=False, label="UInt32")}, name="IKEEXT_CERTIFICATE_CRITERIA0", pack=False, align=None), offset=0)}, name="_Anonymous1_e__Struct", pack=False, align=None), "Anonymous2": SimStruct({"inboundEnterpriseStoreArraySize": SimTypeInt(signed=False, label="UInt32"), "inboundEnterpriseStoreCriteria": SimTypePointer(SimStruct({"certData": SimStruct({"size": SimTypeInt(signed=False, label="UInt32"), "data": SimTypePointer(SimTypeChar(label="Byte"), offset=0)}, name="FWP_BYTE_BLOB", pack=False, align=None), "certHash": SimStruct({"size": SimTypeInt(signed=False, label="UInt32"), "data": SimTypePointer(SimTypeChar(label="Byte"), offset=0)}, name="FWP_BYTE_BLOB", pack=False, align=None), "eku": SimTypePointer(SimStruct({"numEku": SimTypeInt(signed=False, label="UInt32"), "eku": SimTypePointer(SimTypePointer(SimTypeChar(label="Byte"), offset=0), offset=0)}, name="IKEEXT_CERT_EKUS0", pack=False, align=None), offset=0), "name": SimTypePointer(SimStruct({"nameType": SimTypeInt(signed=False, label="IKEEXT_CERT_CRITERIA_NAME_TYPE"), "certName": SimTypePointer(SimTypeChar(label="Char"), offset=0)}, name="IKEEXT_CERT_NAME0", pack=False, align=None), offset=0), "flags": SimTypeInt(signed=False, label="UInt32")}, name="IKEEXT_CERTIFICATE_CRITERIA0", pack=False, align=None), offset=0)}, name="_Anonymous2_e__Struct", pack=False, align=None), "Anonymous3": SimStruct({"inboundRootStoreArraySize": SimTypeInt(signed=False, label="UInt32"), "inboundTrustedRootStoreCriteria": SimTypePointer(SimStruct({"certData": SimStruct({"size": SimTypeInt(signed=False, label="UInt32"), "data": SimTypePointer(SimTypeChar(label="Byte"), offset=0)}, name="FWP_BYTE_BLOB", pack=False, align=None), "certHash": SimStruct({"size": SimTypeInt(signed=False, label="UInt32"), "data": SimTypePointer(SimTypeChar(label="Byte"), offset=0)}, name="FWP_BYTE_BLOB", pack=False, align=None), "eku": SimTypePointer(SimStruct({"numEku": SimTypeInt(signed=False, label="UInt32"), "eku": SimTypePointer(SimTypePointer(SimTypeChar(label="Byte"), offset=0), offset=0)}, name="IKEEXT_CERT_EKUS0", pack=False, align=None), offset=0), "name": SimTypePointer(SimStruct({"nameType": SimTypeInt(signed=False, label="IKEEXT_CERT_CRITERIA_NAME_TYPE"), "certName": SimTypePointer(SimTypeChar(label="Char"), offset=0)}, name="IKEEXT_CERT_NAME0", pack=False, align=None), offset=0), "flags": SimTypeInt(signed=False, label="UInt32")}, name="IKEEXT_CERTIFICATE_CRITERIA0", pack=False, align=None), offset=0)}, name="_Anonymous3_e__Struct", pack=False, align=None)}, name="<anon>", label="None"), "outboundConfigType": SimTypeInt(signed=False, label="IKEEXT_CERT_CONFIG_TYPE"), "Anonymous2": SimUnion({"Anonymous1": SimStruct({"outboundRootArraySize": SimTypeInt(signed=False, label="UInt32"), "outboundRootCriteria": SimTypePointer(SimStruct({"certData": SimStruct({"size": SimTypeInt(signed=False, label="UInt32"), "data": SimTypePointer(SimTypeChar(label="Byte"), offset=0)}, name="FWP_BYTE_BLOB", pack=False, align=None), "certHash": SimStruct({"size": SimTypeInt(signed=False, label="UInt32"), "data": SimTypePointer(SimTypeChar(label="Byte"), offset=0)}, name="FWP_BYTE_BLOB", pack=False, align=None), "eku": SimTypePointer(SimStruct({"numEku": SimTypeInt(signed=False, label="UInt32"), "eku": SimTypePointer(SimTypePointer(SimTypeChar(label="Byte"), offset=0), offset=0)}, name="IKEEXT_CERT_EKUS0", pack=False, align=None), offset=0), "name": SimTypePointer(SimStruct({"nameType": SimTypeInt(signed=False, label="IKEEXT_CERT_CRITERIA_NAME_TYPE"), "certName": SimTypePointer(SimTypeChar(label="Char"), offset=0)}, name="IKEEXT_CERT_NAME0", pack=False, align=None), offset=0), "flags": SimTypeInt(signed=False, label="UInt32")}, name="IKEEXT_CERTIFICATE_CRITERIA0", pack=False, align=None), offset=0)}, name="_Anonymous1_e__Struct", pack=False, align=None), "Anonymous2": SimStruct({"outboundEnterpriseStoreArraySize": SimTypeInt(signed=False, label="UInt32"), "outboundEnterpriseStoreCriteria": SimTypePointer(SimStruct({"certData": SimStruct({"size": SimTypeInt(signed=False, label="UInt32"), "data": SimTypePointer(SimTypeChar(label="Byte"), offset=0)}, name="FWP_BYTE_BLOB", pack=False, align=None), "certHash": SimStruct({"size": SimTypeInt(signed=False, label="UInt32"), "data": SimTypePointer(SimTypeChar(label="Byte"), offset=0)}, name="FWP_BYTE_BLOB", pack=False, align=None), "eku": SimTypePointer(SimStruct({"numEku": SimTypeInt(signed=False, label="UInt32"), "eku": SimTypePointer(SimTypePointer(SimTypeChar(label="Byte"), offset=0), offset=0)}, name="IKEEXT_CERT_EKUS0", pack=False, align=None), offset=0), "name": SimTypePointer(SimStruct({"nameType": SimTypeInt(signed=False, label="IKEEXT_CERT_CRITERIA_NAME_TYPE"), "certName": SimTypePointer(SimTypeChar(label="Char"), offset=0)}, name="IKEEXT_CERT_NAME0", pack=False, align=None), offset=0), "flags": SimTypeInt(signed=False, label="UInt32")}, name="IKEEXT_CERTIFICATE_CRITERIA0", pack=False, align=None), offset=0)}, name="_Anonymous2_e__Struct", pack=False, align=None), "Anonymous3": SimStruct({"outboundRootStoreArraySize": SimTypeInt(signed=False, label="UInt32"), "outboundTrustedRootStoreCriteria": SimTypePointer(SimStruct({"certData": SimStruct({"size": SimTypeInt(signed=False, label="UInt32"), "data": SimTypePointer(SimTypeChar(label="Byte"), offset=0)}, name="FWP_BYTE_BLOB", pack=False, align=None), "certHash": SimStruct({"size": SimTypeInt(signed=False, label="UInt32"), "data": SimTypePointer(SimTypeChar(label="Byte"), offset=0)}, name="FWP_BYTE_BLOB", pack=False, align=None), "eku": SimTypePointer(SimStruct({"numEku": SimTypeInt(signed=False, label="UInt32"), "eku": SimTypePointer(SimTypePointer(SimTypeChar(label="Byte"), offset=0), offset=0)}, name="IKEEXT_CERT_EKUS0", pack=False, align=None), offset=0), "name": SimTypePointer(SimStruct({"nameType": SimTypeInt(signed=False, label="IKEEXT_CERT_CRITERIA_NAME_TYPE"), "certName": SimTypePointer(SimTypeChar(label="Char"), offset=0)}, name="IKEEXT_CERT_NAME0", pack=False, align=None), offset=0), "flags": SimTypeInt(signed=False, label="UInt32")}, name="IKEEXT_CERTIFICATE_CRITERIA0", pack=False, align=None), offset=0)}, name="_Anonymous3_e__Struct", pack=False, align=None)}, name="<anon>", label="None"), "flags": SimTypeInt(signed=False, label="IKEEXT_CERT_AUTH"), "localCertLocationUrl": SimStruct({"size": SimTypeInt(signed=False, label="UInt32"), "data": SimTypePointer(SimTypeChar(label="Byte"), offset=0)}, name="FWP_BYTE_BLOB", pack=False, align=None)}, name="IKEEXT_CERTIFICATE_AUTHENTICATION2", pack=False, align=None), "kerberosAuthentication": SimStruct({"flags": SimTypeInt(signed=False, label="IKEEXT_KERBEROS_AUTHENTICATION_FLAGS"), "proxyServer": SimTypePointer(SimTypeChar(label="Char"), offset=0)}, name="IKEEXT_KERBEROS_AUTHENTICATION1", pack=False, align=None), "reservedAuthentication": SimStruct({"flags": SimTypeInt(signed=False, label="IKEEXT_RESERVED_AUTHENTICATION_FLAGS")}, name="IKEEXT_RESERVED_AUTHENTICATION0", pack=False, align=None), "ntlmV2Authentication": SimStruct({"flags": SimTypeInt(signed=False, label="UInt32")}, name="IKEEXT_NTLM_V2_AUTHENTICATION0", pack=False, align=None), "sslAuthentication": SimStruct({"inboundConfigType": SimTypeInt(signed=False, label="IKEEXT_CERT_CONFIG_TYPE"), "Anonymous1": SimUnion({"Anonymous1": SimStruct({"inboundRootArraySize": SimTypeInt(signed=False, label="UInt32"), "inboundRootCriteria": SimTypePointer(SimStruct({"certData": SimStruct({"size": SimTypeInt(signed=False, label="UInt32"), "data": SimTypePointer(SimTypeChar(label="Byte"), offset=0)}, name="FWP_BYTE_BLOB", pack=False, align=None), "certHash": SimStruct({"size": SimTypeInt(signed=False, label="UInt32"), "data": SimTypePointer(SimTypeChar(label="Byte"), offset=0)}, name="FWP_BYTE_BLOB", pack=False, align=None), "eku": SimTypePointer(SimStruct({"numEku": SimTypeInt(signed=False, label="UInt32"), "eku": SimTypePointer(SimTypePointer(SimTypeChar(label="Byte"), offset=0), offset=0)}, name="IKEEXT_CERT_EKUS0", pack=False, align=None), offset=0), "name": SimTypePointer(SimStruct({"nameType": SimTypeInt(signed=False, label="IKEEXT_CERT_CRITERIA_NAME_TYPE"), "certName": SimTypePointer(SimTypeChar(label="Char"), offset=0)}, name="IKEEXT_CERT_NAME0", pack=False, align=None), offset=0), "flags": SimTypeInt(signed=False, label="UInt32")}, name="IKEEXT_CERTIFICATE_CRITERIA0", pack=False, align=None), offset=0)}, name="_Anonymous1_e__Struct", pack=False, align=None), "Anonymous2": SimStruct({"inboundEnterpriseStoreArraySize": SimTypeInt(signed=False, label="UInt32"), "inboundEnterpriseStoreCriteria": SimTypePointer(SimStruct({"certData": SimStruct({"size": SimTypeInt(signed=False, label="UInt32"), "data": SimTypePointer(SimTypeChar(label="Byte"), offset=0)}, name="FWP_BYTE_BLOB", pack=False, align=None), "certHash": SimStruct({"size": SimTypeInt(signed=False, label="UInt32"), "data": SimTypePointer(SimTypeChar(label="Byte"), offset=0)}, name="FWP_BYTE_BLOB", pack=False, align=None), "eku": SimTypePointer(SimStruct({"numEku": SimTypeInt(signed=False, label="UInt32"), "eku": SimTypePointer(SimTypePointer(SimTypeChar(label="Byte"), offset=0), offset=0)}, name="IKEEXT_CERT_EKUS0", pack=False, align=None), offset=0), "name": SimTypePointer(SimStruct({"nameType": SimTypeInt(signed=False, label="IKEEXT_CERT_CRITERIA_NAME_TYPE"), "certName": SimTypePointer(SimTypeChar(label="Char"), offset=0)}, name="IKEEXT_CERT_NAME0", pack=False, align=None), offset=0), "flags": SimTypeInt(signed=False, label="UInt32")}, name="IKEEXT_CERTIFICATE_CRITERIA0", pack=False, align=None), offset=0)}, name="_Anonymous2_e__Struct", pack=False, align=None), "Anonymous3": SimStruct({"inboundRootStoreArraySize": SimTypeInt(signed=False, label="UInt32"), "inboundTrustedRootStoreCriteria": SimTypePointer(SimStruct({"certData": SimStruct({"size": SimTypeInt(signed=False, label="UInt32"), "data": SimTypePointer(SimTypeChar(label="Byte"), offset=0)}, name="FWP_BYTE_BLOB", pack=False, align=None), "certHash": SimStruct({"size": SimTypeInt(signed=False, label="UInt32"), "data": SimTypePointer(SimTypeChar(label="Byte"), offset=0)}, name="FWP_BYTE_BLOB", pack=False, align=None), "eku": SimTypePointer(SimStruct({"numEku": SimTypeInt(signed=False, label="UInt32"), "eku": SimTypePointer(SimTypePointer(SimTypeChar(label="Byte"), offset=0), offset=0)}, name="IKEEXT_CERT_EKUS0", pack=False, align=None), offset=0), "name": SimTypePointer(SimStruct({"nameType": SimTypeInt(signed=False, label="IKEEXT_CERT_CRITERIA_NAME_TYPE"), "certName": SimTypePointer(SimTypeChar(label="Char"), offset=0)}, name="IKEEXT_CERT_NAME0", pack=False, align=None), offset=0), "flags": SimTypeInt(signed=False, label="UInt32")}, name="IKEEXT_CERTIFICATE_CRITERIA0", pack=False, align=None), offset=0)}, name="_Anonymous3_e__Struct", pack=False, align=None)}, name="<anon>", label="None"), "outboundConfigType": SimTypeInt(signed=False, label="IKEEXT_CERT_CONFIG_TYPE"), "Anonymous2": SimUnion({"Anonymous1": SimStruct({"outboundRootArraySize": SimTypeInt(signed=False, label="UInt32"), "outboundRootCriteria": SimTypePointer(SimStruct({"certData": SimStruct({"size": SimTypeInt(signed=False, label="UInt32"), "data": SimTypePointer(SimTypeChar(label="Byte"), offset=0)}, name="FWP_BYTE_BLOB", pack=False, align=None), "certHash": SimStruct({"size": SimTypeInt(signed=False, label="UInt32"), "data": SimTypePointer(SimTypeChar(label="Byte"), offset=0)}, name="FWP_BYTE_BLOB", pack=False, align=None), "eku": SimTypePointer(SimStruct({"numEku": SimTypeInt(signed=False, label="UInt32"), "eku": SimTypePointer(SimTypePointer(SimTypeChar(label="Byte"), offset=0), offset=0)}, name="IKEEXT_CERT_EKUS0", pack=False, align=None), offset=0), "name": SimTypePointer(SimStruct({"nameType": SimTypeInt(signed=False, label="IKEEXT_CERT_CRITERIA_NAME_TYPE"), "certName": SimTypePointer(SimTypeChar(label="Char"), offset=0)}, name="IKEEXT_CERT_NAME0", pack=False, align=None), offset=0), "flags": SimTypeInt(signed=False, label="UInt32")}, name="IKEEXT_CERTIFICATE_CRITERIA0", pack=False, align=None), offset=0)}, name="_Anonymous1_e__Struct", pack=False, align=None), "Anonymous2": SimStruct({"outboundEnterpriseStoreArraySize": SimTypeInt(signed=False, label="UInt32"), "outboundEnterpriseStoreCriteria": SimTypePointer(SimStruct({"certData": SimStruct({"size": SimTypeInt(signed=False, label="UInt32"), "data": SimTypePointer(SimTypeChar(label="Byte"), offset=0)}, name="FWP_BYTE_BLOB", pack=False, align=None), "certHash": SimStruct({"size": SimTypeInt(signed=False, label="UInt32"), "data": SimTypePointer(SimTypeChar(label="Byte"), offset=0)}, name="FWP_BYTE_BLOB", pack=False, align=None), "eku": SimTypePointer(SimStruct({"numEku": SimTypeInt(signed=False, label="UInt32"), "eku": SimTypePointer(SimTypePointer(SimTypeChar(label="Byte"), offset=0), offset=0)}, name="IKEEXT_CERT_EKUS0", pack=False, align=None), offset=0), "name": SimTypePointer(SimStruct({"nameType": SimTypeInt(signed=False, label="IKEEXT_CERT_CRITERIA_NAME_TYPE"), "certName": SimTypePointer(SimTypeChar(label="Char"), offset=0)}, name="IKEEXT_CERT_NAME0", pack=False, align=None), offset=0), "flags": SimTypeInt(signed=False, label="UInt32")}, name="IKEEXT_CERTIFICATE_CRITERIA0", pack=False, align=None), offset=0)}, name="_Anonymous2_e__Struct", pack=False, align=None), "Anonymous3": SimStruct({"outboundRootStoreArraySize": SimTypeInt(signed=False, label="UInt32"), "outboundTrustedRootStoreCriteria": SimTypePointer(SimStruct({"certData": SimStruct({"size": SimTypeInt(signed=False, label="UInt32"), "data": SimTypePointer(SimTypeChar(label="Byte"), offset=0)}, name="FWP_BYTE_BLOB", pack=False, align=None), "certHash": SimStruct({"size": SimTypeInt(signed=False, label="UInt32"), "data": SimTypePointer(SimTypeChar(label="Byte"), offset=0)}, name="FWP_BYTE_BLOB", pack=False, align=None), "eku": SimTypePointer(SimStruct({"numEku": SimTypeInt(signed=False, label="UInt32"), "eku": SimTypePointer(SimTypePointer(SimTypeChar(label="Byte"), offset=0), offset=0)}, name="IKEEXT_CERT_EKUS0", pack=False, align=None), offset=0), "name": SimTypePointer(SimStruct({"nameType": SimTypeInt(signed=False, label="IKEEXT_CERT_CRITERIA_NAME_TYPE"), "certName": SimTypePointer(SimTypeChar(label="Char"), offset=0)}, name="IKEEXT_CERT_NAME0", pack=False, align=None), offset=0), "flags": SimTypeInt(signed=False, label="UInt32")}, name="IKEEXT_CERTIFICATE_CRITERIA0", pack=False, align=None), offset=0)}, name="_Anonymous3_e__Struct", pack=False, align=None)}, name="<anon>", label="None"), "flags": SimTypeInt(signed=False, label="IKEEXT_CERT_AUTH"), "localCertLocationUrl": SimStruct({"size": SimTypeInt(signed=False, label="UInt32"), "data": SimTypePointer(SimTypeChar(label="Byte"), offset=0)}, name="FWP_BYTE_BLOB", pack=False, align=None)}, name="IKEEXT_CERTIFICATE_AUTHENTICATION2", pack=False, align=None), "cgaAuthentication": SimStruct({"keyContainerName": SimTypePointer(SimTypeChar(label="Char"), offset=0), "cspName": SimTypePointer(SimTypeChar(label="Char"), offset=0), "cspType": SimTypeInt(signed=False, label="UInt32"), "cgaModifier": SimStruct({"byteArray16": SimTypeFixedSizeArray(SimTypeChar(label="Byte"), 16)}, name="FWP_BYTE_ARRAY16", pack=False, align=None), "cgaCollisionCount": SimTypeChar(label="Byte")}, name="IKEEXT_IPV6_CGA_AUTHENTICATION0", pack=False, align=None), "eapAuthentication": SimStruct({"flags": SimTypeInt(signed=False, label="IKEEXT_EAP_AUTHENTICATION_FLAGS")}, name="IKEEXT_EAP_AUTHENTICATION0", pack=False, align=None)}, name="<anon>", label="None")}, name="IKEEXT_AUTHENTICATION_METHOD2", pack=False, align=None), offset=0), "initiatorImpersonationType": SimTypeInt(signed=False, label="IKEEXT_AUTHENTICATION_IMPERSONATION_TYPE")}, name="IKEEXT_EM_POLICY2", pack=False, align=None), offset=0), "fwdPathSaLifetime": SimTypeInt(signed=False, label="UInt32")}, name="IPSEC_TUNNEL_POLICY2", pack=False, align=None), offset=0), "ikeMmPolicy": SimTypePointer(SimStruct({"softExpirationTime": SimTypeInt(signed=False, label="UInt32"), "numAuthenticationMethods": SimTypeInt(signed=False, label="UInt32"), "authenticationMethods": SimTypePointer(SimStruct({"authenticationMethodType": SimTypeInt(signed=False, label="IKEEXT_AUTHENTICATION_METHOD_TYPE"), "Anonymous": SimUnion({"presharedKeyAuthentication": SimStruct({"presharedKey": SimStruct({"size": SimTypeInt(signed=False, label="UInt32"), "data": SimTypePointer(SimTypeChar(label="Byte"), offset=0)}, name="FWP_BYTE_BLOB", pack=False, align=None), "flags": SimTypeInt(signed=False, label="IKEEXT_PRESHARED_KEY_AUTHENTICATION_FLAGS")}, name="IKEEXT_PRESHARED_KEY_AUTHENTICATION1", pack=False, align=None), "certificateAuthentication": SimStruct({"inboundConfigType": SimTypeInt(signed=False, label="IKEEXT_CERT_CONFIG_TYPE"), "Anonymous1": SimUnion({"Anonymous1": SimStruct({"inboundRootArraySize": SimTypeInt(signed=False, label="UInt32"), "inboundRootCriteria": SimTypePointer(SimStruct({"certData": SimStruct({"size": SimTypeInt(signed=False, label="UInt32"), "data": SimTypePointer(SimTypeChar(label="Byte"), offset=0)}, name="FWP_BYTE_BLOB", pack=False, align=None), "certHash": SimStruct({"size": SimTypeInt(signed=False, label="UInt32"), "data": SimTypePointer(SimTypeChar(label="Byte"), offset=0)}, name="FWP_BYTE_BLOB", pack=False, align=None), "eku": SimTypePointer(SimStruct({"numEku": SimTypeInt(signed=False, label="UInt32"), "eku": SimTypePointer(SimTypePointer(SimTypeChar(label="Byte"), offset=0), offset=0)}, name="IKEEXT_CERT_EKUS0", pack=False, align=None), offset=0), "name": SimTypePointer(SimStruct({"nameType": SimTypeInt(signed=False, label="IKEEXT_CERT_CRITERIA_NAME_TYPE"), "certName": SimTypePointer(SimTypeChar(label="Char"), offset=0)}, name="IKEEXT_CERT_NAME0", pack=False, align=None), offset=0), "flags": SimTypeInt(signed=False, label="UInt32")}, name="IKEEXT_CERTIFICATE_CRITERIA0", pack=False, align=None), offset=0)}, name="_Anonymous1_e__Struct", pack=False, align=None), "Anonymous2": SimStruct({"inboundEnterpriseStoreArraySize": SimTypeInt(signed=False, label="UInt32"), "inboundEnterpriseStoreCriteria": SimTypePointer(SimStruct({"certData": SimStruct({"size": SimTypeInt(signed=False, label="UInt32"), "data": SimTypePointer(SimTypeChar(label="Byte"), offset=0)}, name="FWP_BYTE_BLOB", pack=False, align=None), "certHash": SimStruct({"size": SimTypeInt(signed=False, label="UInt32"), "data": SimTypePointer(SimTypeChar(label="Byte"), offset=0)}, name="FWP_BYTE_BLOB", pack=False, align=None), "eku": SimTypePointer(SimStruct({"numEku": SimTypeInt(signed=False, label="UInt32"), "eku": SimTypePointer(SimTypePointer(SimTypeChar(label="Byte"), offset=0), offset=0)}, name="IKEEXT_CERT_EKUS0", pack=False, align=None), offset=0), "name": SimTypePointer(SimStruct({"nameType": SimTypeInt(signed=False, label="IKEEXT_CERT_CRITERIA_NAME_TYPE"), "certName": SimTypePointer(SimTypeChar(label="Char"), offset=0)}, name="IKEEXT_CERT_NAME0", pack=False, align=None), offset=0), "flags": SimTypeInt(signed=False, label="UInt32")}, name="IKEEXT_CERTIFICATE_CRITERIA0", pack=False, align=None), offset=0)}, name="_Anonymous2_e__Struct", pack=False, align=None), "Anonymous3": SimStruct({"inboundRootStoreArraySize": SimTypeInt(signed=False, label="UInt32"), "inboundTrustedRootStoreCriteria": SimTypePointer(SimStruct({"certData": SimStruct({"size": SimTypeInt(signed=False, label="UInt32"), "data": SimTypePointer(SimTypeChar(label="Byte"), offset=0)}, name="FWP_BYTE_BLOB", pack=False, align=None), "certHash": SimStruct({"size": SimTypeInt(signed=False, label="UInt32"), "data": SimTypePointer(SimTypeChar(label="Byte"), offset=0)}, name="FWP_BYTE_BLOB", pack=False, align=None), "eku": SimTypePointer(SimStruct({"numEku": SimTypeInt(signed=False, label="UInt32"), "eku": SimTypePointer(SimTypePointer(SimTypeChar(label="Byte"), offset=0), offset=0)}, name="IKEEXT_CERT_EKUS0", pack=False, align=None), offset=0), "name": SimTypePointer(SimStruct({"nameType": SimTypeInt(signed=False, label="IKEEXT_CERT_CRITERIA_NAME_TYPE"), "certName": SimTypePointer(SimTypeChar(label="Char"), offset=0)}, name="IKEEXT_CERT_NAME0", pack=False, align=None), offset=0), "flags": SimTypeInt(signed=False, label="UInt32")}, name="IKEEXT_CERTIFICATE_CRITERIA0", pack=False, align=None), offset=0)}, name="_Anonymous3_e__Struct", pack=False, align=None)}, name="<anon>", label="None"), "outboundConfigType": SimTypeInt(signed=False, label="IKEEXT_CERT_CONFIG_TYPE"), "Anonymous2": SimUnion({"Anonymous1": SimStruct({"outboundRootArraySize": SimTypeInt(signed=False, label="UInt32"), "outboundRootCriteria": SimTypePointer(SimStruct({"certData": SimStruct({"size": SimTypeInt(signed=False, label="UInt32"), "data": SimTypePointer(SimTypeChar(label="Byte"), offset=0)}, name="FWP_BYTE_BLOB", pack=False, align=None), "certHash": SimStruct({"size": SimTypeInt(signed=False, label="UInt32"), "data": SimTypePointer(SimTypeChar(label="Byte"), offset=0)}, name="FWP_BYTE_BLOB", pack=False, align=None), "eku": SimTypePointer(SimStruct({"numEku": SimTypeInt(signed=False, label="UInt32"), "eku": SimTypePointer(SimTypePointer(SimTypeChar(label="Byte"), offset=0), offset=0)}, name="IKEEXT_CERT_EKUS0", pack=False, align=None), offset=0), "name": SimTypePointer(SimStruct({"nameType": SimTypeInt(signed=False, label="IKEEXT_CERT_CRITERIA_NAME_TYPE"), "certName": SimTypePointer(SimTypeChar(label="Char"), offset=0)}, name="IKEEXT_CERT_NAME0", pack=False, align=None), offset=0), "flags": SimTypeInt(signed=False, label="UInt32")}, name="IKEEXT_CERTIFICATE_CRITERIA0", pack=False, align=None), offset=0)}, name="_Anonymous1_e__Struct", pack=False, align=None), "Anonymous2": SimStruct({"outboundEnterpriseStoreArraySize": SimTypeInt(signed=False, label="UInt32"), "outboundEnterpriseStoreCriteria": SimTypePointer(SimStruct({"certData": SimStruct({"size": SimTypeInt(signed=False, label="UInt32"), "data": SimTypePointer(SimTypeChar(label="Byte"), offset=0)}, name="FWP_BYTE_BLOB", pack=False, align=None), "certHash": SimStruct({"size": SimTypeInt(signed=False, label="UInt32"), "data": SimTypePointer(SimTypeChar(label="Byte"), offset=0)}, name="FWP_BYTE_BLOB", pack=False, align=None), "eku": SimTypePointer(SimStruct({"numEku": SimTypeInt(signed=False, label="UInt32"), "eku": SimTypePointer(SimTypePointer(SimTypeChar(label="Byte"), offset=0), offset=0)}, name="IKEEXT_CERT_EKUS0", pack=False, align=None), offset=0), "name": SimTypePointer(SimStruct({"nameType": SimTypeInt(signed=False, label="IKEEXT_CERT_CRITERIA_NAME_TYPE"), "certName": SimTypePointer(SimTypeChar(label="Char"), offset=0)}, name="IKEEXT_CERT_NAME0", pack=False, align=None), offset=0), "flags": SimTypeInt(signed=False, label="UInt32")}, name="IKEEXT_CERTIFICATE_CRITERIA0", pack=False, align=None), offset=0)}, name="_Anonymous2_e__Struct", pack=False, align=None), "Anonymous3": SimStruct({"outboundRootStoreArraySize": SimTypeInt(signed=False, label="UInt32"), "outboundTrustedRootStoreCriteria": SimTypePointer(SimStruct({"certData": SimStruct({"size": SimTypeInt(signed=False, label="UInt32"), "data": SimTypePointer(SimTypeChar(label="Byte"), offset=0)}, name="FWP_BYTE_BLOB", pack=False, align=None), "certHash": SimStruct({"size": SimTypeInt(signed=False, label="UInt32"), "data": SimTypePointer(SimTypeChar(label="Byte"), offset=0)}, name="FWP_BYTE_BLOB", pack=False, align=None), "eku": SimTypePointer(SimStruct({"numEku": SimTypeInt(signed=False, label="UInt32"), "eku": SimTypePointer(SimTypePointer(SimTypeChar(label="Byte"), offset=0), offset=0)}, name="IKEEXT_CERT_EKUS0", pack=False, align=None), offset=0), "name": SimTypePointer(SimStruct({"nameType": SimTypeInt(signed=False, label="IKEEXT_CERT_CRITERIA_NAME_TYPE"), "certName": SimTypePointer(SimTypeChar(label="Char"), offset=0)}, name="IKEEXT_CERT_NAME0", pack=False, align=None), offset=0), "flags": SimTypeInt(signed=False, label="UInt32")}, name="IKEEXT_CERTIFICATE_CRITERIA0", pack=False, align=None), offset=0)}, name="_Anonymous3_e__Struct", pack=False, align=None)}, name="<anon>", label="None"), "flags": SimTypeInt(signed=False, label="IKEEXT_CERT_AUTH"), "localCertLocationUrl": SimStruct({"size": SimTypeInt(signed=False, label="UInt32"), "data": SimTypePointer(SimTypeChar(label="Byte"), offset=0)}, name="FWP_BYTE_BLOB", pack=False, align=None)}, name="IKEEXT_CERTIFICATE_AUTHENTICATION2", pack=False, align=None), "kerberosAuthentication": SimStruct({"flags": SimTypeInt(signed=False, label="IKEEXT_KERBEROS_AUTHENTICATION_FLAGS"), "proxyServer": SimTypePointer(SimTypeChar(label="Char"), offset=0)}, name="IKEEXT_KERBEROS_AUTHENTICATION1", pack=False, align=None), "reservedAuthentication": SimStruct({"flags": SimTypeInt(signed=False, label="IKEEXT_RESERVED_AUTHENTICATION_FLAGS")}, name="IKEEXT_RESERVED_AUTHENTICATION0", pack=False, align=None), "ntlmV2Authentication": SimStruct({"flags": SimTypeInt(signed=False, label="UInt32")}, name="IKEEXT_NTLM_V2_AUTHENTICATION0", pack=False, align=None), "sslAuthentication": SimStruct({"inboundConfigType": SimTypeInt(signed=False, label="IKEEXT_CERT_CONFIG_TYPE"), "Anonymous1": SimUnion({"Anonymous1": SimStruct({"inboundRootArraySize": SimTypeInt(signed=False, label="UInt32"), "inboundRootCriteria": SimTypePointer(SimStruct({"certData": SimStruct({"size": SimTypeInt(signed=False, label="UInt32"), "data": SimTypePointer(SimTypeChar(label="Byte"), offset=0)}, name="FWP_BYTE_BLOB", pack=False, align=None), "certHash": SimStruct({"size": SimTypeInt(signed=False, label="UInt32"), "data": SimTypePointer(SimTypeChar(label="Byte"), offset=0)}, name="FWP_BYTE_BLOB", pack=False, align=None), "eku": SimTypePointer(SimStruct({"numEku": SimTypeInt(signed=False, label="UInt32"), "eku": SimTypePointer(SimTypePointer(SimTypeChar(label="Byte"), offset=0), offset=0)}, name="IKEEXT_CERT_EKUS0", pack=False, align=None), offset=0), "name": SimTypePointer(SimStruct({"nameType": SimTypeInt(signed=False, label="IKEEXT_CERT_CRITERIA_NAME_TYPE"), "certName": SimTypePointer(SimTypeChar(label="Char"), offset=0)}, name="IKEEXT_CERT_NAME0", pack=False, align=None), offset=0), "flags": SimTypeInt(signed=False, label="UInt32")}, name="IKEEXT_CERTIFICATE_CRITERIA0", pack=False, align=None), offset=0)}, name="_Anonymous1_e__Struct", pack=False, align=None), "Anonymous2": SimStruct({"inboundEnterpriseStoreArraySize": SimTypeInt(signed=False, label="UInt32"), "inboundEnterpriseStoreCriteria": SimTypePointer(SimStruct({"certData": SimStruct({"size": SimTypeInt(signed=False, label="UInt32"), "data": SimTypePointer(SimTypeChar(label="Byte"), offset=0)}, name="FWP_BYTE_BLOB", pack=False, align=None), "certHash": SimStruct({"size": SimTypeInt(signed=False, label="UInt32"), "data": SimTypePointer(SimTypeChar(label="Byte"), offset=0)}, name="FWP_BYTE_BLOB", pack=False, align=None), "eku": SimTypePointer(SimStruct({"numEku": SimTypeInt(signed=False, label="UInt32"), "eku": SimTypePointer(SimTypePointer(SimTypeChar(label="Byte"), offset=0), offset=0)}, name="IKEEXT_CERT_EKUS0", pack=False, align=None), offset=0), "name": SimTypePointer(SimStruct({"nameType": SimTypeInt(signed=False, label="IKEEXT_CERT_CRITERIA_NAME_TYPE"), "certName": SimTypePointer(SimTypeChar(label="Char"), offset=0)}, name="IKEEXT_CERT_NAME0", pack=False, align=None), offset=0), "flags": SimTypeInt(signed=False, label="UInt32")}, name="IKEEXT_CERTIFICATE_CRITERIA0", pack=False, align=None), offset=0)}, name="_Anonymous2_e__Struct", pack=False, align=None), "Anonymous3": SimStruct({"inboundRootStoreArraySize": SimTypeInt(signed=False, label="UInt32"), "inboundTrustedRootStoreCriteria": SimTypePointer(SimStruct({"certData": SimStruct({"size": SimTypeInt(signed=False, label="UInt32"), "data": SimTypePointer(SimTypeChar(label="Byte"), offset=0)}, name="FWP_BYTE_BLOB", pack=False, align=None), "certHash": SimStruct({"size": SimTypeInt(signed=False, label="UInt32"), "data": SimTypePointer(SimTypeChar(label="Byte"), offset=0)}, name="FWP_BYTE_BLOB", pack=False, align=None), "eku": SimTypePointer(SimStruct({"numEku": SimTypeInt(signed=False, label="UInt32"), "eku": SimTypePointer(SimTypePointer(SimTypeChar(label="Byte"), offset=0), offset=0)}, name="IKEEXT_CERT_EKUS0", pack=False, align=None), offset=0), "name": SimTypePointer(SimStruct({"nameType": SimTypeInt(signed=False, label="IKEEXT_CERT_CRITERIA_NAME_TYPE"), "certName": SimTypePointer(SimTypeChar(label="Char"), offset=0)}, name="IKEEXT_CERT_NAME0", pack=False, align=None), offset=0), "flags": SimTypeInt(signed=False, label="UInt32")}, name="IKEEXT_CERTIFICATE_CRITERIA0", pack=False, align=None), offset=0)}, name="_Anonymous3_e__Struct", pack=False, align=None)}, name="<anon>", label="None"), "outboundConfigType": SimTypeInt(signed=False, label="IKEEXT_CERT_CONFIG_TYPE"), "Anonymous2": SimUnion({"Anonymous1": SimStruct({"outboundRootArraySize": SimTypeInt(signed=False, label="UInt32"), "outboundRootCriteria": SimTypePointer(SimStruct({"certData": SimStruct({"size": SimTypeInt(signed=False, label="UInt32"), "data": SimTypePointer(SimTypeChar(label="Byte"), offset=0)}, name="FWP_BYTE_BLOB", pack=False, align=None), "certHash": SimStruct({"size": SimTypeInt(signed=False, label="UInt32"), "data": SimTypePointer(SimTypeChar(label="Byte"), offset=0)}, name="FWP_BYTE_BLOB", pack=False, align=None), "eku": SimTypePointer(SimStruct({"numEku": SimTypeInt(signed=False, label="UInt32"), "eku": SimTypePointer(SimTypePointer(SimTypeChar(label="Byte"), offset=0), offset=0)}, name="IKEEXT_CERT_EKUS0", pack=False, align=None), offset=0), "name": SimTypePointer(SimStruct({"nameType": SimTypeInt(signed=False, label="IKEEXT_CERT_CRITERIA_NAME_TYPE"), "certName": SimTypePointer(SimTypeChar(label="Char"), offset=0)}, name="IKEEXT_CERT_NAME0", pack=False, align=None), offset=0), "flags": SimTypeInt(signed=False, label="UInt32")}, name="IKEEXT_CERTIFICATE_CRITERIA0", pack=False, align=None), offset=0)}, name="_Anonymous1_e__Struct", pack=False, align=None), "Anonymous2": SimStruct({"outboundEnterpriseStoreArraySize": SimTypeInt(signed=False, label="UInt32"), "outboundEnterpriseStoreCriteria": SimTypePointer(SimStruct({"certData": SimStruct({"size": SimTypeInt(signed=False, label="UInt32"), "data": SimTypePointer(SimTypeChar(label="Byte"), offset=0)}, name="FWP_BYTE_BLOB", pack=False, align=None), "certHash": SimStruct({"size": SimTypeInt(signed=False, label="UInt32"), "data": SimTypePointer(SimTypeChar(label="Byte"), offset=0)}, name="FWP_BYTE_BLOB", pack=False, align=None), "eku": SimTypePointer(SimStruct({"numEku": SimTypeInt(signed=False, label="UInt32"), "eku": SimTypePointer(SimTypePointer(SimTypeChar(label="Byte"), offset=0), offset=0)}, name="IKEEXT_CERT_EKUS0", pack=False, align=None), offset=0), "name": SimTypePointer(SimStruct({"nameType": SimTypeInt(signed=False, label="IKEEXT_CERT_CRITERIA_NAME_TYPE"), "certName": SimTypePointer(SimTypeChar(label="Char"), offset=0)}, name="IKEEXT_CERT_NAME0", pack=False, align=None), offset=0), "flags": SimTypeInt(signed=False, label="UInt32")}, name="IKEEXT_CERTIFICATE_CRITERIA0", pack=False, align=None), offset=0)}, name="_Anonymous2_e__Struct", pack=False, align=None), "Anonymous3": SimStruct({"outboundRootStoreArraySize": SimTypeInt(signed=False, label="UInt32"), "outboundTrustedRootStoreCriteria": SimTypePointer(SimStruct({"certData": SimStruct({"size": SimTypeInt(signed=False, label="UInt32"), "data": SimTypePointer(SimTypeChar(label="Byte"), offset=0)}, name="FWP_BYTE_BLOB", pack=False, align=None), "certHash": SimStruct({"size": SimTypeInt(signed=False, label="UInt32"), "data": SimTypePointer(SimTypeChar(label="Byte"), offset=0)}, name="FWP_BYTE_BLOB", pack=False, align=None), "eku": SimTypePointer(SimStruct({"numEku": SimTypeInt(signed=False, label="UInt32"), "eku": SimTypePointer(SimTypePointer(SimTypeChar(label="Byte"), offset=0), offset=0)}, name="IKEEXT_CERT_EKUS0", pack=False, align=None), offset=0), "name": SimTypePointer(SimStruct({"nameType": SimTypeInt(signed=False, label="IKEEXT_CERT_CRITERIA_NAME_TYPE"), "certName": SimTypePointer(SimTypeChar(label="Char"), offset=0)}, name="IKEEXT_CERT_NAME0", pack=False, align=None), offset=0), "flags": SimTypeInt(signed=False, label="UInt32")}, name="IKEEXT_CERTIFICATE_CRITERIA0", pack=False, align=None), offset=0)}, name="_Anonymous3_e__Struct", pack=False, align=None)}, name="<anon>", label="None"), "flags": SimTypeInt(signed=False, label="IKEEXT_CERT_AUTH"), "localCertLocationUrl": SimStruct({"size": SimTypeInt(signed=False, label="UInt32"), "data": SimTypePointer(SimTypeChar(label="Byte"), offset=0)}, name="FWP_BYTE_BLOB", pack=False, align=None)}, name="IKEEXT_CERTIFICATE_AUTHENTICATION2", pack=False, align=None), "cgaAuthentication": SimStruct({"keyContainerName": SimTypePointer(SimTypeChar(label="Char"), offset=0), "cspName": SimTypePointer(SimTypeChar(label="Char"), offset=0), "cspType": SimTypeInt(signed=False, label="UInt32"), "cgaModifier": SimStruct({"byteArray16": SimTypeFixedSizeArray(SimTypeChar(label="Byte"), 16)}, name="FWP_BYTE_ARRAY16", pack=False, align=None), "cgaCollisionCount": SimTypeChar(label="Byte")}, name="IKEEXT_IPV6_CGA_AUTHENTICATION0", pack=False, align=None), "eapAuthentication": SimStruct({"flags": SimTypeInt(signed=False, label="IKEEXT_EAP_AUTHENTICATION_FLAGS")}, name="IKEEXT_EAP_AUTHENTICATION0", pack=False, align=None)}, name="<anon>", label="None")}, name="IKEEXT_AUTHENTICATION_METHOD2", pack=False, align=None), offset=0), "initiatorImpersonationType": SimTypeInt(signed=False, label="IKEEXT_AUTHENTICATION_IMPERSONATION_TYPE"), "numIkeProposals": SimTypeInt(signed=False, label="UInt32"), "ikeProposals": SimTypePointer(SimStruct({"cipherAlgorithm": SimStruct({"algoIdentifier": SimTypeInt(signed=False, label="IKEEXT_CIPHER_TYPE"), "keyLen": SimTypeInt(signed=False, label="UInt32"), "rounds": SimTypeInt(signed=False, label="UInt32")}, name="IKEEXT_CIPHER_ALGORITHM0", pack=False, align=None), "integrityAlgorithm": SimStruct({"algoIdentifier": SimTypeInt(signed=False, label="IKEEXT_INTEGRITY_TYPE")}, name="IKEEXT_INTEGRITY_ALGORITHM0", pack=False, align=None), "maxLifetimeSeconds": SimTypeInt(signed=False, label="UInt32"), "dhGroup": SimTypeInt(signed=False, label="IKEEXT_DH_GROUP"), "quickModeLimit": SimTypeInt(signed=False, label="UInt32")}, name="IKEEXT_PROPOSAL0", pack=False, align=None), offset=0), "flags": SimTypeInt(signed=False, label="IKEEXT_POLICY_FLAG"), "maxDynamicFilters": SimTypeInt(signed=False, label="UInt32"), "retransmitDurationSecs": SimTypeInt(signed=False, label="UInt32")}, name="IKEEXT_POLICY2", pack=False, align=None), offset=0), "authIpMmPolicy": SimTypePointer(SimStruct({"softExpirationTime": SimTypeInt(signed=False, label="UInt32"), "numAuthenticationMethods": SimTypeInt(signed=False, label="UInt32"), "authenticationMethods": SimTypePointer(SimStruct({"authenticationMethodType": SimTypeInt(signed=False, label="IKEEXT_AUTHENTICATION_METHOD_TYPE"), "Anonymous": SimUnion({"presharedKeyAuthentication": SimStruct({"presharedKey": SimStruct({"size": SimTypeInt(signed=False, label="UInt32"), "data": SimTypePointer(SimTypeChar(label="Byte"), offset=0)}, name="FWP_BYTE_BLOB", pack=False, align=None), "flags": SimTypeInt(signed=False, label="IKEEXT_PRESHARED_KEY_AUTHENTICATION_FLAGS")}, name="IKEEXT_PRESHARED_KEY_AUTHENTICATION1", pack=False, align=None), "certificateAuthentication": SimStruct({"inboundConfigType": SimTypeInt(signed=False, label="IKEEXT_CERT_CONFIG_TYPE"), "Anonymous1": SimUnion({"Anonymous1": SimStruct({"inboundRootArraySize": SimTypeInt(signed=False, label="UInt32"), "inboundRootCriteria": SimTypePointer(SimStruct({"certData": SimStruct({"size": SimTypeInt(signed=False, label="UInt32"), "data": SimTypePointer(SimTypeChar(label="Byte"), offset=0)}, name="FWP_BYTE_BLOB", pack=False, align=None), "certHash": SimStruct({"size": SimTypeInt(signed=False, label="UInt32"), "data": SimTypePointer(SimTypeChar(label="Byte"), offset=0)}, name="FWP_BYTE_BLOB", pack=False, align=None), "eku": SimTypePointer(SimStruct({"numEku": SimTypeInt(signed=False, label="UInt32"), "eku": SimTypePointer(SimTypePointer(SimTypeChar(label="Byte"), offset=0), offset=0)}, name="IKEEXT_CERT_EKUS0", pack=False, align=None), offset=0), "name": SimTypePointer(SimStruct({"nameType": SimTypeInt(signed=False, label="IKEEXT_CERT_CRITERIA_NAME_TYPE"), "certName": SimTypePointer(SimTypeChar(label="Char"), offset=0)}, name="IKEEXT_CERT_NAME0", pack=False, align=None), offset=0), "flags": SimTypeInt(signed=False, label="UInt32")}, name="IKEEXT_CERTIFICATE_CRITERIA0", pack=False, align=None), offset=0)}, name="_Anonymous1_e__Struct", pack=False, align=None), "Anonymous2": SimStruct({"inboundEnterpriseStoreArraySize": SimTypeInt(signed=False, label="UInt32"), "inboundEnterpriseStoreCriteria": SimTypePointer(SimStruct({"certData": SimStruct({"size": SimTypeInt(signed=False, label="UInt32"), "data": SimTypePointer(SimTypeChar(label="Byte"), offset=0)}, name="FWP_BYTE_BLOB", pack=False, align=None), "certHash": SimStruct({"size": SimTypeInt(signed=False, label="UInt32"), "data": SimTypePointer(SimTypeChar(label="Byte"), offset=0)}, name="FWP_BYTE_BLOB", pack=False, align=None), "eku": SimTypePointer(SimStruct({"numEku": SimTypeInt(signed=False, label="UInt32"), "eku": SimTypePointer(SimTypePointer(SimTypeChar(label="Byte"), offset=0), offset=0)}, name="IKEEXT_CERT_EKUS0", pack=False, align=None), offset=0), "name": SimTypePointer(SimStruct({"nameType": SimTypeInt(signed=False, label="IKEEXT_CERT_CRITERIA_NAME_TYPE"), "certName": SimTypePointer(SimTypeChar(label="Char"), offset=0)}, name="IKEEXT_CERT_NAME0", pack=False, align=None), offset=0), "flags": SimTypeInt(signed=False, label="UInt32")}, name="IKEEXT_CERTIFICATE_CRITERIA0", pack=False, align=None), offset=0)}, name="_Anonymous2_e__Struct", pack=False, align=None), "Anonymous3": SimStruct({"inboundRootStoreArraySize": SimTypeInt(signed=False, label="UInt32"), "inboundTrustedRootStoreCriteria": SimTypePointer(SimStruct({"certData": SimStruct({"size": SimTypeInt(signed=False, label="UInt32"), "data": SimTypePointer(SimTypeChar(label="Byte"), offset=0)}, name="FWP_BYTE_BLOB", pack=False, align=None), "certHash": SimStruct({"size": SimTypeInt(signed=False, label="UInt32"), "data": SimTypePointer(SimTypeChar(label="Byte"), offset=0)}, name="FWP_BYTE_BLOB", pack=False, align=None), "eku": SimTypePointer(SimStruct({"numEku": SimTypeInt(signed=False, label="UInt32"), "eku": SimTypePointer(SimTypePointer(SimTypeChar(label="Byte"), offset=0), offset=0)}, name="IKEEXT_CERT_EKUS0", pack=False, align=None), offset=0), "name": SimTypePointer(SimStruct({"nameType": SimTypeInt(signed=False, label="IKEEXT_CERT_CRITERIA_NAME_TYPE"), "certName": SimTypePointer(SimTypeChar(label="Char"), offset=0)}, name="IKEEXT_CERT_NAME0", pack=False, align=None), offset=0), "flags": SimTypeInt(signed=False, label="UInt32")}, name="IKEEXT_CERTIFICATE_CRITERIA0", pack=False, align=None), offset=0)}, name="_Anonymous3_e__Struct", pack=False, align=None)}, name="<anon>", label="None"), "outboundConfigType": SimTypeInt(signed=False, label="IKEEXT_CERT_CONFIG_TYPE"), "Anonymous2": SimUnion({"Anonymous1": SimStruct({"outboundRootArraySize": SimTypeInt(signed=False, label="UInt32"), "outboundRootCriteria": SimTypePointer(SimStruct({"certData": SimStruct({"size": SimTypeInt(signed=False, label="UInt32"), "data": SimTypePointer(SimTypeChar(label="Byte"), offset=0)}, name="FWP_BYTE_BLOB", pack=False, align=None), "certHash": SimStruct({"size": SimTypeInt(signed=False, label="UInt32"), "data": SimTypePointer(SimTypeChar(label="Byte"), offset=0)}, name="FWP_BYTE_BLOB", pack=False, align=None), "eku": SimTypePointer(SimStruct({"numEku": SimTypeInt(signed=False, label="UInt32"), "eku": SimTypePointer(SimTypePointer(SimTypeChar(label="Byte"), offset=0), offset=0)}, name="IKEEXT_CERT_EKUS0", pack=False, align=None), offset=0), "name": SimTypePointer(SimStruct({"nameType": SimTypeInt(signed=False, label="IKEEXT_CERT_CRITERIA_NAME_TYPE"), "certName": SimTypePointer(SimTypeChar(label="Char"), offset=0)}, name="IKEEXT_CERT_NAME0", pack=False, align=None), offset=0), "flags": SimTypeInt(signed=False, label="UInt32")}, name="IKEEXT_CERTIFICATE_CRITERIA0", pack=False, align=None), offset=0)}, name="_Anonymous1_e__Struct", pack=False, align=None), "Anonymous2": SimStruct({"outboundEnterpriseStoreArraySize": SimTypeInt(signed=False, label="UInt32"), "outboundEnterpriseStoreCriteria": SimTypePointer(SimStruct({"certData": SimStruct({"size": SimTypeInt(signed=False, label="UInt32"), "data": SimTypePointer(SimTypeChar(label="Byte"), offset=0)}, name="FWP_BYTE_BLOB", pack=False, align=None), "certHash": SimStruct({"size": SimTypeInt(signed=False, label="UInt32"), "data": SimTypePointer(SimTypeChar(label="Byte"), offset=0)}, name="FWP_BYTE_BLOB", pack=False, align=None), "eku": SimTypePointer(SimStruct({"numEku": SimTypeInt(signed=False, label="UInt32"), "eku": SimTypePointer(SimTypePointer(SimTypeChar(label="Byte"), offset=0), offset=0)}, name="IKEEXT_CERT_EKUS0", pack=False, align=None), offset=0), "name": SimTypePointer(SimStruct({"nameType": SimTypeInt(signed=False, label="IKEEXT_CERT_CRITERIA_NAME_TYPE"), "certName": SimTypePointer(SimTypeChar(label="Char"), offset=0)}, name="IKEEXT_CERT_NAME0", pack=False, align=None), offset=0), "flags": SimTypeInt(signed=False, label="UInt32")}, name="IKEEXT_CERTIFICATE_CRITERIA0", pack=False, align=None), offset=0)}, name="_Anonymous2_e__Struct", pack=False, align=None), "Anonymous3": SimStruct({"outboundRootStoreArraySize": SimTypeInt(signed=False, label="UInt32"), "outboundTrustedRootStoreCriteria": SimTypePointer(SimStruct({"certData": SimStruct({"size": SimTypeInt(signed=False, label="UInt32"), "data": SimTypePointer(SimTypeChar(label="Byte"), offset=0)}, name="FWP_BYTE_BLOB", pack=False, align=None), "certHash": SimStruct({"size": SimTypeInt(signed=False, label="UInt32"), "data": SimTypePointer(SimTypeChar(label="Byte"), offset=0)}, name="FWP_BYTE_BLOB", pack=False, align=None), "eku": SimTypePointer(SimStruct({"numEku": SimTypeInt(signed=False, label="UInt32"), "eku": SimTypePointer(SimTypePointer(SimTypeChar(label="Byte"), offset=0), offset=0)}, name="IKEEXT_CERT_EKUS0", pack=False, align=None), offset=0), "name": SimTypePointer(SimStruct({"nameType": SimTypeInt(signed=False, label="IKEEXT_CERT_CRITERIA_NAME_TYPE"), "certName": SimTypePointer(SimTypeChar(label="Char"), offset=0)}, name="IKEEXT_CERT_NAME0", pack=False, align=None), offset=0), "flags": SimTypeInt(signed=False, label="UInt32")}, name="IKEEXT_CERTIFICATE_CRITERIA0", pack=False, align=None), offset=0)}, name="_Anonymous3_e__Struct", pack=False, align=None)}, name="<anon>", label="None"), "flags": SimTypeInt(signed=False, label="IKEEXT_CERT_AUTH"), "localCertLocationUrl": SimStruct({"size": SimTypeInt(signed=False, label="UInt32"), "data": SimTypePointer(SimTypeChar(label="Byte"), offset=0)}, name="FWP_BYTE_BLOB", pack=False, align=None)}, name="IKEEXT_CERTIFICATE_AUTHENTICATION2", pack=False, align=None), "kerberosAuthentication": SimStruct({"flags": SimTypeInt(signed=False, label="IKEEXT_KERBEROS_AUTHENTICATION_FLAGS"), "proxyServer": SimTypePointer(SimTypeChar(label="Char"), offset=0)}, name="IKEEXT_KERBEROS_AUTHENTICATION1", pack=False, align=None), "reservedAuthentication": SimStruct({"flags": SimTypeInt(signed=False, label="IKEEXT_RESERVED_AUTHENTICATION_FLAGS")}, name="IKEEXT_RESERVED_AUTHENTICATION0", pack=False, align=None), "ntlmV2Authentication": SimStruct({"flags": SimTypeInt(signed=False, label="UInt32")}, name="IKEEXT_NTLM_V2_AUTHENTICATION0", pack=False, align=None), "sslAuthentication": SimStruct({"inboundConfigType": SimTypeInt(signed=False, label="IKEEXT_CERT_CONFIG_TYPE"), "Anonymous1": SimUnion({"Anonymous1": SimStruct({"inboundRootArraySize": SimTypeInt(signed=False, label="UInt32"), "inboundRootCriteria": SimTypePointer(SimStruct({"certData": SimStruct({"size": SimTypeInt(signed=False, label="UInt32"), "data": SimTypePointer(SimTypeChar(label="Byte"), offset=0)}, name="FWP_BYTE_BLOB", pack=False, align=None), "certHash": SimStruct({"size": SimTypeInt(signed=False, label="UInt32"), "data": SimTypePointer(SimTypeChar(label="Byte"), offset=0)}, name="FWP_BYTE_BLOB", pack=False, align=None), "eku": SimTypePointer(SimStruct({"numEku": SimTypeInt(signed=False, label="UInt32"), "eku": SimTypePointer(SimTypePointer(SimTypeChar(label="Byte"), offset=0), offset=0)}, name="IKEEXT_CERT_EKUS0", pack=False, align=None), offset=0), "name": SimTypePointer(SimStruct({"nameType": SimTypeInt(signed=False, label="IKEEXT_CERT_CRITERIA_NAME_TYPE"), "certName": SimTypePointer(SimTypeChar(label="Char"), offset=0)}, name="IKEEXT_CERT_NAME0", pack=False, align=None), offset=0), "flags": SimTypeInt(signed=False, label="UInt32")}, name="IKEEXT_CERTIFICATE_CRITERIA0", pack=False, align=None), offset=0)}, name="_Anonymous1_e__Struct", pack=False, align=None), "Anonymous2": SimStruct({"inboundEnterpriseStoreArraySize": SimTypeInt(signed=False, label="UInt32"), "inboundEnterpriseStoreCriteria": SimTypePointer(SimStruct({"certData": SimStruct({"size": SimTypeInt(signed=False, label="UInt32"), "data": SimTypePointer(SimTypeChar(label="Byte"), offset=0)}, name="FWP_BYTE_BLOB", pack=False, align=None), "certHash": SimStruct({"size": SimTypeInt(signed=False, label="UInt32"), "data": SimTypePointer(SimTypeChar(label="Byte"), offset=0)}, name="FWP_BYTE_BLOB", pack=False, align=None), "eku": SimTypePointer(SimStruct({"numEku": SimTypeInt(signed=False, label="UInt32"), "eku": SimTypePointer(SimTypePointer(SimTypeChar(label="Byte"), offset=0), offset=0)}, name="IKEEXT_CERT_EKUS0", pack=False, align=None), offset=0), "name": SimTypePointer(SimStruct({"nameType": SimTypeInt(signed=False, label="IKEEXT_CERT_CRITERIA_NAME_TYPE"), "certName": SimTypePointer(SimTypeChar(label="Char"), offset=0)}, name="IKEEXT_CERT_NAME0", pack=False, align=None), offset=0), "flags": SimTypeInt(signed=False, label="UInt32")}, name="IKEEXT_CERTIFICATE_CRITERIA0", pack=False, align=None), offset=0)}, name="_Anonymous2_e__Struct", pack=False, align=None), "Anonymous3": SimStruct({"inboundRootStoreArraySize": SimTypeInt(signed=False, label="UInt32"), "inboundTrustedRootStoreCriteria": SimTypePointer(SimStruct({"certData": SimStruct({"size": SimTypeInt(signed=False, label="UInt32"), "data": SimTypePointer(SimTypeChar(label="Byte"), offset=0)}, name="FWP_BYTE_BLOB", pack=False, align=None), "certHash": SimStruct({"size": SimTypeInt(signed=False, label="UInt32"), "data": SimTypePointer(SimTypeChar(label="Byte"), offset=0)}, name="FWP_BYTE_BLOB", pack=False, align=None), "eku": SimTypePointer(SimStruct({"numEku": SimTypeInt(signed=False, label="UInt32"), "eku": SimTypePointer(SimTypePointer(SimTypeChar(label="Byte"), offset=0), offset=0)}, name="IKEEXT_CERT_EKUS0", pack=False, align=None), offset=0), "name": SimTypePointer(SimStruct({"nameType": SimTypeInt(signed=False, label="IKEEXT_CERT_CRITERIA_NAME_TYPE"), "certName": SimTypePointer(SimTypeChar(label="Char"), offset=0)}, name="IKEEXT_CERT_NAME0", pack=False, align=None), offset=0), "flags": SimTypeInt(signed=False, label="UInt32")}, name="IKEEXT_CERTIFICATE_CRITERIA0", pack=False, align=None), offset=0)}, name="_Anonymous3_e__Struct", pack=False, align=None)}, name="<anon>", label="None"), "outboundConfigType": SimTypeInt(signed=False, label="IKEEXT_CERT_CONFIG_TYPE"), "Anonymous2": SimUnion({"Anonymous1": SimStruct({"outboundRootArraySize": SimTypeInt(signed=False, label="UInt32"), "outboundRootCriteria": SimTypePointer(SimStruct({"certData": SimStruct({"size": SimTypeInt(signed=False, label="UInt32"), "data": SimTypePointer(SimTypeChar(label="Byte"), offset=0)}, name="FWP_BYTE_BLOB", pack=False, align=None), "certHash": SimStruct({"size": SimTypeInt(signed=False, label="UInt32"), "data": SimTypePointer(SimTypeChar(label="Byte"), offset=0)}, name="FWP_BYTE_BLOB", pack=False, align=None), "eku": SimTypePointer(SimStruct({"numEku": SimTypeInt(signed=False, label="UInt32"), "eku": SimTypePointer(SimTypePointer(SimTypeChar(label="Byte"), offset=0), offset=0)}, name="IKEEXT_CERT_EKUS0", pack=False, align=None), offset=0), "name": SimTypePointer(SimStruct({"nameType": SimTypeInt(signed=False, label="IKEEXT_CERT_CRITERIA_NAME_TYPE"), "certName": SimTypePointer(SimTypeChar(label="Char"), offset=0)}, name="IKEEXT_CERT_NAME0", pack=False, align=None), offset=0), "flags": SimTypeInt(signed=False, label="UInt32")}, name="IKEEXT_CERTIFICATE_CRITERIA0", pack=False, align=None), offset=0)}, name="_Anonymous1_e__Struct", pack=False, align=None), "Anonymous2": SimStruct({"outboundEnterpriseStoreArraySize": SimTypeInt(signed=False, label="UInt32"), "outboundEnterpriseStoreCriteria": SimTypePointer(SimStruct({"certData": SimStruct({"size": SimTypeInt(signed=False, label="UInt32"), "data": SimTypePointer(SimTypeChar(label="Byte"), offset=0)}, name="FWP_BYTE_BLOB", pack=False, align=None), "certHash": SimStruct({"size": SimTypeInt(signed=False, label="UInt32"), "data": SimTypePointer(SimTypeChar(label="Byte"), offset=0)}, name="FWP_BYTE_BLOB", pack=False, align=None), "eku": SimTypePointer(SimStruct({"numEku": SimTypeInt(signed=False, label="UInt32"), "eku": SimTypePointer(SimTypePointer(SimTypeChar(label="Byte"), offset=0), offset=0)}, name="IKEEXT_CERT_EKUS0", pack=False, align=None), offset=0), "name": SimTypePointer(SimStruct({"nameType": SimTypeInt(signed=False, label="IKEEXT_CERT_CRITERIA_NAME_TYPE"), "certName": SimTypePointer(SimTypeChar(label="Char"), offset=0)}, name="IKEEXT_CERT_NAME0", pack=False, align=None), offset=0), "flags": SimTypeInt(signed=False, label="UInt32")}, name="IKEEXT_CERTIFICATE_CRITERIA0", pack=False, align=None), offset=0)}, name="_Anonymous2_e__Struct", pack=False, align=None), "Anonymous3": SimStruct({"outboundRootStoreArraySize": SimTypeInt(signed=False, label="UInt32"), "outboundTrustedRootStoreCriteria": SimTypePointer(SimStruct({"certData": SimStruct({"size": SimTypeInt(signed=False, label="UInt32"), "data": SimTypePointer(SimTypeChar(label="Byte"), offset=0)}, name="FWP_BYTE_BLOB", pack=False, align=None), "certHash": SimStruct({"size": SimTypeInt(signed=False, label="UInt32"), "data": SimTypePointer(SimTypeChar(label="Byte"), offset=0)}, name="FWP_BYTE_BLOB", pack=False, align=None), "eku": SimTypePointer(SimStruct({"numEku": SimTypeInt(signed=False, label="UInt32"), "eku": SimTypePointer(SimTypePointer(SimTypeChar(label="Byte"), offset=0), offset=0)}, name="IKEEXT_CERT_EKUS0", pack=False, align=None), offset=0), "name": SimTypePointer(SimStruct({"nameType": SimTypeInt(signed=False, label="IKEEXT_CERT_CRITERIA_NAME_TYPE"), "certName": SimTypePointer(SimTypeChar(label="Char"), offset=0)}, name="IKEEXT_CERT_NAME0", pack=False, align=None), offset=0), "flags": SimTypeInt(signed=False, label="UInt32")}, name="IKEEXT_CERTIFICATE_CRITERIA0", pack=False, align=None), offset=0)}, name="_Anonymous3_e__Struct", pack=False, align=None)}, name="<anon>", label="None"), "flags": SimTypeInt(signed=False, label="IKEEXT_CERT_AUTH"), "localCertLocationUrl": SimStruct({"size": SimTypeInt(signed=False, label="UInt32"), "data": SimTypePointer(SimTypeChar(label="Byte"), offset=0)}, name="FWP_BYTE_BLOB", pack=False, align=None)}, name="IKEEXT_CERTIFICATE_AUTHENTICATION2", pack=False, align=None), "cgaAuthentication": SimStruct({"keyContainerName": SimTypePointer(SimTypeChar(label="Char"), offset=0), "cspName": SimTypePointer(SimTypeChar(label="Char"), offset=0), "cspType": SimTypeInt(signed=False, label="UInt32"), "cgaModifier": SimStruct({"byteArray16": SimTypeFixedSizeArray(SimTypeChar(label="Byte"), 16)}, name="FWP_BYTE_ARRAY16", pack=False, align=None), "cgaCollisionCount": SimTypeChar(label="Byte")}, name="IKEEXT_IPV6_CGA_AUTHENTICATION0", pack=False, align=None), "eapAuthentication": SimStruct({"flags": SimTypeInt(signed=False, label="IKEEXT_EAP_AUTHENTICATION_FLAGS")}, name="IKEEXT_EAP_AUTHENTICATION0", pack=False, align=None)}, name="<anon>", label="None")}, name="IKEEXT_AUTHENTICATION_METHOD2", pack=False, align=None), offset=0), "initiatorImpersonationType": SimTypeInt(signed=False, label="IKEEXT_AUTHENTICATION_IMPERSONATION_TYPE"), "numIkeProposals": SimTypeInt(signed=False, label="UInt32"), "ikeProposals": SimTypePointer(SimStruct({"cipherAlgorithm": SimStruct({"algoIdentifier": SimTypeInt(signed=False, label="IKEEXT_CIPHER_TYPE"), "keyLen": SimTypeInt(signed=False, label="UInt32"), "rounds": SimTypeInt(signed=False, label="UInt32")}, name="IKEEXT_CIPHER_ALGORITHM0", pack=False, align=None), "integrityAlgorithm": SimStruct({"algoIdentifier": SimTypeInt(signed=False, label="IKEEXT_INTEGRITY_TYPE")}, name="IKEEXT_INTEGRITY_ALGORITHM0", pack=False, align=None), "maxLifetimeSeconds": SimTypeInt(signed=False, label="UInt32"), "dhGroup": SimTypeInt(signed=False, label="IKEEXT_DH_GROUP"), "quickModeLimit": SimTypeInt(signed=False, label="UInt32")}, name="IKEEXT_PROPOSAL0", pack=False, align=None), offset=0), "flags": SimTypeInt(signed=False, label="IKEEXT_POLICY_FLAG"), "maxDynamicFilters": SimTypeInt(signed=False, label="UInt32"), "retransmitDurationSecs": SimTypeInt(signed=False, label="UInt32")}, name="IKEEXT_POLICY2", pack=False, align=None), offset=0), "dataBuffer": SimTypePointer(SimStruct({"size": SimTypeInt(signed=False, label="UInt32"), "data": SimTypePointer(SimTypeChar(label="Byte"), offset=0)}, name="FWP_BYTE_BLOB", pack=False, align=None), offset=0), "classifyOptions": SimTypePointer(SimStruct({"numOptions": SimTypeInt(signed=False, label="UInt32"), "options": SimTypePointer(SimStruct({"type": SimTypeInt(signed=False, label="FWP_CLASSIFY_OPTION_TYPE"), "value": SimStruct({"type": SimTypeInt(signed=False, label="FWP_DATA_TYPE"), "Anonymous": SimUnion({"uint8": SimTypeChar(label="Byte"), "uint16": SimTypeShort(signed=False, label="UInt16"), "uint32": SimTypeInt(signed=False, label="UInt32"), "uint64": SimTypePointer(SimTypeLongLong(signed=False, label="UInt64"), offset=0), "int8": SimTypeChar(label="SByte"), "int16": SimTypeShort(signed=True, label="Int16"), "int32": SimTypeInt(signed=True, label="Int32"), "int64": SimTypePointer(SimTypeLongLong(signed=True, label="Int64"), offset=0), "float32": SimTypeFloat(size=32), "double64": SimTypePointer(SimTypeFloat(size=64), offset=0), "byteArray16": SimTypePointer(SimStruct({"byteArray16": SimTypeFixedSizeArray(SimTypeChar(label="Byte"), 16)}, name="FWP_BYTE_ARRAY16", pack=False, align=None), offset=0), "byteBlob": SimTypePointer(SimStruct({"size": SimTypeInt(signed=False, label="UInt32"), "data": SimTypePointer(SimTypeChar(label="Byte"), offset=0)}, name="FWP_BYTE_BLOB", pack=False, align=None), offset=0), "sid": SimTypePointer(SimTypeBottom(label="SID"), offset=0), "sd": SimTypePointer(SimStruct({"size": SimTypeInt(signed=False, label="UInt32"), "data": SimTypePointer(SimTypeChar(label="Byte"), offset=0)}, name="FWP_BYTE_BLOB", pack=False, align=None), offset=0), "tokenInformation": SimTypePointer(SimStruct({"sidCount": SimTypeInt(signed=False, label="UInt32"), "sids": SimTypePointer(SimTypeBottom(label="SID_AND_ATTRIBUTES"), offset=0), "restrictedSidCount": SimTypeInt(signed=False, label="UInt32"), "restrictedSids": SimTypePointer(SimTypeBottom(label="SID_AND_ATTRIBUTES"), offset=0)}, name="FWP_TOKEN_INFORMATION", pack=False, align=None), offset=0), "tokenAccessInformation": SimTypePointer(SimStruct({"size": SimTypeInt(signed=False, label="UInt32"), "data": SimTypePointer(SimTypeChar(label="Byte"), offset=0)}, name="FWP_BYTE_BLOB", pack=False, align=None), offset=0), "unicodeString": SimTypePointer(SimTypeChar(label="Char"), offset=0), "byteArray6": SimTypePointer(SimStruct({"byteArray6": SimTypeFixedSizeArray(SimTypeChar(label="Byte"), 6)}, name="FWP_BYTE_ARRAY6", pack=False, align=None), offset=0), "bitmapArray64": SimTypePointer(SimStruct({"bitmapArray64": SimTypeFixedSizeArray(SimTypeChar(label="Byte"), 8)}, name="FWP_BITMAP_ARRAY64_", pack=False, align=None), offset=0)}, name="<anon>", label="None")}, name="FWP_VALUE0", pack=False, align=None)}, name="FWPM_CLASSIFY_OPTION0", pack=False, align=None), offset=0)}, name="FWPM_CLASSIFY_OPTIONS0", pack=False, align=None), offset=0), "ikeV2QmTunnelPolicy": SimTypePointer(SimStruct({"flags": SimTypeInt(signed=False, label="IPSEC_POLICY_FLAG"), "numIpsecProposals": SimTypeInt(signed=False, label="UInt32"), "ipsecProposals": SimTypePointer(SimStruct({"lifetime": SimStruct({"lifetimeSeconds": SimTypeInt(signed=False, label="UInt32"), "lifetimeKilobytes": SimTypeInt(signed=False, label="UInt32"), "lifetimePackets": SimTypeInt(signed=False, label="UInt32")}, name="IPSEC_SA_LIFETIME0", pack=False, align=None), "numSaTransforms": SimTypeInt(signed=False, label="UInt32"), "saTransforms": SimTypePointer(SimStruct({"ipsecTransformType": SimTypeInt(signed=False, label="IPSEC_TRANSFORM_TYPE"), "Anonymous": SimUnion({"ahTransform": SimTypePointer(SimStruct({"authTransformId": SimStruct({"authType": SimTypeInt(signed=False, label="IPSEC_AUTH_TYPE"), "authConfig": SimTypeChar(label="Byte")}, name="IPSEC_AUTH_TRANSFORM_ID0", pack=False, align=None), "cryptoModuleId": SimTypePointer(SimTypeBottom(label="Guid"), offset=0)}, name="IPSEC_AUTH_TRANSFORM0", pack=False, align=None), offset=0), "espAuthTransform": SimTypePointer(SimStruct({"authTransformId": SimStruct({"authType": SimTypeInt(signed=False, label="IPSEC_AUTH_TYPE"), "authConfig": SimTypeChar(label="Byte")}, name="IPSEC_AUTH_TRANSFORM_ID0", pack=False, align=None), "cryptoModuleId": SimTypePointer(SimTypeBottom(label="Guid"), offset=0)}, name="IPSEC_AUTH_TRANSFORM0", pack=False, align=None), offset=0), "espCipherTransform": SimTypePointer(SimStruct({"cipherTransformId": SimStruct({"cipherType": SimTypeInt(signed=False, label="IPSEC_CIPHER_TYPE"), "cipherConfig": SimTypeChar(label="Byte")}, name="IPSEC_CIPHER_TRANSFORM_ID0", pack=False, align=None), "cryptoModuleId": SimTypePointer(SimTypeBottom(label="Guid"), offset=0)}, name="IPSEC_CIPHER_TRANSFORM0", pack=False, align=None), offset=0), "espAuthAndCipherTransform": SimTypePointer(SimStruct({"authTransform": SimStruct({"authTransformId": SimStruct({"authType": SimTypeInt(signed=False, label="IPSEC_AUTH_TYPE"), "authConfig": SimTypeChar(label="Byte")}, name="IPSEC_AUTH_TRANSFORM_ID0", pack=False, align=None), "cryptoModuleId": SimTypePointer(SimTypeBottom(label="Guid"), offset=0)}, name="IPSEC_AUTH_TRANSFORM0", pack=False, align=None), "cipherTransform": SimStruct({"cipherTransformId": SimStruct({"cipherType": SimTypeInt(signed=False, label="IPSEC_CIPHER_TYPE"), "cipherConfig": SimTypeChar(label="Byte")}, name="IPSEC_CIPHER_TRANSFORM_ID0", pack=False, align=None), "cryptoModuleId": SimTypePointer(SimTypeBottom(label="Guid"), offset=0)}, name="IPSEC_CIPHER_TRANSFORM0", pack=False, align=None)}, name="IPSEC_AUTH_AND_CIPHER_TRANSFORM0", pack=False, align=None), offset=0), "espAuthFwTransform": SimTypePointer(SimStruct({"authTransformId": SimStruct({"authType": SimTypeInt(signed=False, label="IPSEC_AUTH_TYPE"), "authConfig": SimTypeChar(label="Byte")}, name="IPSEC_AUTH_TRANSFORM_ID0", pack=False, align=None), "cryptoModuleId": SimTypePointer(SimTypeBottom(label="Guid"), offset=0)}, name="IPSEC_AUTH_TRANSFORM0", pack=False, align=None), offset=0)}, name="<anon>", label="None")}, name="IPSEC_SA_TRANSFORM0", pack=False, align=None), offset=0), "pfsGroup": SimTypeInt(signed=False, label="IPSEC_PFS_GROUP")}, name="IPSEC_PROPOSAL0", pack=False, align=None), offset=0), "tunnelEndpoints": SimStruct({"ipVersion": SimTypeInt(signed=False, label="FWP_IP_VERSION"), "Anonymous1": SimUnion({"localV4Address": SimTypeInt(signed=False, label="UInt32"), "localV6Address": SimTypeFixedSizeArray(SimTypeChar(label="Byte"), 16)}, name="<anon>", label="None"), "Anonymous2": SimUnion({"remoteV4Address": SimTypeInt(signed=False, label="UInt32"), "remoteV6Address": SimTypeFixedSizeArray(SimTypeChar(label="Byte"), 16)}, name="<anon>", label="None"), "localIfLuid": SimTypeLongLong(signed=False, label="UInt64"), "remoteFqdn": SimTypePointer(SimTypeChar(label="Char"), offset=0), "numAddresses": SimTypeInt(signed=False, label="UInt32"), "remoteAddresses": SimTypePointer(SimStruct({"ipVersion": SimTypeInt(signed=False, label="FWP_IP_VERSION"), "Anonymous": SimUnion({"v4Address": SimTypeInt(signed=False, label="UInt32"), "v6Address": SimTypeFixedSizeArray(SimTypeChar(label="Byte"), 16)}, name="<anon>", label="None")}, name="IPSEC_TUNNEL_ENDPOINT0", pack=False, align=None), offset=0)}, name="IPSEC_TUNNEL_ENDPOINTS2", pack=False, align=None), "saIdleTimeout": SimStruct({"idleTimeoutSeconds": SimTypeInt(signed=False, label="UInt32"), "idleTimeoutSecondsFailOver": SimTypeInt(signed=False, label="UInt32")}, name="IPSEC_SA_IDLE_TIMEOUT0", pack=False, align=None), "emPolicy": SimTypePointer(SimStruct({"numAuthenticationMethods": SimTypeInt(signed=False, label="UInt32"), "authenticationMethods": SimTypePointer(SimStruct({"authenticationMethodType": SimTypeInt(signed=False, label="IKEEXT_AUTHENTICATION_METHOD_TYPE"), "Anonymous": SimUnion({"presharedKeyAuthentication": SimStruct({"presharedKey": SimStruct({"size": SimTypeInt(signed=False, label="UInt32"), "data": SimTypePointer(SimTypeChar(label="Byte"), offset=0)}, name="FWP_BYTE_BLOB", pack=False, align=None), "flags": SimTypeInt(signed=False, label="IKEEXT_PRESHARED_KEY_AUTHENTICATION_FLAGS")}, name="IKEEXT_PRESHARED_KEY_AUTHENTICATION1", pack=False, align=None), "certificateAuthentication": SimStruct({"inboundConfigType": SimTypeInt(signed=False, label="IKEEXT_CERT_CONFIG_TYPE"), "Anonymous1": SimUnion({"Anonymous1": SimStruct({"inboundRootArraySize": SimTypeInt(signed=False, label="UInt32"), "inboundRootCriteria": SimTypePointer(SimStruct({"certData": SimStruct({"size": SimTypeInt(signed=False, label="UInt32"), "data": SimTypePointer(SimTypeChar(label="Byte"), offset=0)}, name="FWP_BYTE_BLOB", pack=False, align=None), "certHash": SimStruct({"size": SimTypeInt(signed=False, label="UInt32"), "data": SimTypePointer(SimTypeChar(label="Byte"), offset=0)}, name="FWP_BYTE_BLOB", pack=False, align=None), "eku": SimTypePointer(SimStruct({"numEku": SimTypeInt(signed=False, label="UInt32"), "eku": SimTypePointer(SimTypePointer(SimTypeChar(label="Byte"), offset=0), offset=0)}, name="IKEEXT_CERT_EKUS0", pack=False, align=None), offset=0), "name": SimTypePointer(SimStruct({"nameType": SimTypeInt(signed=False, label="IKEEXT_CERT_CRITERIA_NAME_TYPE"), "certName": SimTypePointer(SimTypeChar(label="Char"), offset=0)}, name="IKEEXT_CERT_NAME0", pack=False, align=None), offset=0), "flags": SimTypeInt(signed=False, label="UInt32")}, name="IKEEXT_CERTIFICATE_CRITERIA0", pack=False, align=None), offset=0)}, name="_Anonymous1_e__Struct", pack=False, align=None), "Anonymous2": SimStruct({"inboundEnterpriseStoreArraySize": SimTypeInt(signed=False, label="UInt32"), "inboundEnterpriseStoreCriteria": SimTypePointer(SimStruct({"certData": SimStruct({"size": SimTypeInt(signed=False, label="UInt32"), "data": SimTypePointer(SimTypeChar(label="Byte"), offset=0)}, name="FWP_BYTE_BLOB", pack=False, align=None), "certHash": SimStruct({"size": SimTypeInt(signed=False, label="UInt32"), "data": SimTypePointer(SimTypeChar(label="Byte"), offset=0)}, name="FWP_BYTE_BLOB", pack=False, align=None), "eku": SimTypePointer(SimStruct({"numEku": SimTypeInt(signed=False, label="UInt32"), "eku": SimTypePointer(SimTypePointer(SimTypeChar(label="Byte"), offset=0), offset=0)}, name="IKEEXT_CERT_EKUS0", pack=False, align=None), offset=0), "name": SimTypePointer(SimStruct({"nameType": SimTypeInt(signed=False, label="IKEEXT_CERT_CRITERIA_NAME_TYPE"), "certName": SimTypePointer(SimTypeChar(label="Char"), offset=0)}, name="IKEEXT_CERT_NAME0", pack=False, align=None), offset=0), "flags": SimTypeInt(signed=False, label="UInt32")}, name="IKEEXT_CERTIFICATE_CRITERIA0", pack=False, align=None), offset=0)}, name="_Anonymous2_e__Struct", pack=False, align=None), "Anonymous3": SimStruct({"inboundRootStoreArraySize": SimTypeInt(signed=False, label="UInt32"), "inboundTrustedRootStoreCriteria": SimTypePointer(SimStruct({"certData": SimStruct({"size": SimTypeInt(signed=False, label="UInt32"), "data": SimTypePointer(SimTypeChar(label="Byte"), offset=0)}, name="FWP_BYTE_BLOB", pack=False, align=None), "certHash": SimStruct({"size": SimTypeInt(signed=False, label="UInt32"), "data": SimTypePointer(SimTypeChar(label="Byte"), offset=0)}, name="FWP_BYTE_BLOB", pack=False, align=None), "eku": SimTypePointer(SimStruct({"numEku": SimTypeInt(signed=False, label="UInt32"), "eku": SimTypePointer(SimTypePointer(SimTypeChar(label="Byte"), offset=0), offset=0)}, name="IKEEXT_CERT_EKUS0", pack=False, align=None), offset=0), "name": SimTypePointer(SimStruct({"nameType": SimTypeInt(signed=False, label="IKEEXT_CERT_CRITERIA_NAME_TYPE"), "certName": SimTypePointer(SimTypeChar(label="Char"), offset=0)}, name="IKEEXT_CERT_NAME0", pack=False, align=None), offset=0), "flags": SimTypeInt(signed=False, label="UInt32")}, name="IKEEXT_CERTIFICATE_CRITERIA0", pack=False, align=None), offset=0)}, name="_Anonymous3_e__Struct", pack=False, align=None)}, name="<anon>", label="None"), "outboundConfigType": SimTypeInt(signed=False, label="IKEEXT_CERT_CONFIG_TYPE"), "Anonymous2": SimUnion({"Anonymous1": SimStruct({"outboundRootArraySize": SimTypeInt(signed=False, label="UInt32"), "outboundRootCriteria": SimTypePointer(SimStruct({"certData": SimStruct({"size": SimTypeInt(signed=False, label="UInt32"), "data": SimTypePointer(SimTypeChar(label="Byte"), offset=0)}, name="FWP_BYTE_BLOB", pack=False, align=None), "certHash": SimStruct({"size": SimTypeInt(signed=False, label="UInt32"), "data": SimTypePointer(SimTypeChar(label="Byte"), offset=0)}, name="FWP_BYTE_BLOB", pack=False, align=None), "eku": SimTypePointer(SimStruct({"numEku": SimTypeInt(signed=False, label="UInt32"), "eku": SimTypePointer(SimTypePointer(SimTypeChar(label="Byte"), offset=0), offset=0)}, name="IKEEXT_CERT_EKUS0", pack=False, align=None), offset=0), "name": SimTypePointer(SimStruct({"nameType": SimTypeInt(signed=False, label="IKEEXT_CERT_CRITERIA_NAME_TYPE"), "certName": SimTypePointer(SimTypeChar(label="Char"), offset=0)}, name="IKEEXT_CERT_NAME0", pack=False, align=None), offset=0), "flags": SimTypeInt(signed=False, label="UInt32")}, name="IKEEXT_CERTIFICATE_CRITERIA0", pack=False, align=None), offset=0)}, name="_Anonymous1_e__Struct", pack=False, align=None), "Anonymous2": SimStruct({"outboundEnterpriseStoreArraySize": SimTypeInt(signed=False, label="UInt32"), "outboundEnterpriseStoreCriteria": SimTypePointer(SimStruct({"certData": SimStruct({"size": SimTypeInt(signed=False, label="UInt32"), "data": SimTypePointer(SimTypeChar(label="Byte"), offset=0)}, name="FWP_BYTE_BLOB", pack=False, align=None), "certHash": SimStruct({"size": SimTypeInt(signed=False, label="UInt32"), "data": SimTypePointer(SimTypeChar(label="Byte"), offset=0)}, name="FWP_BYTE_BLOB", pack=False, align=None), "eku": SimTypePointer(SimStruct({"numEku": SimTypeInt(signed=False, label="UInt32"), "eku": SimTypePointer(SimTypePointer(SimTypeChar(label="Byte"), offset=0), offset=0)}, name="IKEEXT_CERT_EKUS0", pack=False, align=None), offset=0), "name": SimTypePointer(SimStruct({"nameType": SimTypeInt(signed=False, label="IKEEXT_CERT_CRITERIA_NAME_TYPE"), "certName": SimTypePointer(SimTypeChar(label="Char"), offset=0)}, name="IKEEXT_CERT_NAME0", pack=False, align=None), offset=0), "flags": SimTypeInt(signed=False, label="UInt32")}, name="IKEEXT_CERTIFICATE_CRITERIA0", pack=False, align=None), offset=0)}, name="_Anonymous2_e__Struct", pack=False, align=None), "Anonymous3": SimStruct({"outboundRootStoreArraySize": SimTypeInt(signed=False, label="UInt32"), "outboundTrustedRootStoreCriteria": SimTypePointer(SimStruct({"certData": SimStruct({"size": SimTypeInt(signed=False, label="UInt32"), "data": SimTypePointer(SimTypeChar(label="Byte"), offset=0)}, name="FWP_BYTE_BLOB", pack=False, align=None), "certHash": SimStruct({"size": SimTypeInt(signed=False, label="UInt32"), "data": SimTypePointer(SimTypeChar(label="Byte"), offset=0)}, name="FWP_BYTE_BLOB", pack=False, align=None), "eku": SimTypePointer(SimStruct({"numEku": SimTypeInt(signed=False, label="UInt32"), "eku": SimTypePointer(SimTypePointer(SimTypeChar(label="Byte"), offset=0), offset=0)}, name="IKEEXT_CERT_EKUS0", pack=False, align=None), offset=0), "name": SimTypePointer(SimStruct({"nameType": SimTypeInt(signed=False, label="IKEEXT_CERT_CRITERIA_NAME_TYPE"), "certName": SimTypePointer(SimTypeChar(label="Char"), offset=0)}, name="IKEEXT_CERT_NAME0", pack=False, align=None), offset=0), "flags": SimTypeInt(signed=False, label="UInt32")}, name="IKEEXT_CERTIFICATE_CRITERIA0", pack=False, align=None), offset=0)}, name="_Anonymous3_e__Struct", pack=False, align=None)}, name="<anon>", label="None"), "flags": SimTypeInt(signed=False, label="IKEEXT_CERT_AUTH"), "localCertLocationUrl": SimStruct({"size": SimTypeInt(signed=False, label="UInt32"), "data": SimTypePointer(SimTypeChar(label="Byte"), offset=0)}, name="FWP_BYTE_BLOB", pack=False, align=None)}, name="IKEEXT_CERTIFICATE_AUTHENTICATION2", pack=False, align=None), "kerberosAuthentication": SimStruct({"flags": SimTypeInt(signed=False, label="IKEEXT_KERBEROS_AUTHENTICATION_FLAGS"), "proxyServer": SimTypePointer(SimTypeChar(label="Char"), offset=0)}, name="IKEEXT_KERBEROS_AUTHENTICATION1", pack=False, align=None), "reservedAuthentication": SimStruct({"flags": SimTypeInt(signed=False, label="IKEEXT_RESERVED_AUTHENTICATION_FLAGS")}, name="IKEEXT_RESERVED_AUTHENTICATION0", pack=False, align=None), "ntlmV2Authentication": SimStruct({"flags": SimTypeInt(signed=False, label="UInt32")}, name="IKEEXT_NTLM_V2_AUTHENTICATION0", pack=False, align=None), "sslAuthentication": SimStruct({"inboundConfigType": SimTypeInt(signed=False, label="IKEEXT_CERT_CONFIG_TYPE"), "Anonymous1": SimUnion({"Anonymous1": SimStruct({"inboundRootArraySize": SimTypeInt(signed=False, label="UInt32"), "inboundRootCriteria": SimTypePointer(SimStruct({"certData": SimStruct({"size": SimTypeInt(signed=False, label="UInt32"), "data": SimTypePointer(SimTypeChar(label="Byte"), offset=0)}, name="FWP_BYTE_BLOB", pack=False, align=None), "certHash": SimStruct({"size": SimTypeInt(signed=False, label="UInt32"), "data": SimTypePointer(SimTypeChar(label="Byte"), offset=0)}, name="FWP_BYTE_BLOB", pack=False, align=None), "eku": SimTypePointer(SimStruct({"numEku": SimTypeInt(signed=False, label="UInt32"), "eku": SimTypePointer(SimTypePointer(SimTypeChar(label="Byte"), offset=0), offset=0)}, name="IKEEXT_CERT_EKUS0", pack=False, align=None), offset=0), "name": SimTypePointer(SimStruct({"nameType": SimTypeInt(signed=False, label="IKEEXT_CERT_CRITERIA_NAME_TYPE"), "certName": SimTypePointer(SimTypeChar(label="Char"), offset=0)}, name="IKEEXT_CERT_NAME0", pack=False, align=None), offset=0), "flags": SimTypeInt(signed=False, label="UInt32")}, name="IKEEXT_CERTIFICATE_CRITERIA0", pack=False, align=None), offset=0)}, name="_Anonymous1_e__Struct", pack=False, align=None), "Anonymous2": SimStruct({"inboundEnterpriseStoreArraySize": SimTypeInt(signed=False, label="UInt32"), "inboundEnterpriseStoreCriteria": SimTypePointer(SimStruct({"certData": SimStruct({"size": SimTypeInt(signed=False, label="UInt32"), "data": SimTypePointer(SimTypeChar(label="Byte"), offset=0)}, name="FWP_BYTE_BLOB", pack=False, align=None), "certHash": SimStruct({"size": SimTypeInt(signed=False, label="UInt32"), "data": SimTypePointer(SimTypeChar(label="Byte"), offset=0)}, name="FWP_BYTE_BLOB", pack=False, align=None), "eku": SimTypePointer(SimStruct({"numEku": SimTypeInt(signed=False, label="UInt32"), "eku": SimTypePointer(SimTypePointer(SimTypeChar(label="Byte"), offset=0), offset=0)}, name="IKEEXT_CERT_EKUS0", pack=False, align=None), offset=0), "name": SimTypePointer(SimStruct({"nameType": SimTypeInt(signed=False, label="IKEEXT_CERT_CRITERIA_NAME_TYPE"), "certName": SimTypePointer(SimTypeChar(label="Char"), offset=0)}, name="IKEEXT_CERT_NAME0", pack=False, align=None), offset=0), "flags": SimTypeInt(signed=False, label="UInt32")}, name="IKEEXT_CERTIFICATE_CRITERIA0", pack=False, align=None), offset=0)}, name="_Anonymous2_e__Struct", pack=False, align=None), "Anonymous3": SimStruct({"inboundRootStoreArraySize": SimTypeInt(signed=False, label="UInt32"), "inboundTrustedRootStoreCriteria": SimTypePointer(SimStruct({"certData": SimStruct({"size": SimTypeInt(signed=False, label="UInt32"), "data": SimTypePointer(SimTypeChar(label="Byte"), offset=0)}, name="FWP_BYTE_BLOB", pack=False, align=None), "certHash": SimStruct({"size": SimTypeInt(signed=False, label="UInt32"), "data": SimTypePointer(SimTypeChar(label="Byte"), offset=0)}, name="FWP_BYTE_BLOB", pack=False, align=None), "eku": SimTypePointer(SimStruct({"numEku": SimTypeInt(signed=False, label="UInt32"), "eku": SimTypePointer(SimTypePointer(SimTypeChar(label="Byte"), offset=0), offset=0)}, name="IKEEXT_CERT_EKUS0", pack=False, align=None), offset=0), "name": SimTypePointer(SimStruct({"nameType": SimTypeInt(signed=False, label="IKEEXT_CERT_CRITERIA_NAME_TYPE"), "certName": SimTypePointer(SimTypeChar(label="Char"), offset=0)}, name="IKEEXT_CERT_NAME0", pack=False, align=None), offset=0), "flags": SimTypeInt(signed=False, label="UInt32")}, name="IKEEXT_CERTIFICATE_CRITERIA0", pack=False, align=None), offset=0)}, name="_Anonymous3_e__Struct", pack=False, align=None)}, name="<anon>", label="None"), "outboundConfigType": SimTypeInt(signed=False, label="IKEEXT_CERT_CONFIG_TYPE"), "Anonymous2": SimUnion({"Anonymous1": SimStruct({"outboundRootArraySize": SimTypeInt(signed=False, label="UInt32"), "outboundRootCriteria": SimTypePointer(SimStruct({"certData": SimStruct({"size": SimTypeInt(signed=False, label="UInt32"), "data": SimTypePointer(SimTypeChar(label="Byte"), offset=0)}, name="FWP_BYTE_BLOB", pack=False, align=None), "certHash": SimStruct({"size": SimTypeInt(signed=False, label="UInt32"), "data": SimTypePointer(SimTypeChar(label="Byte"), offset=0)}, name="FWP_BYTE_BLOB", pack=False, align=None), "eku": SimTypePointer(SimStruct({"numEku": SimTypeInt(signed=False, label="UInt32"), "eku": SimTypePointer(SimTypePointer(SimTypeChar(label="Byte"), offset=0), offset=0)}, name="IKEEXT_CERT_EKUS0", pack=False, align=None), offset=0), "name": SimTypePointer(SimStruct({"nameType": SimTypeInt(signed=False, label="IKEEXT_CERT_CRITERIA_NAME_TYPE"), "certName": SimTypePointer(SimTypeChar(label="Char"), offset=0)}, name="IKEEXT_CERT_NAME0", pack=False, align=None), offset=0), "flags": SimTypeInt(signed=False, label="UInt32")}, name="IKEEXT_CERTIFICATE_CRITERIA0", pack=False, align=None), offset=0)}, name="_Anonymous1_e__Struct", pack=False, align=None), "Anonymous2": SimStruct({"outboundEnterpriseStoreArraySize": SimTypeInt(signed=False, label="UInt32"), "outboundEnterpriseStoreCriteria": SimTypePointer(SimStruct({"certData": SimStruct({"size": SimTypeInt(signed=False, label="UInt32"), "data": SimTypePointer(SimTypeChar(label="Byte"), offset=0)}, name="FWP_BYTE_BLOB", pack=False, align=None), "certHash": SimStruct({"size": SimTypeInt(signed=False, label="UInt32"), "data": SimTypePointer(SimTypeChar(label="Byte"), offset=0)}, name="FWP_BYTE_BLOB", pack=False, align=None), "eku": SimTypePointer(SimStruct({"numEku": SimTypeInt(signed=False, label="UInt32"), "eku": SimTypePointer(SimTypePointer(SimTypeChar(label="Byte"), offset=0), offset=0)}, name="IKEEXT_CERT_EKUS0", pack=False, align=None), offset=0), "name": SimTypePointer(SimStruct({"nameType": SimTypeInt(signed=False, label="IKEEXT_CERT_CRITERIA_NAME_TYPE"), "certName": SimTypePointer(SimTypeChar(label="Char"), offset=0)}, name="IKEEXT_CERT_NAME0", pack=False, align=None), offset=0), "flags": SimTypeInt(signed=False, label="UInt32")}, name="IKEEXT_CERTIFICATE_CRITERIA0", pack=False, align=None), offset=0)}, name="_Anonymous2_e__Struct", pack=False, align=None), "Anonymous3": SimStruct({"outboundRootStoreArraySize": SimTypeInt(signed=False, label="UInt32"), "outboundTrustedRootStoreCriteria": SimTypePointer(SimStruct({"certData": SimStruct({"size": SimTypeInt(signed=False, label="UInt32"), "data": SimTypePointer(SimTypeChar(label="Byte"), offset=0)}, name="FWP_BYTE_BLOB", pack=False, align=None), "certHash": SimStruct({"size": SimTypeInt(signed=False, label="UInt32"), "data": SimTypePointer(SimTypeChar(label="Byte"), offset=0)}, name="FWP_BYTE_BLOB", pack=False, align=None), "eku": SimTypePointer(SimStruct({"numEku": SimTypeInt(signed=False, label="UInt32"), "eku": SimTypePointer(SimTypePointer(SimTypeChar(label="Byte"), offset=0), offset=0)}, name="IKEEXT_CERT_EKUS0", pack=False, align=None), offset=0), "name": SimTypePointer(SimStruct({"nameType": SimTypeInt(signed=False, label="IKEEXT_CERT_CRITERIA_NAME_TYPE"), "certName": SimTypePointer(SimTypeChar(label="Char"), offset=0)}, name="IKEEXT_CERT_NAME0", pack=False, align=None), offset=0), "flags": SimTypeInt(signed=False, label="UInt32")}, name="IKEEXT_CERTIFICATE_CRITERIA0", pack=False, align=None), offset=0)}, name="_Anonymous3_e__Struct", pack=False, align=None)}, name="<anon>", label="None"), "flags": SimTypeInt(signed=False, label="IKEEXT_CERT_AUTH"), "localCertLocationUrl": SimStruct({"size": SimTypeInt(signed=False, label="UInt32"), "data": SimTypePointer(SimTypeChar(label="Byte"), offset=0)}, name="FWP_BYTE_BLOB", pack=False, align=None)}, name="IKEEXT_CERTIFICATE_AUTHENTICATION2", pack=False, align=None), "cgaAuthentication": SimStruct({"keyContainerName": SimTypePointer(SimTypeChar(label="Char"), offset=0), "cspName": SimTypePointer(SimTypeChar(label="Char"), offset=0), "cspType": SimTypeInt(signed=False, label="UInt32"), "cgaModifier": SimStruct({"byteArray16": SimTypeFixedSizeArray(SimTypeChar(label="Byte"), 16)}, name="FWP_BYTE_ARRAY16", pack=False, align=None), "cgaCollisionCount": SimTypeChar(label="Byte")}, name="IKEEXT_IPV6_CGA_AUTHENTICATION0", pack=False, align=None), "eapAuthentication": SimStruct({"flags": SimTypeInt(signed=False, label="IKEEXT_EAP_AUTHENTICATION_FLAGS")}, name="IKEEXT_EAP_AUTHENTICATION0", pack=False, align=None)}, name="<anon>", label="None")}, name="IKEEXT_AUTHENTICATION_METHOD2", pack=False, align=None), offset=0), "initiatorImpersonationType": SimTypeInt(signed=False, label="IKEEXT_AUTHENTICATION_IMPERSONATION_TYPE")}, name="IKEEXT_EM_POLICY2", pack=False, align=None), offset=0), "fwdPathSaLifetime": SimTypeInt(signed=False, label="UInt32")}, name="IPSEC_TUNNEL_POLICY2", pack=False, align=None), offset=0), "ikeV2QmTransportPolicy": SimTypePointer(SimStruct({"numIpsecProposals": SimTypeInt(signed=False, label="UInt32"), "ipsecProposals": SimTypePointer(SimStruct({"lifetime": SimStruct({"lifetimeSeconds": SimTypeInt(signed=False, label="UInt32"), "lifetimeKilobytes": SimTypeInt(signed=False, label="UInt32"), "lifetimePackets": SimTypeInt(signed=False, label="UInt32")}, name="IPSEC_SA_LIFETIME0", pack=False, align=None), "numSaTransforms": SimTypeInt(signed=False, label="UInt32"), "saTransforms": SimTypePointer(SimStruct({"ipsecTransformType": SimTypeInt(signed=False, label="IPSEC_TRANSFORM_TYPE"), "Anonymous": SimUnion({"ahTransform": SimTypePointer(SimStruct({"authTransformId": SimStruct({"authType": SimTypeInt(signed=False, label="IPSEC_AUTH_TYPE"), "authConfig": SimTypeChar(label="Byte")}, name="IPSEC_AUTH_TRANSFORM_ID0", pack=False, align=None), "cryptoModuleId": SimTypePointer(SimTypeBottom(label="Guid"), offset=0)}, name="IPSEC_AUTH_TRANSFORM0", pack=False, align=None), offset=0), "espAuthTransform": SimTypePointer(SimStruct({"authTransformId": SimStruct({"authType": SimTypeInt(signed=False, label="IPSEC_AUTH_TYPE"), "authConfig": SimTypeChar(label="Byte")}, name="IPSEC_AUTH_TRANSFORM_ID0", pack=False, align=None), "cryptoModuleId": SimTypePointer(SimTypeBottom(label="Guid"), offset=0)}, name="IPSEC_AUTH_TRANSFORM0", pack=False, align=None), offset=0), "espCipherTransform": SimTypePointer(SimStruct({"cipherTransformId": SimStruct({"cipherType": SimTypeInt(signed=False, label="IPSEC_CIPHER_TYPE"), "cipherConfig": SimTypeChar(label="Byte")}, name="IPSEC_CIPHER_TRANSFORM_ID0", pack=False, align=None), "cryptoModuleId": SimTypePointer(SimTypeBottom(label="Guid"), offset=0)}, name="IPSEC_CIPHER_TRANSFORM0", pack=False, align=None), offset=0), "espAuthAndCipherTransform": SimTypePointer(SimStruct({"authTransform": SimStruct({"authTransformId": SimStruct({"authType": SimTypeInt(signed=False, label="IPSEC_AUTH_TYPE"), "authConfig": SimTypeChar(label="Byte")}, name="IPSEC_AUTH_TRANSFORM_ID0", pack=False, align=None), "cryptoModuleId": SimTypePointer(SimTypeBottom(label="Guid"), offset=0)}, name="IPSEC_AUTH_TRANSFORM0", pack=False, align=None), "cipherTransform": SimStruct({"cipherTransformId": SimStruct({"cipherType": SimTypeInt(signed=False, label="IPSEC_CIPHER_TYPE"), "cipherConfig": SimTypeChar(label="Byte")}, name="IPSEC_CIPHER_TRANSFORM_ID0", pack=False, align=None), "cryptoModuleId": SimTypePointer(SimTypeBottom(label="Guid"), offset=0)}, name="IPSEC_CIPHER_TRANSFORM0", pack=False, align=None)}, name="IPSEC_AUTH_AND_CIPHER_TRANSFORM0", pack=False, align=None), offset=0), "espAuthFwTransform": SimTypePointer(SimStruct({"authTransformId": SimStruct({"authType": SimTypeInt(signed=False, label="IPSEC_AUTH_TYPE"), "authConfig": SimTypeChar(label="Byte")}, name="IPSEC_AUTH_TRANSFORM_ID0", pack=False, align=None), "cryptoModuleId": SimTypePointer(SimTypeBottom(label="Guid"), offset=0)}, name="IPSEC_AUTH_TRANSFORM0", pack=False, align=None), offset=0)}, name="<anon>", label="None")}, name="IPSEC_SA_TRANSFORM0", pack=False, align=None), offset=0), "pfsGroup": SimTypeInt(signed=False, label="IPSEC_PFS_GROUP")}, name="IPSEC_PROPOSAL0", pack=False, align=None), offset=0), "flags": SimTypeInt(signed=False, label="IPSEC_POLICY_FLAG"), "ndAllowClearTimeoutSeconds": SimTypeInt(signed=False, label="UInt32"), "saIdleTimeout": SimStruct({"idleTimeoutSeconds": SimTypeInt(signed=False, label="UInt32"), "idleTimeoutSecondsFailOver": SimTypeInt(signed=False, label="UInt32")}, name="IPSEC_SA_IDLE_TIMEOUT0", pack=False, align=None), "emPolicy": SimTypePointer(SimStruct({"numAuthenticationMethods": SimTypeInt(signed=False, label="UInt32"), "authenticationMethods": SimTypePointer(SimStruct({"authenticationMethodType": SimTypeInt(signed=False, label="IKEEXT_AUTHENTICATION_METHOD_TYPE"), "Anonymous": SimUnion({"presharedKeyAuthentication": SimStruct({"presharedKey": SimStruct({"size": SimTypeInt(signed=False, label="UInt32"), "data": SimTypePointer(SimTypeChar(label="Byte"), offset=0)}, name="FWP_BYTE_BLOB", pack=False, align=None), "flags": SimTypeInt(signed=False, label="IKEEXT_PRESHARED_KEY_AUTHENTICATION_FLAGS")}, name="IKEEXT_PRESHARED_KEY_AUTHENTICATION1", pack=False, align=None), "certificateAuthentication": SimStruct({"inboundConfigType": SimTypeInt(signed=False, label="IKEEXT_CERT_CONFIG_TYPE"), "Anonymous1": SimUnion({"Anonymous1": SimStruct({"inboundRootArraySize": SimTypeInt(signed=False, label="UInt32"), "inboundRootCriteria": SimTypePointer(SimStruct({"certData": SimStruct({"size": SimTypeInt(signed=False, label="UInt32"), "data": SimTypePointer(SimTypeChar(label="Byte"), offset=0)}, name="FWP_BYTE_BLOB", pack=False, align=None), "certHash": SimStruct({"size": SimTypeInt(signed=False, label="UInt32"), "data": SimTypePointer(SimTypeChar(label="Byte"), offset=0)}, name="FWP_BYTE_BLOB", pack=False, align=None), "eku": SimTypePointer(SimStruct({"numEku": SimTypeInt(signed=False, label="UInt32"), "eku": SimTypePointer(SimTypePointer(SimTypeChar(label="Byte"), offset=0), offset=0)}, name="IKEEXT_CERT_EKUS0", pack=False, align=None), offset=0), "name": SimTypePointer(SimStruct({"nameType": SimTypeInt(signed=False, label="IKEEXT_CERT_CRITERIA_NAME_TYPE"), "certName": SimTypePointer(SimTypeChar(label="Char"), offset=0)}, name="IKEEXT_CERT_NAME0", pack=False, align=None), offset=0), "flags": SimTypeInt(signed=False, label="UInt32")}, name="IKEEXT_CERTIFICATE_CRITERIA0", pack=False, align=None), offset=0)}, name="_Anonymous1_e__Struct", pack=False, align=None), "Anonymous2": SimStruct({"inboundEnterpriseStoreArraySize": SimTypeInt(signed=False, label="UInt32"), "inboundEnterpriseStoreCriteria": SimTypePointer(SimStruct({"certData": SimStruct({"size": SimTypeInt(signed=False, label="UInt32"), "data": SimTypePointer(SimTypeChar(label="Byte"), offset=0)}, name="FWP_BYTE_BLOB", pack=False, align=None), "certHash": SimStruct({"size": SimTypeInt(signed=False, label="UInt32"), "data": SimTypePointer(SimTypeChar(label="Byte"), offset=0)}, name="FWP_BYTE_BLOB", pack=False, align=None), "eku": SimTypePointer(SimStruct({"numEku": SimTypeInt(signed=False, label="UInt32"), "eku": SimTypePointer(SimTypePointer(SimTypeChar(label="Byte"), offset=0), offset=0)}, name="IKEEXT_CERT_EKUS0", pack=False, align=None), offset=0), "name": SimTypePointer(SimStruct({"nameType": SimTypeInt(signed=False, label="IKEEXT_CERT_CRITERIA_NAME_TYPE"), "certName": SimTypePointer(SimTypeChar(label="Char"), offset=0)}, name="IKEEXT_CERT_NAME0", pack=False, align=None), offset=0), "flags": SimTypeInt(signed=False, label="UInt32")}, name="IKEEXT_CERTIFICATE_CRITERIA0", pack=False, align=None), offset=0)}, name="_Anonymous2_e__Struct", pack=False, align=None), "Anonymous3": SimStruct({"inboundRootStoreArraySize": SimTypeInt(signed=False, label="UInt32"), "inboundTrustedRootStoreCriteria": SimTypePointer(SimStruct({"certData": SimStruct({"size": SimTypeInt(signed=False, label="UInt32"), "data": SimTypePointer(SimTypeChar(label="Byte"), offset=0)}, name="FWP_BYTE_BLOB", pack=False, align=None), "certHash": SimStruct({"size": SimTypeInt(signed=False, label="UInt32"), "data": SimTypePointer(SimTypeChar(label="Byte"), offset=0)}, name="FWP_BYTE_BLOB", pack=False, align=None), "eku": SimTypePointer(SimStruct({"numEku": SimTypeInt(signed=False, label="UInt32"), "eku": SimTypePointer(SimTypePointer(SimTypeChar(label="Byte"), offset=0), offset=0)}, name="IKEEXT_CERT_EKUS0", pack=False, align=None), offset=0), "name": SimTypePointer(SimStruct({"nameType": SimTypeInt(signed=False, label="IKEEXT_CERT_CRITERIA_NAME_TYPE"), "certName": SimTypePointer(SimTypeChar(label="Char"), offset=0)}, name="IKEEXT_CERT_NAME0", pack=False, align=None), offset=0), "flags": SimTypeInt(signed=False, label="UInt32")}, name="IKEEXT_CERTIFICATE_CRITERIA0", pack=False, align=None), offset=0)}, name="_Anonymous3_e__Struct", pack=False, align=None)}, name="<anon>", label="None"), "outboundConfigType": SimTypeInt(signed=False, label="IKEEXT_CERT_CONFIG_TYPE"), "Anonymous2": SimUnion({"Anonymous1": SimStruct({"outboundRootArraySize": SimTypeInt(signed=False, label="UInt32"), "outboundRootCriteria": SimTypePointer(SimStruct({"certData": SimStruct({"size": SimTypeInt(signed=False, label="UInt32"), "data": SimTypePointer(SimTypeChar(label="Byte"), offset=0)}, name="FWP_BYTE_BLOB", pack=False, align=None), "certHash": SimStruct({"size": SimTypeInt(signed=False, label="UInt32"), "data": SimTypePointer(SimTypeChar(label="Byte"), offset=0)}, name="FWP_BYTE_BLOB", pack=False, align=None), "eku": SimTypePointer(SimStruct({"numEku": SimTypeInt(signed=False, label="UInt32"), "eku": SimTypePointer(SimTypePointer(SimTypeChar(label="Byte"), offset=0), offset=0)}, name="IKEEXT_CERT_EKUS0", pack=False, align=None), offset=0), "name": SimTypePointer(SimStruct({"nameType": SimTypeInt(signed=False, label="IKEEXT_CERT_CRITERIA_NAME_TYPE"), "certName": SimTypePointer(SimTypeChar(label="Char"), offset=0)}, name="IKEEXT_CERT_NAME0", pack=False, align=None), offset=0), "flags": SimTypeInt(signed=False, label="UInt32")}, name="IKEEXT_CERTIFICATE_CRITERIA0", pack=False, align=None), offset=0)}, name="_Anonymous1_e__Struct", pack=False, align=None), "Anonymous2": SimStruct({"outboundEnterpriseStoreArraySize": SimTypeInt(signed=False, label="UInt32"), "outboundEnterpriseStoreCriteria": SimTypePointer(SimStruct({"certData": SimStruct({"size": SimTypeInt(signed=False, label="UInt32"), "data": SimTypePointer(SimTypeChar(label="Byte"), offset=0)}, name="FWP_BYTE_BLOB", pack=False, align=None), "certHash": SimStruct({"size": SimTypeInt(signed=False, label="UInt32"), "data": SimTypePointer(SimTypeChar(label="Byte"), offset=0)}, name="FWP_BYTE_BLOB", pack=False, align=None), "eku": SimTypePointer(SimStruct({"numEku": SimTypeInt(signed=False, label="UInt32"), "eku": SimTypePointer(SimTypePointer(SimTypeChar(label="Byte"), offset=0), offset=0)}, name="IKEEXT_CERT_EKUS0", pack=False, align=None), offset=0), "name": SimTypePointer(SimStruct({"nameType": SimTypeInt(signed=False, label="IKEEXT_CERT_CRITERIA_NAME_TYPE"), "certName": SimTypePointer(SimTypeChar(label="Char"), offset=0)}, name="IKEEXT_CERT_NAME0", pack=False, align=None), offset=0), "flags": SimTypeInt(signed=False, label="UInt32")}, name="IKEEXT_CERTIFICATE_CRITERIA0", pack=False, align=None), offset=0)}, name="_Anonymous2_e__Struct", pack=False, align=None), "Anonymous3": SimStruct({"outboundRootStoreArraySize": SimTypeInt(signed=False, label="UInt32"), "outboundTrustedRootStoreCriteria": SimTypePointer(SimStruct({"certData": SimStruct({"size": SimTypeInt(signed=False, label="UInt32"), "data": SimTypePointer(SimTypeChar(label="Byte"), offset=0)}, name="FWP_BYTE_BLOB", pack=False, align=None), "certHash": SimStruct({"size": SimTypeInt(signed=False, label="UInt32"), "data": SimTypePointer(SimTypeChar(label="Byte"), offset=0)}, name="FWP_BYTE_BLOB", pack=False, align=None), "eku": SimTypePointer(SimStruct({"numEku": SimTypeInt(signed=False, label="UInt32"), "eku": SimTypePointer(SimTypePointer(SimTypeChar(label="Byte"), offset=0), offset=0)}, name="IKEEXT_CERT_EKUS0", pack=False, align=None), offset=0), "name": SimTypePointer(SimStruct({"nameType": SimTypeInt(signed=False, label="IKEEXT_CERT_CRITERIA_NAME_TYPE"), "certName": SimTypePointer(SimTypeChar(label="Char"), offset=0)}, name="IKEEXT_CERT_NAME0", pack=False, align=None), offset=0), "flags": SimTypeInt(signed=False, label="UInt32")}, name="IKEEXT_CERTIFICATE_CRITERIA0", pack=False, align=None), offset=0)}, name="_Anonymous3_e__Struct", pack=False, align=None)}, name="<anon>", label="None"), "flags": SimTypeInt(signed=False, label="IKEEXT_CERT_AUTH"), "localCertLocationUrl": SimStruct({"size": SimTypeInt(signed=False, label="UInt32"), "data": SimTypePointer(SimTypeChar(label="Byte"), offset=0)}, name="FWP_BYTE_BLOB", pack=False, align=None)}, name="IKEEXT_CERTIFICATE_AUTHENTICATION2", pack=False, align=None), "kerberosAuthentication": SimStruct({"flags": SimTypeInt(signed=False, label="IKEEXT_KERBEROS_AUTHENTICATION_FLAGS"), "proxyServer": SimTypePointer(SimTypeChar(label="Char"), offset=0)}, name="IKEEXT_KERBEROS_AUTHENTICATION1", pack=False, align=None), "reservedAuthentication": SimStruct({"flags": SimTypeInt(signed=False, label="IKEEXT_RESERVED_AUTHENTICATION_FLAGS")}, name="IKEEXT_RESERVED_AUTHENTICATION0", pack=False, align=None), "ntlmV2Authentication": SimStruct({"flags": SimTypeInt(signed=False, label="UInt32")}, name="IKEEXT_NTLM_V2_AUTHENTICATION0", pack=False, align=None), "sslAuthentication": SimStruct({"inboundConfigType": SimTypeInt(signed=False, label="IKEEXT_CERT_CONFIG_TYPE"), "Anonymous1": SimUnion({"Anonymous1": SimStruct({"inboundRootArraySize": SimTypeInt(signed=False, label="UInt32"), "inboundRootCriteria": SimTypePointer(SimStruct({"certData": SimStruct({"size": SimTypeInt(signed=False, label="UInt32"), "data": SimTypePointer(SimTypeChar(label="Byte"), offset=0)}, name="FWP_BYTE_BLOB", pack=False, align=None), "certHash": SimStruct({"size": SimTypeInt(signed=False, label="UInt32"), "data": SimTypePointer(SimTypeChar(label="Byte"), offset=0)}, name="FWP_BYTE_BLOB", pack=False, align=None), "eku": SimTypePointer(SimStruct({"numEku": SimTypeInt(signed=False, label="UInt32"), "eku": SimTypePointer(SimTypePointer(SimTypeChar(label="Byte"), offset=0), offset=0)}, name="IKEEXT_CERT_EKUS0", pack=False, align=None), offset=0), "name": SimTypePointer(SimStruct({"nameType": SimTypeInt(signed=False, label="IKEEXT_CERT_CRITERIA_NAME_TYPE"), "certName": SimTypePointer(SimTypeChar(label="Char"), offset=0)}, name="IKEEXT_CERT_NAME0", pack=False, align=None), offset=0), "flags": SimTypeInt(signed=False, label="UInt32")}, name="IKEEXT_CERTIFICATE_CRITERIA0", pack=False, align=None), offset=0)}, name="_Anonymous1_e__Struct", pack=False, align=None), "Anonymous2": SimStruct({"inboundEnterpriseStoreArraySize": SimTypeInt(signed=False, label="UInt32"), "inboundEnterpriseStoreCriteria": SimTypePointer(SimStruct({"certData": SimStruct({"size": SimTypeInt(signed=False, label="UInt32"), "data": SimTypePointer(SimTypeChar(label="Byte"), offset=0)}, name="FWP_BYTE_BLOB", pack=False, align=None), "certHash": SimStruct({"size": SimTypeInt(signed=False, label="UInt32"), "data": SimTypePointer(SimTypeChar(label="Byte"), offset=0)}, name="FWP_BYTE_BLOB", pack=False, align=None), "eku": SimTypePointer(SimStruct({"numEku": SimTypeInt(signed=False, label="UInt32"), "eku": SimTypePointer(SimTypePointer(SimTypeChar(label="Byte"), offset=0), offset=0)}, name="IKEEXT_CERT_EKUS0", pack=False, align=None), offset=0), "name": SimTypePointer(SimStruct({"nameType": SimTypeInt(signed=False, label="IKEEXT_CERT_CRITERIA_NAME_TYPE"), "certName": SimTypePointer(SimTypeChar(label="Char"), offset=0)}, name="IKEEXT_CERT_NAME0", pack=False, align=None), offset=0), "flags": SimTypeInt(signed=False, label="UInt32")}, name="IKEEXT_CERTIFICATE_CRITERIA0", pack=False, align=None), offset=0)}, name="_Anonymous2_e__Struct", pack=False, align=None), "Anonymous3": SimStruct({"inboundRootStoreArraySize": SimTypeInt(signed=False, label="UInt32"), "inboundTrustedRootStoreCriteria": SimTypePointer(SimStruct({"certData": SimStruct({"size": SimTypeInt(signed=False, label="UInt32"), "data": SimTypePointer(SimTypeChar(label="Byte"), offset=0)}, name="FWP_BYTE_BLOB", pack=False, align=None), "certHash": SimStruct({"size": SimTypeInt(signed=False, label="UInt32"), "data": SimTypePointer(SimTypeChar(label="Byte"), offset=0)}, name="FWP_BYTE_BLOB", pack=False, align=None), "eku": SimTypePointer(SimStruct({"numEku": SimTypeInt(signed=False, label="UInt32"), "eku": SimTypePointer(SimTypePointer(SimTypeChar(label="Byte"), offset=0), offset=0)}, name="IKEEXT_CERT_EKUS0", pack=False, align=None), offset=0), "name": SimTypePointer(SimStruct({"nameType": SimTypeInt(signed=False, label="IKEEXT_CERT_CRITERIA_NAME_TYPE"), "certName": SimTypePointer(SimTypeChar(label="Char"), offset=0)}, name="IKEEXT_CERT_NAME0", pack=False, align=None), offset=0), "flags": SimTypeInt(signed=False, label="UInt32")}, name="IKEEXT_CERTIFICATE_CRITERIA0", pack=False, align=None), offset=0)}, name="_Anonymous3_e__Struct", pack=False, align=None)}, name="<anon>", label="None"), "outboundConfigType": SimTypeInt(signed=False, label="IKEEXT_CERT_CONFIG_TYPE"), "Anonymous2": SimUnion({"Anonymous1": SimStruct({"outboundRootArraySize": SimTypeInt(signed=False, label="UInt32"), "outboundRootCriteria": SimTypePointer(SimStruct({"certData": SimStruct({"size": SimTypeInt(signed=False, label="UInt32"), "data": SimTypePointer(SimTypeChar(label="Byte"), offset=0)}, name="FWP_BYTE_BLOB", pack=False, align=None), "certHash": SimStruct({"size": SimTypeInt(signed=False, label="UInt32"), "data": SimTypePointer(SimTypeChar(label="Byte"), offset=0)}, name="FWP_BYTE_BLOB", pack=False, align=None), "eku": SimTypePointer(SimStruct({"numEku": SimTypeInt(signed=False, label="UInt32"), "eku": SimTypePointer(SimTypePointer(SimTypeChar(label="Byte"), offset=0), offset=0)}, name="IKEEXT_CERT_EKUS0", pack=False, align=None), offset=0), "name": SimTypePointer(SimStruct({"nameType": SimTypeInt(signed=False, label="IKEEXT_CERT_CRITERIA_NAME_TYPE"), "certName": SimTypePointer(SimTypeChar(label="Char"), offset=0)}, name="IKEEXT_CERT_NAME0", pack=False, align=None), offset=0), "flags": SimTypeInt(signed=False, label="UInt32")}, name="IKEEXT_CERTIFICATE_CRITERIA0", pack=False, align=None), offset=0)}, name="_Anonymous1_e__Struct", pack=False, align=None), "Anonymous2": SimStruct({"outboundEnterpriseStoreArraySize": SimTypeInt(signed=False, label="UInt32"), "outboundEnterpriseStoreCriteria": SimTypePointer(SimStruct({"certData": SimStruct({"size": SimTypeInt(signed=False, label="UInt32"), "data": SimTypePointer(SimTypeChar(label="Byte"), offset=0)}, name="FWP_BYTE_BLOB", pack=False, align=None), "certHash": SimStruct({"size": SimTypeInt(signed=False, label="UInt32"), "data": SimTypePointer(SimTypeChar(label="Byte"), offset=0)}, name="FWP_BYTE_BLOB", pack=False, align=None), "eku": SimTypePointer(SimStruct({"numEku": SimTypeInt(signed=False, label="UInt32"), "eku": SimTypePointer(SimTypePointer(SimTypeChar(label="Byte"), offset=0), offset=0)}, name="IKEEXT_CERT_EKUS0", pack=False, align=None), offset=0), "name": SimTypePointer(SimStruct({"nameType": SimTypeInt(signed=False, label="IKEEXT_CERT_CRITERIA_NAME_TYPE"), "certName": SimTypePointer(SimTypeChar(label="Char"), offset=0)}, name="IKEEXT_CERT_NAME0", pack=False, align=None), offset=0), "flags": SimTypeInt(signed=False, label="UInt32")}, name="IKEEXT_CERTIFICATE_CRITERIA0", pack=False, align=None), offset=0)}, name="_Anonymous2_e__Struct", pack=False, align=None), "Anonymous3": SimStruct({"outboundRootStoreArraySize": SimTypeInt(signed=False, label="UInt32"), "outboundTrustedRootStoreCriteria": SimTypePointer(SimStruct({"certData": SimStruct({"size": SimTypeInt(signed=False, label="UInt32"), "data": SimTypePointer(SimTypeChar(label="Byte"), offset=0)}, name="FWP_BYTE_BLOB", pack=False, align=None), "certHash": SimStruct({"size": SimTypeInt(signed=False, label="UInt32"), "data": SimTypePointer(SimTypeChar(label="Byte"), offset=0)}, name="FWP_BYTE_BLOB", pack=False, align=None), "eku": SimTypePointer(SimStruct({"numEku": SimTypeInt(signed=False, label="UInt32"), "eku": SimTypePointer(SimTypePointer(SimTypeChar(label="Byte"), offset=0), offset=0)}, name="IKEEXT_CERT_EKUS0", pack=False, align=None), offset=0), "name": SimTypePointer(SimStruct({"nameType": SimTypeInt(signed=False, label="IKEEXT_CERT_CRITERIA_NAME_TYPE"), "certName": SimTypePointer(SimTypeChar(label="Char"), offset=0)}, name="IKEEXT_CERT_NAME0", pack=False, align=None), offset=0), "flags": SimTypeInt(signed=False, label="UInt32")}, name="IKEEXT_CERTIFICATE_CRITERIA0", pack=False, align=None), offset=0)}, name="_Anonymous3_e__Struct", pack=False, align=None)}, name="<anon>", label="None"), "flags": SimTypeInt(signed=False, label="IKEEXT_CERT_AUTH"), "localCertLocationUrl": SimStruct({"size": SimTypeInt(signed=False, label="UInt32"), "data": SimTypePointer(SimTypeChar(label="Byte"), offset=0)}, name="FWP_BYTE_BLOB", pack=False, align=None)}, name="IKEEXT_CERTIFICATE_AUTHENTICATION2", pack=False, align=None), "cgaAuthentication": SimStruct({"keyContainerName": SimTypePointer(SimTypeChar(label="Char"), offset=0), "cspName": SimTypePointer(SimTypeChar(label="Char"), offset=0), "cspType": SimTypeInt(signed=False, label="UInt32"), "cgaModifier": SimStruct({"byteArray16": SimTypeFixedSizeArray(SimTypeChar(label="Byte"), 16)}, name="FWP_BYTE_ARRAY16", pack=False, align=None), "cgaCollisionCount": SimTypeChar(label="Byte")}, name="IKEEXT_IPV6_CGA_AUTHENTICATION0", pack=False, align=None), "eapAuthentication": SimStruct({"flags": SimTypeInt(signed=False, label="IKEEXT_EAP_AUTHENTICATION_FLAGS")}, name="IKEEXT_EAP_AUTHENTICATION0", pack=False, align=None)}, name="<anon>", label="None")}, name="IKEEXT_AUTHENTICATION_METHOD2", pack=False, align=None), offset=0), "initiatorImpersonationType": SimTypeInt(signed=False, label="IKEEXT_AUTHENTICATION_IMPERSONATION_TYPE")}, name="IKEEXT_EM_POLICY2", pack=False, align=None), offset=0)}, name="IPSEC_TRANSPORT_POLICY2", pack=False, align=None), offset=0), "ikeV2MmPolicy": SimTypePointer(SimStruct({"softExpirationTime": SimTypeInt(signed=False, label="UInt32"), "numAuthenticationMethods": SimTypeInt(signed=False, label="UInt32"), "authenticationMethods": SimTypePointer(SimStruct({"authenticationMethodType": SimTypeInt(signed=False, label="IKEEXT_AUTHENTICATION_METHOD_TYPE"), "Anonymous": SimUnion({"presharedKeyAuthentication": SimStruct({"presharedKey": SimStruct({"size": SimTypeInt(signed=False, label="UInt32"), "data": SimTypePointer(SimTypeChar(label="Byte"), offset=0)}, name="FWP_BYTE_BLOB", pack=False, align=None), "flags": SimTypeInt(signed=False, label="IKEEXT_PRESHARED_KEY_AUTHENTICATION_FLAGS")}, name="IKEEXT_PRESHARED_KEY_AUTHENTICATION1", pack=False, align=None), "certificateAuthentication": SimStruct({"inboundConfigType": SimTypeInt(signed=False, label="IKEEXT_CERT_CONFIG_TYPE"), "Anonymous1": SimUnion({"Anonymous1": SimStruct({"inboundRootArraySize": SimTypeInt(signed=False, label="UInt32"), "inboundRootCriteria": SimTypePointer(SimStruct({"certData": SimStruct({"size": SimTypeInt(signed=False, label="UInt32"), "data": SimTypePointer(SimTypeChar(label="Byte"), offset=0)}, name="FWP_BYTE_BLOB", pack=False, align=None), "certHash": SimStruct({"size": SimTypeInt(signed=False, label="UInt32"), "data": SimTypePointer(SimTypeChar(label="Byte"), offset=0)}, name="FWP_BYTE_BLOB", pack=False, align=None), "eku": SimTypePointer(SimStruct({"numEku": SimTypeInt(signed=False, label="UInt32"), "eku": SimTypePointer(SimTypePointer(SimTypeChar(label="Byte"), offset=0), offset=0)}, name="IKEEXT_CERT_EKUS0", pack=False, align=None), offset=0), "name": SimTypePointer(SimStruct({"nameType": SimTypeInt(signed=False, label="IKEEXT_CERT_CRITERIA_NAME_TYPE"), "certName": SimTypePointer(SimTypeChar(label="Char"), offset=0)}, name="IKEEXT_CERT_NAME0", pack=False, align=None), offset=0), "flags": SimTypeInt(signed=False, label="UInt32")}, name="IKEEXT_CERTIFICATE_CRITERIA0", pack=False, align=None), offset=0)}, name="_Anonymous1_e__Struct", pack=False, align=None), "Anonymous2": SimStruct({"inboundEnterpriseStoreArraySize": SimTypeInt(signed=False, label="UInt32"), "inboundEnterpriseStoreCriteria": SimTypePointer(SimStruct({"certData": SimStruct({"size": SimTypeInt(signed=False, label="UInt32"), "data": SimTypePointer(SimTypeChar(label="Byte"), offset=0)}, name="FWP_BYTE_BLOB", pack=False, align=None), "certHash": SimStruct({"size": SimTypeInt(signed=False, label="UInt32"), "data": SimTypePointer(SimTypeChar(label="Byte"), offset=0)}, name="FWP_BYTE_BLOB", pack=False, align=None), "eku": SimTypePointer(SimStruct({"numEku": SimTypeInt(signed=False, label="UInt32"), "eku": SimTypePointer(SimTypePointer(SimTypeChar(label="Byte"), offset=0), offset=0)}, name="IKEEXT_CERT_EKUS0", pack=False, align=None), offset=0), "name": SimTypePointer(SimStruct({"nameType": SimTypeInt(signed=False, label="IKEEXT_CERT_CRITERIA_NAME_TYPE"), "certName": SimTypePointer(SimTypeChar(label="Char"), offset=0)}, name="IKEEXT_CERT_NAME0", pack=False, align=None), offset=0), "flags": SimTypeInt(signed=False, label="UInt32")}, name="IKEEXT_CERTIFICATE_CRITERIA0", pack=False, align=None), offset=0)}, name="_Anonymous2_e__Struct", pack=False, align=None), "Anonymous3": SimStruct({"inboundRootStoreArraySize": SimTypeInt(signed=False, label="UInt32"), "inboundTrustedRootStoreCriteria": SimTypePointer(SimStruct({"certData": SimStruct({"size": SimTypeInt(signed=False, label="UInt32"), "data": SimTypePointer(SimTypeChar(label="Byte"), offset=0)}, name="FWP_BYTE_BLOB", pack=False, align=None), "certHash": SimStruct({"size": SimTypeInt(signed=False, label="UInt32"), "data": SimTypePointer(SimTypeChar(label="Byte"), offset=0)}, name="FWP_BYTE_BLOB", pack=False, align=None), "eku": SimTypePointer(SimStruct({"numEku": SimTypeInt(signed=False, label="UInt32"), "eku": SimTypePointer(SimTypePointer(SimTypeChar(label="Byte"), offset=0), offset=0)}, name="IKEEXT_CERT_EKUS0", pack=False, align=None), offset=0), "name": SimTypePointer(SimStruct({"nameType": SimTypeInt(signed=False, label="IKEEXT_CERT_CRITERIA_NAME_TYPE"), "certName": SimTypePointer(SimTypeChar(label="Char"), offset=0)}, name="IKEEXT_CERT_NAME0", pack=False, align=None), offset=0), "flags": SimTypeInt(signed=False, label="UInt32")}, name="IKEEXT_CERTIFICATE_CRITERIA0", pack=False, align=None), offset=0)}, name="_Anonymous3_e__Struct", pack=False, align=None)}, name="<anon>", label="None"), "outboundConfigType": SimTypeInt(signed=False, label="IKEEXT_CERT_CONFIG_TYPE"), "Anonymous2": SimUnion({"Anonymous1": SimStruct({"outboundRootArraySize": SimTypeInt(signed=False, label="UInt32"), "outboundRootCriteria": SimTypePointer(SimStruct({"certData": SimStruct({"size": SimTypeInt(signed=False, label="UInt32"), "data": SimTypePointer(SimTypeChar(label="Byte"), offset=0)}, name="FWP_BYTE_BLOB", pack=False, align=None), "certHash": SimStruct({"size": SimTypeInt(signed=False, label="UInt32"), "data": SimTypePointer(SimTypeChar(label="Byte"), offset=0)}, name="FWP_BYTE_BLOB", pack=False, align=None), "eku": SimTypePointer(SimStruct({"numEku": SimTypeInt(signed=False, label="UInt32"), "eku": SimTypePointer(SimTypePointer(SimTypeChar(label="Byte"), offset=0), offset=0)}, name="IKEEXT_CERT_EKUS0", pack=False, align=None), offset=0), "name": SimTypePointer(SimStruct({"nameType": SimTypeInt(signed=False, label="IKEEXT_CERT_CRITERIA_NAME_TYPE"), "certName": SimTypePointer(SimTypeChar(label="Char"), offset=0)}, name="IKEEXT_CERT_NAME0", pack=False, align=None), offset=0), "flags": SimTypeInt(signed=False, label="UInt32")}, name="IKEEXT_CERTIFICATE_CRITERIA0", pack=False, align=None), offset=0)}, name="_Anonymous1_e__Struct", pack=False, align=None), "Anonymous2": SimStruct({"outboundEnterpriseStoreArraySize": SimTypeInt(signed=False, label="UInt32"), "outboundEnterpriseStoreCriteria": SimTypePointer(SimStruct({"certData": SimStruct({"size": SimTypeInt(signed=False, label="UInt32"), "data": SimTypePointer(SimTypeChar(label="Byte"), offset=0)}, name="FWP_BYTE_BLOB", pack=False, align=None), "certHash": SimStruct({"size": SimTypeInt(signed=False, label="UInt32"), "data": SimTypePointer(SimTypeChar(label="Byte"), offset=0)}, name="FWP_BYTE_BLOB", pack=False, align=None), "eku": SimTypePointer(SimStruct({"numEku": SimTypeInt(signed=False, label="UInt32"), "eku": SimTypePointer(SimTypePointer(SimTypeChar(label="Byte"), offset=0), offset=0)}, name="IKEEXT_CERT_EKUS0", pack=False, align=None), offset=0), "name": SimTypePointer(SimStruct({"nameType": SimTypeInt(signed=False, label="IKEEXT_CERT_CRITERIA_NAME_TYPE"), "certName": SimTypePointer(SimTypeChar(label="Char"), offset=0)}, name="IKEEXT_CERT_NAME0", pack=False, align=None), offset=0), "flags": SimTypeInt(signed=False, label="UInt32")}, name="IKEEXT_CERTIFICATE_CRITERIA0", pack=False, align=None), offset=0)}, name="_Anonymous2_e__Struct", pack=False, align=None), "Anonymous3": SimStruct({"outboundRootStoreArraySize": SimTypeInt(signed=False, label="UInt32"), "outboundTrustedRootStoreCriteria": SimTypePointer(SimStruct({"certData": SimStruct({"size": SimTypeInt(signed=False, label="UInt32"), "data": SimTypePointer(SimTypeChar(label="Byte"), offset=0)}, name="FWP_BYTE_BLOB", pack=False, align=None), "certHash": SimStruct({"size": SimTypeInt(signed=False, label="UInt32"), "data": SimTypePointer(SimTypeChar(label="Byte"), offset=0)}, name="FWP_BYTE_BLOB", pack=False, align=None), "eku": SimTypePointer(SimStruct({"numEku": SimTypeInt(signed=False, label="UInt32"), "eku": SimTypePointer(SimTypePointer(SimTypeChar(label="Byte"), offset=0), offset=0)}, name="IKEEXT_CERT_EKUS0", pack=False, align=None), offset=0), "name": SimTypePointer(SimStruct({"nameType": SimTypeInt(signed=False, label="IKEEXT_CERT_CRITERIA_NAME_TYPE"), "certName": SimTypePointer(SimTypeChar(label="Char"), offset=0)}, name="IKEEXT_CERT_NAME0", pack=False, align=None), offset=0), "flags": SimTypeInt(signed=False, label="UInt32")}, name="IKEEXT_CERTIFICATE_CRITERIA0", pack=False, align=None), offset=0)}, name="_Anonymous3_e__Struct", pack=False, align=None)}, name="<anon>", label="None"), "flags": SimTypeInt(signed=False, label="IKEEXT_CERT_AUTH"), "localCertLocationUrl": SimStruct({"size": SimTypeInt(signed=False, label="UInt32"), "data": SimTypePointer(SimTypeChar(label="Byte"), offset=0)}, name="FWP_BYTE_BLOB", pack=False, align=None)}, name="IKEEXT_CERTIFICATE_AUTHENTICATION2", pack=False, align=None), "kerberosAuthentication": SimStruct({"flags": SimTypeInt(signed=False, label="IKEEXT_KERBEROS_AUTHENTICATION_FLAGS"), "proxyServer": SimTypePointer(SimTypeChar(label="Char"), offset=0)}, name="IKEEXT_KERBEROS_AUTHENTICATION1", pack=False, align=None), "reservedAuthentication": SimStruct({"flags": SimTypeInt(signed=False, label="IKEEXT_RESERVED_AUTHENTICATION_FLAGS")}, name="IKEEXT_RESERVED_AUTHENTICATION0", pack=False, align=None), "ntlmV2Authentication": SimStruct({"flags": SimTypeInt(signed=False, label="UInt32")}, name="IKEEXT_NTLM_V2_AUTHENTICATION0", pack=False, align=None), "sslAuthentication": SimStruct({"inboundConfigType": SimTypeInt(signed=False, label="IKEEXT_CERT_CONFIG_TYPE"), "Anonymous1": SimUnion({"Anonymous1": SimStruct({"inboundRootArraySize": SimTypeInt(signed=False, label="UInt32"), "inboundRootCriteria": SimTypePointer(SimStruct({"certData": SimStruct({"size": SimTypeInt(signed=False, label="UInt32"), "data": SimTypePointer(SimTypeChar(label="Byte"), offset=0)}, name="FWP_BYTE_BLOB", pack=False, align=None), "certHash": SimStruct({"size": SimTypeInt(signed=False, label="UInt32"), "data": SimTypePointer(SimTypeChar(label="Byte"), offset=0)}, name="FWP_BYTE_BLOB", pack=False, align=None), "eku": SimTypePointer(SimStruct({"numEku": SimTypeInt(signed=False, label="UInt32"), "eku": SimTypePointer(SimTypePointer(SimTypeChar(label="Byte"), offset=0), offset=0)}, name="IKEEXT_CERT_EKUS0", pack=False, align=None), offset=0), "name": SimTypePointer(SimStruct({"nameType": SimTypeInt(signed=False, label="IKEEXT_CERT_CRITERIA_NAME_TYPE"), "certName": SimTypePointer(SimTypeChar(label="Char"), offset=0)}, name="IKEEXT_CERT_NAME0", pack=False, align=None), offset=0), "flags": SimTypeInt(signed=False, label="UInt32")}, name="IKEEXT_CERTIFICATE_CRITERIA0", pack=False, align=None), offset=0)}, name="_Anonymous1_e__Struct", pack=False, align=None), "Anonymous2": SimStruct({"inboundEnterpriseStoreArraySize": SimTypeInt(signed=False, label="UInt32"), "inboundEnterpriseStoreCriteria": SimTypePointer(SimStruct({"certData": SimStruct({"size": SimTypeInt(signed=False, label="UInt32"), "data": SimTypePointer(SimTypeChar(label="Byte"), offset=0)}, name="FWP_BYTE_BLOB", pack=False, align=None), "certHash": SimStruct({"size": SimTypeInt(signed=False, label="UInt32"), "data": SimTypePointer(SimTypeChar(label="Byte"), offset=0)}, name="FWP_BYTE_BLOB", pack=False, align=None), "eku": SimTypePointer(SimStruct({"numEku": SimTypeInt(signed=False, label="UInt32"), "eku": SimTypePointer(SimTypePointer(SimTypeChar(label="Byte"), offset=0), offset=0)}, name="IKEEXT_CERT_EKUS0", pack=False, align=None), offset=0), "name": SimTypePointer(SimStruct({"nameType": SimTypeInt(signed=False, label="IKEEXT_CERT_CRITERIA_NAME_TYPE"), "certName": SimTypePointer(SimTypeChar(label="Char"), offset=0)}, name="IKEEXT_CERT_NAME0", pack=False, align=None), offset=0), "flags": SimTypeInt(signed=False, label="UInt32")}, name="IKEEXT_CERTIFICATE_CRITERIA0", pack=False, align=None), offset=0)}, name="_Anonymous2_e__Struct", pack=False, align=None), "Anonymous3": SimStruct({"inboundRootStoreArraySize": SimTypeInt(signed=False, label="UInt32"), "inboundTrustedRootStoreCriteria": SimTypePointer(SimStruct({"certData": SimStruct({"size": SimTypeInt(signed=False, label="UInt32"), "data": SimTypePointer(SimTypeChar(label="Byte"), offset=0)}, name="FWP_BYTE_BLOB", pack=False, align=None), "certHash": SimStruct({"size": SimTypeInt(signed=False, label="UInt32"), "data": SimTypePointer(SimTypeChar(label="Byte"), offset=0)}, name="FWP_BYTE_BLOB", pack=False, align=None), "eku": SimTypePointer(SimStruct({"numEku": SimTypeInt(signed=False, label="UInt32"), "eku": SimTypePointer(SimTypePointer(SimTypeChar(label="Byte"), offset=0), offset=0)}, name="IKEEXT_CERT_EKUS0", pack=False, align=None), offset=0), "name": SimTypePointer(SimStruct({"nameType": SimTypeInt(signed=False, label="IKEEXT_CERT_CRITERIA_NAME_TYPE"), "certName": SimTypePointer(SimTypeChar(label="Char"), offset=0)}, name="IKEEXT_CERT_NAME0", pack=False, align=None), offset=0), "flags": SimTypeInt(signed=False, label="UInt32")}, name="IKEEXT_CERTIFICATE_CRITERIA0", pack=False, align=None), offset=0)}, name="_Anonymous3_e__Struct", pack=False, align=None)}, name="<anon>", label="None"), "outboundConfigType": SimTypeInt(signed=False, label="IKEEXT_CERT_CONFIG_TYPE"), "Anonymous2": SimUnion({"Anonymous1": SimStruct({"outboundRootArraySize": SimTypeInt(signed=False, label="UInt32"), "outboundRootCriteria": SimTypePointer(SimStruct({"certData": SimStruct({"size": SimTypeInt(signed=False, label="UInt32"), "data": SimTypePointer(SimTypeChar(label="Byte"), offset=0)}, name="FWP_BYTE_BLOB", pack=False, align=None), "certHash": SimStruct({"size": SimTypeInt(signed=False, label="UInt32"), "data": SimTypePointer(SimTypeChar(label="Byte"), offset=0)}, name="FWP_BYTE_BLOB", pack=False, align=None), "eku": SimTypePointer(SimStruct({"numEku": SimTypeInt(signed=False, label="UInt32"), "eku": SimTypePointer(SimTypePointer(SimTypeChar(label="Byte"), offset=0), offset=0)}, name="IKEEXT_CERT_EKUS0", pack=False, align=None), offset=0), "name": SimTypePointer(SimStruct({"nameType": SimTypeInt(signed=False, label="IKEEXT_CERT_CRITERIA_NAME_TYPE"), "certName": SimTypePointer(SimTypeChar(label="Char"), offset=0)}, name="IKEEXT_CERT_NAME0", pack=False, align=None), offset=0), "flags": SimTypeInt(signed=False, label="UInt32")}, name="IKEEXT_CERTIFICATE_CRITERIA0", pack=False, align=None), offset=0)}, name="_Anonymous1_e__Struct", pack=False, align=None), "Anonymous2": SimStruct({"outboundEnterpriseStoreArraySize": SimTypeInt(signed=False, label="UInt32"), "outboundEnterpriseStoreCriteria": SimTypePointer(SimStruct({"certData": SimStruct({"size": SimTypeInt(signed=False, label="UInt32"), "data": SimTypePointer(SimTypeChar(label="Byte"), offset=0)}, name="FWP_BYTE_BLOB", pack=False, align=None), "certHash": SimStruct({"size": SimTypeInt(signed=False, label="UInt32"), "data": SimTypePointer(SimTypeChar(label="Byte"), offset=0)}, name="FWP_BYTE_BLOB", pack=False, align=None), "eku": SimTypePointer(SimStruct({"numEku": SimTypeInt(signed=False, label="UInt32"), "eku": SimTypePointer(SimTypePointer(SimTypeChar(label="Byte"), offset=0), offset=0)}, name="IKEEXT_CERT_EKUS0", pack=False, align=None), offset=0), "name": SimTypePointer(SimStruct({"nameType": SimTypeInt(signed=False, label="IKEEXT_CERT_CRITERIA_NAME_TYPE"), "certName": SimTypePointer(SimTypeChar(label="Char"), offset=0)}, name="IKEEXT_CERT_NAME0", pack=False, align=None), offset=0), "flags": SimTypeInt(signed=False, label="UInt32")}, name="IKEEXT_CERTIFICATE_CRITERIA0", pack=False, align=None), offset=0)}, name="_Anonymous2_e__Struct", pack=False, align=None), "Anonymous3": SimStruct({"outboundRootStoreArraySize": SimTypeInt(signed=False, label="UInt32"), "outboundTrustedRootStoreCriteria": SimTypePointer(SimStruct({"certData": SimStruct({"size": SimTypeInt(signed=False, label="UInt32"), "data": SimTypePointer(SimTypeChar(label="Byte"), offset=0)}, name="FWP_BYTE_BLOB", pack=False, align=None), "certHash": SimStruct({"size": SimTypeInt(signed=False, label="UInt32"), "data": SimTypePointer(SimTypeChar(label="Byte"), offset=0)}, name="FWP_BYTE_BLOB", pack=False, align=None), "eku": SimTypePointer(SimStruct({"numEku": SimTypeInt(signed=False, label="UInt32"), "eku": SimTypePointer(SimTypePointer(SimTypeChar(label="Byte"), offset=0), offset=0)}, name="IKEEXT_CERT_EKUS0", pack=False, align=None), offset=0), "name": SimTypePointer(SimStruct({"nameType": SimTypeInt(signed=False, label="IKEEXT_CERT_CRITERIA_NAME_TYPE"), "certName": SimTypePointer(SimTypeChar(label="Char"), offset=0)}, name="IKEEXT_CERT_NAME0", pack=False, align=None), offset=0), "flags": SimTypeInt(signed=False, label="UInt32")}, name="IKEEXT_CERTIFICATE_CRITERIA0", pack=False, align=None), offset=0)}, name="_Anonymous3_e__Struct", pack=False, align=None)}, name="<anon>", label="None"), "flags": SimTypeInt(signed=False, label="IKEEXT_CERT_AUTH"), "localCertLocationUrl": SimStruct({"size": SimTypeInt(signed=False, label="UInt32"), "data": SimTypePointer(SimTypeChar(label="Byte"), offset=0)}, name="FWP_BYTE_BLOB", pack=False, align=None)}, name="IKEEXT_CERTIFICATE_AUTHENTICATION2", pack=False, align=None), "cgaAuthentication": SimStruct({"keyContainerName": SimTypePointer(SimTypeChar(label="Char"), offset=0), "cspName": SimTypePointer(SimTypeChar(label="Char"), offset=0), "cspType": SimTypeInt(signed=False, label="UInt32"), "cgaModifier": SimStruct({"byteArray16": SimTypeFixedSizeArray(SimTypeChar(label="Byte"), 16)}, name="FWP_BYTE_ARRAY16", pack=False, align=None), "cgaCollisionCount": SimTypeChar(label="Byte")}, name="IKEEXT_IPV6_CGA_AUTHENTICATION0", pack=False, align=None), "eapAuthentication": SimStruct({"flags": SimTypeInt(signed=False, label="IKEEXT_EAP_AUTHENTICATION_FLAGS")}, name="IKEEXT_EAP_AUTHENTICATION0", pack=False, align=None)}, name="<anon>", label="None")}, name="IKEEXT_AUTHENTICATION_METHOD2", pack=False, align=None), offset=0), "initiatorImpersonationType": SimTypeInt(signed=False, label="IKEEXT_AUTHENTICATION_IMPERSONATION_TYPE"), "numIkeProposals": SimTypeInt(signed=False, label="UInt32"), "ikeProposals": SimTypePointer(SimStruct({"cipherAlgorithm": SimStruct({"algoIdentifier": SimTypeInt(signed=False, label="IKEEXT_CIPHER_TYPE"), "keyLen": SimTypeInt(signed=False, label="UInt32"), "rounds": SimTypeInt(signed=False, label="UInt32")}, name="IKEEXT_CIPHER_ALGORITHM0", pack=False, align=None), "integrityAlgorithm": SimStruct({"algoIdentifier": SimTypeInt(signed=False, label="IKEEXT_INTEGRITY_TYPE")}, name="IKEEXT_INTEGRITY_ALGORITHM0", pack=False, align=None), "maxLifetimeSeconds": SimTypeInt(signed=False, label="UInt32"), "dhGroup": SimTypeInt(signed=False, label="IKEEXT_DH_GROUP"), "quickModeLimit": SimTypeInt(signed=False, label="UInt32")}, name="IKEEXT_PROPOSAL0", pack=False, align=None), offset=0), "flags": SimTypeInt(signed=False, label="IKEEXT_POLICY_FLAG"), "maxDynamicFilters": SimTypeInt(signed=False, label="UInt32"), "retransmitDurationSecs": SimTypeInt(signed=False, label="UInt32")}, name="IKEEXT_POLICY2", pack=False, align=None), offset=0), "idpOptions": SimTypePointer(SimStruct({"stateIdleTimeoutSeconds": SimTypeInt(signed=False, label="UInt32"), "perIPRateLimitQueueIdleTimeoutSeconds": SimTypeInt(signed=False, label="UInt32"), "ipV6IPsecUnauthDscp": SimTypeChar(label="Byte"), "ipV6IPsecUnauthRateLimitBytesPerSec": SimTypeInt(signed=False, label="UInt32"), "ipV6IPsecUnauthPerIPRateLimitBytesPerSec": SimTypeInt(signed=False, label="UInt32"), "ipV6IPsecAuthDscp": SimTypeChar(label="Byte"), "ipV6IPsecAuthRateLimitBytesPerSec": SimTypeInt(signed=False, label="UInt32"), "icmpV6Dscp": SimTypeChar(label="Byte"), "icmpV6RateLimitBytesPerSec": SimTypeInt(signed=False, label="UInt32"), "ipV6FilterExemptDscp": SimTypeChar(label="Byte"), "ipV6FilterExemptRateLimitBytesPerSec": SimTypeInt(signed=False, label="UInt32"), "defBlockExemptDscp": SimTypeChar(label="Byte"), "defBlockExemptRateLimitBytesPerSec": SimTypeInt(signed=False, label="UInt32"), "maxStateEntries": SimTypeInt(signed=False, label="UInt32"), "maxPerIPRateLimitQueues": SimTypeInt(signed=False, label="UInt32"), "flags": SimTypeInt(signed=False, label="IPSEC_DOSP_FLAGS"), "numPublicIFLuids": SimTypeInt(signed=False, label="UInt32"), "publicIFLuids": SimTypePointer(SimTypeLongLong(signed=False, label="UInt64"), offset=0), "numInternalIFLuids": SimTypeInt(signed=False, label="UInt32"), "internalIFLuids": SimTypePointer(SimTypeLongLong(signed=False, label="UInt64"), offset=0), "publicV6AddrMask": SimStruct({"addr": SimTypeFixedSizeArray(SimTypeChar(label="Byte"), 16), "prefixLength": SimTypeChar(label="Byte")}, name="FWP_V6_ADDR_AND_MASK", pack=False, align=None), "internalV6AddrMask": SimStruct({"addr": SimTypeFixedSizeArray(SimTypeChar(label="Byte"), 16), "prefixLength": SimTypeChar(label="Byte")}, name="FWP_V6_ADDR_AND_MASK", pack=False, align=None)}, name="IPSEC_DOSP_OPTIONS0", pack=False, align=None), offset=0)}, name="<anon>", label="None"), "providerContextId": SimTypeLongLong(signed=False, label="UInt64")}, name="FWPM_PROVIDER_CONTEXT2", pack=False, align=None), offset=0), SimTypeInt(signed=False, label="UInt32"), SimTypePointer(SimStruct({"fieldKey": SimTypeBottom(label="Guid"), "matchType": SimTypeInt(signed=False, label="FWP_MATCH_TYPE"), "conditionValue": SimStruct({"type": SimTypeInt(signed=False, label="FWP_DATA_TYPE"), "Anonymous": SimUnion({"uint8": SimTypeChar(label="Byte"), "uint16": SimTypeShort(signed=False, label="UInt16"), "uint32": SimTypeInt(signed=False, label="UInt32"), "uint64": SimTypePointer(SimTypeLongLong(signed=False, label="UInt64"), offset=0), "int8": SimTypeChar(label="SByte"), "int16": SimTypeShort(signed=True, label="Int16"), "int32": SimTypeInt(signed=True, label="Int32"), "int64": SimTypePointer(SimTypeLongLong(signed=True, label="Int64"), offset=0), "float32": SimTypeFloat(size=32), "double64": SimTypePointer(SimTypeFloat(size=64), offset=0), "byteArray16": SimTypePointer(SimStruct({"byteArray16": SimTypeFixedSizeArray(SimTypeChar(label="Byte"), 16)}, name="FWP_BYTE_ARRAY16", pack=False, align=None), offset=0), "byteBlob": SimTypePointer(SimStruct({"size": SimTypeInt(signed=False, label="UInt32"), "data": SimTypePointer(SimTypeChar(label="Byte"), offset=0)}, name="FWP_BYTE_BLOB", pack=False, align=None), offset=0), "sid": SimTypePointer(SimTypeBottom(label="SID"), offset=0), "sd": SimTypePointer(SimStruct({"size": SimTypeInt(signed=False, label="UInt32"), "data": SimTypePointer(SimTypeChar(label="Byte"), offset=0)}, name="FWP_BYTE_BLOB", pack=False, align=None), offset=0), "tokenInformation": SimTypePointer(SimStruct({"sidCount": SimTypeInt(signed=False, label="UInt32"), "sids": SimTypePointer(SimTypeBottom(label="SID_AND_ATTRIBUTES"), offset=0), "restrictedSidCount": SimTypeInt(signed=False, label="UInt32"), "restrictedSids": SimTypePointer(SimTypeBottom(label="SID_AND_ATTRIBUTES"), offset=0)}, name="FWP_TOKEN_INFORMATION", pack=False, align=None), offset=0), "tokenAccessInformation": SimTypePointer(SimStruct({"size": SimTypeInt(signed=False, label="UInt32"), "data": SimTypePointer(SimTypeChar(label="Byte"), offset=0)}, name="FWP_BYTE_BLOB", pack=False, align=None), offset=0), "unicodeString": SimTypePointer(SimTypeChar(label="Char"), offset=0), "byteArray6": SimTypePointer(SimStruct({"byteArray6": SimTypeFixedSizeArray(SimTypeChar(label="Byte"), 6)}, name="FWP_BYTE_ARRAY6", pack=False, align=None), offset=0), "bitmapArray64": SimTypePointer(SimStruct({"bitmapArray64": SimTypeFixedSizeArray(SimTypeChar(label="Byte"), 8)}, name="FWP_BITMAP_ARRAY64_", pack=False, align=None), offset=0), "v4AddrMask": SimTypePointer(SimStruct({"addr": SimTypeInt(signed=False, label="UInt32"), "mask": SimTypeInt(signed=False, label="UInt32")}, name="FWP_V4_ADDR_AND_MASK", pack=False, align=None), offset=0), "v6AddrMask": SimTypePointer(SimStruct({"addr": SimTypeFixedSizeArray(SimTypeChar(label="Byte"), 16), "prefixLength": SimTypeChar(label="Byte")}, name="FWP_V6_ADDR_AND_MASK", pack=False, align=None), offset=0), "rangeValue": SimTypePointer(SimStruct({"valueLow": SimStruct({"type": SimTypeInt(signed=False, label="FWP_DATA_TYPE"), "Anonymous": SimUnion({"uint8": SimTypeChar(label="Byte"), "uint16": SimTypeShort(signed=False, label="UInt16"), "uint32": SimTypeInt(signed=False, label="UInt32"), "uint64": SimTypePointer(SimTypeLongLong(signed=False, label="UInt64"), offset=0), "int8": SimTypeChar(label="SByte"), "int16": SimTypeShort(signed=True, label="Int16"), "int32": SimTypeInt(signed=True, label="Int32"), "int64": SimTypePointer(SimTypeLongLong(signed=True, label="Int64"), offset=0), "float32": SimTypeFloat(size=32), "double64": SimTypePointer(SimTypeFloat(size=64), offset=0), "byteArray16": SimTypePointer(SimStruct({"byteArray16": SimTypeFixedSizeArray(SimTypeChar(label="Byte"), 16)}, name="FWP_BYTE_ARRAY16", pack=False, align=None), offset=0), "byteBlob": SimTypePointer(SimStruct({"size": SimTypeInt(signed=False, label="UInt32"), "data": SimTypePointer(SimTypeChar(label="Byte"), offset=0)}, name="FWP_BYTE_BLOB", pack=False, align=None), offset=0), "sid": SimTypePointer(SimTypeBottom(label="SID"), offset=0), "sd": SimTypePointer(SimStruct({"size": SimTypeInt(signed=False, label="UInt32"), "data": SimTypePointer(SimTypeChar(label="Byte"), offset=0)}, name="FWP_BYTE_BLOB", pack=False, align=None), offset=0), "tokenInformation": SimTypePointer(SimStruct({"sidCount": SimTypeInt(signed=False, label="UInt32"), "sids": SimTypePointer(SimTypeBottom(label="SID_AND_ATTRIBUTES"), offset=0), "restrictedSidCount": SimTypeInt(signed=False, label="UInt32"), "restrictedSids": SimTypePointer(SimTypeBottom(label="SID_AND_ATTRIBUTES"), offset=0)}, name="FWP_TOKEN_INFORMATION", pack=False, align=None), offset=0), "tokenAccessInformation": SimTypePointer(SimStruct({"size": SimTypeInt(signed=False, label="UInt32"), "data": SimTypePointer(SimTypeChar(label="Byte"), offset=0)}, name="FWP_BYTE_BLOB", pack=False, align=None), offset=0), "unicodeString": SimTypePointer(SimTypeChar(label="Char"), offset=0), "byteArray6": SimTypePointer(SimStruct({"byteArray6": SimTypeFixedSizeArray(SimTypeChar(label="Byte"), 6)}, name="FWP_BYTE_ARRAY6", pack=False, align=None), offset=0), "bitmapArray64": SimTypePointer(SimStruct({"bitmapArray64": SimTypeFixedSizeArray(SimTypeChar(label="Byte"), 8)}, name="FWP_BITMAP_ARRAY64_", pack=False, align=None), offset=0)}, name="<anon>", label="None")}, name="FWP_VALUE0", pack=False, align=None), "valueHigh": SimStruct({"type": SimTypeInt(signed=False, label="FWP_DATA_TYPE"), "Anonymous": SimUnion({"uint8": SimTypeChar(label="Byte"), "uint16": SimTypeShort(signed=False, label="UInt16"), "uint32": SimTypeInt(signed=False, label="UInt32"), "uint64": SimTypePointer(SimTypeLongLong(signed=False, label="UInt64"), offset=0), "int8": SimTypeChar(label="SByte"), "int16": SimTypeShort(signed=True, label="Int16"), "int32": SimTypeInt(signed=True, label="Int32"), "int64": SimTypePointer(SimTypeLongLong(signed=True, label="Int64"), offset=0), "float32": SimTypeFloat(size=32), "double64": SimTypePointer(SimTypeFloat(size=64), offset=0), "byteArray16": SimTypePointer(SimStruct({"byteArray16": SimTypeFixedSizeArray(SimTypeChar(label="Byte"), 16)}, name="FWP_BYTE_ARRAY16", pack=False, align=None), offset=0), "byteBlob": SimTypePointer(SimStruct({"size": SimTypeInt(signed=False, label="UInt32"), "data": SimTypePointer(SimTypeChar(label="Byte"), offset=0)}, name="FWP_BYTE_BLOB", pack=False, align=None), offset=0), "sid": SimTypePointer(SimTypeBottom(label="SID"), offset=0), "sd": SimTypePointer(SimStruct({"size": SimTypeInt(signed=False, label="UInt32"), "data": SimTypePointer(SimTypeChar(label="Byte"), offset=0)}, name="FWP_BYTE_BLOB", pack=False, align=None), offset=0), "tokenInformation": SimTypePointer(SimStruct({"sidCount": SimTypeInt(signed=False, label="UInt32"), "sids": SimTypePointer(SimTypeBottom(label="SID_AND_ATTRIBUTES"), offset=0), "restrictedSidCount": SimTypeInt(signed=False, label="UInt32"), "restrictedSids": SimTypePointer(SimTypeBottom(label="SID_AND_ATTRIBUTES"), offset=0)}, name="FWP_TOKEN_INFORMATION", pack=False, align=None), offset=0), "tokenAccessInformation": SimTypePointer(SimStruct({"size": SimTypeInt(signed=False, label="UInt32"), "data": SimTypePointer(SimTypeChar(label="Byte"), offset=0)}, name="FWP_BYTE_BLOB", pack=False, align=None), offset=0), "unicodeString": SimTypePointer(SimTypeChar(label="Char"), offset=0), "byteArray6": SimTypePointer(SimStruct({"byteArray6": SimTypeFixedSizeArray(SimTypeChar(label="Byte"), 6)}, name="FWP_BYTE_ARRAY6", pack=False, align=None), offset=0), "bitmapArray64": SimTypePointer(SimStruct({"bitmapArray64": SimTypeFixedSizeArray(SimTypeChar(label="Byte"), 8)}, name="FWP_BITMAP_ARRAY64_", pack=False, align=None), offset=0)}, name="<anon>", label="None")}, name="FWP_VALUE0", pack=False, align=None)}, name="FWP_RANGE0", pack=False, align=None), offset=0)}, name="<anon>", label="None")}, name="FWP_CONDITION_VALUE0", pack=False, align=None)}, name="FWPM_FILTER_CONDITION0", pack=False, align=None), label="LPArray", offset=0), SimTypePointer(SimTypeBottom(label="Guid"), offset=0), SimTypePointer(SimStruct({"Revision": SimTypeChar(label="Byte"), "Sbz1": SimTypeChar(label="Byte"), "Control": SimTypeShort(signed=False, label="UInt16"), "Owner": SimTypePointer(SimTypeInt(signed=True, label="Int"), label="IntPtr", offset=0), "Group": SimTypePointer(SimTypeInt(signed=True, label="Int"), label="IntPtr", offset=0), "Sacl": SimTypePointer(SimStruct({"AclRevision": SimTypeChar(label="Byte"), "Sbz1": SimTypeChar(label="Byte"), "AclSize": SimTypeShort(signed=False, label="UInt16"), "AceCount": SimTypeShort(signed=False, label="UInt16"), "Sbz2": SimTypeShort(signed=False, label="UInt16")}, name="ACL", pack=False, align=None), offset=0), "Dacl": SimTypePointer(SimStruct({"AclRevision": SimTypeChar(label="Byte"), "Sbz1": SimTypeChar(label="Byte"), "AclSize": SimTypeShort(signed=False, label="UInt16"), "AceCount": SimTypeShort(signed=False, label="UInt16"), "Sbz2": SimTypeShort(signed=False, label="UInt16")}, name="ACL", pack=False, align=None), offset=0)}, name="SECURITY_DESCRIPTOR", pack=False, align=None), offset=0)], SimTypeInt(signed=False, label="UInt32"), arg_names=["engineHandle", "flags", "mainModePolicy", "tunnelPolicy", "numFilterConditions", "filterConditions", "keyModKey", "sd"]),
        # 
        'FwpmIPsecTunnelAdd3': SimTypeFunction([SimTypePointer(SimTypeInt(signed=True, label="Int"), label="IntPtr", offset=0), SimTypeInt(signed=False, label="UInt32"), SimTypePointer(SimStruct({"providerContextKey": SimTypeBottom(label="Guid"), "displayData": SimStruct({"name": SimTypePointer(SimTypeChar(label="Char"), offset=0), "description": SimTypePointer(SimTypeChar(label="Char"), offset=0)}, name="FWPM_DISPLAY_DATA0", pack=False, align=None), "flags": SimTypeInt(signed=False, label="UInt32"), "providerKey": SimTypePointer(SimTypeBottom(label="Guid"), offset=0), "providerData": SimStruct({"size": SimTypeInt(signed=False, label="UInt32"), "data": SimTypePointer(SimTypeChar(label="Byte"), offset=0)}, name="FWP_BYTE_BLOB", pack=False, align=None), "type": SimTypeInt(signed=False, label="FWPM_PROVIDER_CONTEXT_TYPE"), "Anonymous": SimUnion({"keyingPolicy": SimTypePointer(SimStruct({"numKeyMods": SimTypeInt(signed=False, label="UInt32"), "keyModKeys": SimTypePointer(SimTypeBottom(label="Guid"), offset=0), "flags": SimTypeInt(signed=False, label="UInt32")}, name="IPSEC_KEYING_POLICY1", pack=False, align=None), offset=0), "ikeQmTransportPolicy": SimTypePointer(SimStruct({"numIpsecProposals": SimTypeInt(signed=False, label="UInt32"), "ipsecProposals": SimTypePointer(SimStruct({"lifetime": SimStruct({"lifetimeSeconds": SimTypeInt(signed=False, label="UInt32"), "lifetimeKilobytes": SimTypeInt(signed=False, label="UInt32"), "lifetimePackets": SimTypeInt(signed=False, label="UInt32")}, name="IPSEC_SA_LIFETIME0", pack=False, align=None), "numSaTransforms": SimTypeInt(signed=False, label="UInt32"), "saTransforms": SimTypePointer(SimStruct({"ipsecTransformType": SimTypeInt(signed=False, label="IPSEC_TRANSFORM_TYPE"), "Anonymous": SimUnion({"ahTransform": SimTypePointer(SimStruct({"authTransformId": SimStruct({"authType": SimTypeInt(signed=False, label="IPSEC_AUTH_TYPE"), "authConfig": SimTypeChar(label="Byte")}, name="IPSEC_AUTH_TRANSFORM_ID0", pack=False, align=None), "cryptoModuleId": SimTypePointer(SimTypeBottom(label="Guid"), offset=0)}, name="IPSEC_AUTH_TRANSFORM0", pack=False, align=None), offset=0), "espAuthTransform": SimTypePointer(SimStruct({"authTransformId": SimStruct({"authType": SimTypeInt(signed=False, label="IPSEC_AUTH_TYPE"), "authConfig": SimTypeChar(label="Byte")}, name="IPSEC_AUTH_TRANSFORM_ID0", pack=False, align=None), "cryptoModuleId": SimTypePointer(SimTypeBottom(label="Guid"), offset=0)}, name="IPSEC_AUTH_TRANSFORM0", pack=False, align=None), offset=0), "espCipherTransform": SimTypePointer(SimStruct({"cipherTransformId": SimStruct({"cipherType": SimTypeInt(signed=False, label="IPSEC_CIPHER_TYPE"), "cipherConfig": SimTypeChar(label="Byte")}, name="IPSEC_CIPHER_TRANSFORM_ID0", pack=False, align=None), "cryptoModuleId": SimTypePointer(SimTypeBottom(label="Guid"), offset=0)}, name="IPSEC_CIPHER_TRANSFORM0", pack=False, align=None), offset=0), "espAuthAndCipherTransform": SimTypePointer(SimStruct({"authTransform": SimStruct({"authTransformId": SimStruct({"authType": SimTypeInt(signed=False, label="IPSEC_AUTH_TYPE"), "authConfig": SimTypeChar(label="Byte")}, name="IPSEC_AUTH_TRANSFORM_ID0", pack=False, align=None), "cryptoModuleId": SimTypePointer(SimTypeBottom(label="Guid"), offset=0)}, name="IPSEC_AUTH_TRANSFORM0", pack=False, align=None), "cipherTransform": SimStruct({"cipherTransformId": SimStruct({"cipherType": SimTypeInt(signed=False, label="IPSEC_CIPHER_TYPE"), "cipherConfig": SimTypeChar(label="Byte")}, name="IPSEC_CIPHER_TRANSFORM_ID0", pack=False, align=None), "cryptoModuleId": SimTypePointer(SimTypeBottom(label="Guid"), offset=0)}, name="IPSEC_CIPHER_TRANSFORM0", pack=False, align=None)}, name="IPSEC_AUTH_AND_CIPHER_TRANSFORM0", pack=False, align=None), offset=0), "espAuthFwTransform": SimTypePointer(SimStruct({"authTransformId": SimStruct({"authType": SimTypeInt(signed=False, label="IPSEC_AUTH_TYPE"), "authConfig": SimTypeChar(label="Byte")}, name="IPSEC_AUTH_TRANSFORM_ID0", pack=False, align=None), "cryptoModuleId": SimTypePointer(SimTypeBottom(label="Guid"), offset=0)}, name="IPSEC_AUTH_TRANSFORM0", pack=False, align=None), offset=0)}, name="<anon>", label="None")}, name="IPSEC_SA_TRANSFORM0", pack=False, align=None), offset=0), "pfsGroup": SimTypeInt(signed=False, label="IPSEC_PFS_GROUP")}, name="IPSEC_PROPOSAL0", pack=False, align=None), offset=0), "flags": SimTypeInt(signed=False, label="IPSEC_POLICY_FLAG"), "ndAllowClearTimeoutSeconds": SimTypeInt(signed=False, label="UInt32"), "saIdleTimeout": SimStruct({"idleTimeoutSeconds": SimTypeInt(signed=False, label="UInt32"), "idleTimeoutSecondsFailOver": SimTypeInt(signed=False, label="UInt32")}, name="IPSEC_SA_IDLE_TIMEOUT0", pack=False, align=None), "emPolicy": SimTypePointer(SimStruct({"numAuthenticationMethods": SimTypeInt(signed=False, label="UInt32"), "authenticationMethods": SimTypePointer(SimStruct({"authenticationMethodType": SimTypeInt(signed=False, label="IKEEXT_AUTHENTICATION_METHOD_TYPE"), "Anonymous": SimUnion({"presharedKeyAuthentication": SimStruct({"presharedKey": SimStruct({"size": SimTypeInt(signed=False, label="UInt32"), "data": SimTypePointer(SimTypeChar(label="Byte"), offset=0)}, name="FWP_BYTE_BLOB", pack=False, align=None), "flags": SimTypeInt(signed=False, label="IKEEXT_PRESHARED_KEY_AUTHENTICATION_FLAGS")}, name="IKEEXT_PRESHARED_KEY_AUTHENTICATION1", pack=False, align=None), "certificateAuthentication": SimStruct({"inboundConfigType": SimTypeInt(signed=False, label="IKEEXT_CERT_CONFIG_TYPE"), "Anonymous1": SimUnion({"Anonymous1": SimStruct({"inboundRootArraySize": SimTypeInt(signed=False, label="UInt32"), "inboundRootCriteria": SimTypePointer(SimStruct({"certData": SimStruct({"size": SimTypeInt(signed=False, label="UInt32"), "data": SimTypePointer(SimTypeChar(label="Byte"), offset=0)}, name="FWP_BYTE_BLOB", pack=False, align=None), "certHash": SimStruct({"size": SimTypeInt(signed=False, label="UInt32"), "data": SimTypePointer(SimTypeChar(label="Byte"), offset=0)}, name="FWP_BYTE_BLOB", pack=False, align=None), "eku": SimTypePointer(SimStruct({"numEku": SimTypeInt(signed=False, label="UInt32"), "eku": SimTypePointer(SimTypePointer(SimTypeChar(label="Byte"), offset=0), offset=0)}, name="IKEEXT_CERT_EKUS0", pack=False, align=None), offset=0), "name": SimTypePointer(SimStruct({"nameType": SimTypeInt(signed=False, label="IKEEXT_CERT_CRITERIA_NAME_TYPE"), "certName": SimTypePointer(SimTypeChar(label="Char"), offset=0)}, name="IKEEXT_CERT_NAME0", pack=False, align=None), offset=0), "flags": SimTypeInt(signed=False, label="UInt32")}, name="IKEEXT_CERTIFICATE_CRITERIA0", pack=False, align=None), offset=0)}, name="_Anonymous1_e__Struct", pack=False, align=None), "Anonymous2": SimStruct({"inboundEnterpriseStoreArraySize": SimTypeInt(signed=False, label="UInt32"), "inboundEnterpriseStoreCriteria": SimTypePointer(SimStruct({"certData": SimStruct({"size": SimTypeInt(signed=False, label="UInt32"), "data": SimTypePointer(SimTypeChar(label="Byte"), offset=0)}, name="FWP_BYTE_BLOB", pack=False, align=None), "certHash": SimStruct({"size": SimTypeInt(signed=False, label="UInt32"), "data": SimTypePointer(SimTypeChar(label="Byte"), offset=0)}, name="FWP_BYTE_BLOB", pack=False, align=None), "eku": SimTypePointer(SimStruct({"numEku": SimTypeInt(signed=False, label="UInt32"), "eku": SimTypePointer(SimTypePointer(SimTypeChar(label="Byte"), offset=0), offset=0)}, name="IKEEXT_CERT_EKUS0", pack=False, align=None), offset=0), "name": SimTypePointer(SimStruct({"nameType": SimTypeInt(signed=False, label="IKEEXT_CERT_CRITERIA_NAME_TYPE"), "certName": SimTypePointer(SimTypeChar(label="Char"), offset=0)}, name="IKEEXT_CERT_NAME0", pack=False, align=None), offset=0), "flags": SimTypeInt(signed=False, label="UInt32")}, name="IKEEXT_CERTIFICATE_CRITERIA0", pack=False, align=None), offset=0)}, name="_Anonymous2_e__Struct", pack=False, align=None), "Anonymous3": SimStruct({"inboundRootStoreArraySize": SimTypeInt(signed=False, label="UInt32"), "inboundTrustedRootStoreCriteria": SimTypePointer(SimStruct({"certData": SimStruct({"size": SimTypeInt(signed=False, label="UInt32"), "data": SimTypePointer(SimTypeChar(label="Byte"), offset=0)}, name="FWP_BYTE_BLOB", pack=False, align=None), "certHash": SimStruct({"size": SimTypeInt(signed=False, label="UInt32"), "data": SimTypePointer(SimTypeChar(label="Byte"), offset=0)}, name="FWP_BYTE_BLOB", pack=False, align=None), "eku": SimTypePointer(SimStruct({"numEku": SimTypeInt(signed=False, label="UInt32"), "eku": SimTypePointer(SimTypePointer(SimTypeChar(label="Byte"), offset=0), offset=0)}, name="IKEEXT_CERT_EKUS0", pack=False, align=None), offset=0), "name": SimTypePointer(SimStruct({"nameType": SimTypeInt(signed=False, label="IKEEXT_CERT_CRITERIA_NAME_TYPE"), "certName": SimTypePointer(SimTypeChar(label="Char"), offset=0)}, name="IKEEXT_CERT_NAME0", pack=False, align=None), offset=0), "flags": SimTypeInt(signed=False, label="UInt32")}, name="IKEEXT_CERTIFICATE_CRITERIA0", pack=False, align=None), offset=0)}, name="_Anonymous3_e__Struct", pack=False, align=None)}, name="<anon>", label="None"), "outboundConfigType": SimTypeInt(signed=False, label="IKEEXT_CERT_CONFIG_TYPE"), "Anonymous2": SimUnion({"Anonymous1": SimStruct({"outboundRootArraySize": SimTypeInt(signed=False, label="UInt32"), "outboundRootCriteria": SimTypePointer(SimStruct({"certData": SimStruct({"size": SimTypeInt(signed=False, label="UInt32"), "data": SimTypePointer(SimTypeChar(label="Byte"), offset=0)}, name="FWP_BYTE_BLOB", pack=False, align=None), "certHash": SimStruct({"size": SimTypeInt(signed=False, label="UInt32"), "data": SimTypePointer(SimTypeChar(label="Byte"), offset=0)}, name="FWP_BYTE_BLOB", pack=False, align=None), "eku": SimTypePointer(SimStruct({"numEku": SimTypeInt(signed=False, label="UInt32"), "eku": SimTypePointer(SimTypePointer(SimTypeChar(label="Byte"), offset=0), offset=0)}, name="IKEEXT_CERT_EKUS0", pack=False, align=None), offset=0), "name": SimTypePointer(SimStruct({"nameType": SimTypeInt(signed=False, label="IKEEXT_CERT_CRITERIA_NAME_TYPE"), "certName": SimTypePointer(SimTypeChar(label="Char"), offset=0)}, name="IKEEXT_CERT_NAME0", pack=False, align=None), offset=0), "flags": SimTypeInt(signed=False, label="UInt32")}, name="IKEEXT_CERTIFICATE_CRITERIA0", pack=False, align=None), offset=0)}, name="_Anonymous1_e__Struct", pack=False, align=None), "Anonymous2": SimStruct({"outboundEnterpriseStoreArraySize": SimTypeInt(signed=False, label="UInt32"), "outboundEnterpriseStoreCriteria": SimTypePointer(SimStruct({"certData": SimStruct({"size": SimTypeInt(signed=False, label="UInt32"), "data": SimTypePointer(SimTypeChar(label="Byte"), offset=0)}, name="FWP_BYTE_BLOB", pack=False, align=None), "certHash": SimStruct({"size": SimTypeInt(signed=False, label="UInt32"), "data": SimTypePointer(SimTypeChar(label="Byte"), offset=0)}, name="FWP_BYTE_BLOB", pack=False, align=None), "eku": SimTypePointer(SimStruct({"numEku": SimTypeInt(signed=False, label="UInt32"), "eku": SimTypePointer(SimTypePointer(SimTypeChar(label="Byte"), offset=0), offset=0)}, name="IKEEXT_CERT_EKUS0", pack=False, align=None), offset=0), "name": SimTypePointer(SimStruct({"nameType": SimTypeInt(signed=False, label="IKEEXT_CERT_CRITERIA_NAME_TYPE"), "certName": SimTypePointer(SimTypeChar(label="Char"), offset=0)}, name="IKEEXT_CERT_NAME0", pack=False, align=None), offset=0), "flags": SimTypeInt(signed=False, label="UInt32")}, name="IKEEXT_CERTIFICATE_CRITERIA0", pack=False, align=None), offset=0)}, name="_Anonymous2_e__Struct", pack=False, align=None), "Anonymous3": SimStruct({"outboundRootStoreArraySize": SimTypeInt(signed=False, label="UInt32"), "outboundTrustedRootStoreCriteria": SimTypePointer(SimStruct({"certData": SimStruct({"size": SimTypeInt(signed=False, label="UInt32"), "data": SimTypePointer(SimTypeChar(label="Byte"), offset=0)}, name="FWP_BYTE_BLOB", pack=False, align=None), "certHash": SimStruct({"size": SimTypeInt(signed=False, label="UInt32"), "data": SimTypePointer(SimTypeChar(label="Byte"), offset=0)}, name="FWP_BYTE_BLOB", pack=False, align=None), "eku": SimTypePointer(SimStruct({"numEku": SimTypeInt(signed=False, label="UInt32"), "eku": SimTypePointer(SimTypePointer(SimTypeChar(label="Byte"), offset=0), offset=0)}, name="IKEEXT_CERT_EKUS0", pack=False, align=None), offset=0), "name": SimTypePointer(SimStruct({"nameType": SimTypeInt(signed=False, label="IKEEXT_CERT_CRITERIA_NAME_TYPE"), "certName": SimTypePointer(SimTypeChar(label="Char"), offset=0)}, name="IKEEXT_CERT_NAME0", pack=False, align=None), offset=0), "flags": SimTypeInt(signed=False, label="UInt32")}, name="IKEEXT_CERTIFICATE_CRITERIA0", pack=False, align=None), offset=0)}, name="_Anonymous3_e__Struct", pack=False, align=None)}, name="<anon>", label="None"), "flags": SimTypeInt(signed=False, label="IKEEXT_CERT_AUTH"), "localCertLocationUrl": SimStruct({"size": SimTypeInt(signed=False, label="UInt32"), "data": SimTypePointer(SimTypeChar(label="Byte"), offset=0)}, name="FWP_BYTE_BLOB", pack=False, align=None)}, name="IKEEXT_CERTIFICATE_AUTHENTICATION2", pack=False, align=None), "kerberosAuthentication": SimStruct({"flags": SimTypeInt(signed=False, label="IKEEXT_KERBEROS_AUTHENTICATION_FLAGS"), "proxyServer": SimTypePointer(SimTypeChar(label="Char"), offset=0)}, name="IKEEXT_KERBEROS_AUTHENTICATION1", pack=False, align=None), "reservedAuthentication": SimStruct({"flags": SimTypeInt(signed=False, label="IKEEXT_RESERVED_AUTHENTICATION_FLAGS")}, name="IKEEXT_RESERVED_AUTHENTICATION0", pack=False, align=None), "ntlmV2Authentication": SimStruct({"flags": SimTypeInt(signed=False, label="UInt32")}, name="IKEEXT_NTLM_V2_AUTHENTICATION0", pack=False, align=None), "sslAuthentication": SimStruct({"inboundConfigType": SimTypeInt(signed=False, label="IKEEXT_CERT_CONFIG_TYPE"), "Anonymous1": SimUnion({"Anonymous1": SimStruct({"inboundRootArraySize": SimTypeInt(signed=False, label="UInt32"), "inboundRootCriteria": SimTypePointer(SimStruct({"certData": SimStruct({"size": SimTypeInt(signed=False, label="UInt32"), "data": SimTypePointer(SimTypeChar(label="Byte"), offset=0)}, name="FWP_BYTE_BLOB", pack=False, align=None), "certHash": SimStruct({"size": SimTypeInt(signed=False, label="UInt32"), "data": SimTypePointer(SimTypeChar(label="Byte"), offset=0)}, name="FWP_BYTE_BLOB", pack=False, align=None), "eku": SimTypePointer(SimStruct({"numEku": SimTypeInt(signed=False, label="UInt32"), "eku": SimTypePointer(SimTypePointer(SimTypeChar(label="Byte"), offset=0), offset=0)}, name="IKEEXT_CERT_EKUS0", pack=False, align=None), offset=0), "name": SimTypePointer(SimStruct({"nameType": SimTypeInt(signed=False, label="IKEEXT_CERT_CRITERIA_NAME_TYPE"), "certName": SimTypePointer(SimTypeChar(label="Char"), offset=0)}, name="IKEEXT_CERT_NAME0", pack=False, align=None), offset=0), "flags": SimTypeInt(signed=False, label="UInt32")}, name="IKEEXT_CERTIFICATE_CRITERIA0", pack=False, align=None), offset=0)}, name="_Anonymous1_e__Struct", pack=False, align=None), "Anonymous2": SimStruct({"inboundEnterpriseStoreArraySize": SimTypeInt(signed=False, label="UInt32"), "inboundEnterpriseStoreCriteria": SimTypePointer(SimStruct({"certData": SimStruct({"size": SimTypeInt(signed=False, label="UInt32"), "data": SimTypePointer(SimTypeChar(label="Byte"), offset=0)}, name="FWP_BYTE_BLOB", pack=False, align=None), "certHash": SimStruct({"size": SimTypeInt(signed=False, label="UInt32"), "data": SimTypePointer(SimTypeChar(label="Byte"), offset=0)}, name="FWP_BYTE_BLOB", pack=False, align=None), "eku": SimTypePointer(SimStruct({"numEku": SimTypeInt(signed=False, label="UInt32"), "eku": SimTypePointer(SimTypePointer(SimTypeChar(label="Byte"), offset=0), offset=0)}, name="IKEEXT_CERT_EKUS0", pack=False, align=None), offset=0), "name": SimTypePointer(SimStruct({"nameType": SimTypeInt(signed=False, label="IKEEXT_CERT_CRITERIA_NAME_TYPE"), "certName": SimTypePointer(SimTypeChar(label="Char"), offset=0)}, name="IKEEXT_CERT_NAME0", pack=False, align=None), offset=0), "flags": SimTypeInt(signed=False, label="UInt32")}, name="IKEEXT_CERTIFICATE_CRITERIA0", pack=False, align=None), offset=0)}, name="_Anonymous2_e__Struct", pack=False, align=None), "Anonymous3": SimStruct({"inboundRootStoreArraySize": SimTypeInt(signed=False, label="UInt32"), "inboundTrustedRootStoreCriteria": SimTypePointer(SimStruct({"certData": SimStruct({"size": SimTypeInt(signed=False, label="UInt32"), "data": SimTypePointer(SimTypeChar(label="Byte"), offset=0)}, name="FWP_BYTE_BLOB", pack=False, align=None), "certHash": SimStruct({"size": SimTypeInt(signed=False, label="UInt32"), "data": SimTypePointer(SimTypeChar(label="Byte"), offset=0)}, name="FWP_BYTE_BLOB", pack=False, align=None), "eku": SimTypePointer(SimStruct({"numEku": SimTypeInt(signed=False, label="UInt32"), "eku": SimTypePointer(SimTypePointer(SimTypeChar(label="Byte"), offset=0), offset=0)}, name="IKEEXT_CERT_EKUS0", pack=False, align=None), offset=0), "name": SimTypePointer(SimStruct({"nameType": SimTypeInt(signed=False, label="IKEEXT_CERT_CRITERIA_NAME_TYPE"), "certName": SimTypePointer(SimTypeChar(label="Char"), offset=0)}, name="IKEEXT_CERT_NAME0", pack=False, align=None), offset=0), "flags": SimTypeInt(signed=False, label="UInt32")}, name="IKEEXT_CERTIFICATE_CRITERIA0", pack=False, align=None), offset=0)}, name="_Anonymous3_e__Struct", pack=False, align=None)}, name="<anon>", label="None"), "outboundConfigType": SimTypeInt(signed=False, label="IKEEXT_CERT_CONFIG_TYPE"), "Anonymous2": SimUnion({"Anonymous1": SimStruct({"outboundRootArraySize": SimTypeInt(signed=False, label="UInt32"), "outboundRootCriteria": SimTypePointer(SimStruct({"certData": SimStruct({"size": SimTypeInt(signed=False, label="UInt32"), "data": SimTypePointer(SimTypeChar(label="Byte"), offset=0)}, name="FWP_BYTE_BLOB", pack=False, align=None), "certHash": SimStruct({"size": SimTypeInt(signed=False, label="UInt32"), "data": SimTypePointer(SimTypeChar(label="Byte"), offset=0)}, name="FWP_BYTE_BLOB", pack=False, align=None), "eku": SimTypePointer(SimStruct({"numEku": SimTypeInt(signed=False, label="UInt32"), "eku": SimTypePointer(SimTypePointer(SimTypeChar(label="Byte"), offset=0), offset=0)}, name="IKEEXT_CERT_EKUS0", pack=False, align=None), offset=0), "name": SimTypePointer(SimStruct({"nameType": SimTypeInt(signed=False, label="IKEEXT_CERT_CRITERIA_NAME_TYPE"), "certName": SimTypePointer(SimTypeChar(label="Char"), offset=0)}, name="IKEEXT_CERT_NAME0", pack=False, align=None), offset=0), "flags": SimTypeInt(signed=False, label="UInt32")}, name="IKEEXT_CERTIFICATE_CRITERIA0", pack=False, align=None), offset=0)}, name="_Anonymous1_e__Struct", pack=False, align=None), "Anonymous2": SimStruct({"outboundEnterpriseStoreArraySize": SimTypeInt(signed=False, label="UInt32"), "outboundEnterpriseStoreCriteria": SimTypePointer(SimStruct({"certData": SimStruct({"size": SimTypeInt(signed=False, label="UInt32"), "data": SimTypePointer(SimTypeChar(label="Byte"), offset=0)}, name="FWP_BYTE_BLOB", pack=False, align=None), "certHash": SimStruct({"size": SimTypeInt(signed=False, label="UInt32"), "data": SimTypePointer(SimTypeChar(label="Byte"), offset=0)}, name="FWP_BYTE_BLOB", pack=False, align=None), "eku": SimTypePointer(SimStruct({"numEku": SimTypeInt(signed=False, label="UInt32"), "eku": SimTypePointer(SimTypePointer(SimTypeChar(label="Byte"), offset=0), offset=0)}, name="IKEEXT_CERT_EKUS0", pack=False, align=None), offset=0), "name": SimTypePointer(SimStruct({"nameType": SimTypeInt(signed=False, label="IKEEXT_CERT_CRITERIA_NAME_TYPE"), "certName": SimTypePointer(SimTypeChar(label="Char"), offset=0)}, name="IKEEXT_CERT_NAME0", pack=False, align=None), offset=0), "flags": SimTypeInt(signed=False, label="UInt32")}, name="IKEEXT_CERTIFICATE_CRITERIA0", pack=False, align=None), offset=0)}, name="_Anonymous2_e__Struct", pack=False, align=None), "Anonymous3": SimStruct({"outboundRootStoreArraySize": SimTypeInt(signed=False, label="UInt32"), "outboundTrustedRootStoreCriteria": SimTypePointer(SimStruct({"certData": SimStruct({"size": SimTypeInt(signed=False, label="UInt32"), "data": SimTypePointer(SimTypeChar(label="Byte"), offset=0)}, name="FWP_BYTE_BLOB", pack=False, align=None), "certHash": SimStruct({"size": SimTypeInt(signed=False, label="UInt32"), "data": SimTypePointer(SimTypeChar(label="Byte"), offset=0)}, name="FWP_BYTE_BLOB", pack=False, align=None), "eku": SimTypePointer(SimStruct({"numEku": SimTypeInt(signed=False, label="UInt32"), "eku": SimTypePointer(SimTypePointer(SimTypeChar(label="Byte"), offset=0), offset=0)}, name="IKEEXT_CERT_EKUS0", pack=False, align=None), offset=0), "name": SimTypePointer(SimStruct({"nameType": SimTypeInt(signed=False, label="IKEEXT_CERT_CRITERIA_NAME_TYPE"), "certName": SimTypePointer(SimTypeChar(label="Char"), offset=0)}, name="IKEEXT_CERT_NAME0", pack=False, align=None), offset=0), "flags": SimTypeInt(signed=False, label="UInt32")}, name="IKEEXT_CERTIFICATE_CRITERIA0", pack=False, align=None), offset=0)}, name="_Anonymous3_e__Struct", pack=False, align=None)}, name="<anon>", label="None"), "flags": SimTypeInt(signed=False, label="IKEEXT_CERT_AUTH"), "localCertLocationUrl": SimStruct({"size": SimTypeInt(signed=False, label="UInt32"), "data": SimTypePointer(SimTypeChar(label="Byte"), offset=0)}, name="FWP_BYTE_BLOB", pack=False, align=None)}, name="IKEEXT_CERTIFICATE_AUTHENTICATION2", pack=False, align=None), "cgaAuthentication": SimStruct({"keyContainerName": SimTypePointer(SimTypeChar(label="Char"), offset=0), "cspName": SimTypePointer(SimTypeChar(label="Char"), offset=0), "cspType": SimTypeInt(signed=False, label="UInt32"), "cgaModifier": SimStruct({"byteArray16": SimTypeFixedSizeArray(SimTypeChar(label="Byte"), 16)}, name="FWP_BYTE_ARRAY16", pack=False, align=None), "cgaCollisionCount": SimTypeChar(label="Byte")}, name="IKEEXT_IPV6_CGA_AUTHENTICATION0", pack=False, align=None), "eapAuthentication": SimStruct({"flags": SimTypeInt(signed=False, label="IKEEXT_EAP_AUTHENTICATION_FLAGS")}, name="IKEEXT_EAP_AUTHENTICATION0", pack=False, align=None)}, name="<anon>", label="None")}, name="IKEEXT_AUTHENTICATION_METHOD2", pack=False, align=None), offset=0), "initiatorImpersonationType": SimTypeInt(signed=False, label="IKEEXT_AUTHENTICATION_IMPERSONATION_TYPE")}, name="IKEEXT_EM_POLICY2", pack=False, align=None), offset=0)}, name="IPSEC_TRANSPORT_POLICY2", pack=False, align=None), offset=0), "ikeQmTunnelPolicy": SimTypePointer(SimStruct({"flags": SimTypeInt(signed=False, label="UInt32"), "numIpsecProposals": SimTypeInt(signed=False, label="UInt32"), "ipsecProposals": SimTypePointer(SimStruct({"lifetime": SimStruct({"lifetimeSeconds": SimTypeInt(signed=False, label="UInt32"), "lifetimeKilobytes": SimTypeInt(signed=False, label="UInt32"), "lifetimePackets": SimTypeInt(signed=False, label="UInt32")}, name="IPSEC_SA_LIFETIME0", pack=False, align=None), "numSaTransforms": SimTypeInt(signed=False, label="UInt32"), "saTransforms": SimTypePointer(SimStruct({"ipsecTransformType": SimTypeInt(signed=False, label="IPSEC_TRANSFORM_TYPE"), "Anonymous": SimUnion({"ahTransform": SimTypePointer(SimStruct({"authTransformId": SimStruct({"authType": SimTypeInt(signed=False, label="IPSEC_AUTH_TYPE"), "authConfig": SimTypeChar(label="Byte")}, name="IPSEC_AUTH_TRANSFORM_ID0", pack=False, align=None), "cryptoModuleId": SimTypePointer(SimTypeBottom(label="Guid"), offset=0)}, name="IPSEC_AUTH_TRANSFORM0", pack=False, align=None), offset=0), "espAuthTransform": SimTypePointer(SimStruct({"authTransformId": SimStruct({"authType": SimTypeInt(signed=False, label="IPSEC_AUTH_TYPE"), "authConfig": SimTypeChar(label="Byte")}, name="IPSEC_AUTH_TRANSFORM_ID0", pack=False, align=None), "cryptoModuleId": SimTypePointer(SimTypeBottom(label="Guid"), offset=0)}, name="IPSEC_AUTH_TRANSFORM0", pack=False, align=None), offset=0), "espCipherTransform": SimTypePointer(SimStruct({"cipherTransformId": SimStruct({"cipherType": SimTypeInt(signed=False, label="IPSEC_CIPHER_TYPE"), "cipherConfig": SimTypeChar(label="Byte")}, name="IPSEC_CIPHER_TRANSFORM_ID0", pack=False, align=None), "cryptoModuleId": SimTypePointer(SimTypeBottom(label="Guid"), offset=0)}, name="IPSEC_CIPHER_TRANSFORM0", pack=False, align=None), offset=0), "espAuthAndCipherTransform": SimTypePointer(SimStruct({"authTransform": SimStruct({"authTransformId": SimStruct({"authType": SimTypeInt(signed=False, label="IPSEC_AUTH_TYPE"), "authConfig": SimTypeChar(label="Byte")}, name="IPSEC_AUTH_TRANSFORM_ID0", pack=False, align=None), "cryptoModuleId": SimTypePointer(SimTypeBottom(label="Guid"), offset=0)}, name="IPSEC_AUTH_TRANSFORM0", pack=False, align=None), "cipherTransform": SimStruct({"cipherTransformId": SimStruct({"cipherType": SimTypeInt(signed=False, label="IPSEC_CIPHER_TYPE"), "cipherConfig": SimTypeChar(label="Byte")}, name="IPSEC_CIPHER_TRANSFORM_ID0", pack=False, align=None), "cryptoModuleId": SimTypePointer(SimTypeBottom(label="Guid"), offset=0)}, name="IPSEC_CIPHER_TRANSFORM0", pack=False, align=None)}, name="IPSEC_AUTH_AND_CIPHER_TRANSFORM0", pack=False, align=None), offset=0), "espAuthFwTransform": SimTypePointer(SimStruct({"authTransformId": SimStruct({"authType": SimTypeInt(signed=False, label="IPSEC_AUTH_TYPE"), "authConfig": SimTypeChar(label="Byte")}, name="IPSEC_AUTH_TRANSFORM_ID0", pack=False, align=None), "cryptoModuleId": SimTypePointer(SimTypeBottom(label="Guid"), offset=0)}, name="IPSEC_AUTH_TRANSFORM0", pack=False, align=None), offset=0)}, name="<anon>", label="None")}, name="IPSEC_SA_TRANSFORM0", pack=False, align=None), offset=0), "pfsGroup": SimTypeInt(signed=False, label="IPSEC_PFS_GROUP")}, name="IPSEC_PROPOSAL0", pack=False, align=None), offset=0), "tunnelEndpoints": SimStruct({"ipVersion": SimTypeInt(signed=False, label="FWP_IP_VERSION"), "Anonymous1": SimUnion({"localV4Address": SimTypeInt(signed=False, label="UInt32"), "localV6Address": SimTypeFixedSizeArray(SimTypeChar(label="Byte"), 16)}, name="<anon>", label="None"), "Anonymous2": SimUnion({"remoteV4Address": SimTypeInt(signed=False, label="UInt32"), "remoteV6Address": SimTypeFixedSizeArray(SimTypeChar(label="Byte"), 16)}, name="<anon>", label="None"), "localIfLuid": SimTypeLongLong(signed=False, label="UInt64"), "remoteFqdn": SimTypePointer(SimTypeChar(label="Char"), offset=0), "numAddresses": SimTypeInt(signed=False, label="UInt32"), "remoteAddresses": SimTypePointer(SimStruct({"ipVersion": SimTypeInt(signed=False, label="FWP_IP_VERSION"), "Anonymous": SimUnion({"v4Address": SimTypeInt(signed=False, label="UInt32"), "v6Address": SimTypeFixedSizeArray(SimTypeChar(label="Byte"), 16)}, name="<anon>", label="None")}, name="IPSEC_TUNNEL_ENDPOINT0", pack=False, align=None), offset=0)}, name="IPSEC_TUNNEL_ENDPOINTS2", pack=False, align=None), "saIdleTimeout": SimStruct({"idleTimeoutSeconds": SimTypeInt(signed=False, label="UInt32"), "idleTimeoutSecondsFailOver": SimTypeInt(signed=False, label="UInt32")}, name="IPSEC_SA_IDLE_TIMEOUT0", pack=False, align=None), "emPolicy": SimTypePointer(SimStruct({"numAuthenticationMethods": SimTypeInt(signed=False, label="UInt32"), "authenticationMethods": SimTypePointer(SimStruct({"authenticationMethodType": SimTypeInt(signed=False, label="IKEEXT_AUTHENTICATION_METHOD_TYPE"), "Anonymous": SimUnion({"presharedKeyAuthentication": SimStruct({"presharedKey": SimStruct({"size": SimTypeInt(signed=False, label="UInt32"), "data": SimTypePointer(SimTypeChar(label="Byte"), offset=0)}, name="FWP_BYTE_BLOB", pack=False, align=None), "flags": SimTypeInt(signed=False, label="IKEEXT_PRESHARED_KEY_AUTHENTICATION_FLAGS")}, name="IKEEXT_PRESHARED_KEY_AUTHENTICATION1", pack=False, align=None), "certificateAuthentication": SimStruct({"inboundConfigType": SimTypeInt(signed=False, label="IKEEXT_CERT_CONFIG_TYPE"), "Anonymous1": SimUnion({"Anonymous1": SimStruct({"inboundRootArraySize": SimTypeInt(signed=False, label="UInt32"), "inboundRootCriteria": SimTypePointer(SimStruct({"certData": SimStruct({"size": SimTypeInt(signed=False, label="UInt32"), "data": SimTypePointer(SimTypeChar(label="Byte"), offset=0)}, name="FWP_BYTE_BLOB", pack=False, align=None), "certHash": SimStruct({"size": SimTypeInt(signed=False, label="UInt32"), "data": SimTypePointer(SimTypeChar(label="Byte"), offset=0)}, name="FWP_BYTE_BLOB", pack=False, align=None), "eku": SimTypePointer(SimStruct({"numEku": SimTypeInt(signed=False, label="UInt32"), "eku": SimTypePointer(SimTypePointer(SimTypeChar(label="Byte"), offset=0), offset=0)}, name="IKEEXT_CERT_EKUS0", pack=False, align=None), offset=0), "name": SimTypePointer(SimStruct({"nameType": SimTypeInt(signed=False, label="IKEEXT_CERT_CRITERIA_NAME_TYPE"), "certName": SimTypePointer(SimTypeChar(label="Char"), offset=0)}, name="IKEEXT_CERT_NAME0", pack=False, align=None), offset=0), "flags": SimTypeInt(signed=False, label="UInt32")}, name="IKEEXT_CERTIFICATE_CRITERIA0", pack=False, align=None), offset=0)}, name="_Anonymous1_e__Struct", pack=False, align=None), "Anonymous2": SimStruct({"inboundEnterpriseStoreArraySize": SimTypeInt(signed=False, label="UInt32"), "inboundEnterpriseStoreCriteria": SimTypePointer(SimStruct({"certData": SimStruct({"size": SimTypeInt(signed=False, label="UInt32"), "data": SimTypePointer(SimTypeChar(label="Byte"), offset=0)}, name="FWP_BYTE_BLOB", pack=False, align=None), "certHash": SimStruct({"size": SimTypeInt(signed=False, label="UInt32"), "data": SimTypePointer(SimTypeChar(label="Byte"), offset=0)}, name="FWP_BYTE_BLOB", pack=False, align=None), "eku": SimTypePointer(SimStruct({"numEku": SimTypeInt(signed=False, label="UInt32"), "eku": SimTypePointer(SimTypePointer(SimTypeChar(label="Byte"), offset=0), offset=0)}, name="IKEEXT_CERT_EKUS0", pack=False, align=None), offset=0), "name": SimTypePointer(SimStruct({"nameType": SimTypeInt(signed=False, label="IKEEXT_CERT_CRITERIA_NAME_TYPE"), "certName": SimTypePointer(SimTypeChar(label="Char"), offset=0)}, name="IKEEXT_CERT_NAME0", pack=False, align=None), offset=0), "flags": SimTypeInt(signed=False, label="UInt32")}, name="IKEEXT_CERTIFICATE_CRITERIA0", pack=False, align=None), offset=0)}, name="_Anonymous2_e__Struct", pack=False, align=None), "Anonymous3": SimStruct({"inboundRootStoreArraySize": SimTypeInt(signed=False, label="UInt32"), "inboundTrustedRootStoreCriteria": SimTypePointer(SimStruct({"certData": SimStruct({"size": SimTypeInt(signed=False, label="UInt32"), "data": SimTypePointer(SimTypeChar(label="Byte"), offset=0)}, name="FWP_BYTE_BLOB", pack=False, align=None), "certHash": SimStruct({"size": SimTypeInt(signed=False, label="UInt32"), "data": SimTypePointer(SimTypeChar(label="Byte"), offset=0)}, name="FWP_BYTE_BLOB", pack=False, align=None), "eku": SimTypePointer(SimStruct({"numEku": SimTypeInt(signed=False, label="UInt32"), "eku": SimTypePointer(SimTypePointer(SimTypeChar(label="Byte"), offset=0), offset=0)}, name="IKEEXT_CERT_EKUS0", pack=False, align=None), offset=0), "name": SimTypePointer(SimStruct({"nameType": SimTypeInt(signed=False, label="IKEEXT_CERT_CRITERIA_NAME_TYPE"), "certName": SimTypePointer(SimTypeChar(label="Char"), offset=0)}, name="IKEEXT_CERT_NAME0", pack=False, align=None), offset=0), "flags": SimTypeInt(signed=False, label="UInt32")}, name="IKEEXT_CERTIFICATE_CRITERIA0", pack=False, align=None), offset=0)}, name="_Anonymous3_e__Struct", pack=False, align=None)}, name="<anon>", label="None"), "outboundConfigType": SimTypeInt(signed=False, label="IKEEXT_CERT_CONFIG_TYPE"), "Anonymous2": SimUnion({"Anonymous1": SimStruct({"outboundRootArraySize": SimTypeInt(signed=False, label="UInt32"), "outboundRootCriteria": SimTypePointer(SimStruct({"certData": SimStruct({"size": SimTypeInt(signed=False, label="UInt32"), "data": SimTypePointer(SimTypeChar(label="Byte"), offset=0)}, name="FWP_BYTE_BLOB", pack=False, align=None), "certHash": SimStruct({"size": SimTypeInt(signed=False, label="UInt32"), "data": SimTypePointer(SimTypeChar(label="Byte"), offset=0)}, name="FWP_BYTE_BLOB", pack=False, align=None), "eku": SimTypePointer(SimStruct({"numEku": SimTypeInt(signed=False, label="UInt32"), "eku": SimTypePointer(SimTypePointer(SimTypeChar(label="Byte"), offset=0), offset=0)}, name="IKEEXT_CERT_EKUS0", pack=False, align=None), offset=0), "name": SimTypePointer(SimStruct({"nameType": SimTypeInt(signed=False, label="IKEEXT_CERT_CRITERIA_NAME_TYPE"), "certName": SimTypePointer(SimTypeChar(label="Char"), offset=0)}, name="IKEEXT_CERT_NAME0", pack=False, align=None), offset=0), "flags": SimTypeInt(signed=False, label="UInt32")}, name="IKEEXT_CERTIFICATE_CRITERIA0", pack=False, align=None), offset=0)}, name="_Anonymous1_e__Struct", pack=False, align=None), "Anonymous2": SimStruct({"outboundEnterpriseStoreArraySize": SimTypeInt(signed=False, label="UInt32"), "outboundEnterpriseStoreCriteria": SimTypePointer(SimStruct({"certData": SimStruct({"size": SimTypeInt(signed=False, label="UInt32"), "data": SimTypePointer(SimTypeChar(label="Byte"), offset=0)}, name="FWP_BYTE_BLOB", pack=False, align=None), "certHash": SimStruct({"size": SimTypeInt(signed=False, label="UInt32"), "data": SimTypePointer(SimTypeChar(label="Byte"), offset=0)}, name="FWP_BYTE_BLOB", pack=False, align=None), "eku": SimTypePointer(SimStruct({"numEku": SimTypeInt(signed=False, label="UInt32"), "eku": SimTypePointer(SimTypePointer(SimTypeChar(label="Byte"), offset=0), offset=0)}, name="IKEEXT_CERT_EKUS0", pack=False, align=None), offset=0), "name": SimTypePointer(SimStruct({"nameType": SimTypeInt(signed=False, label="IKEEXT_CERT_CRITERIA_NAME_TYPE"), "certName": SimTypePointer(SimTypeChar(label="Char"), offset=0)}, name="IKEEXT_CERT_NAME0", pack=False, align=None), offset=0), "flags": SimTypeInt(signed=False, label="UInt32")}, name="IKEEXT_CERTIFICATE_CRITERIA0", pack=False, align=None), offset=0)}, name="_Anonymous2_e__Struct", pack=False, align=None), "Anonymous3": SimStruct({"outboundRootStoreArraySize": SimTypeInt(signed=False, label="UInt32"), "outboundTrustedRootStoreCriteria": SimTypePointer(SimStruct({"certData": SimStruct({"size": SimTypeInt(signed=False, label="UInt32"), "data": SimTypePointer(SimTypeChar(label="Byte"), offset=0)}, name="FWP_BYTE_BLOB", pack=False, align=None), "certHash": SimStruct({"size": SimTypeInt(signed=False, label="UInt32"), "data": SimTypePointer(SimTypeChar(label="Byte"), offset=0)}, name="FWP_BYTE_BLOB", pack=False, align=None), "eku": SimTypePointer(SimStruct({"numEku": SimTypeInt(signed=False, label="UInt32"), "eku": SimTypePointer(SimTypePointer(SimTypeChar(label="Byte"), offset=0), offset=0)}, name="IKEEXT_CERT_EKUS0", pack=False, align=None), offset=0), "name": SimTypePointer(SimStruct({"nameType": SimTypeInt(signed=False, label="IKEEXT_CERT_CRITERIA_NAME_TYPE"), "certName": SimTypePointer(SimTypeChar(label="Char"), offset=0)}, name="IKEEXT_CERT_NAME0", pack=False, align=None), offset=0), "flags": SimTypeInt(signed=False, label="UInt32")}, name="IKEEXT_CERTIFICATE_CRITERIA0", pack=False, align=None), offset=0)}, name="_Anonymous3_e__Struct", pack=False, align=None)}, name="<anon>", label="None"), "flags": SimTypeInt(signed=False, label="IKEEXT_CERT_AUTH"), "localCertLocationUrl": SimStruct({"size": SimTypeInt(signed=False, label="UInt32"), "data": SimTypePointer(SimTypeChar(label="Byte"), offset=0)}, name="FWP_BYTE_BLOB", pack=False, align=None)}, name="IKEEXT_CERTIFICATE_AUTHENTICATION2", pack=False, align=None), "kerberosAuthentication": SimStruct({"flags": SimTypeInt(signed=False, label="IKEEXT_KERBEROS_AUTHENTICATION_FLAGS"), "proxyServer": SimTypePointer(SimTypeChar(label="Char"), offset=0)}, name="IKEEXT_KERBEROS_AUTHENTICATION1", pack=False, align=None), "reservedAuthentication": SimStruct({"flags": SimTypeInt(signed=False, label="IKEEXT_RESERVED_AUTHENTICATION_FLAGS")}, name="IKEEXT_RESERVED_AUTHENTICATION0", pack=False, align=None), "ntlmV2Authentication": SimStruct({"flags": SimTypeInt(signed=False, label="UInt32")}, name="IKEEXT_NTLM_V2_AUTHENTICATION0", pack=False, align=None), "sslAuthentication": SimStruct({"inboundConfigType": SimTypeInt(signed=False, label="IKEEXT_CERT_CONFIG_TYPE"), "Anonymous1": SimUnion({"Anonymous1": SimStruct({"inboundRootArraySize": SimTypeInt(signed=False, label="UInt32"), "inboundRootCriteria": SimTypePointer(SimStruct({"certData": SimStruct({"size": SimTypeInt(signed=False, label="UInt32"), "data": SimTypePointer(SimTypeChar(label="Byte"), offset=0)}, name="FWP_BYTE_BLOB", pack=False, align=None), "certHash": SimStruct({"size": SimTypeInt(signed=False, label="UInt32"), "data": SimTypePointer(SimTypeChar(label="Byte"), offset=0)}, name="FWP_BYTE_BLOB", pack=False, align=None), "eku": SimTypePointer(SimStruct({"numEku": SimTypeInt(signed=False, label="UInt32"), "eku": SimTypePointer(SimTypePointer(SimTypeChar(label="Byte"), offset=0), offset=0)}, name="IKEEXT_CERT_EKUS0", pack=False, align=None), offset=0), "name": SimTypePointer(SimStruct({"nameType": SimTypeInt(signed=False, label="IKEEXT_CERT_CRITERIA_NAME_TYPE"), "certName": SimTypePointer(SimTypeChar(label="Char"), offset=0)}, name="IKEEXT_CERT_NAME0", pack=False, align=None), offset=0), "flags": SimTypeInt(signed=False, label="UInt32")}, name="IKEEXT_CERTIFICATE_CRITERIA0", pack=False, align=None), offset=0)}, name="_Anonymous1_e__Struct", pack=False, align=None), "Anonymous2": SimStruct({"inboundEnterpriseStoreArraySize": SimTypeInt(signed=False, label="UInt32"), "inboundEnterpriseStoreCriteria": SimTypePointer(SimStruct({"certData": SimStruct({"size": SimTypeInt(signed=False, label="UInt32"), "data": SimTypePointer(SimTypeChar(label="Byte"), offset=0)}, name="FWP_BYTE_BLOB", pack=False, align=None), "certHash": SimStruct({"size": SimTypeInt(signed=False, label="UInt32"), "data": SimTypePointer(SimTypeChar(label="Byte"), offset=0)}, name="FWP_BYTE_BLOB", pack=False, align=None), "eku": SimTypePointer(SimStruct({"numEku": SimTypeInt(signed=False, label="UInt32"), "eku": SimTypePointer(SimTypePointer(SimTypeChar(label="Byte"), offset=0), offset=0)}, name="IKEEXT_CERT_EKUS0", pack=False, align=None), offset=0), "name": SimTypePointer(SimStruct({"nameType": SimTypeInt(signed=False, label="IKEEXT_CERT_CRITERIA_NAME_TYPE"), "certName": SimTypePointer(SimTypeChar(label="Char"), offset=0)}, name="IKEEXT_CERT_NAME0", pack=False, align=None), offset=0), "flags": SimTypeInt(signed=False, label="UInt32")}, name="IKEEXT_CERTIFICATE_CRITERIA0", pack=False, align=None), offset=0)}, name="_Anonymous2_e__Struct", pack=False, align=None), "Anonymous3": SimStruct({"inboundRootStoreArraySize": SimTypeInt(signed=False, label="UInt32"), "inboundTrustedRootStoreCriteria": SimTypePointer(SimStruct({"certData": SimStruct({"size": SimTypeInt(signed=False, label="UInt32"), "data": SimTypePointer(SimTypeChar(label="Byte"), offset=0)}, name="FWP_BYTE_BLOB", pack=False, align=None), "certHash": SimStruct({"size": SimTypeInt(signed=False, label="UInt32"), "data": SimTypePointer(SimTypeChar(label="Byte"), offset=0)}, name="FWP_BYTE_BLOB", pack=False, align=None), "eku": SimTypePointer(SimStruct({"numEku": SimTypeInt(signed=False, label="UInt32"), "eku": SimTypePointer(SimTypePointer(SimTypeChar(label="Byte"), offset=0), offset=0)}, name="IKEEXT_CERT_EKUS0", pack=False, align=None), offset=0), "name": SimTypePointer(SimStruct({"nameType": SimTypeInt(signed=False, label="IKEEXT_CERT_CRITERIA_NAME_TYPE"), "certName": SimTypePointer(SimTypeChar(label="Char"), offset=0)}, name="IKEEXT_CERT_NAME0", pack=False, align=None), offset=0), "flags": SimTypeInt(signed=False, label="UInt32")}, name="IKEEXT_CERTIFICATE_CRITERIA0", pack=False, align=None), offset=0)}, name="_Anonymous3_e__Struct", pack=False, align=None)}, name="<anon>", label="None"), "outboundConfigType": SimTypeInt(signed=False, label="IKEEXT_CERT_CONFIG_TYPE"), "Anonymous2": SimUnion({"Anonymous1": SimStruct({"outboundRootArraySize": SimTypeInt(signed=False, label="UInt32"), "outboundRootCriteria": SimTypePointer(SimStruct({"certData": SimStruct({"size": SimTypeInt(signed=False, label="UInt32"), "data": SimTypePointer(SimTypeChar(label="Byte"), offset=0)}, name="FWP_BYTE_BLOB", pack=False, align=None), "certHash": SimStruct({"size": SimTypeInt(signed=False, label="UInt32"), "data": SimTypePointer(SimTypeChar(label="Byte"), offset=0)}, name="FWP_BYTE_BLOB", pack=False, align=None), "eku": SimTypePointer(SimStruct({"numEku": SimTypeInt(signed=False, label="UInt32"), "eku": SimTypePointer(SimTypePointer(SimTypeChar(label="Byte"), offset=0), offset=0)}, name="IKEEXT_CERT_EKUS0", pack=False, align=None), offset=0), "name": SimTypePointer(SimStruct({"nameType": SimTypeInt(signed=False, label="IKEEXT_CERT_CRITERIA_NAME_TYPE"), "certName": SimTypePointer(SimTypeChar(label="Char"), offset=0)}, name="IKEEXT_CERT_NAME0", pack=False, align=None), offset=0), "flags": SimTypeInt(signed=False, label="UInt32")}, name="IKEEXT_CERTIFICATE_CRITERIA0", pack=False, align=None), offset=0)}, name="_Anonymous1_e__Struct", pack=False, align=None), "Anonymous2": SimStruct({"outboundEnterpriseStoreArraySize": SimTypeInt(signed=False, label="UInt32"), "outboundEnterpriseStoreCriteria": SimTypePointer(SimStruct({"certData": SimStruct({"size": SimTypeInt(signed=False, label="UInt32"), "data": SimTypePointer(SimTypeChar(label="Byte"), offset=0)}, name="FWP_BYTE_BLOB", pack=False, align=None), "certHash": SimStruct({"size": SimTypeInt(signed=False, label="UInt32"), "data": SimTypePointer(SimTypeChar(label="Byte"), offset=0)}, name="FWP_BYTE_BLOB", pack=False, align=None), "eku": SimTypePointer(SimStruct({"numEku": SimTypeInt(signed=False, label="UInt32"), "eku": SimTypePointer(SimTypePointer(SimTypeChar(label="Byte"), offset=0), offset=0)}, name="IKEEXT_CERT_EKUS0", pack=False, align=None), offset=0), "name": SimTypePointer(SimStruct({"nameType": SimTypeInt(signed=False, label="IKEEXT_CERT_CRITERIA_NAME_TYPE"), "certName": SimTypePointer(SimTypeChar(label="Char"), offset=0)}, name="IKEEXT_CERT_NAME0", pack=False, align=None), offset=0), "flags": SimTypeInt(signed=False, label="UInt32")}, name="IKEEXT_CERTIFICATE_CRITERIA0", pack=False, align=None), offset=0)}, name="_Anonymous2_e__Struct", pack=False, align=None), "Anonymous3": SimStruct({"outboundRootStoreArraySize": SimTypeInt(signed=False, label="UInt32"), "outboundTrustedRootStoreCriteria": SimTypePointer(SimStruct({"certData": SimStruct({"size": SimTypeInt(signed=False, label="UInt32"), "data": SimTypePointer(SimTypeChar(label="Byte"), offset=0)}, name="FWP_BYTE_BLOB", pack=False, align=None), "certHash": SimStruct({"size": SimTypeInt(signed=False, label="UInt32"), "data": SimTypePointer(SimTypeChar(label="Byte"), offset=0)}, name="FWP_BYTE_BLOB", pack=False, align=None), "eku": SimTypePointer(SimStruct({"numEku": SimTypeInt(signed=False, label="UInt32"), "eku": SimTypePointer(SimTypePointer(SimTypeChar(label="Byte"), offset=0), offset=0)}, name="IKEEXT_CERT_EKUS0", pack=False, align=None), offset=0), "name": SimTypePointer(SimStruct({"nameType": SimTypeInt(signed=False, label="IKEEXT_CERT_CRITERIA_NAME_TYPE"), "certName": SimTypePointer(SimTypeChar(label="Char"), offset=0)}, name="IKEEXT_CERT_NAME0", pack=False, align=None), offset=0), "flags": SimTypeInt(signed=False, label="UInt32")}, name="IKEEXT_CERTIFICATE_CRITERIA0", pack=False, align=None), offset=0)}, name="_Anonymous3_e__Struct", pack=False, align=None)}, name="<anon>", label="None"), "flags": SimTypeInt(signed=False, label="IKEEXT_CERT_AUTH"), "localCertLocationUrl": SimStruct({"size": SimTypeInt(signed=False, label="UInt32"), "data": SimTypePointer(SimTypeChar(label="Byte"), offset=0)}, name="FWP_BYTE_BLOB", pack=False, align=None)}, name="IKEEXT_CERTIFICATE_AUTHENTICATION2", pack=False, align=None), "cgaAuthentication": SimStruct({"keyContainerName": SimTypePointer(SimTypeChar(label="Char"), offset=0), "cspName": SimTypePointer(SimTypeChar(label="Char"), offset=0), "cspType": SimTypeInt(signed=False, label="UInt32"), "cgaModifier": SimStruct({"byteArray16": SimTypeFixedSizeArray(SimTypeChar(label="Byte"), 16)}, name="FWP_BYTE_ARRAY16", pack=False, align=None), "cgaCollisionCount": SimTypeChar(label="Byte")}, name="IKEEXT_IPV6_CGA_AUTHENTICATION0", pack=False, align=None), "eapAuthentication": SimStruct({"flags": SimTypeInt(signed=False, label="IKEEXT_EAP_AUTHENTICATION_FLAGS")}, name="IKEEXT_EAP_AUTHENTICATION0", pack=False, align=None)}, name="<anon>", label="None")}, name="IKEEXT_AUTHENTICATION_METHOD2", pack=False, align=None), offset=0), "initiatorImpersonationType": SimTypeInt(signed=False, label="IKEEXT_AUTHENTICATION_IMPERSONATION_TYPE")}, name="IKEEXT_EM_POLICY2", pack=False, align=None), offset=0), "fwdPathSaLifetime": SimTypeInt(signed=False, label="UInt32"), "compartmentId": SimTypeInt(signed=False, label="UInt32"), "numTrafficSelectorPolicy": SimTypeInt(signed=False, label="UInt32"), "trafficSelectorPolicies": SimTypePointer(SimStruct({"flags": SimTypeInt(signed=False, label="UInt32"), "numLocalTrafficSelectors": SimTypeInt(signed=False, label="UInt32"), "localTrafficSelectors": SimTypePointer(SimStruct({"protocolId": SimTypeChar(label="Byte"), "portStart": SimTypeShort(signed=False, label="UInt16"), "portEnd": SimTypeShort(signed=False, label="UInt16"), "ipVersion": SimTypeInt(signed=False, label="FWP_IP_VERSION"), "Anonymous1": SimUnion({"startV4Address": SimTypeInt(signed=False, label="UInt32"), "startV6Address": SimTypeFixedSizeArray(SimTypeChar(label="Byte"), 16)}, name="<anon>", label="None"), "Anonymous2": SimUnion({"endV4Address": SimTypeInt(signed=False, label="UInt32"), "endV6Address": SimTypeFixedSizeArray(SimTypeChar(label="Byte"), 16)}, name="<anon>", label="None")}, name="IPSEC_TRAFFIC_SELECTOR0_", pack=False, align=None), offset=0), "numRemoteTrafficSelectors": SimTypeInt(signed=False, label="UInt32"), "remoteTrafficSelectors": SimTypePointer(SimStruct({"protocolId": SimTypeChar(label="Byte"), "portStart": SimTypeShort(signed=False, label="UInt16"), "portEnd": SimTypeShort(signed=False, label="UInt16"), "ipVersion": SimTypeInt(signed=False, label="FWP_IP_VERSION"), "Anonymous1": SimUnion({"startV4Address": SimTypeInt(signed=False, label="UInt32"), "startV6Address": SimTypeFixedSizeArray(SimTypeChar(label="Byte"), 16)}, name="<anon>", label="None"), "Anonymous2": SimUnion({"endV4Address": SimTypeInt(signed=False, label="UInt32"), "endV6Address": SimTypeFixedSizeArray(SimTypeChar(label="Byte"), 16)}, name="<anon>", label="None")}, name="IPSEC_TRAFFIC_SELECTOR0_", pack=False, align=None), offset=0)}, name="IPSEC_TRAFFIC_SELECTOR_POLICY0_", pack=False, align=None), offset=0)}, name="IPSEC_TUNNEL_POLICY3_", pack=False, align=None), offset=0), "authipQmTransportPolicy": SimTypePointer(SimStruct({"numIpsecProposals": SimTypeInt(signed=False, label="UInt32"), "ipsecProposals": SimTypePointer(SimStruct({"lifetime": SimStruct({"lifetimeSeconds": SimTypeInt(signed=False, label="UInt32"), "lifetimeKilobytes": SimTypeInt(signed=False, label="UInt32"), "lifetimePackets": SimTypeInt(signed=False, label="UInt32")}, name="IPSEC_SA_LIFETIME0", pack=False, align=None), "numSaTransforms": SimTypeInt(signed=False, label="UInt32"), "saTransforms": SimTypePointer(SimStruct({"ipsecTransformType": SimTypeInt(signed=False, label="IPSEC_TRANSFORM_TYPE"), "Anonymous": SimUnion({"ahTransform": SimTypePointer(SimStruct({"authTransformId": SimStruct({"authType": SimTypeInt(signed=False, label="IPSEC_AUTH_TYPE"), "authConfig": SimTypeChar(label="Byte")}, name="IPSEC_AUTH_TRANSFORM_ID0", pack=False, align=None), "cryptoModuleId": SimTypePointer(SimTypeBottom(label="Guid"), offset=0)}, name="IPSEC_AUTH_TRANSFORM0", pack=False, align=None), offset=0), "espAuthTransform": SimTypePointer(SimStruct({"authTransformId": SimStruct({"authType": SimTypeInt(signed=False, label="IPSEC_AUTH_TYPE"), "authConfig": SimTypeChar(label="Byte")}, name="IPSEC_AUTH_TRANSFORM_ID0", pack=False, align=None), "cryptoModuleId": SimTypePointer(SimTypeBottom(label="Guid"), offset=0)}, name="IPSEC_AUTH_TRANSFORM0", pack=False, align=None), offset=0), "espCipherTransform": SimTypePointer(SimStruct({"cipherTransformId": SimStruct({"cipherType": SimTypeInt(signed=False, label="IPSEC_CIPHER_TYPE"), "cipherConfig": SimTypeChar(label="Byte")}, name="IPSEC_CIPHER_TRANSFORM_ID0", pack=False, align=None), "cryptoModuleId": SimTypePointer(SimTypeBottom(label="Guid"), offset=0)}, name="IPSEC_CIPHER_TRANSFORM0", pack=False, align=None), offset=0), "espAuthAndCipherTransform": SimTypePointer(SimStruct({"authTransform": SimStruct({"authTransformId": SimStruct({"authType": SimTypeInt(signed=False, label="IPSEC_AUTH_TYPE"), "authConfig": SimTypeChar(label="Byte")}, name="IPSEC_AUTH_TRANSFORM_ID0", pack=False, align=None), "cryptoModuleId": SimTypePointer(SimTypeBottom(label="Guid"), offset=0)}, name="IPSEC_AUTH_TRANSFORM0", pack=False, align=None), "cipherTransform": SimStruct({"cipherTransformId": SimStruct({"cipherType": SimTypeInt(signed=False, label="IPSEC_CIPHER_TYPE"), "cipherConfig": SimTypeChar(label="Byte")}, name="IPSEC_CIPHER_TRANSFORM_ID0", pack=False, align=None), "cryptoModuleId": SimTypePointer(SimTypeBottom(label="Guid"), offset=0)}, name="IPSEC_CIPHER_TRANSFORM0", pack=False, align=None)}, name="IPSEC_AUTH_AND_CIPHER_TRANSFORM0", pack=False, align=None), offset=0), "espAuthFwTransform": SimTypePointer(SimStruct({"authTransformId": SimStruct({"authType": SimTypeInt(signed=False, label="IPSEC_AUTH_TYPE"), "authConfig": SimTypeChar(label="Byte")}, name="IPSEC_AUTH_TRANSFORM_ID0", pack=False, align=None), "cryptoModuleId": SimTypePointer(SimTypeBottom(label="Guid"), offset=0)}, name="IPSEC_AUTH_TRANSFORM0", pack=False, align=None), offset=0)}, name="<anon>", label="None")}, name="IPSEC_SA_TRANSFORM0", pack=False, align=None), offset=0), "pfsGroup": SimTypeInt(signed=False, label="IPSEC_PFS_GROUP")}, name="IPSEC_PROPOSAL0", pack=False, align=None), offset=0), "flags": SimTypeInt(signed=False, label="IPSEC_POLICY_FLAG"), "ndAllowClearTimeoutSeconds": SimTypeInt(signed=False, label="UInt32"), "saIdleTimeout": SimStruct({"idleTimeoutSeconds": SimTypeInt(signed=False, label="UInt32"), "idleTimeoutSecondsFailOver": SimTypeInt(signed=False, label="UInt32")}, name="IPSEC_SA_IDLE_TIMEOUT0", pack=False, align=None), "emPolicy": SimTypePointer(SimStruct({"numAuthenticationMethods": SimTypeInt(signed=False, label="UInt32"), "authenticationMethods": SimTypePointer(SimStruct({"authenticationMethodType": SimTypeInt(signed=False, label="IKEEXT_AUTHENTICATION_METHOD_TYPE"), "Anonymous": SimUnion({"presharedKeyAuthentication": SimStruct({"presharedKey": SimStruct({"size": SimTypeInt(signed=False, label="UInt32"), "data": SimTypePointer(SimTypeChar(label="Byte"), offset=0)}, name="FWP_BYTE_BLOB", pack=False, align=None), "flags": SimTypeInt(signed=False, label="IKEEXT_PRESHARED_KEY_AUTHENTICATION_FLAGS")}, name="IKEEXT_PRESHARED_KEY_AUTHENTICATION1", pack=False, align=None), "certificateAuthentication": SimStruct({"inboundConfigType": SimTypeInt(signed=False, label="IKEEXT_CERT_CONFIG_TYPE"), "Anonymous1": SimUnion({"Anonymous1": SimStruct({"inboundRootArraySize": SimTypeInt(signed=False, label="UInt32"), "inboundRootCriteria": SimTypePointer(SimStruct({"certData": SimStruct({"size": SimTypeInt(signed=False, label="UInt32"), "data": SimTypePointer(SimTypeChar(label="Byte"), offset=0)}, name="FWP_BYTE_BLOB", pack=False, align=None), "certHash": SimStruct({"size": SimTypeInt(signed=False, label="UInt32"), "data": SimTypePointer(SimTypeChar(label="Byte"), offset=0)}, name="FWP_BYTE_BLOB", pack=False, align=None), "eku": SimTypePointer(SimStruct({"numEku": SimTypeInt(signed=False, label="UInt32"), "eku": SimTypePointer(SimTypePointer(SimTypeChar(label="Byte"), offset=0), offset=0)}, name="IKEEXT_CERT_EKUS0", pack=False, align=None), offset=0), "name": SimTypePointer(SimStruct({"nameType": SimTypeInt(signed=False, label="IKEEXT_CERT_CRITERIA_NAME_TYPE"), "certName": SimTypePointer(SimTypeChar(label="Char"), offset=0)}, name="IKEEXT_CERT_NAME0", pack=False, align=None), offset=0), "flags": SimTypeInt(signed=False, label="UInt32")}, name="IKEEXT_CERTIFICATE_CRITERIA0", pack=False, align=None), offset=0)}, name="_Anonymous1_e__Struct", pack=False, align=None), "Anonymous2": SimStruct({"inboundEnterpriseStoreArraySize": SimTypeInt(signed=False, label="UInt32"), "inboundEnterpriseStoreCriteria": SimTypePointer(SimStruct({"certData": SimStruct({"size": SimTypeInt(signed=False, label="UInt32"), "data": SimTypePointer(SimTypeChar(label="Byte"), offset=0)}, name="FWP_BYTE_BLOB", pack=False, align=None), "certHash": SimStruct({"size": SimTypeInt(signed=False, label="UInt32"), "data": SimTypePointer(SimTypeChar(label="Byte"), offset=0)}, name="FWP_BYTE_BLOB", pack=False, align=None), "eku": SimTypePointer(SimStruct({"numEku": SimTypeInt(signed=False, label="UInt32"), "eku": SimTypePointer(SimTypePointer(SimTypeChar(label="Byte"), offset=0), offset=0)}, name="IKEEXT_CERT_EKUS0", pack=False, align=None), offset=0), "name": SimTypePointer(SimStruct({"nameType": SimTypeInt(signed=False, label="IKEEXT_CERT_CRITERIA_NAME_TYPE"), "certName": SimTypePointer(SimTypeChar(label="Char"), offset=0)}, name="IKEEXT_CERT_NAME0", pack=False, align=None), offset=0), "flags": SimTypeInt(signed=False, label="UInt32")}, name="IKEEXT_CERTIFICATE_CRITERIA0", pack=False, align=None), offset=0)}, name="_Anonymous2_e__Struct", pack=False, align=None), "Anonymous3": SimStruct({"inboundRootStoreArraySize": SimTypeInt(signed=False, label="UInt32"), "inboundTrustedRootStoreCriteria": SimTypePointer(SimStruct({"certData": SimStruct({"size": SimTypeInt(signed=False, label="UInt32"), "data": SimTypePointer(SimTypeChar(label="Byte"), offset=0)}, name="FWP_BYTE_BLOB", pack=False, align=None), "certHash": SimStruct({"size": SimTypeInt(signed=False, label="UInt32"), "data": SimTypePointer(SimTypeChar(label="Byte"), offset=0)}, name="FWP_BYTE_BLOB", pack=False, align=None), "eku": SimTypePointer(SimStruct({"numEku": SimTypeInt(signed=False, label="UInt32"), "eku": SimTypePointer(SimTypePointer(SimTypeChar(label="Byte"), offset=0), offset=0)}, name="IKEEXT_CERT_EKUS0", pack=False, align=None), offset=0), "name": SimTypePointer(SimStruct({"nameType": SimTypeInt(signed=False, label="IKEEXT_CERT_CRITERIA_NAME_TYPE"), "certName": SimTypePointer(SimTypeChar(label="Char"), offset=0)}, name="IKEEXT_CERT_NAME0", pack=False, align=None), offset=0), "flags": SimTypeInt(signed=False, label="UInt32")}, name="IKEEXT_CERTIFICATE_CRITERIA0", pack=False, align=None), offset=0)}, name="_Anonymous3_e__Struct", pack=False, align=None)}, name="<anon>", label="None"), "outboundConfigType": SimTypeInt(signed=False, label="IKEEXT_CERT_CONFIG_TYPE"), "Anonymous2": SimUnion({"Anonymous1": SimStruct({"outboundRootArraySize": SimTypeInt(signed=False, label="UInt32"), "outboundRootCriteria": SimTypePointer(SimStruct({"certData": SimStruct({"size": SimTypeInt(signed=False, label="UInt32"), "data": SimTypePointer(SimTypeChar(label="Byte"), offset=0)}, name="FWP_BYTE_BLOB", pack=False, align=None), "certHash": SimStruct({"size": SimTypeInt(signed=False, label="UInt32"), "data": SimTypePointer(SimTypeChar(label="Byte"), offset=0)}, name="FWP_BYTE_BLOB", pack=False, align=None), "eku": SimTypePointer(SimStruct({"numEku": SimTypeInt(signed=False, label="UInt32"), "eku": SimTypePointer(SimTypePointer(SimTypeChar(label="Byte"), offset=0), offset=0)}, name="IKEEXT_CERT_EKUS0", pack=False, align=None), offset=0), "name": SimTypePointer(SimStruct({"nameType": SimTypeInt(signed=False, label="IKEEXT_CERT_CRITERIA_NAME_TYPE"), "certName": SimTypePointer(SimTypeChar(label="Char"), offset=0)}, name="IKEEXT_CERT_NAME0", pack=False, align=None), offset=0), "flags": SimTypeInt(signed=False, label="UInt32")}, name="IKEEXT_CERTIFICATE_CRITERIA0", pack=False, align=None), offset=0)}, name="_Anonymous1_e__Struct", pack=False, align=None), "Anonymous2": SimStruct({"outboundEnterpriseStoreArraySize": SimTypeInt(signed=False, label="UInt32"), "outboundEnterpriseStoreCriteria": SimTypePointer(SimStruct({"certData": SimStruct({"size": SimTypeInt(signed=False, label="UInt32"), "data": SimTypePointer(SimTypeChar(label="Byte"), offset=0)}, name="FWP_BYTE_BLOB", pack=False, align=None), "certHash": SimStruct({"size": SimTypeInt(signed=False, label="UInt32"), "data": SimTypePointer(SimTypeChar(label="Byte"), offset=0)}, name="FWP_BYTE_BLOB", pack=False, align=None), "eku": SimTypePointer(SimStruct({"numEku": SimTypeInt(signed=False, label="UInt32"), "eku": SimTypePointer(SimTypePointer(SimTypeChar(label="Byte"), offset=0), offset=0)}, name="IKEEXT_CERT_EKUS0", pack=False, align=None), offset=0), "name": SimTypePointer(SimStruct({"nameType": SimTypeInt(signed=False, label="IKEEXT_CERT_CRITERIA_NAME_TYPE"), "certName": SimTypePointer(SimTypeChar(label="Char"), offset=0)}, name="IKEEXT_CERT_NAME0", pack=False, align=None), offset=0), "flags": SimTypeInt(signed=False, label="UInt32")}, name="IKEEXT_CERTIFICATE_CRITERIA0", pack=False, align=None), offset=0)}, name="_Anonymous2_e__Struct", pack=False, align=None), "Anonymous3": SimStruct({"outboundRootStoreArraySize": SimTypeInt(signed=False, label="UInt32"), "outboundTrustedRootStoreCriteria": SimTypePointer(SimStruct({"certData": SimStruct({"size": SimTypeInt(signed=False, label="UInt32"), "data": SimTypePointer(SimTypeChar(label="Byte"), offset=0)}, name="FWP_BYTE_BLOB", pack=False, align=None), "certHash": SimStruct({"size": SimTypeInt(signed=False, label="UInt32"), "data": SimTypePointer(SimTypeChar(label="Byte"), offset=0)}, name="FWP_BYTE_BLOB", pack=False, align=None), "eku": SimTypePointer(SimStruct({"numEku": SimTypeInt(signed=False, label="UInt32"), "eku": SimTypePointer(SimTypePointer(SimTypeChar(label="Byte"), offset=0), offset=0)}, name="IKEEXT_CERT_EKUS0", pack=False, align=None), offset=0), "name": SimTypePointer(SimStruct({"nameType": SimTypeInt(signed=False, label="IKEEXT_CERT_CRITERIA_NAME_TYPE"), "certName": SimTypePointer(SimTypeChar(label="Char"), offset=0)}, name="IKEEXT_CERT_NAME0", pack=False, align=None), offset=0), "flags": SimTypeInt(signed=False, label="UInt32")}, name="IKEEXT_CERTIFICATE_CRITERIA0", pack=False, align=None), offset=0)}, name="_Anonymous3_e__Struct", pack=False, align=None)}, name="<anon>", label="None"), "flags": SimTypeInt(signed=False, label="IKEEXT_CERT_AUTH"), "localCertLocationUrl": SimStruct({"size": SimTypeInt(signed=False, label="UInt32"), "data": SimTypePointer(SimTypeChar(label="Byte"), offset=0)}, name="FWP_BYTE_BLOB", pack=False, align=None)}, name="IKEEXT_CERTIFICATE_AUTHENTICATION2", pack=False, align=None), "kerberosAuthentication": SimStruct({"flags": SimTypeInt(signed=False, label="IKEEXT_KERBEROS_AUTHENTICATION_FLAGS"), "proxyServer": SimTypePointer(SimTypeChar(label="Char"), offset=0)}, name="IKEEXT_KERBEROS_AUTHENTICATION1", pack=False, align=None), "reservedAuthentication": SimStruct({"flags": SimTypeInt(signed=False, label="IKEEXT_RESERVED_AUTHENTICATION_FLAGS")}, name="IKEEXT_RESERVED_AUTHENTICATION0", pack=False, align=None), "ntlmV2Authentication": SimStruct({"flags": SimTypeInt(signed=False, label="UInt32")}, name="IKEEXT_NTLM_V2_AUTHENTICATION0", pack=False, align=None), "sslAuthentication": SimStruct({"inboundConfigType": SimTypeInt(signed=False, label="IKEEXT_CERT_CONFIG_TYPE"), "Anonymous1": SimUnion({"Anonymous1": SimStruct({"inboundRootArraySize": SimTypeInt(signed=False, label="UInt32"), "inboundRootCriteria": SimTypePointer(SimStruct({"certData": SimStruct({"size": SimTypeInt(signed=False, label="UInt32"), "data": SimTypePointer(SimTypeChar(label="Byte"), offset=0)}, name="FWP_BYTE_BLOB", pack=False, align=None), "certHash": SimStruct({"size": SimTypeInt(signed=False, label="UInt32"), "data": SimTypePointer(SimTypeChar(label="Byte"), offset=0)}, name="FWP_BYTE_BLOB", pack=False, align=None), "eku": SimTypePointer(SimStruct({"numEku": SimTypeInt(signed=False, label="UInt32"), "eku": SimTypePointer(SimTypePointer(SimTypeChar(label="Byte"), offset=0), offset=0)}, name="IKEEXT_CERT_EKUS0", pack=False, align=None), offset=0), "name": SimTypePointer(SimStruct({"nameType": SimTypeInt(signed=False, label="IKEEXT_CERT_CRITERIA_NAME_TYPE"), "certName": SimTypePointer(SimTypeChar(label="Char"), offset=0)}, name="IKEEXT_CERT_NAME0", pack=False, align=None), offset=0), "flags": SimTypeInt(signed=False, label="UInt32")}, name="IKEEXT_CERTIFICATE_CRITERIA0", pack=False, align=None), offset=0)}, name="_Anonymous1_e__Struct", pack=False, align=None), "Anonymous2": SimStruct({"inboundEnterpriseStoreArraySize": SimTypeInt(signed=False, label="UInt32"), "inboundEnterpriseStoreCriteria": SimTypePointer(SimStruct({"certData": SimStruct({"size": SimTypeInt(signed=False, label="UInt32"), "data": SimTypePointer(SimTypeChar(label="Byte"), offset=0)}, name="FWP_BYTE_BLOB", pack=False, align=None), "certHash": SimStruct({"size": SimTypeInt(signed=False, label="UInt32"), "data": SimTypePointer(SimTypeChar(label="Byte"), offset=0)}, name="FWP_BYTE_BLOB", pack=False, align=None), "eku": SimTypePointer(SimStruct({"numEku": SimTypeInt(signed=False, label="UInt32"), "eku": SimTypePointer(SimTypePointer(SimTypeChar(label="Byte"), offset=0), offset=0)}, name="IKEEXT_CERT_EKUS0", pack=False, align=None), offset=0), "name": SimTypePointer(SimStruct({"nameType": SimTypeInt(signed=False, label="IKEEXT_CERT_CRITERIA_NAME_TYPE"), "certName": SimTypePointer(SimTypeChar(label="Char"), offset=0)}, name="IKEEXT_CERT_NAME0", pack=False, align=None), offset=0), "flags": SimTypeInt(signed=False, label="UInt32")}, name="IKEEXT_CERTIFICATE_CRITERIA0", pack=False, align=None), offset=0)}, name="_Anonymous2_e__Struct", pack=False, align=None), "Anonymous3": SimStruct({"inboundRootStoreArraySize": SimTypeInt(signed=False, label="UInt32"), "inboundTrustedRootStoreCriteria": SimTypePointer(SimStruct({"certData": SimStruct({"size": SimTypeInt(signed=False, label="UInt32"), "data": SimTypePointer(SimTypeChar(label="Byte"), offset=0)}, name="FWP_BYTE_BLOB", pack=False, align=None), "certHash": SimStruct({"size": SimTypeInt(signed=False, label="UInt32"), "data": SimTypePointer(SimTypeChar(label="Byte"), offset=0)}, name="FWP_BYTE_BLOB", pack=False, align=None), "eku": SimTypePointer(SimStruct({"numEku": SimTypeInt(signed=False, label="UInt32"), "eku": SimTypePointer(SimTypePointer(SimTypeChar(label="Byte"), offset=0), offset=0)}, name="IKEEXT_CERT_EKUS0", pack=False, align=None), offset=0), "name": SimTypePointer(SimStruct({"nameType": SimTypeInt(signed=False, label="IKEEXT_CERT_CRITERIA_NAME_TYPE"), "certName": SimTypePointer(SimTypeChar(label="Char"), offset=0)}, name="IKEEXT_CERT_NAME0", pack=False, align=None), offset=0), "flags": SimTypeInt(signed=False, label="UInt32")}, name="IKEEXT_CERTIFICATE_CRITERIA0", pack=False, align=None), offset=0)}, name="_Anonymous3_e__Struct", pack=False, align=None)}, name="<anon>", label="None"), "outboundConfigType": SimTypeInt(signed=False, label="IKEEXT_CERT_CONFIG_TYPE"), "Anonymous2": SimUnion({"Anonymous1": SimStruct({"outboundRootArraySize": SimTypeInt(signed=False, label="UInt32"), "outboundRootCriteria": SimTypePointer(SimStruct({"certData": SimStruct({"size": SimTypeInt(signed=False, label="UInt32"), "data": SimTypePointer(SimTypeChar(label="Byte"), offset=0)}, name="FWP_BYTE_BLOB", pack=False, align=None), "certHash": SimStruct({"size": SimTypeInt(signed=False, label="UInt32"), "data": SimTypePointer(SimTypeChar(label="Byte"), offset=0)}, name="FWP_BYTE_BLOB", pack=False, align=None), "eku": SimTypePointer(SimStruct({"numEku": SimTypeInt(signed=False, label="UInt32"), "eku": SimTypePointer(SimTypePointer(SimTypeChar(label="Byte"), offset=0), offset=0)}, name="IKEEXT_CERT_EKUS0", pack=False, align=None), offset=0), "name": SimTypePointer(SimStruct({"nameType": SimTypeInt(signed=False, label="IKEEXT_CERT_CRITERIA_NAME_TYPE"), "certName": SimTypePointer(SimTypeChar(label="Char"), offset=0)}, name="IKEEXT_CERT_NAME0", pack=False, align=None), offset=0), "flags": SimTypeInt(signed=False, label="UInt32")}, name="IKEEXT_CERTIFICATE_CRITERIA0", pack=False, align=None), offset=0)}, name="_Anonymous1_e__Struct", pack=False, align=None), "Anonymous2": SimStruct({"outboundEnterpriseStoreArraySize": SimTypeInt(signed=False, label="UInt32"), "outboundEnterpriseStoreCriteria": SimTypePointer(SimStruct({"certData": SimStruct({"size": SimTypeInt(signed=False, label="UInt32"), "data": SimTypePointer(SimTypeChar(label="Byte"), offset=0)}, name="FWP_BYTE_BLOB", pack=False, align=None), "certHash": SimStruct({"size": SimTypeInt(signed=False, label="UInt32"), "data": SimTypePointer(SimTypeChar(label="Byte"), offset=0)}, name="FWP_BYTE_BLOB", pack=False, align=None), "eku": SimTypePointer(SimStruct({"numEku": SimTypeInt(signed=False, label="UInt32"), "eku": SimTypePointer(SimTypePointer(SimTypeChar(label="Byte"), offset=0), offset=0)}, name="IKEEXT_CERT_EKUS0", pack=False, align=None), offset=0), "name": SimTypePointer(SimStruct({"nameType": SimTypeInt(signed=False, label="IKEEXT_CERT_CRITERIA_NAME_TYPE"), "certName": SimTypePointer(SimTypeChar(label="Char"), offset=0)}, name="IKEEXT_CERT_NAME0", pack=False, align=None), offset=0), "flags": SimTypeInt(signed=False, label="UInt32")}, name="IKEEXT_CERTIFICATE_CRITERIA0", pack=False, align=None), offset=0)}, name="_Anonymous2_e__Struct", pack=False, align=None), "Anonymous3": SimStruct({"outboundRootStoreArraySize": SimTypeInt(signed=False, label="UInt32"), "outboundTrustedRootStoreCriteria": SimTypePointer(SimStruct({"certData": SimStruct({"size": SimTypeInt(signed=False, label="UInt32"), "data": SimTypePointer(SimTypeChar(label="Byte"), offset=0)}, name="FWP_BYTE_BLOB", pack=False, align=None), "certHash": SimStruct({"size": SimTypeInt(signed=False, label="UInt32"), "data": SimTypePointer(SimTypeChar(label="Byte"), offset=0)}, name="FWP_BYTE_BLOB", pack=False, align=None), "eku": SimTypePointer(SimStruct({"numEku": SimTypeInt(signed=False, label="UInt32"), "eku": SimTypePointer(SimTypePointer(SimTypeChar(label="Byte"), offset=0), offset=0)}, name="IKEEXT_CERT_EKUS0", pack=False, align=None), offset=0), "name": SimTypePointer(SimStruct({"nameType": SimTypeInt(signed=False, label="IKEEXT_CERT_CRITERIA_NAME_TYPE"), "certName": SimTypePointer(SimTypeChar(label="Char"), offset=0)}, name="IKEEXT_CERT_NAME0", pack=False, align=None), offset=0), "flags": SimTypeInt(signed=False, label="UInt32")}, name="IKEEXT_CERTIFICATE_CRITERIA0", pack=False, align=None), offset=0)}, name="_Anonymous3_e__Struct", pack=False, align=None)}, name="<anon>", label="None"), "flags": SimTypeInt(signed=False, label="IKEEXT_CERT_AUTH"), "localCertLocationUrl": SimStruct({"size": SimTypeInt(signed=False, label="UInt32"), "data": SimTypePointer(SimTypeChar(label="Byte"), offset=0)}, name="FWP_BYTE_BLOB", pack=False, align=None)}, name="IKEEXT_CERTIFICATE_AUTHENTICATION2", pack=False, align=None), "cgaAuthentication": SimStruct({"keyContainerName": SimTypePointer(SimTypeChar(label="Char"), offset=0), "cspName": SimTypePointer(SimTypeChar(label="Char"), offset=0), "cspType": SimTypeInt(signed=False, label="UInt32"), "cgaModifier": SimStruct({"byteArray16": SimTypeFixedSizeArray(SimTypeChar(label="Byte"), 16)}, name="FWP_BYTE_ARRAY16", pack=False, align=None), "cgaCollisionCount": SimTypeChar(label="Byte")}, name="IKEEXT_IPV6_CGA_AUTHENTICATION0", pack=False, align=None), "eapAuthentication": SimStruct({"flags": SimTypeInt(signed=False, label="IKEEXT_EAP_AUTHENTICATION_FLAGS")}, name="IKEEXT_EAP_AUTHENTICATION0", pack=False, align=None)}, name="<anon>", label="None")}, name="IKEEXT_AUTHENTICATION_METHOD2", pack=False, align=None), offset=0), "initiatorImpersonationType": SimTypeInt(signed=False, label="IKEEXT_AUTHENTICATION_IMPERSONATION_TYPE")}, name="IKEEXT_EM_POLICY2", pack=False, align=None), offset=0)}, name="IPSEC_TRANSPORT_POLICY2", pack=False, align=None), offset=0), "authipQmTunnelPolicy": SimTypePointer(SimStruct({"flags": SimTypeInt(signed=False, label="UInt32"), "numIpsecProposals": SimTypeInt(signed=False, label="UInt32"), "ipsecProposals": SimTypePointer(SimStruct({"lifetime": SimStruct({"lifetimeSeconds": SimTypeInt(signed=False, label="UInt32"), "lifetimeKilobytes": SimTypeInt(signed=False, label="UInt32"), "lifetimePackets": SimTypeInt(signed=False, label="UInt32")}, name="IPSEC_SA_LIFETIME0", pack=False, align=None), "numSaTransforms": SimTypeInt(signed=False, label="UInt32"), "saTransforms": SimTypePointer(SimStruct({"ipsecTransformType": SimTypeInt(signed=False, label="IPSEC_TRANSFORM_TYPE"), "Anonymous": SimUnion({"ahTransform": SimTypePointer(SimStruct({"authTransformId": SimStruct({"authType": SimTypeInt(signed=False, label="IPSEC_AUTH_TYPE"), "authConfig": SimTypeChar(label="Byte")}, name="IPSEC_AUTH_TRANSFORM_ID0", pack=False, align=None), "cryptoModuleId": SimTypePointer(SimTypeBottom(label="Guid"), offset=0)}, name="IPSEC_AUTH_TRANSFORM0", pack=False, align=None), offset=0), "espAuthTransform": SimTypePointer(SimStruct({"authTransformId": SimStruct({"authType": SimTypeInt(signed=False, label="IPSEC_AUTH_TYPE"), "authConfig": SimTypeChar(label="Byte")}, name="IPSEC_AUTH_TRANSFORM_ID0", pack=False, align=None), "cryptoModuleId": SimTypePointer(SimTypeBottom(label="Guid"), offset=0)}, name="IPSEC_AUTH_TRANSFORM0", pack=False, align=None), offset=0), "espCipherTransform": SimTypePointer(SimStruct({"cipherTransformId": SimStruct({"cipherType": SimTypeInt(signed=False, label="IPSEC_CIPHER_TYPE"), "cipherConfig": SimTypeChar(label="Byte")}, name="IPSEC_CIPHER_TRANSFORM_ID0", pack=False, align=None), "cryptoModuleId": SimTypePointer(SimTypeBottom(label="Guid"), offset=0)}, name="IPSEC_CIPHER_TRANSFORM0", pack=False, align=None), offset=0), "espAuthAndCipherTransform": SimTypePointer(SimStruct({"authTransform": SimStruct({"authTransformId": SimStruct({"authType": SimTypeInt(signed=False, label="IPSEC_AUTH_TYPE"), "authConfig": SimTypeChar(label="Byte")}, name="IPSEC_AUTH_TRANSFORM_ID0", pack=False, align=None), "cryptoModuleId": SimTypePointer(SimTypeBottom(label="Guid"), offset=0)}, name="IPSEC_AUTH_TRANSFORM0", pack=False, align=None), "cipherTransform": SimStruct({"cipherTransformId": SimStruct({"cipherType": SimTypeInt(signed=False, label="IPSEC_CIPHER_TYPE"), "cipherConfig": SimTypeChar(label="Byte")}, name="IPSEC_CIPHER_TRANSFORM_ID0", pack=False, align=None), "cryptoModuleId": SimTypePointer(SimTypeBottom(label="Guid"), offset=0)}, name="IPSEC_CIPHER_TRANSFORM0", pack=False, align=None)}, name="IPSEC_AUTH_AND_CIPHER_TRANSFORM0", pack=False, align=None), offset=0), "espAuthFwTransform": SimTypePointer(SimStruct({"authTransformId": SimStruct({"authType": SimTypeInt(signed=False, label="IPSEC_AUTH_TYPE"), "authConfig": SimTypeChar(label="Byte")}, name="IPSEC_AUTH_TRANSFORM_ID0", pack=False, align=None), "cryptoModuleId": SimTypePointer(SimTypeBottom(label="Guid"), offset=0)}, name="IPSEC_AUTH_TRANSFORM0", pack=False, align=None), offset=0)}, name="<anon>", label="None")}, name="IPSEC_SA_TRANSFORM0", pack=False, align=None), offset=0), "pfsGroup": SimTypeInt(signed=False, label="IPSEC_PFS_GROUP")}, name="IPSEC_PROPOSAL0", pack=False, align=None), offset=0), "tunnelEndpoints": SimStruct({"ipVersion": SimTypeInt(signed=False, label="FWP_IP_VERSION"), "Anonymous1": SimUnion({"localV4Address": SimTypeInt(signed=False, label="UInt32"), "localV6Address": SimTypeFixedSizeArray(SimTypeChar(label="Byte"), 16)}, name="<anon>", label="None"), "Anonymous2": SimUnion({"remoteV4Address": SimTypeInt(signed=False, label="UInt32"), "remoteV6Address": SimTypeFixedSizeArray(SimTypeChar(label="Byte"), 16)}, name="<anon>", label="None"), "localIfLuid": SimTypeLongLong(signed=False, label="UInt64"), "remoteFqdn": SimTypePointer(SimTypeChar(label="Char"), offset=0), "numAddresses": SimTypeInt(signed=False, label="UInt32"), "remoteAddresses": SimTypePointer(SimStruct({"ipVersion": SimTypeInt(signed=False, label="FWP_IP_VERSION"), "Anonymous": SimUnion({"v4Address": SimTypeInt(signed=False, label="UInt32"), "v6Address": SimTypeFixedSizeArray(SimTypeChar(label="Byte"), 16)}, name="<anon>", label="None")}, name="IPSEC_TUNNEL_ENDPOINT0", pack=False, align=None), offset=0)}, name="IPSEC_TUNNEL_ENDPOINTS2", pack=False, align=None), "saIdleTimeout": SimStruct({"idleTimeoutSeconds": SimTypeInt(signed=False, label="UInt32"), "idleTimeoutSecondsFailOver": SimTypeInt(signed=False, label="UInt32")}, name="IPSEC_SA_IDLE_TIMEOUT0", pack=False, align=None), "emPolicy": SimTypePointer(SimStruct({"numAuthenticationMethods": SimTypeInt(signed=False, label="UInt32"), "authenticationMethods": SimTypePointer(SimStruct({"authenticationMethodType": SimTypeInt(signed=False, label="IKEEXT_AUTHENTICATION_METHOD_TYPE"), "Anonymous": SimUnion({"presharedKeyAuthentication": SimStruct({"presharedKey": SimStruct({"size": SimTypeInt(signed=False, label="UInt32"), "data": SimTypePointer(SimTypeChar(label="Byte"), offset=0)}, name="FWP_BYTE_BLOB", pack=False, align=None), "flags": SimTypeInt(signed=False, label="IKEEXT_PRESHARED_KEY_AUTHENTICATION_FLAGS")}, name="IKEEXT_PRESHARED_KEY_AUTHENTICATION1", pack=False, align=None), "certificateAuthentication": SimStruct({"inboundConfigType": SimTypeInt(signed=False, label="IKEEXT_CERT_CONFIG_TYPE"), "Anonymous1": SimUnion({"Anonymous1": SimStruct({"inboundRootArraySize": SimTypeInt(signed=False, label="UInt32"), "inboundRootCriteria": SimTypePointer(SimStruct({"certData": SimStruct({"size": SimTypeInt(signed=False, label="UInt32"), "data": SimTypePointer(SimTypeChar(label="Byte"), offset=0)}, name="FWP_BYTE_BLOB", pack=False, align=None), "certHash": SimStruct({"size": SimTypeInt(signed=False, label="UInt32"), "data": SimTypePointer(SimTypeChar(label="Byte"), offset=0)}, name="FWP_BYTE_BLOB", pack=False, align=None), "eku": SimTypePointer(SimStruct({"numEku": SimTypeInt(signed=False, label="UInt32"), "eku": SimTypePointer(SimTypePointer(SimTypeChar(label="Byte"), offset=0), offset=0)}, name="IKEEXT_CERT_EKUS0", pack=False, align=None), offset=0), "name": SimTypePointer(SimStruct({"nameType": SimTypeInt(signed=False, label="IKEEXT_CERT_CRITERIA_NAME_TYPE"), "certName": SimTypePointer(SimTypeChar(label="Char"), offset=0)}, name="IKEEXT_CERT_NAME0", pack=False, align=None), offset=0), "flags": SimTypeInt(signed=False, label="UInt32")}, name="IKEEXT_CERTIFICATE_CRITERIA0", pack=False, align=None), offset=0)}, name="_Anonymous1_e__Struct", pack=False, align=None), "Anonymous2": SimStruct({"inboundEnterpriseStoreArraySize": SimTypeInt(signed=False, label="UInt32"), "inboundEnterpriseStoreCriteria": SimTypePointer(SimStruct({"certData": SimStruct({"size": SimTypeInt(signed=False, label="UInt32"), "data": SimTypePointer(SimTypeChar(label="Byte"), offset=0)}, name="FWP_BYTE_BLOB", pack=False, align=None), "certHash": SimStruct({"size": SimTypeInt(signed=False, label="UInt32"), "data": SimTypePointer(SimTypeChar(label="Byte"), offset=0)}, name="FWP_BYTE_BLOB", pack=False, align=None), "eku": SimTypePointer(SimStruct({"numEku": SimTypeInt(signed=False, label="UInt32"), "eku": SimTypePointer(SimTypePointer(SimTypeChar(label="Byte"), offset=0), offset=0)}, name="IKEEXT_CERT_EKUS0", pack=False, align=None), offset=0), "name": SimTypePointer(SimStruct({"nameType": SimTypeInt(signed=False, label="IKEEXT_CERT_CRITERIA_NAME_TYPE"), "certName": SimTypePointer(SimTypeChar(label="Char"), offset=0)}, name="IKEEXT_CERT_NAME0", pack=False, align=None), offset=0), "flags": SimTypeInt(signed=False, label="UInt32")}, name="IKEEXT_CERTIFICATE_CRITERIA0", pack=False, align=None), offset=0)}, name="_Anonymous2_e__Struct", pack=False, align=None), "Anonymous3": SimStruct({"inboundRootStoreArraySize": SimTypeInt(signed=False, label="UInt32"), "inboundTrustedRootStoreCriteria": SimTypePointer(SimStruct({"certData": SimStruct({"size": SimTypeInt(signed=False, label="UInt32"), "data": SimTypePointer(SimTypeChar(label="Byte"), offset=0)}, name="FWP_BYTE_BLOB", pack=False, align=None), "certHash": SimStruct({"size": SimTypeInt(signed=False, label="UInt32"), "data": SimTypePointer(SimTypeChar(label="Byte"), offset=0)}, name="FWP_BYTE_BLOB", pack=False, align=None), "eku": SimTypePointer(SimStruct({"numEku": SimTypeInt(signed=False, label="UInt32"), "eku": SimTypePointer(SimTypePointer(SimTypeChar(label="Byte"), offset=0), offset=0)}, name="IKEEXT_CERT_EKUS0", pack=False, align=None), offset=0), "name": SimTypePointer(SimStruct({"nameType": SimTypeInt(signed=False, label="IKEEXT_CERT_CRITERIA_NAME_TYPE"), "certName": SimTypePointer(SimTypeChar(label="Char"), offset=0)}, name="IKEEXT_CERT_NAME0", pack=False, align=None), offset=0), "flags": SimTypeInt(signed=False, label="UInt32")}, name="IKEEXT_CERTIFICATE_CRITERIA0", pack=False, align=None), offset=0)}, name="_Anonymous3_e__Struct", pack=False, align=None)}, name="<anon>", label="None"), "outboundConfigType": SimTypeInt(signed=False, label="IKEEXT_CERT_CONFIG_TYPE"), "Anonymous2": SimUnion({"Anonymous1": SimStruct({"outboundRootArraySize": SimTypeInt(signed=False, label="UInt32"), "outboundRootCriteria": SimTypePointer(SimStruct({"certData": SimStruct({"size": SimTypeInt(signed=False, label="UInt32"), "data": SimTypePointer(SimTypeChar(label="Byte"), offset=0)}, name="FWP_BYTE_BLOB", pack=False, align=None), "certHash": SimStruct({"size": SimTypeInt(signed=False, label="UInt32"), "data": SimTypePointer(SimTypeChar(label="Byte"), offset=0)}, name="FWP_BYTE_BLOB", pack=False, align=None), "eku": SimTypePointer(SimStruct({"numEku": SimTypeInt(signed=False, label="UInt32"), "eku": SimTypePointer(SimTypePointer(SimTypeChar(label="Byte"), offset=0), offset=0)}, name="IKEEXT_CERT_EKUS0", pack=False, align=None), offset=0), "name": SimTypePointer(SimStruct({"nameType": SimTypeInt(signed=False, label="IKEEXT_CERT_CRITERIA_NAME_TYPE"), "certName": SimTypePointer(SimTypeChar(label="Char"), offset=0)}, name="IKEEXT_CERT_NAME0", pack=False, align=None), offset=0), "flags": SimTypeInt(signed=False, label="UInt32")}, name="IKEEXT_CERTIFICATE_CRITERIA0", pack=False, align=None), offset=0)}, name="_Anonymous1_e__Struct", pack=False, align=None), "Anonymous2": SimStruct({"outboundEnterpriseStoreArraySize": SimTypeInt(signed=False, label="UInt32"), "outboundEnterpriseStoreCriteria": SimTypePointer(SimStruct({"certData": SimStruct({"size": SimTypeInt(signed=False, label="UInt32"), "data": SimTypePointer(SimTypeChar(label="Byte"), offset=0)}, name="FWP_BYTE_BLOB", pack=False, align=None), "certHash": SimStruct({"size": SimTypeInt(signed=False, label="UInt32"), "data": SimTypePointer(SimTypeChar(label="Byte"), offset=0)}, name="FWP_BYTE_BLOB", pack=False, align=None), "eku": SimTypePointer(SimStruct({"numEku": SimTypeInt(signed=False, label="UInt32"), "eku": SimTypePointer(SimTypePointer(SimTypeChar(label="Byte"), offset=0), offset=0)}, name="IKEEXT_CERT_EKUS0", pack=False, align=None), offset=0), "name": SimTypePointer(SimStruct({"nameType": SimTypeInt(signed=False, label="IKEEXT_CERT_CRITERIA_NAME_TYPE"), "certName": SimTypePointer(SimTypeChar(label="Char"), offset=0)}, name="IKEEXT_CERT_NAME0", pack=False, align=None), offset=0), "flags": SimTypeInt(signed=False, label="UInt32")}, name="IKEEXT_CERTIFICATE_CRITERIA0", pack=False, align=None), offset=0)}, name="_Anonymous2_e__Struct", pack=False, align=None), "Anonymous3": SimStruct({"outboundRootStoreArraySize": SimTypeInt(signed=False, label="UInt32"), "outboundTrustedRootStoreCriteria": SimTypePointer(SimStruct({"certData": SimStruct({"size": SimTypeInt(signed=False, label="UInt32"), "data": SimTypePointer(SimTypeChar(label="Byte"), offset=0)}, name="FWP_BYTE_BLOB", pack=False, align=None), "certHash": SimStruct({"size": SimTypeInt(signed=False, label="UInt32"), "data": SimTypePointer(SimTypeChar(label="Byte"), offset=0)}, name="FWP_BYTE_BLOB", pack=False, align=None), "eku": SimTypePointer(SimStruct({"numEku": SimTypeInt(signed=False, label="UInt32"), "eku": SimTypePointer(SimTypePointer(SimTypeChar(label="Byte"), offset=0), offset=0)}, name="IKEEXT_CERT_EKUS0", pack=False, align=None), offset=0), "name": SimTypePointer(SimStruct({"nameType": SimTypeInt(signed=False, label="IKEEXT_CERT_CRITERIA_NAME_TYPE"), "certName": SimTypePointer(SimTypeChar(label="Char"), offset=0)}, name="IKEEXT_CERT_NAME0", pack=False, align=None), offset=0), "flags": SimTypeInt(signed=False, label="UInt32")}, name="IKEEXT_CERTIFICATE_CRITERIA0", pack=False, align=None), offset=0)}, name="_Anonymous3_e__Struct", pack=False, align=None)}, name="<anon>", label="None"), "flags": SimTypeInt(signed=False, label="IKEEXT_CERT_AUTH"), "localCertLocationUrl": SimStruct({"size": SimTypeInt(signed=False, label="UInt32"), "data": SimTypePointer(SimTypeChar(label="Byte"), offset=0)}, name="FWP_BYTE_BLOB", pack=False, align=None)}, name="IKEEXT_CERTIFICATE_AUTHENTICATION2", pack=False, align=None), "kerberosAuthentication": SimStruct({"flags": SimTypeInt(signed=False, label="IKEEXT_KERBEROS_AUTHENTICATION_FLAGS"), "proxyServer": SimTypePointer(SimTypeChar(label="Char"), offset=0)}, name="IKEEXT_KERBEROS_AUTHENTICATION1", pack=False, align=None), "reservedAuthentication": SimStruct({"flags": SimTypeInt(signed=False, label="IKEEXT_RESERVED_AUTHENTICATION_FLAGS")}, name="IKEEXT_RESERVED_AUTHENTICATION0", pack=False, align=None), "ntlmV2Authentication": SimStruct({"flags": SimTypeInt(signed=False, label="UInt32")}, name="IKEEXT_NTLM_V2_AUTHENTICATION0", pack=False, align=None), "sslAuthentication": SimStruct({"inboundConfigType": SimTypeInt(signed=False, label="IKEEXT_CERT_CONFIG_TYPE"), "Anonymous1": SimUnion({"Anonymous1": SimStruct({"inboundRootArraySize": SimTypeInt(signed=False, label="UInt32"), "inboundRootCriteria": SimTypePointer(SimStruct({"certData": SimStruct({"size": SimTypeInt(signed=False, label="UInt32"), "data": SimTypePointer(SimTypeChar(label="Byte"), offset=0)}, name="FWP_BYTE_BLOB", pack=False, align=None), "certHash": SimStruct({"size": SimTypeInt(signed=False, label="UInt32"), "data": SimTypePointer(SimTypeChar(label="Byte"), offset=0)}, name="FWP_BYTE_BLOB", pack=False, align=None), "eku": SimTypePointer(SimStruct({"numEku": SimTypeInt(signed=False, label="UInt32"), "eku": SimTypePointer(SimTypePointer(SimTypeChar(label="Byte"), offset=0), offset=0)}, name="IKEEXT_CERT_EKUS0", pack=False, align=None), offset=0), "name": SimTypePointer(SimStruct({"nameType": SimTypeInt(signed=False, label="IKEEXT_CERT_CRITERIA_NAME_TYPE"), "certName": SimTypePointer(SimTypeChar(label="Char"), offset=0)}, name="IKEEXT_CERT_NAME0", pack=False, align=None), offset=0), "flags": SimTypeInt(signed=False, label="UInt32")}, name="IKEEXT_CERTIFICATE_CRITERIA0", pack=False, align=None), offset=0)}, name="_Anonymous1_e__Struct", pack=False, align=None), "Anonymous2": SimStruct({"inboundEnterpriseStoreArraySize": SimTypeInt(signed=False, label="UInt32"), "inboundEnterpriseStoreCriteria": SimTypePointer(SimStruct({"certData": SimStruct({"size": SimTypeInt(signed=False, label="UInt32"), "data": SimTypePointer(SimTypeChar(label="Byte"), offset=0)}, name="FWP_BYTE_BLOB", pack=False, align=None), "certHash": SimStruct({"size": SimTypeInt(signed=False, label="UInt32"), "data": SimTypePointer(SimTypeChar(label="Byte"), offset=0)}, name="FWP_BYTE_BLOB", pack=False, align=None), "eku": SimTypePointer(SimStruct({"numEku": SimTypeInt(signed=False, label="UInt32"), "eku": SimTypePointer(SimTypePointer(SimTypeChar(label="Byte"), offset=0), offset=0)}, name="IKEEXT_CERT_EKUS0", pack=False, align=None), offset=0), "name": SimTypePointer(SimStruct({"nameType": SimTypeInt(signed=False, label="IKEEXT_CERT_CRITERIA_NAME_TYPE"), "certName": SimTypePointer(SimTypeChar(label="Char"), offset=0)}, name="IKEEXT_CERT_NAME0", pack=False, align=None), offset=0), "flags": SimTypeInt(signed=False, label="UInt32")}, name="IKEEXT_CERTIFICATE_CRITERIA0", pack=False, align=None), offset=0)}, name="_Anonymous2_e__Struct", pack=False, align=None), "Anonymous3": SimStruct({"inboundRootStoreArraySize": SimTypeInt(signed=False, label="UInt32"), "inboundTrustedRootStoreCriteria": SimTypePointer(SimStruct({"certData": SimStruct({"size": SimTypeInt(signed=False, label="UInt32"), "data": SimTypePointer(SimTypeChar(label="Byte"), offset=0)}, name="FWP_BYTE_BLOB", pack=False, align=None), "certHash": SimStruct({"size": SimTypeInt(signed=False, label="UInt32"), "data": SimTypePointer(SimTypeChar(label="Byte"), offset=0)}, name="FWP_BYTE_BLOB", pack=False, align=None), "eku": SimTypePointer(SimStruct({"numEku": SimTypeInt(signed=False, label="UInt32"), "eku": SimTypePointer(SimTypePointer(SimTypeChar(label="Byte"), offset=0), offset=0)}, name="IKEEXT_CERT_EKUS0", pack=False, align=None), offset=0), "name": SimTypePointer(SimStruct({"nameType": SimTypeInt(signed=False, label="IKEEXT_CERT_CRITERIA_NAME_TYPE"), "certName": SimTypePointer(SimTypeChar(label="Char"), offset=0)}, name="IKEEXT_CERT_NAME0", pack=False, align=None), offset=0), "flags": SimTypeInt(signed=False, label="UInt32")}, name="IKEEXT_CERTIFICATE_CRITERIA0", pack=False, align=None), offset=0)}, name="_Anonymous3_e__Struct", pack=False, align=None)}, name="<anon>", label="None"), "outboundConfigType": SimTypeInt(signed=False, label="IKEEXT_CERT_CONFIG_TYPE"), "Anonymous2": SimUnion({"Anonymous1": SimStruct({"outboundRootArraySize": SimTypeInt(signed=False, label="UInt32"), "outboundRootCriteria": SimTypePointer(SimStruct({"certData": SimStruct({"size": SimTypeInt(signed=False, label="UInt32"), "data": SimTypePointer(SimTypeChar(label="Byte"), offset=0)}, name="FWP_BYTE_BLOB", pack=False, align=None), "certHash": SimStruct({"size": SimTypeInt(signed=False, label="UInt32"), "data": SimTypePointer(SimTypeChar(label="Byte"), offset=0)}, name="FWP_BYTE_BLOB", pack=False, align=None), "eku": SimTypePointer(SimStruct({"numEku": SimTypeInt(signed=False, label="UInt32"), "eku": SimTypePointer(SimTypePointer(SimTypeChar(label="Byte"), offset=0), offset=0)}, name="IKEEXT_CERT_EKUS0", pack=False, align=None), offset=0), "name": SimTypePointer(SimStruct({"nameType": SimTypeInt(signed=False, label="IKEEXT_CERT_CRITERIA_NAME_TYPE"), "certName": SimTypePointer(SimTypeChar(label="Char"), offset=0)}, name="IKEEXT_CERT_NAME0", pack=False, align=None), offset=0), "flags": SimTypeInt(signed=False, label="UInt32")}, name="IKEEXT_CERTIFICATE_CRITERIA0", pack=False, align=None), offset=0)}, name="_Anonymous1_e__Struct", pack=False, align=None), "Anonymous2": SimStruct({"outboundEnterpriseStoreArraySize": SimTypeInt(signed=False, label="UInt32"), "outboundEnterpriseStoreCriteria": SimTypePointer(SimStruct({"certData": SimStruct({"size": SimTypeInt(signed=False, label="UInt32"), "data": SimTypePointer(SimTypeChar(label="Byte"), offset=0)}, name="FWP_BYTE_BLOB", pack=False, align=None), "certHash": SimStruct({"size": SimTypeInt(signed=False, label="UInt32"), "data": SimTypePointer(SimTypeChar(label="Byte"), offset=0)}, name="FWP_BYTE_BLOB", pack=False, align=None), "eku": SimTypePointer(SimStruct({"numEku": SimTypeInt(signed=False, label="UInt32"), "eku": SimTypePointer(SimTypePointer(SimTypeChar(label="Byte"), offset=0), offset=0)}, name="IKEEXT_CERT_EKUS0", pack=False, align=None), offset=0), "name": SimTypePointer(SimStruct({"nameType": SimTypeInt(signed=False, label="IKEEXT_CERT_CRITERIA_NAME_TYPE"), "certName": SimTypePointer(SimTypeChar(label="Char"), offset=0)}, name="IKEEXT_CERT_NAME0", pack=False, align=None), offset=0), "flags": SimTypeInt(signed=False, label="UInt32")}, name="IKEEXT_CERTIFICATE_CRITERIA0", pack=False, align=None), offset=0)}, name="_Anonymous2_e__Struct", pack=False, align=None), "Anonymous3": SimStruct({"outboundRootStoreArraySize": SimTypeInt(signed=False, label="UInt32"), "outboundTrustedRootStoreCriteria": SimTypePointer(SimStruct({"certData": SimStruct({"size": SimTypeInt(signed=False, label="UInt32"), "data": SimTypePointer(SimTypeChar(label="Byte"), offset=0)}, name="FWP_BYTE_BLOB", pack=False, align=None), "certHash": SimStruct({"size": SimTypeInt(signed=False, label="UInt32"), "data": SimTypePointer(SimTypeChar(label="Byte"), offset=0)}, name="FWP_BYTE_BLOB", pack=False, align=None), "eku": SimTypePointer(SimStruct({"numEku": SimTypeInt(signed=False, label="UInt32"), "eku": SimTypePointer(SimTypePointer(SimTypeChar(label="Byte"), offset=0), offset=0)}, name="IKEEXT_CERT_EKUS0", pack=False, align=None), offset=0), "name": SimTypePointer(SimStruct({"nameType": SimTypeInt(signed=False, label="IKEEXT_CERT_CRITERIA_NAME_TYPE"), "certName": SimTypePointer(SimTypeChar(label="Char"), offset=0)}, name="IKEEXT_CERT_NAME0", pack=False, align=None), offset=0), "flags": SimTypeInt(signed=False, label="UInt32")}, name="IKEEXT_CERTIFICATE_CRITERIA0", pack=False, align=None), offset=0)}, name="_Anonymous3_e__Struct", pack=False, align=None)}, name="<anon>", label="None"), "flags": SimTypeInt(signed=False, label="IKEEXT_CERT_AUTH"), "localCertLocationUrl": SimStruct({"size": SimTypeInt(signed=False, label="UInt32"), "data": SimTypePointer(SimTypeChar(label="Byte"), offset=0)}, name="FWP_BYTE_BLOB", pack=False, align=None)}, name="IKEEXT_CERTIFICATE_AUTHENTICATION2", pack=False, align=None), "cgaAuthentication": SimStruct({"keyContainerName": SimTypePointer(SimTypeChar(label="Char"), offset=0), "cspName": SimTypePointer(SimTypeChar(label="Char"), offset=0), "cspType": SimTypeInt(signed=False, label="UInt32"), "cgaModifier": SimStruct({"byteArray16": SimTypeFixedSizeArray(SimTypeChar(label="Byte"), 16)}, name="FWP_BYTE_ARRAY16", pack=False, align=None), "cgaCollisionCount": SimTypeChar(label="Byte")}, name="IKEEXT_IPV6_CGA_AUTHENTICATION0", pack=False, align=None), "eapAuthentication": SimStruct({"flags": SimTypeInt(signed=False, label="IKEEXT_EAP_AUTHENTICATION_FLAGS")}, name="IKEEXT_EAP_AUTHENTICATION0", pack=False, align=None)}, name="<anon>", label="None")}, name="IKEEXT_AUTHENTICATION_METHOD2", pack=False, align=None), offset=0), "initiatorImpersonationType": SimTypeInt(signed=False, label="IKEEXT_AUTHENTICATION_IMPERSONATION_TYPE")}, name="IKEEXT_EM_POLICY2", pack=False, align=None), offset=0), "fwdPathSaLifetime": SimTypeInt(signed=False, label="UInt32"), "compartmentId": SimTypeInt(signed=False, label="UInt32"), "numTrafficSelectorPolicy": SimTypeInt(signed=False, label="UInt32"), "trafficSelectorPolicies": SimTypePointer(SimStruct({"flags": SimTypeInt(signed=False, label="UInt32"), "numLocalTrafficSelectors": SimTypeInt(signed=False, label="UInt32"), "localTrafficSelectors": SimTypePointer(SimStruct({"protocolId": SimTypeChar(label="Byte"), "portStart": SimTypeShort(signed=False, label="UInt16"), "portEnd": SimTypeShort(signed=False, label="UInt16"), "ipVersion": SimTypeInt(signed=False, label="FWP_IP_VERSION"), "Anonymous1": SimUnion({"startV4Address": SimTypeInt(signed=False, label="UInt32"), "startV6Address": SimTypeFixedSizeArray(SimTypeChar(label="Byte"), 16)}, name="<anon>", label="None"), "Anonymous2": SimUnion({"endV4Address": SimTypeInt(signed=False, label="UInt32"), "endV6Address": SimTypeFixedSizeArray(SimTypeChar(label="Byte"), 16)}, name="<anon>", label="None")}, name="IPSEC_TRAFFIC_SELECTOR0_", pack=False, align=None), offset=0), "numRemoteTrafficSelectors": SimTypeInt(signed=False, label="UInt32"), "remoteTrafficSelectors": SimTypePointer(SimStruct({"protocolId": SimTypeChar(label="Byte"), "portStart": SimTypeShort(signed=False, label="UInt16"), "portEnd": SimTypeShort(signed=False, label="UInt16"), "ipVersion": SimTypeInt(signed=False, label="FWP_IP_VERSION"), "Anonymous1": SimUnion({"startV4Address": SimTypeInt(signed=False, label="UInt32"), "startV6Address": SimTypeFixedSizeArray(SimTypeChar(label="Byte"), 16)}, name="<anon>", label="None"), "Anonymous2": SimUnion({"endV4Address": SimTypeInt(signed=False, label="UInt32"), "endV6Address": SimTypeFixedSizeArray(SimTypeChar(label="Byte"), 16)}, name="<anon>", label="None")}, name="IPSEC_TRAFFIC_SELECTOR0_", pack=False, align=None), offset=0)}, name="IPSEC_TRAFFIC_SELECTOR_POLICY0_", pack=False, align=None), offset=0)}, name="IPSEC_TUNNEL_POLICY3_", pack=False, align=None), offset=0), "ikeMmPolicy": SimTypePointer(SimStruct({"softExpirationTime": SimTypeInt(signed=False, label="UInt32"), "numAuthenticationMethods": SimTypeInt(signed=False, label="UInt32"), "authenticationMethods": SimTypePointer(SimStruct({"authenticationMethodType": SimTypeInt(signed=False, label="IKEEXT_AUTHENTICATION_METHOD_TYPE"), "Anonymous": SimUnion({"presharedKeyAuthentication": SimStruct({"presharedKey": SimStruct({"size": SimTypeInt(signed=False, label="UInt32"), "data": SimTypePointer(SimTypeChar(label="Byte"), offset=0)}, name="FWP_BYTE_BLOB", pack=False, align=None), "flags": SimTypeInt(signed=False, label="IKEEXT_PRESHARED_KEY_AUTHENTICATION_FLAGS")}, name="IKEEXT_PRESHARED_KEY_AUTHENTICATION1", pack=False, align=None), "certificateAuthentication": SimStruct({"inboundConfigType": SimTypeInt(signed=False, label="IKEEXT_CERT_CONFIG_TYPE"), "Anonymous1": SimUnion({"Anonymous1": SimStruct({"inboundRootArraySize": SimTypeInt(signed=False, label="UInt32"), "inboundRootCriteria": SimTypePointer(SimStruct({"certData": SimStruct({"size": SimTypeInt(signed=False, label="UInt32"), "data": SimTypePointer(SimTypeChar(label="Byte"), offset=0)}, name="FWP_BYTE_BLOB", pack=False, align=None), "certHash": SimStruct({"size": SimTypeInt(signed=False, label="UInt32"), "data": SimTypePointer(SimTypeChar(label="Byte"), offset=0)}, name="FWP_BYTE_BLOB", pack=False, align=None), "eku": SimTypePointer(SimStruct({"numEku": SimTypeInt(signed=False, label="UInt32"), "eku": SimTypePointer(SimTypePointer(SimTypeChar(label="Byte"), offset=0), offset=0)}, name="IKEEXT_CERT_EKUS0", pack=False, align=None), offset=0), "name": SimTypePointer(SimStruct({"nameType": SimTypeInt(signed=False, label="IKEEXT_CERT_CRITERIA_NAME_TYPE"), "certName": SimTypePointer(SimTypeChar(label="Char"), offset=0)}, name="IKEEXT_CERT_NAME0", pack=False, align=None), offset=0), "flags": SimTypeInt(signed=False, label="UInt32")}, name="IKEEXT_CERTIFICATE_CRITERIA0", pack=False, align=None), offset=0)}, name="_Anonymous1_e__Struct", pack=False, align=None), "Anonymous2": SimStruct({"inboundEnterpriseStoreArraySize": SimTypeInt(signed=False, label="UInt32"), "inboundEnterpriseStoreCriteria": SimTypePointer(SimStruct({"certData": SimStruct({"size": SimTypeInt(signed=False, label="UInt32"), "data": SimTypePointer(SimTypeChar(label="Byte"), offset=0)}, name="FWP_BYTE_BLOB", pack=False, align=None), "certHash": SimStruct({"size": SimTypeInt(signed=False, label="UInt32"), "data": SimTypePointer(SimTypeChar(label="Byte"), offset=0)}, name="FWP_BYTE_BLOB", pack=False, align=None), "eku": SimTypePointer(SimStruct({"numEku": SimTypeInt(signed=False, label="UInt32"), "eku": SimTypePointer(SimTypePointer(SimTypeChar(label="Byte"), offset=0), offset=0)}, name="IKEEXT_CERT_EKUS0", pack=False, align=None), offset=0), "name": SimTypePointer(SimStruct({"nameType": SimTypeInt(signed=False, label="IKEEXT_CERT_CRITERIA_NAME_TYPE"), "certName": SimTypePointer(SimTypeChar(label="Char"), offset=0)}, name="IKEEXT_CERT_NAME0", pack=False, align=None), offset=0), "flags": SimTypeInt(signed=False, label="UInt32")}, name="IKEEXT_CERTIFICATE_CRITERIA0", pack=False, align=None), offset=0)}, name="_Anonymous2_e__Struct", pack=False, align=None), "Anonymous3": SimStruct({"inboundRootStoreArraySize": SimTypeInt(signed=False, label="UInt32"), "inboundTrustedRootStoreCriteria": SimTypePointer(SimStruct({"certData": SimStruct({"size": SimTypeInt(signed=False, label="UInt32"), "data": SimTypePointer(SimTypeChar(label="Byte"), offset=0)}, name="FWP_BYTE_BLOB", pack=False, align=None), "certHash": SimStruct({"size": SimTypeInt(signed=False, label="UInt32"), "data": SimTypePointer(SimTypeChar(label="Byte"), offset=0)}, name="FWP_BYTE_BLOB", pack=False, align=None), "eku": SimTypePointer(SimStruct({"numEku": SimTypeInt(signed=False, label="UInt32"), "eku": SimTypePointer(SimTypePointer(SimTypeChar(label="Byte"), offset=0), offset=0)}, name="IKEEXT_CERT_EKUS0", pack=False, align=None), offset=0), "name": SimTypePointer(SimStruct({"nameType": SimTypeInt(signed=False, label="IKEEXT_CERT_CRITERIA_NAME_TYPE"), "certName": SimTypePointer(SimTypeChar(label="Char"), offset=0)}, name="IKEEXT_CERT_NAME0", pack=False, align=None), offset=0), "flags": SimTypeInt(signed=False, label="UInt32")}, name="IKEEXT_CERTIFICATE_CRITERIA0", pack=False, align=None), offset=0)}, name="_Anonymous3_e__Struct", pack=False, align=None)}, name="<anon>", label="None"), "outboundConfigType": SimTypeInt(signed=False, label="IKEEXT_CERT_CONFIG_TYPE"), "Anonymous2": SimUnion({"Anonymous1": SimStruct({"outboundRootArraySize": SimTypeInt(signed=False, label="UInt32"), "outboundRootCriteria": SimTypePointer(SimStruct({"certData": SimStruct({"size": SimTypeInt(signed=False, label="UInt32"), "data": SimTypePointer(SimTypeChar(label="Byte"), offset=0)}, name="FWP_BYTE_BLOB", pack=False, align=None), "certHash": SimStruct({"size": SimTypeInt(signed=False, label="UInt32"), "data": SimTypePointer(SimTypeChar(label="Byte"), offset=0)}, name="FWP_BYTE_BLOB", pack=False, align=None), "eku": SimTypePointer(SimStruct({"numEku": SimTypeInt(signed=False, label="UInt32"), "eku": SimTypePointer(SimTypePointer(SimTypeChar(label="Byte"), offset=0), offset=0)}, name="IKEEXT_CERT_EKUS0", pack=False, align=None), offset=0), "name": SimTypePointer(SimStruct({"nameType": SimTypeInt(signed=False, label="IKEEXT_CERT_CRITERIA_NAME_TYPE"), "certName": SimTypePointer(SimTypeChar(label="Char"), offset=0)}, name="IKEEXT_CERT_NAME0", pack=False, align=None), offset=0), "flags": SimTypeInt(signed=False, label="UInt32")}, name="IKEEXT_CERTIFICATE_CRITERIA0", pack=False, align=None), offset=0)}, name="_Anonymous1_e__Struct", pack=False, align=None), "Anonymous2": SimStruct({"outboundEnterpriseStoreArraySize": SimTypeInt(signed=False, label="UInt32"), "outboundEnterpriseStoreCriteria": SimTypePointer(SimStruct({"certData": SimStruct({"size": SimTypeInt(signed=False, label="UInt32"), "data": SimTypePointer(SimTypeChar(label="Byte"), offset=0)}, name="FWP_BYTE_BLOB", pack=False, align=None), "certHash": SimStruct({"size": SimTypeInt(signed=False, label="UInt32"), "data": SimTypePointer(SimTypeChar(label="Byte"), offset=0)}, name="FWP_BYTE_BLOB", pack=False, align=None), "eku": SimTypePointer(SimStruct({"numEku": SimTypeInt(signed=False, label="UInt32"), "eku": SimTypePointer(SimTypePointer(SimTypeChar(label="Byte"), offset=0), offset=0)}, name="IKEEXT_CERT_EKUS0", pack=False, align=None), offset=0), "name": SimTypePointer(SimStruct({"nameType": SimTypeInt(signed=False, label="IKEEXT_CERT_CRITERIA_NAME_TYPE"), "certName": SimTypePointer(SimTypeChar(label="Char"), offset=0)}, name="IKEEXT_CERT_NAME0", pack=False, align=None), offset=0), "flags": SimTypeInt(signed=False, label="UInt32")}, name="IKEEXT_CERTIFICATE_CRITERIA0", pack=False, align=None), offset=0)}, name="_Anonymous2_e__Struct", pack=False, align=None), "Anonymous3": SimStruct({"outboundRootStoreArraySize": SimTypeInt(signed=False, label="UInt32"), "outboundTrustedRootStoreCriteria": SimTypePointer(SimStruct({"certData": SimStruct({"size": SimTypeInt(signed=False, label="UInt32"), "data": SimTypePointer(SimTypeChar(label="Byte"), offset=0)}, name="FWP_BYTE_BLOB", pack=False, align=None), "certHash": SimStruct({"size": SimTypeInt(signed=False, label="UInt32"), "data": SimTypePointer(SimTypeChar(label="Byte"), offset=0)}, name="FWP_BYTE_BLOB", pack=False, align=None), "eku": SimTypePointer(SimStruct({"numEku": SimTypeInt(signed=False, label="UInt32"), "eku": SimTypePointer(SimTypePointer(SimTypeChar(label="Byte"), offset=0), offset=0)}, name="IKEEXT_CERT_EKUS0", pack=False, align=None), offset=0), "name": SimTypePointer(SimStruct({"nameType": SimTypeInt(signed=False, label="IKEEXT_CERT_CRITERIA_NAME_TYPE"), "certName": SimTypePointer(SimTypeChar(label="Char"), offset=0)}, name="IKEEXT_CERT_NAME0", pack=False, align=None), offset=0), "flags": SimTypeInt(signed=False, label="UInt32")}, name="IKEEXT_CERTIFICATE_CRITERIA0", pack=False, align=None), offset=0)}, name="_Anonymous3_e__Struct", pack=False, align=None)}, name="<anon>", label="None"), "flags": SimTypeInt(signed=False, label="IKEEXT_CERT_AUTH"), "localCertLocationUrl": SimStruct({"size": SimTypeInt(signed=False, label="UInt32"), "data": SimTypePointer(SimTypeChar(label="Byte"), offset=0)}, name="FWP_BYTE_BLOB", pack=False, align=None)}, name="IKEEXT_CERTIFICATE_AUTHENTICATION2", pack=False, align=None), "kerberosAuthentication": SimStruct({"flags": SimTypeInt(signed=False, label="IKEEXT_KERBEROS_AUTHENTICATION_FLAGS"), "proxyServer": SimTypePointer(SimTypeChar(label="Char"), offset=0)}, name="IKEEXT_KERBEROS_AUTHENTICATION1", pack=False, align=None), "reservedAuthentication": SimStruct({"flags": SimTypeInt(signed=False, label="IKEEXT_RESERVED_AUTHENTICATION_FLAGS")}, name="IKEEXT_RESERVED_AUTHENTICATION0", pack=False, align=None), "ntlmV2Authentication": SimStruct({"flags": SimTypeInt(signed=False, label="UInt32")}, name="IKEEXT_NTLM_V2_AUTHENTICATION0", pack=False, align=None), "sslAuthentication": SimStruct({"inboundConfigType": SimTypeInt(signed=False, label="IKEEXT_CERT_CONFIG_TYPE"), "Anonymous1": SimUnion({"Anonymous1": SimStruct({"inboundRootArraySize": SimTypeInt(signed=False, label="UInt32"), "inboundRootCriteria": SimTypePointer(SimStruct({"certData": SimStruct({"size": SimTypeInt(signed=False, label="UInt32"), "data": SimTypePointer(SimTypeChar(label="Byte"), offset=0)}, name="FWP_BYTE_BLOB", pack=False, align=None), "certHash": SimStruct({"size": SimTypeInt(signed=False, label="UInt32"), "data": SimTypePointer(SimTypeChar(label="Byte"), offset=0)}, name="FWP_BYTE_BLOB", pack=False, align=None), "eku": SimTypePointer(SimStruct({"numEku": SimTypeInt(signed=False, label="UInt32"), "eku": SimTypePointer(SimTypePointer(SimTypeChar(label="Byte"), offset=0), offset=0)}, name="IKEEXT_CERT_EKUS0", pack=False, align=None), offset=0), "name": SimTypePointer(SimStruct({"nameType": SimTypeInt(signed=False, label="IKEEXT_CERT_CRITERIA_NAME_TYPE"), "certName": SimTypePointer(SimTypeChar(label="Char"), offset=0)}, name="IKEEXT_CERT_NAME0", pack=False, align=None), offset=0), "flags": SimTypeInt(signed=False, label="UInt32")}, name="IKEEXT_CERTIFICATE_CRITERIA0", pack=False, align=None), offset=0)}, name="_Anonymous1_e__Struct", pack=False, align=None), "Anonymous2": SimStruct({"inboundEnterpriseStoreArraySize": SimTypeInt(signed=False, label="UInt32"), "inboundEnterpriseStoreCriteria": SimTypePointer(SimStruct({"certData": SimStruct({"size": SimTypeInt(signed=False, label="UInt32"), "data": SimTypePointer(SimTypeChar(label="Byte"), offset=0)}, name="FWP_BYTE_BLOB", pack=False, align=None), "certHash": SimStruct({"size": SimTypeInt(signed=False, label="UInt32"), "data": SimTypePointer(SimTypeChar(label="Byte"), offset=0)}, name="FWP_BYTE_BLOB", pack=False, align=None), "eku": SimTypePointer(SimStruct({"numEku": SimTypeInt(signed=False, label="UInt32"), "eku": SimTypePointer(SimTypePointer(SimTypeChar(label="Byte"), offset=0), offset=0)}, name="IKEEXT_CERT_EKUS0", pack=False, align=None), offset=0), "name": SimTypePointer(SimStruct({"nameType": SimTypeInt(signed=False, label="IKEEXT_CERT_CRITERIA_NAME_TYPE"), "certName": SimTypePointer(SimTypeChar(label="Char"), offset=0)}, name="IKEEXT_CERT_NAME0", pack=False, align=None), offset=0), "flags": SimTypeInt(signed=False, label="UInt32")}, name="IKEEXT_CERTIFICATE_CRITERIA0", pack=False, align=None), offset=0)}, name="_Anonymous2_e__Struct", pack=False, align=None), "Anonymous3": SimStruct({"inboundRootStoreArraySize": SimTypeInt(signed=False, label="UInt32"), "inboundTrustedRootStoreCriteria": SimTypePointer(SimStruct({"certData": SimStruct({"size": SimTypeInt(signed=False, label="UInt32"), "data": SimTypePointer(SimTypeChar(label="Byte"), offset=0)}, name="FWP_BYTE_BLOB", pack=False, align=None), "certHash": SimStruct({"size": SimTypeInt(signed=False, label="UInt32"), "data": SimTypePointer(SimTypeChar(label="Byte"), offset=0)}, name="FWP_BYTE_BLOB", pack=False, align=None), "eku": SimTypePointer(SimStruct({"numEku": SimTypeInt(signed=False, label="UInt32"), "eku": SimTypePointer(SimTypePointer(SimTypeChar(label="Byte"), offset=0), offset=0)}, name="IKEEXT_CERT_EKUS0", pack=False, align=None), offset=0), "name": SimTypePointer(SimStruct({"nameType": SimTypeInt(signed=False, label="IKEEXT_CERT_CRITERIA_NAME_TYPE"), "certName": SimTypePointer(SimTypeChar(label="Char"), offset=0)}, name="IKEEXT_CERT_NAME0", pack=False, align=None), offset=0), "flags": SimTypeInt(signed=False, label="UInt32")}, name="IKEEXT_CERTIFICATE_CRITERIA0", pack=False, align=None), offset=0)}, name="_Anonymous3_e__Struct", pack=False, align=None)}, name="<anon>", label="None"), "outboundConfigType": SimTypeInt(signed=False, label="IKEEXT_CERT_CONFIG_TYPE"), "Anonymous2": SimUnion({"Anonymous1": SimStruct({"outboundRootArraySize": SimTypeInt(signed=False, label="UInt32"), "outboundRootCriteria": SimTypePointer(SimStruct({"certData": SimStruct({"size": SimTypeInt(signed=False, label="UInt32"), "data": SimTypePointer(SimTypeChar(label="Byte"), offset=0)}, name="FWP_BYTE_BLOB", pack=False, align=None), "certHash": SimStruct({"size": SimTypeInt(signed=False, label="UInt32"), "data": SimTypePointer(SimTypeChar(label="Byte"), offset=0)}, name="FWP_BYTE_BLOB", pack=False, align=None), "eku": SimTypePointer(SimStruct({"numEku": SimTypeInt(signed=False, label="UInt32"), "eku": SimTypePointer(SimTypePointer(SimTypeChar(label="Byte"), offset=0), offset=0)}, name="IKEEXT_CERT_EKUS0", pack=False, align=None), offset=0), "name": SimTypePointer(SimStruct({"nameType": SimTypeInt(signed=False, label="IKEEXT_CERT_CRITERIA_NAME_TYPE"), "certName": SimTypePointer(SimTypeChar(label="Char"), offset=0)}, name="IKEEXT_CERT_NAME0", pack=False, align=None), offset=0), "flags": SimTypeInt(signed=False, label="UInt32")}, name="IKEEXT_CERTIFICATE_CRITERIA0", pack=False, align=None), offset=0)}, name="_Anonymous1_e__Struct", pack=False, align=None), "Anonymous2": SimStruct({"outboundEnterpriseStoreArraySize": SimTypeInt(signed=False, label="UInt32"), "outboundEnterpriseStoreCriteria": SimTypePointer(SimStruct({"certData": SimStruct({"size": SimTypeInt(signed=False, label="UInt32"), "data": SimTypePointer(SimTypeChar(label="Byte"), offset=0)}, name="FWP_BYTE_BLOB", pack=False, align=None), "certHash": SimStruct({"size": SimTypeInt(signed=False, label="UInt32"), "data": SimTypePointer(SimTypeChar(label="Byte"), offset=0)}, name="FWP_BYTE_BLOB", pack=False, align=None), "eku": SimTypePointer(SimStruct({"numEku": SimTypeInt(signed=False, label="UInt32"), "eku": SimTypePointer(SimTypePointer(SimTypeChar(label="Byte"), offset=0), offset=0)}, name="IKEEXT_CERT_EKUS0", pack=False, align=None), offset=0), "name": SimTypePointer(SimStruct({"nameType": SimTypeInt(signed=False, label="IKEEXT_CERT_CRITERIA_NAME_TYPE"), "certName": SimTypePointer(SimTypeChar(label="Char"), offset=0)}, name="IKEEXT_CERT_NAME0", pack=False, align=None), offset=0), "flags": SimTypeInt(signed=False, label="UInt32")}, name="IKEEXT_CERTIFICATE_CRITERIA0", pack=False, align=None), offset=0)}, name="_Anonymous2_e__Struct", pack=False, align=None), "Anonymous3": SimStruct({"outboundRootStoreArraySize": SimTypeInt(signed=False, label="UInt32"), "outboundTrustedRootStoreCriteria": SimTypePointer(SimStruct({"certData": SimStruct({"size": SimTypeInt(signed=False, label="UInt32"), "data": SimTypePointer(SimTypeChar(label="Byte"), offset=0)}, name="FWP_BYTE_BLOB", pack=False, align=None), "certHash": SimStruct({"size": SimTypeInt(signed=False, label="UInt32"), "data": SimTypePointer(SimTypeChar(label="Byte"), offset=0)}, name="FWP_BYTE_BLOB", pack=False, align=None), "eku": SimTypePointer(SimStruct({"numEku": SimTypeInt(signed=False, label="UInt32"), "eku": SimTypePointer(SimTypePointer(SimTypeChar(label="Byte"), offset=0), offset=0)}, name="IKEEXT_CERT_EKUS0", pack=False, align=None), offset=0), "name": SimTypePointer(SimStruct({"nameType": SimTypeInt(signed=False, label="IKEEXT_CERT_CRITERIA_NAME_TYPE"), "certName": SimTypePointer(SimTypeChar(label="Char"), offset=0)}, name="IKEEXT_CERT_NAME0", pack=False, align=None), offset=0), "flags": SimTypeInt(signed=False, label="UInt32")}, name="IKEEXT_CERTIFICATE_CRITERIA0", pack=False, align=None), offset=0)}, name="_Anonymous3_e__Struct", pack=False, align=None)}, name="<anon>", label="None"), "flags": SimTypeInt(signed=False, label="IKEEXT_CERT_AUTH"), "localCertLocationUrl": SimStruct({"size": SimTypeInt(signed=False, label="UInt32"), "data": SimTypePointer(SimTypeChar(label="Byte"), offset=0)}, name="FWP_BYTE_BLOB", pack=False, align=None)}, name="IKEEXT_CERTIFICATE_AUTHENTICATION2", pack=False, align=None), "cgaAuthentication": SimStruct({"keyContainerName": SimTypePointer(SimTypeChar(label="Char"), offset=0), "cspName": SimTypePointer(SimTypeChar(label="Char"), offset=0), "cspType": SimTypeInt(signed=False, label="UInt32"), "cgaModifier": SimStruct({"byteArray16": SimTypeFixedSizeArray(SimTypeChar(label="Byte"), 16)}, name="FWP_BYTE_ARRAY16", pack=False, align=None), "cgaCollisionCount": SimTypeChar(label="Byte")}, name="IKEEXT_IPV6_CGA_AUTHENTICATION0", pack=False, align=None), "eapAuthentication": SimStruct({"flags": SimTypeInt(signed=False, label="IKEEXT_EAP_AUTHENTICATION_FLAGS")}, name="IKEEXT_EAP_AUTHENTICATION0", pack=False, align=None)}, name="<anon>", label="None")}, name="IKEEXT_AUTHENTICATION_METHOD2", pack=False, align=None), offset=0), "initiatorImpersonationType": SimTypeInt(signed=False, label="IKEEXT_AUTHENTICATION_IMPERSONATION_TYPE"), "numIkeProposals": SimTypeInt(signed=False, label="UInt32"), "ikeProposals": SimTypePointer(SimStruct({"cipherAlgorithm": SimStruct({"algoIdentifier": SimTypeInt(signed=False, label="IKEEXT_CIPHER_TYPE"), "keyLen": SimTypeInt(signed=False, label="UInt32"), "rounds": SimTypeInt(signed=False, label="UInt32")}, name="IKEEXT_CIPHER_ALGORITHM0", pack=False, align=None), "integrityAlgorithm": SimStruct({"algoIdentifier": SimTypeInt(signed=False, label="IKEEXT_INTEGRITY_TYPE")}, name="IKEEXT_INTEGRITY_ALGORITHM0", pack=False, align=None), "maxLifetimeSeconds": SimTypeInt(signed=False, label="UInt32"), "dhGroup": SimTypeInt(signed=False, label="IKEEXT_DH_GROUP"), "quickModeLimit": SimTypeInt(signed=False, label="UInt32")}, name="IKEEXT_PROPOSAL0", pack=False, align=None), offset=0), "flags": SimTypeInt(signed=False, label="IKEEXT_POLICY_FLAG"), "maxDynamicFilters": SimTypeInt(signed=False, label="UInt32"), "retransmitDurationSecs": SimTypeInt(signed=False, label="UInt32")}, name="IKEEXT_POLICY2", pack=False, align=None), offset=0), "authIpMmPolicy": SimTypePointer(SimStruct({"softExpirationTime": SimTypeInt(signed=False, label="UInt32"), "numAuthenticationMethods": SimTypeInt(signed=False, label="UInt32"), "authenticationMethods": SimTypePointer(SimStruct({"authenticationMethodType": SimTypeInt(signed=False, label="IKEEXT_AUTHENTICATION_METHOD_TYPE"), "Anonymous": SimUnion({"presharedKeyAuthentication": SimStruct({"presharedKey": SimStruct({"size": SimTypeInt(signed=False, label="UInt32"), "data": SimTypePointer(SimTypeChar(label="Byte"), offset=0)}, name="FWP_BYTE_BLOB", pack=False, align=None), "flags": SimTypeInt(signed=False, label="IKEEXT_PRESHARED_KEY_AUTHENTICATION_FLAGS")}, name="IKEEXT_PRESHARED_KEY_AUTHENTICATION1", pack=False, align=None), "certificateAuthentication": SimStruct({"inboundConfigType": SimTypeInt(signed=False, label="IKEEXT_CERT_CONFIG_TYPE"), "Anonymous1": SimUnion({"Anonymous1": SimStruct({"inboundRootArraySize": SimTypeInt(signed=False, label="UInt32"), "inboundRootCriteria": SimTypePointer(SimStruct({"certData": SimStruct({"size": SimTypeInt(signed=False, label="UInt32"), "data": SimTypePointer(SimTypeChar(label="Byte"), offset=0)}, name="FWP_BYTE_BLOB", pack=False, align=None), "certHash": SimStruct({"size": SimTypeInt(signed=False, label="UInt32"), "data": SimTypePointer(SimTypeChar(label="Byte"), offset=0)}, name="FWP_BYTE_BLOB", pack=False, align=None), "eku": SimTypePointer(SimStruct({"numEku": SimTypeInt(signed=False, label="UInt32"), "eku": SimTypePointer(SimTypePointer(SimTypeChar(label="Byte"), offset=0), offset=0)}, name="IKEEXT_CERT_EKUS0", pack=False, align=None), offset=0), "name": SimTypePointer(SimStruct({"nameType": SimTypeInt(signed=False, label="IKEEXT_CERT_CRITERIA_NAME_TYPE"), "certName": SimTypePointer(SimTypeChar(label="Char"), offset=0)}, name="IKEEXT_CERT_NAME0", pack=False, align=None), offset=0), "flags": SimTypeInt(signed=False, label="UInt32")}, name="IKEEXT_CERTIFICATE_CRITERIA0", pack=False, align=None), offset=0)}, name="_Anonymous1_e__Struct", pack=False, align=None), "Anonymous2": SimStruct({"inboundEnterpriseStoreArraySize": SimTypeInt(signed=False, label="UInt32"), "inboundEnterpriseStoreCriteria": SimTypePointer(SimStruct({"certData": SimStruct({"size": SimTypeInt(signed=False, label="UInt32"), "data": SimTypePointer(SimTypeChar(label="Byte"), offset=0)}, name="FWP_BYTE_BLOB", pack=False, align=None), "certHash": SimStruct({"size": SimTypeInt(signed=False, label="UInt32"), "data": SimTypePointer(SimTypeChar(label="Byte"), offset=0)}, name="FWP_BYTE_BLOB", pack=False, align=None), "eku": SimTypePointer(SimStruct({"numEku": SimTypeInt(signed=False, label="UInt32"), "eku": SimTypePointer(SimTypePointer(SimTypeChar(label="Byte"), offset=0), offset=0)}, name="IKEEXT_CERT_EKUS0", pack=False, align=None), offset=0), "name": SimTypePointer(SimStruct({"nameType": SimTypeInt(signed=False, label="IKEEXT_CERT_CRITERIA_NAME_TYPE"), "certName": SimTypePointer(SimTypeChar(label="Char"), offset=0)}, name="IKEEXT_CERT_NAME0", pack=False, align=None), offset=0), "flags": SimTypeInt(signed=False, label="UInt32")}, name="IKEEXT_CERTIFICATE_CRITERIA0", pack=False, align=None), offset=0)}, name="_Anonymous2_e__Struct", pack=False, align=None), "Anonymous3": SimStruct({"inboundRootStoreArraySize": SimTypeInt(signed=False, label="UInt32"), "inboundTrustedRootStoreCriteria": SimTypePointer(SimStruct({"certData": SimStruct({"size": SimTypeInt(signed=False, label="UInt32"), "data": SimTypePointer(SimTypeChar(label="Byte"), offset=0)}, name="FWP_BYTE_BLOB", pack=False, align=None), "certHash": SimStruct({"size": SimTypeInt(signed=False, label="UInt32"), "data": SimTypePointer(SimTypeChar(label="Byte"), offset=0)}, name="FWP_BYTE_BLOB", pack=False, align=None), "eku": SimTypePointer(SimStruct({"numEku": SimTypeInt(signed=False, label="UInt32"), "eku": SimTypePointer(SimTypePointer(SimTypeChar(label="Byte"), offset=0), offset=0)}, name="IKEEXT_CERT_EKUS0", pack=False, align=None), offset=0), "name": SimTypePointer(SimStruct({"nameType": SimTypeInt(signed=False, label="IKEEXT_CERT_CRITERIA_NAME_TYPE"), "certName": SimTypePointer(SimTypeChar(label="Char"), offset=0)}, name="IKEEXT_CERT_NAME0", pack=False, align=None), offset=0), "flags": SimTypeInt(signed=False, label="UInt32")}, name="IKEEXT_CERTIFICATE_CRITERIA0", pack=False, align=None), offset=0)}, name="_Anonymous3_e__Struct", pack=False, align=None)}, name="<anon>", label="None"), "outboundConfigType": SimTypeInt(signed=False, label="IKEEXT_CERT_CONFIG_TYPE"), "Anonymous2": SimUnion({"Anonymous1": SimStruct({"outboundRootArraySize": SimTypeInt(signed=False, label="UInt32"), "outboundRootCriteria": SimTypePointer(SimStruct({"certData": SimStruct({"size": SimTypeInt(signed=False, label="UInt32"), "data": SimTypePointer(SimTypeChar(label="Byte"), offset=0)}, name="FWP_BYTE_BLOB", pack=False, align=None), "certHash": SimStruct({"size": SimTypeInt(signed=False, label="UInt32"), "data": SimTypePointer(SimTypeChar(label="Byte"), offset=0)}, name="FWP_BYTE_BLOB", pack=False, align=None), "eku": SimTypePointer(SimStruct({"numEku": SimTypeInt(signed=False, label="UInt32"), "eku": SimTypePointer(SimTypePointer(SimTypeChar(label="Byte"), offset=0), offset=0)}, name="IKEEXT_CERT_EKUS0", pack=False, align=None), offset=0), "name": SimTypePointer(SimStruct({"nameType": SimTypeInt(signed=False, label="IKEEXT_CERT_CRITERIA_NAME_TYPE"), "certName": SimTypePointer(SimTypeChar(label="Char"), offset=0)}, name="IKEEXT_CERT_NAME0", pack=False, align=None), offset=0), "flags": SimTypeInt(signed=False, label="UInt32")}, name="IKEEXT_CERTIFICATE_CRITERIA0", pack=False, align=None), offset=0)}, name="_Anonymous1_e__Struct", pack=False, align=None), "Anonymous2": SimStruct({"outboundEnterpriseStoreArraySize": SimTypeInt(signed=False, label="UInt32"), "outboundEnterpriseStoreCriteria": SimTypePointer(SimStruct({"certData": SimStruct({"size": SimTypeInt(signed=False, label="UInt32"), "data": SimTypePointer(SimTypeChar(label="Byte"), offset=0)}, name="FWP_BYTE_BLOB", pack=False, align=None), "certHash": SimStruct({"size": SimTypeInt(signed=False, label="UInt32"), "data": SimTypePointer(SimTypeChar(label="Byte"), offset=0)}, name="FWP_BYTE_BLOB", pack=False, align=None), "eku": SimTypePointer(SimStruct({"numEku": SimTypeInt(signed=False, label="UInt32"), "eku": SimTypePointer(SimTypePointer(SimTypeChar(label="Byte"), offset=0), offset=0)}, name="IKEEXT_CERT_EKUS0", pack=False, align=None), offset=0), "name": SimTypePointer(SimStruct({"nameType": SimTypeInt(signed=False, label="IKEEXT_CERT_CRITERIA_NAME_TYPE"), "certName": SimTypePointer(SimTypeChar(label="Char"), offset=0)}, name="IKEEXT_CERT_NAME0", pack=False, align=None), offset=0), "flags": SimTypeInt(signed=False, label="UInt32")}, name="IKEEXT_CERTIFICATE_CRITERIA0", pack=False, align=None), offset=0)}, name="_Anonymous2_e__Struct", pack=False, align=None), "Anonymous3": SimStruct({"outboundRootStoreArraySize": SimTypeInt(signed=False, label="UInt32"), "outboundTrustedRootStoreCriteria": SimTypePointer(SimStruct({"certData": SimStruct({"size": SimTypeInt(signed=False, label="UInt32"), "data": SimTypePointer(SimTypeChar(label="Byte"), offset=0)}, name="FWP_BYTE_BLOB", pack=False, align=None), "certHash": SimStruct({"size": SimTypeInt(signed=False, label="UInt32"), "data": SimTypePointer(SimTypeChar(label="Byte"), offset=0)}, name="FWP_BYTE_BLOB", pack=False, align=None), "eku": SimTypePointer(SimStruct({"numEku": SimTypeInt(signed=False, label="UInt32"), "eku": SimTypePointer(SimTypePointer(SimTypeChar(label="Byte"), offset=0), offset=0)}, name="IKEEXT_CERT_EKUS0", pack=False, align=None), offset=0), "name": SimTypePointer(SimStruct({"nameType": SimTypeInt(signed=False, label="IKEEXT_CERT_CRITERIA_NAME_TYPE"), "certName": SimTypePointer(SimTypeChar(label="Char"), offset=0)}, name="IKEEXT_CERT_NAME0", pack=False, align=None), offset=0), "flags": SimTypeInt(signed=False, label="UInt32")}, name="IKEEXT_CERTIFICATE_CRITERIA0", pack=False, align=None), offset=0)}, name="_Anonymous3_e__Struct", pack=False, align=None)}, name="<anon>", label="None"), "flags": SimTypeInt(signed=False, label="IKEEXT_CERT_AUTH"), "localCertLocationUrl": SimStruct({"size": SimTypeInt(signed=False, label="UInt32"), "data": SimTypePointer(SimTypeChar(label="Byte"), offset=0)}, name="FWP_BYTE_BLOB", pack=False, align=None)}, name="IKEEXT_CERTIFICATE_AUTHENTICATION2", pack=False, align=None), "kerberosAuthentication": SimStruct({"flags": SimTypeInt(signed=False, label="IKEEXT_KERBEROS_AUTHENTICATION_FLAGS"), "proxyServer": SimTypePointer(SimTypeChar(label="Char"), offset=0)}, name="IKEEXT_KERBEROS_AUTHENTICATION1", pack=False, align=None), "reservedAuthentication": SimStruct({"flags": SimTypeInt(signed=False, label="IKEEXT_RESERVED_AUTHENTICATION_FLAGS")}, name="IKEEXT_RESERVED_AUTHENTICATION0", pack=False, align=None), "ntlmV2Authentication": SimStruct({"flags": SimTypeInt(signed=False, label="UInt32")}, name="IKEEXT_NTLM_V2_AUTHENTICATION0", pack=False, align=None), "sslAuthentication": SimStruct({"inboundConfigType": SimTypeInt(signed=False, label="IKEEXT_CERT_CONFIG_TYPE"), "Anonymous1": SimUnion({"Anonymous1": SimStruct({"inboundRootArraySize": SimTypeInt(signed=False, label="UInt32"), "inboundRootCriteria": SimTypePointer(SimStruct({"certData": SimStruct({"size": SimTypeInt(signed=False, label="UInt32"), "data": SimTypePointer(SimTypeChar(label="Byte"), offset=0)}, name="FWP_BYTE_BLOB", pack=False, align=None), "certHash": SimStruct({"size": SimTypeInt(signed=False, label="UInt32"), "data": SimTypePointer(SimTypeChar(label="Byte"), offset=0)}, name="FWP_BYTE_BLOB", pack=False, align=None), "eku": SimTypePointer(SimStruct({"numEku": SimTypeInt(signed=False, label="UInt32"), "eku": SimTypePointer(SimTypePointer(SimTypeChar(label="Byte"), offset=0), offset=0)}, name="IKEEXT_CERT_EKUS0", pack=False, align=None), offset=0), "name": SimTypePointer(SimStruct({"nameType": SimTypeInt(signed=False, label="IKEEXT_CERT_CRITERIA_NAME_TYPE"), "certName": SimTypePointer(SimTypeChar(label="Char"), offset=0)}, name="IKEEXT_CERT_NAME0", pack=False, align=None), offset=0), "flags": SimTypeInt(signed=False, label="UInt32")}, name="IKEEXT_CERTIFICATE_CRITERIA0", pack=False, align=None), offset=0)}, name="_Anonymous1_e__Struct", pack=False, align=None), "Anonymous2": SimStruct({"inboundEnterpriseStoreArraySize": SimTypeInt(signed=False, label="UInt32"), "inboundEnterpriseStoreCriteria": SimTypePointer(SimStruct({"certData": SimStruct({"size": SimTypeInt(signed=False, label="UInt32"), "data": SimTypePointer(SimTypeChar(label="Byte"), offset=0)}, name="FWP_BYTE_BLOB", pack=False, align=None), "certHash": SimStruct({"size": SimTypeInt(signed=False, label="UInt32"), "data": SimTypePointer(SimTypeChar(label="Byte"), offset=0)}, name="FWP_BYTE_BLOB", pack=False, align=None), "eku": SimTypePointer(SimStruct({"numEku": SimTypeInt(signed=False, label="UInt32"), "eku": SimTypePointer(SimTypePointer(SimTypeChar(label="Byte"), offset=0), offset=0)}, name="IKEEXT_CERT_EKUS0", pack=False, align=None), offset=0), "name": SimTypePointer(SimStruct({"nameType": SimTypeInt(signed=False, label="IKEEXT_CERT_CRITERIA_NAME_TYPE"), "certName": SimTypePointer(SimTypeChar(label="Char"), offset=0)}, name="IKEEXT_CERT_NAME0", pack=False, align=None), offset=0), "flags": SimTypeInt(signed=False, label="UInt32")}, name="IKEEXT_CERTIFICATE_CRITERIA0", pack=False, align=None), offset=0)}, name="_Anonymous2_e__Struct", pack=False, align=None), "Anonymous3": SimStruct({"inboundRootStoreArraySize": SimTypeInt(signed=False, label="UInt32"), "inboundTrustedRootStoreCriteria": SimTypePointer(SimStruct({"certData": SimStruct({"size": SimTypeInt(signed=False, label="UInt32"), "data": SimTypePointer(SimTypeChar(label="Byte"), offset=0)}, name="FWP_BYTE_BLOB", pack=False, align=None), "certHash": SimStruct({"size": SimTypeInt(signed=False, label="UInt32"), "data": SimTypePointer(SimTypeChar(label="Byte"), offset=0)}, name="FWP_BYTE_BLOB", pack=False, align=None), "eku": SimTypePointer(SimStruct({"numEku": SimTypeInt(signed=False, label="UInt32"), "eku": SimTypePointer(SimTypePointer(SimTypeChar(label="Byte"), offset=0), offset=0)}, name="IKEEXT_CERT_EKUS0", pack=False, align=None), offset=0), "name": SimTypePointer(SimStruct({"nameType": SimTypeInt(signed=False, label="IKEEXT_CERT_CRITERIA_NAME_TYPE"), "certName": SimTypePointer(SimTypeChar(label="Char"), offset=0)}, name="IKEEXT_CERT_NAME0", pack=False, align=None), offset=0), "flags": SimTypeInt(signed=False, label="UInt32")}, name="IKEEXT_CERTIFICATE_CRITERIA0", pack=False, align=None), offset=0)}, name="_Anonymous3_e__Struct", pack=False, align=None)}, name="<anon>", label="None"), "outboundConfigType": SimTypeInt(signed=False, label="IKEEXT_CERT_CONFIG_TYPE"), "Anonymous2": SimUnion({"Anonymous1": SimStruct({"outboundRootArraySize": SimTypeInt(signed=False, label="UInt32"), "outboundRootCriteria": SimTypePointer(SimStruct({"certData": SimStruct({"size": SimTypeInt(signed=False, label="UInt32"), "data": SimTypePointer(SimTypeChar(label="Byte"), offset=0)}, name="FWP_BYTE_BLOB", pack=False, align=None), "certHash": SimStruct({"size": SimTypeInt(signed=False, label="UInt32"), "data": SimTypePointer(SimTypeChar(label="Byte"), offset=0)}, name="FWP_BYTE_BLOB", pack=False, align=None), "eku": SimTypePointer(SimStruct({"numEku": SimTypeInt(signed=False, label="UInt32"), "eku": SimTypePointer(SimTypePointer(SimTypeChar(label="Byte"), offset=0), offset=0)}, name="IKEEXT_CERT_EKUS0", pack=False, align=None), offset=0), "name": SimTypePointer(SimStruct({"nameType": SimTypeInt(signed=False, label="IKEEXT_CERT_CRITERIA_NAME_TYPE"), "certName": SimTypePointer(SimTypeChar(label="Char"), offset=0)}, name="IKEEXT_CERT_NAME0", pack=False, align=None), offset=0), "flags": SimTypeInt(signed=False, label="UInt32")}, name="IKEEXT_CERTIFICATE_CRITERIA0", pack=False, align=None), offset=0)}, name="_Anonymous1_e__Struct", pack=False, align=None), "Anonymous2": SimStruct({"outboundEnterpriseStoreArraySize": SimTypeInt(signed=False, label="UInt32"), "outboundEnterpriseStoreCriteria": SimTypePointer(SimStruct({"certData": SimStruct({"size": SimTypeInt(signed=False, label="UInt32"), "data": SimTypePointer(SimTypeChar(label="Byte"), offset=0)}, name="FWP_BYTE_BLOB", pack=False, align=None), "certHash": SimStruct({"size": SimTypeInt(signed=False, label="UInt32"), "data": SimTypePointer(SimTypeChar(label="Byte"), offset=0)}, name="FWP_BYTE_BLOB", pack=False, align=None), "eku": SimTypePointer(SimStruct({"numEku": SimTypeInt(signed=False, label="UInt32"), "eku": SimTypePointer(SimTypePointer(SimTypeChar(label="Byte"), offset=0), offset=0)}, name="IKEEXT_CERT_EKUS0", pack=False, align=None), offset=0), "name": SimTypePointer(SimStruct({"nameType": SimTypeInt(signed=False, label="IKEEXT_CERT_CRITERIA_NAME_TYPE"), "certName": SimTypePointer(SimTypeChar(label="Char"), offset=0)}, name="IKEEXT_CERT_NAME0", pack=False, align=None), offset=0), "flags": SimTypeInt(signed=False, label="UInt32")}, name="IKEEXT_CERTIFICATE_CRITERIA0", pack=False, align=None), offset=0)}, name="_Anonymous2_e__Struct", pack=False, align=None), "Anonymous3": SimStruct({"outboundRootStoreArraySize": SimTypeInt(signed=False, label="UInt32"), "outboundTrustedRootStoreCriteria": SimTypePointer(SimStruct({"certData": SimStruct({"size": SimTypeInt(signed=False, label="UInt32"), "data": SimTypePointer(SimTypeChar(label="Byte"), offset=0)}, name="FWP_BYTE_BLOB", pack=False, align=None), "certHash": SimStruct({"size": SimTypeInt(signed=False, label="UInt32"), "data": SimTypePointer(SimTypeChar(label="Byte"), offset=0)}, name="FWP_BYTE_BLOB", pack=False, align=None), "eku": SimTypePointer(SimStruct({"numEku": SimTypeInt(signed=False, label="UInt32"), "eku": SimTypePointer(SimTypePointer(SimTypeChar(label="Byte"), offset=0), offset=0)}, name="IKEEXT_CERT_EKUS0", pack=False, align=None), offset=0), "name": SimTypePointer(SimStruct({"nameType": SimTypeInt(signed=False, label="IKEEXT_CERT_CRITERIA_NAME_TYPE"), "certName": SimTypePointer(SimTypeChar(label="Char"), offset=0)}, name="IKEEXT_CERT_NAME0", pack=False, align=None), offset=0), "flags": SimTypeInt(signed=False, label="UInt32")}, name="IKEEXT_CERTIFICATE_CRITERIA0", pack=False, align=None), offset=0)}, name="_Anonymous3_e__Struct", pack=False, align=None)}, name="<anon>", label="None"), "flags": SimTypeInt(signed=False, label="IKEEXT_CERT_AUTH"), "localCertLocationUrl": SimStruct({"size": SimTypeInt(signed=False, label="UInt32"), "data": SimTypePointer(SimTypeChar(label="Byte"), offset=0)}, name="FWP_BYTE_BLOB", pack=False, align=None)}, name="IKEEXT_CERTIFICATE_AUTHENTICATION2", pack=False, align=None), "cgaAuthentication": SimStruct({"keyContainerName": SimTypePointer(SimTypeChar(label="Char"), offset=0), "cspName": SimTypePointer(SimTypeChar(label="Char"), offset=0), "cspType": SimTypeInt(signed=False, label="UInt32"), "cgaModifier": SimStruct({"byteArray16": SimTypeFixedSizeArray(SimTypeChar(label="Byte"), 16)}, name="FWP_BYTE_ARRAY16", pack=False, align=None), "cgaCollisionCount": SimTypeChar(label="Byte")}, name="IKEEXT_IPV6_CGA_AUTHENTICATION0", pack=False, align=None), "eapAuthentication": SimStruct({"flags": SimTypeInt(signed=False, label="IKEEXT_EAP_AUTHENTICATION_FLAGS")}, name="IKEEXT_EAP_AUTHENTICATION0", pack=False, align=None)}, name="<anon>", label="None")}, name="IKEEXT_AUTHENTICATION_METHOD2", pack=False, align=None), offset=0), "initiatorImpersonationType": SimTypeInt(signed=False, label="IKEEXT_AUTHENTICATION_IMPERSONATION_TYPE"), "numIkeProposals": SimTypeInt(signed=False, label="UInt32"), "ikeProposals": SimTypePointer(SimStruct({"cipherAlgorithm": SimStruct({"algoIdentifier": SimTypeInt(signed=False, label="IKEEXT_CIPHER_TYPE"), "keyLen": SimTypeInt(signed=False, label="UInt32"), "rounds": SimTypeInt(signed=False, label="UInt32")}, name="IKEEXT_CIPHER_ALGORITHM0", pack=False, align=None), "integrityAlgorithm": SimStruct({"algoIdentifier": SimTypeInt(signed=False, label="IKEEXT_INTEGRITY_TYPE")}, name="IKEEXT_INTEGRITY_ALGORITHM0", pack=False, align=None), "maxLifetimeSeconds": SimTypeInt(signed=False, label="UInt32"), "dhGroup": SimTypeInt(signed=False, label="IKEEXT_DH_GROUP"), "quickModeLimit": SimTypeInt(signed=False, label="UInt32")}, name="IKEEXT_PROPOSAL0", pack=False, align=None), offset=0), "flags": SimTypeInt(signed=False, label="IKEEXT_POLICY_FLAG"), "maxDynamicFilters": SimTypeInt(signed=False, label="UInt32"), "retransmitDurationSecs": SimTypeInt(signed=False, label="UInt32")}, name="IKEEXT_POLICY2", pack=False, align=None), offset=0), "dataBuffer": SimTypePointer(SimStruct({"size": SimTypeInt(signed=False, label="UInt32"), "data": SimTypePointer(SimTypeChar(label="Byte"), offset=0)}, name="FWP_BYTE_BLOB", pack=False, align=None), offset=0), "classifyOptions": SimTypePointer(SimStruct({"numOptions": SimTypeInt(signed=False, label="UInt32"), "options": SimTypePointer(SimStruct({"type": SimTypeInt(signed=False, label="FWP_CLASSIFY_OPTION_TYPE"), "value": SimStruct({"type": SimTypeInt(signed=False, label="FWP_DATA_TYPE"), "Anonymous": SimUnion({"uint8": SimTypeChar(label="Byte"), "uint16": SimTypeShort(signed=False, label="UInt16"), "uint32": SimTypeInt(signed=False, label="UInt32"), "uint64": SimTypePointer(SimTypeLongLong(signed=False, label="UInt64"), offset=0), "int8": SimTypeChar(label="SByte"), "int16": SimTypeShort(signed=True, label="Int16"), "int32": SimTypeInt(signed=True, label="Int32"), "int64": SimTypePointer(SimTypeLongLong(signed=True, label="Int64"), offset=0), "float32": SimTypeFloat(size=32), "double64": SimTypePointer(SimTypeFloat(size=64), offset=0), "byteArray16": SimTypePointer(SimStruct({"byteArray16": SimTypeFixedSizeArray(SimTypeChar(label="Byte"), 16)}, name="FWP_BYTE_ARRAY16", pack=False, align=None), offset=0), "byteBlob": SimTypePointer(SimStruct({"size": SimTypeInt(signed=False, label="UInt32"), "data": SimTypePointer(SimTypeChar(label="Byte"), offset=0)}, name="FWP_BYTE_BLOB", pack=False, align=None), offset=0), "sid": SimTypePointer(SimTypeBottom(label="SID"), offset=0), "sd": SimTypePointer(SimStruct({"size": SimTypeInt(signed=False, label="UInt32"), "data": SimTypePointer(SimTypeChar(label="Byte"), offset=0)}, name="FWP_BYTE_BLOB", pack=False, align=None), offset=0), "tokenInformation": SimTypePointer(SimStruct({"sidCount": SimTypeInt(signed=False, label="UInt32"), "sids": SimTypePointer(SimTypeBottom(label="SID_AND_ATTRIBUTES"), offset=0), "restrictedSidCount": SimTypeInt(signed=False, label="UInt32"), "restrictedSids": SimTypePointer(SimTypeBottom(label="SID_AND_ATTRIBUTES"), offset=0)}, name="FWP_TOKEN_INFORMATION", pack=False, align=None), offset=0), "tokenAccessInformation": SimTypePointer(SimStruct({"size": SimTypeInt(signed=False, label="UInt32"), "data": SimTypePointer(SimTypeChar(label="Byte"), offset=0)}, name="FWP_BYTE_BLOB", pack=False, align=None), offset=0), "unicodeString": SimTypePointer(SimTypeChar(label="Char"), offset=0), "byteArray6": SimTypePointer(SimStruct({"byteArray6": SimTypeFixedSizeArray(SimTypeChar(label="Byte"), 6)}, name="FWP_BYTE_ARRAY6", pack=False, align=None), offset=0), "bitmapArray64": SimTypePointer(SimStruct({"bitmapArray64": SimTypeFixedSizeArray(SimTypeChar(label="Byte"), 8)}, name="FWP_BITMAP_ARRAY64_", pack=False, align=None), offset=0)}, name="<anon>", label="None")}, name="FWP_VALUE0", pack=False, align=None)}, name="FWPM_CLASSIFY_OPTION0", pack=False, align=None), offset=0)}, name="FWPM_CLASSIFY_OPTIONS0", pack=False, align=None), offset=0), "ikeV2QmTunnelPolicy": SimTypePointer(SimStruct({"flags": SimTypeInt(signed=False, label="UInt32"), "numIpsecProposals": SimTypeInt(signed=False, label="UInt32"), "ipsecProposals": SimTypePointer(SimStruct({"lifetime": SimStruct({"lifetimeSeconds": SimTypeInt(signed=False, label="UInt32"), "lifetimeKilobytes": SimTypeInt(signed=False, label="UInt32"), "lifetimePackets": SimTypeInt(signed=False, label="UInt32")}, name="IPSEC_SA_LIFETIME0", pack=False, align=None), "numSaTransforms": SimTypeInt(signed=False, label="UInt32"), "saTransforms": SimTypePointer(SimStruct({"ipsecTransformType": SimTypeInt(signed=False, label="IPSEC_TRANSFORM_TYPE"), "Anonymous": SimUnion({"ahTransform": SimTypePointer(SimStruct({"authTransformId": SimStruct({"authType": SimTypeInt(signed=False, label="IPSEC_AUTH_TYPE"), "authConfig": SimTypeChar(label="Byte")}, name="IPSEC_AUTH_TRANSFORM_ID0", pack=False, align=None), "cryptoModuleId": SimTypePointer(SimTypeBottom(label="Guid"), offset=0)}, name="IPSEC_AUTH_TRANSFORM0", pack=False, align=None), offset=0), "espAuthTransform": SimTypePointer(SimStruct({"authTransformId": SimStruct({"authType": SimTypeInt(signed=False, label="IPSEC_AUTH_TYPE"), "authConfig": SimTypeChar(label="Byte")}, name="IPSEC_AUTH_TRANSFORM_ID0", pack=False, align=None), "cryptoModuleId": SimTypePointer(SimTypeBottom(label="Guid"), offset=0)}, name="IPSEC_AUTH_TRANSFORM0", pack=False, align=None), offset=0), "espCipherTransform": SimTypePointer(SimStruct({"cipherTransformId": SimStruct({"cipherType": SimTypeInt(signed=False, label="IPSEC_CIPHER_TYPE"), "cipherConfig": SimTypeChar(label="Byte")}, name="IPSEC_CIPHER_TRANSFORM_ID0", pack=False, align=None), "cryptoModuleId": SimTypePointer(SimTypeBottom(label="Guid"), offset=0)}, name="IPSEC_CIPHER_TRANSFORM0", pack=False, align=None), offset=0), "espAuthAndCipherTransform": SimTypePointer(SimStruct({"authTransform": SimStruct({"authTransformId": SimStruct({"authType": SimTypeInt(signed=False, label="IPSEC_AUTH_TYPE"), "authConfig": SimTypeChar(label="Byte")}, name="IPSEC_AUTH_TRANSFORM_ID0", pack=False, align=None), "cryptoModuleId": SimTypePointer(SimTypeBottom(label="Guid"), offset=0)}, name="IPSEC_AUTH_TRANSFORM0", pack=False, align=None), "cipherTransform": SimStruct({"cipherTransformId": SimStruct({"cipherType": SimTypeInt(signed=False, label="IPSEC_CIPHER_TYPE"), "cipherConfig": SimTypeChar(label="Byte")}, name="IPSEC_CIPHER_TRANSFORM_ID0", pack=False, align=None), "cryptoModuleId": SimTypePointer(SimTypeBottom(label="Guid"), offset=0)}, name="IPSEC_CIPHER_TRANSFORM0", pack=False, align=None)}, name="IPSEC_AUTH_AND_CIPHER_TRANSFORM0", pack=False, align=None), offset=0), "espAuthFwTransform": SimTypePointer(SimStruct({"authTransformId": SimStruct({"authType": SimTypeInt(signed=False, label="IPSEC_AUTH_TYPE"), "authConfig": SimTypeChar(label="Byte")}, name="IPSEC_AUTH_TRANSFORM_ID0", pack=False, align=None), "cryptoModuleId": SimTypePointer(SimTypeBottom(label="Guid"), offset=0)}, name="IPSEC_AUTH_TRANSFORM0", pack=False, align=None), offset=0)}, name="<anon>", label="None")}, name="IPSEC_SA_TRANSFORM0", pack=False, align=None), offset=0), "pfsGroup": SimTypeInt(signed=False, label="IPSEC_PFS_GROUP")}, name="IPSEC_PROPOSAL0", pack=False, align=None), offset=0), "tunnelEndpoints": SimStruct({"ipVersion": SimTypeInt(signed=False, label="FWP_IP_VERSION"), "Anonymous1": SimUnion({"localV4Address": SimTypeInt(signed=False, label="UInt32"), "localV6Address": SimTypeFixedSizeArray(SimTypeChar(label="Byte"), 16)}, name="<anon>", label="None"), "Anonymous2": SimUnion({"remoteV4Address": SimTypeInt(signed=False, label="UInt32"), "remoteV6Address": SimTypeFixedSizeArray(SimTypeChar(label="Byte"), 16)}, name="<anon>", label="None"), "localIfLuid": SimTypeLongLong(signed=False, label="UInt64"), "remoteFqdn": SimTypePointer(SimTypeChar(label="Char"), offset=0), "numAddresses": SimTypeInt(signed=False, label="UInt32"), "remoteAddresses": SimTypePointer(SimStruct({"ipVersion": SimTypeInt(signed=False, label="FWP_IP_VERSION"), "Anonymous": SimUnion({"v4Address": SimTypeInt(signed=False, label="UInt32"), "v6Address": SimTypeFixedSizeArray(SimTypeChar(label="Byte"), 16)}, name="<anon>", label="None")}, name="IPSEC_TUNNEL_ENDPOINT0", pack=False, align=None), offset=0)}, name="IPSEC_TUNNEL_ENDPOINTS2", pack=False, align=None), "saIdleTimeout": SimStruct({"idleTimeoutSeconds": SimTypeInt(signed=False, label="UInt32"), "idleTimeoutSecondsFailOver": SimTypeInt(signed=False, label="UInt32")}, name="IPSEC_SA_IDLE_TIMEOUT0", pack=False, align=None), "emPolicy": SimTypePointer(SimStruct({"numAuthenticationMethods": SimTypeInt(signed=False, label="UInt32"), "authenticationMethods": SimTypePointer(SimStruct({"authenticationMethodType": SimTypeInt(signed=False, label="IKEEXT_AUTHENTICATION_METHOD_TYPE"), "Anonymous": SimUnion({"presharedKeyAuthentication": SimStruct({"presharedKey": SimStruct({"size": SimTypeInt(signed=False, label="UInt32"), "data": SimTypePointer(SimTypeChar(label="Byte"), offset=0)}, name="FWP_BYTE_BLOB", pack=False, align=None), "flags": SimTypeInt(signed=False, label="IKEEXT_PRESHARED_KEY_AUTHENTICATION_FLAGS")}, name="IKEEXT_PRESHARED_KEY_AUTHENTICATION1", pack=False, align=None), "certificateAuthentication": SimStruct({"inboundConfigType": SimTypeInt(signed=False, label="IKEEXT_CERT_CONFIG_TYPE"), "Anonymous1": SimUnion({"Anonymous1": SimStruct({"inboundRootArraySize": SimTypeInt(signed=False, label="UInt32"), "inboundRootCriteria": SimTypePointer(SimStruct({"certData": SimStruct({"size": SimTypeInt(signed=False, label="UInt32"), "data": SimTypePointer(SimTypeChar(label="Byte"), offset=0)}, name="FWP_BYTE_BLOB", pack=False, align=None), "certHash": SimStruct({"size": SimTypeInt(signed=False, label="UInt32"), "data": SimTypePointer(SimTypeChar(label="Byte"), offset=0)}, name="FWP_BYTE_BLOB", pack=False, align=None), "eku": SimTypePointer(SimStruct({"numEku": SimTypeInt(signed=False, label="UInt32"), "eku": SimTypePointer(SimTypePointer(SimTypeChar(label="Byte"), offset=0), offset=0)}, name="IKEEXT_CERT_EKUS0", pack=False, align=None), offset=0), "name": SimTypePointer(SimStruct({"nameType": SimTypeInt(signed=False, label="IKEEXT_CERT_CRITERIA_NAME_TYPE"), "certName": SimTypePointer(SimTypeChar(label="Char"), offset=0)}, name="IKEEXT_CERT_NAME0", pack=False, align=None), offset=0), "flags": SimTypeInt(signed=False, label="UInt32")}, name="IKEEXT_CERTIFICATE_CRITERIA0", pack=False, align=None), offset=0)}, name="_Anonymous1_e__Struct", pack=False, align=None), "Anonymous2": SimStruct({"inboundEnterpriseStoreArraySize": SimTypeInt(signed=False, label="UInt32"), "inboundEnterpriseStoreCriteria": SimTypePointer(SimStruct({"certData": SimStruct({"size": SimTypeInt(signed=False, label="UInt32"), "data": SimTypePointer(SimTypeChar(label="Byte"), offset=0)}, name="FWP_BYTE_BLOB", pack=False, align=None), "certHash": SimStruct({"size": SimTypeInt(signed=False, label="UInt32"), "data": SimTypePointer(SimTypeChar(label="Byte"), offset=0)}, name="FWP_BYTE_BLOB", pack=False, align=None), "eku": SimTypePointer(SimStruct({"numEku": SimTypeInt(signed=False, label="UInt32"), "eku": SimTypePointer(SimTypePointer(SimTypeChar(label="Byte"), offset=0), offset=0)}, name="IKEEXT_CERT_EKUS0", pack=False, align=None), offset=0), "name": SimTypePointer(SimStruct({"nameType": SimTypeInt(signed=False, label="IKEEXT_CERT_CRITERIA_NAME_TYPE"), "certName": SimTypePointer(SimTypeChar(label="Char"), offset=0)}, name="IKEEXT_CERT_NAME0", pack=False, align=None), offset=0), "flags": SimTypeInt(signed=False, label="UInt32")}, name="IKEEXT_CERTIFICATE_CRITERIA0", pack=False, align=None), offset=0)}, name="_Anonymous2_e__Struct", pack=False, align=None), "Anonymous3": SimStruct({"inboundRootStoreArraySize": SimTypeInt(signed=False, label="UInt32"), "inboundTrustedRootStoreCriteria": SimTypePointer(SimStruct({"certData": SimStruct({"size": SimTypeInt(signed=False, label="UInt32"), "data": SimTypePointer(SimTypeChar(label="Byte"), offset=0)}, name="FWP_BYTE_BLOB", pack=False, align=None), "certHash": SimStruct({"size": SimTypeInt(signed=False, label="UInt32"), "data": SimTypePointer(SimTypeChar(label="Byte"), offset=0)}, name="FWP_BYTE_BLOB", pack=False, align=None), "eku": SimTypePointer(SimStruct({"numEku": SimTypeInt(signed=False, label="UInt32"), "eku": SimTypePointer(SimTypePointer(SimTypeChar(label="Byte"), offset=0), offset=0)}, name="IKEEXT_CERT_EKUS0", pack=False, align=None), offset=0), "name": SimTypePointer(SimStruct({"nameType": SimTypeInt(signed=False, label="IKEEXT_CERT_CRITERIA_NAME_TYPE"), "certName": SimTypePointer(SimTypeChar(label="Char"), offset=0)}, name="IKEEXT_CERT_NAME0", pack=False, align=None), offset=0), "flags": SimTypeInt(signed=False, label="UInt32")}, name="IKEEXT_CERTIFICATE_CRITERIA0", pack=False, align=None), offset=0)}, name="_Anonymous3_e__Struct", pack=False, align=None)}, name="<anon>", label="None"), "outboundConfigType": SimTypeInt(signed=False, label="IKEEXT_CERT_CONFIG_TYPE"), "Anonymous2": SimUnion({"Anonymous1": SimStruct({"outboundRootArraySize": SimTypeInt(signed=False, label="UInt32"), "outboundRootCriteria": SimTypePointer(SimStruct({"certData": SimStruct({"size": SimTypeInt(signed=False, label="UInt32"), "data": SimTypePointer(SimTypeChar(label="Byte"), offset=0)}, name="FWP_BYTE_BLOB", pack=False, align=None), "certHash": SimStruct({"size": SimTypeInt(signed=False, label="UInt32"), "data": SimTypePointer(SimTypeChar(label="Byte"), offset=0)}, name="FWP_BYTE_BLOB", pack=False, align=None), "eku": SimTypePointer(SimStruct({"numEku": SimTypeInt(signed=False, label="UInt32"), "eku": SimTypePointer(SimTypePointer(SimTypeChar(label="Byte"), offset=0), offset=0)}, name="IKEEXT_CERT_EKUS0", pack=False, align=None), offset=0), "name": SimTypePointer(SimStruct({"nameType": SimTypeInt(signed=False, label="IKEEXT_CERT_CRITERIA_NAME_TYPE"), "certName": SimTypePointer(SimTypeChar(label="Char"), offset=0)}, name="IKEEXT_CERT_NAME0", pack=False, align=None), offset=0), "flags": SimTypeInt(signed=False, label="UInt32")}, name="IKEEXT_CERTIFICATE_CRITERIA0", pack=False, align=None), offset=0)}, name="_Anonymous1_e__Struct", pack=False, align=None), "Anonymous2": SimStruct({"outboundEnterpriseStoreArraySize": SimTypeInt(signed=False, label="UInt32"), "outboundEnterpriseStoreCriteria": SimTypePointer(SimStruct({"certData": SimStruct({"size": SimTypeInt(signed=False, label="UInt32"), "data": SimTypePointer(SimTypeChar(label="Byte"), offset=0)}, name="FWP_BYTE_BLOB", pack=False, align=None), "certHash": SimStruct({"size": SimTypeInt(signed=False, label="UInt32"), "data": SimTypePointer(SimTypeChar(label="Byte"), offset=0)}, name="FWP_BYTE_BLOB", pack=False, align=None), "eku": SimTypePointer(SimStruct({"numEku": SimTypeInt(signed=False, label="UInt32"), "eku": SimTypePointer(SimTypePointer(SimTypeChar(label="Byte"), offset=0), offset=0)}, name="IKEEXT_CERT_EKUS0", pack=False, align=None), offset=0), "name": SimTypePointer(SimStruct({"nameType": SimTypeInt(signed=False, label="IKEEXT_CERT_CRITERIA_NAME_TYPE"), "certName": SimTypePointer(SimTypeChar(label="Char"), offset=0)}, name="IKEEXT_CERT_NAME0", pack=False, align=None), offset=0), "flags": SimTypeInt(signed=False, label="UInt32")}, name="IKEEXT_CERTIFICATE_CRITERIA0", pack=False, align=None), offset=0)}, name="_Anonymous2_e__Struct", pack=False, align=None), "Anonymous3": SimStruct({"outboundRootStoreArraySize": SimTypeInt(signed=False, label="UInt32"), "outboundTrustedRootStoreCriteria": SimTypePointer(SimStruct({"certData": SimStruct({"size": SimTypeInt(signed=False, label="UInt32"), "data": SimTypePointer(SimTypeChar(label="Byte"), offset=0)}, name="FWP_BYTE_BLOB", pack=False, align=None), "certHash": SimStruct({"size": SimTypeInt(signed=False, label="UInt32"), "data": SimTypePointer(SimTypeChar(label="Byte"), offset=0)}, name="FWP_BYTE_BLOB", pack=False, align=None), "eku": SimTypePointer(SimStruct({"numEku": SimTypeInt(signed=False, label="UInt32"), "eku": SimTypePointer(SimTypePointer(SimTypeChar(label="Byte"), offset=0), offset=0)}, name="IKEEXT_CERT_EKUS0", pack=False, align=None), offset=0), "name": SimTypePointer(SimStruct({"nameType": SimTypeInt(signed=False, label="IKEEXT_CERT_CRITERIA_NAME_TYPE"), "certName": SimTypePointer(SimTypeChar(label="Char"), offset=0)}, name="IKEEXT_CERT_NAME0", pack=False, align=None), offset=0), "flags": SimTypeInt(signed=False, label="UInt32")}, name="IKEEXT_CERTIFICATE_CRITERIA0", pack=False, align=None), offset=0)}, name="_Anonymous3_e__Struct", pack=False, align=None)}, name="<anon>", label="None"), "flags": SimTypeInt(signed=False, label="IKEEXT_CERT_AUTH"), "localCertLocationUrl": SimStruct({"size": SimTypeInt(signed=False, label="UInt32"), "data": SimTypePointer(SimTypeChar(label="Byte"), offset=0)}, name="FWP_BYTE_BLOB", pack=False, align=None)}, name="IKEEXT_CERTIFICATE_AUTHENTICATION2", pack=False, align=None), "kerberosAuthentication": SimStruct({"flags": SimTypeInt(signed=False, label="IKEEXT_KERBEROS_AUTHENTICATION_FLAGS"), "proxyServer": SimTypePointer(SimTypeChar(label="Char"), offset=0)}, name="IKEEXT_KERBEROS_AUTHENTICATION1", pack=False, align=None), "reservedAuthentication": SimStruct({"flags": SimTypeInt(signed=False, label="IKEEXT_RESERVED_AUTHENTICATION_FLAGS")}, name="IKEEXT_RESERVED_AUTHENTICATION0", pack=False, align=None), "ntlmV2Authentication": SimStruct({"flags": SimTypeInt(signed=False, label="UInt32")}, name="IKEEXT_NTLM_V2_AUTHENTICATION0", pack=False, align=None), "sslAuthentication": SimStruct({"inboundConfigType": SimTypeInt(signed=False, label="IKEEXT_CERT_CONFIG_TYPE"), "Anonymous1": SimUnion({"Anonymous1": SimStruct({"inboundRootArraySize": SimTypeInt(signed=False, label="UInt32"), "inboundRootCriteria": SimTypePointer(SimStruct({"certData": SimStruct({"size": SimTypeInt(signed=False, label="UInt32"), "data": SimTypePointer(SimTypeChar(label="Byte"), offset=0)}, name="FWP_BYTE_BLOB", pack=False, align=None), "certHash": SimStruct({"size": SimTypeInt(signed=False, label="UInt32"), "data": SimTypePointer(SimTypeChar(label="Byte"), offset=0)}, name="FWP_BYTE_BLOB", pack=False, align=None), "eku": SimTypePointer(SimStruct({"numEku": SimTypeInt(signed=False, label="UInt32"), "eku": SimTypePointer(SimTypePointer(SimTypeChar(label="Byte"), offset=0), offset=0)}, name="IKEEXT_CERT_EKUS0", pack=False, align=None), offset=0), "name": SimTypePointer(SimStruct({"nameType": SimTypeInt(signed=False, label="IKEEXT_CERT_CRITERIA_NAME_TYPE"), "certName": SimTypePointer(SimTypeChar(label="Char"), offset=0)}, name="IKEEXT_CERT_NAME0", pack=False, align=None), offset=0), "flags": SimTypeInt(signed=False, label="UInt32")}, name="IKEEXT_CERTIFICATE_CRITERIA0", pack=False, align=None), offset=0)}, name="_Anonymous1_e__Struct", pack=False, align=None), "Anonymous2": SimStruct({"inboundEnterpriseStoreArraySize": SimTypeInt(signed=False, label="UInt32"), "inboundEnterpriseStoreCriteria": SimTypePointer(SimStruct({"certData": SimStruct({"size": SimTypeInt(signed=False, label="UInt32"), "data": SimTypePointer(SimTypeChar(label="Byte"), offset=0)}, name="FWP_BYTE_BLOB", pack=False, align=None), "certHash": SimStruct({"size": SimTypeInt(signed=False, label="UInt32"), "data": SimTypePointer(SimTypeChar(label="Byte"), offset=0)}, name="FWP_BYTE_BLOB", pack=False, align=None), "eku": SimTypePointer(SimStruct({"numEku": SimTypeInt(signed=False, label="UInt32"), "eku": SimTypePointer(SimTypePointer(SimTypeChar(label="Byte"), offset=0), offset=0)}, name="IKEEXT_CERT_EKUS0", pack=False, align=None), offset=0), "name": SimTypePointer(SimStruct({"nameType": SimTypeInt(signed=False, label="IKEEXT_CERT_CRITERIA_NAME_TYPE"), "certName": SimTypePointer(SimTypeChar(label="Char"), offset=0)}, name="IKEEXT_CERT_NAME0", pack=False, align=None), offset=0), "flags": SimTypeInt(signed=False, label="UInt32")}, name="IKEEXT_CERTIFICATE_CRITERIA0", pack=False, align=None), offset=0)}, name="_Anonymous2_e__Struct", pack=False, align=None), "Anonymous3": SimStruct({"inboundRootStoreArraySize": SimTypeInt(signed=False, label="UInt32"), "inboundTrustedRootStoreCriteria": SimTypePointer(SimStruct({"certData": SimStruct({"size": SimTypeInt(signed=False, label="UInt32"), "data": SimTypePointer(SimTypeChar(label="Byte"), offset=0)}, name="FWP_BYTE_BLOB", pack=False, align=None), "certHash": SimStruct({"size": SimTypeInt(signed=False, label="UInt32"), "data": SimTypePointer(SimTypeChar(label="Byte"), offset=0)}, name="FWP_BYTE_BLOB", pack=False, align=None), "eku": SimTypePointer(SimStruct({"numEku": SimTypeInt(signed=False, label="UInt32"), "eku": SimTypePointer(SimTypePointer(SimTypeChar(label="Byte"), offset=0), offset=0)}, name="IKEEXT_CERT_EKUS0", pack=False, align=None), offset=0), "name": SimTypePointer(SimStruct({"nameType": SimTypeInt(signed=False, label="IKEEXT_CERT_CRITERIA_NAME_TYPE"), "certName": SimTypePointer(SimTypeChar(label="Char"), offset=0)}, name="IKEEXT_CERT_NAME0", pack=False, align=None), offset=0), "flags": SimTypeInt(signed=False, label="UInt32")}, name="IKEEXT_CERTIFICATE_CRITERIA0", pack=False, align=None), offset=0)}, name="_Anonymous3_e__Struct", pack=False, align=None)}, name="<anon>", label="None"), "outboundConfigType": SimTypeInt(signed=False, label="IKEEXT_CERT_CONFIG_TYPE"), "Anonymous2": SimUnion({"Anonymous1": SimStruct({"outboundRootArraySize": SimTypeInt(signed=False, label="UInt32"), "outboundRootCriteria": SimTypePointer(SimStruct({"certData": SimStruct({"size": SimTypeInt(signed=False, label="UInt32"), "data": SimTypePointer(SimTypeChar(label="Byte"), offset=0)}, name="FWP_BYTE_BLOB", pack=False, align=None), "certHash": SimStruct({"size": SimTypeInt(signed=False, label="UInt32"), "data": SimTypePointer(SimTypeChar(label="Byte"), offset=0)}, name="FWP_BYTE_BLOB", pack=False, align=None), "eku": SimTypePointer(SimStruct({"numEku": SimTypeInt(signed=False, label="UInt32"), "eku": SimTypePointer(SimTypePointer(SimTypeChar(label="Byte"), offset=0), offset=0)}, name="IKEEXT_CERT_EKUS0", pack=False, align=None), offset=0), "name": SimTypePointer(SimStruct({"nameType": SimTypeInt(signed=False, label="IKEEXT_CERT_CRITERIA_NAME_TYPE"), "certName": SimTypePointer(SimTypeChar(label="Char"), offset=0)}, name="IKEEXT_CERT_NAME0", pack=False, align=None), offset=0), "flags": SimTypeInt(signed=False, label="UInt32")}, name="IKEEXT_CERTIFICATE_CRITERIA0", pack=False, align=None), offset=0)}, name="_Anonymous1_e__Struct", pack=False, align=None), "Anonymous2": SimStruct({"outboundEnterpriseStoreArraySize": SimTypeInt(signed=False, label="UInt32"), "outboundEnterpriseStoreCriteria": SimTypePointer(SimStruct({"certData": SimStruct({"size": SimTypeInt(signed=False, label="UInt32"), "data": SimTypePointer(SimTypeChar(label="Byte"), offset=0)}, name="FWP_BYTE_BLOB", pack=False, align=None), "certHash": SimStruct({"size": SimTypeInt(signed=False, label="UInt32"), "data": SimTypePointer(SimTypeChar(label="Byte"), offset=0)}, name="FWP_BYTE_BLOB", pack=False, align=None), "eku": SimTypePointer(SimStruct({"numEku": SimTypeInt(signed=False, label="UInt32"), "eku": SimTypePointer(SimTypePointer(SimTypeChar(label="Byte"), offset=0), offset=0)}, name="IKEEXT_CERT_EKUS0", pack=False, align=None), offset=0), "name": SimTypePointer(SimStruct({"nameType": SimTypeInt(signed=False, label="IKEEXT_CERT_CRITERIA_NAME_TYPE"), "certName": SimTypePointer(SimTypeChar(label="Char"), offset=0)}, name="IKEEXT_CERT_NAME0", pack=False, align=None), offset=0), "flags": SimTypeInt(signed=False, label="UInt32")}, name="IKEEXT_CERTIFICATE_CRITERIA0", pack=False, align=None), offset=0)}, name="_Anonymous2_e__Struct", pack=False, align=None), "Anonymous3": SimStruct({"outboundRootStoreArraySize": SimTypeInt(signed=False, label="UInt32"), "outboundTrustedRootStoreCriteria": SimTypePointer(SimStruct({"certData": SimStruct({"size": SimTypeInt(signed=False, label="UInt32"), "data": SimTypePointer(SimTypeChar(label="Byte"), offset=0)}, name="FWP_BYTE_BLOB", pack=False, align=None), "certHash": SimStruct({"size": SimTypeInt(signed=False, label="UInt32"), "data": SimTypePointer(SimTypeChar(label="Byte"), offset=0)}, name="FWP_BYTE_BLOB", pack=False, align=None), "eku": SimTypePointer(SimStruct({"numEku": SimTypeInt(signed=False, label="UInt32"), "eku": SimTypePointer(SimTypePointer(SimTypeChar(label="Byte"), offset=0), offset=0)}, name="IKEEXT_CERT_EKUS0", pack=False, align=None), offset=0), "name": SimTypePointer(SimStruct({"nameType": SimTypeInt(signed=False, label="IKEEXT_CERT_CRITERIA_NAME_TYPE"), "certName": SimTypePointer(SimTypeChar(label="Char"), offset=0)}, name="IKEEXT_CERT_NAME0", pack=False, align=None), offset=0), "flags": SimTypeInt(signed=False, label="UInt32")}, name="IKEEXT_CERTIFICATE_CRITERIA0", pack=False, align=None), offset=0)}, name="_Anonymous3_e__Struct", pack=False, align=None)}, name="<anon>", label="None"), "flags": SimTypeInt(signed=False, label="IKEEXT_CERT_AUTH"), "localCertLocationUrl": SimStruct({"size": SimTypeInt(signed=False, label="UInt32"), "data": SimTypePointer(SimTypeChar(label="Byte"), offset=0)}, name="FWP_BYTE_BLOB", pack=False, align=None)}, name="IKEEXT_CERTIFICATE_AUTHENTICATION2", pack=False, align=None), "cgaAuthentication": SimStruct({"keyContainerName": SimTypePointer(SimTypeChar(label="Char"), offset=0), "cspName": SimTypePointer(SimTypeChar(label="Char"), offset=0), "cspType": SimTypeInt(signed=False, label="UInt32"), "cgaModifier": SimStruct({"byteArray16": SimTypeFixedSizeArray(SimTypeChar(label="Byte"), 16)}, name="FWP_BYTE_ARRAY16", pack=False, align=None), "cgaCollisionCount": SimTypeChar(label="Byte")}, name="IKEEXT_IPV6_CGA_AUTHENTICATION0", pack=False, align=None), "eapAuthentication": SimStruct({"flags": SimTypeInt(signed=False, label="IKEEXT_EAP_AUTHENTICATION_FLAGS")}, name="IKEEXT_EAP_AUTHENTICATION0", pack=False, align=None)}, name="<anon>", label="None")}, name="IKEEXT_AUTHENTICATION_METHOD2", pack=False, align=None), offset=0), "initiatorImpersonationType": SimTypeInt(signed=False, label="IKEEXT_AUTHENTICATION_IMPERSONATION_TYPE")}, name="IKEEXT_EM_POLICY2", pack=False, align=None), offset=0), "fwdPathSaLifetime": SimTypeInt(signed=False, label="UInt32"), "compartmentId": SimTypeInt(signed=False, label="UInt32"), "numTrafficSelectorPolicy": SimTypeInt(signed=False, label="UInt32"), "trafficSelectorPolicies": SimTypePointer(SimStruct({"flags": SimTypeInt(signed=False, label="UInt32"), "numLocalTrafficSelectors": SimTypeInt(signed=False, label="UInt32"), "localTrafficSelectors": SimTypePointer(SimStruct({"protocolId": SimTypeChar(label="Byte"), "portStart": SimTypeShort(signed=False, label="UInt16"), "portEnd": SimTypeShort(signed=False, label="UInt16"), "ipVersion": SimTypeInt(signed=False, label="FWP_IP_VERSION"), "Anonymous1": SimUnion({"startV4Address": SimTypeInt(signed=False, label="UInt32"), "startV6Address": SimTypeFixedSizeArray(SimTypeChar(label="Byte"), 16)}, name="<anon>", label="None"), "Anonymous2": SimUnion({"endV4Address": SimTypeInt(signed=False, label="UInt32"), "endV6Address": SimTypeFixedSizeArray(SimTypeChar(label="Byte"), 16)}, name="<anon>", label="None")}, name="IPSEC_TRAFFIC_SELECTOR0_", pack=False, align=None), offset=0), "numRemoteTrafficSelectors": SimTypeInt(signed=False, label="UInt32"), "remoteTrafficSelectors": SimTypePointer(SimStruct({"protocolId": SimTypeChar(label="Byte"), "portStart": SimTypeShort(signed=False, label="UInt16"), "portEnd": SimTypeShort(signed=False, label="UInt16"), "ipVersion": SimTypeInt(signed=False, label="FWP_IP_VERSION"), "Anonymous1": SimUnion({"startV4Address": SimTypeInt(signed=False, label="UInt32"), "startV6Address": SimTypeFixedSizeArray(SimTypeChar(label="Byte"), 16)}, name="<anon>", label="None"), "Anonymous2": SimUnion({"endV4Address": SimTypeInt(signed=False, label="UInt32"), "endV6Address": SimTypeFixedSizeArray(SimTypeChar(label="Byte"), 16)}, name="<anon>", label="None")}, name="IPSEC_TRAFFIC_SELECTOR0_", pack=False, align=None), offset=0)}, name="IPSEC_TRAFFIC_SELECTOR_POLICY0_", pack=False, align=None), offset=0)}, name="IPSEC_TUNNEL_POLICY3_", pack=False, align=None), offset=0), "ikeV2QmTransportPolicy": SimTypePointer(SimStruct({"numIpsecProposals": SimTypeInt(signed=False, label="UInt32"), "ipsecProposals": SimTypePointer(SimStruct({"lifetime": SimStruct({"lifetimeSeconds": SimTypeInt(signed=False, label="UInt32"), "lifetimeKilobytes": SimTypeInt(signed=False, label="UInt32"), "lifetimePackets": SimTypeInt(signed=False, label="UInt32")}, name="IPSEC_SA_LIFETIME0", pack=False, align=None), "numSaTransforms": SimTypeInt(signed=False, label="UInt32"), "saTransforms": SimTypePointer(SimStruct({"ipsecTransformType": SimTypeInt(signed=False, label="IPSEC_TRANSFORM_TYPE"), "Anonymous": SimUnion({"ahTransform": SimTypePointer(SimStruct({"authTransformId": SimStruct({"authType": SimTypeInt(signed=False, label="IPSEC_AUTH_TYPE"), "authConfig": SimTypeChar(label="Byte")}, name="IPSEC_AUTH_TRANSFORM_ID0", pack=False, align=None), "cryptoModuleId": SimTypePointer(SimTypeBottom(label="Guid"), offset=0)}, name="IPSEC_AUTH_TRANSFORM0", pack=False, align=None), offset=0), "espAuthTransform": SimTypePointer(SimStruct({"authTransformId": SimStruct({"authType": SimTypeInt(signed=False, label="IPSEC_AUTH_TYPE"), "authConfig": SimTypeChar(label="Byte")}, name="IPSEC_AUTH_TRANSFORM_ID0", pack=False, align=None), "cryptoModuleId": SimTypePointer(SimTypeBottom(label="Guid"), offset=0)}, name="IPSEC_AUTH_TRANSFORM0", pack=False, align=None), offset=0), "espCipherTransform": SimTypePointer(SimStruct({"cipherTransformId": SimStruct({"cipherType": SimTypeInt(signed=False, label="IPSEC_CIPHER_TYPE"), "cipherConfig": SimTypeChar(label="Byte")}, name="IPSEC_CIPHER_TRANSFORM_ID0", pack=False, align=None), "cryptoModuleId": SimTypePointer(SimTypeBottom(label="Guid"), offset=0)}, name="IPSEC_CIPHER_TRANSFORM0", pack=False, align=None), offset=0), "espAuthAndCipherTransform": SimTypePointer(SimStruct({"authTransform": SimStruct({"authTransformId": SimStruct({"authType": SimTypeInt(signed=False, label="IPSEC_AUTH_TYPE"), "authConfig": SimTypeChar(label="Byte")}, name="IPSEC_AUTH_TRANSFORM_ID0", pack=False, align=None), "cryptoModuleId": SimTypePointer(SimTypeBottom(label="Guid"), offset=0)}, name="IPSEC_AUTH_TRANSFORM0", pack=False, align=None), "cipherTransform": SimStruct({"cipherTransformId": SimStruct({"cipherType": SimTypeInt(signed=False, label="IPSEC_CIPHER_TYPE"), "cipherConfig": SimTypeChar(label="Byte")}, name="IPSEC_CIPHER_TRANSFORM_ID0", pack=False, align=None), "cryptoModuleId": SimTypePointer(SimTypeBottom(label="Guid"), offset=0)}, name="IPSEC_CIPHER_TRANSFORM0", pack=False, align=None)}, name="IPSEC_AUTH_AND_CIPHER_TRANSFORM0", pack=False, align=None), offset=0), "espAuthFwTransform": SimTypePointer(SimStruct({"authTransformId": SimStruct({"authType": SimTypeInt(signed=False, label="IPSEC_AUTH_TYPE"), "authConfig": SimTypeChar(label="Byte")}, name="IPSEC_AUTH_TRANSFORM_ID0", pack=False, align=None), "cryptoModuleId": SimTypePointer(SimTypeBottom(label="Guid"), offset=0)}, name="IPSEC_AUTH_TRANSFORM0", pack=False, align=None), offset=0)}, name="<anon>", label="None")}, name="IPSEC_SA_TRANSFORM0", pack=False, align=None), offset=0), "pfsGroup": SimTypeInt(signed=False, label="IPSEC_PFS_GROUP")}, name="IPSEC_PROPOSAL0", pack=False, align=None), offset=0), "flags": SimTypeInt(signed=False, label="IPSEC_POLICY_FLAG"), "ndAllowClearTimeoutSeconds": SimTypeInt(signed=False, label="UInt32"), "saIdleTimeout": SimStruct({"idleTimeoutSeconds": SimTypeInt(signed=False, label="UInt32"), "idleTimeoutSecondsFailOver": SimTypeInt(signed=False, label="UInt32")}, name="IPSEC_SA_IDLE_TIMEOUT0", pack=False, align=None), "emPolicy": SimTypePointer(SimStruct({"numAuthenticationMethods": SimTypeInt(signed=False, label="UInt32"), "authenticationMethods": SimTypePointer(SimStruct({"authenticationMethodType": SimTypeInt(signed=False, label="IKEEXT_AUTHENTICATION_METHOD_TYPE"), "Anonymous": SimUnion({"presharedKeyAuthentication": SimStruct({"presharedKey": SimStruct({"size": SimTypeInt(signed=False, label="UInt32"), "data": SimTypePointer(SimTypeChar(label="Byte"), offset=0)}, name="FWP_BYTE_BLOB", pack=False, align=None), "flags": SimTypeInt(signed=False, label="IKEEXT_PRESHARED_KEY_AUTHENTICATION_FLAGS")}, name="IKEEXT_PRESHARED_KEY_AUTHENTICATION1", pack=False, align=None), "certificateAuthentication": SimStruct({"inboundConfigType": SimTypeInt(signed=False, label="IKEEXT_CERT_CONFIG_TYPE"), "Anonymous1": SimUnion({"Anonymous1": SimStruct({"inboundRootArraySize": SimTypeInt(signed=False, label="UInt32"), "inboundRootCriteria": SimTypePointer(SimStruct({"certData": SimStruct({"size": SimTypeInt(signed=False, label="UInt32"), "data": SimTypePointer(SimTypeChar(label="Byte"), offset=0)}, name="FWP_BYTE_BLOB", pack=False, align=None), "certHash": SimStruct({"size": SimTypeInt(signed=False, label="UInt32"), "data": SimTypePointer(SimTypeChar(label="Byte"), offset=0)}, name="FWP_BYTE_BLOB", pack=False, align=None), "eku": SimTypePointer(SimStruct({"numEku": SimTypeInt(signed=False, label="UInt32"), "eku": SimTypePointer(SimTypePointer(SimTypeChar(label="Byte"), offset=0), offset=0)}, name="IKEEXT_CERT_EKUS0", pack=False, align=None), offset=0), "name": SimTypePointer(SimStruct({"nameType": SimTypeInt(signed=False, label="IKEEXT_CERT_CRITERIA_NAME_TYPE"), "certName": SimTypePointer(SimTypeChar(label="Char"), offset=0)}, name="IKEEXT_CERT_NAME0", pack=False, align=None), offset=0), "flags": SimTypeInt(signed=False, label="UInt32")}, name="IKEEXT_CERTIFICATE_CRITERIA0", pack=False, align=None), offset=0)}, name="_Anonymous1_e__Struct", pack=False, align=None), "Anonymous2": SimStruct({"inboundEnterpriseStoreArraySize": SimTypeInt(signed=False, label="UInt32"), "inboundEnterpriseStoreCriteria": SimTypePointer(SimStruct({"certData": SimStruct({"size": SimTypeInt(signed=False, label="UInt32"), "data": SimTypePointer(SimTypeChar(label="Byte"), offset=0)}, name="FWP_BYTE_BLOB", pack=False, align=None), "certHash": SimStruct({"size": SimTypeInt(signed=False, label="UInt32"), "data": SimTypePointer(SimTypeChar(label="Byte"), offset=0)}, name="FWP_BYTE_BLOB", pack=False, align=None), "eku": SimTypePointer(SimStruct({"numEku": SimTypeInt(signed=False, label="UInt32"), "eku": SimTypePointer(SimTypePointer(SimTypeChar(label="Byte"), offset=0), offset=0)}, name="IKEEXT_CERT_EKUS0", pack=False, align=None), offset=0), "name": SimTypePointer(SimStruct({"nameType": SimTypeInt(signed=False, label="IKEEXT_CERT_CRITERIA_NAME_TYPE"), "certName": SimTypePointer(SimTypeChar(label="Char"), offset=0)}, name="IKEEXT_CERT_NAME0", pack=False, align=None), offset=0), "flags": SimTypeInt(signed=False, label="UInt32")}, name="IKEEXT_CERTIFICATE_CRITERIA0", pack=False, align=None), offset=0)}, name="_Anonymous2_e__Struct", pack=False, align=None), "Anonymous3": SimStruct({"inboundRootStoreArraySize": SimTypeInt(signed=False, label="UInt32"), "inboundTrustedRootStoreCriteria": SimTypePointer(SimStruct({"certData": SimStruct({"size": SimTypeInt(signed=False, label="UInt32"), "data": SimTypePointer(SimTypeChar(label="Byte"), offset=0)}, name="FWP_BYTE_BLOB", pack=False, align=None), "certHash": SimStruct({"size": SimTypeInt(signed=False, label="UInt32"), "data": SimTypePointer(SimTypeChar(label="Byte"), offset=0)}, name="FWP_BYTE_BLOB", pack=False, align=None), "eku": SimTypePointer(SimStruct({"numEku": SimTypeInt(signed=False, label="UInt32"), "eku": SimTypePointer(SimTypePointer(SimTypeChar(label="Byte"), offset=0), offset=0)}, name="IKEEXT_CERT_EKUS0", pack=False, align=None), offset=0), "name": SimTypePointer(SimStruct({"nameType": SimTypeInt(signed=False, label="IKEEXT_CERT_CRITERIA_NAME_TYPE"), "certName": SimTypePointer(SimTypeChar(label="Char"), offset=0)}, name="IKEEXT_CERT_NAME0", pack=False, align=None), offset=0), "flags": SimTypeInt(signed=False, label="UInt32")}, name="IKEEXT_CERTIFICATE_CRITERIA0", pack=False, align=None), offset=0)}, name="_Anonymous3_e__Struct", pack=False, align=None)}, name="<anon>", label="None"), "outboundConfigType": SimTypeInt(signed=False, label="IKEEXT_CERT_CONFIG_TYPE"), "Anonymous2": SimUnion({"Anonymous1": SimStruct({"outboundRootArraySize": SimTypeInt(signed=False, label="UInt32"), "outboundRootCriteria": SimTypePointer(SimStruct({"certData": SimStruct({"size": SimTypeInt(signed=False, label="UInt32"), "data": SimTypePointer(SimTypeChar(label="Byte"), offset=0)}, name="FWP_BYTE_BLOB", pack=False, align=None), "certHash": SimStruct({"size": SimTypeInt(signed=False, label="UInt32"), "data": SimTypePointer(SimTypeChar(label="Byte"), offset=0)}, name="FWP_BYTE_BLOB", pack=False, align=None), "eku": SimTypePointer(SimStruct({"numEku": SimTypeInt(signed=False, label="UInt32"), "eku": SimTypePointer(SimTypePointer(SimTypeChar(label="Byte"), offset=0), offset=0)}, name="IKEEXT_CERT_EKUS0", pack=False, align=None), offset=0), "name": SimTypePointer(SimStruct({"nameType": SimTypeInt(signed=False, label="IKEEXT_CERT_CRITERIA_NAME_TYPE"), "certName": SimTypePointer(SimTypeChar(label="Char"), offset=0)}, name="IKEEXT_CERT_NAME0", pack=False, align=None), offset=0), "flags": SimTypeInt(signed=False, label="UInt32")}, name="IKEEXT_CERTIFICATE_CRITERIA0", pack=False, align=None), offset=0)}, name="_Anonymous1_e__Struct", pack=False, align=None), "Anonymous2": SimStruct({"outboundEnterpriseStoreArraySize": SimTypeInt(signed=False, label="UInt32"), "outboundEnterpriseStoreCriteria": SimTypePointer(SimStruct({"certData": SimStruct({"size": SimTypeInt(signed=False, label="UInt32"), "data": SimTypePointer(SimTypeChar(label="Byte"), offset=0)}, name="FWP_BYTE_BLOB", pack=False, align=None), "certHash": SimStruct({"size": SimTypeInt(signed=False, label="UInt32"), "data": SimTypePointer(SimTypeChar(label="Byte"), offset=0)}, name="FWP_BYTE_BLOB", pack=False, align=None), "eku": SimTypePointer(SimStruct({"numEku": SimTypeInt(signed=False, label="UInt32"), "eku": SimTypePointer(SimTypePointer(SimTypeChar(label="Byte"), offset=0), offset=0)}, name="IKEEXT_CERT_EKUS0", pack=False, align=None), offset=0), "name": SimTypePointer(SimStruct({"nameType": SimTypeInt(signed=False, label="IKEEXT_CERT_CRITERIA_NAME_TYPE"), "certName": SimTypePointer(SimTypeChar(label="Char"), offset=0)}, name="IKEEXT_CERT_NAME0", pack=False, align=None), offset=0), "flags": SimTypeInt(signed=False, label="UInt32")}, name="IKEEXT_CERTIFICATE_CRITERIA0", pack=False, align=None), offset=0)}, name="_Anonymous2_e__Struct", pack=False, align=None), "Anonymous3": SimStruct({"outboundRootStoreArraySize": SimTypeInt(signed=False, label="UInt32"), "outboundTrustedRootStoreCriteria": SimTypePointer(SimStruct({"certData": SimStruct({"size": SimTypeInt(signed=False, label="UInt32"), "data": SimTypePointer(SimTypeChar(label="Byte"), offset=0)}, name="FWP_BYTE_BLOB", pack=False, align=None), "certHash": SimStruct({"size": SimTypeInt(signed=False, label="UInt32"), "data": SimTypePointer(SimTypeChar(label="Byte"), offset=0)}, name="FWP_BYTE_BLOB", pack=False, align=None), "eku": SimTypePointer(SimStruct({"numEku": SimTypeInt(signed=False, label="UInt32"), "eku": SimTypePointer(SimTypePointer(SimTypeChar(label="Byte"), offset=0), offset=0)}, name="IKEEXT_CERT_EKUS0", pack=False, align=None), offset=0), "name": SimTypePointer(SimStruct({"nameType": SimTypeInt(signed=False, label="IKEEXT_CERT_CRITERIA_NAME_TYPE"), "certName": SimTypePointer(SimTypeChar(label="Char"), offset=0)}, name="IKEEXT_CERT_NAME0", pack=False, align=None), offset=0), "flags": SimTypeInt(signed=False, label="UInt32")}, name="IKEEXT_CERTIFICATE_CRITERIA0", pack=False, align=None), offset=0)}, name="_Anonymous3_e__Struct", pack=False, align=None)}, name="<anon>", label="None"), "flags": SimTypeInt(signed=False, label="IKEEXT_CERT_AUTH"), "localCertLocationUrl": SimStruct({"size": SimTypeInt(signed=False, label="UInt32"), "data": SimTypePointer(SimTypeChar(label="Byte"), offset=0)}, name="FWP_BYTE_BLOB", pack=False, align=None)}, name="IKEEXT_CERTIFICATE_AUTHENTICATION2", pack=False, align=None), "kerberosAuthentication": SimStruct({"flags": SimTypeInt(signed=False, label="IKEEXT_KERBEROS_AUTHENTICATION_FLAGS"), "proxyServer": SimTypePointer(SimTypeChar(label="Char"), offset=0)}, name="IKEEXT_KERBEROS_AUTHENTICATION1", pack=False, align=None), "reservedAuthentication": SimStruct({"flags": SimTypeInt(signed=False, label="IKEEXT_RESERVED_AUTHENTICATION_FLAGS")}, name="IKEEXT_RESERVED_AUTHENTICATION0", pack=False, align=None), "ntlmV2Authentication": SimStruct({"flags": SimTypeInt(signed=False, label="UInt32")}, name="IKEEXT_NTLM_V2_AUTHENTICATION0", pack=False, align=None), "sslAuthentication": SimStruct({"inboundConfigType": SimTypeInt(signed=False, label="IKEEXT_CERT_CONFIG_TYPE"), "Anonymous1": SimUnion({"Anonymous1": SimStruct({"inboundRootArraySize": SimTypeInt(signed=False, label="UInt32"), "inboundRootCriteria": SimTypePointer(SimStruct({"certData": SimStruct({"size": SimTypeInt(signed=False, label="UInt32"), "data": SimTypePointer(SimTypeChar(label="Byte"), offset=0)}, name="FWP_BYTE_BLOB", pack=False, align=None), "certHash": SimStruct({"size": SimTypeInt(signed=False, label="UInt32"), "data": SimTypePointer(SimTypeChar(label="Byte"), offset=0)}, name="FWP_BYTE_BLOB", pack=False, align=None), "eku": SimTypePointer(SimStruct({"numEku": SimTypeInt(signed=False, label="UInt32"), "eku": SimTypePointer(SimTypePointer(SimTypeChar(label="Byte"), offset=0), offset=0)}, name="IKEEXT_CERT_EKUS0", pack=False, align=None), offset=0), "name": SimTypePointer(SimStruct({"nameType": SimTypeInt(signed=False, label="IKEEXT_CERT_CRITERIA_NAME_TYPE"), "certName": SimTypePointer(SimTypeChar(label="Char"), offset=0)}, name="IKEEXT_CERT_NAME0", pack=False, align=None), offset=0), "flags": SimTypeInt(signed=False, label="UInt32")}, name="IKEEXT_CERTIFICATE_CRITERIA0", pack=False, align=None), offset=0)}, name="_Anonymous1_e__Struct", pack=False, align=None), "Anonymous2": SimStruct({"inboundEnterpriseStoreArraySize": SimTypeInt(signed=False, label="UInt32"), "inboundEnterpriseStoreCriteria": SimTypePointer(SimStruct({"certData": SimStruct({"size": SimTypeInt(signed=False, label="UInt32"), "data": SimTypePointer(SimTypeChar(label="Byte"), offset=0)}, name="FWP_BYTE_BLOB", pack=False, align=None), "certHash": SimStruct({"size": SimTypeInt(signed=False, label="UInt32"), "data": SimTypePointer(SimTypeChar(label="Byte"), offset=0)}, name="FWP_BYTE_BLOB", pack=False, align=None), "eku": SimTypePointer(SimStruct({"numEku": SimTypeInt(signed=False, label="UInt32"), "eku": SimTypePointer(SimTypePointer(SimTypeChar(label="Byte"), offset=0), offset=0)}, name="IKEEXT_CERT_EKUS0", pack=False, align=None), offset=0), "name": SimTypePointer(SimStruct({"nameType": SimTypeInt(signed=False, label="IKEEXT_CERT_CRITERIA_NAME_TYPE"), "certName": SimTypePointer(SimTypeChar(label="Char"), offset=0)}, name="IKEEXT_CERT_NAME0", pack=False, align=None), offset=0), "flags": SimTypeInt(signed=False, label="UInt32")}, name="IKEEXT_CERTIFICATE_CRITERIA0", pack=False, align=None), offset=0)}, name="_Anonymous2_e__Struct", pack=False, align=None), "Anonymous3": SimStruct({"inboundRootStoreArraySize": SimTypeInt(signed=False, label="UInt32"), "inboundTrustedRootStoreCriteria": SimTypePointer(SimStruct({"certData": SimStruct({"size": SimTypeInt(signed=False, label="UInt32"), "data": SimTypePointer(SimTypeChar(label="Byte"), offset=0)}, name="FWP_BYTE_BLOB", pack=False, align=None), "certHash": SimStruct({"size": SimTypeInt(signed=False, label="UInt32"), "data": SimTypePointer(SimTypeChar(label="Byte"), offset=0)}, name="FWP_BYTE_BLOB", pack=False, align=None), "eku": SimTypePointer(SimStruct({"numEku": SimTypeInt(signed=False, label="UInt32"), "eku": SimTypePointer(SimTypePointer(SimTypeChar(label="Byte"), offset=0), offset=0)}, name="IKEEXT_CERT_EKUS0", pack=False, align=None), offset=0), "name": SimTypePointer(SimStruct({"nameType": SimTypeInt(signed=False, label="IKEEXT_CERT_CRITERIA_NAME_TYPE"), "certName": SimTypePointer(SimTypeChar(label="Char"), offset=0)}, name="IKEEXT_CERT_NAME0", pack=False, align=None), offset=0), "flags": SimTypeInt(signed=False, label="UInt32")}, name="IKEEXT_CERTIFICATE_CRITERIA0", pack=False, align=None), offset=0)}, name="_Anonymous3_e__Struct", pack=False, align=None)}, name="<anon>", label="None"), "outboundConfigType": SimTypeInt(signed=False, label="IKEEXT_CERT_CONFIG_TYPE"), "Anonymous2": SimUnion({"Anonymous1": SimStruct({"outboundRootArraySize": SimTypeInt(signed=False, label="UInt32"), "outboundRootCriteria": SimTypePointer(SimStruct({"certData": SimStruct({"size": SimTypeInt(signed=False, label="UInt32"), "data": SimTypePointer(SimTypeChar(label="Byte"), offset=0)}, name="FWP_BYTE_BLOB", pack=False, align=None), "certHash": SimStruct({"size": SimTypeInt(signed=False, label="UInt32"), "data": SimTypePointer(SimTypeChar(label="Byte"), offset=0)}, name="FWP_BYTE_BLOB", pack=False, align=None), "eku": SimTypePointer(SimStruct({"numEku": SimTypeInt(signed=False, label="UInt32"), "eku": SimTypePointer(SimTypePointer(SimTypeChar(label="Byte"), offset=0), offset=0)}, name="IKEEXT_CERT_EKUS0", pack=False, align=None), offset=0), "name": SimTypePointer(SimStruct({"nameType": SimTypeInt(signed=False, label="IKEEXT_CERT_CRITERIA_NAME_TYPE"), "certName": SimTypePointer(SimTypeChar(label="Char"), offset=0)}, name="IKEEXT_CERT_NAME0", pack=False, align=None), offset=0), "flags": SimTypeInt(signed=False, label="UInt32")}, name="IKEEXT_CERTIFICATE_CRITERIA0", pack=False, align=None), offset=0)}, name="_Anonymous1_e__Struct", pack=False, align=None), "Anonymous2": SimStruct({"outboundEnterpriseStoreArraySize": SimTypeInt(signed=False, label="UInt32"), "outboundEnterpriseStoreCriteria": SimTypePointer(SimStruct({"certData": SimStruct({"size": SimTypeInt(signed=False, label="UInt32"), "data": SimTypePointer(SimTypeChar(label="Byte"), offset=0)}, name="FWP_BYTE_BLOB", pack=False, align=None), "certHash": SimStruct({"size": SimTypeInt(signed=False, label="UInt32"), "data": SimTypePointer(SimTypeChar(label="Byte"), offset=0)}, name="FWP_BYTE_BLOB", pack=False, align=None), "eku": SimTypePointer(SimStruct({"numEku": SimTypeInt(signed=False, label="UInt32"), "eku": SimTypePointer(SimTypePointer(SimTypeChar(label="Byte"), offset=0), offset=0)}, name="IKEEXT_CERT_EKUS0", pack=False, align=None), offset=0), "name": SimTypePointer(SimStruct({"nameType": SimTypeInt(signed=False, label="IKEEXT_CERT_CRITERIA_NAME_TYPE"), "certName": SimTypePointer(SimTypeChar(label="Char"), offset=0)}, name="IKEEXT_CERT_NAME0", pack=False, align=None), offset=0), "flags": SimTypeInt(signed=False, label="UInt32")}, name="IKEEXT_CERTIFICATE_CRITERIA0", pack=False, align=None), offset=0)}, name="_Anonymous2_e__Struct", pack=False, align=None), "Anonymous3": SimStruct({"outboundRootStoreArraySize": SimTypeInt(signed=False, label="UInt32"), "outboundTrustedRootStoreCriteria": SimTypePointer(SimStruct({"certData": SimStruct({"size": SimTypeInt(signed=False, label="UInt32"), "data": SimTypePointer(SimTypeChar(label="Byte"), offset=0)}, name="FWP_BYTE_BLOB", pack=False, align=None), "certHash": SimStruct({"size": SimTypeInt(signed=False, label="UInt32"), "data": SimTypePointer(SimTypeChar(label="Byte"), offset=0)}, name="FWP_BYTE_BLOB", pack=False, align=None), "eku": SimTypePointer(SimStruct({"numEku": SimTypeInt(signed=False, label="UInt32"), "eku": SimTypePointer(SimTypePointer(SimTypeChar(label="Byte"), offset=0), offset=0)}, name="IKEEXT_CERT_EKUS0", pack=False, align=None), offset=0), "name": SimTypePointer(SimStruct({"nameType": SimTypeInt(signed=False, label="IKEEXT_CERT_CRITERIA_NAME_TYPE"), "certName": SimTypePointer(SimTypeChar(label="Char"), offset=0)}, name="IKEEXT_CERT_NAME0", pack=False, align=None), offset=0), "flags": SimTypeInt(signed=False, label="UInt32")}, name="IKEEXT_CERTIFICATE_CRITERIA0", pack=False, align=None), offset=0)}, name="_Anonymous3_e__Struct", pack=False, align=None)}, name="<anon>", label="None"), "flags": SimTypeInt(signed=False, label="IKEEXT_CERT_AUTH"), "localCertLocationUrl": SimStruct({"size": SimTypeInt(signed=False, label="UInt32"), "data": SimTypePointer(SimTypeChar(label="Byte"), offset=0)}, name="FWP_BYTE_BLOB", pack=False, align=None)}, name="IKEEXT_CERTIFICATE_AUTHENTICATION2", pack=False, align=None), "cgaAuthentication": SimStruct({"keyContainerName": SimTypePointer(SimTypeChar(label="Char"), offset=0), "cspName": SimTypePointer(SimTypeChar(label="Char"), offset=0), "cspType": SimTypeInt(signed=False, label="UInt32"), "cgaModifier": SimStruct({"byteArray16": SimTypeFixedSizeArray(SimTypeChar(label="Byte"), 16)}, name="FWP_BYTE_ARRAY16", pack=False, align=None), "cgaCollisionCount": SimTypeChar(label="Byte")}, name="IKEEXT_IPV6_CGA_AUTHENTICATION0", pack=False, align=None), "eapAuthentication": SimStruct({"flags": SimTypeInt(signed=False, label="IKEEXT_EAP_AUTHENTICATION_FLAGS")}, name="IKEEXT_EAP_AUTHENTICATION0", pack=False, align=None)}, name="<anon>", label="None")}, name="IKEEXT_AUTHENTICATION_METHOD2", pack=False, align=None), offset=0), "initiatorImpersonationType": SimTypeInt(signed=False, label="IKEEXT_AUTHENTICATION_IMPERSONATION_TYPE")}, name="IKEEXT_EM_POLICY2", pack=False, align=None), offset=0)}, name="IPSEC_TRANSPORT_POLICY2", pack=False, align=None), offset=0), "ikeV2MmPolicy": SimTypePointer(SimStruct({"softExpirationTime": SimTypeInt(signed=False, label="UInt32"), "numAuthenticationMethods": SimTypeInt(signed=False, label="UInt32"), "authenticationMethods": SimTypePointer(SimStruct({"authenticationMethodType": SimTypeInt(signed=False, label="IKEEXT_AUTHENTICATION_METHOD_TYPE"), "Anonymous": SimUnion({"presharedKeyAuthentication": SimStruct({"presharedKey": SimStruct({"size": SimTypeInt(signed=False, label="UInt32"), "data": SimTypePointer(SimTypeChar(label="Byte"), offset=0)}, name="FWP_BYTE_BLOB", pack=False, align=None), "flags": SimTypeInt(signed=False, label="IKEEXT_PRESHARED_KEY_AUTHENTICATION_FLAGS")}, name="IKEEXT_PRESHARED_KEY_AUTHENTICATION1", pack=False, align=None), "certificateAuthentication": SimStruct({"inboundConfigType": SimTypeInt(signed=False, label="IKEEXT_CERT_CONFIG_TYPE"), "Anonymous1": SimUnion({"Anonymous1": SimStruct({"inboundRootArraySize": SimTypeInt(signed=False, label="UInt32"), "inboundRootCriteria": SimTypePointer(SimStruct({"certData": SimStruct({"size": SimTypeInt(signed=False, label="UInt32"), "data": SimTypePointer(SimTypeChar(label="Byte"), offset=0)}, name="FWP_BYTE_BLOB", pack=False, align=None), "certHash": SimStruct({"size": SimTypeInt(signed=False, label="UInt32"), "data": SimTypePointer(SimTypeChar(label="Byte"), offset=0)}, name="FWP_BYTE_BLOB", pack=False, align=None), "eku": SimTypePointer(SimStruct({"numEku": SimTypeInt(signed=False, label="UInt32"), "eku": SimTypePointer(SimTypePointer(SimTypeChar(label="Byte"), offset=0), offset=0)}, name="IKEEXT_CERT_EKUS0", pack=False, align=None), offset=0), "name": SimTypePointer(SimStruct({"nameType": SimTypeInt(signed=False, label="IKEEXT_CERT_CRITERIA_NAME_TYPE"), "certName": SimTypePointer(SimTypeChar(label="Char"), offset=0)}, name="IKEEXT_CERT_NAME0", pack=False, align=None), offset=0), "flags": SimTypeInt(signed=False, label="UInt32")}, name="IKEEXT_CERTIFICATE_CRITERIA0", pack=False, align=None), offset=0)}, name="_Anonymous1_e__Struct", pack=False, align=None), "Anonymous2": SimStruct({"inboundEnterpriseStoreArraySize": SimTypeInt(signed=False, label="UInt32"), "inboundEnterpriseStoreCriteria": SimTypePointer(SimStruct({"certData": SimStruct({"size": SimTypeInt(signed=False, label="UInt32"), "data": SimTypePointer(SimTypeChar(label="Byte"), offset=0)}, name="FWP_BYTE_BLOB", pack=False, align=None), "certHash": SimStruct({"size": SimTypeInt(signed=False, label="UInt32"), "data": SimTypePointer(SimTypeChar(label="Byte"), offset=0)}, name="FWP_BYTE_BLOB", pack=False, align=None), "eku": SimTypePointer(SimStruct({"numEku": SimTypeInt(signed=False, label="UInt32"), "eku": SimTypePointer(SimTypePointer(SimTypeChar(label="Byte"), offset=0), offset=0)}, name="IKEEXT_CERT_EKUS0", pack=False, align=None), offset=0), "name": SimTypePointer(SimStruct({"nameType": SimTypeInt(signed=False, label="IKEEXT_CERT_CRITERIA_NAME_TYPE"), "certName": SimTypePointer(SimTypeChar(label="Char"), offset=0)}, name="IKEEXT_CERT_NAME0", pack=False, align=None), offset=0), "flags": SimTypeInt(signed=False, label="UInt32")}, name="IKEEXT_CERTIFICATE_CRITERIA0", pack=False, align=None), offset=0)}, name="_Anonymous2_e__Struct", pack=False, align=None), "Anonymous3": SimStruct({"inboundRootStoreArraySize": SimTypeInt(signed=False, label="UInt32"), "inboundTrustedRootStoreCriteria": SimTypePointer(SimStruct({"certData": SimStruct({"size": SimTypeInt(signed=False, label="UInt32"), "data": SimTypePointer(SimTypeChar(label="Byte"), offset=0)}, name="FWP_BYTE_BLOB", pack=False, align=None), "certHash": SimStruct({"size": SimTypeInt(signed=False, label="UInt32"), "data": SimTypePointer(SimTypeChar(label="Byte"), offset=0)}, name="FWP_BYTE_BLOB", pack=False, align=None), "eku": SimTypePointer(SimStruct({"numEku": SimTypeInt(signed=False, label="UInt32"), "eku": SimTypePointer(SimTypePointer(SimTypeChar(label="Byte"), offset=0), offset=0)}, name="IKEEXT_CERT_EKUS0", pack=False, align=None), offset=0), "name": SimTypePointer(SimStruct({"nameType": SimTypeInt(signed=False, label="IKEEXT_CERT_CRITERIA_NAME_TYPE"), "certName": SimTypePointer(SimTypeChar(label="Char"), offset=0)}, name="IKEEXT_CERT_NAME0", pack=False, align=None), offset=0), "flags": SimTypeInt(signed=False, label="UInt32")}, name="IKEEXT_CERTIFICATE_CRITERIA0", pack=False, align=None), offset=0)}, name="_Anonymous3_e__Struct", pack=False, align=None)}, name="<anon>", label="None"), "outboundConfigType": SimTypeInt(signed=False, label="IKEEXT_CERT_CONFIG_TYPE"), "Anonymous2": SimUnion({"Anonymous1": SimStruct({"outboundRootArraySize": SimTypeInt(signed=False, label="UInt32"), "outboundRootCriteria": SimTypePointer(SimStruct({"certData": SimStruct({"size": SimTypeInt(signed=False, label="UInt32"), "data": SimTypePointer(SimTypeChar(label="Byte"), offset=0)}, name="FWP_BYTE_BLOB", pack=False, align=None), "certHash": SimStruct({"size": SimTypeInt(signed=False, label="UInt32"), "data": SimTypePointer(SimTypeChar(label="Byte"), offset=0)}, name="FWP_BYTE_BLOB", pack=False, align=None), "eku": SimTypePointer(SimStruct({"numEku": SimTypeInt(signed=False, label="UInt32"), "eku": SimTypePointer(SimTypePointer(SimTypeChar(label="Byte"), offset=0), offset=0)}, name="IKEEXT_CERT_EKUS0", pack=False, align=None), offset=0), "name": SimTypePointer(SimStruct({"nameType": SimTypeInt(signed=False, label="IKEEXT_CERT_CRITERIA_NAME_TYPE"), "certName": SimTypePointer(SimTypeChar(label="Char"), offset=0)}, name="IKEEXT_CERT_NAME0", pack=False, align=None), offset=0), "flags": SimTypeInt(signed=False, label="UInt32")}, name="IKEEXT_CERTIFICATE_CRITERIA0", pack=False, align=None), offset=0)}, name="_Anonymous1_e__Struct", pack=False, align=None), "Anonymous2": SimStruct({"outboundEnterpriseStoreArraySize": SimTypeInt(signed=False, label="UInt32"), "outboundEnterpriseStoreCriteria": SimTypePointer(SimStruct({"certData": SimStruct({"size": SimTypeInt(signed=False, label="UInt32"), "data": SimTypePointer(SimTypeChar(label="Byte"), offset=0)}, name="FWP_BYTE_BLOB", pack=False, align=None), "certHash": SimStruct({"size": SimTypeInt(signed=False, label="UInt32"), "data": SimTypePointer(SimTypeChar(label="Byte"), offset=0)}, name="FWP_BYTE_BLOB", pack=False, align=None), "eku": SimTypePointer(SimStruct({"numEku": SimTypeInt(signed=False, label="UInt32"), "eku": SimTypePointer(SimTypePointer(SimTypeChar(label="Byte"), offset=0), offset=0)}, name="IKEEXT_CERT_EKUS0", pack=False, align=None), offset=0), "name": SimTypePointer(SimStruct({"nameType": SimTypeInt(signed=False, label="IKEEXT_CERT_CRITERIA_NAME_TYPE"), "certName": SimTypePointer(SimTypeChar(label="Char"), offset=0)}, name="IKEEXT_CERT_NAME0", pack=False, align=None), offset=0), "flags": SimTypeInt(signed=False, label="UInt32")}, name="IKEEXT_CERTIFICATE_CRITERIA0", pack=False, align=None), offset=0)}, name="_Anonymous2_e__Struct", pack=False, align=None), "Anonymous3": SimStruct({"outboundRootStoreArraySize": SimTypeInt(signed=False, label="UInt32"), "outboundTrustedRootStoreCriteria": SimTypePointer(SimStruct({"certData": SimStruct({"size": SimTypeInt(signed=False, label="UInt32"), "data": SimTypePointer(SimTypeChar(label="Byte"), offset=0)}, name="FWP_BYTE_BLOB", pack=False, align=None), "certHash": SimStruct({"size": SimTypeInt(signed=False, label="UInt32"), "data": SimTypePointer(SimTypeChar(label="Byte"), offset=0)}, name="FWP_BYTE_BLOB", pack=False, align=None), "eku": SimTypePointer(SimStruct({"numEku": SimTypeInt(signed=False, label="UInt32"), "eku": SimTypePointer(SimTypePointer(SimTypeChar(label="Byte"), offset=0), offset=0)}, name="IKEEXT_CERT_EKUS0", pack=False, align=None), offset=0), "name": SimTypePointer(SimStruct({"nameType": SimTypeInt(signed=False, label="IKEEXT_CERT_CRITERIA_NAME_TYPE"), "certName": SimTypePointer(SimTypeChar(label="Char"), offset=0)}, name="IKEEXT_CERT_NAME0", pack=False, align=None), offset=0), "flags": SimTypeInt(signed=False, label="UInt32")}, name="IKEEXT_CERTIFICATE_CRITERIA0", pack=False, align=None), offset=0)}, name="_Anonymous3_e__Struct", pack=False, align=None)}, name="<anon>", label="None"), "flags": SimTypeInt(signed=False, label="IKEEXT_CERT_AUTH"), "localCertLocationUrl": SimStruct({"size": SimTypeInt(signed=False, label="UInt32"), "data": SimTypePointer(SimTypeChar(label="Byte"), offset=0)}, name="FWP_BYTE_BLOB", pack=False, align=None)}, name="IKEEXT_CERTIFICATE_AUTHENTICATION2", pack=False, align=None), "kerberosAuthentication": SimStruct({"flags": SimTypeInt(signed=False, label="IKEEXT_KERBEROS_AUTHENTICATION_FLAGS"), "proxyServer": SimTypePointer(SimTypeChar(label="Char"), offset=0)}, name="IKEEXT_KERBEROS_AUTHENTICATION1", pack=False, align=None), "reservedAuthentication": SimStruct({"flags": SimTypeInt(signed=False, label="IKEEXT_RESERVED_AUTHENTICATION_FLAGS")}, name="IKEEXT_RESERVED_AUTHENTICATION0", pack=False, align=None), "ntlmV2Authentication": SimStruct({"flags": SimTypeInt(signed=False, label="UInt32")}, name="IKEEXT_NTLM_V2_AUTHENTICATION0", pack=False, align=None), "sslAuthentication": SimStruct({"inboundConfigType": SimTypeInt(signed=False, label="IKEEXT_CERT_CONFIG_TYPE"), "Anonymous1": SimUnion({"Anonymous1": SimStruct({"inboundRootArraySize": SimTypeInt(signed=False, label="UInt32"), "inboundRootCriteria": SimTypePointer(SimStruct({"certData": SimStruct({"size": SimTypeInt(signed=False, label="UInt32"), "data": SimTypePointer(SimTypeChar(label="Byte"), offset=0)}, name="FWP_BYTE_BLOB", pack=False, align=None), "certHash": SimStruct({"size": SimTypeInt(signed=False, label="UInt32"), "data": SimTypePointer(SimTypeChar(label="Byte"), offset=0)}, name="FWP_BYTE_BLOB", pack=False, align=None), "eku": SimTypePointer(SimStruct({"numEku": SimTypeInt(signed=False, label="UInt32"), "eku": SimTypePointer(SimTypePointer(SimTypeChar(label="Byte"), offset=0), offset=0)}, name="IKEEXT_CERT_EKUS0", pack=False, align=None), offset=0), "name": SimTypePointer(SimStruct({"nameType": SimTypeInt(signed=False, label="IKEEXT_CERT_CRITERIA_NAME_TYPE"), "certName": SimTypePointer(SimTypeChar(label="Char"), offset=0)}, name="IKEEXT_CERT_NAME0", pack=False, align=None), offset=0), "flags": SimTypeInt(signed=False, label="UInt32")}, name="IKEEXT_CERTIFICATE_CRITERIA0", pack=False, align=None), offset=0)}, name="_Anonymous1_e__Struct", pack=False, align=None), "Anonymous2": SimStruct({"inboundEnterpriseStoreArraySize": SimTypeInt(signed=False, label="UInt32"), "inboundEnterpriseStoreCriteria": SimTypePointer(SimStruct({"certData": SimStruct({"size": SimTypeInt(signed=False, label="UInt32"), "data": SimTypePointer(SimTypeChar(label="Byte"), offset=0)}, name="FWP_BYTE_BLOB", pack=False, align=None), "certHash": SimStruct({"size": SimTypeInt(signed=False, label="UInt32"), "data": SimTypePointer(SimTypeChar(label="Byte"), offset=0)}, name="FWP_BYTE_BLOB", pack=False, align=None), "eku": SimTypePointer(SimStruct({"numEku": SimTypeInt(signed=False, label="UInt32"), "eku": SimTypePointer(SimTypePointer(SimTypeChar(label="Byte"), offset=0), offset=0)}, name="IKEEXT_CERT_EKUS0", pack=False, align=None), offset=0), "name": SimTypePointer(SimStruct({"nameType": SimTypeInt(signed=False, label="IKEEXT_CERT_CRITERIA_NAME_TYPE"), "certName": SimTypePointer(SimTypeChar(label="Char"), offset=0)}, name="IKEEXT_CERT_NAME0", pack=False, align=None), offset=0), "flags": SimTypeInt(signed=False, label="UInt32")}, name="IKEEXT_CERTIFICATE_CRITERIA0", pack=False, align=None), offset=0)}, name="_Anonymous2_e__Struct", pack=False, align=None), "Anonymous3": SimStruct({"inboundRootStoreArraySize": SimTypeInt(signed=False, label="UInt32"), "inboundTrustedRootStoreCriteria": SimTypePointer(SimStruct({"certData": SimStruct({"size": SimTypeInt(signed=False, label="UInt32"), "data": SimTypePointer(SimTypeChar(label="Byte"), offset=0)}, name="FWP_BYTE_BLOB", pack=False, align=None), "certHash": SimStruct({"size": SimTypeInt(signed=False, label="UInt32"), "data": SimTypePointer(SimTypeChar(label="Byte"), offset=0)}, name="FWP_BYTE_BLOB", pack=False, align=None), "eku": SimTypePointer(SimStruct({"numEku": SimTypeInt(signed=False, label="UInt32"), "eku": SimTypePointer(SimTypePointer(SimTypeChar(label="Byte"), offset=0), offset=0)}, name="IKEEXT_CERT_EKUS0", pack=False, align=None), offset=0), "name": SimTypePointer(SimStruct({"nameType": SimTypeInt(signed=False, label="IKEEXT_CERT_CRITERIA_NAME_TYPE"), "certName": SimTypePointer(SimTypeChar(label="Char"), offset=0)}, name="IKEEXT_CERT_NAME0", pack=False, align=None), offset=0), "flags": SimTypeInt(signed=False, label="UInt32")}, name="IKEEXT_CERTIFICATE_CRITERIA0", pack=False, align=None), offset=0)}, name="_Anonymous3_e__Struct", pack=False, align=None)}, name="<anon>", label="None"), "outboundConfigType": SimTypeInt(signed=False, label="IKEEXT_CERT_CONFIG_TYPE"), "Anonymous2": SimUnion({"Anonymous1": SimStruct({"outboundRootArraySize": SimTypeInt(signed=False, label="UInt32"), "outboundRootCriteria": SimTypePointer(SimStruct({"certData": SimStruct({"size": SimTypeInt(signed=False, label="UInt32"), "data": SimTypePointer(SimTypeChar(label="Byte"), offset=0)}, name="FWP_BYTE_BLOB", pack=False, align=None), "certHash": SimStruct({"size": SimTypeInt(signed=False, label="UInt32"), "data": SimTypePointer(SimTypeChar(label="Byte"), offset=0)}, name="FWP_BYTE_BLOB", pack=False, align=None), "eku": SimTypePointer(SimStruct({"numEku": SimTypeInt(signed=False, label="UInt32"), "eku": SimTypePointer(SimTypePointer(SimTypeChar(label="Byte"), offset=0), offset=0)}, name="IKEEXT_CERT_EKUS0", pack=False, align=None), offset=0), "name": SimTypePointer(SimStruct({"nameType": SimTypeInt(signed=False, label="IKEEXT_CERT_CRITERIA_NAME_TYPE"), "certName": SimTypePointer(SimTypeChar(label="Char"), offset=0)}, name="IKEEXT_CERT_NAME0", pack=False, align=None), offset=0), "flags": SimTypeInt(signed=False, label="UInt32")}, name="IKEEXT_CERTIFICATE_CRITERIA0", pack=False, align=None), offset=0)}, name="_Anonymous1_e__Struct", pack=False, align=None), "Anonymous2": SimStruct({"outboundEnterpriseStoreArraySize": SimTypeInt(signed=False, label="UInt32"), "outboundEnterpriseStoreCriteria": SimTypePointer(SimStruct({"certData": SimStruct({"size": SimTypeInt(signed=False, label="UInt32"), "data": SimTypePointer(SimTypeChar(label="Byte"), offset=0)}, name="FWP_BYTE_BLOB", pack=False, align=None), "certHash": SimStruct({"size": SimTypeInt(signed=False, label="UInt32"), "data": SimTypePointer(SimTypeChar(label="Byte"), offset=0)}, name="FWP_BYTE_BLOB", pack=False, align=None), "eku": SimTypePointer(SimStruct({"numEku": SimTypeInt(signed=False, label="UInt32"), "eku": SimTypePointer(SimTypePointer(SimTypeChar(label="Byte"), offset=0), offset=0)}, name="IKEEXT_CERT_EKUS0", pack=False, align=None), offset=0), "name": SimTypePointer(SimStruct({"nameType": SimTypeInt(signed=False, label="IKEEXT_CERT_CRITERIA_NAME_TYPE"), "certName": SimTypePointer(SimTypeChar(label="Char"), offset=0)}, name="IKEEXT_CERT_NAME0", pack=False, align=None), offset=0), "flags": SimTypeInt(signed=False, label="UInt32")}, name="IKEEXT_CERTIFICATE_CRITERIA0", pack=False, align=None), offset=0)}, name="_Anonymous2_e__Struct", pack=False, align=None), "Anonymous3": SimStruct({"outboundRootStoreArraySize": SimTypeInt(signed=False, label="UInt32"), "outboundTrustedRootStoreCriteria": SimTypePointer(SimStruct({"certData": SimStruct({"size": SimTypeInt(signed=False, label="UInt32"), "data": SimTypePointer(SimTypeChar(label="Byte"), offset=0)}, name="FWP_BYTE_BLOB", pack=False, align=None), "certHash": SimStruct({"size": SimTypeInt(signed=False, label="UInt32"), "data": SimTypePointer(SimTypeChar(label="Byte"), offset=0)}, name="FWP_BYTE_BLOB", pack=False, align=None), "eku": SimTypePointer(SimStruct({"numEku": SimTypeInt(signed=False, label="UInt32"), "eku": SimTypePointer(SimTypePointer(SimTypeChar(label="Byte"), offset=0), offset=0)}, name="IKEEXT_CERT_EKUS0", pack=False, align=None), offset=0), "name": SimTypePointer(SimStruct({"nameType": SimTypeInt(signed=False, label="IKEEXT_CERT_CRITERIA_NAME_TYPE"), "certName": SimTypePointer(SimTypeChar(label="Char"), offset=0)}, name="IKEEXT_CERT_NAME0", pack=False, align=None), offset=0), "flags": SimTypeInt(signed=False, label="UInt32")}, name="IKEEXT_CERTIFICATE_CRITERIA0", pack=False, align=None), offset=0)}, name="_Anonymous3_e__Struct", pack=False, align=None)}, name="<anon>", label="None"), "flags": SimTypeInt(signed=False, label="IKEEXT_CERT_AUTH"), "localCertLocationUrl": SimStruct({"size": SimTypeInt(signed=False, label="UInt32"), "data": SimTypePointer(SimTypeChar(label="Byte"), offset=0)}, name="FWP_BYTE_BLOB", pack=False, align=None)}, name="IKEEXT_CERTIFICATE_AUTHENTICATION2", pack=False, align=None), "cgaAuthentication": SimStruct({"keyContainerName": SimTypePointer(SimTypeChar(label="Char"), offset=0), "cspName": SimTypePointer(SimTypeChar(label="Char"), offset=0), "cspType": SimTypeInt(signed=False, label="UInt32"), "cgaModifier": SimStruct({"byteArray16": SimTypeFixedSizeArray(SimTypeChar(label="Byte"), 16)}, name="FWP_BYTE_ARRAY16", pack=False, align=None), "cgaCollisionCount": SimTypeChar(label="Byte")}, name="IKEEXT_IPV6_CGA_AUTHENTICATION0", pack=False, align=None), "eapAuthentication": SimStruct({"flags": SimTypeInt(signed=False, label="IKEEXT_EAP_AUTHENTICATION_FLAGS")}, name="IKEEXT_EAP_AUTHENTICATION0", pack=False, align=None)}, name="<anon>", label="None")}, name="IKEEXT_AUTHENTICATION_METHOD2", pack=False, align=None), offset=0), "initiatorImpersonationType": SimTypeInt(signed=False, label="IKEEXT_AUTHENTICATION_IMPERSONATION_TYPE"), "numIkeProposals": SimTypeInt(signed=False, label="UInt32"), "ikeProposals": SimTypePointer(SimStruct({"cipherAlgorithm": SimStruct({"algoIdentifier": SimTypeInt(signed=False, label="IKEEXT_CIPHER_TYPE"), "keyLen": SimTypeInt(signed=False, label="UInt32"), "rounds": SimTypeInt(signed=False, label="UInt32")}, name="IKEEXT_CIPHER_ALGORITHM0", pack=False, align=None), "integrityAlgorithm": SimStruct({"algoIdentifier": SimTypeInt(signed=False, label="IKEEXT_INTEGRITY_TYPE")}, name="IKEEXT_INTEGRITY_ALGORITHM0", pack=False, align=None), "maxLifetimeSeconds": SimTypeInt(signed=False, label="UInt32"), "dhGroup": SimTypeInt(signed=False, label="IKEEXT_DH_GROUP"), "quickModeLimit": SimTypeInt(signed=False, label="UInt32")}, name="IKEEXT_PROPOSAL0", pack=False, align=None), offset=0), "flags": SimTypeInt(signed=False, label="IKEEXT_POLICY_FLAG"), "maxDynamicFilters": SimTypeInt(signed=False, label="UInt32"), "retransmitDurationSecs": SimTypeInt(signed=False, label="UInt32")}, name="IKEEXT_POLICY2", pack=False, align=None), offset=0), "idpOptions": SimTypePointer(SimStruct({"stateIdleTimeoutSeconds": SimTypeInt(signed=False, label="UInt32"), "perIPRateLimitQueueIdleTimeoutSeconds": SimTypeInt(signed=False, label="UInt32"), "ipV6IPsecUnauthDscp": SimTypeChar(label="Byte"), "ipV6IPsecUnauthRateLimitBytesPerSec": SimTypeInt(signed=False, label="UInt32"), "ipV6IPsecUnauthPerIPRateLimitBytesPerSec": SimTypeInt(signed=False, label="UInt32"), "ipV6IPsecAuthDscp": SimTypeChar(label="Byte"), "ipV6IPsecAuthRateLimitBytesPerSec": SimTypeInt(signed=False, label="UInt32"), "icmpV6Dscp": SimTypeChar(label="Byte"), "icmpV6RateLimitBytesPerSec": SimTypeInt(signed=False, label="UInt32"), "ipV6FilterExemptDscp": SimTypeChar(label="Byte"), "ipV6FilterExemptRateLimitBytesPerSec": SimTypeInt(signed=False, label="UInt32"), "defBlockExemptDscp": SimTypeChar(label="Byte"), "defBlockExemptRateLimitBytesPerSec": SimTypeInt(signed=False, label="UInt32"), "maxStateEntries": SimTypeInt(signed=False, label="UInt32"), "maxPerIPRateLimitQueues": SimTypeInt(signed=False, label="UInt32"), "flags": SimTypeInt(signed=False, label="IPSEC_DOSP_FLAGS"), "numPublicIFLuids": SimTypeInt(signed=False, label="UInt32"), "publicIFLuids": SimTypePointer(SimTypeLongLong(signed=False, label="UInt64"), offset=0), "numInternalIFLuids": SimTypeInt(signed=False, label="UInt32"), "internalIFLuids": SimTypePointer(SimTypeLongLong(signed=False, label="UInt64"), offset=0), "publicV6AddrMask": SimStruct({"addr": SimTypeFixedSizeArray(SimTypeChar(label="Byte"), 16), "prefixLength": SimTypeChar(label="Byte")}, name="FWP_V6_ADDR_AND_MASK", pack=False, align=None), "internalV6AddrMask": SimStruct({"addr": SimTypeFixedSizeArray(SimTypeChar(label="Byte"), 16), "prefixLength": SimTypeChar(label="Byte")}, name="FWP_V6_ADDR_AND_MASK", pack=False, align=None)}, name="IPSEC_DOSP_OPTIONS0", pack=False, align=None), offset=0)}, name="<anon>", label="None"), "providerContextId": SimTypeLongLong(signed=False, label="UInt64")}, name="FWPM_PROVIDER_CONTEXT3_", pack=False, align=None), offset=0), SimTypePointer(SimStruct({"providerContextKey": SimTypeBottom(label="Guid"), "displayData": SimStruct({"name": SimTypePointer(SimTypeChar(label="Char"), offset=0), "description": SimTypePointer(SimTypeChar(label="Char"), offset=0)}, name="FWPM_DISPLAY_DATA0", pack=False, align=None), "flags": SimTypeInt(signed=False, label="UInt32"), "providerKey": SimTypePointer(SimTypeBottom(label="Guid"), offset=0), "providerData": SimStruct({"size": SimTypeInt(signed=False, label="UInt32"), "data": SimTypePointer(SimTypeChar(label="Byte"), offset=0)}, name="FWP_BYTE_BLOB", pack=False, align=None), "type": SimTypeInt(signed=False, label="FWPM_PROVIDER_CONTEXT_TYPE"), "Anonymous": SimUnion({"keyingPolicy": SimTypePointer(SimStruct({"numKeyMods": SimTypeInt(signed=False, label="UInt32"), "keyModKeys": SimTypePointer(SimTypeBottom(label="Guid"), offset=0), "flags": SimTypeInt(signed=False, label="UInt32")}, name="IPSEC_KEYING_POLICY1", pack=False, align=None), offset=0), "ikeQmTransportPolicy": SimTypePointer(SimStruct({"numIpsecProposals": SimTypeInt(signed=False, label="UInt32"), "ipsecProposals": SimTypePointer(SimStruct({"lifetime": SimStruct({"lifetimeSeconds": SimTypeInt(signed=False, label="UInt32"), "lifetimeKilobytes": SimTypeInt(signed=False, label="UInt32"), "lifetimePackets": SimTypeInt(signed=False, label="UInt32")}, name="IPSEC_SA_LIFETIME0", pack=False, align=None), "numSaTransforms": SimTypeInt(signed=False, label="UInt32"), "saTransforms": SimTypePointer(SimStruct({"ipsecTransformType": SimTypeInt(signed=False, label="IPSEC_TRANSFORM_TYPE"), "Anonymous": SimUnion({"ahTransform": SimTypePointer(SimStruct({"authTransformId": SimStruct({"authType": SimTypeInt(signed=False, label="IPSEC_AUTH_TYPE"), "authConfig": SimTypeChar(label="Byte")}, name="IPSEC_AUTH_TRANSFORM_ID0", pack=False, align=None), "cryptoModuleId": SimTypePointer(SimTypeBottom(label="Guid"), offset=0)}, name="IPSEC_AUTH_TRANSFORM0", pack=False, align=None), offset=0), "espAuthTransform": SimTypePointer(SimStruct({"authTransformId": SimStruct({"authType": SimTypeInt(signed=False, label="IPSEC_AUTH_TYPE"), "authConfig": SimTypeChar(label="Byte")}, name="IPSEC_AUTH_TRANSFORM_ID0", pack=False, align=None), "cryptoModuleId": SimTypePointer(SimTypeBottom(label="Guid"), offset=0)}, name="IPSEC_AUTH_TRANSFORM0", pack=False, align=None), offset=0), "espCipherTransform": SimTypePointer(SimStruct({"cipherTransformId": SimStruct({"cipherType": SimTypeInt(signed=False, label="IPSEC_CIPHER_TYPE"), "cipherConfig": SimTypeChar(label="Byte")}, name="IPSEC_CIPHER_TRANSFORM_ID0", pack=False, align=None), "cryptoModuleId": SimTypePointer(SimTypeBottom(label="Guid"), offset=0)}, name="IPSEC_CIPHER_TRANSFORM0", pack=False, align=None), offset=0), "espAuthAndCipherTransform": SimTypePointer(SimStruct({"authTransform": SimStruct({"authTransformId": SimStruct({"authType": SimTypeInt(signed=False, label="IPSEC_AUTH_TYPE"), "authConfig": SimTypeChar(label="Byte")}, name="IPSEC_AUTH_TRANSFORM_ID0", pack=False, align=None), "cryptoModuleId": SimTypePointer(SimTypeBottom(label="Guid"), offset=0)}, name="IPSEC_AUTH_TRANSFORM0", pack=False, align=None), "cipherTransform": SimStruct({"cipherTransformId": SimStruct({"cipherType": SimTypeInt(signed=False, label="IPSEC_CIPHER_TYPE"), "cipherConfig": SimTypeChar(label="Byte")}, name="IPSEC_CIPHER_TRANSFORM_ID0", pack=False, align=None), "cryptoModuleId": SimTypePointer(SimTypeBottom(label="Guid"), offset=0)}, name="IPSEC_CIPHER_TRANSFORM0", pack=False, align=None)}, name="IPSEC_AUTH_AND_CIPHER_TRANSFORM0", pack=False, align=None), offset=0), "espAuthFwTransform": SimTypePointer(SimStruct({"authTransformId": SimStruct({"authType": SimTypeInt(signed=False, label="IPSEC_AUTH_TYPE"), "authConfig": SimTypeChar(label="Byte")}, name="IPSEC_AUTH_TRANSFORM_ID0", pack=False, align=None), "cryptoModuleId": SimTypePointer(SimTypeBottom(label="Guid"), offset=0)}, name="IPSEC_AUTH_TRANSFORM0", pack=False, align=None), offset=0)}, name="<anon>", label="None")}, name="IPSEC_SA_TRANSFORM0", pack=False, align=None), offset=0), "pfsGroup": SimTypeInt(signed=False, label="IPSEC_PFS_GROUP")}, name="IPSEC_PROPOSAL0", pack=False, align=None), offset=0), "flags": SimTypeInt(signed=False, label="IPSEC_POLICY_FLAG"), "ndAllowClearTimeoutSeconds": SimTypeInt(signed=False, label="UInt32"), "saIdleTimeout": SimStruct({"idleTimeoutSeconds": SimTypeInt(signed=False, label="UInt32"), "idleTimeoutSecondsFailOver": SimTypeInt(signed=False, label="UInt32")}, name="IPSEC_SA_IDLE_TIMEOUT0", pack=False, align=None), "emPolicy": SimTypePointer(SimStruct({"numAuthenticationMethods": SimTypeInt(signed=False, label="UInt32"), "authenticationMethods": SimTypePointer(SimStruct({"authenticationMethodType": SimTypeInt(signed=False, label="IKEEXT_AUTHENTICATION_METHOD_TYPE"), "Anonymous": SimUnion({"presharedKeyAuthentication": SimStruct({"presharedKey": SimStruct({"size": SimTypeInt(signed=False, label="UInt32"), "data": SimTypePointer(SimTypeChar(label="Byte"), offset=0)}, name="FWP_BYTE_BLOB", pack=False, align=None), "flags": SimTypeInt(signed=False, label="IKEEXT_PRESHARED_KEY_AUTHENTICATION_FLAGS")}, name="IKEEXT_PRESHARED_KEY_AUTHENTICATION1", pack=False, align=None), "certificateAuthentication": SimStruct({"inboundConfigType": SimTypeInt(signed=False, label="IKEEXT_CERT_CONFIG_TYPE"), "Anonymous1": SimUnion({"Anonymous1": SimStruct({"inboundRootArraySize": SimTypeInt(signed=False, label="UInt32"), "inboundRootCriteria": SimTypePointer(SimStruct({"certData": SimStruct({"size": SimTypeInt(signed=False, label="UInt32"), "data": SimTypePointer(SimTypeChar(label="Byte"), offset=0)}, name="FWP_BYTE_BLOB", pack=False, align=None), "certHash": SimStruct({"size": SimTypeInt(signed=False, label="UInt32"), "data": SimTypePointer(SimTypeChar(label="Byte"), offset=0)}, name="FWP_BYTE_BLOB", pack=False, align=None), "eku": SimTypePointer(SimStruct({"numEku": SimTypeInt(signed=False, label="UInt32"), "eku": SimTypePointer(SimTypePointer(SimTypeChar(label="Byte"), offset=0), offset=0)}, name="IKEEXT_CERT_EKUS0", pack=False, align=None), offset=0), "name": SimTypePointer(SimStruct({"nameType": SimTypeInt(signed=False, label="IKEEXT_CERT_CRITERIA_NAME_TYPE"), "certName": SimTypePointer(SimTypeChar(label="Char"), offset=0)}, name="IKEEXT_CERT_NAME0", pack=False, align=None), offset=0), "flags": SimTypeInt(signed=False, label="UInt32")}, name="IKEEXT_CERTIFICATE_CRITERIA0", pack=False, align=None), offset=0)}, name="_Anonymous1_e__Struct", pack=False, align=None), "Anonymous2": SimStruct({"inboundEnterpriseStoreArraySize": SimTypeInt(signed=False, label="UInt32"), "inboundEnterpriseStoreCriteria": SimTypePointer(SimStruct({"certData": SimStruct({"size": SimTypeInt(signed=False, label="UInt32"), "data": SimTypePointer(SimTypeChar(label="Byte"), offset=0)}, name="FWP_BYTE_BLOB", pack=False, align=None), "certHash": SimStruct({"size": SimTypeInt(signed=False, label="UInt32"), "data": SimTypePointer(SimTypeChar(label="Byte"), offset=0)}, name="FWP_BYTE_BLOB", pack=False, align=None), "eku": SimTypePointer(SimStruct({"numEku": SimTypeInt(signed=False, label="UInt32"), "eku": SimTypePointer(SimTypePointer(SimTypeChar(label="Byte"), offset=0), offset=0)}, name="IKEEXT_CERT_EKUS0", pack=False, align=None), offset=0), "name": SimTypePointer(SimStruct({"nameType": SimTypeInt(signed=False, label="IKEEXT_CERT_CRITERIA_NAME_TYPE"), "certName": SimTypePointer(SimTypeChar(label="Char"), offset=0)}, name="IKEEXT_CERT_NAME0", pack=False, align=None), offset=0), "flags": SimTypeInt(signed=False, label="UInt32")}, name="IKEEXT_CERTIFICATE_CRITERIA0", pack=False, align=None), offset=0)}, name="_Anonymous2_e__Struct", pack=False, align=None), "Anonymous3": SimStruct({"inboundRootStoreArraySize": SimTypeInt(signed=False, label="UInt32"), "inboundTrustedRootStoreCriteria": SimTypePointer(SimStruct({"certData": SimStruct({"size": SimTypeInt(signed=False, label="UInt32"), "data": SimTypePointer(SimTypeChar(label="Byte"), offset=0)}, name="FWP_BYTE_BLOB", pack=False, align=None), "certHash": SimStruct({"size": SimTypeInt(signed=False, label="UInt32"), "data": SimTypePointer(SimTypeChar(label="Byte"), offset=0)}, name="FWP_BYTE_BLOB", pack=False, align=None), "eku": SimTypePointer(SimStruct({"numEku": SimTypeInt(signed=False, label="UInt32"), "eku": SimTypePointer(SimTypePointer(SimTypeChar(label="Byte"), offset=0), offset=0)}, name="IKEEXT_CERT_EKUS0", pack=False, align=None), offset=0), "name": SimTypePointer(SimStruct({"nameType": SimTypeInt(signed=False, label="IKEEXT_CERT_CRITERIA_NAME_TYPE"), "certName": SimTypePointer(SimTypeChar(label="Char"), offset=0)}, name="IKEEXT_CERT_NAME0", pack=False, align=None), offset=0), "flags": SimTypeInt(signed=False, label="UInt32")}, name="IKEEXT_CERTIFICATE_CRITERIA0", pack=False, align=None), offset=0)}, name="_Anonymous3_e__Struct", pack=False, align=None)}, name="<anon>", label="None"), "outboundConfigType": SimTypeInt(signed=False, label="IKEEXT_CERT_CONFIG_TYPE"), "Anonymous2": SimUnion({"Anonymous1": SimStruct({"outboundRootArraySize": SimTypeInt(signed=False, label="UInt32"), "outboundRootCriteria": SimTypePointer(SimStruct({"certData": SimStruct({"size": SimTypeInt(signed=False, label="UInt32"), "data": SimTypePointer(SimTypeChar(label="Byte"), offset=0)}, name="FWP_BYTE_BLOB", pack=False, align=None), "certHash": SimStruct({"size": SimTypeInt(signed=False, label="UInt32"), "data": SimTypePointer(SimTypeChar(label="Byte"), offset=0)}, name="FWP_BYTE_BLOB", pack=False, align=None), "eku": SimTypePointer(SimStruct({"numEku": SimTypeInt(signed=False, label="UInt32"), "eku": SimTypePointer(SimTypePointer(SimTypeChar(label="Byte"), offset=0), offset=0)}, name="IKEEXT_CERT_EKUS0", pack=False, align=None), offset=0), "name": SimTypePointer(SimStruct({"nameType": SimTypeInt(signed=False, label="IKEEXT_CERT_CRITERIA_NAME_TYPE"), "certName": SimTypePointer(SimTypeChar(label="Char"), offset=0)}, name="IKEEXT_CERT_NAME0", pack=False, align=None), offset=0), "flags": SimTypeInt(signed=False, label="UInt32")}, name="IKEEXT_CERTIFICATE_CRITERIA0", pack=False, align=None), offset=0)}, name="_Anonymous1_e__Struct", pack=False, align=None), "Anonymous2": SimStruct({"outboundEnterpriseStoreArraySize": SimTypeInt(signed=False, label="UInt32"), "outboundEnterpriseStoreCriteria": SimTypePointer(SimStruct({"certData": SimStruct({"size": SimTypeInt(signed=False, label="UInt32"), "data": SimTypePointer(SimTypeChar(label="Byte"), offset=0)}, name="FWP_BYTE_BLOB", pack=False, align=None), "certHash": SimStruct({"size": SimTypeInt(signed=False, label="UInt32"), "data": SimTypePointer(SimTypeChar(label="Byte"), offset=0)}, name="FWP_BYTE_BLOB", pack=False, align=None), "eku": SimTypePointer(SimStruct({"numEku": SimTypeInt(signed=False, label="UInt32"), "eku": SimTypePointer(SimTypePointer(SimTypeChar(label="Byte"), offset=0), offset=0)}, name="IKEEXT_CERT_EKUS0", pack=False, align=None), offset=0), "name": SimTypePointer(SimStruct({"nameType": SimTypeInt(signed=False, label="IKEEXT_CERT_CRITERIA_NAME_TYPE"), "certName": SimTypePointer(SimTypeChar(label="Char"), offset=0)}, name="IKEEXT_CERT_NAME0", pack=False, align=None), offset=0), "flags": SimTypeInt(signed=False, label="UInt32")}, name="IKEEXT_CERTIFICATE_CRITERIA0", pack=False, align=None), offset=0)}, name="_Anonymous2_e__Struct", pack=False, align=None), "Anonymous3": SimStruct({"outboundRootStoreArraySize": SimTypeInt(signed=False, label="UInt32"), "outboundTrustedRootStoreCriteria": SimTypePointer(SimStruct({"certData": SimStruct({"size": SimTypeInt(signed=False, label="UInt32"), "data": SimTypePointer(SimTypeChar(label="Byte"), offset=0)}, name="FWP_BYTE_BLOB", pack=False, align=None), "certHash": SimStruct({"size": SimTypeInt(signed=False, label="UInt32"), "data": SimTypePointer(SimTypeChar(label="Byte"), offset=0)}, name="FWP_BYTE_BLOB", pack=False, align=None), "eku": SimTypePointer(SimStruct({"numEku": SimTypeInt(signed=False, label="UInt32"), "eku": SimTypePointer(SimTypePointer(SimTypeChar(label="Byte"), offset=0), offset=0)}, name="IKEEXT_CERT_EKUS0", pack=False, align=None), offset=0), "name": SimTypePointer(SimStruct({"nameType": SimTypeInt(signed=False, label="IKEEXT_CERT_CRITERIA_NAME_TYPE"), "certName": SimTypePointer(SimTypeChar(label="Char"), offset=0)}, name="IKEEXT_CERT_NAME0", pack=False, align=None), offset=0), "flags": SimTypeInt(signed=False, label="UInt32")}, name="IKEEXT_CERTIFICATE_CRITERIA0", pack=False, align=None), offset=0)}, name="_Anonymous3_e__Struct", pack=False, align=None)}, name="<anon>", label="None"), "flags": SimTypeInt(signed=False, label="IKEEXT_CERT_AUTH"), "localCertLocationUrl": SimStruct({"size": SimTypeInt(signed=False, label="UInt32"), "data": SimTypePointer(SimTypeChar(label="Byte"), offset=0)}, name="FWP_BYTE_BLOB", pack=False, align=None)}, name="IKEEXT_CERTIFICATE_AUTHENTICATION2", pack=False, align=None), "kerberosAuthentication": SimStruct({"flags": SimTypeInt(signed=False, label="IKEEXT_KERBEROS_AUTHENTICATION_FLAGS"), "proxyServer": SimTypePointer(SimTypeChar(label="Char"), offset=0)}, name="IKEEXT_KERBEROS_AUTHENTICATION1", pack=False, align=None), "reservedAuthentication": SimStruct({"flags": SimTypeInt(signed=False, label="IKEEXT_RESERVED_AUTHENTICATION_FLAGS")}, name="IKEEXT_RESERVED_AUTHENTICATION0", pack=False, align=None), "ntlmV2Authentication": SimStruct({"flags": SimTypeInt(signed=False, label="UInt32")}, name="IKEEXT_NTLM_V2_AUTHENTICATION0", pack=False, align=None), "sslAuthentication": SimStruct({"inboundConfigType": SimTypeInt(signed=False, label="IKEEXT_CERT_CONFIG_TYPE"), "Anonymous1": SimUnion({"Anonymous1": SimStruct({"inboundRootArraySize": SimTypeInt(signed=False, label="UInt32"), "inboundRootCriteria": SimTypePointer(SimStruct({"certData": SimStruct({"size": SimTypeInt(signed=False, label="UInt32"), "data": SimTypePointer(SimTypeChar(label="Byte"), offset=0)}, name="FWP_BYTE_BLOB", pack=False, align=None), "certHash": SimStruct({"size": SimTypeInt(signed=False, label="UInt32"), "data": SimTypePointer(SimTypeChar(label="Byte"), offset=0)}, name="FWP_BYTE_BLOB", pack=False, align=None), "eku": SimTypePointer(SimStruct({"numEku": SimTypeInt(signed=False, label="UInt32"), "eku": SimTypePointer(SimTypePointer(SimTypeChar(label="Byte"), offset=0), offset=0)}, name="IKEEXT_CERT_EKUS0", pack=False, align=None), offset=0), "name": SimTypePointer(SimStruct({"nameType": SimTypeInt(signed=False, label="IKEEXT_CERT_CRITERIA_NAME_TYPE"), "certName": SimTypePointer(SimTypeChar(label="Char"), offset=0)}, name="IKEEXT_CERT_NAME0", pack=False, align=None), offset=0), "flags": SimTypeInt(signed=False, label="UInt32")}, name="IKEEXT_CERTIFICATE_CRITERIA0", pack=False, align=None), offset=0)}, name="_Anonymous1_e__Struct", pack=False, align=None), "Anonymous2": SimStruct({"inboundEnterpriseStoreArraySize": SimTypeInt(signed=False, label="UInt32"), "inboundEnterpriseStoreCriteria": SimTypePointer(SimStruct({"certData": SimStruct({"size": SimTypeInt(signed=False, label="UInt32"), "data": SimTypePointer(SimTypeChar(label="Byte"), offset=0)}, name="FWP_BYTE_BLOB", pack=False, align=None), "certHash": SimStruct({"size": SimTypeInt(signed=False, label="UInt32"), "data": SimTypePointer(SimTypeChar(label="Byte"), offset=0)}, name="FWP_BYTE_BLOB", pack=False, align=None), "eku": SimTypePointer(SimStruct({"numEku": SimTypeInt(signed=False, label="UInt32"), "eku": SimTypePointer(SimTypePointer(SimTypeChar(label="Byte"), offset=0), offset=0)}, name="IKEEXT_CERT_EKUS0", pack=False, align=None), offset=0), "name": SimTypePointer(SimStruct({"nameType": SimTypeInt(signed=False, label="IKEEXT_CERT_CRITERIA_NAME_TYPE"), "certName": SimTypePointer(SimTypeChar(label="Char"), offset=0)}, name="IKEEXT_CERT_NAME0", pack=False, align=None), offset=0), "flags": SimTypeInt(signed=False, label="UInt32")}, name="IKEEXT_CERTIFICATE_CRITERIA0", pack=False, align=None), offset=0)}, name="_Anonymous2_e__Struct", pack=False, align=None), "Anonymous3": SimStruct({"inboundRootStoreArraySize": SimTypeInt(signed=False, label="UInt32"), "inboundTrustedRootStoreCriteria": SimTypePointer(SimStruct({"certData": SimStruct({"size": SimTypeInt(signed=False, label="UInt32"), "data": SimTypePointer(SimTypeChar(label="Byte"), offset=0)}, name="FWP_BYTE_BLOB", pack=False, align=None), "certHash": SimStruct({"size": SimTypeInt(signed=False, label="UInt32"), "data": SimTypePointer(SimTypeChar(label="Byte"), offset=0)}, name="FWP_BYTE_BLOB", pack=False, align=None), "eku": SimTypePointer(SimStruct({"numEku": SimTypeInt(signed=False, label="UInt32"), "eku": SimTypePointer(SimTypePointer(SimTypeChar(label="Byte"), offset=0), offset=0)}, name="IKEEXT_CERT_EKUS0", pack=False, align=None), offset=0), "name": SimTypePointer(SimStruct({"nameType": SimTypeInt(signed=False, label="IKEEXT_CERT_CRITERIA_NAME_TYPE"), "certName": SimTypePointer(SimTypeChar(label="Char"), offset=0)}, name="IKEEXT_CERT_NAME0", pack=False, align=None), offset=0), "flags": SimTypeInt(signed=False, label="UInt32")}, name="IKEEXT_CERTIFICATE_CRITERIA0", pack=False, align=None), offset=0)}, name="_Anonymous3_e__Struct", pack=False, align=None)}, name="<anon>", label="None"), "outboundConfigType": SimTypeInt(signed=False, label="IKEEXT_CERT_CONFIG_TYPE"), "Anonymous2": SimUnion({"Anonymous1": SimStruct({"outboundRootArraySize": SimTypeInt(signed=False, label="UInt32"), "outboundRootCriteria": SimTypePointer(SimStruct({"certData": SimStruct({"size": SimTypeInt(signed=False, label="UInt32"), "data": SimTypePointer(SimTypeChar(label="Byte"), offset=0)}, name="FWP_BYTE_BLOB", pack=False, align=None), "certHash": SimStruct({"size": SimTypeInt(signed=False, label="UInt32"), "data": SimTypePointer(SimTypeChar(label="Byte"), offset=0)}, name="FWP_BYTE_BLOB", pack=False, align=None), "eku": SimTypePointer(SimStruct({"numEku": SimTypeInt(signed=False, label="UInt32"), "eku": SimTypePointer(SimTypePointer(SimTypeChar(label="Byte"), offset=0), offset=0)}, name="IKEEXT_CERT_EKUS0", pack=False, align=None), offset=0), "name": SimTypePointer(SimStruct({"nameType": SimTypeInt(signed=False, label="IKEEXT_CERT_CRITERIA_NAME_TYPE"), "certName": SimTypePointer(SimTypeChar(label="Char"), offset=0)}, name="IKEEXT_CERT_NAME0", pack=False, align=None), offset=0), "flags": SimTypeInt(signed=False, label="UInt32")}, name="IKEEXT_CERTIFICATE_CRITERIA0", pack=False, align=None), offset=0)}, name="_Anonymous1_e__Struct", pack=False, align=None), "Anonymous2": SimStruct({"outboundEnterpriseStoreArraySize": SimTypeInt(signed=False, label="UInt32"), "outboundEnterpriseStoreCriteria": SimTypePointer(SimStruct({"certData": SimStruct({"size": SimTypeInt(signed=False, label="UInt32"), "data": SimTypePointer(SimTypeChar(label="Byte"), offset=0)}, name="FWP_BYTE_BLOB", pack=False, align=None), "certHash": SimStruct({"size": SimTypeInt(signed=False, label="UInt32"), "data": SimTypePointer(SimTypeChar(label="Byte"), offset=0)}, name="FWP_BYTE_BLOB", pack=False, align=None), "eku": SimTypePointer(SimStruct({"numEku": SimTypeInt(signed=False, label="UInt32"), "eku": SimTypePointer(SimTypePointer(SimTypeChar(label="Byte"), offset=0), offset=0)}, name="IKEEXT_CERT_EKUS0", pack=False, align=None), offset=0), "name": SimTypePointer(SimStruct({"nameType": SimTypeInt(signed=False, label="IKEEXT_CERT_CRITERIA_NAME_TYPE"), "certName": SimTypePointer(SimTypeChar(label="Char"), offset=0)}, name="IKEEXT_CERT_NAME0", pack=False, align=None), offset=0), "flags": SimTypeInt(signed=False, label="UInt32")}, name="IKEEXT_CERTIFICATE_CRITERIA0", pack=False, align=None), offset=0)}, name="_Anonymous2_e__Struct", pack=False, align=None), "Anonymous3": SimStruct({"outboundRootStoreArraySize": SimTypeInt(signed=False, label="UInt32"), "outboundTrustedRootStoreCriteria": SimTypePointer(SimStruct({"certData": SimStruct({"size": SimTypeInt(signed=False, label="UInt32"), "data": SimTypePointer(SimTypeChar(label="Byte"), offset=0)}, name="FWP_BYTE_BLOB", pack=False, align=None), "certHash": SimStruct({"size": SimTypeInt(signed=False, label="UInt32"), "data": SimTypePointer(SimTypeChar(label="Byte"), offset=0)}, name="FWP_BYTE_BLOB", pack=False, align=None), "eku": SimTypePointer(SimStruct({"numEku": SimTypeInt(signed=False, label="UInt32"), "eku": SimTypePointer(SimTypePointer(SimTypeChar(label="Byte"), offset=0), offset=0)}, name="IKEEXT_CERT_EKUS0", pack=False, align=None), offset=0), "name": SimTypePointer(SimStruct({"nameType": SimTypeInt(signed=False, label="IKEEXT_CERT_CRITERIA_NAME_TYPE"), "certName": SimTypePointer(SimTypeChar(label="Char"), offset=0)}, name="IKEEXT_CERT_NAME0", pack=False, align=None), offset=0), "flags": SimTypeInt(signed=False, label="UInt32")}, name="IKEEXT_CERTIFICATE_CRITERIA0", pack=False, align=None), offset=0)}, name="_Anonymous3_e__Struct", pack=False, align=None)}, name="<anon>", label="None"), "flags": SimTypeInt(signed=False, label="IKEEXT_CERT_AUTH"), "localCertLocationUrl": SimStruct({"size": SimTypeInt(signed=False, label="UInt32"), "data": SimTypePointer(SimTypeChar(label="Byte"), offset=0)}, name="FWP_BYTE_BLOB", pack=False, align=None)}, name="IKEEXT_CERTIFICATE_AUTHENTICATION2", pack=False, align=None), "cgaAuthentication": SimStruct({"keyContainerName": SimTypePointer(SimTypeChar(label="Char"), offset=0), "cspName": SimTypePointer(SimTypeChar(label="Char"), offset=0), "cspType": SimTypeInt(signed=False, label="UInt32"), "cgaModifier": SimStruct({"byteArray16": SimTypeFixedSizeArray(SimTypeChar(label="Byte"), 16)}, name="FWP_BYTE_ARRAY16", pack=False, align=None), "cgaCollisionCount": SimTypeChar(label="Byte")}, name="IKEEXT_IPV6_CGA_AUTHENTICATION0", pack=False, align=None), "eapAuthentication": SimStruct({"flags": SimTypeInt(signed=False, label="IKEEXT_EAP_AUTHENTICATION_FLAGS")}, name="IKEEXT_EAP_AUTHENTICATION0", pack=False, align=None)}, name="<anon>", label="None")}, name="IKEEXT_AUTHENTICATION_METHOD2", pack=False, align=None), offset=0), "initiatorImpersonationType": SimTypeInt(signed=False, label="IKEEXT_AUTHENTICATION_IMPERSONATION_TYPE")}, name="IKEEXT_EM_POLICY2", pack=False, align=None), offset=0)}, name="IPSEC_TRANSPORT_POLICY2", pack=False, align=None), offset=0), "ikeQmTunnelPolicy": SimTypePointer(SimStruct({"flags": SimTypeInt(signed=False, label="UInt32"), "numIpsecProposals": SimTypeInt(signed=False, label="UInt32"), "ipsecProposals": SimTypePointer(SimStruct({"lifetime": SimStruct({"lifetimeSeconds": SimTypeInt(signed=False, label="UInt32"), "lifetimeKilobytes": SimTypeInt(signed=False, label="UInt32"), "lifetimePackets": SimTypeInt(signed=False, label="UInt32")}, name="IPSEC_SA_LIFETIME0", pack=False, align=None), "numSaTransforms": SimTypeInt(signed=False, label="UInt32"), "saTransforms": SimTypePointer(SimStruct({"ipsecTransformType": SimTypeInt(signed=False, label="IPSEC_TRANSFORM_TYPE"), "Anonymous": SimUnion({"ahTransform": SimTypePointer(SimStruct({"authTransformId": SimStruct({"authType": SimTypeInt(signed=False, label="IPSEC_AUTH_TYPE"), "authConfig": SimTypeChar(label="Byte")}, name="IPSEC_AUTH_TRANSFORM_ID0", pack=False, align=None), "cryptoModuleId": SimTypePointer(SimTypeBottom(label="Guid"), offset=0)}, name="IPSEC_AUTH_TRANSFORM0", pack=False, align=None), offset=0), "espAuthTransform": SimTypePointer(SimStruct({"authTransformId": SimStruct({"authType": SimTypeInt(signed=False, label="IPSEC_AUTH_TYPE"), "authConfig": SimTypeChar(label="Byte")}, name="IPSEC_AUTH_TRANSFORM_ID0", pack=False, align=None), "cryptoModuleId": SimTypePointer(SimTypeBottom(label="Guid"), offset=0)}, name="IPSEC_AUTH_TRANSFORM0", pack=False, align=None), offset=0), "espCipherTransform": SimTypePointer(SimStruct({"cipherTransformId": SimStruct({"cipherType": SimTypeInt(signed=False, label="IPSEC_CIPHER_TYPE"), "cipherConfig": SimTypeChar(label="Byte")}, name="IPSEC_CIPHER_TRANSFORM_ID0", pack=False, align=None), "cryptoModuleId": SimTypePointer(SimTypeBottom(label="Guid"), offset=0)}, name="IPSEC_CIPHER_TRANSFORM0", pack=False, align=None), offset=0), "espAuthAndCipherTransform": SimTypePointer(SimStruct({"authTransform": SimStruct({"authTransformId": SimStruct({"authType": SimTypeInt(signed=False, label="IPSEC_AUTH_TYPE"), "authConfig": SimTypeChar(label="Byte")}, name="IPSEC_AUTH_TRANSFORM_ID0", pack=False, align=None), "cryptoModuleId": SimTypePointer(SimTypeBottom(label="Guid"), offset=0)}, name="IPSEC_AUTH_TRANSFORM0", pack=False, align=None), "cipherTransform": SimStruct({"cipherTransformId": SimStruct({"cipherType": SimTypeInt(signed=False, label="IPSEC_CIPHER_TYPE"), "cipherConfig": SimTypeChar(label="Byte")}, name="IPSEC_CIPHER_TRANSFORM_ID0", pack=False, align=None), "cryptoModuleId": SimTypePointer(SimTypeBottom(label="Guid"), offset=0)}, name="IPSEC_CIPHER_TRANSFORM0", pack=False, align=None)}, name="IPSEC_AUTH_AND_CIPHER_TRANSFORM0", pack=False, align=None), offset=0), "espAuthFwTransform": SimTypePointer(SimStruct({"authTransformId": SimStruct({"authType": SimTypeInt(signed=False, label="IPSEC_AUTH_TYPE"), "authConfig": SimTypeChar(label="Byte")}, name="IPSEC_AUTH_TRANSFORM_ID0", pack=False, align=None), "cryptoModuleId": SimTypePointer(SimTypeBottom(label="Guid"), offset=0)}, name="IPSEC_AUTH_TRANSFORM0", pack=False, align=None), offset=0)}, name="<anon>", label="None")}, name="IPSEC_SA_TRANSFORM0", pack=False, align=None), offset=0), "pfsGroup": SimTypeInt(signed=False, label="IPSEC_PFS_GROUP")}, name="IPSEC_PROPOSAL0", pack=False, align=None), offset=0), "tunnelEndpoints": SimStruct({"ipVersion": SimTypeInt(signed=False, label="FWP_IP_VERSION"), "Anonymous1": SimUnion({"localV4Address": SimTypeInt(signed=False, label="UInt32"), "localV6Address": SimTypeFixedSizeArray(SimTypeChar(label="Byte"), 16)}, name="<anon>", label="None"), "Anonymous2": SimUnion({"remoteV4Address": SimTypeInt(signed=False, label="UInt32"), "remoteV6Address": SimTypeFixedSizeArray(SimTypeChar(label="Byte"), 16)}, name="<anon>", label="None"), "localIfLuid": SimTypeLongLong(signed=False, label="UInt64"), "remoteFqdn": SimTypePointer(SimTypeChar(label="Char"), offset=0), "numAddresses": SimTypeInt(signed=False, label="UInt32"), "remoteAddresses": SimTypePointer(SimStruct({"ipVersion": SimTypeInt(signed=False, label="FWP_IP_VERSION"), "Anonymous": SimUnion({"v4Address": SimTypeInt(signed=False, label="UInt32"), "v6Address": SimTypeFixedSizeArray(SimTypeChar(label="Byte"), 16)}, name="<anon>", label="None")}, name="IPSEC_TUNNEL_ENDPOINT0", pack=False, align=None), offset=0)}, name="IPSEC_TUNNEL_ENDPOINTS2", pack=False, align=None), "saIdleTimeout": SimStruct({"idleTimeoutSeconds": SimTypeInt(signed=False, label="UInt32"), "idleTimeoutSecondsFailOver": SimTypeInt(signed=False, label="UInt32")}, name="IPSEC_SA_IDLE_TIMEOUT0", pack=False, align=None), "emPolicy": SimTypePointer(SimStruct({"numAuthenticationMethods": SimTypeInt(signed=False, label="UInt32"), "authenticationMethods": SimTypePointer(SimStruct({"authenticationMethodType": SimTypeInt(signed=False, label="IKEEXT_AUTHENTICATION_METHOD_TYPE"), "Anonymous": SimUnion({"presharedKeyAuthentication": SimStruct({"presharedKey": SimStruct({"size": SimTypeInt(signed=False, label="UInt32"), "data": SimTypePointer(SimTypeChar(label="Byte"), offset=0)}, name="FWP_BYTE_BLOB", pack=False, align=None), "flags": SimTypeInt(signed=False, label="IKEEXT_PRESHARED_KEY_AUTHENTICATION_FLAGS")}, name="IKEEXT_PRESHARED_KEY_AUTHENTICATION1", pack=False, align=None), "certificateAuthentication": SimStruct({"inboundConfigType": SimTypeInt(signed=False, label="IKEEXT_CERT_CONFIG_TYPE"), "Anonymous1": SimUnion({"Anonymous1": SimStruct({"inboundRootArraySize": SimTypeInt(signed=False, label="UInt32"), "inboundRootCriteria": SimTypePointer(SimStruct({"certData": SimStruct({"size": SimTypeInt(signed=False, label="UInt32"), "data": SimTypePointer(SimTypeChar(label="Byte"), offset=0)}, name="FWP_BYTE_BLOB", pack=False, align=None), "certHash": SimStruct({"size": SimTypeInt(signed=False, label="UInt32"), "data": SimTypePointer(SimTypeChar(label="Byte"), offset=0)}, name="FWP_BYTE_BLOB", pack=False, align=None), "eku": SimTypePointer(SimStruct({"numEku": SimTypeInt(signed=False, label="UInt32"), "eku": SimTypePointer(SimTypePointer(SimTypeChar(label="Byte"), offset=0), offset=0)}, name="IKEEXT_CERT_EKUS0", pack=False, align=None), offset=0), "name": SimTypePointer(SimStruct({"nameType": SimTypeInt(signed=False, label="IKEEXT_CERT_CRITERIA_NAME_TYPE"), "certName": SimTypePointer(SimTypeChar(label="Char"), offset=0)}, name="IKEEXT_CERT_NAME0", pack=False, align=None), offset=0), "flags": SimTypeInt(signed=False, label="UInt32")}, name="IKEEXT_CERTIFICATE_CRITERIA0", pack=False, align=None), offset=0)}, name="_Anonymous1_e__Struct", pack=False, align=None), "Anonymous2": SimStruct({"inboundEnterpriseStoreArraySize": SimTypeInt(signed=False, label="UInt32"), "inboundEnterpriseStoreCriteria": SimTypePointer(SimStruct({"certData": SimStruct({"size": SimTypeInt(signed=False, label="UInt32"), "data": SimTypePointer(SimTypeChar(label="Byte"), offset=0)}, name="FWP_BYTE_BLOB", pack=False, align=None), "certHash": SimStruct({"size": SimTypeInt(signed=False, label="UInt32"), "data": SimTypePointer(SimTypeChar(label="Byte"), offset=0)}, name="FWP_BYTE_BLOB", pack=False, align=None), "eku": SimTypePointer(SimStruct({"numEku": SimTypeInt(signed=False, label="UInt32"), "eku": SimTypePointer(SimTypePointer(SimTypeChar(label="Byte"), offset=0), offset=0)}, name="IKEEXT_CERT_EKUS0", pack=False, align=None), offset=0), "name": SimTypePointer(SimStruct({"nameType": SimTypeInt(signed=False, label="IKEEXT_CERT_CRITERIA_NAME_TYPE"), "certName": SimTypePointer(SimTypeChar(label="Char"), offset=0)}, name="IKEEXT_CERT_NAME0", pack=False, align=None), offset=0), "flags": SimTypeInt(signed=False, label="UInt32")}, name="IKEEXT_CERTIFICATE_CRITERIA0", pack=False, align=None), offset=0)}, name="_Anonymous2_e__Struct", pack=False, align=None), "Anonymous3": SimStruct({"inboundRootStoreArraySize": SimTypeInt(signed=False, label="UInt32"), "inboundTrustedRootStoreCriteria": SimTypePointer(SimStruct({"certData": SimStruct({"size": SimTypeInt(signed=False, label="UInt32"), "data": SimTypePointer(SimTypeChar(label="Byte"), offset=0)}, name="FWP_BYTE_BLOB", pack=False, align=None), "certHash": SimStruct({"size": SimTypeInt(signed=False, label="UInt32"), "data": SimTypePointer(SimTypeChar(label="Byte"), offset=0)}, name="FWP_BYTE_BLOB", pack=False, align=None), "eku": SimTypePointer(SimStruct({"numEku": SimTypeInt(signed=False, label="UInt32"), "eku": SimTypePointer(SimTypePointer(SimTypeChar(label="Byte"), offset=0), offset=0)}, name="IKEEXT_CERT_EKUS0", pack=False, align=None), offset=0), "name": SimTypePointer(SimStruct({"nameType": SimTypeInt(signed=False, label="IKEEXT_CERT_CRITERIA_NAME_TYPE"), "certName": SimTypePointer(SimTypeChar(label="Char"), offset=0)}, name="IKEEXT_CERT_NAME0", pack=False, align=None), offset=0), "flags": SimTypeInt(signed=False, label="UInt32")}, name="IKEEXT_CERTIFICATE_CRITERIA0", pack=False, align=None), offset=0)}, name="_Anonymous3_e__Struct", pack=False, align=None)}, name="<anon>", label="None"), "outboundConfigType": SimTypeInt(signed=False, label="IKEEXT_CERT_CONFIG_TYPE"), "Anonymous2": SimUnion({"Anonymous1": SimStruct({"outboundRootArraySize": SimTypeInt(signed=False, label="UInt32"), "outboundRootCriteria": SimTypePointer(SimStruct({"certData": SimStruct({"size": SimTypeInt(signed=False, label="UInt32"), "data": SimTypePointer(SimTypeChar(label="Byte"), offset=0)}, name="FWP_BYTE_BLOB", pack=False, align=None), "certHash": SimStruct({"size": SimTypeInt(signed=False, label="UInt32"), "data": SimTypePointer(SimTypeChar(label="Byte"), offset=0)}, name="FWP_BYTE_BLOB", pack=False, align=None), "eku": SimTypePointer(SimStruct({"numEku": SimTypeInt(signed=False, label="UInt32"), "eku": SimTypePointer(SimTypePointer(SimTypeChar(label="Byte"), offset=0), offset=0)}, name="IKEEXT_CERT_EKUS0", pack=False, align=None), offset=0), "name": SimTypePointer(SimStruct({"nameType": SimTypeInt(signed=False, label="IKEEXT_CERT_CRITERIA_NAME_TYPE"), "certName": SimTypePointer(SimTypeChar(label="Char"), offset=0)}, name="IKEEXT_CERT_NAME0", pack=False, align=None), offset=0), "flags": SimTypeInt(signed=False, label="UInt32")}, name="IKEEXT_CERTIFICATE_CRITERIA0", pack=False, align=None), offset=0)}, name="_Anonymous1_e__Struct", pack=False, align=None), "Anonymous2": SimStruct({"outboundEnterpriseStoreArraySize": SimTypeInt(signed=False, label="UInt32"), "outboundEnterpriseStoreCriteria": SimTypePointer(SimStruct({"certData": SimStruct({"size": SimTypeInt(signed=False, label="UInt32"), "data": SimTypePointer(SimTypeChar(label="Byte"), offset=0)}, name="FWP_BYTE_BLOB", pack=False, align=None), "certHash": SimStruct({"size": SimTypeInt(signed=False, label="UInt32"), "data": SimTypePointer(SimTypeChar(label="Byte"), offset=0)}, name="FWP_BYTE_BLOB", pack=False, align=None), "eku": SimTypePointer(SimStruct({"numEku": SimTypeInt(signed=False, label="UInt32"), "eku": SimTypePointer(SimTypePointer(SimTypeChar(label="Byte"), offset=0), offset=0)}, name="IKEEXT_CERT_EKUS0", pack=False, align=None), offset=0), "name": SimTypePointer(SimStruct({"nameType": SimTypeInt(signed=False, label="IKEEXT_CERT_CRITERIA_NAME_TYPE"), "certName": SimTypePointer(SimTypeChar(label="Char"), offset=0)}, name="IKEEXT_CERT_NAME0", pack=False, align=None), offset=0), "flags": SimTypeInt(signed=False, label="UInt32")}, name="IKEEXT_CERTIFICATE_CRITERIA0", pack=False, align=None), offset=0)}, name="_Anonymous2_e__Struct", pack=False, align=None), "Anonymous3": SimStruct({"outboundRootStoreArraySize": SimTypeInt(signed=False, label="UInt32"), "outboundTrustedRootStoreCriteria": SimTypePointer(SimStruct({"certData": SimStruct({"size": SimTypeInt(signed=False, label="UInt32"), "data": SimTypePointer(SimTypeChar(label="Byte"), offset=0)}, name="FWP_BYTE_BLOB", pack=False, align=None), "certHash": SimStruct({"size": SimTypeInt(signed=False, label="UInt32"), "data": SimTypePointer(SimTypeChar(label="Byte"), offset=0)}, name="FWP_BYTE_BLOB", pack=False, align=None), "eku": SimTypePointer(SimStruct({"numEku": SimTypeInt(signed=False, label="UInt32"), "eku": SimTypePointer(SimTypePointer(SimTypeChar(label="Byte"), offset=0), offset=0)}, name="IKEEXT_CERT_EKUS0", pack=False, align=None), offset=0), "name": SimTypePointer(SimStruct({"nameType": SimTypeInt(signed=False, label="IKEEXT_CERT_CRITERIA_NAME_TYPE"), "certName": SimTypePointer(SimTypeChar(label="Char"), offset=0)}, name="IKEEXT_CERT_NAME0", pack=False, align=None), offset=0), "flags": SimTypeInt(signed=False, label="UInt32")}, name="IKEEXT_CERTIFICATE_CRITERIA0", pack=False, align=None), offset=0)}, name="_Anonymous3_e__Struct", pack=False, align=None)}, name="<anon>", label="None"), "flags": SimTypeInt(signed=False, label="IKEEXT_CERT_AUTH"), "localCertLocationUrl": SimStruct({"size": SimTypeInt(signed=False, label="UInt32"), "data": SimTypePointer(SimTypeChar(label="Byte"), offset=0)}, name="FWP_BYTE_BLOB", pack=False, align=None)}, name="IKEEXT_CERTIFICATE_AUTHENTICATION2", pack=False, align=None), "kerberosAuthentication": SimStruct({"flags": SimTypeInt(signed=False, label="IKEEXT_KERBEROS_AUTHENTICATION_FLAGS"), "proxyServer": SimTypePointer(SimTypeChar(label="Char"), offset=0)}, name="IKEEXT_KERBEROS_AUTHENTICATION1", pack=False, align=None), "reservedAuthentication": SimStruct({"flags": SimTypeInt(signed=False, label="IKEEXT_RESERVED_AUTHENTICATION_FLAGS")}, name="IKEEXT_RESERVED_AUTHENTICATION0", pack=False, align=None), "ntlmV2Authentication": SimStruct({"flags": SimTypeInt(signed=False, label="UInt32")}, name="IKEEXT_NTLM_V2_AUTHENTICATION0", pack=False, align=None), "sslAuthentication": SimStruct({"inboundConfigType": SimTypeInt(signed=False, label="IKEEXT_CERT_CONFIG_TYPE"), "Anonymous1": SimUnion({"Anonymous1": SimStruct({"inboundRootArraySize": SimTypeInt(signed=False, label="UInt32"), "inboundRootCriteria": SimTypePointer(SimStruct({"certData": SimStruct({"size": SimTypeInt(signed=False, label="UInt32"), "data": SimTypePointer(SimTypeChar(label="Byte"), offset=0)}, name="FWP_BYTE_BLOB", pack=False, align=None), "certHash": SimStruct({"size": SimTypeInt(signed=False, label="UInt32"), "data": SimTypePointer(SimTypeChar(label="Byte"), offset=0)}, name="FWP_BYTE_BLOB", pack=False, align=None), "eku": SimTypePointer(SimStruct({"numEku": SimTypeInt(signed=False, label="UInt32"), "eku": SimTypePointer(SimTypePointer(SimTypeChar(label="Byte"), offset=0), offset=0)}, name="IKEEXT_CERT_EKUS0", pack=False, align=None), offset=0), "name": SimTypePointer(SimStruct({"nameType": SimTypeInt(signed=False, label="IKEEXT_CERT_CRITERIA_NAME_TYPE"), "certName": SimTypePointer(SimTypeChar(label="Char"), offset=0)}, name="IKEEXT_CERT_NAME0", pack=False, align=None), offset=0), "flags": SimTypeInt(signed=False, label="UInt32")}, name="IKEEXT_CERTIFICATE_CRITERIA0", pack=False, align=None), offset=0)}, name="_Anonymous1_e__Struct", pack=False, align=None), "Anonymous2": SimStruct({"inboundEnterpriseStoreArraySize": SimTypeInt(signed=False, label="UInt32"), "inboundEnterpriseStoreCriteria": SimTypePointer(SimStruct({"certData": SimStruct({"size": SimTypeInt(signed=False, label="UInt32"), "data": SimTypePointer(SimTypeChar(label="Byte"), offset=0)}, name="FWP_BYTE_BLOB", pack=False, align=None), "certHash": SimStruct({"size": SimTypeInt(signed=False, label="UInt32"), "data": SimTypePointer(SimTypeChar(label="Byte"), offset=0)}, name="FWP_BYTE_BLOB", pack=False, align=None), "eku": SimTypePointer(SimStruct({"numEku": SimTypeInt(signed=False, label="UInt32"), "eku": SimTypePointer(SimTypePointer(SimTypeChar(label="Byte"), offset=0), offset=0)}, name="IKEEXT_CERT_EKUS0", pack=False, align=None), offset=0), "name": SimTypePointer(SimStruct({"nameType": SimTypeInt(signed=False, label="IKEEXT_CERT_CRITERIA_NAME_TYPE"), "certName": SimTypePointer(SimTypeChar(label="Char"), offset=0)}, name="IKEEXT_CERT_NAME0", pack=False, align=None), offset=0), "flags": SimTypeInt(signed=False, label="UInt32")}, name="IKEEXT_CERTIFICATE_CRITERIA0", pack=False, align=None), offset=0)}, name="_Anonymous2_e__Struct", pack=False, align=None), "Anonymous3": SimStruct({"inboundRootStoreArraySize": SimTypeInt(signed=False, label="UInt32"), "inboundTrustedRootStoreCriteria": SimTypePointer(SimStruct({"certData": SimStruct({"size": SimTypeInt(signed=False, label="UInt32"), "data": SimTypePointer(SimTypeChar(label="Byte"), offset=0)}, name="FWP_BYTE_BLOB", pack=False, align=None), "certHash": SimStruct({"size": SimTypeInt(signed=False, label="UInt32"), "data": SimTypePointer(SimTypeChar(label="Byte"), offset=0)}, name="FWP_BYTE_BLOB", pack=False, align=None), "eku": SimTypePointer(SimStruct({"numEku": SimTypeInt(signed=False, label="UInt32"), "eku": SimTypePointer(SimTypePointer(SimTypeChar(label="Byte"), offset=0), offset=0)}, name="IKEEXT_CERT_EKUS0", pack=False, align=None), offset=0), "name": SimTypePointer(SimStruct({"nameType": SimTypeInt(signed=False, label="IKEEXT_CERT_CRITERIA_NAME_TYPE"), "certName": SimTypePointer(SimTypeChar(label="Char"), offset=0)}, name="IKEEXT_CERT_NAME0", pack=False, align=None), offset=0), "flags": SimTypeInt(signed=False, label="UInt32")}, name="IKEEXT_CERTIFICATE_CRITERIA0", pack=False, align=None), offset=0)}, name="_Anonymous3_e__Struct", pack=False, align=None)}, name="<anon>", label="None"), "outboundConfigType": SimTypeInt(signed=False, label="IKEEXT_CERT_CONFIG_TYPE"), "Anonymous2": SimUnion({"Anonymous1": SimStruct({"outboundRootArraySize": SimTypeInt(signed=False, label="UInt32"), "outboundRootCriteria": SimTypePointer(SimStruct({"certData": SimStruct({"size": SimTypeInt(signed=False, label="UInt32"), "data": SimTypePointer(SimTypeChar(label="Byte"), offset=0)}, name="FWP_BYTE_BLOB", pack=False, align=None), "certHash": SimStruct({"size": SimTypeInt(signed=False, label="UInt32"), "data": SimTypePointer(SimTypeChar(label="Byte"), offset=0)}, name="FWP_BYTE_BLOB", pack=False, align=None), "eku": SimTypePointer(SimStruct({"numEku": SimTypeInt(signed=False, label="UInt32"), "eku": SimTypePointer(SimTypePointer(SimTypeChar(label="Byte"), offset=0), offset=0)}, name="IKEEXT_CERT_EKUS0", pack=False, align=None), offset=0), "name": SimTypePointer(SimStruct({"nameType": SimTypeInt(signed=False, label="IKEEXT_CERT_CRITERIA_NAME_TYPE"), "certName": SimTypePointer(SimTypeChar(label="Char"), offset=0)}, name="IKEEXT_CERT_NAME0", pack=False, align=None), offset=0), "flags": SimTypeInt(signed=False, label="UInt32")}, name="IKEEXT_CERTIFICATE_CRITERIA0", pack=False, align=None), offset=0)}, name="_Anonymous1_e__Struct", pack=False, align=None), "Anonymous2": SimStruct({"outboundEnterpriseStoreArraySize": SimTypeInt(signed=False, label="UInt32"), "outboundEnterpriseStoreCriteria": SimTypePointer(SimStruct({"certData": SimStruct({"size": SimTypeInt(signed=False, label="UInt32"), "data": SimTypePointer(SimTypeChar(label="Byte"), offset=0)}, name="FWP_BYTE_BLOB", pack=False, align=None), "certHash": SimStruct({"size": SimTypeInt(signed=False, label="UInt32"), "data": SimTypePointer(SimTypeChar(label="Byte"), offset=0)}, name="FWP_BYTE_BLOB", pack=False, align=None), "eku": SimTypePointer(SimStruct({"numEku": SimTypeInt(signed=False, label="UInt32"), "eku": SimTypePointer(SimTypePointer(SimTypeChar(label="Byte"), offset=0), offset=0)}, name="IKEEXT_CERT_EKUS0", pack=False, align=None), offset=0), "name": SimTypePointer(SimStruct({"nameType": SimTypeInt(signed=False, label="IKEEXT_CERT_CRITERIA_NAME_TYPE"), "certName": SimTypePointer(SimTypeChar(label="Char"), offset=0)}, name="IKEEXT_CERT_NAME0", pack=False, align=None), offset=0), "flags": SimTypeInt(signed=False, label="UInt32")}, name="IKEEXT_CERTIFICATE_CRITERIA0", pack=False, align=None), offset=0)}, name="_Anonymous2_e__Struct", pack=False, align=None), "Anonymous3": SimStruct({"outboundRootStoreArraySize": SimTypeInt(signed=False, label="UInt32"), "outboundTrustedRootStoreCriteria": SimTypePointer(SimStruct({"certData": SimStruct({"size": SimTypeInt(signed=False, label="UInt32"), "data": SimTypePointer(SimTypeChar(label="Byte"), offset=0)}, name="FWP_BYTE_BLOB", pack=False, align=None), "certHash": SimStruct({"size": SimTypeInt(signed=False, label="UInt32"), "data": SimTypePointer(SimTypeChar(label="Byte"), offset=0)}, name="FWP_BYTE_BLOB", pack=False, align=None), "eku": SimTypePointer(SimStruct({"numEku": SimTypeInt(signed=False, label="UInt32"), "eku": SimTypePointer(SimTypePointer(SimTypeChar(label="Byte"), offset=0), offset=0)}, name="IKEEXT_CERT_EKUS0", pack=False, align=None), offset=0), "name": SimTypePointer(SimStruct({"nameType": SimTypeInt(signed=False, label="IKEEXT_CERT_CRITERIA_NAME_TYPE"), "certName": SimTypePointer(SimTypeChar(label="Char"), offset=0)}, name="IKEEXT_CERT_NAME0", pack=False, align=None), offset=0), "flags": SimTypeInt(signed=False, label="UInt32")}, name="IKEEXT_CERTIFICATE_CRITERIA0", pack=False, align=None), offset=0)}, name="_Anonymous3_e__Struct", pack=False, align=None)}, name="<anon>", label="None"), "flags": SimTypeInt(signed=False, label="IKEEXT_CERT_AUTH"), "localCertLocationUrl": SimStruct({"size": SimTypeInt(signed=False, label="UInt32"), "data": SimTypePointer(SimTypeChar(label="Byte"), offset=0)}, name="FWP_BYTE_BLOB", pack=False, align=None)}, name="IKEEXT_CERTIFICATE_AUTHENTICATION2", pack=False, align=None), "cgaAuthentication": SimStruct({"keyContainerName": SimTypePointer(SimTypeChar(label="Char"), offset=0), "cspName": SimTypePointer(SimTypeChar(label="Char"), offset=0), "cspType": SimTypeInt(signed=False, label="UInt32"), "cgaModifier": SimStruct({"byteArray16": SimTypeFixedSizeArray(SimTypeChar(label="Byte"), 16)}, name="FWP_BYTE_ARRAY16", pack=False, align=None), "cgaCollisionCount": SimTypeChar(label="Byte")}, name="IKEEXT_IPV6_CGA_AUTHENTICATION0", pack=False, align=None), "eapAuthentication": SimStruct({"flags": SimTypeInt(signed=False, label="IKEEXT_EAP_AUTHENTICATION_FLAGS")}, name="IKEEXT_EAP_AUTHENTICATION0", pack=False, align=None)}, name="<anon>", label="None")}, name="IKEEXT_AUTHENTICATION_METHOD2", pack=False, align=None), offset=0), "initiatorImpersonationType": SimTypeInt(signed=False, label="IKEEXT_AUTHENTICATION_IMPERSONATION_TYPE")}, name="IKEEXT_EM_POLICY2", pack=False, align=None), offset=0), "fwdPathSaLifetime": SimTypeInt(signed=False, label="UInt32"), "compartmentId": SimTypeInt(signed=False, label="UInt32"), "numTrafficSelectorPolicy": SimTypeInt(signed=False, label="UInt32"), "trafficSelectorPolicies": SimTypePointer(SimStruct({"flags": SimTypeInt(signed=False, label="UInt32"), "numLocalTrafficSelectors": SimTypeInt(signed=False, label="UInt32"), "localTrafficSelectors": SimTypePointer(SimStruct({"protocolId": SimTypeChar(label="Byte"), "portStart": SimTypeShort(signed=False, label="UInt16"), "portEnd": SimTypeShort(signed=False, label="UInt16"), "ipVersion": SimTypeInt(signed=False, label="FWP_IP_VERSION"), "Anonymous1": SimUnion({"startV4Address": SimTypeInt(signed=False, label="UInt32"), "startV6Address": SimTypeFixedSizeArray(SimTypeChar(label="Byte"), 16)}, name="<anon>", label="None"), "Anonymous2": SimUnion({"endV4Address": SimTypeInt(signed=False, label="UInt32"), "endV6Address": SimTypeFixedSizeArray(SimTypeChar(label="Byte"), 16)}, name="<anon>", label="None")}, name="IPSEC_TRAFFIC_SELECTOR0_", pack=False, align=None), offset=0), "numRemoteTrafficSelectors": SimTypeInt(signed=False, label="UInt32"), "remoteTrafficSelectors": SimTypePointer(SimStruct({"protocolId": SimTypeChar(label="Byte"), "portStart": SimTypeShort(signed=False, label="UInt16"), "portEnd": SimTypeShort(signed=False, label="UInt16"), "ipVersion": SimTypeInt(signed=False, label="FWP_IP_VERSION"), "Anonymous1": SimUnion({"startV4Address": SimTypeInt(signed=False, label="UInt32"), "startV6Address": SimTypeFixedSizeArray(SimTypeChar(label="Byte"), 16)}, name="<anon>", label="None"), "Anonymous2": SimUnion({"endV4Address": SimTypeInt(signed=False, label="UInt32"), "endV6Address": SimTypeFixedSizeArray(SimTypeChar(label="Byte"), 16)}, name="<anon>", label="None")}, name="IPSEC_TRAFFIC_SELECTOR0_", pack=False, align=None), offset=0)}, name="IPSEC_TRAFFIC_SELECTOR_POLICY0_", pack=False, align=None), offset=0)}, name="IPSEC_TUNNEL_POLICY3_", pack=False, align=None), offset=0), "authipQmTransportPolicy": SimTypePointer(SimStruct({"numIpsecProposals": SimTypeInt(signed=False, label="UInt32"), "ipsecProposals": SimTypePointer(SimStruct({"lifetime": SimStruct({"lifetimeSeconds": SimTypeInt(signed=False, label="UInt32"), "lifetimeKilobytes": SimTypeInt(signed=False, label="UInt32"), "lifetimePackets": SimTypeInt(signed=False, label="UInt32")}, name="IPSEC_SA_LIFETIME0", pack=False, align=None), "numSaTransforms": SimTypeInt(signed=False, label="UInt32"), "saTransforms": SimTypePointer(SimStruct({"ipsecTransformType": SimTypeInt(signed=False, label="IPSEC_TRANSFORM_TYPE"), "Anonymous": SimUnion({"ahTransform": SimTypePointer(SimStruct({"authTransformId": SimStruct({"authType": SimTypeInt(signed=False, label="IPSEC_AUTH_TYPE"), "authConfig": SimTypeChar(label="Byte")}, name="IPSEC_AUTH_TRANSFORM_ID0", pack=False, align=None), "cryptoModuleId": SimTypePointer(SimTypeBottom(label="Guid"), offset=0)}, name="IPSEC_AUTH_TRANSFORM0", pack=False, align=None), offset=0), "espAuthTransform": SimTypePointer(SimStruct({"authTransformId": SimStruct({"authType": SimTypeInt(signed=False, label="IPSEC_AUTH_TYPE"), "authConfig": SimTypeChar(label="Byte")}, name="IPSEC_AUTH_TRANSFORM_ID0", pack=False, align=None), "cryptoModuleId": SimTypePointer(SimTypeBottom(label="Guid"), offset=0)}, name="IPSEC_AUTH_TRANSFORM0", pack=False, align=None), offset=0), "espCipherTransform": SimTypePointer(SimStruct({"cipherTransformId": SimStruct({"cipherType": SimTypeInt(signed=False, label="IPSEC_CIPHER_TYPE"), "cipherConfig": SimTypeChar(label="Byte")}, name="IPSEC_CIPHER_TRANSFORM_ID0", pack=False, align=None), "cryptoModuleId": SimTypePointer(SimTypeBottom(label="Guid"), offset=0)}, name="IPSEC_CIPHER_TRANSFORM0", pack=False, align=None), offset=0), "espAuthAndCipherTransform": SimTypePointer(SimStruct({"authTransform": SimStruct({"authTransformId": SimStruct({"authType": SimTypeInt(signed=False, label="IPSEC_AUTH_TYPE"), "authConfig": SimTypeChar(label="Byte")}, name="IPSEC_AUTH_TRANSFORM_ID0", pack=False, align=None), "cryptoModuleId": SimTypePointer(SimTypeBottom(label="Guid"), offset=0)}, name="IPSEC_AUTH_TRANSFORM0", pack=False, align=None), "cipherTransform": SimStruct({"cipherTransformId": SimStruct({"cipherType": SimTypeInt(signed=False, label="IPSEC_CIPHER_TYPE"), "cipherConfig": SimTypeChar(label="Byte")}, name="IPSEC_CIPHER_TRANSFORM_ID0", pack=False, align=None), "cryptoModuleId": SimTypePointer(SimTypeBottom(label="Guid"), offset=0)}, name="IPSEC_CIPHER_TRANSFORM0", pack=False, align=None)}, name="IPSEC_AUTH_AND_CIPHER_TRANSFORM0", pack=False, align=None), offset=0), "espAuthFwTransform": SimTypePointer(SimStruct({"authTransformId": SimStruct({"authType": SimTypeInt(signed=False, label="IPSEC_AUTH_TYPE"), "authConfig": SimTypeChar(label="Byte")}, name="IPSEC_AUTH_TRANSFORM_ID0", pack=False, align=None), "cryptoModuleId": SimTypePointer(SimTypeBottom(label="Guid"), offset=0)}, name="IPSEC_AUTH_TRANSFORM0", pack=False, align=None), offset=0)}, name="<anon>", label="None")}, name="IPSEC_SA_TRANSFORM0", pack=False, align=None), offset=0), "pfsGroup": SimTypeInt(signed=False, label="IPSEC_PFS_GROUP")}, name="IPSEC_PROPOSAL0", pack=False, align=None), offset=0), "flags": SimTypeInt(signed=False, label="IPSEC_POLICY_FLAG"), "ndAllowClearTimeoutSeconds": SimTypeInt(signed=False, label="UInt32"), "saIdleTimeout": SimStruct({"idleTimeoutSeconds": SimTypeInt(signed=False, label="UInt32"), "idleTimeoutSecondsFailOver": SimTypeInt(signed=False, label="UInt32")}, name="IPSEC_SA_IDLE_TIMEOUT0", pack=False, align=None), "emPolicy": SimTypePointer(SimStruct({"numAuthenticationMethods": SimTypeInt(signed=False, label="UInt32"), "authenticationMethods": SimTypePointer(SimStruct({"authenticationMethodType": SimTypeInt(signed=False, label="IKEEXT_AUTHENTICATION_METHOD_TYPE"), "Anonymous": SimUnion({"presharedKeyAuthentication": SimStruct({"presharedKey": SimStruct({"size": SimTypeInt(signed=False, label="UInt32"), "data": SimTypePointer(SimTypeChar(label="Byte"), offset=0)}, name="FWP_BYTE_BLOB", pack=False, align=None), "flags": SimTypeInt(signed=False, label="IKEEXT_PRESHARED_KEY_AUTHENTICATION_FLAGS")}, name="IKEEXT_PRESHARED_KEY_AUTHENTICATION1", pack=False, align=None), "certificateAuthentication": SimStruct({"inboundConfigType": SimTypeInt(signed=False, label="IKEEXT_CERT_CONFIG_TYPE"), "Anonymous1": SimUnion({"Anonymous1": SimStruct({"inboundRootArraySize": SimTypeInt(signed=False, label="UInt32"), "inboundRootCriteria": SimTypePointer(SimStruct({"certData": SimStruct({"size": SimTypeInt(signed=False, label="UInt32"), "data": SimTypePointer(SimTypeChar(label="Byte"), offset=0)}, name="FWP_BYTE_BLOB", pack=False, align=None), "certHash": SimStruct({"size": SimTypeInt(signed=False, label="UInt32"), "data": SimTypePointer(SimTypeChar(label="Byte"), offset=0)}, name="FWP_BYTE_BLOB", pack=False, align=None), "eku": SimTypePointer(SimStruct({"numEku": SimTypeInt(signed=False, label="UInt32"), "eku": SimTypePointer(SimTypePointer(SimTypeChar(label="Byte"), offset=0), offset=0)}, name="IKEEXT_CERT_EKUS0", pack=False, align=None), offset=0), "name": SimTypePointer(SimStruct({"nameType": SimTypeInt(signed=False, label="IKEEXT_CERT_CRITERIA_NAME_TYPE"), "certName": SimTypePointer(SimTypeChar(label="Char"), offset=0)}, name="IKEEXT_CERT_NAME0", pack=False, align=None), offset=0), "flags": SimTypeInt(signed=False, label="UInt32")}, name="IKEEXT_CERTIFICATE_CRITERIA0", pack=False, align=None), offset=0)}, name="_Anonymous1_e__Struct", pack=False, align=None), "Anonymous2": SimStruct({"inboundEnterpriseStoreArraySize": SimTypeInt(signed=False, label="UInt32"), "inboundEnterpriseStoreCriteria": SimTypePointer(SimStruct({"certData": SimStruct({"size": SimTypeInt(signed=False, label="UInt32"), "data": SimTypePointer(SimTypeChar(label="Byte"), offset=0)}, name="FWP_BYTE_BLOB", pack=False, align=None), "certHash": SimStruct({"size": SimTypeInt(signed=False, label="UInt32"), "data": SimTypePointer(SimTypeChar(label="Byte"), offset=0)}, name="FWP_BYTE_BLOB", pack=False, align=None), "eku": SimTypePointer(SimStruct({"numEku": SimTypeInt(signed=False, label="UInt32"), "eku": SimTypePointer(SimTypePointer(SimTypeChar(label="Byte"), offset=0), offset=0)}, name="IKEEXT_CERT_EKUS0", pack=False, align=None), offset=0), "name": SimTypePointer(SimStruct({"nameType": SimTypeInt(signed=False, label="IKEEXT_CERT_CRITERIA_NAME_TYPE"), "certName": SimTypePointer(SimTypeChar(label="Char"), offset=0)}, name="IKEEXT_CERT_NAME0", pack=False, align=None), offset=0), "flags": SimTypeInt(signed=False, label="UInt32")}, name="IKEEXT_CERTIFICATE_CRITERIA0", pack=False, align=None), offset=0)}, name="_Anonymous2_e__Struct", pack=False, align=None), "Anonymous3": SimStruct({"inboundRootStoreArraySize": SimTypeInt(signed=False, label="UInt32"), "inboundTrustedRootStoreCriteria": SimTypePointer(SimStruct({"certData": SimStruct({"size": SimTypeInt(signed=False, label="UInt32"), "data": SimTypePointer(SimTypeChar(label="Byte"), offset=0)}, name="FWP_BYTE_BLOB", pack=False, align=None), "certHash": SimStruct({"size": SimTypeInt(signed=False, label="UInt32"), "data": SimTypePointer(SimTypeChar(label="Byte"), offset=0)}, name="FWP_BYTE_BLOB", pack=False, align=None), "eku": SimTypePointer(SimStruct({"numEku": SimTypeInt(signed=False, label="UInt32"), "eku": SimTypePointer(SimTypePointer(SimTypeChar(label="Byte"), offset=0), offset=0)}, name="IKEEXT_CERT_EKUS0", pack=False, align=None), offset=0), "name": SimTypePointer(SimStruct({"nameType": SimTypeInt(signed=False, label="IKEEXT_CERT_CRITERIA_NAME_TYPE"), "certName": SimTypePointer(SimTypeChar(label="Char"), offset=0)}, name="IKEEXT_CERT_NAME0", pack=False, align=None), offset=0), "flags": SimTypeInt(signed=False, label="UInt32")}, name="IKEEXT_CERTIFICATE_CRITERIA0", pack=False, align=None), offset=0)}, name="_Anonymous3_e__Struct", pack=False, align=None)}, name="<anon>", label="None"), "outboundConfigType": SimTypeInt(signed=False, label="IKEEXT_CERT_CONFIG_TYPE"), "Anonymous2": SimUnion({"Anonymous1": SimStruct({"outboundRootArraySize": SimTypeInt(signed=False, label="UInt32"), "outboundRootCriteria": SimTypePointer(SimStruct({"certData": SimStruct({"size": SimTypeInt(signed=False, label="UInt32"), "data": SimTypePointer(SimTypeChar(label="Byte"), offset=0)}, name="FWP_BYTE_BLOB", pack=False, align=None), "certHash": SimStruct({"size": SimTypeInt(signed=False, label="UInt32"), "data": SimTypePointer(SimTypeChar(label="Byte"), offset=0)}, name="FWP_BYTE_BLOB", pack=False, align=None), "eku": SimTypePointer(SimStruct({"numEku": SimTypeInt(signed=False, label="UInt32"), "eku": SimTypePointer(SimTypePointer(SimTypeChar(label="Byte"), offset=0), offset=0)}, name="IKEEXT_CERT_EKUS0", pack=False, align=None), offset=0), "name": SimTypePointer(SimStruct({"nameType": SimTypeInt(signed=False, label="IKEEXT_CERT_CRITERIA_NAME_TYPE"), "certName": SimTypePointer(SimTypeChar(label="Char"), offset=0)}, name="IKEEXT_CERT_NAME0", pack=False, align=None), offset=0), "flags": SimTypeInt(signed=False, label="UInt32")}, name="IKEEXT_CERTIFICATE_CRITERIA0", pack=False, align=None), offset=0)}, name="_Anonymous1_e__Struct", pack=False, align=None), "Anonymous2": SimStruct({"outboundEnterpriseStoreArraySize": SimTypeInt(signed=False, label="UInt32"), "outboundEnterpriseStoreCriteria": SimTypePointer(SimStruct({"certData": SimStruct({"size": SimTypeInt(signed=False, label="UInt32"), "data": SimTypePointer(SimTypeChar(label="Byte"), offset=0)}, name="FWP_BYTE_BLOB", pack=False, align=None), "certHash": SimStruct({"size": SimTypeInt(signed=False, label="UInt32"), "data": SimTypePointer(SimTypeChar(label="Byte"), offset=0)}, name="FWP_BYTE_BLOB", pack=False, align=None), "eku": SimTypePointer(SimStruct({"numEku": SimTypeInt(signed=False, label="UInt32"), "eku": SimTypePointer(SimTypePointer(SimTypeChar(label="Byte"), offset=0), offset=0)}, name="IKEEXT_CERT_EKUS0", pack=False, align=None), offset=0), "name": SimTypePointer(SimStruct({"nameType": SimTypeInt(signed=False, label="IKEEXT_CERT_CRITERIA_NAME_TYPE"), "certName": SimTypePointer(SimTypeChar(label="Char"), offset=0)}, name="IKEEXT_CERT_NAME0", pack=False, align=None), offset=0), "flags": SimTypeInt(signed=False, label="UInt32")}, name="IKEEXT_CERTIFICATE_CRITERIA0", pack=False, align=None), offset=0)}, name="_Anonymous2_e__Struct", pack=False, align=None), "Anonymous3": SimStruct({"outboundRootStoreArraySize": SimTypeInt(signed=False, label="UInt32"), "outboundTrustedRootStoreCriteria": SimTypePointer(SimStruct({"certData": SimStruct({"size": SimTypeInt(signed=False, label="UInt32"), "data": SimTypePointer(SimTypeChar(label="Byte"), offset=0)}, name="FWP_BYTE_BLOB", pack=False, align=None), "certHash": SimStruct({"size": SimTypeInt(signed=False, label="UInt32"), "data": SimTypePointer(SimTypeChar(label="Byte"), offset=0)}, name="FWP_BYTE_BLOB", pack=False, align=None), "eku": SimTypePointer(SimStruct({"numEku": SimTypeInt(signed=False, label="UInt32"), "eku": SimTypePointer(SimTypePointer(SimTypeChar(label="Byte"), offset=0), offset=0)}, name="IKEEXT_CERT_EKUS0", pack=False, align=None), offset=0), "name": SimTypePointer(SimStruct({"nameType": SimTypeInt(signed=False, label="IKEEXT_CERT_CRITERIA_NAME_TYPE"), "certName": SimTypePointer(SimTypeChar(label="Char"), offset=0)}, name="IKEEXT_CERT_NAME0", pack=False, align=None), offset=0), "flags": SimTypeInt(signed=False, label="UInt32")}, name="IKEEXT_CERTIFICATE_CRITERIA0", pack=False, align=None), offset=0)}, name="_Anonymous3_e__Struct", pack=False, align=None)}, name="<anon>", label="None"), "flags": SimTypeInt(signed=False, label="IKEEXT_CERT_AUTH"), "localCertLocationUrl": SimStruct({"size": SimTypeInt(signed=False, label="UInt32"), "data": SimTypePointer(SimTypeChar(label="Byte"), offset=0)}, name="FWP_BYTE_BLOB", pack=False, align=None)}, name="IKEEXT_CERTIFICATE_AUTHENTICATION2", pack=False, align=None), "kerberosAuthentication": SimStruct({"flags": SimTypeInt(signed=False, label="IKEEXT_KERBEROS_AUTHENTICATION_FLAGS"), "proxyServer": SimTypePointer(SimTypeChar(label="Char"), offset=0)}, name="IKEEXT_KERBEROS_AUTHENTICATION1", pack=False, align=None), "reservedAuthentication": SimStruct({"flags": SimTypeInt(signed=False, label="IKEEXT_RESERVED_AUTHENTICATION_FLAGS")}, name="IKEEXT_RESERVED_AUTHENTICATION0", pack=False, align=None), "ntlmV2Authentication": SimStruct({"flags": SimTypeInt(signed=False, label="UInt32")}, name="IKEEXT_NTLM_V2_AUTHENTICATION0", pack=False, align=None), "sslAuthentication": SimStruct({"inboundConfigType": SimTypeInt(signed=False, label="IKEEXT_CERT_CONFIG_TYPE"), "Anonymous1": SimUnion({"Anonymous1": SimStruct({"inboundRootArraySize": SimTypeInt(signed=False, label="UInt32"), "inboundRootCriteria": SimTypePointer(SimStruct({"certData": SimStruct({"size": SimTypeInt(signed=False, label="UInt32"), "data": SimTypePointer(SimTypeChar(label="Byte"), offset=0)}, name="FWP_BYTE_BLOB", pack=False, align=None), "certHash": SimStruct({"size": SimTypeInt(signed=False, label="UInt32"), "data": SimTypePointer(SimTypeChar(label="Byte"), offset=0)}, name="FWP_BYTE_BLOB", pack=False, align=None), "eku": SimTypePointer(SimStruct({"numEku": SimTypeInt(signed=False, label="UInt32"), "eku": SimTypePointer(SimTypePointer(SimTypeChar(label="Byte"), offset=0), offset=0)}, name="IKEEXT_CERT_EKUS0", pack=False, align=None), offset=0), "name": SimTypePointer(SimStruct({"nameType": SimTypeInt(signed=False, label="IKEEXT_CERT_CRITERIA_NAME_TYPE"), "certName": SimTypePointer(SimTypeChar(label="Char"), offset=0)}, name="IKEEXT_CERT_NAME0", pack=False, align=None), offset=0), "flags": SimTypeInt(signed=False, label="UInt32")}, name="IKEEXT_CERTIFICATE_CRITERIA0", pack=False, align=None), offset=0)}, name="_Anonymous1_e__Struct", pack=False, align=None), "Anonymous2": SimStruct({"inboundEnterpriseStoreArraySize": SimTypeInt(signed=False, label="UInt32"), "inboundEnterpriseStoreCriteria": SimTypePointer(SimStruct({"certData": SimStruct({"size": SimTypeInt(signed=False, label="UInt32"), "data": SimTypePointer(SimTypeChar(label="Byte"), offset=0)}, name="FWP_BYTE_BLOB", pack=False, align=None), "certHash": SimStruct({"size": SimTypeInt(signed=False, label="UInt32"), "data": SimTypePointer(SimTypeChar(label="Byte"), offset=0)}, name="FWP_BYTE_BLOB", pack=False, align=None), "eku": SimTypePointer(SimStruct({"numEku": SimTypeInt(signed=False, label="UInt32"), "eku": SimTypePointer(SimTypePointer(SimTypeChar(label="Byte"), offset=0), offset=0)}, name="IKEEXT_CERT_EKUS0", pack=False, align=None), offset=0), "name": SimTypePointer(SimStruct({"nameType": SimTypeInt(signed=False, label="IKEEXT_CERT_CRITERIA_NAME_TYPE"), "certName": SimTypePointer(SimTypeChar(label="Char"), offset=0)}, name="IKEEXT_CERT_NAME0", pack=False, align=None), offset=0), "flags": SimTypeInt(signed=False, label="UInt32")}, name="IKEEXT_CERTIFICATE_CRITERIA0", pack=False, align=None), offset=0)}, name="_Anonymous2_e__Struct", pack=False, align=None), "Anonymous3": SimStruct({"inboundRootStoreArraySize": SimTypeInt(signed=False, label="UInt32"), "inboundTrustedRootStoreCriteria": SimTypePointer(SimStruct({"certData": SimStruct({"size": SimTypeInt(signed=False, label="UInt32"), "data": SimTypePointer(SimTypeChar(label="Byte"), offset=0)}, name="FWP_BYTE_BLOB", pack=False, align=None), "certHash": SimStruct({"size": SimTypeInt(signed=False, label="UInt32"), "data": SimTypePointer(SimTypeChar(label="Byte"), offset=0)}, name="FWP_BYTE_BLOB", pack=False, align=None), "eku": SimTypePointer(SimStruct({"numEku": SimTypeInt(signed=False, label="UInt32"), "eku": SimTypePointer(SimTypePointer(SimTypeChar(label="Byte"), offset=0), offset=0)}, name="IKEEXT_CERT_EKUS0", pack=False, align=None), offset=0), "name": SimTypePointer(SimStruct({"nameType": SimTypeInt(signed=False, label="IKEEXT_CERT_CRITERIA_NAME_TYPE"), "certName": SimTypePointer(SimTypeChar(label="Char"), offset=0)}, name="IKEEXT_CERT_NAME0", pack=False, align=None), offset=0), "flags": SimTypeInt(signed=False, label="UInt32")}, name="IKEEXT_CERTIFICATE_CRITERIA0", pack=False, align=None), offset=0)}, name="_Anonymous3_e__Struct", pack=False, align=None)}, name="<anon>", label="None"), "outboundConfigType": SimTypeInt(signed=False, label="IKEEXT_CERT_CONFIG_TYPE"), "Anonymous2": SimUnion({"Anonymous1": SimStruct({"outboundRootArraySize": SimTypeInt(signed=False, label="UInt32"), "outboundRootCriteria": SimTypePointer(SimStruct({"certData": SimStruct({"size": SimTypeInt(signed=False, label="UInt32"), "data": SimTypePointer(SimTypeChar(label="Byte"), offset=0)}, name="FWP_BYTE_BLOB", pack=False, align=None), "certHash": SimStruct({"size": SimTypeInt(signed=False, label="UInt32"), "data": SimTypePointer(SimTypeChar(label="Byte"), offset=0)}, name="FWP_BYTE_BLOB", pack=False, align=None), "eku": SimTypePointer(SimStruct({"numEku": SimTypeInt(signed=False, label="UInt32"), "eku": SimTypePointer(SimTypePointer(SimTypeChar(label="Byte"), offset=0), offset=0)}, name="IKEEXT_CERT_EKUS0", pack=False, align=None), offset=0), "name": SimTypePointer(SimStruct({"nameType": SimTypeInt(signed=False, label="IKEEXT_CERT_CRITERIA_NAME_TYPE"), "certName": SimTypePointer(SimTypeChar(label="Char"), offset=0)}, name="IKEEXT_CERT_NAME0", pack=False, align=None), offset=0), "flags": SimTypeInt(signed=False, label="UInt32")}, name="IKEEXT_CERTIFICATE_CRITERIA0", pack=False, align=None), offset=0)}, name="_Anonymous1_e__Struct", pack=False, align=None), "Anonymous2": SimStruct({"outboundEnterpriseStoreArraySize": SimTypeInt(signed=False, label="UInt32"), "outboundEnterpriseStoreCriteria": SimTypePointer(SimStruct({"certData": SimStruct({"size": SimTypeInt(signed=False, label="UInt32"), "data": SimTypePointer(SimTypeChar(label="Byte"), offset=0)}, name="FWP_BYTE_BLOB", pack=False, align=None), "certHash": SimStruct({"size": SimTypeInt(signed=False, label="UInt32"), "data": SimTypePointer(SimTypeChar(label="Byte"), offset=0)}, name="FWP_BYTE_BLOB", pack=False, align=None), "eku": SimTypePointer(SimStruct({"numEku": SimTypeInt(signed=False, label="UInt32"), "eku": SimTypePointer(SimTypePointer(SimTypeChar(label="Byte"), offset=0), offset=0)}, name="IKEEXT_CERT_EKUS0", pack=False, align=None), offset=0), "name": SimTypePointer(SimStruct({"nameType": SimTypeInt(signed=False, label="IKEEXT_CERT_CRITERIA_NAME_TYPE"), "certName": SimTypePointer(SimTypeChar(label="Char"), offset=0)}, name="IKEEXT_CERT_NAME0", pack=False, align=None), offset=0), "flags": SimTypeInt(signed=False, label="UInt32")}, name="IKEEXT_CERTIFICATE_CRITERIA0", pack=False, align=None), offset=0)}, name="_Anonymous2_e__Struct", pack=False, align=None), "Anonymous3": SimStruct({"outboundRootStoreArraySize": SimTypeInt(signed=False, label="UInt32"), "outboundTrustedRootStoreCriteria": SimTypePointer(SimStruct({"certData": SimStruct({"size": SimTypeInt(signed=False, label="UInt32"), "data": SimTypePointer(SimTypeChar(label="Byte"), offset=0)}, name="FWP_BYTE_BLOB", pack=False, align=None), "certHash": SimStruct({"size": SimTypeInt(signed=False, label="UInt32"), "data": SimTypePointer(SimTypeChar(label="Byte"), offset=0)}, name="FWP_BYTE_BLOB", pack=False, align=None), "eku": SimTypePointer(SimStruct({"numEku": SimTypeInt(signed=False, label="UInt32"), "eku": SimTypePointer(SimTypePointer(SimTypeChar(label="Byte"), offset=0), offset=0)}, name="IKEEXT_CERT_EKUS0", pack=False, align=None), offset=0), "name": SimTypePointer(SimStruct({"nameType": SimTypeInt(signed=False, label="IKEEXT_CERT_CRITERIA_NAME_TYPE"), "certName": SimTypePointer(SimTypeChar(label="Char"), offset=0)}, name="IKEEXT_CERT_NAME0", pack=False, align=None), offset=0), "flags": SimTypeInt(signed=False, label="UInt32")}, name="IKEEXT_CERTIFICATE_CRITERIA0", pack=False, align=None), offset=0)}, name="_Anonymous3_e__Struct", pack=False, align=None)}, name="<anon>", label="None"), "flags": SimTypeInt(signed=False, label="IKEEXT_CERT_AUTH"), "localCertLocationUrl": SimStruct({"size": SimTypeInt(signed=False, label="UInt32"), "data": SimTypePointer(SimTypeChar(label="Byte"), offset=0)}, name="FWP_BYTE_BLOB", pack=False, align=None)}, name="IKEEXT_CERTIFICATE_AUTHENTICATION2", pack=False, align=None), "cgaAuthentication": SimStruct({"keyContainerName": SimTypePointer(SimTypeChar(label="Char"), offset=0), "cspName": SimTypePointer(SimTypeChar(label="Char"), offset=0), "cspType": SimTypeInt(signed=False, label="UInt32"), "cgaModifier": SimStruct({"byteArray16": SimTypeFixedSizeArray(SimTypeChar(label="Byte"), 16)}, name="FWP_BYTE_ARRAY16", pack=False, align=None), "cgaCollisionCount": SimTypeChar(label="Byte")}, name="IKEEXT_IPV6_CGA_AUTHENTICATION0", pack=False, align=None), "eapAuthentication": SimStruct({"flags": SimTypeInt(signed=False, label="IKEEXT_EAP_AUTHENTICATION_FLAGS")}, name="IKEEXT_EAP_AUTHENTICATION0", pack=False, align=None)}, name="<anon>", label="None")}, name="IKEEXT_AUTHENTICATION_METHOD2", pack=False, align=None), offset=0), "initiatorImpersonationType": SimTypeInt(signed=False, label="IKEEXT_AUTHENTICATION_IMPERSONATION_TYPE")}, name="IKEEXT_EM_POLICY2", pack=False, align=None), offset=0)}, name="IPSEC_TRANSPORT_POLICY2", pack=False, align=None), offset=0), "authipQmTunnelPolicy": SimTypePointer(SimStruct({"flags": SimTypeInt(signed=False, label="UInt32"), "numIpsecProposals": SimTypeInt(signed=False, label="UInt32"), "ipsecProposals": SimTypePointer(SimStruct({"lifetime": SimStruct({"lifetimeSeconds": SimTypeInt(signed=False, label="UInt32"), "lifetimeKilobytes": SimTypeInt(signed=False, label="UInt32"), "lifetimePackets": SimTypeInt(signed=False, label="UInt32")}, name="IPSEC_SA_LIFETIME0", pack=False, align=None), "numSaTransforms": SimTypeInt(signed=False, label="UInt32"), "saTransforms": SimTypePointer(SimStruct({"ipsecTransformType": SimTypeInt(signed=False, label="IPSEC_TRANSFORM_TYPE"), "Anonymous": SimUnion({"ahTransform": SimTypePointer(SimStruct({"authTransformId": SimStruct({"authType": SimTypeInt(signed=False, label="IPSEC_AUTH_TYPE"), "authConfig": SimTypeChar(label="Byte")}, name="IPSEC_AUTH_TRANSFORM_ID0", pack=False, align=None), "cryptoModuleId": SimTypePointer(SimTypeBottom(label="Guid"), offset=0)}, name="IPSEC_AUTH_TRANSFORM0", pack=False, align=None), offset=0), "espAuthTransform": SimTypePointer(SimStruct({"authTransformId": SimStruct({"authType": SimTypeInt(signed=False, label="IPSEC_AUTH_TYPE"), "authConfig": SimTypeChar(label="Byte")}, name="IPSEC_AUTH_TRANSFORM_ID0", pack=False, align=None), "cryptoModuleId": SimTypePointer(SimTypeBottom(label="Guid"), offset=0)}, name="IPSEC_AUTH_TRANSFORM0", pack=False, align=None), offset=0), "espCipherTransform": SimTypePointer(SimStruct({"cipherTransformId": SimStruct({"cipherType": SimTypeInt(signed=False, label="IPSEC_CIPHER_TYPE"), "cipherConfig": SimTypeChar(label="Byte")}, name="IPSEC_CIPHER_TRANSFORM_ID0", pack=False, align=None), "cryptoModuleId": SimTypePointer(SimTypeBottom(label="Guid"), offset=0)}, name="IPSEC_CIPHER_TRANSFORM0", pack=False, align=None), offset=0), "espAuthAndCipherTransform": SimTypePointer(SimStruct({"authTransform": SimStruct({"authTransformId": SimStruct({"authType": SimTypeInt(signed=False, label="IPSEC_AUTH_TYPE"), "authConfig": SimTypeChar(label="Byte")}, name="IPSEC_AUTH_TRANSFORM_ID0", pack=False, align=None), "cryptoModuleId": SimTypePointer(SimTypeBottom(label="Guid"), offset=0)}, name="IPSEC_AUTH_TRANSFORM0", pack=False, align=None), "cipherTransform": SimStruct({"cipherTransformId": SimStruct({"cipherType": SimTypeInt(signed=False, label="IPSEC_CIPHER_TYPE"), "cipherConfig": SimTypeChar(label="Byte")}, name="IPSEC_CIPHER_TRANSFORM_ID0", pack=False, align=None), "cryptoModuleId": SimTypePointer(SimTypeBottom(label="Guid"), offset=0)}, name="IPSEC_CIPHER_TRANSFORM0", pack=False, align=None)}, name="IPSEC_AUTH_AND_CIPHER_TRANSFORM0", pack=False, align=None), offset=0), "espAuthFwTransform": SimTypePointer(SimStruct({"authTransformId": SimStruct({"authType": SimTypeInt(signed=False, label="IPSEC_AUTH_TYPE"), "authConfig": SimTypeChar(label="Byte")}, name="IPSEC_AUTH_TRANSFORM_ID0", pack=False, align=None), "cryptoModuleId": SimTypePointer(SimTypeBottom(label="Guid"), offset=0)}, name="IPSEC_AUTH_TRANSFORM0", pack=False, align=None), offset=0)}, name="<anon>", label="None")}, name="IPSEC_SA_TRANSFORM0", pack=False, align=None), offset=0), "pfsGroup": SimTypeInt(signed=False, label="IPSEC_PFS_GROUP")}, name="IPSEC_PROPOSAL0", pack=False, align=None), offset=0), "tunnelEndpoints": SimStruct({"ipVersion": SimTypeInt(signed=False, label="FWP_IP_VERSION"), "Anonymous1": SimUnion({"localV4Address": SimTypeInt(signed=False, label="UInt32"), "localV6Address": SimTypeFixedSizeArray(SimTypeChar(label="Byte"), 16)}, name="<anon>", label="None"), "Anonymous2": SimUnion({"remoteV4Address": SimTypeInt(signed=False, label="UInt32"), "remoteV6Address": SimTypeFixedSizeArray(SimTypeChar(label="Byte"), 16)}, name="<anon>", label="None"), "localIfLuid": SimTypeLongLong(signed=False, label="UInt64"), "remoteFqdn": SimTypePointer(SimTypeChar(label="Char"), offset=0), "numAddresses": SimTypeInt(signed=False, label="UInt32"), "remoteAddresses": SimTypePointer(SimStruct({"ipVersion": SimTypeInt(signed=False, label="FWP_IP_VERSION"), "Anonymous": SimUnion({"v4Address": SimTypeInt(signed=False, label="UInt32"), "v6Address": SimTypeFixedSizeArray(SimTypeChar(label="Byte"), 16)}, name="<anon>", label="None")}, name="IPSEC_TUNNEL_ENDPOINT0", pack=False, align=None), offset=0)}, name="IPSEC_TUNNEL_ENDPOINTS2", pack=False, align=None), "saIdleTimeout": SimStruct({"idleTimeoutSeconds": SimTypeInt(signed=False, label="UInt32"), "idleTimeoutSecondsFailOver": SimTypeInt(signed=False, label="UInt32")}, name="IPSEC_SA_IDLE_TIMEOUT0", pack=False, align=None), "emPolicy": SimTypePointer(SimStruct({"numAuthenticationMethods": SimTypeInt(signed=False, label="UInt32"), "authenticationMethods": SimTypePointer(SimStruct({"authenticationMethodType": SimTypeInt(signed=False, label="IKEEXT_AUTHENTICATION_METHOD_TYPE"), "Anonymous": SimUnion({"presharedKeyAuthentication": SimStruct({"presharedKey": SimStruct({"size": SimTypeInt(signed=False, label="UInt32"), "data": SimTypePointer(SimTypeChar(label="Byte"), offset=0)}, name="FWP_BYTE_BLOB", pack=False, align=None), "flags": SimTypeInt(signed=False, label="IKEEXT_PRESHARED_KEY_AUTHENTICATION_FLAGS")}, name="IKEEXT_PRESHARED_KEY_AUTHENTICATION1", pack=False, align=None), "certificateAuthentication": SimStruct({"inboundConfigType": SimTypeInt(signed=False, label="IKEEXT_CERT_CONFIG_TYPE"), "Anonymous1": SimUnion({"Anonymous1": SimStruct({"inboundRootArraySize": SimTypeInt(signed=False, label="UInt32"), "inboundRootCriteria": SimTypePointer(SimStruct({"certData": SimStruct({"size": SimTypeInt(signed=False, label="UInt32"), "data": SimTypePointer(SimTypeChar(label="Byte"), offset=0)}, name="FWP_BYTE_BLOB", pack=False, align=None), "certHash": SimStruct({"size": SimTypeInt(signed=False, label="UInt32"), "data": SimTypePointer(SimTypeChar(label="Byte"), offset=0)}, name="FWP_BYTE_BLOB", pack=False, align=None), "eku": SimTypePointer(SimStruct({"numEku": SimTypeInt(signed=False, label="UInt32"), "eku": SimTypePointer(SimTypePointer(SimTypeChar(label="Byte"), offset=0), offset=0)}, name="IKEEXT_CERT_EKUS0", pack=False, align=None), offset=0), "name": SimTypePointer(SimStruct({"nameType": SimTypeInt(signed=False, label="IKEEXT_CERT_CRITERIA_NAME_TYPE"), "certName": SimTypePointer(SimTypeChar(label="Char"), offset=0)}, name="IKEEXT_CERT_NAME0", pack=False, align=None), offset=0), "flags": SimTypeInt(signed=False, label="UInt32")}, name="IKEEXT_CERTIFICATE_CRITERIA0", pack=False, align=None), offset=0)}, name="_Anonymous1_e__Struct", pack=False, align=None), "Anonymous2": SimStruct({"inboundEnterpriseStoreArraySize": SimTypeInt(signed=False, label="UInt32"), "inboundEnterpriseStoreCriteria": SimTypePointer(SimStruct({"certData": SimStruct({"size": SimTypeInt(signed=False, label="UInt32"), "data": SimTypePointer(SimTypeChar(label="Byte"), offset=0)}, name="FWP_BYTE_BLOB", pack=False, align=None), "certHash": SimStruct({"size": SimTypeInt(signed=False, label="UInt32"), "data": SimTypePointer(SimTypeChar(label="Byte"), offset=0)}, name="FWP_BYTE_BLOB", pack=False, align=None), "eku": SimTypePointer(SimStruct({"numEku": SimTypeInt(signed=False, label="UInt32"), "eku": SimTypePointer(SimTypePointer(SimTypeChar(label="Byte"), offset=0), offset=0)}, name="IKEEXT_CERT_EKUS0", pack=False, align=None), offset=0), "name": SimTypePointer(SimStruct({"nameType": SimTypeInt(signed=False, label="IKEEXT_CERT_CRITERIA_NAME_TYPE"), "certName": SimTypePointer(SimTypeChar(label="Char"), offset=0)}, name="IKEEXT_CERT_NAME0", pack=False, align=None), offset=0), "flags": SimTypeInt(signed=False, label="UInt32")}, name="IKEEXT_CERTIFICATE_CRITERIA0", pack=False, align=None), offset=0)}, name="_Anonymous2_e__Struct", pack=False, align=None), "Anonymous3": SimStruct({"inboundRootStoreArraySize": SimTypeInt(signed=False, label="UInt32"), "inboundTrustedRootStoreCriteria": SimTypePointer(SimStruct({"certData": SimStruct({"size": SimTypeInt(signed=False, label="UInt32"), "data": SimTypePointer(SimTypeChar(label="Byte"), offset=0)}, name="FWP_BYTE_BLOB", pack=False, align=None), "certHash": SimStruct({"size": SimTypeInt(signed=False, label="UInt32"), "data": SimTypePointer(SimTypeChar(label="Byte"), offset=0)}, name="FWP_BYTE_BLOB", pack=False, align=None), "eku": SimTypePointer(SimStruct({"numEku": SimTypeInt(signed=False, label="UInt32"), "eku": SimTypePointer(SimTypePointer(SimTypeChar(label="Byte"), offset=0), offset=0)}, name="IKEEXT_CERT_EKUS0", pack=False, align=None), offset=0), "name": SimTypePointer(SimStruct({"nameType": SimTypeInt(signed=False, label="IKEEXT_CERT_CRITERIA_NAME_TYPE"), "certName": SimTypePointer(SimTypeChar(label="Char"), offset=0)}, name="IKEEXT_CERT_NAME0", pack=False, align=None), offset=0), "flags": SimTypeInt(signed=False, label="UInt32")}, name="IKEEXT_CERTIFICATE_CRITERIA0", pack=False, align=None), offset=0)}, name="_Anonymous3_e__Struct", pack=False, align=None)}, name="<anon>", label="None"), "outboundConfigType": SimTypeInt(signed=False, label="IKEEXT_CERT_CONFIG_TYPE"), "Anonymous2": SimUnion({"Anonymous1": SimStruct({"outboundRootArraySize": SimTypeInt(signed=False, label="UInt32"), "outboundRootCriteria": SimTypePointer(SimStruct({"certData": SimStruct({"size": SimTypeInt(signed=False, label="UInt32"), "data": SimTypePointer(SimTypeChar(label="Byte"), offset=0)}, name="FWP_BYTE_BLOB", pack=False, align=None), "certHash": SimStruct({"size": SimTypeInt(signed=False, label="UInt32"), "data": SimTypePointer(SimTypeChar(label="Byte"), offset=0)}, name="FWP_BYTE_BLOB", pack=False, align=None), "eku": SimTypePointer(SimStruct({"numEku": SimTypeInt(signed=False, label="UInt32"), "eku": SimTypePointer(SimTypePointer(SimTypeChar(label="Byte"), offset=0), offset=0)}, name="IKEEXT_CERT_EKUS0", pack=False, align=None), offset=0), "name": SimTypePointer(SimStruct({"nameType": SimTypeInt(signed=False, label="IKEEXT_CERT_CRITERIA_NAME_TYPE"), "certName": SimTypePointer(SimTypeChar(label="Char"), offset=0)}, name="IKEEXT_CERT_NAME0", pack=False, align=None), offset=0), "flags": SimTypeInt(signed=False, label="UInt32")}, name="IKEEXT_CERTIFICATE_CRITERIA0", pack=False, align=None), offset=0)}, name="_Anonymous1_e__Struct", pack=False, align=None), "Anonymous2": SimStruct({"outboundEnterpriseStoreArraySize": SimTypeInt(signed=False, label="UInt32"), "outboundEnterpriseStoreCriteria": SimTypePointer(SimStruct({"certData": SimStruct({"size": SimTypeInt(signed=False, label="UInt32"), "data": SimTypePointer(SimTypeChar(label="Byte"), offset=0)}, name="FWP_BYTE_BLOB", pack=False, align=None), "certHash": SimStruct({"size": SimTypeInt(signed=False, label="UInt32"), "data": SimTypePointer(SimTypeChar(label="Byte"), offset=0)}, name="FWP_BYTE_BLOB", pack=False, align=None), "eku": SimTypePointer(SimStruct({"numEku": SimTypeInt(signed=False, label="UInt32"), "eku": SimTypePointer(SimTypePointer(SimTypeChar(label="Byte"), offset=0), offset=0)}, name="IKEEXT_CERT_EKUS0", pack=False, align=None), offset=0), "name": SimTypePointer(SimStruct({"nameType": SimTypeInt(signed=False, label="IKEEXT_CERT_CRITERIA_NAME_TYPE"), "certName": SimTypePointer(SimTypeChar(label="Char"), offset=0)}, name="IKEEXT_CERT_NAME0", pack=False, align=None), offset=0), "flags": SimTypeInt(signed=False, label="UInt32")}, name="IKEEXT_CERTIFICATE_CRITERIA0", pack=False, align=None), offset=0)}, name="_Anonymous2_e__Struct", pack=False, align=None), "Anonymous3": SimStruct({"outboundRootStoreArraySize": SimTypeInt(signed=False, label="UInt32"), "outboundTrustedRootStoreCriteria": SimTypePointer(SimStruct({"certData": SimStruct({"size": SimTypeInt(signed=False, label="UInt32"), "data": SimTypePointer(SimTypeChar(label="Byte"), offset=0)}, name="FWP_BYTE_BLOB", pack=False, align=None), "certHash": SimStruct({"size": SimTypeInt(signed=False, label="UInt32"), "data": SimTypePointer(SimTypeChar(label="Byte"), offset=0)}, name="FWP_BYTE_BLOB", pack=False, align=None), "eku": SimTypePointer(SimStruct({"numEku": SimTypeInt(signed=False, label="UInt32"), "eku": SimTypePointer(SimTypePointer(SimTypeChar(label="Byte"), offset=0), offset=0)}, name="IKEEXT_CERT_EKUS0", pack=False, align=None), offset=0), "name": SimTypePointer(SimStruct({"nameType": SimTypeInt(signed=False, label="IKEEXT_CERT_CRITERIA_NAME_TYPE"), "certName": SimTypePointer(SimTypeChar(label="Char"), offset=0)}, name="IKEEXT_CERT_NAME0", pack=False, align=None), offset=0), "flags": SimTypeInt(signed=False, label="UInt32")}, name="IKEEXT_CERTIFICATE_CRITERIA0", pack=False, align=None), offset=0)}, name="_Anonymous3_e__Struct", pack=False, align=None)}, name="<anon>", label="None"), "flags": SimTypeInt(signed=False, label="IKEEXT_CERT_AUTH"), "localCertLocationUrl": SimStruct({"size": SimTypeInt(signed=False, label="UInt32"), "data": SimTypePointer(SimTypeChar(label="Byte"), offset=0)}, name="FWP_BYTE_BLOB", pack=False, align=None)}, name="IKEEXT_CERTIFICATE_AUTHENTICATION2", pack=False, align=None), "kerberosAuthentication": SimStruct({"flags": SimTypeInt(signed=False, label="IKEEXT_KERBEROS_AUTHENTICATION_FLAGS"), "proxyServer": SimTypePointer(SimTypeChar(label="Char"), offset=0)}, name="IKEEXT_KERBEROS_AUTHENTICATION1", pack=False, align=None), "reservedAuthentication": SimStruct({"flags": SimTypeInt(signed=False, label="IKEEXT_RESERVED_AUTHENTICATION_FLAGS")}, name="IKEEXT_RESERVED_AUTHENTICATION0", pack=False, align=None), "ntlmV2Authentication": SimStruct({"flags": SimTypeInt(signed=False, label="UInt32")}, name="IKEEXT_NTLM_V2_AUTHENTICATION0", pack=False, align=None), "sslAuthentication": SimStruct({"inboundConfigType": SimTypeInt(signed=False, label="IKEEXT_CERT_CONFIG_TYPE"), "Anonymous1": SimUnion({"Anonymous1": SimStruct({"inboundRootArraySize": SimTypeInt(signed=False, label="UInt32"), "inboundRootCriteria": SimTypePointer(SimStruct({"certData": SimStruct({"size": SimTypeInt(signed=False, label="UInt32"), "data": SimTypePointer(SimTypeChar(label="Byte"), offset=0)}, name="FWP_BYTE_BLOB", pack=False, align=None), "certHash": SimStruct({"size": SimTypeInt(signed=False, label="UInt32"), "data": SimTypePointer(SimTypeChar(label="Byte"), offset=0)}, name="FWP_BYTE_BLOB", pack=False, align=None), "eku": SimTypePointer(SimStruct({"numEku": SimTypeInt(signed=False, label="UInt32"), "eku": SimTypePointer(SimTypePointer(SimTypeChar(label="Byte"), offset=0), offset=0)}, name="IKEEXT_CERT_EKUS0", pack=False, align=None), offset=0), "name": SimTypePointer(SimStruct({"nameType": SimTypeInt(signed=False, label="IKEEXT_CERT_CRITERIA_NAME_TYPE"), "certName": SimTypePointer(SimTypeChar(label="Char"), offset=0)}, name="IKEEXT_CERT_NAME0", pack=False, align=None), offset=0), "flags": SimTypeInt(signed=False, label="UInt32")}, name="IKEEXT_CERTIFICATE_CRITERIA0", pack=False, align=None), offset=0)}, name="_Anonymous1_e__Struct", pack=False, align=None), "Anonymous2": SimStruct({"inboundEnterpriseStoreArraySize": SimTypeInt(signed=False, label="UInt32"), "inboundEnterpriseStoreCriteria": SimTypePointer(SimStruct({"certData": SimStruct({"size": SimTypeInt(signed=False, label="UInt32"), "data": SimTypePointer(SimTypeChar(label="Byte"), offset=0)}, name="FWP_BYTE_BLOB", pack=False, align=None), "certHash": SimStruct({"size": SimTypeInt(signed=False, label="UInt32"), "data": SimTypePointer(SimTypeChar(label="Byte"), offset=0)}, name="FWP_BYTE_BLOB", pack=False, align=None), "eku": SimTypePointer(SimStruct({"numEku": SimTypeInt(signed=False, label="UInt32"), "eku": SimTypePointer(SimTypePointer(SimTypeChar(label="Byte"), offset=0), offset=0)}, name="IKEEXT_CERT_EKUS0", pack=False, align=None), offset=0), "name": SimTypePointer(SimStruct({"nameType": SimTypeInt(signed=False, label="IKEEXT_CERT_CRITERIA_NAME_TYPE"), "certName": SimTypePointer(SimTypeChar(label="Char"), offset=0)}, name="IKEEXT_CERT_NAME0", pack=False, align=None), offset=0), "flags": SimTypeInt(signed=False, label="UInt32")}, name="IKEEXT_CERTIFICATE_CRITERIA0", pack=False, align=None), offset=0)}, name="_Anonymous2_e__Struct", pack=False, align=None), "Anonymous3": SimStruct({"inboundRootStoreArraySize": SimTypeInt(signed=False, label="UInt32"), "inboundTrustedRootStoreCriteria": SimTypePointer(SimStruct({"certData": SimStruct({"size": SimTypeInt(signed=False, label="UInt32"), "data": SimTypePointer(SimTypeChar(label="Byte"), offset=0)}, name="FWP_BYTE_BLOB", pack=False, align=None), "certHash": SimStruct({"size": SimTypeInt(signed=False, label="UInt32"), "data": SimTypePointer(SimTypeChar(label="Byte"), offset=0)}, name="FWP_BYTE_BLOB", pack=False, align=None), "eku": SimTypePointer(SimStruct({"numEku": SimTypeInt(signed=False, label="UInt32"), "eku": SimTypePointer(SimTypePointer(SimTypeChar(label="Byte"), offset=0), offset=0)}, name="IKEEXT_CERT_EKUS0", pack=False, align=None), offset=0), "name": SimTypePointer(SimStruct({"nameType": SimTypeInt(signed=False, label="IKEEXT_CERT_CRITERIA_NAME_TYPE"), "certName": SimTypePointer(SimTypeChar(label="Char"), offset=0)}, name="IKEEXT_CERT_NAME0", pack=False, align=None), offset=0), "flags": SimTypeInt(signed=False, label="UInt32")}, name="IKEEXT_CERTIFICATE_CRITERIA0", pack=False, align=None), offset=0)}, name="_Anonymous3_e__Struct", pack=False, align=None)}, name="<anon>", label="None"), "outboundConfigType": SimTypeInt(signed=False, label="IKEEXT_CERT_CONFIG_TYPE"), "Anonymous2": SimUnion({"Anonymous1": SimStruct({"outboundRootArraySize": SimTypeInt(signed=False, label="UInt32"), "outboundRootCriteria": SimTypePointer(SimStruct({"certData": SimStruct({"size": SimTypeInt(signed=False, label="UInt32"), "data": SimTypePointer(SimTypeChar(label="Byte"), offset=0)}, name="FWP_BYTE_BLOB", pack=False, align=None), "certHash": SimStruct({"size": SimTypeInt(signed=False, label="UInt32"), "data": SimTypePointer(SimTypeChar(label="Byte"), offset=0)}, name="FWP_BYTE_BLOB", pack=False, align=None), "eku": SimTypePointer(SimStruct({"numEku": SimTypeInt(signed=False, label="UInt32"), "eku": SimTypePointer(SimTypePointer(SimTypeChar(label="Byte"), offset=0), offset=0)}, name="IKEEXT_CERT_EKUS0", pack=False, align=None), offset=0), "name": SimTypePointer(SimStruct({"nameType": SimTypeInt(signed=False, label="IKEEXT_CERT_CRITERIA_NAME_TYPE"), "certName": SimTypePointer(SimTypeChar(label="Char"), offset=0)}, name="IKEEXT_CERT_NAME0", pack=False, align=None), offset=0), "flags": SimTypeInt(signed=False, label="UInt32")}, name="IKEEXT_CERTIFICATE_CRITERIA0", pack=False, align=None), offset=0)}, name="_Anonymous1_e__Struct", pack=False, align=None), "Anonymous2": SimStruct({"outboundEnterpriseStoreArraySize": SimTypeInt(signed=False, label="UInt32"), "outboundEnterpriseStoreCriteria": SimTypePointer(SimStruct({"certData": SimStruct({"size": SimTypeInt(signed=False, label="UInt32"), "data": SimTypePointer(SimTypeChar(label="Byte"), offset=0)}, name="FWP_BYTE_BLOB", pack=False, align=None), "certHash": SimStruct({"size": SimTypeInt(signed=False, label="UInt32"), "data": SimTypePointer(SimTypeChar(label="Byte"), offset=0)}, name="FWP_BYTE_BLOB", pack=False, align=None), "eku": SimTypePointer(SimStruct({"numEku": SimTypeInt(signed=False, label="UInt32"), "eku": SimTypePointer(SimTypePointer(SimTypeChar(label="Byte"), offset=0), offset=0)}, name="IKEEXT_CERT_EKUS0", pack=False, align=None), offset=0), "name": SimTypePointer(SimStruct({"nameType": SimTypeInt(signed=False, label="IKEEXT_CERT_CRITERIA_NAME_TYPE"), "certName": SimTypePointer(SimTypeChar(label="Char"), offset=0)}, name="IKEEXT_CERT_NAME0", pack=False, align=None), offset=0), "flags": SimTypeInt(signed=False, label="UInt32")}, name="IKEEXT_CERTIFICATE_CRITERIA0", pack=False, align=None), offset=0)}, name="_Anonymous2_e__Struct", pack=False, align=None), "Anonymous3": SimStruct({"outboundRootStoreArraySize": SimTypeInt(signed=False, label="UInt32"), "outboundTrustedRootStoreCriteria": SimTypePointer(SimStruct({"certData": SimStruct({"size": SimTypeInt(signed=False, label="UInt32"), "data": SimTypePointer(SimTypeChar(label="Byte"), offset=0)}, name="FWP_BYTE_BLOB", pack=False, align=None), "certHash": SimStruct({"size": SimTypeInt(signed=False, label="UInt32"), "data": SimTypePointer(SimTypeChar(label="Byte"), offset=0)}, name="FWP_BYTE_BLOB", pack=False, align=None), "eku": SimTypePointer(SimStruct({"numEku": SimTypeInt(signed=False, label="UInt32"), "eku": SimTypePointer(SimTypePointer(SimTypeChar(label="Byte"), offset=0), offset=0)}, name="IKEEXT_CERT_EKUS0", pack=False, align=None), offset=0), "name": SimTypePointer(SimStruct({"nameType": SimTypeInt(signed=False, label="IKEEXT_CERT_CRITERIA_NAME_TYPE"), "certName": SimTypePointer(SimTypeChar(label="Char"), offset=0)}, name="IKEEXT_CERT_NAME0", pack=False, align=None), offset=0), "flags": SimTypeInt(signed=False, label="UInt32")}, name="IKEEXT_CERTIFICATE_CRITERIA0", pack=False, align=None), offset=0)}, name="_Anonymous3_e__Struct", pack=False, align=None)}, name="<anon>", label="None"), "flags": SimTypeInt(signed=False, label="IKEEXT_CERT_AUTH"), "localCertLocationUrl": SimStruct({"size": SimTypeInt(signed=False, label="UInt32"), "data": SimTypePointer(SimTypeChar(label="Byte"), offset=0)}, name="FWP_BYTE_BLOB", pack=False, align=None)}, name="IKEEXT_CERTIFICATE_AUTHENTICATION2", pack=False, align=None), "cgaAuthentication": SimStruct({"keyContainerName": SimTypePointer(SimTypeChar(label="Char"), offset=0), "cspName": SimTypePointer(SimTypeChar(label="Char"), offset=0), "cspType": SimTypeInt(signed=False, label="UInt32"), "cgaModifier": SimStruct({"byteArray16": SimTypeFixedSizeArray(SimTypeChar(label="Byte"), 16)}, name="FWP_BYTE_ARRAY16", pack=False, align=None), "cgaCollisionCount": SimTypeChar(label="Byte")}, name="IKEEXT_IPV6_CGA_AUTHENTICATION0", pack=False, align=None), "eapAuthentication": SimStruct({"flags": SimTypeInt(signed=False, label="IKEEXT_EAP_AUTHENTICATION_FLAGS")}, name="IKEEXT_EAP_AUTHENTICATION0", pack=False, align=None)}, name="<anon>", label="None")}, name="IKEEXT_AUTHENTICATION_METHOD2", pack=False, align=None), offset=0), "initiatorImpersonationType": SimTypeInt(signed=False, label="IKEEXT_AUTHENTICATION_IMPERSONATION_TYPE")}, name="IKEEXT_EM_POLICY2", pack=False, align=None), offset=0), "fwdPathSaLifetime": SimTypeInt(signed=False, label="UInt32"), "compartmentId": SimTypeInt(signed=False, label="UInt32"), "numTrafficSelectorPolicy": SimTypeInt(signed=False, label="UInt32"), "trafficSelectorPolicies": SimTypePointer(SimStruct({"flags": SimTypeInt(signed=False, label="UInt32"), "numLocalTrafficSelectors": SimTypeInt(signed=False, label="UInt32"), "localTrafficSelectors": SimTypePointer(SimStruct({"protocolId": SimTypeChar(label="Byte"), "portStart": SimTypeShort(signed=False, label="UInt16"), "portEnd": SimTypeShort(signed=False, label="UInt16"), "ipVersion": SimTypeInt(signed=False, label="FWP_IP_VERSION"), "Anonymous1": SimUnion({"startV4Address": SimTypeInt(signed=False, label="UInt32"), "startV6Address": SimTypeFixedSizeArray(SimTypeChar(label="Byte"), 16)}, name="<anon>", label="None"), "Anonymous2": SimUnion({"endV4Address": SimTypeInt(signed=False, label="UInt32"), "endV6Address": SimTypeFixedSizeArray(SimTypeChar(label="Byte"), 16)}, name="<anon>", label="None")}, name="IPSEC_TRAFFIC_SELECTOR0_", pack=False, align=None), offset=0), "numRemoteTrafficSelectors": SimTypeInt(signed=False, label="UInt32"), "remoteTrafficSelectors": SimTypePointer(SimStruct({"protocolId": SimTypeChar(label="Byte"), "portStart": SimTypeShort(signed=False, label="UInt16"), "portEnd": SimTypeShort(signed=False, label="UInt16"), "ipVersion": SimTypeInt(signed=False, label="FWP_IP_VERSION"), "Anonymous1": SimUnion({"startV4Address": SimTypeInt(signed=False, label="UInt32"), "startV6Address": SimTypeFixedSizeArray(SimTypeChar(label="Byte"), 16)}, name="<anon>", label="None"), "Anonymous2": SimUnion({"endV4Address": SimTypeInt(signed=False, label="UInt32"), "endV6Address": SimTypeFixedSizeArray(SimTypeChar(label="Byte"), 16)}, name="<anon>", label="None")}, name="IPSEC_TRAFFIC_SELECTOR0_", pack=False, align=None), offset=0)}, name="IPSEC_TRAFFIC_SELECTOR_POLICY0_", pack=False, align=None), offset=0)}, name="IPSEC_TUNNEL_POLICY3_", pack=False, align=None), offset=0), "ikeMmPolicy": SimTypePointer(SimStruct({"softExpirationTime": SimTypeInt(signed=False, label="UInt32"), "numAuthenticationMethods": SimTypeInt(signed=False, label="UInt32"), "authenticationMethods": SimTypePointer(SimStruct({"authenticationMethodType": SimTypeInt(signed=False, label="IKEEXT_AUTHENTICATION_METHOD_TYPE"), "Anonymous": SimUnion({"presharedKeyAuthentication": SimStruct({"presharedKey": SimStruct({"size": SimTypeInt(signed=False, label="UInt32"), "data": SimTypePointer(SimTypeChar(label="Byte"), offset=0)}, name="FWP_BYTE_BLOB", pack=False, align=None), "flags": SimTypeInt(signed=False, label="IKEEXT_PRESHARED_KEY_AUTHENTICATION_FLAGS")}, name="IKEEXT_PRESHARED_KEY_AUTHENTICATION1", pack=False, align=None), "certificateAuthentication": SimStruct({"inboundConfigType": SimTypeInt(signed=False, label="IKEEXT_CERT_CONFIG_TYPE"), "Anonymous1": SimUnion({"Anonymous1": SimStruct({"inboundRootArraySize": SimTypeInt(signed=False, label="UInt32"), "inboundRootCriteria": SimTypePointer(SimStruct({"certData": SimStruct({"size": SimTypeInt(signed=False, label="UInt32"), "data": SimTypePointer(SimTypeChar(label="Byte"), offset=0)}, name="FWP_BYTE_BLOB", pack=False, align=None), "certHash": SimStruct({"size": SimTypeInt(signed=False, label="UInt32"), "data": SimTypePointer(SimTypeChar(label="Byte"), offset=0)}, name="FWP_BYTE_BLOB", pack=False, align=None), "eku": SimTypePointer(SimStruct({"numEku": SimTypeInt(signed=False, label="UInt32"), "eku": SimTypePointer(SimTypePointer(SimTypeChar(label="Byte"), offset=0), offset=0)}, name="IKEEXT_CERT_EKUS0", pack=False, align=None), offset=0), "name": SimTypePointer(SimStruct({"nameType": SimTypeInt(signed=False, label="IKEEXT_CERT_CRITERIA_NAME_TYPE"), "certName": SimTypePointer(SimTypeChar(label="Char"), offset=0)}, name="IKEEXT_CERT_NAME0", pack=False, align=None), offset=0), "flags": SimTypeInt(signed=False, label="UInt32")}, name="IKEEXT_CERTIFICATE_CRITERIA0", pack=False, align=None), offset=0)}, name="_Anonymous1_e__Struct", pack=False, align=None), "Anonymous2": SimStruct({"inboundEnterpriseStoreArraySize": SimTypeInt(signed=False, label="UInt32"), "inboundEnterpriseStoreCriteria": SimTypePointer(SimStruct({"certData": SimStruct({"size": SimTypeInt(signed=False, label="UInt32"), "data": SimTypePointer(SimTypeChar(label="Byte"), offset=0)}, name="FWP_BYTE_BLOB", pack=False, align=None), "certHash": SimStruct({"size": SimTypeInt(signed=False, label="UInt32"), "data": SimTypePointer(SimTypeChar(label="Byte"), offset=0)}, name="FWP_BYTE_BLOB", pack=False, align=None), "eku": SimTypePointer(SimStruct({"numEku": SimTypeInt(signed=False, label="UInt32"), "eku": SimTypePointer(SimTypePointer(SimTypeChar(label="Byte"), offset=0), offset=0)}, name="IKEEXT_CERT_EKUS0", pack=False, align=None), offset=0), "name": SimTypePointer(SimStruct({"nameType": SimTypeInt(signed=False, label="IKEEXT_CERT_CRITERIA_NAME_TYPE"), "certName": SimTypePointer(SimTypeChar(label="Char"), offset=0)}, name="IKEEXT_CERT_NAME0", pack=False, align=None), offset=0), "flags": SimTypeInt(signed=False, label="UInt32")}, name="IKEEXT_CERTIFICATE_CRITERIA0", pack=False, align=None), offset=0)}, name="_Anonymous2_e__Struct", pack=False, align=None), "Anonymous3": SimStruct({"inboundRootStoreArraySize": SimTypeInt(signed=False, label="UInt32"), "inboundTrustedRootStoreCriteria": SimTypePointer(SimStruct({"certData": SimStruct({"size": SimTypeInt(signed=False, label="UInt32"), "data": SimTypePointer(SimTypeChar(label="Byte"), offset=0)}, name="FWP_BYTE_BLOB", pack=False, align=None), "certHash": SimStruct({"size": SimTypeInt(signed=False, label="UInt32"), "data": SimTypePointer(SimTypeChar(label="Byte"), offset=0)}, name="FWP_BYTE_BLOB", pack=False, align=None), "eku": SimTypePointer(SimStruct({"numEku": SimTypeInt(signed=False, label="UInt32"), "eku": SimTypePointer(SimTypePointer(SimTypeChar(label="Byte"), offset=0), offset=0)}, name="IKEEXT_CERT_EKUS0", pack=False, align=None), offset=0), "name": SimTypePointer(SimStruct({"nameType": SimTypeInt(signed=False, label="IKEEXT_CERT_CRITERIA_NAME_TYPE"), "certName": SimTypePointer(SimTypeChar(label="Char"), offset=0)}, name="IKEEXT_CERT_NAME0", pack=False, align=None), offset=0), "flags": SimTypeInt(signed=False, label="UInt32")}, name="IKEEXT_CERTIFICATE_CRITERIA0", pack=False, align=None), offset=0)}, name="_Anonymous3_e__Struct", pack=False, align=None)}, name="<anon>", label="None"), "outboundConfigType": SimTypeInt(signed=False, label="IKEEXT_CERT_CONFIG_TYPE"), "Anonymous2": SimUnion({"Anonymous1": SimStruct({"outboundRootArraySize": SimTypeInt(signed=False, label="UInt32"), "outboundRootCriteria": SimTypePointer(SimStruct({"certData": SimStruct({"size": SimTypeInt(signed=False, label="UInt32"), "data": SimTypePointer(SimTypeChar(label="Byte"), offset=0)}, name="FWP_BYTE_BLOB", pack=False, align=None), "certHash": SimStruct({"size": SimTypeInt(signed=False, label="UInt32"), "data": SimTypePointer(SimTypeChar(label="Byte"), offset=0)}, name="FWP_BYTE_BLOB", pack=False, align=None), "eku": SimTypePointer(SimStruct({"numEku": SimTypeInt(signed=False, label="UInt32"), "eku": SimTypePointer(SimTypePointer(SimTypeChar(label="Byte"), offset=0), offset=0)}, name="IKEEXT_CERT_EKUS0", pack=False, align=None), offset=0), "name": SimTypePointer(SimStruct({"nameType": SimTypeInt(signed=False, label="IKEEXT_CERT_CRITERIA_NAME_TYPE"), "certName": SimTypePointer(SimTypeChar(label="Char"), offset=0)}, name="IKEEXT_CERT_NAME0", pack=False, align=None), offset=0), "flags": SimTypeInt(signed=False, label="UInt32")}, name="IKEEXT_CERTIFICATE_CRITERIA0", pack=False, align=None), offset=0)}, name="_Anonymous1_e__Struct", pack=False, align=None), "Anonymous2": SimStruct({"outboundEnterpriseStoreArraySize": SimTypeInt(signed=False, label="UInt32"), "outboundEnterpriseStoreCriteria": SimTypePointer(SimStruct({"certData": SimStruct({"size": SimTypeInt(signed=False, label="UInt32"), "data": SimTypePointer(SimTypeChar(label="Byte"), offset=0)}, name="FWP_BYTE_BLOB", pack=False, align=None), "certHash": SimStruct({"size": SimTypeInt(signed=False, label="UInt32"), "data": SimTypePointer(SimTypeChar(label="Byte"), offset=0)}, name="FWP_BYTE_BLOB", pack=False, align=None), "eku": SimTypePointer(SimStruct({"numEku": SimTypeInt(signed=False, label="UInt32"), "eku": SimTypePointer(SimTypePointer(SimTypeChar(label="Byte"), offset=0), offset=0)}, name="IKEEXT_CERT_EKUS0", pack=False, align=None), offset=0), "name": SimTypePointer(SimStruct({"nameType": SimTypeInt(signed=False, label="IKEEXT_CERT_CRITERIA_NAME_TYPE"), "certName": SimTypePointer(SimTypeChar(label="Char"), offset=0)}, name="IKEEXT_CERT_NAME0", pack=False, align=None), offset=0), "flags": SimTypeInt(signed=False, label="UInt32")}, name="IKEEXT_CERTIFICATE_CRITERIA0", pack=False, align=None), offset=0)}, name="_Anonymous2_e__Struct", pack=False, align=None), "Anonymous3": SimStruct({"outboundRootStoreArraySize": SimTypeInt(signed=False, label="UInt32"), "outboundTrustedRootStoreCriteria": SimTypePointer(SimStruct({"certData": SimStruct({"size": SimTypeInt(signed=False, label="UInt32"), "data": SimTypePointer(SimTypeChar(label="Byte"), offset=0)}, name="FWP_BYTE_BLOB", pack=False, align=None), "certHash": SimStruct({"size": SimTypeInt(signed=False, label="UInt32"), "data": SimTypePointer(SimTypeChar(label="Byte"), offset=0)}, name="FWP_BYTE_BLOB", pack=False, align=None), "eku": SimTypePointer(SimStruct({"numEku": SimTypeInt(signed=False, label="UInt32"), "eku": SimTypePointer(SimTypePointer(SimTypeChar(label="Byte"), offset=0), offset=0)}, name="IKEEXT_CERT_EKUS0", pack=False, align=None), offset=0), "name": SimTypePointer(SimStruct({"nameType": SimTypeInt(signed=False, label="IKEEXT_CERT_CRITERIA_NAME_TYPE"), "certName": SimTypePointer(SimTypeChar(label="Char"), offset=0)}, name="IKEEXT_CERT_NAME0", pack=False, align=None), offset=0), "flags": SimTypeInt(signed=False, label="UInt32")}, name="IKEEXT_CERTIFICATE_CRITERIA0", pack=False, align=None), offset=0)}, name="_Anonymous3_e__Struct", pack=False, align=None)}, name="<anon>", label="None"), "flags": SimTypeInt(signed=False, label="IKEEXT_CERT_AUTH"), "localCertLocationUrl": SimStruct({"size": SimTypeInt(signed=False, label="UInt32"), "data": SimTypePointer(SimTypeChar(label="Byte"), offset=0)}, name="FWP_BYTE_BLOB", pack=False, align=None)}, name="IKEEXT_CERTIFICATE_AUTHENTICATION2", pack=False, align=None), "kerberosAuthentication": SimStruct({"flags": SimTypeInt(signed=False, label="IKEEXT_KERBEROS_AUTHENTICATION_FLAGS"), "proxyServer": SimTypePointer(SimTypeChar(label="Char"), offset=0)}, name="IKEEXT_KERBEROS_AUTHENTICATION1", pack=False, align=None), "reservedAuthentication": SimStruct({"flags": SimTypeInt(signed=False, label="IKEEXT_RESERVED_AUTHENTICATION_FLAGS")}, name="IKEEXT_RESERVED_AUTHENTICATION0", pack=False, align=None), "ntlmV2Authentication": SimStruct({"flags": SimTypeInt(signed=False, label="UInt32")}, name="IKEEXT_NTLM_V2_AUTHENTICATION0", pack=False, align=None), "sslAuthentication": SimStruct({"inboundConfigType": SimTypeInt(signed=False, label="IKEEXT_CERT_CONFIG_TYPE"), "Anonymous1": SimUnion({"Anonymous1": SimStruct({"inboundRootArraySize": SimTypeInt(signed=False, label="UInt32"), "inboundRootCriteria": SimTypePointer(SimStruct({"certData": SimStruct({"size": SimTypeInt(signed=False, label="UInt32"), "data": SimTypePointer(SimTypeChar(label="Byte"), offset=0)}, name="FWP_BYTE_BLOB", pack=False, align=None), "certHash": SimStruct({"size": SimTypeInt(signed=False, label="UInt32"), "data": SimTypePointer(SimTypeChar(label="Byte"), offset=0)}, name="FWP_BYTE_BLOB", pack=False, align=None), "eku": SimTypePointer(SimStruct({"numEku": SimTypeInt(signed=False, label="UInt32"), "eku": SimTypePointer(SimTypePointer(SimTypeChar(label="Byte"), offset=0), offset=0)}, name="IKEEXT_CERT_EKUS0", pack=False, align=None), offset=0), "name": SimTypePointer(SimStruct({"nameType": SimTypeInt(signed=False, label="IKEEXT_CERT_CRITERIA_NAME_TYPE"), "certName": SimTypePointer(SimTypeChar(label="Char"), offset=0)}, name="IKEEXT_CERT_NAME0", pack=False, align=None), offset=0), "flags": SimTypeInt(signed=False, label="UInt32")}, name="IKEEXT_CERTIFICATE_CRITERIA0", pack=False, align=None), offset=0)}, name="_Anonymous1_e__Struct", pack=False, align=None), "Anonymous2": SimStruct({"inboundEnterpriseStoreArraySize": SimTypeInt(signed=False, label="UInt32"), "inboundEnterpriseStoreCriteria": SimTypePointer(SimStruct({"certData": SimStruct({"size": SimTypeInt(signed=False, label="UInt32"), "data": SimTypePointer(SimTypeChar(label="Byte"), offset=0)}, name="FWP_BYTE_BLOB", pack=False, align=None), "certHash": SimStruct({"size": SimTypeInt(signed=False, label="UInt32"), "data": SimTypePointer(SimTypeChar(label="Byte"), offset=0)}, name="FWP_BYTE_BLOB", pack=False, align=None), "eku": SimTypePointer(SimStruct({"numEku": SimTypeInt(signed=False, label="UInt32"), "eku": SimTypePointer(SimTypePointer(SimTypeChar(label="Byte"), offset=0), offset=0)}, name="IKEEXT_CERT_EKUS0", pack=False, align=None), offset=0), "name": SimTypePointer(SimStruct({"nameType": SimTypeInt(signed=False, label="IKEEXT_CERT_CRITERIA_NAME_TYPE"), "certName": SimTypePointer(SimTypeChar(label="Char"), offset=0)}, name="IKEEXT_CERT_NAME0", pack=False, align=None), offset=0), "flags": SimTypeInt(signed=False, label="UInt32")}, name="IKEEXT_CERTIFICATE_CRITERIA0", pack=False, align=None), offset=0)}, name="_Anonymous2_e__Struct", pack=False, align=None), "Anonymous3": SimStruct({"inboundRootStoreArraySize": SimTypeInt(signed=False, label="UInt32"), "inboundTrustedRootStoreCriteria": SimTypePointer(SimStruct({"certData": SimStruct({"size": SimTypeInt(signed=False, label="UInt32"), "data": SimTypePointer(SimTypeChar(label="Byte"), offset=0)}, name="FWP_BYTE_BLOB", pack=False, align=None), "certHash": SimStruct({"size": SimTypeInt(signed=False, label="UInt32"), "data": SimTypePointer(SimTypeChar(label="Byte"), offset=0)}, name="FWP_BYTE_BLOB", pack=False, align=None), "eku": SimTypePointer(SimStruct({"numEku": SimTypeInt(signed=False, label="UInt32"), "eku": SimTypePointer(SimTypePointer(SimTypeChar(label="Byte"), offset=0), offset=0)}, name="IKEEXT_CERT_EKUS0", pack=False, align=None), offset=0), "name": SimTypePointer(SimStruct({"nameType": SimTypeInt(signed=False, label="IKEEXT_CERT_CRITERIA_NAME_TYPE"), "certName": SimTypePointer(SimTypeChar(label="Char"), offset=0)}, name="IKEEXT_CERT_NAME0", pack=False, align=None), offset=0), "flags": SimTypeInt(signed=False, label="UInt32")}, name="IKEEXT_CERTIFICATE_CRITERIA0", pack=False, align=None), offset=0)}, name="_Anonymous3_e__Struct", pack=False, align=None)}, name="<anon>", label="None"), "outboundConfigType": SimTypeInt(signed=False, label="IKEEXT_CERT_CONFIG_TYPE"), "Anonymous2": SimUnion({"Anonymous1": SimStruct({"outboundRootArraySize": SimTypeInt(signed=False, label="UInt32"), "outboundRootCriteria": SimTypePointer(SimStruct({"certData": SimStruct({"size": SimTypeInt(signed=False, label="UInt32"), "data": SimTypePointer(SimTypeChar(label="Byte"), offset=0)}, name="FWP_BYTE_BLOB", pack=False, align=None), "certHash": SimStruct({"size": SimTypeInt(signed=False, label="UInt32"), "data": SimTypePointer(SimTypeChar(label="Byte"), offset=0)}, name="FWP_BYTE_BLOB", pack=False, align=None), "eku": SimTypePointer(SimStruct({"numEku": SimTypeInt(signed=False, label="UInt32"), "eku": SimTypePointer(SimTypePointer(SimTypeChar(label="Byte"), offset=0), offset=0)}, name="IKEEXT_CERT_EKUS0", pack=False, align=None), offset=0), "name": SimTypePointer(SimStruct({"nameType": SimTypeInt(signed=False, label="IKEEXT_CERT_CRITERIA_NAME_TYPE"), "certName": SimTypePointer(SimTypeChar(label="Char"), offset=0)}, name="IKEEXT_CERT_NAME0", pack=False, align=None), offset=0), "flags": SimTypeInt(signed=False, label="UInt32")}, name="IKEEXT_CERTIFICATE_CRITERIA0", pack=False, align=None), offset=0)}, name="_Anonymous1_e__Struct", pack=False, align=None), "Anonymous2": SimStruct({"outboundEnterpriseStoreArraySize": SimTypeInt(signed=False, label="UInt32"), "outboundEnterpriseStoreCriteria": SimTypePointer(SimStruct({"certData": SimStruct({"size": SimTypeInt(signed=False, label="UInt32"), "data": SimTypePointer(SimTypeChar(label="Byte"), offset=0)}, name="FWP_BYTE_BLOB", pack=False, align=None), "certHash": SimStruct({"size": SimTypeInt(signed=False, label="UInt32"), "data": SimTypePointer(SimTypeChar(label="Byte"), offset=0)}, name="FWP_BYTE_BLOB", pack=False, align=None), "eku": SimTypePointer(SimStruct({"numEku": SimTypeInt(signed=False, label="UInt32"), "eku": SimTypePointer(SimTypePointer(SimTypeChar(label="Byte"), offset=0), offset=0)}, name="IKEEXT_CERT_EKUS0", pack=False, align=None), offset=0), "name": SimTypePointer(SimStruct({"nameType": SimTypeInt(signed=False, label="IKEEXT_CERT_CRITERIA_NAME_TYPE"), "certName": SimTypePointer(SimTypeChar(label="Char"), offset=0)}, name="IKEEXT_CERT_NAME0", pack=False, align=None), offset=0), "flags": SimTypeInt(signed=False, label="UInt32")}, name="IKEEXT_CERTIFICATE_CRITERIA0", pack=False, align=None), offset=0)}, name="_Anonymous2_e__Struct", pack=False, align=None), "Anonymous3": SimStruct({"outboundRootStoreArraySize": SimTypeInt(signed=False, label="UInt32"), "outboundTrustedRootStoreCriteria": SimTypePointer(SimStruct({"certData": SimStruct({"size": SimTypeInt(signed=False, label="UInt32"), "data": SimTypePointer(SimTypeChar(label="Byte"), offset=0)}, name="FWP_BYTE_BLOB", pack=False, align=None), "certHash": SimStruct({"size": SimTypeInt(signed=False, label="UInt32"), "data": SimTypePointer(SimTypeChar(label="Byte"), offset=0)}, name="FWP_BYTE_BLOB", pack=False, align=None), "eku": SimTypePointer(SimStruct({"numEku": SimTypeInt(signed=False, label="UInt32"), "eku": SimTypePointer(SimTypePointer(SimTypeChar(label="Byte"), offset=0), offset=0)}, name="IKEEXT_CERT_EKUS0", pack=False, align=None), offset=0), "name": SimTypePointer(SimStruct({"nameType": SimTypeInt(signed=False, label="IKEEXT_CERT_CRITERIA_NAME_TYPE"), "certName": SimTypePointer(SimTypeChar(label="Char"), offset=0)}, name="IKEEXT_CERT_NAME0", pack=False, align=None), offset=0), "flags": SimTypeInt(signed=False, label="UInt32")}, name="IKEEXT_CERTIFICATE_CRITERIA0", pack=False, align=None), offset=0)}, name="_Anonymous3_e__Struct", pack=False, align=None)}, name="<anon>", label="None"), "flags": SimTypeInt(signed=False, label="IKEEXT_CERT_AUTH"), "localCertLocationUrl": SimStruct({"size": SimTypeInt(signed=False, label="UInt32"), "data": SimTypePointer(SimTypeChar(label="Byte"), offset=0)}, name="FWP_BYTE_BLOB", pack=False, align=None)}, name="IKEEXT_CERTIFICATE_AUTHENTICATION2", pack=False, align=None), "cgaAuthentication": SimStruct({"keyContainerName": SimTypePointer(SimTypeChar(label="Char"), offset=0), "cspName": SimTypePointer(SimTypeChar(label="Char"), offset=0), "cspType": SimTypeInt(signed=False, label="UInt32"), "cgaModifier": SimStruct({"byteArray16": SimTypeFixedSizeArray(SimTypeChar(label="Byte"), 16)}, name="FWP_BYTE_ARRAY16", pack=False, align=None), "cgaCollisionCount": SimTypeChar(label="Byte")}, name="IKEEXT_IPV6_CGA_AUTHENTICATION0", pack=False, align=None), "eapAuthentication": SimStruct({"flags": SimTypeInt(signed=False, label="IKEEXT_EAP_AUTHENTICATION_FLAGS")}, name="IKEEXT_EAP_AUTHENTICATION0", pack=False, align=None)}, name="<anon>", label="None")}, name="IKEEXT_AUTHENTICATION_METHOD2", pack=False, align=None), offset=0), "initiatorImpersonationType": SimTypeInt(signed=False, label="IKEEXT_AUTHENTICATION_IMPERSONATION_TYPE"), "numIkeProposals": SimTypeInt(signed=False, label="UInt32"), "ikeProposals": SimTypePointer(SimStruct({"cipherAlgorithm": SimStruct({"algoIdentifier": SimTypeInt(signed=False, label="IKEEXT_CIPHER_TYPE"), "keyLen": SimTypeInt(signed=False, label="UInt32"), "rounds": SimTypeInt(signed=False, label="UInt32")}, name="IKEEXT_CIPHER_ALGORITHM0", pack=False, align=None), "integrityAlgorithm": SimStruct({"algoIdentifier": SimTypeInt(signed=False, label="IKEEXT_INTEGRITY_TYPE")}, name="IKEEXT_INTEGRITY_ALGORITHM0", pack=False, align=None), "maxLifetimeSeconds": SimTypeInt(signed=False, label="UInt32"), "dhGroup": SimTypeInt(signed=False, label="IKEEXT_DH_GROUP"), "quickModeLimit": SimTypeInt(signed=False, label="UInt32")}, name="IKEEXT_PROPOSAL0", pack=False, align=None), offset=0), "flags": SimTypeInt(signed=False, label="IKEEXT_POLICY_FLAG"), "maxDynamicFilters": SimTypeInt(signed=False, label="UInt32"), "retransmitDurationSecs": SimTypeInt(signed=False, label="UInt32")}, name="IKEEXT_POLICY2", pack=False, align=None), offset=0), "authIpMmPolicy": SimTypePointer(SimStruct({"softExpirationTime": SimTypeInt(signed=False, label="UInt32"), "numAuthenticationMethods": SimTypeInt(signed=False, label="UInt32"), "authenticationMethods": SimTypePointer(SimStruct({"authenticationMethodType": SimTypeInt(signed=False, label="IKEEXT_AUTHENTICATION_METHOD_TYPE"), "Anonymous": SimUnion({"presharedKeyAuthentication": SimStruct({"presharedKey": SimStruct({"size": SimTypeInt(signed=False, label="UInt32"), "data": SimTypePointer(SimTypeChar(label="Byte"), offset=0)}, name="FWP_BYTE_BLOB", pack=False, align=None), "flags": SimTypeInt(signed=False, label="IKEEXT_PRESHARED_KEY_AUTHENTICATION_FLAGS")}, name="IKEEXT_PRESHARED_KEY_AUTHENTICATION1", pack=False, align=None), "certificateAuthentication": SimStruct({"inboundConfigType": SimTypeInt(signed=False, label="IKEEXT_CERT_CONFIG_TYPE"), "Anonymous1": SimUnion({"Anonymous1": SimStruct({"inboundRootArraySize": SimTypeInt(signed=False, label="UInt32"), "inboundRootCriteria": SimTypePointer(SimStruct({"certData": SimStruct({"size": SimTypeInt(signed=False, label="UInt32"), "data": SimTypePointer(SimTypeChar(label="Byte"), offset=0)}, name="FWP_BYTE_BLOB", pack=False, align=None), "certHash": SimStruct({"size": SimTypeInt(signed=False, label="UInt32"), "data": SimTypePointer(SimTypeChar(label="Byte"), offset=0)}, name="FWP_BYTE_BLOB", pack=False, align=None), "eku": SimTypePointer(SimStruct({"numEku": SimTypeInt(signed=False, label="UInt32"), "eku": SimTypePointer(SimTypePointer(SimTypeChar(label="Byte"), offset=0), offset=0)}, name="IKEEXT_CERT_EKUS0", pack=False, align=None), offset=0), "name": SimTypePointer(SimStruct({"nameType": SimTypeInt(signed=False, label="IKEEXT_CERT_CRITERIA_NAME_TYPE"), "certName": SimTypePointer(SimTypeChar(label="Char"), offset=0)}, name="IKEEXT_CERT_NAME0", pack=False, align=None), offset=0), "flags": SimTypeInt(signed=False, label="UInt32")}, name="IKEEXT_CERTIFICATE_CRITERIA0", pack=False, align=None), offset=0)}, name="_Anonymous1_e__Struct", pack=False, align=None), "Anonymous2": SimStruct({"inboundEnterpriseStoreArraySize": SimTypeInt(signed=False, label="UInt32"), "inboundEnterpriseStoreCriteria": SimTypePointer(SimStruct({"certData": SimStruct({"size": SimTypeInt(signed=False, label="UInt32"), "data": SimTypePointer(SimTypeChar(label="Byte"), offset=0)}, name="FWP_BYTE_BLOB", pack=False, align=None), "certHash": SimStruct({"size": SimTypeInt(signed=False, label="UInt32"), "data": SimTypePointer(SimTypeChar(label="Byte"), offset=0)}, name="FWP_BYTE_BLOB", pack=False, align=None), "eku": SimTypePointer(SimStruct({"numEku": SimTypeInt(signed=False, label="UInt32"), "eku": SimTypePointer(SimTypePointer(SimTypeChar(label="Byte"), offset=0), offset=0)}, name="IKEEXT_CERT_EKUS0", pack=False, align=None), offset=0), "name": SimTypePointer(SimStruct({"nameType": SimTypeInt(signed=False, label="IKEEXT_CERT_CRITERIA_NAME_TYPE"), "certName": SimTypePointer(SimTypeChar(label="Char"), offset=0)}, name="IKEEXT_CERT_NAME0", pack=False, align=None), offset=0), "flags": SimTypeInt(signed=False, label="UInt32")}, name="IKEEXT_CERTIFICATE_CRITERIA0", pack=False, align=None), offset=0)}, name="_Anonymous2_e__Struct", pack=False, align=None), "Anonymous3": SimStruct({"inboundRootStoreArraySize": SimTypeInt(signed=False, label="UInt32"), "inboundTrustedRootStoreCriteria": SimTypePointer(SimStruct({"certData": SimStruct({"size": SimTypeInt(signed=False, label="UInt32"), "data": SimTypePointer(SimTypeChar(label="Byte"), offset=0)}, name="FWP_BYTE_BLOB", pack=False, align=None), "certHash": SimStruct({"size": SimTypeInt(signed=False, label="UInt32"), "data": SimTypePointer(SimTypeChar(label="Byte"), offset=0)}, name="FWP_BYTE_BLOB", pack=False, align=None), "eku": SimTypePointer(SimStruct({"numEku": SimTypeInt(signed=False, label="UInt32"), "eku": SimTypePointer(SimTypePointer(SimTypeChar(label="Byte"), offset=0), offset=0)}, name="IKEEXT_CERT_EKUS0", pack=False, align=None), offset=0), "name": SimTypePointer(SimStruct({"nameType": SimTypeInt(signed=False, label="IKEEXT_CERT_CRITERIA_NAME_TYPE"), "certName": SimTypePointer(SimTypeChar(label="Char"), offset=0)}, name="IKEEXT_CERT_NAME0", pack=False, align=None), offset=0), "flags": SimTypeInt(signed=False, label="UInt32")}, name="IKEEXT_CERTIFICATE_CRITERIA0", pack=False, align=None), offset=0)}, name="_Anonymous3_e__Struct", pack=False, align=None)}, name="<anon>", label="None"), "outboundConfigType": SimTypeInt(signed=False, label="IKEEXT_CERT_CONFIG_TYPE"), "Anonymous2": SimUnion({"Anonymous1": SimStruct({"outboundRootArraySize": SimTypeInt(signed=False, label="UInt32"), "outboundRootCriteria": SimTypePointer(SimStruct({"certData": SimStruct({"size": SimTypeInt(signed=False, label="UInt32"), "data": SimTypePointer(SimTypeChar(label="Byte"), offset=0)}, name="FWP_BYTE_BLOB", pack=False, align=None), "certHash": SimStruct({"size": SimTypeInt(signed=False, label="UInt32"), "data": SimTypePointer(SimTypeChar(label="Byte"), offset=0)}, name="FWP_BYTE_BLOB", pack=False, align=None), "eku": SimTypePointer(SimStruct({"numEku": SimTypeInt(signed=False, label="UInt32"), "eku": SimTypePointer(SimTypePointer(SimTypeChar(label="Byte"), offset=0), offset=0)}, name="IKEEXT_CERT_EKUS0", pack=False, align=None), offset=0), "name": SimTypePointer(SimStruct({"nameType": SimTypeInt(signed=False, label="IKEEXT_CERT_CRITERIA_NAME_TYPE"), "certName": SimTypePointer(SimTypeChar(label="Char"), offset=0)}, name="IKEEXT_CERT_NAME0", pack=False, align=None), offset=0), "flags": SimTypeInt(signed=False, label="UInt32")}, name="IKEEXT_CERTIFICATE_CRITERIA0", pack=False, align=None), offset=0)}, name="_Anonymous1_e__Struct", pack=False, align=None), "Anonymous2": SimStruct({"outboundEnterpriseStoreArraySize": SimTypeInt(signed=False, label="UInt32"), "outboundEnterpriseStoreCriteria": SimTypePointer(SimStruct({"certData": SimStruct({"size": SimTypeInt(signed=False, label="UInt32"), "data": SimTypePointer(SimTypeChar(label="Byte"), offset=0)}, name="FWP_BYTE_BLOB", pack=False, align=None), "certHash": SimStruct({"size": SimTypeInt(signed=False, label="UInt32"), "data": SimTypePointer(SimTypeChar(label="Byte"), offset=0)}, name="FWP_BYTE_BLOB", pack=False, align=None), "eku": SimTypePointer(SimStruct({"numEku": SimTypeInt(signed=False, label="UInt32"), "eku": SimTypePointer(SimTypePointer(SimTypeChar(label="Byte"), offset=0), offset=0)}, name="IKEEXT_CERT_EKUS0", pack=False, align=None), offset=0), "name": SimTypePointer(SimStruct({"nameType": SimTypeInt(signed=False, label="IKEEXT_CERT_CRITERIA_NAME_TYPE"), "certName": SimTypePointer(SimTypeChar(label="Char"), offset=0)}, name="IKEEXT_CERT_NAME0", pack=False, align=None), offset=0), "flags": SimTypeInt(signed=False, label="UInt32")}, name="IKEEXT_CERTIFICATE_CRITERIA0", pack=False, align=None), offset=0)}, name="_Anonymous2_e__Struct", pack=False, align=None), "Anonymous3": SimStruct({"outboundRootStoreArraySize": SimTypeInt(signed=False, label="UInt32"), "outboundTrustedRootStoreCriteria": SimTypePointer(SimStruct({"certData": SimStruct({"size": SimTypeInt(signed=False, label="UInt32"), "data": SimTypePointer(SimTypeChar(label="Byte"), offset=0)}, name="FWP_BYTE_BLOB", pack=False, align=None), "certHash": SimStruct({"size": SimTypeInt(signed=False, label="UInt32"), "data": SimTypePointer(SimTypeChar(label="Byte"), offset=0)}, name="FWP_BYTE_BLOB", pack=False, align=None), "eku": SimTypePointer(SimStruct({"numEku": SimTypeInt(signed=False, label="UInt32"), "eku": SimTypePointer(SimTypePointer(SimTypeChar(label="Byte"), offset=0), offset=0)}, name="IKEEXT_CERT_EKUS0", pack=False, align=None), offset=0), "name": SimTypePointer(SimStruct({"nameType": SimTypeInt(signed=False, label="IKEEXT_CERT_CRITERIA_NAME_TYPE"), "certName": SimTypePointer(SimTypeChar(label="Char"), offset=0)}, name="IKEEXT_CERT_NAME0", pack=False, align=None), offset=0), "flags": SimTypeInt(signed=False, label="UInt32")}, name="IKEEXT_CERTIFICATE_CRITERIA0", pack=False, align=None), offset=0)}, name="_Anonymous3_e__Struct", pack=False, align=None)}, name="<anon>", label="None"), "flags": SimTypeInt(signed=False, label="IKEEXT_CERT_AUTH"), "localCertLocationUrl": SimStruct({"size": SimTypeInt(signed=False, label="UInt32"), "data": SimTypePointer(SimTypeChar(label="Byte"), offset=0)}, name="FWP_BYTE_BLOB", pack=False, align=None)}, name="IKEEXT_CERTIFICATE_AUTHENTICATION2", pack=False, align=None), "kerberosAuthentication": SimStruct({"flags": SimTypeInt(signed=False, label="IKEEXT_KERBEROS_AUTHENTICATION_FLAGS"), "proxyServer": SimTypePointer(SimTypeChar(label="Char"), offset=0)}, name="IKEEXT_KERBEROS_AUTHENTICATION1", pack=False, align=None), "reservedAuthentication": SimStruct({"flags": SimTypeInt(signed=False, label="IKEEXT_RESERVED_AUTHENTICATION_FLAGS")}, name="IKEEXT_RESERVED_AUTHENTICATION0", pack=False, align=None), "ntlmV2Authentication": SimStruct({"flags": SimTypeInt(signed=False, label="UInt32")}, name="IKEEXT_NTLM_V2_AUTHENTICATION0", pack=False, align=None), "sslAuthentication": SimStruct({"inboundConfigType": SimTypeInt(signed=False, label="IKEEXT_CERT_CONFIG_TYPE"), "Anonymous1": SimUnion({"Anonymous1": SimStruct({"inboundRootArraySize": SimTypeInt(signed=False, label="UInt32"), "inboundRootCriteria": SimTypePointer(SimStruct({"certData": SimStruct({"size": SimTypeInt(signed=False, label="UInt32"), "data": SimTypePointer(SimTypeChar(label="Byte"), offset=0)}, name="FWP_BYTE_BLOB", pack=False, align=None), "certHash": SimStruct({"size": SimTypeInt(signed=False, label="UInt32"), "data": SimTypePointer(SimTypeChar(label="Byte"), offset=0)}, name="FWP_BYTE_BLOB", pack=False, align=None), "eku": SimTypePointer(SimStruct({"numEku": SimTypeInt(signed=False, label="UInt32"), "eku": SimTypePointer(SimTypePointer(SimTypeChar(label="Byte"), offset=0), offset=0)}, name="IKEEXT_CERT_EKUS0", pack=False, align=None), offset=0), "name": SimTypePointer(SimStruct({"nameType": SimTypeInt(signed=False, label="IKEEXT_CERT_CRITERIA_NAME_TYPE"), "certName": SimTypePointer(SimTypeChar(label="Char"), offset=0)}, name="IKEEXT_CERT_NAME0", pack=False, align=None), offset=0), "flags": SimTypeInt(signed=False, label="UInt32")}, name="IKEEXT_CERTIFICATE_CRITERIA0", pack=False, align=None), offset=0)}, name="_Anonymous1_e__Struct", pack=False, align=None), "Anonymous2": SimStruct({"inboundEnterpriseStoreArraySize": SimTypeInt(signed=False, label="UInt32"), "inboundEnterpriseStoreCriteria": SimTypePointer(SimStruct({"certData": SimStruct({"size": SimTypeInt(signed=False, label="UInt32"), "data": SimTypePointer(SimTypeChar(label="Byte"), offset=0)}, name="FWP_BYTE_BLOB", pack=False, align=None), "certHash": SimStruct({"size": SimTypeInt(signed=False, label="UInt32"), "data": SimTypePointer(SimTypeChar(label="Byte"), offset=0)}, name="FWP_BYTE_BLOB", pack=False, align=None), "eku": SimTypePointer(SimStruct({"numEku": SimTypeInt(signed=False, label="UInt32"), "eku": SimTypePointer(SimTypePointer(SimTypeChar(label="Byte"), offset=0), offset=0)}, name="IKEEXT_CERT_EKUS0", pack=False, align=None), offset=0), "name": SimTypePointer(SimStruct({"nameType": SimTypeInt(signed=False, label="IKEEXT_CERT_CRITERIA_NAME_TYPE"), "certName": SimTypePointer(SimTypeChar(label="Char"), offset=0)}, name="IKEEXT_CERT_NAME0", pack=False, align=None), offset=0), "flags": SimTypeInt(signed=False, label="UInt32")}, name="IKEEXT_CERTIFICATE_CRITERIA0", pack=False, align=None), offset=0)}, name="_Anonymous2_e__Struct", pack=False, align=None), "Anonymous3": SimStruct({"inboundRootStoreArraySize": SimTypeInt(signed=False, label="UInt32"), "inboundTrustedRootStoreCriteria": SimTypePointer(SimStruct({"certData": SimStruct({"size": SimTypeInt(signed=False, label="UInt32"), "data": SimTypePointer(SimTypeChar(label="Byte"), offset=0)}, name="FWP_BYTE_BLOB", pack=False, align=None), "certHash": SimStruct({"size": SimTypeInt(signed=False, label="UInt32"), "data": SimTypePointer(SimTypeChar(label="Byte"), offset=0)}, name="FWP_BYTE_BLOB", pack=False, align=None), "eku": SimTypePointer(SimStruct({"numEku": SimTypeInt(signed=False, label="UInt32"), "eku": SimTypePointer(SimTypePointer(SimTypeChar(label="Byte"), offset=0), offset=0)}, name="IKEEXT_CERT_EKUS0", pack=False, align=None), offset=0), "name": SimTypePointer(SimStruct({"nameType": SimTypeInt(signed=False, label="IKEEXT_CERT_CRITERIA_NAME_TYPE"), "certName": SimTypePointer(SimTypeChar(label="Char"), offset=0)}, name="IKEEXT_CERT_NAME0", pack=False, align=None), offset=0), "flags": SimTypeInt(signed=False, label="UInt32")}, name="IKEEXT_CERTIFICATE_CRITERIA0", pack=False, align=None), offset=0)}, name="_Anonymous3_e__Struct", pack=False, align=None)}, name="<anon>", label="None"), "outboundConfigType": SimTypeInt(signed=False, label="IKEEXT_CERT_CONFIG_TYPE"), "Anonymous2": SimUnion({"Anonymous1": SimStruct({"outboundRootArraySize": SimTypeInt(signed=False, label="UInt32"), "outboundRootCriteria": SimTypePointer(SimStruct({"certData": SimStruct({"size": SimTypeInt(signed=False, label="UInt32"), "data": SimTypePointer(SimTypeChar(label="Byte"), offset=0)}, name="FWP_BYTE_BLOB", pack=False, align=None), "certHash": SimStruct({"size": SimTypeInt(signed=False, label="UInt32"), "data": SimTypePointer(SimTypeChar(label="Byte"), offset=0)}, name="FWP_BYTE_BLOB", pack=False, align=None), "eku": SimTypePointer(SimStruct({"numEku": SimTypeInt(signed=False, label="UInt32"), "eku": SimTypePointer(SimTypePointer(SimTypeChar(label="Byte"), offset=0), offset=0)}, name="IKEEXT_CERT_EKUS0", pack=False, align=None), offset=0), "name": SimTypePointer(SimStruct({"nameType": SimTypeInt(signed=False, label="IKEEXT_CERT_CRITERIA_NAME_TYPE"), "certName": SimTypePointer(SimTypeChar(label="Char"), offset=0)}, name="IKEEXT_CERT_NAME0", pack=False, align=None), offset=0), "flags": SimTypeInt(signed=False, label="UInt32")}, name="IKEEXT_CERTIFICATE_CRITERIA0", pack=False, align=None), offset=0)}, name="_Anonymous1_e__Struct", pack=False, align=None), "Anonymous2": SimStruct({"outboundEnterpriseStoreArraySize": SimTypeInt(signed=False, label="UInt32"), "outboundEnterpriseStoreCriteria": SimTypePointer(SimStruct({"certData": SimStruct({"size": SimTypeInt(signed=False, label="UInt32"), "data": SimTypePointer(SimTypeChar(label="Byte"), offset=0)}, name="FWP_BYTE_BLOB", pack=False, align=None), "certHash": SimStruct({"size": SimTypeInt(signed=False, label="UInt32"), "data": SimTypePointer(SimTypeChar(label="Byte"), offset=0)}, name="FWP_BYTE_BLOB", pack=False, align=None), "eku": SimTypePointer(SimStruct({"numEku": SimTypeInt(signed=False, label="UInt32"), "eku": SimTypePointer(SimTypePointer(SimTypeChar(label="Byte"), offset=0), offset=0)}, name="IKEEXT_CERT_EKUS0", pack=False, align=None), offset=0), "name": SimTypePointer(SimStruct({"nameType": SimTypeInt(signed=False, label="IKEEXT_CERT_CRITERIA_NAME_TYPE"), "certName": SimTypePointer(SimTypeChar(label="Char"), offset=0)}, name="IKEEXT_CERT_NAME0", pack=False, align=None), offset=0), "flags": SimTypeInt(signed=False, label="UInt32")}, name="IKEEXT_CERTIFICATE_CRITERIA0", pack=False, align=None), offset=0)}, name="_Anonymous2_e__Struct", pack=False, align=None), "Anonymous3": SimStruct({"outboundRootStoreArraySize": SimTypeInt(signed=False, label="UInt32"), "outboundTrustedRootStoreCriteria": SimTypePointer(SimStruct({"certData": SimStruct({"size": SimTypeInt(signed=False, label="UInt32"), "data": SimTypePointer(SimTypeChar(label="Byte"), offset=0)}, name="FWP_BYTE_BLOB", pack=False, align=None), "certHash": SimStruct({"size": SimTypeInt(signed=False, label="UInt32"), "data": SimTypePointer(SimTypeChar(label="Byte"), offset=0)}, name="FWP_BYTE_BLOB", pack=False, align=None), "eku": SimTypePointer(SimStruct({"numEku": SimTypeInt(signed=False, label="UInt32"), "eku": SimTypePointer(SimTypePointer(SimTypeChar(label="Byte"), offset=0), offset=0)}, name="IKEEXT_CERT_EKUS0", pack=False, align=None), offset=0), "name": SimTypePointer(SimStruct({"nameType": SimTypeInt(signed=False, label="IKEEXT_CERT_CRITERIA_NAME_TYPE"), "certName": SimTypePointer(SimTypeChar(label="Char"), offset=0)}, name="IKEEXT_CERT_NAME0", pack=False, align=None), offset=0), "flags": SimTypeInt(signed=False, label="UInt32")}, name="IKEEXT_CERTIFICATE_CRITERIA0", pack=False, align=None), offset=0)}, name="_Anonymous3_e__Struct", pack=False, align=None)}, name="<anon>", label="None"), "flags": SimTypeInt(signed=False, label="IKEEXT_CERT_AUTH"), "localCertLocationUrl": SimStruct({"size": SimTypeInt(signed=False, label="UInt32"), "data": SimTypePointer(SimTypeChar(label="Byte"), offset=0)}, name="FWP_BYTE_BLOB", pack=False, align=None)}, name="IKEEXT_CERTIFICATE_AUTHENTICATION2", pack=False, align=None), "cgaAuthentication": SimStruct({"keyContainerName": SimTypePointer(SimTypeChar(label="Char"), offset=0), "cspName": SimTypePointer(SimTypeChar(label="Char"), offset=0), "cspType": SimTypeInt(signed=False, label="UInt32"), "cgaModifier": SimStruct({"byteArray16": SimTypeFixedSizeArray(SimTypeChar(label="Byte"), 16)}, name="FWP_BYTE_ARRAY16", pack=False, align=None), "cgaCollisionCount": SimTypeChar(label="Byte")}, name="IKEEXT_IPV6_CGA_AUTHENTICATION0", pack=False, align=None), "eapAuthentication": SimStruct({"flags": SimTypeInt(signed=False, label="IKEEXT_EAP_AUTHENTICATION_FLAGS")}, name="IKEEXT_EAP_AUTHENTICATION0", pack=False, align=None)}, name="<anon>", label="None")}, name="IKEEXT_AUTHENTICATION_METHOD2", pack=False, align=None), offset=0), "initiatorImpersonationType": SimTypeInt(signed=False, label="IKEEXT_AUTHENTICATION_IMPERSONATION_TYPE"), "numIkeProposals": SimTypeInt(signed=False, label="UInt32"), "ikeProposals": SimTypePointer(SimStruct({"cipherAlgorithm": SimStruct({"algoIdentifier": SimTypeInt(signed=False, label="IKEEXT_CIPHER_TYPE"), "keyLen": SimTypeInt(signed=False, label="UInt32"), "rounds": SimTypeInt(signed=False, label="UInt32")}, name="IKEEXT_CIPHER_ALGORITHM0", pack=False, align=None), "integrityAlgorithm": SimStruct({"algoIdentifier": SimTypeInt(signed=False, label="IKEEXT_INTEGRITY_TYPE")}, name="IKEEXT_INTEGRITY_ALGORITHM0", pack=False, align=None), "maxLifetimeSeconds": SimTypeInt(signed=False, label="UInt32"), "dhGroup": SimTypeInt(signed=False, label="IKEEXT_DH_GROUP"), "quickModeLimit": SimTypeInt(signed=False, label="UInt32")}, name="IKEEXT_PROPOSAL0", pack=False, align=None), offset=0), "flags": SimTypeInt(signed=False, label="IKEEXT_POLICY_FLAG"), "maxDynamicFilters": SimTypeInt(signed=False, label="UInt32"), "retransmitDurationSecs": SimTypeInt(signed=False, label="UInt32")}, name="IKEEXT_POLICY2", pack=False, align=None), offset=0), "dataBuffer": SimTypePointer(SimStruct({"size": SimTypeInt(signed=False, label="UInt32"), "data": SimTypePointer(SimTypeChar(label="Byte"), offset=0)}, name="FWP_BYTE_BLOB", pack=False, align=None), offset=0), "classifyOptions": SimTypePointer(SimStruct({"numOptions": SimTypeInt(signed=False, label="UInt32"), "options": SimTypePointer(SimStruct({"type": SimTypeInt(signed=False, label="FWP_CLASSIFY_OPTION_TYPE"), "value": SimStruct({"type": SimTypeInt(signed=False, label="FWP_DATA_TYPE"), "Anonymous": SimUnion({"uint8": SimTypeChar(label="Byte"), "uint16": SimTypeShort(signed=False, label="UInt16"), "uint32": SimTypeInt(signed=False, label="UInt32"), "uint64": SimTypePointer(SimTypeLongLong(signed=False, label="UInt64"), offset=0), "int8": SimTypeChar(label="SByte"), "int16": SimTypeShort(signed=True, label="Int16"), "int32": SimTypeInt(signed=True, label="Int32"), "int64": SimTypePointer(SimTypeLongLong(signed=True, label="Int64"), offset=0), "float32": SimTypeFloat(size=32), "double64": SimTypePointer(SimTypeFloat(size=64), offset=0), "byteArray16": SimTypePointer(SimStruct({"byteArray16": SimTypeFixedSizeArray(SimTypeChar(label="Byte"), 16)}, name="FWP_BYTE_ARRAY16", pack=False, align=None), offset=0), "byteBlob": SimTypePointer(SimStruct({"size": SimTypeInt(signed=False, label="UInt32"), "data": SimTypePointer(SimTypeChar(label="Byte"), offset=0)}, name="FWP_BYTE_BLOB", pack=False, align=None), offset=0), "sid": SimTypePointer(SimTypeBottom(label="SID"), offset=0), "sd": SimTypePointer(SimStruct({"size": SimTypeInt(signed=False, label="UInt32"), "data": SimTypePointer(SimTypeChar(label="Byte"), offset=0)}, name="FWP_BYTE_BLOB", pack=False, align=None), offset=0), "tokenInformation": SimTypePointer(SimStruct({"sidCount": SimTypeInt(signed=False, label="UInt32"), "sids": SimTypePointer(SimTypeBottom(label="SID_AND_ATTRIBUTES"), offset=0), "restrictedSidCount": SimTypeInt(signed=False, label="UInt32"), "restrictedSids": SimTypePointer(SimTypeBottom(label="SID_AND_ATTRIBUTES"), offset=0)}, name="FWP_TOKEN_INFORMATION", pack=False, align=None), offset=0), "tokenAccessInformation": SimTypePointer(SimStruct({"size": SimTypeInt(signed=False, label="UInt32"), "data": SimTypePointer(SimTypeChar(label="Byte"), offset=0)}, name="FWP_BYTE_BLOB", pack=False, align=None), offset=0), "unicodeString": SimTypePointer(SimTypeChar(label="Char"), offset=0), "byteArray6": SimTypePointer(SimStruct({"byteArray6": SimTypeFixedSizeArray(SimTypeChar(label="Byte"), 6)}, name="FWP_BYTE_ARRAY6", pack=False, align=None), offset=0), "bitmapArray64": SimTypePointer(SimStruct({"bitmapArray64": SimTypeFixedSizeArray(SimTypeChar(label="Byte"), 8)}, name="FWP_BITMAP_ARRAY64_", pack=False, align=None), offset=0)}, name="<anon>", label="None")}, name="FWP_VALUE0", pack=False, align=None)}, name="FWPM_CLASSIFY_OPTION0", pack=False, align=None), offset=0)}, name="FWPM_CLASSIFY_OPTIONS0", pack=False, align=None), offset=0), "ikeV2QmTunnelPolicy": SimTypePointer(SimStruct({"flags": SimTypeInt(signed=False, label="UInt32"), "numIpsecProposals": SimTypeInt(signed=False, label="UInt32"), "ipsecProposals": SimTypePointer(SimStruct({"lifetime": SimStruct({"lifetimeSeconds": SimTypeInt(signed=False, label="UInt32"), "lifetimeKilobytes": SimTypeInt(signed=False, label="UInt32"), "lifetimePackets": SimTypeInt(signed=False, label="UInt32")}, name="IPSEC_SA_LIFETIME0", pack=False, align=None), "numSaTransforms": SimTypeInt(signed=False, label="UInt32"), "saTransforms": SimTypePointer(SimStruct({"ipsecTransformType": SimTypeInt(signed=False, label="IPSEC_TRANSFORM_TYPE"), "Anonymous": SimUnion({"ahTransform": SimTypePointer(SimStruct({"authTransformId": SimStruct({"authType": SimTypeInt(signed=False, label="IPSEC_AUTH_TYPE"), "authConfig": SimTypeChar(label="Byte")}, name="IPSEC_AUTH_TRANSFORM_ID0", pack=False, align=None), "cryptoModuleId": SimTypePointer(SimTypeBottom(label="Guid"), offset=0)}, name="IPSEC_AUTH_TRANSFORM0", pack=False, align=None), offset=0), "espAuthTransform": SimTypePointer(SimStruct({"authTransformId": SimStruct({"authType": SimTypeInt(signed=False, label="IPSEC_AUTH_TYPE"), "authConfig": SimTypeChar(label="Byte")}, name="IPSEC_AUTH_TRANSFORM_ID0", pack=False, align=None), "cryptoModuleId": SimTypePointer(SimTypeBottom(label="Guid"), offset=0)}, name="IPSEC_AUTH_TRANSFORM0", pack=False, align=None), offset=0), "espCipherTransform": SimTypePointer(SimStruct({"cipherTransformId": SimStruct({"cipherType": SimTypeInt(signed=False, label="IPSEC_CIPHER_TYPE"), "cipherConfig": SimTypeChar(label="Byte")}, name="IPSEC_CIPHER_TRANSFORM_ID0", pack=False, align=None), "cryptoModuleId": SimTypePointer(SimTypeBottom(label="Guid"), offset=0)}, name="IPSEC_CIPHER_TRANSFORM0", pack=False, align=None), offset=0), "espAuthAndCipherTransform": SimTypePointer(SimStruct({"authTransform": SimStruct({"authTransformId": SimStruct({"authType": SimTypeInt(signed=False, label="IPSEC_AUTH_TYPE"), "authConfig": SimTypeChar(label="Byte")}, name="IPSEC_AUTH_TRANSFORM_ID0", pack=False, align=None), "cryptoModuleId": SimTypePointer(SimTypeBottom(label="Guid"), offset=0)}, name="IPSEC_AUTH_TRANSFORM0", pack=False, align=None), "cipherTransform": SimStruct({"cipherTransformId": SimStruct({"cipherType": SimTypeInt(signed=False, label="IPSEC_CIPHER_TYPE"), "cipherConfig": SimTypeChar(label="Byte")}, name="IPSEC_CIPHER_TRANSFORM_ID0", pack=False, align=None), "cryptoModuleId": SimTypePointer(SimTypeBottom(label="Guid"), offset=0)}, name="IPSEC_CIPHER_TRANSFORM0", pack=False, align=None)}, name="IPSEC_AUTH_AND_CIPHER_TRANSFORM0", pack=False, align=None), offset=0), "espAuthFwTransform": SimTypePointer(SimStruct({"authTransformId": SimStruct({"authType": SimTypeInt(signed=False, label="IPSEC_AUTH_TYPE"), "authConfig": SimTypeChar(label="Byte")}, name="IPSEC_AUTH_TRANSFORM_ID0", pack=False, align=None), "cryptoModuleId": SimTypePointer(SimTypeBottom(label="Guid"), offset=0)}, name="IPSEC_AUTH_TRANSFORM0", pack=False, align=None), offset=0)}, name="<anon>", label="None")}, name="IPSEC_SA_TRANSFORM0", pack=False, align=None), offset=0), "pfsGroup": SimTypeInt(signed=False, label="IPSEC_PFS_GROUP")}, name="IPSEC_PROPOSAL0", pack=False, align=None), offset=0), "tunnelEndpoints": SimStruct({"ipVersion": SimTypeInt(signed=False, label="FWP_IP_VERSION"), "Anonymous1": SimUnion({"localV4Address": SimTypeInt(signed=False, label="UInt32"), "localV6Address": SimTypeFixedSizeArray(SimTypeChar(label="Byte"), 16)}, name="<anon>", label="None"), "Anonymous2": SimUnion({"remoteV4Address": SimTypeInt(signed=False, label="UInt32"), "remoteV6Address": SimTypeFixedSizeArray(SimTypeChar(label="Byte"), 16)}, name="<anon>", label="None"), "localIfLuid": SimTypeLongLong(signed=False, label="UInt64"), "remoteFqdn": SimTypePointer(SimTypeChar(label="Char"), offset=0), "numAddresses": SimTypeInt(signed=False, label="UInt32"), "remoteAddresses": SimTypePointer(SimStruct({"ipVersion": SimTypeInt(signed=False, label="FWP_IP_VERSION"), "Anonymous": SimUnion({"v4Address": SimTypeInt(signed=False, label="UInt32"), "v6Address": SimTypeFixedSizeArray(SimTypeChar(label="Byte"), 16)}, name="<anon>", label="None")}, name="IPSEC_TUNNEL_ENDPOINT0", pack=False, align=None), offset=0)}, name="IPSEC_TUNNEL_ENDPOINTS2", pack=False, align=None), "saIdleTimeout": SimStruct({"idleTimeoutSeconds": SimTypeInt(signed=False, label="UInt32"), "idleTimeoutSecondsFailOver": SimTypeInt(signed=False, label="UInt32")}, name="IPSEC_SA_IDLE_TIMEOUT0", pack=False, align=None), "emPolicy": SimTypePointer(SimStruct({"numAuthenticationMethods": SimTypeInt(signed=False, label="UInt32"), "authenticationMethods": SimTypePointer(SimStruct({"authenticationMethodType": SimTypeInt(signed=False, label="IKEEXT_AUTHENTICATION_METHOD_TYPE"), "Anonymous": SimUnion({"presharedKeyAuthentication": SimStruct({"presharedKey": SimStruct({"size": SimTypeInt(signed=False, label="UInt32"), "data": SimTypePointer(SimTypeChar(label="Byte"), offset=0)}, name="FWP_BYTE_BLOB", pack=False, align=None), "flags": SimTypeInt(signed=False, label="IKEEXT_PRESHARED_KEY_AUTHENTICATION_FLAGS")}, name="IKEEXT_PRESHARED_KEY_AUTHENTICATION1", pack=False, align=None), "certificateAuthentication": SimStruct({"inboundConfigType": SimTypeInt(signed=False, label="IKEEXT_CERT_CONFIG_TYPE"), "Anonymous1": SimUnion({"Anonymous1": SimStruct({"inboundRootArraySize": SimTypeInt(signed=False, label="UInt32"), "inboundRootCriteria": SimTypePointer(SimStruct({"certData": SimStruct({"size": SimTypeInt(signed=False, label="UInt32"), "data": SimTypePointer(SimTypeChar(label="Byte"), offset=0)}, name="FWP_BYTE_BLOB", pack=False, align=None), "certHash": SimStruct({"size": SimTypeInt(signed=False, label="UInt32"), "data": SimTypePointer(SimTypeChar(label="Byte"), offset=0)}, name="FWP_BYTE_BLOB", pack=False, align=None), "eku": SimTypePointer(SimStruct({"numEku": SimTypeInt(signed=False, label="UInt32"), "eku": SimTypePointer(SimTypePointer(SimTypeChar(label="Byte"), offset=0), offset=0)}, name="IKEEXT_CERT_EKUS0", pack=False, align=None), offset=0), "name": SimTypePointer(SimStruct({"nameType": SimTypeInt(signed=False, label="IKEEXT_CERT_CRITERIA_NAME_TYPE"), "certName": SimTypePointer(SimTypeChar(label="Char"), offset=0)}, name="IKEEXT_CERT_NAME0", pack=False, align=None), offset=0), "flags": SimTypeInt(signed=False, label="UInt32")}, name="IKEEXT_CERTIFICATE_CRITERIA0", pack=False, align=None), offset=0)}, name="_Anonymous1_e__Struct", pack=False, align=None), "Anonymous2": SimStruct({"inboundEnterpriseStoreArraySize": SimTypeInt(signed=False, label="UInt32"), "inboundEnterpriseStoreCriteria": SimTypePointer(SimStruct({"certData": SimStruct({"size": SimTypeInt(signed=False, label="UInt32"), "data": SimTypePointer(SimTypeChar(label="Byte"), offset=0)}, name="FWP_BYTE_BLOB", pack=False, align=None), "certHash": SimStruct({"size": SimTypeInt(signed=False, label="UInt32"), "data": SimTypePointer(SimTypeChar(label="Byte"), offset=0)}, name="FWP_BYTE_BLOB", pack=False, align=None), "eku": SimTypePointer(SimStruct({"numEku": SimTypeInt(signed=False, label="UInt32"), "eku": SimTypePointer(SimTypePointer(SimTypeChar(label="Byte"), offset=0), offset=0)}, name="IKEEXT_CERT_EKUS0", pack=False, align=None), offset=0), "name": SimTypePointer(SimStruct({"nameType": SimTypeInt(signed=False, label="IKEEXT_CERT_CRITERIA_NAME_TYPE"), "certName": SimTypePointer(SimTypeChar(label="Char"), offset=0)}, name="IKEEXT_CERT_NAME0", pack=False, align=None), offset=0), "flags": SimTypeInt(signed=False, label="UInt32")}, name="IKEEXT_CERTIFICATE_CRITERIA0", pack=False, align=None), offset=0)}, name="_Anonymous2_e__Struct", pack=False, align=None), "Anonymous3": SimStruct({"inboundRootStoreArraySize": SimTypeInt(signed=False, label="UInt32"), "inboundTrustedRootStoreCriteria": SimTypePointer(SimStruct({"certData": SimStruct({"size": SimTypeInt(signed=False, label="UInt32"), "data": SimTypePointer(SimTypeChar(label="Byte"), offset=0)}, name="FWP_BYTE_BLOB", pack=False, align=None), "certHash": SimStruct({"size": SimTypeInt(signed=False, label="UInt32"), "data": SimTypePointer(SimTypeChar(label="Byte"), offset=0)}, name="FWP_BYTE_BLOB", pack=False, align=None), "eku": SimTypePointer(SimStruct({"numEku": SimTypeInt(signed=False, label="UInt32"), "eku": SimTypePointer(SimTypePointer(SimTypeChar(label="Byte"), offset=0), offset=0)}, name="IKEEXT_CERT_EKUS0", pack=False, align=None), offset=0), "name": SimTypePointer(SimStruct({"nameType": SimTypeInt(signed=False, label="IKEEXT_CERT_CRITERIA_NAME_TYPE"), "certName": SimTypePointer(SimTypeChar(label="Char"), offset=0)}, name="IKEEXT_CERT_NAME0", pack=False, align=None), offset=0), "flags": SimTypeInt(signed=False, label="UInt32")}, name="IKEEXT_CERTIFICATE_CRITERIA0", pack=False, align=None), offset=0)}, name="_Anonymous3_e__Struct", pack=False, align=None)}, name="<anon>", label="None"), "outboundConfigType": SimTypeInt(signed=False, label="IKEEXT_CERT_CONFIG_TYPE"), "Anonymous2": SimUnion({"Anonymous1": SimStruct({"outboundRootArraySize": SimTypeInt(signed=False, label="UInt32"), "outboundRootCriteria": SimTypePointer(SimStruct({"certData": SimStruct({"size": SimTypeInt(signed=False, label="UInt32"), "data": SimTypePointer(SimTypeChar(label="Byte"), offset=0)}, name="FWP_BYTE_BLOB", pack=False, align=None), "certHash": SimStruct({"size": SimTypeInt(signed=False, label="UInt32"), "data": SimTypePointer(SimTypeChar(label="Byte"), offset=0)}, name="FWP_BYTE_BLOB", pack=False, align=None), "eku": SimTypePointer(SimStruct({"numEku": SimTypeInt(signed=False, label="UInt32"), "eku": SimTypePointer(SimTypePointer(SimTypeChar(label="Byte"), offset=0), offset=0)}, name="IKEEXT_CERT_EKUS0", pack=False, align=None), offset=0), "name": SimTypePointer(SimStruct({"nameType": SimTypeInt(signed=False, label="IKEEXT_CERT_CRITERIA_NAME_TYPE"), "certName": SimTypePointer(SimTypeChar(label="Char"), offset=0)}, name="IKEEXT_CERT_NAME0", pack=False, align=None), offset=0), "flags": SimTypeInt(signed=False, label="UInt32")}, name="IKEEXT_CERTIFICATE_CRITERIA0", pack=False, align=None), offset=0)}, name="_Anonymous1_e__Struct", pack=False, align=None), "Anonymous2": SimStruct({"outboundEnterpriseStoreArraySize": SimTypeInt(signed=False, label="UInt32"), "outboundEnterpriseStoreCriteria": SimTypePointer(SimStruct({"certData": SimStruct({"size": SimTypeInt(signed=False, label="UInt32"), "data": SimTypePointer(SimTypeChar(label="Byte"), offset=0)}, name="FWP_BYTE_BLOB", pack=False, align=None), "certHash": SimStruct({"size": SimTypeInt(signed=False, label="UInt32"), "data": SimTypePointer(SimTypeChar(label="Byte"), offset=0)}, name="FWP_BYTE_BLOB", pack=False, align=None), "eku": SimTypePointer(SimStruct({"numEku": SimTypeInt(signed=False, label="UInt32"), "eku": SimTypePointer(SimTypePointer(SimTypeChar(label="Byte"), offset=0), offset=0)}, name="IKEEXT_CERT_EKUS0", pack=False, align=None), offset=0), "name": SimTypePointer(SimStruct({"nameType": SimTypeInt(signed=False, label="IKEEXT_CERT_CRITERIA_NAME_TYPE"), "certName": SimTypePointer(SimTypeChar(label="Char"), offset=0)}, name="IKEEXT_CERT_NAME0", pack=False, align=None), offset=0), "flags": SimTypeInt(signed=False, label="UInt32")}, name="IKEEXT_CERTIFICATE_CRITERIA0", pack=False, align=None), offset=0)}, name="_Anonymous2_e__Struct", pack=False, align=None), "Anonymous3": SimStruct({"outboundRootStoreArraySize": SimTypeInt(signed=False, label="UInt32"), "outboundTrustedRootStoreCriteria": SimTypePointer(SimStruct({"certData": SimStruct({"size": SimTypeInt(signed=False, label="UInt32"), "data": SimTypePointer(SimTypeChar(label="Byte"), offset=0)}, name="FWP_BYTE_BLOB", pack=False, align=None), "certHash": SimStruct({"size": SimTypeInt(signed=False, label="UInt32"), "data": SimTypePointer(SimTypeChar(label="Byte"), offset=0)}, name="FWP_BYTE_BLOB", pack=False, align=None), "eku": SimTypePointer(SimStruct({"numEku": SimTypeInt(signed=False, label="UInt32"), "eku": SimTypePointer(SimTypePointer(SimTypeChar(label="Byte"), offset=0), offset=0)}, name="IKEEXT_CERT_EKUS0", pack=False, align=None), offset=0), "name": SimTypePointer(SimStruct({"nameType": SimTypeInt(signed=False, label="IKEEXT_CERT_CRITERIA_NAME_TYPE"), "certName": SimTypePointer(SimTypeChar(label="Char"), offset=0)}, name="IKEEXT_CERT_NAME0", pack=False, align=None), offset=0), "flags": SimTypeInt(signed=False, label="UInt32")}, name="IKEEXT_CERTIFICATE_CRITERIA0", pack=False, align=None), offset=0)}, name="_Anonymous3_e__Struct", pack=False, align=None)}, name="<anon>", label="None"), "flags": SimTypeInt(signed=False, label="IKEEXT_CERT_AUTH"), "localCertLocationUrl": SimStruct({"size": SimTypeInt(signed=False, label="UInt32"), "data": SimTypePointer(SimTypeChar(label="Byte"), offset=0)}, name="FWP_BYTE_BLOB", pack=False, align=None)}, name="IKEEXT_CERTIFICATE_AUTHENTICATION2", pack=False, align=None), "kerberosAuthentication": SimStruct({"flags": SimTypeInt(signed=False, label="IKEEXT_KERBEROS_AUTHENTICATION_FLAGS"), "proxyServer": SimTypePointer(SimTypeChar(label="Char"), offset=0)}, name="IKEEXT_KERBEROS_AUTHENTICATION1", pack=False, align=None), "reservedAuthentication": SimStruct({"flags": SimTypeInt(signed=False, label="IKEEXT_RESERVED_AUTHENTICATION_FLAGS")}, name="IKEEXT_RESERVED_AUTHENTICATION0", pack=False, align=None), "ntlmV2Authentication": SimStruct({"flags": SimTypeInt(signed=False, label="UInt32")}, name="IKEEXT_NTLM_V2_AUTHENTICATION0", pack=False, align=None), "sslAuthentication": SimStruct({"inboundConfigType": SimTypeInt(signed=False, label="IKEEXT_CERT_CONFIG_TYPE"), "Anonymous1": SimUnion({"Anonymous1": SimStruct({"inboundRootArraySize": SimTypeInt(signed=False, label="UInt32"), "inboundRootCriteria": SimTypePointer(SimStruct({"certData": SimStruct({"size": SimTypeInt(signed=False, label="UInt32"), "data": SimTypePointer(SimTypeChar(label="Byte"), offset=0)}, name="FWP_BYTE_BLOB", pack=False, align=None), "certHash": SimStruct({"size": SimTypeInt(signed=False, label="UInt32"), "data": SimTypePointer(SimTypeChar(label="Byte"), offset=0)}, name="FWP_BYTE_BLOB", pack=False, align=None), "eku": SimTypePointer(SimStruct({"numEku": SimTypeInt(signed=False, label="UInt32"), "eku": SimTypePointer(SimTypePointer(SimTypeChar(label="Byte"), offset=0), offset=0)}, name="IKEEXT_CERT_EKUS0", pack=False, align=None), offset=0), "name": SimTypePointer(SimStruct({"nameType": SimTypeInt(signed=False, label="IKEEXT_CERT_CRITERIA_NAME_TYPE"), "certName": SimTypePointer(SimTypeChar(label="Char"), offset=0)}, name="IKEEXT_CERT_NAME0", pack=False, align=None), offset=0), "flags": SimTypeInt(signed=False, label="UInt32")}, name="IKEEXT_CERTIFICATE_CRITERIA0", pack=False, align=None), offset=0)}, name="_Anonymous1_e__Struct", pack=False, align=None), "Anonymous2": SimStruct({"inboundEnterpriseStoreArraySize": SimTypeInt(signed=False, label="UInt32"), "inboundEnterpriseStoreCriteria": SimTypePointer(SimStruct({"certData": SimStruct({"size": SimTypeInt(signed=False, label="UInt32"), "data": SimTypePointer(SimTypeChar(label="Byte"), offset=0)}, name="FWP_BYTE_BLOB", pack=False, align=None), "certHash": SimStruct({"size": SimTypeInt(signed=False, label="UInt32"), "data": SimTypePointer(SimTypeChar(label="Byte"), offset=0)}, name="FWP_BYTE_BLOB", pack=False, align=None), "eku": SimTypePointer(SimStruct({"numEku": SimTypeInt(signed=False, label="UInt32"), "eku": SimTypePointer(SimTypePointer(SimTypeChar(label="Byte"), offset=0), offset=0)}, name="IKEEXT_CERT_EKUS0", pack=False, align=None), offset=0), "name": SimTypePointer(SimStruct({"nameType": SimTypeInt(signed=False, label="IKEEXT_CERT_CRITERIA_NAME_TYPE"), "certName": SimTypePointer(SimTypeChar(label="Char"), offset=0)}, name="IKEEXT_CERT_NAME0", pack=False, align=None), offset=0), "flags": SimTypeInt(signed=False, label="UInt32")}, name="IKEEXT_CERTIFICATE_CRITERIA0", pack=False, align=None), offset=0)}, name="_Anonymous2_e__Struct", pack=False, align=None), "Anonymous3": SimStruct({"inboundRootStoreArraySize": SimTypeInt(signed=False, label="UInt32"), "inboundTrustedRootStoreCriteria": SimTypePointer(SimStruct({"certData": SimStruct({"size": SimTypeInt(signed=False, label="UInt32"), "data": SimTypePointer(SimTypeChar(label="Byte"), offset=0)}, name="FWP_BYTE_BLOB", pack=False, align=None), "certHash": SimStruct({"size": SimTypeInt(signed=False, label="UInt32"), "data": SimTypePointer(SimTypeChar(label="Byte"), offset=0)}, name="FWP_BYTE_BLOB", pack=False, align=None), "eku": SimTypePointer(SimStruct({"numEku": SimTypeInt(signed=False, label="UInt32"), "eku": SimTypePointer(SimTypePointer(SimTypeChar(label="Byte"), offset=0), offset=0)}, name="IKEEXT_CERT_EKUS0", pack=False, align=None), offset=0), "name": SimTypePointer(SimStruct({"nameType": SimTypeInt(signed=False, label="IKEEXT_CERT_CRITERIA_NAME_TYPE"), "certName": SimTypePointer(SimTypeChar(label="Char"), offset=0)}, name="IKEEXT_CERT_NAME0", pack=False, align=None), offset=0), "flags": SimTypeInt(signed=False, label="UInt32")}, name="IKEEXT_CERTIFICATE_CRITERIA0", pack=False, align=None), offset=0)}, name="_Anonymous3_e__Struct", pack=False, align=None)}, name="<anon>", label="None"), "outboundConfigType": SimTypeInt(signed=False, label="IKEEXT_CERT_CONFIG_TYPE"), "Anonymous2": SimUnion({"Anonymous1": SimStruct({"outboundRootArraySize": SimTypeInt(signed=False, label="UInt32"), "outboundRootCriteria": SimTypePointer(SimStruct({"certData": SimStruct({"size": SimTypeInt(signed=False, label="UInt32"), "data": SimTypePointer(SimTypeChar(label="Byte"), offset=0)}, name="FWP_BYTE_BLOB", pack=False, align=None), "certHash": SimStruct({"size": SimTypeInt(signed=False, label="UInt32"), "data": SimTypePointer(SimTypeChar(label="Byte"), offset=0)}, name="FWP_BYTE_BLOB", pack=False, align=None), "eku": SimTypePointer(SimStruct({"numEku": SimTypeInt(signed=False, label="UInt32"), "eku": SimTypePointer(SimTypePointer(SimTypeChar(label="Byte"), offset=0), offset=0)}, name="IKEEXT_CERT_EKUS0", pack=False, align=None), offset=0), "name": SimTypePointer(SimStruct({"nameType": SimTypeInt(signed=False, label="IKEEXT_CERT_CRITERIA_NAME_TYPE"), "certName": SimTypePointer(SimTypeChar(label="Char"), offset=0)}, name="IKEEXT_CERT_NAME0", pack=False, align=None), offset=0), "flags": SimTypeInt(signed=False, label="UInt32")}, name="IKEEXT_CERTIFICATE_CRITERIA0", pack=False, align=None), offset=0)}, name="_Anonymous1_e__Struct", pack=False, align=None), "Anonymous2": SimStruct({"outboundEnterpriseStoreArraySize": SimTypeInt(signed=False, label="UInt32"), "outboundEnterpriseStoreCriteria": SimTypePointer(SimStruct({"certData": SimStruct({"size": SimTypeInt(signed=False, label="UInt32"), "data": SimTypePointer(SimTypeChar(label="Byte"), offset=0)}, name="FWP_BYTE_BLOB", pack=False, align=None), "certHash": SimStruct({"size": SimTypeInt(signed=False, label="UInt32"), "data": SimTypePointer(SimTypeChar(label="Byte"), offset=0)}, name="FWP_BYTE_BLOB", pack=False, align=None), "eku": SimTypePointer(SimStruct({"numEku": SimTypeInt(signed=False, label="UInt32"), "eku": SimTypePointer(SimTypePointer(SimTypeChar(label="Byte"), offset=0), offset=0)}, name="IKEEXT_CERT_EKUS0", pack=False, align=None), offset=0), "name": SimTypePointer(SimStruct({"nameType": SimTypeInt(signed=False, label="IKEEXT_CERT_CRITERIA_NAME_TYPE"), "certName": SimTypePointer(SimTypeChar(label="Char"), offset=0)}, name="IKEEXT_CERT_NAME0", pack=False, align=None), offset=0), "flags": SimTypeInt(signed=False, label="UInt32")}, name="IKEEXT_CERTIFICATE_CRITERIA0", pack=False, align=None), offset=0)}, name="_Anonymous2_e__Struct", pack=False, align=None), "Anonymous3": SimStruct({"outboundRootStoreArraySize": SimTypeInt(signed=False, label="UInt32"), "outboundTrustedRootStoreCriteria": SimTypePointer(SimStruct({"certData": SimStruct({"size": SimTypeInt(signed=False, label="UInt32"), "data": SimTypePointer(SimTypeChar(label="Byte"), offset=0)}, name="FWP_BYTE_BLOB", pack=False, align=None), "certHash": SimStruct({"size": SimTypeInt(signed=False, label="UInt32"), "data": SimTypePointer(SimTypeChar(label="Byte"), offset=0)}, name="FWP_BYTE_BLOB", pack=False, align=None), "eku": SimTypePointer(SimStruct({"numEku": SimTypeInt(signed=False, label="UInt32"), "eku": SimTypePointer(SimTypePointer(SimTypeChar(label="Byte"), offset=0), offset=0)}, name="IKEEXT_CERT_EKUS0", pack=False, align=None), offset=0), "name": SimTypePointer(SimStruct({"nameType": SimTypeInt(signed=False, label="IKEEXT_CERT_CRITERIA_NAME_TYPE"), "certName": SimTypePointer(SimTypeChar(label="Char"), offset=0)}, name="IKEEXT_CERT_NAME0", pack=False, align=None), offset=0), "flags": SimTypeInt(signed=False, label="UInt32")}, name="IKEEXT_CERTIFICATE_CRITERIA0", pack=False, align=None), offset=0)}, name="_Anonymous3_e__Struct", pack=False, align=None)}, name="<anon>", label="None"), "flags": SimTypeInt(signed=False, label="IKEEXT_CERT_AUTH"), "localCertLocationUrl": SimStruct({"size": SimTypeInt(signed=False, label="UInt32"), "data": SimTypePointer(SimTypeChar(label="Byte"), offset=0)}, name="FWP_BYTE_BLOB", pack=False, align=None)}, name="IKEEXT_CERTIFICATE_AUTHENTICATION2", pack=False, align=None), "cgaAuthentication": SimStruct({"keyContainerName": SimTypePointer(SimTypeChar(label="Char"), offset=0), "cspName": SimTypePointer(SimTypeChar(label="Char"), offset=0), "cspType": SimTypeInt(signed=False, label="UInt32"), "cgaModifier": SimStruct({"byteArray16": SimTypeFixedSizeArray(SimTypeChar(label="Byte"), 16)}, name="FWP_BYTE_ARRAY16", pack=False, align=None), "cgaCollisionCount": SimTypeChar(label="Byte")}, name="IKEEXT_IPV6_CGA_AUTHENTICATION0", pack=False, align=None), "eapAuthentication": SimStruct({"flags": SimTypeInt(signed=False, label="IKEEXT_EAP_AUTHENTICATION_FLAGS")}, name="IKEEXT_EAP_AUTHENTICATION0", pack=False, align=None)}, name="<anon>", label="None")}, name="IKEEXT_AUTHENTICATION_METHOD2", pack=False, align=None), offset=0), "initiatorImpersonationType": SimTypeInt(signed=False, label="IKEEXT_AUTHENTICATION_IMPERSONATION_TYPE")}, name="IKEEXT_EM_POLICY2", pack=False, align=None), offset=0), "fwdPathSaLifetime": SimTypeInt(signed=False, label="UInt32"), "compartmentId": SimTypeInt(signed=False, label="UInt32"), "numTrafficSelectorPolicy": SimTypeInt(signed=False, label="UInt32"), "trafficSelectorPolicies": SimTypePointer(SimStruct({"flags": SimTypeInt(signed=False, label="UInt32"), "numLocalTrafficSelectors": SimTypeInt(signed=False, label="UInt32"), "localTrafficSelectors": SimTypePointer(SimStruct({"protocolId": SimTypeChar(label="Byte"), "portStart": SimTypeShort(signed=False, label="UInt16"), "portEnd": SimTypeShort(signed=False, label="UInt16"), "ipVersion": SimTypeInt(signed=False, label="FWP_IP_VERSION"), "Anonymous1": SimUnion({"startV4Address": SimTypeInt(signed=False, label="UInt32"), "startV6Address": SimTypeFixedSizeArray(SimTypeChar(label="Byte"), 16)}, name="<anon>", label="None"), "Anonymous2": SimUnion({"endV4Address": SimTypeInt(signed=False, label="UInt32"), "endV6Address": SimTypeFixedSizeArray(SimTypeChar(label="Byte"), 16)}, name="<anon>", label="None")}, name="IPSEC_TRAFFIC_SELECTOR0_", pack=False, align=None), offset=0), "numRemoteTrafficSelectors": SimTypeInt(signed=False, label="UInt32"), "remoteTrafficSelectors": SimTypePointer(SimStruct({"protocolId": SimTypeChar(label="Byte"), "portStart": SimTypeShort(signed=False, label="UInt16"), "portEnd": SimTypeShort(signed=False, label="UInt16"), "ipVersion": SimTypeInt(signed=False, label="FWP_IP_VERSION"), "Anonymous1": SimUnion({"startV4Address": SimTypeInt(signed=False, label="UInt32"), "startV6Address": SimTypeFixedSizeArray(SimTypeChar(label="Byte"), 16)}, name="<anon>", label="None"), "Anonymous2": SimUnion({"endV4Address": SimTypeInt(signed=False, label="UInt32"), "endV6Address": SimTypeFixedSizeArray(SimTypeChar(label="Byte"), 16)}, name="<anon>", label="None")}, name="IPSEC_TRAFFIC_SELECTOR0_", pack=False, align=None), offset=0)}, name="IPSEC_TRAFFIC_SELECTOR_POLICY0_", pack=False, align=None), offset=0)}, name="IPSEC_TUNNEL_POLICY3_", pack=False, align=None), offset=0), "ikeV2QmTransportPolicy": SimTypePointer(SimStruct({"numIpsecProposals": SimTypeInt(signed=False, label="UInt32"), "ipsecProposals": SimTypePointer(SimStruct({"lifetime": SimStruct({"lifetimeSeconds": SimTypeInt(signed=False, label="UInt32"), "lifetimeKilobytes": SimTypeInt(signed=False, label="UInt32"), "lifetimePackets": SimTypeInt(signed=False, label="UInt32")}, name="IPSEC_SA_LIFETIME0", pack=False, align=None), "numSaTransforms": SimTypeInt(signed=False, label="UInt32"), "saTransforms": SimTypePointer(SimStruct({"ipsecTransformType": SimTypeInt(signed=False, label="IPSEC_TRANSFORM_TYPE"), "Anonymous": SimUnion({"ahTransform": SimTypePointer(SimStruct({"authTransformId": SimStruct({"authType": SimTypeInt(signed=False, label="IPSEC_AUTH_TYPE"), "authConfig": SimTypeChar(label="Byte")}, name="IPSEC_AUTH_TRANSFORM_ID0", pack=False, align=None), "cryptoModuleId": SimTypePointer(SimTypeBottom(label="Guid"), offset=0)}, name="IPSEC_AUTH_TRANSFORM0", pack=False, align=None), offset=0), "espAuthTransform": SimTypePointer(SimStruct({"authTransformId": SimStruct({"authType": SimTypeInt(signed=False, label="IPSEC_AUTH_TYPE"), "authConfig": SimTypeChar(label="Byte")}, name="IPSEC_AUTH_TRANSFORM_ID0", pack=False, align=None), "cryptoModuleId": SimTypePointer(SimTypeBottom(label="Guid"), offset=0)}, name="IPSEC_AUTH_TRANSFORM0", pack=False, align=None), offset=0), "espCipherTransform": SimTypePointer(SimStruct({"cipherTransformId": SimStruct({"cipherType": SimTypeInt(signed=False, label="IPSEC_CIPHER_TYPE"), "cipherConfig": SimTypeChar(label="Byte")}, name="IPSEC_CIPHER_TRANSFORM_ID0", pack=False, align=None), "cryptoModuleId": SimTypePointer(SimTypeBottom(label="Guid"), offset=0)}, name="IPSEC_CIPHER_TRANSFORM0", pack=False, align=None), offset=0), "espAuthAndCipherTransform": SimTypePointer(SimStruct({"authTransform": SimStruct({"authTransformId": SimStruct({"authType": SimTypeInt(signed=False, label="IPSEC_AUTH_TYPE"), "authConfig": SimTypeChar(label="Byte")}, name="IPSEC_AUTH_TRANSFORM_ID0", pack=False, align=None), "cryptoModuleId": SimTypePointer(SimTypeBottom(label="Guid"), offset=0)}, name="IPSEC_AUTH_TRANSFORM0", pack=False, align=None), "cipherTransform": SimStruct({"cipherTransformId": SimStruct({"cipherType": SimTypeInt(signed=False, label="IPSEC_CIPHER_TYPE"), "cipherConfig": SimTypeChar(label="Byte")}, name="IPSEC_CIPHER_TRANSFORM_ID0", pack=False, align=None), "cryptoModuleId": SimTypePointer(SimTypeBottom(label="Guid"), offset=0)}, name="IPSEC_CIPHER_TRANSFORM0", pack=False, align=None)}, name="IPSEC_AUTH_AND_CIPHER_TRANSFORM0", pack=False, align=None), offset=0), "espAuthFwTransform": SimTypePointer(SimStruct({"authTransformId": SimStruct({"authType": SimTypeInt(signed=False, label="IPSEC_AUTH_TYPE"), "authConfig": SimTypeChar(label="Byte")}, name="IPSEC_AUTH_TRANSFORM_ID0", pack=False, align=None), "cryptoModuleId": SimTypePointer(SimTypeBottom(label="Guid"), offset=0)}, name="IPSEC_AUTH_TRANSFORM0", pack=False, align=None), offset=0)}, name="<anon>", label="None")}, name="IPSEC_SA_TRANSFORM0", pack=False, align=None), offset=0), "pfsGroup": SimTypeInt(signed=False, label="IPSEC_PFS_GROUP")}, name="IPSEC_PROPOSAL0", pack=False, align=None), offset=0), "flags": SimTypeInt(signed=False, label="IPSEC_POLICY_FLAG"), "ndAllowClearTimeoutSeconds": SimTypeInt(signed=False, label="UInt32"), "saIdleTimeout": SimStruct({"idleTimeoutSeconds": SimTypeInt(signed=False, label="UInt32"), "idleTimeoutSecondsFailOver": SimTypeInt(signed=False, label="UInt32")}, name="IPSEC_SA_IDLE_TIMEOUT0", pack=False, align=None), "emPolicy": SimTypePointer(SimStruct({"numAuthenticationMethods": SimTypeInt(signed=False, label="UInt32"), "authenticationMethods": SimTypePointer(SimStruct({"authenticationMethodType": SimTypeInt(signed=False, label="IKEEXT_AUTHENTICATION_METHOD_TYPE"), "Anonymous": SimUnion({"presharedKeyAuthentication": SimStruct({"presharedKey": SimStruct({"size": SimTypeInt(signed=False, label="UInt32"), "data": SimTypePointer(SimTypeChar(label="Byte"), offset=0)}, name="FWP_BYTE_BLOB", pack=False, align=None), "flags": SimTypeInt(signed=False, label="IKEEXT_PRESHARED_KEY_AUTHENTICATION_FLAGS")}, name="IKEEXT_PRESHARED_KEY_AUTHENTICATION1", pack=False, align=None), "certificateAuthentication": SimStruct({"inboundConfigType": SimTypeInt(signed=False, label="IKEEXT_CERT_CONFIG_TYPE"), "Anonymous1": SimUnion({"Anonymous1": SimStruct({"inboundRootArraySize": SimTypeInt(signed=False, label="UInt32"), "inboundRootCriteria": SimTypePointer(SimStruct({"certData": SimStruct({"size": SimTypeInt(signed=False, label="UInt32"), "data": SimTypePointer(SimTypeChar(label="Byte"), offset=0)}, name="FWP_BYTE_BLOB", pack=False, align=None), "certHash": SimStruct({"size": SimTypeInt(signed=False, label="UInt32"), "data": SimTypePointer(SimTypeChar(label="Byte"), offset=0)}, name="FWP_BYTE_BLOB", pack=False, align=None), "eku": SimTypePointer(SimStruct({"numEku": SimTypeInt(signed=False, label="UInt32"), "eku": SimTypePointer(SimTypePointer(SimTypeChar(label="Byte"), offset=0), offset=0)}, name="IKEEXT_CERT_EKUS0", pack=False, align=None), offset=0), "name": SimTypePointer(SimStruct({"nameType": SimTypeInt(signed=False, label="IKEEXT_CERT_CRITERIA_NAME_TYPE"), "certName": SimTypePointer(SimTypeChar(label="Char"), offset=0)}, name="IKEEXT_CERT_NAME0", pack=False, align=None), offset=0), "flags": SimTypeInt(signed=False, label="UInt32")}, name="IKEEXT_CERTIFICATE_CRITERIA0", pack=False, align=None), offset=0)}, name="_Anonymous1_e__Struct", pack=False, align=None), "Anonymous2": SimStruct({"inboundEnterpriseStoreArraySize": SimTypeInt(signed=False, label="UInt32"), "inboundEnterpriseStoreCriteria": SimTypePointer(SimStruct({"certData": SimStruct({"size": SimTypeInt(signed=False, label="UInt32"), "data": SimTypePointer(SimTypeChar(label="Byte"), offset=0)}, name="FWP_BYTE_BLOB", pack=False, align=None), "certHash": SimStruct({"size": SimTypeInt(signed=False, label="UInt32"), "data": SimTypePointer(SimTypeChar(label="Byte"), offset=0)}, name="FWP_BYTE_BLOB", pack=False, align=None), "eku": SimTypePointer(SimStruct({"numEku": SimTypeInt(signed=False, label="UInt32"), "eku": SimTypePointer(SimTypePointer(SimTypeChar(label="Byte"), offset=0), offset=0)}, name="IKEEXT_CERT_EKUS0", pack=False, align=None), offset=0), "name": SimTypePointer(SimStruct({"nameType": SimTypeInt(signed=False, label="IKEEXT_CERT_CRITERIA_NAME_TYPE"), "certName": SimTypePointer(SimTypeChar(label="Char"), offset=0)}, name="IKEEXT_CERT_NAME0", pack=False, align=None), offset=0), "flags": SimTypeInt(signed=False, label="UInt32")}, name="IKEEXT_CERTIFICATE_CRITERIA0", pack=False, align=None), offset=0)}, name="_Anonymous2_e__Struct", pack=False, align=None), "Anonymous3": SimStruct({"inboundRootStoreArraySize": SimTypeInt(signed=False, label="UInt32"), "inboundTrustedRootStoreCriteria": SimTypePointer(SimStruct({"certData": SimStruct({"size": SimTypeInt(signed=False, label="UInt32"), "data": SimTypePointer(SimTypeChar(label="Byte"), offset=0)}, name="FWP_BYTE_BLOB", pack=False, align=None), "certHash": SimStruct({"size": SimTypeInt(signed=False, label="UInt32"), "data": SimTypePointer(SimTypeChar(label="Byte"), offset=0)}, name="FWP_BYTE_BLOB", pack=False, align=None), "eku": SimTypePointer(SimStruct({"numEku": SimTypeInt(signed=False, label="UInt32"), "eku": SimTypePointer(SimTypePointer(SimTypeChar(label="Byte"), offset=0), offset=0)}, name="IKEEXT_CERT_EKUS0", pack=False, align=None), offset=0), "name": SimTypePointer(SimStruct({"nameType": SimTypeInt(signed=False, label="IKEEXT_CERT_CRITERIA_NAME_TYPE"), "certName": SimTypePointer(SimTypeChar(label="Char"), offset=0)}, name="IKEEXT_CERT_NAME0", pack=False, align=None), offset=0), "flags": SimTypeInt(signed=False, label="UInt32")}, name="IKEEXT_CERTIFICATE_CRITERIA0", pack=False, align=None), offset=0)}, name="_Anonymous3_e__Struct", pack=False, align=None)}, name="<anon>", label="None"), "outboundConfigType": SimTypeInt(signed=False, label="IKEEXT_CERT_CONFIG_TYPE"), "Anonymous2": SimUnion({"Anonymous1": SimStruct({"outboundRootArraySize": SimTypeInt(signed=False, label="UInt32"), "outboundRootCriteria": SimTypePointer(SimStruct({"certData": SimStruct({"size": SimTypeInt(signed=False, label="UInt32"), "data": SimTypePointer(SimTypeChar(label="Byte"), offset=0)}, name="FWP_BYTE_BLOB", pack=False, align=None), "certHash": SimStruct({"size": SimTypeInt(signed=False, label="UInt32"), "data": SimTypePointer(SimTypeChar(label="Byte"), offset=0)}, name="FWP_BYTE_BLOB", pack=False, align=None), "eku": SimTypePointer(SimStruct({"numEku": SimTypeInt(signed=False, label="UInt32"), "eku": SimTypePointer(SimTypePointer(SimTypeChar(label="Byte"), offset=0), offset=0)}, name="IKEEXT_CERT_EKUS0", pack=False, align=None), offset=0), "name": SimTypePointer(SimStruct({"nameType": SimTypeInt(signed=False, label="IKEEXT_CERT_CRITERIA_NAME_TYPE"), "certName": SimTypePointer(SimTypeChar(label="Char"), offset=0)}, name="IKEEXT_CERT_NAME0", pack=False, align=None), offset=0), "flags": SimTypeInt(signed=False, label="UInt32")}, name="IKEEXT_CERTIFICATE_CRITERIA0", pack=False, align=None), offset=0)}, name="_Anonymous1_e__Struct", pack=False, align=None), "Anonymous2": SimStruct({"outboundEnterpriseStoreArraySize": SimTypeInt(signed=False, label="UInt32"), "outboundEnterpriseStoreCriteria": SimTypePointer(SimStruct({"certData": SimStruct({"size": SimTypeInt(signed=False, label="UInt32"), "data": SimTypePointer(SimTypeChar(label="Byte"), offset=0)}, name="FWP_BYTE_BLOB", pack=False, align=None), "certHash": SimStruct({"size": SimTypeInt(signed=False, label="UInt32"), "data": SimTypePointer(SimTypeChar(label="Byte"), offset=0)}, name="FWP_BYTE_BLOB", pack=False, align=None), "eku": SimTypePointer(SimStruct({"numEku": SimTypeInt(signed=False, label="UInt32"), "eku": SimTypePointer(SimTypePointer(SimTypeChar(label="Byte"), offset=0), offset=0)}, name="IKEEXT_CERT_EKUS0", pack=False, align=None), offset=0), "name": SimTypePointer(SimStruct({"nameType": SimTypeInt(signed=False, label="IKEEXT_CERT_CRITERIA_NAME_TYPE"), "certName": SimTypePointer(SimTypeChar(label="Char"), offset=0)}, name="IKEEXT_CERT_NAME0", pack=False, align=None), offset=0), "flags": SimTypeInt(signed=False, label="UInt32")}, name="IKEEXT_CERTIFICATE_CRITERIA0", pack=False, align=None), offset=0)}, name="_Anonymous2_e__Struct", pack=False, align=None), "Anonymous3": SimStruct({"outboundRootStoreArraySize": SimTypeInt(signed=False, label="UInt32"), "outboundTrustedRootStoreCriteria": SimTypePointer(SimStruct({"certData": SimStruct({"size": SimTypeInt(signed=False, label="UInt32"), "data": SimTypePointer(SimTypeChar(label="Byte"), offset=0)}, name="FWP_BYTE_BLOB", pack=False, align=None), "certHash": SimStruct({"size": SimTypeInt(signed=False, label="UInt32"), "data": SimTypePointer(SimTypeChar(label="Byte"), offset=0)}, name="FWP_BYTE_BLOB", pack=False, align=None), "eku": SimTypePointer(SimStruct({"numEku": SimTypeInt(signed=False, label="UInt32"), "eku": SimTypePointer(SimTypePointer(SimTypeChar(label="Byte"), offset=0), offset=0)}, name="IKEEXT_CERT_EKUS0", pack=False, align=None), offset=0), "name": SimTypePointer(SimStruct({"nameType": SimTypeInt(signed=False, label="IKEEXT_CERT_CRITERIA_NAME_TYPE"), "certName": SimTypePointer(SimTypeChar(label="Char"), offset=0)}, name="IKEEXT_CERT_NAME0", pack=False, align=None), offset=0), "flags": SimTypeInt(signed=False, label="UInt32")}, name="IKEEXT_CERTIFICATE_CRITERIA0", pack=False, align=None), offset=0)}, name="_Anonymous3_e__Struct", pack=False, align=None)}, name="<anon>", label="None"), "flags": SimTypeInt(signed=False, label="IKEEXT_CERT_AUTH"), "localCertLocationUrl": SimStruct({"size": SimTypeInt(signed=False, label="UInt32"), "data": SimTypePointer(SimTypeChar(label="Byte"), offset=0)}, name="FWP_BYTE_BLOB", pack=False, align=None)}, name="IKEEXT_CERTIFICATE_AUTHENTICATION2", pack=False, align=None), "kerberosAuthentication": SimStruct({"flags": SimTypeInt(signed=False, label="IKEEXT_KERBEROS_AUTHENTICATION_FLAGS"), "proxyServer": SimTypePointer(SimTypeChar(label="Char"), offset=0)}, name="IKEEXT_KERBEROS_AUTHENTICATION1", pack=False, align=None), "reservedAuthentication": SimStruct({"flags": SimTypeInt(signed=False, label="IKEEXT_RESERVED_AUTHENTICATION_FLAGS")}, name="IKEEXT_RESERVED_AUTHENTICATION0", pack=False, align=None), "ntlmV2Authentication": SimStruct({"flags": SimTypeInt(signed=False, label="UInt32")}, name="IKEEXT_NTLM_V2_AUTHENTICATION0", pack=False, align=None), "sslAuthentication": SimStruct({"inboundConfigType": SimTypeInt(signed=False, label="IKEEXT_CERT_CONFIG_TYPE"), "Anonymous1": SimUnion({"Anonymous1": SimStruct({"inboundRootArraySize": SimTypeInt(signed=False, label="UInt32"), "inboundRootCriteria": SimTypePointer(SimStruct({"certData": SimStruct({"size": SimTypeInt(signed=False, label="UInt32"), "data": SimTypePointer(SimTypeChar(label="Byte"), offset=0)}, name="FWP_BYTE_BLOB", pack=False, align=None), "certHash": SimStruct({"size": SimTypeInt(signed=False, label="UInt32"), "data": SimTypePointer(SimTypeChar(label="Byte"), offset=0)}, name="FWP_BYTE_BLOB", pack=False, align=None), "eku": SimTypePointer(SimStruct({"numEku": SimTypeInt(signed=False, label="UInt32"), "eku": SimTypePointer(SimTypePointer(SimTypeChar(label="Byte"), offset=0), offset=0)}, name="IKEEXT_CERT_EKUS0", pack=False, align=None), offset=0), "name": SimTypePointer(SimStruct({"nameType": SimTypeInt(signed=False, label="IKEEXT_CERT_CRITERIA_NAME_TYPE"), "certName": SimTypePointer(SimTypeChar(label="Char"), offset=0)}, name="IKEEXT_CERT_NAME0", pack=False, align=None), offset=0), "flags": SimTypeInt(signed=False, label="UInt32")}, name="IKEEXT_CERTIFICATE_CRITERIA0", pack=False, align=None), offset=0)}, name="_Anonymous1_e__Struct", pack=False, align=None), "Anonymous2": SimStruct({"inboundEnterpriseStoreArraySize": SimTypeInt(signed=False, label="UInt32"), "inboundEnterpriseStoreCriteria": SimTypePointer(SimStruct({"certData": SimStruct({"size": SimTypeInt(signed=False, label="UInt32"), "data": SimTypePointer(SimTypeChar(label="Byte"), offset=0)}, name="FWP_BYTE_BLOB", pack=False, align=None), "certHash": SimStruct({"size": SimTypeInt(signed=False, label="UInt32"), "data": SimTypePointer(SimTypeChar(label="Byte"), offset=0)}, name="FWP_BYTE_BLOB", pack=False, align=None), "eku": SimTypePointer(SimStruct({"numEku": SimTypeInt(signed=False, label="UInt32"), "eku": SimTypePointer(SimTypePointer(SimTypeChar(label="Byte"), offset=0), offset=0)}, name="IKEEXT_CERT_EKUS0", pack=False, align=None), offset=0), "name": SimTypePointer(SimStruct({"nameType": SimTypeInt(signed=False, label="IKEEXT_CERT_CRITERIA_NAME_TYPE"), "certName": SimTypePointer(SimTypeChar(label="Char"), offset=0)}, name="IKEEXT_CERT_NAME0", pack=False, align=None), offset=0), "flags": SimTypeInt(signed=False, label="UInt32")}, name="IKEEXT_CERTIFICATE_CRITERIA0", pack=False, align=None), offset=0)}, name="_Anonymous2_e__Struct", pack=False, align=None), "Anonymous3": SimStruct({"inboundRootStoreArraySize": SimTypeInt(signed=False, label="UInt32"), "inboundTrustedRootStoreCriteria": SimTypePointer(SimStruct({"certData": SimStruct({"size": SimTypeInt(signed=False, label="UInt32"), "data": SimTypePointer(SimTypeChar(label="Byte"), offset=0)}, name="FWP_BYTE_BLOB", pack=False, align=None), "certHash": SimStruct({"size": SimTypeInt(signed=False, label="UInt32"), "data": SimTypePointer(SimTypeChar(label="Byte"), offset=0)}, name="FWP_BYTE_BLOB", pack=False, align=None), "eku": SimTypePointer(SimStruct({"numEku": SimTypeInt(signed=False, label="UInt32"), "eku": SimTypePointer(SimTypePointer(SimTypeChar(label="Byte"), offset=0), offset=0)}, name="IKEEXT_CERT_EKUS0", pack=False, align=None), offset=0), "name": SimTypePointer(SimStruct({"nameType": SimTypeInt(signed=False, label="IKEEXT_CERT_CRITERIA_NAME_TYPE"), "certName": SimTypePointer(SimTypeChar(label="Char"), offset=0)}, name="IKEEXT_CERT_NAME0", pack=False, align=None), offset=0), "flags": SimTypeInt(signed=False, label="UInt32")}, name="IKEEXT_CERTIFICATE_CRITERIA0", pack=False, align=None), offset=0)}, name="_Anonymous3_e__Struct", pack=False, align=None)}, name="<anon>", label="None"), "outboundConfigType": SimTypeInt(signed=False, label="IKEEXT_CERT_CONFIG_TYPE"), "Anonymous2": SimUnion({"Anonymous1": SimStruct({"outboundRootArraySize": SimTypeInt(signed=False, label="UInt32"), "outboundRootCriteria": SimTypePointer(SimStruct({"certData": SimStruct({"size": SimTypeInt(signed=False, label="UInt32"), "data": SimTypePointer(SimTypeChar(label="Byte"), offset=0)}, name="FWP_BYTE_BLOB", pack=False, align=None), "certHash": SimStruct({"size": SimTypeInt(signed=False, label="UInt32"), "data": SimTypePointer(SimTypeChar(label="Byte"), offset=0)}, name="FWP_BYTE_BLOB", pack=False, align=None), "eku": SimTypePointer(SimStruct({"numEku": SimTypeInt(signed=False, label="UInt32"), "eku": SimTypePointer(SimTypePointer(SimTypeChar(label="Byte"), offset=0), offset=0)}, name="IKEEXT_CERT_EKUS0", pack=False, align=None), offset=0), "name": SimTypePointer(SimStruct({"nameType": SimTypeInt(signed=False, label="IKEEXT_CERT_CRITERIA_NAME_TYPE"), "certName": SimTypePointer(SimTypeChar(label="Char"), offset=0)}, name="IKEEXT_CERT_NAME0", pack=False, align=None), offset=0), "flags": SimTypeInt(signed=False, label="UInt32")}, name="IKEEXT_CERTIFICATE_CRITERIA0", pack=False, align=None), offset=0)}, name="_Anonymous1_e__Struct", pack=False, align=None), "Anonymous2": SimStruct({"outboundEnterpriseStoreArraySize": SimTypeInt(signed=False, label="UInt32"), "outboundEnterpriseStoreCriteria": SimTypePointer(SimStruct({"certData": SimStruct({"size": SimTypeInt(signed=False, label="UInt32"), "data": SimTypePointer(SimTypeChar(label="Byte"), offset=0)}, name="FWP_BYTE_BLOB", pack=False, align=None), "certHash": SimStruct({"size": SimTypeInt(signed=False, label="UInt32"), "data": SimTypePointer(SimTypeChar(label="Byte"), offset=0)}, name="FWP_BYTE_BLOB", pack=False, align=None), "eku": SimTypePointer(SimStruct({"numEku": SimTypeInt(signed=False, label="UInt32"), "eku": SimTypePointer(SimTypePointer(SimTypeChar(label="Byte"), offset=0), offset=0)}, name="IKEEXT_CERT_EKUS0", pack=False, align=None), offset=0), "name": SimTypePointer(SimStruct({"nameType": SimTypeInt(signed=False, label="IKEEXT_CERT_CRITERIA_NAME_TYPE"), "certName": SimTypePointer(SimTypeChar(label="Char"), offset=0)}, name="IKEEXT_CERT_NAME0", pack=False, align=None), offset=0), "flags": SimTypeInt(signed=False, label="UInt32")}, name="IKEEXT_CERTIFICATE_CRITERIA0", pack=False, align=None), offset=0)}, name="_Anonymous2_e__Struct", pack=False, align=None), "Anonymous3": SimStruct({"outboundRootStoreArraySize": SimTypeInt(signed=False, label="UInt32"), "outboundTrustedRootStoreCriteria": SimTypePointer(SimStruct({"certData": SimStruct({"size": SimTypeInt(signed=False, label="UInt32"), "data": SimTypePointer(SimTypeChar(label="Byte"), offset=0)}, name="FWP_BYTE_BLOB", pack=False, align=None), "certHash": SimStruct({"size": SimTypeInt(signed=False, label="UInt32"), "data": SimTypePointer(SimTypeChar(label="Byte"), offset=0)}, name="FWP_BYTE_BLOB", pack=False, align=None), "eku": SimTypePointer(SimStruct({"numEku": SimTypeInt(signed=False, label="UInt32"), "eku": SimTypePointer(SimTypePointer(SimTypeChar(label="Byte"), offset=0), offset=0)}, name="IKEEXT_CERT_EKUS0", pack=False, align=None), offset=0), "name": SimTypePointer(SimStruct({"nameType": SimTypeInt(signed=False, label="IKEEXT_CERT_CRITERIA_NAME_TYPE"), "certName": SimTypePointer(SimTypeChar(label="Char"), offset=0)}, name="IKEEXT_CERT_NAME0", pack=False, align=None), offset=0), "flags": SimTypeInt(signed=False, label="UInt32")}, name="IKEEXT_CERTIFICATE_CRITERIA0", pack=False, align=None), offset=0)}, name="_Anonymous3_e__Struct", pack=False, align=None)}, name="<anon>", label="None"), "flags": SimTypeInt(signed=False, label="IKEEXT_CERT_AUTH"), "localCertLocationUrl": SimStruct({"size": SimTypeInt(signed=False, label="UInt32"), "data": SimTypePointer(SimTypeChar(label="Byte"), offset=0)}, name="FWP_BYTE_BLOB", pack=False, align=None)}, name="IKEEXT_CERTIFICATE_AUTHENTICATION2", pack=False, align=None), "cgaAuthentication": SimStruct({"keyContainerName": SimTypePointer(SimTypeChar(label="Char"), offset=0), "cspName": SimTypePointer(SimTypeChar(label="Char"), offset=0), "cspType": SimTypeInt(signed=False, label="UInt32"), "cgaModifier": SimStruct({"byteArray16": SimTypeFixedSizeArray(SimTypeChar(label="Byte"), 16)}, name="FWP_BYTE_ARRAY16", pack=False, align=None), "cgaCollisionCount": SimTypeChar(label="Byte")}, name="IKEEXT_IPV6_CGA_AUTHENTICATION0", pack=False, align=None), "eapAuthentication": SimStruct({"flags": SimTypeInt(signed=False, label="IKEEXT_EAP_AUTHENTICATION_FLAGS")}, name="IKEEXT_EAP_AUTHENTICATION0", pack=False, align=None)}, name="<anon>", label="None")}, name="IKEEXT_AUTHENTICATION_METHOD2", pack=False, align=None), offset=0), "initiatorImpersonationType": SimTypeInt(signed=False, label="IKEEXT_AUTHENTICATION_IMPERSONATION_TYPE")}, name="IKEEXT_EM_POLICY2", pack=False, align=None), offset=0)}, name="IPSEC_TRANSPORT_POLICY2", pack=False, align=None), offset=0), "ikeV2MmPolicy": SimTypePointer(SimStruct({"softExpirationTime": SimTypeInt(signed=False, label="UInt32"), "numAuthenticationMethods": SimTypeInt(signed=False, label="UInt32"), "authenticationMethods": SimTypePointer(SimStruct({"authenticationMethodType": SimTypeInt(signed=False, label="IKEEXT_AUTHENTICATION_METHOD_TYPE"), "Anonymous": SimUnion({"presharedKeyAuthentication": SimStruct({"presharedKey": SimStruct({"size": SimTypeInt(signed=False, label="UInt32"), "data": SimTypePointer(SimTypeChar(label="Byte"), offset=0)}, name="FWP_BYTE_BLOB", pack=False, align=None), "flags": SimTypeInt(signed=False, label="IKEEXT_PRESHARED_KEY_AUTHENTICATION_FLAGS")}, name="IKEEXT_PRESHARED_KEY_AUTHENTICATION1", pack=False, align=None), "certificateAuthentication": SimStruct({"inboundConfigType": SimTypeInt(signed=False, label="IKEEXT_CERT_CONFIG_TYPE"), "Anonymous1": SimUnion({"Anonymous1": SimStruct({"inboundRootArraySize": SimTypeInt(signed=False, label="UInt32"), "inboundRootCriteria": SimTypePointer(SimStruct({"certData": SimStruct({"size": SimTypeInt(signed=False, label="UInt32"), "data": SimTypePointer(SimTypeChar(label="Byte"), offset=0)}, name="FWP_BYTE_BLOB", pack=False, align=None), "certHash": SimStruct({"size": SimTypeInt(signed=False, label="UInt32"), "data": SimTypePointer(SimTypeChar(label="Byte"), offset=0)}, name="FWP_BYTE_BLOB", pack=False, align=None), "eku": SimTypePointer(SimStruct({"numEku": SimTypeInt(signed=False, label="UInt32"), "eku": SimTypePointer(SimTypePointer(SimTypeChar(label="Byte"), offset=0), offset=0)}, name="IKEEXT_CERT_EKUS0", pack=False, align=None), offset=0), "name": SimTypePointer(SimStruct({"nameType": SimTypeInt(signed=False, label="IKEEXT_CERT_CRITERIA_NAME_TYPE"), "certName": SimTypePointer(SimTypeChar(label="Char"), offset=0)}, name="IKEEXT_CERT_NAME0", pack=False, align=None), offset=0), "flags": SimTypeInt(signed=False, label="UInt32")}, name="IKEEXT_CERTIFICATE_CRITERIA0", pack=False, align=None), offset=0)}, name="_Anonymous1_e__Struct", pack=False, align=None), "Anonymous2": SimStruct({"inboundEnterpriseStoreArraySize": SimTypeInt(signed=False, label="UInt32"), "inboundEnterpriseStoreCriteria": SimTypePointer(SimStruct({"certData": SimStruct({"size": SimTypeInt(signed=False, label="UInt32"), "data": SimTypePointer(SimTypeChar(label="Byte"), offset=0)}, name="FWP_BYTE_BLOB", pack=False, align=None), "certHash": SimStruct({"size": SimTypeInt(signed=False, label="UInt32"), "data": SimTypePointer(SimTypeChar(label="Byte"), offset=0)}, name="FWP_BYTE_BLOB", pack=False, align=None), "eku": SimTypePointer(SimStruct({"numEku": SimTypeInt(signed=False, label="UInt32"), "eku": SimTypePointer(SimTypePointer(SimTypeChar(label="Byte"), offset=0), offset=0)}, name="IKEEXT_CERT_EKUS0", pack=False, align=None), offset=0), "name": SimTypePointer(SimStruct({"nameType": SimTypeInt(signed=False, label="IKEEXT_CERT_CRITERIA_NAME_TYPE"), "certName": SimTypePointer(SimTypeChar(label="Char"), offset=0)}, name="IKEEXT_CERT_NAME0", pack=False, align=None), offset=0), "flags": SimTypeInt(signed=False, label="UInt32")}, name="IKEEXT_CERTIFICATE_CRITERIA0", pack=False, align=None), offset=0)}, name="_Anonymous2_e__Struct", pack=False, align=None), "Anonymous3": SimStruct({"inboundRootStoreArraySize": SimTypeInt(signed=False, label="UInt32"), "inboundTrustedRootStoreCriteria": SimTypePointer(SimStruct({"certData": SimStruct({"size": SimTypeInt(signed=False, label="UInt32"), "data": SimTypePointer(SimTypeChar(label="Byte"), offset=0)}, name="FWP_BYTE_BLOB", pack=False, align=None), "certHash": SimStruct({"size": SimTypeInt(signed=False, label="UInt32"), "data": SimTypePointer(SimTypeChar(label="Byte"), offset=0)}, name="FWP_BYTE_BLOB", pack=False, align=None), "eku": SimTypePointer(SimStruct({"numEku": SimTypeInt(signed=False, label="UInt32"), "eku": SimTypePointer(SimTypePointer(SimTypeChar(label="Byte"), offset=0), offset=0)}, name="IKEEXT_CERT_EKUS0", pack=False, align=None), offset=0), "name": SimTypePointer(SimStruct({"nameType": SimTypeInt(signed=False, label="IKEEXT_CERT_CRITERIA_NAME_TYPE"), "certName": SimTypePointer(SimTypeChar(label="Char"), offset=0)}, name="IKEEXT_CERT_NAME0", pack=False, align=None), offset=0), "flags": SimTypeInt(signed=False, label="UInt32")}, name="IKEEXT_CERTIFICATE_CRITERIA0", pack=False, align=None), offset=0)}, name="_Anonymous3_e__Struct", pack=False, align=None)}, name="<anon>", label="None"), "outboundConfigType": SimTypeInt(signed=False, label="IKEEXT_CERT_CONFIG_TYPE"), "Anonymous2": SimUnion({"Anonymous1": SimStruct({"outboundRootArraySize": SimTypeInt(signed=False, label="UInt32"), "outboundRootCriteria": SimTypePointer(SimStruct({"certData": SimStruct({"size": SimTypeInt(signed=False, label="UInt32"), "data": SimTypePointer(SimTypeChar(label="Byte"), offset=0)}, name="FWP_BYTE_BLOB", pack=False, align=None), "certHash": SimStruct({"size": SimTypeInt(signed=False, label="UInt32"), "data": SimTypePointer(SimTypeChar(label="Byte"), offset=0)}, name="FWP_BYTE_BLOB", pack=False, align=None), "eku": SimTypePointer(SimStruct({"numEku": SimTypeInt(signed=False, label="UInt32"), "eku": SimTypePointer(SimTypePointer(SimTypeChar(label="Byte"), offset=0), offset=0)}, name="IKEEXT_CERT_EKUS0", pack=False, align=None), offset=0), "name": SimTypePointer(SimStruct({"nameType": SimTypeInt(signed=False, label="IKEEXT_CERT_CRITERIA_NAME_TYPE"), "certName": SimTypePointer(SimTypeChar(label="Char"), offset=0)}, name="IKEEXT_CERT_NAME0", pack=False, align=None), offset=0), "flags": SimTypeInt(signed=False, label="UInt32")}, name="IKEEXT_CERTIFICATE_CRITERIA0", pack=False, align=None), offset=0)}, name="_Anonymous1_e__Struct", pack=False, align=None), "Anonymous2": SimStruct({"outboundEnterpriseStoreArraySize": SimTypeInt(signed=False, label="UInt32"), "outboundEnterpriseStoreCriteria": SimTypePointer(SimStruct({"certData": SimStruct({"size": SimTypeInt(signed=False, label="UInt32"), "data": SimTypePointer(SimTypeChar(label="Byte"), offset=0)}, name="FWP_BYTE_BLOB", pack=False, align=None), "certHash": SimStruct({"size": SimTypeInt(signed=False, label="UInt32"), "data": SimTypePointer(SimTypeChar(label="Byte"), offset=0)}, name="FWP_BYTE_BLOB", pack=False, align=None), "eku": SimTypePointer(SimStruct({"numEku": SimTypeInt(signed=False, label="UInt32"), "eku": SimTypePointer(SimTypePointer(SimTypeChar(label="Byte"), offset=0), offset=0)}, name="IKEEXT_CERT_EKUS0", pack=False, align=None), offset=0), "name": SimTypePointer(SimStruct({"nameType": SimTypeInt(signed=False, label="IKEEXT_CERT_CRITERIA_NAME_TYPE"), "certName": SimTypePointer(SimTypeChar(label="Char"), offset=0)}, name="IKEEXT_CERT_NAME0", pack=False, align=None), offset=0), "flags": SimTypeInt(signed=False, label="UInt32")}, name="IKEEXT_CERTIFICATE_CRITERIA0", pack=False, align=None), offset=0)}, name="_Anonymous2_e__Struct", pack=False, align=None), "Anonymous3": SimStruct({"outboundRootStoreArraySize": SimTypeInt(signed=False, label="UInt32"), "outboundTrustedRootStoreCriteria": SimTypePointer(SimStruct({"certData": SimStruct({"size": SimTypeInt(signed=False, label="UInt32"), "data": SimTypePointer(SimTypeChar(label="Byte"), offset=0)}, name="FWP_BYTE_BLOB", pack=False, align=None), "certHash": SimStruct({"size": SimTypeInt(signed=False, label="UInt32"), "data": SimTypePointer(SimTypeChar(label="Byte"), offset=0)}, name="FWP_BYTE_BLOB", pack=False, align=None), "eku": SimTypePointer(SimStruct({"numEku": SimTypeInt(signed=False, label="UInt32"), "eku": SimTypePointer(SimTypePointer(SimTypeChar(label="Byte"), offset=0), offset=0)}, name="IKEEXT_CERT_EKUS0", pack=False, align=None), offset=0), "name": SimTypePointer(SimStruct({"nameType": SimTypeInt(signed=False, label="IKEEXT_CERT_CRITERIA_NAME_TYPE"), "certName": SimTypePointer(SimTypeChar(label="Char"), offset=0)}, name="IKEEXT_CERT_NAME0", pack=False, align=None), offset=0), "flags": SimTypeInt(signed=False, label="UInt32")}, name="IKEEXT_CERTIFICATE_CRITERIA0", pack=False, align=None), offset=0)}, name="_Anonymous3_e__Struct", pack=False, align=None)}, name="<anon>", label="None"), "flags": SimTypeInt(signed=False, label="IKEEXT_CERT_AUTH"), "localCertLocationUrl": SimStruct({"size": SimTypeInt(signed=False, label="UInt32"), "data": SimTypePointer(SimTypeChar(label="Byte"), offset=0)}, name="FWP_BYTE_BLOB", pack=False, align=None)}, name="IKEEXT_CERTIFICATE_AUTHENTICATION2", pack=False, align=None), "kerberosAuthentication": SimStruct({"flags": SimTypeInt(signed=False, label="IKEEXT_KERBEROS_AUTHENTICATION_FLAGS"), "proxyServer": SimTypePointer(SimTypeChar(label="Char"), offset=0)}, name="IKEEXT_KERBEROS_AUTHENTICATION1", pack=False, align=None), "reservedAuthentication": SimStruct({"flags": SimTypeInt(signed=False, label="IKEEXT_RESERVED_AUTHENTICATION_FLAGS")}, name="IKEEXT_RESERVED_AUTHENTICATION0", pack=False, align=None), "ntlmV2Authentication": SimStruct({"flags": SimTypeInt(signed=False, label="UInt32")}, name="IKEEXT_NTLM_V2_AUTHENTICATION0", pack=False, align=None), "sslAuthentication": SimStruct({"inboundConfigType": SimTypeInt(signed=False, label="IKEEXT_CERT_CONFIG_TYPE"), "Anonymous1": SimUnion({"Anonymous1": SimStruct({"inboundRootArraySize": SimTypeInt(signed=False, label="UInt32"), "inboundRootCriteria": SimTypePointer(SimStruct({"certData": SimStruct({"size": SimTypeInt(signed=False, label="UInt32"), "data": SimTypePointer(SimTypeChar(label="Byte"), offset=0)}, name="FWP_BYTE_BLOB", pack=False, align=None), "certHash": SimStruct({"size": SimTypeInt(signed=False, label="UInt32"), "data": SimTypePointer(SimTypeChar(label="Byte"), offset=0)}, name="FWP_BYTE_BLOB", pack=False, align=None), "eku": SimTypePointer(SimStruct({"numEku": SimTypeInt(signed=False, label="UInt32"), "eku": SimTypePointer(SimTypePointer(SimTypeChar(label="Byte"), offset=0), offset=0)}, name="IKEEXT_CERT_EKUS0", pack=False, align=None), offset=0), "name": SimTypePointer(SimStruct({"nameType": SimTypeInt(signed=False, label="IKEEXT_CERT_CRITERIA_NAME_TYPE"), "certName": SimTypePointer(SimTypeChar(label="Char"), offset=0)}, name="IKEEXT_CERT_NAME0", pack=False, align=None), offset=0), "flags": SimTypeInt(signed=False, label="UInt32")}, name="IKEEXT_CERTIFICATE_CRITERIA0", pack=False, align=None), offset=0)}, name="_Anonymous1_e__Struct", pack=False, align=None), "Anonymous2": SimStruct({"inboundEnterpriseStoreArraySize": SimTypeInt(signed=False, label="UInt32"), "inboundEnterpriseStoreCriteria": SimTypePointer(SimStruct({"certData": SimStruct({"size": SimTypeInt(signed=False, label="UInt32"), "data": SimTypePointer(SimTypeChar(label="Byte"), offset=0)}, name="FWP_BYTE_BLOB", pack=False, align=None), "certHash": SimStruct({"size": SimTypeInt(signed=False, label="UInt32"), "data": SimTypePointer(SimTypeChar(label="Byte"), offset=0)}, name="FWP_BYTE_BLOB", pack=False, align=None), "eku": SimTypePointer(SimStruct({"numEku": SimTypeInt(signed=False, label="UInt32"), "eku": SimTypePointer(SimTypePointer(SimTypeChar(label="Byte"), offset=0), offset=0)}, name="IKEEXT_CERT_EKUS0", pack=False, align=None), offset=0), "name": SimTypePointer(SimStruct({"nameType": SimTypeInt(signed=False, label="IKEEXT_CERT_CRITERIA_NAME_TYPE"), "certName": SimTypePointer(SimTypeChar(label="Char"), offset=0)}, name="IKEEXT_CERT_NAME0", pack=False, align=None), offset=0), "flags": SimTypeInt(signed=False, label="UInt32")}, name="IKEEXT_CERTIFICATE_CRITERIA0", pack=False, align=None), offset=0)}, name="_Anonymous2_e__Struct", pack=False, align=None), "Anonymous3": SimStruct({"inboundRootStoreArraySize": SimTypeInt(signed=False, label="UInt32"), "inboundTrustedRootStoreCriteria": SimTypePointer(SimStruct({"certData": SimStruct({"size": SimTypeInt(signed=False, label="UInt32"), "data": SimTypePointer(SimTypeChar(label="Byte"), offset=0)}, name="FWP_BYTE_BLOB", pack=False, align=None), "certHash": SimStruct({"size": SimTypeInt(signed=False, label="UInt32"), "data": SimTypePointer(SimTypeChar(label="Byte"), offset=0)}, name="FWP_BYTE_BLOB", pack=False, align=None), "eku": SimTypePointer(SimStruct({"numEku": SimTypeInt(signed=False, label="UInt32"), "eku": SimTypePointer(SimTypePointer(SimTypeChar(label="Byte"), offset=0), offset=0)}, name="IKEEXT_CERT_EKUS0", pack=False, align=None), offset=0), "name": SimTypePointer(SimStruct({"nameType": SimTypeInt(signed=False, label="IKEEXT_CERT_CRITERIA_NAME_TYPE"), "certName": SimTypePointer(SimTypeChar(label="Char"), offset=0)}, name="IKEEXT_CERT_NAME0", pack=False, align=None), offset=0), "flags": SimTypeInt(signed=False, label="UInt32")}, name="IKEEXT_CERTIFICATE_CRITERIA0", pack=False, align=None), offset=0)}, name="_Anonymous3_e__Struct", pack=False, align=None)}, name="<anon>", label="None"), "outboundConfigType": SimTypeInt(signed=False, label="IKEEXT_CERT_CONFIG_TYPE"), "Anonymous2": SimUnion({"Anonymous1": SimStruct({"outboundRootArraySize": SimTypeInt(signed=False, label="UInt32"), "outboundRootCriteria": SimTypePointer(SimStruct({"certData": SimStruct({"size": SimTypeInt(signed=False, label="UInt32"), "data": SimTypePointer(SimTypeChar(label="Byte"), offset=0)}, name="FWP_BYTE_BLOB", pack=False, align=None), "certHash": SimStruct({"size": SimTypeInt(signed=False, label="UInt32"), "data": SimTypePointer(SimTypeChar(label="Byte"), offset=0)}, name="FWP_BYTE_BLOB", pack=False, align=None), "eku": SimTypePointer(SimStruct({"numEku": SimTypeInt(signed=False, label="UInt32"), "eku": SimTypePointer(SimTypePointer(SimTypeChar(label="Byte"), offset=0), offset=0)}, name="IKEEXT_CERT_EKUS0", pack=False, align=None), offset=0), "name": SimTypePointer(SimStruct({"nameType": SimTypeInt(signed=False, label="IKEEXT_CERT_CRITERIA_NAME_TYPE"), "certName": SimTypePointer(SimTypeChar(label="Char"), offset=0)}, name="IKEEXT_CERT_NAME0", pack=False, align=None), offset=0), "flags": SimTypeInt(signed=False, label="UInt32")}, name="IKEEXT_CERTIFICATE_CRITERIA0", pack=False, align=None), offset=0)}, name="_Anonymous1_e__Struct", pack=False, align=None), "Anonymous2": SimStruct({"outboundEnterpriseStoreArraySize": SimTypeInt(signed=False, label="UInt32"), "outboundEnterpriseStoreCriteria": SimTypePointer(SimStruct({"certData": SimStruct({"size": SimTypeInt(signed=False, label="UInt32"), "data": SimTypePointer(SimTypeChar(label="Byte"), offset=0)}, name="FWP_BYTE_BLOB", pack=False, align=None), "certHash": SimStruct({"size": SimTypeInt(signed=False, label="UInt32"), "data": SimTypePointer(SimTypeChar(label="Byte"), offset=0)}, name="FWP_BYTE_BLOB", pack=False, align=None), "eku": SimTypePointer(SimStruct({"numEku": SimTypeInt(signed=False, label="UInt32"), "eku": SimTypePointer(SimTypePointer(SimTypeChar(label="Byte"), offset=0), offset=0)}, name="IKEEXT_CERT_EKUS0", pack=False, align=None), offset=0), "name": SimTypePointer(SimStruct({"nameType": SimTypeInt(signed=False, label="IKEEXT_CERT_CRITERIA_NAME_TYPE"), "certName": SimTypePointer(SimTypeChar(label="Char"), offset=0)}, name="IKEEXT_CERT_NAME0", pack=False, align=None), offset=0), "flags": SimTypeInt(signed=False, label="UInt32")}, name="IKEEXT_CERTIFICATE_CRITERIA0", pack=False, align=None), offset=0)}, name="_Anonymous2_e__Struct", pack=False, align=None), "Anonymous3": SimStruct({"outboundRootStoreArraySize": SimTypeInt(signed=False, label="UInt32"), "outboundTrustedRootStoreCriteria": SimTypePointer(SimStruct({"certData": SimStruct({"size": SimTypeInt(signed=False, label="UInt32"), "data": SimTypePointer(SimTypeChar(label="Byte"), offset=0)}, name="FWP_BYTE_BLOB", pack=False, align=None), "certHash": SimStruct({"size": SimTypeInt(signed=False, label="UInt32"), "data": SimTypePointer(SimTypeChar(label="Byte"), offset=0)}, name="FWP_BYTE_BLOB", pack=False, align=None), "eku": SimTypePointer(SimStruct({"numEku": SimTypeInt(signed=False, label="UInt32"), "eku": SimTypePointer(SimTypePointer(SimTypeChar(label="Byte"), offset=0), offset=0)}, name="IKEEXT_CERT_EKUS0", pack=False, align=None), offset=0), "name": SimTypePointer(SimStruct({"nameType": SimTypeInt(signed=False, label="IKEEXT_CERT_CRITERIA_NAME_TYPE"), "certName": SimTypePointer(SimTypeChar(label="Char"), offset=0)}, name="IKEEXT_CERT_NAME0", pack=False, align=None), offset=0), "flags": SimTypeInt(signed=False, label="UInt32")}, name="IKEEXT_CERTIFICATE_CRITERIA0", pack=False, align=None), offset=0)}, name="_Anonymous3_e__Struct", pack=False, align=None)}, name="<anon>", label="None"), "flags": SimTypeInt(signed=False, label="IKEEXT_CERT_AUTH"), "localCertLocationUrl": SimStruct({"size": SimTypeInt(signed=False, label="UInt32"), "data": SimTypePointer(SimTypeChar(label="Byte"), offset=0)}, name="FWP_BYTE_BLOB", pack=False, align=None)}, name="IKEEXT_CERTIFICATE_AUTHENTICATION2", pack=False, align=None), "cgaAuthentication": SimStruct({"keyContainerName": SimTypePointer(SimTypeChar(label="Char"), offset=0), "cspName": SimTypePointer(SimTypeChar(label="Char"), offset=0), "cspType": SimTypeInt(signed=False, label="UInt32"), "cgaModifier": SimStruct({"byteArray16": SimTypeFixedSizeArray(SimTypeChar(label="Byte"), 16)}, name="FWP_BYTE_ARRAY16", pack=False, align=None), "cgaCollisionCount": SimTypeChar(label="Byte")}, name="IKEEXT_IPV6_CGA_AUTHENTICATION0", pack=False, align=None), "eapAuthentication": SimStruct({"flags": SimTypeInt(signed=False, label="IKEEXT_EAP_AUTHENTICATION_FLAGS")}, name="IKEEXT_EAP_AUTHENTICATION0", pack=False, align=None)}, name="<anon>", label="None")}, name="IKEEXT_AUTHENTICATION_METHOD2", pack=False, align=None), offset=0), "initiatorImpersonationType": SimTypeInt(signed=False, label="IKEEXT_AUTHENTICATION_IMPERSONATION_TYPE"), "numIkeProposals": SimTypeInt(signed=False, label="UInt32"), "ikeProposals": SimTypePointer(SimStruct({"cipherAlgorithm": SimStruct({"algoIdentifier": SimTypeInt(signed=False, label="IKEEXT_CIPHER_TYPE"), "keyLen": SimTypeInt(signed=False, label="UInt32"), "rounds": SimTypeInt(signed=False, label="UInt32")}, name="IKEEXT_CIPHER_ALGORITHM0", pack=False, align=None), "integrityAlgorithm": SimStruct({"algoIdentifier": SimTypeInt(signed=False, label="IKEEXT_INTEGRITY_TYPE")}, name="IKEEXT_INTEGRITY_ALGORITHM0", pack=False, align=None), "maxLifetimeSeconds": SimTypeInt(signed=False, label="UInt32"), "dhGroup": SimTypeInt(signed=False, label="IKEEXT_DH_GROUP"), "quickModeLimit": SimTypeInt(signed=False, label="UInt32")}, name="IKEEXT_PROPOSAL0", pack=False, align=None), offset=0), "flags": SimTypeInt(signed=False, label="IKEEXT_POLICY_FLAG"), "maxDynamicFilters": SimTypeInt(signed=False, label="UInt32"), "retransmitDurationSecs": SimTypeInt(signed=False, label="UInt32")}, name="IKEEXT_POLICY2", pack=False, align=None), offset=0), "idpOptions": SimTypePointer(SimStruct({"stateIdleTimeoutSeconds": SimTypeInt(signed=False, label="UInt32"), "perIPRateLimitQueueIdleTimeoutSeconds": SimTypeInt(signed=False, label="UInt32"), "ipV6IPsecUnauthDscp": SimTypeChar(label="Byte"), "ipV6IPsecUnauthRateLimitBytesPerSec": SimTypeInt(signed=False, label="UInt32"), "ipV6IPsecUnauthPerIPRateLimitBytesPerSec": SimTypeInt(signed=False, label="UInt32"), "ipV6IPsecAuthDscp": SimTypeChar(label="Byte"), "ipV6IPsecAuthRateLimitBytesPerSec": SimTypeInt(signed=False, label="UInt32"), "icmpV6Dscp": SimTypeChar(label="Byte"), "icmpV6RateLimitBytesPerSec": SimTypeInt(signed=False, label="UInt32"), "ipV6FilterExemptDscp": SimTypeChar(label="Byte"), "ipV6FilterExemptRateLimitBytesPerSec": SimTypeInt(signed=False, label="UInt32"), "defBlockExemptDscp": SimTypeChar(label="Byte"), "defBlockExemptRateLimitBytesPerSec": SimTypeInt(signed=False, label="UInt32"), "maxStateEntries": SimTypeInt(signed=False, label="UInt32"), "maxPerIPRateLimitQueues": SimTypeInt(signed=False, label="UInt32"), "flags": SimTypeInt(signed=False, label="IPSEC_DOSP_FLAGS"), "numPublicIFLuids": SimTypeInt(signed=False, label="UInt32"), "publicIFLuids": SimTypePointer(SimTypeLongLong(signed=False, label="UInt64"), offset=0), "numInternalIFLuids": SimTypeInt(signed=False, label="UInt32"), "internalIFLuids": SimTypePointer(SimTypeLongLong(signed=False, label="UInt64"), offset=0), "publicV6AddrMask": SimStruct({"addr": SimTypeFixedSizeArray(SimTypeChar(label="Byte"), 16), "prefixLength": SimTypeChar(label="Byte")}, name="FWP_V6_ADDR_AND_MASK", pack=False, align=None), "internalV6AddrMask": SimStruct({"addr": SimTypeFixedSizeArray(SimTypeChar(label="Byte"), 16), "prefixLength": SimTypeChar(label="Byte")}, name="FWP_V6_ADDR_AND_MASK", pack=False, align=None)}, name="IPSEC_DOSP_OPTIONS0", pack=False, align=None), offset=0)}, name="<anon>", label="None"), "providerContextId": SimTypeLongLong(signed=False, label="UInt64")}, name="FWPM_PROVIDER_CONTEXT3_", pack=False, align=None), offset=0), SimTypeInt(signed=False, label="UInt32"), SimTypePointer(SimStruct({"fieldKey": SimTypeBottom(label="Guid"), "matchType": SimTypeInt(signed=False, label="FWP_MATCH_TYPE"), "conditionValue": SimStruct({"type": SimTypeInt(signed=False, label="FWP_DATA_TYPE"), "Anonymous": SimUnion({"uint8": SimTypeChar(label="Byte"), "uint16": SimTypeShort(signed=False, label="UInt16"), "uint32": SimTypeInt(signed=False, label="UInt32"), "uint64": SimTypePointer(SimTypeLongLong(signed=False, label="UInt64"), offset=0), "int8": SimTypeChar(label="SByte"), "int16": SimTypeShort(signed=True, label="Int16"), "int32": SimTypeInt(signed=True, label="Int32"), "int64": SimTypePointer(SimTypeLongLong(signed=True, label="Int64"), offset=0), "float32": SimTypeFloat(size=32), "double64": SimTypePointer(SimTypeFloat(size=64), offset=0), "byteArray16": SimTypePointer(SimStruct({"byteArray16": SimTypeFixedSizeArray(SimTypeChar(label="Byte"), 16)}, name="FWP_BYTE_ARRAY16", pack=False, align=None), offset=0), "byteBlob": SimTypePointer(SimStruct({"size": SimTypeInt(signed=False, label="UInt32"), "data": SimTypePointer(SimTypeChar(label="Byte"), offset=0)}, name="FWP_BYTE_BLOB", pack=False, align=None), offset=0), "sid": SimTypePointer(SimTypeBottom(label="SID"), offset=0), "sd": SimTypePointer(SimStruct({"size": SimTypeInt(signed=False, label="UInt32"), "data": SimTypePointer(SimTypeChar(label="Byte"), offset=0)}, name="FWP_BYTE_BLOB", pack=False, align=None), offset=0), "tokenInformation": SimTypePointer(SimStruct({"sidCount": SimTypeInt(signed=False, label="UInt32"), "sids": SimTypePointer(SimTypeBottom(label="SID_AND_ATTRIBUTES"), offset=0), "restrictedSidCount": SimTypeInt(signed=False, label="UInt32"), "restrictedSids": SimTypePointer(SimTypeBottom(label="SID_AND_ATTRIBUTES"), offset=0)}, name="FWP_TOKEN_INFORMATION", pack=False, align=None), offset=0), "tokenAccessInformation": SimTypePointer(SimStruct({"size": SimTypeInt(signed=False, label="UInt32"), "data": SimTypePointer(SimTypeChar(label="Byte"), offset=0)}, name="FWP_BYTE_BLOB", pack=False, align=None), offset=0), "unicodeString": SimTypePointer(SimTypeChar(label="Char"), offset=0), "byteArray6": SimTypePointer(SimStruct({"byteArray6": SimTypeFixedSizeArray(SimTypeChar(label="Byte"), 6)}, name="FWP_BYTE_ARRAY6", pack=False, align=None), offset=0), "bitmapArray64": SimTypePointer(SimStruct({"bitmapArray64": SimTypeFixedSizeArray(SimTypeChar(label="Byte"), 8)}, name="FWP_BITMAP_ARRAY64_", pack=False, align=None), offset=0), "v4AddrMask": SimTypePointer(SimStruct({"addr": SimTypeInt(signed=False, label="UInt32"), "mask": SimTypeInt(signed=False, label="UInt32")}, name="FWP_V4_ADDR_AND_MASK", pack=False, align=None), offset=0), "v6AddrMask": SimTypePointer(SimStruct({"addr": SimTypeFixedSizeArray(SimTypeChar(label="Byte"), 16), "prefixLength": SimTypeChar(label="Byte")}, name="FWP_V6_ADDR_AND_MASK", pack=False, align=None), offset=0), "rangeValue": SimTypePointer(SimStruct({"valueLow": SimStruct({"type": SimTypeInt(signed=False, label="FWP_DATA_TYPE"), "Anonymous": SimUnion({"uint8": SimTypeChar(label="Byte"), "uint16": SimTypeShort(signed=False, label="UInt16"), "uint32": SimTypeInt(signed=False, label="UInt32"), "uint64": SimTypePointer(SimTypeLongLong(signed=False, label="UInt64"), offset=0), "int8": SimTypeChar(label="SByte"), "int16": SimTypeShort(signed=True, label="Int16"), "int32": SimTypeInt(signed=True, label="Int32"), "int64": SimTypePointer(SimTypeLongLong(signed=True, label="Int64"), offset=0), "float32": SimTypeFloat(size=32), "double64": SimTypePointer(SimTypeFloat(size=64), offset=0), "byteArray16": SimTypePointer(SimStruct({"byteArray16": SimTypeFixedSizeArray(SimTypeChar(label="Byte"), 16)}, name="FWP_BYTE_ARRAY16", pack=False, align=None), offset=0), "byteBlob": SimTypePointer(SimStruct({"size": SimTypeInt(signed=False, label="UInt32"), "data": SimTypePointer(SimTypeChar(label="Byte"), offset=0)}, name="FWP_BYTE_BLOB", pack=False, align=None), offset=0), "sid": SimTypePointer(SimTypeBottom(label="SID"), offset=0), "sd": SimTypePointer(SimStruct({"size": SimTypeInt(signed=False, label="UInt32"), "data": SimTypePointer(SimTypeChar(label="Byte"), offset=0)}, name="FWP_BYTE_BLOB", pack=False, align=None), offset=0), "tokenInformation": SimTypePointer(SimStruct({"sidCount": SimTypeInt(signed=False, label="UInt32"), "sids": SimTypePointer(SimTypeBottom(label="SID_AND_ATTRIBUTES"), offset=0), "restrictedSidCount": SimTypeInt(signed=False, label="UInt32"), "restrictedSids": SimTypePointer(SimTypeBottom(label="SID_AND_ATTRIBUTES"), offset=0)}, name="FWP_TOKEN_INFORMATION", pack=False, align=None), offset=0), "tokenAccessInformation": SimTypePointer(SimStruct({"size": SimTypeInt(signed=False, label="UInt32"), "data": SimTypePointer(SimTypeChar(label="Byte"), offset=0)}, name="FWP_BYTE_BLOB", pack=False, align=None), offset=0), "unicodeString": SimTypePointer(SimTypeChar(label="Char"), offset=0), "byteArray6": SimTypePointer(SimStruct({"byteArray6": SimTypeFixedSizeArray(SimTypeChar(label="Byte"), 6)}, name="FWP_BYTE_ARRAY6", pack=False, align=None), offset=0), "bitmapArray64": SimTypePointer(SimStruct({"bitmapArray64": SimTypeFixedSizeArray(SimTypeChar(label="Byte"), 8)}, name="FWP_BITMAP_ARRAY64_", pack=False, align=None), offset=0)}, name="<anon>", label="None")}, name="FWP_VALUE0", pack=False, align=None), "valueHigh": SimStruct({"type": SimTypeInt(signed=False, label="FWP_DATA_TYPE"), "Anonymous": SimUnion({"uint8": SimTypeChar(label="Byte"), "uint16": SimTypeShort(signed=False, label="UInt16"), "uint32": SimTypeInt(signed=False, label="UInt32"), "uint64": SimTypePointer(SimTypeLongLong(signed=False, label="UInt64"), offset=0), "int8": SimTypeChar(label="SByte"), "int16": SimTypeShort(signed=True, label="Int16"), "int32": SimTypeInt(signed=True, label="Int32"), "int64": SimTypePointer(SimTypeLongLong(signed=True, label="Int64"), offset=0), "float32": SimTypeFloat(size=32), "double64": SimTypePointer(SimTypeFloat(size=64), offset=0), "byteArray16": SimTypePointer(SimStruct({"byteArray16": SimTypeFixedSizeArray(SimTypeChar(label="Byte"), 16)}, name="FWP_BYTE_ARRAY16", pack=False, align=None), offset=0), "byteBlob": SimTypePointer(SimStruct({"size": SimTypeInt(signed=False, label="UInt32"), "data": SimTypePointer(SimTypeChar(label="Byte"), offset=0)}, name="FWP_BYTE_BLOB", pack=False, align=None), offset=0), "sid": SimTypePointer(SimTypeBottom(label="SID"), offset=0), "sd": SimTypePointer(SimStruct({"size": SimTypeInt(signed=False, label="UInt32"), "data": SimTypePointer(SimTypeChar(label="Byte"), offset=0)}, name="FWP_BYTE_BLOB", pack=False, align=None), offset=0), "tokenInformation": SimTypePointer(SimStruct({"sidCount": SimTypeInt(signed=False, label="UInt32"), "sids": SimTypePointer(SimTypeBottom(label="SID_AND_ATTRIBUTES"), offset=0), "restrictedSidCount": SimTypeInt(signed=False, label="UInt32"), "restrictedSids": SimTypePointer(SimTypeBottom(label="SID_AND_ATTRIBUTES"), offset=0)}, name="FWP_TOKEN_INFORMATION", pack=False, align=None), offset=0), "tokenAccessInformation": SimTypePointer(SimStruct({"size": SimTypeInt(signed=False, label="UInt32"), "data": SimTypePointer(SimTypeChar(label="Byte"), offset=0)}, name="FWP_BYTE_BLOB", pack=False, align=None), offset=0), "unicodeString": SimTypePointer(SimTypeChar(label="Char"), offset=0), "byteArray6": SimTypePointer(SimStruct({"byteArray6": SimTypeFixedSizeArray(SimTypeChar(label="Byte"), 6)}, name="FWP_BYTE_ARRAY6", pack=False, align=None), offset=0), "bitmapArray64": SimTypePointer(SimStruct({"bitmapArray64": SimTypeFixedSizeArray(SimTypeChar(label="Byte"), 8)}, name="FWP_BITMAP_ARRAY64_", pack=False, align=None), offset=0)}, name="<anon>", label="None")}, name="FWP_VALUE0", pack=False, align=None)}, name="FWP_RANGE0", pack=False, align=None), offset=0)}, name="<anon>", label="None")}, name="FWP_CONDITION_VALUE0", pack=False, align=None)}, name="FWPM_FILTER_CONDITION0", pack=False, align=None), label="LPArray", offset=0), SimTypePointer(SimTypeBottom(label="Guid"), offset=0), SimTypePointer(SimStruct({"Revision": SimTypeChar(label="Byte"), "Sbz1": SimTypeChar(label="Byte"), "Control": SimTypeShort(signed=False, label="UInt16"), "Owner": SimTypePointer(SimTypeInt(signed=True, label="Int"), label="IntPtr", offset=0), "Group": SimTypePointer(SimTypeInt(signed=True, label="Int"), label="IntPtr", offset=0), "Sacl": SimTypePointer(SimStruct({"AclRevision": SimTypeChar(label="Byte"), "Sbz1": SimTypeChar(label="Byte"), "AclSize": SimTypeShort(signed=False, label="UInt16"), "AceCount": SimTypeShort(signed=False, label="UInt16"), "Sbz2": SimTypeShort(signed=False, label="UInt16")}, name="ACL", pack=False, align=None), offset=0), "Dacl": SimTypePointer(SimStruct({"AclRevision": SimTypeChar(label="Byte"), "Sbz1": SimTypeChar(label="Byte"), "AclSize": SimTypeShort(signed=False, label="UInt16"), "AceCount": SimTypeShort(signed=False, label="UInt16"), "Sbz2": SimTypeShort(signed=False, label="UInt16")}, name="ACL", pack=False, align=None), offset=0)}, name="SECURITY_DESCRIPTOR", pack=False, align=None), offset=0)], SimTypeInt(signed=False, label="UInt32"), arg_names=["engineHandle", "flags", "mainModePolicy", "tunnelPolicy", "numFilterConditions", "filterConditions", "keyModKey", "sd"]),
        # 
        'FwpmIPsecTunnelDeleteByKey0': SimTypeFunction([SimTypePointer(SimTypeInt(signed=True, label="Int"), label="IntPtr", offset=0), SimTypePointer(SimTypeBottom(label="Guid"), offset=0)], SimTypeInt(signed=False, label="UInt32"), arg_names=["engineHandle", "key"]),
        # 
        'IPsecGetStatistics0': SimTypeFunction([SimTypePointer(SimTypeInt(signed=True, label="Int"), label="IntPtr", offset=0), SimTypePointer(SimStruct({"aggregateSaStatistics": SimStruct({"activeSas": SimTypeInt(signed=False, label="UInt32"), "pendingSaNegotiations": SimTypeInt(signed=False, label="UInt32"), "totalSasAdded": SimTypeInt(signed=False, label="UInt32"), "totalSasDeleted": SimTypeInt(signed=False, label="UInt32"), "successfulRekeys": SimTypeInt(signed=False, label="UInt32"), "activeTunnels": SimTypeInt(signed=False, label="UInt32"), "offloadedSas": SimTypeInt(signed=False, label="UInt32")}, name="IPSEC_AGGREGATE_SA_STATISTICS0", pack=False, align=None), "espDropPacketStatistics": SimStruct({"invalidSpisOnInbound": SimTypeInt(signed=False, label="UInt32"), "decryptionFailuresOnInbound": SimTypeInt(signed=False, label="UInt32"), "authenticationFailuresOnInbound": SimTypeInt(signed=False, label="UInt32"), "replayCheckFailuresOnInbound": SimTypeInt(signed=False, label="UInt32"), "saNotInitializedOnInbound": SimTypeInt(signed=False, label="UInt32")}, name="IPSEC_ESP_DROP_PACKET_STATISTICS0", pack=False, align=None), "ahDropPacketStatistics": SimStruct({"invalidSpisOnInbound": SimTypeInt(signed=False, label="UInt32"), "authenticationFailuresOnInbound": SimTypeInt(signed=False, label="UInt32"), "replayCheckFailuresOnInbound": SimTypeInt(signed=False, label="UInt32"), "saNotInitializedOnInbound": SimTypeInt(signed=False, label="UInt32")}, name="IPSEC_AH_DROP_PACKET_STATISTICS0", pack=False, align=None), "aggregateDropPacketStatistics": SimStruct({"invalidSpisOnInbound": SimTypeInt(signed=False, label="UInt32"), "decryptionFailuresOnInbound": SimTypeInt(signed=False, label="UInt32"), "authenticationFailuresOnInbound": SimTypeInt(signed=False, label="UInt32"), "udpEspValidationFailuresOnInbound": SimTypeInt(signed=False, label="UInt32"), "replayCheckFailuresOnInbound": SimTypeInt(signed=False, label="UInt32"), "invalidClearTextInbound": SimTypeInt(signed=False, label="UInt32"), "saNotInitializedOnInbound": SimTypeInt(signed=False, label="UInt32"), "receiveOverIncorrectSaInbound": SimTypeInt(signed=False, label="UInt32"), "secureReceivesNotMatchingFilters": SimTypeInt(signed=False, label="UInt32")}, name="IPSEC_AGGREGATE_DROP_PACKET_STATISTICS0", pack=False, align=None), "inboundTrafficStatistics": SimStruct({"encryptedByteCount": SimTypeLongLong(signed=False, label="UInt64"), "authenticatedAHByteCount": SimTypeLongLong(signed=False, label="UInt64"), "authenticatedESPByteCount": SimTypeLongLong(signed=False, label="UInt64"), "transportByteCount": SimTypeLongLong(signed=False, label="UInt64"), "tunnelByteCount": SimTypeLongLong(signed=False, label="UInt64"), "offloadByteCount": SimTypeLongLong(signed=False, label="UInt64")}, name="IPSEC_TRAFFIC_STATISTICS0", pack=False, align=None), "outboundTrafficStatistics": SimStruct({"encryptedByteCount": SimTypeLongLong(signed=False, label="UInt64"), "authenticatedAHByteCount": SimTypeLongLong(signed=False, label="UInt64"), "authenticatedESPByteCount": SimTypeLongLong(signed=False, label="UInt64"), "transportByteCount": SimTypeLongLong(signed=False, label="UInt64"), "tunnelByteCount": SimTypeLongLong(signed=False, label="UInt64"), "offloadByteCount": SimTypeLongLong(signed=False, label="UInt64")}, name="IPSEC_TRAFFIC_STATISTICS0", pack=False, align=None)}, name="IPSEC_STATISTICS0", pack=False, align=None), offset=0)], SimTypeInt(signed=False, label="UInt32"), arg_names=["engineHandle", "ipsecStatistics"]),
        # 
        'IPsecGetStatistics1': SimTypeFunction([SimTypePointer(SimTypeInt(signed=True, label="Int"), label="IntPtr", offset=0), SimTypePointer(SimStruct({"aggregateSaStatistics": SimStruct({"activeSas": SimTypeInt(signed=False, label="UInt32"), "pendingSaNegotiations": SimTypeInt(signed=False, label="UInt32"), "totalSasAdded": SimTypeInt(signed=False, label="UInt32"), "totalSasDeleted": SimTypeInt(signed=False, label="UInt32"), "successfulRekeys": SimTypeInt(signed=False, label="UInt32"), "activeTunnels": SimTypeInt(signed=False, label="UInt32"), "offloadedSas": SimTypeInt(signed=False, label="UInt32")}, name="IPSEC_AGGREGATE_SA_STATISTICS0", pack=False, align=None), "espDropPacketStatistics": SimStruct({"invalidSpisOnInbound": SimTypeInt(signed=False, label="UInt32"), "decryptionFailuresOnInbound": SimTypeInt(signed=False, label="UInt32"), "authenticationFailuresOnInbound": SimTypeInt(signed=False, label="UInt32"), "replayCheckFailuresOnInbound": SimTypeInt(signed=False, label="UInt32"), "saNotInitializedOnInbound": SimTypeInt(signed=False, label="UInt32")}, name="IPSEC_ESP_DROP_PACKET_STATISTICS0", pack=False, align=None), "ahDropPacketStatistics": SimStruct({"invalidSpisOnInbound": SimTypeInt(signed=False, label="UInt32"), "authenticationFailuresOnInbound": SimTypeInt(signed=False, label="UInt32"), "replayCheckFailuresOnInbound": SimTypeInt(signed=False, label="UInt32"), "saNotInitializedOnInbound": SimTypeInt(signed=False, label="UInt32")}, name="IPSEC_AH_DROP_PACKET_STATISTICS0", pack=False, align=None), "aggregateDropPacketStatistics": SimStruct({"invalidSpisOnInbound": SimTypeInt(signed=False, label="UInt32"), "decryptionFailuresOnInbound": SimTypeInt(signed=False, label="UInt32"), "authenticationFailuresOnInbound": SimTypeInt(signed=False, label="UInt32"), "udpEspValidationFailuresOnInbound": SimTypeInt(signed=False, label="UInt32"), "replayCheckFailuresOnInbound": SimTypeInt(signed=False, label="UInt32"), "invalidClearTextInbound": SimTypeInt(signed=False, label="UInt32"), "saNotInitializedOnInbound": SimTypeInt(signed=False, label="UInt32"), "receiveOverIncorrectSaInbound": SimTypeInt(signed=False, label="UInt32"), "secureReceivesNotMatchingFilters": SimTypeInt(signed=False, label="UInt32"), "totalDropPacketsInbound": SimTypeInt(signed=False, label="UInt32")}, name="IPSEC_AGGREGATE_DROP_PACKET_STATISTICS1", pack=False, align=None), "inboundTrafficStatistics": SimStruct({"encryptedByteCount": SimTypeLongLong(signed=False, label="UInt64"), "authenticatedAHByteCount": SimTypeLongLong(signed=False, label="UInt64"), "authenticatedESPByteCount": SimTypeLongLong(signed=False, label="UInt64"), "transportByteCount": SimTypeLongLong(signed=False, label="UInt64"), "tunnelByteCount": SimTypeLongLong(signed=False, label="UInt64"), "offloadByteCount": SimTypeLongLong(signed=False, label="UInt64"), "totalSuccessfulPackets": SimTypeLongLong(signed=False, label="UInt64")}, name="IPSEC_TRAFFIC_STATISTICS1", pack=False, align=None), "outboundTrafficStatistics": SimStruct({"encryptedByteCount": SimTypeLongLong(signed=False, label="UInt64"), "authenticatedAHByteCount": SimTypeLongLong(signed=False, label="UInt64"), "authenticatedESPByteCount": SimTypeLongLong(signed=False, label="UInt64"), "transportByteCount": SimTypeLongLong(signed=False, label="UInt64"), "tunnelByteCount": SimTypeLongLong(signed=False, label="UInt64"), "offloadByteCount": SimTypeLongLong(signed=False, label="UInt64"), "totalSuccessfulPackets": SimTypeLongLong(signed=False, label="UInt64")}, name="IPSEC_TRAFFIC_STATISTICS1", pack=False, align=None)}, name="IPSEC_STATISTICS1", pack=False, align=None), offset=0)], SimTypeInt(signed=False, label="UInt32"), arg_names=["engineHandle", "ipsecStatistics"]),
        # 
        'IPsecSaContextCreate0': SimTypeFunction([SimTypePointer(SimTypeInt(signed=True, label="Int"), label="IntPtr", offset=0), SimTypePointer(SimStruct({"ipVersion": SimTypeInt(signed=False, label="FWP_IP_VERSION"), "Anonymous1": SimUnion({"localV4Address": SimTypeInt(signed=False, label="UInt32"), "localV6Address": SimTypeFixedSizeArray(SimTypeChar(label="Byte"), 16)}, name="<anon>", label="None"), "Anonymous2": SimUnion({"remoteV4Address": SimTypeInt(signed=False, label="UInt32"), "remoteV6Address": SimTypeFixedSizeArray(SimTypeChar(label="Byte"), 16)}, name="<anon>", label="None"), "trafficType": SimTypeInt(signed=False, label="IPSEC_TRAFFIC_TYPE"), "Anonymous3": SimUnion({"ipsecFilterId": SimTypeLongLong(signed=False, label="UInt64"), "tunnelPolicyId": SimTypeLongLong(signed=False, label="UInt64")}, name="<anon>", label="None"), "remotePort": SimTypeShort(signed=False, label="UInt16")}, name="IPSEC_TRAFFIC0", pack=False, align=None), offset=0), SimTypePointer(SimTypeLongLong(signed=False, label="UInt64"), offset=0), SimTypePointer(SimTypeLongLong(signed=False, label="UInt64"), offset=0)], SimTypeInt(signed=False, label="UInt32"), arg_names=["engineHandle", "outboundTraffic", "inboundFilterId", "id"]),
        # 
        'IPsecSaContextCreate1': SimTypeFunction([SimTypePointer(SimTypeInt(signed=True, label="Int"), label="IntPtr", offset=0), SimTypePointer(SimStruct({"ipVersion": SimTypeInt(signed=False, label="FWP_IP_VERSION"), "Anonymous1": SimUnion({"localV4Address": SimTypeInt(signed=False, label="UInt32"), "localV6Address": SimTypeFixedSizeArray(SimTypeChar(label="Byte"), 16)}, name="<anon>", label="None"), "Anonymous2": SimUnion({"remoteV4Address": SimTypeInt(signed=False, label="UInt32"), "remoteV6Address": SimTypeFixedSizeArray(SimTypeChar(label="Byte"), 16)}, name="<anon>", label="None"), "trafficType": SimTypeInt(signed=False, label="IPSEC_TRAFFIC_TYPE"), "Anonymous3": SimUnion({"ipsecFilterId": SimTypeLongLong(signed=False, label="UInt64"), "tunnelPolicyId": SimTypeLongLong(signed=False, label="UInt64")}, name="<anon>", label="None"), "remotePort": SimTypeShort(signed=False, label="UInt16"), "localPort": SimTypeShort(signed=False, label="UInt16"), "ipProtocol": SimTypeChar(label="Byte"), "localIfLuid": SimTypeLongLong(signed=False, label="UInt64"), "realIfProfileId": SimTypeInt(signed=False, label="UInt32")}, name="IPSEC_TRAFFIC1", pack=False, align=None), offset=0), SimTypePointer(SimStruct({"virtualIfTunnelId": SimTypeLongLong(signed=False, label="UInt64"), "trafficSelectorId": SimTypeLongLong(signed=False, label="UInt64")}, name="IPSEC_VIRTUAL_IF_TUNNEL_INFO0", pack=False, align=None), offset=0), SimTypePointer(SimTypeLongLong(signed=False, label="UInt64"), offset=0), SimTypePointer(SimTypeLongLong(signed=False, label="UInt64"), offset=0)], SimTypeInt(signed=False, label="UInt32"), arg_names=["engineHandle", "outboundTraffic", "virtualIfTunnelInfo", "inboundFilterId", "id"]),
        # 
        'IPsecSaContextDeleteById0': SimTypeFunction([SimTypePointer(SimTypeInt(signed=True, label="Int"), label="IntPtr", offset=0), SimTypeLongLong(signed=False, label="UInt64")], SimTypeInt(signed=False, label="UInt32"), arg_names=["engineHandle", "id"]),
        # 
        'IPsecSaContextGetById0': SimTypeFunction([SimTypePointer(SimTypeInt(signed=True, label="Int"), label="IntPtr", offset=0), SimTypeLongLong(signed=False, label="UInt64"), SimTypePointer(SimTypePointer(SimStruct({"saContextId": SimTypeLongLong(signed=False, label="UInt64"), "inboundSa": SimTypePointer(SimStruct({"ipVersion": SimTypeInt(signed=False, label="FWP_IP_VERSION"), "saDirection": SimTypeInt(signed=False, label="FWP_DIRECTION"), "traffic": SimStruct({"ipVersion": SimTypeInt(signed=False, label="FWP_IP_VERSION"), "Anonymous1": SimUnion({"localV4Address": SimTypeInt(signed=False, label="UInt32"), "localV6Address": SimTypeFixedSizeArray(SimTypeChar(label="Byte"), 16)}, name="<anon>", label="None"), "Anonymous2": SimUnion({"remoteV4Address": SimTypeInt(signed=False, label="UInt32"), "remoteV6Address": SimTypeFixedSizeArray(SimTypeChar(label="Byte"), 16)}, name="<anon>", label="None"), "trafficType": SimTypeInt(signed=False, label="IPSEC_TRAFFIC_TYPE"), "Anonymous3": SimUnion({"ipsecFilterId": SimTypeLongLong(signed=False, label="UInt64"), "tunnelPolicyId": SimTypeLongLong(signed=False, label="UInt64")}, name="<anon>", label="None"), "remotePort": SimTypeShort(signed=False, label="UInt16")}, name="IPSEC_TRAFFIC0", pack=False, align=None), "saBundle": SimStruct({"flags": SimTypeInt(signed=False, label="IPSEC_SA_BUNDLE_FLAGS"), "lifetime": SimStruct({"lifetimeSeconds": SimTypeInt(signed=False, label="UInt32"), "lifetimeKilobytes": SimTypeInt(signed=False, label="UInt32"), "lifetimePackets": SimTypeInt(signed=False, label="UInt32")}, name="IPSEC_SA_LIFETIME0", pack=False, align=None), "idleTimeoutSeconds": SimTypeInt(signed=False, label="UInt32"), "ndAllowClearTimeoutSeconds": SimTypeInt(signed=False, label="UInt32"), "ipsecId": SimTypePointer(SimStruct({"mmTargetName": SimTypePointer(SimTypeChar(label="Char"), offset=0), "emTargetName": SimTypePointer(SimTypeChar(label="Char"), offset=0), "numTokens": SimTypeInt(signed=False, label="UInt32"), "tokens": SimTypePointer(SimStruct({"type": SimTypeInt(signed=False, label="IPSEC_TOKEN_TYPE"), "principal": SimTypeInt(signed=False, label="IPSEC_TOKEN_PRINCIPAL"), "mode": SimTypeInt(signed=False, label="IPSEC_TOKEN_MODE"), "token": SimTypeLongLong(signed=False, label="UInt64")}, name="IPSEC_TOKEN0", pack=False, align=None), offset=0), "explicitCredentials": SimTypeLongLong(signed=False, label="UInt64"), "logonId": SimTypeLongLong(signed=False, label="UInt64")}, name="IPSEC_ID0", pack=False, align=None), offset=0), "napContext": SimTypeInt(signed=False, label="UInt32"), "qmSaId": SimTypeInt(signed=False, label="UInt32"), "numSAs": SimTypeInt(signed=False, label="UInt32"), "saList": SimTypePointer(SimStruct({"spi": SimTypeInt(signed=False, label="UInt32"), "saTransformType": SimTypeInt(signed=False, label="IPSEC_TRANSFORM_TYPE"), "Anonymous": SimUnion({"ahInformation": SimTypePointer(SimStruct({"authTransform": SimStruct({"authTransformId": SimStruct({"authType": SimTypeInt(signed=False, label="IPSEC_AUTH_TYPE"), "authConfig": SimTypeChar(label="Byte")}, name="IPSEC_AUTH_TRANSFORM_ID0", pack=False, align=None), "cryptoModuleId": SimTypePointer(SimTypeBottom(label="Guid"), offset=0)}, name="IPSEC_AUTH_TRANSFORM0", pack=False, align=None), "authKey": SimStruct({"size": SimTypeInt(signed=False, label="UInt32"), "data": SimTypePointer(SimTypeChar(label="Byte"), offset=0)}, name="FWP_BYTE_BLOB", pack=False, align=None)}, name="IPSEC_SA_AUTH_INFORMATION0", pack=False, align=None), offset=0), "espAuthInformation": SimTypePointer(SimStruct({"authTransform": SimStruct({"authTransformId": SimStruct({"authType": SimTypeInt(signed=False, label="IPSEC_AUTH_TYPE"), "authConfig": SimTypeChar(label="Byte")}, name="IPSEC_AUTH_TRANSFORM_ID0", pack=False, align=None), "cryptoModuleId": SimTypePointer(SimTypeBottom(label="Guid"), offset=0)}, name="IPSEC_AUTH_TRANSFORM0", pack=False, align=None), "authKey": SimStruct({"size": SimTypeInt(signed=False, label="UInt32"), "data": SimTypePointer(SimTypeChar(label="Byte"), offset=0)}, name="FWP_BYTE_BLOB", pack=False, align=None)}, name="IPSEC_SA_AUTH_INFORMATION0", pack=False, align=None), offset=0), "espCipherInformation": SimTypePointer(SimStruct({"cipherTransform": SimStruct({"cipherTransformId": SimStruct({"cipherType": SimTypeInt(signed=False, label="IPSEC_CIPHER_TYPE"), "cipherConfig": SimTypeChar(label="Byte")}, name="IPSEC_CIPHER_TRANSFORM_ID0", pack=False, align=None), "cryptoModuleId": SimTypePointer(SimTypeBottom(label="Guid"), offset=0)}, name="IPSEC_CIPHER_TRANSFORM0", pack=False, align=None), "cipherKey": SimStruct({"size": SimTypeInt(signed=False, label="UInt32"), "data": SimTypePointer(SimTypeChar(label="Byte"), offset=0)}, name="FWP_BYTE_BLOB", pack=False, align=None)}, name="IPSEC_SA_CIPHER_INFORMATION0", pack=False, align=None), offset=0), "espAuthAndCipherInformation": SimTypePointer(SimStruct({"saCipherInformation": SimStruct({"cipherTransform": SimStruct({"cipherTransformId": SimStruct({"cipherType": SimTypeInt(signed=False, label="IPSEC_CIPHER_TYPE"), "cipherConfig": SimTypeChar(label="Byte")}, name="IPSEC_CIPHER_TRANSFORM_ID0", pack=False, align=None), "cryptoModuleId": SimTypePointer(SimTypeBottom(label="Guid"), offset=0)}, name="IPSEC_CIPHER_TRANSFORM0", pack=False, align=None), "cipherKey": SimStruct({"size": SimTypeInt(signed=False, label="UInt32"), "data": SimTypePointer(SimTypeChar(label="Byte"), offset=0)}, name="FWP_BYTE_BLOB", pack=False, align=None)}, name="IPSEC_SA_CIPHER_INFORMATION0", pack=False, align=None), "saAuthInformation": SimStruct({"authTransform": SimStruct({"authTransformId": SimStruct({"authType": SimTypeInt(signed=False, label="IPSEC_AUTH_TYPE"), "authConfig": SimTypeChar(label="Byte")}, name="IPSEC_AUTH_TRANSFORM_ID0", pack=False, align=None), "cryptoModuleId": SimTypePointer(SimTypeBottom(label="Guid"), offset=0)}, name="IPSEC_AUTH_TRANSFORM0", pack=False, align=None), "authKey": SimStruct({"size": SimTypeInt(signed=False, label="UInt32"), "data": SimTypePointer(SimTypeChar(label="Byte"), offset=0)}, name="FWP_BYTE_BLOB", pack=False, align=None)}, name="IPSEC_SA_AUTH_INFORMATION0", pack=False, align=None)}, name="IPSEC_SA_AUTH_AND_CIPHER_INFORMATION0", pack=False, align=None), offset=0), "espAuthFwInformation": SimTypePointer(SimStruct({"authTransform": SimStruct({"authTransformId": SimStruct({"authType": SimTypeInt(signed=False, label="IPSEC_AUTH_TYPE"), "authConfig": SimTypeChar(label="Byte")}, name="IPSEC_AUTH_TRANSFORM_ID0", pack=False, align=None), "cryptoModuleId": SimTypePointer(SimTypeBottom(label="Guid"), offset=0)}, name="IPSEC_AUTH_TRANSFORM0", pack=False, align=None), "authKey": SimStruct({"size": SimTypeInt(signed=False, label="UInt32"), "data": SimTypePointer(SimTypeChar(label="Byte"), offset=0)}, name="FWP_BYTE_BLOB", pack=False, align=None)}, name="IPSEC_SA_AUTH_INFORMATION0", pack=False, align=None), offset=0)}, name="<anon>", label="None")}, name="IPSEC_SA0", pack=False, align=None), offset=0), "keyModuleState": SimTypePointer(SimStruct({"keyModuleKey": SimTypeBottom(label="Guid"), "stateBlob": SimStruct({"size": SimTypeInt(signed=False, label="UInt32"), "data": SimTypePointer(SimTypeChar(label="Byte"), offset=0)}, name="FWP_BYTE_BLOB", pack=False, align=None)}, name="IPSEC_KEYMODULE_STATE0", pack=False, align=None), offset=0), "ipVersion": SimTypeInt(signed=False, label="FWP_IP_VERSION"), "Anonymous": SimUnion({"peerV4PrivateAddress": SimTypeInt(signed=False, label="UInt32")}, name="<anon>", label="None"), "mmSaId": SimTypeLongLong(signed=False, label="UInt64"), "pfsGroup": SimTypeInt(signed=False, label="IPSEC_PFS_GROUP")}, name="IPSEC_SA_BUNDLE0", pack=False, align=None), "Anonymous": SimUnion({"udpEncapsulation": SimTypePointer(SimStruct({"localUdpEncapPort": SimTypeShort(signed=False, label="UInt16"), "remoteUdpEncapPort": SimTypeShort(signed=False, label="UInt16")}, name="IPSEC_V4_UDP_ENCAPSULATION0", pack=False, align=None), offset=0)}, name="<anon>", label="None"), "transportFilter": SimTypePointer(SimTypeBottom(label="FWPM_FILTER0"), offset=0)}, name="IPSEC_SA_DETAILS0", pack=False, align=None), offset=0), "outboundSa": SimTypePointer(SimStruct({"ipVersion": SimTypeInt(signed=False, label="FWP_IP_VERSION"), "saDirection": SimTypeInt(signed=False, label="FWP_DIRECTION"), "traffic": SimStruct({"ipVersion": SimTypeInt(signed=False, label="FWP_IP_VERSION"), "Anonymous1": SimUnion({"localV4Address": SimTypeInt(signed=False, label="UInt32"), "localV6Address": SimTypeFixedSizeArray(SimTypeChar(label="Byte"), 16)}, name="<anon>", label="None"), "Anonymous2": SimUnion({"remoteV4Address": SimTypeInt(signed=False, label="UInt32"), "remoteV6Address": SimTypeFixedSizeArray(SimTypeChar(label="Byte"), 16)}, name="<anon>", label="None"), "trafficType": SimTypeInt(signed=False, label="IPSEC_TRAFFIC_TYPE"), "Anonymous3": SimUnion({"ipsecFilterId": SimTypeLongLong(signed=False, label="UInt64"), "tunnelPolicyId": SimTypeLongLong(signed=False, label="UInt64")}, name="<anon>", label="None"), "remotePort": SimTypeShort(signed=False, label="UInt16")}, name="IPSEC_TRAFFIC0", pack=False, align=None), "saBundle": SimStruct({"flags": SimTypeInt(signed=False, label="IPSEC_SA_BUNDLE_FLAGS"), "lifetime": SimStruct({"lifetimeSeconds": SimTypeInt(signed=False, label="UInt32"), "lifetimeKilobytes": SimTypeInt(signed=False, label="UInt32"), "lifetimePackets": SimTypeInt(signed=False, label="UInt32")}, name="IPSEC_SA_LIFETIME0", pack=False, align=None), "idleTimeoutSeconds": SimTypeInt(signed=False, label="UInt32"), "ndAllowClearTimeoutSeconds": SimTypeInt(signed=False, label="UInt32"), "ipsecId": SimTypePointer(SimStruct({"mmTargetName": SimTypePointer(SimTypeChar(label="Char"), offset=0), "emTargetName": SimTypePointer(SimTypeChar(label="Char"), offset=0), "numTokens": SimTypeInt(signed=False, label="UInt32"), "tokens": SimTypePointer(SimStruct({"type": SimTypeInt(signed=False, label="IPSEC_TOKEN_TYPE"), "principal": SimTypeInt(signed=False, label="IPSEC_TOKEN_PRINCIPAL"), "mode": SimTypeInt(signed=False, label="IPSEC_TOKEN_MODE"), "token": SimTypeLongLong(signed=False, label="UInt64")}, name="IPSEC_TOKEN0", pack=False, align=None), offset=0), "explicitCredentials": SimTypeLongLong(signed=False, label="UInt64"), "logonId": SimTypeLongLong(signed=False, label="UInt64")}, name="IPSEC_ID0", pack=False, align=None), offset=0), "napContext": SimTypeInt(signed=False, label="UInt32"), "qmSaId": SimTypeInt(signed=False, label="UInt32"), "numSAs": SimTypeInt(signed=False, label="UInt32"), "saList": SimTypePointer(SimStruct({"spi": SimTypeInt(signed=False, label="UInt32"), "saTransformType": SimTypeInt(signed=False, label="IPSEC_TRANSFORM_TYPE"), "Anonymous": SimUnion({"ahInformation": SimTypePointer(SimStruct({"authTransform": SimStruct({"authTransformId": SimStruct({"authType": SimTypeInt(signed=False, label="IPSEC_AUTH_TYPE"), "authConfig": SimTypeChar(label="Byte")}, name="IPSEC_AUTH_TRANSFORM_ID0", pack=False, align=None), "cryptoModuleId": SimTypePointer(SimTypeBottom(label="Guid"), offset=0)}, name="IPSEC_AUTH_TRANSFORM0", pack=False, align=None), "authKey": SimStruct({"size": SimTypeInt(signed=False, label="UInt32"), "data": SimTypePointer(SimTypeChar(label="Byte"), offset=0)}, name="FWP_BYTE_BLOB", pack=False, align=None)}, name="IPSEC_SA_AUTH_INFORMATION0", pack=False, align=None), offset=0), "espAuthInformation": SimTypePointer(SimStruct({"authTransform": SimStruct({"authTransformId": SimStruct({"authType": SimTypeInt(signed=False, label="IPSEC_AUTH_TYPE"), "authConfig": SimTypeChar(label="Byte")}, name="IPSEC_AUTH_TRANSFORM_ID0", pack=False, align=None), "cryptoModuleId": SimTypePointer(SimTypeBottom(label="Guid"), offset=0)}, name="IPSEC_AUTH_TRANSFORM0", pack=False, align=None), "authKey": SimStruct({"size": SimTypeInt(signed=False, label="UInt32"), "data": SimTypePointer(SimTypeChar(label="Byte"), offset=0)}, name="FWP_BYTE_BLOB", pack=False, align=None)}, name="IPSEC_SA_AUTH_INFORMATION0", pack=False, align=None), offset=0), "espCipherInformation": SimTypePointer(SimStruct({"cipherTransform": SimStruct({"cipherTransformId": SimStruct({"cipherType": SimTypeInt(signed=False, label="IPSEC_CIPHER_TYPE"), "cipherConfig": SimTypeChar(label="Byte")}, name="IPSEC_CIPHER_TRANSFORM_ID0", pack=False, align=None), "cryptoModuleId": SimTypePointer(SimTypeBottom(label="Guid"), offset=0)}, name="IPSEC_CIPHER_TRANSFORM0", pack=False, align=None), "cipherKey": SimStruct({"size": SimTypeInt(signed=False, label="UInt32"), "data": SimTypePointer(SimTypeChar(label="Byte"), offset=0)}, name="FWP_BYTE_BLOB", pack=False, align=None)}, name="IPSEC_SA_CIPHER_INFORMATION0", pack=False, align=None), offset=0), "espAuthAndCipherInformation": SimTypePointer(SimStruct({"saCipherInformation": SimStruct({"cipherTransform": SimStruct({"cipherTransformId": SimStruct({"cipherType": SimTypeInt(signed=False, label="IPSEC_CIPHER_TYPE"), "cipherConfig": SimTypeChar(label="Byte")}, name="IPSEC_CIPHER_TRANSFORM_ID0", pack=False, align=None), "cryptoModuleId": SimTypePointer(SimTypeBottom(label="Guid"), offset=0)}, name="IPSEC_CIPHER_TRANSFORM0", pack=False, align=None), "cipherKey": SimStruct({"size": SimTypeInt(signed=False, label="UInt32"), "data": SimTypePointer(SimTypeChar(label="Byte"), offset=0)}, name="FWP_BYTE_BLOB", pack=False, align=None)}, name="IPSEC_SA_CIPHER_INFORMATION0", pack=False, align=None), "saAuthInformation": SimStruct({"authTransform": SimStruct({"authTransformId": SimStruct({"authType": SimTypeInt(signed=False, label="IPSEC_AUTH_TYPE"), "authConfig": SimTypeChar(label="Byte")}, name="IPSEC_AUTH_TRANSFORM_ID0", pack=False, align=None), "cryptoModuleId": SimTypePointer(SimTypeBottom(label="Guid"), offset=0)}, name="IPSEC_AUTH_TRANSFORM0", pack=False, align=None), "authKey": SimStruct({"size": SimTypeInt(signed=False, label="UInt32"), "data": SimTypePointer(SimTypeChar(label="Byte"), offset=0)}, name="FWP_BYTE_BLOB", pack=False, align=None)}, name="IPSEC_SA_AUTH_INFORMATION0", pack=False, align=None)}, name="IPSEC_SA_AUTH_AND_CIPHER_INFORMATION0", pack=False, align=None), offset=0), "espAuthFwInformation": SimTypePointer(SimStruct({"authTransform": SimStruct({"authTransformId": SimStruct({"authType": SimTypeInt(signed=False, label="IPSEC_AUTH_TYPE"), "authConfig": SimTypeChar(label="Byte")}, name="IPSEC_AUTH_TRANSFORM_ID0", pack=False, align=None), "cryptoModuleId": SimTypePointer(SimTypeBottom(label="Guid"), offset=0)}, name="IPSEC_AUTH_TRANSFORM0", pack=False, align=None), "authKey": SimStruct({"size": SimTypeInt(signed=False, label="UInt32"), "data": SimTypePointer(SimTypeChar(label="Byte"), offset=0)}, name="FWP_BYTE_BLOB", pack=False, align=None)}, name="IPSEC_SA_AUTH_INFORMATION0", pack=False, align=None), offset=0)}, name="<anon>", label="None")}, name="IPSEC_SA0", pack=False, align=None), offset=0), "keyModuleState": SimTypePointer(SimStruct({"keyModuleKey": SimTypeBottom(label="Guid"), "stateBlob": SimStruct({"size": SimTypeInt(signed=False, label="UInt32"), "data": SimTypePointer(SimTypeChar(label="Byte"), offset=0)}, name="FWP_BYTE_BLOB", pack=False, align=None)}, name="IPSEC_KEYMODULE_STATE0", pack=False, align=None), offset=0), "ipVersion": SimTypeInt(signed=False, label="FWP_IP_VERSION"), "Anonymous": SimUnion({"peerV4PrivateAddress": SimTypeInt(signed=False, label="UInt32")}, name="<anon>", label="None"), "mmSaId": SimTypeLongLong(signed=False, label="UInt64"), "pfsGroup": SimTypeInt(signed=False, label="IPSEC_PFS_GROUP")}, name="IPSEC_SA_BUNDLE0", pack=False, align=None), "Anonymous": SimUnion({"udpEncapsulation": SimTypePointer(SimStruct({"localUdpEncapPort": SimTypeShort(signed=False, label="UInt16"), "remoteUdpEncapPort": SimTypeShort(signed=False, label="UInt16")}, name="IPSEC_V4_UDP_ENCAPSULATION0", pack=False, align=None), offset=0)}, name="<anon>", label="None"), "transportFilter": SimTypePointer(SimTypeBottom(label="FWPM_FILTER0"), offset=0)}, name="IPSEC_SA_DETAILS0", pack=False, align=None), offset=0)}, name="IPSEC_SA_CONTEXT0", pack=False, align=None), offset=0), offset=0)], SimTypeInt(signed=False, label="UInt32"), arg_names=["engineHandle", "id", "saContext"]),
        # 
        'IPsecSaContextGetById1': SimTypeFunction([SimTypePointer(SimTypeInt(signed=True, label="Int"), label="IntPtr", offset=0), SimTypeLongLong(signed=False, label="UInt64"), SimTypePointer(SimTypePointer(SimStruct({"saContextId": SimTypeLongLong(signed=False, label="UInt64"), "inboundSa": SimTypePointer(SimStruct({"ipVersion": SimTypeInt(signed=False, label="FWP_IP_VERSION"), "saDirection": SimTypeInt(signed=False, label="FWP_DIRECTION"), "traffic": SimStruct({"ipVersion": SimTypeInt(signed=False, label="FWP_IP_VERSION"), "Anonymous1": SimUnion({"localV4Address": SimTypeInt(signed=False, label="UInt32"), "localV6Address": SimTypeFixedSizeArray(SimTypeChar(label="Byte"), 16)}, name="<anon>", label="None"), "Anonymous2": SimUnion({"remoteV4Address": SimTypeInt(signed=False, label="UInt32"), "remoteV6Address": SimTypeFixedSizeArray(SimTypeChar(label="Byte"), 16)}, name="<anon>", label="None"), "trafficType": SimTypeInt(signed=False, label="IPSEC_TRAFFIC_TYPE"), "Anonymous3": SimUnion({"ipsecFilterId": SimTypeLongLong(signed=False, label="UInt64"), "tunnelPolicyId": SimTypeLongLong(signed=False, label="UInt64")}, name="<anon>", label="None"), "remotePort": SimTypeShort(signed=False, label="UInt16"), "localPort": SimTypeShort(signed=False, label="UInt16"), "ipProtocol": SimTypeChar(label="Byte"), "localIfLuid": SimTypeLongLong(signed=False, label="UInt64"), "realIfProfileId": SimTypeInt(signed=False, label="UInt32")}, name="IPSEC_TRAFFIC1", pack=False, align=None), "saBundle": SimStruct({"flags": SimTypeInt(signed=False, label="IPSEC_SA_BUNDLE_FLAGS"), "lifetime": SimStruct({"lifetimeSeconds": SimTypeInt(signed=False, label="UInt32"), "lifetimeKilobytes": SimTypeInt(signed=False, label="UInt32"), "lifetimePackets": SimTypeInt(signed=False, label="UInt32")}, name="IPSEC_SA_LIFETIME0", pack=False, align=None), "idleTimeoutSeconds": SimTypeInt(signed=False, label="UInt32"), "ndAllowClearTimeoutSeconds": SimTypeInt(signed=False, label="UInt32"), "ipsecId": SimTypePointer(SimStruct({"mmTargetName": SimTypePointer(SimTypeChar(label="Char"), offset=0), "emTargetName": SimTypePointer(SimTypeChar(label="Char"), offset=0), "numTokens": SimTypeInt(signed=False, label="UInt32"), "tokens": SimTypePointer(SimStruct({"type": SimTypeInt(signed=False, label="IPSEC_TOKEN_TYPE"), "principal": SimTypeInt(signed=False, label="IPSEC_TOKEN_PRINCIPAL"), "mode": SimTypeInt(signed=False, label="IPSEC_TOKEN_MODE"), "token": SimTypeLongLong(signed=False, label="UInt64")}, name="IPSEC_TOKEN0", pack=False, align=None), offset=0), "explicitCredentials": SimTypeLongLong(signed=False, label="UInt64"), "logonId": SimTypeLongLong(signed=False, label="UInt64")}, name="IPSEC_ID0", pack=False, align=None), offset=0), "napContext": SimTypeInt(signed=False, label="UInt32"), "qmSaId": SimTypeInt(signed=False, label="UInt32"), "numSAs": SimTypeInt(signed=False, label="UInt32"), "saList": SimTypePointer(SimStruct({"spi": SimTypeInt(signed=False, label="UInt32"), "saTransformType": SimTypeInt(signed=False, label="IPSEC_TRANSFORM_TYPE"), "Anonymous": SimUnion({"ahInformation": SimTypePointer(SimStruct({"authTransform": SimStruct({"authTransformId": SimStruct({"authType": SimTypeInt(signed=False, label="IPSEC_AUTH_TYPE"), "authConfig": SimTypeChar(label="Byte")}, name="IPSEC_AUTH_TRANSFORM_ID0", pack=False, align=None), "cryptoModuleId": SimTypePointer(SimTypeBottom(label="Guid"), offset=0)}, name="IPSEC_AUTH_TRANSFORM0", pack=False, align=None), "authKey": SimStruct({"size": SimTypeInt(signed=False, label="UInt32"), "data": SimTypePointer(SimTypeChar(label="Byte"), offset=0)}, name="FWP_BYTE_BLOB", pack=False, align=None)}, name="IPSEC_SA_AUTH_INFORMATION0", pack=False, align=None), offset=0), "espAuthInformation": SimTypePointer(SimStruct({"authTransform": SimStruct({"authTransformId": SimStruct({"authType": SimTypeInt(signed=False, label="IPSEC_AUTH_TYPE"), "authConfig": SimTypeChar(label="Byte")}, name="IPSEC_AUTH_TRANSFORM_ID0", pack=False, align=None), "cryptoModuleId": SimTypePointer(SimTypeBottom(label="Guid"), offset=0)}, name="IPSEC_AUTH_TRANSFORM0", pack=False, align=None), "authKey": SimStruct({"size": SimTypeInt(signed=False, label="UInt32"), "data": SimTypePointer(SimTypeChar(label="Byte"), offset=0)}, name="FWP_BYTE_BLOB", pack=False, align=None)}, name="IPSEC_SA_AUTH_INFORMATION0", pack=False, align=None), offset=0), "espCipherInformation": SimTypePointer(SimStruct({"cipherTransform": SimStruct({"cipherTransformId": SimStruct({"cipherType": SimTypeInt(signed=False, label="IPSEC_CIPHER_TYPE"), "cipherConfig": SimTypeChar(label="Byte")}, name="IPSEC_CIPHER_TRANSFORM_ID0", pack=False, align=None), "cryptoModuleId": SimTypePointer(SimTypeBottom(label="Guid"), offset=0)}, name="IPSEC_CIPHER_TRANSFORM0", pack=False, align=None), "cipherKey": SimStruct({"size": SimTypeInt(signed=False, label="UInt32"), "data": SimTypePointer(SimTypeChar(label="Byte"), offset=0)}, name="FWP_BYTE_BLOB", pack=False, align=None)}, name="IPSEC_SA_CIPHER_INFORMATION0", pack=False, align=None), offset=0), "espAuthAndCipherInformation": SimTypePointer(SimStruct({"saCipherInformation": SimStruct({"cipherTransform": SimStruct({"cipherTransformId": SimStruct({"cipherType": SimTypeInt(signed=False, label="IPSEC_CIPHER_TYPE"), "cipherConfig": SimTypeChar(label="Byte")}, name="IPSEC_CIPHER_TRANSFORM_ID0", pack=False, align=None), "cryptoModuleId": SimTypePointer(SimTypeBottom(label="Guid"), offset=0)}, name="IPSEC_CIPHER_TRANSFORM0", pack=False, align=None), "cipherKey": SimStruct({"size": SimTypeInt(signed=False, label="UInt32"), "data": SimTypePointer(SimTypeChar(label="Byte"), offset=0)}, name="FWP_BYTE_BLOB", pack=False, align=None)}, name="IPSEC_SA_CIPHER_INFORMATION0", pack=False, align=None), "saAuthInformation": SimStruct({"authTransform": SimStruct({"authTransformId": SimStruct({"authType": SimTypeInt(signed=False, label="IPSEC_AUTH_TYPE"), "authConfig": SimTypeChar(label="Byte")}, name="IPSEC_AUTH_TRANSFORM_ID0", pack=False, align=None), "cryptoModuleId": SimTypePointer(SimTypeBottom(label="Guid"), offset=0)}, name="IPSEC_AUTH_TRANSFORM0", pack=False, align=None), "authKey": SimStruct({"size": SimTypeInt(signed=False, label="UInt32"), "data": SimTypePointer(SimTypeChar(label="Byte"), offset=0)}, name="FWP_BYTE_BLOB", pack=False, align=None)}, name="IPSEC_SA_AUTH_INFORMATION0", pack=False, align=None)}, name="IPSEC_SA_AUTH_AND_CIPHER_INFORMATION0", pack=False, align=None), offset=0), "espAuthFwInformation": SimTypePointer(SimStruct({"authTransform": SimStruct({"authTransformId": SimStruct({"authType": SimTypeInt(signed=False, label="IPSEC_AUTH_TYPE"), "authConfig": SimTypeChar(label="Byte")}, name="IPSEC_AUTH_TRANSFORM_ID0", pack=False, align=None), "cryptoModuleId": SimTypePointer(SimTypeBottom(label="Guid"), offset=0)}, name="IPSEC_AUTH_TRANSFORM0", pack=False, align=None), "authKey": SimStruct({"size": SimTypeInt(signed=False, label="UInt32"), "data": SimTypePointer(SimTypeChar(label="Byte"), offset=0)}, name="FWP_BYTE_BLOB", pack=False, align=None)}, name="IPSEC_SA_AUTH_INFORMATION0", pack=False, align=None), offset=0)}, name="<anon>", label="None")}, name="IPSEC_SA0", pack=False, align=None), offset=0), "keyModuleState": SimTypePointer(SimStruct({"keyModuleKey": SimTypeBottom(label="Guid"), "stateBlob": SimStruct({"size": SimTypeInt(signed=False, label="UInt32"), "data": SimTypePointer(SimTypeChar(label="Byte"), offset=0)}, name="FWP_BYTE_BLOB", pack=False, align=None)}, name="IPSEC_KEYMODULE_STATE0", pack=False, align=None), offset=0), "ipVersion": SimTypeInt(signed=False, label="FWP_IP_VERSION"), "Anonymous": SimUnion({"peerV4PrivateAddress": SimTypeInt(signed=False, label="UInt32")}, name="<anon>", label="None"), "mmSaId": SimTypeLongLong(signed=False, label="UInt64"), "pfsGroup": SimTypeInt(signed=False, label="IPSEC_PFS_GROUP"), "saLookupContext": SimTypeBottom(label="Guid"), "qmFilterId": SimTypeLongLong(signed=False, label="UInt64")}, name="IPSEC_SA_BUNDLE1", pack=False, align=None), "Anonymous": SimUnion({"udpEncapsulation": SimTypePointer(SimStruct({"localUdpEncapPort": SimTypeShort(signed=False, label="UInt16"), "remoteUdpEncapPort": SimTypeShort(signed=False, label="UInt16")}, name="IPSEC_V4_UDP_ENCAPSULATION0", pack=False, align=None), offset=0)}, name="<anon>", label="None"), "transportFilter": SimTypePointer(SimTypeBottom(label="FWPM_FILTER0"), offset=0), "virtualIfTunnelInfo": SimStruct({"virtualIfTunnelId": SimTypeLongLong(signed=False, label="UInt64"), "trafficSelectorId": SimTypeLongLong(signed=False, label="UInt64")}, name="IPSEC_VIRTUAL_IF_TUNNEL_INFO0", pack=False, align=None)}, name="IPSEC_SA_DETAILS1", pack=False, align=None), offset=0), "outboundSa": SimTypePointer(SimStruct({"ipVersion": SimTypeInt(signed=False, label="FWP_IP_VERSION"), "saDirection": SimTypeInt(signed=False, label="FWP_DIRECTION"), "traffic": SimStruct({"ipVersion": SimTypeInt(signed=False, label="FWP_IP_VERSION"), "Anonymous1": SimUnion({"localV4Address": SimTypeInt(signed=False, label="UInt32"), "localV6Address": SimTypeFixedSizeArray(SimTypeChar(label="Byte"), 16)}, name="<anon>", label="None"), "Anonymous2": SimUnion({"remoteV4Address": SimTypeInt(signed=False, label="UInt32"), "remoteV6Address": SimTypeFixedSizeArray(SimTypeChar(label="Byte"), 16)}, name="<anon>", label="None"), "trafficType": SimTypeInt(signed=False, label="IPSEC_TRAFFIC_TYPE"), "Anonymous3": SimUnion({"ipsecFilterId": SimTypeLongLong(signed=False, label="UInt64"), "tunnelPolicyId": SimTypeLongLong(signed=False, label="UInt64")}, name="<anon>", label="None"), "remotePort": SimTypeShort(signed=False, label="UInt16"), "localPort": SimTypeShort(signed=False, label="UInt16"), "ipProtocol": SimTypeChar(label="Byte"), "localIfLuid": SimTypeLongLong(signed=False, label="UInt64"), "realIfProfileId": SimTypeInt(signed=False, label="UInt32")}, name="IPSEC_TRAFFIC1", pack=False, align=None), "saBundle": SimStruct({"flags": SimTypeInt(signed=False, label="IPSEC_SA_BUNDLE_FLAGS"), "lifetime": SimStruct({"lifetimeSeconds": SimTypeInt(signed=False, label="UInt32"), "lifetimeKilobytes": SimTypeInt(signed=False, label="UInt32"), "lifetimePackets": SimTypeInt(signed=False, label="UInt32")}, name="IPSEC_SA_LIFETIME0", pack=False, align=None), "idleTimeoutSeconds": SimTypeInt(signed=False, label="UInt32"), "ndAllowClearTimeoutSeconds": SimTypeInt(signed=False, label="UInt32"), "ipsecId": SimTypePointer(SimStruct({"mmTargetName": SimTypePointer(SimTypeChar(label="Char"), offset=0), "emTargetName": SimTypePointer(SimTypeChar(label="Char"), offset=0), "numTokens": SimTypeInt(signed=False, label="UInt32"), "tokens": SimTypePointer(SimStruct({"type": SimTypeInt(signed=False, label="IPSEC_TOKEN_TYPE"), "principal": SimTypeInt(signed=False, label="IPSEC_TOKEN_PRINCIPAL"), "mode": SimTypeInt(signed=False, label="IPSEC_TOKEN_MODE"), "token": SimTypeLongLong(signed=False, label="UInt64")}, name="IPSEC_TOKEN0", pack=False, align=None), offset=0), "explicitCredentials": SimTypeLongLong(signed=False, label="UInt64"), "logonId": SimTypeLongLong(signed=False, label="UInt64")}, name="IPSEC_ID0", pack=False, align=None), offset=0), "napContext": SimTypeInt(signed=False, label="UInt32"), "qmSaId": SimTypeInt(signed=False, label="UInt32"), "numSAs": SimTypeInt(signed=False, label="UInt32"), "saList": SimTypePointer(SimStruct({"spi": SimTypeInt(signed=False, label="UInt32"), "saTransformType": SimTypeInt(signed=False, label="IPSEC_TRANSFORM_TYPE"), "Anonymous": SimUnion({"ahInformation": SimTypePointer(SimStruct({"authTransform": SimStruct({"authTransformId": SimStruct({"authType": SimTypeInt(signed=False, label="IPSEC_AUTH_TYPE"), "authConfig": SimTypeChar(label="Byte")}, name="IPSEC_AUTH_TRANSFORM_ID0", pack=False, align=None), "cryptoModuleId": SimTypePointer(SimTypeBottom(label="Guid"), offset=0)}, name="IPSEC_AUTH_TRANSFORM0", pack=False, align=None), "authKey": SimStruct({"size": SimTypeInt(signed=False, label="UInt32"), "data": SimTypePointer(SimTypeChar(label="Byte"), offset=0)}, name="FWP_BYTE_BLOB", pack=False, align=None)}, name="IPSEC_SA_AUTH_INFORMATION0", pack=False, align=None), offset=0), "espAuthInformation": SimTypePointer(SimStruct({"authTransform": SimStruct({"authTransformId": SimStruct({"authType": SimTypeInt(signed=False, label="IPSEC_AUTH_TYPE"), "authConfig": SimTypeChar(label="Byte")}, name="IPSEC_AUTH_TRANSFORM_ID0", pack=False, align=None), "cryptoModuleId": SimTypePointer(SimTypeBottom(label="Guid"), offset=0)}, name="IPSEC_AUTH_TRANSFORM0", pack=False, align=None), "authKey": SimStruct({"size": SimTypeInt(signed=False, label="UInt32"), "data": SimTypePointer(SimTypeChar(label="Byte"), offset=0)}, name="FWP_BYTE_BLOB", pack=False, align=None)}, name="IPSEC_SA_AUTH_INFORMATION0", pack=False, align=None), offset=0), "espCipherInformation": SimTypePointer(SimStruct({"cipherTransform": SimStruct({"cipherTransformId": SimStruct({"cipherType": SimTypeInt(signed=False, label="IPSEC_CIPHER_TYPE"), "cipherConfig": SimTypeChar(label="Byte")}, name="IPSEC_CIPHER_TRANSFORM_ID0", pack=False, align=None), "cryptoModuleId": SimTypePointer(SimTypeBottom(label="Guid"), offset=0)}, name="IPSEC_CIPHER_TRANSFORM0", pack=False, align=None), "cipherKey": SimStruct({"size": SimTypeInt(signed=False, label="UInt32"), "data": SimTypePointer(SimTypeChar(label="Byte"), offset=0)}, name="FWP_BYTE_BLOB", pack=False, align=None)}, name="IPSEC_SA_CIPHER_INFORMATION0", pack=False, align=None), offset=0), "espAuthAndCipherInformation": SimTypePointer(SimStruct({"saCipherInformation": SimStruct({"cipherTransform": SimStruct({"cipherTransformId": SimStruct({"cipherType": SimTypeInt(signed=False, label="IPSEC_CIPHER_TYPE"), "cipherConfig": SimTypeChar(label="Byte")}, name="IPSEC_CIPHER_TRANSFORM_ID0", pack=False, align=None), "cryptoModuleId": SimTypePointer(SimTypeBottom(label="Guid"), offset=0)}, name="IPSEC_CIPHER_TRANSFORM0", pack=False, align=None), "cipherKey": SimStruct({"size": SimTypeInt(signed=False, label="UInt32"), "data": SimTypePointer(SimTypeChar(label="Byte"), offset=0)}, name="FWP_BYTE_BLOB", pack=False, align=None)}, name="IPSEC_SA_CIPHER_INFORMATION0", pack=False, align=None), "saAuthInformation": SimStruct({"authTransform": SimStruct({"authTransformId": SimStruct({"authType": SimTypeInt(signed=False, label="IPSEC_AUTH_TYPE"), "authConfig": SimTypeChar(label="Byte")}, name="IPSEC_AUTH_TRANSFORM_ID0", pack=False, align=None), "cryptoModuleId": SimTypePointer(SimTypeBottom(label="Guid"), offset=0)}, name="IPSEC_AUTH_TRANSFORM0", pack=False, align=None), "authKey": SimStruct({"size": SimTypeInt(signed=False, label="UInt32"), "data": SimTypePointer(SimTypeChar(label="Byte"), offset=0)}, name="FWP_BYTE_BLOB", pack=False, align=None)}, name="IPSEC_SA_AUTH_INFORMATION0", pack=False, align=None)}, name="IPSEC_SA_AUTH_AND_CIPHER_INFORMATION0", pack=False, align=None), offset=0), "espAuthFwInformation": SimTypePointer(SimStruct({"authTransform": SimStruct({"authTransformId": SimStruct({"authType": SimTypeInt(signed=False, label="IPSEC_AUTH_TYPE"), "authConfig": SimTypeChar(label="Byte")}, name="IPSEC_AUTH_TRANSFORM_ID0", pack=False, align=None), "cryptoModuleId": SimTypePointer(SimTypeBottom(label="Guid"), offset=0)}, name="IPSEC_AUTH_TRANSFORM0", pack=False, align=None), "authKey": SimStruct({"size": SimTypeInt(signed=False, label="UInt32"), "data": SimTypePointer(SimTypeChar(label="Byte"), offset=0)}, name="FWP_BYTE_BLOB", pack=False, align=None)}, name="IPSEC_SA_AUTH_INFORMATION0", pack=False, align=None), offset=0)}, name="<anon>", label="None")}, name="IPSEC_SA0", pack=False, align=None), offset=0), "keyModuleState": SimTypePointer(SimStruct({"keyModuleKey": SimTypeBottom(label="Guid"), "stateBlob": SimStruct({"size": SimTypeInt(signed=False, label="UInt32"), "data": SimTypePointer(SimTypeChar(label="Byte"), offset=0)}, name="FWP_BYTE_BLOB", pack=False, align=None)}, name="IPSEC_KEYMODULE_STATE0", pack=False, align=None), offset=0), "ipVersion": SimTypeInt(signed=False, label="FWP_IP_VERSION"), "Anonymous": SimUnion({"peerV4PrivateAddress": SimTypeInt(signed=False, label="UInt32")}, name="<anon>", label="None"), "mmSaId": SimTypeLongLong(signed=False, label="UInt64"), "pfsGroup": SimTypeInt(signed=False, label="IPSEC_PFS_GROUP"), "saLookupContext": SimTypeBottom(label="Guid"), "qmFilterId": SimTypeLongLong(signed=False, label="UInt64")}, name="IPSEC_SA_BUNDLE1", pack=False, align=None), "Anonymous": SimUnion({"udpEncapsulation": SimTypePointer(SimStruct({"localUdpEncapPort": SimTypeShort(signed=False, label="UInt16"), "remoteUdpEncapPort": SimTypeShort(signed=False, label="UInt16")}, name="IPSEC_V4_UDP_ENCAPSULATION0", pack=False, align=None), offset=0)}, name="<anon>", label="None"), "transportFilter": SimTypePointer(SimTypeBottom(label="FWPM_FILTER0"), offset=0), "virtualIfTunnelInfo": SimStruct({"virtualIfTunnelId": SimTypeLongLong(signed=False, label="UInt64"), "trafficSelectorId": SimTypeLongLong(signed=False, label="UInt64")}, name="IPSEC_VIRTUAL_IF_TUNNEL_INFO0", pack=False, align=None)}, name="IPSEC_SA_DETAILS1", pack=False, align=None), offset=0)}, name="IPSEC_SA_CONTEXT1", pack=False, align=None), offset=0), offset=0)], SimTypeInt(signed=False, label="UInt32"), arg_names=["engineHandle", "id", "saContext"]),
        # 
        'IPsecSaContextGetSpi0': SimTypeFunction([SimTypePointer(SimTypeInt(signed=True, label="Int"), label="IntPtr", offset=0), SimTypeLongLong(signed=False, label="UInt64"), SimTypePointer(SimStruct({"inboundIpsecTraffic": SimStruct({"ipVersion": SimTypeInt(signed=False, label="FWP_IP_VERSION"), "Anonymous1": SimUnion({"localV4Address": SimTypeInt(signed=False, label="UInt32"), "localV6Address": SimTypeFixedSizeArray(SimTypeChar(label="Byte"), 16)}, name="<anon>", label="None"), "Anonymous2": SimUnion({"remoteV4Address": SimTypeInt(signed=False, label="UInt32"), "remoteV6Address": SimTypeFixedSizeArray(SimTypeChar(label="Byte"), 16)}, name="<anon>", label="None"), "trafficType": SimTypeInt(signed=False, label="IPSEC_TRAFFIC_TYPE"), "Anonymous3": SimUnion({"ipsecFilterId": SimTypeLongLong(signed=False, label="UInt64"), "tunnelPolicyId": SimTypeLongLong(signed=False, label="UInt64")}, name="<anon>", label="None"), "remotePort": SimTypeShort(signed=False, label="UInt16")}, name="IPSEC_TRAFFIC0", pack=False, align=None), "ipVersion": SimTypeInt(signed=False, label="FWP_IP_VERSION"), "Anonymous": SimUnion({"inboundUdpEncapsulation": SimTypePointer(SimStruct({"localUdpEncapPort": SimTypeShort(signed=False, label="UInt16"), "remoteUdpEncapPort": SimTypeShort(signed=False, label="UInt16")}, name="IPSEC_V4_UDP_ENCAPSULATION0", pack=False, align=None), offset=0)}, name="<anon>", label="None"), "rngCryptoModuleID": SimTypePointer(SimTypeBottom(label="Guid"), offset=0)}, name="IPSEC_GETSPI0", pack=False, align=None), offset=0), SimTypePointer(SimTypeInt(signed=False, label="UInt32"), offset=0)], SimTypeInt(signed=False, label="UInt32"), arg_names=["engineHandle", "id", "getSpi", "inboundSpi"]),
        # 
        'IPsecSaContextGetSpi1': SimTypeFunction([SimTypePointer(SimTypeInt(signed=True, label="Int"), label="IntPtr", offset=0), SimTypeLongLong(signed=False, label="UInt64"), SimTypePointer(SimStruct({"inboundIpsecTraffic": SimStruct({"ipVersion": SimTypeInt(signed=False, label="FWP_IP_VERSION"), "Anonymous1": SimUnion({"localV4Address": SimTypeInt(signed=False, label="UInt32"), "localV6Address": SimTypeFixedSizeArray(SimTypeChar(label="Byte"), 16)}, name="<anon>", label="None"), "Anonymous2": SimUnion({"remoteV4Address": SimTypeInt(signed=False, label="UInt32"), "remoteV6Address": SimTypeFixedSizeArray(SimTypeChar(label="Byte"), 16)}, name="<anon>", label="None"), "trafficType": SimTypeInt(signed=False, label="IPSEC_TRAFFIC_TYPE"), "Anonymous3": SimUnion({"ipsecFilterId": SimTypeLongLong(signed=False, label="UInt64"), "tunnelPolicyId": SimTypeLongLong(signed=False, label="UInt64")}, name="<anon>", label="None"), "remotePort": SimTypeShort(signed=False, label="UInt16"), "localPort": SimTypeShort(signed=False, label="UInt16"), "ipProtocol": SimTypeChar(label="Byte"), "localIfLuid": SimTypeLongLong(signed=False, label="UInt64"), "realIfProfileId": SimTypeInt(signed=False, label="UInt32")}, name="IPSEC_TRAFFIC1", pack=False, align=None), "ipVersion": SimTypeInt(signed=False, label="FWP_IP_VERSION"), "Anonymous": SimUnion({"inboundUdpEncapsulation": SimTypePointer(SimStruct({"localUdpEncapPort": SimTypeShort(signed=False, label="UInt16"), "remoteUdpEncapPort": SimTypeShort(signed=False, label="UInt16")}, name="IPSEC_V4_UDP_ENCAPSULATION0", pack=False, align=None), offset=0)}, name="<anon>", label="None"), "rngCryptoModuleID": SimTypePointer(SimTypeBottom(label="Guid"), offset=0)}, name="IPSEC_GETSPI1", pack=False, align=None), offset=0), SimTypePointer(SimTypeInt(signed=False, label="UInt32"), offset=0)], SimTypeInt(signed=False, label="UInt32"), arg_names=["engineHandle", "id", "getSpi", "inboundSpi"]),
        # 
        'IPsecSaContextSetSpi0': SimTypeFunction([SimTypePointer(SimTypeInt(signed=True, label="Int"), label="IntPtr", offset=0), SimTypeLongLong(signed=False, label="UInt64"), SimTypePointer(SimStruct({"inboundIpsecTraffic": SimStruct({"ipVersion": SimTypeInt(signed=False, label="FWP_IP_VERSION"), "Anonymous1": SimUnion({"localV4Address": SimTypeInt(signed=False, label="UInt32"), "localV6Address": SimTypeFixedSizeArray(SimTypeChar(label="Byte"), 16)}, name="<anon>", label="None"), "Anonymous2": SimUnion({"remoteV4Address": SimTypeInt(signed=False, label="UInt32"), "remoteV6Address": SimTypeFixedSizeArray(SimTypeChar(label="Byte"), 16)}, name="<anon>", label="None"), "trafficType": SimTypeInt(signed=False, label="IPSEC_TRAFFIC_TYPE"), "Anonymous3": SimUnion({"ipsecFilterId": SimTypeLongLong(signed=False, label="UInt64"), "tunnelPolicyId": SimTypeLongLong(signed=False, label="UInt64")}, name="<anon>", label="None"), "remotePort": SimTypeShort(signed=False, label="UInt16"), "localPort": SimTypeShort(signed=False, label="UInt16"), "ipProtocol": SimTypeChar(label="Byte"), "localIfLuid": SimTypeLongLong(signed=False, label="UInt64"), "realIfProfileId": SimTypeInt(signed=False, label="UInt32")}, name="IPSEC_TRAFFIC1", pack=False, align=None), "ipVersion": SimTypeInt(signed=False, label="FWP_IP_VERSION"), "Anonymous": SimUnion({"inboundUdpEncapsulation": SimTypePointer(SimStruct({"localUdpEncapPort": SimTypeShort(signed=False, label="UInt16"), "remoteUdpEncapPort": SimTypeShort(signed=False, label="UInt16")}, name="IPSEC_V4_UDP_ENCAPSULATION0", pack=False, align=None), offset=0)}, name="<anon>", label="None"), "rngCryptoModuleID": SimTypePointer(SimTypeBottom(label="Guid"), offset=0)}, name="IPSEC_GETSPI1", pack=False, align=None), offset=0), SimTypeInt(signed=False, label="UInt32")], SimTypeInt(signed=False, label="UInt32"), arg_names=["engineHandle", "id", "getSpi", "inboundSpi"]),
        # 
        'IPsecSaContextAddInbound0': SimTypeFunction([SimTypePointer(SimTypeInt(signed=True, label="Int"), label="IntPtr", offset=0), SimTypeLongLong(signed=False, label="UInt64"), SimTypePointer(SimStruct({"flags": SimTypeInt(signed=False, label="IPSEC_SA_BUNDLE_FLAGS"), "lifetime": SimStruct({"lifetimeSeconds": SimTypeInt(signed=False, label="UInt32"), "lifetimeKilobytes": SimTypeInt(signed=False, label="UInt32"), "lifetimePackets": SimTypeInt(signed=False, label="UInt32")}, name="IPSEC_SA_LIFETIME0", pack=False, align=None), "idleTimeoutSeconds": SimTypeInt(signed=False, label="UInt32"), "ndAllowClearTimeoutSeconds": SimTypeInt(signed=False, label="UInt32"), "ipsecId": SimTypePointer(SimStruct({"mmTargetName": SimTypePointer(SimTypeChar(label="Char"), offset=0), "emTargetName": SimTypePointer(SimTypeChar(label="Char"), offset=0), "numTokens": SimTypeInt(signed=False, label="UInt32"), "tokens": SimTypePointer(SimStruct({"type": SimTypeInt(signed=False, label="IPSEC_TOKEN_TYPE"), "principal": SimTypeInt(signed=False, label="IPSEC_TOKEN_PRINCIPAL"), "mode": SimTypeInt(signed=False, label="IPSEC_TOKEN_MODE"), "token": SimTypeLongLong(signed=False, label="UInt64")}, name="IPSEC_TOKEN0", pack=False, align=None), offset=0), "explicitCredentials": SimTypeLongLong(signed=False, label="UInt64"), "logonId": SimTypeLongLong(signed=False, label="UInt64")}, name="IPSEC_ID0", pack=False, align=None), offset=0), "napContext": SimTypeInt(signed=False, label="UInt32"), "qmSaId": SimTypeInt(signed=False, label="UInt32"), "numSAs": SimTypeInt(signed=False, label="UInt32"), "saList": SimTypePointer(SimStruct({"spi": SimTypeInt(signed=False, label="UInt32"), "saTransformType": SimTypeInt(signed=False, label="IPSEC_TRANSFORM_TYPE"), "Anonymous": SimUnion({"ahInformation": SimTypePointer(SimStruct({"authTransform": SimStruct({"authTransformId": SimStruct({"authType": SimTypeInt(signed=False, label="IPSEC_AUTH_TYPE"), "authConfig": SimTypeChar(label="Byte")}, name="IPSEC_AUTH_TRANSFORM_ID0", pack=False, align=None), "cryptoModuleId": SimTypePointer(SimTypeBottom(label="Guid"), offset=0)}, name="IPSEC_AUTH_TRANSFORM0", pack=False, align=None), "authKey": SimStruct({"size": SimTypeInt(signed=False, label="UInt32"), "data": SimTypePointer(SimTypeChar(label="Byte"), offset=0)}, name="FWP_BYTE_BLOB", pack=False, align=None)}, name="IPSEC_SA_AUTH_INFORMATION0", pack=False, align=None), offset=0), "espAuthInformation": SimTypePointer(SimStruct({"authTransform": SimStruct({"authTransformId": SimStruct({"authType": SimTypeInt(signed=False, label="IPSEC_AUTH_TYPE"), "authConfig": SimTypeChar(label="Byte")}, name="IPSEC_AUTH_TRANSFORM_ID0", pack=False, align=None), "cryptoModuleId": SimTypePointer(SimTypeBottom(label="Guid"), offset=0)}, name="IPSEC_AUTH_TRANSFORM0", pack=False, align=None), "authKey": SimStruct({"size": SimTypeInt(signed=False, label="UInt32"), "data": SimTypePointer(SimTypeChar(label="Byte"), offset=0)}, name="FWP_BYTE_BLOB", pack=False, align=None)}, name="IPSEC_SA_AUTH_INFORMATION0", pack=False, align=None), offset=0), "espCipherInformation": SimTypePointer(SimStruct({"cipherTransform": SimStruct({"cipherTransformId": SimStruct({"cipherType": SimTypeInt(signed=False, label="IPSEC_CIPHER_TYPE"), "cipherConfig": SimTypeChar(label="Byte")}, name="IPSEC_CIPHER_TRANSFORM_ID0", pack=False, align=None), "cryptoModuleId": SimTypePointer(SimTypeBottom(label="Guid"), offset=0)}, name="IPSEC_CIPHER_TRANSFORM0", pack=False, align=None), "cipherKey": SimStruct({"size": SimTypeInt(signed=False, label="UInt32"), "data": SimTypePointer(SimTypeChar(label="Byte"), offset=0)}, name="FWP_BYTE_BLOB", pack=False, align=None)}, name="IPSEC_SA_CIPHER_INFORMATION0", pack=False, align=None), offset=0), "espAuthAndCipherInformation": SimTypePointer(SimStruct({"saCipherInformation": SimStruct({"cipherTransform": SimStruct({"cipherTransformId": SimStruct({"cipherType": SimTypeInt(signed=False, label="IPSEC_CIPHER_TYPE"), "cipherConfig": SimTypeChar(label="Byte")}, name="IPSEC_CIPHER_TRANSFORM_ID0", pack=False, align=None), "cryptoModuleId": SimTypePointer(SimTypeBottom(label="Guid"), offset=0)}, name="IPSEC_CIPHER_TRANSFORM0", pack=False, align=None), "cipherKey": SimStruct({"size": SimTypeInt(signed=False, label="UInt32"), "data": SimTypePointer(SimTypeChar(label="Byte"), offset=0)}, name="FWP_BYTE_BLOB", pack=False, align=None)}, name="IPSEC_SA_CIPHER_INFORMATION0", pack=False, align=None), "saAuthInformation": SimStruct({"authTransform": SimStruct({"authTransformId": SimStruct({"authType": SimTypeInt(signed=False, label="IPSEC_AUTH_TYPE"), "authConfig": SimTypeChar(label="Byte")}, name="IPSEC_AUTH_TRANSFORM_ID0", pack=False, align=None), "cryptoModuleId": SimTypePointer(SimTypeBottom(label="Guid"), offset=0)}, name="IPSEC_AUTH_TRANSFORM0", pack=False, align=None), "authKey": SimStruct({"size": SimTypeInt(signed=False, label="UInt32"), "data": SimTypePointer(SimTypeChar(label="Byte"), offset=0)}, name="FWP_BYTE_BLOB", pack=False, align=None)}, name="IPSEC_SA_AUTH_INFORMATION0", pack=False, align=None)}, name="IPSEC_SA_AUTH_AND_CIPHER_INFORMATION0", pack=False, align=None), offset=0), "espAuthFwInformation": SimTypePointer(SimStruct({"authTransform": SimStruct({"authTransformId": SimStruct({"authType": SimTypeInt(signed=False, label="IPSEC_AUTH_TYPE"), "authConfig": SimTypeChar(label="Byte")}, name="IPSEC_AUTH_TRANSFORM_ID0", pack=False, align=None), "cryptoModuleId": SimTypePointer(SimTypeBottom(label="Guid"), offset=0)}, name="IPSEC_AUTH_TRANSFORM0", pack=False, align=None), "authKey": SimStruct({"size": SimTypeInt(signed=False, label="UInt32"), "data": SimTypePointer(SimTypeChar(label="Byte"), offset=0)}, name="FWP_BYTE_BLOB", pack=False, align=None)}, name="IPSEC_SA_AUTH_INFORMATION0", pack=False, align=None), offset=0)}, name="<anon>", label="None")}, name="IPSEC_SA0", pack=False, align=None), offset=0), "keyModuleState": SimTypePointer(SimStruct({"keyModuleKey": SimTypeBottom(label="Guid"), "stateBlob": SimStruct({"size": SimTypeInt(signed=False, label="UInt32"), "data": SimTypePointer(SimTypeChar(label="Byte"), offset=0)}, name="FWP_BYTE_BLOB", pack=False, align=None)}, name="IPSEC_KEYMODULE_STATE0", pack=False, align=None), offset=0), "ipVersion": SimTypeInt(signed=False, label="FWP_IP_VERSION"), "Anonymous": SimUnion({"peerV4PrivateAddress": SimTypeInt(signed=False, label="UInt32")}, name="<anon>", label="None"), "mmSaId": SimTypeLongLong(signed=False, label="UInt64"), "pfsGroup": SimTypeInt(signed=False, label="IPSEC_PFS_GROUP")}, name="IPSEC_SA_BUNDLE0", pack=False, align=None), offset=0)], SimTypeInt(signed=False, label="UInt32"), arg_names=["engineHandle", "id", "inboundBundle"]),
        # 
        'IPsecSaContextAddOutbound0': SimTypeFunction([SimTypePointer(SimTypeInt(signed=True, label="Int"), label="IntPtr", offset=0), SimTypeLongLong(signed=False, label="UInt64"), SimTypePointer(SimStruct({"flags": SimTypeInt(signed=False, label="IPSEC_SA_BUNDLE_FLAGS"), "lifetime": SimStruct({"lifetimeSeconds": SimTypeInt(signed=False, label="UInt32"), "lifetimeKilobytes": SimTypeInt(signed=False, label="UInt32"), "lifetimePackets": SimTypeInt(signed=False, label="UInt32")}, name="IPSEC_SA_LIFETIME0", pack=False, align=None), "idleTimeoutSeconds": SimTypeInt(signed=False, label="UInt32"), "ndAllowClearTimeoutSeconds": SimTypeInt(signed=False, label="UInt32"), "ipsecId": SimTypePointer(SimStruct({"mmTargetName": SimTypePointer(SimTypeChar(label="Char"), offset=0), "emTargetName": SimTypePointer(SimTypeChar(label="Char"), offset=0), "numTokens": SimTypeInt(signed=False, label="UInt32"), "tokens": SimTypePointer(SimStruct({"type": SimTypeInt(signed=False, label="IPSEC_TOKEN_TYPE"), "principal": SimTypeInt(signed=False, label="IPSEC_TOKEN_PRINCIPAL"), "mode": SimTypeInt(signed=False, label="IPSEC_TOKEN_MODE"), "token": SimTypeLongLong(signed=False, label="UInt64")}, name="IPSEC_TOKEN0", pack=False, align=None), offset=0), "explicitCredentials": SimTypeLongLong(signed=False, label="UInt64"), "logonId": SimTypeLongLong(signed=False, label="UInt64")}, name="IPSEC_ID0", pack=False, align=None), offset=0), "napContext": SimTypeInt(signed=False, label="UInt32"), "qmSaId": SimTypeInt(signed=False, label="UInt32"), "numSAs": SimTypeInt(signed=False, label="UInt32"), "saList": SimTypePointer(SimStruct({"spi": SimTypeInt(signed=False, label="UInt32"), "saTransformType": SimTypeInt(signed=False, label="IPSEC_TRANSFORM_TYPE"), "Anonymous": SimUnion({"ahInformation": SimTypePointer(SimStruct({"authTransform": SimStruct({"authTransformId": SimStruct({"authType": SimTypeInt(signed=False, label="IPSEC_AUTH_TYPE"), "authConfig": SimTypeChar(label="Byte")}, name="IPSEC_AUTH_TRANSFORM_ID0", pack=False, align=None), "cryptoModuleId": SimTypePointer(SimTypeBottom(label="Guid"), offset=0)}, name="IPSEC_AUTH_TRANSFORM0", pack=False, align=None), "authKey": SimStruct({"size": SimTypeInt(signed=False, label="UInt32"), "data": SimTypePointer(SimTypeChar(label="Byte"), offset=0)}, name="FWP_BYTE_BLOB", pack=False, align=None)}, name="IPSEC_SA_AUTH_INFORMATION0", pack=False, align=None), offset=0), "espAuthInformation": SimTypePointer(SimStruct({"authTransform": SimStruct({"authTransformId": SimStruct({"authType": SimTypeInt(signed=False, label="IPSEC_AUTH_TYPE"), "authConfig": SimTypeChar(label="Byte")}, name="IPSEC_AUTH_TRANSFORM_ID0", pack=False, align=None), "cryptoModuleId": SimTypePointer(SimTypeBottom(label="Guid"), offset=0)}, name="IPSEC_AUTH_TRANSFORM0", pack=False, align=None), "authKey": SimStruct({"size": SimTypeInt(signed=False, label="UInt32"), "data": SimTypePointer(SimTypeChar(label="Byte"), offset=0)}, name="FWP_BYTE_BLOB", pack=False, align=None)}, name="IPSEC_SA_AUTH_INFORMATION0", pack=False, align=None), offset=0), "espCipherInformation": SimTypePointer(SimStruct({"cipherTransform": SimStruct({"cipherTransformId": SimStruct({"cipherType": SimTypeInt(signed=False, label="IPSEC_CIPHER_TYPE"), "cipherConfig": SimTypeChar(label="Byte")}, name="IPSEC_CIPHER_TRANSFORM_ID0", pack=False, align=None), "cryptoModuleId": SimTypePointer(SimTypeBottom(label="Guid"), offset=0)}, name="IPSEC_CIPHER_TRANSFORM0", pack=False, align=None), "cipherKey": SimStruct({"size": SimTypeInt(signed=False, label="UInt32"), "data": SimTypePointer(SimTypeChar(label="Byte"), offset=0)}, name="FWP_BYTE_BLOB", pack=False, align=None)}, name="IPSEC_SA_CIPHER_INFORMATION0", pack=False, align=None), offset=0), "espAuthAndCipherInformation": SimTypePointer(SimStruct({"saCipherInformation": SimStruct({"cipherTransform": SimStruct({"cipherTransformId": SimStruct({"cipherType": SimTypeInt(signed=False, label="IPSEC_CIPHER_TYPE"), "cipherConfig": SimTypeChar(label="Byte")}, name="IPSEC_CIPHER_TRANSFORM_ID0", pack=False, align=None), "cryptoModuleId": SimTypePointer(SimTypeBottom(label="Guid"), offset=0)}, name="IPSEC_CIPHER_TRANSFORM0", pack=False, align=None), "cipherKey": SimStruct({"size": SimTypeInt(signed=False, label="UInt32"), "data": SimTypePointer(SimTypeChar(label="Byte"), offset=0)}, name="FWP_BYTE_BLOB", pack=False, align=None)}, name="IPSEC_SA_CIPHER_INFORMATION0", pack=False, align=None), "saAuthInformation": SimStruct({"authTransform": SimStruct({"authTransformId": SimStruct({"authType": SimTypeInt(signed=False, label="IPSEC_AUTH_TYPE"), "authConfig": SimTypeChar(label="Byte")}, name="IPSEC_AUTH_TRANSFORM_ID0", pack=False, align=None), "cryptoModuleId": SimTypePointer(SimTypeBottom(label="Guid"), offset=0)}, name="IPSEC_AUTH_TRANSFORM0", pack=False, align=None), "authKey": SimStruct({"size": SimTypeInt(signed=False, label="UInt32"), "data": SimTypePointer(SimTypeChar(label="Byte"), offset=0)}, name="FWP_BYTE_BLOB", pack=False, align=None)}, name="IPSEC_SA_AUTH_INFORMATION0", pack=False, align=None)}, name="IPSEC_SA_AUTH_AND_CIPHER_INFORMATION0", pack=False, align=None), offset=0), "espAuthFwInformation": SimTypePointer(SimStruct({"authTransform": SimStruct({"authTransformId": SimStruct({"authType": SimTypeInt(signed=False, label="IPSEC_AUTH_TYPE"), "authConfig": SimTypeChar(label="Byte")}, name="IPSEC_AUTH_TRANSFORM_ID0", pack=False, align=None), "cryptoModuleId": SimTypePointer(SimTypeBottom(label="Guid"), offset=0)}, name="IPSEC_AUTH_TRANSFORM0", pack=False, align=None), "authKey": SimStruct({"size": SimTypeInt(signed=False, label="UInt32"), "data": SimTypePointer(SimTypeChar(label="Byte"), offset=0)}, name="FWP_BYTE_BLOB", pack=False, align=None)}, name="IPSEC_SA_AUTH_INFORMATION0", pack=False, align=None), offset=0)}, name="<anon>", label="None")}, name="IPSEC_SA0", pack=False, align=None), offset=0), "keyModuleState": SimTypePointer(SimStruct({"keyModuleKey": SimTypeBottom(label="Guid"), "stateBlob": SimStruct({"size": SimTypeInt(signed=False, label="UInt32"), "data": SimTypePointer(SimTypeChar(label="Byte"), offset=0)}, name="FWP_BYTE_BLOB", pack=False, align=None)}, name="IPSEC_KEYMODULE_STATE0", pack=False, align=None), offset=0), "ipVersion": SimTypeInt(signed=False, label="FWP_IP_VERSION"), "Anonymous": SimUnion({"peerV4PrivateAddress": SimTypeInt(signed=False, label="UInt32")}, name="<anon>", label="None"), "mmSaId": SimTypeLongLong(signed=False, label="UInt64"), "pfsGroup": SimTypeInt(signed=False, label="IPSEC_PFS_GROUP")}, name="IPSEC_SA_BUNDLE0", pack=False, align=None), offset=0)], SimTypeInt(signed=False, label="UInt32"), arg_names=["engineHandle", "id", "outboundBundle"]),
        # 
        'IPsecSaContextAddInbound1': SimTypeFunction([SimTypePointer(SimTypeInt(signed=True, label="Int"), label="IntPtr", offset=0), SimTypeLongLong(signed=False, label="UInt64"), SimTypePointer(SimStruct({"flags": SimTypeInt(signed=False, label="IPSEC_SA_BUNDLE_FLAGS"), "lifetime": SimStruct({"lifetimeSeconds": SimTypeInt(signed=False, label="UInt32"), "lifetimeKilobytes": SimTypeInt(signed=False, label="UInt32"), "lifetimePackets": SimTypeInt(signed=False, label="UInt32")}, name="IPSEC_SA_LIFETIME0", pack=False, align=None), "idleTimeoutSeconds": SimTypeInt(signed=False, label="UInt32"), "ndAllowClearTimeoutSeconds": SimTypeInt(signed=False, label="UInt32"), "ipsecId": SimTypePointer(SimStruct({"mmTargetName": SimTypePointer(SimTypeChar(label="Char"), offset=0), "emTargetName": SimTypePointer(SimTypeChar(label="Char"), offset=0), "numTokens": SimTypeInt(signed=False, label="UInt32"), "tokens": SimTypePointer(SimStruct({"type": SimTypeInt(signed=False, label="IPSEC_TOKEN_TYPE"), "principal": SimTypeInt(signed=False, label="IPSEC_TOKEN_PRINCIPAL"), "mode": SimTypeInt(signed=False, label="IPSEC_TOKEN_MODE"), "token": SimTypeLongLong(signed=False, label="UInt64")}, name="IPSEC_TOKEN0", pack=False, align=None), offset=0), "explicitCredentials": SimTypeLongLong(signed=False, label="UInt64"), "logonId": SimTypeLongLong(signed=False, label="UInt64")}, name="IPSEC_ID0", pack=False, align=None), offset=0), "napContext": SimTypeInt(signed=False, label="UInt32"), "qmSaId": SimTypeInt(signed=False, label="UInt32"), "numSAs": SimTypeInt(signed=False, label="UInt32"), "saList": SimTypePointer(SimStruct({"spi": SimTypeInt(signed=False, label="UInt32"), "saTransformType": SimTypeInt(signed=False, label="IPSEC_TRANSFORM_TYPE"), "Anonymous": SimUnion({"ahInformation": SimTypePointer(SimStruct({"authTransform": SimStruct({"authTransformId": SimStruct({"authType": SimTypeInt(signed=False, label="IPSEC_AUTH_TYPE"), "authConfig": SimTypeChar(label="Byte")}, name="IPSEC_AUTH_TRANSFORM_ID0", pack=False, align=None), "cryptoModuleId": SimTypePointer(SimTypeBottom(label="Guid"), offset=0)}, name="IPSEC_AUTH_TRANSFORM0", pack=False, align=None), "authKey": SimStruct({"size": SimTypeInt(signed=False, label="UInt32"), "data": SimTypePointer(SimTypeChar(label="Byte"), offset=0)}, name="FWP_BYTE_BLOB", pack=False, align=None)}, name="IPSEC_SA_AUTH_INFORMATION0", pack=False, align=None), offset=0), "espAuthInformation": SimTypePointer(SimStruct({"authTransform": SimStruct({"authTransformId": SimStruct({"authType": SimTypeInt(signed=False, label="IPSEC_AUTH_TYPE"), "authConfig": SimTypeChar(label="Byte")}, name="IPSEC_AUTH_TRANSFORM_ID0", pack=False, align=None), "cryptoModuleId": SimTypePointer(SimTypeBottom(label="Guid"), offset=0)}, name="IPSEC_AUTH_TRANSFORM0", pack=False, align=None), "authKey": SimStruct({"size": SimTypeInt(signed=False, label="UInt32"), "data": SimTypePointer(SimTypeChar(label="Byte"), offset=0)}, name="FWP_BYTE_BLOB", pack=False, align=None)}, name="IPSEC_SA_AUTH_INFORMATION0", pack=False, align=None), offset=0), "espCipherInformation": SimTypePointer(SimStruct({"cipherTransform": SimStruct({"cipherTransformId": SimStruct({"cipherType": SimTypeInt(signed=False, label="IPSEC_CIPHER_TYPE"), "cipherConfig": SimTypeChar(label="Byte")}, name="IPSEC_CIPHER_TRANSFORM_ID0", pack=False, align=None), "cryptoModuleId": SimTypePointer(SimTypeBottom(label="Guid"), offset=0)}, name="IPSEC_CIPHER_TRANSFORM0", pack=False, align=None), "cipherKey": SimStruct({"size": SimTypeInt(signed=False, label="UInt32"), "data": SimTypePointer(SimTypeChar(label="Byte"), offset=0)}, name="FWP_BYTE_BLOB", pack=False, align=None)}, name="IPSEC_SA_CIPHER_INFORMATION0", pack=False, align=None), offset=0), "espAuthAndCipherInformation": SimTypePointer(SimStruct({"saCipherInformation": SimStruct({"cipherTransform": SimStruct({"cipherTransformId": SimStruct({"cipherType": SimTypeInt(signed=False, label="IPSEC_CIPHER_TYPE"), "cipherConfig": SimTypeChar(label="Byte")}, name="IPSEC_CIPHER_TRANSFORM_ID0", pack=False, align=None), "cryptoModuleId": SimTypePointer(SimTypeBottom(label="Guid"), offset=0)}, name="IPSEC_CIPHER_TRANSFORM0", pack=False, align=None), "cipherKey": SimStruct({"size": SimTypeInt(signed=False, label="UInt32"), "data": SimTypePointer(SimTypeChar(label="Byte"), offset=0)}, name="FWP_BYTE_BLOB", pack=False, align=None)}, name="IPSEC_SA_CIPHER_INFORMATION0", pack=False, align=None), "saAuthInformation": SimStruct({"authTransform": SimStruct({"authTransformId": SimStruct({"authType": SimTypeInt(signed=False, label="IPSEC_AUTH_TYPE"), "authConfig": SimTypeChar(label="Byte")}, name="IPSEC_AUTH_TRANSFORM_ID0", pack=False, align=None), "cryptoModuleId": SimTypePointer(SimTypeBottom(label="Guid"), offset=0)}, name="IPSEC_AUTH_TRANSFORM0", pack=False, align=None), "authKey": SimStruct({"size": SimTypeInt(signed=False, label="UInt32"), "data": SimTypePointer(SimTypeChar(label="Byte"), offset=0)}, name="FWP_BYTE_BLOB", pack=False, align=None)}, name="IPSEC_SA_AUTH_INFORMATION0", pack=False, align=None)}, name="IPSEC_SA_AUTH_AND_CIPHER_INFORMATION0", pack=False, align=None), offset=0), "espAuthFwInformation": SimTypePointer(SimStruct({"authTransform": SimStruct({"authTransformId": SimStruct({"authType": SimTypeInt(signed=False, label="IPSEC_AUTH_TYPE"), "authConfig": SimTypeChar(label="Byte")}, name="IPSEC_AUTH_TRANSFORM_ID0", pack=False, align=None), "cryptoModuleId": SimTypePointer(SimTypeBottom(label="Guid"), offset=0)}, name="IPSEC_AUTH_TRANSFORM0", pack=False, align=None), "authKey": SimStruct({"size": SimTypeInt(signed=False, label="UInt32"), "data": SimTypePointer(SimTypeChar(label="Byte"), offset=0)}, name="FWP_BYTE_BLOB", pack=False, align=None)}, name="IPSEC_SA_AUTH_INFORMATION0", pack=False, align=None), offset=0)}, name="<anon>", label="None")}, name="IPSEC_SA0", pack=False, align=None), offset=0), "keyModuleState": SimTypePointer(SimStruct({"keyModuleKey": SimTypeBottom(label="Guid"), "stateBlob": SimStruct({"size": SimTypeInt(signed=False, label="UInt32"), "data": SimTypePointer(SimTypeChar(label="Byte"), offset=0)}, name="FWP_BYTE_BLOB", pack=False, align=None)}, name="IPSEC_KEYMODULE_STATE0", pack=False, align=None), offset=0), "ipVersion": SimTypeInt(signed=False, label="FWP_IP_VERSION"), "Anonymous": SimUnion({"peerV4PrivateAddress": SimTypeInt(signed=False, label="UInt32")}, name="<anon>", label="None"), "mmSaId": SimTypeLongLong(signed=False, label="UInt64"), "pfsGroup": SimTypeInt(signed=False, label="IPSEC_PFS_GROUP"), "saLookupContext": SimTypeBottom(label="Guid"), "qmFilterId": SimTypeLongLong(signed=False, label="UInt64")}, name="IPSEC_SA_BUNDLE1", pack=False, align=None), offset=0)], SimTypeInt(signed=False, label="UInt32"), arg_names=["engineHandle", "id", "inboundBundle"]),
        # 
        'IPsecSaContextAddOutbound1': SimTypeFunction([SimTypePointer(SimTypeInt(signed=True, label="Int"), label="IntPtr", offset=0), SimTypeLongLong(signed=False, label="UInt64"), SimTypePointer(SimStruct({"flags": SimTypeInt(signed=False, label="IPSEC_SA_BUNDLE_FLAGS"), "lifetime": SimStruct({"lifetimeSeconds": SimTypeInt(signed=False, label="UInt32"), "lifetimeKilobytes": SimTypeInt(signed=False, label="UInt32"), "lifetimePackets": SimTypeInt(signed=False, label="UInt32")}, name="IPSEC_SA_LIFETIME0", pack=False, align=None), "idleTimeoutSeconds": SimTypeInt(signed=False, label="UInt32"), "ndAllowClearTimeoutSeconds": SimTypeInt(signed=False, label="UInt32"), "ipsecId": SimTypePointer(SimStruct({"mmTargetName": SimTypePointer(SimTypeChar(label="Char"), offset=0), "emTargetName": SimTypePointer(SimTypeChar(label="Char"), offset=0), "numTokens": SimTypeInt(signed=False, label="UInt32"), "tokens": SimTypePointer(SimStruct({"type": SimTypeInt(signed=False, label="IPSEC_TOKEN_TYPE"), "principal": SimTypeInt(signed=False, label="IPSEC_TOKEN_PRINCIPAL"), "mode": SimTypeInt(signed=False, label="IPSEC_TOKEN_MODE"), "token": SimTypeLongLong(signed=False, label="UInt64")}, name="IPSEC_TOKEN0", pack=False, align=None), offset=0), "explicitCredentials": SimTypeLongLong(signed=False, label="UInt64"), "logonId": SimTypeLongLong(signed=False, label="UInt64")}, name="IPSEC_ID0", pack=False, align=None), offset=0), "napContext": SimTypeInt(signed=False, label="UInt32"), "qmSaId": SimTypeInt(signed=False, label="UInt32"), "numSAs": SimTypeInt(signed=False, label="UInt32"), "saList": SimTypePointer(SimStruct({"spi": SimTypeInt(signed=False, label="UInt32"), "saTransformType": SimTypeInt(signed=False, label="IPSEC_TRANSFORM_TYPE"), "Anonymous": SimUnion({"ahInformation": SimTypePointer(SimStruct({"authTransform": SimStruct({"authTransformId": SimStruct({"authType": SimTypeInt(signed=False, label="IPSEC_AUTH_TYPE"), "authConfig": SimTypeChar(label="Byte")}, name="IPSEC_AUTH_TRANSFORM_ID0", pack=False, align=None), "cryptoModuleId": SimTypePointer(SimTypeBottom(label="Guid"), offset=0)}, name="IPSEC_AUTH_TRANSFORM0", pack=False, align=None), "authKey": SimStruct({"size": SimTypeInt(signed=False, label="UInt32"), "data": SimTypePointer(SimTypeChar(label="Byte"), offset=0)}, name="FWP_BYTE_BLOB", pack=False, align=None)}, name="IPSEC_SA_AUTH_INFORMATION0", pack=False, align=None), offset=0), "espAuthInformation": SimTypePointer(SimStruct({"authTransform": SimStruct({"authTransformId": SimStruct({"authType": SimTypeInt(signed=False, label="IPSEC_AUTH_TYPE"), "authConfig": SimTypeChar(label="Byte")}, name="IPSEC_AUTH_TRANSFORM_ID0", pack=False, align=None), "cryptoModuleId": SimTypePointer(SimTypeBottom(label="Guid"), offset=0)}, name="IPSEC_AUTH_TRANSFORM0", pack=False, align=None), "authKey": SimStruct({"size": SimTypeInt(signed=False, label="UInt32"), "data": SimTypePointer(SimTypeChar(label="Byte"), offset=0)}, name="FWP_BYTE_BLOB", pack=False, align=None)}, name="IPSEC_SA_AUTH_INFORMATION0", pack=False, align=None), offset=0), "espCipherInformation": SimTypePointer(SimStruct({"cipherTransform": SimStruct({"cipherTransformId": SimStruct({"cipherType": SimTypeInt(signed=False, label="IPSEC_CIPHER_TYPE"), "cipherConfig": SimTypeChar(label="Byte")}, name="IPSEC_CIPHER_TRANSFORM_ID0", pack=False, align=None), "cryptoModuleId": SimTypePointer(SimTypeBottom(label="Guid"), offset=0)}, name="IPSEC_CIPHER_TRANSFORM0", pack=False, align=None), "cipherKey": SimStruct({"size": SimTypeInt(signed=False, label="UInt32"), "data": SimTypePointer(SimTypeChar(label="Byte"), offset=0)}, name="FWP_BYTE_BLOB", pack=False, align=None)}, name="IPSEC_SA_CIPHER_INFORMATION0", pack=False, align=None), offset=0), "espAuthAndCipherInformation": SimTypePointer(SimStruct({"saCipherInformation": SimStruct({"cipherTransform": SimStruct({"cipherTransformId": SimStruct({"cipherType": SimTypeInt(signed=False, label="IPSEC_CIPHER_TYPE"), "cipherConfig": SimTypeChar(label="Byte")}, name="IPSEC_CIPHER_TRANSFORM_ID0", pack=False, align=None), "cryptoModuleId": SimTypePointer(SimTypeBottom(label="Guid"), offset=0)}, name="IPSEC_CIPHER_TRANSFORM0", pack=False, align=None), "cipherKey": SimStruct({"size": SimTypeInt(signed=False, label="UInt32"), "data": SimTypePointer(SimTypeChar(label="Byte"), offset=0)}, name="FWP_BYTE_BLOB", pack=False, align=None)}, name="IPSEC_SA_CIPHER_INFORMATION0", pack=False, align=None), "saAuthInformation": SimStruct({"authTransform": SimStruct({"authTransformId": SimStruct({"authType": SimTypeInt(signed=False, label="IPSEC_AUTH_TYPE"), "authConfig": SimTypeChar(label="Byte")}, name="IPSEC_AUTH_TRANSFORM_ID0", pack=False, align=None), "cryptoModuleId": SimTypePointer(SimTypeBottom(label="Guid"), offset=0)}, name="IPSEC_AUTH_TRANSFORM0", pack=False, align=None), "authKey": SimStruct({"size": SimTypeInt(signed=False, label="UInt32"), "data": SimTypePointer(SimTypeChar(label="Byte"), offset=0)}, name="FWP_BYTE_BLOB", pack=False, align=None)}, name="IPSEC_SA_AUTH_INFORMATION0", pack=False, align=None)}, name="IPSEC_SA_AUTH_AND_CIPHER_INFORMATION0", pack=False, align=None), offset=0), "espAuthFwInformation": SimTypePointer(SimStruct({"authTransform": SimStruct({"authTransformId": SimStruct({"authType": SimTypeInt(signed=False, label="IPSEC_AUTH_TYPE"), "authConfig": SimTypeChar(label="Byte")}, name="IPSEC_AUTH_TRANSFORM_ID0", pack=False, align=None), "cryptoModuleId": SimTypePointer(SimTypeBottom(label="Guid"), offset=0)}, name="IPSEC_AUTH_TRANSFORM0", pack=False, align=None), "authKey": SimStruct({"size": SimTypeInt(signed=False, label="UInt32"), "data": SimTypePointer(SimTypeChar(label="Byte"), offset=0)}, name="FWP_BYTE_BLOB", pack=False, align=None)}, name="IPSEC_SA_AUTH_INFORMATION0", pack=False, align=None), offset=0)}, name="<anon>", label="None")}, name="IPSEC_SA0", pack=False, align=None), offset=0), "keyModuleState": SimTypePointer(SimStruct({"keyModuleKey": SimTypeBottom(label="Guid"), "stateBlob": SimStruct({"size": SimTypeInt(signed=False, label="UInt32"), "data": SimTypePointer(SimTypeChar(label="Byte"), offset=0)}, name="FWP_BYTE_BLOB", pack=False, align=None)}, name="IPSEC_KEYMODULE_STATE0", pack=False, align=None), offset=0), "ipVersion": SimTypeInt(signed=False, label="FWP_IP_VERSION"), "Anonymous": SimUnion({"peerV4PrivateAddress": SimTypeInt(signed=False, label="UInt32")}, name="<anon>", label="None"), "mmSaId": SimTypeLongLong(signed=False, label="UInt64"), "pfsGroup": SimTypeInt(signed=False, label="IPSEC_PFS_GROUP"), "saLookupContext": SimTypeBottom(label="Guid"), "qmFilterId": SimTypeLongLong(signed=False, label="UInt64")}, name="IPSEC_SA_BUNDLE1", pack=False, align=None), offset=0)], SimTypeInt(signed=False, label="UInt32"), arg_names=["engineHandle", "id", "outboundBundle"]),
        # 
        'IPsecSaContextExpire0': SimTypeFunction([SimTypePointer(SimTypeInt(signed=True, label="Int"), label="IntPtr", offset=0), SimTypeLongLong(signed=False, label="UInt64")], SimTypeInt(signed=False, label="UInt32"), arg_names=["engineHandle", "id"]),
        # 
        'IPsecSaContextUpdate0': SimTypeFunction([SimTypePointer(SimTypeInt(signed=True, label="Int"), label="IntPtr", offset=0), SimTypeLongLong(signed=False, label="UInt64"), SimTypePointer(SimStruct({"saContextId": SimTypeLongLong(signed=False, label="UInt64"), "inboundSa": SimTypePointer(SimStruct({"ipVersion": SimTypeInt(signed=False, label="FWP_IP_VERSION"), "saDirection": SimTypeInt(signed=False, label="FWP_DIRECTION"), "traffic": SimStruct({"ipVersion": SimTypeInt(signed=False, label="FWP_IP_VERSION"), "Anonymous1": SimUnion({"localV4Address": SimTypeInt(signed=False, label="UInt32"), "localV6Address": SimTypeFixedSizeArray(SimTypeChar(label="Byte"), 16)}, name="<anon>", label="None"), "Anonymous2": SimUnion({"remoteV4Address": SimTypeInt(signed=False, label="UInt32"), "remoteV6Address": SimTypeFixedSizeArray(SimTypeChar(label="Byte"), 16)}, name="<anon>", label="None"), "trafficType": SimTypeInt(signed=False, label="IPSEC_TRAFFIC_TYPE"), "Anonymous3": SimUnion({"ipsecFilterId": SimTypeLongLong(signed=False, label="UInt64"), "tunnelPolicyId": SimTypeLongLong(signed=False, label="UInt64")}, name="<anon>", label="None"), "remotePort": SimTypeShort(signed=False, label="UInt16"), "localPort": SimTypeShort(signed=False, label="UInt16"), "ipProtocol": SimTypeChar(label="Byte"), "localIfLuid": SimTypeLongLong(signed=False, label="UInt64"), "realIfProfileId": SimTypeInt(signed=False, label="UInt32")}, name="IPSEC_TRAFFIC1", pack=False, align=None), "saBundle": SimStruct({"flags": SimTypeInt(signed=False, label="IPSEC_SA_BUNDLE_FLAGS"), "lifetime": SimStruct({"lifetimeSeconds": SimTypeInt(signed=False, label="UInt32"), "lifetimeKilobytes": SimTypeInt(signed=False, label="UInt32"), "lifetimePackets": SimTypeInt(signed=False, label="UInt32")}, name="IPSEC_SA_LIFETIME0", pack=False, align=None), "idleTimeoutSeconds": SimTypeInt(signed=False, label="UInt32"), "ndAllowClearTimeoutSeconds": SimTypeInt(signed=False, label="UInt32"), "ipsecId": SimTypePointer(SimStruct({"mmTargetName": SimTypePointer(SimTypeChar(label="Char"), offset=0), "emTargetName": SimTypePointer(SimTypeChar(label="Char"), offset=0), "numTokens": SimTypeInt(signed=False, label="UInt32"), "tokens": SimTypePointer(SimStruct({"type": SimTypeInt(signed=False, label="IPSEC_TOKEN_TYPE"), "principal": SimTypeInt(signed=False, label="IPSEC_TOKEN_PRINCIPAL"), "mode": SimTypeInt(signed=False, label="IPSEC_TOKEN_MODE"), "token": SimTypeLongLong(signed=False, label="UInt64")}, name="IPSEC_TOKEN0", pack=False, align=None), offset=0), "explicitCredentials": SimTypeLongLong(signed=False, label="UInt64"), "logonId": SimTypeLongLong(signed=False, label="UInt64")}, name="IPSEC_ID0", pack=False, align=None), offset=0), "napContext": SimTypeInt(signed=False, label="UInt32"), "qmSaId": SimTypeInt(signed=False, label="UInt32"), "numSAs": SimTypeInt(signed=False, label="UInt32"), "saList": SimTypePointer(SimStruct({"spi": SimTypeInt(signed=False, label="UInt32"), "saTransformType": SimTypeInt(signed=False, label="IPSEC_TRANSFORM_TYPE"), "Anonymous": SimUnion({"ahInformation": SimTypePointer(SimStruct({"authTransform": SimStruct({"authTransformId": SimStruct({"authType": SimTypeInt(signed=False, label="IPSEC_AUTH_TYPE"), "authConfig": SimTypeChar(label="Byte")}, name="IPSEC_AUTH_TRANSFORM_ID0", pack=False, align=None), "cryptoModuleId": SimTypePointer(SimTypeBottom(label="Guid"), offset=0)}, name="IPSEC_AUTH_TRANSFORM0", pack=False, align=None), "authKey": SimStruct({"size": SimTypeInt(signed=False, label="UInt32"), "data": SimTypePointer(SimTypeChar(label="Byte"), offset=0)}, name="FWP_BYTE_BLOB", pack=False, align=None)}, name="IPSEC_SA_AUTH_INFORMATION0", pack=False, align=None), offset=0), "espAuthInformation": SimTypePointer(SimStruct({"authTransform": SimStruct({"authTransformId": SimStruct({"authType": SimTypeInt(signed=False, label="IPSEC_AUTH_TYPE"), "authConfig": SimTypeChar(label="Byte")}, name="IPSEC_AUTH_TRANSFORM_ID0", pack=False, align=None), "cryptoModuleId": SimTypePointer(SimTypeBottom(label="Guid"), offset=0)}, name="IPSEC_AUTH_TRANSFORM0", pack=False, align=None), "authKey": SimStruct({"size": SimTypeInt(signed=False, label="UInt32"), "data": SimTypePointer(SimTypeChar(label="Byte"), offset=0)}, name="FWP_BYTE_BLOB", pack=False, align=None)}, name="IPSEC_SA_AUTH_INFORMATION0", pack=False, align=None), offset=0), "espCipherInformation": SimTypePointer(SimStruct({"cipherTransform": SimStruct({"cipherTransformId": SimStruct({"cipherType": SimTypeInt(signed=False, label="IPSEC_CIPHER_TYPE"), "cipherConfig": SimTypeChar(label="Byte")}, name="IPSEC_CIPHER_TRANSFORM_ID0", pack=False, align=None), "cryptoModuleId": SimTypePointer(SimTypeBottom(label="Guid"), offset=0)}, name="IPSEC_CIPHER_TRANSFORM0", pack=False, align=None), "cipherKey": SimStruct({"size": SimTypeInt(signed=False, label="UInt32"), "data": SimTypePointer(SimTypeChar(label="Byte"), offset=0)}, name="FWP_BYTE_BLOB", pack=False, align=None)}, name="IPSEC_SA_CIPHER_INFORMATION0", pack=False, align=None), offset=0), "espAuthAndCipherInformation": SimTypePointer(SimStruct({"saCipherInformation": SimStruct({"cipherTransform": SimStruct({"cipherTransformId": SimStruct({"cipherType": SimTypeInt(signed=False, label="IPSEC_CIPHER_TYPE"), "cipherConfig": SimTypeChar(label="Byte")}, name="IPSEC_CIPHER_TRANSFORM_ID0", pack=False, align=None), "cryptoModuleId": SimTypePointer(SimTypeBottom(label="Guid"), offset=0)}, name="IPSEC_CIPHER_TRANSFORM0", pack=False, align=None), "cipherKey": SimStruct({"size": SimTypeInt(signed=False, label="UInt32"), "data": SimTypePointer(SimTypeChar(label="Byte"), offset=0)}, name="FWP_BYTE_BLOB", pack=False, align=None)}, name="IPSEC_SA_CIPHER_INFORMATION0", pack=False, align=None), "saAuthInformation": SimStruct({"authTransform": SimStruct({"authTransformId": SimStruct({"authType": SimTypeInt(signed=False, label="IPSEC_AUTH_TYPE"), "authConfig": SimTypeChar(label="Byte")}, name="IPSEC_AUTH_TRANSFORM_ID0", pack=False, align=None), "cryptoModuleId": SimTypePointer(SimTypeBottom(label="Guid"), offset=0)}, name="IPSEC_AUTH_TRANSFORM0", pack=False, align=None), "authKey": SimStruct({"size": SimTypeInt(signed=False, label="UInt32"), "data": SimTypePointer(SimTypeChar(label="Byte"), offset=0)}, name="FWP_BYTE_BLOB", pack=False, align=None)}, name="IPSEC_SA_AUTH_INFORMATION0", pack=False, align=None)}, name="IPSEC_SA_AUTH_AND_CIPHER_INFORMATION0", pack=False, align=None), offset=0), "espAuthFwInformation": SimTypePointer(SimStruct({"authTransform": SimStruct({"authTransformId": SimStruct({"authType": SimTypeInt(signed=False, label="IPSEC_AUTH_TYPE"), "authConfig": SimTypeChar(label="Byte")}, name="IPSEC_AUTH_TRANSFORM_ID0", pack=False, align=None), "cryptoModuleId": SimTypePointer(SimTypeBottom(label="Guid"), offset=0)}, name="IPSEC_AUTH_TRANSFORM0", pack=False, align=None), "authKey": SimStruct({"size": SimTypeInt(signed=False, label="UInt32"), "data": SimTypePointer(SimTypeChar(label="Byte"), offset=0)}, name="FWP_BYTE_BLOB", pack=False, align=None)}, name="IPSEC_SA_AUTH_INFORMATION0", pack=False, align=None), offset=0)}, name="<anon>", label="None")}, name="IPSEC_SA0", pack=False, align=None), offset=0), "keyModuleState": SimTypePointer(SimStruct({"keyModuleKey": SimTypeBottom(label="Guid"), "stateBlob": SimStruct({"size": SimTypeInt(signed=False, label="UInt32"), "data": SimTypePointer(SimTypeChar(label="Byte"), offset=0)}, name="FWP_BYTE_BLOB", pack=False, align=None)}, name="IPSEC_KEYMODULE_STATE0", pack=False, align=None), offset=0), "ipVersion": SimTypeInt(signed=False, label="FWP_IP_VERSION"), "Anonymous": SimUnion({"peerV4PrivateAddress": SimTypeInt(signed=False, label="UInt32")}, name="<anon>", label="None"), "mmSaId": SimTypeLongLong(signed=False, label="UInt64"), "pfsGroup": SimTypeInt(signed=False, label="IPSEC_PFS_GROUP"), "saLookupContext": SimTypeBottom(label="Guid"), "qmFilterId": SimTypeLongLong(signed=False, label="UInt64")}, name="IPSEC_SA_BUNDLE1", pack=False, align=None), "Anonymous": SimUnion({"udpEncapsulation": SimTypePointer(SimStruct({"localUdpEncapPort": SimTypeShort(signed=False, label="UInt16"), "remoteUdpEncapPort": SimTypeShort(signed=False, label="UInt16")}, name="IPSEC_V4_UDP_ENCAPSULATION0", pack=False, align=None), offset=0)}, name="<anon>", label="None"), "transportFilter": SimTypePointer(SimTypeBottom(label="FWPM_FILTER0"), offset=0), "virtualIfTunnelInfo": SimStruct({"virtualIfTunnelId": SimTypeLongLong(signed=False, label="UInt64"), "trafficSelectorId": SimTypeLongLong(signed=False, label="UInt64")}, name="IPSEC_VIRTUAL_IF_TUNNEL_INFO0", pack=False, align=None)}, name="IPSEC_SA_DETAILS1", pack=False, align=None), offset=0), "outboundSa": SimTypePointer(SimStruct({"ipVersion": SimTypeInt(signed=False, label="FWP_IP_VERSION"), "saDirection": SimTypeInt(signed=False, label="FWP_DIRECTION"), "traffic": SimStruct({"ipVersion": SimTypeInt(signed=False, label="FWP_IP_VERSION"), "Anonymous1": SimUnion({"localV4Address": SimTypeInt(signed=False, label="UInt32"), "localV6Address": SimTypeFixedSizeArray(SimTypeChar(label="Byte"), 16)}, name="<anon>", label="None"), "Anonymous2": SimUnion({"remoteV4Address": SimTypeInt(signed=False, label="UInt32"), "remoteV6Address": SimTypeFixedSizeArray(SimTypeChar(label="Byte"), 16)}, name="<anon>", label="None"), "trafficType": SimTypeInt(signed=False, label="IPSEC_TRAFFIC_TYPE"), "Anonymous3": SimUnion({"ipsecFilterId": SimTypeLongLong(signed=False, label="UInt64"), "tunnelPolicyId": SimTypeLongLong(signed=False, label="UInt64")}, name="<anon>", label="None"), "remotePort": SimTypeShort(signed=False, label="UInt16"), "localPort": SimTypeShort(signed=False, label="UInt16"), "ipProtocol": SimTypeChar(label="Byte"), "localIfLuid": SimTypeLongLong(signed=False, label="UInt64"), "realIfProfileId": SimTypeInt(signed=False, label="UInt32")}, name="IPSEC_TRAFFIC1", pack=False, align=None), "saBundle": SimStruct({"flags": SimTypeInt(signed=False, label="IPSEC_SA_BUNDLE_FLAGS"), "lifetime": SimStruct({"lifetimeSeconds": SimTypeInt(signed=False, label="UInt32"), "lifetimeKilobytes": SimTypeInt(signed=False, label="UInt32"), "lifetimePackets": SimTypeInt(signed=False, label="UInt32")}, name="IPSEC_SA_LIFETIME0", pack=False, align=None), "idleTimeoutSeconds": SimTypeInt(signed=False, label="UInt32"), "ndAllowClearTimeoutSeconds": SimTypeInt(signed=False, label="UInt32"), "ipsecId": SimTypePointer(SimStruct({"mmTargetName": SimTypePointer(SimTypeChar(label="Char"), offset=0), "emTargetName": SimTypePointer(SimTypeChar(label="Char"), offset=0), "numTokens": SimTypeInt(signed=False, label="UInt32"), "tokens": SimTypePointer(SimStruct({"type": SimTypeInt(signed=False, label="IPSEC_TOKEN_TYPE"), "principal": SimTypeInt(signed=False, label="IPSEC_TOKEN_PRINCIPAL"), "mode": SimTypeInt(signed=False, label="IPSEC_TOKEN_MODE"), "token": SimTypeLongLong(signed=False, label="UInt64")}, name="IPSEC_TOKEN0", pack=False, align=None), offset=0), "explicitCredentials": SimTypeLongLong(signed=False, label="UInt64"), "logonId": SimTypeLongLong(signed=False, label="UInt64")}, name="IPSEC_ID0", pack=False, align=None), offset=0), "napContext": SimTypeInt(signed=False, label="UInt32"), "qmSaId": SimTypeInt(signed=False, label="UInt32"), "numSAs": SimTypeInt(signed=False, label="UInt32"), "saList": SimTypePointer(SimStruct({"spi": SimTypeInt(signed=False, label="UInt32"), "saTransformType": SimTypeInt(signed=False, label="IPSEC_TRANSFORM_TYPE"), "Anonymous": SimUnion({"ahInformation": SimTypePointer(SimStruct({"authTransform": SimStruct({"authTransformId": SimStruct({"authType": SimTypeInt(signed=False, label="IPSEC_AUTH_TYPE"), "authConfig": SimTypeChar(label="Byte")}, name="IPSEC_AUTH_TRANSFORM_ID0", pack=False, align=None), "cryptoModuleId": SimTypePointer(SimTypeBottom(label="Guid"), offset=0)}, name="IPSEC_AUTH_TRANSFORM0", pack=False, align=None), "authKey": SimStruct({"size": SimTypeInt(signed=False, label="UInt32"), "data": SimTypePointer(SimTypeChar(label="Byte"), offset=0)}, name="FWP_BYTE_BLOB", pack=False, align=None)}, name="IPSEC_SA_AUTH_INFORMATION0", pack=False, align=None), offset=0), "espAuthInformation": SimTypePointer(SimStruct({"authTransform": SimStruct({"authTransformId": SimStruct({"authType": SimTypeInt(signed=False, label="IPSEC_AUTH_TYPE"), "authConfig": SimTypeChar(label="Byte")}, name="IPSEC_AUTH_TRANSFORM_ID0", pack=False, align=None), "cryptoModuleId": SimTypePointer(SimTypeBottom(label="Guid"), offset=0)}, name="IPSEC_AUTH_TRANSFORM0", pack=False, align=None), "authKey": SimStruct({"size": SimTypeInt(signed=False, label="UInt32"), "data": SimTypePointer(SimTypeChar(label="Byte"), offset=0)}, name="FWP_BYTE_BLOB", pack=False, align=None)}, name="IPSEC_SA_AUTH_INFORMATION0", pack=False, align=None), offset=0), "espCipherInformation": SimTypePointer(SimStruct({"cipherTransform": SimStruct({"cipherTransformId": SimStruct({"cipherType": SimTypeInt(signed=False, label="IPSEC_CIPHER_TYPE"), "cipherConfig": SimTypeChar(label="Byte")}, name="IPSEC_CIPHER_TRANSFORM_ID0", pack=False, align=None), "cryptoModuleId": SimTypePointer(SimTypeBottom(label="Guid"), offset=0)}, name="IPSEC_CIPHER_TRANSFORM0", pack=False, align=None), "cipherKey": SimStruct({"size": SimTypeInt(signed=False, label="UInt32"), "data": SimTypePointer(SimTypeChar(label="Byte"), offset=0)}, name="FWP_BYTE_BLOB", pack=False, align=None)}, name="IPSEC_SA_CIPHER_INFORMATION0", pack=False, align=None), offset=0), "espAuthAndCipherInformation": SimTypePointer(SimStruct({"saCipherInformation": SimStruct({"cipherTransform": SimStruct({"cipherTransformId": SimStruct({"cipherType": SimTypeInt(signed=False, label="IPSEC_CIPHER_TYPE"), "cipherConfig": SimTypeChar(label="Byte")}, name="IPSEC_CIPHER_TRANSFORM_ID0", pack=False, align=None), "cryptoModuleId": SimTypePointer(SimTypeBottom(label="Guid"), offset=0)}, name="IPSEC_CIPHER_TRANSFORM0", pack=False, align=None), "cipherKey": SimStruct({"size": SimTypeInt(signed=False, label="UInt32"), "data": SimTypePointer(SimTypeChar(label="Byte"), offset=0)}, name="FWP_BYTE_BLOB", pack=False, align=None)}, name="IPSEC_SA_CIPHER_INFORMATION0", pack=False, align=None), "saAuthInformation": SimStruct({"authTransform": SimStruct({"authTransformId": SimStruct({"authType": SimTypeInt(signed=False, label="IPSEC_AUTH_TYPE"), "authConfig": SimTypeChar(label="Byte")}, name="IPSEC_AUTH_TRANSFORM_ID0", pack=False, align=None), "cryptoModuleId": SimTypePointer(SimTypeBottom(label="Guid"), offset=0)}, name="IPSEC_AUTH_TRANSFORM0", pack=False, align=None), "authKey": SimStruct({"size": SimTypeInt(signed=False, label="UInt32"), "data": SimTypePointer(SimTypeChar(label="Byte"), offset=0)}, name="FWP_BYTE_BLOB", pack=False, align=None)}, name="IPSEC_SA_AUTH_INFORMATION0", pack=False, align=None)}, name="IPSEC_SA_AUTH_AND_CIPHER_INFORMATION0", pack=False, align=None), offset=0), "espAuthFwInformation": SimTypePointer(SimStruct({"authTransform": SimStruct({"authTransformId": SimStruct({"authType": SimTypeInt(signed=False, label="IPSEC_AUTH_TYPE"), "authConfig": SimTypeChar(label="Byte")}, name="IPSEC_AUTH_TRANSFORM_ID0", pack=False, align=None), "cryptoModuleId": SimTypePointer(SimTypeBottom(label="Guid"), offset=0)}, name="IPSEC_AUTH_TRANSFORM0", pack=False, align=None), "authKey": SimStruct({"size": SimTypeInt(signed=False, label="UInt32"), "data": SimTypePointer(SimTypeChar(label="Byte"), offset=0)}, name="FWP_BYTE_BLOB", pack=False, align=None)}, name="IPSEC_SA_AUTH_INFORMATION0", pack=False, align=None), offset=0)}, name="<anon>", label="None")}, name="IPSEC_SA0", pack=False, align=None), offset=0), "keyModuleState": SimTypePointer(SimStruct({"keyModuleKey": SimTypeBottom(label="Guid"), "stateBlob": SimStruct({"size": SimTypeInt(signed=False, label="UInt32"), "data": SimTypePointer(SimTypeChar(label="Byte"), offset=0)}, name="FWP_BYTE_BLOB", pack=False, align=None)}, name="IPSEC_KEYMODULE_STATE0", pack=False, align=None), offset=0), "ipVersion": SimTypeInt(signed=False, label="FWP_IP_VERSION"), "Anonymous": SimUnion({"peerV4PrivateAddress": SimTypeInt(signed=False, label="UInt32")}, name="<anon>", label="None"), "mmSaId": SimTypeLongLong(signed=False, label="UInt64"), "pfsGroup": SimTypeInt(signed=False, label="IPSEC_PFS_GROUP"), "saLookupContext": SimTypeBottom(label="Guid"), "qmFilterId": SimTypeLongLong(signed=False, label="UInt64")}, name="IPSEC_SA_BUNDLE1", pack=False, align=None), "Anonymous": SimUnion({"udpEncapsulation": SimTypePointer(SimStruct({"localUdpEncapPort": SimTypeShort(signed=False, label="UInt16"), "remoteUdpEncapPort": SimTypeShort(signed=False, label="UInt16")}, name="IPSEC_V4_UDP_ENCAPSULATION0", pack=False, align=None), offset=0)}, name="<anon>", label="None"), "transportFilter": SimTypePointer(SimTypeBottom(label="FWPM_FILTER0"), offset=0), "virtualIfTunnelInfo": SimStruct({"virtualIfTunnelId": SimTypeLongLong(signed=False, label="UInt64"), "trafficSelectorId": SimTypeLongLong(signed=False, label="UInt64")}, name="IPSEC_VIRTUAL_IF_TUNNEL_INFO0", pack=False, align=None)}, name="IPSEC_SA_DETAILS1", pack=False, align=None), offset=0)}, name="IPSEC_SA_CONTEXT1", pack=False, align=None), offset=0)], SimTypeInt(signed=False, label="UInt32"), arg_names=["engineHandle", "flags", "newValues"]),
        # 
        'IPsecSaContextCreateEnumHandle0': SimTypeFunction([SimTypePointer(SimTypeInt(signed=True, label="Int"), label="IntPtr", offset=0), SimTypePointer(SimStruct({"localSubNet": SimStruct({"type": SimTypeInt(signed=False, label="FWP_DATA_TYPE"), "Anonymous": SimUnion({"uint8": SimTypeChar(label="Byte"), "uint16": SimTypeShort(signed=False, label="UInt16"), "uint32": SimTypeInt(signed=False, label="UInt32"), "uint64": SimTypePointer(SimTypeLongLong(signed=False, label="UInt64"), offset=0), "int8": SimTypeChar(label="SByte"), "int16": SimTypeShort(signed=True, label="Int16"), "int32": SimTypeInt(signed=True, label="Int32"), "int64": SimTypePointer(SimTypeLongLong(signed=True, label="Int64"), offset=0), "float32": SimTypeFloat(size=32), "double64": SimTypePointer(SimTypeFloat(size=64), offset=0), "byteArray16": SimTypePointer(SimStruct({"byteArray16": SimTypeFixedSizeArray(SimTypeChar(label="Byte"), 16)}, name="FWP_BYTE_ARRAY16", pack=False, align=None), offset=0), "byteBlob": SimTypePointer(SimStruct({"size": SimTypeInt(signed=False, label="UInt32"), "data": SimTypePointer(SimTypeChar(label="Byte"), offset=0)}, name="FWP_BYTE_BLOB", pack=False, align=None), offset=0), "sid": SimTypePointer(SimTypeBottom(label="SID"), offset=0), "sd": SimTypePointer(SimStruct({"size": SimTypeInt(signed=False, label="UInt32"), "data": SimTypePointer(SimTypeChar(label="Byte"), offset=0)}, name="FWP_BYTE_BLOB", pack=False, align=None), offset=0), "tokenInformation": SimTypePointer(SimStruct({"sidCount": SimTypeInt(signed=False, label="UInt32"), "sids": SimTypePointer(SimTypeBottom(label="SID_AND_ATTRIBUTES"), offset=0), "restrictedSidCount": SimTypeInt(signed=False, label="UInt32"), "restrictedSids": SimTypePointer(SimTypeBottom(label="SID_AND_ATTRIBUTES"), offset=0)}, name="FWP_TOKEN_INFORMATION", pack=False, align=None), offset=0), "tokenAccessInformation": SimTypePointer(SimStruct({"size": SimTypeInt(signed=False, label="UInt32"), "data": SimTypePointer(SimTypeChar(label="Byte"), offset=0)}, name="FWP_BYTE_BLOB", pack=False, align=None), offset=0), "unicodeString": SimTypePointer(SimTypeChar(label="Char"), offset=0), "byteArray6": SimTypePointer(SimStruct({"byteArray6": SimTypeFixedSizeArray(SimTypeChar(label="Byte"), 6)}, name="FWP_BYTE_ARRAY6", pack=False, align=None), offset=0), "bitmapArray64": SimTypePointer(SimStruct({"bitmapArray64": SimTypeFixedSizeArray(SimTypeChar(label="Byte"), 8)}, name="FWP_BITMAP_ARRAY64_", pack=False, align=None), offset=0), "v4AddrMask": SimTypePointer(SimStruct({"addr": SimTypeInt(signed=False, label="UInt32"), "mask": SimTypeInt(signed=False, label="UInt32")}, name="FWP_V4_ADDR_AND_MASK", pack=False, align=None), offset=0), "v6AddrMask": SimTypePointer(SimStruct({"addr": SimTypeFixedSizeArray(SimTypeChar(label="Byte"), 16), "prefixLength": SimTypeChar(label="Byte")}, name="FWP_V6_ADDR_AND_MASK", pack=False, align=None), offset=0), "rangeValue": SimTypePointer(SimStruct({"valueLow": SimStruct({"type": SimTypeInt(signed=False, label="FWP_DATA_TYPE"), "Anonymous": SimUnion({"uint8": SimTypeChar(label="Byte"), "uint16": SimTypeShort(signed=False, label="UInt16"), "uint32": SimTypeInt(signed=False, label="UInt32"), "uint64": SimTypePointer(SimTypeLongLong(signed=False, label="UInt64"), offset=0), "int8": SimTypeChar(label="SByte"), "int16": SimTypeShort(signed=True, label="Int16"), "int32": SimTypeInt(signed=True, label="Int32"), "int64": SimTypePointer(SimTypeLongLong(signed=True, label="Int64"), offset=0), "float32": SimTypeFloat(size=32), "double64": SimTypePointer(SimTypeFloat(size=64), offset=0), "byteArray16": SimTypePointer(SimStruct({"byteArray16": SimTypeFixedSizeArray(SimTypeChar(label="Byte"), 16)}, name="FWP_BYTE_ARRAY16", pack=False, align=None), offset=0), "byteBlob": SimTypePointer(SimStruct({"size": SimTypeInt(signed=False, label="UInt32"), "data": SimTypePointer(SimTypeChar(label="Byte"), offset=0)}, name="FWP_BYTE_BLOB", pack=False, align=None), offset=0), "sid": SimTypePointer(SimTypeBottom(label="SID"), offset=0), "sd": SimTypePointer(SimStruct({"size": SimTypeInt(signed=False, label="UInt32"), "data": SimTypePointer(SimTypeChar(label="Byte"), offset=0)}, name="FWP_BYTE_BLOB", pack=False, align=None), offset=0), "tokenInformation": SimTypePointer(SimStruct({"sidCount": SimTypeInt(signed=False, label="UInt32"), "sids": SimTypePointer(SimTypeBottom(label="SID_AND_ATTRIBUTES"), offset=0), "restrictedSidCount": SimTypeInt(signed=False, label="UInt32"), "restrictedSids": SimTypePointer(SimTypeBottom(label="SID_AND_ATTRIBUTES"), offset=0)}, name="FWP_TOKEN_INFORMATION", pack=False, align=None), offset=0), "tokenAccessInformation": SimTypePointer(SimStruct({"size": SimTypeInt(signed=False, label="UInt32"), "data": SimTypePointer(SimTypeChar(label="Byte"), offset=0)}, name="FWP_BYTE_BLOB", pack=False, align=None), offset=0), "unicodeString": SimTypePointer(SimTypeChar(label="Char"), offset=0), "byteArray6": SimTypePointer(SimStruct({"byteArray6": SimTypeFixedSizeArray(SimTypeChar(label="Byte"), 6)}, name="FWP_BYTE_ARRAY6", pack=False, align=None), offset=0), "bitmapArray64": SimTypePointer(SimStruct({"bitmapArray64": SimTypeFixedSizeArray(SimTypeChar(label="Byte"), 8)}, name="FWP_BITMAP_ARRAY64_", pack=False, align=None), offset=0)}, name="<anon>", label="None")}, name="FWP_VALUE0", pack=False, align=None), "valueHigh": SimStruct({"type": SimTypeInt(signed=False, label="FWP_DATA_TYPE"), "Anonymous": SimUnion({"uint8": SimTypeChar(label="Byte"), "uint16": SimTypeShort(signed=False, label="UInt16"), "uint32": SimTypeInt(signed=False, label="UInt32"), "uint64": SimTypePointer(SimTypeLongLong(signed=False, label="UInt64"), offset=0), "int8": SimTypeChar(label="SByte"), "int16": SimTypeShort(signed=True, label="Int16"), "int32": SimTypeInt(signed=True, label="Int32"), "int64": SimTypePointer(SimTypeLongLong(signed=True, label="Int64"), offset=0), "float32": SimTypeFloat(size=32), "double64": SimTypePointer(SimTypeFloat(size=64), offset=0), "byteArray16": SimTypePointer(SimStruct({"byteArray16": SimTypeFixedSizeArray(SimTypeChar(label="Byte"), 16)}, name="FWP_BYTE_ARRAY16", pack=False, align=None), offset=0), "byteBlob": SimTypePointer(SimStruct({"size": SimTypeInt(signed=False, label="UInt32"), "data": SimTypePointer(SimTypeChar(label="Byte"), offset=0)}, name="FWP_BYTE_BLOB", pack=False, align=None), offset=0), "sid": SimTypePointer(SimTypeBottom(label="SID"), offset=0), "sd": SimTypePointer(SimStruct({"size": SimTypeInt(signed=False, label="UInt32"), "data": SimTypePointer(SimTypeChar(label="Byte"), offset=0)}, name="FWP_BYTE_BLOB", pack=False, align=None), offset=0), "tokenInformation": SimTypePointer(SimStruct({"sidCount": SimTypeInt(signed=False, label="UInt32"), "sids": SimTypePointer(SimTypeBottom(label="SID_AND_ATTRIBUTES"), offset=0), "restrictedSidCount": SimTypeInt(signed=False, label="UInt32"), "restrictedSids": SimTypePointer(SimTypeBottom(label="SID_AND_ATTRIBUTES"), offset=0)}, name="FWP_TOKEN_INFORMATION", pack=False, align=None), offset=0), "tokenAccessInformation": SimTypePointer(SimStruct({"size": SimTypeInt(signed=False, label="UInt32"), "data": SimTypePointer(SimTypeChar(label="Byte"), offset=0)}, name="FWP_BYTE_BLOB", pack=False, align=None), offset=0), "unicodeString": SimTypePointer(SimTypeChar(label="Char"), offset=0), "byteArray6": SimTypePointer(SimStruct({"byteArray6": SimTypeFixedSizeArray(SimTypeChar(label="Byte"), 6)}, name="FWP_BYTE_ARRAY6", pack=False, align=None), offset=0), "bitmapArray64": SimTypePointer(SimStruct({"bitmapArray64": SimTypeFixedSizeArray(SimTypeChar(label="Byte"), 8)}, name="FWP_BITMAP_ARRAY64_", pack=False, align=None), offset=0)}, name="<anon>", label="None")}, name="FWP_VALUE0", pack=False, align=None)}, name="FWP_RANGE0", pack=False, align=None), offset=0)}, name="<anon>", label="None")}, name="FWP_CONDITION_VALUE0", pack=False, align=None), "remoteSubNet": SimStruct({"type": SimTypeInt(signed=False, label="FWP_DATA_TYPE"), "Anonymous": SimUnion({"uint8": SimTypeChar(label="Byte"), "uint16": SimTypeShort(signed=False, label="UInt16"), "uint32": SimTypeInt(signed=False, label="UInt32"), "uint64": SimTypePointer(SimTypeLongLong(signed=False, label="UInt64"), offset=0), "int8": SimTypeChar(label="SByte"), "int16": SimTypeShort(signed=True, label="Int16"), "int32": SimTypeInt(signed=True, label="Int32"), "int64": SimTypePointer(SimTypeLongLong(signed=True, label="Int64"), offset=0), "float32": SimTypeFloat(size=32), "double64": SimTypePointer(SimTypeFloat(size=64), offset=0), "byteArray16": SimTypePointer(SimStruct({"byteArray16": SimTypeFixedSizeArray(SimTypeChar(label="Byte"), 16)}, name="FWP_BYTE_ARRAY16", pack=False, align=None), offset=0), "byteBlob": SimTypePointer(SimStruct({"size": SimTypeInt(signed=False, label="UInt32"), "data": SimTypePointer(SimTypeChar(label="Byte"), offset=0)}, name="FWP_BYTE_BLOB", pack=False, align=None), offset=0), "sid": SimTypePointer(SimTypeBottom(label="SID"), offset=0), "sd": SimTypePointer(SimStruct({"size": SimTypeInt(signed=False, label="UInt32"), "data": SimTypePointer(SimTypeChar(label="Byte"), offset=0)}, name="FWP_BYTE_BLOB", pack=False, align=None), offset=0), "tokenInformation": SimTypePointer(SimStruct({"sidCount": SimTypeInt(signed=False, label="UInt32"), "sids": SimTypePointer(SimTypeBottom(label="SID_AND_ATTRIBUTES"), offset=0), "restrictedSidCount": SimTypeInt(signed=False, label="UInt32"), "restrictedSids": SimTypePointer(SimTypeBottom(label="SID_AND_ATTRIBUTES"), offset=0)}, name="FWP_TOKEN_INFORMATION", pack=False, align=None), offset=0), "tokenAccessInformation": SimTypePointer(SimStruct({"size": SimTypeInt(signed=False, label="UInt32"), "data": SimTypePointer(SimTypeChar(label="Byte"), offset=0)}, name="FWP_BYTE_BLOB", pack=False, align=None), offset=0), "unicodeString": SimTypePointer(SimTypeChar(label="Char"), offset=0), "byteArray6": SimTypePointer(SimStruct({"byteArray6": SimTypeFixedSizeArray(SimTypeChar(label="Byte"), 6)}, name="FWP_BYTE_ARRAY6", pack=False, align=None), offset=0), "bitmapArray64": SimTypePointer(SimStruct({"bitmapArray64": SimTypeFixedSizeArray(SimTypeChar(label="Byte"), 8)}, name="FWP_BITMAP_ARRAY64_", pack=False, align=None), offset=0), "v4AddrMask": SimTypePointer(SimStruct({"addr": SimTypeInt(signed=False, label="UInt32"), "mask": SimTypeInt(signed=False, label="UInt32")}, name="FWP_V4_ADDR_AND_MASK", pack=False, align=None), offset=0), "v6AddrMask": SimTypePointer(SimStruct({"addr": SimTypeFixedSizeArray(SimTypeChar(label="Byte"), 16), "prefixLength": SimTypeChar(label="Byte")}, name="FWP_V6_ADDR_AND_MASK", pack=False, align=None), offset=0), "rangeValue": SimTypePointer(SimStruct({"valueLow": SimStruct({"type": SimTypeInt(signed=False, label="FWP_DATA_TYPE"), "Anonymous": SimUnion({"uint8": SimTypeChar(label="Byte"), "uint16": SimTypeShort(signed=False, label="UInt16"), "uint32": SimTypeInt(signed=False, label="UInt32"), "uint64": SimTypePointer(SimTypeLongLong(signed=False, label="UInt64"), offset=0), "int8": SimTypeChar(label="SByte"), "int16": SimTypeShort(signed=True, label="Int16"), "int32": SimTypeInt(signed=True, label="Int32"), "int64": SimTypePointer(SimTypeLongLong(signed=True, label="Int64"), offset=0), "float32": SimTypeFloat(size=32), "double64": SimTypePointer(SimTypeFloat(size=64), offset=0), "byteArray16": SimTypePointer(SimStruct({"byteArray16": SimTypeFixedSizeArray(SimTypeChar(label="Byte"), 16)}, name="FWP_BYTE_ARRAY16", pack=False, align=None), offset=0), "byteBlob": SimTypePointer(SimStruct({"size": SimTypeInt(signed=False, label="UInt32"), "data": SimTypePointer(SimTypeChar(label="Byte"), offset=0)}, name="FWP_BYTE_BLOB", pack=False, align=None), offset=0), "sid": SimTypePointer(SimTypeBottom(label="SID"), offset=0), "sd": SimTypePointer(SimStruct({"size": SimTypeInt(signed=False, label="UInt32"), "data": SimTypePointer(SimTypeChar(label="Byte"), offset=0)}, name="FWP_BYTE_BLOB", pack=False, align=None), offset=0), "tokenInformation": SimTypePointer(SimStruct({"sidCount": SimTypeInt(signed=False, label="UInt32"), "sids": SimTypePointer(SimTypeBottom(label="SID_AND_ATTRIBUTES"), offset=0), "restrictedSidCount": SimTypeInt(signed=False, label="UInt32"), "restrictedSids": SimTypePointer(SimTypeBottom(label="SID_AND_ATTRIBUTES"), offset=0)}, name="FWP_TOKEN_INFORMATION", pack=False, align=None), offset=0), "tokenAccessInformation": SimTypePointer(SimStruct({"size": SimTypeInt(signed=False, label="UInt32"), "data": SimTypePointer(SimTypeChar(label="Byte"), offset=0)}, name="FWP_BYTE_BLOB", pack=False, align=None), offset=0), "unicodeString": SimTypePointer(SimTypeChar(label="Char"), offset=0), "byteArray6": SimTypePointer(SimStruct({"byteArray6": SimTypeFixedSizeArray(SimTypeChar(label="Byte"), 6)}, name="FWP_BYTE_ARRAY6", pack=False, align=None), offset=0), "bitmapArray64": SimTypePointer(SimStruct({"bitmapArray64": SimTypeFixedSizeArray(SimTypeChar(label="Byte"), 8)}, name="FWP_BITMAP_ARRAY64_", pack=False, align=None), offset=0)}, name="<anon>", label="None")}, name="FWP_VALUE0", pack=False, align=None), "valueHigh": SimStruct({"type": SimTypeInt(signed=False, label="FWP_DATA_TYPE"), "Anonymous": SimUnion({"uint8": SimTypeChar(label="Byte"), "uint16": SimTypeShort(signed=False, label="UInt16"), "uint32": SimTypeInt(signed=False, label="UInt32"), "uint64": SimTypePointer(SimTypeLongLong(signed=False, label="UInt64"), offset=0), "int8": SimTypeChar(label="SByte"), "int16": SimTypeShort(signed=True, label="Int16"), "int32": SimTypeInt(signed=True, label="Int32"), "int64": SimTypePointer(SimTypeLongLong(signed=True, label="Int64"), offset=0), "float32": SimTypeFloat(size=32), "double64": SimTypePointer(SimTypeFloat(size=64), offset=0), "byteArray16": SimTypePointer(SimStruct({"byteArray16": SimTypeFixedSizeArray(SimTypeChar(label="Byte"), 16)}, name="FWP_BYTE_ARRAY16", pack=False, align=None), offset=0), "byteBlob": SimTypePointer(SimStruct({"size": SimTypeInt(signed=False, label="UInt32"), "data": SimTypePointer(SimTypeChar(label="Byte"), offset=0)}, name="FWP_BYTE_BLOB", pack=False, align=None), offset=0), "sid": SimTypePointer(SimTypeBottom(label="SID"), offset=0), "sd": SimTypePointer(SimStruct({"size": SimTypeInt(signed=False, label="UInt32"), "data": SimTypePointer(SimTypeChar(label="Byte"), offset=0)}, name="FWP_BYTE_BLOB", pack=False, align=None), offset=0), "tokenInformation": SimTypePointer(SimStruct({"sidCount": SimTypeInt(signed=False, label="UInt32"), "sids": SimTypePointer(SimTypeBottom(label="SID_AND_ATTRIBUTES"), offset=0), "restrictedSidCount": SimTypeInt(signed=False, label="UInt32"), "restrictedSids": SimTypePointer(SimTypeBottom(label="SID_AND_ATTRIBUTES"), offset=0)}, name="FWP_TOKEN_INFORMATION", pack=False, align=None), offset=0), "tokenAccessInformation": SimTypePointer(SimStruct({"size": SimTypeInt(signed=False, label="UInt32"), "data": SimTypePointer(SimTypeChar(label="Byte"), offset=0)}, name="FWP_BYTE_BLOB", pack=False, align=None), offset=0), "unicodeString": SimTypePointer(SimTypeChar(label="Char"), offset=0), "byteArray6": SimTypePointer(SimStruct({"byteArray6": SimTypeFixedSizeArray(SimTypeChar(label="Byte"), 6)}, name="FWP_BYTE_ARRAY6", pack=False, align=None), offset=0), "bitmapArray64": SimTypePointer(SimStruct({"bitmapArray64": SimTypeFixedSizeArray(SimTypeChar(label="Byte"), 8)}, name="FWP_BITMAP_ARRAY64_", pack=False, align=None), offset=0)}, name="<anon>", label="None")}, name="FWP_VALUE0", pack=False, align=None)}, name="FWP_RANGE0", pack=False, align=None), offset=0)}, name="<anon>", label="None")}, name="FWP_CONDITION_VALUE0", pack=False, align=None)}, name="IPSEC_SA_CONTEXT_ENUM_TEMPLATE0", pack=False, align=None), offset=0), SimTypePointer(SimTypePointer(SimTypeInt(signed=True, label="Int"), label="IntPtr", offset=0), offset=0)], SimTypeInt(signed=False, label="UInt32"), arg_names=["engineHandle", "enumTemplate", "enumHandle"]),
        # 
        'IPsecSaContextEnum0': SimTypeFunction([SimTypePointer(SimTypeInt(signed=True, label="Int"), label="IntPtr", offset=0), SimTypePointer(SimTypeInt(signed=True, label="Int"), label="IntPtr", offset=0), SimTypeInt(signed=False, label="UInt32"), SimTypePointer(SimTypePointer(SimTypePointer(SimStruct({"saContextId": SimTypeLongLong(signed=False, label="UInt64"), "inboundSa": SimTypePointer(SimStruct({"ipVersion": SimTypeInt(signed=False, label="FWP_IP_VERSION"), "saDirection": SimTypeInt(signed=False, label="FWP_DIRECTION"), "traffic": SimStruct({"ipVersion": SimTypeInt(signed=False, label="FWP_IP_VERSION"), "Anonymous1": SimUnion({"localV4Address": SimTypeInt(signed=False, label="UInt32"), "localV6Address": SimTypeFixedSizeArray(SimTypeChar(label="Byte"), 16)}, name="<anon>", label="None"), "Anonymous2": SimUnion({"remoteV4Address": SimTypeInt(signed=False, label="UInt32"), "remoteV6Address": SimTypeFixedSizeArray(SimTypeChar(label="Byte"), 16)}, name="<anon>", label="None"), "trafficType": SimTypeInt(signed=False, label="IPSEC_TRAFFIC_TYPE"), "Anonymous3": SimUnion({"ipsecFilterId": SimTypeLongLong(signed=False, label="UInt64"), "tunnelPolicyId": SimTypeLongLong(signed=False, label="UInt64")}, name="<anon>", label="None"), "remotePort": SimTypeShort(signed=False, label="UInt16")}, name="IPSEC_TRAFFIC0", pack=False, align=None), "saBundle": SimStruct({"flags": SimTypeInt(signed=False, label="IPSEC_SA_BUNDLE_FLAGS"), "lifetime": SimStruct({"lifetimeSeconds": SimTypeInt(signed=False, label="UInt32"), "lifetimeKilobytes": SimTypeInt(signed=False, label="UInt32"), "lifetimePackets": SimTypeInt(signed=False, label="UInt32")}, name="IPSEC_SA_LIFETIME0", pack=False, align=None), "idleTimeoutSeconds": SimTypeInt(signed=False, label="UInt32"), "ndAllowClearTimeoutSeconds": SimTypeInt(signed=False, label="UInt32"), "ipsecId": SimTypePointer(SimStruct({"mmTargetName": SimTypePointer(SimTypeChar(label="Char"), offset=0), "emTargetName": SimTypePointer(SimTypeChar(label="Char"), offset=0), "numTokens": SimTypeInt(signed=False, label="UInt32"), "tokens": SimTypePointer(SimStruct({"type": SimTypeInt(signed=False, label="IPSEC_TOKEN_TYPE"), "principal": SimTypeInt(signed=False, label="IPSEC_TOKEN_PRINCIPAL"), "mode": SimTypeInt(signed=False, label="IPSEC_TOKEN_MODE"), "token": SimTypeLongLong(signed=False, label="UInt64")}, name="IPSEC_TOKEN0", pack=False, align=None), offset=0), "explicitCredentials": SimTypeLongLong(signed=False, label="UInt64"), "logonId": SimTypeLongLong(signed=False, label="UInt64")}, name="IPSEC_ID0", pack=False, align=None), offset=0), "napContext": SimTypeInt(signed=False, label="UInt32"), "qmSaId": SimTypeInt(signed=False, label="UInt32"), "numSAs": SimTypeInt(signed=False, label="UInt32"), "saList": SimTypePointer(SimStruct({"spi": SimTypeInt(signed=False, label="UInt32"), "saTransformType": SimTypeInt(signed=False, label="IPSEC_TRANSFORM_TYPE"), "Anonymous": SimUnion({"ahInformation": SimTypePointer(SimStruct({"authTransform": SimStruct({"authTransformId": SimStruct({"authType": SimTypeInt(signed=False, label="IPSEC_AUTH_TYPE"), "authConfig": SimTypeChar(label="Byte")}, name="IPSEC_AUTH_TRANSFORM_ID0", pack=False, align=None), "cryptoModuleId": SimTypePointer(SimTypeBottom(label="Guid"), offset=0)}, name="IPSEC_AUTH_TRANSFORM0", pack=False, align=None), "authKey": SimStruct({"size": SimTypeInt(signed=False, label="UInt32"), "data": SimTypePointer(SimTypeChar(label="Byte"), offset=0)}, name="FWP_BYTE_BLOB", pack=False, align=None)}, name="IPSEC_SA_AUTH_INFORMATION0", pack=False, align=None), offset=0), "espAuthInformation": SimTypePointer(SimStruct({"authTransform": SimStruct({"authTransformId": SimStruct({"authType": SimTypeInt(signed=False, label="IPSEC_AUTH_TYPE"), "authConfig": SimTypeChar(label="Byte")}, name="IPSEC_AUTH_TRANSFORM_ID0", pack=False, align=None), "cryptoModuleId": SimTypePointer(SimTypeBottom(label="Guid"), offset=0)}, name="IPSEC_AUTH_TRANSFORM0", pack=False, align=None), "authKey": SimStruct({"size": SimTypeInt(signed=False, label="UInt32"), "data": SimTypePointer(SimTypeChar(label="Byte"), offset=0)}, name="FWP_BYTE_BLOB", pack=False, align=None)}, name="IPSEC_SA_AUTH_INFORMATION0", pack=False, align=None), offset=0), "espCipherInformation": SimTypePointer(SimStruct({"cipherTransform": SimStruct({"cipherTransformId": SimStruct({"cipherType": SimTypeInt(signed=False, label="IPSEC_CIPHER_TYPE"), "cipherConfig": SimTypeChar(label="Byte")}, name="IPSEC_CIPHER_TRANSFORM_ID0", pack=False, align=None), "cryptoModuleId": SimTypePointer(SimTypeBottom(label="Guid"), offset=0)}, name="IPSEC_CIPHER_TRANSFORM0", pack=False, align=None), "cipherKey": SimStruct({"size": SimTypeInt(signed=False, label="UInt32"), "data": SimTypePointer(SimTypeChar(label="Byte"), offset=0)}, name="FWP_BYTE_BLOB", pack=False, align=None)}, name="IPSEC_SA_CIPHER_INFORMATION0", pack=False, align=None), offset=0), "espAuthAndCipherInformation": SimTypePointer(SimStruct({"saCipherInformation": SimStruct({"cipherTransform": SimStruct({"cipherTransformId": SimStruct({"cipherType": SimTypeInt(signed=False, label="IPSEC_CIPHER_TYPE"), "cipherConfig": SimTypeChar(label="Byte")}, name="IPSEC_CIPHER_TRANSFORM_ID0", pack=False, align=None), "cryptoModuleId": SimTypePointer(SimTypeBottom(label="Guid"), offset=0)}, name="IPSEC_CIPHER_TRANSFORM0", pack=False, align=None), "cipherKey": SimStruct({"size": SimTypeInt(signed=False, label="UInt32"), "data": SimTypePointer(SimTypeChar(label="Byte"), offset=0)}, name="FWP_BYTE_BLOB", pack=False, align=None)}, name="IPSEC_SA_CIPHER_INFORMATION0", pack=False, align=None), "saAuthInformation": SimStruct({"authTransform": SimStruct({"authTransformId": SimStruct({"authType": SimTypeInt(signed=False, label="IPSEC_AUTH_TYPE"), "authConfig": SimTypeChar(label="Byte")}, name="IPSEC_AUTH_TRANSFORM_ID0", pack=False, align=None), "cryptoModuleId": SimTypePointer(SimTypeBottom(label="Guid"), offset=0)}, name="IPSEC_AUTH_TRANSFORM0", pack=False, align=None), "authKey": SimStruct({"size": SimTypeInt(signed=False, label="UInt32"), "data": SimTypePointer(SimTypeChar(label="Byte"), offset=0)}, name="FWP_BYTE_BLOB", pack=False, align=None)}, name="IPSEC_SA_AUTH_INFORMATION0", pack=False, align=None)}, name="IPSEC_SA_AUTH_AND_CIPHER_INFORMATION0", pack=False, align=None), offset=0), "espAuthFwInformation": SimTypePointer(SimStruct({"authTransform": SimStruct({"authTransformId": SimStruct({"authType": SimTypeInt(signed=False, label="IPSEC_AUTH_TYPE"), "authConfig": SimTypeChar(label="Byte")}, name="IPSEC_AUTH_TRANSFORM_ID0", pack=False, align=None), "cryptoModuleId": SimTypePointer(SimTypeBottom(label="Guid"), offset=0)}, name="IPSEC_AUTH_TRANSFORM0", pack=False, align=None), "authKey": SimStruct({"size": SimTypeInt(signed=False, label="UInt32"), "data": SimTypePointer(SimTypeChar(label="Byte"), offset=0)}, name="FWP_BYTE_BLOB", pack=False, align=None)}, name="IPSEC_SA_AUTH_INFORMATION0", pack=False, align=None), offset=0)}, name="<anon>", label="None")}, name="IPSEC_SA0", pack=False, align=None), offset=0), "keyModuleState": SimTypePointer(SimStruct({"keyModuleKey": SimTypeBottom(label="Guid"), "stateBlob": SimStruct({"size": SimTypeInt(signed=False, label="UInt32"), "data": SimTypePointer(SimTypeChar(label="Byte"), offset=0)}, name="FWP_BYTE_BLOB", pack=False, align=None)}, name="IPSEC_KEYMODULE_STATE0", pack=False, align=None), offset=0), "ipVersion": SimTypeInt(signed=False, label="FWP_IP_VERSION"), "Anonymous": SimUnion({"peerV4PrivateAddress": SimTypeInt(signed=False, label="UInt32")}, name="<anon>", label="None"), "mmSaId": SimTypeLongLong(signed=False, label="UInt64"), "pfsGroup": SimTypeInt(signed=False, label="IPSEC_PFS_GROUP")}, name="IPSEC_SA_BUNDLE0", pack=False, align=None), "Anonymous": SimUnion({"udpEncapsulation": SimTypePointer(SimStruct({"localUdpEncapPort": SimTypeShort(signed=False, label="UInt16"), "remoteUdpEncapPort": SimTypeShort(signed=False, label="UInt16")}, name="IPSEC_V4_UDP_ENCAPSULATION0", pack=False, align=None), offset=0)}, name="<anon>", label="None"), "transportFilter": SimTypePointer(SimTypeBottom(label="FWPM_FILTER0"), offset=0)}, name="IPSEC_SA_DETAILS0", pack=False, align=None), offset=0), "outboundSa": SimTypePointer(SimStruct({"ipVersion": SimTypeInt(signed=False, label="FWP_IP_VERSION"), "saDirection": SimTypeInt(signed=False, label="FWP_DIRECTION"), "traffic": SimStruct({"ipVersion": SimTypeInt(signed=False, label="FWP_IP_VERSION"), "Anonymous1": SimUnion({"localV4Address": SimTypeInt(signed=False, label="UInt32"), "localV6Address": SimTypeFixedSizeArray(SimTypeChar(label="Byte"), 16)}, name="<anon>", label="None"), "Anonymous2": SimUnion({"remoteV4Address": SimTypeInt(signed=False, label="UInt32"), "remoteV6Address": SimTypeFixedSizeArray(SimTypeChar(label="Byte"), 16)}, name="<anon>", label="None"), "trafficType": SimTypeInt(signed=False, label="IPSEC_TRAFFIC_TYPE"), "Anonymous3": SimUnion({"ipsecFilterId": SimTypeLongLong(signed=False, label="UInt64"), "tunnelPolicyId": SimTypeLongLong(signed=False, label="UInt64")}, name="<anon>", label="None"), "remotePort": SimTypeShort(signed=False, label="UInt16")}, name="IPSEC_TRAFFIC0", pack=False, align=None), "saBundle": SimStruct({"flags": SimTypeInt(signed=False, label="IPSEC_SA_BUNDLE_FLAGS"), "lifetime": SimStruct({"lifetimeSeconds": SimTypeInt(signed=False, label="UInt32"), "lifetimeKilobytes": SimTypeInt(signed=False, label="UInt32"), "lifetimePackets": SimTypeInt(signed=False, label="UInt32")}, name="IPSEC_SA_LIFETIME0", pack=False, align=None), "idleTimeoutSeconds": SimTypeInt(signed=False, label="UInt32"), "ndAllowClearTimeoutSeconds": SimTypeInt(signed=False, label="UInt32"), "ipsecId": SimTypePointer(SimStruct({"mmTargetName": SimTypePointer(SimTypeChar(label="Char"), offset=0), "emTargetName": SimTypePointer(SimTypeChar(label="Char"), offset=0), "numTokens": SimTypeInt(signed=False, label="UInt32"), "tokens": SimTypePointer(SimStruct({"type": SimTypeInt(signed=False, label="IPSEC_TOKEN_TYPE"), "principal": SimTypeInt(signed=False, label="IPSEC_TOKEN_PRINCIPAL"), "mode": SimTypeInt(signed=False, label="IPSEC_TOKEN_MODE"), "token": SimTypeLongLong(signed=False, label="UInt64")}, name="IPSEC_TOKEN0", pack=False, align=None), offset=0), "explicitCredentials": SimTypeLongLong(signed=False, label="UInt64"), "logonId": SimTypeLongLong(signed=False, label="UInt64")}, name="IPSEC_ID0", pack=False, align=None), offset=0), "napContext": SimTypeInt(signed=False, label="UInt32"), "qmSaId": SimTypeInt(signed=False, label="UInt32"), "numSAs": SimTypeInt(signed=False, label="UInt32"), "saList": SimTypePointer(SimStruct({"spi": SimTypeInt(signed=False, label="UInt32"), "saTransformType": SimTypeInt(signed=False, label="IPSEC_TRANSFORM_TYPE"), "Anonymous": SimUnion({"ahInformation": SimTypePointer(SimStruct({"authTransform": SimStruct({"authTransformId": SimStruct({"authType": SimTypeInt(signed=False, label="IPSEC_AUTH_TYPE"), "authConfig": SimTypeChar(label="Byte")}, name="IPSEC_AUTH_TRANSFORM_ID0", pack=False, align=None), "cryptoModuleId": SimTypePointer(SimTypeBottom(label="Guid"), offset=0)}, name="IPSEC_AUTH_TRANSFORM0", pack=False, align=None), "authKey": SimStruct({"size": SimTypeInt(signed=False, label="UInt32"), "data": SimTypePointer(SimTypeChar(label="Byte"), offset=0)}, name="FWP_BYTE_BLOB", pack=False, align=None)}, name="IPSEC_SA_AUTH_INFORMATION0", pack=False, align=None), offset=0), "espAuthInformation": SimTypePointer(SimStruct({"authTransform": SimStruct({"authTransformId": SimStruct({"authType": SimTypeInt(signed=False, label="IPSEC_AUTH_TYPE"), "authConfig": SimTypeChar(label="Byte")}, name="IPSEC_AUTH_TRANSFORM_ID0", pack=False, align=None), "cryptoModuleId": SimTypePointer(SimTypeBottom(label="Guid"), offset=0)}, name="IPSEC_AUTH_TRANSFORM0", pack=False, align=None), "authKey": SimStruct({"size": SimTypeInt(signed=False, label="UInt32"), "data": SimTypePointer(SimTypeChar(label="Byte"), offset=0)}, name="FWP_BYTE_BLOB", pack=False, align=None)}, name="IPSEC_SA_AUTH_INFORMATION0", pack=False, align=None), offset=0), "espCipherInformation": SimTypePointer(SimStruct({"cipherTransform": SimStruct({"cipherTransformId": SimStruct({"cipherType": SimTypeInt(signed=False, label="IPSEC_CIPHER_TYPE"), "cipherConfig": SimTypeChar(label="Byte")}, name="IPSEC_CIPHER_TRANSFORM_ID0", pack=False, align=None), "cryptoModuleId": SimTypePointer(SimTypeBottom(label="Guid"), offset=0)}, name="IPSEC_CIPHER_TRANSFORM0", pack=False, align=None), "cipherKey": SimStruct({"size": SimTypeInt(signed=False, label="UInt32"), "data": SimTypePointer(SimTypeChar(label="Byte"), offset=0)}, name="FWP_BYTE_BLOB", pack=False, align=None)}, name="IPSEC_SA_CIPHER_INFORMATION0", pack=False, align=None), offset=0), "espAuthAndCipherInformation": SimTypePointer(SimStruct({"saCipherInformation": SimStruct({"cipherTransform": SimStruct({"cipherTransformId": SimStruct({"cipherType": SimTypeInt(signed=False, label="IPSEC_CIPHER_TYPE"), "cipherConfig": SimTypeChar(label="Byte")}, name="IPSEC_CIPHER_TRANSFORM_ID0", pack=False, align=None), "cryptoModuleId": SimTypePointer(SimTypeBottom(label="Guid"), offset=0)}, name="IPSEC_CIPHER_TRANSFORM0", pack=False, align=None), "cipherKey": SimStruct({"size": SimTypeInt(signed=False, label="UInt32"), "data": SimTypePointer(SimTypeChar(label="Byte"), offset=0)}, name="FWP_BYTE_BLOB", pack=False, align=None)}, name="IPSEC_SA_CIPHER_INFORMATION0", pack=False, align=None), "saAuthInformation": SimStruct({"authTransform": SimStruct({"authTransformId": SimStruct({"authType": SimTypeInt(signed=False, label="IPSEC_AUTH_TYPE"), "authConfig": SimTypeChar(label="Byte")}, name="IPSEC_AUTH_TRANSFORM_ID0", pack=False, align=None), "cryptoModuleId": SimTypePointer(SimTypeBottom(label="Guid"), offset=0)}, name="IPSEC_AUTH_TRANSFORM0", pack=False, align=None), "authKey": SimStruct({"size": SimTypeInt(signed=False, label="UInt32"), "data": SimTypePointer(SimTypeChar(label="Byte"), offset=0)}, name="FWP_BYTE_BLOB", pack=False, align=None)}, name="IPSEC_SA_AUTH_INFORMATION0", pack=False, align=None)}, name="IPSEC_SA_AUTH_AND_CIPHER_INFORMATION0", pack=False, align=None), offset=0), "espAuthFwInformation": SimTypePointer(SimStruct({"authTransform": SimStruct({"authTransformId": SimStruct({"authType": SimTypeInt(signed=False, label="IPSEC_AUTH_TYPE"), "authConfig": SimTypeChar(label="Byte")}, name="IPSEC_AUTH_TRANSFORM_ID0", pack=False, align=None), "cryptoModuleId": SimTypePointer(SimTypeBottom(label="Guid"), offset=0)}, name="IPSEC_AUTH_TRANSFORM0", pack=False, align=None), "authKey": SimStruct({"size": SimTypeInt(signed=False, label="UInt32"), "data": SimTypePointer(SimTypeChar(label="Byte"), offset=0)}, name="FWP_BYTE_BLOB", pack=False, align=None)}, name="IPSEC_SA_AUTH_INFORMATION0", pack=False, align=None), offset=0)}, name="<anon>", label="None")}, name="IPSEC_SA0", pack=False, align=None), offset=0), "keyModuleState": SimTypePointer(SimStruct({"keyModuleKey": SimTypeBottom(label="Guid"), "stateBlob": SimStruct({"size": SimTypeInt(signed=False, label="UInt32"), "data": SimTypePointer(SimTypeChar(label="Byte"), offset=0)}, name="FWP_BYTE_BLOB", pack=False, align=None)}, name="IPSEC_KEYMODULE_STATE0", pack=False, align=None), offset=0), "ipVersion": SimTypeInt(signed=False, label="FWP_IP_VERSION"), "Anonymous": SimUnion({"peerV4PrivateAddress": SimTypeInt(signed=False, label="UInt32")}, name="<anon>", label="None"), "mmSaId": SimTypeLongLong(signed=False, label="UInt64"), "pfsGroup": SimTypeInt(signed=False, label="IPSEC_PFS_GROUP")}, name="IPSEC_SA_BUNDLE0", pack=False, align=None), "Anonymous": SimUnion({"udpEncapsulation": SimTypePointer(SimStruct({"localUdpEncapPort": SimTypeShort(signed=False, label="UInt16"), "remoteUdpEncapPort": SimTypeShort(signed=False, label="UInt16")}, name="IPSEC_V4_UDP_ENCAPSULATION0", pack=False, align=None), offset=0)}, name="<anon>", label="None"), "transportFilter": SimTypePointer(SimTypeBottom(label="FWPM_FILTER0"), offset=0)}, name="IPSEC_SA_DETAILS0", pack=False, align=None), offset=0)}, name="IPSEC_SA_CONTEXT0", pack=False, align=None), offset=0), offset=0), offset=0), SimTypePointer(SimTypeInt(signed=False, label="UInt32"), offset=0)], SimTypeInt(signed=False, label="UInt32"), arg_names=["engineHandle", "enumHandle", "numEntriesRequested", "entries", "numEntriesReturned"]),
        # 
        'IPsecSaContextEnum1': SimTypeFunction([SimTypePointer(SimTypeInt(signed=True, label="Int"), label="IntPtr", offset=0), SimTypePointer(SimTypeInt(signed=True, label="Int"), label="IntPtr", offset=0), SimTypeInt(signed=False, label="UInt32"), SimTypePointer(SimTypePointer(SimTypePointer(SimStruct({"saContextId": SimTypeLongLong(signed=False, label="UInt64"), "inboundSa": SimTypePointer(SimStruct({"ipVersion": SimTypeInt(signed=False, label="FWP_IP_VERSION"), "saDirection": SimTypeInt(signed=False, label="FWP_DIRECTION"), "traffic": SimStruct({"ipVersion": SimTypeInt(signed=False, label="FWP_IP_VERSION"), "Anonymous1": SimUnion({"localV4Address": SimTypeInt(signed=False, label="UInt32"), "localV6Address": SimTypeFixedSizeArray(SimTypeChar(label="Byte"), 16)}, name="<anon>", label="None"), "Anonymous2": SimUnion({"remoteV4Address": SimTypeInt(signed=False, label="UInt32"), "remoteV6Address": SimTypeFixedSizeArray(SimTypeChar(label="Byte"), 16)}, name="<anon>", label="None"), "trafficType": SimTypeInt(signed=False, label="IPSEC_TRAFFIC_TYPE"), "Anonymous3": SimUnion({"ipsecFilterId": SimTypeLongLong(signed=False, label="UInt64"), "tunnelPolicyId": SimTypeLongLong(signed=False, label="UInt64")}, name="<anon>", label="None"), "remotePort": SimTypeShort(signed=False, label="UInt16"), "localPort": SimTypeShort(signed=False, label="UInt16"), "ipProtocol": SimTypeChar(label="Byte"), "localIfLuid": SimTypeLongLong(signed=False, label="UInt64"), "realIfProfileId": SimTypeInt(signed=False, label="UInt32")}, name="IPSEC_TRAFFIC1", pack=False, align=None), "saBundle": SimStruct({"flags": SimTypeInt(signed=False, label="IPSEC_SA_BUNDLE_FLAGS"), "lifetime": SimStruct({"lifetimeSeconds": SimTypeInt(signed=False, label="UInt32"), "lifetimeKilobytes": SimTypeInt(signed=False, label="UInt32"), "lifetimePackets": SimTypeInt(signed=False, label="UInt32")}, name="IPSEC_SA_LIFETIME0", pack=False, align=None), "idleTimeoutSeconds": SimTypeInt(signed=False, label="UInt32"), "ndAllowClearTimeoutSeconds": SimTypeInt(signed=False, label="UInt32"), "ipsecId": SimTypePointer(SimStruct({"mmTargetName": SimTypePointer(SimTypeChar(label="Char"), offset=0), "emTargetName": SimTypePointer(SimTypeChar(label="Char"), offset=0), "numTokens": SimTypeInt(signed=False, label="UInt32"), "tokens": SimTypePointer(SimStruct({"type": SimTypeInt(signed=False, label="IPSEC_TOKEN_TYPE"), "principal": SimTypeInt(signed=False, label="IPSEC_TOKEN_PRINCIPAL"), "mode": SimTypeInt(signed=False, label="IPSEC_TOKEN_MODE"), "token": SimTypeLongLong(signed=False, label="UInt64")}, name="IPSEC_TOKEN0", pack=False, align=None), offset=0), "explicitCredentials": SimTypeLongLong(signed=False, label="UInt64"), "logonId": SimTypeLongLong(signed=False, label="UInt64")}, name="IPSEC_ID0", pack=False, align=None), offset=0), "napContext": SimTypeInt(signed=False, label="UInt32"), "qmSaId": SimTypeInt(signed=False, label="UInt32"), "numSAs": SimTypeInt(signed=False, label="UInt32"), "saList": SimTypePointer(SimStruct({"spi": SimTypeInt(signed=False, label="UInt32"), "saTransformType": SimTypeInt(signed=False, label="IPSEC_TRANSFORM_TYPE"), "Anonymous": SimUnion({"ahInformation": SimTypePointer(SimStruct({"authTransform": SimStruct({"authTransformId": SimStruct({"authType": SimTypeInt(signed=False, label="IPSEC_AUTH_TYPE"), "authConfig": SimTypeChar(label="Byte")}, name="IPSEC_AUTH_TRANSFORM_ID0", pack=False, align=None), "cryptoModuleId": SimTypePointer(SimTypeBottom(label="Guid"), offset=0)}, name="IPSEC_AUTH_TRANSFORM0", pack=False, align=None), "authKey": SimStruct({"size": SimTypeInt(signed=False, label="UInt32"), "data": SimTypePointer(SimTypeChar(label="Byte"), offset=0)}, name="FWP_BYTE_BLOB", pack=False, align=None)}, name="IPSEC_SA_AUTH_INFORMATION0", pack=False, align=None), offset=0), "espAuthInformation": SimTypePointer(SimStruct({"authTransform": SimStruct({"authTransformId": SimStruct({"authType": SimTypeInt(signed=False, label="IPSEC_AUTH_TYPE"), "authConfig": SimTypeChar(label="Byte")}, name="IPSEC_AUTH_TRANSFORM_ID0", pack=False, align=None), "cryptoModuleId": SimTypePointer(SimTypeBottom(label="Guid"), offset=0)}, name="IPSEC_AUTH_TRANSFORM0", pack=False, align=None), "authKey": SimStruct({"size": SimTypeInt(signed=False, label="UInt32"), "data": SimTypePointer(SimTypeChar(label="Byte"), offset=0)}, name="FWP_BYTE_BLOB", pack=False, align=None)}, name="IPSEC_SA_AUTH_INFORMATION0", pack=False, align=None), offset=0), "espCipherInformation": SimTypePointer(SimStruct({"cipherTransform": SimStruct({"cipherTransformId": SimStruct({"cipherType": SimTypeInt(signed=False, label="IPSEC_CIPHER_TYPE"), "cipherConfig": SimTypeChar(label="Byte")}, name="IPSEC_CIPHER_TRANSFORM_ID0", pack=False, align=None), "cryptoModuleId": SimTypePointer(SimTypeBottom(label="Guid"), offset=0)}, name="IPSEC_CIPHER_TRANSFORM0", pack=False, align=None), "cipherKey": SimStruct({"size": SimTypeInt(signed=False, label="UInt32"), "data": SimTypePointer(SimTypeChar(label="Byte"), offset=0)}, name="FWP_BYTE_BLOB", pack=False, align=None)}, name="IPSEC_SA_CIPHER_INFORMATION0", pack=False, align=None), offset=0), "espAuthAndCipherInformation": SimTypePointer(SimStruct({"saCipherInformation": SimStruct({"cipherTransform": SimStruct({"cipherTransformId": SimStruct({"cipherType": SimTypeInt(signed=False, label="IPSEC_CIPHER_TYPE"), "cipherConfig": SimTypeChar(label="Byte")}, name="IPSEC_CIPHER_TRANSFORM_ID0", pack=False, align=None), "cryptoModuleId": SimTypePointer(SimTypeBottom(label="Guid"), offset=0)}, name="IPSEC_CIPHER_TRANSFORM0", pack=False, align=None), "cipherKey": SimStruct({"size": SimTypeInt(signed=False, label="UInt32"), "data": SimTypePointer(SimTypeChar(label="Byte"), offset=0)}, name="FWP_BYTE_BLOB", pack=False, align=None)}, name="IPSEC_SA_CIPHER_INFORMATION0", pack=False, align=None), "saAuthInformation": SimStruct({"authTransform": SimStruct({"authTransformId": SimStruct({"authType": SimTypeInt(signed=False, label="IPSEC_AUTH_TYPE"), "authConfig": SimTypeChar(label="Byte")}, name="IPSEC_AUTH_TRANSFORM_ID0", pack=False, align=None), "cryptoModuleId": SimTypePointer(SimTypeBottom(label="Guid"), offset=0)}, name="IPSEC_AUTH_TRANSFORM0", pack=False, align=None), "authKey": SimStruct({"size": SimTypeInt(signed=False, label="UInt32"), "data": SimTypePointer(SimTypeChar(label="Byte"), offset=0)}, name="FWP_BYTE_BLOB", pack=False, align=None)}, name="IPSEC_SA_AUTH_INFORMATION0", pack=False, align=None)}, name="IPSEC_SA_AUTH_AND_CIPHER_INFORMATION0", pack=False, align=None), offset=0), "espAuthFwInformation": SimTypePointer(SimStruct({"authTransform": SimStruct({"authTransformId": SimStruct({"authType": SimTypeInt(signed=False, label="IPSEC_AUTH_TYPE"), "authConfig": SimTypeChar(label="Byte")}, name="IPSEC_AUTH_TRANSFORM_ID0", pack=False, align=None), "cryptoModuleId": SimTypePointer(SimTypeBottom(label="Guid"), offset=0)}, name="IPSEC_AUTH_TRANSFORM0", pack=False, align=None), "authKey": SimStruct({"size": SimTypeInt(signed=False, label="UInt32"), "data": SimTypePointer(SimTypeChar(label="Byte"), offset=0)}, name="FWP_BYTE_BLOB", pack=False, align=None)}, name="IPSEC_SA_AUTH_INFORMATION0", pack=False, align=None), offset=0)}, name="<anon>", label="None")}, name="IPSEC_SA0", pack=False, align=None), offset=0), "keyModuleState": SimTypePointer(SimStruct({"keyModuleKey": SimTypeBottom(label="Guid"), "stateBlob": SimStruct({"size": SimTypeInt(signed=False, label="UInt32"), "data": SimTypePointer(SimTypeChar(label="Byte"), offset=0)}, name="FWP_BYTE_BLOB", pack=False, align=None)}, name="IPSEC_KEYMODULE_STATE0", pack=False, align=None), offset=0), "ipVersion": SimTypeInt(signed=False, label="FWP_IP_VERSION"), "Anonymous": SimUnion({"peerV4PrivateAddress": SimTypeInt(signed=False, label="UInt32")}, name="<anon>", label="None"), "mmSaId": SimTypeLongLong(signed=False, label="UInt64"), "pfsGroup": SimTypeInt(signed=False, label="IPSEC_PFS_GROUP"), "saLookupContext": SimTypeBottom(label="Guid"), "qmFilterId": SimTypeLongLong(signed=False, label="UInt64")}, name="IPSEC_SA_BUNDLE1", pack=False, align=None), "Anonymous": SimUnion({"udpEncapsulation": SimTypePointer(SimStruct({"localUdpEncapPort": SimTypeShort(signed=False, label="UInt16"), "remoteUdpEncapPort": SimTypeShort(signed=False, label="UInt16")}, name="IPSEC_V4_UDP_ENCAPSULATION0", pack=False, align=None), offset=0)}, name="<anon>", label="None"), "transportFilter": SimTypePointer(SimTypeBottom(label="FWPM_FILTER0"), offset=0), "virtualIfTunnelInfo": SimStruct({"virtualIfTunnelId": SimTypeLongLong(signed=False, label="UInt64"), "trafficSelectorId": SimTypeLongLong(signed=False, label="UInt64")}, name="IPSEC_VIRTUAL_IF_TUNNEL_INFO0", pack=False, align=None)}, name="IPSEC_SA_DETAILS1", pack=False, align=None), offset=0), "outboundSa": SimTypePointer(SimStruct({"ipVersion": SimTypeInt(signed=False, label="FWP_IP_VERSION"), "saDirection": SimTypeInt(signed=False, label="FWP_DIRECTION"), "traffic": SimStruct({"ipVersion": SimTypeInt(signed=False, label="FWP_IP_VERSION"), "Anonymous1": SimUnion({"localV4Address": SimTypeInt(signed=False, label="UInt32"), "localV6Address": SimTypeFixedSizeArray(SimTypeChar(label="Byte"), 16)}, name="<anon>", label="None"), "Anonymous2": SimUnion({"remoteV4Address": SimTypeInt(signed=False, label="UInt32"), "remoteV6Address": SimTypeFixedSizeArray(SimTypeChar(label="Byte"), 16)}, name="<anon>", label="None"), "trafficType": SimTypeInt(signed=False, label="IPSEC_TRAFFIC_TYPE"), "Anonymous3": SimUnion({"ipsecFilterId": SimTypeLongLong(signed=False, label="UInt64"), "tunnelPolicyId": SimTypeLongLong(signed=False, label="UInt64")}, name="<anon>", label="None"), "remotePort": SimTypeShort(signed=False, label="UInt16"), "localPort": SimTypeShort(signed=False, label="UInt16"), "ipProtocol": SimTypeChar(label="Byte"), "localIfLuid": SimTypeLongLong(signed=False, label="UInt64"), "realIfProfileId": SimTypeInt(signed=False, label="UInt32")}, name="IPSEC_TRAFFIC1", pack=False, align=None), "saBundle": SimStruct({"flags": SimTypeInt(signed=False, label="IPSEC_SA_BUNDLE_FLAGS"), "lifetime": SimStruct({"lifetimeSeconds": SimTypeInt(signed=False, label="UInt32"), "lifetimeKilobytes": SimTypeInt(signed=False, label="UInt32"), "lifetimePackets": SimTypeInt(signed=False, label="UInt32")}, name="IPSEC_SA_LIFETIME0", pack=False, align=None), "idleTimeoutSeconds": SimTypeInt(signed=False, label="UInt32"), "ndAllowClearTimeoutSeconds": SimTypeInt(signed=False, label="UInt32"), "ipsecId": SimTypePointer(SimStruct({"mmTargetName": SimTypePointer(SimTypeChar(label="Char"), offset=0), "emTargetName": SimTypePointer(SimTypeChar(label="Char"), offset=0), "numTokens": SimTypeInt(signed=False, label="UInt32"), "tokens": SimTypePointer(SimStruct({"type": SimTypeInt(signed=False, label="IPSEC_TOKEN_TYPE"), "principal": SimTypeInt(signed=False, label="IPSEC_TOKEN_PRINCIPAL"), "mode": SimTypeInt(signed=False, label="IPSEC_TOKEN_MODE"), "token": SimTypeLongLong(signed=False, label="UInt64")}, name="IPSEC_TOKEN0", pack=False, align=None), offset=0), "explicitCredentials": SimTypeLongLong(signed=False, label="UInt64"), "logonId": SimTypeLongLong(signed=False, label="UInt64")}, name="IPSEC_ID0", pack=False, align=None), offset=0), "napContext": SimTypeInt(signed=False, label="UInt32"), "qmSaId": SimTypeInt(signed=False, label="UInt32"), "numSAs": SimTypeInt(signed=False, label="UInt32"), "saList": SimTypePointer(SimStruct({"spi": SimTypeInt(signed=False, label="UInt32"), "saTransformType": SimTypeInt(signed=False, label="IPSEC_TRANSFORM_TYPE"), "Anonymous": SimUnion({"ahInformation": SimTypePointer(SimStruct({"authTransform": SimStruct({"authTransformId": SimStruct({"authType": SimTypeInt(signed=False, label="IPSEC_AUTH_TYPE"), "authConfig": SimTypeChar(label="Byte")}, name="IPSEC_AUTH_TRANSFORM_ID0", pack=False, align=None), "cryptoModuleId": SimTypePointer(SimTypeBottom(label="Guid"), offset=0)}, name="IPSEC_AUTH_TRANSFORM0", pack=False, align=None), "authKey": SimStruct({"size": SimTypeInt(signed=False, label="UInt32"), "data": SimTypePointer(SimTypeChar(label="Byte"), offset=0)}, name="FWP_BYTE_BLOB", pack=False, align=None)}, name="IPSEC_SA_AUTH_INFORMATION0", pack=False, align=None), offset=0), "espAuthInformation": SimTypePointer(SimStruct({"authTransform": SimStruct({"authTransformId": SimStruct({"authType": SimTypeInt(signed=False, label="IPSEC_AUTH_TYPE"), "authConfig": SimTypeChar(label="Byte")}, name="IPSEC_AUTH_TRANSFORM_ID0", pack=False, align=None), "cryptoModuleId": SimTypePointer(SimTypeBottom(label="Guid"), offset=0)}, name="IPSEC_AUTH_TRANSFORM0", pack=False, align=None), "authKey": SimStruct({"size": SimTypeInt(signed=False, label="UInt32"), "data": SimTypePointer(SimTypeChar(label="Byte"), offset=0)}, name="FWP_BYTE_BLOB", pack=False, align=None)}, name="IPSEC_SA_AUTH_INFORMATION0", pack=False, align=None), offset=0), "espCipherInformation": SimTypePointer(SimStruct({"cipherTransform": SimStruct({"cipherTransformId": SimStruct({"cipherType": SimTypeInt(signed=False, label="IPSEC_CIPHER_TYPE"), "cipherConfig": SimTypeChar(label="Byte")}, name="IPSEC_CIPHER_TRANSFORM_ID0", pack=False, align=None), "cryptoModuleId": SimTypePointer(SimTypeBottom(label="Guid"), offset=0)}, name="IPSEC_CIPHER_TRANSFORM0", pack=False, align=None), "cipherKey": SimStruct({"size": SimTypeInt(signed=False, label="UInt32"), "data": SimTypePointer(SimTypeChar(label="Byte"), offset=0)}, name="FWP_BYTE_BLOB", pack=False, align=None)}, name="IPSEC_SA_CIPHER_INFORMATION0", pack=False, align=None), offset=0), "espAuthAndCipherInformation": SimTypePointer(SimStruct({"saCipherInformation": SimStruct({"cipherTransform": SimStruct({"cipherTransformId": SimStruct({"cipherType": SimTypeInt(signed=False, label="IPSEC_CIPHER_TYPE"), "cipherConfig": SimTypeChar(label="Byte")}, name="IPSEC_CIPHER_TRANSFORM_ID0", pack=False, align=None), "cryptoModuleId": SimTypePointer(SimTypeBottom(label="Guid"), offset=0)}, name="IPSEC_CIPHER_TRANSFORM0", pack=False, align=None), "cipherKey": SimStruct({"size": SimTypeInt(signed=False, label="UInt32"), "data": SimTypePointer(SimTypeChar(label="Byte"), offset=0)}, name="FWP_BYTE_BLOB", pack=False, align=None)}, name="IPSEC_SA_CIPHER_INFORMATION0", pack=False, align=None), "saAuthInformation": SimStruct({"authTransform": SimStruct({"authTransformId": SimStruct({"authType": SimTypeInt(signed=False, label="IPSEC_AUTH_TYPE"), "authConfig": SimTypeChar(label="Byte")}, name="IPSEC_AUTH_TRANSFORM_ID0", pack=False, align=None), "cryptoModuleId": SimTypePointer(SimTypeBottom(label="Guid"), offset=0)}, name="IPSEC_AUTH_TRANSFORM0", pack=False, align=None), "authKey": SimStruct({"size": SimTypeInt(signed=False, label="UInt32"), "data": SimTypePointer(SimTypeChar(label="Byte"), offset=0)}, name="FWP_BYTE_BLOB", pack=False, align=None)}, name="IPSEC_SA_AUTH_INFORMATION0", pack=False, align=None)}, name="IPSEC_SA_AUTH_AND_CIPHER_INFORMATION0", pack=False, align=None), offset=0), "espAuthFwInformation": SimTypePointer(SimStruct({"authTransform": SimStruct({"authTransformId": SimStruct({"authType": SimTypeInt(signed=False, label="IPSEC_AUTH_TYPE"), "authConfig": SimTypeChar(label="Byte")}, name="IPSEC_AUTH_TRANSFORM_ID0", pack=False, align=None), "cryptoModuleId": SimTypePointer(SimTypeBottom(label="Guid"), offset=0)}, name="IPSEC_AUTH_TRANSFORM0", pack=False, align=None), "authKey": SimStruct({"size": SimTypeInt(signed=False, label="UInt32"), "data": SimTypePointer(SimTypeChar(label="Byte"), offset=0)}, name="FWP_BYTE_BLOB", pack=False, align=None)}, name="IPSEC_SA_AUTH_INFORMATION0", pack=False, align=None), offset=0)}, name="<anon>", label="None")}, name="IPSEC_SA0", pack=False, align=None), offset=0), "keyModuleState": SimTypePointer(SimStruct({"keyModuleKey": SimTypeBottom(label="Guid"), "stateBlob": SimStruct({"size": SimTypeInt(signed=False, label="UInt32"), "data": SimTypePointer(SimTypeChar(label="Byte"), offset=0)}, name="FWP_BYTE_BLOB", pack=False, align=None)}, name="IPSEC_KEYMODULE_STATE0", pack=False, align=None), offset=0), "ipVersion": SimTypeInt(signed=False, label="FWP_IP_VERSION"), "Anonymous": SimUnion({"peerV4PrivateAddress": SimTypeInt(signed=False, label="UInt32")}, name="<anon>", label="None"), "mmSaId": SimTypeLongLong(signed=False, label="UInt64"), "pfsGroup": SimTypeInt(signed=False, label="IPSEC_PFS_GROUP"), "saLookupContext": SimTypeBottom(label="Guid"), "qmFilterId": SimTypeLongLong(signed=False, label="UInt64")}, name="IPSEC_SA_BUNDLE1", pack=False, align=None), "Anonymous": SimUnion({"udpEncapsulation": SimTypePointer(SimStruct({"localUdpEncapPort": SimTypeShort(signed=False, label="UInt16"), "remoteUdpEncapPort": SimTypeShort(signed=False, label="UInt16")}, name="IPSEC_V4_UDP_ENCAPSULATION0", pack=False, align=None), offset=0)}, name="<anon>", label="None"), "transportFilter": SimTypePointer(SimTypeBottom(label="FWPM_FILTER0"), offset=0), "virtualIfTunnelInfo": SimStruct({"virtualIfTunnelId": SimTypeLongLong(signed=False, label="UInt64"), "trafficSelectorId": SimTypeLongLong(signed=False, label="UInt64")}, name="IPSEC_VIRTUAL_IF_TUNNEL_INFO0", pack=False, align=None)}, name="IPSEC_SA_DETAILS1", pack=False, align=None), offset=0)}, name="IPSEC_SA_CONTEXT1", pack=False, align=None), offset=0), offset=0), offset=0), SimTypePointer(SimTypeInt(signed=False, label="UInt32"), offset=0)], SimTypeInt(signed=False, label="UInt32"), arg_names=["engineHandle", "enumHandle", "numEntriesRequested", "entries", "numEntriesReturned"]),
        # 
        'IPsecSaContextDestroyEnumHandle0': SimTypeFunction([SimTypePointer(SimTypeInt(signed=True, label="Int"), label="IntPtr", offset=0), SimTypePointer(SimTypeInt(signed=True, label="Int"), label="IntPtr", offset=0)], SimTypeInt(signed=False, label="UInt32"), arg_names=["engineHandle", "enumHandle"]),
        # 
        'IPsecSaContextSubscribe0': SimTypeFunction([SimTypePointer(SimTypeInt(signed=True, label="Int"), label="IntPtr", offset=0), SimTypePointer(SimStruct({"enumTemplate": SimTypePointer(SimStruct({"localSubNet": SimStruct({"type": SimTypeInt(signed=False, label="FWP_DATA_TYPE"), "Anonymous": SimUnion({"uint8": SimTypeChar(label="Byte"), "uint16": SimTypeShort(signed=False, label="UInt16"), "uint32": SimTypeInt(signed=False, label="UInt32"), "uint64": SimTypePointer(SimTypeLongLong(signed=False, label="UInt64"), offset=0), "int8": SimTypeChar(label="SByte"), "int16": SimTypeShort(signed=True, label="Int16"), "int32": SimTypeInt(signed=True, label="Int32"), "int64": SimTypePointer(SimTypeLongLong(signed=True, label="Int64"), offset=0), "float32": SimTypeFloat(size=32), "double64": SimTypePointer(SimTypeFloat(size=64), offset=0), "byteArray16": SimTypePointer(SimStruct({"byteArray16": SimTypeFixedSizeArray(SimTypeChar(label="Byte"), 16)}, name="FWP_BYTE_ARRAY16", pack=False, align=None), offset=0), "byteBlob": SimTypePointer(SimStruct({"size": SimTypeInt(signed=False, label="UInt32"), "data": SimTypePointer(SimTypeChar(label="Byte"), offset=0)}, name="FWP_BYTE_BLOB", pack=False, align=None), offset=0), "sid": SimTypePointer(SimTypeBottom(label="SID"), offset=0), "sd": SimTypePointer(SimStruct({"size": SimTypeInt(signed=False, label="UInt32"), "data": SimTypePointer(SimTypeChar(label="Byte"), offset=0)}, name="FWP_BYTE_BLOB", pack=False, align=None), offset=0), "tokenInformation": SimTypePointer(SimStruct({"sidCount": SimTypeInt(signed=False, label="UInt32"), "sids": SimTypePointer(SimTypeBottom(label="SID_AND_ATTRIBUTES"), offset=0), "restrictedSidCount": SimTypeInt(signed=False, label="UInt32"), "restrictedSids": SimTypePointer(SimTypeBottom(label="SID_AND_ATTRIBUTES"), offset=0)}, name="FWP_TOKEN_INFORMATION", pack=False, align=None), offset=0), "tokenAccessInformation": SimTypePointer(SimStruct({"size": SimTypeInt(signed=False, label="UInt32"), "data": SimTypePointer(SimTypeChar(label="Byte"), offset=0)}, name="FWP_BYTE_BLOB", pack=False, align=None), offset=0), "unicodeString": SimTypePointer(SimTypeChar(label="Char"), offset=0), "byteArray6": SimTypePointer(SimStruct({"byteArray6": SimTypeFixedSizeArray(SimTypeChar(label="Byte"), 6)}, name="FWP_BYTE_ARRAY6", pack=False, align=None), offset=0), "bitmapArray64": SimTypePointer(SimStruct({"bitmapArray64": SimTypeFixedSizeArray(SimTypeChar(label="Byte"), 8)}, name="FWP_BITMAP_ARRAY64_", pack=False, align=None), offset=0), "v4AddrMask": SimTypePointer(SimStruct({"addr": SimTypeInt(signed=False, label="UInt32"), "mask": SimTypeInt(signed=False, label="UInt32")}, name="FWP_V4_ADDR_AND_MASK", pack=False, align=None), offset=0), "v6AddrMask": SimTypePointer(SimStruct({"addr": SimTypeFixedSizeArray(SimTypeChar(label="Byte"), 16), "prefixLength": SimTypeChar(label="Byte")}, name="FWP_V6_ADDR_AND_MASK", pack=False, align=None), offset=0), "rangeValue": SimTypePointer(SimStruct({"valueLow": SimStruct({"type": SimTypeInt(signed=False, label="FWP_DATA_TYPE"), "Anonymous": SimUnion({"uint8": SimTypeChar(label="Byte"), "uint16": SimTypeShort(signed=False, label="UInt16"), "uint32": SimTypeInt(signed=False, label="UInt32"), "uint64": SimTypePointer(SimTypeLongLong(signed=False, label="UInt64"), offset=0), "int8": SimTypeChar(label="SByte"), "int16": SimTypeShort(signed=True, label="Int16"), "int32": SimTypeInt(signed=True, label="Int32"), "int64": SimTypePointer(SimTypeLongLong(signed=True, label="Int64"), offset=0), "float32": SimTypeFloat(size=32), "double64": SimTypePointer(SimTypeFloat(size=64), offset=0), "byteArray16": SimTypePointer(SimStruct({"byteArray16": SimTypeFixedSizeArray(SimTypeChar(label="Byte"), 16)}, name="FWP_BYTE_ARRAY16", pack=False, align=None), offset=0), "byteBlob": SimTypePointer(SimStruct({"size": SimTypeInt(signed=False, label="UInt32"), "data": SimTypePointer(SimTypeChar(label="Byte"), offset=0)}, name="FWP_BYTE_BLOB", pack=False, align=None), offset=0), "sid": SimTypePointer(SimTypeBottom(label="SID"), offset=0), "sd": SimTypePointer(SimStruct({"size": SimTypeInt(signed=False, label="UInt32"), "data": SimTypePointer(SimTypeChar(label="Byte"), offset=0)}, name="FWP_BYTE_BLOB", pack=False, align=None), offset=0), "tokenInformation": SimTypePointer(SimStruct({"sidCount": SimTypeInt(signed=False, label="UInt32"), "sids": SimTypePointer(SimTypeBottom(label="SID_AND_ATTRIBUTES"), offset=0), "restrictedSidCount": SimTypeInt(signed=False, label="UInt32"), "restrictedSids": SimTypePointer(SimTypeBottom(label="SID_AND_ATTRIBUTES"), offset=0)}, name="FWP_TOKEN_INFORMATION", pack=False, align=None), offset=0), "tokenAccessInformation": SimTypePointer(SimStruct({"size": SimTypeInt(signed=False, label="UInt32"), "data": SimTypePointer(SimTypeChar(label="Byte"), offset=0)}, name="FWP_BYTE_BLOB", pack=False, align=None), offset=0), "unicodeString": SimTypePointer(SimTypeChar(label="Char"), offset=0), "byteArray6": SimTypePointer(SimStruct({"byteArray6": SimTypeFixedSizeArray(SimTypeChar(label="Byte"), 6)}, name="FWP_BYTE_ARRAY6", pack=False, align=None), offset=0), "bitmapArray64": SimTypePointer(SimStruct({"bitmapArray64": SimTypeFixedSizeArray(SimTypeChar(label="Byte"), 8)}, name="FWP_BITMAP_ARRAY64_", pack=False, align=None), offset=0)}, name="<anon>", label="None")}, name="FWP_VALUE0", pack=False, align=None), "valueHigh": SimStruct({"type": SimTypeInt(signed=False, label="FWP_DATA_TYPE"), "Anonymous": SimUnion({"uint8": SimTypeChar(label="Byte"), "uint16": SimTypeShort(signed=False, label="UInt16"), "uint32": SimTypeInt(signed=False, label="UInt32"), "uint64": SimTypePointer(SimTypeLongLong(signed=False, label="UInt64"), offset=0), "int8": SimTypeChar(label="SByte"), "int16": SimTypeShort(signed=True, label="Int16"), "int32": SimTypeInt(signed=True, label="Int32"), "int64": SimTypePointer(SimTypeLongLong(signed=True, label="Int64"), offset=0), "float32": SimTypeFloat(size=32), "double64": SimTypePointer(SimTypeFloat(size=64), offset=0), "byteArray16": SimTypePointer(SimStruct({"byteArray16": SimTypeFixedSizeArray(SimTypeChar(label="Byte"), 16)}, name="FWP_BYTE_ARRAY16", pack=False, align=None), offset=0), "byteBlob": SimTypePointer(SimStruct({"size": SimTypeInt(signed=False, label="UInt32"), "data": SimTypePointer(SimTypeChar(label="Byte"), offset=0)}, name="FWP_BYTE_BLOB", pack=False, align=None), offset=0), "sid": SimTypePointer(SimTypeBottom(label="SID"), offset=0), "sd": SimTypePointer(SimStruct({"size": SimTypeInt(signed=False, label="UInt32"), "data": SimTypePointer(SimTypeChar(label="Byte"), offset=0)}, name="FWP_BYTE_BLOB", pack=False, align=None), offset=0), "tokenInformation": SimTypePointer(SimStruct({"sidCount": SimTypeInt(signed=False, label="UInt32"), "sids": SimTypePointer(SimTypeBottom(label="SID_AND_ATTRIBUTES"), offset=0), "restrictedSidCount": SimTypeInt(signed=False, label="UInt32"), "restrictedSids": SimTypePointer(SimTypeBottom(label="SID_AND_ATTRIBUTES"), offset=0)}, name="FWP_TOKEN_INFORMATION", pack=False, align=None), offset=0), "tokenAccessInformation": SimTypePointer(SimStruct({"size": SimTypeInt(signed=False, label="UInt32"), "data": SimTypePointer(SimTypeChar(label="Byte"), offset=0)}, name="FWP_BYTE_BLOB", pack=False, align=None), offset=0), "unicodeString": SimTypePointer(SimTypeChar(label="Char"), offset=0), "byteArray6": SimTypePointer(SimStruct({"byteArray6": SimTypeFixedSizeArray(SimTypeChar(label="Byte"), 6)}, name="FWP_BYTE_ARRAY6", pack=False, align=None), offset=0), "bitmapArray64": SimTypePointer(SimStruct({"bitmapArray64": SimTypeFixedSizeArray(SimTypeChar(label="Byte"), 8)}, name="FWP_BITMAP_ARRAY64_", pack=False, align=None), offset=0)}, name="<anon>", label="None")}, name="FWP_VALUE0", pack=False, align=None)}, name="FWP_RANGE0", pack=False, align=None), offset=0)}, name="<anon>", label="None")}, name="FWP_CONDITION_VALUE0", pack=False, align=None), "remoteSubNet": SimStruct({"type": SimTypeInt(signed=False, label="FWP_DATA_TYPE"), "Anonymous": SimUnion({"uint8": SimTypeChar(label="Byte"), "uint16": SimTypeShort(signed=False, label="UInt16"), "uint32": SimTypeInt(signed=False, label="UInt32"), "uint64": SimTypePointer(SimTypeLongLong(signed=False, label="UInt64"), offset=0), "int8": SimTypeChar(label="SByte"), "int16": SimTypeShort(signed=True, label="Int16"), "int32": SimTypeInt(signed=True, label="Int32"), "int64": SimTypePointer(SimTypeLongLong(signed=True, label="Int64"), offset=0), "float32": SimTypeFloat(size=32), "double64": SimTypePointer(SimTypeFloat(size=64), offset=0), "byteArray16": SimTypePointer(SimStruct({"byteArray16": SimTypeFixedSizeArray(SimTypeChar(label="Byte"), 16)}, name="FWP_BYTE_ARRAY16", pack=False, align=None), offset=0), "byteBlob": SimTypePointer(SimStruct({"size": SimTypeInt(signed=False, label="UInt32"), "data": SimTypePointer(SimTypeChar(label="Byte"), offset=0)}, name="FWP_BYTE_BLOB", pack=False, align=None), offset=0), "sid": SimTypePointer(SimTypeBottom(label="SID"), offset=0), "sd": SimTypePointer(SimStruct({"size": SimTypeInt(signed=False, label="UInt32"), "data": SimTypePointer(SimTypeChar(label="Byte"), offset=0)}, name="FWP_BYTE_BLOB", pack=False, align=None), offset=0), "tokenInformation": SimTypePointer(SimStruct({"sidCount": SimTypeInt(signed=False, label="UInt32"), "sids": SimTypePointer(SimTypeBottom(label="SID_AND_ATTRIBUTES"), offset=0), "restrictedSidCount": SimTypeInt(signed=False, label="UInt32"), "restrictedSids": SimTypePointer(SimTypeBottom(label="SID_AND_ATTRIBUTES"), offset=0)}, name="FWP_TOKEN_INFORMATION", pack=False, align=None), offset=0), "tokenAccessInformation": SimTypePointer(SimStruct({"size": SimTypeInt(signed=False, label="UInt32"), "data": SimTypePointer(SimTypeChar(label="Byte"), offset=0)}, name="FWP_BYTE_BLOB", pack=False, align=None), offset=0), "unicodeString": SimTypePointer(SimTypeChar(label="Char"), offset=0), "byteArray6": SimTypePointer(SimStruct({"byteArray6": SimTypeFixedSizeArray(SimTypeChar(label="Byte"), 6)}, name="FWP_BYTE_ARRAY6", pack=False, align=None), offset=0), "bitmapArray64": SimTypePointer(SimStruct({"bitmapArray64": SimTypeFixedSizeArray(SimTypeChar(label="Byte"), 8)}, name="FWP_BITMAP_ARRAY64_", pack=False, align=None), offset=0), "v4AddrMask": SimTypePointer(SimStruct({"addr": SimTypeInt(signed=False, label="UInt32"), "mask": SimTypeInt(signed=False, label="UInt32")}, name="FWP_V4_ADDR_AND_MASK", pack=False, align=None), offset=0), "v6AddrMask": SimTypePointer(SimStruct({"addr": SimTypeFixedSizeArray(SimTypeChar(label="Byte"), 16), "prefixLength": SimTypeChar(label="Byte")}, name="FWP_V6_ADDR_AND_MASK", pack=False, align=None), offset=0), "rangeValue": SimTypePointer(SimStruct({"valueLow": SimStruct({"type": SimTypeInt(signed=False, label="FWP_DATA_TYPE"), "Anonymous": SimUnion({"uint8": SimTypeChar(label="Byte"), "uint16": SimTypeShort(signed=False, label="UInt16"), "uint32": SimTypeInt(signed=False, label="UInt32"), "uint64": SimTypePointer(SimTypeLongLong(signed=False, label="UInt64"), offset=0), "int8": SimTypeChar(label="SByte"), "int16": SimTypeShort(signed=True, label="Int16"), "int32": SimTypeInt(signed=True, label="Int32"), "int64": SimTypePointer(SimTypeLongLong(signed=True, label="Int64"), offset=0), "float32": SimTypeFloat(size=32), "double64": SimTypePointer(SimTypeFloat(size=64), offset=0), "byteArray16": SimTypePointer(SimStruct({"byteArray16": SimTypeFixedSizeArray(SimTypeChar(label="Byte"), 16)}, name="FWP_BYTE_ARRAY16", pack=False, align=None), offset=0), "byteBlob": SimTypePointer(SimStruct({"size": SimTypeInt(signed=False, label="UInt32"), "data": SimTypePointer(SimTypeChar(label="Byte"), offset=0)}, name="FWP_BYTE_BLOB", pack=False, align=None), offset=0), "sid": SimTypePointer(SimTypeBottom(label="SID"), offset=0), "sd": SimTypePointer(SimStruct({"size": SimTypeInt(signed=False, label="UInt32"), "data": SimTypePointer(SimTypeChar(label="Byte"), offset=0)}, name="FWP_BYTE_BLOB", pack=False, align=None), offset=0), "tokenInformation": SimTypePointer(SimStruct({"sidCount": SimTypeInt(signed=False, label="UInt32"), "sids": SimTypePointer(SimTypeBottom(label="SID_AND_ATTRIBUTES"), offset=0), "restrictedSidCount": SimTypeInt(signed=False, label="UInt32"), "restrictedSids": SimTypePointer(SimTypeBottom(label="SID_AND_ATTRIBUTES"), offset=0)}, name="FWP_TOKEN_INFORMATION", pack=False, align=None), offset=0), "tokenAccessInformation": SimTypePointer(SimStruct({"size": SimTypeInt(signed=False, label="UInt32"), "data": SimTypePointer(SimTypeChar(label="Byte"), offset=0)}, name="FWP_BYTE_BLOB", pack=False, align=None), offset=0), "unicodeString": SimTypePointer(SimTypeChar(label="Char"), offset=0), "byteArray6": SimTypePointer(SimStruct({"byteArray6": SimTypeFixedSizeArray(SimTypeChar(label="Byte"), 6)}, name="FWP_BYTE_ARRAY6", pack=False, align=None), offset=0), "bitmapArray64": SimTypePointer(SimStruct({"bitmapArray64": SimTypeFixedSizeArray(SimTypeChar(label="Byte"), 8)}, name="FWP_BITMAP_ARRAY64_", pack=False, align=None), offset=0)}, name="<anon>", label="None")}, name="FWP_VALUE0", pack=False, align=None), "valueHigh": SimStruct({"type": SimTypeInt(signed=False, label="FWP_DATA_TYPE"), "Anonymous": SimUnion({"uint8": SimTypeChar(label="Byte"), "uint16": SimTypeShort(signed=False, label="UInt16"), "uint32": SimTypeInt(signed=False, label="UInt32"), "uint64": SimTypePointer(SimTypeLongLong(signed=False, label="UInt64"), offset=0), "int8": SimTypeChar(label="SByte"), "int16": SimTypeShort(signed=True, label="Int16"), "int32": SimTypeInt(signed=True, label="Int32"), "int64": SimTypePointer(SimTypeLongLong(signed=True, label="Int64"), offset=0), "float32": SimTypeFloat(size=32), "double64": SimTypePointer(SimTypeFloat(size=64), offset=0), "byteArray16": SimTypePointer(SimStruct({"byteArray16": SimTypeFixedSizeArray(SimTypeChar(label="Byte"), 16)}, name="FWP_BYTE_ARRAY16", pack=False, align=None), offset=0), "byteBlob": SimTypePointer(SimStruct({"size": SimTypeInt(signed=False, label="UInt32"), "data": SimTypePointer(SimTypeChar(label="Byte"), offset=0)}, name="FWP_BYTE_BLOB", pack=False, align=None), offset=0), "sid": SimTypePointer(SimTypeBottom(label="SID"), offset=0), "sd": SimTypePointer(SimStruct({"size": SimTypeInt(signed=False, label="UInt32"), "data": SimTypePointer(SimTypeChar(label="Byte"), offset=0)}, name="FWP_BYTE_BLOB", pack=False, align=None), offset=0), "tokenInformation": SimTypePointer(SimStruct({"sidCount": SimTypeInt(signed=False, label="UInt32"), "sids": SimTypePointer(SimTypeBottom(label="SID_AND_ATTRIBUTES"), offset=0), "restrictedSidCount": SimTypeInt(signed=False, label="UInt32"), "restrictedSids": SimTypePointer(SimTypeBottom(label="SID_AND_ATTRIBUTES"), offset=0)}, name="FWP_TOKEN_INFORMATION", pack=False, align=None), offset=0), "tokenAccessInformation": SimTypePointer(SimStruct({"size": SimTypeInt(signed=False, label="UInt32"), "data": SimTypePointer(SimTypeChar(label="Byte"), offset=0)}, name="FWP_BYTE_BLOB", pack=False, align=None), offset=0), "unicodeString": SimTypePointer(SimTypeChar(label="Char"), offset=0), "byteArray6": SimTypePointer(SimStruct({"byteArray6": SimTypeFixedSizeArray(SimTypeChar(label="Byte"), 6)}, name="FWP_BYTE_ARRAY6", pack=False, align=None), offset=0), "bitmapArray64": SimTypePointer(SimStruct({"bitmapArray64": SimTypeFixedSizeArray(SimTypeChar(label="Byte"), 8)}, name="FWP_BITMAP_ARRAY64_", pack=False, align=None), offset=0)}, name="<anon>", label="None")}, name="FWP_VALUE0", pack=False, align=None)}, name="FWP_RANGE0", pack=False, align=None), offset=0)}, name="<anon>", label="None")}, name="FWP_CONDITION_VALUE0", pack=False, align=None)}, name="IPSEC_SA_CONTEXT_ENUM_TEMPLATE0", pack=False, align=None), offset=0), "flags": SimTypeInt(signed=False, label="UInt32"), "sessionKey": SimTypeBottom(label="Guid")}, name="IPSEC_SA_CONTEXT_SUBSCRIPTION0", pack=False, align=None), offset=0), SimTypePointer(SimTypeFunction([SimTypePointer(SimTypeBottom(label="Void"), offset=0), SimTypePointer(SimStruct({"changeType": SimTypeInt(signed=False, label="IPSEC_SA_CONTEXT_EVENT_TYPE0"), "saContextId": SimTypeLongLong(signed=False, label="UInt64")}, name="IPSEC_SA_CONTEXT_CHANGE0", pack=False, align=None), offset=0)], SimTypeBottom(label="Void"), arg_names=["context", "change"]), offset=0), SimTypePointer(SimTypeBottom(label="Void"), offset=0), SimTypePointer(SimTypePointer(SimTypeInt(signed=True, label="Int"), label="IntPtr", offset=0), offset=0)], SimTypeInt(signed=False, label="UInt32"), arg_names=["engineHandle", "subscription", "callback", "context", "eventsHandle"]),
        # 
        'IPsecSaContextUnsubscribe0': SimTypeFunction([SimTypePointer(SimTypeInt(signed=True, label="Int"), label="IntPtr", offset=0), SimTypePointer(SimTypeInt(signed=True, label="Int"), label="IntPtr", offset=0)], SimTypeInt(signed=False, label="UInt32"), arg_names=["engineHandle", "eventsHandle"]),
        # 
        'IPsecSaContextSubscriptionsGet0': SimTypeFunction([SimTypePointer(SimTypeInt(signed=True, label="Int"), label="IntPtr", offset=0), SimTypePointer(SimTypePointer(SimTypePointer(SimStruct({"enumTemplate": SimTypePointer(SimStruct({"localSubNet": SimStruct({"type": SimTypeInt(signed=False, label="FWP_DATA_TYPE"), "Anonymous": SimUnion({"uint8": SimTypeChar(label="Byte"), "uint16": SimTypeShort(signed=False, label="UInt16"), "uint32": SimTypeInt(signed=False, label="UInt32"), "uint64": SimTypePointer(SimTypeLongLong(signed=False, label="UInt64"), offset=0), "int8": SimTypeChar(label="SByte"), "int16": SimTypeShort(signed=True, label="Int16"), "int32": SimTypeInt(signed=True, label="Int32"), "int64": SimTypePointer(SimTypeLongLong(signed=True, label="Int64"), offset=0), "float32": SimTypeFloat(size=32), "double64": SimTypePointer(SimTypeFloat(size=64), offset=0), "byteArray16": SimTypePointer(SimStruct({"byteArray16": SimTypeFixedSizeArray(SimTypeChar(label="Byte"), 16)}, name="FWP_BYTE_ARRAY16", pack=False, align=None), offset=0), "byteBlob": SimTypePointer(SimStruct({"size": SimTypeInt(signed=False, label="UInt32"), "data": SimTypePointer(SimTypeChar(label="Byte"), offset=0)}, name="FWP_BYTE_BLOB", pack=False, align=None), offset=0), "sid": SimTypePointer(SimTypeBottom(label="SID"), offset=0), "sd": SimTypePointer(SimStruct({"size": SimTypeInt(signed=False, label="UInt32"), "data": SimTypePointer(SimTypeChar(label="Byte"), offset=0)}, name="FWP_BYTE_BLOB", pack=False, align=None), offset=0), "tokenInformation": SimTypePointer(SimStruct({"sidCount": SimTypeInt(signed=False, label="UInt32"), "sids": SimTypePointer(SimTypeBottom(label="SID_AND_ATTRIBUTES"), offset=0), "restrictedSidCount": SimTypeInt(signed=False, label="UInt32"), "restrictedSids": SimTypePointer(SimTypeBottom(label="SID_AND_ATTRIBUTES"), offset=0)}, name="FWP_TOKEN_INFORMATION", pack=False, align=None), offset=0), "tokenAccessInformation": SimTypePointer(SimStruct({"size": SimTypeInt(signed=False, label="UInt32"), "data": SimTypePointer(SimTypeChar(label="Byte"), offset=0)}, name="FWP_BYTE_BLOB", pack=False, align=None), offset=0), "unicodeString": SimTypePointer(SimTypeChar(label="Char"), offset=0), "byteArray6": SimTypePointer(SimStruct({"byteArray6": SimTypeFixedSizeArray(SimTypeChar(label="Byte"), 6)}, name="FWP_BYTE_ARRAY6", pack=False, align=None), offset=0), "bitmapArray64": SimTypePointer(SimStruct({"bitmapArray64": SimTypeFixedSizeArray(SimTypeChar(label="Byte"), 8)}, name="FWP_BITMAP_ARRAY64_", pack=False, align=None), offset=0), "v4AddrMask": SimTypePointer(SimStruct({"addr": SimTypeInt(signed=False, label="UInt32"), "mask": SimTypeInt(signed=False, label="UInt32")}, name="FWP_V4_ADDR_AND_MASK", pack=False, align=None), offset=0), "v6AddrMask": SimTypePointer(SimStruct({"addr": SimTypeFixedSizeArray(SimTypeChar(label="Byte"), 16), "prefixLength": SimTypeChar(label="Byte")}, name="FWP_V6_ADDR_AND_MASK", pack=False, align=None), offset=0), "rangeValue": SimTypePointer(SimStruct({"valueLow": SimStruct({"type": SimTypeInt(signed=False, label="FWP_DATA_TYPE"), "Anonymous": SimUnion({"uint8": SimTypeChar(label="Byte"), "uint16": SimTypeShort(signed=False, label="UInt16"), "uint32": SimTypeInt(signed=False, label="UInt32"), "uint64": SimTypePointer(SimTypeLongLong(signed=False, label="UInt64"), offset=0), "int8": SimTypeChar(label="SByte"), "int16": SimTypeShort(signed=True, label="Int16"), "int32": SimTypeInt(signed=True, label="Int32"), "int64": SimTypePointer(SimTypeLongLong(signed=True, label="Int64"), offset=0), "float32": SimTypeFloat(size=32), "double64": SimTypePointer(SimTypeFloat(size=64), offset=0), "byteArray16": SimTypePointer(SimStruct({"byteArray16": SimTypeFixedSizeArray(SimTypeChar(label="Byte"), 16)}, name="FWP_BYTE_ARRAY16", pack=False, align=None), offset=0), "byteBlob": SimTypePointer(SimStruct({"size": SimTypeInt(signed=False, label="UInt32"), "data": SimTypePointer(SimTypeChar(label="Byte"), offset=0)}, name="FWP_BYTE_BLOB", pack=False, align=None), offset=0), "sid": SimTypePointer(SimTypeBottom(label="SID"), offset=0), "sd": SimTypePointer(SimStruct({"size": SimTypeInt(signed=False, label="UInt32"), "data": SimTypePointer(SimTypeChar(label="Byte"), offset=0)}, name="FWP_BYTE_BLOB", pack=False, align=None), offset=0), "tokenInformation": SimTypePointer(SimStruct({"sidCount": SimTypeInt(signed=False, label="UInt32"), "sids": SimTypePointer(SimTypeBottom(label="SID_AND_ATTRIBUTES"), offset=0), "restrictedSidCount": SimTypeInt(signed=False, label="UInt32"), "restrictedSids": SimTypePointer(SimTypeBottom(label="SID_AND_ATTRIBUTES"), offset=0)}, name="FWP_TOKEN_INFORMATION", pack=False, align=None), offset=0), "tokenAccessInformation": SimTypePointer(SimStruct({"size": SimTypeInt(signed=False, label="UInt32"), "data": SimTypePointer(SimTypeChar(label="Byte"), offset=0)}, name="FWP_BYTE_BLOB", pack=False, align=None), offset=0), "unicodeString": SimTypePointer(SimTypeChar(label="Char"), offset=0), "byteArray6": SimTypePointer(SimStruct({"byteArray6": SimTypeFixedSizeArray(SimTypeChar(label="Byte"), 6)}, name="FWP_BYTE_ARRAY6", pack=False, align=None), offset=0), "bitmapArray64": SimTypePointer(SimStruct({"bitmapArray64": SimTypeFixedSizeArray(SimTypeChar(label="Byte"), 8)}, name="FWP_BITMAP_ARRAY64_", pack=False, align=None), offset=0)}, name="<anon>", label="None")}, name="FWP_VALUE0", pack=False, align=None), "valueHigh": SimStruct({"type": SimTypeInt(signed=False, label="FWP_DATA_TYPE"), "Anonymous": SimUnion({"uint8": SimTypeChar(label="Byte"), "uint16": SimTypeShort(signed=False, label="UInt16"), "uint32": SimTypeInt(signed=False, label="UInt32"), "uint64": SimTypePointer(SimTypeLongLong(signed=False, label="UInt64"), offset=0), "int8": SimTypeChar(label="SByte"), "int16": SimTypeShort(signed=True, label="Int16"), "int32": SimTypeInt(signed=True, label="Int32"), "int64": SimTypePointer(SimTypeLongLong(signed=True, label="Int64"), offset=0), "float32": SimTypeFloat(size=32), "double64": SimTypePointer(SimTypeFloat(size=64), offset=0), "byteArray16": SimTypePointer(SimStruct({"byteArray16": SimTypeFixedSizeArray(SimTypeChar(label="Byte"), 16)}, name="FWP_BYTE_ARRAY16", pack=False, align=None), offset=0), "byteBlob": SimTypePointer(SimStruct({"size": SimTypeInt(signed=False, label="UInt32"), "data": SimTypePointer(SimTypeChar(label="Byte"), offset=0)}, name="FWP_BYTE_BLOB", pack=False, align=None), offset=0), "sid": SimTypePointer(SimTypeBottom(label="SID"), offset=0), "sd": SimTypePointer(SimStruct({"size": SimTypeInt(signed=False, label="UInt32"), "data": SimTypePointer(SimTypeChar(label="Byte"), offset=0)}, name="FWP_BYTE_BLOB", pack=False, align=None), offset=0), "tokenInformation": SimTypePointer(SimStruct({"sidCount": SimTypeInt(signed=False, label="UInt32"), "sids": SimTypePointer(SimTypeBottom(label="SID_AND_ATTRIBUTES"), offset=0), "restrictedSidCount": SimTypeInt(signed=False, label="UInt32"), "restrictedSids": SimTypePointer(SimTypeBottom(label="SID_AND_ATTRIBUTES"), offset=0)}, name="FWP_TOKEN_INFORMATION", pack=False, align=None), offset=0), "tokenAccessInformation": SimTypePointer(SimStruct({"size": SimTypeInt(signed=False, label="UInt32"), "data": SimTypePointer(SimTypeChar(label="Byte"), offset=0)}, name="FWP_BYTE_BLOB", pack=False, align=None), offset=0), "unicodeString": SimTypePointer(SimTypeChar(label="Char"), offset=0), "byteArray6": SimTypePointer(SimStruct({"byteArray6": SimTypeFixedSizeArray(SimTypeChar(label="Byte"), 6)}, name="FWP_BYTE_ARRAY6", pack=False, align=None), offset=0), "bitmapArray64": SimTypePointer(SimStruct({"bitmapArray64": SimTypeFixedSizeArray(SimTypeChar(label="Byte"), 8)}, name="FWP_BITMAP_ARRAY64_", pack=False, align=None), offset=0)}, name="<anon>", label="None")}, name="FWP_VALUE0", pack=False, align=None)}, name="FWP_RANGE0", pack=False, align=None), offset=0)}, name="<anon>", label="None")}, name="FWP_CONDITION_VALUE0", pack=False, align=None), "remoteSubNet": SimStruct({"type": SimTypeInt(signed=False, label="FWP_DATA_TYPE"), "Anonymous": SimUnion({"uint8": SimTypeChar(label="Byte"), "uint16": SimTypeShort(signed=False, label="UInt16"), "uint32": SimTypeInt(signed=False, label="UInt32"), "uint64": SimTypePointer(SimTypeLongLong(signed=False, label="UInt64"), offset=0), "int8": SimTypeChar(label="SByte"), "int16": SimTypeShort(signed=True, label="Int16"), "int32": SimTypeInt(signed=True, label="Int32"), "int64": SimTypePointer(SimTypeLongLong(signed=True, label="Int64"), offset=0), "float32": SimTypeFloat(size=32), "double64": SimTypePointer(SimTypeFloat(size=64), offset=0), "byteArray16": SimTypePointer(SimStruct({"byteArray16": SimTypeFixedSizeArray(SimTypeChar(label="Byte"), 16)}, name="FWP_BYTE_ARRAY16", pack=False, align=None), offset=0), "byteBlob": SimTypePointer(SimStruct({"size": SimTypeInt(signed=False, label="UInt32"), "data": SimTypePointer(SimTypeChar(label="Byte"), offset=0)}, name="FWP_BYTE_BLOB", pack=False, align=None), offset=0), "sid": SimTypePointer(SimTypeBottom(label="SID"), offset=0), "sd": SimTypePointer(SimStruct({"size": SimTypeInt(signed=False, label="UInt32"), "data": SimTypePointer(SimTypeChar(label="Byte"), offset=0)}, name="FWP_BYTE_BLOB", pack=False, align=None), offset=0), "tokenInformation": SimTypePointer(SimStruct({"sidCount": SimTypeInt(signed=False, label="UInt32"), "sids": SimTypePointer(SimTypeBottom(label="SID_AND_ATTRIBUTES"), offset=0), "restrictedSidCount": SimTypeInt(signed=False, label="UInt32"), "restrictedSids": SimTypePointer(SimTypeBottom(label="SID_AND_ATTRIBUTES"), offset=0)}, name="FWP_TOKEN_INFORMATION", pack=False, align=None), offset=0), "tokenAccessInformation": SimTypePointer(SimStruct({"size": SimTypeInt(signed=False, label="UInt32"), "data": SimTypePointer(SimTypeChar(label="Byte"), offset=0)}, name="FWP_BYTE_BLOB", pack=False, align=None), offset=0), "unicodeString": SimTypePointer(SimTypeChar(label="Char"), offset=0), "byteArray6": SimTypePointer(SimStruct({"byteArray6": SimTypeFixedSizeArray(SimTypeChar(label="Byte"), 6)}, name="FWP_BYTE_ARRAY6", pack=False, align=None), offset=0), "bitmapArray64": SimTypePointer(SimStruct({"bitmapArray64": SimTypeFixedSizeArray(SimTypeChar(label="Byte"), 8)}, name="FWP_BITMAP_ARRAY64_", pack=False, align=None), offset=0), "v4AddrMask": SimTypePointer(SimStruct({"addr": SimTypeInt(signed=False, label="UInt32"), "mask": SimTypeInt(signed=False, label="UInt32")}, name="FWP_V4_ADDR_AND_MASK", pack=False, align=None), offset=0), "v6AddrMask": SimTypePointer(SimStruct({"addr": SimTypeFixedSizeArray(SimTypeChar(label="Byte"), 16), "prefixLength": SimTypeChar(label="Byte")}, name="FWP_V6_ADDR_AND_MASK", pack=False, align=None), offset=0), "rangeValue": SimTypePointer(SimStruct({"valueLow": SimStruct({"type": SimTypeInt(signed=False, label="FWP_DATA_TYPE"), "Anonymous": SimUnion({"uint8": SimTypeChar(label="Byte"), "uint16": SimTypeShort(signed=False, label="UInt16"), "uint32": SimTypeInt(signed=False, label="UInt32"), "uint64": SimTypePointer(SimTypeLongLong(signed=False, label="UInt64"), offset=0), "int8": SimTypeChar(label="SByte"), "int16": SimTypeShort(signed=True, label="Int16"), "int32": SimTypeInt(signed=True, label="Int32"), "int64": SimTypePointer(SimTypeLongLong(signed=True, label="Int64"), offset=0), "float32": SimTypeFloat(size=32), "double64": SimTypePointer(SimTypeFloat(size=64), offset=0), "byteArray16": SimTypePointer(SimStruct({"byteArray16": SimTypeFixedSizeArray(SimTypeChar(label="Byte"), 16)}, name="FWP_BYTE_ARRAY16", pack=False, align=None), offset=0), "byteBlob": SimTypePointer(SimStruct({"size": SimTypeInt(signed=False, label="UInt32"), "data": SimTypePointer(SimTypeChar(label="Byte"), offset=0)}, name="FWP_BYTE_BLOB", pack=False, align=None), offset=0), "sid": SimTypePointer(SimTypeBottom(label="SID"), offset=0), "sd": SimTypePointer(SimStruct({"size": SimTypeInt(signed=False, label="UInt32"), "data": SimTypePointer(SimTypeChar(label="Byte"), offset=0)}, name="FWP_BYTE_BLOB", pack=False, align=None), offset=0), "tokenInformation": SimTypePointer(SimStruct({"sidCount": SimTypeInt(signed=False, label="UInt32"), "sids": SimTypePointer(SimTypeBottom(label="SID_AND_ATTRIBUTES"), offset=0), "restrictedSidCount": SimTypeInt(signed=False, label="UInt32"), "restrictedSids": SimTypePointer(SimTypeBottom(label="SID_AND_ATTRIBUTES"), offset=0)}, name="FWP_TOKEN_INFORMATION", pack=False, align=None), offset=0), "tokenAccessInformation": SimTypePointer(SimStruct({"size": SimTypeInt(signed=False, label="UInt32"), "data": SimTypePointer(SimTypeChar(label="Byte"), offset=0)}, name="FWP_BYTE_BLOB", pack=False, align=None), offset=0), "unicodeString": SimTypePointer(SimTypeChar(label="Char"), offset=0), "byteArray6": SimTypePointer(SimStruct({"byteArray6": SimTypeFixedSizeArray(SimTypeChar(label="Byte"), 6)}, name="FWP_BYTE_ARRAY6", pack=False, align=None), offset=0), "bitmapArray64": SimTypePointer(SimStruct({"bitmapArray64": SimTypeFixedSizeArray(SimTypeChar(label="Byte"), 8)}, name="FWP_BITMAP_ARRAY64_", pack=False, align=None), offset=0)}, name="<anon>", label="None")}, name="FWP_VALUE0", pack=False, align=None), "valueHigh": SimStruct({"type": SimTypeInt(signed=False, label="FWP_DATA_TYPE"), "Anonymous": SimUnion({"uint8": SimTypeChar(label="Byte"), "uint16": SimTypeShort(signed=False, label="UInt16"), "uint32": SimTypeInt(signed=False, label="UInt32"), "uint64": SimTypePointer(SimTypeLongLong(signed=False, label="UInt64"), offset=0), "int8": SimTypeChar(label="SByte"), "int16": SimTypeShort(signed=True, label="Int16"), "int32": SimTypeInt(signed=True, label="Int32"), "int64": SimTypePointer(SimTypeLongLong(signed=True, label="Int64"), offset=0), "float32": SimTypeFloat(size=32), "double64": SimTypePointer(SimTypeFloat(size=64), offset=0), "byteArray16": SimTypePointer(SimStruct({"byteArray16": SimTypeFixedSizeArray(SimTypeChar(label="Byte"), 16)}, name="FWP_BYTE_ARRAY16", pack=False, align=None), offset=0), "byteBlob": SimTypePointer(SimStruct({"size": SimTypeInt(signed=False, label="UInt32"), "data": SimTypePointer(SimTypeChar(label="Byte"), offset=0)}, name="FWP_BYTE_BLOB", pack=False, align=None), offset=0), "sid": SimTypePointer(SimTypeBottom(label="SID"), offset=0), "sd": SimTypePointer(SimStruct({"size": SimTypeInt(signed=False, label="UInt32"), "data": SimTypePointer(SimTypeChar(label="Byte"), offset=0)}, name="FWP_BYTE_BLOB", pack=False, align=None), offset=0), "tokenInformation": SimTypePointer(SimStruct({"sidCount": SimTypeInt(signed=False, label="UInt32"), "sids": SimTypePointer(SimTypeBottom(label="SID_AND_ATTRIBUTES"), offset=0), "restrictedSidCount": SimTypeInt(signed=False, label="UInt32"), "restrictedSids": SimTypePointer(SimTypeBottom(label="SID_AND_ATTRIBUTES"), offset=0)}, name="FWP_TOKEN_INFORMATION", pack=False, align=None), offset=0), "tokenAccessInformation": SimTypePointer(SimStruct({"size": SimTypeInt(signed=False, label="UInt32"), "data": SimTypePointer(SimTypeChar(label="Byte"), offset=0)}, name="FWP_BYTE_BLOB", pack=False, align=None), offset=0), "unicodeString": SimTypePointer(SimTypeChar(label="Char"), offset=0), "byteArray6": SimTypePointer(SimStruct({"byteArray6": SimTypeFixedSizeArray(SimTypeChar(label="Byte"), 6)}, name="FWP_BYTE_ARRAY6", pack=False, align=None), offset=0), "bitmapArray64": SimTypePointer(SimStruct({"bitmapArray64": SimTypeFixedSizeArray(SimTypeChar(label="Byte"), 8)}, name="FWP_BITMAP_ARRAY64_", pack=False, align=None), offset=0)}, name="<anon>", label="None")}, name="FWP_VALUE0", pack=False, align=None)}, name="FWP_RANGE0", pack=False, align=None), offset=0)}, name="<anon>", label="None")}, name="FWP_CONDITION_VALUE0", pack=False, align=None)}, name="IPSEC_SA_CONTEXT_ENUM_TEMPLATE0", pack=False, align=None), offset=0), "flags": SimTypeInt(signed=False, label="UInt32"), "sessionKey": SimTypeBottom(label="Guid")}, name="IPSEC_SA_CONTEXT_SUBSCRIPTION0", pack=False, align=None), offset=0), offset=0), offset=0), SimTypePointer(SimTypeInt(signed=False, label="UInt32"), offset=0)], SimTypeInt(signed=False, label="UInt32"), arg_names=["engineHandle", "entries", "numEntries"]),
        # 
        'IPsecSaCreateEnumHandle0': SimTypeFunction([SimTypePointer(SimTypeInt(signed=True, label="Int"), label="IntPtr", offset=0), SimTypePointer(SimStruct({"saDirection": SimTypeInt(signed=False, label="FWP_DIRECTION")}, name="IPSEC_SA_ENUM_TEMPLATE0", pack=False, align=None), offset=0), SimTypePointer(SimTypePointer(SimTypeInt(signed=True, label="Int"), label="IntPtr", offset=0), offset=0)], SimTypeInt(signed=False, label="UInt32"), arg_names=["engineHandle", "enumTemplate", "enumHandle"]),
        # 
        'IPsecSaEnum0': SimTypeFunction([SimTypePointer(SimTypeInt(signed=True, label="Int"), label="IntPtr", offset=0), SimTypePointer(SimTypeInt(signed=True, label="Int"), label="IntPtr", offset=0), SimTypeInt(signed=False, label="UInt32"), SimTypePointer(SimTypePointer(SimTypePointer(SimStruct({"ipVersion": SimTypeInt(signed=False, label="FWP_IP_VERSION"), "saDirection": SimTypeInt(signed=False, label="FWP_DIRECTION"), "traffic": SimStruct({"ipVersion": SimTypeInt(signed=False, label="FWP_IP_VERSION"), "Anonymous1": SimUnion({"localV4Address": SimTypeInt(signed=False, label="UInt32"), "localV6Address": SimTypeFixedSizeArray(SimTypeChar(label="Byte"), 16)}, name="<anon>", label="None"), "Anonymous2": SimUnion({"remoteV4Address": SimTypeInt(signed=False, label="UInt32"), "remoteV6Address": SimTypeFixedSizeArray(SimTypeChar(label="Byte"), 16)}, name="<anon>", label="None"), "trafficType": SimTypeInt(signed=False, label="IPSEC_TRAFFIC_TYPE"), "Anonymous3": SimUnion({"ipsecFilterId": SimTypeLongLong(signed=False, label="UInt64"), "tunnelPolicyId": SimTypeLongLong(signed=False, label="UInt64")}, name="<anon>", label="None"), "remotePort": SimTypeShort(signed=False, label="UInt16")}, name="IPSEC_TRAFFIC0", pack=False, align=None), "saBundle": SimStruct({"flags": SimTypeInt(signed=False, label="IPSEC_SA_BUNDLE_FLAGS"), "lifetime": SimStruct({"lifetimeSeconds": SimTypeInt(signed=False, label="UInt32"), "lifetimeKilobytes": SimTypeInt(signed=False, label="UInt32"), "lifetimePackets": SimTypeInt(signed=False, label="UInt32")}, name="IPSEC_SA_LIFETIME0", pack=False, align=None), "idleTimeoutSeconds": SimTypeInt(signed=False, label="UInt32"), "ndAllowClearTimeoutSeconds": SimTypeInt(signed=False, label="UInt32"), "ipsecId": SimTypePointer(SimStruct({"mmTargetName": SimTypePointer(SimTypeChar(label="Char"), offset=0), "emTargetName": SimTypePointer(SimTypeChar(label="Char"), offset=0), "numTokens": SimTypeInt(signed=False, label="UInt32"), "tokens": SimTypePointer(SimStruct({"type": SimTypeInt(signed=False, label="IPSEC_TOKEN_TYPE"), "principal": SimTypeInt(signed=False, label="IPSEC_TOKEN_PRINCIPAL"), "mode": SimTypeInt(signed=False, label="IPSEC_TOKEN_MODE"), "token": SimTypeLongLong(signed=False, label="UInt64")}, name="IPSEC_TOKEN0", pack=False, align=None), offset=0), "explicitCredentials": SimTypeLongLong(signed=False, label="UInt64"), "logonId": SimTypeLongLong(signed=False, label="UInt64")}, name="IPSEC_ID0", pack=False, align=None), offset=0), "napContext": SimTypeInt(signed=False, label="UInt32"), "qmSaId": SimTypeInt(signed=False, label="UInt32"), "numSAs": SimTypeInt(signed=False, label="UInt32"), "saList": SimTypePointer(SimStruct({"spi": SimTypeInt(signed=False, label="UInt32"), "saTransformType": SimTypeInt(signed=False, label="IPSEC_TRANSFORM_TYPE"), "Anonymous": SimUnion({"ahInformation": SimTypePointer(SimStruct({"authTransform": SimStruct({"authTransformId": SimStruct({"authType": SimTypeInt(signed=False, label="IPSEC_AUTH_TYPE"), "authConfig": SimTypeChar(label="Byte")}, name="IPSEC_AUTH_TRANSFORM_ID0", pack=False, align=None), "cryptoModuleId": SimTypePointer(SimTypeBottom(label="Guid"), offset=0)}, name="IPSEC_AUTH_TRANSFORM0", pack=False, align=None), "authKey": SimStruct({"size": SimTypeInt(signed=False, label="UInt32"), "data": SimTypePointer(SimTypeChar(label="Byte"), offset=0)}, name="FWP_BYTE_BLOB", pack=False, align=None)}, name="IPSEC_SA_AUTH_INFORMATION0", pack=False, align=None), offset=0), "espAuthInformation": SimTypePointer(SimStruct({"authTransform": SimStruct({"authTransformId": SimStruct({"authType": SimTypeInt(signed=False, label="IPSEC_AUTH_TYPE"), "authConfig": SimTypeChar(label="Byte")}, name="IPSEC_AUTH_TRANSFORM_ID0", pack=False, align=None), "cryptoModuleId": SimTypePointer(SimTypeBottom(label="Guid"), offset=0)}, name="IPSEC_AUTH_TRANSFORM0", pack=False, align=None), "authKey": SimStruct({"size": SimTypeInt(signed=False, label="UInt32"), "data": SimTypePointer(SimTypeChar(label="Byte"), offset=0)}, name="FWP_BYTE_BLOB", pack=False, align=None)}, name="IPSEC_SA_AUTH_INFORMATION0", pack=False, align=None), offset=0), "espCipherInformation": SimTypePointer(SimStruct({"cipherTransform": SimStruct({"cipherTransformId": SimStruct({"cipherType": SimTypeInt(signed=False, label="IPSEC_CIPHER_TYPE"), "cipherConfig": SimTypeChar(label="Byte")}, name="IPSEC_CIPHER_TRANSFORM_ID0", pack=False, align=None), "cryptoModuleId": SimTypePointer(SimTypeBottom(label="Guid"), offset=0)}, name="IPSEC_CIPHER_TRANSFORM0", pack=False, align=None), "cipherKey": SimStruct({"size": SimTypeInt(signed=False, label="UInt32"), "data": SimTypePointer(SimTypeChar(label="Byte"), offset=0)}, name="FWP_BYTE_BLOB", pack=False, align=None)}, name="IPSEC_SA_CIPHER_INFORMATION0", pack=False, align=None), offset=0), "espAuthAndCipherInformation": SimTypePointer(SimStruct({"saCipherInformation": SimStruct({"cipherTransform": SimStruct({"cipherTransformId": SimStruct({"cipherType": SimTypeInt(signed=False, label="IPSEC_CIPHER_TYPE"), "cipherConfig": SimTypeChar(label="Byte")}, name="IPSEC_CIPHER_TRANSFORM_ID0", pack=False, align=None), "cryptoModuleId": SimTypePointer(SimTypeBottom(label="Guid"), offset=0)}, name="IPSEC_CIPHER_TRANSFORM0", pack=False, align=None), "cipherKey": SimStruct({"size": SimTypeInt(signed=False, label="UInt32"), "data": SimTypePointer(SimTypeChar(label="Byte"), offset=0)}, name="FWP_BYTE_BLOB", pack=False, align=None)}, name="IPSEC_SA_CIPHER_INFORMATION0", pack=False, align=None), "saAuthInformation": SimStruct({"authTransform": SimStruct({"authTransformId": SimStruct({"authType": SimTypeInt(signed=False, label="IPSEC_AUTH_TYPE"), "authConfig": SimTypeChar(label="Byte")}, name="IPSEC_AUTH_TRANSFORM_ID0", pack=False, align=None), "cryptoModuleId": SimTypePointer(SimTypeBottom(label="Guid"), offset=0)}, name="IPSEC_AUTH_TRANSFORM0", pack=False, align=None), "authKey": SimStruct({"size": SimTypeInt(signed=False, label="UInt32"), "data": SimTypePointer(SimTypeChar(label="Byte"), offset=0)}, name="FWP_BYTE_BLOB", pack=False, align=None)}, name="IPSEC_SA_AUTH_INFORMATION0", pack=False, align=None)}, name="IPSEC_SA_AUTH_AND_CIPHER_INFORMATION0", pack=False, align=None), offset=0), "espAuthFwInformation": SimTypePointer(SimStruct({"authTransform": SimStruct({"authTransformId": SimStruct({"authType": SimTypeInt(signed=False, label="IPSEC_AUTH_TYPE"), "authConfig": SimTypeChar(label="Byte")}, name="IPSEC_AUTH_TRANSFORM_ID0", pack=False, align=None), "cryptoModuleId": SimTypePointer(SimTypeBottom(label="Guid"), offset=0)}, name="IPSEC_AUTH_TRANSFORM0", pack=False, align=None), "authKey": SimStruct({"size": SimTypeInt(signed=False, label="UInt32"), "data": SimTypePointer(SimTypeChar(label="Byte"), offset=0)}, name="FWP_BYTE_BLOB", pack=False, align=None)}, name="IPSEC_SA_AUTH_INFORMATION0", pack=False, align=None), offset=0)}, name="<anon>", label="None")}, name="IPSEC_SA0", pack=False, align=None), offset=0), "keyModuleState": SimTypePointer(SimStruct({"keyModuleKey": SimTypeBottom(label="Guid"), "stateBlob": SimStruct({"size": SimTypeInt(signed=False, label="UInt32"), "data": SimTypePointer(SimTypeChar(label="Byte"), offset=0)}, name="FWP_BYTE_BLOB", pack=False, align=None)}, name="IPSEC_KEYMODULE_STATE0", pack=False, align=None), offset=0), "ipVersion": SimTypeInt(signed=False, label="FWP_IP_VERSION"), "Anonymous": SimUnion({"peerV4PrivateAddress": SimTypeInt(signed=False, label="UInt32")}, name="<anon>", label="None"), "mmSaId": SimTypeLongLong(signed=False, label="UInt64"), "pfsGroup": SimTypeInt(signed=False, label="IPSEC_PFS_GROUP")}, name="IPSEC_SA_BUNDLE0", pack=False, align=None), "Anonymous": SimUnion({"udpEncapsulation": SimTypePointer(SimStruct({"localUdpEncapPort": SimTypeShort(signed=False, label="UInt16"), "remoteUdpEncapPort": SimTypeShort(signed=False, label="UInt16")}, name="IPSEC_V4_UDP_ENCAPSULATION0", pack=False, align=None), offset=0)}, name="<anon>", label="None"), "transportFilter": SimTypePointer(SimTypeBottom(label="FWPM_FILTER0"), offset=0)}, name="IPSEC_SA_DETAILS0", pack=False, align=None), offset=0), offset=0), offset=0), SimTypePointer(SimTypeInt(signed=False, label="UInt32"), offset=0)], SimTypeInt(signed=False, label="UInt32"), arg_names=["engineHandle", "enumHandle", "numEntriesRequested", "entries", "numEntriesReturned"]),
        # 
        'IPsecSaEnum1': SimTypeFunction([SimTypePointer(SimTypeInt(signed=True, label="Int"), label="IntPtr", offset=0), SimTypePointer(SimTypeInt(signed=True, label="Int"), label="IntPtr", offset=0), SimTypeInt(signed=False, label="UInt32"), SimTypePointer(SimTypePointer(SimTypePointer(SimStruct({"ipVersion": SimTypeInt(signed=False, label="FWP_IP_VERSION"), "saDirection": SimTypeInt(signed=False, label="FWP_DIRECTION"), "traffic": SimStruct({"ipVersion": SimTypeInt(signed=False, label="FWP_IP_VERSION"), "Anonymous1": SimUnion({"localV4Address": SimTypeInt(signed=False, label="UInt32"), "localV6Address": SimTypeFixedSizeArray(SimTypeChar(label="Byte"), 16)}, name="<anon>", label="None"), "Anonymous2": SimUnion({"remoteV4Address": SimTypeInt(signed=False, label="UInt32"), "remoteV6Address": SimTypeFixedSizeArray(SimTypeChar(label="Byte"), 16)}, name="<anon>", label="None"), "trafficType": SimTypeInt(signed=False, label="IPSEC_TRAFFIC_TYPE"), "Anonymous3": SimUnion({"ipsecFilterId": SimTypeLongLong(signed=False, label="UInt64"), "tunnelPolicyId": SimTypeLongLong(signed=False, label="UInt64")}, name="<anon>", label="None"), "remotePort": SimTypeShort(signed=False, label="UInt16"), "localPort": SimTypeShort(signed=False, label="UInt16"), "ipProtocol": SimTypeChar(label="Byte"), "localIfLuid": SimTypeLongLong(signed=False, label="UInt64"), "realIfProfileId": SimTypeInt(signed=False, label="UInt32")}, name="IPSEC_TRAFFIC1", pack=False, align=None), "saBundle": SimStruct({"flags": SimTypeInt(signed=False, label="IPSEC_SA_BUNDLE_FLAGS"), "lifetime": SimStruct({"lifetimeSeconds": SimTypeInt(signed=False, label="UInt32"), "lifetimeKilobytes": SimTypeInt(signed=False, label="UInt32"), "lifetimePackets": SimTypeInt(signed=False, label="UInt32")}, name="IPSEC_SA_LIFETIME0", pack=False, align=None), "idleTimeoutSeconds": SimTypeInt(signed=False, label="UInt32"), "ndAllowClearTimeoutSeconds": SimTypeInt(signed=False, label="UInt32"), "ipsecId": SimTypePointer(SimStruct({"mmTargetName": SimTypePointer(SimTypeChar(label="Char"), offset=0), "emTargetName": SimTypePointer(SimTypeChar(label="Char"), offset=0), "numTokens": SimTypeInt(signed=False, label="UInt32"), "tokens": SimTypePointer(SimStruct({"type": SimTypeInt(signed=False, label="IPSEC_TOKEN_TYPE"), "principal": SimTypeInt(signed=False, label="IPSEC_TOKEN_PRINCIPAL"), "mode": SimTypeInt(signed=False, label="IPSEC_TOKEN_MODE"), "token": SimTypeLongLong(signed=False, label="UInt64")}, name="IPSEC_TOKEN0", pack=False, align=None), offset=0), "explicitCredentials": SimTypeLongLong(signed=False, label="UInt64"), "logonId": SimTypeLongLong(signed=False, label="UInt64")}, name="IPSEC_ID0", pack=False, align=None), offset=0), "napContext": SimTypeInt(signed=False, label="UInt32"), "qmSaId": SimTypeInt(signed=False, label="UInt32"), "numSAs": SimTypeInt(signed=False, label="UInt32"), "saList": SimTypePointer(SimStruct({"spi": SimTypeInt(signed=False, label="UInt32"), "saTransformType": SimTypeInt(signed=False, label="IPSEC_TRANSFORM_TYPE"), "Anonymous": SimUnion({"ahInformation": SimTypePointer(SimStruct({"authTransform": SimStruct({"authTransformId": SimStruct({"authType": SimTypeInt(signed=False, label="IPSEC_AUTH_TYPE"), "authConfig": SimTypeChar(label="Byte")}, name="IPSEC_AUTH_TRANSFORM_ID0", pack=False, align=None), "cryptoModuleId": SimTypePointer(SimTypeBottom(label="Guid"), offset=0)}, name="IPSEC_AUTH_TRANSFORM0", pack=False, align=None), "authKey": SimStruct({"size": SimTypeInt(signed=False, label="UInt32"), "data": SimTypePointer(SimTypeChar(label="Byte"), offset=0)}, name="FWP_BYTE_BLOB", pack=False, align=None)}, name="IPSEC_SA_AUTH_INFORMATION0", pack=False, align=None), offset=0), "espAuthInformation": SimTypePointer(SimStruct({"authTransform": SimStruct({"authTransformId": SimStruct({"authType": SimTypeInt(signed=False, label="IPSEC_AUTH_TYPE"), "authConfig": SimTypeChar(label="Byte")}, name="IPSEC_AUTH_TRANSFORM_ID0", pack=False, align=None), "cryptoModuleId": SimTypePointer(SimTypeBottom(label="Guid"), offset=0)}, name="IPSEC_AUTH_TRANSFORM0", pack=False, align=None), "authKey": SimStruct({"size": SimTypeInt(signed=False, label="UInt32"), "data": SimTypePointer(SimTypeChar(label="Byte"), offset=0)}, name="FWP_BYTE_BLOB", pack=False, align=None)}, name="IPSEC_SA_AUTH_INFORMATION0", pack=False, align=None), offset=0), "espCipherInformation": SimTypePointer(SimStruct({"cipherTransform": SimStruct({"cipherTransformId": SimStruct({"cipherType": SimTypeInt(signed=False, label="IPSEC_CIPHER_TYPE"), "cipherConfig": SimTypeChar(label="Byte")}, name="IPSEC_CIPHER_TRANSFORM_ID0", pack=False, align=None), "cryptoModuleId": SimTypePointer(SimTypeBottom(label="Guid"), offset=0)}, name="IPSEC_CIPHER_TRANSFORM0", pack=False, align=None), "cipherKey": SimStruct({"size": SimTypeInt(signed=False, label="UInt32"), "data": SimTypePointer(SimTypeChar(label="Byte"), offset=0)}, name="FWP_BYTE_BLOB", pack=False, align=None)}, name="IPSEC_SA_CIPHER_INFORMATION0", pack=False, align=None), offset=0), "espAuthAndCipherInformation": SimTypePointer(SimStruct({"saCipherInformation": SimStruct({"cipherTransform": SimStruct({"cipherTransformId": SimStruct({"cipherType": SimTypeInt(signed=False, label="IPSEC_CIPHER_TYPE"), "cipherConfig": SimTypeChar(label="Byte")}, name="IPSEC_CIPHER_TRANSFORM_ID0", pack=False, align=None), "cryptoModuleId": SimTypePointer(SimTypeBottom(label="Guid"), offset=0)}, name="IPSEC_CIPHER_TRANSFORM0", pack=False, align=None), "cipherKey": SimStruct({"size": SimTypeInt(signed=False, label="UInt32"), "data": SimTypePointer(SimTypeChar(label="Byte"), offset=0)}, name="FWP_BYTE_BLOB", pack=False, align=None)}, name="IPSEC_SA_CIPHER_INFORMATION0", pack=False, align=None), "saAuthInformation": SimStruct({"authTransform": SimStruct({"authTransformId": SimStruct({"authType": SimTypeInt(signed=False, label="IPSEC_AUTH_TYPE"), "authConfig": SimTypeChar(label="Byte")}, name="IPSEC_AUTH_TRANSFORM_ID0", pack=False, align=None), "cryptoModuleId": SimTypePointer(SimTypeBottom(label="Guid"), offset=0)}, name="IPSEC_AUTH_TRANSFORM0", pack=False, align=None), "authKey": SimStruct({"size": SimTypeInt(signed=False, label="UInt32"), "data": SimTypePointer(SimTypeChar(label="Byte"), offset=0)}, name="FWP_BYTE_BLOB", pack=False, align=None)}, name="IPSEC_SA_AUTH_INFORMATION0", pack=False, align=None)}, name="IPSEC_SA_AUTH_AND_CIPHER_INFORMATION0", pack=False, align=None), offset=0), "espAuthFwInformation": SimTypePointer(SimStruct({"authTransform": SimStruct({"authTransformId": SimStruct({"authType": SimTypeInt(signed=False, label="IPSEC_AUTH_TYPE"), "authConfig": SimTypeChar(label="Byte")}, name="IPSEC_AUTH_TRANSFORM_ID0", pack=False, align=None), "cryptoModuleId": SimTypePointer(SimTypeBottom(label="Guid"), offset=0)}, name="IPSEC_AUTH_TRANSFORM0", pack=False, align=None), "authKey": SimStruct({"size": SimTypeInt(signed=False, label="UInt32"), "data": SimTypePointer(SimTypeChar(label="Byte"), offset=0)}, name="FWP_BYTE_BLOB", pack=False, align=None)}, name="IPSEC_SA_AUTH_INFORMATION0", pack=False, align=None), offset=0)}, name="<anon>", label="None")}, name="IPSEC_SA0", pack=False, align=None), offset=0), "keyModuleState": SimTypePointer(SimStruct({"keyModuleKey": SimTypeBottom(label="Guid"), "stateBlob": SimStruct({"size": SimTypeInt(signed=False, label="UInt32"), "data": SimTypePointer(SimTypeChar(label="Byte"), offset=0)}, name="FWP_BYTE_BLOB", pack=False, align=None)}, name="IPSEC_KEYMODULE_STATE0", pack=False, align=None), offset=0), "ipVersion": SimTypeInt(signed=False, label="FWP_IP_VERSION"), "Anonymous": SimUnion({"peerV4PrivateAddress": SimTypeInt(signed=False, label="UInt32")}, name="<anon>", label="None"), "mmSaId": SimTypeLongLong(signed=False, label="UInt64"), "pfsGroup": SimTypeInt(signed=False, label="IPSEC_PFS_GROUP"), "saLookupContext": SimTypeBottom(label="Guid"), "qmFilterId": SimTypeLongLong(signed=False, label="UInt64")}, name="IPSEC_SA_BUNDLE1", pack=False, align=None), "Anonymous": SimUnion({"udpEncapsulation": SimTypePointer(SimStruct({"localUdpEncapPort": SimTypeShort(signed=False, label="UInt16"), "remoteUdpEncapPort": SimTypeShort(signed=False, label="UInt16")}, name="IPSEC_V4_UDP_ENCAPSULATION0", pack=False, align=None), offset=0)}, name="<anon>", label="None"), "transportFilter": SimTypePointer(SimTypeBottom(label="FWPM_FILTER0"), offset=0), "virtualIfTunnelInfo": SimStruct({"virtualIfTunnelId": SimTypeLongLong(signed=False, label="UInt64"), "trafficSelectorId": SimTypeLongLong(signed=False, label="UInt64")}, name="IPSEC_VIRTUAL_IF_TUNNEL_INFO0", pack=False, align=None)}, name="IPSEC_SA_DETAILS1", pack=False, align=None), offset=0), offset=0), offset=0), SimTypePointer(SimTypeInt(signed=False, label="UInt32"), offset=0)], SimTypeInt(signed=False, label="UInt32"), arg_names=["engineHandle", "enumHandle", "numEntriesRequested", "entries", "numEntriesReturned"]),
        # 
        'IPsecSaDestroyEnumHandle0': SimTypeFunction([SimTypePointer(SimTypeInt(signed=True, label="Int"), label="IntPtr", offset=0), SimTypePointer(SimTypeInt(signed=True, label="Int"), label="IntPtr", offset=0)], SimTypeInt(signed=False, label="UInt32"), arg_names=["engineHandle", "enumHandle"]),
        # 
        'IPsecSaDbGetSecurityInfo0': SimTypeFunction([SimTypePointer(SimTypeInt(signed=True, label="Int"), label="IntPtr", offset=0), SimTypeInt(signed=False, label="UInt32"), SimTypePointer(SimTypePointer(SimTypeInt(signed=True, label="Int"), label="IntPtr", offset=0), offset=0), SimTypePointer(SimTypePointer(SimTypeInt(signed=True, label="Int"), label="IntPtr", offset=0), offset=0), SimTypePointer(SimTypePointer(SimStruct({"AclRevision": SimTypeChar(label="Byte"), "Sbz1": SimTypeChar(label="Byte"), "AclSize": SimTypeShort(signed=False, label="UInt16"), "AceCount": SimTypeShort(signed=False, label="UInt16"), "Sbz2": SimTypeShort(signed=False, label="UInt16")}, name="ACL", pack=False, align=None), offset=0), offset=0), SimTypePointer(SimTypePointer(SimStruct({"AclRevision": SimTypeChar(label="Byte"), "Sbz1": SimTypeChar(label="Byte"), "AclSize": SimTypeShort(signed=False, label="UInt16"), "AceCount": SimTypeShort(signed=False, label="UInt16"), "Sbz2": SimTypeShort(signed=False, label="UInt16")}, name="ACL", pack=False, align=None), offset=0), offset=0), SimTypePointer(SimTypePointer(SimStruct({"Revision": SimTypeChar(label="Byte"), "Sbz1": SimTypeChar(label="Byte"), "Control": SimTypeShort(signed=False, label="UInt16"), "Owner": SimTypePointer(SimTypeInt(signed=True, label="Int"), label="IntPtr", offset=0), "Group": SimTypePointer(SimTypeInt(signed=True, label="Int"), label="IntPtr", offset=0), "Sacl": SimTypePointer(SimStruct({"AclRevision": SimTypeChar(label="Byte"), "Sbz1": SimTypeChar(label="Byte"), "AclSize": SimTypeShort(signed=False, label="UInt16"), "AceCount": SimTypeShort(signed=False, label="UInt16"), "Sbz2": SimTypeShort(signed=False, label="UInt16")}, name="ACL", pack=False, align=None), offset=0), "Dacl": SimTypePointer(SimStruct({"AclRevision": SimTypeChar(label="Byte"), "Sbz1": SimTypeChar(label="Byte"), "AclSize": SimTypeShort(signed=False, label="UInt16"), "AceCount": SimTypeShort(signed=False, label="UInt16"), "Sbz2": SimTypeShort(signed=False, label="UInt16")}, name="ACL", pack=False, align=None), offset=0)}, name="SECURITY_DESCRIPTOR", pack=False, align=None), offset=0), offset=0)], SimTypeInt(signed=False, label="UInt32"), arg_names=["engineHandle", "securityInfo", "sidOwner", "sidGroup", "dacl", "sacl", "securityDescriptor"]),
        # 
        'IPsecSaDbSetSecurityInfo0': SimTypeFunction([SimTypePointer(SimTypeInt(signed=True, label="Int"), label="IntPtr", offset=0), SimTypeInt(signed=False, label="UInt32"), SimTypePointer(SimStruct({"Revision": SimTypeChar(label="Byte"), "SubAuthorityCount": SimTypeChar(label="Byte"), "IdentifierAuthority": SimStruct({"Value": SimTypeFixedSizeArray(SimTypeChar(label="Byte"), 6)}, name="SID_IDENTIFIER_AUTHORITY", pack=False, align=None), "SubAuthority": SimTypePointer(SimTypeInt(signed=False, label="UInt32"), offset=0)}, name="SID", pack=False, align=None), offset=0), SimTypePointer(SimStruct({"Revision": SimTypeChar(label="Byte"), "SubAuthorityCount": SimTypeChar(label="Byte"), "IdentifierAuthority": SimStruct({"Value": SimTypeFixedSizeArray(SimTypeChar(label="Byte"), 6)}, name="SID_IDENTIFIER_AUTHORITY", pack=False, align=None), "SubAuthority": SimTypePointer(SimTypeInt(signed=False, label="UInt32"), offset=0)}, name="SID", pack=False, align=None), offset=0), SimTypePointer(SimStruct({"AclRevision": SimTypeChar(label="Byte"), "Sbz1": SimTypeChar(label="Byte"), "AclSize": SimTypeShort(signed=False, label="UInt16"), "AceCount": SimTypeShort(signed=False, label="UInt16"), "Sbz2": SimTypeShort(signed=False, label="UInt16")}, name="ACL", pack=False, align=None), offset=0), SimTypePointer(SimStruct({"AclRevision": SimTypeChar(label="Byte"), "Sbz1": SimTypeChar(label="Byte"), "AclSize": SimTypeShort(signed=False, label="UInt16"), "AceCount": SimTypeShort(signed=False, label="UInt16"), "Sbz2": SimTypeShort(signed=False, label="UInt16")}, name="ACL", pack=False, align=None), offset=0)], SimTypeInt(signed=False, label="UInt32"), arg_names=["engineHandle", "securityInfo", "sidOwner", "sidGroup", "dacl", "sacl"]),
        # 
        'IPsecDospGetStatistics0': SimTypeFunction([SimTypePointer(SimTypeInt(signed=True, label="Int"), label="IntPtr", offset=0), SimTypePointer(SimStruct({"totalStateEntriesCreated": SimTypeLongLong(signed=False, label="UInt64"), "currentStateEntries": SimTypeLongLong(signed=False, label="UInt64"), "totalInboundAllowedIPv6IPsecUnauthPkts": SimTypeLongLong(signed=False, label="UInt64"), "totalInboundRatelimitDiscardedIPv6IPsecUnauthPkts": SimTypeLongLong(signed=False, label="UInt64"), "totalInboundPerIPRatelimitDiscardedIPv6IPsecUnauthPkts": SimTypeLongLong(signed=False, label="UInt64"), "totalInboundOtherDiscardedIPv6IPsecUnauthPkts": SimTypeLongLong(signed=False, label="UInt64"), "totalInboundAllowedIPv6IPsecAuthPkts": SimTypeLongLong(signed=False, label="UInt64"), "totalInboundRatelimitDiscardedIPv6IPsecAuthPkts": SimTypeLongLong(signed=False, label="UInt64"), "totalInboundOtherDiscardedIPv6IPsecAuthPkts": SimTypeLongLong(signed=False, label="UInt64"), "totalInboundAllowedICMPv6Pkts": SimTypeLongLong(signed=False, label="UInt64"), "totalInboundRatelimitDiscardedICMPv6Pkts": SimTypeLongLong(signed=False, label="UInt64"), "totalInboundAllowedIPv6FilterExemptPkts": SimTypeLongLong(signed=False, label="UInt64"), "totalInboundRatelimitDiscardedIPv6FilterExemptPkts": SimTypeLongLong(signed=False, label="UInt64"), "totalInboundDiscardedIPv6FilterBlockPkts": SimTypeLongLong(signed=False, label="UInt64"), "totalInboundAllowedDefBlockExemptPkts": SimTypeLongLong(signed=False, label="UInt64"), "totalInboundRatelimitDiscardedDefBlockExemptPkts": SimTypeLongLong(signed=False, label="UInt64"), "totalInboundDiscardedDefBlockPkts": SimTypeLongLong(signed=False, label="UInt64"), "currentInboundIPv6IPsecUnauthPerIPRateLimitQueues": SimTypeLongLong(signed=False, label="UInt64")}, name="IPSEC_DOSP_STATISTICS0", pack=False, align=None), offset=0)], SimTypeInt(signed=False, label="UInt32"), arg_names=["engineHandle", "idpStatistics"]),
        # 
        'IPsecDospStateCreateEnumHandle0': SimTypeFunction([SimTypePointer(SimTypeInt(signed=True, label="Int"), label="IntPtr", offset=0), SimTypePointer(SimStruct({"publicV6AddrMask": SimStruct({"addr": SimTypeFixedSizeArray(SimTypeChar(label="Byte"), 16), "prefixLength": SimTypeChar(label="Byte")}, name="FWP_V6_ADDR_AND_MASK", pack=False, align=None), "internalV6AddrMask": SimStruct({"addr": SimTypeFixedSizeArray(SimTypeChar(label="Byte"), 16), "prefixLength": SimTypeChar(label="Byte")}, name="FWP_V6_ADDR_AND_MASK", pack=False, align=None)}, name="IPSEC_DOSP_STATE_ENUM_TEMPLATE0", pack=False, align=None), offset=0), SimTypePointer(SimTypePointer(SimTypeInt(signed=True, label="Int"), label="IntPtr", offset=0), offset=0)], SimTypeInt(signed=False, label="UInt32"), arg_names=["engineHandle", "enumTemplate", "enumHandle"]),
        # 
        'IPsecDospStateEnum0': SimTypeFunction([SimTypePointer(SimTypeInt(signed=True, label="Int"), label="IntPtr", offset=0), SimTypePointer(SimTypeInt(signed=True, label="Int"), label="IntPtr", offset=0), SimTypeInt(signed=False, label="UInt32"), SimTypePointer(SimTypePointer(SimTypePointer(SimStruct({"publicHostV6Addr": SimTypeFixedSizeArray(SimTypeChar(label="Byte"), 16), "internalHostV6Addr": SimTypeFixedSizeArray(SimTypeChar(label="Byte"), 16), "totalInboundIPv6IPsecAuthPackets": SimTypeLongLong(signed=False, label="UInt64"), "totalOutboundIPv6IPsecAuthPackets": SimTypeLongLong(signed=False, label="UInt64"), "durationSecs": SimTypeInt(signed=False, label="UInt32")}, name="IPSEC_DOSP_STATE0", pack=False, align=None), offset=0), offset=0), offset=0), SimTypePointer(SimTypeInt(signed=False, label="UInt32"), offset=0)], SimTypeInt(signed=False, label="UInt32"), arg_names=["engineHandle", "enumHandle", "numEntriesRequested", "entries", "numEntries"]),
        # 
        'IPsecDospStateDestroyEnumHandle0': SimTypeFunction([SimTypePointer(SimTypeInt(signed=True, label="Int"), label="IntPtr", offset=0), SimTypePointer(SimTypeInt(signed=True, label="Int"), label="IntPtr", offset=0)], SimTypeInt(signed=False, label="UInt32"), arg_names=["engineHandle", "enumHandle"]),
        # 
        'IPsecDospGetSecurityInfo0': SimTypeFunction([SimTypePointer(SimTypeInt(signed=True, label="Int"), label="IntPtr", offset=0), SimTypeInt(signed=False, label="UInt32"), SimTypePointer(SimTypePointer(SimTypeInt(signed=True, label="Int"), label="IntPtr", offset=0), offset=0), SimTypePointer(SimTypePointer(SimTypeInt(signed=True, label="Int"), label="IntPtr", offset=0), offset=0), SimTypePointer(SimTypePointer(SimStruct({"AclRevision": SimTypeChar(label="Byte"), "Sbz1": SimTypeChar(label="Byte"), "AclSize": SimTypeShort(signed=False, label="UInt16"), "AceCount": SimTypeShort(signed=False, label="UInt16"), "Sbz2": SimTypeShort(signed=False, label="UInt16")}, name="ACL", pack=False, align=None), offset=0), offset=0), SimTypePointer(SimTypePointer(SimStruct({"AclRevision": SimTypeChar(label="Byte"), "Sbz1": SimTypeChar(label="Byte"), "AclSize": SimTypeShort(signed=False, label="UInt16"), "AceCount": SimTypeShort(signed=False, label="UInt16"), "Sbz2": SimTypeShort(signed=False, label="UInt16")}, name="ACL", pack=False, align=None), offset=0), offset=0), SimTypePointer(SimTypePointer(SimStruct({"Revision": SimTypeChar(label="Byte"), "Sbz1": SimTypeChar(label="Byte"), "Control": SimTypeShort(signed=False, label="UInt16"), "Owner": SimTypePointer(SimTypeInt(signed=True, label="Int"), label="IntPtr", offset=0), "Group": SimTypePointer(SimTypeInt(signed=True, label="Int"), label="IntPtr", offset=0), "Sacl": SimTypePointer(SimStruct({"AclRevision": SimTypeChar(label="Byte"), "Sbz1": SimTypeChar(label="Byte"), "AclSize": SimTypeShort(signed=False, label="UInt16"), "AceCount": SimTypeShort(signed=False, label="UInt16"), "Sbz2": SimTypeShort(signed=False, label="UInt16")}, name="ACL", pack=False, align=None), offset=0), "Dacl": SimTypePointer(SimStruct({"AclRevision": SimTypeChar(label="Byte"), "Sbz1": SimTypeChar(label="Byte"), "AclSize": SimTypeShort(signed=False, label="UInt16"), "AceCount": SimTypeShort(signed=False, label="UInt16"), "Sbz2": SimTypeShort(signed=False, label="UInt16")}, name="ACL", pack=False, align=None), offset=0)}, name="SECURITY_DESCRIPTOR", pack=False, align=None), offset=0), offset=0)], SimTypeInt(signed=False, label="UInt32"), arg_names=["engineHandle", "securityInfo", "sidOwner", "sidGroup", "dacl", "sacl", "securityDescriptor"]),
        # 
        'IPsecDospSetSecurityInfo0': SimTypeFunction([SimTypePointer(SimTypeInt(signed=True, label="Int"), label="IntPtr", offset=0), SimTypeInt(signed=False, label="UInt32"), SimTypePointer(SimStruct({"Revision": SimTypeChar(label="Byte"), "SubAuthorityCount": SimTypeChar(label="Byte"), "IdentifierAuthority": SimStruct({"Value": SimTypeFixedSizeArray(SimTypeChar(label="Byte"), 6)}, name="SID_IDENTIFIER_AUTHORITY", pack=False, align=None), "SubAuthority": SimTypePointer(SimTypeInt(signed=False, label="UInt32"), offset=0)}, name="SID", pack=False, align=None), offset=0), SimTypePointer(SimStruct({"Revision": SimTypeChar(label="Byte"), "SubAuthorityCount": SimTypeChar(label="Byte"), "IdentifierAuthority": SimStruct({"Value": SimTypeFixedSizeArray(SimTypeChar(label="Byte"), 6)}, name="SID_IDENTIFIER_AUTHORITY", pack=False, align=None), "SubAuthority": SimTypePointer(SimTypeInt(signed=False, label="UInt32"), offset=0)}, name="SID", pack=False, align=None), offset=0), SimTypePointer(SimStruct({"AclRevision": SimTypeChar(label="Byte"), "Sbz1": SimTypeChar(label="Byte"), "AclSize": SimTypeShort(signed=False, label="UInt16"), "AceCount": SimTypeShort(signed=False, label="UInt16"), "Sbz2": SimTypeShort(signed=False, label="UInt16")}, name="ACL", pack=False, align=None), offset=0), SimTypePointer(SimStruct({"AclRevision": SimTypeChar(label="Byte"), "Sbz1": SimTypeChar(label="Byte"), "AclSize": SimTypeShort(signed=False, label="UInt16"), "AceCount": SimTypeShort(signed=False, label="UInt16"), "Sbz2": SimTypeShort(signed=False, label="UInt16")}, name="ACL", pack=False, align=None), offset=0)], SimTypeInt(signed=False, label="UInt32"), arg_names=["engineHandle", "securityInfo", "sidOwner", "sidGroup", "dacl", "sacl"]),
        # 
        'IPsecKeyManagerAddAndRegister0': SimTypeFunction([SimTypePointer(SimTypeInt(signed=True, label="Int"), label="IntPtr", offset=0), SimTypePointer(SimStruct({"keyManagerKey": SimTypeBottom(label="Guid"), "displayData": SimStruct({"name": SimTypePointer(SimTypeChar(label="Char"), offset=0), "description": SimTypePointer(SimTypeChar(label="Char"), offset=0)}, name="FWPM_DISPLAY_DATA0", pack=False, align=None), "flags": SimTypeInt(signed=False, label="UInt32"), "keyDictationTimeoutHint": SimTypeChar(label="Byte")}, name="IPSEC_KEY_MANAGER0", pack=False, align=None), offset=0), SimTypePointer(SimStruct({"reserved": SimTypeBottom(label="Guid"), "flags": SimTypeInt(signed=False, label="UInt32"), "keyDictationCheck": SimTypePointer(SimTypeFunction([SimTypePointer(SimStruct({"ipVersion": SimTypeInt(signed=False, label="FWP_IP_VERSION"), "Anonymous1": SimUnion({"localV4Address": SimTypeInt(signed=False, label="UInt32"), "localV6Address": SimTypeFixedSizeArray(SimTypeChar(label="Byte"), 16)}, name="<anon>", label="None"), "Anonymous2": SimUnion({"remoteV4Address": SimTypeInt(signed=False, label="UInt32"), "remoteV6Address": SimTypeFixedSizeArray(SimTypeChar(label="Byte"), 16)}, name="<anon>", label="None"), "authIpFilterId": SimTypeLongLong(signed=False, label="UInt64")}, name="IKEEXT_TRAFFIC0", pack=False, align=None), offset=0), SimTypePointer(SimTypeInt(signed=True, label="Int32"), offset=0), SimTypePointer(SimTypeInt(signed=False, label="UInt32"), offset=0)], SimTypeBottom(label="Void"), arg_names=["ikeTraffic", "willDictateKey", "weight"]), offset=0), "keyDictation": SimTypePointer(SimTypeFunction([SimTypePointer(SimStruct({"ipVersion": SimTypeInt(signed=False, label="FWP_IP_VERSION"), "saDirection": SimTypeInt(signed=False, label="FWP_DIRECTION"), "traffic": SimStruct({"ipVersion": SimTypeInt(signed=False, label="FWP_IP_VERSION"), "Anonymous1": SimUnion({"localV4Address": SimTypeInt(signed=False, label="UInt32"), "localV6Address": SimTypeFixedSizeArray(SimTypeChar(label="Byte"), 16)}, name="<anon>", label="None"), "Anonymous2": SimUnion({"remoteV4Address": SimTypeInt(signed=False, label="UInt32"), "remoteV6Address": SimTypeFixedSizeArray(SimTypeChar(label="Byte"), 16)}, name="<anon>", label="None"), "trafficType": SimTypeInt(signed=False, label="IPSEC_TRAFFIC_TYPE"), "Anonymous3": SimUnion({"ipsecFilterId": SimTypeLongLong(signed=False, label="UInt64"), "tunnelPolicyId": SimTypeLongLong(signed=False, label="UInt64")}, name="<anon>", label="None"), "remotePort": SimTypeShort(signed=False, label="UInt16"), "localPort": SimTypeShort(signed=False, label="UInt16"), "ipProtocol": SimTypeChar(label="Byte"), "localIfLuid": SimTypeLongLong(signed=False, label="UInt64"), "realIfProfileId": SimTypeInt(signed=False, label="UInt32")}, name="IPSEC_TRAFFIC1", pack=False, align=None), "saBundle": SimStruct({"flags": SimTypeInt(signed=False, label="IPSEC_SA_BUNDLE_FLAGS"), "lifetime": SimStruct({"lifetimeSeconds": SimTypeInt(signed=False, label="UInt32"), "lifetimeKilobytes": SimTypeInt(signed=False, label="UInt32"), "lifetimePackets": SimTypeInt(signed=False, label="UInt32")}, name="IPSEC_SA_LIFETIME0", pack=False, align=None), "idleTimeoutSeconds": SimTypeInt(signed=False, label="UInt32"), "ndAllowClearTimeoutSeconds": SimTypeInt(signed=False, label="UInt32"), "ipsecId": SimTypePointer(SimStruct({"mmTargetName": SimTypePointer(SimTypeChar(label="Char"), offset=0), "emTargetName": SimTypePointer(SimTypeChar(label="Char"), offset=0), "numTokens": SimTypeInt(signed=False, label="UInt32"), "tokens": SimTypePointer(SimStruct({"type": SimTypeInt(signed=False, label="IPSEC_TOKEN_TYPE"), "principal": SimTypeInt(signed=False, label="IPSEC_TOKEN_PRINCIPAL"), "mode": SimTypeInt(signed=False, label="IPSEC_TOKEN_MODE"), "token": SimTypeLongLong(signed=False, label="UInt64")}, name="IPSEC_TOKEN0", pack=False, align=None), offset=0), "explicitCredentials": SimTypeLongLong(signed=False, label="UInt64"), "logonId": SimTypeLongLong(signed=False, label="UInt64")}, name="IPSEC_ID0", pack=False, align=None), offset=0), "napContext": SimTypeInt(signed=False, label="UInt32"), "qmSaId": SimTypeInt(signed=False, label="UInt32"), "numSAs": SimTypeInt(signed=False, label="UInt32"), "saList": SimTypePointer(SimStruct({"spi": SimTypeInt(signed=False, label="UInt32"), "saTransformType": SimTypeInt(signed=False, label="IPSEC_TRANSFORM_TYPE"), "Anonymous": SimUnion({"ahInformation": SimTypePointer(SimStruct({"authTransform": SimStruct({"authTransformId": SimStruct({"authType": SimTypeInt(signed=False, label="IPSEC_AUTH_TYPE"), "authConfig": SimTypeChar(label="Byte")}, name="IPSEC_AUTH_TRANSFORM_ID0", pack=False, align=None), "cryptoModuleId": SimTypePointer(SimTypeBottom(label="Guid"), offset=0)}, name="IPSEC_AUTH_TRANSFORM0", pack=False, align=None), "authKey": SimStruct({"size": SimTypeInt(signed=False, label="UInt32"), "data": SimTypePointer(SimTypeChar(label="Byte"), offset=0)}, name="FWP_BYTE_BLOB", pack=False, align=None)}, name="IPSEC_SA_AUTH_INFORMATION0", pack=False, align=None), offset=0), "espAuthInformation": SimTypePointer(SimStruct({"authTransform": SimStruct({"authTransformId": SimStruct({"authType": SimTypeInt(signed=False, label="IPSEC_AUTH_TYPE"), "authConfig": SimTypeChar(label="Byte")}, name="IPSEC_AUTH_TRANSFORM_ID0", pack=False, align=None), "cryptoModuleId": SimTypePointer(SimTypeBottom(label="Guid"), offset=0)}, name="IPSEC_AUTH_TRANSFORM0", pack=False, align=None), "authKey": SimStruct({"size": SimTypeInt(signed=False, label="UInt32"), "data": SimTypePointer(SimTypeChar(label="Byte"), offset=0)}, name="FWP_BYTE_BLOB", pack=False, align=None)}, name="IPSEC_SA_AUTH_INFORMATION0", pack=False, align=None), offset=0), "espCipherInformation": SimTypePointer(SimStruct({"cipherTransform": SimStruct({"cipherTransformId": SimStruct({"cipherType": SimTypeInt(signed=False, label="IPSEC_CIPHER_TYPE"), "cipherConfig": SimTypeChar(label="Byte")}, name="IPSEC_CIPHER_TRANSFORM_ID0", pack=False, align=None), "cryptoModuleId": SimTypePointer(SimTypeBottom(label="Guid"), offset=0)}, name="IPSEC_CIPHER_TRANSFORM0", pack=False, align=None), "cipherKey": SimStruct({"size": SimTypeInt(signed=False, label="UInt32"), "data": SimTypePointer(SimTypeChar(label="Byte"), offset=0)}, name="FWP_BYTE_BLOB", pack=False, align=None)}, name="IPSEC_SA_CIPHER_INFORMATION0", pack=False, align=None), offset=0), "espAuthAndCipherInformation": SimTypePointer(SimStruct({"saCipherInformation": SimStruct({"cipherTransform": SimStruct({"cipherTransformId": SimStruct({"cipherType": SimTypeInt(signed=False, label="IPSEC_CIPHER_TYPE"), "cipherConfig": SimTypeChar(label="Byte")}, name="IPSEC_CIPHER_TRANSFORM_ID0", pack=False, align=None), "cryptoModuleId": SimTypePointer(SimTypeBottom(label="Guid"), offset=0)}, name="IPSEC_CIPHER_TRANSFORM0", pack=False, align=None), "cipherKey": SimStruct({"size": SimTypeInt(signed=False, label="UInt32"), "data": SimTypePointer(SimTypeChar(label="Byte"), offset=0)}, name="FWP_BYTE_BLOB", pack=False, align=None)}, name="IPSEC_SA_CIPHER_INFORMATION0", pack=False, align=None), "saAuthInformation": SimStruct({"authTransform": SimStruct({"authTransformId": SimStruct({"authType": SimTypeInt(signed=False, label="IPSEC_AUTH_TYPE"), "authConfig": SimTypeChar(label="Byte")}, name="IPSEC_AUTH_TRANSFORM_ID0", pack=False, align=None), "cryptoModuleId": SimTypePointer(SimTypeBottom(label="Guid"), offset=0)}, name="IPSEC_AUTH_TRANSFORM0", pack=False, align=None), "authKey": SimStruct({"size": SimTypeInt(signed=False, label="UInt32"), "data": SimTypePointer(SimTypeChar(label="Byte"), offset=0)}, name="FWP_BYTE_BLOB", pack=False, align=None)}, name="IPSEC_SA_AUTH_INFORMATION0", pack=False, align=None)}, name="IPSEC_SA_AUTH_AND_CIPHER_INFORMATION0", pack=False, align=None), offset=0), "espAuthFwInformation": SimTypePointer(SimStruct({"authTransform": SimStruct({"authTransformId": SimStruct({"authType": SimTypeInt(signed=False, label="IPSEC_AUTH_TYPE"), "authConfig": SimTypeChar(label="Byte")}, name="IPSEC_AUTH_TRANSFORM_ID0", pack=False, align=None), "cryptoModuleId": SimTypePointer(SimTypeBottom(label="Guid"), offset=0)}, name="IPSEC_AUTH_TRANSFORM0", pack=False, align=None), "authKey": SimStruct({"size": SimTypeInt(signed=False, label="UInt32"), "data": SimTypePointer(SimTypeChar(label="Byte"), offset=0)}, name="FWP_BYTE_BLOB", pack=False, align=None)}, name="IPSEC_SA_AUTH_INFORMATION0", pack=False, align=None), offset=0)}, name="<anon>", label="None")}, name="IPSEC_SA0", pack=False, align=None), offset=0), "keyModuleState": SimTypePointer(SimStruct({"keyModuleKey": SimTypeBottom(label="Guid"), "stateBlob": SimStruct({"size": SimTypeInt(signed=False, label="UInt32"), "data": SimTypePointer(SimTypeChar(label="Byte"), offset=0)}, name="FWP_BYTE_BLOB", pack=False, align=None)}, name="IPSEC_KEYMODULE_STATE0", pack=False, align=None), offset=0), "ipVersion": SimTypeInt(signed=False, label="FWP_IP_VERSION"), "Anonymous": SimUnion({"peerV4PrivateAddress": SimTypeInt(signed=False, label="UInt32")}, name="<anon>", label="None"), "mmSaId": SimTypeLongLong(signed=False, label="UInt64"), "pfsGroup": SimTypeInt(signed=False, label="IPSEC_PFS_GROUP"), "saLookupContext": SimTypeBottom(label="Guid"), "qmFilterId": SimTypeLongLong(signed=False, label="UInt64")}, name="IPSEC_SA_BUNDLE1", pack=False, align=None), "Anonymous": SimUnion({"udpEncapsulation": SimTypePointer(SimStruct({"localUdpEncapPort": SimTypeShort(signed=False, label="UInt16"), "remoteUdpEncapPort": SimTypeShort(signed=False, label="UInt16")}, name="IPSEC_V4_UDP_ENCAPSULATION0", pack=False, align=None), offset=0)}, name="<anon>", label="None"), "transportFilter": SimTypePointer(SimTypeBottom(label="FWPM_FILTER0"), offset=0), "virtualIfTunnelInfo": SimStruct({"virtualIfTunnelId": SimTypeLongLong(signed=False, label="UInt64"), "trafficSelectorId": SimTypeLongLong(signed=False, label="UInt64")}, name="IPSEC_VIRTUAL_IF_TUNNEL_INFO0", pack=False, align=None)}, name="IPSEC_SA_DETAILS1", pack=False, align=None), offset=0), SimTypePointer(SimStruct({"ipVersion": SimTypeInt(signed=False, label="FWP_IP_VERSION"), "saDirection": SimTypeInt(signed=False, label="FWP_DIRECTION"), "traffic": SimStruct({"ipVersion": SimTypeInt(signed=False, label="FWP_IP_VERSION"), "Anonymous1": SimUnion({"localV4Address": SimTypeInt(signed=False, label="UInt32"), "localV6Address": SimTypeFixedSizeArray(SimTypeChar(label="Byte"), 16)}, name="<anon>", label="None"), "Anonymous2": SimUnion({"remoteV4Address": SimTypeInt(signed=False, label="UInt32"), "remoteV6Address": SimTypeFixedSizeArray(SimTypeChar(label="Byte"), 16)}, name="<anon>", label="None"), "trafficType": SimTypeInt(signed=False, label="IPSEC_TRAFFIC_TYPE"), "Anonymous3": SimUnion({"ipsecFilterId": SimTypeLongLong(signed=False, label="UInt64"), "tunnelPolicyId": SimTypeLongLong(signed=False, label="UInt64")}, name="<anon>", label="None"), "remotePort": SimTypeShort(signed=False, label="UInt16"), "localPort": SimTypeShort(signed=False, label="UInt16"), "ipProtocol": SimTypeChar(label="Byte"), "localIfLuid": SimTypeLongLong(signed=False, label="UInt64"), "realIfProfileId": SimTypeInt(signed=False, label="UInt32")}, name="IPSEC_TRAFFIC1", pack=False, align=None), "saBundle": SimStruct({"flags": SimTypeInt(signed=False, label="IPSEC_SA_BUNDLE_FLAGS"), "lifetime": SimStruct({"lifetimeSeconds": SimTypeInt(signed=False, label="UInt32"), "lifetimeKilobytes": SimTypeInt(signed=False, label="UInt32"), "lifetimePackets": SimTypeInt(signed=False, label="UInt32")}, name="IPSEC_SA_LIFETIME0", pack=False, align=None), "idleTimeoutSeconds": SimTypeInt(signed=False, label="UInt32"), "ndAllowClearTimeoutSeconds": SimTypeInt(signed=False, label="UInt32"), "ipsecId": SimTypePointer(SimStruct({"mmTargetName": SimTypePointer(SimTypeChar(label="Char"), offset=0), "emTargetName": SimTypePointer(SimTypeChar(label="Char"), offset=0), "numTokens": SimTypeInt(signed=False, label="UInt32"), "tokens": SimTypePointer(SimStruct({"type": SimTypeInt(signed=False, label="IPSEC_TOKEN_TYPE"), "principal": SimTypeInt(signed=False, label="IPSEC_TOKEN_PRINCIPAL"), "mode": SimTypeInt(signed=False, label="IPSEC_TOKEN_MODE"), "token": SimTypeLongLong(signed=False, label="UInt64")}, name="IPSEC_TOKEN0", pack=False, align=None), offset=0), "explicitCredentials": SimTypeLongLong(signed=False, label="UInt64"), "logonId": SimTypeLongLong(signed=False, label="UInt64")}, name="IPSEC_ID0", pack=False, align=None), offset=0), "napContext": SimTypeInt(signed=False, label="UInt32"), "qmSaId": SimTypeInt(signed=False, label="UInt32"), "numSAs": SimTypeInt(signed=False, label="UInt32"), "saList": SimTypePointer(SimStruct({"spi": SimTypeInt(signed=False, label="UInt32"), "saTransformType": SimTypeInt(signed=False, label="IPSEC_TRANSFORM_TYPE"), "Anonymous": SimUnion({"ahInformation": SimTypePointer(SimStruct({"authTransform": SimStruct({"authTransformId": SimStruct({"authType": SimTypeInt(signed=False, label="IPSEC_AUTH_TYPE"), "authConfig": SimTypeChar(label="Byte")}, name="IPSEC_AUTH_TRANSFORM_ID0", pack=False, align=None), "cryptoModuleId": SimTypePointer(SimTypeBottom(label="Guid"), offset=0)}, name="IPSEC_AUTH_TRANSFORM0", pack=False, align=None), "authKey": SimStruct({"size": SimTypeInt(signed=False, label="UInt32"), "data": SimTypePointer(SimTypeChar(label="Byte"), offset=0)}, name="FWP_BYTE_BLOB", pack=False, align=None)}, name="IPSEC_SA_AUTH_INFORMATION0", pack=False, align=None), offset=0), "espAuthInformation": SimTypePointer(SimStruct({"authTransform": SimStruct({"authTransformId": SimStruct({"authType": SimTypeInt(signed=False, label="IPSEC_AUTH_TYPE"), "authConfig": SimTypeChar(label="Byte")}, name="IPSEC_AUTH_TRANSFORM_ID0", pack=False, align=None), "cryptoModuleId": SimTypePointer(SimTypeBottom(label="Guid"), offset=0)}, name="IPSEC_AUTH_TRANSFORM0", pack=False, align=None), "authKey": SimStruct({"size": SimTypeInt(signed=False, label="UInt32"), "data": SimTypePointer(SimTypeChar(label="Byte"), offset=0)}, name="FWP_BYTE_BLOB", pack=False, align=None)}, name="IPSEC_SA_AUTH_INFORMATION0", pack=False, align=None), offset=0), "espCipherInformation": SimTypePointer(SimStruct({"cipherTransform": SimStruct({"cipherTransformId": SimStruct({"cipherType": SimTypeInt(signed=False, label="IPSEC_CIPHER_TYPE"), "cipherConfig": SimTypeChar(label="Byte")}, name="IPSEC_CIPHER_TRANSFORM_ID0", pack=False, align=None), "cryptoModuleId": SimTypePointer(SimTypeBottom(label="Guid"), offset=0)}, name="IPSEC_CIPHER_TRANSFORM0", pack=False, align=None), "cipherKey": SimStruct({"size": SimTypeInt(signed=False, label="UInt32"), "data": SimTypePointer(SimTypeChar(label="Byte"), offset=0)}, name="FWP_BYTE_BLOB", pack=False, align=None)}, name="IPSEC_SA_CIPHER_INFORMATION0", pack=False, align=None), offset=0), "espAuthAndCipherInformation": SimTypePointer(SimStruct({"saCipherInformation": SimStruct({"cipherTransform": SimStruct({"cipherTransformId": SimStruct({"cipherType": SimTypeInt(signed=False, label="IPSEC_CIPHER_TYPE"), "cipherConfig": SimTypeChar(label="Byte")}, name="IPSEC_CIPHER_TRANSFORM_ID0", pack=False, align=None), "cryptoModuleId": SimTypePointer(SimTypeBottom(label="Guid"), offset=0)}, name="IPSEC_CIPHER_TRANSFORM0", pack=False, align=None), "cipherKey": SimStruct({"size": SimTypeInt(signed=False, label="UInt32"), "data": SimTypePointer(SimTypeChar(label="Byte"), offset=0)}, name="FWP_BYTE_BLOB", pack=False, align=None)}, name="IPSEC_SA_CIPHER_INFORMATION0", pack=False, align=None), "saAuthInformation": SimStruct({"authTransform": SimStruct({"authTransformId": SimStruct({"authType": SimTypeInt(signed=False, label="IPSEC_AUTH_TYPE"), "authConfig": SimTypeChar(label="Byte")}, name="IPSEC_AUTH_TRANSFORM_ID0", pack=False, align=None), "cryptoModuleId": SimTypePointer(SimTypeBottom(label="Guid"), offset=0)}, name="IPSEC_AUTH_TRANSFORM0", pack=False, align=None), "authKey": SimStruct({"size": SimTypeInt(signed=False, label="UInt32"), "data": SimTypePointer(SimTypeChar(label="Byte"), offset=0)}, name="FWP_BYTE_BLOB", pack=False, align=None)}, name="IPSEC_SA_AUTH_INFORMATION0", pack=False, align=None)}, name="IPSEC_SA_AUTH_AND_CIPHER_INFORMATION0", pack=False, align=None), offset=0), "espAuthFwInformation": SimTypePointer(SimStruct({"authTransform": SimStruct({"authTransformId": SimStruct({"authType": SimTypeInt(signed=False, label="IPSEC_AUTH_TYPE"), "authConfig": SimTypeChar(label="Byte")}, name="IPSEC_AUTH_TRANSFORM_ID0", pack=False, align=None), "cryptoModuleId": SimTypePointer(SimTypeBottom(label="Guid"), offset=0)}, name="IPSEC_AUTH_TRANSFORM0", pack=False, align=None), "authKey": SimStruct({"size": SimTypeInt(signed=False, label="UInt32"), "data": SimTypePointer(SimTypeChar(label="Byte"), offset=0)}, name="FWP_BYTE_BLOB", pack=False, align=None)}, name="IPSEC_SA_AUTH_INFORMATION0", pack=False, align=None), offset=0)}, name="<anon>", label="None")}, name="IPSEC_SA0", pack=False, align=None), offset=0), "keyModuleState": SimTypePointer(SimStruct({"keyModuleKey": SimTypeBottom(label="Guid"), "stateBlob": SimStruct({"size": SimTypeInt(signed=False, label="UInt32"), "data": SimTypePointer(SimTypeChar(label="Byte"), offset=0)}, name="FWP_BYTE_BLOB", pack=False, align=None)}, name="IPSEC_KEYMODULE_STATE0", pack=False, align=None), offset=0), "ipVersion": SimTypeInt(signed=False, label="FWP_IP_VERSION"), "Anonymous": SimUnion({"peerV4PrivateAddress": SimTypeInt(signed=False, label="UInt32")}, name="<anon>", label="None"), "mmSaId": SimTypeLongLong(signed=False, label="UInt64"), "pfsGroup": SimTypeInt(signed=False, label="IPSEC_PFS_GROUP"), "saLookupContext": SimTypeBottom(label="Guid"), "qmFilterId": SimTypeLongLong(signed=False, label="UInt64")}, name="IPSEC_SA_BUNDLE1", pack=False, align=None), "Anonymous": SimUnion({"udpEncapsulation": SimTypePointer(SimStruct({"localUdpEncapPort": SimTypeShort(signed=False, label="UInt16"), "remoteUdpEncapPort": SimTypeShort(signed=False, label="UInt16")}, name="IPSEC_V4_UDP_ENCAPSULATION0", pack=False, align=None), offset=0)}, name="<anon>", label="None"), "transportFilter": SimTypePointer(SimTypeBottom(label="FWPM_FILTER0"), offset=0), "virtualIfTunnelInfo": SimStruct({"virtualIfTunnelId": SimTypeLongLong(signed=False, label="UInt64"), "trafficSelectorId": SimTypeLongLong(signed=False, label="UInt64")}, name="IPSEC_VIRTUAL_IF_TUNNEL_INFO0", pack=False, align=None)}, name="IPSEC_SA_DETAILS1", pack=False, align=None), offset=0), SimTypePointer(SimTypeInt(signed=True, label="Int32"), offset=0)], SimTypeInt(signed=False, label="UInt32"), arg_names=["inboundSaDetails", "outboundSaDetails", "keyingModuleGenKey"]), offset=0), "keyNotify": SimTypePointer(SimTypeFunction([SimTypePointer(SimStruct({"ipVersion": SimTypeInt(signed=False, label="FWP_IP_VERSION"), "saDirection": SimTypeInt(signed=False, label="FWP_DIRECTION"), "traffic": SimStruct({"ipVersion": SimTypeInt(signed=False, label="FWP_IP_VERSION"), "Anonymous1": SimUnion({"localV4Address": SimTypeInt(signed=False, label="UInt32"), "localV6Address": SimTypeFixedSizeArray(SimTypeChar(label="Byte"), 16)}, name="<anon>", label="None"), "Anonymous2": SimUnion({"remoteV4Address": SimTypeInt(signed=False, label="UInt32"), "remoteV6Address": SimTypeFixedSizeArray(SimTypeChar(label="Byte"), 16)}, name="<anon>", label="None"), "trafficType": SimTypeInt(signed=False, label="IPSEC_TRAFFIC_TYPE"), "Anonymous3": SimUnion({"ipsecFilterId": SimTypeLongLong(signed=False, label="UInt64"), "tunnelPolicyId": SimTypeLongLong(signed=False, label="UInt64")}, name="<anon>", label="None"), "remotePort": SimTypeShort(signed=False, label="UInt16"), "localPort": SimTypeShort(signed=False, label="UInt16"), "ipProtocol": SimTypeChar(label="Byte"), "localIfLuid": SimTypeLongLong(signed=False, label="UInt64"), "realIfProfileId": SimTypeInt(signed=False, label="UInt32")}, name="IPSEC_TRAFFIC1", pack=False, align=None), "saBundle": SimStruct({"flags": SimTypeInt(signed=False, label="IPSEC_SA_BUNDLE_FLAGS"), "lifetime": SimStruct({"lifetimeSeconds": SimTypeInt(signed=False, label="UInt32"), "lifetimeKilobytes": SimTypeInt(signed=False, label="UInt32"), "lifetimePackets": SimTypeInt(signed=False, label="UInt32")}, name="IPSEC_SA_LIFETIME0", pack=False, align=None), "idleTimeoutSeconds": SimTypeInt(signed=False, label="UInt32"), "ndAllowClearTimeoutSeconds": SimTypeInt(signed=False, label="UInt32"), "ipsecId": SimTypePointer(SimStruct({"mmTargetName": SimTypePointer(SimTypeChar(label="Char"), offset=0), "emTargetName": SimTypePointer(SimTypeChar(label="Char"), offset=0), "numTokens": SimTypeInt(signed=False, label="UInt32"), "tokens": SimTypePointer(SimStruct({"type": SimTypeInt(signed=False, label="IPSEC_TOKEN_TYPE"), "principal": SimTypeInt(signed=False, label="IPSEC_TOKEN_PRINCIPAL"), "mode": SimTypeInt(signed=False, label="IPSEC_TOKEN_MODE"), "token": SimTypeLongLong(signed=False, label="UInt64")}, name="IPSEC_TOKEN0", pack=False, align=None), offset=0), "explicitCredentials": SimTypeLongLong(signed=False, label="UInt64"), "logonId": SimTypeLongLong(signed=False, label="UInt64")}, name="IPSEC_ID0", pack=False, align=None), offset=0), "napContext": SimTypeInt(signed=False, label="UInt32"), "qmSaId": SimTypeInt(signed=False, label="UInt32"), "numSAs": SimTypeInt(signed=False, label="UInt32"), "saList": SimTypePointer(SimStruct({"spi": SimTypeInt(signed=False, label="UInt32"), "saTransformType": SimTypeInt(signed=False, label="IPSEC_TRANSFORM_TYPE"), "Anonymous": SimUnion({"ahInformation": SimTypePointer(SimStruct({"authTransform": SimStruct({"authTransformId": SimStruct({"authType": SimTypeInt(signed=False, label="IPSEC_AUTH_TYPE"), "authConfig": SimTypeChar(label="Byte")}, name="IPSEC_AUTH_TRANSFORM_ID0", pack=False, align=None), "cryptoModuleId": SimTypePointer(SimTypeBottom(label="Guid"), offset=0)}, name="IPSEC_AUTH_TRANSFORM0", pack=False, align=None), "authKey": SimStruct({"size": SimTypeInt(signed=False, label="UInt32"), "data": SimTypePointer(SimTypeChar(label="Byte"), offset=0)}, name="FWP_BYTE_BLOB", pack=False, align=None)}, name="IPSEC_SA_AUTH_INFORMATION0", pack=False, align=None), offset=0), "espAuthInformation": SimTypePointer(SimStruct({"authTransform": SimStruct({"authTransformId": SimStruct({"authType": SimTypeInt(signed=False, label="IPSEC_AUTH_TYPE"), "authConfig": SimTypeChar(label="Byte")}, name="IPSEC_AUTH_TRANSFORM_ID0", pack=False, align=None), "cryptoModuleId": SimTypePointer(SimTypeBottom(label="Guid"), offset=0)}, name="IPSEC_AUTH_TRANSFORM0", pack=False, align=None), "authKey": SimStruct({"size": SimTypeInt(signed=False, label="UInt32"), "data": SimTypePointer(SimTypeChar(label="Byte"), offset=0)}, name="FWP_BYTE_BLOB", pack=False, align=None)}, name="IPSEC_SA_AUTH_INFORMATION0", pack=False, align=None), offset=0), "espCipherInformation": SimTypePointer(SimStruct({"cipherTransform": SimStruct({"cipherTransformId": SimStruct({"cipherType": SimTypeInt(signed=False, label="IPSEC_CIPHER_TYPE"), "cipherConfig": SimTypeChar(label="Byte")}, name="IPSEC_CIPHER_TRANSFORM_ID0", pack=False, align=None), "cryptoModuleId": SimTypePointer(SimTypeBottom(label="Guid"), offset=0)}, name="IPSEC_CIPHER_TRANSFORM0", pack=False, align=None), "cipherKey": SimStruct({"size": SimTypeInt(signed=False, label="UInt32"), "data": SimTypePointer(SimTypeChar(label="Byte"), offset=0)}, name="FWP_BYTE_BLOB", pack=False, align=None)}, name="IPSEC_SA_CIPHER_INFORMATION0", pack=False, align=None), offset=0), "espAuthAndCipherInformation": SimTypePointer(SimStruct({"saCipherInformation": SimStruct({"cipherTransform": SimStruct({"cipherTransformId": SimStruct({"cipherType": SimTypeInt(signed=False, label="IPSEC_CIPHER_TYPE"), "cipherConfig": SimTypeChar(label="Byte")}, name="IPSEC_CIPHER_TRANSFORM_ID0", pack=False, align=None), "cryptoModuleId": SimTypePointer(SimTypeBottom(label="Guid"), offset=0)}, name="IPSEC_CIPHER_TRANSFORM0", pack=False, align=None), "cipherKey": SimStruct({"size": SimTypeInt(signed=False, label="UInt32"), "data": SimTypePointer(SimTypeChar(label="Byte"), offset=0)}, name="FWP_BYTE_BLOB", pack=False, align=None)}, name="IPSEC_SA_CIPHER_INFORMATION0", pack=False, align=None), "saAuthInformation": SimStruct({"authTransform": SimStruct({"authTransformId": SimStruct({"authType": SimTypeInt(signed=False, label="IPSEC_AUTH_TYPE"), "authConfig": SimTypeChar(label="Byte")}, name="IPSEC_AUTH_TRANSFORM_ID0", pack=False, align=None), "cryptoModuleId": SimTypePointer(SimTypeBottom(label="Guid"), offset=0)}, name="IPSEC_AUTH_TRANSFORM0", pack=False, align=None), "authKey": SimStruct({"size": SimTypeInt(signed=False, label="UInt32"), "data": SimTypePointer(SimTypeChar(label="Byte"), offset=0)}, name="FWP_BYTE_BLOB", pack=False, align=None)}, name="IPSEC_SA_AUTH_INFORMATION0", pack=False, align=None)}, name="IPSEC_SA_AUTH_AND_CIPHER_INFORMATION0", pack=False, align=None), offset=0), "espAuthFwInformation": SimTypePointer(SimStruct({"authTransform": SimStruct({"authTransformId": SimStruct({"authType": SimTypeInt(signed=False, label="IPSEC_AUTH_TYPE"), "authConfig": SimTypeChar(label="Byte")}, name="IPSEC_AUTH_TRANSFORM_ID0", pack=False, align=None), "cryptoModuleId": SimTypePointer(SimTypeBottom(label="Guid"), offset=0)}, name="IPSEC_AUTH_TRANSFORM0", pack=False, align=None), "authKey": SimStruct({"size": SimTypeInt(signed=False, label="UInt32"), "data": SimTypePointer(SimTypeChar(label="Byte"), offset=0)}, name="FWP_BYTE_BLOB", pack=False, align=None)}, name="IPSEC_SA_AUTH_INFORMATION0", pack=False, align=None), offset=0)}, name="<anon>", label="None")}, name="IPSEC_SA0", pack=False, align=None), offset=0), "keyModuleState": SimTypePointer(SimStruct({"keyModuleKey": SimTypeBottom(label="Guid"), "stateBlob": SimStruct({"size": SimTypeInt(signed=False, label="UInt32"), "data": SimTypePointer(SimTypeChar(label="Byte"), offset=0)}, name="FWP_BYTE_BLOB", pack=False, align=None)}, name="IPSEC_KEYMODULE_STATE0", pack=False, align=None), offset=0), "ipVersion": SimTypeInt(signed=False, label="FWP_IP_VERSION"), "Anonymous": SimUnion({"peerV4PrivateAddress": SimTypeInt(signed=False, label="UInt32")}, name="<anon>", label="None"), "mmSaId": SimTypeLongLong(signed=False, label="UInt64"), "pfsGroup": SimTypeInt(signed=False, label="IPSEC_PFS_GROUP"), "saLookupContext": SimTypeBottom(label="Guid"), "qmFilterId": SimTypeLongLong(signed=False, label="UInt64")}, name="IPSEC_SA_BUNDLE1", pack=False, align=None), "Anonymous": SimUnion({"udpEncapsulation": SimTypePointer(SimStruct({"localUdpEncapPort": SimTypeShort(signed=False, label="UInt16"), "remoteUdpEncapPort": SimTypeShort(signed=False, label="UInt16")}, name="IPSEC_V4_UDP_ENCAPSULATION0", pack=False, align=None), offset=0)}, name="<anon>", label="None"), "transportFilter": SimTypePointer(SimTypeBottom(label="FWPM_FILTER0"), offset=0), "virtualIfTunnelInfo": SimStruct({"virtualIfTunnelId": SimTypeLongLong(signed=False, label="UInt64"), "trafficSelectorId": SimTypeLongLong(signed=False, label="UInt64")}, name="IPSEC_VIRTUAL_IF_TUNNEL_INFO0", pack=False, align=None)}, name="IPSEC_SA_DETAILS1", pack=False, align=None), offset=0), SimTypePointer(SimStruct({"ipVersion": SimTypeInt(signed=False, label="FWP_IP_VERSION"), "saDirection": SimTypeInt(signed=False, label="FWP_DIRECTION"), "traffic": SimStruct({"ipVersion": SimTypeInt(signed=False, label="FWP_IP_VERSION"), "Anonymous1": SimUnion({"localV4Address": SimTypeInt(signed=False, label="UInt32"), "localV6Address": SimTypeFixedSizeArray(SimTypeChar(label="Byte"), 16)}, name="<anon>", label="None"), "Anonymous2": SimUnion({"remoteV4Address": SimTypeInt(signed=False, label="UInt32"), "remoteV6Address": SimTypeFixedSizeArray(SimTypeChar(label="Byte"), 16)}, name="<anon>", label="None"), "trafficType": SimTypeInt(signed=False, label="IPSEC_TRAFFIC_TYPE"), "Anonymous3": SimUnion({"ipsecFilterId": SimTypeLongLong(signed=False, label="UInt64"), "tunnelPolicyId": SimTypeLongLong(signed=False, label="UInt64")}, name="<anon>", label="None"), "remotePort": SimTypeShort(signed=False, label="UInt16"), "localPort": SimTypeShort(signed=False, label="UInt16"), "ipProtocol": SimTypeChar(label="Byte"), "localIfLuid": SimTypeLongLong(signed=False, label="UInt64"), "realIfProfileId": SimTypeInt(signed=False, label="UInt32")}, name="IPSEC_TRAFFIC1", pack=False, align=None), "saBundle": SimStruct({"flags": SimTypeInt(signed=False, label="IPSEC_SA_BUNDLE_FLAGS"), "lifetime": SimStruct({"lifetimeSeconds": SimTypeInt(signed=False, label="UInt32"), "lifetimeKilobytes": SimTypeInt(signed=False, label="UInt32"), "lifetimePackets": SimTypeInt(signed=False, label="UInt32")}, name="IPSEC_SA_LIFETIME0", pack=False, align=None), "idleTimeoutSeconds": SimTypeInt(signed=False, label="UInt32"), "ndAllowClearTimeoutSeconds": SimTypeInt(signed=False, label="UInt32"), "ipsecId": SimTypePointer(SimStruct({"mmTargetName": SimTypePointer(SimTypeChar(label="Char"), offset=0), "emTargetName": SimTypePointer(SimTypeChar(label="Char"), offset=0), "numTokens": SimTypeInt(signed=False, label="UInt32"), "tokens": SimTypePointer(SimStruct({"type": SimTypeInt(signed=False, label="IPSEC_TOKEN_TYPE"), "principal": SimTypeInt(signed=False, label="IPSEC_TOKEN_PRINCIPAL"), "mode": SimTypeInt(signed=False, label="IPSEC_TOKEN_MODE"), "token": SimTypeLongLong(signed=False, label="UInt64")}, name="IPSEC_TOKEN0", pack=False, align=None), offset=0), "explicitCredentials": SimTypeLongLong(signed=False, label="UInt64"), "logonId": SimTypeLongLong(signed=False, label="UInt64")}, name="IPSEC_ID0", pack=False, align=None), offset=0), "napContext": SimTypeInt(signed=False, label="UInt32"), "qmSaId": SimTypeInt(signed=False, label="UInt32"), "numSAs": SimTypeInt(signed=False, label="UInt32"), "saList": SimTypePointer(SimStruct({"spi": SimTypeInt(signed=False, label="UInt32"), "saTransformType": SimTypeInt(signed=False, label="IPSEC_TRANSFORM_TYPE"), "Anonymous": SimUnion({"ahInformation": SimTypePointer(SimStruct({"authTransform": SimStruct({"authTransformId": SimStruct({"authType": SimTypeInt(signed=False, label="IPSEC_AUTH_TYPE"), "authConfig": SimTypeChar(label="Byte")}, name="IPSEC_AUTH_TRANSFORM_ID0", pack=False, align=None), "cryptoModuleId": SimTypePointer(SimTypeBottom(label="Guid"), offset=0)}, name="IPSEC_AUTH_TRANSFORM0", pack=False, align=None), "authKey": SimStruct({"size": SimTypeInt(signed=False, label="UInt32"), "data": SimTypePointer(SimTypeChar(label="Byte"), offset=0)}, name="FWP_BYTE_BLOB", pack=False, align=None)}, name="IPSEC_SA_AUTH_INFORMATION0", pack=False, align=None), offset=0), "espAuthInformation": SimTypePointer(SimStruct({"authTransform": SimStruct({"authTransformId": SimStruct({"authType": SimTypeInt(signed=False, label="IPSEC_AUTH_TYPE"), "authConfig": SimTypeChar(label="Byte")}, name="IPSEC_AUTH_TRANSFORM_ID0", pack=False, align=None), "cryptoModuleId": SimTypePointer(SimTypeBottom(label="Guid"), offset=0)}, name="IPSEC_AUTH_TRANSFORM0", pack=False, align=None), "authKey": SimStruct({"size": SimTypeInt(signed=False, label="UInt32"), "data": SimTypePointer(SimTypeChar(label="Byte"), offset=0)}, name="FWP_BYTE_BLOB", pack=False, align=None)}, name="IPSEC_SA_AUTH_INFORMATION0", pack=False, align=None), offset=0), "espCipherInformation": SimTypePointer(SimStruct({"cipherTransform": SimStruct({"cipherTransformId": SimStruct({"cipherType": SimTypeInt(signed=False, label="IPSEC_CIPHER_TYPE"), "cipherConfig": SimTypeChar(label="Byte")}, name="IPSEC_CIPHER_TRANSFORM_ID0", pack=False, align=None), "cryptoModuleId": SimTypePointer(SimTypeBottom(label="Guid"), offset=0)}, name="IPSEC_CIPHER_TRANSFORM0", pack=False, align=None), "cipherKey": SimStruct({"size": SimTypeInt(signed=False, label="UInt32"), "data": SimTypePointer(SimTypeChar(label="Byte"), offset=0)}, name="FWP_BYTE_BLOB", pack=False, align=None)}, name="IPSEC_SA_CIPHER_INFORMATION0", pack=False, align=None), offset=0), "espAuthAndCipherInformation": SimTypePointer(SimStruct({"saCipherInformation": SimStruct({"cipherTransform": SimStruct({"cipherTransformId": SimStruct({"cipherType": SimTypeInt(signed=False, label="IPSEC_CIPHER_TYPE"), "cipherConfig": SimTypeChar(label="Byte")}, name="IPSEC_CIPHER_TRANSFORM_ID0", pack=False, align=None), "cryptoModuleId": SimTypePointer(SimTypeBottom(label="Guid"), offset=0)}, name="IPSEC_CIPHER_TRANSFORM0", pack=False, align=None), "cipherKey": SimStruct({"size": SimTypeInt(signed=False, label="UInt32"), "data": SimTypePointer(SimTypeChar(label="Byte"), offset=0)}, name="FWP_BYTE_BLOB", pack=False, align=None)}, name="IPSEC_SA_CIPHER_INFORMATION0", pack=False, align=None), "saAuthInformation": SimStruct({"authTransform": SimStruct({"authTransformId": SimStruct({"authType": SimTypeInt(signed=False, label="IPSEC_AUTH_TYPE"), "authConfig": SimTypeChar(label="Byte")}, name="IPSEC_AUTH_TRANSFORM_ID0", pack=False, align=None), "cryptoModuleId": SimTypePointer(SimTypeBottom(label="Guid"), offset=0)}, name="IPSEC_AUTH_TRANSFORM0", pack=False, align=None), "authKey": SimStruct({"size": SimTypeInt(signed=False, label="UInt32"), "data": SimTypePointer(SimTypeChar(label="Byte"), offset=0)}, name="FWP_BYTE_BLOB", pack=False, align=None)}, name="IPSEC_SA_AUTH_INFORMATION0", pack=False, align=None)}, name="IPSEC_SA_AUTH_AND_CIPHER_INFORMATION0", pack=False, align=None), offset=0), "espAuthFwInformation": SimTypePointer(SimStruct({"authTransform": SimStruct({"authTransformId": SimStruct({"authType": SimTypeInt(signed=False, label="IPSEC_AUTH_TYPE"), "authConfig": SimTypeChar(label="Byte")}, name="IPSEC_AUTH_TRANSFORM_ID0", pack=False, align=None), "cryptoModuleId": SimTypePointer(SimTypeBottom(label="Guid"), offset=0)}, name="IPSEC_AUTH_TRANSFORM0", pack=False, align=None), "authKey": SimStruct({"size": SimTypeInt(signed=False, label="UInt32"), "data": SimTypePointer(SimTypeChar(label="Byte"), offset=0)}, name="FWP_BYTE_BLOB", pack=False, align=None)}, name="IPSEC_SA_AUTH_INFORMATION0", pack=False, align=None), offset=0)}, name="<anon>", label="None")}, name="IPSEC_SA0", pack=False, align=None), offset=0), "keyModuleState": SimTypePointer(SimStruct({"keyModuleKey": SimTypeBottom(label="Guid"), "stateBlob": SimStruct({"size": SimTypeInt(signed=False, label="UInt32"), "data": SimTypePointer(SimTypeChar(label="Byte"), offset=0)}, name="FWP_BYTE_BLOB", pack=False, align=None)}, name="IPSEC_KEYMODULE_STATE0", pack=False, align=None), offset=0), "ipVersion": SimTypeInt(signed=False, label="FWP_IP_VERSION"), "Anonymous": SimUnion({"peerV4PrivateAddress": SimTypeInt(signed=False, label="UInt32")}, name="<anon>", label="None"), "mmSaId": SimTypeLongLong(signed=False, label="UInt64"), "pfsGroup": SimTypeInt(signed=False, label="IPSEC_PFS_GROUP"), "saLookupContext": SimTypeBottom(label="Guid"), "qmFilterId": SimTypeLongLong(signed=False, label="UInt64")}, name="IPSEC_SA_BUNDLE1", pack=False, align=None), "Anonymous": SimUnion({"udpEncapsulation": SimTypePointer(SimStruct({"localUdpEncapPort": SimTypeShort(signed=False, label="UInt16"), "remoteUdpEncapPort": SimTypeShort(signed=False, label="UInt16")}, name="IPSEC_V4_UDP_ENCAPSULATION0", pack=False, align=None), offset=0)}, name="<anon>", label="None"), "transportFilter": SimTypePointer(SimTypeBottom(label="FWPM_FILTER0"), offset=0), "virtualIfTunnelInfo": SimStruct({"virtualIfTunnelId": SimTypeLongLong(signed=False, label="UInt64"), "trafficSelectorId": SimTypeLongLong(signed=False, label="UInt64")}, name="IPSEC_VIRTUAL_IF_TUNNEL_INFO0", pack=False, align=None)}, name="IPSEC_SA_DETAILS1", pack=False, align=None), offset=0)], SimTypeBottom(label="Void"), arg_names=["inboundSa", "outboundSa"]), offset=0)}, name="IPSEC_KEY_MANAGER_CALLBACKS0", pack=False, align=None), offset=0), SimTypePointer(SimTypePointer(SimTypeInt(signed=True, label="Int"), label="IntPtr", offset=0), offset=0)], SimTypeInt(signed=False, label="UInt32"), arg_names=["engineHandle", "keyManager", "keyManagerCallbacks", "keyMgmtHandle"]),
        # 
        'IPsecKeyManagerUnregisterAndDelete0': SimTypeFunction([SimTypePointer(SimTypeInt(signed=True, label="Int"), label="IntPtr", offset=0), SimTypePointer(SimTypeInt(signed=True, label="Int"), label="IntPtr", offset=0)], SimTypeInt(signed=False, label="UInt32"), arg_names=["engineHandle", "keyMgmtHandle"]),
        # 
        'IPsecKeyManagersGet0': SimTypeFunction([SimTypePointer(SimTypeInt(signed=True, label="Int"), label="IntPtr", offset=0), SimTypePointer(SimTypePointer(SimTypePointer(SimStruct({"keyManagerKey": SimTypeBottom(label="Guid"), "displayData": SimStruct({"name": SimTypePointer(SimTypeChar(label="Char"), offset=0), "description": SimTypePointer(SimTypeChar(label="Char"), offset=0)}, name="FWPM_DISPLAY_DATA0", pack=False, align=None), "flags": SimTypeInt(signed=False, label="UInt32"), "keyDictationTimeoutHint": SimTypeChar(label="Byte")}, name="IPSEC_KEY_MANAGER0", pack=False, align=None), offset=0), offset=0), offset=0), SimTypePointer(SimTypeInt(signed=False, label="UInt32"), offset=0)], SimTypeInt(signed=False, label="UInt32"), arg_names=["engineHandle", "entries", "numEntries"]),
        # 
        'IPsecKeyManagerGetSecurityInfoByKey0': SimTypeFunction([SimTypePointer(SimTypeInt(signed=True, label="Int"), label="IntPtr", offset=0), SimTypePointer(SimTypeBottom(label="Void"), offset=0), SimTypeInt(signed=False, label="UInt32"), SimTypePointer(SimTypePointer(SimTypeInt(signed=True, label="Int"), label="IntPtr", offset=0), offset=0), SimTypePointer(SimTypePointer(SimTypeInt(signed=True, label="Int"), label="IntPtr", offset=0), offset=0), SimTypePointer(SimTypePointer(SimStruct({"AclRevision": SimTypeChar(label="Byte"), "Sbz1": SimTypeChar(label="Byte"), "AclSize": SimTypeShort(signed=False, label="UInt16"), "AceCount": SimTypeShort(signed=False, label="UInt16"), "Sbz2": SimTypeShort(signed=False, label="UInt16")}, name="ACL", pack=False, align=None), offset=0), offset=0), SimTypePointer(SimTypePointer(SimStruct({"AclRevision": SimTypeChar(label="Byte"), "Sbz1": SimTypeChar(label="Byte"), "AclSize": SimTypeShort(signed=False, label="UInt16"), "AceCount": SimTypeShort(signed=False, label="UInt16"), "Sbz2": SimTypeShort(signed=False, label="UInt16")}, name="ACL", pack=False, align=None), offset=0), offset=0), SimTypePointer(SimTypePointer(SimStruct({"Revision": SimTypeChar(label="Byte"), "Sbz1": SimTypeChar(label="Byte"), "Control": SimTypeShort(signed=False, label="UInt16"), "Owner": SimTypePointer(SimTypeInt(signed=True, label="Int"), label="IntPtr", offset=0), "Group": SimTypePointer(SimTypeInt(signed=True, label="Int"), label="IntPtr", offset=0), "Sacl": SimTypePointer(SimStruct({"AclRevision": SimTypeChar(label="Byte"), "Sbz1": SimTypeChar(label="Byte"), "AclSize": SimTypeShort(signed=False, label="UInt16"), "AceCount": SimTypeShort(signed=False, label="UInt16"), "Sbz2": SimTypeShort(signed=False, label="UInt16")}, name="ACL", pack=False, align=None), offset=0), "Dacl": SimTypePointer(SimStruct({"AclRevision": SimTypeChar(label="Byte"), "Sbz1": SimTypeChar(label="Byte"), "AclSize": SimTypeShort(signed=False, label="UInt16"), "AceCount": SimTypeShort(signed=False, label="UInt16"), "Sbz2": SimTypeShort(signed=False, label="UInt16")}, name="ACL", pack=False, align=None), offset=0)}, name="SECURITY_DESCRIPTOR", pack=False, align=None), offset=0), offset=0)], SimTypeInt(signed=False, label="UInt32"), arg_names=["engineHandle", "reserved", "securityInfo", "sidOwner", "sidGroup", "dacl", "sacl", "securityDescriptor"]),
        # 
        'IPsecKeyManagerSetSecurityInfoByKey0': SimTypeFunction([SimTypePointer(SimTypeInt(signed=True, label="Int"), label="IntPtr", offset=0), SimTypePointer(SimTypeBottom(label="Void"), offset=0), SimTypeInt(signed=False, label="UInt32"), SimTypePointer(SimStruct({"Revision": SimTypeChar(label="Byte"), "SubAuthorityCount": SimTypeChar(label="Byte"), "IdentifierAuthority": SimStruct({"Value": SimTypeFixedSizeArray(SimTypeChar(label="Byte"), 6)}, name="SID_IDENTIFIER_AUTHORITY", pack=False, align=None), "SubAuthority": SimTypePointer(SimTypeInt(signed=False, label="UInt32"), offset=0)}, name="SID", pack=False, align=None), offset=0), SimTypePointer(SimStruct({"Revision": SimTypeChar(label="Byte"), "SubAuthorityCount": SimTypeChar(label="Byte"), "IdentifierAuthority": SimStruct({"Value": SimTypeFixedSizeArray(SimTypeChar(label="Byte"), 6)}, name="SID_IDENTIFIER_AUTHORITY", pack=False, align=None), "SubAuthority": SimTypePointer(SimTypeInt(signed=False, label="UInt32"), offset=0)}, name="SID", pack=False, align=None), offset=0), SimTypePointer(SimStruct({"AclRevision": SimTypeChar(label="Byte"), "Sbz1": SimTypeChar(label="Byte"), "AclSize": SimTypeShort(signed=False, label="UInt16"), "AceCount": SimTypeShort(signed=False, label="UInt16"), "Sbz2": SimTypeShort(signed=False, label="UInt16")}, name="ACL", pack=False, align=None), offset=0), SimTypePointer(SimStruct({"AclRevision": SimTypeChar(label="Byte"), "Sbz1": SimTypeChar(label="Byte"), "AclSize": SimTypeShort(signed=False, label="UInt16"), "AceCount": SimTypeShort(signed=False, label="UInt16"), "Sbz2": SimTypeShort(signed=False, label="UInt16")}, name="ACL", pack=False, align=None), offset=0)], SimTypeInt(signed=False, label="UInt32"), arg_names=["engineHandle", "reserved", "securityInfo", "sidOwner", "sidGroup", "dacl", "sacl"]),
        # 
        'IkeextGetStatistics0': SimTypeFunction([SimTypePointer(SimTypeInt(signed=True, label="Int"), label="IntPtr", offset=0), SimTypePointer(SimStruct({"ikeStatistics": SimStruct({"v4Statistics": SimStruct({"currentActiveMainModes": SimTypeInt(signed=False, label="UInt32"), "totalMainModesStarted": SimTypeInt(signed=False, label="UInt32"), "totalSuccessfulMainModes": SimTypeInt(signed=False, label="UInt32"), "totalFailedMainModes": SimTypeInt(signed=False, label="UInt32"), "totalResponderMainModes": SimTypeInt(signed=False, label="UInt32"), "currentNewResponderMainModes": SimTypeInt(signed=False, label="UInt32"), "currentActiveQuickModes": SimTypeInt(signed=False, label="UInt32"), "totalQuickModesStarted": SimTypeInt(signed=False, label="UInt32"), "totalSuccessfulQuickModes": SimTypeInt(signed=False, label="UInt32"), "totalFailedQuickModes": SimTypeInt(signed=False, label="UInt32"), "totalAcquires": SimTypeInt(signed=False, label="UInt32"), "totalReinitAcquires": SimTypeInt(signed=False, label="UInt32"), "currentActiveExtendedModes": SimTypeInt(signed=False, label="UInt32"), "totalExtendedModesStarted": SimTypeInt(signed=False, label="UInt32"), "totalSuccessfulExtendedModes": SimTypeInt(signed=False, label="UInt32"), "totalFailedExtendedModes": SimTypeInt(signed=False, label="UInt32"), "totalImpersonationExtendedModes": SimTypeInt(signed=False, label="UInt32"), "totalImpersonationMainModes": SimTypeInt(signed=False, label="UInt32")}, name="IKEEXT_IP_VERSION_SPECIFIC_KEYMODULE_STATISTICS0", pack=False, align=None), "v6Statistics": SimStruct({"currentActiveMainModes": SimTypeInt(signed=False, label="UInt32"), "totalMainModesStarted": SimTypeInt(signed=False, label="UInt32"), "totalSuccessfulMainModes": SimTypeInt(signed=False, label="UInt32"), "totalFailedMainModes": SimTypeInt(signed=False, label="UInt32"), "totalResponderMainModes": SimTypeInt(signed=False, label="UInt32"), "currentNewResponderMainModes": SimTypeInt(signed=False, label="UInt32"), "currentActiveQuickModes": SimTypeInt(signed=False, label="UInt32"), "totalQuickModesStarted": SimTypeInt(signed=False, label="UInt32"), "totalSuccessfulQuickModes": SimTypeInt(signed=False, label="UInt32"), "totalFailedQuickModes": SimTypeInt(signed=False, label="UInt32"), "totalAcquires": SimTypeInt(signed=False, label="UInt32"), "totalReinitAcquires": SimTypeInt(signed=False, label="UInt32"), "currentActiveExtendedModes": SimTypeInt(signed=False, label="UInt32"), "totalExtendedModesStarted": SimTypeInt(signed=False, label="UInt32"), "totalSuccessfulExtendedModes": SimTypeInt(signed=False, label="UInt32"), "totalFailedExtendedModes": SimTypeInt(signed=False, label="UInt32"), "totalImpersonationExtendedModes": SimTypeInt(signed=False, label="UInt32"), "totalImpersonationMainModes": SimTypeInt(signed=False, label="UInt32")}, name="IKEEXT_IP_VERSION_SPECIFIC_KEYMODULE_STATISTICS0", pack=False, align=None), "errorFrequencyTable": SimTypeFixedSizeArray(SimTypeInt(signed=False, label="UInt32"), 97), "mainModeNegotiationTime": SimTypeInt(signed=False, label="UInt32"), "quickModeNegotiationTime": SimTypeInt(signed=False, label="UInt32"), "extendedModeNegotiationTime": SimTypeInt(signed=False, label="UInt32")}, name="IKEEXT_KEYMODULE_STATISTICS0", pack=False, align=None), "authipStatistics": SimStruct({"v4Statistics": SimStruct({"currentActiveMainModes": SimTypeInt(signed=False, label="UInt32"), "totalMainModesStarted": SimTypeInt(signed=False, label="UInt32"), "totalSuccessfulMainModes": SimTypeInt(signed=False, label="UInt32"), "totalFailedMainModes": SimTypeInt(signed=False, label="UInt32"), "totalResponderMainModes": SimTypeInt(signed=False, label="UInt32"), "currentNewResponderMainModes": SimTypeInt(signed=False, label="UInt32"), "currentActiveQuickModes": SimTypeInt(signed=False, label="UInt32"), "totalQuickModesStarted": SimTypeInt(signed=False, label="UInt32"), "totalSuccessfulQuickModes": SimTypeInt(signed=False, label="UInt32"), "totalFailedQuickModes": SimTypeInt(signed=False, label="UInt32"), "totalAcquires": SimTypeInt(signed=False, label="UInt32"), "totalReinitAcquires": SimTypeInt(signed=False, label="UInt32"), "currentActiveExtendedModes": SimTypeInt(signed=False, label="UInt32"), "totalExtendedModesStarted": SimTypeInt(signed=False, label="UInt32"), "totalSuccessfulExtendedModes": SimTypeInt(signed=False, label="UInt32"), "totalFailedExtendedModes": SimTypeInt(signed=False, label="UInt32"), "totalImpersonationExtendedModes": SimTypeInt(signed=False, label="UInt32"), "totalImpersonationMainModes": SimTypeInt(signed=False, label="UInt32")}, name="IKEEXT_IP_VERSION_SPECIFIC_KEYMODULE_STATISTICS0", pack=False, align=None), "v6Statistics": SimStruct({"currentActiveMainModes": SimTypeInt(signed=False, label="UInt32"), "totalMainModesStarted": SimTypeInt(signed=False, label="UInt32"), "totalSuccessfulMainModes": SimTypeInt(signed=False, label="UInt32"), "totalFailedMainModes": SimTypeInt(signed=False, label="UInt32"), "totalResponderMainModes": SimTypeInt(signed=False, label="UInt32"), "currentNewResponderMainModes": SimTypeInt(signed=False, label="UInt32"), "currentActiveQuickModes": SimTypeInt(signed=False, label="UInt32"), "totalQuickModesStarted": SimTypeInt(signed=False, label="UInt32"), "totalSuccessfulQuickModes": SimTypeInt(signed=False, label="UInt32"), "totalFailedQuickModes": SimTypeInt(signed=False, label="UInt32"), "totalAcquires": SimTypeInt(signed=False, label="UInt32"), "totalReinitAcquires": SimTypeInt(signed=False, label="UInt32"), "currentActiveExtendedModes": SimTypeInt(signed=False, label="UInt32"), "totalExtendedModesStarted": SimTypeInt(signed=False, label="UInt32"), "totalSuccessfulExtendedModes": SimTypeInt(signed=False, label="UInt32"), "totalFailedExtendedModes": SimTypeInt(signed=False, label="UInt32"), "totalImpersonationExtendedModes": SimTypeInt(signed=False, label="UInt32"), "totalImpersonationMainModes": SimTypeInt(signed=False, label="UInt32")}, name="IKEEXT_IP_VERSION_SPECIFIC_KEYMODULE_STATISTICS0", pack=False, align=None), "errorFrequencyTable": SimTypeFixedSizeArray(SimTypeInt(signed=False, label="UInt32"), 97), "mainModeNegotiationTime": SimTypeInt(signed=False, label="UInt32"), "quickModeNegotiationTime": SimTypeInt(signed=False, label="UInt32"), "extendedModeNegotiationTime": SimTypeInt(signed=False, label="UInt32")}, name="IKEEXT_KEYMODULE_STATISTICS0", pack=False, align=None), "commonStatistics": SimStruct({"v4Statistics": SimStruct({"totalSocketReceiveFailures": SimTypeInt(signed=False, label="UInt32"), "totalSocketSendFailures": SimTypeInt(signed=False, label="UInt32")}, name="IKEEXT_IP_VERSION_SPECIFIC_COMMON_STATISTICS0", pack=False, align=None), "v6Statistics": SimStruct({"totalSocketReceiveFailures": SimTypeInt(signed=False, label="UInt32"), "totalSocketSendFailures": SimTypeInt(signed=False, label="UInt32")}, name="IKEEXT_IP_VERSION_SPECIFIC_COMMON_STATISTICS0", pack=False, align=None), "totalPacketsReceived": SimTypeInt(signed=False, label="UInt32"), "totalInvalidPacketsReceived": SimTypeInt(signed=False, label="UInt32"), "currentQueuedWorkitems": SimTypeInt(signed=False, label="UInt32")}, name="IKEEXT_COMMON_STATISTICS0", pack=False, align=None)}, name="IKEEXT_STATISTICS0", pack=False, align=None), offset=0)], SimTypeInt(signed=False, label="UInt32"), arg_names=["engineHandle", "ikeextStatistics"]),
        # 
        'IkeextGetStatistics1': SimTypeFunction([SimTypePointer(SimTypeInt(signed=True, label="Int"), label="IntPtr", offset=0), SimTypePointer(SimStruct({"ikeStatistics": SimStruct({"v4Statistics": SimStruct({"currentActiveMainModes": SimTypeInt(signed=False, label="UInt32"), "totalMainModesStarted": SimTypeInt(signed=False, label="UInt32"), "totalSuccessfulMainModes": SimTypeInt(signed=False, label="UInt32"), "totalFailedMainModes": SimTypeInt(signed=False, label="UInt32"), "totalResponderMainModes": SimTypeInt(signed=False, label="UInt32"), "currentNewResponderMainModes": SimTypeInt(signed=False, label="UInt32"), "currentActiveQuickModes": SimTypeInt(signed=False, label="UInt32"), "totalQuickModesStarted": SimTypeInt(signed=False, label="UInt32"), "totalSuccessfulQuickModes": SimTypeInt(signed=False, label="UInt32"), "totalFailedQuickModes": SimTypeInt(signed=False, label="UInt32"), "totalAcquires": SimTypeInt(signed=False, label="UInt32"), "totalReinitAcquires": SimTypeInt(signed=False, label="UInt32"), "currentActiveExtendedModes": SimTypeInt(signed=False, label="UInt32"), "totalExtendedModesStarted": SimTypeInt(signed=False, label="UInt32"), "totalSuccessfulExtendedModes": SimTypeInt(signed=False, label="UInt32"), "totalFailedExtendedModes": SimTypeInt(signed=False, label="UInt32"), "totalImpersonationExtendedModes": SimTypeInt(signed=False, label="UInt32"), "totalImpersonationMainModes": SimTypeInt(signed=False, label="UInt32")}, name="IKEEXT_IP_VERSION_SPECIFIC_KEYMODULE_STATISTICS1", pack=False, align=None), "v6Statistics": SimStruct({"currentActiveMainModes": SimTypeInt(signed=False, label="UInt32"), "totalMainModesStarted": SimTypeInt(signed=False, label="UInt32"), "totalSuccessfulMainModes": SimTypeInt(signed=False, label="UInt32"), "totalFailedMainModes": SimTypeInt(signed=False, label="UInt32"), "totalResponderMainModes": SimTypeInt(signed=False, label="UInt32"), "currentNewResponderMainModes": SimTypeInt(signed=False, label="UInt32"), "currentActiveQuickModes": SimTypeInt(signed=False, label="UInt32"), "totalQuickModesStarted": SimTypeInt(signed=False, label="UInt32"), "totalSuccessfulQuickModes": SimTypeInt(signed=False, label="UInt32"), "totalFailedQuickModes": SimTypeInt(signed=False, label="UInt32"), "totalAcquires": SimTypeInt(signed=False, label="UInt32"), "totalReinitAcquires": SimTypeInt(signed=False, label="UInt32"), "currentActiveExtendedModes": SimTypeInt(signed=False, label="UInt32"), "totalExtendedModesStarted": SimTypeInt(signed=False, label="UInt32"), "totalSuccessfulExtendedModes": SimTypeInt(signed=False, label="UInt32"), "totalFailedExtendedModes": SimTypeInt(signed=False, label="UInt32"), "totalImpersonationExtendedModes": SimTypeInt(signed=False, label="UInt32"), "totalImpersonationMainModes": SimTypeInt(signed=False, label="UInt32")}, name="IKEEXT_IP_VERSION_SPECIFIC_KEYMODULE_STATISTICS1", pack=False, align=None), "errorFrequencyTable": SimTypeFixedSizeArray(SimTypeInt(signed=False, label="UInt32"), 97), "mainModeNegotiationTime": SimTypeInt(signed=False, label="UInt32"), "quickModeNegotiationTime": SimTypeInt(signed=False, label="UInt32"), "extendedModeNegotiationTime": SimTypeInt(signed=False, label="UInt32")}, name="IKEEXT_KEYMODULE_STATISTICS1", pack=False, align=None), "authipStatistics": SimStruct({"v4Statistics": SimStruct({"currentActiveMainModes": SimTypeInt(signed=False, label="UInt32"), "totalMainModesStarted": SimTypeInt(signed=False, label="UInt32"), "totalSuccessfulMainModes": SimTypeInt(signed=False, label="UInt32"), "totalFailedMainModes": SimTypeInt(signed=False, label="UInt32"), "totalResponderMainModes": SimTypeInt(signed=False, label="UInt32"), "currentNewResponderMainModes": SimTypeInt(signed=False, label="UInt32"), "currentActiveQuickModes": SimTypeInt(signed=False, label="UInt32"), "totalQuickModesStarted": SimTypeInt(signed=False, label="UInt32"), "totalSuccessfulQuickModes": SimTypeInt(signed=False, label="UInt32"), "totalFailedQuickModes": SimTypeInt(signed=False, label="UInt32"), "totalAcquires": SimTypeInt(signed=False, label="UInt32"), "totalReinitAcquires": SimTypeInt(signed=False, label="UInt32"), "currentActiveExtendedModes": SimTypeInt(signed=False, label="UInt32"), "totalExtendedModesStarted": SimTypeInt(signed=False, label="UInt32"), "totalSuccessfulExtendedModes": SimTypeInt(signed=False, label="UInt32"), "totalFailedExtendedModes": SimTypeInt(signed=False, label="UInt32"), "totalImpersonationExtendedModes": SimTypeInt(signed=False, label="UInt32"), "totalImpersonationMainModes": SimTypeInt(signed=False, label="UInt32")}, name="IKEEXT_IP_VERSION_SPECIFIC_KEYMODULE_STATISTICS1", pack=False, align=None), "v6Statistics": SimStruct({"currentActiveMainModes": SimTypeInt(signed=False, label="UInt32"), "totalMainModesStarted": SimTypeInt(signed=False, label="UInt32"), "totalSuccessfulMainModes": SimTypeInt(signed=False, label="UInt32"), "totalFailedMainModes": SimTypeInt(signed=False, label="UInt32"), "totalResponderMainModes": SimTypeInt(signed=False, label="UInt32"), "currentNewResponderMainModes": SimTypeInt(signed=False, label="UInt32"), "currentActiveQuickModes": SimTypeInt(signed=False, label="UInt32"), "totalQuickModesStarted": SimTypeInt(signed=False, label="UInt32"), "totalSuccessfulQuickModes": SimTypeInt(signed=False, label="UInt32"), "totalFailedQuickModes": SimTypeInt(signed=False, label="UInt32"), "totalAcquires": SimTypeInt(signed=False, label="UInt32"), "totalReinitAcquires": SimTypeInt(signed=False, label="UInt32"), "currentActiveExtendedModes": SimTypeInt(signed=False, label="UInt32"), "totalExtendedModesStarted": SimTypeInt(signed=False, label="UInt32"), "totalSuccessfulExtendedModes": SimTypeInt(signed=False, label="UInt32"), "totalFailedExtendedModes": SimTypeInt(signed=False, label="UInt32"), "totalImpersonationExtendedModes": SimTypeInt(signed=False, label="UInt32"), "totalImpersonationMainModes": SimTypeInt(signed=False, label="UInt32")}, name="IKEEXT_IP_VERSION_SPECIFIC_KEYMODULE_STATISTICS1", pack=False, align=None), "errorFrequencyTable": SimTypeFixedSizeArray(SimTypeInt(signed=False, label="UInt32"), 97), "mainModeNegotiationTime": SimTypeInt(signed=False, label="UInt32"), "quickModeNegotiationTime": SimTypeInt(signed=False, label="UInt32"), "extendedModeNegotiationTime": SimTypeInt(signed=False, label="UInt32")}, name="IKEEXT_KEYMODULE_STATISTICS1", pack=False, align=None), "ikeV2Statistics": SimStruct({"v4Statistics": SimStruct({"currentActiveMainModes": SimTypeInt(signed=False, label="UInt32"), "totalMainModesStarted": SimTypeInt(signed=False, label="UInt32"), "totalSuccessfulMainModes": SimTypeInt(signed=False, label="UInt32"), "totalFailedMainModes": SimTypeInt(signed=False, label="UInt32"), "totalResponderMainModes": SimTypeInt(signed=False, label="UInt32"), "currentNewResponderMainModes": SimTypeInt(signed=False, label="UInt32"), "currentActiveQuickModes": SimTypeInt(signed=False, label="UInt32"), "totalQuickModesStarted": SimTypeInt(signed=False, label="UInt32"), "totalSuccessfulQuickModes": SimTypeInt(signed=False, label="UInt32"), "totalFailedQuickModes": SimTypeInt(signed=False, label="UInt32"), "totalAcquires": SimTypeInt(signed=False, label="UInt32"), "totalReinitAcquires": SimTypeInt(signed=False, label="UInt32"), "currentActiveExtendedModes": SimTypeInt(signed=False, label="UInt32"), "totalExtendedModesStarted": SimTypeInt(signed=False, label="UInt32"), "totalSuccessfulExtendedModes": SimTypeInt(signed=False, label="UInt32"), "totalFailedExtendedModes": SimTypeInt(signed=False, label="UInt32"), "totalImpersonationExtendedModes": SimTypeInt(signed=False, label="UInt32"), "totalImpersonationMainModes": SimTypeInt(signed=False, label="UInt32")}, name="IKEEXT_IP_VERSION_SPECIFIC_KEYMODULE_STATISTICS1", pack=False, align=None), "v6Statistics": SimStruct({"currentActiveMainModes": SimTypeInt(signed=False, label="UInt32"), "totalMainModesStarted": SimTypeInt(signed=False, label="UInt32"), "totalSuccessfulMainModes": SimTypeInt(signed=False, label="UInt32"), "totalFailedMainModes": SimTypeInt(signed=False, label="UInt32"), "totalResponderMainModes": SimTypeInt(signed=False, label="UInt32"), "currentNewResponderMainModes": SimTypeInt(signed=False, label="UInt32"), "currentActiveQuickModes": SimTypeInt(signed=False, label="UInt32"), "totalQuickModesStarted": SimTypeInt(signed=False, label="UInt32"), "totalSuccessfulQuickModes": SimTypeInt(signed=False, label="UInt32"), "totalFailedQuickModes": SimTypeInt(signed=False, label="UInt32"), "totalAcquires": SimTypeInt(signed=False, label="UInt32"), "totalReinitAcquires": SimTypeInt(signed=False, label="UInt32"), "currentActiveExtendedModes": SimTypeInt(signed=False, label="UInt32"), "totalExtendedModesStarted": SimTypeInt(signed=False, label="UInt32"), "totalSuccessfulExtendedModes": SimTypeInt(signed=False, label="UInt32"), "totalFailedExtendedModes": SimTypeInt(signed=False, label="UInt32"), "totalImpersonationExtendedModes": SimTypeInt(signed=False, label="UInt32"), "totalImpersonationMainModes": SimTypeInt(signed=False, label="UInt32")}, name="IKEEXT_IP_VERSION_SPECIFIC_KEYMODULE_STATISTICS1", pack=False, align=None), "errorFrequencyTable": SimTypeFixedSizeArray(SimTypeInt(signed=False, label="UInt32"), 97), "mainModeNegotiationTime": SimTypeInt(signed=False, label="UInt32"), "quickModeNegotiationTime": SimTypeInt(signed=False, label="UInt32"), "extendedModeNegotiationTime": SimTypeInt(signed=False, label="UInt32")}, name="IKEEXT_KEYMODULE_STATISTICS1", pack=False, align=None), "commonStatistics": SimStruct({"v4Statistics": SimStruct({"totalSocketReceiveFailures": SimTypeInt(signed=False, label="UInt32"), "totalSocketSendFailures": SimTypeInt(signed=False, label="UInt32")}, name="IKEEXT_IP_VERSION_SPECIFIC_COMMON_STATISTICS1", pack=False, align=None), "v6Statistics": SimStruct({"totalSocketReceiveFailures": SimTypeInt(signed=False, label="UInt32"), "totalSocketSendFailures": SimTypeInt(signed=False, label="UInt32")}, name="IKEEXT_IP_VERSION_SPECIFIC_COMMON_STATISTICS1", pack=False, align=None), "totalPacketsReceived": SimTypeInt(signed=False, label="UInt32"), "totalInvalidPacketsReceived": SimTypeInt(signed=False, label="UInt32"), "currentQueuedWorkitems": SimTypeInt(signed=False, label="UInt32")}, name="IKEEXT_COMMON_STATISTICS1", pack=False, align=None)}, name="IKEEXT_STATISTICS1", pack=False, align=None), offset=0)], SimTypeInt(signed=False, label="UInt32"), arg_names=["engineHandle", "ikeextStatistics"]),
        # 
        'IkeextSaDeleteById0': SimTypeFunction([SimTypePointer(SimTypeInt(signed=True, label="Int"), label="IntPtr", offset=0), SimTypeLongLong(signed=False, label="UInt64")], SimTypeInt(signed=False, label="UInt32"), arg_names=["engineHandle", "id"]),
        # 
        'IkeextSaGetById0': SimTypeFunction([SimTypePointer(SimTypeInt(signed=True, label="Int"), label="IntPtr", offset=0), SimTypeLongLong(signed=False, label="UInt64"), SimTypePointer(SimTypePointer(SimStruct({"saId": SimTypeLongLong(signed=False, label="UInt64"), "keyModuleType": SimTypeInt(signed=False, label="IKEEXT_KEY_MODULE_TYPE"), "ipVersion": SimTypeInt(signed=False, label="FWP_IP_VERSION"), "Anonymous": SimUnion({"v4UdpEncapsulation": SimTypePointer(SimTypeBottom(label="IPSEC_V4_UDP_ENCAPSULATION0"), offset=0)}, name="<anon>", label="None"), "ikeTraffic": SimStruct({"ipVersion": SimTypeInt(signed=False, label="FWP_IP_VERSION"), "Anonymous1": SimUnion({"localV4Address": SimTypeInt(signed=False, label="UInt32"), "localV6Address": SimTypeFixedSizeArray(SimTypeChar(label="Byte"), 16)}, name="<anon>", label="None"), "Anonymous2": SimUnion({"remoteV4Address": SimTypeInt(signed=False, label="UInt32"), "remoteV6Address": SimTypeFixedSizeArray(SimTypeChar(label="Byte"), 16)}, name="<anon>", label="None"), "authIpFilterId": SimTypeLongLong(signed=False, label="UInt64")}, name="IKEEXT_TRAFFIC0", pack=False, align=None), "ikeProposal": SimStruct({"cipherAlgorithm": SimStruct({"algoIdentifier": SimTypeInt(signed=False, label="IKEEXT_CIPHER_TYPE"), "keyLen": SimTypeInt(signed=False, label="UInt32"), "rounds": SimTypeInt(signed=False, label="UInt32")}, name="IKEEXT_CIPHER_ALGORITHM0", pack=False, align=None), "integrityAlgorithm": SimStruct({"algoIdentifier": SimTypeInt(signed=False, label="IKEEXT_INTEGRITY_TYPE")}, name="IKEEXT_INTEGRITY_ALGORITHM0", pack=False, align=None), "maxLifetimeSeconds": SimTypeInt(signed=False, label="UInt32"), "dhGroup": SimTypeInt(signed=False, label="IKEEXT_DH_GROUP"), "quickModeLimit": SimTypeInt(signed=False, label="UInt32")}, name="IKEEXT_PROPOSAL0", pack=False, align=None), "cookiePair": SimStruct({"initiator": SimTypeLongLong(signed=False, label="UInt64"), "responder": SimTypeLongLong(signed=False, label="UInt64")}, name="IKEEXT_COOKIE_PAIR0", pack=False, align=None), "ikeCredentials": SimStruct({"numCredentials": SimTypeInt(signed=False, label="UInt32"), "credentials": SimTypePointer(SimStruct({"localCredentials": SimStruct({"authenticationMethodType": SimTypeInt(signed=False, label="IKEEXT_AUTHENTICATION_METHOD_TYPE"), "impersonationType": SimTypeInt(signed=False, label="IKEEXT_AUTHENTICATION_IMPERSONATION_TYPE"), "Anonymous": SimUnion({"presharedKey": SimTypePointer(SimStruct({"presharedKey": SimStruct({"size": SimTypeInt(signed=False, label="UInt32"), "data": SimTypePointer(SimTypeChar(label="Byte"), offset=0)}, name="FWP_BYTE_BLOB", pack=False, align=None)}, name="IKEEXT_PRESHARED_KEY_AUTHENTICATION0", pack=False, align=None), offset=0), "certificate": SimTypePointer(SimStruct({"subjectName": SimStruct({"size": SimTypeInt(signed=False, label="UInt32"), "data": SimTypePointer(SimTypeChar(label="Byte"), offset=0)}, name="FWP_BYTE_BLOB", pack=False, align=None), "certHash": SimStruct({"size": SimTypeInt(signed=False, label="UInt32"), "data": SimTypePointer(SimTypeChar(label="Byte"), offset=0)}, name="FWP_BYTE_BLOB", pack=False, align=None), "flags": SimTypeInt(signed=False, label="UInt32")}, name="IKEEXT_CERTIFICATE_CREDENTIAL0", pack=False, align=None), offset=0), "name": SimTypePointer(SimStruct({"principalName": SimTypePointer(SimTypeChar(label="Char"), offset=0)}, name="IKEEXT_NAME_CREDENTIAL0", pack=False, align=None), offset=0)}, name="<anon>", label="None")}, name="IKEEXT_CREDENTIAL0", pack=False, align=None), "peerCredentials": SimStruct({"authenticationMethodType": SimTypeInt(signed=False, label="IKEEXT_AUTHENTICATION_METHOD_TYPE"), "impersonationType": SimTypeInt(signed=False, label="IKEEXT_AUTHENTICATION_IMPERSONATION_TYPE"), "Anonymous": SimUnion({"presharedKey": SimTypePointer(SimStruct({"presharedKey": SimStruct({"size": SimTypeInt(signed=False, label="UInt32"), "data": SimTypePointer(SimTypeChar(label="Byte"), offset=0)}, name="FWP_BYTE_BLOB", pack=False, align=None)}, name="IKEEXT_PRESHARED_KEY_AUTHENTICATION0", pack=False, align=None), offset=0), "certificate": SimTypePointer(SimStruct({"subjectName": SimStruct({"size": SimTypeInt(signed=False, label="UInt32"), "data": SimTypePointer(SimTypeChar(label="Byte"), offset=0)}, name="FWP_BYTE_BLOB", pack=False, align=None), "certHash": SimStruct({"size": SimTypeInt(signed=False, label="UInt32"), "data": SimTypePointer(SimTypeChar(label="Byte"), offset=0)}, name="FWP_BYTE_BLOB", pack=False, align=None), "flags": SimTypeInt(signed=False, label="UInt32")}, name="IKEEXT_CERTIFICATE_CREDENTIAL0", pack=False, align=None), offset=0), "name": SimTypePointer(SimStruct({"principalName": SimTypePointer(SimTypeChar(label="Char"), offset=0)}, name="IKEEXT_NAME_CREDENTIAL0", pack=False, align=None), offset=0)}, name="<anon>", label="None")}, name="IKEEXT_CREDENTIAL0", pack=False, align=None)}, name="IKEEXT_CREDENTIAL_PAIR0", pack=False, align=None), offset=0)}, name="IKEEXT_CREDENTIALS0", pack=False, align=None), "ikePolicyKey": SimTypeBottom(label="Guid"), "virtualIfTunnelId": SimTypeLongLong(signed=False, label="UInt64")}, name="IKEEXT_SA_DETAILS0", pack=False, align=None), offset=0), offset=0)], SimTypeInt(signed=False, label="UInt32"), arg_names=["engineHandle", "id", "sa"]),
        # 
        'IkeextSaGetById1': SimTypeFunction([SimTypePointer(SimTypeInt(signed=True, label="Int"), label="IntPtr", offset=0), SimTypeLongLong(signed=False, label="UInt64"), SimTypePointer(SimTypeBottom(label="Guid"), offset=0), SimTypePointer(SimTypePointer(SimStruct({"saId": SimTypeLongLong(signed=False, label="UInt64"), "keyModuleType": SimTypeInt(signed=False, label="IKEEXT_KEY_MODULE_TYPE"), "ipVersion": SimTypeInt(signed=False, label="FWP_IP_VERSION"), "Anonymous": SimUnion({"v4UdpEncapsulation": SimTypePointer(SimTypeBottom(label="IPSEC_V4_UDP_ENCAPSULATION0"), offset=0)}, name="<anon>", label="None"), "ikeTraffic": SimStruct({"ipVersion": SimTypeInt(signed=False, label="FWP_IP_VERSION"), "Anonymous1": SimUnion({"localV4Address": SimTypeInt(signed=False, label="UInt32"), "localV6Address": SimTypeFixedSizeArray(SimTypeChar(label="Byte"), 16)}, name="<anon>", label="None"), "Anonymous2": SimUnion({"remoteV4Address": SimTypeInt(signed=False, label="UInt32"), "remoteV6Address": SimTypeFixedSizeArray(SimTypeChar(label="Byte"), 16)}, name="<anon>", label="None"), "authIpFilterId": SimTypeLongLong(signed=False, label="UInt64")}, name="IKEEXT_TRAFFIC0", pack=False, align=None), "ikeProposal": SimStruct({"cipherAlgorithm": SimStruct({"algoIdentifier": SimTypeInt(signed=False, label="IKEEXT_CIPHER_TYPE"), "keyLen": SimTypeInt(signed=False, label="UInt32"), "rounds": SimTypeInt(signed=False, label="UInt32")}, name="IKEEXT_CIPHER_ALGORITHM0", pack=False, align=None), "integrityAlgorithm": SimStruct({"algoIdentifier": SimTypeInt(signed=False, label="IKEEXT_INTEGRITY_TYPE")}, name="IKEEXT_INTEGRITY_ALGORITHM0", pack=False, align=None), "maxLifetimeSeconds": SimTypeInt(signed=False, label="UInt32"), "dhGroup": SimTypeInt(signed=False, label="IKEEXT_DH_GROUP"), "quickModeLimit": SimTypeInt(signed=False, label="UInt32")}, name="IKEEXT_PROPOSAL0", pack=False, align=None), "cookiePair": SimStruct({"initiator": SimTypeLongLong(signed=False, label="UInt64"), "responder": SimTypeLongLong(signed=False, label="UInt64")}, name="IKEEXT_COOKIE_PAIR0", pack=False, align=None), "ikeCredentials": SimStruct({"numCredentials": SimTypeInt(signed=False, label="UInt32"), "credentials": SimTypePointer(SimStruct({"localCredentials": SimStruct({"authenticationMethodType": SimTypeInt(signed=False, label="IKEEXT_AUTHENTICATION_METHOD_TYPE"), "impersonationType": SimTypeInt(signed=False, label="IKEEXT_AUTHENTICATION_IMPERSONATION_TYPE"), "Anonymous": SimUnion({"presharedKey": SimTypePointer(SimStruct({"presharedKey": SimStruct({"size": SimTypeInt(signed=False, label="UInt32"), "data": SimTypePointer(SimTypeChar(label="Byte"), offset=0)}, name="FWP_BYTE_BLOB", pack=False, align=None), "flags": SimTypeInt(signed=False, label="IKEEXT_PRESHARED_KEY_AUTHENTICATION_FLAGS")}, name="IKEEXT_PRESHARED_KEY_AUTHENTICATION1", pack=False, align=None), offset=0), "certificate": SimTypePointer(SimStruct({"subjectName": SimStruct({"size": SimTypeInt(signed=False, label="UInt32"), "data": SimTypePointer(SimTypeChar(label="Byte"), offset=0)}, name="FWP_BYTE_BLOB", pack=False, align=None), "certHash": SimStruct({"size": SimTypeInt(signed=False, label="UInt32"), "data": SimTypePointer(SimTypeChar(label="Byte"), offset=0)}, name="FWP_BYTE_BLOB", pack=False, align=None), "flags": SimTypeInt(signed=False, label="UInt32"), "certificate": SimStruct({"size": SimTypeInt(signed=False, label="UInt32"), "data": SimTypePointer(SimTypeChar(label="Byte"), offset=0)}, name="FWP_BYTE_BLOB", pack=False, align=None)}, name="IKEEXT_CERTIFICATE_CREDENTIAL1", pack=False, align=None), offset=0), "name": SimTypePointer(SimStruct({"principalName": SimTypePointer(SimTypeChar(label="Char"), offset=0)}, name="IKEEXT_NAME_CREDENTIAL0", pack=False, align=None), offset=0)}, name="<anon>", label="None")}, name="IKEEXT_CREDENTIAL1", pack=False, align=None), "peerCredentials": SimStruct({"authenticationMethodType": SimTypeInt(signed=False, label="IKEEXT_AUTHENTICATION_METHOD_TYPE"), "impersonationType": SimTypeInt(signed=False, label="IKEEXT_AUTHENTICATION_IMPERSONATION_TYPE"), "Anonymous": SimUnion({"presharedKey": SimTypePointer(SimStruct({"presharedKey": SimStruct({"size": SimTypeInt(signed=False, label="UInt32"), "data": SimTypePointer(SimTypeChar(label="Byte"), offset=0)}, name="FWP_BYTE_BLOB", pack=False, align=None), "flags": SimTypeInt(signed=False, label="IKEEXT_PRESHARED_KEY_AUTHENTICATION_FLAGS")}, name="IKEEXT_PRESHARED_KEY_AUTHENTICATION1", pack=False, align=None), offset=0), "certificate": SimTypePointer(SimStruct({"subjectName": SimStruct({"size": SimTypeInt(signed=False, label="UInt32"), "data": SimTypePointer(SimTypeChar(label="Byte"), offset=0)}, name="FWP_BYTE_BLOB", pack=False, align=None), "certHash": SimStruct({"size": SimTypeInt(signed=False, label="UInt32"), "data": SimTypePointer(SimTypeChar(label="Byte"), offset=0)}, name="FWP_BYTE_BLOB", pack=False, align=None), "flags": SimTypeInt(signed=False, label="UInt32"), "certificate": SimStruct({"size": SimTypeInt(signed=False, label="UInt32"), "data": SimTypePointer(SimTypeChar(label="Byte"), offset=0)}, name="FWP_BYTE_BLOB", pack=False, align=None)}, name="IKEEXT_CERTIFICATE_CREDENTIAL1", pack=False, align=None), offset=0), "name": SimTypePointer(SimStruct({"principalName": SimTypePointer(SimTypeChar(label="Char"), offset=0)}, name="IKEEXT_NAME_CREDENTIAL0", pack=False, align=None), offset=0)}, name="<anon>", label="None")}, name="IKEEXT_CREDENTIAL1", pack=False, align=None)}, name="IKEEXT_CREDENTIAL_PAIR1", pack=False, align=None), offset=0)}, name="IKEEXT_CREDENTIALS1", pack=False, align=None), "ikePolicyKey": SimTypeBottom(label="Guid"), "virtualIfTunnelId": SimTypeLongLong(signed=False, label="UInt64"), "correlationKey": SimStruct({"size": SimTypeInt(signed=False, label="UInt32"), "data": SimTypePointer(SimTypeChar(label="Byte"), offset=0)}, name="FWP_BYTE_BLOB", pack=False, align=None)}, name="IKEEXT_SA_DETAILS1", pack=False, align=None), offset=0), offset=0)], SimTypeInt(signed=False, label="UInt32"), arg_names=["engineHandle", "id", "saLookupContext", "sa"]),
        # 
        'IkeextSaGetById2': SimTypeFunction([SimTypePointer(SimTypeInt(signed=True, label="Int"), label="IntPtr", offset=0), SimTypeLongLong(signed=False, label="UInt64"), SimTypePointer(SimTypeBottom(label="Guid"), offset=0), SimTypePointer(SimTypePointer(SimStruct({"saId": SimTypeLongLong(signed=False, label="UInt64"), "keyModuleType": SimTypeInt(signed=False, label="IKEEXT_KEY_MODULE_TYPE"), "ipVersion": SimTypeInt(signed=False, label="FWP_IP_VERSION"), "Anonymous": SimUnion({"v4UdpEncapsulation": SimTypePointer(SimTypeBottom(label="IPSEC_V4_UDP_ENCAPSULATION0"), offset=0)}, name="<anon>", label="None"), "ikeTraffic": SimStruct({"ipVersion": SimTypeInt(signed=False, label="FWP_IP_VERSION"), "Anonymous1": SimUnion({"localV4Address": SimTypeInt(signed=False, label="UInt32"), "localV6Address": SimTypeFixedSizeArray(SimTypeChar(label="Byte"), 16)}, name="<anon>", label="None"), "Anonymous2": SimUnion({"remoteV4Address": SimTypeInt(signed=False, label="UInt32"), "remoteV6Address": SimTypeFixedSizeArray(SimTypeChar(label="Byte"), 16)}, name="<anon>", label="None"), "authIpFilterId": SimTypeLongLong(signed=False, label="UInt64")}, name="IKEEXT_TRAFFIC0", pack=False, align=None), "ikeProposal": SimStruct({"cipherAlgorithm": SimStruct({"algoIdentifier": SimTypeInt(signed=False, label="IKEEXT_CIPHER_TYPE"), "keyLen": SimTypeInt(signed=False, label="UInt32"), "rounds": SimTypeInt(signed=False, label="UInt32")}, name="IKEEXT_CIPHER_ALGORITHM0", pack=False, align=None), "integrityAlgorithm": SimStruct({"algoIdentifier": SimTypeInt(signed=False, label="IKEEXT_INTEGRITY_TYPE")}, name="IKEEXT_INTEGRITY_ALGORITHM0", pack=False, align=None), "maxLifetimeSeconds": SimTypeInt(signed=False, label="UInt32"), "dhGroup": SimTypeInt(signed=False, label="IKEEXT_DH_GROUP"), "quickModeLimit": SimTypeInt(signed=False, label="UInt32")}, name="IKEEXT_PROPOSAL0", pack=False, align=None), "cookiePair": SimStruct({"initiator": SimTypeLongLong(signed=False, label="UInt64"), "responder": SimTypeLongLong(signed=False, label="UInt64")}, name="IKEEXT_COOKIE_PAIR0", pack=False, align=None), "ikeCredentials": SimStruct({"numCredentials": SimTypeInt(signed=False, label="UInt32"), "credentials": SimTypePointer(SimStruct({"localCredentials": SimStruct({"authenticationMethodType": SimTypeInt(signed=False, label="IKEEXT_AUTHENTICATION_METHOD_TYPE"), "impersonationType": SimTypeInt(signed=False, label="IKEEXT_AUTHENTICATION_IMPERSONATION_TYPE"), "Anonymous": SimUnion({"presharedKey": SimTypePointer(SimStruct({"presharedKey": SimStruct({"size": SimTypeInt(signed=False, label="UInt32"), "data": SimTypePointer(SimTypeChar(label="Byte"), offset=0)}, name="FWP_BYTE_BLOB", pack=False, align=None), "flags": SimTypeInt(signed=False, label="IKEEXT_PRESHARED_KEY_AUTHENTICATION_FLAGS")}, name="IKEEXT_PRESHARED_KEY_AUTHENTICATION1", pack=False, align=None), offset=0), "certificate": SimTypePointer(SimStruct({"subjectName": SimStruct({"size": SimTypeInt(signed=False, label="UInt32"), "data": SimTypePointer(SimTypeChar(label="Byte"), offset=0)}, name="FWP_BYTE_BLOB", pack=False, align=None), "certHash": SimStruct({"size": SimTypeInt(signed=False, label="UInt32"), "data": SimTypePointer(SimTypeChar(label="Byte"), offset=0)}, name="FWP_BYTE_BLOB", pack=False, align=None), "flags": SimTypeInt(signed=False, label="UInt32"), "certificate": SimStruct({"size": SimTypeInt(signed=False, label="UInt32"), "data": SimTypePointer(SimTypeChar(label="Byte"), offset=0)}, name="FWP_BYTE_BLOB", pack=False, align=None)}, name="IKEEXT_CERTIFICATE_CREDENTIAL1", pack=False, align=None), offset=0), "name": SimTypePointer(SimStruct({"principalName": SimTypePointer(SimTypeChar(label="Char"), offset=0)}, name="IKEEXT_NAME_CREDENTIAL0", pack=False, align=None), offset=0)}, name="<anon>", label="None")}, name="IKEEXT_CREDENTIAL2", pack=False, align=None), "peerCredentials": SimStruct({"authenticationMethodType": SimTypeInt(signed=False, label="IKEEXT_AUTHENTICATION_METHOD_TYPE"), "impersonationType": SimTypeInt(signed=False, label="IKEEXT_AUTHENTICATION_IMPERSONATION_TYPE"), "Anonymous": SimUnion({"presharedKey": SimTypePointer(SimStruct({"presharedKey": SimStruct({"size": SimTypeInt(signed=False, label="UInt32"), "data": SimTypePointer(SimTypeChar(label="Byte"), offset=0)}, name="FWP_BYTE_BLOB", pack=False, align=None), "flags": SimTypeInt(signed=False, label="IKEEXT_PRESHARED_KEY_AUTHENTICATION_FLAGS")}, name="IKEEXT_PRESHARED_KEY_AUTHENTICATION1", pack=False, align=None), offset=0), "certificate": SimTypePointer(SimStruct({"subjectName": SimStruct({"size": SimTypeInt(signed=False, label="UInt32"), "data": SimTypePointer(SimTypeChar(label="Byte"), offset=0)}, name="FWP_BYTE_BLOB", pack=False, align=None), "certHash": SimStruct({"size": SimTypeInt(signed=False, label="UInt32"), "data": SimTypePointer(SimTypeChar(label="Byte"), offset=0)}, name="FWP_BYTE_BLOB", pack=False, align=None), "flags": SimTypeInt(signed=False, label="UInt32"), "certificate": SimStruct({"size": SimTypeInt(signed=False, label="UInt32"), "data": SimTypePointer(SimTypeChar(label="Byte"), offset=0)}, name="FWP_BYTE_BLOB", pack=False, align=None)}, name="IKEEXT_CERTIFICATE_CREDENTIAL1", pack=False, align=None), offset=0), "name": SimTypePointer(SimStruct({"principalName": SimTypePointer(SimTypeChar(label="Char"), offset=0)}, name="IKEEXT_NAME_CREDENTIAL0", pack=False, align=None), offset=0)}, name="<anon>", label="None")}, name="IKEEXT_CREDENTIAL2", pack=False, align=None)}, name="IKEEXT_CREDENTIAL_PAIR2", pack=False, align=None), offset=0)}, name="IKEEXT_CREDENTIALS2", pack=False, align=None), "ikePolicyKey": SimTypeBottom(label="Guid"), "virtualIfTunnelId": SimTypeLongLong(signed=False, label="UInt64"), "correlationKey": SimStruct({"size": SimTypeInt(signed=False, label="UInt32"), "data": SimTypePointer(SimTypeChar(label="Byte"), offset=0)}, name="FWP_BYTE_BLOB", pack=False, align=None)}, name="IKEEXT_SA_DETAILS2", pack=False, align=None), offset=0), offset=0)], SimTypeInt(signed=False, label="UInt32"), arg_names=["engineHandle", "id", "saLookupContext", "sa"]),
        # 
        'IkeextSaCreateEnumHandle0': SimTypeFunction([SimTypePointer(SimTypeInt(signed=True, label="Int"), label="IntPtr", offset=0), SimTypePointer(SimStruct({"localSubNet": SimStruct({"type": SimTypeInt(signed=False, label="FWP_DATA_TYPE"), "Anonymous": SimUnion({"uint8": SimTypeChar(label="Byte"), "uint16": SimTypeShort(signed=False, label="UInt16"), "uint32": SimTypeInt(signed=False, label="UInt32"), "uint64": SimTypePointer(SimTypeLongLong(signed=False, label="UInt64"), offset=0), "int8": SimTypeChar(label="SByte"), "int16": SimTypeShort(signed=True, label="Int16"), "int32": SimTypeInt(signed=True, label="Int32"), "int64": SimTypePointer(SimTypeLongLong(signed=True, label="Int64"), offset=0), "float32": SimTypeFloat(size=32), "double64": SimTypePointer(SimTypeFloat(size=64), offset=0), "byteArray16": SimTypePointer(SimStruct({"byteArray16": SimTypeFixedSizeArray(SimTypeChar(label="Byte"), 16)}, name="FWP_BYTE_ARRAY16", pack=False, align=None), offset=0), "byteBlob": SimTypePointer(SimStruct({"size": SimTypeInt(signed=False, label="UInt32"), "data": SimTypePointer(SimTypeChar(label="Byte"), offset=0)}, name="FWP_BYTE_BLOB", pack=False, align=None), offset=0), "sid": SimTypePointer(SimTypeBottom(label="SID"), offset=0), "sd": SimTypePointer(SimStruct({"size": SimTypeInt(signed=False, label="UInt32"), "data": SimTypePointer(SimTypeChar(label="Byte"), offset=0)}, name="FWP_BYTE_BLOB", pack=False, align=None), offset=0), "tokenInformation": SimTypePointer(SimStruct({"sidCount": SimTypeInt(signed=False, label="UInt32"), "sids": SimTypePointer(SimTypeBottom(label="SID_AND_ATTRIBUTES"), offset=0), "restrictedSidCount": SimTypeInt(signed=False, label="UInt32"), "restrictedSids": SimTypePointer(SimTypeBottom(label="SID_AND_ATTRIBUTES"), offset=0)}, name="FWP_TOKEN_INFORMATION", pack=False, align=None), offset=0), "tokenAccessInformation": SimTypePointer(SimStruct({"size": SimTypeInt(signed=False, label="UInt32"), "data": SimTypePointer(SimTypeChar(label="Byte"), offset=0)}, name="FWP_BYTE_BLOB", pack=False, align=None), offset=0), "unicodeString": SimTypePointer(SimTypeChar(label="Char"), offset=0), "byteArray6": SimTypePointer(SimStruct({"byteArray6": SimTypeFixedSizeArray(SimTypeChar(label="Byte"), 6)}, name="FWP_BYTE_ARRAY6", pack=False, align=None), offset=0), "bitmapArray64": SimTypePointer(SimStruct({"bitmapArray64": SimTypeFixedSizeArray(SimTypeChar(label="Byte"), 8)}, name="FWP_BITMAP_ARRAY64_", pack=False, align=None), offset=0), "v4AddrMask": SimTypePointer(SimStruct({"addr": SimTypeInt(signed=False, label="UInt32"), "mask": SimTypeInt(signed=False, label="UInt32")}, name="FWP_V4_ADDR_AND_MASK", pack=False, align=None), offset=0), "v6AddrMask": SimTypePointer(SimStruct({"addr": SimTypeFixedSizeArray(SimTypeChar(label="Byte"), 16), "prefixLength": SimTypeChar(label="Byte")}, name="FWP_V6_ADDR_AND_MASK", pack=False, align=None), offset=0), "rangeValue": SimTypePointer(SimStruct({"valueLow": SimStruct({"type": SimTypeInt(signed=False, label="FWP_DATA_TYPE"), "Anonymous": SimUnion({"uint8": SimTypeChar(label="Byte"), "uint16": SimTypeShort(signed=False, label="UInt16"), "uint32": SimTypeInt(signed=False, label="UInt32"), "uint64": SimTypePointer(SimTypeLongLong(signed=False, label="UInt64"), offset=0), "int8": SimTypeChar(label="SByte"), "int16": SimTypeShort(signed=True, label="Int16"), "int32": SimTypeInt(signed=True, label="Int32"), "int64": SimTypePointer(SimTypeLongLong(signed=True, label="Int64"), offset=0), "float32": SimTypeFloat(size=32), "double64": SimTypePointer(SimTypeFloat(size=64), offset=0), "byteArray16": SimTypePointer(SimStruct({"byteArray16": SimTypeFixedSizeArray(SimTypeChar(label="Byte"), 16)}, name="FWP_BYTE_ARRAY16", pack=False, align=None), offset=0), "byteBlob": SimTypePointer(SimStruct({"size": SimTypeInt(signed=False, label="UInt32"), "data": SimTypePointer(SimTypeChar(label="Byte"), offset=0)}, name="FWP_BYTE_BLOB", pack=False, align=None), offset=0), "sid": SimTypePointer(SimTypeBottom(label="SID"), offset=0), "sd": SimTypePointer(SimStruct({"size": SimTypeInt(signed=False, label="UInt32"), "data": SimTypePointer(SimTypeChar(label="Byte"), offset=0)}, name="FWP_BYTE_BLOB", pack=False, align=None), offset=0), "tokenInformation": SimTypePointer(SimStruct({"sidCount": SimTypeInt(signed=False, label="UInt32"), "sids": SimTypePointer(SimTypeBottom(label="SID_AND_ATTRIBUTES"), offset=0), "restrictedSidCount": SimTypeInt(signed=False, label="UInt32"), "restrictedSids": SimTypePointer(SimTypeBottom(label="SID_AND_ATTRIBUTES"), offset=0)}, name="FWP_TOKEN_INFORMATION", pack=False, align=None), offset=0), "tokenAccessInformation": SimTypePointer(SimStruct({"size": SimTypeInt(signed=False, label="UInt32"), "data": SimTypePointer(SimTypeChar(label="Byte"), offset=0)}, name="FWP_BYTE_BLOB", pack=False, align=None), offset=0), "unicodeString": SimTypePointer(SimTypeChar(label="Char"), offset=0), "byteArray6": SimTypePointer(SimStruct({"byteArray6": SimTypeFixedSizeArray(SimTypeChar(label="Byte"), 6)}, name="FWP_BYTE_ARRAY6", pack=False, align=None), offset=0), "bitmapArray64": SimTypePointer(SimStruct({"bitmapArray64": SimTypeFixedSizeArray(SimTypeChar(label="Byte"), 8)}, name="FWP_BITMAP_ARRAY64_", pack=False, align=None), offset=0)}, name="<anon>", label="None")}, name="FWP_VALUE0", pack=False, align=None), "valueHigh": SimStruct({"type": SimTypeInt(signed=False, label="FWP_DATA_TYPE"), "Anonymous": SimUnion({"uint8": SimTypeChar(label="Byte"), "uint16": SimTypeShort(signed=False, label="UInt16"), "uint32": SimTypeInt(signed=False, label="UInt32"), "uint64": SimTypePointer(SimTypeLongLong(signed=False, label="UInt64"), offset=0), "int8": SimTypeChar(label="SByte"), "int16": SimTypeShort(signed=True, label="Int16"), "int32": SimTypeInt(signed=True, label="Int32"), "int64": SimTypePointer(SimTypeLongLong(signed=True, label="Int64"), offset=0), "float32": SimTypeFloat(size=32), "double64": SimTypePointer(SimTypeFloat(size=64), offset=0), "byteArray16": SimTypePointer(SimStruct({"byteArray16": SimTypeFixedSizeArray(SimTypeChar(label="Byte"), 16)}, name="FWP_BYTE_ARRAY16", pack=False, align=None), offset=0), "byteBlob": SimTypePointer(SimStruct({"size": SimTypeInt(signed=False, label="UInt32"), "data": SimTypePointer(SimTypeChar(label="Byte"), offset=0)}, name="FWP_BYTE_BLOB", pack=False, align=None), offset=0), "sid": SimTypePointer(SimTypeBottom(label="SID"), offset=0), "sd": SimTypePointer(SimStruct({"size": SimTypeInt(signed=False, label="UInt32"), "data": SimTypePointer(SimTypeChar(label="Byte"), offset=0)}, name="FWP_BYTE_BLOB", pack=False, align=None), offset=0), "tokenInformation": SimTypePointer(SimStruct({"sidCount": SimTypeInt(signed=False, label="UInt32"), "sids": SimTypePointer(SimTypeBottom(label="SID_AND_ATTRIBUTES"), offset=0), "restrictedSidCount": SimTypeInt(signed=False, label="UInt32"), "restrictedSids": SimTypePointer(SimTypeBottom(label="SID_AND_ATTRIBUTES"), offset=0)}, name="FWP_TOKEN_INFORMATION", pack=False, align=None), offset=0), "tokenAccessInformation": SimTypePointer(SimStruct({"size": SimTypeInt(signed=False, label="UInt32"), "data": SimTypePointer(SimTypeChar(label="Byte"), offset=0)}, name="FWP_BYTE_BLOB", pack=False, align=None), offset=0), "unicodeString": SimTypePointer(SimTypeChar(label="Char"), offset=0), "byteArray6": SimTypePointer(SimStruct({"byteArray6": SimTypeFixedSizeArray(SimTypeChar(label="Byte"), 6)}, name="FWP_BYTE_ARRAY6", pack=False, align=None), offset=0), "bitmapArray64": SimTypePointer(SimStruct({"bitmapArray64": SimTypeFixedSizeArray(SimTypeChar(label="Byte"), 8)}, name="FWP_BITMAP_ARRAY64_", pack=False, align=None), offset=0)}, name="<anon>", label="None")}, name="FWP_VALUE0", pack=False, align=None)}, name="FWP_RANGE0", pack=False, align=None), offset=0)}, name="<anon>", label="None")}, name="FWP_CONDITION_VALUE0", pack=False, align=None), "remoteSubNet": SimStruct({"type": SimTypeInt(signed=False, label="FWP_DATA_TYPE"), "Anonymous": SimUnion({"uint8": SimTypeChar(label="Byte"), "uint16": SimTypeShort(signed=False, label="UInt16"), "uint32": SimTypeInt(signed=False, label="UInt32"), "uint64": SimTypePointer(SimTypeLongLong(signed=False, label="UInt64"), offset=0), "int8": SimTypeChar(label="SByte"), "int16": SimTypeShort(signed=True, label="Int16"), "int32": SimTypeInt(signed=True, label="Int32"), "int64": SimTypePointer(SimTypeLongLong(signed=True, label="Int64"), offset=0), "float32": SimTypeFloat(size=32), "double64": SimTypePointer(SimTypeFloat(size=64), offset=0), "byteArray16": SimTypePointer(SimStruct({"byteArray16": SimTypeFixedSizeArray(SimTypeChar(label="Byte"), 16)}, name="FWP_BYTE_ARRAY16", pack=False, align=None), offset=0), "byteBlob": SimTypePointer(SimStruct({"size": SimTypeInt(signed=False, label="UInt32"), "data": SimTypePointer(SimTypeChar(label="Byte"), offset=0)}, name="FWP_BYTE_BLOB", pack=False, align=None), offset=0), "sid": SimTypePointer(SimTypeBottom(label="SID"), offset=0), "sd": SimTypePointer(SimStruct({"size": SimTypeInt(signed=False, label="UInt32"), "data": SimTypePointer(SimTypeChar(label="Byte"), offset=0)}, name="FWP_BYTE_BLOB", pack=False, align=None), offset=0), "tokenInformation": SimTypePointer(SimStruct({"sidCount": SimTypeInt(signed=False, label="UInt32"), "sids": SimTypePointer(SimTypeBottom(label="SID_AND_ATTRIBUTES"), offset=0), "restrictedSidCount": SimTypeInt(signed=False, label="UInt32"), "restrictedSids": SimTypePointer(SimTypeBottom(label="SID_AND_ATTRIBUTES"), offset=0)}, name="FWP_TOKEN_INFORMATION", pack=False, align=None), offset=0), "tokenAccessInformation": SimTypePointer(SimStruct({"size": SimTypeInt(signed=False, label="UInt32"), "data": SimTypePointer(SimTypeChar(label="Byte"), offset=0)}, name="FWP_BYTE_BLOB", pack=False, align=None), offset=0), "unicodeString": SimTypePointer(SimTypeChar(label="Char"), offset=0), "byteArray6": SimTypePointer(SimStruct({"byteArray6": SimTypeFixedSizeArray(SimTypeChar(label="Byte"), 6)}, name="FWP_BYTE_ARRAY6", pack=False, align=None), offset=0), "bitmapArray64": SimTypePointer(SimStruct({"bitmapArray64": SimTypeFixedSizeArray(SimTypeChar(label="Byte"), 8)}, name="FWP_BITMAP_ARRAY64_", pack=False, align=None), offset=0), "v4AddrMask": SimTypePointer(SimStruct({"addr": SimTypeInt(signed=False, label="UInt32"), "mask": SimTypeInt(signed=False, label="UInt32")}, name="FWP_V4_ADDR_AND_MASK", pack=False, align=None), offset=0), "v6AddrMask": SimTypePointer(SimStruct({"addr": SimTypeFixedSizeArray(SimTypeChar(label="Byte"), 16), "prefixLength": SimTypeChar(label="Byte")}, name="FWP_V6_ADDR_AND_MASK", pack=False, align=None), offset=0), "rangeValue": SimTypePointer(SimStruct({"valueLow": SimStruct({"type": SimTypeInt(signed=False, label="FWP_DATA_TYPE"), "Anonymous": SimUnion({"uint8": SimTypeChar(label="Byte"), "uint16": SimTypeShort(signed=False, label="UInt16"), "uint32": SimTypeInt(signed=False, label="UInt32"), "uint64": SimTypePointer(SimTypeLongLong(signed=False, label="UInt64"), offset=0), "int8": SimTypeChar(label="SByte"), "int16": SimTypeShort(signed=True, label="Int16"), "int32": SimTypeInt(signed=True, label="Int32"), "int64": SimTypePointer(SimTypeLongLong(signed=True, label="Int64"), offset=0), "float32": SimTypeFloat(size=32), "double64": SimTypePointer(SimTypeFloat(size=64), offset=0), "byteArray16": SimTypePointer(SimStruct({"byteArray16": SimTypeFixedSizeArray(SimTypeChar(label="Byte"), 16)}, name="FWP_BYTE_ARRAY16", pack=False, align=None), offset=0), "byteBlob": SimTypePointer(SimStruct({"size": SimTypeInt(signed=False, label="UInt32"), "data": SimTypePointer(SimTypeChar(label="Byte"), offset=0)}, name="FWP_BYTE_BLOB", pack=False, align=None), offset=0), "sid": SimTypePointer(SimTypeBottom(label="SID"), offset=0), "sd": SimTypePointer(SimStruct({"size": SimTypeInt(signed=False, label="UInt32"), "data": SimTypePointer(SimTypeChar(label="Byte"), offset=0)}, name="FWP_BYTE_BLOB", pack=False, align=None), offset=0), "tokenInformation": SimTypePointer(SimStruct({"sidCount": SimTypeInt(signed=False, label="UInt32"), "sids": SimTypePointer(SimTypeBottom(label="SID_AND_ATTRIBUTES"), offset=0), "restrictedSidCount": SimTypeInt(signed=False, label="UInt32"), "restrictedSids": SimTypePointer(SimTypeBottom(label="SID_AND_ATTRIBUTES"), offset=0)}, name="FWP_TOKEN_INFORMATION", pack=False, align=None), offset=0), "tokenAccessInformation": SimTypePointer(SimStruct({"size": SimTypeInt(signed=False, label="UInt32"), "data": SimTypePointer(SimTypeChar(label="Byte"), offset=0)}, name="FWP_BYTE_BLOB", pack=False, align=None), offset=0), "unicodeString": SimTypePointer(SimTypeChar(label="Char"), offset=0), "byteArray6": SimTypePointer(SimStruct({"byteArray6": SimTypeFixedSizeArray(SimTypeChar(label="Byte"), 6)}, name="FWP_BYTE_ARRAY6", pack=False, align=None), offset=0), "bitmapArray64": SimTypePointer(SimStruct({"bitmapArray64": SimTypeFixedSizeArray(SimTypeChar(label="Byte"), 8)}, name="FWP_BITMAP_ARRAY64_", pack=False, align=None), offset=0)}, name="<anon>", label="None")}, name="FWP_VALUE0", pack=False, align=None), "valueHigh": SimStruct({"type": SimTypeInt(signed=False, label="FWP_DATA_TYPE"), "Anonymous": SimUnion({"uint8": SimTypeChar(label="Byte"), "uint16": SimTypeShort(signed=False, label="UInt16"), "uint32": SimTypeInt(signed=False, label="UInt32"), "uint64": SimTypePointer(SimTypeLongLong(signed=False, label="UInt64"), offset=0), "int8": SimTypeChar(label="SByte"), "int16": SimTypeShort(signed=True, label="Int16"), "int32": SimTypeInt(signed=True, label="Int32"), "int64": SimTypePointer(SimTypeLongLong(signed=True, label="Int64"), offset=0), "float32": SimTypeFloat(size=32), "double64": SimTypePointer(SimTypeFloat(size=64), offset=0), "byteArray16": SimTypePointer(SimStruct({"byteArray16": SimTypeFixedSizeArray(SimTypeChar(label="Byte"), 16)}, name="FWP_BYTE_ARRAY16", pack=False, align=None), offset=0), "byteBlob": SimTypePointer(SimStruct({"size": SimTypeInt(signed=False, label="UInt32"), "data": SimTypePointer(SimTypeChar(label="Byte"), offset=0)}, name="FWP_BYTE_BLOB", pack=False, align=None), offset=0), "sid": SimTypePointer(SimTypeBottom(label="SID"), offset=0), "sd": SimTypePointer(SimStruct({"size": SimTypeInt(signed=False, label="UInt32"), "data": SimTypePointer(SimTypeChar(label="Byte"), offset=0)}, name="FWP_BYTE_BLOB", pack=False, align=None), offset=0), "tokenInformation": SimTypePointer(SimStruct({"sidCount": SimTypeInt(signed=False, label="UInt32"), "sids": SimTypePointer(SimTypeBottom(label="SID_AND_ATTRIBUTES"), offset=0), "restrictedSidCount": SimTypeInt(signed=False, label="UInt32"), "restrictedSids": SimTypePointer(SimTypeBottom(label="SID_AND_ATTRIBUTES"), offset=0)}, name="FWP_TOKEN_INFORMATION", pack=False, align=None), offset=0), "tokenAccessInformation": SimTypePointer(SimStruct({"size": SimTypeInt(signed=False, label="UInt32"), "data": SimTypePointer(SimTypeChar(label="Byte"), offset=0)}, name="FWP_BYTE_BLOB", pack=False, align=None), offset=0), "unicodeString": SimTypePointer(SimTypeChar(label="Char"), offset=0), "byteArray6": SimTypePointer(SimStruct({"byteArray6": SimTypeFixedSizeArray(SimTypeChar(label="Byte"), 6)}, name="FWP_BYTE_ARRAY6", pack=False, align=None), offset=0), "bitmapArray64": SimTypePointer(SimStruct({"bitmapArray64": SimTypeFixedSizeArray(SimTypeChar(label="Byte"), 8)}, name="FWP_BITMAP_ARRAY64_", pack=False, align=None), offset=0)}, name="<anon>", label="None")}, name="FWP_VALUE0", pack=False, align=None)}, name="FWP_RANGE0", pack=False, align=None), offset=0)}, name="<anon>", label="None")}, name="FWP_CONDITION_VALUE0", pack=False, align=None), "localMainModeCertHash": SimStruct({"size": SimTypeInt(signed=False, label="UInt32"), "data": SimTypePointer(SimTypeChar(label="Byte"), offset=0)}, name="FWP_BYTE_BLOB", pack=False, align=None)}, name="IKEEXT_SA_ENUM_TEMPLATE0", pack=False, align=None), offset=0), SimTypePointer(SimTypePointer(SimTypeInt(signed=True, label="Int"), label="IntPtr", offset=0), offset=0)], SimTypeInt(signed=False, label="UInt32"), arg_names=["engineHandle", "enumTemplate", "enumHandle"]),
        # 
        'IkeextSaEnum0': SimTypeFunction([SimTypePointer(SimTypeInt(signed=True, label="Int"), label="IntPtr", offset=0), SimTypePointer(SimTypeInt(signed=True, label="Int"), label="IntPtr", offset=0), SimTypeInt(signed=False, label="UInt32"), SimTypePointer(SimTypePointer(SimTypePointer(SimStruct({"saId": SimTypeLongLong(signed=False, label="UInt64"), "keyModuleType": SimTypeInt(signed=False, label="IKEEXT_KEY_MODULE_TYPE"), "ipVersion": SimTypeInt(signed=False, label="FWP_IP_VERSION"), "Anonymous": SimUnion({"v4UdpEncapsulation": SimTypePointer(SimTypeBottom(label="IPSEC_V4_UDP_ENCAPSULATION0"), offset=0)}, name="<anon>", label="None"), "ikeTraffic": SimStruct({"ipVersion": SimTypeInt(signed=False, label="FWP_IP_VERSION"), "Anonymous1": SimUnion({"localV4Address": SimTypeInt(signed=False, label="UInt32"), "localV6Address": SimTypeFixedSizeArray(SimTypeChar(label="Byte"), 16)}, name="<anon>", label="None"), "Anonymous2": SimUnion({"remoteV4Address": SimTypeInt(signed=False, label="UInt32"), "remoteV6Address": SimTypeFixedSizeArray(SimTypeChar(label="Byte"), 16)}, name="<anon>", label="None"), "authIpFilterId": SimTypeLongLong(signed=False, label="UInt64")}, name="IKEEXT_TRAFFIC0", pack=False, align=None), "ikeProposal": SimStruct({"cipherAlgorithm": SimStruct({"algoIdentifier": SimTypeInt(signed=False, label="IKEEXT_CIPHER_TYPE"), "keyLen": SimTypeInt(signed=False, label="UInt32"), "rounds": SimTypeInt(signed=False, label="UInt32")}, name="IKEEXT_CIPHER_ALGORITHM0", pack=False, align=None), "integrityAlgorithm": SimStruct({"algoIdentifier": SimTypeInt(signed=False, label="IKEEXT_INTEGRITY_TYPE")}, name="IKEEXT_INTEGRITY_ALGORITHM0", pack=False, align=None), "maxLifetimeSeconds": SimTypeInt(signed=False, label="UInt32"), "dhGroup": SimTypeInt(signed=False, label="IKEEXT_DH_GROUP"), "quickModeLimit": SimTypeInt(signed=False, label="UInt32")}, name="IKEEXT_PROPOSAL0", pack=False, align=None), "cookiePair": SimStruct({"initiator": SimTypeLongLong(signed=False, label="UInt64"), "responder": SimTypeLongLong(signed=False, label="UInt64")}, name="IKEEXT_COOKIE_PAIR0", pack=False, align=None), "ikeCredentials": SimStruct({"numCredentials": SimTypeInt(signed=False, label="UInt32"), "credentials": SimTypePointer(SimStruct({"localCredentials": SimStruct({"authenticationMethodType": SimTypeInt(signed=False, label="IKEEXT_AUTHENTICATION_METHOD_TYPE"), "impersonationType": SimTypeInt(signed=False, label="IKEEXT_AUTHENTICATION_IMPERSONATION_TYPE"), "Anonymous": SimUnion({"presharedKey": SimTypePointer(SimStruct({"presharedKey": SimStruct({"size": SimTypeInt(signed=False, label="UInt32"), "data": SimTypePointer(SimTypeChar(label="Byte"), offset=0)}, name="FWP_BYTE_BLOB", pack=False, align=None)}, name="IKEEXT_PRESHARED_KEY_AUTHENTICATION0", pack=False, align=None), offset=0), "certificate": SimTypePointer(SimStruct({"subjectName": SimStruct({"size": SimTypeInt(signed=False, label="UInt32"), "data": SimTypePointer(SimTypeChar(label="Byte"), offset=0)}, name="FWP_BYTE_BLOB", pack=False, align=None), "certHash": SimStruct({"size": SimTypeInt(signed=False, label="UInt32"), "data": SimTypePointer(SimTypeChar(label="Byte"), offset=0)}, name="FWP_BYTE_BLOB", pack=False, align=None), "flags": SimTypeInt(signed=False, label="UInt32")}, name="IKEEXT_CERTIFICATE_CREDENTIAL0", pack=False, align=None), offset=0), "name": SimTypePointer(SimStruct({"principalName": SimTypePointer(SimTypeChar(label="Char"), offset=0)}, name="IKEEXT_NAME_CREDENTIAL0", pack=False, align=None), offset=0)}, name="<anon>", label="None")}, name="IKEEXT_CREDENTIAL0", pack=False, align=None), "peerCredentials": SimStruct({"authenticationMethodType": SimTypeInt(signed=False, label="IKEEXT_AUTHENTICATION_METHOD_TYPE"), "impersonationType": SimTypeInt(signed=False, label="IKEEXT_AUTHENTICATION_IMPERSONATION_TYPE"), "Anonymous": SimUnion({"presharedKey": SimTypePointer(SimStruct({"presharedKey": SimStruct({"size": SimTypeInt(signed=False, label="UInt32"), "data": SimTypePointer(SimTypeChar(label="Byte"), offset=0)}, name="FWP_BYTE_BLOB", pack=False, align=None)}, name="IKEEXT_PRESHARED_KEY_AUTHENTICATION0", pack=False, align=None), offset=0), "certificate": SimTypePointer(SimStruct({"subjectName": SimStruct({"size": SimTypeInt(signed=False, label="UInt32"), "data": SimTypePointer(SimTypeChar(label="Byte"), offset=0)}, name="FWP_BYTE_BLOB", pack=False, align=None), "certHash": SimStruct({"size": SimTypeInt(signed=False, label="UInt32"), "data": SimTypePointer(SimTypeChar(label="Byte"), offset=0)}, name="FWP_BYTE_BLOB", pack=False, align=None), "flags": SimTypeInt(signed=False, label="UInt32")}, name="IKEEXT_CERTIFICATE_CREDENTIAL0", pack=False, align=None), offset=0), "name": SimTypePointer(SimStruct({"principalName": SimTypePointer(SimTypeChar(label="Char"), offset=0)}, name="IKEEXT_NAME_CREDENTIAL0", pack=False, align=None), offset=0)}, name="<anon>", label="None")}, name="IKEEXT_CREDENTIAL0", pack=False, align=None)}, name="IKEEXT_CREDENTIAL_PAIR0", pack=False, align=None), offset=0)}, name="IKEEXT_CREDENTIALS0", pack=False, align=None), "ikePolicyKey": SimTypeBottom(label="Guid"), "virtualIfTunnelId": SimTypeLongLong(signed=False, label="UInt64")}, name="IKEEXT_SA_DETAILS0", pack=False, align=None), offset=0), offset=0), offset=0), SimTypePointer(SimTypeInt(signed=False, label="UInt32"), offset=0)], SimTypeInt(signed=False, label="UInt32"), arg_names=["engineHandle", "enumHandle", "numEntriesRequested", "entries", "numEntriesReturned"]),
        # 
        'IkeextSaEnum1': SimTypeFunction([SimTypePointer(SimTypeInt(signed=True, label="Int"), label="IntPtr", offset=0), SimTypePointer(SimTypeInt(signed=True, label="Int"), label="IntPtr", offset=0), SimTypeInt(signed=False, label="UInt32"), SimTypePointer(SimTypePointer(SimTypePointer(SimStruct({"saId": SimTypeLongLong(signed=False, label="UInt64"), "keyModuleType": SimTypeInt(signed=False, label="IKEEXT_KEY_MODULE_TYPE"), "ipVersion": SimTypeInt(signed=False, label="FWP_IP_VERSION"), "Anonymous": SimUnion({"v4UdpEncapsulation": SimTypePointer(SimTypeBottom(label="IPSEC_V4_UDP_ENCAPSULATION0"), offset=0)}, name="<anon>", label="None"), "ikeTraffic": SimStruct({"ipVersion": SimTypeInt(signed=False, label="FWP_IP_VERSION"), "Anonymous1": SimUnion({"localV4Address": SimTypeInt(signed=False, label="UInt32"), "localV6Address": SimTypeFixedSizeArray(SimTypeChar(label="Byte"), 16)}, name="<anon>", label="None"), "Anonymous2": SimUnion({"remoteV4Address": SimTypeInt(signed=False, label="UInt32"), "remoteV6Address": SimTypeFixedSizeArray(SimTypeChar(label="Byte"), 16)}, name="<anon>", label="None"), "authIpFilterId": SimTypeLongLong(signed=False, label="UInt64")}, name="IKEEXT_TRAFFIC0", pack=False, align=None), "ikeProposal": SimStruct({"cipherAlgorithm": SimStruct({"algoIdentifier": SimTypeInt(signed=False, label="IKEEXT_CIPHER_TYPE"), "keyLen": SimTypeInt(signed=False, label="UInt32"), "rounds": SimTypeInt(signed=False, label="UInt32")}, name="IKEEXT_CIPHER_ALGORITHM0", pack=False, align=None), "integrityAlgorithm": SimStruct({"algoIdentifier": SimTypeInt(signed=False, label="IKEEXT_INTEGRITY_TYPE")}, name="IKEEXT_INTEGRITY_ALGORITHM0", pack=False, align=None), "maxLifetimeSeconds": SimTypeInt(signed=False, label="UInt32"), "dhGroup": SimTypeInt(signed=False, label="IKEEXT_DH_GROUP"), "quickModeLimit": SimTypeInt(signed=False, label="UInt32")}, name="IKEEXT_PROPOSAL0", pack=False, align=None), "cookiePair": SimStruct({"initiator": SimTypeLongLong(signed=False, label="UInt64"), "responder": SimTypeLongLong(signed=False, label="UInt64")}, name="IKEEXT_COOKIE_PAIR0", pack=False, align=None), "ikeCredentials": SimStruct({"numCredentials": SimTypeInt(signed=False, label="UInt32"), "credentials": SimTypePointer(SimStruct({"localCredentials": SimStruct({"authenticationMethodType": SimTypeInt(signed=False, label="IKEEXT_AUTHENTICATION_METHOD_TYPE"), "impersonationType": SimTypeInt(signed=False, label="IKEEXT_AUTHENTICATION_IMPERSONATION_TYPE"), "Anonymous": SimUnion({"presharedKey": SimTypePointer(SimStruct({"presharedKey": SimStruct({"size": SimTypeInt(signed=False, label="UInt32"), "data": SimTypePointer(SimTypeChar(label="Byte"), offset=0)}, name="FWP_BYTE_BLOB", pack=False, align=None), "flags": SimTypeInt(signed=False, label="IKEEXT_PRESHARED_KEY_AUTHENTICATION_FLAGS")}, name="IKEEXT_PRESHARED_KEY_AUTHENTICATION1", pack=False, align=None), offset=0), "certificate": SimTypePointer(SimStruct({"subjectName": SimStruct({"size": SimTypeInt(signed=False, label="UInt32"), "data": SimTypePointer(SimTypeChar(label="Byte"), offset=0)}, name="FWP_BYTE_BLOB", pack=False, align=None), "certHash": SimStruct({"size": SimTypeInt(signed=False, label="UInt32"), "data": SimTypePointer(SimTypeChar(label="Byte"), offset=0)}, name="FWP_BYTE_BLOB", pack=False, align=None), "flags": SimTypeInt(signed=False, label="UInt32"), "certificate": SimStruct({"size": SimTypeInt(signed=False, label="UInt32"), "data": SimTypePointer(SimTypeChar(label="Byte"), offset=0)}, name="FWP_BYTE_BLOB", pack=False, align=None)}, name="IKEEXT_CERTIFICATE_CREDENTIAL1", pack=False, align=None), offset=0), "name": SimTypePointer(SimStruct({"principalName": SimTypePointer(SimTypeChar(label="Char"), offset=0)}, name="IKEEXT_NAME_CREDENTIAL0", pack=False, align=None), offset=0)}, name="<anon>", label="None")}, name="IKEEXT_CREDENTIAL1", pack=False, align=None), "peerCredentials": SimStruct({"authenticationMethodType": SimTypeInt(signed=False, label="IKEEXT_AUTHENTICATION_METHOD_TYPE"), "impersonationType": SimTypeInt(signed=False, label="IKEEXT_AUTHENTICATION_IMPERSONATION_TYPE"), "Anonymous": SimUnion({"presharedKey": SimTypePointer(SimStruct({"presharedKey": SimStruct({"size": SimTypeInt(signed=False, label="UInt32"), "data": SimTypePointer(SimTypeChar(label="Byte"), offset=0)}, name="FWP_BYTE_BLOB", pack=False, align=None), "flags": SimTypeInt(signed=False, label="IKEEXT_PRESHARED_KEY_AUTHENTICATION_FLAGS")}, name="IKEEXT_PRESHARED_KEY_AUTHENTICATION1", pack=False, align=None), offset=0), "certificate": SimTypePointer(SimStruct({"subjectName": SimStruct({"size": SimTypeInt(signed=False, label="UInt32"), "data": SimTypePointer(SimTypeChar(label="Byte"), offset=0)}, name="FWP_BYTE_BLOB", pack=False, align=None), "certHash": SimStruct({"size": SimTypeInt(signed=False, label="UInt32"), "data": SimTypePointer(SimTypeChar(label="Byte"), offset=0)}, name="FWP_BYTE_BLOB", pack=False, align=None), "flags": SimTypeInt(signed=False, label="UInt32"), "certificate": SimStruct({"size": SimTypeInt(signed=False, label="UInt32"), "data": SimTypePointer(SimTypeChar(label="Byte"), offset=0)}, name="FWP_BYTE_BLOB", pack=False, align=None)}, name="IKEEXT_CERTIFICATE_CREDENTIAL1", pack=False, align=None), offset=0), "name": SimTypePointer(SimStruct({"principalName": SimTypePointer(SimTypeChar(label="Char"), offset=0)}, name="IKEEXT_NAME_CREDENTIAL0", pack=False, align=None), offset=0)}, name="<anon>", label="None")}, name="IKEEXT_CREDENTIAL1", pack=False, align=None)}, name="IKEEXT_CREDENTIAL_PAIR1", pack=False, align=None), offset=0)}, name="IKEEXT_CREDENTIALS1", pack=False, align=None), "ikePolicyKey": SimTypeBottom(label="Guid"), "virtualIfTunnelId": SimTypeLongLong(signed=False, label="UInt64"), "correlationKey": SimStruct({"size": SimTypeInt(signed=False, label="UInt32"), "data": SimTypePointer(SimTypeChar(label="Byte"), offset=0)}, name="FWP_BYTE_BLOB", pack=False, align=None)}, name="IKEEXT_SA_DETAILS1", pack=False, align=None), offset=0), offset=0), offset=0), SimTypePointer(SimTypeInt(signed=False, label="UInt32"), offset=0)], SimTypeInt(signed=False, label="UInt32"), arg_names=["engineHandle", "enumHandle", "numEntriesRequested", "entries", "numEntriesReturned"]),
        # 
        'IkeextSaEnum2': SimTypeFunction([SimTypePointer(SimTypeInt(signed=True, label="Int"), label="IntPtr", offset=0), SimTypePointer(SimTypeInt(signed=True, label="Int"), label="IntPtr", offset=0), SimTypeInt(signed=False, label="UInt32"), SimTypePointer(SimTypePointer(SimTypePointer(SimStruct({"saId": SimTypeLongLong(signed=False, label="UInt64"), "keyModuleType": SimTypeInt(signed=False, label="IKEEXT_KEY_MODULE_TYPE"), "ipVersion": SimTypeInt(signed=False, label="FWP_IP_VERSION"), "Anonymous": SimUnion({"v4UdpEncapsulation": SimTypePointer(SimTypeBottom(label="IPSEC_V4_UDP_ENCAPSULATION0"), offset=0)}, name="<anon>", label="None"), "ikeTraffic": SimStruct({"ipVersion": SimTypeInt(signed=False, label="FWP_IP_VERSION"), "Anonymous1": SimUnion({"localV4Address": SimTypeInt(signed=False, label="UInt32"), "localV6Address": SimTypeFixedSizeArray(SimTypeChar(label="Byte"), 16)}, name="<anon>", label="None"), "Anonymous2": SimUnion({"remoteV4Address": SimTypeInt(signed=False, label="UInt32"), "remoteV6Address": SimTypeFixedSizeArray(SimTypeChar(label="Byte"), 16)}, name="<anon>", label="None"), "authIpFilterId": SimTypeLongLong(signed=False, label="UInt64")}, name="IKEEXT_TRAFFIC0", pack=False, align=None), "ikeProposal": SimStruct({"cipherAlgorithm": SimStruct({"algoIdentifier": SimTypeInt(signed=False, label="IKEEXT_CIPHER_TYPE"), "keyLen": SimTypeInt(signed=False, label="UInt32"), "rounds": SimTypeInt(signed=False, label="UInt32")}, name="IKEEXT_CIPHER_ALGORITHM0", pack=False, align=None), "integrityAlgorithm": SimStruct({"algoIdentifier": SimTypeInt(signed=False, label="IKEEXT_INTEGRITY_TYPE")}, name="IKEEXT_INTEGRITY_ALGORITHM0", pack=False, align=None), "maxLifetimeSeconds": SimTypeInt(signed=False, label="UInt32"), "dhGroup": SimTypeInt(signed=False, label="IKEEXT_DH_GROUP"), "quickModeLimit": SimTypeInt(signed=False, label="UInt32")}, name="IKEEXT_PROPOSAL0", pack=False, align=None), "cookiePair": SimStruct({"initiator": SimTypeLongLong(signed=False, label="UInt64"), "responder": SimTypeLongLong(signed=False, label="UInt64")}, name="IKEEXT_COOKIE_PAIR0", pack=False, align=None), "ikeCredentials": SimStruct({"numCredentials": SimTypeInt(signed=False, label="UInt32"), "credentials": SimTypePointer(SimStruct({"localCredentials": SimStruct({"authenticationMethodType": SimTypeInt(signed=False, label="IKEEXT_AUTHENTICATION_METHOD_TYPE"), "impersonationType": SimTypeInt(signed=False, label="IKEEXT_AUTHENTICATION_IMPERSONATION_TYPE"), "Anonymous": SimUnion({"presharedKey": SimTypePointer(SimStruct({"presharedKey": SimStruct({"size": SimTypeInt(signed=False, label="UInt32"), "data": SimTypePointer(SimTypeChar(label="Byte"), offset=0)}, name="FWP_BYTE_BLOB", pack=False, align=None), "flags": SimTypeInt(signed=False, label="IKEEXT_PRESHARED_KEY_AUTHENTICATION_FLAGS")}, name="IKEEXT_PRESHARED_KEY_AUTHENTICATION1", pack=False, align=None), offset=0), "certificate": SimTypePointer(SimStruct({"subjectName": SimStruct({"size": SimTypeInt(signed=False, label="UInt32"), "data": SimTypePointer(SimTypeChar(label="Byte"), offset=0)}, name="FWP_BYTE_BLOB", pack=False, align=None), "certHash": SimStruct({"size": SimTypeInt(signed=False, label="UInt32"), "data": SimTypePointer(SimTypeChar(label="Byte"), offset=0)}, name="FWP_BYTE_BLOB", pack=False, align=None), "flags": SimTypeInt(signed=False, label="UInt32"), "certificate": SimStruct({"size": SimTypeInt(signed=False, label="UInt32"), "data": SimTypePointer(SimTypeChar(label="Byte"), offset=0)}, name="FWP_BYTE_BLOB", pack=False, align=None)}, name="IKEEXT_CERTIFICATE_CREDENTIAL1", pack=False, align=None), offset=0), "name": SimTypePointer(SimStruct({"principalName": SimTypePointer(SimTypeChar(label="Char"), offset=0)}, name="IKEEXT_NAME_CREDENTIAL0", pack=False, align=None), offset=0)}, name="<anon>", label="None")}, name="IKEEXT_CREDENTIAL2", pack=False, align=None), "peerCredentials": SimStruct({"authenticationMethodType": SimTypeInt(signed=False, label="IKEEXT_AUTHENTICATION_METHOD_TYPE"), "impersonationType": SimTypeInt(signed=False, label="IKEEXT_AUTHENTICATION_IMPERSONATION_TYPE"), "Anonymous": SimUnion({"presharedKey": SimTypePointer(SimStruct({"presharedKey": SimStruct({"size": SimTypeInt(signed=False, label="UInt32"), "data": SimTypePointer(SimTypeChar(label="Byte"), offset=0)}, name="FWP_BYTE_BLOB", pack=False, align=None), "flags": SimTypeInt(signed=False, label="IKEEXT_PRESHARED_KEY_AUTHENTICATION_FLAGS")}, name="IKEEXT_PRESHARED_KEY_AUTHENTICATION1", pack=False, align=None), offset=0), "certificate": SimTypePointer(SimStruct({"subjectName": SimStruct({"size": SimTypeInt(signed=False, label="UInt32"), "data": SimTypePointer(SimTypeChar(label="Byte"), offset=0)}, name="FWP_BYTE_BLOB", pack=False, align=None), "certHash": SimStruct({"size": SimTypeInt(signed=False, label="UInt32"), "data": SimTypePointer(SimTypeChar(label="Byte"), offset=0)}, name="FWP_BYTE_BLOB", pack=False, align=None), "flags": SimTypeInt(signed=False, label="UInt32"), "certificate": SimStruct({"size": SimTypeInt(signed=False, label="UInt32"), "data": SimTypePointer(SimTypeChar(label="Byte"), offset=0)}, name="FWP_BYTE_BLOB", pack=False, align=None)}, name="IKEEXT_CERTIFICATE_CREDENTIAL1", pack=False, align=None), offset=0), "name": SimTypePointer(SimStruct({"principalName": SimTypePointer(SimTypeChar(label="Char"), offset=0)}, name="IKEEXT_NAME_CREDENTIAL0", pack=False, align=None), offset=0)}, name="<anon>", label="None")}, name="IKEEXT_CREDENTIAL2", pack=False, align=None)}, name="IKEEXT_CREDENTIAL_PAIR2", pack=False, align=None), offset=0)}, name="IKEEXT_CREDENTIALS2", pack=False, align=None), "ikePolicyKey": SimTypeBottom(label="Guid"), "virtualIfTunnelId": SimTypeLongLong(signed=False, label="UInt64"), "correlationKey": SimStruct({"size": SimTypeInt(signed=False, label="UInt32"), "data": SimTypePointer(SimTypeChar(label="Byte"), offset=0)}, name="FWP_BYTE_BLOB", pack=False, align=None)}, name="IKEEXT_SA_DETAILS2", pack=False, align=None), offset=0), offset=0), offset=0), SimTypePointer(SimTypeInt(signed=False, label="UInt32"), offset=0)], SimTypeInt(signed=False, label="UInt32"), arg_names=["engineHandle", "enumHandle", "numEntriesRequested", "entries", "numEntriesReturned"]),
        # 
        'IkeextSaDestroyEnumHandle0': SimTypeFunction([SimTypePointer(SimTypeInt(signed=True, label="Int"), label="IntPtr", offset=0), SimTypePointer(SimTypeInt(signed=True, label="Int"), label="IntPtr", offset=0)], SimTypeInt(signed=False, label="UInt32"), arg_names=["engineHandle", "enumHandle"]),
        # 
        'IkeextSaDbGetSecurityInfo0': SimTypeFunction([SimTypePointer(SimTypeInt(signed=True, label="Int"), label="IntPtr", offset=0), SimTypeInt(signed=False, label="UInt32"), SimTypePointer(SimTypePointer(SimTypeInt(signed=True, label="Int"), label="IntPtr", offset=0), offset=0), SimTypePointer(SimTypePointer(SimTypeInt(signed=True, label="Int"), label="IntPtr", offset=0), offset=0), SimTypePointer(SimTypePointer(SimStruct({"AclRevision": SimTypeChar(label="Byte"), "Sbz1": SimTypeChar(label="Byte"), "AclSize": SimTypeShort(signed=False, label="UInt16"), "AceCount": SimTypeShort(signed=False, label="UInt16"), "Sbz2": SimTypeShort(signed=False, label="UInt16")}, name="ACL", pack=False, align=None), offset=0), offset=0), SimTypePointer(SimTypePointer(SimStruct({"AclRevision": SimTypeChar(label="Byte"), "Sbz1": SimTypeChar(label="Byte"), "AclSize": SimTypeShort(signed=False, label="UInt16"), "AceCount": SimTypeShort(signed=False, label="UInt16"), "Sbz2": SimTypeShort(signed=False, label="UInt16")}, name="ACL", pack=False, align=None), offset=0), offset=0), SimTypePointer(SimTypePointer(SimStruct({"Revision": SimTypeChar(label="Byte"), "Sbz1": SimTypeChar(label="Byte"), "Control": SimTypeShort(signed=False, label="UInt16"), "Owner": SimTypePointer(SimTypeInt(signed=True, label="Int"), label="IntPtr", offset=0), "Group": SimTypePointer(SimTypeInt(signed=True, label="Int"), label="IntPtr", offset=0), "Sacl": SimTypePointer(SimStruct({"AclRevision": SimTypeChar(label="Byte"), "Sbz1": SimTypeChar(label="Byte"), "AclSize": SimTypeShort(signed=False, label="UInt16"), "AceCount": SimTypeShort(signed=False, label="UInt16"), "Sbz2": SimTypeShort(signed=False, label="UInt16")}, name="ACL", pack=False, align=None), offset=0), "Dacl": SimTypePointer(SimStruct({"AclRevision": SimTypeChar(label="Byte"), "Sbz1": SimTypeChar(label="Byte"), "AclSize": SimTypeShort(signed=False, label="UInt16"), "AceCount": SimTypeShort(signed=False, label="UInt16"), "Sbz2": SimTypeShort(signed=False, label="UInt16")}, name="ACL", pack=False, align=None), offset=0)}, name="SECURITY_DESCRIPTOR", pack=False, align=None), offset=0), offset=0)], SimTypeInt(signed=False, label="UInt32"), arg_names=["engineHandle", "securityInfo", "sidOwner", "sidGroup", "dacl", "sacl", "securityDescriptor"]),
        # 
        'IkeextSaDbSetSecurityInfo0': SimTypeFunction([SimTypePointer(SimTypeInt(signed=True, label="Int"), label="IntPtr", offset=0), SimTypeInt(signed=False, label="UInt32"), SimTypePointer(SimStruct({"Revision": SimTypeChar(label="Byte"), "SubAuthorityCount": SimTypeChar(label="Byte"), "IdentifierAuthority": SimStruct({"Value": SimTypeFixedSizeArray(SimTypeChar(label="Byte"), 6)}, name="SID_IDENTIFIER_AUTHORITY", pack=False, align=None), "SubAuthority": SimTypePointer(SimTypeInt(signed=False, label="UInt32"), offset=0)}, name="SID", pack=False, align=None), offset=0), SimTypePointer(SimStruct({"Revision": SimTypeChar(label="Byte"), "SubAuthorityCount": SimTypeChar(label="Byte"), "IdentifierAuthority": SimStruct({"Value": SimTypeFixedSizeArray(SimTypeChar(label="Byte"), 6)}, name="SID_IDENTIFIER_AUTHORITY", pack=False, align=None), "SubAuthority": SimTypePointer(SimTypeInt(signed=False, label="UInt32"), offset=0)}, name="SID", pack=False, align=None), offset=0), SimTypePointer(SimStruct({"AclRevision": SimTypeChar(label="Byte"), "Sbz1": SimTypeChar(label="Byte"), "AclSize": SimTypeShort(signed=False, label="UInt16"), "AceCount": SimTypeShort(signed=False, label="UInt16"), "Sbz2": SimTypeShort(signed=False, label="UInt16")}, name="ACL", pack=False, align=None), offset=0), SimTypePointer(SimStruct({"AclRevision": SimTypeChar(label="Byte"), "Sbz1": SimTypeChar(label="Byte"), "AclSize": SimTypeShort(signed=False, label="UInt16"), "AceCount": SimTypeShort(signed=False, label="UInt16"), "Sbz2": SimTypeShort(signed=False, label="UInt16")}, name="ACL", pack=False, align=None), offset=0)], SimTypeInt(signed=False, label="UInt32"), arg_names=["engineHandle", "securityInfo", "sidOwner", "sidGroup", "dacl", "sacl"]),
        # 
        'FwpmNetEventCreateEnumHandle0': SimTypeFunction([SimTypePointer(SimTypeInt(signed=True, label="Int"), label="IntPtr", offset=0), SimTypePointer(SimStruct({"startTime": SimStruct({"dwLowDateTime": SimTypeInt(signed=False, label="UInt32"), "dwHighDateTime": SimTypeInt(signed=False, label="UInt32")}, name="FILETIME", pack=False, align=None), "endTime": SimStruct({"dwLowDateTime": SimTypeInt(signed=False, label="UInt32"), "dwHighDateTime": SimTypeInt(signed=False, label="UInt32")}, name="FILETIME", pack=False, align=None), "numFilterConditions": SimTypeInt(signed=False, label="UInt32"), "filterCondition": SimTypePointer(SimStruct({"fieldKey": SimTypeBottom(label="Guid"), "matchType": SimTypeInt(signed=False, label="FWP_MATCH_TYPE"), "conditionValue": SimStruct({"type": SimTypeInt(signed=False, label="FWP_DATA_TYPE"), "Anonymous": SimUnion({"uint8": SimTypeChar(label="Byte"), "uint16": SimTypeShort(signed=False, label="UInt16"), "uint32": SimTypeInt(signed=False, label="UInt32"), "uint64": SimTypePointer(SimTypeLongLong(signed=False, label="UInt64"), offset=0), "int8": SimTypeChar(label="SByte"), "int16": SimTypeShort(signed=True, label="Int16"), "int32": SimTypeInt(signed=True, label="Int32"), "int64": SimTypePointer(SimTypeLongLong(signed=True, label="Int64"), offset=0), "float32": SimTypeFloat(size=32), "double64": SimTypePointer(SimTypeFloat(size=64), offset=0), "byteArray16": SimTypePointer(SimStruct({"byteArray16": SimTypeFixedSizeArray(SimTypeChar(label="Byte"), 16)}, name="FWP_BYTE_ARRAY16", pack=False, align=None), offset=0), "byteBlob": SimTypePointer(SimStruct({"size": SimTypeInt(signed=False, label="UInt32"), "data": SimTypePointer(SimTypeChar(label="Byte"), offset=0)}, name="FWP_BYTE_BLOB", pack=False, align=None), offset=0), "sid": SimTypePointer(SimTypeBottom(label="SID"), offset=0), "sd": SimTypePointer(SimStruct({"size": SimTypeInt(signed=False, label="UInt32"), "data": SimTypePointer(SimTypeChar(label="Byte"), offset=0)}, name="FWP_BYTE_BLOB", pack=False, align=None), offset=0), "tokenInformation": SimTypePointer(SimStruct({"sidCount": SimTypeInt(signed=False, label="UInt32"), "sids": SimTypePointer(SimTypeBottom(label="SID_AND_ATTRIBUTES"), offset=0), "restrictedSidCount": SimTypeInt(signed=False, label="UInt32"), "restrictedSids": SimTypePointer(SimTypeBottom(label="SID_AND_ATTRIBUTES"), offset=0)}, name="FWP_TOKEN_INFORMATION", pack=False, align=None), offset=0), "tokenAccessInformation": SimTypePointer(SimStruct({"size": SimTypeInt(signed=False, label="UInt32"), "data": SimTypePointer(SimTypeChar(label="Byte"), offset=0)}, name="FWP_BYTE_BLOB", pack=False, align=None), offset=0), "unicodeString": SimTypePointer(SimTypeChar(label="Char"), offset=0), "byteArray6": SimTypePointer(SimStruct({"byteArray6": SimTypeFixedSizeArray(SimTypeChar(label="Byte"), 6)}, name="FWP_BYTE_ARRAY6", pack=False, align=None), offset=0), "bitmapArray64": SimTypePointer(SimStruct({"bitmapArray64": SimTypeFixedSizeArray(SimTypeChar(label="Byte"), 8)}, name="FWP_BITMAP_ARRAY64_", pack=False, align=None), offset=0), "v4AddrMask": SimTypePointer(SimStruct({"addr": SimTypeInt(signed=False, label="UInt32"), "mask": SimTypeInt(signed=False, label="UInt32")}, name="FWP_V4_ADDR_AND_MASK", pack=False, align=None), offset=0), "v6AddrMask": SimTypePointer(SimStruct({"addr": SimTypeFixedSizeArray(SimTypeChar(label="Byte"), 16), "prefixLength": SimTypeChar(label="Byte")}, name="FWP_V6_ADDR_AND_MASK", pack=False, align=None), offset=0), "rangeValue": SimTypePointer(SimStruct({"valueLow": SimStruct({"type": SimTypeInt(signed=False, label="FWP_DATA_TYPE"), "Anonymous": SimUnion({"uint8": SimTypeChar(label="Byte"), "uint16": SimTypeShort(signed=False, label="UInt16"), "uint32": SimTypeInt(signed=False, label="UInt32"), "uint64": SimTypePointer(SimTypeLongLong(signed=False, label="UInt64"), offset=0), "int8": SimTypeChar(label="SByte"), "int16": SimTypeShort(signed=True, label="Int16"), "int32": SimTypeInt(signed=True, label="Int32"), "int64": SimTypePointer(SimTypeLongLong(signed=True, label="Int64"), offset=0), "float32": SimTypeFloat(size=32), "double64": SimTypePointer(SimTypeFloat(size=64), offset=0), "byteArray16": SimTypePointer(SimStruct({"byteArray16": SimTypeFixedSizeArray(SimTypeChar(label="Byte"), 16)}, name="FWP_BYTE_ARRAY16", pack=False, align=None), offset=0), "byteBlob": SimTypePointer(SimStruct({"size": SimTypeInt(signed=False, label="UInt32"), "data": SimTypePointer(SimTypeChar(label="Byte"), offset=0)}, name="FWP_BYTE_BLOB", pack=False, align=None), offset=0), "sid": SimTypePointer(SimTypeBottom(label="SID"), offset=0), "sd": SimTypePointer(SimStruct({"size": SimTypeInt(signed=False, label="UInt32"), "data": SimTypePointer(SimTypeChar(label="Byte"), offset=0)}, name="FWP_BYTE_BLOB", pack=False, align=None), offset=0), "tokenInformation": SimTypePointer(SimStruct({"sidCount": SimTypeInt(signed=False, label="UInt32"), "sids": SimTypePointer(SimTypeBottom(label="SID_AND_ATTRIBUTES"), offset=0), "restrictedSidCount": SimTypeInt(signed=False, label="UInt32"), "restrictedSids": SimTypePointer(SimTypeBottom(label="SID_AND_ATTRIBUTES"), offset=0)}, name="FWP_TOKEN_INFORMATION", pack=False, align=None), offset=0), "tokenAccessInformation": SimTypePointer(SimStruct({"size": SimTypeInt(signed=False, label="UInt32"), "data": SimTypePointer(SimTypeChar(label="Byte"), offset=0)}, name="FWP_BYTE_BLOB", pack=False, align=None), offset=0), "unicodeString": SimTypePointer(SimTypeChar(label="Char"), offset=0), "byteArray6": SimTypePointer(SimStruct({"byteArray6": SimTypeFixedSizeArray(SimTypeChar(label="Byte"), 6)}, name="FWP_BYTE_ARRAY6", pack=False, align=None), offset=0), "bitmapArray64": SimTypePointer(SimStruct({"bitmapArray64": SimTypeFixedSizeArray(SimTypeChar(label="Byte"), 8)}, name="FWP_BITMAP_ARRAY64_", pack=False, align=None), offset=0)}, name="<anon>", label="None")}, name="FWP_VALUE0", pack=False, align=None), "valueHigh": SimStruct({"type": SimTypeInt(signed=False, label="FWP_DATA_TYPE"), "Anonymous": SimUnion({"uint8": SimTypeChar(label="Byte"), "uint16": SimTypeShort(signed=False, label="UInt16"), "uint32": SimTypeInt(signed=False, label="UInt32"), "uint64": SimTypePointer(SimTypeLongLong(signed=False, label="UInt64"), offset=0), "int8": SimTypeChar(label="SByte"), "int16": SimTypeShort(signed=True, label="Int16"), "int32": SimTypeInt(signed=True, label="Int32"), "int64": SimTypePointer(SimTypeLongLong(signed=True, label="Int64"), offset=0), "float32": SimTypeFloat(size=32), "double64": SimTypePointer(SimTypeFloat(size=64), offset=0), "byteArray16": SimTypePointer(SimStruct({"byteArray16": SimTypeFixedSizeArray(SimTypeChar(label="Byte"), 16)}, name="FWP_BYTE_ARRAY16", pack=False, align=None), offset=0), "byteBlob": SimTypePointer(SimStruct({"size": SimTypeInt(signed=False, label="UInt32"), "data": SimTypePointer(SimTypeChar(label="Byte"), offset=0)}, name="FWP_BYTE_BLOB", pack=False, align=None), offset=0), "sid": SimTypePointer(SimTypeBottom(label="SID"), offset=0), "sd": SimTypePointer(SimStruct({"size": SimTypeInt(signed=False, label="UInt32"), "data": SimTypePointer(SimTypeChar(label="Byte"), offset=0)}, name="FWP_BYTE_BLOB", pack=False, align=None), offset=0), "tokenInformation": SimTypePointer(SimStruct({"sidCount": SimTypeInt(signed=False, label="UInt32"), "sids": SimTypePointer(SimTypeBottom(label="SID_AND_ATTRIBUTES"), offset=0), "restrictedSidCount": SimTypeInt(signed=False, label="UInt32"), "restrictedSids": SimTypePointer(SimTypeBottom(label="SID_AND_ATTRIBUTES"), offset=0)}, name="FWP_TOKEN_INFORMATION", pack=False, align=None), offset=0), "tokenAccessInformation": SimTypePointer(SimStruct({"size": SimTypeInt(signed=False, label="UInt32"), "data": SimTypePointer(SimTypeChar(label="Byte"), offset=0)}, name="FWP_BYTE_BLOB", pack=False, align=None), offset=0), "unicodeString": SimTypePointer(SimTypeChar(label="Char"), offset=0), "byteArray6": SimTypePointer(SimStruct({"byteArray6": SimTypeFixedSizeArray(SimTypeChar(label="Byte"), 6)}, name="FWP_BYTE_ARRAY6", pack=False, align=None), offset=0), "bitmapArray64": SimTypePointer(SimStruct({"bitmapArray64": SimTypeFixedSizeArray(SimTypeChar(label="Byte"), 8)}, name="FWP_BITMAP_ARRAY64_", pack=False, align=None), offset=0)}, name="<anon>", label="None")}, name="FWP_VALUE0", pack=False, align=None)}, name="FWP_RANGE0", pack=False, align=None), offset=0)}, name="<anon>", label="None")}, name="FWP_CONDITION_VALUE0", pack=False, align=None)}, name="FWPM_FILTER_CONDITION0", pack=False, align=None), offset=0)}, name="FWPM_NET_EVENT_ENUM_TEMPLATE0", pack=False, align=None), offset=0), SimTypePointer(SimTypePointer(SimTypeInt(signed=True, label="Int"), label="IntPtr", offset=0), offset=0)], SimTypeInt(signed=False, label="UInt32"), arg_names=["engineHandle", "enumTemplate", "enumHandle"]),
        # 
        'FwpmNetEventEnum0': SimTypeFunction([SimTypePointer(SimTypeInt(signed=True, label="Int"), label="IntPtr", offset=0), SimTypePointer(SimTypeInt(signed=True, label="Int"), label="IntPtr", offset=0), SimTypeInt(signed=False, label="UInt32"), SimTypePointer(SimTypePointer(SimTypePointer(SimStruct({"header": SimStruct({"timeStamp": SimStruct({"dwLowDateTime": SimTypeInt(signed=False, label="UInt32"), "dwHighDateTime": SimTypeInt(signed=False, label="UInt32")}, name="FILETIME", pack=False, align=None), "flags": SimTypeInt(signed=False, label="UInt32"), "ipVersion": SimTypeInt(signed=False, label="FWP_IP_VERSION"), "ipProtocol": SimTypeChar(label="Byte"), "Anonymous1": SimUnion({"localAddrV4": SimTypeInt(signed=False, label="UInt32"), "localAddrV6": SimStruct({"byteArray16": SimTypeFixedSizeArray(SimTypeChar(label="Byte"), 16)}, name="FWP_BYTE_ARRAY16", pack=False, align=None)}, name="<anon>", label="None"), "Anonymous2": SimUnion({"remoteAddrV4": SimTypeInt(signed=False, label="UInt32"), "remoteAddrV6": SimStruct({"byteArray16": SimTypeFixedSizeArray(SimTypeChar(label="Byte"), 16)}, name="FWP_BYTE_ARRAY16", pack=False, align=None)}, name="<anon>", label="None"), "localPort": SimTypeShort(signed=False, label="UInt16"), "remotePort": SimTypeShort(signed=False, label="UInt16"), "scopeId": SimTypeInt(signed=False, label="UInt32"), "appId": SimStruct({"size": SimTypeInt(signed=False, label="UInt32"), "data": SimTypePointer(SimTypeChar(label="Byte"), offset=0)}, name="FWP_BYTE_BLOB", pack=False, align=None), "userId": SimTypePointer(SimTypeBottom(label="SID"), offset=0)}, name="FWPM_NET_EVENT_HEADER0", pack=False, align=None), "type": SimTypeInt(signed=False, label="FWPM_NET_EVENT_TYPE"), "Anonymous": SimUnion({"ikeMmFailure": SimTypePointer(SimStruct({"failureErrorCode": SimTypeInt(signed=False, label="UInt32"), "failurePoint": SimTypeInt(signed=False, label="IPSEC_FAILURE_POINT"), "flags": SimTypeInt(signed=False, label="UInt32"), "keyingModuleType": SimTypeInt(signed=False, label="IKEEXT_KEY_MODULE_TYPE"), "mmState": SimTypeInt(signed=False, label="IKEEXT_MM_SA_STATE"), "saRole": SimTypeInt(signed=False, label="IKEEXT_SA_ROLE"), "mmAuthMethod": SimTypeInt(signed=False, label="IKEEXT_AUTHENTICATION_METHOD_TYPE"), "endCertHash": SimTypeFixedSizeArray(SimTypeChar(label="Byte"), 20), "mmId": SimTypeLongLong(signed=False, label="UInt64"), "mmFilterId": SimTypeLongLong(signed=False, label="UInt64")}, name="FWPM_NET_EVENT_IKEEXT_MM_FAILURE0", pack=False, align=None), offset=0), "ikeQmFailure": SimTypePointer(SimStruct({"failureErrorCode": SimTypeInt(signed=False, label="UInt32"), "failurePoint": SimTypeInt(signed=False, label="IPSEC_FAILURE_POINT"), "keyingModuleType": SimTypeInt(signed=False, label="IKEEXT_KEY_MODULE_TYPE"), "qmState": SimTypeInt(signed=False, label="IKEEXT_QM_SA_STATE"), "saRole": SimTypeInt(signed=False, label="IKEEXT_SA_ROLE"), "saTrafficType": SimTypeInt(signed=False, label="IPSEC_TRAFFIC_TYPE"), "Anonymous1": SimUnion({"localSubNet": SimStruct({"type": SimTypeInt(signed=False, label="FWP_DATA_TYPE"), "Anonymous": SimUnion({"uint8": SimTypeChar(label="Byte"), "uint16": SimTypeShort(signed=False, label="UInt16"), "uint32": SimTypeInt(signed=False, label="UInt32"), "uint64": SimTypePointer(SimTypeLongLong(signed=False, label="UInt64"), offset=0), "int8": SimTypeChar(label="SByte"), "int16": SimTypeShort(signed=True, label="Int16"), "int32": SimTypeInt(signed=True, label="Int32"), "int64": SimTypePointer(SimTypeLongLong(signed=True, label="Int64"), offset=0), "float32": SimTypeFloat(size=32), "double64": SimTypePointer(SimTypeFloat(size=64), offset=0), "byteArray16": SimTypePointer(SimStruct({"byteArray16": SimTypeFixedSizeArray(SimTypeChar(label="Byte"), 16)}, name="FWP_BYTE_ARRAY16", pack=False, align=None), offset=0), "byteBlob": SimTypePointer(SimStruct({"size": SimTypeInt(signed=False, label="UInt32"), "data": SimTypePointer(SimTypeChar(label="Byte"), offset=0)}, name="FWP_BYTE_BLOB", pack=False, align=None), offset=0), "sid": SimTypePointer(SimTypeBottom(label="SID"), offset=0), "sd": SimTypePointer(SimStruct({"size": SimTypeInt(signed=False, label="UInt32"), "data": SimTypePointer(SimTypeChar(label="Byte"), offset=0)}, name="FWP_BYTE_BLOB", pack=False, align=None), offset=0), "tokenInformation": SimTypePointer(SimStruct({"sidCount": SimTypeInt(signed=False, label="UInt32"), "sids": SimTypePointer(SimTypeBottom(label="SID_AND_ATTRIBUTES"), offset=0), "restrictedSidCount": SimTypeInt(signed=False, label="UInt32"), "restrictedSids": SimTypePointer(SimTypeBottom(label="SID_AND_ATTRIBUTES"), offset=0)}, name="FWP_TOKEN_INFORMATION", pack=False, align=None), offset=0), "tokenAccessInformation": SimTypePointer(SimStruct({"size": SimTypeInt(signed=False, label="UInt32"), "data": SimTypePointer(SimTypeChar(label="Byte"), offset=0)}, name="FWP_BYTE_BLOB", pack=False, align=None), offset=0), "unicodeString": SimTypePointer(SimTypeChar(label="Char"), offset=0), "byteArray6": SimTypePointer(SimStruct({"byteArray6": SimTypeFixedSizeArray(SimTypeChar(label="Byte"), 6)}, name="FWP_BYTE_ARRAY6", pack=False, align=None), offset=0), "bitmapArray64": SimTypePointer(SimStruct({"bitmapArray64": SimTypeFixedSizeArray(SimTypeChar(label="Byte"), 8)}, name="FWP_BITMAP_ARRAY64_", pack=False, align=None), offset=0), "v4AddrMask": SimTypePointer(SimStruct({"addr": SimTypeInt(signed=False, label="UInt32"), "mask": SimTypeInt(signed=False, label="UInt32")}, name="FWP_V4_ADDR_AND_MASK", pack=False, align=None), offset=0), "v6AddrMask": SimTypePointer(SimStruct({"addr": SimTypeFixedSizeArray(SimTypeChar(label="Byte"), 16), "prefixLength": SimTypeChar(label="Byte")}, name="FWP_V6_ADDR_AND_MASK", pack=False, align=None), offset=0), "rangeValue": SimTypePointer(SimStruct({"valueLow": SimStruct({"type": SimTypeInt(signed=False, label="FWP_DATA_TYPE"), "Anonymous": SimUnion({"uint8": SimTypeChar(label="Byte"), "uint16": SimTypeShort(signed=False, label="UInt16"), "uint32": SimTypeInt(signed=False, label="UInt32"), "uint64": SimTypePointer(SimTypeLongLong(signed=False, label="UInt64"), offset=0), "int8": SimTypeChar(label="SByte"), "int16": SimTypeShort(signed=True, label="Int16"), "int32": SimTypeInt(signed=True, label="Int32"), "int64": SimTypePointer(SimTypeLongLong(signed=True, label="Int64"), offset=0), "float32": SimTypeFloat(size=32), "double64": SimTypePointer(SimTypeFloat(size=64), offset=0), "byteArray16": SimTypePointer(SimStruct({"byteArray16": SimTypeFixedSizeArray(SimTypeChar(label="Byte"), 16)}, name="FWP_BYTE_ARRAY16", pack=False, align=None), offset=0), "byteBlob": SimTypePointer(SimStruct({"size": SimTypeInt(signed=False, label="UInt32"), "data": SimTypePointer(SimTypeChar(label="Byte"), offset=0)}, name="FWP_BYTE_BLOB", pack=False, align=None), offset=0), "sid": SimTypePointer(SimTypeBottom(label="SID"), offset=0), "sd": SimTypePointer(SimStruct({"size": SimTypeInt(signed=False, label="UInt32"), "data": SimTypePointer(SimTypeChar(label="Byte"), offset=0)}, name="FWP_BYTE_BLOB", pack=False, align=None), offset=0), "tokenInformation": SimTypePointer(SimStruct({"sidCount": SimTypeInt(signed=False, label="UInt32"), "sids": SimTypePointer(SimTypeBottom(label="SID_AND_ATTRIBUTES"), offset=0), "restrictedSidCount": SimTypeInt(signed=False, label="UInt32"), "restrictedSids": SimTypePointer(SimTypeBottom(label="SID_AND_ATTRIBUTES"), offset=0)}, name="FWP_TOKEN_INFORMATION", pack=False, align=None), offset=0), "tokenAccessInformation": SimTypePointer(SimStruct({"size": SimTypeInt(signed=False, label="UInt32"), "data": SimTypePointer(SimTypeChar(label="Byte"), offset=0)}, name="FWP_BYTE_BLOB", pack=False, align=None), offset=0), "unicodeString": SimTypePointer(SimTypeChar(label="Char"), offset=0), "byteArray6": SimTypePointer(SimStruct({"byteArray6": SimTypeFixedSizeArray(SimTypeChar(label="Byte"), 6)}, name="FWP_BYTE_ARRAY6", pack=False, align=None), offset=0), "bitmapArray64": SimTypePointer(SimStruct({"bitmapArray64": SimTypeFixedSizeArray(SimTypeChar(label="Byte"), 8)}, name="FWP_BITMAP_ARRAY64_", pack=False, align=None), offset=0)}, name="<anon>", label="None")}, name="FWP_VALUE0", pack=False, align=None), "valueHigh": SimStruct({"type": SimTypeInt(signed=False, label="FWP_DATA_TYPE"), "Anonymous": SimUnion({"uint8": SimTypeChar(label="Byte"), "uint16": SimTypeShort(signed=False, label="UInt16"), "uint32": SimTypeInt(signed=False, label="UInt32"), "uint64": SimTypePointer(SimTypeLongLong(signed=False, label="UInt64"), offset=0), "int8": SimTypeChar(label="SByte"), "int16": SimTypeShort(signed=True, label="Int16"), "int32": SimTypeInt(signed=True, label="Int32"), "int64": SimTypePointer(SimTypeLongLong(signed=True, label="Int64"), offset=0), "float32": SimTypeFloat(size=32), "double64": SimTypePointer(SimTypeFloat(size=64), offset=0), "byteArray16": SimTypePointer(SimStruct({"byteArray16": SimTypeFixedSizeArray(SimTypeChar(label="Byte"), 16)}, name="FWP_BYTE_ARRAY16", pack=False, align=None), offset=0), "byteBlob": SimTypePointer(SimStruct({"size": SimTypeInt(signed=False, label="UInt32"), "data": SimTypePointer(SimTypeChar(label="Byte"), offset=0)}, name="FWP_BYTE_BLOB", pack=False, align=None), offset=0), "sid": SimTypePointer(SimTypeBottom(label="SID"), offset=0), "sd": SimTypePointer(SimStruct({"size": SimTypeInt(signed=False, label="UInt32"), "data": SimTypePointer(SimTypeChar(label="Byte"), offset=0)}, name="FWP_BYTE_BLOB", pack=False, align=None), offset=0), "tokenInformation": SimTypePointer(SimStruct({"sidCount": SimTypeInt(signed=False, label="UInt32"), "sids": SimTypePointer(SimTypeBottom(label="SID_AND_ATTRIBUTES"), offset=0), "restrictedSidCount": SimTypeInt(signed=False, label="UInt32"), "restrictedSids": SimTypePointer(SimTypeBottom(label="SID_AND_ATTRIBUTES"), offset=0)}, name="FWP_TOKEN_INFORMATION", pack=False, align=None), offset=0), "tokenAccessInformation": SimTypePointer(SimStruct({"size": SimTypeInt(signed=False, label="UInt32"), "data": SimTypePointer(SimTypeChar(label="Byte"), offset=0)}, name="FWP_BYTE_BLOB", pack=False, align=None), offset=0), "unicodeString": SimTypePointer(SimTypeChar(label="Char"), offset=0), "byteArray6": SimTypePointer(SimStruct({"byteArray6": SimTypeFixedSizeArray(SimTypeChar(label="Byte"), 6)}, name="FWP_BYTE_ARRAY6", pack=False, align=None), offset=0), "bitmapArray64": SimTypePointer(SimStruct({"bitmapArray64": SimTypeFixedSizeArray(SimTypeChar(label="Byte"), 8)}, name="FWP_BITMAP_ARRAY64_", pack=False, align=None), offset=0)}, name="<anon>", label="None")}, name="FWP_VALUE0", pack=False, align=None)}, name="FWP_RANGE0", pack=False, align=None), offset=0)}, name="<anon>", label="None")}, name="FWP_CONDITION_VALUE0", pack=False, align=None)}, name="<anon>", label="None"), "Anonymous2": SimUnion({"remoteSubNet": SimStruct({"type": SimTypeInt(signed=False, label="FWP_DATA_TYPE"), "Anonymous": SimUnion({"uint8": SimTypeChar(label="Byte"), "uint16": SimTypeShort(signed=False, label="UInt16"), "uint32": SimTypeInt(signed=False, label="UInt32"), "uint64": SimTypePointer(SimTypeLongLong(signed=False, label="UInt64"), offset=0), "int8": SimTypeChar(label="SByte"), "int16": SimTypeShort(signed=True, label="Int16"), "int32": SimTypeInt(signed=True, label="Int32"), "int64": SimTypePointer(SimTypeLongLong(signed=True, label="Int64"), offset=0), "float32": SimTypeFloat(size=32), "double64": SimTypePointer(SimTypeFloat(size=64), offset=0), "byteArray16": SimTypePointer(SimStruct({"byteArray16": SimTypeFixedSizeArray(SimTypeChar(label="Byte"), 16)}, name="FWP_BYTE_ARRAY16", pack=False, align=None), offset=0), "byteBlob": SimTypePointer(SimStruct({"size": SimTypeInt(signed=False, label="UInt32"), "data": SimTypePointer(SimTypeChar(label="Byte"), offset=0)}, name="FWP_BYTE_BLOB", pack=False, align=None), offset=0), "sid": SimTypePointer(SimTypeBottom(label="SID"), offset=0), "sd": SimTypePointer(SimStruct({"size": SimTypeInt(signed=False, label="UInt32"), "data": SimTypePointer(SimTypeChar(label="Byte"), offset=0)}, name="FWP_BYTE_BLOB", pack=False, align=None), offset=0), "tokenInformation": SimTypePointer(SimStruct({"sidCount": SimTypeInt(signed=False, label="UInt32"), "sids": SimTypePointer(SimTypeBottom(label="SID_AND_ATTRIBUTES"), offset=0), "restrictedSidCount": SimTypeInt(signed=False, label="UInt32"), "restrictedSids": SimTypePointer(SimTypeBottom(label="SID_AND_ATTRIBUTES"), offset=0)}, name="FWP_TOKEN_INFORMATION", pack=False, align=None), offset=0), "tokenAccessInformation": SimTypePointer(SimStruct({"size": SimTypeInt(signed=False, label="UInt32"), "data": SimTypePointer(SimTypeChar(label="Byte"), offset=0)}, name="FWP_BYTE_BLOB", pack=False, align=None), offset=0), "unicodeString": SimTypePointer(SimTypeChar(label="Char"), offset=0), "byteArray6": SimTypePointer(SimStruct({"byteArray6": SimTypeFixedSizeArray(SimTypeChar(label="Byte"), 6)}, name="FWP_BYTE_ARRAY6", pack=False, align=None), offset=0), "bitmapArray64": SimTypePointer(SimStruct({"bitmapArray64": SimTypeFixedSizeArray(SimTypeChar(label="Byte"), 8)}, name="FWP_BITMAP_ARRAY64_", pack=False, align=None), offset=0), "v4AddrMask": SimTypePointer(SimStruct({"addr": SimTypeInt(signed=False, label="UInt32"), "mask": SimTypeInt(signed=False, label="UInt32")}, name="FWP_V4_ADDR_AND_MASK", pack=False, align=None), offset=0), "v6AddrMask": SimTypePointer(SimStruct({"addr": SimTypeFixedSizeArray(SimTypeChar(label="Byte"), 16), "prefixLength": SimTypeChar(label="Byte")}, name="FWP_V6_ADDR_AND_MASK", pack=False, align=None), offset=0), "rangeValue": SimTypePointer(SimStruct({"valueLow": SimStruct({"type": SimTypeInt(signed=False, label="FWP_DATA_TYPE"), "Anonymous": SimUnion({"uint8": SimTypeChar(label="Byte"), "uint16": SimTypeShort(signed=False, label="UInt16"), "uint32": SimTypeInt(signed=False, label="UInt32"), "uint64": SimTypePointer(SimTypeLongLong(signed=False, label="UInt64"), offset=0), "int8": SimTypeChar(label="SByte"), "int16": SimTypeShort(signed=True, label="Int16"), "int32": SimTypeInt(signed=True, label="Int32"), "int64": SimTypePointer(SimTypeLongLong(signed=True, label="Int64"), offset=0), "float32": SimTypeFloat(size=32), "double64": SimTypePointer(SimTypeFloat(size=64), offset=0), "byteArray16": SimTypePointer(SimStruct({"byteArray16": SimTypeFixedSizeArray(SimTypeChar(label="Byte"), 16)}, name="FWP_BYTE_ARRAY16", pack=False, align=None), offset=0), "byteBlob": SimTypePointer(SimStruct({"size": SimTypeInt(signed=False, label="UInt32"), "data": SimTypePointer(SimTypeChar(label="Byte"), offset=0)}, name="FWP_BYTE_BLOB", pack=False, align=None), offset=0), "sid": SimTypePointer(SimTypeBottom(label="SID"), offset=0), "sd": SimTypePointer(SimStruct({"size": SimTypeInt(signed=False, label="UInt32"), "data": SimTypePointer(SimTypeChar(label="Byte"), offset=0)}, name="FWP_BYTE_BLOB", pack=False, align=None), offset=0), "tokenInformation": SimTypePointer(SimStruct({"sidCount": SimTypeInt(signed=False, label="UInt32"), "sids": SimTypePointer(SimTypeBottom(label="SID_AND_ATTRIBUTES"), offset=0), "restrictedSidCount": SimTypeInt(signed=False, label="UInt32"), "restrictedSids": SimTypePointer(SimTypeBottom(label="SID_AND_ATTRIBUTES"), offset=0)}, name="FWP_TOKEN_INFORMATION", pack=False, align=None), offset=0), "tokenAccessInformation": SimTypePointer(SimStruct({"size": SimTypeInt(signed=False, label="UInt32"), "data": SimTypePointer(SimTypeChar(label="Byte"), offset=0)}, name="FWP_BYTE_BLOB", pack=False, align=None), offset=0), "unicodeString": SimTypePointer(SimTypeChar(label="Char"), offset=0), "byteArray6": SimTypePointer(SimStruct({"byteArray6": SimTypeFixedSizeArray(SimTypeChar(label="Byte"), 6)}, name="FWP_BYTE_ARRAY6", pack=False, align=None), offset=0), "bitmapArray64": SimTypePointer(SimStruct({"bitmapArray64": SimTypeFixedSizeArray(SimTypeChar(label="Byte"), 8)}, name="FWP_BITMAP_ARRAY64_", pack=False, align=None), offset=0)}, name="<anon>", label="None")}, name="FWP_VALUE0", pack=False, align=None), "valueHigh": SimStruct({"type": SimTypeInt(signed=False, label="FWP_DATA_TYPE"), "Anonymous": SimUnion({"uint8": SimTypeChar(label="Byte"), "uint16": SimTypeShort(signed=False, label="UInt16"), "uint32": SimTypeInt(signed=False, label="UInt32"), "uint64": SimTypePointer(SimTypeLongLong(signed=False, label="UInt64"), offset=0), "int8": SimTypeChar(label="SByte"), "int16": SimTypeShort(signed=True, label="Int16"), "int32": SimTypeInt(signed=True, label="Int32"), "int64": SimTypePointer(SimTypeLongLong(signed=True, label="Int64"), offset=0), "float32": SimTypeFloat(size=32), "double64": SimTypePointer(SimTypeFloat(size=64), offset=0), "byteArray16": SimTypePointer(SimStruct({"byteArray16": SimTypeFixedSizeArray(SimTypeChar(label="Byte"), 16)}, name="FWP_BYTE_ARRAY16", pack=False, align=None), offset=0), "byteBlob": SimTypePointer(SimStruct({"size": SimTypeInt(signed=False, label="UInt32"), "data": SimTypePointer(SimTypeChar(label="Byte"), offset=0)}, name="FWP_BYTE_BLOB", pack=False, align=None), offset=0), "sid": SimTypePointer(SimTypeBottom(label="SID"), offset=0), "sd": SimTypePointer(SimStruct({"size": SimTypeInt(signed=False, label="UInt32"), "data": SimTypePointer(SimTypeChar(label="Byte"), offset=0)}, name="FWP_BYTE_BLOB", pack=False, align=None), offset=0), "tokenInformation": SimTypePointer(SimStruct({"sidCount": SimTypeInt(signed=False, label="UInt32"), "sids": SimTypePointer(SimTypeBottom(label="SID_AND_ATTRIBUTES"), offset=0), "restrictedSidCount": SimTypeInt(signed=False, label="UInt32"), "restrictedSids": SimTypePointer(SimTypeBottom(label="SID_AND_ATTRIBUTES"), offset=0)}, name="FWP_TOKEN_INFORMATION", pack=False, align=None), offset=0), "tokenAccessInformation": SimTypePointer(SimStruct({"size": SimTypeInt(signed=False, label="UInt32"), "data": SimTypePointer(SimTypeChar(label="Byte"), offset=0)}, name="FWP_BYTE_BLOB", pack=False, align=None), offset=0), "unicodeString": SimTypePointer(SimTypeChar(label="Char"), offset=0), "byteArray6": SimTypePointer(SimStruct({"byteArray6": SimTypeFixedSizeArray(SimTypeChar(label="Byte"), 6)}, name="FWP_BYTE_ARRAY6", pack=False, align=None), offset=0), "bitmapArray64": SimTypePointer(SimStruct({"bitmapArray64": SimTypeFixedSizeArray(SimTypeChar(label="Byte"), 8)}, name="FWP_BITMAP_ARRAY64_", pack=False, align=None), offset=0)}, name="<anon>", label="None")}, name="FWP_VALUE0", pack=False, align=None)}, name="FWP_RANGE0", pack=False, align=None), offset=0)}, name="<anon>", label="None")}, name="FWP_CONDITION_VALUE0", pack=False, align=None)}, name="<anon>", label="None"), "qmFilterId": SimTypeLongLong(signed=False, label="UInt64")}, name="FWPM_NET_EVENT_IKEEXT_QM_FAILURE0", pack=False, align=None), offset=0), "ikeEmFailure": SimTypePointer(SimStruct({"failureErrorCode": SimTypeInt(signed=False, label="UInt32"), "failurePoint": SimTypeInt(signed=False, label="IPSEC_FAILURE_POINT"), "flags": SimTypeInt(signed=False, label="UInt32"), "emState": SimTypeInt(signed=False, label="IKEEXT_EM_SA_STATE"), "saRole": SimTypeInt(signed=False, label="IKEEXT_SA_ROLE"), "emAuthMethod": SimTypeInt(signed=False, label="IKEEXT_AUTHENTICATION_METHOD_TYPE"), "endCertHash": SimTypeFixedSizeArray(SimTypeChar(label="Byte"), 20), "mmId": SimTypeLongLong(signed=False, label="UInt64"), "qmFilterId": SimTypeLongLong(signed=False, label="UInt64")}, name="FWPM_NET_EVENT_IKEEXT_EM_FAILURE0", pack=False, align=None), offset=0), "classifyDrop": SimTypePointer(SimStruct({"filterId": SimTypeLongLong(signed=False, label="UInt64"), "layerId": SimTypeShort(signed=False, label="UInt16")}, name="FWPM_NET_EVENT_CLASSIFY_DROP0", pack=False, align=None), offset=0), "ipsecDrop": SimTypePointer(SimStruct({"failureStatus": SimTypeInt(signed=True, label="Int32"), "direction": SimTypeInt(signed=False, label="FWP_DIRECTION"), "spi": SimTypeInt(signed=False, label="UInt32"), "filterId": SimTypeLongLong(signed=False, label="UInt64"), "layerId": SimTypeShort(signed=False, label="UInt16")}, name="FWPM_NET_EVENT_IPSEC_KERNEL_DROP0", pack=False, align=None), offset=0), "idpDrop": SimTypePointer(SimStruct({"ipVersion": SimTypeInt(signed=False, label="FWP_IP_VERSION"), "Anonymous1": SimUnion({"publicHostV4Addr": SimTypeInt(signed=False, label="UInt32"), "publicHostV6Addr": SimTypeFixedSizeArray(SimTypeChar(label="Byte"), 16)}, name="<anon>", label="None"), "Anonymous2": SimUnion({"internalHostV4Addr": SimTypeInt(signed=False, label="UInt32"), "internalHostV6Addr": SimTypeFixedSizeArray(SimTypeChar(label="Byte"), 16)}, name="<anon>", label="None"), "failureStatus": SimTypeInt(signed=True, label="Int32"), "direction": SimTypeInt(signed=False, label="FWP_DIRECTION")}, name="FWPM_NET_EVENT_IPSEC_DOSP_DROP0", pack=False, align=None), offset=0)}, name="<anon>", label="None")}, name="FWPM_NET_EVENT0", pack=False, align=None), offset=0), offset=0), offset=0), SimTypePointer(SimTypeInt(signed=False, label="UInt32"), offset=0)], SimTypeInt(signed=False, label="UInt32"), arg_names=["engineHandle", "enumHandle", "numEntriesRequested", "entries", "numEntriesReturned"]),
        # 
        'FwpmNetEventEnum1': SimTypeFunction([SimTypePointer(SimTypeInt(signed=True, label="Int"), label="IntPtr", offset=0), SimTypePointer(SimTypeInt(signed=True, label="Int"), label="IntPtr", offset=0), SimTypeInt(signed=False, label="UInt32"), SimTypePointer(SimTypePointer(SimTypePointer(SimStruct({"header": SimStruct({"timeStamp": SimStruct({"dwLowDateTime": SimTypeInt(signed=False, label="UInt32"), "dwHighDateTime": SimTypeInt(signed=False, label="UInt32")}, name="FILETIME", pack=False, align=None), "flags": SimTypeInt(signed=False, label="UInt32"), "ipVersion": SimTypeInt(signed=False, label="FWP_IP_VERSION"), "ipProtocol": SimTypeChar(label="Byte"), "Anonymous1": SimUnion({"localAddrV4": SimTypeInt(signed=False, label="UInt32"), "localAddrV6": SimStruct({"byteArray16": SimTypeFixedSizeArray(SimTypeChar(label="Byte"), 16)}, name="FWP_BYTE_ARRAY16", pack=False, align=None)}, name="<anon>", label="None"), "Anonymous2": SimUnion({"remoteAddrV4": SimTypeInt(signed=False, label="UInt32"), "remoteAddrV6": SimStruct({"byteArray16": SimTypeFixedSizeArray(SimTypeChar(label="Byte"), 16)}, name="FWP_BYTE_ARRAY16", pack=False, align=None)}, name="<anon>", label="None"), "localPort": SimTypeShort(signed=False, label="UInt16"), "remotePort": SimTypeShort(signed=False, label="UInt16"), "scopeId": SimTypeInt(signed=False, label="UInt32"), "appId": SimStruct({"size": SimTypeInt(signed=False, label="UInt32"), "data": SimTypePointer(SimTypeChar(label="Byte"), offset=0)}, name="FWP_BYTE_BLOB", pack=False, align=None), "userId": SimTypePointer(SimTypeBottom(label="SID"), offset=0), "Anonymous3": SimUnion({"Anonymous": SimStruct({"reserved1": SimTypeInt(signed=False, label="FWP_AF"), "Anonymous": SimUnion({"Anonymous": SimStruct({"reserved2": SimStruct({"byteArray6": SimTypeFixedSizeArray(SimTypeChar(label="Byte"), 6)}, name="FWP_BYTE_ARRAY6", pack=False, align=None), "reserved3": SimStruct({"byteArray6": SimTypeFixedSizeArray(SimTypeChar(label="Byte"), 6)}, name="FWP_BYTE_ARRAY6", pack=False, align=None), "reserved4": SimTypeInt(signed=False, label="UInt32"), "reserved5": SimTypeInt(signed=False, label="UInt32"), "reserved6": SimTypeShort(signed=False, label="UInt16"), "reserved7": SimTypeInt(signed=False, label="UInt32"), "reserved8": SimTypeInt(signed=False, label="UInt32"), "reserved9": SimTypeShort(signed=False, label="UInt16"), "reserved10": SimTypeLongLong(signed=False, label="UInt64")}, name="_Anonymous_e__Struct", pack=False, align=None)}, name="<anon>", label="None")}, name="_Anonymous_e__Struct", pack=False, align=None)}, name="<anon>", label="None")}, name="FWPM_NET_EVENT_HEADER1", pack=False, align=None), "type": SimTypeInt(signed=False, label="FWPM_NET_EVENT_TYPE"), "Anonymous": SimUnion({"ikeMmFailure": SimTypePointer(SimStruct({"failureErrorCode": SimTypeInt(signed=False, label="UInt32"), "failurePoint": SimTypeInt(signed=False, label="IPSEC_FAILURE_POINT"), "flags": SimTypeInt(signed=False, label="UInt32"), "keyingModuleType": SimTypeInt(signed=False, label="IKEEXT_KEY_MODULE_TYPE"), "mmState": SimTypeInt(signed=False, label="IKEEXT_MM_SA_STATE"), "saRole": SimTypeInt(signed=False, label="IKEEXT_SA_ROLE"), "mmAuthMethod": SimTypeInt(signed=False, label="IKEEXT_AUTHENTICATION_METHOD_TYPE"), "endCertHash": SimTypeFixedSizeArray(SimTypeChar(label="Byte"), 20), "mmId": SimTypeLongLong(signed=False, label="UInt64"), "mmFilterId": SimTypeLongLong(signed=False, label="UInt64"), "localPrincipalNameForAuth": SimTypePointer(SimTypeChar(label="Char"), offset=0), "remotePrincipalNameForAuth": SimTypePointer(SimTypeChar(label="Char"), offset=0), "numLocalPrincipalGroupSids": SimTypeInt(signed=False, label="UInt32"), "localPrincipalGroupSids": SimTypePointer(SimTypePointer(SimTypeChar(label="Char"), offset=0), offset=0), "numRemotePrincipalGroupSids": SimTypeInt(signed=False, label="UInt32"), "remotePrincipalGroupSids": SimTypePointer(SimTypePointer(SimTypeChar(label="Char"), offset=0), offset=0)}, name="FWPM_NET_EVENT_IKEEXT_MM_FAILURE1", pack=False, align=None), offset=0), "ikeQmFailure": SimTypePointer(SimStruct({"failureErrorCode": SimTypeInt(signed=False, label="UInt32"), "failurePoint": SimTypeInt(signed=False, label="IPSEC_FAILURE_POINT"), "keyingModuleType": SimTypeInt(signed=False, label="IKEEXT_KEY_MODULE_TYPE"), "qmState": SimTypeInt(signed=False, label="IKEEXT_QM_SA_STATE"), "saRole": SimTypeInt(signed=False, label="IKEEXT_SA_ROLE"), "saTrafficType": SimTypeInt(signed=False, label="IPSEC_TRAFFIC_TYPE"), "Anonymous1": SimUnion({"localSubNet": SimStruct({"type": SimTypeInt(signed=False, label="FWP_DATA_TYPE"), "Anonymous": SimUnion({"uint8": SimTypeChar(label="Byte"), "uint16": SimTypeShort(signed=False, label="UInt16"), "uint32": SimTypeInt(signed=False, label="UInt32"), "uint64": SimTypePointer(SimTypeLongLong(signed=False, label="UInt64"), offset=0), "int8": SimTypeChar(label="SByte"), "int16": SimTypeShort(signed=True, label="Int16"), "int32": SimTypeInt(signed=True, label="Int32"), "int64": SimTypePointer(SimTypeLongLong(signed=True, label="Int64"), offset=0), "float32": SimTypeFloat(size=32), "double64": SimTypePointer(SimTypeFloat(size=64), offset=0), "byteArray16": SimTypePointer(SimStruct({"byteArray16": SimTypeFixedSizeArray(SimTypeChar(label="Byte"), 16)}, name="FWP_BYTE_ARRAY16", pack=False, align=None), offset=0), "byteBlob": SimTypePointer(SimStruct({"size": SimTypeInt(signed=False, label="UInt32"), "data": SimTypePointer(SimTypeChar(label="Byte"), offset=0)}, name="FWP_BYTE_BLOB", pack=False, align=None), offset=0), "sid": SimTypePointer(SimTypeBottom(label="SID"), offset=0), "sd": SimTypePointer(SimStruct({"size": SimTypeInt(signed=False, label="UInt32"), "data": SimTypePointer(SimTypeChar(label="Byte"), offset=0)}, name="FWP_BYTE_BLOB", pack=False, align=None), offset=0), "tokenInformation": SimTypePointer(SimStruct({"sidCount": SimTypeInt(signed=False, label="UInt32"), "sids": SimTypePointer(SimTypeBottom(label="SID_AND_ATTRIBUTES"), offset=0), "restrictedSidCount": SimTypeInt(signed=False, label="UInt32"), "restrictedSids": SimTypePointer(SimTypeBottom(label="SID_AND_ATTRIBUTES"), offset=0)}, name="FWP_TOKEN_INFORMATION", pack=False, align=None), offset=0), "tokenAccessInformation": SimTypePointer(SimStruct({"size": SimTypeInt(signed=False, label="UInt32"), "data": SimTypePointer(SimTypeChar(label="Byte"), offset=0)}, name="FWP_BYTE_BLOB", pack=False, align=None), offset=0), "unicodeString": SimTypePointer(SimTypeChar(label="Char"), offset=0), "byteArray6": SimTypePointer(SimStruct({"byteArray6": SimTypeFixedSizeArray(SimTypeChar(label="Byte"), 6)}, name="FWP_BYTE_ARRAY6", pack=False, align=None), offset=0), "bitmapArray64": SimTypePointer(SimStruct({"bitmapArray64": SimTypeFixedSizeArray(SimTypeChar(label="Byte"), 8)}, name="FWP_BITMAP_ARRAY64_", pack=False, align=None), offset=0), "v4AddrMask": SimTypePointer(SimStruct({"addr": SimTypeInt(signed=False, label="UInt32"), "mask": SimTypeInt(signed=False, label="UInt32")}, name="FWP_V4_ADDR_AND_MASK", pack=False, align=None), offset=0), "v6AddrMask": SimTypePointer(SimStruct({"addr": SimTypeFixedSizeArray(SimTypeChar(label="Byte"), 16), "prefixLength": SimTypeChar(label="Byte")}, name="FWP_V6_ADDR_AND_MASK", pack=False, align=None), offset=0), "rangeValue": SimTypePointer(SimStruct({"valueLow": SimStruct({"type": SimTypeInt(signed=False, label="FWP_DATA_TYPE"), "Anonymous": SimUnion({"uint8": SimTypeChar(label="Byte"), "uint16": SimTypeShort(signed=False, label="UInt16"), "uint32": SimTypeInt(signed=False, label="UInt32"), "uint64": SimTypePointer(SimTypeLongLong(signed=False, label="UInt64"), offset=0), "int8": SimTypeChar(label="SByte"), "int16": SimTypeShort(signed=True, label="Int16"), "int32": SimTypeInt(signed=True, label="Int32"), "int64": SimTypePointer(SimTypeLongLong(signed=True, label="Int64"), offset=0), "float32": SimTypeFloat(size=32), "double64": SimTypePointer(SimTypeFloat(size=64), offset=0), "byteArray16": SimTypePointer(SimStruct({"byteArray16": SimTypeFixedSizeArray(SimTypeChar(label="Byte"), 16)}, name="FWP_BYTE_ARRAY16", pack=False, align=None), offset=0), "byteBlob": SimTypePointer(SimStruct({"size": SimTypeInt(signed=False, label="UInt32"), "data": SimTypePointer(SimTypeChar(label="Byte"), offset=0)}, name="FWP_BYTE_BLOB", pack=False, align=None), offset=0), "sid": SimTypePointer(SimTypeBottom(label="SID"), offset=0), "sd": SimTypePointer(SimStruct({"size": SimTypeInt(signed=False, label="UInt32"), "data": SimTypePointer(SimTypeChar(label="Byte"), offset=0)}, name="FWP_BYTE_BLOB", pack=False, align=None), offset=0), "tokenInformation": SimTypePointer(SimStruct({"sidCount": SimTypeInt(signed=False, label="UInt32"), "sids": SimTypePointer(SimTypeBottom(label="SID_AND_ATTRIBUTES"), offset=0), "restrictedSidCount": SimTypeInt(signed=False, label="UInt32"), "restrictedSids": SimTypePointer(SimTypeBottom(label="SID_AND_ATTRIBUTES"), offset=0)}, name="FWP_TOKEN_INFORMATION", pack=False, align=None), offset=0), "tokenAccessInformation": SimTypePointer(SimStruct({"size": SimTypeInt(signed=False, label="UInt32"), "data": SimTypePointer(SimTypeChar(label="Byte"), offset=0)}, name="FWP_BYTE_BLOB", pack=False, align=None), offset=0), "unicodeString": SimTypePointer(SimTypeChar(label="Char"), offset=0), "byteArray6": SimTypePointer(SimStruct({"byteArray6": SimTypeFixedSizeArray(SimTypeChar(label="Byte"), 6)}, name="FWP_BYTE_ARRAY6", pack=False, align=None), offset=0), "bitmapArray64": SimTypePointer(SimStruct({"bitmapArray64": SimTypeFixedSizeArray(SimTypeChar(label="Byte"), 8)}, name="FWP_BITMAP_ARRAY64_", pack=False, align=None), offset=0)}, name="<anon>", label="None")}, name="FWP_VALUE0", pack=False, align=None), "valueHigh": SimStruct({"type": SimTypeInt(signed=False, label="FWP_DATA_TYPE"), "Anonymous": SimUnion({"uint8": SimTypeChar(label="Byte"), "uint16": SimTypeShort(signed=False, label="UInt16"), "uint32": SimTypeInt(signed=False, label="UInt32"), "uint64": SimTypePointer(SimTypeLongLong(signed=False, label="UInt64"), offset=0), "int8": SimTypeChar(label="SByte"), "int16": SimTypeShort(signed=True, label="Int16"), "int32": SimTypeInt(signed=True, label="Int32"), "int64": SimTypePointer(SimTypeLongLong(signed=True, label="Int64"), offset=0), "float32": SimTypeFloat(size=32), "double64": SimTypePointer(SimTypeFloat(size=64), offset=0), "byteArray16": SimTypePointer(SimStruct({"byteArray16": SimTypeFixedSizeArray(SimTypeChar(label="Byte"), 16)}, name="FWP_BYTE_ARRAY16", pack=False, align=None), offset=0), "byteBlob": SimTypePointer(SimStruct({"size": SimTypeInt(signed=False, label="UInt32"), "data": SimTypePointer(SimTypeChar(label="Byte"), offset=0)}, name="FWP_BYTE_BLOB", pack=False, align=None), offset=0), "sid": SimTypePointer(SimTypeBottom(label="SID"), offset=0), "sd": SimTypePointer(SimStruct({"size": SimTypeInt(signed=False, label="UInt32"), "data": SimTypePointer(SimTypeChar(label="Byte"), offset=0)}, name="FWP_BYTE_BLOB", pack=False, align=None), offset=0), "tokenInformation": SimTypePointer(SimStruct({"sidCount": SimTypeInt(signed=False, label="UInt32"), "sids": SimTypePointer(SimTypeBottom(label="SID_AND_ATTRIBUTES"), offset=0), "restrictedSidCount": SimTypeInt(signed=False, label="UInt32"), "restrictedSids": SimTypePointer(SimTypeBottom(label="SID_AND_ATTRIBUTES"), offset=0)}, name="FWP_TOKEN_INFORMATION", pack=False, align=None), offset=0), "tokenAccessInformation": SimTypePointer(SimStruct({"size": SimTypeInt(signed=False, label="UInt32"), "data": SimTypePointer(SimTypeChar(label="Byte"), offset=0)}, name="FWP_BYTE_BLOB", pack=False, align=None), offset=0), "unicodeString": SimTypePointer(SimTypeChar(label="Char"), offset=0), "byteArray6": SimTypePointer(SimStruct({"byteArray6": SimTypeFixedSizeArray(SimTypeChar(label="Byte"), 6)}, name="FWP_BYTE_ARRAY6", pack=False, align=None), offset=0), "bitmapArray64": SimTypePointer(SimStruct({"bitmapArray64": SimTypeFixedSizeArray(SimTypeChar(label="Byte"), 8)}, name="FWP_BITMAP_ARRAY64_", pack=False, align=None), offset=0)}, name="<anon>", label="None")}, name="FWP_VALUE0", pack=False, align=None)}, name="FWP_RANGE0", pack=False, align=None), offset=0)}, name="<anon>", label="None")}, name="FWP_CONDITION_VALUE0", pack=False, align=None)}, name="<anon>", label="None"), "Anonymous2": SimUnion({"remoteSubNet": SimStruct({"type": SimTypeInt(signed=False, label="FWP_DATA_TYPE"), "Anonymous": SimUnion({"uint8": SimTypeChar(label="Byte"), "uint16": SimTypeShort(signed=False, label="UInt16"), "uint32": SimTypeInt(signed=False, label="UInt32"), "uint64": SimTypePointer(SimTypeLongLong(signed=False, label="UInt64"), offset=0), "int8": SimTypeChar(label="SByte"), "int16": SimTypeShort(signed=True, label="Int16"), "int32": SimTypeInt(signed=True, label="Int32"), "int64": SimTypePointer(SimTypeLongLong(signed=True, label="Int64"), offset=0), "float32": SimTypeFloat(size=32), "double64": SimTypePointer(SimTypeFloat(size=64), offset=0), "byteArray16": SimTypePointer(SimStruct({"byteArray16": SimTypeFixedSizeArray(SimTypeChar(label="Byte"), 16)}, name="FWP_BYTE_ARRAY16", pack=False, align=None), offset=0), "byteBlob": SimTypePointer(SimStruct({"size": SimTypeInt(signed=False, label="UInt32"), "data": SimTypePointer(SimTypeChar(label="Byte"), offset=0)}, name="FWP_BYTE_BLOB", pack=False, align=None), offset=0), "sid": SimTypePointer(SimTypeBottom(label="SID"), offset=0), "sd": SimTypePointer(SimStruct({"size": SimTypeInt(signed=False, label="UInt32"), "data": SimTypePointer(SimTypeChar(label="Byte"), offset=0)}, name="FWP_BYTE_BLOB", pack=False, align=None), offset=0), "tokenInformation": SimTypePointer(SimStruct({"sidCount": SimTypeInt(signed=False, label="UInt32"), "sids": SimTypePointer(SimTypeBottom(label="SID_AND_ATTRIBUTES"), offset=0), "restrictedSidCount": SimTypeInt(signed=False, label="UInt32"), "restrictedSids": SimTypePointer(SimTypeBottom(label="SID_AND_ATTRIBUTES"), offset=0)}, name="FWP_TOKEN_INFORMATION", pack=False, align=None), offset=0), "tokenAccessInformation": SimTypePointer(SimStruct({"size": SimTypeInt(signed=False, label="UInt32"), "data": SimTypePointer(SimTypeChar(label="Byte"), offset=0)}, name="FWP_BYTE_BLOB", pack=False, align=None), offset=0), "unicodeString": SimTypePointer(SimTypeChar(label="Char"), offset=0), "byteArray6": SimTypePointer(SimStruct({"byteArray6": SimTypeFixedSizeArray(SimTypeChar(label="Byte"), 6)}, name="FWP_BYTE_ARRAY6", pack=False, align=None), offset=0), "bitmapArray64": SimTypePointer(SimStruct({"bitmapArray64": SimTypeFixedSizeArray(SimTypeChar(label="Byte"), 8)}, name="FWP_BITMAP_ARRAY64_", pack=False, align=None), offset=0), "v4AddrMask": SimTypePointer(SimStruct({"addr": SimTypeInt(signed=False, label="UInt32"), "mask": SimTypeInt(signed=False, label="UInt32")}, name="FWP_V4_ADDR_AND_MASK", pack=False, align=None), offset=0), "v6AddrMask": SimTypePointer(SimStruct({"addr": SimTypeFixedSizeArray(SimTypeChar(label="Byte"), 16), "prefixLength": SimTypeChar(label="Byte")}, name="FWP_V6_ADDR_AND_MASK", pack=False, align=None), offset=0), "rangeValue": SimTypePointer(SimStruct({"valueLow": SimStruct({"type": SimTypeInt(signed=False, label="FWP_DATA_TYPE"), "Anonymous": SimUnion({"uint8": SimTypeChar(label="Byte"), "uint16": SimTypeShort(signed=False, label="UInt16"), "uint32": SimTypeInt(signed=False, label="UInt32"), "uint64": SimTypePointer(SimTypeLongLong(signed=False, label="UInt64"), offset=0), "int8": SimTypeChar(label="SByte"), "int16": SimTypeShort(signed=True, label="Int16"), "int32": SimTypeInt(signed=True, label="Int32"), "int64": SimTypePointer(SimTypeLongLong(signed=True, label="Int64"), offset=0), "float32": SimTypeFloat(size=32), "double64": SimTypePointer(SimTypeFloat(size=64), offset=0), "byteArray16": SimTypePointer(SimStruct({"byteArray16": SimTypeFixedSizeArray(SimTypeChar(label="Byte"), 16)}, name="FWP_BYTE_ARRAY16", pack=False, align=None), offset=0), "byteBlob": SimTypePointer(SimStruct({"size": SimTypeInt(signed=False, label="UInt32"), "data": SimTypePointer(SimTypeChar(label="Byte"), offset=0)}, name="FWP_BYTE_BLOB", pack=False, align=None), offset=0), "sid": SimTypePointer(SimTypeBottom(label="SID"), offset=0), "sd": SimTypePointer(SimStruct({"size": SimTypeInt(signed=False, label="UInt32"), "data": SimTypePointer(SimTypeChar(label="Byte"), offset=0)}, name="FWP_BYTE_BLOB", pack=False, align=None), offset=0), "tokenInformation": SimTypePointer(SimStruct({"sidCount": SimTypeInt(signed=False, label="UInt32"), "sids": SimTypePointer(SimTypeBottom(label="SID_AND_ATTRIBUTES"), offset=0), "restrictedSidCount": SimTypeInt(signed=False, label="UInt32"), "restrictedSids": SimTypePointer(SimTypeBottom(label="SID_AND_ATTRIBUTES"), offset=0)}, name="FWP_TOKEN_INFORMATION", pack=False, align=None), offset=0), "tokenAccessInformation": SimTypePointer(SimStruct({"size": SimTypeInt(signed=False, label="UInt32"), "data": SimTypePointer(SimTypeChar(label="Byte"), offset=0)}, name="FWP_BYTE_BLOB", pack=False, align=None), offset=0), "unicodeString": SimTypePointer(SimTypeChar(label="Char"), offset=0), "byteArray6": SimTypePointer(SimStruct({"byteArray6": SimTypeFixedSizeArray(SimTypeChar(label="Byte"), 6)}, name="FWP_BYTE_ARRAY6", pack=False, align=None), offset=0), "bitmapArray64": SimTypePointer(SimStruct({"bitmapArray64": SimTypeFixedSizeArray(SimTypeChar(label="Byte"), 8)}, name="FWP_BITMAP_ARRAY64_", pack=False, align=None), offset=0)}, name="<anon>", label="None")}, name="FWP_VALUE0", pack=False, align=None), "valueHigh": SimStruct({"type": SimTypeInt(signed=False, label="FWP_DATA_TYPE"), "Anonymous": SimUnion({"uint8": SimTypeChar(label="Byte"), "uint16": SimTypeShort(signed=False, label="UInt16"), "uint32": SimTypeInt(signed=False, label="UInt32"), "uint64": SimTypePointer(SimTypeLongLong(signed=False, label="UInt64"), offset=0), "int8": SimTypeChar(label="SByte"), "int16": SimTypeShort(signed=True, label="Int16"), "int32": SimTypeInt(signed=True, label="Int32"), "int64": SimTypePointer(SimTypeLongLong(signed=True, label="Int64"), offset=0), "float32": SimTypeFloat(size=32), "double64": SimTypePointer(SimTypeFloat(size=64), offset=0), "byteArray16": SimTypePointer(SimStruct({"byteArray16": SimTypeFixedSizeArray(SimTypeChar(label="Byte"), 16)}, name="FWP_BYTE_ARRAY16", pack=False, align=None), offset=0), "byteBlob": SimTypePointer(SimStruct({"size": SimTypeInt(signed=False, label="UInt32"), "data": SimTypePointer(SimTypeChar(label="Byte"), offset=0)}, name="FWP_BYTE_BLOB", pack=False, align=None), offset=0), "sid": SimTypePointer(SimTypeBottom(label="SID"), offset=0), "sd": SimTypePointer(SimStruct({"size": SimTypeInt(signed=False, label="UInt32"), "data": SimTypePointer(SimTypeChar(label="Byte"), offset=0)}, name="FWP_BYTE_BLOB", pack=False, align=None), offset=0), "tokenInformation": SimTypePointer(SimStruct({"sidCount": SimTypeInt(signed=False, label="UInt32"), "sids": SimTypePointer(SimTypeBottom(label="SID_AND_ATTRIBUTES"), offset=0), "restrictedSidCount": SimTypeInt(signed=False, label="UInt32"), "restrictedSids": SimTypePointer(SimTypeBottom(label="SID_AND_ATTRIBUTES"), offset=0)}, name="FWP_TOKEN_INFORMATION", pack=False, align=None), offset=0), "tokenAccessInformation": SimTypePointer(SimStruct({"size": SimTypeInt(signed=False, label="UInt32"), "data": SimTypePointer(SimTypeChar(label="Byte"), offset=0)}, name="FWP_BYTE_BLOB", pack=False, align=None), offset=0), "unicodeString": SimTypePointer(SimTypeChar(label="Char"), offset=0), "byteArray6": SimTypePointer(SimStruct({"byteArray6": SimTypeFixedSizeArray(SimTypeChar(label="Byte"), 6)}, name="FWP_BYTE_ARRAY6", pack=False, align=None), offset=0), "bitmapArray64": SimTypePointer(SimStruct({"bitmapArray64": SimTypeFixedSizeArray(SimTypeChar(label="Byte"), 8)}, name="FWP_BITMAP_ARRAY64_", pack=False, align=None), offset=0)}, name="<anon>", label="None")}, name="FWP_VALUE0", pack=False, align=None)}, name="FWP_RANGE0", pack=False, align=None), offset=0)}, name="<anon>", label="None")}, name="FWP_CONDITION_VALUE0", pack=False, align=None)}, name="<anon>", label="None"), "qmFilterId": SimTypeLongLong(signed=False, label="UInt64")}, name="FWPM_NET_EVENT_IKEEXT_QM_FAILURE0", pack=False, align=None), offset=0), "ikeEmFailure": SimTypePointer(SimStruct({"failureErrorCode": SimTypeInt(signed=False, label="UInt32"), "failurePoint": SimTypeInt(signed=False, label="IPSEC_FAILURE_POINT"), "flags": SimTypeInt(signed=False, label="UInt32"), "emState": SimTypeInt(signed=False, label="IKEEXT_EM_SA_STATE"), "saRole": SimTypeInt(signed=False, label="IKEEXT_SA_ROLE"), "emAuthMethod": SimTypeInt(signed=False, label="IKEEXT_AUTHENTICATION_METHOD_TYPE"), "endCertHash": SimTypeFixedSizeArray(SimTypeChar(label="Byte"), 20), "mmId": SimTypeLongLong(signed=False, label="UInt64"), "qmFilterId": SimTypeLongLong(signed=False, label="UInt64"), "localPrincipalNameForAuth": SimTypePointer(SimTypeChar(label="Char"), offset=0), "remotePrincipalNameForAuth": SimTypePointer(SimTypeChar(label="Char"), offset=0), "numLocalPrincipalGroupSids": SimTypeInt(signed=False, label="UInt32"), "localPrincipalGroupSids": SimTypePointer(SimTypePointer(SimTypeChar(label="Char"), offset=0), offset=0), "numRemotePrincipalGroupSids": SimTypeInt(signed=False, label="UInt32"), "remotePrincipalGroupSids": SimTypePointer(SimTypePointer(SimTypeChar(label="Char"), offset=0), offset=0), "saTrafficType": SimTypeInt(signed=False, label="IPSEC_TRAFFIC_TYPE")}, name="FWPM_NET_EVENT_IKEEXT_EM_FAILURE1", pack=False, align=None), offset=0), "classifyDrop": SimTypePointer(SimStruct({"filterId": SimTypeLongLong(signed=False, label="UInt64"), "layerId": SimTypeShort(signed=False, label="UInt16"), "reauthReason": SimTypeInt(signed=False, label="UInt32"), "originalProfile": SimTypeInt(signed=False, label="UInt32"), "currentProfile": SimTypeInt(signed=False, label="UInt32"), "msFwpDirection": SimTypeInt(signed=False, label="UInt32"), "isLoopback": SimTypeInt(signed=True, label="Int32")}, name="FWPM_NET_EVENT_CLASSIFY_DROP1", pack=False, align=None), offset=0), "ipsecDrop": SimTypePointer(SimStruct({"failureStatus": SimTypeInt(signed=True, label="Int32"), "direction": SimTypeInt(signed=False, label="FWP_DIRECTION"), "spi": SimTypeInt(signed=False, label="UInt32"), "filterId": SimTypeLongLong(signed=False, label="UInt64"), "layerId": SimTypeShort(signed=False, label="UInt16")}, name="FWPM_NET_EVENT_IPSEC_KERNEL_DROP0", pack=False, align=None), offset=0), "idpDrop": SimTypePointer(SimStruct({"ipVersion": SimTypeInt(signed=False, label="FWP_IP_VERSION"), "Anonymous1": SimUnion({"publicHostV4Addr": SimTypeInt(signed=False, label="UInt32"), "publicHostV6Addr": SimTypeFixedSizeArray(SimTypeChar(label="Byte"), 16)}, name="<anon>", label="None"), "Anonymous2": SimUnion({"internalHostV4Addr": SimTypeInt(signed=False, label="UInt32"), "internalHostV6Addr": SimTypeFixedSizeArray(SimTypeChar(label="Byte"), 16)}, name="<anon>", label="None"), "failureStatus": SimTypeInt(signed=True, label="Int32"), "direction": SimTypeInt(signed=False, label="FWP_DIRECTION")}, name="FWPM_NET_EVENT_IPSEC_DOSP_DROP0", pack=False, align=None), offset=0)}, name="<anon>", label="None")}, name="FWPM_NET_EVENT1", pack=False, align=None), offset=0), offset=0), offset=0), SimTypePointer(SimTypeInt(signed=False, label="UInt32"), offset=0)], SimTypeInt(signed=False, label="UInt32"), arg_names=["engineHandle", "enumHandle", "numEntriesRequested", "entries", "numEntriesReturned"]),
        # 
        'FwpmNetEventEnum2': SimTypeFunction([SimTypePointer(SimTypeInt(signed=True, label="Int"), label="IntPtr", offset=0), SimTypePointer(SimTypeInt(signed=True, label="Int"), label="IntPtr", offset=0), SimTypeInt(signed=False, label="UInt32"), SimTypePointer(SimTypePointer(SimTypePointer(SimStruct({"header": SimStruct({"timeStamp": SimStruct({"dwLowDateTime": SimTypeInt(signed=False, label="UInt32"), "dwHighDateTime": SimTypeInt(signed=False, label="UInt32")}, name="FILETIME", pack=False, align=None), "flags": SimTypeInt(signed=False, label="UInt32"), "ipVersion": SimTypeInt(signed=False, label="FWP_IP_VERSION"), "ipProtocol": SimTypeChar(label="Byte"), "Anonymous1": SimUnion({"localAddrV4": SimTypeInt(signed=False, label="UInt32"), "localAddrV6": SimStruct({"byteArray16": SimTypeFixedSizeArray(SimTypeChar(label="Byte"), 16)}, name="FWP_BYTE_ARRAY16", pack=False, align=None)}, name="<anon>", label="None"), "Anonymous2": SimUnion({"remoteAddrV4": SimTypeInt(signed=False, label="UInt32"), "remoteAddrV6": SimStruct({"byteArray16": SimTypeFixedSizeArray(SimTypeChar(label="Byte"), 16)}, name="FWP_BYTE_ARRAY16", pack=False, align=None)}, name="<anon>", label="None"), "localPort": SimTypeShort(signed=False, label="UInt16"), "remotePort": SimTypeShort(signed=False, label="UInt16"), "scopeId": SimTypeInt(signed=False, label="UInt32"), "appId": SimStruct({"size": SimTypeInt(signed=False, label="UInt32"), "data": SimTypePointer(SimTypeChar(label="Byte"), offset=0)}, name="FWP_BYTE_BLOB", pack=False, align=None), "userId": SimTypePointer(SimTypeBottom(label="SID"), offset=0), "addressFamily": SimTypeInt(signed=False, label="FWP_AF"), "packageSid": SimTypePointer(SimTypeBottom(label="SID"), offset=0)}, name="FWPM_NET_EVENT_HEADER2", pack=False, align=None), "type": SimTypeInt(signed=False, label="FWPM_NET_EVENT_TYPE"), "Anonymous": SimUnion({"ikeMmFailure": SimTypePointer(SimStruct({"failureErrorCode": SimTypeInt(signed=False, label="UInt32"), "failurePoint": SimTypeInt(signed=False, label="IPSEC_FAILURE_POINT"), "flags": SimTypeInt(signed=False, label="UInt32"), "keyingModuleType": SimTypeInt(signed=False, label="IKEEXT_KEY_MODULE_TYPE"), "mmState": SimTypeInt(signed=False, label="IKEEXT_MM_SA_STATE"), "saRole": SimTypeInt(signed=False, label="IKEEXT_SA_ROLE"), "mmAuthMethod": SimTypeInt(signed=False, label="IKEEXT_AUTHENTICATION_METHOD_TYPE"), "endCertHash": SimTypeFixedSizeArray(SimTypeChar(label="Byte"), 20), "mmId": SimTypeLongLong(signed=False, label="UInt64"), "mmFilterId": SimTypeLongLong(signed=False, label="UInt64"), "localPrincipalNameForAuth": SimTypePointer(SimTypeChar(label="Char"), offset=0), "remotePrincipalNameForAuth": SimTypePointer(SimTypeChar(label="Char"), offset=0), "numLocalPrincipalGroupSids": SimTypeInt(signed=False, label="UInt32"), "localPrincipalGroupSids": SimTypePointer(SimTypePointer(SimTypeChar(label="Char"), offset=0), offset=0), "numRemotePrincipalGroupSids": SimTypeInt(signed=False, label="UInt32"), "remotePrincipalGroupSids": SimTypePointer(SimTypePointer(SimTypeChar(label="Char"), offset=0), offset=0)}, name="FWPM_NET_EVENT_IKEEXT_MM_FAILURE1", pack=False, align=None), offset=0), "ikeQmFailure": SimTypePointer(SimStruct({"failureErrorCode": SimTypeInt(signed=False, label="UInt32"), "failurePoint": SimTypeInt(signed=False, label="IPSEC_FAILURE_POINT"), "keyingModuleType": SimTypeInt(signed=False, label="IKEEXT_KEY_MODULE_TYPE"), "qmState": SimTypeInt(signed=False, label="IKEEXT_QM_SA_STATE"), "saRole": SimTypeInt(signed=False, label="IKEEXT_SA_ROLE"), "saTrafficType": SimTypeInt(signed=False, label="IPSEC_TRAFFIC_TYPE"), "Anonymous1": SimUnion({"localSubNet": SimStruct({"type": SimTypeInt(signed=False, label="FWP_DATA_TYPE"), "Anonymous": SimUnion({"uint8": SimTypeChar(label="Byte"), "uint16": SimTypeShort(signed=False, label="UInt16"), "uint32": SimTypeInt(signed=False, label="UInt32"), "uint64": SimTypePointer(SimTypeLongLong(signed=False, label="UInt64"), offset=0), "int8": SimTypeChar(label="SByte"), "int16": SimTypeShort(signed=True, label="Int16"), "int32": SimTypeInt(signed=True, label="Int32"), "int64": SimTypePointer(SimTypeLongLong(signed=True, label="Int64"), offset=0), "float32": SimTypeFloat(size=32), "double64": SimTypePointer(SimTypeFloat(size=64), offset=0), "byteArray16": SimTypePointer(SimStruct({"byteArray16": SimTypeFixedSizeArray(SimTypeChar(label="Byte"), 16)}, name="FWP_BYTE_ARRAY16", pack=False, align=None), offset=0), "byteBlob": SimTypePointer(SimStruct({"size": SimTypeInt(signed=False, label="UInt32"), "data": SimTypePointer(SimTypeChar(label="Byte"), offset=0)}, name="FWP_BYTE_BLOB", pack=False, align=None), offset=0), "sid": SimTypePointer(SimTypeBottom(label="SID"), offset=0), "sd": SimTypePointer(SimStruct({"size": SimTypeInt(signed=False, label="UInt32"), "data": SimTypePointer(SimTypeChar(label="Byte"), offset=0)}, name="FWP_BYTE_BLOB", pack=False, align=None), offset=0), "tokenInformation": SimTypePointer(SimStruct({"sidCount": SimTypeInt(signed=False, label="UInt32"), "sids": SimTypePointer(SimTypeBottom(label="SID_AND_ATTRIBUTES"), offset=0), "restrictedSidCount": SimTypeInt(signed=False, label="UInt32"), "restrictedSids": SimTypePointer(SimTypeBottom(label="SID_AND_ATTRIBUTES"), offset=0)}, name="FWP_TOKEN_INFORMATION", pack=False, align=None), offset=0), "tokenAccessInformation": SimTypePointer(SimStruct({"size": SimTypeInt(signed=False, label="UInt32"), "data": SimTypePointer(SimTypeChar(label="Byte"), offset=0)}, name="FWP_BYTE_BLOB", pack=False, align=None), offset=0), "unicodeString": SimTypePointer(SimTypeChar(label="Char"), offset=0), "byteArray6": SimTypePointer(SimStruct({"byteArray6": SimTypeFixedSizeArray(SimTypeChar(label="Byte"), 6)}, name="FWP_BYTE_ARRAY6", pack=False, align=None), offset=0), "bitmapArray64": SimTypePointer(SimStruct({"bitmapArray64": SimTypeFixedSizeArray(SimTypeChar(label="Byte"), 8)}, name="FWP_BITMAP_ARRAY64_", pack=False, align=None), offset=0), "v4AddrMask": SimTypePointer(SimStruct({"addr": SimTypeInt(signed=False, label="UInt32"), "mask": SimTypeInt(signed=False, label="UInt32")}, name="FWP_V4_ADDR_AND_MASK", pack=False, align=None), offset=0), "v6AddrMask": SimTypePointer(SimStruct({"addr": SimTypeFixedSizeArray(SimTypeChar(label="Byte"), 16), "prefixLength": SimTypeChar(label="Byte")}, name="FWP_V6_ADDR_AND_MASK", pack=False, align=None), offset=0), "rangeValue": SimTypePointer(SimStruct({"valueLow": SimStruct({"type": SimTypeInt(signed=False, label="FWP_DATA_TYPE"), "Anonymous": SimUnion({"uint8": SimTypeChar(label="Byte"), "uint16": SimTypeShort(signed=False, label="UInt16"), "uint32": SimTypeInt(signed=False, label="UInt32"), "uint64": SimTypePointer(SimTypeLongLong(signed=False, label="UInt64"), offset=0), "int8": SimTypeChar(label="SByte"), "int16": SimTypeShort(signed=True, label="Int16"), "int32": SimTypeInt(signed=True, label="Int32"), "int64": SimTypePointer(SimTypeLongLong(signed=True, label="Int64"), offset=0), "float32": SimTypeFloat(size=32), "double64": SimTypePointer(SimTypeFloat(size=64), offset=0), "byteArray16": SimTypePointer(SimStruct({"byteArray16": SimTypeFixedSizeArray(SimTypeChar(label="Byte"), 16)}, name="FWP_BYTE_ARRAY16", pack=False, align=None), offset=0), "byteBlob": SimTypePointer(SimStruct({"size": SimTypeInt(signed=False, label="UInt32"), "data": SimTypePointer(SimTypeChar(label="Byte"), offset=0)}, name="FWP_BYTE_BLOB", pack=False, align=None), offset=0), "sid": SimTypePointer(SimTypeBottom(label="SID"), offset=0), "sd": SimTypePointer(SimStruct({"size": SimTypeInt(signed=False, label="UInt32"), "data": SimTypePointer(SimTypeChar(label="Byte"), offset=0)}, name="FWP_BYTE_BLOB", pack=False, align=None), offset=0), "tokenInformation": SimTypePointer(SimStruct({"sidCount": SimTypeInt(signed=False, label="UInt32"), "sids": SimTypePointer(SimTypeBottom(label="SID_AND_ATTRIBUTES"), offset=0), "restrictedSidCount": SimTypeInt(signed=False, label="UInt32"), "restrictedSids": SimTypePointer(SimTypeBottom(label="SID_AND_ATTRIBUTES"), offset=0)}, name="FWP_TOKEN_INFORMATION", pack=False, align=None), offset=0), "tokenAccessInformation": SimTypePointer(SimStruct({"size": SimTypeInt(signed=False, label="UInt32"), "data": SimTypePointer(SimTypeChar(label="Byte"), offset=0)}, name="FWP_BYTE_BLOB", pack=False, align=None), offset=0), "unicodeString": SimTypePointer(SimTypeChar(label="Char"), offset=0), "byteArray6": SimTypePointer(SimStruct({"byteArray6": SimTypeFixedSizeArray(SimTypeChar(label="Byte"), 6)}, name="FWP_BYTE_ARRAY6", pack=False, align=None), offset=0), "bitmapArray64": SimTypePointer(SimStruct({"bitmapArray64": SimTypeFixedSizeArray(SimTypeChar(label="Byte"), 8)}, name="FWP_BITMAP_ARRAY64_", pack=False, align=None), offset=0)}, name="<anon>", label="None")}, name="FWP_VALUE0", pack=False, align=None), "valueHigh": SimStruct({"type": SimTypeInt(signed=False, label="FWP_DATA_TYPE"), "Anonymous": SimUnion({"uint8": SimTypeChar(label="Byte"), "uint16": SimTypeShort(signed=False, label="UInt16"), "uint32": SimTypeInt(signed=False, label="UInt32"), "uint64": SimTypePointer(SimTypeLongLong(signed=False, label="UInt64"), offset=0), "int8": SimTypeChar(label="SByte"), "int16": SimTypeShort(signed=True, label="Int16"), "int32": SimTypeInt(signed=True, label="Int32"), "int64": SimTypePointer(SimTypeLongLong(signed=True, label="Int64"), offset=0), "float32": SimTypeFloat(size=32), "double64": SimTypePointer(SimTypeFloat(size=64), offset=0), "byteArray16": SimTypePointer(SimStruct({"byteArray16": SimTypeFixedSizeArray(SimTypeChar(label="Byte"), 16)}, name="FWP_BYTE_ARRAY16", pack=False, align=None), offset=0), "byteBlob": SimTypePointer(SimStruct({"size": SimTypeInt(signed=False, label="UInt32"), "data": SimTypePointer(SimTypeChar(label="Byte"), offset=0)}, name="FWP_BYTE_BLOB", pack=False, align=None), offset=0), "sid": SimTypePointer(SimTypeBottom(label="SID"), offset=0), "sd": SimTypePointer(SimStruct({"size": SimTypeInt(signed=False, label="UInt32"), "data": SimTypePointer(SimTypeChar(label="Byte"), offset=0)}, name="FWP_BYTE_BLOB", pack=False, align=None), offset=0), "tokenInformation": SimTypePointer(SimStruct({"sidCount": SimTypeInt(signed=False, label="UInt32"), "sids": SimTypePointer(SimTypeBottom(label="SID_AND_ATTRIBUTES"), offset=0), "restrictedSidCount": SimTypeInt(signed=False, label="UInt32"), "restrictedSids": SimTypePointer(SimTypeBottom(label="SID_AND_ATTRIBUTES"), offset=0)}, name="FWP_TOKEN_INFORMATION", pack=False, align=None), offset=0), "tokenAccessInformation": SimTypePointer(SimStruct({"size": SimTypeInt(signed=False, label="UInt32"), "data": SimTypePointer(SimTypeChar(label="Byte"), offset=0)}, name="FWP_BYTE_BLOB", pack=False, align=None), offset=0), "unicodeString": SimTypePointer(SimTypeChar(label="Char"), offset=0), "byteArray6": SimTypePointer(SimStruct({"byteArray6": SimTypeFixedSizeArray(SimTypeChar(label="Byte"), 6)}, name="FWP_BYTE_ARRAY6", pack=False, align=None), offset=0), "bitmapArray64": SimTypePointer(SimStruct({"bitmapArray64": SimTypeFixedSizeArray(SimTypeChar(label="Byte"), 8)}, name="FWP_BITMAP_ARRAY64_", pack=False, align=None), offset=0)}, name="<anon>", label="None")}, name="FWP_VALUE0", pack=False, align=None)}, name="FWP_RANGE0", pack=False, align=None), offset=0)}, name="<anon>", label="None")}, name="FWP_CONDITION_VALUE0", pack=False, align=None)}, name="<anon>", label="None"), "Anonymous2": SimUnion({"remoteSubNet": SimStruct({"type": SimTypeInt(signed=False, label="FWP_DATA_TYPE"), "Anonymous": SimUnion({"uint8": SimTypeChar(label="Byte"), "uint16": SimTypeShort(signed=False, label="UInt16"), "uint32": SimTypeInt(signed=False, label="UInt32"), "uint64": SimTypePointer(SimTypeLongLong(signed=False, label="UInt64"), offset=0), "int8": SimTypeChar(label="SByte"), "int16": SimTypeShort(signed=True, label="Int16"), "int32": SimTypeInt(signed=True, label="Int32"), "int64": SimTypePointer(SimTypeLongLong(signed=True, label="Int64"), offset=0), "float32": SimTypeFloat(size=32), "double64": SimTypePointer(SimTypeFloat(size=64), offset=0), "byteArray16": SimTypePointer(SimStruct({"byteArray16": SimTypeFixedSizeArray(SimTypeChar(label="Byte"), 16)}, name="FWP_BYTE_ARRAY16", pack=False, align=None), offset=0), "byteBlob": SimTypePointer(SimStruct({"size": SimTypeInt(signed=False, label="UInt32"), "data": SimTypePointer(SimTypeChar(label="Byte"), offset=0)}, name="FWP_BYTE_BLOB", pack=False, align=None), offset=0), "sid": SimTypePointer(SimTypeBottom(label="SID"), offset=0), "sd": SimTypePointer(SimStruct({"size": SimTypeInt(signed=False, label="UInt32"), "data": SimTypePointer(SimTypeChar(label="Byte"), offset=0)}, name="FWP_BYTE_BLOB", pack=False, align=None), offset=0), "tokenInformation": SimTypePointer(SimStruct({"sidCount": SimTypeInt(signed=False, label="UInt32"), "sids": SimTypePointer(SimTypeBottom(label="SID_AND_ATTRIBUTES"), offset=0), "restrictedSidCount": SimTypeInt(signed=False, label="UInt32"), "restrictedSids": SimTypePointer(SimTypeBottom(label="SID_AND_ATTRIBUTES"), offset=0)}, name="FWP_TOKEN_INFORMATION", pack=False, align=None), offset=0), "tokenAccessInformation": SimTypePointer(SimStruct({"size": SimTypeInt(signed=False, label="UInt32"), "data": SimTypePointer(SimTypeChar(label="Byte"), offset=0)}, name="FWP_BYTE_BLOB", pack=False, align=None), offset=0), "unicodeString": SimTypePointer(SimTypeChar(label="Char"), offset=0), "byteArray6": SimTypePointer(SimStruct({"byteArray6": SimTypeFixedSizeArray(SimTypeChar(label="Byte"), 6)}, name="FWP_BYTE_ARRAY6", pack=False, align=None), offset=0), "bitmapArray64": SimTypePointer(SimStruct({"bitmapArray64": SimTypeFixedSizeArray(SimTypeChar(label="Byte"), 8)}, name="FWP_BITMAP_ARRAY64_", pack=False, align=None), offset=0), "v4AddrMask": SimTypePointer(SimStruct({"addr": SimTypeInt(signed=False, label="UInt32"), "mask": SimTypeInt(signed=False, label="UInt32")}, name="FWP_V4_ADDR_AND_MASK", pack=False, align=None), offset=0), "v6AddrMask": SimTypePointer(SimStruct({"addr": SimTypeFixedSizeArray(SimTypeChar(label="Byte"), 16), "prefixLength": SimTypeChar(label="Byte")}, name="FWP_V6_ADDR_AND_MASK", pack=False, align=None), offset=0), "rangeValue": SimTypePointer(SimStruct({"valueLow": SimStruct({"type": SimTypeInt(signed=False, label="FWP_DATA_TYPE"), "Anonymous": SimUnion({"uint8": SimTypeChar(label="Byte"), "uint16": SimTypeShort(signed=False, label="UInt16"), "uint32": SimTypeInt(signed=False, label="UInt32"), "uint64": SimTypePointer(SimTypeLongLong(signed=False, label="UInt64"), offset=0), "int8": SimTypeChar(label="SByte"), "int16": SimTypeShort(signed=True, label="Int16"), "int32": SimTypeInt(signed=True, label="Int32"), "int64": SimTypePointer(SimTypeLongLong(signed=True, label="Int64"), offset=0), "float32": SimTypeFloat(size=32), "double64": SimTypePointer(SimTypeFloat(size=64), offset=0), "byteArray16": SimTypePointer(SimStruct({"byteArray16": SimTypeFixedSizeArray(SimTypeChar(label="Byte"), 16)}, name="FWP_BYTE_ARRAY16", pack=False, align=None), offset=0), "byteBlob": SimTypePointer(SimStruct({"size": SimTypeInt(signed=False, label="UInt32"), "data": SimTypePointer(SimTypeChar(label="Byte"), offset=0)}, name="FWP_BYTE_BLOB", pack=False, align=None), offset=0), "sid": SimTypePointer(SimTypeBottom(label="SID"), offset=0), "sd": SimTypePointer(SimStruct({"size": SimTypeInt(signed=False, label="UInt32"), "data": SimTypePointer(SimTypeChar(label="Byte"), offset=0)}, name="FWP_BYTE_BLOB", pack=False, align=None), offset=0), "tokenInformation": SimTypePointer(SimStruct({"sidCount": SimTypeInt(signed=False, label="UInt32"), "sids": SimTypePointer(SimTypeBottom(label="SID_AND_ATTRIBUTES"), offset=0), "restrictedSidCount": SimTypeInt(signed=False, label="UInt32"), "restrictedSids": SimTypePointer(SimTypeBottom(label="SID_AND_ATTRIBUTES"), offset=0)}, name="FWP_TOKEN_INFORMATION", pack=False, align=None), offset=0), "tokenAccessInformation": SimTypePointer(SimStruct({"size": SimTypeInt(signed=False, label="UInt32"), "data": SimTypePointer(SimTypeChar(label="Byte"), offset=0)}, name="FWP_BYTE_BLOB", pack=False, align=None), offset=0), "unicodeString": SimTypePointer(SimTypeChar(label="Char"), offset=0), "byteArray6": SimTypePointer(SimStruct({"byteArray6": SimTypeFixedSizeArray(SimTypeChar(label="Byte"), 6)}, name="FWP_BYTE_ARRAY6", pack=False, align=None), offset=0), "bitmapArray64": SimTypePointer(SimStruct({"bitmapArray64": SimTypeFixedSizeArray(SimTypeChar(label="Byte"), 8)}, name="FWP_BITMAP_ARRAY64_", pack=False, align=None), offset=0)}, name="<anon>", label="None")}, name="FWP_VALUE0", pack=False, align=None), "valueHigh": SimStruct({"type": SimTypeInt(signed=False, label="FWP_DATA_TYPE"), "Anonymous": SimUnion({"uint8": SimTypeChar(label="Byte"), "uint16": SimTypeShort(signed=False, label="UInt16"), "uint32": SimTypeInt(signed=False, label="UInt32"), "uint64": SimTypePointer(SimTypeLongLong(signed=False, label="UInt64"), offset=0), "int8": SimTypeChar(label="SByte"), "int16": SimTypeShort(signed=True, label="Int16"), "int32": SimTypeInt(signed=True, label="Int32"), "int64": SimTypePointer(SimTypeLongLong(signed=True, label="Int64"), offset=0), "float32": SimTypeFloat(size=32), "double64": SimTypePointer(SimTypeFloat(size=64), offset=0), "byteArray16": SimTypePointer(SimStruct({"byteArray16": SimTypeFixedSizeArray(SimTypeChar(label="Byte"), 16)}, name="FWP_BYTE_ARRAY16", pack=False, align=None), offset=0), "byteBlob": SimTypePointer(SimStruct({"size": SimTypeInt(signed=False, label="UInt32"), "data": SimTypePointer(SimTypeChar(label="Byte"), offset=0)}, name="FWP_BYTE_BLOB", pack=False, align=None), offset=0), "sid": SimTypePointer(SimTypeBottom(label="SID"), offset=0), "sd": SimTypePointer(SimStruct({"size": SimTypeInt(signed=False, label="UInt32"), "data": SimTypePointer(SimTypeChar(label="Byte"), offset=0)}, name="FWP_BYTE_BLOB", pack=False, align=None), offset=0), "tokenInformation": SimTypePointer(SimStruct({"sidCount": SimTypeInt(signed=False, label="UInt32"), "sids": SimTypePointer(SimTypeBottom(label="SID_AND_ATTRIBUTES"), offset=0), "restrictedSidCount": SimTypeInt(signed=False, label="UInt32"), "restrictedSids": SimTypePointer(SimTypeBottom(label="SID_AND_ATTRIBUTES"), offset=0)}, name="FWP_TOKEN_INFORMATION", pack=False, align=None), offset=0), "tokenAccessInformation": SimTypePointer(SimStruct({"size": SimTypeInt(signed=False, label="UInt32"), "data": SimTypePointer(SimTypeChar(label="Byte"), offset=0)}, name="FWP_BYTE_BLOB", pack=False, align=None), offset=0), "unicodeString": SimTypePointer(SimTypeChar(label="Char"), offset=0), "byteArray6": SimTypePointer(SimStruct({"byteArray6": SimTypeFixedSizeArray(SimTypeChar(label="Byte"), 6)}, name="FWP_BYTE_ARRAY6", pack=False, align=None), offset=0), "bitmapArray64": SimTypePointer(SimStruct({"bitmapArray64": SimTypeFixedSizeArray(SimTypeChar(label="Byte"), 8)}, name="FWP_BITMAP_ARRAY64_", pack=False, align=None), offset=0)}, name="<anon>", label="None")}, name="FWP_VALUE0", pack=False, align=None)}, name="FWP_RANGE0", pack=False, align=None), offset=0)}, name="<anon>", label="None")}, name="FWP_CONDITION_VALUE0", pack=False, align=None)}, name="<anon>", label="None"), "qmFilterId": SimTypeLongLong(signed=False, label="UInt64")}, name="FWPM_NET_EVENT_IKEEXT_QM_FAILURE0", pack=False, align=None), offset=0), "ikeEmFailure": SimTypePointer(SimStruct({"failureErrorCode": SimTypeInt(signed=False, label="UInt32"), "failurePoint": SimTypeInt(signed=False, label="IPSEC_FAILURE_POINT"), "flags": SimTypeInt(signed=False, label="UInt32"), "emState": SimTypeInt(signed=False, label="IKEEXT_EM_SA_STATE"), "saRole": SimTypeInt(signed=False, label="IKEEXT_SA_ROLE"), "emAuthMethod": SimTypeInt(signed=False, label="IKEEXT_AUTHENTICATION_METHOD_TYPE"), "endCertHash": SimTypeFixedSizeArray(SimTypeChar(label="Byte"), 20), "mmId": SimTypeLongLong(signed=False, label="UInt64"), "qmFilterId": SimTypeLongLong(signed=False, label="UInt64"), "localPrincipalNameForAuth": SimTypePointer(SimTypeChar(label="Char"), offset=0), "remotePrincipalNameForAuth": SimTypePointer(SimTypeChar(label="Char"), offset=0), "numLocalPrincipalGroupSids": SimTypeInt(signed=False, label="UInt32"), "localPrincipalGroupSids": SimTypePointer(SimTypePointer(SimTypeChar(label="Char"), offset=0), offset=0), "numRemotePrincipalGroupSids": SimTypeInt(signed=False, label="UInt32"), "remotePrincipalGroupSids": SimTypePointer(SimTypePointer(SimTypeChar(label="Char"), offset=0), offset=0), "saTrafficType": SimTypeInt(signed=False, label="IPSEC_TRAFFIC_TYPE")}, name="FWPM_NET_EVENT_IKEEXT_EM_FAILURE1", pack=False, align=None), offset=0), "classifyDrop": SimTypePointer(SimStruct({"filterId": SimTypeLongLong(signed=False, label="UInt64"), "layerId": SimTypeShort(signed=False, label="UInt16"), "reauthReason": SimTypeInt(signed=False, label="UInt32"), "originalProfile": SimTypeInt(signed=False, label="UInt32"), "currentProfile": SimTypeInt(signed=False, label="UInt32"), "msFwpDirection": SimTypeInt(signed=False, label="UInt32"), "isLoopback": SimTypeInt(signed=True, label="Int32"), "vSwitchId": SimStruct({"size": SimTypeInt(signed=False, label="UInt32"), "data": SimTypePointer(SimTypeChar(label="Byte"), offset=0)}, name="FWP_BYTE_BLOB", pack=False, align=None), "vSwitchSourcePort": SimTypeInt(signed=False, label="UInt32"), "vSwitchDestinationPort": SimTypeInt(signed=False, label="UInt32")}, name="FWPM_NET_EVENT_CLASSIFY_DROP2", pack=False, align=None), offset=0), "ipsecDrop": SimTypePointer(SimStruct({"failureStatus": SimTypeInt(signed=True, label="Int32"), "direction": SimTypeInt(signed=False, label="FWP_DIRECTION"), "spi": SimTypeInt(signed=False, label="UInt32"), "filterId": SimTypeLongLong(signed=False, label="UInt64"), "layerId": SimTypeShort(signed=False, label="UInt16")}, name="FWPM_NET_EVENT_IPSEC_KERNEL_DROP0", pack=False, align=None), offset=0), "idpDrop": SimTypePointer(SimStruct({"ipVersion": SimTypeInt(signed=False, label="FWP_IP_VERSION"), "Anonymous1": SimUnion({"publicHostV4Addr": SimTypeInt(signed=False, label="UInt32"), "publicHostV6Addr": SimTypeFixedSizeArray(SimTypeChar(label="Byte"), 16)}, name="<anon>", label="None"), "Anonymous2": SimUnion({"internalHostV4Addr": SimTypeInt(signed=False, label="UInt32"), "internalHostV6Addr": SimTypeFixedSizeArray(SimTypeChar(label="Byte"), 16)}, name="<anon>", label="None"), "failureStatus": SimTypeInt(signed=True, label="Int32"), "direction": SimTypeInt(signed=False, label="FWP_DIRECTION")}, name="FWPM_NET_EVENT_IPSEC_DOSP_DROP0", pack=False, align=None), offset=0), "classifyAllow": SimTypePointer(SimStruct({"filterId": SimTypeLongLong(signed=False, label="UInt64"), "layerId": SimTypeShort(signed=False, label="UInt16"), "reauthReason": SimTypeInt(signed=False, label="UInt32"), "originalProfile": SimTypeInt(signed=False, label="UInt32"), "currentProfile": SimTypeInt(signed=False, label="UInt32"), "msFwpDirection": SimTypeInt(signed=False, label="UInt32"), "isLoopback": SimTypeInt(signed=True, label="Int32")}, name="FWPM_NET_EVENT_CLASSIFY_ALLOW0", pack=False, align=None), offset=0), "capabilityDrop": SimTypePointer(SimStruct({"networkCapabilityId": SimTypeInt(signed=False, label="FWPM_APPC_NETWORK_CAPABILITY_TYPE"), "filterId": SimTypeLongLong(signed=False, label="UInt64"), "isLoopback": SimTypeInt(signed=True, label="Int32")}, name="FWPM_NET_EVENT_CAPABILITY_DROP0", pack=False, align=None), offset=0), "capabilityAllow": SimTypePointer(SimStruct({"networkCapabilityId": SimTypeInt(signed=False, label="FWPM_APPC_NETWORK_CAPABILITY_TYPE"), "filterId": SimTypeLongLong(signed=False, label="UInt64"), "isLoopback": SimTypeInt(signed=True, label="Int32")}, name="FWPM_NET_EVENT_CAPABILITY_ALLOW0", pack=False, align=None), offset=0), "classifyDropMac": SimTypePointer(SimStruct({"localMacAddr": SimStruct({"byteArray6": SimTypeFixedSizeArray(SimTypeChar(label="Byte"), 6)}, name="FWP_BYTE_ARRAY6", pack=False, align=None), "remoteMacAddr": SimStruct({"byteArray6": SimTypeFixedSizeArray(SimTypeChar(label="Byte"), 6)}, name="FWP_BYTE_ARRAY6", pack=False, align=None), "mediaType": SimTypeInt(signed=False, label="UInt32"), "ifType": SimTypeInt(signed=False, label="UInt32"), "etherType": SimTypeShort(signed=False, label="UInt16"), "ndisPortNumber": SimTypeInt(signed=False, label="UInt32"), "reserved": SimTypeInt(signed=False, label="UInt32"), "vlanTag": SimTypeShort(signed=False, label="UInt16"), "ifLuid": SimTypeLongLong(signed=False, label="UInt64"), "filterId": SimTypeLongLong(signed=False, label="UInt64"), "layerId": SimTypeShort(signed=False, label="UInt16"), "reauthReason": SimTypeInt(signed=False, label="UInt32"), "originalProfile": SimTypeInt(signed=False, label="UInt32"), "currentProfile": SimTypeInt(signed=False, label="UInt32"), "msFwpDirection": SimTypeInt(signed=False, label="UInt32"), "isLoopback": SimTypeInt(signed=True, label="Int32"), "vSwitchId": SimStruct({"size": SimTypeInt(signed=False, label="UInt32"), "data": SimTypePointer(SimTypeChar(label="Byte"), offset=0)}, name="FWP_BYTE_BLOB", pack=False, align=None), "vSwitchSourcePort": SimTypeInt(signed=False, label="UInt32"), "vSwitchDestinationPort": SimTypeInt(signed=False, label="UInt32")}, name="FWPM_NET_EVENT_CLASSIFY_DROP_MAC0", pack=False, align=None), offset=0)}, name="<anon>", label="None")}, name="FWPM_NET_EVENT2", pack=False, align=None), offset=0), offset=0), offset=0), SimTypePointer(SimTypeInt(signed=False, label="UInt32"), offset=0)], SimTypeInt(signed=False, label="UInt32"), arg_names=["engineHandle", "enumHandle", "numEntriesRequested", "entries", "numEntriesReturned"]),
        # 
        'FwpmNetEventEnum3': SimTypeFunction([SimTypePointer(SimTypeInt(signed=True, label="Int"), label="IntPtr", offset=0), SimTypePointer(SimTypeInt(signed=True, label="Int"), label="IntPtr", offset=0), SimTypeInt(signed=False, label="UInt32"), SimTypePointer(SimTypePointer(SimTypePointer(SimStruct({"header": SimStruct({"timeStamp": SimStruct({"dwLowDateTime": SimTypeInt(signed=False, label="UInt32"), "dwHighDateTime": SimTypeInt(signed=False, label="UInt32")}, name="FILETIME", pack=False, align=None), "flags": SimTypeInt(signed=False, label="UInt32"), "ipVersion": SimTypeInt(signed=False, label="FWP_IP_VERSION"), "ipProtocol": SimTypeChar(label="Byte"), "Anonymous1": SimUnion({"localAddrV4": SimTypeInt(signed=False, label="UInt32"), "localAddrV6": SimStruct({"byteArray16": SimTypeFixedSizeArray(SimTypeChar(label="Byte"), 16)}, name="FWP_BYTE_ARRAY16", pack=False, align=None)}, name="<anon>", label="None"), "Anonymous2": SimUnion({"remoteAddrV4": SimTypeInt(signed=False, label="UInt32"), "remoteAddrV6": SimStruct({"byteArray16": SimTypeFixedSizeArray(SimTypeChar(label="Byte"), 16)}, name="FWP_BYTE_ARRAY16", pack=False, align=None)}, name="<anon>", label="None"), "localPort": SimTypeShort(signed=False, label="UInt16"), "remotePort": SimTypeShort(signed=False, label="UInt16"), "scopeId": SimTypeInt(signed=False, label="UInt32"), "appId": SimStruct({"size": SimTypeInt(signed=False, label="UInt32"), "data": SimTypePointer(SimTypeChar(label="Byte"), offset=0)}, name="FWP_BYTE_BLOB", pack=False, align=None), "userId": SimTypePointer(SimTypeBottom(label="SID"), offset=0), "addressFamily": SimTypeInt(signed=False, label="FWP_AF"), "packageSid": SimTypePointer(SimTypeBottom(label="SID"), offset=0), "enterpriseId": SimTypePointer(SimTypeChar(label="Char"), offset=0), "policyFlags": SimTypeLongLong(signed=False, label="UInt64"), "effectiveName": SimStruct({"size": SimTypeInt(signed=False, label="UInt32"), "data": SimTypePointer(SimTypeChar(label="Byte"), offset=0)}, name="FWP_BYTE_BLOB", pack=False, align=None)}, name="FWPM_NET_EVENT_HEADER3", pack=False, align=None), "type": SimTypeInt(signed=False, label="FWPM_NET_EVENT_TYPE"), "Anonymous": SimUnion({"ikeMmFailure": SimTypePointer(SimStruct({"failureErrorCode": SimTypeInt(signed=False, label="UInt32"), "failurePoint": SimTypeInt(signed=False, label="IPSEC_FAILURE_POINT"), "flags": SimTypeInt(signed=False, label="UInt32"), "keyingModuleType": SimTypeInt(signed=False, label="IKEEXT_KEY_MODULE_TYPE"), "mmState": SimTypeInt(signed=False, label="IKEEXT_MM_SA_STATE"), "saRole": SimTypeInt(signed=False, label="IKEEXT_SA_ROLE"), "mmAuthMethod": SimTypeInt(signed=False, label="IKEEXT_AUTHENTICATION_METHOD_TYPE"), "endCertHash": SimTypeFixedSizeArray(SimTypeChar(label="Byte"), 20), "mmId": SimTypeLongLong(signed=False, label="UInt64"), "mmFilterId": SimTypeLongLong(signed=False, label="UInt64"), "localPrincipalNameForAuth": SimTypePointer(SimTypeChar(label="Char"), offset=0), "remotePrincipalNameForAuth": SimTypePointer(SimTypeChar(label="Char"), offset=0), "numLocalPrincipalGroupSids": SimTypeInt(signed=False, label="UInt32"), "localPrincipalGroupSids": SimTypePointer(SimTypePointer(SimTypeChar(label="Char"), offset=0), offset=0), "numRemotePrincipalGroupSids": SimTypeInt(signed=False, label="UInt32"), "remotePrincipalGroupSids": SimTypePointer(SimTypePointer(SimTypeChar(label="Char"), offset=0), offset=0)}, name="FWPM_NET_EVENT_IKEEXT_MM_FAILURE1", pack=False, align=None), offset=0), "ikeQmFailure": SimTypePointer(SimStruct({"failureErrorCode": SimTypeInt(signed=False, label="UInt32"), "failurePoint": SimTypeInt(signed=False, label="IPSEC_FAILURE_POINT"), "keyingModuleType": SimTypeInt(signed=False, label="IKEEXT_KEY_MODULE_TYPE"), "qmState": SimTypeInt(signed=False, label="IKEEXT_QM_SA_STATE"), "saRole": SimTypeInt(signed=False, label="IKEEXT_SA_ROLE"), "saTrafficType": SimTypeInt(signed=False, label="IPSEC_TRAFFIC_TYPE"), "Anonymous1": SimUnion({"localSubNet": SimStruct({"type": SimTypeInt(signed=False, label="FWP_DATA_TYPE"), "Anonymous": SimUnion({"uint8": SimTypeChar(label="Byte"), "uint16": SimTypeShort(signed=False, label="UInt16"), "uint32": SimTypeInt(signed=False, label="UInt32"), "uint64": SimTypePointer(SimTypeLongLong(signed=False, label="UInt64"), offset=0), "int8": SimTypeChar(label="SByte"), "int16": SimTypeShort(signed=True, label="Int16"), "int32": SimTypeInt(signed=True, label="Int32"), "int64": SimTypePointer(SimTypeLongLong(signed=True, label="Int64"), offset=0), "float32": SimTypeFloat(size=32), "double64": SimTypePointer(SimTypeFloat(size=64), offset=0), "byteArray16": SimTypePointer(SimStruct({"byteArray16": SimTypeFixedSizeArray(SimTypeChar(label="Byte"), 16)}, name="FWP_BYTE_ARRAY16", pack=False, align=None), offset=0), "byteBlob": SimTypePointer(SimStruct({"size": SimTypeInt(signed=False, label="UInt32"), "data": SimTypePointer(SimTypeChar(label="Byte"), offset=0)}, name="FWP_BYTE_BLOB", pack=False, align=None), offset=0), "sid": SimTypePointer(SimTypeBottom(label="SID"), offset=0), "sd": SimTypePointer(SimStruct({"size": SimTypeInt(signed=False, label="UInt32"), "data": SimTypePointer(SimTypeChar(label="Byte"), offset=0)}, name="FWP_BYTE_BLOB", pack=False, align=None), offset=0), "tokenInformation": SimTypePointer(SimStruct({"sidCount": SimTypeInt(signed=False, label="UInt32"), "sids": SimTypePointer(SimTypeBottom(label="SID_AND_ATTRIBUTES"), offset=0), "restrictedSidCount": SimTypeInt(signed=False, label="UInt32"), "restrictedSids": SimTypePointer(SimTypeBottom(label="SID_AND_ATTRIBUTES"), offset=0)}, name="FWP_TOKEN_INFORMATION", pack=False, align=None), offset=0), "tokenAccessInformation": SimTypePointer(SimStruct({"size": SimTypeInt(signed=False, label="UInt32"), "data": SimTypePointer(SimTypeChar(label="Byte"), offset=0)}, name="FWP_BYTE_BLOB", pack=False, align=None), offset=0), "unicodeString": SimTypePointer(SimTypeChar(label="Char"), offset=0), "byteArray6": SimTypePointer(SimStruct({"byteArray6": SimTypeFixedSizeArray(SimTypeChar(label="Byte"), 6)}, name="FWP_BYTE_ARRAY6", pack=False, align=None), offset=0), "bitmapArray64": SimTypePointer(SimStruct({"bitmapArray64": SimTypeFixedSizeArray(SimTypeChar(label="Byte"), 8)}, name="FWP_BITMAP_ARRAY64_", pack=False, align=None), offset=0), "v4AddrMask": SimTypePointer(SimStruct({"addr": SimTypeInt(signed=False, label="UInt32"), "mask": SimTypeInt(signed=False, label="UInt32")}, name="FWP_V4_ADDR_AND_MASK", pack=False, align=None), offset=0), "v6AddrMask": SimTypePointer(SimStruct({"addr": SimTypeFixedSizeArray(SimTypeChar(label="Byte"), 16), "prefixLength": SimTypeChar(label="Byte")}, name="FWP_V6_ADDR_AND_MASK", pack=False, align=None), offset=0), "rangeValue": SimTypePointer(SimStruct({"valueLow": SimStruct({"type": SimTypeInt(signed=False, label="FWP_DATA_TYPE"), "Anonymous": SimUnion({"uint8": SimTypeChar(label="Byte"), "uint16": SimTypeShort(signed=False, label="UInt16"), "uint32": SimTypeInt(signed=False, label="UInt32"), "uint64": SimTypePointer(SimTypeLongLong(signed=False, label="UInt64"), offset=0), "int8": SimTypeChar(label="SByte"), "int16": SimTypeShort(signed=True, label="Int16"), "int32": SimTypeInt(signed=True, label="Int32"), "int64": SimTypePointer(SimTypeLongLong(signed=True, label="Int64"), offset=0), "float32": SimTypeFloat(size=32), "double64": SimTypePointer(SimTypeFloat(size=64), offset=0), "byteArray16": SimTypePointer(SimStruct({"byteArray16": SimTypeFixedSizeArray(SimTypeChar(label="Byte"), 16)}, name="FWP_BYTE_ARRAY16", pack=False, align=None), offset=0), "byteBlob": SimTypePointer(SimStruct({"size": SimTypeInt(signed=False, label="UInt32"), "data": SimTypePointer(SimTypeChar(label="Byte"), offset=0)}, name="FWP_BYTE_BLOB", pack=False, align=None), offset=0), "sid": SimTypePointer(SimTypeBottom(label="SID"), offset=0), "sd": SimTypePointer(SimStruct({"size": SimTypeInt(signed=False, label="UInt32"), "data": SimTypePointer(SimTypeChar(label="Byte"), offset=0)}, name="FWP_BYTE_BLOB", pack=False, align=None), offset=0), "tokenInformation": SimTypePointer(SimStruct({"sidCount": SimTypeInt(signed=False, label="UInt32"), "sids": SimTypePointer(SimTypeBottom(label="SID_AND_ATTRIBUTES"), offset=0), "restrictedSidCount": SimTypeInt(signed=False, label="UInt32"), "restrictedSids": SimTypePointer(SimTypeBottom(label="SID_AND_ATTRIBUTES"), offset=0)}, name="FWP_TOKEN_INFORMATION", pack=False, align=None), offset=0), "tokenAccessInformation": SimTypePointer(SimStruct({"size": SimTypeInt(signed=False, label="UInt32"), "data": SimTypePointer(SimTypeChar(label="Byte"), offset=0)}, name="FWP_BYTE_BLOB", pack=False, align=None), offset=0), "unicodeString": SimTypePointer(SimTypeChar(label="Char"), offset=0), "byteArray6": SimTypePointer(SimStruct({"byteArray6": SimTypeFixedSizeArray(SimTypeChar(label="Byte"), 6)}, name="FWP_BYTE_ARRAY6", pack=False, align=None), offset=0), "bitmapArray64": SimTypePointer(SimStruct({"bitmapArray64": SimTypeFixedSizeArray(SimTypeChar(label="Byte"), 8)}, name="FWP_BITMAP_ARRAY64_", pack=False, align=None), offset=0)}, name="<anon>", label="None")}, name="FWP_VALUE0", pack=False, align=None), "valueHigh": SimStruct({"type": SimTypeInt(signed=False, label="FWP_DATA_TYPE"), "Anonymous": SimUnion({"uint8": SimTypeChar(label="Byte"), "uint16": SimTypeShort(signed=False, label="UInt16"), "uint32": SimTypeInt(signed=False, label="UInt32"), "uint64": SimTypePointer(SimTypeLongLong(signed=False, label="UInt64"), offset=0), "int8": SimTypeChar(label="SByte"), "int16": SimTypeShort(signed=True, label="Int16"), "int32": SimTypeInt(signed=True, label="Int32"), "int64": SimTypePointer(SimTypeLongLong(signed=True, label="Int64"), offset=0), "float32": SimTypeFloat(size=32), "double64": SimTypePointer(SimTypeFloat(size=64), offset=0), "byteArray16": SimTypePointer(SimStruct({"byteArray16": SimTypeFixedSizeArray(SimTypeChar(label="Byte"), 16)}, name="FWP_BYTE_ARRAY16", pack=False, align=None), offset=0), "byteBlob": SimTypePointer(SimStruct({"size": SimTypeInt(signed=False, label="UInt32"), "data": SimTypePointer(SimTypeChar(label="Byte"), offset=0)}, name="FWP_BYTE_BLOB", pack=False, align=None), offset=0), "sid": SimTypePointer(SimTypeBottom(label="SID"), offset=0), "sd": SimTypePointer(SimStruct({"size": SimTypeInt(signed=False, label="UInt32"), "data": SimTypePointer(SimTypeChar(label="Byte"), offset=0)}, name="FWP_BYTE_BLOB", pack=False, align=None), offset=0), "tokenInformation": SimTypePointer(SimStruct({"sidCount": SimTypeInt(signed=False, label="UInt32"), "sids": SimTypePointer(SimTypeBottom(label="SID_AND_ATTRIBUTES"), offset=0), "restrictedSidCount": SimTypeInt(signed=False, label="UInt32"), "restrictedSids": SimTypePointer(SimTypeBottom(label="SID_AND_ATTRIBUTES"), offset=0)}, name="FWP_TOKEN_INFORMATION", pack=False, align=None), offset=0), "tokenAccessInformation": SimTypePointer(SimStruct({"size": SimTypeInt(signed=False, label="UInt32"), "data": SimTypePointer(SimTypeChar(label="Byte"), offset=0)}, name="FWP_BYTE_BLOB", pack=False, align=None), offset=0), "unicodeString": SimTypePointer(SimTypeChar(label="Char"), offset=0), "byteArray6": SimTypePointer(SimStruct({"byteArray6": SimTypeFixedSizeArray(SimTypeChar(label="Byte"), 6)}, name="FWP_BYTE_ARRAY6", pack=False, align=None), offset=0), "bitmapArray64": SimTypePointer(SimStruct({"bitmapArray64": SimTypeFixedSizeArray(SimTypeChar(label="Byte"), 8)}, name="FWP_BITMAP_ARRAY64_", pack=False, align=None), offset=0)}, name="<anon>", label="None")}, name="FWP_VALUE0", pack=False, align=None)}, name="FWP_RANGE0", pack=False, align=None), offset=0)}, name="<anon>", label="None")}, name="FWP_CONDITION_VALUE0", pack=False, align=None)}, name="<anon>", label="None"), "Anonymous2": SimUnion({"remoteSubNet": SimStruct({"type": SimTypeInt(signed=False, label="FWP_DATA_TYPE"), "Anonymous": SimUnion({"uint8": SimTypeChar(label="Byte"), "uint16": SimTypeShort(signed=False, label="UInt16"), "uint32": SimTypeInt(signed=False, label="UInt32"), "uint64": SimTypePointer(SimTypeLongLong(signed=False, label="UInt64"), offset=0), "int8": SimTypeChar(label="SByte"), "int16": SimTypeShort(signed=True, label="Int16"), "int32": SimTypeInt(signed=True, label="Int32"), "int64": SimTypePointer(SimTypeLongLong(signed=True, label="Int64"), offset=0), "float32": SimTypeFloat(size=32), "double64": SimTypePointer(SimTypeFloat(size=64), offset=0), "byteArray16": SimTypePointer(SimStruct({"byteArray16": SimTypeFixedSizeArray(SimTypeChar(label="Byte"), 16)}, name="FWP_BYTE_ARRAY16", pack=False, align=None), offset=0), "byteBlob": SimTypePointer(SimStruct({"size": SimTypeInt(signed=False, label="UInt32"), "data": SimTypePointer(SimTypeChar(label="Byte"), offset=0)}, name="FWP_BYTE_BLOB", pack=False, align=None), offset=0), "sid": SimTypePointer(SimTypeBottom(label="SID"), offset=0), "sd": SimTypePointer(SimStruct({"size": SimTypeInt(signed=False, label="UInt32"), "data": SimTypePointer(SimTypeChar(label="Byte"), offset=0)}, name="FWP_BYTE_BLOB", pack=False, align=None), offset=0), "tokenInformation": SimTypePointer(SimStruct({"sidCount": SimTypeInt(signed=False, label="UInt32"), "sids": SimTypePointer(SimTypeBottom(label="SID_AND_ATTRIBUTES"), offset=0), "restrictedSidCount": SimTypeInt(signed=False, label="UInt32"), "restrictedSids": SimTypePointer(SimTypeBottom(label="SID_AND_ATTRIBUTES"), offset=0)}, name="FWP_TOKEN_INFORMATION", pack=False, align=None), offset=0), "tokenAccessInformation": SimTypePointer(SimStruct({"size": SimTypeInt(signed=False, label="UInt32"), "data": SimTypePointer(SimTypeChar(label="Byte"), offset=0)}, name="FWP_BYTE_BLOB", pack=False, align=None), offset=0), "unicodeString": SimTypePointer(SimTypeChar(label="Char"), offset=0), "byteArray6": SimTypePointer(SimStruct({"byteArray6": SimTypeFixedSizeArray(SimTypeChar(label="Byte"), 6)}, name="FWP_BYTE_ARRAY6", pack=False, align=None), offset=0), "bitmapArray64": SimTypePointer(SimStruct({"bitmapArray64": SimTypeFixedSizeArray(SimTypeChar(label="Byte"), 8)}, name="FWP_BITMAP_ARRAY64_", pack=False, align=None), offset=0), "v4AddrMask": SimTypePointer(SimStruct({"addr": SimTypeInt(signed=False, label="UInt32"), "mask": SimTypeInt(signed=False, label="UInt32")}, name="FWP_V4_ADDR_AND_MASK", pack=False, align=None), offset=0), "v6AddrMask": SimTypePointer(SimStruct({"addr": SimTypeFixedSizeArray(SimTypeChar(label="Byte"), 16), "prefixLength": SimTypeChar(label="Byte")}, name="FWP_V6_ADDR_AND_MASK", pack=False, align=None), offset=0), "rangeValue": SimTypePointer(SimStruct({"valueLow": SimStruct({"type": SimTypeInt(signed=False, label="FWP_DATA_TYPE"), "Anonymous": SimUnion({"uint8": SimTypeChar(label="Byte"), "uint16": SimTypeShort(signed=False, label="UInt16"), "uint32": SimTypeInt(signed=False, label="UInt32"), "uint64": SimTypePointer(SimTypeLongLong(signed=False, label="UInt64"), offset=0), "int8": SimTypeChar(label="SByte"), "int16": SimTypeShort(signed=True, label="Int16"), "int32": SimTypeInt(signed=True, label="Int32"), "int64": SimTypePointer(SimTypeLongLong(signed=True, label="Int64"), offset=0), "float32": SimTypeFloat(size=32), "double64": SimTypePointer(SimTypeFloat(size=64), offset=0), "byteArray16": SimTypePointer(SimStruct({"byteArray16": SimTypeFixedSizeArray(SimTypeChar(label="Byte"), 16)}, name="FWP_BYTE_ARRAY16", pack=False, align=None), offset=0), "byteBlob": SimTypePointer(SimStruct({"size": SimTypeInt(signed=False, label="UInt32"), "data": SimTypePointer(SimTypeChar(label="Byte"), offset=0)}, name="FWP_BYTE_BLOB", pack=False, align=None), offset=0), "sid": SimTypePointer(SimTypeBottom(label="SID"), offset=0), "sd": SimTypePointer(SimStruct({"size": SimTypeInt(signed=False, label="UInt32"), "data": SimTypePointer(SimTypeChar(label="Byte"), offset=0)}, name="FWP_BYTE_BLOB", pack=False, align=None), offset=0), "tokenInformation": SimTypePointer(SimStruct({"sidCount": SimTypeInt(signed=False, label="UInt32"), "sids": SimTypePointer(SimTypeBottom(label="SID_AND_ATTRIBUTES"), offset=0), "restrictedSidCount": SimTypeInt(signed=False, label="UInt32"), "restrictedSids": SimTypePointer(SimTypeBottom(label="SID_AND_ATTRIBUTES"), offset=0)}, name="FWP_TOKEN_INFORMATION", pack=False, align=None), offset=0), "tokenAccessInformation": SimTypePointer(SimStruct({"size": SimTypeInt(signed=False, label="UInt32"), "data": SimTypePointer(SimTypeChar(label="Byte"), offset=0)}, name="FWP_BYTE_BLOB", pack=False, align=None), offset=0), "unicodeString": SimTypePointer(SimTypeChar(label="Char"), offset=0), "byteArray6": SimTypePointer(SimStruct({"byteArray6": SimTypeFixedSizeArray(SimTypeChar(label="Byte"), 6)}, name="FWP_BYTE_ARRAY6", pack=False, align=None), offset=0), "bitmapArray64": SimTypePointer(SimStruct({"bitmapArray64": SimTypeFixedSizeArray(SimTypeChar(label="Byte"), 8)}, name="FWP_BITMAP_ARRAY64_", pack=False, align=None), offset=0)}, name="<anon>", label="None")}, name="FWP_VALUE0", pack=False, align=None), "valueHigh": SimStruct({"type": SimTypeInt(signed=False, label="FWP_DATA_TYPE"), "Anonymous": SimUnion({"uint8": SimTypeChar(label="Byte"), "uint16": SimTypeShort(signed=False, label="UInt16"), "uint32": SimTypeInt(signed=False, label="UInt32"), "uint64": SimTypePointer(SimTypeLongLong(signed=False, label="UInt64"), offset=0), "int8": SimTypeChar(label="SByte"), "int16": SimTypeShort(signed=True, label="Int16"), "int32": SimTypeInt(signed=True, label="Int32"), "int64": SimTypePointer(SimTypeLongLong(signed=True, label="Int64"), offset=0), "float32": SimTypeFloat(size=32), "double64": SimTypePointer(SimTypeFloat(size=64), offset=0), "byteArray16": SimTypePointer(SimStruct({"byteArray16": SimTypeFixedSizeArray(SimTypeChar(label="Byte"), 16)}, name="FWP_BYTE_ARRAY16", pack=False, align=None), offset=0), "byteBlob": SimTypePointer(SimStruct({"size": SimTypeInt(signed=False, label="UInt32"), "data": SimTypePointer(SimTypeChar(label="Byte"), offset=0)}, name="FWP_BYTE_BLOB", pack=False, align=None), offset=0), "sid": SimTypePointer(SimTypeBottom(label="SID"), offset=0), "sd": SimTypePointer(SimStruct({"size": SimTypeInt(signed=False, label="UInt32"), "data": SimTypePointer(SimTypeChar(label="Byte"), offset=0)}, name="FWP_BYTE_BLOB", pack=False, align=None), offset=0), "tokenInformation": SimTypePointer(SimStruct({"sidCount": SimTypeInt(signed=False, label="UInt32"), "sids": SimTypePointer(SimTypeBottom(label="SID_AND_ATTRIBUTES"), offset=0), "restrictedSidCount": SimTypeInt(signed=False, label="UInt32"), "restrictedSids": SimTypePointer(SimTypeBottom(label="SID_AND_ATTRIBUTES"), offset=0)}, name="FWP_TOKEN_INFORMATION", pack=False, align=None), offset=0), "tokenAccessInformation": SimTypePointer(SimStruct({"size": SimTypeInt(signed=False, label="UInt32"), "data": SimTypePointer(SimTypeChar(label="Byte"), offset=0)}, name="FWP_BYTE_BLOB", pack=False, align=None), offset=0), "unicodeString": SimTypePointer(SimTypeChar(label="Char"), offset=0), "byteArray6": SimTypePointer(SimStruct({"byteArray6": SimTypeFixedSizeArray(SimTypeChar(label="Byte"), 6)}, name="FWP_BYTE_ARRAY6", pack=False, align=None), offset=0), "bitmapArray64": SimTypePointer(SimStruct({"bitmapArray64": SimTypeFixedSizeArray(SimTypeChar(label="Byte"), 8)}, name="FWP_BITMAP_ARRAY64_", pack=False, align=None), offset=0)}, name="<anon>", label="None")}, name="FWP_VALUE0", pack=False, align=None)}, name="FWP_RANGE0", pack=False, align=None), offset=0)}, name="<anon>", label="None")}, name="FWP_CONDITION_VALUE0", pack=False, align=None)}, name="<anon>", label="None"), "qmFilterId": SimTypeLongLong(signed=False, label="UInt64")}, name="FWPM_NET_EVENT_IKEEXT_QM_FAILURE0", pack=False, align=None), offset=0), "ikeEmFailure": SimTypePointer(SimStruct({"failureErrorCode": SimTypeInt(signed=False, label="UInt32"), "failurePoint": SimTypeInt(signed=False, label="IPSEC_FAILURE_POINT"), "flags": SimTypeInt(signed=False, label="UInt32"), "emState": SimTypeInt(signed=False, label="IKEEXT_EM_SA_STATE"), "saRole": SimTypeInt(signed=False, label="IKEEXT_SA_ROLE"), "emAuthMethod": SimTypeInt(signed=False, label="IKEEXT_AUTHENTICATION_METHOD_TYPE"), "endCertHash": SimTypeFixedSizeArray(SimTypeChar(label="Byte"), 20), "mmId": SimTypeLongLong(signed=False, label="UInt64"), "qmFilterId": SimTypeLongLong(signed=False, label="UInt64"), "localPrincipalNameForAuth": SimTypePointer(SimTypeChar(label="Char"), offset=0), "remotePrincipalNameForAuth": SimTypePointer(SimTypeChar(label="Char"), offset=0), "numLocalPrincipalGroupSids": SimTypeInt(signed=False, label="UInt32"), "localPrincipalGroupSids": SimTypePointer(SimTypePointer(SimTypeChar(label="Char"), offset=0), offset=0), "numRemotePrincipalGroupSids": SimTypeInt(signed=False, label="UInt32"), "remotePrincipalGroupSids": SimTypePointer(SimTypePointer(SimTypeChar(label="Char"), offset=0), offset=0), "saTrafficType": SimTypeInt(signed=False, label="IPSEC_TRAFFIC_TYPE")}, name="FWPM_NET_EVENT_IKEEXT_EM_FAILURE1", pack=False, align=None), offset=0), "classifyDrop": SimTypePointer(SimStruct({"filterId": SimTypeLongLong(signed=False, label="UInt64"), "layerId": SimTypeShort(signed=False, label="UInt16"), "reauthReason": SimTypeInt(signed=False, label="UInt32"), "originalProfile": SimTypeInt(signed=False, label="UInt32"), "currentProfile": SimTypeInt(signed=False, label="UInt32"), "msFwpDirection": SimTypeInt(signed=False, label="UInt32"), "isLoopback": SimTypeInt(signed=True, label="Int32"), "vSwitchId": SimStruct({"size": SimTypeInt(signed=False, label="UInt32"), "data": SimTypePointer(SimTypeChar(label="Byte"), offset=0)}, name="FWP_BYTE_BLOB", pack=False, align=None), "vSwitchSourcePort": SimTypeInt(signed=False, label="UInt32"), "vSwitchDestinationPort": SimTypeInt(signed=False, label="UInt32")}, name="FWPM_NET_EVENT_CLASSIFY_DROP2", pack=False, align=None), offset=0), "ipsecDrop": SimTypePointer(SimStruct({"failureStatus": SimTypeInt(signed=True, label="Int32"), "direction": SimTypeInt(signed=False, label="FWP_DIRECTION"), "spi": SimTypeInt(signed=False, label="UInt32"), "filterId": SimTypeLongLong(signed=False, label="UInt64"), "layerId": SimTypeShort(signed=False, label="UInt16")}, name="FWPM_NET_EVENT_IPSEC_KERNEL_DROP0", pack=False, align=None), offset=0), "idpDrop": SimTypePointer(SimStruct({"ipVersion": SimTypeInt(signed=False, label="FWP_IP_VERSION"), "Anonymous1": SimUnion({"publicHostV4Addr": SimTypeInt(signed=False, label="UInt32"), "publicHostV6Addr": SimTypeFixedSizeArray(SimTypeChar(label="Byte"), 16)}, name="<anon>", label="None"), "Anonymous2": SimUnion({"internalHostV4Addr": SimTypeInt(signed=False, label="UInt32"), "internalHostV6Addr": SimTypeFixedSizeArray(SimTypeChar(label="Byte"), 16)}, name="<anon>", label="None"), "failureStatus": SimTypeInt(signed=True, label="Int32"), "direction": SimTypeInt(signed=False, label="FWP_DIRECTION")}, name="FWPM_NET_EVENT_IPSEC_DOSP_DROP0", pack=False, align=None), offset=0), "classifyAllow": SimTypePointer(SimStruct({"filterId": SimTypeLongLong(signed=False, label="UInt64"), "layerId": SimTypeShort(signed=False, label="UInt16"), "reauthReason": SimTypeInt(signed=False, label="UInt32"), "originalProfile": SimTypeInt(signed=False, label="UInt32"), "currentProfile": SimTypeInt(signed=False, label="UInt32"), "msFwpDirection": SimTypeInt(signed=False, label="UInt32"), "isLoopback": SimTypeInt(signed=True, label="Int32")}, name="FWPM_NET_EVENT_CLASSIFY_ALLOW0", pack=False, align=None), offset=0), "capabilityDrop": SimTypePointer(SimStruct({"networkCapabilityId": SimTypeInt(signed=False, label="FWPM_APPC_NETWORK_CAPABILITY_TYPE"), "filterId": SimTypeLongLong(signed=False, label="UInt64"), "isLoopback": SimTypeInt(signed=True, label="Int32")}, name="FWPM_NET_EVENT_CAPABILITY_DROP0", pack=False, align=None), offset=0), "capabilityAllow": SimTypePointer(SimStruct({"networkCapabilityId": SimTypeInt(signed=False, label="FWPM_APPC_NETWORK_CAPABILITY_TYPE"), "filterId": SimTypeLongLong(signed=False, label="UInt64"), "isLoopback": SimTypeInt(signed=True, label="Int32")}, name="FWPM_NET_EVENT_CAPABILITY_ALLOW0", pack=False, align=None), offset=0), "classifyDropMac": SimTypePointer(SimStruct({"localMacAddr": SimStruct({"byteArray6": SimTypeFixedSizeArray(SimTypeChar(label="Byte"), 6)}, name="FWP_BYTE_ARRAY6", pack=False, align=None), "remoteMacAddr": SimStruct({"byteArray6": SimTypeFixedSizeArray(SimTypeChar(label="Byte"), 6)}, name="FWP_BYTE_ARRAY6", pack=False, align=None), "mediaType": SimTypeInt(signed=False, label="UInt32"), "ifType": SimTypeInt(signed=False, label="UInt32"), "etherType": SimTypeShort(signed=False, label="UInt16"), "ndisPortNumber": SimTypeInt(signed=False, label="UInt32"), "reserved": SimTypeInt(signed=False, label="UInt32"), "vlanTag": SimTypeShort(signed=False, label="UInt16"), "ifLuid": SimTypeLongLong(signed=False, label="UInt64"), "filterId": SimTypeLongLong(signed=False, label="UInt64"), "layerId": SimTypeShort(signed=False, label="UInt16"), "reauthReason": SimTypeInt(signed=False, label="UInt32"), "originalProfile": SimTypeInt(signed=False, label="UInt32"), "currentProfile": SimTypeInt(signed=False, label="UInt32"), "msFwpDirection": SimTypeInt(signed=False, label="UInt32"), "isLoopback": SimTypeInt(signed=True, label="Int32"), "vSwitchId": SimStruct({"size": SimTypeInt(signed=False, label="UInt32"), "data": SimTypePointer(SimTypeChar(label="Byte"), offset=0)}, name="FWP_BYTE_BLOB", pack=False, align=None), "vSwitchSourcePort": SimTypeInt(signed=False, label="UInt32"), "vSwitchDestinationPort": SimTypeInt(signed=False, label="UInt32")}, name="FWPM_NET_EVENT_CLASSIFY_DROP_MAC0", pack=False, align=None), offset=0)}, name="<anon>", label="None")}, name="FWPM_NET_EVENT3", pack=False, align=None), offset=0), offset=0), offset=0), SimTypePointer(SimTypeInt(signed=False, label="UInt32"), offset=0)], SimTypeInt(signed=False, label="UInt32"), arg_names=["engineHandle", "enumHandle", "numEntriesRequested", "entries", "numEntriesReturned"]),
        # 
        'FwpmNetEventEnum4': SimTypeFunction([SimTypePointer(SimTypeInt(signed=True, label="Int"), label="IntPtr", offset=0), SimTypePointer(SimTypeInt(signed=True, label="Int"), label="IntPtr", offset=0), SimTypeInt(signed=False, label="UInt32"), SimTypePointer(SimTypePointer(SimTypePointer(SimStruct({"header": SimStruct({"timeStamp": SimStruct({"dwLowDateTime": SimTypeInt(signed=False, label="UInt32"), "dwHighDateTime": SimTypeInt(signed=False, label="UInt32")}, name="FILETIME", pack=False, align=None), "flags": SimTypeInt(signed=False, label="UInt32"), "ipVersion": SimTypeInt(signed=False, label="FWP_IP_VERSION"), "ipProtocol": SimTypeChar(label="Byte"), "Anonymous1": SimUnion({"localAddrV4": SimTypeInt(signed=False, label="UInt32"), "localAddrV6": SimStruct({"byteArray16": SimTypeFixedSizeArray(SimTypeChar(label="Byte"), 16)}, name="FWP_BYTE_ARRAY16", pack=False, align=None)}, name="<anon>", label="None"), "Anonymous2": SimUnion({"remoteAddrV4": SimTypeInt(signed=False, label="UInt32"), "remoteAddrV6": SimStruct({"byteArray16": SimTypeFixedSizeArray(SimTypeChar(label="Byte"), 16)}, name="FWP_BYTE_ARRAY16", pack=False, align=None)}, name="<anon>", label="None"), "localPort": SimTypeShort(signed=False, label="UInt16"), "remotePort": SimTypeShort(signed=False, label="UInt16"), "scopeId": SimTypeInt(signed=False, label="UInt32"), "appId": SimStruct({"size": SimTypeInt(signed=False, label="UInt32"), "data": SimTypePointer(SimTypeChar(label="Byte"), offset=0)}, name="FWP_BYTE_BLOB", pack=False, align=None), "userId": SimTypePointer(SimTypeBottom(label="SID"), offset=0), "addressFamily": SimTypeInt(signed=False, label="FWP_AF"), "packageSid": SimTypePointer(SimTypeBottom(label="SID"), offset=0), "enterpriseId": SimTypePointer(SimTypeChar(label="Char"), offset=0), "policyFlags": SimTypeLongLong(signed=False, label="UInt64"), "effectiveName": SimStruct({"size": SimTypeInt(signed=False, label="UInt32"), "data": SimTypePointer(SimTypeChar(label="Byte"), offset=0)}, name="FWP_BYTE_BLOB", pack=False, align=None)}, name="FWPM_NET_EVENT_HEADER3", pack=False, align=None), "type": SimTypeInt(signed=False, label="FWPM_NET_EVENT_TYPE"), "Anonymous": SimUnion({"ikeMmFailure": SimTypePointer(SimStruct({"failureErrorCode": SimTypeInt(signed=False, label="UInt32"), "failurePoint": SimTypeInt(signed=False, label="IPSEC_FAILURE_POINT"), "flags": SimTypeInt(signed=False, label="UInt32"), "keyingModuleType": SimTypeInt(signed=False, label="IKEEXT_KEY_MODULE_TYPE"), "mmState": SimTypeInt(signed=False, label="IKEEXT_MM_SA_STATE"), "saRole": SimTypeInt(signed=False, label="IKEEXT_SA_ROLE"), "mmAuthMethod": SimTypeInt(signed=False, label="IKEEXT_AUTHENTICATION_METHOD_TYPE"), "endCertHash": SimTypeFixedSizeArray(SimTypeChar(label="Byte"), 20), "mmId": SimTypeLongLong(signed=False, label="UInt64"), "mmFilterId": SimTypeLongLong(signed=False, label="UInt64"), "localPrincipalNameForAuth": SimTypePointer(SimTypeChar(label="Char"), offset=0), "remotePrincipalNameForAuth": SimTypePointer(SimTypeChar(label="Char"), offset=0), "numLocalPrincipalGroupSids": SimTypeInt(signed=False, label="UInt32"), "localPrincipalGroupSids": SimTypePointer(SimTypePointer(SimTypeChar(label="Char"), offset=0), offset=0), "numRemotePrincipalGroupSids": SimTypeInt(signed=False, label="UInt32"), "remotePrincipalGroupSids": SimTypePointer(SimTypePointer(SimTypeChar(label="Char"), offset=0), offset=0), "providerContextKey": SimTypePointer(SimTypeBottom(label="Guid"), offset=0)}, name="FWPM_NET_EVENT_IKEEXT_MM_FAILURE2_", pack=False, align=None), offset=0), "ikeQmFailure": SimTypePointer(SimStruct({"failureErrorCode": SimTypeInt(signed=False, label="UInt32"), "failurePoint": SimTypeInt(signed=False, label="IPSEC_FAILURE_POINT"), "keyingModuleType": SimTypeInt(signed=False, label="IKEEXT_KEY_MODULE_TYPE"), "qmState": SimTypeInt(signed=False, label="IKEEXT_QM_SA_STATE"), "saRole": SimTypeInt(signed=False, label="IKEEXT_SA_ROLE"), "saTrafficType": SimTypeInt(signed=False, label="IPSEC_TRAFFIC_TYPE"), "Anonymous1": SimUnion({"localSubNet": SimStruct({"type": SimTypeInt(signed=False, label="FWP_DATA_TYPE"), "Anonymous": SimUnion({"uint8": SimTypeChar(label="Byte"), "uint16": SimTypeShort(signed=False, label="UInt16"), "uint32": SimTypeInt(signed=False, label="UInt32"), "uint64": SimTypePointer(SimTypeLongLong(signed=False, label="UInt64"), offset=0), "int8": SimTypeChar(label="SByte"), "int16": SimTypeShort(signed=True, label="Int16"), "int32": SimTypeInt(signed=True, label="Int32"), "int64": SimTypePointer(SimTypeLongLong(signed=True, label="Int64"), offset=0), "float32": SimTypeFloat(size=32), "double64": SimTypePointer(SimTypeFloat(size=64), offset=0), "byteArray16": SimTypePointer(SimStruct({"byteArray16": SimTypeFixedSizeArray(SimTypeChar(label="Byte"), 16)}, name="FWP_BYTE_ARRAY16", pack=False, align=None), offset=0), "byteBlob": SimTypePointer(SimStruct({"size": SimTypeInt(signed=False, label="UInt32"), "data": SimTypePointer(SimTypeChar(label="Byte"), offset=0)}, name="FWP_BYTE_BLOB", pack=False, align=None), offset=0), "sid": SimTypePointer(SimTypeBottom(label="SID"), offset=0), "sd": SimTypePointer(SimStruct({"size": SimTypeInt(signed=False, label="UInt32"), "data": SimTypePointer(SimTypeChar(label="Byte"), offset=0)}, name="FWP_BYTE_BLOB", pack=False, align=None), offset=0), "tokenInformation": SimTypePointer(SimStruct({"sidCount": SimTypeInt(signed=False, label="UInt32"), "sids": SimTypePointer(SimTypeBottom(label="SID_AND_ATTRIBUTES"), offset=0), "restrictedSidCount": SimTypeInt(signed=False, label="UInt32"), "restrictedSids": SimTypePointer(SimTypeBottom(label="SID_AND_ATTRIBUTES"), offset=0)}, name="FWP_TOKEN_INFORMATION", pack=False, align=None), offset=0), "tokenAccessInformation": SimTypePointer(SimStruct({"size": SimTypeInt(signed=False, label="UInt32"), "data": SimTypePointer(SimTypeChar(label="Byte"), offset=0)}, name="FWP_BYTE_BLOB", pack=False, align=None), offset=0), "unicodeString": SimTypePointer(SimTypeChar(label="Char"), offset=0), "byteArray6": SimTypePointer(SimStruct({"byteArray6": SimTypeFixedSizeArray(SimTypeChar(label="Byte"), 6)}, name="FWP_BYTE_ARRAY6", pack=False, align=None), offset=0), "bitmapArray64": SimTypePointer(SimStruct({"bitmapArray64": SimTypeFixedSizeArray(SimTypeChar(label="Byte"), 8)}, name="FWP_BITMAP_ARRAY64_", pack=False, align=None), offset=0), "v4AddrMask": SimTypePointer(SimStruct({"addr": SimTypeInt(signed=False, label="UInt32"), "mask": SimTypeInt(signed=False, label="UInt32")}, name="FWP_V4_ADDR_AND_MASK", pack=False, align=None), offset=0), "v6AddrMask": SimTypePointer(SimStruct({"addr": SimTypeFixedSizeArray(SimTypeChar(label="Byte"), 16), "prefixLength": SimTypeChar(label="Byte")}, name="FWP_V6_ADDR_AND_MASK", pack=False, align=None), offset=0), "rangeValue": SimTypePointer(SimStruct({"valueLow": SimStruct({"type": SimTypeInt(signed=False, label="FWP_DATA_TYPE"), "Anonymous": SimUnion({"uint8": SimTypeChar(label="Byte"), "uint16": SimTypeShort(signed=False, label="UInt16"), "uint32": SimTypeInt(signed=False, label="UInt32"), "uint64": SimTypePointer(SimTypeLongLong(signed=False, label="UInt64"), offset=0), "int8": SimTypeChar(label="SByte"), "int16": SimTypeShort(signed=True, label="Int16"), "int32": SimTypeInt(signed=True, label="Int32"), "int64": SimTypePointer(SimTypeLongLong(signed=True, label="Int64"), offset=0), "float32": SimTypeFloat(size=32), "double64": SimTypePointer(SimTypeFloat(size=64), offset=0), "byteArray16": SimTypePointer(SimStruct({"byteArray16": SimTypeFixedSizeArray(SimTypeChar(label="Byte"), 16)}, name="FWP_BYTE_ARRAY16", pack=False, align=None), offset=0), "byteBlob": SimTypePointer(SimStruct({"size": SimTypeInt(signed=False, label="UInt32"), "data": SimTypePointer(SimTypeChar(label="Byte"), offset=0)}, name="FWP_BYTE_BLOB", pack=False, align=None), offset=0), "sid": SimTypePointer(SimTypeBottom(label="SID"), offset=0), "sd": SimTypePointer(SimStruct({"size": SimTypeInt(signed=False, label="UInt32"), "data": SimTypePointer(SimTypeChar(label="Byte"), offset=0)}, name="FWP_BYTE_BLOB", pack=False, align=None), offset=0), "tokenInformation": SimTypePointer(SimStruct({"sidCount": SimTypeInt(signed=False, label="UInt32"), "sids": SimTypePointer(SimTypeBottom(label="SID_AND_ATTRIBUTES"), offset=0), "restrictedSidCount": SimTypeInt(signed=False, label="UInt32"), "restrictedSids": SimTypePointer(SimTypeBottom(label="SID_AND_ATTRIBUTES"), offset=0)}, name="FWP_TOKEN_INFORMATION", pack=False, align=None), offset=0), "tokenAccessInformation": SimTypePointer(SimStruct({"size": SimTypeInt(signed=False, label="UInt32"), "data": SimTypePointer(SimTypeChar(label="Byte"), offset=0)}, name="FWP_BYTE_BLOB", pack=False, align=None), offset=0), "unicodeString": SimTypePointer(SimTypeChar(label="Char"), offset=0), "byteArray6": SimTypePointer(SimStruct({"byteArray6": SimTypeFixedSizeArray(SimTypeChar(label="Byte"), 6)}, name="FWP_BYTE_ARRAY6", pack=False, align=None), offset=0), "bitmapArray64": SimTypePointer(SimStruct({"bitmapArray64": SimTypeFixedSizeArray(SimTypeChar(label="Byte"), 8)}, name="FWP_BITMAP_ARRAY64_", pack=False, align=None), offset=0)}, name="<anon>", label="None")}, name="FWP_VALUE0", pack=False, align=None), "valueHigh": SimStruct({"type": SimTypeInt(signed=False, label="FWP_DATA_TYPE"), "Anonymous": SimUnion({"uint8": SimTypeChar(label="Byte"), "uint16": SimTypeShort(signed=False, label="UInt16"), "uint32": SimTypeInt(signed=False, label="UInt32"), "uint64": SimTypePointer(SimTypeLongLong(signed=False, label="UInt64"), offset=0), "int8": SimTypeChar(label="SByte"), "int16": SimTypeShort(signed=True, label="Int16"), "int32": SimTypeInt(signed=True, label="Int32"), "int64": SimTypePointer(SimTypeLongLong(signed=True, label="Int64"), offset=0), "float32": SimTypeFloat(size=32), "double64": SimTypePointer(SimTypeFloat(size=64), offset=0), "byteArray16": SimTypePointer(SimStruct({"byteArray16": SimTypeFixedSizeArray(SimTypeChar(label="Byte"), 16)}, name="FWP_BYTE_ARRAY16", pack=False, align=None), offset=0), "byteBlob": SimTypePointer(SimStruct({"size": SimTypeInt(signed=False, label="UInt32"), "data": SimTypePointer(SimTypeChar(label="Byte"), offset=0)}, name="FWP_BYTE_BLOB", pack=False, align=None), offset=0), "sid": SimTypePointer(SimTypeBottom(label="SID"), offset=0), "sd": SimTypePointer(SimStruct({"size": SimTypeInt(signed=False, label="UInt32"), "data": SimTypePointer(SimTypeChar(label="Byte"), offset=0)}, name="FWP_BYTE_BLOB", pack=False, align=None), offset=0), "tokenInformation": SimTypePointer(SimStruct({"sidCount": SimTypeInt(signed=False, label="UInt32"), "sids": SimTypePointer(SimTypeBottom(label="SID_AND_ATTRIBUTES"), offset=0), "restrictedSidCount": SimTypeInt(signed=False, label="UInt32"), "restrictedSids": SimTypePointer(SimTypeBottom(label="SID_AND_ATTRIBUTES"), offset=0)}, name="FWP_TOKEN_INFORMATION", pack=False, align=None), offset=0), "tokenAccessInformation": SimTypePointer(SimStruct({"size": SimTypeInt(signed=False, label="UInt32"), "data": SimTypePointer(SimTypeChar(label="Byte"), offset=0)}, name="FWP_BYTE_BLOB", pack=False, align=None), offset=0), "unicodeString": SimTypePointer(SimTypeChar(label="Char"), offset=0), "byteArray6": SimTypePointer(SimStruct({"byteArray6": SimTypeFixedSizeArray(SimTypeChar(label="Byte"), 6)}, name="FWP_BYTE_ARRAY6", pack=False, align=None), offset=0), "bitmapArray64": SimTypePointer(SimStruct({"bitmapArray64": SimTypeFixedSizeArray(SimTypeChar(label="Byte"), 8)}, name="FWP_BITMAP_ARRAY64_", pack=False, align=None), offset=0)}, name="<anon>", label="None")}, name="FWP_VALUE0", pack=False, align=None)}, name="FWP_RANGE0", pack=False, align=None), offset=0)}, name="<anon>", label="None")}, name="FWP_CONDITION_VALUE0", pack=False, align=None)}, name="<anon>", label="None"), "Anonymous2": SimUnion({"remoteSubNet": SimStruct({"type": SimTypeInt(signed=False, label="FWP_DATA_TYPE"), "Anonymous": SimUnion({"uint8": SimTypeChar(label="Byte"), "uint16": SimTypeShort(signed=False, label="UInt16"), "uint32": SimTypeInt(signed=False, label="UInt32"), "uint64": SimTypePointer(SimTypeLongLong(signed=False, label="UInt64"), offset=0), "int8": SimTypeChar(label="SByte"), "int16": SimTypeShort(signed=True, label="Int16"), "int32": SimTypeInt(signed=True, label="Int32"), "int64": SimTypePointer(SimTypeLongLong(signed=True, label="Int64"), offset=0), "float32": SimTypeFloat(size=32), "double64": SimTypePointer(SimTypeFloat(size=64), offset=0), "byteArray16": SimTypePointer(SimStruct({"byteArray16": SimTypeFixedSizeArray(SimTypeChar(label="Byte"), 16)}, name="FWP_BYTE_ARRAY16", pack=False, align=None), offset=0), "byteBlob": SimTypePointer(SimStruct({"size": SimTypeInt(signed=False, label="UInt32"), "data": SimTypePointer(SimTypeChar(label="Byte"), offset=0)}, name="FWP_BYTE_BLOB", pack=False, align=None), offset=0), "sid": SimTypePointer(SimTypeBottom(label="SID"), offset=0), "sd": SimTypePointer(SimStruct({"size": SimTypeInt(signed=False, label="UInt32"), "data": SimTypePointer(SimTypeChar(label="Byte"), offset=0)}, name="FWP_BYTE_BLOB", pack=False, align=None), offset=0), "tokenInformation": SimTypePointer(SimStruct({"sidCount": SimTypeInt(signed=False, label="UInt32"), "sids": SimTypePointer(SimTypeBottom(label="SID_AND_ATTRIBUTES"), offset=0), "restrictedSidCount": SimTypeInt(signed=False, label="UInt32"), "restrictedSids": SimTypePointer(SimTypeBottom(label="SID_AND_ATTRIBUTES"), offset=0)}, name="FWP_TOKEN_INFORMATION", pack=False, align=None), offset=0), "tokenAccessInformation": SimTypePointer(SimStruct({"size": SimTypeInt(signed=False, label="UInt32"), "data": SimTypePointer(SimTypeChar(label="Byte"), offset=0)}, name="FWP_BYTE_BLOB", pack=False, align=None), offset=0), "unicodeString": SimTypePointer(SimTypeChar(label="Char"), offset=0), "byteArray6": SimTypePointer(SimStruct({"byteArray6": SimTypeFixedSizeArray(SimTypeChar(label="Byte"), 6)}, name="FWP_BYTE_ARRAY6", pack=False, align=None), offset=0), "bitmapArray64": SimTypePointer(SimStruct({"bitmapArray64": SimTypeFixedSizeArray(SimTypeChar(label="Byte"), 8)}, name="FWP_BITMAP_ARRAY64_", pack=False, align=None), offset=0), "v4AddrMask": SimTypePointer(SimStruct({"addr": SimTypeInt(signed=False, label="UInt32"), "mask": SimTypeInt(signed=False, label="UInt32")}, name="FWP_V4_ADDR_AND_MASK", pack=False, align=None), offset=0), "v6AddrMask": SimTypePointer(SimStruct({"addr": SimTypeFixedSizeArray(SimTypeChar(label="Byte"), 16), "prefixLength": SimTypeChar(label="Byte")}, name="FWP_V6_ADDR_AND_MASK", pack=False, align=None), offset=0), "rangeValue": SimTypePointer(SimStruct({"valueLow": SimStruct({"type": SimTypeInt(signed=False, label="FWP_DATA_TYPE"), "Anonymous": SimUnion({"uint8": SimTypeChar(label="Byte"), "uint16": SimTypeShort(signed=False, label="UInt16"), "uint32": SimTypeInt(signed=False, label="UInt32"), "uint64": SimTypePointer(SimTypeLongLong(signed=False, label="UInt64"), offset=0), "int8": SimTypeChar(label="SByte"), "int16": SimTypeShort(signed=True, label="Int16"), "int32": SimTypeInt(signed=True, label="Int32"), "int64": SimTypePointer(SimTypeLongLong(signed=True, label="Int64"), offset=0), "float32": SimTypeFloat(size=32), "double64": SimTypePointer(SimTypeFloat(size=64), offset=0), "byteArray16": SimTypePointer(SimStruct({"byteArray16": SimTypeFixedSizeArray(SimTypeChar(label="Byte"), 16)}, name="FWP_BYTE_ARRAY16", pack=False, align=None), offset=0), "byteBlob": SimTypePointer(SimStruct({"size": SimTypeInt(signed=False, label="UInt32"), "data": SimTypePointer(SimTypeChar(label="Byte"), offset=0)}, name="FWP_BYTE_BLOB", pack=False, align=None), offset=0), "sid": SimTypePointer(SimTypeBottom(label="SID"), offset=0), "sd": SimTypePointer(SimStruct({"size": SimTypeInt(signed=False, label="UInt32"), "data": SimTypePointer(SimTypeChar(label="Byte"), offset=0)}, name="FWP_BYTE_BLOB", pack=False, align=None), offset=0), "tokenInformation": SimTypePointer(SimStruct({"sidCount": SimTypeInt(signed=False, label="UInt32"), "sids": SimTypePointer(SimTypeBottom(label="SID_AND_ATTRIBUTES"), offset=0), "restrictedSidCount": SimTypeInt(signed=False, label="UInt32"), "restrictedSids": SimTypePointer(SimTypeBottom(label="SID_AND_ATTRIBUTES"), offset=0)}, name="FWP_TOKEN_INFORMATION", pack=False, align=None), offset=0), "tokenAccessInformation": SimTypePointer(SimStruct({"size": SimTypeInt(signed=False, label="UInt32"), "data": SimTypePointer(SimTypeChar(label="Byte"), offset=0)}, name="FWP_BYTE_BLOB", pack=False, align=None), offset=0), "unicodeString": SimTypePointer(SimTypeChar(label="Char"), offset=0), "byteArray6": SimTypePointer(SimStruct({"byteArray6": SimTypeFixedSizeArray(SimTypeChar(label="Byte"), 6)}, name="FWP_BYTE_ARRAY6", pack=False, align=None), offset=0), "bitmapArray64": SimTypePointer(SimStruct({"bitmapArray64": SimTypeFixedSizeArray(SimTypeChar(label="Byte"), 8)}, name="FWP_BITMAP_ARRAY64_", pack=False, align=None), offset=0)}, name="<anon>", label="None")}, name="FWP_VALUE0", pack=False, align=None), "valueHigh": SimStruct({"type": SimTypeInt(signed=False, label="FWP_DATA_TYPE"), "Anonymous": SimUnion({"uint8": SimTypeChar(label="Byte"), "uint16": SimTypeShort(signed=False, label="UInt16"), "uint32": SimTypeInt(signed=False, label="UInt32"), "uint64": SimTypePointer(SimTypeLongLong(signed=False, label="UInt64"), offset=0), "int8": SimTypeChar(label="SByte"), "int16": SimTypeShort(signed=True, label="Int16"), "int32": SimTypeInt(signed=True, label="Int32"), "int64": SimTypePointer(SimTypeLongLong(signed=True, label="Int64"), offset=0), "float32": SimTypeFloat(size=32), "double64": SimTypePointer(SimTypeFloat(size=64), offset=0), "byteArray16": SimTypePointer(SimStruct({"byteArray16": SimTypeFixedSizeArray(SimTypeChar(label="Byte"), 16)}, name="FWP_BYTE_ARRAY16", pack=False, align=None), offset=0), "byteBlob": SimTypePointer(SimStruct({"size": SimTypeInt(signed=False, label="UInt32"), "data": SimTypePointer(SimTypeChar(label="Byte"), offset=0)}, name="FWP_BYTE_BLOB", pack=False, align=None), offset=0), "sid": SimTypePointer(SimTypeBottom(label="SID"), offset=0), "sd": SimTypePointer(SimStruct({"size": SimTypeInt(signed=False, label="UInt32"), "data": SimTypePointer(SimTypeChar(label="Byte"), offset=0)}, name="FWP_BYTE_BLOB", pack=False, align=None), offset=0), "tokenInformation": SimTypePointer(SimStruct({"sidCount": SimTypeInt(signed=False, label="UInt32"), "sids": SimTypePointer(SimTypeBottom(label="SID_AND_ATTRIBUTES"), offset=0), "restrictedSidCount": SimTypeInt(signed=False, label="UInt32"), "restrictedSids": SimTypePointer(SimTypeBottom(label="SID_AND_ATTRIBUTES"), offset=0)}, name="FWP_TOKEN_INFORMATION", pack=False, align=None), offset=0), "tokenAccessInformation": SimTypePointer(SimStruct({"size": SimTypeInt(signed=False, label="UInt32"), "data": SimTypePointer(SimTypeChar(label="Byte"), offset=0)}, name="FWP_BYTE_BLOB", pack=False, align=None), offset=0), "unicodeString": SimTypePointer(SimTypeChar(label="Char"), offset=0), "byteArray6": SimTypePointer(SimStruct({"byteArray6": SimTypeFixedSizeArray(SimTypeChar(label="Byte"), 6)}, name="FWP_BYTE_ARRAY6", pack=False, align=None), offset=0), "bitmapArray64": SimTypePointer(SimStruct({"bitmapArray64": SimTypeFixedSizeArray(SimTypeChar(label="Byte"), 8)}, name="FWP_BITMAP_ARRAY64_", pack=False, align=None), offset=0)}, name="<anon>", label="None")}, name="FWP_VALUE0", pack=False, align=None)}, name="FWP_RANGE0", pack=False, align=None), offset=0)}, name="<anon>", label="None")}, name="FWP_CONDITION_VALUE0", pack=False, align=None)}, name="<anon>", label="None"), "qmFilterId": SimTypeLongLong(signed=False, label="UInt64"), "mmSaLuid": SimTypeLongLong(signed=False, label="UInt64"), "mmProviderContextKey": SimTypeBottom(label="Guid")}, name="FWPM_NET_EVENT_IKEEXT_QM_FAILURE1_", pack=False, align=None), offset=0), "ikeEmFailure": SimTypePointer(SimStruct({"failureErrorCode": SimTypeInt(signed=False, label="UInt32"), "failurePoint": SimTypeInt(signed=False, label="IPSEC_FAILURE_POINT"), "flags": SimTypeInt(signed=False, label="UInt32"), "emState": SimTypeInt(signed=False, label="IKEEXT_EM_SA_STATE"), "saRole": SimTypeInt(signed=False, label="IKEEXT_SA_ROLE"), "emAuthMethod": SimTypeInt(signed=False, label="IKEEXT_AUTHENTICATION_METHOD_TYPE"), "endCertHash": SimTypeFixedSizeArray(SimTypeChar(label="Byte"), 20), "mmId": SimTypeLongLong(signed=False, label="UInt64"), "qmFilterId": SimTypeLongLong(signed=False, label="UInt64"), "localPrincipalNameForAuth": SimTypePointer(SimTypeChar(label="Char"), offset=0), "remotePrincipalNameForAuth": SimTypePointer(SimTypeChar(label="Char"), offset=0), "numLocalPrincipalGroupSids": SimTypeInt(signed=False, label="UInt32"), "localPrincipalGroupSids": SimTypePointer(SimTypePointer(SimTypeChar(label="Char"), offset=0), offset=0), "numRemotePrincipalGroupSids": SimTypeInt(signed=False, label="UInt32"), "remotePrincipalGroupSids": SimTypePointer(SimTypePointer(SimTypeChar(label="Char"), offset=0), offset=0), "saTrafficType": SimTypeInt(signed=False, label="IPSEC_TRAFFIC_TYPE")}, name="FWPM_NET_EVENT_IKEEXT_EM_FAILURE1", pack=False, align=None), offset=0), "classifyDrop": SimTypePointer(SimStruct({"filterId": SimTypeLongLong(signed=False, label="UInt64"), "layerId": SimTypeShort(signed=False, label="UInt16"), "reauthReason": SimTypeInt(signed=False, label="UInt32"), "originalProfile": SimTypeInt(signed=False, label="UInt32"), "currentProfile": SimTypeInt(signed=False, label="UInt32"), "msFwpDirection": SimTypeInt(signed=False, label="UInt32"), "isLoopback": SimTypeInt(signed=True, label="Int32"), "vSwitchId": SimStruct({"size": SimTypeInt(signed=False, label="UInt32"), "data": SimTypePointer(SimTypeChar(label="Byte"), offset=0)}, name="FWP_BYTE_BLOB", pack=False, align=None), "vSwitchSourcePort": SimTypeInt(signed=False, label="UInt32"), "vSwitchDestinationPort": SimTypeInt(signed=False, label="UInt32")}, name="FWPM_NET_EVENT_CLASSIFY_DROP2", pack=False, align=None), offset=0), "ipsecDrop": SimTypePointer(SimStruct({"failureStatus": SimTypeInt(signed=True, label="Int32"), "direction": SimTypeInt(signed=False, label="FWP_DIRECTION"), "spi": SimTypeInt(signed=False, label="UInt32"), "filterId": SimTypeLongLong(signed=False, label="UInt64"), "layerId": SimTypeShort(signed=False, label="UInt16")}, name="FWPM_NET_EVENT_IPSEC_KERNEL_DROP0", pack=False, align=None), offset=0), "idpDrop": SimTypePointer(SimStruct({"ipVersion": SimTypeInt(signed=False, label="FWP_IP_VERSION"), "Anonymous1": SimUnion({"publicHostV4Addr": SimTypeInt(signed=False, label="UInt32"), "publicHostV6Addr": SimTypeFixedSizeArray(SimTypeChar(label="Byte"), 16)}, name="<anon>", label="None"), "Anonymous2": SimUnion({"internalHostV4Addr": SimTypeInt(signed=False, label="UInt32"), "internalHostV6Addr": SimTypeFixedSizeArray(SimTypeChar(label="Byte"), 16)}, name="<anon>", label="None"), "failureStatus": SimTypeInt(signed=True, label="Int32"), "direction": SimTypeInt(signed=False, label="FWP_DIRECTION")}, name="FWPM_NET_EVENT_IPSEC_DOSP_DROP0", pack=False, align=None), offset=0), "classifyAllow": SimTypePointer(SimStruct({"filterId": SimTypeLongLong(signed=False, label="UInt64"), "layerId": SimTypeShort(signed=False, label="UInt16"), "reauthReason": SimTypeInt(signed=False, label="UInt32"), "originalProfile": SimTypeInt(signed=False, label="UInt32"), "currentProfile": SimTypeInt(signed=False, label="UInt32"), "msFwpDirection": SimTypeInt(signed=False, label="UInt32"), "isLoopback": SimTypeInt(signed=True, label="Int32")}, name="FWPM_NET_EVENT_CLASSIFY_ALLOW0", pack=False, align=None), offset=0), "capabilityDrop": SimTypePointer(SimStruct({"networkCapabilityId": SimTypeInt(signed=False, label="FWPM_APPC_NETWORK_CAPABILITY_TYPE"), "filterId": SimTypeLongLong(signed=False, label="UInt64"), "isLoopback": SimTypeInt(signed=True, label="Int32")}, name="FWPM_NET_EVENT_CAPABILITY_DROP0", pack=False, align=None), offset=0), "capabilityAllow": SimTypePointer(SimStruct({"networkCapabilityId": SimTypeInt(signed=False, label="FWPM_APPC_NETWORK_CAPABILITY_TYPE"), "filterId": SimTypeLongLong(signed=False, label="UInt64"), "isLoopback": SimTypeInt(signed=True, label="Int32")}, name="FWPM_NET_EVENT_CAPABILITY_ALLOW0", pack=False, align=None), offset=0), "classifyDropMac": SimTypePointer(SimStruct({"localMacAddr": SimStruct({"byteArray6": SimTypeFixedSizeArray(SimTypeChar(label="Byte"), 6)}, name="FWP_BYTE_ARRAY6", pack=False, align=None), "remoteMacAddr": SimStruct({"byteArray6": SimTypeFixedSizeArray(SimTypeChar(label="Byte"), 6)}, name="FWP_BYTE_ARRAY6", pack=False, align=None), "mediaType": SimTypeInt(signed=False, label="UInt32"), "ifType": SimTypeInt(signed=False, label="UInt32"), "etherType": SimTypeShort(signed=False, label="UInt16"), "ndisPortNumber": SimTypeInt(signed=False, label="UInt32"), "reserved": SimTypeInt(signed=False, label="UInt32"), "vlanTag": SimTypeShort(signed=False, label="UInt16"), "ifLuid": SimTypeLongLong(signed=False, label="UInt64"), "filterId": SimTypeLongLong(signed=False, label="UInt64"), "layerId": SimTypeShort(signed=False, label="UInt16"), "reauthReason": SimTypeInt(signed=False, label="UInt32"), "originalProfile": SimTypeInt(signed=False, label="UInt32"), "currentProfile": SimTypeInt(signed=False, label="UInt32"), "msFwpDirection": SimTypeInt(signed=False, label="UInt32"), "isLoopback": SimTypeInt(signed=True, label="Int32"), "vSwitchId": SimStruct({"size": SimTypeInt(signed=False, label="UInt32"), "data": SimTypePointer(SimTypeChar(label="Byte"), offset=0)}, name="FWP_BYTE_BLOB", pack=False, align=None), "vSwitchSourcePort": SimTypeInt(signed=False, label="UInt32"), "vSwitchDestinationPort": SimTypeInt(signed=False, label="UInt32")}, name="FWPM_NET_EVENT_CLASSIFY_DROP_MAC0", pack=False, align=None), offset=0)}, name="<anon>", label="None")}, name="FWPM_NET_EVENT4_", pack=False, align=None), offset=0), offset=0), offset=0), SimTypePointer(SimTypeInt(signed=False, label="UInt32"), offset=0)], SimTypeInt(signed=False, label="UInt32"), arg_names=["engineHandle", "enumHandle", "numEntriesRequested", "entries", "numEntriesReturned"]),
        # 
        'FwpmNetEventEnum5': SimTypeFunction([SimTypePointer(SimTypeInt(signed=True, label="Int"), label="IntPtr", offset=0), SimTypePointer(SimTypeInt(signed=True, label="Int"), label="IntPtr", offset=0), SimTypeInt(signed=False, label="UInt32"), SimTypePointer(SimTypePointer(SimTypePointer(SimStruct({"header": SimStruct({"timeStamp": SimStruct({"dwLowDateTime": SimTypeInt(signed=False, label="UInt32"), "dwHighDateTime": SimTypeInt(signed=False, label="UInt32")}, name="FILETIME", pack=False, align=None), "flags": SimTypeInt(signed=False, label="UInt32"), "ipVersion": SimTypeInt(signed=False, label="FWP_IP_VERSION"), "ipProtocol": SimTypeChar(label="Byte"), "Anonymous1": SimUnion({"localAddrV4": SimTypeInt(signed=False, label="UInt32"), "localAddrV6": SimStruct({"byteArray16": SimTypeFixedSizeArray(SimTypeChar(label="Byte"), 16)}, name="FWP_BYTE_ARRAY16", pack=False, align=None)}, name="<anon>", label="None"), "Anonymous2": SimUnion({"remoteAddrV4": SimTypeInt(signed=False, label="UInt32"), "remoteAddrV6": SimStruct({"byteArray16": SimTypeFixedSizeArray(SimTypeChar(label="Byte"), 16)}, name="FWP_BYTE_ARRAY16", pack=False, align=None)}, name="<anon>", label="None"), "localPort": SimTypeShort(signed=False, label="UInt16"), "remotePort": SimTypeShort(signed=False, label="UInt16"), "scopeId": SimTypeInt(signed=False, label="UInt32"), "appId": SimStruct({"size": SimTypeInt(signed=False, label="UInt32"), "data": SimTypePointer(SimTypeChar(label="Byte"), offset=0)}, name="FWP_BYTE_BLOB", pack=False, align=None), "userId": SimTypePointer(SimTypeBottom(label="SID"), offset=0), "addressFamily": SimTypeInt(signed=False, label="FWP_AF"), "packageSid": SimTypePointer(SimTypeBottom(label="SID"), offset=0), "enterpriseId": SimTypePointer(SimTypeChar(label="Char"), offset=0), "policyFlags": SimTypeLongLong(signed=False, label="UInt64"), "effectiveName": SimStruct({"size": SimTypeInt(signed=False, label="UInt32"), "data": SimTypePointer(SimTypeChar(label="Byte"), offset=0)}, name="FWP_BYTE_BLOB", pack=False, align=None)}, name="FWPM_NET_EVENT_HEADER3", pack=False, align=None), "type": SimTypeInt(signed=False, label="FWPM_NET_EVENT_TYPE"), "Anonymous": SimUnion({"ikeMmFailure": SimTypePointer(SimStruct({"failureErrorCode": SimTypeInt(signed=False, label="UInt32"), "failurePoint": SimTypeInt(signed=False, label="IPSEC_FAILURE_POINT"), "flags": SimTypeInt(signed=False, label="UInt32"), "keyingModuleType": SimTypeInt(signed=False, label="IKEEXT_KEY_MODULE_TYPE"), "mmState": SimTypeInt(signed=False, label="IKEEXT_MM_SA_STATE"), "saRole": SimTypeInt(signed=False, label="IKEEXT_SA_ROLE"), "mmAuthMethod": SimTypeInt(signed=False, label="IKEEXT_AUTHENTICATION_METHOD_TYPE"), "endCertHash": SimTypeFixedSizeArray(SimTypeChar(label="Byte"), 20), "mmId": SimTypeLongLong(signed=False, label="UInt64"), "mmFilterId": SimTypeLongLong(signed=False, label="UInt64"), "localPrincipalNameForAuth": SimTypePointer(SimTypeChar(label="Char"), offset=0), "remotePrincipalNameForAuth": SimTypePointer(SimTypeChar(label="Char"), offset=0), "numLocalPrincipalGroupSids": SimTypeInt(signed=False, label="UInt32"), "localPrincipalGroupSids": SimTypePointer(SimTypePointer(SimTypeChar(label="Char"), offset=0), offset=0), "numRemotePrincipalGroupSids": SimTypeInt(signed=False, label="UInt32"), "remotePrincipalGroupSids": SimTypePointer(SimTypePointer(SimTypeChar(label="Char"), offset=0), offset=0), "providerContextKey": SimTypePointer(SimTypeBottom(label="Guid"), offset=0)}, name="FWPM_NET_EVENT_IKEEXT_MM_FAILURE2_", pack=False, align=None), offset=0), "ikeQmFailure": SimTypePointer(SimStruct({"failureErrorCode": SimTypeInt(signed=False, label="UInt32"), "failurePoint": SimTypeInt(signed=False, label="IPSEC_FAILURE_POINT"), "keyingModuleType": SimTypeInt(signed=False, label="IKEEXT_KEY_MODULE_TYPE"), "qmState": SimTypeInt(signed=False, label="IKEEXT_QM_SA_STATE"), "saRole": SimTypeInt(signed=False, label="IKEEXT_SA_ROLE"), "saTrafficType": SimTypeInt(signed=False, label="IPSEC_TRAFFIC_TYPE"), "Anonymous1": SimUnion({"localSubNet": SimStruct({"type": SimTypeInt(signed=False, label="FWP_DATA_TYPE"), "Anonymous": SimUnion({"uint8": SimTypeChar(label="Byte"), "uint16": SimTypeShort(signed=False, label="UInt16"), "uint32": SimTypeInt(signed=False, label="UInt32"), "uint64": SimTypePointer(SimTypeLongLong(signed=False, label="UInt64"), offset=0), "int8": SimTypeChar(label="SByte"), "int16": SimTypeShort(signed=True, label="Int16"), "int32": SimTypeInt(signed=True, label="Int32"), "int64": SimTypePointer(SimTypeLongLong(signed=True, label="Int64"), offset=0), "float32": SimTypeFloat(size=32), "double64": SimTypePointer(SimTypeFloat(size=64), offset=0), "byteArray16": SimTypePointer(SimStruct({"byteArray16": SimTypeFixedSizeArray(SimTypeChar(label="Byte"), 16)}, name="FWP_BYTE_ARRAY16", pack=False, align=None), offset=0), "byteBlob": SimTypePointer(SimStruct({"size": SimTypeInt(signed=False, label="UInt32"), "data": SimTypePointer(SimTypeChar(label="Byte"), offset=0)}, name="FWP_BYTE_BLOB", pack=False, align=None), offset=0), "sid": SimTypePointer(SimTypeBottom(label="SID"), offset=0), "sd": SimTypePointer(SimStruct({"size": SimTypeInt(signed=False, label="UInt32"), "data": SimTypePointer(SimTypeChar(label="Byte"), offset=0)}, name="FWP_BYTE_BLOB", pack=False, align=None), offset=0), "tokenInformation": SimTypePointer(SimStruct({"sidCount": SimTypeInt(signed=False, label="UInt32"), "sids": SimTypePointer(SimTypeBottom(label="SID_AND_ATTRIBUTES"), offset=0), "restrictedSidCount": SimTypeInt(signed=False, label="UInt32"), "restrictedSids": SimTypePointer(SimTypeBottom(label="SID_AND_ATTRIBUTES"), offset=0)}, name="FWP_TOKEN_INFORMATION", pack=False, align=None), offset=0), "tokenAccessInformation": SimTypePointer(SimStruct({"size": SimTypeInt(signed=False, label="UInt32"), "data": SimTypePointer(SimTypeChar(label="Byte"), offset=0)}, name="FWP_BYTE_BLOB", pack=False, align=None), offset=0), "unicodeString": SimTypePointer(SimTypeChar(label="Char"), offset=0), "byteArray6": SimTypePointer(SimStruct({"byteArray6": SimTypeFixedSizeArray(SimTypeChar(label="Byte"), 6)}, name="FWP_BYTE_ARRAY6", pack=False, align=None), offset=0), "bitmapArray64": SimTypePointer(SimStruct({"bitmapArray64": SimTypeFixedSizeArray(SimTypeChar(label="Byte"), 8)}, name="FWP_BITMAP_ARRAY64_", pack=False, align=None), offset=0), "v4AddrMask": SimTypePointer(SimStruct({"addr": SimTypeInt(signed=False, label="UInt32"), "mask": SimTypeInt(signed=False, label="UInt32")}, name="FWP_V4_ADDR_AND_MASK", pack=False, align=None), offset=0), "v6AddrMask": SimTypePointer(SimStruct({"addr": SimTypeFixedSizeArray(SimTypeChar(label="Byte"), 16), "prefixLength": SimTypeChar(label="Byte")}, name="FWP_V6_ADDR_AND_MASK", pack=False, align=None), offset=0), "rangeValue": SimTypePointer(SimStruct({"valueLow": SimStruct({"type": SimTypeInt(signed=False, label="FWP_DATA_TYPE"), "Anonymous": SimUnion({"uint8": SimTypeChar(label="Byte"), "uint16": SimTypeShort(signed=False, label="UInt16"), "uint32": SimTypeInt(signed=False, label="UInt32"), "uint64": SimTypePointer(SimTypeLongLong(signed=False, label="UInt64"), offset=0), "int8": SimTypeChar(label="SByte"), "int16": SimTypeShort(signed=True, label="Int16"), "int32": SimTypeInt(signed=True, label="Int32"), "int64": SimTypePointer(SimTypeLongLong(signed=True, label="Int64"), offset=0), "float32": SimTypeFloat(size=32), "double64": SimTypePointer(SimTypeFloat(size=64), offset=0), "byteArray16": SimTypePointer(SimStruct({"byteArray16": SimTypeFixedSizeArray(SimTypeChar(label="Byte"), 16)}, name="FWP_BYTE_ARRAY16", pack=False, align=None), offset=0), "byteBlob": SimTypePointer(SimStruct({"size": SimTypeInt(signed=False, label="UInt32"), "data": SimTypePointer(SimTypeChar(label="Byte"), offset=0)}, name="FWP_BYTE_BLOB", pack=False, align=None), offset=0), "sid": SimTypePointer(SimTypeBottom(label="SID"), offset=0), "sd": SimTypePointer(SimStruct({"size": SimTypeInt(signed=False, label="UInt32"), "data": SimTypePointer(SimTypeChar(label="Byte"), offset=0)}, name="FWP_BYTE_BLOB", pack=False, align=None), offset=0), "tokenInformation": SimTypePointer(SimStruct({"sidCount": SimTypeInt(signed=False, label="UInt32"), "sids": SimTypePointer(SimTypeBottom(label="SID_AND_ATTRIBUTES"), offset=0), "restrictedSidCount": SimTypeInt(signed=False, label="UInt32"), "restrictedSids": SimTypePointer(SimTypeBottom(label="SID_AND_ATTRIBUTES"), offset=0)}, name="FWP_TOKEN_INFORMATION", pack=False, align=None), offset=0), "tokenAccessInformation": SimTypePointer(SimStruct({"size": SimTypeInt(signed=False, label="UInt32"), "data": SimTypePointer(SimTypeChar(label="Byte"), offset=0)}, name="FWP_BYTE_BLOB", pack=False, align=None), offset=0), "unicodeString": SimTypePointer(SimTypeChar(label="Char"), offset=0), "byteArray6": SimTypePointer(SimStruct({"byteArray6": SimTypeFixedSizeArray(SimTypeChar(label="Byte"), 6)}, name="FWP_BYTE_ARRAY6", pack=False, align=None), offset=0), "bitmapArray64": SimTypePointer(SimStruct({"bitmapArray64": SimTypeFixedSizeArray(SimTypeChar(label="Byte"), 8)}, name="FWP_BITMAP_ARRAY64_", pack=False, align=None), offset=0)}, name="<anon>", label="None")}, name="FWP_VALUE0", pack=False, align=None), "valueHigh": SimStruct({"type": SimTypeInt(signed=False, label="FWP_DATA_TYPE"), "Anonymous": SimUnion({"uint8": SimTypeChar(label="Byte"), "uint16": SimTypeShort(signed=False, label="UInt16"), "uint32": SimTypeInt(signed=False, label="UInt32"), "uint64": SimTypePointer(SimTypeLongLong(signed=False, label="UInt64"), offset=0), "int8": SimTypeChar(label="SByte"), "int16": SimTypeShort(signed=True, label="Int16"), "int32": SimTypeInt(signed=True, label="Int32"), "int64": SimTypePointer(SimTypeLongLong(signed=True, label="Int64"), offset=0), "float32": SimTypeFloat(size=32), "double64": SimTypePointer(SimTypeFloat(size=64), offset=0), "byteArray16": SimTypePointer(SimStruct({"byteArray16": SimTypeFixedSizeArray(SimTypeChar(label="Byte"), 16)}, name="FWP_BYTE_ARRAY16", pack=False, align=None), offset=0), "byteBlob": SimTypePointer(SimStruct({"size": SimTypeInt(signed=False, label="UInt32"), "data": SimTypePointer(SimTypeChar(label="Byte"), offset=0)}, name="FWP_BYTE_BLOB", pack=False, align=None), offset=0), "sid": SimTypePointer(SimTypeBottom(label="SID"), offset=0), "sd": SimTypePointer(SimStruct({"size": SimTypeInt(signed=False, label="UInt32"), "data": SimTypePointer(SimTypeChar(label="Byte"), offset=0)}, name="FWP_BYTE_BLOB", pack=False, align=None), offset=0), "tokenInformation": SimTypePointer(SimStruct({"sidCount": SimTypeInt(signed=False, label="UInt32"), "sids": SimTypePointer(SimTypeBottom(label="SID_AND_ATTRIBUTES"), offset=0), "restrictedSidCount": SimTypeInt(signed=False, label="UInt32"), "restrictedSids": SimTypePointer(SimTypeBottom(label="SID_AND_ATTRIBUTES"), offset=0)}, name="FWP_TOKEN_INFORMATION", pack=False, align=None), offset=0), "tokenAccessInformation": SimTypePointer(SimStruct({"size": SimTypeInt(signed=False, label="UInt32"), "data": SimTypePointer(SimTypeChar(label="Byte"), offset=0)}, name="FWP_BYTE_BLOB", pack=False, align=None), offset=0), "unicodeString": SimTypePointer(SimTypeChar(label="Char"), offset=0), "byteArray6": SimTypePointer(SimStruct({"byteArray6": SimTypeFixedSizeArray(SimTypeChar(label="Byte"), 6)}, name="FWP_BYTE_ARRAY6", pack=False, align=None), offset=0), "bitmapArray64": SimTypePointer(SimStruct({"bitmapArray64": SimTypeFixedSizeArray(SimTypeChar(label="Byte"), 8)}, name="FWP_BITMAP_ARRAY64_", pack=False, align=None), offset=0)}, name="<anon>", label="None")}, name="FWP_VALUE0", pack=False, align=None)}, name="FWP_RANGE0", pack=False, align=None), offset=0)}, name="<anon>", label="None")}, name="FWP_CONDITION_VALUE0", pack=False, align=None)}, name="<anon>", label="None"), "Anonymous2": SimUnion({"remoteSubNet": SimStruct({"type": SimTypeInt(signed=False, label="FWP_DATA_TYPE"), "Anonymous": SimUnion({"uint8": SimTypeChar(label="Byte"), "uint16": SimTypeShort(signed=False, label="UInt16"), "uint32": SimTypeInt(signed=False, label="UInt32"), "uint64": SimTypePointer(SimTypeLongLong(signed=False, label="UInt64"), offset=0), "int8": SimTypeChar(label="SByte"), "int16": SimTypeShort(signed=True, label="Int16"), "int32": SimTypeInt(signed=True, label="Int32"), "int64": SimTypePointer(SimTypeLongLong(signed=True, label="Int64"), offset=0), "float32": SimTypeFloat(size=32), "double64": SimTypePointer(SimTypeFloat(size=64), offset=0), "byteArray16": SimTypePointer(SimStruct({"byteArray16": SimTypeFixedSizeArray(SimTypeChar(label="Byte"), 16)}, name="FWP_BYTE_ARRAY16", pack=False, align=None), offset=0), "byteBlob": SimTypePointer(SimStruct({"size": SimTypeInt(signed=False, label="UInt32"), "data": SimTypePointer(SimTypeChar(label="Byte"), offset=0)}, name="FWP_BYTE_BLOB", pack=False, align=None), offset=0), "sid": SimTypePointer(SimTypeBottom(label="SID"), offset=0), "sd": SimTypePointer(SimStruct({"size": SimTypeInt(signed=False, label="UInt32"), "data": SimTypePointer(SimTypeChar(label="Byte"), offset=0)}, name="FWP_BYTE_BLOB", pack=False, align=None), offset=0), "tokenInformation": SimTypePointer(SimStruct({"sidCount": SimTypeInt(signed=False, label="UInt32"), "sids": SimTypePointer(SimTypeBottom(label="SID_AND_ATTRIBUTES"), offset=0), "restrictedSidCount": SimTypeInt(signed=False, label="UInt32"), "restrictedSids": SimTypePointer(SimTypeBottom(label="SID_AND_ATTRIBUTES"), offset=0)}, name="FWP_TOKEN_INFORMATION", pack=False, align=None), offset=0), "tokenAccessInformation": SimTypePointer(SimStruct({"size": SimTypeInt(signed=False, label="UInt32"), "data": SimTypePointer(SimTypeChar(label="Byte"), offset=0)}, name="FWP_BYTE_BLOB", pack=False, align=None), offset=0), "unicodeString": SimTypePointer(SimTypeChar(label="Char"), offset=0), "byteArray6": SimTypePointer(SimStruct({"byteArray6": SimTypeFixedSizeArray(SimTypeChar(label="Byte"), 6)}, name="FWP_BYTE_ARRAY6", pack=False, align=None), offset=0), "bitmapArray64": SimTypePointer(SimStruct({"bitmapArray64": SimTypeFixedSizeArray(SimTypeChar(label="Byte"), 8)}, name="FWP_BITMAP_ARRAY64_", pack=False, align=None), offset=0), "v4AddrMask": SimTypePointer(SimStruct({"addr": SimTypeInt(signed=False, label="UInt32"), "mask": SimTypeInt(signed=False, label="UInt32")}, name="FWP_V4_ADDR_AND_MASK", pack=False, align=None), offset=0), "v6AddrMask": SimTypePointer(SimStruct({"addr": SimTypeFixedSizeArray(SimTypeChar(label="Byte"), 16), "prefixLength": SimTypeChar(label="Byte")}, name="FWP_V6_ADDR_AND_MASK", pack=False, align=None), offset=0), "rangeValue": SimTypePointer(SimStruct({"valueLow": SimStruct({"type": SimTypeInt(signed=False, label="FWP_DATA_TYPE"), "Anonymous": SimUnion({"uint8": SimTypeChar(label="Byte"), "uint16": SimTypeShort(signed=False, label="UInt16"), "uint32": SimTypeInt(signed=False, label="UInt32"), "uint64": SimTypePointer(SimTypeLongLong(signed=False, label="UInt64"), offset=0), "int8": SimTypeChar(label="SByte"), "int16": SimTypeShort(signed=True, label="Int16"), "int32": SimTypeInt(signed=True, label="Int32"), "int64": SimTypePointer(SimTypeLongLong(signed=True, label="Int64"), offset=0), "float32": SimTypeFloat(size=32), "double64": SimTypePointer(SimTypeFloat(size=64), offset=0), "byteArray16": SimTypePointer(SimStruct({"byteArray16": SimTypeFixedSizeArray(SimTypeChar(label="Byte"), 16)}, name="FWP_BYTE_ARRAY16", pack=False, align=None), offset=0), "byteBlob": SimTypePointer(SimStruct({"size": SimTypeInt(signed=False, label="UInt32"), "data": SimTypePointer(SimTypeChar(label="Byte"), offset=0)}, name="FWP_BYTE_BLOB", pack=False, align=None), offset=0), "sid": SimTypePointer(SimTypeBottom(label="SID"), offset=0), "sd": SimTypePointer(SimStruct({"size": SimTypeInt(signed=False, label="UInt32"), "data": SimTypePointer(SimTypeChar(label="Byte"), offset=0)}, name="FWP_BYTE_BLOB", pack=False, align=None), offset=0), "tokenInformation": SimTypePointer(SimStruct({"sidCount": SimTypeInt(signed=False, label="UInt32"), "sids": SimTypePointer(SimTypeBottom(label="SID_AND_ATTRIBUTES"), offset=0), "restrictedSidCount": SimTypeInt(signed=False, label="UInt32"), "restrictedSids": SimTypePointer(SimTypeBottom(label="SID_AND_ATTRIBUTES"), offset=0)}, name="FWP_TOKEN_INFORMATION", pack=False, align=None), offset=0), "tokenAccessInformation": SimTypePointer(SimStruct({"size": SimTypeInt(signed=False, label="UInt32"), "data": SimTypePointer(SimTypeChar(label="Byte"), offset=0)}, name="FWP_BYTE_BLOB", pack=False, align=None), offset=0), "unicodeString": SimTypePointer(SimTypeChar(label="Char"), offset=0), "byteArray6": SimTypePointer(SimStruct({"byteArray6": SimTypeFixedSizeArray(SimTypeChar(label="Byte"), 6)}, name="FWP_BYTE_ARRAY6", pack=False, align=None), offset=0), "bitmapArray64": SimTypePointer(SimStruct({"bitmapArray64": SimTypeFixedSizeArray(SimTypeChar(label="Byte"), 8)}, name="FWP_BITMAP_ARRAY64_", pack=False, align=None), offset=0)}, name="<anon>", label="None")}, name="FWP_VALUE0", pack=False, align=None), "valueHigh": SimStruct({"type": SimTypeInt(signed=False, label="FWP_DATA_TYPE"), "Anonymous": SimUnion({"uint8": SimTypeChar(label="Byte"), "uint16": SimTypeShort(signed=False, label="UInt16"), "uint32": SimTypeInt(signed=False, label="UInt32"), "uint64": SimTypePointer(SimTypeLongLong(signed=False, label="UInt64"), offset=0), "int8": SimTypeChar(label="SByte"), "int16": SimTypeShort(signed=True, label="Int16"), "int32": SimTypeInt(signed=True, label="Int32"), "int64": SimTypePointer(SimTypeLongLong(signed=True, label="Int64"), offset=0), "float32": SimTypeFloat(size=32), "double64": SimTypePointer(SimTypeFloat(size=64), offset=0), "byteArray16": SimTypePointer(SimStruct({"byteArray16": SimTypeFixedSizeArray(SimTypeChar(label="Byte"), 16)}, name="FWP_BYTE_ARRAY16", pack=False, align=None), offset=0), "byteBlob": SimTypePointer(SimStruct({"size": SimTypeInt(signed=False, label="UInt32"), "data": SimTypePointer(SimTypeChar(label="Byte"), offset=0)}, name="FWP_BYTE_BLOB", pack=False, align=None), offset=0), "sid": SimTypePointer(SimTypeBottom(label="SID"), offset=0), "sd": SimTypePointer(SimStruct({"size": SimTypeInt(signed=False, label="UInt32"), "data": SimTypePointer(SimTypeChar(label="Byte"), offset=0)}, name="FWP_BYTE_BLOB", pack=False, align=None), offset=0), "tokenInformation": SimTypePointer(SimStruct({"sidCount": SimTypeInt(signed=False, label="UInt32"), "sids": SimTypePointer(SimTypeBottom(label="SID_AND_ATTRIBUTES"), offset=0), "restrictedSidCount": SimTypeInt(signed=False, label="UInt32"), "restrictedSids": SimTypePointer(SimTypeBottom(label="SID_AND_ATTRIBUTES"), offset=0)}, name="FWP_TOKEN_INFORMATION", pack=False, align=None), offset=0), "tokenAccessInformation": SimTypePointer(SimStruct({"size": SimTypeInt(signed=False, label="UInt32"), "data": SimTypePointer(SimTypeChar(label="Byte"), offset=0)}, name="FWP_BYTE_BLOB", pack=False, align=None), offset=0), "unicodeString": SimTypePointer(SimTypeChar(label="Char"), offset=0), "byteArray6": SimTypePointer(SimStruct({"byteArray6": SimTypeFixedSizeArray(SimTypeChar(label="Byte"), 6)}, name="FWP_BYTE_ARRAY6", pack=False, align=None), offset=0), "bitmapArray64": SimTypePointer(SimStruct({"bitmapArray64": SimTypeFixedSizeArray(SimTypeChar(label="Byte"), 8)}, name="FWP_BITMAP_ARRAY64_", pack=False, align=None), offset=0)}, name="<anon>", label="None")}, name="FWP_VALUE0", pack=False, align=None)}, name="FWP_RANGE0", pack=False, align=None), offset=0)}, name="<anon>", label="None")}, name="FWP_CONDITION_VALUE0", pack=False, align=None)}, name="<anon>", label="None"), "qmFilterId": SimTypeLongLong(signed=False, label="UInt64"), "mmSaLuid": SimTypeLongLong(signed=False, label="UInt64"), "mmProviderContextKey": SimTypeBottom(label="Guid")}, name="FWPM_NET_EVENT_IKEEXT_QM_FAILURE1_", pack=False, align=None), offset=0), "ikeEmFailure": SimTypePointer(SimStruct({"failureErrorCode": SimTypeInt(signed=False, label="UInt32"), "failurePoint": SimTypeInt(signed=False, label="IPSEC_FAILURE_POINT"), "flags": SimTypeInt(signed=False, label="UInt32"), "emState": SimTypeInt(signed=False, label="IKEEXT_EM_SA_STATE"), "saRole": SimTypeInt(signed=False, label="IKEEXT_SA_ROLE"), "emAuthMethod": SimTypeInt(signed=False, label="IKEEXT_AUTHENTICATION_METHOD_TYPE"), "endCertHash": SimTypeFixedSizeArray(SimTypeChar(label="Byte"), 20), "mmId": SimTypeLongLong(signed=False, label="UInt64"), "qmFilterId": SimTypeLongLong(signed=False, label="UInt64"), "localPrincipalNameForAuth": SimTypePointer(SimTypeChar(label="Char"), offset=0), "remotePrincipalNameForAuth": SimTypePointer(SimTypeChar(label="Char"), offset=0), "numLocalPrincipalGroupSids": SimTypeInt(signed=False, label="UInt32"), "localPrincipalGroupSids": SimTypePointer(SimTypePointer(SimTypeChar(label="Char"), offset=0), offset=0), "numRemotePrincipalGroupSids": SimTypeInt(signed=False, label="UInt32"), "remotePrincipalGroupSids": SimTypePointer(SimTypePointer(SimTypeChar(label="Char"), offset=0), offset=0), "saTrafficType": SimTypeInt(signed=False, label="IPSEC_TRAFFIC_TYPE")}, name="FWPM_NET_EVENT_IKEEXT_EM_FAILURE1", pack=False, align=None), offset=0), "classifyDrop": SimTypePointer(SimStruct({"filterId": SimTypeLongLong(signed=False, label="UInt64"), "layerId": SimTypeShort(signed=False, label="UInt16"), "reauthReason": SimTypeInt(signed=False, label="UInt32"), "originalProfile": SimTypeInt(signed=False, label="UInt32"), "currentProfile": SimTypeInt(signed=False, label="UInt32"), "msFwpDirection": SimTypeInt(signed=False, label="UInt32"), "isLoopback": SimTypeInt(signed=True, label="Int32"), "vSwitchId": SimStruct({"size": SimTypeInt(signed=False, label="UInt32"), "data": SimTypePointer(SimTypeChar(label="Byte"), offset=0)}, name="FWP_BYTE_BLOB", pack=False, align=None), "vSwitchSourcePort": SimTypeInt(signed=False, label="UInt32"), "vSwitchDestinationPort": SimTypeInt(signed=False, label="UInt32")}, name="FWPM_NET_EVENT_CLASSIFY_DROP2", pack=False, align=None), offset=0), "ipsecDrop": SimTypePointer(SimStruct({"failureStatus": SimTypeInt(signed=True, label="Int32"), "direction": SimTypeInt(signed=False, label="FWP_DIRECTION"), "spi": SimTypeInt(signed=False, label="UInt32"), "filterId": SimTypeLongLong(signed=False, label="UInt64"), "layerId": SimTypeShort(signed=False, label="UInt16")}, name="FWPM_NET_EVENT_IPSEC_KERNEL_DROP0", pack=False, align=None), offset=0), "idpDrop": SimTypePointer(SimStruct({"ipVersion": SimTypeInt(signed=False, label="FWP_IP_VERSION"), "Anonymous1": SimUnion({"publicHostV4Addr": SimTypeInt(signed=False, label="UInt32"), "publicHostV6Addr": SimTypeFixedSizeArray(SimTypeChar(label="Byte"), 16)}, name="<anon>", label="None"), "Anonymous2": SimUnion({"internalHostV4Addr": SimTypeInt(signed=False, label="UInt32"), "internalHostV6Addr": SimTypeFixedSizeArray(SimTypeChar(label="Byte"), 16)}, name="<anon>", label="None"), "failureStatus": SimTypeInt(signed=True, label="Int32"), "direction": SimTypeInt(signed=False, label="FWP_DIRECTION")}, name="FWPM_NET_EVENT_IPSEC_DOSP_DROP0", pack=False, align=None), offset=0), "classifyAllow": SimTypePointer(SimStruct({"filterId": SimTypeLongLong(signed=False, label="UInt64"), "layerId": SimTypeShort(signed=False, label="UInt16"), "reauthReason": SimTypeInt(signed=False, label="UInt32"), "originalProfile": SimTypeInt(signed=False, label="UInt32"), "currentProfile": SimTypeInt(signed=False, label="UInt32"), "msFwpDirection": SimTypeInt(signed=False, label="UInt32"), "isLoopback": SimTypeInt(signed=True, label="Int32")}, name="FWPM_NET_EVENT_CLASSIFY_ALLOW0", pack=False, align=None), offset=0), "capabilityDrop": SimTypePointer(SimStruct({"networkCapabilityId": SimTypeInt(signed=False, label="FWPM_APPC_NETWORK_CAPABILITY_TYPE"), "filterId": SimTypeLongLong(signed=False, label="UInt64"), "isLoopback": SimTypeInt(signed=True, label="Int32")}, name="FWPM_NET_EVENT_CAPABILITY_DROP0", pack=False, align=None), offset=0), "capabilityAllow": SimTypePointer(SimStruct({"networkCapabilityId": SimTypeInt(signed=False, label="FWPM_APPC_NETWORK_CAPABILITY_TYPE"), "filterId": SimTypeLongLong(signed=False, label="UInt64"), "isLoopback": SimTypeInt(signed=True, label="Int32")}, name="FWPM_NET_EVENT_CAPABILITY_ALLOW0", pack=False, align=None), offset=0), "classifyDropMac": SimTypePointer(SimStruct({"localMacAddr": SimStruct({"byteArray6": SimTypeFixedSizeArray(SimTypeChar(label="Byte"), 6)}, name="FWP_BYTE_ARRAY6", pack=False, align=None), "remoteMacAddr": SimStruct({"byteArray6": SimTypeFixedSizeArray(SimTypeChar(label="Byte"), 6)}, name="FWP_BYTE_ARRAY6", pack=False, align=None), "mediaType": SimTypeInt(signed=False, label="UInt32"), "ifType": SimTypeInt(signed=False, label="UInt32"), "etherType": SimTypeShort(signed=False, label="UInt16"), "ndisPortNumber": SimTypeInt(signed=False, label="UInt32"), "reserved": SimTypeInt(signed=False, label="UInt32"), "vlanTag": SimTypeShort(signed=False, label="UInt16"), "ifLuid": SimTypeLongLong(signed=False, label="UInt64"), "filterId": SimTypeLongLong(signed=False, label="UInt64"), "layerId": SimTypeShort(signed=False, label="UInt16"), "reauthReason": SimTypeInt(signed=False, label="UInt32"), "originalProfile": SimTypeInt(signed=False, label="UInt32"), "currentProfile": SimTypeInt(signed=False, label="UInt32"), "msFwpDirection": SimTypeInt(signed=False, label="UInt32"), "isLoopback": SimTypeInt(signed=True, label="Int32"), "vSwitchId": SimStruct({"size": SimTypeInt(signed=False, label="UInt32"), "data": SimTypePointer(SimTypeChar(label="Byte"), offset=0)}, name="FWP_BYTE_BLOB", pack=False, align=None), "vSwitchSourcePort": SimTypeInt(signed=False, label="UInt32"), "vSwitchDestinationPort": SimTypeInt(signed=False, label="UInt32")}, name="FWPM_NET_EVENT_CLASSIFY_DROP_MAC0", pack=False, align=None), offset=0), "lpmPacketArrival": SimTypePointer(SimStruct({"spi": SimTypeInt(signed=False, label="UInt32")}, name="FWPM_NET_EVENT_LPM_PACKET_ARRIVAL0_", pack=False, align=None), offset=0)}, name="<anon>", label="None")}, name="FWPM_NET_EVENT5_", pack=False, align=None), offset=0), offset=0), offset=0), SimTypePointer(SimTypeInt(signed=False, label="UInt32"), offset=0)], SimTypeInt(signed=False, label="UInt32"), arg_names=["engineHandle", "enumHandle", "numEntriesRequested", "entries", "numEntriesReturned"]),
        # 
        'FwpmNetEventDestroyEnumHandle0': SimTypeFunction([SimTypePointer(SimTypeInt(signed=True, label="Int"), label="IntPtr", offset=0), SimTypePointer(SimTypeInt(signed=True, label="Int"), label="IntPtr", offset=0)], SimTypeInt(signed=False, label="UInt32"), arg_names=["engineHandle", "enumHandle"]),
        # 
        'FwpmNetEventsGetSecurityInfo0': SimTypeFunction([SimTypePointer(SimTypeInt(signed=True, label="Int"), label="IntPtr", offset=0), SimTypeInt(signed=False, label="UInt32"), SimTypePointer(SimTypePointer(SimTypeInt(signed=True, label="Int"), label="IntPtr", offset=0), offset=0), SimTypePointer(SimTypePointer(SimTypeInt(signed=True, label="Int"), label="IntPtr", offset=0), offset=0), SimTypePointer(SimTypePointer(SimStruct({"AclRevision": SimTypeChar(label="Byte"), "Sbz1": SimTypeChar(label="Byte"), "AclSize": SimTypeShort(signed=False, label="UInt16"), "AceCount": SimTypeShort(signed=False, label="UInt16"), "Sbz2": SimTypeShort(signed=False, label="UInt16")}, name="ACL", pack=False, align=None), offset=0), offset=0), SimTypePointer(SimTypePointer(SimStruct({"AclRevision": SimTypeChar(label="Byte"), "Sbz1": SimTypeChar(label="Byte"), "AclSize": SimTypeShort(signed=False, label="UInt16"), "AceCount": SimTypeShort(signed=False, label="UInt16"), "Sbz2": SimTypeShort(signed=False, label="UInt16")}, name="ACL", pack=False, align=None), offset=0), offset=0), SimTypePointer(SimTypePointer(SimStruct({"Revision": SimTypeChar(label="Byte"), "Sbz1": SimTypeChar(label="Byte"), "Control": SimTypeShort(signed=False, label="UInt16"), "Owner": SimTypePointer(SimTypeInt(signed=True, label="Int"), label="IntPtr", offset=0), "Group": SimTypePointer(SimTypeInt(signed=True, label="Int"), label="IntPtr", offset=0), "Sacl": SimTypePointer(SimStruct({"AclRevision": SimTypeChar(label="Byte"), "Sbz1": SimTypeChar(label="Byte"), "AclSize": SimTypeShort(signed=False, label="UInt16"), "AceCount": SimTypeShort(signed=False, label="UInt16"), "Sbz2": SimTypeShort(signed=False, label="UInt16")}, name="ACL", pack=False, align=None), offset=0), "Dacl": SimTypePointer(SimStruct({"AclRevision": SimTypeChar(label="Byte"), "Sbz1": SimTypeChar(label="Byte"), "AclSize": SimTypeShort(signed=False, label="UInt16"), "AceCount": SimTypeShort(signed=False, label="UInt16"), "Sbz2": SimTypeShort(signed=False, label="UInt16")}, name="ACL", pack=False, align=None), offset=0)}, name="SECURITY_DESCRIPTOR", pack=False, align=None), offset=0), offset=0)], SimTypeInt(signed=False, label="UInt32"), arg_names=["engineHandle", "securityInfo", "sidOwner", "sidGroup", "dacl", "sacl", "securityDescriptor"]),
        # 
        'FwpmNetEventsSetSecurityInfo0': SimTypeFunction([SimTypePointer(SimTypeInt(signed=True, label="Int"), label="IntPtr", offset=0), SimTypeInt(signed=False, label="UInt32"), SimTypePointer(SimStruct({"Revision": SimTypeChar(label="Byte"), "SubAuthorityCount": SimTypeChar(label="Byte"), "IdentifierAuthority": SimStruct({"Value": SimTypeFixedSizeArray(SimTypeChar(label="Byte"), 6)}, name="SID_IDENTIFIER_AUTHORITY", pack=False, align=None), "SubAuthority": SimTypePointer(SimTypeInt(signed=False, label="UInt32"), offset=0)}, name="SID", pack=False, align=None), offset=0), SimTypePointer(SimStruct({"Revision": SimTypeChar(label="Byte"), "SubAuthorityCount": SimTypeChar(label="Byte"), "IdentifierAuthority": SimStruct({"Value": SimTypeFixedSizeArray(SimTypeChar(label="Byte"), 6)}, name="SID_IDENTIFIER_AUTHORITY", pack=False, align=None), "SubAuthority": SimTypePointer(SimTypeInt(signed=False, label="UInt32"), offset=0)}, name="SID", pack=False, align=None), offset=0), SimTypePointer(SimStruct({"AclRevision": SimTypeChar(label="Byte"), "Sbz1": SimTypeChar(label="Byte"), "AclSize": SimTypeShort(signed=False, label="UInt16"), "AceCount": SimTypeShort(signed=False, label="UInt16"), "Sbz2": SimTypeShort(signed=False, label="UInt16")}, name="ACL", pack=False, align=None), offset=0), SimTypePointer(SimStruct({"AclRevision": SimTypeChar(label="Byte"), "Sbz1": SimTypeChar(label="Byte"), "AclSize": SimTypeShort(signed=False, label="UInt16"), "AceCount": SimTypeShort(signed=False, label="UInt16"), "Sbz2": SimTypeShort(signed=False, label="UInt16")}, name="ACL", pack=False, align=None), offset=0)], SimTypeInt(signed=False, label="UInt32"), arg_names=["engineHandle", "securityInfo", "sidOwner", "sidGroup", "dacl", "sacl"]),
        # 
        'FwpmNetEventSubscribe0': SimTypeFunction([SimTypePointer(SimTypeInt(signed=True, label="Int"), label="IntPtr", offset=0), SimTypePointer(SimStruct({"enumTemplate": SimTypePointer(SimStruct({"startTime": SimStruct({"dwLowDateTime": SimTypeInt(signed=False, label="UInt32"), "dwHighDateTime": SimTypeInt(signed=False, label="UInt32")}, name="FILETIME", pack=False, align=None), "endTime": SimStruct({"dwLowDateTime": SimTypeInt(signed=False, label="UInt32"), "dwHighDateTime": SimTypeInt(signed=False, label="UInt32")}, name="FILETIME", pack=False, align=None), "numFilterConditions": SimTypeInt(signed=False, label="UInt32"), "filterCondition": SimTypePointer(SimStruct({"fieldKey": SimTypeBottom(label="Guid"), "matchType": SimTypeInt(signed=False, label="FWP_MATCH_TYPE"), "conditionValue": SimStruct({"type": SimTypeInt(signed=False, label="FWP_DATA_TYPE"), "Anonymous": SimUnion({"uint8": SimTypeChar(label="Byte"), "uint16": SimTypeShort(signed=False, label="UInt16"), "uint32": SimTypeInt(signed=False, label="UInt32"), "uint64": SimTypePointer(SimTypeLongLong(signed=False, label="UInt64"), offset=0), "int8": SimTypeChar(label="SByte"), "int16": SimTypeShort(signed=True, label="Int16"), "int32": SimTypeInt(signed=True, label="Int32"), "int64": SimTypePointer(SimTypeLongLong(signed=True, label="Int64"), offset=0), "float32": SimTypeFloat(size=32), "double64": SimTypePointer(SimTypeFloat(size=64), offset=0), "byteArray16": SimTypePointer(SimStruct({"byteArray16": SimTypeFixedSizeArray(SimTypeChar(label="Byte"), 16)}, name="FWP_BYTE_ARRAY16", pack=False, align=None), offset=0), "byteBlob": SimTypePointer(SimStruct({"size": SimTypeInt(signed=False, label="UInt32"), "data": SimTypePointer(SimTypeChar(label="Byte"), offset=0)}, name="FWP_BYTE_BLOB", pack=False, align=None), offset=0), "sid": SimTypePointer(SimTypeBottom(label="SID"), offset=0), "sd": SimTypePointer(SimStruct({"size": SimTypeInt(signed=False, label="UInt32"), "data": SimTypePointer(SimTypeChar(label="Byte"), offset=0)}, name="FWP_BYTE_BLOB", pack=False, align=None), offset=0), "tokenInformation": SimTypePointer(SimStruct({"sidCount": SimTypeInt(signed=False, label="UInt32"), "sids": SimTypePointer(SimTypeBottom(label="SID_AND_ATTRIBUTES"), offset=0), "restrictedSidCount": SimTypeInt(signed=False, label="UInt32"), "restrictedSids": SimTypePointer(SimTypeBottom(label="SID_AND_ATTRIBUTES"), offset=0)}, name="FWP_TOKEN_INFORMATION", pack=False, align=None), offset=0), "tokenAccessInformation": SimTypePointer(SimStruct({"size": SimTypeInt(signed=False, label="UInt32"), "data": SimTypePointer(SimTypeChar(label="Byte"), offset=0)}, name="FWP_BYTE_BLOB", pack=False, align=None), offset=0), "unicodeString": SimTypePointer(SimTypeChar(label="Char"), offset=0), "byteArray6": SimTypePointer(SimStruct({"byteArray6": SimTypeFixedSizeArray(SimTypeChar(label="Byte"), 6)}, name="FWP_BYTE_ARRAY6", pack=False, align=None), offset=0), "bitmapArray64": SimTypePointer(SimStruct({"bitmapArray64": SimTypeFixedSizeArray(SimTypeChar(label="Byte"), 8)}, name="FWP_BITMAP_ARRAY64_", pack=False, align=None), offset=0), "v4AddrMask": SimTypePointer(SimStruct({"addr": SimTypeInt(signed=False, label="UInt32"), "mask": SimTypeInt(signed=False, label="UInt32")}, name="FWP_V4_ADDR_AND_MASK", pack=False, align=None), offset=0), "v6AddrMask": SimTypePointer(SimStruct({"addr": SimTypeFixedSizeArray(SimTypeChar(label="Byte"), 16), "prefixLength": SimTypeChar(label="Byte")}, name="FWP_V6_ADDR_AND_MASK", pack=False, align=None), offset=0), "rangeValue": SimTypePointer(SimStruct({"valueLow": SimStruct({"type": SimTypeInt(signed=False, label="FWP_DATA_TYPE"), "Anonymous": SimUnion({"uint8": SimTypeChar(label="Byte"), "uint16": SimTypeShort(signed=False, label="UInt16"), "uint32": SimTypeInt(signed=False, label="UInt32"), "uint64": SimTypePointer(SimTypeLongLong(signed=False, label="UInt64"), offset=0), "int8": SimTypeChar(label="SByte"), "int16": SimTypeShort(signed=True, label="Int16"), "int32": SimTypeInt(signed=True, label="Int32"), "int64": SimTypePointer(SimTypeLongLong(signed=True, label="Int64"), offset=0), "float32": SimTypeFloat(size=32), "double64": SimTypePointer(SimTypeFloat(size=64), offset=0), "byteArray16": SimTypePointer(SimStruct({"byteArray16": SimTypeFixedSizeArray(SimTypeChar(label="Byte"), 16)}, name="FWP_BYTE_ARRAY16", pack=False, align=None), offset=0), "byteBlob": SimTypePointer(SimStruct({"size": SimTypeInt(signed=False, label="UInt32"), "data": SimTypePointer(SimTypeChar(label="Byte"), offset=0)}, name="FWP_BYTE_BLOB", pack=False, align=None), offset=0), "sid": SimTypePointer(SimTypeBottom(label="SID"), offset=0), "sd": SimTypePointer(SimStruct({"size": SimTypeInt(signed=False, label="UInt32"), "data": SimTypePointer(SimTypeChar(label="Byte"), offset=0)}, name="FWP_BYTE_BLOB", pack=False, align=None), offset=0), "tokenInformation": SimTypePointer(SimStruct({"sidCount": SimTypeInt(signed=False, label="UInt32"), "sids": SimTypePointer(SimTypeBottom(label="SID_AND_ATTRIBUTES"), offset=0), "restrictedSidCount": SimTypeInt(signed=False, label="UInt32"), "restrictedSids": SimTypePointer(SimTypeBottom(label="SID_AND_ATTRIBUTES"), offset=0)}, name="FWP_TOKEN_INFORMATION", pack=False, align=None), offset=0), "tokenAccessInformation": SimTypePointer(SimStruct({"size": SimTypeInt(signed=False, label="UInt32"), "data": SimTypePointer(SimTypeChar(label="Byte"), offset=0)}, name="FWP_BYTE_BLOB", pack=False, align=None), offset=0), "unicodeString": SimTypePointer(SimTypeChar(label="Char"), offset=0), "byteArray6": SimTypePointer(SimStruct({"byteArray6": SimTypeFixedSizeArray(SimTypeChar(label="Byte"), 6)}, name="FWP_BYTE_ARRAY6", pack=False, align=None), offset=0), "bitmapArray64": SimTypePointer(SimStruct({"bitmapArray64": SimTypeFixedSizeArray(SimTypeChar(label="Byte"), 8)}, name="FWP_BITMAP_ARRAY64_", pack=False, align=None), offset=0)}, name="<anon>", label="None")}, name="FWP_VALUE0", pack=False, align=None), "valueHigh": SimStruct({"type": SimTypeInt(signed=False, label="FWP_DATA_TYPE"), "Anonymous": SimUnion({"uint8": SimTypeChar(label="Byte"), "uint16": SimTypeShort(signed=False, label="UInt16"), "uint32": SimTypeInt(signed=False, label="UInt32"), "uint64": SimTypePointer(SimTypeLongLong(signed=False, label="UInt64"), offset=0), "int8": SimTypeChar(label="SByte"), "int16": SimTypeShort(signed=True, label="Int16"), "int32": SimTypeInt(signed=True, label="Int32"), "int64": SimTypePointer(SimTypeLongLong(signed=True, label="Int64"), offset=0), "float32": SimTypeFloat(size=32), "double64": SimTypePointer(SimTypeFloat(size=64), offset=0), "byteArray16": SimTypePointer(SimStruct({"byteArray16": SimTypeFixedSizeArray(SimTypeChar(label="Byte"), 16)}, name="FWP_BYTE_ARRAY16", pack=False, align=None), offset=0), "byteBlob": SimTypePointer(SimStruct({"size": SimTypeInt(signed=False, label="UInt32"), "data": SimTypePointer(SimTypeChar(label="Byte"), offset=0)}, name="FWP_BYTE_BLOB", pack=False, align=None), offset=0), "sid": SimTypePointer(SimTypeBottom(label="SID"), offset=0), "sd": SimTypePointer(SimStruct({"size": SimTypeInt(signed=False, label="UInt32"), "data": SimTypePointer(SimTypeChar(label="Byte"), offset=0)}, name="FWP_BYTE_BLOB", pack=False, align=None), offset=0), "tokenInformation": SimTypePointer(SimStruct({"sidCount": SimTypeInt(signed=False, label="UInt32"), "sids": SimTypePointer(SimTypeBottom(label="SID_AND_ATTRIBUTES"), offset=0), "restrictedSidCount": SimTypeInt(signed=False, label="UInt32"), "restrictedSids": SimTypePointer(SimTypeBottom(label="SID_AND_ATTRIBUTES"), offset=0)}, name="FWP_TOKEN_INFORMATION", pack=False, align=None), offset=0), "tokenAccessInformation": SimTypePointer(SimStruct({"size": SimTypeInt(signed=False, label="UInt32"), "data": SimTypePointer(SimTypeChar(label="Byte"), offset=0)}, name="FWP_BYTE_BLOB", pack=False, align=None), offset=0), "unicodeString": SimTypePointer(SimTypeChar(label="Char"), offset=0), "byteArray6": SimTypePointer(SimStruct({"byteArray6": SimTypeFixedSizeArray(SimTypeChar(label="Byte"), 6)}, name="FWP_BYTE_ARRAY6", pack=False, align=None), offset=0), "bitmapArray64": SimTypePointer(SimStruct({"bitmapArray64": SimTypeFixedSizeArray(SimTypeChar(label="Byte"), 8)}, name="FWP_BITMAP_ARRAY64_", pack=False, align=None), offset=0)}, name="<anon>", label="None")}, name="FWP_VALUE0", pack=False, align=None)}, name="FWP_RANGE0", pack=False, align=None), offset=0)}, name="<anon>", label="None")}, name="FWP_CONDITION_VALUE0", pack=False, align=None)}, name="FWPM_FILTER_CONDITION0", pack=False, align=None), offset=0)}, name="FWPM_NET_EVENT_ENUM_TEMPLATE0", pack=False, align=None), offset=0), "flags": SimTypeInt(signed=False, label="UInt32"), "sessionKey": SimTypeBottom(label="Guid")}, name="FWPM_NET_EVENT_SUBSCRIPTION0", pack=False, align=None), offset=0), SimTypePointer(SimTypeFunction([SimTypePointer(SimTypeBottom(label="Void"), offset=0), SimTypePointer(SimStruct({"header": SimStruct({"timeStamp": SimStruct({"dwLowDateTime": SimTypeInt(signed=False, label="UInt32"), "dwHighDateTime": SimTypeInt(signed=False, label="UInt32")}, name="FILETIME", pack=False, align=None), "flags": SimTypeInt(signed=False, label="UInt32"), "ipVersion": SimTypeInt(signed=False, label="FWP_IP_VERSION"), "ipProtocol": SimTypeChar(label="Byte"), "Anonymous1": SimUnion({"localAddrV4": SimTypeInt(signed=False, label="UInt32"), "localAddrV6": SimStruct({"byteArray16": SimTypeFixedSizeArray(SimTypeChar(label="Byte"), 16)}, name="FWP_BYTE_ARRAY16", pack=False, align=None)}, name="<anon>", label="None"), "Anonymous2": SimUnion({"remoteAddrV4": SimTypeInt(signed=False, label="UInt32"), "remoteAddrV6": SimStruct({"byteArray16": SimTypeFixedSizeArray(SimTypeChar(label="Byte"), 16)}, name="FWP_BYTE_ARRAY16", pack=False, align=None)}, name="<anon>", label="None"), "localPort": SimTypeShort(signed=False, label="UInt16"), "remotePort": SimTypeShort(signed=False, label="UInt16"), "scopeId": SimTypeInt(signed=False, label="UInt32"), "appId": SimStruct({"size": SimTypeInt(signed=False, label="UInt32"), "data": SimTypePointer(SimTypeChar(label="Byte"), offset=0)}, name="FWP_BYTE_BLOB", pack=False, align=None), "userId": SimTypePointer(SimTypeBottom(label="SID"), offset=0), "Anonymous3": SimUnion({"Anonymous": SimStruct({"reserved1": SimTypeInt(signed=False, label="FWP_AF"), "Anonymous": SimUnion({"Anonymous": SimStruct({"reserved2": SimStruct({"byteArray6": SimTypeFixedSizeArray(SimTypeChar(label="Byte"), 6)}, name="FWP_BYTE_ARRAY6", pack=False, align=None), "reserved3": SimStruct({"byteArray6": SimTypeFixedSizeArray(SimTypeChar(label="Byte"), 6)}, name="FWP_BYTE_ARRAY6", pack=False, align=None), "reserved4": SimTypeInt(signed=False, label="UInt32"), "reserved5": SimTypeInt(signed=False, label="UInt32"), "reserved6": SimTypeShort(signed=False, label="UInt16"), "reserved7": SimTypeInt(signed=False, label="UInt32"), "reserved8": SimTypeInt(signed=False, label="UInt32"), "reserved9": SimTypeShort(signed=False, label="UInt16"), "reserved10": SimTypeLongLong(signed=False, label="UInt64")}, name="_Anonymous_e__Struct", pack=False, align=None)}, name="<anon>", label="None")}, name="_Anonymous_e__Struct", pack=False, align=None)}, name="<anon>", label="None")}, name="FWPM_NET_EVENT_HEADER1", pack=False, align=None), "type": SimTypeInt(signed=False, label="FWPM_NET_EVENT_TYPE"), "Anonymous": SimUnion({"ikeMmFailure": SimTypePointer(SimStruct({"failureErrorCode": SimTypeInt(signed=False, label="UInt32"), "failurePoint": SimTypeInt(signed=False, label="IPSEC_FAILURE_POINT"), "flags": SimTypeInt(signed=False, label="UInt32"), "keyingModuleType": SimTypeInt(signed=False, label="IKEEXT_KEY_MODULE_TYPE"), "mmState": SimTypeInt(signed=False, label="IKEEXT_MM_SA_STATE"), "saRole": SimTypeInt(signed=False, label="IKEEXT_SA_ROLE"), "mmAuthMethod": SimTypeInt(signed=False, label="IKEEXT_AUTHENTICATION_METHOD_TYPE"), "endCertHash": SimTypeFixedSizeArray(SimTypeChar(label="Byte"), 20), "mmId": SimTypeLongLong(signed=False, label="UInt64"), "mmFilterId": SimTypeLongLong(signed=False, label="UInt64"), "localPrincipalNameForAuth": SimTypePointer(SimTypeChar(label="Char"), offset=0), "remotePrincipalNameForAuth": SimTypePointer(SimTypeChar(label="Char"), offset=0), "numLocalPrincipalGroupSids": SimTypeInt(signed=False, label="UInt32"), "localPrincipalGroupSids": SimTypePointer(SimTypePointer(SimTypeChar(label="Char"), offset=0), offset=0), "numRemotePrincipalGroupSids": SimTypeInt(signed=False, label="UInt32"), "remotePrincipalGroupSids": SimTypePointer(SimTypePointer(SimTypeChar(label="Char"), offset=0), offset=0)}, name="FWPM_NET_EVENT_IKEEXT_MM_FAILURE1", pack=False, align=None), offset=0), "ikeQmFailure": SimTypePointer(SimStruct({"failureErrorCode": SimTypeInt(signed=False, label="UInt32"), "failurePoint": SimTypeInt(signed=False, label="IPSEC_FAILURE_POINT"), "keyingModuleType": SimTypeInt(signed=False, label="IKEEXT_KEY_MODULE_TYPE"), "qmState": SimTypeInt(signed=False, label="IKEEXT_QM_SA_STATE"), "saRole": SimTypeInt(signed=False, label="IKEEXT_SA_ROLE"), "saTrafficType": SimTypeInt(signed=False, label="IPSEC_TRAFFIC_TYPE"), "Anonymous1": SimUnion({"localSubNet": SimStruct({"type": SimTypeInt(signed=False, label="FWP_DATA_TYPE"), "Anonymous": SimUnion({"uint8": SimTypeChar(label="Byte"), "uint16": SimTypeShort(signed=False, label="UInt16"), "uint32": SimTypeInt(signed=False, label="UInt32"), "uint64": SimTypePointer(SimTypeLongLong(signed=False, label="UInt64"), offset=0), "int8": SimTypeChar(label="SByte"), "int16": SimTypeShort(signed=True, label="Int16"), "int32": SimTypeInt(signed=True, label="Int32"), "int64": SimTypePointer(SimTypeLongLong(signed=True, label="Int64"), offset=0), "float32": SimTypeFloat(size=32), "double64": SimTypePointer(SimTypeFloat(size=64), offset=0), "byteArray16": SimTypePointer(SimStruct({"byteArray16": SimTypeFixedSizeArray(SimTypeChar(label="Byte"), 16)}, name="FWP_BYTE_ARRAY16", pack=False, align=None), offset=0), "byteBlob": SimTypePointer(SimStruct({"size": SimTypeInt(signed=False, label="UInt32"), "data": SimTypePointer(SimTypeChar(label="Byte"), offset=0)}, name="FWP_BYTE_BLOB", pack=False, align=None), offset=0), "sid": SimTypePointer(SimTypeBottom(label="SID"), offset=0), "sd": SimTypePointer(SimStruct({"size": SimTypeInt(signed=False, label="UInt32"), "data": SimTypePointer(SimTypeChar(label="Byte"), offset=0)}, name="FWP_BYTE_BLOB", pack=False, align=None), offset=0), "tokenInformation": SimTypePointer(SimStruct({"sidCount": SimTypeInt(signed=False, label="UInt32"), "sids": SimTypePointer(SimTypeBottom(label="SID_AND_ATTRIBUTES"), offset=0), "restrictedSidCount": SimTypeInt(signed=False, label="UInt32"), "restrictedSids": SimTypePointer(SimTypeBottom(label="SID_AND_ATTRIBUTES"), offset=0)}, name="FWP_TOKEN_INFORMATION", pack=False, align=None), offset=0), "tokenAccessInformation": SimTypePointer(SimStruct({"size": SimTypeInt(signed=False, label="UInt32"), "data": SimTypePointer(SimTypeChar(label="Byte"), offset=0)}, name="FWP_BYTE_BLOB", pack=False, align=None), offset=0), "unicodeString": SimTypePointer(SimTypeChar(label="Char"), offset=0), "byteArray6": SimTypePointer(SimStruct({"byteArray6": SimTypeFixedSizeArray(SimTypeChar(label="Byte"), 6)}, name="FWP_BYTE_ARRAY6", pack=False, align=None), offset=0), "bitmapArray64": SimTypePointer(SimStruct({"bitmapArray64": SimTypeFixedSizeArray(SimTypeChar(label="Byte"), 8)}, name="FWP_BITMAP_ARRAY64_", pack=False, align=None), offset=0), "v4AddrMask": SimTypePointer(SimStruct({"addr": SimTypeInt(signed=False, label="UInt32"), "mask": SimTypeInt(signed=False, label="UInt32")}, name="FWP_V4_ADDR_AND_MASK", pack=False, align=None), offset=0), "v6AddrMask": SimTypePointer(SimStruct({"addr": SimTypeFixedSizeArray(SimTypeChar(label="Byte"), 16), "prefixLength": SimTypeChar(label="Byte")}, name="FWP_V6_ADDR_AND_MASK", pack=False, align=None), offset=0), "rangeValue": SimTypePointer(SimStruct({"valueLow": SimStruct({"type": SimTypeInt(signed=False, label="FWP_DATA_TYPE"), "Anonymous": SimUnion({"uint8": SimTypeChar(label="Byte"), "uint16": SimTypeShort(signed=False, label="UInt16"), "uint32": SimTypeInt(signed=False, label="UInt32"), "uint64": SimTypePointer(SimTypeLongLong(signed=False, label="UInt64"), offset=0), "int8": SimTypeChar(label="SByte"), "int16": SimTypeShort(signed=True, label="Int16"), "int32": SimTypeInt(signed=True, label="Int32"), "int64": SimTypePointer(SimTypeLongLong(signed=True, label="Int64"), offset=0), "float32": SimTypeFloat(size=32), "double64": SimTypePointer(SimTypeFloat(size=64), offset=0), "byteArray16": SimTypePointer(SimStruct({"byteArray16": SimTypeFixedSizeArray(SimTypeChar(label="Byte"), 16)}, name="FWP_BYTE_ARRAY16", pack=False, align=None), offset=0), "byteBlob": SimTypePointer(SimStruct({"size": SimTypeInt(signed=False, label="UInt32"), "data": SimTypePointer(SimTypeChar(label="Byte"), offset=0)}, name="FWP_BYTE_BLOB", pack=False, align=None), offset=0), "sid": SimTypePointer(SimTypeBottom(label="SID"), offset=0), "sd": SimTypePointer(SimStruct({"size": SimTypeInt(signed=False, label="UInt32"), "data": SimTypePointer(SimTypeChar(label="Byte"), offset=0)}, name="FWP_BYTE_BLOB", pack=False, align=None), offset=0), "tokenInformation": SimTypePointer(SimStruct({"sidCount": SimTypeInt(signed=False, label="UInt32"), "sids": SimTypePointer(SimTypeBottom(label="SID_AND_ATTRIBUTES"), offset=0), "restrictedSidCount": SimTypeInt(signed=False, label="UInt32"), "restrictedSids": SimTypePointer(SimTypeBottom(label="SID_AND_ATTRIBUTES"), offset=0)}, name="FWP_TOKEN_INFORMATION", pack=False, align=None), offset=0), "tokenAccessInformation": SimTypePointer(SimStruct({"size": SimTypeInt(signed=False, label="UInt32"), "data": SimTypePointer(SimTypeChar(label="Byte"), offset=0)}, name="FWP_BYTE_BLOB", pack=False, align=None), offset=0), "unicodeString": SimTypePointer(SimTypeChar(label="Char"), offset=0), "byteArray6": SimTypePointer(SimStruct({"byteArray6": SimTypeFixedSizeArray(SimTypeChar(label="Byte"), 6)}, name="FWP_BYTE_ARRAY6", pack=False, align=None), offset=0), "bitmapArray64": SimTypePointer(SimStruct({"bitmapArray64": SimTypeFixedSizeArray(SimTypeChar(label="Byte"), 8)}, name="FWP_BITMAP_ARRAY64_", pack=False, align=None), offset=0)}, name="<anon>", label="None")}, name="FWP_VALUE0", pack=False, align=None), "valueHigh": SimStruct({"type": SimTypeInt(signed=False, label="FWP_DATA_TYPE"), "Anonymous": SimUnion({"uint8": SimTypeChar(label="Byte"), "uint16": SimTypeShort(signed=False, label="UInt16"), "uint32": SimTypeInt(signed=False, label="UInt32"), "uint64": SimTypePointer(SimTypeLongLong(signed=False, label="UInt64"), offset=0), "int8": SimTypeChar(label="SByte"), "int16": SimTypeShort(signed=True, label="Int16"), "int32": SimTypeInt(signed=True, label="Int32"), "int64": SimTypePointer(SimTypeLongLong(signed=True, label="Int64"), offset=0), "float32": SimTypeFloat(size=32), "double64": SimTypePointer(SimTypeFloat(size=64), offset=0), "byteArray16": SimTypePointer(SimStruct({"byteArray16": SimTypeFixedSizeArray(SimTypeChar(label="Byte"), 16)}, name="FWP_BYTE_ARRAY16", pack=False, align=None), offset=0), "byteBlob": SimTypePointer(SimStruct({"size": SimTypeInt(signed=False, label="UInt32"), "data": SimTypePointer(SimTypeChar(label="Byte"), offset=0)}, name="FWP_BYTE_BLOB", pack=False, align=None), offset=0), "sid": SimTypePointer(SimTypeBottom(label="SID"), offset=0), "sd": SimTypePointer(SimStruct({"size": SimTypeInt(signed=False, label="UInt32"), "data": SimTypePointer(SimTypeChar(label="Byte"), offset=0)}, name="FWP_BYTE_BLOB", pack=False, align=None), offset=0), "tokenInformation": SimTypePointer(SimStruct({"sidCount": SimTypeInt(signed=False, label="UInt32"), "sids": SimTypePointer(SimTypeBottom(label="SID_AND_ATTRIBUTES"), offset=0), "restrictedSidCount": SimTypeInt(signed=False, label="UInt32"), "restrictedSids": SimTypePointer(SimTypeBottom(label="SID_AND_ATTRIBUTES"), offset=0)}, name="FWP_TOKEN_INFORMATION", pack=False, align=None), offset=0), "tokenAccessInformation": SimTypePointer(SimStruct({"size": SimTypeInt(signed=False, label="UInt32"), "data": SimTypePointer(SimTypeChar(label="Byte"), offset=0)}, name="FWP_BYTE_BLOB", pack=False, align=None), offset=0), "unicodeString": SimTypePointer(SimTypeChar(label="Char"), offset=0), "byteArray6": SimTypePointer(SimStruct({"byteArray6": SimTypeFixedSizeArray(SimTypeChar(label="Byte"), 6)}, name="FWP_BYTE_ARRAY6", pack=False, align=None), offset=0), "bitmapArray64": SimTypePointer(SimStruct({"bitmapArray64": SimTypeFixedSizeArray(SimTypeChar(label="Byte"), 8)}, name="FWP_BITMAP_ARRAY64_", pack=False, align=None), offset=0)}, name="<anon>", label="None")}, name="FWP_VALUE0", pack=False, align=None)}, name="FWP_RANGE0", pack=False, align=None), offset=0)}, name="<anon>", label="None")}, name="FWP_CONDITION_VALUE0", pack=False, align=None)}, name="<anon>", label="None"), "Anonymous2": SimUnion({"remoteSubNet": SimStruct({"type": SimTypeInt(signed=False, label="FWP_DATA_TYPE"), "Anonymous": SimUnion({"uint8": SimTypeChar(label="Byte"), "uint16": SimTypeShort(signed=False, label="UInt16"), "uint32": SimTypeInt(signed=False, label="UInt32"), "uint64": SimTypePointer(SimTypeLongLong(signed=False, label="UInt64"), offset=0), "int8": SimTypeChar(label="SByte"), "int16": SimTypeShort(signed=True, label="Int16"), "int32": SimTypeInt(signed=True, label="Int32"), "int64": SimTypePointer(SimTypeLongLong(signed=True, label="Int64"), offset=0), "float32": SimTypeFloat(size=32), "double64": SimTypePointer(SimTypeFloat(size=64), offset=0), "byteArray16": SimTypePointer(SimStruct({"byteArray16": SimTypeFixedSizeArray(SimTypeChar(label="Byte"), 16)}, name="FWP_BYTE_ARRAY16", pack=False, align=None), offset=0), "byteBlob": SimTypePointer(SimStruct({"size": SimTypeInt(signed=False, label="UInt32"), "data": SimTypePointer(SimTypeChar(label="Byte"), offset=0)}, name="FWP_BYTE_BLOB", pack=False, align=None), offset=0), "sid": SimTypePointer(SimTypeBottom(label="SID"), offset=0), "sd": SimTypePointer(SimStruct({"size": SimTypeInt(signed=False, label="UInt32"), "data": SimTypePointer(SimTypeChar(label="Byte"), offset=0)}, name="FWP_BYTE_BLOB", pack=False, align=None), offset=0), "tokenInformation": SimTypePointer(SimStruct({"sidCount": SimTypeInt(signed=False, label="UInt32"), "sids": SimTypePointer(SimTypeBottom(label="SID_AND_ATTRIBUTES"), offset=0), "restrictedSidCount": SimTypeInt(signed=False, label="UInt32"), "restrictedSids": SimTypePointer(SimTypeBottom(label="SID_AND_ATTRIBUTES"), offset=0)}, name="FWP_TOKEN_INFORMATION", pack=False, align=None), offset=0), "tokenAccessInformation": SimTypePointer(SimStruct({"size": SimTypeInt(signed=False, label="UInt32"), "data": SimTypePointer(SimTypeChar(label="Byte"), offset=0)}, name="FWP_BYTE_BLOB", pack=False, align=None), offset=0), "unicodeString": SimTypePointer(SimTypeChar(label="Char"), offset=0), "byteArray6": SimTypePointer(SimStruct({"byteArray6": SimTypeFixedSizeArray(SimTypeChar(label="Byte"), 6)}, name="FWP_BYTE_ARRAY6", pack=False, align=None), offset=0), "bitmapArray64": SimTypePointer(SimStruct({"bitmapArray64": SimTypeFixedSizeArray(SimTypeChar(label="Byte"), 8)}, name="FWP_BITMAP_ARRAY64_", pack=False, align=None), offset=0), "v4AddrMask": SimTypePointer(SimStruct({"addr": SimTypeInt(signed=False, label="UInt32"), "mask": SimTypeInt(signed=False, label="UInt32")}, name="FWP_V4_ADDR_AND_MASK", pack=False, align=None), offset=0), "v6AddrMask": SimTypePointer(SimStruct({"addr": SimTypeFixedSizeArray(SimTypeChar(label="Byte"), 16), "prefixLength": SimTypeChar(label="Byte")}, name="FWP_V6_ADDR_AND_MASK", pack=False, align=None), offset=0), "rangeValue": SimTypePointer(SimStruct({"valueLow": SimStruct({"type": SimTypeInt(signed=False, label="FWP_DATA_TYPE"), "Anonymous": SimUnion({"uint8": SimTypeChar(label="Byte"), "uint16": SimTypeShort(signed=False, label="UInt16"), "uint32": SimTypeInt(signed=False, label="UInt32"), "uint64": SimTypePointer(SimTypeLongLong(signed=False, label="UInt64"), offset=0), "int8": SimTypeChar(label="SByte"), "int16": SimTypeShort(signed=True, label="Int16"), "int32": SimTypeInt(signed=True, label="Int32"), "int64": SimTypePointer(SimTypeLongLong(signed=True, label="Int64"), offset=0), "float32": SimTypeFloat(size=32), "double64": SimTypePointer(SimTypeFloat(size=64), offset=0), "byteArray16": SimTypePointer(SimStruct({"byteArray16": SimTypeFixedSizeArray(SimTypeChar(label="Byte"), 16)}, name="FWP_BYTE_ARRAY16", pack=False, align=None), offset=0), "byteBlob": SimTypePointer(SimStruct({"size": SimTypeInt(signed=False, label="UInt32"), "data": SimTypePointer(SimTypeChar(label="Byte"), offset=0)}, name="FWP_BYTE_BLOB", pack=False, align=None), offset=0), "sid": SimTypePointer(SimTypeBottom(label="SID"), offset=0), "sd": SimTypePointer(SimStruct({"size": SimTypeInt(signed=False, label="UInt32"), "data": SimTypePointer(SimTypeChar(label="Byte"), offset=0)}, name="FWP_BYTE_BLOB", pack=False, align=None), offset=0), "tokenInformation": SimTypePointer(SimStruct({"sidCount": SimTypeInt(signed=False, label="UInt32"), "sids": SimTypePointer(SimTypeBottom(label="SID_AND_ATTRIBUTES"), offset=0), "restrictedSidCount": SimTypeInt(signed=False, label="UInt32"), "restrictedSids": SimTypePointer(SimTypeBottom(label="SID_AND_ATTRIBUTES"), offset=0)}, name="FWP_TOKEN_INFORMATION", pack=False, align=None), offset=0), "tokenAccessInformation": SimTypePointer(SimStruct({"size": SimTypeInt(signed=False, label="UInt32"), "data": SimTypePointer(SimTypeChar(label="Byte"), offset=0)}, name="FWP_BYTE_BLOB", pack=False, align=None), offset=0), "unicodeString": SimTypePointer(SimTypeChar(label="Char"), offset=0), "byteArray6": SimTypePointer(SimStruct({"byteArray6": SimTypeFixedSizeArray(SimTypeChar(label="Byte"), 6)}, name="FWP_BYTE_ARRAY6", pack=False, align=None), offset=0), "bitmapArray64": SimTypePointer(SimStruct({"bitmapArray64": SimTypeFixedSizeArray(SimTypeChar(label="Byte"), 8)}, name="FWP_BITMAP_ARRAY64_", pack=False, align=None), offset=0)}, name="<anon>", label="None")}, name="FWP_VALUE0", pack=False, align=None), "valueHigh": SimStruct({"type": SimTypeInt(signed=False, label="FWP_DATA_TYPE"), "Anonymous": SimUnion({"uint8": SimTypeChar(label="Byte"), "uint16": SimTypeShort(signed=False, label="UInt16"), "uint32": SimTypeInt(signed=False, label="UInt32"), "uint64": SimTypePointer(SimTypeLongLong(signed=False, label="UInt64"), offset=0), "int8": SimTypeChar(label="SByte"), "int16": SimTypeShort(signed=True, label="Int16"), "int32": SimTypeInt(signed=True, label="Int32"), "int64": SimTypePointer(SimTypeLongLong(signed=True, label="Int64"), offset=0), "float32": SimTypeFloat(size=32), "double64": SimTypePointer(SimTypeFloat(size=64), offset=0), "byteArray16": SimTypePointer(SimStruct({"byteArray16": SimTypeFixedSizeArray(SimTypeChar(label="Byte"), 16)}, name="FWP_BYTE_ARRAY16", pack=False, align=None), offset=0), "byteBlob": SimTypePointer(SimStruct({"size": SimTypeInt(signed=False, label="UInt32"), "data": SimTypePointer(SimTypeChar(label="Byte"), offset=0)}, name="FWP_BYTE_BLOB", pack=False, align=None), offset=0), "sid": SimTypePointer(SimTypeBottom(label="SID"), offset=0), "sd": SimTypePointer(SimStruct({"size": SimTypeInt(signed=False, label="UInt32"), "data": SimTypePointer(SimTypeChar(label="Byte"), offset=0)}, name="FWP_BYTE_BLOB", pack=False, align=None), offset=0), "tokenInformation": SimTypePointer(SimStruct({"sidCount": SimTypeInt(signed=False, label="UInt32"), "sids": SimTypePointer(SimTypeBottom(label="SID_AND_ATTRIBUTES"), offset=0), "restrictedSidCount": SimTypeInt(signed=False, label="UInt32"), "restrictedSids": SimTypePointer(SimTypeBottom(label="SID_AND_ATTRIBUTES"), offset=0)}, name="FWP_TOKEN_INFORMATION", pack=False, align=None), offset=0), "tokenAccessInformation": SimTypePointer(SimStruct({"size": SimTypeInt(signed=False, label="UInt32"), "data": SimTypePointer(SimTypeChar(label="Byte"), offset=0)}, name="FWP_BYTE_BLOB", pack=False, align=None), offset=0), "unicodeString": SimTypePointer(SimTypeChar(label="Char"), offset=0), "byteArray6": SimTypePointer(SimStruct({"byteArray6": SimTypeFixedSizeArray(SimTypeChar(label="Byte"), 6)}, name="FWP_BYTE_ARRAY6", pack=False, align=None), offset=0), "bitmapArray64": SimTypePointer(SimStruct({"bitmapArray64": SimTypeFixedSizeArray(SimTypeChar(label="Byte"), 8)}, name="FWP_BITMAP_ARRAY64_", pack=False, align=None), offset=0)}, name="<anon>", label="None")}, name="FWP_VALUE0", pack=False, align=None)}, name="FWP_RANGE0", pack=False, align=None), offset=0)}, name="<anon>", label="None")}, name="FWP_CONDITION_VALUE0", pack=False, align=None)}, name="<anon>", label="None"), "qmFilterId": SimTypeLongLong(signed=False, label="UInt64")}, name="FWPM_NET_EVENT_IKEEXT_QM_FAILURE0", pack=False, align=None), offset=0), "ikeEmFailure": SimTypePointer(SimStruct({"failureErrorCode": SimTypeInt(signed=False, label="UInt32"), "failurePoint": SimTypeInt(signed=False, label="IPSEC_FAILURE_POINT"), "flags": SimTypeInt(signed=False, label="UInt32"), "emState": SimTypeInt(signed=False, label="IKEEXT_EM_SA_STATE"), "saRole": SimTypeInt(signed=False, label="IKEEXT_SA_ROLE"), "emAuthMethod": SimTypeInt(signed=False, label="IKEEXT_AUTHENTICATION_METHOD_TYPE"), "endCertHash": SimTypeFixedSizeArray(SimTypeChar(label="Byte"), 20), "mmId": SimTypeLongLong(signed=False, label="UInt64"), "qmFilterId": SimTypeLongLong(signed=False, label="UInt64"), "localPrincipalNameForAuth": SimTypePointer(SimTypeChar(label="Char"), offset=0), "remotePrincipalNameForAuth": SimTypePointer(SimTypeChar(label="Char"), offset=0), "numLocalPrincipalGroupSids": SimTypeInt(signed=False, label="UInt32"), "localPrincipalGroupSids": SimTypePointer(SimTypePointer(SimTypeChar(label="Char"), offset=0), offset=0), "numRemotePrincipalGroupSids": SimTypeInt(signed=False, label="UInt32"), "remotePrincipalGroupSids": SimTypePointer(SimTypePointer(SimTypeChar(label="Char"), offset=0), offset=0), "saTrafficType": SimTypeInt(signed=False, label="IPSEC_TRAFFIC_TYPE")}, name="FWPM_NET_EVENT_IKEEXT_EM_FAILURE1", pack=False, align=None), offset=0), "classifyDrop": SimTypePointer(SimStruct({"filterId": SimTypeLongLong(signed=False, label="UInt64"), "layerId": SimTypeShort(signed=False, label="UInt16"), "reauthReason": SimTypeInt(signed=False, label="UInt32"), "originalProfile": SimTypeInt(signed=False, label="UInt32"), "currentProfile": SimTypeInt(signed=False, label="UInt32"), "msFwpDirection": SimTypeInt(signed=False, label="UInt32"), "isLoopback": SimTypeInt(signed=True, label="Int32")}, name="FWPM_NET_EVENT_CLASSIFY_DROP1", pack=False, align=None), offset=0), "ipsecDrop": SimTypePointer(SimStruct({"failureStatus": SimTypeInt(signed=True, label="Int32"), "direction": SimTypeInt(signed=False, label="FWP_DIRECTION"), "spi": SimTypeInt(signed=False, label="UInt32"), "filterId": SimTypeLongLong(signed=False, label="UInt64"), "layerId": SimTypeShort(signed=False, label="UInt16")}, name="FWPM_NET_EVENT_IPSEC_KERNEL_DROP0", pack=False, align=None), offset=0), "idpDrop": SimTypePointer(SimStruct({"ipVersion": SimTypeInt(signed=False, label="FWP_IP_VERSION"), "Anonymous1": SimUnion({"publicHostV4Addr": SimTypeInt(signed=False, label="UInt32"), "publicHostV6Addr": SimTypeFixedSizeArray(SimTypeChar(label="Byte"), 16)}, name="<anon>", label="None"), "Anonymous2": SimUnion({"internalHostV4Addr": SimTypeInt(signed=False, label="UInt32"), "internalHostV6Addr": SimTypeFixedSizeArray(SimTypeChar(label="Byte"), 16)}, name="<anon>", label="None"), "failureStatus": SimTypeInt(signed=True, label="Int32"), "direction": SimTypeInt(signed=False, label="FWP_DIRECTION")}, name="FWPM_NET_EVENT_IPSEC_DOSP_DROP0", pack=False, align=None), offset=0)}, name="<anon>", label="None")}, name="FWPM_NET_EVENT1", pack=False, align=None), offset=0)], SimTypeBottom(label="Void"), arg_names=["context", "event"]), offset=0), SimTypePointer(SimTypeBottom(label="Void"), offset=0), SimTypePointer(SimTypePointer(SimTypeInt(signed=True, label="Int"), label="IntPtr", offset=0), offset=0)], SimTypeInt(signed=False, label="UInt32"), arg_names=["engineHandle", "subscription", "callback", "context", "eventsHandle"]),
        # 
        'FwpmNetEventUnsubscribe0': SimTypeFunction([SimTypePointer(SimTypeInt(signed=True, label="Int"), label="IntPtr", offset=0), SimTypePointer(SimTypeInt(signed=True, label="Int"), label="IntPtr", offset=0)], SimTypeInt(signed=False, label="UInt32"), arg_names=["engineHandle", "eventsHandle"]),
        # 
        'FwpmNetEventSubscriptionsGet0': SimTypeFunction([SimTypePointer(SimTypeInt(signed=True, label="Int"), label="IntPtr", offset=0), SimTypePointer(SimTypePointer(SimTypePointer(SimStruct({"enumTemplate": SimTypePointer(SimStruct({"startTime": SimStruct({"dwLowDateTime": SimTypeInt(signed=False, label="UInt32"), "dwHighDateTime": SimTypeInt(signed=False, label="UInt32")}, name="FILETIME", pack=False, align=None), "endTime": SimStruct({"dwLowDateTime": SimTypeInt(signed=False, label="UInt32"), "dwHighDateTime": SimTypeInt(signed=False, label="UInt32")}, name="FILETIME", pack=False, align=None), "numFilterConditions": SimTypeInt(signed=False, label="UInt32"), "filterCondition": SimTypePointer(SimStruct({"fieldKey": SimTypeBottom(label="Guid"), "matchType": SimTypeInt(signed=False, label="FWP_MATCH_TYPE"), "conditionValue": SimStruct({"type": SimTypeInt(signed=False, label="FWP_DATA_TYPE"), "Anonymous": SimUnion({"uint8": SimTypeChar(label="Byte"), "uint16": SimTypeShort(signed=False, label="UInt16"), "uint32": SimTypeInt(signed=False, label="UInt32"), "uint64": SimTypePointer(SimTypeLongLong(signed=False, label="UInt64"), offset=0), "int8": SimTypeChar(label="SByte"), "int16": SimTypeShort(signed=True, label="Int16"), "int32": SimTypeInt(signed=True, label="Int32"), "int64": SimTypePointer(SimTypeLongLong(signed=True, label="Int64"), offset=0), "float32": SimTypeFloat(size=32), "double64": SimTypePointer(SimTypeFloat(size=64), offset=0), "byteArray16": SimTypePointer(SimStruct({"byteArray16": SimTypeFixedSizeArray(SimTypeChar(label="Byte"), 16)}, name="FWP_BYTE_ARRAY16", pack=False, align=None), offset=0), "byteBlob": SimTypePointer(SimStruct({"size": SimTypeInt(signed=False, label="UInt32"), "data": SimTypePointer(SimTypeChar(label="Byte"), offset=0)}, name="FWP_BYTE_BLOB", pack=False, align=None), offset=0), "sid": SimTypePointer(SimTypeBottom(label="SID"), offset=0), "sd": SimTypePointer(SimStruct({"size": SimTypeInt(signed=False, label="UInt32"), "data": SimTypePointer(SimTypeChar(label="Byte"), offset=0)}, name="FWP_BYTE_BLOB", pack=False, align=None), offset=0), "tokenInformation": SimTypePointer(SimStruct({"sidCount": SimTypeInt(signed=False, label="UInt32"), "sids": SimTypePointer(SimTypeBottom(label="SID_AND_ATTRIBUTES"), offset=0), "restrictedSidCount": SimTypeInt(signed=False, label="UInt32"), "restrictedSids": SimTypePointer(SimTypeBottom(label="SID_AND_ATTRIBUTES"), offset=0)}, name="FWP_TOKEN_INFORMATION", pack=False, align=None), offset=0), "tokenAccessInformation": SimTypePointer(SimStruct({"size": SimTypeInt(signed=False, label="UInt32"), "data": SimTypePointer(SimTypeChar(label="Byte"), offset=0)}, name="FWP_BYTE_BLOB", pack=False, align=None), offset=0), "unicodeString": SimTypePointer(SimTypeChar(label="Char"), offset=0), "byteArray6": SimTypePointer(SimStruct({"byteArray6": SimTypeFixedSizeArray(SimTypeChar(label="Byte"), 6)}, name="FWP_BYTE_ARRAY6", pack=False, align=None), offset=0), "bitmapArray64": SimTypePointer(SimStruct({"bitmapArray64": SimTypeFixedSizeArray(SimTypeChar(label="Byte"), 8)}, name="FWP_BITMAP_ARRAY64_", pack=False, align=None), offset=0), "v4AddrMask": SimTypePointer(SimStruct({"addr": SimTypeInt(signed=False, label="UInt32"), "mask": SimTypeInt(signed=False, label="UInt32")}, name="FWP_V4_ADDR_AND_MASK", pack=False, align=None), offset=0), "v6AddrMask": SimTypePointer(SimStruct({"addr": SimTypeFixedSizeArray(SimTypeChar(label="Byte"), 16), "prefixLength": SimTypeChar(label="Byte")}, name="FWP_V6_ADDR_AND_MASK", pack=False, align=None), offset=0), "rangeValue": SimTypePointer(SimStruct({"valueLow": SimStruct({"type": SimTypeInt(signed=False, label="FWP_DATA_TYPE"), "Anonymous": SimUnion({"uint8": SimTypeChar(label="Byte"), "uint16": SimTypeShort(signed=False, label="UInt16"), "uint32": SimTypeInt(signed=False, label="UInt32"), "uint64": SimTypePointer(SimTypeLongLong(signed=False, label="UInt64"), offset=0), "int8": SimTypeChar(label="SByte"), "int16": SimTypeShort(signed=True, label="Int16"), "int32": SimTypeInt(signed=True, label="Int32"), "int64": SimTypePointer(SimTypeLongLong(signed=True, label="Int64"), offset=0), "float32": SimTypeFloat(size=32), "double64": SimTypePointer(SimTypeFloat(size=64), offset=0), "byteArray16": SimTypePointer(SimStruct({"byteArray16": SimTypeFixedSizeArray(SimTypeChar(label="Byte"), 16)}, name="FWP_BYTE_ARRAY16", pack=False, align=None), offset=0), "byteBlob": SimTypePointer(SimStruct({"size": SimTypeInt(signed=False, label="UInt32"), "data": SimTypePointer(SimTypeChar(label="Byte"), offset=0)}, name="FWP_BYTE_BLOB", pack=False, align=None), offset=0), "sid": SimTypePointer(SimTypeBottom(label="SID"), offset=0), "sd": SimTypePointer(SimStruct({"size": SimTypeInt(signed=False, label="UInt32"), "data": SimTypePointer(SimTypeChar(label="Byte"), offset=0)}, name="FWP_BYTE_BLOB", pack=False, align=None), offset=0), "tokenInformation": SimTypePointer(SimStruct({"sidCount": SimTypeInt(signed=False, label="UInt32"), "sids": SimTypePointer(SimTypeBottom(label="SID_AND_ATTRIBUTES"), offset=0), "restrictedSidCount": SimTypeInt(signed=False, label="UInt32"), "restrictedSids": SimTypePointer(SimTypeBottom(label="SID_AND_ATTRIBUTES"), offset=0)}, name="FWP_TOKEN_INFORMATION", pack=False, align=None), offset=0), "tokenAccessInformation": SimTypePointer(SimStruct({"size": SimTypeInt(signed=False, label="UInt32"), "data": SimTypePointer(SimTypeChar(label="Byte"), offset=0)}, name="FWP_BYTE_BLOB", pack=False, align=None), offset=0), "unicodeString": SimTypePointer(SimTypeChar(label="Char"), offset=0), "byteArray6": SimTypePointer(SimStruct({"byteArray6": SimTypeFixedSizeArray(SimTypeChar(label="Byte"), 6)}, name="FWP_BYTE_ARRAY6", pack=False, align=None), offset=0), "bitmapArray64": SimTypePointer(SimStruct({"bitmapArray64": SimTypeFixedSizeArray(SimTypeChar(label="Byte"), 8)}, name="FWP_BITMAP_ARRAY64_", pack=False, align=None), offset=0)}, name="<anon>", label="None")}, name="FWP_VALUE0", pack=False, align=None), "valueHigh": SimStruct({"type": SimTypeInt(signed=False, label="FWP_DATA_TYPE"), "Anonymous": SimUnion({"uint8": SimTypeChar(label="Byte"), "uint16": SimTypeShort(signed=False, label="UInt16"), "uint32": SimTypeInt(signed=False, label="UInt32"), "uint64": SimTypePointer(SimTypeLongLong(signed=False, label="UInt64"), offset=0), "int8": SimTypeChar(label="SByte"), "int16": SimTypeShort(signed=True, label="Int16"), "int32": SimTypeInt(signed=True, label="Int32"), "int64": SimTypePointer(SimTypeLongLong(signed=True, label="Int64"), offset=0), "float32": SimTypeFloat(size=32), "double64": SimTypePointer(SimTypeFloat(size=64), offset=0), "byteArray16": SimTypePointer(SimStruct({"byteArray16": SimTypeFixedSizeArray(SimTypeChar(label="Byte"), 16)}, name="FWP_BYTE_ARRAY16", pack=False, align=None), offset=0), "byteBlob": SimTypePointer(SimStruct({"size": SimTypeInt(signed=False, label="UInt32"), "data": SimTypePointer(SimTypeChar(label="Byte"), offset=0)}, name="FWP_BYTE_BLOB", pack=False, align=None), offset=0), "sid": SimTypePointer(SimTypeBottom(label="SID"), offset=0), "sd": SimTypePointer(SimStruct({"size": SimTypeInt(signed=False, label="UInt32"), "data": SimTypePointer(SimTypeChar(label="Byte"), offset=0)}, name="FWP_BYTE_BLOB", pack=False, align=None), offset=0), "tokenInformation": SimTypePointer(SimStruct({"sidCount": SimTypeInt(signed=False, label="UInt32"), "sids": SimTypePointer(SimTypeBottom(label="SID_AND_ATTRIBUTES"), offset=0), "restrictedSidCount": SimTypeInt(signed=False, label="UInt32"), "restrictedSids": SimTypePointer(SimTypeBottom(label="SID_AND_ATTRIBUTES"), offset=0)}, name="FWP_TOKEN_INFORMATION", pack=False, align=None), offset=0), "tokenAccessInformation": SimTypePointer(SimStruct({"size": SimTypeInt(signed=False, label="UInt32"), "data": SimTypePointer(SimTypeChar(label="Byte"), offset=0)}, name="FWP_BYTE_BLOB", pack=False, align=None), offset=0), "unicodeString": SimTypePointer(SimTypeChar(label="Char"), offset=0), "byteArray6": SimTypePointer(SimStruct({"byteArray6": SimTypeFixedSizeArray(SimTypeChar(label="Byte"), 6)}, name="FWP_BYTE_ARRAY6", pack=False, align=None), offset=0), "bitmapArray64": SimTypePointer(SimStruct({"bitmapArray64": SimTypeFixedSizeArray(SimTypeChar(label="Byte"), 8)}, name="FWP_BITMAP_ARRAY64_", pack=False, align=None), offset=0)}, name="<anon>", label="None")}, name="FWP_VALUE0", pack=False, align=None)}, name="FWP_RANGE0", pack=False, align=None), offset=0)}, name="<anon>", label="None")}, name="FWP_CONDITION_VALUE0", pack=False, align=None)}, name="FWPM_FILTER_CONDITION0", pack=False, align=None), offset=0)}, name="FWPM_NET_EVENT_ENUM_TEMPLATE0", pack=False, align=None), offset=0), "flags": SimTypeInt(signed=False, label="UInt32"), "sessionKey": SimTypeBottom(label="Guid")}, name="FWPM_NET_EVENT_SUBSCRIPTION0", pack=False, align=None), offset=0), offset=0), offset=0), SimTypePointer(SimTypeInt(signed=False, label="UInt32"), offset=0)], SimTypeInt(signed=False, label="UInt32"), arg_names=["engineHandle", "entries", "numEntries"]),
        # 
        'FwpmNetEventSubscribe1': SimTypeFunction([SimTypePointer(SimTypeInt(signed=True, label="Int"), label="IntPtr", offset=0), SimTypePointer(SimStruct({"enumTemplate": SimTypePointer(SimStruct({"startTime": SimStruct({"dwLowDateTime": SimTypeInt(signed=False, label="UInt32"), "dwHighDateTime": SimTypeInt(signed=False, label="UInt32")}, name="FILETIME", pack=False, align=None), "endTime": SimStruct({"dwLowDateTime": SimTypeInt(signed=False, label="UInt32"), "dwHighDateTime": SimTypeInt(signed=False, label="UInt32")}, name="FILETIME", pack=False, align=None), "numFilterConditions": SimTypeInt(signed=False, label="UInt32"), "filterCondition": SimTypePointer(SimStruct({"fieldKey": SimTypeBottom(label="Guid"), "matchType": SimTypeInt(signed=False, label="FWP_MATCH_TYPE"), "conditionValue": SimStruct({"type": SimTypeInt(signed=False, label="FWP_DATA_TYPE"), "Anonymous": SimUnion({"uint8": SimTypeChar(label="Byte"), "uint16": SimTypeShort(signed=False, label="UInt16"), "uint32": SimTypeInt(signed=False, label="UInt32"), "uint64": SimTypePointer(SimTypeLongLong(signed=False, label="UInt64"), offset=0), "int8": SimTypeChar(label="SByte"), "int16": SimTypeShort(signed=True, label="Int16"), "int32": SimTypeInt(signed=True, label="Int32"), "int64": SimTypePointer(SimTypeLongLong(signed=True, label="Int64"), offset=0), "float32": SimTypeFloat(size=32), "double64": SimTypePointer(SimTypeFloat(size=64), offset=0), "byteArray16": SimTypePointer(SimStruct({"byteArray16": SimTypeFixedSizeArray(SimTypeChar(label="Byte"), 16)}, name="FWP_BYTE_ARRAY16", pack=False, align=None), offset=0), "byteBlob": SimTypePointer(SimStruct({"size": SimTypeInt(signed=False, label="UInt32"), "data": SimTypePointer(SimTypeChar(label="Byte"), offset=0)}, name="FWP_BYTE_BLOB", pack=False, align=None), offset=0), "sid": SimTypePointer(SimTypeBottom(label="SID"), offset=0), "sd": SimTypePointer(SimStruct({"size": SimTypeInt(signed=False, label="UInt32"), "data": SimTypePointer(SimTypeChar(label="Byte"), offset=0)}, name="FWP_BYTE_BLOB", pack=False, align=None), offset=0), "tokenInformation": SimTypePointer(SimStruct({"sidCount": SimTypeInt(signed=False, label="UInt32"), "sids": SimTypePointer(SimTypeBottom(label="SID_AND_ATTRIBUTES"), offset=0), "restrictedSidCount": SimTypeInt(signed=False, label="UInt32"), "restrictedSids": SimTypePointer(SimTypeBottom(label="SID_AND_ATTRIBUTES"), offset=0)}, name="FWP_TOKEN_INFORMATION", pack=False, align=None), offset=0), "tokenAccessInformation": SimTypePointer(SimStruct({"size": SimTypeInt(signed=False, label="UInt32"), "data": SimTypePointer(SimTypeChar(label="Byte"), offset=0)}, name="FWP_BYTE_BLOB", pack=False, align=None), offset=0), "unicodeString": SimTypePointer(SimTypeChar(label="Char"), offset=0), "byteArray6": SimTypePointer(SimStruct({"byteArray6": SimTypeFixedSizeArray(SimTypeChar(label="Byte"), 6)}, name="FWP_BYTE_ARRAY6", pack=False, align=None), offset=0), "bitmapArray64": SimTypePointer(SimStruct({"bitmapArray64": SimTypeFixedSizeArray(SimTypeChar(label="Byte"), 8)}, name="FWP_BITMAP_ARRAY64_", pack=False, align=None), offset=0), "v4AddrMask": SimTypePointer(SimStruct({"addr": SimTypeInt(signed=False, label="UInt32"), "mask": SimTypeInt(signed=False, label="UInt32")}, name="FWP_V4_ADDR_AND_MASK", pack=False, align=None), offset=0), "v6AddrMask": SimTypePointer(SimStruct({"addr": SimTypeFixedSizeArray(SimTypeChar(label="Byte"), 16), "prefixLength": SimTypeChar(label="Byte")}, name="FWP_V6_ADDR_AND_MASK", pack=False, align=None), offset=0), "rangeValue": SimTypePointer(SimStruct({"valueLow": SimStruct({"type": SimTypeInt(signed=False, label="FWP_DATA_TYPE"), "Anonymous": SimUnion({"uint8": SimTypeChar(label="Byte"), "uint16": SimTypeShort(signed=False, label="UInt16"), "uint32": SimTypeInt(signed=False, label="UInt32"), "uint64": SimTypePointer(SimTypeLongLong(signed=False, label="UInt64"), offset=0), "int8": SimTypeChar(label="SByte"), "int16": SimTypeShort(signed=True, label="Int16"), "int32": SimTypeInt(signed=True, label="Int32"), "int64": SimTypePointer(SimTypeLongLong(signed=True, label="Int64"), offset=0), "float32": SimTypeFloat(size=32), "double64": SimTypePointer(SimTypeFloat(size=64), offset=0), "byteArray16": SimTypePointer(SimStruct({"byteArray16": SimTypeFixedSizeArray(SimTypeChar(label="Byte"), 16)}, name="FWP_BYTE_ARRAY16", pack=False, align=None), offset=0), "byteBlob": SimTypePointer(SimStruct({"size": SimTypeInt(signed=False, label="UInt32"), "data": SimTypePointer(SimTypeChar(label="Byte"), offset=0)}, name="FWP_BYTE_BLOB", pack=False, align=None), offset=0), "sid": SimTypePointer(SimTypeBottom(label="SID"), offset=0), "sd": SimTypePointer(SimStruct({"size": SimTypeInt(signed=False, label="UInt32"), "data": SimTypePointer(SimTypeChar(label="Byte"), offset=0)}, name="FWP_BYTE_BLOB", pack=False, align=None), offset=0), "tokenInformation": SimTypePointer(SimStruct({"sidCount": SimTypeInt(signed=False, label="UInt32"), "sids": SimTypePointer(SimTypeBottom(label="SID_AND_ATTRIBUTES"), offset=0), "restrictedSidCount": SimTypeInt(signed=False, label="UInt32"), "restrictedSids": SimTypePointer(SimTypeBottom(label="SID_AND_ATTRIBUTES"), offset=0)}, name="FWP_TOKEN_INFORMATION", pack=False, align=None), offset=0), "tokenAccessInformation": SimTypePointer(SimStruct({"size": SimTypeInt(signed=False, label="UInt32"), "data": SimTypePointer(SimTypeChar(label="Byte"), offset=0)}, name="FWP_BYTE_BLOB", pack=False, align=None), offset=0), "unicodeString": SimTypePointer(SimTypeChar(label="Char"), offset=0), "byteArray6": SimTypePointer(SimStruct({"byteArray6": SimTypeFixedSizeArray(SimTypeChar(label="Byte"), 6)}, name="FWP_BYTE_ARRAY6", pack=False, align=None), offset=0), "bitmapArray64": SimTypePointer(SimStruct({"bitmapArray64": SimTypeFixedSizeArray(SimTypeChar(label="Byte"), 8)}, name="FWP_BITMAP_ARRAY64_", pack=False, align=None), offset=0)}, name="<anon>", label="None")}, name="FWP_VALUE0", pack=False, align=None), "valueHigh": SimStruct({"type": SimTypeInt(signed=False, label="FWP_DATA_TYPE"), "Anonymous": SimUnion({"uint8": SimTypeChar(label="Byte"), "uint16": SimTypeShort(signed=False, label="UInt16"), "uint32": SimTypeInt(signed=False, label="UInt32"), "uint64": SimTypePointer(SimTypeLongLong(signed=False, label="UInt64"), offset=0), "int8": SimTypeChar(label="SByte"), "int16": SimTypeShort(signed=True, label="Int16"), "int32": SimTypeInt(signed=True, label="Int32"), "int64": SimTypePointer(SimTypeLongLong(signed=True, label="Int64"), offset=0), "float32": SimTypeFloat(size=32), "double64": SimTypePointer(SimTypeFloat(size=64), offset=0), "byteArray16": SimTypePointer(SimStruct({"byteArray16": SimTypeFixedSizeArray(SimTypeChar(label="Byte"), 16)}, name="FWP_BYTE_ARRAY16", pack=False, align=None), offset=0), "byteBlob": SimTypePointer(SimStruct({"size": SimTypeInt(signed=False, label="UInt32"), "data": SimTypePointer(SimTypeChar(label="Byte"), offset=0)}, name="FWP_BYTE_BLOB", pack=False, align=None), offset=0), "sid": SimTypePointer(SimTypeBottom(label="SID"), offset=0), "sd": SimTypePointer(SimStruct({"size": SimTypeInt(signed=False, label="UInt32"), "data": SimTypePointer(SimTypeChar(label="Byte"), offset=0)}, name="FWP_BYTE_BLOB", pack=False, align=None), offset=0), "tokenInformation": SimTypePointer(SimStruct({"sidCount": SimTypeInt(signed=False, label="UInt32"), "sids": SimTypePointer(SimTypeBottom(label="SID_AND_ATTRIBUTES"), offset=0), "restrictedSidCount": SimTypeInt(signed=False, label="UInt32"), "restrictedSids": SimTypePointer(SimTypeBottom(label="SID_AND_ATTRIBUTES"), offset=0)}, name="FWP_TOKEN_INFORMATION", pack=False, align=None), offset=0), "tokenAccessInformation": SimTypePointer(SimStruct({"size": SimTypeInt(signed=False, label="UInt32"), "data": SimTypePointer(SimTypeChar(label="Byte"), offset=0)}, name="FWP_BYTE_BLOB", pack=False, align=None), offset=0), "unicodeString": SimTypePointer(SimTypeChar(label="Char"), offset=0), "byteArray6": SimTypePointer(SimStruct({"byteArray6": SimTypeFixedSizeArray(SimTypeChar(label="Byte"), 6)}, name="FWP_BYTE_ARRAY6", pack=False, align=None), offset=0), "bitmapArray64": SimTypePointer(SimStruct({"bitmapArray64": SimTypeFixedSizeArray(SimTypeChar(label="Byte"), 8)}, name="FWP_BITMAP_ARRAY64_", pack=False, align=None), offset=0)}, name="<anon>", label="None")}, name="FWP_VALUE0", pack=False, align=None)}, name="FWP_RANGE0", pack=False, align=None), offset=0)}, name="<anon>", label="None")}, name="FWP_CONDITION_VALUE0", pack=False, align=None)}, name="FWPM_FILTER_CONDITION0", pack=False, align=None), offset=0)}, name="FWPM_NET_EVENT_ENUM_TEMPLATE0", pack=False, align=None), offset=0), "flags": SimTypeInt(signed=False, label="UInt32"), "sessionKey": SimTypeBottom(label="Guid")}, name="FWPM_NET_EVENT_SUBSCRIPTION0", pack=False, align=None), offset=0), SimTypePointer(SimTypeFunction([SimTypePointer(SimTypeBottom(label="Void"), offset=0), SimTypePointer(SimStruct({"header": SimStruct({"timeStamp": SimStruct({"dwLowDateTime": SimTypeInt(signed=False, label="UInt32"), "dwHighDateTime": SimTypeInt(signed=False, label="UInt32")}, name="FILETIME", pack=False, align=None), "flags": SimTypeInt(signed=False, label="UInt32"), "ipVersion": SimTypeInt(signed=False, label="FWP_IP_VERSION"), "ipProtocol": SimTypeChar(label="Byte"), "Anonymous1": SimUnion({"localAddrV4": SimTypeInt(signed=False, label="UInt32"), "localAddrV6": SimStruct({"byteArray16": SimTypeFixedSizeArray(SimTypeChar(label="Byte"), 16)}, name="FWP_BYTE_ARRAY16", pack=False, align=None)}, name="<anon>", label="None"), "Anonymous2": SimUnion({"remoteAddrV4": SimTypeInt(signed=False, label="UInt32"), "remoteAddrV6": SimStruct({"byteArray16": SimTypeFixedSizeArray(SimTypeChar(label="Byte"), 16)}, name="FWP_BYTE_ARRAY16", pack=False, align=None)}, name="<anon>", label="None"), "localPort": SimTypeShort(signed=False, label="UInt16"), "remotePort": SimTypeShort(signed=False, label="UInt16"), "scopeId": SimTypeInt(signed=False, label="UInt32"), "appId": SimStruct({"size": SimTypeInt(signed=False, label="UInt32"), "data": SimTypePointer(SimTypeChar(label="Byte"), offset=0)}, name="FWP_BYTE_BLOB", pack=False, align=None), "userId": SimTypePointer(SimTypeBottom(label="SID"), offset=0), "addressFamily": SimTypeInt(signed=False, label="FWP_AF"), "packageSid": SimTypePointer(SimTypeBottom(label="SID"), offset=0)}, name="FWPM_NET_EVENT_HEADER2", pack=False, align=None), "type": SimTypeInt(signed=False, label="FWPM_NET_EVENT_TYPE"), "Anonymous": SimUnion({"ikeMmFailure": SimTypePointer(SimStruct({"failureErrorCode": SimTypeInt(signed=False, label="UInt32"), "failurePoint": SimTypeInt(signed=False, label="IPSEC_FAILURE_POINT"), "flags": SimTypeInt(signed=False, label="UInt32"), "keyingModuleType": SimTypeInt(signed=False, label="IKEEXT_KEY_MODULE_TYPE"), "mmState": SimTypeInt(signed=False, label="IKEEXT_MM_SA_STATE"), "saRole": SimTypeInt(signed=False, label="IKEEXT_SA_ROLE"), "mmAuthMethod": SimTypeInt(signed=False, label="IKEEXT_AUTHENTICATION_METHOD_TYPE"), "endCertHash": SimTypeFixedSizeArray(SimTypeChar(label="Byte"), 20), "mmId": SimTypeLongLong(signed=False, label="UInt64"), "mmFilterId": SimTypeLongLong(signed=False, label="UInt64"), "localPrincipalNameForAuth": SimTypePointer(SimTypeChar(label="Char"), offset=0), "remotePrincipalNameForAuth": SimTypePointer(SimTypeChar(label="Char"), offset=0), "numLocalPrincipalGroupSids": SimTypeInt(signed=False, label="UInt32"), "localPrincipalGroupSids": SimTypePointer(SimTypePointer(SimTypeChar(label="Char"), offset=0), offset=0), "numRemotePrincipalGroupSids": SimTypeInt(signed=False, label="UInt32"), "remotePrincipalGroupSids": SimTypePointer(SimTypePointer(SimTypeChar(label="Char"), offset=0), offset=0)}, name="FWPM_NET_EVENT_IKEEXT_MM_FAILURE1", pack=False, align=None), offset=0), "ikeQmFailure": SimTypePointer(SimStruct({"failureErrorCode": SimTypeInt(signed=False, label="UInt32"), "failurePoint": SimTypeInt(signed=False, label="IPSEC_FAILURE_POINT"), "keyingModuleType": SimTypeInt(signed=False, label="IKEEXT_KEY_MODULE_TYPE"), "qmState": SimTypeInt(signed=False, label="IKEEXT_QM_SA_STATE"), "saRole": SimTypeInt(signed=False, label="IKEEXT_SA_ROLE"), "saTrafficType": SimTypeInt(signed=False, label="IPSEC_TRAFFIC_TYPE"), "Anonymous1": SimUnion({"localSubNet": SimStruct({"type": SimTypeInt(signed=False, label="FWP_DATA_TYPE"), "Anonymous": SimUnion({"uint8": SimTypeChar(label="Byte"), "uint16": SimTypeShort(signed=False, label="UInt16"), "uint32": SimTypeInt(signed=False, label="UInt32"), "uint64": SimTypePointer(SimTypeLongLong(signed=False, label="UInt64"), offset=0), "int8": SimTypeChar(label="SByte"), "int16": SimTypeShort(signed=True, label="Int16"), "int32": SimTypeInt(signed=True, label="Int32"), "int64": SimTypePointer(SimTypeLongLong(signed=True, label="Int64"), offset=0), "float32": SimTypeFloat(size=32), "double64": SimTypePointer(SimTypeFloat(size=64), offset=0), "byteArray16": SimTypePointer(SimStruct({"byteArray16": SimTypeFixedSizeArray(SimTypeChar(label="Byte"), 16)}, name="FWP_BYTE_ARRAY16", pack=False, align=None), offset=0), "byteBlob": SimTypePointer(SimStruct({"size": SimTypeInt(signed=False, label="UInt32"), "data": SimTypePointer(SimTypeChar(label="Byte"), offset=0)}, name="FWP_BYTE_BLOB", pack=False, align=None), offset=0), "sid": SimTypePointer(SimTypeBottom(label="SID"), offset=0), "sd": SimTypePointer(SimStruct({"size": SimTypeInt(signed=False, label="UInt32"), "data": SimTypePointer(SimTypeChar(label="Byte"), offset=0)}, name="FWP_BYTE_BLOB", pack=False, align=None), offset=0), "tokenInformation": SimTypePointer(SimStruct({"sidCount": SimTypeInt(signed=False, label="UInt32"), "sids": SimTypePointer(SimTypeBottom(label="SID_AND_ATTRIBUTES"), offset=0), "restrictedSidCount": SimTypeInt(signed=False, label="UInt32"), "restrictedSids": SimTypePointer(SimTypeBottom(label="SID_AND_ATTRIBUTES"), offset=0)}, name="FWP_TOKEN_INFORMATION", pack=False, align=None), offset=0), "tokenAccessInformation": SimTypePointer(SimStruct({"size": SimTypeInt(signed=False, label="UInt32"), "data": SimTypePointer(SimTypeChar(label="Byte"), offset=0)}, name="FWP_BYTE_BLOB", pack=False, align=None), offset=0), "unicodeString": SimTypePointer(SimTypeChar(label="Char"), offset=0), "byteArray6": SimTypePointer(SimStruct({"byteArray6": SimTypeFixedSizeArray(SimTypeChar(label="Byte"), 6)}, name="FWP_BYTE_ARRAY6", pack=False, align=None), offset=0), "bitmapArray64": SimTypePointer(SimStruct({"bitmapArray64": SimTypeFixedSizeArray(SimTypeChar(label="Byte"), 8)}, name="FWP_BITMAP_ARRAY64_", pack=False, align=None), offset=0), "v4AddrMask": SimTypePointer(SimStruct({"addr": SimTypeInt(signed=False, label="UInt32"), "mask": SimTypeInt(signed=False, label="UInt32")}, name="FWP_V4_ADDR_AND_MASK", pack=False, align=None), offset=0), "v6AddrMask": SimTypePointer(SimStruct({"addr": SimTypeFixedSizeArray(SimTypeChar(label="Byte"), 16), "prefixLength": SimTypeChar(label="Byte")}, name="FWP_V6_ADDR_AND_MASK", pack=False, align=None), offset=0), "rangeValue": SimTypePointer(SimStruct({"valueLow": SimStruct({"type": SimTypeInt(signed=False, label="FWP_DATA_TYPE"), "Anonymous": SimUnion({"uint8": SimTypeChar(label="Byte"), "uint16": SimTypeShort(signed=False, label="UInt16"), "uint32": SimTypeInt(signed=False, label="UInt32"), "uint64": SimTypePointer(SimTypeLongLong(signed=False, label="UInt64"), offset=0), "int8": SimTypeChar(label="SByte"), "int16": SimTypeShort(signed=True, label="Int16"), "int32": SimTypeInt(signed=True, label="Int32"), "int64": SimTypePointer(SimTypeLongLong(signed=True, label="Int64"), offset=0), "float32": SimTypeFloat(size=32), "double64": SimTypePointer(SimTypeFloat(size=64), offset=0), "byteArray16": SimTypePointer(SimStruct({"byteArray16": SimTypeFixedSizeArray(SimTypeChar(label="Byte"), 16)}, name="FWP_BYTE_ARRAY16", pack=False, align=None), offset=0), "byteBlob": SimTypePointer(SimStruct({"size": SimTypeInt(signed=False, label="UInt32"), "data": SimTypePointer(SimTypeChar(label="Byte"), offset=0)}, name="FWP_BYTE_BLOB", pack=False, align=None), offset=0), "sid": SimTypePointer(SimTypeBottom(label="SID"), offset=0), "sd": SimTypePointer(SimStruct({"size": SimTypeInt(signed=False, label="UInt32"), "data": SimTypePointer(SimTypeChar(label="Byte"), offset=0)}, name="FWP_BYTE_BLOB", pack=False, align=None), offset=0), "tokenInformation": SimTypePointer(SimStruct({"sidCount": SimTypeInt(signed=False, label="UInt32"), "sids": SimTypePointer(SimTypeBottom(label="SID_AND_ATTRIBUTES"), offset=0), "restrictedSidCount": SimTypeInt(signed=False, label="UInt32"), "restrictedSids": SimTypePointer(SimTypeBottom(label="SID_AND_ATTRIBUTES"), offset=0)}, name="FWP_TOKEN_INFORMATION", pack=False, align=None), offset=0), "tokenAccessInformation": SimTypePointer(SimStruct({"size": SimTypeInt(signed=False, label="UInt32"), "data": SimTypePointer(SimTypeChar(label="Byte"), offset=0)}, name="FWP_BYTE_BLOB", pack=False, align=None), offset=0), "unicodeString": SimTypePointer(SimTypeChar(label="Char"), offset=0), "byteArray6": SimTypePointer(SimStruct({"byteArray6": SimTypeFixedSizeArray(SimTypeChar(label="Byte"), 6)}, name="FWP_BYTE_ARRAY6", pack=False, align=None), offset=0), "bitmapArray64": SimTypePointer(SimStruct({"bitmapArray64": SimTypeFixedSizeArray(SimTypeChar(label="Byte"), 8)}, name="FWP_BITMAP_ARRAY64_", pack=False, align=None), offset=0)}, name="<anon>", label="None")}, name="FWP_VALUE0", pack=False, align=None), "valueHigh": SimStruct({"type": SimTypeInt(signed=False, label="FWP_DATA_TYPE"), "Anonymous": SimUnion({"uint8": SimTypeChar(label="Byte"), "uint16": SimTypeShort(signed=False, label="UInt16"), "uint32": SimTypeInt(signed=False, label="UInt32"), "uint64": SimTypePointer(SimTypeLongLong(signed=False, label="UInt64"), offset=0), "int8": SimTypeChar(label="SByte"), "int16": SimTypeShort(signed=True, label="Int16"), "int32": SimTypeInt(signed=True, label="Int32"), "int64": SimTypePointer(SimTypeLongLong(signed=True, label="Int64"), offset=0), "float32": SimTypeFloat(size=32), "double64": SimTypePointer(SimTypeFloat(size=64), offset=0), "byteArray16": SimTypePointer(SimStruct({"byteArray16": SimTypeFixedSizeArray(SimTypeChar(label="Byte"), 16)}, name="FWP_BYTE_ARRAY16", pack=False, align=None), offset=0), "byteBlob": SimTypePointer(SimStruct({"size": SimTypeInt(signed=False, label="UInt32"), "data": SimTypePointer(SimTypeChar(label="Byte"), offset=0)}, name="FWP_BYTE_BLOB", pack=False, align=None), offset=0), "sid": SimTypePointer(SimTypeBottom(label="SID"), offset=0), "sd": SimTypePointer(SimStruct({"size": SimTypeInt(signed=False, label="UInt32"), "data": SimTypePointer(SimTypeChar(label="Byte"), offset=0)}, name="FWP_BYTE_BLOB", pack=False, align=None), offset=0), "tokenInformation": SimTypePointer(SimStruct({"sidCount": SimTypeInt(signed=False, label="UInt32"), "sids": SimTypePointer(SimTypeBottom(label="SID_AND_ATTRIBUTES"), offset=0), "restrictedSidCount": SimTypeInt(signed=False, label="UInt32"), "restrictedSids": SimTypePointer(SimTypeBottom(label="SID_AND_ATTRIBUTES"), offset=0)}, name="FWP_TOKEN_INFORMATION", pack=False, align=None), offset=0), "tokenAccessInformation": SimTypePointer(SimStruct({"size": SimTypeInt(signed=False, label="UInt32"), "data": SimTypePointer(SimTypeChar(label="Byte"), offset=0)}, name="FWP_BYTE_BLOB", pack=False, align=None), offset=0), "unicodeString": SimTypePointer(SimTypeChar(label="Char"), offset=0), "byteArray6": SimTypePointer(SimStruct({"byteArray6": SimTypeFixedSizeArray(SimTypeChar(label="Byte"), 6)}, name="FWP_BYTE_ARRAY6", pack=False, align=None), offset=0), "bitmapArray64": SimTypePointer(SimStruct({"bitmapArray64": SimTypeFixedSizeArray(SimTypeChar(label="Byte"), 8)}, name="FWP_BITMAP_ARRAY64_", pack=False, align=None), offset=0)}, name="<anon>", label="None")}, name="FWP_VALUE0", pack=False, align=None)}, name="FWP_RANGE0", pack=False, align=None), offset=0)}, name="<anon>", label="None")}, name="FWP_CONDITION_VALUE0", pack=False, align=None)}, name="<anon>", label="None"), "Anonymous2": SimUnion({"remoteSubNet": SimStruct({"type": SimTypeInt(signed=False, label="FWP_DATA_TYPE"), "Anonymous": SimUnion({"uint8": SimTypeChar(label="Byte"), "uint16": SimTypeShort(signed=False, label="UInt16"), "uint32": SimTypeInt(signed=False, label="UInt32"), "uint64": SimTypePointer(SimTypeLongLong(signed=False, label="UInt64"), offset=0), "int8": SimTypeChar(label="SByte"), "int16": SimTypeShort(signed=True, label="Int16"), "int32": SimTypeInt(signed=True, label="Int32"), "int64": SimTypePointer(SimTypeLongLong(signed=True, label="Int64"), offset=0), "float32": SimTypeFloat(size=32), "double64": SimTypePointer(SimTypeFloat(size=64), offset=0), "byteArray16": SimTypePointer(SimStruct({"byteArray16": SimTypeFixedSizeArray(SimTypeChar(label="Byte"), 16)}, name="FWP_BYTE_ARRAY16", pack=False, align=None), offset=0), "byteBlob": SimTypePointer(SimStruct({"size": SimTypeInt(signed=False, label="UInt32"), "data": SimTypePointer(SimTypeChar(label="Byte"), offset=0)}, name="FWP_BYTE_BLOB", pack=False, align=None), offset=0), "sid": SimTypePointer(SimTypeBottom(label="SID"), offset=0), "sd": SimTypePointer(SimStruct({"size": SimTypeInt(signed=False, label="UInt32"), "data": SimTypePointer(SimTypeChar(label="Byte"), offset=0)}, name="FWP_BYTE_BLOB", pack=False, align=None), offset=0), "tokenInformation": SimTypePointer(SimStruct({"sidCount": SimTypeInt(signed=False, label="UInt32"), "sids": SimTypePointer(SimTypeBottom(label="SID_AND_ATTRIBUTES"), offset=0), "restrictedSidCount": SimTypeInt(signed=False, label="UInt32"), "restrictedSids": SimTypePointer(SimTypeBottom(label="SID_AND_ATTRIBUTES"), offset=0)}, name="FWP_TOKEN_INFORMATION", pack=False, align=None), offset=0), "tokenAccessInformation": SimTypePointer(SimStruct({"size": SimTypeInt(signed=False, label="UInt32"), "data": SimTypePointer(SimTypeChar(label="Byte"), offset=0)}, name="FWP_BYTE_BLOB", pack=False, align=None), offset=0), "unicodeString": SimTypePointer(SimTypeChar(label="Char"), offset=0), "byteArray6": SimTypePointer(SimStruct({"byteArray6": SimTypeFixedSizeArray(SimTypeChar(label="Byte"), 6)}, name="FWP_BYTE_ARRAY6", pack=False, align=None), offset=0), "bitmapArray64": SimTypePointer(SimStruct({"bitmapArray64": SimTypeFixedSizeArray(SimTypeChar(label="Byte"), 8)}, name="FWP_BITMAP_ARRAY64_", pack=False, align=None), offset=0), "v4AddrMask": SimTypePointer(SimStruct({"addr": SimTypeInt(signed=False, label="UInt32"), "mask": SimTypeInt(signed=False, label="UInt32")}, name="FWP_V4_ADDR_AND_MASK", pack=False, align=None), offset=0), "v6AddrMask": SimTypePointer(SimStruct({"addr": SimTypeFixedSizeArray(SimTypeChar(label="Byte"), 16), "prefixLength": SimTypeChar(label="Byte")}, name="FWP_V6_ADDR_AND_MASK", pack=False, align=None), offset=0), "rangeValue": SimTypePointer(SimStruct({"valueLow": SimStruct({"type": SimTypeInt(signed=False, label="FWP_DATA_TYPE"), "Anonymous": SimUnion({"uint8": SimTypeChar(label="Byte"), "uint16": SimTypeShort(signed=False, label="UInt16"), "uint32": SimTypeInt(signed=False, label="UInt32"), "uint64": SimTypePointer(SimTypeLongLong(signed=False, label="UInt64"), offset=0), "int8": SimTypeChar(label="SByte"), "int16": SimTypeShort(signed=True, label="Int16"), "int32": SimTypeInt(signed=True, label="Int32"), "int64": SimTypePointer(SimTypeLongLong(signed=True, label="Int64"), offset=0), "float32": SimTypeFloat(size=32), "double64": SimTypePointer(SimTypeFloat(size=64), offset=0), "byteArray16": SimTypePointer(SimStruct({"byteArray16": SimTypeFixedSizeArray(SimTypeChar(label="Byte"), 16)}, name="FWP_BYTE_ARRAY16", pack=False, align=None), offset=0), "byteBlob": SimTypePointer(SimStruct({"size": SimTypeInt(signed=False, label="UInt32"), "data": SimTypePointer(SimTypeChar(label="Byte"), offset=0)}, name="FWP_BYTE_BLOB", pack=False, align=None), offset=0), "sid": SimTypePointer(SimTypeBottom(label="SID"), offset=0), "sd": SimTypePointer(SimStruct({"size": SimTypeInt(signed=False, label="UInt32"), "data": SimTypePointer(SimTypeChar(label="Byte"), offset=0)}, name="FWP_BYTE_BLOB", pack=False, align=None), offset=0), "tokenInformation": SimTypePointer(SimStruct({"sidCount": SimTypeInt(signed=False, label="UInt32"), "sids": SimTypePointer(SimTypeBottom(label="SID_AND_ATTRIBUTES"), offset=0), "restrictedSidCount": SimTypeInt(signed=False, label="UInt32"), "restrictedSids": SimTypePointer(SimTypeBottom(label="SID_AND_ATTRIBUTES"), offset=0)}, name="FWP_TOKEN_INFORMATION", pack=False, align=None), offset=0), "tokenAccessInformation": SimTypePointer(SimStruct({"size": SimTypeInt(signed=False, label="UInt32"), "data": SimTypePointer(SimTypeChar(label="Byte"), offset=0)}, name="FWP_BYTE_BLOB", pack=False, align=None), offset=0), "unicodeString": SimTypePointer(SimTypeChar(label="Char"), offset=0), "byteArray6": SimTypePointer(SimStruct({"byteArray6": SimTypeFixedSizeArray(SimTypeChar(label="Byte"), 6)}, name="FWP_BYTE_ARRAY6", pack=False, align=None), offset=0), "bitmapArray64": SimTypePointer(SimStruct({"bitmapArray64": SimTypeFixedSizeArray(SimTypeChar(label="Byte"), 8)}, name="FWP_BITMAP_ARRAY64_", pack=False, align=None), offset=0)}, name="<anon>", label="None")}, name="FWP_VALUE0", pack=False, align=None), "valueHigh": SimStruct({"type": SimTypeInt(signed=False, label="FWP_DATA_TYPE"), "Anonymous": SimUnion({"uint8": SimTypeChar(label="Byte"), "uint16": SimTypeShort(signed=False, label="UInt16"), "uint32": SimTypeInt(signed=False, label="UInt32"), "uint64": SimTypePointer(SimTypeLongLong(signed=False, label="UInt64"), offset=0), "int8": SimTypeChar(label="SByte"), "int16": SimTypeShort(signed=True, label="Int16"), "int32": SimTypeInt(signed=True, label="Int32"), "int64": SimTypePointer(SimTypeLongLong(signed=True, label="Int64"), offset=0), "float32": SimTypeFloat(size=32), "double64": SimTypePointer(SimTypeFloat(size=64), offset=0), "byteArray16": SimTypePointer(SimStruct({"byteArray16": SimTypeFixedSizeArray(SimTypeChar(label="Byte"), 16)}, name="FWP_BYTE_ARRAY16", pack=False, align=None), offset=0), "byteBlob": SimTypePointer(SimStruct({"size": SimTypeInt(signed=False, label="UInt32"), "data": SimTypePointer(SimTypeChar(label="Byte"), offset=0)}, name="FWP_BYTE_BLOB", pack=False, align=None), offset=0), "sid": SimTypePointer(SimTypeBottom(label="SID"), offset=0), "sd": SimTypePointer(SimStruct({"size": SimTypeInt(signed=False, label="UInt32"), "data": SimTypePointer(SimTypeChar(label="Byte"), offset=0)}, name="FWP_BYTE_BLOB", pack=False, align=None), offset=0), "tokenInformation": SimTypePointer(SimStruct({"sidCount": SimTypeInt(signed=False, label="UInt32"), "sids": SimTypePointer(SimTypeBottom(label="SID_AND_ATTRIBUTES"), offset=0), "restrictedSidCount": SimTypeInt(signed=False, label="UInt32"), "restrictedSids": SimTypePointer(SimTypeBottom(label="SID_AND_ATTRIBUTES"), offset=0)}, name="FWP_TOKEN_INFORMATION", pack=False, align=None), offset=0), "tokenAccessInformation": SimTypePointer(SimStruct({"size": SimTypeInt(signed=False, label="UInt32"), "data": SimTypePointer(SimTypeChar(label="Byte"), offset=0)}, name="FWP_BYTE_BLOB", pack=False, align=None), offset=0), "unicodeString": SimTypePointer(SimTypeChar(label="Char"), offset=0), "byteArray6": SimTypePointer(SimStruct({"byteArray6": SimTypeFixedSizeArray(SimTypeChar(label="Byte"), 6)}, name="FWP_BYTE_ARRAY6", pack=False, align=None), offset=0), "bitmapArray64": SimTypePointer(SimStruct({"bitmapArray64": SimTypeFixedSizeArray(SimTypeChar(label="Byte"), 8)}, name="FWP_BITMAP_ARRAY64_", pack=False, align=None), offset=0)}, name="<anon>", label="None")}, name="FWP_VALUE0", pack=False, align=None)}, name="FWP_RANGE0", pack=False, align=None), offset=0)}, name="<anon>", label="None")}, name="FWP_CONDITION_VALUE0", pack=False, align=None)}, name="<anon>", label="None"), "qmFilterId": SimTypeLongLong(signed=False, label="UInt64")}, name="FWPM_NET_EVENT_IKEEXT_QM_FAILURE0", pack=False, align=None), offset=0), "ikeEmFailure": SimTypePointer(SimStruct({"failureErrorCode": SimTypeInt(signed=False, label="UInt32"), "failurePoint": SimTypeInt(signed=False, label="IPSEC_FAILURE_POINT"), "flags": SimTypeInt(signed=False, label="UInt32"), "emState": SimTypeInt(signed=False, label="IKEEXT_EM_SA_STATE"), "saRole": SimTypeInt(signed=False, label="IKEEXT_SA_ROLE"), "emAuthMethod": SimTypeInt(signed=False, label="IKEEXT_AUTHENTICATION_METHOD_TYPE"), "endCertHash": SimTypeFixedSizeArray(SimTypeChar(label="Byte"), 20), "mmId": SimTypeLongLong(signed=False, label="UInt64"), "qmFilterId": SimTypeLongLong(signed=False, label="UInt64"), "localPrincipalNameForAuth": SimTypePointer(SimTypeChar(label="Char"), offset=0), "remotePrincipalNameForAuth": SimTypePointer(SimTypeChar(label="Char"), offset=0), "numLocalPrincipalGroupSids": SimTypeInt(signed=False, label="UInt32"), "localPrincipalGroupSids": SimTypePointer(SimTypePointer(SimTypeChar(label="Char"), offset=0), offset=0), "numRemotePrincipalGroupSids": SimTypeInt(signed=False, label="UInt32"), "remotePrincipalGroupSids": SimTypePointer(SimTypePointer(SimTypeChar(label="Char"), offset=0), offset=0), "saTrafficType": SimTypeInt(signed=False, label="IPSEC_TRAFFIC_TYPE")}, name="FWPM_NET_EVENT_IKEEXT_EM_FAILURE1", pack=False, align=None), offset=0), "classifyDrop": SimTypePointer(SimStruct({"filterId": SimTypeLongLong(signed=False, label="UInt64"), "layerId": SimTypeShort(signed=False, label="UInt16"), "reauthReason": SimTypeInt(signed=False, label="UInt32"), "originalProfile": SimTypeInt(signed=False, label="UInt32"), "currentProfile": SimTypeInt(signed=False, label="UInt32"), "msFwpDirection": SimTypeInt(signed=False, label="UInt32"), "isLoopback": SimTypeInt(signed=True, label="Int32"), "vSwitchId": SimStruct({"size": SimTypeInt(signed=False, label="UInt32"), "data": SimTypePointer(SimTypeChar(label="Byte"), offset=0)}, name="FWP_BYTE_BLOB", pack=False, align=None), "vSwitchSourcePort": SimTypeInt(signed=False, label="UInt32"), "vSwitchDestinationPort": SimTypeInt(signed=False, label="UInt32")}, name="FWPM_NET_EVENT_CLASSIFY_DROP2", pack=False, align=None), offset=0), "ipsecDrop": SimTypePointer(SimStruct({"failureStatus": SimTypeInt(signed=True, label="Int32"), "direction": SimTypeInt(signed=False, label="FWP_DIRECTION"), "spi": SimTypeInt(signed=False, label="UInt32"), "filterId": SimTypeLongLong(signed=False, label="UInt64"), "layerId": SimTypeShort(signed=False, label="UInt16")}, name="FWPM_NET_EVENT_IPSEC_KERNEL_DROP0", pack=False, align=None), offset=0), "idpDrop": SimTypePointer(SimStruct({"ipVersion": SimTypeInt(signed=False, label="FWP_IP_VERSION"), "Anonymous1": SimUnion({"publicHostV4Addr": SimTypeInt(signed=False, label="UInt32"), "publicHostV6Addr": SimTypeFixedSizeArray(SimTypeChar(label="Byte"), 16)}, name="<anon>", label="None"), "Anonymous2": SimUnion({"internalHostV4Addr": SimTypeInt(signed=False, label="UInt32"), "internalHostV6Addr": SimTypeFixedSizeArray(SimTypeChar(label="Byte"), 16)}, name="<anon>", label="None"), "failureStatus": SimTypeInt(signed=True, label="Int32"), "direction": SimTypeInt(signed=False, label="FWP_DIRECTION")}, name="FWPM_NET_EVENT_IPSEC_DOSP_DROP0", pack=False, align=None), offset=0), "classifyAllow": SimTypePointer(SimStruct({"filterId": SimTypeLongLong(signed=False, label="UInt64"), "layerId": SimTypeShort(signed=False, label="UInt16"), "reauthReason": SimTypeInt(signed=False, label="UInt32"), "originalProfile": SimTypeInt(signed=False, label="UInt32"), "currentProfile": SimTypeInt(signed=False, label="UInt32"), "msFwpDirection": SimTypeInt(signed=False, label="UInt32"), "isLoopback": SimTypeInt(signed=True, label="Int32")}, name="FWPM_NET_EVENT_CLASSIFY_ALLOW0", pack=False, align=None), offset=0), "capabilityDrop": SimTypePointer(SimStruct({"networkCapabilityId": SimTypeInt(signed=False, label="FWPM_APPC_NETWORK_CAPABILITY_TYPE"), "filterId": SimTypeLongLong(signed=False, label="UInt64"), "isLoopback": SimTypeInt(signed=True, label="Int32")}, name="FWPM_NET_EVENT_CAPABILITY_DROP0", pack=False, align=None), offset=0), "capabilityAllow": SimTypePointer(SimStruct({"networkCapabilityId": SimTypeInt(signed=False, label="FWPM_APPC_NETWORK_CAPABILITY_TYPE"), "filterId": SimTypeLongLong(signed=False, label="UInt64"), "isLoopback": SimTypeInt(signed=True, label="Int32")}, name="FWPM_NET_EVENT_CAPABILITY_ALLOW0", pack=False, align=None), offset=0), "classifyDropMac": SimTypePointer(SimStruct({"localMacAddr": SimStruct({"byteArray6": SimTypeFixedSizeArray(SimTypeChar(label="Byte"), 6)}, name="FWP_BYTE_ARRAY6", pack=False, align=None), "remoteMacAddr": SimStruct({"byteArray6": SimTypeFixedSizeArray(SimTypeChar(label="Byte"), 6)}, name="FWP_BYTE_ARRAY6", pack=False, align=None), "mediaType": SimTypeInt(signed=False, label="UInt32"), "ifType": SimTypeInt(signed=False, label="UInt32"), "etherType": SimTypeShort(signed=False, label="UInt16"), "ndisPortNumber": SimTypeInt(signed=False, label="UInt32"), "reserved": SimTypeInt(signed=False, label="UInt32"), "vlanTag": SimTypeShort(signed=False, label="UInt16"), "ifLuid": SimTypeLongLong(signed=False, label="UInt64"), "filterId": SimTypeLongLong(signed=False, label="UInt64"), "layerId": SimTypeShort(signed=False, label="UInt16"), "reauthReason": SimTypeInt(signed=False, label="UInt32"), "originalProfile": SimTypeInt(signed=False, label="UInt32"), "currentProfile": SimTypeInt(signed=False, label="UInt32"), "msFwpDirection": SimTypeInt(signed=False, label="UInt32"), "isLoopback": SimTypeInt(signed=True, label="Int32"), "vSwitchId": SimStruct({"size": SimTypeInt(signed=False, label="UInt32"), "data": SimTypePointer(SimTypeChar(label="Byte"), offset=0)}, name="FWP_BYTE_BLOB", pack=False, align=None), "vSwitchSourcePort": SimTypeInt(signed=False, label="UInt32"), "vSwitchDestinationPort": SimTypeInt(signed=False, label="UInt32")}, name="FWPM_NET_EVENT_CLASSIFY_DROP_MAC0", pack=False, align=None), offset=0)}, name="<anon>", label="None")}, name="FWPM_NET_EVENT2", pack=False, align=None), offset=0)], SimTypeBottom(label="Void"), arg_names=["context", "event"]), offset=0), SimTypePointer(SimTypeBottom(label="Void"), offset=0), SimTypePointer(SimTypePointer(SimTypeInt(signed=True, label="Int"), label="IntPtr", offset=0), offset=0)], SimTypeInt(signed=False, label="UInt32"), arg_names=["engineHandle", "subscription", "callback", "context", "eventsHandle"]),
        # 
        'FwpmNetEventSubscribe2': SimTypeFunction([SimTypePointer(SimTypeInt(signed=True, label="Int"), label="IntPtr", offset=0), SimTypePointer(SimStruct({"enumTemplate": SimTypePointer(SimStruct({"startTime": SimStruct({"dwLowDateTime": SimTypeInt(signed=False, label="UInt32"), "dwHighDateTime": SimTypeInt(signed=False, label="UInt32")}, name="FILETIME", pack=False, align=None), "endTime": SimStruct({"dwLowDateTime": SimTypeInt(signed=False, label="UInt32"), "dwHighDateTime": SimTypeInt(signed=False, label="UInt32")}, name="FILETIME", pack=False, align=None), "numFilterConditions": SimTypeInt(signed=False, label="UInt32"), "filterCondition": SimTypePointer(SimStruct({"fieldKey": SimTypeBottom(label="Guid"), "matchType": SimTypeInt(signed=False, label="FWP_MATCH_TYPE"), "conditionValue": SimStruct({"type": SimTypeInt(signed=False, label="FWP_DATA_TYPE"), "Anonymous": SimUnion({"uint8": SimTypeChar(label="Byte"), "uint16": SimTypeShort(signed=False, label="UInt16"), "uint32": SimTypeInt(signed=False, label="UInt32"), "uint64": SimTypePointer(SimTypeLongLong(signed=False, label="UInt64"), offset=0), "int8": SimTypeChar(label="SByte"), "int16": SimTypeShort(signed=True, label="Int16"), "int32": SimTypeInt(signed=True, label="Int32"), "int64": SimTypePointer(SimTypeLongLong(signed=True, label="Int64"), offset=0), "float32": SimTypeFloat(size=32), "double64": SimTypePointer(SimTypeFloat(size=64), offset=0), "byteArray16": SimTypePointer(SimStruct({"byteArray16": SimTypeFixedSizeArray(SimTypeChar(label="Byte"), 16)}, name="FWP_BYTE_ARRAY16", pack=False, align=None), offset=0), "byteBlob": SimTypePointer(SimStruct({"size": SimTypeInt(signed=False, label="UInt32"), "data": SimTypePointer(SimTypeChar(label="Byte"), offset=0)}, name="FWP_BYTE_BLOB", pack=False, align=None), offset=0), "sid": SimTypePointer(SimTypeBottom(label="SID"), offset=0), "sd": SimTypePointer(SimStruct({"size": SimTypeInt(signed=False, label="UInt32"), "data": SimTypePointer(SimTypeChar(label="Byte"), offset=0)}, name="FWP_BYTE_BLOB", pack=False, align=None), offset=0), "tokenInformation": SimTypePointer(SimStruct({"sidCount": SimTypeInt(signed=False, label="UInt32"), "sids": SimTypePointer(SimTypeBottom(label="SID_AND_ATTRIBUTES"), offset=0), "restrictedSidCount": SimTypeInt(signed=False, label="UInt32"), "restrictedSids": SimTypePointer(SimTypeBottom(label="SID_AND_ATTRIBUTES"), offset=0)}, name="FWP_TOKEN_INFORMATION", pack=False, align=None), offset=0), "tokenAccessInformation": SimTypePointer(SimStruct({"size": SimTypeInt(signed=False, label="UInt32"), "data": SimTypePointer(SimTypeChar(label="Byte"), offset=0)}, name="FWP_BYTE_BLOB", pack=False, align=None), offset=0), "unicodeString": SimTypePointer(SimTypeChar(label="Char"), offset=0), "byteArray6": SimTypePointer(SimStruct({"byteArray6": SimTypeFixedSizeArray(SimTypeChar(label="Byte"), 6)}, name="FWP_BYTE_ARRAY6", pack=False, align=None), offset=0), "bitmapArray64": SimTypePointer(SimStruct({"bitmapArray64": SimTypeFixedSizeArray(SimTypeChar(label="Byte"), 8)}, name="FWP_BITMAP_ARRAY64_", pack=False, align=None), offset=0), "v4AddrMask": SimTypePointer(SimStruct({"addr": SimTypeInt(signed=False, label="UInt32"), "mask": SimTypeInt(signed=False, label="UInt32")}, name="FWP_V4_ADDR_AND_MASK", pack=False, align=None), offset=0), "v6AddrMask": SimTypePointer(SimStruct({"addr": SimTypeFixedSizeArray(SimTypeChar(label="Byte"), 16), "prefixLength": SimTypeChar(label="Byte")}, name="FWP_V6_ADDR_AND_MASK", pack=False, align=None), offset=0), "rangeValue": SimTypePointer(SimStruct({"valueLow": SimStruct({"type": SimTypeInt(signed=False, label="FWP_DATA_TYPE"), "Anonymous": SimUnion({"uint8": SimTypeChar(label="Byte"), "uint16": SimTypeShort(signed=False, label="UInt16"), "uint32": SimTypeInt(signed=False, label="UInt32"), "uint64": SimTypePointer(SimTypeLongLong(signed=False, label="UInt64"), offset=0), "int8": SimTypeChar(label="SByte"), "int16": SimTypeShort(signed=True, label="Int16"), "int32": SimTypeInt(signed=True, label="Int32"), "int64": SimTypePointer(SimTypeLongLong(signed=True, label="Int64"), offset=0), "float32": SimTypeFloat(size=32), "double64": SimTypePointer(SimTypeFloat(size=64), offset=0), "byteArray16": SimTypePointer(SimStruct({"byteArray16": SimTypeFixedSizeArray(SimTypeChar(label="Byte"), 16)}, name="FWP_BYTE_ARRAY16", pack=False, align=None), offset=0), "byteBlob": SimTypePointer(SimStruct({"size": SimTypeInt(signed=False, label="UInt32"), "data": SimTypePointer(SimTypeChar(label="Byte"), offset=0)}, name="FWP_BYTE_BLOB", pack=False, align=None), offset=0), "sid": SimTypePointer(SimTypeBottom(label="SID"), offset=0), "sd": SimTypePointer(SimStruct({"size": SimTypeInt(signed=False, label="UInt32"), "data": SimTypePointer(SimTypeChar(label="Byte"), offset=0)}, name="FWP_BYTE_BLOB", pack=False, align=None), offset=0), "tokenInformation": SimTypePointer(SimStruct({"sidCount": SimTypeInt(signed=False, label="UInt32"), "sids": SimTypePointer(SimTypeBottom(label="SID_AND_ATTRIBUTES"), offset=0), "restrictedSidCount": SimTypeInt(signed=False, label="UInt32"), "restrictedSids": SimTypePointer(SimTypeBottom(label="SID_AND_ATTRIBUTES"), offset=0)}, name="FWP_TOKEN_INFORMATION", pack=False, align=None), offset=0), "tokenAccessInformation": SimTypePointer(SimStruct({"size": SimTypeInt(signed=False, label="UInt32"), "data": SimTypePointer(SimTypeChar(label="Byte"), offset=0)}, name="FWP_BYTE_BLOB", pack=False, align=None), offset=0), "unicodeString": SimTypePointer(SimTypeChar(label="Char"), offset=0), "byteArray6": SimTypePointer(SimStruct({"byteArray6": SimTypeFixedSizeArray(SimTypeChar(label="Byte"), 6)}, name="FWP_BYTE_ARRAY6", pack=False, align=None), offset=0), "bitmapArray64": SimTypePointer(SimStruct({"bitmapArray64": SimTypeFixedSizeArray(SimTypeChar(label="Byte"), 8)}, name="FWP_BITMAP_ARRAY64_", pack=False, align=None), offset=0)}, name="<anon>", label="None")}, name="FWP_VALUE0", pack=False, align=None), "valueHigh": SimStruct({"type": SimTypeInt(signed=False, label="FWP_DATA_TYPE"), "Anonymous": SimUnion({"uint8": SimTypeChar(label="Byte"), "uint16": SimTypeShort(signed=False, label="UInt16"), "uint32": SimTypeInt(signed=False, label="UInt32"), "uint64": SimTypePointer(SimTypeLongLong(signed=False, label="UInt64"), offset=0), "int8": SimTypeChar(label="SByte"), "int16": SimTypeShort(signed=True, label="Int16"), "int32": SimTypeInt(signed=True, label="Int32"), "int64": SimTypePointer(SimTypeLongLong(signed=True, label="Int64"), offset=0), "float32": SimTypeFloat(size=32), "double64": SimTypePointer(SimTypeFloat(size=64), offset=0), "byteArray16": SimTypePointer(SimStruct({"byteArray16": SimTypeFixedSizeArray(SimTypeChar(label="Byte"), 16)}, name="FWP_BYTE_ARRAY16", pack=False, align=None), offset=0), "byteBlob": SimTypePointer(SimStruct({"size": SimTypeInt(signed=False, label="UInt32"), "data": SimTypePointer(SimTypeChar(label="Byte"), offset=0)}, name="FWP_BYTE_BLOB", pack=False, align=None), offset=0), "sid": SimTypePointer(SimTypeBottom(label="SID"), offset=0), "sd": SimTypePointer(SimStruct({"size": SimTypeInt(signed=False, label="UInt32"), "data": SimTypePointer(SimTypeChar(label="Byte"), offset=0)}, name="FWP_BYTE_BLOB", pack=False, align=None), offset=0), "tokenInformation": SimTypePointer(SimStruct({"sidCount": SimTypeInt(signed=False, label="UInt32"), "sids": SimTypePointer(SimTypeBottom(label="SID_AND_ATTRIBUTES"), offset=0), "restrictedSidCount": SimTypeInt(signed=False, label="UInt32"), "restrictedSids": SimTypePointer(SimTypeBottom(label="SID_AND_ATTRIBUTES"), offset=0)}, name="FWP_TOKEN_INFORMATION", pack=False, align=None), offset=0), "tokenAccessInformation": SimTypePointer(SimStruct({"size": SimTypeInt(signed=False, label="UInt32"), "data": SimTypePointer(SimTypeChar(label="Byte"), offset=0)}, name="FWP_BYTE_BLOB", pack=False, align=None), offset=0), "unicodeString": SimTypePointer(SimTypeChar(label="Char"), offset=0), "byteArray6": SimTypePointer(SimStruct({"byteArray6": SimTypeFixedSizeArray(SimTypeChar(label="Byte"), 6)}, name="FWP_BYTE_ARRAY6", pack=False, align=None), offset=0), "bitmapArray64": SimTypePointer(SimStruct({"bitmapArray64": SimTypeFixedSizeArray(SimTypeChar(label="Byte"), 8)}, name="FWP_BITMAP_ARRAY64_", pack=False, align=None), offset=0)}, name="<anon>", label="None")}, name="FWP_VALUE0", pack=False, align=None)}, name="FWP_RANGE0", pack=False, align=None), offset=0)}, name="<anon>", label="None")}, name="FWP_CONDITION_VALUE0", pack=False, align=None)}, name="FWPM_FILTER_CONDITION0", pack=False, align=None), offset=0)}, name="FWPM_NET_EVENT_ENUM_TEMPLATE0", pack=False, align=None), offset=0), "flags": SimTypeInt(signed=False, label="UInt32"), "sessionKey": SimTypeBottom(label="Guid")}, name="FWPM_NET_EVENT_SUBSCRIPTION0", pack=False, align=None), offset=0), SimTypePointer(SimTypeFunction([SimTypePointer(SimTypeBottom(label="Void"), offset=0), SimTypePointer(SimStruct({"header": SimStruct({"timeStamp": SimStruct({"dwLowDateTime": SimTypeInt(signed=False, label="UInt32"), "dwHighDateTime": SimTypeInt(signed=False, label="UInt32")}, name="FILETIME", pack=False, align=None), "flags": SimTypeInt(signed=False, label="UInt32"), "ipVersion": SimTypeInt(signed=False, label="FWP_IP_VERSION"), "ipProtocol": SimTypeChar(label="Byte"), "Anonymous1": SimUnion({"localAddrV4": SimTypeInt(signed=False, label="UInt32"), "localAddrV6": SimStruct({"byteArray16": SimTypeFixedSizeArray(SimTypeChar(label="Byte"), 16)}, name="FWP_BYTE_ARRAY16", pack=False, align=None)}, name="<anon>", label="None"), "Anonymous2": SimUnion({"remoteAddrV4": SimTypeInt(signed=False, label="UInt32"), "remoteAddrV6": SimStruct({"byteArray16": SimTypeFixedSizeArray(SimTypeChar(label="Byte"), 16)}, name="FWP_BYTE_ARRAY16", pack=False, align=None)}, name="<anon>", label="None"), "localPort": SimTypeShort(signed=False, label="UInt16"), "remotePort": SimTypeShort(signed=False, label="UInt16"), "scopeId": SimTypeInt(signed=False, label="UInt32"), "appId": SimStruct({"size": SimTypeInt(signed=False, label="UInt32"), "data": SimTypePointer(SimTypeChar(label="Byte"), offset=0)}, name="FWP_BYTE_BLOB", pack=False, align=None), "userId": SimTypePointer(SimTypeBottom(label="SID"), offset=0), "addressFamily": SimTypeInt(signed=False, label="FWP_AF"), "packageSid": SimTypePointer(SimTypeBottom(label="SID"), offset=0), "enterpriseId": SimTypePointer(SimTypeChar(label="Char"), offset=0), "policyFlags": SimTypeLongLong(signed=False, label="UInt64"), "effectiveName": SimStruct({"size": SimTypeInt(signed=False, label="UInt32"), "data": SimTypePointer(SimTypeChar(label="Byte"), offset=0)}, name="FWP_BYTE_BLOB", pack=False, align=None)}, name="FWPM_NET_EVENT_HEADER3", pack=False, align=None), "type": SimTypeInt(signed=False, label="FWPM_NET_EVENT_TYPE"), "Anonymous": SimUnion({"ikeMmFailure": SimTypePointer(SimStruct({"failureErrorCode": SimTypeInt(signed=False, label="UInt32"), "failurePoint": SimTypeInt(signed=False, label="IPSEC_FAILURE_POINT"), "flags": SimTypeInt(signed=False, label="UInt32"), "keyingModuleType": SimTypeInt(signed=False, label="IKEEXT_KEY_MODULE_TYPE"), "mmState": SimTypeInt(signed=False, label="IKEEXT_MM_SA_STATE"), "saRole": SimTypeInt(signed=False, label="IKEEXT_SA_ROLE"), "mmAuthMethod": SimTypeInt(signed=False, label="IKEEXT_AUTHENTICATION_METHOD_TYPE"), "endCertHash": SimTypeFixedSizeArray(SimTypeChar(label="Byte"), 20), "mmId": SimTypeLongLong(signed=False, label="UInt64"), "mmFilterId": SimTypeLongLong(signed=False, label="UInt64"), "localPrincipalNameForAuth": SimTypePointer(SimTypeChar(label="Char"), offset=0), "remotePrincipalNameForAuth": SimTypePointer(SimTypeChar(label="Char"), offset=0), "numLocalPrincipalGroupSids": SimTypeInt(signed=False, label="UInt32"), "localPrincipalGroupSids": SimTypePointer(SimTypePointer(SimTypeChar(label="Char"), offset=0), offset=0), "numRemotePrincipalGroupSids": SimTypeInt(signed=False, label="UInt32"), "remotePrincipalGroupSids": SimTypePointer(SimTypePointer(SimTypeChar(label="Char"), offset=0), offset=0)}, name="FWPM_NET_EVENT_IKEEXT_MM_FAILURE1", pack=False, align=None), offset=0), "ikeQmFailure": SimTypePointer(SimStruct({"failureErrorCode": SimTypeInt(signed=False, label="UInt32"), "failurePoint": SimTypeInt(signed=False, label="IPSEC_FAILURE_POINT"), "keyingModuleType": SimTypeInt(signed=False, label="IKEEXT_KEY_MODULE_TYPE"), "qmState": SimTypeInt(signed=False, label="IKEEXT_QM_SA_STATE"), "saRole": SimTypeInt(signed=False, label="IKEEXT_SA_ROLE"), "saTrafficType": SimTypeInt(signed=False, label="IPSEC_TRAFFIC_TYPE"), "Anonymous1": SimUnion({"localSubNet": SimStruct({"type": SimTypeInt(signed=False, label="FWP_DATA_TYPE"), "Anonymous": SimUnion({"uint8": SimTypeChar(label="Byte"), "uint16": SimTypeShort(signed=False, label="UInt16"), "uint32": SimTypeInt(signed=False, label="UInt32"), "uint64": SimTypePointer(SimTypeLongLong(signed=False, label="UInt64"), offset=0), "int8": SimTypeChar(label="SByte"), "int16": SimTypeShort(signed=True, label="Int16"), "int32": SimTypeInt(signed=True, label="Int32"), "int64": SimTypePointer(SimTypeLongLong(signed=True, label="Int64"), offset=0), "float32": SimTypeFloat(size=32), "double64": SimTypePointer(SimTypeFloat(size=64), offset=0), "byteArray16": SimTypePointer(SimStruct({"byteArray16": SimTypeFixedSizeArray(SimTypeChar(label="Byte"), 16)}, name="FWP_BYTE_ARRAY16", pack=False, align=None), offset=0), "byteBlob": SimTypePointer(SimStruct({"size": SimTypeInt(signed=False, label="UInt32"), "data": SimTypePointer(SimTypeChar(label="Byte"), offset=0)}, name="FWP_BYTE_BLOB", pack=False, align=None), offset=0), "sid": SimTypePointer(SimTypeBottom(label="SID"), offset=0), "sd": SimTypePointer(SimStruct({"size": SimTypeInt(signed=False, label="UInt32"), "data": SimTypePointer(SimTypeChar(label="Byte"), offset=0)}, name="FWP_BYTE_BLOB", pack=False, align=None), offset=0), "tokenInformation": SimTypePointer(SimStruct({"sidCount": SimTypeInt(signed=False, label="UInt32"), "sids": SimTypePointer(SimTypeBottom(label="SID_AND_ATTRIBUTES"), offset=0), "restrictedSidCount": SimTypeInt(signed=False, label="UInt32"), "restrictedSids": SimTypePointer(SimTypeBottom(label="SID_AND_ATTRIBUTES"), offset=0)}, name="FWP_TOKEN_INFORMATION", pack=False, align=None), offset=0), "tokenAccessInformation": SimTypePointer(SimStruct({"size": SimTypeInt(signed=False, label="UInt32"), "data": SimTypePointer(SimTypeChar(label="Byte"), offset=0)}, name="FWP_BYTE_BLOB", pack=False, align=None), offset=0), "unicodeString": SimTypePointer(SimTypeChar(label="Char"), offset=0), "byteArray6": SimTypePointer(SimStruct({"byteArray6": SimTypeFixedSizeArray(SimTypeChar(label="Byte"), 6)}, name="FWP_BYTE_ARRAY6", pack=False, align=None), offset=0), "bitmapArray64": SimTypePointer(SimStruct({"bitmapArray64": SimTypeFixedSizeArray(SimTypeChar(label="Byte"), 8)}, name="FWP_BITMAP_ARRAY64_", pack=False, align=None), offset=0), "v4AddrMask": SimTypePointer(SimStruct({"addr": SimTypeInt(signed=False, label="UInt32"), "mask": SimTypeInt(signed=False, label="UInt32")}, name="FWP_V4_ADDR_AND_MASK", pack=False, align=None), offset=0), "v6AddrMask": SimTypePointer(SimStruct({"addr": SimTypeFixedSizeArray(SimTypeChar(label="Byte"), 16), "prefixLength": SimTypeChar(label="Byte")}, name="FWP_V6_ADDR_AND_MASK", pack=False, align=None), offset=0), "rangeValue": SimTypePointer(SimStruct({"valueLow": SimStruct({"type": SimTypeInt(signed=False, label="FWP_DATA_TYPE"), "Anonymous": SimUnion({"uint8": SimTypeChar(label="Byte"), "uint16": SimTypeShort(signed=False, label="UInt16"), "uint32": SimTypeInt(signed=False, label="UInt32"), "uint64": SimTypePointer(SimTypeLongLong(signed=False, label="UInt64"), offset=0), "int8": SimTypeChar(label="SByte"), "int16": SimTypeShort(signed=True, label="Int16"), "int32": SimTypeInt(signed=True, label="Int32"), "int64": SimTypePointer(SimTypeLongLong(signed=True, label="Int64"), offset=0), "float32": SimTypeFloat(size=32), "double64": SimTypePointer(SimTypeFloat(size=64), offset=0), "byteArray16": SimTypePointer(SimStruct({"byteArray16": SimTypeFixedSizeArray(SimTypeChar(label="Byte"), 16)}, name="FWP_BYTE_ARRAY16", pack=False, align=None), offset=0), "byteBlob": SimTypePointer(SimStruct({"size": SimTypeInt(signed=False, label="UInt32"), "data": SimTypePointer(SimTypeChar(label="Byte"), offset=0)}, name="FWP_BYTE_BLOB", pack=False, align=None), offset=0), "sid": SimTypePointer(SimTypeBottom(label="SID"), offset=0), "sd": SimTypePointer(SimStruct({"size": SimTypeInt(signed=False, label="UInt32"), "data": SimTypePointer(SimTypeChar(label="Byte"), offset=0)}, name="FWP_BYTE_BLOB", pack=False, align=None), offset=0), "tokenInformation": SimTypePointer(SimStruct({"sidCount": SimTypeInt(signed=False, label="UInt32"), "sids": SimTypePointer(SimTypeBottom(label="SID_AND_ATTRIBUTES"), offset=0), "restrictedSidCount": SimTypeInt(signed=False, label="UInt32"), "restrictedSids": SimTypePointer(SimTypeBottom(label="SID_AND_ATTRIBUTES"), offset=0)}, name="FWP_TOKEN_INFORMATION", pack=False, align=None), offset=0), "tokenAccessInformation": SimTypePointer(SimStruct({"size": SimTypeInt(signed=False, label="UInt32"), "data": SimTypePointer(SimTypeChar(label="Byte"), offset=0)}, name="FWP_BYTE_BLOB", pack=False, align=None), offset=0), "unicodeString": SimTypePointer(SimTypeChar(label="Char"), offset=0), "byteArray6": SimTypePointer(SimStruct({"byteArray6": SimTypeFixedSizeArray(SimTypeChar(label="Byte"), 6)}, name="FWP_BYTE_ARRAY6", pack=False, align=None), offset=0), "bitmapArray64": SimTypePointer(SimStruct({"bitmapArray64": SimTypeFixedSizeArray(SimTypeChar(label="Byte"), 8)}, name="FWP_BITMAP_ARRAY64_", pack=False, align=None), offset=0)}, name="<anon>", label="None")}, name="FWP_VALUE0", pack=False, align=None), "valueHigh": SimStruct({"type": SimTypeInt(signed=False, label="FWP_DATA_TYPE"), "Anonymous": SimUnion({"uint8": SimTypeChar(label="Byte"), "uint16": SimTypeShort(signed=False, label="UInt16"), "uint32": SimTypeInt(signed=False, label="UInt32"), "uint64": SimTypePointer(SimTypeLongLong(signed=False, label="UInt64"), offset=0), "int8": SimTypeChar(label="SByte"), "int16": SimTypeShort(signed=True, label="Int16"), "int32": SimTypeInt(signed=True, label="Int32"), "int64": SimTypePointer(SimTypeLongLong(signed=True, label="Int64"), offset=0), "float32": SimTypeFloat(size=32), "double64": SimTypePointer(SimTypeFloat(size=64), offset=0), "byteArray16": SimTypePointer(SimStruct({"byteArray16": SimTypeFixedSizeArray(SimTypeChar(label="Byte"), 16)}, name="FWP_BYTE_ARRAY16", pack=False, align=None), offset=0), "byteBlob": SimTypePointer(SimStruct({"size": SimTypeInt(signed=False, label="UInt32"), "data": SimTypePointer(SimTypeChar(label="Byte"), offset=0)}, name="FWP_BYTE_BLOB", pack=False, align=None), offset=0), "sid": SimTypePointer(SimTypeBottom(label="SID"), offset=0), "sd": SimTypePointer(SimStruct({"size": SimTypeInt(signed=False, label="UInt32"), "data": SimTypePointer(SimTypeChar(label="Byte"), offset=0)}, name="FWP_BYTE_BLOB", pack=False, align=None), offset=0), "tokenInformation": SimTypePointer(SimStruct({"sidCount": SimTypeInt(signed=False, label="UInt32"), "sids": SimTypePointer(SimTypeBottom(label="SID_AND_ATTRIBUTES"), offset=0), "restrictedSidCount": SimTypeInt(signed=False, label="UInt32"), "restrictedSids": SimTypePointer(SimTypeBottom(label="SID_AND_ATTRIBUTES"), offset=0)}, name="FWP_TOKEN_INFORMATION", pack=False, align=None), offset=0), "tokenAccessInformation": SimTypePointer(SimStruct({"size": SimTypeInt(signed=False, label="UInt32"), "data": SimTypePointer(SimTypeChar(label="Byte"), offset=0)}, name="FWP_BYTE_BLOB", pack=False, align=None), offset=0), "unicodeString": SimTypePointer(SimTypeChar(label="Char"), offset=0), "byteArray6": SimTypePointer(SimStruct({"byteArray6": SimTypeFixedSizeArray(SimTypeChar(label="Byte"), 6)}, name="FWP_BYTE_ARRAY6", pack=False, align=None), offset=0), "bitmapArray64": SimTypePointer(SimStruct({"bitmapArray64": SimTypeFixedSizeArray(SimTypeChar(label="Byte"), 8)}, name="FWP_BITMAP_ARRAY64_", pack=False, align=None), offset=0)}, name="<anon>", label="None")}, name="FWP_VALUE0", pack=False, align=None)}, name="FWP_RANGE0", pack=False, align=None), offset=0)}, name="<anon>", label="None")}, name="FWP_CONDITION_VALUE0", pack=False, align=None)}, name="<anon>", label="None"), "Anonymous2": SimUnion({"remoteSubNet": SimStruct({"type": SimTypeInt(signed=False, label="FWP_DATA_TYPE"), "Anonymous": SimUnion({"uint8": SimTypeChar(label="Byte"), "uint16": SimTypeShort(signed=False, label="UInt16"), "uint32": SimTypeInt(signed=False, label="UInt32"), "uint64": SimTypePointer(SimTypeLongLong(signed=False, label="UInt64"), offset=0), "int8": SimTypeChar(label="SByte"), "int16": SimTypeShort(signed=True, label="Int16"), "int32": SimTypeInt(signed=True, label="Int32"), "int64": SimTypePointer(SimTypeLongLong(signed=True, label="Int64"), offset=0), "float32": SimTypeFloat(size=32), "double64": SimTypePointer(SimTypeFloat(size=64), offset=0), "byteArray16": SimTypePointer(SimStruct({"byteArray16": SimTypeFixedSizeArray(SimTypeChar(label="Byte"), 16)}, name="FWP_BYTE_ARRAY16", pack=False, align=None), offset=0), "byteBlob": SimTypePointer(SimStruct({"size": SimTypeInt(signed=False, label="UInt32"), "data": SimTypePointer(SimTypeChar(label="Byte"), offset=0)}, name="FWP_BYTE_BLOB", pack=False, align=None), offset=0), "sid": SimTypePointer(SimTypeBottom(label="SID"), offset=0), "sd": SimTypePointer(SimStruct({"size": SimTypeInt(signed=False, label="UInt32"), "data": SimTypePointer(SimTypeChar(label="Byte"), offset=0)}, name="FWP_BYTE_BLOB", pack=False, align=None), offset=0), "tokenInformation": SimTypePointer(SimStruct({"sidCount": SimTypeInt(signed=False, label="UInt32"), "sids": SimTypePointer(SimTypeBottom(label="SID_AND_ATTRIBUTES"), offset=0), "restrictedSidCount": SimTypeInt(signed=False, label="UInt32"), "restrictedSids": SimTypePointer(SimTypeBottom(label="SID_AND_ATTRIBUTES"), offset=0)}, name="FWP_TOKEN_INFORMATION", pack=False, align=None), offset=0), "tokenAccessInformation": SimTypePointer(SimStruct({"size": SimTypeInt(signed=False, label="UInt32"), "data": SimTypePointer(SimTypeChar(label="Byte"), offset=0)}, name="FWP_BYTE_BLOB", pack=False, align=None), offset=0), "unicodeString": SimTypePointer(SimTypeChar(label="Char"), offset=0), "byteArray6": SimTypePointer(SimStruct({"byteArray6": SimTypeFixedSizeArray(SimTypeChar(label="Byte"), 6)}, name="FWP_BYTE_ARRAY6", pack=False, align=None), offset=0), "bitmapArray64": SimTypePointer(SimStruct({"bitmapArray64": SimTypeFixedSizeArray(SimTypeChar(label="Byte"), 8)}, name="FWP_BITMAP_ARRAY64_", pack=False, align=None), offset=0), "v4AddrMask": SimTypePointer(SimStruct({"addr": SimTypeInt(signed=False, label="UInt32"), "mask": SimTypeInt(signed=False, label="UInt32")}, name="FWP_V4_ADDR_AND_MASK", pack=False, align=None), offset=0), "v6AddrMask": SimTypePointer(SimStruct({"addr": SimTypeFixedSizeArray(SimTypeChar(label="Byte"), 16), "prefixLength": SimTypeChar(label="Byte")}, name="FWP_V6_ADDR_AND_MASK", pack=False, align=None), offset=0), "rangeValue": SimTypePointer(SimStruct({"valueLow": SimStruct({"type": SimTypeInt(signed=False, label="FWP_DATA_TYPE"), "Anonymous": SimUnion({"uint8": SimTypeChar(label="Byte"), "uint16": SimTypeShort(signed=False, label="UInt16"), "uint32": SimTypeInt(signed=False, label="UInt32"), "uint64": SimTypePointer(SimTypeLongLong(signed=False, label="UInt64"), offset=0), "int8": SimTypeChar(label="SByte"), "int16": SimTypeShort(signed=True, label="Int16"), "int32": SimTypeInt(signed=True, label="Int32"), "int64": SimTypePointer(SimTypeLongLong(signed=True, label="Int64"), offset=0), "float32": SimTypeFloat(size=32), "double64": SimTypePointer(SimTypeFloat(size=64), offset=0), "byteArray16": SimTypePointer(SimStruct({"byteArray16": SimTypeFixedSizeArray(SimTypeChar(label="Byte"), 16)}, name="FWP_BYTE_ARRAY16", pack=False, align=None), offset=0), "byteBlob": SimTypePointer(SimStruct({"size": SimTypeInt(signed=False, label="UInt32"), "data": SimTypePointer(SimTypeChar(label="Byte"), offset=0)}, name="FWP_BYTE_BLOB", pack=False, align=None), offset=0), "sid": SimTypePointer(SimTypeBottom(label="SID"), offset=0), "sd": SimTypePointer(SimStruct({"size": SimTypeInt(signed=False, label="UInt32"), "data": SimTypePointer(SimTypeChar(label="Byte"), offset=0)}, name="FWP_BYTE_BLOB", pack=False, align=None), offset=0), "tokenInformation": SimTypePointer(SimStruct({"sidCount": SimTypeInt(signed=False, label="UInt32"), "sids": SimTypePointer(SimTypeBottom(label="SID_AND_ATTRIBUTES"), offset=0), "restrictedSidCount": SimTypeInt(signed=False, label="UInt32"), "restrictedSids": SimTypePointer(SimTypeBottom(label="SID_AND_ATTRIBUTES"), offset=0)}, name="FWP_TOKEN_INFORMATION", pack=False, align=None), offset=0), "tokenAccessInformation": SimTypePointer(SimStruct({"size": SimTypeInt(signed=False, label="UInt32"), "data": SimTypePointer(SimTypeChar(label="Byte"), offset=0)}, name="FWP_BYTE_BLOB", pack=False, align=None), offset=0), "unicodeString": SimTypePointer(SimTypeChar(label="Char"), offset=0), "byteArray6": SimTypePointer(SimStruct({"byteArray6": SimTypeFixedSizeArray(SimTypeChar(label="Byte"), 6)}, name="FWP_BYTE_ARRAY6", pack=False, align=None), offset=0), "bitmapArray64": SimTypePointer(SimStruct({"bitmapArray64": SimTypeFixedSizeArray(SimTypeChar(label="Byte"), 8)}, name="FWP_BITMAP_ARRAY64_", pack=False, align=None), offset=0)}, name="<anon>", label="None")}, name="FWP_VALUE0", pack=False, align=None), "valueHigh": SimStruct({"type": SimTypeInt(signed=False, label="FWP_DATA_TYPE"), "Anonymous": SimUnion({"uint8": SimTypeChar(label="Byte"), "uint16": SimTypeShort(signed=False, label="UInt16"), "uint32": SimTypeInt(signed=False, label="UInt32"), "uint64": SimTypePointer(SimTypeLongLong(signed=False, label="UInt64"), offset=0), "int8": SimTypeChar(label="SByte"), "int16": SimTypeShort(signed=True, label="Int16"), "int32": SimTypeInt(signed=True, label="Int32"), "int64": SimTypePointer(SimTypeLongLong(signed=True, label="Int64"), offset=0), "float32": SimTypeFloat(size=32), "double64": SimTypePointer(SimTypeFloat(size=64), offset=0), "byteArray16": SimTypePointer(SimStruct({"byteArray16": SimTypeFixedSizeArray(SimTypeChar(label="Byte"), 16)}, name="FWP_BYTE_ARRAY16", pack=False, align=None), offset=0), "byteBlob": SimTypePointer(SimStruct({"size": SimTypeInt(signed=False, label="UInt32"), "data": SimTypePointer(SimTypeChar(label="Byte"), offset=0)}, name="FWP_BYTE_BLOB", pack=False, align=None), offset=0), "sid": SimTypePointer(SimTypeBottom(label="SID"), offset=0), "sd": SimTypePointer(SimStruct({"size": SimTypeInt(signed=False, label="UInt32"), "data": SimTypePointer(SimTypeChar(label="Byte"), offset=0)}, name="FWP_BYTE_BLOB", pack=False, align=None), offset=0), "tokenInformation": SimTypePointer(SimStruct({"sidCount": SimTypeInt(signed=False, label="UInt32"), "sids": SimTypePointer(SimTypeBottom(label="SID_AND_ATTRIBUTES"), offset=0), "restrictedSidCount": SimTypeInt(signed=False, label="UInt32"), "restrictedSids": SimTypePointer(SimTypeBottom(label="SID_AND_ATTRIBUTES"), offset=0)}, name="FWP_TOKEN_INFORMATION", pack=False, align=None), offset=0), "tokenAccessInformation": SimTypePointer(SimStruct({"size": SimTypeInt(signed=False, label="UInt32"), "data": SimTypePointer(SimTypeChar(label="Byte"), offset=0)}, name="FWP_BYTE_BLOB", pack=False, align=None), offset=0), "unicodeString": SimTypePointer(SimTypeChar(label="Char"), offset=0), "byteArray6": SimTypePointer(SimStruct({"byteArray6": SimTypeFixedSizeArray(SimTypeChar(label="Byte"), 6)}, name="FWP_BYTE_ARRAY6", pack=False, align=None), offset=0), "bitmapArray64": SimTypePointer(SimStruct({"bitmapArray64": SimTypeFixedSizeArray(SimTypeChar(label="Byte"), 8)}, name="FWP_BITMAP_ARRAY64_", pack=False, align=None), offset=0)}, name="<anon>", label="None")}, name="FWP_VALUE0", pack=False, align=None)}, name="FWP_RANGE0", pack=False, align=None), offset=0)}, name="<anon>", label="None")}, name="FWP_CONDITION_VALUE0", pack=False, align=None)}, name="<anon>", label="None"), "qmFilterId": SimTypeLongLong(signed=False, label="UInt64")}, name="FWPM_NET_EVENT_IKEEXT_QM_FAILURE0", pack=False, align=None), offset=0), "ikeEmFailure": SimTypePointer(SimStruct({"failureErrorCode": SimTypeInt(signed=False, label="UInt32"), "failurePoint": SimTypeInt(signed=False, label="IPSEC_FAILURE_POINT"), "flags": SimTypeInt(signed=False, label="UInt32"), "emState": SimTypeInt(signed=False, label="IKEEXT_EM_SA_STATE"), "saRole": SimTypeInt(signed=False, label="IKEEXT_SA_ROLE"), "emAuthMethod": SimTypeInt(signed=False, label="IKEEXT_AUTHENTICATION_METHOD_TYPE"), "endCertHash": SimTypeFixedSizeArray(SimTypeChar(label="Byte"), 20), "mmId": SimTypeLongLong(signed=False, label="UInt64"), "qmFilterId": SimTypeLongLong(signed=False, label="UInt64"), "localPrincipalNameForAuth": SimTypePointer(SimTypeChar(label="Char"), offset=0), "remotePrincipalNameForAuth": SimTypePointer(SimTypeChar(label="Char"), offset=0), "numLocalPrincipalGroupSids": SimTypeInt(signed=False, label="UInt32"), "localPrincipalGroupSids": SimTypePointer(SimTypePointer(SimTypeChar(label="Char"), offset=0), offset=0), "numRemotePrincipalGroupSids": SimTypeInt(signed=False, label="UInt32"), "remotePrincipalGroupSids": SimTypePointer(SimTypePointer(SimTypeChar(label="Char"), offset=0), offset=0), "saTrafficType": SimTypeInt(signed=False, label="IPSEC_TRAFFIC_TYPE")}, name="FWPM_NET_EVENT_IKEEXT_EM_FAILURE1", pack=False, align=None), offset=0), "classifyDrop": SimTypePointer(SimStruct({"filterId": SimTypeLongLong(signed=False, label="UInt64"), "layerId": SimTypeShort(signed=False, label="UInt16"), "reauthReason": SimTypeInt(signed=False, label="UInt32"), "originalProfile": SimTypeInt(signed=False, label="UInt32"), "currentProfile": SimTypeInt(signed=False, label="UInt32"), "msFwpDirection": SimTypeInt(signed=False, label="UInt32"), "isLoopback": SimTypeInt(signed=True, label="Int32"), "vSwitchId": SimStruct({"size": SimTypeInt(signed=False, label="UInt32"), "data": SimTypePointer(SimTypeChar(label="Byte"), offset=0)}, name="FWP_BYTE_BLOB", pack=False, align=None), "vSwitchSourcePort": SimTypeInt(signed=False, label="UInt32"), "vSwitchDestinationPort": SimTypeInt(signed=False, label="UInt32")}, name="FWPM_NET_EVENT_CLASSIFY_DROP2", pack=False, align=None), offset=0), "ipsecDrop": SimTypePointer(SimStruct({"failureStatus": SimTypeInt(signed=True, label="Int32"), "direction": SimTypeInt(signed=False, label="FWP_DIRECTION"), "spi": SimTypeInt(signed=False, label="UInt32"), "filterId": SimTypeLongLong(signed=False, label="UInt64"), "layerId": SimTypeShort(signed=False, label="UInt16")}, name="FWPM_NET_EVENT_IPSEC_KERNEL_DROP0", pack=False, align=None), offset=0), "idpDrop": SimTypePointer(SimStruct({"ipVersion": SimTypeInt(signed=False, label="FWP_IP_VERSION"), "Anonymous1": SimUnion({"publicHostV4Addr": SimTypeInt(signed=False, label="UInt32"), "publicHostV6Addr": SimTypeFixedSizeArray(SimTypeChar(label="Byte"), 16)}, name="<anon>", label="None"), "Anonymous2": SimUnion({"internalHostV4Addr": SimTypeInt(signed=False, label="UInt32"), "internalHostV6Addr": SimTypeFixedSizeArray(SimTypeChar(label="Byte"), 16)}, name="<anon>", label="None"), "failureStatus": SimTypeInt(signed=True, label="Int32"), "direction": SimTypeInt(signed=False, label="FWP_DIRECTION")}, name="FWPM_NET_EVENT_IPSEC_DOSP_DROP0", pack=False, align=None), offset=0), "classifyAllow": SimTypePointer(SimStruct({"filterId": SimTypeLongLong(signed=False, label="UInt64"), "layerId": SimTypeShort(signed=False, label="UInt16"), "reauthReason": SimTypeInt(signed=False, label="UInt32"), "originalProfile": SimTypeInt(signed=False, label="UInt32"), "currentProfile": SimTypeInt(signed=False, label="UInt32"), "msFwpDirection": SimTypeInt(signed=False, label="UInt32"), "isLoopback": SimTypeInt(signed=True, label="Int32")}, name="FWPM_NET_EVENT_CLASSIFY_ALLOW0", pack=False, align=None), offset=0), "capabilityDrop": SimTypePointer(SimStruct({"networkCapabilityId": SimTypeInt(signed=False, label="FWPM_APPC_NETWORK_CAPABILITY_TYPE"), "filterId": SimTypeLongLong(signed=False, label="UInt64"), "isLoopback": SimTypeInt(signed=True, label="Int32")}, name="FWPM_NET_EVENT_CAPABILITY_DROP0", pack=False, align=None), offset=0), "capabilityAllow": SimTypePointer(SimStruct({"networkCapabilityId": SimTypeInt(signed=False, label="FWPM_APPC_NETWORK_CAPABILITY_TYPE"), "filterId": SimTypeLongLong(signed=False, label="UInt64"), "isLoopback": SimTypeInt(signed=True, label="Int32")}, name="FWPM_NET_EVENT_CAPABILITY_ALLOW0", pack=False, align=None), offset=0), "classifyDropMac": SimTypePointer(SimStruct({"localMacAddr": SimStruct({"byteArray6": SimTypeFixedSizeArray(SimTypeChar(label="Byte"), 6)}, name="FWP_BYTE_ARRAY6", pack=False, align=None), "remoteMacAddr": SimStruct({"byteArray6": SimTypeFixedSizeArray(SimTypeChar(label="Byte"), 6)}, name="FWP_BYTE_ARRAY6", pack=False, align=None), "mediaType": SimTypeInt(signed=False, label="UInt32"), "ifType": SimTypeInt(signed=False, label="UInt32"), "etherType": SimTypeShort(signed=False, label="UInt16"), "ndisPortNumber": SimTypeInt(signed=False, label="UInt32"), "reserved": SimTypeInt(signed=False, label="UInt32"), "vlanTag": SimTypeShort(signed=False, label="UInt16"), "ifLuid": SimTypeLongLong(signed=False, label="UInt64"), "filterId": SimTypeLongLong(signed=False, label="UInt64"), "layerId": SimTypeShort(signed=False, label="UInt16"), "reauthReason": SimTypeInt(signed=False, label="UInt32"), "originalProfile": SimTypeInt(signed=False, label="UInt32"), "currentProfile": SimTypeInt(signed=False, label="UInt32"), "msFwpDirection": SimTypeInt(signed=False, label="UInt32"), "isLoopback": SimTypeInt(signed=True, label="Int32"), "vSwitchId": SimStruct({"size": SimTypeInt(signed=False, label="UInt32"), "data": SimTypePointer(SimTypeChar(label="Byte"), offset=0)}, name="FWP_BYTE_BLOB", pack=False, align=None), "vSwitchSourcePort": SimTypeInt(signed=False, label="UInt32"), "vSwitchDestinationPort": SimTypeInt(signed=False, label="UInt32")}, name="FWPM_NET_EVENT_CLASSIFY_DROP_MAC0", pack=False, align=None), offset=0)}, name="<anon>", label="None")}, name="FWPM_NET_EVENT3", pack=False, align=None), offset=0)], SimTypeBottom(label="Void"), arg_names=["context", "event"]), offset=0), SimTypePointer(SimTypeBottom(label="Void"), offset=0), SimTypePointer(SimTypePointer(SimTypeInt(signed=True, label="Int"), label="IntPtr", offset=0), offset=0)], SimTypeInt(signed=False, label="UInt32"), arg_names=["engineHandle", "subscription", "callback", "context", "eventsHandle"]),
        # 
        'FwpmNetEventSubscribe3': SimTypeFunction([SimTypePointer(SimTypeInt(signed=True, label="Int"), label="IntPtr", offset=0), SimTypePointer(SimStruct({"enumTemplate": SimTypePointer(SimStruct({"startTime": SimStruct({"dwLowDateTime": SimTypeInt(signed=False, label="UInt32"), "dwHighDateTime": SimTypeInt(signed=False, label="UInt32")}, name="FILETIME", pack=False, align=None), "endTime": SimStruct({"dwLowDateTime": SimTypeInt(signed=False, label="UInt32"), "dwHighDateTime": SimTypeInt(signed=False, label="UInt32")}, name="FILETIME", pack=False, align=None), "numFilterConditions": SimTypeInt(signed=False, label="UInt32"), "filterCondition": SimTypePointer(SimStruct({"fieldKey": SimTypeBottom(label="Guid"), "matchType": SimTypeInt(signed=False, label="FWP_MATCH_TYPE"), "conditionValue": SimStruct({"type": SimTypeInt(signed=False, label="FWP_DATA_TYPE"), "Anonymous": SimUnion({"uint8": SimTypeChar(label="Byte"), "uint16": SimTypeShort(signed=False, label="UInt16"), "uint32": SimTypeInt(signed=False, label="UInt32"), "uint64": SimTypePointer(SimTypeLongLong(signed=False, label="UInt64"), offset=0), "int8": SimTypeChar(label="SByte"), "int16": SimTypeShort(signed=True, label="Int16"), "int32": SimTypeInt(signed=True, label="Int32"), "int64": SimTypePointer(SimTypeLongLong(signed=True, label="Int64"), offset=0), "float32": SimTypeFloat(size=32), "double64": SimTypePointer(SimTypeFloat(size=64), offset=0), "byteArray16": SimTypePointer(SimStruct({"byteArray16": SimTypeFixedSizeArray(SimTypeChar(label="Byte"), 16)}, name="FWP_BYTE_ARRAY16", pack=False, align=None), offset=0), "byteBlob": SimTypePointer(SimStruct({"size": SimTypeInt(signed=False, label="UInt32"), "data": SimTypePointer(SimTypeChar(label="Byte"), offset=0)}, name="FWP_BYTE_BLOB", pack=False, align=None), offset=0), "sid": SimTypePointer(SimTypeBottom(label="SID"), offset=0), "sd": SimTypePointer(SimStruct({"size": SimTypeInt(signed=False, label="UInt32"), "data": SimTypePointer(SimTypeChar(label="Byte"), offset=0)}, name="FWP_BYTE_BLOB", pack=False, align=None), offset=0), "tokenInformation": SimTypePointer(SimStruct({"sidCount": SimTypeInt(signed=False, label="UInt32"), "sids": SimTypePointer(SimTypeBottom(label="SID_AND_ATTRIBUTES"), offset=0), "restrictedSidCount": SimTypeInt(signed=False, label="UInt32"), "restrictedSids": SimTypePointer(SimTypeBottom(label="SID_AND_ATTRIBUTES"), offset=0)}, name="FWP_TOKEN_INFORMATION", pack=False, align=None), offset=0), "tokenAccessInformation": SimTypePointer(SimStruct({"size": SimTypeInt(signed=False, label="UInt32"), "data": SimTypePointer(SimTypeChar(label="Byte"), offset=0)}, name="FWP_BYTE_BLOB", pack=False, align=None), offset=0), "unicodeString": SimTypePointer(SimTypeChar(label="Char"), offset=0), "byteArray6": SimTypePointer(SimStruct({"byteArray6": SimTypeFixedSizeArray(SimTypeChar(label="Byte"), 6)}, name="FWP_BYTE_ARRAY6", pack=False, align=None), offset=0), "bitmapArray64": SimTypePointer(SimStruct({"bitmapArray64": SimTypeFixedSizeArray(SimTypeChar(label="Byte"), 8)}, name="FWP_BITMAP_ARRAY64_", pack=False, align=None), offset=0), "v4AddrMask": SimTypePointer(SimStruct({"addr": SimTypeInt(signed=False, label="UInt32"), "mask": SimTypeInt(signed=False, label="UInt32")}, name="FWP_V4_ADDR_AND_MASK", pack=False, align=None), offset=0), "v6AddrMask": SimTypePointer(SimStruct({"addr": SimTypeFixedSizeArray(SimTypeChar(label="Byte"), 16), "prefixLength": SimTypeChar(label="Byte")}, name="FWP_V6_ADDR_AND_MASK", pack=False, align=None), offset=0), "rangeValue": SimTypePointer(SimStruct({"valueLow": SimStruct({"type": SimTypeInt(signed=False, label="FWP_DATA_TYPE"), "Anonymous": SimUnion({"uint8": SimTypeChar(label="Byte"), "uint16": SimTypeShort(signed=False, label="UInt16"), "uint32": SimTypeInt(signed=False, label="UInt32"), "uint64": SimTypePointer(SimTypeLongLong(signed=False, label="UInt64"), offset=0), "int8": SimTypeChar(label="SByte"), "int16": SimTypeShort(signed=True, label="Int16"), "int32": SimTypeInt(signed=True, label="Int32"), "int64": SimTypePointer(SimTypeLongLong(signed=True, label="Int64"), offset=0), "float32": SimTypeFloat(size=32), "double64": SimTypePointer(SimTypeFloat(size=64), offset=0), "byteArray16": SimTypePointer(SimStruct({"byteArray16": SimTypeFixedSizeArray(SimTypeChar(label="Byte"), 16)}, name="FWP_BYTE_ARRAY16", pack=False, align=None), offset=0), "byteBlob": SimTypePointer(SimStruct({"size": SimTypeInt(signed=False, label="UInt32"), "data": SimTypePointer(SimTypeChar(label="Byte"), offset=0)}, name="FWP_BYTE_BLOB", pack=False, align=None), offset=0), "sid": SimTypePointer(SimTypeBottom(label="SID"), offset=0), "sd": SimTypePointer(SimStruct({"size": SimTypeInt(signed=False, label="UInt32"), "data": SimTypePointer(SimTypeChar(label="Byte"), offset=0)}, name="FWP_BYTE_BLOB", pack=False, align=None), offset=0), "tokenInformation": SimTypePointer(SimStruct({"sidCount": SimTypeInt(signed=False, label="UInt32"), "sids": SimTypePointer(SimTypeBottom(label="SID_AND_ATTRIBUTES"), offset=0), "restrictedSidCount": SimTypeInt(signed=False, label="UInt32"), "restrictedSids": SimTypePointer(SimTypeBottom(label="SID_AND_ATTRIBUTES"), offset=0)}, name="FWP_TOKEN_INFORMATION", pack=False, align=None), offset=0), "tokenAccessInformation": SimTypePointer(SimStruct({"size": SimTypeInt(signed=False, label="UInt32"), "data": SimTypePointer(SimTypeChar(label="Byte"), offset=0)}, name="FWP_BYTE_BLOB", pack=False, align=None), offset=0), "unicodeString": SimTypePointer(SimTypeChar(label="Char"), offset=0), "byteArray6": SimTypePointer(SimStruct({"byteArray6": SimTypeFixedSizeArray(SimTypeChar(label="Byte"), 6)}, name="FWP_BYTE_ARRAY6", pack=False, align=None), offset=0), "bitmapArray64": SimTypePointer(SimStruct({"bitmapArray64": SimTypeFixedSizeArray(SimTypeChar(label="Byte"), 8)}, name="FWP_BITMAP_ARRAY64_", pack=False, align=None), offset=0)}, name="<anon>", label="None")}, name="FWP_VALUE0", pack=False, align=None), "valueHigh": SimStruct({"type": SimTypeInt(signed=False, label="FWP_DATA_TYPE"), "Anonymous": SimUnion({"uint8": SimTypeChar(label="Byte"), "uint16": SimTypeShort(signed=False, label="UInt16"), "uint32": SimTypeInt(signed=False, label="UInt32"), "uint64": SimTypePointer(SimTypeLongLong(signed=False, label="UInt64"), offset=0), "int8": SimTypeChar(label="SByte"), "int16": SimTypeShort(signed=True, label="Int16"), "int32": SimTypeInt(signed=True, label="Int32"), "int64": SimTypePointer(SimTypeLongLong(signed=True, label="Int64"), offset=0), "float32": SimTypeFloat(size=32), "double64": SimTypePointer(SimTypeFloat(size=64), offset=0), "byteArray16": SimTypePointer(SimStruct({"byteArray16": SimTypeFixedSizeArray(SimTypeChar(label="Byte"), 16)}, name="FWP_BYTE_ARRAY16", pack=False, align=None), offset=0), "byteBlob": SimTypePointer(SimStruct({"size": SimTypeInt(signed=False, label="UInt32"), "data": SimTypePointer(SimTypeChar(label="Byte"), offset=0)}, name="FWP_BYTE_BLOB", pack=False, align=None), offset=0), "sid": SimTypePointer(SimTypeBottom(label="SID"), offset=0), "sd": SimTypePointer(SimStruct({"size": SimTypeInt(signed=False, label="UInt32"), "data": SimTypePointer(SimTypeChar(label="Byte"), offset=0)}, name="FWP_BYTE_BLOB", pack=False, align=None), offset=0), "tokenInformation": SimTypePointer(SimStruct({"sidCount": SimTypeInt(signed=False, label="UInt32"), "sids": SimTypePointer(SimTypeBottom(label="SID_AND_ATTRIBUTES"), offset=0), "restrictedSidCount": SimTypeInt(signed=False, label="UInt32"), "restrictedSids": SimTypePointer(SimTypeBottom(label="SID_AND_ATTRIBUTES"), offset=0)}, name="FWP_TOKEN_INFORMATION", pack=False, align=None), offset=0), "tokenAccessInformation": SimTypePointer(SimStruct({"size": SimTypeInt(signed=False, label="UInt32"), "data": SimTypePointer(SimTypeChar(label="Byte"), offset=0)}, name="FWP_BYTE_BLOB", pack=False, align=None), offset=0), "unicodeString": SimTypePointer(SimTypeChar(label="Char"), offset=0), "byteArray6": SimTypePointer(SimStruct({"byteArray6": SimTypeFixedSizeArray(SimTypeChar(label="Byte"), 6)}, name="FWP_BYTE_ARRAY6", pack=False, align=None), offset=0), "bitmapArray64": SimTypePointer(SimStruct({"bitmapArray64": SimTypeFixedSizeArray(SimTypeChar(label="Byte"), 8)}, name="FWP_BITMAP_ARRAY64_", pack=False, align=None), offset=0)}, name="<anon>", label="None")}, name="FWP_VALUE0", pack=False, align=None)}, name="FWP_RANGE0", pack=False, align=None), offset=0)}, name="<anon>", label="None")}, name="FWP_CONDITION_VALUE0", pack=False, align=None)}, name="FWPM_FILTER_CONDITION0", pack=False, align=None), offset=0)}, name="FWPM_NET_EVENT_ENUM_TEMPLATE0", pack=False, align=None), offset=0), "flags": SimTypeInt(signed=False, label="UInt32"), "sessionKey": SimTypeBottom(label="Guid")}, name="FWPM_NET_EVENT_SUBSCRIPTION0", pack=False, align=None), offset=0), SimTypePointer(SimTypeFunction([SimTypePointer(SimTypeBottom(label="Void"), offset=0), SimTypePointer(SimStruct({"header": SimStruct({"timeStamp": SimStruct({"dwLowDateTime": SimTypeInt(signed=False, label="UInt32"), "dwHighDateTime": SimTypeInt(signed=False, label="UInt32")}, name="FILETIME", pack=False, align=None), "flags": SimTypeInt(signed=False, label="UInt32"), "ipVersion": SimTypeInt(signed=False, label="FWP_IP_VERSION"), "ipProtocol": SimTypeChar(label="Byte"), "Anonymous1": SimUnion({"localAddrV4": SimTypeInt(signed=False, label="UInt32"), "localAddrV6": SimStruct({"byteArray16": SimTypeFixedSizeArray(SimTypeChar(label="Byte"), 16)}, name="FWP_BYTE_ARRAY16", pack=False, align=None)}, name="<anon>", label="None"), "Anonymous2": SimUnion({"remoteAddrV4": SimTypeInt(signed=False, label="UInt32"), "remoteAddrV6": SimStruct({"byteArray16": SimTypeFixedSizeArray(SimTypeChar(label="Byte"), 16)}, name="FWP_BYTE_ARRAY16", pack=False, align=None)}, name="<anon>", label="None"), "localPort": SimTypeShort(signed=False, label="UInt16"), "remotePort": SimTypeShort(signed=False, label="UInt16"), "scopeId": SimTypeInt(signed=False, label="UInt32"), "appId": SimStruct({"size": SimTypeInt(signed=False, label="UInt32"), "data": SimTypePointer(SimTypeChar(label="Byte"), offset=0)}, name="FWP_BYTE_BLOB", pack=False, align=None), "userId": SimTypePointer(SimTypeBottom(label="SID"), offset=0), "addressFamily": SimTypeInt(signed=False, label="FWP_AF"), "packageSid": SimTypePointer(SimTypeBottom(label="SID"), offset=0), "enterpriseId": SimTypePointer(SimTypeChar(label="Char"), offset=0), "policyFlags": SimTypeLongLong(signed=False, label="UInt64"), "effectiveName": SimStruct({"size": SimTypeInt(signed=False, label="UInt32"), "data": SimTypePointer(SimTypeChar(label="Byte"), offset=0)}, name="FWP_BYTE_BLOB", pack=False, align=None)}, name="FWPM_NET_EVENT_HEADER3", pack=False, align=None), "type": SimTypeInt(signed=False, label="FWPM_NET_EVENT_TYPE"), "Anonymous": SimUnion({"ikeMmFailure": SimTypePointer(SimStruct({"failureErrorCode": SimTypeInt(signed=False, label="UInt32"), "failurePoint": SimTypeInt(signed=False, label="IPSEC_FAILURE_POINT"), "flags": SimTypeInt(signed=False, label="UInt32"), "keyingModuleType": SimTypeInt(signed=False, label="IKEEXT_KEY_MODULE_TYPE"), "mmState": SimTypeInt(signed=False, label="IKEEXT_MM_SA_STATE"), "saRole": SimTypeInt(signed=False, label="IKEEXT_SA_ROLE"), "mmAuthMethod": SimTypeInt(signed=False, label="IKEEXT_AUTHENTICATION_METHOD_TYPE"), "endCertHash": SimTypeFixedSizeArray(SimTypeChar(label="Byte"), 20), "mmId": SimTypeLongLong(signed=False, label="UInt64"), "mmFilterId": SimTypeLongLong(signed=False, label="UInt64"), "localPrincipalNameForAuth": SimTypePointer(SimTypeChar(label="Char"), offset=0), "remotePrincipalNameForAuth": SimTypePointer(SimTypeChar(label="Char"), offset=0), "numLocalPrincipalGroupSids": SimTypeInt(signed=False, label="UInt32"), "localPrincipalGroupSids": SimTypePointer(SimTypePointer(SimTypeChar(label="Char"), offset=0), offset=0), "numRemotePrincipalGroupSids": SimTypeInt(signed=False, label="UInt32"), "remotePrincipalGroupSids": SimTypePointer(SimTypePointer(SimTypeChar(label="Char"), offset=0), offset=0), "providerContextKey": SimTypePointer(SimTypeBottom(label="Guid"), offset=0)}, name="FWPM_NET_EVENT_IKEEXT_MM_FAILURE2_", pack=False, align=None), offset=0), "ikeQmFailure": SimTypePointer(SimStruct({"failureErrorCode": SimTypeInt(signed=False, label="UInt32"), "failurePoint": SimTypeInt(signed=False, label="IPSEC_FAILURE_POINT"), "keyingModuleType": SimTypeInt(signed=False, label="IKEEXT_KEY_MODULE_TYPE"), "qmState": SimTypeInt(signed=False, label="IKEEXT_QM_SA_STATE"), "saRole": SimTypeInt(signed=False, label="IKEEXT_SA_ROLE"), "saTrafficType": SimTypeInt(signed=False, label="IPSEC_TRAFFIC_TYPE"), "Anonymous1": SimUnion({"localSubNet": SimStruct({"type": SimTypeInt(signed=False, label="FWP_DATA_TYPE"), "Anonymous": SimUnion({"uint8": SimTypeChar(label="Byte"), "uint16": SimTypeShort(signed=False, label="UInt16"), "uint32": SimTypeInt(signed=False, label="UInt32"), "uint64": SimTypePointer(SimTypeLongLong(signed=False, label="UInt64"), offset=0), "int8": SimTypeChar(label="SByte"), "int16": SimTypeShort(signed=True, label="Int16"), "int32": SimTypeInt(signed=True, label="Int32"), "int64": SimTypePointer(SimTypeLongLong(signed=True, label="Int64"), offset=0), "float32": SimTypeFloat(size=32), "double64": SimTypePointer(SimTypeFloat(size=64), offset=0), "byteArray16": SimTypePointer(SimStruct({"byteArray16": SimTypeFixedSizeArray(SimTypeChar(label="Byte"), 16)}, name="FWP_BYTE_ARRAY16", pack=False, align=None), offset=0), "byteBlob": SimTypePointer(SimStruct({"size": SimTypeInt(signed=False, label="UInt32"), "data": SimTypePointer(SimTypeChar(label="Byte"), offset=0)}, name="FWP_BYTE_BLOB", pack=False, align=None), offset=0), "sid": SimTypePointer(SimTypeBottom(label="SID"), offset=0), "sd": SimTypePointer(SimStruct({"size": SimTypeInt(signed=False, label="UInt32"), "data": SimTypePointer(SimTypeChar(label="Byte"), offset=0)}, name="FWP_BYTE_BLOB", pack=False, align=None), offset=0), "tokenInformation": SimTypePointer(SimStruct({"sidCount": SimTypeInt(signed=False, label="UInt32"), "sids": SimTypePointer(SimTypeBottom(label="SID_AND_ATTRIBUTES"), offset=0), "restrictedSidCount": SimTypeInt(signed=False, label="UInt32"), "restrictedSids": SimTypePointer(SimTypeBottom(label="SID_AND_ATTRIBUTES"), offset=0)}, name="FWP_TOKEN_INFORMATION", pack=False, align=None), offset=0), "tokenAccessInformation": SimTypePointer(SimStruct({"size": SimTypeInt(signed=False, label="UInt32"), "data": SimTypePointer(SimTypeChar(label="Byte"), offset=0)}, name="FWP_BYTE_BLOB", pack=False, align=None), offset=0), "unicodeString": SimTypePointer(SimTypeChar(label="Char"), offset=0), "byteArray6": SimTypePointer(SimStruct({"byteArray6": SimTypeFixedSizeArray(SimTypeChar(label="Byte"), 6)}, name="FWP_BYTE_ARRAY6", pack=False, align=None), offset=0), "bitmapArray64": SimTypePointer(SimStruct({"bitmapArray64": SimTypeFixedSizeArray(SimTypeChar(label="Byte"), 8)}, name="FWP_BITMAP_ARRAY64_", pack=False, align=None), offset=0), "v4AddrMask": SimTypePointer(SimStruct({"addr": SimTypeInt(signed=False, label="UInt32"), "mask": SimTypeInt(signed=False, label="UInt32")}, name="FWP_V4_ADDR_AND_MASK", pack=False, align=None), offset=0), "v6AddrMask": SimTypePointer(SimStruct({"addr": SimTypeFixedSizeArray(SimTypeChar(label="Byte"), 16), "prefixLength": SimTypeChar(label="Byte")}, name="FWP_V6_ADDR_AND_MASK", pack=False, align=None), offset=0), "rangeValue": SimTypePointer(SimStruct({"valueLow": SimStruct({"type": SimTypeInt(signed=False, label="FWP_DATA_TYPE"), "Anonymous": SimUnion({"uint8": SimTypeChar(label="Byte"), "uint16": SimTypeShort(signed=False, label="UInt16"), "uint32": SimTypeInt(signed=False, label="UInt32"), "uint64": SimTypePointer(SimTypeLongLong(signed=False, label="UInt64"), offset=0), "int8": SimTypeChar(label="SByte"), "int16": SimTypeShort(signed=True, label="Int16"), "int32": SimTypeInt(signed=True, label="Int32"), "int64": SimTypePointer(SimTypeLongLong(signed=True, label="Int64"), offset=0), "float32": SimTypeFloat(size=32), "double64": SimTypePointer(SimTypeFloat(size=64), offset=0), "byteArray16": SimTypePointer(SimStruct({"byteArray16": SimTypeFixedSizeArray(SimTypeChar(label="Byte"), 16)}, name="FWP_BYTE_ARRAY16", pack=False, align=None), offset=0), "byteBlob": SimTypePointer(SimStruct({"size": SimTypeInt(signed=False, label="UInt32"), "data": SimTypePointer(SimTypeChar(label="Byte"), offset=0)}, name="FWP_BYTE_BLOB", pack=False, align=None), offset=0), "sid": SimTypePointer(SimTypeBottom(label="SID"), offset=0), "sd": SimTypePointer(SimStruct({"size": SimTypeInt(signed=False, label="UInt32"), "data": SimTypePointer(SimTypeChar(label="Byte"), offset=0)}, name="FWP_BYTE_BLOB", pack=False, align=None), offset=0), "tokenInformation": SimTypePointer(SimStruct({"sidCount": SimTypeInt(signed=False, label="UInt32"), "sids": SimTypePointer(SimTypeBottom(label="SID_AND_ATTRIBUTES"), offset=0), "restrictedSidCount": SimTypeInt(signed=False, label="UInt32"), "restrictedSids": SimTypePointer(SimTypeBottom(label="SID_AND_ATTRIBUTES"), offset=0)}, name="FWP_TOKEN_INFORMATION", pack=False, align=None), offset=0), "tokenAccessInformation": SimTypePointer(SimStruct({"size": SimTypeInt(signed=False, label="UInt32"), "data": SimTypePointer(SimTypeChar(label="Byte"), offset=0)}, name="FWP_BYTE_BLOB", pack=False, align=None), offset=0), "unicodeString": SimTypePointer(SimTypeChar(label="Char"), offset=0), "byteArray6": SimTypePointer(SimStruct({"byteArray6": SimTypeFixedSizeArray(SimTypeChar(label="Byte"), 6)}, name="FWP_BYTE_ARRAY6", pack=False, align=None), offset=0), "bitmapArray64": SimTypePointer(SimStruct({"bitmapArray64": SimTypeFixedSizeArray(SimTypeChar(label="Byte"), 8)}, name="FWP_BITMAP_ARRAY64_", pack=False, align=None), offset=0)}, name="<anon>", label="None")}, name="FWP_VALUE0", pack=False, align=None), "valueHigh": SimStruct({"type": SimTypeInt(signed=False, label="FWP_DATA_TYPE"), "Anonymous": SimUnion({"uint8": SimTypeChar(label="Byte"), "uint16": SimTypeShort(signed=False, label="UInt16"), "uint32": SimTypeInt(signed=False, label="UInt32"), "uint64": SimTypePointer(SimTypeLongLong(signed=False, label="UInt64"), offset=0), "int8": SimTypeChar(label="SByte"), "int16": SimTypeShort(signed=True, label="Int16"), "int32": SimTypeInt(signed=True, label="Int32"), "int64": SimTypePointer(SimTypeLongLong(signed=True, label="Int64"), offset=0), "float32": SimTypeFloat(size=32), "double64": SimTypePointer(SimTypeFloat(size=64), offset=0), "byteArray16": SimTypePointer(SimStruct({"byteArray16": SimTypeFixedSizeArray(SimTypeChar(label="Byte"), 16)}, name="FWP_BYTE_ARRAY16", pack=False, align=None), offset=0), "byteBlob": SimTypePointer(SimStruct({"size": SimTypeInt(signed=False, label="UInt32"), "data": SimTypePointer(SimTypeChar(label="Byte"), offset=0)}, name="FWP_BYTE_BLOB", pack=False, align=None), offset=0), "sid": SimTypePointer(SimTypeBottom(label="SID"), offset=0), "sd": SimTypePointer(SimStruct({"size": SimTypeInt(signed=False, label="UInt32"), "data": SimTypePointer(SimTypeChar(label="Byte"), offset=0)}, name="FWP_BYTE_BLOB", pack=False, align=None), offset=0), "tokenInformation": SimTypePointer(SimStruct({"sidCount": SimTypeInt(signed=False, label="UInt32"), "sids": SimTypePointer(SimTypeBottom(label="SID_AND_ATTRIBUTES"), offset=0), "restrictedSidCount": SimTypeInt(signed=False, label="UInt32"), "restrictedSids": SimTypePointer(SimTypeBottom(label="SID_AND_ATTRIBUTES"), offset=0)}, name="FWP_TOKEN_INFORMATION", pack=False, align=None), offset=0), "tokenAccessInformation": SimTypePointer(SimStruct({"size": SimTypeInt(signed=False, label="UInt32"), "data": SimTypePointer(SimTypeChar(label="Byte"), offset=0)}, name="FWP_BYTE_BLOB", pack=False, align=None), offset=0), "unicodeString": SimTypePointer(SimTypeChar(label="Char"), offset=0), "byteArray6": SimTypePointer(SimStruct({"byteArray6": SimTypeFixedSizeArray(SimTypeChar(label="Byte"), 6)}, name="FWP_BYTE_ARRAY6", pack=False, align=None), offset=0), "bitmapArray64": SimTypePointer(SimStruct({"bitmapArray64": SimTypeFixedSizeArray(SimTypeChar(label="Byte"), 8)}, name="FWP_BITMAP_ARRAY64_", pack=False, align=None), offset=0)}, name="<anon>", label="None")}, name="FWP_VALUE0", pack=False, align=None)}, name="FWP_RANGE0", pack=False, align=None), offset=0)}, name="<anon>", label="None")}, name="FWP_CONDITION_VALUE0", pack=False, align=None)}, name="<anon>", label="None"), "Anonymous2": SimUnion({"remoteSubNet": SimStruct({"type": SimTypeInt(signed=False, label="FWP_DATA_TYPE"), "Anonymous": SimUnion({"uint8": SimTypeChar(label="Byte"), "uint16": SimTypeShort(signed=False, label="UInt16"), "uint32": SimTypeInt(signed=False, label="UInt32"), "uint64": SimTypePointer(SimTypeLongLong(signed=False, label="UInt64"), offset=0), "int8": SimTypeChar(label="SByte"), "int16": SimTypeShort(signed=True, label="Int16"), "int32": SimTypeInt(signed=True, label="Int32"), "int64": SimTypePointer(SimTypeLongLong(signed=True, label="Int64"), offset=0), "float32": SimTypeFloat(size=32), "double64": SimTypePointer(SimTypeFloat(size=64), offset=0), "byteArray16": SimTypePointer(SimStruct({"byteArray16": SimTypeFixedSizeArray(SimTypeChar(label="Byte"), 16)}, name="FWP_BYTE_ARRAY16", pack=False, align=None), offset=0), "byteBlob": SimTypePointer(SimStruct({"size": SimTypeInt(signed=False, label="UInt32"), "data": SimTypePointer(SimTypeChar(label="Byte"), offset=0)}, name="FWP_BYTE_BLOB", pack=False, align=None), offset=0), "sid": SimTypePointer(SimTypeBottom(label="SID"), offset=0), "sd": SimTypePointer(SimStruct({"size": SimTypeInt(signed=False, label="UInt32"), "data": SimTypePointer(SimTypeChar(label="Byte"), offset=0)}, name="FWP_BYTE_BLOB", pack=False, align=None), offset=0), "tokenInformation": SimTypePointer(SimStruct({"sidCount": SimTypeInt(signed=False, label="UInt32"), "sids": SimTypePointer(SimTypeBottom(label="SID_AND_ATTRIBUTES"), offset=0), "restrictedSidCount": SimTypeInt(signed=False, label="UInt32"), "restrictedSids": SimTypePointer(SimTypeBottom(label="SID_AND_ATTRIBUTES"), offset=0)}, name="FWP_TOKEN_INFORMATION", pack=False, align=None), offset=0), "tokenAccessInformation": SimTypePointer(SimStruct({"size": SimTypeInt(signed=False, label="UInt32"), "data": SimTypePointer(SimTypeChar(label="Byte"), offset=0)}, name="FWP_BYTE_BLOB", pack=False, align=None), offset=0), "unicodeString": SimTypePointer(SimTypeChar(label="Char"), offset=0), "byteArray6": SimTypePointer(SimStruct({"byteArray6": SimTypeFixedSizeArray(SimTypeChar(label="Byte"), 6)}, name="FWP_BYTE_ARRAY6", pack=False, align=None), offset=0), "bitmapArray64": SimTypePointer(SimStruct({"bitmapArray64": SimTypeFixedSizeArray(SimTypeChar(label="Byte"), 8)}, name="FWP_BITMAP_ARRAY64_", pack=False, align=None), offset=0), "v4AddrMask": SimTypePointer(SimStruct({"addr": SimTypeInt(signed=False, label="UInt32"), "mask": SimTypeInt(signed=False, label="UInt32")}, name="FWP_V4_ADDR_AND_MASK", pack=False, align=None), offset=0), "v6AddrMask": SimTypePointer(SimStruct({"addr": SimTypeFixedSizeArray(SimTypeChar(label="Byte"), 16), "prefixLength": SimTypeChar(label="Byte")}, name="FWP_V6_ADDR_AND_MASK", pack=False, align=None), offset=0), "rangeValue": SimTypePointer(SimStruct({"valueLow": SimStruct({"type": SimTypeInt(signed=False, label="FWP_DATA_TYPE"), "Anonymous": SimUnion({"uint8": SimTypeChar(label="Byte"), "uint16": SimTypeShort(signed=False, label="UInt16"), "uint32": SimTypeInt(signed=False, label="UInt32"), "uint64": SimTypePointer(SimTypeLongLong(signed=False, label="UInt64"), offset=0), "int8": SimTypeChar(label="SByte"), "int16": SimTypeShort(signed=True, label="Int16"), "int32": SimTypeInt(signed=True, label="Int32"), "int64": SimTypePointer(SimTypeLongLong(signed=True, label="Int64"), offset=0), "float32": SimTypeFloat(size=32), "double64": SimTypePointer(SimTypeFloat(size=64), offset=0), "byteArray16": SimTypePointer(SimStruct({"byteArray16": SimTypeFixedSizeArray(SimTypeChar(label="Byte"), 16)}, name="FWP_BYTE_ARRAY16", pack=False, align=None), offset=0), "byteBlob": SimTypePointer(SimStruct({"size": SimTypeInt(signed=False, label="UInt32"), "data": SimTypePointer(SimTypeChar(label="Byte"), offset=0)}, name="FWP_BYTE_BLOB", pack=False, align=None), offset=0), "sid": SimTypePointer(SimTypeBottom(label="SID"), offset=0), "sd": SimTypePointer(SimStruct({"size": SimTypeInt(signed=False, label="UInt32"), "data": SimTypePointer(SimTypeChar(label="Byte"), offset=0)}, name="FWP_BYTE_BLOB", pack=False, align=None), offset=0), "tokenInformation": SimTypePointer(SimStruct({"sidCount": SimTypeInt(signed=False, label="UInt32"), "sids": SimTypePointer(SimTypeBottom(label="SID_AND_ATTRIBUTES"), offset=0), "restrictedSidCount": SimTypeInt(signed=False, label="UInt32"), "restrictedSids": SimTypePointer(SimTypeBottom(label="SID_AND_ATTRIBUTES"), offset=0)}, name="FWP_TOKEN_INFORMATION", pack=False, align=None), offset=0), "tokenAccessInformation": SimTypePointer(SimStruct({"size": SimTypeInt(signed=False, label="UInt32"), "data": SimTypePointer(SimTypeChar(label="Byte"), offset=0)}, name="FWP_BYTE_BLOB", pack=False, align=None), offset=0), "unicodeString": SimTypePointer(SimTypeChar(label="Char"), offset=0), "byteArray6": SimTypePointer(SimStruct({"byteArray6": SimTypeFixedSizeArray(SimTypeChar(label="Byte"), 6)}, name="FWP_BYTE_ARRAY6", pack=False, align=None), offset=0), "bitmapArray64": SimTypePointer(SimStruct({"bitmapArray64": SimTypeFixedSizeArray(SimTypeChar(label="Byte"), 8)}, name="FWP_BITMAP_ARRAY64_", pack=False, align=None), offset=0)}, name="<anon>", label="None")}, name="FWP_VALUE0", pack=False, align=None), "valueHigh": SimStruct({"type": SimTypeInt(signed=False, label="FWP_DATA_TYPE"), "Anonymous": SimUnion({"uint8": SimTypeChar(label="Byte"), "uint16": SimTypeShort(signed=False, label="UInt16"), "uint32": SimTypeInt(signed=False, label="UInt32"), "uint64": SimTypePointer(SimTypeLongLong(signed=False, label="UInt64"), offset=0), "int8": SimTypeChar(label="SByte"), "int16": SimTypeShort(signed=True, label="Int16"), "int32": SimTypeInt(signed=True, label="Int32"), "int64": SimTypePointer(SimTypeLongLong(signed=True, label="Int64"), offset=0), "float32": SimTypeFloat(size=32), "double64": SimTypePointer(SimTypeFloat(size=64), offset=0), "byteArray16": SimTypePointer(SimStruct({"byteArray16": SimTypeFixedSizeArray(SimTypeChar(label="Byte"), 16)}, name="FWP_BYTE_ARRAY16", pack=False, align=None), offset=0), "byteBlob": SimTypePointer(SimStruct({"size": SimTypeInt(signed=False, label="UInt32"), "data": SimTypePointer(SimTypeChar(label="Byte"), offset=0)}, name="FWP_BYTE_BLOB", pack=False, align=None), offset=0), "sid": SimTypePointer(SimTypeBottom(label="SID"), offset=0), "sd": SimTypePointer(SimStruct({"size": SimTypeInt(signed=False, label="UInt32"), "data": SimTypePointer(SimTypeChar(label="Byte"), offset=0)}, name="FWP_BYTE_BLOB", pack=False, align=None), offset=0), "tokenInformation": SimTypePointer(SimStruct({"sidCount": SimTypeInt(signed=False, label="UInt32"), "sids": SimTypePointer(SimTypeBottom(label="SID_AND_ATTRIBUTES"), offset=0), "restrictedSidCount": SimTypeInt(signed=False, label="UInt32"), "restrictedSids": SimTypePointer(SimTypeBottom(label="SID_AND_ATTRIBUTES"), offset=0)}, name="FWP_TOKEN_INFORMATION", pack=False, align=None), offset=0), "tokenAccessInformation": SimTypePointer(SimStruct({"size": SimTypeInt(signed=False, label="UInt32"), "data": SimTypePointer(SimTypeChar(label="Byte"), offset=0)}, name="FWP_BYTE_BLOB", pack=False, align=None), offset=0), "unicodeString": SimTypePointer(SimTypeChar(label="Char"), offset=0), "byteArray6": SimTypePointer(SimStruct({"byteArray6": SimTypeFixedSizeArray(SimTypeChar(label="Byte"), 6)}, name="FWP_BYTE_ARRAY6", pack=False, align=None), offset=0), "bitmapArray64": SimTypePointer(SimStruct({"bitmapArray64": SimTypeFixedSizeArray(SimTypeChar(label="Byte"), 8)}, name="FWP_BITMAP_ARRAY64_", pack=False, align=None), offset=0)}, name="<anon>", label="None")}, name="FWP_VALUE0", pack=False, align=None)}, name="FWP_RANGE0", pack=False, align=None), offset=0)}, name="<anon>", label="None")}, name="FWP_CONDITION_VALUE0", pack=False, align=None)}, name="<anon>", label="None"), "qmFilterId": SimTypeLongLong(signed=False, label="UInt64"), "mmSaLuid": SimTypeLongLong(signed=False, label="UInt64"), "mmProviderContextKey": SimTypeBottom(label="Guid")}, name="FWPM_NET_EVENT_IKEEXT_QM_FAILURE1_", pack=False, align=None), offset=0), "ikeEmFailure": SimTypePointer(SimStruct({"failureErrorCode": SimTypeInt(signed=False, label="UInt32"), "failurePoint": SimTypeInt(signed=False, label="IPSEC_FAILURE_POINT"), "flags": SimTypeInt(signed=False, label="UInt32"), "emState": SimTypeInt(signed=False, label="IKEEXT_EM_SA_STATE"), "saRole": SimTypeInt(signed=False, label="IKEEXT_SA_ROLE"), "emAuthMethod": SimTypeInt(signed=False, label="IKEEXT_AUTHENTICATION_METHOD_TYPE"), "endCertHash": SimTypeFixedSizeArray(SimTypeChar(label="Byte"), 20), "mmId": SimTypeLongLong(signed=False, label="UInt64"), "qmFilterId": SimTypeLongLong(signed=False, label="UInt64"), "localPrincipalNameForAuth": SimTypePointer(SimTypeChar(label="Char"), offset=0), "remotePrincipalNameForAuth": SimTypePointer(SimTypeChar(label="Char"), offset=0), "numLocalPrincipalGroupSids": SimTypeInt(signed=False, label="UInt32"), "localPrincipalGroupSids": SimTypePointer(SimTypePointer(SimTypeChar(label="Char"), offset=0), offset=0), "numRemotePrincipalGroupSids": SimTypeInt(signed=False, label="UInt32"), "remotePrincipalGroupSids": SimTypePointer(SimTypePointer(SimTypeChar(label="Char"), offset=0), offset=0), "saTrafficType": SimTypeInt(signed=False, label="IPSEC_TRAFFIC_TYPE")}, name="FWPM_NET_EVENT_IKEEXT_EM_FAILURE1", pack=False, align=None), offset=0), "classifyDrop": SimTypePointer(SimStruct({"filterId": SimTypeLongLong(signed=False, label="UInt64"), "layerId": SimTypeShort(signed=False, label="UInt16"), "reauthReason": SimTypeInt(signed=False, label="UInt32"), "originalProfile": SimTypeInt(signed=False, label="UInt32"), "currentProfile": SimTypeInt(signed=False, label="UInt32"), "msFwpDirection": SimTypeInt(signed=False, label="UInt32"), "isLoopback": SimTypeInt(signed=True, label="Int32"), "vSwitchId": SimStruct({"size": SimTypeInt(signed=False, label="UInt32"), "data": SimTypePointer(SimTypeChar(label="Byte"), offset=0)}, name="FWP_BYTE_BLOB", pack=False, align=None), "vSwitchSourcePort": SimTypeInt(signed=False, label="UInt32"), "vSwitchDestinationPort": SimTypeInt(signed=False, label="UInt32")}, name="FWPM_NET_EVENT_CLASSIFY_DROP2", pack=False, align=None), offset=0), "ipsecDrop": SimTypePointer(SimStruct({"failureStatus": SimTypeInt(signed=True, label="Int32"), "direction": SimTypeInt(signed=False, label="FWP_DIRECTION"), "spi": SimTypeInt(signed=False, label="UInt32"), "filterId": SimTypeLongLong(signed=False, label="UInt64"), "layerId": SimTypeShort(signed=False, label="UInt16")}, name="FWPM_NET_EVENT_IPSEC_KERNEL_DROP0", pack=False, align=None), offset=0), "idpDrop": SimTypePointer(SimStruct({"ipVersion": SimTypeInt(signed=False, label="FWP_IP_VERSION"), "Anonymous1": SimUnion({"publicHostV4Addr": SimTypeInt(signed=False, label="UInt32"), "publicHostV6Addr": SimTypeFixedSizeArray(SimTypeChar(label="Byte"), 16)}, name="<anon>", label="None"), "Anonymous2": SimUnion({"internalHostV4Addr": SimTypeInt(signed=False, label="UInt32"), "internalHostV6Addr": SimTypeFixedSizeArray(SimTypeChar(label="Byte"), 16)}, name="<anon>", label="None"), "failureStatus": SimTypeInt(signed=True, label="Int32"), "direction": SimTypeInt(signed=False, label="FWP_DIRECTION")}, name="FWPM_NET_EVENT_IPSEC_DOSP_DROP0", pack=False, align=None), offset=0), "classifyAllow": SimTypePointer(SimStruct({"filterId": SimTypeLongLong(signed=False, label="UInt64"), "layerId": SimTypeShort(signed=False, label="UInt16"), "reauthReason": SimTypeInt(signed=False, label="UInt32"), "originalProfile": SimTypeInt(signed=False, label="UInt32"), "currentProfile": SimTypeInt(signed=False, label="UInt32"), "msFwpDirection": SimTypeInt(signed=False, label="UInt32"), "isLoopback": SimTypeInt(signed=True, label="Int32")}, name="FWPM_NET_EVENT_CLASSIFY_ALLOW0", pack=False, align=None), offset=0), "capabilityDrop": SimTypePointer(SimStruct({"networkCapabilityId": SimTypeInt(signed=False, label="FWPM_APPC_NETWORK_CAPABILITY_TYPE"), "filterId": SimTypeLongLong(signed=False, label="UInt64"), "isLoopback": SimTypeInt(signed=True, label="Int32")}, name="FWPM_NET_EVENT_CAPABILITY_DROP0", pack=False, align=None), offset=0), "capabilityAllow": SimTypePointer(SimStruct({"networkCapabilityId": SimTypeInt(signed=False, label="FWPM_APPC_NETWORK_CAPABILITY_TYPE"), "filterId": SimTypeLongLong(signed=False, label="UInt64"), "isLoopback": SimTypeInt(signed=True, label="Int32")}, name="FWPM_NET_EVENT_CAPABILITY_ALLOW0", pack=False, align=None), offset=0), "classifyDropMac": SimTypePointer(SimStruct({"localMacAddr": SimStruct({"byteArray6": SimTypeFixedSizeArray(SimTypeChar(label="Byte"), 6)}, name="FWP_BYTE_ARRAY6", pack=False, align=None), "remoteMacAddr": SimStruct({"byteArray6": SimTypeFixedSizeArray(SimTypeChar(label="Byte"), 6)}, name="FWP_BYTE_ARRAY6", pack=False, align=None), "mediaType": SimTypeInt(signed=False, label="UInt32"), "ifType": SimTypeInt(signed=False, label="UInt32"), "etherType": SimTypeShort(signed=False, label="UInt16"), "ndisPortNumber": SimTypeInt(signed=False, label="UInt32"), "reserved": SimTypeInt(signed=False, label="UInt32"), "vlanTag": SimTypeShort(signed=False, label="UInt16"), "ifLuid": SimTypeLongLong(signed=False, label="UInt64"), "filterId": SimTypeLongLong(signed=False, label="UInt64"), "layerId": SimTypeShort(signed=False, label="UInt16"), "reauthReason": SimTypeInt(signed=False, label="UInt32"), "originalProfile": SimTypeInt(signed=False, label="UInt32"), "currentProfile": SimTypeInt(signed=False, label="UInt32"), "msFwpDirection": SimTypeInt(signed=False, label="UInt32"), "isLoopback": SimTypeInt(signed=True, label="Int32"), "vSwitchId": SimStruct({"size": SimTypeInt(signed=False, label="UInt32"), "data": SimTypePointer(SimTypeChar(label="Byte"), offset=0)}, name="FWP_BYTE_BLOB", pack=False, align=None), "vSwitchSourcePort": SimTypeInt(signed=False, label="UInt32"), "vSwitchDestinationPort": SimTypeInt(signed=False, label="UInt32")}, name="FWPM_NET_EVENT_CLASSIFY_DROP_MAC0", pack=False, align=None), offset=0)}, name="<anon>", label="None")}, name="FWPM_NET_EVENT4_", pack=False, align=None), offset=0)], SimTypeBottom(label="Void"), arg_names=["context", "event"]), offset=0), SimTypePointer(SimTypeBottom(label="Void"), offset=0), SimTypePointer(SimTypePointer(SimTypeInt(signed=True, label="Int"), label="IntPtr", offset=0), offset=0)], SimTypeInt(signed=False, label="UInt32"), arg_names=["engineHandle", "subscription", "callback", "context", "eventsHandle"]),
        # 
        'FwpmNetEventSubscribe4': SimTypeFunction([SimTypePointer(SimTypeInt(signed=True, label="Int"), label="IntPtr", offset=0), SimTypePointer(SimStruct({"enumTemplate": SimTypePointer(SimStruct({"startTime": SimStruct({"dwLowDateTime": SimTypeInt(signed=False, label="UInt32"), "dwHighDateTime": SimTypeInt(signed=False, label="UInt32")}, name="FILETIME", pack=False, align=None), "endTime": SimStruct({"dwLowDateTime": SimTypeInt(signed=False, label="UInt32"), "dwHighDateTime": SimTypeInt(signed=False, label="UInt32")}, name="FILETIME", pack=False, align=None), "numFilterConditions": SimTypeInt(signed=False, label="UInt32"), "filterCondition": SimTypePointer(SimStruct({"fieldKey": SimTypeBottom(label="Guid"), "matchType": SimTypeInt(signed=False, label="FWP_MATCH_TYPE"), "conditionValue": SimStruct({"type": SimTypeInt(signed=False, label="FWP_DATA_TYPE"), "Anonymous": SimUnion({"uint8": SimTypeChar(label="Byte"), "uint16": SimTypeShort(signed=False, label="UInt16"), "uint32": SimTypeInt(signed=False, label="UInt32"), "uint64": SimTypePointer(SimTypeLongLong(signed=False, label="UInt64"), offset=0), "int8": SimTypeChar(label="SByte"), "int16": SimTypeShort(signed=True, label="Int16"), "int32": SimTypeInt(signed=True, label="Int32"), "int64": SimTypePointer(SimTypeLongLong(signed=True, label="Int64"), offset=0), "float32": SimTypeFloat(size=32), "double64": SimTypePointer(SimTypeFloat(size=64), offset=0), "byteArray16": SimTypePointer(SimStruct({"byteArray16": SimTypeFixedSizeArray(SimTypeChar(label="Byte"), 16)}, name="FWP_BYTE_ARRAY16", pack=False, align=None), offset=0), "byteBlob": SimTypePointer(SimStruct({"size": SimTypeInt(signed=False, label="UInt32"), "data": SimTypePointer(SimTypeChar(label="Byte"), offset=0)}, name="FWP_BYTE_BLOB", pack=False, align=None), offset=0), "sid": SimTypePointer(SimTypeBottom(label="SID"), offset=0), "sd": SimTypePointer(SimStruct({"size": SimTypeInt(signed=False, label="UInt32"), "data": SimTypePointer(SimTypeChar(label="Byte"), offset=0)}, name="FWP_BYTE_BLOB", pack=False, align=None), offset=0), "tokenInformation": SimTypePointer(SimStruct({"sidCount": SimTypeInt(signed=False, label="UInt32"), "sids": SimTypePointer(SimTypeBottom(label="SID_AND_ATTRIBUTES"), offset=0), "restrictedSidCount": SimTypeInt(signed=False, label="UInt32"), "restrictedSids": SimTypePointer(SimTypeBottom(label="SID_AND_ATTRIBUTES"), offset=0)}, name="FWP_TOKEN_INFORMATION", pack=False, align=None), offset=0), "tokenAccessInformation": SimTypePointer(SimStruct({"size": SimTypeInt(signed=False, label="UInt32"), "data": SimTypePointer(SimTypeChar(label="Byte"), offset=0)}, name="FWP_BYTE_BLOB", pack=False, align=None), offset=0), "unicodeString": SimTypePointer(SimTypeChar(label="Char"), offset=0), "byteArray6": SimTypePointer(SimStruct({"byteArray6": SimTypeFixedSizeArray(SimTypeChar(label="Byte"), 6)}, name="FWP_BYTE_ARRAY6", pack=False, align=None), offset=0), "bitmapArray64": SimTypePointer(SimStruct({"bitmapArray64": SimTypeFixedSizeArray(SimTypeChar(label="Byte"), 8)}, name="FWP_BITMAP_ARRAY64_", pack=False, align=None), offset=0), "v4AddrMask": SimTypePointer(SimStruct({"addr": SimTypeInt(signed=False, label="UInt32"), "mask": SimTypeInt(signed=False, label="UInt32")}, name="FWP_V4_ADDR_AND_MASK", pack=False, align=None), offset=0), "v6AddrMask": SimTypePointer(SimStruct({"addr": SimTypeFixedSizeArray(SimTypeChar(label="Byte"), 16), "prefixLength": SimTypeChar(label="Byte")}, name="FWP_V6_ADDR_AND_MASK", pack=False, align=None), offset=0), "rangeValue": SimTypePointer(SimStruct({"valueLow": SimStruct({"type": SimTypeInt(signed=False, label="FWP_DATA_TYPE"), "Anonymous": SimUnion({"uint8": SimTypeChar(label="Byte"), "uint16": SimTypeShort(signed=False, label="UInt16"), "uint32": SimTypeInt(signed=False, label="UInt32"), "uint64": SimTypePointer(SimTypeLongLong(signed=False, label="UInt64"), offset=0), "int8": SimTypeChar(label="SByte"), "int16": SimTypeShort(signed=True, label="Int16"), "int32": SimTypeInt(signed=True, label="Int32"), "int64": SimTypePointer(SimTypeLongLong(signed=True, label="Int64"), offset=0), "float32": SimTypeFloat(size=32), "double64": SimTypePointer(SimTypeFloat(size=64), offset=0), "byteArray16": SimTypePointer(SimStruct({"byteArray16": SimTypeFixedSizeArray(SimTypeChar(label="Byte"), 16)}, name="FWP_BYTE_ARRAY16", pack=False, align=None), offset=0), "byteBlob": SimTypePointer(SimStruct({"size": SimTypeInt(signed=False, label="UInt32"), "data": SimTypePointer(SimTypeChar(label="Byte"), offset=0)}, name="FWP_BYTE_BLOB", pack=False, align=None), offset=0), "sid": SimTypePointer(SimTypeBottom(label="SID"), offset=0), "sd": SimTypePointer(SimStruct({"size": SimTypeInt(signed=False, label="UInt32"), "data": SimTypePointer(SimTypeChar(label="Byte"), offset=0)}, name="FWP_BYTE_BLOB", pack=False, align=None), offset=0), "tokenInformation": SimTypePointer(SimStruct({"sidCount": SimTypeInt(signed=False, label="UInt32"), "sids": SimTypePointer(SimTypeBottom(label="SID_AND_ATTRIBUTES"), offset=0), "restrictedSidCount": SimTypeInt(signed=False, label="UInt32"), "restrictedSids": SimTypePointer(SimTypeBottom(label="SID_AND_ATTRIBUTES"), offset=0)}, name="FWP_TOKEN_INFORMATION", pack=False, align=None), offset=0), "tokenAccessInformation": SimTypePointer(SimStruct({"size": SimTypeInt(signed=False, label="UInt32"), "data": SimTypePointer(SimTypeChar(label="Byte"), offset=0)}, name="FWP_BYTE_BLOB", pack=False, align=None), offset=0), "unicodeString": SimTypePointer(SimTypeChar(label="Char"), offset=0), "byteArray6": SimTypePointer(SimStruct({"byteArray6": SimTypeFixedSizeArray(SimTypeChar(label="Byte"), 6)}, name="FWP_BYTE_ARRAY6", pack=False, align=None), offset=0), "bitmapArray64": SimTypePointer(SimStruct({"bitmapArray64": SimTypeFixedSizeArray(SimTypeChar(label="Byte"), 8)}, name="FWP_BITMAP_ARRAY64_", pack=False, align=None), offset=0)}, name="<anon>", label="None")}, name="FWP_VALUE0", pack=False, align=None), "valueHigh": SimStruct({"type": SimTypeInt(signed=False, label="FWP_DATA_TYPE"), "Anonymous": SimUnion({"uint8": SimTypeChar(label="Byte"), "uint16": SimTypeShort(signed=False, label="UInt16"), "uint32": SimTypeInt(signed=False, label="UInt32"), "uint64": SimTypePointer(SimTypeLongLong(signed=False, label="UInt64"), offset=0), "int8": SimTypeChar(label="SByte"), "int16": SimTypeShort(signed=True, label="Int16"), "int32": SimTypeInt(signed=True, label="Int32"), "int64": SimTypePointer(SimTypeLongLong(signed=True, label="Int64"), offset=0), "float32": SimTypeFloat(size=32), "double64": SimTypePointer(SimTypeFloat(size=64), offset=0), "byteArray16": SimTypePointer(SimStruct({"byteArray16": SimTypeFixedSizeArray(SimTypeChar(label="Byte"), 16)}, name="FWP_BYTE_ARRAY16", pack=False, align=None), offset=0), "byteBlob": SimTypePointer(SimStruct({"size": SimTypeInt(signed=False, label="UInt32"), "data": SimTypePointer(SimTypeChar(label="Byte"), offset=0)}, name="FWP_BYTE_BLOB", pack=False, align=None), offset=0), "sid": SimTypePointer(SimTypeBottom(label="SID"), offset=0), "sd": SimTypePointer(SimStruct({"size": SimTypeInt(signed=False, label="UInt32"), "data": SimTypePointer(SimTypeChar(label="Byte"), offset=0)}, name="FWP_BYTE_BLOB", pack=False, align=None), offset=0), "tokenInformation": SimTypePointer(SimStruct({"sidCount": SimTypeInt(signed=False, label="UInt32"), "sids": SimTypePointer(SimTypeBottom(label="SID_AND_ATTRIBUTES"), offset=0), "restrictedSidCount": SimTypeInt(signed=False, label="UInt32"), "restrictedSids": SimTypePointer(SimTypeBottom(label="SID_AND_ATTRIBUTES"), offset=0)}, name="FWP_TOKEN_INFORMATION", pack=False, align=None), offset=0), "tokenAccessInformation": SimTypePointer(SimStruct({"size": SimTypeInt(signed=False, label="UInt32"), "data": SimTypePointer(SimTypeChar(label="Byte"), offset=0)}, name="FWP_BYTE_BLOB", pack=False, align=None), offset=0), "unicodeString": SimTypePointer(SimTypeChar(label="Char"), offset=0), "byteArray6": SimTypePointer(SimStruct({"byteArray6": SimTypeFixedSizeArray(SimTypeChar(label="Byte"), 6)}, name="FWP_BYTE_ARRAY6", pack=False, align=None), offset=0), "bitmapArray64": SimTypePointer(SimStruct({"bitmapArray64": SimTypeFixedSizeArray(SimTypeChar(label="Byte"), 8)}, name="FWP_BITMAP_ARRAY64_", pack=False, align=None), offset=0)}, name="<anon>", label="None")}, name="FWP_VALUE0", pack=False, align=None)}, name="FWP_RANGE0", pack=False, align=None), offset=0)}, name="<anon>", label="None")}, name="FWP_CONDITION_VALUE0", pack=False, align=None)}, name="FWPM_FILTER_CONDITION0", pack=False, align=None), offset=0)}, name="FWPM_NET_EVENT_ENUM_TEMPLATE0", pack=False, align=None), offset=0), "flags": SimTypeInt(signed=False, label="UInt32"), "sessionKey": SimTypeBottom(label="Guid")}, name="FWPM_NET_EVENT_SUBSCRIPTION0", pack=False, align=None), offset=0), SimTypePointer(SimTypeFunction([SimTypePointer(SimTypeBottom(label="Void"), offset=0), SimTypePointer(SimStruct({"header": SimStruct({"timeStamp": SimStruct({"dwLowDateTime": SimTypeInt(signed=False, label="UInt32"), "dwHighDateTime": SimTypeInt(signed=False, label="UInt32")}, name="FILETIME", pack=False, align=None), "flags": SimTypeInt(signed=False, label="UInt32"), "ipVersion": SimTypeInt(signed=False, label="FWP_IP_VERSION"), "ipProtocol": SimTypeChar(label="Byte"), "Anonymous1": SimUnion({"localAddrV4": SimTypeInt(signed=False, label="UInt32"), "localAddrV6": SimStruct({"byteArray16": SimTypeFixedSizeArray(SimTypeChar(label="Byte"), 16)}, name="FWP_BYTE_ARRAY16", pack=False, align=None)}, name="<anon>", label="None"), "Anonymous2": SimUnion({"remoteAddrV4": SimTypeInt(signed=False, label="UInt32"), "remoteAddrV6": SimStruct({"byteArray16": SimTypeFixedSizeArray(SimTypeChar(label="Byte"), 16)}, name="FWP_BYTE_ARRAY16", pack=False, align=None)}, name="<anon>", label="None"), "localPort": SimTypeShort(signed=False, label="UInt16"), "remotePort": SimTypeShort(signed=False, label="UInt16"), "scopeId": SimTypeInt(signed=False, label="UInt32"), "appId": SimStruct({"size": SimTypeInt(signed=False, label="UInt32"), "data": SimTypePointer(SimTypeChar(label="Byte"), offset=0)}, name="FWP_BYTE_BLOB", pack=False, align=None), "userId": SimTypePointer(SimTypeBottom(label="SID"), offset=0), "addressFamily": SimTypeInt(signed=False, label="FWP_AF"), "packageSid": SimTypePointer(SimTypeBottom(label="SID"), offset=0), "enterpriseId": SimTypePointer(SimTypeChar(label="Char"), offset=0), "policyFlags": SimTypeLongLong(signed=False, label="UInt64"), "effectiveName": SimStruct({"size": SimTypeInt(signed=False, label="UInt32"), "data": SimTypePointer(SimTypeChar(label="Byte"), offset=0)}, name="FWP_BYTE_BLOB", pack=False, align=None)}, name="FWPM_NET_EVENT_HEADER3", pack=False, align=None), "type": SimTypeInt(signed=False, label="FWPM_NET_EVENT_TYPE"), "Anonymous": SimUnion({"ikeMmFailure": SimTypePointer(SimStruct({"failureErrorCode": SimTypeInt(signed=False, label="UInt32"), "failurePoint": SimTypeInt(signed=False, label="IPSEC_FAILURE_POINT"), "flags": SimTypeInt(signed=False, label="UInt32"), "keyingModuleType": SimTypeInt(signed=False, label="IKEEXT_KEY_MODULE_TYPE"), "mmState": SimTypeInt(signed=False, label="IKEEXT_MM_SA_STATE"), "saRole": SimTypeInt(signed=False, label="IKEEXT_SA_ROLE"), "mmAuthMethod": SimTypeInt(signed=False, label="IKEEXT_AUTHENTICATION_METHOD_TYPE"), "endCertHash": SimTypeFixedSizeArray(SimTypeChar(label="Byte"), 20), "mmId": SimTypeLongLong(signed=False, label="UInt64"), "mmFilterId": SimTypeLongLong(signed=False, label="UInt64"), "localPrincipalNameForAuth": SimTypePointer(SimTypeChar(label="Char"), offset=0), "remotePrincipalNameForAuth": SimTypePointer(SimTypeChar(label="Char"), offset=0), "numLocalPrincipalGroupSids": SimTypeInt(signed=False, label="UInt32"), "localPrincipalGroupSids": SimTypePointer(SimTypePointer(SimTypeChar(label="Char"), offset=0), offset=0), "numRemotePrincipalGroupSids": SimTypeInt(signed=False, label="UInt32"), "remotePrincipalGroupSids": SimTypePointer(SimTypePointer(SimTypeChar(label="Char"), offset=0), offset=0), "providerContextKey": SimTypePointer(SimTypeBottom(label="Guid"), offset=0)}, name="FWPM_NET_EVENT_IKEEXT_MM_FAILURE2_", pack=False, align=None), offset=0), "ikeQmFailure": SimTypePointer(SimStruct({"failureErrorCode": SimTypeInt(signed=False, label="UInt32"), "failurePoint": SimTypeInt(signed=False, label="IPSEC_FAILURE_POINT"), "keyingModuleType": SimTypeInt(signed=False, label="IKEEXT_KEY_MODULE_TYPE"), "qmState": SimTypeInt(signed=False, label="IKEEXT_QM_SA_STATE"), "saRole": SimTypeInt(signed=False, label="IKEEXT_SA_ROLE"), "saTrafficType": SimTypeInt(signed=False, label="IPSEC_TRAFFIC_TYPE"), "Anonymous1": SimUnion({"localSubNet": SimStruct({"type": SimTypeInt(signed=False, label="FWP_DATA_TYPE"), "Anonymous": SimUnion({"uint8": SimTypeChar(label="Byte"), "uint16": SimTypeShort(signed=False, label="UInt16"), "uint32": SimTypeInt(signed=False, label="UInt32"), "uint64": SimTypePointer(SimTypeLongLong(signed=False, label="UInt64"), offset=0), "int8": SimTypeChar(label="SByte"), "int16": SimTypeShort(signed=True, label="Int16"), "int32": SimTypeInt(signed=True, label="Int32"), "int64": SimTypePointer(SimTypeLongLong(signed=True, label="Int64"), offset=0), "float32": SimTypeFloat(size=32), "double64": SimTypePointer(SimTypeFloat(size=64), offset=0), "byteArray16": SimTypePointer(SimStruct({"byteArray16": SimTypeFixedSizeArray(SimTypeChar(label="Byte"), 16)}, name="FWP_BYTE_ARRAY16", pack=False, align=None), offset=0), "byteBlob": SimTypePointer(SimStruct({"size": SimTypeInt(signed=False, label="UInt32"), "data": SimTypePointer(SimTypeChar(label="Byte"), offset=0)}, name="FWP_BYTE_BLOB", pack=False, align=None), offset=0), "sid": SimTypePointer(SimTypeBottom(label="SID"), offset=0), "sd": SimTypePointer(SimStruct({"size": SimTypeInt(signed=False, label="UInt32"), "data": SimTypePointer(SimTypeChar(label="Byte"), offset=0)}, name="FWP_BYTE_BLOB", pack=False, align=None), offset=0), "tokenInformation": SimTypePointer(SimStruct({"sidCount": SimTypeInt(signed=False, label="UInt32"), "sids": SimTypePointer(SimTypeBottom(label="SID_AND_ATTRIBUTES"), offset=0), "restrictedSidCount": SimTypeInt(signed=False, label="UInt32"), "restrictedSids": SimTypePointer(SimTypeBottom(label="SID_AND_ATTRIBUTES"), offset=0)}, name="FWP_TOKEN_INFORMATION", pack=False, align=None), offset=0), "tokenAccessInformation": SimTypePointer(SimStruct({"size": SimTypeInt(signed=False, label="UInt32"), "data": SimTypePointer(SimTypeChar(label="Byte"), offset=0)}, name="FWP_BYTE_BLOB", pack=False, align=None), offset=0), "unicodeString": SimTypePointer(SimTypeChar(label="Char"), offset=0), "byteArray6": SimTypePointer(SimStruct({"byteArray6": SimTypeFixedSizeArray(SimTypeChar(label="Byte"), 6)}, name="FWP_BYTE_ARRAY6", pack=False, align=None), offset=0), "bitmapArray64": SimTypePointer(SimStruct({"bitmapArray64": SimTypeFixedSizeArray(SimTypeChar(label="Byte"), 8)}, name="FWP_BITMAP_ARRAY64_", pack=False, align=None), offset=0), "v4AddrMask": SimTypePointer(SimStruct({"addr": SimTypeInt(signed=False, label="UInt32"), "mask": SimTypeInt(signed=False, label="UInt32")}, name="FWP_V4_ADDR_AND_MASK", pack=False, align=None), offset=0), "v6AddrMask": SimTypePointer(SimStruct({"addr": SimTypeFixedSizeArray(SimTypeChar(label="Byte"), 16), "prefixLength": SimTypeChar(label="Byte")}, name="FWP_V6_ADDR_AND_MASK", pack=False, align=None), offset=0), "rangeValue": SimTypePointer(SimStruct({"valueLow": SimStruct({"type": SimTypeInt(signed=False, label="FWP_DATA_TYPE"), "Anonymous": SimUnion({"uint8": SimTypeChar(label="Byte"), "uint16": SimTypeShort(signed=False, label="UInt16"), "uint32": SimTypeInt(signed=False, label="UInt32"), "uint64": SimTypePointer(SimTypeLongLong(signed=False, label="UInt64"), offset=0), "int8": SimTypeChar(label="SByte"), "int16": SimTypeShort(signed=True, label="Int16"), "int32": SimTypeInt(signed=True, label="Int32"), "int64": SimTypePointer(SimTypeLongLong(signed=True, label="Int64"), offset=0), "float32": SimTypeFloat(size=32), "double64": SimTypePointer(SimTypeFloat(size=64), offset=0), "byteArray16": SimTypePointer(SimStruct({"byteArray16": SimTypeFixedSizeArray(SimTypeChar(label="Byte"), 16)}, name="FWP_BYTE_ARRAY16", pack=False, align=None), offset=0), "byteBlob": SimTypePointer(SimStruct({"size": SimTypeInt(signed=False, label="UInt32"), "data": SimTypePointer(SimTypeChar(label="Byte"), offset=0)}, name="FWP_BYTE_BLOB", pack=False, align=None), offset=0), "sid": SimTypePointer(SimTypeBottom(label="SID"), offset=0), "sd": SimTypePointer(SimStruct({"size": SimTypeInt(signed=False, label="UInt32"), "data": SimTypePointer(SimTypeChar(label="Byte"), offset=0)}, name="FWP_BYTE_BLOB", pack=False, align=None), offset=0), "tokenInformation": SimTypePointer(SimStruct({"sidCount": SimTypeInt(signed=False, label="UInt32"), "sids": SimTypePointer(SimTypeBottom(label="SID_AND_ATTRIBUTES"), offset=0), "restrictedSidCount": SimTypeInt(signed=False, label="UInt32"), "restrictedSids": SimTypePointer(SimTypeBottom(label="SID_AND_ATTRIBUTES"), offset=0)}, name="FWP_TOKEN_INFORMATION", pack=False, align=None), offset=0), "tokenAccessInformation": SimTypePointer(SimStruct({"size": SimTypeInt(signed=False, label="UInt32"), "data": SimTypePointer(SimTypeChar(label="Byte"), offset=0)}, name="FWP_BYTE_BLOB", pack=False, align=None), offset=0), "unicodeString": SimTypePointer(SimTypeChar(label="Char"), offset=0), "byteArray6": SimTypePointer(SimStruct({"byteArray6": SimTypeFixedSizeArray(SimTypeChar(label="Byte"), 6)}, name="FWP_BYTE_ARRAY6", pack=False, align=None), offset=0), "bitmapArray64": SimTypePointer(SimStruct({"bitmapArray64": SimTypeFixedSizeArray(SimTypeChar(label="Byte"), 8)}, name="FWP_BITMAP_ARRAY64_", pack=False, align=None), offset=0)}, name="<anon>", label="None")}, name="FWP_VALUE0", pack=False, align=None), "valueHigh": SimStruct({"type": SimTypeInt(signed=False, label="FWP_DATA_TYPE"), "Anonymous": SimUnion({"uint8": SimTypeChar(label="Byte"), "uint16": SimTypeShort(signed=False, label="UInt16"), "uint32": SimTypeInt(signed=False, label="UInt32"), "uint64": SimTypePointer(SimTypeLongLong(signed=False, label="UInt64"), offset=0), "int8": SimTypeChar(label="SByte"), "int16": SimTypeShort(signed=True, label="Int16"), "int32": SimTypeInt(signed=True, label="Int32"), "int64": SimTypePointer(SimTypeLongLong(signed=True, label="Int64"), offset=0), "float32": SimTypeFloat(size=32), "double64": SimTypePointer(SimTypeFloat(size=64), offset=0), "byteArray16": SimTypePointer(SimStruct({"byteArray16": SimTypeFixedSizeArray(SimTypeChar(label="Byte"), 16)}, name="FWP_BYTE_ARRAY16", pack=False, align=None), offset=0), "byteBlob": SimTypePointer(SimStruct({"size": SimTypeInt(signed=False, label="UInt32"), "data": SimTypePointer(SimTypeChar(label="Byte"), offset=0)}, name="FWP_BYTE_BLOB", pack=False, align=None), offset=0), "sid": SimTypePointer(SimTypeBottom(label="SID"), offset=0), "sd": SimTypePointer(SimStruct({"size": SimTypeInt(signed=False, label="UInt32"), "data": SimTypePointer(SimTypeChar(label="Byte"), offset=0)}, name="FWP_BYTE_BLOB", pack=False, align=None), offset=0), "tokenInformation": SimTypePointer(SimStruct({"sidCount": SimTypeInt(signed=False, label="UInt32"), "sids": SimTypePointer(SimTypeBottom(label="SID_AND_ATTRIBUTES"), offset=0), "restrictedSidCount": SimTypeInt(signed=False, label="UInt32"), "restrictedSids": SimTypePointer(SimTypeBottom(label="SID_AND_ATTRIBUTES"), offset=0)}, name="FWP_TOKEN_INFORMATION", pack=False, align=None), offset=0), "tokenAccessInformation": SimTypePointer(SimStruct({"size": SimTypeInt(signed=False, label="UInt32"), "data": SimTypePointer(SimTypeChar(label="Byte"), offset=0)}, name="FWP_BYTE_BLOB", pack=False, align=None), offset=0), "unicodeString": SimTypePointer(SimTypeChar(label="Char"), offset=0), "byteArray6": SimTypePointer(SimStruct({"byteArray6": SimTypeFixedSizeArray(SimTypeChar(label="Byte"), 6)}, name="FWP_BYTE_ARRAY6", pack=False, align=None), offset=0), "bitmapArray64": SimTypePointer(SimStruct({"bitmapArray64": SimTypeFixedSizeArray(SimTypeChar(label="Byte"), 8)}, name="FWP_BITMAP_ARRAY64_", pack=False, align=None), offset=0)}, name="<anon>", label="None")}, name="FWP_VALUE0", pack=False, align=None)}, name="FWP_RANGE0", pack=False, align=None), offset=0)}, name="<anon>", label="None")}, name="FWP_CONDITION_VALUE0", pack=False, align=None)}, name="<anon>", label="None"), "Anonymous2": SimUnion({"remoteSubNet": SimStruct({"type": SimTypeInt(signed=False, label="FWP_DATA_TYPE"), "Anonymous": SimUnion({"uint8": SimTypeChar(label="Byte"), "uint16": SimTypeShort(signed=False, label="UInt16"), "uint32": SimTypeInt(signed=False, label="UInt32"), "uint64": SimTypePointer(SimTypeLongLong(signed=False, label="UInt64"), offset=0), "int8": SimTypeChar(label="SByte"), "int16": SimTypeShort(signed=True, label="Int16"), "int32": SimTypeInt(signed=True, label="Int32"), "int64": SimTypePointer(SimTypeLongLong(signed=True, label="Int64"), offset=0), "float32": SimTypeFloat(size=32), "double64": SimTypePointer(SimTypeFloat(size=64), offset=0), "byteArray16": SimTypePointer(SimStruct({"byteArray16": SimTypeFixedSizeArray(SimTypeChar(label="Byte"), 16)}, name="FWP_BYTE_ARRAY16", pack=False, align=None), offset=0), "byteBlob": SimTypePointer(SimStruct({"size": SimTypeInt(signed=False, label="UInt32"), "data": SimTypePointer(SimTypeChar(label="Byte"), offset=0)}, name="FWP_BYTE_BLOB", pack=False, align=None), offset=0), "sid": SimTypePointer(SimTypeBottom(label="SID"), offset=0), "sd": SimTypePointer(SimStruct({"size": SimTypeInt(signed=False, label="UInt32"), "data": SimTypePointer(SimTypeChar(label="Byte"), offset=0)}, name="FWP_BYTE_BLOB", pack=False, align=None), offset=0), "tokenInformation": SimTypePointer(SimStruct({"sidCount": SimTypeInt(signed=False, label="UInt32"), "sids": SimTypePointer(SimTypeBottom(label="SID_AND_ATTRIBUTES"), offset=0), "restrictedSidCount": SimTypeInt(signed=False, label="UInt32"), "restrictedSids": SimTypePointer(SimTypeBottom(label="SID_AND_ATTRIBUTES"), offset=0)}, name="FWP_TOKEN_INFORMATION", pack=False, align=None), offset=0), "tokenAccessInformation": SimTypePointer(SimStruct({"size": SimTypeInt(signed=False, label="UInt32"), "data": SimTypePointer(SimTypeChar(label="Byte"), offset=0)}, name="FWP_BYTE_BLOB", pack=False, align=None), offset=0), "unicodeString": SimTypePointer(SimTypeChar(label="Char"), offset=0), "byteArray6": SimTypePointer(SimStruct({"byteArray6": SimTypeFixedSizeArray(SimTypeChar(label="Byte"), 6)}, name="FWP_BYTE_ARRAY6", pack=False, align=None), offset=0), "bitmapArray64": SimTypePointer(SimStruct({"bitmapArray64": SimTypeFixedSizeArray(SimTypeChar(label="Byte"), 8)}, name="FWP_BITMAP_ARRAY64_", pack=False, align=None), offset=0), "v4AddrMask": SimTypePointer(SimStruct({"addr": SimTypeInt(signed=False, label="UInt32"), "mask": SimTypeInt(signed=False, label="UInt32")}, name="FWP_V4_ADDR_AND_MASK", pack=False, align=None), offset=0), "v6AddrMask": SimTypePointer(SimStruct({"addr": SimTypeFixedSizeArray(SimTypeChar(label="Byte"), 16), "prefixLength": SimTypeChar(label="Byte")}, name="FWP_V6_ADDR_AND_MASK", pack=False, align=None), offset=0), "rangeValue": SimTypePointer(SimStruct({"valueLow": SimStruct({"type": SimTypeInt(signed=False, label="FWP_DATA_TYPE"), "Anonymous": SimUnion({"uint8": SimTypeChar(label="Byte"), "uint16": SimTypeShort(signed=False, label="UInt16"), "uint32": SimTypeInt(signed=False, label="UInt32"), "uint64": SimTypePointer(SimTypeLongLong(signed=False, label="UInt64"), offset=0), "int8": SimTypeChar(label="SByte"), "int16": SimTypeShort(signed=True, label="Int16"), "int32": SimTypeInt(signed=True, label="Int32"), "int64": SimTypePointer(SimTypeLongLong(signed=True, label="Int64"), offset=0), "float32": SimTypeFloat(size=32), "double64": SimTypePointer(SimTypeFloat(size=64), offset=0), "byteArray16": SimTypePointer(SimStruct({"byteArray16": SimTypeFixedSizeArray(SimTypeChar(label="Byte"), 16)}, name="FWP_BYTE_ARRAY16", pack=False, align=None), offset=0), "byteBlob": SimTypePointer(SimStruct({"size": SimTypeInt(signed=False, label="UInt32"), "data": SimTypePointer(SimTypeChar(label="Byte"), offset=0)}, name="FWP_BYTE_BLOB", pack=False, align=None), offset=0), "sid": SimTypePointer(SimTypeBottom(label="SID"), offset=0), "sd": SimTypePointer(SimStruct({"size": SimTypeInt(signed=False, label="UInt32"), "data": SimTypePointer(SimTypeChar(label="Byte"), offset=0)}, name="FWP_BYTE_BLOB", pack=False, align=None), offset=0), "tokenInformation": SimTypePointer(SimStruct({"sidCount": SimTypeInt(signed=False, label="UInt32"), "sids": SimTypePointer(SimTypeBottom(label="SID_AND_ATTRIBUTES"), offset=0), "restrictedSidCount": SimTypeInt(signed=False, label="UInt32"), "restrictedSids": SimTypePointer(SimTypeBottom(label="SID_AND_ATTRIBUTES"), offset=0)}, name="FWP_TOKEN_INFORMATION", pack=False, align=None), offset=0), "tokenAccessInformation": SimTypePointer(SimStruct({"size": SimTypeInt(signed=False, label="UInt32"), "data": SimTypePointer(SimTypeChar(label="Byte"), offset=0)}, name="FWP_BYTE_BLOB", pack=False, align=None), offset=0), "unicodeString": SimTypePointer(SimTypeChar(label="Char"), offset=0), "byteArray6": SimTypePointer(SimStruct({"byteArray6": SimTypeFixedSizeArray(SimTypeChar(label="Byte"), 6)}, name="FWP_BYTE_ARRAY6", pack=False, align=None), offset=0), "bitmapArray64": SimTypePointer(SimStruct({"bitmapArray64": SimTypeFixedSizeArray(SimTypeChar(label="Byte"), 8)}, name="FWP_BITMAP_ARRAY64_", pack=False, align=None), offset=0)}, name="<anon>", label="None")}, name="FWP_VALUE0", pack=False, align=None), "valueHigh": SimStruct({"type": SimTypeInt(signed=False, label="FWP_DATA_TYPE"), "Anonymous": SimUnion({"uint8": SimTypeChar(label="Byte"), "uint16": SimTypeShort(signed=False, label="UInt16"), "uint32": SimTypeInt(signed=False, label="UInt32"), "uint64": SimTypePointer(SimTypeLongLong(signed=False, label="UInt64"), offset=0), "int8": SimTypeChar(label="SByte"), "int16": SimTypeShort(signed=True, label="Int16"), "int32": SimTypeInt(signed=True, label="Int32"), "int64": SimTypePointer(SimTypeLongLong(signed=True, label="Int64"), offset=0), "float32": SimTypeFloat(size=32), "double64": SimTypePointer(SimTypeFloat(size=64), offset=0), "byteArray16": SimTypePointer(SimStruct({"byteArray16": SimTypeFixedSizeArray(SimTypeChar(label="Byte"), 16)}, name="FWP_BYTE_ARRAY16", pack=False, align=None), offset=0), "byteBlob": SimTypePointer(SimStruct({"size": SimTypeInt(signed=False, label="UInt32"), "data": SimTypePointer(SimTypeChar(label="Byte"), offset=0)}, name="FWP_BYTE_BLOB", pack=False, align=None), offset=0), "sid": SimTypePointer(SimTypeBottom(label="SID"), offset=0), "sd": SimTypePointer(SimStruct({"size": SimTypeInt(signed=False, label="UInt32"), "data": SimTypePointer(SimTypeChar(label="Byte"), offset=0)}, name="FWP_BYTE_BLOB", pack=False, align=None), offset=0), "tokenInformation": SimTypePointer(SimStruct({"sidCount": SimTypeInt(signed=False, label="UInt32"), "sids": SimTypePointer(SimTypeBottom(label="SID_AND_ATTRIBUTES"), offset=0), "restrictedSidCount": SimTypeInt(signed=False, label="UInt32"), "restrictedSids": SimTypePointer(SimTypeBottom(label="SID_AND_ATTRIBUTES"), offset=0)}, name="FWP_TOKEN_INFORMATION", pack=False, align=None), offset=0), "tokenAccessInformation": SimTypePointer(SimStruct({"size": SimTypeInt(signed=False, label="UInt32"), "data": SimTypePointer(SimTypeChar(label="Byte"), offset=0)}, name="FWP_BYTE_BLOB", pack=False, align=None), offset=0), "unicodeString": SimTypePointer(SimTypeChar(label="Char"), offset=0), "byteArray6": SimTypePointer(SimStruct({"byteArray6": SimTypeFixedSizeArray(SimTypeChar(label="Byte"), 6)}, name="FWP_BYTE_ARRAY6", pack=False, align=None), offset=0), "bitmapArray64": SimTypePointer(SimStruct({"bitmapArray64": SimTypeFixedSizeArray(SimTypeChar(label="Byte"), 8)}, name="FWP_BITMAP_ARRAY64_", pack=False, align=None), offset=0)}, name="<anon>", label="None")}, name="FWP_VALUE0", pack=False, align=None)}, name="FWP_RANGE0", pack=False, align=None), offset=0)}, name="<anon>", label="None")}, name="FWP_CONDITION_VALUE0", pack=False, align=None)}, name="<anon>", label="None"), "qmFilterId": SimTypeLongLong(signed=False, label="UInt64"), "mmSaLuid": SimTypeLongLong(signed=False, label="UInt64"), "mmProviderContextKey": SimTypeBottom(label="Guid")}, name="FWPM_NET_EVENT_IKEEXT_QM_FAILURE1_", pack=False, align=None), offset=0), "ikeEmFailure": SimTypePointer(SimStruct({"failureErrorCode": SimTypeInt(signed=False, label="UInt32"), "failurePoint": SimTypeInt(signed=False, label="IPSEC_FAILURE_POINT"), "flags": SimTypeInt(signed=False, label="UInt32"), "emState": SimTypeInt(signed=False, label="IKEEXT_EM_SA_STATE"), "saRole": SimTypeInt(signed=False, label="IKEEXT_SA_ROLE"), "emAuthMethod": SimTypeInt(signed=False, label="IKEEXT_AUTHENTICATION_METHOD_TYPE"), "endCertHash": SimTypeFixedSizeArray(SimTypeChar(label="Byte"), 20), "mmId": SimTypeLongLong(signed=False, label="UInt64"), "qmFilterId": SimTypeLongLong(signed=False, label="UInt64"), "localPrincipalNameForAuth": SimTypePointer(SimTypeChar(label="Char"), offset=0), "remotePrincipalNameForAuth": SimTypePointer(SimTypeChar(label="Char"), offset=0), "numLocalPrincipalGroupSids": SimTypeInt(signed=False, label="UInt32"), "localPrincipalGroupSids": SimTypePointer(SimTypePointer(SimTypeChar(label="Char"), offset=0), offset=0), "numRemotePrincipalGroupSids": SimTypeInt(signed=False, label="UInt32"), "remotePrincipalGroupSids": SimTypePointer(SimTypePointer(SimTypeChar(label="Char"), offset=0), offset=0), "saTrafficType": SimTypeInt(signed=False, label="IPSEC_TRAFFIC_TYPE")}, name="FWPM_NET_EVENT_IKEEXT_EM_FAILURE1", pack=False, align=None), offset=0), "classifyDrop": SimTypePointer(SimStruct({"filterId": SimTypeLongLong(signed=False, label="UInt64"), "layerId": SimTypeShort(signed=False, label="UInt16"), "reauthReason": SimTypeInt(signed=False, label="UInt32"), "originalProfile": SimTypeInt(signed=False, label="UInt32"), "currentProfile": SimTypeInt(signed=False, label="UInt32"), "msFwpDirection": SimTypeInt(signed=False, label="UInt32"), "isLoopback": SimTypeInt(signed=True, label="Int32"), "vSwitchId": SimStruct({"size": SimTypeInt(signed=False, label="UInt32"), "data": SimTypePointer(SimTypeChar(label="Byte"), offset=0)}, name="FWP_BYTE_BLOB", pack=False, align=None), "vSwitchSourcePort": SimTypeInt(signed=False, label="UInt32"), "vSwitchDestinationPort": SimTypeInt(signed=False, label="UInt32")}, name="FWPM_NET_EVENT_CLASSIFY_DROP2", pack=False, align=None), offset=0), "ipsecDrop": SimTypePointer(SimStruct({"failureStatus": SimTypeInt(signed=True, label="Int32"), "direction": SimTypeInt(signed=False, label="FWP_DIRECTION"), "spi": SimTypeInt(signed=False, label="UInt32"), "filterId": SimTypeLongLong(signed=False, label="UInt64"), "layerId": SimTypeShort(signed=False, label="UInt16")}, name="FWPM_NET_EVENT_IPSEC_KERNEL_DROP0", pack=False, align=None), offset=0), "idpDrop": SimTypePointer(SimStruct({"ipVersion": SimTypeInt(signed=False, label="FWP_IP_VERSION"), "Anonymous1": SimUnion({"publicHostV4Addr": SimTypeInt(signed=False, label="UInt32"), "publicHostV6Addr": SimTypeFixedSizeArray(SimTypeChar(label="Byte"), 16)}, name="<anon>", label="None"), "Anonymous2": SimUnion({"internalHostV4Addr": SimTypeInt(signed=False, label="UInt32"), "internalHostV6Addr": SimTypeFixedSizeArray(SimTypeChar(label="Byte"), 16)}, name="<anon>", label="None"), "failureStatus": SimTypeInt(signed=True, label="Int32"), "direction": SimTypeInt(signed=False, label="FWP_DIRECTION")}, name="FWPM_NET_EVENT_IPSEC_DOSP_DROP0", pack=False, align=None), offset=0), "classifyAllow": SimTypePointer(SimStruct({"filterId": SimTypeLongLong(signed=False, label="UInt64"), "layerId": SimTypeShort(signed=False, label="UInt16"), "reauthReason": SimTypeInt(signed=False, label="UInt32"), "originalProfile": SimTypeInt(signed=False, label="UInt32"), "currentProfile": SimTypeInt(signed=False, label="UInt32"), "msFwpDirection": SimTypeInt(signed=False, label="UInt32"), "isLoopback": SimTypeInt(signed=True, label="Int32")}, name="FWPM_NET_EVENT_CLASSIFY_ALLOW0", pack=False, align=None), offset=0), "capabilityDrop": SimTypePointer(SimStruct({"networkCapabilityId": SimTypeInt(signed=False, label="FWPM_APPC_NETWORK_CAPABILITY_TYPE"), "filterId": SimTypeLongLong(signed=False, label="UInt64"), "isLoopback": SimTypeInt(signed=True, label="Int32")}, name="FWPM_NET_EVENT_CAPABILITY_DROP0", pack=False, align=None), offset=0), "capabilityAllow": SimTypePointer(SimStruct({"networkCapabilityId": SimTypeInt(signed=False, label="FWPM_APPC_NETWORK_CAPABILITY_TYPE"), "filterId": SimTypeLongLong(signed=False, label="UInt64"), "isLoopback": SimTypeInt(signed=True, label="Int32")}, name="FWPM_NET_EVENT_CAPABILITY_ALLOW0", pack=False, align=None), offset=0), "classifyDropMac": SimTypePointer(SimStruct({"localMacAddr": SimStruct({"byteArray6": SimTypeFixedSizeArray(SimTypeChar(label="Byte"), 6)}, name="FWP_BYTE_ARRAY6", pack=False, align=None), "remoteMacAddr": SimStruct({"byteArray6": SimTypeFixedSizeArray(SimTypeChar(label="Byte"), 6)}, name="FWP_BYTE_ARRAY6", pack=False, align=None), "mediaType": SimTypeInt(signed=False, label="UInt32"), "ifType": SimTypeInt(signed=False, label="UInt32"), "etherType": SimTypeShort(signed=False, label="UInt16"), "ndisPortNumber": SimTypeInt(signed=False, label="UInt32"), "reserved": SimTypeInt(signed=False, label="UInt32"), "vlanTag": SimTypeShort(signed=False, label="UInt16"), "ifLuid": SimTypeLongLong(signed=False, label="UInt64"), "filterId": SimTypeLongLong(signed=False, label="UInt64"), "layerId": SimTypeShort(signed=False, label="UInt16"), "reauthReason": SimTypeInt(signed=False, label="UInt32"), "originalProfile": SimTypeInt(signed=False, label="UInt32"), "currentProfile": SimTypeInt(signed=False, label="UInt32"), "msFwpDirection": SimTypeInt(signed=False, label="UInt32"), "isLoopback": SimTypeInt(signed=True, label="Int32"), "vSwitchId": SimStruct({"size": SimTypeInt(signed=False, label="UInt32"), "data": SimTypePointer(SimTypeChar(label="Byte"), offset=0)}, name="FWP_BYTE_BLOB", pack=False, align=None), "vSwitchSourcePort": SimTypeInt(signed=False, label="UInt32"), "vSwitchDestinationPort": SimTypeInt(signed=False, label="UInt32")}, name="FWPM_NET_EVENT_CLASSIFY_DROP_MAC0", pack=False, align=None), offset=0), "lpmPacketArrival": SimTypePointer(SimStruct({"spi": SimTypeInt(signed=False, label="UInt32")}, name="FWPM_NET_EVENT_LPM_PACKET_ARRIVAL0_", pack=False, align=None), offset=0)}, name="<anon>", label="None")}, name="FWPM_NET_EVENT5_", pack=False, align=None), offset=0)], SimTypeBottom(label="Void"), arg_names=["context", "event"]), offset=0), SimTypePointer(SimTypeBottom(label="Void"), offset=0), SimTypePointer(SimTypePointer(SimTypeInt(signed=True, label="Int"), label="IntPtr", offset=0), offset=0)], SimTypeInt(signed=False, label="UInt32"), arg_names=["engineHandle", "subscription", "callback", "context", "eventsHandle"]),
        # 
        'FwpmSystemPortsGet0': SimTypeFunction([SimTypePointer(SimTypeInt(signed=True, label="Int"), label="IntPtr", offset=0), SimTypePointer(SimTypePointer(SimStruct({"numTypes": SimTypeInt(signed=False, label="UInt32"), "types": SimTypePointer(SimStruct({"type": SimTypeInt(signed=False, label="FWPM_SYSTEM_PORT_TYPE"), "numPorts": SimTypeInt(signed=False, label="UInt32"), "ports": SimTypePointer(SimTypeShort(signed=False, label="UInt16"), offset=0)}, name="FWPM_SYSTEM_PORTS_BY_TYPE0", pack=False, align=None), offset=0)}, name="FWPM_SYSTEM_PORTS0", pack=False, align=None), offset=0), offset=0)], SimTypeInt(signed=False, label="UInt32"), arg_names=["engineHandle", "sysPorts"]),
        # 
        'FwpmSystemPortsSubscribe0': SimTypeFunction([SimTypePointer(SimTypeInt(signed=True, label="Int"), label="IntPtr", offset=0), SimTypePointer(SimTypeBottom(label="Void"), offset=0), SimTypePointer(SimTypeFunction([SimTypePointer(SimTypeBottom(label="Void"), offset=0), SimTypePointer(SimStruct({"numTypes": SimTypeInt(signed=False, label="UInt32"), "types": SimTypePointer(SimStruct({"type": SimTypeInt(signed=False, label="FWPM_SYSTEM_PORT_TYPE"), "numPorts": SimTypeInt(signed=False, label="UInt32"), "ports": SimTypePointer(SimTypeShort(signed=False, label="UInt16"), offset=0)}, name="FWPM_SYSTEM_PORTS_BY_TYPE0", pack=False, align=None), offset=0)}, name="FWPM_SYSTEM_PORTS0", pack=False, align=None), offset=0)], SimTypeBottom(label="Void"), arg_names=["context", "sysPorts"]), offset=0), SimTypePointer(SimTypeBottom(label="Void"), offset=0), SimTypePointer(SimTypePointer(SimTypeInt(signed=True, label="Int"), label="IntPtr", offset=0), offset=0)], SimTypeInt(signed=False, label="UInt32"), arg_names=["engineHandle", "reserved", "callback", "context", "sysPortsHandle"]),
        # 
        'FwpmSystemPortsUnsubscribe0': SimTypeFunction([SimTypePointer(SimTypeInt(signed=True, label="Int"), label="IntPtr", offset=0), SimTypePointer(SimTypeInt(signed=True, label="Int"), label="IntPtr", offset=0)], SimTypeInt(signed=False, label="UInt32"), arg_names=["engineHandle", "sysPortsHandle"]),
        # 
        'FwpmConnectionGetById0': SimTypeFunction([SimTypePointer(SimTypeInt(signed=True, label="Int"), label="IntPtr", offset=0), SimTypeLongLong(signed=False, label="UInt64"), SimTypePointer(SimTypePointer(SimStruct({"connectionId": SimTypeLongLong(signed=False, label="UInt64"), "ipVersion": SimTypeInt(signed=False, label="FWP_IP_VERSION"), "Anonymous1": SimUnion({"localV4Address": SimTypeInt(signed=False, label="UInt32"), "localV6Address": SimTypeFixedSizeArray(SimTypeChar(label="Byte"), 16)}, name="<anon>", label="None"), "Anonymous2": SimUnion({"remoteV4Address": SimTypeInt(signed=False, label="UInt32"), "remoteV6Address": SimTypeFixedSizeArray(SimTypeChar(label="Byte"), 16)}, name="<anon>", label="None"), "providerKey": SimTypePointer(SimTypeBottom(label="Guid"), offset=0), "ipsecTrafficModeType": SimTypeInt(signed=False, label="IPSEC_TRAFFIC_TYPE"), "keyModuleType": SimTypeInt(signed=False, label="IKEEXT_KEY_MODULE_TYPE"), "mmCrypto": SimStruct({"cipherAlgorithm": SimStruct({"algoIdentifier": SimTypeInt(signed=False, label="IKEEXT_CIPHER_TYPE"), "keyLen": SimTypeInt(signed=False, label="UInt32"), "rounds": SimTypeInt(signed=False, label="UInt32")}, name="IKEEXT_CIPHER_ALGORITHM0", pack=False, align=None), "integrityAlgorithm": SimStruct({"algoIdentifier": SimTypeInt(signed=False, label="IKEEXT_INTEGRITY_TYPE")}, name="IKEEXT_INTEGRITY_ALGORITHM0", pack=False, align=None), "maxLifetimeSeconds": SimTypeInt(signed=False, label="UInt32"), "dhGroup": SimTypeInt(signed=False, label="IKEEXT_DH_GROUP"), "quickModeLimit": SimTypeInt(signed=False, label="UInt32")}, name="IKEEXT_PROPOSAL0", pack=False, align=None), "mmPeer": SimStruct({"authenticationMethodType": SimTypeInt(signed=False, label="IKEEXT_AUTHENTICATION_METHOD_TYPE"), "impersonationType": SimTypeInt(signed=False, label="IKEEXT_AUTHENTICATION_IMPERSONATION_TYPE"), "Anonymous": SimUnion({"presharedKey": SimTypePointer(SimStruct({"presharedKey": SimStruct({"size": SimTypeInt(signed=False, label="UInt32"), "data": SimTypePointer(SimTypeChar(label="Byte"), offset=0)}, name="FWP_BYTE_BLOB", pack=False, align=None), "flags": SimTypeInt(signed=False, label="IKEEXT_PRESHARED_KEY_AUTHENTICATION_FLAGS")}, name="IKEEXT_PRESHARED_KEY_AUTHENTICATION1", pack=False, align=None), offset=0), "certificate": SimTypePointer(SimStruct({"subjectName": SimStruct({"size": SimTypeInt(signed=False, label="UInt32"), "data": SimTypePointer(SimTypeChar(label="Byte"), offset=0)}, name="FWP_BYTE_BLOB", pack=False, align=None), "certHash": SimStruct({"size": SimTypeInt(signed=False, label="UInt32"), "data": SimTypePointer(SimTypeChar(label="Byte"), offset=0)}, name="FWP_BYTE_BLOB", pack=False, align=None), "flags": SimTypeInt(signed=False, label="UInt32"), "certificate": SimStruct({"size": SimTypeInt(signed=False, label="UInt32"), "data": SimTypePointer(SimTypeChar(label="Byte"), offset=0)}, name="FWP_BYTE_BLOB", pack=False, align=None)}, name="IKEEXT_CERTIFICATE_CREDENTIAL1", pack=False, align=None), offset=0), "name": SimTypePointer(SimStruct({"principalName": SimTypePointer(SimTypeChar(label="Char"), offset=0)}, name="IKEEXT_NAME_CREDENTIAL0", pack=False, align=None), offset=0)}, name="<anon>", label="None")}, name="IKEEXT_CREDENTIAL2", pack=False, align=None), "emPeer": SimStruct({"authenticationMethodType": SimTypeInt(signed=False, label="IKEEXT_AUTHENTICATION_METHOD_TYPE"), "impersonationType": SimTypeInt(signed=False, label="IKEEXT_AUTHENTICATION_IMPERSONATION_TYPE"), "Anonymous": SimUnion({"presharedKey": SimTypePointer(SimStruct({"presharedKey": SimStruct({"size": SimTypeInt(signed=False, label="UInt32"), "data": SimTypePointer(SimTypeChar(label="Byte"), offset=0)}, name="FWP_BYTE_BLOB", pack=False, align=None), "flags": SimTypeInt(signed=False, label="IKEEXT_PRESHARED_KEY_AUTHENTICATION_FLAGS")}, name="IKEEXT_PRESHARED_KEY_AUTHENTICATION1", pack=False, align=None), offset=0), "certificate": SimTypePointer(SimStruct({"subjectName": SimStruct({"size": SimTypeInt(signed=False, label="UInt32"), "data": SimTypePointer(SimTypeChar(label="Byte"), offset=0)}, name="FWP_BYTE_BLOB", pack=False, align=None), "certHash": SimStruct({"size": SimTypeInt(signed=False, label="UInt32"), "data": SimTypePointer(SimTypeChar(label="Byte"), offset=0)}, name="FWP_BYTE_BLOB", pack=False, align=None), "flags": SimTypeInt(signed=False, label="UInt32"), "certificate": SimStruct({"size": SimTypeInt(signed=False, label="UInt32"), "data": SimTypePointer(SimTypeChar(label="Byte"), offset=0)}, name="FWP_BYTE_BLOB", pack=False, align=None)}, name="IKEEXT_CERTIFICATE_CREDENTIAL1", pack=False, align=None), offset=0), "name": SimTypePointer(SimStruct({"principalName": SimTypePointer(SimTypeChar(label="Char"), offset=0)}, name="IKEEXT_NAME_CREDENTIAL0", pack=False, align=None), offset=0)}, name="<anon>", label="None")}, name="IKEEXT_CREDENTIAL2", pack=False, align=None), "bytesTransferredIn": SimTypeLongLong(signed=False, label="UInt64"), "bytesTransferredOut": SimTypeLongLong(signed=False, label="UInt64"), "bytesTransferredTotal": SimTypeLongLong(signed=False, label="UInt64"), "startSysTime": SimStruct({"dwLowDateTime": SimTypeInt(signed=False, label="UInt32"), "dwHighDateTime": SimTypeInt(signed=False, label="UInt32")}, name="FILETIME", pack=False, align=None)}, name="FWPM_CONNECTION0", pack=False, align=None), offset=0), offset=0)], SimTypeInt(signed=False, label="UInt32"), arg_names=["engineHandle", "id", "connection"]),
        # 
        'FwpmConnectionEnum0': SimTypeFunction([SimTypePointer(SimTypeInt(signed=True, label="Int"), label="IntPtr", offset=0), SimTypePointer(SimTypeInt(signed=True, label="Int"), label="IntPtr", offset=0), SimTypeInt(signed=False, label="UInt32"), SimTypePointer(SimTypePointer(SimTypePointer(SimStruct({"connectionId": SimTypeLongLong(signed=False, label="UInt64"), "ipVersion": SimTypeInt(signed=False, label="FWP_IP_VERSION"), "Anonymous1": SimUnion({"localV4Address": SimTypeInt(signed=False, label="UInt32"), "localV6Address": SimTypeFixedSizeArray(SimTypeChar(label="Byte"), 16)}, name="<anon>", label="None"), "Anonymous2": SimUnion({"remoteV4Address": SimTypeInt(signed=False, label="UInt32"), "remoteV6Address": SimTypeFixedSizeArray(SimTypeChar(label="Byte"), 16)}, name="<anon>", label="None"), "providerKey": SimTypePointer(SimTypeBottom(label="Guid"), offset=0), "ipsecTrafficModeType": SimTypeInt(signed=False, label="IPSEC_TRAFFIC_TYPE"), "keyModuleType": SimTypeInt(signed=False, label="IKEEXT_KEY_MODULE_TYPE"), "mmCrypto": SimStruct({"cipherAlgorithm": SimStruct({"algoIdentifier": SimTypeInt(signed=False, label="IKEEXT_CIPHER_TYPE"), "keyLen": SimTypeInt(signed=False, label="UInt32"), "rounds": SimTypeInt(signed=False, label="UInt32")}, name="IKEEXT_CIPHER_ALGORITHM0", pack=False, align=None), "integrityAlgorithm": SimStruct({"algoIdentifier": SimTypeInt(signed=False, label="IKEEXT_INTEGRITY_TYPE")}, name="IKEEXT_INTEGRITY_ALGORITHM0", pack=False, align=None), "maxLifetimeSeconds": SimTypeInt(signed=False, label="UInt32"), "dhGroup": SimTypeInt(signed=False, label="IKEEXT_DH_GROUP"), "quickModeLimit": SimTypeInt(signed=False, label="UInt32")}, name="IKEEXT_PROPOSAL0", pack=False, align=None), "mmPeer": SimStruct({"authenticationMethodType": SimTypeInt(signed=False, label="IKEEXT_AUTHENTICATION_METHOD_TYPE"), "impersonationType": SimTypeInt(signed=False, label="IKEEXT_AUTHENTICATION_IMPERSONATION_TYPE"), "Anonymous": SimUnion({"presharedKey": SimTypePointer(SimStruct({"presharedKey": SimStruct({"size": SimTypeInt(signed=False, label="UInt32"), "data": SimTypePointer(SimTypeChar(label="Byte"), offset=0)}, name="FWP_BYTE_BLOB", pack=False, align=None), "flags": SimTypeInt(signed=False, label="IKEEXT_PRESHARED_KEY_AUTHENTICATION_FLAGS")}, name="IKEEXT_PRESHARED_KEY_AUTHENTICATION1", pack=False, align=None), offset=0), "certificate": SimTypePointer(SimStruct({"subjectName": SimStruct({"size": SimTypeInt(signed=False, label="UInt32"), "data": SimTypePointer(SimTypeChar(label="Byte"), offset=0)}, name="FWP_BYTE_BLOB", pack=False, align=None), "certHash": SimStruct({"size": SimTypeInt(signed=False, label="UInt32"), "data": SimTypePointer(SimTypeChar(label="Byte"), offset=0)}, name="FWP_BYTE_BLOB", pack=False, align=None), "flags": SimTypeInt(signed=False, label="UInt32"), "certificate": SimStruct({"size": SimTypeInt(signed=False, label="UInt32"), "data": SimTypePointer(SimTypeChar(label="Byte"), offset=0)}, name="FWP_BYTE_BLOB", pack=False, align=None)}, name="IKEEXT_CERTIFICATE_CREDENTIAL1", pack=False, align=None), offset=0), "name": SimTypePointer(SimStruct({"principalName": SimTypePointer(SimTypeChar(label="Char"), offset=0)}, name="IKEEXT_NAME_CREDENTIAL0", pack=False, align=None), offset=0)}, name="<anon>", label="None")}, name="IKEEXT_CREDENTIAL2", pack=False, align=None), "emPeer": SimStruct({"authenticationMethodType": SimTypeInt(signed=False, label="IKEEXT_AUTHENTICATION_METHOD_TYPE"), "impersonationType": SimTypeInt(signed=False, label="IKEEXT_AUTHENTICATION_IMPERSONATION_TYPE"), "Anonymous": SimUnion({"presharedKey": SimTypePointer(SimStruct({"presharedKey": SimStruct({"size": SimTypeInt(signed=False, label="UInt32"), "data": SimTypePointer(SimTypeChar(label="Byte"), offset=0)}, name="FWP_BYTE_BLOB", pack=False, align=None), "flags": SimTypeInt(signed=False, label="IKEEXT_PRESHARED_KEY_AUTHENTICATION_FLAGS")}, name="IKEEXT_PRESHARED_KEY_AUTHENTICATION1", pack=False, align=None), offset=0), "certificate": SimTypePointer(SimStruct({"subjectName": SimStruct({"size": SimTypeInt(signed=False, label="UInt32"), "data": SimTypePointer(SimTypeChar(label="Byte"), offset=0)}, name="FWP_BYTE_BLOB", pack=False, align=None), "certHash": SimStruct({"size": SimTypeInt(signed=False, label="UInt32"), "data": SimTypePointer(SimTypeChar(label="Byte"), offset=0)}, name="FWP_BYTE_BLOB", pack=False, align=None), "flags": SimTypeInt(signed=False, label="UInt32"), "certificate": SimStruct({"size": SimTypeInt(signed=False, label="UInt32"), "data": SimTypePointer(SimTypeChar(label="Byte"), offset=0)}, name="FWP_BYTE_BLOB", pack=False, align=None)}, name="IKEEXT_CERTIFICATE_CREDENTIAL1", pack=False, align=None), offset=0), "name": SimTypePointer(SimStruct({"principalName": SimTypePointer(SimTypeChar(label="Char"), offset=0)}, name="IKEEXT_NAME_CREDENTIAL0", pack=False, align=None), offset=0)}, name="<anon>", label="None")}, name="IKEEXT_CREDENTIAL2", pack=False, align=None), "bytesTransferredIn": SimTypeLongLong(signed=False, label="UInt64"), "bytesTransferredOut": SimTypeLongLong(signed=False, label="UInt64"), "bytesTransferredTotal": SimTypeLongLong(signed=False, label="UInt64"), "startSysTime": SimStruct({"dwLowDateTime": SimTypeInt(signed=False, label="UInt32"), "dwHighDateTime": SimTypeInt(signed=False, label="UInt32")}, name="FILETIME", pack=False, align=None)}, name="FWPM_CONNECTION0", pack=False, align=None), offset=0), offset=0), offset=0), SimTypePointer(SimTypeInt(signed=False, label="UInt32"), offset=0)], SimTypeInt(signed=False, label="UInt32"), arg_names=["engineHandle", "enumHandle", "numEntriesRequested", "entries", "numEntriesReturned"]),
        # 
        'FwpmConnectionCreateEnumHandle0': SimTypeFunction([SimTypePointer(SimTypeInt(signed=True, label="Int"), label="IntPtr", offset=0), SimTypePointer(SimStruct({"connectionId": SimTypeLongLong(signed=False, label="UInt64"), "flags": SimTypeInt(signed=False, label="UInt32")}, name="FWPM_CONNECTION_ENUM_TEMPLATE0", pack=False, align=None), offset=0), SimTypePointer(SimTypePointer(SimTypeInt(signed=True, label="Int"), label="IntPtr", offset=0), offset=0)], SimTypeInt(signed=False, label="UInt32"), arg_names=["engineHandle", "enumTemplate", "enumHandle"]),
        # 
        'FwpmConnectionDestroyEnumHandle0': SimTypeFunction([SimTypePointer(SimTypeInt(signed=True, label="Int"), label="IntPtr", offset=0), SimTypePointer(SimTypeInt(signed=True, label="Int"), label="IntPtr", offset=0)], SimTypeInt(signed=False, label="UInt32"), arg_names=["engineHandle", "enumHandle"]),
        # 
        'FwpmConnectionGetSecurityInfo0': SimTypeFunction([SimTypePointer(SimTypeInt(signed=True, label="Int"), label="IntPtr", offset=0), SimTypeInt(signed=False, label="UInt32"), SimTypePointer(SimTypePointer(SimTypeInt(signed=True, label="Int"), label="IntPtr", offset=0), offset=0), SimTypePointer(SimTypePointer(SimTypeInt(signed=True, label="Int"), label="IntPtr", offset=0), offset=0), SimTypePointer(SimTypePointer(SimStruct({"AclRevision": SimTypeChar(label="Byte"), "Sbz1": SimTypeChar(label="Byte"), "AclSize": SimTypeShort(signed=False, label="UInt16"), "AceCount": SimTypeShort(signed=False, label="UInt16"), "Sbz2": SimTypeShort(signed=False, label="UInt16")}, name="ACL", pack=False, align=None), offset=0), offset=0), SimTypePointer(SimTypePointer(SimStruct({"AclRevision": SimTypeChar(label="Byte"), "Sbz1": SimTypeChar(label="Byte"), "AclSize": SimTypeShort(signed=False, label="UInt16"), "AceCount": SimTypeShort(signed=False, label="UInt16"), "Sbz2": SimTypeShort(signed=False, label="UInt16")}, name="ACL", pack=False, align=None), offset=0), offset=0), SimTypePointer(SimTypePointer(SimStruct({"Revision": SimTypeChar(label="Byte"), "Sbz1": SimTypeChar(label="Byte"), "Control": SimTypeShort(signed=False, label="UInt16"), "Owner": SimTypePointer(SimTypeInt(signed=True, label="Int"), label="IntPtr", offset=0), "Group": SimTypePointer(SimTypeInt(signed=True, label="Int"), label="IntPtr", offset=0), "Sacl": SimTypePointer(SimStruct({"AclRevision": SimTypeChar(label="Byte"), "Sbz1": SimTypeChar(label="Byte"), "AclSize": SimTypeShort(signed=False, label="UInt16"), "AceCount": SimTypeShort(signed=False, label="UInt16"), "Sbz2": SimTypeShort(signed=False, label="UInt16")}, name="ACL", pack=False, align=None), offset=0), "Dacl": SimTypePointer(SimStruct({"AclRevision": SimTypeChar(label="Byte"), "Sbz1": SimTypeChar(label="Byte"), "AclSize": SimTypeShort(signed=False, label="UInt16"), "AceCount": SimTypeShort(signed=False, label="UInt16"), "Sbz2": SimTypeShort(signed=False, label="UInt16")}, name="ACL", pack=False, align=None), offset=0)}, name="SECURITY_DESCRIPTOR", pack=False, align=None), offset=0), offset=0)], SimTypeInt(signed=False, label="UInt32"), arg_names=["engineHandle", "securityInfo", "sidOwner", "sidGroup", "dacl", "sacl", "securityDescriptor"]),
        # 
        'FwpmConnectionSetSecurityInfo0': SimTypeFunction([SimTypePointer(SimTypeInt(signed=True, label="Int"), label="IntPtr", offset=0), SimTypeInt(signed=False, label="UInt32"), SimTypePointer(SimStruct({"Revision": SimTypeChar(label="Byte"), "SubAuthorityCount": SimTypeChar(label="Byte"), "IdentifierAuthority": SimStruct({"Value": SimTypeFixedSizeArray(SimTypeChar(label="Byte"), 6)}, name="SID_IDENTIFIER_AUTHORITY", pack=False, align=None), "SubAuthority": SimTypePointer(SimTypeInt(signed=False, label="UInt32"), offset=0)}, name="SID", pack=False, align=None), offset=0), SimTypePointer(SimStruct({"Revision": SimTypeChar(label="Byte"), "SubAuthorityCount": SimTypeChar(label="Byte"), "IdentifierAuthority": SimStruct({"Value": SimTypeFixedSizeArray(SimTypeChar(label="Byte"), 6)}, name="SID_IDENTIFIER_AUTHORITY", pack=False, align=None), "SubAuthority": SimTypePointer(SimTypeInt(signed=False, label="UInt32"), offset=0)}, name="SID", pack=False, align=None), offset=0), SimTypePointer(SimStruct({"AclRevision": SimTypeChar(label="Byte"), "Sbz1": SimTypeChar(label="Byte"), "AclSize": SimTypeShort(signed=False, label="UInt16"), "AceCount": SimTypeShort(signed=False, label="UInt16"), "Sbz2": SimTypeShort(signed=False, label="UInt16")}, name="ACL", pack=False, align=None), offset=0), SimTypePointer(SimStruct({"AclRevision": SimTypeChar(label="Byte"), "Sbz1": SimTypeChar(label="Byte"), "AclSize": SimTypeShort(signed=False, label="UInt16"), "AceCount": SimTypeShort(signed=False, label="UInt16"), "Sbz2": SimTypeShort(signed=False, label="UInt16")}, name="ACL", pack=False, align=None), offset=0)], SimTypeInt(signed=False, label="UInt32"), arg_names=["engineHandle", "securityInfo", "sidOwner", "sidGroup", "dacl", "sacl"]),
        # 
        'FwpmConnectionSubscribe0': SimTypeFunction([SimTypePointer(SimTypeInt(signed=True, label="Int"), label="IntPtr", offset=0), SimTypePointer(SimStruct({"enumTemplate": SimTypePointer(SimStruct({"connectionId": SimTypeLongLong(signed=False, label="UInt64"), "flags": SimTypeInt(signed=False, label="UInt32")}, name="FWPM_CONNECTION_ENUM_TEMPLATE0", pack=False, align=None), offset=0), "flags": SimTypeInt(signed=False, label="UInt32"), "sessionKey": SimTypeBottom(label="Guid")}, name="FWPM_CONNECTION_SUBSCRIPTION0", pack=False, align=None), offset=0), SimTypePointer(SimTypeFunction([SimTypePointer(SimTypeBottom(label="Void"), offset=0), SimTypeInt(signed=False, label="FWPM_CONNECTION_EVENT_TYPE"), SimTypePointer(SimStruct({"connectionId": SimTypeLongLong(signed=False, label="UInt64"), "ipVersion": SimTypeInt(signed=False, label="FWP_IP_VERSION"), "Anonymous1": SimUnion({"localV4Address": SimTypeInt(signed=False, label="UInt32"), "localV6Address": SimTypeFixedSizeArray(SimTypeChar(label="Byte"), 16)}, name="<anon>", label="None"), "Anonymous2": SimUnion({"remoteV4Address": SimTypeInt(signed=False, label="UInt32"), "remoteV6Address": SimTypeFixedSizeArray(SimTypeChar(label="Byte"), 16)}, name="<anon>", label="None"), "providerKey": SimTypePointer(SimTypeBottom(label="Guid"), offset=0), "ipsecTrafficModeType": SimTypeInt(signed=False, label="IPSEC_TRAFFIC_TYPE"), "keyModuleType": SimTypeInt(signed=False, label="IKEEXT_KEY_MODULE_TYPE"), "mmCrypto": SimStruct({"cipherAlgorithm": SimStruct({"algoIdentifier": SimTypeInt(signed=False, label="IKEEXT_CIPHER_TYPE"), "keyLen": SimTypeInt(signed=False, label="UInt32"), "rounds": SimTypeInt(signed=False, label="UInt32")}, name="IKEEXT_CIPHER_ALGORITHM0", pack=False, align=None), "integrityAlgorithm": SimStruct({"algoIdentifier": SimTypeInt(signed=False, label="IKEEXT_INTEGRITY_TYPE")}, name="IKEEXT_INTEGRITY_ALGORITHM0", pack=False, align=None), "maxLifetimeSeconds": SimTypeInt(signed=False, label="UInt32"), "dhGroup": SimTypeInt(signed=False, label="IKEEXT_DH_GROUP"), "quickModeLimit": SimTypeInt(signed=False, label="UInt32")}, name="IKEEXT_PROPOSAL0", pack=False, align=None), "mmPeer": SimStruct({"authenticationMethodType": SimTypeInt(signed=False, label="IKEEXT_AUTHENTICATION_METHOD_TYPE"), "impersonationType": SimTypeInt(signed=False, label="IKEEXT_AUTHENTICATION_IMPERSONATION_TYPE"), "Anonymous": SimUnion({"presharedKey": SimTypePointer(SimStruct({"presharedKey": SimStruct({"size": SimTypeInt(signed=False, label="UInt32"), "data": SimTypePointer(SimTypeChar(label="Byte"), offset=0)}, name="FWP_BYTE_BLOB", pack=False, align=None), "flags": SimTypeInt(signed=False, label="IKEEXT_PRESHARED_KEY_AUTHENTICATION_FLAGS")}, name="IKEEXT_PRESHARED_KEY_AUTHENTICATION1", pack=False, align=None), offset=0), "certificate": SimTypePointer(SimStruct({"subjectName": SimStruct({"size": SimTypeInt(signed=False, label="UInt32"), "data": SimTypePointer(SimTypeChar(label="Byte"), offset=0)}, name="FWP_BYTE_BLOB", pack=False, align=None), "certHash": SimStruct({"size": SimTypeInt(signed=False, label="UInt32"), "data": SimTypePointer(SimTypeChar(label="Byte"), offset=0)}, name="FWP_BYTE_BLOB", pack=False, align=None), "flags": SimTypeInt(signed=False, label="UInt32"), "certificate": SimStruct({"size": SimTypeInt(signed=False, label="UInt32"), "data": SimTypePointer(SimTypeChar(label="Byte"), offset=0)}, name="FWP_BYTE_BLOB", pack=False, align=None)}, name="IKEEXT_CERTIFICATE_CREDENTIAL1", pack=False, align=None), offset=0), "name": SimTypePointer(SimStruct({"principalName": SimTypePointer(SimTypeChar(label="Char"), offset=0)}, name="IKEEXT_NAME_CREDENTIAL0", pack=False, align=None), offset=0)}, name="<anon>", label="None")}, name="IKEEXT_CREDENTIAL2", pack=False, align=None), "emPeer": SimStruct({"authenticationMethodType": SimTypeInt(signed=False, label="IKEEXT_AUTHENTICATION_METHOD_TYPE"), "impersonationType": SimTypeInt(signed=False, label="IKEEXT_AUTHENTICATION_IMPERSONATION_TYPE"), "Anonymous": SimUnion({"presharedKey": SimTypePointer(SimStruct({"presharedKey": SimStruct({"size": SimTypeInt(signed=False, label="UInt32"), "data": SimTypePointer(SimTypeChar(label="Byte"), offset=0)}, name="FWP_BYTE_BLOB", pack=False, align=None), "flags": SimTypeInt(signed=False, label="IKEEXT_PRESHARED_KEY_AUTHENTICATION_FLAGS")}, name="IKEEXT_PRESHARED_KEY_AUTHENTICATION1", pack=False, align=None), offset=0), "certificate": SimTypePointer(SimStruct({"subjectName": SimStruct({"size": SimTypeInt(signed=False, label="UInt32"), "data": SimTypePointer(SimTypeChar(label="Byte"), offset=0)}, name="FWP_BYTE_BLOB", pack=False, align=None), "certHash": SimStruct({"size": SimTypeInt(signed=False, label="UInt32"), "data": SimTypePointer(SimTypeChar(label="Byte"), offset=0)}, name="FWP_BYTE_BLOB", pack=False, align=None), "flags": SimTypeInt(signed=False, label="UInt32"), "certificate": SimStruct({"size": SimTypeInt(signed=False, label="UInt32"), "data": SimTypePointer(SimTypeChar(label="Byte"), offset=0)}, name="FWP_BYTE_BLOB", pack=False, align=None)}, name="IKEEXT_CERTIFICATE_CREDENTIAL1", pack=False, align=None), offset=0), "name": SimTypePointer(SimStruct({"principalName": SimTypePointer(SimTypeChar(label="Char"), offset=0)}, name="IKEEXT_NAME_CREDENTIAL0", pack=False, align=None), offset=0)}, name="<anon>", label="None")}, name="IKEEXT_CREDENTIAL2", pack=False, align=None), "bytesTransferredIn": SimTypeLongLong(signed=False, label="UInt64"), "bytesTransferredOut": SimTypeLongLong(signed=False, label="UInt64"), "bytesTransferredTotal": SimTypeLongLong(signed=False, label="UInt64"), "startSysTime": SimStruct({"dwLowDateTime": SimTypeInt(signed=False, label="UInt32"), "dwHighDateTime": SimTypeInt(signed=False, label="UInt32")}, name="FILETIME", pack=False, align=None)}, name="FWPM_CONNECTION0", pack=False, align=None), offset=0)], SimTypeBottom(label="Void"), arg_names=["context", "eventType", "connection"]), offset=0), SimTypePointer(SimTypeBottom(label="Void"), offset=0), SimTypePointer(SimTypePointer(SimTypeInt(signed=True, label="Int"), label="IntPtr", offset=0), offset=0)], SimTypeInt(signed=False, label="UInt32"), arg_names=["engineHandle", "subscription", "callback", "context", "eventsHandle"]),
        # 
        'FwpmConnectionUnsubscribe0': SimTypeFunction([SimTypePointer(SimTypeInt(signed=True, label="Int"), label="IntPtr", offset=0), SimTypePointer(SimTypeInt(signed=True, label="Int"), label="IntPtr", offset=0)], SimTypeInt(signed=False, label="UInt32"), arg_names=["engineHandle", "eventsHandle"]),
        # 
        'FwpmvSwitchEventSubscribe0': SimTypeFunction([SimTypePointer(SimTypeInt(signed=True, label="Int"), label="IntPtr", offset=0), SimTypePointer(SimStruct({"flags": SimTypeInt(signed=False, label="UInt32"), "sessionKey": SimTypeBottom(label="Guid")}, name="FWPM_VSWITCH_EVENT_SUBSCRIPTION0", pack=False, align=None), offset=0), SimTypePointer(SimTypeFunction([SimTypePointer(SimTypeBottom(label="Void"), offset=0), SimTypePointer(SimStruct({"eventType": SimTypeInt(signed=False, label="FWPM_VSWITCH_EVENT_TYPE"), "vSwitchId": SimTypePointer(SimTypeChar(label="Char"), offset=0), "Anonymous": SimUnion({"positionInfo": SimStruct({"numvSwitchFilterExtensions": SimTypeInt(signed=False, label="UInt32"), "vSwitchFilterExtensions": SimTypePointer(SimTypePointer(SimTypeChar(label="Char"), offset=0), offset=0)}, name="_positionInfo_e__Struct", pack=False, align=None), "reorderInfo": SimStruct({"inRequiredPosition": SimTypeInt(signed=True, label="Int32"), "numvSwitchFilterExtensions": SimTypeInt(signed=False, label="UInt32"), "vSwitchFilterExtensions": SimTypePointer(SimTypePointer(SimTypeChar(label="Char"), offset=0), offset=0)}, name="_reorderInfo_e__Struct", pack=False, align=None)}, name="<anon>", label="None")}, name="FWPM_VSWITCH_EVENT0", pack=False, align=None), offset=0)], SimTypeInt(signed=False, label="UInt32"), arg_names=["context", "vSwitchEvent"]), offset=0), SimTypePointer(SimTypeBottom(label="Void"), offset=0), SimTypePointer(SimTypePointer(SimTypeInt(signed=True, label="Int"), label="IntPtr", offset=0), offset=0)], SimTypeInt(signed=False, label="UInt32"), arg_names=["engineHandle", "subscription", "callback", "context", "subscriptionHandle"]),
        # 
        'FwpmvSwitchEventUnsubscribe0': SimTypeFunction([SimTypePointer(SimTypeInt(signed=True, label="Int"), label="IntPtr", offset=0), SimTypePointer(SimTypeInt(signed=True, label="Int"), label="IntPtr", offset=0)], SimTypeInt(signed=False, label="UInt32"), arg_names=["engineHandle", "subscriptionHandle"]),
        # 
        'FwpmvSwitchEventsGetSecurityInfo0': SimTypeFunction([SimTypePointer(SimTypeInt(signed=True, label="Int"), label="IntPtr", offset=0), SimTypeInt(signed=False, label="UInt32"), SimTypePointer(SimTypePointer(SimTypeInt(signed=True, label="Int"), label="IntPtr", offset=0), offset=0), SimTypePointer(SimTypePointer(SimTypeInt(signed=True, label="Int"), label="IntPtr", offset=0), offset=0), SimTypePointer(SimTypePointer(SimStruct({"AclRevision": SimTypeChar(label="Byte"), "Sbz1": SimTypeChar(label="Byte"), "AclSize": SimTypeShort(signed=False, label="UInt16"), "AceCount": SimTypeShort(signed=False, label="UInt16"), "Sbz2": SimTypeShort(signed=False, label="UInt16")}, name="ACL", pack=False, align=None), offset=0), offset=0), SimTypePointer(SimTypePointer(SimStruct({"AclRevision": SimTypeChar(label="Byte"), "Sbz1": SimTypeChar(label="Byte"), "AclSize": SimTypeShort(signed=False, label="UInt16"), "AceCount": SimTypeShort(signed=False, label="UInt16"), "Sbz2": SimTypeShort(signed=False, label="UInt16")}, name="ACL", pack=False, align=None), offset=0), offset=0), SimTypePointer(SimTypePointer(SimStruct({"Revision": SimTypeChar(label="Byte"), "Sbz1": SimTypeChar(label="Byte"), "Control": SimTypeShort(signed=False, label="UInt16"), "Owner": SimTypePointer(SimTypeInt(signed=True, label="Int"), label="IntPtr", offset=0), "Group": SimTypePointer(SimTypeInt(signed=True, label="Int"), label="IntPtr", offset=0), "Sacl": SimTypePointer(SimStruct({"AclRevision": SimTypeChar(label="Byte"), "Sbz1": SimTypeChar(label="Byte"), "AclSize": SimTypeShort(signed=False, label="UInt16"), "AceCount": SimTypeShort(signed=False, label="UInt16"), "Sbz2": SimTypeShort(signed=False, label="UInt16")}, name="ACL", pack=False, align=None), offset=0), "Dacl": SimTypePointer(SimStruct({"AclRevision": SimTypeChar(label="Byte"), "Sbz1": SimTypeChar(label="Byte"), "AclSize": SimTypeShort(signed=False, label="UInt16"), "AceCount": SimTypeShort(signed=False, label="UInt16"), "Sbz2": SimTypeShort(signed=False, label="UInt16")}, name="ACL", pack=False, align=None), offset=0)}, name="SECURITY_DESCRIPTOR", pack=False, align=None), offset=0), offset=0)], SimTypeInt(signed=False, label="UInt32"), arg_names=["engineHandle", "securityInfo", "sidOwner", "sidGroup", "dacl", "sacl", "securityDescriptor"]),
        # 
        'FwpmvSwitchEventsSetSecurityInfo0': SimTypeFunction([SimTypePointer(SimTypeInt(signed=True, label="Int"), label="IntPtr", offset=0), SimTypeInt(signed=False, label="UInt32"), SimTypePointer(SimStruct({"Revision": SimTypeChar(label="Byte"), "SubAuthorityCount": SimTypeChar(label="Byte"), "IdentifierAuthority": SimStruct({"Value": SimTypeFixedSizeArray(SimTypeChar(label="Byte"), 6)}, name="SID_IDENTIFIER_AUTHORITY", pack=False, align=None), "SubAuthority": SimTypePointer(SimTypeInt(signed=False, label="UInt32"), offset=0)}, name="SID", pack=False, align=None), offset=0), SimTypePointer(SimStruct({"Revision": SimTypeChar(label="Byte"), "SubAuthorityCount": SimTypeChar(label="Byte"), "IdentifierAuthority": SimStruct({"Value": SimTypeFixedSizeArray(SimTypeChar(label="Byte"), 6)}, name="SID_IDENTIFIER_AUTHORITY", pack=False, align=None), "SubAuthority": SimTypePointer(SimTypeInt(signed=False, label="UInt32"), offset=0)}, name="SID", pack=False, align=None), offset=0), SimTypePointer(SimStruct({"AclRevision": SimTypeChar(label="Byte"), "Sbz1": SimTypeChar(label="Byte"), "AclSize": SimTypeShort(signed=False, label="UInt16"), "AceCount": SimTypeShort(signed=False, label="UInt16"), "Sbz2": SimTypeShort(signed=False, label="UInt16")}, name="ACL", pack=False, align=None), offset=0), SimTypePointer(SimStruct({"AclRevision": SimTypeChar(label="Byte"), "Sbz1": SimTypeChar(label="Byte"), "AclSize": SimTypeShort(signed=False, label="UInt16"), "AceCount": SimTypeShort(signed=False, label="UInt16"), "Sbz2": SimTypeShort(signed=False, label="UInt16")}, name="ACL", pack=False, align=None), offset=0)], SimTypeInt(signed=False, label="UInt32"), arg_names=["engineHandle", "securityInfo", "sidOwner", "sidGroup", "dacl", "sacl"]),
    }

lib.set_prototypes(prototypes)
